C     ==================================================================
      SUBROUTINE TOL_INIT (LTOLAD, DEACC, GENVMX)
C     ==--------------------------------------------------------------==
C     == INITIALIZATION OF ELECTRONIC CONVERGENCE CRITERIA            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'lscal.inc'
C     ==--------------------------------------------------------------==
C     == ARGUMENTS:                                                   ==
C     == LTOLAD (out) Adaptive tolerance on by default                ==
C     == DEACC  (out) No accepted step yet (checkpoint: DELAST kept)  ==
C     == GENVMX (out) No maximum gradient of environment (TOLOG kept) ==
C     ==--------------------------------------------------------------==
      LOGICAL LTOLAD
      REAL*8  DEACC, GENVMX
C     ==--------------------------------------------------------------==
      INTEGER MSGLEN
C     ==--------------------------------------------------------------==
C     THESE VARIABLES NEED TO BE BROADCAST
      IF(TOLMIN.EQ.0.0D0) THEN ! INIT IF NO TOLERANCES FROM CHECKPOINT
        DELAST = 0.0D0
        TOLMIN = TOLOG
        IF(TOLINI.NE.0.0D0) THEN
          TOLOG = TOLINI
        ENDIF
      ENDIF
      IF(PARENT.AND.TOLENE.NE.0.0D0) THEN
        IF(DELAST.EQ.0.0D0) THEN
          DELAST = 1.0D3 * DEMIN
          WRITE (6,'(1X,A,1PE10.4)')
     &         'DEFAULT WAVEFUNCTION ENERGY TOLERANCE:   ', DELAST
        ELSE
          WRITE (6,'(1X,A,1PE10.4)')
     &          'WAVEFUNCTION ENERGY TOLERANCE:   ', DELAST
        ENDIF
      ENDIF
      MSGLEN = 8
      CALL MY_BCAST (TOLENE, MSGLEN, SOURCE, ALLGRP)
      CALL MY_BCAST (TOLMIN, MSGLEN, SOURCE, ALLGRP)
      CALL MY_BCAST (DELAST, MSGLEN, SOURCE, ALLGRP)
C     THESE VARIABLES DO NOT NEED TO BE BROADCAST (FOR PARENT USE ONLY)
      LTOLAD = .TRUE.
      GENVMX = 0.0D0
      DEACC  = 0.0D0
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE TOL_CHK_FORCE (TFOR, CALSTE, GEMAX, INFW)
C     ==--------------------------------------------------------------==
C     == CHECK IF FORCES ON IONS NEED TO BE CALCULATED                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'lscal.inc'
C     ==--------------------------------------------------------------==
C     == ARGUMENTS:                                                   ==
C     == LFOR   (out) Switch for forces                               ==
C     == CALSTE (out) Switch for stress tensor                        ==
C     == GEMAX  (in)  Maximum component of electronic gradient        ==
C     == INFW   (in)  Number of WFN step for current geometry         ==
C     ==--------------------------------------------------------------==
      LOGICAL TFOR, CALSTE
      INTEGER INFW
      REAL*8  GEMAX
C     ==--------------------------------------------------------------==
      IF (INFW.EQ.1) THEN
         CALSTE = .FALSE.
         TFOR   = .FALSE.
         TNOFOR = .FALSE.       ! Based on energy change
      ELSE IF (GEMAX.LT.TOLREL*TOLFOR*TOLOG .AND. .NOT.TNOFOR) THEN
         CALSTE = TPRCP
         TFOR   = .TRUE.
      END IF
      IF (NSTCNV.GE.1) THEN
         IF (INFW.EQ.1) THEN
            IF (TOLREL.NE.1.0D0.AND.PARENT) WRITE (6,'(1X,A,1F4.1)')
     &           'RESETTING WAVEFUNCTION TOLERANCE FACTOR TO: ', 1.0D0
            TOLREL = 1.0D0
            NRLLST = 1
         ELSE IF (INFW-NRLLST.GE.NSTCNV) THEN
            TOLREL = TOLREL * 2.0D0
            NRLLST = INFW
            IF (PARENT) WRITE (6,'(1X,A,1F4.1)')
     &           'RELAXING WAVEFUNCTION TOLERANCE, NEW FACTOR: ',
     &           TOLREL
         END IF
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE TOL_CHK_CNVGRAD (CONVWF, EXSOFT, GEMAX)
C     ==--------------------------------------------------------------==
C     == CHECK IF WAVEFUNCTION IS CONVERGED BASED ON GRADIENT         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'lscal.inc'
C     ==--------------------------------------------------------------==
C     == ARGUMENTS:                                                   ==
C     == CONVWF (upd) Flag if converged                               ==
C     == EXSOFT (in)  Flag if exit requested                          ==
C     == GEMAX  (in)  Maximum wavefunction component (BROADCAST)      ==
C     ==--------------------------------------------------------------==
      LOGICAL CONVWF, EXSOFT
      REAL*8  GEMAX
C     ==--------------------------------------------------------------==
      IF (GEMAX.LT.TOLREL*TOLOG .OR. EXSOFT) THEN
         CONVWF=.TRUE.
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE TOL_CHK_CNVENER (CONVWF, ETOT, ETOTO)
C     ==--------------------------------------------------------------==
C     == CHECK IF WAVEFUNCTION IS NOT CONVERGED BASED ON ENERGY       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'lscal.inc'
C     ==--------------------------------------------------------------==
C     == IMPLICIT RESULTS:                                            ==
C     == TNOFOR (out) Force forces not to be calculated               ==
C     ==                                                              ==
C     == ARGUMENTS:                                                   ==
C     == CONVWF (upd) Flag if converged                               ==
C     == ETOT   (in)  Current energy (MUST BE BROADCAST ALREADY)      ==
C     == ETOTO  (in)  Energy of previous wfn step (BROADCAST)         ==
C     ==--------------------------------------------------------------==
      LOGICAL CONVWF
      REAL*8  ETOT, ETOTO
C     ==--------------------------------------------------------------==
      IF (TOLENE.NE.0.0D0 .AND. DELAST.NE.0.0D0) THEN
         IF (DABS(ETOT-ETOTO).GT.TOLREL*DELAST) THEN
            CONVWF = .FALSE.
         END IF
         TNOFOR = (DABS(ETOT-ETOTO).GT.TOLREL*TOLFOR*DELAST)
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE TOL_DET_GRAD (LTOLAD, GNMAX, GENVMX)
C     ==--------------------------------------------------------------==
C     == DETERMINE THE WAVEFUNCTION GRADIENT TOLERANCE FOR NEXT STEP  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'lscal.inc'
C     ==--------------------------------------------------------------==
C     == IMPLICIT RESULTS:                                            ==
C     == TOLOG  (out) Gradient tolerance                              ==
C     ==                                                              ==
C     == ARGUMENTS:                                                   ==
C     == LTOLAD (in)  Switch for adaptive tolerance                   ==
C     == GNMAX  (in)  Maximum gradient component on all ions          ==
C     == GENVMX (in)  Maximum gradient component on environment ions  ==
C     ==--------------------------------------------------------------==
      LOGICAL LTOLAD
      REAL*8  GNMAX, GENVMX
C     ==--------------------------------------------------------------==
      INTEGER MSGLEN
C     ==--------------------------------------------------------------==
      IF (PARENT) THEN
         IF (LTOLAD .AND. TOLAD.GT.0.0D0) THEN
            IF (GENVMX.EQ.0.0D0) THEN
               GNMIN = MIN(GNMAX,GNMIN)
            ELSE
               GNMIN = MIN(GENVMX,GNMIN)
            ENDIF
            TOLOG = MAX(TOLAD*GNMIN,TOLMIN)
            WRITE(6,'(1X,A,1PE10.4)')
     &           'WAVEFUNCTION GRADIENT TOLERANCE: ', TOLOG
            IF (TOLMIN.GE.TOLOG) THEN
               TOLAD = 0.0D0
               WRITE (6,'(1X,A,A)') 'MINIMUM TOLERANCE REACHED, ',
     &              'SWITCHING OFF ADAPTIVE TOLERANCE'
            END IF
         ELSE
            TOLOG = TOLMIN
         ENDIF
      ENDIF
      MSGLEN = 8
      CALL MY_BCAST (TOLOG, MSGLEN, SOURCE, ALLGRP)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE TOL_DET_ENER (LTOLAD, INFI, DETOT, DEACC)
C     ==--------------------------------------------------------------==
C     == DETERMINE THE NEXT WAVEFUNCTION ENERGY CHANGE TOLERANCE      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'lscal.inc'
C     ==--------------------------------------------------------------==
C     == IMPLICIT RESULTS:                                            ==
C     == TOLOG  (out) Gradient tolerance                              ==
C     ==                                                              ==
C     == ARGUMENTS:                                                   ==
C     == LTOLAD (in)  Switch for adaptive tolerance                   ==
C     == INFI   (in)  Number of geometry step                         ==
C     == DETOT  (in)  Energy change of last ions step                 ==
C     == DEACC  (in)  Energy change of last successful ions step      ==
C     ==--------------------------------------------------------------==
      LOGICAL LTOLAD
      INTEGER INFI
      REAL*8  DETOT, DEACC
C     ==--------------------------------------------------------------==
      INTEGER MSGLEN
C     ==--------------------------------------------------------------==
      IF (PARENT) THEN
         IF (.FALSE..AND..NOT.LTOLAD) THEN
            DELAST = 0.0D0
         ELSE IF (TOLENE.GT.0.0D0 .AND. INFI.GT.1) THEN
            IF (DEACC.EQ.0.0D0) THEN
               DELAST = DETOT*TOLENE
            ELSE
               DELAST = DEACC*TOLENE
            ENDIF
            IF (DELAST.NE.0.0D0) THEN
               DELAST = MAX(DABS(DELAST),DEMIN)
               WRITE (6,'(1X,A,1PE10.4)')
     &              'WAVEFUNCTION ENERGY TOLERANCE:   ', DELAST
            ENDIF
         ENDIF
      ENDIF                     ! IF (PARENT)
      MSGLEN = 8
      CALL MY_BCAST (TOLENE, MSGLEN, SOURCE, ALLGRP)
      CALL MY_BCAST (DELAST, MSGLEN, SOURCE, ALLGRP)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE TOL_INIT_SHARED
C     ==--------------------------------------------------------------==
C     == Block data: init variables for P-RFO and MI TS search        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'lscal.inc'
C     ==--------------------------------------------------------------==
      TNOFOR = .TRUE.            ! No forces on ions
      NRLLST = 0                 ! Last WF step convergence relaxed
      DELAST = 0.0D0             ! No successful step yet (BCAST)
      DEMIN  = 1.0D-7            ! Force to be converged (no BCAST)
      GNMIN  = 1.0D0             ! Lowest gradient comp. so far
      TOLMIN = 0.0D0             ! Minimum GNMAX (switch - BCAST)
      TOLREL = 1.0D0             ! Full convergence criteria
C     ==--------------------------------------------------------------==
      END
C     ==================================================================
