C     ==================================================================
      SUBROUTINE ANNEAL(VELP,CM,NSTATE,HTVEL)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'bsym.inc'
C     Arguments
      REAL*8  VELP(3,NAX,NSP),HTVEL(*)
      COMPLEX*16 CM(NGW,*)
      INTEGER NSTATE
C     Variables
      REAL*8  ALFAP
      INTEGER I,J,IS,IA
      LOGICAL SKIPANIC
C
C     NN: IF BS AND HS-WF SKIP SCALING OF IONIC AND CELL VELOCITIES 
      SKIPANIC=.FALSE.
      IF(BSYMM.AND.(BSCLCS.EQ.2))SKIPANIC=.TRUE.
C
C     ==--------------------------------------------------------------==
C     ==  ANNEALING (IONS)  Fixed rescaling factor (anner)            ==
C     ==--------------------------------------------------------------==
      IF(ANNEI.AND.(.NOT.SKIPANIC)) THEN
        ALFAP=ANNERI**(0.25D0)
#if defined(__VECTOR)
C$OMP parallel do private(I,IS,IA)
#else
C$OMP parallel do private(I,IS,IA) schedule(static)
#endif
        DO I=1,NAT
          IA=IATPT(1,I)
          IS=IATPT(2,I)
          VELP(1,IA,IS)=ALFAP*VELP(1,IA,IS)
          VELP(2,IA,IS)=ALFAP*VELP(2,IA,IS)
          VELP(3,IA,IS)=ALFAP*VELP(3,IA,IS)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  ANNEALING (ELECTRONS)  Fixed rescaling factor (anner)       ==
C     ==--------------------------------------------------------------==
      IF(ANNEE) THEN
        ALFAP=ANNERE**(0.25D0)
        CALL DSCAL(2*NGW*NSTATE,ALFAP,CM,1)
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  ANNEALING (CELL)  Fixed rescaling factor (anner)            ==
C     ==--------------------------------------------------------------==
      IF(ANNEC.AND.(.NOT.SKIPANIC)) THEN
        ALFAP=ANNERC**(0.25D0)
        DO I=1,9
          HTVEL(I)=ALFAP*HTVEL(I)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE DAMPDYN(VELP,FION,CM,C2,NSTATE,HTVEL,HTFOR)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'bsym.inc'
C     Arguments
      REAL*8  VELP(3,NAX,*),FION(3,NAX,*),HTVEL(*),HTFOR(*)
      COMPLEX*16 CM(NGW,*),C2(NGW,*)
      INTEGER NSTATE
C     Variables
      REAL*8  ALFAP
      INTEGER I,J,IS,IA
      LOGICAL SKIPANIC
C
C     NN: IF BS AND HS-WF SKIP SCALING OF IONIC AND CELL VELOCITIES 
      SKIPANIC=.FALSE.
      IF(BSYMM.AND.(BSCLCS.EQ.2))SKIPANIC=.TRUE.
C
C     ==--------------------------------------------------------------==
C     ==  DAMPING (IONS)  Fixed friction factor                       ==
C     ==--------------------------------------------------------------==
      IF(DAMPI.AND.(.NOT.SKIPANIC)) THEN
        ALFAP=DAMPGI
#if defined(__VECTOR)
C$OMP parallel do private(I,IS,IA)
#else
C$OMP parallel do private(I,IS,IA) schedule(static)
#endif
        DO I=1,NAT
          IA=IATPT(1,I)
          IS=IATPT(2,I)
          FION(1,IA,IS)=FION(1,IA,IS)-ALFAP*VELP(1,IA,IS)
          FION(2,IA,IS)=FION(2,IA,IS)-ALFAP*VELP(2,IA,IS)
          FION(3,IA,IS)=FION(3,IA,IS)-ALFAP*VELP(3,IA,IS)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  DAMPING (ELECTRONS)  Fixed friction factor                  ==
C     ==--------------------------------------------------------------==
      IF(DAMPE) THEN
        ALFAP=-DAMPGE
        CALL DAXPY(2*NGW*NSTATE,ALFAP,CM,1,C2,1)
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  DAMPING (CELL)  Fixed friction factor                       ==
C     ==--------------------------------------------------------------==
      IF(DAMPC.AND.(.NOT.SKIPANIC)) THEN
        ALFAP=DAMPGC
        DO I=1,9
          HTFOR(I)=HTFOR(I)-ALFAP*HTVEL(I)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE BERENDSEN(VELP,CM,NSTATE,HTVEL,EKINC,EKINH)
C     NOTE: we get called twice for each half step of the velocity 
C     verlet so the the scaling is only considering half a timestep.
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'tpar.inc' ! for DT_IONS
      INCLUDE 'bsym.inc'
      INCLUDE 'nose.inc' ! for GLIB
      INCLUDE 'cnst.inc' ! for FACTEM
C     Arguments
      REAL*8  VELP(3,NAX,*),HTVEL(*),EKINC,EKINH
      COMPLEX*16 CM(NGW,*)
      INTEGER NSTATE
C     Variables
      REAL*8  LAMBDA,EKINTMP,TEMPCUR,THRESH
      INTEGER I,J,IS,IA
      LOGICAL SKIPANIC
      PARAMETER(THRESH=1.0D-6)  ! don't scale if we are too "cold".
C
C     NN: IF BS AND HS-WF SKIP SCALING OF IONIC AND CELL VELOCITIES 
      SKIPANIC=.FALSE.
      IF(BSYMM.AND.(BSCLCS.EQ.2))SKIPANIC=.TRUE.
C     ==--------------------------------------------------------------==
C     ==  BERENDSEN THERMOSTAT (IONS)                                 ==
C     ==--------------------------------------------------------------==
      IF(PARENT.AND.TBERP.AND.(.NOT.SKIPANIC)) THEN
        CALL EKINPP(EKINTMP,VELP)
        TEMPCUR=EKINTMP*FACTEM*2.0D0/GLIB
        IF(TEMPCUR/TEMPW.GT.THRESH) THEN
          LAMBDA=DSQRT(1.0D0+0.5D0*DT_IONS/TAUBP*(TEMPW/TEMPCUR-1.0D0))
#if defined(__VECTOR)
C$OMP parallel do private(I,IS,IA)
#else
C$OMP parallel do private(I,IS,IA) schedule(static)
#endif
          DO I=1,NAT
            IA=IATPT(1,I)
            IS=IATPT(2,I)
            VELP(1,IA,IS)=LAMBDA*VELP(1,IA,IS)
            VELP(2,IA,IS)=LAMBDA*VELP(2,IA,IS)
            VELP(3,IA,IS)=LAMBDA*VELP(3,IA,IS)
          ENDDO
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  BERENDSEN THERMOSTAT (ELECTRONS)                            ==
C     ==--------------------------------------------------------------==
      IF(TBERE) THEN
        IF(EKINC/EKINW.GT.THRESH)THEN
          LAMBDA=DSQRT(1.0D0+0.5D0*DT_IONS/TAUBE*(EKINW/EKINC-1.0D0))
          CALL DSCAL(2*NGW*NSTATE,LAMBDA,CM,1)
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  BERENDSEN THERMOSTAT (CELL)                                 ==
C     ==--------------------------------------------------------------==
      IF(PARENT.AND.TBERC.AND.(.NOT.SKIPANIC)) THEN
        IF(EKINH/EKINHR.GT.THRESH)THEN
          LAMBDA=DSQRT(1.0D0+0.5D0*DT_IONS/TAUBC*(EKINHR/EKINH-1.0D0))
          DO I=1,9
            HTVEL(I)=LAMBDA*HTVEL(I)
          ENDDO
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE TEMPRAMP(TEMP1,TEMP2)
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'tpar.inc'
C     Arguments
      REAL*8 TEMP1,TEMP2
C     ==--------------------------------------------------------------==
      IF (PARENT.AND.TRAMPR.GT.1.0D-30) THEN
C     ==--------------------------------------------------------------==
        IF(DABS(TEMPW-TRAMPT).GT.TRAMPR) THEN
          IF(TRAMPT.GT.TEMPW) THEN
            TEMPW=TEMPW+DT_IONS*TRAMPR
          ELSE
            TEMPW=TEMPW-DT_IONS*TRAMPR
          ENDIF
        ELSE
          TEMPW=TRAMPT
          WRITE(6,*) 'TEMPRAMP| TEMPERATURE RAMP COMPLETED AT: ',TEMPW
          TRAMPR=0.0D0
        ENDIF
        TEMP1=TEMPW+TOLP
        TEMP2=TEMPW-TOLP
C     ==--------------------------------------------------------------==
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
