C     ==================================================================
      SUBROUTINE ATOMWF(C0,NSTATE,TAU0,FION,RHOE,PSI,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     == ATOMIC WAVEFUNCTIONS AS INITIAL GUESS                        ==
C     == WORKS WITH K POINTS: WITH KPOINT=GAMMA, SAME EIGENVALUES AS  ==
C     == NO KPOINT VERSION x EXP(I THETA).                            ==
C     == MOREOVER, IF EIGENSTATES ARE DEGENERATED, IN THE SUBSPACE    ==
C     == EIGENVECTORS ARE DIFFERENT.                                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ener.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'        !TIVAN
      INCLUDE 'atwf.inc'        !CATOM ZXMAT XXMAT
      INCLUDE 'nlps.inc'        !IMAGP
      INCLUDE 'irat.inc'        !IRAT
      INCLUDE 'spin.inc'        !NLSD NLSX
      INCLUDE 'nlcc.inc'        !TINLC
      INCLUDE 'kpts.inc'        !TKPNT
      INCLUDE 'sfac.inc'        !FNL DFNL
      INCLUDE 'sphe.inc'        !TSPHERE
      INCLUDE 'func.inc'
C     Arguments
      INTEGER    NSTATE,LSCR
      COMPLEX*16 C0(NGWK,NSTATE,*),CX(NGWK,*),PSI(*)
      REAL*8     RHOE(*),SCR(*),TAU0(*),FION(*)
C     Variables
      DIMENSION  CATOM(NGWK,NATTOT)
      COMPLEX*16 ZONE,ZZERO
      PARAMETER  (ZZERO=(0.D0,0.D0),ZONE=(1.D0,0.D0))
      COMPLEX*16 ZMATAT(NATTOT,*),ZORKAT(*)
      REAL*8     XMATAT(NATTOT,*),EIVAT(*),
     &           WORKAT(*),SMAT(*),ISMA(*)
      POINTER    (IP_ZMATAT,ZMATAT),(IP_CX,CX),
     &           (IP_XMATAT,XMATAT),(IP_EIVAT,EIVAT),
     &           (IP_WORKAT,WORKAT),(IP_ZORKAT,ZORKAT),
     &           (IP_SMAT,SMAT),(IP_ISMA,ISMA)
      CHARACTER  TAG*30
      LOGICAL    TLSD2,DEBUG,TLSE2,TTAU2
      REAL*8     FNL(*),DFNL(*),DUMMY
      INTEGER    ISUB,KBEG,KEND,KINC,IKPT,NKPOINT,IKIND,IKK,NN,
     &           IL_XMATAT,IL_EIVAT,IL_WORKAT,IOPT,LATOMWF,
     &           NLEFT,NSMAT
      COMPLEX*16 CSCR(*)
      POINTER    (IP_CSCR,CSCR)
C     ==--------------------------------------------------------------==
#ifdef PARALLEL
      INTEGER    MSGLEN
#endif
C     ==--------------------------------------------------------------==
C     Phase factors
      CALL PHFAC(TAU0)
C     ==--------------------------------------------------------------==
      IF(NSTATE.EQ.0) RETURN
C     ==--------------------------------------------------------------==
      CALL TISET('    ATOMWF',ISUB)
C     ==--------------------------------------------------------------==
      IF(TDMAL.AND.PARENT) THEN
        WRITE(6,*) 
     &      'NOTE: ATOMIC GUESS USING DISTRIBUTED LINALG WITH LANCZOS'
      ENDIF
C     ==--------------------------------------------------------------==
C     Test SCR
      CALL GIVE_SCR_ATOMWF(LATOMWF,TAG,NSTATE)
      CALL TEST_SCR('ATOMWF','LENGTH',LSCR,LATOMWF)
C     SCR partition
C     XMATAT and ZMATAT are matrices NATTOTxNATTOT
#ifdef _BGL 
      IF(TDMAL) THEN
        IL_XMATAT=NATTOT
      ELSE
        IL_XMATAT=IMAGP*NATTOT*NATTOT+MOD(IMAGP*NATTOT*NATTOT,2)
      ENDIF
#else
      IF(TDMAL) THEN
       IL_XMATAT=NATTOT
      ELSE
       IL_XMATAT=IMAGP*NATTOT*NATTOT
      ENDIF
#endif
      CALL GET_ADDR(IP_XMATAT,SCR(1)) !is REAL*8
      CALL GET_ADDR(IP_ZMATAT,SCR(1)) !is COMPLEX*16
C     EIVAT is eigenvalues.
#ifdef _BGL 
      IL_EIVAT=IMAGP*NATTOT+MOD(IMAGP*NATTOT,2)
#else
      IL_EIVAT=IMAGP*NATTOT
#endif
C     WORKAT is an auxiliary array.
#ifdef _BGL 
      IL_WORKAT=3*NATTOT+MOD(3*NATTOT,2)
#else
      IL_WORKAT=3*NATTOT
#endif
C     Assignation
      CALL GET_ADDR(IP_EIVAT,SCR(1+IL_XMATAT))
      CALL GET_ADDR(IP_WORKAT,SCR(1+IL_XMATAT+IL_EIVAT))
      IF(TKPNT) THEN
        CALL GET_ADDR(IP_ZORKAT,SCR(1+IL_XMATAT+IL_EIVAT+IL_WORKAT))
      ELSE
        CALL GET_ADDR(IP_ZORKAT,DUMMY)
      ENDIF
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_CX,2*NGWK*NUMAORMAX,'CX')
      CALL ZAZZERO(CX,NGWK*NUMAORMAX)
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_CATOM,2*NGWK*NATTOT,'CATOM')
      IF(TKPNT) THEN
        CALL MEMORY(IP_ZXMAT,2*NATTOT*NATTOT,'ZXMAT')
        CALL MEMORY(IP_ZSMAT,2*NATTOT*NATTOT,'ZSMAT')
      ELSE
         IF (.NOT.TDMAL) THEN
           CALL MEMORY(IP_XXMAT,NATTOT*NATTOT,'XXMAT')
           CALL MEMORY(IP_XSMAT,NATTOT*NATTOT,'XSMAT')
         ENDIF
      ENDIF
      IF(PARENT) CALL PRMEM('    ATOMWF')
C     ==--------------------------------------------------------------==
C     Compute the potential and energies for XC for the core charges.
C     Need to have the correct LSD flag.
      IF(TINLC) CALL COPOT(RHOE,PSI,SCR,LSCR,.FALSE.)
C     ==--------------------------------------------------------------==
C     Backup Spin polarization flag
      TLSD2=TLSD
      TLSD=.FALSE.
      TLSE2=TLSE
      TLSE=.FALSE.
C     Backup Meta functional
      TTAU2 = TTAU
      TTAU  = .FALSE.
      IF(TTAU2) THEN
        MFXCC = 9
        TGC = .FALSE.
      ENDIF
      DEBUG=.FALSE.
C     ==--------------------------------------------------------------==
C     Charge density
      CALL ATRHO(RHOE,PSI,SCR,LSCR,NSTATE)

C     Orthogonalize atomic wavefunctions
      IF (TDMAL) THEN
        IF (TKPNT) CALL STOPGM('ATOMWF','DISTRIBUTED INITIALIZATION '
     &                        // 'NOT AVAILABLE WITH K-POINTS')
        CALL MEMORY(IP_CSCR,2*NGWK*MAX(NSTATE,NATTOT),'CSCR')
        CALL ZAZZERO(CSCR, NGWK*MAX(NSTATE,NATTOT))
        CALL GS_DISORTHO(NATTOT, CATOM(1,1), CSCR(1))
        CALL FREEM(IP_CSCR)
      ENDIF
C     The other kpoints are the same atomic wf (CATOM).
      IF(TKPNT) CALL SETKWF(NGW,NATTOT,CATOM)
C     Local potential
      CALL VOFRHO(TAU0,FION,RHOE,PSI,SCR,LSCR,.FALSE.,.FALSE.)
#ifdef PARALLEL
C     SUM ETOT, EKIN, EHT, EPSEU, ENL and EXC.
      CALL GLOSUM(6,ETOT)
#endif
      IF (TDMAL) THEN
         CALL DIST_KSMAT(CX,RHOE,PSI,SCR,LSCR,NSTATE,1,C0,
     &              MIN(NSTATE,NATTOT),TLSD2,.FALSE.)
         GOTO 100
      ENDIF
      IF (TIPAO)THEN
        IF (TLSD2)THEN
          IF(PARENT)WRITE(6,*)"INITIALISING WAVEFUNCTION WITH", 
     &      " PRIMITIVE PSEUDO ATOMIC ORBITALS"
          CALL PRIMAO(C0)
          CALL MEMORY(IP_CSCR,2*NGWK*MAX(NSTATE,NATTOT),'CSCR')
          CALL RGS(C0(1,1,1),NSUP,CSCR,1)
          CALL RGS(C0(1,NSUP+1,1),NSDOWN,CSCR,1)
          GOTO 100
        ELSE
          IF(PARENT)WRITE(6,'(A,I5,A)')
     &           ' ATOMWF| WARNING! PRIMITIVE AO METHOD ONLY WORKS ',
     &           'WITH LSD'
        ENDIF
      ENDIF

C     ==--------------------------------------------------------------==
C     Loop over k points
      CALL INQ_SWAP(KBEG,KEND,KINC)
      DO IKPT=KBEG,KEND,KINC
        NKPOINT=NKPBL(IKPT)
          IF(TKBLOCK) CALL RKPT_SWAP(DUMMY,1,IKPT,
     &         'HGKP HGKM TWNL EIGKR MASKGW')
        DO IKIND=1,NKPOINT
          IKK=KPBEG(IKPT)+IKIND
C         Kohn-Sham and Overlap matrix
          CALL KSMAT(CX,RHOE,PSI,SCR,LSCR,NSTATE,IKIND)
          IF(TIVAN) THEN
            CALL FNL_SET('SAVE')
            CALL FNLALLOC(NATTOT,.FALSE.,.FALSE.)
            CALL RNLSM(CATOM,NATTOT,PSI,SCR,LSCR,IKPT,IKIND,.FALSE.)
            CALL CSMAT(XSMAT,CATOM,FNL,NATTOT,IKIND)
#ifdef PARALLEL
            CALL SUMMAT(XXMAT,NATTOT,SCR,LSCR)
#endif
            CALL FNLDEALLOC(.FALSE.,.FALSE.)
            CALL FNL_SET('RECV')
          ELSE
            IF(TKPNT) THEN
              CALL OVLAP_H(NATTOT,ZSMAT,CATOM,CATOM)
#ifdef PARALLEL
              CALL SUMHMAT(ZXMAT,NATTOT,SCR,LSCR)
              CALL SUMHMAT(ZSMAT,NATTOT,SCR,LSCR)
#endif
            ELSE
              CALL OVLAP(NATTOT,XSMAT,CATOM,CATOM)
#ifdef PARALLEL
              CALL SUMMAT(XXMAT,NATTOT,SCR,LSCR)
              CALL SUMMAT(XSMAT,NATTOT,SCR,LSCR)
#endif
            ENDIF
          ENDIF
C         XMATAT,EIVAT,WORKAT and ZORKAT are needed only here
          CALL AZZERO(XMATAT,IL_XMATAT)
          IOPT=1
          IF(PARENT) THEN
            IF(TKPNT) THEN
              CALL ZSYGVX(IOPT,ZXMAT,NATTOT,ZSMAT,NATTOT,EIVAT,
     &             ZMATAT,NATTOT,NATTOT,ZORKAT,3*NATTOT,WORKAT)
            ELSE
              CALL DSYGVX(IOPT,XXMAT,NATTOT,XSMAT,NATTOT,EIVAT,
     &             XMATAT,NATTOT,NATTOT,WORKAT,3*NATTOT)
            ENDIF
          ENDIF
#ifdef PARALLEL
          MSGLEN = IL_XMATAT * 8
          CALL MY_BCAST(XMATAT,MSGLEN,SOURCE,ALLGRP)
#endif
          IF(TKPNT) THEN
            IF(TLSD2) THEN
              NN=MIN(NSUP,NATTOT)
              IF(NN.GT.0) 
     *        CALL ZGEMM('N','N',NGWK,NN,NATTOT,ZONE,CATOM(1,1),
     *             NGWK,ZMATAT,NATTOT,ZZERO,C0(1,1,IKIND),NGWK)
              NN=MIN(NSDOWN,NATTOT)
              IF(NN.GT.0) 
     *        CALL ZGEMM('N','N',NGWK,NN,NATTOT,ZONE,CATOM(1,1),
     *             NGWK,ZMATAT,NATTOT,ZZERO,C0(1,NSUP+1,IKIND),
     &             NGWK)
            ELSE
              NN=MIN(NSTATE,NATTOT)
              IF(NN.GT.0) 
     *        CALL ZGEMM('N','N',NGWK,NN,NATTOT,ZONE,CATOM(1,1),
     *             NGWK,ZMATAT,NATTOT,ZZERO,C0(1,1,IKIND),NGWK)
            ENDIF
C           Spherical cutoff (Clean and Reorthogonalisation).
            IF(TSPHERE) THEN
              IF(TLSD2) THEN
                NN=MIN(NSUP,NATTOT)
                IF(NN.GT.0) THEN
                  CALL C_CLEAN(C0(1,1,IKIND),NN,IKIND)
                  CALL RGS_C  (C0(1,1,IKIND),NN,ZMATAT,SCR)
                ENDIF
                NN=MIN(NSDOWN,NATTOT)
                IF(NN.GT.0) THEN
                  CALL C_CLEAN(C0(1,NSUP+1,IKIND),NN,IKIND)
                  CALL RGS_C  (C0(1,NSUP+1,IKIND),NN,ZMATAT,SCR)
                ENDIF
              ELSE
                NN=MIN(NSTATE,NATTOT)
                CALL C_CLEAN(C0(1,1,IKIND),NN,IKIND)
                CALL RGS_C  (C0(1,1,IKIND),NN,ZMATAT,SCR)
              ENDIF
            ENDIF
          ELSE
            IF(TLSD2) THEN
              NN=MIN(NSUP,NATTOT)
              IF(NN.GT.0 .AND. NGWK.GT.0)
     *        CALL DGEMM('N','N',2*NGWK,NN,NATTOT,1.0D0,CATOM(1,1),
     *             2*NGWK,XMATAT,NATTOT,0.0D0,C0(1,1,IKIND),2*NGWK)
              NN=MIN(NSDOWN,NATTOT)
              IF(NN.GT.0 .AND. NGWK.GT.0)
     *        CALL DGEMM('N','N',2*NGWK,NN,NATTOT,1.0D0,CATOM(1,1),
     *             2*NGWK,XMATAT,NATTOT,0.0D0,C0(1,NSUP+1,IKIND),
     *             2*NGWK)
            ELSE
              NN=MIN(NSTATE,NATTOT)
              IF(NN.GT.0 .AND. NGWK.GT.0)
     *        CALL DGEMM('N','N',2*NGWK,NN,NATTOT,1.0D0,CATOM(1,1),
     *             2*NGWK,XMATAT,NATTOT,0.0D0,C0(1,1,IKIND),2*NGWK)
            ENDIF
          ENDIF
C         Randomize other states and orthogonalize.
          IF((.NOT.TLSD2 .AND. NSTATE.GT.NATTOT) .OR. 
     &       (TLSD2 .AND.(NSUP.GT.NATTOT .OR. NSDOWN.GT.NATTOT) ) ) THEN
            IF(TLSD2) THEN
              NN=MAX(0,NSUP-NATTOT)+MAX(0,NSDOWN-NATTOT)
              IF(PARENT.AND.IKK.EQ.1) WRITE(6,'(A,I5,A)')
     &           ' ATOMWF| WARNING! RANDOMIZATION OF ',
     &           NN,' STATES'
              IF(NSUP-NATTOT .GT. 0) THEN
                CALL RANDTOWF(C0(1,NATTOT+1,IKIND),
     &                        NSUP-NATTOT,IKIND,IKK)
              ENDIF
              IF(NSDOWN-NATTOT .GT. 0) THEN
                CALL RANDTOWF(C0(1,NSUP+NATTOT+1,IKIND),
     &                        NSTATE-(NSUP+NATTOT),IKIND,IKK)
              ENDIF
            ELSE
              IF(PARENT.AND.IKK.EQ.1) WRITE(6,'(A,I5,A)')
     &           ' ATOMWF| WARNING! RANDOMIZATION OF ',
     &           NSTATE-NATTOT,' STATES'
              IF(NSTATE-NATTOT .GT. 0) THEN
                CALL RANDTOWF(C0(1,NATTOT+1,IKIND),
     &                        NSTATE-NATTOT,IKIND,IKK)
              ENDIF
            ENDIF
            CALL SCRSET(SCR,LSCR,DEBUG)
            IF(TKPNT) THEN
              CALL C_CLEAN(C0(1,1,IKIND),NATTOT,IKIND)
              IF(TLSD2) THEN
                NLEFT=MAX(NSUP-NATTOT,NSDOWN-NATTOT)
                NSMAT=MAX(2*NLEFT*NATTOT,2*NLEFT*NLEFT)
                CALL SCRPTR(SCR,LSCR,IP_SMAT,NSMAT,'SMAT',DEBUG)
                CALL SCRPTR(SCR,LSCR,IP_ISMA,NLEFT,'ISMA',DEBUG)
                NLEFT=NSUP-NATTOT
                IF(NLEFT.GT.0)
     *            CALL GS_ORTHO_C(C0(1,1,IKIND),NATTOT,
     *                          C0(1,NATTOT+1,IKIND),NLEFT,SMAT,ISMA)
                NLEFT=NSDOWN-NATTOT
                IF(NLEFT.GT.0)
     *            CALL GS_ORTHO_C(C0(1,NSUP+1,IKIND),NATTOT,
     *                        C0(1,NSUP+NATTOT+1,IKIND),NLEFT,SMAT,ISMA)
              ELSE
                NLEFT=NSTATE-NATTOT
                NSMAT=MAX(2*NLEFT*NATTOT,2*NLEFT*NLEFT)
                CALL SCRPTR(SCR,LSCR,IP_SMAT,NSMAT,'SMAT',DEBUG)
                CALL SCRPTR(SCR,LSCR,IP_ISMA,NLEFT,'ISMA',DEBUG)
                IF(NLEFT.GT.0)
     *          CALL GS_ORTHO_C(C0(1,1,IKIND),NATTOT,
     *                          C0(1,NATTOT+1,IKIND),NLEFT,SMAT,ISMA)
              ENDIF
            ELSE
              IF(TLSD2) THEN
                NLEFT=MAX(NSUP-NATTOT,NSDOWN-NATTOT)
                NSMAT=MAX(NLEFT*NATTOT,NLEFT*NLEFT)
                CALL SCRPTR(SCR,LSCR,IP_SMAT,NSMAT,'SMAT',DEBUG)
                CALL SCRPTR(SCR,LSCR,IP_ISMA,NLEFT,'ISMA',DEBUG)
                NLEFT=NSUP-NATTOT
                IF(NLEFT.GT.0)
     *          CALL GS_ORTHO(C0(1,1,IKIND),NATTOT,C0(1,NATTOT+1,IKIND),
     *                        NLEFT,SMAT,ISMA)
                NLEFT=NSDOWN-NATTOT
                IF(NLEFT.GT.0)
     *          CALL GS_ORTHO(C0(1,NSUP+1,IKIND),NATTOT,
     *                        C0(1,NSUP+NATTOT+1,IKIND),NLEFT,SMAT,ISMA)
              ELSE
                NLEFT=NSTATE-NATTOT
                NSMAT=MAX(NLEFT*NATTOT,NLEFT*NLEFT)
                CALL SCRPTR(SCR,LSCR,IP_SMAT,NSMAT,'SMAT',DEBUG)
                CALL SCRPTR(SCR,LSCR,IP_ISMA,NLEFT,'ISMA',DEBUG)
                IF(NLEFT.GT.0)
     *          CALL GS_ORTHO(C0(1,1,IKIND),NATTOT,C0(1,NATTOT+1,IKIND),
     *                        NLEFT,SMAT,ISMA)
              ENDIF
            ENDIF
            CALL SCRREL(SCR,LSCR,DEBUG)
          ENDIF
        ENDDO
        IF(TKBLOCK) THEN
          CALL WKPT_SWAP(C0,NSTATE,IKPT,'C0')
C         We calculate only for a k point.
          IF(TKNOSWAP) GOTO 100
        ENDIF
      ENDDO
 100  CONTINUE
      IF(TKPNT) THEN
        CALL FREEM(IP_ZXMAT)
        CALL FREEM(IP_ZSMAT)
      ELSE
         IF (.NOT.TDMAL) THEN
           CALL FREEM(IP_XXMAT)
           CALL FREEM(IP_XSMAT)
         ENDIF
      ENDIF
      CALL FREEM(IP_CATOM)
      CALL FREEM(IP_CX)
C     Spin polarization flag
      TLSD=TLSD2
      TLSE=TLSE2
      TTAU = TTAU2
      IF(TTAU) THEN
        MFXCC = 0
        TGC = .TRUE.
      END IF
      CALL TIHALT('    ATOMWF',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_ATOMWF(LATOMWF,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nlcc.inc'        !TINLC
      INCLUDE 'atwf.inc'        !NATTOT
      INCLUDE 'kpts.inc'        !TKPNT
      INCLUDE 'nlps.inc'        !IMAGP
      INCLUDE 'pslo.inc'        !TIVAN
      INCLUDE 'spin.inc'        !NLSD 
C     Arguments
      INTEGER   LATOMWF,NSTATE
      CHARACTER TAG*30
C     Variables
      INTEGER LATRHO,LCOPOT,LVOFRHO,LRNLSM,LSUMMAT,LKSMAT
      INTEGER NGSO,NLEFT
      LOGICAL TLSD2,TLSE2
C     ==--------------------------------------------------------------==
      TLSD2=TLSD
      TLSD=.FALSE.
      TLSE2=TLSE
      TLSE=.FALSE.
      LCOPOT=0
      LKSMAT=0
      LRNLSM=0
C
      CALL GIVE_SCR_ATRHO(LATRHO,TAG)
C     COPOT
      IF(TINLC) THEN
        CALL GIVE_SCR_COPOT(LCOPOT,TAG)
      ENDIF
C     VOFRHO
      CALL GIVE_SCR_VOFRHO(LVOFRHO,TAG)
      IF (TDMAL) THEN
         CALL GIVE_SCR_DIST_KSMAT(LKSMAT,TAG)
      ELSE
         CALL GIVE_SCR_KSMAT(LKSMAT,TAG)
      ENDIF
      IF(TIVAN) THEN
        CALL GIVE_SCR_RNLSM(LRNLSM,TAG,NSTATE,.FALSE.)
      ENDIF
      IF(TDMAL) THEN
        LATOMWF=IMAGP*NATTOT +             !OBSOLETE: XMATAT or ZMATAT
     &        IMAGP*NATTOT+                    !EIVAT
     &        IMAGP*3*NATTOT+10                !WORKAT
      ELSE
        LATOMWF=IMAGP*NATTOT*NATTOT+           !XMATAT or ZMATAT
     &        IMAGP*NATTOT+                    !EIVAT
     &        IMAGP*3*NATTOT+10                !WORKAT
      ENDIF
      IF(TKPNT) LATOMWF=LATOMWF+3*NATTOT       !ZORKAT
      IF(NSTATE.GT.NLSD*NATTOT) THEN
        NLEFT=NSTATE-NLSD*NATTOT
        NGSO=MAX(NLEFT**2,NLEFT*NATTOT)+NLEFT
        LATOMWF=MAX(LATOMWF,NGSO)
      ENDIF
      IF(TDMAL) THEN
        LSUMMAT=0
      ELSE
        CALL GIVE_SCR_SUMMAT(LSUMMAT,TAG,NATTOT)
      ENDIF
      LATOMWF=MAX(LATRHO,LATOMWF,LCOPOT,LVOFRHO,LRNLSM,LSUMMAT,LKSMAT)
C
      TLSD=TLSD2
      TLSE=TLSE2
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE DSYGVX(IOPT,A,LDA,B,LDB,W,Z,LDZ,N,AUX,NAUX)
C     ==--------------------------------------------------------------==
C     == GENERALIZED DIAGONALIZATION ROUTINE:                         ==
C     == FOLLOW THE ESSL CONVENTION                                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER  IOPT,LDA,LDB,LDZ,N,NAUX
      REAL*8   A(LDA,N),B(LDB,N),W(N),Z(LDZ,N),AUX(NAUX)
C     Variables
      INTEGER  INFO,I
C     ==--------------------------------------------------------------==
      CALL TEST_SCR('DSYGVX','NAUX',NAUX,3*N)
      IF(IOPT.EQ.1) THEN
        INFO=0
        CALL DSYGV(1,'V','U',N,A,LDA,B,LDB,W,AUX,NAUX,INFO)
        DO I=1,N
          CALL DCOPY(N,A(1,I),1,Z(1,I),1)
        ENDDO
      ELSE
        CALL STOPGM('DSYGVX','MISSING LIBRARY OPTION')
      ENDIF
      IF(INFO.NE.0) THEN
        WRITE(6,'(/," DSYGVX| INFO=",I5)') INFO
        IF(INFO.LT.0) THEN
          WRITE(6,'(" DSYGVX| THE ",I5,A)')
     &         INFO,'TH ARGUMENT HAS AN ILLEGAL VALUE'
        ELSEIF(INFO.LE.N) THEN
          WRITE(6,
     &         '(" DSYGVX| FAILED TO CONVERGE WHEN DIAGONALIZING A")')
        ELSEIF(INFO.GT.N) THEN
          WRITE(6,'(" DSYGVX| PB WITH OVERLAP MATRIX FROM CATOM")')
          WRITE(6,'(/,"-> CHECK IF YOUR CELL IS NOT TOO SMALL!!!")')
        ENDIF
        CALL STOPGM ('DSYGVX','FAILED TO DIAGONALIZE')
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE ZSYGVX(IOPT,A,LDA,B,LDB,W,Z,LDZ,N,AUX,NAUX,RWORK)
C     ==--------------------------------------------------------------==
C     == GENERALIZED DIAGONALIZATION ROUTINE:                         ==
C     == FOLLOW DSYGVX CONVENTION                                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER IOPT,LDA,LDB,LDZ,N,NAUX
      COMPLEX*16 A(LDA,N),B(LDB,N),Z(LDZ,N),AUX(NAUX)
      REAL*8 W(N),RWORK(NAUX)
C     Variables
      INTEGER INFO, I
C     ==--------------------------------------------------------------==
      CALL TEST_SCR('ZSYGVX','NAUX',NAUX,3*N)
      INFO=0
      CALL ZHEGV(IOPT,'V','U',N,A,LDA,B,LDB,W,AUX,NAUX,RWORK,INFO)
      DO I=1,N
#ifdef __NEC
        CALL ZCOPY(N,A(1,I),1,Z(1,I),1)
#else
        CALL DCOPY(2*N,A(1,I),1,Z(1,I),1)
#endif
      ENDDO
      IF(INFO.NE.0) THEN
        WRITE(6,'(/," ZSYGVX| INFO=",I5)') INFO
        IF(INFO.LT.0) THEN
          WRITE(6,'(" ZSYGVX| THE ",I5,A)')
     &         INFO,'TH ARGUMENT HAS AN ILLEGAL VALUE'
        ELSEIF(INFO.LE.N) THEN
          WRITE(6,
     &         '(" ZSYGVX| FAILED TO CONVERGE WHEN DIAGONALIZING A")')
        ELSEIF(INFO.GT.N) THEN
          WRITE(6,'(" ZSYGVX| PB WITH OVERLAP MATRIX FROM CATOM")')
          WRITE(6,'(/,"-> CHECK IF YOUR CELL IS NOT TOO SMALL!!!")')
        ENDIF
        CALL STOPGM ('ZSYGVX','FAILED TO DIAGONALIZE')
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE PRIMAO(C0)
C     ==--------------------------------------------------------------==
C     == Initialise wavefunctions with atomic orbitals                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'elct.inc'

      DIMENSION  CATOM(NGWK,NATTOT)
      COMPLEX*16 C0(NGWK,N,*)
      INTEGER    OC1

      INTEGER    IS,IA,ISH,IUP,IDOWN,OCU,NUM,O,LS,IAT
C TODO FIGURE OUT LS
      IUP=1
      IAT=1
      IDOWN=NSUP+1
      DO IS=1,NSP
        DO IA=1,NA(IS)
          DO ISH=1,NSHELL(IS)
            OCU=OC(ISH,IS)
            LS=CATORD(ISH,IAT)
            NUM=MODULO(OCU,2)
            DO O=1,OCU-NUM,2
              CALL DCOPY(2*NGWK,CATOM(1,LS),1,C0(1,IUP,1),1)
              IUP=IUP+1
              CALL DCOPY(2*NGWK,CATOM(1,LS),1,C0(1,IDOWN,1),1)
              IDOWN=IDOWN+1
              LS=LS+1
            ENDDO
            IF(NUM.EQ.1)THEN
              CALL ZCOPY(NGWK,CATOM(1,LS),1,C0(1,IDOWN,1),1)
              IDOWN=IDOWN+1
            ENDIF
          ENDDO !Shells
          IAT=IAT+1
        ENDDO !Atoms
      ENDDO !Species

      IF(NSUP.NE.IUP-1)THEN
        IF(PARENT)WRITE(6,'(A,I5,A,I5,A)') "PRIMAO| GOT",IUP-1,
     &         "INSTEAD OF",NSUP," UP ELECTRONS"
        CALL STOPGM( 'PRIMAO', 'ILLEGAL RESULTS')
      ELSEIF(N.NE.IDOWN-1)THEN
        IF(PARENT)WRITE(6,'(A,I5,A,I5,A)') "PRIMAO| GOT",IDOWN-1-NSUP,
     &         "INSTEAD OF",NSDOWN," DOWN ELECTRONS"
        CALL STOPGM( 'PRIMAO', 'ILLEGAL RESULTS')
      ENDIF

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
