C     ==================================================================
C     AZZERO(A,N) -> A(I)=0 I=1..N for A real*8
C In order to inline DCOPY I add this module
C The problem with inlining azzero is that it's called with real
C and complex variables, so no inlining possible.
C This module allows inlining without any direct code changes.
C The CRAY X1 compiler can use the -A compiler to have an
C implicit USE MODAZZERO in each routine
C (wouldn't have been needed if ZAZZERO had been used)
      MODULE MODAZZERO

      INTERFACE AZZERO
       module procedure AZZERO8, AZZERO16, LAZERO4
      END INTERFACE AZZERO

      CONTAINS

C     ==================================================================
      SUBROUTINE AZZERO8(A,N)
C     ==================================================================
#ifdef CRAYX1
!DIR$ ignore_tkr (r) a
#endif
      IMPLICIT NONE
C     Arguments
      INTEGER N
      REAL*8 A(N)
C     Variables
      INTEGER I

C$OMP parallel do private (I)
      DO I=1,N
        A(I)=0.0D0
      ENDDO
      RETURN
      END SUBROUTINE

C     ==================================================================
      SUBROUTINE AZZERO16(A,N)
C     ==================================================================
#ifdef CRAYX1
!DIR$ ignore_tkr (r) a
#endif
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER N
      complex*16 A(*)
C     Variables
      INTEGER I, NN
C     ==--------------------------------------------------------------==
      NN=N/2
C$OMP parallel do private (I)
      DO I=1,NN
        A(I)=(0.0d0, 0.0d0)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END SUBROUTINE

C     ==================================================================
      SUBROUTINE LAZERO4(A,N)
C     ==================================================================
#ifdef CRAYX1
!DIR$ ignore_tkr (r) a
#endif
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER N
      logical*4 A(*)
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
      call IgeneralAZERO4(A,N)
C     ==--------------------------------------------------------------==
      RETURN
      END SUBROUTINE

C     ==================================================================
      SUBROUTINE IgeneralAZERO4(A,N)
C     ==================================================================
#ifdef CRAYX1
!DIR$ ignore_tkr (tkr) a
#endif
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER N
      integer*4 A(*)
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
C$OMP parallel do private (I)
      DO I=1,N
        A(I)=0
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END SUBROUTINE



C     ==================================================================
C     ZAZZERO(A,N) -> A(I)=0 I=1..N for A complex*16
C     ==================================================================
      SUBROUTINE ZAZZERO(A,N)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER N
      COMPLEX*16 A(N)
C     Variables
      INTEGER I
      COMPLEX*16 ZZERO
      PARAMETER  (ZZERO=(0.D0,0.D0))
C     ==--------------------------------------------------------------==
C$OMP parallel do private (I)
      DO I=1,N
        A(I)=ZZERO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END SUBROUTINE

C     ==================================================================
C     IAZZERO(IA,N) -> IA(I)=0 I=1..N for IA default integer
C     ==================================================================
      SUBROUTINE IAZZERO(IA,N)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER N
      INTEGER IA(N)
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
C$omp parallel do private(I)
      DO I=1,N
        IA(I)=0
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END SUBROUTINE

C     ==================================================================
C     I8AZZERO(IA,N) -> IA(I)=0 I=1..N for IA integer*8
C     ==================================================================
      SUBROUTINE I8AZZERO(IA,N)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER N
      INTEGER*8 IA(N)
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
C$omp parallel do private(I)
      DO I=1,N
        IA(I)=0
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END SUBROUTINE

      END MODULE MODAZZERO
