#ifdef __SR8000
*option MP(P(0)), LANGLVL(SAVE(0))
#endif
C
C     .....................................................GETF.........
      SUBROUTINE GETF(NR,F,X,RES)
C     **                                                              **
C     **  CALCULATES THE VALUE RES OF THE FUNCTION F AT X             **
C     **  BY INTERPOLATION BY A THIRD ORDER POLYNOM                   **
C     **  F(X=I)=F(I)                                                 **
C     **                                                              **
C     **          P.E. BLOECHL, IBM RESEARCH LABORATORY ZURICH (1991) **
C     **                                                              **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION F(NR)
      INCR=INT(X)
      INCR=MIN0(INCR,NR-2)
      INCR=MAX0(INCR,2)
      XX1=X-DBLE(INCR-1)
      XX2=XX1-1.D0
      XX3=XX2-1.D0
      XX4=XX3-1.D0
      P1=F(INCR-1)
      P2=F(INCR)
      P3=F(INCR+1)
      P4=F(INCR+2)
      P21=-XX2*P1+XX1*P2
      P32=-XX3*P2+XX2*P3
      P43=-XX4*P3+XX3*P4
      P321=0.5D0*( -XX3*P21+XX1*P32 )
      P432=0.5D0*( -XX4*P32+XX2*P43 )
      P4321=1.D0/3.D0 * ( -XX4*P321+XX1*P432 )
      RES=P4321
      RETURN
      END
C
C     .....................................................BESSOV.......
      SUBROUTINE BESSOV(R1,DEX,NR,F,L,G,RMAX,RES)
C     **                                                              **
C     **  CALCULATES THE OVERLAP BETWEEN A SPHERICAL BESSEL FUNCTION  **
C     **  WITH THE FUNCTION F GIVEN ON AN LOGARITHMIC GRID            **
C     **  (3-D OVERLAPP)                                              **
C     **                                                              **
C     **          P.E. BLOECHL, IBM RESEARCH LABORATORY ZURICH (1991) **
C     **                                                              **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION F(NR),FAC(4)
      EXTERNAL BESSL
      NSTEP=INT(20.D0*G/6.D0)
      NSTEP=MAX0(NSTEP,400)
      RSTEP=RMAX/NSTEP
      FAC(1)=RSTEP*17.D0/48.D0
      FAC(2)=RSTEP*59.D0/48.D0
      FAC(3)=RSTEP*43.D0/48.D0
      FAC(4)=RSTEP*49.D0/48.D0
      X=1.D0+DLOG(RMAX/R1) / DEX
      CALL GETF(NR,F,X,VAL)
      RES=FAC(1)*VAL*BESSL(L,RMAX*G)*RMAX**2
      DO I=2,4
        R=RSTEP*DBLE(I-1)
        X=1.D0+DLOG(R/R1) / DEX
        CALL GETF(NR,F,X,VAL)
        RES=RES+FAC(I)*VAL*BESSL(L,R*G)*R**2
        R=RMAX-R
        X=1.D0+DLOG(R/R1) / DEX
        CALL GETF(NR,F,X,VAL)
        RES=RES+FAC(I)*VAL*BESSL(L,R*G)*R**2
      ENDDO
      DO IR=5,NSTEP-3
        R=RSTEP*DBLE(IR-1)
        X=1.D0+DLOG(R/R1) / DEX
        CALL GETF(NR,F,X,VAL)
        RES=RES+RSTEP*VAL*BESSL(L,R*G)*R**2
      ENDDO
      RETURN
      END
C
C     ..............................................BESSL...............
      REAL*8 FUNCTION BESSL(L,X)
C     **                                                              **
C     **  CALCULATES THE SPHERICAL BESSEL FUNCTION                    **
C     **  ACCORDING TO ABRAMOWITZ AND STEGUN                          **
C     **    FORMULA 10.1.2              FOR   X < 8                   **
C     **    FORMULA 10.1.8 AND  10.1.9  FOR   X > 8                   **
C     **                                                              **
C     **          P.E. BLOECHL, IBM RESEARCH LABORATORY ZURICH (1991) **
C     **                                                              **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION TRIG(4),FACUL(0:100)
      DATA TOL/1.D-10/
      IF(X.GT.DBLE(L)) THEN
        PI=4.D0*DATAN(1.D0)
        ARG=X-0.5D0*DBLE(L)*PI
        TRIG(1)=DSIN(ARG)/X
        TRIG(2)=DCOS(ARG)/X
        TRIG(3)=-TRIG(1)
        TRIG(4)=-TRIG(2)
        BESSL=TRIG(1)
        IF(L.EQ.0) RETURN
        FACUL(0)=1.D0
        DO I=1,2*L
          FACUL(I)=FACUL(I-1)*DBLE(I)
        ENDDO
        XSQ=0.5D0/X
        FAC=1.D0
#ifdef _vpp_
!OCL SCALAR 
#endif 
        DO K=1,L
          II=MOD(K,4)+1
          FAC=FACUL(K+L)/FACUL(K)/FACUL(L-K)*XSQ**K
C         FAC=FAC*XSQ*DBLE(L+K)/DBLE(K*(L-K))
          BESSL=BESSL+FAC*TRIG(II)
        ENDDO
C       II=MOD(L,4)+1
C       FAC=FAC*XSQ*DBLE(2*L)/DBLE(L)
C       BESSL=BESSL+FAC*TRIG(II)
        RETURN
      END IF
C     ==================================================================
C     ==  TAYLOR EXPANSION FOR SMALL ARGUMENTS                        ==
C     ==================================================================
      ISVAR=1
      DO IL=1,L
        ISVAR=ISVAR*(2*IL+1)
      ENDDO
      IF(L.NE.0.D0) THEN
        FAC=X**L/DBLE(ISVAR)
      ELSE
        FAC=1.D0/DBLE(ISVAR)
      END IF
      BESSL=FAC
      XSQ=-0.5D0*X*X
      ISVAR=2*L+1
      DO I=1,1000
        ISVAR=ISVAR+2
        FAC=FAC*XSQ/DBLE(I*ISVAR)
        BESSL=BESSL+FAC
        IF(DABS(FAC).LT.TOL) GOTO 9999
      ENDDO
      PRINT*,'BESSL NOT CONVERGED X=',X,' L= ',L
      CALL STOPGM('BESSL',' ')
9999  CONTINUE
      RETURN
      END
