C     ==================================================================
      SUBROUTINE DGEMM(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC)
C     ==--------------------------------------------------------------==
C     == Purpose                                                      ==
C     == =======                                                      ==
C     ==                                                              ==
C     == DGEMM performs one of the matrix-matrix operations           ==
C     ==                                                              ==
C     == C := alpha*op(A)*op(B)+beta*C,                               ==
C     ==                                                              ==
C     ==   where op(X) is one of                                      ==
C     ==                                                              ==
C     ==   op(X) = X or op(X) = X',                                   ==
C     ==                                                              ==
C     == alpha and beta are scalars, and A, B and C are matrices,     ==
C     == with op(A) an m by k matrix, op(B) a k by n matrix           ==
C     == and C an m by n matrix.                                      ==
C     ==                                                              ==
C     == Parameters                                                   ==
C     == ==========                                                   ==
C     ==                                                              ==
C     == TRANSA - CHARACTER*1.                                        ==
C     ==   On entry, TRANSA specifies the form of op(A) to be used    ==
C     ==   in the matrix multiplication as follows:                   ==
C     ==                                                              ==
C     ==     TRANSA = 'N' or 'n', op(A) = A.                          ==
C     ==                                                              ==
C     ==     TRANSA = 'T' or 't', op(A) = A'.                         ==
C     ==                                                              ==
C     ==     TRANSA = 'C' or 'c', op(A) = A'.                         ==
C     ==                                                              ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     == TRANSB - CHARACTER*1.                                        ==
C     ==   On entry, TRANSB specifies the form of op(B) to be used    ==
C     ==   in the matrix multiplication as follows:                   ==
C     ==                                                              ==
C     ==     TRANSB = 'N' or 'n', op(B) = B.                          ==
C     ==                                                              ==
C     ==     TRANSB = 'T' or 't', op(B) = B'.                         ==
C     ==                                                              ==
C     ==     TRANSB = 'C' or 'c', op(B) = B'.                         ==
C     ==                                                              ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     == M - INTEGER.                                                 ==
C     ==   On entry, M specifies the number of rows of the matrix     ==
C     ==   op(A) and of the matrix C. M must be at least zero.        ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     == N - INTEGER.                                                 ==
C     ==   On entry, N specifies the number of columns of the matrix  ==
C     ==   op(B) and the number of columns of the matrix C. N must    ==
C     ==   be at least zero.                                          ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     == K - INTEGER.                                                 ==
C     ==   On entry, K specifies the number of columns of the matrix  ==
C     ==   op(A) and the number of rows of the matrix op(B). K must   ==
C     ==   be at least  zero.                                         ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     == ALPHA - DOUBLE PRECISION.                                    ==
C     ==   On entry, ALPHA specifies the scalar alpha.                ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     == A - DOUBLE PRECISION array of DIMENSION (LDA,ka), where ka   ==
C     ==   is k when TRANSA = 'N' or 'n', and is m otherwise.         ==
C     ==   Before entry with TRANSA = 'N' or 'n', the leading m by k  ==
C     ==   part of the array A must contain the matrix A, otherwise   ==
C     ==   the leading k by m part of the array A must contain the    ==
C     ==   matrix A.                                                  ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     == LDA - INTEGER.                                               ==
C     ==   On entry, LDA specifies the first dimension of A as        ==
C     ==   declared in the calling (sub) program. When TRANSA = 'N'   ==
C     ==   or 'n' then LDA must be at least max(1,m), otherwise LDA   ==
C     ==   must be at least max(1,k).                                 ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     == B - DOUBLE PRECISION array of DIMENSION (LDB,kb), where kb   ==
C     ==   is n when TRANSB = 'N' or 'n', and is k otherwise.         ==
C     ==   Before entry with TRANSB = 'N' or 'n', the leading k by n  ==
C     ==   part of the array B must contain the matrix B, otherwise   ==
C     ==   the leading n by k part of the array B must contain the    ==
C     ==   matrix B.                                                  ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     == LDB - INTEGER.                                               ==
C     ==   On entry, LDB specifies the first dimension of B as        ==
C     ==   declared in the calling (sub) program. When TRANSB = 'N'   ==
C     ==   or 'n' then LDB must be at least max(1,k), otherwise LDB   ==
C     ==   must be at least max(1,n).                                 ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     == BETA - DOUBLE PRECISION.                                     ==
C     ==   On entry, BETA specifies the scalar beta. When BETA is     ==
C     ==   supplied as zero then C need not be set on input.          ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     == C - DOUBLE PRECISION array of DIMENSION (LDC,n). Before      ==
C     ==   entry, the leading m by n part of the array C must contain ==
C     ==   the matrix C, except when beta is zero, in which case C    ==
C     ==   need not be set on entry. On exit, the array C is          ==
C     ==   overwritten by the m by n matrix                           ==
C     ==   (alpha*op(A)*op(B)+beta*C).                                ==
C     ==                                                              ==
C     == LDC - INTEGER.                                               ==
C     ==   On entry, LDC specifies the first dimension of C as        ==
C     ==   declared in the calling (sub) program. LDC must be         ==
C     ==   at least max(1,m).                                         ==
C     ==   Unchanged on exit.                                         ==
C     ==                                                              ==
C     ==                                                              ==
C     == Level 3 Blas routine.                                        ==
C     ==                                                              ==
C     ==  -- Written on 8-February-1989.                              ==
C     ==     Jack Dongarra, Argonne National Laboratory.              ==
C     ==     Iain Duff, AERE Harwell.                                 ==
C     ==     Jeremy Du Croz, Numerical Algorithms Group Ltd.          ==
C     ==     Sven Hammarling, Numerical Algorithms Group Ltd.         ==
C     ==--------------------------------------------------------------==
C     Scalar Arguments
      CHARACTER*1      TRANSA,TRANSB
      INTEGER          M,N,K,LDA,LDB,LDC
      DOUBLE PRECISION ALPHA,BETA
C     Array Arguments
      DOUBLE PRECISION A(LDA,*),B(LDB,*),C(LDC,*)
C     External Functions
      LOGICAL          LSAME
      EXTERNAL         LSAME
C     External Subroutines
      EXTERNAL         XERBLA
C     Intrinsic Functions
      INTRINSIC        MAX
C     Local Scalars
      LOGICAL          NOTA,NOTB
      INTEGER          I,INFO,J,L,NCOLA,NROWA,NROWB
      DOUBLE PRECISION TEMP
C     Parameters
      DOUBLE PRECISION ONE,ZERO
      PARAMETER       (ONE=1.0D+0,ZERO=0.0D+0)
C     Auxiliary variables 
      INTEGER          MODJ,MODL,MODI
      DOUBLE PRECISION T00,T01,T02,T03,T04,T05,T06,T07
      DOUBLE PRECISION U00,U01,U02,U03,U04,U05,U06,U07,
     &                 U08,U09,U10,U11,U12,U13,U14,U15
      DOUBLE PRECISION V00,V01,V02,V03,V04,V05,V06,V07,
     &                 V08,V09,V10,V11,V12,V13,V14,V15
C     ==--------------------------------------------------------------==
C
C     Set NOTA and NOTB as true if A and B respectively are not
C     transposed and set NROWA, NCOLA and NROWB as the number of 
C     rows and columns of A and the number of rows of B respectively.
C     
      NOTA=LSAME(TRANSA,'N')
      NOTB=LSAME(TRANSB,'N')
      IF(NOTA) THEN
        NROWA=M
        NCOLA=K
      ELSE
        NROWA=K
        NCOLA=M
      ENDIF
      IF(NOTB) THEN
        NROWB=K
      ELSE
        NROWB=N
      ENDIF
C
C     Test the input parameters
C
      INFO=0
      IF((.NOT.NOTA).AND.
     $   (.NOT.LSAME(TRANSA,'C')).AND.
     $   (.NOT.LSAME(TRANSA,'T'))) THEN
        INFO=1
      ELSE IF((.NOT.NOTB).AND.
     $        (.NOT.LSAME(TRANSB,'C')).AND.
     $        (.NOT.LSAME(TRANSB,'T'))) THEN
        INFO=2
      ELSE IF(M.LT.0) THEN
        INFO=3
      ELSE IF(N.LT.0) THEN
        INFO=4
      ELSE IF(K.LT.0)THEN
        INFO=5 
      ELSE IF(LDA.LT.MAX(1,NROWA)) THEN
        INFO=8
      ELSE IF(LDB.LT.MAX(1,NROWB)) THEN
        INFO=10
      ELSE IF(LDC.LT.MAX(1,M)) THEN
        INFO=13
      ENDIF
      IF(INFO.NE.0) THEN
        CALL XERBLA('DGEMM ',INFO)
        RETURN
      ENDIF
C       
C     Quick return if possible
C
      IF((M.EQ.0).OR.(N.EQ.0).OR.
     $  (((ALPHA.EQ.ZERO).OR.(K.EQ.0)).AND.(BETA.EQ.ONE)))
     $  RETURN
C
C     And if alpha.eq.zero
C
      IF(ALPHA.EQ.ZERO) THEN
        IF(BETA.EQ.ZERO) THEN
!$OMP parallel do private(J,I)
          DO J=1,N
            DO I=1,M
              C(I,J)=ZERO
            ENDDO
          ENDDO
        ELSE
!$OMP parallel do private(J,I)
          DO J=1,N
            DO I=1,M
              C(I,J)=BETA*C(I,J)
            ENDDO
          ENDDO
        ENDIF
        RETURN
      ENDIF
C
C     Start the operations
C
      IF(NOTB) THEN
        IF(NOTA) THEN
          IF(BETA.EQ.ZERO) THEN 
C     ==--------------------------------------------------------------==
C     == Form C := alpha*A*B                                          ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,2)
            MODL=MOD(K,16)
!$OMP parallel do private(J,I,L,TEMP,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15)
            DO J=1,MODJ
              DO I=1,M
                C(I,J)=ZERO
              ENDDO
              DO L=1,MODL
                TEMP=ALPHA*B(L,J)
                DO I=1,M
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
              DO L=MODL+1,K,16
                U00=ALPHA*B(L   ,J)
                U01=ALPHA*B(L+ 1,J)
                U02=ALPHA*B(L+ 2,J)
                U03=ALPHA*B(L+ 3,J)
                U04=ALPHA*B(L+ 4,J)
                U05=ALPHA*B(L+ 5,J)
                U06=ALPHA*B(L+ 6,J)
                U07=ALPHA*B(L+ 7,J)
                U08=ALPHA*B(L+ 8,J)
                U09=ALPHA*B(L+ 9,J)
                U10=ALPHA*B(L+10,J)
                U11=ALPHA*B(L+11,J)
                U12=ALPHA*B(L+12,J)
                U13=ALPHA*B(L+13,J)
                U14=ALPHA*B(L+14,J)
                U15=ALPHA*B(L+15,J)
                DO I=1,M
                  C(I,J)=C(I,J)+U00*A(I,L   )+U01*A(I,L+ 1)
     &                         +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                         +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                         +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                         +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                         +U10*A(I,L+10)+U11*A(I,L+11)
     &                         +U12*A(I,L+12)+U13*A(I,L+13)
     &                         +U14*A(I,L+14)+U15*A(I,L+15)
                ENDDO
              ENDDO
            ENDDO  
!$OMP parallel do private(J,I,L,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15,
!$OMP+ V00,V01,V02,V03,V04,V05,V06,V07,
!$OMP+ V08,V09,V10,V11,V12,V13,V14,V15)
            DO J=1+MODJ,N,2
              DO I=1,M     
                C(I,J  )=ZERO
                C(I,J+1)=ZERO
              ENDDO          
              DO L=1,MODL
                U00=ALPHA*B(L,J  )
                V00=ALPHA*B(L,J+1)
                DO I=1,M
                  C(I,J  )=C(I,J  )+U00*A(I,L)
                  C(I,J+1)=C(I,J+1)+V00*A(I,L)
                ENDDO
              ENDDO  
              DO L=MODL+1,K,16
                U00=ALPHA*B(L   ,J  )
                U01=ALPHA*B(L+ 1,J  )
                U02=ALPHA*B(L+ 2,J  )
                U03=ALPHA*B(L+ 3,J  )
                U04=ALPHA*B(L+ 4,J  )
                U05=ALPHA*B(L+ 5,J  )
                U06=ALPHA*B(L+ 6,J  )
                U07=ALPHA*B(L+ 7,J  )
                U08=ALPHA*B(L+ 8,J  )
                U09=ALPHA*B(L+ 9,J  )
                U10=ALPHA*B(L+10,J  )
                U11=ALPHA*B(L+11,J  )
                U12=ALPHA*B(L+12,J  )
                U13=ALPHA*B(L+13,J  )
                U14=ALPHA*B(L+14,J  )
                U15=ALPHA*B(L+15,J  )
                V00=ALPHA*B(L   ,J+1)
                V01=ALPHA*B(L+ 1,J+1)
                V02=ALPHA*B(L+ 2,J+1)
                V03=ALPHA*B(L+ 3,J+1)
                V04=ALPHA*B(L+ 4,J+1)
                V05=ALPHA*B(L+ 5,J+1)
                V06=ALPHA*B(L+ 6,J+1)
                V07=ALPHA*B(L+ 7,J+1)
                V08=ALPHA*B(L+ 8,J+1)
                V09=ALPHA*B(L+ 9,J+1)
                V10=ALPHA*B(L+10,J+1)
                V11=ALPHA*B(L+11,J+1)
                V12=ALPHA*B(L+12,J+1)
                V13=ALPHA*B(L+13,J+1)
                V14=ALPHA*B(L+14,J+1)
                V15=ALPHA*B(L+15,J+1)
                DO I=1,M
                  C(I,J  )=C(I,J  )+U00*A(I,L   )+U01*A(I,L+ 1)
     &                             +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                             +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                             +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                             +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                             +U10*A(I,L+10)+U11*A(I,L+11)
     &                             +U12*A(I,L+12)+U13*A(I,L+13)
     &                             +U14*A(I,L+14)+U15*A(I,L+15)
                  C(I,J+1)=C(I,J+1)+V00*A(I,L   )+V01*A(I,L+ 1)
     &                             +V02*A(I,L+ 2)+V03*A(I,L+ 3)
     &                             +V04*A(I,L+ 4)+V05*A(I,L+ 5)
     &                             +V06*A(I,L+ 6)+V07*A(I,L+ 7)
     &                             +V08*A(I,L+ 8)+V09*A(I,L+ 9)
     &                             +V10*A(I,L+10)+V11*A(I,L+11)
     &                             +V12*A(I,L+12)+V13*A(I,L+13)
     &                             +V14*A(I,L+14)+V15*A(I,L+15)
                ENDDO
              ENDDO  
            ENDDO 
          ELSE IF(BETA.EQ.ONE) THEN 
C     ==--------------------------------------------------------------==
C     == Form C := alpha*A*B+C                                        ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,2)
            MODL=MOD(K,16)
!$OMP parallel do private(J,I,L,TEMP,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15)
            DO J=1,MODJ
              DO L=1,MODL
                TEMP=ALPHA*B(L,J)
                DO I=1,M         
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO  
              DO L=MODL+1,K,16
                U00=ALPHA*B(L   ,J)
                U01=ALPHA*B(L+ 1,J)
                U02=ALPHA*B(L+ 2,J)
                U03=ALPHA*B(L+ 3,J)
                U04=ALPHA*B(L+ 4,J)
                U05=ALPHA*B(L+ 5,J)
                U06=ALPHA*B(L+ 6,J)
                U07=ALPHA*B(L+ 7,J)
                U08=ALPHA*B(L+ 8,J)
                U09=ALPHA*B(L+ 9,J)
                U10=ALPHA*B(L+10,J)
                U11=ALPHA*B(L+11,J)
                U12=ALPHA*B(L+12,J)
                U13=ALPHA*B(L+13,J)
                U14=ALPHA*B(L+14,J)
                U15=ALPHA*B(L+15,J)
                DO I=1,M
                  C(I,J)=C(I,J)+U00*A(I,L   )+U01*A(I,L+ 1)
     &                         +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                         +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                         +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                         +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                         +U10*A(I,L+10)+U11*A(I,L+11)
     &                         +U12*A(I,L+12)+U13*A(I,L+13)
     &                         +U14*A(I,L+14)+U15*A(I,L+15)
                ENDDO
              ENDDO  
            ENDDO  
!$OMP parallel do private(J,I,L,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15,
!$OMP+ V00,V01,V02,V03,V04,V05,V06,V07,
!$OMP+ V08,V09,V10,V11,V12,V13,V14,V15)
            DO J=MODJ+1,N,2
              DO L=1,MODL  
                U00=ALPHA*B(L,J  )
                V00=ALPHA*B(L,J+1)
                DO I=1,M
                  C(I,J  )=C(I,J  )+U00*A(I,L)
                  C(I,J+1)=C(I,J+1)+V00*A(I,L)
                ENDDO
              ENDDO  
              DO L=MODL+1,K,16
                U00=ALPHA*B(L   ,J  )
                U01=ALPHA*B(L+ 1,J  )
                U02=ALPHA*B(L+ 2,J  )
                U03=ALPHA*B(L+ 3,J  )
                U04=ALPHA*B(L+ 4,J  )
                U05=ALPHA*B(L+ 5,J  )
                U06=ALPHA*B(L+ 6,J  )
                U07=ALPHA*B(L+ 7,J  )
                U08=ALPHA*B(L+ 8,J  )
                U09=ALPHA*B(L+ 9,J  )
                U10=ALPHA*B(L+10,J  )
                U11=ALPHA*B(L+11,J  )
                U12=ALPHA*B(L+12,J  )
                U13=ALPHA*B(L+13,J  )
                U14=ALPHA*B(L+14,J  )
                U15=ALPHA*B(L+15,J  )
                V00=ALPHA*B(L   ,J+1)
                V01=ALPHA*B(L+ 1,J+1)
                V02=ALPHA*B(L+ 2,J+1)
                V03=ALPHA*B(L+ 3,J+1)
                V04=ALPHA*B(L+ 4,J+1)
                V05=ALPHA*B(L+ 5,J+1)
                V06=ALPHA*B(L+ 6,J+1)
                V07=ALPHA*B(L+ 7,J+1)
                V08=ALPHA*B(L+ 8,J+1)
                V09=ALPHA*B(L+ 9,J+1)
                V10=ALPHA*B(L+10,J+1)
                V11=ALPHA*B(L+11,J+1)
                V12=ALPHA*B(L+12,J+1)
                V13=ALPHA*B(L+13,J+1)
                V14=ALPHA*B(L+14,J+1)
                V15=ALPHA*B(L+15,J+1)
                DO I=1,M
                  C(I,J  )=C(I,J  )+U00*A(I,L   )+U01*A(I,L+ 1)
     &                             +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                             +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                             +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                             +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                             +U10*A(I,L+10)+U11*A(I,L+11)
     &                             +U12*A(I,L+12)+U13*A(I,L+13)
     &                             +U14*A(I,L+14)+U15*A(I,L+15)
                  C(I,J+1)=C(I,J+1)+V00*A(I,L   )+V01*A(I,L+ 1)
     &                             +V02*A(I,L+ 2)+V03*A(I,L+ 3)
     &                             +V04*A(I,L+ 4)+V05*A(I,L+ 5)
     &                             +V06*A(I,L+ 6)+V07*A(I,L+ 7)
     &                             +V08*A(I,L+ 8)+V09*A(I,L+ 9)
     &                             +V10*A(I,L+10)+V11*A(I,L+11)
     &                             +V12*A(I,L+12)+V13*A(I,L+13)
     &                             +V14*A(I,L+14)+V15*A(I,L+15)
                ENDDO
              ENDDO  
            ENDDO  
          ELSE 
C     ==--------------------------------------------------------------==
C     == Form C := alpha*A*B+beta*C                                   ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,2)
            MODL=MOD(K,16)
!$OMP parallel do private(J,I,L,TEMP,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15)
            DO J=1,MODJ
              DO I=1,M      
                C(I,J)=BETA*C(I,J)
              ENDDO
              DO L=1,MODL
                TEMP=ALPHA*B(L,J)
                DO I=1,M         
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO  
              DO L=MODL+1,K,16
                U00=ALPHA*B(L   ,J)
                U01=ALPHA*B(L+ 1,J)
                U02=ALPHA*B(L+ 2,J)
                U03=ALPHA*B(L+ 3,J)
                U04=ALPHA*B(L+ 4,J)
                U05=ALPHA*B(L+ 5,J)
                U06=ALPHA*B(L+ 6,J)
                U07=ALPHA*B(L+ 7,J)
                U08=ALPHA*B(L+ 8,J)
                U09=ALPHA*B(L+ 9,J)
                U10=ALPHA*B(L+10,J)
                U11=ALPHA*B(L+11,J)
                U12=ALPHA*B(L+12,J)
                U13=ALPHA*B(L+13,J)
                U14=ALPHA*B(L+14,J)
                U15=ALPHA*B(L+15,J)
                DO I=1,M
                  C(I,J)=C(I,J)+U00*A(I,L   )+U01*A(I,L+ 1)
     &                         +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                         +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                         +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                         +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                         +U10*A(I,L+10)+U11*A(I,L+11)
     &                         +U12*A(I,L+12)+U13*A(I,L+13)
     &                         +U14*A(I,L+14)+U15*A(I,L+15)
                ENDDO
              ENDDO
            ENDDO  
!$OMP parallel do private(J,I,L,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15,
!$OMP+ V00,V01,V02,V03,V04,V05,V06,V07,
!$OMP+ V08,V09,V10,V11,V12,V13,V14,V15)
            DO J=MODJ+1,N,2
              DO I=1,M     
                C(I,J  )=BETA*C(I,J  )
                C(I,J+1)=BETA*C(I,J+1)
              ENDDO
              DO L=1,MODL
                U00=ALPHA*B(L,J  )
                V00=ALPHA*B(L,J+1)
                DO I=1,M
                  C(I,J  )=C(I,J  )+U00*A(I,L)
                  C(I,J+1)=C(I,J+1)+V00*A(I,L)
                ENDDO
              ENDDO  
              DO L=MODL+1,K,16
                U00=ALPHA*B(L   ,J  )
                U01=ALPHA*B(L+ 1,J  )
                U02=ALPHA*B(L+ 2,J  )
                U03=ALPHA*B(L+ 3,J  )
                U04=ALPHA*B(L+ 4,J  )
                U05=ALPHA*B(L+ 5,J  )
                U06=ALPHA*B(L+ 6,J  )
                U07=ALPHA*B(L+ 7,J  )
                U08=ALPHA*B(L+ 8,J  )
                U09=ALPHA*B(L+ 9,J  )
                U10=ALPHA*B(L+10,J  )
                U11=ALPHA*B(L+11,J  )
                U12=ALPHA*B(L+12,J  )
                U13=ALPHA*B(L+13,J  )
                U14=ALPHA*B(L+14,J  )
                U15=ALPHA*B(L+15,J  )
                V00=ALPHA*B(L   ,J+1)
                V01=ALPHA*B(L+ 1,J+1)
                V02=ALPHA*B(L+ 2,J+1)
                V03=ALPHA*B(L+ 3,J+1)
                V04=ALPHA*B(L+ 4,J+1)
                V05=ALPHA*B(L+ 5,J+1)
                V06=ALPHA*B(L+ 6,J+1)
                V07=ALPHA*B(L+ 7,J+1)
                V08=ALPHA*B(L+ 8,J+1)
                V09=ALPHA*B(L+ 9,J+1)
                V10=ALPHA*B(L+10,J+1)
                V11=ALPHA*B(L+11,J+1)
                V12=ALPHA*B(L+12,J+1)
                V13=ALPHA*B(L+13,J+1)
                V14=ALPHA*B(L+14,J+1)
                V15=ALPHA*B(L+15,J+1)
                DO I=1,M
                  C(I,J  )=C(I,J  )+U00*A(I,L   )+U01*A(I,L+ 1)
     &                             +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                             +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                             +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                             +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                             +U10*A(I,L+10)+U11*A(I,L+11)
     &                             +U12*A(I,L+12)+U13*A(I,L+13)
     &                             +U14*A(I,L+14)+U15*A(I,L+15)
                  C(I,J+1)=C(I,J+1)+V00*A(I,L   )+V01*A(I,L+ 1)
     &                             +V02*A(I,L+ 2)+V03*A(I,L+ 3)
     &                             +V04*A(I,L+ 4)+V05*A(I,L+ 5)
     &                             +V06*A(I,L+ 6)+V07*A(I,L+ 7)
     &                             +V08*A(I,L+ 8)+V09*A(I,L+ 9)
     &                             +V10*A(I,L+10)+V11*A(I,L+11)
     &                             +V12*A(I,L+12)+V13*A(I,L+13)
     &                             +V14*A(I,L+14)+V15*A(I,L+15)
                ENDDO
              ENDDO  
            ENDDO
          ENDIF 
        ELSE
          IF(BETA.EQ.ZERO) THEN 
C     ==--------------------------------------------------------------==
C     == Form C := alpha*A'*B                                         ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,8)
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,MODJ
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(L,J)
                ENDDO 
                C(I,J)=ALPHA*TEMP
              ENDDO 
            ENDDO 
!$OMP parallel do private(J,I,L,T00,T01,T02,T03,T04,T05,T06,T07)
            DO J=MODJ+1,N,8
              DO I=1,M
                T00=ZERO
                T01=ZERO
                T02=ZERO
                T03=ZERO
                T04=ZERO
                T05=ZERO
                T06=ZERO
                T07=ZERO
                DO L=1,K
                  T00=T00+A(L,I)*B(L,J  )
                  T01=T01+A(L,I)*B(L,J+1)
                  T02=T02+A(L,I)*B(L,J+2)
                  T03=T03+A(L,I)*B(L,J+3)
                  T04=T04+A(L,I)*B(L,J+4)
                  T05=T05+A(L,I)*B(L,J+5)
                  T06=T06+A(L,I)*B(L,J+6)
                  T07=T07+A(L,I)*B(L,J+7)
                ENDDO 
                C(I,J  )=ALPHA*T00
                C(I,J+1)=ALPHA*T01
                C(I,J+2)=ALPHA*T02
                C(I,J+3)=ALPHA*T03
                C(I,J+4)=ALPHA*T04
                C(I,J+5)=ALPHA*T05
                C(I,J+6)=ALPHA*T06
                C(I,J+7)=ALPHA*T07
              ENDDO 
            ENDDO 
          ELSE IF(BETA.EQ.ONE) THEN 
C     ==--------------------------------------------------------------==
C     == Form C := alpha*A'*B+C                                       ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,2)
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,MODJ
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(L,J)
                ENDDO 
                C(I,J)=ALPHA*TEMP+C(I,J)
              ENDDO 
            ENDDO 
!$OMP parallel do private(J,I,L,T00,T01)
            DO J=MODJ+1,N,2
              DO I=1,M
                T00=ZERO
                T01=ZERO
                DO L=1,K
                  T00=T00+A(L,I)*B(L,J  )
                  T01=T01+A(L,I)*B(L,J+1)
                ENDDO
                C(I,J  )=ALPHA*T00+C(I,J  )
                C(I,J+1)=ALPHA*T01+C(I,J+1)
              ENDDO 
            ENDDO 
          ELSE 
C     ==--------------------------------------------------------------==
C     == Form C := alpha*A'*B+beta*C                                  ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,2)
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,MODJ
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(L,J)
                ENDDO 
                C(I,J)=ALPHA*TEMP+BETA*C(I,J)
              ENDDO 
            ENDDO 
!$OMP parallel do private(J,I,L,T00,T01)
            DO J=MODJ+1,N,2
              DO I=1,M
                T00=ZERO
                T01=ZERO
                DO L=1,K
                  T00=T00+A(L,I)*B(L,J  )
                  T01=T01+A(L,I)*B(L,J+1)
                ENDDO
                C(I,J  )=ALPHA*T00+BETA*C(I,J  )
                C(I,J+1)=ALPHA*T01+BETA*C(I,J+1)
              ENDDO 
            ENDDO 
          ENDIF 
        ENDIF
      ELSE
        IF(NOTA)THEN
          IF(BETA.EQ.ZERO) THEN 
C     ==--------------------------------------------------------------==
C     == Form C := alpha*A*B'                                         ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,2)
            MODL=MOD(K,16)
!$OMP parallel do private(J,I,L,TEMP,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15)
            DO J=1,MODJ
              DO I=1,M      
                C(I,J)=ZERO
              ENDDO        
              DO L=1,MODL
                TEMP=ALPHA*B(J,L)
                DO I=1,M         
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
              DO L=MODL+1,K,16
                U00=ALPHA*B(J,L   )
                U01=ALPHA*B(J,L+ 1)
                U02=ALPHA*B(J,L+ 2)
                U03=ALPHA*B(J,L+ 3)
                U04=ALPHA*B(J,L+ 4)
                U05=ALPHA*B(J,L+ 5)
                U06=ALPHA*B(J,L+ 6)
                U07=ALPHA*B(J,L+ 7)
                U08=ALPHA*B(J,L+ 8)
                U09=ALPHA*B(J,L+ 9)
                U10=ALPHA*B(J,L+10)
                U11=ALPHA*B(J,L+11)
                U12=ALPHA*B(J,L+12)
                U13=ALPHA*B(J,L+13)
                U14=ALPHA*B(J,L+14)
                U15=ALPHA*B(J,L+15)
                DO I=1,M
                  C(I,J)=C(I,J)+U00*A(I,L   )+U01*A(I,L+ 1)
     &                         +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                         +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                         +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                         +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                         +U10*A(I,L+10)+U11*A(I,L+11)
     &                         +U12*A(I,L+12)+U13*A(I,L+13)
     &                         +U14*A(I,L+14)+U15*A(I,L+15)
                ENDDO
              ENDDO
            ENDDO  
!$OMP parallel do private(J,I,L,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15,
!$OMP+ V00,V01,V02,V03,V04,V05,V06,V07,
!$OMP+ V08,V09,V10,V11,V12,V13,V14,V15)
            DO J=MODJ+1,N,2
              DO I=1,M     
                C(I,J  )=ZERO
                C(I,J+1)=ZERO
              ENDDO          
              DO L=1,MODL
                U00=ALPHA*B(J  ,L)
                V00=ALPHA*B(J+1,L)
                DO I=1,M
                  C(I,J  )=C(I,J  )+U00*A(I,L)
                  C(I,J+1)=C(I,J+1)+V00*A(I,L)
                ENDDO
              ENDDO  
              DO L=MODL+1,K,16
                U00=ALPHA*B(J  ,L   )
                U01=ALPHA*B(J  ,L+ 1)
                U02=ALPHA*B(J  ,L+ 2)
                U03=ALPHA*B(J  ,L+ 3)
                U04=ALPHA*B(J  ,L+ 4)
                U05=ALPHA*B(J  ,L+ 5)
                U06=ALPHA*B(J  ,L+ 6)
                U07=ALPHA*B(J  ,L+ 7)
                U08=ALPHA*B(J  ,L+ 8)
                U09=ALPHA*B(J  ,L+ 9)
                U10=ALPHA*B(J  ,L+10)
                U11=ALPHA*B(J  ,L+11)
                U12=ALPHA*B(J  ,L+12)
                U13=ALPHA*B(J  ,L+13)
                U14=ALPHA*B(J  ,L+14)
                U15=ALPHA*B(J  ,L+15)
                V00=ALPHA*B(J+1,L   )
                V01=ALPHA*B(J+1,L+ 1)
                V02=ALPHA*B(J+1,L+ 2)
                V03=ALPHA*B(J+1,L+ 3)
                V04=ALPHA*B(J+1,L+ 4)
                V05=ALPHA*B(J+1,L+ 5)
                V06=ALPHA*B(J+1,L+ 6)
                V07=ALPHA*B(J+1,L+ 7)
                V08=ALPHA*B(J+1,L+ 8)
                V09=ALPHA*B(J+1,L+ 9)
                V10=ALPHA*B(J+1,L+10)
                V11=ALPHA*B(J+1,L+11)
                V12=ALPHA*B(J+1,L+12)
                V13=ALPHA*B(J+1,L+13)
                V14=ALPHA*B(J+1,L+14)
                V15=ALPHA*B(J+1,L+15)
                DO I=1,M
                  C(I,J  )=C(I,J  )+U00*A(I,L   )+U01*A(I,L+ 1)
     &                             +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                             +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                             +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                             +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                             +U10*A(I,L+10)+U11*A(I,L+11)
     &                             +U12*A(I,L+12)+U13*A(I,L+13)
     &                             +U14*A(I,L+14)+U15*A(I,L+15)
                  C(I,J+1)=C(I,J+1)+V00*A(I,L   )+V01*A(I,L+ 1)
     &                             +V02*A(I,L+ 2)+V03*A(I,L+ 3)
     &                             +V04*A(I,L+ 4)+V05*A(I,L+ 5)
     &                             +V06*A(I,L+ 6)+V07*A(I,L+ 7)
     &                             +V08*A(I,L+ 8)+V09*A(I,L+ 9)
     &                             +V10*A(I,L+10)+V11*A(I,L+11)
     &                             +V12*A(I,L+12)+V13*A(I,L+13)
     &                             +V14*A(I,L+14)+V15*A(I,L+15)
                ENDDO
              ENDDO  
            ENDDO  
          ELSE IF(BETA.EQ.ONE) THEN 
C     ==--------------------------------------------------------------==
C     == Form C := alpha*A*B'+C                                       ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,2)
            MODL=MOD(K,16)
!$OMP parallel do private(J,I,L,TEMP,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15)
            DO J=1,MODJ
              DO L=1,MODL
                TEMP=ALPHA*B(J,L)
                DO I=1,M         
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO  
              DO L=MODL+1,K,16
                U00=ALPHA*B(J,L   )
                U01=ALPHA*B(J,L+ 1)
                U02=ALPHA*B(J,L+ 2)
                U03=ALPHA*B(J,L+ 3)
                U04=ALPHA*B(J,L+ 4)
                U05=ALPHA*B(J,L+ 5)
                U06=ALPHA*B(J,L+ 6)
                U07=ALPHA*B(J,L+ 7)
                U08=ALPHA*B(J,L+ 8)
                U09=ALPHA*B(J,L+ 9)
                U10=ALPHA*B(J,L+10)
                U11=ALPHA*B(J,L+11)
                U12=ALPHA*B(J,L+12)
                U13=ALPHA*B(J,L+13)
                U14=ALPHA*B(J,L+14)
                U15=ALPHA*B(J,L+15)
                DO I=1,M
                  C(I,J)=C(I,J)+U00*A(I,L   )+U01*A(I,L+ 1)
     &                         +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                         +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                         +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                         +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                         +U10*A(I,L+10)+U11*A(I,L+11)
     &                         +U12*A(I,L+12)+U13*A(I,L+13)
     &                         +U14*A(I,L+14)+U15*A(I,L+15)
                ENDDO
              ENDDO  
            ENDDO  
!$OMP parallel do private(J,I,L,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15,
!$OMP+ V00,V01,V02,V03,V04,V05,V06,V07,
!$OMP+ V08,V09,V10,V11,V12,V13,V14,V15)
            DO J=MODJ+1,N,2
              DO L=1,MODL  
                U00=ALPHA*B(J  ,L)
                V00=ALPHA*B(J+1,L)
                DO I=1,M
                  C(I,J  )=C(I,J  )+U00*A(I,L)
                  C(I,J+1)=C(I,J+1)+V00*A(I,L)
                ENDDO
              ENDDO  
              DO L=MODL+1,K,16
                U00=ALPHA*B(J  ,L   )
                U01=ALPHA*B(J  ,L+ 1)
                U02=ALPHA*B(J  ,L+ 2)
                U03=ALPHA*B(J  ,L+ 3)
                U04=ALPHA*B(J  ,L+ 4)
                U05=ALPHA*B(J  ,L+ 5)
                U06=ALPHA*B(J  ,L+ 6)
                U07=ALPHA*B(J  ,L+ 7)
                U08=ALPHA*B(J  ,L+ 8)
                U09=ALPHA*B(J  ,L+ 9)
                U10=ALPHA*B(J  ,L+10)
                U11=ALPHA*B(J  ,L+11)
                U12=ALPHA*B(J  ,L+12)
                U13=ALPHA*B(J  ,L+13)
                U14=ALPHA*B(J  ,L+14)
                U15=ALPHA*B(J  ,L+15)
                V00=ALPHA*B(J+1,L   )
                V01=ALPHA*B(J+1,L+ 1)
                V02=ALPHA*B(J+1,L+ 2)
                V03=ALPHA*B(J+1,L+ 3)
                V04=ALPHA*B(J+1,L+ 4)
                V05=ALPHA*B(J+1,L+ 5)
                V06=ALPHA*B(J+1,L+ 6)
                V07=ALPHA*B(J+1,L+ 7)
                V08=ALPHA*B(J+1,L+ 8)
                V09=ALPHA*B(J+1,L+ 9)
                V10=ALPHA*B(J+1,L+10)
                V11=ALPHA*B(J+1,L+11)
                V12=ALPHA*B(J+1,L+12)
                V13=ALPHA*B(J+1,L+13)
                V14=ALPHA*B(J+1,L+14)
                V15=ALPHA*B(J+1,L+15)
                DO I=1,M
                  C(I,J  )=C(I,J  )+U00*A(I,L   )+U01*A(I,L+ 1)
     &                             +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                             +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                             +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                             +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                             +U10*A(I,L+10)+U11*A(I,L+11)
     &                             +U12*A(I,L+12)+U13*A(I,L+13)
     &                             +U14*A(I,L+14)+U15*A(I,L+15)
                  C(I,J+1)=C(I,J+1)+V00*A(I,L   )+V01*A(I,L+ 1)
     &                             +V02*A(I,L+ 2)+V03*A(I,L+ 3)
     &                             +V04*A(I,L+ 4)+V05*A(I,L+ 5)
     &                             +V06*A(I,L+ 6)+V07*A(I,L+ 7)
     &                             +V08*A(I,L+ 8)+V09*A(I,L+ 9)
     &                             +V10*A(I,L+10)+V11*A(I,L+11)
     &                             +V12*A(I,L+12)+V13*A(I,L+13)
     &                             +V14*A(I,L+14)+V15*A(I,L+15)
                ENDDO
              ENDDO  
            ENDDO  
          ELSE 
C     ==--------------------------------------------------------------==
C     == Form C := alpha*A*B'+beta*C                                  ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,2)
            MODL=MOD(K,16)
!$OMP parallel do private(J,I,L,TEMP,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15)
            DO J=1,MODJ
              DO I=1,M      
                C(I,J)=BETA*C(I,J)
              ENDDO
              DO L=1,MODL
                TEMP=ALPHA*B(J,L)
                DO I=1,M         
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO  
              DO L=MODL+1,K,16
                U00=ALPHA*B(J,L   )
                U01=ALPHA*B(J,L+ 1)
                U02=ALPHA*B(J,L+ 2)
                U03=ALPHA*B(J,L+ 3)
                U04=ALPHA*B(J,L+ 4)
                U05=ALPHA*B(J,L+ 5)
                U06=ALPHA*B(J,L+ 6)
                U07=ALPHA*B(J,L+ 7)
                U08=ALPHA*B(J,L+ 8)
                U09=ALPHA*B(J,L+ 9)
                U10=ALPHA*B(J,L+10)
                U11=ALPHA*B(J,L+11)
                U12=ALPHA*B(J,L+12)
                U13=ALPHA*B(J,L+13)
                U14=ALPHA*B(J,L+14)
                U15=ALPHA*B(J,L+15)
                DO I=1,M
                  C(I,J)=C(I,J)+U00*A(I,L   )+U01*A(I,L+ 1)
     &                         +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                         +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                         +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                         +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                         +U10*A(I,L+10)+U11*A(I,L+11)
     &                         +U12*A(I,L+12)+U13*A(I,L+13)
     &                         +U14*A(I,L+14)+U15*A(I,L+15)
                ENDDO
              ENDDO
            ENDDO  
!$OMP parallel do private(J,I,L,
!$OMP+ U00,U01,U02,U03,U04,U05,U06,U07,
!$OMP+ U08,U09,U10,U11,U12,U13,U14,U15,
!$OMP+ V00,V01,V02,V03,V04,V05,V06,V07,
!$OMP+ V08,V09,V10,V11,V12,V13,V14,V15)
            DO J=MODJ+1,N,2
              DO I=1,M     
                C(I,J  )=BETA*C(I,J  )
                C(I,J+1)=BETA*C(I,J+1)
              ENDDO
              DO L=1,MODL
                T00=ALPHA*B(J  ,L)
                T01=ALPHA*B(J+1,L)
                DO I=1,M
                  C(I,J  )=C(I,J  )+T00*A(I,L)
                  C(I,J+1)=C(I,J+1)+T01*A(I,L)
                ENDDO
              ENDDO  
              DO L=MODL+1,K,16
                U00=ALPHA*B(J  ,L   )
                U01=ALPHA*B(J  ,L+ 1)
                U02=ALPHA*B(J  ,L+ 2)
                U03=ALPHA*B(J  ,L+ 3)
                U04=ALPHA*B(J  ,L+ 4)
                U05=ALPHA*B(J  ,L+ 5)
                U06=ALPHA*B(J  ,L+ 6)
                U07=ALPHA*B(J  ,L+ 7)
                U08=ALPHA*B(J  ,L+ 8)
                U09=ALPHA*B(J  ,L+ 9)
                U10=ALPHA*B(J  ,L+10)
                U11=ALPHA*B(J  ,L+11)
                U12=ALPHA*B(J  ,L+12)
                U13=ALPHA*B(J  ,L+13)
                U14=ALPHA*B(J  ,L+14)
                U15=ALPHA*B(J  ,L+15)
                V00=ALPHA*B(J+1,L   )
                V01=ALPHA*B(J+1,L+ 1)
                V02=ALPHA*B(J+1,L+ 2)
                V03=ALPHA*B(J+1,L+ 3)
                V04=ALPHA*B(J+1,L+ 4)
                V05=ALPHA*B(J+1,L+ 5)
                V06=ALPHA*B(J+1,L+ 6)
                V07=ALPHA*B(J+1,L+ 7)
                V08=ALPHA*B(J+1,L+ 8)
                V09=ALPHA*B(J+1,L+ 9)
                V10=ALPHA*B(J+1,L+10)
                V11=ALPHA*B(J+1,L+11)
                V12=ALPHA*B(J+1,L+12)
                V13=ALPHA*B(J+1,L+13)
                V14=ALPHA*B(J+1,L+14)
                V15=ALPHA*B(J+1,L+15)
                DO I=1,M
                  C(I,J  )=C(I,J  )+U00*A(I,L   )+U01*A(I,L+ 1)
     &                             +U02*A(I,L+ 2)+U03*A(I,L+ 3)
     &                             +U04*A(I,L+ 4)+U05*A(I,L+ 5)
     &                             +U06*A(I,L+ 6)+U07*A(I,L+ 7)
     &                             +U08*A(I,L+ 8)+U09*A(I,L+ 9)
     &                             +U10*A(I,L+10)+U11*A(I,L+11)
     &                             +U12*A(I,L+12)+U13*A(I,L+13)
     &                             +U14*A(I,L+14)+U15*A(I,L+15)
                  C(I,J+1)=C(I,J+1)+V00*A(I,L   )+V01*A(I,L+ 1)
     &                             +V02*A(I,L+ 2)+V03*A(I,L+ 3)
     &                             +V04*A(I,L+ 4)+V05*A(I,L+ 5)
     &                             +V06*A(I,L+ 6)+V07*A(I,L+ 7)
     &                             +V08*A(I,L+ 8)+V09*A(I,L+ 9)
     &                             +V10*A(I,L+10)+V11*A(I,L+11)
     &                             +V12*A(I,L+12)+V13*A(I,L+13)
     &                             +V14*A(I,L+14)+V15*A(I,L+15)
                ENDDO
              ENDDO  
            ENDDO  
          ENDIF 
        ELSE
          IF(BETA.EQ.ZERO) THEN 
C     ==--------------------------------------------------------------==
C     == Form  C := alpha*A'*B'                                       ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,2)
!$OMP parallel do private(J,I,L,TEMP) 
            DO J=1,MODJ
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(J,L)
                ENDDO
                C(I,J)=ALPHA*TEMP
              ENDDO
            ENDDO
!$OMP parallel do private(J,I,L,T00,T01)
            DO J=MODJ+1,N,2
              DO I=1,M
                T00=ZERO
                T01=ZERO
                DO L=1,K
                  T00=T00+A(L,I)*B(J  ,L)
                  T01=T01+A(L,I)*B(J+1,L)
                ENDDO
                C(I,J  )=ALPHA*T00
                C(I,J+1)=ALPHA*T01
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN 
C     ==--------------------------------------------------------------==
C     == Form C := alpha*A'*B'+C                                      ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,2)
!$OMP parallel do private(J,I,L,TEMP) 
            DO J=1,MODJ
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(J,L)
                ENDDO
                C(I,J)=ALPHA*TEMP+C(I,J)
              ENDDO
            ENDDO
!$OMP parallel do private(J,I,L,T00,T01)
            DO J=MODJ+1,N,2
              DO I=1,M
                T00=ZERO
                T01=ZERO
                DO L=1,K
                  T00=T00+A(L,I)*B(J  ,L)
                  T01=T01+A(L,I)*B(J+1,L)
                ENDDO
                C(I,J  )=ALPHA*T00+C(I,J  )
                C(I,J+1)=ALPHA*T01+C(I,J+1)
              ENDDO
            ENDDO
          ELSE 
C     ==--------------------------------------------------------------==
C     == Form C := alpha*A'*B'+beta*C                                 ==
C     ==--------------------------------------------------------------==
            MODJ=MOD(N,2)
!$OMP parallel do private(J,I,L,TEMP) 
            DO J=1,MODJ
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(J,L)
                ENDDO
                C(I,J)=ALPHA*TEMP+BETA*C(I,J)
              ENDDO
            ENDDO
!$OMP parallel do private(J,I,L,T00,T01)
            DO J=MODJ+1,N,2
              DO I=1,M
                T00=ZERO
                T01=ZERO
                DO L=1,K
                  T00=T00+A(L,I)*B(J  ,L)
                  T01=T01+A(L,I)*B(J+1,L)
                ENDDO
                C(I,J  )=ALPHA*T00+BETA*C(I,J  )
                C(I,J+1)=ALPHA*T01+BETA*C(I,J+1)
              ENDDO
            ENDDO
          ENDIF 
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE DGEMV(TRANS,M,N,ALPHA,A,LDA,X,INCX,BETA,Y,INCY)
C     ==--------------------------------------------------------------==
C     == Purpose                                                      ==
C     == =======                                                      ==
C     ==                                                              ==
C     == DGEMV performs one of the matrix-vector operations           ==
C     ==                                                              ==
C     == y := alpha*A*x + beta*y, or y := alpha*A'*x + beta*y,        ==
C     ==                                                              ==
C     == where alpha and beta are scalars, x and y are vectors        ==
C     == and A is an m by n matrix.                                   ==
C     ==                                                              ==
C     == Parameters                                                   ==
C     == ==========                                                   ==
C     ==                                                              ==
C     == TRANS  - CHARACTER*1.                                        ==
C     ==    On entry, TRANS specifies the operation to be performed   ==
C     ==    as follows:                                               ==
C     ==                                                              ==
C     ==       TRANS = 'N' or 'n'   y := alpha*A*x + beta*y.          ==
C     ==                                                              ==
C     ==       TRANS = 'T' or 't'   y := alpha*A'*x + beta*y.         ==
C     ==                                                              ==
C     ==       TRANS = 'C' or 'c'   y := alpha*A'*x + beta*y.         ==
C     ==                                                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == M      - INTEGER.                                            ==
C     ==    On entry, M specifies the number of rows of the matrix A. ==
C     ==    M must be at least zero.                                  ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == N      - INTEGER.                                            ==
C     ==    On entry, N specifies the number of columns of            ==
C     ==    the matrix A. N must be at least zero.                    ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == ALPHA  - DOUBLE PRECISION.                                   ==
C     ==    On entry, ALPHA specifies the scalar alpha.               ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == A      - DOUBLE PRECISION array of DIMENSION (LDA,n).        ==
C     ==    Before entry, the leading m by n part of the array A      ==
C     ==    must contain the matrix of coefficients.                  ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == LDA    - INTEGER.                                            ==
C     ==    On entry, LDA specifies the first dimension of A          ==
C     ==    as declared in the calling (sub) program.                 ==
C     ==    LDA must be at least max(1,m).                            ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == X      - DOUBLE PRECISION array of DIMENSION at least        ==
C     ==    (1+(n-1)*abs(INCX)) when TRANS = 'N' or 'n' and           ==
C     ==    at least (1+(m-1)*abs(INCX)) otherwise. Before entry,     ==
C     ==    the incremented array X must contain the vector x.        ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == INCX   - INTEGER.                                            ==
C     ==    On entry, INCX specifies the increment for the elements   ==
C     ==    of X. INCX must not be zero.                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == BETA   - DOUBLE PRECISION.                                   ==
C     ==    On entry, BETA specifies the scalar beta. When BETA is    ==
C     ==    supplied as zero then Y need not be set on input.         ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == Y      - DOUBLE PRECISION array of DIMENSION at least        ==
C     ==    (1+(m-1)*abs(INCY)) when TRANS = 'N' or 'n' and at least  ==
C     ==    (1+(n-1)*abs(INCY)) otherwise. Before entry with BETA     ==
C     ==    non-zero, the incremented array Y must contain the vector ==
C     ==    y. On exit, Y is overwritten by the updated vector y.     ==
C     ==                                                              ==
C     == INCY   - INTEGER.                                            ==
C     ==    On entry, INCY specifies the increment for the elements   ==
C     ==    of Y. INCY must not be zero.                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     ==                                                              ==
C     == Level 2 Blas routine.                                        ==
C     ==                                                              ==
C     == -- Written on 22-October-1986.                               ==
C     ==    Jack Dongarra, Argonne National Lab.                      ==
C     ==    Jeremy Du Croz, Nag Central Office.                       ==
C     ==    Sven Hammarling, Nag Central Office.                      ==
C     ==    Richard Hanson, Sandia National Labs                      ==
C     ==--------------------------------------------------------------==
C     Scalar Arguments
      DOUBLE PRECISION ALPHA,BETA
      INTEGER          INCX,INCY,LDA,M,N
      CHARACTER*1      TRANS 
C     Array Arguments
      DOUBLE PRECISION A(LDA,*),X(*),Y(*)
C     Parameters
      DOUBLE PRECISION ONE,ZERO
      PARAMETER       (ONE=1.0D+0,ZERO=0.0D+0)
C     Local Scalars
      DOUBLE PRECISION TEMP
      INTEGER          I,INFO,IX,IY,J,JX,JY,KX,KY,LENX,LENY
C     External Functions
      LOGICAL          LSAME
      EXTERNAL         LSAME
C     External Subroutines
      EXTERNAL         XERBLA
C     Intrinsic Functions
      INTRINSIC        MAX
C     Auxiliary variables
      DOUBLE PRECISION T0,T1,T2,T3,T4,T5,T6,T7
      INTEGER          MODJ,JX0,JX1,JX2,JX3,JX4,JX5,JX6,JX7
      INTEGER          IJY(N),IIX(M),IJX(N),IIY(M)
C     ==---------------------------------------------------------------=
C
C     Test the input parameters
C
      INFO=0
      IF(.NOT.LSAME(TRANS,'N').AND.
     $   .NOT.LSAME(TRANS,'T').AND.
     $   .NOT.LSAME(TRANS,'C')) THEN
        INFO=1
      ELSE IF(M.LT.0) THEN
        INFO=2         
      ELSE IF(N.LT.0) THEN
        INFO=3         
      ELSE IF(LDA.LT.MAX(1,M)) THEN
        INFO=6
      ELSE IF(INCX.EQ.0) THEN
        INFO=8
      ELSE IF(INCY.EQ.0) THEN
        INFO=11
      ENDIF
      IF(INFO.NE.0) THEN
        CALL XERBLA('DGEMV ',INFO)
        RETURN
      ENDIF
C
C     Quick return if possible 
C     
      IF((M.EQ.0).OR.(N.EQ.0).OR.
     $   ((ALPHA.EQ.ZERO).AND.(BETA.EQ.ONE)))  RETURN
C
C     Set LENX and LENY, the lengths of the vectors x and y, 
C     and set up the start points in X and Y.
C
      IF(LSAME(TRANS,'N')) THEN
        LENX=N
        LENY=M
      ELSE
        LENX=M
        LENY=N
      ENDIF
      IF(INCX.GT.0) THEN
        KX=1
      ELSE
        KX=1-(LENX-1)*INCX
      ENDIF
      IF(INCY.GT.0) THEN
        KY=1
      ELSE
        KY=1-(LENY-1)*INCY
      ENDIF
C    
C     Start the operations. In this version the elements of A 
C     are accessed sequentially with one pass through A.
C
C     First form y := beta*y.
C     
      IF(BETA.NE.ONE) THEN
        IF(INCY.EQ.1) THEN
          IF(BETA.EQ.ZERO) THEN
!$OMP parallel do private(I)
            DO I=1,LENY
              Y(I)=ZERO
            ENDDO
          ELSE
!$OMP parallel do private(I)
            DO I=1,LENY
              Y(I)=BETA*Y(I)
            ENDDO
          ENDIF
        ELSE
          IF(BETA.EQ.ZERO) THEN
!$OMP parallel do private(I,IY)
#ifdef __SR8000
*poption parallel, tlocal(I,IY)
#endif 
            DO I=1,LENY
              IY=KY+(I-1)*INCY
              Y(IY)=ZERO
            ENDDO
          ELSE
!$OMP parallel do private(I,IY)
!OCL NOVREC(Y)
!CDIR NODEP(Y)
#ifdef __SR8000
*poption parallel, tlocal(I,IY)
#endif 
            DO I=1,LENY
              IY=KY+(I-1)*INCY
              Y(IY)=BETA*Y(IY)
            ENDDO
          ENDIF
        ENDIF
      ENDIF
      IF(ALPHA.EQ.ZERO) RETURN
      IF(LSAME(TRANS,'N')) THEN
C     ==--------------------------------------------------------------==
C     == Form y := alpha*A*x + y                                      ==
C     ==--------------------------------------------------------------==
        IF(INCY.EQ.1) THEN
          IF(INCX.EQ.1) THEN 
            MODJ=MOD(N,8) 
            DO J=1,MODJ 
              TEMP=ALPHA*X(J)
!$OMP parallel do private(I)
              DO I=1,M
                Y(I)=Y(I)+TEMP*A(I,J)
              ENDDO
            ENDDO
            DO J=MODJ+1,N,8
              T0=ALPHA*X(J  )
              T1=ALPHA*X(J+1)
              T2=ALPHA*X(J+2)
              T3=ALPHA*X(J+3)
              T4=ALPHA*X(J+4)
              T5=ALPHA*X(J+5)
              T6=ALPHA*X(J+6)
              T7=ALPHA*X(J+7)
!$OMP parallel do private(I)
              DO I=1,M
                Y(I)=Y(I)
     &          +T0*A(I,J  )+T1*A(I,J+1)+T2*A(I,J+2)+T3*A(I,J+3)
     &          +T4*A(I,J+4)+T5*A(I,J+5)+T6*A(I,J+6)+T7*A(I,J+7)
              ENDDO
            ENDDO
          ELSE 
C..Fill list vector IJX
C..IJX(1:N) -> {KX,KX+INCX,KX+2*INCX,...,KX+(N-1)*INCX}
            DO J=1,N 
              IJX(J)=KX+(J-1)*INCX
            ENDDO 
            MODJ=MOD(N,8) 
            DO J=1,MODJ 
              JX=IJX(J)
              TEMP=ALPHA*X(JX)
!$OMP parallel do private(I)
              DO I=1,M
                Y(I)=Y(I)+TEMP*A(I,J)
              ENDDO
            ENDDO
            DO J=MODJ+1,N,8
              JX0=IJX(J  )
              JX1=IJX(J+1)
              JX2=IJX(J+2)
              JX3=IJX(J+3)
              JX4=IJX(J+4)
              JX5=IJX(J+5)
              JX6=IJX(J+6)
              JX7=IJX(J+7)
              T0=ALPHA*X(JX0)
              T1=ALPHA*X(JX1)
              T2=ALPHA*X(JX2)
              T3=ALPHA*X(JX3)
              T4=ALPHA*X(JX4)
              T5=ALPHA*X(JX5)
              T6=ALPHA*X(JX6)
              T7=ALPHA*X(JX7)
!$OMP parallel do private(I)
              DO I=1,M
                Y(I)=Y(I)
     &          +T0*A(I,J  )+T1*A(I,J+1)+T2*A(I,J+2)+T3*A(I,J+3)
     &          +T4*A(I,J+4)+T5*A(I,J+5)+T6*A(I,J+6)+T7*A(I,J+7)
              ENDDO
            ENDDO
          ENDIF 
        ELSE
C..Fill list vector IIY
C..IIY(1:M) -> {KY,KY+INCY,KY+2*INCY,...,KY+(M-1)*INCY}
          DO J=1,M 
            IIY(J)=KY+(J-1)*INCY
          ENDDO 
          IF(INCX.EQ.1) THEN 
!OCL NOVREC(Y)
            DO J=1,N
              TEMP=ALPHA*X(J)
!$OMP parallel do private(I,IY)
!CDIR NODEP(Y)
#ifdef __SR8000
*poption parallel, tlocal(I,IY)
#endif 
              DO I=1,M
                IY=IIY(I)
                Y(IY)=Y(IY)+TEMP*A(I,J)
              ENDDO
            ENDDO
          ELSE 
C..Fill list vector IJX
C..IJX(1:N) -> {KX,KX+INCX,KX+2*INCX,...,KX+(N-1)*INCX}
            DO J=1,N 
              IJX(J)=KX+(J-1)*INCX
            ENDDO 
!OCL NOVREC(Y)
            DO J=1,N
              JX=IJX(J)
              TEMP=ALPHA*X(JX)
!$OMP parallel do private(I,IY)
!CDIR NODEP(Y)
#ifdef __SR8000
*poption parallel, tlocal(I,IY)
#endif 
              DO I=1,M
                IY=IIY(I)
                Y(IY)=Y(IY)+TEMP*A(I,J)
              ENDDO
            ENDDO
          ENDIF 
        ENDIF
      ELSE  
C     ==--------------------------------------------------------------==
C     == Form y := alpha*A'*x + y                                     ==
C     ==--------------------------------------------------------------==
C..Fill list vector IJY
C..IJY(1:N) -> {KY,KY+INCY,KY+2*INCY,...,KY+(N-1)*INCY}
        DO J=1,N 
          IJY(J)=KY+(J-1)*INCY
        ENDDO 
        IF(INCX.EQ.1) THEN
!$OMP parallel do private(J,I,JY,TEMP) 
!OCL NOVREC(Y)
!CDIR NODEP(Y)
#ifdef __SR8000
*poption parallel, tlocal(J,I,JY,TEMP)
#endif 
          DO J=1,N
            TEMP=ZERO
            DO I=1,M
              TEMP=TEMP+A(I,J)*X(I)
            ENDDO
            JY=IJY(J)
            Y(JY)=Y(JY)+ALPHA*TEMP
          ENDDO
        ELSE
C..Fill list vector IIX
C..IIX(1:M) -> {KX,KX+INCX,KX+2*INCX,...,KX+(M-1)*INCX}
          DO J=1,M
            IIX(J)=KX+(J-1)*INCX
          ENDDO
!$OMP parallel do private(J,I,IX,JY,TEMP) 
!OCL NOVREC(Y)
!CDIR NODEP(Y)
#ifdef __SR8000
*poption parallel, tlocal(J,I,IX,JY,TEMP)
#endif 
          DO J=1,N
            TEMP=ZERO
            DO I=1,M
              IX=IIX(I)
              TEMP=TEMP+A(I,J)*X(IX)
            ENDDO
            JY=IJY(J)
            Y(JY)=Y(JY)+ALPHA*TEMP
          ENDDO
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN   
      END      
C     ==================================================================

C     ==================================================================
      SUBROUTINE DGER(M,N,ALPHA,X,INCX,Y,INCY,A,LDA)
C     ==--------------------------------------------------------------==
C     == Purpose                                                      ==
C     == =======                                                      ==
C     ==                                                              ==
C     == DGER   performs the rank 1 operation                         ==
C     ==                                                              ==
C     == A := alpha*x*y' + A,                                         ==
C     ==                                                              ==
C     == where alpha is a scalar, x is an m element vector,           ==
C     == y is an n element vector and A is an m by n matrix.          ==
C     ==                                                              ==
C     == Parameters                                                   ==
C     == ==========                                                   ==
C     ==                                                              ==
C     == M      - INTEGER.                                            ==
C     ==    On entry, M specifies the number of rows of the matrix A. ==
C     ==    M must be at least zero.                                  ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == N      - INTEGER.                                            ==
C     ==    On entry, N specifies the number of columns               ==
C     ==    of the matrix A. N must be at least zero.                 ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == ALPHA  - DOUBLE PRECISION.                                   ==
C     ==    On entry, ALPHA specifies the scalar alpha.               ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == X      - DOUBLE PRECISION array of dimension at least        ==
C     ==    (1+(m-1)*abs(INCX)). Before entry, the incremented        ==
C     ==    array X must contain the m element vector x.              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == INCX   - INTEGER.                                            ==
C     ==    On entry, INCX specifies the increment for the elements   ==
C     ==    of X. INCX must not be zero.                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == Y      - DOUBLE PRECISION array of dimension at least        ==
C     ==    (1+(n-1)*abs(INCY)). Before entry, the incremented        ==
C     ==    array Y must contain the n element vector y.              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == INCY   - INTEGER.                                            ==
C     ==    On entry, INCY specifies the increment for the elements   ==
C     ==    of Y. INCY must not be zero.                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == A      - DOUBLE PRECISION array of DIMENSION (LDA,n).        ==
C     ==    Before entry, the leading m by n part of the array A      ==
C     ==    must contain the matrix of coefficients. On exit, A       ==
C     ==    is overwritten by the updated matrix.                     ==
C     ==                                                              ==
C     == LDA    - INTEGER.                                            ==
C     ==    On entry, LDA specifies the first dimension of A as       ==
C     ==    declared in the calling (sub) program. LDA must be        ==
C     ==    at least max(1,m).                                        ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     ==                                                              ==
C     == Level 2 Blas routine.                                        ==
C     ==                                                              ==
C     == -- Written on 22-October-1986.                               ==
C     ==    Jack Dongarra, Argonne National Lab.                      ==
C     ==    Jeremy Du Croz, Nag Central Office.                       ==
C     ==    Sven Hammarling, Nag Central Office.                      ==
C     ==    Richard Hanson, Sandia National Labs.                     ==
C     ==--------------------------------------------------------------==
C     Scalar Arguments
      DOUBLE PRECISION ALPHA
      INTEGER          INCX,INCY,LDA,M,N
C     Array Arguments
      DOUBLE PRECISION A(LDA,*),X(*),Y(*)
C     Parameters
      DOUBLE PRECISION ZERO
      PARAMETER       (ZERO=0.0D+0)
C     Local Scalars
      DOUBLE PRECISION TEMP
      INTEGER          I,INFO,IX,J,JY,KX
C     External Subroutines
      EXTERNAL         XERBLA
C     Intrinsic Functions
      INTRINSIC        MAX
C     Auxiliary variables
      INTEGER          IIX(M),IJY(N)
C     ==--------------------------------------------------------------==
C
C     Test the input parameters
C
      INFO=0
      IF(M.LT.0) THEN
        INFO=1
      ELSE IF(N.LT.0) THEN
        INFO=2         
      ELSE IF(INCX.EQ.0) THEN
        INFO=5
      ELSE IF(INCY.EQ.0) THEN
        INFO=7
      ELSE IF(LDA.LT.MAX(1,M)) THEN
        INFO=9
      ENDIF
      IF(INFO.NE.0) THEN
        CALL XERBLA('DGER  ',INFO)
        RETURN         
      ENDIF
C     
C     Quick return if possible
C
      IF((M.EQ.0).OR.(N.EQ.0).OR.(ALPHA.EQ.ZERO)) RETURN
C
C     Start the operations. In this version the elements of A 
C     are accessed sequentially with one pass through A.
C
      IF(INCY.GT.0) THEN
        JY=1
      ELSE
        JY=1-(N-1)*INCY
      ENDIF
C..Fill list vector IIX
C..IJY(1:N) -> {JY,JY+INCY,JY+2*INCY,...,JY+(N-1)*INCY}
      DO J=1,N 
        IJY(J)=JY+(J-1)*INCY
      ENDDO 
      IF(INCX.EQ.1) THEN
!$OMP parallel do private(J,I,JY,TEMP)
        DO J=1,N
          JY=IJY(J)
          TEMP=ALPHA*Y(JY)
          DO I=1,M
            A(I,J)=A(I,J)+X(I)*TEMP
          ENDDO
        ENDDO
      ELSE
        IF(INCX.GT.0) THEN
          KX=1
        ELSE
          KX=1-(M-1)*INCX
        ENDIF
C..Fill list vector IIX
C..IIX(1:M) -> {KX,KX+INCX,KX+2*INCX,...,KX+(M-1)*INCX}
        DO J=1,M 
          IIX(J)=KX+(J-1)*INCX
        ENDDO 
!$OMP parallel do private(J,I,JY,IX,TEMP)
        DO J=1,N
          JY=IJY(J)
          TEMP=ALPHA*Y(JY)
          DO I=1,M
            IX=IIX(I)
            A(I,J)=A(I,J)+X(IX)*TEMP
          ENDDO
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE DSYRK(UPLO,TRANS,N,K,ALPHA,A,LDA,BETA,C,LDC)
C     ==--------------------------------------------------------------==
C     == Purpose                                                      ==
C     == =======                                                      ==
C     ==                                                              ==
C     == DSYRK  performs one of the symmetric rank k operations       ==
C     ==                                                              ==
C     == C := alpha*A*A' + beta*C,                                    ==
C     ==                                                              ==
C     == or                                                           ==
C     ==                                                              ==
C     == C := alpha*A'*A + beta*C,                                    ==
C     ==                                                              ==
C     == where alpha and beta  are scalars, C is an n by n symmetric  ==
C     == matrix and A is an n by k matrix in the first case and       ==
C     == a k by n matrix in the second case.                          ==
C     ==                                                              ==
C     == Parameters                                                   ==
C     == ==========                                                   ==
C     ==                                                              ==
C     == UPLO   - CHARACTER*1.                                        ==
C     ==    On entry, UPLO specifies whether the upper or lower       ==
C     ==    triangular part of the array C is to be referenced        ==
C     ==    as follows:                                               ==
C     ==                                                              ==
C     ==       UPLO = 'U' or 'u' Only the upper triangular part of C  ==
C     ==                         is to be referenced.                 ==
C     ==                                                              ==
C     ==       UPLO = 'L' or 'l' Only the lower triangular part of C  ==
C     ==                         is to be referenced.                 ==
C     ==                                                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == TRANS  - CHARACTER*1.                                        ==
C     ==    On entry, TRANS specifies the operation to be performed   ==
C     ==    as follows:                                               ==
C     ==                                                              ==
C     ==       TRANS = 'N' or 'n'   C := alpha*A*A' + beta*C.         ==
C     ==                                                              ==
C     ==       TRANS = 'T' or 't'   C := alpha*A'*A + beta*C.         ==
C     ==                                                              ==
C     ==       TRANS = 'C' or 'c'   C := alpha*A'*A + beta*C.         ==
C     ==                                                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == N      - INTEGER.                                            ==
C     ==    On entry, N specifies the order of the matrix C.          ==
C     ==    N must be at least zero.                                  ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == K      - INTEGER.                                            ==
C     ==    On entry with TRANS = 'N' or 'n', K specifies the number  ==
C     ==    of columns of the matrix A, and on entry with             ==
C     ==    TRANS = 'T' or 't' or 'C' or 'c', K specifies the number  ==
C     ==    of rows of the matrix  A.  K must be at least zero.       ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == ALPHA  - DOUBLE PRECISION.                                   ==
C     ==    On entry, ALPHA specifies the scalar alpha.               ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == A      - DOUBLE PRECISION array of DIMENSION (LDA,ka),       ==
C     ==    where ka is k when TRANS = 'N' or 'n', and is n otherwise.==
C     ==    Before entry with TRANS = 'N' or 'n', the leading n by k  ==
C     ==    part of the array A must contain the matrix A, otherwise  ==
C     ==    the leading k by n part of the array A must contain the   ==
C     ==    matrix A.                                                 ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == LDA    - INTEGER.                                            ==
C     ==    On entry, LDA specifies the first dimension of A as       ==
C     ==    declared in the calling (sub) program. When TRANS = 'N'   ==
C     ==    or 'n' then LDA must be at least max(1,n), otherwise      ==
C     ==    LDA must be at least max(1 k).                            ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == BETA   - DOUBLE PRECISION.                                   ==
C     ==    On entry, BETA specifies the scalar beta.                 ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == C      - DOUBLE PRECISION array of DIMENSION (LDC,n).        ==
C     ==    Before entry with UPLO = 'U' or 'u', the leading n by n   ==
C     ==    upper triangular part of the array C must contain the     ==
C     ==    upper triangular part of the symmetric matrix and the     ==
C     ==    strictly lower triangular part of C is not referenced.    ==
C     ==    On exit, the upper triangular part of the array C is      ==
C     ==    overwritten by the upper triangular part of the updated   ==
C     ==    matrix. Before entry with UPLO = 'L' or 'l', the leading  ==
C     ==    n by n lower triangular part of the array C must contain  ==
C     ==    the lower triangular part of the symmetric matrix and the ==
C     ==    strictly upper triangular part of C is not referenced.    ==
C     ==    On exit, the lower triangular part of the array C is      ==
C     ==    overwritten by the lower triangular part of the updated   ==
C     ==    matrix.                                                   ==
C     ==                                                              ==
C     == LDC    - INTEGER.                                            ==
C     ==    On entry, LDC specifies the first dimension of C as       ==
C     ==    declared in the calling (sub) program. LDC must be at     ==
C     ==    least max(1,n).                                           ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     ==                                                              ==
C     == Level 3 Blas routine.                                        ==
C     ==                                                              ==
C     == -- Written on 8-February-1989.                               ==
C     ==    Jack Dongarra, Argonne National Laboratory.               ==
C     ==    Iain Duff, AERE Harwell.                                  ==
C     ==    Jeremy Du Croz, Numerical Algorithms Group Ltd.           ==
C     ==    Sven Hammarling, Numerical Algorithms Group Ltd.          ==
C     ==--------------------------------------------------------------==
C     Scalar Arguments
      CHARACTER*1      UPLO,TRANS
      INTEGER          N,K,LDA,LDC
      DOUBLE PRECISION ALPHA,BETA
C     Array Arguments
      DOUBLE PRECISION A(LDA,*),C(LDC,*)
C     External Functions
      LOGICAL          LSAME
      EXTERNAL         LSAME
C     External Subroutines
      EXTERNAL         XERBLA
C     Intrinsic Functions
      INTRINSIC        MAX
C     Local Scalars
      LOGICAL          UPPER
      INTEGER          I,INFO,J,L,NROWA
      DOUBLE PRECISION TEMP
C     Parameters
      DOUBLE PRECISION ONE,ZERO
      PARAMETER       (ONE=1.0D+0,ZERO=0.0D+0)
C     ==--------------------------------------------------------------==
C
C     Test the input parameters
C     
      IF(LSAME(TRANS,'N')) THEN
        NROWA=N
      ELSE
        NROWA=K
      ENDIF
      UPPER=LSAME(UPLO,'U')
C
      INFO=0
      IF((.NOT.UPPER).AND.
     $   (.NOT.LSAME(UPLO,'L'))) THEN
        INFO=1
      ELSE IF((.NOT.LSAME(TRANS,'N')).AND.
     $        (.NOT.LSAME(TRANS,'T')).AND.
     $        (.NOT.LSAME(TRANS,'C'))) THEN
        INFO=2
      ELSE IF(N.LT.0) THEN
        INFO=3
      ELSE IF(K.LT.0) THEN
        INFO=4
      ELSE IF(LDA.LT.MAX(1,NROWA)) THEN
        INFO=7
      ELSE IF(LDC.LT.MAX(1,N)) THEN
        INFO=10
      ENDIF
      IF(INFO.NE.0) THEN
        CALL XERBLA('DSYRK ',INFO)
        RETURN
      ENDIF
C
C     Quick return if possible
C
      IF((N.EQ.0).OR.
     $   (((ALPHA.EQ.ZERO).OR.(K.EQ.0)).AND.(BETA.EQ.ONE)))
     $  RETURN
C
C     And when alpha.eq.zero
C
      IF(ALPHA.EQ.ZERO) THEN
        IF(UPPER) THEN
          IF(BETA.EQ.ZERO) THEN
!$OMP parallel do private(J,I) schedule(static,1)
            DO J=1,N
              DO I=1,J
                C(I,J)=ZERO
              ENDDO
            ENDDO
          ELSE
!$OMP parallel do private(J,I) schedule(static,1)
            DO J=1,N
              DO I=1,J
                C(I,J)=BETA*C(I,J)
              ENDDO 
            ENDDO
          ENDIF
        ELSE
          IF(BETA.EQ.ZERO) THEN
!$OMP parallel do private(J,I) schedule(static,1)
            DO J=1,N
              DO I=J,N
                C(I,J)=ZERO
              ENDDO
            ENDDO
          ELSE
!$OMP parallel do private(J,I) schedule(static,1)
            DO J=1,N
              DO I=J,N
                C(I,J)=BETA*C(I,J)
              ENDDO
            ENDDO
          ENDIF
        ENDIF
        RETURN
      ENDIF
C
C     Start the operations
C
      IF(LSAME(TRANS,'N')) THEN
        IF(UPPER) THEN
          IF(BETA.EQ.ZERO) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A*A' (upper triangle referenced)            ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N
              DO I=1,J
                C(I,J)=ZERO
              ENDDO
              DO L=1,K
                TEMP=ALPHA*A(J,L)
                DO I=1,J
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A*A' + C (upper triangle referenced)        ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N
              DO L=1,K
                TEMP=ALPHA*A(J,L)
                DO I=1,J
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ELSE
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A*A' + beta*C (upper triangle referenced)   ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N
              DO I=1,J
                C(I,J)=BETA*C(I,J)
              ENDDO
              DO L=1,K
                TEMP=ALPHA*A(J,L)
                DO I=1,J
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ENDIF
        ELSE
          IF(BETA.EQ.ZERO) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A*A' (lower triangle referenced)            ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N
              DO I=J,N
                C(I,J)=ZERO
              ENDDO
              DO L=1,K
                TEMP=ALPHA*A(J,L)
                DO I=J,N
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN 
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A*A' + C (lower triangle referenced)        ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N
              DO L=1,K
                TEMP=ALPHA*A(J,L)
                DO I=J,N
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ELSE
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A*A' + beta*C (lower triangle referenced)   ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N
              DO I=J,N
                C(I,J)=BETA*C(I,J)
              ENDDO
              DO L=1,K
                TEMP=ALPHA*A(J,L)
                DO I=J,N
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ENDIF
        ENDIF
      ELSE      
        IF(UPPER) THEN
          IF(BETA.EQ.ZERO) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A'*A (upper triangle referenced)            ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N 
!OCL UNROLL(16)
              DO I=1,J 
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*A(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP
              ENDDO
            ENDDO 
          ELSE IF(BETA.EQ.ONE) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A'*A + C (upper triangle referenced)        ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N 
              DO I=1,J
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*A(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP+C(I,J)
              ENDDO
            ENDDO 
          ELSE
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A'*A + beta*C (upper triangle referenced)   ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N 
              DO I=1,J 
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*A(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP+BETA*C(I,J)
              ENDDO 
            ENDDO 
          ENDIF 
        ELSE  
          IF(BETA.EQ.ZERO) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A'*A (lower triangle referenced)            ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N 
              DO I=J,N
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*A(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP
              ENDDO
            ENDDO 
          ELSE IF(BETA.EQ.ONE) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A'*A + C (lower triangle referenced)        ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N 
              DO I=J,N
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*A(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP+C(I,J)
              ENDDO
            ENDDO 
          ELSE 
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A'*A + beta*C (lower triangle referenced)   ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(J,I,L,TEMP) schedule(static,1)
            DO J=1,N 
              DO I=J,N
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*A(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP+BETA*C(I,J)
              ENDDO 
            ENDDO 
          ENDIF 
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE DTRMM(SIDE,UPLO,TRANSA,DIAG,M,N,ALPHA,A,LDA,B,LDB)
C     ==--------------------------------------------------------------==
C     == Purpose                                                      ==
C     == =======                                                      ==
C     ==                                                              ==
C     == DTRMM performs one of the matrix-matrix operations           ==
C     ==                                                              ==
C     == B := alpha*op(A)*B,   or   B := alpha*B*op(A),               ==
C     ==                                                              ==
C     == where alpha is a scalar, B is an m by n matrix, A is a unit, ==
C     == or non-unit, upper or lower triangular matrix and op(A) is   ==
C     == one of                                                       ==
C     ==                                                              ==
C     == op(A) = A   or   op(A) = A'.                                 ==
C     ==                                                              ==
C     == Parameters                                                   ==
C     == ==========                                                   ==
C     ==                                                              ==
C     == SIDE   - CHARACTER*1.                                        ==
C     ==    On entry, SIDE specifies whether op(A) multiplies B       ==
C     ==    form the left or right as follows:                        ==
C     ==                                                              ==
C     ==       SIDE = 'L' or 'l'   B := alpha*op(A)*B.                ==
C     ==                                                              ==
C     ==       SIDE = 'R' or 'r'   B := alpha*B*op(A).                ==
C     ==                                                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == UPLO   - CHARACTER*1.                                        ==
C     ==    On entry, UPLO specifies whether the matrix A is an upper ==
C     ==    or lower triangular matrix as follows:                    ==
C     ==                                                              ==
C     ==       UPLO = 'U' or 'u'  A is an upper triangular matrix.    ==
C     ==                                                              ==
C     ==       UPLO = 'L' or 'l'  A is a lower triangular matrix.     ==
C     ==                                                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == TRANSA - CHARACTER*1.                                        ==
C     ==    On entry, TRANSA specifies the form of op(A) to be        ==
C     ==    used in the matrix multiplication as follows:             ==
C     ==                                                              ==
C     ==       TRANSA = 'N' or 'n'   op(A) = A.                       ==
C     ==                                                              ==
C     ==       TRANSA = 'T' or 't'   op(A) = A'.                      ==
C     ==                                                              ==
C     ==       TRANSA = 'C' or 'c'   op(A) = A'.                      ==
C     ==                                                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == DIAG   - CHARACTER*1.                                        ==
C     ==    On entry, DIAG specifies whether or not A is unit         ==
C     ==    triangular as follows:                                    ==
C     ==                                                              ==
C     ==       DIAG = 'U' or 'u' A is assumed to be unit triangular.  ==
C     ==                                                              ==
C     ==       DIAG = 'N' or 'n' A is not assumed to be unit          ==
C     ==                         triangular.                          ==
C     ==                                                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == M      - INTEGER.                                            ==
C     ==    On entry, M specifies the number of rows of B. M must     ==
C     ==    be at least zero.                                         ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == N      - INTEGER.                                            ==
C     ==    On entry, N specifies the number of columns of B. N must  ==
C     ==    be at least zero.                                         ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == ALPHA  - DOUBLE PRECISION.                                   ==
C     ==    On entry, ALPHA specifies the scalar alpha. When alpha    ==
C     ==    is zero then A is not referenced and B need not be set    ==
C     ==    before entry.                                             ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == A      - DOUBLE PRECISION array of DIMENSION (LDA,k),        ==
C     ==    where k is m when SIDE = 'L' or 'l' and is n when         ==
C     ==    SIDE = 'R' or 'r'. Before entry with UPLO = 'U' or 'u',   ==
C     ==    the leading k by k upper triangular part of the array A   ==
C     ==    must contain the upper triangular matrix and the strictly ==
C     ==    lower triangular part of A is not referenced.             ==
C     ==    Before entry with UPLO = 'L' or 'l', the leading k by k   ==
C     ==    lower triangular part of the array A must contain the     ==
C     ==    lower triangular matrix and the strictly upper triangular ==
C     ==    part of A is not referenced.                              ==
C     ==    Note that when DIAG = 'U' or 'u', the diagonal elements   ==
C     ==    of A are not referenced either, but are assumed to be     ==
C     ==    unity.                                                    ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == LDA    - INTEGER.                                            ==
C     ==    On entry, LDA specifies the first dimension of A as       ==
C     ==    declared in the calling (sub) program. When SIDE = 'L'    ==
C     ==    or 'l'  then LDA  must be at least max(1,m), when         ==
C     ==    SIDE = 'R' or 'r' then LDA must be at least max(1,n).     ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == B      - DOUBLE PRECISION array of DIMENSION (LDB,n).        ==
C     ==    Before entry, the leading m by n part of the array B must ==
C     ==    contain the matrix B, and on exit is overwritten by the   ==
C     ==    transformed matrix.                                       ==
C     ==                                                              ==
C     == LDB    - INTEGER.                                            ==
C     ==    On entry, LDB specifies the first dimension of B as       ==
C     ==    declared in the calling (sub) program. LDB must be at     ==
C     ==    least max(1,m).                                           ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     ==                                                              ==
C     == Level 3 Blas routine.                                        ==
C     ==                                                              ==
C     == -- Written on 8-February-1989.                               ==
C     ==    Jack Dongarra, Argonne National Laboratory.               ==
C     ==    Iain Duff, AERE Harwell.                                  ==
C     ==    Jeremy Du Croz, Numerical Algorithms Group Ltd.           ==
C     ==    Sven Hammarling, Numerical Algorithms Group Ltd.          ==
C     ==--------------------------------------------------------------==
C     Scalar Arguments
      CHARACTER*1      SIDE,UPLO,TRANSA,DIAG
      INTEGER          M,N,LDA,LDB
      DOUBLE PRECISION ALPHA
C     Array Arguments
      DOUBLE PRECISION A(LDA,*),B(LDB,*)
C     External Functions
      LOGICAL          LSAME
      EXTERNAL         LSAME
C     External Subroutines
      EXTERNAL         XERBLA
C     Intrinsic Functions
      INTRINSIC        MAX
C     Local Scalars 
      LOGICAL          LSIDE,NOUNIT,UPPER
      INTEGER          I,INFO,J,K,NROWA
      DOUBLE PRECISION TEMP
C     Parameters
      DOUBLE PRECISION ONE,ZERO
      PARAMETER       (ONE=1.0D+0,ZERO=0.0D+0)
C     ==--------------------------------------------------------------==
C
C     Test the input parameters
C
      LSIDE=LSAME(SIDE,'L')
      IF(LSIDE) THEN
        NROWA=M
      ELSE
        NROWA=N
      ENDIF
      NOUNIT=LSAME(DIAG,'N')
      UPPER =LSAME(UPLO,'U')
C
      INFO=0
      IF((.NOT.LSIDE).AND.
     $   (.NOT.LSAME(SIDE,'R'))) THEN
        INFO=1
      ELSE IF((.NOT.UPPER).AND.
     $        (.NOT.LSAME(UPLO,'L'))) THEN
        INFO=2
      ELSE IF((.NOT.LSAME(TRANSA,'N')).AND.
     $        (.NOT.LSAME(TRANSA,'T')).AND.
     $        (.NOT.LSAME(TRANSA,'C'))) THEN
        INFO=3
      ELSE IF((.NOT.LSAME(DIAG,'U')).AND.
     $        (.NOT.LSAME(DIAG,'N'))) THEN
        INFO=4
      ELSE IF(M.LT.0) THEN
        INFO=5
      ELSE IF(N.LT.0) THEN
        INFO=6
      ELSE IF(LDA.LT.MAX(1,NROWA)) THEN
        INFO=9
      ELSE IF(LDB.LT.MAX(1,M)) THEN
        INFO=11
      ENDIF
      IF(INFO.NE.0) THEN
        CALL XERBLA('DTRMM ',INFO)
        RETURN
      ENDIF
C
C     Quick return if possible
C
      IF(N.EQ.0) RETURN
C
C     And when alpha.eq.zero
C
      IF(ALPHA.EQ.ZERO) THEN
!$OMP parallel do private(J,I)
        DO J=1,N
          DO I=1,M
            B(I,J)=ZERO
          ENDDO
        ENDDO
        RETURN
      ENDIF
C
C     Start the operations
C
      IF(LSIDE) THEN
        IF(LSAME(TRANSA,'N')) THEN
C     ==--------------------------------------------------------------==
C     ==  Form B := alpha*A*B                                         ==
C     ==--------------------------------------------------------------==
          IF(UPPER) THEN
!$OMP parallel do private(J,K,I,TEMP) 
            DO J=1,N
              DO K=1,M
!!              IF(B(K,J).NE.ZERO) THEN
                  TEMP=ALPHA*B(K,J)
                  DO I=1,K-1
                    B(I,J)=B(I,J)+TEMP*A(I,K)
                  ENDDO
                  IF(NOUNIT) TEMP=TEMP*A(K,K)
                  B(K,J)=TEMP
!!              ENDIF
              ENDDO
            ENDDO
          ELSE
!$OMP parallel do private(J,K,I,TEMP) 
            DO J=1,N
              DO K=M,1,-1
!!              IF(B(K,J).NE.ZERO) THEN
                  TEMP=ALPHA*B(K,J)
                  B(K,J)=TEMP
                  IF(NOUNIT) B(K,J)=B(K,J)*A(K,K)
                  DO I=K+1,M
                    B(I,J)=B(I,J)+TEMP*A(I,K)
                  ENDDO
!!              ENDIF
              ENDDO
            ENDDO
          ENDIF
        ELSE
C     ==--------------------------------------------------------------==
C     ==  Form B := alpha*A'*B                                        ==
C     ==--------------------------------------------------------------==
          IF(UPPER) THEN
!$OMP parallel do private(J,I,K,TEMP) 
            DO J=1,N
              DO I=M,1,-1
                TEMP=B(I,J)
                IF(NOUNIT) TEMP=TEMP*A(I,I)
                DO K=1,I-1
                  TEMP=TEMP+A(K,I)*B(K,J)
                ENDDO
                B(I,J)=ALPHA*TEMP
              ENDDO
            ENDDO
          ELSE
!$OMP parallel do private(J,I,K,TEMP) 
            DO J=1,N
              DO I=1,M
                TEMP=B(I,J)
                IF(NOUNIT) TEMP=TEMP*A(I,I)
                DO K=I+1,M
                  TEMP=TEMP+A(K,I)*B(K,J)
                ENDDO
                B(I,J)=ALPHA*TEMP
              ENDDO
            ENDDO
          ENDIF
        ENDIF
      ELSE
        IF(LSAME(TRANSA,'N')) THEN
C     ==--------------------------------------------------------------==
C     ==  Form B := alpha*B*A                                         ==
C     ==--------------------------------------------------------------==
          IF(UPPER) THEN
!$OMP parallel do private(J,I,K,TEMP) schedule(static,1)
            DO J=N,1,-1
              TEMP=ALPHA
              IF(NOUNIT) TEMP=TEMP*A(J,J)
              DO I=1,M
                B(I,J)=TEMP*B(I,J)
              ENDDO
              DO K=1,J-1
                TEMP=ALPHA*A(K,J)
                DO I=1,M
                  B(I,J)=B(I,J)+TEMP*B(I,K)
                ENDDO 
              ENDDO
            ENDDO 
          ELSE
!$OMP parallel do private(J,I,K,TEMP) schedule(static,1)
            DO J=1,N
              TEMP=ALPHA
              IF(NOUNIT) TEMP=TEMP*A(J,J)
              DO I=1,M
                B(I,J)=TEMP*B(I,J)
              ENDDO
              DO K=J+1,N
                TEMP=ALPHA*A(K,J)
                DO I=1,M
                  B(I,J)=B(I,J)+TEMP*B(I,K)
                ENDDO
              ENDDO
            ENDDO
          ENDIF
        ELSE
C     ==--------------------------------------------------------------==
C     ==  Form B := alpha*B*A'                                        ==
C     ==--------------------------------------------------------------==
          IF(UPPER) THEN
!$OMP parallel do private(J,I,K,TEMP) schedule(static,1)
            DO K=1,N
              DO J=1,K-1
                TEMP=ALPHA*A(J,K)
                DO I=1,M
                  B(I,J)=B(I,J)+TEMP*B(I,K)
                ENDDO
              ENDDO
              TEMP=ALPHA
              IF(NOUNIT) TEMP=TEMP*A(K,K)
              IF(TEMP.NE.ONE) THEN
                DO I=1,M
                  B(I,K)=TEMP*B(I,K)
                ENDDO
              ENDIF
            ENDDO
          ELSE
!$OMP parallel do private(J,I,K,TEMP) schedule(static,1)
            DO K=N,1,-1
              DO J=K+1,N
                TEMP=ALPHA*A(J,K)
                DO I=1,M
                  B(I,J)=B(I,J)+TEMP*B(I,K)
                ENDDO
              ENDDO
              TEMP=ALPHA
              IF(NOUNIT) TEMP=TEMP*A(K,K)
              IF(TEMP.NE.ONE) THEN
                DO I=1,M
                  B(I,K)=TEMP*B(I,K)
                ENDDO
              ENDIF
            ENDDO
          ENDIF
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE ZGEMM(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,BETA,C,LDC)
C     ==--------------------------------------------------------------==
C     == Purpose                                                      ==
C     == =======                                                      ==
C     ==                                                              ==
C     == ZGEMM performs one of the matrix-matrix operations           ==
C     ==                                                              ==
C     == C := alpha*op(A)*op(B) + beta*C,                             ==
C     ==                                                              ==
C     == where op(X) is one of                                        ==
C     ==                                                              ==
C     == op(X) = X  or  op(X) = X'  or  op(X) = conjg(X'),            ==
C     ==                                                              ==
C     == alpha and beta are scalars, and A, B and C are matrices,     ==
C     == with op(A) an m by k matrix, op(B) a k by n matrix and C     ==
C     == an m by n matrix.                                            ==
C     ==                                                              ==
C     == Parameters                                                   ==
C     == ==========                                                   ==
C     ==                                                              ==
C     == TRANSA - CHARACTER*1.                                        ==
C     ==    On entry,TRANSA specifies the form of op(A) to be         ==
C     ==    used in the matrix multiplication as follows:             ==
C     ==                                                              ==
C     ==       TRANSA = 'N' or 'n',  op(A) = A.                       ==
C     ==                                                              ==
C     ==       TRANSA = 'T' or 't',  op(A) = A'.                      ==
C     ==                                                              ==
C     ==       TRANSA = 'C' or 'c',  op(A) = conjg(A').               ==
C     ==                                                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == TRANSB - CHARACTER*1.                                        ==
C     ==    On entry, TRANSB specifies the form of op(B) to be        == 
C     ==    used in the matrix multiplication as follows:             ==
C     ==                                                              ==
C     ==       TRANSB = 'N' or 'n',  op(B) = B.                       ==
C     ==                                                              ==
C     ==       TRANSB = 'T' or 't',  op(B) = B'.                      ==
C     ==                                                              ==
C     ==       TRANSB = 'C' or 'c',  op(B) = conjg(B').               ==
C     ==                                                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == M      - INTEGER.                                            ==
C     ==    On entry, M specifies the number of rows of the matrix    ==
C     ==    op(A) and of the matrix C. M must be at least zero.       ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == N      - INTEGER.                                            ==
C     ==    On entry, N specifies the number of columns of the matrix ==
C     ==    op(B) and the number of columns of the matrix C. N must   ==
C     ==    be at least zero.                                         ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == K      - INTEGER.                                            ==
C     ==    On entry, K specifies the number of columns of the matrix ==
C     ==    op(A) and the number of rows of the matrix op(B).         ==
C     ==    K must be at least zero.                                  ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == ALPHA  - COMPLEX*16.                                         ==
C     ==    On entry, ALPHA specifies the scalar alpha.               ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == A      - COMPLEX*16 array of DIMENSION (LDA,ka), where ka    ==      
C     ==    is k when TRANSA = 'N' or 'n', and is m otherwise.        ==
C     ==    Before entry with TRANSA = 'N' or 'n', the leading m by k ==
C     ==    part of the array A must contain the matrix A, otherwise  ==
C     ==    the leading k by m part of the array A must contain the   ==
C     ==    matrix A.                                                 ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == LDA    - INTEGER.                                            ==
C     ==    On entry, LDA specifies the first dimension of A as       ==
C     ==    declared in the calling (sub) program. When               ==
C     ==    TRANSA = 'N' or 'n' then LDA must be at least             ==
C     ==    max(1,m), otherwise LDA must be at least max(1,k).        ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == B      - COMPLEX*16 array of DIMENSION (LDB,kb), where kb    ==
C     ==    is n when TRANSB = 'N' or 'n', and is k otherwise.        ==
C     ==    Before entry with TRANSB = 'N' or 'n', the leading k by n ==
C     ==    part of the array B must contain the matrix B, otherwise  ==
C     ==    the leading n by k part of the array B must contain the   ==
C     ==    matrix B.                                                 ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == LDB    - INTEGER.                                            ==
C     ==    On entry, LDB specifies the first dimension of B as       ==
C     ==    declared in the calling (sub) program. When TRANSB = 'N'  ==
C     ==    or 'n' then LDB must be at least max(1,k), otherwise      ==
C     ==    LDB must be at least max(1,n).                            ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == BETA   - COMPLEX*16.                                         ==
C     ==    On entry, BETA specifies the scalar beta. When BETA is    ==
C     ==    supplied as zero then C need not be set on input.         ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == C      - COMPLEX*16 array of DIMENSION (LDC,n).              ==
C     ==    Before entry, the leading m by n part of the array C must ==
C     ==    contain the matrix C, except when beta is zero, in which  ==
C     ==    case C need not be set on entry. On exit, the array C is  ==
C     ==    overwritten by the m by n matrix                          ==
C     ==    (alpha*op(A)*op(B) + beta*C).                             ==
C     ==                                                              ==
C     == LDC    - INTEGER.                                            ==
C     ==    On entry, LDC specifies the first dimension of C as       ==
C     ==    declared in the calling (sub) program. LDC must be at     ==
C     ==    least max(1,m).                                           ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     ==                                                              ==
C     == Level 3 Blas routine.                                        ==
C     ==                                                              ==
C     == -- Written on 8-February-1989.                               ==
C     ==    Jack Dongarra, Argonne National Laboratory.               ==
C     ==    Iain Duff, AERE Harwell.                                  ==
C     ==    Jeremy Du Croz, Numerical Algorithms Group Ltd.           ==
C     ==    Sven Hammarling, Numerical Algorithms Group Ltd.          ==
C     ==--------------------------------------------------------------==
C     Scalar Arguments
      CHARACTER*1 TRANSA,TRANSB
      INTEGER     M,N,K,LDA,LDB,LDC
      COMPLEX*16  ALPHA,BETA
C     Array Arguments
      COMPLEX*16  A(LDA,*),B(LDB,*),C(LDC,*)
C     External Functions
      LOGICAL     LSAME
      EXTERNAL    LSAME
C     External Subroutines
      EXTERNAL    XERBLA
C     Intrinsic Functions
      INTRINSIC   DCONJG,MAX
C     Local Scalars
      LOGICAL     CONJA,CONJB,NOTA,NOTB
      INTEGER     I,INFO,J,L,NCOLA,NROWA,NROWB
      COMPLEX*16  TEMP
C     Parameters
      COMPLEX*16  ONE
      PARAMETER  (ONE=(1.0D+0,0.0D+0))
      COMPLEX*16  ZERO
      PARAMETER  (ZERO=(0.0D+0,0.0D+0))
C     ==--------------------------------------------------------------==
C     
C     Set NOTA and NOTB as true if A and B respectively are not
C     conjugated or transposed, set CONJA and CONJB as true if A and
C     B respectively are to be transposed but not conjugated and set
C     NROWA, NCOLA and NROWB as the number of rows and columns of A
C     and the number of rows of B respectively.
C    
      NOTA =LSAME(TRANSA,'N')
      NOTB =LSAME(TRANSB,'N')
      CONJA=LSAME(TRANSA,'C')
      CONJB=LSAME(TRANSB,'C')
      IF(NOTA) THEN
        NROWA=M
        NCOLA=K
      ELSE
        NROWA=K
        NCOLA=M
      ENDIF
      IF(NOTB) THEN
        NROWB=K
      ELSE
        NROWB=N
      ENDIF
C
C     Test the input parameters
C
      INFO=0
      IF((.NOT.NOTA).AND.
     $   (.NOT.CONJA).AND.
     $   (.NOT.LSAME(TRANSA,'T'))) THEN
        INFO=1
      ELSE IF((.NOT.NOTB).AND.
     $        (.NOT.CONJB).AND.
     $        (.NOT.LSAME(TRANSB,'T'))) THEN
        INFO=2
      ELSE IF(M.LT.0) THEN
        INFO=3
      ELSE IF(N.LT.0) THEN
        INFO=4
      ELSE IF(K.LT.0) THEN
        INFO=5
      ELSE IF(LDA.LT.MAX(1,NROWA)) THEN
        INFO=8
      ELSE IF(LDB.LT.MAX(1,NROWB)) THEN
        INFO=10
      ELSE IF(LDC.LT.MAX(1,M)) THEN
        INFO=13
      ENDIF
      IF(INFO.NE.0) THEN
        CALL XERBLA('ZGEMM ',INFO)
        RETURN
      ENDIF
C
C     Quick return if possible
C
      IF((M.EQ.0).OR.(N.EQ.0).OR.
     $   (((ALPHA.EQ.ZERO).OR.(K.EQ.0)).AND.(BETA.EQ.ONE)))
     $  RETURN
C
C     And when alpha.eq.zero
C
      IF(ALPHA.EQ.ZERO) THEN
        IF(BETA.EQ.ZERO) THEN
!$OMP parallel do private(J,I) 
          DO J=1,N
            DO I=1,M
              C(I,J)=ZERO
            ENDDO
          ENDDO
        ELSE
!$OMP parallel do private(J,I) 
          DO J=1,N
            DO I=1,M
              C(I,J)=BETA*C(I,J)
            ENDDO
          ENDDO
        ENDIF
        RETURN
      ENDIF
C
C     Start the operations
C
      IF(NOTB) THEN
        IF(NOTA) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A*B + beta*C                                ==
C     ==--------------------------------------------------------------==
          IF(BETA.EQ.ZERO) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                C(I,J)=ZERO
              ENDDO
              DO L=1,K
                TEMP=ALPHA*B(L,J)
                DO I=1,M
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO L=1,K
                TEMP=ALPHA*B(L,J)
                DO I=1,M
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ELSE 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                C(I,J)=BETA*C(I,J)
              ENDDO
              DO L=1,K
                TEMP=ALPHA*B(L,J)
                DO I=1,M
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ENDIF 
        ELSE IF(CONJA) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*conjg(A')*B + beta*C                        ==
C     ==--------------------------------------------------------------==
          IF(BETA.EQ.ZERO) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+DCONJG(A(L,I))*B(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+DCONJG(A(L,I))*B(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP+C(I,J)
              ENDDO
            ENDDO
          ELSE 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+DCONJG(A(L,I))*B(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP+BETA*C(I,J)
              ENDDO
            ENDDO
          ENDIF 
        ELSE
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A'*B + beta*C                               ==
C     ==--------------------------------------------------------------==
          IF(BETA.EQ.ZERO) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP+C(I,J)
              ENDDO
            ENDDO
          ELSE 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(L,J)
                ENDDO
                C(I,J)=ALPHA*TEMP+BETA*C(I,J)
              ENDDO
            ENDDO
          ENDIF 
        ENDIF
      ELSE IF(NOTA) THEN
        IF(CONJB) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A*conjg(B') + beta*C                        ==
C     ==--------------------------------------------------------------==
          IF(BETA.EQ.ZERO) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                C(I,J)=ZERO
              ENDDO
              DO L=1,K
                TEMP=ALPHA*DCONJG(B(J,L))
                DO I=1,M
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO L=1,K
                TEMP=ALPHA*DCONJG(B(J,L))
                DO I=1,M
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ELSE 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                C(I,J)=BETA*C(I,J)
              ENDDO
              DO L=1,K
                TEMP=ALPHA*DCONJG(B(J,L))
                DO I=1,M
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ENDIF 
        ELSE
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A*B' + beta*C                               ==
C     ==--------------------------------------------------------------==
          IF(BETA.EQ.ZERO) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                C(I,J)=ZERO
              ENDDO
              DO L=1,K
                TEMP=ALPHA*B(J,L)
                DO I=1,M
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO L=1,K
                TEMP=ALPHA*B(J,L)
                DO I=1,M
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ELSE 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                C(I,J)=BETA*C(I,J)
              ENDDO
              DO L=1,K
                TEMP=ALPHA*B(J,L)
                DO I=1,M
                  C(I,J)=C(I,J)+TEMP*A(I,L)
                ENDDO
              ENDDO
            ENDDO
          ENDIF 
        ENDIF
      ELSE IF(CONJA) THEN
        IF(CONJB) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*conjg(A')*conjg(B') + beta*C                ==
C     ==--------------------------------------------------------------==
          IF(BETA.EQ.ZERO) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+DCONJG(A(L,I))*DCONJG(B(J,L))
                ENDDO
                C(I,J)=ALPHA*TEMP
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+DCONJG(A(L,I))*DCONJG(B(J,L))
                ENDDO
                C(I,J)=ALPHA*TEMP+C(I,J)
              ENDDO
            ENDDO
          ELSE 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+DCONJG(A(L,I))*DCONJG(B(J,L))
                ENDDO
                C(I,J)=ALPHA*TEMP+BETA*C(I,J)
              ENDDO
            ENDDO
          ENDIF 
        ELSE
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*conjg(A')*B' + beta*C                       ==
C     ==--------------------------------------------------------------==
          IF(BETA.EQ.ZERO) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+DCONJG(A(L,I))*B(J,L)
                ENDDO
                C(I,J)=ALPHA*TEMP
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+DCONJG(A(L,I))*B(J,L)
                ENDDO
                C(I,J)=ALPHA*TEMP+C(I,J)
              ENDDO
            ENDDO
          ELSE 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+DCONJG(A(L,I))*B(J,L)
                ENDDO
                C(I,J)=ALPHA*TEMP+BETA*C(I,J)
              ENDDO
            ENDDO
          ENDIF 
        ENDIF
      ELSE
        IF(CONJB) THEN
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A'*conjg(B') + beta*C                       ==
C     ==--------------------------------------------------------------==
          IF(BETA.EQ.ZERO) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*DCONJG(B(J,L))
                ENDDO
                C(I,J)=ALPHA*TEMP
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*DCONJG(B(J,L))
                ENDDO
                C(I,J)=ALPHA*TEMP+C(I,J)
              ENDDO
            ENDDO
          ELSE 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*DCONJG(B(J,L))
                ENDDO
                C(I,J)=ALPHA*TEMP+BETA*C(I,J)
              ENDDO
            ENDDO
          ENDIF 
        ELSE
C     ==--------------------------------------------------------------==
C     ==  Form C := alpha*A'*B' + beta*C                              ==
C     ==--------------------------------------------------------------==
          IF(BETA.EQ.ZERO) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(J,L)
                ENDDO
                C(I,J)=ALPHA*TEMP
              ENDDO
            ENDDO
          ELSE IF(BETA.EQ.ONE) THEN 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(J,L)
                ENDDO
                C(I,J)=ALPHA*TEMP+C(I,J)
              ENDDO
            ENDDO
          ELSE 
!$OMP parallel do private(J,I,L,TEMP)
            DO J=1,N
              DO I=1,M
                TEMP=ZERO
                DO L=1,K
                  TEMP=TEMP+A(L,I)*B(J,L)
                ENDDO
                C(I,J)=ALPHA*TEMP+C(I,J)
              ENDDO
            ENDDO
          ENDIF 
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE ZGEMV(TRANS,M,N,ALPHA,A,LDA,X,INCX,BETA,Y,INCY)
C     ==--------------------------------------------------------------==
C     == Purpose                                                      ==
C     == =======                                                      ==
C     ==                                                              ==
C     == ZGEMV performs one of the matrix-vector operations           ==
C     ==                                                              ==
C     == y := alpha*A*x + beta*y, or y := alpha*A'*x + beta*y, or     ==
C     ==                                                              ==
C     == y := alpha*conjg(A')*x + beta*y,                             ==
C     ==                                                              ==
C     == where alpha and beta are scalars, x and y are vectors        ==
C     == and A is an m by n matrix.                                   ==
C     ==                                                              ==
C     == Parameters                                                   ==
C     == ==========                                                   ==
C     ==                                                              ==
C     == TRANS  - CHARACTER*1.                                        ==
C     ==    On entry, TRANS specifies the operation to be performed   ==
C     ==    as follows:                                               ==
C     ==                                                              ==
C     ==       TRANS = 'N' or 'n'  y := alpha*A*x + beta*y.           ==
C     ==                                                              ==
C     ==       TRANS = 'T' or 't'  y := alpha*A'*x + beta*y.          ==
C     ==                                                              ==
C     ==       TRANS = 'C' or 'c'  y := alpha*conjg(A')*x + beta*y.   ==
C     ==                                                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == M      - INTEGER.                                            ==
C     ==    On entry, M specifies the number of rows of the matrix A. ==
C     ==    M must be at least zero.                                  ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == N      - INTEGER.                                            ==
C     ==    On entry, N specifies the number of columns of the matrix ==
C     ==    A. N must be at least zero.                               ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == ALPHA  - COMPLEX*16.                                         ==
C     ==    On entry, ALPHA specifies the scalar alpha.               ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == A      - COMPLEX*16 array of DIMENSION (LDA,n).              ==
C     ==    Before entry, the leading m by n part of the array A      ==
C     ==    must contain the matrix of coefficients.                  ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == LDA    - INTEGER.                                            ==
C     ==    On entry, LDA specifies the first dimension of A as       ==
C     ==    declared in the calling (sub) program. LDA must be at     ==
C     ==    least max(1,m).                                           ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == X      - COMPLEX*16 array of DIMENSION at least              ==
C     ==    (1+(n-1)*abs(INCX)) when TRANS = 'N' or 'n' and at least  ==
C     ==    (1+(m-1)*abs(INCX)) otherwise. Before entry, the          ==
C     ==    incremented array X must contain the vector x.            ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == INCX   - INTEGER.                                            ==
C     ==    On entry, INCX specifies the increment for the elements   ==
C     ==    of X. INCX must not be zero.                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == BETA   - COMPLEX*16.                                         ==
C     ==    On entry, BETA specifies the scalar beta. When BETA is    ==
C     ==    supplied as zero then Y need not be set on input.         ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     == Y      - COMPLEX*16 array of DIMENSION at least              ==
C     ==    (1+(m-1)*abs(INCY)) when TRANS = 'N' or 'n' and at least  ==
C     ==    (1+(n-1)*abs(INCY)) otherwise. Before entry with BETA     ==
C     ==    non-zero, the incremented array Y must contain the vector ==
C     ==    y. On exit, Y is overwritten by the updated vector y.     ==
C     ==                                                              ==
C     == INCY   - INTEGER.                                            ==
C     ==    On entry, INCY specifies the increment for the elements   ==
C     ==    of Y. INCY must not be zero.                              ==
C     ==    Unchanged on exit.                                        ==
C     ==                                                              ==
C     ==                                                              ==
C     == Level 2 Blas routine.                                        ==
C     ==                                                              ==
C     == -- Written on 22-October-1986.                               ==
C     ==    Jack Dongarra, Argonne National Lab.                      ==
C     ==    Jeremy Du Croz, Nag Central Office.                       ==
C     ==    Sven Hammarling, Nag Central Office.                      ==
C     ==    Richard Hanson, Sandia National Labs.                     ==
C     ==--------------------------------------------------------------==
C     Scalar Arguments
      COMPLEX*16  ALPHA,BETA
      INTEGER     INCX,INCY,LDA,M,N
      CHARACTER*1 TRANS
C     Array Arguments
      COMPLEX*16  A(LDA,*),X(*),Y(*)
C     Parameters
      COMPLEX*16  ONE
      PARAMETER  (ONE =(1.0D+0,0.0D+0))
      COMPLEX*16  ZERO
      PARAMETER  (ZERO=(0.0D+0,0.0D+0))
C     Local Scalars
      COMPLEX*16  TEMP
      INTEGER     I,INFO,IX,IY,J,JX,JY,KX,KY,LENX,LENY
      LOGICAL     NOCONJ
C     External Functions
      LOGICAL     LSAME
      EXTERNAL    LSAME
C     External Subroutines
      EXTERNAL    XERBLA
C     Intrinsic Functions
      INTRINSIC   DCONJG,MAX
C     Auxiliary variables
      INTEGER     IJY(N),IIX(M),IJX(N),IIY(M)
C     ==--------------------------------------------------------------==
C
C     Test the input parameters
C
      INFO=0
      IF(.NOT.LSAME(TRANS,'N').AND.
     $   .NOT.LSAME(TRANS,'T').AND.
     $   .NOT.LSAME(TRANS,'C')) THEN
        INFO=1
      ELSE IF(M.LT.0) THEN
        INFO=2
      ELSE IF(N.LT.0) THEN
        INFO=3
      ELSE IF(LDA.LT.MAX(1,M)) THEN
        INFO=6
      ELSE IF(INCX.EQ.0) THEN
        INFO=8
      ELSE IF(INCY.EQ.0) THEN
        INFO=11
      ENDIF
      IF(INFO.NE.0) THEN
        CALL XERBLA('ZGEMV ',INFO)
        RETURN
      ENDIF
C
C     Quick return if possible
C
      IF((M.EQ.0).OR.(N.EQ.0).OR.
     $   ((ALPHA.EQ.ZERO).AND.(BETA.EQ.ONE))) RETURN
C
      NOCONJ=LSAME(TRANS,'T')
C
C     Set LENX and LENY, the lengths of the vectors x and y, 
C     and set up the start points in X and Y.
C
      IF(LSAME(TRANS,'N')) THEN
        LENX=N
        LENY=M
      ELSE
        LENX=M
        LENY=N
      ENDIF
      IF(INCX.GT.0) THEN
        KX=1
      ELSE
        KX=1-(LENX-1)*INCX
      ENDIF
      IF(INCY.GT.0) THEN
        KY=1
      ELSE
        KY=1-(LENY-1)*INCY
      ENDIF
C
C     Start the operations. In this version the elements of A
C     are accessed sequentially with one pass through A.
C
C     First form  y := beta*y.
C
      IF(BETA.NE.ONE) THEN
        IF(INCY.EQ.1) THEN
          IF(BETA.EQ.ZERO) THEN
!$OMP parallel do private(I)
            DO I=1,LENY
              Y(I)=ZERO
            ENDDO
          ELSE
!$OMP parallel do private(I)
            DO I=1,LENY
              Y(I)=BETA*Y(I)
            ENDDO
          ENDIF
        ELSE
          IF(BETA.EQ.ZERO) THEN
!$OMP parallel do private(I,IY)
#ifdef __SR8000
*poption parallel, tlocal(I,IY)
#endif 
            DO I=1,LENY
              IY=KY+(I-1)*INCY
              Y(IY)=ZERO
            ENDDO
          ELSE
!$OMP parallel do private(I,IY)
!OCL NOVREC(Y)
!CDIR NODEP(Y)
#ifdef __SR8000
*poption parallel, tlocal(I,IY)
#endif 
            DO I=1,LENY
              IY=KY+(I-1)*INCY
              Y(IY)=BETA*Y(IY)
            ENDDO
          ENDIF
        ENDIF
      ENDIF
      IF(ALPHA.EQ.ZERO) RETURN 
      IF(LSAME(TRANS,'N')) THEN
C     ==--------------------------------------------------------------==
C     ==  Form y := alpha*A*x + y                                     ==
C     ==--------------------------------------------------------------==
C..Fill list vector IJX
C..IJX(1:N) -> {KX,KX+INCX,KX+2*INCX,...,KX+(N-1)*INCX}
        DO J=1,N 
          IJX(J)=KX+(J-1)*INCX
        ENDDO 
        IF(INCY.EQ.1) THEN
          DO J=1,N
            JX=IJX(J)
            TEMP=ALPHA*X(JX)
!$OMP parallel do private(I)
            DO I=1,M
              Y(I)=Y(I)+TEMP*A(I,J)
            ENDDO
          ENDDO
        ELSE
C..Fill list vector IIY
C..IIY(1:M) -> {KY,KY+INCY,KY+2*INCY,...,KY+(M-1)*INCY}
          DO J=1,M 
            IIY(J)=KY+(J-1)*INCY
          ENDDO 
!OCL NOVREC(Y)
          DO J=1,N
            JX=IJX(J)
            TEMP=ALPHA*X(JX)
!$OMP parallel do private(I,IY)
!CDIR NODEP(Y)
#ifdef __SR8000
*poption parallel, tlocal(I,IY)
#endif 
            DO I=1,M
              IY=IIY(I)
              Y(IY)=Y(IY)+TEMP*A(I,J)
            ENDDO
          ENDDO
        ENDIF
      ELSE
C     ==--------------------------------------------------------------==
C     ==  Form y := alpha*A'*x + y or y := alpha*conjg( A' )*x + y    ==
C     ==--------------------------------------------------------------==
C..Fill list vector IJY
C..IJY(1:N) -> {KY,KY+INCY,KY+2*INCY,...,KY+(N-1)*INCY}
        DO J=1,N 
          IJY(J)=KY+(J-1)*INCY
        ENDDO 
        IF(INCX.EQ.1) THEN
          IF(NOCONJ) THEN
!$OMP parallel do private(J,I,TEMP,JY)
!OCL NOVREC(Y)
!CDIR NODEP(Y)
#ifdef __SR8000
*poption parallel, tlocal(J,I,TEMP,JY)
#endif 
            DO J=1,N
              TEMP=ZERO
              DO I=1,M
                TEMP=TEMP+A(I,J)*X(I)
              ENDDO
              JY=IJY(J) 
              Y(JY)=Y(JY)+ALPHA*TEMP
            ENDDO
          ELSE 
!$OMP parallel do private(J,I,TEMP,JY)
!OCL NOVREC(Y)
!CDIR NODEP(Y)
#ifdef __SR8000
*poption parallel, tlocal(J,I,TEMP,JY)
#endif 
            DO J=1,N
              TEMP=ZERO
              DO I=1,M
                TEMP=TEMP+DCONJG(A(I,J))*X(I)
              ENDDO
              JY=IJY(J) 
              Y(JY)=Y(JY)+ALPHA*TEMP
            ENDDO
          ENDIF 
        ELSE
C..Fill list vector IIX
C..IIX(1:M) -> {KX,KX+INCX,KX+2*INCX,...,KX+(M-1)*INCX}
          DO J=1,M 
            IIX(J)=KX+(J-1)*INCX
          ENDDO 
          IF(NOCONJ) THEN 
!$OMP parallel do private(J,I,IX,JY,TEMP)
!OCL NOVREC(Y)
!CDIR NODEP(Y)
#ifdef __SR8000
*poption parallel, tlocal(J,I,IX,JY,TEMP)
#endif 
            DO J=1,N
              TEMP=ZERO
              DO I=1,M
                IX=IIX(I)
                TEMP=TEMP+A(I,J)*X(IX)
              ENDDO
              JY=IJY(J)
              Y(JY)=Y(JY)+ALPHA*TEMP
            ENDDO
          ELSE 
!$OMP parallel do private(J,I,IX,JY,TEMP)
!OCL NOVREC(Y)
!CDIR NODEP(Y)
#ifdef __SR8000
*poption parallel, tlocal(J,I,IX,JY,TEMP)
#endif 
            DO J=1,N
              TEMP=ZERO
              DO I=1,M
                IX=IIX(I)
                TEMP=TEMP+DCONJG(A(I,J))*X(IX)
              ENDDO
              JY=IJY(J)
              Y(JY)=Y(JY)+ALPHA*TEMP
            ENDDO
          ENDIF 
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE DGEMM2(TRANSA,TRANSB,M,N,K,ALPHA,A,LDA,B,LDB,
     $                  BETA,C,LDC)
C     ==--------------------------------------------------------------==
C     Scalar Arguments
      CHARACTER*1       TRANSA,TRANSB
      INTEGER           M,N,K,LDA,LDB,LDC
      DOUBLE PRECISION  ALPHA,BETA
C     Array Arguments
      DOUBLE PRECISION  A(LDA,*),B(LDB,*),C(LDC,*)
      INTEGER           I,INFO,J,L,NCOLA,NROWA,NROWB
      DOUBLE PRECISION  TEMP
C     Parameters
      DOUBLE PRECISION  ONE,ZERO
      PARAMETER        (ONE=1.0D+0,ZERO=0.0D+0)
      INTEGER           II,NM 
C     Auxiliary variables
      INTEGER           LISTJ(N*M),LISTI(N*M) 
C     ==--------------------------------------------------------------==
      II=0
      DO J=1,N 
        DO I=1,M 
          II=II+1
          LISTJ(II)=J 
          LISTI(II)=I 
        ENDDO 
      ENDDO 
      NM=II
      IF(BETA.EQ.ZERO) THEN 
! Form  C := alpha*A'*B
!$OMP parallel do private(II,J,I,L,TEMP)
        DO II=1,NM 
          J=LISTJ(II) 
          I=LISTI(II) 
          TEMP=ZERO
          DO L=1,K
            TEMP=TEMP+A(L,I)*B(L,J)
          ENDDO 
          C(I,J)=ALPHA*TEMP
        ENDDO
      ELSE 
! Form  C := alpha*A'*B + beta*C
!$OMP parallel do private(II,J,I,L,TEMP) 
!OCL NOVREC(C)
!CDIR NODEP(C)
        DO II=1,NM 
          J=LISTJ(II) 
          I=LISTI(II) 
          TEMP=ZERO
          DO L=1,K
            TEMP=TEMP+A(L,I)*B(L,J)
          ENDDO 
          C(I,J)=ALPHA*TEMP+BETA*C(I,J) 
        ENDDO
      ENDIF 
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      subroutine  dcopy2(n,dx,incx,dy,incy)
C     ==--------------------------------------------------------------==
c
c     copies a vector, x, to a vector, y.
c     uses unrolled loops for increments equal to one.
c     jack dongarra, linpack, 3/11/78.
c     modified 12/3/93, array(1) declarations changed to array(*)
c
      double precision dx(*),dy(*)
      integer i,incx,incy,ix,iy,m,mp1,n
C     ==--------------------------------------------------------------==
c
      if(incx.ne.1.or.incy.ne.1) then
c
c       code for unequal increments or equal increments
c       not equal to 1
c
        ix = 1
        iy = 1
        if(incx.lt.0)ix = (-n+1)*incx + 1
        if(incy.lt.0)iy = (-n+1)*incy + 1
        do i = 1,n
          dy(iy) = dx(ix)
          ix = ix + incx
          iy = iy + incy
        end do
      else
c
c       code for both increments equal to 1
c
!$OMP parallel do private(i)
        do i = 1,n
          dy(i) = dx(i)
        end do
      end if
C     ==--------------------------------------------------------------==
      return
      end
C     ==================================================================

C     ==================================================================
      SUBROUTINE DSYMM(SIDE,UPLO,M,N,ALPHA,A,LDA,B,LDB,BETA,C,LDC)
C     ==--------------------------------------------------------------==
C  Purpose
C  =======
C
C  DSYMM  performs one of the matrix-matrix operations
C
C     C := alpha*A*B + beta*C,
C
C  or
C
C     C := alpha*B*A + beta*C,
C
C  where alpha and beta are scalars,  A is a symmetric matrix and  B and
C  C are  m by n matrices.
C
C  Parameters
C  ==========
C
C  SIDE   - CHARACTER*1.
C           On entry,  SIDE  specifies whether  the  symmetric matrix  A
C           appears on the  left or right  in the  operation as follows:
C
C              SIDE = 'L' or 'l'   C := alpha*A*B + beta*C,
C
C              SIDE = 'R' or 'r'   C := alpha*B*A + beta*C,
C
C           Unchanged on exit.
C
C  UPLO   - CHARACTER*1.
C           On  entry,   UPLO  specifies  whether  the  upper  or  lower
C           triangular  part  of  the  symmetric  matrix   A  is  to  be
C           referenced as follows:
C
C              UPLO = 'U' or 'u'   Only the upper triangular part of the
C                                  symmetric matrix is to be referenced.
C
C              UPLO = 'L' or 'l'   Only the lower triangular part of the
C                                  symmetric matrix is to be referenced.
C
C           Unchanged on exit.
C
C  M      - INTEGER.
C           On entry,  M  specifies the number of rows of the matrix  C.
C           M  must be at least zero.
C           Unchanged on exit.
C
C  N      - INTEGER.
C           On entry, N specifies the number of columns of the matrix C.
C           N  must be at least zero.
C           Unchanged on exit.
C
C  ALPHA  - DOUBLE PRECISION.
C           On entry, ALPHA specifies the scalar alpha.
C           Unchanged on exit.
C
C  A      - DOUBLE PRECISION array of DIMENSION ( LDA, ka ), where ka is
C           m  when  SIDE = 'L' or 'l'  and is  n otherwise.
C           Before entry  with  SIDE = 'L' or 'l',  the  m by m  part of
C           the array  A  must contain the  symmetric matrix,  such that
C           when  UPLO = 'U' or 'u', the leading m by m upper triangular
C           part of the array  A  must contain the upper triangular part
C           of the  symmetric matrix and the  strictly  lower triangular
C           part of  A  is not referenced,  and when  UPLO = 'L' or 'l',
C           the leading  m by m  lower triangular part  of the  array  A
C           must  contain  the  lower triangular part  of the  symmetric
C           matrix and the  strictly upper triangular part of  A  is not
C           referenced.
C           Before entry  with  SIDE = 'R' or 'r',  the  n by n  part of
C           the array  A  must contain the  symmetric matrix,  such that
C           when  UPLO = 'U' or 'u', the leading n by n upper triangular
C           part of the array  A  must contain the upper triangular part
C           of the  symmetric matrix and the  strictly  lower triangular
C           part of  A  is not referenced,  and when  UPLO = 'L' or 'l',
C           the leading  n by n  lower triangular part  of the  array  A
C           must  contain  the  lower triangular part  of the  symmetric
C           matrix and the  strictly upper triangular part of  A  is not
C           referenced.
C           Unchanged on exit.
C
C  LDA    - INTEGER.
C           On entry, LDA specifies the first dimension of A as declared
C           in the calling (sub) program.  When  SIDE = 'L' or 'l'  then
C           LDA must be at least  max( 1, m ), otherwise  LDA must be at
C           least  max( 1, n ).
C           Unchanged on exit.
C
C  B      - DOUBLE PRECISION array of DIMENSION ( LDB, n ).
C           Before entry, the leading  m by n part of the array  B  must
C           contain the matrix B.
C           Unchanged on exit.
C
C  LDB    - INTEGER.
C           On entry, LDB specifies the first dimension of B as declared
C           in  the  calling  (sub)  program.   LDB  must  be  at  least
C           max( 1, m ).
C           Unchanged on exit.
C
C  BETA   - DOUBLE PRECISION.
C           On entry,  BETA  specifies the scalar  beta.  When  BETA  is
C           supplied as zero then C need not be set on input.
C           Unchanged on exit.
C
C  C      - DOUBLE PRECISION array of DIMENSION ( LDC, n ).
C           Before entry, the leading  m by n  part of the array  C must
C           contain the matrix  C,  except when  beta  is zero, in which
C           case C need not be set on entry.
C           On exit, the array  C  is overwritten by the  m by n updated
C           matrix.
C
C  LDC    - INTEGER.
C           On entry, LDC specifies the first dimension of C as declared
C           in  the  calling  (sub)  program.   LDC  must  be  at  least
C           max( 1, m ).
C           Unchanged on exit.
C
C
C  Level 3 Blas routine.
C
C  -- Written on 8-February-1989.
C     Jack Dongarra, Argonne National Laboratory.
C     Iain Duff, AERE Harwell.
C     Jeremy Du Croz, Numerical Algorithms Group Ltd.
C     Sven Hammarling, Numerical Algorithms Group Ltd.
C     ==--------------------------------------------------------------==
C     Scalar Arguments
      CHARACTER*1      SIDE,UPLO
      INTEGER          M,N,LDA,LDB,LDC
      DOUBLE PRECISION ALPHA,BETA
C     Array Arguments
      DOUBLE PRECISION A(LDA,*),B(LDB,*),C(LDC,*)
C     External Functions
      LOGICAL          LSAME
      EXTERNAL         LSAME
C     External Subroutines
      EXTERNAL         XERBLA
C     Intrinsic Functions
      INTRINSIC        MAX
C     Local Scalars
      LOGICAL          UPPER
      INTEGER          I,INFO,J,K,NROWA
      DOUBLE PRECISION TEMP1,TEMP2
C     Parameters
      DOUBLE PRECISION ONE,ZERO
      PARAMETER       (ONE=1.0D+0,ZERO=0.0D+0)
C     ==--------------------------------------------------------------==
C
C     Set NROWA as the number of rows of A.
C
      IF(LSAME(SIDE,'L')) THEN
        NROWA = M
      ELSE
        NROWA = N
      ENDIF
      UPPER = LSAME(UPLO,'U')
C
C     Test the input parameters.
C
      INFO = 0
      IF((.NOT.LSAME(SIDE,'L')).AND.
     $   (.NOT.LSAME(SIDE,'R'))) THEN
        INFO = 1
      ELSE IF((.NOT.UPPER).AND.
     $        (.NOT.LSAME(UPLO,'L'))) THEN
        INFO = 2
      ELSE IF(M.LT.0) THEN
        INFO = 3
      ELSE IF(N.LT.0) THEN
        INFO = 4
      ELSE IF(LDA.LT.MAX(1,NROWA)) THEN
        INFO = 7
      ELSE IF(LDB.LT.MAX(1,M)) THEN
        INFO = 9
      ELSE IF(LDC.LT.MAX(1,M)) THEN
        INFO = 12
      ENDIF
      IF(INFO.NE.0) THEN
        CALL XERBLA('DSYMM ',INFO)
        RETURN
      ENDIF
C
C     Quick return if possible.
C
      IF((M.EQ.0).OR.(N.EQ.0).OR.
     $   ((ALPHA.EQ.ZERO).AND.(BETA.EQ.ONE)))
     $  RETURN
C
C     And when alpha.eq.zero.
C
      IF(ALPHA.EQ.ZERO) THEN
        IF(BETA.EQ.ZERO) THEN
!$OMP parallel do private(J,I)
          DO J=1,N
            DO I=1,M
              C(I,J)=ZERO
            ENDDO
          ENDDO
        ELSE
!$OMP parallel do private(J,I)
          DO J=1,N
            DO I=1,M
              C(I,J)=BETA*C(I,J)
            ENDDO
          ENDDO 
        ENDIF
        RETURN
      ENDIF
C
C     Start the operations.
C
      IF(LSAME(SIDE,'L')) THEN
C
C       Form  C := alpha*A*B + beta*C.
C
        IF(UPPER) THEN
!$OMP parallel do private(J,I,K,TEMP1,TEMP2)
          DO J=1,N
            DO I=1,M
              TEMP1=ALPHA*B(I,J)
              TEMP2=ZERO
              DO K=1,I-1
                C(K,J)=C(K,J)+TEMP1*A(K,I)
                TEMP2=TEMP2+B(K,J)*A(K,I)
              ENDDO
              IF(BETA.EQ.ZERO) THEN
                C(I,J)=TEMP1*A(I,I)+ALPHA*TEMP2
              ELSE
                C(I,J)=BETA*C(I,J)+TEMP1*A(I,I)+ALPHA*TEMP2
              ENDIF
            ENDDO
          ENDDO
        ELSE
!$OMP parallel do private(J,I,K,TEMP1,TEMP2)
          DO J=1,N
            DO I=M,1,-1
              TEMP1=ALPHA*B(I,J)
              TEMP2=ZERO
              DO K=I+1,M
                C(K,J)=C(K,J)+TEMP1*A(K,I)
                TEMP2=TEMP2+B(K,J)*A(K,I)
              ENDDO 
              IF(BETA.EQ.ZERO) THEN
                C(I,J)=TEMP1*A(I,I)+ALPHA*TEMP2
              ELSE
                C(I,J)=BETA*C(I,J)+TEMP1*A(I,I)+ALPHA*TEMP2
              ENDIF
            ENDDO
          ENDDO
        ENDIF
      ELSE
C
C       Form  C := alpha*B*A + beta*C.
C
!$OMP parallel do private(J,I,K,TEMP1,TEMP2)
        DO J=1,N
          TEMP1=ALPHA*A(J,J)
          IF(BETA.EQ.ZERO) THEN
            DO I=1,M
              C(I,J)=TEMP1*B(I,J)
            ENDDO
          ELSE
            DO I=1,M
              C(I,J)=BETA*C(I,J)+TEMP1*B(I,J)
            ENDDO 
          ENDIF
          DO K=1,J-1
            IF(UPPER) THEN
              TEMP1=ALPHA*A(K,J)
            ELSE
              TEMP1=ALPHA*A(J,K)
            ENDIF
            DO I=1,M
              C(I,J)=C(I,J)+TEMP1*B(I,K)
            ENDDO
          ENDDO
          DO K=J+1,N
            IF(UPPER) THEN
              TEMP1=ALPHA*A(J,K)
            ELSE
              TEMP1=ALPHA*A(K,J)
            ENDIF
            DO I=1,M
              C(I,J)=C(I,J)+TEMP1*B(I,K)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
