C     ==================================================================
      SUBROUTINE BOX_BOUNDARY(TAUP,VELP) 
C     ==--------------------------------------------------------------==
C     == Imposing reflecting walls to a system free from periodic     ==
C     == boundary conditions to keep atoms inside the box             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE 
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'cell.inc'
      INCLUDE 'comvel.inc'
C     Arguments 
      REAL*8  TAUP(3,NAX,NSP),VELP(3,NAX,NSP)
      REAL*8  VCMIO(4),ALAY,ALAZ
C     Variables 
      INTEGER IS,IA
C     ==--------------------------------------------------------------==
      IF(.NOT.TCLUST) RETURN 
C     Modify velocities of the ions near to the boundary 
      IF(TTWOD) THEN ! two-dimensional PBC
C     We assume orthorhobmic box with surface in specified plane
        IF(SNORMAL.EQ.1) THEN      ! YZ PLANE
          DO IS=1,NSP
            DO IA=1,NA(IS)
              IF(TAUP(1,IA,IS).LT.WALL_SKIN) 
     .                            VELP(1,IA,IS) =  DABS(VELP(1,IA,IS))
              IF((CELLDM(1)-TAUP(1,IA,IS)).LT.WALL_SKIN)
     .                            VELP(1,IA,IS) = -DABS(VELP(1,IA,IS))
            ENDDO
          ENDDO
        ELSE IF(SNORMAL.EQ.2) THEN ! ZX PLANE
          ALAY=CELLDM(2)*CELLDM(1)
          DO IS=1,NSP
            DO IA=1,NA(IS)
              IF(TAUP(2,IA,IS).LT.WALL_SKIN) 
     .                            VELP(2,IA,IS) =  DABS(VELP(2,IA,IS))
              IF((ALAY-TAUP(2,IA,IS)).LT.WALL_SKIN)
     .                            VELP(2,IA,IS) = -DABS(VELP(2,IA,IS))
            ENDDO
          ENDDO
        ELSE IF(SNORMAL.EQ.3) THEN ! XY PLANE
          ALAZ=CELLDM(3)*CELLDM(1)
          DO IS=1,NSP
            DO IA=1,NA(IS)
              IF(TAUP(3,IA,IS).LT.WALL_SKIN) 
     .                            VELP(3,IA,IS) =  DABS(VELP(3,IA,IS))
              IF((ALAZ-TAUP(3,IA,IS)).LT.WALL_SKIN)
     .                            VELP(3,IA,IS) = -DABS(VELP(3,IA,IS))
            ENDDO
          ENDDO
        ENDIF
      ELSE IF(TONED) THEN  ! one-dimensional PBC
C     We assume periodicity along x, orthorhobmic box in y and z
        ALAZ=CELLDM(3)*CELLDM(1)
        ALAY=CELLDM(2)*CELLDM(1) 
        DO IS=1,NSP
          DO IA=1,NA(IS)
            IF(TAUP(2,IA,IS).LT.WALL_SKIN)
     .                          VELP(2,IA,IS) =  DABS(VELP(2,IA,IS))
            IF((ALAY-TAUP(2,IA,IS)).LT.WALL_SKIN)
     .                          VELP(2,IA,IS) = -DABS(VELP(2,IA,IS))
            IF(TAUP(3,IA,IS).LT.WALL_SKIN)
     .                          VELP(3,IA,IS) =  DABS(VELP(3,IA,IS))
            IF((ALAZ-TAUP(3,IA,IS)).LT.WALL_SKIN)
     .                          VELP(3,IA,IS) = -DABS(VELP(3,IA,IS))
          ENDDO 
        ENDDO 
      ELSE ! no PBC along any direction
        ALAZ=CELLDM(3)*CELLDM(1)
        ALAY=CELLDM(2)*CELLDM(1)
        DO IS=1,NSP
          DO IA=1,NA(IS)
            IF(TAUP(1,IA,IS).LT.WALL_SKIN)
     .                          VELP(1,IA,IS) =  DABS(VELP(1,IA,IS))
            IF((CELLDM(1)-TAUP(1,IA,IS)).LT.WALL_SKIN)
     .                          VELP(1,IA,IS) = -DABS(VELP(1,IA,IS))
            IF(TAUP(2,IA,IS).LT.WALL_SKIN)
     .                          VELP(2,IA,IS) =  DABS(VELP(2,IA,IS))
            IF((ALAY-TAUP(2,IA,IS)).LT.WALL_SKIN)
     .                          VELP(2,IA,IS) = -DABS(VELP(2,IA,IS))
            IF(TAUP(3,IA,IS).LT.WALL_SKIN)
     .                          VELP(3,IA,IS) =  DABS(VELP(3,IA,IS))
            IF((ALAZ-TAUP(3,IA,IS)).LT.WALL_SKIN)
     .                          VELP(3,IA,IS) = -DABS(VELP(3,IA,IS))
          ENDDO 
        ENDDO 
      ENDIF 
C     Subtract center of mass velocity of the whole system 
C     after each walls reflection
      IF(TSUBCOM) THEN
        CALL COMVEL(VELP,VCMIO,.TRUE.)
        CALL COMVEL(VELP,VCMIO,.FALSE.)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN 
      END 
C     ==================================================================
