C     ==================================================================
      SUBROUTINE BROYDEN(XMIX,IT,RINM1,RIN0,ROUT0,RINP,NHG,NHGB,
     &                   ILSD,HG,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     == GENERATE NEXT ITERATION USING BROYDENS  METHOD               ==
C     == (FROM DD JOHNSON, PHYS. REV. B, 38,12807,(1988)              ==
C     == NHGB IS THE NUMBER OF PLANE-WAVES TO BE MIXED                ==
C     == RIN0 IS INPUT DENSITY FROM CURRENT STEP (IT)                 ==
C     == ROUT0 IS OUTPUT DENSITY FROM CURRENT STEP                    ==
C     == RINM1 IS INPUT DENSITY FROM PREVIOUS STEP                    ==
C     == RINP  IS NEW MIXED DENSITY                                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'geq0.inc'
      INCLUDE 'broy.inc' 
      INCLUDE 'spin.inc'     !NLSD
C     Arguments
      INTEGER    IT,NHG,NHGB,ILSD,LSCR
      COMPLEX*16 RINM1(*),RIN0(*),ROUT0(*),RINP(*)
      REAL*8     XMIX,HG(*),SCR(*)
C     Variables
      REAL*8     W(IBRESET,NLSD),A(IBRESET,IBRESET),B(IBRESET,IBRESET),
     &           C(IBRESET),GAMMA(IBRESET)
      POINTER    (IP_W,W),(IP_A,A),(IP_B,B),(IP_C,C),(IP_GAMMA,GAMMA)
      COMPLEX*16 DELTAF(NHGB,IBRESET,NLSD),F(NHGB),
     &           U(NHGB,IBRESET,NLSD),FM1(NHGB,NLSD)
      POINTER    (IP_DELTAF,DELTAF),(IP_U,U),(IP_F,F),(IP_FM1,FM1)
      CHARACTER  TAG*30
      REAL*8     SN,AUX
      REAL*8     DOTP
      EXTERNAL   DOTP
      INTEGER    I,J,IG,ISUB,INFO,K,LBROYDEN,
     &           IFIRST
      DATA       IFIRST /0/
      SAVE       IFIRST,IP_FM1,IP_U,IP_DELTAF,IP_W
C     ==--------------------------------------------------------------==
      CALL TISET('   BROYDEN',ISUB)
      IF(IT.GT.IBRESET) THEN 
        WRITE(6,*) ' WARNING| IT GT IBRESET IN BROYDEN. RESETTING IT=1'
        IT=1
      ENDIF
      IF(IFIRST.EQ.0) THEN
C       Allocate FM1 (use to store F for the next generation).
        CALL MEMORY(IP_FM1,2*NHGB*NLSD,'FM1')
C       Allocate U   (use to store Delta RHO).
        CALL MEMORY(IP_U,  2*NHGB*IBRESET*NLSD,'U')
C       Allocate DELTAF (use to store F(:)-FM1(:))
        CALL MEMORY(IP_DELTAF,2*NHGB*IBRESET*NLSD,'DELTAF')
C       Allocate W   (use to store weights)
        CALL MEMORY(IP_W,IBRESET*NLSD,'W')
        IFIRST=1
      ENDIF
C     ==--------------------------------------------------------------==
C     Test SCR
      CALL GIVE_SCR_BROYDEN(LBROYDEN,TAG,NHGB)
      CALL TEST_SCR('BROYDEN',TAG,LSCR,LBROYDEN)
C     Allocation inside SCR
      LBROYDEN=LBROYDEN-2*NHGB+1 !Now LBROYDEN is THE INDEX (+1)
      CALL GET_ADDR(IP_F,SCR(LBROYDEN))
      LBROYDEN=LBROYDEN-IBRESET*IBRESET
      CALL GET_ADDR(IP_A,SCR(LBROYDEN))
      LBROYDEN=LBROYDEN-IBRESET*IBRESET
      CALL GET_ADDR(IP_B,SCR(LBROYDEN))
      LBROYDEN=LBROYDEN-IBRESET
      CALL GET_ADDR(IP_C,SCR(LBROYDEN))
      LBROYDEN=LBROYDEN-IBRESET
      IF(LBROYDEN.NE.1) CALL STOPGM('BROYDEN','PB ALLOCATION')
      CALL GET_ADDR(IP_GAMMA,SCR(LBROYDEN))
C     ==--------------------------------------------------------------==
C     For SUMMAT, A(:,:)=0 to avoid floating point exception
      CALL AZZERO(A,IBRESET*IBRESET)
C..F=nout-n
!$OMP parallel do private(IG)
      DO IG=1,NHGB
        F(IG)=ROUT0(IG)-RIN0(IG)
      ENDDO
C..For the first iteration use linear mixing
      IF(IT.EQ.1) THEN 
        IF(TADBROY) THEN
C..Calculate W(IT)
          W(1,ILSD)=DOTP(NHGB,F,F)
#ifdef PARALLEL
          CALL GLOSUM(1,W(1,ILSD))
#endif
          W(1,ILSD)=1.D0/DSQRT(W(1,ILSD))
          W(1,ILSD)=MAX(1.D0,W(1,ILSD))
        ELSE
          W(1,ILSD)=1.D0
        ENDIF
C..Mixed density on first iteration
        IF(KERMIX.EQ.0.D0) THEN
!$OMP parallel do private(IG)
          DO IG=1,NHGB
            RINP(IG)=RIN0(IG)+XMIX*F(IG) !Linear mixing
          ENDDO
        ELSE 
!$OMP parallel do private(IG)
          DO IG=1,NHGB
            RINP(IG)=RIN0(IG)+XMIX*F(IG)
     &                *HG(IG)/(HG(IG)+KERMIX*KERMIX) !Kerker mixing
          ENDDO
        ENDIF
C..Store F in FM1 for next iteration
        CALL DCOPY(2*NHGB,F,1,FM1(1,ILSD),1)
      ENDIF
C     ==============================================================
      IF(IT.GT.1) THEN
C..DELTA F
!$OMP parallel do private(IG)
        DO IG=1,NHGB
          DELTAF(IG,IT-1,ILSD)=F(IG)-FM1(IG,ILSD)
        ENDDO
C..Store F in FM1 for next iteration
        CALL DCOPY(2*NHGB,F,1,FM1(1,ILSD),1)
        SN=DOTP(NHGB,DELTAF(1,IT-1,ILSD),DELTAF(1,IT-1,ILSD))
#ifdef PARALLEL
        CALL GLOSUM(1,SN)
#endif
        SN=1.D0/DSQRT(SN)
        CALL DSCAL(NHGB*2,SN,DELTAF(1,IT-1,ILSD),1)
C..DELTA u
!$OMP parallel do private(IG)
        DO IG=1,NHGB
          U(IG,IT-1,ILSD)=RIN0(IG)-RINM1(IG)
        ENDDO
        CALL DSCAL(NHGB*2,SN,U(1,IT-1,ILSD),1)
C..|U(n)>=G(1)|DELTAF(n)+|DELTA n(n)>
        CALL DAXPY(NHGB*2,XMIX,DELTAF(1,IT-1,ILSD),1,U(1,IT-1,ILSD),1)
        IF(TADBROY) THEN
C..Calculate W(IT-1)
          W(IT-1,ILSD)=DOTP(NHGB,F,F)
#ifdef PARALLEL
          CALL GLOSUM(1,W(IT-1,ILSD))
#endif
          W(IT-1,ILSD)=1.D0/DSQRT(W(IT-1,ILSD))
          W(IT-1,ILSD)=MAX(1.D0,W(IT-1,ILSD))
        ELSE
          W(IT-1,ILSD)=1.D0
        ENDIF
C..Calculate aij array
        CALL DSYRK('U','T',IT-1,2*NHGB,2.D0,DELTAF(1,1,ILSD),2*NHGB,
     &             0.D0,A,IBRESET)
        CALL DMATC('U',IT-1,A,IBRESET)
        IF(GEQ0)
     *  CALL DGER(IT-1,IT-1,-1.D0,DELTAF(1,1,ILSD),2*NHGB,
     *            DELTAF(1,1,ILSD),2*NHGB,A,IBRESET)
#ifdef PARALLEL
        CALL SUMMAT(A,IBRESET,B,IBRESET*IBRESET)
#endif
        DO I=1,IT-1
          DO J=1,IT-1
            A(J,I)=W(I,ILSD)*W(J,ILSD)*A(J,I)
          ENDDO
          A(I,I)=W02BROY+A(I,I)
        ENDDO
C..Invert A array. Output in A.
        CALL INVERSEMAT(IT-1,A,IBRESET,B,INFO)
C..Compute C(k)=w(k)*<deltaf(k)|f(m)>        
        DO K=1,IT-1
          C(K)=W(K,ILSD)*DOTP(NHGB,DELTAF(1,K,ILSD),F)
        ENDDO
#ifdef PARALLEL
        CALL GLOSUM(IT-1,C(1))
#endif
C..Compute Gamma(L)=sum_k c(k) a(k,l)
        CALL DGEMV('T',IT-1,IT-1,1.D0,A,IBRESET,C,1,0.D0,GAMMA,1)
C..Compute mixed density
!$OMP parallel do private(IG)
        DO IG=1,NHGB
          RINP(IG)=RIN0(IG)+XMIX*F(IG)
        ENDDO
        DO I=1,IT-1
          AUX=W(I,ILSD)*GAMMA(I)
          CALL DAXPY(2*NHGB,-AUX,U(1,I,ILSD),1,RINP,1)
        ENDDO
      ENDIF
C..Copy the last nhg-nhgb elements of F to rinp
      IF(NHGB.LT.NHG)
     &     CALL ZCOPY(NHG-NHGB,ROUT0(NHGB+1),1,RINP(NHGB+1),1)
      CALL TIHALT('   BROYDEN',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_BROYDEN(LBROYDEN,TAG,NHGB)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'broy.inc'        !IBRESET
C     Arguments
      INTEGER   LBROYDEN,NHGB
      CHARACTER TAG*30
C     ==--------------------------------------------------------------==
      LBROYDEN=
     &     2*NHGB+              !F
     &     IBRESET*IBRESET+     !A
     &     IBRESET*IBRESET+     !B
     &     IBRESET+             !C
     &     IBRESET              !GAMMA
      TAG='2*NHGB+IBRESET*(2*NHGB+...)'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE KERKER(XMIX,YMIX,RIN0,ROUT0,RINP,NHG,HG)
C     ==--------------------------------------------------------------==
C     == GENERATE NEXT ITERATION WITH KERKER DAMPING FACTOR           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8     XMIX,YMIX,HG(*)
      COMPLEX*16 RIN0(*),ROUT0(*),RINP(*)
      INTEGER    NHG
C     Variables
      INTEGER    IG
C     ==--------------------------------------------------------------==
!$OMP parallel do private(IG)
      DO IG=1,NHG
        RINP(IG)=RIN0(IG)+XMIX*(ROUT0(IG)-RIN0(IG))
     &                   *HG(IG)/(HG(IG)+YMIX*YMIX)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
