C     ==================================================================
      SUBROUTINE BS_FORCES_DIAG(NSTATE,C0,C2,CR,SC0,CSCR,VPP,EIGV,
     &                       RHOE,PSI,SCR,LSCR,
     &                       TAU0,VELP,TAUI,FION,IFCALC,
     &                       IREC,TFOR,TINFO)
C     ==--------------------------------------------------------------==
C     == ENTER WITH THE IONIC POSITIONS IN TAU0 AND AN INPUT GUESS    ==
C     == FOR THE DENSITY IN RIN0, THIS ROUTINE RETURNS THE CONVERGED  ==
C     == DENSITY, WAVEFUNCTIONS AND IONIC FORCES.                     ==
C     ==--------------------------------------------------------------==
C     == TAU0: ATOMIC POSITION                                        ==
C     == VELP and TAUI are necessary for RESTART FILE                 ==
C     == FION: IONIC FORCES                                           ==
C     == IFCALC: total number of iterations                           ==
C     ==--------------------------------------------------------------==
C     == This subroutine is called in BS calculations instead of      ==
C     == FORCES_DIAG. Much of that code is repeated here.             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'        !NLSD
      INCLUDE 'elct.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'norm.inc'
      INCLUDE 'ropt.inc'        !CALSTE
      INCLUDE 'andr.inc'
      INCLUDE 'andp.inc'
      INCLUDE 'soft.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'tpar.inc'
      INCLUDE 'store.inc'
      include 'bsym.inc'
      include 'bsympnt.inc'
C     Arguments
      INTEGER    NSTATE,IFCALC,IREC(100),LSCR
      COMPLEX*16 C0(NGWK,N,BSFAC),C2(NGWK,N,BSFAC),CR(*),
     &           SC0(NGWK,N,BSFAC),CSCR(*),PSI(NNR1)
      REAL*8     EIGV(N,BSFAC),TAU0(*),VELP(*),TAUI(*),FION(*),
     &           VPP(*),
     &           RHOE(NNR1,NLSD),SCR(LSCR),THL(2),ETOTBS,ETOTHS
      LOGICAL    TFOR,TINFO,CVBSWF,CVHSWF
C     Variables
      CHARACTER TAG*30
      REAL*8    TIME1,TIMEF,ETOT0,DETOT,TIME2,TCPU
      EXTERNAL  TIMEF
      INTEGER   I,INFR,NHPSI,LFORCES_DIAG,MSGLEN
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_FNBS,3*NAX*NSX,'FNBS')
C     ==--------------------------------------------------------------==
      TIME1 = TIMEF()
      CALL GIVE_SCR_FORCES_DIAG(LFORCES_DIAG,TAG,NSTATE,TFOR)
      CALL TEST_SCR('FORCES_DIAG',TAG,LSCR,LFORCES_DIAG)
C     ==--------------------------------------------------------------==
      ECNSTR=0.0D0
      ERESTR=0.0D0
      CONVWF=.FALSE.
      CVBSWF=.FALSE.
      CVHSWF=.FALSE.
      ETOT0=0.0D0
C     ==--------------------------------------------------------------==
      if(PARENT) write(6,*) 'BSYMM: HS STATE'
      IF(.NOT.TINFO.AND.PARENT) WRITE(6,'(T9,A)')
     &     'INFR         GEMAX            ETOT         DETOT     TCPU'
      SDIIS=.TRUE.
      SPCG=.TRUE.
C     ==================================================================
C     ==                        MAIN  LOOP                            ==
C     ==================================================================
      DO INFR=1,NOMORE_ITER
        TIME1=TIMEF()
        IFCALC=IFCALC+1
C       Diagonalization
        if(.not.CVHSWF) then
C       HS state
          if(INFR.eq.1) then
            BSCLCS=2
            call SETBSSTATE
          endif  
          CALL UPDWF(C0(1,1,2),C2(1,1,2),SC0(1,1,2),TAU0,FION,CR,CSCR,
     &               VPP,EIGV(1,2),RHOE,PSI,SCR,LSCR,
     &               NSTATE,.FALSE.,.TRUE.)
          CVHSWF=CONVWF
        elseif(.not.CVBSWF) then      
C       BS state
          if(BSCLCS.eq.2) then 
            if(PARENT) then
              write(6,*) 'BSYMM: HS STATE CONVERGED. ',
     &             'SWITCHING TO BS STATE.'
              ETOT0=0.0D0
              IF(INFI.EQ.1)THEN 
                CALL DCOPY(2*NGWK*N,C0(1,1,2),1,C0,1)
                CALL DCOPY(N,EIGV(1,2),1,EIGV,1)
                CALL DCOPY(2*NGWK*N,C2(1,1,2),1,C2,1)
              END IF 
            endif
            BSCLCS=1
            call SETBSSTATE
            CALL UPDWF(C0,C2,SC0,TAU0,FION,CR,CSCR,VPP,
     &                 EIGV,RHOE,PSI,SCR,LSCR,NSTATE,
     &                 .FALSE.,.TRUE.)
            if(TINFO.and.PARENT) then
              CALL WRENER
              WRITE(6,'(2A)') ' NFI      GEMAX       CNORM',
     &              '           ETOT        DETOT      TCPU'
            endif
          else
            CALL UPDWF(C0,C2,SC0,TAU0,FION,CR,CSCR,VPP,
     &                 EIGV,RHOE,PSI,SCR,LSCR,NSTATE,
     &                 .FALSE.,.TRUE.)
          endif
          CVBSWF=CONVWF
        endif    
        CONVWF=CVHSWF.and.CVBSWF
C       Printout the evolution of the iterative optimization
        IF(PARENT) THEN
          DETOT=ETOT-ETOT0
          IF(INFR.EQ.1) DETOT=0.0D0
          TIME2=TIMEF()
          TCPU=(TIME2-TIME1)*0.001D0
          IF(TINFO) THEN
            CALL WRPRINT_WFOPT(EIGV(1,BSCLCS),F,AMU,NSTATE,
     &         ETOT,ETOT0,TCPU,GEMAX,CNORM,THL,.FALSE.,IFCALC,INFR)
          ELSE
            WRITE(6,
     &       '(T9,I4,T15,1PE12.3,T31,0PF12.6,T45,1PE12.3,T59,0PF7.2) ')
     &       INFR,GEMAX,ETOT,DETOT,TCPU
          ENDIF
          ETOT0=ETOT
        ENDIF
        IF(MOD(IFCALC,ISTORE).EQ.0)
     &       CALL ZHWWF(2,IREC,C0,C2,NSTATE,EIGV,TAU0,VELP,TAUI,NFI)
C       Check to break the loop.
        IF(.NOT.EXSOFT) CALL TESTEX(EXSOFT)
        IF(EXSOFT) GOTO 200     !Soft Exit (no force calculation)
        IF(CONVWF) GOTO 100     !Convergence of wavefunctions
      ENDDO
C     ==================================================================
C     ==                      END OF MAIN  LOOP                       ==
C     ==================================================================
      IF(PARENT) WRITE(6,'(A)') ' WARNING! DENSITY NOT CONVERGED !!!'
      IF(PARENT) WRITE(6,'(A)') ' WARNING! DENSITY NOT CONVERGED !!!'
      IF(PARENT) WRITE(6,'(A)') ' WARNING! DENSITY NOT CONVERGED !!!'
 100  CONTINUE
C     BS energy calculation 
C     BS state
      BSCLCS=1
      call SETBSSTATE
      CALL UPDWF(C0,C2,SC0,TAU0,FION,CR,CSCR,VPP,EIGV,
     &           RHOE,PSI,SCR,LSCR,NSTATE,TFOR,.TRUE.)
      if(PARENT) then
        if(INFI.eq.NOMORE) then
          print *
          print *, 'BSYMM: BS STATE ENERGIES:'
          call WRENER
        endif  
        ETOTBS=ETOT
        call DCOPY(3*NAX*NSX,FION,1,FNBS,1)
      endif
C     HS state      
      BSCLCS=2
      call SETBSSTATE
      CALL UPDWF(C0(1,1,2),C2(1,1,2),SC0(1,1,2),TAU0,FION,CR,CSCR,
     &           VPP,EIGV(1,2),RHOE,PSI,SCR,LSCR,NSTATE,
     &           TFOR,.TRUE.)
      if(PARENT) then
        if(INFI.eq.NOMORE) then
          print *
          print *, 'BSYMM: HS STATE ENERGIES:'
          call WRENER
        endif
        ETOTHS=ETOT
C       LS state      
        ETOT = (1 + CNSTWGT) * ETOTBS - CNSTWGT * ETOTHS
        call WRCCFL(ETOT,ETOTBS,ETOTHS)
      endif
      IF(TFOR) THEN
C       Forces from geometrical restraints
        IF(PARENT) THEN
          call LSFORCE(FNBS,FION)
          DO I=1,MRESTR
            RESVAL(I)=RESVAL(I)+GSRATE(I)*DT_IONS
          ENDDO
          CALL RESTFC(TAU0,SCR,FION)
        ENDIF
        MSGLEN = 3*NAX*NSX * 8
        CALL MY_BCAST(FION,MSGLEN,SOURCE,ALLGRP)
      ENDIF
 200  CONTINUE
      BSCLCS=1
      call SETBSSTATE
      CALL FREEM(IP_FNBS)
C     ==--------------------------------------------------------------==
      RETURN
      END
      
