C     ==================================================================
C     CDFT main source file containing all MD and WF-OPT related functions
C     H. Oberhofer (ho246@cam.ac.uk) 2009
C     OMP & optimizations - MB & TI - Strasbourg/Hyogo 2010
C     ==================================================================

C     ==================================================================
      SUBROUTINE CDFT_W(RHOE,TAU0,CHRG)
C     ==--------------------------------------------------------------==
C     == Computes weight functions (W) and Charges (CHRG)             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'qspl.inc'
      INCLUDE 'cdft.inc'
C     Arguments
      REAL*8    RHOE(NNR1,*),TAU0(3,NAX,NSX),
     &          CHRG(2),WN(NNR1)
C     Variables
      DIMENSION ATRG(MMAXX,*)
      REAL*8    DATOM(NSPLPO,2),ARHO(MMAXX),WORK(MMAXX)
      POINTER   (IP_ARHO,ARHO),(IP_WORK,WORK)
      INTEGER   IS,ISH,MMAX,L,IA,IAT,J,
     &          IFIRST,AINDEX,IT
      REAL*8    RCUT,CHGT,CHRG1,CHRG2
C     Allocation of local variables
      CALL MEMORY(IP_ARHO,MMAXX,'ARHO')
      CALL MEMORY(IP_WORK,MMAXX,'WORK')
C     ==--------------------------------------------------------------==
      IF(.NOT.TSPINC)THEN
        CALL DCOPY(NNR1,RHOE,1,RHOL,1)
      ELSE
!$OMP parallel do private(is)
        DO IS=1,NNR1
          RHOL(IS)=RHOE(IS,1)-2.0D0*RHOE(IS,2)
        ENDDO
!$OMP end parallel do
      ENDIF
c..Recalculate weights and charges
      IF(RECW) THEN
        IF(PARENT)WRITE(6,*) "RECALCULATING WEIGHTS"
        CALL AZZERO(WA,NNR1)
        IF(.NOT.THDAS)CALL AZZERO(WD,NNR1)
        CALL AZZERO(WN,NNR1)
        CHRG(1)=0.0D0
        CHRG(2)=0.0D0
c..Get Normalisation WN
        DO IS=1,NSP
         CALL ATDENS_NEW2(IS,DATOM,MMAX,RCUT,ARHO,WORK)
         DO IA=1,NA(IS)
           CALL PUTRHO(WN,TAU0(1,IA,IS),ATRG(1,IS),DATOM,
     *                MMAX,RCUT,1.D0)
          ENDDO
        ENDDO
c..ACCEPTORS
        DO IT=1,NACCR
          CALL ATDENS_NEW2(SPA(IT),DATOM,MMAX,RCUT,ARHO,WORK)
          MMX_A=MMAX
          CALL CDFT_CHG(RHOL,WA,WN,TAU0(1,SNA(IT),SPA(IT)),
     *             ATRG(1,SPA(IT)),DATOM,MMAX,RCUT,CHGT)
          CHRG(1)=CHRG(1)+CHGT
        ENDDO
c..DONORS
        DO IT=1,NDON
          CALL ATDENS_NEW2(SPD(IT),DATOM,MMAX,RCUT,ARHO,WORK)
          MMX_D=MMAX
          CALL CDFT_CHG(RHOL,WD,WN,TAU0(1,SND(IT),SPD(IT)),
     *             ATRG(1,SPD(IT)),DATOM,MMAX,RCUT,CHGT)
          CHRG(2)=CHRG(2)+CHGT
        ENDDO
        IF(.NOT.THDAS)THEN
!$OMP parallel do private(is)
          DO IS=1,NNR1
            WDIFF(IS)=WA(IS)-WD(IS)
          ENDDO
!$OMP end parallel do
        ELSE
!$OMP parallel do private(is)
          DO IS=1,NNR1
            WDIFF(IS)=WA(IS)
          ENDDO
!$OMP end parallel do
        ENDIF
        RECW=.FALSE.
        NEWGRAD=.FALSE.
      ELSE
c..only calculate the charges
        CALL CDFT_INT(RHOL,WA,TAU0(1,SNA,SPA),MMX_A,
     *              CHRG(1))
        IF(NDON.GT.0)THEN
          CALL CDFT_INT(RHOL,WD,TAU0(1,SND,SPD),MMX_D,
     *              CHRG(2))
        ENDIF
      ENDIF
      CALL GLOSUM(2,CHRG)
      CHRG(1)=-CHRG(1)*OMEGA/(NR1S*NR2S*NR3S)
      IF(NDON.GT.0)THEN
        CHRG(2)=-CHRG(2)*OMEGA/(NR1S*NR2S*NR3S)
      ELSE
        CHRG(2)=0
      ENDIF
      IF(.NOT.TSPINC)THEN
        CHRG1=CHRG(1)
        CHRG2=CHRG(2)
!$OMP parallel do private(it) reduction(+:CHRG1)
        DO IT=1,NACCR
!!        CHRG(1)=CHRG(1)+ZV(SPA(IT))
          CHRG1=CHRG1+ZV(SPA(IT))
        ENDDO
!$OMP end parallel do
!$OMP parallel do private(it) reduction(+:CHRG2)
        DO IT=1,NDON
!!        CHRG(2)=CHRG(2)+ZV(SPD(IT))
          CHRG2=CHRG2+ZV(SPD(IT))
        ENDDO
!$OMP end parallel do
        CHRG(1)=CHRG1
        CHRG(2)=CHRG2
      ENDIF
C     ==--------------------------------------------------------------==
C     Deallocation of local variables
      CALL FREEM(IP_WORK)
      CALL FREEM(IP_ARHO)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE CDFT_CHG(RHOE,WEIGHT,PSI,TAU,RG,DATOM,MMAX,
     *                    RCUT,CHG)
C     ==--------------------------------------------------------------==
C     == Computes weight functions (W) and Charges (CHG)              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'qspl.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'metr.inc'
C     Arguments
      INTEGER   MMAX
      REAL*8    RHOE(KR1,KR2S,*),PSI(KR1,KR2S,*),TAU(3),RG(*),
     &          DATOM(NSPLPO,2),WEIGHT(KR1,KR2S,*)
C     Variables
      REAL*8    SC1(3),SC2(3),DA1(3,3),XPTAU(3),X1,Y1,Z1,X,Y,Z,
     &          R,RCUT,CHG,CURV2,WCUR
      INTEGER   NPTAU(3),MR(3),I,IORGX,IORGY,IORGZ,IGRID,IX,II,I1,J,JJ,
     &          J1,K,KK,K1
C     ==--------------------------------------------------------------==
C..Atom position
      X1=TAU(1)
      Y1=TAU(2)
      Z1=TAU(3)
      CALL PBC(X1,Y1,Z1,SC1(1),SC1(2),SC1(3),1,APBC,IBRAV)
      CALL DGEMV('T',3,3,1.D0,HTM1,3,SC1,1,0.D0,SC2,1)
      IF(SC2(1).LT.0.D0) SC2(1)=SC2(1)+1.0D0
      IF(SC2(2).LT.0.D0) SC2(2)=SC2(2)+1.0D0
      IF(SC2(3).LT.0.D0) SC2(3)=SC2(3)+1.0D0
      NPTAU(1)=NINT(SC2(1)*NR1S)+1
      NPTAU(2)=NINT(SC2(2)*NR2S)+1
      NPTAU(3)=NINT(SC2(3)*NR3S)+1
      SC1(1)=SC2(1)-DBLE(NPTAU(1)-1)/DBLE(NR1S)
      SC1(2)=SC2(2)-DBLE(NPTAU(2)-1)/DBLE(NR2S)
      SC1(3)=SC2(3)-DBLE(NPTAU(3)-1)/DBLE(NR3S)
      IF(NPTAU(1).GT.NR1S) NPTAU(1)=1
      IF(NPTAU(2).GT.NR2S) NPTAU(2)=1
      IF(NPTAU(3).GT.NR3S) NPTAU(3)=1
      CALL DGEMV('T',3,3,1.D0,HT,3,SC1,1,0.D0,XPTAU,1)
      DO I=1,3
        DA1(I,1)=A1(I)/DBLE(NR1S)
        DA1(I,2)=A2(I)/DBLE(NR2S)
        DA1(I,3)=A3(I)/DBLE(NR3S)
      ENDDO
C..Interaction region
      SC1(1)=RCUT
      SC1(2)=RCUT
      SC1(3)=RCUT
      CALL DGEMV('T',3,3,1.D0,HTM1,3,SC1,1,0.D0,SC2,1)
      MR(1)=NINT(SC2(1)*NR1S)+1
      MR(2)=NINT(SC2(2)*NR2S)+1
      MR(3)=NINT(SC2(3)*NR3S)+1
      MR(1)=MIN(MR(1),NR1S/2-1)
      MR(2)=MIN(MR(2),NR2S/2-1)
      MR(3)=MIN(MR(3),NR3S/2-1)
C..Origin of density grid
      IORGX=NPTAU(1)-MR(1)
      IORGY=NPTAU(2)-MR(2)
      IORGZ=NPTAU(3)-MR(3)
      IF(IORGX.LT.1) IORGX=IORGX+NR1S
      IF(IORGY.LT.1) IORGY=IORGY+NR2S
      IF(IORGZ.LT.1) IORGZ=IORGZ+NR3S
C..loop over all grid points
      CHG=0.0D0
      IGRID=0
      DO I=0,2*MR(1)
        IX=IORGX+I
        IF(IX.GT.NR1S) IX=IX-NR1S
        IF(IX.GE.NRXPL(MEPOS,1).AND.IX.LE.NRXPL(MEPOS,2)) THEN
          II=IX-NRXPL(MEPOS,1)+1
          I1=MR(1)-I
          DO J=0,2*MR(2)
            JJ=IORGY+J
            IF(JJ.GT.NR2S) JJ=JJ-NR2S
            J1=MR(2)-J
            DO K=0,2*MR(3)
              KK=IORGZ+K
              IF(KK.GT.NR3S) KK=KK-NR3S
              K1=MR(3)-K
              CALL POSITX(X,Y,Z,I1,J1,K1,DA1,XPTAU(1))
              IF(TCLUST)THEN
                IF(X1-X.LT.A1(1).AND.Y1-Y.LT.A2(2).AND.Z1-Z.LT.A3(3)
     *           .AND.X1-X.GT.0.AND.Y1-Y.GT.0.AND.Z1-Z.GT.0)THEN
                R=DSQRT(X*X+Y*Y+Z*Z)
                IF(R.LT.RCUT) THEN
                  IGRID=IGRID+1
                  WCUR=CURV2(R,MMAX,RG,DATOM(1,1),DATOM(1,2),0.D0)/
     *               PSI(II,JJ,KK)
                  CHG=CHG+RHOE(II,JJ,KK)*WCUR
                  WEIGHT(II,JJ,KK)=WEIGHT(II,JJ,KK)+WCUR
                ENDIF
                ENDIF
              ELSE
                R=DSQRT(X*X+Y*Y+Z*Z)
                IF(R.LT.RCUT) THEN
                  IGRID=IGRID+1
                  WCUR=CURV2(R,MMAX,RG,DATOM(1,1),DATOM(1,2),0.D0)/
     *               PSI(II,JJ,KK)
                  CHG=CHG+RHOE(II,JJ,KK)*WCUR
                  WEIGHT(II,JJ,KK)=WEIGHT(II,JJ,KK)+WCUR
                ENDIF
              ENDIF
            ENDDO
          ENDDO
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE CDFT_INT(RHOE,WEIGHT,TAU,MMAX,CHG)
C     ==--------------------------------------------------------------==
C     == Computes only Charges (CHG) by integrating RHO*WEIGHT        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'qspl.inc'
      INCLUDE 'metr.inc'
C     Arguments
      INTEGER   MMAX
      REAL*8    RHOE(KR1,KR2S,*),TAU(3),
     &          WEIGHT(KR1,KR2S,*)
C     Variables
      REAL*8    SC1(3),SC2(3),DA1(3,3),XPTAU(3),X1,Y1,Z1,X,Y,Z,
     &          R,RCUT,CHG,CURV2
      INTEGER   NPTAU(3),MR(3),I,IORGX,IORGY,IORGZ,IGRID,IX,II,I1,J,JJ,
     &          J1,K,KK,K1
C     ==--------------------------------------------------------------==
      CHG=0.0D0
!!    IGRID=0
!$OMP parallel do private(II,JJ,KK) reduction(+:CHG)
      DO II=1,KR1
          DO JJ=1,KR2S
            DO KK=1,KR3S
!!              IGRID=IGRID+1
                CHG=CHG+RHOE(II,JJ,KK)*WEIGHT(II,JJ,KK)
            ENDDO
          ENDDO
      ENDDO
!$OMP end parallel do
      IGRID=KR1*KR2S*KR3S
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE INIT_CDFT
C     ==--------------------------------------------------------------==
C     == Initialises everything necessary for a CDFT calculation      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'qspl.inc'
      INCLUDE 'cdft.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'cnst.inc'
      INTEGER L,AINDEX,IS,IT,ISH,J
      LOGICAL CFIRST
      SAVE    CFIRST
      DATA    CFIRST /.TRUE./
      INTEGER    ELC ! number of elements in the buffer
      COMMON/RBUFF/ELC
C     For the calculation of RCUT
      REAL*8    DATOM(NSPLPO,2),ARHO(MMAXX),WORK(MMAXX)
      POINTER   (IP_ARHO,ARHO),(IP_WORK,WORK)
      INTEGER   MMAX,MSGLEN
      REAL*8    RCUT
C     Allocation of local variables
      CALL MEMORY(IP_ARHO,MMAXX,'ARHO')
      CALL MEMORY(IP_WORK,MMAXX,'WORK')


      IF(CFIRST)THEN
      CALL MEMORY(IP_RHOL,NNR1,'RHOL')
      CALL MY_BCAST(CDFT_A,(6*CDFT_MAT+4)*8/IRAT,SOURCE,ALLGRP)
      IF(NACCR.EQ.0.AND.PARENT) THEN
        CALL STOPGM('CDFT',"ERROR ACCEPTOR ATOMS NOT SET")
      ENDIF
      IF(MD.AND.TSPINC) THEN
        CALL STOPGM('CDFT',
     *     "ERROR SPIN DENSITY CONSTRAINT MD NOT IMPLEMENTED")
      ENDIF
      IF(GEOPT.AND.TSPINC) THEN
        CALL STOPGM('CDFT',
     *     "ERROR SPIN DENSITY CONSTRAINT GEO-OPT NOT IMPLEMENTED")
      ENDIF
      CALL MY_BCAST(TWGAUSS,8*2/IRAT,SOURCE,ALLGRP)
      CALL MY_BCAST(WCUT,8,SOURCE,ALLGRP)
      CALL MY_BCAST(VGSTEP,8*2,SOURCE,ALLGRP)
      IF(.NOT.BOHR)WCUT=WCUT*FBOHR
      IF(TWGAUSS)THEN
        IF(WG_N.EQ.1.AND.NSP.NE.1)THEN
          DO IS=2,NSP
            WG_SIGMA(IS)=WG_SIGMA(1)
          ENDDO
        ENDIF
        IF(.NOT.BOHR)THEN
          DO IS=1,NSP
            WG_SIGMA(IS)=WG_SIGMA(IS)*FBOHR
          ENDDO
        ENDIF
        CALL MY_BCAST(WG_SIGMA,NSP*8,SOURCE,ALLGRP)
      ENDIF
      IF(NDON.EQ.0.AND.PARENT)THEN
        CDFT_NC=-1.0d0*CDFT_NC
        NOTHER=-1.0d0*NOTHER
      ENDIF
      NATTOT=0
      NEWGRAD=.FALSE.
      DEKK=.FALSE.
      DO IS=1,NSP
        NUMAOR(IS)=0
        DO ISH=1,NSHELL(IS)
          L=LSHELL(ISH,IS)
          NATTOT=NATTOT+NA(IS)*(2*L+1)
          NUMAOR(IS)=NUMAOR(IS)+(2*L+1)
        ENDDO
      ENDDO

      CALL CDFT_ADARRAYS
      
      CALL MEMORY(IP_WA,NNR1,'WEIGHTA')
      CALL MEMORY(IP_WD,NNR1,'WEIGHTD')
      CALL MEMORY(IP_WDIFF,NNR1,'WDIFF')
      IF(TCZONES)THEN
        IF(CDFT_V.NE.0.0D0)THEN
          TOLOG=CZONES(3,2)
        ELSE
          TOLOG=CZONES(1,2)
        ENDIF
      ENDIF
      CALL MY_BCAST(CDFT_A,(6*CDFT_MAT+4)*8/IRAT,SOURCE,ALLGRP)
      CFIRST=.FALSE.


      IF(THDA)HDAFIRST=.TRUE.

      IF(RCDFT)CALL RCDFT_RESTART()
      ENDIF !CFIRST

      RECW=.TRUE.
      RECONVV=.TRUE.

!       IF(.NOT.OCSET)THEN
!         CALL DCOPY(M1SHLX*MAXSP,OC,1,OCUN,1)
!         MSGLEN = M1SHLX*(2*MAXSP)*8
!         CALL MY_BCAST(OCUN,MSGLEN,SOURCE,ALLGRP)
!       ENDIF

      CALL DERIV_INIT()

      DO IS=1,NSP
         CALL ATDENS_NEW2(IS,DATOM,MMAX,RCUT,ARHO,WORK)
         CDFT_RC(IS)=RCUT
      ENDDO

      IF(PARENT.AND.MD.AND.TPRED)THEN
        ELC=0
        IF(PREDORD+1.GT.MAXSAVEV)THEN
          WRITE(6,*) 'ERROR IN CDFT PREDICTOR'
          WRITE(6,*) 'PREDICTOR ORDER TOO LARGE, RESETTING ORDER TO',
     *     MAXSAVEV
          PREDORD=MAXSAVEV-1
        ENDIF
      ENDIF

      IF(PARENT)CALL CDFT_PRI()
      IF(TSYSCOMB)CALL MY_BCAST(N_S0,4/IRAT,SOURCE,ALLGRP)

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE CDFT_ADARRAYS()
C     ==--------------------------------------------------------------==
C     == (Re-)initialise the Donor and Acceptor arrays                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cdft.inc'
      INCLUDE 'ions.inc'

      INTEGER IT,IS,J,AINDEX

      DO IT=1,NDON
        AINDEX=1
        DO IS=1,NSP
          DO J=1,NA(IS)
            IF(AINDEX.EQ.CDFT_D(IT)) THEN
              SPD(IT)=IS
              SND(IT)=J
            ENDIF
            AINDEX=AINDEX+1
          ENDDO
        ENDDO
      ENDDO
      DO IT=1,NACCR
        AINDEX=1
        DO IS=1,NSP
          DO J=1,NA(IS)
            IF(AINDEX.EQ.CDFT_A(IT)) THEN
              SPA(IT)=IS
              SNA(IT)=J
            ENDIF
            AINDEX=AINDEX+1
          ENDDO
        ENDDO
      ENDDO

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE CDFT_REINIT()
C     ==--------------------------------------------------------------==
C     == Re-initialise the optimisers after an MD oder GeomOpt step   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'atwf.inc'
      INCLUDE 'cdft.inc'

      RECW=.TRUE.
      NEWGRAD=.FALSE.
      DEKK=.FALSE.
      IF(THDA)THEN
        IF(OCSET)THEN
          IF(PARENT)WRITE(6,*)"RESETTING INITIAL OCCUPATION NUMBER"
          CALL DCOPY(M1SHLX*MAXSP,OC2,1,OC,1)
        ENDIF
        IF(RCDFT)CALL RCDFT_RESTART()
      ENDIF

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE VUPDATE(C0,PSI,RHOE,SCR,LSCR,CONVTEST)
C     ==--------------------------------------------------------------==
C     == Calculate the Gradient of W wrt V perform a V optimisation   ==
C     == Step and check convergence                                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cdft.inc'
      INCLUDE 'pslo.inc' !<-- for Hirshfeld charges
      INCLUDE 'ions.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'qspl.inc' !<-- end
      INCLUDE 'elct.inc' !<-- electronic states
      INCLUDE 'prop.inc'
      INCLUDE 'coor.inc' !<-- coordinates
      INCLUDE 'spin.inc' !<-- for NLSD
      INCLUDE 'adat.inc' !<-- for EL
      INCLUDE 'ener.inc' !<-- for correct Parallel execution
      INCLUDE 'irat.inc'
      COMPLEX*16 C0(NGWK,N,NKPNT)
      REAL*8     RHOE(NNR1,*),SCR(*),CHRG(2),
     &           VSD,S,M,TEMPV,VGA
      INTEGER    LSCR,ISUB
      COMPLEX*16 PSI(*)
      LOGICAL    CONVTEST
C     ==--------------------------------------------------------------==
      CALL TISET('   VUPDATE',ISUB)
C     ==--------------------------------------------------------------==

      CALL AZZERO(CHRG,2)
      IF(TIVAN) CALL RNLSM(C0,N,PSI,SCR,LSCR,1,1,.FALSE.)
      CALL RHOOFR(C0,RHOE,PSI,SCR,LSCR,N)
#ifdef PARALLEL
      CALL GLOSUM(1,EKIN)
#endif
      CALL CDFT_W(RHOE,TAU0,CHRG)
      IF(PARENT)THEN
       VGRAD=CHRG(2)-CHRG(1)-CDFT_NC
       VGA=DABS(VGRAD)
       IF(VCCONU.NE.VCCON)THEN
         IF(VGA.GT.VCCONU) THEN
           RECONVV=.TRUE.
         ELSE IF(.NOT.RECONVV) THEN
          CONVTEST=.TRUE.
          FINALCHRG(1)=CHRG(1)
          FINALCHRG(2)=CHRG(2)
          WRITE(6,*) "VGRAD: ",VGRAD,"V: ",CDFT_V
         ENDIF
       ELSE
         RECONVV=.TRUE.
       ENDIF
       IF(VGA.LT.VCCON) THEN
        CONVTEST=.TRUE.
        FINALCHRG(1)=CHRG(1)
        FINALCHRG(2)=CHRG(2)
          WRITE(6,*) "VGRAD: ",VGRAD,"V: ",CDFT_V
        RECONVV=.FALSE.
       ELSE IF(RECONVV) THEN
C..Dekkers Optimisation Method with Newton up to first sign change
        IF(NEWGRAD.AND.CDFT_DEKK) THEN
          IF(DEKK)THEN
            IF(.NOT.(VGRAD*VGRADA.LT.0))THEN
              VGRADA=VGRADO
              OLDVA=OLDV
            ELSE
              IF(DABS(VGRADA).LT.VGA)THEN
                TEMPV=VGRADA
                VGRADA=VGRAD
                VGRAD=TEMPV
                TEMPV=OLDVA
                OLDVA=CDFT_V
                CDFT_V=TEMPV
              ENDIF
            ENDIF
          ELSE
            IF(VGRAD*VGRADO.LT.0)THEN
              DEKK=.TRUE.
              VGRADA=VGRADO
              OLDVA=OLDV
            ENDIF
          ENDIF
          VSD=(VGRAD-VGRADO)/(CDFT_V-OLDV)
          OLDV=CDFT_V
          IF(DEKK)THEN
            M=(OLDV+OLDVA)/2.0D0
            S=OLDV-VGRAD/VSD
            IF((VGRAD.LT.S.AND.S.LT.M).OR.(M.LT.S.AND.S.LT.VGRAD))THEN
              CDFT_V=S
            ELSE
              CDFT_V=M
            ENDIF
            VGRADO=VGRAD
          ELSE
            CDFT_V=CDFT_V-VGRAD/VSD
            VGRADO=VGRAD
          ENDIF
        ENDIF
C..Newtons Optimisation Method
        IF(NEWGRAD.AND.CDFT_NEWT) THEN
          VSD=(VGRAD-VGRADO)/(CDFT_V-OLDV)
          IF(MAXVMOV*DABS(VSD).LT.DABS(VGRAD)) THEN
            VSD=SIGN(VGSTEP,VSD)
          ENDIF
          OLDV=CDFT_V
          CDFT_V=CDFT_V-VGRAD/VSD
          VGRADO=VGRAD
        ENDIF
C..Gradient Optimisation Method stepsize 0.1 (First step every Method to get a second point for the finite difference)
        IF((.NOT.CDFT_DEKK.AND..NOT.CDFT_NEWT).OR..NOT.NEWGRAD)THEN
          OLDV=CDFT_V
          CDFT_V=CDFT_V+VGRAD/VGSTEP
          NEWGRAD=.TRUE.
          VGRADO=VGRAD
        ENDIF
        CONVTEST=.FALSE.
        IF(TCZONES)CALL UPDATE_ZONE(VGRAD)
        IF(PARENT) THEN
          WRITE(6,*) "OLD V: ",OLDV,"NEW V: ",CDFT_V
          WRITE(6,*) "VGRAD: ",VGRAD
          IF(NDON.EQ.1.AND.NACCR.EQ.1) THEN
            WRITE(6,'(1X,"DONOR (",A,"):",F10.6,T30,"ACCEPTOR (",A,"):"
     *              ,F10.6)') EL(IATYP(SPD(1))),CHRG(2),
     *              EL(IATYP(SPA(1))),CHRG(1)
          ELSE IF(NDON.EQ.0.AND.NACCR.EQ.1) THEN
            WRITE(6,*) "ACCEPTOR (",EL(IATYP(SPA(1))),"):",CHRG(1)
          ELSE
            WRITE(6,'(1X,"DONOR:",F10.6,T30,"ACCEPTOR:",F10.6)')
     *              CHRG(2),CHRG(1)
          ENDIF
        ENDIF
       ENDIF
      ENDIF
      CALL MY_BCAST(CDFT_V,1*8,SOURCE,ALLGRP)
      CALL MY_BCAST(CONVTEST,1*8/IRAT,SOURCE,ALLGRP)
      IF(TCZONES)CALL MY_BCAST(TOLOG,1*8,SOURCE,ALLGRP)
      CALL TIHALT('   VUPDATE',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE UPDATE_ZONE(GRAD)
C     ==--------------------------------------------------------------==
C     == Updates the Tolerance according to the new zone              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cdft.inc'
      REAL*8  GRAD,GRADA
      GRADA=DABS(GRAD)
      IF(GRADA<CZONES(3,1))THEN
        TOLOG=CZONES(3,2)
      ELSEIF(GRADA<CZONES(2,1))THEN
        TOLOG=CZONES(2,2)
      ELSE
        TOLOG=CZONES(1,2)
      ENDIF
      WRITE(6,*)'TOLOG=',TOLOG

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE CDFT_FINALIZE()
C     ==--------------------------------------------------------------==
C     == Write out everything and delete the memory                   ==
C     ==--------------------------------------------------------------==
      INCLUDE 'system.h'
      INCLUDE 'cdft.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'adat.inc' !<-- for EL
      INTEGER   IT
      IF(PARENT) THEN
        WRITE(6,'(1X,64("-"),/)')
        WRITE(6,'(1X,A)') '*** CDFT FINAL RESULTS ***'
        WRITE(6,'(2X,A,T54,D10.4)') "FINAL V: ",OLDV
        WRITE(6,'(2X,A,T54,E10.2E1)') "VGRAD: ",FINALCHRG(2)-
     *            FINALCHRG(1)-CDFT_NC
        WRITE(6,'(2X,"DONOR:",F10.4,T30,"ACCEPTOR:",F10.4)')
     *              FINALCHRG(2),FINALCHRG(1)
        IF(NDON.GT.0) WRITE(6,'(2X,"DONOR ATOMS")')
        DO IT=1,NDON
          WRITE(6,'(3X,I4,"(",A,")")')CDFT_D(IT),EL(IATYP(SPD(IT)))
        ENDDO
        WRITE(6,'(2X,"ACCEPTOR ATOMS")')
        DO IT=1,NACCR
          WRITE(6,'(3X,I4,"(",A,")")')CDFT_A(IT),EL(IATYP(SPA(IT)))
        ENDDO
        WRITE(6,'(1X,64("-"),/)')

      ENDIF

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE WRITE_W(WEIGHT,SUFF)
C     ==--------------------------------------------------------------==
C     == Writes the weight to disk                                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'qspl.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'coor.inc'
      INCLUDE 'cdft.inc'
      INCLUDE 'cppt.inc'
C     Variables
      REAL*8    DA1(3,3)
      REAL*8    WEIGHT(KR1,KR2,*)
      REAL*8    WEIGHT2(*)
      REAL*8    WBUFF(3*NNR1)
      COMPLEX*16 WTEMP(NNR1),W2(NNR1)
      REAL*8    WFULL(NR1S,NR2S,NR3S)
      POINTER   (IP_WFULL,WFULL)
      POINTER   (IP_WEIGHT2,WEIGHT2)
      REAL*8    WS
      INTEGER   I,J,K,ZSLICE,IPP,IP,MSGID,MSGLEN,NBYTES
      CHARACTER SUFF*(*)
      INTEGER   ML,IL
      IF(CDFT_WF)THEN ! Write out the full weight using the cpmd DENSITY functionality
        CALL MEMORY(IP_WEIGHT2,NNR1,'WEIGHT2')
        CALL DCOPY(NNR1,WEIGHT,1,WEIGHT2,1)
!$OMP parallel do private(IL)
        DO IL=1,NNR1
          WTEMP(IL)=DCMPLX(WEIGHT2(IL),0.D0)
        ENDDO
!$OMP end parallel do
        CALL FWFFT(WTEMP)
        CALL AZZERO(W2,2*NHG)
!$OMP parallel do private(IL)
        DO IL=1,NHG
          W2(IL)=WTEMP(NZH(IL))
        ENDDO
!$OMP end parallel do

        IF(PARENT) THEN
          WRITE(6,'(1X,64("-"),/)')
          WRITE(6,'(2X,"WRITING OUT FULL WEIGHT")')
          WRITE(6,'(2X,"to ",A)')"WEIGHT-"//SUFF
        ENDIF
        CALL DENSTO(W2,TAU0,"WEIGHT-"//SUFF)
        IF(PARENT) WRITE(6,'(1X,64("-"),/)')
      ELSE ! Write only a given slice of the weight
C     ==PARA----------------------------------------------------------==
      IF(PARENT)THEN
        CALL MEMORY(IP_WFULL,KR1S*KR2S*KR3S,'WFULL')
!$OMP parallel do private(I,J,K) shared(ME)
        DO I=1,KR1
          DO J=1,KR2
            DO K=1,KR3
              WFULL(NRXPL(ME,1)+I,J,K)=WEIGHT(I,J,K)
            ENDDO
          ENDDO
        ENDDO
!$OMP end parallel do
      ENDIF
      IF(NPROC.GT.1)THEN
        IF(PARENT)THEN
          DO IPP=1,NPROC
            IP=PGROUP(IPP)
            ML=SPARM(5,IP)
            ML=ML+MOD(ML+1,2)
            MSGLEN=(ML*KR2S*KR3S)*8
            IF(IP.NE.ME) THEN
              CALL MY_BRECV(WBUFF,MSGLEN,IP,IP,NBYTES)
              DO I=1,SPARM(5,IP)
                DO J=1,KR2S
                  DO K=1,KR3S
                    WFULL(NRXPL(IP,1)+I,J,K)=
     .                    WBUFF(I+(J-1)*ML+(K-1)*ML*KR2)
                  ENDDO
                ENDDO
              ENDDO
            ENDIF
          ENDDO
        ELSE
          MSGLEN=(NNR1)*8
          CALL MY_BSEND(WEIGHT,MSGLEN,SOURCE,ME)
        ENDIF
      ENDIF
C     ==PARA----------------------------------------------------------==
      IF(PARENT)THEN
        DO I=1,3
          DA1(I,1)=A1(I)*0.529177249D0/DBLE(NR1S)
          DA1(I,2)=A2(I)*0.529177249D0/DBLE(NR2S)
          DA1(I,3)=A3(I)*0.529177249D0/DBLE(NR3S)
        ENDDO
        IF(WSLICE.LT.0.0d0)THEN
          WS=TAU0(3,SNA(1),SPA(1))/A3(3)
        ELSE
          WS=WSLICE
        ENDIF
        WRITE(6,'(1X,64("-"),/)')
        WRITE(6,'(2X,"WRITING OUT Weight SLICE AT z=",F8.4)')WS*A3(3)
        WRITE(6,'(2X,"to ",A)')"WEIGHT-"//SUFF//".dat"
        WRITE(6,'(1X,64("-"),/)')
        ZSLICE=WS*KR3S
        OPEN(72,FILE="WEIGHT-"//SUFF//".dat",STATUS='UNKNOWN')
        DO I=1,KR1S,WSTEP
          DO J=1,KR2S,WSTEP
             WRITE(72,*)(I)*DA1(1,1),(J)*DA1(2,2),WFULL(I,J,ZSLICE)
          ENDDO
          WRITE(72,*)
        ENDDO
        CLOSE(72)
      ENDIF
      ENDIF !Slice or no slice
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE WCDFT_RESTART()
C     ==--------------------------------------------------------------==
C     == Writes the CDFT_RESTART file                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cdft.inc'
      LOGICAL  LOGBUFF
      REAL*8   OLDV2

      OPEN(73,FILE="CDFT_RESTART",STATUS='UNKNOWN')
      REWIND(UNIT=73)
      IF(THDA.AND..NOT.HDAFIRST)THEN
        READ(73,*)LOGBUFF
        READ(73,*)OLDV2
        REWIND(UNIT=73)
        WRITE(73,*).TRUE. !There is a second record to be written
        WRITE(73,*) OLDV2
        WRITE(73,*) CDFT_V
      ELSE
        WRITE(73,*).FALSE.!There is no second record to be read
        WRITE(73,*) CDFT_V
      ENDIF
      CLOSE(73)

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE RCDFT_RESTART()
C     ==--------------------------------------------------------------==
C     == Writes the CDFT_RESTART file                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'irat.inc'
      INCLUDE 'cdft.inc'
      LOGICAL  LOGBUFF

      IF(PARENT)THEN
        WRITE(6,*)"READING CDFT_V FROM CDFT_RESTART FILE"
        OPEN(73,FILE="CDFT_RESTART",STATUS='OLD')
        READ(73,*) LOGBUFF
        IF(.NOT.THDA)THEN
          READ(73,*) CDFT_V
        ELSE
          IF(HDAFIRST)THEN
            HDARESB=LOGBUFF
            READ(73,*) CDFT_V
            IF(LOGBUFF)THEN
              READ(73,*) VBUFF
            ELSE
              WRITE(6,*) "NO SECOND STATE INFORMATION FOUND!"
            ENDIF
          ELSE
            CDFT_V=VBUFF
          ENDIF
        ENDIF
        CLOSE(73)
      ENDIF
      CALL MY_BCAST(CDFT_V,8,SOURCE,ALLGRP)
      CALL MY_BCAST(HDAFIRST,2*8/IRAT,SOURCE,ALLGRP)

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE CDFT_PRI()
C     ==--------------------------------------------------------------==
C     == Writes some info to term                                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cdft.inc'
      INCLUDE 'adat.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cnst.inc'
      INTEGER  IT
        WRITE(6,*)
        WRITE(6,'(64("-"))')
        IF(.NOT.TSPINC)THEN
          WRITE(6,'(A,/)') 'CHARGE CONSTRAINED DFT'
        ELSE
          WRITE(6,'(A,/)') 'SPIN DENSITY CONSTRAINED DFT'
        ENDIF
        IF(CDFT_NEWT)THEN
          WRITE(6,'(2X,A,/)')'NEWTON OPTIMISER'
        ELSEIF(CDFT_DEKK) THEN
          WRITE(6,'(2X,A,/)')'DEKKER OPTIMISER'
        ELSE
          WRITE(6,'(2X,A,/)')'GRADIENT OPTIMISER'
        ENDIF
        IF(TPCGFIC) THEN
          WRITE(6,'(2X,A,/)')'SWITCH TO PCG MINIMIZE AT FIRST CDFT STEP'
        ENDIF
        IF(TCZONES)THEN
          WRITE(6,'(2X,A,/)')'WITH CONVERGENCE ZONES'
          WRITE(6,'(3X,E10.2,T20,A1,T28,E10.2,T45,A1,T53,E10.2)')
     &           CZONES(1,2),'|',CZONES(2,2),'|',CZONES(3,2)
          WRITE(6,'(3X,T14,E10.2E1,T39,E10.2E1)')CZONES(2,1),CZONES(3,1)
        ENDIF
        IF(MD)THEN
          IF(CCOR)THEN
            WRITE(6,'(2X,A)')'MD WITH CUTOFF CORRECTION'
          ELSE
            WRITE(6,'(2X,A)')'MD WITHOUT CUTOFF CORRECTION'
          ENDIF
          IF(TPRED)THEN
            WRITE(6,'(2X,A)')'AND V PREDICTION'
            WRITE(6,'(2X,A,T54,I10)')'WITH EXTRAPOLATION ORDER k =',
     *       PREDORD
          ELSE
            WRITE(6,'(/)')
          ENDIF
        ENDIF
        IF(GEOPT)THEN
          IF(CCOR)THEN
            WRITE(6,'(2X,A,/)')'GEOPT WITH CUTOFF CORRECTION'
          ELSE
            WRITE(6,'(2X,A,/)')'GEOPT WITHOUT CUTOFF CORRECTION'
          ENDIF
        ENDIF
        WRITE(6,'(1X,A)') 'PARAMETERS:'
        IF(NDON.GT.0)WRITE(6,'(2X,A,T54,I10)')'DONOR ATOM(S):',NDON
        DO IT=1,NDON
          WRITE(6,'(3X,T50,I4,T60,A)')CDFT_D(IT),EL(IATYP(SPD(IT)))
        ENDDO
        WRITE(6,'(2X,A,T54,I10)')'ACCEPTOR ATOM(S):',NACCR
        DO IT=1,NACCR
          WRITE(6,'(3X,T50,I4,T60,A)')CDFT_A(IT),EL(IATYP(SPA(IT)))
        ENDDO
        IF(NDON.GT.0)THEN
          WRITE(6,'(2X,A,T54,F10.3,/)')'CHARGE DIFFERENCE:',CDFT_NC
        ELSE
          WRITE(6,'(2X,A,T54,F10.3,/)')'CHARGE OF ACCEPTOR:',-CDFT_NC
        ENDIF
        WRITE(6,'(2X,A,T54,I10)')'MAXIMUM NUMBER OF OPTIMISATION STEPS:'
     &           ,CDFT_END
        WRITE(6,'(2X,A,T54,F10.3)')'INITIAL LAGRANGE MULTIPLIER:',CDFT_V
        WRITE(6,'(2X,A,T44,E10.2E1,E10.2E1)')'CONSTRAINT CONVERGENCE:',
     *            VCCON,VCCONU
        IF(TWGAUSS)THEN
          WRITE(6,*)
          WRITE(6,'(2X,A)')"GAUSSIAN WEIGHT"
          WRITE(6,'(2X,A)')"SIGMA:"
          DO IT=1,NSP
            IF(BOHR)THEN
              WRITE(6,'(3X,T35,I4,T45,A,T54,F10.3)')IT,EL(IATYP(IT)),
     *            WG_SIGMA(IT)
            ELSE
              WRITE(6,'(3X,T35,I4,T45,A,T54,F10.3)')IT,EL(IATYP(IT)),
     *            WG_SIGMA(IT)/FBOHR
            ENDIF
          ENDDO
        ENDIF
        WRITE(6,*)
        IF(WCUT.GT.0.0D0)THEN
          IF(BOHR)THEN
            WRITE(6,'(2X,A,T54,F10.3)')"WEIGHT CUTOFF",WCUT
          ELSE
            WRITE(6,'(2X,A,T54,F10.3)')"WEIGHT CUTOFF",WCUT/FBOHR
          ENDIF
        ELSE
          WRITE(6,'(2X,A)')"VARIABLE CUTOFF"
         DO IT=1,NSP
            IF(BOHR)THEN
              WRITE(6,'(3X,T35,I4,T45,A,T54,F10.3)')IT,EL(IATYP(IT)),
     *            CDFT_RC(IT)
            ELSE
              WRITE(6,'(3X,T35,I4,T45,A,T54,F10.3)')IT,EL(IATYP(IT)),
     *            CDFT_RC(IT)/FBOHR
            ENDIF
         ENDDO

        ENDIF
        WRITE(6,'(64("-"))')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE DERIV_INIT()
C     ==--------------------------------------------------------------==
C     == Setup the weight derivatives                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'qspl.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'cdft.inc'
      DIMENSION ATRG(MMAXX,*),ATWFR(MMAXX,M1SHLX,*),WDER(MMAXX,MAXSP,*)
      REAL*8    DATOM(NSPLPO,2),ARHO(MMAXX),WORK(MMAXX)
      INTEGER   IS,ISH,IR,IERR,MMAX
      REAL*8    FPION,CURVD,DX,DY,DZ

      CALL MEMORY(IP_WDER,MMAXX*MAXSP*3,'WDER')
      DX=A1(1)/DBLE(NR1S)
      DY=A2(2)/DBLE(NR2S)
      DZ=A3(3)/DBLE(NR3S)
      CDFT_SHELL=DSQRT(DX*DX+DY*DY+DZ*DZ)

      DO IS=1,NSP
C Set everything up
        CALL AZZERO(ARHO,MMAXX)
        CALL AZZERO(DATOM,NSPLPO*2)
        CALL AZZERO(WORK,MMAXX)
        MMAX=MESHAT(IS)
        DO ISH=1,NSHELL(IS)
!$OMP parallel do private(IR) shared(IS,ISH)
          DO IR=1,MMAX
            ARHO(IR)=ARHO(IR)+
     *             OCUN(ISH,IS)*(ATWFR(IR,ISH,IS)/ATRG(IR,IS))**2
          ENDDO
!$OMP end parallel do
        ENDDO
        MMAX=MIN(MMAX,NSPLPO)
        CDFT_MMAX(IS)=MMAX
        FPION=1.D0/FPI
!$OMP parallel do private(IR)
        DO IR=1,MMAX
          DATOM(IR,1)=ARHO(IR)*FPION
        ENDDO
!$OMP end parallel do
        CALL CURV1(MMAX,ATRG(1,IS),DATOM(1,1),0.D0,0.D0,3,DATOM(1,2),
     *           WORK,0.D0,IERR)
C Construct the first derivative for the cutoff correction
!$OMP parallel do private(IR) shared(IS)
        DO IR=1,MMAX
          WDER(IR,IS,1)=ATRG(IR,IS)
          WDER(IR,IS,2)=CURVD(ATRG(IR,IS),MMAX,ATRG(1,IS),DATOM(1,1),
     *             DATOM(1,2),0.D0)
        ENDDO
!$OMP end parallel do
        CALL AZZERO(WORK,MMAXX)
        CALL CURV1(MMAX,WDER(1,IS,1),WDER(1,IS,2),0.D0,0.D0,3,
     *           WDER(1,IS,3),WORK,0.D0,IERR)
      ENDDO

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE CDFT_FORCES(FION,TAU0,RHO)
C     ==--------------------------------------------------------------==
C     == Calculate the Bias Forces                                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'qspl.inc'
      INCLUDE 'cdft.inc'
      REAL*8     DATOM(NSPLPO,2),ARHO(MMAXX),WORK(MMAXX),FAT(3),FB(3)
      POINTER   (IP_ARHO,ARHO),(IP_WORK,WORK)
      REAL*8     TAU0(3,NAX,NSX),FION(3,NAX,NSX),RHO(NNR1),WN(NNR1),
     *           RCUT,MULT
      INTEGER    ISA,IA,IS,DONAC,IT,MMAX,ISUB
      DIMENSION  ATRG(MMAXX,*),WDER(MMAXX,MAXSP,*)

      CALL TISET('CDFT-FORCE',ISUB)
      CALL MEMORY(IP_ARHO,MMAXX,'ARHO')
      CALL MEMORY(IP_WORK,MMAXX,'WORK')

C Get Normalisation
      CALL AZZERO(WN,NNR1)
      DO IS=1,NSP
      CALL ATDENS_NEW2(IS,DATOM,MMAX,RCUT,ARHO,WORK)
      DO IA=1,NA(IS)
        CALL PUTRHO(WN,TAU0(1,IA,IS),ATRG(1,IS),DATOM,
     *                MMAX,RCUT,1.D0)
        ENDDO
      ENDDO
C Loop over all Atoms
      MULT=-OMEGA*CDFT_V/(NR1S*NR2S*NR3S)
      DO ISA=1,NAT
        IA=IATPT(1,ISA)
        IS=IATPT(2,ISA)
C Find out if its in Donor or Acceptor
        DONAC=0
        DO IT=1,NACCR
          IF(ISA.EQ.CDFT_A(IT))THEN
            DONAC=1
            GOTO 101
          ENDIF
        ENDDO
        DO IT=1,NDON
          IF(ISA.EQ.CDFT_D(IT))THEN
            DONAC=2
            GOTO 101
          ENDIF
        ENDDO
  101   CONTINUE
        CALL CDFT_FINT(RHO,WN,WDIFF,TAU0(1,IA,IS),CDFT_MMAX(IS),FAT,
     *              FB,DONAC,IS,CDFT_RC(IS))
!!      DO IT=1,3
!!        FION(IT,IA,IS)=FION(IT,IA,IS)-MULT*FAT(IT)-CDFT_V*FB(IT)
!!      ENDDO
        FION(1,IA,IS)=FION(1,IA,IS)-MULT*FAT(1)-CDFT_V*FB(1)
        FION(2,IA,IS)=FION(2,IA,IS)-MULT*FAT(2)-CDFT_V*FB(2)
        FION(3,IA,IS)=FION(3,IA,IS)-MULT*FAT(3)-CDFT_V*FB(3)
      ENDDO
      CALL FREEM(IP_ARHO)
      CALL FREEM(IP_WORK)
      
      CALL TIHALT('CDFT-FORCE',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE CDFT_FINT(RHOE,PSI,WEIGHT,TAU,MMAX,FAT,FB,DONAC,SP
     *            ,RCUT)
C     ==--------------------------------------------------------------==
C     == Computes the integral for the Bias Forces                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'qspl.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'cdft.inc'
C     Arguments
      INTEGER   MMAX
      REAL*8    RHOE(KR1,KR2S,*),PSI(KR1,KR2S,*),WEIGHT(KR1,KR2S,*),
     *          TAU(3),FAT(3),FB(3)
C     Variables
      REAL*8    SC1(3),SC2(3),DA1(3,3),XPTAU(3),X1,Y1,Z1,X,Y,Z,
     &          R,RCUT,CURV2,WBUF,WBUF2,WN,SURFE,RCS
      INTEGER   NPTAU(3),MR(3),I,IORGX,IORGY,IORGZ,IGRID,IX,II,I1,J,
     &          JJ,J1,K,KK,K1,DONAC,SP
      DIMENSION WDER(MMAXX,MAXSP,*)
C     ==--------------------------------------------------------------==
C..Atom position
      X1=TAU(1)
      Y1=TAU(2)
      Z1=TAU(3)
      CALL PBC(X1,Y1,Z1,SC1(1),SC1(2),SC1(3),1,APBC,IBRAV)
      CALL DGEMV('T',3,3,1.D0,HTM1,3,SC1,1,0.D0,SC2,1)
      IF(SC2(1).LT.0.D0) SC2(1)=SC2(1)+1.0D0
      IF(SC2(2).LT.0.D0) SC2(2)=SC2(2)+1.0D0
      IF(SC2(3).LT.0.D0) SC2(3)=SC2(3)+1.0D0
      NPTAU(1)=NINT(SC2(1)*NR1S)+1
      NPTAU(2)=NINT(SC2(2)*NR2S)+1
      NPTAU(3)=NINT(SC2(3)*NR3S)+1
      SC1(1)=SC2(1)-DBLE(NPTAU(1)-1)/DBLE(NR1S)
      SC1(2)=SC2(2)-DBLE(NPTAU(2)-1)/DBLE(NR2S)
      SC1(3)=SC2(3)-DBLE(NPTAU(3)-1)/DBLE(NR3S)
      IF(NPTAU(1).GT.NR1S) NPTAU(1)=1
      IF(NPTAU(2).GT.NR2S) NPTAU(2)=1
      IF(NPTAU(3).GT.NR3S) NPTAU(3)=1
      CALL DGEMV('T',3,3,1.D0,HT,3,SC1,1,0.D0,XPTAU,1)
      DO I=1,3
        DA1(I,1)=A1(I)/DBLE(NR1S)
        DA1(I,2)=A2(I)/DBLE(NR2S)
        DA1(I,3)=A3(I)/DBLE(NR3S)
      ENDDO
C..Interaction region
      SC1(1)=RCUT
      SC1(2)=RCUT
      SC1(3)=RCUT
      CALL DGEMV('T',3,3,1.D0,HTM1,3,SC1,1,0.D0,SC2,1)
      MR(1)=NINT(SC2(1)*NR1S)+1
      MR(2)=NINT(SC2(2)*NR2S)+1
      MR(3)=NINT(SC2(3)*NR3S)+1
      MR(1)=MIN(MR(1),NR1S/2-1)
      MR(2)=MIN(MR(2),NR2S/2-1)
      MR(3)=MIN(MR(3),NR3S/2-1)
C..Origin of density grid
      IORGX=NPTAU(1)-MR(1)
      IORGY=NPTAU(2)-MR(2)
      IORGZ=NPTAU(3)-MR(3)
      IF(IORGX.LT.1) IORGX=IORGX+NR1S
      IF(IORGY.LT.1) IORGY=IORGY+NR2S
      IF(IORGZ.LT.1) IORGZ=IORGZ+NR3S
C..loop over all grid points
      DO I=1,3
        FAT(I)=0.0d0
        FB(I)=0.0d00
      ENDDO
      RCS=RCUT-CDFT_SHELL
      DO I=0,2*MR(1)
        IX=IORGX+I
        IF(IX.GT.NR1S) IX=IX-NR1S
        IF(IX.GE.NRXPL(MEPOS,1).AND.IX.LE.NRXPL(MEPOS,2)) THEN
          II=IX-NRXPL(MEPOS,1)+1
          I1=MR(1)-I
          DO J=0,2*MR(2)
            JJ=IORGY+J
            IF(JJ.GT.NR2S) JJ=JJ-NR2S
            J1=MR(2)-J
            DO K=0,2*MR(3)
              KK=IORGZ+K
              IF(KK.GT.NR3S) KK=KK-NR3S
              K1=MR(3)-K
              CALL POSITX(X,Y,Z,I1,J1,K1,DA1,XPTAU(1))
              IF(TCLUST)THEN
                IF(X1-X.GE.A1(1).OR.Y1-Y.GE.A2(2).OR.Z1-Z.GE.A3(3)
     *           .OR.X1-X.LE.0.OR.Y1-Y.LE.0.OR.Z1-Z.LE.0)THEN
                    GOTO 201
                ENDIF
              ENDIF
              R=DSQRT(X*X+Y*Y+Z*Z)
              WN=PSI(II,JJ,KK)
              IF(R.LT.RCUT.AND.R.GT.1.0d-10)THEN
                IF(DONAC.EQ.0)THEN
                  WBUF=WEIGHT(II,JJ,KK)
                ELSEIF(DONAC.EQ.1)THEN
                  WBUF=WEIGHT(II,JJ,KK)-1 ! sign change due to optimisation ...
                ELSEIF(DONAC.EQ.2)THEN
                  WBUF=1+WEIGHT(II,JJ,KK) ! sign change due to optimisation ...
                ENDIF
                WBUF2=CURV2(R,MMAX,WDER(1,SP,1),WDER(1,SP,2),
     *               WDER(1,SP,3),0.D0)*WBUF*RHOE(II,JJ,KK)/WN
                FAT(1)=FAT(1)+WBUF2*X/R
                FAT(2)=FAT(2)+WBUF2*Y/R
                FAT(3)=FAT(3)+WBUF2*Z/R
                IF(CCOR.AND.R.GE.RCS) THEN
                  WBUF2=RHOE(II,JJ,KK)*WBUF*SURFE(X,Y,Z,RCUT,DA1)/WN
                  FB(1)=FB(1)+WBUF2*X
                  FB(2)=FB(2)+WBUF2*Y
                  FB(3)=FB(3)+WBUF2*Z
                ENDIF
              ENDIF
  201         CONTINUE
            ENDDO
          ENDDO
        ENDIF
      ENDDO
!!    DO I=1,3
!!      FB(I)=RHOCUT(SP)*RCUT*FB(I)
!!    ENDDO
      FB(1)=RHOCUT(SP)*RCUT*FB(1)
      FB(2)=RHOCUT(SP)*RCUT*FB(2)
      FB(3)=RHOCUT(SP)*RCUT*FB(3)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      FUNCTION SURFE(X,Y,Z,R,DX)
C     ==--------------------------------------------------------------==
C     == Calculate the Surface Element                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      REAL*8  SURFE
      REAL*8  DX(3,3),X,Y,Z,R
      SURFE=SIGN(1.0D0,Z)
     .     *(Y*DX(1,1)*DX(3,3)-X*DX(2,2)*DX(3,3))/(R**3-R*Z**2)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE ATDENS_NEW2(IS,DATOM,MMAX,RCUT,ARHO,WORK)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'qspl.inc'
      INCLUDE 'cdft.inc'
C     Arguments
      INTEGER   IS,MMAX
      REAL*8    DATOM(NSPLPO,2),ARHO(MMAXX),WORK(MMAXX),RCUT
C     Variables
      DIMENSION ATRG(MMAXX,*),ATWFR(MMAXX,M1SHLX,*)
      REAL*8    FPION,NORM,ISIG,CURV2
      INTEGER   ISH,IR,IERR,OCSPEC
C     ==--------------------------------------------------------------==
      CALL AZZERO(ARHO,MMAXX)
      MMAX=MESHAT(IS)
C..Sum density
      ISIG=WG_SIGMA(IS)*DSQRT(2.0D0)
      NORM=1.0D0/(ISIG*DSQRT(PI))

      IF(TWGAUSS)THEN
        OCSPEC=0
!$OMP  parallel do private(ISH) 
!$OMP+ reduction(+:OCSPEC)
        DO ISH=1,NSHELL(IS)
          OCSPEC=OCSPEC+OCUN(ISH,IS)
        ENDDO
!$OMP  end parallel do
!$OMP  parallel do private(IR) 
        DO IR=1,MMAX
            ARHO(IR)=OCSPEC*NORM*DEXP(-(ATRG(IR,IS)/ISIG)**2)
          ENDDO
!$OMP  end parallel do
      ELSE
        DO ISH=1,NSHELL(IS)
!$OMP  parallel do private(IR) shared(IS,ISH)
          DO IR=1,MMAX
            ARHO(IR)=ARHO(IR)+
     *             OCUN(ISH,IS)*(ATWFR(IR,ISH,IS)/ATRG(IR,IS))**2
          ENDDO
!$OMP  end parallel do
        ENDDO
      ENDIF
      MMAX=MIN(MMAX,NSPLPO)
      FPION=1.D0/FPI
!$OMP  parallel do private(IR)
      DO IR=1,MMAX
        DATOM(IR,1)=ARHO(IR)*FPION
      ENDDO
!$OMP  end parallel do
      CALL CURV1(MMAX,ATRG(1,IS),DATOM(1,1),0.D0,0.D0,3,DATOM(1,2),
     *           WORK,0.D0,IERR)
      IF(WCUT.GT.0)THEN
        RCUT=WCUT
        RHOCUT(IS)=CURV2(RCUT,MMAX,ATRG(1,IS),DATOM(1,1),
     *             DATOM(1,2),0.D0)
      ELSE
        DO IR=1,MMAX
          IF(DATOM(IR,1).LT.1.D-6.AND.ATRG(IR,IS).GT.1.D0) THEN
            RCUT=ATRG(IR,IS)
            RHOCUT(IS)=DATOM(IR,1)
            GOTO 100
          ENDIF
        ENDDO
        RCUT=ATRG(MMAX,IS)
        WRITE(6,*)"REFERENCE DENSITY DECAYS TOO SLOW,
     *     Rc SET TO MAXIMUM!"
  100   CONTINUE
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE RINGBUFF(VIN,FULL)
C     ==--------------------------------------------------------------==
      INCLUDE 'system.h'
      INCLUDE 'cdft.inc'
C     Arguments
      REAL*8    VIN ! input V value
      LOGICAL   FULL !returns true if the number of elements in the buffer=PREDORD
C     Variables
      INTEGER    ELC ! number of elements in the buffer
      INTEGER    I
      COMMON/RBUFF/ELC

      FULL=.FALSE.

      IF(ELC.LT.PREDORD+1)THEN
        VPREDBUF(ELC+1)=VIN
        ELC=ELC+1
      ELSE
        DO I=1,PREDORD
          VPREDBUF(I)=VPREDBUF(I+1)
        ENDDO
        VPREDBUF(PREDORD+1)=VIN
      ENDIF

      IF(ELC.EQ.PREDORD+1)FULL=.TRUE.
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE V_PRED()
C     ==--------------------------------------------------------------==
      INCLUDE 'system.h'
      INCLUDE 'cdft.inc'
C     Variables
      REAL*8    V ! V value
      LOGICAL   FULL
      INTEGER   I,J
      REAL*8    l
      V=0.0D0

      IF(PARENT)THEN
        CALL RINGBUFF(CDFT_V,FULL)
        IF(FULL) THEN
          DO J=0,PREDORD
            l=1.0D0
            DO I=0,PREDORD
              IF(I.NE.J)THEN
               l=l*DBLE(PREDORD+1-I)/DBLE(J-I)
              ENDIF
            ENDDO
            V=V+l*VPREDBUF(J+1)
          ENDDO
          CDFT_V=V
          WRITE(6,*)"PREDICTED V TO BE: ",V
        ELSE
          WRITE(6,*)"TOO FEW VALUES FOR PREDICTION,"
          WRITE(6,*)"USING OLD V!"
        ENDIF
      ENDIF
      CALL MY_BCAST(CDFT_V,1*8,SOURCE,ALLGRP)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
