C     ==================================================================
      SUBROUTINE CHAIN_DR(NRES1,NAA,NRES2,NH,EX1_H,EX2_H,R0_H,
     &              EX1_A,EX2_A,R0_A,IATOM,TSCR,AN,
     &              LSK,PTOT) 
C     ==--------------------------------------------------------------==
C     == Calculates the order parameters that estimates the amount of ==
C     == hydrogen bond chains in the system (PTOT) and the derivatives==
C     == with respect to the positions of the ions involved (AN)      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE

      include 'system.h'
      include 'ions.inc'
      include 'cnst_dyn.inc'
      include 'metr.inc'      ! HT
      include 'mm_input.inc'  ! qmmm
      include 'mm_dim.inc'    ! box_au

C     Arguments
      INTEGER  NRES1,NRES2,NAA,NH,EX1_H,EX2_H,EX1_A,EX2_A
      REAL*8   R0_H,R0_A
      INTEGER  LSK(3,*),IATOM(*)
      REAL*8   TSCR(3,*),AN(*),PTOT

C     Variables
      REAL*8 POS(3,NAT),CELL_CH(9),force(3,NAT)
      POINTER (IP_POS,POS),(IP_FORCE,FORCE)
      INTEGER  IX,I,NOX,NATOM,ISUB,IA,IIA,L1,L2,L3
      INTEGER  IFIRST
      DATA     IFIRST /0/
      SAVE     IFIRST

C     ==--------------------------------------------------------------==
      CALL TISET('  CHAIN_DR',ISUB)

C     Allocation of memory
      if(IFIRST .EQ. 0) THEN
        CALL MEMORY(IP_POS,3*NAT,'POS')
        CALL AZZERO(POS,3*NAT)
        CALL MEMORY(IP_FORCE,3*NAT,'FORCE')
        CALL AZZERO(FORCE,3*NAT)
        IFIRST = 1
      ENDIF

C     Cell parameters for PBC
      if(qmmm)then
        i=0
        do ix=1,3
          cell_ch(ix+i)=box_au(ix)
          i = i+3
        enddo
      else
        do i=1,9
          cell_ch(i)=HT(i,1)
        enddo
      endif

C     Initialization of the positions array
      do ia =1,NRES1+NAA+NRES2+NH
        IIA=IATOM(ia)
        do ix=1,3
          pos(ix,IA)=TSCR(ix,IIA)
        enddo
      enddo 

      
      NOx  = NRES1+NAA
      natom=NOx+NRES2+NH

C     Calculation of H-bond chains (length from 1 to 5)
      call chain_int(natom,NOx,NRES1,NRES2,EX1_H,EX2_H,R0_H,
     &              EX1_A,EX2_A,R0_A,pos,force,cell_ch,PTOT)

C     Derivatives wrt to the ions involved 
      DO ia = 1,NATOM
        iia = IATOM(ia)
        L1 = LSK(1,IIA)
        L2 = LSK(2,IIA)
        L3 = LSK(3,IIA)
        IF(L1 .NE. 0) AN(L1) = force(1,IA) 
        IF(L2 .NE. 0) AN(L2) = force(2,IA) 
        IF(L3 .NE. 0) AN(L3) = force(3,IA) 

      ENDDO

      CALL TIHALT('  CHAIN_DR',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE chain_int(natom,NOx,nres1,nres2,nh,mh,rh,no,mo,ro,
     &                     pos,force,hpass,ptot)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE

      INCLUDE 'system.h'
      INCLUDE 'cnst_dyn.inc'
      INCLUDE 'irat.inc'

C     Arguments
      INTEGER   natom,NOx,nres1,nres2
      INTEGER   nh,mh,no,mo
      REAL*8    rh,ro
      REAL*8    pos(3,natom),force(3,natom),hpass(3,3)

C     Variables
      integer i,j,k,l,ii,jj,is
      integer kf,kk,ix
      integer nomax,nhmax       ! maximum number of neighbours
      parameter (nomax=15)
      parameter (nhmax=5)
  
      real*8 h                  ! metric tensor
      real*8 hinv               ! inverse metric tensor
      
      common/metr_my/h(3,3),hinv(3,3)

      integer wlist(NOx+nres2),hlist(NOx+nres2,NOx+nres2)
      integer nwlist(nhmax,NOx+nres2)
      integer nhlist(nhmax,NOx+nres2,NOx+nres2)

      pointer (ip_wlist,wlist),(ip_hlist,hlist)
      pointer (ip_nwlist,nwlist),(ip_nhlist,nhlist)

      common/savec/ ip_wlist, ip_hlist, ip_nwlist, ip_nhlist

      real*8 r(4,natom,natom)

      integer nc,no2
      parameter (nc=5)
C AK 2005/03/26: not used.
C      integer ch(nc+2,nc)

      real*8 dij(3),sij(3),rrij(3),Rij ! variables for neighbours list
      real*8 d0,d,dn,dm,num,den,func,func1,func2,dfunc,dfunc1
      real*8 SW(nomax,NOx)                 ! switching function
      real*8 SWF(NOx+nres2,nres2)          ! final switching function
      real*8 DSW(9,nhmax,nomax,NOx)        ! derivatve of switching function
      real*8 DSWF(9,nhmax,NOx+nres2,nres2) ! derivative of final switching function
      real*8 F(3,natom,nc)                 ! temporary force array
      pointer (IP_SW,SW),(IP_SWF,SWF),(IP_DSW,DSW)
      pointer (IP_DSWF,DSWF),(IP_F,F)

      integer p1,p2,p3,p4,p5
      integer a0,aa1,aa2,aa3,a4,a5

      real*8 OrderParameter(nc)
      real*8 PT1,PT2,PT3,PT4,PT5           ! order parameters
      real*8 MaxP(nc)
      real*8 Ptmp1,Ptmp2,Ptmp3,Ptmp4,Ptmp5 ! temporary order parameters
      real*8 PTot
      real*8 tmp,eve,tmp1,tmp2,tmp3,tmp4,tmp5,tmpf
      real*8 a,b,cfact

      real*8 small
      parameter (small=1.0d-34)

      integer it
      save it
      data it /0/
C     ==--------------------------------------------------------------==
      it=it+1
      if (it.eq.1) then
        no2 = NOx+nres2
        CALL MEMORY(ip_wlist,(no2/IRAT)+1,'wlist')
        CALL MEMORY(ip_hlist,(2*no2/IRAT)+1,'hlist')
        CALL MEMORY(ip_nwlist,(nomax+no2)/IRAT +1,'nwlist')
        CALL MEMORY(ip_nhlist,(nhmax+no2+no2)/IRAT+1,'nhlist')
      endif

      CALL MEMORY(IP_SW,nomax+NOx,'SW')
      CALL MEMORY(IP_SWF,NOx+nres2+nres2,'SWF')
      CALL MEMORY(IP_DSW,9+nhmax+nomax+NOx,'DSW')
      CALL MEMORY(IP_DSWF,9+nhmax+NOx+nres2+nres2,'DSWF')
      CALL MEMORY(IP_F,3+natom+nc,'F')
C     ==--------------------------------------------------------------==

      do i=1,3
        h(1,i)=hpass(1,i)
        h(2,i)=hpass(2,i)
        h(3,i)=hpass(3,i)
      enddo
      call inv3(h,hinv)

C     ==--------------------------------------------------------------==
C     Neighbours list for the center of mass of each Ox pair

      tmp=0
      if (mod(it,LUPD).eq.1)then
        call iazzero(wlist,no2)
        call iazzero(hlist,no2*no2)
        call iazzero(nwlist,nomax+no2)
        call iazzero(nhlist,nhmax+no2+no2)

        is=0
        
        do i=1,NOx+nres2
          k=0
          do j=1,NOx+nres2
            if ( i .ne. j) then
              do ix=1,3
                dij(ix)=POS(ix,i)-POS(ix,j)
              enddo
              call r_to_s(dij,sij)
              do ix=1,3
                sij(ix)=sij(ix)-anint(sij(ix))
              enddo
              call s_to_r(sij,rrij)
              Rij=DSQRT(rrij(1)*rrij(1)+rrij(2)*rrij(2)+rrij(3)*rrij(3))
              r(4,i,j)=Rij
              do ix=1,3
                r(ix,j,i) = rrij(ix)
              enddo
              if(Rij.lt.Rcw) then
                k=k+1
                wlist(i)=wlist(i)+1
                nwlist(k,i)=j
              endif
            endif
          enddo
          is=max(is,wlist(i))
        enddo
        if(is.gt.nomax) then
          CALL STOPGM('CHAIN_INT','too many O neighbors')
        endif
        is=0
        do i=1,NOx+nres2
          do j=NOx+nres2+1,natom
            do ix=1,3
              dij(ix)=POS(ix,i)-POS(ix,j)
            enddo
            call r_to_s(dij,sij)
            do ix=1,3
              sij(ix)=sij(ix)-anint(sij(ix))
            enddo
            call s_to_r(sij,rrij)
            Rij=DSQRT(rrij(1)**2+rrij(2)**2+rrij(3)**2)
            r(4,j,i)=Rij
            r(4,i,j)=Rij
            do ix=1,3
              r(ix,j,i) = rrij(ix)
              r(ix,i,j) = -rrij(ix)
            enddo
          enddo
        enddo

C       ==------------------------------------------------------------==
C       List of H atoms w.r.t. center of mass of Ox
        do i=1,NOx+nres2
          do ii=1,wlist(i)
            j=nwlist(ii,i)
            l=0
            do k=NOx+nres2+1,natom
              do ix=1,3
                dij(ix)=POS(ix,i)-POS(ix,j)
              enddo
              call r_to_s(dij,sij)
              do ix=1,3
                sij(ix)=sij(ix)-anint(sij(ix))
              enddo
              call s_to_r(sij,rrij)
              do ix=1,3
                dij(ix)=POS(ix,k)-POS(ix,j)-rrij(ix)*0.5d0
              enddo
              call r_to_s(dij,sij)
              do ix=1,3
                sij(ix)=sij(ix)-anint(sij(ix))
              enddo
              call s_to_r(sij,rrij)
              Rij=DSQRT(rrij(1)**2+rrij(2)**2+rrij(3)**2)
              if(Rij.lt.Rch) then
                l=l+1
                hlist(j,i)=l
                nhlist(l,j,i)=k
                hlist(i,j)=l
                nhlist(l,i,j)=k
              endif
            enddo
            is=max(is,hlist(i,j))
          enddo
        enddo
        if(is.gt.nhmax) then
          CALL STOPGM('CHAIN_INT','too many H neighbors')
        endif
      else
        do i =1,NOx+nres2
          do jj=1,wlist(i)
            j=nwlist(jj,i)
            do ix=1,3
              dij(ix)=POS(ix,i)-POS(ix,j)
            enddo
            call r_to_s(dij,sij)
            do ix=1,3
              sij(ix)=sij(ix)-anint(sij(ix))
            enddo
            call s_to_r(sij,rrij)
            Rij=DSQRT(rrij(1)**2+rrij(2)**2+rrij(3)**2)
            r(4,j,i)=Rij
            r(4,i,j)=Rij
            do ix=1,3
              r(ix,j,i) = rrij(ix)
              r(ix,i,j) = -rrij(ix)
            enddo
            do kk=1,hlist(j,i)
              k=nhlist(kk,j,i)
              do ix=1,3
                dij(ix)=POS(ix,i)-POS(ix,k)
              enddo
              call r_to_s(dij,sij)
              do ix=1,3
                sij(ix)=sij(ix)-anint(sij(ix))
              enddo
              call s_to_r(sij,rrij)
              Rij=DSQRT(rrij(1)**2+rrij(2)**2+rrij(3)**2)
              r(4,k,i)=Rij
              r(4,i,k)=Rij
              do ix=1,3
                r(ix,k,i) = rrij(ix)
                r(ix,i,k) = -rrij(ix)
                dij(ix)=POS(ix,k)-POS(ix,j)
              enddo
              call r_to_s(dij,sij)
              do ix=1,3
                sij(ix)=sij(ix)-anint(sij(ix))
              enddo
              call s_to_r(sij,rrij)
              Rij=DSQRT(rrij(1)**2+rrij(2)**2+rrij(3)**2)
              r(4,j,k)=Rij
              r(4,k,j)=Rij
              do ix=1,3
                r(ix,j,k) = rrij(ix)
                r(ix,k,j) = -rrij(ix)
              enddo
            enddo
          enddo
        enddo
      endif
C     ==--------------------------------------------------------------==
C     Compute the switching function on every H2O and active residues
      CALL AZZERO(SW,nomax+NOx)
      CALL AZZERO(SWF,NOx+nres2+nres2)
      CALL AZZERO(DSW,9+nhmax+nomax+NOx)
      CALL AZZERO(DSWF,9+nhmax+NOx+nres2+nres2)
      CALL AZZERO(F,3+natom+nc)
      CALL AZZERO(Force,3*natom)
      tmp=0.D0

      do 100 i=1,NOx
        do j=1,wlist(i)
          l=nwlist(j,i)
          d0=r(4,l,i)-OPTDIST
          d=d0/ro
          dn=d**no
          dm=d**mo
          num=1.d0-dn
          den=1.d0-dm
          func1=num/den
          
          dfunc1=(-dn*no/den +num/den/den *dm*mo)/d/ro/r(4,l,i)
          
          func2=0
          
          DO k=1,hlist(i,l)
            ii=nhlist(k,i,l)
            d0=r(4,ii,i)+r(4,ii,l)-r(4,l,i)
            d=d0/rh
            dn=d**nh
            dm=d**mh
            num=1.d0-dn
            den=1.d0-dm
            func=num/den
            
            func2=func2+func
            
            dfunc=(-dn*nh/den +num/den/den *dm*mh)/d/rh
            
            DSW(1,k,j,i)=r(1,l,i)*dfunc1*func  +
     &           r(1,ii,i)*func1*dfunc/r(4,ii,i)- 
     &           r(1,l ,i)*func1*dfunc/r(4,l,i)
            DSW(2,k,j,i)=r(2,l,i)*dfunc1*func  +
     &           r(2,ii,i)*func1*dfunc/r(4,ii,i)-
     &           r(2,l ,i)*func1*dfunc/r(4,l,i)
            DSW(3,k,j,i)=r(3,l,i)*dfunc1*func  +
     &           r(3,ii,i)*func1*dfunc/r(4,ii,i)-
     &           r(3,l ,i)*func1*dfunc/r(4,l,i)
            
            DSW(4,k,j,i)=r(1,i,l)*dfunc1*func  +
     &           r(1,ii,l)*func1*dfunc/r(4,ii,l)- 
     &           r(1,i ,l)*func1*dfunc/r(4,i,l)
            DSW(5,k,j,i)=r(2,i,l)*dfunc1*func  +
     &           r(2,ii,l)*func1*dfunc/r(4,ii,l)- 
     &           r(2,i ,l)*func1*dfunc/r(4,i,l)
            DSW(6,k,j,i)=r(3,i,l)*dfunc1*func  +
     &           r(3,ii,l)*func1*dfunc/r(4,ii,l)- 
     &           r(3,i ,l)*func1*dfunc/r(4,i,l)
            
            DSW(7,k,j,i)=r(1,i,ii)*func1*dfunc/r(4,i,ii)+
     &           r(1,l,ii)*func1*dfunc/r(4,l,ii)
            DSW(8,k,j,i)=r(2,i,ii)*func1*dfunc/r(4,i,ii)+
     &           r(2,l,ii)*func1*dfunc/r(4,l,ii)
            DSW(9,k,j,i)=r(3,i,ii)*func1*dfunc/r(4,i,ii)+
     &           r(3,l,ii)*func1*dfunc/r(4,l,ii)

          ENDDO                 ! vectorize do k=1,hlist(i,l)
          
c         write(6,*) 'func ',i,j,func1,func2,dfunc
          a=func1*func2
          SW(j,i)=a
          b=a**20
c         write(6,*) a,b,a-b,1-b
          SW(j,i)=(a-b)/(1-b)
          cfact=(a-20*b+20*(a-b)*b/(1-b))/a/(1-b)
          DO k=1,hlist(i,l)
            DSW(1,k,j,i)=DSW(1,k,j,i)*cfact
            DSW(2,k,j,i)=DSW(2,k,j,i)*cfact
            DSW(3,k,j,i)=DSW(3,k,j,i)*cfact
            DSW(4,k,j,i)=DSW(4,k,j,i)*cfact
            DSW(5,k,j,i)=DSW(5,k,j,i)*cfact
            DSW(6,k,j,i)=DSW(6,k,j,i)*cfact
            DSW(7,k,j,i)=DSW(7,k,j,i)*cfact
            DSW(8,k,j,i)=DSW(8,k,j,i)*cfact
            DSW(9,k,j,i)=DSW(9,k,j,i)*cfact
          ENDDO
          
          tmp=tmp+sw(j,i)
          
        enddo                   !do j=1,wlist(i)
 100  continue
      
C     run over all the last terms
      do jj=1,nres2
        i=NOx+jj
        do l=1,NOx
          d0=r(4,l,i)-OPTDIST
          d=d0/ro
          dn=d**no
          dm=d**mo
          num=1.d0-dn
          den=1.d0-dm
          func1=num/den
          dfunc1=(-dn*no/den +num/den/den *dm*mo)/d/ro/r(4,l,i)
          
          func2=0
          
          do k=1,hlist(i,l)
            ii=nhlist(k,i,l)
            d0=r(4,ii,i)+r(4,ii,l)-r(4,l,i)
            d=d0/rh
            dn=d**nh
            dm=d**mh
            num=1.d0-dn
            den=1.d0-dm
            func=num/den
            func2=func2+func
            
            dfunc=(-dn*nh/den +num/den/den *dm*mh)/d/rh
            
            DSWF(1,k,l,jj)=r(1,l,i)*dfunc1*func  +
     &           r(1,ii,i)*func1*dfunc/r(4,ii,i)- 
     &           r(1,l ,i)*func1*dfunc/r(4,l,i)
            DSWF(2,k,l,jj)=r(2,l,i)*dfunc1*func  +
     &           r(2,ii,i)*func1*dfunc/r(4,ii,i)- 
     &           r(2,l ,i)*func1*dfunc/r(4,l,i)
            DSWF(3,k,l,jj)=r(3,l,i)*dfunc1*func  +
     &           r(3,ii,i)*func1*dfunc/r(4,ii,i)- 
     &           r(3,l ,i)*func1*dfunc/r(4,l,i)

            DSWF(4,k,l,jj)=r(1,i,l)*dfunc1*func  +
     &           r(1,ii,l)*func1*dfunc/r(4,ii,l)- 
     &           r(1,i ,l)*func1*dfunc/r(4,i,l)
            DSWF(5,k,l,jj)=r(2,i,l)*dfunc1*func  +
     &           r(2,ii,l)*func1*dfunc/r(4,ii,l)- 
     &           r(2,i ,l)*func1*dfunc/r(4,i,l)
            DSWF(6,k,l,jj)=r(3,i,l)*dfunc1*func  +
     &           r(3,ii,l)*func1*dfunc/r(4,ii,l)- 
     &           r(3,i ,l)*func1*dfunc/r(4,i,l)

            DSWF(7,k,l,jj)=r(1,i,ii)*func1*dfunc/r(4,i,ii)+
     &           r(1,l,ii)*func1*dfunc/r(4,l,ii)
            DSWF(8,k,l,jj)=r(2,i,ii)*func1*dfunc/r(4,i,ii)+
     &           r(2,l,ii)*func1*dfunc/r(4,l,ii)
            DSWF(9,k,l,jj)=r(3,i,ii)*func1*dfunc/r(4,i,ii)+
     &           r(3,l,ii)*func1*dfunc/r(4,l,ii)

          enddo
          
C         SWF(l,jj)=func2*func1
          a=func1*func2
          SWF(l,jj)=a
          b=a**20
          SWF(l,jj)=(a-b)/(1-b)
          cfact=(a-20*b+20*(a-b)*b/(1-b))/a/(1-b)
          do k=1,hlist(i,l)
            DSWF(1,k,l,jj)=DSWF(1,k,l,jj)*cfact
            DSWF(2,k,l,jj)=DSWF(2,k,l,jj)*cfact
            DSWF(3,k,l,jj)=DSWF(3,k,l,jj)*cfact
            DSWF(4,k,l,jj)=DSWF(4,k,l,jj)*cfact
            DSWF(5,k,l,jj)=DSWF(5,k,l,jj)*cfact
            DSWF(6,k,l,jj)=DSWF(6,k,l,jj)*cfact
            DSWF(7,k,l,jj)=DSWF(7,k,l,jj)*cfact
            DSWF(8,k,l,jj)=DSWF(8,k,l,jj)*cfact
            DSWF(9,k,l,jj)=DSWF(9,k,l,jj)*cfact
          enddo
C         tmp=tmp+swf(l,jj)
        enddo
      enddo                     !do jj=1,nres2

C      write(6,*)'Average # of H bonds',tmp/dble(NOx+nres2)
C      call fileopen(16,'Hbonds',FO_APP,ferror)
C      write(16,*)it,tmp/dble(NOx+nres2)
C      call fileclose(16)
C      return

C     ==--------------------------------------------------------------==
C     compute the order parameter d for the chain (P)
C     start from residue 1 and go to residue 2
C     WARNING: the chain must be symmetric
      PT1=0.d0
      PT2=0.d0
      PT3=0.d0
      PT4=0.d0
      PT5=0.d0
C      PT6=0.d0
      CALL AZZERO(MaxP,nc)
      CALL AZZERO(F,3+natom+nc)

      do 5000 l=1,NRES1
        a0=l
        do 4000 p1=1,wlist(a0)
          aa1=nwlist(p1,a0)
          if (DABS(sw(p1,a0)) .lt. small) then
            GOTO 5000
          endif
          Ptmp1=sw(p1,a0)
          do k=1,nres2
            ii=NOx+k
            tmp=swf(aa1,k)
            if (aa1 .eq. ii) then
              CALL STOPGM('CHAIN_INT','atoms too close: no chain')
            endif
            eve=Ptmp1*tmp
            PT1 = PT1 + DEXP(bbeta*eve)
            eve=eve*DEXP(bbeta*eve)
Compute forces
            tmp1=SW(p1,a0)
            tmpf=eve/tmp1
            do kf=1,hlist(a0,aa1)
              kk=nhlist(kf,a0,aa1)
              F(1,a0,1) =F(1,a0,1) +DSW(1,kf,p1,a0)*tmpf
              F(2,a0,1) =F(2,a0,1) +DSW(2,kf,p1,a0)*tmpf
              F(3,a0,1) =F(3,a0,1) +DSW(3,kf,p1,a0)*tmpf
              F(1,aa1,1)=F(1,aa1,1)+DSW(4,kf,p1,a0)*tmpf
              F(2,aa1,1)=F(2,aa1,1)+DSW(5,kf,p1,a0)*tmpf
              F(3,aa1,1)=F(3,aa1,1)+DSW(6,kf,p1,a0)*tmpf
              F(1,kk,1) =F(1,kk,1) +DSW(7,kf,p1,a0)*tmpf
              F(2,kk,1) =F(2,kk,1) +DSW(8,kf,p1,a0)*tmpf
              F(3,kk,1) =F(3,kk,1) +DSW(9,kf,p1,a0)*tmpf
            enddo
            tmpf=eve/SWF(aa1,k)
            do kf=1,hlist(aa1,ii)
              kk=nhlist(kf,aa1,ii)
              F(1,ii,1) =F(1,ii,1) +DSWF(1,kf,aa1,k)*tmpf
              F(2,ii,1) =F(2,ii,1) +DSWF(2,kf,aa1,k)*tmpf
              F(3,ii,1) =F(3,ii,1) +DSWF(3,kf,aa1,k)*tmpf
              F(1,aa1,1)=F(1,aa1,1)+DSWF(4,kf,aa1,k)*tmpf
              F(2,aa1,1)=F(2,aa1,1)+DSWF(5,kf,aa1,k)*tmpf
              F(3,aa1,1)=F(3,aa1,1)+DSWF(6,kf,aa1,k)*tmpf
              F(1,kk,1) =F(1,kk,1) +DSWF(7,kf,aa1,k)*tmpf
              F(2,kk,1) =F(2,kk,1) +DSWF(8,kf,aa1,k)*tmpf
              F(3,kk,1) =F(3,kk,1) +DSWF(9,kf,aa1,k)*tmpf
            enddo

            if (Ptmp1*tmp.gt.MaxP(1)) then
              MaxP(1)=Ptmp1*tmp
C AK 2005/03/26: not used.
C              ch(1,1)=a0
C              ch(2,1)=aa1
C              ch(3,1)=ii  
            endif
          enddo
          if(NLONG.eq.1) then
            GOTO 4000
          endif

          do 101 p2=1,wlist(aa1)
            aa2=nwlist(p2,aa1)  ! take a neighbour Ox
            if (aa2 .eq. a0) then
              GOTO 101 
            endif
            if (DABS(sw(p2,aa1)) .lt. small) then
              GOTO 101
            endif
            Ptmp2=Ptmp1*sw(p2,aa1)
            do k=1,nres2
              ii=NOx+k
              tmp=swf(aa2,k)
              if (aa1 .eq. ii) Ptmp2=0.d0
              eve=Ptmp2*tmp
              PT2 = PT2 + DEXP(bbeta*eve)
              eve=eve*DEXP(bbeta*eve)
Compute forces 
              tmpf=eve/tmp1
              do kf=1,hlist(a0,aa1)
                kk=nhlist(kf,a0,aa1)
                F(1,a0,2) =F(1,a0,2) +DSW(1,kf,p1,a0)*tmpf
                F(2,a0,2) =F(2,a0,2) +DSW(2,kf,p1,a0)*tmpf
                F(3,a0,2) =F(3,a0,2) +DSW(3,kf,p1,a0)*tmpf
                F(1,aa1,2)=F(1,aa1,2)+DSW(4,kf,p1,a0)*tmpf
                F(2,aa1,2)=F(2,aa1,2)+DSW(5,kf,p1,a0)*tmpf
                F(3,aa1,2)=F(3,aa1,2)+DSW(6,kf,p1,a0)*tmpf
                F(1,kk,2) =F(1,kk,2) +DSW(7,kf,p1,a0)*tmpf
                F(2,kk,2) =F(2,kk,2) +DSW(8,kf,p1,a0)*tmpf
                F(3,kk,2) =F(3,kk,2) +DSW(9,kf,p1,a0)*tmpf
              enddo
              tmp2=SW(p2,aa1)
              tmpf=eve/tmp2
              do kf=1,hlist(aa1,aa2)
                kk=nhlist(kf,aa1,aa2)
                F(1,aa1,2)=F(1,aa1,2)+DSW(1,kf,p2,aa1)*tmpf
                F(2,aa1,2)=F(2,aa1,2)+DSW(2,kf,p2,aa1)*tmpf
                F(3,aa1,2)=F(3,aa1,2)+DSW(3,kf,p2,aa1)*tmpf
                F(1,aa2,2)=F(1,aa2,2)+DSW(4,kf,p2,aa1)*tmpf
                F(2,aa2,2)=F(2,aa2,2)+DSW(5,kf,p2,aa1)*tmpf
                F(3,aa2,2)=F(3,aa2,2)+DSW(6,kf,p2,aa1)*tmpf
                F(1,kk,2) =F(1,kk,2) +DSW(7,kf,p2,aa1)*tmpf
                F(2,kk,2) =F(2,kk,2) +DSW(8,kf,p2,aa1)*tmpf
                F(3,kk,2) =F(3,kk,2) +DSW(9,kf,p2,aa1)*tmpf
              enddo
              tmpf=eve/SWF(aa2,k)
              do kf=1,hlist(aa2,ii)
                kk=nhlist(kf,aa2,ii)
                F(1,ii,2) =F(1,ii,2) +DSWF(1,kf,aa2,k)*tmpf
                F(2,ii,2) =F(2,ii,2) +DSWF(2,kf,aa2,k)*tmpf
                F(3,ii,2) =F(3,ii,2) +DSWF(3,kf,aa2,k)*tmpf
                F(1,aa2,2)=F(1,aa2,2)+DSWF(4,kf,aa2,k)*tmpf
                F(2,aa2,2)=F(2,aa2,2)+DSWF(5,kf,aa2,k)*tmpf
                F(3,aa2,2)=F(3,aa2,2)+DSWF(6,kf,aa2,k)*tmpf
                F(1,kk,2) =F(1,kk,2) +DSWF(7,kf,aa2,k)*tmpf
                F(2,kk,2) =F(2,kk,2) +DSWF(8,kf,aa2,k)*tmpf
                F(3,kk,2) =F(3,kk,2) +DSWF(9,kf,aa2,k)*tmpf
              enddo

              if (Ptmp2*tmp.gt.MaxP(2)) then
                MaxP(2)=Ptmp2*tmp
C AK 2005/03/26: not used.
C                ch(1,2)=a0
C                ch(2,2)=aa1
C                ch(3,2)=aa2
C                ch(4,2)=ii
              endif
            enddo
            if(NLONG.eq.2) then
              GOTO 101
            endif
            
            do 102 p3=1,wlist(aa2)
              aa3=nwlist(p3,aa2) !prendo un Ox suo vicino
              if (aa3 .eq. a0) then
                GOTO 102
              endif
              if (aa3 .eq. aa1) then
                GOTO 102
              endif
              if (DABS(sw(p3,aa2)) .lt. small) then
                GOTO 102
              endif
              Ptmp3=Ptmp2*sw(p3,aa2)
              do k=1,nres2
                ii=NOx+k
                tmp=swf(aa3,k)
                if (aa2 .eq. ii) then
                  Ptmp3=0.d0
                endif
                eve=Ptmp3*tmp
                PT3 = PT3 + DEXP(bbeta*eve)
                eve=eve*DEXP(bbeta*eve)
Compute forces
                tmpf=eve/tmp1
                do kf=1,hlist(a0,aa1)
                  kk=nhlist(kf,a0,aa1)
                  F(1,a0,3) =F(1,a0,3) +DSW(1,kf,p1,a0)*tmpf
                  F(2,a0,3) =F(2,a0,3) +DSW(2,kf,p1,a0)*tmpf
                  F(3,a0,3) =F(3,a0,3) +DSW(3,kf,p1,a0)*tmpf
                  F(1,aa1,3)=F(1,aa1,3)+DSW(4,kf,p1,a0)*tmpf
                  F(2,aa1,3)=F(2,aa1,3)+DSW(5,kf,p1,a0)*tmpf
                  F(3,aa1,3)=F(3,aa1,3)+DSW(6,kf,p1,a0)*tmpf
                  F(1,kk,3) =F(1,kk,3) +DSW(7,kf,p1,a0)*tmpf
                  F(2,kk,3) =F(2,kk,3) +DSW(8,kf,p1,a0)*tmpf
                  F(3,kk,3) =F(3,kk,3) +DSW(9,kf,p1,a0)*tmpf
                enddo
                tmpf=eve/tmp2
                do kf=1,hlist(aa1,aa2)
                  kk=nhlist(kf,aa1,aa2)
                  F(1,aa1,3)=F(1,aa1,3)+DSW(1,kf,p2,aa1)*tmpf
                  F(2,aa1,3)=F(2,aa1,3)+DSW(2,kf,p2,aa1)*tmpf
                  F(3,aa1,3)=F(3,aa1,3)+DSW(3,kf,p2,aa1)*tmpf
                  F(1,aa2,3)=F(1,aa2,3)+DSW(4,kf,p2,aa1)*tmpf
                  F(2,aa2,3)=F(2,aa2,3)+DSW(5,kf,p2,aa1)*tmpf
                  F(3,aa2,3)=F(3,aa2,3)+DSW(6,kf,p2,aa1)*tmpf
                  F(1,kk,3) =F(1,kk,3) +DSW(7,kf,p2,aa1)*tmpf
                  F(2,kk,3) =F(2,kk,3) +DSW(8,kf,p2,aa1)*tmpf
                  F(3,kk,3) =F(3,kk,3) +DSW(9,kf,p2,aa1)*tmpf
                enddo
                tmp3=SW(p3,aa2)
                tmpf=eve/tmp3
                do kf=1,hlist(aa2,aa3)
                  kk=nhlist(kf,aa2,aa3)
                  F(1,aa2,3)=F(1,aa2,3)+DSW(1,kf,p3,aa2)*tmpf
                  F(2,aa2,3)=F(2,aa2,3)+DSW(2,kf,p3,aa2)*tmpf
                  F(3,aa2,3)=F(3,aa2,3)+DSW(3,kf,p3,aa2)*tmpf
                  F(1,aa3,3)=F(1,aa3,3)+DSW(4,kf,p3,aa2)*tmpf
                  F(2,aa3,3)=F(2,aa3,3)+DSW(5,kf,p3,aa2)*tmpf
                  F(3,aa3,3)=F(3,aa3,3)+DSW(6,kf,p3,aa2)*tmpf
                  F(1,kk,3) =F(1,kk,3) +DSW(7,kf,p3,aa2)*tmpf
                  F(2,kk,3) =F(2,kk,3) +DSW(8,kf,p3,aa2)*tmpf
                  F(3,kk,3) =F(3,kk,3) +DSW(9,kf,p3,aa2)*tmpf
                enddo
                tmpf=eve/SWF(aa3,k)
                do kf=1,hlist(aa3,ii)
                  kk=nhlist(kf,aa3,ii)
                  F(1,ii,3) =F(1,ii,3) +DSWF(1,kf,aa3,k)*tmpf
                  F(2,ii,3) =F(2,ii,3) +DSWF(2,kf,aa3,k)*tmpf
                  F(3,ii,3) =F(3,ii,3) +DSWF(3,kf,aa3,k)*tmpf
                  F(1,aa3,3)=F(1,aa3,3)+DSWF(4,kf,aa3,k)*tmpf
                  F(2,aa3,3)=F(2,aa3,3)+DSWF(5,kf,aa3,k)*tmpf
                  F(3,aa3,3)=F(3,aa3,3)+DSWF(6,kf,aa3,k)*tmpf
                  F(1,kk,3) =F(1,kk,3) +DSWF(7,kf,aa3,k)*tmpf
                  F(2,kk,3) =F(2,kk,3) +DSWF(8,kf,aa3,k)*tmpf
                  F(3,kk,3) =F(3,kk,3) +DSWF(9,kf,aa3,k)*tmpf
                enddo

                if (Ptmp3*tmp.gt.MaxP(3)) then
                  MaxP(3)=Ptmp3*tmp
C AK 2005/03/26: not used.
C                  ch(1,3)=a0
C                  ch(2,3)=aa1
C                  ch(3,3)=aa2
C                  ch(4,3)=aa3
C                  ch(5,3)=ii
                endif
              enddo
              if(NLONG.eq.3) then
                GOTO 102
              endif
              do 103 p4=1,wlist(aa3)
                a4=nwlist(p4,aa3) ! take an Ox next neighbour
                if (a4 .eq. a0) then
                  GOTO 103
                endif
                if (a4 .eq. aa1) then
                  GOTO 103
                endif
                if (a4 .eq. aa2) then
                  GOTO 103
                endif
                if (DABS(sw(p4,aa3)) .lt. small) then
                  GOTO 103
                endif
                Ptmp4=Ptmp3*sw(p4,aa3)
                do k=1,nres2
                  ii=NOx+k
                  tmp=swf(a4,k)
                  if (aa3 .eq. ii) then
                    Ptmp4=0.d0
                  endif
                  eve=Ptmp4*tmp
                  PT4 = PT4 + DEXP(bbeta*eve)
                  eve=eve*DEXP(bbeta*eve)
Compute forces
                  tmpf=eve/tmp1
                  do kf=1,hlist(a0,aa1)
                    kk=nhlist(kf,a0,aa1)
                    F(1,a0,4) =F(1,a0,4) +DSW(1,kf,p1,a0)*tmpf
                    F(2,a0,4) =F(2,a0,4) +DSW(2,kf,p1,a0)*tmpf
                    F(3,a0,4) =F(3,a0,4) +DSW(3,kf,p1,a0)*tmpf
                    F(1,aa1,4)=F(1,aa1,4)+DSW(4,kf,p1,a0)*tmpf
                    F(2,aa1,4)=F(2,aa1,4)+DSW(5,kf,p1,a0)*tmpf
                    F(3,aa1,4)=F(3,aa1,4)+DSW(6,kf,p1,a0)*tmpf
                    F(1,kk,4) =F(1,kk,4) +DSW(7,kf,p1,a0)*tmpf
                    F(2,kk,4) =F(2,kk,4) +DSW(8,kf,p1,a0)*tmpf
                    F(3,kk,4) =F(3,kk,4) +DSW(9,kf,p1,a0)*tmpf
                  enddo
                  tmpf=eve/tmp2
                  do kf=1,hlist(aa1,aa2)
                    kk=nhlist(kf,aa1,aa2)
                    F(1,aa1,4)=F(1,aa1,4)+DSW(1,kf,p2,aa1)*tmpf
                    F(2,aa1,4)=F(2,aa1,4)+DSW(2,kf,p2,aa1)*tmpf
                    F(3,aa1,4)=F(3,aa1,4)+DSW(3,kf,p2,aa1)*tmpf
                    F(1,aa2,4)=F(1,aa2,4)+DSW(4,kf,p2,aa1)*tmpf
                    F(2,aa2,4)=F(2,aa2,4)+DSW(5,kf,p2,aa1)*tmpf
                    F(3,aa2,4)=F(3,aa2,4)+DSW(6,kf,p2,aa1)*tmpf
                    F(1,kk,4) =F(1,kk,4) +DSW(7,kf,p2,aa1)*tmpf
                    F(2,kk,4) =F(2,kk,4) +DSW(8,kf,p2,aa1)*tmpf
                    F(3,kk,4) =F(3,kk,4) +DSW(9,kf,p2,aa1)*tmpf
                  enddo
                  tmpf=eve/tmp3
                  do kf=1,hlist(aa2,aa3)
                    kk=nhlist(kf,aa2,aa3)
                    F(1,aa2,4)=F(1,aa2,4)+DSW(1,kf,p3,aa2)*tmpf
                    F(2,aa2,4)=F(2,aa2,4)+DSW(2,kf,p3,aa2)*tmpf
                    F(3,aa2,4)=F(3,aa2,4)+DSW(3,kf,p3,aa2)*tmpf
                    F(1,aa3,4)=F(1,aa3,4)+DSW(4,kf,p3,aa2)*tmpf
                    F(2,aa3,4)=F(2,aa3,4)+DSW(5,kf,p3,aa2)*tmpf
                    F(3,aa3,4)=F(3,aa3,4)+DSW(6,kf,p3,aa2)*tmpf
                    F(1,kk,4) =F(1,kk,4) +DSW(7,kf,p3,aa2)*tmpf
                    F(2,kk,4) =F(2,kk,4) +DSW(8,kf,p3,aa2)*tmpf
                    F(3,kk,4) =F(3,kk,4) +DSW(9,kf,p3,aa2)*tmpf
                  enddo
                  tmp4=SW(p4,aa3)
                  tmpf=eve/tmp4
                  do kf=1,hlist(aa3,a4)
                    kk=nhlist(kf,aa3,a4)
                    F(1,aa3,4)=F(1,aa3,4)+DSW(1,kf,p4,aa3)*tmpf
                    F(2,aa3,4)=F(2,aa3,4)+DSW(2,kf,p4,aa3)*tmpf
                    F(3,aa3,4)=F(3,aa3,4)+DSW(3,kf,p4,aa3)*tmpf
                    F(1,a4,4) =F(1,a4,4) +DSW(4,kf,p4,aa3)*tmpf
                    F(2,a4,4) =F(2,a4,4) +DSW(5,kf,p4,aa3)*tmpf
                    F(3,a4,4) =F(3,a4,4) +DSW(6,kf,p4,aa3)*tmpf
                    F(1,kk,4) =F(1,kk,4) +DSW(7,kf,p4,aa3)*tmpf
                    F(2,kk,4) =F(2,kk,4) +DSW(8,kf,p4,aa3)*tmpf
                    F(3,kk,4) =F(3,kk,4) +DSW(9,kf,p4,aa3)*tmpf
                  enddo
                  tmpf=eve/SWF(a4,k)
                  do kf=1,hlist(a4,ii)
                    kk=nhlist(kf,a4,ii)
                    F(1,ii,4)=F(1,ii,4)+DSWF(1,kf,a4,k)*tmpf
                    F(2,ii,4)=F(2,ii,4)+DSWF(2,kf,a4,k)*tmpf
                    F(3,ii,4)=F(3,ii,4)+DSWF(3,kf,a4,k)*tmpf
                    F(1,a4,4)=F(1,a4,4)+DSWF(4,kf,a4,k)*tmpf
                    F(2,a4,4)=F(2,a4,4)+DSWF(5,kf,a4,k)*tmpf
                    F(3,a4,4)=F(3,a4,4)+DSWF(6,kf,a4,k)*tmpf
                    F(1,kk,4)=F(1,kk,4)+DSWF(7,kf,a4,k)*tmpf
                    F(2,kk,4)=F(2,kk,4)+DSWF(8,kf,a4,k)*tmpf
                    F(3,kk,4)=F(3,kk,4)+DSWF(9,kf,a4,k)*tmpf
                  enddo

                  if (Ptmp4*tmp.gt.MaxP(4)) then
                    MaxP(4)=Ptmp4*tmp
C AK 2005/03/26: not used.
C                    ch(1,4)=a0
C                    ch(2,4)=aa1
C                    ch(3,4)=aa2
C                    ch(4,4)=aa3
C                    ch(5,4)=a4
C                    ch(6,4)=ii
                  endif
                enddo
                if(NLONG.eq.4) then
                  GOTO 103
                endif

                do 104 p5=1,wlist(a4)
                  a5=nwlist(p5,a4) !prendo un Ox suo vicino
                  if (a5 .eq. a0) then
                    GOTO 104
                  endif
                  if (a5 .eq. aa1) then
                    GOTO 104
                  endif
                  if (a5 .eq. aa2) then
                    GOTO 104
                  endif
                  if (a5 .eq. aa3) then
                    GOTO 104 
                  endif
                  if (DABS(sw(p5,a4)) .lt. small) then
                    GOTO 104
                  endif
                  Ptmp5=Ptmp4*sw(p5,a4)
                  do k=1,nres2
                    ii=NOx+k
                    tmp=swf(a5,k)
                    if (a4 .eq. ii) Ptmp5=0.d0
                    eve=Ptmp5*tmp
                    PT5 = PT5 + DEXP(bbeta*eve)
                    eve=eve*DEXP(bbeta*eve)

Compute forces
                    tmpf=eve/tmp1
                    do kf=1,hlist(a0,aa1)
                      kk=nhlist(kf,a0,aa1)
                      F(1,a0,5) =F(1,a0,5) +DSW(1,kf,p1,a0)*tmpf
                      F(2,a0,5) =F(2,a0,5) +DSW(2,kf,p1,a0)*tmpf
                      F(3,a0,5) =F(3,a0,5) +DSW(3,kf,p1,a0)*tmpf
                      F(1,aa1,5)=F(1,aa1,5)+DSW(4,kf,p1,a0)*tmpf
                      F(2,aa1,5)=F(2,aa1,5)+DSW(5,kf,p1,a0)*tmpf
                      F(3,aa1,5)=F(3,aa1,5)+DSW(6,kf,p1,a0)*tmpf
                      F(1,kk,5) =F(1,kk,5) +DSW(7,kf,p1,a0)*tmpf
                      F(2,kk,5) =F(2,kk,5) +DSW(8,kf,p1,a0)*tmpf
                      F(3,kk,5) =F(3,kk,5) +DSW(9,kf,p1,a0)*tmpf
                    enddo
                    tmpf=eve/tmp2
                    do kf=1,hlist(aa1,aa2)
                      kk=nhlist(kf,aa1,aa2)
                      F(1,aa1,5)=F(1,aa1,5)+DSW(1,kf,p2,aa1)*tmpf
                      F(2,aa1,5)=F(2,aa1,5)+DSW(2,kf,p2,aa1)*tmpf
                      F(3,aa1,5)=F(3,aa1,5)+DSW(3,kf,p2,aa1)*tmpf
                      F(1,aa2,5)=F(1,aa2,5)+DSW(4,kf,p2,aa1)*tmpf
                      F(2,aa2,5)=F(2,aa2,5)+DSW(5,kf,p2,aa1)*tmpf
                      F(3,aa2,5)=F(3,aa2,5)+DSW(6,kf,p2,aa1)*tmpf
                      F(1,kk,5) =F(1,kk,5) +DSW(7,kf,p2,aa1)*tmpf
                      F(2,kk,5) =F(2,kk,5) +DSW(8,kf,p2,aa1)*tmpf
                      F(3,kk,5) =F(3,kk,5) +DSW(9,kf,p2,aa1)*tmpf
                    enddo
                    tmpf=eve/tmp3
                    do kf=1,hlist(aa2,aa3)
                      kk=nhlist(kf,aa2,aa3)
                      F(1,aa2,5)=F(1,aa2,5)+DSW(1,kf,p3,aa2)*tmpf
                      F(2,aa2,5)=F(2,aa2,5)+DSW(2,kf,p3,aa2)*tmpf
                      F(3,aa2,5)=F(3,aa2,5)+DSW(3,kf,p3,aa2)*tmpf
                      F(1,aa3,5)=F(1,aa3,5)+DSW(4,kf,p3,aa2)*tmpf
                      F(2,aa3,5)=F(2,aa3,5)+DSW(5,kf,p3,aa2)*tmpf
                      F(3,aa3,5)=F(3,aa3,5)+DSW(6,kf,p3,aa2)*tmpf
                      F(1,kk,5) =F(1,kk,5) +DSW(7,kf,p3,aa2)*tmpf
                      F(2,kk,5) =F(2,kk,5) +DSW(8,kf,p3,aa2)*tmpf
                      F(3,kk,5) =F(3,kk,5) +DSW(9,kf,p3,aa2)*tmpf
                    enddo
                    tmpf=eve/tmp4
                    do kf=1,hlist(aa3,a4)
                      kk=nhlist(kf,aa3,a4)
                      F(1,aa3,5)=F(1,aa3,5)+DSW(1,kf,p4,aa3)*tmpf
                      F(2,aa3,5)=F(2,aa3,5)+DSW(2,kf,p4,aa3)*tmpf
                      F(3,aa3,5)=F(3,aa3,5)+DSW(3,kf,p4,aa3)*tmpf
                      F(1,a4,5) =F(1,a4,5) +DSW(4,kf,p4,aa3)*tmpf
                      F(2,a4,5) =F(2,a4,5) +DSW(5,kf,p4,aa3)*tmpf
                      F(3,a4,5) =F(3,a4,5) +DSW(6,kf,p4,aa3)*tmpf
                      F(1,kk,5) =F(1,kk,5) +DSW(7,kf,p4,aa3)*tmpf
                      F(2,kk,5) =F(2,kk,5) +DSW(8,kf,p4,aa3)*tmpf
                      F(3,kk,5) =F(3,kk,5) +DSW(9,kf,p4,aa3)*tmpf
                    enddo
                    tmp5=SW(p5,a4)
                    tmpf=eve/tmp5
                    do kf=1,hlist(a4,a5)
                      kk=nhlist(kf,a4,a5)
                      F(1,a4,5)=F(1,a4,5)+DSW(1,kf,p5,a4)*tmpf
                      F(2,a4,5)=F(2,a4,5)+DSW(2,kf,p5,a4)*tmpf
                      F(3,a4,5)=F(3,a4,5)+DSW(3,kf,p5,a4)*tmpf
                      F(1,a5,5)=F(1,a5,5)+DSW(4,kf,p5,a4)*tmpf
                      F(2,a5,5)=F(2,a5,5)+DSW(5,kf,p5,a4)*tmpf
                      F(3,a5,5)=F(3,a5,5)+DSW(6,kf,p5,a4)*tmpf
                      F(1,kk,5)=F(1,kk,5)+DSW(7,kf,p5,a4)*tmpf
                      F(2,kk,5)=F(2,kk,5)+DSW(8,kf,p5,a4)*tmpf
                      F(3,kk,5)=F(3,kk,5)+DSW(9,kf,p5,a4)*tmpf
                    enddo
                    tmpf=eve/SWF(a5,k)
                    do kf=1,hlist(a5,ii)
                      kk=nhlist(kf,a5,ii)
                      F(1,ii,5)=F(1,ii,5)+DSWF(1,kf,a5,k)*tmpf
                      F(2,ii,5)=F(2,ii,5)+DSWF(2,kf,a5,k)*tmpf
                      F(3,ii,5)=F(3,ii,5)+DSWF(3,kf,a5,k)*tmpf
                      F(1,a5,5)=F(1,a5,5)+DSWF(4,kf,a5,k)*tmpf
                      F(2,a5,5)=F(2,a5,5)+DSWF(5,kf,a5,k)*tmpf
                      F(3,a5,5)=F(3,a5,5)+DSWF(6,kf,a5,k)*tmpf
                      F(1,kk,5)=F(1,kk,5)+DSWF(7,kf,a5,k)*tmpf
                      F(2,kk,5)=F(2,kk,5)+DSWF(8,kf,a5,k)*tmpf
                      F(3,kk,5)=F(3,kk,5)+DSWF(9,kf,a5,k)*tmpf
                    enddo
                    
                    if (Ptmp5*tmp.gt.MaxP(5)) then
                      MaxP(5)=Ptmp5*tmp
C AK 2005/03/26: not used.
C                      ch(1,5)=a0
C                      ch(2,5)=aa1
C                      ch(3,5)=aa2
C                      ch(4,5)=aa3
C                      ch(5,5)=a4
C                      ch(6,5)=a5
C                      ch(7,5)=ii
                    endif
                  enddo
!!$                        if(NLONG.eq.5)cycle                                 
!!$
!!$                        do 105 p6=1,wlist(a5)
!!$                           a6=nwlist(p6,a5) !prendo un Ox suo vicino
!!$                           if (a6 .eq. a0) cycle 
!!$                           if (a6 .eq. aa1) cycle 
!!$                           if (a6 .eq. aa2) cycle 
!!$                           if (a6 .eq. aa3) cycle 
!!$                           if (a6 .eq. a4) cycle 
!!$                           if (sw(p6,a5) .eq. 0.) cycle 
!!$                           Ptmp6=Ptmp5*sw(p6,a5)
!!$                           do k=1,nres2
!!$                              ii=NOx+k
!!$                              tmp=swf(a6,k)
!!$                              if (a5 .eq. ii) Ptmp6=0.
!!$                              eve=Ptmp6*tmp
!!$                              PT6 = PT6 + exp(bbeta*eve)
!!$                              eve=eve*exp(bbeta*eve)
!!$!     ****************************************
!!$!     calcolo la forza
!!$
!!$                              tmpf=eve/tmp1
!!$                              do kf=1,hlist(a0,aa1)
!!$                                 kk=nhlist(kf,a0,aa1)
!!$                                 do ix=1,3
!!$                                    F(ix,a0,6)=F(ix,a0,6)+DSW(ix  ,kf,p1,a0)*tmpf
!!$                                    F(ix,aa1,6)=F(ix,aa1,6)+DSW(ix+3,kf,p1,a0)*tmpf
!!$                                    F(ix,kk,6)=F(ix,kk,6)+DSW(ix+6,kf,p1,a0)*tmpf
!!$                                 enddo
!!$                              enddo
!!$                              tmpf=eve/tmp2
!!$                              do kf=1,hlist(aa1,aa2)
!!$                                 kk=nhlist(kf,aa1,aa2)
!!$                                 do ix=1,3
!!$                                    F(ix,aa1,6)=F(ix,aa1,6)+DSW(ix  ,kf,p2,aa1)*tmpf
!!$                                    F(ix,aa2,6)=F(ix,aa2,6)+DSW(ix+3,kf,p2,aa1)*tmpf
!!$                                    F(ix,kk,6)=F(ix,kk,6)+DSW(ix+6,kf,p2,aa1)*tmpf
!!$                                 enddo
!!$                              enddo
!!$                              tmpf=eve/tmp3
!!$                              do kf=1,hlist(aa2,aa3)
!!$                                 kk=nhlist(kf,aa2,aa3)
!!$                                 do ix=1,3
!!$                                    F(ix,aa2,6)=F(ix,aa2,6)+DSW(ix  ,kf,p3,aa2)*tmpf
!!$                                    F(ix,aa3,6)=F(ix,aa3,6)+DSW(ix+3,kf,p3,aa2)*tmpf
!!$                                    F(ix,kk,6)=F(ix,kk,6)+DSW(ix+6,kf,p3,aa2)*tmpf
!!$                                 enddo
!!$                              enddo
!!$                              tmpf=eve/tmp4
!!$                              do kf=1,hlist(aa3,a4)
!!$                                 kk=nhlist(kf,aa3,a4)
!!$                                 do ix=1,3
!!$                                    F(ix,aa3,6)=F(ix,aa3,6)+DSW(ix  ,kf,p4,aa3)*tmpf
!!$                                    F(ix,a4,6)=F(ix,a4,6)+DSW(ix+3,kf,p4,aa3)*tmpf
!!$                                    F(ix,kk,6)=F(ix,kk,6)+DSW(ix+6,kf,p4,aa3)*tmpf
!!$                                 enddo
!!$                              enddo
!!$                              tmp5=SW(p5,a4)
!!$                              tmpf=eve/tmp5
!!$                              do kf=1,hlist(a4,a5)
!!$                                 kk=nhlist(kf,a4,a5)
!!$                                 do ix=1,3
!!$                                    F(ix,a4,6)=F(ix,a4,6)+DSW(ix  ,kf,p5,a4)*tmpf
!!$                                    F(ix,a5,6)=F(ix,a5,6)+DSW(ix+3,kf,p5,a4)*tmpf
!!$                                    F(ix,kk,6)=F(ix,kk,6)+DSW(ix+6,kf,p5,a4)*tmpf
!!$                                 enddo
!!$                              enddo
!!$                              tmp6=SW(p6,a5)
!!$                              tmpf=eve/tmp6
!!$                              do kf=1,hlist(a5,a6)
!!$                                 kk=nhlist(kf,a5,a6)
!!$                                 do ix=1,3
!!$                                    F(ix,a4,6)=F(ix,a4,6)+DSW(ix  ,kf,p6,a5)*tmpf
!!$                                    F(ix,a5,6)=F(ix,a5,6)+DSW(ix+3,kf,p6,a5)*tmpf
!!$                                    F(ix,kk,6)=F(ix,kk,6)+DSW(ix+6,kf,p6,a5)*tmpf
!!$                                 enddo
!!$                              enddo
!!$                              tmpf=eve/SWF(a6,k)
!!$                              do kf=1,hlist(a6,ii)
!!$                                 kk=nhlist(kf,a6,ii)
!!$                                 do ix=1,3
!!$                                    F(ix,ii,6)=F(ix,ii,6)+DSWF(ix  ,kf,a6,k)*tmpf
!!$                                    F(ix,a5,6)=F(ix,a5,6)+DSWF(ix+3,kf,a6,k)*tmpf
!!$                                    F(ix,kk,6)=F(ix,kk,6)+DSWF(ix+6,kf,a6,k)*tmpf
!!$                                 enddo
!!$                              enddo
!!$!     ****************************************
!!$                              if (Ptmp6*tmp.gt.MaxP(6)) then
!!$                                 MaxP(6)=Ptmp6*tmp
!!$                                 ch(1,6)=a0
!!$                                 ch(2,6)=aa1
!!$                                 ch(3,6)=aa2
!!$                                 ch(4,6)=aa3
!!$                                 ch(5,6)=a4
!!$                                 ch(6,6)=a5
!!$                                 ch(7,6)=a6
!!$                                 ch(8,6)=ii
!!$                              endif
!!$                           enddo
!!$ 105              continue
 104            continue
 103          continue
 102        continue
 101      continue
 4000   CONTINUE
 5000 CONTINUE

      
      
      OrderParameter(1)=PT1
      OrderParameter(1)=PT2
      OrderParameter(3)=PT3
      OrderParameter(4)=PT4
      OrderParameter(5)=PT5
C     OrderParameter(6)=PT6
      
      PT1=log(PT1)/bbeta
      PT2=log(PT2)/bbeta
      PT3=log(PT3)/bbeta
      PT4=log(PT4)/bbeta
      PT5=log(PT5)/bbeta
C     PT6=log(PT6)/bbeta
      
      Ptot=0.d0
      
      do i=NSHORT,NLONG
        Ptot=Ptot+OrderParameter(i)
      enddo
      
      do j=NSHORT,NLONG
        do i=1,natom
          Force(1,i)=Force(1,i)+F(1,i,j)/PTot
          Force(2,i)=Force(2,i)+F(2,i,j)/PTot
          Force(3,i)=Force(3,i)+F(3,i,j)/PTot
        enddo
      enddo
      
      PTOT = DLOG(PTot)/bbeta
      
!!$      if (mod(it,nsave).eq.1)then
!!$         call fileopen(11,'OrderParameter.dat',FO_APP,ferror)
!!$         write(11,'(f20.12)') energy
!!$         call fileclose(11)
!!$         write(33,'(i6,1x,7f10.5)')it,MaxP(1),MaxP(2),MaxP(3),MaxP(4),MaxP(5),MaxP(6),rij
!!$      endif

      CALL FREEM(IP_SW)
      CALL FREEM(IP_SWF)
      CALL FREEM(IP_DSW)
      CALL FREEM(IP_DSWF)
      CALL FREEM(IP_F)
C     ==--------------------------------------------------------------==
      RETURN 
      END
C     ==================================================================
      SUBROUTINE INV3(A,C)
C     ==--------------------------------------------------------------==
C     ==            COMPUTES THE INVERSE OF A MATRIX 3*3              ==
C     ==--------------------------------------------------------------==
      implicit none
      integer iperm,l,i,j,k,ir 
      REAL*8 A(3,3),B(3,3),C(3,3)
      REAL*8 s,den
C     ==--------------------------------------------------------------==
      DEN = 0.D0
      S   = 1.D0
      I   = 1
      J   = 2
      K   = 3
 1    CONTINUE
      DO IPERM=1,3
        DEN = DEN + S*A(1,I)*A(2,J)*A(3,K)
        L   = I
        I   = J
        J   = K
        K   = L
      END DO
      
      I = 2
      J = 1
      K = 3
      S = - S
      IF(S.LT.0.D0) THEN
        GO TO 1
      ENDIF

      IF(DABS(DEN).LT.1.D-20) THEN
        write(6,*)'singular matrix' 
      ENDIF

      I = 1
      J = 2
      K = 3

      DO IR=1,3
        B(IR,1) = (A(2,J)*A(3,K) - A(2,K)*A(3,J)) / DEN
        B(IR,2) = (A(3,J)*A(1,K) - A(3,K)*A(1,J)) / DEN
        B(IR,3) = (A(1,J)*A(2,K) - A(1,K)*A(2,J)) / DEN
        L = I
        I = J
        J = K
        K = L
      END DO

      DO L=1,3
        DO K=1,3
          C(K,L) = B(K,L)
        END DO
      END DO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE R_TO_S (R,S)
C     ==--------------------------------------------------------------==
C     == TRANSFORMS REAL VECTOR R IN SCALED VECTOR S                  ==
C     == (THROUGH MATRIX HINV)                                        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      
      REAL*8  S(*),R(*)
      REAL*8  H,HINV
      
      COMMON/metr_my/ H(3,3),HINV(3,3)

      INTEGER I,J
C     ==--------------------------------------------------------------==
      DO I=1,3
        S(I) = 0.D0
        DO J=1,3
          S(I) = S(I) + R(J)*HINV(J,I)
        END DO
      END DO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE S_TO_R (S,R)
C     ==--------------------------------------------------------------==
C     ==  TRANSFORMS SCALED VECTOR S IN REAL VECTOR R                 ==
C     ==  (THROUGH MATRIX H)                                          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
 
      REAL*8  S(*),R(*)
      REAL*8  H,HINV

      COMMON/metr_my/ H(3,3),HINV(3,3)
      
      INTEGER I,J
C     ==--------------------------------------------------------------==
      DO I=1,3
        R(I) = 0.D0
        DO J=1,3
          R(I) = R(I) + S(J)*H(J,I)
        END DO
      END DO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
!!$      subroutine  print(pos,hlist,nhlist,ch,n1,n2,MaxP)
!!$      
!!$      implicit none
!!$      integer n1,n2,ch(6+2,*)
!!$      real*8 pos(3,*)
!!$      integer hlist(n2,*),nhlist(n1,n2,*)
!!$      integer i,j,l,jj,kkk,kk,icont
!!$      real*8 conv,MaxP(*)
!!$!      real*8 avg_corr_time(2,*)
!!$
!!$      conv =.5291772
!!$
!!$      do i=1,6
!!$         icont =0
!!$         write(200+i,*)'REMARK ',MaxP(i)
!!$         do j=1,i+2
!!$            if (ch(j,i).eq.0)cycle
!!$            l=ch(j,i)
!!$            icont=icont+1
!!$            write(200+i,1)'ATOM  ',icont,'O   ','WAT',1,pos(1,l)*conv,pos(2,l)*conv,pos(3,l)*conv
!!$         enddo
!!$         do j=1,i+1
!!$            if (ch(j,i).eq.0)cycle
!!$            jj=ch(j+1,i)
!!$            l=ch(j,i)
!!$            do kk=1,hlist(jj,l)
!!$               kkk=nhlist(kk,jj,l)
!!$               icont=icont+1               
!!$               write(200+i,1)'ATOM  ',icont,'H   ','WAT',1, 
!!$     &              pos(1,kkk)*conv,pos(2,kkk)*conv,pos(3,kkk)*conv
!!$            enddo
!!$         enddo
!!$         write(200+i,*)'TER'
!!$      enddo
!!$
!!$      return
!!$1     FORMAT(a6,i5,1x,a5,aa3,2x,i4,4x,3f8.3)
!!$      end
