C     ==================================================================
      SUBROUTINE GIVESYM(TAU0)
C     ==--------------------------------------------------------------==
C     == Find automatically symmetry from atomic coordinates (TAU0)   ==
C     == and lattice vectors                                          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'symm.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'irat.inc'
C     Arguments
      REAL*8   TAU0(3,NAX,NSX)
C     Variables
      INTEGER TY(*),ISC(*),F0(49,*)
      REAL*8  XKAPA(3,*),RX(3,*),TVEC0(3,*)
      POINTER (IP_TY,TY),(IP_ISC,ISC),(IP_F0,F0),
     &        (IP_XKAPA,XKAPA),(IP_RX,RX),(IP_TVEC0,TVEC0)
      REAL*8  A0(3,3),B0(3,3),TEMP(3,3),R(3,3,48)
      INTEGER ISUB,NAT0,IS,IOUT,IHC,ISY,IA,I,I1,I2,LI,
     &        IB(48),
     &        IFIRST
      DATA    IFIRST /0/
C     ==--------------------------------------------------------------==
      CALL TISET('   GIVESYM',ISUB)
C     ==--------------------------------------------------------------==
C     == Allocations of arrays                                        ==
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_XKAPA, 3*NAT,            'XKAPA')
      CALL MEMORY(IP_RX,    3*NAT,            'RX')
      CALL MEMORY(IP_TVEC0, 3*NAT,            'TVEC0')
      CALL MEMORY(IP_TY,    NAT/IRAT+1,       'TY')
      CALL MEMORY(IP_ISC,   NAT/IRAT+1,       'ISC')
      CALL MEMORY(IP_F0,    (49*NAT)/IRAT+1,  'F0')
C     ==--------------------------------------------------------------==
      NAT0=0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          NAT0=NAT0+1
          TY(NAT0) = IS
          DO I=1,3
            XKAPA(I,NAT0) = TAU0(I,IA,IS)
          ENDDO
        ENDDO
      ENDDO
      IF(PARENT) THEN
C       Message from GROUP1 to standard output
        IOUT=6
      ELSE
C       No message for other processors
        IOUT=0
      ENDIF
C     ==--------------------------------------------------------------==
      CALL GROUP1(IOUT,A1,A2,A3,NAT,TY,XKAPA,B0(1,1),B0(1,2),B0(1,3),
     &            IHG,IHC,ISY,LI,NROT,INDPG,IB,NTVEC,
     &            FTAU,F0,R,TVEC0,ORIGIN,RX,ISC,DELTASYM)
C     ==--------------------------------------------------------------==
      CALL FREEM(IP_XKAPA)
      CALL FREEM(IP_RX)
      CALL FREEM(IP_TY)
      CALL FREEM(IP_ISC)
C     ==--------------------------------------------------------------==
C     Save Atomic table transformation
      IF(IFIRST.EQ.0) THEN
        CALL MEMORY(IP_IRT,  (120*NAT)/IRAT + 1,'IRT')
        CALL MEMORY(IP_TVEC, (3*NTVEC) + 1,'TVEC')
        IFIRST=1
      ENDIF
      CALL IAZZERO(IRT,120*NAT)
      DO I=1,NROT
        DO IA=1,NAT
          IRT(I,IA)=F0(I,IA)
        ENDDO
      ENDDO
      IF(NTVEC.NE.0) THEN
C       Save 49th row of F0 containing inequivalent atoms for transla.
        DO IA=1,NAT
          IRT(49,IA)=F0(49,IA)
        ENDDO
        IF(PARENT) THEN
          WRITE(6,*) 'TRANSLATION VECTORS:'
          DO I=1,NTVEC
            WRITE(6,'(" TVEC(",I3,"): [",F6.3,", ",F6.3,", ",F6.3,"]")')
     &           I,TVEC0(1,I),TVEC0(2,I),TVEC0(3,I)
          ENDDO
          WRITE(6,*)
        ENDIF
        CALL FREEM(IP_TVEC)
        CALL MEMORY(IP_TVEC, (3*NTVEC),'TVEC')
        CALL DCOPY(3*NTVEC,TVEC0,1,TVEC,1)
      ENDIF
C     ==--------------------------------------------------------------==
      CALL FREEM(IP_F0)
      CALL FREEM(IP_TVEC0)
C     ==--------------------------------------------------------------==
      IF( (INDPG.NE.0).AND.(.NOT.TMSYM) ) THEN
        IF(ISY.EQ.0.OR.ISY.EQ.-2) THEN
          TSYMMORPHIC=.FALSE.
        ELSEIF(ISY.EQ.1) THEN
          TSYMMORPHIC=.TRUE.
          TORIGIN=.TRUE.
        ELSEIF(ISY.EQ.-1) THEN
          TSYMMORPHIC=.TRUE.
          TORIGIN=.FALSE.
        ENDIF
        DO I = 1,3
          A0(I,1) = A1(I)/ALAT
          A0(I,2) = A2(I)/ALAT
          A0(I,3) = A3(I)/ALAT
        ENDDO
        DO I2=1,3
          DO I1=1,3
            B0(I1,I2) = ALAT*B0(I1,I2)
          ENDDO
        ENDDO
        IF(LI.GT.0) THEN
          INVERSION=LI
        ELSE
          INVERSION=0
        ENDIF
C       Transpose(B).A0 = 1
C       Put in crystal coordinates
        DO I=1,NROT
          CALL DGEMM('N','N',3,3,3,1.D0,R(1,1,IB(I)),3,
     *         A0(1,1),3,0.D0,TEMP(1,1),3)
          CALL DGEMM('T','N',3,3,3,1.D0,B0(1,1),3,
     *         TEMP(1,1),3,0.D0,XTABLE(1,1,I),3)
        ENDDO
      ELSEIF(INDPG.NE.0.AND.TMSYM) THEN
        CALL MOLSYM(STAG,NAXIS,XTABLE,NROT)
      ENDIF
      CALL TIHALT('   GIVESYM',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CHKSYM(TAU0)
C     ==--------------------------------------------------------------==
C     == Check symmetry operations and                                ==
C     == find fractional translation vectors, translation vectors     ==
C     == associated with Identity symmetry                            ==
C     == Built some arrays used for symmetrisation of some quantities ==
C     == (forces, density)                                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'symm.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'irat.inc'
C     Arguments
      REAL*8    TAU0(3,NAX,NSX)
C     Variables
      DIMENSION IRTVEC(NAT,*)
      LOGICAL   SYM(120)
      REAL*8    XAU(3,*),RAU(3,*)
      INTEGER   ITYP(*),ISC(*),IRT0(*)
      POINTER   (IP_XAU,XAU),(IP_RAU,RAU),(IP_IRT0,IRT0),
     &          (IP_ITYP,ITYP),(IP_ISC,ISC)
      REAL*8    FT(3),CM(3),AA(3,3),BB(3,3),TEMP(3,3),DELTASYM2,
     &          SUM,VS
      INTEGER   LXAU,IS,IA,IB,IC,IV,IAT,IR,I,J,K,NSYM,INVERS,INDPG0,
     &          IFIRST
      LOGICAL   TSYMMOR2,OKSYM,NODUPLI
      CHARACTER IHGP(7)*12
      DATA      IHGP
     &         /'   TRICLINIC','  MONOCLINIC','ORTHORHOMBIC',
     &          '  TETRAGONAL','       CUBIC','    TRIGONAL',
     &          '   HEXAGONAL'/,
     &          IFIRST /0/
C     ==--------------------------------------------------------------==
      IF(INDPG.EQ.0) RETURN
C     ==--------------------------------------------------------------==
      DELTASYM2=1.D-4*DELTASYM
      LXAU=3*NAT*2              !Used as scratch array for symvec
      CALL MEMORY(IP_XAU,  LXAU,              'XAU')
      CALL MEMORY(IP_RAU,  3*NAT,             'RAU')
      CALL MEMORY(IP_ITYP, NAT/IRAT + 1,      'ITYP')
      CALL MEMORY(IP_ISC,  NAT/IRAT + 1,      'ISC')
      CALL MEMORY(IP_IRT0, NAT/IRAT + 1,      'IRT0')
      IF(IFIRST.EQ.0) THEN
        IF(.NOT.TPGAUTO.AND..NOT.TSYMM) THEN
          CALL MEMORY(IP_IRT,  (120*NAT)/IRAT + 1,'IRT')
        ENDIF
        CALL MEMORY(IP_ISYMU,NAT/IRAT + 1,      'ISYMU')
      ENDIF
C     ==--------------------------------------------------------------==
      IF(TMSYM) THEN
C       MOVE CENTER OF MASS TO ORIGIN
        CALL AZZERO(CM,3)
        DO IS=1,NSP
          DO IA=1,NA(IS)
            CM(1)=CM(1)+TAU0(1,IA,IS)*PMA(IS)
            CM(2)=CM(2)+TAU0(2,IA,IS)*PMA(IS)
            CM(3)=CM(3)+TAU0(3,IA,IS)*PMA(IS)
          ENDDO
        ENDDO
        CM(1)=CM(1)/PMATOT
        CM(2)=CM(2)/PMATOT
        CM(3)=CM(3)/PMATOT
        DO IS=1,NSP
          DO IA=1,NA(IS)
            TAU0(1,IA,IS)=TAU0(1,IA,IS)-CM(1)
            TAU0(2,IA,IS)=TAU0(2,IA,IS)-CM(2)
            TAU0(3,IA,IS)=TAU0(3,IA,IS)-CM(3)
          ENDDO
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     ATOMIC COORDINATES IN DIRECT LATTICE VECTOR BASIS
      IAT=0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          IAT=IAT+1
          ITYP(IAT)=IS
          CALL DGEMV('T',3,3,1.0D0,HTM1,3,TAU0(1,IA,IS),1,0.0D0,
     *               XAU(1,IAT),1)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     Check translation vectors
      IF(TPGAUTO) THEN
C       We know the number of translation vectors:
C       Calculation of the atom transf. table
      ELSE
        NTVEC=1
        RAU(1,1)=0.D0
        RAU(2,1)=0.D0
        RAU(3,1)=0.D0
C       We do not know the number of translation vectors:
        DO IA=2,NAT
          IF(ITYP(1).EQ.ITYP(IA)) THEN
            FT(1)=XAU(1,IA)-XAU(1,1)-NINT(XAU(1,IA)-XAU(1,1))
            FT(2)=XAU(2,IA)-XAU(2,1)-NINT(XAU(2,IA)-XAU(2,1))
            FT(3)=XAU(3,IA)-XAU(3,1)-NINT(XAU(3,IA)-XAU(3,1))
            CALL CHECKSYM(NAT,ITYP,XAU,XAU,FT,.TRUE.,
     &                    OKSYM,IRT0,ISC,DELTASYM)
            IF(OKSYM) THEN
              NTVEC=NTVEC+1
              RAU(1,NTVEC)=FT(1)
              RAU(2,NTVEC)=FT(2)
              RAU(3,NTVEC)=FT(3)
            ENDIF
          ENDIF
        ENDDO
        IF(NTVEC.NE.0) THEN
          IF(IFIRST.EQ.0) THEN
            CALL MEMORY(IP_TVEC,3*NTVEC,'TVEC')
          ENDIF
          DO IR=1,NTVEC
            TVEC(1,IR)=RAU(1,IR)
            TVEC(2,IR)=RAU(2,IR)
            TVEC(3,IR)=RAU(3,IR)
          ENDDO
        ENDIF
      ENDIF
C     Calculation of IRTVEC(NTVEC,NAT):
C     Atomic tranformation table for translation vectors
      IF(IFIRST.EQ.0) THEN
        CALL MEMORY(IP_IRTVEC,NAT*NTVEC,'IRTVEC/C')
      ELSE
        CALL FREEM(IP_IRTVEC)
        CALL MEMORY(IP_IRTVEC,NAT*NTVEC,'IRTVEC/C')
      ENDIF
      DO IR=1,NTVEC
        CALL CHECKSYM(NAT,ITYP,XAU,XAU,TVEC(1,IR),
     &                .TRUE.,OKSYM,IRTVEC(1,IR),ISC,DELTASYM)
        IF(.NOT.OKSYM.AND.PARENT) THEN
          CALL STOPGM('CHKSYM','WRONG TRANSLATION VECTORS')
        ENDIF
      ENDDO
      NODUPLI=NTVEC.EQ.1
C     ==--------------------------------------------------------------==
C     LOOP OVER SYMMETRY OPERATIONS
      IF(TPGAUTO) TSYMMOR2=TSYMMORPHIC
      TSYMMORPHIC=.TRUE.
      DO IR=1,NROT
        TSYMMOR(IR)=.TRUE.
      ENDDO
      VS=0.D0
      DO IR=1,NROT
        DO IA=1,NAT
          DO K=1,3
            RAU(K,IA)=XTABLE(K,1,IR)*XAU(1,IA)+
     *                XTABLE(K,2,IR)*XAU(2,IA)+
     *                XTABLE(K,3,IR)*XAU(3,IA)
          ENDDO
        ENDDO
        IF(TPGAUTO) THEN
C         TRY TO FIND A FRACTIONAL TRANSLATION VECTOR
          IB=IRT(IR,1)
          IF(ITYP(1).EQ.ITYP(IB)) THEN
            FT(1)=XAU(1,IB)-RAU(1,1)-NINT(XAU(1,IB)-RAU(1,1))
            FT(2)=XAU(2,IB)-RAU(2,1)-NINT(XAU(2,IB)-RAU(2,1))
            FT(3)=XAU(3,IB)-RAU(3,1)-NINT(XAU(3,IB)-RAU(3,1))
            CALL CHECKSYM(NAT,ITYP,XAU,RAU,FT,NODUPLI,
     &                    SYM(IR),IRT0,ISC,DELTASYM)
            IF(SYM(IR)) THEN
              IF(ABS(FT(1))+ABS(FT(2))+ABS(FT(3)).GE.DELTASYM) THEN
                VS=VS+ABS(FT(1))+ABS(FT(2))+ABS(FT(3))
                TSYMMORPHIC=.FALSE.
                TSYMMOR(IR)=.FALSE.
              ENDIF
              GOTO 100
            ELSE
              IF(PARENT) CALL STOPGM('CHKSYM',
     &             'WRONG TRANSLATION VECTOR FROM AUTO')
            ENDIF
          ELSE
            IF(PARENT) CALL STOPGM('CHKSYM',
     &           'WRONG ATOMIC TRANSFORMATION TABLE FROM AUTO')
          ENDIF
        ELSE
          CALL AZZERO(FT,3)
C         CHECK FOR THIS SYMMETRY WITH FT=0
C         (IMPORTANT IF NOT UNIQUE ATOM)
          CALL CHECKSYM(NAT,ITYP,XAU,RAU,FT,NODUPLI,
     &                  SYM(IR),IRT0,ISC,DELTASYM)
          IF(.NOT.SYM(IR).AND..NOT.TMSYM) THEN
            DO IB=1,NAT
              IF(ITYP(1).EQ.ITYP(IB)) THEN
                FT(1)=XAU(1,1)-RAU(1,IB)-NINT(XAU(1,1)-RAU(1,IB))
                FT(2)=XAU(2,1)-RAU(2,IB)-NINT(XAU(2,1)-RAU(2,IB))
                FT(3)=XAU(3,1)-RAU(3,IB)-NINT(XAU(3,1)-RAU(3,IB))
                CALL CHECKSYM(NAT,ITYP,XAU,RAU,FT,NODUPLI,SYM(IR),
     &                        IRT0,ISC,DELTASYM)
                IF(SYM(IR)) THEN
                  VS=VS+ABS(FT(1))+ABS(FT(2))+ABS(FT(3))
                  TSYMMORPHIC=.FALSE.
                  TSYMMOR(IR)=.FALSE.
                  GOTO 100
                ENDIF
              ENDIF
            ENDDO
          ENDIF
        ENDIF
 100    CONTINUE
        IF(SYM(IR)) THEN
          IF(TPGAUTO) THEN
C           FT and FTAU should be the same for the same atomic table
C           multiplication.
            DO IA=1,NAT
              IF(IRT(IR,IA).NE.IRT0(IA)) THEN
                IF(PARENT) CALL STOPGM('CHKSYM',
     &                         'WRONG ATOMIC TRANSFORMATION TABLE')
              ENDIF
            ENDDO
            IF(  (ABS((FTAU(1,IR)-FT(1)) - NINT(FTAU(1,IR)-FT(1)))
     &           .GT.DELTASYM).OR.
     &           (ABS((FTAU(2,IR)-FT(2)) - NINT(FTAU(2,IR)-FT(2)))
     &           .GT.DELTASYM).OR.
     &           (ABS((FTAU(3,IR)-FT(3)) - NINT(FTAU(3,IR)-FT(3)))
     &           .GT.DELTASYM) ) THEN
              IF(PARENT) THEN
                WRITE(6,'(" IR=",I3,T10,"FTAU=",3F12.6)')
     &               IR,(FTAU(K,IR),K=1,3)
                WRITE(6,'(T10,"  FT=",3F12.6)') (FT(K),K=1,3)
C               FT and FTAU should be the same
                IF(PARENT)
     &               CALL STOPGM('CHKSYM','WRONG TRANSLATION VECTOR')
              ENDIF
            ENDIF
          ELSE
            DO IA=1,NAT
              IRT(IR,IA)=IRT0(IA)
            ENDDO
            FTAU(1,IR) = FT(1)
            FTAU(2,IR) = FT(2)
            FTAU(3,IR) = FT(3)
          ENDIF
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
      NSYM=0
      DO IR=1,NROT
        IF(SYM(IR))NSYM=NSYM+1
        IF(.NOT.SYM(IR)) THEN
          IF(PARENT) THEN
            WRITE(6,'(1X,"!!!",I3," OPERATION NOT CORRECT!!!")') IR
            IF(TPGAUTO) CALL STOPGM('CHKSYM',
     &           'WRONG AUTOMATIC DETERMINATION OF SYMMETRY')
          ENDIF
        ENDIF
      ENDDO
      IF(NSYM.NE.NROT) THEN
        IF(PARENT) THEN
          WRITE(6,'(/,1X,64("!"))')
          WRITE(6,'(" !!",A,T64,"!!")') 
     &         ' CHKSYM| NUMBER OF SYMMETRY OPERATIONS HAS BEEN CHANGED'
          WRITE(6,'(" !!",T13,A,I2,A,I2,T64,"!!")') 
     &         'FROM ',NROT,' TO ',NSYM
          WRITE(6,'(1X,64("!"))')
        ENDIF
        IS=0
        DO IR=1,NROT
          IF(SYM(IR)) THEN
            IS=IS+1
            TSYMMOR(IS)=TSYMMOR(IR)
            DO J=1,3
              DO I=1,3
                XTABLE(I,J,IS)=XTABLE(I,J,IR)
              ENDDO
            ENDDO
            DO I=1,3
              FTAU(I,IS)=FTAU(I,IR)
            ENDDO
            DO I=1,NAT
              IRT(IS,I)=IRT(IR,I)
            ENDDO
          ENDIF
        ENDDO
        NROT=NSYM
        INDPG=-INDPG
      ENDIF
C     ==--------------------------------------------------------------==
C     Check if there is inversion symmetry
      INVERS=0
      DO IR=1,NROT
        IF(  (ABS(XTABLE(1,1,IR)+1.D0).LT.DELTASYM2).AND.
     &       (ABS(XTABLE(2,2,IR)+1.D0).LT.DELTASYM2).AND.
     &       (ABS(XTABLE(3,3,IR)+1.D0).LT.DELTASYM2) ) THEN
          SUM=   ABS(XTABLE(1,2,IR))
     &         + ABS(XTABLE(1,3,IR))
     &         + ABS(XTABLE(2,1,IR))
     &         + ABS(XTABLE(2,3,IR))
     &         + ABS(XTABLE(3,1,IR))
     &         + ABS(XTABLE(3,2,IR))
          IF(SUM.LT.DELTASYM2) THEN
            INVERS=IR
          ENDIF
        ENDIF
      ENDDO
      IF(TPGAUTO) THEN
        IF(INVERS.NE.INVERSION) THEN
          IF(PARENT) CALL STOPGM('CHKSYM',
     &       'INCORRECT DETERMINATION OF INVERSION')
        ENDIF
      ENDIF
      INVERSION=INVERS
C     ==--------------------------------------------------------------==
C     Look for center of symmetry (ORIGIN)
      IF(TPGAUTO) THEN
        IF(TSYMMORPHIC.AND.TORIGIN) THEN
          IF(  (TSYMMORPHIC.AND..NOT.TSYMMOR2).OR.
     &         (.NOT.TSYMMORPHIC.AND.TSYMMOR2) ) THEN
            IF(PARENT) CALL STOPGM('CHKSYM',
     &           'INCORRECT DETERMINATION OF SPACE GROUP TYPE')
          ENDIF
        ENDIF
      ELSE
        IF(TSYMMORPHIC) THEN
          TORIGIN=.TRUE.
        ELSE
          TORIGIN=.FALSE.
        ENDIF
        DO I=1,3
          ORIGIN(I)=0.D0
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     CONSTRUCTION OF FTABLE (ROTATION MATRICES FOR FOURIER SPACE)
C     `B.A0 = 1 (`B=Transpose of B) [HT=A=`B, HTM1= B=`A]
C     Put in reciprocal coordinates : 
C     FTABLE = `B.`A.XTABLE.A.B = A.`A.XTABLE.`B.B
      CALL DGEMM('N','T',3,3,3,1.D0/(ALAT*ALAT),
     &     HT(1,1),3,HT(1,1),3,0.D0,AA(1,1),3)
      CALL DGEMM('T','N',3,3,3,(ALAT*ALAT),
     &     HTM1(1,1),3,HTM1(1,1),3,0.D0,BB(1,1),3)
      DO IR=1,NROT
        CALL DGEMM('N','N',3,3,3,1.D0,
     &       XTABLE(1,1,IR),3,BB(1,1),3,0.D0,TEMP(1,1),3)
        CALL DGEMM('N','N',3,3,3,1.D0,
     &       AA(1,1),3,TEMP(1,1),3,0.D0,FTABLE(1,1,IR),3)
      ENDDO
C     ==--------------------------------------------------------------==
C     LOOK FOR SYMMETRY UNIQUE ATOMS
      DO IA=1,NAT
        ISYMU(IA)=1
      ENDDO
C     Unique atoms for rotations
      DO IA=1,NAT
        IF(ISYMU(IA).NE.0) THEN
          DO IR=1,NROT
            IB=IRT(IR,IA)
            IF(IB.GT.IA) THEN
              IF(ISYMU(IB).NE.0) THEN
                ISYMU(IA)=ISYMU(IA)+1
                ISYMU(IB)=0
              ENDIF
            ENDIF
C           Unique atoms for translation vectors
            IF(NTVEC.NE.1) THEN
              DO IV=1,NTVEC
                IC=IRTVEC(IB,IV)
                IF(IC.GT.IA) THEN
                  IF(ISYMU(IC).NE.0) THEN
                    ISYMU(IA)=ISYMU(IA)+1
                    ISYMU(IC)=0
                  ENDIF
                ENDIF
              ENDDO
            ENDIF
          ENDDO
        ENDIF
      ENDDO
      IUNIQUE=0
      DO IA=1,NAT
        IF(ISYMU(IA).NE.0) IUNIQUE=IUNIQUE+1
      ENDDO
C     ==--------------------------------------------------------------==
      IF(TMSYM) THEN
        DO IS=1,NSP
          DO IA=1,NA(IS)
            TAU0(1,IA,IS)=TAU0(1,IA,IS)+CM(1)
            TAU0(2,IA,IS)=TAU0(2,IA,IS)+CM(2)
            TAU0(3,IA,IS)=TAU0(3,IA,IS)+CM(3)
          ENDDO
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      IF(.NOT.TPGAUTO) THEN
        INDPG0=ABS(INDPG)
        IF(INDPG0.GE.1.AND.INDPG0.LE.2) THEN
          IHG=1
        ELSEIF(INDPG0.GE.3.AND.INDPG0.LE.5) THEN
          IHG=2
        ELSEIF(INDPG0.GE.6.AND.INDPG0.LE.10) THEN
          IHG=6
        ELSEIF(INDPG0.GE.11.AND.INDPG0.LE.17) THEN
          IHG=4
        ELSEIF(INDPG0.GE.18.AND.INDPG0.LE.24) THEN
          IHG=7
        ELSEIF(INDPG0.GE.25.AND.INDPG0.LE.27) THEN
          IHG=3
        ELSEIF(INDPG0.GE.28.AND.INDPG0.LE.32) THEN
          IHG=5
        ELSEIF(INDPG0.EQ.100) THEN
          IHG=100
        ELSE
          CALL STOPGM('CHKSYM','BAD GROUP NUMBER')
        ENDIF
        IF(PARENT) THEN
          IF(IBRAV.EQ.4 .AND. NROT.EQ.24) THEN
            WRITE(6,'(/,A,A)')
     &           ' THE POINT GROUP OF THE CRYSTAL IS THE FULL ',
     &           'HEXAGONAL GROUP'
          ELSEIF(IBRAV.EQ.1 .AND. NROT.EQ.48) THEN
            WRITE(6,'(/,A,A)')
     &           ' THE POINT GROUP OF THE CRYSTAL IS THE FULL ',
     &           'CUBIC GROUP'
          ELSEIF(IHG.EQ.100) THEN
            CALL XSTRING(IHGP(IHG),IA,IB)
            WRITE(6,'(/,A,I2,A)')
     &           ' MOLECULAR SYSTEM  WITH ',NROT,' OPERATIONS'
          ELSE
            CALL XSTRING(IHGP(IHG),IA,IB)
            WRITE(6,'(/,A,A,A,I2,A)')
     &           ' THE CRYSTAL SYSTEM IS ',
     &           IHGP(IHG)(IA:IB),' WITH ',NROT,' OPERATIONS'
          ENDIF
          IF(IHG.NE.100) THEN
            IF(TSYMMORPHIC) THEN
              WRITE(6,'(A)')
     &           ' THE SPACE GROUP OF THE CRYSTAL IS SYMMORPHIC'
            ELSE
              WRITE(6,'(A,/,A,A,/,3X,A,F15.6,A)')
     &           ' THE SPACE GROUP IS NON-SYMMORPHIC,',
     &           ' OR ELSE A NON STANDARD ORIGIN OF COORDINATES WAS',
     &           ' USED.',
     &           ' (SUM OF TRANSLATION VECTORS=',VS,')'
            ENDIF
            IF(NTVEC.EQ.1) THEN
              WRITE(6,'(A,T60,I6)')
     &           ' NUMBER OF PRIMITIVE CELL:',NTVEC
            ELSE
              WRITE(6,'(A,T60,I6)')
     &           ' NUMBER OF PRIMITIVE CELLS:',NTVEC
            ENDIF
            IF(INVERSION.NE.0) WRITE(6,'(1X,A)')
     &        'THE POINT GROUP OF THE CRYSTAL CONTAINS THE INVERSION'
          ENDIF
          WRITE(6,*)
        ENDIF
      ENDIF
      IF(PARENT) THEN
        IF(IUNIQUE.EQ.1) THEN
          WRITE(6,'(A,T61,I5)')
     &         ' SYMMETRY UNIQUE (INEQUIVALENT) ATOM:',IUNIQUE
        ELSE
          WRITE(6,'(A,T61,I5)')
     &         ' SYMMETRY UNIQUE (INEQUIVALENT) ATOMS:',IUNIQUE
          IF(IUNIQUE.NE.NAT) THEN
            WRITE(6,'(A,6X,$)') ' INDEXES:'
            J=0
            DO I=1,NAT
              IF(ISYMU(I).NE.0) THEN
                J=J+1
                IF(J.LE.10) THEN
                  WRITE(6,'(I4,$)') I
                ELSE
                  WRITE(6,'(/,15X,I4,$)') I
                  J=0
                ENDIF
              ENDIF
            ENDDO
            WRITE(6,*)
          ENDIF
        ENDIF
        WRITE(6,'(A,T56,1PE10.2)')
     &       ' REQUIRED PRECISION FOR SYMMETRY:',DELTASYM
        WRITE(6,*)
      ENDIF
C     ==--------------------------------------------------------------==
      IF(IFIRST.EQ.0) THEN
        IFIRST=1
      ENDIF
      CALL FREEM(IP_XAU)
      CALL FREEM(IP_RAU)
      CALL FREEM(IP_ITYP)
      CALL FREEM(IP_ISC)
      CALL FREEM(IP_IRT0)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SYMTAU(TAU0)
C     ==--------------------------------------------------------------==
C     == Symmetrisation of atomic coordinates from symmetry operations==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'symm.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'irat.inc'
C     Arguments
      REAL*8    TAU0(3,NAX,NSP)
C     Variables
      LOGICAL   SYM(120)
      INTEGER   ITYP(NAT),ISC(NAT),IRT0(NAT)
      REAL*8    XAU(3,2*NAX*NSX),RAU(3,2*NAX*NSX)
      POINTER   (IP_XAU,XAU),(IP_RAU,RAU),(IP_IRT0,IRT0),
     &          (IP_ITYP,ITYP),(IP_ISC,ISC)
      REAL*8    CM(3),FT(3),DISP,VS
      REAL*8    DGIVE
      EXTERNAL  DGIVE
      INTEGER   IS,IA,IAT,IR,K,IB,NSYM,IMAX,LXAU,
     &          IDAMAX,IFIRST
      DATA IFIRST /0/
      SAVE IFIRST
C     ==--------------------------------------------------------------==
      IF(.NOT.TSYMM) RETURN
      IF(INDPG.EQ.0) RETURN
      IF(PARENT) WRITE(6,'(A)')
     &     ' SYMTAU| SYMMETRIZATION OF ATOMIC COORDINATES '
C     ==--------------------------------------------------------------==
      LXAU=6*NAX*NSX
      CALL MEMORY(IP_XAU, LXAU,              'XAU')
      CALL MEMORY(IP_RAU, LXAU,              'RAU')
      CALL MEMORY(IP_ITYP,NAT/IRAT + 1,      'ITYP')
      CALL MEMORY(IP_ISC, NAT/IRAT + 1,      'ISC')
      CALL MEMORY(IP_IRT0, NAT/IRAT + 1,      'IRT0')
C     ==--------------------------------------------------------------==
      IF(IFIRST.EQ.0) THEN
        IF(.NOT.TPGAUTO) THEN
          CALL MEMORY(IP_IRT,  (120*NAT)/IRAT + 1,'IRT')
        ENDIF
      ENDIF
      IFIRST=1
C     ==--------------------------------------------------------------==
C     MOVE CENTER OF MASS TO ORIGIN
      CALL AZZERO(CM,3)
      DO IS=1,NSP
        DO IA=1,NA(IS)
          CM(1)=CM(1)+TAU0(1,IA,IS)*PMA(IS)
          CM(2)=CM(2)+TAU0(2,IA,IS)*PMA(IS)
          CM(3)=CM(3)+TAU0(3,IA,IS)*PMA(IS)
        ENDDO
      ENDDO
      CM(1)=CM(1)/PMATOT
      CM(2)=CM(2)/PMATOT
      CM(3)=CM(3)/PMATOT
      DO IS=1,NSP
        DO IA=1,NA(IS)
          TAU0(1,IA,IS)=TAU0(1,IA,IS)-CM(1)
          TAU0(2,IA,IS)=TAU0(2,IA,IS)-CM(2)
          TAU0(3,IA,IS)=TAU0(3,IA,IS)-CM(3)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     ATOMIC COORDINATES IN DIRECT LATTICE VECTOR BASIS
      IAT=0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          IAT=IAT+1
          ITYP(IAT)=IS
          CALL DGEMV('T',3,3,1.0D0,HTM1,3,TAU0(1,IA,IS),1,0.0D0,
     *               XAU(1,IAT),1)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     LOOP TO FIND MATCHING TOLERANCE
 200  CONTINUE
C     ==--------------------------------------------------------------==
C     LOOP OVER SYMMETRY OPERATIONS
      CALL AZZERO(FTAU,3*NROT)
      VS=0.D0
      TSYMMORPHIC=.TRUE.
      DO IR=1,NROT
        TSYMMOR(IR)=.TRUE.
      ENDDO
      DO IR=1,NROT
        DO IA=1,NAT
          DO K=1,3
            RAU(K,IA)=XTABLE(K,1,IR)*XAU(1,IA)+
     *                XTABLE(K,2,IR)*XAU(2,IA)+
     *                XTABLE(K,3,IR)*XAU(3,IA)
          ENDDO
        ENDDO
        CALL AZZERO(FT,3)
C       CHECK FOR THIS SYMMETRY WITH FT=0 (IMPORTANT IF NOT UNIQUE ATOM)
        CALL CHECKSYM(NAT,ITYP,XAU,RAU,FT,.TRUE.,SYM(IR),IRT0,
     &                ISC,DELTASYM)
        IF(.NOT.SYM(IR).AND..NOT.TMSYM) THEN
C         TRY TO FIND A FRACTIONAL TRANSLATION VECTOR
          DO IB=1,NAT
            IF(ITYP(1).EQ.ITYP(IB)) THEN
              FT(1)=XAU(1,1)-RAU(1,IB)-NINT(XAU(1,1)-RAU(1,IB))
              FT(2)=XAU(2,1)-RAU(2,IB)-NINT(XAU(2,1)-RAU(2,IB))
              FT(3)=XAU(3,1)-RAU(3,IB)-NINT(XAU(3,1)-RAU(3,IB))
              CALL CHECKSYM(NAT,ITYP,XAU,RAU,FT,.TRUE.,
     &                      SYM(IR),IRT0,ISC,DELTASYM)
              IF(SYM(IR)) THEN
                VS=VS+ABS(FT(1))+ABS(FT(2))+ABS(FT(3))
                TSYMMORPHIC=.FALSE.
                TSYMMOR(IR)=.FALSE.
                GOTO 100
              ENDIF
            ENDIF
          ENDDO
        ENDIF
 100    CONTINUE
        IF(SYM(IR)) THEN
          DO IAT=1,NAT
            IRT(IR,IAT)=IRT0(IAT)
          ENDDO
          FTAU(1,IR) = FT(1)
          FTAU(2,IR) = FT(2)
          FTAU(3,IR) = FT(3)
        ENDIF
      ENDDO
      NSYM=0
      DO IR=1,NROT
        IF(SYM(IR))NSYM=NSYM+1
      ENDDO
      IF(NSYM.EQ.NROT) GOTO 300
      IF(ALAT.GT.2.D0*DELTASYM) THEN
        IF(PARENT) THEN
          WRITE(6,'(A)') 
     *         ' SYMTAU! FAILED TO SYMMETRIZE ATOMIC COORDINATES '
          CALL STOPGM('SYMTAU',' ERROR ')
        ENDIF
      ENDIF
      GOTO 200
 300  CONTINUE
      IF(PARENT) WRITE(6,'(A,G10.3)') 
     *     ' SYMTAU! TOLERANCE NEEDED FOR FULL SYMMETRY      :',
     &     DELTASYM
      CALL DCOPY(3*NAX*NSX,TAU0(1,1,1),1,RAU(1,1),1)
      CALL SYMVEC(TAU0,XAU,LXAU)
      DISP=0.0D0
      CALL DAXPY(3*NAX*NSX,-1.0D0,TAU0(1,1,1),1,RAU(1,1),1)
      IMAX=IDAMAX(3*NAX*NSX,RAU(1,1),1)
      DISP=ABS(DGIVE(RAU(1,1),IMAX))
      IF(PARENT) WRITE(6,'(A,G10.3)') 
     *     ' SYMTAU! LARGEST DISPLACEMENT FOR SYMMETRIZATION :',DISP 
      DO IS=1,NSP
        DO IA=1,NA(IS)
          TAU0(1,IA,IS)=TAU0(1,IA,IS)+CM(1)
          TAU0(2,IA,IS)=TAU0(2,IA,IS)+CM(2)
          TAU0(3,IA,IS)=TAU0(3,IA,IS)+CM(3)
        ENDDO
      ENDDO
      CALL FREEM(IP_XAU)
      CALL FREEM(IP_RAU)
      CALL FREEM(IP_ITYP)
      CALL FREEM(IP_IRT0)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GENTAU(TAU0)
C     ==--------------------------------------------------------------==
C     == Generate atomic coordinates from unique atoms                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'symm.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'irat.inc'
C     Arguments
      REAL*8    TAU0(3,NAX,NSP)
C     Variables
      REAL*8    XAU(3,NAX*NSX),RAU(3,NAX*NSX)
      POINTER   (IP_XAU,XAU),(IP_RAU,RAU)
      REAL*8    D1,D2,D3,DD
      INTEGER   IAT,IS,IA,IB,K,IR,IATN
C     ==--------------------------------------------------------------==
      IF(.NOT.TGENC) RETURN
      IF(INDPG.EQ.0) RETURN
      IF(PARENT) WRITE(6,'(/,A,A)')
     &     ' GENERATE ATOMIC COORDINATES FROM ',
     *     'SYMMETRY UNIQUE ONES'
      CALL MEMORY(IP_XAU,3*NAX*NSX,'XAU')
      CALL MEMORY(IP_RAU,3*NAX*NSX,'RAU')
C     LOOP OVER SPECIES AND SYMMETRY UNIQUE ATOMS
      IAT=0
      DO IS=1,NSP
C       ATOMIC COORDINATES IN DIRECT LATTICE VECTOR BASIS
        DO IA=1,NA(IS)
          CALL DGEMV('T',3,3,1.0D0,HTM1,3,TAU0(1,IA,IS),1,0.0D0,
     *               XAU(1,IA),1)
          DO K=1,3
            XAU(K,IA)=XAU(K,IA)-NINT(XAU(K,IA))
          ENDDO
        ENDDO
        IAT=IUN(IS)
        DO IR=1,NROT
          DO IA=1,IAT
            DO K=1,3
              RAU(K,IA)=XTABLE(K,1,IR)*XAU(1,IA)+
     *                  XTABLE(K,2,IR)*XAU(2,IA)+
     *                  XTABLE(K,3,IR)*XAU(3,IA)
              RAU(K,IA)=RAU(K,IA)-NINT(RAU(K,IAT))
            ENDDO
          ENDDO
          IATN=IAT
          DO IA=1,IAT
            DO IB=1,IATN
              D1=ABS(RAU(1,IA)-XAU(1,IB))
              D2=ABS(RAU(2,IA)-XAU(2,IB))
              D3=ABS(RAU(3,IA)-XAU(3,IB))
              DD=SQRT(D1*D1+D2*D2+D3*D3)
              IF(DD.LT.DELTASYM) GOTO 100
            ENDDO
C           NEW ATOM
            IATN=IATN+1
            CALL DCOPY(3,RAU(1,IA),1,XAU(1,IATN),1)
  100       CONTINUE
          ENDDO
          IAT=IATN
        ENDDO
C       CHECK THAT ALL ATOMS HAVE BEEN GENERATED
        IF(NA(IS).NE.IAT) THEN
          IF(PARENT) THEN
            WRITE(6,'(A,T61,I5)')
     &           ' GENTAU| INCONSISTENCY FOR SPECIES:',IS
            WRITE(6,'(A,T51,I5,A5,I5)')
     &           ' GENTAU| NUMBER OF ATOMS:',IAT,' NOT ',NA(IS)
            CALL STOPGM('GENTAU',' CHECK INPUT ')
          ENDIF
        ENDIF
C       ATOMIC COORDINATES IN CARTESIAN COORDINATES
        DO IA=1,NA(IS)
          CALL DGEMV('T',3,3,1.0D0,HT,3,XAU(1,IA),1,0.0D0,
     *               TAU0(1,IA,IS),1)
        ENDDO
      ENDDO
      CALL FREEM(IP_XAU)
      CALL FREEM(IP_RAU)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE UPDATSYM(TAU0)
C     ==--------------------------------------------------------------==
C     == UPDATE SYMMETRY OPERATIONS                                   ==
C     == (GEOMETRY OPTIMISATION OR MOLECULAR DYNAMICS)                ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   TAU0(1:3,NAX,NSX) Atomic coordinates                       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     ==--------------------------------------------------------------==
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'        !symm.inc
      INCLUDE 'symm.inc'        !INDPG,NROT,XTABLE,INVE
C     Arguments
      REAL*8  TAU0(3,NAX,NSX)
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
      IF(TPGAUTO) CALL GIVESYM(TAU0)
C     Initialisation for all processors
      IF(INDPG.EQ.0) THEN
        NROT=1
        INVERSION=0
        TORIGIN=.FALSE.
        DO I=1,3
          ORIGIN(I)=0.D0
        ENDDO
        RETURN
      ENDIF
C     ==--------------------------------------------------------------==
      CALL CHKSYM(TAU0)
C     Construct table multiplication.
      CALL MULTTB(INDPG,NROT,NTVEC,NAT,XTABLE,INVE,MULTAB,IRT,.FALSE.)
C     Number of equivalent shell (0 for serial version)
C     Use for symmetrisation of density (force call to BUILDSHELL)
      NUMSHEL=0
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CHECKSYM(NAT,ITYP,XAU,RAU,FT,NODUPLI,SYM,IRT,ISC,DELTA)
C     ==--------------------------------------------------------------==
C     == CHECK FOR EACH ATOM IA=1:NAT THAT THE IMAGE ATOM RAU(3,1:NAT)==
C     == GIVEN BY THE IR ROTATION OPERATION WITH THE TRANSLATION      ==
C     == VECTOR FT(3) CORRESPONDS TO AN ATOM                          ==
C     == BUILD ALSO THE ATOMIC TRANSFORMATION TABLE                   ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   NAT         Number of atoms                                ==
C     ==   ITYP(NAT)   Type of atoms                                  ==
C     ==   XAU(3,NAT)  Atomic coordinates                             ==
C     ==   RAU(3,NAT)  Images from pure rotation of atomic coordinates==
C     ==   FT(3)       Translation vector                             ==
C     ==   NODUPLI     .TRUE. the cell is duplicated, we can speed up ==
C     ==   DELTA       Precision required                             ==
C     == OUTPUT:                                                      ==
C     ==   SYM         .TRUE. if everything is o.k.                   ==
C     ==   IRT(NAT)    Atomic transformation table                    ==
C     ==   ISC(NAT)    Scratch array                                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER  NAT,ITYP(NAT),IRT(NAT),ISC(NAT)
      REAL*8   XAU(3,NAT), RAU(3,NAT), FT(3), DELTA
      LOGICAL  SYM,NODUPLI
C     Variables
      INTEGER  IA,IB
      LOGICAL  EQVECT
      EXTERNAL EQVECT
C     ==--------------------------------------------------------------==
      DO IA=1,NAT
        ISC(IA)=0
      ENDDO
      DO 10 IA = 1, NAT
        SYM = .FALSE.
        DO IB = 1, NAT
          IF(ITYP(IA).EQ.ITYP(IB).AND.ISC(IB).EQ.0) THEN
            SYM = EQVECT(RAU(1,IA),XAU(1,IB),FT,DELTA)
            IF(SYM) THEN
C             THE ROTATED ATOM DOES COINCIDE WITH ONE OF THE LIKE ATOMS
C             KEEP TRACK OF WHICH ATOM THE ROTATED ATOM COINCIDES WITH
              IRT(IA)=IB
              IF(NODUPLI) ISC(IB)=1
              GO TO 10
            END IF
          ENDIF
        ENDDO
C       THE ROTATED ATOM DOES NOT COINCIDE WITH ANY OF THE LIKE ATOMS
C       S(IR) + FT IS NOT A SYMMETRY OPERATION
        RETURN
 10   CONTINUE
C     S(IR) + FT IS A SYMMETRY OPERATION
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION EQVECT(X,Y,F,DELTA)
C     ==--------------------------------------------------------------==
C     == TEST IF Y = X + F (VECTOR)                                   ==
C     == DELTA GIVES THE REQUIRED PRECISION                           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8 X(3), Y(3), F(3), DELTA
      LOGICAL EQVECT
C     ==--------------------------------------------------------------==
      EQVECT = ABS(X(1)-Y(1)+F(1)-NINT(X(1)-Y(1)+F(1))).LT. DELTA  .AND.
     &         ABS(X(2)-Y(2)+F(2)-NINT(X(2)-Y(2)+F(2))).LT. DELTA  .AND.
     &         ABS(X(3)-Y(3)+F(3)-NINT(X(3)-Y(3)+F(3))).LT. DELTA 
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
