C     ==================================================================
      SUBROUTINE CL_INIT(IUNIT)
C     ==--------------------------------------------------------------==
C     ==  THIS ROUTINE READS THE SECTION &CLASSIC &END                ==
C     ==--------------------------------------------------------------==
C     ==  THE INPUT HAS TO BE IN THE FOLLOWING FORMAT                 ==
C     ==     &CLASSIC                                                 ==
C     ==        Options                                               ==
C     ==     &END                                                     ==
C     ==--------------------------------------------------------------==
C     ==  LIST OF OPTIONS                                             ==
C     ==                                                              ==
C     ==    FORCE FIELD                                               ==
C     ==      ......                                                  ==
C     ==    END FORCE FIELD                                           ==
C     ==                                                              ==
C     ==    FREEZE QUANTUM                                            ==
C     ==    PRINT COORDINATES                                         ==
C     ==    PRINT FF                                                  ==
C     ==    FULL TRAJECTORY                                           ==
C     ==                                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'clas.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'irat.inc'
      INTEGER IUNIT,MSGLEN,IERR,INSCAN,I,J,IS
      CHARACTER*80 LINE
      REAL*8 XAT,XSNOW,XSAIM
      INTEGER N1,N2
C     ==--------------------------------------------------------------==
C     ==  DEFAULTS                                                    ==
C     ==--------------------------------------------------------------==
      TFREEZE=.FALSE.
      TWCOOR=.FALSE.
      TPFF=.FALSE.
      TFTRA=.FALSE.
      TDEFO=.FALSE.
      TMOVC=.FALSE.
c NTRAC frequency of writing FF trajectory
c NDEFO frequency of applying deformation
      NTRAC=1
      NDEFO=1
      IMOVC=1
C     ==--------------------------------------------------------------==
C     ==  FORCE FIELDS (DEFAULTS)                                     ==
C     ==--------------------------------------------------------------==
      T_R12=.FALSE.
      INITR12=0
C     ==--------------------------------------------------------------==
      IF(PARENT) THEN
C     ===--------------------------------------------------------------==
        CALL CLEAN_LABEL
        IERR=INSCAN(IUNIT,'&CLAS')
        IF(IERR.NE.0) GOTO 100
C     ==--------------------------------------------------------------==
 10     CONTINUE
        READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
        IF(INDEX(LINE,'&END').NE.0) GOTO 100
        IF(INDEX(LINE,'FORCE').NE.0 .AND. INDEX(LINE,'FIELD').NE.0)THEN
C..Classical Force Field
          CALL READFF(IUNIT)
          GOTO 10
        ELSE IF (INDEX(LINE,'FREEZE').NE.0 
     *          .AND. INDEX(LINE,'QUANT').NE.0) THEN
C..freeze quantum particles
          TFREEZE=.TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'PRINT').NE.0
     *          .AND. INDEX(LINE,'COORD').NE.0) THEN
C..print all coordinates
          TWCOOR=.TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'PRINT').NE.0
     *          .AND. INDEX(LINE,'FF').NE.0) THEN
C..print force fields
          TPFF=.TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'FULL').NE.0
     *          .AND. INDEX(LINE,'TRAJ').NE.0) THEN
C..save full trajectory

          TFTRA=.TRUE.
          IF(INDEX(LINE,'SAMPLE').NE.0) THEN
            READ(IUNIT,ERR=99,END=99,FMT=*) NTRAC
          ENDIF
          GOTO 10
C     ==--------------------------------------------------------------==
          ELSE IF(INDEX(LINE,'MOVIE').NE.0) THEN 
          TMOVC=.TRUE.
          READ(IUNIT,ERR=99,END=99,FMT=*) IMOVC
          GOTO 10
C     ==--------------------------------------------------------------==
          ELSE 
          IF(INDEX(LINE,'DEFORM').NE.0) THEN 
          TDEFO=.TRUE.
           IF(INDEX(LINE,'SAMPLE').NE.0) THEN
            READ(IUNIT,ERR=99,END=99,FMT=*) NDEFO 
           ENDIF
           READ(IUNIT,ERR=99,END=99,FMT=*) EXPX, EXPY, EXPZ
           READ(IUNIT,ERR=99,END=99,FMT=*) XCEN, YCEN, ZCEN 
         ENDIF
C..Dummy line
          GOTO 10
        ENDIF
C     ==--------------------------------------------------------------==
 99     CONTINUE
        WRITE(6,*) ' CL_INIT : ERROR IN READING INPUT FILE'
        CALL STOPGM('CL_INIT',' ')
C     ==--------------------------------------------------------------==
 100    CONTINUE
C     ==--------------------------------------------------------------==
C     ==  Test of options                                             ==
C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     ==  WRITE INFO TO OUTPUT                                        ==
C     ==--------------------------------------------------------------==
        WRITE(6,*)
        IF(TFREEZE) WRITE(6,'(A)') ' QUANTUM PARTICLES WILL NOT MOVE'
        IF(TFTRA) WRITE(6,'(A,I6,A)') 
     *  ' FULL TRAJECTORY WILL BE SAVED EVERY', 
     *   NTRAC,' STEPS'
        IF(TMOVC) WRITE(6,'(A,I6,A)') 
     *  ' FULL TRAJECTORY WILL BE SAVED EVERY', 
     *   IMOVC,' STEPS IN MOVIE FORMAT'
        IF(TDEFO) WRITE(6,'(A,I6,A)') 
     *  ' DEFORMATION WILL BE APPLIED EVERY', 
     *   NDEFO,' STEPS'
        IF(TPFF) THEN
          IF(T_R12) THEN
            WRITE(6,'(A)') ' Force field of type: (a/R)^12'
            DO I=1,NCLTYP
              DO J=1,I
                WRITE(6,'(6X,A,A,5X,A,A,5X,A,F10.4,A,F8.2)')
     *            'ATOM=',CLAB(I),'ATOM=',CLAB(J),
     *            'PARAM=',PR12(I,J),'   RC=',RCR12(I,J)
              ENDDO
            ENDDO
          ENDIF
        ENDIF
        WRITE(6,*)
C     ==--------------------------------------------------------------==
      ENDIF
C..distribute data
      MSGLEN = NOPTCL * 8/IRAT
      CALL MY_BCAST(TFREEZE,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(TDEFO,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(NDEFO,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 8
      CALL MY_BCAST(XCEN,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(YCEN,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(ZCEN,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EXPX,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EXPY,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EXPZ,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(T_R12,MSGLEN,SOURCE,ALLGRP)
      IF(T_R12) THEN
        IF(INITR12.EQ.0) THEN
          CALL MEMORY(IP_PR12,NCLTYP*NCLTYP,'PR12')
          CALL MEMORY(IP_RCR12,NCLTYP*NCLTYP,'RCR12')
          INITR12=1
        ENDIF
        MSGLEN = NCLTYP*NCLTYP * 8
        CALL MY_BCAST(PR12,MSGLEN,SOURCE,ALLGRP)
        MSGLEN = NCLTYP*NCLTYP * 8
        CALL MY_BCAST(RCR12,MSGLEN,SOURCE,ALLGRP)
      ENDIF
C     ==--------------------------------------------------------------==
C..distribute work for parallel jobs
      XAT=DBLE(NCLATOM)
      XSNOW=0.0D0
      DO I=NPROC,1,-1
        XSAIM = XSNOW + XAT/NPROC
        N1=NINT(XSNOW)+1
        N2=NINT(XSAIM)
        IF(NINT(XSAIM).GT.NCLATOM) N2=NCLATOM
        IF(I.EQ.1) N2=NCLATOM
        XSNOW = XSAIM
        IF(I-1.EQ.MEPOS) THEN
          MYCLAT1=N1
          MYCLAT2=N2
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
C..update mass information
      DO I=1,NCLTYP
        IS=IS_QM(I)
        IF(IS.GT.0) CLMAS(I)=PMA0(IS)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CLEAN_LABEL
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'clas.inc'
      INTEGER I,I1,I2,LEN
      CHARACTER*4 TEMP
C     ==--------------------------------------------------------------==
      DO I=1,NCLTYP
        TEMP='    '
        CALL XSTRING(CLAB(I),I1,I2)
        LEN=I2-I1+1
        IF(LEN.GT.0) THEN
          TEMP(1:LEN)=CLAB(I)(I1:I2)
        ENDIF
        CLAB(I)=TEMP
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
