C     ==================================================================
      SUBROUTINE CLINBCG(B,X,C0,VPOT,PSI,SCR,LSCR,
     &                   NSTATE,F,ITOL,TOL,ITMAX,ERR)
C     ==--------------------------------------------------------------==
C     == Conjugate gradient method for a sparse system                ==
C     == Numerical recipes p 79 2nd edition (LINBCG routine)          ==
C     == (A. Alavi 1997)                                              ==
C     == USES ATIMES,DIAGO,SNRMM                                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'kpts.inc'
      INCLUDE 'spin.inc'
C     Arguments
      INTEGER    NSTATE,ITOL,ITMAX,LSCR
      REAL*8     TOL,ERR,F(*),VPOT(NNR1,*),SCR(*)
      COMPLEX*16 B(*),X(*),C0(*),PSI(*)
C     Variables
      COMPLEX*16 ZZERO
      REAL*8     EPS
      PARAMETER  (EPS=1.D-14,ZZERO=(0.D0,0.D0))
      COMPLEX*16 R(*),RR(*),Z(*),ZZ(*),P(*),PP(*)
      POINTER    (IP_R,R),(IP_RR,RR),(IP_Z,Z),(IP_ZZ,ZZ),
     &           (IP_P,P),(IP_PP,PP)
      CHARACTER  TAG*30
      COMPLEX*16 BKNUM,AKDEN,BKDEN,AK,BK
      REAL*8     BNRM,DXNRM,XNRM,ZM1NRM,ZNRM,SNRMM
      COMPLEX*16 ZDOTC
      EXTERNAL   ZDOTC
      REAL*8     DOTP
      EXTERNAL   DOTP
      INTEGER    J,LCLINBCG,ITER,
     &           IFIRST
      DATA       IFIRST/0/
      SAVE       IP_R,IP_RR,IP_Z,IP_ZZ,IP_P,IP_PP,
     &           IFIRST
      LOGICAL    TDEBUG 
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_CLINBCG(LCLINBCG,TAG,NSTATE)
      CALL TEST_SCR('CLINBCG',TAG,LSCR,LCLINBCG)
C     ==--------------------------------------------------------------==
      TDEBUG=.TRUE. 
      ITER=0
      IF(IFIRST.EQ.0) THEN
        CALL MEMORY(IP_R,2*NGWK,'R')
        CALL MEMORY(IP_RR,2*NGWK,'RR')
        CALL MEMORY(IP_Z,2*NGWK,'Z')
        CALL MEMORY(IP_ZZ,2*NGWK,'ZZ')
        CALL MEMORY(IP_P,2*NGWK,'P')
        CALL MEMORY(IP_PP,2*NGWK,'PP')
        IFIRST=1
      ENDIF
      CALL ATIMES(X,R,C0,VPOT,PSI,SCR,LSCR,NSTATE,F)
!$OMP parallel do private(J)
      DO J=1,NGWK
        R(J)=B(J)-R(J)
        RR(J)=R(J)
      ENDDO
C     CALL ATIMES(R,RR,C0,NSTATE,F)
      ZNRM=1.D0
      IF(ITOL.EQ.1) THEN
         BNRM=SNRMM(NGWK,B)  
      ELSE IF (ITOL.EQ.2) THEN
        CALL DIAGO(B,Z)
        BNRM=SNRMM(NGWK,Z)  
      ELSE IF (ITOL.EQ.3.OR.ITOL.EQ.4) THEN
        CALL DIAGO(B,Z)
        BNRM=SNRMM(NGWK,Z)  
        CALL DIAGO(R,Z)
        ZNRM=SNRMM(NGWK,Z)  
      ELSE
        CALL STOPGM('CLINBCG','ILLEGAL ITOL')
      ENDIF
      CALL DIAGO(R,Z)
C     ==--------------------------------------------------------------==
100   CONTINUE
      IF (ITER.LE.ITMAX) THEN
        ITER=ITER+1
        ZM1NRM=ZNRM
        CALL DIAGO(RR,ZZ)
        IF(TKPNT) THEN 
          BKNUM=ZDOTC(NGWK,RR,1,Z,1)
        ELSE
          BKNUM=DCMPLX(DOTP(NGWK,RR,Z),0.D0)
        ENDIF
        IF(ITER.EQ.1) THEN
!$OMP parallel do private(J)
          DO J=1,NGWK
            P(J)=Z(J)
            PP(J)=ZZ(J)
          ENDDO
          BKDEN=0.D0            !Not useful
        ELSE
          BK=BKNUM/BKDEN
!$OMP parallel do private(J)
          DO J=1,NGWK
            P(J)=BK*P(J)+Z(J)
            PP(J)=DCONJG(BK)*PP(J)+ZZ(J)
          ENDDO
        ENDIF
        BKDEN=BKNUM
        CALL ATIMES(P,Z,C0,VPOT,PSI,SCR,LSCR,NSTATE,F)
        AKDEN=ZZERO
        IF(TKPNT) THEN 
          AKDEN = ZDOTC(NGWK,PP,1,Z,1)
        ELSE
          AKDEN=DCMPLX(DOTP(NGWK,PP,Z),0.D0)
        ENDIF
        AK=BKNUM/AKDEN
        CALL ATIMES(PP,ZZ,C0,VPOT,PSI,SCR,LSCR,NSTATE,F)
!$OMP parallel do private(J)
        DO J=1,NGWK
          X(J)=X(J)+AK*P(J)
          R(J)=R(J)-AK*Z(J)
          RR(J)=RR(J)-DCONJG(AK)*ZZ(J)
        ENDDO
        CALL DIAGO(R,Z)
        IF(ITOL.EQ.1.OR.ITOL.EQ.2)THEN
          ZNRM=1.D0
          ERR=SNRMM(NGWK,R)
          ERR=ERR/BNRM 
        ELSE IF(ITOL.EQ.3.OR.ITOL.EQ.4)THEN
          ZNRM=SNRMM(NGWK,Z)  
          IF(DABS(ZM1NRM-ZNRM).GT.EPS*ZNRM) THEN
            DXNRM=ABS(AK)*SNRMM(NGWK,P)  
            ERR=ZNRM/DABS(ZM1NRM-ZNRM)*DXNRM
          ELSE
            ERR=ZNRM/BNRM
            GOTO 100
          ENDIF
          XNRM=SNRMM(NGWK,X) 
          IF(ERR.LE.0.5D0*XNRM) THEN
            ERR=ERR/XNRM
          ELSE
            ERR=ZNRM/BNRM
            GOTO 100
          ENDIF
        ENDIF
        IF(TDEBUG) THEN 
          WRITE(6,*) ' ITER=',ITER,' ERR=',ERR
          CALL MY_FLUSH(6)
        ENDIF
        IF(ERR.GT.TOL) GOTO 100
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION SNRMM(N,Z)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'kpts.inc'
C     Arguments
      REAL*8     SNRMM
      INTEGER    N
      COMPLEX*16 Z(N)
C     Variables
      REAL*8     DOTP,DZNRM2
      EXTERNAL   DOTP
C     ==--------------------------------------------------------------==
      IF(.NOT.TKPNT) THEN
        SNRMM = DOTP(N,Z,Z) 
        SNRMM = DSQRT(SNRMM) 
      ELSE 
        SNRMM=DZNRM2(N,Z,1)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN 
      END 
C     ==================================================================
      SUBROUTINE ATIMES(CIN,COUT,C0,VPOT,PSI,SCR,LSCR,NSTATE,F)
C     ==--------------------------------------------------------------==
C     == ROUTINE TO APPLY (H-E) TO PSI
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'
      INCLUDE 'atimes.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'pola.inc'
C     Arguments
      INTEGER    NSTATE,LSCR
      COMPLEX*16 CIN(NGWK),COUT(NGWK),C0(NGWK,NSTATE),PSI(NNR1)
      REAL*8     F(NSTATE),VPOT(NNR1,NLSD),SCR(LSCR)
C     Variables
      CHARACTER TAG*30
      COMPLEX*16 SC0
      INTEGER    LATIMES
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_ATIMES(LATIMES,TAG,NSTATE)
      CALL TEST_SCR('ATIMES',TAG,LSCR,LATIMES)
C     ==--------------------------------------------------------------==
      IF(TIVAN) CALL STOPGM('ATIMES','TIVAN NOT IMPLEMENTED')
      CALL HPSI(CIN,COUT,SC0,VPOT,PSI,SCR,LSCR,1,IKIND_ATIMES,NLSD)
      CALL DSCAL(NGWK*2,-1.D0,COUT,1)
      CALL DAXPY(NGWK*2,-ATIMES_EVAL,CIN,1,COUT,1)
C     Project out valence states 
      CALL PROJV(COUT,C0,F,NSTATE)
C     ==--------------------------------------------------------------==
      RETURN
      END 
C     ==================================================================
      SUBROUTINE DIAGO(C0,C1)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc' 
      INCLUDE 'atimes.inc' 
      INCLUDE 'kpts.inc'
C     Arguments
      COMPLEX*16 C0(NGWK),C1(NGWK)
C     VAriables
      INTEGER    IG
C     ==--------------------------------------------------------------==
      IF(.NOT.TKPNT) THEN
!$OMP parallel do private(IG)
        DO IG=1,NGW
          C1(IG) = C0(IG)/(HG(IG)*TPIBA2*0.5D0-ATIMES_EVAL)
        ENDDO
      ELSE
!$OMP parallel do private(IG)
        DO IG=1,NGW
          C1(IG) = C0(IG)/(HG(IG)*TPIBA2*0.5D0-ATIMES_EVAL)
          C1(NGW+IG) = C0(NGW+IG)/(HG(IG)*TPIBA2*0.5D0-ATIMES_EVAL)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END 
C     ==================================================================
      SUBROUTINE GIVE_SCR_CLINBCG(LCLINBCG,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   LCLINBCG,NSTATE
      CHARACTER TAG*30
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_ATIMES(LCLINBCG,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_ATIMES(LATIMES,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   LATIMES,NSTATE
      CHARACTER TAG*30
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_HPSI(LATIMES,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
