C     ==================================================================
      SUBROUTINE COMPRESS(N,A,IA,SCALE,ICOMP,NX)
C     ==--------------------------------------------------------------==
C     == COMPRESS THE DATA FROM A TO IA                               ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   ICOMP      compression type                                ==
C     ==              64 bits -> 64/ICOMP bits                        ==
C     ==   N          Size of A array                                 ==
C     ==   A(1:N)     Array to compress                               ==
C     ==   SCALE      Scale of data                                   ==
C     == OUTPUT:                                                      ==
C     ==   NX         Size of given array IA (in integer*8)           ==
C     ==   IA(1:NX)   Compressed array          (integer*8)           ==
C     ==--------------------------------------------------------------==
#if defined(CRAY64) || defined(__NEC) || defined(__SUN) || defined(__Linux) || defined(__SR2201) || defined(__SR8000) || defined(__WINNT)
      IMPLICIT NONE
C     Arguments
      INTEGER N,IA(2*N),ICOMP,NX
      REAL*8  A(N),SCALE
C     Variables
      REAL*8  A0(N)
      POINTER (IP_A0,A0)
      INTEGER NB
      REAL*8  FX
C     ==--------------------------------------------------------------==
      IF(ICOMP.EQ.1) THEN
        CALL GET_ADDR(IP_A0,IA(1))
        CALL DCOPY(N,A(1),1,A0(1),1)
        NX=N
      ELSE
        NB=64/ICOMP
C       Overflow: Integer 2.D0**(NB-1)-1
        FX=2.D0**(NB-1)-1.D0
        FX=FX/SCALE
        CALL SPACK(IA,A,N,NX,NB,FX)
      ENDIF
#else
      IMPLICIT NONE
      INTEGER   N,ICOMP,NX
      REAL*8    A(N), SCALE
      INTEGER*8 IA(N)
C     Variables
      INTEGER*1 IA1(N)
      INTEGER*2 IA2(N)
      INTEGER*4 IA4(N)
      POINTER   (IP_IA1,IA1),(IP_IA2,IA2),(IP_IA4,IA4)
      REAL*8    A0(N)
      POINTER   (IP_A0,A0)
      INTEGER   NB,I
      REAL*8    FX
C     ==--------------------------------------------------------------==
      IF(ICOMP.EQ.1) THEN
        CALL GET_ADDR(IP_A0,IA(1))
        CALL DCOPY(N,A(1),1,A0(1),1)
        NX=N
      ELSE
        CALL GET_ADDR(IP_IA1,IA(1))
        CALL GET_ADDR(IP_IA2,IA(1))
        CALL GET_ADDR(IP_IA4,IA(1))
        NB=64/ICOMP
C       Overflow in integer*4 max=2**31-1
C        NN=2**(NB-1)
        FX=2.D0**(NB-1)-1.D0
        FX=FX/SCALE
        NX=N/ICOMP+1
        IF(ICOMP.EQ.2) THEN
          DO I=1,N
            IA4(I)=NINT(A(I)*FX)
          ENDDO
        ELSEIF(ICOMP.EQ.4) THEN
          DO I=1,N
            IA2(I)=NINT(A(I)*FX)
          ENDDO
        ELSEIF(ICOMP.EQ.8) THEN
          DO I=1,N
            IA1(I)=NINT(A(I)*FX)
          ENDDO
        ENDIF
      ENDIF
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE DECOMPR(NA,A,IA,SCALE,ICOMP,NX)
C     ==--------------------------------------------------------------==
C     == DECOMPRESS THE DATA FROM IA TO A                             ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   ICOMP      Compression type                                ==
C     ==              64 bits -> 64/ICOMP bits                        ==
C     ==   NA         Maximum size of A                               ==
C     ==   NX         Size of IA array (in integer*8)                 ==
C     ==   IA(1:NX)   Array to decompress (integer*8)                 ==
C     ==   SCALE      Scale for the decompressed data                 ==
C     == OUTPUT:                                                      ==
C     ==   A(1:N)     Decompressed array                              ==
C     ==--------------------------------------------------------------==
#if defined(CRAY64) || defined(__NEC) || defined(__SUN) || defined(__Linux) || defined(__SR2201) || defined(__SR8000)
      IMPLICIT NONE
C     Arguments
      INTEGER NA,ICOMP,NX,IA(2*NX)
      REAL*8  A(NA),SCALE
C     Variables
      REAL*8  A0(NX)
      POINTER (IP_A0,A0)
      INTEGER N,NB
      REAL*8  FX
C     ==--------------------------------------------------------------==
      IF(ICOMP.EQ.1) THEN
        N=NX
      ELSE
        N=(NX-1)*ICOMP
      ENDIF
      IF(N.GT.NA) THEN
        WRITE(6,'(1X,A,I8,A,I8)')
     &       'DECOMPR! THE SIZE OF THE DECOMPRESSED ARRAY ',N,
     &       ' ARE BIGGER THAN THE SIZE OF THE GIVEN ARRAY ',NA
        CALL STOPGM('DECOMPR','READ ERROR')
      ENDIF
      IF(ICOMP.EQ.1) THEN
        N=NX
        CALL GET_ADDR(IP_A0,IA(1))
        CALL DCOPY(N,A0(1),1,A(1),1)
      ELSE
        NB=64/ICOMP
C       Overflowfor integer: max=2.D0**(NB-1)-1.D0
        FX=2.D0**(NB-1)-1.D0
        FX=SCALE/FX
        N=(NX-1)*ICOMP
        CALL SUNPACK(IA,A,N,NX,NB,FX)
      ENDIF
#else
      IMPLICIT NONE
      INTEGER   NA,ICOMP,NX
      REAL*8    A(NA), SCALE
      INTEGER*8 IA(NX)
C     Variables
      INTEGER*1 IA1(*)
      INTEGER*2 IA2(*)
      INTEGER*4 IA4(*)
      POINTER   (IP_IA1,IA1),(IP_IA2,IA2),(IP_IA4,IA4)
      REAL*8    A0(NX)
      POINTER   (IP_A0,A0)
      INTEGER   NB,N,I
      REAL*8    FX
C     ==--------------------------------------------------------------==
      IF(ICOMP.EQ.1) THEN
        N=NX
      ELSE
        N=(NX-1)*ICOMP
      ENDIF
      IF(N.GT.NA) THEN
        WRITE(6,'(1X,A,I8,A,I8)')
     &       'DECOMPR! THE SIZE OF THE DECOMPRESSED ARRAY ',N,
     &       ' ARE BIGGER THAN THE SIZE OF THE GIVEN ARRAY ',NA
        CALL STOPGM('DECOMPR','READ ERROR')
      ENDIF
      IF(ICOMP.EQ.1) THEN
        N=NX
        CALL GET_ADDR(IP_A0,IA(1))
        CALL DCOPY(N,A0(1),1,A(1),1)
      ELSE
        CALL GET_ADDR(IP_IA1,IA(1))
        CALL GET_ADDR(IP_IA2,IA(1))
        CALL GET_ADDR(IP_IA4,IA(1))
        NB=64/ICOMP
C       Overflow in integer*4 max=2**31-1
C        NN=2**(NB-1)
        FX=2.D0**(NB-1)-1.D0
        FX=SCALE/FX
        N=(NX-1)*ICOMP
        IF(ICOMP.EQ.2) THEN
          DO I=1,N
            A(I)=IA4(I)*FX
          ENDDO
        ELSEIF(ICOMP.EQ.4) THEN
          DO I=1,N
            A(I)=IA2(I)*FX
          ENDDO
        ELSEIF(ICOMP.EQ.8) THEN
          DO I=1,N
            A(I)=IA1(I)*FX
          ENDDO
        ENDIF
      ENDIF
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#if defined(CRAY64) || defined(__NEC)
      SUBROUTINE SPACK(IA,A,N,NX,IBYTE,FX)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER IA(*),N,NX,IBYTE
      REAL*8  A(*),FX
C     Variables
      INTEGER NN,M,I,II,
     &        IB1,IB2,IB3,IB4,IB5,IB6,IB7,IB8
C     ==--------------------------------------------------------------==
      NN=2**(IBYTE-1)
      IF(IBYTE.EQ.32) THEN
        M=N+MOD(N,2_8)
        NX=M/2
        DO I=1,M,2
          II=(I+1)/2
          IB1=NINT(A(I)*FX)+NN
          IB2=NINT(A(I+1)*FX)+NN
          IA(II)=ISHFT(IB1,32)+IB2
        ENDDO
      ELSEIF(IBYTE.EQ.16) THEN
        M=N+MOD(N,4_8)
        NX=M/4
        DO I=1,M,4
          II=(I+3)/4
          IB1=NINT(A(I)*FX)+NN
          IB2=NINT(A(I+1)*FX)+NN
          IB3=NINT(A(I+2)*FX)+NN
          IB4=NINT(A(I+3)*FX)+NN
          IA(II)=ISHFT(IB1,48)+ISHFT(IB2,32)+ISHFT(IB3,16)+IB4
        ENDDO
      ELSEIF(IBYTE.EQ.8) THEN
        M=N+MOD(N,8_8)
        NX=M/8
        DO I=1,M,8
          II=(I+7)/8
          IB1=NINT(A(I)*FX)+NN
          IB2=NINT(A(I+1)*FX)+NN
          IB3=NINT(A(I+2)*FX)+NN
          IB4=NINT(A(I+3)*FX)+NN
          IB5=NINT(A(I+4)*FX)+NN
          IB6=NINT(A(I+5)*FX)+NN
          IB7=NINT(A(I+6)*FX)+NN
          IB8=NINT(A(I+7)*FX)+NN
          IA(II)=ISHFT(IB1,56)+ISHFT(IB2,48)+ISHFT(IB3,40)+
     *         ISHFT(IB4,32)+ISHFT(IB5,24)+ISHFT(IB6,16)+
     *         ISHFT(IB7,8)+IB8
        ENDDO
      ELSE
        CALL STOPGM('SPACK','IBYTE NOT IMPLEMENTED')
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SUNPACK(IA,A,N,NX,IBYTE,FX)
C     ==--------------------------------------------------------------==
C     == USED BY DECOMPR TO DECOMPRESS DATA FROM IA TO A              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER*8 IA(*),N,NX,IBYTE
cmb      INTEGER IA(*),N,NX,IBYTE
      REAL*8  A(*),FX
C     Variables
      INTEGER NN,I,II,
     &        IB1,IB2,IB3,IB4,IB5,IB6,IB7,IB8
C     ==--------------------------------------------------------------==
      NN=2**(IBYTE-1)
      IF(IBYTE.EQ.32) THEN
        DO I=1,N,2
          II=(I+1)/2
          IB1=ISHFT(IA(II),-32_8)
          IB2=ISHFT(ISHFT(IA(II),32_8),-32_8)
          A(I)=(IB1-NN)*FX
          A(I+1)=(IB2-NN)*FX
        ENDDO
      ELSEIF(IBYTE.EQ.16) THEN
        DO I=1,N,4
          II=(I+3)/4
          IB1=ISHFT(IA(II),-48_8)
          IB2=ISHFT(ISHFT(IA(II),16_8),-48_8)
          IB3=ISHFT(ISHFT(IA(II),32_8),-48_8)
          IB4=ISHFT(ISHFT(IA(II),48_8),-48_8)
          A(I)=(IB1-NN)*FX
          A(I+1)=(IB2-NN)*FX
          A(I+2)=(IB3-NN)*FX
          A(I+3)=(IB4-NN)*FX
        ENDDO
      ELSEIF(IBYTE.EQ.8) THEN
        DO I=1,N,8
          II=(I+7)/8
          IB1=ISHFT(IA(II),-56_8)
          IB2=ISHFT(ISHFT(IA(II), 8_8),-56_8)
          IB3=ISHFT(ISHFT(IA(II),16_8),-56_8)
          IB4=ISHFT(ISHFT(IA(II),24_8),-56_8)
          IB5=ISHFT(ISHFT(IA(II),32_8),-56_8)
          IB6=ISHFT(ISHFT(IA(II),40_8),-56_8)
          IB7=ISHFT(ISHFT(IA(II),48_8),-56_8)
          IB8=ISHFT(ISHFT(IA(II),56_8),-56_8)
          A(I)=(IB1-NN)*FX
          A(I+1)=(IB2-NN)*FX
          A(I+2)=(IB3-NN)*FX
          A(I+3)=(IB4-NN)*FX
          A(I+4)=(IB5-NN)*FX
          A(I+5)=(IB6-NN)*FX
          A(I+6)=(IB7-NN)*FX
          A(I+7)=(IB8-NN)*FX
        ENDDO
      ELSE
        CALL STOPGM('SUNPACK','IBYTE NOT IMPLEMENTED')
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#elif defined(__SUN) || defined(__Linux) || defined(__SR2201) || defined(__SR8000) || defined(__WINNT)
      SUBROUTINE SPACK(IA,A,N,NX,IBYTE,FX)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER IA(*),N,NX,IBYTE
      REAL*8  A(*),FX
C     Variables
      INTEGER IB1,IB2,IB3,IB4,NN,M,I,II
C     ==--------------------------------------------------------------==
      NN=2**(IBYTE-1)
      IF(IBYTE.EQ.32) THEN
        M=N+MOD(N,2)
        NX=M/2
        DO I=1,M
          II=I
          IB1=NINT(A(I)*FX)+NN
          IA(II)=IB1
        ENDDO
      ELSEIF(IBYTE.EQ.16) THEN
        M=N+MOD(N,4)
        NX=M/4
        DO I=1,M,2
          II=(I+1)/2
          IB1=NINT(A(I)*FX)+NN
          IB2=NINT(A(I+1)*FX)+NN
          IA(II)=ISHFT(IB1,16)+IB2
        ENDDO
      ELSEIF(IBYTE.EQ.8) THEN
        M=N+MOD(N,8)
        NX=M/8
        DO I=1,M,4
          II=(I+3)/4
          IB1=NINT(A(I)*FX)+NN
          IB2=NINT(A(I+1)*FX)+NN
          IB3=NINT(A(I+2)*FX)+NN
          IB4=NINT(A(I+3)*FX)+NN
          IA(II)=ISHFT(IB1,24)+ISHFT(IB2,16)+ISHFT(IB3,8)+IB4
        ENDDO
      ELSE
        CALL STOPGM('SPACK','IBYTE NOT IMPLEMENTED')
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SUNPACK(IA,A,N,NX,IBYTE,FX)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER IA(*),N,NX,IBYTE
C     Variables
      REAL*8  A(*),FX
      INTEGER IB1,IB2,IB3,IB4,NN,I,II
C     ==--------------------------------------------------------------==
      NN=2**(IBYTE-1)
      IF(IBYTE.EQ.32) THEN
        DO I=1,N
          II=I
          IB1=IA(II)
          A(I)=(IB1-NN)*FX
        ENDDO
      ELSEIF(IBYTE.EQ.16) THEN
        DO I=1,N,2
          II=(I+1)/2
          IB1=ISHFT(IA(II),-16)
          IB2=ISHFT(ISHFT(IA(II),16),-16)
          A(I)=(IB1-NN)*FX
          A(I+1)=(IB2-NN)*FX
        ENDDO
      ELSEIF(IBYTE.EQ.8) THEN
        DO I=1,N,4
          II=(I+3)/4
          IB1=ISHFT(IA(II),-24)
          IB2=ISHFT(ISHFT(IA(II), 8),-24)
          IB3=ISHFT(ISHFT(IA(II),16),-24)
          IB4=ISHFT(ISHFT(IA(II),24),-24)
          A(I)=(IB1-NN)*FX
          A(I+1)=(IB2-NN)*FX
          A(I+2)=(IB3-NN)*FX
          A(I+3)=(IB4-NN)*FX
        ENDDO
      ELSE
        CALL STOPGM('SUNPACK','IBYTE NOT IMPLEMENTED')
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#endif
