C     ==================================================================
      SUBROUTINE COMVEL(VELP,VCMIO,INOUT)
C     REMOVE TRANSLATIONAL VELOCITIES. 
C     Last Update: 2008/06/11 by AK
C     To integrate more cleanly with the logic of the velocity verlet
C     algorithm you have to call this subroutine twice. First with 
C     INOUT set to .TRUE., where the routine will store the center of
C     mass velocity vector and the total mass in VCMIO(4). This call 
C     will remove only half of the velocity and has to be called before 
C     the first (half) velocity update step. For the second call, which 
C     has to come after the second (half) velocity update step, with 
C     INOUT set to .FALSE. those data is then used to remove the 
C     remainder of the center of mass motion.
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'nose.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cotr.inc'! LSKCOR
C     Arguments
      REAL*8     VELP(3,NAX,*),VCMIO(4)
      LOGICAL    INOUT  ! true  -> write VCMIO
                        ! false -> read  VCMIO
C     Variables
      REAL*8     VCM(3),PMA00,EKINP1,TEMPP1,EKINP2,TEMPP2,TSCAL,VSCALE
      INTEGER    IS,IA,IAT
      REAL*8     ONE3RD
      PARAMETER (ONE3RD=1.0D0/3.0D0)
C     ==--------------------------------------------------------------==
C     COMPUTE THE IONIC TEMPERATURE TEMPP1
      CALL EKINPP(EKINP1,VELP)
      TEMPP1=EKINP1*FACTEM*2.0D0/GLIB
      IF(INOUT) THEN
C     DETERMINE CENTER OF MASS VELOCITY
        VCM(1)=0.0D0
        VCM(2)=0.0D0
        VCM(3)=0.0D0
        PMA00=0.D0
#if defined(__VECTOR)
C$OMP parallel do private(IS,IA,IAT) reduction(+:VCM,PMA00)
#else
C$OMP parallel do private(IS,IA,IAT) reduction(+:VCM,PMA00)
C$OMP+ schedule(static)
#endif
        DO IAT=1,NAT
          IA=IATPT(1,IAT)
          IS=IATPT(2,IAT)
          VCM(1)=VCM(1)+VELP(1,IA,IS)*PMA0(IS)
          VCM(2)=VCM(2)+VELP(2,IA,IS)*PMA0(IS)
          VCM(3)=VCM(3)+VELP(3,IA,IS)*PMA0(IS)
!!        PMA00=PMA00+PMA0(IS)
          PMA00=PMA00+ONE3RD*(LSKCOR(1,IAT)+
     .          LSKCOR(2,IAT)+LSKCOR(3,IAT))*PMA0(IS)
        ENDDO
        VCMIO(1)=VCM(1)
        VCMIO(2)=VCM(2)
        VCMIO(3)=VCM(3)
        VCMIO(4)=PMA00
      ELSE
C     TAKE CENTER OF MASS VELOCITY FROM LAST CALL
        VCM(1)=VCMIO(1)
        VCM(2)=VCMIO(2)
        VCM(3)=VCMIO(3)
        PMA00=VCMIO(4)
      ENDIF
#if defined(__VECTOR)
C$OMP parallel do private(IS,IA,IAT) shared(VCM,PMA00)
#else
C$OMP parallel do private(IS,IA,IAT) shared(VCM,PMA00)
C$OMP+ schedule(static)
#endif
      DO IAT=1,NAT
        IA=IATPT(1,IAT)
        IS=IATPT(2,IAT)
        VELP(1,IA,IS)=VELP(1,IA,IS)-0.5d0*VCM(1)/PMA00
        VELP(2,IA,IS)=VELP(2,IA,IS)-0.5d0*VCM(2)/PMA00
        VELP(3,IA,IS)=VELP(3,IA,IS)-0.5d0*VCM(3)/PMA00
      ENDDO
      CALL TAUCL(VELP)
C     RESCALE ALL VELOCITIES TO COMPENSATE FOR LOSS OF KINETIC ENERGY
      CALL EKINPP(EKINP2,VELP)
      TEMPP2=EKINP2*FACTEM*2.0D0/GLIB   
      IF(TEMPP2.GT.1.D-5) THEN
        TSCAL=TEMPP1/TEMPP2
        VSCALE=DSQRT(TSCAL)
#if defined(__VECTOR)
C$OMP parallel do private(IS,IA,IAT) shared(VSCALE)
#else
C$OMP parallel do private(IS,IA,IAT) shared(VSCALE)
C$OMP+ schedule(static)
#endif
        DO IAT=1,NAT
          IA=IATPT(1,IAT)
          IS=IATPT(2,IAT)
          VELP(1,IA,IS)=VELP(1,IA,IS)*VSCALE
          VELP(2,IA,IS)=VELP(2,IA,IS)*VSCALE
          VELP(3,IA,IS)=VELP(3,IA,IS)*VSCALE
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
