C     ==================================================================
      SUBROUTINE CONDUCTIVITY(C0,WE,F,NSTATE,NKPOINT)
C     ==--------------------------------------------------------------==
C     == Compute optical conductivity using Greenwood-Kubo formula    ==
C     == Sigma(w) = 2 pi e^2/(3 m^2 Vw) sum_i,j (f_i-fj) |<j|p|i>|^2  ==
C     == * delta(Ej-Ei-h_bar w)                                       ==
C     ==                                                              ==
C     == Original version - P. L. Silvestrelli (18/10/1995)           ==
C     == Updated version - M. Boero (22/1/2002)                       ==
C     ==                   M. Boero & P. L. Silvestrelli (10/ 4/2002) ==
C     ==                   M. Boero & F. Gervasio        ( 8/ 5/2002) ==
C     ==                   M. Boero                      (02/06/2005) ==
C     ==     (printout <i|x|j> ~ (f_i-f_j)*|M(i,j)|^2/(eps_j - eps_i) ==
C     ==                                                              ==
C     == N. B.:  If TSEGHE is .true. additional quantities are printed==
C     == on the FORT.89 file, in the following form:                  ==
C     ==                                                              ==
C     == i              f_i    eps_i                                  ==
C     == i    j=i+1     (f_i - f_j)   (eps_j - eps_i)   M_ij          ==
C     == i    j=i+2     (f_i - f_j)   (eps_j - eps_i)   M_ij          ==
C     == ...................................................          ==
C     == ...................................................          ==
C     == ...................................................          ==
C     == i    j=N       (f_i - f_j)   (eps_j - eps_i)   M_ij          ==
C     ==                                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'condu.inc'
      INCLUDE 'fileopen.inc'
C     Arguments
      INTEGER    NSTATE,NKPOINT
      COMPLEX*16 C0(NGWK,NSTATE,NKPOINT)
      REAL*8     WE(NSTATE,NKPOINT),F(NSTATE,NKPOINT)
C     Variables
      DIMENSION  RK(3,NKPOINT),WK(NKPOINT)
      COMPLEX*16 ZZERO
      REAL*8     FTOLL
      PARAMETER  (ZZERO=(0.D0,0.D0),FTOLL=1.D-10)
      COMPLEX*16 CAUX,CSUMX,CSUMY,CSUMZ
      REAL*8     FACTOR,RYBY2,ACON,DF,DENG,SUMX,SUMY,SUMZ,SUM,AUX
     .          ,AIAUX,AUXX,AUXY,AUXZ,SUMX2,SUMY2,SUMZ2
     .          ,fracon
#ifdef PARALLEL
      REAL*8     CSRI(2)
#endif
      LOGICAL    FERROR
      INTEGER    IK,I,J,IBIN,IG
     &          ,IFIRST
      DATA       IFIRST/0/
      SAVE       IFIRST
C     ==--------------------------------------------------------------==
C     == FACTOR = (2.D0*PI/3.D0/OMEGA)                                ==
C     == FACTOR is divided by 2 because this factor is already        ==
C     == contained in f_i: 4.6E+04 is (?) the conversion factor       ==
C     == from a.u. to (ohm cm)**(-1)                                  ==
C     ==--------------------------------------------------------------==
      RYBY2=2.D0*RY
      FACTOR=PI/3.D0/OMEGA*RYBY2**2*4.6D04
      ACON=TPIBA2
      IF(IFIRST.EQ.0) THEN
        IFIRST=1
        CALL MEMORY(IP_CONDUCT ,NCONDUCT,'CONDUCT' )
        CALL MEMORY(IP_CONDUCT2,NCONDUCT,'CONDUCT2')
        CALL MEMORY(IP_CONDAVG ,NCONDUCT,'CONDAVG' )
        CALL MEMORY(IP_CONDAVG2,NCONDUCT,'CONDAVG2')
        CALL MEMORY(IP_NORMCON,NCONDUCT/IRAT+1,'NORMCON')
        CALL AZZERO(CONDAVG,NCONDUCT)
        CALL AZZERO(CONDAVG2,NCONDUCT)
        CALL IAZZERO(NORMCON,NCONDUCT)
      ENDIF
      CALL AZZERO(CONDUCT,NCONDUCT)
cmb-> write the transition matrix elements into the file MATRIX.DAT
cmb-> Note: r=(x,y,z); only NON-ZERO transitions are written
      IF(PARENT) THEN
        CALL FILEOPEN(59,'MATRIX.DAT',FO_DEF,FERROR)
        WRITE(59,'(A,A,A)')
     .      "  |k>  |i>  |j>  f(i) f(j)",
     .      "  E(i) [eV]  E(j) [eV]     |<i|x|j>|**2",
     .      "     |<i|y|j>|**2     |<i|z|j>|**2     |<i|r|j>|**2"
        WRITE(59,*)
      ENDIF 
cmb-> uncomment the following section to check and print out
cmb-> eigenvalues and occupation numbers in debugging
*      IF(PARENT) THEN
*        DO IK=1,NKPOINT
*           WRITE(6,'(1x,''K-POINT ='',i6)')IK
*           WRITE(6,'(1x,'' STATE  OCCUPATION  EIGENVALUE (eV)'')')
*           DO I=1,NSTATE
*              WRITE(6,'(1x,i5,2x,f7.3,6x,f10.3)')
*     .              I,F(I,IK),RYBY2*WE(I,IK)
*           ENDDO
*           WRITE(6,*)
*        ENDDO
*      ENDIF
cmb-> end of eigenvalues and occupation section
      DO IK=1,NKPOINT
        DO I=1,NSTATE-1
          DO J=I+1,NSTATE
            DF=F(I,IK)-F(J,IK)
            IF(DF.LT.FTOLL) GOTO 999
            DENG=RYBY2*(WE(J,IK)-WE(I,IK))
            IBIN=INT(DENG/CONDSTEP)+1
            IF(IBIN.LE.NCONDUCT) THEN
              IF(TKPNT) THEN
                CSUMX=ZZERO
                CSUMY=ZZERO
                CSUMZ=ZZERO
cmb-> first G > 0 with the k-points
#ifdef __SR8000
*poption parallel
#endif
!$OMP parallel do private(IG,CAUX) reduction(+:CSUMX,CSUMY,CSUMZ)
                DO IG=1,NGW
                  CAUX=DCONJG(C0(IG,I,IK))*C0(IG,J,IK)
                  CSUMX=CSUMX+(RK(1,IK)+GK(1,IG))*CAUX
                  CSUMY=CSUMY+(RK(2,IK)+GK(2,IG))*CAUX
                  CSUMZ=CSUMZ+(RK(3,IK)+GK(3,IG))*CAUX
                ENDDO
cmb-> then G < 0 with the k-points
#ifdef __SR8000
*poption parallel
#endif
!$OMP parallel do private(IG,CAUX) reduction(+:CSUMX,CSUMY,CSUMZ)
                DO IG=1,NGW
                  CAUX=DCONJG(C0(IG+NGW,I,IK))*C0(IG+NGW,J,IK)
                  CSUMX=CSUMX+(RK(1,IK)-GK(1,IG))*CAUX
                  CSUMY=CSUMY+(RK(2,IK)-GK(2,IG))*CAUX
                  CSUMZ=CSUMZ+(RK(3,IK)-GK(3,IG))*CAUX
                ENDDO
#ifdef PARALLEL
                CSRI(1)=DREAL(CSUMX)
                CSRI(2)=DIMAG(CSUMX)
                CALL GLOSUM(2,CSRI)
                CSUMX=DCMPLX(CSRI(1),CSRI(2))
                CSRI(1)=DREAL(CSUMY)
                CSRI(2)=DIMAG(CSUMY)
                CALL GLOSUM(2,CSRI)
                CSUMY=DCMPLX(CSRI(1),CSRI(2))
                CSRI(1)=DREAL(CSUMZ)
                CSRI(2)=DIMAG(CSUMZ)
                CALL GLOSUM(2,CSRI)
                CSUMZ=DCMPLX(CSRI(1),CSRI(2))
#endif
                SUMX2=DREAL(CSUMX*DCONJG(CSUMX))
                SUMY2=DREAL(CSUMY*DCONJG(CSUMY))
                SUMZ2=DREAL(CSUMZ*DCONJG(CSUMZ))
                SUM=SUMX2+SUMY2+SUMZ2
              ELSE
                SUMX=0.D0
                SUMY=0.D0
                SUMZ=0.D0
cmb->  Sum_{G} <c(j)|G|c(i)> = 2 * Sum_{G>0} G Im{<c(j)|c(i)>} at Gamma
#ifdef __SR8000
*poption parallel
#endif
!$OMP parallel do private(IG,AIAUX) reduction(+:SUMX,SUMY,SUMZ)
                DO IG=1,NGW
                  AIAUX=DIMAG(DCONJG(C0(IG,I,IK))*C0(IG,J,IK))
                  SUMX=SUMX+GK(1,IG)*AIAUX
                  SUMY=SUMY+GK(2,IG)*AIAUX
                  SUMZ=SUMZ+GK(3,IG)*AIAUX
                ENDDO
#ifdef PARALLEL
                CALL GLOSUM(1,SUMX)
                CALL GLOSUM(1,SUMY)
                CALL GLOSUM(1,SUMZ)
#endif
                SUMX2=4.0D0*SUMX*SUMX
                SUMY2=4.0D0*SUMY*SUMY
                SUMZ2=4.0D0*SUMZ*SUMZ
                SUM=SUMX2+SUMY2+SUMZ2
              ENDIF
C AK 2005/03/26: not used
C              FREQU=(DBLE(IBIN-1)+0.5D0)*CONDSTEP
              fracon=FACTOR*DF*ACON/DENG/CONDSTEP
              AUXX=fracon*SUMX2
              AUXY=fracon*SUMY2
              AUXZ=fracon*SUMZ2
              AUX=fracon*SUM
              IF(TKPNT) THEN
                AUXX=AUXX*WK(IK)
                AUXY=AUXY*WK(IK)
                AUXZ=AUXZ*WK(IK)
                AUX=AUX*WK(IK)
              ENDIF
              CONDUCT(IBIN)=CONDUCT(IBIN)+AUX
              NORMCON(IBIN)=NORMCON(IBIN)+1
              IF(PARENT) 
     .        WRITE(59,'(3i5,1x,2f5.2,2f11.3,4e17.8)')
     .         IK,I,J,F(I,IK),F(J,IK)
     .        ,RYBY2*WE(I,IK),RYBY2*WE(J,IK)
     .        ,AUXX,AUXY,AUXZ,AUX
            ENDIF
 999        CONTINUE
          ENDDO
        ENDDO
      ENDDO
cmb-> and now fill the remaining bins
      DO IBIN=1,NCONDUCT
         CONDUCT2(IBIN)=CONDUCT(IBIN)*CONDUCT(IBIN)
         CONDAVG(IBIN) =CONDAVG(IBIN)+CONDUCT(IBIN)
         CONDAVG2(IBIN)=CONDAVG2(IBIN)+CONDUCT2(IBIN)
      ENDDO
C     ==--------------------------------------------------------------==
      IF(PARENT) CALL FILECLOSE(59)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
