C     ==================================================================
      SUBROUTINE CONINP(IUNIT)
C     ==--------------------------------------------------------------==
C     ==  Reads Constraint Input for Geometry                         ==
C     ==--------------------------------------------------------------==
C     ==                                                              ==
C     ==  FIX {ALL,QM,MM,SOLUTE}                                      ==
C     ==  FIX ATOM                                                    ==
C     ==    nfix  n1  n2  ...                                         ==
C     ==  FIX SEQUENCE                                                ==
C     ==    nfirst nlast                                              ==
C     ==  FIX ELEMENT [SEQUENCE]                                      ==
C     ==    zv [ifirst ilast]                                         ==
C     ==  FIX PPTYPE [SEQUENCE]                                       ==
C     ==    isp [ifist ilast]                                         ==
C     ==  FIX COORDINATES                                             ==
C     ==    nfix                                                      ==
C     ==    n1   ix   iy   iz                                         ==
C     ==    ...                                                       ==
C     ==  FIX COM                                                     ==
C     ==  FIX STRUCTURE [SHOVE]                                       ==
C     ==   nfix                                                       ==
C     ==  DIST      n1  n2   R0  [+1,-1,0] GROWTH aa DEST Rd          ==
C     ==  STRETCH   n1  n2   R0  [+1,-1,0] GROWTH aa DEST Rd          ==
C     ==  DISAXIS   n1  n2  k   R0  [+1,-1,0] GROWTH aa DEST Rd       ==
C     ==  BEND      n1  n2  n3 A0  [+1,-1,0] GROWTH aa DEST Ad        ==
C     ==  TORSION   n1  n2  n3  n4  D0 [+1,-1,0] GROWTH aa DEST Dd    ==
C     ==  OUTP      n1  n2  n3  n4  D0 [+1,-1,0] GROWTH aa DEST Dd    ==
C     ==  DIFFER    n1  n2  n3  dR0 [+1,-1,0] GROWTH aa DEST dRd      ==
C     ==  RIGID     na  n1  ...                                       ==
C     ==            .......  n_{na} [+1,-1,0]                         ==
C     ==  COORD     n1   K  RC  C0  [+1,-1,0] GROWTH aa DEST Cd       ==
C     ==  COORSP    n1  is  K   RC  C0  [+1,-1,0] GROWTH aa DEST Cd   ==
C     ==  COOR_RF   n1  is  N   M   RC  C0  [+1,-1,0] GROWTH aa DEST Cd=
C     ==  BNSWT     n1  n2  N   M   RC  C0  [+1,-1,0]                  =
C     ==  TOT_COOR  is1 is2 N   M   RC  C0  [+1,-1,0] GROWTH aa DEST Cd=
C     ==  MMFIELD   n1   F0                                           ==
C     ==  ...                                                         ==
C     ==  RESTRAINTS                                                  ==
C     ==   nres                                                       ==
C     ==  DIST      n1  n2   R0  kval       GROWTH aa DEST Rd         ==
C     ==  STRETCH   n1  n2   R0  kval       GROWTH aa DEST Rd         ==
C     ==  BEND      n1  n2  n3 A0  kval     GROWTH aa DEST Ad         ==
C     ==  TORSION   n1  n2  n3  n4  D0 kval GROWTH aa DEST Dd         ==
C     ==  OUTP      n1  n2  n3  n4  D0 kval GROWTH aa DEST Dd         ==
C     ==  COOR_RF   n1  is  N   M   RC  C0  kval GROWTH aa DEST Cd    ==
C     ==  DIFFER    n1  n2  n3  dR0 kval    GROWTH aa DEST DdRd       ==
C     ==  DISAXIS   n1  n2  k   R0  kval    GROWTH aa DEST Rd         ==
C     ==  RESPOS    n1  x0  y0  z0  D0 kval (NO GROWTH ALLOWED)       ==
C     ==  ...                                                         ==
C     ==  PENALTY                                                     ==
C     ==   dsigma bsigma tsigma                                       ==
C     ==                                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'coor.inc'
      INCLUDE 'dpot.inc'
      INCLUDE 'atom.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'mm_dim.inc'
      INCLUDE 'mm_input.inc'
      INCLUDE 'fileopen.inc'
C     Arguments
      INTEGER   IUNIT
C     Variables
      INTEGER   LFIX(*),NARIG(*)
      POINTER   (IP_LFIX,LFIX),(IP_NARIG,NARIG)
      CHARACTER LINE*80
      INTEGER   NFIX,IDUMMY,I,NAFIX,I1,IOUT,NRIG,NRES,MY_NAT
      INTEGER   ig,IFIRST,ILAST,IDATA(4)
      LOGICAL   ERREAD,status,FERROR
      REAL*8    KPLANE
C     ==--------------------------------------------------------------==
      DSIGMA=1.D0
      BSIGMA=5.D0
      TSIGMA=5.D0
      LFCOM=.FALSE.
      LSHOVE=.FALSE.

C     FIXME: AK 2005/05/25 TODO LIST:
c     - make all atom index inputs run through CPMD_nat()
c     - check all arguments for out-of-range errors.
c     - fix and check documentation

C     get total number of atoms and set indexing.
      if(qmmm) then
        call mm_dim(mm_go_mm,status)
        MY_NAT=NAT
      else
        MY_NAT=NATm
      endif
      CALL MEMORY(IP_LFIX,(MY_NAT+1)/IRAT,'LFIX')
C     ==--------------------------------------------------------------==
   10 CONTINUE
      READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
      IF(INDEX(LINE,'END').NE.0 .AND. INDEX(LINE,'CONSTR').NE.0)
     &     GOTO 30
C     FIX ALL and FIX QM MM are the same
      IF(INDEX(LINE,'FIX').NE.0 .AND. (INDEX(LINE,'ALL').NE.0 .OR.
     &     (INDEX(LINE,'QM').NE.0.AND.INDEX(LINE,'MM').NE.0)) ) THEN
        DO I=1,MY_NAT
          LSKCOR(1,I)=0
          LSKCOR(2,I)=0
          LSKCOR(3,I)=0
        ENDDO
        GOTO 10
      ELSEIF(INDEX(LINE,'FIX').NE.0 .AND. INDEX(LINE,'QM').NE.0) THEN
        IF(QMMM) THEN
          IDUMMY=NATq
        ELSE
          IDUMMY=MY_NAT
        ENDIF
        DO I=1,IDUMMY
          LSKCOR(1,I)=0
          LSKCOR(2,I)=0
          LSKCOR(3,I)=0
        ENDDO
        GOTO 10
      ELSEIF(INDEX(LINE,'FIX').NE.0 .AND. INDEX(LINE,'MM').NE.0) THEN
        IF(QMMM) THEN
          DO I=NATq+1,NATm
            LSKCOR(1,I)=0
            LSKCOR(2,I)=0
            LSKCOR(3,I)=0
          ENDDO
        ENDIF
        GOTO 10
      ELSEIF(INDEX(LINE,'FIX').NE.0 .AND. INDEX(LINE,'SOLU').NE.0) THEN
        IF(QMMM) THEN
          DO I=1,NRPT
            LSKCOR(1,I)=0
            LSKCOR(2,I)=0
            LSKCOR(3,I)=0
          ENDDO
        ENDIF
        GOTO 10
      ELSEIF(INDEX(LINE,'FIX').NE.0.AND.INDEX(LINE,'ELEM').NE.0)THEN
        IF(INDEX(LINE,'SEQ').NE.0) THEN
          read(IUNIT,ERR=20,FMT=*) IDUMMY,IFIRST,ILAST
        ELSE
          read(IUNIT,ERR=20,FMT=*) IDUMMY
          IFIRST=1
          ILAST=MY_NAT
        ENDIF
        DO ig=IFIRST,ILAST
          I=NAT_cpmd(ig)
          IF(IATYP(cpsp(I)).eq.IDUMMY)THEN
            LSKCOR(1,I)=0
            LSKCOR(2,I)=0
            LSKCOR(3,I)=0
          ENDIF
        ENDDO
        GOTO 10
      ELSEIF(INDEX(LINE,'FIX').NE.0.AND.INDEX(LINE,'PPTY').NE.0)THEN
        IF(INDEX(LINE,'SEQ').NE.0) THEN
          read(IUNIT,ERR=20,FMT=*) IDUMMY,IFIRST,ILAST
        ELSE
          read(IUNIT,ERR=20,FMT=*) IDUMMY
          IFIRST=1
          ILAST=MY_NAT
        ENDIF
        DO ig=IFIRST,ILAST
          I=NAT_cpmd(ig)
          if(cpsp(I).eq.IDUMMY)then
            LSKCOR(1,I)=0
            LSKCOR(2,I)=0
            LSKCOR(3,I)=0
          endif
        enddo
        GOTO 10
      ELSEIF(INDEX(LINE,'FIX').NE.0.AND.INDEX(LINE,'SEQ').NE.0)THEN
        read(IUNIT,ERR=20,FMT=*)IFIRST,ILAST
        do ig=IFIRST,ILAST
          I=NAT_cpmd(ig)
          LSKCOR(1,I)=0
          LSKCOR(2,I)=0
          LSKCOR(3,I)=0
        enddo
        GOTO 10
      ELSE IF(INDEX(LINE,'FIX').NE.0 .AND. INDEX(LINE,'ATOM').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) IDUMMY, (LFIX(I),I=1,IDUMMY)
        DO IG=1,IDUMMY
          I=NAT_cpmd(LFIX(IG))
          LSKCOR(1,I)=0
          LSKCOR(2,I)=0
          LSKCOR(3,I)=0        
        ENDDO
        GOTO 10
      ELSEIF(INDEX(LINE,'FIX').NE.0 .AND. INDEX(LINE,'COOR').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) IDUMMY
        DO I=1,IDUMMY
          READ(IUNIT,ERR=20,END=20,FMT=*) NAFIX,(IDATA(ig),ig=1,3)
          LSKCOR(1,NAT_cpmd(NAFIX))=IDATA(1)
          LSKCOR(2,NAT_cpmd(NAFIX))=IDATA(2)
          LSKCOR(3,NAT_cpmd(NAFIX))=IDATA(3)
        ENDDO
        GOTO 10
      ELSEIF(INDEX(LINE,'FIX').NE.0 .AND. INDEX(LINE,'COM').NE.0) THEN
        LFCOM=.TRUE.
        GOTO 10
      ELSEIF(INDEX(LINE,'FIX').NE.0 .AND. INDEX(LINE,'STRU').NE.0) THEN
        IF(INDEX(LINE,'SHOVE').NE.0) LSHOVE=.TRUE.
        READ(IUNIT,ERR=20,END=20,FMT=*) NFIX
        CALL MEMORY(IP_NTCNST,6*NFIX/IRAT+1,'NTCNST')
        CALL MEMORY(IP_CNSVAL,NFIX,'CNSVAL')
        CALL MEMORY(IP_CNPAR,2*NFIX,'CNPAR')
        CALL MEMORY(IP_GRATE,NFIX,'GRATE')
        CALL MEMORY(IP_CNSVAL_DEST,NFIX,'CNSVAL_DEST')
        CALL AZZERO(CNPAR,2*NFIX)
        CALL AZZERO(GRATE,NFIX)
        CALL IAZZERO(NTCNST,6*NFIX)
!$OMP parallel do private(I)
        DO I=1,NFIX
          CNSVAL(I)=-999.D0
          CNSVAL_DEST(I)=-999.D0
        ENDDO
        MCNSTR=0
C       ==------------------------------------------------------------==
C       Read each structure.
   11   CONTINUE
        READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
        IF(INDEX(LINE,'END').NE.0 .AND. INDEX(LINE,'CONSTR').NE.0) THEN
C         Problem the number of constraints MCNSTR .NE. given NFIX.
          WRITE(6,'(1X,A,I5)') 'CONINP! # OF GIVEN CONSTRAINTS=',NFIX
          WRITE(6,'(1X,A,I5)') 'CONINP! # OF FOUND CONSTRAINTS=',MCNSTR
          WRITE(6,'(1X,A)')    'CONINP! END CONSTRAINTS IS REACHED.'
          CALL STOPGM('CONINP','ERROR WHILE READING FIX STRUCTURES')
C       ==------------------- DIST ---------------------------------==
        ELSEIF(INDEX(LINE,'DIST').NE.0) THEN
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)=4
          I1=INDEX(LINE,'DIST')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(2,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(3,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MCNSTR),ERREAD)
            I1=INDEX(LINE,'DEST') 
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,CNSVAL_DEST(MCNSTR),ERREAD)
            ENDIF
          ENDIF
        ELSEIF(INDEX(LINE,'DISAXIS').NE.0) THEN
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)=12
          I1=INDEX(LINE,'DISAXIS')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(2,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(3,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(4,MCNSTR)=IDUMMY
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MCNSTR),ERREAD)
            I1=INDEX(LINE,'DEST') 
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,CNSVAL_DEST(MCNSTR),ERREAD)
            ENDIF
          ENDIF
C       ==------------------- SM -----------------------------------==
        ELSEIF(INDEX(LINE,'DIFFER').NE.0) THEN
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)=7
          I1=INDEX(LINE,'DIFFER')+6
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(2,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(3,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(4,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MCNSTR),ERREAD)
            I1=INDEX(LINE,'DEST') 
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,CNSVAL_DEST(MCNSTR),ERREAD)
            ENDIF
          ENDIF
C       ==------------------ END SM ----------------------------------==
        ELSEIF(INDEX(LINE,'STRETCH').NE.0) THEN
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)=1
          I1=INDEX(LINE,'STRETCH')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(2,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(3,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MCNSTR),ERREAD)
            I1=INDEX(LINE,'DEST') 
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,CNSVAL_DEST(MCNSTR),ERREAD)
            ENDIF
          ENDIF
        ELSEIF(INDEX(LINE,'BEND').NE.0) THEN
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)=2
          I1=INDEX(LINE,'BEND')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(2,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(3,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(4,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          CALL RADDEG(CNSVAL(MCNSTR),-1)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MCNSTR),ERREAD)
            CALL RADDEG(GRATE(MCNSTR),-1)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,CNSVAL_DEST(MCNSTR),ERREAD)
              CALL RADDEG(CNSVAL_DEST(MCNSTR),-1)
            ENDIF
          ENDIF
        ELSEIF(INDEX(LINE,'TORSION').NE.0) THEN
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)=3
          I1=INDEX(LINE,'TORSION')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(2,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(3,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(4,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(5,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          CALL RADDEG(CNSVAL(MCNSTR),-1)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MCNSTR),ERREAD)
            CALL RADDEG(GRATE(MCNSTR),-1)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,CNSVAL_DEST(MCNSTR),ERREAD)
              CALL RADDEG(CNSVAL_DEST(MCNSTR),-1)
            ENDIF
          ENDIF
        ELSEIF(INDEX(LINE,'OUTP').NE.0) THEN
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)=5
          I1=INDEX(LINE,'OUTP')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(2,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(3,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(4,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(5,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          CALL RADDEG(CNSVAL(MCNSTR),-1)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MCNSTR),ERREAD)
            CALL RADDEG(GRATE(MCNSTR),-1)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,CNSVAL_DEST(MCNSTR),ERREAD)
              CALL RADDEG(CNSVAL_DEST(MCNSTR),-1)
            ENDIF
          ENDIF
        ELSEIF(INDEX(LINE,'RIGID').NE.0) THEN
          I1=INDEX(LINE,'RIGID')+5
          CALL READSI(LINE,I1,IOUT,NRIG,ERREAD)
          IF(ERREAD) GOTO 20
          CALL MEMORY(IP_NARIG,NRIG/IRAT+1,'NARIG')
          DO I=1,NRIG
            I1=IOUT
            CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
            NARIG(I)=NAT_cpmd(IDUMMY)
            IF(ERREAD) THEN
C             We read the next line
              READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
              I1=1
              CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
              NARIG(I)=NAT_cpmd(IDUMMY)
              IF(ERREAD) GOTO 20
            ENDIF
          ENDDO
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          CALL SETRIG(MCNSTR,NRIG,NARIG,NTCNST)
          CALL FREEM(IP_NARIG)
        ELSEIF(INDEX(LINE,'COORD').NE.0) THEN
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)=6
          I1=INDEX(LINE,'COORD')+5
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(2,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNPAR(1,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNPAR(2,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MCNSTR),ERREAD)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,CNSVAL_DEST(MCNSTR),ERREAD)
            ENDIF
          ENDIF
C       ==------------ SPECIES DEPENDENT COORDINATION NUMBER  -----------==
C       ==                 f = 1/(1+exp(k*(R-R_0)))                      ==
        ELSEIF(INDEX(LINE,'COORSP').NE.0) THEN
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)= 8
          I1=INDEX(LINE,'COORSP')+6
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(2,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTCNST(3,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNPAR(1,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNPAR(2,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MCNSTR),ERREAD)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,CNSVAL_DEST(MCNSTR),ERREAD)
            ENDIF
          ENDIF
C       ==------------ SPECIES DEPENDENT COORDINATION NUMBER -------------==
C       ==            f =sum_i (1+(R/R_0)^n)/(1+(R/R_0)^(n+m))
        ELSEIF(INDEX(LINE,'COOR_RF').NE.0) THEN
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)= 9
          I1=INDEX(LINE,'COOR_RF')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(2,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTCNST(3,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTCNST(4,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTCNST(5,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNPAR(1,MCNSTR),ERREAD)
          I1=IOUT
c          II=INDEX(LINE,'2SHELL')
c          IF(II.NE.0) THEN
c            CALL READSI(LINE,I1,IOUT,CNPAR(2,MCNSTR),ERREAD)
c            I1=IOUT
c          ENDIF

          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MCNSTR),ERREAD)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,CNSVAL_DEST(MCNSTR),ERREAD)
            ENDIF
          ENDIF
C       ==--------------- BOND SWITCH WITH RATIONAL F -------------------==
C       ==            f = (1+(R/R_0)^n)/(1+(R/R_0)^(n+m))
        ELSEIF(INDEX(LINE,'BNSWT').NE.0) THEN
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)= 10
          I1=INDEX(LINE,'BNSWT')+5
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(2,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTCNST(3,MCNSTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTCNST(4,MCNSTR),ERREAD)
          I1=IOUT
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTCNST(5,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNPAR(1,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
C       ==--------- TOTAL SPECIES DEPENDENT COORDINATION NUMBER ---------==
C       ==            f =SUM_j[sum_i (1+(R/R_0)^n)/(1+(R/R_0)^(n+m))]    ==
        ELSEIF(INDEX(LINE,'TOT_COOR').NE.0) THEN
          MCNSTR = MCNSTR + 1
          NTCNST(1,MCNSTR) = 11
          I1=INDEX(LINE,'TOT_COOR') + 8
          CALL READSI(LINE,I1,IOUT,NTCNST(2,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTCNST(3,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTCNST(4,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTCNST(5,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNPAR(1,MCNSTR),ERREAD)
          I1=IOUT
c          II=INDEX(LINE,'2SHELL')
c          IF(II.NE.0) THEN
c            CALL READSI(LINE,I1,IOUT,CNPAR(2,MCNSTR),ERREAD)
c            I1=IOUT
c          ENDIF
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
          I1=IOUT
          IF(LSHOVE) CALL READSI(LINE,I1,IOUT,NTCNST(6,MCNSTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MCNSTR),ERREAD)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,CNSVAL_DEST(MCNSTR),ERREAD)
            ENDIF
          ENDIF
        ELSEIF(INDEX(LINE,'MMFIELD').NE.0) THEN
C     FIXME. this type of contraint seems not to be used.
C     initially, this was set to type no. 8, but that is
C     now spoken for. from now on all QM/MM special contraints
C     start with a type no. > 100.  AK 2005/05/12.
          MCNSTR=MCNSTR+1
          NTCNST(1,MCNSTR)= 101
          I1=INDEX(LINE,'MMFIELD')+7
          CALL READSI(LINE,I1,IOUT,NTCNST(2,MCNSTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CNSVAL(MCNSTR),ERREAD)
        ENDIF
        IF(MCNSTR.LT.NFIX) THEN
          GOTO 11
        ELSE
          GOTO 10
        ENDIF
C     ==-- END OF FIX STRUCTURES -------------------------------------==
      ELSEIF(INDEX(LINE,'RESTRAINT').NE.0) THEN
        LHYPERPLANE=.FALSE.
        IF (INDEX(LINE,'HYPER').NE.0) THEN 
           LHYPERPLANE=.TRUE.
           KPLANE=1.0D0
           I1=INDEX(LINE,'K=') 
           IF (I1.NE.0) THEN
              I1=I1+2
              CALL READSR(LINE,I1,IOUT,KPLANE,ERREAD)
           ENDIF
        ENDIF
        READ(IUNIT,ERR=20,END=20,FMT=*) NRES
        CALL MEMORY(IP_NTREST,6*NRES/IRAT+1,'NTREST')
        CALL MEMORY(IP_RESVAL,NRES,'RESVAL')
        CALL MEMORY(IP_RESFOR,NRES,'RESFOR')
        CALL MEMORY(IP_RESPAR,2*NRES,'RESPAR')
        CALL MEMORY(IP_GSRATE,NRES,'GSRATE')
        CALL MEMORY(IP_RESVAL_DEST,NRES,'RESVAL_DEST')
        CALL MEMORY(IP_RESPOS,3*NRES,'RESPOS') ! cmb-kk
        CALL AZZERO(RESPAR,2*NRES)
        CALL AZZERO(GSRATE,NRES)
        CALL IAZZERO(NTREST,6*NRES)
        DO I=1,NRES
          RESVAL(I)=-999.D0
          RESVAL_DEST(I)=-999.D0
        ENDDO
        MRESTR=0
C       ==------------------------------------------------------------==
C       Read each structure.
   21   CONTINUE
        READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
        IF(INDEX(LINE,'END').NE.0 .AND. INDEX(LINE,'CONSTR').NE.0) THEN
C         Problem the number of constraints MCNSTR .NE. given NFIX.
          WRITE(6,'(1X,A,I5)') 'CONINP! # OF GIVEN RESTRAINTS=',NRES
          WRITE(6,'(1X,A,I5)') 'CONINP! # OF FOUND RESTRAINTS=',MRESTR
          WRITE(6,'(1X,A)')    'CONINP! END CONSTRAINTS IS REACHED.'
          CALL STOPGM('CONINP','ERROR WHILE READING RESTRAINTS')
C       ==------------------- DIST ---------------------------------==
        ELSEIF(INDEX(LINE,'DIST').NE.0) THEN
          MRESTR=MRESTR+1
          NTREST(1,MRESTR)=4
          I1=INDEX(LINE,'DIST')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(2,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(3,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESVAL(MRESTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESFOR(MRESTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GSRATE(MRESTR),ERREAD)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,RESVAL_DEST(MRESTR),ERREAD)
            ENDIF
          ENDIF
C       ==------------------- SM -----------------------------------==
        ELSEIF(INDEX(LINE,'DIFFER').NE.0) THEN
          MRESTR=MRESTR+1
          NTREST(1,MRESTR)=7
          I1=INDEX(LINE,'DIFFER')+6
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(2,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(3,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(4,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESVAL(MRESTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESFOR(MRESTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GSRATE(MRESTR),ERREAD)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,RESVAL_DEST(MRESTR),ERREAD)
            ENDIF
          ENDIF
C       ==------------------ END SM ----------------------------------==
C       ==------------------- RESPOS ---------------------------------==
        ELSEIF(INDEX(LINE,'RESPOS').NE.0) THEN ! cmb-kk
          MRESTR=MRESTR+1
          NTREST(1,MRESTR)=13      ! RESPOS is changed to 13
          I1=INDEX(LINE,'RESPOS')+6
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(2,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESPOS(1,MRESTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESPOS(2,MRESTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESPOS(3,MRESTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESVAL(MRESTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESFOR(MRESTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
           WRITE(6,'(A)') 
     &           ' WARNING: RESPOS IS NOT SUPPOSED TO BE USED',
     &           ' ALONG WITH THE KEYWORD [GROWTH]'
           CALL STOPGM('CONINP','ERROR WHILE READING RESTRAINTS')
          ENDIF
C       ==------------------ END RESPOS ------------------------------==
        ELSEIF(INDEX(LINE,'STRETCH').NE.0) THEN
          MRESTR=MRESTR+1
          NTREST(1,MRESTR)=1
          I1=INDEX(LINE,'STRETCH')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(2,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(3,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESVAL(MRESTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESFOR(MRESTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GSRATE(MRESTR),ERREAD)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,RESVAL_DEST(MRESTR),ERREAD)
            ENDIF
          ENDIF
        ELSEIF(INDEX(LINE,'BEND').NE.0) THEN
          MRESTR=MRESTR+1
          NTREST(1,MRESTR)=2
          I1=INDEX(LINE,'BEND')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(2,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(3,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(4,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESVAL(MRESTR),ERREAD)
          CALL RADDEG(RESVAL(MRESTR),-1)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESFOR(MRESTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GSRATE(MRESTR),ERREAD)
            CALL RADDEG(GSRATE(MRESTR),-1)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,RESVAL_DEST(MRESTR),ERREAD)
              CALL RADDEG(RESVAL_DEST(MRESTR),-1)
            ENDIF
          ENDIF
        ELSEIF(INDEX(LINE,'TORSION').NE.0) THEN
          MRESTR=MRESTR+1
          NTREST(1,MRESTR)=3
          I1=INDEX(LINE,'TORSION')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(2,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(3,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(4,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(5,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESVAL(MRESTR),ERREAD)
          CALL RADDEG(RESVAL(MRESTR),-1)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESFOR(MRESTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GSRATE(MRESTR),ERREAD)
            CALL RADDEG(GSRATE(MRESTR),-1)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,RESVAL_DEST(MRESTR),ERREAD)
              CALL RADDEG(RESVAL_DEST(MRESTR),-1)
            ENDIF
          ENDIF
        ELSEIF(INDEX(LINE,'OUTP').NE.0) THEN
          MRESTR=MRESTR+1
          NTREST(1,MRESTR)=5
          I1=INDEX(LINE,'OUTP')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(2,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(3,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(4,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(5,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESVAL(MRESTR),ERREAD)
          CALL RADDEG(RESVAL(MRESTR),-1)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESFOR(MRESTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GSRATE(MRESTR),ERREAD)
            CALL RADDEG(GSRATE(MRESTR),-1)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,RESVAL_DEST(MRESTR),ERREAD)
              CALL RADDEG(RESVAL_DEST(MRESTR),-1)
            ENDIF
          ENDIF
        ELSEIF(INDEX(LINE,'COOR_RF').NE.0) THEN
          MRESTR=MRESTR+1
          NTREST(1,MRESTR)=9
          I1=INDEX(LINE,'COOR_RF')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(2,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTREST(3,MRESTR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTREST(4,MRESTR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,NTREST(5,MRESTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESPAR(1,MRESTR),ERREAD)
          I1=IOUT
c         II=INDEX(LINE,'2SHELL')
c         IF(II.NE.0) THEN
c           CALL READSR(LINE,I1,IOUT,RESPAR(2,MRESTR),ERREAD)
c           I1=IOUT
c         ENDIF
          CALL READSR(LINE,I1,IOUT,RESVAL(MRESTR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESFOR(MRESTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MRESTR),ERREAD)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,RESVAL_DEST(MRESTR),ERREAD)
            ENDIF
          ENDIF
        ELSEIF(INDEX(LINE,'DISAXIS').NE.0) THEN
          MRESTR=MRESTR+1
          NTREST(1,MRESTR)=12
          I1=INDEX(LINE,'DISAXIS')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(2,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(3,MRESTR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          NTREST(4,MRESTR)=IDUMMY
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,RESVAL(MRESTR),ERREAD)
          I1=INDEX(LINE,'GROWTH')
          IF(I1.NE.0) THEN
            I1=I1+6
            CALL READSR(LINE,I1,IOUT,GRATE(MRESTR),ERREAD)
            I1=INDEX(LINE,'DEST')
            IF(I1.NE.0) THEN
              I1=I1+4
              CALL READSR(LINE,I1,IOUT,RESVAL_DEST(MRESTR),ERREAD)
            ENDIF
          ENDIF
        ENDIF
        IF(MRESTR.LT.NRES) THEN
          GOTO 21
        ELSE
C     read new restraint target values from file RESVAL
          CALL FILEOPEN(91,'RESVAL',FO_OLD,FERROR)
          IF (.NOT.FERROR) THEN
            WRITE(6,*)'CONINP| READING RESTRAINT TARGET VALUES ',
     &          'FROM FILE "RESVAL"'
            READ(91,ERR=20,END=20,FMT=*) (RESVAL(I),I=1,MRESTR)
            DO I=1,MRESTR 
              IF (NTREST(1,I).EQ.2
     &            .OR.NTREST(1,I).EQ.3
     &            .OR.NTREST(1,I).EQ.5) 
     &            CALL RADDEG(RESVAL(I),-1)
            ENDDO
            CALL FILECLOSE(91)
          ENDIF
          IF (LHYPERPLANE) THEN
            WRITE(6,'(2A,F6.2)')' CONINP| USING HYPERPLANE TYPE ',
     &          'RESTRAINTS WITH SCALING FACTOR: ',KPLANE
             DO I=1,MRESTR 
                RESFOR(I)=RESFOR(I)*KPLANE
             ENDDO
          ENDIF
          GOTO 10
        ENDIF
C     ==-- END OF RESTRAINTS -----------------------------------------==
      ELSEIF(INDEX(LINE,'PENALTY').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) DSIGMA,BSIGMA,TSIGMA
        GOTO 10
      ELSE
        GOTO 10
      ENDIF
   20 CONTINUE
        WRITE(6,*) ' ERROR WHILE READING CONSTRAINTS SECTION'
        CALL STOPGM('CONINP','INPUT FILE ERROR')
   30 CONTINUE
C     Free memory
      CALL FREEM(IP_LFIX)
      if(qmmm) call mm_dim(mm_revert,status)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE RADDEG(A,ITAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'cnst.inc'
C     Arguments
      INTEGER ITAG
      REAL*8  A
C     ==--------------------------------------------------------------==
      IF(A.EQ.-999.D0) RETURN
      IF(ITAG.EQ.1) THEN
        A=A/PI*180.D0
      ELSEIF(ITAG.EQ.-1) THEN
        A=A/180.0D0*PI
      ELSE
        CALL STOPGM('RADDEG','OPTION')
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SETRIG(M,NRIG,NARIG,NTCNST)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER M,NRIG,NARIG(NRIG),NTCNST(6,M+NRIG-1+NRIG-2+NRIG-3)
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
C     Stretch
      DO I=2,NRIG
        M=M+1
        NTCNST(1,M)=1
        NTCNST(2,M)=NARIG(I-1)
        NTCNST(3,M)=NARIG(I)
      ENDDO
C     Bend   
      DO I=3,NRIG
        M=M+1
        NTCNST(1,M)=2
        NTCNST(2,M)=NARIG(I-2)
        NTCNST(3,M)=NARIG(I-1)
        NTCNST(4,M)=NARIG(I)
      ENDDO
C     Torsion
      DO I=4,NRIG
        M=M+1
        NTCNST(1,M)=3
        NTCNST(2,M)=NARIG(I-3)
        NTCNST(3,M)=NARIG(I-2)
        NTCNST(4,M)=NARIG(I-1)
        NTCNST(5,M)=NARIG(I)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
