#ifdef __SR8000
*option MP(P(0)), LANGLVL(SAVE(0))
#endif
C     ==================================================================
      SUBROUTINE CONTROL
C     ==--------------------------------------------------------------==
C     ==  THIS ROUTINE READS THE INPUT FILE                           ==
C     ==--------------------------------------------------------------==
C     ==  THE INPUT HAS TO BE IN THE FOLLOWING FORMAT                 ==
C     ==     &CPMD                                                    ==
C     ==        Options                                               ==
C     ==     &END                                                     ==
C     ==--------------------------------------------------------------==
C     ==  LIST OF OPTIONS                                             ==
C     ==                                                              ==
C     ==    MOLECULAR DYNAMICS [CP,BO,PT,                             ==
C     ==                        FILE XYZ [NSKIP=N] [NSAMPLE=N],       ==
C     ==                        CLASSICAL]                            ==
C     ==    PARRINELLO-RAHMAN NPT,SHOCK                               ==
C     ==    OPTIMIZE {GEOMETRY,WAVEFUNCTION,COMBINED} [XYZ] [SAMPLE]  ==
C     ==      ngxyz                                                   ==
C     ==    FREE ENERGY FUNCTIONAL                                    ==
C     ==    INTERFACE {EGO,GMX} [MULLIKEN,LOWDIN,ESP,HIRSHFELD]       ==
C     ==    PATH SAMPLING                                             ==
C     ==    KOHN-SHAM ENERGIES {OFF} [NOWAVEFUNCTION]                 ==
C     ==      nkssta                                                  ==
C     ==    VIBRATIONAL ANALYSIS [LR,FD,IN] [GAUSS]                   ==
C     ==      nvib                                                    ==
C     ==    PROPERTIES                                                ==
C     ==    FORCEMATCH                                                ==
C     ==    DEBUG FORCES                                              ==
C     ==    DEBUG [CODE] [MEMORY] [RESTART] [FILEOPEN] [NOACC] [IO]   ==
C     ==    ORBITAL HARDNESS [LR,FD]                                  ==
C     ==    ELECTRONIC SPECTRA                                        ==
C     ==    PATH INTEGRALS                                            ==
C     ==    PATH MINIMISATION                                         ==
C     ==    SURFACE HOPPING                                           ==
C     ==    QMMM [QMMMEASY]                                           ==
C     ==    LINEAR RESPONSE                                           ==
C     ==    NONORTHOGONAL ORBITALS {OFF}                              ==
C     ==       slimit                                                 ==
C     ==    LSD                                                       ==
C     ==    LOCAL SPIN DENSITY                                        ==
C     ==    HARMONIC REFERENCE SYSTEM {OFF}                           ==
C     ==    SCALED MASSES {OFF}                                       ==
C     ==    STEEPEST DESCENT [ELECTRONS,IONS,CELL]                    ==
C     ==                     [NOPRECONDITIONING] [LINE]               ==
C     ==      or {TSDE,TSDP} [LINE]                                   ==
C     ==      or {TSDC}                                               ==
C     ==    CONJUGATE GRADIENT [ELECTRONS,IONS]                       ==
C     ==                     [MINIMIZE,NOPRECONDITIONING]             ==
C     ==      or PCG  [MINIMIZE,NOPRECONDITIONING]                    ==
C     ==      or TCGP                                                 ==
C     ==    ODIIS [NOPRECONDITIONING,NO_RESET=nreset]                 ==
C     ==      mdiis                                                   ==
C     ==    HAMILTONIAN CUTOFF                                        ==
C     ==      hthrs                                                   ==
C     ==    GDIIS                                                     ==
C     ==      mgdiis                                                  ==
C     ==    BFGS                                                      ==
C     ==    LBFGS [NREM,NTRUST,NRESTT,NTRSTR]                         ==
C     ==      nrem|ntrust|nrestt|ntrstr                               ==
C     ==    PRFO [NVAR,MODE,MDLOCK,TOLENV,TRUSTP,OMIN,CORE,NSMAXP,    ==
C     ==          NSVIB,PRJHES,DISPL,HESSTYPE]                        ==
C     ==      nvar|mode|mdlock|tolenv|trustp|omin|core|nsmaxp|m_hess  ==
C     ==    PRFO CORE=natcor                                          ==
C     ==      <atoms_list>                                            ==
C     ==    HESSCORE                                                  ==
C     ==    RFO [ORDER=nsorder]                                       ==
C     ==    IMPLICIT NEWTON RAPHSON []                                ==
C     ==        [PREC,CONTINUE,VERBOSE,ALTERNATIVE STEP]              ==
C     ==      itmax_inr                                               ==
C     ==    IMPLICIT NEWTON RAPHSON PARAMETERS N=nreg                 ==
C     ==      gnx_inr(i)   tolx_inr(i)                                ==
C     ==          ...                                                 ==
C     ==    MIXDIIS                                                   ==
C     ==      rmixsd (USED BY INR)                                    ==
C     ==    MIXSD                                                     ==
C     ==      rmixsd (USED BY INR)                                    ==
C     ==    RESTART [WAVEFUNCTION,COORDINATES,DENSITY,ACCUMULATORS,   ==
C     ==             HESSIAN,VELOCITIES,NOSEE,NOSEP,GEOFILE,KPOINTS,  ==
C     ==             VIBANALYSIS,CELL,NOSEC,POTENTIAL,OCCUPATION,     ==
C     ==             ORBHARDNESS,PHESSIAN,LSSTAT,ADPTTL,EXTRAP,GLE,   ==
C     ==             PRNG,LATEST]                                     ==
C     ==    INTFILE {READ,WRITE,FILENAME}                             ==
C     ==      intfn                                                   ==
C     ==    FILE FUSION                                               ==
C     ==      s0_filn                                                 ==
C     ==      s1_filn                                                 ==
C     ==    FILE MERGE                                                ==
C     ==      mfiln1                                                  ==
C     ==      mfiln2                                                  ==
C     ==      mnst1 mnst2 mortho                                      ==
C     ==      mshift(1) mshift(2) mshift(3)                           ==
C     ==    INITIALIZE WAVEFUNCTIONS {RANDOM,ATOMS}                   ==
C     ==    MAXSTEP                                                   ==
C     ==      nomore                                                  ==
C     ==    MAXITER                                                   ==
C     ==      nomore_iter                                             ==
C     ==    MAXCPUTIME                                                ==
C     ==      tjlimit                                                 ==
C     ==    ORTHOGONALIZATION {LOWDIN,GRAM-SCHMIDT}                   ==
C     ==    RATTLE                                                    ==
C     ==      [ maxit  epsog ]                                        ==
C     ==    QUENCH [IONS, ELECTRONS, CELL, BO]                        ==
C     ==    BOGOLIUBOV CORRECTION {OFF}                               ==
C     ==    TROTTER FACTORISATION OFF                                 ==
C     ==    TROTTER FACTOR                                            ==
C     ==      betap                                                   ==
C     ==    TROTTER FACTOR N=ntabbetap                                ==
C     ==      densbetap(1)         tabbetap(1)                        ==
C     ==          ...                                                 ==
C     ==      densbetap(ntabbetap) tabbetap(ntabbetap)                ==
C     ==    LANCZOS DIAGONALIZATION [ALL,NEW,OPT,PART,PERI,RESET]     ==
C     ==    LANCZOS PARAMETER [ALL]                                   ==
C     ==      n_fries nkry_max nkry_block b2limit                     ==
C     ==    LANCZOS PARAMETER   N=ntabtrot [ALL]                      ==
C     ==      n_fries nkry_max  b2limit                               ==
C     ==     denstrot(2)        b2trot(2)                             ==
C     ==          ...                                                 ==
C     ==     denstrot(ntabtrot) b2trot(ntabtrot)                      ==
C     ==    DAVIDSON DIAGONALIZATION                                  ==
C     ==    DAVIDSON PARAMETER                                        ==
C     ==      ndavv epsdav n_fries                                    ==
C     ==    FIXRHO UPWFN                                              ==
C     ==    FIXRHO VECT                                               ==
C     ==       mdiis_fr                                               ==
C     ==    FIXRHO LOOP                                               ==
C     ==       minldiis  maxldiis                                     ==
C     ==    FIXRHO WFTOL                                              ==
C     ==       tolrhofix                                              ==
C     ==    ANDERSON MIXING [G-SPACE]                                 ==
C     ==     andrmix                                                  ==
C     ==    ANDERSON MIXING N=ntabmix                                 ==
C     ==     densmix(1)       andmix(1)                               ==
C     ==          ...                                                 ==
C     ==     densmix(ntabmix) andmix(ntabmix)                         ==
C     ==    DIIS MIXING                                               ==
C     ==     nrdiis                                                   ==
C     ==    DIIS MIXING N=ntabnrdiis                                  ==
C     ==     densnrdiis(1)          nrdiis(1)                         ==
C     ==          ...                                                 ==
C     ==     densnrdiis(ntabnrdiis) nrdiis(ntabmix)                   ==
C     ==    BROYDEN MIXING                                            ==
C     ==     broymix,ecutbroy,w02broy,nfrbroy,ibreset,kermix          ==
C     ==    BROYDEN MIXING                                            ==
C     ==     DEFAULT                                                  ==
C     ==    BROYDEN MIXING                                            ==
C     ==     [BROYMIX=broymix] [ECUTBROY=ecutbroy] [W02BROY=w02broy]  ==
C     ==     [NFRBROY=nfrbroy] [IBRESET=ibreset] [KERMIX=kermix]      ==
C     ==    ALEXANDER MIXING                                          ==
C     ==     alxmix                                                   ==
C     ==    MOVERHO                                                   ==
C     ==     dmovrmix                                                 ==
C     ==    EXTRAPOLATE WFN {ASPC,POLY} [STORE] [CSTEPS=naspc]        ==
C     ==     mextra                                                   ==
C     ==    RANDOMIZE [WAVEFUNCTION,COORDINATES,CELL,DENSITY]         ==
C     ==      [ ampre amprp amprc amprd ]                             ==
C     ==    TIMESTEP                                                  ==
C     ==      delt                                                    ==
C     ==    TIMESTEP ELECTRONS                                        ==
C     ==      delt_elec                                               ==
C     ==    TIMESTEP IONS                                             ==
C     ==      delt_ions                                               ==
C     ==    EMASS                                                     ==
C     ==      emass                                                   ==
C     ==    CMASS                                                     ==
C     ==      cmass                                                   ==
C     ==    TEMPERATURE  ELECTRON                                     ==
C     ==      { betael }                                              ==
C     ==    TEMPERATURE [RAMP]                                        ==
C     ==      { tempw trampt trampr }                                 ==
C     ==    RESCALE OLD VELOCITIES                                    ==
C     ==    REVERSE VELOCITIES                                        ==
C     ==    SUBTRACT COMVEL                                           ==
C     ==      ncomv                                                   ==
C     ==    SUBTRACT ROTVEL                                           ==
C     ==      nrotv                                                   ==
C     ==    PRNGSEED                                                  ==
C     ==      iprng                                                   ==
C     ==    LANGEVIN [WHITE,CPMD,OPTIMAL,CUSTOM] [MOVECM]             ==
C     ==      [ omega0, custom_ns ]                                   ==
C     ==    TEMPCONTROL {IONS,ELECTRONS,CELL}                         ==
C     ==      { tempw tolp , ekinw  toll , ekinhr tolc}               ==
C     ==    BERENDSEN {IONS,ELECTRONS,CELL}                           ==
C     ==      { tempw taubp, ekinw taube,  ekinhr taubc}              ==
C     ==    NOSE [IONS,ELECTRONS,CELL] [ULTRA,MASSIVE,CAFES]          ==
C     ==      { tempw wnosep , ekinw wnosee , tempc wnosec}           ==
C     ==    NOSE PARAMETERS                                           ==
C     ==      nchp nche nchb nedof ncalls nit                         ==
C     ==    PRINT [INFO,EIGENVALUES,COORDINATES,FORCES,WANNIER]       ==
C     ==      iprint_step                                             ==
C     ==    PRINT {ON,OFF} [INFO,EIGENVALUES,COORDINATES,FORCES,      ==
C     ==                    WANNIER]                                  ==
C     ==    PRINT ENERGY {ON,OFF} [EKIN,ELECTROSTATIC,ELEC1,ELEC2,    ==
C     ==                    ESR,ESELF,                                ==
C     ==                    EFREE,EBAND,ENTROPY,EPSEU,EHEP,EHEE,EHII, ==
C     ==                    ENL,EXC,VXC,EGC,EBOGO,ETOT1,ETOT2,ECAS,   ==
C     ==                    ETDDFT,EPEN]                              ==
C     ==    STRUCTURE [BONDS,ANGLES,DIHEDRALS,SELECT]                 ==
C     ==     natom                                                    ==
C     ==     n1, n2, n3, ... nn                                       ==
C     ==    ISOLATED MOLECULE                                         ==
C     ==    CENTER MOLECULE [OFF]                                     ==
C     ==    STORE [WAVEFUNCTIONS,DENSITY,POTENTIAL]                   ==
C     ==      istore [SC=isctore]                                     ==
C     ==    STORE OFF [WAVEFUNCTIONS,DENSITY,POTENTIAL]               ==
C     ==    RESTFILE [SAMPLE]                                         ==
C     ==      nrestf                                                  ==
C     ==      [s1 s2 ...]                                             ==
C     ==    CONVERGENCE {ORBITALS,GEOMETRY,CELL}                      ==
C     ==      tolog   tolng   tolcg                                   ==
C     ==    CONVERGENCE {ADAPT,ENERGY,CALFOR,RELAX,INITIAL}           ==
C     ==      tolad,tolene,tolfor,nstcnv,initial                      ==
C     ==    ANNEALING [IONS,ELECTRONS,CELL]                           ==
C     ==      {anneri,annere,annerc}                                  ==
C     ==    DAMPING [IONS,ELECTRONS,CELL]                             ==
C     ==      {dampgi,dampge,dampgc}                                  ==
C     ==    HESSIAN {DISCO,SCHLEGEL,UNIT} [PARTIAL]                   ==
C     ==    PROJECT {NONE,DIAGONAL,FULL}                              ==
C     ==    STRESS TENSOR                                             ==
C     ==      npres                                                   ==
C     ==    CLASSTRESS                                                ==
C     ==      nprec                                                   ==
C     ==    RHOOUT  [BANDS,SAMPLE=nrhooout]                           ==
C     ==      numbands                                                ==
C     ==      n1 n2 ... nx                                            ==
C     ==    ELF [PARAMETER]                                           ==
C     ==      elfcut elfeps                                           ==
C     ==    ENERGYBANDS                                               ==
C     ==    EXTERNAL POTENTIAL [ADD]                                  ==
C     ==    ELECTROSTATIC POTENTIAL  [SAMPLE=nrhooout]                ==
C     ==    DIPOLE DYNAMICS [SAMPLE,WANNIER]                          ==
C     ==      npdip                                                   ==
C     ==    WANNIER PARAMETER                                         ==
C     ==      w_step w_eps w_ran w_maxs                               ==
C     ==    WANNIER OPTIMISATION {SD,JACOBI}                          ==
C     ==    WANNIER SERIAL                                            ==
C     ==    WANNIER TYPE {VANDERBILT,RESTA}                           ==
C     ==    WANNIER REFERENCE                                         ==
C     ==      w_ref(1..3)                                             ==
C     ==    WANNIER DOS                                               ==
C     ==    WANNIER MOLECULAR                                         ==
C     ==    WANNIER WFNOUT [ALL,PARTIAL,LIST,DENSITY,SPREAD]          ==
C     ==      sw_orb                                                  ==
C     ==      sw_list(1) ...                                          ==
C     ==    TRAJECTORY [XYZ] [SAMPLE] {OFF}                           ==
C     ==      ntraj                                                   ==
C     ==    FINITE DIFFERENCES                                        ==
C     ==     fdiff COORD=coord_fdiff(1..3) RADIUS=r_fdiff             ==
C     ==    MOVIE [SAMPLE] {OFF}                                      ==
C     ==      imovie                                                  ==
C     ==    MEMORY {SMALL,BIG}                                        ==
C     ==    COMPRESS WRITEnn                                          ==
C     ==    FILEPATH                                                  ==
C     ==      fpath                                                   ==
C     ==    TASKGROUPS {MAXIMUM,MINIMUM,CARTESIAN}                    ==
C     ==      nogrp                                                   ==
C     ==    DISTRIBUTE FNL [ON,OFF]                                   ==
C     ==    SPLINE [POINTS QFUNCTION INIT RANGE]                      ==
C     ==      nsplp qsrang                                            ==
C     ==    REAL SPACE WFN [KEEP, SIZE]                               ==
C     ==      memsize                                                 ==
C     ==    BENCHMARK                                                 ==
C     ==     b1 b2 b3 b4 b5 b6 b7 b8 b9 b0                            ==
C     ==    CHECK MEMORY                                              ==
C     ==    MIRROR                                                    ==
C     ==    SHIFT POTENTIAL                                           ==
C     ==     dshift                                                   ==
C     ==    GLOCALIZATION PARAMETER                                   ==
C     ==      gloc_step gloc_eps gloc_ran gloc_maxs                   ==
C     ==    GLOCALIZATION OPTIMISATION                                ==
C     ==        {SD INTEGRATION,ALMOST IDENTITY}                      ==
C     ==    GFUNCTIONAL TYPE {GSPREAD,ZETANUMBER}                     ==
C     ==    UNITARITY CONSTRAINT                                      ==
C     ==       {TGLAGRANGE, TGORTHO, TGAPPROX}                        ==
C     ==        if tglagrange: G_LAGRANGE PARAMETERS                  ==
C     ==         epslag , gloc_maxit                                  ==
C     ==    RANDOM KICK                                               == 
C     ==       gloc_kick                                              ==
C     ==    G_CONJUGATE GRADIENT (only if SD INTEGRATION )            ==
C     ==    GLOCALIZE STARTING WAFEFUNCTION                           ==
C     ==      {complex, real}                                         ==
C     ==    G_STEP MODULATION                                         ==
C     ==       parameters                                             ==
C     ==    GLOCALAZED WFNOUT [ALL,PARTIAL,LIST,DENSITY]              ==
C     ==      tglocprint                                              ==
C     ==      gloc_orb                                                ==
C     ==      gloc_list(1) ...                                        ==
C     ==    NOGEOCHECK                                                ==
C     ==    ALLTOALL [SINGLE,DOUBLE]                                  ==
C     ==    DISTRIBUTED LINALG [ON,OFF]                               ==
C     ==    BLOCKSIZE STATES                                          ==
C     ==     nstblk                                                   ==
C     ==    GSHELL                                                    ==
C     ==    VDW CORRECTION                                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'struc.inc'
      INCLUDE 'prden.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'benc.inc'
      INCLUDE 'nort.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'nose.inc'
      INCLUDE 'qspl.inc'
      INCLUDE 'andr.inc'
      INCLUDE 'broy.inc' 
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'store.inc'
      INCLUDE 'wann.inc'
      INCLUDE 'envj.inc'        !TJLIMIT
      INCLUDE 'cotr.inc'        !TSDPLINE
      INCLUDE 'kpts.inc'        !TKNOSWAP
      INCLUDE 'prop.inc'
      INCLUDE 'shop.inc'
      INCLUDE 'shop_rest.inc'
      INCLUDE 'mm_input.inc'
      INCLUDE 'xinr.inc'
      INCLUDE 'g_loc.inc'
      INCLUDE 'lscal.inc'
      INCLUDE 'comvel.inc'      !TSUBCOM
      INCLUDE 'time.inc'        !TDEBUG
      INCLUDE 'memc.inc'        !MEMDEB
      INCLUDE 'vdwc.inc'        !VDWC
      INCLUDE 'fileopen.inc'
      INCLUDE 'merge.inc'
      INCLUDE 'cdft.inc'
      INCLUDE 'prng.inc'        ! PRNG
      INCLUDE 'gle.inc'

C     Variables
#if defined(POINTER8)
      INTEGER*8 I_AD1,I_AD2     !in memc.inc (not used here)
#else
      INTEGER   I_AD1,I_AD2     !in memc.inc (not used here)
#endif
      INTEGER   MAXUNKNOWN,MAXINFO,LINELEN,PATHLEN
      PARAMETER (MAXUNKNOWN=30,MAXINFO=30,LINELEN=80,PATHLEN=255)
      CHARACTER LINEFOR*5
      PARAMETER (LINEFOR='(A80)')
      CHARACTER FILENAME*(PATHLEN),LINE*(LINELEN),LINEOLD*(LINELEN),
     &          UNKNOWN(MAXUNKNOWN)*(LINELEN),FFORMAT*30,
     &          INFOMSG(MAXINFO)*(LINELEN)
      LOGICAL   ERREAD,TSRHO,TOFF,TEST,MIRROR
      INTEGER   ICARG,IARGC,IUNIT,IERR,INSCAN,IUNK,IFLAG,
     &          I,II,IOUT,NOLD,I1,IT,IA,IE,INFOLINES
#ifdef _BGL
      INTEGER   LENECP
#endif
      REAL*8    VAR
      CALL LSCAL_INIT           ! Block data with variable size arrays
C     ==--------------------------------------------------------------==
C     ==  OPEN INPUT FILE (THE FILE NAME IS GIVEN ON THE COMMAND LINE)==
C     ==--------------------------------------------------------------==
C     QM/MM code
      QMMM=.FALSE.
      QMNODE=.TRUE.
      MMNODE=.FALSE.
      IF(.NOT.PARENT) GOTO 9999
      IUNIT = 5
      ICARG=IARGC()
#ifdef _BGL
      IF(ICARG.LT.1) THEN
        CALL MY_GETENV('CPMD_INPUT',FILENAME)
        LENECP=INDEX(FILENAME,' ')-1
        IF(LENECP.EQ.0) THEN
             FILENAME='./input'
             LENECP=7
        ENDIF
        FILENAME='./'//FILENAME(1:LENECP)
        LENECP=LENECP+2
      ELSE
        CALL MY_GETARG(1,FILENAME)
      ENDIF
#else
      IF(ICARG.LT.1) THEN
        WRITE(6,*) ' NO INPUT FILE NAME SPECIFIED '
        CALL STOPGM('CONTROL',' ')
      ELSE
        CALL MY_GETARG(1,FILENAME)
      ENDIF
#endif
C.....Test whether the extension of the input file is .run
C.....if this is the case wait for the file QMSTART before
C.....reading in the input file (needed for the interface mode).
      IF (INDEX(FILENAME,'.run').NE.0) THEN
    1   CONTINUE
        WRITE(6,*) ' CPMD is waiting for QMSTART.'
        INQUIRE(FILE='QMSTART',EXIST=TEST)
        IF(TEST) THEN
          WRITE(6,*) ' CPMD detected QMSTART.'
          OPEN(5,FILE='QMSTART',STATUS='UNKNOWN')
          CLOSE(5,STATUS='DELETE')
          GOTO 2
        ENDIF
         CALL MY_SLEEP(1)
         GOTO 1
    2   CONTINUE
      ENDIF
      OPEN(UNIT=IUNIT,FILE=FILENAME,STATUS='OLD',ERR=200)

C..copy the info section 
      INFOLINES=0
      IERR=INSCAN(IUNIT,'&INFO')
      IF(IERR.NE.0) GOTO 5
      
      DO I=1,MAXINFO
        READ(IUNIT,END=5,ERR=5,FMT='(A80)') LINE
        IF(INDEX(LINE,'&END').NE.0) GOTO 5
        INFOLINES=INFOLINES+1
        INFOMSG(INFOLINES) = LINE
      ENDDO
   5  CONTINUE

      IERR=INSCAN(IUNIT,'&CPMD')
      IF(IERR.NE.0) THEN
        WRITE(6,*) ' CONTROL| INPUT SECTION &CPMD NOT FOUND'
        CALL STOPGM('CONTROL',' ')
      ENDIF
      IUNK=0
C     ==--------------------------------------------------------------==
C     ==  DEFAULTS                                                    ==
C     ==--------------------------------------------------------------==
      MIRROR=.FALSE.
      TSRHO=.FALSE.
      CLASSICAL=.FALSE.
      TRAJSMALL=.FALSE.
      TRAJSMALLN=1
      CALL CONTROL_DEF
      WRITE(LINE,'(80(" "))')
C     ==--------------------------------------------------------------==

   10 CONTINUE
        LINEOLD=LINE
        READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
        IF(INDEX(LINE,'&END').NE.0) GOTO 20
C..CDFT
        IF(INDEX(LINE,'CDFT').NE.0) THEN
          CDFT=.TRUE.
          IF(INDEX(LINE,'SPIN').NE.0)TSPINC=.TRUE.
          IF(INDEX(LINE,'PCGFI').NE.0) TPCGFIC=.TRUE.
          IF(INDEX(LINE,'NEWT').NE.0)THEN
            CDFT_NEWT=.TRUE.
          ELSE
            IF(INDEX(LINE,'DEKK').NE.0)CDFT_DEKK=.TRUE.
          ENDIF
          IF(INDEX(LINE,'HDA').NE.0)THEN
            THDA=.TRUE.
            KRWFN=.TRUE.
            IF(INDEX(LINE,'PROJ').NE.0)THDAPROJ=.TRUE.
          ENDIF
          IF(INDEX(LINE,'AUTO').NE.0)TAUTO=.TRUE.
          IF(INDEX(LINE,'NOCCOR').NE.0)CCOR=.FALSE.
          IF(INDEX(LINE,'RESWF').NE.0)RESWF=.TRUE.
          IF(INDEX(LINE,'PHIO').NE.0)TPHIO=.TRUE.
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSR(LINE,IA,IE,CDFT_NC,ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSR(LINE,IA,IE,CDFT_V,ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,CDFT_END,ERREAD)
          IF(ERREAD) GOTO 99
          IF(THDA)THEN
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            IA=1
            CALL READSR(LINE,IA,IE,NOTHER,ERREAD)
          ENDIF
          GOTO 10
        ELSEIF(INDEX(LINE,'VGFACTOR').NE.0)THEN
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSR(LINE,IA,IE,VGSTEP,ERREAD)
          IF(ERREAD.OR.VGSTEP.LE.0.0D0) GOTO 99
          IA=IE
          CALL READSR(LINE,IA,IE,MAXVMOV,ERREAD)
          IF(ERREAD.OR.MAXVMOV.LE.0.0D0) MAXVMOV=0.1D0
          GOTO 10
        ELSEIF(INDEX(LINE,'COMBINE SYSTEMS').NE.0)THEN
          TSYSCOMB=.TRUE.
          IF(INDEX(LINE,'REFLECT').NE.0)TSCOMBM=.TRUE.
          IF(INDEX(LINE,'NONORTH').NE.0)TSCORTHO=.FALSE.
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSI(LINE,IA,IE,N_S0,ERREAD)
          IA=IE
          CALL READSI(LINE,IA,IE,N_S1,ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,N_S0UP,ERREAD)
          IA=IE
          CALL READSI(LINE,IA,IE,N_S1UP,ERREAD)
          ERREAD=.FALSE.
         GOTO 10
        ELSEIF(INDEX(LINE,'KSHAM').NE.0)THEN
          TKSHAM=.TRUE.
          IF(INDEX(LINE,'ROUT').NE.0) THEN
            RHOOUT=.TRUE.
            NUMPR=2
            CALL MEMORY(IP_MWFN,NUMPR,'MWFN')
          ENDIF
          GOTO 10
        ENDIF
C..Convergence Zones for CDFT
        IF(INDEX(LINE,'CZONES').NE.0) THEN
          TCZONES=.TRUE.
          IF(INDEX(LINE,'SET').NE.0) THEN
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            IA=1
            CALL READSR(LINE,IA,IE,CZONES(1,2),ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,CZONES(2,2),ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,CZONES(3,2),ERREAD)
            IF(ERREAD) GOTO 99
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            IA=1
            CALL READSR(LINE,IA,IE,CZONES(2,1),ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,CZONES(3,1),ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ENDIF
C..CDFT write out weight slice
        IF(INDEX(LINE,'WOUT').NE.0) THEN
          CDFT_WEIGHT=.TRUE.
          IF(INDEX(LINE,'FULL').NE.0) THEN
            CDFT_WF=.TRUE.
          ELSE
            CDFT_WF=.FALSE.
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            IA=1
            CALL READSR(LINE,IA,IE,WSLICE,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSI(LINE,IA,IE,WSTEP,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ENDIF

        IF(INDEX(LINE,'MOLE').NE.0 .AND. INDEX(LINE,'DYNA').NE.0) THEN
C..Molecular Dynamics
          MD=.TRUE.
          IF(INDEX(LINE,'BO').NE.0) TMDBO=.TRUE.
          IF(INDEX(LINE,'FILE').NE.0)THEN
               IF(INDEX(LINE,'XYZ').NE.0) XTIN=.TRUE.
               TMDFILE=.TRUE.
               IA=INDEX(LINE,'NSKIP=')
               IF (IA.NE.0) CALL READSI(LINE,IA+6,IE,NSKIP,ERREAD)
               IA=INDEX(LINE,'NSAMPLE=')
               IF (IA.NE.0) CALL READSI(LINE,IA+8,IE,NSAMPLE,ERREAD)
          ENDIF
          IF(TMDBO.AND.INDEX(LINE,'PT').NE.0) TRESPONSE=.TRUE.
          IF(INDEX(LINE,'CP').NE.0) TMDBO=.FALSE.
          IF(INDEX(LINE,'CLASSICAL').NE.0) classical=.TRUE.
CEHR[
          IF(INDEX(LINE,'EH').NE.0) THEN
            TMDBO=.FALSE.
            TMDEH=.TRUE.
            CMPLX_WF=.TRUE.
          ENDIF
CEHR]
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'PARRINELLO').NE.0 
     *     .AND. INDEX(LINE,'RAHMAN').NE.0) THEN
          TPRCP=.TRUE.
          IF(INDEX(LINE,'NPT').NE.0) TPENN=.TRUE.
          IF(INDEX(LINE,'SHOCK').NE.0) THEN
           TPENN=.TRUE.
           TSHOCK=.TRUE.
          ENDIF
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'OPT').NE.0 .AND. INDEX(LINE,'GEOM').NE.0) THEN
C..Optimisation of Ionic Coordinates
          GEOPT=.TRUE.
          IPRINT(IPRINT_FORCE) = 1
          IF(INDEX(LINE,'CLASSICAL').NE.0) classical=.TRUE.
          IF(INDEX(LINE,'XYZ').NE.0) THEN
             XGOUT=.TRUE.
             IF(INDEX(LINE,'SAMPLE').NE.0) THEN
               LINEOLD=LINE
               READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
               CALL READSI(LINE,1,IE,NGXYZ,ERREAD)
               IF(ERREAD) GOTO 99
             ENDIF
          ENDIF
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'OPT').NE.0 .AND. INDEX(LINE,'COMB').NE.0) THEN
C..Optimisation of Ionic Coordinates (combined scheme)
          GEOPT=.TRUE.
          SIMUL=.TRUE.
          THARM=.TRUE.
          TMASS=.TRUE.
          IF(INDEX(LINE,'XYZ').NE.0) THEN
             XGOUT=.TRUE.
             IF(INDEX(LINE,'SAMPLE').NE.0) THEN
               LINEOLD=LINE
               READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
               CALL READSI(LINE,1,IE,NGXYZ,ERREAD)
               IF(ERREAD) GOTO 99
             ENDIF
          ENDIF
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'OPT').NE.0 .AND. INDEX(LINE,'WAVE').NE.0) THEN
C..Optimisation of Wavefunction
          WFOPT=.TRUE.
          GOTO 10
        ELSE IF
CEHR[
     *     (INDEX(LINE,'CHEBY').NE.0) THEN
          cheby=.true.
          cayley=.false.
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'CAYLEY').NE.0) THEN
          cayley=.true.
          cheby=.false.
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'RUNGE-KUTTA').NE.0) THEN
          ruku=.true.
          GOTO 10
        ELSE IF
CEHR]
     *     (INDEX(LINE,'FORCEMATCH').NE.0) THEN
C..Forcematching
          FMATCH=.TRUE.
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'DEBUG').NE.0) THEN
C..Debug force calculation
          IF(INDEX(LINE,'FORCES').NE.0) THEN
            TDEBFOR=.TRUE.
          ENDIF
C..Debug program using tiset and tihalt
          IF(INDEX(LINE,'CODE').NE.0) THEN
            TDEBUG=.TRUE.
          ENDIF
C..Debug memory using memory and freem
          IF(INDEX(LINE,'MEMORY').NE.0) THEN
            MEMDEB=.TRUE.
          ENDIF
C..Debug FILEOPEN processing
          IF(INDEX(LINE,'FILE').NE.0) THEN
            FO_TDEBUG=.TRUE.
          ENDIF
C..Debug RESTART file read or write
          IF(INDEX(LINE,'IO').NE.0) THEN
            TDEBIO=.TRUE.
          ENDIF
C..Do not read/write accumulator information from/to the RESTART file
C..This avoids putting timing information to the RESTART and makes 
C..RESTART files identical for identical runs.
          IF(INDEX(LINE,'NOACC').NE.0) THEN
            TDEBACC=.TRUE.
          ENDIF
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'KOHN').NE.0 .AND. INDEX(LINE,'ENER').NE.0) THEN
C..Calculate Kohn-Sham energies for the final potential
          IF(INDEX(LINE,'OFF').NE.0) THEN 
            KSENER=.FALSE.
          ELSE
            KSENER=.TRUE.
          ENDIF
          IF(KSENER) THEN
            IF(INDEX(LINE,'NOWAVE').NE.0) TKNOSWAP=.TRUE.
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,NKSSTA,ERREAD)
            IF(ERREAD) GOTO 99
            IF(NKSSTA.LT.0) THEN
              WRITE(6,'(A,I6)') ' CONTROL| NKSSTA=',NKSSTA
              CALL STOPGM('CONTROL','GREAT OR EQUAL TO ZERO')
            ENDIF
          ENDIF
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'SURF').NE.0 .AND. INDEX(LINE,'HOPP').NE.0) THEN
C..Surface Hopping molecular dynamics                 
          TSHOP=.TRUE.
          TSHOPOLD=.TRUE.
          TSHOPRES=.FALSE.
          EADDSH=0.0D0
          IF ( INDEX(LINE,'NEW').NE.0 )     TSHOPOLD=.FALSE.
          IF ( INDEX(LINE,'RESTART').NE.0 ) TSHOPRES=.TRUE.
          II=INDEX(LINE,'SHIFT=')
          IF ( II.NE.0 ) THEN
              CALL READSR(LINE,II+6,IOUT,EADDSH,ERREAD)
              IF(ERREAD) GOTO 99
          ENDIF
          IF ( INDEX(LINE,'NEW').NE.0 ) TSHOPOLD=.FALSE.
          NSURF=2
          GOTO 10
        ELSE IF (INDEX(LINE,'ROKS').NE.0) THEN
C  RESTRICTED OPEN SHELL KOHN SHAM
           TROS=.TRUE.
           NLSD=4
           NLSX=3
C  SINGLET/TRIPLET: PARAMETERS FOR LOW SPIN EXCITATION
           IF (INDEX(LINE,'TRIPLET').NE.0) THEN
             LSEA=0.D0
             LSEB=1.D0
           ENDIF
C  SINGLET IS DEFAULT
           IF ((INDEX(LINE,'GOEDECKER').NE.0).AND.
     *         (INDEX(LINE,'LOCA').EQ.0)) THEN
C  (UNMODIFIED) GOEDECKER
                MGAB(9) = 0.5D0
                MGAB(10) = 0.5D0
            ELSE IF (INDEX(LINE,'EXPERT').NE.0) THEN
                READ (IUNIT, ERR=99, END=99, FMT=*) MGAB(1), MGAB(2),
     *          MGAB(3), MGAB(4), MGAB(5), MGAB(6), MGAB(7),
     *          MGAB(8), MGAB(9), MGAB(10), MGAB(11), MGAB(12)
C  MGAB(1) = A_AC, MGAB(2) = B_AC, MGAB(3) = A_BC,
C  MGAB(4) = B_BC, MGAB(5) = A_CA, MGAB(6) = B_CA,
C  MGAB(7) = A_CB, MGAB(8) = B_CB, MGAB(9) = A_BA,
C  MGAB(10)= B_BA, MGAB(11)= A_AB, MGAB(12)= B_AB
           ELSE IF ((INDEX(LINE,'LOCALI').NE.0).AND.
     *              (INDEX(LINE,'DE').EQ.0)) THEN
C  ANTISYMMETRIZED GOEDECKER (LOCALIZED VARIANT)
                MGAB(9) = 0.5D0
                MGAB(10) = -0.5D0
                MGAB(12) = -0.5D0
C  MODIFIED GOEDECKER (DELOCALIZED VARIANT) IS DEFAULT
           ENDIF
           GOTO 10
        ELSE IF
     *     (INDEX(LINE,'PATH').NE.0 .AND. INDEX(LINE,'SAMPL').NE.0) THEN
C..MC sampling of reaction path                       
          TSAMPL=.TRUE.
          GOTO 10
        ELSE IF
     *         (INDEX(LINE,'FREE').NE.0 .AND.
     &         INDEX(LINE,'ENER').NE.0 .AND.
     &         INDEX(LINE,'FUNC').NE.0) THEN
C..Free Energy functional (by default trotter and bogoliubov).
          TFINT=.TRUE.
          TDIAG=.TRUE.
          TLANC=.TRUE.
          TTROT=.TRUE.
          TBOGO=.TRUE.
          BETAP=0.001D0
          GOTO 10
C..Wfn Opt. via DIIS at fixed RHO 
C       (recommended for metals with virtual states)
        ELSE IF (INDEX(LINE,'FIXRHO') .NE.0 .AND.
     &         INDEX(LINE,'UPWFN') .NE.0) THEN
          IF(INDEX(LINE,'PCG').NE.0) THEN
            PCG=.TRUE.
            DIIS=.FALSE.
          ELSE
            PCG=.FALSE.
            DIIS =.TRUE.
          ENDIF
          TDIAG=.TRUE.
          TLANC=.FALSE.
          TDAVI=.FALSE.
          DIIS =.TRUE.
          PREC=.TRUE.
          TFRHO_UPW=.TRUE. 
          TFINT=.TRUE.
          TTROT=.FALSE.
          TBOGO=.FALSE.
          BETAP=0.001D0
          GOTO 10
C..Number of vectors for DIIS storage
        ELSE IF (INDEX(LINE,'FIXRHO') .NE.0 .AND.
     &         INDEX(LINE,'VECT')  .NE.0) THEN
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE,1,IE,MDIIS_FR,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
C..Min and Max Number of DIIS Cycles at FIXRHO
        ELSE IF (INDEX(LINE,'FIXRHO') .NE.0 .AND.
     &         INDEX(LINE,'LOOP') .NE.0) THEN
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE,1,IE,MINLDIIS,ERREAD)
          IF(ERREAD) GOTO 99
          IA = IE+1
          CALL READSI(LINE,IA,IE,MAXLDIIS,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
C..Tolerance parameter for DIIS convergence of wfn at FIXRHO
        ELSE IF (INDEX(LINE,'FIXRHO') .NE.0 .AND.
     &         INDEX(LINE,'WFTOL') .NE.0) THEN
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSR(LINE,1,IE,TOLRHOFIX,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
C..Bogoliubov correction
        ELSE IF(INDEX(LINE,'BOGO').NE.0.AND.INDEX(LINE,'CORR').NE.0)THEN
          IF(INDEX(LINE,'OFF').NE.0) THEN
            TBOGO=.FALSE.
          ELSE
            TBOGO=.TRUE.
          ENDIF
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'VIBRATIONAL').NE.0 
     *        .AND. INDEX(LINE,'ANALYSIS').NE.0) THEN
C..Vibrational Analysis          
          VIBRAT=.TRUE.
          TSDIN=.FALSE.
          MODE=0

          IF(INDEX(LINE,'FD').NE.0) TSDAN=.FALSE.
          IF(INDEX(LINE,'LR').NE.0) TSDAN=.TRUE.
          IF(INDEX(LINE,'IN').NE.0) TSDIN=.TRUE.
          IF(INDEX(LINE,'ACLIMAX').NE.0) ACOUT=.TRUE.
          IF(INDEX(LINE,'GAUSS').NE.0) VOUT=.TRUE.
          IF(INDEX(LINE,'SAMPLE').NE.0) THEN
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,NVIB,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          IF(INDEX(LINE,'MODE').NE.0) THEN
            II=INDEX(LINE,'=')
            IF(II.NE.0) THEN
              CALL READSI(LINE,II+1,IOUT,MODE,ERREAD)
              IF(ERREAD) GOTO 99
            ENDIF
          ENDIF
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'ELECTRO').NE.0
     *        .AND. INDEX(LINE,'SPECTRA').NE.0) THEN
C..Electronic spectra using TDDFT
          TSPEC=.TRUE.
          GOTO 10
CEHR[
        ELSE IF
     *    (INDEX(LINE,'PROPAGATION').NE.0
     *        .AND. INDEX(LINE,'SPECTRA').NE.0) THEN
C..Electronic spectra using real time propoagation DFT
          CMPLX_WF=.TRUE.
          TPSPEC=.TRUE.
          TSDE=.TRUE.
          TCENT=.FALSE.
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'PROPAGATION').NE.0
     *        .AND. INDEX(LINE,'DISTURB').NE.0) THEN
C..Electronic spectra using real time propoagation DFT
          CMPLX_WF=.TRUE.
          TPDIST=.TRUE.
          TSDE=.TRUE.
          GOTO 10
       ELSE IF
     *    (INDEX(LINE,'GAUGEPULSE').NE.0) THEN
C..Constant external electronic potential added as gauge field
          TGAUGEP=.TRUE.
          GOTO 10
       ELSE IF
     *    (INDEX(LINE,'GAUGEFIELD').NE.0) THEN
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSR(LINE ,1,IE,GFREQ,ERREAD)
C..Constant external electronic potential added as gauge field
          TGAUGEF=.TRUE.
          TGAUGEP=.TRUE.
          GOTO 10
CEHR]
        ELSE IF
     *    (INDEX(LINE,'NACV').NE.0) THEN
C..Non adiabatic coupling vectors using TDDFT
          TNACVS=.TRUE.
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'ORBITAL').NE.0
     *        .AND. INDEX(LINE,'HARDNESS').NE.0) THEN
C..Orbital Hardness Matrix
          THARD=.TRUE.
          IF(INDEX(LINE,'FD').NE.0) TSDAN=.FALSE.
          IF(INDEX(LINE,'LR').NE.0) TSDAN=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'PROPERT').NE.0) THEN
C..Properties                    
          PROPER=.TRUE.
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'PATH').NE.0 
     *        .AND. INDEX(LINE,'INTEGRAL').NE.0) THEN
C..Path Integrals                
          TPATH=.TRUE.
          TPIMD=.TRUE.
          GOTO 10
        ELSE IF
     &         (INDEX(LINE,'PATH').NE.0 
     &         .AND. INDEX(LINE,'MINIM').NE.0) THEN
C..Path Minimisation (Saddle point determination)
          TPATH=.TRUE.
          TPMIN=.TRUE.
          GOTO 10
        ELSE IF
     &         (INDEX(LINE,'PRNGSEED').NE.0) THEN               ! PRNG
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSI(LINE,IA,IE,IPRNG,ERREAD)
          GOTO 10
        ELSE IF
     &         (INDEX(LINE,'LANGEVIN').NE.0) THEN               ! PRNG
          IF (INDEX(LINE,'MOVECM').NE.0) GLE_COM=0
          IF (INDEX(LINE,'WHITE').NE.0) THEN
            GLE_MODE=GLE_WHITE
            GLE_NS=0
          ELSE IF (INDEX(LINE,'CPMD').NE.0) THEN
            GLE_MODE=GLE_CPMD
            GLE_NS=GLE_OPT_NS
          ELSE IF (INDEX(LINE,'OPTIMAL').NE.0) THEN
            GLE_MODE=GLE_OPT
            GLE_NS=GLE_CP_NS
          ELSE IF (INDEX(LINE,'CUSTOM').NE.0) THEN
            GLE_MODE=GLE_CUST
          END IF
	  LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IF (GLE_MODE.NE.GLE_CUST) THEN
            IA=1
            CALL READSR(LINE ,IA,IE,GLE_OMEGA,ERREAD)
            IF(ERREAD) GOTO 99
          ELSE
            IA=1
            CALL READSI(LINE ,IA,IE,GLE_NS,ERREAD)
            IF(ERREAD .OR. GLE_NS.LT.0) CALL STOPGM('CONTROL',
     *         'MUST SPECIFY size_ns FOR LANGEVIN CUSTOM')
          END IF
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'QMMM').NE.0) THEN
C..QMMM
          IF(INDEX(LINE,'QMMMEASY').NE.0) THEN
            TQMMECH=.TRUE.
          ELSE
            TQMMM=.TRUE.
          ENDIF
          GOTO 10
        ELSE IF (INDEX(LINE,'INTERFACE').NE.0) THEN
C..Interface to classical MD program
          TINTER=.TRUE.
C     original EGO interface
          IF(INDEX(LINE,'EGO').NE.0) THEN
            IFTYPE=1
C..biswas/2005. modified EGO interface for Gromacs
          ELSE IF(INDEX(LINE,'GMX').NE.0) THEN
            IFTYPE=2
          ELSE
            CALL STOPGM('CONTROL','UNKNOWN MM-INTERFACE TYPE')
          ENDIF
          IF(INDEX(LINE,'MULLI').NE.0) ICMET=1
          IF(INDEX(LINE,'HIRSH').NE.0) ICMET=2
          IF(INDEX(LINE,'ESP').NE.0) ICMET=3
          IF(INDEX(LINE,'LOWDI').NE.0) ICMET=4
          IF(INDEX(LINE,'PCGFI').NE.0) TPCGFI=.TRUE.
          GOTO 10
        ELSE IF 
     *    (INDEX(LINE,'TROTTER').NE.0.AND.INDEX(LINE,'FACT').NE.0) THEN
C..Trotter factor for free energy functional
          IF(INDEX(LINE,'OFF').NE.0) THEN
            TTROT=.FALSE.
            TBOGO=.FALSE.
            GOTO 10
          ENDIF
          TTROT=.TRUE.
          II=INDEX(LINE,'=')
          IF(II.NE.0) THEN
            CALL READSI(LINE,II+1,IOUT,NTABBETAP,ERREAD)
            IF(ERREAD) GOTO 99
            IF(NTABBETAP.LE.0.OR.NTABBETAP.GT.MAXBETAP) THEN
              WRITE(6,'(A,I3)')
     &             ' CONTROL| NUMBER OF TROTTER FACTORS : ',NTABBETAP
              WRITE(6,'(A,I3)')
     &             ' CONTROL| BETWEEN 0 AND',MAXBETAP
              CALL STOPGM('CONTROL','WRONG TROTTER FACTOR NUMBER')
            ENDIF
            LINEOLD=LINE
            DO I=1,NTABBETAP
              READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
              IA=1
              CALL READSR(LINE,IA,IE,DENSBETAP(I),ERREAD)
              IF(ERREAD) GOTO 99
              IA=IE
              CALL READSR(LINE,IA,IE,TABBETAP(I), ERREAD)
              IF(ERREAD) GOTO 99
            ENDDO
            BETAP=TABBETAP(1)
          ELSE
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSR(LINE,1,IE,BETAP,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSEIF(INDEX(LINE,'LINEAR').NE.0.AND.
     &         INDEX(LINE,'RESPONSE').NE.0) THEN
C..   LINear RESPone calculation          
          TRESPONSE  = .TRUE.
          WFOPT=.TRUE.
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'HARM').NE.0 .AND.
     *     INDEX(LINE,'REF').NE.0 .AND. INDEX(LINE,'SYST').NE.0) THEN
C..Switch on/off harmonic reference system
          IF(INDEX(LINE,'OFF').NE.0) THEN
            THARM=.FALSE.
          ELSE
            THARM=.TRUE. 
          ENDIF
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'SCAL').NE.0.AND.INDEX(LINE,'MASS').NE.0) THEN
C..Switch on/off scaling of electron masses
          IF(INDEX(LINE,'OFF').NE.0) THEN
            TMASS=.FALSE.
          ELSE
            TMASS=.TRUE. 
          ENDIF
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'TDDFT').NE.0) THEN
          TDDFT=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'LSD').NE.0.OR.
     &         (INDEX(LINE,'LOCAL').NE.0.AND.
     &          INDEX(LINE,'SPIN').NE.0.AND.
     &          INDEX(LINE,'DENSIT').NE.0) ) THEN
C..Local Spin Density Approximation
          TLSD=.TRUE.
          NLSD=2
          NLSX=3
          GOTO 10
        ELSE IF (INDEX(LINE,'SSIC').NE.0) THEN  ! cmb_ssic
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSR(LINE,IA,IE,asic,ERREAD)
          IF(ERREAD) GOTO 99
cmb          IA=IE ! xc not yet implemented
cmb          CALL READSR(LINE,IA,IE,bsic,ERREAD)
cmb          IF(ERREAD) GOTO 99
          TLSD=.TRUE.
          NLSD=2
          NLSX=3
          TSIC=.TRUE.
          GOTO 10  ! cmb_ssic
        ELSE IF
     *    (INDEX(LINE,'NONOR').NE.0 .AND. INDEX(LINE,'ORB').NE.0) THEN
C..Use nonorthogonal orbital
          IF(INDEX(LINE,'OFF').NE.0) THEN
            NONORT=.FALSE.
            CNSTMD=.TRUE. 
          ELSE
            NONORT=.TRUE.
            CNSTMD=.FALSE.
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSR(LINE,1,IE,SLIMIT,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF
     *   (INDEX(LINE,'LANCZOS').NE.0.AND.INDEX(LINE,'DIAG').NE.0) THEN
C..Lanczos diagonalization (Friesner type)
          TDIAG=.TRUE.
          TLANC=.TRUE.
          IF(INDEX(LINE,'ALL').NE.0) TFRAL=.TRUE.
          IF(INDEX(LINE,'NEW').NE.0) TFRSBLK=.TRUE.
          IF(INDEX(LINE,'OPT').NE.0) THEN
            TDIAG=.FALSE.
            TDIAGOPT=.TRUE.
          ELSE IF(INDEX(LINE,'PERI').NE.0) THEN
            TDIAG=.FALSE.
            TDIAGOPT=.TRUE.
            IF(B2LIMIT.LE.1.0D-16) B2LIMIT=1.0D-8
            II=INDEX(LINE,'=')
            IF(II.NE.0) THEN
              CALL READSI(LINE,II+1,IOUT,NPERID,ERREAD)
              IF(ERREAD) GOTO 99
            ELSE
              WRITE(6,'(A)')
     *              ' PERIODIC DIAGONALIZATION: NO FREQUENCY SPECIFIED'
              CALL STOPGM('CONTROL','INSUFFICIENT INPUT')
            ENDIF
          ELSE IF(INDEX(LINE,'RESET').NE.0) THEN
            TDIAG=.FALSE.
            TDIAGOPT=.TRUE.
            IF(B2LIMIT.LE.1.0D-16) B2LIMIT=1.0D-8
            II=INDEX(LINE,'=')
            IF(II.NE.0) THEN
              CALL READSI(LINE,II+1,IOUT,NRPERI,ERREAD)
              IF(ERREAD) GOTO 99
            ELSE
              WRITE(6,'(A)') ' PERIODIC DIAGONALIZATION: ',
     *              'NO NUMBER OF RESETS SPECIFIED'
              CALL STOPGM('CONTROL','INSUFFICIENT INPUT')
            ENDIF
          ENDIF
          GOTO 10
        ELSE IF 
     *   (INDEX(LINE,'LANCZOS').NE.0.AND.INDEX(LINE,'PARA').NE.0) THEN
C..Parameter for Lanczos diagonalization (Friesner type)
          IF(INDEX(LINE,'ALL').NE.0) TFRAL=.TRUE.
          II=INDEX(LINE,'=')
          IF(II.NE.0) THEN
            CALL READSI(LINE,II+1,IOUT,NTABTROT,ERREAD)
            IF(ERREAD) GOTO 99
            IF(NTABTROT.LE.0.OR.NTABTROT.GT.MAXTROT) THEN
              WRITE(6,'(A,I3)')
     &             ' CONTROL| NUMBER OF BETATOL : ',NTABTROT
              WRITE(6,'(A,I3)')
     &             ' CONTROL| BETWEEN 1 AND',MAXTROT
              CALL STOPGM('CONTROL','WRONG BETATOL NUMBER')
            ENDIF
          ELSE
            NTABTROT=1
          ENDIF
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSI(LINE,IA,IE,N_FRIES,   ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,NKRY_MAX,  ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,NKRY_BLOCK,ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSR(LINE,IA,IE,B2LIMIT,   ERREAD)
          IF(ERREAD) GOTO 99
          IF(II.NE.0) THEN
            B2TROT(1)=B2LIMIT
            DO I=2,NTABTROT
              READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
              IA=1
              CALL READSR(LINE,IA,IE,DENSTROT(I),ERREAD)
              IF(ERREAD) GOTO 99
              IA=IE
              CALL READSR(LINE,IA,IE,B2TROT(I),  ERREAD)
              IF(ERREAD) GOTO 99
            ENDDO
            IF(NTABTROT.EQ.1) THEN
              DENSTROT(1)=1.D0
            ELSE
              DENSTROT(1)=2.D0*DENSTROT(2)
            ENDIF
          ENDIF
          GOTO 10
        ELSE IF 
     *   (INDEX(LINE,'DAVIDSON').NE.0.AND.INDEX(LINE,'DIAG').NE.0) THEN
C..Davidson diagonalization 
          TDIAG=.TRUE.
          TDAVI=.TRUE.
          GOTO 10
        ELSE IF 
     *   (INDEX(LINE,'DAVIDSON').NE.0.AND.INDEX(LINE,'PARA').NE.0) THEN
C..Parameter for Davidson diagonalization
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSI(LINE,IA,IE,NDAVV, ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSR(LINE,IA,IE,EPSDAV, ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,N_FRIES,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'ALEX').NE.0 .AND. INDEX(LINE,'MIXI').NE.0) THEN
C..Alexander mixing parameter
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSR(LINE,1,IE,ALXMIX, ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'ANDE').NE.0 .AND. INDEX(LINE,'MIXI').NE.0) THEN
C..Anderson mixing parameter
          IF(INDEX(LINE,'G-SPACE').NE.0) TGMIX=.TRUE. 
          II=INDEX(LINE,'=')
          IF(II.NE.0) THEN
            CALL READSI(LINE,II+1,IOUT,NTABMIX,ERREAD)
            IF(ERREAD) GOTO 99
            IF(NTABMIX.LE.0.OR.NTABMIX.GT.MAXMIX) THEN
              WRITE(6,'(A,I3)')
     &             ' CONTROL| NUMBER OF MIXING : ',NTABMIX
              WRITE(6,'(A,I3)')
     &             ' CONTROL| BETWEEN 0 AND',MAXMIX
              CALL STOPGM('CONTROL','WRONG MIXING NUMBER')
            ENDIF
            LINEOLD=LINE
            DO I=1,NTABMIX
              READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
              IA=1
              CALL READSR(LINE,IA,IE,DENSMIX(I),ERREAD)
              IF(ERREAD) GOTO 99
              IA=IE
              CALL READSR(LINE,IA,IE,ANDMIX(I), ERREAD)
              IF(ERREAD) GOTO 99
            ENDDO
            ANDRMIX=DENSMIX(1)
          ELSE
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSR(LINE,1,IE,ANDRMIX,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'BROY').NE.0 .AND. INDEX(LINE,'MIXI').NE.0) THEN
C..Broyden mixing parameter
          TGMIX=.TRUE.
          TGBROY=.TRUE.
          IF(INDEX(LINE,'ADAPT').NE.0) TADBROY=.TRUE.
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IF(INDEX(LINE,'=').EQ.0) THEN
C           Use default
            IF(INDEX(LINE,'DEFAULT').NE.0) GOTO 10
            READ(LINE ,ERR=99,END=99,FMT=*) 
     &           BROYMIX,ECUTBROY,W02BROY,NFRBROY,IBRESET,KERMIX
            IA=1
            CALL READSR(LINE,IA,IE,BROYMIX, ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,ECUTBROY,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,W02BROY, ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSI(LINE,IA,IE,NFRBROY, ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSI(LINE,IA,IE,IBRESET, ERREAD)
            IF(ERREAD) GOTO 99
            IF(IBRESET.EQ.0) IBRESET=50
            IA=IE
            CALL READSR(LINE,IA,IE,KERMIX,  ERREAD)
            IF(ERREAD) GOTO 99
          ELSE
C           Expert use.
            II=INDEX(LINE,'MIX=')
            IF(II.NE.0) THEN
              CALL READSR(LINE,II+4,IOUT,BROYMIX,ERREAD)
              IF(ERREAD) GOTO 99
            ENDIF
            II=INDEX(LINE,'CUTBROY=')
            IF(II.NE.0) THEN
              CALL READSR(LINE,II+8,IOUT,ECUTBROY,ERREAD)
              IF(ERREAD) GOTO 99
            ENDIF
            II=INDEX(LINE,'W02BROY=')
            IF(II.NE.0) THEN
              CALL READSR(LINE,II+8,IOUT,W02BROY,ERREAD)
              IF(ERREAD) GOTO 99
            ENDIF
            II=INDEX(LINE,'W02=')
            IF(II.NE.0) THEN
              CALL READSR(LINE,II+4,IOUT,W02BROY,ERREAD)
              IF(ERREAD) GOTO 99
            ENDIF
            II=INDEX(LINE,'NFRBROY=')
            IF(II.NE.0) THEN
              CALL READSI(LINE,II+8,IOUT,NFRBROY,ERREAD)
              IF(ERREAD) GOTO 99
            ENDIF
            II=INDEX(LINE,'RESET=')
            IF(II.NE.0) THEN
              CALL READSI(LINE,II+6,IOUT,IBRESET,ERREAD)
              IF(ERREAD) GOTO 99
            ENDIF
            II=INDEX(LINE,'KERMIX=')
            IF(II.NE.0) THEN
              CALL READSR(LINE,II+7,IOUT,KERMIX,ERREAD)
              IF(ERREAD) GOTO 99
            ENDIF
          ENDIF
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'DIIS').NE.0 .AND. INDEX(LINE,'MIXI').NE.0) THEN
C..DIIS mixing for density  
          II=INDEX(LINE,'=')
          IF(II.NE.0) THEN
            CALL READSI(LINE,II+1,IOUT,NTABNRDIIS,ERREAD)
            IF(ERREAD) GOTO 99
            IF(NTABNRDIIS.LE.0.OR.NTABNRDIIS.GT.MAXMIX) THEN
              WRITE(6,'(A,I3)')
     &             ' CONTROL| NUMBER OF DIIS MIXING : ',NTABNRDIIS
              WRITE(6,'(A,I3)')
     &             ' CONTROL| BETWEEN 0 AND',MAXMIX
              CALL STOPGM('CONTROL','WRONG DIIS MIXING NUMBER')
            ENDIF
            NOLD=0
            LINEOLD=LINE
            DO I=1,NTABNRDIIS
              READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
              IA=1
              CALL READSR(LINE,IA,IE,DENSNRDIIS(I),ERREAD)
              IF(ERREAD) GOTO 99
              IA=IE
              CALL READSI(LINE,IA,IE,TABNRDIIS(I), ERREAD)
              IF(ERREAD) GOTO 99
              IF(TABNRDIIS(I).LT.NOLD) CALL STOPGM('CONTROL',
     &             'THE DIIS MIXING NUMBERS HAVE TO INCREASE')
              NOLD=TABNRDIIS(I)
            ENDDO
            NRDIISMAX=TABNRDIIS(NTABNRDIIS)
          ELSE
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE ,1,IE,NRDIIS,ERREAD)
            IF(ERREAD) GOTO 99
            NRDIISMAX=NRDIIS
            TABNRDIIS(1)=NRDIIS
          ENDIF
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'MOVERHO').NE.0) THEN
C..Moverho 
          TMOVR=.TRUE.
          ALXMIX=0.D0
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSR(LINE,1,IE,DMOVRMIX,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
C..Wavefunction extrapolation (polynomial or aspc) for BOMD.
        ELSE IF
     *    (INDEX(LINE,'EXTRAPOL').NE.0 .AND.
     *     INDEX(LINE,'WFN').NE.0) THEN
          TEXTRAP=.TRUE.
          IF(INDEX(LINE,'STOR').NE.0) TSTRXTP=.TRUE.
          IF(INDEX(LINE,'POLY').NE.0) TASPC=.FALSE.
          IF(INDEX(LINE,'ASPC').NE.0) TASPC=.TRUE.
          II=INDEX(LINE,'CSTEPS=')
          IF(II.NE.0) THEN
            CALL READSI(LINE,II+7,IOUT,NASPC,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE ,1,IE,MEXTRA,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'EXTRAPOL').NE.0 .AND.
     *     INDEX(LINE,'CONSTR').NE.0) THEN
          TPRED=.TRUE.
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE ,1,IE,PREDORD,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'TSDE').NE.0) THEN
C..Electronic dynamics by Steepest descent
          TSDE=.TRUE.
          PREC=.TRUE.
          IF(INDEX(LINE,'NOPREC').NE.0) PREC=.FALSE.
          GOTO 10
        ELSE IF(INDEX(LINE,'TSDP').NE.0) THEN
C..Ionic dynamics by Steepest descent
          TSDP=.TRUE.
C         Steepest descent with line minimisation
          IF(INDEX(LINE,'LINE').NE.0) TSDPLINE=.TRUE.
          GOTO 10
C..Ionic minimization by Conjugate Gradient
        ELSE IF(INDEX(LINE,'TCGP').NE.0) THEN
          TCGP=.TRUE.
          TSDP=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'TSDC').NE.0) THEN
C..Cell dynamics by Steepest descent
          TSDC=.TRUE.
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'STEE').NE.0 .AND. INDEX(LINE,'DESC').NE.0) THEN
             IF(INDEX(LINE,'ELEC').NE.0) THEN
                TSDE=.TRUE.
                PREC=.TRUE.
                IF(INDEX(LINE,'NOPREC').NE.0) PREC=.FALSE.
             ENDIF
             IF(INDEX(LINE,'IONS').NE.0) THEN
               TSDP=.TRUE.
               IF(INDEX(LINE,'LINE').NE.0) TSDPLINE=.TRUE.
             ENDIF
             IF(INDEX(LINE,'CELL').NE.0) TSDC=.TRUE.
          GOTO 10
C..Conjugate Gradient
          ELSE IF
     *     (INDEX(LINE,'CONJ').NE.0 .AND. INDEX(LINE,'GRAD').NE.0) THEN
            IF(INDEX(LINE,'ELEC').NE.0) THEN
              PCG=.TRUE.
              PREC=.TRUE.
              IF(INDEX(LINE,'NOPREC').NE.0) PREC=.FALSE.
              IF(INDEX(LINE,'MINIMIZE').NE.0) PCGMIN=.TRUE.
            ENDIF
            IF(INDEX(LINE,'IONS').NE.0) THEN
              TCGP=.TRUE.
              TSDP=.TRUE.
            ENDIF
            GOTO 10
        ELSE IF(INDEX(LINE,'ODIIS').NE.0) THEN
C..Wavefunction optimization by DIIS
          DIIS=.TRUE.
          PREC=.TRUE.
          IF(INDEX(LINE,'NOPREC').NE.0) PREC=.FALSE.
          IF (INDEX(LINE,'NO_RES').NE.0) THEN
            I1=INDEX(LINE,'=')
            CALL READSI(LINE,I1+1,IE,NRESET,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE,1,IE,MDIIS,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'PCG').NE.0) THEN
C..Wavefunction optimization by Precond. Conjugate Gradients
          PCG=.TRUE.
          PREC=.TRUE.
          IF(INDEX(LINE,'NOPREC').NE.0) PREC=.FALSE.
          IF(INDEX(LINE,'MINIMIZE').NE.0) PCGMIN=.TRUE.
          GOTO 10
        ELSE IF((INDEX(LINE,'HTHRS').NE.0) .OR.
     *    (INDEX(LINE,'HAMILTON').NE.0.AND.INDEX(LINE,'CUT').NE.0))THEN
C..Threshold for Hessian cutoff in preconditioning matrix for DIIS
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSR(LINE ,1,IE,HTHRS,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'GDIIS').NE.0) THEN
C..Geometry optimization by GDIIS/BFGS
          GDIIS=.TRUE.
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE ,1,IE,MGDIIS,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'LBFGS').NE.0) THEN
C..Geometry optimization by low-memory BFGS
          LBFGS=.TRUE.
          IF (INDEX(LINE,'NREM').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSI(LINE,1,IE,NREM,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'NTRUST').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSI(LINE,1,IE,NTRUST,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'NRESTT').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSI(LINE,1,IE,NRESTT,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'NTRSTR').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSI(LINE,1,IE,NTRSTR,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'TRUSTR').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSR(LINE,1,IE,TRUSTR,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE
             GOTO 10
          END IF
        ELSE IF(INDEX(LINE,'PRFO').NE.0) THEN
C..Transition state search by partitioned RFO
          PRFO=.TRUE.
          IF (INDEX(LINE,'NVAR').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSI(LINE,1,IE,NVAR,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'MODE').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSI(LINE,1,IE,MODE,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'LOCK').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSI(LINE,1,IE,MODELK,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'TOLENV').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSR(LINE,1,IE,TOLENV,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'TRUSTP').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSR(LINE,1,IE,TRUSTP,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'OMIN').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSR(LINE,1,IE,OMIN,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'NSVIB').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSI(LINE,1,IE,NSVIB,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'CORE').NE.0) THEN
             I1=INDEX(LINE,'=')
             CALL READSI(LINE,I1+1,IE,NCORE,ERREAD)
             IF(ERREAD) GOTO 99
             LNOMAP = .FALSE.
             CALL MEMORY(IP_ICORE,NCORE,'ICORE')
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             IA=1
             DO I = 1,NCORE
                CALL READSI(LINE,IA,IE,ICORE(I),ERREAD)
                IF(ERREAD) GOTO 99
                IA=IE
             ENDDO
             IF (NVAR.EQ.0) NVAR = 3*NCORE
             GOTO 10
          ELSE IF (INDEX(LINE,'NSMAXP').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSI(LINE,1,IE,NSMAXP,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'PRJHES').NE.0) THEN
             LPRHES=.TRUE.
             GOTO 10
          ELSE IF (INDEX(LINE,'DISPL').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSR(LINE,1,EPS_H,NSMAXP,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE IF (INDEX(LINE,'HESSTYPE').NE.0) THEN
             READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
             CALL READSI(LINE,1,IE,M_HESS,ERREAD)
             IF(ERREAD) GOTO 99
             GOTO 10
          ELSE
             GOTO 10
          END IF
        ELSE IF(INDEX(LINE,'HESSCORE').NE.0) THEN
C..Relaxation of environment and calculation of partial Hessian
          PRFO=.TRUE.
          NSMAXP=0
          GOTO 10
        ELSE IF(INDEX(LINE,'BFGS').NE.0) THEN
C..Geometry optimization by quasi-Newton update
          BFGS=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'RFO').NE.0) THEN
C..Geometry optimization by Rational Function Approximation
          RFO=.TRUE.
          IF(INDEX(LINE,'ORDER').NE.0) THEN
            I1=INDEX(LINE,'=')
            CALL READSI(LINE,I1+1,IOUT,NSORDER,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF 
     *   (((INDEX(LINE,'INR').NE.0).OR.(INDEX(LINE,'TURBOCICCIO').NE.0))
     &         .AND.INDEX(LINE,'PARA').NE.0) THEN
C..Param. for INR geometry optim.: nreg; gnx_inr(nreg); tolx_inr(nreg) 
          II=INDEX(LINE,'=')
          IF(II.NE.0) THEN
            CALL READSI(LINE,II+1,IOUT,NREG,ERREAD)
            IF(NREG.LE.0.OR.NREG.GT.MAXREG) THEN
              WRITE(6,'(A,I3)')
     &             ' CONTROL| NUMBER OF GRADIENT REGIONS : ',NREG
              WRITE(6,'(A,I3)')
     &             ' CONTROL| BETWEEN 1 AND',MAXREG
              CALL STOPGM('CONTROL','WRONG NREG NUMBER')
            ENDIF
          ENDIF
          DO I=1,NREG
             READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
             CALL READSR(LINE,1,IE,GNX_INR(I),ERREAD)
             CALL READSR(LINE,IE+1,IE,TOLX_INR(I),ERREAD)
          ENDDO
          GOTO 10
        ELSE IF((INDEX(LINE,'IMPLICIT').NE.0 .AND.
     &           INDEX(LINE,'NEWTON').NE.0 .AND.
     &           INDEX(LINE,'RAPHSON').NE.0)
     &         .OR.(INDEX(LINE,'TURBOCICCIO').NE.0)) THEN
           TINR=.TRUE.
C..Geometry optimization by Implicit Newton-Raphson
           LINEOLD=LINE
           READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
           CALL READSI(LINE,1,IE,ITMAX_INR,ERREAD)
           IF(INDEX(LINEOLD,'CONT').ne.0) inr_cont=.true.
           IF(INDEX(LINEOLD,'PREC').NE.0) THEN
              INR_PREC=.TRUE.
           ENDIF
           IF ((INDEX(LINEOLD,'ALTERNATIVE').NE.0)
     &           .AND.(INDEX(LINEOLD,'STEP').NE.0))
     &          INR_STEP=.TRUE.
           IF (INDEX(LINEOLD,'VERBOSE').NE.0) INR_VERBOSE=.TRUE.
           GOTO 10
        ELSE IF(INDEX(LINE,'MIXSD').NE.0) THEN
           tmixsd=.true.
           READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
           CALL READSR(LINE,1,IE,rmixsd,ERREAD)
           GOTO 10
        ELSE IF(INDEX(LINE,'MIXDIIS').NE.0) THEN
           tmixgdiis=.true.
           READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
           CALL READSR(LINE,1,IE,rmixsd,ERREAD)
           GOTO 10

        ELSE IF(INDEX(LINE,'RESTART').NE.0) THEN
C..Restart
          RESTART=.TRUE.
          IF(INDEX(LINE,'WAVE')  .NE.0) RWF  =.TRUE.
          IF(INDEX(LINE,'COOR')  .NE.0) RCO  =.TRUE.
          IF(INDEX(LINE,'VELO')  .NE.0) RVEL =.TRUE.
          IF(INDEX(LINE,'ACCU')  .NE.0) RAC  =.TRUE.
          IF(INDEX(LINE,'HESS')  .NE.0) RHE  =.TRUE.
          IF(INDEX(LINE,'NOSEE') .NE.0) RNOE =.TRUE.
          IF(INDEX(LINE,'NOSEP') .NE.0) RNOP =.TRUE.
          IF(INDEX(LINE,'NOSEC') .NE.0) RNOC =.TRUE.
          IF(INDEX(LINE,'GEOFI') .NE.0) RGEO =.TRUE.
          IF(INDEX(LINE,'LATE')  .NE.0) RLATE=.TRUE.
          IF(INDEX(LINE,'VIBA')  .NE.0) RVIB =.TRUE.
          IF(INDEX(LINE,'CELL')  .NE.0) RCELL=.TRUE.
          IF(INDEX(LINE,'POTENT').NE.0) RPOT =.TRUE.
          IF(INDEX(LINE,'DENSIT').NE.0) RRHO =.TRUE.
          IF(INDEX(LINE,'KPOINT').NE.0) RKPT =.TRUE.
          IF(INDEX(LINE,'OCCUP') .NE.0) ROCC =.TRUE.
          IF(INDEX(LINE,'ORBHA') .NE.0) ROH  =.TRUE.
          IF(INDEX(LINE,'LINRES').NE.0) RLR  =.TRUE.
          IF(INDEX(LINE,'PHES')  .NE.0) RPHES=.TRUE.
          IF(INDEX(LINE,'LSSTAT').NE.0) RLSCL=.TRUE.
          IF(INDEX(LINE,'ADPTTL').NE.0) RDTL =.TRUE.
          IF(INDEX(LINE,'CONSTR').NE.0) RCON =.TRUE.
          IF(INDEX(LINE,'EXTRAP').NE.0) RXTP =.TRUE.
          IF(INDEX(LINE,'NOREST').NE.0) RNON =.TRUE.
          IF(INDEX(LINE,'CVDFT') .NE.0) RCDFT=.TRUE.
          IF(INDEX(LINE,'PRNG')  .NE.0) RPRNG=.TRUE.
          IF(INDEX(LINE,'GLE')   .NE.0) RGLE= .TRUE.
           
          IF(INDEX(LINE,'ALL')   .NE.0) THEN
            RWF  =.TRUE.
            RCO  =.TRUE.
            RVEL =.TRUE.
            RAC  =.TRUE.
            RHE  =.TRUE.
            RNOE =.TRUE.
            RNOP =.TRUE.
            RNOC =.TRUE.
            RGEO =.TRUE.
            RLATE=.TRUE.
            RVIB =.TRUE.
            RCELL=.TRUE.
            RPOT =.TRUE.
            RRHO =.TRUE.
            RKPT =.TRUE.
            ROCC =.TRUE.
            ROH  =.TRUE.
            RLR  =.TRUE.
            RCON =.TRUE.
            RCDFT=.TRUE.
            RXTP =.TRUE.
            RPRNG=.TRUE.
            RGLE =.TRUE.
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'INTFILE').NE.0) THEN
          IF(INDEX(LINE,'READ').NE.0) INTREAD=.TRUE.
          IF(INDEX(LINE,'WRITE').NE.0) INTWRITE=.TRUE.
          IF(INDEX(LINE,'FILENAME').NE.0) THEN
            READ(IUNIT,ERR=99,END=99,FMT='(A)') INTFN
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'MAXS').NE.0) THEN
C..Maximum number of steps
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE ,1,IE,NOMORE,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'MAXI').NE.0) THEN
C..Maximum number of iterations (for diag or MD BO)
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE ,1,IE,NOMORE_ITER,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'MAXC').NE.0) THEN
C..Maximum time for the job
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSR(LINE ,1,IE,VAR,ERREAD)
          IF(ERREAD) GOTO 99
          IF(TJLIMIT.LE.0.D0) THEN
            TJLIMIT=VAR
          ELSE
            TJLIMIT=MIN(TJLIMIT,VAR)
          ENDIF
          GOTO 10
        ELSE IF 
     *    (INDEX(LINE,'INITI').NE.0.AND.INDEX(LINE,'WAVE').NE.0) THEN
C..Wavefunction initialization
          IF(INDEX(LINE,'RAND').NE.0) INWFUN=1
          IF(INDEX(LINE,'ATOM').NE.0) THEN
            INWFUN=2
            IF(INDEX(LINE,'PRIM').NE.0) TIPAO=.TRUE.
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'RATTLE').NE.0) THEN
C..Parameters for the RATTLE step
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSI(LINE ,IA,IE,MAXIT,ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSR(LINE ,IA,IE,EPSOG,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'ORTHOGONAL').NE.0) THEN
C..Method for orthogonalization  
          IF(INDEX(LINE,'GRAM').NE.0) TLOWD=.FALSE.
          IF(INDEX(LINE,'LOWD').NE.0) TLOWD=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'QUENCH').NE.0) THEN
C..Quench velocities
          IF(INDEX(LINE,'IONS').NE.0) QUENCHP=.TRUE.
          IF(INDEX(LINE,'ELEC').NE.0) QUENCHE=.TRUE.
          IF(INDEX(LINE,'BO').NE.0) QUENCHB=.TRUE.
          IF(INDEX(LINE,'CELL').NE.0) QUENCHC=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'RAND').NE.0) THEN
C..Randomize ionic positions and/or wavefunction
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IF(INDEX(LINEOLD,'WAVE').NE.0) THEN
            TRANE=.TRUE.
            CALL READSR(LINE,1,IE,AMPRE, ERREAD)
          ELSE IF(INDEX(LINEOLD,'COOR').NE.0) THEN
            TRANP=.TRUE.
            CALL READSR(LINE,1,IE,AMPRP, ERREAD)
          ELSE IF(INDEX(LINEOLD,'CELL').NE.0) THEN
            TRANC=.TRUE.
            CALL READSR(LINE,1,IE,AMPRC, ERREAD)
          ELSE IF(INDEX(LINEOLD,'DENS').NE.0) THEN
            TRAND=.TRUE.
            CALL READSR(LINE,1,IE,AMPRD, ERREAD)
          ELSE
            CALL STOPGM('CONTROL',
     &           'UNKNOWN OPTION FOR RANDOMIZATION')
          ENDIF
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'TIMESTEP').NE.0) THEN
C..Time step for electrons and ions
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IF(INDEX(LINEOLD,'ION').EQ.0 .AND.
     &           INDEX(LINEOLD,'ELEC').EQ.0) THEN
            CALL READSR(LINE ,1,IE,DELT_ELEC,ERREAD)
            IF(ERREAD) GOTO 99
            DELT_IONS = DELT_ELEC
          ELSEIF(INDEX(LINEOLD,'ION').NE.0 .AND.
     &           INDEX(LINEOLD,'ELEC').NE.0) THEN
            CALL READSR(LINE ,1,IE,DELT_ELEC,ERREAD)
            IF(ERREAD) GOTO 99
            DELT_IONS = DELT_ELEC
          ELSEIF(INDEX(LINEOLD,'ELEC').NE.0) THEN
            CALL READSR(LINE ,1,IE,DELT_ELEC,ERREAD)
            IF(ERREAD) GOTO 99
          ELSEIF(INDEX(LINEOLD,'ION').NE.0) THEN
            CALL READSR(LINE ,1,IE,DELT_IONS,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'EMASS').NE.0) THEN
C..Electron mass
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSR(LINE ,1,IE,EMASS,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'CMASS').NE.0) THEN
C..Cell mass
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSR(LINE ,1,IE,CMASS,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'TEMPERA').NE.0) THEN
C..Temperature 
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          IF(INDEX(LINEOLD,'ELECT').NE.0) THEN
            CALL READSR(LINE,IA,IE,BETAEL,ERREAD)
            IF(ERREAD) GOTO 99
          ELSE
            CALL READSR(LINE,IA,IE,TEMPW,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            IF(INDEX(LINEOLD,'RAMP').NE.0) THEN
              CALL READSR(LINE,IA,IE,TRAMPT,ERREAD)
              IF(ERREAD) GOTO 99
              IA=IE
              CALL READSR(LINE,IA,IE,TRAMPR,ERREAD)
              IF(ERREAD) GOTO 99
            ELSE
              TRAMPT=TEMPW
              TRAMPR=0.0d0
            ENDIF
          ENDIF
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'RESCA').NE.0.AND.INDEX(LINE,'OLD').NE.0
     *     .AND.INDEX(LINE,'VELOCI').NE.0) THEN
C..Re-adjust ionic velocities only after restart to
C..desired temperature TEMPW
          TRESCALE=.TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'SUBTR').NE.0) THEN
C..   Subtract the center of mass velocity each NCOMV steps
C..   and/or subtract the rotational velocity each NROTV steps
          IF ((INDEX(LINE,'COMVEL').NE.0).AND.
     &         (INDEX(LINE,'ROTVEL').NE.0)) THEN
C..   do both
            TSUBCOM=.TRUE.
            TSUBROT=.TRUE.
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,NCOMV,ERREAD)
            IF(ERREAD) GOTO 99
            NROTV=NCOMV
          ELSE IF (INDEX(LINE,'COMVEL').NE.0) THEN
C..   only COM
            TSUBCOM=.TRUE.
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,NCOMV,ERREAD)
            IF(ERREAD) GOTO 99
          ELSE IF (INDEX(LINE,'ROTVEL').NE.0) THEN
            TSUBROT=.TRUE.
C..   only ROT
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,NROTV,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'TEMPCO').NE.0) THEN
C..Temperature control
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IF(INDEX(LINEOLD,'IONS').NE.0) THEN
C..For the ions
            TCP=.TRUE.
            IA=1
            CALL READSR(LINE,IA,IE,TEMPW,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,TOLP, ERREAD)
            IF(ERREAD) GOTO 99
          ELSE IF(INDEX(LINEOLD,'ELEC').NE.0) THEN
C..For the electrons
            TC=.TRUE.
            IA=1
            CALL READSR(LINE,IA,IE,EKINW,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,TOLL, ERREAD)
            IF(ERREAD) GOTO 99
          ELSE IF (INDEX(LINEOLD,'CELL').NE.0) THEN
C..For the cell
            TCC=.TRUE.
            IA=1
            CALL READSR(LINE,IA,IE,EKINHR,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,TOLC, ERREAD)
            IF(ERREAD) GOTO 99
          ELSE
            CALL STOPGM('CONTROL','UNKNOWN TEMPERATURE CONTROL')
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'BERENDS').NE.0) THEN
C..Temperature control
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IF(INDEX(LINEOLD,'IONS').NE.0) THEN
C..For the ions
            TBERP=.TRUE.
            IA=1
            CALL READSR(LINE,IA,IE,TEMPW,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,TAUBP, ERREAD)
            IF(ERREAD) GOTO 99
          ELSE IF(INDEX(LINEOLD,'ELEC').NE.0) THEN
C..For the electrons
            TBERE=.TRUE.
            IA=1
            CALL READSR(LINE,IA,IE,EKINW,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,TAUBE, ERREAD)
            IF(ERREAD) GOTO 99
          ELSE IF (INDEX(LINEOLD,'CELL').NE.0) THEN
C..For the cell
            TBERC=.TRUE.
            IA=1
            CALL READSR(LINE,IA,IE,EKINHR,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,TAUBC, ERREAD)
            IF(ERREAD) GOTO 99
          ELSE
            CALL STOPGM('CONTROL','UNKNOWN BERENDSEN FLAG')
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'NOSE').NE.0 .AND.
     *           INDEX(LINE,'IONS').NE.0) THEN
C..Nose thermostats for ions
            TNOSEP=.TRUE.
            IF(INDEX(LINE,'ULTRA').NE.0) TULTRA=.TRUE.
            IF(INDEX(LINE,'MASSIVE').NE.0) TMNOSE=.TRUE.
            IF(INDEX(LINE,'CAFES').NE.0) THEN
                TCAFES=.TRUE.
                TMNOSE=.TRUE.
                TULTRA=.FALSE.
                READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
                IA=1
                CALL READSI(LINE,IA,IE,NCAFESGRP,ERREAD)
                IF (ERREAD) GOTO 99
                IF (NCAFESGRP.LE.1) THEN
                  WRITE(6,*) 'CONTROL| WARNING! NCAFESGRP SHOULD BE > 1'
                ENDIF
                CALL MEMORY(IP_CAFESINI,2/IRAT*NCAFESGRP,'CAFESINI')
                CALL MEMORY(IP_CAFESINR,2*NCAFESGRP,'CAFESINR')
                DO I=1,NCAFESGRP
                   READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
                   IA=1
                   CALL READSI(LINE,IA,IE,CAFESINI(1,I),ERREAD)
                   IF (ERREAD) GOTO 99
                   IA=IE
                   CALL READSI(LINE,IA,IE,CAFESINI(2,I),ERREAD)
                   IF (ERREAD) GOTO 99
                   IA=IE
                   CALL READSR(LINE,IA,IE,CAFESINR(1,I),ERREAD)
                   IF (ERREAD) GOTO 99
                   IA=IE
                   CALL READSR(LINE,IA,IE,CAFESINR(2,I),ERREAD)
                   IF (ERREAD) GOTO 99
                ENDDO
                DO IA=1,NAT
                  IFLAG=0
                  DO IE=1,NCAFESGRP
                    IF ((CAFESINI(1,IE).LE.IA)
     *                  .AND. (CAFESINI(2,IE).GE.IA)) THEN
                      IFLAG=IFLAG+1
                    ENDIF
                  ENDDO
                  IF (IFLAG.NE.1) THEN
                        WRITE(6,*) 'ATOM ',IA,' IS LISTED IN ',
     *                             IFLAG,'CAFES GROUPS'
                        CALL STOPGM('CONTROL','CAFES INPUT ERROR')
                  ENDIF
                ENDDO
C (GM) local thermostats
            ELSEIF(INDEX(LINE,'LOCAL').NE.0) THEN 
                TLOCT=.TRUE.
                TCAFES=.FALSE.
                TMNOSE=.FALSE.
                TULTRA=.FALSE.
                LINEOLD=LINE
                READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
                IA=1
C (GM) NUMBER OF LOCAL THEROMOSTATS
                CALL READSI(LINE,IA,IE,NLOCT,ERREAD)
                IF (ERREAD) GOTO 99
                CALL MEMORY(IP_LOCTPIN,2*NLOCT,'LOCTPIN')
                CALL MEMORY(IP_LOCTT0,NLOCT,'LOCTT0')
C (GM) PARAMETERS FOR EACH THERMOSTAT
                DO I=1,NLOCT
                   READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
                   IA=1
                   CALL READSR(LINE,IA,IE,LOCTPIN(1,I),ERREAD)
                   IF (ERREAD) GOTO 99
                   IA=IE
                   CALL READSR(LINE,IA,IE,LOCTPIN(2,I),ERREAD)
                   IF (ERREAD) GOTO 99
                   IF(INDEX(LINEOLD,'T0').NE.0)THEN
                      IA=IE
                      CALL READSR(LINE,IA,IE,LOCTT0(I),ERREAD)
                      IF (ERREAD) GOTO 99
                   ELSE
                      LOCTT0(I)=LOCTPIN(1,I)
                   END IF
                ENDDO
                TEMPW=LOCTPIN(1,1)
C (GM) NEXT LINE: NUMBER OF LOCAL THERMOSTAT RANGES
                READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
                IA=1
                CALL READSI(LINE,IA,IE,NLOCRNG,ERREAD)
                IF (ERREAD) GOTO 99
                CALL MEMORY(IP_LCTRNG,(3 * NLOCRNG+1) / IRAT,'LCTRNG')
C (GM) READ IN THE RANGES:  <THERMOSTAT NO>  <START ATOM>  <END ATOM>
                DO I=1,NLOCRNG
                   READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
                   IA=1
                   CALL READSI(LINE,IA,IE,LCTRNG(1,I),ERREAD)
                   IF (ERREAD) GOTO 99
                   IA=IE
                   CALL READSI(LINE,IA,IE,LCTRNG(2,I),ERREAD)
                   IF (ERREAD) GOTO 99
                   IA=IE
                   CALL READSI(LINE,IA,IE,LCTRNG(3,I),ERREAD)
                   IF (ERREAD) GOTO 99
                ENDDO
            ELSE
              LINEOLD=LINE
              READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
              IA=1
              CALL READSR(LINE,IA,IE,TEMPW, ERREAD)
              IF(ERREAD) GOTO 99
              IA=IE
              CALL READSR(LINE,IA,IE,WNOSP0,ERREAD)
              IF(ERREAD) GOTO 99
              IF(INDEX(LINEOLD,'T0').NE.0)THEN
                 IA=IE
                 CALL READSR(LINE,IA,IE,NOSPT0,ERREAD)
              ELSE
                 NOSPT0=TEMPW
              ENDIF
            ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'NOSE').NE.0 .AND.
     *           INDEX(LINE,'ELEC').NE.0) THEN
C..Nose thermostats for electrons
            TNOSEE=.TRUE.
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            IA=1
            CALL READSR(LINE,IA,IE,EKINW, ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,WNOSE0,ERREAD)
            IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'NOSE').NE.0 .AND.
     *           INDEX(LINE,'CELL').NE.0) THEN
C..Nose thermostats for cell
            TNOSEC=.TRUE.
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            IA=1
            CALL READSR(LINE,IA,IE,TEMPC, ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,WNOSC0,ERREAD)
            IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'NOSE').NE.0 .AND.
     *           INDEX(LINE,'PARAMETER').NE.0) THEN
C..Nose thermostats explicit form
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSI(LINE,IA,IE,NCHP,   ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,NCHS,   ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,NCHB,   ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSR(LINE,IA,IE,NEDOF0, ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,NCALLS0,ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,NIT0,   ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'PRINT').NE.0) THEN
C..Print intermediate results
          TPRINT=.TRUE.
          IF(INDEX(LINE,'OFF').NE.0) THEN
            IFLAG=-1
          ELSE
            IFLAG=1
          ENDIF
          IF(INDEX(LINE,'INFO').NE.0)  IPRINT(IPRINT_INFO)  = IFLAG
          IF(INDEX(LINE,'EIGEN').NE.0) IPRINT(IPRINT_EIGEN) = IFLAG
          IF(INDEX(LINE,'COOR').NE.0)  IPRINT(IPRINT_COOR)  = IFLAG
          IF(INDEX(LINE,'FORC').NE.0)  IPRINT(IPRINT_FORCE) = IFLAG
          IF(INDEX(LINE,'WANN').NE.0)  IPRINT(IPRINT_WANN)  = IFLAG
          IF(INDEX(LINE,'LSCAL').NE.0) IPRINT(IPRINT_LSCAL) = IFLAG
          IF(INDEX(LINE,'ALL').NE.0) THEN
            IPRINT(IPRINT_INFO)  = IFLAG
            IPRINT(IPRINT_EIGEN) = IFLAG
            IPRINT(IPRINT_COOR)  = IFLAG
            IPRINT(IPRINT_FORCE) = IFLAG
            IPRINT(IPRINT_WANN)  = IFLAG
          ENDIF
          IF(INDEX(LINE,'ENERG').NE.0) THEN
            IF(INDEX(LINE,'EKIN').NE.0)  IPRINT(IPRINT_EKIN)  = IFLAG
            IF(INDEX(LINE,'EFREE').NE.0) IPRINT(IPRINT_EEIG)  = IFLAG
            IF(INDEX(LINE,'EBAND').NE.0) IPRINT(IPRINT_EBAND) = IFLAG
            IF(INDEX(LINE,'ENTROPY').NE.0)
     &                                   IPRINT(IPRINT_ENTROPY) = IFLAG
            IF(INDEX(LINE,'ELEC').NE.0)  THEN
              IPRINT(IPRINT_EHT)   = IFLAG
              IF(INDEX(LINE,'ELEC1').NE.0)
     &             IPRINT(IPRINT_ELEC1)   = IFLAG
              IF(INDEX(LINE,'ELEC2').NE.0)
     &             IPRINT(IPRINT_ELEC2)   = IFLAG
            ENDIF
            IF(INDEX(LINE,'EHEP').NE.0)  IPRINT(IPRINT_EHEP)  = IFLAG
            IF(INDEX(LINE,'EHEE').NE.0)  IPRINT(IPRINT_EHEE)  = IFLAG
            IF(INDEX(LINE,'EHII').NE.0)  IPRINT(IPRINT_EHII)  = IFLAG
            IF(INDEX(LINE,'ESR').NE.0)   IPRINT(IPRINT_ESR)   = IFLAG
            IF(INDEX(LINE,'ESELF').NE.0) IPRINT(IPRINT_ESELF) = IFLAG
            IF(INDEX(LINE,'EPSEU').NE.0) IPRINT(IPRINT_EPSEU) = IFLAG
            IF(INDEX(LINE,'ENL').NE.0)   IPRINT(IPRINT_ENL)   = IFLAG
            IF(INDEX(LINE,'EXCH').NE.0)  IPRINT(IPRINT_EXC)   = IFLAG
            IF(INDEX(LINE,'VXC').NE.0)   IPRINT(IPRINT_VXC)   = IFLAG
            IF(INDEX(LINE,'EGC').NE.0)   IPRINT(IPRINT_EGC)   = IFLAG
            IF(INDEX(LINE,'EBOGO').NE.0) IPRINT(IPRINT_EBOGO) = IFLAG
            IF(INDEX(LINE,'ETOT1').NE.0) IPRINT(IPRINT_ETOT1) = IFLAG
            IF(INDEX(LINE,'ETOT2').NE.0) IPRINT(IPRINT_ETOT2) = IFLAG
            IF(INDEX(LINE,'ECAS').NE.0)  IPRINT(IPRINT_ECAS)  = IFLAG
            IF(INDEX(LINE,'EPEN').NE.0)  IPRINT(IPRINT_EPEN)  = IFLAG
            IF(INDEX(LINE,'ETDDFT').NE.0)IPRINT(IPRINT_ETDDFT)= IFLAG
          ENDIF
          IF(TSIC) IPRINT(IPRINT_EHSIC) = 1 ! cmb_ssic
          IF(INDEX(LINE,'ON').EQ.0.AND.INDEX(LINE,'OFF').EQ.0) THEN
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,IPRINT_STEP,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'STRUCTURE').NE.0) THEN
C..Print structure parameters
          IF(INDEX(LINE,'BONDS').NE.0) BOND=.TRUE.
          IF(INDEX(LINE,'ANGLES').NE.0) ANGLE=.TRUE.
          IF(INDEX(LINE,'DIHEDRALS').NE.0) DIHEDRAL=.TRUE.
          IF(INDEX(LINE,'SELECT').NE.0) THEN
            TSSEL=.TRUE.
            READ(IUNIT,ERR=99,END=99,FMT=*) NSSEL
            CALL MEMORY(IP_NASSEL,NSSEL,'NASSEL')
            READ(IUNIT,ERR=99,END=99,FMT=*) (NASSEL(I),I=1,NSSEL)
          ELSE
            TSSEL=.FALSE.
            NSSEL=0
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'STORE').NE.0) THEN
C..Store intermediate results
          LINEOLD=LINE
          IF(INDEX(LINE,'OFF').NE.0) THEN
            TOFF=.FALSE.
          ELSE
            TOFF=.TRUE.
          ENDIF
          IF(INDEX(LINE,'POTEN').NE.0) SPOT=.TRUE..AND.TOFF
          IF(INDEX(LINE,'WAVEFUNC').NE.0) SWF=.TRUE..AND.TOFF
          IF(INDEX(LINE,'DENSIT').NE.0) THEN
            TSRHO=.TRUE..AND.TOFF
            SRHO=.TRUE..AND.TOFF
          ENDIF
          IF(TOFF) THEN
            READ(IUNIT,ERR=99,END=99,FMT='(A)') LINE
            CALL READSI(LINE,1,IE,ISTORE,ERREAD)
            IF(ERREAD) GOTO 99
            II=INDEX(LINE,'=')
            IF(II.NE.0) THEN
              CALL READSI(LINE,II+1,IOUT,ISCTORE,ERREAD)
              IF(ERREAD) GOTO 99
            ENDIF
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'RESTFILE').NE.0) THEN
C..Number of different restart files
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE,1,IE,NRESTF,ERREAD)
          IF(ERREAD) GOTO 99
          IF(INDEX(LINEOLD,'SAMPLE').NE.0) THEN
            IF(NRESTF.GT.MAXRF) 
     *         CALL STOPGM('CONTROL','TOO MANY RESTART FILES')
C           We can use more than one line.
            READ(IUNIT,ERR=99,END=99,FMT=*) (NSTEPWR(I),I=1,NRESTF)
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'CONVERG').NE.0) THEN
C..Convergence criteria
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IF (INDEX(LINEOLD,'ORBITAL').NE.0) THEN
            CALL READSR(LINE,1,IE,TOLOG,ERREAD)
            IA=IE
            CALL READSR(LINE,IA,IE,TOLDETOT,ERREAD)
            ERREAD=.FALSE.
          ELSE IF (INDEX(LINEOLD,'GEOMETRY').NE.0) THEN
            CALL READSR(LINE,1,IE,TOLNG,ERREAD)
          ELSE IF (INDEX(LINEOLD,'CELL').NE.0) THEN
            CALL READSR(LINE,1,IE,TOLCG,ERREAD)
          ELSE IF (INDEX(LINEOLD,'ADAPT').NE.0) THEN
            CALL READSR(LINE,1,IE,TOLAD,ERREAD)
          ELSE IF (INDEX(LINEOLD,'ENERGY').NE.0) THEN
            CALL READSR(LINE,1,IE,TOLENE,ERREAD)
          ELSE IF (INDEX(LINEOLD,'CALFOR').NE.0) THEN
            CALL READSR(LINE,1,IE,TOLFOR,ERREAD)
          ELSE IF (INDEX(LINEOLD,'RELAX').NE.0) THEN
            CALL READSI(LINE,1,IE,NSTCNV,ERREAD)
          ELSE IF (INDEX(LINEOLD,'RHOFIX').NE.0) THEN
            CALL READSR(LINE,1,IE,TOLRHOFIX,ERREAD)
          ELSE IF (INDEX(LINEOLD,'INITIAL').NE.0) THEN
            CALL READSR(LINE,1,IE,TOLINI,ERREAD)
          ELSE IF (INDEX(LINEOLD,'CONSTR').NE.0) THEN
            CALL READSR(LINE,1,IE,VCCON,ERREAD)
            IA=IE
            IF(ERREAD) GOTO 99
            CALL READSR(LINE,IA,IE,VCCONU,ERREAD)
            IF(ERREAD) VCCONU=VCCON
            ERREAD=.FALSE.
          ELSE
            IA=1
            CALL READSR(LINE,IA,IE,TOLOG,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,TOLNG,ERREAD)
            ERREAD=.FALSE.
          ENDIF
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'ANNE').NE.0) THEN
C..Simulated Annealing
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IF (INDEX(LINEOLD,'IONS').NE.0) THEN
            ANNEI=.TRUE.
            CALL READSR(LINE,1,IE,ANNERI,ERREAD)
          ELSEIF (INDEX(LINEOLD,'ELECTRON').NE.0) THEN
            ANNEE=.TRUE.
            CALL READSR(LINE,1,IE,ANNERE,ERREAD)
          ELSEIF (INDEX(LINEOLD,'CELL').NE.0) THEN
            ANNEC=.TRUE.
            CALL READSR(LINE,1,IE,ANNERC,ERREAD)
          ELSE
            ANNEI=.TRUE.
            ANNEE=.TRUE.
            ANNEC=.TRUE.
            IA=1
            CALL READSR(LINE,IA,IE,ANNERI,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,ANNERE,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,ANNERC,ERREAD)
          ENDIF
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'DAMP').NE.0) THEN
C..Damped Dynamics
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IF (INDEX(LINEOLD,'IONS').NE.0) THEN
            DAMPI=.TRUE.
            CALL READSR(LINE,1,IE,DAMPGI,ERREAD)
          ELSEIF (INDEX(LINEOLD,'ELEC').NE.0) THEN
            DAMPE=.TRUE.
            CALL READSR(LINE,1,IE,DAMPGE,ERREAD)
          ELSEIF (INDEX(LINEOLD,'CELL').NE.0) THEN
            DAMPC=.TRUE.
            CALL READSR(LINE,1,IE,DAMPGC,ERREAD)
          ELSE
            DAMPI=.TRUE.
            DAMPE=.TRUE.
            DAMPC=.TRUE.
            IA=1
            CALL READSR(LINE,IA,IE,DAMPGI,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,DAMPGE,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,DAMPGC,ERREAD)
          ENDIF
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'MOLECULE').NE.0 .AND.
     *           INDEX(LINE,'ISOLATED').NE.0) THEN
C..Isolated molecule
          TISOS=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'MOLECULE').NE.0 .AND.
     *           INDEX(LINE,'CENTER').NE.0) THEN
C..Put COM to center of box
          IF(INDEX(LINE,'OFF').NE.0) THEN
            TCENT=.FALSE.
          ELSE
            TCENT=.TRUE.
          ENDIF
          GOTO 10
        ELSE IF (INDEX(LINE,'EXTERN').NE.0 .AND.
     *           INDEX(LINE,'POTENTIAL').NE.0) THEN
          TEXPOT=.TRUE.
          IF (INDEX(LINE,'ADD').NE.0) TEXADD=.TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'ELECTRO').NE.0 .AND.
     *           INDEX(LINE,'POTENTIAL').NE.0) THEN
          TEPOT=.TRUE.
          RHOOUT=.TRUE.
          IF(INDEX(LINE,'SAMPLE=').NE.0) THEN
            IA=INDEX(LINE,'=')
            CALL READSI(LINE,IA+1,IE,NRHOOUT,ERREAD)
            IF(ERREAD) GOTO 99
            IF(NRHOOUT.LT.0) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF (INDEX(LINE,'DIPO').NE.0 .AND.
     *           INDEX(LINE,'DYNA').NE.0) THEN
C..Store Dipole moments
          TDIPD=.TRUE.
          IF(INDEX(LINE,'WANNIER').NE.0) TWANN=.TRUE.
          IF(INDEX(LINE,'SAMPLE').NE.0) THEN
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,NPDIP,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF (INDEX(LINE,'WANNIER').NE.0 .AND.
     *           INDEX(LINE,'PARAM').NE.0) THEN
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSR(LINE,IA,IE,W_STEP,ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSR(LINE,IA,IE,W_EPS, ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSR(LINE,IA,IE,W_RAN, ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,W_MAXS,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF (INDEX(LINE,'WANNIER').NE.0 .AND.
     *           INDEX(LINE,'OPTI').NE.0) THEN
          IF(INDEX(LINE,'SD').NE.0) W_OPT=1     
          IF(INDEX(LINE,'JACOBI').NE.0) W_OPT=2     
          GOTO 10
        ELSE IF (INDEX(LINE,'WANNIER').NE.0 .AND.
     *           INDEX(LINE,'TYPE').NE.0) THEN
          IF(INDEX(LINE,'VANDERBILT').NE.0) W_TYPE=1     
          IF(INDEX(LINE,'RESTA').NE.0) W_TYPE=2     
          GOTO 10
        ELSE IF (INDEX(LINE,'WANNIER').NE.0 .AND.
     *           INDEX(LINE,'REFE').NE.0) THEN
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          DO I=1,3
            CALL READSR(LINE,IA,IE,W_REF(I),ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
          ENDDO
          GOTO 10
        ELSE IF (INDEX(LINE,'WANNIER').NE.0 .AND.
     *           INDEX(LINE,'SERIAL').NE.0) THEN
          TWSERIAL=.TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'WANNIER').NE.0 .AND.
     *           INDEX(LINE,'DOS').NE.0) THEN
          TWDOS=.TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'WANNIER').NE.0 .AND.
     *           INDEX(LINE,'MOLECULAR').NE.0) THEN
          TWMOL=.TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'WANNIER').NE.0 .AND.
     *           INDEX(LINE,'WFNOUT').NE.0) THEN
          TWPRI=.TRUE.
          IF(INDEX(LINE,'DENSITY').NE.0) TSDENS=.TRUE.
          IF(INDEX(LINE,'ALL').NE.0) THEN
            SW_ALL=1
          ELSE 
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            IF(INDEX(LINEOLD,'PART').NE.0) THEN
              IA=1
              CALL READSI(LINE,IA,IE,SW_FIRST,ERREAD)
              IF(ERREAD) GOTO 99
              IA=IE
              CALL READSI(LINE,IA,IE,SW_LAST,ERREAD)
              IF(ERREAD) GOTO 99
            ELSEIF (INDEX(LINEOLD,'LIST').NE.0) THEN
              CALL READSI(LINE,1,IE,SW_ORB,ERREAD)
              IF(ERREAD) GOTO 99
              CALL MEMORY(IP_SW_LIST,SW_ORB,'SW_LIST')
C     We can use more than one line
              READ(IUNIT,ERR=99,END=99,FMT=*) (SW_LIST(I),I=1,SW_ORB)
            ELSEIF (INDEX(LINEOLD,'SPREAD').NE.0) THEN
              CALL READSR(LINE,1,IE,SW_SPREAD,ERREAD) 
              IF(ERREAD) GOTO 99
            ELSE
              GOTO 99
            ENDIF
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'RHOOUT').NE.0) THEN
C..Store density
          RHOOUT=.TRUE.
          IF(INDEX(LINE,'SAMPLE=').NE.0) THEN
            IA=INDEX(LINE,'=')
            CALL READSI(LINE,IA+1,IE,NRHOOUT,ERREAD)
            IF(ERREAD) GOTO 99
            IF(NRHOOUT.LT.0) GOTO 99
          ENDIF
          IF(INDEX(LINE,'BANDS').NE.0) THEN
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,NUMPR,ERREAD)
            IF(ERREAD) GOTO 99
            IF(NUMPR.LE.0) GOTO 99
            CALL MEMORY(IP_MWFN,NUMPR,'MWFN')
C           We can use more than one line
            READ(IUNIT,ERR=99,END=99,FMT=*) (MWFN(I),I=1,NUMPR )
            RHOOUT=.TRUE.
          ENDIF
          GOTO 10
        ELSE IF (INDEX(LINE,'ELF').NE.0) THEN
          TELF=.TRUE.
          RHOOUT=.TRUE.
          IF (INDEX(LINE,'PARAME').NE.0) THEN
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            IA=1
            CALL READSR(LINE,IA,IE,ELFCUT,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSR(LINE,IA,IE,ELFEPS,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'ENERGYBAND').NE.0) THEN
          TEBAND=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'HESSIAN').NE.0) THEN
C..Initial hessian
          IF(INDEX(LINE,'UNIT').NE.0) NPARA=-1
          IF(INDEX(LINE,'DISCO').NE.0) NPARA=0
          IF(INDEX(LINE,'SCHLEGEL').NE.0) NPARA=1
          IF(INDEX(LINE,'PARTIAL').NE.0) NPARA=NPARA+3
          GOTO 10
        ELSE IF(INDEX(LINE,'PROJECT').NE.0) THEN
C..Electronic gradient projection to be used
          IF(INDEX(LINE,'NONE').NE.0) IPROJ=0
          IF(INDEX(LINE,'DIAGONAL').NE.0) IPROJ=1
          IF(INDEX(LINE,'FULL').NE.0) IPROJ=2
          GOTO 10
        ELSE IF(INDEX(LINE,'TRAJ').NE.0) THEN
C..Store trajectory
          ROUT=.TRUE.
          IF(INDEX(LINE,'XYZ').NE.0) XTOUT=.TRUE.
          IF(INDEX(LINE,'DCD').NE.0) DCOUT=.TRUE.
          IF(INDEX(LINE,'OFF').NE.0) THEN
            ROUT=.FALSE.
            XTOUT=.FALSE.
            DCOUT=.FALSE.
          ENDIF
          IF(INDEX(LINE,'BIN').NE.0) THEN
            TWRITEBINTRAJECTORY=.TRUE.
          ENDIF
          IF(INDEX(LINE,'FORCE').NE.0) THEN
            TWRITEFORCETRAJECTORY=.TRUE.
          ENDIF
          IF(INDEX(LINE,'SMALL').NE.0) TRAJSMALL=.TRUE.
          IF(INDEX(LINE,'RANGE').NE.0) THEN
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=*)
     $           MINWRITEATOM, MAXWRITEATOM
          END IF
          IF(INDEX(LINE,'SAMPLE').NE.0) THEN
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,NTRAJ,ERREAD)
            IF(ERREAD) GOTO 99
C     turn off trajectory writing for NTRAJ=0
C     and disable TRAJECTORY file, but not TRAJEC.xyz for NTRAJ < 0
            IF(NTRAJ.EQ.0) THEN
              NTRAJ=1
              ROUT=.FALSE.
              XTOUT=.FALSE.
              DCOUT=.FALSE.
            ELSE IF(NTRAJ.LT.0) THEN
              ROUT=.FALSE.
              NTRAJ=-NTRAJ
            END IF
            IF (TRAJSMALL) THEN
              READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
              CALL READSI(LINE,1,IE,TRAJSMALLN,ERREAD)
            ENDIF
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'MOVIE').NE.0) THEN
C..Write movie file every imovie steps    
          MOUT=.TRUE.
          IF(INDEX(LINE,'OFF').NE.0) MOUT=.FALSE.
          IF(INDEX(LINE,'SAMPLE').NE.0) THEN
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,IMOVIE,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'STRESS TENSOR').NE.0) THEN
C..Stress tensor calculation
          TPRES=.TRUE.
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE,1,IE,NPRES,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'CLASSTRESS').NE.0) THEN
          TPREC=.TRUE.
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE,1,IE,NPREC,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'COMPRESS').NE.0) THEN
C..Wavefunction in compressed form          
          IF(INDEX(LINE,'WRITE').NE.0) THEN
             WCOMP=.TRUE.
             WCOMPB=2
          ENDIF
          IF(INDEX(LINE,'WRITEAO').NE.0) WCOMPB=-1
          IF(INDEX(LINE,'WRITE32').NE.0) WCOMPB=2
          IF(INDEX(LINE,'WRITE16').NE.0) WCOMPB=4
          IF(INDEX(LINE,'WRITE8' ).NE.0) WCOMPB=8
          GOTO 10
        ELSE IF(INDEX(LINE,'MEMORY').NE.0) THEN
C..Use small memory version of Vanderbilt code
          IF(INDEX(LINE,'SMALL').NE.0) BIGMEM=.FALSE.
          IF(INDEX(LINE,'BIG').NE.0) BIGMEM=.TRUE.
C..Check memory frequently
          IF(INDEX(LINE,'CHECK').NE.0) TMEMCHK=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'REAL').NE.0 .AND.
     *           INDEX(LINE,'SPACE').NE.0 .AND.
     *           INDEX(LINE,'WFN').NE.0) THEN
C..Info on real space wavefunctions
          IF(INDEX(LINE,'KEEP').NE.0) THEN
            KRWFN=.TRUE.
          ELSE
            KRWFN=.FALSE.
          ENDIF
          IF(INDEX(LINE,'SIZE').NE.0) THEN
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSR(LINE,1,IE,MEMSIZE,ERREAD)
          ELSE
            MEMSIZE=-1.D0
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'SPLINE').NE.0) THEN
          IF(INDEX(LINE,'POINTS').NE.0) THEN
C..Number of spline points
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,NSPLP,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          IF(INDEX(LINE,'QFUNC').NE.0) QSPLINE=.TRUE.
          IF(INDEX(LINE,'INIT').NE.0) INITSPL=.TRUE.
          IF (INDEX(LINE,'RANGE').NE.0) THEN
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSR(LINE,1,IE,QSRANG,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'FINITE').NE.0 .AND.
     *           INDEX(LINE,'DIFFER').NE.0) THEN
C..Step length for finite differences
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSR(LINE,1,IE,FDIFF,ERREAD)
          IF(ERREAD) THEN
            GOTO 99
          ENDIF
          IA=INDEX(LINE,'COORD=')
          IF (IA.NE.0) THEN
            TREF_FDIFF=.TRUE.
            IA=IA+6
C..Position of the finite differences sphere.
            DO I=1,3
              CALL READSR(LINE,IA,IE,COORD_FDIFF(I),ERREAD)
              IF(ERREAD) THEN
                GOTO 99
              ENDIF
              IA=IE+1
            ENDDO
            IA=INDEX(LINE,'RADIUS=')
            IF (IA.EQ.0) THEN
              WRITE(6,*)
     &             'CONTROL! FINITE DIFFERENCE WITH ATOM NEEDS RADIUS'
              CALL STOPGM('CONTROL','NO VALUE OF RADIUS GIVEN')
            ELSE
              IA=IA+7
              CALL READSR(LINE,IA,IE,R_FDIFF,ERREAD)
              IF(ERREAD) THEN
                GOTO 99
              ENDIF
            ENDIF
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'TASKGROU').NE.0) THEN
C..Taskgroups in parallel FFT
          IF(INDEX(LINE,'MAXI').NE.0) THEN
            DO I=MAXGRP,1,-1
              IT=MOD(NPROC,I)
              IF(IT.EQ.0) THEN
                NOGRP=I
                GOTO 80
              ENDIF
            ENDDO
   80       CONTINUE
          ELSEIF(INDEX(LINE,'MINI').NE.0) THEN
            DO I=2,MAXGRP
              IT=MOD(NPROC,I)
              IF(IT.EQ.0) THEN
                NOGRP=I
                GOTO 70
              ENDIF
            ENDDO
   70       CONTINUE
          ELSE
            IF(INDEX(LINE,'CART').NE.0) THEN
             TCART=.TRUE.
            ENDIF
            LINEOLD=LINE
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,NOGRP,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'DISTRI').NE.0 .AND.
     *           INDEX(LINE,'FNL').NE.0) THEN
C..Storage form of FNL
          IF(INDEX(LINE,'ON').NE.0) TFDIST=.TRUE.
          IF(INDEX(LINE,'OFF').NE.0) TFDIST=.FALSE.
          GOTO 10
        ELSE IF(INDEX(LINE,'FILEPATH').NE.0) THEN
C..Path to the restart files (all the line)
          READ(IUNIT,ERR=99,END=99,FMT='(A)') FPATH
          GOTO 10
        ELSE IF(INDEX(LINE,'FUSION').NE.0 .AND.
     *           INDEX(LINE,'FILE').NE.0) THEN
C..File fusion
          TFUSI=.TRUE.
          READ(IUNIT,ERR=99,END=99,FMT='(A)') S0_FILN
          READ(IUNIT,ERR=99,END=99,FMT='(A)') S1_FILN
          GOTO 10
        ELSE IF(INDEX(LINE,'MERGE').NE.0 .AND.
     *           INDEX(LINE,'FILE').NE.0) THEN
C..File merge
          TMERGE=.TRUE.
          READ(IUNIT,ERR=99,END=99,FMT='(A)') MFILN1
          READ(IUNIT,ERR=99,END=99,FMT='(A)') MFILN2
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSI(LINE,IA,IE,MNST1,ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,MNST2,ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,MORTHO,ERREAD)
          IF(ERREAD) GOTO 99
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSR(LINE,IA,IE,MSHIFT(1),ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSR(LINE,IA,IE,MSHIFT(2),ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSR(LINE,IA,IE,MSHIFT(3),ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'SEPARATION').NE.0 .AND.
     *           INDEX(LINE,'FILE').NE.0) THEN
C..File separation
          TSEP=.TRUE.
          READ(IUNIT,ERR=99,END=99,FMT='(A)') S0_FILN
          READ(IUNIT,ERR=99,END=99,FMT='(A)') S1_FILN
          GOTO 10
        ELSE IF(INDEX(LINE,'SHIFT').NE.0 .AND.
     *           INDEX(LINE,'POTENTIAL').NE.0) THEN
C..Shift potential
          READ(IUNIT,ERR=99,END=99,FMT='(A)') DSHIFT
          GOTO 10
C..Reverse Velocities (allow old 'INVERSE' and new 'REVERSE')
        ELSE IF(INDEX(LINE,'VERSE').NE.0 .AND.
     *         INDEX(LINE,'VEL').NE.0) THEN
          TREVERS=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'BENCHMARK').NE.0) THEN
C..Info on Benchmark
C         We can use more than one line.
          READ(IUNIT,ERR=99,END=99,FMT=*) (IBENCH(I),I=1,NBENTR)
          GOTO 10
        ELSE IF(INDEX(LINE,'MIRROR').NE.0) THEN
C..MIRROR input file on output file
          MIRROR=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'NO_RESET').NE.0) THEN
C..Number of steps between ODIIS resets if poor convergence
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
          CALL READSI(LINE,1,IE,NRESET,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
C..LOCALIZAZION of WF in RECIPROCAL SPACE
        ELSE IF (INDEX(LINE,'GLOCALIZATION').NE.0 .AND.
     *           INDEX(LINE,'PARAM').NE.0) THEN
          LINEOLD=LINE
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          IA=1
          CALL READSR(LINE,IA,IE,GLOC_STEP,ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSR(LINE,IA,IE,GLOC_EPS, ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSR(LINE,IA,IE,GLOC_RAN, ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,GLOC_MAXS,ERREAD)
          IF(ERREAD) GOTO 99
          IA=IE
          CALL READSI(LINE,IA,IE,GLOC_INIT,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF (INDEX(LINE,'GLOCALIZATION').NE.0 .AND.
     *           INDEX(LINE,'OPTI').NE.0) THEN
          TGLOC = .TRUE.
          IF(INDEX(LINE,'SD').NE.0 .AND. INDEX(LINE,'INTEGR').NE.0)
     &              GLOC_OPT=1     
          IF(INDEX(LINE,'ALMO').NE.0 .AND. INDEX(LINE,'IDENT').NE.0)
     &               GLOC_OPT=2
          GOTO 10
        ELSE IF (INDEX(LINE,'GFUNCTIONAL').NE.0 .AND.
     *           INDEX(LINE,'TYPE').NE.0) THEN
          IF(INDEX(LINE,'GSPREAD').NE.0) GLOC_TYPE=1     
          IF(INDEX(LINE,'ZETANUMBER').NE.0) GLOC_TYPE=2   
          GOTO 10
        ELSE IF (INDEX(LINE,'SPREAD').NE.0 .AND.
     *           INDEX(LINE,'RSPACE').NE.0) THEN
          TGWANNIER = .TRUE.
            IA=INDEX(LINE,'=')
            IF(IA.NE.0) THEN
              CALL READSR(LINE,IA+1,IE,G2G_WEIGHT,ERREAD)
              IF(ERREAD) GOTO 99
              IA=IE
              CALL READSR(LINE,IA,IE,WAN_WEIGHT,ERREAD)
              IF(ERREAD) GOTO 99
c              IF(WAN_WEIGHT+G2G_WEIGHT .GT. 1.0D0) 
c     *         CALL STOPGM('CONTROL','GLOC: SUM OF WEIGHTS > 1')
            ENDIF
c            G2G_WEIGHT = 1.D0
c            WAN_WEIGHT = 0.D0
          GOTO 10
        ELSE IF (INDEX(LINE,'PIPPO').NE.0) THEN
c        ELSE IF (INDEX(LINE,'UNITARY').NE.0 .AND.
c     *           INDEX(LINE,'CONST').NE.0) THEN
          IF(INDEX(LINE,'LAGRANGE').NE.0) THEN 
             GLOC_CONST = 1
            IA=INDEX(LINE,'=')
            IF(IA.NE.0) THEN
              CALL READSR(LINE,IA+1,IE,GEPSLAG,ERREAD)
              IF(ERREAD) GOTO 99
              IA=IE
              CALL READSI(LINE,IA,IE,GLOC_MAXIT,ERREAD)
              IF(ERREAD) GOTO 99
            ENDIF
          ELSE IF(INDEX(LINE,'ORTHO').NE.0) THEN
             GLOC_CONST = 2
          ELSE IF(INDEX(LINE,'APPROX').NE.0) THEN
            GLOC_CONST = 3
          ENDIF
          GOTO 10
        ELSE IF (INDEX(LINE,'STEP').NE. 0 .AND.
     *           INDEX(LINE,'TUNING').NE.0) THEN
          TG_LINESEARCH = .TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'G_ANTISYM').NE.0) THEN
          TG_ANTISYMM = .TRUE.
          IF (INDEX(LINE,'PENALTY').NE.0) TG_PENALTY = .TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'G_KICK').NE.0) THEN
          TG_KICK = .TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'G_COMPLEX').NE.0) THEN
          TG_COMPLEX = .TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'G_REAL').NE.0) THEN
          TG_REAL = .TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'READ') .NE.0 .AND.
     *           INDEX(LINE,'MATRIX').NE.0) THEN
          TG_READ_MATRIX = .TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'G_STEP').NE.0 .AND.
     *           INDEX(LINE,'TUNE').NE.0) THEN
          TG_TUNE_ST = .TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'GLOC').NE.0 .AND.
     *           INDEX(LINE,'WFNOUT').NE.0) THEN
          TGLOCPRINT = .TRUE.
          IF(INDEX(LINE,'REAL').NE.0) TGLOCREALP = .true.
          IF(INDEX(LINE,'ALL').NE.0) GLOC_ALL=1
          LINEOLD=LINE
          IF(INDEX(LINE,'PART').NE.0) THEN
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            IA=1
            CALL READSI(LINE,IA,IE,GLOC_FIRST,ERREAD)
            IF(ERREAD) GOTO 99
            IA=IE
            CALL READSI(LINE,IA,IE,GLOC_LAST,ERREAD)
            IF(ERREAD) GOTO 99
          ENDIF
          IF(INDEX(LINEOLD,'LIST').NE.0) THEN
            READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
            CALL READSI(LINE,1,IE,GLOC_ORB,ERREAD)
            IF(ERREAD) GOTO 99
            CALL MEMORY(IP_GLOCLIST,GLOC_ORB,'GLOCLIST')
C           We can use more than one line
            READ(IUNIT,ERR=99,END=99,FMT=*) (GLOC_LIST(I),I=1,GLOC_ORB)
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'NOGEOCHECK').NE.0) THEN
C..Dont check geometry for close atoms
          TNOGEOCHECK=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'BROKEN').NE.0) THEN
C..Use Noodleman (broken symmetry) method for spin coupled systems
          BSYMM=.TRUE.
          GOTO 10
C..Distributed Linear Algebra
        ELSE IF(INDEX(LINE,'LINALG').NE.0 .AND.
     &          INDEX(LINE,'DIST').NE.0) THEN
          IF(INDEX(LINE,'ON').NE.0) THEN
            TDMAL=.TRUE.
          ELSEIF(INDEX(LINE,'OFF').NE.0) THEN
            TDMAL=.FALSE.
          ENDIF
          GOTO 10
C..Block size of states distribution
        ELSE IF(INDEX(LINE,'BLOCK').NE.0 .AND.
     &          INDEX(LINE,'STATES').NE.0) THEN
          READ(IUNIT,ERR=99,END=99,FMT=LINEFOR) LINE
          CALL READSI(LINE,1,IE,NSTBLK,ERREAD)
          IF(ERREAD) GOTO 99
          GOTO 10
        ELSE IF(INDEX(LINE,'ALLTOALL').NE.0) THEN
C..Use single/double precision in ALLTOALL communicaton
          IF(INDEX(LINE,'SINGLE').NE.0) TR4A2A=.TRUE.
          IF(INDEX(LINE,'DOUBLE').NE.0) TR4A2A=.FALSE.
          GOTO 10
        ELSE IF(INDEX(LINE,'GSHELL').NE.0) THEN
C..Write file with GSHELL information
          GSHELLOUT=.TRUE.
          GOTO 10
C..Use FFTW wisdom
        ELSE IF(INDEX(LINE,'FFT').NE.0 .AND.
     &          INDEX(LINE,'WISDO').NE.0) THEN
          IF(INDEX(LINE,'OFF').NE.0) THEN
            TFFTWSDM=.FALSE.
          ELSE
            TFFTWSDM=.TRUE.
          ENDIF
          GOTO 10
C..Empirical VDW correction
        ELSE IF(INDEX(LINE,'VDW').NE.0.AND.INDEX(LINE,'CORR').NE.0)THEN
          IF(INDEX(LINE,'OFF').NE.0) THEN
            VDWC=.FALSE.
          ELSE
            VDWC=.TRUE.
          ENDIF
          GOTO 10
C..Printing local potential
        ELSE IF (INDEX(LINE,'LOCAL').NE.0 .AND.
     .           INDEX(LINE,'POTENTIAL').NE.0) THEN
          TLPOT=.TRUE.  ! cmb-kk -Printing local potential
          GOTO 10
        ELSE 
C..Unknown keyword
          IF(' '.NE.LINE) THEN
            IF(IUNK.LT.MAXUNKNOWN) THEN
              IUNK=IUNK+1
              UNKNOWN(IUNK)=LINE
            ELSE
              DO I=1,MAXUNKNOWN-1
                UNKNOWN(I)=UNKNOWN(I+1)
              ENDDO
              UNKNOWN(IUNK)=LINE
            ENDIF
          ENDIF
          GOTO 10
        ENDIF
C     ==--------------------------------------------------------------==
   99 CONTINUE
      WRITE(6,*) ' CONTROL| ERROR IN READING INPUT FILE'
      WRITE(6,*) ' THE LAST TWO LINES READ WERE '
      CALL XSTRING(LINEOLD,IA,IE)
      WRITE(6,*) LINEOLD(1:IE)
      CALL XSTRING(LINE,   IA,IE)
      WRITE(6,*) LINE(1:IE)
      CALL STOPGM('CONTROL',' ')
  200 CONTINUE
      WRITE(6,*) ' CONTROL| INPUT FILE NOT FOUND'
      CALL XSTRING(FILENAME,IA,IE)
      WRITE(6,*) FILENAME(1:IE)
      CALL STOPGM('CONTROL',' ')
C     ==--------------------------------------------------------------==
   20 CONTINUE
      CALL XSTRING(FPATH,IAPATH,IEPATH)
      IF(FPATH(IEPATH:IEPATH).NE.'/') THEN
        IEPATH=IEPATH+1
        FPATH(IEPATH:IEPATH)='/'
      ENDIF
      CALL HEADER(FILENAME)
      IF (INFOLINES.GT.0) THEN
        WRITE(6,'(/,1X,78("*"))') 
        WRITE(6,'(" * ",10("INFO - "),"INFO *")')
        WRITE(6,'(1X,78("*"))') 
        DO I=1,INFOLINES,1
          WRITE(6,'(" * ",A74," *")') INFOMSG(I)
        ENDDO
        WRITE(6,'(1X,78("*"))') 
      ENDIF
C     ==--------------------------------------------------------------==
      IF (MIRROR) THEN
        REWIND(IUNIT)
        WRITE(6,'(/,1X,78("*"))')
        WRITE(6,'(" ** ",T33,A,T77," ** ")') ' INPUT FILE '
        WRITE(6,'(1X,78("*"))')
        DO I=1,100000
          READ(IUNIT,'(A80)',END=112) line
          WRITE(6,'(A4,A72,A4)')' ** ',line,' ** '
        ENDDO
  112   CONTINUE
        WRITE(6,'(1X,78("*"))')
        WRITE(6,'(1X,78("*"))')
        REWIND(IUNIT)
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  Test of options                                             ==
C     ==--------------------------------------------------------------==
      CALL CONTROL_TEST(TSRHO)
C     ==--------------------------------------------------------------==
C     ==  WRITE INFO TO OUTPUT                                        ==
C     ==--------------------------------------------------------------==
      CALL CONTROL_PRI(UNKNOWN,IUNK,FFORMAT)
C     ==--------------------------------------------------------------==
 9999 CONTINUE
#ifdef PARALLEL
      CALL CONTROL_BCAST
#endif
C     ==--------------------------------------------------------------==
#if defined(FFT_FFTW) || defined(FFT_FFTW2) || defined(FFT_FFTW3)
C     ==--------------------------------------------------------------==
C     ==  Read FFTW wisdom, if available and requested                ==
C     ==--------------------------------------------------------------==
      IF (TFFTWSDM) THEN
        IERR=0
        CALL fftw_f77_import_wisdom_from_file(FPATH(IAPATH:IEPATH)//
     &                                        'FFTW_WISDOM',IERR)
#ifdef PARALLEL
        IA=8/IRAT
        CALL MY_COMBINE(IERR,IOUT,IA,2,ALLGRP)
#else
        IOUT=IERR
#endif
        IF(IOUT.EQ.0) THEN
          IF(PARENT) WRITE(6,*) 'CONTROL| FFTW_WISDOM ' //
     &        'SUCCESSFULLY IMPORTED'
        ELSE
          IF(PARENT) WRITE(6,*) 'WARNING! FFTW_WISDOM IMPORT ' //
     &        'FAILED SOMEWHERE. FORGETTING...'
          CALL fftw_f77_forget_wisdom
        ENDIF
      ENDIF
#endif
C     ==--------------------------------------------------------------==
C     NSPLP is broadcasted (CONTROL_BCAST).
C     Set MMAXX (maximum spline points). +20 to have NADD (setspline).
C     999 was the old value (cpmd 3.0)
      MMAXX=MAX(NSPLP+20,999)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
