C     ==================================================================
      SUBROUTINE CORE_SPECTRA(TAU0,C0,WE,F,NSTATE,NKPOINT)
C     ==--------------------------------------------------------------==
C     == Compute the X-ray core spectra within the framework of       ==
C     == M. Cavalleri et al. J. Chem. Phys. 121, 10065 (2004)         ==
C     == Present release: Tokai-mura/Tsukuba, 14 June 2005            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'adat.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'sphe.inc'
      INCLUDE 'qspl.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'cores.inc'
      INCLUDE 'fileopen.inc'
C     Arguments 
      INTEGER    NSTATE,NKPOINT
      COMPLEX*16 C0(NGWK,NSTATE,NKPOINT)
      REAL*8     TAU0(3,NAX,NSX),WE(NSTATE,NKPOINT),F(NSTATE,NKPOINT)
      DIMENSION  RK(3,NKPOINT),WK(NKPOINT)
C     Parameters
      REAL*8 DLN2
C     Variables 
      DIMENSION  CORE_ATWFR(MMAXX),CORE_CAT(NSPLPO,2),CORE_C0(NGWK,*)
      DIMENSION  ATRG(MMAXX,NSX),GGNG(NSPLPO),GGNH(NSPLPO),EIGR(NGW,NAT)
      COMPLEX*16 WORK(2*MMAXX,5)
      REAL*8     FINT(2*MMAXX),BSINT(2*MMAXX),GG(2*MMAXX),TEMP(MMAXX,3)
      POINTER    (IP_WORK,WORK),
     &           (IP_FINT,FINT),(IP_BSINT,BSINT),(IP_GG,GG),
     &           (IP_TEMP,TEMP)
      COMPLEX*16 CI,CAUX,CSUMX,CSUMY,CSUMZ
      REAL*8     RYBY2,XMAX,DISC,RMIN,GMIN,
     &           VOL,CC,CURV2,YLMR,SUMX,SUMY,SUMZ,AIAUX
      REAL*8     SUMX2(9),SUMY2(9),SUMZ2(9),SUM2(9),FNL(*),DFNL(*)
#ifdef PARALLEL
      REAL*8     CSRI(2)
#endif
      LOGICAL    SAVED,FERROR
      INTEGER    IL,IR,IV,IK,I,IG,K
      INTEGER    IAT,IA,IS,MMAX,NWORK,N2,N22,L,LL,IERR,LY,
     &           LPP(5)
      DATA       LPP /0,1,4,9,16/
      CHARACTER*1 CANG(5)
      DATA        CANG /'S','P','D','F','G'/
C     ==--------------------------------------------------------------==
      RYBY2=2.D0*RY
      DLN2=1.D0/DLOG(2.D0)
      IAT=CORE_ATOM
      IA=IATPT(1,IAT)
      IS=IATPT(2,IAT)
      L=CORE_LSHELL
      LL=2*L+1
      NWORK=2*MMAXX
C..Allocation of local arrays
      CALL MEMORY(IP_WORK,10*NWORK,'WORK')
      CALL MEMORY(IP_FINT,NWORK,'WORK')
      CALL MEMORY(IP_BSINT,NWORK,'BSINT')
      CALL MEMORY(IP_GG,NWORK,'GG')
      CALL MEMORY(IP_TEMP,3*MMAXX,'MMAXX')
      CALL MEMORY(IP_CORE_ATWFR,MMAXX,'CORE_ATWFR')
      CALL MEMORY(IP_CORE_CAT,2*NSPLPO,'CORE_CAT')
      CALL MEMORY(IP_CORE_C0,2*NGWK*LL,'CORE_C0')
C..Construct core AO 
      CALL STORPS(ATRG(1,IS),CORE_ATWFR,MESHAT(IS),
     &            CORE_NQSTO,L,CORE_STOEXP)
C..Transform core AO in R-space to G-space
      CALL AZZERO(CORE_CAT,NSPLPO*2)
      MMAX=MESHAT(IS)
      XMAX=DBLE(MMAX)
      N2=NINT(DLN2*DLOG(XMAX)+0.499999D0)
      N22=2**N2
      RMIN=DLOG(ATRG(1,IS))
      GMIN=DLOG(DSQRT(GCUTW+GCUTKA)*TPIBA)-(MMAX-1)*CLOGAT(IS)
!$OMP parallel do private(IL)
#ifdef __SR8000
*poption parallel
#endif
      DO IL=1,MMAX
        GG(IL)=(DEXP(GMIN+(IL-1)*CLOGAT(IS))/TPIBA)**2
      ENDDO
      CALL AZZERO(FINT,N22)
!$OMP parallel do private(IR)
#ifdef __SR8000
*poption parallel
#endif
      DO IR=1,MMAX
        FINT(IR)=FPI*CORE_ATWFR(IR)/ATRG(IR,IS)
      ENDDO
C..Fourier transformation
      SAVED=.FALSE. 
      CALL LSFBTR(FINT,BSINT,L,RMIN,GMIN,CLOGAT(IS),N2,SAVED,
     .     WORK(1,1),WORK(1,2),WORK(1,3),WORK(1,5),NWORK,DISC)
      CALL TGRID(GG,MMAX,GGNG,NSPLPO,BSINT,MMAXX,
     .           TEMP(1,1),TEMP(1,2),TEMP(1,3))
      CALL DCOPY(NSPLPO,BSINT(1),1,CORE_CAT(1,1),1)
      IF(L.GT.0.AND.GGNG(1).LT.1.D-12) CORE_CAT(1,1)=0.0D0
      CALL CURV1(NSPLPO,GGNG,CORE_CAT(1,1),0.0D0,0.0D0,3,
     .           CORE_CAT(1,2),TEMP,0.0D0,IERR)
C..Load core AO in plane wave basis
      VOL=1.D0/DSQRT(OMEGA)
      CI=(0.0D0,1.0D0)**L
      CALL AZZERO(CORE_C0(1,1),2*NGWK*LL)
      DO IV=1,LL
        LY=LPP(L+1)+IV
        DO IG=1,NGW
          CC=CURV2(HG(IG),NSPLPO,GGNG(1),CORE_CAT(1,1),
     *             CORE_CAT(1,2),0.0D0)*VOL
          CORE_C0(IG,IV)=CI*YLMR(LY,IG,GK(1,1))*CC*EIGR(IG,IAT)
        ENDDO 
      ENDDO 
      IF(PARENT) THEN 
        IF(TKPNT) THEN
          WRITE(6,'(/,A,A)')'CORE_SPECTRA:',' K-POINTS CALCULATION'
        ELSE
          WRITE(6,'(/,A,A)')'CORE_SPECTRA:',' GAMMA-POINT CALCULATION'
        ENDIF
        WRITE(6,'(/,A)') ' EXCITED ATOM:'
        WRITE(6,'(A,A)') 
     .    '    NR   TYPE        X(bohr)        Y(bohr)        Z(bohr)'
        WRITE(6,'(X,I5,5X,A2,3F15.6)') 
     .    IAT,EL(IATYP(IS)),(TAU0(K,IA,IS),K=1,3)
        WRITE(6,'(A)') '    EXCITED STATE:'
        WRITE(6,'(A)') 
     .    '                      n   l          Ec[Ha]         Ec[eV]'
        WRITE(6,'(22X,I1,3X,A,X,2F15.6,/)') 
     .    CORE_NQSTO,CANG(L+1),CORE_LEVEL,RYBY2*CORE_LEVEL
        WRITE(6,'(A,A)') 
     .    '  k    i   Ei[eV]  Ei-Ec[eV]   fi  ',
     .    ' |<i|px|c>|**2  |<i|py|c>|**2  |<i|pz|c>|**2'
cmb-> write on file 60
        CALL FILEOPEN(60,'XRAYSPEC.DAT',FO_DEF,FERROR)
        IF(TKPNT) THEN
          WRITE(60,'(A,A)')'CORE_SPECTRA:',' K-POINTS CALCULATION'
        ELSE
          WRITE(60,'(A,A)')'CORE_SPECTRA:',' GAMMA-POINT CALCULATION'
        ENDIF
        WRITE(60,'(/,A)') ' EXCITED ATOM:'
        WRITE(60,'(A,A)') 
     .    '    NR   TYPE        X(bohr)        Y(bohr)        Z(bohr)'
        WRITE(60,'(X,I5,5X,A2,3F15.6)') 
     .    IAT,EL(IATYP(IS)),(TAU0(K,IA,IS),K=1,3)
        WRITE(60,'(A)') '    EXCITED STATE:'
        WRITE(60,'(A)') 
     .    '                      n   l          Ec[Ha]         Ec[eV]'
        WRITE(60,'(22X,I1,3X,A,X,2F15.6,/)') 
     .    CORE_NQSTO,CANG(L+1),CORE_LEVEL,RYBY2*CORE_LEVEL
        WRITE(60,'(A,A)') 
     .    '  k    i   Ei[eV]  Ei-Ec[eV]   fi  ',
     .    ' |<i|px|c>|**2  |<i|py|c>|**2  |<i|pz|c>|**2  |<i|p|c>|**2'
      ENDIF 
C..Compute the transition matrix 
      IF(TKPNT) THEN ! K-points
        DO IK=1,NKPOINT
          DO I=1,NSTATE 
            DO IV=1,LL
              CSUMX=(0.0D0,0.0D0)
              CSUMY=(0.0D0,0.0D0)
              CSUMZ=(0.0D0,0.0D0)
cmb-> first G > 0 with the k-points
#ifdef __SR8000
*poption parallel
#endif
!$OMP parallel do private(IG,CAUX) reduction(+:CSUMX,CSUMY,CSUMZ)
                DO IG=1,NGW
                  CAUX=DCONJG(C0(IG,I,IK))*CORE_C0(IG,IV)
                  CSUMX=CSUMX+(RK(1,IK)+GK(1,IG))*CAUX
                  CSUMY=CSUMY+(RK(2,IK)+GK(2,IG))*CAUX
                  CSUMZ=CSUMZ+(RK(3,IK)+GK(3,IG))*CAUX
                ENDDO
cmb-> then G < 0 with the k-points
#ifdef __SR8000
*poption parallel
#endif
!$OMP parallel do private(IG,CAUX) reduction(+:CSUMX,CSUMY,CSUMZ)
                DO IG=1,NGW
                  CAUX=DCONJG(C0(IG+NGW,I,IK))*CORE_C0(IG,IV)
                  CSUMX=CSUMX+(RK(1,IK)-GK(1,IG))*CAUX
                  CSUMY=CSUMY+(RK(2,IK)-GK(2,IG))*CAUX
                  CSUMZ=CSUMZ+(RK(3,IK)-GK(3,IG))*CAUX
                ENDDO
#ifdef PARALLEL
                CSRI(1)=DREAL(CSUMX)
                CSRI(2)=DIMAG(CSUMX)
                CALL GLOSUM(2,CSRI)
                CSUMX=DCMPLX(CSRI(1),CSRI(2))
                CSRI(1)=DREAL(CSUMY)
                CSRI(2)=DIMAG(CSUMY)
                CALL GLOSUM(2,CSRI)
                CSUMY=DCMPLX(CSRI(1),CSRI(2))
                CSRI(1)=DREAL(CSUMZ)
                CSRI(2)=DIMAG(CSUMZ)
                CALL GLOSUM(2,CSRI)
                CSUMZ=DCMPLX(CSRI(1),CSRI(2))
#endif
                SUMX2(IV)=DREAL(DCONJG(CSUMX)*CSUMX)
                SUMY2(IV)=DREAL(DCONJG(CSUMY)*CSUMY)
                SUMZ2(IV)=DREAL(DCONJG(CSUMZ)*CSUMZ)
                SUM2(IV)=SUMX2(IV)+SUMY2(IV)+SUMZ2(IV) 
            ENDDO 
            IF(PARENT) THEN
              WRITE(6,'(I3,I5,F9.3,F11.3,F6.2,27E15.7)')
     .        IK,I,RYBY2*WE(I,IK),RYBY2*(WE(I,IK)-CORE_LEVEL),F(I,IK),
     .        (SUMX2(IV),SUMY2(IV),SUMZ2(IV),IV=1,LL)
              WRITE(60,'(I3,I5,F9.3,F11.3,F6.2,36E15.7)')
     .        IK,I,RYBY2*WE(I,IK),RYBY2*(WE(I,IK)-CORE_LEVEL),F(I,IK),
     .        (SUMX2(IV),SUMY2(IV),SUMZ2(IV),SUM2(IV),IV=1,LL)
            ENDIF
          ENDDO 
        ENDDO 
      ELSE  ! Gamma-point only
      IK=1
      DO I=1,NSTATE 
        DO IV=1,LL
          SUMX=0.0D0
          SUMY=0.0D0
          SUMZ=0.0D0
#ifdef __SR8000
*poption parallel
#endif
!$OMP parallel do private(IG,AIAUX) reduction(+:SUMX,SUMY,SUMZ)
          DO IG=1,NGW 
            AIAUX=DIMAG(DCONJG(C0(IG,I,IK))*CORE_C0(IG,IV))
            SUMX=SUMX+GK(1,IG)*AIAUX
            SUMY=SUMY+GK(2,IG)*AIAUX
            SUMZ=SUMZ+GK(3,IG)*AIAUX
          ENDDO 
#ifdef PARALLEL 
          CALL GLOSUM(1,SUMX)
          CALL GLOSUM(1,SUMY)
          CALL GLOSUM(1,SUMZ)
#endif 
          SUMX2(IV)=4.0D0*SUMX*SUMX
          SUMY2(IV)=4.0D0*SUMY*SUMY
          SUMZ2(IV)=4.0D0*SUMZ*SUMZ
          SUM2(IV)=SUMX2(IV)+SUMY2(IV)+SUMZ2(IV)
        ENDDO 
        IF(PARENT) THEN 
          WRITE(6,'(I3,I5,F9.3,F11.3,F6.2,27E15.7)') 
     .      IK,I,RYBY2*WE(I,IK),RYBY2*(WE(I,IK)-CORE_LEVEL),F(I,IK),
     .      (SUMX2(IV),SUMY2(IV),SUMZ2(IV),IV=1,LL) 
          WRITE(60,'(I3,I5,F9.3,F11.3,F6.2,36E15.7)') 
     .      IK,I,RYBY2*WE(I,IK),RYBY2*(WE(I,IK)-CORE_LEVEL),F(I,IK),
     .      (SUMX2(IV),SUMY2(IV),SUMZ2(IV),SUM2(IV),IV=1,LL) 
        ENDIF 
      ENDDO 
      ENDIF
C     ==--------------------------------------------------------------==
C     Deallocation of local arrays
      CALL FREEM(IP_WORK)
      CALL FREEM(IP_FINT)
      CALL FREEM(IP_BSINT)
      CALL FREEM(IP_GG)
      CALL FREEM(IP_TEMP)
      CALL FREEM(IP_CORE_ATWFR)
      CALL FREEM(IP_CORE_CAT)
      CALL FREEM(IP_CORE_C0)
C     ==--------------------------------------------------------------==
      IF(PARENT) THEN
        WRITE(6,*) 
        CALL FILECLOSE(60)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN 
      END
C     ==================================================================
