C     ==================================================================
      SUBROUTINE COREC(RHOE,VTMP,V)
C     ==--------------------------------------------------------------==
C     == Calculates Core charges in G-space and total one in R space  ==
C     == RHOE:  in valence charge densities in real space             ==
C     ==       out total charge densities in real space               ==
C     == VTMP: out core charges in G space                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      INCLUDE 'spin.inc'
      include 'ions.inc'
      include 'nlcc.inc'
      include 'sfac.inc'
      include 'cppt.inc'
      include 'geq0.inc'
      include 'fft.inc'
C     Arguments
      REAL*8     RHOE(NNR1,NLSD)
      COMPLEX*16 VTMP(NHG),V(NNR1)
C     Variables
      DIMENSION  RHOC(NHG,NSP),EIGRB(NHG,NAT)
      COMPLEX*16 EI123
      REAL*8     FNL(*),DFNL(*),VVRE
      INTEGER    ISUB,ISA0,IS,IA,ISA,IG,IR
C     ==--------------------------------------------------------------==
      CALL TISET('     COREC',ISUB)
C     SUM CORE CHARGES IN G-SPACE
      CALL AZZERO(VTMP,2*NHG)
      IF(BIGMEM) THEN
        ISA0=0
        DO IS=1,NSP
          IF(TNLCC(IS)) THEN
            DO IA=1,NA(IS)
              ISA=ISA0+IA
#ifdef __SR8000
*poption parallel, tlocal(IG)
#endif 
              DO IG=1,NHG
                VTMP(IG)=VTMP(IG)+RHOC(IG,IS)*EIGRB(IG,ISA)
              ENDDO
            ENDDO
          ENDIF
          ISA0=ISA0+NA(IS)
        ENDDO
      ELSE
        ISA0=0
        DO IS=1,NSP
          IF(TNLCC(IS)) THEN
            DO IA=1,NA(IS)
              ISA=ISA0+IA
#ifdef __SR8000
*poption parallel, tlocal(IG,EI123)
#endif 
              DO IG=1,NHG
                EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     *                EI3(ISA,INYH(3,IG))
                VTMP(IG)=VTMP(IG)+RHOC(IG,IS)*EI123
              ENDDO
            ENDDO
          ENDIF
          ISA0=ISA0+NA(IS)
        ENDDO
      ENDIF
C     FFT TO R-SPACE
      CALL AZZERO(V,2*MAXFFT)
#ifdef __SR8000
*poption parallel, tlocal(IG)
#endif 
      DO IG=1,NHG
        V(NZH(IG))=VTMP(IG)
        V(INDZ(IG))=DCONJG(VTMP(IG))
      ENDDO
      IF(GEQ0) V(NZH(1))=VTMP(1)
      CALL INVFFT(V)
C     ADD UP THE TOTAL CHARGE IN R-SPACE
      IF(TLSD) THEN
C       Remember: RHOE(*,1) is the total charge,
C                 RHOE(*,2) the beta charge
!$OMP  parallel do private(IR,VVRE)
        DO IR=1,NNR1
          VVRE=DREAL(V(IR))
          RHOE(IR,1)=RHOE(IR,1)+VVRE
          RHOE(IR,2)=RHOE(IR,2)+0.5D0*VVRE
        ENDDO
      ELSE
!$OMP  parallel do private(IR)
        DO IR=1,NNR1
          RHOE(IR,1)=RHOE(IR,1)+DREAL(V(IR))
        ENDDO
      ENDIF
      CALL TIHALT('     COREC',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
