C     ==================================================================
      SUBROUTINE DAVIDSON(NDIAG,NADD,C0,CS,CR,SC0,CSCR,VPOT,
     *                    CGS,FOCC,NSTATE,
     *                    PSI,EDAV,VPP,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     ==  NDIAG      NUMBER OF STATES                                 ==
C     ==  NADD       DIMENSION OF DAVIDSON MATRIX                     ==
C     ==  C0         WAVEFUNCTIONS                                    ==
C     ==  CS         SIGMA VECTORS                                    ==
C     ==  CR         RESIDUALS                                        ==
C     ==  SC0        S*C0 (only Vanderbilt)                           ==
C     ==  CSCR       SCRATCH                                          ==
C     ==  EDAV       EIGENVALUES OF ADAV                              ==
C     ==--------------------------------------------------------------==
C     ==  SPECIAL INPUT NEEDED FOR HARTREE-FOCK EXCHANGE              ==
C     ==  CGS        GROUND STATE ORBITALS -> DEFINING VPOT           ==
C     ==  FOCC       OCCUPATION NUMBERS OF CGS                        ==
C     ==  NSTATE     NUMBER OF CGS ORBITALS                           ==
C     ==--------------------------------------------------------------==
C     ==  ADAV       DAVIDSON MATRIX                                  ==
C     ==  UDAV       EIGENVECTORS OF ADAV                             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'soft.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'irat.inc'        !IRAT
C     Arguments
      INTEGER    NDIAG,NADD,LSCR,NSTATE
      COMPLEX*16 C0(NGW,NADD),CS(NGW,NADD),SC0(NGW,*),
     &           CR(NGW,NADD),CSCR(NGW,NADD),PSI(NNR1),CGS(NGW,NSTATE)
      REAL*8     VPOT(NNR1),EDAV(NADD),VPP(NGW),SCR(LSCR),FOCC(NSTATE)
C     Variables
      INTEGER    ISPIN
      PARAMETER  (ISPIN=1)
      INTEGER    INDEX(NDIAG)
      REAL*8     ADAV(NADD*(NADD+1)/2),UDAV(NADD,NADD),SC2(*)
      POINTER    (IP_INDEX,INDEX),(IP_ADAV,ADAV),(IP_UDAV,UDAV)
      POINTER    (IP_SC2,SC2)
      LOGICAL    TLSD2,DEBUG
      CHARACTER  TAG*30
      REAL*8     TIME1,EIMAX,ERMAX,TCPU,E1Z,E1N,E1,SIGN
      REAL*8     DOTP,TIMEF
      EXTERNAL   DOTP,TIMEF
      INTEGER    ISUB,IOPT,IR,IG,K,I,J,NCURR,NDIMA,NCONV,ICYCLE,
     &           IXABS,IPRIE,NNEW,IS,NROT2,LDAVIDSON,LSCR2,
     &           IZAMAX
C     ==--------------------------------------------------------------==
      IF(NDIAG.EQ.0) RETURN
C     ==--------------------------------------------------------------==
      CALL TISET('  DAVIDSON',ISUB)
C     Test SCR
      CALL GIVE_SCR_DAVIDSON(LDAVIDSON,TAG,NDIAG,NADD)
      CALL TEST_SCR('DAVIDSON',TAG,LSCR,LDAVIDSON)
      DEBUG=.FALSE.
      CALL SCRSET(SCR,LSCR,DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_INDEX,NDIAG,'INDEX',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_ADAV,(NADD*(NADD+1))/2,'ADAV',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_UDAV,NADD*NADD,'UDAV',DEBUG)
      CALL SCRGET(SCR,LSCR,IP_SC2,LSCR2,"SC2",DEBUG)
C     ==--------------------------------------------------------------==
      SIGN=1.D0
      IF(TLSD) SIGN=2.D0
      TLSD2=TLSD
      TLSD=.FALSE.
      TIME1=TIMEF()
C     ==--------------------------------------------------------------==
C     ==  SHIFT POTENTIAL                                             ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        VPOT(IR)=VPOT(IR)+DSHIFT
      ENDDO
!$OMP parallel do private(IG)
      DO IG=1,NGW
        VPP(IG)=VPP(IG)+DSHIFT
      ENDDO
      IF(TIVAN)
     &     CALL DAXPY(NHXS*NHXS*NSX,DSHIFT,QQ(1,1,1),1,DVAN(1,1,1),1)
C     ==--------------------------------------------------------------==
C     ==  INITIALIZE SIGMA VECTORS                                    ==
C     ==--------------------------------------------------------------==
      CALL HPSI(C0,CS,SC0,VPOT,PSI,SC2,LSCR2,NDIAG,1,ISPIN)
      CALL HFXPSI(CGS,C0,CS,FOCC,SIGN,PSI,NSTATE,NDIAG)
C     ==--------------------------------------------------------------==
C     ==  DAVIDSON MATRIX                                             ==
C     ==--------------------------------------------------------------==
      IOPT=1
      K=0
      DO J=1,NDIAG
        DO I=J,NDIAG
          K=K+1
          ADAV(K)=DOTP(NGW,C0(1,I),CS(1,J))
        ENDDO
      ENDDO
#ifdef PARALLEL
      CALL GLOSUM((NDIAG*(NDIAG+1))/2,ADAV)
#endif
      CALL DSPEVY(IOPT,ADAV,EDAV,UDAV,NADD,NDIAG,SC2,LSCR2)
C     ==--------------------------------------------------------------==
C     ==  RESIDUALS                                                   ==
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(CR,NGW*NDIAG)
      IF(TIVAN) THEN  !cmb- avoid IF inside loop for good vectorization
        DO I=1,NDIAG
          DO J=1,NDIAG
            DO IG=1,NGW
              CR(IG,I)=CR(IG,I)+(CS(IG,J)-SC0(IG,J)*EDAV(I))*UDAV(J,I)
            ENDDO
          ENDDO
        ENDDO
      ELSE
        DO I=1,NDIAG
          DO J=1,NDIAG
            DO IG=1,NGW
              CR(IG,I)=CR(IG,I)+(CS(IG,J)-C0(IG,J)*EDAV(I))*UDAV(J,I)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  ROTATE ORBITALS AND SIGMA VECTORS                           ==
C     ==--------------------------------------------------------------==
      CALL DGEMM('N','N',2*NGW,NDIAG,NDIAG,1.0D0,C0(1,1),2*NGW,
     &           UDAV(1,1),NADD,0.0D0,CSCR(1,1),2*NGW)
      CALL DCOPY(2*NGW*NDIAG,CSCR(1,1),1,C0(1,1),1)
      CALL DGEMM('N','N',2*NGW,NDIAG,NDIAG,1.0D0,CS(1,1),2*NGW,
     &           UDAV(1,1),NADD,0.0D0,CSCR(1,1),2*NGW)
      CALL DCOPY(2*NGW*NDIAG,CSCR(1,1),1,CS(1,1),1)
      IF(TIVAN) THEN
        CALL DGEMM('N','N',2*NGW,NDIAG,NDIAG,1.0D0,SC0(1,1),2*NGW,
     &           UDAV,NADD,0.0D0,CSCR(1,1),2*NGW)
        CALL DCOPY(2*NGW*NDIAG,CSCR(1,1),1,SC0(1,1),1)
      ENDIF
      NCURR=NDIAG
      NDIMA=NDIAG
      NCONV=0
C     ==--------------------------------------------------------------==
C     ==          THE BASIC LOOP FOR DAVIDSON DIAGONALIZATION         ==
C     ==--------------------------------------------------------------==
      IF(PARENT)
     *WRITE(6,'(/,A,A)') ' ITER  STATES  SUBSPACE    STATE     ENERGY ',
     *               '    RESIDUAL     TCPU  '
      DO ICYCLE=0,N_FRIES
        DO I=1,NDIAG
          IXABS=IZAMAX(NGW,CR(1,I),1)
          EIMAX=CDABS(CR(IXABS,I))
#ifdef PARALLEL
          CALL GLOMAX(1,EIMAX)
#endif
          IF(EIMAX.GT.EPSDAV) THEN
            NCONV=I-1
            GOTO 200
          ENDIF
        ENDDO
        NCONV=NDIAG
  200   CONTINUE
        IXABS=IZAMAX(NGW*NDIAG,CR,1)
        ERMAX=CDABS(CR(IXABS,1))
#ifdef PARALLEL
        CALL GLOMAX(1,ERMAX)
#endif
        TCPU=(TIMEF()-TIME1)*0.001D0
        IPRIE=NCONV+1
        IF(NCONV.EQ.NDIAG) IPRIE=NDIAG
        IF(PARENT)
     *  WRITE(6,'(I5,3X,I4,4X,I4,7X,I4,F13.6,2X,E10.2,F9.2)') 
     *     ICYCLE,NCONV,NDIMA,IPRIE,EDAV(IPRIE)-DSHIFT,ERMAX,TCPU
        TIME1=TIMEF()
        IF(.NOT.EXSOFT) CALL TESTEX(EXSOFT)
        IF(NCONV.EQ.NDIAG .OR. EXSOFT) GOTO 100
C     ==--------------------------------------------------------------==
C     ==  ADD NEW VECTORS TO THE SUBSPACE                             ==
C     ==--------------------------------------------------------------==
        NNEW=NDIAG-NCONV
        NNEW=MIN(NNEW,NADD-NCURR)
        NDIMA=NCURR+NNEW-NCONV
        DO I=NCURR+1,NCURR+NNEW
          IS=NCONV+I-NCURR
          DO IG=1,NGW
            CSCR(IG,1)=1.0D0/(VPP(IG)-EDAV(IS))
            C0(IG,I)=C0(IG,IS)*CSCR(IG,1)
          ENDDO
          E1Z=DOTP(NGW,C0(1,I),CR(1,IS))
          E1N=DOTP(NGW,C0(1,I),C0(1,IS))
#ifdef PARALLEL
          CALL GLOSUM(1,E1Z)
          CALL GLOSUM(1,E1N)
#endif
          E1=E1Z/E1N
          IF(TIVAN) THEN
!$OMP parallel do private(IG)
            DO IG=1,NGW
              C0(IG,I)=(CR(IG,IS)-E1*SC0(IG,IS))*CSCR(IG,1)
            ENDDO
          ELSE
!$OMP parallel do private(IG)
            DO IG=1,NGW
              C0(IG,I)=(CR(IG,IS)-E1*C0(IG,IS))*CSCR(IG,1)
            ENDDO
          ENDIF
        ENDDO
C     ==--------------------------------------------------------------==
C     ==  ORTHOGONALIZE NEW VECTORS                                   ==
C     ==--------------------------------------------------------------==
        IF(TIVAN) THEN
          CALL RNLSM(C0(1,NCURR+1),NNEW,PSI,SC2,LSCR2,1,1,.FALSE.)
          CALL DCOPY(2*NGW*NNEW,C0(1,NCURR+1),1,SC0(1,NCURR+1),1)
          CALL SPSI(NNEW,SC0(1,NCURR+1),SC2,LSCR2)
          CALL VGSORTHO(C0,SC0,NGW,NCURR+1,NCURR+NNEW)
        ELSE
          CALL GS_ORTHO(C0,NCURR,C0(1,NCURR+1),NNEW,ADAV,SC2)
c         CALL GSORTHO(C0,NGW,NCURR+1,NCURR+NNEW)
        ENDIF
C     ==--------------------------------------------------------------==
C     ==  NEW SIGMA VECTORS                                           ==
C     ==--------------------------------------------------------------==
        CALL HPSI(C0(1,NCURR+1),CS(1,NCURR+1),SC0(1,NCURR+1),
     &            VPOT,PSI,SC2,LSCR2,NNEW,1,ISPIN)
        CALL HFXPSI(CGS,C0(1,NCURR+1),CS(1,NCURR+1),FOCC,SIGN,
     &              PSI,NSTATE,NNEW)
C     ==--------------------------------------------------------------==
C     ==  DAVIDSON MATRIX                                             ==
C     ==--------------------------------------------------------------==
        IOPT=1
        K=0
        DO J=NCONV+1,NCURR+NNEW
          DO I=J,NCURR+NNEW
            K=K+1
            ADAV(K)=DOTP(NGW,C0(1,I),CS(1,J))
          ENDDO
        ENDDO
#ifdef PARALLEL
        CALL GLOSUM(K,ADAV)
#endif
        CALL DSPEVY(IOPT,ADAV,EDAV(NCONV+1),UDAV,NADD,NDIMA,SC2,LSCR2)
C     ==--------------------------------------------------------------==
C     ==  RESIDUALS                                                   ==
C     ==--------------------------------------------------------------==
        CALL ZAZZERO(CR,NGW*NDIAG)
        IF(TIVAN) THEN !cmb- avoid IF inside loop for good vectorization
          DO I=NCONV+1,NDIAG
            DO J=NCONV+1,NCURR+NNEW
              DO IG=1,NGW
                CR(IG,I)=CR(IG,I)+(CS(IG,J)-SC0(IG,J)*EDAV(I))*
     *                             UDAV(J-NCONV,I-NCONV)
              ENDDO
            ENDDO
          ENDDO
        ELSE
          DO I=NCONV+1,NDIAG
            DO J=NCONV+1,NCURR+NNEW
              DO IG=1,NGW
                CR(IG,I)=CR(IG,I)+(CS(IG,J)-C0(IG,J)*EDAV(I))*
     *                             UDAV(J-NCONV,I-NCONV)
              ENDDO
            ENDDO
          ENDDO
        ENDIF
C     ==--------------------------------------------------------------==
C     ==  ROTATE ORBITALS AND SIGMA VECTORS                           ==
C     ==--------------------------------------------------------------==
        NROT2=NDIMA
        IF(NDIMA+NDIAG.GT.NADD) NROT2=NDIAG-NCONV
        CALL DGEMM('N','N',2*NGW,NROT2,NDIMA,1.0D0,C0(1,NCONV+1),
     *             2*NGW,UDAV(1,1),NADD,0.0D0,CSCR(1,1),2*NGW)
        CALL DCOPY(2*NGW*NROT2,CSCR(1,1),1,C0(1,NCONV+1),1)
        CALL DGEMM('N','N',2*NGW,NROT2,NDIMA,1.0D0,CS(1,NCONV+1),
     *             2*NGW,UDAV(1,1),NADD,0.0D0,CSCR(1,1),2*NGW)
        CALL DCOPY(2*NGW*NROT2,CSCR(1,1),1,CS(1,NCONV+1),1)
        IF(TIVAN) THEN
          CALL DGEMM('N','N',2*NGW,NROT2,NDIMA,1.0D0,SC0(1,NCONV+1),
     *               2*NGW,UDAV(1,1),NADD,0.0D0,CSCR(1,1),2*NGW)
          CALL DCOPY(2*NGW*NROT2,CSCR(1,1),1,SC0(1,NCONV+1),1)
        ENDIF
        NCURR=NCONV+NROT2
C     ==--------------------------------------------------------------==
C     ==                      END OF MAIN LOOP                        ==
C     ==--------------------------------------------------------------==
      ENDDO
      IF(PARENT)
     *  WRITE(6,'(A)') ' !!!!!!! NOT ALL ROOTS ARE CONVERGED !!!!!!! '
  100 CONTINUE
C     ==--------------------------------------------------------------==
C     ==  ORDER THE STATES WITH RESPECT TO ENERGY                     ==
C     ==--------------------------------------------------------------==
      CALL SORT2(EDAV,NDIAG,INDEX)
      CALL DCOPY(2*NGW*NDIAG,C0(1,1),1,CS(1,1),1)
      DO I=1,NDIAG
        J=INDEX(I)
        CALL DCOPY(2*NGW,CS(1,J),1,C0(1,I),1)
      ENDDO
      TLSD=TLSD2
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        VPOT(IR)=VPOT(IR)-DSHIFT
      ENDDO
!$OMP parallel do private(IG)
      DO IG=1,NGW
        VPP(IG)=VPP(IG)-DSHIFT
      ENDDO
!$OMP parallel do private(I)
      DO I=1,NDIAG
        EDAV(I)=EDAV(I)-DSHIFT
      ENDDO
      IF(TIVAN)
     &     CALL DAXPY(NHXS*NHXS*NSX,-DSHIFT,QQ(1,1,1),1,DVAN(1,1,1),1)
C
      CALL SCRREL(SCR,LSCR,DEBUG)
      CALL TIHALT('  DAVIDSON',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_DAVIDSON(LDAVIDSON,TAG,NDIAG,NADD)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !pslo.inc
      INCLUDE 'pslo.inc'        !TIVAN
C     Arguments
      INTEGER   LDAVIDSON,NDIAG,NADD
      CHARACTER TAG*30
C     Variables
      INTEGER   LHPSI,LDSPEVY,LRNLSM,LSPSI,LSCR2
C     ==--------------------------------------------------------------==
      LSCR2 =  NDIAG+        !INDEX
     &         NADD*(NADD+1)/2+     !ADAV
     &         NADD*NADD            !UDAV
      CALL GIVE_SCR_HPSI(LHPSI,TAG,NDIAG)
      LDSPEVY=3*NDIAG
      IF(TIVAN) THEN
        CALL GIVE_SCR_RNLSM(LRNLSM,TAG,NDIAG,.FALSE.)
      ELSE
        LRNLSM=0
      ENDIF
      CALL GIVE_SCR_SPSI(LSPSI,TAG)
      LDAVIDSON=LSCR2+MAX(LHPSI,LDSPEVY,LRNLSM,LSPSI)+200
      TAG='LSCR+MAX(HPSI,DIAG,RNLSM,SPSI)'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
