C     ==================================================================
      SUBROUTINE DD_XC(RHOE,DRHOE,RHOX,PSI,DXC,VTMP,VTEMP,GRAD,
     $     SCR,LSCR,SWITCH)
C     ==--------------------------------------------------------------==
C     == Calculate dVxc/dn*n1 by finite differences                   ==
C     ==--------------------------------------------------------------==
C     == INPUT: rhoe = density                                        ==
C     ==        drhoe = LR density                                    ==
C     ==        rhox = undef.                                         ==
C     ==        psi = undef.                                          ==
C     ==        dxc = undef.                                          ==
C     ==        vtmp = undef.                                         ==
C     ==        vtemp = undef.                                        ==
C     ==        grad = undef.                                         ==
C     == OUTPUT: rhoe = density                                       ==
C     ==        drhoe = LR density                                    ==
C     ==        rhox = undef.                                         ==
C     ==        psi = undef.                                          ==
C     ==        dxc = dVxc/dn*n1                                      ==
C     ==        vtmp = potential in G space                           ==
C     ==        vtemp = undef.                                        ==
C     ==        grad = undef.                                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'linres.inc'
      REAL*8     RHOE(NNR1,*),DRHOE(NNR1,*),RHOX(NNR1,*),DXC(NNR1,*),
     *           GRAD(*),SCR(*)
      COMPLEX*16 PSI(MAXFFT,*),VTMP(*),VTEMP(*)
      LOGICAL    SWITCH
      INTEGER    ISUB,IR,LSCR
      REAL*8     OEPS
C     ==--------------------------------------------------------------==
      IF(TXC_DD_ANA) THEN
        CALL DD_XC_ANA(RHOE,DRHOE,RHOX,PSI,DXC,VTMP,VTEMP,GRAD,
     $        SCR,LSCR,SWITCH)
        RETURN
      ENDIF
C     ==--------------------------------------------------------------==
      CALL TISET('     DD_XC',ISUB)
C
      IF(SWITCH) CALL SWITCH_FUNCTIONALS
C
      IF(TTAU) THEN
        CALL STOPGM('DD_XC','META FUNCTIONALS NOT IMPLENTED')
      ENDIF
C
      IF(.NOT. TLSD) THEN
C..NON-spin polarised
        IF(NDPOINT.EQ.2) THEN
C..2-point formula
          CALL DCOPY(NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(NNR1,XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DREAL(PSI(IR,1))
          ENDDO
          CALL DCOPY(NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(NNR1,-XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)-DREAL(PSI(IR,1))
          ENDDO
          OEPS=0.5D0/XC_EPS
          CALL DSCAL(NNR1,OEPS,DXC,1)
        ELSEIF(NDPOINT.EQ.4) THEN
C..4-point formula
          CALL DCOPY(NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(NNR1,2.D0*XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DREAL(PSI(IR,1))
          ENDDO
          CALL DCOPY(NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(NNR1,XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)+8.D0*DREAL(PSI(IR,1))
          ENDDO
          CALL DCOPY(NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(NNR1,-XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)-8.D0*DREAL(PSI(IR,1))
          ENDDO
          CALL DCOPY(NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(NNR1,-2.D0*XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)-DREAL(PSI(IR,1))
          ENDDO
          OEPS=1.D0/(12.D0*XC_EPS)
          CALL DSCAL(NNR1,OEPS,DXC,1)
        ELSE
          CALL STOPGM('DD_XC','N Point formula not supported')
        ENDIF
      ELSE
C..spin polarised
        IF(NDPOINT.EQ.2) THEN
C..2-point formula
          CALL DCOPY(2*NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(2*NNR1,XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DREAL(PSI(IR,1))
            DXC(IR,2)=DREAL(PSI(IR,2))
          ENDDO
          CALL DCOPY(2*NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(2*NNR1,-XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)-DREAL(PSI(IR,1))
            DXC(IR,2)=DXC(IR,2)-DREAL(PSI(IR,2))
          ENDDO
          OEPS=0.5D0/XC_EPS
          CALL DSCAL(NNR1,OEPS,DXC(1,1),1)
          CALL DSCAL(NNR1,OEPS,DXC(1,2),1)
        ELSEIF(NDPOINT.EQ.4) THEN
C..4-point formula
          CALL DCOPY(2*NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(2*NNR1,2.D0*XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DREAL(PSI(IR,1))
            DXC(IR,2)=DREAL(PSI(IR,2))
          ENDDO
          CALL DCOPY(2*NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(2*NNR1,XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)+8.D0*DREAL(PSI(IR,1))
            DXC(IR,2)=DXC(IR,2)+8.D0*DREAL(PSI(IR,2))
          ENDDO
          CALL DCOPY(2*NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(2*NNR1,-XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)-8.D0*DREAL(PSI(IR,1))
            DXC(IR,2)=DXC(IR,2)-8.D0*DREAL(PSI(IR,2))
          ENDDO
          CALL DCOPY(2*NNR1,RHOE,1,RHOX,1)
          CALL DAXPY(2*NNR1,-2.D0*XC_EPS,DRHOE,1,RHOX,1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)-DREAL(PSI(IR,1))
            DXC(IR,2)=DXC(IR,2)-DREAL(PSI(IR,2))
          ENDDO
          OEPS=1.D0/(12.D0*XC_EPS)
          CALL DSCAL(NNR1,OEPS,DXC(1,1),1)
          CALL DSCAL(NNR1,OEPS,DXC(1,2),1)
        ELSE
          CALL STOPGM('DD_XC','N Point formula not supported')
        ENDIF
      ENDIF
C
      IF(SWITCH) CALL SWITCH_FUNCTIONALS
C
      CALL TIHALT('     DD_XC',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE DD_XCT(RHOE,DRHOE,RHOX,PSI,DXC,VTMP,VTEMP,GRAD,
     $     SCR,LSCR,SWITCH)
C     ==--------------------------------------------------------------==
C     == Calculate dVxc/dn*n1 by finite differences                   ==
C     == TDDFT triplet case                                           ==
C     ==--------------------------------------------------------------==
C     == INPUT: rhoe = density                                        ==
C     ==        drhoe = LR density                                    ==
C     ==        rhox = undef.                                         ==
C     ==        psi = undef.                                          ==
C     ==        dxc = undef.                                          ==
C     ==        vtmp = undef.                                         ==
C     ==        vtemp = undef.                                        ==
C     ==        grad = undef.                                         ==
C     == OUTPUT: rhoe = density                                       ==
C     ==        drhoe = LR density                                    ==
C     ==        rhox = undef.                                         ==
C     ==        psi = undef.                                          ==
C     ==        dxc = dVxc/dn*n1                                      ==
C     ==        vtmp = potential in G space                           ==
C     ==        vtemp = undef.                                        ==
C     ==        grad = undef.                                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'linres.inc'
      REAL*8     RHOE(NNR1,*),DRHOE(NNR1,*),RHOX(NNR1,*),DXC(NNR1,*),
     *           GRAD(*),SCR(*)
      COMPLEX*16 PSI(MAXFFT,*),VTMP(*),VTEMP(*)
      LOGICAL    SWITCH,TLSDSV
      INTEGER    ISUB,IR,LSCR,NLSDSV
      REAL*8     OEPS
C     ==--------------------------------------------------------------==
      IF(TXC_DD_ANA) THEN
C       CALL STOPGM('DD_XCT','Please implement analytic derivatives')
        TLSDSV=TLSD
        TLSD=.TRUE.
        NLSDSV=NLSD
        NLSD=2
        CALL DCOPY(NNR1,RHOE(1,1),1,RHOE(1,2),1)
        CALL DSCAL(NNR1,0.5D0,RHOE(1,2),1)
        CALL DCOPY(NNR1,DRHOE(1,1),1,DRHOE(1,2),1)
        CALL AZZERO(DRHOE(1,1),NNR1)
        CALL DD_XC_ANA(RHOE,DRHOE,RHOX,PSI,DXC,VTMP,VTEMP,GRAD,
     $        SCR,LSCR,SWITCH)
        CALL DSCAL(NNR1,-0.5D0,DXC(1,1),1)
        TLSD=TLSDSV
        NLSD=NLSDSV
        RETURN
      ENDIF
C     ==--------------------------------------------------------------==
      CALL TISET('     DD_XC',ISUB)
C
      IF(SWITCH) CALL SWITCH_FUNCTIONALS
C
      IF(TTAU) THEN
        CALL STOPGM('DD_XC','META FUNCTIONALS NOT IMPLENTED')
      ENDIF
      TLSD=.TRUE.
C
      IF(NDPOINT.EQ.2) THEN
C..2-point formula
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,1),1)
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,2),1)
          CALL DAXPY(NNR1,2.D0*XC_EPS,DRHOE,1,RHOX(1,2),1)
          CALL DSCAL(NNR1,0.5D0,RHOX(1,2),1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DREAL(PSI(IR,1))
          ENDDO
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,1),1)
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,2),1)
          CALL DAXPY(NNR1,-2.D0*XC_EPS,DRHOE,1,RHOX(1,2),1)
          CALL DSCAL(NNR1,0.5D0,RHOX(1,2),1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)-DREAL(PSI(IR,1))
          ENDDO
          OEPS=0.5D0/XC_EPS
          CALL DSCAL(NNR1,-0.5D0*OEPS,DXC(1,1),1)
      ELSEIF(NDPOINT.EQ.4) THEN
C..4-point formula
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,1),1)
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,2),1)
          CALL DAXPY(NNR1,4.D0*XC_EPS,DRHOE,1,RHOX(1,2),1)
          CALL DSCAL(NNR1,0.5D0,RHOX(1,2),1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DREAL(PSI(IR,1))
          ENDDO
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,1),1)
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,2),1)
          CALL DAXPY(NNR1,2.D0*XC_EPS,DRHOE,1,RHOX(1,2),1)
          CALL DSCAL(NNR1,0.5D0,RHOX(1,2),1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)+8.D0*DREAL(PSI(IR,1))
          ENDDO
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,1),1)
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,2),1)
          CALL DAXPY(NNR1,-2.D0*XC_EPS,DRHOE,1,RHOX(1,2),1)
          CALL DSCAL(NNR1,0.5D0,RHOX(1,2),1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)-8.D0*DREAL(PSI(IR,1))
          ENDDO
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,1),1)
          CALL DCOPY(NNR1,RHOE,1,RHOX(1,2),1)
          CALL DAXPY(NNR1,-4.D0*XC_EPS,DRHOE,1,RHOX(1,2),1)
          CALL DSCAL(NNR1,0.5D0,RHOX(1,2),1)
          CALL VXC_CALC(RHOX,PSI,VTEMP,VTMP,GRAD)
          DO IR=1,NNR1
            DXC(IR,1)=DXC(IR,1)-DREAL(PSI(IR,1))
          ENDDO
          OEPS=1.D0/(12.D0*XC_EPS)
          CALL DSCAL(NNR1,-0.5D0*OEPS,DXC(1,1),1)
      ELSE
          CALL STOPGM('DD_XC','N Point formula not supported')
      ENDIF
C
      IF(SWITCH) CALL SWITCH_FUNCTIONALS
C
      TLSD=.FALSE.
      CALL TIHALT('     DD_XC',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE VXC_CALC(RHOE,V,VTEMP,VTMP,GRAD)
C     ==--------------------------------------------------------------==
C     == COMPUTES XC POTENTIAL                                        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'nlcc.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'fft.inc'
C     Arguments
      REAL*8     RHOE(NNR1,*),GRAD(NNR1,*)
      COMPLEX*16 V(MAXFFT,*),VTEMP(NHG,*),VTMP(*)
C     Variables
      REAL*8     SGCC,SGCX,SXC,VXC
      INTEGER    IR
C     ==--------------------------------------------------------------==
C     == ADD CORE CHARGE TO RHOE                                      ==
C     ==--------------------------------------------------------------==
      IF(TTAU) THEN
        CALL STOPGM('VXC_CALC','META FUNCTIONALS NOT IMPLENTED')
      ENDIF
      IF(TINLC) THEN
        CALL DAXPY(NLSD*NNR1,1.D0,ROCT,1,RHOE,1)
      ENDIF
C     ==--------------------------------------------------------------==
C     == COMPUTE EXCHANGE AND CORRELATION ENERGY (EXC)                ==
C     ==--------------------------------------------------------------==
      CALL AZZERO(V,2*MAXFFT*NLSD)
      CALL XCENER(SXC,VXC,RHOE,RHOE,V)
      IF(TGC) THEN
C..CALCULATE THE GRADIENT OF THE DENSITY
        IF(TLSD) THEN
          CALL FWFFT(V(1,1))
          CALL ZGTHR(NHG,V(1,1),VTEMP(1,1),NZH)
          CALL FWFFT(V(1,2))
          CALL ZGTHR(NHG,V(1,2),VTEMP(1,2),NZH)
          DO IR=1,NNR1
            RHOE(IR,1)=RHOE(IR,1)-RHOE(IR,2)
          ENDDO
          CALL GRADEN(RHOE(1,1),V,GRAD(1,1),VTMP)
          CALL GRADEN(RHOE(1,2),V,GRAD(1,5),VTMP)
        ELSE
          CALL FWFFT(V)
          CALL ZGTHR(NHG,V,VTEMP,NZH)
          CALL GRADEN(RHOE,V,GRAD,VTMP)
        ENDIF
C..GRADIENT CORRECTION TO THE EXCHANGE ENERGY
        IF(TLSD) THEN
          CALL GCLSD(SGCX,SGCC,RHOE,V,VTEMP,VTMP,GRAD,.FALSE.)
        ELSE
          CALL GCENER(SGCX,SGCC,RHOE,V,VTEMP,VTMP,GRAD,.FALSE.)
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
