c     ==================================================================
      SUBROUTINE DD_XC_ANA(RHOE,DRHOE,RHOX,V,DXC,VTMP,VTEMP,GRAD,SCR,
     $     LSCR,SWITCH)
c     ==--------------------------------------------------------------==
c     Calculate dVxc/dn*n1 using analytical second derivatives
c     ==--------------------------------------------------------------==
c     == INPUT: rhoe = density (a+b,b)                                ==
c     ==        drhoe = LR density (a+b,b)                            ==
c     ==        v = undef. (size: psi)                                ==
C     ==        dxc = undef.                                          ==
c     ==        vtmp = undef.                                         ==
c     ==        vtemp = undef.                                        ==
c     ==        grad = undef.                                         ==
c     == OUTPUT: rhoe = density (a+b,b)                               ==
c     ==        drhoe = LR density (a+b,b)                            ==
c     ==        v = undef. (size: psi)                                ==
c     ==        dxc = dVxc/dn*n1                                      ==
c     ==        vtmp = undef.                                         ==
c     ==        vtemp = undef.                                        ==
c     ==        grad = (|grad|**2,x,y,z)                              ==
c     ==--------------------------------------------------------------==
      Implicit none
      Include 'system.h'
      Include 'cnst.inc'
      Include 'fft.inc'
      Include 'cppt.inc'
      Include 'spin.inc'
      Include 'func.inc'
c     Arguments
      Real*8 rhoe(NNR1,NLSD),grad(NNR1,4*NLSD),drhoe(NNR1,NLSD),
     &     scr(*),dxc(NNR1,*),rhox(NNR1,NLSD)
      Complex*16 V(MAXFFT,NLSD),VTEMP(NHG),VTMP(NHG)
      Integer lscr
      Logical switch
c     Local variables
      Real*8 skalar,tmp1(NNR1),tmp2(NNR1),tmp3(NNR1),tmp4(NNR1),
     $     tmp5(NNR1),dgrad(NNR1,3*NLSD)
      Real*8 densa,densb,grada,gradb,gradax,gradbx,graday,ganb,
     $     skab,skbb,naga,nana,nanb,A,gradby,gradaz,gradbz,granb,skaa,
     $     skba,gagb,nagb,nagrb,dfdn,dfdg,dfdga,d2fdn2,d2fdndg,d2fdg2,
     $     LDA,sgrad,hf,qt,tw,loca,locb,rho,rhob
      Real*8 p86a,p86b,sknb,sgradb,skgb,gask,nask,gaga,sksk,ga,sk
      Pointer (IP_tmp1,tmp1),(IP_tmp2,tmp2),(IP_tmp3,tmp3),
     &     (IP_tmp4,tmp4),(IP_tmp5,tmp5),(IP_dgrad,dgrad)
      Logical debug
      Integer IR,IG,ISPIN,ISUB,ldd_xc_ana 
      Character*30 tag
      Parameter (A=-0.930525736349100025002010218071667D0)
      Parameter (hf=0.5D0, qt=0.25D0, tw=2.0D0)
c     ==--------------------------------------------------------------==
      call TISET(' DD_XC_ANA',ISUB)
c     ==--------------------------------------------------------------==
      call GIVE_SCR_DD_XC_ANA(ldd_xc_ana,tag)
      call TEST_SCR('DD_XC_ANA',tag,lscr,ldd_xc_ana)
c     Allocation of scratch space for gradients (if necessary)
      debug=.false.
      if (TGC) then
        call SCRSET(scr,lscr,debug)
        call get_addr(ip_tmp1,rhox)
        if(TLSD) then
        call get_addr(ip_tmp2,rhox(1,2))
        else
          call SCRPTR(scr,lscr,ip_tmp2,NNR1,'tmp2',debug)
        endif
        call SCRPTR(scr,lscr,ip_tmp3,NNR1,'tmp3',debug)
        call SCRPTR(scr,lscr,ip_tmp4,NNR1,'tmp4',debug)
        call SCRPTR(scr,lscr,ip_tmp5,NNR1,'tmp5',debug)
        call SCRPTR(scr,lscr,ip_dgrad,NNR1*3*NLSD,'dgrad',debug)
        call azzero(tmp1,nnr1)
        call azzero(tmp2,nnr1)
        call azzero(tmp3,nnr1)
        call azzero(tmp4,nnr1)
        call azzero(tmp5,nnr1)
        call azzero(dgrad,nnr1*3*nlsd)
      endif
c     Calculation of gradients (if necessary)
      if(.not.TLSD.and.TGC) then
        call graden(rhoe,V,grad,vtmp)
        call graden_comp(drhoe,V,dgrad,vtmp)
      elseif(TLSD.and..not.TGC) then
         do IR=1,NNR1
            rhoe(IR,1)=rhoe(IR,1)-rhoe(IR,2)
            drhoe(IR,1)=drhoe(IR,1)-drhoe(IR,2)
         enddo
      elseif(TLSD.and.TGC) then
         do IR=1,NNR1
            rhoe(IR,1)=rhoe(IR,1)-rhoe(IR,2)
            drhoe(IR,1)=drhoe(IR,1)-drhoe(IR,2)
         enddo
         call graden(rhoe,V,grad,vtmp)
         call graden(rhoe(1,2),V,grad(1,5),vtmp)
         call graden_comp(drhoe,V,dgrad,vtmp)
         call graden_comp(drhoe(1,2),V,dgrad(1,4),vtmp)
      endif

      if(switch) call switch_functionals

      if(TLSD) then
         call azzero(V,4*MAXFFT)
      else
         call azzero(V,2*MAXFFT)
      endif
      if(TGC) call azzero(VTEMP,2*NHG)

      if(.not.TLSD) then
c     no spin polarisation --> a-dens = b-dens = 1/2 * dens      
         call dscal(NNR1,hf,rhoe,1)
         call dscal(NNR1,hf,drhoe,1)
         if(TGC) then
            call dscal(NNR1,qt,grad(1,1),1)
            call dscal(3*NNR1,hf,grad(1,2),1)
            call dscal(3*NNR1,hf,dgrad(1,1),1)
         endif
c     ==--------------------------------------------------------------==
c     ==  SLATER LDA only  -  not spin polarised                      ==
c     ==--------------------------------------------------------------==
        if(.not.TGC) then
          do IR=1,NNR1
            if (rhoe(IR,1).le.0.D0) then
              V(IR,1)=0.D0
            else
              call SLATER_X(rhoe(IR,1),LDA)
              V(IR,1)=LDA*drhoe(IR,1)
            endif
          enddo
          goto 99
c     ==--------------------------------------------------------------==
c     ==  BECKE88 ONLY - not spin polarised                           ==
c     ==--------------------------------------------------------------==
c     no spin polarisation --> a-dens = b-dens = 1/2 * dens      
        elseif(MGCX.eq.1 .and. MGCC.eq.0) then
          do IR=1,NNR1
            if (rhoe(IR,1).le.0.D0) then
            elseif(rhoe(IR,1).gt.0.D0.and.(rhoe(IR,1).lt.gceps.or.
     $              grad(IR,1).le.0.D0)) then                           
              call SLATER_X(rhoe(IR,1),LDA)
              tmp1(IR)=LDA*drhoe(IR,1)
            else
              call B88_x(rhoe(IR,1),grad(IR,1),dfdn,dfdg,
     &             d2fdn2,d2fdndg,d2fdg2,LDA)
              skaa=grad(IR,2)*dgrad(IR,1)+grad(IR,3)*dgrad(IR,2)
     &             +grad(IR,4)*dgrad(IR,3)
              sgrad=dsqrt(grad(IR,1))
              skalar=skaa/sgrad
              tmp1(IR)=(d2fdn2+LDA)*drhoe(IR,1)+d2fdndg*skalar
              tmp2(IR)=1.D0/sgrad*(d2fdndg*drhoe(IR,1)+
     $             d2fdg2*skalar-dfdg*skalar/sgrad)
              tmp3(IR)=dfdg/sgrad
            endif
          enddo
c     ==--------------------------------------------------------------==
c     ==  BLYP  -  not spin polarised                                 ==
c     ==--------------------------------------------------------------==
c     no spin polarisation --> a-dens = b-dens = 1/2 * dens
        elseif(MGCX.eq.1 .and. MGCC.eq.2) then
          do IR=1,NNR1
            if(rhoe(IR,1).gt.1.D-24.and.(rhoe(IR,1).lt.gceps.or.
     $            grad(IR,1).le.1.D-48)) then       
              call lyp88_c_loc(rhoe(IR,1),rhoe(IR,1),loca,locb)
              tmp1(IR)=(loca+locb)*drhoe(IR,1)
            elseif(rhoe(IR,1).gt.1.D-24) then
              call lyp88_c(rhoe(IR,1),rhoe(IR,1),
     &          grad(IR,1),grad(IR,2),grad(IR,3),grad(IR,4),
     $          grad(IR,1),grad(IR,2),grad(IR,3),grad(IR,4),
     $          dfdga,nana,nanb,naga,granb,gagb,nagb,nagrb,ganb)
              call B88_x(rhoe(IR,1),grad(IR,1),dfdn,dfdg,
     &             d2fdn2,d2fdndg,d2fdg2,LDA)
              skaa=grad(IR,2)*dgrad(IR,1)+grad(IR,3)*dgrad(IR,2)
     &             +grad(IR,4)*dgrad(IR,3)
              sgrad=dsqrt(grad(IR,1))
              skalar=skaa/sgrad
              tmp1(IR)=(nana+d2fdn2+LDA)*drhoe(IR,1)
     &             +nanb*drhoe(IR,1)+naga*2.D0*skaa+nagb*skaa
     &             +nagrb*2.D0*skaa+nagb*skaa+d2fdndg*skalar
              tmp2(IR)=2.D0*(naga*drhoe(IR,1)+granb*drhoe(IR,1))
     &             +1.D0/sgrad*(d2fdndg*drhoe(IR,1)
     &             +skalar*(d2fdg2-dfdg/sgrad))
              tmp3(IR)=dfdga*2.D0+dfdg/sgrad
              tmp4(IR)=nagb*drhoe(IR,1)+ganb*drhoe(IR,1)
              tmp5(IR)=gagb
            endif
          enddo
c     ==--------------------------------------------------------------==
c     ==  BP  -  not spin polarised                                   ==
c     ==--------------------------------------------------------------==
c     no spin polarisation --> a-dens = b-dens = 1/2 * dens
        elseif(MGCX.eq.1 .and. MGCC.eq.1) then          
          do IR=1,NNR1
            if (rhoe(IR,1).gt.1.D-24.and.(rhoe(IR,1).lt.gceps.or.
     $            grad(IR,1).le.1.D-48)) then
               call SLATER_X(rhoe(IR,1),LDA)
               call p86_c_loc(rhoe(IR,1),rhoe(IR,1),p86a,p86b)
               tmp1(IR)=(LDA+p86a)*drhoe(IR,1)+p86b*drhoe(IR,1)
            elseif(rhoe(IR,1).gt.1.D-24) then
              call P86_c(rhoe(IR,1),rhoe(IR,1),grad(IR,1),
     $              grad(IR,2),grad(IR,3),grad(IR,4),
     $              grad(IR,1),grad(IR,2),grad(IR,3),
     &              grad(IR,4),nana,naga,gaga,nask,gask,sk,ga,sksk,
     $              gagb,nagb,skgb,sknb,ganb,nanb)
              call B88_x(rhoe(IR,1),grad(IR,1),dfdn,dfdg,d2fdn2,
     $             d2fdndg,d2fdg2,LDA)
              skaa=(grad(IR,2)*dgrad(IR,1)+grad(IR,3)*dgrad(IR,2)
     &          +grad(IR,4)*dgrad(IR,3))
              skab=skaa
              skba=skaa
              skbb=skaa
              sgrad=dsqrt(grad(IR,1))
              sgradb=sgrad
              skalar=skaa/sgrad
              tmp1(IR)=(nana+d2fdn2+LDA)*drhoe(IR,1)+
     $             nanb*drhoe(IR,1)
     $             +naga*skalar+nask*(skba+skab)+nagb*skbb/sgradb
     $             +d2fdndg*skalar
              tmp2(IR)=(naga*drhoe(IR,1)+ganb*drhoe(IR,1)+gask*
     $             (skba+skab)+gaga*skalar+gagb*skbb/sgradb
     $             -ga*skalar/sgrad)/sgrad
     $             +(d2fdndg*drhoe(IR,1)+skalar*
     $             (d2fdg2-dfdg/sgrad))/sgrad
              tmp3(IR)=(ga+dfdg)/sgrad
              tmp4(IR)=nask*drhoe(IR,1)+sknb*drhoe(IR,1)
     $             +gask*skalar+skgb*skbb/sgradb+sksk*(skba+skab)
              tmp5(IR)=sk
            endif
          enddo
c     ==--------------------------------------------------------------==
c     ==  PBE  -  not spin polarised
c     ==--------------------------------------------------------------==
c     no spin polarisation --> a-dens = b-dens = 1/2 * dens
        elseif(MGCX.eq.3 .and. MGCC.eq.4) then          
          do IR=1,NNR1
            if (rhoe(IR,1).gt.1.D-24.and.(rhoe(IR,1).lt.gceps.or.
     $            grad(IR,1).le.1.D-48)) then  
               call SLATER_X(rhoe(IR,1),LDA)
               call pbe96_c_loc(rhoe(IR,1),rhoe(IR,1),nana,nanb)
               tmp1(IR)=(LDA+nana)*drhoe(IR,1)+nanb*drhoe(IR,1)
            elseif(rhoe(IR,1).gt.1.D-24) then
              call pbe96_x(rhoe(IR,1),grad(IR,1),dfdg,d2fdn2,
     $              d2fdndg,d2fdg2)
              call pbe96_c(rhoe(IR,1),rhoe(IR,1),grad(IR,1),
     $             grad(IR,2),grad(IR,3),grad(IR,4),
     $             grad(IR,1),grad(IR,2),grad(IR,3),
     &             grad(IR,4),nana,naga,gaga,nask,gask,sk,ga,sksk,gagb,
     $             nagb,skgb,sknb,ganb,nanb)
              skaa=(grad(IR,2)*dgrad(IR,1)+grad(IR,3)*dgrad(IR,2)
     &          +grad(IR,4)*dgrad(IR,3))
              skab=skaa
              skba=skaa
              skbb=skaa
              tmp1(IR)=(nana+d2fdn2)*drhoe(IR,1)+nanb*
     $             drhoe(IR,1)+naga*2.D0*skaa+nask*(skba+skab)+
     $             nagb*2.D0*skbb+d2fdndg*2.D0*skaa
              tmp2(IR)=2.D0*(naga*drhoe(IR,1)+ganb*drhoe(IR,1)+
     $             gask*(skba+skab)+gaga*2.D0*skaa+gagb*2.D0*skbb
     $             +d2fdndg*drhoe(IR,1)+2.D0*skaa*d2fdg2)
              tmp3(IR)=2.D0*(ga+dfdg)
              tmp4(IR)=nask*drhoe(IR,1)+sknb*drhoe(IR,1)+
     $             gask*2.D0*skaa+skgb*2.D0*skbb+sksk*(skba+skab)
              tmp5(IR)=sk
            endif
          enddo
        else
          CALL STOPGM('DD_XC_ANA','Functional not implemented')
        endif

c     ==--------------------------------------------------------------==
c     CALCULATION OF DIVERGENCE IN RECIPROCAL SPACE:
c              real space               G-space
c       dot(nabla,gradient) --> dot(i*G-Vector,gradient)
c     tmp2: terms to be multiplied with gradient of a-density
c     tmp3: terms to be multiplied with gradient of a-linres-density
c     tmp4: terms to be multiplied with gradient of b-density
c     tmp5: terms to be multiplied with gradient of b-linres-density
c     ==--------------------------------------------------------------==
c     FFTMULT:calculates dot product of i*G-Vector with gradient 
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp2(IR)*grad(IR,2),tmp2(IR)*grad(IR,3))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,1,2)

        call AZZERO(V,2*MAXFFT)
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp2(IR)*grad(IR,4),tmp3(IR)*dgrad(IR,1))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,3,1)

        call AZZERO(V,2*MAXFFT)
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp3(IR)*dgrad(IR,2),tmp3(IR)*dgrad(IR,3))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,2,3)

        call AZZERO(V,2*MAXFFT)
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp4(IR)*grad(IR,2),tmp4(IR)*grad(IR,3))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,1,2)

        call azzero(V,2*MAXFFT)
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp4(IR)*grad(IR,4),tmp5(IR)*dgrad(IR,1))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,3,1)

        call azzero(V,2*MAXFFT)
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp5(IR)*dgrad(IR,2),tmp5(IR)*dgrad(IR,3))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,2,3)

        call azzero(V,2*MAXFFT)
        do IG=1,NHG
           V(NZH(IG),1) = VTEMP(IG)
           V(INDZ(IG),1) = DCONJG(VTEMP(IG))
        enddo

        call INVFFT(V(1,1))
        do IR=1,NNR1
           V(IR,1)=V(IR,1)+dcmplx(tmp1(IR),0.D0)
        enddo 
        call dscal(NNR1,tw,rhoe,1)
        call dscal(NNR1,tw,drhoe,1)
         if(TGC) then
            call dscal(NNR1,tw*tw,grad(1,1),1)
            call dscal(3*NNR1,tw,grad(1,2),1)
            call dscal(3*NNR1,tw,dgrad(1,1),1)
         endif

      else
c     SPIN POLARISED
c     ==--------------------------------------------------------------==
c     ==  SLATER LDA only
c     ==--------------------------------------------------------------==
        if(.not.TGC) then
          do IR=1,NNR1
            if(rhoe(IR,1).gt.0.D0) then
              call SLATER_X(rhoe(IR,1),LDA)
              V(IR,1)=LDA*drhoe(IR,1)
            endif
            if(rhoe(IR,2).gt.0.D0) then
              call SLATER_X(rhoe(IR,2),LDA)
              V(IR,2)=LDA*drhoe(IR,2)
            endif
          enddo
          goto 99
c     Alpha Spin
c     ==--------------------------------------------------------------==
c     ==  BECKE88 ONLY: individual terms
c     ==--------------------------------------------------------------==
        elseif(MGCX.eq.1 .and. MGCC.eq.0) then
          do IR=1,NNR1
            if (rhoe(IR,1).le.0.D0) then
            elseif(rhoe(IR,1).gt.0.D0.and.(rhoe(IR,1).lt.gceps.or.
     $              grad(IR,1).le.0.D0)) then              
              call SLATER_X(rhoe(IR,1),LDA)
              tmp1(IR)=LDA*drhoe(IR,1)
            else
              call B88_x(rhoe(IR,1),grad(IR,1),dfdn,dfdg,
     &             d2fdn2,d2fdndg,d2fdg2,LDA)
              skaa=grad(IR,2)*dgrad(IR,1)+grad(IR,3)*dgrad(IR,2)
     &             +grad(IR,4)*dgrad(IR,3)
              sgrad=dsqrt(grad(IR,1))
              skalar=skaa/sgrad
              tmp1(IR)=(d2fdn2+LDA)*drhoe(IR,1)+d2fdndg*skalar
              tmp2(IR)=1.D0/sgrad*(d2fdndg*drhoe(IR,1)+d2fdg2*skalar
     &             -dfdg*skalar/sgrad)
              tmp3(IR)=dfdg/sgrad
            endif
          enddo

c     ==--------------------------------------------------------------==
c     ==  BLYP: individual terms
c     ==--------------------------------------------------------------==
        elseif(MGCX.eq.1 .and. MGCC.eq.2) then
          do IR=1,NNR1
            if(rhoe(IR,1).gt.1.D-24.and.(rhoe(IR,1).lt.gceps.or.
     $            grad(IR,1).le.1.D-48)) then       
              call lyp88_c_loc(rhoe(IR,1),rhoe(IR,2),loca,locb)
              tmp1(IR)=loca*drhoe(IR,1)+locb*drhoe(IR,2)
            elseif(rhoe(IR,1).gt.1.D-24) then
              call lyp88_c(rhoe(IR,1),rhoe(IR,2),grad(IR,1),grad(IR,2)
     &          ,grad(IR,3),grad(IR,4),grad(IR,5),grad(IR,6),grad(IR,7)
     &          ,grad(IR,8),dfdga,nana,nanb,naga,granb,gagb,nagb,
     &          nagrb,ganb)
              call B88_x(rhoe(IR,1),grad(IR,1),dfdn,dfdg,d2fdn2,d2fdndg
     $          ,d2fdg2,LDA)
              skaa=grad(IR,2)*dgrad(IR,1)+grad(IR,3)*dgrad(IR,2)
     &          +grad(IR,4)*dgrad(IR,3)
              skab=grad(IR,2)*dgrad(IR,4)+grad(IR,3)*dgrad(IR,5)
     &          +grad(IR,4)*dgrad(IR,6)
              skba=grad(IR,6)*dgrad(IR,1)+grad(IR,7)*dgrad(IR,2)
     &          +grad(IR,8)*dgrad(IR,3)
              skbb=grad(IR,6)*dgrad(IR,4)+grad(IR,7)*dgrad(IR,5)
     &          +grad(IR,8)*dgrad(IR,6)
              sgrad=dsqrt(grad(IR,1))
              skalar=skaa/sgrad
              tmp1(IR)=(nana+d2fdn2+LDA)*drhoe(IR,1)
     &          +nanb*drhoe(IR,2)+naga*2.D0*skaa+nagb*skab
     &          +nagrb*2.D0*skbb+nagb*skba+d2fdndg*skalar
              tmp2(IR)=2.D0*(naga*drhoe(IR,1)+granb*drhoe(IR,2))
     &          +1.D0/sgrad*(d2fdndg*drhoe(IR,1)
     &          +skalar*(d2fdg2-dfdg/sgrad))
              tmp3(IR)=dfdga*2.D0+dfdg/sgrad
              tmp4(IR)=nagb*drhoe(IR,1)+ganb*drhoe(IR,2)
              tmp5(IR)=gagb
            endif
          enddo
c     ==--------------------------------------------------------------==
c     ==  BP
c     ==--------------------------------------------------------------==
        elseif(MGCX.eq.1 .and. MGCC.eq.1) then          
          do IR=1,NNR1
            if (rhoe(IR,1).gt.1.D-24.and.(rhoe(IR,1).lt.gceps.or.
     $            grad(IR,1).le.1.D-48)) then  
               call SLATER_X(rhoe(IR,1),LDA)
               call p86_c_loc(rhoe(IR,1),rhoe(IR,2),p86a,p86b)
               tmp1(IR)=(LDA+p86a)*drhoe(IR,1)+p86b*drhoe(IR,2)
            elseif(rhoe(IR,1).gt.1.D-24) then
              call P86_c(rhoe(IR,1),rhoe(IR,2),grad(IR,1),grad(IR,2)
     &          ,grad(IR,3),grad(IR,4),grad(IR,5),grad(IR,6),grad(IR,7)
     &          ,grad(IR,8),nana,naga,gaga,nask,gask,sk,ga,sksk,gagb
     $          ,nagb,skgb,sknb,ganb,nanb)
              call B88_x(rhoe(IR,1),grad(IR,1),dfdn,dfdg,d2fdn2,d2fdndg
     $          ,d2fdg2,LDA)
              skaa=grad(IR,2)*dgrad(IR,1)+grad(IR,3)*dgrad(IR,2)
     &          +grad(IR,4)*dgrad(IR,3)
              skab=grad(IR,2)*dgrad(IR,4)+grad(IR,3)*dgrad(IR,5)
     &          +grad(IR,4)*dgrad(IR,6)
              skba=grad(IR,6)*dgrad(IR,1)+grad(IR,7)*dgrad(IR,2)
     &          +grad(IR,8)*dgrad(IR,3)
              skbb=grad(IR,6)*dgrad(IR,4)+grad(IR,7)*dgrad(IR,5)
     &          +grad(IR,8)*dgrad(IR,6)
              sgrad=dsqrt(grad(IR,1))
              sgradb=dsqrt(grad(IR,5))
              skalar=skaa/sgrad
              tmp1(IR)=(nana+d2fdn2+LDA)*drhoe(IR,1)+nanb*drhoe(IR,2)
     $          +naga*skalar+nask*(skba+skab)+nagb*skbb/sgradb
     $             +d2fdndg*skalar
              tmp2(IR)=(naga*drhoe(IR,1)+ganb*drhoe(IR,2)+gask*
     $             (skba+skab)+gaga*skalar+gagb*skbb/sgradb
     $             -ga*skalar/sgrad)/sgrad
     $             +1.D0/sgrad*(d2fdndg*drhoe(IR,1)+skalar*
     $             (d2fdg2-dfdg/sgrad))
              tmp3(IR)=(ga+dfdg)/sgrad
              tmp4(IR)=nask*drhoe(IR,1)+sknb*drhoe(IR,2)+gask*skalar+
     $             skgb*skbb/sgradb+sksk*(skba+skab)
              tmp5(IR)=sk
            endif
          enddo
c     ==--------------------------------------------------------------==
c     ==  PBE
c     ==--------------------------------------------------------------==
        elseif(MGCX.eq.3 .and. MGCC.eq.4) then          
          do IR=1,NNR1
            if (rhoe(IR,1).gt.1.D-24.and.(rhoe(IR,1).lt.gceps.or.
     $            grad(IR,1).le.1.D-48)) then  
               call SLATER_X(rhoe(IR,1),LDA)
               call pbe96_c_loc(rhoe(IR,1),rhoe(IR,2),nana,nanb)
               tmp1(IR)=(LDA+nana)*drhoe(IR,1)+nanb*drhoe(IR,2)
            elseif(rhoe(IR,1).gt.1.D-24) then
              call pbe96_x(rhoe(IR,1),grad(IR,1),dfdg,d2fdn2,d2fdndg,
     $              d2fdg2)
              call pbe96_c(rhoe(IR,1),rhoe(IR,2),grad(IR,1),grad(IR,2)
     &          ,grad(IR,3),grad(IR,4),grad(IR,5),grad(IR,6),grad(IR,7)
     &          ,grad(IR,8),nana,naga,gaga,nask,gask,sk,ga,sksk,gagb
     $          ,nagb,skgb,sknb,ganb,nanb)
              skaa=grad(IR,2)*dgrad(IR,1)+grad(IR,3)*dgrad(IR,2)
     &          +grad(IR,4)*dgrad(IR,3)
              skab=grad(IR,2)*dgrad(IR,4)+grad(IR,3)*dgrad(IR,5)
     &          +grad(IR,4)*dgrad(IR,6)
              skba=grad(IR,6)*dgrad(IR,1)+grad(IR,7)*dgrad(IR,2)
     &          +grad(IR,8)*dgrad(IR,3)
              skbb=grad(IR,6)*dgrad(IR,4)+grad(IR,7)*dgrad(IR,5)
     &          +grad(IR,8)*dgrad(IR,6)
              tmp1(IR)=(nana+d2fdn2)*drhoe(IR,1)+nanb*drhoe(IR,2)
     $          +naga*2.D0*skaa+nask*(skba+skab)+nagb*2.D0*skbb
     $             +d2fdndg*2.D0*skaa
              tmp2(IR)=2.D0*(naga*drhoe(IR,1)+ganb*drhoe(IR,2)+gask*
     $             (skba+skab)+gaga*2.D0*skaa+gagb*2.D0*skbb
     $             +d2fdndg*drhoe(IR,1)+2.D0*skaa*d2fdg2)
              tmp3(IR)=2.D0*(ga+dfdg)
              tmp4(IR)=nask*drhoe(IR,1)+sknb*drhoe(IR,2)+gask*2.D0*skaa+
     $             skgb*2.D0*skbb+sksk*(skba+skab)
              tmp5(IR)=sk
            endif
          enddo
        else
          CALL STOPGM('DD_XC_ANA','Functional not implemented')
        endif

c     ==--------------------------------------------------------------==
c     CALCULATION OF DIVERGENCE IN RECIPROCAL SPACE:
c              real space               G-space
c       dot(nabla,gradient) --> dot(i*G-Vector,gradient)
c     tmp2: terms to be multiplied with gradient of a-density
c     tmp3: terms to be multiplied with gradient of a-linres-density
c     tmp4: terms to be multiplied with gradient of b-density
c     tmp5: terms to be multiplied with gradient of b-linres-density
c     ==--------------------------------------------------------------==
c     FFTMULT:calculates dot product ofi*G-Vector with gradient 
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp2(IR)*grad(IR,2),tmp2(IR)*grad(IR,3))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,1,2)

        call AZZERO(V,2*MAXFFT)
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp2(IR)*grad(IR,4),tmp3(IR)*dgrad(IR,1))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,3,1)

        call AZZERO(V,2*MAXFFT)
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp3(IR)*dgrad(IR,2),tmp3(IR)*dgrad(IR,3))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,2,3)

        call AZZERO(V,2*MAXFFT)
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp4(IR)*grad(IR,6),tmp4(IR)*grad(IR,7))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,1,2)

        call azzero(V,2*MAXFFT)
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp4(IR)*grad(IR,8),tmp5(IR)*dgrad(IR,4))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,3,1)

        call azzero(V,2*MAXFFT)
        do IR=1,NNR1
           V(IR,1)=dcmplx(tmp5(IR)*dgrad(IR,5),tmp5(IR)*dgrad(IR,6))
        enddo
        call FWFFT(V(1,1))
        call FFTMULT(V,VTEMP,2,3)

        call azzero(V,2*MAXFFT)
        do IG=1,NHG
           V(NZH(IG),1) = VTEMP(IG)
           V(INDZ(IG),1) = DCONJG(VTEMP(IG))
        enddo
        call INVFFT(V(1,1))
        do IR=1,NNR1
           V(IR,1)=V(IR,1)+dcmplx(tmp1(IR),0.D0)
        enddo 

c     Beta Spin
        call azzero(VTEMP,2*NHG)
        call azzero(tmp1,NNR1)
        call azzero(tmp2,NNR1)
        call azzero(tmp3,NNR1)
        call azzero(tmp4,NNR1)
        call azzero(tmp5,NNR1)

c     ==--------------------------------------------------------------==
c     ==  BECKE88 ONLY: individual terms
c     ==--------------------------------------------------------------==
        if(MGCX.eq.1 .and. MGCC.eq.0) then
          do IR=1,NNR1
            if (rhoe(IR,2).le.0.D0) then
            elseif(rhoe(IR,2).gt.0.D0.and.(rhoe(IR,2).lt.gceps.or.
     $              grad(IR,5).le.0.D0)) then              
              call SLATER_X(rhoe(IR,2),LDA)
              tmp1(IR)=LDA*drhoe(IR,2)
            else
              call B88_x(rhoe(IR,2),grad(IR,5),dfdn,dfdg,
     &             d2fdn2,d2fdndg,d2fdg2,LDA)
              skaa=grad(IR,6)*dgrad(IR,4)+grad(IR,7)*dgrad(IR,5)
     &             +grad(IR,8)*dgrad(IR,6)
              sgrad=dsqrt(grad(IR,5))
              skalar=skaa/sgrad
              tmp1(IR)=(d2fdn2+LDA)*drhoe(IR,2)+d2fdndg*skalar
              tmp2(IR)=1.D0/sgrad*(d2fdndg*drhoe(IR,2)+d2fdg2*skalar
     &             -dfdg*skalar/sgrad)
              tmp3(IR)=dfdg/sgrad
            endif
          enddo
          
c     ==--------------------------------------------------------------==
c     ==  BLYP: individual terms
c     ==--------------------------------------------------------------==
        elseif(MGCX.eq.1 .and. MGCC.eq.2) then
          do IR=1,NNR1
            if(rhoe(IR,2).gt.1.D-24.and.(rhoe(IR,2).lt.gceps.or.
     $            grad(IR,5).le.1.D-48)) then       
              call lyp88_c_loc(rhoe(IR,2),rhoe(IR,1),loca,locb)
              tmp1(IR)=loca*drhoe(IR,2)+locb*drhoe(IR,1)
            elseif(rhoe(IR,2).gt.1.D-24) then
              call lyp88_c(rhoe(IR,2),rhoe(IR,1),grad(IR,5),grad(IR,6)
     &          ,grad(IR,7),grad(IR,8),grad(IR,1),grad(IR,2),grad(IR,3)
     &          ,grad(IR,4),dfdga,nana,nanb,naga,granb,gagb,nagb,nagrb
     &          ,ganb)
              call B88_x(rhoe(IR,2),grad(IR,5),dfdn,dfdg,d2fdn2,
     &          d2fdndg,d2fdg2,LDA)
              skaa=grad(IR,6)*dgrad(IR,4)+grad(IR,7)*dgrad(IR,5)
     &          +grad(IR,8)*dgrad(IR,6)
              skab=grad(IR,6)*dgrad(IR,1)+grad(IR,7)*dgrad(IR,2)
     &          +grad(IR,8)*dgrad(IR,3)
              skba=grad(IR,2)*dgrad(IR,4)+grad(IR,3)*dgrad(IR,5)
     &          +grad(IR,4)*dgrad(IR,6)
              skbb=grad(IR,2)*dgrad(IR,1)+grad(IR,3)*dgrad(IR,2)
     &          +grad(IR,4)*dgrad(IR,3)
              sgrad=dsqrt(grad(IR,5))
              skalar=skaa/sgrad
              tmp1(IR)=(nana+d2fdn2+LDA)*drhoe(IR,2)
     &          +nanb*drhoe(IR,1)+naga*2.D0*skaa+nagb*skab
     &          +nagrb*2.D0*skbb+nagb*skba+d2fdndg*skalar
              tmp2(IR)=2.D0*(naga*drhoe(IR,2)+granb*drhoe(IR,1))
     &          +1.D0/sgrad*(d2fdndg*drhoe(IR,2)
     &          +skalar*(d2fdg2-dfdg/sgrad))
              tmp3(IR)=dfdga*2.D0+dfdg/sgrad
              tmp4(IR)=nagb*drhoe(IR,2)+ganb*drhoe(IR,1)
              tmp5(IR)=gagb
            endif
          enddo
c     ==--------------------------------------------------------------==
c     ==  BP
c     ==--------------------------------------------------------------==
        elseif(MGCX.eq.1 .and. MGCC.eq.1) then          
          do IR=1,NNR1
            if (rhoe(IR,2).gt.1.D-24.and.(rhoe(IR,2).lt.gceps.or.
     $            grad(IR,5).le.1.D-48)) then
               call  SLATER_X(rhoe(IR,2),LDA)
               call p86_c_loc(rhoe(IR,2),rhoe(IR,1),p86a,p86b)
               tmp1(IR)=(LDA+p86a)*drhoe(IR,2)+p86b*drhoe(IR,1)         
            elseif(rhoe(IR,2).gt.1.D-24) then
              call P86_c(rhoe(IR,2),rhoe(IR,1),grad(IR,5),grad(IR,6)
     &          ,grad(IR,7),grad(IR,8),grad(IR,1),grad(IR,2),grad(IR,3)
     &          ,grad(IR,4),nana,naga,gaga,nask,gask,sk,ga,sksk,gagb
     $          ,nagb,skgb,sknb,ganb,nanb)
              call B88_x(rhoe(IR,2),grad(IR,5),dfdn,dfdg,d2fdn2,d2fdndg
     $          ,d2fdg2,LDA)
              skaa=grad(IR,6)*dgrad(IR,4)+grad(IR,7)*dgrad(IR,5)
     &          +grad(IR,8)*dgrad(IR,6)
              skab=grad(IR,6)*dgrad(IR,1)+grad(IR,7)*dgrad(IR,2)
     &          +grad(IR,8)*dgrad(IR,3)
              skba=grad(IR,2)*dgrad(IR,4)+grad(IR,3)*dgrad(IR,5)
     &          +grad(IR,4)*dgrad(IR,6)
              skbb=grad(IR,2)*dgrad(IR,1)+grad(IR,3)*dgrad(IR,2)
     &          +grad(IR,4)*dgrad(IR,3)
              sgrad=dsqrt(grad(IR,5))
              sgradb=dsqrt(grad(IR,1))
              skalar=skaa/sgrad
              tmp1(IR)=(nana+d2fdn2+LDA)*drhoe(IR,2)+nanb*drhoe(IR,1)
     $          +naga*skalar+nask*(skba+skab)+nagb*skbb/sgradb
     $             +d2fdndg*skalar
              tmp2(IR)=(naga*drhoe(IR,2)+ganb*drhoe(IR,1)+gask*
     $             (skba+skab)+gaga*skalar+gagb*skbb/sgradb
     $             -ga*skalar/sgrad)/sgrad
     $             +1.D0/sgrad*(d2fdndg*drhoe(IR,2)+skalar*
     $             (d2fdg2-dfdg/sgrad))
              tmp3(IR)=(ga+dfdg)/sgrad
              tmp4(IR)=nask*drhoe(IR,2)+sknb*drhoe(IR,1)+gask*skalar+
     $             skgb*skbb/sgradb+sksk*(skba+skab)
              tmp5(IR)=sk
            endif
          enddo
c     ==--------------------------------------------------------------==
c     ==  PBE
c     ==--------------------------------------------------------------==
        elseif(MGCX.eq.3 .and. MGCC.eq.4) then          
          do IR=1,NNR1
            if (rhoe(IR,2).gt.1.D-24.and.(rhoe(IR,2).lt.gceps.or.
     $            grad(IR,5).le.1.D-48)) then  
               call SLATER_X(rhoe(IR,2),LDA)
               call pbe96_c_loc(rhoe(IR,2),rhoe(IR,1),nana,nanb)
               tmp1(IR)=(LDA+nana)*drhoe(IR,2)+nanb*drhoe(IR,1)
            elseif(rhoe(IR,2).gt.1.D-24) then
              call pbe96_x(rhoe(IR,2),grad(IR,5),dfdg,d2fdn2,d2fdndg,
     $              d2fdg2)
              call pbe96_c(rhoe(IR,2),rhoe(IR,1),grad(IR,5),grad(IR,6)
     &          ,grad(IR,7),grad(IR,8),grad(IR,1),grad(IR,2),grad(IR,3)
     &          ,grad(IR,4),nana,naga,gaga,nask,gask,sk,ga,sksk,gagb
     $          ,nagb,skgb,sknb,ganb,nanb)
              skaa=grad(IR,6)*dgrad(IR,4)+grad(IR,7)*dgrad(IR,5)
     &          +grad(IR,8)*dgrad(IR,6)
              skab=grad(IR,6)*dgrad(IR,1)+grad(IR,7)*dgrad(IR,2)
     &          +grad(IR,8)*dgrad(IR,3)
              skba=grad(IR,2)*dgrad(IR,4)+grad(IR,3)*dgrad(IR,5)
     &          +grad(IR,4)*dgrad(IR,6)
              skbb=grad(IR,2)*dgrad(IR,1)+grad(IR,3)*dgrad(IR,2)
     &          +grad(IR,4)*dgrad(IR,3)
              tmp1(IR)=(nana+d2fdn2)*drhoe(IR,2)+nanb*drhoe(IR,1)
     $          +naga*2.D0*skaa+nask*(skba+skab)+nagb*2.D0*skbb
     $             +d2fdndg*2.D0*skaa
              tmp2(IR)=2.D0*(naga*drhoe(IR,2)+ganb*drhoe(IR,1)+gask*
     $             (skba+skab)+gaga*2.D0*skaa+gagb*2.D0*skbb
     $             +d2fdndg*drhoe(IR,2)+2.D0*skaa*d2fdg2)
              tmp3(IR)=2.D0*(ga+dfdg)
              tmp4(IR)=nask*drhoe(IR,2)+sknb*drhoe(IR,1)+gask*2.D0*skaa+
     $             skgb*2.D0*skbb+sksk*(skba+skab)
              tmp5(IR)=sk
            endif
          enddo
        else
          CALL STOPGM('DD_XC_ANA','Functional not implemented')
        endif
        
c     ==--------------------------------------------------------------==
c     CALCULATION OF DIVERGENCE IN RECIPROCAL SPACE:
c              real space               G-space
c       dot(nabla,gradient) --> dot(G-Vector,gradient)
c     tmp2: terms to be multiplied with gradient of a-density
c     tmp3: terms to be multiplied with gradient of a-linres-density
c     tmp4: terms to be multiplied with gradient of b-density
c     tmp5: terms to be multiplied with gradient of b-linres-density
c     ==--------------------------------------------------------------==
c     FFTMULT:calculates dot product of G-Vector with gradient 
        do IR=1,NNR1
           V(IR,2)=dcmplx(tmp2(IR)*grad(IR,6),tmp2(IR)*grad(IR,7))
        enddo
        call FWFFT(V(1,2))
        call FFTMULT(V(1,2),VTEMP,1,2)

        call AZZERO(V(1,2),2*MAXFFT)
        do IR=1,NNR1
           V(IR,2)=dcmplx(tmp2(IR)*grad(IR,8),tmp3(IR)*dgrad(IR,4))
        enddo
        call FWFFT(V(1,2))
        call FFTMULT(V(1,2),VTEMP,3,1)

        call AZZERO(V(1,2),2*MAXFFT)
        do IR=1,NNR1
           V(IR,2)=dcmplx(tmp3(IR)*dgrad(IR,5),tmp3(IR)*dgrad(IR,6))
        enddo
        call FWFFT(V(1,2))
        call FFTMULT(V(1,2),VTEMP,2,3)

        call AZZERO(V(1,2),2*MAXFFT)
        do IR=1,NNR1
           V(IR,2)=dcmplx(tmp4(IR)*grad(IR,2),tmp4(IR)*grad(IR,3))
        enddo
        call FWFFT(V(1,2))
        call FFTMULT(V(1,2),VTEMP,1,2)

        call azzero(V(1,2),2*MAXFFT)
        do IR=1,NNR1
           V(IR,2)=dcmplx(tmp4(IR)*grad(IR,4),tmp5(IR)*dgrad(IR,1))
        enddo
        call FWFFT(V(1,2))
        call FFTMULT(V(1,2),VTEMP,3,1)

        call azzero(V(1,2),2*MAXFFT)
        do IR=1,NNR1
           V(IR,2)=dcmplx(tmp5(IR)*dgrad(IR,2),tmp5(IR)*dgrad(IR,3))
        enddo
        call FWFFT(V(1,2))
        call FFTMULT(V(1,2),VTEMP,2,3)

        call azzero(V(1,2),2*MAXFFT)
        do IG=1,NHG
           V(NZH(IG),2) = VTEMP(IG)
           V(INDZ(IG),2) = DCONJG(VTEMP(IG))
        enddo
        call INVFFT(V(1,2))
        
        do IR=1,NNR1
           V(IR,2)=V(IR,2)+dcmplx(tmp1(IR),0.D0)
        enddo
      endif
c     ==--------------------------------------------------------------==
 99   continue
      if(TLSD) then
         do IR=1,NNR1
            rhoe(IR,1)=rhoe(IR,1)+rhoe(IR,2)
            drhoe(IR,1)=drhoe(IR,1)+drhoe(IR,2)
         enddo
      endif
      do ISPIN=1,NLSD
        do IR=1,NNR1
          DXC(IR,ISPIN) = DREAL(V(IR,ISPIN))
        enddo
      enddo
      if(TGC) call SCRREL(scr,lscr,debug)
c     ==--------------------------------------------------------------==
      call TIHALT(' DD_XC_ANA',ISUB)
c     ==--------------------------------------------------------------==
      Return
      END
C     ==================================================================
      SUBROUTINE GRADEN_COMP(RHOE,V,GRAD,VTMP)
C     ==--------------------------------------------------------------==
C     ==   SMOOTHING OF THE DENSITY AND CALCULATION OF |nabla.RHO|    ==
C     ==   ON INPUT : RHOE : DENSITY IN REAL SPACE                    ==
C     ==              V    : UNDEFINED                                ==
C     ==              GRAD : UNDEFINED                                ==
C     ==              VTMP : UNDEFINED                                ==
C     ==   ON OUTPUT: RHOE : DENSITY IN REAL SPACE (SMOOTH)           ==
C     ==              V    : UNDEFINED                                ==
C     ==              GRAD : (GRADIENT OF RHO)^2 IN REAL SPACE        ==
C     ==              VTMP : DENSITY IN G SPACE (SMOOTH)              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
C     Arguments
      COMPLEX*16 V(MAXFFT),VTMP(NHG)
      REAL*8     RHOE(NNR1),GRAD(NNR1,3)
C     Variables
      REAL*8     GMAX,GCS,SMFAC,EG
      INTEGER    ISUB,IR,IG
C     ==--------------------------------------------------------------==
C     ==  TRANSFORM DENSITY TO G SPACE                                ==
C     ==--------------------------------------------------------------==
      call azzero(V,2*MAXFFT)

      CALL TISET('GRADEN_COM',ISUB)
      DO IR=1,NNR1
        V(IR) = DCMPLX(RHOE(IR),0.0D0)
      ENDDO
      CALL FWFFT(V)
C     ==--------------------------------------------------------------==
C     ==  SMOOTHING                                                   ==
C     ==--------------------------------------------------------------==
      IF(TSMOOTH) THEN
        GMAX=HG(NHG)
#ifdef PARALLEL
        CALL GLOMAX(1,GMAX)
#endif
        GCS=SMF*GMAX
        DO IG=1,NHG
          EG=(HG(IG)-GCS)/(SDELTA*GMAX)
          SMFAC=1.0D0/(1.0D0+EXP(EG))
          VTMP(IG)=V(NZH(IG))*SMFAC
        ENDDO
      ELSE
        CALL ZGTHR(NHG,V,VTMP,NZH)
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  FFT OF RHO AND NABLA(X)*RHOE                                ==
C     ==--------------------------------------------------------------==
      CALL AZZERO(V,2*MAXFFT)
      DO IG=1,NHG
        V(NZH(IG))=VTMP(IG)-TPIBA*GK(1,IG)*VTMP(IG)
        V(INDZ(IG))=DCONJG(VTMP(IG)+TPIBA*GK(1,IG)*VTMP(IG))
      ENDDO
      CALL INVFFT(V)
      DO IR=1,NNR1
        RHOE(IR)=DREAL(V(IR))
        GRAD(IR,1)=DIMAG(V(IR))
      ENDDO
C     ==--------------------------------------------------------------==
C     ==  FFT OF NABLA(Y)*RHO AND NABLA(Z)*RHOE                       ==
C     ==--------------------------------------------------------------==
      CALL AZZERO(V,2*MAXFFT)
      DO IG=1,NHG
        V(NZH(IG))=TPIBA*(UIMAG*GK(2,IG)-GK(3,IG))*VTMP(IG)
        V(INDZ(IG))=TPIBA*(-UIMAG*GK(2,IG)+GK(3,IG))*DCONJG(VTMP(IG))
      ENDDO
      CALL INVFFT(V)
      DO IR=1,NNR1
        GRAD(IR,2)=DREAL(V(IR))
        GRAD(IR,3)=DIMAG(V(IR))
      ENDDO
      CALL TIHALT('GRADEN_COM',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      Subroutine GIVE_SCR_DD_XC_ANA(ldd_xc_ana,tag)
c     ==--------------------------------------------------------------==
      Implicit none
      Include 'system.h'
      Include 'spin.inc'
c     Arguments
      Integer ldd_xc_ana
      Character tag*30
c     ==--------------------------------------------------------------==
      if(.not.(TLSD.or.TLSE).and.TGC) then
      ldd_xc_ana=5*NNR1+3*NNR1-1*NNR1
      elseif(TLSD.and..not.TGC) then
      ldd_xc_ana=1
      elseif((TLSD.or.TLSE).and.TGC) then
      ldd_xc_ana=5*NNR1+6*NNR1-2*NNR1
      else
      ldd_xc_ana=1
      endif
c     ==--------------------------------------------------------------==
      Return
      End
c     =================================================================
      Subroutine FFTMULT(V,VTEMP,COORD1,COORD2)
      Implicit none
      Include 'system.h'
      Include 'cnst.inc'
      Include 'cppt.inc'
      Include 'fft.inc'
      Real*8 VFAC1,VFAC2
      Complex*16 V(MAXFFT),VTEMP(NHG),FA,FB,VG1,VG2
      Integer COORD1,COORD2,IG
c     ==--------------------------------------------------------------==
      do IG=1,NHG
         VFAC1=TPIBA*GK(COORD1,IG)
         VFAC2=TPIBA*GK(COORD2,IG)
         FA  = V(NZH(IG)) + V(INDZ(IG))
         FB  = V(NZH(IG)) - V(INDZ(IG))
         VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
         VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
         VTEMP(IG) = VTEMP(IG) - VFAC1*UIMAG*VG1 - VFAC2*UIMAG*VG2
      enddo
c     ==--------------------------------------------------------------==
      Return
      End
c     =================================================================     
