C     ==================================================================
      SUBROUTINE DDIPO(TAU0,C0,CM,C2,SC0,SCR,LSCR,NSTATE)
C     ==--------------------------------------------------------------==
C     == CALCULATES THE DIPOLE MOMENT                                 ==
C     == Modified (CPLS) to compute Wannier functions (August 1997)   ==
C     ==                                                              ==
C     == N.B. This version introduces the 'WANNIER_SD' subroutine for ==
C     ==      the SD procedure and allows to work with the Wannier    ==
C     ==      functions 'CWANN' using a PARALLEL MACHINE.             ==
C     == N.B. This version works for NON-CUBIC CELLS too !            ==
C     ==      Hence x,y,z must be interpreted as 1,2,3                ==
C     ==      IT GIVES NON-DISTORTED WANNIER CENTERS IN ALL THE       ==
C     ==      CASES (see 'CNC') :                                     ==
C     ==                          SIMPLE CUBIC (IBRAV=1)              ==
C     ==                          TETRAGONAL   (IBRAV=6)              ==
C     ==                          ORTHOROMBIC  (IBRAV=8,0)            ==
C     ==                          BCC          (IBRAV=3)              ==
C     ==                          FCC          (IBRAV=2)              ==
C     ==                          HEXAGONAL    (IBRAV=4)              ==
C     ==                          MONOCLINIC   (IBRAV=12)             ==
C     ==                          TRICLINIC    (IBRAV=14)             ==
C     ==                                                              ==
C     == N.B. In this version <i| exp(-Gx*x)*exp(-Gy*y) |i> is        ==
C     ==      computed BEFORE the unitary rotation (it is useful      ==
C     ==      for use with GENERAL NON-CUBIC CELLS) and then it is    ==
C     ==      rotated at the end of the calculation (see CNEW).       ==
C     ==                                                              ==
C     == 20/08/1998 Modified by Simone Raugei                         ==
C     ==            3d ---> 3f,g...                                   ==
C     ==                                                              ==
C     == 01/09/1998 Modified by Simone Raugei                         ==
C     ==            In MD, the WF are updated starting from the last  ==
C     ==            rotations sequence                                ==
C     ==                                                              ==
C     == 03/09/1998 Modified by Simone Raugei                         ==
C     ==            The subroutines were rearranged                   ==
C     ==                                                              ==
C     == 05/09/1998 Modified by Simone Raugei                         ==
C     ==            A new interface was  added                        ==
C     ==                                                              ==
C     == 10/09/1998 Modified by Simone Raugei                         ==
C     ==            Boero s modifications fo VPP were added           ==
C     ==                                                              ==
C     == 14/09/1998 Modified by Simone Raugei                         ==
C     ==            A different functional involving                  ==
C     ==            log(|<...>|^2) terms (Resta, Sorella) is used     ==
C     ==            (see wanlog) (P. L. Silvestrelli)                 ==
C     ==                                                              ==
C     == 26/09/1998 Modified by Simone Raugei                         ==
C     ==            In LSD calculations  the omega maximization is    ==
C     ==            performed in separately in alpha and beta         ==
C     ==            subspaces.                                        ==
C     ==                                                              ==
C     == 30/09/1998 New modifications for VPP-Fujitsu added           ==
C     ==            (M. Boero)                                        ==
C     ==                                                              ==
C     == 04/12/1998 Rewritten for Version 3.3 (J. Hutter)             ==
C     ==                                                              ==
C     == 01/07/1999 Arbitrary symmetry code for Wannier functions     ==
C     ==            using metric tensor                               ==
C     ==                                                              ==
C     == 04/10/1999 Fixed problem in G-vectors for the variable       ==
C     ==            cell case                                         ==
C     == 25/11/1999 Fixed dipole moment calculation                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      include 'geq0.inc'
      include 'ions.inc'
      include 'cnst.inc'
      include 'ddip.inc'
      include 'gvec.inc'
      include 'irat.inc'
      include 'rmas.inc'
      include 'wann.inc'
      include 'spin.inc'
      INCLUDE 'dipo.inc'  ! cmb
C input
      REAL*8 TAU0(3,NAX,*),SCR(*)
      COMPLEX*16 C0(NGW,*),C2(*),SC0(*),CM(*)
      INTEGER LSCR,NSTATE
C local
      COMPLEX*16 EPHASE1,EPHASE2,EPHASE3,DET,DD1,DD2,DD3,DDX
      INTEGER IP1,IP2,I,ISUB,LSCR2,IA,IS,NMCOL,N1
      INTEGER IX1,IX2,IXL,NXMAX,MSGLEN,LENGTH
      CHARACTER*30 TAG
      REAL*8 X0(3),P(3),PHASE1,PHASE2,PHASE3,OMEGON,D1,D2,D3
      REAL*8 TPI,FAC,COST,ZVTOT,TPZZ
      INTEGER MAPFUL(*),MAPCOL(*)
      POINTER (IP_MAPFUL,MAPFUL),(IP_MAPCOL,MAPCOL)
      SAVE IP_MAPFUL,IP_MAPCOL
C     ==--------------------------------------------------------------==
C..Wannier functions
      COMPLEX*16 XYZMAT(NSTATE,NSTATE,*)
      REAL*8     ROTMAT(NSTATE,NSTATE),ROTLSD(*)
      POINTER (IP_XYZMAT,XYZMAT),(IP_ROTMAT,ROTMAT),(IP_ROTLSD,ROTLSD)
      SAVE IP_XYZMAT,IP_ROTMAT,IP_ROTLSD
C     ==--------------------------------------------------------------==
      SAVE IFIRST
      INTEGER IFIRST
      DATA IFIRST /0/
C     ==--------------------------------------------------------------==
      CALL TISET('     DDIPO',ISUB)
      CALL GIVE_SCR_DDIPO(LENGTH,TAG)
      CALL TEST_SCR('DDIPO','LENGTH',LSCR,LENGTH)
C..initialization on first call to subroutine
      IF(IFIRST.EQ.0) THEN
        N1=0
        DO I=0,NPROC-1
          N1=MAX(N1,SPARM(3,I))
        ENDDO
        NGWMAX=N1
        CALL MEMORY(IP_MAPFUL,2*(NGWS/IRAT+1),'MAPFUL')
#ifdef PARALLEL
        NMCOL=(NPROC*NGWMAX)/IRAT+1
#else
        NMCOL=1
#endif
        CALL MEMORY(IP_MAPCOL,NMCOL,'MAPCOL')
        CALL SETDIP(MAPFUL,MAPCOL,SCR,LSCR)
C..initialization for Wannier function part
        IF(TWANN) THEN 
          CALL SET_OPERATOR(.TRUE.)
          CALL MEMORY(IP_XYZMAT,2*NSTATE*NSTATE*NWANOPT,'XYZMAT')
          CALL ZAZZERO(XYZMAT,NSTATE*NSTATE*NWANOPT)
          CALL MEMORY(IP_ROTMAT,NSTATE*NSTATE,'ROTMAT')
          IF(TLSD) THEN
            NXMAX=MAX(NSUP,NSDOWN)
            CALL MEMORY(IP_ROTLSD,NXMAX*NXMAX,'ROTLSD')
          ELSEIF(TLSE) THEN
            NXMAX=NSTATE-2
            CALL MEMORY(IP_ROTLSD,NXMAX*NXMAX,'ROTLSD')
          ENDIF
        ENDIF
        IFIRST=1
      ENDIF
      IF(TLSD) THEN
        FAC=1.D0
      ELSEIF(TLSE) THEN
        FAC=1.D0
      ELSE
        FAC=2.D0
      ENDIF
      OMEGON=1.0D0/OMEGA
C..ionic contribution
C..center of charge
      P(1)=0.D0
      P(2)=0.D0
      P(3)=0.D0
      X0(1)=0.D0
      X0(2)=0.D0
      X0(3)=0.D0
      ZVTOT=0.D0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          X0(1)=X0(1)+ZV(IS)*TAU0(1,IA,IS)
          X0(2)=X0(2)+ZV(IS)*TAU0(2,IA,IS)
          X0(3)=X0(3)+ZV(IS)*TAU0(3,IA,IS)
        ENDDO
        ZVTOT=ZVTOT+NA(IS)*ZV(IS)
      ENDDO
      X0(1)=X0(1)/ZVTOT
      X0(2)=X0(2)/ZVTOT 
      X0(3)=X0(3)/ZVTOT 
      DO IS=1,NSP
        DO IA=1,NA(IS)
          P(1)=P(1)+ZV(IS)*(TAU0(1,IA,IS)-X0(1))
          P(2)=P(2)+ZV(IS)*(TAU0(2,IA,IS)-X0(2))
          P(3)=P(3)+ZV(IS)*(TAU0(3,IA,IS)-X0(3))
        ENDDO
      ENDDO
      P(1)=P(1)*OMEGON
      P(2)=P(2)*OMEGON
      P(3)=P(3)*OMEGON
      IF(DABS(P(1)+P(2)+P(3)) .GT. 1.D-10) 
     &        CALL STOPGM("DDIPO|","REFERENCE POINT")
C..phase factors due to centor of charge motion
      TPZZ=TPIBA*NSTATE
      PHASE1=(B1(1)*X0(1)+B1(2)*X0(2)+B1(3)*X0(3))*TPZZ
      PHASE2=(B2(1)*X0(1)+B2(2)*X0(2)+B2(3)*X0(3))*TPZZ
      PHASE3=(B3(1)*X0(1)+B3(2)*X0(2)+B3(3)*X0(3))*TPZZ
      EPHASE1=DCMPLX(COS(PHASE1),SIN(PHASE1))
      EPHASE2=DCMPLX(COS(PHASE2),SIN(PHASE2))
      EPHASE3=DCMPLX(COS(PHASE3),SIN(PHASE3))
C..electronic contribution
      IP1=1
      IP2=1+2*NSTATE*NSTATE
      LSCR2=LSCR-IP2
      IF(LSCR2.LT.1) CALL STOPGM('DDIPO','NOT ENOUGH SCRATCH SPACE')
      CALL OPEIGR(C0,C2,SC0,NSTATE,MAPFUL,MAPCOL,SCR(IP1),
     *            SCR(IP2),LSCR2,1,0,DD1)
      IF(TWANN) CALL DCOPY(2*NSTATE*NSTATE,SCR(IP1),1,XYZMAT(1,1,1),1)
      DET=DD1*EPHASE1
      D1=DATAN(DIMAG(DET)/DREAL(DET))
      CALL OPEIGR(C0,C2,SC0,NSTATE,MAPFUL,MAPCOL,SCR(IP1),
     *            SCR(IP2),LSCR2,2,0,DD2)
      IF(TWANN) CALL DCOPY(2*NSTATE*NSTATE,SCR(IP1),1,XYZMAT(1,1,2),1)
      DET=DD2*EPHASE2
      D2=DATAN(DIMAG(DET)/DREAL(DET))
      CALL OPEIGR(C0,C2,SC0,NSTATE,MAPFUL,MAPCOL,SCR(IP1),
     *            SCR(IP2),LSCR2,3,0,DD3)
      IF(TWANN) CALL DCOPY(2*NSTATE*NSTATE,SCR(IP1),1,XYZMAT(1,1,3),1)
      DET=DD3*EPHASE3
      D3=DATAN(DIMAG(DET)/DREAL(DET))
C..Sum up dipoles
      TPI=2.D0*ACOS(-1.D0)
      COST=FAC*OMEGON/TPI
      DO I=1,3
        PDIPOLE(I)=COST*(D1*A1(I)+D2*A2(I)+D3*A3(I))
        PDIPOLT(I)=PDIPOLE(I) + P(I)
        DMOM(I)=OMEGA*PDIPOLT(I) ! cmb
      ENDDO
C     Wannier function calculation
      IF(TWANN) THEN
C       Additional operators
        DO I=4,NWANOPT
          CALL OPEIGR(C0,C2,SC0,NSTATE,MAPFUL,MAPCOL,SCR(IP1),
     *              SCR(IP2),LSCR2,IOW(1,I-3),IOW(2,I-3),DDX)
          CALL DCOPY(2*NSTATE*NSTATE,SCR(IP1),1,XYZMAT(1,1,I),1)
        ENDDO
C       Optimisers
        IF(W_OPT.EQ.1) THEN
          IF(PARENT) THEN
C           steepest descent
            IF(TLSD) THEN
              CALL AZZERO(ROTMAT,NSTATE*NSTATE)
              NXMAX=MAX(NSUP,NSDOWN)
              IX1=1
              IX2=IX1+NXMAX*NXMAX
              IXL=IX2+NXMAX*NXMAX
              LSCR2=LSCR-IXL
              CALL SD_WANNIER(ROTLSD,XYZMAT(1,1,1),NSTATE,NSUP,
     *                        SCR(IX1),SCR(IX2),SCR(IXL),LSCR2)
              CALL RMATMOV(NSUP,NSUP,ROTLSD,NSUP,ROTMAT,NSTATE)
              CALL SD_WANNIER(ROTLSD,XYZMAT(NSUP+1,NSUP+1,1),
     *                        NSTATE,NSDOWN,
     *                        SCR(IX1),SCR(IX2),SCR(IXL),LSCR2)
              CALL RMATMOV(NSDOWN,NSDOWN,ROTLSD,NSDOWN,
     *                     ROTMAT(NSUP+1,NSUP+1),NSTATE)
            ELSEIF(TLSE) THEN
              NXMAX=NSTATE-2
              IX1=1
              IX2=IX1+NXMAX*NXMAX
              IXL=IX2+NXMAX*NXMAX
              LSCR2=LSCR-IXL
              CALL SD_WANNIER(ROTLSD,XYZMAT,NSTATE,NXMAX,
     *                        SCR(IX1),SCR(IX2),SCR(IXL),LSCR2)
              CALL AZZERO(ROTMAT,NSTATE*NSTATE)
              CALL RMATMOV(NXMAX,NXMAX,ROTLSD,NXMAX,ROTMAT,NSTATE)
              ROTMAT(NXMAX+1,NXMAX+1)=1.D0
              ROTMAT(NSTATE,NSTATE)=1.D0
            ELSE
              IX1=1
              IX2=IX1+NSTATE*NSTATE
              IXL=IX2+NSTATE*NSTATE
              LSCR2=LSCR-IXL
              CALL SD_WANNIER(ROTMAT,XYZMAT,NSTATE,NSTATE,
     *                        SCR(IX1),SCR(IX2),SCR(IXL),LSCR2)
            ENDIF
          ENDIF
          MSGLEN=NSTATE*NSTATE * 8
          CALL MY_BCAST(ROTMAT,MSGLEN,SOURCE,ALLGRP)
        ELSEIF(W_OPT.EQ.2) THEN
C     
C           Jacobi rotations
          IF(TLSD) THEN
            CALL AZZERO(ROTMAT,NSTATE*NSTATE)
            NXMAX=MAX(NSUP,NSDOWN)
            CALL JROTATION(ROTLSD,XYZMAT(1,1,1),NSTATE,NSUP,SCR,LSCR)
            CALL RMATMOV(NSUP,NSUP,ROTLSD,NSUP,ROTMAT,NSTATE)
            CALL JROTATION(ROTLSD,XYZMAT(NSUP+1,NSUP+1,1),
     *                        NSTATE,NSDOWN,SCR,LSCR2)
            CALL RMATMOV(NSDOWN,NSDOWN,ROTLSD,NSDOWN,
     *                   ROTMAT(NSUP+1,NSUP+1),NSTATE)
          ELSEIF(TLSE) THEN
            NXMAX=NSTATE-2
            CALL JROTATION(ROTLSD,XYZMAT,NSTATE,NXMAX,SCR,LSCR2)
            CALL AZZERO(ROTMAT,NSTATE*NSTATE)
            CALL RMATMOV(NXMAX,NXMAX,ROTLSD,NXMAX,ROTMAT,NSTATE)
            ROTMAT(NXMAX+1,NXMAX+1)=1.D0
            ROTMAT(NSTATE,NSTATE)=1.D0
          ELSE
            IX1=1
            IX2=IX1+NSTATE*NSTATE
            LSCR2=LSCR-IX2
            CALL JROTATION(ROTMAT,XYZMAT,NSTATE,NSTATE,SCR,LSCR2)
          ENDIF
        ELSE
          CALL STOPGM('DDIPO','not implemented')
        ENDIF
C       reset randomization variable (only randomize once)
        W_RAN=0.D0
C       rotate the wavefunctions and velocities 
C       to the Wannier representation
        CALL ROTATE(1.D0,C0,0.D0,C2,ROTMAT,NSTATE,
     *              2*NGW,TLSD,NSUP,NSDOWN)
        CALL DCOPY(2*NGW*NSTATE,C2,1,C0,1)
        CALL ROTATE(1.D0,CM,0.D0,C2,ROTMAT,NSTATE,
     *              2*NGW,TLSD,NSUP,NSDOWN)
        CALL DCOPY(2*NGW*NSTATE,C2,1,CM,1)
C       calculate the centers and spread of the wannier functions
        CALL WANNIER_CENTER(XYZMAT,NSTATE,NSTATE,SCR,TAU0)
      ENDIF
      CALL TIHALT('     DDIPO',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_DDIPO(LDDIPO,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        
      INCLUDE 'elct.inc'       
      INCLUDE 'wann.inc'       
      INCLUDE 'spin.inc'       
C     Arguments
      INTEGER   LDDIPO
      CHARACTER TAG*30
C     Variables
      INTEGER   NSTATE,LSETDIP,LOPEIGR,LWANN,LSELF,NMAX,LWANC,LFORCEDR
C     ==--------------------------------------------------------------==
      NSTATE=N
      NMAX=NSTATE
      IF(TLSD) NMAX=MAX(NSUP,NSDOWN)
      LSELF=2*NSTATE*NSTATE
      LWANC=4*NSTATE
      CALL GIVE_SCR_OPEIGR(LOPEIGR,TAG,NSTATE)
      LWANN=0
      LFORCEDR=0
      IF(TWANN)THEN
        IF(NWANOPT.EQ.0) CALL SET_OPERATOR(.FALSE.)
        IF(W_OPT.EQ.1) THEN
          CALL GIVE_SCR_SDWANN(LWANN,TAG,NMAX)
        ELSEIF(W_OPT.EQ.2) THEN
          CALL GIVE_SCR_JROT(LWANN,TAG,NMAX,.FALSE.)
        ENDIF
        IF (TWMOL.OR.TWDOS) THEN
           CALL GIVE_SCR_FORCEDR(LFORCEDR,TAG,N,.false.,.true.)
        ENDIF
        LWANN=MAX(LWANN,LFORCEDR)
      ENDIF
      LSETDIP=NGWS/2+1
      LDDIPO=MAX(LSETDIP,LSELF+LOPEIGR,LSELF+LWANN,LSELF+LWANC)+100
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SETDIP(MAPFUL,MAPCOL,ISCR,LSCR)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'gvec.inc'         
      INCLUDE 'irat.inc'
      INCLUDE 'ddip.inc'
      INCLUDE 'prcp.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'sphe.inc'
C input
      INTEGER MAPFUL(2,*),MAPCOL(NGWMAX,*),ISCR(*),LSCR
C local
      INTEGER INY(3,*)
      REAL*8 HGT(*),G2,t1,t2,t3,OMEGAN,GKPWF
      REAL*8 aux1(3),aux2(3),aux3(3),bax1(3),bax2(3),bax3(3)
      POINTER (IP_INY,INY),(IP_HGT,HGT)
      INTEGER I,J,K,I1,I2,I3,J1,J2,J3,IG,KMIN,KMAX,IT,
     *        NGG1H,NGG2H,NGG3H,ICURR,ISUB
#ifdef PARALLEL
      INTEGER BLKLEN
#endif
#ifdef __VECTOR 
      INTEGER IJK,IESRI,IESRJ,IESRK,IALL
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('    SETDIP',ISUB)
      CALL MEMORY(IP_INY,3*(NGWS/IRAT+1),'INY')
      CALL MEMORY(IP_HGT,NGWS,'HGT')
      IF(NGWMAX.GT.2*LSCR) CALL STOPGM('SETDIP','SCRATCH TOO SMALL')
#ifdef PARALLEL
      CALL ICOPY(NGW,MAPGP,1,ISCR,1)
      CALL IAZZERO(ISCR(NGW+1),NGWMAX-NGW)
      BLKLEN=NGWMAX*(8/IRAT)
      CALL MY_CONCAT(ISCR,MAPCOL,BLKLEN,ALLGRP)
#endif
C..construction of g-vectors
      IF(TPRCP.OR.TPRES) THEN     ! variable cell case or stress tensor
        CALL LATGEN(IBRAV,CELLRF,aux1,aux2,aux3,OMEGAN)
        CALL RECIPS(ALAT,aux1,aux2,aux3,bax1,bax2,bax3)
      ELSE                        ! fixed cell case
        bax1(1)=B1(1)
        bax1(2)=B1(2)
        bax1(3)=B1(3)
        bax2(1)=B2(1)
        bax2(2)=B2(2)
        bax2(3)=B2(3)
        bax3(1)=B3(1)
        bax3(2)=B3(2)
        bax3(3)=B3(3)
      ENDIF
#ifdef __VECTOR
      IESRI=(2*NR3S-1)
      IESRJ=(2*NR2S-1)*IESRI
      IESRK=(NR1S-1)*IESRJ
      IALL=IESRK
#endif
C     Maximum K point amplitude (Kmax) in Brillouin zone.
      GCUTKA=GKPWF(Bax1,Bax2,Bax3,TKPNT.AND.TSPHERE)
      IF(GCUTKA.EQ.0.D0) THEN
        GCUTWMIN=GCUTW
        GCUTWMAX=GCUTW
      ELSE
C       For each k and each G, |k+G|^2 < GCUTW
C       so we define GCUTWMIN and GCUTWMAX as:
C       GCUTWMIN < |G|^2 < GCUTWMAX we have to apply a mask.
        GCUTWMIN = MAX(0.D0,DSQRT(GCUTW)-DSQRT(GCUTKA))**2
        GCUTWMAX = (DSQRT(GCUTW)+DSQRT(GCUTKA))**2
      ENDIF
      IG=0
Cmb...I=0 
      DO J=0,NR2S-1
        KMIN=-NR3S+1
        KMAX=NR3S-1
        IF(J.EQ.0) KMIN=0
        DO K=KMIN,KMAX
          t1=DBLE(J)*bax2(1)+DBLE(K)*bax3(1)
          t2=DBLE(J)*bax2(2)+DBLE(K)*bax3(2)
          t3=DBLE(J)*bax2(3)+DBLE(K)*bax3(3)
          G2=t1*t1+t2*t2+t3*t3
          IF(G2.LT.GCUTWMAX) THEN
            IG=IG+1
            HGT(IG)=G2-DSQRT(DBLE(IG-1))*EPSG
            INY(1,IG)=0
            INY(2,IG)=J
            INY(3,IG)=K
          ENDIF
        ENDDO
      ENDDO
Cmb...I<>0
#ifdef __VECTOR 
      DO IJK=1,IALL
        I=1+INT((IJK-1)/IESRJ)
        J=1-NR2S+INT(MOD(IJK-1,IESRJ)/IESRI)
        K=1-NR3S+MOD(IJK-1,IESRI)
        t1=DBLE(I)*bax1(1)+DBLE(J)*bax2(1)+DBLE(K)*bax3(1)
        t2=DBLE(I)*bax1(2)+DBLE(J)*bax2(2)+DBLE(K)*bax3(2)
        t3=DBLE(I)*bax1(3)+DBLE(J)*bax2(3)+DBLE(K)*bax3(3)
        G2=t1*t1+t2*t2+t3*t3
        IF(G2.LT.GCUTWMAX) THEN
          IG=IG+1
          HGT(IG)=G2-DSQRT(DBLE(IG-1))*EPSG
          INY(1,IG)=I
          INY(2,IG)=J
          INY(3,IG)=K
         ENDIF
      ENDDO
#else 
      DO I=1,NR1S-1
        DO J=-NR2S+1,NR2S-1
          DO K=-NR3S+1,NR3S-1
            t1=DBLE(I)*bax1(1)+DBLE(J)*bax2(1)+DBLE(K)*bax3(1)
            t2=DBLE(I)*bax1(2)+DBLE(J)*bax2(2)+DBLE(K)*bax3(2)
            t3=DBLE(I)*bax1(3)+DBLE(J)*bax2(3)+DBLE(K)*bax3(3)
            G2=t1*t1+t2*t2+t3*t3
            IF(G2.LT.GCUTWMAX) THEN
              IG=IG+1
              HGT(IG)=G2-DSQRT(DBLE(IG-1))*EPSG
              INY(1,IG)=I
              INY(2,IG)=J
              INY(3,IG)=K
            ENDIF
          ENDDO
        ENDDO
      ENDDO
#endif
      IF(IG.NE.NGWS) CALL STOPGM('SETDIP',
     *                           'INCONSISTENT NUMBER OF PLANE WAVES')
      CALL SORT2(HGT,NGWS,ISCR)
      DO IG=1,NGWS-1
        ICURR=IG
   30   IF(ISCR(ICURR).NE.IG) THEN
          IT=INY(1,ICURR)
          INY(1,ICURR)=INY(1,ISCR(ICURR))
          INY(1,ISCR(ICURR))=IT
          IT=INY(2,ICURR)
          INY(2,ICURR)=INY(2,ISCR(ICURR))
          INY(2,ISCR(ICURR))=IT
          IT=INY(3,ICURR)
          INY(3,ICURR)=INY(3,ISCR(ICURR))
          INY(3,ISCR(ICURR))=IT
          IT=ICURR
          ICURR=ISCR(ICURR)
          ISCR(IT)=IT
          IF(ISCR(ICURR).EQ.IG) THEN
            ISCR(ICURR)=ICURR
            GOTO 25
          ENDIF
         GOTO 30
        ENDIF
   25   CONTINUE
      ENDDO
      NGG1=0
      NGG2=0
      NGG3=0
#ifdef __SR8000
*poption parallel, tlocal(IG), max(NGG1,NGG2,NGG3)
#endif
!$OMP parallel do private(IG)
      DO IG=1,NGWS
        NGG1=MAX(ABS(INY(1,IG)),NGG1)
        NGG2=MAX(ABS(INY(2,IG)),NGG2)
        NGG3=MAX(ABS(INY(3,IG)),NGG3)
      ENDDO
      NGG1H=NGG1+2
      NGG2H=NGG2+2
      NGG3H=NGG3+2
      NGG1=2*(NGG1+1)+1
      NGG2=2*(NGG2+1)+1
      NGG3=2*(NGG3+1)+1
#ifdef __SR8000
*poption parallel, tlocal(IG,I1,I2,I3,J1,J2,J3)
#endif
!$OMP parallel do private(IG,I1,I2,I3,J1,J2,J3)
      DO IG=1,NGWS
        I1=INY(1,IG)+NGG1H
        I2=INY(2,IG)+NGG2H
        I3=INY(3,IG)+NGG3H
        J1=-INY(1,IG)+NGG1H
        J2=-INY(2,IG)+NGG2H
        J3=-INY(3,IG)+NGG3H
        MAPFUL(1,IG)=I1+I2*NGG1+I3*(NGG1*NGG2)
        MAPFUL(2,IG)=J1+J2*NGG1+J3*(NGG1*NGG2)
      ENDDO
      CALL FREEM(IP_HGT)
      CALL FREEM(IP_INY)
      CALL TIHALT('    SETDIP',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SET_OPERATOR(TPRI)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'wann.inc'         
      INCLUDE 'metr.inc'         
C     Arguments
      LOGICAL TPRI
C     Local Variables
      REAL*8 METRIC(3,3)
      INTEGER I,K,L
      CHARACTER*2 XYZ(1:3)
      DATA XYZ /' X',' Y',' Z'/
C     ==--------------------------------------------------------------==
      CALL DGEMM('N','T',3,3,3,1.D0,HT,3,HT,3,0.D0,METRIC,3)
      CALL IAZZERO(IOW,6)
      CALL AZZERO(WWEI,6)
      NWANOPT=6

      WWEI(1)=METRIC(1,1)-METRIC(1,2)-METRIC(1,3)
      WWEI(2)=METRIC(2,2)-METRIC(1,2)-METRIC(2,3)
      WWEI(3)=METRIC(3,3)-METRIC(1,3)-METRIC(2,3)
      WWEI(4)=METRIC(1,2)
      WWEI(5)=METRIC(1,3)
      WWEI(6)=METRIC(2,3)
      IOW(1,1)=1
      IOW(2,1)=2
      IOW(1,2)=1
      IOW(2,2)=3
      IOW(1,3)=2
      IOW(2,3)=3

CRV----quadrupole------------
c xx
      DO K=1,3
         DO L=1,3
            METRIC(K,L)=HT(K,1)*HT(L,1)
         ENDDO
      ENDDO
      WQUADI(1,1)=METRIC(1,1)-METRIC(1,2)-METRIC(1,3)
      WQUADI(1,2)=METRIC(2,2)-METRIC(1,2)-METRIC(2,3)
      WQUADI(1,3)=METRIC(3,3)-METRIC(1,3)-METRIC(2,3)
      WQUADI(1,4)=METRIC(1,2)
      WQUADI(1,5)=METRIC(1,3)
      WQUADI(1,6)=METRIC(2,3)

c xy 

      DO K=1,3
         DO L=1,3
            METRIC(K,L)=0.5D0*(HT(K,1)*HT(L,2)+HT(K,2)*HT(L,1))
         ENDDO
      ENDDO
      WQUADI(2,1)=METRIC(1,1)-METRIC(1,2)-METRIC(1,3)
      WQUADI(2,2)=METRIC(2,2)-METRIC(1,2)-METRIC(2,3)
      WQUADI(2,3)=METRIC(3,3)-METRIC(1,3)-METRIC(2,3)
      WQUADI(2,4)=METRIC(1,2)
      WQUADI(2,5)=METRIC(1,3)
      WQUADI(2,6)=METRIC(2,3)

c xz

      DO K=1,3
         DO L=1,3
            METRIC(K,L)=0.5D0*(HT(K,1)*HT(L,3)+HT(K,3)*HT(L,1))
         ENDDO
      ENDDO
      WQUADI(3,1)=METRIC(1,1)-METRIC(1,2)-METRIC(1,3)
      WQUADI(3,2)=METRIC(2,2)-METRIC(1,2)-METRIC(2,3)
      WQUADI(3,3)=METRIC(3,3)-METRIC(1,3)-METRIC(2,3)
      WQUADI(3,4)=METRIC(1,2)
      WQUADI(3,5)=METRIC(1,3)
      WQUADI(3,6)=METRIC(2,3)

c yy
      DO K=1,3
         DO L=1,3
            METRIC(K,L)=HT(K,2)*HT(L,2)
         ENDDO
      ENDDO
      WQUADI(4,1)=METRIC(1,1)-METRIC(1,2)-METRIC(1,3)
      WQUADI(4,2)=METRIC(2,2)-METRIC(1,2)-METRIC(2,3)
      WQUADI(4,3)=METRIC(3,3)-METRIC(1,3)-METRIC(2,3)
      WQUADI(4,4)=METRIC(1,2)
      WQUADI(4,5)=METRIC(1,3)
      WQUADI(4,6)=METRIC(2,3)

c yz

      DO K=1,3
         DO L=1,3
            METRIC(K,L)=0.5D0*(HT(K,2)*HT(L,3)+HT(K,3)*HT(L,2))
         ENDDO
      ENDDO
      WQUADI(5,1)=METRIC(1,1)-METRIC(1,2)-METRIC(1,3)
      WQUADI(5,2)=METRIC(2,2)-METRIC(1,2)-METRIC(2,3)
      WQUADI(5,3)=METRIC(3,3)-METRIC(1,3)-METRIC(2,3)
      WQUADI(5,4)=METRIC(1,2)
      WQUADI(5,5)=METRIC(1,3)
      WQUADI(5,6)=METRIC(2,3)

c zz
      DO K=1,3
         DO L=1,3
            METRIC(K,L)=HT(K,3)*HT(L,3)
         ENDDO
      ENDDO
      WQUADI(6,1)=METRIC(1,1)-METRIC(1,2)-METRIC(1,3)
      WQUADI(6,2)=METRIC(2,2)-METRIC(1,2)-METRIC(2,3)
      WQUADI(6,3)=METRIC(3,3)-METRIC(1,3)-METRIC(2,3)
      WQUADI(6,4)=METRIC(1,2)
      WQUADI(6,5)=METRIC(1,3)
      WQUADI(6,6)=METRIC(2,3)
CRV----quadrupole------------

      IF(PARENT.AND.TPRI) THEN
        DO I=1,3
          WRITE(6,'(T9,A,T19,I3,T30,A,T40,A,T56,F10.4)') 
     *        ' OPERATOR:',I,XYZ(I),' WEIGHT/L^2=',WWEI(I)/ALAT**2
        ENDDO
        DO I=4,6
          WRITE(6,'(T9,A,T19,I3,T25,A,T30,A,T40,A,T56,F10.4)') 
     *        ' OPERATOR:',I,XYZ(IOW(1,I-3)),XYZ(IOW(2,I-3)),
     *        ' WEIGHT/L^2=',WWEI(I)/ALAT**2
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
