C     ==================================================================
      SUBROUTINE DENSRD(RG,FILIN)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cell.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'response_p.inc'
      INCLUDE 'fileopen.inc'
C     Arguments
      COMPLEX*16 RG(NHG)
      CHARACTER  FILIN*(*)
C     Variables
      INTEGER    LEN,IG,I
      INTEGER    NR1SR,NR2SR,NR3SR,NHGSR
      REAL*8     GCUTR,GCUTWR
      LOGICAL    FERROR
#if defined(CRAY) || defined(__NEC)
      COMPLEX*16 RGSX(*),RG2(*)
#else
      COMPLEX*8  RGSX(*),RG2(*)
#endif
      POINTER    (IP_RGSX,RGSX),(IP_RG2,RG2)
#if defined(PARALLEL)
      INTEGER    MAPW(*)
      POINTER    (IP_MAPW,MAPW)
      REAL*8     X1
      INTEGER    N1,NHGX,IPP,IP,MSGID,MSGLEN,NBYTES
#endif
C     ==--------------------------------------------------------------==
C     ==  READS THE DENSITY FROM FILE                                 ==
C     ==--------------------------------------------------------------==
      IF(PARENT) THEN
        LEN=2*NHGS/IRAT
      ELSE
        LEN=2*NHG/IRAT
      ENDIF
      CALL MEMORY(IP_RGSX,LEN,'RGSX')
      IF(PARENT) THEN
        FERROR=.FALSE.
        CALL FILEOPEN(12,FILIN,FO_OLD+FO_UFO,FERROR)
        IF(FERROR) GOTO 30
        READ(12,END=20,ERR=30) 
        READ(12,END=20,ERR=30) 
        READ(12,END=20,ERR=30) NR1SR,NR2SR,NR3SR
        READ(12,END=20,ERR=30) GCUTR,NHGSR,GCUTWR
        IF (NR1SR.ne.NR1S) CALL STOPGM('DENSRD','NUMBER OF PLANE WAVES')
        IF (NR2SR.ne.NR2S) CALL STOPGM('DENSRD','NUMBER OF PLANE WAVES')
        IF (NR3SR.ne.NR3S) CALL STOPGM('DENSRD','NUMBER OF PLANE WAVES')
        IF (NHGSR.ne.NHGS) CALL STOPGM('DENSRD','NUMBER OF PLANE WAVES')
        IF ((ABS(GCUTR-GCUT)+ABS(GCUTWR-GCUTW)).gt.1d-12)
     *       CALL STOPGM('DENSRD','PLANE WAVES CUTOFFS')
        READ(12,END=20,ERR=30) 
        READ(12,END=20,ERR=30) 
C       For Anatoles reference density: the number (!) of atoms in the
C       reference molecule may be different:
        if(.not.tdummyatom) then
         DO I=1,NAT
            READ(12,END=20,ERR=30) 
         ENDDO
        endif
        READ(12,END=20,ERR=30) (RGSX(IG),IG=1,NHGS)
        CALL FILECLOSE(12)
      ENDIF

#ifdef PARALLEL
      IF(PARENT) THEN
        X1=DBLE(NHGS)/DBLE(NPROC)*1.4D0
        N1=MIN(NHGS,NINT(X1))*2/IRAT
        CALL MEMORY(IP_RG2,N1,'RG2')
        NHGX=NINT(DBLE(NHG)*1.4D0)
        NHGX=MIN(NHGS,NHGX)
        NHGX=NHGX/IRAT + 1
        CALL MEMORY(IP_MAPW,NHGX,'MAPW')
      ENDIF
      DO IPP=1,NPROC
         IP=PGROUP(IPP)
         MSGID=IP
         IF (PARENT) THEN
            IF (IP.EQ.ME) THEN
               DO IG=1,NHG
                  RG(IG)=RGSX(MAPGP(IG))
               ENDDO
            ELSE
               MSGID=1
               MSGLEN = SPARM(1,IPP-1) * 8/IRAT
               CALL MY_BRECV(MAPW,MSGLEN,IP,MSGID,NBYTES)
               DO IG=1,SPARM(1,IPP-1)
                  RG2(IG)=RGSX(MAPW(IG))
               ENDDO
               MSGID=2
               MSGLEN = SPARM(1,IPP-1) * 8
               CALL MY_BSEND(RG2,MSGLEN,IP,MSGID)
            ENDIF
         ELSE
            IF (IP.EQ.ME) THEN
               MSGID=1
               MSGLEN = NHG * 8/IRAT
               CALL MY_BSEND(MAPGP,MSGLEN,PGROUP(1),MSGID)
               MSGID=2
               MSGLEN = NHG * 8
               CALL MY_BRECV(RGSX,MSGLEN,PGROUP(1),MSGID,NBYTES)
               DO IG=1,NHG
                  RG(IG)=RGSX(IG)
               ENDDO
            ENDIF
         ENDIF
         CALL MY_SYNC(ALLGRP)
      ENDDO
#else
      DO IG=1,NHG
        RG(IG)=RGSX(IG)
      ENDDO
#endif


      IF(PARENT) THEN
        WRITE(6,*) ' WAVEFUNCTION READ FROM FILE ',FILIN
#ifdef PARALLEL
        CALL FREEM(IP_RG2)
        CALL FREEM(IP_MAPW)
#endif
      ENDIF
      CALL FREEM(IP_RGSX)
C     ==--------------------------------------------------------------==
      RETURN
C     ==--------------------------------------------------------------==
 20   CONTINUE
      CALL STOPGM('DENSRD','END OF FILE')
 30   CONTINUE
      CALL STOPGM('DENSRD','READ ERROR')
      END
C     ==================================================================
