C     ==================================================================
      SUBROUTINE DIFRHO(C0,RHOE,PSI,SCR,LSCR,NSTATE)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==            ELECTRON DENSITY RHOE IN REAL SPACE               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'rswf.inc'
      INCLUDE 'ions.inc'
C     Arguments
      INTEGER    NSTATE,LSCR
      COMPLEX*16 C0(NGW,NSTATE),PSI(NNR1)
      REAL*8     RHOE(NNR1,NLSD),SCR(LSCR)
C     Variables
      DIMENSION F(NSTATE)
      COMPLEX*16 PSIRE(*)
      REAL*8     PSIX(*),RHOEG(*)
      POINTER    (IP_PSIRE,PSIRE),(IP_PSIX,PSIX),(IP_RHOEG,RHOEG)
      CHARACTER  TAG*30
      REAL*8     TFF,COEF3,COEF4,R1,R2,RSUMV
      REAL*8     DASUM
      EXTERNAL   DASUM
      INTEGER    ISUB,LEAD,LEADX,NNRX,IFFT,ID,NSTA,IB,I,IBB,IS1,IS2,IG,
     &           ISPIN1,ISPIN2,L,LDIFRHO
#ifdef PARALLEL
      INTEGER    NL2,IX1,IXP,IX,
     &           MSGLEN
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('    DIFRHO',ISUB)
      CALL GIVE_SCR_DIFRHO(LDIFRHO,TAG)
      CALL TEST_SCR('DIFRHO',TAG,LSCR,LDIFRHO)
#ifdef PARALLEL
      IF(NOGRP.EQ.1) THEN
        LEAD  = KR1S*NGRAYS
        LEADX = NNR1
        IFFT=1
        NNRX=NNR1
      ELSE
        LEAD  = KR1S*NGRM
        LEADX = KRX*KR2S*KR3S
        IFFT=2
        NNRX=KRX*KR2S*KR3S
      ENDIF
#else
      LEAD  = NNR1
      LEADX = NNR1
      NNRX=NNR1
      IFFT=1
#endif
      CALL GET_ADDR(IP_PSIRE,PSI(1))
      CALL GET_ADDR(IP_PSIX,PSI(1))
      CALL GET_ADDR(IP_RHOEG,RHOE(1,1))
C     Initialize
      CALL AZZERO(RHOEG,NLSD*LEADX)
C     Loop over the electronic states
      DO ID=1,NSTATE,2*NOGRP
        NSTA=MIN((NSTATE-ID+2)/2,NOGRP)
        CALL ZAZZERO(PSI,MAXFFT*NOGRP)
        TFF=DASUM(2*NSTA,F(ID),1)
        IF(TFF.GT.1.D-5) THEN
          DO IB=1,NSTA
            I=ID+2*(IB-1)
            IBB=(IB-1)*LEAD
            IS1=I
            IS2=I+1
            IF(IS2.GT.NSTATE) THEN
!CDIR NODEP
              DO IG=1,NGW
                PSI(NZHS(IG)+IBB)=C0(IG,IS1)
                PSI(INDZS(IG)+IBB)=DCONJG(C0(IG,IS1))
              ENDDO
            ELSE
!CDIR NODEP
              DO IG=1,NGW
                PSI(NZHS(IG)+IBB)=C0(IG,IS1)+UIMAG*C0(IG,IS2)
                PSI(INDZS(IG)+IBB)=DCONJG(C0(IG,IS1))+
     *                                     UIMAG*DCONJG(C0(IG,IS2))
              ENDDO
            ENDIF
            IF(GEQ0.AND.(IS2.GT.NSTATE)) THEN
              PSI(NZHS(1)+IBB)=C0(1,IS1)
            ELSEIF(GEQ0) THEN
              PSI(NZHS(1)+IBB)=C0(1,IS1)+UIMAG*C0(1,IS2)
            ENDIF
          ENDDO
C         Compute the charge density from the wave functions
C         in real space 
          IF(NOGRP.GT.1) THEN
            IS2=0
            DO IB=1,NSTA
              IF(NOLIST(IB).EQ.ME) THEN
                I=ID+2*(IB-1)
                IS1=I
                IS2=I+1
              ENDIF
            ENDDO
          ELSE
            IS1=ID
            IS2=ID+1
          ENDIF
          COEF3=F(IS1)/OMEGA
          IF(IS2.GT.NSTATE) THEN
            COEF4=0.0D0
          ELSE
            COEF4=F(IS2)/OMEGA
          ENDIF
          IF(IS2.EQ.0) THEN
            COEF3=0.0D0
            COEF4=0.0D0
          ENDIF
          IF(IFFT.EQ.1) THEN
            CALL S_INVFFT(PSI)
          ELSE
            CALL SG_INVFFT(PSI)
          ENDIF
          IF(TLSD) THEN
            ISPIN1=0
            ISPIN2=0
            IF(IS1.GT.NSUP) ISPIN1=LEADX
            IF(IS2.GT.NSUP) ISPIN2=LEADX
!$OMP parallel do private(L,R1,R2) shared(COEF3,COEF4)
            DO L=1,NNRX
              R1=DREAL(PSIRE(L))
              R2=DIMAG(PSIRE(L))
              RHOEG(L+ISPIN1)=RHOEG(L+ISPIN1)+COEF3*R1*R1
              RHOEG(L+ISPIN2)=RHOEG(L+ISPIN2)+COEF4*R2*R2
            ENDDO
          ELSE
!$OMP parallel do private(L,R1,R2) shared(COEF3,COEF4)
            DO L=1,NNRX
              R1=DREAL(PSIRE(L))
              R2=DIMAG(PSIRE(L))
              RHOEG(L)=RHOEG(L)+COEF3*R1*R1+COEF4*R2*R2
            ENDDO
          ENDIF
        ENDIF
C       End loop over the electronic states
      ENDDO
#ifdef PARALLEL
      IF(NOGRP.GT.1) THEN
C       Summation of density within orbital split
        NL2=NLINK(NOLIST(NOGRP))
        IX1=NRXPL(MEPOS,1)-NRXPL(NL2,1)
        MSGLEN= 8 * NNRX
        IF(TLSD) THEN
          CALL MY_COMBINE(RHOEG(1),PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE(1,1),NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,1),KR1)
          ENDDO
          CALL MY_COMBINE(RHOEG(LEADX+1),PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE(1,2),NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,2),KR1)
          ENDDO
        ELSE
          CALL MY_COMBINE(RHOEG,PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE,NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,1),KR1)
          ENDDO
        ENDIF
      ENDIF
#endif
C     CONTRIBUTION OF THE VANDERBILT PP TO RHOE
      IF(TIVAN) THEN
        IF(TLSD) THEN
C         ALPHA SPIN
          CALL RHOV(NSTATE,1,NSUP,RSUMV,PSI,SCR,LSCR)
!$OMP parallel do private(I)
          DO I=1,NNR1
            RHOE(I,1)=RHOE(I,1)+DREAL(PSIRE(I))
          ENDDO
C         BETA SPIN
          CALL RHOV(NSTATE,NSUP+1,NSTATE,RSUMV,PSI,SCR,LSCR)
!$OMP parallel do private(I)
          DO I=1,NNR1
            RHOE(I,2)=RHOE(I,2)+DREAL(PSIRE(I))
          ENDDO
        ELSE
          CALL RHOV(NSTATE,1,NSTATE,RSUMV,PSI,SCR,LSCR)
!$OMP parallel do private(I)
          DO I=1,NNR1
            RHOE(I,1)=RHOE(I,1)+DREAL(PSIRE(I))
          ENDDO
        ENDIF
      ENDIF
C     ALPHA+BETA DENSITY IN RHOE(*,1), BETA DENSITY IN RHOE(*,2)
      IF(TLSD) THEN
!$OMP parallel do private(I)
        DO I=1,NNR1
          RHOE(I,1) = RHOE(I,1) + RHOE(I,2)
        ENDDO
      ENDIF
      CALL TIHALT('    DIFRHO',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_DIFRHO(LDIFRHO,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !pslo.inc
      INCLUDE 'pslo.inc'        !TIVAN
C     Arguments
      INTEGER   LDIFRHO
      CHARACTER TAG*30
C     ==--------------------------------------------------------------==
      IF(TIVAN) THEN
        CALL GIVE_SCR_RHOV(LDIFRHO,TAG)
      ELSE
        LDIFRHO=0
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
