C     ==================================================================
      SUBROUTINE DIPO(TAU0,EIROP,V)
C     ==--------------------------------------------------------------==
C     == CALCULATES THE DIPOLE MOMENT                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      include 'dipo.inc'
      include 'geq0.inc'
      include 'ions.inc'
      include 'cppt.inc'
      include 'cnst.inc'
C     Arguments
      REAL*8     TAU0(3,NAX,NSP)
      COMPLEX*16 EIROP(NHG),V(NNR1)
C     Variables
      REAL*8     RRX,RRY,RRZ,ZZ,RMAX,DX,DY,DZ,RG,RHO,B2,XKR,DFAC,
     &           BESSL
      EXTERNAL   BESSL
      INTEGER    ISUB,IS,IA,I,IG1,ISH,IG
C     ==--------------------------------------------------------------==
      CALL TISET('      DIPO',ISUB)
C     Define the center of integration
      RRX=0.0D0
      RRY=0.0D0
      RRZ=0.0D0
      ZZ=0.0D0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          RRX=RRX+TAU0(1,IA,IS)*ZV(IS)
          RRY=RRY+TAU0(2,IA,IS)*ZV(IS)
          RRZ=RRZ+TAU0(3,IA,IS)*ZV(IS)
          ZZ=ZZ+ZV(IS)
        ENDDO
      ENDDO
      RRX=RRX/ZZ
      RRY=RRY/ZZ
      RRZ=RRZ/ZZ
CEHR[
      IF (TMDEH) THEN
C       compute dipole of the electron density alone"
        RRX=0.D0
        RRY=0.D0
        RRZ=0.D0
      ENDIF
      IF(PARENT.AND.(.NOT.CMPLX_WF)) WRITE (6,'(/,A,3F9.5,/)')
     $     ' CENTER OF INTEGRATION (CORE CHARGE): ', RRX, RRY, RRZ
CEHR]
C     Calculate the radius of the integrated sphere
      IF(IBRAV.EQ.1) THEN
        RMAX=0.5D0*ALAT
      ELSE IF(IBRAV.EQ.2) THEN
        RMAX=ALAT*0.5D0/DSQRT(2.D0)
      ELSE IF(IBRAV.EQ.8) THEN
        RMAX=0.5D0*OMEGA**(1.0D0/3.0D0)
      ELSE IF(IBRAV.GT.2) THEN
        CALL STOPGM('DIPO',' FIND THE INTEGRATION RADIUS YOURSELF')
      END IF
      DO I=1,3
        DMOM(I)=0.0D0
        DNUC(I)=0.0D0
      ENDDO
      IF(GEQ0) THEN
        IG1=2
      ELSE
        IG1=1
      ENDIF
      DO ISH=IG1,NHGL
        DX=0.0D0
        DY=0.0D0
        DZ=0.0D0
        DO IG=ISPTR(ISH),ISPTR(ISH+1)-1
          RG=(GK(1,IG)*RRX+GK(2,IG)*RRY+GK(3,IG)*RRZ)*TPIBA
          RHO=DIMAG((V(NZH(IG))+EIROP(IG))*EXP(UIMAG*RG))
          DX=DX+RHO*GK(1,IG)
          DY=DY+RHO*GK(2,IG)
          DZ=DZ+RHO*GK(3,IG)
        ENDDO
        XKR=RMAX*TPIBA*SQRT(GL(ISH))
        B2=BESSL(2,XKR)
        DMOM(1)=DMOM(1)+B2*DX/GL(ISH)
        DMOM(2)=DMOM(2)+B2*DY/GL(ISH)
        DMOM(3)=DMOM(3)+B2*DZ/GL(ISH)
      ENDDO
      DFAC=8.0D0*PI/TPIBA*RMAX**3
      DMOM(1)=DFAC*DMOM(1)
      DMOM(2)=DFAC*DMOM(2)
      DMOM(3)=DFAC*DMOM(3)
#ifdef PARALLEL
      CALL GLOSUM(3,DMOM)
#endif
      CALL TIHALT('      DIPO',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE RSDIPO(TAU0,EIROP,PSI,RHOR)
C     ==--------------------------------------------------------------==
C     == CALCULATES THE DIPOLE MOMENT                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      include 'dipo.inc'
      include 'geq0.inc'
      include 'ions.inc'
      include 'cppt.inc'
      include 'cnst.inc'
      include 'fft.inc'
C     Arguments
      REAL*8     TAU0(3,NAX,NSP),RHOR(KR1,KR2,KR3)
      COMPLEX*16 EIROP(NHG),PSI(MAXFFT)
C     Variables
      REAL*8     RR(3),ZZ,RHO,DX(3,3),X(3),SCL
      INTEGER    ISUB,IS,IA,I,IX,IY,IZ,NX0
C     ==--------------------------------------------------------------==
      CALL TISET('    RSDIPO',ISUB)
C     Define the center of integration
      DO I=1,3
        RR(I)=0.0D0
      ENDDO
      ZZ=0.0D0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          DO I=1,3
            RR(I)=RR(I)+TAU0(I,IA,IS)*ZV(IS)
          ENDDO
          ZZ=ZZ+ZV(IS)
        ENDDO
      ENDDO
      DO I=1,3
        RR(I)=RR(I)/ZZ
      ENDDO
      IF(PARENT) WRITE (6,'(/,A,3F9.5,/)')
     $     ' CENTER OF INTEGRATION (CORE CHARGE): ', (RR(I), I=1,3)
      CALL INVFFT(PSI)
C     Calculate the dipole moment in real space
      SCL=OMEGA/DBLE(NR1S*NR2S*NR3S)
      DO I=1,3
        DMOM(I)=0.0D0
        DNUC(I)=0.0D0
      ENDDO
      DO I=1,3
        DX(I,1)=A1(I)/NR1S
        DX(I,2)=A2(I)/NR2S
        DX(I,3)=A3(I)/NR3S
      ENDDO
#ifdef PARALLEL
      NX0=NRXPL(MEPOS,1)
#else
      NX0=1
#endif
      DO IZ=1,NR3
        DO IY=1,NR2
          DO I=1,3
            X(I)=(IZ-1)*DX(I,3) + (IY-1)*DX(I,2) + (NX0-1)*DX(I,1)
          ENDDO
          DO IX=1,NR1
            RHO= -RHOR(IX,IY,IZ)
            DO I=1,3
              DMOM(I)=DMOM(I) + (X(I)-RR(I))*RHO
              X(I)=X(I)+DX(I,1)
            ENDDO
          ENDDO
        ENDDO
      ENDDO
      CALL DSCAL(3,SCL,DMOM,1)
#ifdef PARALLEL
      CALL GLOSUM(3,DMOM)
#endif
      CALL TIHALT('    RSDIPO',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
