#ifdef NEW_ORTHO
C     ==================================================================
      SUBROUTINE DISORTHO(NSTATE,C0)
C     ==--------------------------------------------------------------==
C     ==     ORTHOGONALIZE A SET OF WAVEFUNCTIONS C0                  ==
C     ==     USING A DISTRIBUTED OVERLAP MATRIX ALGORITHM             ==
C     ==     Cache Optimized Blocked Grahm-Schmidt                    ==
C     ==     C.Bekas and A.Curioni Comp. Phys. Comm. (2010)           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'mpif.h'
      INCLUDE 'geq0.inc'
C
      LOGICAL DEBUG
      INTEGER NSTATE
      COMPLEX*16 C0(NGW,NSTATE)
      REAL*8  OPW(*), LOCAL_OPW(*), LOCAL_NRM, NRM, DDOT
      REAL*8  LOCALW(2*NGWK, *), LOCAL_H(*), GLOBAL_H(*)
      POINTER (IP_OPW, OPW), (IP_LOCAL_OPW, LOCAL_OPW)
      POINTER (IP_LOCALW, LOCALW), (IP_LOCAL_H, LOCAL_H)
      POINTER (IP_GLOBAL_H, GLOBAL_H)
      REAL*8 DREAL, DTEMP , GDTEMP 
      INTEGER NORB,NORBX, NORBX1, MSGLEN
      INTEGER ISUB, RANK, IERR, INDEX1, INDEX, INDEX2
C     ==--------------------------------------------------------------==
      CALL TISET('  DISORTHO',ISUB)
      DEBUG=.FALSE.
      IF(NSTATE.LE.0) RETURN
      CALL SET_ORBDIST(NSTATE,NSTBLK,NORBX)
      NORB=NWA12(MEPOS,2)-NWA12(MEPOS,1)+1
      NORB=MAX(0,NORB)
      NORBX1 = NORBX	
      IF (NORBX.GT.NSTATE) THEN
         CALL STOPGM('DISORTHO','BLOCK SIZE LARGER THAN # OF STATES')
      ENDIF
      DO INDEX = 1, NSTATE/NORBX
	 CALL MEMORY(IP_LOCALW, NORBX1*NGWK*2, 'LOCALW')
C        COPY LOCAL BLOCK TO ORTHOGONALIZE
	 CALL DCOPY(2*NGWK*NORBX1, C0(1, (INDEX-1)*NORBX+1), 1, 
     &              LOCALW(1,1), 1)
C        COMPUTE PROJECTIONS
         IF (INDEX.GT.1) THEN
            CALL MEMORY(IP_LOCAL_H, NORBX1*NORBX*(INDEX-1), 'LOCAL_H') 
            CALL MEMORY(IP_GLOBAL_H, NORBX1*NORBX*(INDEX-1), 'GLOBAL_H')
            CALL AZZERO(LOCAL_H(1), NORBX1*NORBX*(INDEX-1))  
            CALL DGEMM("T", "N", (INDEX-1)*NORBX, NORBX1, 2*NGWK, 2.0D0,
     &              C0(1,1), 2*NGWK, LOCALW(1,1), 2*NGWK, 0.0D0,
     &              LOCAL_H(1), NORBX*(INDEX-1))
            IF (GEQ0) THEN
               CALL DGER((INDEX-1)*NORBX, NORBX1, -1.0D0, C0(1,1), 
     &             2*NGWK, LOCALW(1,1), 2*NGWK, LOCAL_H(1),
     &             (INDEX-1)*NORBX)
            ENDIF
            MSGLEN = NORBX1*NORBX*(INDEX-1)*8
            CALL MY_COMBINE(LOCAL_H(1), GLOBAL_H(1),
     &                      MSGLEN, 1, ALLGRP)	
	    CALL DGEMM("N", "N", 2*NGWK, NORBX1, NORBX*(INDEX-1),
     &		        -1.0D0, C0(1,1),
     &                  2*NGWK, GLOBAL_H(1), NORBX*(INDEX-1), 
     &                  1.0D0, LOCALW(1,1), 2*NGWK)
            CALL FREEM(IP_LOCAL_H)
	    CALL FREEM(IP_GLOBAL_H)	
         ENDIF   
C        ORTHOGONALIZE W AND STORE IT BACK TO C0
         CALL MEMORY(IP_LOCAL_H, NORBX1*NORBX1, 'LOCAL_H')
         CALL MEMORY(IP_GLOBAL_H, NORBX1*NORBX1, 'GLOBAL_H')
         CALL AZZERO(LOCAL_H(1), NORBX1*NORBX1)
         CALL AZZERO(GLOBAL_H(1), NORBX1*NORBX1) 
         CALL DSYRK("U", "T", NORBX1, 2*NGWK, 2.D0, 
     &              LOCALW(1,1), 2*NGWK, 0.D0,
     &              LOCAL_H(1), NORBX1)
         IF (GEQ0) THEN
            CALL DGER(NORBX1, NORBX1, -1.0D0, LOCALW(1,1), 2*NGWK,
     &             LOCALW(1,1), 2*NGWK, LOCAL_H(1), NORBX1)
         ENDIF
         CALL DMATC("U", NORBX1, LOCAL_H(1), NORBX1)
         MSGLEN = NORBX1*NORBX1*8
         CALL MY_COMBINE(LOCAL_H(1), GLOBAL_H(1), 
     &                   MSGLEN, 1, ALLGRP)
C        CALCULATE CHOLESKY FACTORIZATION
         CALL DPOTRF("U", NORBX1, GLOBAL_H(1), NORBX1, IERR)
	 IF (IERR.LT.0) THEN
            IF (PARENT) PRINT *, "WRONG ENTRY IN CHOL:", -IERR 
            CALL STOPGM('DISORTHO','DPOTRF: WRONG ENTRY') 
         ENDIF
         IF (IERR.GT.0) THEN
C           LAST BLOCK IS LINEARLY DEPENDENT. USE MOD. GRAM SCHMIDT.
            CALL GSORTHO(LOCALW(1,1),NGWK,1,NORBX1)
            CALL DCOPY(2*NGWK*NORBX1, LOCALW(1,1), 1, 
     &                 C0(1,(INDEX-1)*NORBX+1), 1)
            GOTO 999
         ENDIF
         DO INDEX1=1, NORBX1*NORBX1
	    LOCAL_H(INDEX1) = 0D0
         ENDDO 
         DO INDEX1=1, NORBX1
            LOCAL_H((INDEX1-1)*NORBX1+INDEX1) = 1D0
         ENDDO
C        CALCULATE INVERSE OF CHOLESKY FACTOR BY SOLVING
         DO INDEX1=1, NORBX1
            CALL DTRSV("U", "N", "N", NORBX1, GLOBAL_H(1), NORBX1, 
     &             LOCAL_H(NORBX1*(INDEX1-1)+1), 1) 
         ENDDO
C        MULTIPLY BACK TO ORIGINAL MATRIX
         CALL DGEMM("N", "N", 2*NGWK, NORBX1, NORBX1, 1.0D0, 
     &               LOCALW(1,1), 2*NGWK, LOCAL_H(1), NORBX1, 0D0, 
     &               C0(1,(INDEX-1)*NORBX+1), 2*NGWK)
999      IF (INDEX.EQ.((NSTATE/NORBX)-1)) THEN 
            NORBX1 = NSTATE - INDEX*NORBX	
         ENDIF
         CALL FREEM(IP_GLOBAL_H)
         CALL FREEM(IP_LOCAL_H)
         CALL FREEM(IP_LOCALW)
      ENDDO
      CALL TIHALT('  DISORTHO',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#else
C     ==================================================================
      SUBROUTINE DISORTHO(NSTATE,C0,CSCR,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     ==     ORTHOGONALIZE A SET OF WAVEFUNCTIONS C0                  ==
C     ==     USING A DISTRIBUTED OVERLAP MATRIX ALGORITHM             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'ions.inc'
C
      LOGICAL DEBUG
      INTEGER NSTATE,LSCR
      COMPLEX*16 C0(NGW,NSTATE),CSCR(NGW,NSTATE)
      REAL*8 SCR(LSCR)
      INTEGER NORB,NORBX,MMX,MSGLEN,IP,NX,IPJ,NPJ,IPI,NPI
      INTEGER I,J,I1,J1,IJ,N1,N2,INFO,I2,MMX1
      REAL*8 SMAT(NSTATE,*),TMAT(NSTATE,*),RMAT(*)
      POINTER (IP_SMAT,SMAT),(IP_TMAT,TMAT),(IP_RMAT,RMAT)
C     ==--------------------------------------------------------------==
      DEBUG=.FALSE.
      IF(NSTATE.LE.0) RETURN
      CALL SET_ORBDIST(NSTATE,NSTBLK,NORBX)
      NORB=NWA12(MEPOS,2)-NWA12(MEPOS,1)+1
      NORB=MAX(0,NORB)
C..overlap matrix
#ifdef  _BGL 
      MMX=MAX(NSTATE*NORBX,NORBX*NORBX)
      MMX1=NSTATE*NORBX
      CALL SCRSET(SCR,LSCR,DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_SMAT,MMX,'SMAT',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_RMAT,MMX,'RMAT',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_TMAT,MMX1,'TMAT',DEBUG)
      CALL AZZERO(SMAT,NSTATE*NORB)
      CALL AZZERO(TMAT,NSTATE*NORB)
      CALL AZZERO(RMAT,NSTATE*NORB)
#else
      MMX=MAX(NSTATE*NORBX,NORBX*NORBX*NPROC)
      CALL SCRSET(SCR,LSCR,DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_SMAT,MMX,'SMAT',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_RMAT,MMX,'RMAT',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_TMAT,MMX,'TMAT',DEBUG)
      CALL AZZERO(SMAT,NSTATE*NORB)
      CALL AZZERO(RMAT,NSTATE*NORB)
#endif
      MSGLEN=NSTATE*NORBX * 8
      DO IP=0,NPROC-1
         NX=NWA12(IP,2)-NWA12(IP,1)+1
         CALL AZZERO(TMAT,NSTATE*NORBX)
         IF(NX.GT.0) THEN
           CALL OVLAP2(NGW,NSTATE,NX,TMAT,C0,C0(1,NWA12(IP,1)))
           CALL MY_COMBINE_SOURCE(TMAT,RMAT,MSGLEN,1,
     &                            PGROUP(IP+1),ALLGRP)
           IF(MEPOS.EQ.PGROUP(IP+1)) THEN
             CALL DCOPY(NSTATE*NORBX,RMAT,1,SMAT,1)
           ENDIF
         ENDIF
       ENDDO
C..Cholesky decomposition
      DO IPJ=0,NPROC-1
        NPJ=NWA12(IPJ,2)-NWA12(IPJ,1)+1
        J1=NWA12(IPJ,1)
        IF(NPJ.GT.0) THEN
          DO IPI=IPJ,NPROC-1
            NPI=NWA12(IPI,2)-NWA12(IPI,1)+1
            I1=NWA12(IPI,1)
            IF(NPI.GT.0) THEN
              IF(MEPOS.LT.IPJ .AND. NORB.GT.0) THEN
                CALL DGEMM('N','T',NPI,NORB,NPJ,-1.D0,SMAT(I1,1),NSTATE,
     *                     SMAT(J1,1),NSTATE,0.D0,RMAT,NPI)
              ELSE
                CALL AZZERO(RMAT,NPI*NPJ)
              ENDIF
              CALL GLOSUM(NPI*NPJ,RMAT)
              IF(MEPOS.EQ.IPJ) THEN
                DO J=1,NPJ
                  DO I=1,NPI
                    IJ=(J-1)*NPI+I
                    RMAT(IJ)=RMAT(IJ)+SMAT(I1+I-1,J)
                  ENDDO
                ENDDO
                IF(IPJ.EQ.IPI) THEN
                  CALL DPOTRF('L',NPI,RMAT,NPI,INFO)
                ELSE
                  CALL DTRSM('R','L','T','N',NPI,NPJ,1.D0,SMAT(J1,1),
     *                       NSTATE,RMAT,NPI)
                ENDIF
                DO J=1,NPJ
                  DO I=1,NPI
                    IJ=(J-1)*NPI+I
                    SMAT(I1+I-1,J)=RMAT(IJ)
                  ENDDO
                ENDDO
              ENDIF
              CALL MY_SYNC(ALLGRP)
            ENDIF
          ENDDO
        ENDIF
      ENDDO
C..Inversion of Cholesky-Matrix
      CALL AZZERO(TMAT,NSTATE*NORBX)
C....on site block
      DO I=NWA12(MEPOS,1),NWA12(MEPOS,2)
        I1=I-NWA12(MEPOS,1)+1
        TMAT(I,I1)=1.D0
      ENDDO
      I1=NWA12(MEPOS,1)
      IF(NORB.GT.0)
     *  CALL DTRTRS('L','N','N',NORB,NORB,SMAT(I1,1),NSTATE,
     *              TMAT(I1,1),NSTATE,INFO)
      IF(NWA12(MEPOS,2)+1.LE.NSTATE) THEN
        I2=NWA12(MEPOS,2)+1
        N2=NSTATE-NWA12(MEPOS,2)
        IF(NORB.GT.0)
     *    CALL DGEMM('N','N',N2,NORB,NORB,-1.D0,SMAT(I2,1),NSTATE,
     *               TMAT(I1,1),NSTATE,0.D0,TMAT(I2,1),NSTATE)
      ENDIF
C....other blocks
      MSGLEN=NSTATE*NORBX * 8
      CALL DCOPY(NSTATE*NORBX,SMAT,1,RMAT,1)
      DO IP=1,NPROC-1
        N1=NWA12(IP,2)-NWA12(IP,1)+1
        I1=NWA12(IP,1)
        IF(N1.GT.0) THEN
          CALL DCOPY(NSTATE*NORBX,RMAT,1,SMAT,1)
          CALL MY_BCAST(SMAT,MSGLEN,PGROUP(IP+1),ALLGRP)
        ENDIF
        IF(IP.GT.MEPOS) THEN
C......solve for block mepos+ip
          IF(N1.GT.0) THEN
            IF(NORB.GT.0)
     *        CALL DTRTRS('L','N','N',N1,NORB,SMAT(I1,1),NSTATE,
     *                    TMAT(I1,1),NSTATE,INFO)
            IF(NWA12(IP,2)+1.LE.NSTATE) THEN
              I2=NWA12(IP,2)+1
              N2=NSTATE-NWA12(IP,2)
              IF(NORB.GT.0)
     *          CALL DGEMM('N','N',N2,N1,NORB,-1.D0,SMAT(I2,1),NSTATE,
     *                     TMAT(I1,1),NSTATE,1.D0,TMAT(I2,1),NSTATE)
            ENDIF
          ENDIF
        ENDIF
      ENDDO
C..Transpose TMAT
      CALL TRANS_DA(TMAT,SMAT,RMAT,NSTATE,NWA12(0,1),NWA12(0,2),
     *              NORBX,MEPOS,NPROC,ALLGRP)
C..Rotate orbitals
      CALL ROTATE_DA(1.D0,C0,0.D0,CSCR,TMAT,SMAT,2*NGW,NSTATE,
     *        NWA12(0,1),NWA12(0,2),NORBX,MEPOS,PGROUP,NPROC,ALLGRP,
     *        .FALSE.,1,1)
      CALL DCOPY(2*NSTATE*NGW,CSCR,1,C0,1)
      CALL SCRREL(SCR,LSCR,DEBUG)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#endif
