C     ==================================================================
      SUBROUTINE DIST_FRIESNER(NDIAG,NEL,NCONV,NHPSI,NSTATE,
     &                    C0,CS,SC0,CSCR,CGS,FOCC,
     &                    VPOT,PSI,EDAV,SCR,LSCR,
     &                    JSPIN,TREFINE,TINFO)
C     ==--------------------------------------------------------------==
C     ==  Kohn-Sham Matrix Diagonalization by Krylov-Space Method     ==
C     ==  W.T. Pollard and R.A. Friesner                              ==
C     ==  J. Chem. Phys. 99, 6742 (1993)                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'irat.inc'
C     Arguments
      INTEGER    NDIAG,NCONV,NHPSI,LSCR,JSPIN,NSTATE
      COMPLEX*16 C0(NGW,*),CS(NGW,*),CGS(NGW,*),
     &           SC0(NGW,*),CSCR(NGW,*),PSI(*)
      REAL*8     VPOT(*),EDAV(*),SCR(*),NEL,FOCC(*)
      LOGICAL    TREFINE,TINFO
C     Variables
      REAL*8     EPS,EDAVMAX,EDAVMIN,BIG
      PARAMETER  (EPS=1.D-14,EDAVMAX=300D0,EDAVMIN=-300D0,BIG=1.D33)
      INTEGER    ISPIN
      PARAMETER  (ISPIN=1)
      INTEGER    INDEX0(NDIAG)
      REAL*8     ADAV(NDIAG,*),ALPHA0(NDIAG),BETA1(NDIAG)
      POINTER    (IP_INDEX0,INDEX0),
     &           (IP_ADAV,ADAV),(IP_ALPHA0,ALPHA0),(IP_BETA1,BETA1)
      REAL*8     TIM1,TIM2,TIM,TROTMIN,TROTMAX,B2MIN,B2MAX,
     &           B2,FAC,AUX,AMU,SIGN
      REAL*8     DOTP,DDOT,TIMEF
      EXTERNAL   DOTP,DDOT,TIMEF
      INTEGER    MY_MAXVAL
      EXTERNAL   MY_MAXVAL
      INTEGER    ISUB,NGW2,NCONVOLD,NHPSIOLD,NTEST,I,J,LFRIESNER,
     &           IERR,ITEST,ICYCLE,NCURR,IG,NLEFT,IAUX,
     &           LSCR2,LSCR3
      LOGICAL    TLSD2,TCONV,TCHECK,TTEST,TDEBUG
      CHARACTER  CHARSPIN*20,TAG*30
C     ==================================================================
C     DISTRIBUTED EIGENSOLVER VARIABLES
C     ==================================================================
      LOGICAL    DEBUG,CONV_FLAG,MEM_FLAG
      REAL*8     TMAT(*),RMAT(*)
      POINTER    (IP_TMAT,TMAT),(IP_RMAT,RMAT)
      INTEGER    NORB,NORBX,MSGLEN,IP,NX,IS
      CHARACTER  LABEL(1000)*15
      INTEGER    LC_INDEX(NSTATE)
C     LANCZOS BASIS AND VARIABLES
      REAL*8     LANV(*),W(*),PW(*),LOCAL_PW(*),V_PREV(*)
      REAL*8     VCUR(*),V0(*),BETA,Z(NDIAG,*),CONV_TOL
      REAL*8     E(NDIAG),D(NDIAG),SM,SM_OLD,XTMAT(NDIAG,*)
      POINTER    (IP_LANV, LANV),(IP_W, W),(IP_PW, PW)
      POINTER    (IP_LOCAL_PW, LOCAL_PW),(IP_V_PREV, V_PREV)
      POINTER    (IP_VCUR,VCUR),(IP_V0,V0),(IP_Z,Z)
      POINTER    (IP_XTMAT,XTMAT) 
      REAL*8     NRM,DNRM2,LOCAL_ALPHA,ALPHA,LOCAL_BETA,SCAL
      INTEGER    INDEX1,INDEX2,CHUNK,CHUNK_NEW,CHUNK_E
      INTEGER    INDEX3,START,START_COL,INT_SM
      INTEGER    CHUNK_BEGIN,CHUNK_END,INDEX4,LAN_MAX
      INTEGER    CHUNK_BEGIN_E,CHUNK_END_E
      INTEGER    CHUNK_NEW_E,STR,STRC
C     MPI_PARAMETERS
      INTEGER    RANK,SZ,SZ_OLD,NOEIG
      INTEGER    SEND_CNT(NPROC),SEND_DISPL(NPROC)
      INTEGER    WORLD, NEWGROUP,LANGRP,NOLAN,LANLIST(NPROC)
      INTEGER    SEND_CNT_E(NPROC),SEND_DISPL_E(NPROC)
      INTEGER    OLD_SEND_CNT_E(NPROC)
C     VARIABLES FOR THE CALCULATION OF EIGENVALUES
C     LAPACK ROUTINE: DSTEVX
      INTEGER    IL,IU,LDZ,NFOUND,NUM_EIG
      REAL*8     ABTOL,VL,VU
      INTEGER    IFAIL(NDIAG),IWORK(5*NDIAG)
      REAL*8     LD(NDIAG),LE(NDIAG)
      REAL*8     TEIG(NDIAG),WORK(5*NDIAG)
      REAL*8     E_TEMP(*),Z_TEMP(*),EIG_VEC(*),TEIG_VEC(*)
      POINTER    (IP_E_TEMP, E_TEMP),(IP_Z_TEMP, Z_TEMP)
      POINTER    (IP_EIG_VEC, EIG_VEC),(IP_TEIG_VEC, TEIG_VEC)
C     ==================================================================
      RANK     =  ME
      MEM_FLAG = .TRUE.
C     ==--------------------------------------------------------------==
C     With LSD and 1 electron...
      IF(NDIAG.EQ.0) RETURN
C     ==--------------------------------------------------------------==
      CALL TISET('  DIST_FRIESNER',ISUB)
#ifdef PARALLEL
C     ==--------------------------------------------------------------==
C     Test SCR
      CALL GIVE_SCR_DIST_FRIESNER(LFRIESNER,TAG,NDIAG)
      CALL TEST_SCR('DIST_FRIESNER',TAG,LSCR,LFRIESNER)
C     Allocation of SCR
#ifdef _BGL 
      CALL GET_ADDR(IP_BETA1,SCR(1))
      LSCR2=1+NDIAG+MOD(NDIAG,2)
      CALL GET_ADDR(IP_ALPHA0,SCR(LSCR2))
      LSCR2=LSCR2+NDIAG+MOD(NDIAG,2)
      CALL GET_ADDR(IP_ADAV,SCR(LSCR2))
      LSCR2=LSCR2+NDIAG+MOD(NDIAG*NDIAG,2)
      CALL GET_ADDR(IP_INDEX0,SCR(LSCR2))
      LSCR2=LSCR2+(NDIAG/IRAT+1)+MOD((NDIAG/IRAT+1),2)
      LSCR3=LSCR-LSCR2
#else
      LSCR2=LSCR-(NDIAG/IRAT+1)
      CALL GET_ADDR(IP_INDEX0,SCR(LSCR2+1))
      LSCR2=LSCR2-NDIAG*NDIAG
      CALL GET_ADDR(IP_ADAV,SCR(LSCR2+1))
      LSCR2=LSCR2-NDIAG
      CALL GET_ADDR(IP_ALPHA0,SCR(LSCR2+1))
      LSCR2=LSCR2-NDIAG
      CALL GET_ADDR(IP_BETA1,SCR(LSCR2+1))
#endif
C     ==--------------------------------------------------------------==
      TIM1=TIMEF()
      SIGN=-1.D0
      IF(TLSD) SIGN=-2.D0
      TLSD2=TLSD
      TLSD=.FALSE.
      TCONV=.FALSE.
      TDEBUG=.FALSE.
      CONV_FLAG=.FALSE.
      NGW2=NGW*2
      NCONV=0
      NCONVOLD=0
      NHPSIOLD=0
      IF(TTROT) THEN
        TROTMIN=DEXP(-EDAVMAX*BETAP)
        TROTMAX=DEXP(-EDAVMIN*BETAP)
      ENDIF
C     ==--------------------------------------------------------------==
      IF(PARENT.AND.TINFO) THEN
        IF(JSPIN.EQ.0) THEN
          CHARSPIN='<<<<<<<<<<<<<<<<<<<<'
        ELSEIF(JSPIN.EQ.1) THEN
          CHARSPIN='<<<<<<< ALPHA SPIN<<'
        ELSEIF(JSPIN.EQ.2) THEN
          CHARSPIN='<<<<<<<< BETA SPIN<<'
        ENDIF
        WRITE(6,'(/,1X,19("<"),A,A20)')
     &     ' LANCZOS DIAGONALIZATION ',CHARSPIN
      ENDIF
C     ==--------------------------------------------------------------==
C     ==================================================================
C     DEFINE BLOCKS FOR DISTRIBUTED LANCZOS EIGENSOLVER
C     ==================================================================
      CALL MY_SET_ORBDIST(NDIAG,NSTBLK,NORBX)
      NORB=NWA12(MEPOS,2)-NWA12(MEPOS,1)+1
      NORB=MAX(0,NORB)
      SZ = NPROC
C     ==================================================================
C     CREATE LANCZOS GROUP
C     ==================================================================
      IF (PARENT) THEN
         DO INDEX4=1,SZ
           LANLIST(INDEX4) = 0
         ENDDO
         NOLAN = 0
         DO INDEX4=1, SZ
            IF((NWA12(INDEX4-1,2)-NWA12(INDEX4-1,1)+1).NE.0) THEN
                LANLIST(INDEX4) = INDEX4-1
               NOLAN = NOLAN + 1
            ENDIF
         ENDDO
      ENDIF
      MSGLEN=SZ*8/IRAT
      CALL MY_BCAST(LANLIST, MSGLEN, SOURCE, ALLGRP)
      MSGLEN=8/IRAT
      CALL MY_BCAST(NOLAN, MSGLEN, SOURCE, ALLGRP)
      CALL MY_GROUP(NOLAN,LANLIST,LABEL,LANGRP,ALLGRP)
C     ==================================================================
C     Subspace Matrix
      TCHECK=.TRUE.
      NTEST=0
      DO WHILE(TCHECK)
        IF(TTROT) THEN
          CALL EHPSI(C0,CS,VPOT,PSI,NDIAG)
        ELSE
#ifdef _BGL
          CALL HPSI(C0,CS,SC0,VPOT,PSI,SCR(LSCR2),LSCR3,NDIAG,1,ISPIN)
#else
          CALL HPSI(C0,CS,SC0,VPOT,PSI,SCR,LSCR2,NDIAG,1,ISPIN)
#endif
          CALL HFXPSI(CGS,C0,CS,FOCC,SIGN,PSI,NSTATE,NDIAG)
        ENDIF
        NHPSI=NDIAG
C       ================================================================
C       DISTRIBUTED PROJECTED HAMILTONIAN IS CALCULATED HERE 
C       ================================================================
        MSGLEN=NDIAG*NORBX*8
        IF (MEM_FLAG) THEN 
          CALL MEMORY(IP_XTMAT,NDIAG*NORBX,'XTMAT')
          CALL MEMORY(IP_RMAT,NDIAG*NORBX,'RMAT')
          CALL MEMORY(IP_TMAT,NDIAG*NORBX,'TMAT') 
        ENDIF
        DO IP=0,NPROC-1
          NX=NWA12(IP,2)-NWA12(IP,1)+1
          CALL AZZERO(TMAT,NDIAG*NORBX)
          IF(NX.GT.0) THEN
            CALL OVLAP2(NGW,NDIAG,NX,TMAT,C0,CS(1,NWA12(IP,1)))
            CALL MY_COMBINE(TMAT,RMAT,MSGLEN,1,ALLGRP)
            IF(ME.EQ.IP) THEN
              CALL DCOPY(NDIAG*NORBX,RMAT(1),1,XTMAT(1,1),1)
            ENDIF
          ENDIF
        ENDDO
C       ================================================================
C       WE NEED TO SYMMETRIZE OVERLAP MATRIX
C       ================================================================
C        CALL SYMM_DA(XTMAT,RMAT,TMAT,NDIAG,
C     *              NWA12(0,1),NWA12(0,2),NORBX,MEPOS,NOLAN,LANGRP)
C        CALL MY_SYNC(ALLGRP)
C       ================================================================
C       LANCZOS PARALLEL EIGENSOLVER: DIAGONALIZE XTMAT
C       ONLY PROCESSORS THAT HAVE A NON-ZERO CHUNK OF THE RESTRICTED
C       HAMILTONIAN IN XTMAT PARTICIPATE IN LANCZOS
C       ================================================================
        IF ((NWA12(ME,2)-NWA12(ME,1)+1).NE.0) THEN
          SZ = 0
          CHUNK_NEW = NWA12(ME,2)-NWA12(ME,1)+1
          DO IP=0, NPROC-1
            IF ((NWA12(IP,2)-NWA12(IP,1)+1).NE.0) SZ = SZ+1
          ENDDO
          NOLAN = SZ
          LAN_MAX = NDIAG+1
C         ==================
C         ALLOCATE ONLY ONCE
C         ==================
          IF (MEM_FLAG) THEN
            CALL MEMORY(IP_VCUR,NDIAG,'VCUR')
            CALL MEMORY(IP_V0,NDIAG,'V0')
            CALL MEMORY(IP_LANV, CHUNK_NEW*LAN_MAX, 'LANV')
            CALL MEMORY(IP_W,CHUNK_NEW, 'W')
            CALL MEMORY(IP_PW, LAN_MAX, 'PW')
            CALL MEMORY(IP_LOCAL_PW, LAN_MAX, 'LOCAL_PW')
            CALL MEMORY(IP_V_PREV, CHUNK_NEW, 'V_PREV')
          ENDIF

          CALL AZZERO(PW(1),LAN_MAX)
          CALL AZZERO(LANV(1),CHUNK_NEW*LAN_MAX)
          CALL AZZERO(W(1),CHUNK_NEW)
C         DETERMINE SEND AND RECEIVE BUFFERS
          MSGLEN=8/IRAT
          CALL MY_CONCAT(CHUNK_NEW, SEND_CNT(1), MSGLEN, LANGRP)
          SEND_DISPL(1) = 0
          DO INDEX4=2, SZ
            SEND_DISPL(INDEX4)=SEND_DISPL(INDEX4-1)+SEND_CNT(INDEX4-1)
          ENDDO
C         DETERMINE A UNIT NORM INITIAL VECTOR V0
          IF (PARENT) THEN
            CALL DURAND(12344.0D0,NDIAG,V0) 
            NRM = DNRM2(NDIAG, V0(1), 1)
            NRM = 1.D0/NRM
            CALL DSCAL(NDIAG, NRM, V0(1), 1)
          ENDIF
C         BROADCAST INITIAL VECTOR TO ALL PROCESSORS IN LANCZOS GROUP
          MSGLEN=8*NDIAG
          CALL MY_BCAST(V0(1), MSGLEN, SOURCE, LANGRP)
          CALL DCOPY(NDIAG, V0(1), 1, VCUR(1), 1)
C         INITIALIZE V_PREV and BETA
          DO IS=1, CHUNK_NEW
            V_PREV(IS) = 0D0
            W(IS)=0D0
          ENDDO
          BETA = 0D0
          CHUNK_BEGIN = NWA12(ME,1)
          CHUNK_END   = NWA12(ME,2)
C         INITIALIZE FIRST LANCZOS VECTOR
          CALL DCOPY(CHUNK_NEW, VCUR(CHUNK_BEGIN), 1, LANV(1), 1)
C         INITIALIZE INITIAL SUM OF WANTED EIGENVALUES
          SM_OLD = 0D0
C         INITIALIZE CONVERGENCE TOLERANCE
          CONV_TOL = 1D-8
C         ==============================================================
C         LANCZOS LOOP
C         ==============================================================
          NUM_EIG = NDIAG
          DO INDEX4 = 1, LAN_MAX-1
C           DISTRIBUTED MATRIX-VECTOR PRODUCT
C           W: Holds the result
            CALL DGEMV("T", NDIAG,CHUNK_NEW,1D0,XTMAT(1,1),NDIAG,
     &                 VCUR(1), 1, 0D0, W(1), 1)
C           DAXPY: W = W - BETA*V_(J-1)
            CALL DAXPY(CHUNK_NEW, -BETA, V_PREV(1), 1, W(1), 1)
C           CALCULATE ALPHA: INNER PRODUCT WITH CURRENT LANCZOS VECTOR
            LOCAL_ALPHA = DDOT(CHUNK_NEW,W(1),1,
     &                         LANV((INDEX4-1)*CHUNK_NEW+1), 1)
            CALL MY_COMBINE(LOCAL_ALPHA, ALPHA, 8, 1, LANGRP)
C           DAXPY: W = W - ALPHA*V_J
            CALL DAXPY(CHUNK_NEW,-ALPHA,LANV((INDEX4-1)*CHUNK_NEW+1),1,
     &                 W(1),1)
C           ============================================================
C           REORTHOGONALIZE AGAINST ALL PREVIOUS BASIS VECTORS
C           ============================================================
            DO INDEX1=1,INDEX4
              LOCAL_PW(INDEX1)=DDOT(CHUNK_NEW,LANV((INDEX1-1)*
     &                              CHUNK_NEW+1),1, W(1), 1)
            ENDDO
            CALL MY_COMBINE(LOCAL_PW(1), PW(1), INDEX4*8, 1, LANGRP)
            DO INDEX1=1, INDEX4
              CALL DAXPY(CHUNK_NEW, -PW(INDEX1),
     &                   LANV((INDEX1-1)*CHUNK_NEW+1), 1, W(1), 1)
            ENDDO
C           ============================================================
C           CALCULATE BETA: NORM OF VECTOR W
            LOCAL_BETA = DDOT(CHUNK_NEW, W(1), 1, W(1), 1)
            CALL MY_COMBINE(LOCAL_BETA, BETA, 8, 1, LANGRP)
            BETA = DSQRT(BETA)
C           NORMALIZE NEXT LANCZOS VECTOR
            CALL DSCAL(CHUNK_NEW, 1/BETA, W(1), 1)
            CALL DCOPY(CHUNK_NEW, W(1), 1, LANV(INDEX4*CHUNK_NEW+1), 1)
            D(INDEX4) = ALPHA
            E(INDEX4) = BETA
C           UPDATE AUXILIARY VECTORS
            CALL MY_CONCATV(LANV((INDEX4)*CHUNK_NEW+1),VCUR(1),
     &                      CHUNK_NEW,SEND_CNT(1),SEND_DISPL(1),LANGRP)
            CALL DCOPY(CHUNK_NEW,LANV((INDEX4-1)*CHUNK_NEW+1),1,
     &                 V_PREV(1),1)
C           CHECK CONVERGENCE OF EIGENVALUES: OBSOLETE (FUTURE USE ONLY)
            CALL DCOPY(INDEX4, D(1), 1, LD(1), 1)
            CALL DCOPY(INDEX4-1, E(1), 1, LE(1), 1)
            ABTOL = -1
          ENDDO
C         =============================================================
C         END OF LANCZOS LOOP
C         =============================================================
          CALL MY_SYNC(LANGRP)
          IF (.NOT.CONV_FLAG) INDEX4 = INDEX4-1
          CHUNK_BEGIN_E = NWA12(MEPOS,1)
          CHUNK_END_E   = NWA12(MEPOS,2)
          CHUNK_NEW_E   = CHUNK_END_E - CHUNK_BEGIN_E +1
C         =======================
C         DETERMINE SEND  BUFFERS
C         =======================
          MSGLEN=8/IRAT
          CALL MY_CONCAT(CHUNK_NEW_E, SEND_CNT(1), MSGLEN, LANGRP)
          SEND_DISPL(1) = 0
          DO INDEX1=2, SZ
            SEND_DISPL(INDEX1) = SEND_DISPL(INDEX1-1) + 
     &                           SEND_CNT(INDEX1-1)
          ENDDO
C         ALLOCATE MEMORY FOR EIGENVECTORS
          IF (MEM_FLAG) THEN 
            CALL MEMORY(IP_Z, INDEX4*(CHUNK_END_E-CHUNK_BEGIN_E+1),
     &                  'Z')
          ENDIF
          CALL  DSTEVX('V','I',INDEX4,D(1),E(1),VL,VU,CHUNK_BEGIN_E,
     &                 CHUNK_END_E, ABTOL, NFOUND, TEIG(1), Z(1,1),
     &                 INDEX4, WORK(1), IWORK(1), IFAIL(1), IERR)

          CALL MY_SYNC(LANGRP)
C         ==============================================================
C         CALCULATION OF APPROXIMATE EIGENVECTORS
C         EACH PROCESSOR WILL STORE COMPLETE EIGENVECTORS
C         EIGENVECTORS ARE DISTRIBUTED ACROSS PROCESSORS
C         ==============================================================
          IF (MEM_FLAG) THEN
            CALL MEMORY(IP_Z_TEMP, MY_MAXVAL(SEND_CNT(1),SZ)*NDIAG,
     &                  'Z_TEMP')
          ENDIF 
C         ALLOCATION OF MEMORY FOR APPROXIMATE EIGENVECTORS
          IF (MEM_FLAG) THEN
            CALL MEMORY(IP_EIG_VEC, NDIAG*CHUNK_NEW_E, 'EIG_VEC')
          ENDIF
C         ALLOCATE ADDITIONAL MEMORY FOR BOOK KEEPING
          MSGLEN=8/IRAT
          CALL MY_CONCAT(CHUNK_NEW, OLD_SEND_CNT_E(1), MSGLEN, LANGRP)
          IF (MEM_FLAG) THEN 
            CALL MEMORY(IP_E_TEMP, 2*OLD_SEND_CNT_E(ME+1)*
     &                  MAX(CHUNK_NEW_E,SEND_CNT(1)),
     &                 'E_TEMP')
          ENDIF 
          DO INDEX1=1, NOLAN
            IF (RANK.EQ.(INDEX1-1) ) THEN
              CALL DCOPY(INDEX4*CHUNK_NEW_E, Z(1,1), 1, Z_TEMP(1), 1)
              DO INDEX2 = 1, SZ
                 SEND_CNT_E(INDEX2) = OLD_SEND_CNT_E(INDEX2) *
     &                                SEND_CNT(RANK+1)
              ENDDO
              SEND_DISPL_E(1) = 0
              DO INDEX2=2, SZ
                 SEND_DISPL_E(INDEX2) = SEND_DISPL_E(INDEX2-1) +
     &                        SEND_CNT_E(INDEX2-1)
              ENDDO
            ENDIF
C           BROADCAST COUNTS AND DISPL. DATA FOR ALLGATHER
            MSGLEN=SZ*8/IRAT
            CALL MY_BCAST(SEND_CNT_E(1), MSGLEN, INDEX1-1, LANGRP)
            CALL MY_BCAST(SEND_DISPL_E(1), MSGLEN, INDEX1-1, LANGRP)
C           BROADCAST MY EIGENVECTORS OF THE TRIDIAGONAL MATRIX
            CALL MY_BCAST(Z_TEMP(1), SEND_CNT(INDEX1)*INDEX4*8, 
     &                    INDEX1-1,LANGRP)
C           MULTIPLY RECEIVED EIGENVECTORS WITH MY PART OF THE LANCZOS
C           BASIS
            CALL DGEMM("N","N",CHUNK_NEW,SEND_CNT(INDEX1),INDEX4,1D0,
     &                 LANV(1),CHUNK_NEW,Z_TEMP(1),INDEX4,
     &                 0D0, E_TEMP(1), CHUNK_NEW)
C           BRING RESULTS BACK TO THE CORESSPONDING PROCESSOR
            CALL MY_SOURCE_CONCATV(E_TEMP(1),EIG_VEC(1),
     &                             CHUNK_NEW*SEND_CNT(INDEX1),
     &                             SEND_CNT_E(1), SEND_DISPL_E(1),
     &                             INDEX1-1, LANGRP)
          ENDDO
C         ==================================
C         SET CORRECT ORDER FOR EIGENVECTORS
C         ==================================
!$OMP parallel do private(INDEX1)
          DO INDEX1 = 1, NDIAG
            LC_INDEX(INDEX1) = 0
          ENDDO
          START     = 1
          START_COL = 1
          DO INDEX1 = 1, SZ
            DO INDEX2 = 1, CHUNK_NEW_E
              CALL DCOPY(OLD_SEND_CNT_E(INDEX1), EIG_VEC(START_COL), 1,
     &                   Z_TEMP(START+LC_INDEX(1)), 1)
              START_COL = START_COL + OLD_SEND_CNT_E(INDEX1)
              START     = START + NDIAG
            ENDDO
            LC_INDEX(1) = LC_INDEX(1) + OLD_SEND_CNT_E(INDEX1)
            START = 1
          ENDDO
          CALL DCOPY(NDIAG*CHUNK_NEW_E, Z_TEMP(1), 1, EIG_VEC(1), 1)
C         ======================
C         NORMALIZE EIGENVECTORS
C         ======================
          DO INDEX1=1, CHUNK_NEW
            NRM = DNRM2(NDIAG, EIG_VEC((INDEX1-1)*NDIAG+1), 1)
            NRM = 1/DSQRT(NRM)
            CALL DSCAL(NDIAG, NRM, EIG_VEC((INDEX1-1)*NDIAG+1), 1)
          ENDDO 
          CALL MY_SYNC(LANGRP)
C         ==============================================================
C         END LANCZOS EIGENSOLVER ON LANGRP
C         ==============================================================
        ELSE
C          PROCESSORS THAT DID NOT PARTICIPATE IN LANCZOS
C          WILL WAIT IN SUBSEQUENT COLLECTIVE
        ENDIF
C       ==============================================================
C       ALL PROCESSORS GET ALL EIGENVALUES
C       ==============================================================
        IF (CHUNK_NEW.NE.0) THEN
           CALL MY_CONCATV(TEIG(1),EDAV(1),CHUNK_NEW,
     &                     SEND_CNT(1),SEND_DISPL(1),LANGRP)
        ENDIF
        CALL MY_BCAST(EDAV(1), 8*NDIAG, SOURCE, ALLGRP) 
C       ==============================================================
C       SEND CHUNK INFORMATION TO ALL PROCESSORS
        MSGLEN=8/IRAT
        CALL MY_CONCAT(CHUNK_NEW_E, SEND_CNT(1), MSGLEN, ALLGRP)
C       FIND LARGEST NUMBER OF APPROX. EIGENVECTORS HELP BY PROCEESORS
        IF (CHUNK_NEW.EQ.0) THEN
          CHUNK_NEW_E = SEND_CNT(1)
          DO INDEX1=2,NOLAN
            IF (SEND_CNT(INDEX1).GT.CHUNK_NEW_E)
     &                         CHUNK_NEW_E = SEND_CNT(INDEX1)
          ENDDO
          IF (MEM_FLAG) THEN
            CALL MEMORY(IP_Z_TEMP, CHUNK_NEW_E*NDIAG, 'Z_TEMP')
          ENDIF
        ENDIF
C       ====================
C       ROTATE WAVEFUNCTIONS
C       ====================
        START_COL = 0
        START = 1
        DO INDEX1=1, NOLAN
          IF (RANK.EQ.(INDEX1-1) ) THEN
C         INITIALIZE TEMP. SPACE TO ZERO
            CALL AZZERO(Z_TEMP(1), NDIAG*CHUNK_NEW_E)
            CALL DCOPY(NDIAG*CHUNK_NEW_E, EIG_VEC(1), 1, Z_TEMP(1), 1)
          ENDIF

C         CURRENT ROOT BROADCASTS ITS APPROX. EIGENVECTORS
          CALL MY_BCAST(Z_TEMP(1), SEND_CNT(INDEX1)*NDIAG*8, INDEX1-1,
     &                  ALLGRP)
C         EACH PROCESSOR MULTIPLIES THE APPROXIMATE EIGVECS
C         RECEIVED AND STORES THEM AT THE APPROPRIATE POSITION
C         IN MATRIX CSCR
          CALL DGEMM("N","N",2*NGW,SEND_CNT(INDEX1),NDIAG,1.D0,
     &               C0(1,1),2*NGW,Z_TEMP(1),NDIAG,
     &               0.D0, CSCR(1,START), 2*NGW)
          START = START + SEND_CNT(INDEX1)
        ENDDO
        CALL DCOPY(NGW2*NDIAG,CSCR(1,1),1,C0(1,1),1)
C       Check if an eigenvalue is not equal to 0 (dim < NDIAG)
        TCHECK=.FALSE.
        ITEST=0
        DO I=1,NDIAG
          IF(TTROT) THEN
            TTEST = (EDAV(I).GT.TROTMAX).OR.(EDAV(I).LT.TROTMIN)
          ELSE
            TTEST = DABS(EDAV(I)).LT.EPS
          ENDIF
          IF(TTEST) THEN
            TCHECK=.TRUE.
            IF(PARENT)
     &           WRITE(6,*) 'DIST_FRIESNER| EIGENVECTOR',I,
     &                      ' IS VERY BAD!'
C           Not useful: Now we use H|C0>.
C            CALL RANDTOWF(C0(1,I),1,0,0)
            ITEST=ITEST+1
            EDAV(I)=1.D33
          ENDIF
        ENDDO
        IF(TCHECK) THEN
          NTEST=NTEST+1
          IF (NTEST.GT.10.AND.PARENT)
     &       CALL STOPGM(' DIST_FRIESNER', 'CAN''T FIND GUEST VECTORS')
          CALL SORT2(EDAV,NDIAG,INDEX0)
C         Put at the end randomized wavefunctions.
          CALL DCOPY(NGW2*NDIAG,C0,1,CS,1)
          DO I=1,NDIAG
            J=INDEX0(I)
            IF(EDAV(I).EQ.BIG) THEN
              CALL DCOPY(NGW2,CS(1,J),1,C0(1,I),1)
            ELSE
              CALL DCOPY(NGW2,CSCR(1,J),1,C0(1,I),1)
            ENDIF
          ENDDO
C         Orthogonalize.
#if defined(__PGI)
C.. gs_ortho seems to cause problems on pcs 
          CALL GSORTHO(C0,NGW,ITEST+1,NDIAG)
#else
#ifdef  _BGL
          CALL GSORTHO(C0,NGW,ITEST+1,NDIAG)

C        COMMENTED OUT. STANDARD GS IS USED TO SAVE MEMORY
C          CALL GS_ORTHO(C0,NDIAG-ITEST,
C     &         C0(1,NDIAG-ITEST+1),ITEST,ADAV,SCR(LSCR2))
#else
          CALL GS_ORTHO(C0,NDIAG-ITEST,
     &         C0(1,NDIAG-ITEST+1),ITEST,ADAV,SCR)
#endif
#endif
        ENDIF  
C       SET MEMORY ALLOCATION FLAG TO FALSE
        MEM_FLAG = .FALSE.
      ENDDO
C     ==================================================================
      CALL MY_SYNC(ALLGRP)   
C     ====================
C     ROTATE WAVEFUNCTIONS
C     ====================
      START_COL = 0
      START = 1
      DO INDEX1=1, NOLAN
        IF (RANK.EQ.(INDEX1-1) ) THEN
C       INITIALIZE TEMP. SPACE TO ZERO
          CALL AZZERO(Z_TEMP(1), NDIAG*CHUNK_NEW_E)
          CALL DCOPY(NDIAG*CHUNK_NEW_E, EIG_VEC(1), 1, Z_TEMP(1), 1)
        ENDIF
C       CURRENT ROOT BROADCASTS ITS APPROX. EIGENVECTORS
        CALL MY_BCAST(Z_TEMP(1), SEND_CNT(INDEX1)*NDIAG*8, INDEX1-1,
     &                ALLGRP)
C       EACH PROCESSOR MULTIPLIES THE APPROXIMATE EIGVECS
C       RECEIVED AND STORES THEM AT THE APPROPRIATE POSITION
C       IN MATRIX CSCR
        CALL DGEMM("N","N",2*NGW,SEND_CNT(INDEX1),NDIAG,1.D0,
     &             CS(1,1),2*NGW,Z_TEMP(1),NDIAG,
     &             0.D0, CSCR(1,START), 2*NGW)
        START = START + SEND_CNT(INDEX1)
      ENDDO
      CALL DCOPY(NGW2*NDIAG,CSCR(1,1),1,CS(1,1),1)
      TIM2=TIMEF()
      TIM=(TIM2-TIM1)*0.001D0
      IF(PARENT.AND.TINFO) THEN
        WRITE(6,'(A,T58,F8.2)') 
     *    ' >> TIME FOR INITIAL SUBSPACE DIAGONALIZATION:  ',TIM
        WRITE(6,'(A,8X,A,8X,A)') ' >> CYCLE     NCONV',
     *    'B2MAX','B2MIN     #HPSI      TIME'
      ENDIF
C     ==--------------------------------------------------------------==
      CALL MY_SYNC(ALLGRP)  
      CALL IAZZERO(INDEX0,NDIAG)
      DO ICYCLE=1,N_FRIES
        TIM1=TIMEF()
        B2MAX=0.D0
        B2MIN=1.D30
C       We calculate again the residue for all vectors
        NCONV=0
        NCURR=NCONV+1
C       First Lanczos Step
        DO I=NCURR,NDIAG
!$OMP parallel do private(IG)
          DO IG=1,NGW
            CSCR(IG,1)=CS(IG,I)-EDAV(I)*C0(IG,I)
          ENDDO
          B2=DOTP(NGW,CSCR,CSCR)
#ifdef PARALLEL
          CALL GLOSUM(1,B2)
#endif
          BETA1(I)=DSQRT(B2)
          ALPHA0(I)=EDAV(I)
C          IF (PARENT) PRINT *, "RESIDUAL !!!", I, BETA, B2LIMIT
          IF(B2.LT.B2LIMIT) THEN
            NCONV=NCONV+1
            INDEX0(I)=NCONV
            IF(B2.GT.B2MAX) B2MAX=B2
            IF(B2.LT.B2MIN) B2MIN=B2
          ELSE
            CALL DCOPY(NGW2,CSCR(1,1),1,CS(1,I),1)
            FAC=1.D0/BETA1(I)
            CALL DSCAL(NGW2,FAC,CS(1,I),1)
          ENDIF
        ENDDO
C       Order states: converged first
        DO I=NCURR,NDIAG
          J=INDEX0(I)
          IF(J.NE.0.AND.J.NE.I) THEN
            CALL DSWAP(NGW2,C0(1,I),1,C0(1,J),1)
            CALL DSWAP(NGW2,CS(1,I),1,CS(1,J),1)
            INDEX0(J)=J
            INDEX0(I)=0
            AUX = BETA1(I)
            BETA1(I) = BETA1(J)
            BETA1(J) = AUX  
            AUX = ALPHA0(I)
            ALPHA0(I) = ALPHA0(J)
            ALPHA0(J) = AUX  
          ENDIF
        ENDDO
        NCURR=NCONV+1
C       Lanczos Refinement
        IF(ICYCLE.EQ.1.AND.NCURR.GT.NDIAG) THEN
          TREFINE=.FALSE.
        ELSE
          TREFINE=.TRUE.
        ENDIF
#ifdef  _BGL
        CALL KRYLOV_REF(NCURR,NDIAG,NCONV,NHPSI,NSTATE,
     &       C0,CS,SC0,CSCR,CGS,FOCC,SIGN,
     &       VPOT,PSI,INDEX0,ALPHA0,BETA1,SCR(LSCR2),LSCR3,B2MIN,B2MAX)
#else
        CALL KRYLOV_REF(NCURR,NDIAG,NCONV,NHPSI,NSTATE,
     &       C0,CS,SC0,CSCR,CGS,FOCC,SIGN,
     &       VPOT,PSI,INDEX0,ALPHA0,BETA1,SCR,LSCR2,B2MIN,B2MAX)
#endif
C       Order states : converged first
        DO I=NCURR,NDIAG
          J=INDEX0(I)
          IF(J.NE.0.AND.J.NE.I) THEN
            CALL DSWAP(NGW2,C0(1,I),1,C0(1,J),1)
            CALL DSWAP(NGW2,CS(1,I),1,CS(1,J),1)
            INDEX0(J)=J
            INDEX0(I)=0
            AUX = BETA1(I)
            BETA1(I) = BETA1(J)
            BETA1(J) = AUX
          ENDIF
        ENDDO
C       Reorthogonalize
        IF(NCURR.LE.NDIAG) THEN
#if defined(__PGI)
          CALL GSORTHO(C0,NGW,NCURR,NDIAG)
#else
#ifdef  _BGL
C     CALL GS_ORTHO(C0,NCURR-1,C0(1,NCURR),NDIAG-NCURR+1,
C     *              ADAV,SCR(LSCR2))
           CALL GSORTHO(C0,NGW,NCURR,NDIAG)    
#else
          CALL GS_ORTHO(C0,NCURR-1,C0(1,NCURR),NDIAG-NCURR+1,ADAV,SCR)
#endif
#endif
        ENDIF
C       Calculate new forces; only for states that entered Lanczos
        NLEFT=NDIAG-NCURR+1

        IF (NCURR.LE.NDIAG) THEN
          IF(TTROT) THEN
            CALL EHPSI(C0(1,NCURR),CS(1,NCURR),VPOT,PSI,NLEFT)
          ELSE
#ifdef  _BGL
            CALL HPSI(C0(1,NCURR),CS(1,NCURR),SC0(1,NCURR),
     &           VPOT,PSI,SCR(LSCR2),LSCR3,NLEFT,1,ISPIN)
#else
            CALL HPSI(C0(1,NCURR),CS(1,NCURR),SC0(1,NCURR),
     &           VPOT,PSI,SCR,LSCR2,NLEFT,1,ISPIN)
#endif
            CALL HFXPSI(CGS,C0(1,NCURR),CS(1,NCURR),FOCC,SIGN,
     &           PSI,NSTATE,NLEFT)
          ENDIF
          NHPSI=NHPSI+NLEFT
        ENDIF
C       ===============================================================
C       DISTRIBUTED PROJECTED HAMILTONIAN IS CALCULATED HERE 
C       ===============================================================
        MSGLEN=NDIAG*NORBX*8
        DO IP=0,NPROC-1
          NX=NWA12(IP,2)-NWA12(IP,1)+1
          CALL AZZERO(TMAT,NDIAG*NORBX)
          IF(NX.GT.0) THEN
            CALL OVLAP2(NGW,NDIAG,NX,TMAT,C0,CS(1,NWA12(IP,1)))
            CALL MY_COMBINE(TMAT,RMAT,MSGLEN,1,ALLGRP)

            IF(ME.EQ.IP) THEN
              CALL DCOPY(NDIAG*NORBX,RMAT(1),1,XTMAT(1,1),1)
            ENDIF
          ENDIF
        ENDDO
C       ================================================================ 
C       ====================================
C       WE NEED TO SYMMETRIZE OVERLAP MATRIX
C       ====================================
C        CALL SYMM_DA(XTMAT,RMAT,TMAT,NDIAG,
C     *              NWA12(0,1),NWA12(0,2),NORBX,MEPOS,NPROC,LANGRP)
C
C       ================================================================
C       LANCZOS PARALLEL EIGENSOLVER
C       ================================================================
        IF ((NWA12(ME,2)-NWA12(ME,1)+1).NE.0) THEN
          SZ = 0
          CHUNK_NEW = NWA12(ME,2)-NWA12(ME,1)+1
          DO IP=0, NPROC-1
            IF ((NWA12(IP,2)-NWA12(IP,1)+1).NE.0) SZ = SZ+1
          ENDDO
          NOLAN = SZ
          LAN_MAX = NDIAG+1
          CALL AZZERO(PW(1),LAN_MAX)
          CALL AZZERO(LANV(1),CHUNK_NEW*LAN_MAX)
          CALL AZZERO(W(1),CHUNK_NEW)
C         DETERMINE SEND AND RECEIVE BUFFERS
          MSGLEN=8/IRAT
          CALL MY_CONCAT(CHUNK_NEW, SEND_CNT(1), MSGLEN, LANGRP)
          SEND_DISPL(1) = 0
          DO INDEX4=2, SZ
            SEND_DISPL(INDEX4) = SEND_DISPL(INDEX4-1) + 
     &                           SEND_CNT(INDEX4-1)
          ENDDO
C         DETERMINE A UNIT NORM INITIAL VECTOR V0
          IF (PARENT) THEN
            CALL DURAND(12344.0D0,NDIAG,V0) 
            NRM = DNRM2(NDIAG, V0(1), 1)
            NRM = 1/NRM
            CALL DSCAL(NDIAG, NRM, V0(1), 1)
          ENDIF
C         BROADCAST INITIAL VECTOR TO ALL PROCESSORS
          CALL MY_BCAST(V0(1), 8*NDIAG, SOURCE, LANGRP)
          CALL DCOPY(NDIAG, V0(1), 1, VCUR(1), 1)
C         INITIALIZE V_PREV and BETA
          DO IS=1, CHUNK_NEW
            V_PREV(IS) = 0D0
            W(IS)=0D0
          ENDDO
          BETA = 0D0
          CHUNK_BEGIN = NWA12(ME,1)
          CHUNK_END   = NWA12(ME,2)
C         INITIALIZE FIRST LANCZOS VECTOR
          CALL DCOPY(CHUNK_NEW, VCUR(CHUNK_BEGIN), 1, LANV(1), 1)
C         INITIALIZE INITIAL SUM OF WANTED EIGENVALUES
          SM_OLD = 0D0
C         INITIALIZE CONVERGENCE TOLERANCE
          CONV_TOL = 1D-8
C         ============
C         LANCZOS LOOP
C         ============
          NUM_EIG = NDIAG
          DO INDEX4 = 1, LAN_MAX-1
C           DISTRIBUTED MATRIX-VECTOR PRODUCT
C           W: Holds the result
            CALL DGEMV("T", NDIAG,CHUNK_NEW,1D0,XTMAT(1,1),NDIAG,
     &               VCUR(1), 1, 0D0, W(1), 1)
C           DAXPY: W = W - BETA*V_(J-1)
            CALL DAXPY(CHUNK_NEW, -BETA, V_PREV(1), 1, W(1), 1)
C           CALCULATE ALPHA: INNER PRODUCT WITH CURRENT LANCZOS VECTOR
            LOCAL_ALPHA = DDOT(CHUNK_NEW,W(1),1,
     &                    LANV((INDEX4-1)*CHUNK_NEW+1), 1)
            CALL MY_COMBINE(LOCAL_ALPHA, ALPHA, 8, 1, LANGRP)
C           DAXPY: W = W - ALPHA*V_J
            CALL DAXPY(CHUNK_NEW,-ALPHA,LANV((INDEX4-1)*CHUNK_NEW+1),1,
     &                 W(1),1)
C           ============================================================
C           REORTHOGONALIZE AGAINST ALL PREVIOUS BASIS VECTORS
C           ============================================================
            DO INDEX1=1,INDEX4
              LOCAL_PW(INDEX1)=DDOT(CHUNK_NEW,LANV((INDEX1-1)*
     &                              CHUNK_NEW+1),
     &                              1, W(1), 1)
            ENDDO

            CALL MY_COMBINE(LOCAL_PW(1), PW(1), INDEX4*8, 1, LANGRP)

            DO INDEX1=1, INDEX4
              CALL DAXPY(CHUNK_NEW, -PW(INDEX1),
     &                   LANV((INDEX1-1)*CHUNK_NEW+1), 1, W(1), 1)
            ENDDO
C           ============================================================
C           CALCULATE BETA: NORM OF VECTOR W
            LOCAL_BETA = DDOT(CHUNK_NEW, W(1), 1, W(1), 1)
            CALL MY_COMBINE(LOCAL_BETA, BETA, 8, 1, LANGRP)
            BETA = DSQRT(BETA)
C           NORMALIZE NEXT LANCZOS VECTOR
            CALL DSCAL(CHUNK_NEW, 1/BETA, W(1), 1)
            CALL DCOPY(CHUNK_NEW, W(1), 1, LANV(INDEX4*CHUNK_NEW+1), 1)
            D(INDEX4) = ALPHA
            E(INDEX4) = BETA

C           UPDATE AUXILIARY VECTORS
            CALL MY_CONCATV(LANV((INDEX4)*CHUNK_NEW+1),VCUR(1),
     &                      CHUNK_NEW, SEND_CNT(1),SEND_DISPL(1),LANGRP)
            CALL DCOPY(CHUNK_NEW,LANV((INDEX4-1)*CHUNK_NEW+1),1,
     &                 V_PREV(1),1)
C           CHECK CONVERGENCE FOR EIGENVALUES: FUTURE USE ONLY
            CALL DCOPY(INDEX4, D(1), 1, LD(1), 1)
            CALL DCOPY(INDEX4-1, E(1), 1, LE(1), 1)
            ABTOL = -1
        ENDDO
C       ===================
C       END OF LANCZOS LOOP
C       ===================
        IF (.NOT.CONV_FLAG) INDEX4 = INDEX4-1
        CHUNK_BEGIN_E = NWA12(MEPOS,1)
        CHUNK_END_E   = NWA12(MEPOS,2)
        CHUNK_NEW_E   = CHUNK_END_E - CHUNK_BEGIN_E +1
C       =======================
C       DETERMINE SEND  BUFFERS
C       =======================
        MSGLEN=8/IRAT
        CALL MY_CONCAT(CHUNK_NEW_E, SEND_CNT(1), MSGLEN, LANGRP)
        SEND_DISPL(1) = 0
        DO INDEX1=2, SZ
          SEND_DISPL(INDEX1) = SEND_DISPL(INDEX1-1) + SEND_CNT(INDEX1-1)
        ENDDO
        CALL  DSTEVX('V','I',INDEX4,D(1),E(1),VL,VU,CHUNK_BEGIN_E,
     &               CHUNK_END_E,
     &               ABTOL, NFOUND, TEIG(1), Z(1,1), INDEX4, WORK(1),
     &               IWORK(1), IFAIL(1), IERR)
C       ================================================================
C       CALCULATION OF APPROXIMATE EIGENVECTORS
C       EACH PROCESSOR WILL STORE COMPLETE EIGENVECTORS
C       EIGENVECTORS ARE DISTRIBUTED ACROSS PROCESSORS
C       ================================================================
C       ALLOCATE ADDITIONAL MEMORY FOR BOOK KEEPING
        MSGLEN=8/IRAT
        CALL MY_CONCAT(CHUNK_NEW, OLD_SEND_CNT_E(1), MSGLEN, LANGRP)
        DO INDEX1=1, NOLAN
          IF (RANK.EQ.(INDEX1-1) ) THEN
            CALL DCOPY(INDEX4*CHUNK_NEW_E, Z(1,1), 1, Z_TEMP(1), 1)

            DO INDEX2 = 1, SZ
               SEND_CNT_E(INDEX2) = OLD_SEND_CNT_E(INDEX2) *
     &                              SEND_CNT(RANK+1)
            ENDDO
            SEND_DISPL_E(1) = 0
            DO INDEX2=2, SZ
               SEND_DISPL_E(INDEX2) = SEND_DISPL_E(INDEX2-1) +
     &                       SEND_CNT_E(INDEX2-1)
            ENDDO
          ENDIF
C         BROADCAST COUNTS AND DISPL. DATA FOR ALLGATHER
          MSGLEN=SZ*8/IRAT
          CALL MY_BCAST(SEND_CNT_E(1), MSGLEN, INDEX1-1, LANGRP)
          CALL MY_BCAST(SEND_DISPL_E(1), MSGLEN, INDEX1-1, LANGRP)

C         BROADCAST MY EIGENVECTORS OF THE TRIDIAGONAL MATRIX
          CALL MY_BCAST(Z_TEMP(1), SEND_CNT(INDEX1)*INDEX4*8, INDEX1-1,
     &                  LANGRP)
C         MULTIPLY RECEIVED EIGENVECTORS WITH MY PART OF THE LANCZOS
C         BASIS
          CALL DGEMM("N","N",CHUNK_NEW,SEND_CNT(INDEX1),INDEX4,1D0,
     &               LANV(1),CHUNK_NEW,Z_TEMP(1),INDEX4,
     &               0D0, E_TEMP(1), CHUNK_NEW)
C         BRING RESULTS BACK TO THE CORESSPONDING PROCESSOR
          CALL MY_SOURCE_CONCATV(E_TEMP(1),EIG_VEC(1),
     &                           CHUNK_NEW*SEND_CNT(INDEX1),
     &                           SEND_CNT_E(1), SEND_DISPL_E(1),
     &                           INDEX1-1, LANGRP)
        ENDDO
C       ================================================================
        DO INDEX1 = 1, NDIAG
          LC_INDEX(INDEX1) = 0
        ENDDO
        START     = 1
        START_COL = 1
        DO INDEX1 = 1, SZ
          DO INDEX2 = 1, CHUNK_NEW_E
            CALL DCOPY(OLD_SEND_CNT_E(INDEX1), EIG_VEC(START_COL), 1,
     &                 Z_TEMP(START+LC_INDEX(1)), 1)
            START_COL = START_COL + OLD_SEND_CNT_E(INDEX1)
            START     = START + NDIAG
         ENDDO
         LC_INDEX(1) = LC_INDEX(1) + OLD_SEND_CNT_E(INDEX1)
         START = 1
        ENDDO
        CALL DCOPY(NDIAG*CHUNK_NEW_E, Z_TEMP(1), 1, EIG_VEC(1), 1)
C       ======================
C       NORMALIZE EIGENVECTORS
C       ======================
        DO INDEX1=1, CHUNK_NEW
          NRM = DNRM2(NDIAG, EIG_VEC((INDEX1-1)*NDIAG+1), 1)
          NRM = 1/DSQRT(NRM)
          CALL DSCAL(NDIAG, NRM, EIG_VEC((INDEX1-1)*NDIAG+1), 1)
        ENDDO 
        CALL MY_SYNC(LANGRP)
C       ================================================================
C       END LANCZOS ITERATION ON LANGRP
C       ================================================================
        ELSE
C         PROCESSORS THAT DID NOT PARTICIPATE IN LANCZOS
C         WILL WAIT IN SUBSEQUENT COLLECTIVE
        ENDIF
C       ==============================================================
C       ALL PROCESSORS GET ALL EIGENVALUES
C       ==============================================================
        IF (CHUNK_NEW.NE.0) THEN
           CALL MY_CONCATV(TEIG(1),EDAV(1),CHUNK_NEW,
     &                     SEND_CNT(1),SEND_DISPL(1),LANGRP)
        ENDIF
        CALL MY_BCAST(EDAV(1), 8*NDIAG, SOURCE, ALLGRP)
C       ==============================================================
C       SEND CHUNK INFORMATION TO ALL PROCESSORS
        MSGLEN=8/IRAT
        CALL MY_CONCAT(CHUNK_NEW_E, SEND_CNT(1), MSGLEN, ALLGRP)
C       FIND LARGEST NUMBER OF APPROX. EIGENVECTORS HELP BY PROCEESORS
        IF (CHUNK_NEW.EQ.0) THEN
          CHUNK_NEW_E = SEND_CNT(1)
          DO INDEX1=2,NOLAN
            IF (SEND_CNT(INDEX1).GT.CHUNK_NEW_E)
     &                               CHUNK_NEW_E = SEND_CNT(INDEX1)
          ENDDO
        ENDIF
C       ====================
C       ROTATE WAVEFUNCTIONS
C       ====================
        START_COL = 0
        START = 1
        DO INDEX1=1, NOLAN
           IF (RANK.EQ.(INDEX1-1) ) THEN
C          INITIALIZE TEMP. SPACE TO ZERO
             CALL AZZERO(Z_TEMP(1), NDIAG*CHUNK_NEW_E)
             CALL DCOPY(NDIAG*CHUNK_NEW_E, EIG_VEC(1), 1, Z_TEMP(1), 1)
           ENDIF
C          CURRENT ROOT BROADCASTS ITS APPROX. EIGENVECTORS
           CALL MY_BCAST(Z_TEMP(1), SEND_CNT(INDEX1)*NDIAG*8, INDEX1-1,
     &                   ALLGRP)
C          EACH PROCESSOR MULTIPLIES THE APPROXIMATE EIGVECS
C          RECEIVED AND STORES THEM AT THE APPROPRIATE POSITION
C          IN MATRIX CSCR
           CALL DGEMM("N","N",2*NGW,SEND_CNT(INDEX1),NDIAG,1.D0,
     &                C0(1,1),2*NGW,Z_TEMP(1),NDIAG,
     &                0.D0, CSCR(1,START), 2*NGW)
           START = START + SEND_CNT(INDEX1)
        ENDDO
        CALL DCOPY(NGW2*NDIAG,CSCR(1,1),1,C0(1,1),1)
        START_COL = 0
        START = 1
        DO INDEX1=1, NOLAN
          IF (RANK.EQ.(INDEX1-1) ) THEN
C         INITIALIZE TEMP. SPACE TO ZERO
            CALL AZZERO(Z_TEMP(1), NDIAG*CHUNK_NEW_E)
            CALL DCOPY(NDIAG*CHUNK_NEW_E, EIG_VEC(1), 1, Z_TEMP(1), 1)
          ENDIF
C         CURRENT ROOT BROADCASTS ITS APPROX. EIGENVECTORS
          CALL MY_BCAST(Z_TEMP(1), SEND_CNT(INDEX1)*NDIAG*8, INDEX1-1,
     &                   ALLGRP)
C         EACH PROCESSOR MULTIPLIES THE APPROXIMATE EIGVECS
C         RECEIVED AND STORES THEM AT THE APPROPRIATE POSITION
C         IN MATRIX CSCR
          CALL DGEMM("N","N",2*NGW,SEND_CNT(INDEX1),NDIAG,1.D0,
     &               CS(1,1),2*NGW,Z_TEMP(1),NDIAG,
     &               0.D0, CSCR(1,START), 2*NGW)
          START = START + SEND_CNT(INDEX1)
        ENDDO
        CALL DCOPY(NGW2*NDIAG,CSCR(1,1),1,CS(1,1),1)
C       If one eigenvalue is equal to zero (like friesner_c).
        NTRANS=NCONV
        DO I=1,NCONV
          IF(TTROT) THEN
            TTEST = (EDAV(I).GT.TROTMAX).OR.(EDAV(I).LT.TROTMIN)
          ELSE
            TTEST = DABS(EDAV(I)).LT.EPS
          ENDIF
          IF(TTEST) THEN
            NCONV=NCONV-1
            NTRANS=NTRANS+1
            IF(PARENT)
     &           WRITE(6,*) ' DIST_FRIESNER| ','VERY BAD EIGENVALUE !!'
            IF(NTRANS.GT.NDIAG) THEN
              CALL RANDTOWF(C0(1,I),1,0,0)
            ELSE
              CALL DSWAP(NGW2,C0(1,I),1,C0(1,NTRANS),1)
              CALL DSWAP(NGW2,CS(1,I),1,CS(1,NTRANS),1)
            ENDIF
          ENDIF
        ENDDO
        TIM2=TIMEF()
        TIM=(TIM2-TIM1)*0.001D0
        IF(PARENT.AND.TINFO) THEN
           WRITE(6,'(4X,I5,5X,I5,2(1PE13.3),0PF10.2,0PF10.2)')
     &          ICYCLE,NCONV,B2MAX,B2MIN,
     &          DBLE(NHPSI-NHPSIOLD)/DBLE(NDIAG),TIM
        ENDIF
        NHPSIOLD=NHPSI
C       Test if the number of eigenvalues is enough to stop.
        IF((.NOT.TFRAL).AND.TFINT.AND.
     &       (NCONV.NE.NCONVOLD).AND.
     &       (NCONV.NE.NDIAG)) THEN
C          Spin or not.
           IF(TLSD2.AND.(NCONV.GE.NEL).OR.
     &          (.NOT.TLSD2.AND.(NCONV.GE.(NEL+1)/2)) ) THEN
              TCONV=.TRUE.
              NCONVOLD=NCONV
#ifdef  _BGL
              CALL CONVFRIE(NDIAG,NCONV,NEL,EDAV,TLSD,
     &             AMU,SCR(LSCR2),SCR(LSCR2+NDIAG+MOD(NDIAG,2)),TCONV)
#else
              CALL CONVFRIE(NDIAG,NCONV,NEL,EDAV,TLSD,
     &             AMU,SCR(1),SCR(NDIAG+1),TCONV)
#endif
           ENDIF
        ENDIF
        IF(TCONV.OR.NCONV.EQ.NDIAG) THEN
           GOTO 200
        ENDIF
C       End of Krylov Space Cycles
      ENDDO
  200 CONTINUE
      IF(PARENT.AND.((.NOT.TCONV).AND.NCONV.NE.NDIAG).AND.TDEBUG) THEN
        WRITE(6,'(1X,64("!"))')
        WRITE(6,'(" !!",A,T64,"!!")')
     &       ' DIST_FRIESNER| NOT ALL ROOTS ARE CONVERGED'
        WRITE(6,'(1X,64("!"))')
      ENDIF
C     Order Roots with respect to energy
      CALL SORT2(EDAV,NDIAG,INDEX0)
      CALL DCOPY(NGW2*NDIAG,C0(1,1),1,CS(1,1),1)
      IF(.NOT.TTROT) CALL DSCAL(NDIAG,-1.D0,EDAV(1),1)
      DO I = 1 , NDIAG/2
         AUX = EDAV(NDIAG-I+1)
         EDAV(NDIAG-I+1) = EDAV(I)
         EDAV(I) = AUX
         IAUX = INDEX0(NDIAG-I+1)
         INDEX0(NDIAG-I+1) = INDEX0(I)
         INDEX0(I) = IAUX
      ENDDO
      DO I=1,NDIAG
        J=INDEX0(I)
        CALL DCOPY(2*NGW,CS(1,J),1,C0(1,I),1)
      ENDDO
      TLSD=TLSD2
C     =================================================================
C     FREE ALLOCATED MEMORY
C     =================================================================
      IF (CHUNK_NEW.NE.0) THEN
         CALL FREEM(IP_V_PREV)
         CALL FREEM(IP_LOCAL_PW)
         CALL FREEM(IP_PW)
         CALL FREEM(IP_W)
         CALL FREEM(IP_LANV)
         CALL FREEM(IP_V0)
         CALL FREEM(IP_VCUR)
*
         CALL FREEM(IP_E_TEMP)
         CALL FREEM(IP_EIG_VEC)
         CALL FREEM(IP_Z_TEMP)
         CALL FREEM(IP_Z)
      ELSE
         CALL FREEM(IP_Z_TEMP)
      ENDIF
C     GLOBAL DEALLOCATIONS
      CALL FREEM(IP_TMAT)
      CALL FREEM(IP_RMAT)
      CALL FREEM(IP_XTMAT)

      CALL TIHALT('  DIST_FRIESNER',ISUB)
C     ==--------------------------------------------------------------==
#endif
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_DIST_FRIESNER(LFRIESNER,TAG,NDIAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'kpts.inc'        !TKPNT
      INCLUDE 'irat.inc'        !IRAT
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'        !TTROT
C     Arguments
      INTEGER   LFRIESNER,NDIAG
      CHARACTER TAG*30
C     Variables
      INTEGER   LHPSI,LSUMMAT,LKRYLOV_REF,LDIAG,LSCR
C     ==--------------------------------------------------------------==
      LSCR=0
      LDIAG=0
      LKRYLOV_REF=0
      LSUMMAT=0
      LHPSI=0
#ifdef PARALLEL
      IF(.NOT.TTROT) THEN
        CALL GIVE_SCR_HPSI(LHPSI,TAG,NDIAG)
      ENDIF
      IF (.FALSE.) THEN 
         CALL GIVE_SCR_SUMMAT(LSUMMAT,TAG,NDIAG)
      ELSE
         LSUMMAT = 0
      ENDIF
      CALL GIVE_SCR_KRYLOV_REF(LKRYLOV_REF,TAG,NDIAG)
      IF(TKPNT) THEN
        LDIAG=((3*NDIAG-2) + 5*(2*NDIAG)) !ZHEEV
        LSCR=NDIAG/IRAT+1+2*NDIAG*NDIAG+2*NDIAG !INDEX ADAV ALPHA0 BETA1
      ELSE
        LDIAG=5*NDIAG           !DSYEV
        IF (.FALSE.) THEN
           LSCR=NDIAG/IRAT+1+NDIAG*NDIAG+2*NDIAG
        ELSE
           LSCR = NDIAG/IRAT+1+NDIAG+2*NDIAG
        ENDIF          !INDEX ADAV ALPHA0 BETA1
      ENDIF
      LDIAG=MAX(LDIAG,2*NDIAG)  !CONVFRIE
      LFRIESNER=LSCR+MAX(LHPSI,LSUMMAT,LKRYLOV_REF,LDIAG)+10
      TAG='LSCR+MAX(HPSI,SUMHMAT,...)'
C     ==--------------------------------------------------------------==
#endif
      RETURN
      END
C     ==================================================================
