C     ==================================================================
      SUBROUTINE DNLPS_DK(scr,LSCR)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'        ! NSP
      INCLUDE 'pslo.inc'        ! TIVAN
      INCLUDE 'nlcc.inc'        ! TINLC
      INCLUDE 'dpot.inc'        ! TKB
      INCLUDE 'str2.inc'        ! stress
      INCLUDE 'sgpp.inc'        ! TSGP
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'        ! TTROT
      INCLUDE 'response_p.inc'

      INTEGER    LSCR
      REAL*8     SCR(*)
C     Variables
      DIMENSION  dtwnl_dk(2*NGW,NHXS,NSX,*),
     &           ddtwnl_ddk(2*NGW,NHXS,NSX,3,*)
c     &           TWNL(NGWK,NHXS,NSX,NKPNT),

      DIMENSION  twnl_p(2*NGW,NHXS,NSX,3), twnl_m(2*NGW,NHXS,NSX,*)
      REAL*8     DELTAK

      INTEGER    IS,ISUB,IV

C     ==--------------------------------------------------------------==
C     ==  ALLOCATION OF MEMORY                                        ==
C     ==--------------------------------------------------------------==
      CALL TISET('  DNLPS_DK',ISUB)

      IF(TIVAN) THEN
         CALL STOPGM('DNLPS_DK','Vanderbilt not implemented')
      ENDIF

C     ==--------------------------------------------------------------==
C     Calculation of Maximal value of L
      LPMAX=0
      DO IS=1,NSP
         IF(NGH(IS).GT.0) THEN
            IF(TKB(IS)) THEN
               IV=NGH(IS)
               LPMAX=MAX(LPMAX,NGHCOM(IV,IS))
            ELSEIF(TSGP(IS)) THEN
               IV=NGH(IS)
               LPMAX=MAX(LPMAX,LPVAL(IV,IS))
            ENDIF
         ENDIF
      ENDDO

      CALL MEMORY(IP_TWNL_P,3*NSX*NHXS*2*NGW,'TWNL_P')
      CALL AZZERO(TWNL_P,3*NSX*NHXS*2*NGW)
      CALL MEMORY(IP_TWNL_M,3*NSX*NHXS*2*NGW,'TWNL_M')
      CALL AZZERO(TWNL_M,3*NSX*NHXS*2*NGW)

C     Computation of the form factors in G+k and G-k
C     Saved in TWNL_P and TWNL_M
C     Computed separatedly for Kx Ky and Kz for each G vector from 1:2*NGW
C     DELTAK is the increment given in (2*PI/A) units

      DELTAK = 0.0050d0
      CALL PUTWNL_KPERT(DELTAK,SCR,LSCR)

C     ==--------------------------------------------------------------==
C     ==  NONLINEAR CORE CORRECTION                                   ==
C     ==--------------------------------------------------------------==
      IF(TINLC) 
     &     CALL STOPGM('DNLPS_DK','nonlinear core not implemented')
C     ==--------------------------------------------------------------==
C     ==  FINITE TEMPERATURE ELECTRONS                                ==
C     ==--------------------------------------------------------------==
      IF(TTROT) CALL STOPGM
     &     ('DNLPS_DK','finite electron temperature not implemented')
C     ==--------------------------------------------------------------==
C     ==  INITIALIZE DERIVATIVE ARRAYS FOR STRESS CALCULATION         ==
C     ==--------------------------------------------------------------==
      IF(TPRCP.OR.TPRES) THEN
         CALL STOPGM('DNLPS_DK','stress calculation not implemented')
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  DEALLOCATE ALL MEMORY NOT LONGER NEEDED                     ==
C     ==--------------------------------------------------------------==
      IF(PARENT) CALL PRMEM('  DNLPS_DK')

      CALL FREEM(IP_TWNL_P)
      CALL FREEM(IP_TWNL_M)

      CALL TIHALT('  DNLPS_DK',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     ==================================================================
      SUBROUTINE PUTWNL_KPERT(DELTAK,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     == CALCULATE TWNL_P(1:2*NGW,1:NGH(IS),1:NSP,3) [response_p.inc] ==
C     == CALCULATE TWNL_M(1:2*NGW,1:NGH(IS),1:NSP,3) [response_p.inc] ==
C     ==      Non-Local projectors array                              ==
C     ==      for each G-components +/- K(I)(Kleinman-Bylander form)  ==
C     == FOR TIVAN is not implemented                                 ==
C     ==--------------------------------------------------------------==

      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'dpot.inc'
      INCLUDE 'qspl.inc'
      INCLUDE 'sgpp.inc'
      INCLUDE 'response_p.inc'
C     Arguments
      INTEGER   LSCR
      REAL*8    SCR(LSCR),DELTAK
C     Variables
      DIMENSION TWNS(NSPLPO,2,NHXS,NSX),
     &          twnl_p(2*NGW,NHXS,NSX,3),twnl_m(2*NGW,NHXS,NSX,3),
     &          dtwnl_dk(2*NGW,NHXS,NSX,3),GGNG(NSPLPO),
     &          ddtwnl_ddk(2*NGW,NHXS,NSX,3,3),
     &          TWNL(NGWK,NHXS,NSX,NKPNT)
      REAL*8    YLMB_P(2*NHG,LPMAX),YLMB_M(2*NHG,LPMAX)
      pointer   (ip_ylmb_p,ylmb_p),(ip_ylmb_m,ylmb_m)
      REAL*8    HGKP_P(NHG),HGKM_P(NHG),HGKP_M(NHG),HGKM_M(NHG)
      pointer   (ip_hgkp_p,hgkp_p), (ip_hgkp_m,hgkp_m),
     &          (ip_hgkm_p,hgkm_p), (ip_hgkm_m,hgkm_m)
      REAL*8    YLMB_PP(2*NHG,LPMAX),YLMB_MM(2*NHG,LPMAX),
     &          YLMB_PM(2*NHG,LPMAX,2)         
      pointer   (ip_ylmb_pp,ylmb_pp),(ip_ylmb_mm,ylmb_mm),
     &          (ip_ylmb_pm,ylmb_pm)
      REAL*8    HGKP_PP(NHG),HGKM_PP(NHG),HGKP_MM(NHG),HGKM_MM(NHG),
     &          HGKP_PM(NHG,2),HGKM_PM(NHG,2)
      pointer   (ip_hgkp_pp,hgkp_pp), (ip_hgkp_mm,hgkp_mm),
     &          (ip_hgkp_pm,hgkp_pm),
     &          (ip_hgkm_pp,hgkm_pp), (ip_hgkm_mm,hgkm_mm),
     &          (ip_hgkm_pm,hgkm_pm)

      REAL*8    GKRK(3,NHG),KPLUS(3)
      POINTER   (IP_GKRK,GKRK)
      CHARACTER TAG*30
      INTEGER   ISUB,IDIR,IDIR2,LP,IG,I,IS,IV,
     &          LPUTWNL_KPERT
      REAL*8    VOL,XX,TW_P,TW_M,U2DK,UDK2,CURV2
      REAL*8    TW_PP, TW_MM, TW_PM, TW_MP
      REAL*8    DASUM
      EXTERNAL  DASUM

C     ==--------------------------------------------------------------==
      CALL TISET(' PUTWNL_KP',ISUB)
      CALL GIVE_SCR_PUTWNL_KPERT(LPUTWNL_KPERT,TAG)
      CALL TEST_SCR('PUTWNL_KPERT',TAG,LSCR,LPUTWNL_KPERT)
#ifdef _BGL 
      CALL GET_ADDR(IP_YLMB_P,SCR(1))
      CALL GET_ADDR(IP_YLMB_M,SCR(2*NHG*LPMAX+MOD(2*NHG*LPMAX,2)+1))
      CALL GET_ADDR(IP_GKRK,SCR(4*NHG*LPMAX+1+MOD(4*NHG*LPMAX,2)))
#else
      CALL GET_ADDR(IP_YLMB_P,SCR(1))
      CALL GET_ADDR(IP_YLMB_M,SCR(2*NHG*LPMAX+1))
      CALL GET_ADDR(IP_GKRK,SCR(4*NHG*LPMAX+1))
#endif

      CALL MEMORY(IP_HGKP_P,NHG,'HGKP_P')
      CALL MEMORY(IP_HGKM_P,NHG,'HGKM_P')
      CALL MEMORY(IP_HGKP_M,NHG,'HGKP_M')
      CALL MEMORY(IP_HGKM_M,NHG,'HGKM_M')

      CALL AZZERO(HGKP_P,NHG)
      CALL AZZERO(HGKM_P,NHG)
      CALL AZZERO(HGKP_M,NHG)
      CALL AZZERO(HGKM_M,NHG)

      CALL AZZERO(TWNL_P,3*2*NGW*NHXS*NSX)
      CALL AZZERO(TWNL_M,3*2*NGW*NHXS*NSX)

C     ==--------------------------------------------------------------==
      VOL=1.D0/SQRT(OMEGA)
      U2DK = 1.0d0/(2.0d0*DELTAK*TPIBA)
      UDK2 = 1.0d0/(DELTAK*DELTAK*TPIBA2)

      DO  IDIR = 1,3
         CALL AZZERO(KPLUS,3)
         KPLUS(IDIR)=DELTAK

         CALL AZZERO(YLMB_P,2*NHG*LPMAX)
         CALL AZZERO(YLMB_M,2*NHG*LPMAX)

         CALL AZZERO(GKRK,3*NHG)

         DO LP=1,LPMAX
            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=GK(I,IG)+KPLUS(I)
               ENDDO
               HGKP_P(IG) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            CALL YLMR2(LP,NHG,HGKP_P,GKRK,YLMB_P(1,LP))

            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=-GK(I,IG)+KPLUS(I)
               ENDDO
               HGKM_P(IG) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            CALL YLMR2(LP,NHG,HGKM_P,GKRK,YLMB_P(1+NHG,LP))

            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=GK(I,IG)-KPLUS(I)
               ENDDO
               HGKP_M(IG) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            CALL YLMR2(LP,NHG,HGKP_M,GKRK,YLMB_M(1,LP))

            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=-GK(I,IG)-KPLUS(I)
               ENDDO
               HGKM_M(IG) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            CALL YLMR2(LP,NHG,HGKM_M,GKRK,YLMB_M(1+NHG,LP))
         ENDDO        

C     Form factors calculated in G+K and in G-K
C     The K increment is considered in the three cartesian 
C     directions separatedly
C     for each species the product "radial part" times " angular part"
C     is performed 

         DO IS=1,NSP
            DO IV=1,NGH(IS)
               IF(TKB(IS)) THEN
C AK 2005/03/26: unused
C                  L=NGHTOL(IV,IS)+1
                  LP=NGHCOM(IV,IS)
               ELSEIF(TSGP(IS)) THEN
                  LP=LPVAL(IV,IS)
               ENDIF
               XX=DASUM(NSPLPO,TWNS(1,1,IV,IS),1)
               IF(XX.GT.1.D-12) THEN
                  DO IG=1,NGW
                     TW_P=CURV2(HGKP_P(IG),NSPLPO,GGNG(1),
     *                    TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)
                     TW_M=CURV2(HGKP_M(IG),NSPLPO,GGNG(1),
     *                    TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)

                     TWNL_P(IG,IV,IS,IDIR)=YLMB_P(IG,LP)*TW_P*VOL
                     TWNL_M(IG,IV,IS,IDIR)=YLMB_M(IG,LP)*TW_M*VOL
C     FIRST  DERIVATIVE
                     DTWNL_DK(IG,IV,IS,IDIR)= U2DK *
     *                    (TWNL_P(IG,IV,IS,IDIR)-TWNL_M(IG,IV,IS,IDIR))
C     SECOND  DERIVATIVE

                     DDTWNL_DDK(IG,IV,IS,IDIR,IDIR)= UDK2 *
     *                    (TWNL_P(IG,IV,IS,IDIR)+TWNL_M(IG,IV,IS,IDIR)
     *                    -2.0d0*TWNL(IG,IV,IS,1))
                  ENDDO

                  DO IG=1,NGW
                     TW_P=CURV2(HGKM_P(IG),NSPLPO,GGNG(1),
     *                    TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)
                     TW_M=CURV2(HGKM_M(IG),NSPLPO,GGNG(1),
     *                    TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)

                     TWNL_P(IG+NGW,IV,IS,IDIR)=YLMB_P(IG+NHG,LP)*
     *                    TW_P*VOL
                     TWNL_M(IG+NGW,IV,IS,IDIR)=YLMB_M(IG+NHG,LP)*
     *                    TW_M*VOL
C     FIRST  DERIVATIVE
                     DTWNL_DK(IG+NGW,IV,IS,IDIR)= U2DK *
     *                    (TWNL_P(IG+NGW,IV,IS,IDIR)-
     *                    TWNL_M(IG+NGW,IV,IS,IDIR))
C     SECOND DERIVATIVE
                     DDTWNL_DDK(IG+NGW,IV,IS,IDIR,IDIR)= UDK2 *
     *                    (TWNL_P(IG+NGW,IV,IS,IDIR)+
     *                    TWNL_M(IG+NGW,IV,IS,IDIR)
     *                    -(-1)**(NGHTOL(IV,IS))
     *                     *2.0d0*TWNL(IG,IV,IS,1))
                  ENDDO
               ENDIF
            ENDDO
         ENDDO
      ENDDO                     ! END OF 1,IDIR

C     ==--------------------------------------------------------------==
C     ==  DEALLOCATE ALL MEMORY NOT LONGER NEEDED                     ==
C     ==--------------------------------------------------------------==

      CALL FREEM(IP_HGKP_P)
      CALL FREEM(IP_HGKM_P)
      CALL FREEM(IP_HGKP_M)
      CALL FREEM(IP_HGKM_M)

C     ==================================================================
C     ===    MIXED SECOND DERIVATIVES                                 ==
C     ==================================================================

      CALL MEMORY(IP_YLMB_PP,2*NHG*LPMAX,'YLMB_PP')
      CALL MEMORY(IP_YLMB_MM,2*NHG*LPMAX,'YLMB_MM')
      CALL MEMORY(IP_YLMB_PM,2*NHG*LPMAX*2,'YLMB_PM')


      CALL MEMORY(IP_HGKP_PP,NHG,'HGKP_PP')
      CALL MEMORY(IP_HGKM_PP,NHG,'HGKM_PP')
      CALL MEMORY(IP_HGKP_MM,NHG,'HGKP_MM')
      CALL MEMORY(IP_HGKM_MM,NHG,'HGKM_MM')
      CALL MEMORY(IP_HGKP_PM,2*NHG,'HGKP_PM')
      CALL MEMORY(IP_HGKM_PM,2*NHG,'HGKM_PM')


C     Calculation of the following twrms
C     PP : Gx+Delta,Gy+Delta,Gz - Gx+Delta,Gy,Gz+Delta - Gx,Gy+Delta,Gz+Delta 
C     MM : Gx-Delta,Gy-Delta,Gz - Gx-Delta,Gy,Gz-Delta - Gx,Gy-Delta,Gz-Delta 
C     PM : Gx+Delta,Gy-Delta,Gz - Gx+Delta,Gy,Gz-Delta - Gx,Gy+Delta,Gz-Delta 
C     Gx-Delta,Gy+Delta,Gz - Gx-Delta,Gy,Gz+Delta - Gx,Gy-Delta,Gz+Delta 


      DO IDIR = 1,3
         DO IDIR2 = IDIR+1,3
C     
            CALL AZZERO(YLMB_PP,2*NHG*LPMAX)
            CALL AZZERO(YLMB_MM,2*NHG*LPMAX)
            CALL AZZERO(YLMB_PM,2*NHG*LPMAX*2)

            CALL AZZERO(HGKP_PP,NHG)
            CALL AZZERO(HGKM_PP,NHG)
            CALL AZZERO(HGKP_MM,NHG)
            CALL AZZERO(HGKM_MM,NHG)
            CALL AZZERO(HGKP_PM,2*NHG)
            CALL AZZERO(HGKM_PM,2*NHG)

            CALL AZZERO(GKRK,3*NHG)

C     PP part
            CALL AZZERO(KPLUS,3)
            KPLUS(IDIR)=DELTAK
            KPLUS(IDIR2)=DELTAK

            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=GK(I,IG)+KPLUS(I)
               ENDDO
               HGKP_PP(IG) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            DO LP=1,LPMAX
               CALL YLMR2(LP,NHG,HGKP_PP,GKRK,YLMB_PP(1,LP))
            ENDDO

            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=-GK(I,IG)+KPLUS(I)
               ENDDO
               HGKM_PP(IG) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            DO LP=1,LPMAX
               CALL YLMR2(LP,NHG,HGKM_PP,GKRK,YLMB_PP(1+NHG,LP))
            ENDDO

C     MM part
            CALL AZZERO(KPLUS,3)
            KPLUS(IDIR)=-DELTAK
            KPLUS(IDIR2)=-DELTAK

            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=GK(I,IG)+KPLUS(I)
               ENDDO
               HGKP_MM(IG) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            DO LP=1,LPMAX
               CALL YLMR2(LP,NHG,HGKP_MM,GKRK,YLMB_MM(1,LP))
            ENDDO

            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=-GK(I,IG)+KPLUS(I)
               ENDDO
               HGKM_MM(IG) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            DO LP=1,LPMAX
               CALL YLMR2(LP,NHG,HGKM_MM,GKRK,YLMB_MM(1+NHG,LP))
            ENDDO        

C     PM part
C     part1
            CALL AZZERO(KPLUS,3)
            KPLUS(IDIR)=DELTAK
            KPLUS(IDIR2)=-DELTAK

            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=GK(I,IG)+KPLUS(I)
               ENDDO
               HGKP_PM(IG,1) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            DO LP=1,LPMAX
               CALL YLMR2(LP,NHG,HGKP_PM(1,1),GKRK,YLMB_PM(1,LP,1))
            ENDDO

            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=-GK(I,IG)+KPLUS(I)
               ENDDO
               HGKM_PM(IG,1) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            DO LP=1,LPMAX
               CALL YLMR2(LP,NHG,HGKM_PM(1,1),GKRK,YLMB_PM(1+NHG,LP,1))
            ENDDO        

C     part2
            CALL AZZERO(KPLUS,3)
            KPLUS(IDIR)=-DELTAK
            KPLUS(IDIR2)=DELTAK

            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=GK(I,IG)+KPLUS(I)
               ENDDO
               HGKP_PM(IG,2) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            DO LP=1,LPMAX
               CALL YLMR2(LP,NHG,HGKP_PM(1,2),GKRK,YLMB_PM(1,LP,2))
            ENDDO

            DO IG = 1 , NHG
               DO I=1,3
                  GKRK(I,IG)=-GK(I,IG)+KPLUS(I)
               ENDDO
               HGKM_PM(IG,2) = GKRK(1,IG)*GKRK(1,IG)+
     &              GKRK(2,IG)*GKRK(2,IG)+
     &              GKRK(3,IG)*GKRK(3,IG)
            ENDDO
            DO LP=1,LPMAX
               CALL YLMR2(LP,NHG,HGKM_PM(1,2),GKRK,YLMB_PM(1+NHG,LP,2))
            ENDDO        

C     for each species the product "radial part" times " angular part"
C     is performed 

            DO IS=1,NSP
               DO IV=1,NGH(IS)
                  IF(TKB(IS)) THEN
C AK 2005/03/26: unused
C                     L=NGHTOL(IV,IS)+1
                     LP=NGHCOM(IV,IS)
                  ELSEIF(TSGP(IS)) THEN
                     LP=LPVAL(IV,IS)
                  ENDIF
                  XX=DASUM(NSPLPO,TWNS(1,1,IV,IS),1)
                  IF(XX.GT.1.D-12) THEN
                     DO IG=1,NGW
                        TW_PP=CURV2(HGKP_PP(IG),NSPLPO,GGNG(1),
     *                       TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)
                        TW_MM=CURV2(HGKP_MM(IG),NSPLPO,GGNG(1),
     *                       TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)
                        TW_PM=CURV2(HGKP_PM(IG,1),NSPLPO,GGNG(1),
     *                       TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)
                        TW_MP=CURV2(HGKP_PM(IG,2),NSPLPO,GGNG(1),
     *                       TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)

                        TW_PP=YLMB_PP(IG,LP)*TW_PP*VOL
                        TW_MM=YLMB_MM(IG,LP)*TW_MM*VOL
                        TW_PM=YLMB_PM(IG,LP,1)*TW_PM*VOL
                        TW_MP=YLMB_PM(IG,LP,2)*TW_MP*VOL

C     SECOND  DERIVATIVE
                           DDTWNL_DDK(IG,IV,IS,IDIR,IDIR2)= UDK2 *
     *                          (TW_PP-TW_PM-TW_MP+TW_MM)
                           DDTWNL_DDK(IG,IV,IS,IDIR2,IDIR)=
     *                      DDTWNL_DDK(IG,IV,IS,IDIR,IDIR2)
                     ENDDO

                     DO IG=1,NGW
                        TW_PP=CURV2(HGKM_PP(IG),NSPLPO,GGNG(1),
     *                       TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)
                        TW_MM=CURV2(HGKM_MM(IG),NSPLPO,GGNG(1),
     *                       TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)
                        TW_PM=CURV2(HGKM_PM(IG,1),NSPLPO,GGNG(1),
     *                       TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)
                        TW_MP=CURV2(HGKM_PM(IG,2),NSPLPO,GGNG(1),
     *                       TWNS(1,1,IV,IS),TWNS(1,2,IV,IS),0.0D0)

                        TW_PP=YLMB_PP(IG+NHG,LP)*TW_PP*VOL
                        TW_MM=YLMB_MM(IG+NHG,LP)*TW_MM*VOL
                        TW_PM=YLMB_PM(IG+NHG,LP,1)*TW_PM*VOL
                        TW_MP=YLMB_PM(IG+NHG,LP,2)*TW_MP*VOL

C     SECOND DERIVATIVE
                           DDTWNL_DDK(IG+NGW,IV,IS,IDIR,IDIR2)= UDK2 *
     *                          (TW_PP-TW_PM-TW_MP+TW_MM)
                           DDTWNL_DDK(IG+NGW,IV,IS,IDIR2,IDIR)=
     *                      DDTWNL_DDK(IG+NGW,IV,IS,IDIR,IDIR2)
                     ENDDO
                  ENDIF
               ENDDO                 ! IV
            ENDDO                    ! IS
         ENDDO                       ! IDIR2
      ENDDO                          ! IDIR1

      CALL FREEM(IP_YLMB_PP)
      CALL FREEM(IP_YLMB_MM)
      CALL FREEM(IP_YLMB_PM)

      CALL FREEM(IP_HGKP_PP)
      CALL FREEM(IP_HGKM_PP)
      CALL FREEM(IP_HGKP_MM)
      CALL FREEM(IP_HGKM_MM)
      CALL FREEM(IP_HGKP_PM)
      CALL FREEM(IP_HGKM_PM)

      CALL TIHALT(' PUTWNL_KP',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END

C     ====================================================================
C     ====================================================================
      SUBROUTINE GIVE_SCR_PUTWNL_KPERT(LPUTWNL,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NHG and LPMAX

C     Arguments
      INTEGER   LPUTWNL
      CHARACTER TAG*30
C     Variables
      INTEGER   LYLMB
C     ==--------------------------------------------------------------==

      LPUTWNL= 3*NHG
      LYLMB=2*NHG*LPMAX
      LPUTWNL=MAX(1,LPUTWNL+2*LYLMB)
      TAG='3*NHG+4*NHG*LPMAX'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     ==================================================================

