c     ==================================================================
      SUBROUTINE do_perturbation(c0,c1,nstate)
c     ==--------------------------------------------------------------==
c     The main routine that controls the linear response calculations.
c     INPUT: C0: Converged ground state wavefunctions
c            NSTATE
c     OUTPUT: C1: LINEAR RESPONSE wavefunctions
c     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      include 'ions.inc'
      include 'irat.inc'
      include 'symm.inc'
      include 'rmas.inc'
      include 'cotr.inc'
      include 'adat.inc'
      include 'coor.inc'
      include 'soft.inc'
      include 'store.inc'
      include 'elct.inc'
      include 'spin.inc'
      include 'sfac.inc'
      include 'str2.inc'
      include 'nlps.inc'
      include 'pslo.inc'
      include 'atwf.inc'
      include 'kpts.inc'
      include 'kpnt.inc'
      include 'fft.inc'
      include 'ropt.inc'
      include 'isos.inc'
      include 'mm_input.inc'
      include 'response_p.inc'
c     ==--------------------------------------------------------------==
      integer nstate
      complex*16 c0(ngw,nstate),c1(ngw,nstate)
c     ==--------------------------------------------------------------==
c     internal variables
      integer     il_rhoe, il_psi,  lscr, msglen
      character  tag*30
c     ==--------------------------------------------------------------==
c     Wavefunctions, densities and related:
      complex*16 psi(*),rhoe(*),drhoe(*)
      pointer    (ip_psi,psi),(ip_rhoe,rhoe),(ip_drhoe,drhoe)
      real*8     FNL(*), DFNL(*)

c     potentials and related:
      complex*16 eirop(*),eivps(*)
      pointer    (ip_eirop,eirop),(ip_eivps,eivps)

c     others & scratch:
      real*8     z11(nstate,nstate)
      pointer    (ip_z11,z11)
      real*8     scr(*)
      pointer    (ip_scr,scr)

c     ==--------------------------------------------------------------==
      IF (TPGAUTO .or. TMSYM .or. INDPG.NE.0) THEN
         call stopgm('do_perturb',
     &       'POINT GROUP symmetry not implemented.')
      ENDIF
      IF (TLSD) THEN
         IF (traman .or. tlanphon .or.
     &        tnmr .or. tkpert .or. tinteraction .or.
     &        teigensystem .or. thardness) THEN
            if (parent) then
               write (6,*) 'LSD calculation'
     &              //' not yet tested. Please complain to'
               write (6,*) 'D.S. (sebastia@mpip-mainz.mpg.de)'
            endif
         ENDIF
         IF (tlanphon .or.
     &       tkpert .or. tinteraction .or.
     &       teigensystem .or. thardness) THEN
            call stopgm('do_perturb','LSD: drugs are illegal'
     &           //'in the U.S. while guns are not.')
         ENDIF
      ENDIF
      if (QMMM) then
         write (6,*) '!! QM/MM + LINEAR RESPONSE: This combination is '
     &      // ' not yet fully tested.  !!'
         write (6,*) '!! QM/MM + LINEAR RESPONSE: Please use with '
     &      // 'special care.              !!'
      endif
      if (TKPNT .and. .not. tkpert) 
     &     call stopgm('do_perturb','K POINTS NOT IMPLEMENTED.')
      if (TIVAN) call stopgm('do_perturb','VDB USPP NOT IMPLEMENTED.')
      if (TESR .or. IESR .ne.0) 
     &     call stopgm('do_perturb','TESR/IESR NOT IMPLEMENTED.')
      if (PS_TYPE.eq.1) 
     &     call stopgm('do_perturb','HOCKNEY NOT IMPLEMENTED.')
      if (TCLUST .and. .not.qmmm) then
         write (6,*) '!! BE CAREFUL WITH ISOLATED MOLECULES        !!'
         write (6,*) '!! NOT FULLY TESTED WITH PERTURBATION THEORY !!'
      endif
      if (TLSE) then
        call stopgm('do_perturb','LSE NOT IMPLEMENTED.')
      endif
      RESPONSE_RUNNING = .TRUE. ! This is for WRENER
c     ==--------------------------------------------------------------==
      IF(PARENT) THEN
        WRITE(6,'(/,1X,64("*"))')
        WRITE(6,'(" *",19X,A,20X,"*")') ' RESPONSE CALCULATIONS '
        WRITE(6,'(1X,64("*"),/)')
C     Format: 65 Spalten
 81     FORMAT ("* RESPONSE TYPE:        ",A35,5X,"*")
 82     FORMAT ("* PRECONDITIONER TYPE:  ",A35,5X,"*")
 83     FORMAT ("* PRECONDITIONER CUTOFF:",F10.5,30X,"*")
 84     FORMAT ("* WFN CONVERGENCE:      ",E12.3,28X,"*")
 85     FORMAT ("* ALGORITHM:            ",A35,5X,"*")
 86     FORMAT ("* CG: ANALYTIC STEPS:   ",I4,36X,"*"
     &      /,  "* CG: LENGTH PREFACTOR: ",F10.5,30X,"*")
 87     FORMAT ("* ORTHO-SCHEME:         ",A35,5X,"*")
 88     FORMAT ("* GND STATE WFN:        ",A35,5X,"*")
 89     FORMAT ("* ",22X,A35,5X,"*")
 90     FORMAT (56("*"),"RESPONSE*")
        write (6,*)
        write (6,90)
        if (traman)   write (6,81)'RAMAN spectra                   '
        if (phonon)   write (6,81)'PHONON frequencies              '
        if (tlanphon) write (6,81)'PHONON frequencies (Lanczos)    '
        if (tnmr)     write (6,81)'NMR chemical shifts             '
        if (tepr)     write (6,81)'EPR G tensor / Hyperfine tensor '
        if (tkpert)   write (6,81)'K point effect as linear response'
        if (tINTERACTION) write (6,81)'INTERACTION of Wannier orbitals '
        if (Teigensystem)
     &      write (6,81)          'EIGENSYSTEM or softness states  '
        if (THARDNESS)
     &      write (6,81)          'HARDNESS kernel diagonalization '
        if (PRECONDITIONER_P.eq.1) then
          write (6,82)            'smooth (ds)                     '
        else if (PRECONDITIONER_P.EQ.2) then
          write (6,82)            'tight (ap)                      '
        else IF (PRECONDITIONER_P.EQ.3) THEN
          write  (6,82) 'State dependent + DS            '
        else IF (PRECONDITIONER_P.EQ.4) THEN
          write  (6,82) 'State dependent + AP            '
        else if (PRECONDITIONER_P.EQ.5) then
          write (6,82)  'precondition with cutoff        '
        endif   
        write (6,83) hthrs_p
        write (6,84) tolog_p
        if (tkeeprealspacewfn) then
          write (6,88) 'KEPT in REAL SPACE'
        else
          write (6,88) 'reciprocal space only'
        endif
        if (PCG_P) then
          if (PREC_P) then
            write (6,85) 'preconditioned conj gradient    '
          else
            write (6,85) 'bare conj gradient              '
          endif
          if (TPOLAK_P) then
            write (6,89) 'Polak-Ribiere variant           '
          else
            write (6,89) 'Fletcher-Reeves variant         '
          endif
          if (PCGMIN_P) then
            write (6,86) CG_ANALYTIC,CG_FACTOR
          else
            write (6,89) 'using FIXED STEP LENGTH         '
          endif
        elseif (TSDE_P) then
          if(PREC_P) then
            write (6,85) 'preconditioned steepest descent '
          else
            write (6,85) 'bare steepest descent           '
          endif
        elseif (DIIS_P) then
          if(PREC_P) then
             write (6,85) 'preconditioned DIIS            '
          else
            write (6,85) 'bare DIIS optimization          '
          endif
        endif
        if (TINTERACTION) then
          if (TLINEARSCALING) then
            write (6,87) 'MOLECULAR'
          else
            write (6,87) 'complete (std)'
          endif
          if (FASTDEXC) write (6,89) 'Exc second derivative using LDA '
          if (TATOMICWAVEFUNCTION)
     $        write (6,89) 'Atomic wavefunction for W0      '
          if (TATOMICRHO)
     $        write (6,89) 'Atomic density for Rho0         '
          if (.NOT.TORTHOG_WANNIER)
     $        write (6,89) 'Using NON ORTHOGONAL W0         '
          if (TSIMPLE_MODEL)
     $        write (6,89) 'Using SIMPLE interaction model  '
          if (TMANNO)
     $        write (6,89) 'Using OLD MANNOPT code (serial) '
        endif
        write (6,90)
      ENDIF                     ! Parent.
c     ==--------------------------------------------------------------==
c     First, ALLOCATE MEMORY which is needed for any kind of perturbation.
c     Then, the specific perturbation subroutine is called.
c     ==--------------------------------------------------------------==
c     Wavefunctions, densities and related:
      call rhoe_psi_size(il_rhoe,il_psi)
      call memory(ip_psi,il_psi,'psi')
      call azzero(psi,il_psi)
      call memory(ip_rhoe,il_rhoe,'rhoe')
      call azzero(rhoe,il_rhoe)

       if ((TNMR .OR. TKPERT).OR.TEPR) then 
         CALL GET_ADDR(IP_rho0,rhoe(1))
         CALL GET_ADDR(IP_drhoe,rhoe(1)) 
         CALL GET_ADDR(IP_vxc0,rhoe(1)) 
         TRHO1 = .false.
       else
         call memory(ip_rho0,il_rhoe,'rho0')
         call azzero(rho0,il_rhoe)
         call memory(ip_drhoe,il_rhoe,'drhoe')
         call azzero(drhoe,il_rhoe)
         call memory(ip_vxc0,il_rhoe,'vxc0')
         call azzero(vxc0,il_rhoe)
         TRHO1 = .true.
       endif 

c     potentials and related:
      call memory(ip_vofrho0,il_rhoe,'v[n0]')
      call azzero(vofrho0,il_rhoe)
      call memory(ip_eirop,2*nhg,'eirop')
      call azzero(eirop,2*nhg)
      call memory(ip_eivps,2*nhg,'eivps')
      call azzero(eivps,2*nhg)


c     others & scratch:
      call memory(ip_z11,nstate*nstate,'z11')
      call azzero(z11,nstate*nstate)
      if(.not.tmdfile) call memory(ip_fion,3*nax*nsx,'fion')
      call azzero(fion,3*nax*nsx)
      call give_scr_perturbation(lscr,tag,nstate)

      call memory(ip_scr,lscr,'scr')
      call azzero(scr,lscr)
c     ==--------------------------------------------------------------==
      call phfac(tau0)
      call eicalc(eivps,eirop)
      if(tmemchk) call print_memory

c     the potential v[n0]:
      if (.not. TINTERACTION) ! in the INTERACTION case, the wfns
     &  then                  ! are restarted lateron
        call rhoofr(c0,rho0,psi,scr,lscr,nstate)
        if(ttau) call stopgm("inr_dr","no tau functionals")
        call DCOPY(nnr1*nlsd, rho0, 1, vofrho0, 1)
        call vofrho(tau0,fion,vofrho0,psi,scr,lscr,.false.,.false.)
      endif

      if(parent) call prmem('  DO_PERTURBATION')
c     calculate the lagrange parameters z11
      if ((.not. TINTERACTION) .and. (.NOT. (TNMR.OR.TEPR))) then
        call lag_mult(c0,c1,psi,scr,lscr,rhoe,z11,nstate)
      endif
c     ==--------------------------------------------------------------==
      IF(TDUMMYATOM) THEN
        call nuclear_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &      eirop,eivps,z11,nstate)
      ELSEIF(TDUMMYATOM_REF) THEN
        call nuclear_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &      eirop,eivps,z11,nstate)
        TDUMMYATOM_REF=.FALSE.
        IF(RHOOUT) THEN
          CALL RHOPRI(C0,TAU0,RHOE,PSI,SCR,LSCR,NSTATE,NKPNT)
        END IF
      ELSEIF(TDUMMYATOM_TEC) THEN
        call nuclear_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &      eirop,eivps,z11,nstate)
      ELSEIF(TOACP4F) THEN
        call nuclear_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &      eirop,eivps,z11,nstate)
      ELSEIF(TRAMAN) THEN
        if (parent) print*, '*** RAMAN calculation'
C     PAGLIAI/CINECA ADDED TAU0 TO THE CALL 
        call raman_p(TAU0,c0,c1,psi,rhoe,drhoe,scr,lscr,
     &      eirop,eivps,z11,nstate)
      ELSEIF(TINTERACTION) THEN
        if(TMANNO) then
          call interaction_manno_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &      eirop,eivps,z11,nstate)
        else
          call interaction_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &      eirop,eivps,z11,nstate)
        endif
      ELSEIF(PHONON) THEN
         if (parent) then
            write(6,'(2x,a)') '*** PHONON calculation '//
     &           '(classical diagonalization). '
            if(projout.and.rotout) then
               write(6,'(2x,a)') 'TRANSLATIONS AND ROTATIONS HAVE'//
     &              ' BEEN DISCARDED'
            ELSEIF(PROJOUT.AND..NOT.ROTOUT) THEN
               write(6,'(2x,a)') 'TRANSLATIONS ONLY HAVE BEEN'//
     &              ' DISCARDED'
            else
               write(6,'(2x,a)') 'TRIVIAL EIGENMODES ARE INCLUDED'
            endif
         endif
         call phonons_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &        eirop,eivps,z11,nstate)
      ELSEIF(tlanPHON) THEN
        if(parent) CALL DETDOF(TAU0,SCR)
#ifdef PARALLEL
         msglen=1*8/IRAT
         CALL MY_BCAST(NODIM,MSGLEN,SOURCE,ALLGRP)
#endif

         if(parent) then
            write(6,'(2x,a)') '*** PHONON calculation  '//
     &        '(LANCZOS iterative method). '
            if(projout.and.rotout) then
               write(6,'(2x,a)') 'TRANSLATIONS AND ROTATIONS HAVE'//
     &              ' BEEN DISCARDED'
            ELSEIF(PROJOUT.AND..NOT.ROTOUT) THEN
               write(6,'(2x,a)') 'TRANSLATIONS ONLY HAVE BEEN'//
     &              ' DISCARDED'
            else
               write(6,'(2x,a)') 'TRIVIAL EIGENMODES ARE INCLUDED'
            endif
         endif
         CALL lanc_phon_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &        eirop,eivps,z11,nstate)
      ELSEIF(TKPERT) THEN
         IF (parent) print *,' *** KPERT CALCULATION ***'
         CALL pert_kpoint_p(c0,c1,psi,rhoe,scr,lscr,
     &        eirop,eivps,z11,nstate)
      ELSEIF(TEIGENSYSTEM) THEN
         IF (parent) print *,' *** EIGENSYSTEM CALCULATION ***'
         CALL eigensystem_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &        eirop,eivps,z11,nstate)
      ELSEIF(THARDNESS) THEN
         IF (parent) print *,' *** HARDNESS CALCULATION ***'
         CALL HARDNESS_P(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &        eirop,eivps,z11,nstate)
      ELSEIF(TFUKUI) THEN
         IF(parent) WRITE(6,*)' *** FUKUI FUNCTIONS CALCULATION ***'
         CALL my_flush(6)
         CALL FUKUI_P(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &        eirop,eivps,z11,nstate)
      ELSEIF (TNMR) THEN
         CALL nmr_p(c0,c1,psi,rhoe,scr,lscr,
     &        eirop,eivps,z11,nstate)
      ELSEIF (TEPR) THEN
         CALL epr_p(c0,c1,psi,rhoe,scr,lscr,
     &        eirop,eivps,z11,nstate)
      ENDIF
c     ==--------------------------------------------------------------==
c     DE-ALLOCATE MEMORY
c     ==--------------------------------------------------------------==
      CALL freem(ip_scr)  ! last allocated = first de-allocated
      IF(.NOT. (TINTERACTION.or.tmdfile)) CALL freem(ip_fion)
      CALL freem(ip_z11)
      CALL freem(ip_eivps)
      CALL freem(ip_eirop)
      CALL freem(ip_vofrho0)
      IF(.NOT. (TEPR .OR. TNMR .OR. TKPERT)) THEN
        CALL freem(ip_vxc0)
        CALL freem(ip_drhoe)
        CALL freem(ip_rho0)
      ENDIF
      CALL freem(ip_rhoe)
      CALL freem(ip_psi)
c     ==--------------------------------------------------------------==
      RESPONSE_RUNNING = .FALSE.
      return
      end
c     ==================================================================
      subroutine give_scr_perturbation(l_max,tag,nstate)
      implicit none
      include 'system.h'        !nhg tgc
      include 'spin.inc'        !nlsd nlsx
      include 'ropt.inc'        ! calste
      character*(*) tag
      integer l_max,nstate,lrnlsm,lrho,lrhoofr,lrscpot,lforce1,lnmr,
     &    l_forcedr,l_opeigr,l_updwf_p,lepr,
     &    lpert_kpoint_p,linteraction
c     ==--------------------------------------------------------------==
      l_max=nstate
      calste=.false.
      call give_scr_nmr(lnmr,tag)
      call give_scr_epr(lepr,tag)
      call give_scr_interaction(linteraction,tag,nstate)
      call give_scr_rnlsm(lrnlsm,tag,nstate,.true.)
      call give_scr_rhoofr(lrho,tag)
      call give_scr_rscpot(lrscpot,tag,calste)
      call give_scr_rhoofr_p(lrhoofr,tag)
      call give_scr_forces_p(lforce1,tag,nstate)
      call give_scr_forcedr(l_forcedr,tag,nstate,.false.,.false.)
      call give_scr_opeigr(l_opeigr,tag,nstate)
      call give_scr_updwf_p(l_updwf_p,tag,nstate)
      call give_scr_pert_kpoint_p(lpert_kpoint_p,tag,nstate)

      l_max = max(linteraction,lrho,lrscpot,lrnlsm,lrhoofr,l_max,
     &    lforce1,lnmr,l_forcedr,l_opeigr,l_updwf_p,lepr,
     &    lpert_kpoint_p)
c     ==--------------------------------------------------------------==
      return
      end
c     ==================================================================
