C     ==================================================================
      SUBROUTINE EAMIN
C     ==--------------------------------------------------------------==
C     ==  THIS ROUTINE READS THE SECTION &EAM &END ON UNIT IUNIT      ==
C     ==--------------------------------------------------------------==
C     ==  THE INPUT HAS TO BE IN THE FOLLOWING FORMAT                 ==
C     ==     &EAM                                                     ==
C     ==        Species 1 eamparam                                    ==
C     ==        Species 2 eamparam                                    ==
C     ==        ...                                                   ==
C     ==        Species n eamparam                                    ==
C     ==     &END                                                     ==
C     ==--------------------------------------------------------------==
C     ==  rc re fe phi_e alpha beta gamma rho_e PHI_e E_c             ==
C     ==  rc re : bohr                                                ==
C     ==  fe    : dimensionless                                       ==
C     ==  phi_e : eV                                                  ==
C     ==  alpha beta gamma : dimensionless                            ==
C     ==  rho_e : dimensionless                                       ==
C     ==  PHI_e : eV                                                  ==
C     ==  E_c   : eV                                                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'eam.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'adat.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'dpot.inc'
      INTEGER   IUNIT,IERR,INSCAN
      INTEGER   MSGLEN,IS,IS2,I
C     ==--------------------------------------------------------------==
C     ==  DEFAULTS                                                    ==
C     ==--------------------------------------------------------------==
      IF (.NOT.PARENT) GOTO 9999
      IUNIT = 5
      CALL AZZERO(EAMRE ,MAXSP)
      CALL AZZERO(EAMFE ,MAXSP)
      CALL AZZERO(EAMPE ,MAXSP)
      CALL AZZERO(EAMA  ,MAXSP)
      CALL AZZERO(EAMB  ,MAXSP)
      CALL AZZERO(EAMC  ,MAXSP)
      CALL AZZERO(EAMRHO,MAXSP)
      CALL AZZERO(EAMBPE,MAXSP)
      CALL AZZERO(EAMEC ,MAXSP)
      CALL AZZERO(EAMCUT,MAXSP)
      CALL AZZERO(EAMSMOOTH,MAXSP)
      DO IS2=1,NSP
        DO IS=1,NSP
          TEAM_CROSS(IS,IS2) = .FALSE.
        END DO
      END DO
C     ==--------------------------------------------------------------==
      IERR=INSCAN(IUNIT,'&EAM')
      IF (IERR.NE.0) GOTO 200
C     ==--------------------------------------------------------------==
      DO IS=1,NSP
        READ(IUNIT,END=300,ERR=300,FMT=*)
     *       EAMRE(IS),EAMFE(IS),EAMPE(IS),
     *       EAMA(IS),EAMB(IS),EAMC(IS),EAMRHO(IS),
     *       EAMBPE(IS),EAMEC(IS),EAMCUT(IS),EAMSMOOTH(IS)
      END DO
      DO I=1,NSP*NSP
        READ(IUNIT,END=100,ERR=100,FMT=*) IS,IS2
        IF (TEAM(IS).AND.TEAM(IS2)) TEAM_CROSS(IS,IS2)=.TRUE.
      END DO
C     ==--------------------------------------------------------------==
  100 CONTINUE
      DO IS2=1,NSP
        DO IS=1,NSP
          IF ( TEAM_CROSS(IS,IS2) ) THEN
            TEAM_CROSS(IS2,IS)=.TRUE.
            TEAM(IS)=.TRUE.
            TEAM(IS2)=.TRUE.
          END IF
        END DO
      END DO
C     ==--------------------------------------------------------------==
C     ==  WRITE INFO TO OUTPUT                                        ==
C     ==--------------------------------------------------------------==
      WRITE(6,*)
      WRITE(6,'(A)') ' EMBEDDED-ATOM-MODEL PARAMETER '
      DO IS=1,NSP
        WRITE(6,'(A,A3,T20,A,F7.2,A,T48,A,F7.2,A)') 
     *       " ATOM= ",EL(IATYP(IS)),"r_c = ",EAMCUT(IS)," a.u.",
     *       "r_e = ",EAMRE(IS)," a.u. "
        WRITE(6,'(T19,A,F9.5,A)') 
     *       " r_s = ",EAMSMOOTH(IS)," a.u."
        WRITE(6,'(T20,A,F7.2,T48,A,F7.2,A)') 
     *       "f_e = ",EAMFE(IS),"phi_e = ",EAMPE(IS)," eV"
        WRITE(6,'(T20,A,F7.2,T40,A,F7.2,T55,A,F7.2)') 
     *       "a = ",EAMA(IS),"b = ",EAMB(IS),"c = ",EAMC(IS)
        WRITE(6,'(T20,A,F7.2,T48,A,F7.2,A)') 
     *       "rho_e = ",EAMRHO(IS),"PHI_e = ",EAMBPE(IS)," eV"
        WRITE(6,'(T33,A,F7.2,A)') "Cohesive energy: E_c = ",
     *       EAMEC(IS)," eV"
      END DO
      WRITE(6,*)
C     ==--------------------------------------------------------------==
      DO IS=1,NSP
        EAMPE(IS)=EAMPE(IS)/(2.D0*RY)
        EAMBPE(IS)=EAMBPE(IS)/(2.D0*RY)
        EAMEC(IS)=EAMEC(IS)/(2.D0*RY)
      END DO
C     ==--------------------------------------------------------------==
 9999 CONTINUE
      
#ifdef PARALLEL
      MSGLEN = NSP * 8
      CALL MY_BCAST(EAMRE,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EAMFE,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EAMPE,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EAMA,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EAMB,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EAMC,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EAMRHO,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EAMBPE,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EAMEC,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EAMCUT,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EAMSMOOTH,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = MAXSP * NSP * 8/IRAT
      CALL MY_BCAST(TEAM_CROSS,MSGLEN,SOURCE,ALLGRP)
#endif
      RETURN
      
 200  CONTINUE
      WRITE(6,*) "EAMIN: No &EAM section"
      CALL STOPGM("EAMIN","No &EAM section")
      
 300  CONTINUE
      WRITE(6,*) "EAMIN: Error reading &EAM section"
      CALL STOPGM("EAMIN","Error reading &EAM section")
C     ==--------------------------------------------------------------==
      END
C     ==================================================================
      SUBROUTINE EAM_POT(ESR,TAU0,IESR,FION,TFOR)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'eam.inc'
      INCLUDE 'dpot.inc'
C     Arguments
      REAL*8  ESR,TAU0(3,NAX,NSP),FION(3,NAX,NSP)
      LOGICAL TFOR
      INTEGER IESR
C     Variables
      REAL*8    SKIN, SMOOTH_EPS, DENS_EPS
      PARAMETER (SKIN=2.0D0,SMOOTH_EPS=1.0D-12,DENS_EPS=1.0D-12)
C..maximum number of nearest neighbours on the list = NMAXNN
      INTEGER   NMAXNN
      PARAMETER (NMAXNN=100)
      LOGICAL   LUPDATE, TPRINT
      INTEGER   ISUB,IAT,JAT,IS1,IS2,IA1,IA2,IDAMAX,IOFF,II
      INTEGER   ATIDX(NAX,NSP)
      REAL*8    DENS(*),OTAU(3,NAX,NSP),EFIO(3,NAX,NSP),DESR
      REAL*8    DD,DX,DY,DZ,DCUT,REDIST,DFI,DFJ,XP,YP,FR12,RSMOOTH
      REAL*8    FE,B,RE,RR,X,Y,RRHO,PAA,PBB,FA,FB,CUTOFF,DCUTOFF
      REAL*8    B1,B2,C1,C2,R1,R2,RHO,DX1,DY1,DZ1, TRANSL(3,NMAXNN,*)
      INTEGER   NLISTE(NMAXNN,2,*)
      POINTER   (IP_DENS,DENS),(IP_OTAU,OTAU),(IP_NLISTE,NLISTE)
      POINTER   (IP_EFIO,EFIO),(IP_TRANSL,TRANSL)
      INTEGER   IFLAG,LEN
      DATA      IFLAG /0/
      SAVE      IFLAG,IP_DENS,IP_OTAU,IP_EFIO,IP_NLISTE,IP_TRANSL,REDIST
C     ==--------------------------------------------------------------==
      IF (IFLAG.EQ.0) THEN
        CALL MEMORY(IP_DENS,NAT,"DENS")
        CALL MEMORY(IP_OTAU,3*NAX*NSP,"OTAU")
        CALL MEMORY(IP_EFIO,3*NAX*NSP,"EFIO")
        NATPE=IPEPT(2,MEPOS)-IPEPT(1,MEPOS)+1
        LEN=(NMAXNN*2*NATPE+1)/IRAT
        CALL MEMORY(IP_NLISTE,LEN,"NLISTE")
        LEN=(3*NMAXNN*NATPE)
        CALL MEMORY(IP_TRANSL,LEN,"TRANSL")
        LUPDATE=.TRUE.
        IAT=IDAMAX(NSP,EAMCUT,1)
        JAT=IDAMAX(NSP,EAMSMOOTH,1)
        REDIST=EAMCUT(IAT)+SKIN+EAMSMOOTH(JAT)*LOG(1.D0/SMOOTH_EPS-1.D0)
        REDIST=REDIST**2
        IFLAG=1
        TPRINT=.TRUE.
      ELSE
        CALL CHECK_TAU(TAU0,OTAU,SKIN,LUPDATE)
        TPRINT=.FALSE.
      END IF
      IF (LUPDATE) CALL DO_LIST(TAU0,NLISTE,REDIST,IESR,NMAXNN,TRANSL)
C     ==--------------------------------------------------------------==
      CALL TISET('   EAM_POT',ISUB)
      
      IAT=0
      DO IS1=1,NSP
        DO IA1=1,NA(IS1)
          IAT=IAT+1
          ATIDX(IA1,IS1) = IAT
        END DO
      END DO
      
C..calculate the density
      IOFF=IPEPT(1,MEPOS)-1
      CALL AZZERO(DENS,NAT)
      IAT=0
      DO IS1=1,NSP
        IF (.NOT.TEAM(IS1)) GO TO 1100
        DO IA1=1,NA(IS1)
          IAT=IAT+1
          II=IAT-IOFF
#ifdef PARALLEL
          IF (IATPE(IAT).NE.MEPOS) GOTO 1000
#endif
          DO JAT=1,NMAXNN
            IF (NLISTE(JAT,1,II).EQ.0) GOTO 1000
            IA2=NLISTE(JAT,1,II)
            IS2=NLISTE(JAT,2,II)
            
            IF (.NOT.TEAM(IS2).OR..NOT.TEAM_CROSS(IS1,IS2)) GO TO 1200
            
            DX=TAU0(1,IA1,IS1)-TAU0(1,IA2,IS2)
            DY=TAU0(2,IA1,IS1)-TAU0(2,IA2,IS2)
            DZ=TAU0(3,IA1,IS1)-TAU0(3,IA2,IS2)
            
            DCUT=(EAMCUT(IS1)+EAMCUT(IS2))/2.D0
            RSMOOTH=(EAMSMOOTH(IS1)+EAMSMOOTH(IS2))/2.D0
            
            DX1=DX+TRANSL(1,JAT,II)
            DY1=DY+TRANSL(2,JAT,II)
            DZ1=DZ+TRANSL(3,JAT,II)
            DD=SQRT(DX1*DX1+DY1*DY1+DZ1*DZ1)
            
            CUTOFF=1.D0/(EXP((DD-DCUT)/RSMOOTH)+1.D0)
            IF (CUTOFF.GT.SMOOTH_EPS) THEN
              RE=EAMRE(IS2)
              FE=EAMFE(IS2)
              B =EAMB (IS2)
              RR=DD/RE
              DENS(ATIDX(IA1,IS1))=DENS(ATIDX(IA1,IS1))
     $             + CUTOFF * FE*EXP(-B*(RR-1.D0))
              RE=EAMRE(IS1)
              FE=EAMFE(IS1)
              B =EAMB (IS1)
              RR=DD/RE
              DENS(ATIDX(IA2,IS2))=DENS(ATIDX(IA2,IS2))
     $             + CUTOFF * FE*EXP(-B*(RR-1.D0))
            END IF
            
 1200       CONTINUE
          END DO
          
 1000     CONTINUE
        END DO
        
 1100   CONTINUE
      END DO
      CALL GLOSUM(NAT,DENS)
      
      IF ( TPRINT .AND. PARENT ) THEN
        WRITE(6,'(/,A,F20.16,/)') "DBG: rho(nat) = ", DENS(NAT)
      END IF
      
C..calculate energy and forces
      IF (TFOR) CALL AZZERO(EFIO,3*NAX*NSP)
      IAT=0
      DO IS1=1,NSP
        IF (.NOT.TEAM(IS1)) GO TO 2100
        
!*apsi* TMPTMPTMP
!        EAMEC(:NSP) = 0.0
!        EAMBPE(:NSP) = 0.0
!        EAMPE(:NSP) = 0.0
        
        DO IA1=1,NA(IS1)
          IAT=IAT+1
          II=IAT-IOFF
#ifdef PARALLEL
          IF (IATPE(IAT).NE.MEPOS) GOTO 2000
#endif
          RHO=DENS(ATIDX(IA1,IS1))
          IF (RHO.GT.DENS_EPS) THEN
            RRHO=RHO/EAMRHO(IS1)
            X=RRHO**(EAMA(IS1)/EAMB(IS1))
            Y=RRHO**(EAMC(IS1)/EAMB(IS1))
            ESR=ESR-EAMEC(IS1)*(1.D0-LOG(X))*X-EAMBPE(IS1)*Y
            XP=EAMA(IS1)/EAMB(IS1) * X/RHO
            YP=EAMC(IS1)/EAMB(IS1) * Y/RHO
            DFI=EAMEC(IS1)*LOG(X)*XP-EAMBPE(IS1)*YP
          ELSE
            DFI=0.D0
          END IF
          
          DO JAT=1,NMAXNN
            IF (NLISTE(JAT,1,II).EQ.0) GOTO 2000
            IA2=NLISTE(JAT,1,II)
            IS2=NLISTE(JAT,2,II)
            IF (.NOT.TEAM(IS2).OR..NOT.TEAM_CROSS(IS1,IS2)) GO TO 2200
            
            RHO=DENS(ATIDX(IA2,IS2))
            IF (RHO.GT.DENS_EPS) THEN
              RRHO=RHO/EAMRHO(IS2)
              X=RRHO**(EAMA(IS2)/EAMB(IS2))
              Y=RRHO**(EAMC(IS2)/EAMB(IS2))
              
              XP=EAMA(IS2)/EAMB(IS2) * X/RHO
              YP=EAMC(IS2)/EAMB(IS2) * Y/RHO
              DFJ=EAMEC(IS2)*LOG(X)*XP-EAMBPE(IS2)*YP
            ELSE
              DFJ=0.D0
            END IF
            
            DX=TAU0(1,IA1,IS1)-TAU0(1,IA2,IS2)
            DY=TAU0(2,IA1,IS1)-TAU0(2,IA2,IS2)
            DZ=TAU0(3,IA1,IS1)-TAU0(3,IA2,IS2)
            
            DX1=DX+TRANSL(1,JAT,II)
            DY1=DY+TRANSL(2,JAT,II)
            DZ1=DZ+TRANSL(3,JAT,II)
            DD=SQRT(DX1*DX1+DY1*DY1+DZ1*DZ1)
            
            FR12=0.D0
            DESR=0.D0
            DCUT=(EAMCUT(IS1)+EAMCUT(IS2))/2.D0
            RSMOOTH=(EAMSMOOTH(IS1)+EAMSMOOTH(IS2))/2.D0
            CUTOFF=1.D0/(EXP((DD-DCUT)/RSMOOTH)+1.D0)
            DCUTOFF=-EXP((DD-DCUT)/RSMOOTH)/RSMOOTH*CUTOFF**2
            
            IF (CUTOFF.GT.SMOOTH_EPS) THEN
              IF (IS1.EQ.IS2) THEN
                RR=DD/EAMRE(IS1)
                FA=EAMPE(IS1)*EXP(-EAMC(IS1)*(RR-1.D0))
                DESR=DESR+FA
                IF (TFOR) THEN
                  FR12=FR12-EAMC(IS1)/EAMRE(IS1)*FA
                END IF
              ELSE
                R1=EAMRE(IS1)
                B1=EAMB(IS1)
                C1=EAMC(IS1)
                RR=DD/R1
                FA=EAMFE(IS1)*EXP(-B1*(RR-1.D0))
                PAA=EAMPE(IS1)*EXP(-C1*(RR-1.D0))
                
                R2=EAMRE(IS2)
                B2=EAMB(IS2)
                C2=EAMC(IS2)
                RR=DD/EAMRE(IS2)
                FB=EAMFE(IS2)*EXP(-B2*(RR-1.D0))
                PBB=EAMPE(IS2)*EXP(-C2*(RR-1.D0))
                
                DESR=DESR+0.5D0*(FB/FA*PAA+FA/FB*PBB)
                IF (TFOR) THEN
                  FR12=FR12+0.5D0*
     *                 ( (-B2/R2+B1/R1-C1/R1)*FB/FA*PAA
     *                 + (-B1/R1+B2/R2-C2/R2)*FA/FB*PBB )
                END IF
              END IF
              
              IF (TFOR) THEN
                R1=DD/EAMRE(IS1)
                R2=DD/EAMRE(IS2)
                FR12=FR12
     *               +DFI*EAMFE(IS2)*EXP(-EAMB(IS2)*(R2-1.D0))*
     *               (-EAMB(IS2)/EAMRE(IS2))
     *               +DFJ*EAMFE(IS1)*EXP(-EAMB(IS1)*(R1-1.D0))*
     *               (-EAMB(IS1)/EAMRE(IS1))
                FR12=FR12*CUTOFF
     $               +(DFI*EAMFE(IS2)
     $               *EXP(-EAMB(IS2)*(R2-1.D0))+DESR*0.5D0)*DCUTOFF
     $               +(DFJ*EAMFE(IS1)
     $               *EXP(-EAMB(IS1)*(R1-1.D0))+DESR*0.5D0)*DCUTOFF
                EFIO(1,IA1,IS1)=EFIO(1,IA1,IS1)-DX1/DD*FR12
                EFIO(2,IA1,IS1)=EFIO(2,IA1,IS1)-DY1/DD*FR12
                EFIO(3,IA1,IS1)=EFIO(3,IA1,IS1)-DZ1/DD*FR12
                EFIO(1,IA2,IS2)=EFIO(1,IA2,IS2)+DX1/DD*FR12
                EFIO(2,IA2,IS2)=EFIO(2,IA2,IS2)+DY1/DD*FR12
                EFIO(3,IA2,IS2)=EFIO(3,IA2,IS2)+DZ1/DD*FR12
              END IF
              
              ESR = ESR + DESR * CUTOFF
            END IF
            
 2200       CONTINUE
          END DO
          
 2000     CONTINUE
        END DO
        
 2100   CONTINUE
      END DO
      
      IF ( TFOR ) THEN
        DO IS1=1,NSP
          DO IA1=1,NA(IS1)
            FION(1,IA1,IS1)=FION(1,IA1,IS1)+EFIO(1,IA1,IS1)
            FION(2,IA1,IS1)=FION(2,IA1,IS1)+EFIO(2,IA1,IS1)
            FION(3,IA1,IS1)=FION(3,IA1,IS1)+EFIO(3,IA1,IS1)
          END DO
        END DO
      END IF
      
      CALL TIHALT('   EAM_POT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CHECK_TAU(TAU0,OTAU,SKIN,LUPDATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'metr.inc'
C     Arguments
      REAL*8  TAU0(3,NAX,NSP),OTAU(3,NAX,NSP),SKIN
      LOGICAL LUPDATE
C     VARIABLES
      INTEGER IA,IS
      REAL*8  DD,DX,DY,DZ,DCUT
C     ==--------------------------------------------------------------==
      DCUT=SKIN**2
      LUPDATE=.FALSE.
      DO IS=1,NSP
        DO IA=1,NA(IS)
          DX=TAU0(1,IA,IS)-OTAU(1,IA,IS)
          DY=TAU0(2,IA,IS)-OTAU(2,IA,IS)
          DZ=TAU0(3,IA,IS)-OTAU(3,IA,IS)
          CALL PBC(DX,DY,DZ,DX,DY,DZ,1,APBC,IBRAV)
          DD=DX*DX+DY*DY+DZ*DZ
          LUPDATE=LUPDATE .OR. (DD.GT.DCUT)
        END DO
      END DO
      IF (LUPDATE) CALL DCOPY(3*NAX*NSP,TAU0,1,OTAU,1)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE DO_LIST(TAU0,NLISTE,REDIST,IESR,NMAXNN,TRANSL)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'eam.inc'
C     Arguments
      INTEGER NMAXNN
      REAL*8  TAU0(3,NAX,NSP),REDIST,TRANSL(3,NMAXNN,*)
      INTEGER NLISTE(NMAXNN,2,*),IESR
C     Variables
      INTEGER IS1,IS2,IA1,IA2,JAT,IAT1,II,IOFF,IAS
      INTEGER IX,IY,IZ,MINX
      REAL*8  DD,DX,DY,DZ,DX1,DY1,DZ1,DX2,DY2,DZ2
C     ==--------------------------------------------------------------==
      NATPE=IPEPT(2,MEPOS)-IPEPT(1,MEPOS)+1
      CALL IAZZERO(NLISTE,NMAXNN*2*NATPE)
      CALL AZZERO(TRANSL,3*NMAXNN*NATPE)
      IOFF=IPEPT(1,MEPOS)-1
      IAT1=0
      DO IS1=1,NSP
        DO IA1=1,NA(IS1)
          IAT1=IAT1+1
          II=IAT1-IOFF
#ifdef PARALLEL
          IF (IATPE(IAT1).NE.MEPOS) GOTO 1000
#endif
          JAT=0
          DO IS2=IS1,NSP
            IAS=1
            IF (IS1.EQ.IS2) IAS=IA1
            DO IA2=IAS,NA(IS2)
              DX=TAU0(1,IA1,IS1)-TAU0(1,IA2,IS2)
              DY=TAU0(2,IA1,IS1)-TAU0(2,IA2,IS2)
              DZ=TAU0(3,IA1,IS1)-TAU0(3,IA2,IS2)
              CALL PBC(DX,DY,DZ,DX1,DY1,DZ1,1,APBC,IBRAV)
              
              MINX=-IESR
              IF (IA1.EQ.IA2.AND.IS1.EQ.IS2) MINX=0
              
              DO IZ=-IESR,IESR
                DO IY=MINX,IESR
                  DO IX=MINX,IESR
                    DX2=DX1+IX*HT(1,1)+IY*HT(2,1)+IZ*HT(3,1)
                    DY2=DY1+IX*HT(1,2)+IY*HT(2,2)+IZ*HT(3,2)
                    DZ2=DZ1+IX*HT(1,3)+IY*HT(2,3)+IZ*HT(3,3)
                    DD=DX2*DX2+DY2*DY2+DZ2*DZ2
                    
                    IF (DD.LT.REDIST.AND.DD.GT.1.0D-10) THEN
                      JAT=JAT+1
                      IF (JAT.GT.NMAXNN) THEN
                        CALL STOPGM('DO_LIST','TOO MANY NEIGHBOURS')
                      END IF
                      NLISTE(JAT,1,II)=IA2
                      NLISTE(JAT,2,II)=IS2
                      TRANSL(1,JAT,II)=DX2-DX
                      TRANSL(2,JAT,II)=DY2-DY
                      TRANSL(3,JAT,II)=DZ2-DZ
                    END IF
                  END DO
                END DO
              END DO
              
            END DO
          END DO
          
 1000     CONTINUE
        END DO
      END DO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
