C     ==================================================================
      SUBROUTINE EEXTERN(RHOE,V,EIROP,TAU0,FION,EEXT,TFOR)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  ENERGY OF INTERACTION OF AN EXTERNAL FIELD (EXTF) WITH      ==
C     ==  THE QM CHARGE DISTIBUTION (RHOE) AND THE FORCES ON THE      ==
C     ==  IONS DUE TO THIS INTERACTION                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'efld.inc'
      INCLUDE 'ragg.inc'
      INCLUDE 'epot.inc'           ! added biswas/2005: for GMX interface
      INCLUDE 'mm_ion_dens.inc'
      INCLUDE 'mm_input.inc'
C     Arguments
      REAL*8     RHOE(*),TAU0(3,NAX,*),FION(3,NAX,*),EEXT
      COMPLEX*16 V(*),EIROP(*)
      LOGICAL    TFOR
C     Variables
      DIMENSION EIGRB(NHG,NAT)
      REAL*8    EXTF(*),FNL(*),DFNL(*)
      INTEGER   ISUB,IS,IA
      REAL*8    FFI(3),GFAC,EEXT2,XI,YI,ZI,EEXTI
      REAL*8    DDOT
      EXTERNAL  DDOT
C     ==--------------------------------------------------------------==
      CALL TISET('      EEXT',ISUB)
C..ENERGY VEXT*RHOE
      EEXT=0.D0
      IF(TFOR.OR.TEXADD) THEN
        GFAC=OMEGA/(NR1S*NR2S*NR3S)
        EEXT=DDOT(NNR1,RHOE,1,EXTF,1)*GFAC
C..ENERGY VEXT*RHOI AND GRADIENT
        EEXT2=0.D0
        EEXTI=0.D0
        IF(.NOT.QMMM)THEN
C     EGO-CPMD interface
          IF(IFTYPE.EQ.1) THEN
            DO IS=1,NSP
              DO IA=1,NA(IS)
                CALL VEXTNU(EXTF,TAU0(1,IA,IS),ZV(IS),
     *               RAGGIO(IS),EEXT2,FFI,TFOR)
                FION(1,IA,IS)=FION(1,IA,IS)+FFI(1)*GFAC
                FION(2,IA,IS)=FION(2,IA,IS)+FFI(2)*GFAC
                FION(3,IA,IS)=FION(3,IA,IS)+FFI(3)*GFAC
              ENDDO
            ENDDO
            EEXT2=EEXT2*GFAC
C     Gromacs-CPMD interface. biswas/2005
          ELSE IF(IFTYPE.EQ.2.AND.PARENT) THEN
            DO IS=1,NSP
              DO IA=1,NA(IS)
                XI=TAU0(1,IA,IS)
                YI=TAU0(2,IA,IS)
                ZI=TAU0(3,IA,IS)          
                CALL VEXTNUB(IS,IA,XI,YI,ZI,ZV(IS),EEXTI,FFI)
                FION(1,IA,IS)=FION(1,IA,IS)+FFI(1)
                FION(2,IA,IS)=FION(2,IA,IS)+FFI(2)
                FION(3,IA,IS)=FION(3,IA,IS)+FFI(3)
                EEXT2 = EEXT2 + EEXTI
              ENDDO
            ENDDO
          ENDIF
        ENDIF
        EEXT=EEXT+EEXT2
      ENDIF
      CALL TIHALT('      EEXT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     external potential for the EGO-CPMD interface
C     ==================================================================
      SUBROUTINE VEXTNU(EXTF,TAU,ZVAL,RAGGIO,EEXT,FORC,TFOR)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'qspl.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'cnst.inc'
C     Arguments
      REAL*8  EXTF(KR1,KR2,*),TAU(*),ZVAL,RAGGIO,EEXT,FORC(3)
      LOGICAL TFOR
C     Variables
      REAL*8    REPS
      PARAMETER (REPS=1.D-8)
      INTEGER   NPTAU(3),MR(3),I,IORGX,IORGY,IORGZ,IX,II,I1,
     &          J,JJ,J1,K,KK,K1
      REAL*8    SC1(3),SC2(3),XPTAU(3),DA1(3,3),ZR1,ZR2,X1,Y1,Z1,
     &          XX1,XX2,RCUT,RCUT2,X,Y,Z,R2,XVAL,YVAL
C     ==--------------------------------------------------------------==
      ZR1=-ZVAL/RAGGIO**3*PI**(-1.5D0)
      ZR2=1.D0/RAGGIO**2
      FORC(1)=0.D0
      FORC(2)=0.D0
      FORC(3)=0.D0
C..Atom position
      X1=TAU(1)
      Y1=TAU(2)
      Z1=TAU(3)
      CALL PBC(X1,Y1,Z1,SC1(1),SC1(2),SC1(3),1,APBC,IBRAV)
      CALL DGEMV('T',3,3,1.D0,HTM1,3,SC1,1,0.D0,SC2,1)
      IF(SC2(1).LT.0.D0) SC2(1)=SC2(1)+1.0D0
      IF(SC2(2).LT.0.D0) SC2(2)=SC2(2)+1.0D0
      IF(SC2(3).LT.0.D0) SC2(3)=SC2(3)+1.0D0
      NPTAU(1)=NINT(SC2(1)*NR1S)+1
      NPTAU(2)=NINT(SC2(2)*NR2S)+1
      NPTAU(3)=NINT(SC2(3)*NR3S)+1
      SC1(1)=SC2(1)-DBLE(NPTAU(1)-1)/DBLE(NR1S)
      SC1(2)=SC2(2)-DBLE(NPTAU(2)-1)/DBLE(NR2S)
      SC1(3)=SC2(3)-DBLE(NPTAU(3)-1)/DBLE(NR3S)
      IF(NPTAU(1).GT.NR1S) NPTAU(1)=1
      IF(NPTAU(2).GT.NR2S) NPTAU(2)=1
      IF(NPTAU(3).GT.NR3S) NPTAU(3)=1
      CALL DGEMV('T',3,3,1.D0,HT,3,SC1,1,0.D0,XPTAU,1)
      DO I=1,3
        DA1(I,1)=A1(I)/DBLE(NR1S)
        DA1(I,2)=A2(I)/DBLE(NR2S)
        DA1(I,3)=A3(I)/DBLE(NR3S)
      ENDDO
C..Interaction region
      XX1=REPS*RAGGIO**3/ZVAL*PI**1.5D0
      XX2=LOG(XX1)
      RCUT=-RAGGIO**2*XX2
      RCUT2=RCUT*RCUT
      SC1(1)=RCUT
      SC1(2)=RCUT
      SC1(3)=RCUT
      CALL DGEMV('T',3,3,1.D0,HTM1,3,SC1,1,0.D0,SC2,1)
      MR(1)=NINT(SC2(1)*NR1S)+1
      MR(2)=NINT(SC2(2)*NR2S)+1
      MR(3)=NINT(SC2(3)*NR3S)+1
      MR(1)=MIN(MR(1),NR1S/2-1)
      MR(2)=MIN(MR(2),NR2S/2-1)
      MR(3)=MIN(MR(3),NR3S/2-1)
C..Origin of density grid
      IORGX=NPTAU(1)-MR(1)
      IORGY=NPTAU(2)-MR(2)
      IORGZ=NPTAU(3)-MR(3)
      IF(IORGX.LT.1) IORGX=IORGX+NR1S
      IF(IORGY.LT.1) IORGY=IORGY+NR2S
      IF(IORGZ.LT.1) IORGZ=IORGZ+NR3S
C..loop over all grid points
      DO I=0,2*MR(1)
        IX=IORGX+I
        IF(IX.GT.NR1S) IX=IX-NR1S
        IF(IX.GE.NRXPL(MEPOS,1).AND.IX.LE.NRXPL(MEPOS,2)) THEN
          II=IX-NRXPL(MEPOS,1)+1
          I1=MR(1)-I
          DO J=0,2*MR(2)
            JJ=IORGY+J
            IF(JJ.GT.NR2S) JJ=JJ-NR2S
            J1=MR(2)-J
            DO K=0,2*MR(3)
              KK=IORGZ+K
              IF(KK.GT.NR3S) KK=KK-NR3S
              K1=MR(3)-K
              CALL POSITX(X,Y,Z,I1,J1,K1,DA1,XPTAU(1))
              R2=X*X+Y*Y+Z*Z
              IF(R2.LT.RCUT2) THEN
                XVAL=ZR1*EXP(-R2*ZR2)*EXTF(II,JJ,KK)
                EEXT=EEXT+XVAL
                IF(TFOR) THEN
                  YVAL=-2.D0*XVAL*ZR2
                  FORC(1)=FORC(1)-X*YVAL
                  FORC(2)=FORC(2)-Y*YVAL
                  FORC(3)=FORC(3)-Z*YVAL
                ENDIF
              ENDIF
            ENDDO
          ENDDO
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C..this subroutine is added for gromacs-cpmd interface: biswas
C..it adopts a localized expansion for the MM atom point charges and a
C..multilayer (inner, intermediate, outer) scheme with periodic updating 
C..of qmmm energy & forecs for the intermediate and outer layer MM-atoms
C..while evaluating the energy and forces for the inner layer at each step.
C     ==================================================================
      SUBROUTINE VEXTNUB(IS,IA,XI,YI,ZI,ZVP,EEXT,FORC)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'epot.inc'
C     Variables
      REAL*8  EEXT,FORC(3)
      REAL*8  EINTM,FINTM_1,FINTM_2,FINTM_3
      REAL*8  EOUTR,FOUTR_1,FOUTR_2,FOUTR_3
      REAL*8  XI,YI,ZI,XJ,YJ,ZJ,RIJS,RIJ,RIJQ
      REAL*8  ALAM,ALFA,BETA
      REAL*8  ZVP,ZFAC,ZVAC
      INTEGER INR,IDNR,IA,IS
      REAL*8     EFINTM(4,100,100),EFOUTR(4,100,100)
      COMMON /MM_STORE/ EFINTM,EFOUTR
      DATA ALAM/1.3d0/ !!parameter to control smearing of ESP charge.
C     ==--------------------------------------------------------------==
      FORC(1) = 0.D0
      FORC(2) = 0.D0
      FORC(3) = 0.D0
      EEXT = 0.d0
C
      DO INR=1,NNEAR   ! NNEAR= NO OF NEAR MM ATOMS TO THE QM SYSTEM
        IDNR=IDNEAR(INR)
        XJ=KONEAR(1,INR)
        YJ=KONEAR(2,INR)
        ZJ=KONEAR(3,INR)
C  
        ALFA=ALAM*DATNEAR(3,INR) !!DATNEAR(3,IT) is column-6 of .run
        BETA=2.d0*ALFA
C
        RIJS = (XJ-XI)**2+(YJ-YI)**2+(ZJ-ZI)**2
        RIJ = DSQRT(RIJS)
        RIJQ = RIJ*RIJS
C     
        ZFAC=1.D0/RIJQ 
     &       - DEXP(-BETA*RIJ)*(1.D0/RIJQ+BETA/RIJS+ALFA*BETA/RIJ)
        ZVAC=1.D0/RIJ - DEXP(-BETA*RIJ)*(1.D0/RIJ + ALFA)
C..   
        FORC(1)=FORC(1)+DATNEAR(1,INR)*ZVP*(XI-XJ)*ZFAC
C..       
        FORC(2)=FORC(2)+DATNEAR(1,INR)*ZVP*(YI-YJ)*ZFAC
C..   
        FORC(3)=FORC(3)+DATNEAR(1,INR)*ZVP*(ZI-ZJ)*ZFAC
C..
        EEXT=EEXT+DATNEAR(1,INR)*ZVP*ZVAC
C..
      ENDDO
C     ==--------------------------------------------------------------==      
C..
      IF(MOD(STPNO,INTMF).EQ.0.AND.NINTM.GT.0) THEN
        EINTM   = 0.0D0
        FINTM_1  = 0.0D0
        FINTM_2  = 0.0D0
        FINTM_3  = 0.0D0
        DO INR=1,NINTM          ! NINTM= NO OF INTERMEDIATE MM ATOMS 
          IDNR=IDINTM(INR)
          XJ=KOINTM(1,INR)
          YJ=KOINTM(2,INR)
          ZJ=KOINTM(3,INR)
C  
          ALFA = ALAM*QINTM(3,INR)     
          BETA = 2.d0*ALFA
C     
          RIJS = (XJ-XI)**2+(YJ-YI)**2+(ZJ-ZI)**2
          RIJ = DSQRT(RIJS)
          RIJQ = RIJ*RIJS
C     
          ZFAC=1.D0/RIJQ
     &        - DEXP(-BETA*RIJ)*(1.D0/RIJQ+BETA/RIJS+ALFA*BETA/RIJ)
          ZVAC=1.D0/RIJ - DEXP(-BETA*RIJ)*(1.D0/RIJ + ALFA)
C..   
          FINTM_1 = FINTM_1 + QINTM(1,INR)*ZVP*(XI-XJ)*ZFAC
C..   
          FINTM_2 = FINTM_2 + QINTM(1,INR)*ZVP*(YI-YJ)*ZFAC
C..   
          FINTM_3 = FINTM_3 + QINTM(1,INR)*ZVP*(ZI-ZJ)*ZFAC
C..   
          EINTM = EINTM + QINTM(1,INR)*ZVP*ZVAC
C..
        ENDDO
C..
        EFINTM(1,IS,IA) = FINTM_1
        EFINTM(2,IS,IA) = FINTM_2 
        EFINTM(3,IS,IA) = FINTM_3 
        EFINTM(4,IS,IA) = EINTM         
C.. 
      ENDIF
C..      
C     ==--------------------------------------------------------------==      
C..     
      IF(MOD(STPNO,OUTMF).EQ.0.AND.NOUTR.GT.0) THEN
        EOUTR   = 0.0d0
        FOUTR_1  = 0.0d0
        FOUTR_2  = 0.0d0
        FOUTR_3  = 0.0d0
        DO INR=1,NOUTR          ! NOUTR= NO OF OUTERMOST MM ATOMS 
          IDNR=IDOUTR(INR)
          XJ=KOOUTR(1,INR)
          YJ=KOOUTR(2,INR)
          ZJ=KOOUTR(3,INR)
C  
          ALFA = ALAM*QOUTR(3,INR)     
          BETA = 2.d0*ALFA
C     
          RIJS = (XJ-XI)**2+(YJ-YI)**2+(ZJ-ZI)**2
          RIJ = DSQRT(RIJS)
          RIJQ = RIJ*RIJS
C
          ZFAC=1.D0/RIJQ
     &        - DEXP(-BETA*RIJ)*(1.D0/RIJQ+BETA/RIJS+ALFA*BETA/RIJ)
          ZVAC=1.D0/RIJ - DEXP(-BETA*RIJ)*(1.D0/RIJ + ALFA)
C..   
          FOUTR_1 = FOUTR_1 + QOUTR(1,INR)*ZVP*(XI-XJ)*ZFAC
C..   
          FOUTR_2 = FOUTR_2 + QOUTR(1,INR)*ZVP*(YI-YJ)*ZFAC
C..   
          FOUTR_3 = FOUTR_3 + QOUTR(1,INR)*ZVP*(ZI-ZJ)*ZFAC
C..
          EOUTR = EOUTR + QOUTR(1,INR)*ZVP*ZVAC
C..
        ENDDO
C..
        EFOUTR(1,IS,IA) = FOUTR_1
        EFOUTR(2,IS,IA) = FOUTR_2 
        EFOUTR(3,IS,IA) = FOUTR_3 
        EFOUTR(4,IS,IA) = EOUTR         
C.. 
      ENDIF
C..      
      FORC(1)=FORC(1)+EFINTM(1,IS,IA)+EFOUTR(1,IS,IA)
      FORC(2)=FORC(2)+EFINTM(2,IS,IA)+EFOUTR(2,IS,IA)
      FORC(3)=FORC(3)+EFINTM(3,IS,IA)+EFOUTR(3,IS,IA)
      EEXT=EEXT+EFINTM(4,IS,IA)+EFOUTR(4,IS,IA)
C..
C     ==--------------------------------------------------------------==
      RETURN
      END
      
