C     ==================================================================
cMcB  SUBROUTINE EFIELD(TAU0,FION,C0,C2,SCR,LSCR,NSTATE)
      SUBROUTINE EXTFIELD(TAU0,FION,C0,C2,SCR,LSCR,NSTATE)
C     ==--------------------------------------------------------------==
C     == APPLIES FINITE HOMOGENEOUS ELECTRIC FIELD USING BERRY PHASE  ==
C     ==                                                              ==
C     == 20/03/2009 Rewritten for general use (JAEA-SPring8/IPCMS)    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'irat.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'ddip.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'gvec.inc'
C     ==--------------------------------------------------------------==
C     Arguments
      INTEGER LSCR,NSTATE
      REAL*8 TAU0(3,NAX,NSP),FION(3,NAX,NSP)
      REAL*8 C0(*),C2(*),SCR(LSCR)
C     Local Variables
      INTEGER ISUB,NXX,NMCOL,N1,NOP1,NOP2,NOP3,
     &        IS,IA,K,I,IG,IS1,INFO
      REAL*8 FAC,PIN,ZVTOT,P(3),PHASE1,PHASE2,PHASE3,A,B,C,D,UFG
      COMPLEX*16 EPHASE(3),DD,ZONE,ZZERO
      PARAMETER (ZONE=(1.D0,0.D0),ZZERO=(0.D0,0.D0))
      CHARACTER*30 TAG
      INTEGER MAPFUL(*),MAPCOL(*),IPIV(*)
      COMPLEX*16 CWORK(NGW,NSTATE,*),FFIELD(NGW,*),
     &           DDMAT_E(NSTATE,*),CW(NGW,*),SC0(*),WORK(*)
      POINTER (IP_MAPFUL,MAPFUL),(IP_MAPCOL,MAPCOL),(IP_IPIV,IPIV),
     &        (IP_CWORK,CWORK),(IP_FFIELD,FFIELD),(IP_DDMAT_E,DDMAT_E),
     &        (IP_CW,CW),(IP_SC0,SC0),(IP_WORK,WORK)
      SAVE IP_MAPFUL,IP_MAPCOL,IP_CWORK,IP_FFIELD,IP_CW,IP_SC0
#if defined(PARALLEL)
      INTEGER  NXXFUN
      EXTERNAL NXXFUN
#endif
      REAL*8 EPSILON
      PARAMETER (EPSILON=1.D-12)
      SAVE IFIRST
      INTEGER IFIRST
      DATA IFIRST /0/
C     ==--------------------------------------------------------------==
      CALL TISET('    EFIELD',ISUB)
C..initialization on first call to subroutine
      IF(IFIRST.EQ.0) THEN
        N1=0
        DO I=0,NPROC-1
          N1=MAX(N1,SPARM(3,I))
        ENDDO
        CALL MEMORY(IP_MAPFUL,2*(NGWS/IRAT+1),'MAPFUL')
        NGWMAX=N1
#ifdef PARALLEL
        NMCOL=(NPROC*NGWMAX)/IRAT+1
#else 
        NMCOL=1
#endif
        CALL MEMORY(IP_MAPCOL,NMCOL,'MAPCOL')
        CALL SETDIP(MAPFUL,MAPCOL,SCR,LSCR)
        CALL MEMORY(IP_FFIELD,2*NGW*NSTATE+12,'FFIELD')
        CALL MEMORY(IP_CWORK,4*NGW*NSTATE,'CWORK')
#ifdef PARALLEL
        LENBK=NXXFUN(NSTATE)
        NXX=MAX(2*LENBK*NPROC,2*NGW*NSTATE+8)
#else 
        NXX=2*NGW*NSTATE+8
#endif
        CALL MEMORY(IP_SC0,NXX,'SC0')
        CALL MEMORY(IP_CW,NXX,'CW')
C..electric field in lattice frame
        IF(TLSD) THEN
           FAC=1.D0
        ELSE
           FAC=2.D0
        ENDIF
        PIN=FAC*0.5D0/ACOS(-1.D0)
        AFIELD(1)=PIN*(AIFIELD(1)*A1(1)+AIFIELD(2)*A1(2)
     .                +AIFIELD(3)*A1(3))
        AFIELD(2)=PIN*(AIFIELD(1)*A2(1)+AIFIELD(2)*A2(2)
     .                +AIFIELD(3)*A2(3))
        AFIELD(3)=PIN*(AIFIELD(1)*A3(1)+AIFIELD(2)*A3(2)
     .                +AIFIELD(3)*A3(3))
        IFIRST=1
      ENDIF
      CALL MEMORY(IP_DDMAT_E,2*NSTATE*NSTATE,'DDMAT_E')
      CALL MEMORY(IP_IPIV,20*(NSTATE/IRAT+1),'IPIV')
      CALL MEMORY(IP_WORK,2*NSTATE*20,'WORK')
      CALL AZZERO(P,3)
      ZVTOT=0.D0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          P(1)=P(1)+ZV(IS)*TAU0(1,IA,IS)
          P(2)=P(2)+ZV(IS)*TAU0(2,IA,IS)
          P(3)=P(3)+ZV(IS)*TAU0(3,IA,IS)
C..corrections to ionic forces
          IF(PARENT) THEN 
            FION(1,IA,IS)=FION(1,IA,IS)+ZV(IS)*AIFIELD(1)
            FION(2,IA,IS)=FION(2,IA,IS)+ZV(IS)*AIFIELD(2)
            FION(3,IA,IS)=FION(3,IA,IS)+ZV(IS)*AIFIELD(3)
          ENDIF 
        ENDDO
        ZVTOT=ZVTOT+ZV(IS)*NA(IS)
      ENDDO
      DO I=1,3
        P(I)=P(I)/ZVTOT
      ENDDO
C..phase factors due to center of charge motion
      UFG=TPIBA*NSTATE
      PHASE1=UFG*(P(1)*B1(1)+P(2)*B1(2)+P(3)*B1(3))
      PHASE2=UFG*(P(1)*B2(1)+P(2)*B2(2)+P(3)*B2(3))
      PHASE3=UFG*(P(1)*B3(1)+P(2)*B3(2)+P(3)*B3(3))
      EPHASE(1)=DCMPLX(COS(PHASE1),SIN(PHASE1))
      EPHASE(2)=DCMPLX(COS(PHASE2),SIN(PHASE2))
      EPHASE(3)=DCMPLX(COS(PHASE3),SIN(PHASE3))
C..electronic contribution
      EEFIELD=0.D0
      DO K=1,3
        IF(DABS(AFIELD(K)).GT.EPSILON) THEN 
          NOP1=K
          NOP2=0
          NOP3=0
          CALL OPEIGR_P(C0,CW,SC0,NSTATE,MAPFUL,MAPCOL,DDMAT_E,
     &         SCR,LSCR,NOP1,NOP2,NOP3,DD,CWORK)
          CALL ZGETRF(NSTATE,NSTATE,DDMAT_E,NSTATE,IPIV,INFO)
          IF(INFO.EQ.0) THEN
            DD=ZONE 
!$OMP parallel do private(I) reduction(*:DD)
            DO I=1,NSTATE
              DD=DD*DDMAT_E(I,I)
            ENDDO 
            CALL ZGETRI(NSTATE,DDMAT_E,NSTATE,IPIV,WORK,NSTATE,INFO)
          ELSE
            CALL STOPGM('EFIELD','ERROR IN MATRIX INVERSION')
          ENDIF
          CALL ZGEMM('N','N',NGW,NSTATE,NSTATE,ZONE,CWORK(1,1,1)
     &         ,NGW,DDMAT_E(1,1),NSTATE,ZZERO,CW(1,1),NGW)
          CALL ZCOPY(NGW*NSTATE,CW(1,1),1,CWORK(1,1,1),1)
          CALL ZGEMM('N','N',NGW,NSTATE,NSTATE,ZONE,CWORK(1,1,2)
     &         ,NGW,DDMAT_E(1,1),NSTATE,ZZERO,CW(1,1),NGW)
          CALL ZCOPY(NGW*NSTATE,CW(1,1),1,CWORK(1,1,2),1)
!$OMP parallel private(I,IG,A,B,C,D) default(shared)
          IF(GEQ0) THEN
            IS1=2
!$OMP do
            DO I=1,NSTATE
              B=DIMAG(CWORK(1,I,1))
              FFIELD(1,I)=DCMPLX(B,0.D0)
            ENDDO
!$OMP end do
          ELSE
            IS1=1
          ENDIF
!$OMP do
          DO I=1,NSTATE 
            DO IG=IS1,NGW         
              A=DREAL(CWORK(IG,I,1))
              B=DIMAG(CWORK(IG,I,1))
              C=DREAL(CWORK(IG,I,2))
              D=DIMAG(CWORK(IG,I,2))
              FFIELD(IG,I)=0.5D0*DCMPLX(B+D,C-A)
            ENDDO
          ENDDO
!$OMP end do
!$OMP end parallel
          DD=DD*EPHASE(K)
          P(K)=DATAN(DIMAG(DD)/DREAL(DD))
C..electric enthalpy
          EEFIELD=EEFIELD-P(K)*AFIELD(K)
C..corrections to electric forces
          CALL DAXPY(2*NGW*NSTATE,AFIELD(K),FFIELD(1,1),1,C2,1)
        ENDIF 
      ENDDO
C..corrections to total energy 
      ETOT=ETOT+EEFIELD
      CALL FREEM(IP_DDMAT_E)
      CALL FREEM(IP_IPIV)
      CALL FREEM(IP_WORK)
      CALL TIHALT('    EFIELD',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
