C***************************************************************************
C***************************************************************************
C.....Interface to the MD simulation program EGO.
C.....by M. Eichinger and J. Hutter
C.....extended by P.K.Biswas to support Gromacs.
C.....
C.....Functions in this file:
C.....INTERFACE(...)    The main loop for the interface mode.
C.....INTERFACE_WAIT(.) Waits for the QMCONTINUE file created by EGO.
C.....INTERFACE_READ(.) Reads the new data from EGO.
C.....RDEGOF(..)        Reads the external field.
C.....EXT_FIELD         Calculates the external field at grid points.
C.....CLCPOT(..)        Calculates the electrostatic field at any point.
C.....CLCFLD(..)        Calculates the same for a s-wave expanded point charge.
C.....INTERFACE_WRITE() Writes the forces on the QM atoms for EGO.
C.....EGOSTOP           Creates a file for stopping EGO in case of problems.
C.....MYPROPPT(...)     Calculates the dipole moment
C.....GMX_EL_FORCE(...) Calculates QM-2-MM electrostatic forces on MM atoms.
C***************************************************************************
C***************************************************************************
C     ==================================================================
      SUBROUTINE INTERFACE(C0,C2,SC0,PME,GDE,VPP,EIGV)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ener.inc' 
      INCLUDE 'elct.inc' 
      INCLUDE 'tpar.inc' 
      INCLUDE 'pslo.inc' 
      INCLUDE 'norm.inc' 
      INCLUDE 'ropt.inc' 
      INCLUDE 'cppt.inc' 
      INCLUDE 'sfac.inc' 
      INCLUDE 'coor.inc' 
      INCLUDE 'nlcc.inc' 
      INCLUDE 'ions.inc' 
      INCLUDE 'epot.inc'
      INCLUDE 'efld.inc' 
      INCLUDE 'efld2.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'store.inc'
C     Arguments
      COMPLEX*16 C0(*),C2(*),SC0(*),PME(*),GDE(*)
      REAL*8     VPP(*),EIGV(*)
C     Variables
      REAL*8     FNL(*),DFNL(*)
      COMPLEX*16 PSI(NNR1,NLSD)
      REAL*8     RHOE(NNR1,NLSD),SCR(*)
      POINTER    (IP_PSI,PSI),(IP_RHOE,RHOE),(IP_SCR,SCR)
      REAL*8     EXTF(*), EXTFL1(*), EXTFL2(*)
      CHARACTER  TAG*30
      LOGICAL    EOFRUN,FNOWF
      CHARACTER  INT_FILEN*80,LINE*80
      INTEGER    IREC(100),IL_RHOE,IL_PSI,LSCR,I
      REAL*8     TIME1,TIMEF,ETOTO,EKINCP,EKIN1,EKIN2,TEMP1,TEMP2,
     &           EKINH1,EKINH2,DETOT,TIME2,TCPU,DUMMY
      EXTERNAL   TIMEF
C     ==--------------------------------------------------------------==
C     SCR creation
      CALL RHOE_PSI_SIZE(IL_RHOE,IL_PSI)
      CALL MEMORY(IP_RHOE,IL_RHOE,'RHOE')
      CALL MEMORY(IP_PSI,IL_PSI,'PSI')
      CALL GIVE_SCR_INTERFACE(LSCR,TAG)
      CALL MEMORY(IP_SCR,LSCR,'SCR')
C     ==--------------------------------------------------------------==
      CLOSE(5)
      TIME1 = TIMEF()
C..+1 for esp charges
      CALL MEMORY(IP_TAUP,3*NAX*NSX+1,'TAUP')
      CALL MEMORY(IP_FION,3*NAX*NSX,'FION')
      CALL MEMORY(IP_VELP,3*NAX*NSX,'VELP')
C..added: biswas
      CALL MEMORY(IP_EXTF,KR1*KR2S*KR3S,'EXTF')  
      CALL MEMORY(IP_EXTFL1,KR1*KR2S*KR3S,'EXTFL1')
      CALL MEMORY(IP_EXTFL2,KR1*KR2S*KR3S,'EXTFL2')  
      CALL AZZERO(EXTF,KR1*KR2S*KR3S) 
      CALL AZZERO(EXTFL1,KR1*KR2S*KR3S)    
      CALL AZZERO(EXTFL2,KR1*KR2S*KR3S)
C..addition ends
      CALL AZZERO(TAUP,3*NAX*NSX)
      CALL AZZERO(VELP,3*NAX*NSX)
      CALL AZZERO(BOXQM,6)
      CALL AZZERO(BOXDUM,6)
      CALL AZZERO(KOLOC,3*MAXIND) 
      CALL AZZERO(LOCEXP,10*MAXIND)
      CALL AZZERO(DATNEAR,5*MAXNEAR)
      CALL AZZERO(KONEAR,3*MAXNEAR)
      CALL IAZZERO(LOCID,MAXIND)
      CALL IAZZERO(NINCL,MAXIND)
      CALL IAZZERO(IDNEAR,MAXNEAR)
      CALL IAZZERO(LINCL,MAXIND*MAXNEAR)       
      DO I=1,100
         MYRAG(I)=1.2d0
      ENDDO 
      MYRAG(1)=0.85d0 
      MYRAG(8)=1.23d0    
         
      NNEAR = 0
      NLOC  = 0

      RESTART=.FALSE.  
      RWF    =.TRUE. 
      EOFRUN=.FALSE.
      NFI   = 0
      ECNSTR= 0.0D0
      ERESTR= 0.0D0
      ETOTO = 0.0D0

      IF(IFTYPE.EQ.1.AND.PARENT)
     *     WRITE(6,'(A)') ' INTERFACE| ENTERING EGO-INTERFACE'
      IF(IFTYPE.EQ.2.AND.PARENT)
     *     WRITE(6,'(A)') ' INTERFACE| ENTERING GROMACS-INTERFACE'

      CALL DYNIT(EKINCP,EKIN1,EKIN2,TEMP1,TEMP2,EKINH1,EKINH2)
C     ==--------------------------------------------------------------==
C     ==      THE BASIC MOLECULAR DYNAMICS LOOP                       ==
C     ==--------------------------------------------------------------==
    1 CONTINUE
        NFI=NFI+1
        CONVWF=.FALSE.
        SDIIS =.TRUE. 
        SPCG  =.TRUE. 
        MODENS=.FALSE.
        ENGPRI=.FALSE.
        CALSTE=.FALSE.
C
C.......wait for the interface file
C
        CALL INTERFACE_WAIT(INT_FILEN)
        IF(.NOT.EOFRUN) CALL TESTEX(EOFRUN)
        IF(EOFRUN) GOTO 2
C
C.......read and process interface file
C
        CALL INTERFACE_READ(TAU0)
        CALL PHFAC(TAU0)
        IF(TINLC) CALL COPOT(RHOE,PSI,SCR,LSCR,.FALSE.)
        CALL MY_SYNC(ALLGRP)
C
C.......Randomize Wavefunction
C
        IF(TRANE) THEN
           WRITE(6,'(A,F12.8)') 
     *   ' RANDOMIZE INITIAL WAVEFUNCTION,  AMPLITUDE=',AMPRE
           CALL RRANE(C0,C2,N)
        ENDIF
C       
        FNOWF=.FALSE.
        DO INFI=1,NOMORE
          TIME1=TIMEF()
          IF(PARENT) THEN
            ENGPRI=MOD(INFI-1,IPRINT_STEP).EQ.0
          ELSE
            ENGPRI=.FALSE.
          ENDIF
C
C.......update the wavefunctions 
C
          CALL UPDWF(C0,C2,SC0,TAU0,FION,PME,GDE,VPP,
     *               EIGV,RHOE,PSI,SCR,LSCR,N,FNOWF,.TRUE.)
C
C.......printout evolution of the progress
C
          IF(PARENT) THEN
            DETOT=ETOT+EEXT-ETOTO
            IF(INFI.EQ.1) DETOT=0.0D0
            TIME2=TIMEF()
            TCPU=(TIME2-TIME1)*0.001D0
            IF(ENGPRI) CALL WRENER
            IF(ENGPRI) WRITE(6,'(A)')
     *' NFI     GEMAX     CNORM          ETOT         DETOT        TCPU'
            WRITE(6,'(I4,F10.6,F10.6,F14.5,F14.6,F12.2) ')
     *       INFI,GEMAX,CNORM,ETOT+EEXT,DETOT,TCPU
            ETOTO=ETOT+EEXT
          ENDIF
          IF(CONVWF.AND.FNOWF) GOTO 100
          IF(CONVWF) THEN
             CONVWF=.FALSE.
             FNOWF=.TRUE.
          ENDIF
        ENDDO
  100   CONTINUE
        IF (PARENT) THEN
         PRINT*,'EXTERNAL ENERGY    =',EEXT,' AU'
         PRINT*,'REAL TOTAL ENERGY  =',EEXT+ETOT,' AU'
        ENDIF
C..
C..Added by biswas: dec/2004.
C..   For calculating electrostatic forces on Gromacs (MM) atoms
        IF(IFTYPE.EQ.2) THEN
          CALL GMX_EL_FORCES(C0,TAU0,RHOE,PSI,SCR,LSCR,N,1)
c         CALL ELF(C0,TAU0,RHOE,PSI,SCR,LSCR,N,1)
        ENDIF
C..addition ends here.
C..
C..Calculate gradient
C
        CALL FORCEDR(C0,C2,SC0,RHOE,PSI,TAU0,FION,EIGV,SCR,LSCR,
     *                   N,1,.TRUE.,.TRUE.)
C
C
C..Calculate the dipole moment
C 
        IF(PROPER) THEN
            CALL MY_GETARG(1,LINE)
            OPEN(UNIT=5,FILE=LINE,STATUS='OLD',ERR=2)
            CALL MYPROPPT(C0,TAU0,RHOE,PSI,SCR,LSCR)
            CLOSE(5)
        ENDIF
C
C..impose constraints on the gradient
C
        CALL PURGE(TAU0,FION)
C
C..process data and write the interface file
C
        CALL INTERFACE_WRITE(INT_FILEN,TAU0,FION,C0,TAUP,
     *                       RHOE,PSI,SCR,LSCR)
C
C..write the restart file
C
        CALL WRITE_IREC(IREC)
        CALL ZHWWF(2,IREC,C0,C2,N,DUMMY,TAU0,VELP,TAUP,NFI)
C
C..test for exit-file
C        
        IF(.NOT.EOFRUN) CALL TESTEX(EOFRUN)
        IF(EOFRUN) GOTO 2
      GOTO 1
    2 CONTINUE
C
      CALL WRITE_IREC(IREC)
      CALL ZHWWF(2,IREC,C0,C2,N,DUMMY,TAU0,VELP,TAUP,NFI)
      IF(PARENT) CALL FINALP(TAU0,FION,VELP,DUMMY)
      CALL FREEM(IP_TAUP)
      CALL FREEM(IP_FION)
      CALL FREEM(IP_VELP)
C..added: biswas.
      CALL FREEM(IP_EXTF)  
      CALL FREEM(IP_EXTFL1) 
      CALL FREEM(IP_EXTFL2) 
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C
C
C
C     ==================================================================
      SUBROUTINE INTERFACE_READ(TAU0)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'efld.inc'
      INCLUDE 'efld2.inc'
      INCLUDE 'epot.inc'
      INCLUDE 'irat.inc'
C     Arguments
      REAL*8    TAU0(3,NAX,NSX)
C     Variables
      CHARACTER LINE*80
      INTEGER   IUNIT,IERR,ISP,INSCAN,IMDUM,IARGC,NAIT,I,MSGLEN
      REAL*8    EXTF(*), EXTFL1(*), EXTFL2(*)
C     ==--------------------------------------------------------------==
      IF(PARENT) THEN
        IUNIT=5
        IMDUM=IARGC()
        IF(IMDUM.LT.1) THEN
           WRITE(6,*) ' INTERFACE| NO INPUT FILE NAME SPECIFIED '
           CALL STOPGM('INTERFACE',' ')
        ENDIF        
        CALL MY_GETARG(1,LINE)
        WRITE(6,*) ' INTERFACE| READING NEW COORDINATES FROM FILE ',LINE
        OPEN(UNIT=IUNIT,FILE=LINE,STATUS='OLD',ERR=300)
C
C.......read atomic coordinates
C
        IERR=INSCAN(IUNIT,'&ATOM')
        ISP=0
   10   CONTINUE
           READ(IUNIT,END=200,ERR=300,FMT='(A)') LINE
           IF(INDEX(LINE,'&END').NE.0) GOTO 30
           IF(LINE(1:1).NE.'*') GOTO 10
             ISP=ISP+1
             READ(IUNIT,*)
             READ(IUNIT,*) NAIT
             DO I=1,NAIT
               READ(IUNIT,*) TAU0(1,I,ISP),TAU0(2,I,ISP),TAU0(3,I,ISP)
             ENDDO
           GOTO 10
   30   CONTINUE
C
C.......read external field
C
        IERR=INSCAN(IUNIT,'&EXTE')
        IF(IERR.EQ.0) THEN
          CALL RDEGOF(IUNIT)
          TEXTFLD=.TRUE.
        ELSE
          TEXTFLD=.FALSE.
        ENDIF      
        CLOSE(IUNIT)
C
C.......terminate reading normal
C
  200   CONTINUE
        WRITE(6,*) 'INTERFACE| READING NEW COORDINATES ... DONE'
        GOTO 400
  300   CONTINUE
        WRITE(6,*) 'INTERFACE| CAN NOT READ FROM FILE ',LINE
        CALL STOPGM('INTERFACE',' ')
  400 CONTINUE
      ENDIF
C.....broadcast information to all processors
      CALL MY_SYNC(ALLGRP)
c.....tau0
      MSGLEN = 3*NAX*NSX * 8
      CALL MY_BCAST(TAU0,MSGLEN,SOURCE,ALLGRP)
c.....textfld
      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(TEXTFLD,MSGLEN,SOURCE,ALLGRP)
      CALL MY_SYNC(ALLGRP)
      IF(TEXTFLD) THEN
c.....epot1
        MSGLEN = (2+2*MAXIND+MAXNEAR+MAXIND*MAXNEAR)*8/IRAT
        CALL MY_BCAST(NNEAR,MSGLEN,SOURCE,ALLGRP)
c.....epot2
        MSGLEN = (12+13*MAXIND+8*MAXNEAR) * 8
        CALL MY_BCAST(BOXQM,MSGLEN,SOURCE,ALLGRP)
C..added: biswas
        IF (IFTYPE.EQ.2) THEN
c.....epot3
          MSGLEN = (5+MAXINTM+MAXOUTR)*8/IRAT
          CALL MY_BCAST(STPNO,MSGLEN,SOURCE,ALLGRP)
c.....epot4
          MSGLEN = (MAXNEAR+8*MAXINTM+8*MAXOUTR) * 8
          CALL MY_BCAST(ZMMNEAR,MSGLEN,SOURCE,ALLGRP) 
c.....epot5
          MSGLEN = (3*MAXNEAR+3*MAXINTM+3*MAXOUTR) * 8
          CALL MY_BCAST(GMX_FNEAR,MSGLEN,SOURCE,ALLGRP)  
        ENDIF
C..addition ends
      ENDIF
C.....calculate external field at grid points
      IF(TEXTFLD) CALL EXT_FIELD
      CALL MY_SYNC(ALLGRP)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C
C
C
C##############################################
      SUBROUTINE RDEGOF(IUNIT)
C##############################################
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'epot.inc'
      INTEGER IUNIT,ILINES,K,JJ
C
      CHARACTER STRING*100
      CHARACTER SUBSTRING*7
      INTEGER I,J
C***** Read bounding box **********************************
      READ(IUNIT,'(A)') STRING
      READ(IUNIT,*) BOXDUM(1),BOXDUM(2),
     @              BOXDUM(3),BOXDUM(4),
     @              BOXDUM(5),BOXDUM(6)
      PRINT'(A10,F11.5,F11.5,F11.5,F11.5,F11.5,F11.5)',STRING,
     @              BOXDUM(1),BOXDUM(2),
     @              BOXDUM(3),BOXDUM(4),
     @              BOXDUM(5),BOXDUM(6)
C
C***** Read QM-bounding box ************************************
      READ(IUNIT,'(A)') STRING
      READ(IUNIT,*) BOXQM(1),BOXQM(2),
     @              BOXQM(3),BOXQM(4),
     @              BOXQM(5),BOXQM(6)
      PRINT'(A10,F11.5,F11.5,F11.5,F11.5,F11.5,F11.5)',STRING,
     @              BOXQM(1),BOXQM(2),
     @              BOXQM(3),BOXQM(4),
     @              BOXQM(5),BOXQM(6)
C
C..added: biswas // to implement multiply MM layer; 
      IF(IFTYPE.EQ.2) THEN
C***** Read Step-No *****************************************
        READ(IUNIT,'(A)') STRING
        READ(IUNIT,*) STPNO
        PRINT'(A15,I5)',STRING, STPNO
C      
C***** Read LMAX for partial wave expansion of MM-charge*******
        READ(IUNIT,'(A)') STRING
        READ(IUNIT,*) PWLMAX
        PRINT'(A15,I5)',STRING, PWLMAX
C      
C***** Read Intm-Freq *****************************************
        READ(IUNIT,'(A)') STRING
        READ(IUNIT,*) INTMF
        PRINT'(A15,I5)',STRING, INTMF
C      
C***** Read Outm-Freq *****************************************
        READ(IUNIT,'(A)') STRING
        READ(IUNIT,*) OUTMF
        PRINT'(A15,I5)',STRING, OUTMF
C
      ELSEIF(IFTYPE.EQ.1) THEN
C.....Read local expansions ************************************
C
        READ(IUNIT,'(A)') STRING
        READ(IUNIT,*) NLOC
        PRINT'(A11,I8)',STRING,NLOC
        IF (NLOC.GE.MAXIND) THEN
          WRITE(6,*) ' INTERFACE| Error: MAXIND not big enough!'
          WRITE(6,*) '            Increase this value in epot.inc'     
          CALL STOPGM('INTERFACE','To many QM-atoms.')
        ENDIF
C.....
C.....Loop over all local expansions
C
        DO 30 I=1,NLOC
C...........Read position of local expansion
          READ(IUNIT,*) LOCID(I),KOLOC(1,I),KOLOC(2,I),KOLOC(3,I)
          READ(IUNIT,*) LOCEXP(1,I)
          READ(IUNIT,*) LOCEXP(2,I),LOCEXP(3,I),LOCEXP(4,I)
          READ(IUNIT,*) LOCEXP(5,I),LOCEXP(6,I),LOCEXP(7,I),
     @         LOCEXP(8,I),LOCEXP(9,I),LOCEXP(10,I)
          READ(IUNIT,*) NINCL(I)
C.....
          IF (I.LE.3) THEN
            PRINT'(I8,TR2,F11.5,F11.5,F11.5,TR2,A,I3)',LOCID(I),
     @           KOLOC(1,I),KOLOC(2,I),KOLOC(3,I),'INCL: ',NINCL(I)
          ENDIF
          IF (NINCL(I).GE.MAXNEAR) THEN
            WRITE(6,*) ' INTERFACE| Error: MAXNEAR not big enough!'
            WRITE(6,*) '            Increase this value in epot.inc'
            CALL STOPGM('INTERFACE','To many near atoms.')
          ENDIF
C
C...........Read the near inclusion list *********
C
          ILINES = INT(NINCL(I)/10) + 1
          DO 35 K=0,ILINES-1
            READ(IUNIT,'(A)') STRING
            DO 40 J=0,9
              JJ = 10*K + J
              IF (JJ.GE.NINCL(I)) GOTO 30 
              SUBSTRING = STRING(1+J*7:J*7+7)
              READ(SUBSTRING,*) LINCL(I,JJ+1)
              LINCL(I,JJ+1) = LINCL(I,JJ+1) + 1
 40         CONTINUE
 35       CONTINUE
 30     CONTINUE
      ENDIF
C
C.....Read list of near atoms ***************************
C
      READ(IUNIT,'(A)') STRING
      READ(IUNIT,*) NNEAR
      PRINT'(A11,I5)',STRING,NNEAR
      IF (NNEAR.GE.MAXNEAR) THEN
        WRITE(6,*) ' INTERFACE| Error: MAXNEAR not big enough!'
        WRITE(6,*) '            Increase this value in epot.inc'     
        CALL STOPGM('INTERFACE','To many near atoms.')
      ENDIF
      DO 50 I=1,NNEAR
        READ(IUNIT,*) IDNEAR(I),
     @       DATNEAR(1,I),      ! Partial Charge
     @       DATNEAR(4,I),      ! CutOffWidth
     @       DATNEAR(5,I),      ! CutOffValue
     @       DATNEAR(2,I),      ! Core charge
     @       DATNEAR(3,I),      ! Sigma
     @       KONEAR(1,I),KONEAR(2,I),KONEAR(3,I) ! the position of the atoms
C..modified: biswas/2005.
            ZMMNEAR(I) = DATNEAR(2,I) !!nuclear charges are stored: biswas.
C..
            IF(IFTYPE.EQ.1) THEN 
              IF (DATNEAR(3,I).LE.0.0D0) THEN
                DATNEAR(2,I) =0.0D0
                DATNEAR(3,I) =-DATNEAR(3,I)
              ENDIF
            ENDIF
C..modification ends here.
            IF (I.LE.3) THEN
              PRINT'(I8,TR2,F11.5,F11.5,F11.5,F11.5,F11.5,F11.5)',
     @             IDNEAR(I),
     @             DATNEAR(1,I),
     @             DATNEAR(2,I),DATNEAR(3,I),
     @             KONEAR(1,I),KONEAR(2,I),KONEAR(3,I)            
            ENDIF
50    CONTINUE
C.....Read list of intermediate layer MM atoms *************************C
C      
      IF(IFTYPE.EQ.2) THEN
C     
        READ(IUNIT,'(A)') STRING
        READ(IUNIT,*) NINTM
        PRINT'(A11,I5)',STRING,NINTM
        IF (NINTM.GE.MAXINTM) THEN
          WRITE(6,*) ' INTERFACE| Error: MAXINTM not big enough!'
          WRITE(6,*) '            Increase this value in epot.inc'     
          CALL STOPGM('INTERFACE','To many intermediate MM atoms.')
        ENDIF       
        IF(MOD(STPNO,INTMF).EQ.0.AND.NINTM.GT.0) THEN
          DO 51 I=1,NINTM
            READ(IUNIT,*) IDINTM(I),
     @           QINTM(1,I), !......Partial Charge
     @           QINTM(2,I), !......Sigma
     @           KOINTM(1,I),KOINTM(2,I),KOINTM(3,I) !  position of the atoms
C..   
 51       CONTINUE
        ENDIF
C
C.....Read list of outermost layer MM atoms *************************C
C
        READ(IUNIT,'(A)') STRING
        READ(IUNIT,*) NOUTR
        PRINT'(A11,I5)',STRING,NOUTR
        IF (NOUTR.GE.MAXOUTR) THEN
          WRITE(6,*) ' INTERFACE| Error: MAXOUTR not big enough!'
          WRITE(6,*) '            Increase this value in epot.inc'     
          CALL STOPGM('INTERFACE','To many outermost MM atoms.')
        ENDIF       
C..
        IF(MOD(STPNO,OUTMF).EQ.0.AND.NOUTR.GT.0) THEN
          DO 52 I=1,NOUTR
            READ(IUNIT,*) IDOUTR(I),
     @               QOUTR(1,I), !..............Partial Charge
     @               QOUTR(2,I), !...........Sigma, there is no QOUTR(2,I); 
     @               KOOUTR(1,I),KOOUTR(2,I),KOOUTR(3,I)
C     addl. dimension is to facilitate calc.
C....................the position of the atoms
C..
 52       CONTINUE
        ENDIF
C..
      ENDIF
C
C*****
      END
C*****END of SUBROUTINE RDEGOF(FNAME)
C
C
C
C     ==================================================================
      SUBROUTINE EXT_FIELD
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'epot.inc'
      INCLUDE 'efld.inc'
C     Variables
      REAL*8  EXTF(KR1,KR2S,KR3S)
      REAL*8  K0(0:2),DI,DJ,DK,CLCPOT
      INTEGER ISUB,I,J,K,IKR1,IR
      REAL*8  CLCFLD
C     ==--------------------------------------------------------------==
      CALL TISET(' EXT_FIELD',ISUB)
C.. memory for EXTF already allocated in INTERFACE(): biswas
      IKR1 = MOD(NR1+1,2)   !!added: biswas

      DK=(BOXDUM(6)-BOXDUM(5))/DBLE(NR3S)
      DJ=(BOXDUM(4)-BOXDUM(3))/DBLE(NR2S)
      DI=(BOXDUM(2)-BOXDUM(1))/DBLE(NR1S)

      IF (IFTYPE.EQ.1) THEN
        DO K=1,KR3S
          K0(2)=BOXDUM(5)+DBLE(K-1)*DK
          DO J=1,KR2S
            K0(1)=BOXDUM(3)+DBLE(J-1)*DJ
            DO I=NRXPL(MEPOS,1),NRXPL(MEPOS,2)+IKR1  !IKR1 added: biswas
              K0(0)=BOXDUM(1)+DBLE(I-1)*DI
              EXTF(I-NRXPL(MEPOS,1)+1,J,K)=CLCPOT(K0)
            ENDDO
          ENDDO
        ENDDO
C..
      ELSEIF (IFTYPE.EQ.2) THEN
C..
           IR = 0
        DO K=1,KR3S
          K0(2)=BOXDUM(5)+DBLE(K-1)*DK
          DO J=1,KR2S
            K0(1)=BOXDUM(3)+DBLE(J-1)*DJ
            DO I=NRXPL(MEPOS,1),NRXPL(MEPOS,2)+IKR1 !IKR1 added: biswas
              IR = IR +1
              K0(0)=BOXDUM(1)+DBLE(I-1)*DI
              EXTF(I-NRXPL(MEPOS,1)+1,J,K)=CLCFLD(IR,K0)
            ENDDO
          ENDDO
        ENDDO
C..   
      ENDIF
C..
      IF(PARENT) THEN
          WRITE (6,'(A)') ' INTERFACE| EXTERNAL FIELD CALCULATED '
      ENDIF
      CALL TIHALT(' EXT_FIELD',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C
C
C
C##############################################
      FUNCTION CLCPOT(KO)
C##############################################
      IMPLICIT NONE
      INCLUDE 'epot.inc'
C     Arguments
      REAL*8  KO(0:2),CLCPOT
C     Variables
      INTEGER I,LI,IT
      REAL*8  DX,DY,DZ,ADDEPOT
      REAL*8  DMIN,D
#ifdef __DERF
      REAL*8   DERF       
      EXTERNAL DERF
#endif
C
C.... find the nearest local expansion ********
C
      CLCPOT = 0.D0
      DMIN   = 999999D0
      LI     = -1
C
      IF (NLOC .EQ. 0) GOTO 150
C
      DO 100 I=1,NLOC
        DX=KO(0)-KOLOC(1,I)
        DY=KO(1)-KOLOC(2,I)
        DZ=KO(2)-KOLOC(3,I)
        D=DX*DX+DY*DY+DZ*DZ
        IF (D .LT. DMIN) THEN
          DMIN=D
          LI=I
        END IF
100   CONTINUE
C
C.... evaluate potential from local expansion *****
C
      DX=KO(0)-KOLOC(1,LI)
      DY=KO(1)-KOLOC(2,LI)
      DZ=KO(2)-KOLOC(3,LI)
C
      CLCPOT =   -( LOCEXP(1,LI) +
     @     DX*LOCEXP(2,LI) + 
     @     DY*LOCEXP(3,LI) + 
     @     DZ*LOCEXP(4,LI) +
     @     0.5D0*DX*DX*LOCEXP(5,LI) +
     @     0.5D0*DY*DY*LOCEXP(6,LI) +
     @     0.5D0*DZ*DZ*LOCEXP(7,LI) +
     @     DX*DY*LOCEXP(8,LI) +
     @     DX*DZ*LOCEXP(9,LI) +
     @     DY*DZ*LOCEXP(10,LI) )
C
150   CONTINUE
C
      IF (NINCL(LI) .EQ. 0) GOTO 300
C
C.....add El.Pot from all near atoms      
C
      DO 200 I=1,NINCL(LI)
        IT = LINCL(LI,I)
C
        DX=KO(0)-KONEAR(1,IT)
        DY=KO(1)-KONEAR(2,IT)
        DZ=KO(2)-KONEAR(3,IT)
        
C............Distance in Bohr. 
        d = SQRT(DX*DX + DY*DY + DZ*DZ)
        IF (d.LT.0.1d0) d=0.1d0
C
        IF (d .GT. 6.0d0) THEN
C     If we a far away take simple Coulumb-Potential
C     Phi = q / d
          ADDEPOT = DATNEAR(1,IT) / d
        ELSE IF (DATNEAR(2,IT) .GT. 0.5d0) THEN
C     If core-charges are provided use the more
C     advanced atom-potential.
          DY=MYRAG(NINT(DATNEAR(2,IT)))
          DX = (DATNEAR(2,IT) - DATNEAR(1,IT))*
     *         (1.0d0-exp(-d/DATNEAR(3,IT)))                
          ADDEPOT = DERF(d/DY)*(DATNEAR(2,IT)-DX)/d
        ELSE
C     Use the simple atom-potential given 
C     just by partial charges.            
C     .              Phi = q * erf(d/sigma) / d
          DX = DERF(d/DATNEAR(3,IT))
          ADDEPOT = DATNEAR(1,IT) * DX / d
        ENDIF
                                     
C............we have an excluded atom here
        IF (DATNEAR(4,IT) .GT. 0.01d0) THEN
          IF (d .GT. (DATNEAR(5,IT)+DATNEAR(4,IT))) THEN
C     do nothing in that case
C     because we are far away from it 
          ELSE IF (d .LT. DATNEAR(5,IT)) THEN
C     we are to close, dont add any contribution from it
            ADDEPOT = 0.0d0
          ELSE            
C     Use a switching function: No typo here!
            DX = (d - DATNEAR(5,IT)) / DATNEAR(4,IT)
            DX = 1.0d0 - DX*DX
            ADDEPOT = (1.0d0 - DX*DX) * ADDEPOT
          ENDIF
        ENDIF
        
        CLCPOT = CLCPOT - ADDEPOT
200   CONTINUE
      IF (CLCPOT .LT. -0.3d0) CLCPOT = -0.3d0 - log(0.7d0-CLCPOT)
C
300   CONTINUE
C
      END

C
C..
C..added by biswas for GMX interface.
C##############################################
      FUNCTION CLCFLD(IR,KO)
C##############################################
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'epot.inc'
      INCLUDE 'efld2.inc'
C     Arguments
      REAL*8  KO(0:2),CLCFLD
C     Variables
      INTEGER I,IR
      REAL*8  D,DX,DY,DZ,ADDEPOT
      REAL*8  ALFA,BETA,ALAM
      REAL*8  POTINTM,POTOUTR
      REAL*8  EXTFL1(KR1*KR2S*KR3S), EXTFL2(KR1*KR2S*KR3S)
C
      CLCFLD = 0.D0
        ALAM = 1.3d0
C
C..!! For atoms in the innermost layer
      IF (NNEAR.GT.0) THEN
      DO 200 I=1,NNEAR   
C
        DX=KO(0)-KONEAR(1,I)
        DY=KO(1)-KONEAR(2,I)
        DZ=KO(2)-KONEAR(3,I)
        
C............Distance in Bohr
        D = DSQRT(DX*DX + DY*DY + DZ*DZ)
C..             
        ALFA = ALAM*DATNEAR(3,I)           
        BETA = 2.d0*ALFA
C............biswas-modified Coulomb
        ADDEPOT = DATNEAR(1,I)*
     &       (1.D0/D - DEXP(-BETA*D)*(1.D0/D + ALFA))
C     
        CLCFLD = CLCFLD - ADDEPOT
C
200   CONTINUE
      ENDIF
C
C..   !! For atoms in the intermediate layer
      IF(MOD(STPNO,INTMF).EQ.0.AND.NINTM.GT.0) THEN
        POTINTM = 0.d0
      DO 300 I=1,NINTM
C
        DX=KO(0)-KOINTM(1,I)
        DY=KO(1)-KOINTM(2,I)
        DZ=KO(2)-KOINTM(3,I)
C..   
        D = DSQRT(DX*DX + DY*DY + DZ*DZ)
C..   
        ALFA = ALAM*QINTM(3,I)           
        BETA = 2.d0*ALFA
C.............simple Coulomb
C             POTINTM = POTINTM + QINTM(I)/d  !! 
C............biswas-modified Coulomb
        POTINTM = POTINTM + QINTM(1,I)*
     &       (1.D0/D - EXP(-BETA*D)*(1.D0/D + ALFA))
C                                     
300   CONTINUE
C
      EXTFL1(IR) = POTINTM
      ENDIF
C
C..   !! For atoms in the outermost layer
      IF(MOD(STPNO,OUTMF).EQ.0.AND.NOUTR.GT.0) THEN
        POTOUTR = 0.d0
        DO 400 I=1,NOUTR
C
          DX=KO(0)-KOOUTR(1,I)
          DY=KO(1)-KOOUTR(2,I)
          DZ=KO(2)-KOOUTR(3,I)
C..   
          D = DSQRT(DX*DX + DY*DY + DZ*DZ)
C..   
          ALFA = ALAM*QOUTR(3,I)           
          BETA = 2.d0*ALFA
C.............simple Coulomb
C             POTOUTR = POTOUTR + QOUTR(1,I)/d  !! 
C............biswas-modified Coulomb
          POTOUTR = POTOUTR + QOUTR(1,I)*
     &         (1.d0/D - DEXP(-BETA*D)*(1.d0/D + ALFA))
C     
400   CONTINUE
      EXTFL2(IR) = POTOUTR
      ENDIF
C..
      CLCFLD = CLCFLD - EXTFL1(IR) - EXTFL2(IR)
C..
      RETURN
      END
C
C     ==================================================================
      SUBROUTINE INTERFACE_WAIT(INT_FILEN)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'fileopen.inc'
C     Arguments
      CHARACTER INT_FILEN*(*)
C     Variables
      LOGICAL   TEST
      CHARACTER FILEN*80
      INTEGER   ISUB
C     ==--------------------------------------------------------------==
      CALL TISET('INTER_WAIT',ISUB)
      IF(PARENT) THEN
        WRITE(6,'(/,1X,63("="))')
        FILEN=FPATH(IAPATH:IEPATH)//'QMCONTINUE'
        WRITE(6,*) ' INTERFACE| WAIT FOR CONTINUE-FILE ', FILEN
    1   CONTINUE
        INQUIRE(FILE=FILEN,EXIST=TEST)
        IF(TEST) THEN
          INT_FILEN=FPATH(IAPATH:IEPATH)//'qmoutput.out'
C.......Now delete the QMCONTINUE file
          OPEN(5,FILE=FILEN,STATUS='UNKNOWN')
          CLOSE(5,STATUS='DELETE')
          WRITE(6,*) ' INTERFACE| CONTINUE QM CALCULATION ',INT_FILEN
          GOTO 3
        ENDIF
        INQUIRE(FILE='EXIT',EXIST=TEST)
        IF(TEST) GOTO 3
        CALL MY_SLEEP(1)
        GOTO 1
    3   CONTINUE
      ENDIF
      CALL MY_SYNC(ALLGRP)
      CALL TIHALT('INTER_WAIT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C
C
C
C     ==================================================================
      SUBROUTINE INTERFACE_WRITE(INT_FILEN,TAU0,FION,C0,ACHRG,
     *                           RHOE,V,SCR,LSCR)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'epot.inc'
      INCLUDE 'efld.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'prop.inc'
      INCLUDE 'dipo.inc'
      INCLUDE 'lodp.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'fileopen.inc'

C     Arguments
      CHARACTER  INT_FILEN*(*)
      INTEGER    LSCR
      REAL*8     TAU0(3,NAX,NSX),FION(3,NAX,NSX),SCR(LSCR),ACHRG(*),
     &           RHOE(*)
      COMPLEX*16 C0(*),V(*)
C     Variables
      REAL*8     EXTF
      CHARACTER  LINE*80,STR*80
      INTEGER    ISEL(*),INR,IDNR
      REAL*8     EFIELD(*),VR(NNR1)
      POINTER    (IP_ISEL,ISEL),(IP_EFIELD,EFIELD),(IP_VR,VR)
      COMPLEX*16 EIROP(NHG),EIVPS(NHG),VTEMP(NHG),QPHI(*)
      POINTER    (IP_EIROP,EIROP),(IP_EIVPS,EIVPS),(IP_VTEMP,VTEMP),
     &           (IP_QPHI,QPHI)
      CHARACTER  TAG*30
      INTEGER    LINTERW,IS,ISH,L,NSTATE,I,IG,MLEN,IPPC,IAT,IA,IE
      REAL*8     K0(0:2),K1(0:2),DX,DY,DZ,EF(0:2),EEF,
     &           D1,D2,D3,DD,CLCPOT
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_INTERFACE_WRITE(LINTERW,TAG)
      CALL TEST_SCR('INTERFACE_WRITE',TAG,LSCR,LINTERW)
C     ==--------------------------------------------------------------==
      CALL AZZERO(ACHRG,NAT)
C
C..calculate atomic charges
C
      IF(TEXTFLD.AND.IFTYPE.EQ.1) THEN
        IF(ICMET.EQ.2 .OR. ICMET.EQ.3) THEN
C         SCR partition
#ifdef _BGL 
          CALL GET_ADDR(IP_VTEMP,SCR(1))
          CALL GET_ADDR(IP_EIVPS,SCR(2*NHG+1))
          CALL GET_ADDR(IP_EIROP,SCR(4*NHG+1))
          CALL GET_ADDR(IP_QPHI,SCR(6*NHG+1))
#else
          CALL GET_ADDR(IP_VTEMP,SCR(1))
          CALL GET_ADDR(IP_EIVPS,SCR(2*NHG+1))
          CALL GET_ADDR(IP_EIROP,SCR(4*NHG+1))
          CALL GET_ADDR(IP_QPHI,SCR(6*NHG+1))
#endif
        ENDIF
        IF(ICMET.EQ.1) THEN
C..Mulliken Charges
          NATTOT=0
          DO IS=1,NSP
            NUMAOR(IS)=0
            DO ISH=1,NSHELL(IS)
              L=LSHELL(ISH,IS)
              NATTOT=NATTOT+NA(IS)*(2*L+1)
              NUMAOR(IS)=NUMAOR(IS)+(2*L+1)
            ENDDO
          ENDDO
          CALL MULLIKEN(1,C0,NSTATE,TAU0,ACHRG,SCR,LSCR)
        ELSEIF(ICMET.EQ.4) THEN
C..Lowdin Charges
          NATTOT=0
          DO IS=1,NSP
            NUMAOR(IS)=0
            DO ISH=1,NSHELL(IS)
              L=LSHELL(ISH,IS)
              NATTOT=NATTOT+NA(IS)*(2*L+1)
              NUMAOR(IS)=NUMAOR(IS)+(2*L+1)
            ENDDO
          ENDDO
          CALL MULLIKEN(2,C0,NSTATE,TAU0,ACHRG,SCR,LSCR)
        ELSEIF(ICMET.EQ.2) THEN
C..Hirshfeld Charges
          IF(TIVAN) CALL RNLSM(C0,N,V,SCR,LSCR,1,1,.FALSE.)
          CALL RHOOFR(C0,RHOE,V,SCR,LSCR,N)
          CALL GET_ADDR(IP_VR,V)
          CALL ATOMC(RHOE,VR,TAU0,ACHRG)
        ELSEIF(ICMET.EQ.3) THEN
C..ESP Charges
          IF(TIVAN) CALL RNLSM(C0,N,V,SCR,LSCR,1,1,.FALSE.)
          CALL RHOOFR(C0,RHOE,V,SCR,LSCR,N)
          CALL EICALC(EIVPS,EIROP)
          DO I=1,NNR1
            V(I)=DCMPLX(RHOE(I),0.D0)
          ENDDO
          CALL FWFFT(V)
          DO IG=1,NHG
            VTEMP(IG) = V(NZH(IG))
          ENDDO
          CALL ELSTPO(VTEMP,EIROP,EIVPS,V,QPHI,TAU0)
          CALL DCOPY(2*NHG,V,1,VTEMP,1)
          CALL ZAZZERO(V,MAXFFT)
!CDIR NODEP
          DO IG=1,NHG
            V(INDZ(IG)) = DCONJG(VTEMP(IG))
            V(NZH(IG))  = VTEMP(IG)
          ENDDO
          IF(GEQ0.AND.TCLUST) THEN
            V(NZH(1)) = VTEMP(1)
          ELSEIF(GEQ0) THEN
            V(NZH(1)) = DCMPLX(0.D0,0.D0)
          ENDIF
          CALL INVFFT(V)
          DO I=1,NNR1
            RHOE(I)=DREAL(V(I))
          ENDDO
          MLEN=NNR1/IRAT + 1
          IPPC=0
          CALL MEMORY(IP_ISEL,MLEN,'ISEL')
          CALL SELECTP(ISEL,TAU0,IPPC)
          MLEN=(NAT+1)*IPPC
          CALL MEMORY(IP_EFIELD,MLEN,'EFIELD')
          DO I=1,IPPC
            EFIELD(I)=RHOE(ISEL(I))
          ENDDO
          CALL ATFIELD(EFIELD,V,VTEMP,EIROP,QPHI,ISEL,IPPC)
          CALL ESPSOLV(EFIELD,ACHRG,IPPC)
          CALL FREEM(IP_ISEL)
          CALL FREEM(IP_EFIELD)
        ELSE
          CALL STOPGM('INTERFACE_WRITE','INVALID ICMET')
        ENDIF
      ENDIF
C
C..writeout all data to interface-file
C
      IF(PARENT) THEN
        OPEN(5,FILE=INT_FILEN,STATUS='UNKNOWN')
        REWIND 5
        IAT=0
        WRITE(5,'(G15.6,TR1,G15.6)') ETOT,EEXT
        DO IS=1,NSP
          DO IA=1,NA(IS)
            IAT=IAT+1
            WRITE(5,'(G15.6,TR1,G15.6,TR1,G15.6,TR1,G15.6)')
     *      FION(1,IA,IS),FION(2,IA,IS),FION(3,IA,IS),ACHRG(IAT)
          ENDDO
        ENDDO
C
C..Calculate mean E-field and writeout dipole-moment
C
        DZ=BOXDUM(6)-BOXDUM(5)
        DY=BOXDUM(4)-BOXDUM(3)
        DX=BOXDUM(2)-BOXDUM(1)
C..E-Field X-Komponente
        K0(0) = BOXDUM(1) + DX/3.0D0
        K0(1) = BOXDUM(3) + DY/2.0D0
        K0(2) = BOXDUM(5) + DZ/2.0D0
        K1(0) = BOXDUM(1) + 2.0D0*DX/3.0D0
        K1(1) = BOXDUM(3) + DY/2.0D0
        K1(2) = BOXDUM(5) + DZ/2.0D0
        EF(0) = 3.0D0*(CLCPOT(K1) - CLCPOT(K0))/DX
C..E-Field Y-Komponente
        K0(0) = BOXDUM(1) + DX/2.0D0
        K0(1) = BOXDUM(3) + DY/3.0D0
        K0(2) = BOXDUM(5) + DZ/2.0D0
        K1(0) = BOXDUM(1) + DX/2.0D0
        K1(1) = BOXDUM(3) + 2.0D0*DY/3.0D0
        K1(2) = BOXDUM(5) + DZ/2.0D0
        EF(1) = 3.0D0*(CLCPOT(K1) - CLCPOT(K0))/DY
C..E-Field Z-Komponente
        K0(0) = BOXDUM(1) + DX/2.0D0
        K0(1) = BOXDUM(3) + DY/2.0D0
        K0(2) = BOXDUM(5) + DZ/3.0D0
        K1(0) = BOXDUM(1) + DX/2.0D0
        K1(1) = BOXDUM(3) + DY/2.0D0
        K1(2) = BOXDUM(5) + 2.0D0*DZ/3.0D0
        EF(2) = 3.0D0*(CLCPOT(K1) - CLCPOT(K0))/DZ
        EEF   = sqrt(EF(0)*EF(0) + EF(1)*EF(1) + EF(2)*EF(2))
C..Dipole Moment
        IF(LOCD) THEN
           D1=-AU_DEB*DMOMLO(1,1)
           D2=-AU_DEB*DMOMLO(2,1)
           D3=-AU_DEB*DMOMLO(3,1)
           DD=SQRT(D1*D1+D2*D2+D3*D3)
        ELSE IF(LDIP) THEN
           D1=-AU_DEB*DMOM(1)
           D2=-AU_DEB*DMOM(2)
           D3=-AU_DEB*DMOM(3)
           DD=SQRT(D1*D1+D2*D2+D3*D3)
        ELSE
           D1=-99999.9D0
           D2=-99999.9D0
           D3=-99999.9D0
           DD=-99999.9D0      
        ENDIF
C..Write it to file
        WRITE(5,'(F12.5,TR1,F12.5,TR1,F12.5,TR1,F12.5,A)')
     @            EF(0),EF(1),EF(2),EEF,' E-Field'
        WRITE(5,'(F12.5,TR1,F12.5,TR1,F12.5,TR1,F12.5,A)')
     @            D1,D2,D3,DD,' Debye'
C..
C..added by biswas (dec/2004) to write the forces on MM atoms by QM in qmoutput.out
        IF(IFTYPE.EQ.2) THEN
C..
         DO INR=1,NNEAR
            IDNR=IDNEAR(INR)
            WRITE(5,'(I6,TR1,E15.6,TR1,E15.6,TR1,E15.6)')
     @      IDNR,GMX_FNEAR(1,INR),GMX_FNEAR(2,INR),GMX_FNEAR(3,INR)
         ENDDO
C..
        IF(MOD(STPNO,INTMF).EQ.0.AND.NINTM.GT.0) THEN
         DO INR=1,NINTM
            IDNR=IDINTM(INR)
            WRITE(5,'(I6,TR1,E15.6,TR1,E15.6,TR1,E15.6)')
     @      IDNR,GMX_FINTM(1,INR),GMX_FINTM(2,INR),GMX_FINTM(3,INR)
         ENDDO
        ENDIF
C..
        IF(MOD(STPNO,OUTMF).EQ.0.AND.NOUTR.GT.0) THEN
         DO INR=1,NOUTR
            IDNR=IDOUTR(INR)
            WRITE(5,'(I6,TR1,E15.6,TR1,E15.6,TR1,E15.6)')
     @      IDNR,GMX_FOUTR(1,INR),GMX_FOUTR(2,INR),GMX_FOUTR(3,INR)
         ENDDO
        ENDIF
C..       
        ENDIF
        CLOSE(5)
C
C..Create the EGO/GMXCONTINUE file
C
        IF(IFTYPE.EQ.1) THEN
          STR=FPATH(IAPATH:IEPATH)//'EGOCONTINUE'
          CALL XSTRING(STR,IA,IE)
          LINE=STR(IA:IE)
          OPEN(5,FILE=LINE,STATUS='UNKNOWN')
          REWIND 5
          WRITE(5,'(A)') 'EGOCONTINUE'
          CLOSE(5)
C..added
        ELSEIF(IFTYPE.EQ.2) THEN
          STR=FPATH(IAPATH:IEPATH)//'GMXCONTINUE'
          CALL XSTRING(STR,IA,IE)
          LINE=STR(IA:IE)
          OPEN(5,FILE=LINE,STATUS='UNKNOWN')
          REWIND 5
          WRITE(5,'(A)') 'GMXCONTINUE'
          CLOSE(5)
        ELSE
          CALL STOPGM('INTERFACE','NOT PROGRAMMED')
        ENDIF
        CALL WRGEOF(TAU0,FION)
        WRITE(6,*) ' INTERFACE| FORCES WRITTEN TO FILE '
      ENDIF
      CALL MY_SYNC(ALLGRP)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C
C
C
C     ==================================================================
      SUBROUTINE EGOSTOP
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'fileopen.inc'
C     Variables
      CHARACTER LINE*80,STR*80
      INTEGER   IA,IE
C     ==--------------------------------------------------------------==
      IF(IFTYPE.EQ.1) THEN
        WRITE(6,'(//,A)') ' Creating EGOEXIT file'
        STR=FPATH(IAPATH:IEPATH)//'EGOEXIT'
        CALL XSTRING(STR,IA,IE)
        LINE=STR(IA:IE)
        OPEN(5,FILE=LINE,STATUS='UNKNOWN')
        REWIND 5
        WRITE(5,'(A)') 'EGOEXIT'
        CLOSE(5)
       ELSEIF(IFTYPE.EQ.2) THEN
        WRITE(6,'(//,A)') ' Creating GMXEXIT file'
        STR=FPATH(IAPATH:IEPATH)//'GMXEXIT'
        CALL XSTRING(STR,IA,IE)
        LINE=STR(IA:IE)
        OPEN(5,FILE=LINE,STATUS='UNKNOWN')
        REWIND 5
        WRITE(5,'(A)') 'GMXEXIT'
        CLOSE(5)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MYPROPPT(C0,TAU0,RHOE,V,SCR,LSCR)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'prop.inc'
      INCLUDE 'soft.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'lodp.inc'
      INCLUDE 'atwf.inc'
C     Arguments
      INTEGER    LSCR
      COMPLEX*16 C0(NGW,*),V(*)
      REAL*8     TAU0(*),RHOE(*),SCR(LSCR)
C     Variables
      DIMENSION  F(*)
      REAL*8     FNL(*),DFNL(*)
      REAL*8     FBACK(*)
      POINTER    (IP_FBACK,FBACK)
      CHARACTER  TAG*30
      COMPLEX*16 EIVPS(NHG),EIROP(NHG)
      POINTER    (IP_EIROP,EIROP),(IP_EIVPS,EIVPS)
      INTEGER    I,J,JI,IR,LMYPROPPT
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_MYPROPPT(LMYPROPPT,TAG)
      CALL TEST_SCR('MYPROPPT',TAG,LSCR,LMYPROPPT)
C     ==--------------------------------------------------------------==
      
      IF(PARENT) THEN
        WRITE(6,'(/,1X,63("*"))')
        WRITE(6,'(" *",9X,A,19X,"*")') 
     @                  ' INTERFACE| PROPERTY CALCULATIONS'
        WRITE(6,'(1X,63("*"),/)')
      ENDIF
      CALL PROPIN(INSYS)
C     ==--------------------------------------------------------------==
      NUMORB = MAX(N,NUMORB)
C     ==--------------------------------------------------------------==
C     ==  CALCULATE DIPOLE MOMENTS                                    ==
C     ==--------------------------------------------------------------==
      IF(LDIP.OR.LOCD.OR.LEXT) THEN
        CALL PHFAC(TAU0)
        CALL RNLSM(C0,NUMORB,V,SCR,LSCR,1,1,.FALSE.)
        CALL GET_ADDR(IP_EIROP,SCR(1))
        CALL GET_ADDR(IP_EIVPS,SCR(2*NHG+1))
        IF(LEXT) THEN
          CALL MEMORY(IP_FBACK,NUMORB,'FBACK')
          CALL DCOPY(NUMORB,F,1,FBACK,1)
          DO I=1,NEXDIP
            DO J=1,NUMORB
              JI=(I-1)*NUMORB+J
              F(J)=FBACK(J)-FOCC(JI)
            ENDDO
            CALL DIFRHO(C0,RHOE,V,SCR,LSCR,NUMORB)
            CALL ZAZZERO(EIROP,NHG)
            DO IR=1,NNR1
              V(IR)=DCMPLX(RHOE(IR),0.D0)
            ENDDO
            CALL FWFFT(V)
            CALL EXDIPO(I,TAU0,V,EIROP)
          ENDDO
          CALL DCOPY(NUMORB,FBACK,1,F,1)
        ENDIF
        IF(LDIP.OR.LOCD) THEN
          CALL RHOOFR(C0,RHOE,V,SCR,LSCR,N)
          CALL EICALC(EIVPS,EIROP)
          DO IR=1,NNR1
            V(IR)=DCMPLX(RHOE(IR),0.D0)
          ENDDO
          CALL FWFFT(V)
          IF(LDIP) CALL DIPO(TAU0,EIROP,V)
          IF(LOCD) CALL LODIPO(EIROP,V)
        ENDIF
        IF(PARENT) THEN
          CALL PRDIP
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      IF(TMEMCHK) CALL PRINT_MEMORY
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_INTERFACE_WRITE(LINTERW,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !ICMET
      INCLUDE 'elct.inc'        !N number of states
      INCLUDE 'pslo.inc'        !TIVAN
C     Arguments
      INTEGER   LINTERW 
      CHARACTER TAG*30
C     Variables
      INTEGER   NSTATE,LRHOOFR,LESP,LRNLSM,LMULLIKEN
C     ==--------------------------------------------------------------==
      NSTATE=N
      LRHOOFR=0
      LRNLSM=0
      LESP=0
      LMULLIKEN=0
      IF(ICMET.EQ.2 .OR. ICMET.EQ.3) THEN
        CALL GIVE_SCR_RHOOFR(LRHOOFR,TAG)
        CALL GIVE_SCR_ESPC(LESP,TAG)
      ENDIF
      IF(ICMET.EQ.1 .OR. ICMET.EQ.4) THEN
        CALL GIVE_SCR_MULLIKEN(LMULLIKEN,TAG,NSTATE)
      ELSEIF(ICMET.EQ.2 .OR. ICMET.EQ.3) THEN
        IF(TIVAN) CALL GIVE_SCR_RNLSM(LRNLSM,TAG,NSTATE,.FALSE.)
      ENDIF
      LINTERW=MAX(LRHOOFR,LESP,LRNLSM,LMULLIKEN)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_MYPROPPT(LMYPROPPT,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'        !N number of states
      INCLUDE 'prop.inc'        !NUMORB
C     Arguments
      INTEGER   LMYPROPPT 
      CHARACTER TAG*30
C     Variables
      INTEGER   NUM,LRHOPRI,LRNLSM,LRHOOFR
C     ==--------------------------------------------------------------==
      LRHOPRI=0
      LRNLSM=0
      LRHOOFR=0
      LMYPROPPT=0
      IF(LDIP.OR.LOCD.OR.LEXT) THEN
        NUM=MAX(N,NUMORB)
        CALL GIVE_SCR_RHOPRI(LRHOPRI,TAG,NUM)
        CALL GIVE_SCR_RNLSM(LRNLSM,TAG,NUM,.FALSE.)
C       EIVPS (2*NHG) EIROP (2*NHG)
        LMYPROPPT=4*NHG
        IF(LDIP.OR.LOCD) THEN
          CALL GIVE_SCR_RHOOFR(LRHOOFR,TAG)
        ENDIF
      ENDIF
      LMYPROPPT=MAX(LMYPROPPT,LRNLSM,LRHOOFR)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_INTERFACE(LINTER,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'        !N number of states
      INCLUDE 'nlcc.inc'        !TINLC
C     Arguments
      INTEGER   LINTER 
      CHARACTER TAG*30
C     Variables
      INTEGER   NSTATE,LCOPOT,LUPD,LFORCEDR,LMYPROPPT,LINTERW
C     ==--------------------------------------------------------------==
      NSTATE=N
      LCOPOT=0
      LMYPROPPT=0
      IF(TINLC) CALL GIVE_SCR_COPOT(LCOPOT,TAG)
      CALL GIVE_SCR_UPDWF(LUPD,TAG,NSTATE,.FALSE.)
      CALL GIVE_SCR_FORCEDR(LFORCEDR,TAG,NSTATE,.TRUE.,.TRUE.)
      IF(PROPER) CALL GIVE_SCR_MYPROPPT(LMYPROPPT,TAG)
      CALL GIVE_SCR_INTERFACE_WRITE(LINTERW,TAG)
      LINTER=MAX(LCOPOT,LUPD,LFORCEDR,LMYPROPPT,LINTERW)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
C..added in dec/2004: biswas; calculates forces on  gmx atoms due to QM.
C     ==================================================================
      SUBROUTINE GMX_EL_FORCES(C0,TAU0,RHOE,V,SCR,LSCR,NSTATE,NKPOINT)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'epot.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'kpts.inc'
C     Variables
      INTEGER   INR
      REAL*8    TAU0(3,NAX,NSX)
      INTEGER   LSCR,NSTATE,NKPOINT
      INTEGER   NDIM,IKIND
      COMPLEX*16 C0(NGWK,NSTATE,NKPOINT),V(MAXFFT)
      REAL*8     RHOE(NNR1),SCR(LSCR),GMX_FMM(3,MAXOUTR)
      COMPLEX*16 VTEMP(NHG)
      POINTER    (IP_VTEMP,VTEMP)
C     ==--------------------------------------------------------------==
C..To create rho(r):
      CALL GET_ADDR(IP_VTEMP,SCR(1))
      DO IKIND=1,NKPOINT
        CALL RNLSM(C0(1,1,IKIND),NSTATE,V,SCR,LSCR,1,IKIND,.FALSE.)
      ENDDO
      IF(TKPNT) THEN
        CALL RHOOFR_C(C0,RHOE,V,SCR,LSCR,NSTATE)
      ELSE
        CALL RHOOFR  (C0,RHOE,V,SCR,LSCR,NSTATE)
      ENDIF
C..End of creation of rho(r)
C     ==--------------------------------------------------------------==
C..Calls to calculate forces for three different layers: near, intermediate, outer
C     ==--------------------------------------------------------------==
      CALL GMX_EL_FORCE(NNEAR,KONEAR,DATNEAR,TAU0,RHOE,GMX_FMM)
      DO INR=1,NNEAR
        DO NDIM=1,3
          GMX_FNEAR(NDIM,INR)=GMX_FMM(NDIM,INR)
        ENDDO
      ENDDO
C..
      IF(MOD(STPNO,INTMF).EQ.0.AND.NINTM.GT.0) THEN 
        CALL GMX_EL_FORCE(NINTM,KOINTM,QINTM,TAU0,RHOE,GMX_FMM)
        DO INR=1,NINTM
          DO NDIM=1,3
            GMX_FINTM(NDIM,INR)=GMX_FMM(NDIM,INR)
          ENDDO
        ENDDO
      ENDIF
C..
      IF(MOD(STPNO,INTMF).EQ.0.AND.NINTM.GT.0) THEN
        CALL GMX_EL_FORCE(NOUTR,KOOUTR,QOUTR,TAU0,RHOE,GMX_FMM)
        DO INR=1,NNEAR
          DO NDIM=1,3
            GMX_FOUTR(NDIM,INR)=GMX_FMM(NDIM,INR)
          ENDDO
        ENDDO
      ENDIF
C..
C     ==--------------------------------------------------------------==
      RETURN
      END
C
C     ==================================================================
C.....Calculates QM-2-MM electrostatic forces on MM atoms.
C     ==================================================================
      SUBROUTINE GMX_EL_FORCE(NMM,KOMM,DATMM,TAU0,RHOE,GMX_FMM)     
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'epot.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cnst.inc'
C     Variables
      INTEGER   I,J,K,IR,INR,IDNR,IS,IA
      REAL*8    KO(0:2),XJ,YJ,ZJ,RRJS,RRJ,RRJQ,DI,DJ,DK,GFAC
      REAL*8    TAU0(3,NAX,NSX)
      REAL*8    ALFA,BETA,ALAM,RFFAC,ZFAC
      REAL*8    XI,YI,ZI,RIJS,RIJ,RIJQ
      REAL*8    FORCE_CONST,BOHR_TO_NM
      REAL*8    RHOE(NNR1),QJ_RHO
      REAL*8    GMX_F_IP(3,MAXOUTR),GMX_F_IPS(3,MAXOUTR)
      REAL*8    GMX_FMM(3,MAXOUTR),KOMM(3,MAXOUTR),DATMM(5,MAXOUTR)
#if defined(PARALLEL)
      INTEGER   MY_LEN,MY_BYTES,MSGID,IP,IPP
#endif
      INTEGER   NDIM,IKR1,NMM,IDMM(MAXOUTR)
      COMPLEX*16 VTEMP(NHG)
      POINTER    (IP_VTEMP,VTEMP)
C..Constants
C   1/(4*pi*epsilon_0)=138.935485 KJ*mol^-1*nm*e^-2; 
      PARAMETER (FORCE_CONST=138.935485D0)
      DATA ALAM /1.3d0/
C     ==--------------------------------------------------------------==
      BOHR_TO_NM=0.1D0/FBOHR
C..
      DK=(BOXDUM(6)-BOXDUM(5))/DBLE(NR3S)
      DJ=(BOXDUM(4)-BOXDUM(3))/DBLE(NR2S)
      DI=(BOXDUM(2)-BOXDUM(1))/DBLE(NR1S)
C..   
      GFAC=OMEGA/DBLE(NR1S*NR2S*NR3S)
      IKR1 = MOD(NR1+1,2)
C..
      DO INR=1,NMM
        DO NDIM=1,3
          GMX_FMM(NDIM,INR)=0.D0
          GMX_F_IP(NDIM,INR)=0.D0
          GMX_F_IPS(NDIM,INR)=0.D0        
        ENDDO
      ENDDO
C..   
C----------------------------------------------------------------------C
C..Calculate contributions from densities on the grid.
C----------------------------------------------------------------------C
      DO INR=1,NMM 
        IDNR=IDMM(INR)
        XJ=KOMM(1,INR)
        YJ=KOMM(2,INR)
        ZJ=KOMM(3,INR)
C..   
        ALFA = ALAM*DATMM(3,INR)          
        BETA = 2.d0*ALFA
C..   
        IR = 0
        DO K=1,KR3S
          KO(2)=BOXDUM(5)+DBLE(K-1)*DK
          DO J=1,KR2S
            KO(1)=BOXDUM(3)+DBLE(J-1)*DJ
            DO I=NRXPL(MEPOS,1),NRXPL(MEPOS,2)+IKR1
              KO(0)=BOXDUM(1)+DBLE(I-1)*DI
C..   
              IR=IR+1
C..   
              IF(IR.GT.NNR1) print*,'Error in looping in GMX_EL'
C..   
              RRJS = (XJ-KO(0))**2+(YJ-KO(1))**2+(ZJ-KO(2))**2
              RRJ = DSQRT(RRJS)
              RRJQ = RRJ*RRJS
              QJ_RHO = DATMM(1,INR)*RHOE(IR)
C..   
              RFFAC=1.D0/RRJQ-DEXP(-BETA*RRJ)*(1.D0/RRJQ+BETA/RRJS+
     &             ALFA*BETA/RRJ)
C                      RVFAC=1./RRJ - EXP(-BETA*RRJ)*(1./RRJ + ALFA) 
C----------------------------------------------------------------------C
              GMX_F_IP(1,INR)=GMX_F_IP(1,INR)-QJ_RHO*(XJ-KO(0))*RFFAC
              GMX_F_IP(2,INR)=GMX_F_IP(2,INR)-QJ_RHO*(YJ-KO(1))*RFFAC
              GMX_F_IP(3,INR)=GMX_F_IP(3,INR)-QJ_RHO*(ZJ-KO(2))*RFFAC
C----------------------------------------------------------------------C       
            ENDDO
          ENDDO
        ENDDO
C== Normalization for rho === MM-charge is already multiplied.
        GMX_F_IP(1,INR)=GMX_F_IP(1,INR)*GFAC
        GMX_F_IP(2,INR)=GMX_F_IP(2,INR)*GFAC
        GMX_F_IP(3,INR)=GMX_F_IP(3,INR)*GFAC
C..   
      ENDDO
C----------------------------------------------------------------------C
C..
#if defined(PARALLEL)
      DO IPP=1,NPROC
        IP=PGROUP(IPP)
        MSGID = IP
        MY_LEN=NMM*24
        IF(PARENT) THEN
          IF(IP.EQ.ME) THEN
            DO INR=1,NMM
              DO NDIM=1,3
                GMX_F_IPS(NDIM,INR) = GMX_F_IPS(NDIM,INR) + 
     &               GMX_F_IP(NDIM,INR)
              ENDDO
            ENDDO
          ELSE
            MSGID=1
            CALL MY_BRECV(GMX_F_IP,MY_LEN,IP,MSGID,MY_BYTES)
            DO INR=1,NMM
              DO NDIM=1,3
                GMX_F_IPS(NDIM,INR) = GMX_F_IPS(NDIM,INR) + 
     &               GMX_F_IP(NDIM,INR)
              ENDDO
            ENDDO
          ENDIF
C..   
        ELSE
          IF(IP.EQ.ME) THEN
            MSGID=1
            CALL MY_BSEND(GMX_F_IP,MY_LEN,PGROUP(1),MSGID)
          ENDIF
        ENDIF
      ENDDO
C..   
      IF(PARENT) THEN
        DO INR=1,NMM
          DO NDIM=1,3
            GMX_FMM(NDIM,INR) = GMX_F_IPS(NDIM,INR)
          ENDDO
        ENDDO
      ENDIF
C..   
#else 
C..   
      DO INR=1,NMM
        DO NDIM=1,3
          GMX_FMM(NDIM,INR) = GMX_F_IP(NDIM,INR)
        ENDDO
      ENDDO
C..
#endif
C..
C----------------------------------------------------------------------C
C..Calculate & add contributions from QM-core ions.
C----------------------------------------------------------------------C
C..
      IF(PARENT) THEN
C..
        DO INR=1,NMM 
          IDNR=IDMM(INR)
          XJ=KOMM(1,INR)
          YJ=KOMM(2,INR)
          ZJ=KOMM(3,INR)
C..   
          ALFA = ALAM*DATMM(3,INR)             
          BETA = 2.d0*ALFA
C..   
          DO IS=1,NSP
            DO IA=1,NA(IS)
              XI=TAU0(1,IA,IS)
              YI=TAU0(2,IA,IS)
              ZI=TAU0(3,IA,IS)          
              RIJS = (XJ-XI)**2+(YJ-YI)**2+(ZJ-ZI)**2
              RIJ = DSQRT(RIJS)
              RIJQ = RIJ*RIJS
              ZFAC=1.d0/RIJQ-DEXP(-BETA*RIJ)*
     &             (1.d0/RIJQ+BETA/RIJS+ALFA*BETA/RIJ)
C     ZVAC=1./RIJ - EXP(-BETA*RIJ)*(1./RIJ + ALFA) 
C..
              GMX_FMM(1,INR)=GMX_FMM(1,INR)+DATMM(1,INR)*ZV(IS)
     &             *(XJ-XI)*ZFAC       
              GMX_FMM(2,INR)=GMX_FMM(2,INR)+DATMM(1,INR)*ZV(IS)
     &             *(YJ-YI)*ZFAC   
              GMX_FMM(3,INR)=GMX_FMM(3,INR)+DATMM(1,INR)*ZV(IS)
     &             *(ZJ-ZI)*ZFAC
C..   
            ENDDO
          ENDDO
C..   
          GMX_FMM(1,INR)=GMX_FMM(1,INR)*FORCE_CONST/BOHR_TO_NM**2
          GMX_FMM(2,INR)=GMX_FMM(2,INR)*FORCE_CONST/BOHR_TO_NM**2
          GMX_FMM(3,INR)=GMX_FMM(3,INR)*FORCE_CONST/BOHR_TO_NM**2
C..   
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
