C     ==================================================================
      SUBROUTINE EHPSI(C0,C2,EXP_VPOT,PSI,NSTATE)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE ACTION OF APPLYING e^{(-beta/P)H}  TO A WAVEFUNCTION C0,==
C     ==  RETURNING THE ANSWER IN C2.                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'norm.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
      INCLUDE 'coor.inc' 
      INCLUDE 'sfac.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'kpts.inc'
C     Arguments
      INTEGER    NSTATE      
      COMPLEX*16 C0(NGW,NSTATE),C2(NGW,NSTATE),PSI(MAXFFT*NOGRP)
      REAL*8     EXP_VPOT(NNR1)             
C     Variables
      DIMENSION  EMK2(NGW),CNL(NGW,NLM)
      REAL*8     FNL(*),DFNL(*)
      REAL*8     PSIX(*),EXP_VPOTX(*)
      POINTER    (IP_PSIX,PSIX),(IP_EXP_VPOTX,EXP_VPOTX)
      COMPLEX*16 FP,FM
      INTEGER    ISUB,IFFT,I,IG,LEAD,NNRX,
     &           NJUMP,IA,NSTA,IB,IBB,IS1,IS2
#ifdef PARALLEL
      INTEGER    MSGLEN,NL2,IX1,IXP,IX
#endif
C     ==--------------------------------------------------------------==
C     == INITIALIZE ELEC-FORCE ARRAYS                                 ==
C     ==--------------------------------------------------------------==
      CALL TISET('     EHPSI',ISUB)
C     ==--------------------------------------------------------------==
#ifdef PARALLEL
      IF(NOGRP.EQ.1) THEN
        LEAD  = KR1S*NGRAYS
        IFFT  = 1
        NNRX=NNR1
      ELSE
        LEAD  = KR1S*NGRM
        IFFT=2
        NNRX=KRX*KR2S*KR3S
      ENDIF
#else
      LEAD  = NNR1
      NNRX=NNR1
      IFFT=1
#endif
      CALL GET_ADDR(IP_PSIX,PSI(1))
      CALL GET_ADDR(IP_EXP_VPOTX,EXP_VPOT(1))
#ifdef PARALLEL
      IF(NOGRP.GT.1) THEN
C       Summation of density within orbital split
        NL2=NLINK(NOLIST(NOGRP))
        IX1=NRXPL(MEPOS,1)-NRXPL(NL2,1)
        MSGLEN= 8 * NNRX
C       EXP(-betaP*0) = 1!!!
!$OMP parallel do private(I)
#ifdef _vpp_
!OCL NOVREC
#endif
        DO I=1,NNRX
          PSIX(I)=1.D0
        ENDDO
        DO IXP=1,NR1
          IX=IX1+IXP
          CALL DCOPY(KR2S*KR3S,EXP_VPOT(IXP),KR1,PSIX(IX),KRX)
        ENDDO
C       WARNING: COMBINE with MULTIPLICATION operation
        CALL MY_COMBINE(PSIX,EXP_VPOTX,MSGLEN,5,MEOGRP)
      ENDIF
#endif
C     ==-----------------------------------------------------------==
C     == APPLY THE KINETIC ENERGY PROPAGATOR TO C0, RETURNING THE  ==
C     == ANSWER IN PSI. ARRAY EMK2 CONTAINS e^{-beta g^2/4P}       ==
C     ==-----------------------------------------------------------==
!$OMP parallel do private(I,IG)
      DO I=1,NSTATE
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(1000))
!IBM* INDEPENDENT
#endif
        DO IG = 1 , NGW
          C2(IG,I) = EMK2(IG)*C0(IG,I)
        ENDDO
      ENDDO
      IF(NLM.GT.0) THEN
        CALL OVLAP2(NGW,NLM,NSTATE,AFNL,CNL,C2(1,1))
#ifdef PARALLEL
        CALL GLOSUM(NSTATE*NLM,AFNL)
#endif
        CALL CALC_BILN(NSTATE,1)
      ENDIF
C     ==--------------------------------------------------------------==
C     == WARNING: 
C     == HERE WE APPLY EH PSI TWO AT A TIME, MAKING USE OF THE FACT   ==
C     == THE WAVEFUNCTIONS ARE REAL.                                  ==
C     ==--------------------------------------------------------------==
      NJUMP=2*NOGRP
      DO IA = 1 , NSTATE , NJUMP
        NSTA=MIN((NSTATE-IA+2)/2,NOGRP)
        CALL ZAZZERO(PSI,MAXFFT*NOGRP)
C       ==------------------------------------------------------------==
C       == Store the wavefunctions in array PSI                       ==
C       == in the following way:                                      ==
C       == Since we use the Gamma point only, the wavefunctions in    ==
C       == real space are real. This fact is used to save time by     ==
C       == using complex arrays where the wavefunctions of            ==
C       == 2 different states (i and i+1) are combined, and           ==
C       == performing complex Fourier Transforms.                     ==
C       == Warning: If you use K-points other than Gamma, or zone     ==
C       == boundary, this symmetry is broken and the trick is not     ==
C       == valid anymore. First step is to build the array PSI.       ==
C       ==------------------------------------------------------------==
        DO IB=1,NSTA
          I=IA+2*(IB-1)
          IBB=(IB-1)*LEAD
          IS1 = I
          IS2 = I+1
          IF(IS2.GT.NSTATE) THEN
C           One state
!$OMP parallel do private(IG)
            DO IG=1,NGW
              PSI(NZHS(IG)+IBB)=C2(IG,IS1)
cmb            ENDDO         ! useless to repeat
cmb            DO IG=1,NGW   ! twice the loop
              PSI(INDZS(IG)+IBB)=DCONJG(C2(IG,IS1))
            ENDDO
            IF(GEQ0) PSI(NZHS(1)+IBB)=C2(1,IS1)
          ELSE
C           Two states
!$OMP parallel do private(IG)
            DO IG=1,NGW
              PSI(NZHS(IG)+IBB)=C2(IG,IS1)+UIMAG*C2(IG,IS2)
cmb            ENDDO        ! useless to repeat
cmb            DO IG=1,NGW  ! twice the loop
              PSI(INDZS(IG)+IBB)=DCONJG(C2(IG,IS1))+
     *                     UIMAG*DCONJG(C2(IG,IS2))
            ENDDO
            IF(GEQ0) PSI(NZHS(1)+IBB)=C2(1,IS1)+UIMAG*DCONJG(C2(1,IS2))
          ENDIF
        ENDDO
C       ==------------------------------------------------------------==
C       == INVERSE FFT PSI TO OBTAIN PSI ON REAL SPACE MESH           ==
C       ==------------------------------------------------------------==
        IF(IFFT.EQ.1) THEN 
          CALL S_INVFFT(PSI)
        ELSE 
          CALL SG_INVFFT(PSI) 
        ENDIF
C       ==------------------------------------------------------------==
C       == APPLY THE EXPONENTIATED POTENTIAL TO PSI                   ==
C       == APPLY e^(-b/P V) IN REAL SPACE, AND RETURN ANSWER IN PSI.  ==
C       == ARRAY POT CONTAINS e^(-b/P V(r)).                          ==
C       ==------------------------------------------------------------==
        CALL EVPSI(NNRX,PSI,EXP_VPOTX)
C       ==------------------------------------------------------------==
C       == TRANSFORM BACK TO RECIPROCAL SPACE                         ==
C       ==------------------------------------------------------------==
        IF(IFFT.EQ.1) THEN 
          CALL S_FWFFT(PSI)
        ELSE 
          CALL SG_FWFFT(PSI) 
        ENDIF
C       ==------------------------------------------------------------==
C       == Decode the combination of wavefunctions (now multiplied by ==
C       == the potential), back to those of states i and i+1.         ==
C       ==------------------------------------------------------------==
        DO IB=1,NSTA
          I=IA+2*(IB-1)
          IBB=(IB-1)*LEAD
          IS1 = I
          IS2 = I+1
          IF(IS2.GT.NSTATE) THEN
!$OMP parallel do private(IG)
            DO IG = 1 , NGW
              C2(IG,IS1) = PSI(NZHS(IG)+IBB)
            ENDDO
          ELSE
!$OMP parallel do private(IG,FP,FM)
            DO IG = 1 , NGW
              FP=(PSI(NZHS(IG)+IBB)+PSI(INDZS(IG)+IBB))*0.5D0
              FM=(PSI(NZHS(IG)+IBB)-PSI(INDZS(IG)+IBB))*0.5D0
              C2(IG,IS1) = DCMPLX(DREAL(FP), DIMAG(FM))
              C2(IG,IS2) = DCMPLX(DIMAG(FP),-DREAL(FM))
            ENDDO
          ENDIF
        ENDDO
      ENDDO
C     ==------------------------------------------------------------==
C     == ADD NON-LOCAL PP PART                                      ==
C     ==------------------------------------------------------------==
      IF(NLM.GT.0) THEN
        CALL DGEMM('N','N',2*NGW,NSTATE,NLM,1.D0,CNL,2*NGW,
     &       BILN(1,1,1),NLM,1.D0,C2(1,1),2*NGW)
      ENDIF
C     ==--------------------------------------------------------------==
C     == APPLY THE KINETIC ENERGY PROPAGATOR TO C0, RETURNING THE     ==
C     == ANSWER IN C2                                                 ==
C     ==--------------------------------------------------------------==
      DO I=1,NSTATE
!$OMP parallel do private(IG)
        DO IG = 1 , NGW
          C2(IG,I)  = EMK2(IG)*C2(IG,I)
        ENDDO
        IF(GEQ0) THEN 
          C2(1,I) = DCMPLX(DREAL(C2(1,I)),0.D0)
        ENDIF
      ENDDO 
#ifdef PARALLEL
C     Reestablish initial conditions for the potential
      IF(NOGRP.GT.1) THEN
        NL2=NLINK(NOLIST(NOGRP))
        IX1=NRXPL(MEPOS,1)-NRXPL(NL2,1)
        CALL DCOPY(NNRX,EXP_VPOTX(1),1,PSIX(1),1)
        CALL AZZERO(EXP_VPOTX,NNRX)
        DO IXP=1,NR1
          IX=IX1+IXP
          CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,EXP_VPOT(IXP),KR1)
        ENDDO
      ENDIF
#endif
C     ==--------------------------------------------------------------==
      CALL TIHALT('     EHPSI',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE EHPSI_C(C0,C2,EXP_VPOT,PSI,NSTATE,IKIND)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE ACTION OF APPLYING e^{(-beta/P)H}  TO A WAVEFUNCTION C0,==
C     ==  RETURNING THE ANSWER IN C2.                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'coor.inc' 
      INCLUDE 'norm.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'kpts.inc'
C     Arguments
      INTEGER    NSTATE,IKIND
      COMPLEX*16 C0(NGWK,NSTATE),C2(NGWK,NSTATE),PSI(MAXFFT*NOGRP)
      REAL*8     EXP_VPOT(NNR1)
C     Variables
      COMPLEX*16 ZONE
      PARAMETER  (ZONE=(1.D0,0.D0))
      DIMENSION  EMK2(NGWK,NKPNT),CNL(NGWK,NLM)
      REAL*8     PSIX(*),EXP_VPOTX(*)
      POINTER    (IP_PSIX,PSIX),(IP_EXP_VPOTX,EXP_VPOTX)
      INTEGER    ISUB,IFFT,I,IG,LEAD,NNRX,IA,IBB,IB,NSTA
#ifdef PARALLEL
      INTEGER    MSGLEN,NL2,IX1,IXP,IX
#endif
C     ==--------------------------------------------------------------==
C     == INITIALIZE ELEC-FORCE ARRAYS                                 ==
C     ==--------------------------------------------------------------==
      CALL TISET('   EHPSI_C',ISUB)
C     ==--------------------------------------------------------------==
#ifdef PARALLEL
      IF(NOGRP.EQ.1) THEN
        LEAD  = KR1S*NGRAYS
        IFFT  = 1
        NNRX=NNR1
      ELSE
        LEAD  = KR1S*NGRM
        IFFT=2
        NNRX=KRX*KR2S*KR3S
      ENDIF
#else
      LEAD  = NNR1
      NNRX=NNR1
      IFFT=1
#endif
      CALL GET_ADDR(IP_PSIX,PSI(1))
      CALL GET_ADDR(IP_EXP_VPOTX,EXP_VPOT(1))
#ifdef PARALLEL
      IF(NOGRP.GT.1) THEN
C       Summation of density within orbital split
        NL2=NLINK(NOLIST(NOGRP))
        IX1=NRXPL(MEPOS,1)-NRXPL(NL2,1)
        MSGLEN= 8 * NNRX
C       EXP(-betaP*0) = 1!!!
!$OMP parallel do private(I)
        DO I=1,NNRX
          PSIX(I)=1.D0
        ENDDO
        DO IXP=1,NR1
          IX=IX1+IXP
          CALL DCOPY(KR2S*KR3S,EXP_VPOT(IXP),KR1,PSIX(IX),KRX)
        ENDDO
C       WARNING: COMBINE with MULTIPLICATION operation
        CALL MY_COMBINE(PSIX,EXP_VPOTX,MSGLEN,5,MEOGRP)
      ENDIF
#endif
C     ==--------------------------------------------------------------==
C     == APPLY THE KINETIC ENERGY PROPAGATOR TO C0, RETURNING THE     ==
C     == ANSWER IN PSI. ARRAY EMK2 CONTAINS e^{-beta g^2/4P}          ==
C     ==--------------------------------------------------------------==
!$OMP parallel do private(I,IG)
      DO I=1,NSTATE
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(1000))
!IBM* INDEPENDENT
#endif
        DO IG = 1,NGWK
          C2(IG,I) = EMK2(IG,IKIND)*C0(IG,I)
        ENDDO
      ENDDO
      IF(NLM.GT.0) THEN
        CALL OVLAP2_C(NGWK,NLM,NSTATE,AFNL,CNL(1,1),C2(1,1))
#ifdef PARALLEL
        CALL GLOSUM(2*NLM*NSTATE,AFNL)
#endif
        CALL CALC_BILN(NSTATE,IKIND)
      ENDIF
C     ==--------------------------------------------------------------==
      DO IA=1,NSTATE,NOGRP
        NSTA=MIN((NSTATE-IA+1),NOGRP)
        CALL ZAZZERO(PSI,MAXFFT*NOGRP)
        DO IB=1,NSTA
          I=IA+(IB-1)
          IBB=(IB-1)*LEAD
!$OMP parallel do private(IG)
          DO IG = 1 , NGW
            PSI(NZHS(IG)+IBB) = C2(IG,I)
cmb          ENDDO          ! useless to repeat
cmb          DO IG=1,NGW    ! twice the loop
            PSI(INDZS(IG)+IBB) = C2(IG+NGW,I)
          ENDDO
          IF(GEQ0) PSI(NZHS(1)+IBB) = C2(1,I)
        ENDDO
C       ==------------------------------------------------------------==
C       == INVERSE FFT PSI TO OBTAIN PSI ON REAL SPACE MESH           ==
C       ==------------------------------------------------------------==
        IF(IFFT.EQ.1) THEN 
          CALL S_INVFFT(PSI)
        ELSE 
          CALL SG_INVFFT(PSI) 
        ENDIF
C       ==------------------------------------------------------------==
C       == APPLY THE EXPONENTIATED POTENTIAL TO PSI                   ==
C       == APPLY e^(-b/P V) IN REAL SPACE, AND RETURN ANSWER IN PSI.  ==
C       == ARRAY POT CONTAINS e^(-b/P V(r)).                          ==
C       ==------------------------------------------------------------==
        CALL EVPSI(NNRX,PSI,EXP_VPOTX)
C       ==------------------------------------------------------------==
C       == TRANSFORM BACK TO FOURIER SPACE                            ==
C       ==------------------------------------------------------------==
        IF(IFFT.EQ.1) THEN 
          CALL S_FWFFT(PSI)
        ELSE 
          CALL SG_FWFFT(PSI) 
        ENDIF
C       ==------------------------------------------------------------==
        DO IB=1,NSTA
          I=IA+(IB-1)
          IBB=(IB-1)*LEAD
!$OMP parallel do private(IG)
          DO IG = 1 , NGW
            C2(IG,I) = PSI(NZHS(IG)+IBB)
cmb          ENDDO            ! useless to repeat
cmb          DO IG = 1 , NGW  ! twice the loop
            C2(IG+NGW,I) = PSI(INDZS(IG)+IBB)
          ENDDO
          IF(GEQ0) C2(NGW+1,I) = DCMPLX(0.D0,0.D0)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     == ADD NON-LOCAL PP PART                                        ==
C     ==--------------------------------------------------------------==
      IF(NLM.GT.0) THEN
        CALL ZGEMM('N','N',NGWK,NSTATE,NLM,ZONE,CNL(1,1),NGWK,
     &       BILN(1,1,1),NLM,ZONE,C2(1,1),NGWK)
      ENDIF
C     ==--------------------------------------------------------------==
C     == APPLY THE KINETIC ENERGY PROPAGATOR TO C0, RETURNING THE     ==
C     == ANSWER IN C2                                                 ==
C     ==--------------------------------------------------------------==
      DO I=1,NSTATE 
!$OMP parallel do private(IG)
        DO IG = 1,NGWK
          C2(IG,I) = C2(IG,I)*EMK2(IG,IKIND)
        ENDDO
        IF(GEQ0) C2(NGW+1,I) = DCMPLX(0.D0,0.D0)
      ENDDO
      CALL C_CLEAN(C2,NSTATE,IKIND)
#ifdef PARALLEL
C     Reestablish initial conditions for the potential
      IF(NOGRP.GT.1) THEN
        NL2=NLINK(NOLIST(NOGRP))
        IX1=NRXPL(MEPOS,1)-NRXPL(NL2,1)
        CALL DCOPY(NNRX,EXP_VPOTX(1),1,PSIX(1),1)
        CALL AZZERO(EXP_VPOTX,NNRX)
        DO IXP=1,NR1
          IX=IX1+IXP
          CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,EXP_VPOT(IXP),KR1)
        ENDDO
      ENDIF
#endif
C     ==--------------------------------------------------------------==
      CALL TIHALT('   EHPSI_C',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE EPOT(VPOT)
C     ==--------------------------------------------------------------==
C     ==  RETURNS THE EXPONENTIATED POTENTIAL IN VPOT                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
C     Arguments
      REAL*8  VPOT(NNR1,NLSD)
C     Variables
      INTEGER II,IR
C     ==--------------------------------------------------------------==
!$OMP parallel do private(II,IR)
      DO II = 1 , NLSD
        DO IR = 1 , NNR1 
          VPOT(IR,II)=DEXP(-BETAP*VPOT(IR,II))
        ENDDO
      ENDDO
      RETURN
      END
C     ==================================================================
      SUBROUTINE EG2
C     ==--------------------------------------------------------------==
C     ==  RETURNS THE EXPONENTIATED KINETIC ENERGY IN G-SPACE         ==
C     ==  IN ARRAY EMK2                                               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
C     Variables
      DIMENSION EMK2(NGWK,NKPNT),HGKP(NHG,NKPNT),HGKM(NHG,NKPNT)
      REAL*8    RS1
      INTEGER   KBEG,KEND,KINC,IKPT,IG
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_EMK2,NGWK*NKPTALL,'EMK2')
      IF(TKPNT) THEN
        CALL INQ_SWAP(KBEG,KEND,KINC)
        DO IKPT=KBEG,KEND,KINC
          IF(TKBLOCK) CALL RKPT_SWAP(RS1,1,IKPT,'HGKP HGKM')
          CALL CALC_EMK2(IKPT)
          IF(TKBLOCK) CALL WKPT_SWAP(EMK2,1,IKPT,'EMK2')
        ENDDO
      ELSE
!$OMP parallel do private(IG)
        DO IG = 1, NGW
          EMK2(IG,1) = DEXP(-BETAP*HG(IG)*TPIBA2*0.25D0)
        ENDDO
        IF(GEQ0) EMK2(1,1) = 1.D0 
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CALC_EMK2(IKPT)
C     ==--------------------------------------------------------------==
C     == CALCULATE EMK2 FOR A SET OF K POINTS (SEE EG2)               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NKPNT NGW
      INCLUDE 'kpnt.inc'        !HGKP HGKM
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'        !EMK2 AFNL BILN ALM
      INCLUDE 'geq0.inc'        !GEQ0
C     Arguments
      INTEGER IKPT
C     Variables
      DIMENSION EMK2(NGWK,NKPNT),HGKP(NHG,NKPNT),HGKM(NHG,NKPNT)
      INTEGER   IK,IG
C     ==--------------------------------------------------------------==
      DO IK=1,NKPBL(IKPT)
!$OMP parallel do private(IG)
        DO IG = 1, NGW
          EMK2(IG,IK)     = DEXP(-BETAP*HGKP(IG,IK)*TPIBA2*0.25D0)
          EMK2(NGW+IG,IK) = DEXP(-BETAP*HGKM(IG,IK)*TPIBA2*0.25D0)
        ENDDO
        IF(GEQ0) EMK2(NGW+1,IK) = 0.D0 
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CALC_BILN(NXX,IKIND)
C     ==--------------------------------------------------------------==
C     == COMPUTES BILN                                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'pslo.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'sgpp.inc'
C     Arguments
      INTEGER    NXX,IKIND
C     Variables
      COMPLEX*16 ZZERO
      PARAMETER  (ZZERO=(0.D0,0.D0))
      REAL*8     BFNL(2,NHX)
      REAL*8     FNL(*),DFNL(*)
      INTEGER    ISUB,I,J,INLM,IS,IA,IV,KI,L,LI,JV,L2,LJ,KJ,IM
      COMPLEX*16 ZBETAP
C     ==--------------------------------------------------------------==
      CALL TISET(' CALC_BILN',ISUB)
      DO I=1,NXX
        INLM=0
        DO IS=1,NSP
          DO IA=1,NA(IS)
            IF(TSGP(IS)) THEN
              CALL AZZERO(BFNL,2*NGH(IS))
              DO IV=1,NGH(IS)
                L=NGHTOL(IV,IS)+1
                KI=LFVAL(IV,IS)
                LI=LPVAL(IV,IS)
                IF(TKPNT) THEN
                  DO JV=1,NGH(IS)
                    L2=NGHTOL(JV,IS)+1
                    LJ=LPVAL(JV,IS)
                    IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                      KJ=LFVAL(JV,IS)
                      BFNL(1,IV)=BFNL(1,IV)+
     &                     HLSG(KI,KJ,L,IS)*AFNL(1,INLM+JV,I)
                      BFNL(2,IV)=BFNL(2,IV)+
     &                     HLSG(KI,KJ,L,IS)*AFNL(2,INLM+JV,I)
                    ENDIF
                  ENDDO
                ELSE
                  DO JV=1,NGH(IS)
                    L2=NGHTOL(JV,IS)+1
                    LJ=LPVAL(JV,IS)
                    IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                      KJ=LFVAL(JV,IS)
                      BFNL(1,IV)=BFNL(1,IV)+
     &                     HLSG(KI,KJ,L,IS)*AFNL(1,INLM+JV,I)
                    ENDIF
                  ENDDO
                ENDIF
              ENDDO
              IF(TKPNT) THEN
                CALL DCOPY(IMAGP*NGH(IS),BFNL(1,1),1,AFNL(1,INLM+1,I),1)
              ELSE
                CALL DCOPY(NGH(IS),BFNL(1,1),2,AFNL(1,INLM+1,I),1)
              ENDIF
              INLM=INLM+NGH(IS)
            ELSE
              DO IV=1,NGH(IS)
                INLM=INLM+1
                AFNL(1,INLM,I)=AFNL(1,INLM,I)*WSG(IS,IV)
              ENDDO
              IF(TKPNT) THEN
                INLM=INLM-NGH(IS)
                DO IV=1,NGH(IS)
                  INLM=INLM+1
                  AFNL(2,INLM,I)=AFNL(2,INLM,I)*WSG(IS,IV)
                ENDDO
              ENDIF
            ENDIF
          ENDDO
        ENDDO
      ENDDO
      IF(TKPNT) THEN
        ZBETAP=DCMPLX(0.5D0*BETAP,0.D0)
        CALL ZGEMM('N','N',NLM,NXX,NLM,ZBETAP,ALM(1,1,1,IKIND),
     &       NLM,AFNL,NLM,ZZERO,BILN,NLM)
      ELSE
        CALL DGEMM('N','N',NLM,NXX,NLM,0.5D0*BETAP,ALM(1,1,1,IKIND),
     &       NLM,AFNL,NLM,0.D0 ,BILN,NLM)
      ENDIF
      DO I=1,NXX
        INLM=0
        DO IS=1,NSP
          DO IA=1,NA(IS)
            IF(TSGP(IS)) THEN
              CALL AZZERO(BFNL,2*NGH(IS))
              DO IV=1,NGH(IS)
                L=NGHTOL(IV,IS)+1
                KI=LFVAL(IV,IS)
                LI=LPVAL(IV,IS)
                IF(TKPNT) THEN
                  DO JV=1,NGH(IS)
                    L2=NGHTOL(JV,IS)+1
                    LJ=LPVAL(JV,IS)
                    IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                      KJ=LFVAL(JV,IS)
                      BFNL(1,IV)=BFNL(1,IV)+
     &                     HLSG(KI,KJ,L,IS)*BILN(1,INLM+JV,I)
                      BFNL(2,IV)=BFNL(2,IV)+
     &                     HLSG(KI,KJ,L,IS)*BILN(2,INLM+JV,I)
                    ENDIF
                  ENDDO
                ELSE
                  DO JV=1,NGH(IS)
                    L2=NGHTOL(JV,IS)+1
                    LJ=LPVAL(JV,IS)
                    IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                      KJ=LFVAL(JV,IS)
                      BFNL(1,IV)=BFNL(1,IV)+
     &                     HLSG(KI,KJ,L,IS)*BILN(1,INLM+JV,I)
                    ENDIF
                  ENDDO
                ENDIF
              ENDDO
              IF(TKPNT) THEN
                CALL DCOPY(IMAGP*NGH(IS),BFNL(1,1),1,BILN(1,INLM+1,I),1)
              ELSE
                CALL DCOPY(NGH(IS),BFNL(1,1),2,BILN(1,INLM+1,I),1)
              ENDIF
              INLM=INLM+NGH(IS)
            ELSE
              DO IV=1,NGH(IS)
                INLM=INLM+1
                BILN(1,INLM,I)=BILN(1,INLM,I)*WSG(IS,IV)
              ENDDO
              IF(TKPNT) THEN
                INLM=INLM-NGH(IS)
                DO IV=1,NGH(IS)
                  INLM=INLM+1
                  BILN(2,INLM,I)=BILN(2,INLM,I)*WSG(IS,IV)
                ENDDO
              ENDIF
            ENDIF
          ENDDO
        ENDDO
        DO J=1,NLM
          DO IM=1,IMAGP
            BILN(IM,J,I)=BETAP*(BILN(IM,J,I)-AFNL(IM,J,I))
          ENDDO
        ENDDO
      ENDDO
      CALL TIHALT(' CALC_BILN',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END 
C     ==================================================================
      SUBROUTINE SET_B2L()
C     ==--------------------------------------------------------------==
      IMPLICIT  NONE
      INCLUDE 'system.h'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
C     Variables
C     ==--------------------------------------------------------------==
      IF (NTABTROT.LE.1) THEN 
         RETURN
      ENDIF
      ITROT=1
      B2LIMIT=B2TROT(ITROT)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CHANGE_B2L(DRHOMAX,TREFINE)
C     ==--------------------------------------------------------------==
      IMPLICIT  NONE
      INCLUDE 'system.h'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
C     Arguments
      REAL*8  DRHOMAX
      LOGICAL TREFINE
C     Variables
      INTEGER I,
     &        IFIRST
      DATA    IFIRST /0/
      SAVE    IFIRST       
C     ==--------------------------------------------------------------==
      IF (NTABTROT.LE.1) THEN
         RETURN
      ENDIF
      IF (IFIRST.EQ.0) THEN
        ITROT=1
        IFIRST=1
      ENDIF
      B2LIMIT=B2TROT(ITROT)
      IF(.NOT.TREFINE) THEN
        ITROT=MIN(ITROT+1,NTABTROT)
      ENDIF
      DO I=ITROT,NTABTROT
        IF(DRHOMAX.LT.DENSTROT(I)) THEN
          B2LIMIT=B2TROT(I)
          ITROT=I
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CHANGE_BETAP(DRHOMAX,TREFINE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
C     Arguments
      REAL*8    DRHOMAX
      LOGICAL   TREFINE
C     Variables
      REAL*8    BETAELMAX
      PARAMETER (BETAELMAX=1.D33)
      INTEGER   I,
     &          IFIRST
      DATA      IFIRST /0/
      SAVE      IFIRST
C     ==--------------------------------------------------------------==
      IF (NTABBETAP.LE.1) RETURN
      IF (IFIRST.EQ.0) THEN
        IBETAP=1
        IFIRST=1
      ENDIF
      BETAP=TABBETAP(IBETAP)
      IF (.NOT.TREFINE) THEN
        IBETAP=MIN(IBETAP+1,NTABBETAP)
      ENDIF
      DO I=IBETAP,NTABBETAP
        IF(DRHOMAX.LT.DENSBETAP(I)) THEN
          BETAP=TABBETAP(I)
          IBETAP=I
        ENDIF
      ENDDO
      IF(BETAEL.LT.BETAELMAX) P = BETAEL/BETAP
C     Exponentiated kinetic energy in G-space.
      CALL EG2
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE EVPSI(NNR1,PSI,EXP_VPOT) 
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER    NNR1 
      COMPLEX*16 PSI(NNR1) 
      REAL*8     EXP_VPOT(NNR1)
C     Variables
      INTEGER IR,II,ISUB 
C     ==--------------------------------------------------------------==
      CALL TISET('     EVPSI',ISUB) 
      II=MOD(NNR1,4)
!$OMP parallel do private(IR)
      DO IR=1,II
         PSI(IR) = PSI(IR)*EXP_VPOT(IR)
      ENDDO
      DO IR=1+II,NNR1,4
         PSI(IR) = PSI(IR)*EXP_VPOT(IR)
         PSI(IR+1) = PSI(IR+1)*EXP_VPOT(IR+1)
         PSI(IR+2) = PSI(IR+2)*EXP_VPOT(IR+2)
         PSI(IR+3) = PSI(IR+3)*EXP_VPOT(IR+3)
      ENDDO
      CALL TIHALT('     EVPSI',ISUB) 
C     ==--------------------------------------------------------------==
      RETURN 
      END 
C     ==================================================================
      SUBROUTINE RCALC_BILN(NXX)
C     ==--------------------------------------------------------------==
C     == COMPUTES BILN FOR REAL SPACE CODE WITH KPOINTS               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'pslo.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'sgpp.inc'
C     Arguments
      INTEGER    NXX
C     Variables
      REAL*8     BFNL(NHX)
      REAL*8     FNL(*),DFNL(*)
      INTEGER    ISUB,I,J,INLM,IS,IA,IV,KI,L,LI,JV,L2,LJ,KJ
C     ==--------------------------------------------------------------==
      CALL TISET('RCALC_BILN',ISUB)
      DO I=1,NXX
        INLM=0
        DO IS=1,NSP
          DO IA=1,NA(IS)
            IF(TSGP(IS)) THEN
              CALL AZZERO(BFNL,NGH(IS))
              DO IV=1,NGH(IS)
                L=NGHTOL(IV,IS)+1
                KI=LFVAL(IV,IS)
                LI=LPVAL(IV,IS)
                DO JV=1,NGH(IS)
                  L2=NGHTOL(JV,IS)+1
                  LJ=LPVAL(JV,IS)
                  IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                     KJ=LFVAL(JV,IS)
                     BFNL(IV)=BFNL(IV)+
     &                    HLSG(KI,KJ,L,IS)*AFNL(1,INLM+JV,I)
                  ENDIF
                ENDDO
              ENDDO
              CALL DCOPY(NGH(IS),BFNL(1),1,AFNL(1,INLM+1,I),1)
              INLM=INLM+NGH(IS)
            ELSE
              DO IV=1,NGH(IS)
                INLM=INLM+1
                AFNL(1,INLM,I)=AFNL(1,INLM,I)*WSG(IS,IV)
              ENDDO
            ENDIF
          ENDDO
        ENDDO
      ENDDO
      CALL DGEMM('N','N',NLM,NXX,NLM,0.5D0*BETAP,ALM,
     &     NLM,AFNL,NLM,0.D0 ,BILN,NLM)
      DO I=1,NXX
        INLM=0
        DO IS=1,NSP
          DO IA=1,NA(IS)
            IF(TSGP(IS)) THEN
              CALL AZZERO(BFNL,NGH(IS))
              DO IV=1,NGH(IS)
                L=NGHTOL(IV,IS)+1
                KI=LFVAL(IV,IS)
                LI=LPVAL(IV,IS)
                DO JV=1,NGH(IS)
                  L2=NGHTOL(JV,IS)+1
                  LJ=LPVAL(JV,IS)
                  IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                     KJ=LFVAL(JV,IS)
                     BFNL(IV)=BFNL(IV)+
     &                    HLSG(KI,KJ,L,IS)*BILN(1,INLM+JV,I)
                  ENDIF
                ENDDO
              ENDDO
              CALL DCOPY(NGH(IS),BFNL(1),1,BILN(1,INLM+1,I),1)
              INLM=INLM+NGH(IS)
            ELSE
              DO IV=1,NGH(IS)
                INLM=INLM+1
                BILN(1,INLM,I)=BILN(1,INLM,I)*WSG(IS,IV)
              ENDDO
            ENDIF
          ENDDO
        ENDDO
        DO J=1,NLM
          BILN(1,J,I)=BETAP*(BILN(1,J,I)-AFNL(1,J,I))
        ENDDO
      ENDDO
      CALL TIHALT('RCALC_BILN',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END 
C     ==================================================================
