c     ==================================================================
      subroutine eigensystem_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &     eirop,eivps,z11,nstate)
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'fft.inc'
      include 'coor.inc'
      include 'adat.inc'
      include 'ions.inc'
      include 'nlps.inc'
      include 'cppt.inc' 
      include 'geq0.inc' 
      include 'response_p.inc'
c     ==--------------------------------------------------------------==
c     Arguments
      integer    nstate,lscr
      complex*16 c0(ngw,*),c1(ngw,*)
      complex*16 psi(*)!,hnm1(*)
      complex*16 eirop(*),eivps(*)
      real*8     drhoe(*),z11(nstate,*)
      real*8     scr(*),rhoe(*)
c     ==--------------------------------------------------------------==
c     Local variables
      complex*16 response_vectors(nhg,*)
      pointer    (ip_response_vectors, response_vectors)
      real*8     eigenvect(lanczos_dim,*)
      pointer    (ip_eigenvect, eigenvect)
c     Here, we store the eigensystem of the tridiagonal
c     Lanczos matrix. Thus the dimension is lanczos_dim x lanczos_dim.
      real*8     alpha(*),beta(*)
      pointer    (ip_alpha,alpha), (ip_beta, beta)
      real*8     diagonals(*),subdiagonals(*)
      pointer    (ip_diagonals,diagonals), 
     &     (ip_subdiagonals, subdiagonals)
      complex*16 tmp(*)
      pointer    (ip_tmp, tmp)
      real*8     norm
c     CPMDP/BLAS/LAPACK functions
      real*8     dotp
      external   dotp
c     Auxiliary variables
      integer    i,ir,iter,info,ie,
     &     ia,istate,lastvalid,lastconverged
      real*8     dummy, work(*)
      pointer    (ip_work, work)

      character*2 zahlenchars(99)
      data zahlenchars /'01','02','03','04','05','06','07','08','09',
     &     '10','11','12','13','14','15','16','17','18','19',
     &     '20','21','22','23','24','25','26','27','28','29',
     &     '30','31','32','33','34','35','36','37','38','39',
     &     '40','41','42','43','44','45','46','47','48','49',
     &     '50','51','52','53','54','55','56','57','58','59',
     &     '60','61','62','63','64','65','66','67','68','69',
     &     '70','71','72','73','74','75','76','77','78','79',
     &     '80','81','82','83','84','85','86','87','88','89',
     &     '90','91','92','93','94','95','96','97','98','99' /
      character*80 filename
      character*30 converged
c     ==--------------------------------------------------------------==
      if (parent) then
        print *,' *** calculating EIGENSYSTEM of the ',
     &      'perturbation via Lanczos'
        print *,' *** WITH COMPLETE REORTHOGONALIZATION.'
        print *,'     info: beta calculated after ENTIRE orthogon'
        print *,' *** Maximum dim of Lanczos space:  ',lanczos_dim
        print *,' *** Number of desired eigenstates: ',
     &      NUM_LANCZOS_STATES
        print *,' *** Threashold for convergence:    ',CONV_THRESHOLD
        print *,' *** The response operator is shifted by:',
     &      eigenvalue_shift
        print *,' *** INVERTING the response sign.'
        print *,' *** ELIMINATING any constant response charge.'
      endif


      if ( NUM_LANCZOS_STATES+20 .gt. LANCZOS_DIM)
     &   call stopgm('EIGENSYS','lanczos dim too small.')
      if ( NUM_LANCZOS_STATES    .le. 2)
     &   call stopgm('EIGENSYS','too few response states!')
      if ( CONV_THRESHOLD        .lt. 1d-8)
     &   call stopgm('EIGENSYS','conv threshold too low ( <1e-8 ).')
      if ( CONV_THRESHOLD        .gt. 1d-2)
     &   call stopgm('EIGENSYS','conv threshold too high ( >.01 ).')


      call memory(ip_response_vectors, 2*lanczos_dim*nhg, 'response')
      call memory(ip_eigenvect, lanczos_dim*lanczos_dim, 'eigenvect')
      call memory(ip_work, lanczos_dim*lanczos_dim, 'work')
      call memory(ip_alpha,         2*lanczos_dim, 'lanczos_a')
      call memory(ip_beta,          2*lanczos_dim, 'lanczos_b')
      call memory(ip_diagonals,     2*lanczos_dim, 'lanczos_a2')
      call memory(ip_subdiagonals,  2*lanczos_dim, 'lanczos_b2')
      call memory(ip_tmp,           2*nhg, 'Lanczos tmp')


c     ==--------------------------------------------------------------==
c     Initialize potential
c     is specific FOR A PARTICULAR PHYSICAL SYSTEM (not specific for a
c     particular perturbation!), the result of the diagonalization of
c     chi must always be the same (!)

      call rhoofr(C0,RHOE,PSI,SCR,LSCR,NSTATE)
c     transform rhoe into g space
      do ir=1,nnr1
         psi(ir)=dcmplx(rhoe(ir),0.d0)
      enddo
      call fwfft(psi)
c     Save the density response vector:
      call ZGTHR(nhg,psi,response_vectors(1,1),nzh)


      norm = dotp(nhg,response_vectors(1,1),
     &     response_vectors(1,1))
#ifdef PARALLEL
      call GLOSUM(1,norm)
#endif
      norm = dsqrt(norm)
      call DSCAL(2*nhg, 1d0/norm, response_vectors,1)

c     All other arrays are expected to be correctly initialized by the calling
c     (and allocating) routine, do_perturbation_p().

      beta(1) = 0d0             ! for security (not really needed).





c     ==--------------------------------------------------------------==
c     Do the Lanczos steps.
      iter = 1
      lastconverged = 1
      do while ( (iter .lt. lanczos_dim)
     &    .and.  (iter - num_lanczos_states .lt. lastconverged))
        
        i = 1                   ! dummy.
        call rwfopt_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &      eirop,eivps,response_vectors(1,iter),dummy,
     &      z11,nstate,dummy)


c     calculate the linear response of the density 
c     \   drhoe(r)   =   <c0|r><r|c1> + cc
        call rhoofr_p(c0,c1,drhoe,psi,scr,lscr,nstate)

c     transform drhoe into g space
        do ir=1,nnr1
          psi(ir)=dcmplx(drhoe(ir),0.d0)
        enddo
        call fwfft(psi)
c     Save the density response vector:
        call ZGTHR(nhg,psi,response_vectors(1,iter+1),nzh)
c     The response is defined by (-1) times what we have calculated so far:
        call DSCAL(2*nhg,-1.0d0,response_vectors(1,iter+1),1)
        if (parent)
     &      print *,' *+* response INVERTED.  ',
     &      'Resp(i+1)   =   - Potential[ Resp(i) ]'
c     the new response vector now contains the (packed) response
c     of the system.


c     this is to work around the Lanczos problem that eigenvalues around
c     zero are difficult to determine. After each run of rwfopt, we 
c     ADD -0.xx times the INPUT potential to the output. Then from all the
c     eigenvalues we have to subtract this -0.xx.
        call DAXPY(2*nhg, eigenvalue_shift, 
     &      response_vectors(1,iter), 1,
     &      response_vectors(1,iter+1), 1)

        norm = dotp(nhg, response_vectors(1,iter+1),
     &      response_vectors(1,iter+1))
#ifdef PARALLEL
        call glosum(1,norm)
#endif
        norm = dsqrt(norm)
        if (parent)
     &      print *,' *+* L2 norm[n[i+1]]       = ',norm

c     ==--------------------------------------------------------------==
c     Proceed with the Lanczos algorithm:
c     A v1 (the answer) is in response(:,iter+1) == v2:
c     v2   = A v1
c     ==--------------------------------------------------------------==
        call DCOPY(2*nhg, response_vectors(1,iter+1),1,tmp,1)
c     tmp  = v2

        alpha(iter) = DOTP(nhg, response_vectors(1,iter), tmp)
#ifdef PARALLEL
        call glosum(1,alpha(iter))
#endif
c     a1   = v1 A v1
c     \    = v1 tmp
c     \    This is the projection of the input vector (iter)
c     \    onto the reponse vector (iter+1). -> Orthogonalization.
        
        call DAXPY(2*nhg, - alpha(iter), response_vectors(1,iter), 1,
     &      response_vectors(1,iter+1), 1)
c     v2   = v2  -  a1 v1


        if (.not. (iter .eq. 1))
     &      call DAXPY(2*nhg, - beta(iter),
     &      response_vectors(1,iter-1), 1,
     &      response_vectors(1,iter+1), 1)
c     v2   = v2  -  b1 v0





c     *** Here, we do a complete orthogonalization of the new
c     \   vector (tmp=A v_i) wrt the old ones (v_j, j=1..i-1).
c     Method: Grahm-Schmitt "by hand".
        if (iter .ge. 2) then
          do i=1,iter
            norm = - DOTP(nhg, response_vectors(1,iter+1), 
     &          response_vectors(1,i))
#ifdef PARALLEL
            call glosum(1,norm)
#endif
            call DAXPY(2*nhg, norm, response_vectors(1,i),1,
     &          response_vectors(1,iter+1), 1)
          enddo
        endif



        norm = (DOTP(nhg,response_vectors(1,iter+1),
     &      response_vectors(1,iter+1)))
#ifdef PARALLEL
        call glosum(1,norm)
#endif
        norm = dsqrt(norm)
        beta(iter+1) = norm

        call DSCAL(2*nhg, 1d0/norm,
     &      response_vectors(1,iter+1), 1)
c     v2 is normalized.

        if (parent) then
          print *,' *=*   overlap: alpha[',
     &        iter,'] = ',alpha(iter)
          print *,' *=*   off-diag: beta[',
     &        iter+1,'] = ',beta(iter+1)
          print *,' *=*   norm:               = ',norm
        endif
c     ==--------------------------------------------------------------==
        print *,'##############################',
     &      '##################################'
        call DCOPY(iter, alpha, 1, diagonals, 1)
        call DCOPY(iter-1, beta(2), 1, subdiagonals, 1)
c     Now, the diagonal [alpha(*)] and the subdiagonal [beta(*)]
c     elements of the (tridiagonal) Lanczos matrix 
c     are calculated. This matrix is now diagonalized:
        call DSTEV('V', iter, diagonals, subdiagonals,
     &      eigenvect, lanczos_dim, work, info)
        if (info .ne. 0) print *,'DSTEV exit status: ', info
c     output of dstev:
c     \  alpha = eigenvalues,
c     \  eigenvect = corresponding eigenvectors.

        if (parent)
     &      print *,' *** SPECTRUM, run',iter,':'
        lastconverged = iter
        do i=iter,1,-1
          if ( abs(eigenvect(iter,i)) .lt. conv_threshold) then
            converged = '       (converged: '
            if ( lastconverged .eq. i+1)
     &          lastconverged = i
          else
            converged = '   (NOT converged: '
          endif
          call XSTRING(converged,ia,ie)
          if (parent) 
     &        print '(A,I2,A,F12.7,A,F8.6,A)',
     &        ' *** eigenvalue ',i,' = ',
     &        diagonals(i)-eigenvalue_shift,
     &        converged(ia:ie),abs(eigenvect(iter,i)),').'
        enddo
        if (parent) then
          print *,'Last cnverged state:',lastconverged
          print *,' *** Eigenvalue for Lanczos ghost states is ',
     &        -eigenvalue_shift
          print *,'##############################',
     &        '##################################'
        endif
        lastvalid = iter
        iter = iter + 1
      enddo
c     ==--------------------------------------------------------------==
c     == END / Lanczos runs.
c     ==--------------------------------------------------------------==

c     ==--------------------------------------------------------------==
c     ==--------------------------------------------------------------==
      do istate=lastvalid,1, -1
c     build the eigenvector of the linear answer machinery:
        call DGEMV('N', 2*nhg, lastvalid, 1d0, response_vectors, 2*nhg,
     &      eigenvect(1,istate),1, 0d0, tmp, 1)


        if (parent) then
          do i=1,lastvalid
            print '(A,I2,A,I2,A,F15.11)',
     &          '  \*  Lanczos(',i,'; ',istate,') = ',
     &          eigenvect(i,istate)
          enddo
        endif

c     write the output file...
        filename='eigst-'//zahlenchars(istate)
        call XSTRING(filename,ia,ie)
        print *,' _i_ Writing eigenstate ',istate,
     &      ' to file ',filename(ia:ie),'.'
        call  densto(tmp,tau0,filename)
      enddo
c     ==--------------------------------------------------------------==
c     == END / Output of eigensolutions.
c     ==--------------------------------------------------------------==


      call freem(ip_tmp)
      call freem(ip_response_vectors)
      call freem(ip_eigenvect)
      call freem(ip_work)
      call freem(ip_alpha)
      call freem(ip_beta)
      call freem(ip_diagonals)
      call freem(ip_subdiagonals)

      print *,'memory de-allocated.'

      return
      end
c     ==--------------------------------------------------------------==






