C     ==================================================================
      SUBROUTINE ELF(C0,TAU0,RHOE,V,SCR,LSCR,NSTATE,NKPOINT)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE NORMALIZED ELECTRON DENSITY RHOE IN REAL SPACE          ==
C     ==     AND WRITES IT TO THE FILE rho.movie                      ==
C     ==  THE ELECTRON LOCALIZATION FUNCTION:                         ==
C     ==     ELF=1/(1+( (D+ELFEPS) /D_h)^2)                           ==
C     ==     D  =(1/2) sum_i |nabla psi_i|^2 -                        ==
C     ==         (1/8) {|nabla rho|^2} / rho                          ==
C     ==     D_h=(3/10) (3 pi^2)^2/3 rho^5/3                          ==
C     ==     ELFEPS=2.87D-5 (-> set ELF=0.5 for D=0 and the van der   ==
C     ==     Waals density rho=0.001 to avoid artifacts for small     ==
C     ==     densities rho)                                           ==
C     ==     (ELF is set to zero for ELF<ELFCUT)                      ==
C     ==                                                              ==
C     ==     psi_i is the KS-spin(!)orbital, rho is the total         ==
C     ==     density, and i=1, ..., number of electrons               ==
C     ==     (this definition also applies to spin polarized systems  ==
C     ==     and yields a kind of averaged ELF value)                 ==
C     ==     see                                                      ==
C     ==     A. D. Becke and K. E. Edgecombe, JCP 92 (1990) 5397      ==
C     ==     B. Silvi and A. Savin, Nature 371 (1994) 683             ==
C     ==     (note that the exponent of 3pi^2 is wrong in this paper) ==
C     ==     A. Savin, private communication (for epsilon)            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'poin.inc'
      INCLUDE 'qspl.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'prden.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'pimd.inc'
      INCLUDE 'bsym.inc'
      INCLUDE 'mw.inc'
C     Arguments
      INTEGER    LSCR,NSTATE,NKPOINT
      COMPLEX*16 C0(NGWK,NSTATE,NKPOINT),V(MAXFFT)
      REAL*8     TAU0(*),RHOE(NNR1),SCR(LSCR)
C     Variables
      DIMENSION  F(NSTATE,NKPTS)
      COMPLEX*16 VTEMP(NHG)
      REAL*8     ELFF(NNR1)
      POINTER    (IP_ELFF,ELFF),(IP_VTEMP,VTEMP)
      CHARACTER  CFLBOD*12,CIPNUM*12
      CHARACTER  FILEN*15,TAG*30
      REAL*8     COEFE,RHOM,COEFS,AELF,ELFMIN,ELFMAX
      INTEGER    IKIND,LELF,IR,IG,I,K,N1,N2,I1,I2,SPACE
C     ==--------------------------------------------------------------==
      IF(PARENT) WRITE(6,'(/,A)') ' CALCULATE ELF FUNCTION '
      CALL GIVE_SCR_ELF(LELF,TAG,NSTATE)
      CALL TEST_SCR('ELF',TAG,LSCR,LELF)
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_ELFF,NNR1,'ELFF')
      CALL GET_ADDR(IP_VTEMP,SCR(1))
      CALL PHFAC(TAU0)
      DO IKIND=1,NKPOINT
        CALL RNLSM(C0(1,1,IKIND),NSTATE,V,SCR,LSCR,1,IKIND,.FALSE.)
      ENDDO
      IF(TKPNT) THEN
        CALL RHOOFR_C(C0,RHOE,V,SCR,LSCR,NSTATE)
      ELSE
        CALL RHOOFR  (C0,RHOE,V,SCR,LSCR,NSTATE)
      ENDIF
C..VTEMP=density in G-space
#if defined(__VECTOR)
C$OMP parallel do private(IR)
#else
C$OMP parallel do private(IR) schedule(static)
#endif
      DO IR=1,NNR1
        V(IR)=DCMPLX(RHOE(IR),0.D0)
      ENDDO
      CALL FWFFT(V)
#if defined(__VECTOR)
C$OMP parallel do private(IG)
#else
C$OMP parallel do private(IG) schedule(static)
#endif
      DO IG=1,NHG
        VTEMP(IG) = V(NZH(IG))
      ENDDO
C..calculate |nabla*rho|^2
      CALL ZAZZERO(V,MAXFFT)
!CDIR NODEP
      DO IG=1,NHG
        V(NZH(IG))=TPIBA*UIMAG*GK(1,IG)*VTEMP(IG)
        V(INDZ(IG))=-TPIBA*UIMAG*GK(1,IG)*DCONJG(VTEMP(IG))
      ENDDO
      CALL INVFFT(V)
#if defined(__VECTOR)
C$OMP parallel do private(IR)
#else
C$OMP parallel do private(IR) schedule(static)
#endif
      DO IR=1,NNR1
        ELFF(IR)=DREAL(V(IR))*DREAL(V(IR))
      ENDDO
      CALL ZAZZERO(V,MAXFFT)
!CDIR NODEP
      DO IG=1,NHG
        V(NZH(IG))=TPIBA*(UIMAG*GK(2,IG)-GK(3,IG))*VTEMP(IG)
        V(INDZ(IG))=TPIBA*(-UIMAG*GK(2,IG)+GK(3,IG))*DCONJG(VTEMP(IG))
      ENDDO
      CALL INVFFT(V)
#if defined(__VECTOR)
C$OMP parallel do private(IR)
#else
C$OMP parallel do private(IR) schedule(static)
#endif
      DO IR=1,NNR1
        ELFF(IR)=ELFF(IR)+DREAL(V(IR)*DCONJG(V(IR)))
      ENDDO
C..calculate -{|nabla rho|^2 / 8*rho} / D_h
      COEFE=PI**2
      COEFE=(3.D0*COEFE)**(2.D0/3.D0)
      COEFE=0.3D0*COEFE
      DO IR=1,NNR1
        RHOM=DMAX1(RHOE(IR),0.D0)
        IF(RHOM.GT.ELFCUT) THEN
          ELFF(IR)=-ELFF(IR)/(8.D0*COEFE*RHOE(IR)**2.6666666667D0)
        ELSE
          ELFF(IR)=0.D0
        ENDIF
      ENDDO
C..calculate 0.5*SUM_i{|nabla*phi_i|^2 / D_h }
      DO IKIND=1,NKPOINT
        DO I=1,NSTATE
          COEFS=0.5D0*F(I,IKIND)/OMEGA
          DO K=1,3
            CALL ZAZZERO(V,MAXFFT)
            IF(TKPNT) THEN
              call stopgm('elf','not programmed for k-points')
!CDIR NODEP
              DO IG=1,NGW
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C here we have to add GK for the K-POINTS
C does this also need symmetrization?????
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                V(NZHS(IG))=TPIBA*UIMAG*GK(K,IG)*C0(IG,I,IKIND)
                V(INDZS(IG))=DCONJG(TPIBA*UIMAG*GK(K,IG)*C0(IG,I,IKIND))
              ENDDO
            ELSE
!CDIR NODEP
              DO IG=1,NGW
                V(NZHS(IG))=TPIBA*UIMAG*GK(K,IG)*C0(IG,I,IKIND)
                V(INDZS(IG))=DCONJG(TPIBA*UIMAG*GK(K,IG)*C0(IG,I,IKIND))
              ENDDO
            ENDIF
            CALL S_INVFFT(V)
C..here to add Vanderbilt part of nabla*psi
            IF(TIVAN) THEN
              CALL STOPGM('ELF','VANDERBILT NOT IMPLEMENTED')
            ENDIF
            DO IR=1,NNR1
              RHOM=DMAX1(RHOE(IR),0.D0)
              IF(RHOM.GT.ELFCUT) THEN
                ELFF(IR)=ELFF(IR)+COEFS*DREAL(V(IR))**2/
     *                        (COEFE*RHOE(IR)**1.6666666667D0)
              ENDIF
            ENDDO
          ENDDO
        ENDDO
      ENDDO
C..calculate 1/(1+(D/D_h)^2)
      DO IR=1,NNR1
        RHOM=DMAX1(RHOE(IR),0.D0)
        IF(RHOM.GT.ELFCUT) THEN
          AELF=ELFF(IR)+ELFEPS/(COEFE*RHOE(IR)**1.6666666667D0)
          ELFF(IR)=1.D0/(1.D0+AELF**2)
        ENDIF
      ENDDO
      FILEN='ELF'
C..VTEMP=ELF in G-space
#if defined(__VECTOR)
C$OMP parallel do private(IR)
#else
C$OMP parallel do private(IR) schedule(static)
#endif
      DO IR=1,NNR1
        V(IR)=DCMPLX(ELFF(IR),0.D0)
      ENDDO
      CALL FWFFT(V)
#if defined(__VECTOR)
C$OMP parallel do private(IG)
#else
C$OMP parallel do private(IG) schedule(static)
#endif
      DO IG=1,NHG
        VTEMP(IG) = V(NZH(IG))
      ENDDO
      IF(TPATH) THEN
        IF(IPCURR.EQ.0) THEN
          FILEN='ELF'
        ELSE
          CFLBOD='ELF_'
          WRITE(CIPNUM,'(I4)') IPCURR
          CALL XSTRING(CFLBOD,N1,N2)
          CALL XSTRING(CIPNUM,I1,I2)
          FILEN=CFLBOD(N1:N2)//CIPNUM(I1:I2)
        ENDIF
      ELSEIF (TMW) THEN
        CFLBOD='ELF_'
        CALL MW_FILENAME(CFLBOD,FILEN,WALKER_ID)
      ELSE
        FILEN='ELF'
      ENDIF
C     ==--------------------------------------------------------------==
C CB: Filenames for the BS states
      if(BSYMM) then
        SPACE=index(FILEN,' ')
        if(BSCLCS.eq.1) then
          if(SPACE.GT.10) then
            FILEN='DENSITY.bs'
          else        
            FILEN(SPACE:SPACE+2)='.bs'
          endif  
        else
          if(SPACE.GT.10) then
            FILEN='DENSITY.hs'
          else        
            FILEN(SPACE:SPACE+2)='.hs'
          endif  
        endif  
      endif
C     ==--------------------------------------------------------------==
      CALL DENSTO(VTEMP,TAU0,FILEN)
      ELFMIN=1.D30
      ELFMAX=-1.D30
      DO IR=1,NNR1
        ELFMAX=DMAX1(ELFMAX,ELFF(IR))
        ELFMIN=DMIN1(ELFMIN,ELFF(IR))
      ENDDO
      CALL GLOMAX(1,ELFMAX)
      CALL GLOMIN(1,ELFMIN)
      IF(PARENT) THEN
        WRITE(6,'(A,T50,F15.8)')  ' ELF: MINIMUM OF ELF   = ',ELFMIN
        WRITE(6,'(A,T52,F13.8)')  ' ELF: MAXIMUM OF ELF   = ',ELFMAX
      ENDIF
      CALL FREEM(IP_ELFF)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_ELF(LELF,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'        !NHG
C     Arguments
      INTEGER   LELF,NSTATE
      CHARACTER TAG*30
C     Variables
      INTEGER   LRNLSM,LRHOOFR
C     ==--------------------------------------------------------------==
C     VTEMP(2*NHG)
      LELF=2*NHG
      CALL GIVE_SCR_RNLSM(LRNLSM,TAG,NSTATE,.FALSE.)
      CALL GIVE_SCR_RHOOFR(LRHOOFR,TAG)
      LELF=MAX(LELF,LRNLSM,LRHOOFR)
      TAG='MAX(LELF,LRNLSM,LRHOOFR)'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
