C     ==================================================================
      SUBROUTINE EMPFOR(TAU0,HESS)
C     ==--------------------------------------------------------------==
C     ==  INTERFACE TO THE "DISCO" EMPIRICAL FORCE FIELD CODE         ==
C     ==  THE DISCO CODE IS A GIFT OF THOMAS FISCHER (IPS ETHZ)       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'adat.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'irat.inc'
C     Arguments
      REAL*8  HESS(*),TAU0(3,NAX,NSP)
C     Variables
      INTEGER NBONDS(NAT)
      POINTER (IP_NBONDS,NBONDS)
      REAL*8  RCOV(0:104),
     &        XMFAC
      INTEGER NMPA1,K,I,J,NCC
C     ==--------------------------------------------------------------==
      NMPA1=(NAT*(NAT+1))/2
      CALL MEMORY(IP_ZAN,NAT,'ZAN')
      CALL MEMORY(IP_C,3*NAT,'C')
      CALL MEMORY(IP_IBIND,NMPA1/IRAT+1,'IBIND')
      CALL MEMORY(IP_NBONDS,NAT/IRAT+1,'NBONDS')
      CALL IAZZERO(IBIND,NMPA1)
      CALL IAZZERO(NBONDS,NAT)
      NATY=NAT
      K=0
      DO I=1,NSP
        DO J=1,NA(I)
          K=K+1
          ZAN(K)=DBLE(IATYP(I))
          C(1,K)=TAU0(1,J,I)
          C(2,K)=TAU0(2,J,I)
          C(3,K)=TAU0(3,J,I)
        ENDDO
      ENDDO
      DO I=1,99
        RCOV(I)=COVRAD(I)*FBOHR
      ENDDO
      NCC=3*NAT
      WRITE(6,'(A)') ' INITIALIZE EMPIRICAL HESSIAN '
C     1.35 : SCHLEGEL multiplication factor
      XMFAC=1.35D0
      CALL CPMDBONDS(RCOV,XMFAC,NBONDS)
      CALL FSTART(HESS,RCOV,NBONDS,NCC,.FALSE.)
C     ==--------------------------------------------------------------==
      CALL FREEM(IP_ZAN)
      CALL FREEM(IP_C)
      CALL FREEM(IP_IBIND)
      CALL FREEM(IP_NBONDS)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CPMDBONDS(RCOV,XMFAC,NBONDS)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
      INCLUDE 'cotr.inc'
C     Arguments
      REAL*8  RCOV(0:104),XMFAC
      INTEGER NBONDS(NATY)
C     Variables
      INTEGER IMOL(NATY)
      POINTER (IP_IMOL,IMOL)
      REAL*8  DIJ,CDIJ,DMIN,XCDIJ,
     &        DIST
      INTEGER I,J,II,K,L,IA,IB,I1,I2,K1,K2,J1,J2,IAB,NUMMOL
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_IMOL,NATY,'IMOL')
      CALL IAZZERO(IMOL,NATY)
C     Normal bonds
      DO I=2,NATY
        DO J=1,I-1
          II=I*(I-1)/2+J
          DIJ=DSQRT(DIST(I,J))
          CDIJ=XMFAC*(RCOV(NINT(ZAN(I)))+RCOV(NINT(ZAN(J))))
          IF(CDIJ.GE.DIJ) THEN
            IBIND(II)=1
          ELSE
            IBIND(II)=0
          ENDIF
        ENDDO
      ENDDO
C     Added or deleted bonds
      IF(NBOAD.GT.0) THEN
        DO K=1,NBOAD
          I=BOAD(1,K)
          J=BOAD(2,K)
          L=BOAD(3,K)
          IF(J.GT.I) THEN
            I=BOAD(2,K)
            J=BOAD(1,K)
          ENDIF
          II=I*(I-1)/2+J
          IF(L.LT.0) IBIND(II)=0
          IF(L.GT.0) IBIND(II)=1
        ENDDO
      ENDIF
C     Hydrogen bonds
      DO I=2,NATY
        DO J=1,I-1
          IA=NINT(ZAN(I))
          IB=NINT(ZAN(J))
          IF((IA.EQ.1.AND.IB.NE.1).OR.(IA.NE.1.AND.IB.EQ.1)) THEN
            II=I*(I-1)/2+J
            DIJ=DSQRT(DIST(I,J))
            CDIJ=XMFAC*(RCOV(NINT(ZAN(I)))+RCOV(NINT(ZAN(J))))
            IF(CDIJ.LT.DIJ) THEN
              XCDIJ=CDIJ/XMFAC*2.40D0
              IF(XCDIJ.GT.DIJ) IBIND(II)=2
            ENDIF
          ENDIF
        ENDDO
      ENDDO
C     Search for molecules
      CALL SMOLEC(IMOL,NUMMOL)
C     Add auxilliary bonds between molecules
      DO I1=2,NUMMOL
        DO I2=1,I1-1
          K1=0
          K2=0
          DMIN=100.0D9
          DO J1=1,NATY
            IF(IMOL(J1).EQ.I1) THEN
              DO J2=1,NATY
                IF(IMOL(J2).EQ.I2) THEN
                  DIJ=SQRT(DIST(J1,J2))
                  IF(DIJ.LT.DMIN) THEN
                    K1=J1
                    K2=J2
                    DMIN=DIJ
                  ENDIF
                ENDIF
              ENDDO
            ENDIF
          ENDDO
          IA=MAX(K1,K2)
          IB=MIN(K1,K2)
          K=IB+IA*(IA-1)/2
          IF(IBIND(K).EQ.0) IBIND(K)=3
        ENDDO
      ENDDO
C     Number of bonds per atom
      CALL IAZZERO(NBONDS,NATY)
      DO IA=2,NATY
        DO IB=1,IA-1
          IAB=IBIND(IB+IA*(IA-1)/2)
          IF(IAB.EQ.1) THEN
            NBONDS(IA)=NBONDS(IA)+1
            NBONDS(IB)=NBONDS(IB)+1
          ENDIF
        ENDDO
      ENDDO
C     Print bonds
      CALL PBOND(IMOL,NUMMOL)
      CALL FREEM(IP_IMOL)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PBOND(IMOL,NUMMOL)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
C     Arguments
      INTEGER IPRT(20),IMOL(NATY)
C     Variables
      INTEGER IAB,IHB,IAU,IA,IB,K,IDUM,I,J,NUMMOL
C     ==--------------------------------------------------------------==
      IAB=0
      IHB=0
      IAU=0
      DO IA=2,NATY
        DO IB=1,IA-1
          K = IB+IA*(IA-1)/2
          IF(IBIND(K).EQ.1) IAB=IAB+1
          IF(IBIND(K).EQ.2) IHB=IHB+1
          IF(IBIND(K).EQ.3) IAU=IAU+1
        END DO
      END DO
      IF(IAB.GT.0) THEN
        WRITE(IW,'(A,20X,A)') '       ','<<<<< ASSUMED BONDS >>>>>'
        IDUM = 0
        DO IA=2,NATY
          DO IB=1,IA-1
            K = IB+IA*(IA-1)/2
            IF(IBIND(K).EQ.1) THEN
              IDUM         = IDUM + 2
              IPRT(IDUM-1) = IA
              IPRT(IDUM)   = IB
              IF(IDUM.EQ.10) THEN
                WRITE(IW,1010) (IPRT(J),J=1,10)
                IDUM = 0
              END IF
            END IF
          END DO
        END DO
        IF(IDUM.LT.10) WRITE(IW,1010) (IPRT(J),J=1,IDUM)
      ENDIF
      IF(IHB.GT.0) THEN
        WRITE(IW,'(A,20X,A)') '       ','<<<<< HYDROGEN BONDS >>>>>'
        IDUM = 0
        DO IA=2,NATY
          DO IB=1,IA-1
            K = IB+IA*(IA-1)/2
            IF(IBIND(K).EQ.2) THEN
              IDUM         = IDUM + 2
              IPRT(IDUM-1) = IA
              IPRT(IDUM)   = IB
              IF(IDUM.EQ.10) THEN
                WRITE(IW,1010) (IPRT(J),J=1,10)
                IDUM = 0
              END IF
            END IF
          END DO
        END DO
        IF(IDUM.LT.10) WRITE(IW,1010) (IPRT(J),J=1,IDUM)
      ENDIF
      IF(IAU.GT.0) THEN
        WRITE(IW,'(A,20X,A)') '      ','<<<<< AUXILIARY BONDS >>>>>'
        IDUM = 0
        DO IA=2,NATY
          DO IB=1,IA-1
            K = IB+IA*(IA-1)/2
            IF(IBIND(K).EQ.3) THEN
              IDUM         = IDUM + 2
              IPRT(IDUM-1) = IA
              IPRT(IDUM)   = IB
              IF(IDUM.EQ.10) THEN
                WRITE(IW,1010) (IPRT(J),J=1,10)
                IDUM = 0
              END IF
            END IF
          END DO
        END DO
        IF(IDUM.LT.10) WRITE(IW,1010) (IPRT(J),J=1,IDUM)
      ENDIF
      WRITE(IW,'(A,I3)') ' TOTAL NUMBER OF MOLECULAR STRUCTURES:',
     *                   NUMMOL
      IF(NUMMOL.GT.1) THEN
        WRITE(IW,'(A,20X,A)') '         ','<<<<< MOLECULES >>>>>'
        DO I=1,NUMMOL
          IDUM=0
          DO IA=1,NATY
            IF(IMOL(IA).EQ.I) THEN
              IDUM=IDUM+1
              IPRT(IDUM)=IA
              IF(IDUM.EQ.15) WRITE(IW,1020) I,(IPRT(J),J=1,IDUM)
              IF(IDUM.EQ.15) IDUM=0
            ENDIF
          ENDDO
          IF(IDUM.LT.15) WRITE(IW,1020) I,(IPRT(J),J=1,IDUM)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
 1010 FORMAT(2X,5(I3,' <-->',I3,2X))
 1020 FORMAT(' MOLECULE:',I3,' <>   ',15I5)
      RETURN
      END
C     ==================================================================
      SUBROUTINE SMOLEC(IMOL,NUMMOL)
C     ==--------------------------------------------------------------==
C     == Calculates the TOTAL NUMBER OF MOLECULAR STRUCTURES (NUMMOL) ==
C     == i.e. check the connection between all atoms in order         ==
C     == to determine "isolated" molecule                             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
C     Arguments
      INTEGER IMOL(NATY),NUMMOL
C     Variables
      INTEGER I,K,J,IPAR
C     ==--------------------------------------------------------------==
      NUMMOL=0
   10 CONTINUE
      DO I=1,NATY
        IF(IMOL(I).EQ.0) THEN
          IPAR=I
          NUMMOL=NUMMOL+1
          IMOL(IPAR)=NUMMOL
          GOTO 20
        ENDIF
      ENDDO
      GOTO 30
   20 CONTINUE
      DO I=1,NATY
        IF(IMOL(I).EQ.NUMMOL) THEN
          DO J=I+1,NATY
            K = I+J*(J-1)/2
            IF(IBIND(K).EQ.1) IMOL(J)=NUMMOL
          ENDDO
          DO J=1,I-1
            K = J+I*(I-1)/2
            IF(IBIND(K).EQ.1) IMOL(J)=NUMMOL
          ENDDO
        ENDIF
      ENDDO
      GOTO 10
   30 CONTINUE
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
