C     ==================================================================
      SUBROUTINE DV0(V0,VT,GRAD,VT2)
C     ==--------------------------------------------------------------==
C     ==   CALCULATION OF |nabla.V0|                                  ==
C     ==   ON INPUT : V0 : V0 IN REAL SPACE                           ==
C     ==              VT   : UNDEFINED                                ==
C     ==              GRAD : UNDEFINED                                ==
C     ==              VT2  : UNDeFINED                                ==
C     ==   ON OUTPUT: V0  : V0 IN REAL SPACE (SMOOTH)                 ==
C     ==              VT   : UNDEFINED                                ==
C     ==              GRAD : GRADIENT OF V0 IN REAL SPACE             ==
C     ==              VT2  : V0 IN G SPACE                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
C     Arguments
      COMPLEX*16 VT(MAXFFT),VT2(NHG)
      REAL*8     V0(NNR1),GRAD(NNR1,3)
C     Variables
      INTEGER    ISUB,IR,IG
C     ==--------------------------------------------------------------==
C     ==  TRANSFORM DENSITY TO G SPACE                                ==
C     ==--------------------------------------------------------------==
      CALL TISET('    DIFFV0',ISUB)
C$OMP parallel do private(IR)
      DO IR=1,NNR1
        VT(IR) = DCMPLX(V0(IR),0.0D0)
      ENDDO
      CALL FWFFT(VT)
      CALL ZGTHR(NHG,VT,VT2,NZH)
C     ==--------------------------------------------------------------==
C     ==  FFT OF RHO AND NABLA(X)*RHOE                                ==
C     ==--------------------------------------------------------------==
#ifdef _BGL
      CALL AZZERO(VT,2*MAXFFT)
#else
      CALL ZAZZERO(VT,MAXFFT)
#endif
C$OMP parallel do private(IG)
#ifdef __NEC
!CDIR NODEP
#endif
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC(VT)
#endif
#ifdef CRAYX1
!dir$ concurrent
#endif
#ifdef _BGL
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT
#endif
      DO IG=1,NHG
        VT(NZH(IG))=VT2(IG)-TPIBA*GK(1,IG)*VT2(IG)
        VT(INDZ(IG))=DCONJG(VT2(IG)+TPIBA*GK(1,IG)*VT2(IG))
      ENDDO
      CALL INVFFT(VT)
C$OMP parallel do private(IR)
#ifdef _BGL
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT
#endif
      DO IR=1,NNR1
        V0(IR)=DREAL(VT(IR))
        GRAD(IR,1)=DIMAG(VT(IR))
      ENDDO
C     ==--------------------------------------------------------------==
C     ==  FFT OF NABLA(Y)*RHO AND NABLA(Z)*RHOE                       ==
C     ==--------------------------------------------------------------==
#ifdef _BGL
      CALL AZZERO(VT,2*MAXFFT)
#else
      CALL ZAZZERO(VT,MAXFFT)
#endif
#ifdef __ES  
!CDIR NODEP(VT)
      DO IG=1,NHG
        VT(NZH(IG))=TPIBA*(UIMAG*GK(2,IG)-GK(3,IG))*VT2(IG)
        VT(INDZ(IG))=TPIBA*(-UIMAG*GK(2,IG)+GK(3,IG))*DCONJG(VT2(IG))
      ENDDO
#else 
C$OMP parallel do private(IG)
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC(VT)
#endif
#ifdef CRAYX1
!dir$ concurrent
#endif
#ifdef _BGL
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT
#endif
      DO IG=1,NHG
        VT(NZH(IG))=TPIBA*(UIMAG*GK(2,IG)-GK(3,IG))*VT2(IG)
        VT(INDZ(IG))=TPIBA*(-UIMAG*GK(2,IG)+GK(3,IG))*DCONJG(VT2(IG))
      ENDDO
#endif 
      CALL INVFFT(VT)
C$OMP parallel do private(IR)
#ifdef _BGL
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT
#endif
      DO IR=1,NNR1
        GRAD(IR,2)=DREAL(VT(IR))
        GRAD(IR,3)=DIMAG(VT(IR))
      ENDDO
      CALL TIHALT('    DIFFV0',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
