C     ==================================================================
      SUBROUTINE EPR_EFFPOT(effpot,rhoe,vt,vt2)
C     ==================================================================
C     PARALLEL
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'qspl.inc' 
      INCLUDE 'ions.inc'
      INCLUDE 'adat.inc'
C     ==--------------------------------------------------------------==
C     Arguments & common variables
      REAL*8     effpot(nnr1,2), rhoe(nnr1,2)
      COMPLEX*16 vt(MAXFFT), vt2(NHG)
C     ==--------------------------------------------------------------==
C     Local variables
      DIMENSION  GGNH(NSPLPO)
      INTEGER    I1, IG, IG1, IS, GRIDMESH(NSP)
      REAL*8     NUCPOT(NSPLPO,2,*), GRIDR(MMAXX,*),
     &           GRIDF(MMAXX,*),
     &           POTSCR1(*), POTSCR2(*),
     &           TOTCLBPOT(NSP,*), VOL, CURV2,
     &           DUMMY1, DUMMY2, GRIDSTEP(NSP)
      POINTER    (IP_NUCPOT,NUCPOT), (IP_GRIDR,GRIDR), (IP_GRIDF,GRIDF),
     &           (IP_POTSCR1, POTSCR1),
     &           (IP_POTSCR2, POTSCR2),
     &           (IP_TOTCLBPOT, TOTCLBPOT),
     &           (IP_GRIDSTEP, GRIDSTEP)
      COMPLEX*16 VTMP(MAXFFT,*)
      POINTER    (IP_VTMP,VTMP)
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_NUCPOT,2*NSPLPO*NSX,'NUCPOT')
      CALL AZZERO(NUCPOT,2*NSPLPO*NSX)
      CALL MEMORY(IP_GRIDR,MMAXX*NSP,'GRIDR')
      CALL AZZERO(GRIDR,MMAXX*NSP)
      CALL MEMORY(IP_GRIDF,MMAXX*NSP,'GRIDF')
      CALL AZZERO(GRIDF,MMAXX*NSP)
      CALL MEMORY(IP_POTSCR1,MMAXX,'POTSCR1')
      CALL AZZERO(POTSCR1,MMAXX)
      CALL MEMORY(IP_POTSCR2,MMAXX,'POTSCR2')
      CALL AZZERO(POTSCR2,MMAXX)
      CALL MEMORY(IP_TOTCLBPOT,NSP*NHG,'TOTCLBPOT')
      CALL AZZERO(TOTCLBPOT,NSP*NHG)
      CALL MEMORY(IP_VTMP,4*MAXFFT,'VTMP')
      CALL AZZERO(VTMP,4*MAXFFT)
      CALL MEMORY(IP_GRIDSTEP,NSP,'GRIDSTEP')
      CALL AZZERO(GRIDSTEP,NSP)
C     ==--------------------------------------------------------------==
C     Calculation of the Coulomb potential
      DO IS=1,NSP
C       Real grid (radial)
        CALL EPR_POT_RG(IS,GRIDR(1,1),GRIDF(1,1),GRIDMESH(1),
     &                  GRIDSTEP(1))        
      ENDDO
      DO IS=1,NSP
C       Reciprocal grid (radial)
        CALL EPR_POT_GG(IS,NUCPOT(1,1,1),
     &    GRIDR(1,1),GRIDF(1,1),GRIDMESH(1),GRIDSTEP(1),
     &    POTSCR1(1),POTSCR2(1))
      ENDDO
C     Reciprocal grid (normal)
      VOL=1.D0/OMEGA
      DO IS=1,NSP
        DO IG=1,NHG
          TOTCLBPOT(IS,IG)=VOL*CURV2(HG(IG),NSPLPO,GGNH(1),
     &                     NUCPOT(1,1,IS),NUCPOT(1,2,IS),0.0D0)
        ENDDO
      ENDDO
C     Sum over all atoms
      CALL TOTCLBPOTCALC(VT2(1),TOTCLBPOT(1,1))

      IG1=1
C$OMP parallel do private(IG)
      DO IG=IG1,NHG
        VT(NZH(IG))=VT2(IG)
        VT(INDZ(IG))=DCONJG(VT2(IG))
      ENDDO
      
      CALL INVFFT(VT)

      DO I1=1,NNR1
        effpot(I1,1) = DREAL(VT(I1))
      ENDDO
C     ==--------------------------------------------------------------==
C     Calculation of the Hartree potential
      CALL AZZERO(VT, 2*MAXFFT)
      CALL AZZERO(VT2, 2*NHG)

C$OMP parallel do private(I1)
      DO I1=1,NNR1
        VT(I1) = DCMPLX(rhoe(I1,1),0.0D0)
      ENDDO

      CALL FWFFT(VT)

      IG1=1
      IF (GEQ0) THEN
        VT2(1) = DCMPLX(0.0D0,0.0D0)
        IG1=2
      ENDIF

C$OMP parallel do private(IG)
      DO IG=IG1,NHG
        VT2(IG) = VT(NZH(IG)) * DCMPLX(FPI/(TPIBA2*HG(IG)),0.0D0)
      ENDDO

      CALL AZZERO(VT, 2*MAXFFT)
      
C$OMP parallel do private(IG)
      DO IG=1,NHG
        VT(INDZ(IG)) = DCONJG(VT2(IG))
        VT(NZH(IG))  = VT2(IG)
      ENDDO

      CALL INVFFT(VT)

      DO I1=1,NNR1
        effpot(I1,1) = effpot(I1,1) + DREAL(VT(I1))
      ENDDO

      CALL AZZERO(VT, 2*MAXFFT)
      CALL AZZERO(VT2, 2*NHG)

C     ==--------------------------------------------------------------==
C     Calculation of the XC potential
C$OMP parallel do private(I1)
      DO I1=1,NNR1
        effpot(I1,2) = effpot(I1,1)
      ENDDO

      CALL XCENER(DUMMY1,DUMMY2,RHOE,RHOE,VTMP)

C$OMP parallel do private(I1)
      DO I1=1,NNR1
        effpot(I1,1) = effpot(I1,1) + DREAL(VTMP(I1,1))
        effpot(I1,2) = effpot(I1,2) + DREAL(VTMP(I1,2))
      ENDDO
C     ==--------------------------------------------------------------==
      CALL FREEM(IP_NUCPOT)
      CALL FREEM(IP_GRIDR)
      CALL FREEM(IP_GRIDF)
      CALL FREEM(IP_POTSCR1)
      CALL FREEM(IP_POTSCR2)
      CALL FREEM(IP_TOTCLBPOT)
      CALL FREEM(IP_VTMP)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE EPR_POT_GG(IS,NUCPOT,POT_RG_RV,POT_RG_VR,POT_RG_MESHV,
     &                      POT_RG_STEP,DFINT,FINT)
C     ==--------------------------------------------------------------==
C     ==  COMPUTES THE COULOMB PSEUDOPOTENTIAL IN G-SPACE             ==
C     ==  (NOT PARALLEL RIGHT NOW)                                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'atom.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'dpot.inc'
      INCLUDE 'ragg.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'qspl.inc'
      INCLUDE 'gvec.inc'
C     Arguments
      INTEGER   IS, POT_RG_MESHV(NSP)
      REAL*8    NUCPOT(NSPLPO,2,NSP), POT_RG_RV(MMAXX,NSP), 
     &          POT_RG_VR(MMAXX,NSP), POT_RG_STEP(NSP),
     &          FINT(MMAXX), DFINT(MMAXX)
C     Variables
      DIMENSION GGNH(*)
      REAL*8    CHECK,XG,XRG,FLX,R2,RCPF,GG
      INTEGER   IR,IL,ISUB,IERR
      LOGICAL   ZER
#ifdef __DERF
      REAL*8    DERF
      EXTERNAL  DERF
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('EPR_POT_GG',ISUB)
      CALL AZZERO(FINT,MMAXX)
      DO IR=1,POT_RG_MESHV(IS)
        FINT(IR)=(POT_RG_RV(IR,IS)*POT_RG_RV(IR,IS)*POT_RG_RV(IR,IS)
     &          *POT_RG_VR(IR,IS)+
     &          DBLE(IATYP(IS))*DERF(POT_RG_RV(IR,IS)/RAGGIO(IS))
     &          *POT_RG_RV(IR,IS)*POT_RG_RV(IR,IS))
        ZER = DABS(POT_RG_RV(IR,IS)).LT.1.D-8
        IF(.NOT.ZER) THEN
          CHECK = POT_RG_VR(IR,IS)+DBLE(IATYP(IS))*
     *            DERF(POT_RG_RV(IR,IS)/RAGGIO(IS))/POT_RG_RV(IR,IS)
        ELSE
          CHECK = POT_RG_VR(IR,IS)+2.D0*
     *            DBLE(IATYP(IS))/(DSQRT(PI)*RAGGIO(IS))
        ENDIF
        IF(DABS(CHECK).LT.1.D-8) FINT(IR)=0.D0
      ENDDO
      DO IL=1,NSPLPO
        XG=SQRT(GGNH(IL))*TPIBA
        IF(XG.GT.1.D-6) THEN
          DO IR=1,POT_RG_MESHV(IS)
            XRG = POT_RG_RV(IR,IS)*XG
            DFINT (IR) = FINT(IR)* SIN(XRG)/XRG
          ENDDO
        ELSE
           CALL DCOPY(POT_RG_MESHV(IS),FINT(1),1,DFINT(1),1)
        ENDIF
        CALL SIMPSN (POT_RG_MESHV(IS), DFINT, FLX)
        NUCPOT(IL,1,IS) = POT_RG_STEP(IS)*FPI* FLX
      ENDDO
      CALL CURV1(NSPLPO,GGNH,NUCPOT(1,1,IS),0.0d0,0.0d0,3,
     &           NUCPOT(1,2,IS),FINT,0.0d0,IERR)
      CALL TIHALT('EPR_POT_GG',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE EPR_POT_RG(ISP,POT_RG_RV,POT_RG_VR,
     &                      POT_RG_MESHV,POT_RG_STEP)
C     ==--------------------------------------------------------------==
C     == Create one-electron all-electron potentials in real space    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'atom.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'adat.inc'
      INCLUDE 'response_p.inc'
C     Arguments
      INTEGER  ISP, POT_RG_MESHV(NSP)
      REAL*8   POT_RG_RV(MMAXX,NSP),POT_RG_VR(MMAXX,NSP),
     &         POT_RG_STEP(NSP)
C     Variables
      INTEGER  MESHV,IR
      REAL*8   RR1,RX,RR2,STEP
#ifdef __DERF
      REAL*8   DERF
      EXTERNAL DERF
#endif
C     ==--------------------------------------------------------------==
      STEP=LOG(1.025D0)
      MESHV=LOG(7200.0D0*DBLE(IATYP(ISP)))/STEP
      IF(MESHV.GT.MMAXX) THEN
        STEP=DBLE(MESHV)/DBLE(MMAXX)*STEP
        MESHV=MMAXX
      ENDIF
      RR1=.00625D0/DBLE(IATYP(ISP))
      RX=LOG(RR1)+MESHV*STEP
      RR2=EXP(RX)
      CALL EPR_LOG_GRID(RR1,RR2,POT_RG_RV(1,ISP),MESHV,STEP)
      DO IR=1,MESHV
        RX=POT_RG_RV(IR,ISP)
        POT_RG_VR(IR,ISP)=-ZV(ISP)/RX*
     &     DERF(RX/EPR_OWNPOT_VALUE)
C     &    DERF(RX/(1.0D0/5.0D0*COVRAD(IATYP(ISP))*FBOHR))
      ENDDO
      POT_RG_MESHV(ISP)=MESHV
      POT_RG_STEP(ISP)=STEP
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE EPR_LOG_GRID(RP1,RPL,RW,MESH,CLOG)
C     ==--------------------------------------------------------------==
C     == Construct logarithmic grid from RP1 to RPL with MESH points  ==
C     == in RW(1:MESH).                                               ==
C     == CLOG is the step                                             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER MESH
      REAL*8  RP1,RPL,RW(MESH),CLOG
C     Variables
      REAL*8  CF
      INTEGER IR
C     ==--------------------------------------------------------------==
      CLOG=DLOG(RPL/RP1)/DBLE(MESH-1)
      CF=DEXP(CLOG)
      RW(1)=RP1
      DO IR=2,MESH
        RW(IR)=RW(IR-1)*CF
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE TOTCLBPOTCALC(EIVPS,TOTCLBPOT)
C     ==--------------------------------------------------------------==
C     ==  derived from eicalc.F                                       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'sfac.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cppt.inc'
C     Arguments
      COMPLEX*16 EIVPS(NHG)
      REAL*8     TOTCLBPOT(NSP,NHG)
C     Variables
      DIMENSION  EIGRB(NHG,NAT)
      COMPLEX*16 EI123
      REAL*8     FNL(*),DFNL(*)
      REAL*8     ER,EI
      INTEGER    ISUB,IG,ISA,IS,IA
#ifdef __NEC
      INTEGER    ISA0
      INTEGER    ISV(*)
      POINTER    (IP_ISV,ISV)
#endif
#ifdef __CRAYX1
      INTEGER    ISA0
      INTEGER    ISV(*)
      POINTER    (IP_ISV,ISV)
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('TOTCLBCALC',ISUB)
#ifdef __VECTOR
#ifdef __CRAYX1
      CALL ZAZZERO(EIVPS,NHG)
      ISA0=0
      DO IS=1,NSP
        ISA0 = ISA0 + NA(IS)
      ENDDO
      CALL MEMORY(IP_ISV,ISA0,'ISV')
      ISA=0
!dir$ preferstream
!dir$ nointerchange 
      DO IS=1,NSP
!dir$ prefervector
        DO IA=1,NA(IS)
          ISA=ISA+1
          ISV(ISA) = IS
        ENDDO
      ENDDO
      IF(BIGMEM) THEN
!$OMP parallel do private(IG,ISA,IS)
        DO IG=1,NHG
          DO ISA = 1, ISA0
            IS=ISV(ISA)
            EIVPS(IG)=EIVPS(IG)+TOTCLBPOT(IS,IG)*EIGRB(IG,ISA)
          ENDDO
        ENDDO
      ELSE
!$OMP parallel do private(IG,ISA,IS,EI123,ER,EI)
!dir$ preferstream
!dir$ nointerchange 
        DO IG=1,NHG
!dir$ prefervector
          DO ISA = 1, ISA0
            IS=ISV(ISA)
            EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     &            EI3(ISA,INYH(3,IG))
            ER=DREAL(EI123)
            EI=DIMAG(EI123)
            EIVPS(IG)=EIVPS(IG)
     &             +DCMPLX(ER*TOTCLBPOT(IS,IG),EI*TOTCLBPOT(IS,IG))
          ENDDO
        ENDDO
      ENDIF
      CALL FREEM(IP_ISV)
#elif defined __NEC
      CALL ZAZZERO(EIVPS,NHG)
      ISA0=0
      DO IS=1,NSP
        ISA0 = ISA0 + NA(IS)
      ENDDO
      CALL MEMORY(IP_ISV,ISA0,'ISV')
      ISA=0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          ISA=ISA+1
          ISV(ISA) = IS
        ENDDO
      ENDDO
      IF(BIGMEM) THEN
!$OMP parallel do private(IG,ISA,IS)
        DO IG=1,NHG
          DO ISA = 1, ISA0
            IS=ISV(ISA)
            EIVPS(IG)=EIVPS(IG)+TOTCLBPOT(IS,IG)*EIGRB(IG,ISA)
          ENDDO
        ENDDO
      ELSE
!$OMP parallel do private(IG,ISA,IS,EI123,ER,EI)
        DO IG=1,NHG
          DO ISA = 1, ISA0
            IS=ISV(ISA)
            EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     &            EI3(ISA,INYH(3,IG))
            ER=DREAL(EI123)
            EI=DIMAG(EI123)
            EIVPS(IG)=EIVPS(IG)
     &             +DCMPLX(ER*TOTCLBPOT(IS,IG),EI*TOTCLBPOT(IS,IG))
          ENDDO
        ENDDO
      ENDIF
      CALL FREEM(IP_ISV)
#elif defined(_vpp_)
      CALL ZAZZERO(EIVPS,NHG)
      IF(BIGMEM) THEN
        DO IG=1,NHG
          DO ISA=1,NAT
            IS=IATPT(2,ISA)
            EIVPS(IG)=EIVPS(IG)+TOTCLBPOT(IS,IG)*EIGRB(IG,ISA)
          ENDDO
        ENDDO
      ELSE
        DO IG=1,NHG
          DO ISA=1,NAT
            IS=IATPT(2,ISA)
            EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     &            EI3(ISA,INYH(3,IG))
            EIVPS(IG)=EIVPS(IG)+TOTCLBPOT(IS,IG)*EI123
          ENDDO
        ENDDO
      ENDIF
#else
      CALL ZAZZERO(EIVPS,NHG)
      IF(BIGMEM) THEN
#ifdef __SR8000
*poption parallel
*voption indep(RHOPS,EIGRB,TOTCLBPOT,EIVPS)
#endif
        DO IG=1,NHG
          ISA=0
          DO IS=1,NSP
            DO IA=1,NA(IS)
              ISA=ISA+1
              EIVPS(IG)=EIVPS(IG)+TOTCLBPOT(IS,IG)*EIGRB(IG,ISA)
            ENDDO
          ENDDO
        ENDDO
      ELSE
#ifdef __SR8000
*poption parallel
*voption indep(EI3,EI2,EI1,INYH,RHOPS,TOTCLBPOT,EIVPS)
#endif
        DO IG=1,NHG
          ISA=0
          DO IS=1,NSP
            DO IA=1,NA(IS)
              ISA=ISA+1
              EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     &              EI3(ISA,INYH(3,IG))
              ER=DREAL(EI123)
              EI=DIMAG(EI123)
              EIVPS(IG)=EIVPS(IG)
     &            +DCMPLX(ER*TOTCLBPOT(IS,IG),EI*TOTCLBPOT(IS,IG))
            ENDDO
          ENDDO
        ENDDO
      ENDIF
#endif
#else
#ifdef  _BGL 
      CALL AZZERO(EIVPS,2*NHG)
#else
      CALL ZAZZERO(EIVPS,NHG)
#endif
      IF(BIGMEM) THEN
        ISA=0
        DO IS=1,NSP
          DO IA=1,NA(IS)
            ISA=ISA+1
C$OMP parallel do private(IG) shared(ISA,EIVPS)
#ifdef  _BGL 
!IBM*  ASSERT (ITERCNT(10000))
!IBM*  INDEPENDENT
#endif
            DO IG=1,NHG
              EIVPS(IG)=EIVPS(IG)+TOTCLBPOT(IS,IG)*EIGRB(IG,ISA)
            ENDDO
          ENDDO
        ENDDO
      ELSE
        ISA=0
        DO IS=1,NSP
          DO IA=1,NA(IS)
            ISA=ISA+1
C$OMP parallel do private(IG,ER,EI,EI123) shared(ISA,EIVPS)
#ifdef  _BGL 
!IBM*  ASSERT (ITERCNT(10000))
!IBM*  INDEPENDENT
#endif
            DO IG=1,NHG
              EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     &              EI3(ISA,INYH(3,IG))
              ER=DREAL(EI123)
              EI=DIMAG(EI123)
              EIVPS(IG)=EIVPS(IG)
     & +DCMPLX(ER*TOTCLBPOT(IS,IG),EI*TOTCLBPOT(IS,IG))
            ENDDO
          ENDDO
        ENDDO
      ENDIF
#endif
      CALL TIHALT('TOTCLBCALC',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
