C     ============================================================
      subroutine epr_efg (rhoe,scr,lscr,psi,nfi)
C     ============================================================
      implicit none
      include 'system.h'
      include 'prop.inc'
      include 'spin.inc'

      complex*16 scr(nhg,3),rhoe(nhg),psi(nnr1,nlsd)
      integer    lscr,ix,nfi

      if (teprefg) then
         call calc_gradient(rho_save_epr,rhoe,scr,lscr,psi,1,nfi)
      endif
      if (tefg) then 
         call calc_gradient(rho_save_efg,rhoe,scr,lscr,psi,2,nfi)
      endif
      return
      end

C     =======================================================
      subroutine calc_gradient (rhor,rhoe,scr,lscr,psi,i,nfi)
C     =======================================================  
      implicit none
      include 'system.h'
      include 'ions.inc'
      include 'spin.inc'
      
      complex*16 scr(nhg,3),rhoe(nhg),psi(nnr1,nlsd)
      real*8     gradient_matrix(3,3,nax,nsx),eigr_dot,rhor(nnr1)
      real*8     efield(3,nax,nsx)
      real*8     pot(nax,nsx)
      real*8     eigenvalues(3,nax,nsx),work(3*3)
      pointer    (ip_efield,efield)
      pointer    (ip_gradient_matrix,gradient_matrix)
      pointer    (ip_pot,pot)
      pointer    (ip_eigenvalues,eigenvalues)
      integer    isp,iat,isa,ig,dir,dir1,i,lscr,info,nfi

      integer ifirst
      save    ifirst
      data    ifirst /0/
      save    ip_efield
      save    ip_gradient_matrix
      save    ip_pot
      save    ip_eigenvalues

      if(2*3*nhg.gt.lscr) call stopgm('GRADIENT','SCRATCH TOO SMALL')
      
      if (ifirst.eq.0) then
         ifirst=1   
         call memory(ip_gradient_matrix,3*3*nax*nsx,'grad_matrix')
         call memory(ip_efield,3*nax*nsx,'efield')
         call memory(ip_pot,nax*nsx,'potential')
         call memory(ip_eigenvalues,3*nax*nsx,'ev')
         call azzero(gradient_matrix,3*3*nax*nsx)
         call azzero(efield,3*nax*nsx)
         call azzero(pot,nax*nsx)
         call azzero(eigenvalues,3*nax*nsx)
      endif
      call fftTOg(rhor,rhoe,psi,nhg,.true.)
      if (i.eq.2) then
         call eicalc(scr(1,1),scr(1,2))
         do ig=1,nhg
            rhoe(ig)=rhoe(ig)+scr(ig,2)
         enddo   
      endif
      isa = 0    
      do isp=1,nsp
         do iat=1,na(isp)
            isa = isa + 1
            do ig=1,nhg
               scr(ig,1)=rhoe(ig)
            enddo
            if (i.eq.2) then
               call calcion(scr(1,2),isa,isp)
               do ig=1,nhg
                  scr(ig,1)=scr(ig,1)-scr(ig,2)
               enddo
            endif
            call apply_fpibg2 (scr(1,1))
            if (i.eq.1) then
               pot(iat,isp)=eigr_dot(isa,rhoe)
            else
               pot(iat,isp)=(-1.d0)*eigr_dot(isa,scr(1,1))
            endif
            do dir=1,3
               call apply_p_d (scr(1,1),scr(1,2),dir)
               if (i.eq.2) then
                  efield(dir,iat,isp)=eigr_dot(isa,scr(1,2))
               endif 
               do dir1=dir,3
                  call apply_p_d (scr(1,2),scr(1,3),dir1)
                  if ((i.eq.2).and.(dir1.eq.dir)) then
                     efield(dir,iat,isp)=eigr_dot(isa,scr(1,2))
                  endif
                  gradient_matrix(dir1,dir,iat,isp) = 
     &                 eigr_dot(isa,scr(1,3))
               enddo
            enddo
         enddo
      enddo
      
#ifdef PARALLEL
      call glosum(3*3*nax*nsx,gradient_matrix(1,1,1,1))
      call glosum(3*nax*nsx,efield(1,1,1))
      call glosum(nax*nsx,pot(1,1))
#endif
      if (i.eq.2) call trace(gradient_matrix)
      call epr_efg_outp(gradient_matrix,efield,pot,i,nfi)
      do isp=1,nsp
         do iat=1,na(isp)
            call DSYEV('N','L',3,gradient_matrix(1,1,iat,isp),
     &           3,eigenvalues(1,iat,isp),work,9,info)
            if (info .ne. 0) write (6, *)'ERROR IN DSYEV, code ',info
         enddo
      enddo
      call epr_efg_outp_ev(eigenvalues,i,nfi)
      return
      end

C     ==========================================================      
      subroutine apply_fpibg2 (src)
C     ==========================================================
      implicit none
      include 'system.h'
      include 'cnst.inc'
      include 'geq0.inc'
      include 'cppt.inc'

      complex*16 src(nhg)
      integer    ig,ig1
      real*8     x,xx
      
      xx = fpi / tpiba2
      ig1=1
      if (geq0) ig1=2
      do ig=ig1,nhg
        x = xx / HG(ig)
        src(ig) = src(ig) * x     
      enddo
      return
      end

C     ==========================================================      
      subroutine apply_p_d (src,dest,coordinate)
C     ==========================================================
      implicit none
      include 'system.h'
      include 'coor.inc'
      include 'cppt.inc'
      include 'gvec.inc'
      include 'fft.inc'  

      complex*16 src(nhg), dest(nhg)
      integer    coordinate
      integer    ig
      real*8     coeff,re,im

      do ig=1,nhg
        coeff    =   tpiba  * GK(coordinate,ig)
        re       =   -coeff*DIMAG(src(ig))
        im       =   coeff*DREAL(src(ig))
        dest(ig) =   DCMPLX(re,im)
c        dest(ig) =   DCMPLX(0.0D0,coeff)  * src(ig)
      enddo

      return
      end


C     ==============================================================
      subroutine save_rho (rhoe)
C     ==============================================================
      implicit none
      include 'system.h'
      include 'prop.inc'
      include 'spin.inc'

      real*8  rhoe(nnr1,nlsd)
      integer ifirst
      save    ifirst
      data    ifirst /0/
      
      if (teprefg.or.tefg) then
         if (ifirst .EQ. 0) then
            ifirst = 1
            if (teprefg) then
               call memory(ip_rho_save_epr,nnr1,'epr')
               call azzero(   rho_save_epr,nnr1)
            endif
            if (tefg) then
               call memory(ip_rho_save_efg,nnr1,'efg')
               call azzero(   rho_save_efg,nnr1)
            endif
         endif
         if (teprefg) then
            call DCOPY(nnr1,rhoe(1,1),1,rho_save_epr,1)
            call DAXPY(nnr1,-2d0,rhoe(1,2),1,rho_save_epr,1)
         endif
         if (tefg) then
            call DCOPY(nnr1,rhoe(1,1),1,rho_save_efg,1)
         endif
      endif
      return
      end

C     ============================================================
      subroutine epr_efg_outp (m,f,p,i,nfi)
C     ============================================================ 
      implicit none
      include 'system.h'
      include 'ions.inc'
      include 'fileopen.inc'

      real*8      m(3,3,nax,*),f(3,nax,*),p(nax,*)
      integer     i,isa,isp,iat,nfi,ifirst
      character*3 filename
      logical     err
      data        ifirst /-1/
      save        ifirst
      
      if (parent) then
        if (i.eq.1) then
          filename='EPR'
        elseif (i.eq.2) then
          filename='EFG'
        endif
        if(ifirst.lt.0) ifirst=FO_VERB+FO_MARK
        call fileopen(95,filename,FO_APP+ifirst,err)
        ifirst=0
        if (i.eq.1) then
 10       format(a3,1x,'matrix elements '
     &        //'(1,1) (2,2) (3,3) (1,2) (1,3) (2,3) rho(0)',/,1x)
          write(95,10) filename
        elseif (i.eq.2) then
 15       format(a3,1x,'matrix elements '
     &        //'(1,1) (2,2) (3,3) (1,2) (1,3) (2,3)  Ex Ey Ez',/,1x)
          write(95,15) filename
        endif
        write(95,*) nfi
        isa = 0
        do isp=1,nsp
          do iat=1,na(isp)
            isa = isa + 1
            if (i.eq.1) then
              write(95,20) isa,m(1,1,iat,isp),m(2,2,iat,isp),
     &            m(3,3,iat,isp),m(2,1,iat,isp),m(3,1,iat,isp),
     &            m(3,2,iat,isp),p(iat,isp)
 20           format(i3,3x,f8.5,2x,f8.5,2x,f8.5,2x,
     &            f8.5,2x,f8.5,2x,f8.5,4x,f13.10)
            elseif (i.eq.2) then
              write(95,25) isa,m(1,1,iat,isp),m(2,2,iat,isp),
     &            m(3,3,iat,isp),m(2,1,iat,isp),m(3,1,iat,isp),
     &            m(3,2,iat,isp),
     &            f(1,iat,isp),f(2,iat,isp),f(3,iat,isp),p(iat,isp)
 25           format(i3,3x,f8.5,2x,f8.5,2x,f8.5,2x,
     &            f8.5,2x,f8.5,2x,f8.5,4x,f8.5,2x,
     &            f8.5,2x,f8.5,3x,f13.10) 
            endif
          enddo
        enddo
      call fileclose(95)
      endif
      return
      end

C     ===============================================================
      subroutine trace (matrix)
C     ===============================================================
      implicit none
      include 'system.h'
      include 'ions.inc'

      real*8  matrix(3,3,nax,*)
      real*8  sum  
      integer i,iat,isp

      if (parent) then
         do isp=1,nsp
            do iat=1,na(isp) 
               sum=0.d0
               do i=1,3
                  sum=sum+matrix(i,i,iat,isp)
               enddo
               do i=1,3
                  matrix(i,i,iat,isp)=matrix(i,i,iat,isp)-sum/3
               enddo
            enddo
         enddo
      endif

      return
      end

C     =============================================================
      subroutine epr_efg_outp_ev (eigenvalues,i,nfi)
C     =============================================================
      implicit none
      include 'system.h'
      include 'ions.inc'
      include 'fileopen.inc'
      
      real*8      eigenvalues(3,nax,*)
      integer     i,isa,isp,iat,nfi,ifirst
      character*6 filename
      character*3 stat
      logical     fexist,err
      data        ifirst /-1/
      save        ifirst

      if (parent) then
        if (i.eq.1) then
          filename='EV_EPR'
        elseif (i.eq.2) then
          filename='EV_EFG'
        endif

        if(ifirst.lt.0) ifirst=FO_VERB+FO_MARK
        call fileopen(95,filename,FO_APP+ifirst,err)
        ifirst=0
        if (i.eq.1) then
          write(95,10) filename
 10       format(a3,1x,'Anisotropic Spin Dipole Couplings '
     &        //'Eigenvalues (principal axis system) ',/,1x)
        elseif (i.eq.2) then
          write(95,15) filename
 15       format(a3,1x,'Electric field gradient '
     &        //'Eigenvalues (principal axis system) ',/,1x)
        endif
      endif
      write(95,*) nfi
      isa = 0
      do isp=1,nsp
        do iat=1,na(isp)
          isa = isa + 1
          write(95,20) isa,eigenvalues(1,iat,isp),
     &        eigenvalues(2,iat,isp),eigenvalues(3,iat,isp)
 20       format(i3,3x,f8.4,2x,f8.4,2x,f8.4)
        enddo
      enddo
      call fileclose(95)
      return
      end

C     =============================================================
      subroutine calcion (EIROP,isa,is)
C     =============================================================
      implicit none 
      INCLUDE 'system.h'
      INCLUDE 'sfac.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cppt.inc'
C     Arguments
      COMPLEX*16 EIROP(*)
C     Variables
      DIMENSION  EIGRB(NHG,*)
      COMPLEX*16 EI123
      REAL*8     FNL(*),DFNL(*),
     &     ER,EI
      INTEGER    ISUB,IG,ISA,IS,IA
      
      CALL ZAZZERO(EIROP,NHG)
      DO IG=1,NHG
         EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     *        EI3(ISA,INYH(3,IG))
         ER=DREAL(EI123)
         EI=DIMAG(EI123)
         EIROP(IG)=EIROP(IG)+
     *        DCMPLX(ER*RHOPS(IS,IG),EI*RHOPS(IS,IG))
      ENDDO

      return
      end


C     =========================================================================

