C     ==================================================================
      SUBROUTINE INVFFT(F)
C     ==--------------------------------------------------------------==
C     == COMPUTES THE INVERSE FOURIER TRANSFORM OF A COMPLEX          ==
C     == FUNCTION F. THE FOURIER TRANSFORM IS                         ==
C     == RETURNED IN F (THE INPUT F IS OVERWRITTEN).                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      COMPLEX*16 F(*)
C     Variables
      INTEGER    ISUB,ISIGN,ITASK
C     ==--------------------------------------------------------------==
      CALL TISET('    INVFFT',ISUB)
      ITASK=0
      ISIGN=-1
#if defined(FFT_DEFAULT)
      CALL GFFTMAIN(ISIGN,ITASK,F)
#elif defined(FFT_ESSL)
      CALL ESSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_HP)
      CALL HPFFT(ISIGN,ITASK,F)
#elif defined(FFT_YMP)
      CALL YMPFFT(ISIGN,ITASK,F)
#elif defined(FFT_X1)
      CALL X1FFT(ISIGN,ITASK,F)
#elif defined(FFT_T3D)
      CALL T3DFFT(ISIGN,ITASK,F)
#elif defined(FFT_SCSL) 
      CALL SCSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_FFTW) || defined(FFT_FFTW2) || defined(FFT_FFTW3)
      CALL FFTWFFT(ISIGN,ITASK,F)
#else
      CALL STOPGM("INVFFT","LIBRARY NOT AVAILABLE")
#endif
      CALL TIHALT('    INVFFT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FWFFT(F)
C     ==--------------------------------------------------------------==
C     == COMPUTES THE FORWARD FOURIER TRANSFORM OF A COMPLEX          ==
C     == FUNCTION F. THE FOURIER TRANSFORM IS                         ==
C     == RETURNED IN F IN OUTPUT (THE INPUT F IS OVERWRITTEN).        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      COMPLEX*16 F(*)
C     Variables
      INTEGER    ISUB,ISIGN,ITASK
C     ==--------------------------------------------------------------==
      CALL TISET('     FWFFT',ISUB)
      ITASK=0
      ISIGN=1
#if defined(FFT_DEFAULT)
      CALL GFFTMAIN(ISIGN,ITASK,F)
#elif defined(FFT_ESSL)
      CALL ESSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_HP)
      CALL HPFFT(ISIGN,ITASK,F)
#elif defined(FFT_X1)
      CALL X1FFT(ISIGN,ITASK,F)
#elif defined(FFT_YMP)
      CALL YMPFFT(ISIGN,ITASK,F)
#elif defined(FFT_T3D)
      CALL T3DFFT(ISIGN,ITASK,F)
#elif defined(FFT_SCSL) 
      CALL SCSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_FFTW) || defined(FFT_FFTW2) || defined(FFT_FFTW3)
      CALL FFTWFFT(ISIGN,ITASK,F)
#else
      CALL STOPGM("FWFFT","LIBRARY NOT AVAILABLE")
#endif
      CALL TIHALT('     FWFFT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE S_INVFFT(F)
C     ==--------------------------------------------------------------==
C     == COMPUTES THE INVERSE FOURIER TRANSFORM OF A COMPLEX          ==
C     == FUNCTION F. THE FOURIER TRANSFORM IS                         ==
C     == RETURNED IN F (THE INPUT F IS OVERWRITTEN).                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      COMPLEX*16 F(*)
C     Variables
      INTEGER    ISUB,ISIGN,ITASK
C     ==--------------------------------------------------------------==
      CALL TISET('  S_INVFFT',ISUB)
      ITASK=1
      ISIGN=-1
#if defined(FFT_DEFAULT)
      CALL GFFTMAIN(ISIGN,ITASK,F)
#elif defined(FFT_ESSL)
      CALL ESSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_HP)
      CALL HPFFT(ISIGN,ITASK,F)
#elif defined(FFT_YMP)
      CALL YMPFFT(ISIGN,ITASK,F)
#elif defined(FFT_X1)
      CALL X1FFT(ISIGN,ITASK,F)
#elif defined(FFT_T3D)
      CALL T3DFFT(ISIGN,ITASK,F)
#elif defined(FFT_SCSL) 
      CALL SCSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_FFTW) || defined(FFT_FFTW2) || defined(FFT_FFTW3)
      CALL FFTWFFT(ISIGN,ITASK,F)
#else
      CALL STOPGM("S_INVFFT","LIBRARY NOT AVAILABLE")
#endif
      CALL TIHALT('  S_INVFFT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE S_FWFFT(F)
C     ==--------------------------------------------------------------==
C     == COMPUTES THE FORWARD FOURIER TRANSFORM OF A COMPLEX          ==
C     == FUNCTION F. THE FOURIER TRANSFORM IS                         ==
C     == RETURNED IN F IN OUTPUT (THE INPUT F IS OVERWRITTEN).        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      COMPLEX*16 F(*)
C     Variables
      INTEGER    ISUB,ISIGN,ITASK
C     ==--------------------------------------------------------------==
      CALL TISET('   S_FWFFT',ISUB)
      ITASK=1
      ISIGN=1
#if defined(FFT_DEFAULT)
      CALL GFFTMAIN(ISIGN,ITASK,F)
#elif defined(FFT_ESSL)
      CALL ESSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_HP)
      CALL HPFFT(ISIGN,ITASK,F)
#elif defined(FFT_YMP)
      CALL YMPFFT(ISIGN,ITASK,F)
#elif defined(FFT_X1)
      CALL X1FFT(ISIGN,ITASK,F)
#elif defined(FFT_T3D)
      CALL T3DFFT(ISIGN,ITASK,F)
#elif defined(FFT_SCSL) 
      CALL SCSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_FFTW) || defined(FFT_FFTW2) || defined(FFT_FFTW3)
      CALL FFTWFFT(ISIGN,ITASK,F)
#else
      CALL STOPGM("S_FWFFT","LIBRARY NOT AVAILABLE")
#endif
      CALL TIHALT('   S_FWFFT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SG_INVFFT(F)
C     ==--------------------------------------------------------------==
C     == COMPUTES THE INVERSE FOURIER TRANSFORM OF A COMPLEX          ==
C     == FUNCTION F. THE FOURIER TRANSFORM IS                         ==
C     == RETURNED IN F (THE INPUT F IS OVERWRITTEN).                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      COMPLEX*16 F(*)
C     Variables
      INTEGER    ISUB,ISIGN,ITASK
C     ==--------------------------------------------------------------==
      CALL TISET(' SG_INVFFT',ISUB)
      ITASK=2
      ISIGN=-1
#if defined(FFT_DEFAULT)
      CALL GFFTMAIN(ISIGN,ITASK,F)
#elif defined(FFT_ESSL)
      CALL ESSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_HP)
      CALL HPFFT(ISIGN,ITASK,F)
#elif defined(FFT_YMP)
      CALL YMPFFT(ISIGN,ITASK,F)
#elif defined(FFT_X1)
      CALL X1FFT(ISIGN,ITASK,F)
#elif defined(FFT_T3D)
      CALL T3DFFT(ISIGN,ITASK,F)
#elif defined(FFT_SCSL) 
      CALL SCSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_FFTW) || defined(FFT_FFTW2) || defined(FFT_FFTW3)
      CALL FFTWFFT(ISIGN,ITASK,F)
#else
      CALL STOPGM("SG_INVFFT","LIBRARY NOT AVAILABLE")
#endif
      CALL TIHALT(' SG_INVFFT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SG_FWFFT(F)
C     ==--------------------------------------------------------------==
C     == COMPUTES THE FORWARD FOURIER TRANSFORM OF A COMPLEX          ==
C     == FUNCTION F. THE FOURIER TRANSFORM IS                         ==
C     == RETURNED IN F IN OUTPUT (THE INPUT F IS OVERWRITTEN).        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      COMPLEX*16 F(*)
C     Variables
      INTEGER    ISUB,ISIGN,ITASK
C     ==--------------------------------------------------------------==
      CALL TISET('  SG_FWFFT',ISUB)
      ITASK=2
      ISIGN=1
#if defined(FFT_DEFAULT)
      CALL GFFTMAIN(ISIGN,ITASK,F)
#elif defined(FFT_ESSL)
      CALL ESSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_HP)
      CALL HPFFT(ISIGN,ITASK,F)
#elif defined(FFT_YMP)
      CALL YMPFFT(ISIGN,ITASK,F)
#elif defined(FFT_X1)
      CALL X1FFT(ISIGN,ITASK,F)
#elif defined(FFT_T3D)
      CALL T3DFFT(ISIGN,ITASK,F)
#elif defined(FFT_SCSL) 
      CALL SCSLFFT(ISIGN,ITASK,F)
#elif defined(FFT_FFTW) || defined(FFT_FFTW2) || defined(FFT_FFTW3)
      CALL FFTWFFT(ISIGN,ITASK,F)
#else
      CALL STOPGM("SG_FWFFT","LIBRARY NOT AVAILABLE")
#endif
      CALL TIHALT('  SG_FWFFT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION FFTCHK(M,N)
C     ==--------------------------------------------------------------==
C     N < 0 : take the next smaller one
C     N = 1 : take the next bigger one
C     N = 2 : take the next bigger even one
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   FFTCHK
      INTEGER   M,N
      INTEGER   FFTCHK_DEF, FFTCHK_ESSL, FFTCHK_FFTW
      EXTERNAL FFTCHK_DEF, FFTCHK_ESSL, FFTCHK_FFTW
C     ==--------------------------------------------------------------==
#if defined(FFT_DEFAULT)
      FFTCHK = FFTCHK_DEF(M,N)
#elif defined(FFT_ESSL)
      FFTCHK = FFTCHK_ESSL(M,N)
#elif defined(FFT_HP)
      FFTCHK = FFTCHK_DEF(M,N)
#elif defined(FFT_YMP)
      FFTCHK = FFTCHK_DEF(M,N)
#elif defined(FFT_X1)
      FFTCHK = FFTCHK_DEF(M,N)
#elif defined(FFT_T3D)
      FFTCHK = FFTCHK_DEF(M,N)
#elif defined(FFT_SCSL) 
      FFTCHK = FFTCHK_DEF(M,N)
#elif defined(FFT_FFTW) || defined(FFT_FFTW2) || defined(FFT_FFTW3)
      FFTCHK = FFTCHK_FFTW(M,N)
#else
      CALL STOPGM("FFTCHK","TABLE OF ROOTS NOT AVAILABLE")
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FFTPRP
C     ==--------------------------------------------------------------==
#if defined(FFT_DEFAULT)
      CALL FFTPRP_DEFAULT
#elif defined(FFT_ESSL)
      CALL FFTPRP_DEFAULT
#elif defined(FFT_HP)
      CALL FFTPRP_DEFAULT
#elif defined(FFT_YMP)
      CALL FFTPRP_DEFAULT
#elif defined(FFT_X1)
      CALL FFTPRP_DEFAULT
#elif defined(FFT_T3D)
      CALL FFTPRP_DEFAULT
#elif defined(FFT_SCSL) 
      CALL FFTPRP_DEFAULT
#elif defined(FFT_FFTW) || defined(FFT_FFTW2) || defined(FFT_FFTW3)
      CALL FFTPRP_DEFAULT
#else
      CALL STOPGM("FFTPRP","FFTPRP ROUTINE NOT AVAILABLE")
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MLTFFT(TRANSA,TRANSB,A,LDAX,LDAY,
     *                  B,LDBX,LDBY,N,M,ISIGN,SCALE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
#if defined(FFT_FFTW) || defined(FFT_FFTW2) || defined(FFT_FFTW3)
      INCLUDE 'system.h'
#endif
C     Arguments 
      CHARACTER  TRANSA*1,TRANSB*1
      INTEGER    LDAX,LDAY,LDBX,LDBY,N,M,ISIGN
      COMPLEX*16 A(LDAX,LDAY),B(LDBX,LDBY)
      REAL*8     SCALE
C     ==--------------------------------------------------------------==
#if defined(FFT_DEFAULT)
      CALL MLTFFT_DEFAULT(TRANSA,TRANSB,A,LDAX,LDAY,
     *                    B,LDBX,LDBY,N,M,ISIGN,SCALE)
#elif defined(FFT_ESSL)
      CALL MLTFFT_ESSL(TRANSA,TRANSB,A,LDAX,LDAY,
     *                 B,LDBX,LDBY,N,M,ISIGN,SCALE)
#elif defined(FFT_HP)
      CALL MLTFFT_HP(TRANSA,TRANSB,A,LDAX,LDAY,
     *               B,LDBX,LDBY,N,M,ISIGN,SCALE)
#elif defined(FFT_YMP)
      CALL MLTFFT_CRAY(TRANSA,TRANSB,A,LDAX,LDAY,
#elif defined(FFT_X1)
      CALL MLTFFT_X1(TRANSA,TRANSB,A,LDAX,LDAY,
     *                 B,LDBX,LDBY,N,M,ISIGN,SCALE)
#elif defined(FFT_T3D)
      CALL MLTFFT_CRAY(TRANSA,TRANSB,A,LDAX,LDAY,
     *                 B,LDBX,LDBY,N,M,ISIGN,SCALE)
#elif defined(FFT_SCSL) 
      CALL MLTFFT_SCSL(TRANSA,TRANSB,A,LDAX,LDAY,
     *                 B,LDBX,LDBY,N,M,ISIGN,SCALE)
#elif defined(FFT_FFTW) || defined(FFT_FFTW2) || defined(FFT_FFTW3)
      CALL MLTFFT_FFTW(TRANSA,TRANSB,A,LDAX,LDAY,
     *                 B,LDBX,LDBY,N,M,ISIGN,SCALE,TFFTWSDM)
#else
      CALL STOPGM("MLTFFT","MLTFFT ROUTINE NOT AVAILABLE")
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     NEW FFT CODE
C     ==================================================================
      SUBROUTINE INVFFTN(F,SPARSE)
C     ==--------------------------------------------------------------==
C     == COMPUTES THE INVERSE FOURIER TRANSFORM OF A COMPLEX          ==
C     == FUNCTION F. THE FOURIER TRANSFORM IS                         ==
C     == RETURNED IN F (THE INPUT F IS OVERWRITTEN).                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      COMPLEX*16 F(*)
      LOGICAL SPARSE
C     Variables
      INTEGER    ISUB,ISIGN,ITASK
C     ==--------------------------------------------------------------==
      CALL TISET('   INVFFTN',ISUB)
      ITASK=10
      ISIGN=-1
      CALL FFTNEW(ISIGN,ITASK,F,SPARSE)
      CALL TIHALT('   INVFFTN',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FWFFTN(F,SPARSE)
C     ==--------------------------------------------------------------==
C     == COMPUTES THE FORWARD FOURIER TRANSFORM OF A COMPLEX          ==
C     == FUNCTION F. THE FOURIER TRANSFORM IS                         ==
C     == RETURNED IN F IN OUTPUT (THE INPUT F IS OVERWRITTEN).        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      COMPLEX*16 F(*)
      LOGICAL SPARSE
C     Variables
      INTEGER    ISUB,ISIGN,ITASK
C     ==--------------------------------------------------------------==
      CALL TISET('    FWFFTN',ISUB)
      ITASK=10
      ISIGN=1
      CALL FFTNEW(ISIGN,ITASK,F,SPARSE)
      CALL TIHALT('    FWFFTN',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
