C     ==================================================================
C     INITIALIZE ARRAYS FOR THE FILEOPEN CALLS.
      SUBROUTINE INIT_FILEOPEN
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INTEGER I
      INCLUDE 'fileopen.inc'
      FO_TDEBUG=.FALSE.
      IAPATH=1
      IEPATH=2
      FPATH='./                            '
      DO I=1,FO_STAT_MAX
        FO_STAT(I)=.FALSE.
      ENDDO
C     SOME DEFAULT UNITS ARE ALREADY FLAGGED OPEN.
      FO_STAT(5)=.TRUE.
      FO_STAT(6)=.TRUE.
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     see fileopen.inc for documentation of the API.
C     ==================================================================
      SUBROUTINE FILEOPEN(IUNIT,FILEN,FO_MODE,ERROR)
      IMPLICIT NONE
      INCLUDE 'fileopen.inc'
C     Arguments
      INTEGER   IUNIT,FO_MODE
      LOGICAL   ERROR
      CHARACTER FILEN*(*)
C     Locals
      CHARACTER FULLPATH*255,FO_FLAGS*255
      CHARACTER*8 FO_MODE_STRING(0:9)
      CHARACTER FO_FORM*12
      INTEGER   I,IA,IE,IA1,IE1,IA2,IE2,ISUB,OPENMODE
      LOGICAL   FEXIST,TFOVRB,TFODBG
      DATA FO_MODE_STRING/'FO_DEF','FO_NEW','FO_OLD','FO_APP',
     &     'FO_UFO','FO_VERB','FO_MARK','FO_NOCHK','FO_NOFP','FO_DEBUG'/
C     ==--------------------------------------------------------------==
      ERROR=.FALSE.
      FEXIST=.FALSE.
      TFOVRB=.FALSE.
      TFODBG=FO_TDEBUG
#if defined(__ES)
cmb   TISET no allowed here on NEC-SX6-ES and NEC-SX7-ES
#else
      CALL TISET('  FILEOPEN',ISUB)
#endif
      CALL XSTRING(FILEN,IA1,IE1)
C========================================================================
C     HANDLE PLATFORM SPECIFIC OPEN WITH APPEND.
C     we do the messy preprocessor stuff here so the rest of
C     the code is more readable. currently supported:
C     AppendType 1: USE STATUS='OLD', ACCESS='APPEND'
C     AppendType 2: USE STATUS='OLD', POSITION='APPEND'
C     can be overridden with -DOPEN_APPEND_TYPE=X
C     AK 2005/04/09
#if !defined(OPEN_APPEND_TYPE)
#  define AppendType 1
#  if defined(__IBM) || defined(__ABSOFT) || defined(__SGI) || defined(__OSX) || defined(__OSX_IFC)
#    undef AppendType
#    define AppendType 2
#  elif defined(CRAY) || defined(__HP) || defined(__SR2201) || defined(__SR8000)
#    undef AppendType
#    define AppendType 2
#  elif defined(__PGI) && ( defined(__pgf90) || defined(__PATHSCALE) )
#    undef AppendType
#    define AppendType 2
#  endif
#else 
#  define AppendType OPEN_APPEND_TYPE
#endif

 222  FORMAT(X,A,I4)

C     BE VERY VERBOSE, IF REQUESTED
      IF(MOD(FO_MODE/FO_DEBUG,2).NE.0) TFODBG=.TRUE.
      IF(TFODBG) THEN
        WRITE(FO_FLAGS,'(A)') FO_MODE_STRING(MOD(FO_MODE,4))
        CALL XSTRING(FO_FLAGS,IA,IE)
        DO I=4,9
          IF(MOD(FO_MODE/(2**(I-2)),2).NE.0) THEN
            WRITE(FO_FLAGS(IE+1:255),'(2A)') '+',FO_MODE_STRING(I)
            CALL XSTRING(FO_FLAGS,IA,IE)
          ENDIF
        ENDDO
        WRITE(6,'(A,I4,5A)') ' FILEOPEN: OPEN UNIT=',IUNIT,
     &       ' FLAGS=',FO_FLAGS(1:IE),' FILENAME=',FILEN(IA1:IE1),'"'
      ENDIF

C     BE VERBOSE, IF REQUESTED
      IF(MOD(FO_MODE/FO_VERB,2).NE.0) THEN
        IF(TFODBG) WRITE(6,*) 'FILEOPEN: VERBOSE FLAG'
        TFOVRB=.TRUE.
      ENDIF

C     do not use FILEPATH keyword (or CPMD_FILEPATH environment variable)
      IF(MOD(FO_MODE/FO_NOFP,2).NE.0) THEN
        IF(TFODBG) WRITE(6,*) 'FILEOPEN: DO NOT USE FPATH'
        FULLPATH=FILEN(IA1:IE1)
      ELSE
        FULLPATH=FPATH(IAPATH:IEPATH)//FILEN(IA1:IE1)
      ENDIF
      CALL XSTRING(FULLPATH,IA2,IE2)
      IF(TFODBG) WRITE(6,*) 'FILEOPEN: FULLPATH=',FULLPATH(IA2:IE2)

C     CHECK WHETHER THE CHANNEL IS ALREADY OPEN
      IF(MOD(FO_MODE/FO_NOCHK,2).EQ.0) THEN
        IF(TFODBG) WRITE(6,222) 'FILEOPEN: CHECK FOR OPEN UNIT:',IUNIT
        IF(IUNIT.GT.0.AND.IUNIT.LT.FO_STAT_MAX) THEN
          IF(FO_STAT(IUNIT)) THEN
            WRITE(6,'(A,I4,2A)') ' FILEOPEN: UNIT ',IUNIT,
     &           ' ALREADY IN USE WHEN OPENING FILE ',FILEN(IA1:IE1)
            ERROR=.TRUE.
#if defined(__ES)
cmb   TIHALT not allowed on NEC-ES (Bus error)
            CALL STOPGM('FILEOPEN','TIHALT ESCAPE')
#else
            CALL TIHALT('  FILEOPEN',ISUB)
#endif
            RETURN
          ENDIF
        ENDIF
      ELSE
        IF(TFODBG) WRITE(6,*) 'FILEOPEN: UNIT OPEN CHECK SUPPRESSED'
      ENDIF

C     CHECK WHETHER THE FILE ALREADY EXISTS AND ADJUST THE OPEN MODE IF NOT.
      OPENMODE=MOD(FO_MODE,4)
      IF(TFODBG) WRITE(6,222) 'FILEOPEN: OPENMODE:',OPENMODE
      IF((MOD(FO_MODE/FO_NOCHK,2).EQ.0).AND.(OPENMODE.EQ.FO_APP)) THEN
        INQUIRE(FILE=FULLPATH(IA2:IE2),EXIST=FEXIST)
        IF(.NOT.FEXIST) OPENMODE=0
        IF(TFODBG) WRITE(6,222) 'FILEOPEN: OPENMODE NOW:',OPENMODE
      ENDIF

C     SET FORMATTED/UNFORMATTED
      FO_FORM='FORMATTED'
      IF(MOD(FO_MODE/FO_UFO,2).NE.0) FO_FORM='UNFORMATTED'
C     NOW OPEN FILE
      IF(OPENMODE.EQ.0) THEN
        IF(TFODBG) WRITE(6,*) 'FILEOPEN: OPEN FILE WITH STATUS=UNKNOWN'
        OPEN(UNIT=IUNIT,FILE=FULLPATH(IA2:IE2),STATUS='UNKNOWN',
     &       FORM=FO_FORM,ERR=999)
      ELSEIF (OPENMODE.EQ.1) THEN
        IF(TFODBG) WRITE(6,*) 'FILEOPEN: OPEN FILE WITH STATUS=NEW'
        OPEN(UNIT=IUNIT,FILE=FULLPATH(IA2:IE2),STATUS='NEW',
     &       FORM=FO_FORM,ERR=999)
      ELSEIF (OPENMODE.EQ.2) THEN
        IF(TFODBG) WRITE(6,*) 'FILEOPEN: OPEN FILE WITH STATUS=OLD'
        OPEN(UNIT=IUNIT,FILE=FULLPATH(IA2:IE2),STATUS='OLD',
     &       FORM=FO_FORM,ERR=999)
      ELSEIF (OPENMODE.EQ.3) THEN
        IF(TFODBG) WRITE(6,*) 'FILEOPEN: OPEN FILE FOR APPENDING'
        OPEN(UNIT=IUNIT,FILE=FULLPATH(IA2:IE2),STATUS='OLD',
#if AppendType == 1
     &       ACCESS='APPEND',
#elif AppendType == 2
     &       POSITION='APPEND',
#else
C    Unknown AccessType force an ERROR:
        ,
#endif
     &       FORM=FO_FORM,ERR=999)
        IF(TFOVRB) WRITE(6,'(3A)') ' FILE ',FILEN(IA1:IE1),
     &       ' EXISTS, NEW DATA WILL BE APPENDED'
      ELSE
C     we should never get here
        CALL STOPGM('FILEOPEN','UNKNOWN FILE ACCESS MODE')
      ENDIF
C     RECORD THAT WE HAVE OPENED THIS UNIT
      IF(IUNIT.LT.FO_STAT_MAX) THEN
        FO_STAT(IUNIT)=.TRUE.
      ENDIF

C     WRITE NEW DATA MARK
      IF(OPENMODE.EQ.3.AND.MOD(FO_MODE/FO_MARK,2).NE.0) THEN
        IF(TFODBG) WRITE(6,*) 'FILEOPEN: WRITE NEW DATA MARK'
        WRITE(IUNIT,'(A)') '   <<<<<<  NEW DATA  >>>>>>'
      ENDIF
      IF(TFODBG) WRITE(6,*) 'FILEOPEN: --------------------'
#if defined(__ES)
cmb   TIHALT not allowed on NEC-ES (Bus error)
      RETURN
#else
      CALL TIHALT('  FILEOPEN',ISUB)
      RETURN
#endif

C     FILE OPEN ERROR
 999  CONTINUE
      IF(TFODBG) WRITE(6,*) 'FILEOPEN: ERROR WHEN OPENING FILE'
      ERROR=.TRUE.
#if defined(__ES)
cmb   TIHALT not allowed on NEC-ES (Bus error)
      CALL STOPGM('FILEOPEN','TIHALT ESCAPE')
#else
      CALL TIHALT('  FILEOPEN',ISUB)
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================


C     ==================================================================
      SUBROUTINE FILECLOSE(IUNIT)
      IMPLICIT NONE
      INCLUDE 'fileopen.inc'
      INTEGER IUNIT
C     ==--------------------------------------------------------------==
      CLOSE(IUNIT)
C     RECORD THAT WE HAVE CLOSED THIS UNIT
      IF(FO_TDEBUG) WRITE(6,'(A,I4)') ' FILECLOSE: CLOSING UNIT:',IUNIT
      IF(IUNIT.LT.FO_STAT_MAX) THEN
        FO_STAT(IUNIT)=.FALSE.
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
