C     ==================================================================
      SUBROUTINE FNONLOC(C2,F,NSTATE,AUXC,DDIA,IKIND,ISPIN)
C     ==--------------------------------------------------------------==
C     == CALCULATES THE NON-LOCAL PP CONTRIBUTION OF THE HAMILTONIAN  ==
C     ==--------------------------------------------------------------==
C     == INPUT/OUPUT:                                                 ==
C     ==   C2(NGWK,NSTATE) Wavefunctions, output C2 = C2 + |N.L.PART> ==
C     == INPUT:                                                       ==
C     ==   NSTATE      Number of states                               ==
C     ==   F(1:NSTATE) Occupation numbers                             ==
C     ==   IKIND  Index of k point                                    ==
C     ==   ISPIN  Need with LSD option for diagonalization scheme     ==
C     ==          Does not work with TDIAG (ISPIN=1) and TIVAN        ==
C     == SCRATCH                                                      ==
C     ==   AUXC(NGWK)                                                 ==
C     ==   DDIA(IMAGP*NAX)                                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sgpp.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
C     Arguments
      INTEGER    NSTATE,IKIND,ISPIN
      COMPLEX*16 C2(NGWK,NSTATE),AUXC(NGWK,NSTATE)
      REAL*8     F(NSTATE),DDIA(IMAGP*NAX,NSTATE)
C     Variables
      DIMENSION  EIGR(NGW,*),TWNL(NGWK,NHXS,NSX,*),
     &           EIGKR(NGWK,NAT,*),FNL2(IMAGP,NAT,NHXS,LDF2,*)
      COMPLEX*16 ZZERO,ZONE
      PARAMETER  (ZZERO=(0.D0,0.D0),ZONE=(1.D0,0.D0))
      COMPLEX*16 CTM
      REAL*8     FNL(IMAGP,LDF1,NHXS,NSTATE,NKPNT),DFNL(*)
      REAL*8     DDKI(2,NAX,NSTATE),FFI,T1,TR,TI,T
      POINTER    (IP_DDKI,DDKI)
      INTEGER    ISUB,LSPIN,I,ISA0,IV,IS,IA,ISA,JV,L,
     &           KI,LI,L2,LJ,KJ,IG,IAA
C     ==--------------------------------------------------------------==
C     == Compute the force on the electronic degrees of freedom due   ==
C     == to the non-local part of the potential, and add it to the    ==
C     == other piece, coming from the local contribution.             ==
C     ==--------------------------------------------------------------==
      IF(NGWK.EQ.0 .AND. .NOT.TFDIST) RETURN
      CALL TISET('   FNONLOC',ISUB)
      CALL GET_ADDR(IP_DDKI,DDIA(1,1))
      ISA0=0
      DO IS=1,NSP
        IF(TVAN(IS)) THEN
C     ==--------------------------------------------------------------==
C     ==  VANDERBILT PP                                               ==
C     ==--------------------------------------------------------------==
          LSPIN=1
          DO IV=1,NGH(IS)
            CALL AZZERO(DDIA,IMAGP*NAX*NSTATE)
#if defined(__ES)
!$OMP parallel do private(I,IA,JV,LSPIN,ISA,IAA)
#endif
#ifdef __SR8000
*poption parallel, tlocal(I,IA,JV,LSPIN,ISA,IAA)
#endif 
            DO I=1,NSTATE
              IF(TLSD.AND.ISPIN.EQ.2) THEN
                IF(I.LE.NSUP) THEN
                  LSPIN=1
                ELSE
                  LSPIN=2
                ENDIF
              ENDIF
              DO IA=1,NA(IS)
                ISA=ISA0+IA
                IF(TFDIST) THEN
                  IF(IATPE(ISA).EQ.MEPOS) THEN
                    IAA=ISA-IPEPT(1,MEPOS)+1
#ifdef  _BGL  
!IBM*  UNROLL(10)
!IBM*   ASSERT (ITERCNT(10000))
#endif
                    DO JV=1,NGH(IS)
                      DDIA(IA,I)=DDIA(IA,I)+FNL(1,IAA,JV,I,IKIND)*
     *                  (DEEQ(ISA,IV,JV,LSPIN)+DVAN(IV,JV,IS))
                    ENDDO
                  ENDIF
                ELSE
#ifdef  _BGL  
!IBM*  UNROLL(10)
!IBM*   ASSERT (ITERCNT(10000))
#endif
                  DO JV=1,NGH(IS)
                    DDIA(IA,I)=DDIA(IA,I)+FNL(1,ISA,JV,I,IKIND)*
     *                  (DEEQ(ISA,IV,JV,LSPIN)+DVAN(IV,JV,IS))
                  ENDDO
                ENDIF
              ENDDO
            ENDDO
            IF(TFDIST) CALL GLOSUM(IMAGP*NAX*NSTATE,DDIA)
            IF(NGWK.GT.0) THEN
             IF(TKPNT) THEN
               CALL ZGEMM("N","N",NGWK,NSTATE,NA(IS),ZONE,
     *            EIGKR(1,ISA0+1,IKIND),NGWK,DDKI(1,1,1),
     *            NAX,ZZERO,AUXC,NGWK)
             ELSE
               CALL DGEMM("N","N",2*NGW,NSTATE,NA(IS),1.D0,
     *            EIGR(1,ISA0+1),2*NGW,DDIA(1,1),NAX,0.D0,AUXC,2*NGW)
             ENDIF
!$OMP parallel do private(I,IG,FFI,CTM,T1,TR,TI)
#ifdef __SR8000
*poption parallel, tlocal(I,IG,FFI,CTM,T1,TR,TI)
#endif 
             DO I=1,NSTATE
               FFI=F(I)
               IF(FFI.LT.1.D-5) FFI=1.0D0
               CTM=-FFI*(0.0D0,-1.0D0)**NGHTOL(IV,IS)
C              MAKE USE OF THE SPECIAL STRUCTURE OF CTM
               IF(DABS(DREAL(CTM)).GT.0.01D0) THEN
C                CTM IS REAL
#ifdef  _BGL  
!IBM*  INDEPENDENT, NEW(T1,TR,TI)
!IBM*   ASSERT (ITERCNT(1000))
#endif
                 DO IG=1,NGWK
                   T1=TWNL(IG,IV,IS,IKIND)*DREAL(CTM)
                   TR=DREAL(AUXC(IG,I))
                   TI=DIMAG(AUXC(IG,I))
                   C2(IG,I)=C2(IG,I)+DCMPLX(T1*TR,T1*TI)
                 ENDDO
               ELSE
C                CTM IS IMAGINARY
#ifdef  _BGL  
!IBM*  INDEPENDENT, NEW(T1,TR,TI)
!IBM*   ASSERT (ITERCNT(1000))
#endif
                 DO IG=1,NGWK
                   T1=TWNL(IG,IV,IS,IKIND)*DIMAG(CTM)
                   TR=DREAL(AUXC(IG,I))
                   TI=DIMAG(AUXC(IG,I))
                   C2(IG,I)=C2(IG,I)+DCMPLX(-T1*TI,T1*TR)
                 ENDDO
               ENDIF
               IF(TKPNT.AND.GEQ0) C2(1+NGW,I)=DCMPLX(0.D0,0.D0)
             ENDDO
            ENDIF
          ENDDO
        ELSE IF(TSGP(IS)) THEN
C     ==--------------------------------------------------------------==
C     ==  Stefan Goedeckers PP                                        ==
C     ==--------------------------------------------------------------==
          DO IV=1,NGH(IS)
            CALL AZZERO(DDIA,IMAGP*NAX*NSTATE)
            L=NGHTOL(IV,IS)+1
            KI=LFVAL(IV,IS)
            LI=LPVAL(IV,IS)
            DO JV=1,NGH(IS)
              L2=NGHTOL(JV,IS)+1
              LJ=LPVAL(JV,IS)
              IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                KJ=LFVAL(JV,IS)
#if defined(__ES)
!$OMP parallel do private(I,IA,JV,LSPIN,ISA,IAA)
#endif
#ifdef __SR8000
*poption parallel, tlocal(I,IA,ISA,IAA)
#endif 
                DO I=1,NSTATE
#ifdef _vpp_
!OCL NOALIAS 
#endif 
                  DO IA=1,NA(IS)
                    ISA=ISA0+IA
                    IF(TFDIST) THEN
                      IF(IATPE(ISA).EQ.MEPOS) THEN
                        IAA=ISA-IPEPT(1,MEPOS)+1
                        IF(TKPNT) THEN
                          DDKI(1,IA,I)=DDKI(1,IA,I)+
     &                      FNL(1,IAA,JV,I,IKIND)*HLSG(KI,KJ,L,IS)
                          DDKI(2,IA,I)=DDKI(2,IA,I)+
     &                      FNL(2,IAA,JV,I,IKIND)*HLSG(KI,KJ,L,IS)
                        ELSE
                          DDIA(IA,I)=DDIA(IA,I)+
     &                      FNL(1,IAA,JV,I,IKIND)*HLSG(KI,KJ,L,IS)
                        ENDIF
                      ENDIF
                    ELSE
                      IF(TKPNT) THEN
                        DDKI(1,IA,I)=DDKI(1,IA,I)+
     &                      FNL(1,ISA,JV,I,IKIND)*HLSG(KI,KJ,L,IS)
                        DDKI(2,IA,I)=DDKI(2,IA,I)+
     &                      FNL(2,ISA,JV,I,IKIND)*HLSG(KI,KJ,L,IS)
                      ELSE
                        DDIA(IA,I)=DDIA(IA,I)+
     &                      FNL(1,ISA,JV,I,IKIND)*HLSG(KI,KJ,L,IS)
                      ENDIF
                    ENDIF
                  ENDDO 
                ENDDO
              ENDIF
            ENDDO
            IF(TFDIST) CALL GLOSUM(IMAGP*NAX*NSTATE,DDIA)
            IF(NGWK.GT.0) THEN
             IF(TKPNT) THEN
               CALL ZGEMM("N","N",NGWK,NSTATE,NA(IS),ZONE,
     *            EIGKR(1,ISA0+1,IKIND),NGWK,DDKI(1,1,1),
     *            NAX,ZZERO,AUXC,NGWK)
             ELSE
               CALL DGEMM("N","N",2*NGW,NSTATE,NA(IS),1.D0,
     *            EIGR(1,ISA0+1),2*NGW,DDIA(1,1),NAX,0.D0,AUXC,2*NGW)
             ENDIF
!$OMP parallel do private(I,IG,FFI,CTM,T1,TR,TI)
#ifdef __SR8000
*poption parallel, tlocal(I,IG,FFI,CTM,T1,TR,TI)
#endif 
             DO I=1,NSTATE
               FFI=F(I)
               IF(FFI.LT.1.D-5) FFI=1.0D0
               CTM=-FFI*(0.0D0,-1.0D0)**NGHTOL(IV,IS)
C              MAKE USE OF THE SPECIAL STRUCTURE OF CTM
               IF(DABS(DREAL(CTM)).GT.0.01D0) THEN
C                CTM IS REAL
#ifdef  _BGL  
!IBM*  INDEPENDENT, NEW(T1,TR,TI)
!IBM*   ASSERT (ITERCNT(1000))
#endif
                 DO IG=1,NGWK
                   T1=TWNL(IG,IV,IS,IKIND)*DREAL(CTM)
                   TR=DREAL(AUXC(IG,I))
                   TI=DIMAG(AUXC(IG,I))
                   C2(IG,I)=C2(IG,I)+DCMPLX(T1*TR,T1*TI)
                 ENDDO
               ELSE
C                CTM IS IMAGINARY
#ifdef  _BGL  
!IBM*  INDEPENDENT, NEW(T1,TR,TI)
!IBM*   ASSERT (ITERCNT(1000))
#endif
                 DO IG=1,NGWK
                   T1=TWNL(IG,IV,IS,IKIND)*DIMAG(CTM)
                   TR=DREAL(AUXC(IG,I))
                   TI=DIMAG(AUXC(IG,I))
                   C2(IG,I)=C2(IG,I)+DCMPLX(-T1*TI,T1*TR)
                 ENDDO
               ENDIF
               IF(TKPNT.AND.GEQ0) C2(1+NGW,I)=DCMPLX(0.D0,0.D0)
             ENDDO
            ENDIF
          ENDDO
        ELSE
C     ==--------------------------------------------------------------==
C     ==  BACHELET HAMANN SCHLUTER                                    ==
C     ==--------------------------------------------------------------==
          DO IV=1,NGH(IS)
            IF(TFDIST) THEN
              CALL AZZERO(DDIA,IMAGP*NAX*NSTATE)
              DO I=1,NSTATE
                IF(I.GE.NST12(MEPOS,1).AND.I.LE.NST12(MEPOS,2)) THEN
                  IAA=I-NST12(MEPOS,1)+1
                  CALL DCOPY(IMAGP*NA(IS),
     &                       FNL2(1,ISA0+1,IV,IAA,IKIND),1,DDIA(1,I),1)
                ENDIF
              ENDDO
              CALL GLOSUM(IMAGP*NAX*NSTATE,DDIA)
            ELSE
              DO I=1,NSTATE
                CALL DCOPY(IMAGP*NA(IS),FNL(1,ISA0+1,IV,I,IKIND),
     &                     1,DDIA(1,I),1)
              ENDDO
            ENDIF
            IF(NGWK.GT.0) THEN
             IF(TKPNT) THEN
               CALL ZGEMM('N','N',NGWK,NSTATE,NA(IS),ZONE,
     &                  EIGKR(1,ISA0+1,IKIND),NGWK,DDIA,NAX,
     &                  ZZERO,AUXC,NGWK)
             ELSE
               CALL DGEMM('N','N',2*NGW,NSTATE,NA(IS),1.D0,
     &                  EIGR(1,ISA0+1),2*NGW,DDIA,NAX,0.0D0,AUXC,2*NGW)
             ENDIF
!$OMP parallel do private(I,IG,CTM,FFI,T)
#ifdef __SR8000
*poption parallel, tlocal(I,IG,CTM,FFI,T)
#endif 
             DO I=1,NSTATE
               CTM=(0.0D0,-1.0D0)**NGHTOL(IV,IS)
               FFI=F(I)
               IF(FFI.LT.1.D-5) FFI=1.0D0
               T=-FFI*WSG(IS,IV)
               CTM=CTM*T
#ifdef  _BGL  
!IBM*   UNROLL(10)
!IBM*   ASSERT (ITERCNT(1000))
#endif
               DO IG=1,NGWK
                 AUXC(IG,I)=CTM*AUXC(IG,I) 
                 C2(IG,I)=C2(IG,I)+TWNL(IG,IV,IS,IKIND)*AUXC(IG,I)
               ENDDO
             ENDDO
            ENDIF
          ENDDO
C     ==--------------------------------------------------------------==
        ENDIF
        ISA0=ISA0+NA(IS)
      ENDDO
C     NL PP FORCE CONTRIBUTION TO CAS22 METHOD
      IF(TLSE .AND. TCAS22) CALL FCASNL(C2,AUXC,DDIA)
C
      CALL TIHALT('   FNONLOC',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_FNONLOC(IL_AUXC,IL_DDIA,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NGWK NAX
C     Arguments
      INTEGER IL_AUXC,IL_DDIA,NSTATE
C     ==--------------------------------------------------------------==
      IL_AUXC=2*NGWK*NSTATE+2
      IL_DDIA=2*NAX*NSTATE+2
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FCASNL(C2,AUXC,DDIA)
C     ==--------------------------------------------------------------==
C     == CALCULATES THE NON-LOCAL PP CONTRIBUTION OF THE HAMILTONIAN  ==
C     == SPECIAL TERMS FOR THE CAS22 METHOD                           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sgpp.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'ener.inc'
C     Arguments
      COMPLEX*16 C2(NGWK,*),AUXC(NGWK)
      REAL*8     DDIA(NAX)
C     Variables
      DIMENSION  EIGR(NGW,NAT),TWNL(NGWK,NHXS,NSX),
     &           EIGKR(NGWK,NAT),FNL2(NAT,NHXS,LDF2)
      COMPLEX*16 CTM
      REAL*8     FNL(LDF1,NHXS,*),DFNL(*)
      REAL*8     DD,T1,TR,TI,T,CTEST,CA,SA
      INTEGER    ISA0,IV,IS,IA,ISA,JV,L,
     &           KI,LI,L2,LJ,KJ,IG,IAA
C     ==--------------------------------------------------------------==
      IF(NGWK.EQ.0) RETURN
      IF(TKPNT) CALL STOPGM("FCASNL","NOT IMPLEMENTED")
      IF(TIVAN) CALL STOPGM("FCASNL","NOT IMPLEMENTED")
      CA=DCOS(CASANG)
      SA=DSIN(CASANG)
      ISA0=0
      DO IS=1,NSP
        IF(TSGP(IS)) THEN
C         Stefan Goedeckers PP
          DO IV=1,NGH(IS)
C           STATE=ALPHA
#ifdef  _BGL 
            CALL AZZERO(AUXC,2*NGWK)
#else
            CALL ZAZZERO(AUXC,NGWK)
#endif
            DO IA=1,NA(IS)
              ISA=ISA0+IA
              L=NGHTOL(IV,IS)+1
              KI=LFVAL(IV,IS)
              LI=LPVAL(IV,IS)
              DD=0.0D0
#ifdef  _BGL  
!IBM*   ASSERT (ITERCNT(10000))
#endif
              DO JV=1,NGH(IS)
                L2=NGHTOL(JV,IS)+1
                LJ=LPVAL(JV,IS)
                IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                  KJ=LFVAL(JV,IS)
                  IF(TFDIST) THEN
                    IF(IATPE(ISA).EQ.MEPOS) THEN
                      IAA=ISA-IPEPT(1,MEPOS)+1
                      DD=DD+(-0.5D0*CA*SA*FNL(IAA,JV,IBETA)+
     *                       SA*SA*FNL(IAA,JV,IALPHA))*HLSG(KI,KJ,L,IS)
                    ENDIF
                  ELSE
                    DD=DD+(-0.5D0*CA*SA*FNL(ISA,JV,IBETA)+
     *                     SA*SA*FNL(ISA,JV,IALPHA))*HLSG(KI,KJ,L,IS)
                  ENDIF
                ENDIF
              ENDDO
              IF(TFDIST) CALL GLOSUM(1,DD)
              CALL DAXPY(2*NGW,DD,EIGR(1,ISA),1,AUXC(1),1)
            ENDDO
            CTM=-(0.0D0,-1.0D0)**NGHTOL(IV,IS)
C           MAKE USE OF THE SPECIAL STRUCTURE OF CTM
            IF(DABS(DREAL(CTM)).GT.0.01D0) THEN
C             CTM IS REAL
!$OMP  parallel do private(IG,T1,TR,TI) 
#ifdef  _BGL  
!IBM*   ASSERT (ITERCNT(1000))
!IBM*   INDEPENDENT, NEW(T1,TR,TI)
#endif
              DO IG=1,NGWK
                T1=TWNL(IG,IV,IS)*DREAL(CTM)
                TR=DREAL(AUXC(IG))
                TI=DIMAG(AUXC(IG))
                C2(IG,IALPHA)=C2(IG,IALPHA)+DCMPLX(T1*TR,T1*TI)
              ENDDO
            ELSE
C             CTM IS IMAGINARY
!$OMP  parallel do private(IG,T1,TR,TI) 
#ifdef  _BGL  
!IBM*   ASSERT (ITERCNT(1000))
!IBM*   INDEPENDENT, NEW(T1,TR,TI)
#endif
              DO IG=1,NGWK
                T1=TWNL(IG,IV,IS)*DIMAG(CTM)
                TR=DREAL(AUXC(IG))
                TI=DIMAG(AUXC(IG))
                C2(IG,IALPHA)=C2(IG,IALPHA)+DCMPLX(-T1*TI,T1*TR)
              ENDDO
            ENDIF
C           STATE=BETA
#ifdef  _BGL 
            CALL AZZERO(AUXC,2*NGWK)
#else
            CALL ZAZZERO(AUXC,NGWK)
#endif
            DO IA=1,NA(IS)
              ISA=ISA0+IA
              L=NGHTOL(IV,IS)+1
              KI=LFVAL(IV,IS)
              LI=LPVAL(IV,IS)
              DD=0.0D0
#ifdef  _BGL  
!IBM*   ASSERT (ITERCNT(10000))
#endif
              DO JV=1,NGH(IS)
                L2=NGHTOL(JV,IS)+1
                LJ=LPVAL(JV,IS)
                IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                  KJ=LFVAL(JV,IS)
                  IF(TFDIST) THEN
                    IF(IATPE(ISA).EQ.MEPOS) THEN
                      IAA=ISA-IPEPT(1,MEPOS)+1
                      DD=DD+(-0.5D0*CA*SA*FNL(IAA,JV,IALPHA)-
     *                       SA*SA*FNL(IAA,JV,IBETA))*HLSG(KI,KJ,L,IS)
                    ENDIF
                  ELSE
                    DD=DD+(-0.5D0*SA*CA*FNL(ISA,JV,IALPHA)-
     *                     SA*SA*FNL(ISA,JV,IBETA))*HLSG(KI,KJ,L,IS)
                  ENDIF
                ENDIF
              ENDDO
              IF(TFDIST) CALL GLOSUM(1,DD)
              CALL DAXPY(2*NGW,DD,EIGR(1,ISA),1,AUXC(1),1)
            ENDDO
            CTM=-(0.0D0,-1.0D0)**NGHTOL(IV,IS)
C           MAKE USE OF THE SPECIAL STRUCTURE OF CTM
            IF(DABS(DREAL(CTM)).GT.0.01D0) THEN
C             CTM IS REAL
!$OMP  parallel do private(IG,T1,TR,TI) 
#ifdef  _BGL  
!IBM*   ASSERT (ITERCNT(1000))
!IBM*   INDEPENDENT, NEW(T1,TR,TI)
#endif
              DO IG=1,NGWK
                T1=TWNL(IG,IV,IS)*DREAL(CTM)
                TR=DREAL(AUXC(IG))
                TI=DIMAG(AUXC(IG))
                C2(IG,IBETA)=C2(IG,IBETA)+DCMPLX(T1*TR,T1*TI)
              ENDDO
            ELSE
C             CTM IS IMAGINARY
!$OMP  parallel do private(IG,T1,TR,TI) 
#ifdef  _BGL  
!IBM*   ASSERT (ITERCNT(1000))
!IBM*   INDEPENDENT, NEW(T1,TR,TI)
#endif
              DO IG=1,NGWK
                T1=TWNL(IG,IV,IS)*DIMAG(CTM)
                TR=DREAL(AUXC(IG))
                TI=DIMAG(AUXC(IG))
                C2(IG,IBETA)=C2(IG,IBETA)+DCMPLX(-T1*TI,T1*TR)
              ENDDO
            ENDIF
          ENDDO
        ELSE
C       BACHELET HAMANN SCHLUTER
#ifdef  _BGL  
!IBM*   ASSERT (ITERCNT(10000))
#endif
          DO IV=1,NGH(IS)
C           STATE=ALPHA
            IF(TFDIST) THEN
              CALL AZZERO(DDIA,NA(IS))
              IF(IBETA.GE.NST12(MEPOS,1).AND.
     *           IBETA.LE.NST12(MEPOS,2)) THEN
                IAA=IBETA-NST12(MEPOS,1)+1
                CALL DCOPY(NA(IS),FNL2(ISA0+1,IV,IAA),1,DDIA,1)
                CALL DSCAL(NA(IS),-0.5D0*CA*SA,DDIA,1)
              ENDIF
              IF(IALPHA.GE.NST12(MEPOS,1).AND.
     *           IALPHA.LE.NST12(MEPOS,2)) THEN
                IAA=IALPHA-NST12(MEPOS,1)+1
                CALL DAXPY(NA(IS),SA*SA,FNL2(ISA0+1,IV,IAA),1,DDIA,1)
              ENDIF
              CALL GLOSUM(NA(IS),DDIA)
            ELSE
              CALL DCOPY(NA(IS),FNL(ISA0+1,IV,IBETA),1,DDIA,1)
              CALL DSCAL(NA(IS),-0.5D0*CA*SA,DDIA,1)
              CALL DAXPY(NA(IS),SA*SA,FNL(ISA0+1,IV,IALPHA),1,DDIA,1)
            ENDIF
            CALL DGEMV('N',2*NGW,NA(IS),1.0D0,EIGR(1,ISA0+1),2*NGW,
     *           DDIA,1,0.0D0,AUXC(1),1)
            T=-WSG(IS,IV)
            CTM=(0.0D0,-1.0D0)**NGHTOL(IV,IS)
            CTEST=DREAL(CTM)
C           MAKE USE OF THE SPECIAL STRUCTURE OF CTM
            IF(DABS(CTEST).GT.1.D-2) THEN
C             CTM IS REAL
              CTM=T*CTM
!$OMP  parallel do private(IG,T1,TR,TI) 
#ifdef  _BGL  
!IBM*   ASSERT (ITERCNT(1000))
!IBM*   INDEPENDENT, NEW(T1,TR,TI)
#endif
              DO IG=1,NGWK
                T1=TWNL(IG,IV,IS)*DREAL(CTM)
                TR=DREAL(AUXC(IG))
                TI=DIMAG(AUXC(IG))
                C2(IG,IALPHA)=C2(IG,IALPHA)+DCMPLX(T1*TR,T1*TI)
              ENDDO
            ELSE
C             CTM IS IMAGINARY
              CTM=T*CTM
!$OMP  parallel do private(IG,T1,TR,TI) 
#ifdef  _BGL  
!IBM*   ASSERT (ITERCNT(1000))
!IBM*   INDEPENDENT, NEW(T1,TR,TI)
#endif
              DO IG=1,NGWK
                T1=TWNL(IG,IV,IS)*DIMAG(CTM)
                TR=DREAL(AUXC(IG))
                TI=DIMAG(AUXC(IG))
                C2(IG,IALPHA)=C2(IG,IALPHA)+DCMPLX(-T1*TI,T1*TR)
              ENDDO
            ENDIF
C           STATE=BETA
            IF(TFDIST) THEN
              CALL AZZERO(DDIA,NA(IS))
              IF(IALPHA.GE.NST12(MEPOS,1).AND.
     *           IALPHA.LE.NST12(MEPOS,2)) THEN
                IAA=IALPHA-NST12(MEPOS,1)+1
                CALL DCOPY(NA(IS),FNL2(ISA0+1,IV,IAA),1,DDIA,1)
                CALL DSCAL(NA(IS),-0.5D0*CA*SA,DDIA,1)
              ENDIF
              IF(IBETA.GE.NST12(MEPOS,1).AND.
     *           IBETA.LE.NST12(MEPOS,2)) THEN
                IAA=IBETA-NST12(MEPOS,1)+1
                CALL DAXPY(NA(IS),-SA*SA,FNL2(ISA0+1,IV,IAA),1,DDIA,1)
              ENDIF
              CALL GLOSUM(NA(IS),DDIA)
            ELSE
              CALL DCOPY(NA(IS),FNL(ISA0+1,IV,IALPHA),1,DDIA,1)
              CALL DSCAL(NA(IS),-0.5D0*SA*CA,DDIA,1)
              CALL DAXPY(NA(IS),-SA*SA,FNL(ISA0+1,IV,IBETA),1,DDIA,1)
            ENDIF
            CALL DGEMV('N',2*NGW,NA(IS),1.0D0,EIGR(1,ISA0+1),2*NGW,
     *           DDIA,1,0.0D0,AUXC(1),1)
            T=-WSG(IS,IV)
            CTM=(0.0D0,-1.0D0)**NGHTOL(IV,IS)
            CTEST=DREAL(CTM)
C           MAKE USE OF THE SPECIAL STRUCTURE OF CTM
            IF(DABS(CTEST).GT.1.D-2) THEN
C             CTM IS REAL
              CTM=T*CTM
!$OMP  parallel do private(IG,T1,TR,TI) 
#ifdef  _BGL  
!IBM*   ASSERT (ITERCNT(1000))
!IBM*   INDEPENDENT, NEW(T1,TR,TI)
#endif
              DO IG=1,NGWK
                T1=TWNL(IG,IV,IS)*DREAL(CTM)
                TR=DREAL(AUXC(IG))
                TI=DIMAG(AUXC(IG))
                C2(IG,IBETA)=C2(IG,IBETA)+DCMPLX(T1*TR,T1*TI)
              ENDDO
            ELSE
C             CTM IS IMAGINARY
              CTM=T*CTM
!$OMP  parallel do private(IG,T1,TR,TI) 
#ifdef  _BGL  
!IBM*   ASSERT (ITERCNT(1000))
!IBM*   INDEPENDENT, NEW(T1,TR,TI)
#endif
              DO IG=1,NGWK
                T1=TWNL(IG,IV,IS)*DIMAG(CTM)
                TR=DREAL(AUXC(IG))
                TI=DIMAG(AUXC(IG))
                C2(IG,IBETA)=C2(IG,IBETA)+DCMPLX(-T1*TI,T1*TR)
              ENDDO
            ENDIF
          ENDDO
C     ==--------------------------------------------------------------==
        ENDIF
        ISA0=ISA0+NA(IS)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
