C     ==================================================================
      SUBROUTINE FNONLOC_P(C0,PSI,SCR,LSCR,
     &      H1NL,F,NSTATE,K_,ISP_,ISA_,IKIND)
C     ==--------------------------------------------------------------==
C     == Computes H1NL (from FNL00), the perturbation needed for      ==
C     == the phonon perturbations.                                    ==
C     == the derivative of the nonlocal pseudopotential with respect  ==
C     == to the ionic position R(is,isa), applied to c0.              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nlps.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sgpp.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc' 
      INCLUDE 'geq0.inc'
      INCLUDE 'response_p.inc'
C     Arguments
      INTEGER    LSCR,ISA_,ISP_,K_,NSTATE,IKIND
      COMPLEX*16 H1NL(NGWK,NSTATE),C0(NGWk,NSTATE),PSI(*)
      REAL*8     F(NSTATE),SCR(LSCR)
      real*8     fnl,dfnl       ! from sfac.inc
      DIMENSION  EIGR(NGW,NAT),TWNL(NGWK,NHXS,NSX,NKPNT),
     &    EIGKR(NGWK,NAT,NKPNT),
     &    fnl00 (IMAGP,NAT,NHXS,NSTATE, NKPNT),
     &    dfnl00(IMAGP,NAT,NHXS,3,NDFNL,NKPNT)
C     Variables
      COMPLEX*16 CTM,G,i_to_l,expigr
      REAL*8     DLOC(NHXS,NSTATE),Sum_fnl,Sum_dfnl,tw
      INTEGER    ISUB,istate,istate_rel,IV,JV,IG
      POINTER    (IP_DLOC,DLOC) ! the DDIA in fnonloc.F
      LOGICAL    match
C     ==--------------------------------------------------------------==
      CALL TISET(' FNONLOC_P',ISUB)
      IF (IKIND .NE. 1)
     &    CALL STOPGM('F_NL_P','iKind not supported.')
#ifdef  _BGL 
      CALL AZZERO(H1NL,2*NGWK*NSTATE)
#else
      CALL ZAZZERO(H1NL,NGWK*NSTATE)
#endif
C     ==--------------------------------------------------------------==
C     Prepare the DFNL on all processors:
      CALL MEMORY(IP_DLOC,NSTATE*NHXS,'DLOC')
      CALL AZZERO(   DLOC,NSTATE*NHXS)
      DO istate=NST12(MEPOS,1),NST12(MEPOS,2)
        istate_rel = istate - NST12(MEPOS,1)+1
        DO IV=1,NGH(ISP_)
          DLOC(IV,istate)=DFNL00(1,ISA_,IV,K_,istate_rel,IKIND) 
        END DO
      END DO
#ifdef PARALLEL
      CALL GLOSUM(NSTATE*NHXS,DLOC)
#endif

C     ==--------------------------------------------------------------==
C     ==  VANDERBILT                                                  ==
C     ==--------------------------------------------------------------==
      IF(TVAN(ISP_)) THEN
        call STOPGM('F_NL_P','Vanderbilt PP not implemented')
      ELSEIF (TSGP(ISP_)) THEN


C     ==--------------------------------------------------------------==
C     ==  STEFAN GOEDECKER                                            ==
C     ==--------------------------------------------------------------==
c     (ds 3/2000)
        if (tkpnt) call stopgm('F_NL_P','K-points are complicated.')
        if (TFDIST) call stopgm('F_NL_P',
     &      'distributed DFNLs not supported yet.')

        do istate=1,nstate
          do iv=1,ngh(isp_)
            Sum_FNL  = 0d0
            Sum_DFNL = 0d0
#ifdef  _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
            do jv=1,ngh(isp_)
              match =    (NGHTOL(iv,isp_) .eq. NGHTOL(jv,isp_)) 
     &            .AND. ( LPVAL(iv,isp_) .eq.  LPVAL(jv,isp_))

              if (match) then
                Sum_FNL = Sum_FNL
     &              + FNL00(1,isa_,jv,istate,ikind)
     &              * HLSG( LFVAL(iv,isp_), LFVAL(jv,isp_),
     &              NGHTOL(iv,isp_)+1, isp_)
                Sum_DFNL = Sum_DFNL
     &              + DLOC(jv,istate)
     &              * HLSG( LFVAL(iv,isp_), LFVAL(jv,isp_),
     &              NGHTOL(iv,isp_)+1, isp_)
              endif
            enddo               ! jv (projectors)
c     if (tfdist) call glosum(1,dd)!  for later

C     Second, prepare i^^L, where L is that of the projector "iv":
            i_to_l = - DCMPLX(0d0,-1d0)**NGHTOL(iv,isp_)

C     Finally, put everything together:
C     H1nl += [ FNL*(-iG) + DFNL ]  * exp(iGR)  * i^L * TWnl * HLSM:
!$OMP parallel do private(ig,expigr,G,tw) shared(i_to_l)
#ifdef  _BGL 
!IBM* ASSERT (ITERCNT(1000))
#endif
            DO ig=1,ngw
              expigr = EIGR(ig,isa_)
              G      = DCMPLX(0.D0,-tpiba*GK(K_,IG))
              tw     = f(istate) * TWNL(ig,iv,isp_,ikind)
              H1NL(ig,istate) = H1NL(ig,istate)
     &            + (Sum_FNL * G + Sum_DFNL)
     &            * expigr * i_to_l * tw
            ENDDO               ! ig
          enddo                 ! iv (projectors)
        enddo                   ! istate
      ELSE


C     ==--------------------------------------------------------------==
C     ==  BACHELET HAMANN SCHLUTER                                    ==
C     ==--------------------------------------------------------------==
        DO istate=1,NSTATE
#ifdef  _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
          DO IV=1,NGH(ISP_)
            i_to_l = (0.0D0,-1.0D0)**NGHTOL(IV,ISP_)
            CTM    = - i_to_l * F(istate)*WSG(ISP_,IV)
#ifdef  _BGL 
!IBM* ASSERT (ITERCNT(1000))
#endif
            DO IG=1,NGWK
              G=DCMPLX(0.D0,-tpiba*GK(K_,IG))
              
              H1NL(IG,istate)=H1NL(IG,istate)+
     &            TWNL(IG,IV,ISP_,IKIND) * CTM * EIGR(IG,ISA_)
     &            * (DLOC(IV,istate)+ G*FNL00(1,ISA_,IV,istate,IKIND))
              IF(TKPNT.AND.GEQ0) H1NL(1+NGW,istate)=DCMPLX(0.D0,0.D0)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
      CALL FREEM(IP_DLOC)
      CALL TIHALT(' FNONLOC_P',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
