C Author: Patrik Mauer
C Revision: Ivano Tavernelli (Dec. 2010)
C     ==================================================================
      subroutine fm_get_size_of_qfit_arrays(nframes, nr_nn_max, nr_qm)
C     ==--------------------------------------------------------------==
        implicit none
        include 'fileopen.inc'
        include 'forcematch.inc'
C       Arguments
C       nr of frames, max nr. of NN atoms, and nr of QM atoms
        integer nframes, nr_nn_max, nr_qm
C       Variables
C       QM or MM identifier
        character*2     qm_or_mm
C       iterators, temporaries, etc.
        integer iframe, inn, inr_nn, iqm, ia, ie
C       dummies for reading files
        logical ferror
        integer idum
        real*8  posdum(3)
C       Parameters
C       file units
        integer, parameter ::   pipu = 72

C get nr of frames from TRAJECTORY_PIP
        call FILEOPEN(pipu, fm_pip_ref_filename, FO_OLD, ferror)
        if (ferror) call STOPGM('fm_read_forces','error opening file '
     $                          // fm_pip_ref_filename)
        call xstring(fm_pip_ref_filename, ia, ie)
        write(6,'(A)') '  fm  Determine nr of frames'
        write(6,'(A,A,A)',ADVANCE='NO') "  fm  Reading file ", 
     $                                  fm_pip_ref_filename(ia:ie) 
     $                                  // ".."
       
        nr_nn_max = -1
        iframe = 0
        do
          iqm = 0
          read(pipu, *, end = 310, err = 311) inr_nn
          if (inr_nn .ge. nr_nn_max) nr_nn_max = inr_nn
          do inn = 1, inr_nn
            read(pipu, *, end=311, err=311) posdum, qm_or_mm
            if (qm_or_mm .eq. 'QM') iqm = iqm + 1
          enddo
          if (iframe .eq. 0) then
            nr_qm = iqm
          else
            if (nr_qm .ne. iqm) then
              call STOPGM('fm_read_ref_forces', 'inconsistent nr. of QM'
     $                    // ' atoms in file ' // fm_pip_ref_filename)
            endif
          endif
          iframe = iframe + 1
        enddo

C reached end of file
 310    continue
        call FILECLOSE(pipu)
        nframes = iframe
        write(6, '(A)') "..done!"

        write(6,'(A, i8)') '  fm  Nr. of frames:       ', nframes
        write(6,'(A, i8)') '  fm  Nr. of QM atoms:     ', nr_qm
        write(6,'(A, i8)') '  fm  Max nr. of NN atoms: ', nr_nn_max
        return

C error while reading file nn_ref_filename
 311    continue
        call STOPGM('fm_read_ref_forces', 'error while reading file ' 
     $              // fm_pip_ref_filename)

      end subroutine fm_get_size_of_qfit_arrays
C
C     ==================================================================
      subroutine qfit_optimize_weights(nframes, nr_nn_max, nr_nn, r_nn,
     $                                 v_nn, f_nn, nr_qm, r_qm,
     $                                 q_hirshfeld)
C     ==--------------------------------------------------------------==
C     Try to find optimal weights for the charge fitting procedure
C     ==================================================================
        implicit none
        include 'forcematch.inc'
C       Arguments
C       nr of frames
        integer         nframes 
C       maximal nr. of NN atoms in a frame
        integer         nr_nn_max
C       nr of NN atoms for each frame
        integer         nr_nn(nframes)
C       coordinates of NN atoms 
        real*8          r_nn(3, nr_nn_max, nframes)
C       potential on NN atoms 
        real*8          v_nn(nr_nn_max, nframes)
C       field on NN atoms 
        real*8          f_nn(3, nr_nn_max, nframes)
C       nr of QM atoms
        integer         nr_qm
C       coordinates of QM atoms 
        real*8          r_qm(3, nr_qm, nframes)
C       average hirshfeld charge on QM atoms
        real*8          q_hirshfeld(nr_qm)
C       local variables
C       iterators and temporaries
        real*8          wv, wf
        integer         iframe
C
C        write(6, '(A)') '================ ERROR ====================='
C        write(6, '(A)') ' finding optimal weights for charge fitting'
C        write(6, '(A)') ' is not yet implemented!'
C        call STOPGM('qfit_optimize_weights', 'not yet implemented')
C
        fm_wtot = 1.0D+09
        fm_wq = 1.d0 / dsqrt(sum(q_hirshfeld**2))
C
        wv = 0.d0
        wf = 0.d0
        do iframe = 1, nframes
          wv = wv + sum(v_nn(1:nr_nn(iframe), iframe)**2)
          wf = wf + sum(f_nn(:, 1:nr_nn(iframe), iframe)**2)
        enddo
C
C_DEBUG[
C        write(6,*) "deb: sum(q_hirsh**2), wv, wf"
C        write(6,*) sum(q_hirshfeld**2), wv, wf
        fm_wq = 0.01d0
C_DEBUG]
        fm_wv = 1.d0 / dsqrt(wv)
        fm_wf = 1.d0 / dsqrt(wf)
C        
        return
      end subroutine qfit_optimize_weights
C
C     ==================================================================
      subroutine qfit(nframes, nr_nn_max, nr_nn, r_nn, v_nn, f_nn, 
     $                nr_qm, r_qm, q_hirshfeld, equiv, q_opt)
C     ==--------------------------------------------------------------==
C     qfit is the main routine for charge fitting. The optimized charges are
C     returned in array q_opt. The fitting works by solving an
C     overdetermined system of linear equations (SLE) in the least-squares 
C     sense.
C     The SLE is I * q_opt = T, where:
C     - q_opt is a vector containing the optimized charges, with equivalencies
C     being taken into account.
C     - T is the "target vector", i.e. a vector containing the potential and
C     field on the NN atoms, plus Hirshfeld (or other) charges and the total
C     charge for restraining.
C     - I is the "influence matrix" containing all the factors 1/rij and
C     1/rij**2 for the potential and field, resp.
C     ==================================================================
        implicit none
        include 'system.h'
        include 'mm_dim.inc'
        include 'forcematch.inc'
C       Arguments
C       nr of frames
        integer         nframes 
C       maximal nr. of NN atoms in a frame
        integer         nr_nn_max
C       nr of NN atoms for each frame
        integer         nr_nn(nframes)
C       coordinates of NN atoms 
        real*8          r_nn(3, nr_nn_max, nframes)
C       potential on NN atoms 
        real*8          v_nn(nr_nn_max, nframes)
C       field on NN atoms 
        real*8          f_nn(3, nr_nn_max, nframes)
C       nr of QM atoms
        integer         nr_qm
C       coordinates of QM atoms 
        real*8          r_qm(3, nr_qm, nframes)
C       average hirshfeld charge on QM atoms
        real*8          q_hirshfeld(nr_qm)
C       equivalencies
        integer         equiv(nr_qm)
C       optimized charges
        real*8          q_opt(nr_qm)
C       Variables
C       nr of atoms a unique charge represents
        integer, allocatable :: n_eq(:)
C       target vector and "influence" matrix
        real*8, allocatable ::  trget(:), infl_mat(:, :)
C       charges to restrain to
        real*8, allocatable ::  q_restrain(:)
C       nr of unique charges (i.e., atom types, sort of)
        integer         n_unique
C       index array for unique charges/atoms
        integer         unique(nr_qm)
C       total charge of QM system
        real*8          qtot
C       size of target vector
        integer         trget_size
C       rms of potential and field
        real*8          vrms, frms
C       for LAPACK
        integer         lwork, info
        real*8, allocatable ::  work(:)
        character(40)           err_msg
C       iterators
        integer                 ieq, iqm
C       Manuel: printing stuff for debugging
        integer debug_cpmd_indx,debug_grms_indx,debug_unique_indx
        integer  debug_equiv2,i
        real*8   debug_weight

        write(6,'(A)') '  fm '
        write(6, '(A)') '  fm  Starting QFIT'
        write(6, '(A, i8)') '  fm  nr. of frames: ', nframes
C        write(6, '(A)') '  fm  Weights for fitting are: '
C        write(6, '(A, e9.3)') '  fm  WV (Potential)        = ', fm_wv
C        write(6, '(A, e9.3)') '  fm  WF (Field)            = ', fm_wf
C        write(6, '(A, e9.3)') '  fm  WQ (Charge restraint) = ', fm_wq
C        write(6, '(A, e9.3)') '  fm  WTOT (Total charge)   = ', fm_wtot


C compute nr of unique charges
        if (fm_equivalence .or. fm_capping) then
          n_unique = 0
          do iqm = 1, nr_qm
            if (qm_equiv(iqm).eq.iqm .and. .not.fm_iscap(iqm)) then
              n_unique = n_unique + 1
            endif
          enddo
        else
          n_unique = nr_qm
          do iqm = 1, nr_qm
            equiv(iqm) = iqm
          enddo
        endif
        write(6, *) ' fm  Nr. of unique charges: ', n_unique
        allocate(q_restrain(n_unique))
        allocate(n_eq(n_unique))
C        call MEMORY(ip_q_restrain, n_unique, 'qfit unique charges')
C        call MEMORY(ip_n_eq, n_unique, 'qfit n_eq')

C compute total charge as sum over Hirshfeld charges
        qtot = sum(q_hirshfeld)
C set up equiv array from fm_equiv (read in input file), this also
C handles capping hydrogens,
C the charges that are kept fixed
C and the individual weights for the charge fitting
        call qfit_setup_equiv(nr_qm, n_unique, q_hirshfeld, q_restrain, 
     $                        equiv, n_eq, unique)
C find optimal weights if requested
        if (fm_optimize_weights) then
          call qfit_optimize_weights (nframes, nr_nn_max, nr_nn, r_nn,
     $                                v_nn, f_nn, nr_qm, r_qm,
     $                                q_hirshfeld, n_unique)
        endif
        write(6, '(A)') '  fm  Weights for fitting are: '
        write(6, '(A, e9.3)') '  fm  WV (Potential)        = ', fm_wv
        write(6, '(A, e9.3)') '  fm  WF (Field)            = ', fm_wf
        write(6, '(A)') '  fm  WQ '
        write(6, '(2A)') 
     & '  fm    cpmd         gromos   ',
     &  '  unique(i)     equiv to        weight           target'
        do i=1,nr_qm
          debug_cpmd_indx=i
          debug_grms_indx=NAT_grm(debug_cpmd_indx)
          debug_unique_indx=unique(i)
          debug_equiv2=equiv(i)
          debug_weight=fm_wq_unique(debug_unique_indx)
          write(6, '(i0,1x,i0,1x,i0,1x,i0,1x,f15.8,1x,f15.8)')
     &   debug_cpmd_indx,debug_grms_indx,debug_unique_indx,debug_equiv2,
     &   debug_weight,q_restrain(debug_unique_indx)
        end do
        write(6, '(A, e9.3)') '  fm  WTOT (Total charge)   = ', fm_wtot
C setup target vector and "influence" matrix
C size of target vector is 1+3=4 times the total nr. of NN atoms (1 for
C potential, 3 for field), plus the nr. of charges to restrain to, plus
C 1 for the total charge
C
        write(6, '(A, i10)') '  fm  Total nr of NN atoms:      ',
     $                      sum(nr_nn)
        trget_size = 4 * sum(nr_nn) + n_unique + 1
        write(6, '(A, i10)') '  fm  Size of the target vector: ',
     $                      trget_size
        write(6, '(A)') '  fm  Allocating target vector and influence '
     $                  // 'matrix'
        allocate(trget(trget_size))
        allocate(infl_mat(trget_size, n_unique))
        write(6, '(A)', ADVANCE='NO') '  fm  Setting up target vector..'
        call setup_qfit_target(trget_size, trget, nframes, nr_nn_max, 
     $                         nr_nn, v_nn, f_nn, n_unique, n_eq, 
     $                         q_restrain, qtot)
        write(6, '(A)') '..done!'
        write(6, '(A)', ADVANCE='NO') '  fm  Setting up influence '
     $                                   // 'matrix..'
        call setup_qfit_infl_mat(trget_size, infl_mat, nframes, 
     $                           nr_nn_max, nr_nn, r_nn, nr_qm, r_qm, 
     $                           n_unique, n_eq, unique)
        write(6, '(A)') '..done!'
CDEBUG[
C        write(998, *) trget
C        do ieq=1, trget_size
C          write(999, *) infl_mat(ieq, :)
C        enddo
CDEBUG]
C find least squares solution to overdetermined system of linear 
C equations using the routine DGELS from LAPACK
        lwork = min(trget_size,n_unique) + 32*max(trget_size,n_unique)
        allocate(work(lwork))
        write(6, '(A)', ADVANCE='NO') '  fm  Solving..'
        call DGELS('N', trget_size, n_unique, 1, infl_mat, trget_size,
     $             trget, trget_size, work, lwork, info)
        if (info .ne. 0) then
          write(err_msg, '(A, i5)') ' DGELS returned info = ', info
          call STOPGM('QFIT', err_msg)
        endif
        write(6, '(A)') '..done!'
        deallocate(work)

C DGELS returns optimal parameters as the first elements of array trget:
C trget(1:n_unique)
C put charges back into q_opt for each QM atom
        do iqm = 1, nr_qm
          if (fm_iscap(iqm)) then
            q_opt(iqm) = 0.d0
          else
            if (unique(iqm) .lt. 0) then
              write(6, *) '  fm Apparently we are trying to do 
     &                   something we'
     $                   // ' should not to a capping H'
              write(6, *) '  fm iqm is: ', iqm
              call STOPGM('qfit',' unique(iqm) is -1')
            endif
            q_opt(iqm) = trget(unique(iqm))
          endif
        enddo
CDEBUG[
        write(6, '(2A)') '  fm    cpmd         gromos   ',
     &  '  unique(i)     equiv to        q_opt(i)'
        do i=1,nr_qm
          debug_cpmd_indx=i
          debug_grms_indx=NAT_grm(debug_cpmd_indx)
          debug_unique_indx=unique(i)
          debug_equiv2=equiv(i)
          write(6, '(i0,1x,i0,1x,i0,1x,i0,1x,f15.8)')
     &    debug_cpmd_indx,debug_grms_indx,debug_unique_indx,
     &    debug_equiv2,q_opt(i)
        end do
        write(6,*) ' fm  qtot: ', sum(q_opt)
CDEBUG]
C
C compute RMS of potential and field
        call qfit_rms(vrms, frms, nframes, nr_qm, nr_nn_max, nr_nn, 
     $                r_qm, r_nn, v_nn, f_nn, q_opt, q_hirshfeld)

C clean up
C        call FREEM(ip_trget)
C        call FREEM(ip_infl_mat)
C        call FREEM(ip_q_restrain)
C        call FREEM(ip_n_eq)
          deallocate(trget)
          deallocate(infl_mat)
          deallocate(q_restrain)
          deallocate(n_eq)
        return
      end subroutine qfit
C
C     ==================================================================
      subroutine qfit_setup_equiv(nr_qm, n_unique, q_hirshfeld, 
     $                            q_restrain, equiv, n_eq, unique)
C     ==--------------------------------------------------------------==
C     qfit_setup_equiv handles equivalencies for QFIT 
C     it also handles capping hydrogens
C     and the charges that are fixed during the optimization
C     and individual weights for the charge fitting
C     ==================================================================
        implicit none
        include 'system.h'
        include 'mm_dim.inc'
        include 'forcematch.inc'
C       Arguments
C       nr of QM atoms
        integer         nr_qm
C       nr of unique charges
        integer         n_unique
C       average Hirshfeld charges
        real*8          q_hirshfeld(nr_qm)
C       unique charges to restrain to
        real*8          q_restrain(n_unique)
C       equivalencies
        integer         equiv(nr_qm)
C       nr of atoms a charge represents
        integer         n_eq(n_unique)
C       index from QM atom to unique atom
        integer         unique(nr_qm)
C       Variables
C       index to unique arrays
C        integer         iun(nr_qm)
C       iterators
        integer      iqm, jqm, ieq, icap,ifixers,jeq,iqweight
C
C_DEBUG[
C        write(6,*) " now in qfit_setup_equiv"
C        write(6,*) " size equiv, qm_equiv: ", size(equiv),
C     $             size(qm_equiv)
C_DEBUG]
        do iqm = 1, nr_qm
          equiv(iqm) = qm_equiv(iqm)
        enddo
        do ieq = 1, n_unique
          n_eq(ieq) = 0
        enddo
C
C find capping H and add Hirshfeld charge to capped atom
CDEBUG[
C        write(6,*) "  fm q_hirsh 0:", q_hirshfeld
CDEBUG]
        if (fm_capping) then
          do icap = 1, fm_ncap
            iqm = fm_cap(1, icap)
            jqm = fm_cap(2, icap)
            q_hirshfeld(jqm) = q_hirshfeld(jqm) + q_hirshfeld(iqm)
            q_hirshfeld(iqm) = 0.d0
          enddo
        endif
CDEBUG[
        write(6,'(A)') '  fm  put capping H charge on capped atom'
C        write(6,*) "q_hirsh 1:", q_hirshfeld
CDEBUG]
C
C count nr of unique charges
C first find atoms that have no equivalency (i.e., are equivalent to
C themselves) and are not a capping H
        n_unique = 0
        do iqm = 1, nr_qm
          if (equiv(iqm) .eq. iqm .and. .not.fm_iscap(iqm)) then
            n_unique = n_unique + 1
            unique(iqm) = n_unique
            n_eq(n_unique) = n_eq(n_unique) + 1 
          endif
        enddo
C now atoms that are equivalent to some other
        do iqm = 1, nr_qm
          if (equiv(iqm) .ne. iqm .and. .not.fm_iscap(iqm)) then
            ieq = unique(equiv(iqm))
            unique(iqm) = ieq
            n_eq(ieq) = n_eq(ieq) + 1
          endif
        enddo
C set uniqe() to -1 for capping H, we can check for that or we'll get a
C segfault...
        if (fm_capping) then
          do iqm = 1, fm_ncap
            unique(fm_cap(1, iqm)) = -1
          enddo
        endif
C
C charge restraint
        call AZZERO(q_restrain, n_unique)
        do iqm = 1, nr_qm
          if (fm_iscap(iqm)) cycle
          ieq = unique(iqm)
            q_restrain(ieq) = q_restrain(ieq) + q_hirshfeld(iqm)
        enddo
        q_restrain = q_restrain / n_eq
C
C Manuel setup the index arrays for the individual weights and
C the fixed charges in cpmd ordering
       if (fm_fix_q) then
          do ifixers=1,fm_n_fix_q
        fm_fixed_q_indx(ifixers)=NAT_cpmd(fm_fixed_q_grm_indx(ifixers))
          end do
          do ifixers=1,fm_n_qw
        fm_wq_indx(ifixers)=NAT_cpmd(fm_wq_grm_indx(ifixers))
          end do
C Manuel now overwrite the restraints for the atoms whose  
C charges are fixed to some value defined in the input
          do ifixers=1,fm_n_fix_q
            iqm=fm_fixed_q_indx(ifixers)
            ieq=unique(iqm)
            q_restrain(ieq) = fm_fixed_q_trgt(ifixers)
          end do
        end if
C Manuel organize the weights on the charges
C fm_wq() holds the weights for the charges on each QM atom
C fm_wq_general holds the weight for these atoms for which 
C no weight was specified in the input
C fm_wq_unique() will hold the weights for all the unique atoms
        do ieq=1,fm_max_qw   
           fm_wq_unique(ieq)=fm_wq_general
        end do   
        do iqweight=1,fm_n_qw
            iqm=fm_wq_indx(iqweight)
            ieq=unique(iqm)
            fm_wq_unique(ieq)=fm_wq(iqm)
        end do    
CDEBUG[
C        write(6,*) " fm  n_eq:", n_eq
C        write(6,*) " fm  uniq: ", unique
C        write(6,*) " fm  q_restr: ", q_restrain(1:n_unique)
        if (fm_fix_q) then
           write(6,*) " fm  wq: ",fm_wq(1:nr_qm)
           write(6,*) " fm  wq_unique: ",fm_wq_unique(1:n_unique)
        end if
CDEBUG]
        return
      end subroutine qfit_setup_equiv
C
C     ==================================================================
      subroutine setup_qfit_infl_mat(trget_size, infl_mat, nframes, 
     $                               nr_nn_max, nr_nn, r_nn, nr_qm, 
     $                               r_qm, n_unique, n_eq, unique)
C     ==--------------------------------------------------------------==
C     setup_qfit_infl_mat sets up the "influence" matrix for qfit
C     ==================================================================
        implicit none
        include 'forcematch.inc'
C       Arguments
C       size of target vector
        integer         trget_size
C       influence matrix
        real*8, target :: infl_mat(trget_size, n_unique)
C       nr of frames
        integer         nframes
C       maximal nr. of NN atoms in a frame
        integer         nr_nn_max
C       nr of NN atoms for each frame
        integer         nr_nn(nframes)
C       coordinates of NN atoms 
        real*8          r_nn(3, nr_nn_max, nframes)
C       nr of QM atoms
        integer         nr_qm
C       coordinates of QM atoms
        real*8          r_qm(3, nr_qm, nframes)
C       nr of unique charges
        integer         n_unique
C       nr of atoms a unique charge represents
        integer         n_eq(n_unique)
C       equivalencies
        integer         unique(nr_qm)
C       Variables
C       matrix section for a single frame
        real*8, pointer :: section(:, :)
C       total nr of NN atoms in all frames
        integer         nr_nn_tot
C       iterators and array boundaries
        integer         iframe, lb, ub, ieq

        call AZZERO(infl_mat, trget_size * n_unique)

C first the factors 1/r and 1/r**2 for potential and field, resp.
        ub = 0
        do iframe = 1, nframes
          lb = ub + 1
          ub = lb + 4*nr_nn(iframe) - 1
          section => infl_mat(lb:ub, :)
          call infl_mat_singleframe(nr_nn(iframe), n_unique, nr_qm,
     $                              section, r_nn(:, :, iframe), 
     $                              r_qm(:, :, iframe), unique)
        enddo

C now add the Hirshfeld charges and the total charge
        nr_nn_tot = sum(nr_nn)
        lb = 4*nr_nn_tot + 1
        do ieq = 1, n_unique
C          infl_mat(lb+ieq-1, ieq) = fm_wq * n_eq(ieq)
           infl_mat(lb+ieq-1, ieq) = fm_wq_unique(ieq) * n_eq(ieq)
        enddo
        lb = lb + n_unique
        if (lb .ne. trget_size) then
          write(6, *) " ERROR when setting up influence matrix"
          write(6, *) " lb should be equal trget_size, but they are:"
          write(6, *) "         lb = ", lb
          write(6, *) " trget_size = ", trget_size
          call STOPGM('setup_qfit_infl_mat', 'lb .ne. trget_size')
        endif
        infl_mat(lb, :) = fm_wtot * n_eq
        return
      end subroutine setup_qfit_infl_mat
C
C     ==================================================================
      subroutine infl_mat_singleframe(nr_nn, n_unique, nr_qm, infl_mat,
     $                                r_nn, r_qm, unique)
C     ==--------------------------------------------------------------==
C     infl_mat_singleframe computes a section of the influence matrix for a
C     single frame and with only potential and field
C     ==================================================================
        implicit none
        include 'forcematch.inc'
C       Arguments
C       nr of NN atoms
        integer         nr_nn
C       nr of unique charges
        integer         n_unique
C       nr of QM atoms
        integer         nr_qm
C       influence matrix
        real*8          infl_mat(4*nr_nn, n_unique)
C       coordinates of NN and QM atoms
        real*8          r_nn(3, nr_nn), r_qm(3, nr_qm)
C       equivalencies
        integer         unique(nr_qm)
C       Variables
C       distance vector, distance and inverse distance (1/r)
        real*8          rijvec(3), rij, rijm1
C       iterators
        integer         inn, jnn, iqm, jqm

        do iqm = 1, nr_qm
          if (fm_iscap(iqm)) cycle
          do inn = 1, nr_nn
            rijvec = r_nn(:, inn) - r_qm(:, iqm)
            rij = sqrt(dot_product(rijvec, rijvec))
            rijm1 = 1.d0 / rij
            rijvec = rijm1 * rijvec
            jnn = 4*(inn-1) + 1
            jqm = unique(iqm)
            infl_mat(jnn, jqm) = infl_mat(jnn, jqm) + fm_wv * rijm1
            infl_mat(jnn+1:jnn+3, jqm) = infl_mat(jnn+1:jnn+3, jqm) 
     $                                 + fm_wf * rijm1**2 * rijvec
          enddo
        enddo
        return
      end subroutine infl_mat_singleframe
C
C     ==================================================================
      subroutine setup_qfit_target(trget_size, trget, nframes, 
     $                             nr_nn_max, nr_nn, v_nn, f_nn, 
     $                             n_unique, n_eq, q_restrain, qtot)
C     ==--------------------------------------------------------------==
C     setup_qfit_target sets up the target vector for qfit
C     ==================================================================
        implicit none
        include 'forcematch.inc'
C       Arguments
C       size of target vector
        integer         trget_size
C       target vector
        real*8          trget(trget_size)
C       nr of frames
        integer         nframes 
C       maximal nr. of NN atoms in a frame
        integer         nr_nn_max
C       nr of NN atoms for each frame
        integer         nr_nn(nframes)
C       potential on NN atoms 
        real*8          v_nn(nr_nn_max, nframes)
C       field on NN atoms 
        real*8          f_nn(3, nr_nn_max, nframes)
C       nr of unique charges
        integer         n_unique
C       nr. of atoms a unique charge represents
        integer         n_eq(n_unique)
C       charges to restrain to 
        real*8          q_restrain(n_unique)
C       total charge of QM system
        real*8          qtot
C       Variables
C       iterators and array bounds
        integer         iframe, inn, it

        call AZZERO(trget, trget_size)

C fill up the target vector, first potential and field on NN atoms
        it = 1
        do iframe = 1, nframes
          do inn = 1, nr_nn(iframe)
            trget(it) = fm_wv * v_nn(inn, iframe)
            trget(it+1:it+3) = fm_wf * f_nn(:, inn, iframe)
            it = it + 4
          enddo
      if (it.gt.trget_size) write(6,*) 'iframe, tgsize, it: ', iframe, 
     $                                  trget_size, it
        enddo
C       Manuel fixed charges        
C        trget(it:it+n_unique-1) = fm_wq * q_restrain * n_eq
        trget(it:it+n_unique-1) = fm_wq_unique(1:n_unique) * 
     &                            q_restrain * n_eq
        it = it + n_unique
      if (it.gt.trget_size) write(6,*) 'tgsize, it: ', trget_size, it
        trget(it) = fm_wtot * qtot
        if (it .ne. trget_size) then
          write(6, *)
          write(6, *) " ========== ERROR ! ============"
          write(6, *) " it and trget_size should be equal, but they are"
          write(6, *) " it: ", it
          write(6, *) " trget_size:", trget_size
          call stopgm('setup_qfit_target', 
     $                'it and trget_size not equal')
        endif
        return
      end subroutine setup_qfit_target
C
C     ==================================================================
      subroutine qfit_rms(vrms, frms, nframes, nr_qm, nr_nn_max, nr_nn, 
     $                    r_qm, r_nn, v_nn_ref, f_nn_ref, q, 
     $                    q_hirshfeld)
C     ==--------------------------------------------------------------==
C     qfit_rms computes the RMSD of the field, potential, and Hirshfeld
C     charges for a set of charges
C     ==================================================================
        implicit none
C       Arguments
C       rms of potential and field
        real*8          vrms, frms
C       nr of frames
        integer         nframes
C       nr of QM atoms
        integer         nr_qm
C       max nr. of NN atoms
        integer         nr_nn_max
C       nr of NN atoms in each frame
        integer         nr_nn(nframes)
C       coordinates of QM atoms
        real*8          r_qm(3, nr_qm, nframes)
C       coordinates of NN atoms
        real*8          r_nn(3, nr_nn_max, nframes)
C       reference potential and field on NN atoms
        real*8          v_nn_ref(nr_nn_max, nframes)
        real*8          f_nn_ref(3, nr_nn_max, nframes)
C       charges on QM atoms
        real*8          q(nr_qm)
C       Hirshfeld charges 
        real*8          q_hirshfeld(nr_qm)
C       Variables
C       potential and field with fitted charges
        real*8          v_nn(nr_nn_max), f_nn(3, nr_nn_max)
C       difference of potential, field, and charge to reference values
        real*8          vdiff, fdiff(3), qdiff
C       mean square reference field, potential, and charge
        real*8          vref, fref, qref, qrms
C       distance vector, distance, and inverse distance 1/r
        real*8          rijvec(3), rij, rijm1
C       maximal RMS
        real*8          rms, maxvrms, maxfrms
        integer         imaxvrms, imaxfrms, imaxvframe, imaxfframe
C       total nr. of NN atoms 
        integer         nr_nn_tot
C       iterators
        integer         iframe, iqm, inn

C first compute RMS of potential and field     
        vrms = 0.d0
        vref = 0.d0
        frms = 0.d0
        fref = 0.d0
        maxvrms = 0.d0
        maxfrms = 0.d0

        nr_nn_tot = 0
        do iframe = 1, nframes
          call AZZERO(v_nn, nr_nn_max)
          call AZZERO(f_nn, 3*nr_nn_max)
          do inn = 1, nr_nn(iframe)
            do iqm = 1, nr_qm
              rijvec = r_nn(:, inn, iframe) - r_qm(:, iqm, iframe)
              rij = sqrt(dot_product(rijvec, rijvec))
              rijm1 = 1.d0 / rij
              rijvec = rijm1 * rijvec
              v_nn(inn) = v_nn(inn) + q(iqm) * rijm1
              f_nn(:, inn) = f_nn(:, inn) + q(iqm) * rijvec * rijm1**2
            enddo
            vdiff = v_nn(inn) - v_nn_ref(inn, iframe)
            fdiff = f_nn(:, inn) - f_nn_ref(:, inn, iframe)
C            vrms = vrms + vdiff**2
            rms = vdiff**2
            if (rms.gt.maxvrms) then
              maxvrms = rms
              imaxvrms = inn
              imaxvframe = iframe
            endif
            vrms = vrms + rms
            vref = vref + v_nn_ref(inn, iframe)**2

C            frms = frms + dot_product(fdiff, fdiff)
            rms = dot_product(fdiff, fdiff)
            if (rms.gt.maxfrms) then
              maxfrms = rms
              imaxfrms = inn
              imaxfframe = iframe
            endif
            frms = frms + rms
            fref = fref + dot_product(f_nn_ref(:, inn, iframe),
     $                                f_nn_ref(:, inn, iframe))
          enddo
          nr_nn_tot = nr_nn_tot + nr_nn(iframe)
        enddo

C now Hirshfeld charges
        qrms = 0.d0
        qref = 0.d0
        do iqm = 1, nr_qm
          qdiff = q(iqm) - q_hirshfeld(iqm)
          qrms = qrms + qdiff**2
          qref = qref + q_hirshfeld(iqm)**2
        enddo

        write(6, '(A)') "  fm ---------------------------------------"
        write(6, '(A)') "  fm  QFIT RMS, absolute: "
        write(6, '(A, f10.5)') "  fm  Potential: ", sqrt(vrms/nr_nn_tot)
        write(6, '(A, f10.5)') "  fm  Field:     ", sqrt(frms/nr_nn_tot)
        write(6, '(A, f10.5)') "  fm  Charges:   ", sqrt(qrms/nr_qm)
        write(6, '(A)') "  fm ---------------------------------------"
        write(6, '(A)') "  fm  QFIT RMS, relative: "
        write(6, '(A, f10.5)') "  fm  Potential: ", sqrt(vrms/vref)
        write(6, '(A, f10.5)') "  fm  Field:     ", sqrt(frms/fref)
        write(6, '(A, f10.5)') "  fm  Charges:   ", sqrt(qrms/qref)
        write(6, '(A)') "  fm ---------------------------------------"
        write(6, '(A)') "  fm  Largest absolute RMS:"
        write(6, '(A)') "  fm               iframe   inn    rms      "
        write(6, '(A,2(i6),e9.2)') "  fm  Potential:", imaxvframe,
     $                             imaxvrms,maxvrms
        write(6, '(A,2(i6),e9.2)') "  fm  Field:    ", imaxfframe,
     $                             imaxfrms,maxfrms
        write(6, '(A)') "  fm ---------------------------------------"

        return
      end subroutine qfit_rms
C
C     ==================================================================
      subroutine fm_update_topo_charges(nr_q, qm_indx, qnew)
C     ==--------------------------------------------------------------==
C     update the GROMOS topology with the new charges
C     ==================================================================
        implicit none
C       Arguments
C       nr of charges to update
        integer nr_q
C       index array with CPMD numbers of QM atoms
        integer qm_indx(nr_q)
C       new charges
        real*8  qnew(nr_q)
#if defined (__GROMOS)
        include 'system.h'
        include 'mm_dim.inc'
        include 'gromos.h'
C       Variables
C       indices and iterators
        integer igrm, icpmd, iqm
C       conversion factor for charges (GROMOS stores charges with factor
C       sqrt(1/(PI*EPS_0)) )
        real*8  cgfac

        cgfac = dsqrt(FPEPSI)
        do iqm = 1, nr_q
          icpmd = qm_indx(iqm)
          igrm = NAT_grm(icpmd)
          CG(igrm) = cgfac * qnew(iqm)
        enddo
#endif
        return
      end subroutine fm_update_topo_charges
C
