C Author: Patrik Mauer
C Revision: Ivano Tavernelli (Dec. 2010)
C     ==================================================================
C     Routines for forcematching
C     ==================================================================
      subroutine fm_setup_grm_equiv
C     ==--------------------------------------------------------------==
C     setup equivalencies with gromos numbering
C     ==================================================================
        implicit none
        include 'system.h'
        include 'mm_dim.inc'
        include 'forcematch.inc'
C       Arguments
C       Variables
C       nr of QM atoms
        integer nr_qm
C       iterators, temporaries, etc.
        integer iqm, imm, jmm, ieq
C          
        WRITE(6,*)
     c' fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm'
        write(6,*) ' fm ' 
        write(6,'(A)',ADVANCE='NO') '  fm  Setting up equivalencies..'
        nr_qm = NATq

        allocate(grm_equiv(NRPT))
        do imm = 1, NRPT
          grm_equiv(imm) = imm
        enddo
        if (fm_equivalence) then
          do ieq = 1, fm_nreq
            imm = fm_equiv(1, ieq)
            jmm = fm_equiv(2, ieq)
            grm_equiv(jmm) = imm
          enddo
        endif
        write(6,'(A)') '..done!'
        return
      end subroutine fm_setup_grm_equiv
C
C     ==================================================================
      subroutine fm_setup_cpmd_equiv(nr_qm)
C     ==--------------------------------------------------------------==
C     setup equivalencies with CPMD numbering, also handle capping
C     ==================================================================
        implicit none
        include 'system.h'
        include 'mm_dim.inc'
        include 'forcematch.inc'
C       Arguments
        integer nr_qm
C       Variables
C       iterators etc.
        integer iqm, ieq, icap

C setup global qm_uniq
        allocate(qm_equiv(nr_qm))
        if (fm_equivalence) then
          do iqm = 1, nr_qm
            ieq = NAT_cpmd(grm_equiv(NAT_grm(iqm)))
            if (ieq.gt.nr_qm) then
              write(6, *) '========= ERROR! ================'
              write(6, *) ' iqm, ieq, nr_qm:'
              write(6, *) iqm, ieq, nr_qm
              call STOPGM('fm_setup_cpmd_equiv','ieq .gt. nr_qm, you '
     $             // 'probably specified equivalence for an MM atom')
            else
              qm_equiv(iqm) = ieq
            endif
          enddo
        else
          do iqm = 1, nr_qm
            qm_equiv(iqm) = iqm
          enddo
        endif
C now capping
        allocate(fm_iscap(nr_qm))
        fm_iscap = .false.
        if (fm_capping) then
          do icap = 1, fm_ncap
            fm_cap(1, icap) = NAT_cpmd(fm_cap(1, icap))
            fm_cap(2, icap) = NAT_cpmd(fm_cap(2, icap))
            fm_iscap(fm_cap(1, icap)) = .true.
CDEBUG[
            write(6,*) ' QM capping: ', fm_cap(:, icap)
CDEBUG]
          enddo
        endif

        return
      end subroutine fm_setup_cpmd_equiv
C
C     ==================================================================
      subroutine fm_qmatoms_in_grmorder(nr_qm, grm_qm)
C     ==--------------------------------------------------------------==
C     create an index array that has the CPMD number of QM atoms in gromos
C     order, i.e. grm_qm(1) contains the CPMD number of the QM atom with 
C     lowest GROMOS number.
C     We need this for reading TRAJECTORY_PIP (maybe also elsewhere)      
C     May be obsolete now.
C     ==================================================================
        implicit none
        include 'system.h'
        include 'mm_dim.inc'
C       Arguments
        integer nr_qm
        integer grm_qm(nr_qm)
C       Variables
C       array for sorting
        integer tmp_grm(nr_qm)
C       iterators, temporaries, and such
        integer iqm, jqm, curr_qm, lowest

        if (nr_qm .ne. NATq) then
          write(6,*) "========== ERROR ! ==============="
          write(6,*) " internal inconsistency, nr_qm is not equal NATq"
          write(6,*) " nr_qm: ", nr_qm
          write(6,*) " NATq: ", NATq
          call STOPGM('fm_qmatoms_in_grmorder','nr_qm .ne. NATq')
        endif

C the following is not very sophisticated, but for the small nr of QM
C atoms will do...
        do iqm = 1, nr_qm
          tmp_grm(iqm) = NAT_grm(iqm)
        enddo

        curr_qm = 0
        do 
          lowest = NRPT + 1
          do iqm = 1, nr_qm
            if (tmp_grm(iqm).le.lowest) then
              lowest = tmp_grm(iqm)
              jqm = iqm
            endif
          enddo
          curr_qm = curr_qm + 1
          grm_qm(curr_qm) = jqm
          tmp_grm(jqm) = NRPT + 1
          if (curr_qm .eq. nr_qm) exit
        enddo

CDEBUG[
C        write(6,*) 'grm_qm: ', grm_qm
CDEBUG]
        return
      end subroutine fm_qmatoms_in_grmorder
C
C     ==================================================================
      subroutine fm_read_ref_forces(nframes, nr_nn_max, nr_nn, r_nn, 
     $                              v_nn, f_nn, nr_qm, r_qm, f_qm,
     $                              q_hirshfeld)
C     ==--------------------------------------------------------------==
        implicit none
        include 'fileopen.inc'
        include 'forcematch.inc'
C       Arguments
C       nr of frames
        integer         nframes 
C       maximal nr. of NN atoms in a frame
        integer         nr_nn_max
C       nr of NN atoms for each frame
        integer         nr_nn(nframes)
C       coordinates of NN atoms 
        real*8          r_nn(3, nr_nn_max, nframes)
C       potential on NN atoms 
        real*8          v_nn(nr_nn_max, nframes)
C       field on NN atoms 
        real*8          f_nn(3, nr_nn_max, nframes)
C       nr of QM atoms
        integer         nr_qm
C       coordinates of QM atoms 
        real*8          r_qm(3, nr_qm, nframes)
C       forces on QM atoms 
        real*8          f_qm(3, nr_qm, nframes)
C       average hirshfeld charge on QM atoms
        real*8          q_hirshfeld(nr_qm)
C       Variables
C       CPMD atoms in gromos order
        integer         grm_qm(nr_qm)
C       variables in TRAJECTORY_PIP
        character*2     qm_or_mm
        real*8          v, f(3)
C       error upon opening file
        logical         ferror
C       string handling
        integer         ia, ie
C       iterators and temporaries
        integer         inr_nn, inn, jnn, iqm, jqm, iframe
C       for reading hirshfeld charges
        real*8          read_chj(nr_qm, nframes)
C       (dummy) variables for file reading
        integer         idum, ios
        real*8          posdum(3), rdum, q_mm
C       Parameters
C       file units
        integer, parameter      :: forceu = 71, pipu = 72, chju = 73


C       read coordinates and forces on QM atoms. 
        call FILEOPEN(forceu, fm_forces_ref_filename, FO_OLD, ferror)
        if (ferror) call STOPGM('fm_read_forces','error opening file '
     $                          // fm_forces_ref_filename)
        do iframe = 1, nframes
          read(forceu, *, end=301, err=302) idum
          do iqm = 1, nr_qm
            read(forceu, *, end=302, err=302) idum,
     $                                        r_qm(:, iqm, iframe),
     $                                        f_qm(:, iqm, iframe)
          enddo
        enddo
C      Manuel        
        if (.not. fm_fit_charges) return
     
C       fill arrays with values from TRAJECTORY_PIP
C        call fm_qmatoms_in_grmorder(nr_qm, grm_qm)
        call FILEOPEN(pipu, fm_pip_ref_filename, FO_OLD, ferror)
        if (ferror) call STOPGM('fm_read_forces','error opening file '
     $                          // fm_pip_ref_filename)
        call xstring(fm_pip_ref_filename, ia, ie)
        write(6,'(A)') "  fm  Read potential and field from file "
     $                 // fm_pip_ref_filename(ia:ie)

        do iframe = 1, nframes
          iqm = 0
          inn = 0
          read(pipu, *, end=321, err=321) inr_nn
          nr_nn(iframe) = inr_nn - nr_qm
          do jnn = 1, inr_nn
            read(pipu, *, end=321, err=321) posdum, qm_or_mm, 
     $                                      q_mm, v, rdum, rdum, rdum, f
            if (qm_or_mm .eq. 'QM') then
              iqm = iqm + 1
C do not read coordinates for QM atoms (we have them from FORCES file,
C and here they are in Gromos order...)
C              r_qm(:, jqm, iframe) = posdum
            elseif (qm_or_mm .eq. 'MM') then
              inn = inn + 1
              r_nn(:, inn, iframe) = posdum
              v_nn(inn, iframe) = v
C the check below is necessary because TRAJECTORY_PIP contains the
C FORCE, not the field, we should change that...
C              if (abs(q_mm).gt.1.d-6) then
C                f_nn(:, inn, iframe) = f / q_mm
C              else
C                f_nn(:, inn, iframe) = 0.d0
C              endif
C We write out field now, so we are fine
              f_nn(:, inn, iframe) = f
            else
              call STOPGM('fm_read_ref_forces', 'Unknown identifier ' 
     $                    // qm_or_mm)
            endif
          enddo
        enddo
        call FILECLOSE(pipu)

C read Hirshfeld charges from file 
        call xstring(fm_chj_ref_filename, ia, ie)
        call FILEOPEN(chju, fm_chj_ref_filename, FO_OLD, ferror)
        if (ferror) call STOPGM('fm_read_forces','error opening file '
     $                          // fm_chj_ref_filename(ia:ie))
        write(6,'(A)') '  fm  Reading reference charges from file '
     $                  // fm_chj_ref_filename(ia:ie)
        do iframe = 1, nframes
          read(chju, *, end=323, err=322) idum
          read(chju, *, end=323, err=322) read_chj(:, iframe)
        enddo
 323    continue
        iframe = iframe - 1
        write(6, '(A, i9)') '  fm  Nr. of frames read: ', iframe
        if (iframe.lt.nframes) then
          write(6, '(A)') '  fm  ======== WARNING ========'
          write(6, '(A)') '  fm  not enough frames in file ' 
     $                    // fm_chj_ref_filename(ia:ie)
          write(6, '(A)') '  fm  Make sure this is OK!'
          write(6, '(A)') '  fm  ========================='
        else ! try to read one more line...
          read(chju, *, iostat = ios) idum, q_hirshfeld
          if (ios .gt. 0) then ! error
            goto 322
          elseif (ios .lt. 0) then ! end of file, so OK
            write(6, '(A)') '  fm  correct nr. of frames in file '
     $                      // fm_chj_ref_filename(ia:ie)
          else ! there are more frames..
            write(6, '(A)') '  fm  ======== WARNING ========'
            write(6, '(A)') '  fm  excessive nr. of frames in file '
     $                         // fm_chj_ref_filename(ia:ie) 
            write(6, '(A)') '  fm  Make sure this is OK!'
            write(6, '(A)') '  fm  ========================='
          endif
        endif
        call FILECLOSE(chju)
        do iqm = 1, nr_qm
          q_hirshfeld(iqm) = sum(read_chj(iqm, 1:iframe)) / dble(iframe)
        enddo
        write(6, '(A, f8.3)') '  fm  Total charge of QM system is ',
     $                           sum(q_hirshfeld)
        return

C premature end of file while reading fm_forces_ref_filename
 301    continue
        call STOPGM('fm_read_ref_forces', 'premature end of file '
     $              // fm_forces_ref_filename)

C error while reading fm_forces_ref_filename
 302    continue
        call STOPGM('fm_read_ref_forces', 'error reading file '
     $              // fm_forces_ref_filename)

C error while reading file fm_pip_ref_filename
 321    continue
        call STOPGM('fm_read_ref_forces', 'error while reading file ' 
     $              // fm_pip_ref_filename)

C error while reading file fm_pip_ref_filename
 322    continue
        call STOPGM('fm_read_ref_forces', 'error while reading file ' 
     $              // fm_chj_ref_filename)

      end subroutine fm_read_ref_forces
C
C     =================================================================
      subroutine count_nframes_ref_traj(nrats,nrframes)
C     ==--------------------------------------------------------------==
C     count number of frames in the reference trajectory file
C     ==================================================================
      implicit none
      include 'fileopen.inc'
      include 'forcematch.inc'
C     total number of atoms, loop index      
      integer nrats,iats
C     total number of frames in file      
      integer nrframes
      integer iou
      character*500 dummyline
      logical ferror
      iou=623
      nrframes=0
C     open TRAJECTORY_REF
      ferror=.false.
      call FILEOPEN(iou,fm_ref_traj_filename,FO_OLD,ferror)
      if (ferror) call STOPGM('count_nframes_ref_traj',
     $      'error opening ref traj file ' // fm_ref_traj_filename)
      do 
        do iats=1,nrats
           read(iou,'(A500)',end=310,err=311) dummyline
C
C          jump over "NEW DATA" marks
C
           IF (INDEX(dummyline,'NEW DATA') .NE. 0) THEN
C          actually, the NEW DATA mark should not be inside
C          one frame
              write(6,'(A,i9,A,i9)') '  fm NEW DATA mark iats: ',iats,
     &                   ' nrframe: ',nrframes
              IF (iats .gt. 1) goto 311
              read(iou,'(A500)',end=310,err=311) dummyline
           END IF
        end do 
         nrframes=nrframes+1
      end do
C     reached end of file
310   continue
      call FILECLOSE(iou)
      call my_flush(6)
      return  
C     error while reading file 
 311  continue
      call STOPGM('fm_forcematch', 
     $    'error while reading ref traf file ' 
     $               //fm_ref_traj_filename)
      return
      end subroutine count_nframes_ref_traj 
C
C     ==================================================================
      subroutine sp_rest_count_skip_ref_traj(nratqm,nrats,nskipped)
C     ==--------------------------------------------------------------==
C     in case of a restart of the single points calculations we want to
C     check how many frames are already in the fm_ref_forces file and
C     accordingly how many frames we have to skip in the ref_traj file
C     before we can read out the next new frame
C     ==================================================================
      implicit none
      include 'fileopen.inc'
      include 'forcematch.inc'
      integer iats,nrats,nratqm,force_frame_indx,nskipped
      integer check_ref_traj_indx
      character*500 dummyline
      logical ferror,ferror2,ex
C     check whether the files exist
      INQUIRE(file=fm_forces_ref_filename,EXIST=ex)
           if (.not. ex) then
              write (6,*)   
     &        '  fm ',fm_forces_ref_filename,' does not exist'
              call STOPGM('mm_forcematch', 'file not exist')         
           end if 
           INQUIRE(file=fm_chj_ref_filename,EXIST=ex)
           if (.not. ex) then
              write (6,*) 
     &        '  fm ',fm_chj_ref_filename,' does not exist'
              call STOPGM('mm_forcematch', 'not exist')         
           end if 
           INQUIRE(file=fm_pip_ref_filename,EXIST=ex)
           if (.not. ex) then
              write (6,*) 
     &        '  fm ',fm_pip_ref_filename,' does not exist'
              call STOPGM('mm_forcematch', 'not exist')         
           end if 
C          go to all the header lines in FM_REF_FORCES, 
C          one after the other
           nskipped=0
           ferror=.false.
           ferror2=.false.
           CALL FILEOPEN(63,fm_forces_ref_filename,FO_OLD,FERROR)       
           call FILEOPEN(64,fm_ref_traj_filename,FO_OLD,ferror2)
           if (ferror) then
             call STOPGM('sp_rest_count_skip_ref_traj',
     $      'error opening ref forces file ' // fm_forces_ref_filename)
           else if (ferror2) then
               call STOPGM('sp_rest_count_skip_ref_traj',
     $      'error opening ref traj file ' // fm_ref_traj_filename)
           end if
           do
              read(63,*,end=316,err=318) force_frame_indx
C              write(6,*) fm_frame_indx
              do iats=1,nratqm
                   read(63,'(A500)',end=316,err=318) dummyline
              end do
             write(6,*) '  fm found frame index in forces file',
     &              force_frame_indx
C          search for the frame indexes downstream in the TRAJECTORY_REF
C          file (this way it is not a problem if one frame index exists
C          more then once)           
              do
               do iats=1,nrats  
C                read(fm_iou,'(I)',end=318,err=318) check_ref_traj_indx
                read(64,'(A500)',end=316,err=319) dummyline
C
C          jump over "NEW DATA" marks
C
                IF (INDEX(dummyline,'NEW DATA') .NE. 0) THEN
C          actually, the NEW DATA mark should not be inside
C          one frame
                  write(6,*) '  fm found NEW DATA mark in iats ',iats
                  IF (iats .gt. 1) goto 319
                  read(64,'(A500)',end=316,err=319) dummyline
                END IF
               end do !iats
               read(dummyline,*,err=319) check_ref_traj_indx
               nskipped=nskipped+1
               IF (check_ref_traj_indx .EQ. force_frame_indx) THEN
                  write(6,*) '  fm found frame index in ref traj file',
     &                 check_ref_traj_indx
                     exit
                END IF
              end do !loop over all the lines in the ref traj file
           end do
 316  continue
      call fileclose(63)
      call fileclose(64)           
      return
 318   call STOPGM('sp_rest_count_skip_ref_traj', 
     $    'error while reading ref forces file ' 
     $               //fm_ref_traj_filename)
 319   call STOPGM('sp_rest_count_skip_ref_traj', 
     $    'error while reading ref traj file ' 
     $               //fm_ref_traj_filename)

      end subroutine sp_rest_count_skip_ref_traj
C
C
C     ==================================================================
      subroutine fm_restore_exclusions
C     ==--------------------------------------------------------------==
C     This subroutine restores exclusions so that they are correct for a
C     purely classical run (in QM/MM, all QM atoms are mutually excluded)
C     ==================================================================
        implicit none
#if defined (__GROMOS)
        include 'gromos.h'
        include 'forcematch.inc'
C
        INE = fm_save_INE
        KNE = fm_save_KNE
        JSNE = fm_save_JSNE
        INE14 = fm_save_INE14
        KNE14 = fm_save_KNE14
        JSNE14 = fm_save_JSNE14
C        
        deallocate(fm_save_INE)
        deallocate(fm_save_KNE)
        deallocate(fm_save_JSNE)
        deallocate(fm_save_INE14)
        deallocate(fm_save_KNE14)
        deallocate(fm_save_JSNE14)
#endif
        return
      end subroutine fm_restore_exclusions
C
C     ==================================================================
      subroutine fm_writeout_topo(filename)
C     ==--------------------------------------------------------------==
C     write out the topology
C     ==================================================================
        implicit none
        include 'fileopen.inc'
        include 'forcematch.inc'
C       Arguments
        character(*)    filename
C       Variables
C       file unit
        integer topou
C       error with file
        logical ferror

#if defined (__GROMOS)
        write(6,*) ' '
        write(6,'(A)',ADVANCE='NO') '  fm  Writing out new topology..'
        topou = 77
        call FILEOPEN(topou, filename, FO_DEF, ferror)
        if (ferror) then
          call STOPGM('fm_writeout_topo', 'Could not open file '
     $                // filename)
        endif
        call WRTOPO(topou)
        call FILECLOSE(topou)
#endif
        return  
      end subroutine fm_writeout_topo
C
C     ==================================================================
      subroutine fm_combine_capping(nframes, nr_qm, f_qm)
C     ==--------------------------------------------------------------==
C     Combine forces on capping hydrogens with those on capped atoms (for
C     now, simply add them)
C     ==================================================================
        implicit none
        include 'forcematch.inc'
C       Arguments
C       nr of frames, nr of QM atoms
        integer nframes, nr_qm
C       forces on QM atoms
        real*8  f_qm(3,nr_qm,nframes)
C       Variables
C       loop counters, temporaries, etc.
        integer iframe, icap, hcap, ccap

        if (.not.fm_capping) return

        do iframe = 1, nframes
          do icap = 1, fm_ncap
            hcap = fm_cap(1,icap)
            ccap = fm_cap(2,icap)
            f_qm(:,ccap,iframe) = f_qm(:,ccap,iframe)
     $                          + f_qm(:,hcap,iframe)
            f_qm(:,hcap,iframe) = 0.d0
          enddo
        enddo

      end subroutine fm_combine_capping
C

