c     ==================================================================
      subroutine forces_p(c0,c1,c12,h1nl,z11,drhoe,v1_loc,eirop1,
     &     tau0,fion,rhoe,psi,scr,lscr,nstate,firstcall)
c     =================================================================
c     This routine calculates the gradient with respect to the first
c     order perturbation wavefunctions psi_1 (C1) of the second order
c     energy functional to be optimized:
c
c     etot = sum_a<c1_a|h0|c1_a>-
c         sum_a,b <c1_a|z11_b,a|c1_b>+
c         sum_a <c0|h1|c1>+<c1|h1|c0>+
c          1/2 int rr' {n1(r) n1(r')  [ 1/|r-r'|  +  dmu_xc/dn d(r-r')]}
c
c     xavier gonze  PRA 52 p1096 (1995) formula (91)
c     =================================================================
c     The resulting force is NOT orthogonalized with respect to
c     the ground state orbitals. The maximum component is NOT calculated.
c
c     =================================================================
      implicit none
      include 'system.h'
      include 'norm.inc'
      include 'pslo.inc'
      include 'ropt.inc'
      include 'elct.inc'
      include 'geq0.inc'
      include 'cppt.inc'
      include 'spin.inc'
      include 'ions.inc'
      include 'kpts.inc'
      include 'nlps.inc'
      include 'response_p.inc'
      include 'ener.inc'
c     arguments
      integer    nstate,lscr
      dimension  F(nstate)
      complex*16 c0(ngwk,nstate),
     &           psi(nnr1),c1(ngwk,nstate),c12(ngwk,nstate),
     &           h1nl(ngwk,nstate),
     &           eirop1(nhg),v1_loc(nhg,nlsd)
      real*8     tau0(3*nax*nsp),fion(3*nax*nsp),
     &           rhoe(nnr1,nlsd),scr(lscr),z11(nstate,nstate),
     &           drhoe(nnr1,nlsd)
c     variables
      real*8     energy
      character  tag*30

C     TESTING, in progress...

      integer    isub,is1,lforces
      logical    firstcall
c     ==--------------------------------------------------------------==
      call tiset('  forces_p',isub)
      gnmax  = 0.0d0
      gnorm  = 0.0d0
      eh0    = 0.0d0
      elag1  = 0.0d0
      exc1   = 0.0d0
      enl1   = 0.0d0
c     ==--------------------------------------------------------------==
      call give_scr_forces_p(lforces,tag,nstate)
      call test_scr('forces_p',tag,lscr,lforces)

c     ==--  |C12>  =  H0  |C1>   -------------------------------------==
      if (.NOT. FIRSTCALL) then ! because IF FIRSTCALL, then c1=0 anyway
        call h0psi1_p(c1,c0,drhoe,f,tau0,fion,psi,scr,lscr,
     &      nstate,c12)
        eh0 = - ENERGY(c1,c12,nstate)
      else
         if (trho1) call azzero(drhoe,nnr1*nlsd)
      endif                     ! if not firstcall
C     This routine also computes the FIRST ORDER DENSITY (drhoe)

c     ==--  Lagrange multipliers  ------------------------------------==
c     ==--  |C12>  +=  eps_ij  |C1>  ---------------------------------==
c     elag1  =  sum_a,b  <c1_a|  z11_b,a  |c1_b>
      if (.NOT. FIRSTCALL) then ! because IF FIRSTCALL, then c1=0 anyway
        IF (((TNMR.OR.TEPR) .AND. TLOCALIZE) .OR. (Tinteraction)) THEN
c          call DSYMM ( 'R','U', 2*ngw, nstate, 1.d0,
c     &        z11, nstate, c1, 2*ngw, 1.d0, c12, 2*ngw)
           call ROTATE(1.d0, c1, 1.d0, c12, z11, nstate, 2*ngw,
     &          tlsd, nsup, nsdown)
        else
          do is1=1,nstate
            CALL DAXPY(2*ngw, z11(is1,is1), c1(1,is1),1, c12(1,is1),1)
          enddo
        endif
        elag1 = - ENERGY(c1,c12,nstate) - eh0
      endif                     ! if not firstcall

c     ==--  The perturbation potential  ------------------------------==
c     ==--  external (v1_loc) and internal  v[n^1]  ------------------==
c     ==--  |C12>  +=  v^1 |C0>  -------------------------------------==
      if (trho1) then
         if (FASTDEXC .AND. tgc) then
C          calculate d2Exc/dn1dn1 using LDA...
            tgc=.FALSE.
            call v1ofrho_p(v1_loc,eirop1,scr,lscr,drhoe,rhoe,psi,
     &                     firstcall)
            tgc=.TRUE.
         ELSE
            call v1ofrho_p(v1_loc,eirop1,scr,lscr,drhoe,rhoe,psi,
     &                     firstcall)
         ENDIF

C     \ returns the total potential in rhoe
         call v1psi0_p(c0,c12,rhoe,psi,nstate)
C     Calculates also the following energies:
C     \  EHT1  =    <1|  v_Coul[n^(1)]  |0>  the implicit Coulomb potential 
C     \                                      of rho1
C     \  ELOC1 =    <1|  v_ext |0>           the explicit external potential
C     \  EXC1  =    <1|  v_xc [n^(1)]  |0>   the implicit xc-potential of rho1
      endif

c     ==--  The nonlocal part   --------------------------------------==
c     ==--  |C12>  +=  |H1NL>   --------------------------------------==
      if (tnonlocal) then
        call DAXPY(2*nstate*ngw,1.d0,h1nl(1,1),1,c12(1,1),1) 
        enl1 = - 2d0 * ENERGY(c1,h1nl,nstate)
      endif



c     ==--  ENERGIES  ------------------------------------------------==
#ifdef  PARALLEL
      call glosum(L_ENER,eloc1)
#endif
      etot =  eh0 + eloc1 + eht1 + enl1 + exc1 + elag1



      call tihalt('  forces_p',isub)
c     ==--------------------------------------------------------------==
      return
      end
c     ==================================================================
c     ==================================================================
      subroutine give_scr_forces_p(l_forces,tag,nstate)
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'response_p.inc'
c     arguments
      integer   l_forces,nstate
      character tag*30
c     variables
      integer   l_h0psi1,l_rhoofr_p,l_v1ofrho
c     ==--------------------------------------------------------------==
      call give_scr_h0psi1(l_h0psi1,tag,nstate)
      
      if (phonon .or. tlanphon .or. teigens .or. tinr 
     &    .or. teigensystem .or. thardness .or. traman .or. tfukui
     &    .or. tdummyatom .or. TINTERACTION ) then
        call give_scr_rhoofr_p(l_rhoofr_p,tag)
        call give_scr_v1ofrho(l_v1ofrho,tag)
      else
        l_rhoofr_p=0
        l_v1ofrho=0
      endif


      l_forces = max(l_h0psi1,l_rhoofr_p,l_v1ofrho)
      tag     = 'perturbation forces'
      return
      end
c     ==================================================================
