C     ==================================================================
      SUBROUTINE FRIESNER_C(NDIAG,NEL,NCONV,NHPSI,C0,CS,SC0,CSCR,
     &                    VPOT,PSI,EDAV,SCR,LSCR,
     &                    JSPIN,IKIND,IKK,TREFINE,TINFO)
C     ==--------------------------------------------------------------==
C     ==  Kohn-Sham Matrix Diagonalization by Krylov-Space Method     ==
C     ==  W.T. Pollard and R.A. Friesner                              ==
C     ==  J. Chem. Phys. 99, 6742 (1993)                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'irat.inc'
C     Arguments
      INTEGER    NDIAG,NCONV,NHPSI,LSCR,
     &           JSPIN,IKIND,IKK
      COMPLEX*16 C0(NGWK,*),CS(NGWK,*),SC0(NGWK,*),
     &           CSCR(NGWK,*),PSI(*)
      REAL*8     VPOT(*),EDAV(*),SCR(*),NEL
      LOGICAL    TREFINE,TINFO
C     Variables
      REAL*8     EPS,EDAVMAX,EDAVMIN,BIG    
      PARAMETER  (EPS=1.D-14,EDAVMAX=300D0,EDAVMIN=-300D0,BIG=1.D33)
      INTEGER    ISPIN
      PARAMETER  (ISPIN=1)
      COMPLEX*16 ZZERO,ZONE
      PARAMETER  (ZZERO=(0.D0,0.D0),ZONE=(1.D0,0.D0))
      INTEGER    INDEX(NDIAG)
      COMPLEX*16 ADAV(NDIAG,NDIAG)
      POINTER    (IP_INDEX,INDEX),(IP_ADAV,ADAV)
      REAL*8     ALPHA0(NDIAG),BETA1(NDIAG)
      POINTER    (IP_ALPHA0,ALPHA0),(IP_BETA1,BETA1)
      REAL*8     TIM1,TIM2,TIM,TROTMIN,TROTMAX,
     &           B2MIN,B2MAX,B2,FAC,AUX,AMU
      REAL*8     DDOT,TIMEF
      EXTERNAL   DDOT,TIMEF
      INTEGER    ISUB,NGWK2,NCONVOLD,NHPSIOLD,IFORMLANC,NTEST,IERR,I,J,
     &           ITEST,ICYCLE,NCURR,IG,NLEFT,
     &           LFRIESNER,IAUX,LWORK,LSCR2,LSCR3
      LOGICAL    TLSD2,TCONV,TCHECK,TTEST,TDEBUG
      CHARACTER  TAG*30,CHARSPIN*20,FORMLANC*100
C     ==--------------------------------------------------------------==
C     With LSD and 1 electron...
      IF(NDIAG.EQ.0) RETURN
C     ==--------------------------------------------------------------==
      CALL TISET('FRIESNER_C',ISUB)
      CALL GIVE_SCR_FRIESNER(LFRIESNER,TAG,NDIAG)
      CALL TEST_SCR('FRIESNER_C',TAG,LSCR,LFRIESNER)
C     Allocation of SCR
#ifdef _BGL 
      CALL GET_ADDR(IP_BETA1,SCR(1))
      LSCR2=1+NDIAG+MOD(NDIAG,2)
      CALL GET_ADDR(IP_ALPHA0,SCR(LSCR2))
      LSCR2=LSCR2+NDIAG+MOD(NDIAG,2)
      CALL GET_ADDR(IP_ADAV,SCR(LSCR2))
      LSCR2=LSCR2+2*NDIAG*NDIAG
      CALL GET_ADDR(IP_INDEX,SCR(LSCR2))
      LSCR2=LSCR2+(NDIAG/IRAT+1)+MOD((NDIAG/IRAT+1),2)
      LSCR3=LSCR-LSCR2
#else
      LSCR2=LSCR-(NDIAG/IRAT+1)
      CALL GET_ADDR(IP_INDEX,SCR(LSCR2+1))
      LSCR2=LSCR2-2*NDIAG*NDIAG
      CALL GET_ADDR(IP_ADAV,SCR(LSCR2+1))
      LSCR2=LSCR2-NDIAG
      CALL GET_ADDR(IP_ALPHA0,SCR(LSCR2+1))
      LSCR2=LSCR2-NDIAG
      CALL GET_ADDR(IP_BETA1,SCR(LSCR2+1))
#endif
C     ==--------------------------------------------------------------==
      TIM1=TIMEF()
      TLSD2=TLSD
      TLSD=.FALSE.
      TCONV=.FALSE.
      TDEBUG=.FALSE. 
      NGWK2=2*NGWK
      NCONV=0
      NCONVOLD=0
      NHPSIOLD=0
#ifdef _BGL
      LWORK=(LSCR3-3*NDIAG)/2
#else
      LWORK=(LSCR2-3*NDIAG)/2
#endif
      IF(TTROT) THEN
        TROTMIN=EXP(-EDAVMAX*BETAP)
        TROTMAX=EXP(-EDAVMIN*BETAP)
      ENDIF
      CALL IAZZERO(INDEX,NDIAG)
C     ==--------------------------------------------------------------==
      IF(PARENT.AND.TINFO) THEN
        IF(JSPIN.EQ.0) THEN
          CHARSPIN='<<<<<<<<<<<<<<<<<<<<'
        ELSEIF(JSPIN.EQ.1) THEN
          CHARSPIN='<<<<<<< ALPHA SPIN<<'
        ELSEIF(JSPIN.EQ.2) THEN
          CHARSPIN='<<<<<<<< BETA SPIN<<'
        ENDIF
        IFORMLANC=INT(LOG10(DBLE(NKPTS)))+1
        WRITE(FORMLANC,'(A,I1,A,I1,A,I2,A)')
     &       '(/,1X,"<<",I',IFORMLANC,',":",I',IFORMLANC,",",
     &       16-2*IFORMLANC,'("<")," LANCZOS DIAGONALIZATION ",A20)'
        WRITE(6,FORMLANC) IKK,NKPTS,CHARSPIN
      ENDIF
C     ==--------------------------------------------------------------==
C     Subspace Matrix
      TCHECK=.TRUE.
      NTEST=0
      DO WHILE(TCHECK)
        IF(TTROT) THEN
          CALL EHPSI_C(C0,CS,VPOT,PSI,NDIAG,IKIND)
        ELSE
#ifdef  _BGL
      CALL HPSI(C0,CS,SC0,VPOT,PSI,SCR(LSCR2),LSCR3,NDIAG,IKIND,ISPIN)
#else
          CALL HPSI(C0,CS,SC0,VPOT,PSI,SCR,LSCR2,NDIAG,IKIND,ISPIN)
#endif
        ENDIF
        NHPSI=NDIAG
        CALL OVLAP_H(NDIAG,ADAV,C0,CS)
#ifdef PARALLEL
#ifdef  _BGL
        CALL SUMHMAT(ADAV,NDIAG,SCR(LSCR2),LSCR3)
#else
        CALL SUMHMAT(ADAV,NDIAG,SCR,LSCR2)
#endif
#endif
#ifdef  _BGL
        CALL ZHEEV('V','U',NDIAG,ADAV,NDIAG,EDAV,
     &       SCR(LSCR2+3*NDIAG-1),LWORK,SCR(LSCR2),IERR)
        IF(LWORK.LT.INT(SCR(LSCR2))) THEN 
          WRITE(6,*) ' WARNING| LWORK SUBOPTIMAL FOR ZHEEV'
          WRITE(6,*) '   LWORK:',LWORK
          WRITE(6,*) ' OPTIMAL:',INT(SCR(LSCR2))
        ENDIF
#else
        CALL ZHEEV('V','U',NDIAG,ADAV,NDIAG,EDAV,
     &       SCR(3*NDIAG-1),LWORK,SCR(1),IERR)
        IF(LWORK.LT.INT(SCR(1))) THEN 
          WRITE(6,*) ' WARNING| LWORK SUBOPTIMAL FOR ZHEEV'
          WRITE(6,*) '   LWORK:',LWORK
          WRITE(6,*) ' OPTIMAL:',INT(SCR(1))
        ENDIF
#endif
        CALL REORDER_C(NDIAG,NDIAG,EDAV,ADAV)
C       Rotate Orbitals
        CALL ZGEMM('N','N',NGWK,NDIAG,NDIAG,ZONE,C0(1,1),NGWK,
     *       ADAV(1,1),NDIAG,ZZERO,CSCR(1,1),NGWK)
        CALL DCOPY(NGWK2*NDIAG,CSCR(1,1),1,C0(1,1),1)
        TCHECK=.FALSE.
        ITEST=0
        DO I=1,NDIAG
          IF(TTROT) THEN
            TTEST = (EDAV(I).GT.TROTMAX).OR.(EDAV(I).LT.TROTMIN)
          ELSE
            TTEST = ABS(EDAV(I)).LT.EPS
          ENDIF
          IF(TTEST) THEN
            TCHECK=.TRUE.
            IF(PARENT)
     &           WRITE(6,*) 'FRIESNER_C| EIGENVECTOR',I,' IS VERY BAD!'
C           Not useful: Now we use H|C0>.
            CALL RANDTOWF(C0(1,I),1,IKIND,IKK)
            ITEST=ITEST+1
            EDAV(I)=BIG
          ENDIF
        ENDDO
        IF(TCHECK) THEN
          NTEST=NTEST+1
          IF (NTEST.GT.10.AND.PARENT)
     &         CALL STOPGM(' FRIESNER_C', 'CAN''T FIND GUEST VECTORS')
          CALL SORT2(EDAV,NDIAG,INDEX)
C         Put at the end randomized wavefunctions.
          CALL DCOPY(NGWK2*NDIAG,C0,1,CSCR,1)
          DO I=1,NDIAG
            J=INDEX(I)
            IF(EDAV(I).EQ.BIG) THEN
              CALL DCOPY(NGWK2,CS(1,J),1,C0(1,I),1)
            ELSE
              CALL DCOPY(NGWK2,CSCR(1,J),1,C0(1,I),1)
            ENDIF
          ENDDO
C         Orthogonalize.
#if defined(__PGI)
C         The fast rgs seems to cause problems on the pc.Use slow gsortho_c
          CALL GS_ORTHO_C(C0,NDIAG-ITEST,
     &         C0(1,NDIAG-ITEST+1),ITEST,ADAV,SCR)
#else
#ifdef _BGL
          CALL GS_ORTHO_C(C0,NDIAG-ITEST,
     &         C0(1,NDIAG-ITEST+1),ITEST,ADAV,SCR(LSCR2))
#else
          CALL GS_ORTHO_C(C0,NDIAG-ITEST,
     &         C0(1,NDIAG-ITEST+1),ITEST,ADAV,SCR)
#endif
#endif 
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
C     Rotate also H|C0> (IF TCHECK=.FALSE., ADAV unchanged).
      CALL ZGEMM('N','N',NGWK,NDIAG,NDIAG,ZONE,CS(1,1),NGWK,
     *           ADAV(1,1),NDIAG,ZZERO,CSCR(1,1),NGWK)
      CALL DCOPY(NGWK2*NDIAG,CSCR(1,1),1,CS(1,1),1)
      TIM2=TIMEF()
      TIM=(TIM2-TIM1)*0.001D0
      IF(PARENT.AND.TINFO) THEN
        WRITE(6,'(A,T58,F8.2)') 
     *    ' >> TIME FOR INITIAL SUBSPACE DIAGONALIZATION:  ',TIM
        WRITE(6,'(A,8X,A,8X,A)') ' >> CYCLE     NCONV',
     *    'B2MAX','B2MIN     #HPSI      TIME'
      ENDIF
C     ==--------------------------------------------------------------==
      CALL IAZZERO(INDEX,NDIAG)
      DO ICYCLE=1,N_FRIES
        TIM1=TIMEF()
        B2MAX=0.D0
        B2MIN=1.D30
C       We calculate again the residue for all vectors
        NCONV=0
        NCURR=NCONV+1
C       First Lanczos Step
        DO I=NCURR,NDIAG
          DO IG=1,NGWK
            CSCR(IG,1)=CS(IG,I)-EDAV(I)*C0(IG,I)
          ENDDO
          B2=DDOT(NGWK2,CSCR,1,CSCR,1)
#ifdef PARALLEL
          CALL GLOSUM(1,B2)
#endif
          BETA1(I)=SQRT(B2)
          ALPHA0(I)=EDAV(I)
          IF(B2.LT.B2LIMIT) THEN
            NCONV=NCONV+1
            INDEX(I)=NCONV
            IF(B2.GT.B2MAX) B2MAX=B2
            IF(B2.LT.B2MIN) B2MIN=B2
          ELSE
            CALL DCOPY(NGWK2,CSCR,1,CS(1,I),1)
            FAC=1.D0/BETA1(I)
            CALL DSCAL(NGWK2,FAC,CS(1,I),1)
          ENDIF
        ENDDO
C       Order states: converged first
        DO I=NCURR,NDIAG
          J=INDEX(I)
          IF(J.NE.0.AND.J.NE.I) THEN
            CALL DSWAP(NGWK2,C0(1,I),1,C0(1,J),1)
            CALL DSWAP(NGWK2,CS(1,I),1,CS(1,J),1)
            INDEX(J)=J
            INDEX(I)=0
            AUX = BETA1(I)
            BETA1(I) = BETA1(J)
            BETA1(J) = AUX  
            AUX = ALPHA0(I)
            ALPHA0(I) = ALPHA0(J)
            ALPHA0(J) = AUX  
          ENDIF
        ENDDO
        NCURR=NCONV+1
C       Lanczos Refinement
        IF(ICYCLE.EQ.1.AND.NCURR.GT.NDIAG) THEN
          TREFINE=.FALSE.
        ELSE
          TREFINE=.TRUE.
        ENDIF
#ifdef  _BGL
        CALL KRYLOV_REF_C(NCURR,NDIAG,NCONV,NHPSI,C0,CS,SC0,CSCR,
     &                    VPOT,PSI,
     *                    INDEX,ALPHA0,BETA1,SCR(LSCR2),LSCR3,
     &                    B2MIN,B2MAX,IKIND)
#else
        CALL KRYLOV_REF_C(NCURR,NDIAG,NCONV,NHPSI,C0,CS,SC0,CSCR,
     &                    VPOT,PSI,
     *                    INDEX,ALPHA0,BETA1,SCR,LSCR2,
     &                    B2MIN,B2MAX,IKIND)
#endif
C       Order states : converged first
        DO I=NCURR,NDIAG
          J=INDEX(I)
          IF(J.NE.0.AND.J.NE.I) THEN
            CALL DSWAP(NGWK2,C0(1,I),1,C0(1,J),1)
            CALL DSWAP(NGWK2,CS(1,I),1,CS(1,J),1)
            INDEX(J)=J
            INDEX(I)=0
C           Swap BETA1
            AUX=BETA1(J)
            BETA1(J)=BETA1(I)
            BETA1(I)=AUX
          ENDIF
        ENDDO
C       Reorthogonalize
        IF(NCURR.LE.NDIAG) THEN
#if defined(__PGI)
          CALL GS_ORTHO_C(C0,NCURR-1,C0(1,NCURR),NDIAG-NCURR+1,ADAV,SCR)
#else
#ifdef   _BGL
          CALL GS_ORTHO_C(C0,NCURR-1,C0(1,NCURR),NDIAG-NCURR+1,
     *                ADAV,SCR(LSCR2))
#else
          CALL GS_ORTHO_C(C0,NCURR-1,C0(1,NCURR),NDIAG-NCURR+1,ADAV,SCR)
#endif
#endif
        ENDIF
C       Calculate new forces; only for states that entered Lanczos
        NLEFT=NDIAG-NCURR+1
        IF (NCURR.LE.NDIAG) THEN
          IF(TTROT) THEN
            CALL EHPSI_C(C0(1,NCURR),CS(1,NCURR),VPOT,PSI,
     &           NLEFT,IKIND)
          ELSE
#ifdef _BGL
            CALL HPSI(C0(1,NCURR),CS(1,NCURR),SC0(1,NCURR),
     &           VPOT,PSI,SCR(LSCR2),LSCR3,NLEFT,IKIND,ISPIN)
#else
            CALL HPSI(C0(1,NCURR),CS(1,NCURR),SC0(1,NCURR),
     &           VPOT,PSI,SCR,LSCR2,NLEFT,IKIND,ISPIN)
#endif
          ENDIF
          NHPSI=NHPSI+NLEFT
        ENDIF
C       Diagonalize Kohn-Sham Matrix
        CALL OVLAP_H(NDIAG,ADAV,C0,CS)
#ifdef PARALLEL
#ifdef _BGL
        CALL SUMHMAT(ADAV,NDIAG,SCR(LSCR2),LSCR3)
#else
        CALL SUMHMAT(ADAV,NDIAG,SCR,LSCR2)
#endif
#endif
C       We inverse the order
C       (not the same one for ZHEEV than for this routine).
#ifdef _BGL
        CALL ZHEEV('V','U',NDIAG,ADAV,NDIAG,EDAV,
     &       SCR(LSCR2+3*NDIAG+MOD(3*NDIAG,2)),LWORK,SCR(LSCR2),IERR)
#else
        CALL ZHEEV('V','U',NDIAG,ADAV,NDIAG,EDAV,
     &       SCR(3*NDIAG-1),LWORK,SCR(1),IERR)
#endif
        CALL REORDER_C(NDIAG,NDIAG,EDAV,ADAV)
        CALL ZGEMM('N','N',NGWK,NDIAG,NDIAG,ZONE,C0(1,1),NGWK,
     *             ADAV(1,1),NDIAG,ZZERO,CSCR(1,1),NGWK)
        CALL DCOPY(NGWK2*NDIAG,CSCR(1,1),1,C0(1,1),1)
        CALL ZGEMM('N','N',NGWK,NDIAG,NDIAG,ZONE,CS(1,1),NGWK,
     *             ADAV(1,1),NDIAG,ZZERO,CSCR(1,1),NGWK)
        CALL DCOPY(NGWK2*NDIAG,CSCR(1,1),1,CS(1,1),1)
C       If one eigenvalue is equal to zero (very rare: bug ??).
        NTRANS=NCONV
        DO I=1,NCONV
          IF(TTROT) THEN
            TTEST = (EDAV(I).GT.TROTMAX).OR.(EDAV(I).LT.TROTMIN)
          ELSE
            TTEST = ABS(EDAV(I)).LT.EPS
          ENDIF
          IF(TTEST) THEN
            NCONV=NCONV-1
            NTRANS=NTRANS+1
            IF(PARENT)
     &           WRITE(6,*) ' FRIESNER_C| ','VERY BAD EIGENVALUE !!'
            IF(NTRANS.GT.NDIAG) THEN
              CALL RANDTOWF(C0(1,I),1,IKIND,IKK)
            ELSE
              CALL DSWAP(NGWK2,C0(1,I),1,C0(1,NTRANS),1)
              CALL DSWAP(NGWK2,CS(1,I),1,CS(1,NTRANS),1)
            ENDIF
          ENDIF
        ENDDO
        TIM2=TIMEF()
        TIM=(TIM2-TIM1)*0.001D0
        IF(PARENT.AND.TINFO) THEN
           WRITE(6,'(4X,I5,5X,I5,2(1PE13.3),0PF10.2,0PF10.2)')
     &          ICYCLE,NCONV,B2MAX,B2MIN,
     &          DBLE(NHPSI-NHPSIOLD)/DBLE(NDIAG),TIM
        ENDIF
        NHPSIOLD=NHPSI
C       Test if the number of eigenvalues is enough to stop.
        IF((.NOT.TFRAL).AND.TFINT.AND.
     &       (NCONV.NE.NCONVOLD).AND.
     &       (NCONV.NE.NDIAG)) THEN
C          Spin or not.
           IF(TLSD2.AND.(NCONV.GE.NEL).OR.
     &          (.NOT.TLSD2.AND.(NCONV.GE.(NEL+1)/2)) ) THEN
              TCONV=.TRUE.
              NCONVOLD=NCONV
#ifdef   _BGL
              CALL CONVFRIE(NDIAG,NCONV,NEL,EDAV,TLSD,
     &             AMU,SCR(LSCR2),SCR(LSCR2+NDIAG+MOD(NDIAG,2)),TCONV)
#else
              CALL CONVFRIE(NDIAG,NCONV,NEL,EDAV,TLSD,
     &             AMU,SCR(1),SCR(NDIAG+1),TCONV)
#endif
           ENDIF
        ENDIF
        IF(TCONV.OR.NCONV.EQ.NDIAG) THEN
           GOTO 200
        ENDIF
C       End of Krylov Space Cycles
      ENDDO
  200 CONTINUE
      IF(PARENT.AND.((.NOT.TCONV).AND.NCONV.NE.NDIAG).AND.TDEBUG) THEN
        WRITE(6,'(1X,64("!"))')
        WRITE(6,'(" !!",A,T64,"!!")')
     &       ' FRIESNER_C| NOT ALL ROOTS ARE CONVERGED'
        WRITE(6,'(1X,64("!"))')
      ENDIF
C     Order Roots with respect to energy
      CALL SORT2(EDAV,NDIAG,INDEX)
      CALL DCOPY(NGWK2*NDIAG,C0(1,1),1,CS(1,1),1)
      IF(.NOT.TTROT) CALL DSCAL(NDIAG,-1.D0,EDAV(1),1)
      DO I = 1 , NDIAG/2 
         AUX = EDAV(NDIAG-I+1)
         EDAV(NDIAG-I+1) = EDAV(I)
         EDAV(I) = AUX
         IAUX = INDEX(NDIAG-I+1)
         INDEX(NDIAG-I+1) = INDEX(I)
         INDEX(I) = IAUX
      ENDDO   
      DO I=1,NDIAG
        J=INDEX(I)
        CALL DCOPY(NGWK2,CS(1,J),1,C0(1,I),1)
      ENDDO
      TLSD=TLSD2
      CALL TIHALT('FRIESNER_C',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE KRYLOV_REF_C(NCURR,NDIAG,NCONV,NHPSI,C0,CS,SC0,CSCR,
     &                    VPOT,PSI,INDEX,ALPHA0,BETA1,SCR,LSCR,
     *                    B2MIN,B2MAX,IKIND)
C     ==--------------------------------------------------------------==
C     ==  LANCZOS REFINEMENT LOOPS                                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'irat.inc'        !IRAT
C     Arguments
      INTEGER    NCURR,NDIAG,NCONV,NHPSI,LSCR,IKIND,INDEX(*)
      COMPLEX*16 C0(NGWK,*),CS(NGWK,*),SC0(NGWK,*),
     &           CSCR(NGWK,*),PSI(*)
      REAL*8     VPOT(*),ALPHA0(*),BETA1(*),SCR(*),
     &           B2MAX,B2MIN
C     Variables
      INTEGER    ISPIN
      PARAMETER  (ISPIN=1)
      INTEGER    JCONV(NKRY_BLOCK),IND_K(NKRY_BLOCK)
      POINTER    (IP_JCONV,JCONV),(IP_IND_K,IND_K)
      REAL*8     TMAT(NKRY_MAX,NKRY_MAX),BETA(NKRY_MAX,NKRY_BLOCK),
     &           ALPHA(NKRY_MAX,NKRY_BLOCK),YMAT(NKRY_MAX,NKRY_MAX),
     &           ZETA(NKRY_MAX),B2(NKRY_BLOCK),
     &           YOMAT(NKRY_MAX,NKRY_BLOCK)
      POINTER    (IP_TMAT,TMAT),(IP_BETA,BETA),(IP_ALPHA,ALPHA),
     &           (IP_YMAT,YMAT),(IP_ZETA,ZETA),(IP_B2,B2),
     &           (IP_YOMAT,YOMAT)
      INTEGER    NKBEFF,IB,NKRY,JL,J,II,IK,IOVLM,IERR,I,LKRYLOV_REF,
     &           IBN0,IBN1,IBN2,NKB0,NKB1,IKK,JKK,NKBEFFNEW,LSCR2,LSCR3
      COMPLEX*16 ZDOTC
      REAL*8     B22,FAC,ZMAX,DDOT
      EXTERNAL   ZDOTC,DDOT
      CHARACTER  TAG*30
C     ==--------------------------------------------------------------==
C     Test of SCR.
      CALL GIVE_SCR_KRYLOV_REF(LKRYLOV_REF,TAG,NDIAG)
      CALL TEST_SCR('KRYLOV_REF_C',TAG,LSCR,LKRYLOV_REF)
C     Allocation of SCR.
#ifdef _BGL 
      CALL GET_ADDR(IP_YOMAT,SCR(1))
      LSCR2=1+NKRY_MAX*NKRY_BLOCK+MOD(NKRY_MAX*NKRY_BLOCK,2)
      CALL GET_ADDR(IP_B2,SCR(LSCR2))
      LSCR2=LSCR2+NKRY_BLOCK+MOD(NKRY_BLOCK,2)
      CALL GET_ADDR(IP_ZETA,SCR(LSCR2))
      LSCR2=LSCR2+NKRY_MAX+MOD(NKRY_MAX,2)
      CALL GET_ADDR(IP_YMAT,SCR(LSCR2))
      LSCR2=LSCR2+NKRY_MAX*NKRY_MAX+MOD(NKRY_MAX*NKRY_MAX,2)
      CALL GET_ADDR(IP_ALPHA,SCR(LSCR2))
      LSCR2=LSCR2+NKRY_MAX*NKRY_BLOCK+MOD(NKRY_MAX*NKRY_BLOCK,2)
      CALL GET_ADDR(IP_BETA,SCR(LSCR2))
      LSCR2=LSCR2+NKRY_MAX*NKRY_BLOCK+MOD(NKRY_MAX*NKRY_BLOCK,2)
      CALL GET_ADDR(IP_TMAT,SCR(LSCR2))
      LSCR2=LSCR2+NKRY_MAX*NKRY_MAX+MOD(NKRY_MAX*NKRY_MAX,2)
      CALL GET_ADDR(IP_IND_K,SCR(LSCR2))
      LSCR2=LSCR2+(NKRY_BLOCK/IRAT+1)+MOD((NKRY_BLOCK/IRAT+1),2)
      CALL GET_ADDR(IP_JCONV,SCR(LSCR2))
      LSCR2=LSCR2+(NKRY_BLOCK/IRAT+1)+MOD((NKRY_BLOCK/IRAT+1),2)
      LSCR3=LSCR-LSCR2
#else
      LSCR2=LSCR-(NKRY_BLOCK/IRAT+1)
      CALL GET_ADDR(IP_JCONV,SCR(LSCR2+1))
      LSCR2=LSCR2-(NKRY_BLOCK/IRAT+1)
      CALL GET_ADDR(IP_IND_K,SCR(LSCR2+1))
      LSCR2=LSCR2-NKRY_MAX*NKRY_MAX
      CALL GET_ADDR(IP_TMAT,SCR(LSCR2+1))
      LSCR2=LSCR2-NKRY_MAX*NKRY_BLOCK
      CALL GET_ADDR(IP_BETA,SCR(LSCR2+1))
      LSCR2=LSCR2-NKRY_MAX*NKRY_BLOCK
      CALL GET_ADDR(IP_ALPHA,SCR(LSCR2+1))
      LSCR2=LSCR2-NKRY_MAX*NKRY_MAX
      CALL GET_ADDR(IP_YMAT,SCR(LSCR2+1))
      LSCR2=LSCR2-NKRY_MAX
      CALL GET_ADDR(IP_ZETA,SCR(LSCR2+1))
      LSCR2=LSCR2-NKRY_BLOCK
      CALL GET_ADDR(IP_B2,SCR(LSCR2+1))
      LSCR2=LSCR2-NKRY_MAX*NKRY_BLOCK
      CALL GET_ADDR(IP_YOMAT,SCR(LSCR2+1))
#endif
C     ==--------------------------------------------------------------==
      DO I=NCURR,NDIAG,NKRY_BLOCK
        NKBEFF=MIN(NKRY_BLOCK,NDIAG-I+1)
        CALL DCOPY(NKBEFF*NGWK*2,C0(1,I),1,CSCR(1,1),1)
        CALL DCOPY(NKBEFF*NGWK*2,CS(1,I),1,CSCR(1,NKBEFF+1),1)
        DO IB=1,NKBEFF
          JCONV(IB)=0
          IND_K(IB)=I+IB-1
          BETA(1,IB)=0.0D0
          BETA(2,IB)=BETA1(I+IB-1)
          ALPHA(1,IB)=ALPHA0(I+IB-1)
        ENDDO
        DO NKRY=3,NKRY_MAX
          NKB0=(NKRY-1)*NKBEFF+1
          NKB1=(NKRY-2)*NKBEFF+1
          IF(TTROT) THEN
            CALL EHPSI_C(CSCR(1,NKB1),CSCR(1,NKB0),VPOT,PSI,
     &             NKBEFF,IKIND)
          ELSE
#ifdef   _BGL
            CALL HPSI(CSCR(1,NKB1),CSCR(1,NKB0),SC0,
     &           VPOT,PSI,SCR(LSCR2),LSCR3,NKBEFF,IKIND,ISPIN)
#else
            CALL HPSI(CSCR(1,NKB1),CSCR(1,NKB0),SC0,
     &           VPOT,PSI,SCR,LSCR2,NKBEFF,IKIND,ISPIN)
#endif
          ENDIF
          NHPSI=NHPSI+NKBEFF
          DO IB=1,NKBEFF
            IBN0=(NKRY-1)*NKBEFF+IB
            IBN1=(NKRY-2)*NKBEFF+IB
            IBN2=(NKRY-3)*NKBEFF+IB
            CALL DAXPY(NGWK*2,-BETA(NKRY-1,IB),CSCR(1,IBN2),1,
     *                 CSCR(1,IBN0),1)
            ALPHA(NKRY-1,IB)=DREAL(
     &           ZDOTC(NGWK,CSCR(1,IBN1),1,CSCR(1,IBN0),1))
#ifdef PARALLEL
            CALL GLOSUM(1,ALPHA(NKRY-1,IB))
#endif
            CALL DAXPY(NGWK*2,-ALPHA(NKRY-1,IB),CSCR(1,IBN1),1,
     *                 CSCR(1,IBN0),1)
            BETA(NKRY,IB)=DDOT(NGWK*2,CSCR(1,IBN0),1,
     *                         CSCR(1,IBN0),1)
#ifdef PARALLEL
            CALL GLOSUM(1,BETA(NKRY,IB))
#endif
            BETA(NKRY,IB)=SQRT(BETA(NKRY,IB))
            FAC=1.D0/BETA(NKRY,IB)
            CALL DSCAL(NGWK*2,FAC,CSCR(1,IBN0),1)
C           Diagonalize the Lanczos Matrix
            CALL AZZERO(TMAT,NKRY_MAX*NKRY_MAX)
            DO J=1,NKRY-1
              TMAT(J,J)=ALPHA(J,IB)
              TMAT(J+1,J)=BETA(J+1,IB)
              TMAT(J,J+1)=BETA(J+1,IB)
            ENDDO
            CALL JACOBI(NKRY_MAX,NKRY-1,TMAT,ZETA,YMAT,IERR)
C           Calculate beta^2 for the eigenvector
C           with biggest overlap with initial vector
            ZMAX=-1.D30
            DO II=1,NKRY-1
              IF(ZETA(II).GE.ZMAX) THEN
                ZMAX=ZETA(II)
                IOVLM=II
                CALL DCOPY(NKRY-1,YMAT(1,IOVLM),1,YOMAT(1,IB),1)
              ENDIF
            ENDDO
            B2(IB)=BETA(NKRY,IB)*YMAT(NKRY-1,IOVLM)
            B2(IB)=B2(IB)*B2(IB)
            IF(B2(IB).LT.B2LIMIT) THEN
              JCONV(IB)=1
              NCONV=NCONV+1
              INDEX(IND_K(IB))=NCONV
              IF(NKBEFF.NE.1) THEN
                CALL ZAZZERO(C0(1,IND_K(IB)),NGWK)
                DO IK=1,NKRY-1
                  IKK=(IK-1)*NKBEFF+IB
                  CALL DAXPY(NGWK*2,YMAT(IK,IOVLM),CSCR(1,IKK),1,
     *                       C0(1,IND_K(IB)),1)
                ENDDO
              ELSE
                CALL DGEMV('N',NGWK*2,NKRY-1,1.D0,CSCR(1,1),NGWK*2,
     *                 YMAT(1,IOVLM),1,0.D0,C0(1,IND_K(1)),1)
              ENDIF
            ENDIF
          ENDDO
          JL=0
          DO IB=1,NKBEFF
            IF(JCONV(IB).EQ.0) THEN
              JL=JL+1
              IF(JL.NE.IB) THEN
                CALL DCOPY(NKRY_MAX,BETA(1,IB),1,BETA(1,JL),1)
                CALL DCOPY(NKRY_MAX,ALPHA(1,IB),1,ALPHA(1,JL),1)
                CALL DCOPY(NKRY_MAX,YOMAT(1,IB),1,YOMAT(1,JL),1)
                B22=B2(IB)
                B2(IB)=B2(JL)
                B2(JL)=B22
                II=IND_K(IB)
                IND_K(IB)=IND_K(JL)
                IND_K(JL)=II
              ENDIF
            ENDIF
          ENDDO
          NKBEFFNEW=JL
          IF(NKBEFF.NE.NKBEFFNEW.AND.NKBEFFNEW.NE.0) THEN
            DO IK=1,NKRY
              JL=0
              DO IB=1,NKBEFF
                IF(JCONV(IB).EQ.0) THEN
                  JL=JL+1
                  IKK=(IK-1)*NKBEFF+IB
                  JKK=(IK-1)*NKBEFFNEW+JL
                  CALL DCOPY(NGWK*2,CSCR(1,IKK),1,CSCR(1,JKK),1)
                ENDIF
              ENDDO
            ENDDO
          ENDIF
          NKBEFF=NKBEFFNEW
          DO IB=1,NKBEFF
            JCONV(IB)=0
          ENDDO
          IF(NKBEFF.EQ.0) GOTO 100   
        ENDDO
        IF(NKBEFF.NE.1) THEN
          DO IB=1,NKBEFF
            CALL ZAZZERO(C0(1,IND_K(IB)),NGWK)
            DO IK=1,NKRY-2
              IKK=(IK-1)*NKBEFF+IB
              CALL DAXPY(NGWK*2,YOMAT(IK,IB),CSCR(1,IKK),1,
     *                   C0(1,IND_K(IB)),1)
            ENDDO
          ENDDO
        ELSE
          CALL DGEMV('N',NGWK*2,NKRY-2,1.D0,CSCR(1,1),NGWK*2,
     *               YOMAT(1,1),1,0.D0,C0(1,IND_K(1)),1)
        ENDIF
  100   CONTINUE
        DO IB=1,MIN(NKRY_BLOCK,NDIAG-I+1)
          IF(B2(IB).GT.B2MAX) B2MAX=B2(IB)
          IF(B2(IB).LT.B2MIN) B2MIN=B2(IB)
          BETA1(IND_K(IB))=SQRT(B2(IB))
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
