C     ==================================================================
      SUBROUTINE FSTART(FX,RCOV,NBONDS,NCC,LPRT)
C     :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C     GENERATION OF AN INITIAL FORCE CONSTANT MATRIX (FX MATRIX):
C                                                   THF 11/05/90:
C     :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
C     ==--------------------------------------------------------------==
C     THIS CODE IS PART OF THE DISCO PROGRAM SYSTEM
C     DISCO IS AN EXPERIMENTAL CODE FOR DIRECT SCF AND MP2 CALCULATIONS
C     WRITTEN BY
C     J. ALMLOF, K.FAEGRI,JR., M. FEYEREISEN, T.H. FISCHER, 
C     K.KORSELL & H.P. LUETHI
C
C     INPUT
C        RCOV(0:104)   : COVALENT RADIUS
C        IBIND         : CONNECTION MATRIX (IN empf.inc)
C        NCC           : NUMBER OF DEGREES OF FREEDOM
C        LPRT          : .FALSE. MINIMAL PRINT OUT
C                        .TRUE.  FULL PRINT OUT
C     OUTPUT
C        FX(NCC,NCC)   : THE EMPIRICAL CARTESIAN FORCE CONSTANT MATRIX
C     LOCAL 
C        NBONDS(1:NATY): NUMBER OF BONDS OF ATOM NA
C        BRED(1:NCC)   : 
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
C     Arguments
      INTEGER NBONDS(NATY),NCC
      REAL*8  FX(NCC,NCC),RCOV(0:104)
      LOGICAL LPRT
C     Variables
      REAL*8  BRED(12),FRED,FCRED,ALPHA,BETA,GAMMA,DBA,DCA,SUMLIG,
     &        WINKEL
      INTEGER IRED(12),JRED,IPRT,NIF,IA,IB,IAB,I,J,IC,IAC,ID,
     &        I1,J1,I2,J2,IBD,ICD,IAD,IOOP
      LOGICAL GOOD
C     ==--------------------------------------------------------------==
      IPRT=0
      IF(LPRT) THEN
        WRITE(IW,'(/,21X,"<<<<< ESTIMATED FORCE CONSTANTS >>>>>",/)')
        WRITE(IW,'("   NO."," TYPE",7X,"ATOMS",8X,"QUADRATIC")')
        IPRT=1
      END IF
      NIF=0
      DO I=1,NCC
        DO J=1,NCC
          FX(J,I)=ZERO
        ENDDO
      ENDDO
C     ::::::::::::::::::::
C     STRETCH COORDINATES:
C     ::::::::::::::::::::
      DO IA=2,NATY
        DO IB=1,IA-1
          IAB=IBIND(IB+IA*(IA-1)/2)
          IF(IAB.NE.0) THEN
            CALL FSTRE(FRED,FCRED,BRED,IRED,JRED,
     &                 RCOV,IA,IB,NIF,NCC,IPRT,GOOD)
            IF(GOOD) CALL FXMAT(FX,BRED,IRED,JRED,FRED,ONE,NCC)
          END IF
        ENDDO
      ENDDO
C     ..................
C     LOOP OVER ATOMS A.
C     ..................
      DO IA=1,NATY
        IF(NBONDS(IA).GE.2) THEN
C         .....................................
C         LOOP OVER ALL ATOMS B CONNECTED TO A.
C         .....................................
          DO IB=1,NATY
            I  =MAX(IA,IB)
            J  =MIN(IA,IB)
            IAB=IBIND(J+I*(I-1)/2)
            IF(IB.NE.IA.AND.IAB.NE.0) THEN
C             .....................................
C             LOOP OVER ALL ATOMS C CONNECTED TO A.
C             .....................................
              DO IC=IB+1,NATY
                I  =MAX(IA,IC)
                J  =MIN(IA,IC)
                IAC=IBIND(J+I*(I-1)/2)
                IF(IC.NE.IA.AND.IAC.NE.0) THEN
                  ALPHA=WINKEL(IB,IC,IA)
C                 :::::::::::::::
C                 LINEAR BENDING:
C                 :::::::::::::::
                  IF(ALPHA.EQ.180.0D+00) THEN
C                   
                    DO ID=1,NATY
                      IF(ID.NE.IA.AND.ID.NE.IB.AND.ID.NE.IC) THEN
                        I1 =MAX(IB,ID)
                        J1 =MIN(IB,ID)
                        I2 =MAX(IC,ID)
                        J2 =MIN(IC,ID)
                        IBD=IBIND(J1+I1*(I1-1)/2)
                        ICD=IBIND(J2+I2*(I2-1)/2)
                        IF(IBD.NE.0) THEN
                          DBA=WINKEL(ID,IB,IA)
                          IF(DBA.NE.180.0D+00) THEN
                            CALL FLINC(FRED,FCRED,BRED,IRED,JRED,RCOV,
     &                                 IA,IB,IC,ID,NIF,NCC,IPRT,GOOD)
                            IF(GOOD) CALL FXMAT(FX,BRED,IRED,JRED,
     &                                          FRED,THREE,NCC)
                            CALL FLINP(FRED,FCRED,BRED,IRED,JRED,
     &                                 IA,IB,IC,ID,NIF,NCC,IPRT,GOOD)
                            IF(GOOD) CALL FXMAT(FX,BRED,IRED,JRED,
     &                                          FRED,THREE,NCC)
                          END IF
                        END IF
                        IF(ICD.NE.0) THEN
                          DCA=WINKEL(ID,IC,IA)
                          IF(DCA.NE.180.0D+00) THEN
                            CALL FLINC(FRED,FCRED,BRED,IRED,JRED,RCOV,
     &                                 IA,IC,IB,ID,NIF,NCC,IPRT,GOOD)
                            IF(GOOD) CALL FXMAT(FX,BRED,IRED,JRED,
     &                                          FRED,THREE,NCC)
                            CALL FLINP(FRED,FCRED,BRED,IRED,JRED,
     &                                 IA,IC,IB,ID,NIF,NCC,IPRT,GOOD)
                            IF(GOOD) CALL FXMAT(FX,BRED,IRED,JRED,
     &                                          FRED,THREE,NCC)
                          END IF
                        END IF
                      END IF
                    ENDDO
C                   :::::::::::::::::::
C                   NON LINEAR BENDING:
C                   :::::::::::::::::::
                  ELSE
                    CALL FBEND(FRED,FCRED,BRED,IRED,JRED,RCOV,
     &                         IA,IB,IC,NIF,NCC,IPRT,GOOD)
                    IF(GOOD) CALL FXMAT(FX,BRED,IRED,JRED,FRED,ONE,NCC)
                  END IF
C                 :::::::::::::::::::::::::
C                 OUT OF PLANE COORDINATES:
C                 :::::::::::::::::::::::::
                  IF(NBONDS(IA).GE.3) THEN
C                   .....................................
C                   LOOP OVER ALL ATOMS D CONNECTED TO A.
C                   .....................................
                    DO ID=IC+1,NATY
                      I  =MAX(IA,ID)
                      J  =MIN(IA,ID)
                      IAD=IBIND(J+I*(I-1)/2)
                      IF(ID.NE.IA.AND.IAD.NE.0) THEN
                        ALPHA=WINKEL(IB,IC,IA)
                        BETA =WINKEL(IC,ID,IA)
                        GAMMA=WINKEL(IB,ID,IA)
                        IF(ALPHA.NE.180.0D+00.AND.BETA.NE.180.0D+00.AND.
     &                     GAMMA.NE.180.0D+00) THEN
                           DO IOOP=1,3
                             CALL FOUTP(FRED,FCRED,BRED,IRED,JRED,RCOV,
     &                                  IA,IB,IC,ID,IOOP,
     &                                  NIF,NCC,IPRT,GOOD)
                             IF(GOOD) CALL FXMAT(FX,BRED,IRED,JRED,
     &                                           FRED,THREE,NCC)
                           ENDDO
                        END IF
C                       ......................
C                       END LOOP OVER ATOMS D.
C                       ......................
                      END IF
                    ENDDO
                  END IF
C                 ......................
C                 END LOOP OVER ATOMS C.
C                 ......................
                END IF
              ENDDO
C             ::::::::::::::::::::
C             TORSION COORDINATES:
C             ::::::::::::::::::::
              IF(NATY.GE.4.AND.NBONDS(IB).GE.2.AND.IA.LT.IB) THEN
C               .....................................
C               LOOP OVER ALL ATOMS C CONNECTED TO A.
C               .....................................
                DO IC=1,NATY
                  I  =MAX(IC,IA)
                  J  =MIN(IC,IA)
                  IAC=IBIND(J+I*(I-1)/2)
                  IF(IC.NE.IA.AND.IC.NE.IB.AND.IAC.NE.0) THEN
C                   .....................................
C                   LOOP OVER ALL ATOMS D CONNECTED TO A.
C                   .....................................
                    DO ID=1,NATY
                      I  =MAX(IB,ID)
                      J  =MIN(IB,ID)
                      IBD=IBIND(J+I*(I-1)/2)
                      IF(ID.NE.IA.AND.ID.NE.IB.AND.ID.NE.IC.AND.
     &                     IBD.NE.0) THEN
                        ALPHA=WINKEL(IC,IB,IA)
                        BETA =WINKEL(IA,ID,IB)
                        IF(ALPHA.NE.180.0D+00.AND.
     &                      BETA.NE.180.0D+00) THEN
                          SUMLIG=DBLE(NBONDS(IA)+NBONDS(IB)-2)
                          CALL FTORS(FRED,FCRED,BRED,IRED,JRED,
     &                               RCOV,SUMLIG,IA,IB,IC,ID,
     &                               NIF,NCC,IPRT,GOOD)
                          IF(GOOD) CALL FXMAT(FX,BRED,IRED,JRED,
     &                                        FRED,ONE,NCC)
                        END IF
C                       ......................
C                       END LOOP OVER ATOMS D.
C                       ......................
                      END IF
                    ENDDO
C                   ......................
C                   END LOOP OVER ATOMS C.
C                   ......................
                  END IF
                ENDDO
              END IF
C             ......................
C             END LOOP OVER ATOMS B.
C             ......................
            END IF
          ENDDO
C         ......................
C         END LOOP OVER ATOMS A.
C         ......................
        END IF
      ENDDO
C     Complete the matrix FX (half filling)
      DO I=1,NCC
        DO J=I+1,NCC
          FX(I,J)=FX(I,J)+FX(J,I)
          FX(J,I)=FX(I,J)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FBEND(FRED,FCRED,BRED,IRED,JRED,
     &                 RCOV,IA,IB,IC,NIF,NCC,IPRT,GOOD)
C     ==--------------------------------------------------------------==
C     == BENDING FORCE CONSTANTS.                                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
      INCLUDE 'cnst.inc'
C     Arguments
      INTEGER IRED(12),JRED,IA,IB,IC,NIF,NCC,IPRT  
      REAL*8  FRED,FCRED,BRED(12),RCOV(0:104),ABEND(2)
      LOGICAL GOOD
C     Variables
      REAL*8  AHARM,BHARM,CHARM,DHARM,DELTA,PROD,QRED,BSET,DIST
      INTEGER IZA,IZB,IZC,MINZ
C     ==--------------------------------------------------------------==
      CALL BANDQ(BRED,IRED,JRED,QRED,'BEND',IB,IC,IA,0,NCC,GOOD)
      IF(.NOT.GOOD) RETURN
      AHARM=-0.089D+00
      BHARM= 0.11D+00
      CHARM= 0.44D+00
      DHARM=-0.42D+00
      IZA  =NINT(ZAN(IA))
      IZB  =NINT(ZAN(IB))
      IZC  =NINT(ZAN(IC))
      DELTA=DSQRT(DIST(IA,IB))-RCOV(IZA)-RCOV(IZB)+
     &      DSQRT(DIST(IA,IC))-RCOV(IZA)-RCOV(IZC)
      PROD =(RCOV(IZA)+RCOV(IZB))*(RCOV(IZA)+RCOV(IZC))
      FRED =AHARM+BHARM*DEXP(-CHARM*DELTA) / (PROD)**DHARM
C     .......................................
C     SCHLEGEL PARAMETRIZATION FOR HARMONICS.
C     .......................................
      IF(NPARA.GT.0) THEN
        BSET     = ONE
C       BSET     = 1.3D00
        ABEND(1) = 0.130D+00*BSET
        ABEND(2) = 0.112D+00*BSET
C
        IZA =NINT(ZAN(IA))
        IZB =NINT(ZAN(IB))
        IZC =NINT(ZAN(IC))
        MINZ=MIN(IZB,IZC)
        IF(MINZ.EQ.1) THEN
          IF(NPARA.EQ.1) THEN
            FRED=0.160D0
          ELSE
            FRED=ABEND(1)*((RCOV(IZA)+RCOV(IZB))*
     &           (RCOV(IZA)+RCOV(IZC)))**2/
     &           (DSQRT(DIST(IA,IB))+DSQRT(DIST(IA,IC)))**2
          END IF
        ELSE
          IF(NPARA.EQ.1) THEN
            FRED=0.250D0
          ELSE
            FRED=ABEND(2)*((RCOV(IZA)+RCOV(IZB))*
     &           (RCOV(IZA)+RCOV(IZC)))**2/
     &           (DSQRT(DIST(IA,IB))+DSQRT(DIST(IA,IC)))**2
          END IF
        END IF
      END IF
C
      NIF=NIF+1
      IF(IPRT.EQ.1) WRITE(IW,'(I5,".",A5,4I4,2F14.8)')
     &     NIF,'BEND',IB,IC,IA,0,FRED
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FLINC(FRED,FCRED,BRED,IRED,JRED,
     &                 RCOV,IA,IB,IC,ID,NIF,NCC,IPRT,GOOD)
C     ==--------------------------------------------------------------==
C     == COLLINEAR BENDING CONSTANTS.                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
C     Arguments
      INTEGER IRED(12),JRED,IA,IB,IC,ID,NIF,NCC,IPRT
      REAL*8  FRED,FCRED,BRED(12),RCOV(0:104)
      LOGICAL GOOD
C     Variables
      REAL*8  AHARM,BHARM,CHARM,EHARM,DELTA,PROD,QRED,DIST
      INTEGER IZA,IZB,IZC,IZD
C     ==--------------------------------------------------------------==
      CALL BANDQ(BRED,IRED,JRED,QRED,'LINC',IC,IB,IA,ID,NCC,GOOD)
      IF(.NOT.GOOD) RETURN
      AHARM= 0.0025D+00
      BHARM= 0.0061D+00
      CHARM= 3.0D+00
      EHARM= 0.80D+00
      IZA =NINT(ZAN(IA))
      IZB =NINT(ZAN(IB))
      IZC =NINT(ZAN(IC))
      IZD =NINT(ZAN(ID))
      DELTA=DSQRT(DIST(IA,IC))-RCOV(IZA)-RCOV(IZC)
      PROD =(RCOV(IZA)+RCOV(IZB))*(RCOV(IZA)+RCOV(IZD))
      FRED =AHARM+BHARM * PROD**EHARM * DEXP(-CHARM*DELTA)
C     ==--------------------------------------------------------------==
      NIF=NIF+1
      IF(IPRT.EQ.1) WRITE(IW,'(I5,".",A5,4I4,2F14.8)')
     &     NIF,'LINC',IC,IB,IA,ID,FRED
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FLINP(FRED,FCRED,BRED,IRED,JRED,
     &                 IA,IB,IC,ID,NIF,NCC,IPRT,GOOD)
C     ==--------------------------------------------------------------==
C     == PERPENDICULAR LINEAR BENDING CONSTANTS.                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
C     Arguments
      INTEGER IRED(12),JRED,IA,IB,IC,ID,NIF,NCC,IPRT
      REAL*8  FRED,FCRED,BRED(12)
      LOGICAL GOOD
C     Variables
      REAL*8  QRED
C     ==--------------------------------------------------------------==
      CALL BANDQ(BRED,IRED,JRED,QRED,'LINP',IC,IB,IA,ID,NCC,GOOD)
      IF(.NOT.GOOD) RETURN
      NIF=NIF+1
      IF(IPRT.EQ.1) WRITE(IW,'(I5,".",A5,4I4,2F14.8)')
     &     NIF,'LINP',IC,IB,IA,ID,FRED
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FOUTP(FRED,FCRED,BRED,IRED,JRED,
     &                 RCOV,IA,IB,IC,ID,IOOP,NIF,NCC,IPRT,GOOD)
C     ==--------------------------------------------------------------==
C     == OUT OF PLANE FORCE CONSTANTS.                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
      INCLUDE 'cnst.inc'
C     Arguments
      INTEGER IRED(12),JRED,IA,IB,IC,ID,IOOP,NIF,NCC,IPRT
      REAL*8  FRED,FCRED,BRED(12),RCOV(0:104)
      LOGICAL GOOD
C     Variables
      REAL*8  AHARM,BHARM,CHARM,DHARM,EHARM,DELTA,PROD,BSET,QRED,AOUTP,
     &        PI2,DIST
      INTEGER IZA,IZ1,IZ2,IZ3,I1,I2,I3
C     ==--------------------------------------------------------------==
      IF(IOOP.EQ.1) THEN
        I1=ID
        I2=IB
        I3=IC
      ELSE IF(IOOP.EQ.2) THEN
        I1=IB
        I2=IC
        I3=ID
      ELSE
        I1=IC
        I2=IB
        I3=ID
      END IF
      CALL BANDQ(BRED,IRED,JRED,QRED,'OUTP',I1,I2,I3,IA,NCC,GOOD)
      IF(.NOT.GOOD) RETURN
      AHARM= 0.0025D+00
      BHARM= 0.0061D+00
      CHARM= 3.0D+00
      DHARM= 4.0D+00
      EHARM= 0.80D+00
      IZA =NINT(ZAN(IA))
      IZ1 =NINT(ZAN(I1))
      IZ2 =NINT(ZAN(I2))
      IZ3 =NINT(ZAN(I3))
      DELTA=DSQRT(DIST(IA,I1))-RCOV(IZA)-RCOV(IZ1)
      PROD =(RCOV(IZA)+RCOV(IZ2))*(RCOV(IZA)+RCOV(IZ3))
      FRED =AHARM+BHARM * PROD**EHARM * (DCOS(QRED))**DHARM *
     &      DEXP(-CHARM*DELTA)
C     .......................................
C     SCHLEGEL PARAMETERZATION FOR HARMONICS.
C     .......................................
      IF(NPARA.GT.0) THEN
        PI2=PI/TWO
        BSET     = ONE
C       BSET     = 1.3D00
        AOUTP = 0.045D+00 *BSET
        FRED=AOUTP/PI2**4*(PI2-DABS(QRED))**4
      END IF
C     ==--------------------------------------------------------------==
      NIF=NIF+1
      IF(IPRT.EQ.1) WRITE(IW,'(I5,".",A5,4I4,2F14.8)')
     &     NIF,'OUTP',I1,I2,I3,IA,FRED
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FSTRE(FRED,FCRED,BRED,IRED,JRED,
     &                 RCOV,IA,IB,NIF,NCC,IPRT,GOOD)
C     ==--------------------------------------------------------------==
C     == STRETCHING FORCE CONSTANTS.                                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
C     Arguments
      INTEGER IRED(12),JRED,IA,IB,NIF,NCC,IPRT
      REAL*8  FRED,FCRED,BRED(12),RCOV(0:104)
      LOGICAL GOOD
C     Variables
      REAL*8  BSTRE(3,3),AHARM,BHARM,DELTA,BSET,ASTRE,QRED
      INTEGER IZA,IZB,MINZ,MAXZ,IPER,JPER
C     ==--------------------------------------------------------------==
      CALL BANDQ(BRED,IRED,JRED,QRED,'STRE',IA,IB,0,0,NCC,GOOD)
      IF(.NOT.GOOD) RETURN
      AHARM=0.3601D+00
      BHARM=-1.944D+00
      IZA  =NINT(ZAN(IA))
      IZB  =NINT(ZAN(IB))
      DELTA=QRED-RCOV(IZA)-RCOV(IZB)
      FRED =AHARM*DEXP(BHARM*DELTA)
C     .......................................
C     SCHLEGEL PARAMETERZATION FOR HARMONICS.
C     .......................................
      IF(NPARA.GT.0) THEN
        BSET      = ONE
C       BSET      = 1.3D00
        ASTRE     = 1.734D+00*BSET
        BSTRE(1,1)=-0.244D+00
        BSTRE(1,2)= 0.352D+00
        BSTRE(2,2)= 1.085D+00
        BSTRE(1,3)= 0.660D+00
        BSTRE(2,3)= 1.522D+00
        BSTRE(3,3)= 2.068D+00
C
        MINZ=NINT(MIN(ZAN(IA),ZAN(IB)))
        MAXZ=NINT(MAX(ZAN(IA),ZAN(IB)))
        IPER=1
        JPER=1
        IF(MINZ.GT.2)  IPER=2
        IF(MINZ.GT.10) IPER=3
        IF(MAXZ.GT.2)  JPER=2
        IF(MAXZ.GT.10) JPER=3
        FRED=ASTRE/(QRED-BSTRE(IPER,JPER))**3
      END IF
      NIF=NIF+1
      IF(IPRT.EQ.1) WRITE(IW,'(I5,".",A5,4I4,2F14.8)')
     &     NIF,'STRE',IA,IB,0,0,FRED
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FTORS(FRED,FCRED,BRED,IRED,JRED,
     &                 RCOV,SUMLIG,IA,IB,IC,ID,NIF,NCC,IPRT,GOOD)
C     ==--------------------------------------------------------------==
C     == Torsion Force Constants calculation                          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
C     Arguments
      INTEGER IRED(12),JRED,IA,IB,IC,ID,NIF,NCC,IPRT
      REAL*8  FRED,FCRED,BRED(12),RCOV(0:104),SUMLIG
      LOGICAL GOOD
C     Variables
      REAL*8  AHARM,BHARM,CHARM,DHARM,EHARM,BSET,DELTA,PROD,
     &        QRED,ATORS,BTORS,DIST
      INTEGER IZA,IZB
C     ==--------------------------------------------------------------==
      CALL BANDQ(BRED,IRED,JRED,QRED,'TORS',IC,IA,IB,ID,NCC,GOOD)
      IF(.NOT.GOOD) RETURN
      AHARM= 0.0015D+00
      BHARM= 14.0D+00
      CHARM= 2.85D+00
      DHARM= 0.57D+00
      EHARM= 4.0D+00
      IZA =NINT(ZAN(IA))
      IZB =NINT(ZAN(IB))
      DELTA=DSQRT(DIST(IA,IB))-RCOV(IZA)-RCOV(IZB)
      PROD =(RCOV(IZA)+RCOV(IZB))*DSQRT(DIST(IA,IB))
      FRED =AHARM+BHARM*SUMLIG**DHARM/PROD**EHARM*DEXP(-CHARM*DELTA)
C     .......................................
C     SCHLEGEL PARAMETRIZATION FOR HARMONICS.
C     .......................................
      IF(NPARA.GT.0) THEN
        BSET      = ONE
C       BSET      = 1.3D00
        ATORS     = 0.0059D+00*BSET
        BTORS     = 4.348D+00 *BSET
        IZA =NINT(ZAN(IA))
        IZB =NINT(ZAN(IB))
        IF(NPARA.EQ.1) THEN
          FRED=0.0023D0-0.07D0*(DSQRT(DIST(IA,IB))-RCOV(IZA)-RCOV(IZB))
          FRED=DABS(FRED)
        ELSE
          FRED=ATORS*DEXP(-BTORS*(DSQRT(DIST(IA,IB))
     &                    -RCOV(IZA)-RCOV(IZB)))
        END IF
      END IF
C     ==--------------------------------------------------------------==
      NIF=NIF+1
      IF(IPRT.EQ.1) WRITE(IW,'(I5,".",A5,4I4,2F14.8)')
     &     NIF,'TORS',IC,IA,IB,ID,FRED
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE BANDQ(BRED,IRED,JRED,QRED,ITYPF,IA,IB,IC,ID,NCC,GOOD)
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   IA,IB,IC,ID  NUMBERS OF 4 ATOMS                            ==
C     ==   NCC          NUMBER OF FREEDOM DEGREES                     ==
C     ==   ITYPF        TYPE OF CALCULATION                           ==
C     == OUTPUT:                                                      ==
C     ==   JRED         Number of factors                             ==
C     ==   BRED(1:JRED) Factors for force constant                    ==
C     ==   IRED(1:JRED) Index in (1:NCC) for factors                  ==
C     ==   QRED         Force constant                                ==
C     ==   GOOD         .TRUE. if the calculation is possible         ==
C     ==                .FALSE. otherwise                             ==
C     ==--------------------------------------------------------------==
C     == Conventions used in this routine:                            ==
C     == UU(1:3) or UV(1:3)                                           ==
C     == VV(1:3) or UV(4:6)                                           ==
C     == WW(1:3) or UV(7:9)                                           ==
C     == ZZ(1:3) or UV(10:12)                                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
      INCLUDE 'cnst.inc'
C     Arguments
      INTEGER   IRED(12),JRED,IA,IB,IC,ID,NCC
      REAL*8    BRED(12),QRED
      CHARACTER ITYPF*4
      LOGICAL   GOOD
C     Variables
C     Number of considered atoms (for array IAF)
      REAL*8    PIH
      PARAMETER (PIH=PI/TWO)
      INTEGER   IH
      PARAMETER (IH=4)
      INTEGER   IAF(IH),
     &          I,J,L,M,J1,M1,L1
      REAL*8    U(3),V(3),W(3),Z(3),X(3),UU(3),VV(3),
     &          WW(3),ZZ(3),UV(12),
     &          R1,RM1,RM2,R2,CO,SI,R3,CTETA,STETA,CFI1,CFI2,CFI3,
     &          SFI1,DEN,ST2,ST3,CX,CO2,SI2,S,SI3,
     &          SCALARU,ARC1
C
      EQUIVALENCE (UV(1),UU(1)),(UV(4),VV(1)),(UV(7),WW(1)),(UV(10),
     &             ZZ(1))
C
      CHARACTER ITIPUS(7)*4
      DATA      (ITIPUS(I),I=1,7)/'STRE','INVR','BEND','OUTP',
     &                            'TORS','LINC','LINP'/
C     ==--------------------------------------------------------------==
      GOOD=.TRUE.
      DO L=1,3
        UU(L)=ZERO
      ENDDO
      DO L=1,3
        VV(L)=ZERO
      ENDDO
      DO L=1,3
        WW(L)=ZERO
      ENDDO
      DO L=1,3
        ZZ(L)=ZERO
      ENDDO
      QRED=ZERO
      IAF(1)=IA
      IAF(2)=IB
      IAF(3)=IC
      IAF(4)=ID
C     
      IF(ITYPF.EQ.ITIPUS(1)) THEN
C       ...........
C       STRETCHING.
C       ...........
        CALL VEKTOR(UU,R1,IAF(IH-3),IAF(IH-2))
        VV(1)=-UU(1)
        VV(2)=-UU(2)
        VV(3)=-UU(3)
        QRED=QRED+R1
      ELSEIF(ITYPF.EQ.ITIPUS(2)) THEN
C       ........
C       INVERSE.
C       ........
C       ==------------------------------------------------------------==
C       == For Inverse:                                               ==
C       ==     DIST(IA,IB) /= 0                                       ==
C       ==------------------------------------------------------------==
        CALL VEKTOR(UU,R1,IAF(IH-3),IAF(IH-2))
        IF(R1.EQ.ZERO) THEN
          GOOD=.FALSE.
          RETURN
        ENDIF
        RM1  =ONE/R1
        RM2  =RM1*RM1
        UU(1)=-UU(1)*RM2
        UU(2)=-UU(2)*RM2
        UU(3)=-UU(3)*RM2
        VV(1)=-UU(1)
        VV(2)=-UU(2)
        VV(3)=-UU(3)
        QRED=QRED+RM1
      ELSEIF(ITYPF.EQ.ITIPUS(3)) THEN
C       ........
C       BENDING.
C       ........
C       ==------------------------------------------------------------==
C       == For Bending:                                               ==
C       ==     R1=dist(IA,IC) /= 0 and R2=dist(IB,IC) /= 0            ==
C       ==     SI=(IA,IC).(IB,IC) /= 1                                ==
C       ==        i.e. IC,IA,IB not aligned                           ==
C       ==------------------------------------------------------------==
        CALL VEKTOR(U,R1,IAF(IH-3),IAF(IH-1))
        CALL VEKTOR(V,R2,IAF(IH-2),IAF(IH-1))
        IF(R1.EQ.ZERO.OR.R2.EQ.ZERO) THEN
          GOOD=.FALSE.
          RETURN
        ENDIF
        CO=SCALARU(U,V)
        SI=DSQRT(ONE-CO*CO)
        IF(SI.EQ.ZERO) THEN
          GOOD=.FALSE.
          RETURN
        ENDIF
        DO L=1,3
          UU(L)=(CO*U(L)-V(L))/(SI*R1)
          VV(L)=(CO*V(L)-U(L))/(SI*R2)
          WW(L)=-UU(L)-VV(L)
        ENDDO
        QRED=QRED+ACOS(CO)
      ELSEIF(ITYPF.EQ.ITIPUS(4)) THEN
C       .............
C       OUT OF PLANE.
C       .............
C       ==------------------------------------------------------------==
C       == For Out of Plane:                                          ==
C       ==     R1=DIST(IA,IC) /= 0 AND R2=dist(IB,ID) /= 0 and        ==
C       ==     R3=dist(IC,ID) /= 0                                    ==
C       ==     DEN=CTETA*SFI1*SFI1 /= 0                               ==
C       ==        CTETA=((IA,ID).normal(IB,IC,ID)) /= 1               ==
C       ==             i.e. (IA,ID) orthogonal to (IB,IC,ID) plane    ==
C       ==        SFI1=(IB,ID).(IC,ID) /= 1                           ==
C       ==             i.e. ID,IB,IC not aligned                      ==
C       ==------------------------------------------------------------==
        CALL VEKTOR(U,R1,IAF(IH-3),IAF(IH))
        CALL VEKTOR(V,R2,IAF(IH-2),IAF(IH))
        CALL VEKTOR(W,R3,IAF(IH-1),IAF(IH))
        IF(R1.EQ.0.OR.R2.EQ.ZERO.OR.R3.EQ.ZERO) THEN
          GOOD=.FALSE.
          RETURN
        ENDIF
        CALL NORMAL(V,W,Z)
        STETA=SCALARU(U,Z)
        CTETA=DSQRT(ONE-STETA*STETA)
        CFI1 =SCALARU(V,W)
        SFI1 =DSQRT(ONE-CFI1*CFI1)
        CFI2 =SCALARU(W,U)
        CFI3 =SCALARU(V,U)
        DEN  =CTETA*SFI1*SFI1
        IF(DEN.EQ.ZERO) THEN
          GOOD=.FALSE.
          RETURN
        ENDIF
        ST2  =(CFI1*CFI2-CFI3)/(R2*DEN)
        ST3  =(CFI1*CFI3-CFI2)/(R3*DEN)
        DO L=1,3
          VV(L)=Z(L)*ST2
          WW(L)=Z(L)*ST3
        ENDDO
        CALL NORMAL(Z,U,X)
        CALL NORMAL(U,X,Z)
        DO L=1,3
          UU(L)=Z(L)/R1
          ZZ(L)=-UU(L)-VV(L)-WW(L)
        ENDDO
        CX=-ONE
        IF(STETA.LT.ZERO) CX=ONE
        QRED=QRED-CX*ACOS(CTETA)
      ELSEIF(ITYPF.EQ.ITIPUS(5)) THEN
C       ........
C       TORSION.
C       ........
C       ==------------------------------------------------------------==
C       == For Torsion:                                               ==
C       ==     dist(IA,IB) /= 0, dist(IB,IC) /=0 and dist(IC,ID) /= 0 ==
C       ==     The angle ((IA,IB),(IC,IB)) MUST BE /=   0 or 180      ==
C       ==     The angle ((IC,IB),(IC,ID)) MUST BE /=   0 or 180      ==
C       ==------------------------------------------------------------==
        CALL VEKTOR(U,R1,IAF(IH-3),IAF(IH-2))
        CALL VEKTOR(V,R2,IAF(IH-1),IAF(IH-2))
        CALL VEKTOR(W,R3,IAF(IH-1),IAF(IH)  )
        IF(R1.EQ.ZERO.OR.R2.EQ.ZERO.OR.R3.EQ.ZERO) THEN
          GOOD=.FALSE.
          RETURN
        ENDIF
        CALL NORMAL(U,V,Z)
        CALL NORMAL(W,V,X)
        CO =SCALARU(U,V)
        CO2=SCALARU(V,W)
        SI =DSQRT(ONE-CO*CO)
        SI2=DSQRT(ONE-CO2*CO2)
        IF(SI.EQ.ZERO.OR.SI2.EQ.ZERO) THEN
          GOOD=.FALSE.
          RETURN
        ENDIF
        DO L=1,3
          UU(L)=Z(L)/(R1*SI)
          ZZ(L)=X(L)/(R3*SI2)
          VV(L)=(R1*CO/R2-ONE)*UU(L)-R3*CO2/R2*ZZ(L)
          WW(L)=-UU(L)-VV(L)-ZZ(L)
        ENDDO
        CO=SCALARU(Z,X)
        U(1)=Z(2)*X(3)-Z(3)*X(2)
        U(2)=Z(3)*X(1)-Z(1)*X(3)
        U(3)=Z(1)*X(2)-Z(2)*X(1)
        SI3 =DSQRT(U(1)**2+U(2)**2+U(3)**2)
        CO2 =SCALARU(U,V)
        S   =ARC1(-CO,SI3)
        IF(CO2.LT.ZERO) S=-S
C       ..............................................
C       THE RANGE OF TORSION ANGLE IS -PI/2 TO 3*PI/2.
C       ..............................................
        IF(S.GT.PIH) S=S-TWO*PI
        QRED=QRED-S
      ELSEIF(ITYPF.EQ.ITIPUS(6)) THEN
C       ........................
C       LINEAR COPLANAR BENDING.
C       ........................
C       ==------------------------------------------------------------==
C       == For Linear coplanar Bending:                               ==
C       ==     dist(IA,IC) /= 0 and dist(IB,IC) /= 0                  ==
C       ==------------------------------------------------------------==
        CALL VEKTOR(U,R1,IAF(IH-3),IAF(IH-1))
        CALL VEKTOR(V,R2,IAF(IH)  ,IAF(IH-1))
        CALL VEKTOR(X,R2,IAF(IH-2),IAF(IH-1))
        IF(R1.EQ.0.OR.R2.EQ.ZERO) THEN
          GOOD=.FALSE.
          RETURN
        ENDIF
        CO   =SCALARU(V,U)
        CO2  =SCALARU(X,V)
        QRED=QRED+(PI-ACOS(CO)-ACOS(CO2))
        CALL NORMAL(V,U,W)
        CALL NORMAL(U,W,Z)
        CALL NORMAL(X,V,W)
        CALL NORMAL(W,X,U)
C       ............................................................
C       INTERNAL COORDINATE POSITIVE IF ATOM A MOVES TOWARDS ATOM D.
C       ............................................................
        DO L=1,3
          UU(L)=Z(L)/R1
          VV(L)=U(L)/R2
          WW(L)=-UU(L)-VV(L)
        ENDDO
      ELSEIF(ITYPF.EQ.ITIPUS(7)) THEN
C       .............................
C       LINEAR PERPENDICULAR BENDING.
C       .............................
C       ==------------------------------------------------------------==
C       == For Linear Perpendicular Bending:                          ==
C       ==     dist(IA,IC) /= 0 and dist(IC,ID) /= 0                  ==
C       ==------------------------------------------------------------==
        CALL VEKTOR(U,R1,IAF(IH-3),IAF(IH-1))
        CALL VEKTOR(V,R2,IAF(IH)  ,IAF(IH-1))
        CALL VEKTOR(Z,R2,IAF(IH-2),IAF(IH-1))
        IF(R1.EQ.ZERO.OR.R2.EQ.ZERO) THEN
          GOOD=.FALSE.
          RETURN
        ENDIF
        CALL NORMAL(V,U,W)
        CALL NORMAL(Z,V,X)
        DO L=1,3
          UU(L)=W(L)/R1
          VV(L)=X(L)/R2
          WW(L)=-UU(L)-VV(L)
        ENDDO
        CO   =SCALARU(U,W)
        CO2  =SCALARU(Z,W)
        QRED=QRED+(PI-ACOS(CO)-ACOS(CO2))
      ELSE
        CALL STOPGM('BANDQ','THIS OPTION DOES NOT EXIST')
      ENDIF
C     Calculate BRED(1:JRED)
      JRED=0
C     For each atom (given by IAF)
      DO J=1,4
        M=IAF(IH-4+J)
        IF(M.GT.0) THEN
          M =3*(M-1)
          J1=3*(J-1)
C         For each coordinates of atom
          DO L=1,3
            JRED=JRED+1
            M1      =M+L
            IRED(JRED)=M1
            L1      =J1+L
            BRED(JRED)=UV(L1)
          ENDDO
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION ARC1 (X,Y)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'empf.inc'
      INCLUDE 'cnst.inc'
C     Arguments
      REAL*8 X,Y
      REAL*8 ARC1
C     Variables
      REAL*8 S,PIH
C     ==--------------------------------------------------------------==
      PIH=PI/TWO
      IF(ABS(X).LT.1.0D-11) THEN
        ARC1=PIH
      ELSE
        S=ATAN(Y/X)
        IF(X.LT.ZERO) S=S+PI
        ARC1=S
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION  DIST(NUC1,NUC2)
C     ==--------------------------------------------------------------==
C     == W A R N I N G:                                               ==
C     ==             THIS ROUTINE COMPUTES THE SQUARE OF THE DISTANCE ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
C     Arguments
      INTEGER  NUC1,NUC2
      REAL*8   DIST
C     Variables
      REAL*8   C1,C2,C3
C     ==--------------------------------------------------------------==
      C1=C(1,NUC1)-C(1,NUC2)
      C2=C(2,NUC1)-C(2,NUC2)
      C3=C(3,NUC1)-C(3,NUC2)
      CALL PBC(C1,C2,C3,C1,C2,C3,1,APBC,IBRAV)
      DIST = C1*C1 + C2*C2 + C3*C3
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE NOM(U)
C     ==--------------------------------------------------------------==
C     == Normalize the vector U(1:3)                                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'empf.inc'
C     Arguments
      REAL*8  U(3)
C     Variables
      REAL*8  X,D
      INTEGER I
C     ==--------------------------------------------------------------==
      D=DSQRT(U(1)*U(1)+U(2)*U(2)+U(3)*U(3))
      IF(D.LE.ZERO) RETURN
      X=ONE/D
      DO I=1,3
        U(I)=U(I)*X
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE NORMAL(U,V,W)
C     ==--------------------------------------------------------------==
C     == Normalized vector product W(1:3) of U(1:3) and V(1:3)        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8 U(3),V(3),W(3)
C     Variables
      REAL*8 TOL
      PARAMETER (TOL=1.D-20)
C     ==--------------------------------------------------------------==
      W(1)=U(2)*V(3)-U(3)*V(2)
      W(2)=U(3)*V(1)-U(1)*V(3)
      W(3)=U(1)*V(2)-U(2)*V(1)
      IF(ABS(W(1)).LT.TOL.AND.ABS(W(2)).LT.TOL.AND.ABS(W(3)).LT.TOL)
     &     RETURN
      CALL NOM(W)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION SCALARU(U,V)
C     ==--------------------------------------------------------------==
C     == Scalar product of unit vectors                               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'empf.inc'
C     Arguments
      REAL*8 U(3),V(3)
      REAL*8 SCALARU
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
      SCALARU=ZERO
      DO I=1,3
        SCALARU=SCALARU+U(I)*V(I)
      ENDDO
C     Some computer has a small error (DEC)
      IF(SCALARU.GT.ONE) THEN
        SCALARU=ONE
      ELSEIF(SCALARU.LT.-ONE) THEN
        SCALARU=-ONE
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE VEKTOR(U,R,I,J)
C     ==--------------------------------------------------------------==
C     == U=unit vector (I,J) (V_I - V_J) and R=dist(I,J)              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'empf.inc'
C     Arguments
      REAL*8  U(3),R
      INTEGER I,J
C     ==--------------------------------------------------------------==
      U(1)=C(1,I)-C(1,J)
      U(2)=C(2,I)-C(2,J)
      U(3)=C(3,I)-C(3,J)
      CALL PBC(U(1),U(2),U(3),U(1),U(2),U(3),1,APBC,IBRAV)
      R=DSQRT(U(1)*U(1)+U(2)*U(2)+U(3)*U(3))
      CALL NOM(U)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION WINKEL(IA,IB,IC)
C     ==--------------------------------------------------------------==
C     == CALCULATION OF THE ANGLE: A                                  ==
C     == (IN DEGREES)               \                                 ==
C     ==                             C - B                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'empf.inc'
      INCLUDE 'cnst.inc'
C     Arguments
      INTEGER IA,IB,IC
      REAL*8 WINKEL
C     Variables
      REAL*8 U(3),V(3),R1,R2,SCALARU,COSIN
C     ==--------------------------------------------------------------==
      CALL VEKTOR(U,R1,IC,IA)
      CALL VEKTOR(V,R2,IC,IB)
      COSIN=SCALARU(U,V)
      IF(DABS(COSIN+ONE).LE.1.D-6) THEN
        WINKEL=180.0D+00
      ELSE
        WINKEL=180.0D+00*DACOS(COSIN)/PI
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FXMAT(FX,BRED,IRED,JRED,FRED,XNRM,NCC)
C     ==--------------------------------------------------------------==
C     == Fill the force constant matrix FX                            ==
C     == INPUT:                                                       ==
C     ==    NCC         Number of freedom degrees                     ==
C     ==    BRED(1:NCC)                                               ==
C     ==    FRED        Force constant                                ==
C     ==    XNRM        Normalisation number                          ==
C     == OUTPUT:                                                      ==
C     ==    FX(1:NCC,1:NCC) Force constant matrix                     ==
C     ==      W A R N I N G: Fill only half FX                        ==
C     ==--------------------------------------------------------------==
C     == Condition:XNRM /= 0                                          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER IRED(12),JRED,NCC
      REAL*8  FX(NCC,NCC),BRED(12),FRED,XNRM
C     Variables
      INTEGER I,J,II,JJ
C     ==--------------------------------------------------------------==
      IF(FRED.EQ.0.D0) RETURN
      DO I=1,JRED
        IF(BRED(I).NE.0.D0) THEN
          II=IRED(I)
          DO J=I,JRED
            JJ=IRED(J)
            FX(II,JJ)=FX(II,JJ)+BRED(I)*BRED(J)*FRED/XNRM
          ENDDO
        ENDIF
      ENDDO  
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

