#ifdef __SR8000
*option MP(P(0)), LANGLVL(SAVE(0))
#endif
C     ==================================================================
      SUBROUTINE XC(RHO,EX,EC,VX,VC)
C     ==--------------------------------------------------------------==
C     ==  LDA EXCHANGE AND CORRELATION FUNCTIONALS                    ==
C     ==                                                              ==
C     ==  EXCHANGE  :  SLATER alpha                                   ==
C     ==  CORRELATION : CEPERLEY & ALDER (PERDEW-ZUNGER PARAMETERS)   ==
C     ==                VOSKO, WILK & NUSSAIR                         ==
C     ==                LEE, YANG & PARR                              ==
C     ==                PERDEW & WANG                                 ==
C     ==                WIGNER                                        ==
C     ==                HEDIN & LUNDQVIST                             ==
C     ==                ORTIZ & BALLONE (PERDEW-ZUNGER FORMULA)       ==
C     ==                ORTIZ & BALLONE (PERDEW-WANG FORMULA)         ==
C     ==                HCTH/120                                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'func.inc'
      INCLUDE 'cnst.inc'
      PARAMETER (SMALL=1.D-10)
      PARAMETER (PI34= 0.75D0/PI, THIRD=1.D0/3.D0)
C     ==--------------------------------------------------------------==
C..Exchange
      IF(MFXCX.EQ.1) THEN
        CALL SLATERX(RHO,EX,VX,SALPHA)
      ELSE
        EX=0.0D0
        VX=0.0D0
      ENDIF
      IF(RHO.LE.SMALL) THEN
        EC = 0.0D0
        VC = 0.0D0
        EX = 0.0D0
        VX = 0.0D0
      ELSE IF(MFXCC.EQ.1) THEN
        RS=(PI34/RHO)**THIRD
        IFLG=2
        IF(RS.LT.1.0D0) IFLG=1
        CALL PZ(RS,EC,VC,IFLG)
      ELSEIF(MFXCC.EQ.2) THEN
        RS = (PI34/RHO)**THIRD
        CALL VWN(RS,EC,VC)
      ELSEIF(MFXCC.EQ.3) THEN
        CALL LYP(RHO,EC,VC)
      ELSEIF(MFXCC.EQ.4) THEN
        RS=(PI34/RHO)**THIRD
        IFLG=2
        IF(RS.LT.0.5D0) IFLG=1
        IF(RS.GT.100.D0) IFLG=3
        CALL PW(RS,EC,VC,IFLG)
      ELSEIF(MFXCC.EQ.5) THEN
        CALL WIGNER(RHO,EC,VC)
      ELSEIF(MFXCC.EQ.6) THEN
        CALL HEDIN(RHO,EC,VC)
      ELSEIF(MFXCC.EQ.7) THEN
        RS=(PI34/RHO)**THIRD
        IFLG=2
        IF(RS.LT.1.0D0) IFLG=1
        CALL OBPZ(RS,EC,VC,IFLG)
      ELSEIF(MFXCC.EQ.8) THEN
        RS=(PI34/RHO)**THIRD
        IFLG=2
        IF(RS.LT.0.5D0) IFLG=1
        IF(RS.GT.100.D0) IFLG=3
        CALL OBPW(RS,EC,VC,IFLG)
      ELSEIF(MFXCC.EQ.9) THEN
        RS=(PI34/RHO)**THIRD
        CALL PADE(RS,EC,VC)
      ELSE
        EC=0.0D0
        VC=0.0D0
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GCXC(RHO,GRHO,SX,SC,V1X,V2X,V1C,V2C)
C     ==--------------------------------------------------------------==
C     ==  GRADIENT CORRECTIONS FOR EXCHANGE AND CORRELATION           ==
C     ==                                                              ==
C     ==  EXCHANGE  :  BECKE88                                        ==
C     ==               GGAX                                           ==
C     ==               PBEX                                           ==
C     ==               PBESX                                           ==
C     ==               revPBEX                                        ==
C     ==               HCTH/120                                       ==
C     ==               OPTX                                           ==
C     ==  CORRELATION : PERDEW86                                      ==
C     ==                LEE, YANG & PARR                              ==
C     ==                GGAC                                          ==
C     ==                PBEC                                          ==
C     ==                PBESC                                          ==
C     ==                HCTH/120                                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (SMALL=1.D-10)
      INCLUDE 'func.inc'
C     ==--------------------------------------------------------------==
C..Exchange
      IF(RHO.LE.SMALL) THEN
        SX  = 0.0D0
        V1X = 0.0D0
        V2X = 0.0D0
      ELSEIF(MGCX.EQ.1) THEN
        CALL BECKE88(bbeta,RHO,GRHO,SX,V1X,V2X)
      ELSEIF(MGCX.EQ.2) THEN
        CALL GGAX(RHO,GRHO,SX,V1X,V2X)
      ELSEIF(MGCX.EQ.3) THEN
        CALL PBEX(RHO,GRHO,SX,V1X,V2X)
      ELSEIF(MGCX.EQ.4) THEN
        CALL revPBEX(RHO,GRHO,SX,V1X,V2X)
      ELSEIF(MGCX.EQ.5.AND.MGCC.EQ.5) THEN
        CALL HCTH(RHO,GRHO,SX,V1X,V2X) ! x&c
        SC=0.0D0
        V1C=0.0D0 
        V2C=0.0D0
      ELSEIF(MGCX.EQ.6) THEN
        CALL OPTX(RHO,GRHO,SX,V1X,V2X)
      ELSEIF(MGCX.EQ.7) THEN
        CALL BECKE88(BBETA,RHO,GRHO,SXA,V1XA,V2XA)
        CALL GGAX(RHO,GRHO,SXB,V1XB,V2XB)
        SX=0.722D0*SXA+0.347D0*SXB
        V1X=0.722D0*V1XA+0.347D0*V1XB
        V2X=0.722D0*V2XA+0.347D0*V2XB
      ELSEIF(MGCX.EQ.8) THEN
        CALL BECKE88(BBETA,RHO,GRHO,SXA,V1XA,V2XA)
        CALL GGAX(RHO,GRHO,SXB,V1XB,V2XB)
        SX=0.542D0*SXA+0.167D0*SXB
        V1X=0.542D0*V1XA+0.167D0*V1XB
        V2X=0.542D0*V2XA+0.167D0*V2XB
      ELSEIF(MGCX.EQ.9) THEN
        CALL PBESX(RHO,GRHO,SX,V1X,V2X)
      ELSE
        SX=0.0D0
        V1X=0.0D0
        V2X=0.0D0
      ENDIF
C..Correlation
      IF(RHO.LE.SMALL) THEN
        SC  = 0.0D0
        V1C = 0.0D0
        V2C = 0.0D0
      ELSEIF(MGCC.EQ.1) THEN
        CALL PERDEW86(RHO,GRHO,SC,V1C,V2C)
      ELSEIF(MGCC.EQ.2) THEN
        CALL GLYP(RHO,GRHO,SC,V1C,V2C)
      ELSEIF(MGCC.EQ.3) THEN
        CALL GGAC(RHO,GRHO,SC,V1C,V2C)
      ELSEIF(MGCC.EQ.4) THEN
        W1=1.D0
        CALL PBEC(RHO,GRHO,W1,SC,V1C,V2C)
      ELSEIF(MGCC.EQ.6) THEN
        W1=0.74D0
        CALL PBEC(RHO,GRHO,W1,SC,V1C,V2C)
      ELSEIF(MGCC.EQ.7) THEN
        CALL PBESC(RHO,GRHO,W1,SC,V1C,V2C)
      ELSE
        SC=0.0D0
        V1C=0.0D0
        V2C=0.0D0
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SLATERX(RHO,EX,VX,ALPHA)
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (SMALL=1.D-10)
      PARAMETER (F1 = -1.10783814957303361D0)
      PARAMETER (THIRD=1.D0/3.D0,F43=4.D0/3.D0)
C     ==--------------------------------------------------------------==
      IF(RHO.LE.SMALL) THEN
        EX = 0.0D0
        VX = 0.0D0
      ELSE
        RS = RHO**THIRD
        EX = F1*ALPHA*RS
        VX = F43*F1*ALPHA*RS
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PZ(RS,EPZ,VPZ,IFLG)
C     ==--------------------------------------------------------------==
C     ==  J.P. PERDEW AND ALEX ZUNGER PRB 23, 5048 (1981)             ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A=0.0311D0,B=-0.048D0,C=0.0020D0,D=-0.0116D0,
     *           GC=-0.1423D0,B1=1.0529D0,B2=0.3334D0)
C     ==--------------------------------------------------------------==
      IF(IFLG.EQ.1) THEN
C..High density formula
        XLN=LOG(RS)
        EPZ=A*XLN+B+C*RS*XLN+D*RS
        VPZ=A*XLN+(B-A/3.D0)+2.D0/3.D0*C*RS*XLN+
     *              (2.D0*D-C)/3.D0*RS
      ELSEIF(IFLG.EQ.2) THEN
C..Interpolation formula
        RS1=SQRT(RS)
        RS2=RS
        OX=1.D0+B1*RS1+B2*RS2
        DOX=1.D0+7.D0/6.D0*B1*RS1+4.D0/3.D0*B2*RS2
        EPZ=GC/OX
        VPZ=EPZ*DOX/OX
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE VWN(RS,EVWN,VVWN)
C     ==--------------------------------------------------------------==
C     ==  S.H VOSKO, L.WILK, AND M. NUSAIR,                           ==
C     ==                 CAN. J. PHYS. 58 1200  (1980)                ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A=0.0310907D0,B=3.72744D0,C=12.9352D0,X0=-0.10498D0)
      PARAMETER (TWO=2.0D0)
C     ==--------------------------------------------------------------==
      Q  = SQRT(4.D0*C-B*B)
      F1 = TWO*B/Q
      F2 = B*X0/(X0*X0+B*X0+C)
      F3 = TWO*(TWO*X0+B)/Q
      X  = SQRT(RS)
      FX = X*X+B*X+C
      QX = ATAN(Q/(TWO*X+B))
      EVWN=A*(LOG(RS/FX)+F1*QX-F2*(LOG((X-X0)**2/FX)+F3*QX))
      TXPB=TWO*X+B
      TTQQ=TXPB*TXPB+Q*Q
      VVWN=EVWN - X*A/6.D0*(TWO/X-TXPB/FX-4.D0*B/TTQQ-F2*(TWO/(X-X0)
     *          -TXPB/FX-4.D0*(TWO*X0+B)/TTQQ))
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LYP(RHO,ELYP,VLYP)
C     ==--------------------------------------------------------------==
C     ==  C. LEE, W. YANG, AND R.G. PARR, PRB 37, 785 (1988)          ==
C     ==  THIS IS ONLY THE LDA PART                                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A=0.04918D0,B=0.132D0,C=0.2533D0,D=0.349D0)
      PARAMETER (CF=2.87123400018819108D0)
C     ==--------------------------------------------------------------==
      RS=RHO**(-1.D0/3.D0)
      ECRS=B*CF*EXP(-C*RS)
      OX=1.D0/(1.D0+D*RS)
      ELYP=-A*OX*(1.D0+ECRS)
      VLYP=ELYP-RS/3.D0*A*OX*(D*OX+ECRS*(D*OX+C))
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PW(RS,EPWC,VPWC,IFLG)
C     ==--------------------------------------------------------------==
C     ==  J.P. PERDEW AND YUE WANG PRB 45, 13244 (1992)               ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A=0.031091D0,A1=0.21370D0,B1=7.5957D0,B2=3.5876D0,
     *           B3=1.6382D0,B4=0.49294D0,C0=A,C1=0.046644D0,
     *           C2=0.00664D0,C3=0.01043D0,D0=0.4335D0,D1=1.4408D0)
C     ==--------------------------------------------------------------==
      EPWC=0.0D0
      VPWC=0.0D0
      IF(IFLG.EQ.1) THEN
C..High density formula
        XLN=LOG(RS)
        EPWC=C0*XLN-C1+C2*RS*XLN-C3*RS
        VPWC=C0*XLN-(C1+C0/3.D0)+2.D0/3.D0*C2*RS*XLN-
     *              (2.D0*C3+C2)/3.D0*RS
      ELSEIF(IFLG.EQ.2) THEN
C..Interpolation formula
        RS1=SQRT(RS)
        RS2=RS
        RS3=RS2*RS1
        RS4=RS2*RS2
        OM=2.D0*A*(B1*RS1+B2*RS2+B3*RS3+B4*RS4)
        DOM=2.D0*A*(0.5D0*B1*RS1+B2*RS2+1.5D0*B3*RS3+2.D0*B4*RS4)
        OLOG=LOG(1.D0+1.0D0/OM)
        EPWC=-2.D0*A*(1.D0+A1*RS)*OLOG
        VPWC=-2.D0*A*(1.D0+2.D0/3.D0*A1*RS)*OLOG
     *       -2.D0/3.D0*A*(1.D0+A1*RS)*DOM/(OM*(OM+1.D0))
      ELSEIF(IFLG.EQ.3) THEN
C..Low density formula
        EPWC=-D0/RS+D1/RS**1.5D0
        VPWC=-4.D0/3.D0*D0/RS+1.5D0*D1/RS**1.5D0
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE WIGNER(RHO,EXC,FXC)
      IMPLICIT REAL*8 (A-H,O-Z)
      RH=RHO
      X=RH**0.33333333333333333D0
      FXC=-X*((0.943656D0+8.8963D0*X)/(1.0D0+12.57D0*X)**2)
      EXC=-0.738D0*X*(0.959D0/(1.0D0+12.57D0*X))
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE HEDIN(RHO,ECP,FCP)
      IMPLICIT REAL*8 (A-H,O-Z)
cmb-ike   VARIABLES with SAVE attribute hinder the vectorization
cmb-ike      SAVE RH
cmb-ike      IF(RH .EQ. 0.0D0) RETURN
      RH=RHO
      RSM1=0.62035049D0*RH**(0.3333333333333333D0)
      ALN=DLOG(1.0D0 + 21.0D0*RSM1)
      X=21.0D0/RSM1
      ECP = ALN+(X**3*ALN-X*X)+X/2.0D0-1.0D0/3.0D0
      ECP = -0.0225D0*ECP
      FCP = -0.0225D0*ALN
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE OBPZ(RS,EPZ,VPZ,IFLG)
C     ==--------------------------------------------------------------==
C     ==  G.ORTIZ AND P. BALLONE PRB 50, 1391 (1994)                  ==
C     ==  PERDEW-ZUNGER FORMULA                                       ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A=0.031091D0,B=-0.046644D0,C=0.00419D0,D=-0.00983D0,
     *           GC=-0.103756D0,B1=0.56371D0,B2=0.27358D0)
C     ==--------------------------------------------------------------==
      IF(IFLG.EQ.1) THEN
C..High density formula
        XLN=LOG(RS)
        EPZ=A*XLN+B+C*RS*XLN+D*RS
        VPZ=A*XLN+(B-A/3.D0)+2.D0/3.D0*C*RS*XLN+
     *              (2.D0*D-C)/3.D0*RS
      ELSEIF(IFLG.EQ.2) THEN
C..Interpolation formula
        RS1=SQRT(RS)
        RS2=RS
        OX=1.D0+B1*RS1+B2*RS2
        DOX=1.D0+7.D0/6.D0*B1*RS1+4.D0/3.D0*B2*RS2
        EPZ=GC/OX
        VPZ=EPZ*DOX/OX
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE OBPW(RS,EPWC,VPWC,IFLG)
C     ==--------------------------------------------------------------==
C     ==  G.ORTIZ AND P. BALLONE PRB 50, 1391 (1994)                  ==
C     ==  PERDEW-WANG FORMULA                                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A=0.031091D0,A1=0.026481D0,B1=7.5957D0,B2=3.5876D0,
     *           B3=-0.46647D0,B4=0.13354D0,C0=A,C1=0.046644D0,
     *           C2=0.00664D0,C3=0.01043D0,D0=0.4335D0,D1=1.4408D0)
C     ==--------------------------------------------------------------==
      EPWC=0.0D0
      VPWC=0.0D0
      IF(IFLG.EQ.1) THEN
C..High density formula
        XLN=LOG(RS)
        EPWC=C0*XLN-C1+C2*RS*XLN-C3*RS
        VPWC=C0*XLN-(C1+C0/3.D0)+2.D0/3.D0*C2*RS*XLN-
     *              (2.D0*C3+C2)/3.D0*RS
      ELSEIF(IFLG.EQ.2) THEN
C..Interpolation formula
        RS1=SQRT(RS)
        RS2=RS
        RS3=RS2*RS1
        RS4=RS2*RS2
        OM=2.D0*A*(B1*RS1+B2*RS2+B3*RS3+B4*RS4)
        DOM=2.D0*A*(0.5D0*B1*RS1+B2*RS2+1.5D0*B3*RS3+2.D0*B4*RS4)
        OLOG=LOG(1.D0+1.0D0/OM)
        EPWC=-2.D0*A*(1.0D0+A1*RS)*OLOG
        VPWC=-2.D0*A*(1.D0+2.D0/3.D0*A1*RS)*OLOG
     *       -2.D0/3.D0*A*(1.D0+A1*RS)*DOM/(OM*(OM+1.D0))
      ELSEIF(IFLG.EQ.3) THEN
C..Low density formula
        EPWC=-D0/RS+D1/RS**1.5D0
        VPWC=-4.D0/3.D0*D0/RS+1.5D0*D1/RS**1.5D0
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PADE(RS,EC,VC)
C     ==--------------------------------------------------------------==
C     ==  PADE APPROXIMATION                                          ==
C     ==  S. GOEDECKER, M. TETER, J. HUTTER, PRB in press             ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A0=0.4581652932831429D0,A1=2.217058676663745D0,
     *           A2=0.7405551735357053D0,A3=0.01968227878617998D0)
      PARAMETER (B1=1.0000000000000000D0,B2=4.504130959426697D0,
     *           B3=1.110667363742916D0,B4=0.02359291751427506D0)
      PARAMETER (O3=1.D0/3.D0)
C     ==--------------------------------------------------------------==
      TOP=A0+RS*(A1+RS*(A2+RS*A3))
      DTOP=A1+RS*(2.D0*A2+3.D0*A3*RS)
      BOT=RS*(B1+RS*(B2+RS*(B3+RS*B4)))
      DBOT=B1+RS*(2.D0*B2+RS*(3.D0*B3+RS*4.D0*B4))
      EC=-TOP/BOT
      VC=EC+RS*O3*(DTOP/BOT-TOP*DBOT/(BOT*BOT))
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE BECKE88(B1,RHO,GRHO,SX,V1X,V2X)
C     ==--------------------------------------------------------------==
C BECKE EXCHANGE: PRA 38, 3098 (1988)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(OB3=1.D0/3.D0)
C     ==--------------------------------------------------------------==
      TWO13 = 2.0D0**(1.D0/3.D0)
      AA    = GRHO
      A     = SQRT(AA)
      BR1   = RHO**OB3
      BR2   = BR1*BR1
      BR4   = BR2*BR2
      XS    = TWO13*A/BR4
      XS2   = XS*XS
      SA2B8 = SQRT(1.0D0+XS2)
      SHM1  = LOG(XS+SA2B8)
      DD    = 1.0D0 + 6.0D0*B1*XS*SHM1
      DD2   = DD*DD
      EE    = 6.0D0*B1*XS2/SA2B8 - 1.D0
      SX    = TWO13*AA/BR4*(-B1/DD)
      V1X   = -(4.D0/3.D0)/TWO13*XS2*B1*BR1*EE/DD2
      V2X   = TWO13*B1*(EE-DD)/(BR4*DD2)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GGAX(RHO,GRHO,SX,V1X,V2X)
C     ==--------------------------------------------------------------==
C J.P.PERDEW ET AL. PRB 46 6671 (1992)
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'cnst.inc'
      PARAMETER(F1=0.19645D0,F2=7.7956D0,F3=0.2743D0,F4=0.1508D0,
     *          F5=0.004D9)
C     ==--------------------------------------------------------------==
      FP1   = -3.D0/(16.D0*PI)*(3.D0*PI*PI)**(-1.D0/3.D0)
      FP2   = 0.5D0*(3.D0*PI*PI)**(-1.D0/3.D0)
      AA    = GRHO
      A     = SQRT(AA)
      RR    = RHO**(-4.D0/3.D0)
      S     = FP2*A*RR
      S2    = S*S
      S3    = S2*S
      S4    = S2*S2
      EXPS  = F4*EXP(-100.D0*S2)
      AS    = F3-EXPS-F5*S2
      SA2B8 = SQRT(1.0D0+F2*F2*S2)
      SHM1  = LOG(F2*S+SA2B8)
      BS    = 1.D0+F1*S*SHM1+F5*S4
      DAS   = 200.D0*S*EXPS-2.D0*S*F5
      DBS   = F1*(SHM1+F2*S/SA2B8)+4.D0*F5*S3
      DLS   = (DAS/AS-DBS/BS)
      SX    = FP1*AA*RR*AS/BS
      V1X   = -4.D0/3.D0*SX/RHO*(1.D0+S*DLS)
      V2X   = FP1*RR*AS/BS*(2.D0+S*DLS)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PBESX(RHO,GRHO,SX,V1X,V2X)
C     ==--------------------------------------------------------------==
C PBESol functional
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(US=0.161620459673995492D0,AX=-0.738558766382022406D0,
     *          UM=0.123456790123456789D0,UK=0.8040D0,UL=UM/UK)
C     ==--------------------------------------------------------------==
      AA    = GRHO
      RR    = RHO**(-4.D0/3.D0)
      EX    = AX/RR
      S2    = AA*RR*RR*US*US
      PO    = 1.D0/(1.D0 + UL*S2)
      FX    = UK-UK*PO
      SX    = EX*FX
      DFX   = 2.D0*UK*UL*PO*PO
      V1X   = 1.33333333333333D0*AX*RHO**0.333333333333D0*(FX-S2*DFX)
      V2X   = EX*DFX*(US*RR)**2
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PBEX(RHO,GRHO,SX,V1X,V2X)
C     ==--------------------------------------------------------------==
C J.P.PERDEW ET AL. PRL 77 3865 (1996)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(US=0.161620459673995492D0,AX=-0.738558766382022406D0,
     *          UM=0.2195149727645171D0,UK=0.8040D0,UL=UM/UK)
C     ==--------------------------------------------------------------==
      AA    = GRHO
      RR    = RHO**(-4.D0/3.D0)
      EX    = AX/RR
      S2    = AA*RR*RR*US*US
      PO    = 1.D0/(1.D0 + UL*S2)
      FX    = UK-UK*PO
      SX    = EX*FX
      DFX   = 2.D0*UK*UL*PO*PO
      V1X   = 1.33333333333333D0*AX*RHO**0.333333333333D0*(FX-S2*DFX)
      V2X   = EX*DFX*(US*RR)**2
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE revPBEX(RHO,GRHO,SX,V1X,V2X)
C     ==--------------------------------------------------------------==
C Y. ZHANG ET AL. PRL 80 890 (1998)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(US=0.161620459673995492D0,AX=-0.738558766382022406D0,
     *          UM=0.2195149727645171D0,UK=1.2450D0,UL=UM/UK)
C     ==--------------------------------------------------------------==
      AA    = GRHO
      RR    = RHO**(-4.D0/3.D0)
      EX    = AX/RR
      S2    = AA*RR*RR*US*US
      PO    = 1.D0/(1.D0 + UL*S2)
      FX    = UK-UK*PO
      SX    = EX*FX
      DFX   = 2.D0*UK*UL*PO*PO
      V1X   = 1.33333333333333D0*AX*RHO**0.333333333333D0*(FX-S2*DFX)
      V2X   = EX*DFX*(US*RR)**2
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PERDEW86(RHO,GRHO,SC,V1C,V2C)
C     ==--------------------------------------------------------------==
C PERDEW CORRELATION: PRB 33, 8822 (1986)
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'cnst.inc'
      PARAMETER(P1=0.023266D0,P2=7.389D-6,P3=8.723D0,P4=0.472D0)
      PARAMETER(PC1=0.001667D0,PC2=0.002568D0,PCI=PC1+PC2)
      PARAMETER(OB3=1.D0/3.D0)
C     ==--------------------------------------------------------------==
      AA    = GRHO
      A     = SQRT(AA)
      BR1   = RHO**OB3
      BR2   = BR1*BR1
      BR4   = BR2*BR2
      RS    = (3.D0/(FPI*RHO))**OB3
      RS2   = RS*RS
      RS3   = RS*RS2
      CNA   = PC2+P1*RS+P2*RS2
      CNB   = 1.D0+P3*RS+P4*RS2+1.D4*P2*RS3
      CN    = PC1 + CNA/CNB
      DRS   = -OB3*(3.D0/FPI)**OB3 / BR4
      DCNA  = (P1+2.D0*P2*RS)*DRS
      DCNB  = (P3+2.D0*P4*RS+3.D4*P2*RS2)*DRS
      DCN   = DCNA/CNB - CNA/(CNB*CNB)*DCNB
      PHI   = 0.192D0*PCI/CN*A*RHO**(-7.D0/6.D0)
      EPHI  = EXP(-PHI)
      SC    = AA/BR4*CN*EPHI
      V1C   = SC*((1.D0+PHI)*DCN/CN -((4.D0/3.D0)-(7.D0/6.D0)*PHI)/RHO)
      V2C   = CN*EPHI/BR4*(2.D0-PHI)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GLYP(RHO,GRHO,SC,V1C,V2C)
C     ==--------------------------------------------------------------==
C LEE, YANG PARR: GRADIENT CORRECTION PART
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(A=0.04918D0,B=0.132D0,C=0.2533D0,D=0.349D0)
C     ==--------------------------------------------------------------==
      AA    = GRHO
      R     = RHO**(-1.d0/3.d0)
      OM    = EXP(-C*R)/(1.d0+D*R)
      R5    = R**5
      XL    = 1.d0+(7.d0/3.d0)*(C*R + D*R/(1.d0+D*R))
      FF    = A*B*AA/24.d0
      SC    = FF*R5*OM*XL
      DR5   = 5.d0*R*R*R*R
      DOM   = -OM*(C+D+C*D*R)/(1.d0+D*R)
      DXL   = (7.d0/3.d0)*(C+D+2.d0*C*D*R+C*D*D*R*R)/(1.d0+D*R)**2
      V1C   = -FF*(R*R*R*R)/3.d0*( DR5*OM*XL + R5*DOM*XL + R5*OM*DXL)
      V2C   = A*B*R5*OM*XL/12.d0
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GGAC(RHO,GRHO,SC,V1C,V2C)
C     ==--------------------------------------------------------------==
C PERDEW & WANG GGA CORRELATION PART      
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'cnst.inc'
      PARAMETER(AL=0.09D0,PA=0.023266D0,PB=7.389D-6,PC=8.723D0,
     *          PD=0.472D0,CX=-0.001667D0,CXC0=0.002568D0,CC0=-CX+CXC0)
      PARAMETER(OB3=1.D0/3.D0)
C     ==--------------------------------------------------------------==
      XNU   = 16.D0/PI*(3.D0*PI*PI)**OB3
      BE    = XNU*CC0
      CALL XC(RHO,EX,EC,VX,VC)
      AA    = GRHO
      A     = SQRT(AA)
      RS    = (3.D0/(4.D0*PI*RHO))**OB3
      RS2   = RS*RS
      RS3   = RS*RS2
      XKF   = (9.D0*PI/4.D0)**OB3/RS
      XKS   = SQRT(4.D0*XKF/PI)
      T     = A/(2.D0*XKS*RHO)
      EXPE  = EXP(-2.D0*AL*EC/(BE*BE))
      AF    = 2.D0*AL/BE * (1.D0/(EXPE-1.D0))
      BF    = EXPE*(VC-EC)
      Y     = AF*T*T
      XY    = (1.D0+Y)/(1.D0+Y+Y*Y)
      QY    = Y*Y*(2.D0+Y)/(1.D0+Y+Y*Y)**2
      S1    = 1.D0+2.D0*AL/BE*T*T*XY
      H0    = BE*BE/(2.D0*AL) * LOG(S1)
      DH0   = BE*T*T/S1*(-7.D0/3.D0*XY-QY*(AF*BF/BE-7.D0/3.D0))
      DDH0  = BE/(2.D0*XKS*XKS*RHO)*(XY-QY)/S1
      EE    = -100.D0*(XKS/XKF*T)**2
      CNA   = CXC0+PA*RS+PB*RS2
      DCNA  = -(PA*RS+2.D0*PB*RS2)/3.D0
      CNB   = 1.D0+PC*RS+PD*RS2+1.D4*PB*RS3
      DCNB  = -(PC*RS+2.D0*PD*RS2+3.D4*PB*RS3)/3.D0
      CN    = CNA/CNB - CX
      DCN   = DCNA/CNB - CNA*DCNB/(CNB*CNB)
      H1    = XNU*(CN-CC0-3.D0/7.D0*CX)*T*T*EXP(EE)
      DH1   = -OB3*(H1*(7.D0+8.D0*EE)+XNU*T*T*EXP(EE)*DCN)
      DDH1  = 2.D0*H1*(1.D0+EE)*RHO/AA
      SC    = RHO*(H0+H1)
      V1C   = H0+H1+DH0+DH1
      V2C   = DDH0+DDH1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PBESC(RHO,GRHO,W1,SC,V1C,V2C)
C     ==--------------------------------------------------------------==
C PBESol Correlation functional
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'cnst.inc'
      PARAMETER(BE=0.046D0,GA=0.031090690869654895D0)
      PARAMETER(OB3=1.D0/3.D0)
C     ==--------------------------------------------------------------==
      CALL XC(RHO,EX,EC,VX,VC)
      AA    = GRHO
      A     = SQRT(AA)
      RS    = (3.D0/(4.D0*PI*RHO))**OB3
      XKF   = (9.D0*PI/4.D0)**OB3/RS
      XKS   = SQRT(4.D0*XKF/PI)
      T     = A/(2.D0*XKS*RHO)
      EXPE  = EXP(-EC/GA)
      AF    = BE/GA * (1.D0/(EXPE-1.D0))
      Y     = AF*T*T
      XY    = (1.D0+Y)/(1.D0+Y+Y*Y)
      S1    = 1.D0+BE/GA*T*T*XY
      H0    = GA * LOG(S1)
      DTDR  = -T*7.D0/(6.D0*RHO)
      DADR  = AF*AF*EXPE/BE*(VC-EC)/RHO
      DSDA  = -BE/GA * AF * T**6 * (2.D0+Y) / (1.D0+Y+Y*Y)**2
      DSDT  = 2.D0*BE/GA * T * (1.D0+2.D0*Y) / (1.D0+Y+Y*Y)**2
      DSDR  = DSDA*DADR + DSDT*DTDR
      DHDT  = GA/S1*DSDT
      DHDR  = GA/S1*DSDR
      SC    = W1*RHO*H0
      V1C   = W1*H0+W1*DHDR*RHO
      V2C   = W1*RHO*DHDT*T/AA
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PBEC(RHO,GRHO,W1,SC,V1C,V2C)
C     ==--------------------------------------------------------------==
C PBE Correlation functional
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'cnst.inc'
      PARAMETER(BE=0.06672455060314922D0,GA=0.031090690869654895D0)
      PARAMETER(OB3=1.D0/3.D0)
C     ==--------------------------------------------------------------==
      CALL XC(RHO,EX,EC,VX,VC)
      AA    = GRHO
      A     = SQRT(AA)
      RS    = (3.D0/(4.D0*PI*RHO))**OB3
      XKF   = (9.D0*PI/4.D0)**OB3/RS
      XKS   = SQRT(4.D0*XKF/PI)
      T     = A/(2.D0*XKS*RHO)
      EXPE  = EXP(-EC/GA)
      AF    = BE/GA * (1.D0/(EXPE-1.D0))
      Y     = AF*T*T
      XY    = (1.D0+Y)/(1.D0+Y+Y*Y)
      S1    = 1.D0+BE/GA*T*T*XY
      H0    = GA * LOG(S1)
      DTDR  = -T*7.D0/(6.D0*RHO)
      DADR  = AF*AF*EXPE/BE*(VC-EC)/RHO
      DSDA  = -BE/GA * AF * T**6 * (2.D0+Y) / (1.D0+Y+Y*Y)**2
      DSDT  = 2.D0*BE/GA * T * (1.D0+2.D0*Y) / (1.D0+Y+Y*Y)**2
      DSDR  = DSDA*DADR + DSDT*DTDR
      DHDT  = GA/S1*DSDT
      DHDR  = GA/S1*DSDR
      SC    = W1*RHO*H0
      V1C   = W1*H0+W1*DHDR*RHO
      V2C   = W1*RHO*DHDT*T/AA
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE hcth(rho,grho,sx,v1x,v2x)
C     HCTH, JCP 109, 6264 (1998)
C     Parameters set-up after N.L. Doltsisnis & M. Sprik (1999)
C     Present release: Tsukuba, 09/02/2005
c--------------------------------------------------------------------------
c     rhoa = rhob = 0.5 * rho
c     grho is the SQUARE of the gradient of rho! --> gr=sqrt(grho)
c     sx  : total exchange correlation energy at point r 
c     v1x : d(sx)/drho  (eq. dfdra = dfdrb in original)
c     v2x : 1/gr*d(sx)/d(gr) (eq. 0.5 * dfdza = 0.5 * dfdzb in original)
c--------------------------------------------------------------------------
      IMPLICIT REAL*8 (a-h,o-z)
      PARAMETER(o3=1.0d0/3.0d0,fr83=8.d0/3.d0)
      DIMENSION cg0(6),cg1(6),caa(6),cab(6),cx(6)
      r3q2=DEXP(-o3*0.69314718055994531d0)
      r3pi=DEXP(-o3*0.04611759718129048d0)
c.....coefficients for PW correlation......................................
      cg0(1)= 0.031091d0
      cg0(2)= 0.213700d0
      cg0(3)= 7.595700d0
      cg0(4)= 3.587600d0
      cg0(5)= 1.638200d0
      cg0(6)= 0.492940d0
      cg1(1)= 0.015545d0
      cg1(2)= 0.205480d0
      cg1(3)=14.118900d0
      cg1(4)= 6.197700d0
      cg1(5)= 3.366200d0
      cg1(6)= 0.625170d0
C......HCTH-19-4.....................................
      caa(1)=  0.489508D+00
      caa(2)= -0.260699D+00
      caa(3)=  0.432917D+00
      caa(4)= -0.199247D+01
      caa(5)=  0.248531D+01
      caa(6)=  0.200000D+00
      cab(1)=  0.514730D+00
      cab(2)=  0.692982D+01
      cab(3)= -0.247073D+02
      cab(4)=  0.231098D+02
      cab(5)= -0.113234D+02
      cab(6)=  0.006000D+00
      cx(1) =  0.109163D+01
      cx(2) = -0.747215D+00
      cx(3) =  0.507833D+01
      cx(4) = -0.410746D+01
      cx(5) =  0.117173D+01
      cx(6) =  0.004000D+00
c...........................................................................
      gr=DSQRT(grho)
      rho_o3=rho**(o3) 
      rho_o34=rho*rho_o3
      xa=1.25992105d0*gr/rho_o34
      xa2=xa*xa
      ra=0.781592642d0/rho_o3
      rab=r3q2*ra
      dra_drho=-0.260530881d0/rho_o34
      drab_drho=r3q2*dra_drho
      CALL pwcorr(ra,cg1,g,dg)
      era1=g
      dera1_dra=dg
      CALL pwcorr(rab,cg0,g,dg)
      erab0=g
      derab0_drab=dg
      ex=-0.75d0*r3pi*rho_o34
      dex_drho=-r3pi*rho_o3
      uaa=caa(6)*xa2
      uaa=uaa/(1.0d0+uaa)
      uab=cab(6)*xa2
      uab=uab/(1.0d0+uab)
      ux=cx(6)*xa2
      ux=ux/(1.0d0+ux)
      ffaa=rho*era1
      ffab=rho*erab0-ffaa
      dffaa_drho=era1+rho*dera1_dra*dra_drho
      dffab_drho=erab0+rho*derab0_drab*drab_drho-dffaa_drho
cmb-> i-loop removed
      denaa=1.d0/(1.0d0+caa(6)*xa2)
      denab=1.d0/(1.0d0+cab(6)*xa2)
      denx =1.d0/(1.0d0+cx(6)*xa2)
      f83rho=fr83/rho
      bygr=2.0d0/gr
      gaa=caa(1)+uaa*(caa(2)+uaa*(caa(3)+uaa*(caa(4)+uaa*caa(5))))
      gab=cab(1)+uab*(cab(2)+uab*(cab(3)+uab*(cab(4)+uab*cab(5))))
      gx=cx(1)+ux*(cx(2)+ux*(cx(3)+ux*(cx(4)+ux*cx(5))))
      taa=denaa*uaa*(caa(2)+uaa*(2.d0*caa(3)+uaa 
     &    *(3.d0*caa(4)+uaa*4.d0*caa(5))))
      tab=denab*uab*(cab(2)+uab*(2.d0*cab(3)+uab
     &    *(3.d0*cab(4)+uab*4.d0*cab(5))))
      txx=denx*ux*(cx(2)+ux*(2.d0*cx(3)+ux
     &    *(3.d0*cx(4)+ux*4.d0*cx(5))))
      dgaa_drho=-f83rho*taa
      dgab_drho=-f83rho*tab
      dgx_drho=-f83rho*txx
      dgaa_dgr=bygr*taa
      dgab_dgr=bygr*tab
      dgx_dgr=bygr*txx
cmb
      sx=ex*gx+ffaa*gaa+ffab*gab
      v1x=dex_drho*gx+ex*dgx_drho
     .   +dffaa_drho*gaa+ffaa*dgaa_drho
     .   +dffab_drho*gab+ffab*dgab_drho
      v2x=(ex*dgx_dgr+ffaa*dgaa_dgr+ffab*dgab_dgr)/gr
      RETURN
      END
C =-------------------------------------------------------------------=
      SUBROUTINE pwcorr(r,c,g,dg)
      IMPLICIT real*8 (a-h,o-z)
      DIMENSION c(6)
      r12=DSQRT(r)
      r32=r*r12
      r2=r*r
      rb=c(3)*r12+c(4)*r+c(5)*r32+c(6)*r2
      sb=1.0d0+1.0d0/(2.0d0*c(1)*rb)
      g=-2.0d0*c(1)*(1.0d0+c(2)*r)*DLOG(sb)
      drb=c(3)/(2.0d0*r12)+c(4)+1.5d0*c(5)*r12+2.0d0*c(6)*r
      dg=(1.0d0+c(2)*r)*drb/(rb*rb*sb)-2.0d0*c(1)*c(2)*DLOG(sb)
      RETURN
      END 
C     ==================================================================
      SUBROUTINE OPTX(rho,grho,sx,v1x,v2x)
C     OPTX, Handy et al. JCP 116, p. 5411 (2002) and refs. therein
C     Present release: Tsukuba, 20/6/2002
c--------------------------------------------------------------------------
c     rhoa = rhob = 0.5 * rho in LDA implementation
c     grho is the SQUARE of the gradient of rho! --> gr=sqrt(grho)
c     sx  : total exchange correlation energy at point r
c     v1x : d(sx)/drho
c     v2x : 1/gr*d(sx)/d(gr)
c--------------------------------------------------------------------------
      IMPLICIT REAL*8 (a-h,o-z)
      PARAMETER(SMALL=1.D-20,SMAL2=1.D-08)
C.......coefficients and exponents....................
      PARAMETER(o43=4.0d0/3.0d0,two13=1.259921049894873D0
     .         ,two53=3.174802103936399D0,gam=0.006D0
     .         ,a1cx=0.9784571170284421D0,a2=1.43169D0)
C.......OPTX in compact form..........................
      IF(RHO.LE.SMALL) THEN
       sx=0.0D0
       v1x=0.0D0
       v2x=0.0D0
      ELSE
       gr=DMAX1(grho,SMAL2)
       rho43=rho**o43
       xa=two13*DSQRT(gr)/rho43
       gamx2=gam*xa*xa
       uden=1.d+00/(1.d+00+gamx2)
       uu=a2*gamx2*gamx2*uden*uden
       uden=rho43*uu*uden
       sx=-rho43*(a1cx+uu)/two13
       v1x=o43*(sx+two53*uden)/rho
       v2x=-two53*uden/gr
      ENDIF
C
      RETURN
      END
C =-------------------------------------------------------------------=



! These are wrappers to the reference implementation
! below by Heyd & Scuseria.
! For more information about implementation, see also
! Komsa, Broqvist, Pasquarello, Phys. Rev. B 81, 205118 (2010)
!
! Maybe this should be moved to lsd_func.F

C     ==================================================================
      SUBROUTINE PBEXSR_LSD(RHOA,RHOB,GRHOAA,GRHOBB,SX,
     *                      V1XA,V2XA,V1XB,V2XB,V2XAB,OMEGA)
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(SMALL=1.D-20)
C     ==--------------------------------------------------------------==
      SXA=0.0D0
      SXB=0.0D0
      V1XA=0.0D0
      V2XA=0.0D0
      V1XB=0.0D0
      V2XB=0.0D0
      V2XAB=0.0D0
      IF(ABS(RHOA).GT.SMALL) THEN
        CALL PBEXSR(2.D0*RHOA, 4.D0*GRHOAA, SXA, V1XA, V2XA, OMEGA)
      ENDIF
      IF(ABS(RHOB).GT.SMALL) THEN
        CALL PBEXSR(2.D0*RHOB, 4.D0*GRHOBB, SXB, V1XB, V2XB, OMEGA)
      ENDIF
      SX = 0.5D0*(SXA+SXB)
      V2XA = 2.D0*V2XA
      V2XB = 2.D0*V2XB          ! I REALLY HOPE THIS WORKS JUST LIKE THIS

C     ==--------------------------------------------------------------==
      RETURN
      END

C     ==================================================================
      SUBROUTINE PBEXSR(RHO,GRHO,SX,V1X,V2X,OMEGA)
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'cnst.inc'
      PARAMETER(SMALL=1.D-20,SMAL2=1.D-08)
      PARAMETER(US=0.161620459673995492D0,AX=-0.738558766382022406D0,
     *          UM=0.2195149727645171D0,UK=0.8040D0,UL=UM/UK)
!C     ==--------------------------------------------------------------==

      CALL XC(RHO,EX,EC,VX,VC)

!      AA    = DMAX1(GRHO,SMAL2)
      AA    = GRHO
      RR    = RHO**(-4.D0/3.D0)
      EX    = AX/RR
      S2    = AA*RR*RR*US*US

      S = SQRT(S2)
! Lieb-Oxford bound
      IF(S.GT.8.3D0) THEN
         S = 8.572844D0 - 18.796223D0/S2
      ENDIF
      CALL wpbe_analytical_erfc_approx_grad(RHO,S,OMEGA,FX,D1X,D2X)
      SX = EX*FX        ! - EX
      DSDN = -4.D0/3.D0*S/RHO
      V1X = VX*FX + (DSDN*D2X+D1X)*EX   ! - VX
      DSDG = US*RR
      V2X = EX*1.D0/SQRT(AA)*DSDG*D2X

! NOTE, in PBEX() SX is only the gradient correction energy density(?),
! but here it is the total energy density
! And same for the potential V1X

C     ==--------------------------------------------------------------==
      RETURN
      END


c--------------------------------------------------------------------
c
c     wPBE Enhancement Factor (erfc approx.,analytical, gradients)
c
c--------------------------------------------------------------------

      subroutine wpbe_analytical_erfc_approx_grad(rho,s,omega,Fx_wpbe,
     $                d1rfx,d1sfx)

      Implicit None

      Real*8 rho,s,omega,Fx_wpbe,d1sfx,d1rfx

      Real*8 f12,f13,f14,f18,f23,f43,f32,f72,f34,f94,f1516,f98
      Real*8 pi,pi2,pi_23,srpi
      Real*8 Three_13
    
      Real*8 ea1,ea2,ea3,ea4,ea5,ea6,ea7,ea8
      Real*8 eb1
      Real*8 A,B,C,D,E
      Real*8 Ha1,Ha2,Ha3,Ha4,Ha5
      Real*8 Fc1,Fc2
      Real*8 EGa1,EGa2,EGa3
      Real*8 EGscut,wcutoff,expfcutoff

      Real*8 xkf, xkfrho
      Real*8 w,w2,w3,w4,w5,w6,w7,w8
      Real*8 d1rw
      Real*8 A2,A3,A4,A12,A32,A52,A72
      Real*8 X
      Real*8 s2,s3,s4,s5,s6

      Real*8 H,F
      Real*8 Hnum,Hden,d1sHnum,d1sHden
      Real*8 d1sH,d1sF
      Real*8 G_a,G_b,EG
      Real*8 d1sG_a,d1sG_b,d1sEG

      Real*8 Hsbw,Hsbw2,Hsbw3,Hsbw4,Hsbw12,Hsbw32,Hsbw52,Hsbw72
      Real*8 DHsbw,DHsbw2,DHsbw3,DHsbw4,DHsbw5
      Real*8 DHsbw12,DHsbw32,DHsbw52,DHsbw72,DHsbw92
      Real*8 d1sHsbw,d1rHsbw
      Real*8 d1sDHsbw,d1rDHsbw
      Real*8 HsbwA94,HsbwA9412
      Real*8 HsbwA942,HsbwA943,HsbwA945
      Real*8 piexperf,expei
      Real*8 piexperfd1,expeid1
      Real*8 d1spiexperf,d1sexpei
      Real*8 d1rpiexperf,d1rexpei
      Real*8 expei1,expei2,expei3,expei4

      Real*8 DHs,DHs2,DHs3,DHs4,DHs72,DHs92,DHsw,DHsw2,DHsw52,DHsw72
      Real*8 d1sDHs,d1rDHsw

      Real*8 np1,np2
      Real*8 d1rnp1,d1rnp2
      Real*8 t1,t2t9,t10,t10d1
      Real*8 f2,f3,f4,f5,f6,f7,f8,f9
      Real*8 f2d1,f3d1,f4d1,f5d1,f6d1,f8d1,f9d1
      Real*8 d1sf2,d1sf3,d1sf4,d1sf5,d1sf6,d1sf7,d1sf8,d1sf9
      Real*8 d1rf2,d1rf3,d1rf4,d1rf5,d1rf6,d1rf7,d1rf8,d1rf9
      Real*8 d1st1,d1rt1
      Real*8 d1st2t9,d1rt2t9
      Real*8 d1st10,d1rt10
      Real*8 d1sterm1,d1rterm1,term1d1
      Real*8 d1sterm2
      Real*8 d1sterm3,d1rterm3
      Real*8 d1sterm4,d1rterm4
      Real*8 d1sterm5,d1rterm5

      Real*8 term1,term2,term3,term4,term5

      Real*8 ax,um,uk,ul
      Real*8 gc1,gc2

      Real*8 derf,derfc,fexpei
!      Real*8 ei
      Real*8 expint

      Real*8 Zero,One,Two,Three,Four,Five,Six,Seven,Eight,Nine,Ten
      Real*8 Fifteen,Sixteen
      Real*8 r12,r64,r36,r81,r256,r384,r864,r1944,r4374
      Real*8 r20,r25,r27,r48,r120,r128,r144,r288,r324,r512,r729
      Real*8 r30,r32,r75,r243,r2187,r6561,r40,r105,r54,r135
      Real*8 r1215,r15309

      Save Zero,One,Two,Three,Four,Five,Six,Seven,Eight,Nine,Ten
      Data Zero,One,Two,Three,Four,Five,Six,Seven,Eight,Nine,Ten
     $  / 0D0,1D0,2D0,3D0,4D0,5D0,6D0,7D0,8D0,9D0,10D0 /
      Save Fifteen,Sixteen
      Data Fifteen,Sixteen / 1.5D1, 1.6D1 /
      Save r36,r64,r81,r256,r384,r864,r1944,r4374
      Data r36,r64,r81,r256,r384,r864,r1944,r4374
     $  / 3.6D1,6.4D1,8.1D1,2.56D2,3.84D2,8.64D2,1.944D3,4.374D3 /
      Save r27,r48,r120,r128,r144,r288,r324,r512,r729
      Data r27,r48,r120,r128,r144,r288,r324,r512,r729
     $  / 2.7D1,4.8D1,1.2D2,1.28D2,1.44D2,2.88D2,3.24D2,5.12D2,7.29D2 /
      Save r20,r32,r243,r2187,r6561,r40
      Data r20,r32,r243,r2187,r6561,r40
     $  / 2.0d1,3.2D1,2.43D2,2.187D3,6.561D3,4.0d1 /
      Save r12,r25,r30,r54,r75,r105,r135,r1215,r15309
      Data r12,r25,r30,r54,r75,r105,r135,r1215,r15309
     $  / 1.2D1,2.5d1,3.0d1,5.4D1,7.5d1,1.05D2,1.35D2,1.215D3,1.5309D4 /

c     General constants

      f12    = 0.5d0
      f13    = One/Three
      f14    = 0.25d0
      f18    = 0.125d0

      f23    = Two * f13
      f43    = Two * f23

      f32    = 1.5d0
      f72    = 3.5d0
      f34    = 0.75d0
      f94    = 2.25d0
      f98    = 1.125d0
      f1516  = Fifteen / Sixteen

      pi     = ACos(-One)
      pi2    = pi*pi
      pi_23  = pi2**f13
      srpi   = sqrt(pi)

      Three_13 = Three**f13

c     Constants from fit

      ea1 = -1.128223946706117d0
      ea2 = 1.452736265762971d0
      ea3 = -1.243162299390327d0
      ea4 = 0.971824836115601d0
      ea5 = -0.568861079687373d0
      ea6 = 0.246880514820192d0
      ea7 = -0.065032363850763d0
      ea8 = 0.008401793031216d0

      eb1 = 1.455915450052607d0

c     Constants for PBE hole

      A      =  1.0161144d0
      B      = -3.7170836d-1
      C      = -7.7215461d-2
      D      =  5.7786348d-1
      E      = -5.1955731d-2
      X      = - Eight/Nine

c     Constants for fit of H(s) (PBE)

      Ha1    = 9.79681d-3
      Ha2    = 4.10834d-2
      Ha3    = 1.87440d-1
      Ha4    = 1.20824d-3
      Ha5    = 3.47188d-2

c     Constants for F(H) (PBE)

      Fc1    = 6.4753871d0
      Fc2    = 4.7965830d-1

c     Constants for polynomial expansion for EG for small s

      EGa1   = -2.628417880d-2
      EGa2   = -7.117647788d-2
      EGa3   =  8.534541323d-2

c     Constants for large x expansion of exp(x)*ei(-x)

      expei1 = 4.03640D0
      expei2 = 1.15198D0
      expei3 = 5.03627D0
      expei4 = 4.19160D0

c     Cutoff criterion below which to use polynomial expansion

      EGscut     = 8.0d-2
      wcutoff    = 1.4D1
      expfcutoff = 7.0D2

c     Calculate prelim variables

      xkf    = (Three*pi2*rho) ** f13
      xkfrho = xkf * rho

      A2 = A*A
      A3 = A2*A
      A4 = A3*A
      A12 = Sqrt(A)
      A32 = A12*A
      A52 = A32*A
      A72 = A52*A

      w      = omega / xkf
      w2    = w * w
      w3    = w2 * w
      w4    = w2 * w2
      w5    = w3 * w2
      w6    = w5 * w
      w7    = w6 * w
      w8    = w7 * w

      d1rw  = -(One/(Three*rho))*w

      X      = - Eight/Nine

      s2     = s*s
      s3     = s2*s
      s4     = s2*s2
      s5     = s4*s
      s6     = s5*s

c     Calculate wPBE enhancement factor

      Hnum    = Ha1*s2 + Ha2*s4
      Hden    = One + Ha3*s4 + Ha4*s5 + Ha5*s6

      H       = Hnum/Hden

      d1sHnum = Two*Ha1*s + Four*Ha2*s3
      d1sHden = Four*Ha3*s3 + Five*Ha4*s4 + Six*Ha5*s5

      d1sH    = (Hden*d1sHnum - Hnum*d1sHden) / (Hden*Hden)

      F      = Fc1*H + Fc2
      d1sF   = Fc1*d1sH

c     Change exponent of Gaussian if we're using the simple approx.

      if(w .gt. wcutoff) then

        eb1 = 2.0d0

      endif

c     Calculate helper variables (should be moved later on...)

      Hsbw = s2*H + eb1*w2
      Hsbw2 = Hsbw*Hsbw
      Hsbw3 = Hsbw2*Hsbw
      Hsbw4 = Hsbw3*Hsbw
      Hsbw12 = Sqrt(Hsbw)
      Hsbw32 = Hsbw12*Hsbw
      Hsbw52 = Hsbw32*Hsbw
      Hsbw72 = Hsbw52*Hsbw

      d1sHsbw  = d1sH*s2 + Two*s*H
      d1rHsbw  = Two*eb1*d1rw*w
      
      DHsbw = D + s2*H + eb1*w2
      DHsbw2 = DHsbw*DHsbw
      DHsbw3 = DHsbw2*DHsbw
      DHsbw4 = DHsbw3*DHsbw
      DHsbw5 = DHsbw4*DHsbw
      DHsbw12 = Sqrt(DHsbw)
      DHsbw32 = DHsbw12*DHsbw
      DHsbw52 = DHsbw32*DHsbw
      DHsbw72 = DHsbw52*DHsbw
      DHsbw92 = DHsbw72*DHsbw
      
      HsbwA94   = f94 * Hsbw / A
      HsbwA942  = HsbwA94*HsbwA94
      HsbwA943  = HsbwA942*HsbwA94
      HsbwA945  = HsbwA943*HsbwA942
      HsbwA9412 = Sqrt(HsbwA94)

      DHs    = D + s2*H
      DHs2   = DHs*DHs
      DHs3   = DHs2*DHs
      DHs4   = DHs3*DHs
      DHs72  = DHs3*sqrt(DHs)
      DHs92  = DHs72*DHs

      d1sDHs = Two*s*H + s2*d1sH

      DHsw   = DHs + w2
      DHsw2  = DHsw*DHsw
      DHsw52 = sqrt(DHsw)*DHsw2
      DHsw72 = DHsw52*DHsw

      d1rDHsw = Two*d1rw*w

      if(s .gt. EGscut) then

        G_a    = srpi * (Fifteen*E + Six*C*(One+F*s2)*DHs +
     1                   Four*B*(DHs2) + Eight*A*(DHs3))
     2                * (One / (Sixteen * DHs72))
     3                 - f34*pi*sqrt(A) * exp(f94*H*s2/A) *
     4                   (One - derf(f32*s*sqrt(H/A)))

        d1sG_a = (One/r32)*srpi *
     1           ((r36*(Two*H + d1sH*s) / (A12*sqrt(H/A)))
     2            + (One/DHs92) *
     3               (-Eight*A*d1sDHs*DHs3 - r105*d1sDHs*E
     4                -r30*C*d1sDHs*DHs*(One+s2*F)
     5                +r12*DHs2*(-B*d1sDHs + C*s*(d1sF*s + Two*F)))
     6            - ((r54*exp(f94*H*s2/A)*srpi*s*(Two*H+d1sH*s)*
     7               derfc(f32*sqrt(H/A)*s))
     8               / A12))

        G_b    = (f1516 * srpi * s2) / DHs72

        d1sG_b = (Fifteen*srpi*s*(Four*DHs - Seven*d1sDHs*s))
     1           / (r32*DHs92)

        EG     = - (f34*pi + G_a) / G_b

        d1sEG  = (-Four*d1sG_a*G_b + d1sG_b*(Four*G_a + Three*pi))
     1           / (Four*G_b*G_b)

      else

        EG    = EGa1 + EGa2*s2 + EGa3*s4
        d1sEG = Two*EGa2*s + Four*EGa3*s3

      endif

c    Calculate the terms needed in any case

      term2 =       (DHs2*B + DHs*C + Two*E + DHs*s2*C*F + Two*s2*EG) /
     1              (Two*DHs3)

      d1sterm2 = (-Six*d1sDHs*(EG*s2 + E) 
     1            + DHs2 * (-d1sDHs*B + s*C*(d1sF*s + Two*F))
     2            + Two*DHs * (Two*EG*s - d1sDHs*C 
     3            + s2 * (d1sEG - d1sDHs*C*F)))
     4           / (Two*DHs4)

      term3 = - w  * (Four*DHsw2*B + Six*DHsw*C + Fifteen*E
     1                + Six*DHsw*s2*C*F + Fifteen*s2*EG) /
     2               (Eight*DHs*DHsw52)

      d1sterm3 = w * (Two*d1sDHs*DHsw * (Four*DHsw2*B 
     1                   + Six*DHsw*C + Fifteen*E
     2                   + Three*s2*(Five*EG + Two*DHsw*C*F))
     3                + DHs * (r75*d1sDHs*(EG*s2 + E)
     4                   + Four*DHsw2*(d1sDHs*B 
     5                        - Three*s*C*(d1sF*s + Two*F))
     6                   - Six*DHsw*(-Three*d1sDHs*C 
     7                        + s*(Ten*EG + Five*d1sEG*s
     8                            - Three*d1sDHs*s*C*F))))
     9           / (Sixteen*DHs2*DHsw72)

      d1rterm3 = (-Two*d1rw*DHsw * (Four*DHsw2*B 
     1                   + Six*DHsw*C + Fifteen*E
     2                   + Three*s2*(Five*EG + Two*DHsw*C*F))
     3                + w * d1rDHsw * (r75*(EG*s2 + E) 
     4                   + Two*DHsw*(Two*DHsw*B + Nine*C 
     5                               + Nine*s2*C*F)))
     5           / (Sixteen*DHs*DHsw72)

      term4 = - w3 * (DHsw*C + Five*E + DHsw*s2*C*F + Five*s2*EG) /
     1               (Two*DHs2*DHsw52)

      d1sterm4 = (w3 * (Four*d1sDHs*DHsw * (DHsw*C + Five*E 
     1                       + s2 * (Five*EG + DHsw*C*F))
     2                  + DHs * (r25*d1sDHs*(EG*s2 + E)
     3                       - Two*DHsw2*s*C*(d1sF*s + Two*F)
     4                       + DHsw * (Three*d1sDHs*C + s*(-r20*EG
     5                             - Ten*d1sEG*s 
     6                             + Three*d1sDHs*s*C*F)))))
     7           / (Four*DHs3*DHsw72)
  
      d1rterm4 = (w2 * (-Six*d1rw*DHsw * (DHsw*C + Five*E 
     1                       + s2 * (Five*EG + DHsw*C*F))
     2                  + w * d1rDHsw * (r25*(EG*s2 + E) +
     3                       Three*DHsw*C*(One + s2*F))))
     4           / (Four*DHs2*DHsw72)

      term5 = - w5 * (E + s2*EG) /
     1               (DHs3*DHsw52)

      d1sterm5 = (w5 * (Six*d1sDHs*DHsw*(EG*s2 + E)
     1                  + DHs * (-Two*DHsw*s * (Two*EG + d1sEG*s)
     2                       + Five*d1sDHs * (EG*s2 + E))))
     3           / (Two*DHs4*DHsw72)

      d1rterm5 = (w4 * Five*(EG*s2 + E) * (-Two*d1rw*DHsw
     1                                     + d1rDHsw * w))
     2           / (Two*DHs3*DHsw72)


      if((s.gt.0.0d0).or.(w.gt.0.0d0)) then

        t10    = (f12)*A*Log(Hsbw / DHsbw)
        t10d1  = f12*A*(One/Hsbw - One/DHsbw)
        d1st10 = d1sHsbw*t10d1
        d1rt10 = d1rHsbw*t10d1

      endif

c     Calculate exp(x)*f(x) depending on size of x

      if(HsbwA94 .lt. expfcutoff) then

        piexperf = pi*Exp(HsbwA94)*dErfc(HsbwA9412)
!        expei    = Exp(HsbwA94)*Ei(-HsbwA94)
        expei    = Exp(HsbwA94)*(-EXPINT(1,HsbwA94))

      else

        piexperf = pi*(One/(srpi*HsbwA9412) 
     1             - One/(Two*Sqrt(pi*HsbwA943))
     2             + Three/(Four*Sqrt(pi*HsbwA945)))

        expei  = - (One/HsbwA94) *
     1             (HsbwA942 + expei1*HsbwA94 + expei2) /
     2             (HsbwA942 + expei3*HsbwA94 + expei4)

      endif

c     Calculate the derivatives (based on the orig. expression)
c     --> Is this ok? ==> seems to be ok...

      piexperfd1  = - (Three*srpi*sqrt(Hsbw/A))/(Two*Hsbw)
     1              + (Nine*piexperf)/(Four*A)
      d1spiexperf = d1sHsbw*piexperfd1
      d1rpiexperf = d1rHsbw*piexperfd1

      expeid1  = f14*(Four/Hsbw + (Nine*expei)/A)
      d1sexpei = d1sHsbw*expeid1
      d1rexpei = d1rHsbw*expeid1

      if (w .eq. Zero) then

c       Fall back to original expression for the PBE hole

        t1 = -f12*A*expei
        d1st1 = -f12*A*d1sexpei
        d1rt1 = -f12*A*d1rexpei

        if(s .gt. 0.0D0) then

          term1    = t1 + t10
          d1sterm1 = d1st1 + d1st10
          d1rterm1 = d1rt1 + d1rt10

          Fx_wpbe = X * (term1 + term2)

          d1sfx = X * (d1sterm1 + d1sterm2)
          d1rfx = X * d1rterm1

        else

          Fx_wpbe = 1.0d0

c TODO    This is checked to be true for term1
c         How about the other terms???

          d1sfx   = 0.0d0
          d1rfx   = 0.0d0

        endif


      elseif(w .gt. wcutoff) then

c       Use simple Gaussian approximation for large w

c       print *,rho,s," LARGE w"

        term1 = -f12*A*(expei+log(DHsbw)-log(Hsbw))

        term1d1  = - A/(Two*DHsbw) - f98*expei
        d1sterm1 = d1sHsbw*term1d1
        d1rterm1 = d1rHsbw*term1d1

        Fx_wpbe = X * (term1 + term2 + term3 + term4 + term5)

        d1sfx = X * (d1sterm1 + d1sterm2 + d1sterm3 
     1                        + d1sterm4 + d1sterm5)

        d1rfx = X * (d1rterm1 + d1rterm3 + d1rterm4 + d1rterm5)

      else

c       For everything else, use the full blown expression

c       First, we calculate the polynomials for the first term

        np1    = -f32*ea1*A12*w + r27*ea3*w3/(Eight*A12)
     $            - r243*ea5*w5/(r32*A32) + r2187*ea7*w7/(r128*A52)

        d1rnp1 = - f32*ea1*d1rw*A12 + (r81*ea3*d1rw*w2)/(Eight*A12)
     1           - (r1215*ea5*d1rw*w4)/(r32*A32)
     2           + (r15309*ea7*d1rw*w6)/(r128*A52)

        np2 = -A + f94*ea2*w2 - r81*ea4*w4/(Sixteen*A)
     $        + r729*ea6*w6/(r64*A2) - r6561*ea8*w8/(r256*A3)

        d1rnp2 =   f12*(Nine*ea2*d1rw*w) 
     1           - (r81*ea4*d1rw*w3)/(Four*A)
     2           + (r2187*ea6*d1rw*w5)/(r32*A2)
     3           - (r6561*ea8*d1rw*w7)/(r32*A3)

c       The first term is

        t1    = f12*(np1*piexperf + np2*expei)
        d1st1 = f12*(d1spiexperf*np1 + d1sexpei*np2)
        d1rt1 = f12*(d1rnp2*expei + d1rpiexperf*np1 +
     1               d1rexpei*np2 + d1rnp1*piexperf)

c       The factors for the main polynomoal in w and their derivatives

        f2    = (f12)*ea1*srpi*A / DHsbw12
        f2d1  = - ea1*srpi*A / (Four*DHsbw32)
        d1sf2 = d1sHsbw*f2d1
        d1rf2 = d1rHsbw*f2d1

        f3    = (f12)*ea2*A / DHsbw
        f3d1  = - ea2*A / (Two*DHsbw2)
        d1sf3 = d1sHsbw*f3d1
        d1rf3 = d1rHsbw*f3d1

        f4    =  ea3*srpi*(-f98 / Hsbw12 
     $           + f14*A / DHsbw32)
        f4d1  = ea3*srpi*((Nine/(Sixteen*Hsbw32))-
     $                    (Three*A/(Eight*DHsbw52)))
        d1sf4 = d1sHsbw*f4d1
        d1rf4 = d1rHsbw*f4d1

        f5    = ea4*(One/r128) * (-r144*(One/Hsbw) 
     $           + r64*(One/DHsbw2)*A)
        f5d1  = ea4*((f98/Hsbw2)-(A/DHsbw3))
        d1sf5 = d1sHsbw*f5d1
        d1rf5 = d1rHsbw*f5d1

        f6    = ea5*(Three*srpi*(Three*DHsbw52*(Nine*Hsbw-Two*A)
     $           + Four*Hsbw32*A2))
     $           / (r32*DHsbw52*Hsbw32*A)
        f6d1  = ea5*srpi*((r27/(r32*Hsbw52))-
     $              (r81/(r64*Hsbw32*A))-
     $              ((Fifteen*A)/(Sixteen*DHsbw72)))
        d1sf6 = d1sHsbw*f6d1
        d1rf6 = d1rHsbw*f6d1

        f7    = ea6*(((r32*A)/DHsbw3
     $           + (-r36 + (r81*s2*H)/A)/Hsbw2)) / r32
        d1sf7 = ea6*(Three*(r27*d1sH*DHsbw4*Hsbw*s2 + 
     $          Eight*d1sHsbw*A*(Three*DHsbw4 - Four*Hsbw3*A) + 
     $          r54*DHsbw4*s*(Hsbw - d1sHsbw*s)*H))/
     $          (r32*DHsbw4*Hsbw3*A)
        d1rf7 = ea6*d1rHsbw*((f94/Hsbw3)-((Three*A)/DHsbw4)
     $                     -((r81*s2*H)/(Sixteen*Hsbw3*A)))

        f8    = ea7*(-Three*srpi*(-r40*Hsbw52*A3
     $           +Nine*DHsbw72*(r27*Hsbw2-Six*Hsbw*A+Four*A2)))
     $           / (r128 * DHsbw72*Hsbw52*A2)
        f8d1  = ea7*srpi*((r135/(r64*Hsbw72)) + (r729/(r256*Hsbw32*A2))
     $                   -(r243/(r128*Hsbw52*A)) 
     $                   -((r105*A)/(r32*DHsbw92)))
        d1sf8 = d1sHsbw*f8d1
        d1rf8 = d1rHsbw*f8d1

        f9    = (r324*ea6*eb1*DHsbw4*Hsbw*A 
     $          + ea8*(r384*Hsbw3*A3 + DHsbw4*(-r729*Hsbw2
     $          + r324*Hsbw*A - r288*A2))) / (r128*DHsbw4*Hsbw3*A2)
        f9d1  = -((r81*ea6*eb1)/(Sixteen*Hsbw3*A))
     $          + ea8*((r27/(Four*Hsbw4))+(r729/(r128*Hsbw2*A2))
     $                -(r81/(Sixteen*Hsbw3*A))
     $                -((r12*A/DHsbw5)))
        d1sf9 = d1sHsbw*f9d1
        d1rf9 = d1rHsbw*f9d1

        t2t9    = f2*w  + f3*w2 + f4*w3 + f5*w4 + f6*w5
     $                  + f7*w6 + f8*w7 + f9*w8
        d1st2t9 = d1sf2*w + d1sf3*w2 + d1sf4*w3 + d1sf5*w4
     $                    + d1sf6*w5 + d1sf7*w6 + d1sf8*w7
     $                    + d1sf9*w8
        d1rt2t9 = d1rw*f2 + d1rf2*w + Two*d1rw*f3*w 
     1            + d1rf3*w2 + Three*d1rw*f4*w2 
     1            + d1rf4*w3 + Four*d1rw*f5*w3 
     2            + d1rf5*w4 + Five*d1rw*f6*w4 
     3            + d1rf6*w5 + Six*d1rw*f7*w5 
     4            + d1rf7*w6 + Seven*d1rw*f8*w6 
     5            + d1rf8*w7 + Eight*d1rw*f9*w7 + d1rf9*w8

c       The final value of term1 for 0 < omega < wcutoff is:

        term1 = t1 + t2t9 + t10

        d1sterm1 = d1st1 + d1st2t9 + d1st10
        d1rterm1 = d1rt1 + d1rt2t9 + d1rt10

c       The final value for the enhancement factor and its
c       derivatives is:

        Fx_wpbe = X * (term1 + term2 + term3 + term4 + term5)

        d1sfx = X * (d1sterm1 + d1sterm2 + d1sterm3 
     1                        + d1sterm4 + d1sterm5)

        d1rfx = X * (d1rterm1 + d1rterm3 + d1rterm4 + d1rterm5)

      endif

      end


C     ==================================================================
      FUNCTION EXPINT(N,X)
      IMPLICIT NONE
C     ==================================================================
      REAL*8 EXPINT
      REAL*8 X,EULER,EPS,FPMIN
      REAL*8 B,C,D,H,A,DEL,VACT,VSI  
      INTEGER N,MAXIT,I,J,N1

      PARAMETER (EPS=1D-9,FPMIN=1D-30)
      PARAMETER (MAXIT=100)
      PARAMETER (EULER=0.5772156649015328D0) 
C =-------------------------------------------------------------------=
     
      IF ((N<0).OR.(X<0.0D0).OR.((X==0.D0).AND.(N==0.OR.N==1)))
     &   CALL STOPGM('EXPINT',"Passing bad arguments in EXPINT!")

      N1=N-1
      IF (N==0) THEN
         EXPINT=EXP(-X)/X
      ELSE
         IF (X==0.0D0) THEN
            EXPINT=1.D0/N1
         ELSE IF (X>1.0D0) THEN
            B=X+N
            C=1.D0/FPMIN
            D=1.0D0/B
            H=D
            I=0
            DO WHILE ((I.LE.MAXIT).AND.(ABS(DEL-1.D0)>=EPS))
               I=I+1
               A=-I*(N1+I)
               B=B+2.D0
               D=1.0D0/(A*D+B)
               C=B+A/C
               DEL=C*D 
               H=H*DEL
            ENDDO 
            IF (I.GT.MAXIT) CALL STOPGM('EXPINT',
     &         "Error in EXPINT! (X>1.0) : Not converged!")
            EXPINT=H*EXP(-X) 
         ELSE 
            IF (N1==0) THEN 
               EXPINT=-LOG(X)-EULER
            ELSE
               EXPINT=1.0D0/N1
            ENDIF
            VACT=1.D0
            I=0
            DO WHILE ((I.LE.MAXIT).AND.(ABS(DEL)>=ABS(EXPINT)*EPS))
               I=I+1
               VACT=-VACT*X/I
               IF (I==N1) THEN
                  VSI=-EULER
                  DO J=1,N1
                     VSI=VSI+1.D0/J
                  END DO
                  DEL=VACT*(-LOG(X)+VSI)
               ELSE
                  DEL=-VACT/(I-N1)
               END IF
               EXPINT=EXPINT+DEL
            END DO
            IF (I.GT.MAXIT) CALL STOPGM('EXPINT',
     &         "Error in EXPINT! (X<1.0) : Not converged!")
         ENDIF
      END IF
      END FUNCTION EXPINT
C =-------------------------------------------------------------------=
