C hello emacs, this is -*- fortran -*-
C     ==================================================================
C     == INCLUDE FILE FOR FUNCTIONS LOCALIZED IN RECIPROCAL SPACE     ==
C     ==--------------------------------------------------------------==
C     == TGLOC     : calculate  orbitals localized in G space         ==
C     == TGLOCPRINT  : print orbitals loc. in G space                 ==
C     == TG_KICK   : starting random unitary transformation on C0     ==
C     == TG_COMPLEX: starting C0 already complex in real space        ==
C     == TG_REAL   : starting C0 real in real space, to be expanded   ==
C     == TG_TUNE_ST: tuning the step on the size of the gradient      ==
C     == TG_ANTISYMM: wfn forced  not to be symmetric wrt  G = 0      ==
C     == TG_LINESEARCH: tuning of the time step by 3 points method    ==
C     == TG_READ_MATRIX: read unitary  transformation in file MATRIX  ==
C     == TGLOCREALP: print orbitals in real space                     ==
C     == TGWANNIER:  localization in real space                       ==
C     ==--------------------------------------------------------------==
      INTEGER      LGLOCLOG
      PARAMETER    (LGLOCLOG = 13)
      LOGICAL      TGLOC,TGLOCPRINT,TG_KICK,TG_COMPLEX,
     &             TG_REAL,TG_TUNE_ST,TG_ANTISYMM,TG_PENALTY,
     &             TG_LINESEARCH,TG_READ_MATRIX,TGLOCREALP,
     &             TGWANNIER,TORBDIST
      COMMON/GLOCAL/TGLOC,TGLOCPRINT,TG_KICK,TG_COMPLEX,
     &             TG_REAL,TG_TUNE_ST,TG_ANTISYMM,TG_PENALTY,
     &             TG_LINESEARCH,TG_READ_MATRIX,TGLOCREALP,
     &             TGWANNIER,TORBDIST
C     ==--------------------------------------------------------------==
C     == GLOC_MAXS : Maximum number of optimization steps             ==
C     == GLOC_TYPE : Type of optimization functional                  ==
C     ==          1   SUM_(n,i) -<n|Gi|n>^2                           == 
C     ==          2   SUM_(n,i) 1-|<n|exp{iGi*r_0}|n>|^2              ==
C     == GLOC_OPT  : Optimisation procedure                           ==
C     ==          1 U matrix integration C0 update at the very end    ==
C     ==          2 U kept as identity + D_U, C0 update each step     ==
C     == GLOC_CONST: How to impose the unitaruty constraint           ==
C     ==          1 Lagrangian multipliers                            ==
C     ==          2 Exact orthogonalization by S^(-1/2)               ==
C     ==          3 Approximation U = 1+iA, + orthogonalization of C0 ==
C     == GLOC_FIRST : first orbital to be printed                     ==
C     == GLOC_LAST  : last  orbital to be printed                     ==
C     == GLOC_ALL   : all   orbital to be printed                     ==
C     == GLOC_ORB : number of orbitals to be printed                  ==
C     == GLOC_LIST: list of orbitals to be printed                    ==
C     == GLOC_MAXIT: Maximum number of U constraint iteration         ==
C     == GLOC_INIT : First step -1, in case of restart                ==
C     ==--------------------------------------------------------------==
      INTEGER      GLOC_MAXS,GLOC_TYPE,GLOC_OPT,
     &             GLOC_CONST,GLOC_ORB,GLOC_FIRST,
     *             GLOC_LAST,GLOC_ALL,GLOC_LIST(*),GLOC_MAXIT,
     &             GLOC_INIT
      POINTER(IP_GLOCLIST,GLOC_LIST)
      COMMON/GLOCI/GLOC_MAXS,GLOC_TYPE,GLOC_OPT,
     &        GLOC_CONST,GLOC_ORB,GLOC_FIRST,
     *        GLOC_LAST,GLOC_ALL,GLOC_MAXIT,GLOC_INIT
      COMMON/GLOCP/IP_GLOCLIST
C     ==--------------------------------------------------------------==
C     == GLOC_STEP : step size for optimisation                       ==
C     == GLOC_EPS  : convergence criteria                             ==
C     == GLOC_RAN  : randomization amplitude                          ==
C     == GEPSLAG   : convergence criteria for Lagrangian multipliers  ==
C     == WAN_WEIGHT : weight of R spread functional (wannier) (def=0) ==
C     == G2-G_WEIGHT: weight of G spread functional      (def=1)      ==
C     ==--------------------------------------------------------------==
      INTEGER      LGLOCPAR
      parameter    (LGLOCPAR = 6)
      REAL*8       GLOC_STEP,GLOC_EPS,GLOC_RAN, GEPSLAG,
     &              WAN_WEIGHT,G2G_WEIGHT
      COMMON/GLOCR/GLOC_STEP,GLOC_EPS,GLOC_RAN, GEPSLAG,
     &              WAN_WEIGHT,G2G_WEIGHT
C     ==--------------------------------------------------------------==
      COMPLEX*16    LAG_MUL
      REAL*8        OMEGA_TOT,OMEGA_N,GMAX,OFUN,OFUN0,GGNORM,DIF_FUN,
     &              XYZFUN,MASS_R,MASS_G,SHIFTKX,SHIFTKY,SHIFTKZ,
     &              WAN_MEM,G2G_MEM
      INTEGER       NSTEP,GIFIRST,LGLOCRE
      parameter     (LGLOCRE = 11)
      pointer       (IP_LAG_MUL,LAG_MUL),(IP_OMEGA_N,OMEGA_N)
      COMMON/GLOC_IP/IP_LAG_MUL, IP_OMEGA_N
      COMMON/GLOC_RE/OFUN,OMEGA_TOT,GMAX,OFUN0,GGNORM,DIF_FUN,XYZFUN,
     &              MASS_R,MASS_G
      COMMON/GLOC_IN/NSTEP,GIFIRST
      COMMON/SHK/SHIFTKX,SHIFTKY,SHIFTKZ,WAN_MEM,G2G_MEM
      LOGICAL       STATE_ALL,STATE_RANGE,STATE_LIST
      INTEGER       IST_FIRST,IST_LAST,NST_LIST,IST_LIST,IND_ST
      pointer       (IP_IST_LIST,IST_LIST),(IP_IND_ST,IND_ST)
      COMMON/INDSTATE/ IST_FIRST,IST_LAST,NST_LIST
      COMMON/LOSTATE/ STATE_ALL,STATE_RANGE,STATE_LIST
      COMMON/IPSTATE/ IP_IST_LIST,IP_IND_ST
C     ==================================================================
      CHARACTER*20         FILSPR,FILCEN,FILPEN,FILMAT
      COMMON/GLOC_FILE/ FILSPR,FILCEN,FILPEN,FILMAT

C     ==================================================================


