C     ==================================================================
      SUBROUTINE U_BY_ORTHO(U_MAT,U_GRAD,Us_MAT,OV_MAT,SCR,
     &                     LSCR,NSTATE,DELTA)
C     ==--------------------------------------------------------------==
C     ==   ORTHOGONALIZE THE MATRIX U IN ORDER TO KEEP THE UNITARITY  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'g_loc.inc'

      INTEGER    NSTATE,LSCR
      REAL*8     SCR(LSCR)
      COMPLEX*16 U_MAT(NSTATE,NSTATE),U_GRAD(NSTATE,NSTATE)
      complex*16 Us_MAT(NSTATE,NSTATE),OV_MAT(NSTATE,NSTATE)
      REAL*8     DELTA
      INTEGER    I,J,LENGTH
      COMPLEX*16 ZZERO,ZONE
      PARAMETER (ZZERO=(0.0D0,0.0D0),ZONE=(1.0D0,0.0D0))
C     ==--------------------------------------------------------------==

      LENGTH = 3*2*NSTATE+2*2*NSTATE+NSTATE
      CALL  TEST_SCR('APPROX2','LENGTH',LSCR,LENGTH)

      IF(GLOC_OPT .EQ. 1) THEN
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            U_MAT(I,J)  = U_MAT(I,J) + DELTA*U_GRAD(I,J)
          ENDDO
        ENDDO
      ELSE       
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            U_MAT(I,J)  =  DELTA*U_GRAD(I,J)
          ENDDO
            U_MAT(I,I)  =  U_MAT(I,I) + 1.0D0
        ENDDO
      ENDIF

      CALL DCOPY(2*NSTATE*NSTATE,U_MAT,1,Us_MAT,1)

      CALL OVLAP2_C(NSTATE,NSTATE,NSTATE,OV_MAT,Us_MAT,U_MAT)

      CALL DCOPY(2*NSTATE*NSTATE,OV_MAT,1,Us_MAT,1)

      CALL  S12ORTHO(Us_MAT,OV_MAT,SCR,LSCR,NSTATE)

      CALL ZGEMM('N','N',NSTATE,NSTATE,NSTATE,ZONE,U_MAT,
     &           NSTATE,OV_MAT,NSTATE,ZZERO,Us_MAT,NSTATE)

      CALL DCOPY(2*NSTATE*NSTATE,Us_MAT,1,U_MAT,1)

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE  S12ORTHO(Us_MAT,OV_MAT,SCR,LSCR,NSTATE)
      IMPLICIT NONE
      include 'system.h'

      INTEGER    NSTATE,LSCR
      REAL*8     SCR(LSCR)
      complex*16 Us_MAT(NSTATE,NSTATE),OV_MAT(NSTATE,NSTATE),ZONE
      REAL*8     RE,IM,RE1,IM1
      INTEGER    I,J,K
      COMPLEX*16 WORK(*)
      REAL*8     AUX(3*2*NSTATE),W(NSTATE)
      pointer    (IP_WORK,WORK),(IP_AUX,AUX),(IP_W,W)
      INTEGER    LWORK,IERR,L_W,L_AUX

      L_W   = NSTATE
      L_AUX = 3*2*NSTATE
      LWORK  = MAX(1,2*2*NSTATE-1)

      CALL GET_ADDR(IP_W     ,SCR(1))
      CALL GET_ADDR(IP_AUX   ,SCR(1+L_W))
      CALL GET_ADDR(IP_WORK  ,SCR(1+L_W+L_AUX))

      CALL ZHEEV('V','U',NSTATE,Us_MAT,NSTATE,W,WORK,LWORK,AUX,IERR)

      DO I=1,NSTATE
        W(I)=1.D0/SQRT(W(I))
      ENDDO

      CALL ZAZZERO(OV_MAT,NSTATE*NSTATE)
      DO K=1,NSTATE
        DO J=1,NSTATE
          RE = W(K)*DREAL(Us_MAT(J,K))
          IM =-W(K)*DIMAG(Us_MAT(J,K))
          DO I=1,NSTATE
           RE1 = RE*DREAL(Us_MAT(I,K)) - IM*DIMAG(Us_MAT(I,K))
           IM1 = RE*DIMAG(Us_MAT(I,K)) + IM*DREAL(Us_MAT(I,K))
           OV_MAT(I,J) =  OV_MAT(I,J) + DCMPLX(RE1,IM1)
          ENDDO
        ENDDO
      ENDDO

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LAGRMULT(U_MAT,U_TEMP,A,B1,B2,GAM,SCR,LSCR,NSTATE)
C     ==--------------------------------------------------------------==
C     ==    CALCULATES THE MATRIX OF LAGRANGE MULTIPLIERS             ==
C     ==    IN ORDER TO IMPOSE THE UNITARITY CONDITION                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      include 'g_loc.inc'

c     Arguments
      INTEGER       NSTATE,LSCR
      COMPLEX*16    U_MAT(NSTATE,NSTATE),A(NSTATE,NSTATE),
     &              B1(NSTATE,NSTATE),B2(NSTATE,NSTATE),
     &              GAM(NSTATE*NSTATE),U_TEMP(NSTATE,NSTATE)
      REAL*8        SCR(LSCR)

c     Variables
      COMPLEX*16    SCR1(NSTATE*NSTATE),SCR2(NSTATE*NSTATE)
      pointer       (IP_SCR1,SCR1),(IP_SCR2,SCR2)
      COMPLEX*16    ZONE,ZZERO,ZGIVE
      EXTERNAL      ZGIVE
      REAL*8        DIFGAM
      DIMENSION     LAG_MUL(NSTATE*NSTATE)
      INTEGER       I,J,II,ISUB,ITER,LENGTH,IMAX,IZAMAX

c     correction due to orthonormality constraint
c     iterative scheme for evaluation of Lagrangian multipliers
c     LAG_MUL_new = 0.5[1 - A +(1-B^*)LAG_MUL_old + 
c                   LAG_MUL_old(1-B) + LAG_MUL_old LAG_MUL_old ]
c     A = U_MAT_new * U_MAT_new
c     B = U_MAT_old * U_MAT_new


C     ==-------------------------------------------------------------==
      CALL TISET('  LAGRMULT',ISUB)

      LENGTH = 2*2*NSTATE*NSTATE
      CALL TEST_SCR('LAGRMULT','LENGTH',LSCR,LENGTH)

      ZZERO = DCMPLX(0.0D0,0.0D0)
      ZONE  = DCMPLX(1.0D0,0.0D0)


      CALL GET_ADDR(IP_SCR1  ,SCR(1))
      CALL GET_ADDR(IP_SCR2  ,SCR(1+2*NSTATE*NSTATE))

      CALL ZGEMM('C','N',NSTATE,NSTATE,NSTATE,ZONE,U_TEMP,NSTATE,
     &             U_TEMP,NSTATE,ZZERO,A(1,1),NSTATE)


      CALL ZGEMM('C','N',NSTATE,NSTATE,NSTATE,ZONE,U_MAT,NSTATE,
     &             U_TEMP,NSTATE,ZZERO,B1(1,1),NSTATE)

      DO I=1,NSTATE
        DO J=1,NSTATE
           B2(J,I)=DCONJG(B1(I,J))
        ENDDO
      ENDDO

      CALL DSCAL(2*NSTATE*NSTATE,-1.0D0,A(1,1),1)
      CALL DSCAL(2*NSTATE*NSTATE,-1.0D0,B1(1,1),1)
      CALL DSCAL(2*NSTATE*NSTATE,-1.0D0,B2(1,1),1)

      DO I=1,NSTATE
        A(I,I)=DCMPLX(1.0D0,0.0D0)+A(I,I)
        B1(I,I)=DCMPLX(1.0D0,0.0D0)+B1(I,I)
        B2(I,I)=DCMPLX(1.0D0,0.0D0)+B2(I,I)
      ENDDO

C     ==----------------------------------------------------------==
C     == INITIALIZE LAG_MUL                                       ==
C     ==----------------------------------------------------------==
      DO J=1,NSTATE 
        DO I=1,NSTATE
          II=I+(J-1)*NSTATE
          LAG_MUL(II)=A(I,J)*0.5D0
        ENDDO
      ENDDO

C     ==----------------------------------------------------------==
C     == INITIALIZE FIXED TERM OF RECURRENCE                      ==
C     ==----------------------------------------------------------==
      CALL ZGEMM('N','N',NSTATE,NSTATE,NSTATE,(0.5D0,0.0D0),B2(1,1),
     *           NSTATE,B1(1,1),NSTATE,(0.5D0,0.0D0),A(1,1),NSTATE)

      ITER=0
C     ==----------------------------------------------------------==
C     == ITERATIVE LOOP                                           ==
C     ==----------------------------------------------------------==

100   CONTINUE
      ITER=ITER+1
      CALL DCOPY(2*NSTATE*NSTATE,A(1,1),1,GAM(1),1)
      DO J=1,NSTATE
        DO I=1,NSTATE
          II=I+(J-1)*NSTATE
          SCR1(II)=B1(I,J)-LAG_MUL(II)
          SCR2(II)=B2(I,J)-LAG_MUL(II)
        ENDDO
      ENDDO

      CALL ZGEMM('N','N',NSTATE,NSTATE,NSTATE,(-0.5D0,0.0D0),
     *          SCR2,NSTATE,SCR1,NSTATE,ZONE,GAM(1),NSTATE)

      DO I=1,NSTATE*NSTATE
        SCR1(I)=GAM(I)-LAG_MUL(I)
      ENDDO
      CALL DCOPY(2*NSTATE*NSTATE,GAM(1),1,LAG_MUL(1),1)
      IMAX=IZAMAX(NSTATE*NSTATE,SCR1,1)
      DIFGAM=CDABS(ZGIVE(SCR1(1),IMAX))
      CALL DCOPY(2*NSTATE*NSTATE,A(1,1),1,GAM(1),1)

c     convergence criterium

      IF(DIFGAM.GT.GEPSLAG*0.5D0.AND.ITER.LE.GLOC_MAXIT) GOTO 100
      IF(ITER.GT.GLOC_MAXIT+20) THEN
        PRINT *, 'U_UPD_SPREAD:DIFGAM=',DIFGAM,' ITER = ',
     &                 ITER,' > ',GLOC_MAXIT
        PRINT *, '       MAXIMUM NUMBER OF ITERATIONS EXCEEDED'
        CALL STOPGM('U_UPD_SPREAD',' ')
      ENDIF

C     ==--------------------------------------------------------------==
C     == UPDATE  U_MAT_new by  LAG_MU                                 ==
C     =---------------------------------------------------------------==
      CALL ZGEMM('N','N',NSTATE,NSTATE,NSTATE,ZONE,U_MAT,NSTATE,
     &           LAG_MUL,NSTATE,ZONE,U_TEMP,NSTATE)
C     ==--------------------------------------------------------------==
      CALL TIHALT('  LAGRMULT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C
      SUBROUTINE APPROX_U(U_MAT,U_GRAD,NSTATE)
C     ==--------------------------------------------------------------==
C     ==      U APPROXIMATED BY 1 +iA WHERE A IS HERMITIAN
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'

      INTEGER   NSTATE
      REAL*8    U_MAT(2,NSTATE,NSTATE),U_GRAD(2,NSTATE,NSTATE)
      REAL*8    RE,IM,FAC
      INTEGER   I,J


      FAC = 0.5D0

      DO I = 1,NSTATE
        DO J = I+1,NSTATE
          RE = FAC*(U_GRAD(1,I,J)-U_GRAD(1,J,I))
          IM = FAC*(U_GRAD(2,I,J)+U_GRAD(2,J,I))
          U_MAT(1,I,J) =  RE
          U_MAT(2,I,J) =  IM
          U_MAT(1,J,I) = -RE
          U_MAT(2,J,I) =  IM
        ENDDO
          U_MAT(1,I,I) = 1.0D0 + U_MAT(1,I,I)
      ENDDO
C     =--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     ==================================================================
      SUBROUTINE SIMM_OP(C1,OMEGA_N,OMEGA_TOT,NSTATE)
C     ==--------------------------------------------------------------==
C     ==    CALCULATE THE PENALTY FUNCTION 
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'

      INTEGER     NSTATE
      REAL*8      C1(2,NGWK,NSTATE)
      REAL*8      OMEGA_N(NSTATE),OMEGA_TOT

      INTEGER     ISTATE,IG
      REAL*8      ReA,ImA,ReB,ImB,ReO,ImO


      OMEGA_TOT = 0.0D0
      DO ISTATE = 1,NSTATE
        OMEGA_N(ISTATE) = 0.0D0
        DO IG = 1,NGW
          ReA = C1(1,IG,ISTATE)
          ImA = C1(2,IG,ISTATE)
          ReB = C1(1,IG+NGW,ISTATE)
          ImB = C1(2,IG+NGW,ISTATE)
          ReO = ReA*ReB + ImA*ImB
          ImO = ReA*ImB - ImA*ReB
          OMEGA_N(ISTATE) = OMEGA_N(ISTATE) + ReO*ReO + ImO*ImO
        ENDDO
        OMEGA_TOT = OMEGA_TOT + OMEGA_N(ISTATE)
      ENDDO

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     ==================================================================
      SUBROUTINE  OMEGA_GRAD(C1,U_GRAD,NSTATE,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     ==  CALCULATES THE GRADIENT OF THE SIMMETRY  FUNCTIONAL         ==
C     ==  WITH RESPECT TO THE UNITARY MATRIX COEFFICIENTS             ==
C     ==  U_GRAD_ij = -d OMEGA / d U_ij                               ==
C     ==  the gradient is calculated in U = 1                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'

C input
      INTEGER    NSTATE,LSCR
      REAL*8     C1(2,NGWK,NSTATE),U_GRAD(2,NSTATE,NSTATE)
      REAL*8     SCR(2,NSTATE,NSTATE)
      REAL*8     ReA_I,ImA_I,ReA_J,ImA_J,
     &           ReB_I,ImB_I,ReB_J,ImB_J,
     &           RE_IJ,IM_IJ,RE_JI,IM_JI,RE_JJ,IM_JJ

C local
      INTEGER    I,J,IG

C     ==--------------------------------------------------------------==

       CALL AZZERO(SCR,2*NSTATE*NSTATE)

        DO J = 1,NSTATE
          DO I = 1,NSTATE
            DO IG = 1,NGW
            ReA_I = C1(1,IG,I)
            ImA_I = C1(2,IG,I)
            ReB_I = C1(1,IG+NGW,I)
            ImB_I = C1(2,IG+NGW,I)
            ReA_J = C1(1,IG,J)
            ImA_J = C1(2,IG,J)
            ReB_J = C1(1,IG+NGW,J)
            ImB_J = C1(2,IG+NGW,J)

            RE_IJ = ReA_I*ReB_J + ImA_I*ImB_J
            IM_IJ = ReA_I*ImB_J - ImA_I*ReB_J

            RE_JI = ReA_J*ReB_I + ImA_J*ImB_I
            IM_JI = ReA_J*ImB_I - ImA_J*ReB_I

            RE_JJ = ReA_J*ReB_J + ImA_J*ImB_J
            IM_JJ = ReA_J*ImB_J - ImA_J*ReB_J

            SCR(1,I,J) = SCR(1,I,J) + RE_IJ*RE_JJ + IM_IJ*IM_JJ +
     &                   RE_JJ*RE_JI + IM_JJ*IM_JI
            SCR(2,I,J) = SCR(2,I,J) - RE_IJ*IM_JJ + IM_IJ*RE_JJ 
     &                   -RE_JJ*IM_JI + IM_JJ*RE_JI
            ENDDO

            U_GRAD(1,I,J) = U_GRAD(1,I,J) - SCR(1,I,J) 
            U_GRAD(2,I,J) = U_GRAD(2,I,J) - SCR(2,I,J) 

          ENDDO
        ENDDO

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==--------------------------------------------------------------==
C     ==================================================================
      SUBROUTINE LINE_SEARCH_EXE(U_MAT,U_GRAD,Z_MAT,XYZMAT,A_MAT,B_MAT,
     &                     SCR,LSCR,NSTATE,DELTA,FACTOR)
C     ==--------------------------------------------------------------==
C     ==   ORTHOGONALIZE THE MATRIX U IN ORDER TO KEEP THE UNITARITY  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'g_loc.inc'
      INCLUDE 'wann.inc'

      INTEGER    NSTATE,LSCR
      REAL*8     SCR(LSCR)
      COMPLEX*16 U_MAT(NSTATE,NSTATE),U_GRAD(NSTATE,NSTATE),
     &           Z_MAT(NSTATE,NSTATE,3),XYZMAT(*)
      complex*16 A_MAT(NSTATE,NSTATE),B_MAT(NSTATE,NSTATE)
      COMPLEX*16 Uis_MAT(NSTATE,NSTATE),ZOLD(NSTATE,NSTATE,3)
      pointer    (IP_Uis_MAT,Uis_MAT),(IP_ZOLD,ZOLD)
      complex*16 C_MAT(NSTATE,NSTATE),D_MAT(NSTATE,NSTATE)
      pointer    (IP_C_MAT,C_MAT),(IP_D_MAT,D_MAT)
      REAL*8     DELTA,FACTOR,DX,X0,OFUN1,OFUN2,AFAC,TOT_FUN
      INTEGER    I,J,LENGTH,IX1,IX2,LSCR1,LSCR2,ISUB
      COMPLEX*16 ZZERO,ZONE
C     ==--------------------------------------------------------------==
      CALL TISET(' LINE_SRCH',ISUB)

      LENGTH = 3*2*NSTATE+2*2*NSTATE+NSTATE+2*NSTATE*NSTATE
      CALL  TEST_SCR('LINESEARCH','LENGTH',LSCR,LENGTH)

      ZONE  = DCMPLX(1.0D0,0.0D0)
      ZZERO = DCMPLX(0.0D0,0.0D0)

C     INITIALIZE VARIABLE
      OFUN0  = G2G_WEIGHT*OFUN+ WAN_WEIGHT*XYZFUN
      AFAC   = 1.0D0
c      write(6,*) 1,G2G_WEIGHT,OFUN, WAN_WEIGHT,XYZFUN
C....  FIRST POINT
      CALL GET_ADDR(IP_Uis_MAT,SCR(1))
      IX1 = 1 + 2*NSTATE*NSTATE
      LSCR1 = LSCR - 2*NSTATE*NSTATE


C     TEMPORARY U
      IF(GLOC_OPT .EQ. 1) THEN
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            Uis_MAT(I,J)  = U_MAT(I,J) + DELTA*U_GRAD(I,J)
          ENDDO
        ENDDO
      ELSE       
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            Uis_MAT(I,J)  =  DELTA*U_GRAD(I,J)
          ENDDO
            Uis_MAT(I,I)  =  Uis_MAT(I,I) + 1.0D0
        ENDDO
      ENDIF

C     ORTHOGONALIZATION OF TEMPORARY U
      IF(GLOC_CONST .EQ. 2) THEN
        CALL DCOPY(2*NSTATE*NSTATE,Uis_MAT,1,A_MAT,1)
        CALL OVLAP2_C(NSTATE,NSTATE,NSTATE,B_MAT,A_MAT,Uis_MAT)
        CALL DCOPY(2*NSTATE*NSTATE,B_MAT,1,A_MAT,1)
        CALL  S12ORTHO(A_MAT,B_MAT,SCR(IX1),LSCR1,NSTATE)
        CALL ZGEMM('N','N',NSTATE,NSTATE,NSTATE,ZONE,Uis_MAT,
     &           NSTATE,B_MAT,NSTATE,ZZERO,A_MAT,NSTATE)
        CALL DCOPY(2*NSTATE*NSTATE,A_MAT,1,Uis_MAT,1)

      ELSEIF(GLOC_CONST .EQ. 1) THEN

        CALL AZZERO(SCR(IX1),LSCR1)
        CALL GET_ADDR(  IP_C_MAT,SCR(IX1))
        CALL GET_ADDR(IP_D_MAT,SCR(IX1+2*NSTATE*NSTATE))
        IX2      = IX1+2*2*NSTATE*NSTATE
        LSCR2    = LSCR1 -  2*2*NSTATE*NSTATE
        CALL  LAGRMULT(U_MAT,Uis_MAT,A_MAT,B_MAT,C_MAT,D_MAT,
     &                 SCR(IX2),LSCR2,NSTATE)

      ENDIF


C     STORE Z_MAT AND UPDATE Z_MAT AND FUNCTIONAL
      LENGTH = 3*2*NSTATE*NSTATE+2*2*NSTATE*NSTATE
      CALL  TEST_SCR('LINESEARCH','ZOLD',LSCR1,LENGTH)
      IF(G2G_WEIGHT .EQ. 0.0D0) GOTO 100
      CALL GET_ADDR(IP_ZOLD,SCR(IX1))
      IX2     = IX1 + 3*2*NSTATE*NSTATE
c      CALL AZZERO(SCR(IX1),LSCR1)
      CALL DCOPY(3*2*NSTATE*NSTATE,Z_MAT,1,ZOLD,1)
      CALL Z_UPDATE(Uis_MAT,Z_MAT,SCR(IX2),NSTATE,NSTATE,3)
      CALL GZFUNC(Z_MAT,NSTATE,NSTATE)
      CALL DCOPY(3*2*NSTATE*NSTATE,ZOLD,1,Z_MAT,1)
 100  CONTINUE
      IF(TGWANNIER .AND. WAN_WEIGHT .NE. 0.0D0) THEN
        CALL GET_ADDR(IP_ZOLD,SCR(IX1))
        IX2     = IX1 + NWANOPT*2*NSTATE*NSTATE
c        CALL AZZERO(SCR(IX1),LSCR1)
        CALL DCOPY(NWANOPT*2*NSTATE*NSTATE,XYZMAT,1,ZOLD,1)
        CALL Z_UPDATE(Uis_MAT,XYZMAT,SCR(IX2),NSTATE,NSTATE,NWANOPT)
        CALL GXYZFUNC(XYZMAT,NSTATE)
        CALL DCOPY(NWANOPT*2*NSTATE*NSTATE,ZOLD,1,XYZMAT,1)
      ENDIF

      TOT_FUN = G2G_WEIGHT*OFUN + WAN_WEIGHT*XYZFUN
C     COMPARE OLD AND NEW FUNCTIONAL
      IF(TOT_FUN.GT.OFUN0) THEN
        OFUN2= TOT_FUN
        AFAC=0.5D0
      ELSE
        OFUN1= TOT_FUN
        AFAC=2.0D0
      ENDIF

c      CALL AZZERO(SCR,LSCR)

C....  SECOND POINT
      CALL GET_ADDR(IP_Uis_MAT,SCR(1))
      IX1 = 1 + 2*NSTATE*NSTATE
      LSCR1 = LSCR - 2*NSTATE*NSTATE

C     TEMPORARY U
      IF(GLOC_OPT .EQ. 1) THEN
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            Uis_MAT(I,J)  = U_MAT(I,J) + AFAC * DELTA*U_GRAD(I,J)
          ENDDO
        ENDDO
      ELSE       
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            Uis_MAT(I,J)  =  AFAC * DELTA*U_GRAD(I,J)
          ENDDO
            Uis_MAT(I,I)  =  Uis_MAT(I,I) + 1.0D0
        ENDDO
      ENDIF
C     ORTHOGONALIZATION OF TEMPORARY U
      IF(GLOC_CONST .EQ. 2) THEN
        CALL DCOPY(2*NSTATE*NSTATE,Uis_MAT,1,A_MAT,1)
        CALL OVLAP2_C(NSTATE,NSTATE,NSTATE,B_MAT,A_MAT,Uis_MAT)
        CALL DCOPY(2*NSTATE*NSTATE,B_MAT,1,A_MAT,1)
        CALL  S12ORTHO(A_MAT,B_MAT,SCR(IX1),LSCR1,NSTATE)
        CALL ZGEMM('N','N',NSTATE,NSTATE,NSTATE,ZONE,Uis_MAT,
     &           NSTATE,B_MAT,NSTATE,ZZERO,A_MAT,NSTATE)
        CALL DCOPY(2*NSTATE*NSTATE,A_MAT,1,Uis_MAT,1)

      ELSEIF(GLOC_CONST .EQ. 1) THEN

        CALL AZZERO(SCR(IX1),LSCR1)
        CALL GET_ADDR(IP_C_MAT,SCR(IX1))
        CALL GET_ADDR(IP_D_MAT,SCR(IX1+2*NSTATE*NSTATE))
        IX2      = IX1+2*2*NSTATE*NSTATE
        LSCR2    = LSCR1 -  2*2*NSTATE*NSTATE
        CALL  LAGRMULT(U_MAT,Uis_MAT,A_MAT,B_MAT,C_MAT,D_MAT,
     &                 SCR(IX2),LSCR2,NSTATE)

      ENDIF

C     STORE Z_MAT AND UPDATE Z_MAT AND FUNCTIONAL
      IF(G2G_WEIGHT .EQ. 0.0D0)  GOTO 200
      CALL GET_ADDR(IP_ZOLD,SCR(IX1))
      IX2     = IX1 + 3*2*NSTATE*NSTATE
c      CALL AZZERO(SCR(IX1),LSCR1)
      CALL DCOPY(3*2*NSTATE*NSTATE,Z_MAT,1,ZOLD,1)
      CALL Z_UPDATE(Uis_MAT,Z_MAT,SCR(IX2),NSTATE,NSTATE,3)
      CALL GZFUNC(Z_MAT,NSTATE,NSTATE)
      CALL DCOPY(3*2*NSTATE*NSTATE,ZOLD,1,Z_MAT,1)
 200  CONTINUE
      IF(TGWANNIER .AND. WAN_WEIGHT .NE. 0.0D0) THEN
        CALL GET_ADDR(IP_ZOLD,SCR(IX1))
        IX2     = IX1 + NWANOPT*2*NSTATE*NSTATE
c        CALL AZZERO(SCR(IX1),LSCR1)
        CALL DCOPY(NWANOPT*2*NSTATE*NSTATE,XYZMAT,1,ZOLD,1)
        CALL Z_UPDATE(Uis_MAT,XYZMAT,SCR(IX2),NSTATE,NSTATE,NWANOPT)
        CALL GXYZFUNC(XYZMAT,NSTATE)
        CALL DCOPY(NWANOPT*2*NSTATE*NSTATE,ZOLD,1,XYZMAT,1)
      ENDIF

      TOT_FUN = G2G_WEIGHT*OFUN + WAN_WEIGHT*XYZFUN
      IF(AFAC.GT.1.D0) THEN
        OFUN2=TOT_FUN
        DX=1.0D0
      ELSE
        OFUN1=TOT_FUN
        DX=0.5D0
      ENDIF

C     Extrapolation
      IF((2.0D0*OFUN0-4.0D0*OFUN1+2.0D0*OFUN2).EQ.0.D0) THEN
        X0=DX
      ELSE
        X0=DX*(4.0D0*OFUN1-OFUN2-3.0D0*OFUN0)/
     &            (2.0D0*OFUN0-4.0D0*OFUN1+2.0D0*OFUN2)
      ENDIF
c      write(6,'(I5,6(2x,1PE12.4))') NSTEP,DX,OFUN0,OFUN1,OFUN2,X0,DELTA
c      ALPHA=(OFUN1-OFUN0)/DX/(2*X0+DX)
c      EE=OFUN0-ALPHA*X0*X0
      AFAC=-X0
      IF(AFAC.GT.0.D0) THEN
        IF(AFAC.GT.3.D0) AFAC=3.0D0
      ELSE
        IF(OFUN2.LT.OFUN0) THEN
          AFAC=2*DX
        ELSEIF(OFUN1.LT.OFUN0) THEN
          AFAC=DX
        ELSE
          AFAC=0.05D0
        ENDIF
      ENDIF
c      DE=EE+ALPHA*(X0+AFAC)**2
c      ALPHA=ALPHA/DD

c.....FINAL EVALUATION
c      CALL AZZERO(SCR,LSCR)


C     ORTHOGONALIZATION OF TEMPORARY U
      IF(GLOC_CONST .EQ. 2) THEN

C     TEMPORARY U
      IF(GLOC_OPT .EQ. 1) THEN
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            U_MAT(I,J)  = U_MAT(I,J) + AFAC * DELTA*U_GRAD(I,J)
          ENDDO
        ENDDO
      ELSE       
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            U_MAT(I,J)  =  AFAC * DELTA*U_GRAD(I,J)
          ENDDO
            U_MAT(I,I)  =  U_MAT(I,I) + 1.0D0
        ENDDO
      ENDIF

        CALL DCOPY(2*NSTATE*NSTATE,U_MAT,1,A_MAT,1)
        CALL OVLAP2_C(NSTATE,NSTATE,NSTATE,B_MAT,A_MAT,U_MAT)
        CALL DCOPY(2*NSTATE*NSTATE,B_MAT,1,A_MAT,1)
        CALL  S12ORTHO(A_MAT,B_MAT,SCR(IX1),LSCR1,NSTATE)
        CALL ZGEMM('N','N',NSTATE,NSTATE,NSTATE,ZONE,U_MAT,
     &           NSTATE,B_MAT,NSTATE,ZZERO,A_MAT,NSTATE)
        CALL DCOPY(2*NSTATE*NSTATE,A_MAT,1,U_MAT,1)

      ELSEIF(GLOC_CONST .EQ. 1) THEN

C     TEMPORARY U
      IF(GLOC_OPT .EQ. 1) THEN
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            Uis_MAT(I,J)  = U_MAT(I,J) + AFAC * DELTA*U_GRAD(I,J)
          ENDDO
        ENDDO
      ELSE       
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            Uis_MAT(I,J)  =  AFAC * DELTA*U_GRAD(I,J)
          ENDDO
            Uis_MAT(I,I)  =  Uis_MAT(I,I) + 1.0D0
        ENDDO
      ENDIF

        CALL AZZERO(SCR(IX1),LSCR1)
        CALL GET_ADDR(IP_C_MAT,SCR(IX1))
        CALL GET_ADDR(IP_D_MAT,SCR(IX1+2*NSTATE*NSTATE))
        IX2      = IX1+2*2*NSTATE*NSTATE
        LSCR2    = LSCR1 -  2*2*NSTATE*NSTATE
        CALL  LAGRMULT(U_MAT,Uis_MAT,A_MAT,B_MAT,C_MAT,D_MAT,
     &                 SCR(IX2),LSCR2,NSTATE)

        CALL DCOPY(2*NSTATE*NSTATE,Uis_MAT,1,U_MAT,1)
      ENDIF


      IF(AFAC.LT.0.1D0) THEN
        FACTOR=FACTOR*0.25D0
      ELSEIF(AFAC.GT.1.00D0) THEN
        FACTOR=FACTOR*1.50D0
      ELSEIF(AFAC.LT.0.20D0) THEN
        FACTOR=FACTOR*0.75D0
      ENDIF
c      write(6,'(I8,7(1PE12.4))') NSTEP, ofun0,ofun1-ofun0,ofun2-ofun0,
c     &                          ofun2-ofun1,afac,x0,factor

      CALL TIHALT(' LINE_SRCH',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     ==================================================================
      SUBROUTINE LINE_SEARCH_S(U_MAT,U_GRAD,Z1_MAT,Z2_MAT,A_MAT,
     &                     B_MAT,SCR,LSCR,NSTATE,DELTA,FACTOR)
C     ==--------------------------------------------------------------==
C     ==   ORTHOGONALIZE THE MATRIX U IN ORDER TO KEEP THE UNITARITY  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'g_loc.inc'

      INTEGER    NSTATE,LSCR
      REAL*8     SCR(LSCR)
      COMPLEX*16 U_MAT(NSTATE,NSTATE),U_GRAD(NSTATE,NSTATE),
     &           Z1_MAT(NSTATE,NSTATE,3),Z2_MAT(NSTATE,NSTATE,3)
      complex*16 A_MAT(NSTATE,NSTATE),B_MAT(NSTATE,NSTATE)
      COMPLEX*16 Uis_MAT(NSTATE,NSTATE),ZOLD(NSTATE*NSTATE*6)
      pointer    (IP_Uis_MAT,Uis_MAT),(IP_ZOLD,ZOLD)
      REAL*8     DELTA,FACTOR,DX,X0,OFUN1,OFUN2,AFAC
      INTEGER    I,J,LENGTH,IX1,IX2,IX3,IP1,LSCR1
      COMPLEX*16 ZZERO,ZONE
C     ==--------------------------------------------------------------==


      LENGTH = 3*2*NSTATE+2*2*NSTATE+NSTATE+2*NSTATE*NSTATE
      CALL  TEST_SCR('LINESEARCH','LENGTH',LSCR,LENGTH)

      ZONE  = DCMPLX(1.0D0,0.0D0)
      ZZERO = DCMPLX(0.0D0,0.0D0)

C     INITIALIZE VARIABLE
      OFUN0  = OFUN
      AFAC   = 1.0D0

C....  FIRST POINT
      CALL GET_ADDR(IP_Uis_MAT,SCR(1))
      IX1 = 1 + 2*NSTATE*NSTATE
      LSCR1 = LSCR - 2*NSTATE*NSTATE


C     TEMPORARY U
      IF(GLOC_OPT .EQ. 1) THEN
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            Uis_MAT(I,J)  = U_MAT(I,J) + DELTA*U_GRAD(I,J)
          ENDDO
        ENDDO
      ELSE       
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            Uis_MAT(I,J)  =  DELTA*U_GRAD(I,J)
          ENDDO
            Uis_MAT(I,I)  =  Uis_MAT(I,I) + 1.0D0
        ENDDO
      ENDIF
C     ORTHOGONALIZATION OF TEMPORARY U
      CALL DCOPY(2*NSTATE*NSTATE,Uis_MAT,1,A_MAT,1)
      CALL OVLAP2_C(NSTATE,NSTATE,NSTATE,B_MAT,A_MAT,Uis_MAT)
      CALL DCOPY(2*NSTATE*NSTATE,B_MAT,1,A_MAT,1)

      CALL  S12ORTHO(A_MAT,B_MAT,SCR(IX1),LSCR1,NSTATE)

      CALL ZGEMM('N','N',NSTATE,NSTATE,NSTATE,ZONE,Uis_MAT,
     &           NSTATE,B_MAT,NSTATE,ZZERO,A_MAT,NSTATE)
      CALL DCOPY(2*NSTATE*NSTATE,A_MAT,1,Uis_MAT,1)


C     STORE Z_MAT AND UPDATE Z_MAT AND FUNCTIONAL
      LENGTH = 2*3*2*NSTATE*NSTATE+2*2*NSTATE*NSTATE
      CALL  TEST_SCR('LINESEARCH','ZOLD',LSCR1,LENGTH)
      CALL GET_ADDR(IP_ZOLD,SCR(IX1))
      IX2     = IX1 + 3*2*NSTATE*NSTATE
      IP1     = 1   + 3*NSTATE*NSTATE  !3*2*NSTATE*NSTATE
      IX3     = IX2 + 3*2*NSTATE*NSTATE
      CALL AZZERO(ZOLD,2*3*2*NSTATE*NSTATE)


      CALL DCOPY(3*2*NSTATE*NSTATE,Z1_MAT,1,ZOLD(1),1)

      CALL DCOPY(3*2*NSTATE*NSTATE,Z2_MAT,1,ZOLD(IP1),1)
      CALL Z12_UPDATE(Uis_MAT,Z1_MAT,Z2_MAT,SCR(IX3),NSTATE,NSTATE)
      CALL SPREADFUNC(Z1_MAT,Z2_MAT,NSTATE,NSTATE)
      CALL DCOPY(3*2*NSTATE*NSTATE,ZOLD(1),1,Z1_MAT,1)
      CALL DCOPY(3*2*NSTATE*NSTATE,ZOLD(IP1),1,Z2_MAT,1)


C     COMPARE OLD AND NEW FUNCTIONAL
      IF(OFUN.GT.OFUN0) THEN
        OFUN2=OFUN
        AFAC=0.5D0
      ELSE
        OFUN1=OFUN
        AFAC=2.0D0
      ENDIF

      CALL AZZERO(SCR,LSCR)

C....  SECOND POINT
      CALL GET_ADDR(IP_Uis_MAT,SCR(1))
      IX1 = 1 + 2*NSTATE*NSTATE
      LSCR1 = LSCR - 2*NSTATE*NSTATE

C     TEMPORARY U
      IF(GLOC_OPT .EQ. 1) THEN
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            Uis_MAT(I,J)  = U_MAT(I,J) + AFAC * DELTA*U_GRAD(I,J)
          ENDDO
        ENDDO
      ELSE       
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            Uis_MAT(I,J)  =  AFAC * DELTA*U_GRAD(I,J)
          ENDDO
            Uis_MAT(I,I)  =  Uis_MAT(I,I) + 1.0D0
        ENDDO
      ENDIF
C     ORTHOGONALIZATION OF TEMPORARY U
      CALL DCOPY(2*NSTATE*NSTATE,Uis_MAT,1,A_MAT,1)
      CALL OVLAP2_C(NSTATE,NSTATE,NSTATE,B_MAT,A_MAT,Uis_MAT)
      CALL DCOPY(2*NSTATE*NSTATE,B_MAT,1,A_MAT,1)
      CALL  S12ORTHO(A_MAT,B_MAT,SCR(IX1),LSCR1,NSTATE)
      CALL ZGEMM('N','N',NSTATE,NSTATE,NSTATE,ZONE,Uis_MAT,
     &           NSTATE,B_MAT,NSTATE,ZZERO,A_MAT,NSTATE)
      CALL DCOPY(2*NSTATE*NSTATE,A_MAT,1,Uis_MAT,1)

C     STORE Z_MAT AND UPDATE Z_MAT AND FUNCTIONAL
      LENGTH = 2*3*2*NSTATE*NSTATE+2*2*NSTATE*NSTATE
      CALL  TEST_SCR('LINESEARCH','ZOLD',LSCR1,LENGTH)
      CALL GET_ADDR(IP_ZOLD,SCR(IX1))
      IX2     = IX1 + 3*2*NSTATE*NSTATE
      IP1     = 1   + 3*NSTATE*NSTATE  !3*2*NSTATE*NSTATE
      IX3     = IX2 + 3*2*NSTATE*NSTATE
      CALL AZZERO(SCR(IX1),LSCR1)
      CALL DCOPY(3*2*NSTATE*NSTATE,Z1_MAT,1,ZOLD(1),1)
      CALL DCOPY(3*2*NSTATE*NSTATE,Z2_MAT,1,ZOLD(IP1),1)
      CALL Z12_UPDATE(Uis_MAT,Z1_MAT,Z2_MAT,SCR(IX3),NSTATE,NSTATE)
      CALL SPREADFUNC(Z1_MAT,Z2_MAT,NSTATE,NSTATE)
      CALL DCOPY(3*2*NSTATE*NSTATE,ZOLD(1),1,Z1_MAT,1)
      CALL DCOPY(3*2*NSTATE*NSTATE,ZOLD(IP1),1,Z2_MAT,1)

      IF(AFAC.GT.1.D0) THEN
        OFUN2=OFUN
        DX=1.0D0
      ELSE
        OFUN1=OFUN
        DX=0.5D0
      ENDIF
C     Extrapolation
      IF((2*OFUN0-4*OFUN1+2*OFUN2).EQ.0.D0) THEN
        X0=DX
      ELSE
        X0=DX*(4*OFUN1-OFUN2-3*OFUN0)/(2*OFUN0-4*OFUN1+2*OFUN2)
      ENDIF
c      ALPHA=(OFUN1-OFUN0)/DX/(2*X0+DX)
c      EE=OFUN0-ALPHA*X0*X0
      AFAC=-X0
      IF(AFAC.GT.0.D0) THEN
        IF(AFAC.GT.3.D0) AFAC=3.0D0
      ELSE
        IF(OFUN2.LT.OFUN0) THEN
          AFAC=2*DX
        ELSEIF(OFUN1.LT.OFUN0) THEN
          AFAC=DX
        ELSE
          AFAC=0.05D0
        ENDIF
      ENDIF
c      DE=EE+ALPHA*(X0+AFAC)**2
c      ALPHA=ALPHA/DD

c.....FINAL EVALUATION
      CALL AZZERO(SCR,LSCR)

C     TEMPORARY U
      IF(GLOC_OPT .EQ. 1) THEN
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            U_MAT(I,J)  = U_MAT(I,J) + AFAC * DELTA*U_GRAD(I,J)
          ENDDO
        ENDDO
      ELSE       
        DO I = 1,NSTATE
          DO J = 1,NSTATE
            U_MAT(I,J)  =  AFAC * DELTA*U_GRAD(I,J)
          ENDDO
            U_MAT(I,I)  =  U_MAT(I,I) + 1.0D0
        ENDDO
      ENDIF
C     ORTHOGONALIZATION OF TEMPORARY U
      CALL DCOPY(2*NSTATE*NSTATE,U_MAT,1,A_MAT,1)
      CALL OVLAP2_C(NSTATE,NSTATE,NSTATE,B_MAT,A_MAT,U_MAT)
      CALL DCOPY(2*NSTATE*NSTATE,B_MAT,1,A_MAT,1)
      CALL  S12ORTHO(A_MAT,B_MAT,SCR(IX1),LSCR1,NSTATE)
      CALL ZGEMM('N','N',NSTATE,NSTATE,NSTATE,ZONE,U_MAT,
     &           NSTATE,B_MAT,NSTATE,ZZERO,A_MAT,NSTATE)
      CALL DCOPY(2*NSTATE*NSTATE,A_MAT,1,U_MAT,1)


      IF(AFAC.LT.0.1D0) THEN
        FACTOR=FACTOR*0.25D0
      ELSEIF(AFAC.GT.1.0D0) THEN
        FACTOR=FACTOR*1.5D0
      ELSEIF(AFAC.LT.0.2D0) THEN
        FACTOR=FACTOR*0.75D0
      ENDIF
c      write(6,*)  ofun0,ofun1,ofun2,factor
c      write(6,'(I8,7(1PE12.4))') NSTEP, ofun0,ofun1-ofun0,ofun2-ofun0,
c     &                          ofun2-ofun1,afac,x0,factor

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
