C     ==================================================================
      SUBROUTINE GCENER(SGCX,SGCC,RHOE,V,VTEMP,VTMP,GRAD,TSTRESS)
C     ==--------------------------------------------------------------==
C     ==   CALCULATION OF THE GRADIENT CORRECTION TO THE EXCHANGE     ==
C     ==   AND CORRELATION FUNCTIONAL                                 ==
C     ==                                                              ==
C     ==   ON INPUT : RHOE : DENSITY IN REAL SPACE                    ==
C     ==              V    : UNDEFINED                                ==
C     ==              GRAD : (NABLA*RHOE)^2,RHOx,RHOy,RHOz            ==
C     ==              VTEMP: POTENTIAL IN G SPACE                     ==
C     ==              VTMP : DENSITY IN G SPACE                       ==
C     ==   ON OUTPUT: RHOE : DENSITY IN REAL SPACE                    ==
C     ==              V    : TOTAL POTENTIAL IN REAL SPACE            ==
C     ==              GRAD : (NABLA*RHOE)^2,RHOx,RHOy,RHOz            ==
C     ==              VTEMP: UNDEFINED                                ==
C     ==              VTMP : UNDEFINED                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'strs.inc'
      INCLUDE 'func.inc'
      INCLUDE 'tbxc.inc'
C     Arguments
      REAL*8     SGCX,SGCC,RHOE(*),VTMP(*),GRAD(NNR1,*)
      COMPLEX*16 V(*),VTEMP(*)
      LOGICAL    TSTRESS
C     Variables
      COMPLEX*16 FA,FB,VG1,VG2,vnz,vin
      REAL*8     FLOPS,GMAX,EG,VFAC,VFAC2,VFAC3,SMFAC,GCS
      INTEGER    ISUB,IA,IB,IR,IG,KK,I1,I2,I3
C     ==--------------------------------------------------------------==
      IF(.NOT.TGCX .AND. .NOT.TGCC) RETURN
      CALL TISET('    GCENER',ISUB)
C     ==--------------------------------------------------------------==
C     ==  V1 = dF/dn stored in V(*)                                   ==
C     ==  V2 = (1/Nabla.n) dF/d(Nabla.n) stored in VTMP(*)            ==
C     ==--------------------------------------------------------------==
#ifdef _BGL 
      CALL ZAZZERO(V,NNR1)
#else
      CALL AZZERO(V,2*NNR1)
#endif
      CALL AZZERO(VTMP,NNR1)
      SGCX = 0.0D0
      SGCC = 0.0D0
      IF(TOLDCODE) THEN
        CALL GCOLD(SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
      ELSE
        IF(MGCX.EQ.1) THEN
          IF(MGCC.EQ.0) THEN
            CALL GCXONLY(BBETA,SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
          ELSEIF(MGCC.EQ.1) THEN
            CALL GCXP86(BBETA,SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
          ELSEIF(MGCC.EQ.2) THEN
            CALL GCXLYP(BBETA,SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
          ELSE
            CALL STOPGM('GCENER','NOT IMPLEMENTED')
          ENDIF
        ELSEIF(MGCX.EQ.2.AND.MGCC.EQ.3) THEN
          CALL GCGGA(SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
        ELSEIF(MGCX.EQ.3.AND.MGCC.EQ.4) THEN
          CALL GCPBE(SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
        ELSEIF(MGCX.EQ.4.AND.MGCC.EQ.4) THEN
          CALL GCrevPBE(SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
        ELSE
          CALL GCOLD(SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
        ENDIF
C       Set to 0.0 VTMP(1:KR1,1:KR2S,KR3S),VTMP(1:NR1,KR2S,:), and
C                  VTMP(KR1,1:KR2S,1:KR3S).
        DO I3=NR3S+1,KR3S
          CALL AZZERO(VTMP((I3-1)*KR1*KR2S+1),KR1*KR2S)
        ENDDO
        DO I3=1,NR3S
          DO I2=NR2S+1,KR2S
            CALL AZZERO(VTMP((I3-1)*KR1*KR2S+(I2-1)*KR1+1),KR1)
          ENDDO
          DO I2=1,NR2S
            DO I1=NR1+1,KR1
              VTMP((I3-1)*KR1*KR2S+(I2-1)*KR1+I1)=0.D0
            ENDDO
          ENDDO
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      IF(TSTRESS) THEN
C       Contribution of dF/d(delta.n) to the stress tensor
!$OMP parallel do private(KK,IR,IA,IB)
        DO KK=1,6
          IA=ALPHA(KK)+1
          IB=BETA(KK)+1
          DEGC(KK)=0.0D0
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
          DO IR=1,NNR1
            DEGC(KK)=DEGC(KK)+VTMP(IR)*GRAD(IR,IA)*GRAD(IR,IB)
          ENDDO
          DEGC(KK)=DEGC(KK)*OMEGA/(NR1S*NR2S*NR3S)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     FFT of V1 and V2*RHOx to G-Space
!$OMP parallel do private(IR)
#ifdef _BGL 
!IBM* INDEPENDENT
!IBM* ASSERT (ITERCNT(10000))
#endif
      DO IR=1,NNR1
        V(IR) = V(IR) + DCMPLX(0.0D0,VTMP(IR)*GRAD(IR,2))
      ENDDO
      CALL FWFFT(V)
      GCS=SMF*HG(NHG)
      IF(TSMOOTH) THEN
        GMAX=HG(NHG)
        CALL GLOMAX(1,GMAX)
        GCS=SMF*GMAX
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          EG = (HG(IG)-GCS)/(SDELTA*GMAX)
          SMFAC=1.0D0/(1.0D0+DEXP(EG))
          VFAC=SMFAC*TPIBA*GK(1,IG)
!!        FA  = V(NZH(IG)) + V(INDZ(IG))
!!        FB  = V(NZH(IG)) - V(INDZ(IG))
          vnz = v(nzh(ig))
          vin = v(indz(ig))
          FA  = vnz + vin
          FB  = vnz - vin
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG) = VTEMP(IG) + SMFAC*VG1 - VFAC*UIMAG*VG2
        ENDDO
      ELSE
!$OMP parallel do private(IG,VFAC,vnz,vin,FA,FB,VG1,VG2)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          VFAC=TPIBA*GK(1,IG)
!!        FA  = V(NZH(IG)) + V(INDZ(IG))
!!        FB  = V(NZH(IG)) - V(INDZ(IG))
          vnz = v(nzh(ig))
          vin = v(indz(ig))
          FA  = vnz + vin
          FB  = vnz - vin
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG) = VTEMP(IG) + VG1 - VFAC*UIMAG*VG2
        ENDDO
      ENDIF
C     FFT of V2*RHOy and V2*RHOz to G-Space
#ifdef _BGL 
      CALL AZZERO(V,2*NNR1)
#else
      CALL ZAZZERO(V,NNR1)
#endif
!$OMP parallel do private(IR)
#ifdef _BGL 
!IBM* INDEPENDENT
!IBM* ASSERT (ITERCNT(10000))
#endif
      DO IR=1,NNR1
        V(IR) = DCMPLX(VTMP(IR)*GRAD(IR,3),VTMP(IR)*GRAD(IR,4))
      ENDDO
      CALL FWFFT(V)
      GCS=SMF*HG(NHG)
      IF(TSMOOTH) THEN
        GMAX=HG(NHG)
        CALL GLOMAX(1,GMAX)
        GCS=SMF*GMAX
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          EG = (HG(IG)-GCS)/(SDELTA*GMAX)
          SMFAC=1.0D0/(1.0D0+DEXP(EG))
          VFAC2=SMFAC*TPIBA*GK(2,IG)
          VFAC3=SMFAC*TPIBA*GK(3,IG)
!!        FA  = V(NZH(IG)) + V(INDZ(IG))
!!        FB  = V(NZH(IG)) - V(INDZ(IG))
          vnz = v(nzh(ig))
          vin = v(indz(ig))
          FA  = vnz + vin
          FB  = vnz - vin
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG) = VTEMP(IG) - VFAC2*UIMAG*VG1 - VFAC3*UIMAG*VG2
        ENDDO
      ELSE
!$OMP parallel do private(IG,VFAC2,VFAC3,vnz,vin,FA,FB,VG1,VG2)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          VFAC2=TPIBA*GK(2,IG)
          VFAC3=TPIBA*GK(3,IG)
!!        FA  = V(NZH(IG)) + V(INDZ(IG))
!!        FB  = V(NZH(IG)) - V(INDZ(IG))
          vnz = v(nzh(ig))
          vin = v(indz(ig))
          FA  = vnz + vin
          FB  = vnz - vin
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG) = VTEMP(IG) - VFAC2*UIMAG*VG1 - VFAC3*UIMAG*VG2
        ENDDO
      ENDIF
#ifdef _BGL 
      CALL AZZERO(V,2*MAXFFT)
#else
      CALL ZAZZERO(V,MAXFFT)
#endif
!CDIR NODEP
#if defined(_vpp_) || defined(__PRIMERGY)
!OCL NOVREC
#endif
#ifdef CRAYX1
!dir$ concurrent
#endif
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT
#endif
      DO IG=1,NHG
        V(NZH(IG)) = VTEMP(IG)
        V(INDZ(IG)) = DCONJG(VTEMP(IG))
      ENDDO
      CALL INVFFT(V)
      CALL TIHALT('    GCENER',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GCLSD(SGCX,SGCC,RHOE,V,VTEMP,VTMP,GRAD,TSTRESS)
C     ==--------------------------------------------------------------==
C     ==   CALCULATION OF THE GRADIENT CORRECTION TO THE EXCHANGE     ==
C     ==   AND CORRELATION FUNCTIONAL FOR LSD CASE                    ==
C     ==                                                              ==
C     ==   ON INPUT : RHOE : DENSITY IN REAL SPACE                    ==
C     ==              V    : UNDEFINED                                ==
C     ==              GRAD : (NABLA*RHOE)^2,RHOx,RHOy,RHOz ALPHA      ==
C     ==                     (NABLA*RHOE)^2,RHOx,RHOy,RHOz BETA       ==
C     ==              VTEMP: POTENTIAL IN G SPACE                     ==
C     ==              VTMP : DENSITY IN G SPACE                       ==
C     ==   ON OUTPUT: RHOE : DENSITY IN REAL SPACE                    ==
C     ==              V    : TOTAL POTENTIAL IN REAL SPACE            ==
C     ==              GRAD : (NABLA*RHOE)^2,RHOx,RHOy,RHOz ALPHA      ==
C     ==                     (NABLA*RHOE)^2,RHOx,RHOy,RHOz BETA       ==
C     ==              VTEMP: UNDEFINED                                ==
C     ==              VTMP : UNDEFINED                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'strs.inc'
      INCLUDE 'func.inc'
      INCLUDE 'tbxc.inc'
C     Arguments
      REAL*8     SGCX,SGCC,RHOE(NNR1,2),VTMP(NNR1,3),
     &           GRAD(NNR1,8)
      COMPLEX*16 V(MAXFFT,2),VTEMP(NHG,2)
      LOGICAL TSTRESS
C     Variables
      COMPLEX*16 FA,FB,VG1,VG2,vnz,vin
      REAL*8     FLOPS,GCS,EG,SMFAC,VFAC,VFAC2,VFAC3
      INTEGER    ISUB,IR,IG,IA,IB,KK
C     ==--------------------------------------------------------------==
      IF(.NOT.TGCX .AND. .NOT.TGCC) RETURN
      CALL TISET('     GCLSD',ISUB)
C     ==--------------------------------------------------------------==
C     ==  V1 = dF/dn stored in V(*)                                   ==
C     ==  V2 = (1/Nabla.n) dF/d(Nabla.n) stored in VTMP(*)            ==
C     ==--------------------------------------------------------------==
#ifdef _BGL 
      CALL AZZERO(V,4*MAXFFT)
#else
      CALL ZAZZERO(V,2*MAXFFT)
#endif
      CALL AZZERO(VTMP,4*NHG)
      SGCX = 0.0D0
      SGCC = 0.0D0
C     ==--------------------------------------------------------------==
C     ==  POTENTIALS                                                  ==
C     ==--------------------------------------------------------------==
      IF(TOLDCODE) THEN
        CALL GCLSDOLD(SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
      ELSE
        IF(MGCX.EQ.1) THEN
          IF(MGCC.EQ.0) THEN
            CALL GCSXONLY(BBETA,SGCX,SGCC,RHOE,V(1,1),V(1,2),
     *                    VTMP,GRAD,FLOPS)
          ELSEIF(MGCC.EQ.1) THEN
            CALL GCSXP86(BBETA,SGCX,SGCC,RHOE,V(1,1),V(1,2),
     *                   VTMP,GRAD,FLOPS)
          ELSEIF(MGCC.EQ.2) THEN
            CALL GCSXLYP(BBETA,SGCX,SGCC,RHOE,V(1,1),V(1,2),
     *                   VTMP,GRAD,FLOPS)
          ELSE
            CALL STOPGM('GCENER','NOT IMPLEMENTED')
          ENDIF
        ELSEIF(MGCX.EQ.3 .AND. MGCC.EQ.4) THEN
          CALL GCSPBE(SGCX,SGCC,RHOE,V(1,1),V(1,2),VTMP,GRAD,FLOPS)
        ELSEIF(MGCX.EQ.4 .AND. MGCC.EQ.4) THEN
          CALL GCSrevPBE(SGCX,SGCC,RHOE,V(1,1),V(1,2),VTMP,GRAD,FLOPS)
        ELSE
C       this is also somewhat broken for some combinations of functionals.
          CALL GCLSDOLD(SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
        ENDIF
      ENDIF
C HACK_TEST
c       DO I3=NR3S+1,KR3S
c         CALL AZZERO(VTMP((I3-1)*KR1*KR2S+1,1),KR1*KR2S)
c       ENDDO
c       DO I3=1,NR3S
c         DO I2=NR2S+1,KR2S
c           CALL AZZERO(VTMP((I3-1)*KR1*KR2S+(I2-1)*KR1+1,1),KR1)
c         ENDDO
c         DO I2=1,NR2S
c           DO I1=NR1+1,KR1
c             VTMP((I3-1)*KR1*KR2S+(I2-1)*KR1+I1,1)=0.D0
c           ENDDO
c         ENDDO
c       ENDDO
c       DO I3=NR3S+1,KR3S
c         CALL AZZERO(VTMP((I3-1)*KR1*KR2S+1,2),KR1*KR2S)
c       ENDDO
c       DO I3=1,NR3S
c         DO I2=NR2S+1,KR2S
c           CALL AZZERO(VTMP((I3-1)*KR1*KR2S+(I2-1)*KR1+1,2),KR1)
c         ENDDO
c         DO I2=1,NR2S
c           DO I1=NR1+1,KR1
c             VTMP((I3-1)*KR1*KR2S+(I2-1)*KR1+I1,2)=0.D0
c           ENDDO
c         ENDDO
c       ENDDO

      IF(TSTRESS) THEN
C       Contribution of dF/d(delta.n) to the stress tensor
!$OMP parallel do private(KK,IR,IA,IB)
        DO KK=1,6
          IA=ALPHA(KK)+1
          IB=BETA(KK)+1
          DEGC(KK)=0.0D0
#ifdef _BGL
!IBM* ASSERT (ITERCNT(10000))
#endif
          DO IR=1,NNR1
       DEGC(KK)=DEGC(KK)+(VTMP(IR,1)+VTMP(IR,3))*
     &              GRAD(IR,IA)*GRAD(IR,IB)
          ENDDO
        ENDDO
        DO KK=1,6
          IA=ALPHA(KK)+5
          IB=BETA(KK)+5
#ifdef _BGL
!IBM* ASSERT (ITERCNT(10000))
#endif
          DO IR=1,NNR1
            DEGC(KK)=DEGC(KK)+(VTMP(IR,2)+VTMP(IR,3))*
     &             GRAD(IR,IA)*GRAD(IR,IB)
          ENDDO
          DEGC(KK)=DEGC(KK)*OMEGA/(NR1S*NR2S*NR3S)
        ENDDO
      ENDIF
C HACK_TEST
C     ==--------------------------------------------------------------==
C     ==  ALPHA SPIN                                                  ==
C     ==--------------------------------------------------------------==
C     FFT of V1 and V2*RHOx to G-Space
!$OMP parallel do private(IR)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT
#endif
      DO IR=1,NNR1
        V(IR,1) = V(IR,1) + DCMPLX(0.0D0,VTMP(IR,1)*GRAD(IR,2))
     *                    + DCMPLX(0.0D0,VTMP(IR,3)*GRAD(IR,6))
      ENDDO
      CALL FWFFT(V(1,1))
      GCS=SMF*HG(NHG)
      IF(TSMOOTH) THEN
        GCS=SMF*HG(NHG)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          EG = (HG(IG)-GCS)/SDELTA
          SMFAC=1.0D0/(1.0D0+DEXP(EG))
          VFAC=SMFAC*TPIBA*GK(1,IG)
!!        FA  = V(NZH(IG),1) + V(INDZ(IG),1)
!!        FB  = V(NZH(IG),1) - V(INDZ(IG),1)
          vnz = v(nzh(ig),1)
          vin = v(indz(ig),1)
          FA  = vnz + vin
          FB  = vnz - vin
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG,1) = VTEMP(IG,1) + SMFAC*VG1 - VFAC*UIMAG*VG2
        ENDDO
      ELSE
!$OMP parallel do private(IG,VFAC,vnz,vin,FA,FB,VG1,VG2)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          VFAC=TPIBA*GK(1,IG)
!!        FA  = V(NZH(IG),1) + V(INDZ(IG),1)
!!        FB  = V(NZH(IG),1) - V(INDZ(IG),1)
          vnz = v(nzh(ig),1)
          vin = v(indz(ig),1)
          FA  = vnz + vin
          FB  = vnz - vin
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG,1) = VTEMP(IG,1) + VG1 - VFAC*UIMAG*VG2
        ENDDO
      ENDIF
C     FFT of V2*RHOy and V2*RHOz to G-Space
#ifdef _BGL 
      CALL AZZERO(V(1,1),2*NNR1)
#else
      CALL ZAZZERO(V(1,1),NNR1)
#endif
!$OMP parallel do private(IR)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT
#endif
      DO IR=1,NNR1
        V(IR,1) = DCMPLX(VTMP(IR,1)*GRAD(IR,3),VTMP(IR,1)*GRAD(IR,4))
     *           +DCMPLX(VTMP(IR,3)*GRAD(IR,7),VTMP(IR,3)*GRAD(IR,8))
      ENDDO
      CALL FWFFT(V(1,1))
      GCS=SMF*HG(NHG)
      IF(TSMOOTH) THEN
        GCS=SMF*HG(NHG)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          EG = (HG(IG)-GCS)/SDELTA
          SMFAC=1.0D0/(1.0D0+DEXP(EG))
          VFAC2=SMFAC*TPIBA*GK(2,IG)
          VFAC3=SMFAC*TPIBA*GK(3,IG)
!!        FA  = V(NZH(IG),1) + V(INDZ(IG),1)
!!        FB  = V(NZH(IG),1) - V(INDZ(IG),1)
          vnz = v(nzh(ig),1)
          vin = v(indz(ig),1)
          FA  = vnz + vin
          FB  = vnz - vin
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG,1) = VTEMP(IG,1) - VFAC2*UIMAG*VG1 - VFAC3*UIMAG*VG2
        ENDDO
      ELSE
!$OMP parallel do private(IG,VFAC2,VFAC3,vnz,vin,FA,FB,VG1,VG2)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          VFAC2=TPIBA*GK(2,IG)
          VFAC3=TPIBA*GK(3,IG)
!!        FA  = V(NZH(IG),1) + V(INDZ(IG),1)
!!        FB  = V(NZH(IG),1) - V(INDZ(IG),1)
          vnz = v(nzh(ig),1)
          vin = v(indz(ig),1)
          FA  = vnz + vin 
          FB  = vnz - vin 
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG,1) = VTEMP(IG,1) - VFAC2*UIMAG*VG1 - VFAC3*UIMAG*VG2
        ENDDO
      ENDIF
#ifdef _BGL 
      CALL AZZERO(V(1,1),2*NNR1)
#else
      CALL ZAZZERO(V(1,1),NNR1)
#endif
!CDIR NODEP
#if defined(_vpp_) || defined(__PRIMERGY)
!OCL NOVREC
#endif
#ifdef CRAYX1
!dir$ concurrent
#endif
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
!IBM# INDEPENDENT
#endif
#if defined(__SR11000) || defined(__PRIMERGY)
!$OMP parallel do private(IG)
#endif
      DO IG=1,NHG
        V(NZH(IG),1) = VTEMP(IG,1)
        V(INDZ(IG),1) = DCONJG(VTEMP(IG,1))
      ENDDO
      CALL INVFFT(V(1,1))
C     ==--------------------------------------------------------------==
C     ==  BETA SPIN                                                   ==
C     ==--------------------------------------------------------------==
C     FFT of V1 and V2*RHOx to G-Space
!$OMP parallel do private(IR)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
!IBM# INDEPENDENT
#endif
      DO IR=1,NNR1
        V(IR,2) = V(IR,2) + DCMPLX(0.0D0,VTMP(IR,2)*GRAD(IR,6))
     *                    + DCMPLX(0.0D0,VTMP(IR,3)*GRAD(IR,2))
      ENDDO
      CALL FWFFT(V(1,2))
      GCS=SMF*HG(NHG)
      IF(TSMOOTH) THEN
        GCS=SMF*HG(NHG)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          EG = (HG(IG)-GCS)/SDELTA
          SMFAC=1.0D0/(1.0D0+DEXP(EG))
          VFAC=SMFAC*TPIBA*GK(1,IG)
!!        FA  = V(NZH(IG),2) + V(INDZ(IG),2)
!!        FB  = V(NZH(IG),2) - V(INDZ(IG),2)
          vnz = v(nzh(ig),2) 
          vin = v(indz(ig),2)
          FA  = vnz + vin
          FB  = vnz - vin
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG,2) = VTEMP(IG,2) + SMFAC*VG1 - VFAC*UIMAG*VG2
        ENDDO
      ELSE
!$OMP parallel do private(IG,VFAC,vnz,vin,FA,FB,VG1,VG2)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          VFAC=TPIBA*GK(1,IG)
!!        FA  = V(NZH(IG),2) + V(INDZ(IG),2)
!!        FB  = V(NZH(IG),2) - V(INDZ(IG),2)
          vnz = v(nzh(ig),2)
          vin = v(indz(ig),2)
          FA  = vnz + vin
          FB  = vnz - vin
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG,2) = VTEMP(IG,2) + VG1 - VFAC*UIMAG*VG2
        ENDDO
      ENDIF
C     FFT of V2*RHOy and V2*RHOz to G-Space
#ifdef _BGL 
      CALL AZZERO(V(1,2),2*NNR1)
#else
      CALL ZAZZERO(V(1,2),NNR1)
#endif
!$OMP parallel do private(IR)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT
#endif
      DO IR=1,NNR1
        V(IR,2) = DCMPLX(VTMP(IR,2)*GRAD(IR,7),VTMP(IR,2)*GRAD(IR,8))
     *           +DCMPLX(VTMP(IR,3)*GRAD(IR,3),VTMP(IR,3)*GRAD(IR,4))
      ENDDO
      CALL FWFFT(V(1,2))
      GCS=SMF*HG(NHG)
      IF(TSMOOTH) THEN
        GCS=SMF*HG(NHG)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          EG = (HG(IG)-GCS)/SDELTA
          SMFAC=1.0D0/(1.0D0+DEXP(EG))
          VFAC2=SMFAC*TPIBA*GK(2,IG)
          VFAC3=SMFAC*TPIBA*GK(3,IG)
!!        FA  = V(NZH(IG),2) + V(INDZ(IG),2)
!!        FB  = V(NZH(IG),2) - V(INDZ(IG),2)
          vnz = v(nzh(ig),2)
          vin = v(indz(ig),2)
          FA  = vnz + vin
          FB  = vnz - vin
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG,2) = VTEMP(IG,2) - VFAC2*UIMAG*VG1 - VFAC3*UIMAG*VG2
        ENDDO
      ELSE
!$OMP parallel do private(IG,VFAC2,VFAC3,vnz,vin,FA,FB,VG1,VG2)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
#endif
        DO IG=1,NHG
          VFAC2=TPIBA*GK(2,IG)
          VFAC3=TPIBA*GK(3,IG)
!!        FA  = V(NZH(IG),2) + V(INDZ(IG),2)
!!        FB  = V(NZH(IG),2) - V(INDZ(IG),2)
          vnz = v(nzh(ig),2)
          vin = v(indz(ig),2)
          FA  = vnz + vin
          FB  = vnz - vin
          VG1 = 0.5D0*DCMPLX(DREAL(FA),DIMAG(FB))
          VG2 = 0.5D0*DCMPLX(DIMAG(FA),-DREAL(FB))
          VTEMP(IG,2) = VTEMP(IG,2) - VFAC2*UIMAG*VG1 - VFAC3*UIMAG*VG2
        ENDDO
      ENDIF
#ifdef _BGL 
      CALL AZZERO(V(1,2),2*NNR1)
#else
      CALL ZAZZERO(V(1,2),NNR1)
#endif
!CDIR NODEP
#if defined(_vpp_) || defined(__PRIMERGY)
!OCL NOVREC
#endif
#ifdef CRAYX1
!dir$ concurrent
#endif
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT
#endif
#if defined(__SR11000) || defined(__PRIMERGY)
!$OMP parallel do private(IG)
#endif
      DO IG=1,NHG
        V(NZH(IG),2) = VTEMP(IG,2)
        V(INDZ(IG),2) = DCONJG(VTEMP(IG,2))
      ENDDO
      CALL INVFFT(V(1,2))
      CALL TIHALT('     GCLSD',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GCOLD(SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'strs.inc'
      INCLUDE 'tbxc.inc'
      INCLUDE 'tauf.inc'
      INCLUDE 'func.inc'
C     Arguments
      REAL*8     SGCX,SGCC,RHOE(NNR1),VTMP(NNR1),GRAD(NNR1,4),FLOPS
      COMPLEX*16 V(NNR1)
C     Variables
      REAL*8     TAU(*),VTAU(*)
      REAL*8     RHO,SMOO,DSMOO,TEXP,GRHO,V1,V2,SX,SC,V1X,V2X,V1C,V2C
      REAL*8     SXSR,V1XSR,V2XSR
      REAL*8     TAR,VTT
      INTEGER    IR
C     ==--------------------------------------------------------------==
      IF(.NOT.TTAU) THEN
C$OMP parallel do private(IR,RHO,SMOO,DSMOO,TEXP,GRHO,SX,SC)
C$OMP+            private(V1X,V2X,V1C,V2C,V1,V2,TAR)
C$OMP+            private(SXSR,V1XSR,V2XSR,SRXA)
C$OMP+            shared(GRAD,V,VTMP,GCEPS,NNR1,RHOE)
C$OMP+            reduction(+:SGCX,SGCC)
#ifdef __SR8000
*poption parallel
*poption tlocal(SX,SC,V1X,V2X,V1C,V2C)
#endif
#if defined (__SR11000)
*poption psum(SGCX,SGCC)
#endif
        DO IR=1,NNR1
          RHO   = DMAX1(RHOE(IR),0.0D0)
          IF(RHO.GT.0.1D0*GCEPS) THEN
            IF(RHO.GT.4.0D0*GCEPS) THEN
              SMOO  = 1.0D0
              DSMOO = 0.0D0
            ELSE
              TEXP=DEXP(3.0D0*(1.0D0-RHO/GCEPS))
              SMOO=1.0D0/(1.0D0+TEXP)
              DSMOO=3.0D0/GCEPS * TEXP*SMOO*SMOO
            ENDIF
            GRHO    = GRAD(IR,1)
            CALL GCXC(RHO,GRHO,SX,SC,V1X,V2X,V1C,V2C)
            IF(THYBRID) THEN
! wPBE stabbing, adding short-range GC
              IF(MGCSRX.NE.0) THEN
                CALL PBEXSR(RHO,GRHO,SXSR,V1XSR,V2XSR,SRXA)
                SX=SX*PXGC-PHFX*SXSR;
                V1X=V1X*PXGC-PHFX*V1XSR;
                V2X=V2X*PXGC-PHFX*V2XSR;
              ELSE
                SX=SX*PXGC
                V1X=V1X*PXGC
                V2X=V2X*PXGC
              ENDIF
              SC=SC*PCGC
              V1C=V1C*PCGC
              V2C=V2C*PCGC
            ENDIF
            SGCX  = SGCX + SMOO*SX
            SGCC  = SGCC + SMOO*SC
            V1    = DSMOO*(SX+SC) + SMOO*(V1X+V1C)
            V2    = SMOO*(V2X+V2C)
          ELSE
            V1    = 0.0D0
            V2    = 0.0D0
          ENDIF
          V(IR) = V1
          VTMP(IR) = V2
        ENDDO
      ELSE
        DO IR=1,NNR1
          RHO   = DMAX1(RHOE(IR),0.0D0)
          IF(RHO.GT.0.1D0*GCEPS) THEN
            IF(RHO.GT.4.0D0*GCEPS) THEN
              SMOO  = 1.0D0
              DSMOO = 0.0D0
            ELSE
              TEXP=DEXP(3.0D0*(1.0D0-RHO/GCEPS))
              SMOO=1.0D0/(1.0D0+TEXP)
              DSMOO=3.0D0/GCEPS * TEXP*SMOO*SMOO
            ENDIF
            GRHO    = GRAD(IR,1)
            TAR=TAU(IR)
            CALL TAUFUN(RHO,GRHO,TAR,SX,SC,V1X,V2X,V1C,V2C,VTT)
            SGCX  = SGCX + SMOO*SX
            SGCC  = SGCC + SMOO*SC
            V1    = DSMOO*(SX+SC) + SMOO*(V1X+V1C)
            V2    = SMOO*(V2X+V2C)
          ELSE
            V1    = 0.0D0
            V2    = 0.0D0
            VTT   = 0.0D0
          ENDIF
          V(IR) = V1
          VTMP(IR) = V2
          VTAU(IR) = VTT
        ENDDO
      ENDIF
      FLOPS=0.D0
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GCLSDOLD(SGCX,SGCC,RHOE,V,VTMP,GRAD,FLOPS)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'strs.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'tauf.inc'
      INCLUDE 'func.inc'
C     Arguments
      REAL*8     SGCX,SGCC,RHOE(NNR1,NLSD),VTMP(NNR1,NLSX),
     &           GRAD(NNR1,4*NLSD),FLOPS
      COMPLEX*16 V(MAXFFT,NLSD)
C     Variables
      REAL*8     TAU(NNR1,*),VTAU(NNR1,*)
      REAL*8     RHOA,RHOB,RHO,SMOO,DSMOO,TEXP,GRHOA,GRHOB,GRHOAB,
     &           V1A,V2A,V1B,V2B,V2AB,SX,SC,V1XA,V2XA,V1XB,V2XB,V1CA,
     &           V2CA,V1CB,V2CB,V2XAB,V2CAB,VTTA,VTTB,TARA,TARB
      REAL*8     SXSR,V1XASR,V2XASR,V1XBSR,V2XBSR,V2XABSR
      INTEGER    IR
C     ==--------------------------------------------------------------==
      FLOPS=0.D0
      IF(.NOT.TTAU) THEN
C$OMP parallel do private(IR,RHO,RHOA,RHOB,SMOO,DSMOO,TEXP,SX,SC)
C$OMP+      private(GRHOA,GRHOB,GRHOAB,V1XA,V1XB,V2XA,V2XB,V1CA,V2CA)
C$OMP+      private(V1CB,V2CB,V2XAB,V2CAB,V1A,V2A,V1B,V2B,V2AB)
C$OMP+      private(TARA,TARB,VTTA,VTTB)
C$OMP+      private(SXSR,V1XASR,V2XASR,V1XBSR,V2XBSR,V2XABSR,SRXA)
C$OMP+      shared(GRAD,V,VTMP,GCEPS,NNR1,RHOE)
C$OMP+      reduction(+:SGCX,SGCC)
#ifdef __SR8000
*poption parallel
*poption tlocal(SX,SC)
*poption tlocal(V1XA,V2XA,V1XB,V2XB,V1CA,V2CA,V1CB,V2CB)
*poption tlocal(V2XAB,V2CAB)
#endif
#if defined (__SR11000)
*poption psum(SGCX,SGCC)
#endif
        DO IR=1,NNR1
          RHOA   = DMAX1(RHOE(IR,1),0.0D0)
          RHOB   = DMAX1(RHOE(IR,2),0.0D0)
          RHO    = RHOA+RHOB
          IF(RHO.GT.0.1D0*GCEPS) THEN
            IF(RHO.GT.4.0D0*GCEPS) THEN
              SMOO  = 1.0D0
              DSMOO = 0.0D0
            ELSE
              TEXP=DEXP(3.0D0*(1.0D0-RHO/GCEPS))
              SMOO=1.0D0/(1.0D0+TEXP)
              DSMOO=3.0D0/GCEPS * TEXP*SMOO*SMOO
            ENDIF
            IF(RHOA.LT.0.1D0*GCEPS) RHOA=0.0D0
            IF(RHOB.LT.0.1D0*GCEPS) RHOB=0.0D0
            GRHOA   = GRAD(IR,1)
            GRHOB   = GRAD(IR,5)
            GRHOAB  = GRAD(IR,2)*GRAD(IR,6)+GRAD(IR,3)*GRAD(IR,7)+
     *                GRAD(IR,4)*GRAD(IR,8)
            CALL GC_LSD(RHOA,RHOB,GRHOA,GRHOB,GRHOAB,SX,SC,V1XA,V2XA,
     *                  V1XB,V2XB,V1CA,V2CA,V1CB,V2CB,V2XAB,V2CAB)
            IF(THYBRID) THEN
! wPBE stabbing, adding short-range GC
              IF(MGCSRX.NE.0) THEN
                CALL PBEXSR_LSD(RHOA,RHOB,GRHOA,GRHOB,SXSR,
     *                  V1XASR,V2XASR,V1XBSR,V2XBSR,V2XABSR,SRXA)
                SX=SX*PXGC - PHFX*SXSR
                V1XA=V1XA*PXGC - PHFX*V1XASR
                V2XA=V2XA*PXGC - PHFX*V2XASR
                V1XB=V1XB*PXGC - PHFX*V1XBSR
                V2XB=V2XB*PXGC - PHFX*V2XBSR
                V2XAB=V2XAB*PXGC - PHFX*V2XABSR
              ELSE
                SX=SX*PXGC
                V1XA=V1XA*PXGC
                V2XA=V2XA*PXGC
                V1XB=V1XB*PXGC
                V2XB=V2XB*PXGC
                V2XAB=V2XAB*PXGC
              ENDIF
              SC=SC*PCGC 
              V1CA=V1CA*PCGC
              V2CA=V2CA*PCGC
              V1CB=V1CB*PCGC
              V2CB=V2CB*PCGC
              V2CAB=V2CAB*PCGC
            ENDIF
            SGCX  = SGCX + SMOO*SX
            SGCC  = SGCC + SMOO*SC
            V1A   = DSMOO*(SX+SC) + SMOO*(V1XA+V1CA)
            V2A   = SMOO*(V2XA+V2CA)
            V1B   = DSMOO*(SX+SC) + SMOO*(V1XB+V1CB)
            V2B   = SMOO*(V2XB+V2CB)
            V2AB  = SMOO*(V2XAB+V2CAB)
          ELSE
            V1A   = 0.0D0
            V1B   = 0.0D0
            V2A   = 0.0D0
            V2B   = 0.0D0
            V2AB  = 0.0D0
            VTTA  = 0.0D0
            VTTB  = 0.0D0
          ENDIF
          V(IR,1) = V1A
          V(IR,2) = V1B
          VTMP(IR,1) = V2A
          VTMP(IR,2) = V2B
          VTMP(IR,3) = V2AB
        ENDDO
      ELSE
        DO IR=1,NNR1
          RHOA   = DMAX1(RHOE(IR,1),0.0D0)
          RHOB   = DMAX1(RHOE(IR,2),0.0D0)
          RHO    = RHOA+RHOB
          IF(RHO.GT.0.1D0*GCEPS) THEN
            IF(RHO.GT.4.0D0*GCEPS) THEN
              SMOO  = 1.0D0
              DSMOO = 0.0D0
            ELSE
              TEXP=DEXP(3.0D0*(1.0D0-RHO/GCEPS))
              SMOO=1.0D0/(1.0D0+TEXP)
              DSMOO=3.0D0/GCEPS * TEXP*SMOO*SMOO
            ENDIF
            IF(RHOA.LT.0.1D0*GCEPS) RHOA=0.0D0
            IF(RHOB.LT.0.1D0*GCEPS) RHOB=0.0D0
            GRHOA   = GRAD(IR,1)
            GRHOB   = GRAD(IR,5)
            GRHOAB  = GRAD(IR,2)*GRAD(IR,6)+GRAD(IR,3)*GRAD(IR,7)+
     *                GRAD(IR,4)*GRAD(IR,8)
            TARA = TAU(IR,1)
            TARB = TAU(IR,2)
            CALL TAUFUNS(RHOA,RHOB,GRHOA,GRHOB,GRHOAB,TARA,TARB,
     *                  SX,SC,V1XA,V2XA,
     *                  V1XB,V2XB,V1CA,V2CA,V1CB,V2CB,V2XAB,V2CAB,
     *                  VTTA,VTTB)
            SGCX  = SGCX + SMOO*SX
            SGCC  = SGCC + SMOO*SC
            V1A   = DSMOO*(SX+SC) + SMOO*(V1XA+V1CA)
            V2A   = SMOO*(V2XA+V2CA)
            V1B   = DSMOO*(SX+SC) + SMOO*(V1XB+V1CB)
            V2B   = SMOO*(V2XB+V2CB)
            V2AB  = SMOO*(V2XAB+V2CAB)
          ELSE
            V1A   = 0.0D0
            V1B   = 0.0D0
            V2A   = 0.0D0
            V2B   = 0.0D0
            V2AB  = 0.0D0
            VTTA  = 0.0D0
            VTTB  = 0.0D0
          ENDIF
          V(IR,1) = V1A
          V(IR,2) = V1B
          VTMP(IR,1) = V2A
          VTMP(IR,2) = V2B
          VTMP(IR,3) = V2AB
          VTAU(IR,1) = VTTA
          VTAU(IR,2) = VTTB
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
