C     ==================================================================
      SUBROUTINE GETGYRATION(TAUP,VELP)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'pimd.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'nose.inc'
C     Arguments
      REAL*8    TAUP(3,NAX,NSX,*),VELP(3,NAX,NSX,*)
C     Variables
      REAL*8    RMEAN(3),RNP,VAR,XLM,YLM,ZLM,VECPRO,EVIRI,RCLASS,
     &          RTOT,RNORM,EKINP1,TEMPP1
      INTEGER   IS,IA,K,IP,NPHALF,ITAU
C     ==--------------------------------------------------------------==
C     ==  RADII OF GYRATION OF DIFFERENT IONS                         ==
C     ==    GILLAN, IN COMPUTER MODELLING OF FLUIDS, SOLIDS, AND      ==
C     ==            POLYMERS, 1990                                    ==
C     ==--------------------------------------------------------------==
      RNP=DBLE(NP_TOTAL)
      CALL AZZERO(AVGYRA,MAXSP*2)
      CALL AZZERO(RGYRA,NSX*NAX*2)

C     If we're doing PI with NVE we need to use the instantaneous
C     temperature here. For details, see:
C     http://cpmd.org/mailman/htdig/cpmd-list/2008-April/004350.html
      IF(TNOSEP) THEN
        TEMPP1=TEMPW
      ELSE
        CALL EKINPP(EKINP1,VELP)
        TEMPP1=EKINP1*FACTEM*2.0D0/GLIB
      ENDIF

      DO IS=1,NSP
C       free particle reference
        FPGYR(IS)=DSQRT(FACTEM/(4.0D0*PMAR(IS)*TEMPP1))
        AVGYR(IS)=0.0D0
        DO IA=1,NA(IS)
          DO K=1,3
            RMEAN(K)=0.0D0
            DO IP=1,NP_TOTAL
              RMEAN(K)=RMEAN(K)+TAUP(K,IA,IS,IP)
            ENDDO
            RMEAN(K)=RMEAN(K)/RNP
          ENDDO
          VAR=0.0D0
          DO IP=1,NP_TOTAL
            XLM=TAUP(1,IA,IS,IP)-RMEAN(1)
            YLM=TAUP(2,IA,IS,IP)-RMEAN(2)
            ZLM=TAUP(3,IA,IS,IP)-RMEAN(3)
            CALL PBC(XLM,YLM,ZLM,XLM,YLM,ZLM,1,APBC,IBRAV)
            VAR=VAR+XLM**2+YLM**2+ZLM**2
          ENDDO
C         per atom 
          IF(NP_TOTAL.EQ.1) THEN
            RGYR(IA,IS)=0.D0
          ELSE
            RGYR(IA,IS)=DSQRT(VAR/RNP)
          ENDIF
C         accumulate first and second moment 
          RGYRA(IA,IS,1)=RGYRA(IA,IS,1)+RGYR(IA,IS)
          RGYRA(IA,IS,2)=RGYRA(IA,IS,2)+RGYR(IA,IS)**2
C         per species
          AVGYR(IS)=AVGYR(IS)+RGYR(IA,IS)
        ENDDO
        AVGYR(IS)=AVGYR(IS)/DBLE(NA(IS))
C       accumulate first and second moment 
        AVGYRA(IS,1)=AVGYRA(IS,1)+AVGYR(IS) 
        AVGYRA(IS,2)=AVGYRA(IS,2)+AVGYR(IS)**2 
      ENDDO
C     ==--------------------------------------------------------------==
C     ==  SUSCEPTIBILITY RADII OF DIFFERENT IONS                      ==
C     ==    PARRINELLO,RAHMAN JCP 80, 860 (1984)                      ==
C     ==--------------------------------------------------------------==
      CALL AZZERO(AVSUSA,MAXSP*2)
      CALL AZZERO(RSUSA,NSX*NAX*2)
      DO IS=1,NSP
C       free particle reference
        FPSUS(IS)=DSQRT(FACTEM/(2.0D0*PMAR(IS)*TEMPP1))
        AVSUS(IS)=0.0D0
        DO IA=1,NA(IS)
C         vector product
          RMEAN(1)=0.0D0
          RMEAN(2)=0.0D0
          RMEAN(3)=0.0D0
          DO IP=1,NP_TOTAL
            RMEAN(1)=RMEAN(1)+( TAUP(2,IA,IS,IP)*TAUP(3,IA,IS,IPP1(IP)) 
     *                        -TAUP(3,IA,IS,IP)*TAUP(2,IA,IS,IPP1(IP))) 
            RMEAN(2)=RMEAN(2)+( TAUP(1,IA,IS,IP)*TAUP(3,IA,IS,IPP1(IP))
     *                        -TAUP(3,IA,IS,IP)*TAUP(1,IA,IS,IPP1(IP)))
            RMEAN(3)=RMEAN(3)+( TAUP(1,IA,IS,IP)*TAUP(2,IA,IS,IPP1(IP))
     *                        -TAUP(2,IA,IS,IP)*TAUP(1,IA,IS,IPP1(IP)))
          ENDDO
          VECPRO=RMEAN(1)**2+RMEAN(2)**2+RMEAN(3)**2
          VECPRO=VECPRO*(TEMPP1*PMAR(IS))**2/(FACTEM*FACTEM)
C         spring kinetic contribution (virial representation)
          EVIRI=0.0D0
          DO K=1,3
            RMEAN(K)=0.0D0
            DO IP=1,NP_TOTAL
              RMEAN(K)=RMEAN(K)+TAUP(K,IA,IS,IP)
            ENDDO
            RMEAN(K)=RMEAN(K)/RNP
          ENDDO
          DO IP=1,NP_TOTAL
            XLM=TAUP(1,IA,IS,IP)-RMEAN(1)
            YLM=TAUP(2,IA,IS,IP)-RMEAN(2)
            ZLM=TAUP(3,IA,IS,IP)-RMEAN(3)
            CALL PBC(XLM,YLM,ZLM,XLM,YLM,ZLM,1,APBC,IBRAV)
            EVIRI=EVIRI-XLM*FIONKS(1,IA,IS,IP)
     *                 -YLM*FIONKS(2,IA,IS,IP)
     *                 -ZLM*FIONKS(3,IA,IS,IP)
          ENDDO
          EVIRI=EVIRI*2.0D0*FACTEM/(3.0D0*RNP*RNP*TEMPP1)
C         classical term
          RCLASS=(3.0D0*RNP-2.0D0)/(RNP**2)
C         total estimator 
          RTOT=RCLASS-EVIRI+VECPRO
C         be careful with crazy start configurations
          IF(RTOT.LE.0.0D0) RTOT=0.0D0
C         per atom
          RSUS(IA,IS)=FPSUS(IS)*DSQRT(RTOT)
C         accumulate first and second moment 
          RSUSA(IA,IS,1)=RSUSA(IA,IS,1)+RSUS(IA,IS)
          RSUSA(IA,IS,2)=RSUSA(IA,IS,2)+RSUS(IA,IS)**2
C         per species
          AVSUS(IS)=AVSUS(IS)+RSUS(IA,IS)
        ENDDO
        AVSUS(IS)=AVSUS(IS)/DBLE(NA(IS))
C       accumulate first and second moment  
        AVSUSA(IS,1)=AVSUSA(IS,1)+AVSUS(IS)
        AVSUSA(IS,2)=AVSUSA(IS,2)+AVSUS(IS)**2
      ENDDO
C     ==--------------------------------------------------------------==
C     ==  RADII FROM IMAGINARY TIME CORRELATION FUNCTION              ==
C     ==    CHANDLER, LES HOUCHES 1989                                ==
C     ==--------------------------------------------------------------==
      CALL AZZERO(AVCORA,MAXSP*2)
      CALL AZZERO(RCORA,NSX*NAX*2)
      NPHALF=NP_TOTAL/2
      DO IS=1,NSP
C       free particle reference
        FPCOR(IS)=DSQRT(3.0D0*FACTEM/(4.0D0*PMAR(IS)*TEMPP1))
        AVCOR(IS)=0.0D0
        DO IA=1,NA(IS)
          VAR=0.0D0
          RNORM=0
          DO IP=1,NPHALF
            ITAU=IP+NPHALF
            RNORM=RNORM+1
            XLM=TAUP(1,IA,IS,IP)-TAUP(1,IA,IS,ITAU)
            YLM=TAUP(2,IA,IS,IP)-TAUP(2,IA,IS,ITAU)
            ZLM=TAUP(3,IA,IS,IP)-TAUP(3,IA,IS,ITAU)
            CALL PBC(XLM,YLM,ZLM,XLM,YLM,ZLM,1,APBC,IBRAV)
            VAR=VAR+XLM**2+YLM**2+ZLM**2
          ENDDO
C         per atom
          IF(NP_TOTAL.EQ.1) THEN
            RCOR(IA,IS)=0.D0
          ELSE
            RCOR(IA,IS)=DSQRT(VAR/RNORM)
          ENDIF
cc          RCOR(IA,IS)=DSQRT(VAR/RNORM)
C         accumulate first and second moment
          RCORA(IA,IS,1)=RCORA(IA,IS,1)+RCOR(IA,IS)
          RCORA(IA,IS,2)=RCORA(IA,IS,2)+RCOR(IA,IS)**2
C         per species
          AVCOR(IS)=AVCOR(IS)+RCOR(IA,IS)
        ENDDO
        AVCOR(IS)=AVCOR(IS)/DBLE(NA(IS))
C       accumulate first and second moment
        AVCORA(IS,1)=AVCORA(IS,1)+AVCOR(IS)
        AVCORA(IS,2)=AVCORA(IS,2)+AVCOR(IS)**2
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
