      SUBROUTINE GLE_INIT()
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'gle.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'store.inc'
      INCLUDE 'prng.inc'
      INCLUDE 'coor.inc'
      INCLUDE 'isos.inc'

      REAL*8 LGLEA(GLE_NS+1,GLE_NS+1), LGLEC(GLE_NS+1,GLE_NS+1)
      REAL*8 LGLES(GLE_NS+1,GLE_NS+1), LGLET(GLE_NS+1,GLE_NS+1)
      REAL*8 LTMP(GLE_NS+1,GLE_NS+1), RV(GLE_NS+1)
      
      INTEGER i,is,ia,id,iostat, MSGLEN
      REAL*8 sqrtm

      IF (GLE_MODE.EQ.0) THEN
        EGLE=0.0d0
        RETURN
      END IF 
 
      !initializes static covariance to identity (i.e., enforces FDT)
      LGLEC=0.0d0
      do i=1,GLE_NS+1
        LGLEC(i,i)=TEMPW/FACTEM
      enddo 

      !initializes friction matrix, using presets or custom file
      IF (GLE_MODE .EQ. GLE_WHITE) THEN
          LGLEA(1,1)=GLE_OMEGA*ICM2AU
      ELSEIF (GLE_MODE .EQ. GLE_OPT) THEN
          LGLEA=GLE_OPT_A*GLE_OMEGA*0.01d0*ICM2AU
      ELSEIF (GLE_MODE .EQ. GLE_CPMD) THEN
          LGLEA=GLE_CP_A*GLE_OMEGA*ICM2AU
      ELSEIF (GLE_MODE .EQ. GLE_CUST) THEN
          !reads on root & broadcasts 
          IF (PARENT) THEN
            OPEN(444,file="GLE-A",status="OLD",iostat=iostat)
            if(iostat==0) then
              do i=1,GLE_NS+1
                read(444,*) LGLEA(i,:)
              end do
            else
              CALL STOPGM("GLE_INIT","Could not read GLE-A file
     &              required for LANGEVIN CUSTOM")
            endif
            close(444)
            open(444,file="GLE-C",status="OLD",iostat=iostat)
            if(iostat==0) then
              do i=1,GLE_NS+1
                read(444,*) LGLEC(i,:)
              end do
              LGLEC=LGLEC*(1./FACTEM)
            end if
           END IF
           MSGLEN=(GLE_NS+1)*(GLE_NS+1)*8
           CALL MY_BCAST(LGLEA,MSGLEN,SOURCE,ALLGRP)
           CALL MY_BCAST(LGLEC,MSGLEN,SOURCE,ALLGRP)
      END IF
      
      ! computes deterministic and stochastic parts of the propagator
      call matexp(LGLEA*(-DELT_IONS*0.5),LGLET,GLE_NS+1,15,15)
      IF (LGLEC(1,1).NE.0.0d0) THEN
        LTMP=LGLEC-matmul(LGLET,matmul(LGLEC,transpose(LGLET)))
        call scholesky(LTMP,LGLES,GLE_NS+1)
      ELSE
        IF (PARENT) WRITE(6,*) " Zero-temperature simulation"
        LGLES=0.0d0
      END IF

      IF (PARENT) THEN
        WRITE(6,*) " RUNNING GENERALIZED LANGEVIN DYNAMICS WITH:"
        WRITE(6,*) " A MATRIX: "
        do i=1,GLE_NS+1
          WRITE(6,*) LGLEA(i,:)
        END DO
        WRITE(6,*) " C MATRIX: "
        do i=1,GLE_NS+1
          WRITE(6,*) LGLEC(i,:)
        END DO
!        WRITE(6,*) " T MATRIX: "
!        do i=1,GLE_NS+1
!          WRITE(6,*) LGLET(i,:)
!        END DO
!        WRITE(6,*) " S MATRIX: "
!        do i=1,GLE_NS+1
!          WRITE(6,*) LGLES(i,:)
!        END DO
      END IF

      IF (PARENT) THEN
        IF (.NOT. RGLE) THEN
          EGLE=0.0
          write (6,*) " Initializing GLE momenta"
          IF (LGLEC(1,1).NE.0.0d0) THEN
            call scholesky(LGLEC,LTMP,GLE_NS+1) ! used for initialization of velocities
            do is=1,nsx
              sqrtm=sqrt(PMA(is))
              do ia=1,na(is)
                do id=1,3
                  do i=1,GLE_NS+1
                    RV(i)=REPPRNGG()
                  end do
                  RV=matmul(LTMP,RV)
                  do  i=1,GLE_NS+1
                    GLEP(id,ia,is,i)=RV(i)
                  end do
                end do
              end do
            end do
          ELSE
            CALL AZZERO(GLEP,NSX*NAX*3*(GLE_NS+1))
          END IF
          IF (GLE_COM.gt.0) CALL GLE_RMCOM()
cMcB      IF (GLE_COM.gt.0 .and. TISOS) CALL GLE_RMROT()
          IF (GLE_COM.gt.0 .and. TISOS) CALL GLE_RMROT(TAU0)
        END IF
        IF (.NOT. RVEL) THEN
          do is=1,nsx
            sqrtm=sqrt(PMA(is))
            do ia=1,na(is)
              VELP(:,ia,is)=GLEP(:,ia,is,1)/sqrtm       
            enddo
          enddo 
        ENDIF
!        write (6,*) " Initial GLE momenta"
!        do i=1,GLE_NS+1
!          write(6,*) i,":",  GLEP(:,:,:,i)
!        end do
!        write (6,*) " Initial velocities", VELP
      END IF

      IS=NSX*NAX*3*(GLE_NS+1)*8
      CALL MY_BCAST(GLEP,IS,SOURCE,ALLGRP)
      IS=NSX*NAX*3*8
      CALL MY_BCAST(VELP,IS,SOURCE,ALLGRP)

      CALL DCOPY((GLE_NS+1)*(GLE_NS+1),LGLEA,1,GLEA,1)
      CALL DCOPY((GLE_NS+1)*(GLE_NS+1),LGLEC,1,GLEC,1)
      CALL DCOPY((GLE_NS+1)*(GLE_NS+1),LGLES,1,GLES,1)
      CALL DCOPY((GLE_NS+1)*(GLE_NS+1),LGLET,1,GLET,1)
      END SUBROUTINE

      SUBROUTINE GLE_STEP
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'gle.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'store.inc'
      INCLUDE 'prng.inc'
      INCLUDE 'coor.inc'
      INCLUDE 'isos.inc'

      REAL*8 DEGLE
      REAL*8 RV(GLE_NS+1), XV(GLE_NS+1)
      REAL*8 LGLES(GLE_NS+1,GLE_NS+1), LGLET(GLE_NS+1,GLE_NS+1)
      INTEGER i,is,ia,id
      REAL*8 sqrtm, mm, com(3)

      IF (GLE_MODE.EQ.0) RETURN

      CALL DCOPY((GLE_NS+1)*(GLE_NS+1),GLES,1,LGLES,1)
      CALL DCOPY((GLE_NS+1)*(GLE_NS+1),GLET,1,LGLET,1)

      DEGLE=0
      do is=1,nsx
        sqrtm=sqrt(PMA(is))
        do ia=1,NA(is)
          GLEP(1:3,ia,is,1)=velp(1:3,ia,is)*sqrtm
          DEGLE=DEGLE+GLEP(1,ia,is,1)**2+GLEP(2,ia,is,1)**2
     &                 +GLEP(3,ia,is,1)**2
        end do
      end do
c     performs evolution
      do is=1,nsx
         do ia=1,na(is)
            do id=1,3
               ! evolution vector is read in (p, s_1, .., s_N) for the selected DOF
               do i=1,GLE_NS+1
                  xv(i)=GLEP(id,ia,is,i)
               end do

               IF(GLEC(1).EQ.0.0d0) then
                 xv=matmul(LGLET,xv)
               ELSE
                 ! fills up a vector of gaussian numbers
                 do i=1,GLE_NS+1
                    rv(i)=REPPRNGG()
                 end do

                 ! vector is evolved
                 xv=matmul(LGLET,xv)+matmul(LGLES,RV)
               END IF

               ! data is saved back
               do i=1,GLE_NS+1
                  GLEP(id,ia,is,i)=xv(i)
               end do
            end do
         end do
      end do

c     removes COM velocity
      if (GLE_COM.gt.0) CALL GLE_RMCOM()
cMcB  if (GLE_COM.gt.0 .and. TISOS) CALL GLE_RMROT()
      if (GLE_COM.gt.0 .and. TISOS) CALL GLE_RMROT(TAU0)
c     scale back velocities & updates Dconserved      
      do is=1,nsx
        sqrtm=sqrt(PMA(is))
        do ia=1,NA(is)
          velp(1:3,ia,is)=GLEP(1:3,ia,is,1)/sqrtm
          DEGLE=DEGLE-(GLEP(1,ia,is,1)**2+GLEP(2,ia,is,1)**2
     &                   +GLEP(3,ia,is,1)**2)
        end do
      end do

      ! code to deal with white noise would be much simpler. however, the GLE code is general enough and I'd rather not duplicate efforts for negligible speedup !
!        mm=0.0d0
!        COM=0.0d0
!        do is=1,nsx
!          mm=mm+PMA(is)*na(is)
!          IF (GLEC(1).eq.0.0d0) THEN
!            do ia=1,na(is)
!              do id=1,3
!                DEGLE=DEGLE+VELP(id,ia,is)**2*PMA(IS)
!                VELP(id,ia,is)=LGLET(1,1)*VELP(id,ia,is)
!                DEGLE=DEGLE-VELP(id,ia,is)**2*PMA(IS)
!              end do
!              COM(:)=COM(:)+PMA(is)*VELP(:,ia,is)
!            end do
!          ELSE
!            sqrtm=LGLES(1,1)/sqrt(PMA(is))
!            do ia=1,na(is)
!              do id=1,3
!                DEGLE=DEGLE+VELP(id,ia,is)**2*PMA(IS)
!                VELP(id,ia,is)=LGLET(1,1)*VELP(id,ia,is)
!     &                        +sqrtm*REPPRNGG()
!                DEGLE=DEGLE-VELP(id,ia,is)**2*PMA(IS)
!              end do
!              COM(:)=COM(:)+PMA(is)*VELP(:,ia,is)
!            end do
!          ENDIF
!        end do
!        if (GLE_COM.gt.0) THEN ! COM REMOVAL
!          DEGLE=DEGLE+(COM(1)**2+COM(2)**2+COM(3)**2)/MM
!          COM=COM*(1/MM)
!          do is=1,nsx
!            do ia=1,na(is)
!              VELP(:,ia,is)=VELP(:,ia,is)-COM(:)
!            end do
!          end do
!        END IF

      DEGLE=DEGLE*0.5
      EGLE=EGLE+DEGLE

!      IF (PARENT) THEN
!          write (6,*) " WRITING GLE momenta"
!          do i=1,GLE_NS+1
!           write(6,*) i,":",  GLEP(:,:,:,i)
!          end do
!      END IF 

      END SUBROUTINE

      SUBROUTINE GLE_ALLOC()
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'gle.inc'
      CALL MEMORY(IP_GLEA,(GLE_NS+1)*(GLE_NS+1),"GLEA")
      CALL MEMORY(IP_GLEC,(GLE_NS+1)*(GLE_NS+1),"GLEC")
      CALL MEMORY(IP_GLES,(GLE_NS+1)*(GLE_NS+1),"GLES")
      CALL MEMORY(IP_GLET,(GLE_NS+1)*(GLE_NS+1),"GLET")
      CALL MEMORY(IP_GLEP,3*NAX*NSX*(GLE_NS+1),"GLEP")
      END SUBROUTINE

      SUBROUTINE GLE_RMCOM()
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'gle.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'rmas.inc'

      INTEGER i,is,ia
      REAL*8 sqrtm, MM, COM(3)

      do i=1, GLE_NS+1 ! also do it for additional momenta (for consistency and to preserve analytical predictions) 
        COM=0.0d0
        MM=0.0d0
        do is=1,nsx
          sqrtm=sqrt(PMA(is))
          mm=mm+PMA(is)*na(is)
          do ia=1,na(is)
            com(:)=com(:)+glep(:,ia,is,i)*sqrtm
          end do
        end do
        com=com/mm  !COM velocity
        do is=1,nsx
          sqrtm=sqrt(PMA(is))
          do ia=1,na(is)
            glep(:,ia,is,i)=glep(:,ia,is,i)-com(:)*sqrtm
          end do
        end do
      end do
      END SUBROUTINE

      ! removes rotational velocities, to additional momenta as well. assumes no COM velocity is present
cMcB  SUBROUTINE GLE_RMROT()
      SUBROUTINE GLE_RMROT(TAU0)
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'gle.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'rmas.inc'
cMcB  INCLUDE 'coor.inc'
cMcB  REAL*8 TAUM(3,NAX,NSX), COM(3), LM(3)
      REAL*8 TAU0(3,NAX,NSX),TAUM(3,NAX,NSX), COM(3), LM(3)
      REAL*8 IM(3,3), IM1(3,3), RM, WM(3), TM
      INTEGER i,is,ia,j,info,IPIV(3),work(9)

      TAUM(:,:,:)=TAU0(:,:,:)
      COM=0.0d0
      TM=0.0d0
      do is=1,NSX
        tm=tm+NA(is)*PMA(is)
        do ia=1,NA(is)
          COM=COM+TAUM(:,IA,IS)*PMA(IS)
        end do
      end do
      COM=COM*(1./tm)
      do i=1,3
        TAUM(i,:,:)=TAUM(i,:,:)-COM(i)
      enddo

      IM=0.0d0
      do is=1,NSX
        do ia=1,NA(is)
          RM=(TAUM(1,ia,is)**2+TAUM(2,ia,is)**2+TAUM(3,ia,is)**2)
          do i=1,3
            IM(i,i)=IM(i,i)+PMA(is)*(RM-TAUM(i,ia,is)**2)
            do j=i+1,3
              IM(i,j)=IM(i,j)-PMA(is)*TAUM(i,ia,is)*TAUM(j,ia,is)
              IM(j,i)=IM(i,j)
            enddo
          enddo
        enddo
      enddo
      
      IM1=IM
      CALL DGETRF(3,3,IM1,3,IPIV,INFO)
      CALL DGETRI(3,IM1,3,IPIV,WORK,9,INFO)
    
      do i=1, GLE_NS+1
        !get angular momentum
        LM=0.0d0
        do is=1,NSX
          RM=sqrt(PMA(is))
          do ia=1,NA(is)
            LM(1)=LM(1)+RM*
     *    (TAUM(2,ia,is)*GLEP(3,ia,is,i)-TAUM(3,ia,is)*GLEP(2,ia,is,i))
            LM(2)=LM(2)+RM*
     *    (TAUM(3,ia,is)*GLEP(1,ia,is,i)-TAUM(1,ia,is)*GLEP(3,ia,is,i))
            LM(3)=LM(3)+RM*
     *    (TAUM(1,ia,is)*GLEP(2,ia,is,i)-TAUM(2,ia,is)*GLEP(1,ia,is,i))
          enddo
        enddo
        WM=matmul(IM1,LM) ! angular velocity
        do is=1,NSX
          RM=sqrt(PMA(is))
          do ia=1,NA(is)
            !gets velocity
            LM(1)=(WM(2)*TAUM(3,ia,is)-WM(3)*TAUM(2,ia,is))
            LM(2)=(WM(3)*TAUM(1,ia,is)-WM(1)*TAUM(3,ia,is))
            LM(3)=(WM(1)*TAUM(2,ia,is)-WM(2)*TAUM(1,ia,is))
            LM=LM*RM
            !subtract (rembember GLEP contains mass-scaled momenta!)             
            GLEP(:,ia,is,i)=GLEP(:,ia,is,i)-LM
          enddo
        enddo
      enddo
      END SUBROUTINE

C Exponential of a NxN matrix by scaling and squaring
      subroutine matexp(m,expm,n,j,k)
        implicit none
        real*8, intent(in) :: m(n,n)
        real*8, intent(out):: expm(n,n)
        integer, intent(in) :: n, j, k
        integer i, ik
        real*8  :: sm(n,n), TC(k)
        sm=m*1./(2.**j)
c     taylor expansion
        tc(1)=1.
        do i=2,k
           tc(i)=tc(i-1)*(1./i)
        end do
        expm=0
        do ik=k,1,-1
           do i=1,n
              expm(i,i)=expm(i,i)+tc(ik)
           end do
           expm=matmul(sm,expm)
c           call dgemm('n','n,',n,n,n,1.,t,n,sm,n,0.,expm,n)           
        end do
        do i=1,n
           expm(i,i)=expm(i,i)+1.
        end do
        do ik=1,j
           expm=matmul(expm,expm)
c           t=expm 
c           call dgemm('n','n,',n,n,n,1.,t,n,t,n,0.,expm,n)
        end do
      end subroutine matexp

      ! brute-force "stabilized" cholesky decomposition.
      ! in practice, we compute LDL^T decomposition, and force
      ! to zero negative eigenvalues.
      subroutine scholesky(SST, S, n)
        integer, intent(in)  :: n
        real*8, intent(in)   :: SST(n,n)
        real*8, intent(out)   :: S(n,n)
        real *8 :: L(n,n), D(n,n) 
        integer i,j,k
        S=0.
        L=0.
        D=0.
        do i=1,n
           L(i,i)=1.0
           do j=1,i-1
              L(i,j)=SST(i,j);
              do k=1,j-1
                 L(i,j)=L(i,j)-L(i,k)*L(j,k)*D(k,k)
              enddo
              if (D(j,j).ne. 0.0) then
                L(i,j)=L(i,j)/D(j,j) 
              else
                write(6,*) " Warning: zero eigenvalue in LDL^T 
     x                      decomposition."
                L(i,j)=0.
              end if
           enddo
           D(i,i)=SST(i,i)
           do k=1,i-1
              D(i,i)=D(i,i)-L(i,k)**2*D(k,k)
           end do
        enddo
        do i=1,n
           if ( D(i,i).ge. 0.0d0 ) then
             D(i,i)=sqrt(D(i,i))
           else
             write(6,*) " Warning: negative eigenvalue (",D(i,i), 
     x ")in LDL^T decomposition."
             D(i,i)=0.0
           end if
        end do
        S=matmul(L,D)
      end subroutine scholesky
