c     ==================================================================
      subroutine second_moment(c0,coc,matrix,psi,scr,lscr)
C     NOW PARALLEL!
c     ==================================================================
c     CALCULATES the matrix of second moments for ONE STATE C0:
c     matrix(i,j) = Int[  phi(r) *  r_i r_j * phi(r) ]
C     \                -  Int[  phi(r) *  r_i * phi(r) ]
C     \                  *Int[  phi(r) *  r_j * phi(r) ]
C     \           = second_moment_(i,j)
C
C     Here, r_i means (r-r0)_i, where r-r0 is the position operator
C     relative to the center of charge r0. This r0 is returned in COC.
c     =----------------------------------------------------------------=
C     *** INPUT: ***
C     C0:       one orbital state in G space
C     psi,scr:  scratch arrays
C     lscr:     dimension (in real*8) of scr
C     *** OUTPUT: ***
C     coc:      center of charge, 3-vector which will hold 
C     \         the direct space coordinates (rx,ry,rz) in
C     \         atomic units of the Center Of Charge of the orbital.
C     matrix:   real*8 3x3 matrix (symmetric) containing 
C     \         Int[  phi(r) *  r_i r_j * phi(r) ].
C     coc and matrix can both directly be re-inserted into the
C     routine generate_gaussian_g below.
c     =----------------------------------------------------------------=
      implicit none
      include 'system.h'
      include 'gvec.inc'
      include 'irat.inc'
      include 'response_p.inc'
c     =----------------------------------------------------------------=
c     Arguments:
      complex*16 c0(ngw)
      real*8   coc(3),matrix(3,3),
     &    scr(kr1,kr2s,kr3s,2),psi(kr1,kr2s,kr3s)
      integer  lscr
c     =----------------------------------------------------------------=
c     variables:
      integer  ir,isub,imax(3),ix,iy,iz,i1,i2,PE_bcast,msglen,bound
      real*8   rhoM,global_rhoM,PE_bcast_d,r_exp(3),delta_omega,
     &    projection
      real*8 DDOT
      external DDOT
      logical  debug
c     =----------------------------------------------------------------=
      call tiset('     r_psi',isub)
      call test_scr('second_moment','2*nnr1',lscr,2*nnr1)
      debug = .true.
      if (debug) print *,'calculating second moments...'

      delta_omega = 1d0/DBLE(NR1S*NR2S*NR3S) ! wfn normalization for
                                ! real space integration


      call fftTOr(c0,psi,scr,ngw,.true.)
!$OMP parallel do private(ir)
      do ir=1,nnr1
        scr(ir,1,1,1) = psi(ir,1,1)*psi(ir,1,1)
      enddo

      rhoM = 0d0
      do ix=1,nr1
        do iy=1,nr2
          do iz=1,nr3
            if (scr(ix,iy,iz,1) .gt. rhoM) then
              rhoM = scr(ix,iy,iz,1)
              imax(1)=ix + NRXPL(MEPOS,1)
              imax(2)=iy
              imax(3)=iz
            endif
          enddo
        enddo
      enddo

      if (debug)
     &    print *,'PE ',MEPOS,
     &    'MAX point found at ',imax(1),imax(2),imax(3)

C     find the DIRECT SPACE LOCATION of the center of
C     charge of the orbital:
C this does not make sense here, r_exp is not initialized
C and coc will be overwritten in the next statement anyway.
C     call DCOPY(3,r_exp,1,coc,1)
      coc(1) = 
     &    (imax(1)-1)*a1(1)/DBLE(nr1s)
     &    + (imax(2)-1)*a2(1)/DBLE(nr2s)
     &    + (imax(3)-1)*a3(1)/DBLE(nr3s)
      coc(2) = 
     &    (imax(1)-1)*a1(2)/DBLE(nr1s)
     &    + (imax(2)-1)*a2(2)/DBLE(nr2s)
     &    + (imax(3)-1)*a3(2)/DBLE(nr3s)
      coc(3) = 
     &    (imax(1)-1)*a1(3)/DBLE(nr1s)
     &    + (imax(2)-1)*a2(3)/DBLE(nr2s)
     &    + (imax(3)-1)*a3(3)/DBLE(nr3s)

      if (debug) write (6,'(A,3F10.4)') 
     &    'corresponding to point: ',coc


      global_rhoM = rhoM        ! now, determine the maximum 
                                ! over all processors:
#ifdef PARALLEL
      call glomax(1,global_rhoM)
      if ( abs(global_rhoM-rhoM) .LT. 1d-10 ) then
        PE_BCAST_d = DBLE(MEPOS)
      else
        PE_BCAST_d = 0d0
      endif
      call glomax(1,PE_BCAST_d) ! makes the broadcasting PE unique (!)
      PE_bcast = NINT(PE_BCAST_d)
      msglen = 3 * 8 / IRAT
      call my_bcast(imax,msglen,pe_bcast,ALLGRP)
#endif
                          ! dirty but simple.



C     Initialization necessary for the routine that 
C     applies the position operator:
      TSMOOTHING = .FALSE.
      llc_val(1) = - NR1S/2
      llc_val(2) = - NR2S/2
      llc_val(3) = - NR3S/2
      llc_ind(1) = bound(imax(1) + NR1S/2 ,NR1S)
      llc_ind(2) = bound(imax(2) + NR2S/2 ,NR2S)
      llc_ind(3) = bound(imax(3) + NR3S/2 ,NR3S)



      do i1=1,3
        call apply_op_rx(psi,1d0,scr(1,1,1,1),0d0,i1)
        r_exp(i1) = DDOT(nnr1,psi,1,scr(1,1,1,1),1)*delta_omega
        if (debug .and. parent)
     &    write (6,'(A,I2,A,F14.8)') 'r expectation ',i1,' = ',r_exp(i1)
      enddo


C     find the DIRECT SPACE LOCATION of the center of
C     charge of the orbital:
      call DAXPY(3,1d0,r_exp,1,coc,1)
c      coc(1) = coc(1) + 
c     &    (imax(1)-1)*a1(1)/DBLE(nr1s)
c     &    + (imax(2)-1)*a2(1)/DBLE(nr2s)
c     &    + (imax(3)-1)*a3(1)/DBLE(nr3s)
c      coc(2) = coc(2) + 
c     &    (imax(1)-1)*a1(2)/DBLE(nr1s)
c     &    + (imax(2)-1)*a2(2)/DBLE(nr2s)
c     &    + (imax(3)-1)*a3(2)/DBLE(nr3s)
c      coc(3) = coc(3) + 
c     &    (imax(1)-1)*a1(3)/DBLE(nr1s)
c     &    + (imax(2)-1)*a2(3)/DBLE(nr2s)
c     &    + (imax(3)-1)*a3(3)/DBLE(nr3s)

      if (debug) write (6,'(A,3F10.4)') 
     &    'NEW CENTER OF CHARGE at ',coc



C     find the REAL SPACE MESH indexes which define the center of
C     charge of the orbital, i.e. the desired center for our gaussian:
C     N_1  =  <r_exp> . b1    and so on.

C     FIRST DIMENSION (nr1s)
      projection = r_exp(1) * b1(1)  +   r_exp(2) * b1(2)
     &    + r_exp(3) * b1(3)
      projection = projection / alat * DBLE(NR1S)
      imax(1) = imax(1) + NINT(projection)
      llc_ind(1) = bound(imax(1) - NR1S/2,NR1S)
C     SECOND DIMENSION (nr2s)
      projection = r_exp(1) * b2(1)  +   r_exp(2) * b2(2)
     &    + r_exp(3) * b2(3)
      projection = projection / alat * DBLE(NR2S)
      imax(2) = imax(2) + NINT(projection)
      llc_ind(2) = bound(imax(2) - NR2S/2,NR2S)
C     THIRD DIMENSION (nr3s)
      projection = r_exp(1) * b3(1)  +   r_exp(2) * b3(2)
     &    + r_exp(3) * b3(3)
      projection = projection / alat * DBLE(NR3S)
      imax(3) = imax(3) + NINT(projection)
      llc_ind(3) = bound(imax(3) - NR3S/2,NR3S)

      if (debug) write (6,'(A,3I4)') 
     &    'NEW CENTER POINT at ',imax



C     Start calculating the second moments...
      do i1=1,3
        call apply_op_rx(psi,1d0,scr(1,1,1,1),0d0,i1)
        r_exp(i1) = DDOT(nnr1,psi,1,scr(1,1,1,1),1)*delta_omega
        if (debug .and. parent)
     &    write (6,'(A,I2,A,F14.8)') 'r expectation ',i1,' = ',r_exp(i1)
        do i2=i1,3
          call apply_op_rx(scr(1,1,1,1),1d0,scr(1,1,1,2),0d0,i2)
          matrix(i1,i2) = DDOT(nnr1,psi,1,scr(1,1,1,2),1)*delta_omega
          matrix(i2,i1) = matrix(i1,i2)
        enddo                   ! i2
      enddo                     ! i1

#ifdef PARALLEL
      call glosum(3*3,matrix)
      call glosum(3,r_exp)
#endif

      do i1=1,3
        do i2=1,3
          matrix(i1,i2) = matrix(i1,i2) - r_exp(i1)*r_exp(i2)
        enddo
      enddo

      call tihalt('     r_psi',isub)
      return
      end
c     ==================================================================




c     ==================================================================
      subroutine generate_gaussian_in_g(c0,coc,m)
C     PARALLEL!
c     ==================================================================
C     GENERATES a test gaussian in c0 (returned in g space) with
C     CENTER coc  (direct space position in atomic units)
C     SPREAD defined by MATRIX (a spread matrix as computed 
C     with the routine before)
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'response_p.inc'

      complex*16 c0(ngw)
      real*8  m(3,3),coc(3)

      integer ig
      real*8 scaling,DOTP,arg,gg,g1,g2,g3

      logical debug

      debug = .true.
      if(debug) print *,'constructing gaussian orbital...'

      call zazzero(c0,ngw)
!$OMP parallel do private(ig,g1,g2,g3,arg,gg)
      do ig=1,ngw
        g1  = gk(1,ig)*tpiba
        g2  = gk(2,ig)*tpiba
        g3  = gk(3,ig)*tpiba

        arg = g1*coc(1) + g2*coc(2) + g3*coc(3)

C       gg  = g1*g1*m(1,1) + g2*g2*m(2,2) + g3*g3*m(3,3)
C    &          + 2*(g1*g2*m(1,2) + g1*g3*m(1,3) + g2*g3*m(2,3))

C       DMB: physically we only need one time xy, xz, and yz
C            (otherwise the gaussian is twice as large in these directions)
        gg  = g1*g1*m(1,1) + g2*g2*m(2,2) + g3*g3*m(3,3)
     &        + (g1*g2*m(1,2) + g1*g3*m(1,3) + g2*g3*m(2,3))

C       c0(ig) = DCMPLX(cos(arg),-sin(arg))*exp(-gg)
C       DMB: changed factor...
        c0(ig) = DCMPLX(DCOS(arg),-DSIN(arg))*DEXP(-.5D0*gg)
      enddo

      scaling = dsqrt(DOTP(ngw,c0,c0))
      call DSCAL(2*ngw,1d0/scaling,c0,1)

      return
      end
c     ==================================================================


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                 C
C     This subroutine generates the wannier functions from        C
C     a gaussian fit saved in the file "gauss-wan.dat",           C
C     and build up the corresponding states for each molecules.   C
C                                                                 C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE Gauss_gen_read(NSTATE,C0,NMOL,STATEmin,STATEmax)
      IMPLICIT NONE

C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
      include 'fileopen.inc'

C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INTEGER NSTATE
      COMPLEX*16 C0(NGW,NSTATE)
      INTEGER NMOL,STATEmin(NMOL),STATEmax(NMOL)

      INTEGER istate,Gindx,mol,kstate
      INTEGER R_NSTATE,STATES_TO_READ
      INTEGER i,j
      LOGICAL FERROR
      REAL*8 ETOT,R_R0(3)
      REAL*8 Sxx,Syy,Szz,Sxy,Sxz,Syz
      REAL*8 R_sigma_mat(3,3)
      COMPLEX*16 Ct(NGW,*)
      POINTER (IP_Ct,Ct)
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

C     opening the file
      CALL FILEOPEN(500,'gauss-wan.dat',FO_OLD,FERROR)
C     Did the file open OK?
      IF (FERROR) THEN
         write(6,*) 'Error:the file gauss-wan.dat is not there!'
         CALL STOPGM('GAUSS_GEN_READ','FILE NOT FOUND')
      ENDIF
 
      write(6,*) 'Reading data from file...'

C     Reading index data from file..
      read(500,*) ETOT
      read(500,*) R_NSTATE

      write(6,*) 'The file contains data for ',R_NSTATE,' states'
      write(6,*) 'Total energy of the saved state Etot = ',ETOT

C     Calculating the number of states to read
      STATES_TO_READ=NSTATE/NMOL

      write(6,*) 'Number of states per molecule ',STATES_TO_READ

C     checking the number of states
      IF (STATES_TO_READ.NE.R_NSTATE) THEN
         write(6,*) 'Need more states for each molecules...exiting'
         RETURN
      ENDIF

C     Everything is OK...allocating the Ct array (2*dims for CPLX*16)
      
      CALL MEMORY(IP_Ct,2*NGW*R_NSTATE,'Ct')

C     reading the wannier functions
      write(6,*) 'Proceeding... loading the data for ',NMOL,
     $           ' molecule(s)'

C     Main loop on the read states
      DO istate=1,R_NSTATE
C    CHECK IF SIGMA or SIGMA2 IS SAVED!!!
C        Reading gaussian parameters, Rx,ry,rz and S matrix
         read(500,*) R_R0(1),R_R0(2),R_R0(3),Sxx,Sxy,Sxz,
     $               Syy,Syz,Szz
         write(6,FMT='(1X,A,3(1X,F8.5))') 'POS:',R_R0(1),R_R0(2),R_R0(3)
         write(6,FMT='(1X,A,6(1X,F8.5))') 'SIG:',Sxx,Sxy,Sxz,Syy,Syz,Szz

C        Generating the symetric S matrix
C        (scaled to fit to Wannier parameters,but S is already sigma^2...)
         R_sigma_mat(1,1) = Sxx
         R_sigma_mat(1,2) = Sxy
         R_sigma_mat(2,1) = Sxy
         R_sigma_mat(1,3) = Sxz
         R_sigma_mat(3,1) = Sxz
         R_sigma_mat(2,2) = Syy
         R_sigma_mat(2,3) = Syz
         R_sigma_mat(3,2) = Syz
         R_sigma_mat(3,3) = Szz

         write(6,*) 'SIGMA matrix'
         DO i=1,3
            write(6,FMT='(3(1X,F8.5))') 
     $             (R_sigma_mat(i,j),j=1,3)
         ENDDO
C        Fully anisotropic generation... S is a 3x3 symetric matrix of sigma^2
         call generate_gaussian_in_g(Ct(1,istate),R_R0,R_sigma_mat)
     
C        Building the Fourrier representation of the Gaussian
C        using Cg = [E^(.5 G^2 Sig^2) E^(-i G R0)]
c         DO Gindx=1,NGW
C        First: calculating the product E^(-i G.r0)
C           calculating G.r0
c            GbyR0=GK(1,Gindx)*R_R0(1)+GK(2,Gindx)*R_R0(2)
c     $            +GK(3,Gindx)*R_R0(3)
C           Adjusting the units
c            GbyR0=-GbyR0*TPIBA
C           Calculating E^i(G.r) using cos(x)+i*sin(x)
c            EXPiGR0=DCMPLX(DCOS(GbyR0),DSIN(GbyR0))

C        Second: calculating E^(.5 G^2 Sigma^2)
C           Isotropic case
c           EGSIGMA = EXP(-.5D0*HG(Gindx)*TPIBA2*R_sigma(1)**2)
C           Anisotropic case Sx,Sy,Sz
c            Gbysigma=(GK(1,Gindx)*R_sigma(1))**2 
c     $              +(GK(2,Gindx)*R_sigma(2))**2
c     $              +(GK(3,Gindx)*R_sigma(3))**2
C           Adjusting the units
c            Gbysigma=Gbysigma*TPIBA2
c            EGSIGMA = EXP(-.5D0*Gbysigma)

C        Now calculating the G representation 
c            Ct(Gindx,istate)= EGSIGMA*EXPiGR0
c         ENDDO
      ENDDO

C     Closing the file...
      CALL FILECLOSE(500)

      write(6,*) 'Building',NSTATE,' Wannier states'
C     Now filling up the C0 array with the required ammount of data
C     initialising the global state counter
      kstate=1
C     Loop on number of molecules
      DO mol=1,NMOL
         STATEmin(mol)=kstate
C        Loop on number of read states
         DO istate=1,R_NSTATE
C           Loop on Gvecs
!$OMP parallel do private(Gindx)
            DO Gindx=1,NGW
              C0(Gindx,kstate)=Ct(Gindx,istate)
            ENDDO
C           updating the global state counter
            kstate=kstate+1
         ENDDO
         STATEmax(mol)=kstate-1
      ENDDO

      DO mol=1,NMOL
         write(6,*) 'molecule: ',mol,' STATEmin = ',
     $   STATEmin(mol),' STATEmax = ',STATEmax(mol)
      ENDDO

      write(6,*) 'Done!'
C     Freeing memory
      CALL FREEM(IP_Ct)

      END


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                 C
C     This subroutine saves the wannier functions in the file     C
C     "gauss-wan.dat" in gaussian format.                         C 
C     Upon completion the file will contain:                      C
C     ETOT                                                        C
C     NSTATE                                                      C
C     Rx Ry Rz Sxx Sxy Sxz Syy Syz Szz                            C
C     -------- -----------------------                            C
C     position     spread matrix                                  C
C                                                                 C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE Save_My_Wannier_Gaussian(NSTATE,C0,ETOT,lscr,scr,psi)
      IMPLICIT NONE

C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
      include 'fileopen.inc'

C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INTEGER NSTATE,i,j
      COMPLEX*16 C0(NGW,NSTATE)
      REAL*8 ETOT,Sxx,Sxy,Sxz,Syy,Syz,Szz
      REAL*8 R0(3),S_matrix(3,3)
C     scratch variables
      INTEGER lscr
      REAL*8 scr(*),psi(*)


      INTEGER istate
      LOGICAL FERROR

#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

C     opening the file
      CALL FILEOPEN(500,'gauss-wan.dat',FO_DEF,FERROR) 
      write(6,*) 'Saving ',NSTATE,' states...'
      write(6,*) 'Total energy for this set Etot = ',ETOT

C     Saving data to file..
      write(500,*) ETOT
      write(500,*) NSTATE

C     Main loop on the states
      DO istate=1,NSTATE
C        Determining the centre+spread matrix
         call second_moment(c0(1,istate),R0,S_matrix(1,1),psi,scr,lscr)

C        printing out the data...
         write(6,*) '-----------------------------------------------'
         write(6,*) 'STATE: ',istate
         write(6,FMT='(1X,A,3(1X,F8.5))') 'POS:',R0(1),R0(2),R0(3)

         write(6,*) 'SIGMA matrix'
         DO i=1,3
            write(6,FMT='(3(1X,F8.5))') 
     $             (S_matrix(i,j),j=1,3)
         ENDDO

C        Picking the spread parameters 
C        (equivalent to sigma^2 with respect to Gaussian parameters)
         Sxx = S_matrix(1,1)
         Syy = S_matrix(2,2)
         Szz = S_matrix(3,3)
         
         Sxy = S_matrix(1,2)
         Sxz = S_matrix(1,3)

         Syz = S_matrix(2,3)

         write(6,FMT='(1X,A,6(F8.5))') 'SIG:',Sxx,Sxy,Sxz,Syy,Syz,Szz
         write(6,*) '-----------------------------------------------'

C        Saving the data
         write(500,*) R0(1),R0(2),R0(3),Sxx,Sxy,Sxz,Syy,Syz,Szz

      ENDDO

C     Closing the file...
      CALL FILECLOSE(500)

      END




















c     ==================================================================
C     ** KILL * KILL * KILL * KILL * KILL * KILL * KILL * KILL * KILL **
C     ** KILL * KILL * KILL * KILL * KILL * KILL * KILL * KILL * KILL **
C     ** KILL * KILL * KILL * KILL * KILL * KILL * KILL * KILL * KILL **
c     ==================================================================
      subroutine generate_gaussian(c0,coc,matrix,
     &    psi,scr,lscr)
C     GENERATES a test gaussian in c0 (returned in g space) with
C     CENTER coc  (integer grid points)
C     SPREAD defined by MATRIX being a spread matrix 
C     (which will be inverted first)
      implicit none
      include 'system.h'
      include 'response_p.inc'

      complex*16 c0
      integer coc(3),lscr
      real*8  matrix(3,3),psi(kr1s,kr2s,kr3s),scr(kr1s,kr2s,kr3s,2)


      integer i1,i2,i3,ipiv(4),info
      real*8 scaling,DOTP,delta_r,f,inv(3,3),work(9)



      real*8 rhoM,matrix_cp(3,3),DDOT,d1,d2,d3
      integer ix,iy,iz

C     NOT PARALLEL!


      print *,'preparing inverse spread matrix...'
C     FIRST, invert the spread matrix:
      call DCOPY(9,matrix,1,matrix_cp,1)
      call azzero(inv,9)
      inv(1,1) = 1d0
      inv(2,2) = 1d0
      inv(3,3) = 1d0
      call DSYSV('U',3,3,matrix_cp, 3, ipiv, 
     &    inv, 3, work, 9, info)
      if (info .ne. 0) 
     &    call stopgm('GENERATE GAUSSIAN','Matrix inversion failed!')
c     Now, inv contains the inverse spread matrix.


      write (6,*) 'INVERSE MATRIX FOUND:'
      write (6,'(3F15.9)') 
     &    (inv(1,i1),inv(2,i1),inv(3,i1),i1=1,3)


      f = 0.25d0
      call azzero(psi,nnr1)

!$OMP parallel do private(i1,i2,i3,d1,d2,d3,delta_r)
      do i1=1,nr1
        do i2=1,nr2
          do i3=1,nr3
            d1 = DBLE(i1-coc(1)) * a1(1)/DBLE(nr1s)
            d2 = DBLE(i2-coc(2)) * a2(2)/DBLE(nr2s)
            d3 = DBLE(i3-coc(3)) * a3(3)/DBLE(nr3s)
            delta_r = d1*d1*inv(1,1) + d2*d2*inv(2,2) + d3*d3*inv(3,3)
     &          + 2*(d1*d2*inv(1,2) + d1*d3*inv(1,3) + d2*d3*inv(2,3))
            psi(i1,i2,i3) = DEXP(- f*delta_r )
          enddo
        enddo
      enddo


      scaling = DDOT(nnr1,psi,1,psi,1)

      call DSCAL(nnr1,DSQRT(DBLE(NR1S*NR2S*NR3S)/scaling), psi,1)



      rhoM = 0d0
      do ix=1,nr1
        do iy=1,nr2
          do iz=1,nr3
            if (psi(ix,iy,iz)**2 .gt. rhoM) then
              rhoM = psi(ix,iy,iz)**2
              coc(1)=ix
              coc(2)=iy
              coc(3)=iz
            endif
          enddo
        enddo
      enddo

      print *,'MAX point found at ',coc(1),coc(2),coc(3)






      call fftTOg(psi,c0,scr,ngw,.true.)
      scaling = dsqrt(DOTP(ngw,c0,c0))
      print *,'additional scaling necessary: ',1d0/scaling
      call DSCAL(2*ngw,1d0/scaling,c0,1)



      return
      end
c     ==================================================================

