C     ==================================================================
      SUBROUTINE GS_ORTHO(C0,N0,CP,NP,SMAT,ISMA)   
C     ==--------------------------------------------------------------==
C     == GRAM-SCHMIDT ORTHOGONALISATION                               ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   N0         Number of wavefunctions                         ==
C     ==   C0(NGW,N0) are already orthogonalized.                     ==
C     ==   NP         Number of wavefunctions to be orthogonalized    ==
C     == OUTPUT:                                                      ==
C     ==   CP(NGW,NP) Wavefunctions to be orthogonalized              ==
C     ==   SMAT(NP,*) !MAX(N0,NP) Scratch array                       ==
C     ==   ISMA(NP)   Unused                                          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
C     Arguments
      INTEGER    N0,NP,ISMA(*)
      COMPLEX*16 C0(NGW,*),CP(NGW,*)
      REAL*8     SMAT(NP,*)             !MAX(N0,NP)
C     Variables
      INTEGER    ISUB
C     ==--------------------------------------------------------------==
      IF(NP.LT.1) RETURN
      IF(2*NGWS-1.LT.(N0+NP)) THEN
        WRITE(6,'(A,I6,/,A,I6)')
     &       ' GS_ORTHO! The number of components (2*NGWS-1)',
     &       2*NGWS-1,
     &       ' GS_ORTHO! is not greater than the number of states',N0+NP
        CALL STOPGM('GS_ORTHO','TOO MANY STATES')
      ENDIF
C     ==--------------------------------------------------------------==
      CALL TISET('   GSORTHO',ISUB)
      IF(N0.GT.0) THEN
        CALL OVLAP2(NGW,NP,N0,SMAT,CP,C0)
        CALL GLOSUM(NP*N0,SMAT)
        CALL DGEMM('N','T',2*NGW,NP,N0,-1.D0,C0,2*NGW,SMAT,NP,
     *             1.D0,CP,2*NGW)
      ENDIF
      CALL RGS(CP,NP,SMAT,ISMA)
      CALL TIHALT('   GSORTHO',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GS_ORTHO_C(C0,N0,CP,NP,SMAT,ISMA)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
C     Arguments
      INTEGER    N0,NP,ISMA(*)
      COMPLEX*16 C0(NGWK,*),CP(NGWK,*)
      COMPLEX*16 SMAT(NP,*)             !MAX(N0,NP)
C     Variables
      COMPLEX*16 ZONE
      PARAMETER(ZONE=(1.D0,0.D0))
      INTEGER    ISUB
C     ==--------------------------------------------------------------==
      IF(NP.LT.1) RETURN
      IF(NGWKS.LT.(N0+NP)) THEN
        WRITE(6,'(A,I6,/,A,I6)')
     &       ' GS_ORTHO_C! The number of components NGWKS',NGWKS,
     &       ' GS_ORTHO_C! is not greater than the number of states',
     &       N0+NP
        CALL STOPGM('GS_ORTHO_C','TOO MANY STATES')
      ENDIF
C     ==--------------------------------------------------------------==
      CALL TISET(' GSORTHO_C',ISUB)
      IF(N0.GT.0) THEN
        CALL OVLAP2_C(NGWK,NP,N0,SMAT,CP,C0)
        CALL GLOSUM(2*NP*N0,SMAT)
        CALL ZGEMM('N','C',NGWK,NP,N0,-ZONE,C0,NGWK,SMAT,NP,
     *             ZONE,CP,NGWK)
      ENDIF
      CALL RGS_C(CP,NP,SMAT,ISMA)
      CALL TIHALT(' GSORTHO_C',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE GSORTHO(CP,N,NFIRST,NLAST)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'geq0.inc'
C     Arguments
      INTEGER    N,NFIRST,NLAST
      COMPLEX*16 CP(N,*)
C     Variables
      INTEGER    ISUB,I,J
      REAL*8     S,ANORM
      REAL*8     DOTP
      EXTERNAL   DOTP
C     ==--------------------------------------------------------------==
      CALL TISET('   GSORTHO',ISUB)
      DO I=NFIRST,NLAST
        DO J=1,I-1
          S=-DOTP(N,CP(1,J),CP(1,I))
#ifdef PARALLEL
          CALL GLOSUM(1,S)
#endif
          CALL DAXPY(2*N,S,CP(1,J),1,CP(1,I),1)
        ENDDO
C       Because c0(-g)^*=c0(g) (Important).
        IF(GEQ0) CP(1,I)=DCMPLX(DREAL(CP(1,I)),0.D0)
        ANORM=DOTP(N,CP(1,I),CP(1,I))
#ifdef PARALLEL
        CALL GLOSUM(1,ANORM)
#endif
        ANORM=1.0D0/DSQRT(ANORM)
        CALL DSCAL(2*N,ANORM,CP(1,I),1)
      ENDDO
      CALL TIHALT('   GSORTHO',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GSORTHO_C(CP,N,NFIRST,NLAST)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE  'geq0.inc'
C     Arguments
      INTEGER    N,NFIRST,NLAST
      COMPLEX*16 CP(N,*)
C     Variables
      INTEGER    ISUB,N2,I,J
      COMPLEX*16 ZS,ZDOTC
      REAL*8     DDOT,ANORM
      EXTERNAL ZDOTC,DDOT
C     ==--------------------------------------------------------------==
      CALL TISET(' GSORTHO_C',ISUB)
      N2=2*N
      DO I=NFIRST,NLAST
        DO J=1,I-1
          ZS=-ZDOTC(N,CP(1,J),1,CP(1,I),1)
#ifdef PARALLEL
          CALL GLOSUM(2,ZS)
#endif
          CALL ZAXPY(N,ZS,CP(1,J),1,CP(1,I),1)
        ENDDO
C..Convention
        IF(GEQ0) CP(N/2+1,I)=DCMPLX(0.D0,0.D0)
        ANORM=DDOT(N2,CP(1,I),1,CP(1,I),1)
#ifdef PARALLEL
        CALL GLOSUM(1,ANORM)
#endif
        ANORM=1.0D0/DSQRT(ANORM)
        CALL DSCAL(N2,ANORM,CP(1,I),1)
      ENDDO
      CALL TIHALT(' GSORTHO_C',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
