c     ==================================================================
      subroutine hardness_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &     eirop,eivps,z11,nstate)
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'fft.inc'
      include 'coor.inc'
      include 'cnst.inc'
      include 'adat.inc'
      include 'ions.inc'
      include 'nlps.inc'
      include 'cppt.inc' 
      include 'geq0.inc' 
      include 'soft.inc' 
      include 'response_p.inc'
      include 'fileopen.inc'

c     ==--------------------------------------------------------------==
c     Arguments
      integer    nstate,lscr
      complex*16 c0(ngw,nstate),c1(ngw,nstate),
     &     psi(*),eirop(nhg),eivps(nhg),drhoe(*)
      real*8     z11(nstate,nstate)
      real*8     scr(*),rhoe(*)
c     ==--------------------------------------------------------------==
c     Local variables, external functions
      real*8     dotp
      external   dotp
      complex*16 response_vectors(nhg,*)
      pointer    (ip_response_vectors, response_vectors)
      real*8     eigenvect(lanczos_dim,lanczos_dim)
      pointer    (ip_eigenvect, eigenvect)
c     Here, we store the eigensystem of the tridiagonal
c     Lanczos matrix. Thus the dimension is lanczos_dim x lanczos_dim.

      real*8     alpha(*),beta(*)
      pointer    (ip_alpha,alpha), (ip_beta, beta)
      real*8     diagonals(*),subdiagonals(*)
      pointer    (ip_diagonals,diagonals), 
     &     (ip_subdiagonals, subdiagonals)

      real*8     norm
c     Auxiliary variables
      integer    i,iter,info,istate,ia,ie,ir,
     &     lastvalid,nconverged,iii,iat,isp,nnat
      real*8     dummy,work,center(3),temp(2)
      pointer    (ip_work, work)
      data center /0d0,0d0,0d0/
      logical    ferror

      character*3 zahl
      character*80 filename
      character*20 converged
c     ==--------------------------------------------------------------==


c     ==--------------------------------------------------------------==
      if (parent) then
        print *,' *** calculating HARDNESS  dv(r)/dn(r) via Lanczos'
        print *,' *** WITH COMPLETE REORTHOGONALIZATION.'
        print *,'     info: beta calculated after ENTIRE orthogon'
        print *,' *** Maximum dim of Lanczos space:  ',lanczos_dim
        print *,' *** Number of desired eigenstates: ',
     &      NUM_LANCZOS_STATES
        print *,' *** Threashold for convergence:    ',CONV_THRESHOLD
        print *,' *** The hardness response operator is shifted by:',
     &      eigenvalue_shift
        print *,' *** INVERTING the response sign.'
        print *,' *** ELIMINATING any constant charge/potential.'
      endif


      if ( NUM_LANCZOS_STATES+20 .gt. LANCZOS_DIM)
     &   call stopgm('EIGENSYS','lanczos dim too small.')
      if ( NUM_LANCZOS_STATES    .lt. 1)
     &   call stopgm('EIGENSYS','too few response states!')
      if ( CONV_THRESHOLD        .lt. 1d-8)
     &   call stopgm('EIGENSYS','conv threshold too low ( <1e-8 ).')
      if ( CONV_THRESHOLD        .gt. 1d-1)
     &   call stopgm('EIGENSYS','conv threshold too high ( >.01 ).')


      call memory(ip_response_vectors, 2*lanczos_dim*nhg, 'response')
      call zazzero(  response_vectors,   lanczos_dim*nhg)
      call memory(ip_eigenvect, lanczos_dim*lanczos_dim, 'eigenvect')
      call azzero(   eigenvect, lanczos_dim*lanczos_dim)
      call memory(ip_work, lanczos_dim*lanczos_dim, 'work')
      call azzero(   work, lanczos_dim*lanczos_dim)
      call memory(ip_alpha,         2*lanczos_dim, 'lanczos_a')
      call azzero(   alpha,         2*lanczos_dim)
      call memory(ip_beta,          2*lanczos_dim, 'lanczos_b')
      call azzero(   beta,          2*lanczos_dim)
      call memory(ip_diagonals,     2*lanczos_dim, 'lanczos_a2')
      call azzero(   diagonals,     2*lanczos_dim)
      call memory(ip_subdiagonals,  2*lanczos_dim, 'lanczos_b2')
      call azzero(   subdiagonals,  2*lanczos_dim)
c     ==--------------------------------------------------------------==
c     Initialize perturbation density to be the gnd state density:
      call rhoofr(C0,RHOE,PSI,SCR,LSCR,NSTATE)
      call fftTOg(rhoe,response_vectors(1,1),psi,nhg,.true.)
      if (geq0) response_vectors(1,1) = DCMPLX(0.d0,0.d0)
C     All "vectors" in this context should integrate to zero.

      norm = dotp(nhg,response_vectors(1,1),
     &     response_vectors(1,1))
#ifdef PARALLEL
      call GLOSUM(1,norm)
#endif
      norm = dsqrt(norm)
      call DSCAL(2*nhg, 1d0/norm, response_vectors(1,1),1)

c     All other arrays are expected to be correctly initialized by the calling
c     (and allocating) routine, do_perturbation_p().

      beta(1) = 0d0             ! for security (not really needed).
c     ==--------------------------------------------------------------==
c     Do the Lanczos steps.
      iter = 1
      nconverged = 0
      i = 1                     ! dummy.
      do while ( (iter .lt. lanczos_dim-1)
     &     .and.  (num_lanczos_states .gt. nconverged)
     &     .and. .not. EXSOFT)


         call rwfopt_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &        eirop,eivps,response_vectors(1,iter),dummy,
     &        z11,nstate,dummy)
         call rhoofr_p(c0,c1,drhoe,psi,scr,lscr,nstate)
         call fftTOg(drhoe, 
     &        response_vectors(1,iter+1), psi, nhg, .true.)
         call DSCAL(2*nhg,-1d0,response_vectors(1,iter+1),1)

         if (geq0) response_vectors(1,iter+1) = DCMPLX(0d0,0d0)


         temp (1) = dotp(nhg,response_vectors(1,iter),
     &        response_vectors(1,iter))
         temp (2) = dotp(nhg,response_vectors(1,iter+1),
     &        response_vectors(1,iter+1))
         call glosum(2,temp)


c     This is to work around the Lanczos problem that eigenvalues around
c     zero are difficult to determine. After each run of rwfopt, we 
c     ADD -0.xx times the INPUT potential to the output. Then from all the
c     eigenvalues we have to SUBTRACT this -0.xx. Remember that
c     this value is contained WITH ITS SIGN in eigenvalue_shift=-0.xx:
c        call DAXPY(2*nhg, eigenvalue_shift, 
c     &      response_vectors(1,iter), 1,
c     &      response_vectors(1,iter+1), 1)
c     NB: In the hardness calculation, however, this eigenvalue_shift
c     should be zero.


        norm = dotp(nhg, response_vectors(1,iter+1),
     &      response_vectors(1,iter+1))
#ifdef PARALLEL
        call glosum(1,norm)
#endif
        norm = dsqrt(norm)
        if (parent)
     &      print *,' *+* L2 norm[dv[i+1]]       = ',norm

c     ==--------------------------------------------------------------==
c     Proceed with the Lanczos algorithm:
c     A v1 (the answer) is in response(:,iter+1) == v2:
c     v2   = A v1
c     ==--------------------------------------------------------------==

        alpha(iter) = DOTP(nhg, response_vectors(1,iter), 
     &       response_vectors(1,iter+1))
#ifdef PARALLEL
        call glosum(1,alpha(iter))
#endif
c     a1   = v1 (A v1)
c     \    This is the projection of the input vector (iter)
c     \    onto the reponse vector (iter+1). -> Orthogonalization.
        
        call DAXPY(2*nhg, - alpha(iter), response_vectors(1,iter), 1,
     &      response_vectors(1,iter+1), 1)
c     v2   = v2  -  a1 v1


        if (.not. (iter .eq. 1))
     &      call DAXPY(2*nhg, - beta(iter),
     &      response_vectors(1,iter-1), 1,
     &      response_vectors(1,iter+1), 1)
c     v2   = v2  -  b1 v0


c     *** Here, we do a complete orthogonalization of the new
c     \   vector wrt the old ones (v_j, j=1..i-1).
c     Method: Grahm-Schmitt "by hand".
        if (iter .ge. 2) then
          do i=1,iter
            norm = - DOTP(nhg, response_vectors(1,iter+1), 
     &          response_vectors(1,i))
#ifdef PARALLEL
            call glosum(1,norm)
#endif
            call DAXPY(2*nhg, norm, response_vectors(1,i),1,
     &          response_vectors(1,iter+1), 1)
          enddo
        endif


        norm = (DOTP(nhg,response_vectors(1,iter+1),
     &      response_vectors(1,iter+1)))
#ifdef PARALLEL
        call glosum(1,norm)
#endif
        norm = dsqrt(norm)
        beta(iter+1) = norm

        call DSCAL(2*nhg, 1d0/norm,
     &      response_vectors(1,iter+1), 1)
c     v2 is normalized.

        if (parent) then
          print *,' *=*   overlap: alpha[',
     &        iter,'] = ',alpha(iter)
          print *,' *=*   off-diag: beta[',
     &        iter+1,'] = ',beta(iter+1)
c          print *,' *=*   norm:               = ',norm
          print *,'##############################',
     &        '##################################'
        endif
c     ==--------------------------------------------------------------==
        call DCOPY(iter, alpha, 1, diagonals, 1)
        call DCOPY(iter-1, beta(2), 1, subdiagonals, 1)
c     Now, the diagonal [alpha(*)] and the subdiagonal [beta(*)]
c     elements of the (tridiagonal) Lanczos matrix 
c     are calculated. This matrix is now diagonalized:
        call DSTEV('V', iter, diagonals, subdiagonals,
     &      eigenvect, lanczos_dim, work, info)
        if (info .ne. 0) print *,'DSTEV exit status: ', info
c     output of dstev:
c     \  alpha = eigenvalues,
c     \  eigenvect = corresponding eigenvectors.

        if (parent)
     &      print *,' *** SPECTRUM, run',iter,':'
        nconverged = 0
        do i=iter,1,-1
          if ( abs(eigenvect(iter,i)) .lt. conv_threshold) then
            converged = '       (converged: '
            nconverged = nconverged + 1
          else
            converged = '   (NOT converged: '
          endif
          call XSTRING(converged,ia,ie)
          if (parent) 
     &        print '(A,I3,A,F12.7,A,F8.6,A)',
     &        ' *** eigenvalue ',i,' = ',
     &        diagonals(i)-eigenvalue_shift,
     &        converged(ia:ie),abs(eigenvect(iter,i)),').'
        enddo
        if (parent) then
          print *,' *** Converged states:',nconverged
          print *,' *** Eigenvalue for Lanczos ghost states is ',
     &        -eigenvalue_shift
          print *,'##############################',
     &        '##################################'
        endif
        lastvalid = iter
        iter = iter + 1
      enddo
c     ==--------------------------------------------------------------==
c     == END / Lanczos runs.
c     ==--------------------------------------------------------------==
C     Find a suitable "center" of the system, for plotting:
      nnat=0
      center(1) = 0d0
      center(2) = 0d0
      center(3) = 0d0
      do isp=1,nsp
         do iat=1,na(isp)
            center(1) = center(1) + tau0(1,iat,isp)
            center(2) = center(2) + tau0(2,iat,isp)
            center(3) = center(3) + tau0(3,iat,isp)
            nnat = nnat + 1
         enddo
      enddo
      center(1) =  center(1) / DBLE(nnat)
      center(2) =  center(2) / DBLE(nnat)
      center(3) =  center(3) / DBLE(nnat)
C     Center is now the arithmetic average (NOT the center of mass!!)
C     of all atomic positions.

      IF(PARENT) CALL FILEOPEN(53,'LANCZOS_COEFF',FO_DEF,FERROR)
      IF(FERROR) GOTO 90
 100  CONTINUE

      do istate=lastvalid,1, -1
         iii=lastvalid-istate+1



c     build the eigenvector of the linear answer machinery:
         call DGEMV('N', 2*nhg, lastvalid, 1d0, response_vectors, 2*nhg,
     &        eigenvect(1,istate),1, 0d0, scr, 1)


         if (parent) then
            write (53,FMT='(A,I3,A,F20.12)',ERR=90)
     &           '***** LANCZOS(',istate,') >>> ',
     &           diagonals(istate)-eigenvalue_shift
            do i=1,lastvalid
               write (53,'(A,I3,A,I3,A,F15.11)')
     &              '  \*  Lanczos(',i,'; ',istate,') = ',
     &              eigenvect(i,istate)
            enddo
         endif

         write (zahl,'(I3.3)') iii
         filename='eigst-'//zahl//'.cube'
         call XSTRING(filename,ia,ie)
         if (parent) write (6,'(A,I3,3A)')
     &        ' _i_ Writing eigenstate ',istate,
     &        ' to file ',filename(ia:ie),'.'
         call ffttor(scr,scr,psi,nhg,.true.)
         call cubefile(filename,scr,center,psi,.true.)

C     TESTING: Zero integral?
         temp(1) = 0d0
         do ir=1,nnr1
            temp(1) = temp(1) + scr(ir)
         enddo
         call glosum(1,temp(1))
         if (parent) write (6,*) 'STATE ',istate,': integral -> ',
     &        temp(1)



      enddo
      if (parent) CALL FILECLOSE(53)

c     ==--------------------------------------------------------------==
c     == END / Output of eigensolutions.
c     ==--------------------------------------------------------------==


      call freem(ip_response_vectors)
      call freem(ip_eigenvect)
      call freem(ip_work)
      call freem(ip_alpha)
      call freem(ip_beta)
      call freem(ip_diagonals)
      call freem(ip_subdiagonals)

      return
c     ==--------------------------------------------------------------==
 90   continue
      write(6,*) 'HARDNESS_P| COULD NOT OPEN FILE "LANCZOS_COEFF"'
      CALL STOPGM('HARDNESS_P','FILE OPEN ERROR')
c     ==--------------------------------------------------------------==
      end
c     ==--------------------------------------------------------------==
