C     ==================================================================
      SUBROUTINE HESSUP(XPAR,YPAR,DXPAR,DYPAR,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     ==  BFGS update of the nuclear hessian                          ==
C     ==  BFGS: Broyden-Fletcher-Goldfarb-Shanno                      ==
C     ==  See Numerical recipes p308                                  ==
C     ==  (Variable Metric Methods in Multidimensions)                ==
C     ==--------------------------------------------------------------==
C     ==  IN OUR CASE WE COMPUTE THE HESSIAN BY DEFAULT               ==
C     ==  (if TINVBFGS=.TRUE. use inverse Hessian matrix)             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cotr.inc'
C     Arguments
      INTEGER LSCR
      REAL*8  XPAR(*),YPAR(*),DXPAR(*),DYPAR(*),
     &        SCR(*)
C     Variables
      REAL*8  EPS
      PARAMETER (EPS=1.D-10)
      REAL*8  S(NODIM),Y(NODIM),HS(NODIM)
      POINTER (IP_S,S),(IP_Y,Y),(IP_HS,HS)
      REAL*8  YS,SHS
      INTEGER ISUB,I
      REAL*8  DDOT
      EXTERNAL DDOT
C     ==--------------------------------------------------------------==
      CALL TISET('    HESSUP',ISUB)
C     ==--------------------------------------------------------------==
C     Allocation of SCR
      CALL TEST_SCR('HESSUP','3*NODIM',LSCR,3*NODIM)
#if(_BGL)
      CALL GET_ADDR(IP_S,SCR(1))
c     NODIM=NODIM+MOD(NODIM,2)
      CALL GET_ADDR(IP_Y,SCR(1+NODIM))
      CALL GET_ADDR(IP_HS,SCR(1+2*NODIM))
#else
      CALL GET_ADDR(IP_S,SCR(1))
      CALL GET_ADDR(IP_Y,SCR(1+NODIM))
      CALL GET_ADDR(IP_HS,SCR(1+2*NODIM))
#endif
C     ==--------------------------------------------------------------==
      DO I=1,NODIM
        S(I)=XPAR(I)-YPAR(I)
        Y(I)=DXPAR(I)-DYPAR(I)
      ENDDO
      YS=DDOT(NODIM,Y(1),1,S(1),1)
      IF(TINVBFGS) THEN
C       BFGS using inverse Hessian matrix
        CALL DGEMV('N',NODIM,NODIM,1.0D0,HESS,NODIM,Y,1,0.0D0,HS,1)
        SHS=DDOT(NODIM,Y(1),1,HS(1),1)
C       Davidon_fletcher-Powell (DFP) update
        IF(ABS(YS).GT.EPS .AND. ABS(SHS).GT.EPS) THEN
          CALL DGER(NODIM,NODIM,1.0D0/YS,S,1,S,1,HESS,NODIM)
          CALL DGER(NODIM,NODIM,-1.0D0/SHS,HS,1,HS,1,HESS,NODIM)
C         Additional term of BFGS
          CALL DSCAL(NODIM,1.0D0/YS,S,1)
          CALL DAXPY(NODIM,-1.0D0/SHS,HS,1,S,1)
          CALL DGER(NODIM,NODIM,SHS,S,1,S,1,HESS,NODIM)
        ENDIF
      ELSE
C       BFGS using Hessian matrix
        CALL DGEMV('N',NODIM,NODIM,1.0D0,HESS,NODIM,S,1,0.0D0,HS,1)
        SHS=DDOT(NODIM,S(1),1,HS(1),1)
        IF(ABS(YS).GT.EPS .AND. ABS(SHS).GT.EPS) THEN
          CALL DGER(NODIM,NODIM,1.0D0/YS,Y,1,Y,1,HESS,NODIM)
          CALL DGER(NODIM,NODIM,-1.0D0/SHS,HS,1,HS,1,HESS,NODIM)
        ENDIF
      ENDIF
      CALL TIHALT('    HESSUP',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE HESPOW(XPAR,YPAR,DXPAR,DYPAR,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     ==  POWELL update of the nuclear hessian (for transition state  ==
C     ==  search)                                                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cotr.inc'
C     Arguments
      INTEGER LSCR
      REAL*8  XPAR(NODIM),YPAR(NODIM),DXPAR(NODIM),DYPAR(NODIM),
     &        SCR(LSCR)
C     Variables
      REAL*8  S(NODIM),Y(NODIM),HS(NODIM)
      POINTER (IP_S,S),(IP_Y,Y),(IP_HS,HS)
      REAL*8  SS,YS
      REAL*8  DDOT
      EXTERNAL DDOT
      INTEGER ISUB,I
C     ==--------------------------------------------------------------==
      CALL TISET('    HESPOW',ISUB)
C     ==--------------------------------------------------------------==
C     Allocation of SCR
      CALL TEST_SCR('HESPOW','3*NODIM',LSCR,3*NODIM)
      CALL GET_ADDR(IP_S,SCR(1))
      CALL GET_ADDR(IP_Y,SCR(1+NODIM))
      CALL GET_ADDR(IP_HS,SCR(1+2*NODIM))
C     ==--------------------------------------------------------------==
      DO I=1,NODIM
        S(I)=XPAR(I)-YPAR(I)
        Y(I)=DXPAR(I)-DYPAR(I)
      ENDDO
      SS=DDOT(NODIM,S,1,S,1)
      CALL DGEMV('N',NODIM,NODIM,-1.0D0,HESS,NODIM,S,1,1.0D0,Y,1)
      YS=DDOT(NODIM,Y,1,S,1)
      YS=YS/SS
      CALL DGER(NODIM,NODIM,1.0D0/SS,Y,1,S,1,HESS,NODIM)
      CALL DGER(NODIM,NODIM,1.0D0/SS,S,1,Y,1,HESS,NODIM)
      CALL DGER(NODIM,NODIM,-YS/SS,S,1,S,1,HESS,NODIM)
      CALL TIHALT('    HESPOW',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
