C     ==================================================================
      SUBROUTINE HPSI(C0,C2,SC0,VPOT,PSI,SCR,LSCR,NSTATE,IKIND,ISPIN)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE GENERALIZED FORCE C2=CMPLX(DFR,DFI) ACTING ON ALL THE   ==
C     ==  ELECTRONIC STATES REPRESENTED BY THE VECTOR C0=CMPLX(CR,CI) ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   C0(NGWK,NSTATE) Input Wavefunctions                        ==
C     ==   VPOT    Local Potential (in real space)                    ==
C     ==   LSCR    Dimension of scratch array SCR                     ==
C     ==   NSTATE  Number of states                                   ==
C     ==   IKIND   Index of k point                                   ==
C     ==   ISPIN   Need with LSD option for diagonalization scheme    ==
C     ==         dimension of VPOT(NNR1,ISPIN)                        ==
C     == OUTPUT:                                                      ==
C     ==   C2 = H|C0>                                                 ==
C     ==   PSI              Used for FFT                              ==
C     ==   SCR(LSCR)        Scratch array                             ==
C     ==   SC0(NGWK,NSTATE) Used only by TIVAN                        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'pslo.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'norm.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'spin.inc'        !TLSE
      INCLUDE 'nlps.inc'        !IMAGP
      INCLUDE 'kpts.inc'        !TKPNT
      INCLUDE 'fft.inc'         !MAXFFT
C     Arguments
      INTEGER    LSCR,NSTATE,IKIND,ISPIN
      COMPLEX*16 C0(NGWK,NSTATE),C2(NGWK,NSTATE),SC0(NGWK,NSTATE),
     &           PSI(NNR1)
      REAL*8     VPOT(NNR1,ISPIN),SCR(LSCR)
C     Variables
      COMPLEX*16 AUXC(NGWK),PAB(*)
      REAL*8     FOC(NSTATE),DDIA(IMAGP*NAX)
      POINTER    (IP_FOC,FOC),(IP_AUXC,AUXC),(IP_DDIA,DDIA),(IP_PAB,PAB)
      CHARACTER  TAG*30
      INTEGER    ISUB,I,LHPSI,LRNLSM,IL_AUXC,IL_DDIA,IL_PAB
      LOGICAL    DEBUG
C     ==--------------------------------------------------------------==
      CALL TISET('      HPSI',ISUB)
      IF(TLSE) THEN
        IL_PAB=2*MAXFFT
      ELSE
        IL_PAB = 2
      ENDIF
C     SCR partition.
      CALL GIVE_SCR_HPSI(LHPSI,TAG,NSTATE)
      CALL TEST_SCR('HPSI',TAG,LSCR,LHPSI)
C     Allocation of SCR.
      DEBUG=.FALSE.
      CALL SCRSET(SCR,LSCR,DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_FOC,NSTATE,'FOC',DEBUG)
C     ==--------------------------------------------------------------==
C     == INITIALIZE ELEC-FORCE ARRAYS                                 ==
C     ==--------------------------------------------------------------==
      CALL AZZERO(C2,2*NGWK*NSTATE)
      CALL GIVE_SCR_RNLSM(LRNLSM,TAG,NSTATE,.FALSE.)
      LRNLSM=LRNLSM+100
      CALL SCRPTR(SCR,LSCR,IP_PAB,IL_PAB,'PAB',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_AUXC,LRNLSM,'AUXC',DEBUG)
      CALL RNLSM(C0,NSTATE,PSI,AUXC,LRNLSM,1,IKIND,.FALSE.)
      CALL SCRPTF(SCR,LSCR,IP_AUXC,LRNLSM,'AUXC',DEBUG)
      DO I=1,NSTATE
        FOC(I)=1.0D0
      ENDDO
      IF(TIVAN) THEN
        CALL DCOPY(2*NGWK*NSTATE,C0(1,1),1,SC0(1,1),1)
        CALL SPSI(NSTATE,SC0,PSI,NNR1*2)
      ENDIF
C     ==--------------------------------------------------------------==
C     == Compute the force on the electronic degrees of freedom due   ==
C     == to the local potential (stored in VPOT)                      ==
C     ==--------------------------------------------------------------==
      CALL VPSI(C0,C2,FOC,VPOT,PSI,PAB,NSTATE,IKIND,ISPIN)
C     ==--------------------------------------------------------------==
C     == Compute the force on the electronic degrees of freedom due   ==
C     == to the non-local part of the potential, and add it to the    ==
C     == other piece, coming from the local contribution.             ==
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_FNONLOC(IL_AUXC,IL_DDIA,NSTATE)
      CALL SCRPTR(SCR,LSCR,IP_DDIA,IL_DDIA,'DDIA',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_AUXC,IL_AUXC,'AUXC',DEBUG)
      CALL FNONLOC(C2,FOC,NSTATE,AUXC,DDIA,IKIND,ISPIN)
Ckpt  Seems to be unnecessary.
      IF(GEQ0) THEN
        IF(TKPNT) THEN 
          CALL ZCLEAN_K(C2,NSTATE,NGW)
        ELSE 
          CALL ZCLEAN(C2,NSTATE,NGW)
        ENDIF
      ENDIF
      IF(TDAVI) CALL DSCAL(2*NGWK*NSTATE,-1.0D0,C2(1,1),1)
      CALL SCRREL(SCR,LSCR,DEBUG)
      CALL TIHALT('      HPSI',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_HPSI(LHPSI,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NGWK NAX
      INCLUDE 'spin.inc'        !TLSE
      INCLUDE 'fft.inc'         !MAXFFT
C     Arguments
      INTEGER   LHPSI,NSTATE
      CHARACTER TAG*30
C     Variables
      INTEGER   LRNLSM,IL_DDIA,IL_AUXC
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_RNLSM(LRNLSM,TAG,NSTATE,.FALSE.)
      CALL GIVE_SCR_FNONLOC(IL_AUXC,IL_DDIA,NSTATE)
      LHPSI=NSTATE+             !FOC
     &     IL_DDIA+             !DDIA
     &     MAX(IL_AUXC,LRNLSM)+  !AUXC and SCR
     &     100                  !SCRATCH tool
      IF(TLSE) LHPSI=LHPSI+2*MAXFFT
      TAG ='NSTATE+IL_DDIA+MAX(IL_AUXC...)'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
