C     ==================================================================
      SUBROUTINE ICDFT(CA,CB,CM,SCR,LSCR,PSI,RHOE)
C     ==--------------------------------------------------------------==
C     == Computes the transition matrix element from                  ==
C     == an inverse CDFT                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'coor.inc'
      INCLUDE 'rswf.inc'
      INCLUDE 'cdft.inc'
      INCLUDE 'filn.inc'
      INCLUDE 'store.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'transme.inc'

      COMPLEX*16   CA(*),CB(*),CM(*),RSAVE(NNR1,*)
      COMPLEX*16   PSI(NNR1,*)
      REAL*8       RHOE(NNR1,*),DEB
      POINTER      (IP_RSAVE,RSAVE)
      INTEGER      IREC(100),NFI,LSCR
      REAL*8       EIGV(N),TAUI(3,NAX,NSX),ESAVE
      CHARACTER*20 FILN_BACK
      REAL*8       SCR(*)
      CALL MEMORY(IP_RSAVE,2*NNR1*N+8,'RSAVE')
      CALL MEMORY(IP_WA,NNR1,'WEIGHTA')
      CALL MEMORY(IP_WD,NNR1,'WEIGHTD')

      FILN_BACK=FILN
      RWF=.TRUE.
      RCO=.TRUE.
      CALL READ_IREC(IREC)

C     Load reference state A and calculate energy gradient
      IF(PARENT)WRITE(6,*)"Reading reference state 1"
      FILN = "RESTART.REF1"
      CALL ZHRWF(1,IREC,CA,CM,N,EIGV,TAU0,VELP,TAUI,NFI)
      IF(PARENT)WRITE(6,*)"Calculating Density"
      CALL RHOOFR(CA,RHOE,PSI,SCR,LSCR,N)
      IF(PARENT)WRITE(6,*)"Computing Energy Gradient"
      CALL FORCES(CA,CM,TAU0,FION,RHOE,PSI,SCR,LSCR,
     *              N,1,.TRUE.,.FALSE.)
      IF(PARENT)WRITE(6,*)"Saving Realspace WF"
      CALL DCOPY(NNR1*(N),RSWF,1,RSAVE,1)
      IF(PARENT)WRITE(6,*)"Generating External Potential"
      CALL FIND_EPOT(CA,CM,WA)
      ESAVE=ETOT
      IF(PARENT)WRITE(6,*)"Writing External Potential"
      IF(CDFT_WEIGHT)CALL WRITE_W(WA,"STATE_1")


!       DEBUG DEBUG DEBUG
      CDFT=.TRUE.
      CDFT_V=1.0D0
!       CALL CDFT_INT(RHOE,WA,TAU0(1,1,1),1,DEB)
!       WRITE(6,*)"Charge",ZV(1)-DEB*OMEGA/(NR1S*NR2S*NR3S)
      CALL MEMORY(IP_WDIFF,NNR1,'WDIFF')
      CALL DCOPY(NNR1,WA,1,WDIFF,1)
      CALL FORCES(CA,CM,TAU0,FION,RHOE,PSI,SCR,LSCR,
     *              N,1,.TRUE.,.FALSE.)
      CALL FIND_EPOT(CA,CM,WA)


! C     Load reference state B and calculate energy gradient
!       IF(PARENT)WRITE(6,*)"Reading reference state 2"
!       FILN = "RESTART.REF2"
!       CALL ZHRWF(1,IREC,CB,CM,N,EIGV,TAU0,VELP,TAUI,NFI)
!       CALL RHOOFR(CB,RHOE,PSI,SCR,LSCR,N)
!       CALL FORCES(CB,CM,TAU0,FION,RHOE,PSI,SCR,LSCR,
!      *              N,1,.TRUE.,.FALSE.)
!       CALL FIND_EPOT(CB,CM,WD)
!       IF(CDFT_WEIGHT)CALL WRITE_W(WD,"STATE_2")

!       CALL TRANSMEEXT(CA,CB,RSAVE,RSWF,ESAVE,ETOT)

      FILN=FILN_BACK
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE FIND_EPOT(C0,CG,EXPOT)
C     ==--------------------------------------------------------------==
C     == Computes the external potential via inverse CDFT             ==
C     == IN : C0 the reference WF                                     ==
C     == IN : CG the energy gradient                                  ==
C     == OUT: EXPOT the external potential                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'cdft.inc'
      INCLUDE 'rswf.inc'

      COMPLEX*16 C0(NGW,*),CG(NGW,*),PSI1(*),PSI2(*),T,T2,ONE,ZCOLD
      REAL*8     C0F(NNR1,*),CGF(NNR1,*) !factors
      POINTER    (IP_C0F,C0F),(IP_CGF,CGF)
      POINTER    (IP_PSI1,PSI1),(IP_PSI2,PSI2)
      REAL*8     PAR(NNR1,N),EXPOT(NNR1),TR,TR2,NORM,COE1,COE2,DCOLD
      INTEGER    I,J
      DIMENSION  F(N)
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_C0F,NNR1*N,'C0F')
      CALL MEMORY(IP_CGF,NNR1*N,'CGF')
      CALL MEMORY(IP_PSI1,2*NNR1,'PSI1')
      CALL MEMORY(IP_PSI2,2*NNR1,'PSI2')
      CALL AZZERO(EXPOT,NNR1)
      CALL AZZERO(C0F,NNR1*N)
      CALL AZZERO(CGF,NNR1*N)
!       NORM=OMEGA/NR1S*NR2S*NR3S
      NORM=NR1S*NR2S*NR3S
      ONE=DCMPLX(1.0D0,0.0D0)
      CALL SETFFTN(0)
!       CALL AZZERO(CG,NGW*N*2)

C     Calculate factors
      DO I=1,N,2
        CALL AZZERO(PSI1,NNR1)
        CALL AZZERO(PSI2,NNR1)
        IF(I.EQ.N)THEN
          DO J=1,NGW
            T=C0(J,I)
!             T=ONE/C0(J,I)
            PSI1(NZFS(J))=T
            PSI1(INZS(J))=DCONJG(T)
            T=CG(J,I)
            PSI2(NZFS(J))=DCONJG(T)
            PSI2(INZS(J))=T
          ENDDO
        ELSE
          DO J=1,NGW
            T=C0(J,I)
            T2=C0(J,I+1)
!             T=ONE/C0(J,I)
!             T2=ONE/C0(J,I+1)
            PSI1(NZFS(J))=T+UIMAG*T2
            PSI1(INZS(J))=DCONJG(T)+UIMAG*DCONJG(T2)
            T=CG(J,I)
            T2=CG(J,I+1)
            PSI2(NZFS(J))=DCONJG(T)+UIMAG*DCONJG(T2)
            PSI2(INZS(J))=T+UIMAG*T2
          ENDDO
        ENDIF
        IF(GEQ0.AND.(I.EQ.N)) THEN
!           PSI1(NZFS(1))=ONE/DCONJG(C0(1,I))
          PSI1(NZFS(1))=C0(1,I)
          PSI2(NZFS(1))=DCONJG(CG(1,I))
        ELSEIF(GEQ0) THEN
!           PSI1(NZFS(1))=ONE/DCONJG(C0(1,I))+UIMAG/DCONJG(C0(1,I+1))
          PSI1(NZFS(1))=C0(1,I)+UIMAG*C0(1,I+1)
          PSI2(NZFS(1))=DCONJG(CG(1,I))+UIMAG*DCONJG(CG(1,I+1))
        ENDIF

        CALL INVFFTN(PSI1,.FALSE.)
        CALL INVFFTN(PSI2,.FALSE.)
        COE1=SQRT(F(I)/OMEGA)
        IF(I.LT.N) COE2=SQRT(F(I+1)/OMEGA)
        DO J=1,NNR1
          C0F(J,I)=DREAL(PSI1(J))*COE1
          IF(I.LT.N) C0F(J,I+1)=DIMAG(PSI1(J))*COE2
          CGF(J,I)=DREAL(PSI2(J))*COE1
          IF(I.LT.N) CGF(J,I+1)=DIMAG(PSI2(J))*COE2
        ENDDO
!         CALL DCOPY(2*NNR1,PSI1(1),1,C0F(1,IWF),1)
!         CALL DCOPY(2*NNR1,PSI2(1),1,CGF(1,IWF),1)
      ENDDO

C     Calculate external potential
      DO I=1,NNR1
        TR=0.0D0
        TR2=0.0D0
        DO J=1,N
          TR=TR+C0F(I,J)
          TR2=TR2+CGF(I,J)
!           IF(ABS(CGF(I,J)).GT.DCOLD)DCOLD=C0F(I,J)
        ENDDO
        EXPOT(I)=TR2/(N*TR)!*OMEGA/NORM
      ENDDO

      DCOLD=0.0D0
      DO I=1,NGW
       DO J=1,N
        IF(ABS(C0(I,J)).GT.DCOLD)DCOLD=ABS(CG(I,J))
       ENDDO
      ENDDO
      WRITE(6,*)"Largest component of the Gradient, ",DCOLD

!       DCOLD=0.0D0
!       DO J=1,NNR1
!         DCOLD=DCOLD+C0F(J,1)*C0F(J,1)!CGF(J,1)*CGF(J,1)
! !         DCOLD=DCOLD+DREAL(RSWF(J,1))*DREAL(RSWF(J,1))
!       ENDDO
!       WRITE(6,*)"RS Integral of the Gradient: ",DCOLD/NORM
!       DCOLD=DREAL(CG(1,1)*CG(1,1))
!       DO J=2,NGW
!         DCOLD=DCOLD+2.0D0*DREAL(CG(J,1)*CG(J,1))
!       ENDDO
!       WRITE(6,*)"Integral: ",DCOLD
!       DO I=1,N-1
!         EXPOT(I)=0.0D0
!         DO J=1,NNR1
!           EXPOT(I)=EXPOT(I)+C0F(J,I)*C0F(J,I)
!         ENDDO
!       ENDDO
!       CALL GLOSUM(N,EXPOT)
!       IF(PARENT)THEN
!         DO J=1,N-1
!           WRITE(6,*)EXPOT(J)
!         ENDDO
!       ENDIF
      RETURN
      END
C     ==================================================================
