C     =====================================================================
      SUBROUTINE inr_dr(c0,c1,nstate,XPAR,DXPAR,SCR,LSCR,psi,rhoe,
     &              tras,tmp,eirop,eivps,told,gold)
C     ==-----------------------------------------------------------------==
c     ==  Calls INR routine or SDION/RGDIIS, if mixed scheme is adopted  ==
C     ==  SOLVE THE EXACT QUADRATIC EQUATION AND DETERMINE AN IONIC STEP ==
C     ==-----------------------------------------------------------------==
C     == INPUT:                                                          ==
C     ==   XPAR(NODIM)                                                   ==
C     ==   DXPAR(NODIM)                                                  ==
C     ==   TOL_INR                                                       ==
C     == OUTPUT:                                                         ==
C     ==   XPAR(NODIM)                                                   ==
C     ==   (NODIM Number of degrees of freedom                           ==
C     ==   LSCR maximum value needed for scratch array                   ==
C     ==-----------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'xinr.inc'
      Include 'nlps.inc'
      INclude 'elct.inc'
      include 'response_p.inc'
      include 'fft.inc'
      include 'implhv.inc'
      include 'coor.inc'
      include 'sfac.inc'
      include 'ropt.inc'
      include 'norm.inc'
      include 'soft.inc'
      include 'irat.inc'

C     Arguments
      INTEGER LSCR,nstate,i
      REAL*8  XPAR(*),DXPAR(*),SCR(LSCR)
      real*8  told(*),gold(*)
C     Variable
c     ==--------------------------------------------------------------==
cccc
      complex*16 c0(ngw,nstate),c1(ngw,nstate)
      complex*16 psi(*)
      complex*16 eirop(nhg),eivps(nhg)
      real*8     rhoe(*),tmp(*),tras(3*nat,*)
      real*8     FNL(imagp,nat,nhxs,nstate,nkpnt)
      real*8     DFNL(imagp,nat,nhxs,3,nstate,nkpnt)
      integer    msglen

      logical done
      save done

      INTEGER IFIRST
      DATA    IFIRST /0/
      SAVE    IFIRST       

C     ==--------------------------------------------------------------==
C     ==  SOLVE HESS*S = -FION FOR HESS POSITIVE DIFINITE             ==
C     ==--------------------------------------------------------------==
c      CALL TISET('    INR_DR',ISUB)
C     ==--------------------------------------------------------------==
      
      if (ifirst.eq.0) then
         ifirst=1
         if ((infi.eq.1).and.(tmixsd.or.tmixgdiis)) then
            done=.false.
         else
            done=.true.
         endif
      endif

      if (.not.done) then
         if(parent) then
            if (tmixsd) CALL SDION(XPAR,DXPAR,SCR,LSCR)
            if (tmixgdiis) CALL rgdiis(XPAR,DXPAR,TOLD,GOLD,SCR,LSCR)
            CALL GNODIM(DXPAR,NODIM,GNMAX,GNORM)

            if (gnmax.lt.rmixsd) then 
               done=.true.
c ... changing tmixgdiis for hessup in rgmopt
               tmixgdiis=.false.
               write(6,'(/,4x,a,/)') 'THE MIXED PROCEDURE HAS REACHED'
     &         //' THE PRESCRIBED THRESHOLD'
               WRITE(6,'(4x,a,/)') '******* STARTING NOW INR *******'
               WRITE(6,*) ' '
            endif
         endif
#ifdef PARALLEL
         msglen= 1 * 8 / IRAT
         call my_bcast(done,msglen,source,allgrp)
#endif
         if(.not.exsoft) call testex(exsoft)
         if (exsoft) goto 541
      endif

c ... at this point "done" can be true because we are not using a mixed 
c ... optimization OR we reached the condition to start INR. The double 
c ... "if" statement avoids a second, useless, c0 optimization 

      if(done) then
         if(parent) then
            CALL GNODIM(DXPAR,NODIM,GNMAX,GNORM)
            do i=1,nreg
               if(gnmax.gt.gnx_inr(i)) then
                  tol_inr=tolx_inr(i)
                  goto 312
               endif
            enddo
 312        continue
         endif
#ifdef PARALLEL
         msglen= 1 * 8
         call my_bcast(tol_inr,msglen,source,allgrp)
#endif

         CALL INR(c0,c1,nstate,XPAR,DXPAR,SCR,LSCR,psi,rhoe,
     &        tras,tmp,eirop,eivps)
         if(.not.exsoft) call testex(exsoft)
         if(exsoft) goto 541
      endif

 541  continue

c      CALL TIHALT('    INR_DR',ISUB)

      return
      end


c     *********************************************************************

C     =====================================================================
      SUBROUTINE INR(c0,c1,nstate,XPAR,DXPAR,SCR,LSCR,psi,rhoe,
     &              tras,tmp,eirop,eivps)
C     ==-----------------------------------------------------------------==
C     ==  SOLVE THE EXACT QUADRATIC EQUATION AND DETERMINE AN IONIC STEP ==
C     ==-----------------------------------------------------------------==
C     == INPUT:                                                          ==
C     ==   XPAR(NODIM)                                                    ==
C     ==   DXPAR(NODIM)                                                  ==
C     ==   TOL_INR                                                       ==
C     == OUTPUT:                                                         ==
C     ==   XPAR(NODIM)                                                   ==
C     ==   (NODIM Number of Freedom degres                               ==
C     ==   SCR(LSCR) Scratch array                                       ==
C     ==   LSCR maximum value needed for scratch array                   ==
C     ==-----------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      include 'ener.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'xinr.inc'
      include 'elct.inc'
      include 'nlps.inc'
      include 'fft.inc'
      include 'response_p.inc'
      include 'cppt.inc'
      include 'implhv.inc'  
      include 'coor.inc'
      include 'sfac.inc'
      include 'ropt.inc'
      include 'cnst.inc'
      include 'irat.inc'
      include 'soft.inc'
      
C     Arguments
      INTEGER LSCR,nstate,i
      REAL*8  XPAR(*),DXPAR(*),SCR(LSCR)
C     Variable
      INTEGER ITER
      REAL*8 BK1,BK2,AK1,AK2,AK,BK,ER1,DNRM2,GNORM
      REAL*8 DDOT
      EXTERNAL DDOT
c     ==--------------------------------------------------------------==
      complex*16 c0(ngw,*),c1(ngw,*)
      complex*16 psi(*),pippo(nhg)
      complex*16 eirop(nhg),eivps(nhg)
      real*8     rhoe(*),tmp(*),tras(3*nat,*)
      real*8     FNL(imagp,nat,nhxs,nstate,nkpnt),norm
      real*8     DFNL(imagp,nat,nhxs,3,nstate,nkpnt)
      integer    dummy,msglen,lfnl,ldfnl
      logical    occurred

      real*8     tscr(*),fscr(*),z11(nstate,nstate),drhoe(*)
      pointer    (ip_z11,z11),(ip_drhoe,drhoe),(ip_tscr,tscr),
     &           (ip_fscr,fscr)

      INTEGER IFIRST
      DATA    IFIRST /0/
      SAVE    IFIRST       
c ... HESSUP
      real*8 ypar(*),dypar(*),dxmax,dx
      pointer (ip_ypar,ypar),(ip_dypar,dypar),(ip_pippo,pippo)
      real*8 dequad,deq_acc

      if (parent) call memory(ip_ypar,nodim,' local_ypar')
      if (parent) call memory(ip_dypar,nodim,' local_dypar')
      call memory(ip_pippo,2*maxfft,'pippo')
      call memory(ip_z11  ,nstate*nstate,'z11')
      call memory(ip_drhoe,2*maxfft,'drhoe')
      call memory(ip_tscr ,3*NAX*NSX,'tscr')
      call memory(ip_fscr ,3*NAX*NSX,'fscr')

c ... init
      occurred=.false.
c      sconvgd=.false.
      dxmax=0.2d0

C     ==--------------------------------------------------------------==
C     ==  SOLVE HESS*S = -FION FOR HESS POSITIVE DIFINITE             ==
C     ==--------------------------------------------------------------==
c      CALL TISET('      INR',ISUB)
      
      IF (INR_PREC.AND.PARENT) THEN
         CALL MEMORY(IP_HESS_1,NODIM*NODIM,' HESS_1')
         CALL HESSINinr
      ENDIF
      
c     the potential v[n0]:
      call rhoofr(c0,rho0,psi,scr,lscr,nstate)
      if(ttau) call stopgm("inr_dr","no tau functionals")
      call DCOPY(nnr1, rho0, 1, vofrho0, 1)
      call vofrho(tau0,fion,vofrho0,psi,scr,lscr,.false.,.false.)

c     calculate the lagrange parameters z11
      call lag_mult(c0,c1,psi,scr,lscr,rhoe,z11,nstate)
      call eicalc(eivps,eirop)

      call rnlsm3(c0,scr,lscr,nstate,ddfnl)
      
c     ==--------------------------------------------------------------==
C     FNL,DFNL,DDFNL...
      ndfnl=nst12(mepos,2)-nst12(mepos,1)+1
      ldfnl=imagp*3*nat*nhxs*ndfnl*nkpnt
      if(ldfnl.le.0) ldfnl=1
      lfnl = imagp*nstate*nat*nhxs*nkpnt

      call rnlsm(c0,nstate,psi,scr,lscr,1,1,.true.)
      call DCOPY(lfnl,   fnl,1,  fnl00,1)
      call DCOPY(ldfnl, dfnl,1, dfnl00,1)

      call azzero(sd0,3*3*nat*nat)
      call setsd0(eirop,ddfnl,nstate)
      call azzero(rs_v,2*3*nat)

      if(parent) call settras(tras)
         
      ITER=0
      deq_acc=0.d0
      if(parent) then
         if(inr_cont.and.(ifirst.eq.0)) then
            call cont_inr(nodim,iter,bk1,tol_inr,deq_acc,
     &           correct,xpar,'READ')
            write(6,'(/,2x,a,3x,i6,/)') 'RESTARTING  A PREVIOUS JOB --'
     &           //' WITH ITERATION',iter+1
            ifirst=1
         endif
         write(6,'(/,2x,a,e10.4)') 'STARTING  INR:  '//
     &        'RESIDUAL THRESHOLD FOR CONVERGENCE =',tol_inr
      endif

#ifdef PARALLEL
      msglen =1 * 8 /IRAT
      CALL MY_BCAST(IFIRST,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(ITER,MSGLEN,SOURCE,ALLGRP)
      msglen = 8
      call my_bcast(tol_inr,msglen,source,allgrp)
      msglen = nodim * 8
      call my_bcast(correct,msglen,source,allgrp)
#endif 
      

 199  FORMAT(18x,'RESIDUAL AT ',I4,3x,'ITERATION ',6x,'=',e10.4)
 100  IF(ITER.LT.ITMAX_INR) THEN 

         ITER=ITER+1

         IF(ITER.EQ.1) THEN

C.. r(0)=gradient
            if(parent) then

               call DCOPY(nodim,dxpar,1,correct,1)
               call DSCAL(nodim,-1.d0,correct,1)
               call no_mode(tras,correct,nat)
               
C..   compute z(0)=A(prec)^(-1)*r(0)
               IF (INR_PREC) THEN
                  CALL inrSOLVE(CORRECT,ZPREC)
               ELSE
                  CALL DCOPY(NODIM,CORRECT,1,ZPREC,1)
               ENDIF
               call no_mode(tras,zprec,nat)
               
               call DCOPY(nodim,zprec,1,direct,1)
               BK1=DDOT(nodim,correct,1,zprec,1)

               CALL GNODIM(dxpar,NODIM,ER1,GNORM)

               WRITE(6,199)ITER,ER1
            endif

#ifdef PARALLEL
            msglen = nodim * 8
            call my_bcast(direct,msglen,source,allgrp)
#endif 
         ELSE
            if(parent) then
C.. compute z(0)=A(prec)^(-1)*r(0)
               IF (INR_PREC) THEN
                  CALL inrSOLVE(CORRECT,ZPREC)
               ELSE
                  CALL DCOPY(NODIM,CORRECT,1,ZPREC,1)
               ENDIF
               call no_mode(tras,zprec,nat)
               
c..   calculation of the coefficients necessary for the linear combination
               BK2=BK1
               
               BK1=DDOT(NODIM,CORRECT,1,ZPREC,1)

               BK=BK1/BK2
C..   Z_k=Z_k+bk*p_(k-1)
               CALL DAXPY(NODIM,BK,DIRECT,1,ZPREC,1)
C..   P_K=Z=K
               CALL DCOPY(NODIM,ZPREC,1,DIRECT,1)
            endif

#ifdef PARALLEL
            msglen = nodim * 8
            call my_bcast(direct,msglen,source,allgrp)
#endif

         ENDIF

C ...  compute Hp(iter) and store it in z 
         CALL DCOPY(NODIM,direct,1,rs_v(1,1),1)

c ... normalizing to 1 the perturbation vector for numerical reasons
         NORM=DNRM2(3*NAT,rs_v(1,1),1)
         call DSCAL(3*nat, 1.d0/norm,rs_v(1,1),1)
         CALL DCOPY(NODIM,rs_v(1,1),1,tmp,1)

c     ==--------------------------------------------------------------==
c ... H.p product
c     ==--------------------------------------------------------------==
         call hess_eta_p(c0,c1,psi,rhoe,drhoe,scr,lscr,
     &        eirop,eivps,z11,nstate,1,dummy,ddfnl,tmp)
c     ==--------------------------------------------------------------==

         if(.not.exsoft) call testex(exsoft)

         if (exsoft) then 
            call puttau(tau0,xpar)
            goto 214
         endif

c     ... restoring the original norm in the H.p vector
         call DSCAL(3*nat,norm,rs_v(1,2),1)
#ifdef PARALLEL
         call glosum(NODIM,RS_V(1,2))
#endif
         
         if(parent) then
            call no_mode(tras,rs_v(1,2),nat)

            IF (INR_STEP) then
               AK1=DDOT(nodim,rs_v(1,2),1,correct,1)
               AK2=(DNRM2(nodim,rs_v(1,2),1))**2
            ELSE
               AK2=DDOT(NODIM,direct,1,rs_v(1,2),1)
               AK1=BK1
            ENDIF
            
            AK=AK1/AK2
            if(ak.lt.0.d0) then
               write(6,*) ' ' 
               WRITE(6,559) iter,AK
               write(6,*) ' !!!!!!!!!  WARNING WARNING WARNING  !!!!!!!'
               write(6,*) ' ALPHA(K) IS NEGATIVE: BREAKING INR CYCLE'
               write(6,*) ' ' 
c ... ak is set to tolng, minimal displacement
               ak=tolng
               occurred=.true.
            endif
 559        FORMAT(1x,' ALPHA(',i3,') =',f10.5)
            
            call gettau(tscr,xpar)
            call gettau(fscr,direct)
            call wrgeof_inr(tscr,fscr)
            WRITE(6,'(1X,64("*"),/)')

cccc  .... Hessian....
            call DCOPY (nodim,xpar,1,ypar,1)
            call DCOPY (nodim,correct,1,dypar,1)

c     ... x_k=x_(k-1)+alfa_k.p_k
            call DAXPY(NODIM,AK,DIRECT,1,XPAR,1)

c ... change in quadratic energy
            dequad = DDOT(nodim,direct,1,correct,1)
            dequad = -dequad*ak+ak*ak*ak2*.5d0
            deq_acc=deq_acc+dequad

c     ... r_k=r_(k-1)-alfa_k.A.p_k
            call DAXPY(NODIM,-AK,RS_V(1,2),1,CORRECT,1)

c ... HESSIAN UPDATE
            call HESSUP(xpar,ypar,dypar,correct,SCR,LSCR)

            DXMAX=0.0D0
C     Check convergence on atomic positions (for the next it.)
C     (DXMAX .LT. 0.01*TOLNG)
            DO I=1,NODIM
               DX=ABS(XPAR(I)-YPAR(I))
               DXMAX=MAX(DX,DXMAX)
            ENDDO
            IF(DXMAX.LT.1.D-2*TOLNG) then
               write(6,'(1x,a,/)') 'NEGLIGIBLE DISPLACEMENT;'
     &              //' BREAKING INR CYCLE'
               occurred=.true.
            endif

            CALL GNODIM(CORRECT,NODIM,ER1,GNORM)

            WRITE(6,'(1X,64("*"))')

            WRITE(6,561) iter,iter,AK
            write(6,198) er1,dequad,dxmax
            IF(INR_VERBOSE) call delta_rcm(direct,nodim)

            WRITE(6,'(1X,64("*"),/)')
 198        format(x,'RESIDUAL=',2x,e10.4,3x,'DEQUAD.=',x,e10.4,4x,
     &           'DXMAX=',x,e10.4)
 561        FORMAT(1x,'ITERATION',3X,i3,4x,' ALPHA(',i3,') =',f9.5)

         endif

#ifdef PARALLEL
         msglen = 1 * 8
         call my_bcast(er1,msglen,source,allgrp)
         msglen = 1 * 8/irat
         call my_bcast(occurred,msglen,source,allgrp)
#endif

         if(parent) then
            call gettau(tscr,xpar)
            call geofile(tscr,fscr,'WRITE')
            call cont_inr(3*nat,iter,bk1,tol_inr,deq_acc
     &           ,correct,xpar,'WRITE')
         endif

         if(occurred) goto 214
         
         if(.not.exsoft) call testex(exsoft)
         if (exsoft) then 
            call puttau(tau0,xpar)
            goto 214
         endif
         IF(ER1.GT.TOL_INR) GOTO 100
      ENDIF

 214  continue

      if(parent) then
         IF(ER1.LE.TOL_INR) THEN
            WRITE(6,215) deq_acc
         else
            WRITE(6,216) deq_acc
         ENDIF
         WRITE(6,'(1X,64("*"),/)')
         IF (INR_PREC) CALL FREEM(IP_HESS_1)
      endif
 215  format(1x,'INR CONVERGED:        CYCLE CHANGE IN ENERGY',3X,E10.4,
     &     2X,'A.U.')
 216  format(1x,'INR DID NOT CONVERGE: CYCLE CHANGE IN ENERGY',3X,E10.4,
     &     2X,'A.U.')
      
      if(parent) call freem(ip_ypar)
      if(parent) call freem(ip_dypar)
      call freem(ip_pippo)

c      CALL TIHALT('      INR',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================


C**************************** SUBROUTINE **********************************



C     ==================================================================
      SUBROUTINE INRSOLVE(X,Y)
C     ==================================================================
C     == Y=A^(-1)*X                                                   ==
C     ==--------------------------------------------------------------==
      implicit none
      include 'cotr.inc'
      INCLUDE 'xinr.inc'
c     arguments
      REAL*8 X(NODIM),Y(NODIM)

      CALL AZZERO(Y,NODIM)
      CALL DGEMV ('N',NODIM,NODIM,1.d0,HESS_1,NODIM,X,1,0.D0,Y,1)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE HESSININR
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cotr.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'xinr.inc'

c     variables
      INTEGER I,J,K
      REAL*8 HESS_LPACK(*)
      REAL*8 W(NODIM),VECT(NODIM,NODIM),VECT_1(NODIM,NODIM)
      REAL*8 WORK(3*NODIM)
      real*8 wwus,wwu,wwd,wwds
      POINTER (IP_HESS_LPACK,HESS_LPACK)
      WRITE(6,'(2x,a)') 'INR: CONSTRUCTION OF THE PRECONDITIONER'
c     allocations
      CALL MEMORY(IP_HESS_LPACK,NODIM*(NODIM+1)/2,' HESS_LPACK')

c ... parameters
c ... 1.5d-1 a.u. ~ 65 cm^-1

      WWD=1.D-2
      WWDS=0.9d0
      WWU=1.05d0
      wwus=1.05d0

      K = 0
      DO  I=1,NODIM
         DO  J=I,NODIM
            K = K+1
            HESS_LPACK(K)=HESS(J,I)
         END DO
      END DO
      CALL DSPEVY(1,HESS_LPACK,W,VECT,NODIM,NODIM,WORK,3*NODIM)

      CALL AZZERO(VECT_1,NODIM*NODIM)

      do i=1,nodim
         IF(W(I).LE.wwd) THEN

            CALL DCOPY(NODIM,VECT(1,I),1,VECT_1(1,I),1)
            IF (INR_VERBOSE) WRITE(6,890)I,W(I),
     &           SIGN(5140.487D0*DSQRT(DABS(W(I))),W(I)),WWDS
            CALL DSCAL(NODIM,1/WWDS,VECT_1(1,I),1)

         ELSE IF(W(I).GT.wwu) THEN

            CALL DCOPY(NODIM,VECT(1,I),1,VECT_1(1,I),1)
            CALL DSCAL(NODIM,1/WWUS,VECT_1(1,I),1)
            IF (INR_VERBOSE) WRITE(6,890)I,W(I),
     &           SIGN(5140.487D0*DSQRT(DABS(W(I))),W(I)),WWUS

         ELSE

            IF (INR_VERBOSE) WRITE(6,889)I,W(I),
     &           SIGN(5140.487D0*DSQRT(DABS(W(I))),W(I))
            CALL DCOPY(NODIM,VECT(1,I),1,VECT_1(1,I),1)
            CALL DSCAL(NODIM,1/W(I),VECT_1(1,I),1)

         END IF
      ENDDO

 889  FORMAT(2x,'EIGENV. ', I3,' (', e11.4,')','(', f12.3,')',' SCALED') 
 890  FORMAT(2x,'EIGENV. ', I3,' (', e11.4,')','(', f12.3,')',
     &     ' SCALED BY', F10.6)
      CALL DGEMM('N','T',NODIM,NODIM,NODIM,1.D0,VECT,
     &     NODIM,VECT_1,NODIM,0.D0,HESS_1,NODIM)


      CALL FREEM(IP_HESS_LPACK)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================



C     ==================================================================
      SUBROUTINE GIVE_SCR_INR(LRINR,TAG,nstate)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !PARENT
      INCLUDE 'cotr.inc'        !NODIM MCNSTR
C     Arguments
      INTEGER   LRINR,lphonon,nstate
      CHARACTER TAG*30
C     ==--------------------------------------------------------------==
      LRINR=NODIM*4
      call give_scr_phonon(lphonon,tag,nstate)
      lrinr=max(lphonon,lrinr)
      call give_scr_perturbation(lphonon,tag,nstate)
      lrinr=max(lphonon,lrinr)
      TAG ='NODIM'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE cont_inr(ndim,iter,bk1,tol_inr,deq_acc,
     &     correct,xpar,tag)
C     ==--------------------------------------------------------------==
      implicit none
      include 'fileopen.inc'
c     Arguments
      integer iter,ndim
      real*8 bk1,tol_inr,deq_acc,correct(*),xpar(*)
C     Variables
      CHARACTER FILEN*100,tag*5
      INTEGER   K,iunit
      LOGICAL   FERROR
C     ==--------------------------------------------------------------==
      iunit=21

      IF(INDEX(TAG,'READ').NE.0) THEN
         FILEN='INR_CONTINUE'
         CALL FILEOPEN(IUNIT,FILEN,FO_OLD,FERROR)
         IF(FERROR) THEN
            WRITE(6,'(A)') ' CONT_INR! NO FILE INR_CONTINUE WAS FOUND' 
            CALL STOPGM('CONT_INR','CANNOT CONTINUE PREVIOUS JOB')
         ENDIF

         REWIND(IUNIT)
         read(iunit,'(i12,3f15.10)') iter,bk1,tol_inr,deq_acc
         DO K=1,NDIM
            read(iunit,*) correct(k),xpar(k)
         ENDDO
      ELSEIF(INDEX(TAG,'WRITE').NE.0) THEN
         FILEN='INR_CONTINUE.1'
         CALL FILEOPEN(IUNIT,FILEN,FO_DEF,FERROR)

         REWIND(IUNIT)
         write(iunit,'(i12,3f15.10)') iter,bk1,tol_inr,deq_acc
         DO K=1,NDIM
            WRITE(iunit,*) correct(k),xpar(k)
         ENDDO
      ELSE
        WRITE(6,'(A,A)') ' CONT_INR| UNKNOWN TAG: ',TAG
      ENDIF
      
      CALL FILECLOSE(IUNIT)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================


C     ==================================================================

c     ==--------------------------------------------------------------==
      subroutine delta_rcm(mode,nrighe)
c     ==--------------------------------------------------------------==
      implicit none

      include 'system.h'
      include 'rmas.inc'
      include 'ions.inc'
      include 'response_p.inc'
      
c ... arguments
      integer nrighe
      real*8 mode(*)

c ... internal variables
      integer i,j,l,k
      real*8 sum(3),norm

      norm=0.d0
      do j=1,nsp
         norm=norm+na(j)*pma0(j)**2
      enddo
      l=0
      do k=1,3
         sum(k)=0.d0
      enddo
      do j=1,nsp
         do i=1,na(j)
            do k=1,3
               l=l+1
               sum(k)=sum(k)+mode(l)*pma0(j)/norm
c     sum=sum+mode(l)*dsqrt(masse(i))
            enddo
         enddo
      enddo
      write(6,198) (sum(k),k=1,3)
 198  format(x,'COM',3x,'DX=  ',e10.4,8x,'DY= ',e10.4,7x,'DZ= ',e10.4)

      return
      end

c     ==--------------------------------------------------------------==
      
 
