C     ==================================================================
      SUBROUTINE K290(IOUT,NAT,NKPOINT,NSP,IQ1,IQ2,IQ3,ISTRIZ,
     &                A1,A2,A3,ALAT,STRAIN,XKAPA,RX,TVEC,
     &                TY,ISC,F0,NTVEC,WVK0,WVKL,LWGHT,LROT,
     &                NHASH,INCLUD,LIST,RLIST,DELTA)
C     ==================================================================
C     WRITTEN ON SEPTEMBER 12TH, 1979.
C     IBM-RETOUCHED ON OCTOBER 27TH, 1980.
C     Tsukuba-retouched on March 19th, 2008.
C     GENERATION OF SPECIAL POINTS MODIFIED ON 26-MAY-82 BY OHN.
C     RETOUCHED ON JANUARY 8TH, 1997
C     INTEGRATION IN CPMD-FEMD PROGRAM BY THIERRY DEUTSCH
C     ==--------------------------------------------------------------==
C     PLAYING WITH SPECIAL POINTS AND CREATION OF 'CRYSTALLOGRAPHIC'
C     FILE FOR BAND STRUCTURE CALCULATIONS.
C     GENERATION OF SPECIAL POINTS IN K-SPACE FOR AN ARBITRARY LATTICE,
C     FOLLOWING THE METHOD MONKHORST,PACK, PHYS. REV. B13 (1976) 5188
C     MODIFIED BY MACDONALD, PHYS. REV. B18 (1978) 5897
C     MODIFIED ALSO BY OLE HOLM NIELSEN ("SYMMETRIZATION")
C     ==--------------------------------------------------------------==
C     TESTING THEIR EFFICIENCY AND PREPARATION OF THE
C     "STRUCTURAL" FILE FOR RUNNING THE
C     SELF-CONSISTENT BAND STRUCTURE PROGRAMS.
C     IN THE CASES WHERE THE POINT GROUP OF THE CRYSTAL DOES NOT
C     CONTAIN INVERSION, THE LATTER IS ARTIFICIALLY ADDED, IN ORDER
C     TO MAKE USE OF THE HERMITICITY OF THE HAMILTONIAN
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   IOUT    LOGIC FILE NUMBER                                  ==
C     ==   NAT     NUMBER OF ATOMS                                    ==
C     ==   NKPOINT MAXIMAL NUMBER OF K POINTS                         ==
C     ==   NSP     NUMBER OF SPECIES                                  ==
C     ==   IQ1,IQ2,IQ3 THE MONKHORST-PACK MESH PARAMETERS             ==
C     ==   ISTRIZ  SWITCH FOR SYMMETRIZATION                          ==
C     ==   A1(3),A2(3),A3(3) LATTICE VECTORS                          ==
C     ==   ALAT    LATTICE CONSTANT                                   ==
C     ==   STRAIN(3,3)  STRAIN APPLIED TO LATTICE IN ORDER            ==
C     ==           TO HAVE K POINTS WITH SYMMETRY OF STRAINED LATTICE ==
C     ==   XKAPA(3,NAT)   ATOMS COORDINATES                           ==
C     ==   TY(NAT)      TYPES OF ATOMS                                ==
C     ==   WVK0(3) SHIFT FOR K POINTS MESh (MACDONALD ARTICLE)        ==
C     ==   NHASH   SIZE OF THE HASH TABLES (LIST)                     ==
C     ==   DELTA   REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)          ==
C     ==           K-VECTOR < DELTA IS CONSIDERED ZERO                ==
C     == OUTPUT:                                                      ==
C     ==   RX(3,NAT) SCRATCH ARRAY USED BY GROUP1 ROUTINE             ==
C     ==   TVEC(1:3,1:NTVEC) TRANSLATION VECTORS (SEE NTVEC)          ==
C     ==   ISC(NAT)  SCRATCH ARRAY USED BY GROUP1 ROUTINE             ==
C     ==   F0(49,NAT) ATOM TRANSFORMATION TABLE                       ==
C     ==              IF NTVEC/=1 THE 49TH GIVES INEQUIVALENT ATOMS   ==
C     ==   NTVEC NUMBER OF TRANSLATION VECTORS (IF NOT PRIMITIVE CELL)==
C     ==   WVKL(3,NKPOINT) SPECIAL KPOINTS GENERATED                  ==
C     ==   LWGHT(NKPOINT)  WEIGHT FOR EACH K POINT                    ==
C     ==   LROT(48,NKPOINT) SYMMETRY OPERATION FOR EACh K POINTS      ==
C     ==   INCLUD(NKPOINT)  SCRATCH ARRAY USED BY SPPT2               ==
C     ==   LIST(NKPOINT+NHASH) HASH TABLE USED BY SPPT2               ==
C     ==   RLIST(3,NKPOINT) SCRATCH ARRAY USED BY SPPT2               ==
C     ==--------------------------------------------------------------==
C     SUBROUTINES NEEDED: 
C        SPPT2, GROUP1, PGL1, ATFTM1, ROT1, STRUCT,
C        BZRDUC, INBZ, MESH, BZDEFI
C     (GROUP1, PGL1, ATFTM1, ROT1 FROM THE
C      "COMPUTER PHYSICS COMMUNICATIONS" PACKAGE "ACMI" - (1971,1974)
C      WORLTON-WARREN).
C     ==================================================================
      IMPLICIT  NONE
C     Arguments
      INTEGER   IOUT,NAT,NKPOINT,NSP,NHASH,IQ1,IQ2,IQ3,ISTRIZ,NTVEC,
     &          TY(NAT),ISC(NAT),
     &          F0(49,NAT),
     &          LWGHT(NKPOINT),LROT(48,NKPOINT),
     &          INCLUD(NKPOINT),LIST(NKPOINT+NHASH)
      REAL*8    A1(3),A2(3),A3(3),ALAT,STRAIN(6),XKAPA(3,NAT),RX(3,NAT),
     &          TVEC(3,NAT),WVK0(3),WVKL(3,NKPOINT),RLIST(3,NKPOINT),
     &          DELTA
C     ==--------------------------------------------------------------==
C     Variables
      INTEGER   IB0(48),IB(48),
     &          I,ISTRIN,ITYPE,
     &          IHG,IHC,ISY,
     &          LI,NC,INDPG,J,INVADD,K,
     &          IHG0,IHC0,ISY0,LI0,NC0,INDPG0,NTVEC0,
     &          N,NTOT,ISWGHT,L,LMAX
      REAL*8    R(3,3,48),R0(3,3,48),
     &          B1(3),B2(3),B3(3),ORIGIN(3),ORIGIN0(3),VV0(3),
     &          A01(3),A02(3),A03(3),B01(3),B02(3),B03(3),TVEC0(3,1),
     &          TOTSTR,VOLUM,PROJ1,PROJ2,PROJ3,DTOTSTR
C     ==--------------------------------------------------------------==
C     Constants
      INTEGER   F00(49,1)
      REAL*8    V(3,48),V0(3,48),X0(3,1)
      CHARACTER ICST(7)*12
C     Name of 48 rotations (convention Warren-Worlton)
      CHARACTER RNAME_CUBIC(48)*10, RNAME_HEXA(24)*11
      DATA      F00 /49 * 0/
      DATA      V,V0 /288*0.D0/
      DATA      X0 /3*0.D0/
      DATA ICST
     &         /'TRICLINIC','MONOCLINIC','ORTHORHOMBIC','TETRAGONAL',
     &          'CUBIC','TRIGONAL','HEXAGONAL'/
      DATA      RNAME_CUBIC
     &    /' 1        ',' 2[ 1 0 0]',' 2[ 0 1 0]',' 2[ 0 0 1]',
     &     ' 3[-1-1-1]',' 3[ 1 1-1]',' 3[-1 1 1]',' 3[ 1-1 1]',
     &     ' 3[ 1 1 1]',' 3[-1 1-1]',' 3[-1-1 1]',' 3[ 1-1-1]',
     &     ' 2[-1 1 0]',' 4[ 0 0 1]',' 4[ 0 0-1]',' 2[ 1 1 0]',
     &     ' 2[ 0-1 1]',' 2[ 0 1 1]',' 4[ 1 0 0]',' 4[-1 0 0]',
     &     ' 2[-1 0 1]',' 4[ 0-1 0]',' 2[ 1 0 1]',' 4[ 0 1 0]',
     &     '-1        ','-2[ 1 0 0]','-2[ 0 1 0]','-2[ 0 0 1]',
     &     '-3[-1-1-1]','-3[ 1 1-1]','-3[-1 1 1]','-3[ 1-1 1]',
     &     '-3[ 1 1 1]','-3[-1 1-1]','-3[-1-1 1]','-3[ 1-1-1]',
     &     '-2[-1 1 0]','-4[ 0 0 1]','-4[ 0 0-1]','-2[ 1 1 0]',
     &     '-2[ 0-1 1]','-2[ 0 1 1]','-4[ 1 0 0]','-4[-1 0 0]',
     &     '-2[-1 0 1]','-4[ 0-1 0]','-2[ 1 0 1]','-4[ 0 1 0]'/
      DATA      RNAME_HEXA
     &    /' 1         ',' 6[ 0 0  1]',' 3[ 0 0  1]',' 2[ 0 0  1]',
     &     ' 3[ 0 0 -1]',' 6[ 0 0 -1]',' 2[ 0 1  0]',' 2[-1 1  0]',
     &     ' 2[ 1 0  0]',' 2[ 2 1  0]',' 2[ 1 1  0]',' 2[ 1 2  0]',
     &     '-1         ','-6[ 0 0  1]','-3[ 0 0  1]','-2[ 0 0  1]',
     &     '-3[ 0 0 -1]','-6[ 0 0 -1]','-2[ 0 1  0]','-2[-1 1  0]',
     &     '-2[ 1 0  0]','-2[ 2 1  0]','-2[ 1 1  0]','-2[ 1 2  0]'/
C     ==--------------------------------------------------------------==
C     READ IN LATTICE STRUCTURE
C     ==--------------------------------------------------------------==
      DO I = 1,3
        A01(I) = A1(I)/ALAT
        A02(I) = A2(I)/ALAT
        A03(I) = A3(I)/ALAT
      ENDDO
      WRITE(IOUT,'(" A1",3F10.5,/," A2",3F10.5,/," A3",3F10.5)')
     &     A01,A02,A03
      WRITE(IOUT,'(/," NUMBER OF ATOMS (STRUCT):",I6)') NAT
      WRITE(IOUT,'(5X,"K   TYPE",14X,"X(K)")')
      ITYPE = 0
      DO I = 1,NAT
C       Assign an atomic type (for internal purposes)
        IF(I .NE. 1) THEN
          DO J = 1, (I - 1)
            IF(TY(J) .EQ. TY(I)) THEN
C             Type located
              GOTO 178
            ENDIF
          ENDDO
C         New type
        ENDIF
        ITYPE = ITYPE + 1
        IF(ITYPE .GT. NSP) THEN
          WRITE(6,'(A,I4,")")')
     &         ' NUMBER OF ATOMIC TYPES EXCEEDS DIMENSION (NSP=)', 
     &         NSP
          WRITE(6,'(" THE ARRAY TY IS:",/,9(1X,10I7,/))')
     &         (TY(J),J=1,NAT)
          CALL STOPGM('K290','FATAL ERROR')
        ENDIF
 178    CONTINUE
        WRITE(IOUT,'(1X,I5,I6,3F10.5)')
     &       I,TY(I),(XKAPA(J,I),J=1,3)
      ENDDO
C     ==--------------------------------------------------------------==
C     IS THE STRAIN SIGNIFICANT ?
C     ==--------------------------------------------------------------==
      DTOTSTR = DELTA*DELTA
      TOTSTR = 0.D0
      ISTRIN = 0
      DO I = 1,6
        TOTSTR  = TOTSTR + DABS(STRAIN(I))
      ENDDO
      IF(TOTSTR .GT. DTOTSTR) ISTRIN = 1
C     ==--------------------------------------------------------------==
C     Volume of the cell.
      VOLUM  = A1(1)*A2(2)*A3(3) + A2(1)*A3(2)*A1(3) +
     +         A3(1)*A1(2)*A2(3) - A1(3)*A2(2)*A3(1) -
     +         A2(3)*A3(2)*A1(1) - A3(3)*A1(2)*A2(1)
      VOLUM  = DABS(VOLUM)
      B1(1) = (A2(2)*A3(3) - A2(3)*A3(2)) / VOLUM
      B1(2) = (A2(3)*A3(1) - A2(1)*A3(3)) / VOLUM
      B1(3) = (A2(1)*A3(2) - A2(2)*A3(1)) / VOLUM
      B2(1) = (A3(2)*A1(3) - A3(3)*A1(2)) / VOLUM
      B2(2) = (A3(3)*A1(1) - A3(1)*A1(3)) / VOLUM
      B2(3) = (A3(1)*A1(2) - A3(2)*A1(1)) / VOLUM
      B3(1) = (A1(2)*A2(3) - A1(3)*A2(2)) / VOLUM
      B3(2) = (A1(3)*A2(1) - A1(1)*A2(3)) / VOLUM
      B3(3) = (A1(1)*A2(2) - A1(2)*A2(1)) / VOLUM
C     ==--------------------------------------------------------------==
      DO I=1,3
        B01(I)=B1(I)*ALAT
        B02(I)=B2(I)*ALAT
        B03(I)=B3(I)*ALAT
      ENDDO
      WRITE(IOUT,'(/," B1",3F10.5,/," B2",3F10.5,/," B3",3F10.5,/)')
     &     B01,B02,B03
C     ==--------------------------------------------------------------==
C     == GROUP-THEORY ANALYSIS OF LATTICE                             ==
C     ==--------------------------------------------------------------==
      CALL GROUP1(IOUT,A1,A2,A3,NAT,TY,XKAPA,B1,B2,B3,
     &            IHG,IHC,ISY,LI,NC,INDPG,IB,NTVEC,
     &            V,F0,R,TVEC,ORIGIN,RX,ISC,DELTA)
C     ==--------------------------------------------------------------==
      DO N = NC+1,48
        IB(N) = 0
      ENDDO
C     ==--------------------------------------------------------------==
      INVADD=0
      IF(LI .EQ. 0) THEN
        WRITE(IOUT,'(A,/,A,/,A)')
     &    ' ALTHOUGH THE POINT GROUP OF THE CRYSTAL DOES NOT',
     &    ' CONTAIN INVERSION, THE SPECIAL POINT GENERATION ALGORITHM',
     &    ' WILL CONSIDER IT AS A SYMMETRY OPERATION'
        INVADD=1
      ENDIF
C     ==--------------------------------------------------------------==
C     == CRYSTALLOGRAPHIC DATA                                        ==
C     ==--------------------------------------------------------------==
      WRITE(IOUT,'(/," CRYSTALLOGRAPHIC DATA:")')
      WRITE(IOUT,'(4X,"A1",3F10.5/,4X,"A2",3F10.5/,4X,"A3",3F10.5,/)')
     &     A1,A2,A3
      WRITE(IOUT,'(4X,"B1",3F10.5/,4X,"B2",3F10.5/,4X,"B3",3F10.5)')
     &     B1,B2,B3
C     ==--------------------------------------------------------------==
C     == GROUP-THEORETICAL INFORMATION                                ==
C     ==--------------------------------------------------------------==
      WRITE(IOUT,'(/," GROUP-THEORETICAL INFORMATION:")')
C     IHG .... Point group of the primitive lattice, holohedral
      WRITE(IOUT,
     &  '("    POINT GROUP OF THE PRIMITIVE LATTICE: ",A," SYSTEM")')
     &  ICST(IHG)
C     IHC .... Code distinguishing between hexagonal and cubic groups
C     ISY .... Code indicating whether the space group is symmorphic
      IF(ISY.EQ.0) THEN
        WRITE(IOUT,'(4X,"NONSYMMORPHIC GROUP")')
      ELSEIF(ISY.EQ.1) THEN
        WRITE(IOUT,'(4X,"SYMMORPHIC GROUP")')
      ELSEIF(ISY.EQ.-1) THEN
        WRITE(IOUT,'(4X,"SYMMORPHIC GROUP WITH NON-STANDARD ORIGIN")')
      ELSEIF(ISY.EQ.-2) THEN
        WRITE(IOUT,'(4X,"NONSYMMORPHIC GROUP???")')
      ENDIF
C     LI ..... Inversions symmetry
      IF(LI.EQ.0) THEN
        WRITE(IOUT,'(4X,"NO INVERSION SYMMETRY")')
      ELSEIF(LI.GT.0) THEN
        WRITE(IOUT,'(4X,"INVERSION SYMMETRY")')
      ENDIF
C     NC ..... Total number of elements in the point group
      WRITE(IOUT,
     &     '(4X,"TOTAL NUMBER OF ELEMENTS IN THE POINT GROUP:",I3)') NC
      WRITE(IOUT,'(4X,"TO SUM UP: (",I1,5I3,")")')
     &     IHG,IHC,ISY,LI,NC,INDPG
C     IB ..... List of the rotations constituting the point group
      WRITE(IOUT,'(/,4X,"LIST OF THE ROTATIONS:")')
      WRITE(IOUT,'(7X,12I4)') (IB(I),I=1,NC)
C     V ...... Nonprimitive translations (for nonsymmorphic groups)
      IF(ISY.LE.0) THEN
        WRITE(IOUT,'(/,4X,"NONPRIMITIVE TRANSLATIONS:")')
        WRITE(IOUT,'(A,A)')
     &       ' ROT    V  IN THE BASIS A1, A2, A3      ',
     &       'V  IN CARTESIAN COORDINATES'
C       Cartesian components of nonprimitive translation.
        DO I=1,NC
          DO J=1,3
            VV0(J) = V(1,I)*A1(J) + V(2,I)*A2(J) + V(3,I)*A3(J)
          ENDDO
          WRITE(IOUT,'(1X,I3,3F10.5,3X,3F10.5)')
     &         IB(I),(V(J,I),J=1,3),VV0
        ENDDO
      ENDIF
C     F0 ..... The function defined in Maradudin, Ipatova by
C              eq. (3.2.12): atom transformation table.
      WRITE(IOUT,
     &     '(/,4X,"ATOM TRANSFORMATION TABLE (MARADUDIN,VOSKO):")')
      WRITE(IOUT,'(5(4X,"R  AT->AT"))')
      WRITE(IOUT,'(I5," [Identity]")') 1
      DO K=2,NC
        DO J=1,NAT
          WRITE(IOUT,'(I5,2I4,$)') IB(K),J,F0(K,J)
          IF(MOD(J,5).EQ.0) WRITE(IOUT, *)
        ENDDO
        IF(MOD(J-1,5).NE.0) WRITE(IOUT, *)
      ENDDO
C     R ...... List of the 3 x 3 rotation matrices
      WRITE(IOUT,'(/,4X,"LIST OF THE 3 X 3 ROTATION MATRICES:")')
      IF(IHC.EQ.0) THEN
        DO K=1,NC
          WRITE(IOUT,
     &         '(4X,I3," (",I2,": ",A11,")",2(3F14.6,/,25X),3F14.6)')
     &         K,IB(K),RNAME_HEXA(IB(K)),((R(I,J,IB(K)),J=1,3),I=1,3)
        ENDDO
      ELSE
        DO K=1,NC
          WRITE(IOUT,
     &         '(4X,I3," (",I2,": ",A10,") ",2(3F14.6,/,25X),3F14.6)')
     &         K,IB(K),RNAME_CUBIC(IB(K)),((R(I,J,IB(K)),J=1,3),I=1,3)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     GENERATE THE BRAVAIS LATTICE
C     ==--------------------------------------------------------------==
      WRITE(IOUT,'(/,1X,20("-"),A,20("-"),/,A)')
     &   ' THE (UNSTRAINED) BRAVAIS LATTICE ',
     &   ' (USED FOR GENERATING THE LARGEST POSSIBLE MESH IN THE B.Z.)'
C     ==--------------------------------------------------------------==
      CALL GROUP1(IOUT,A01,A02,A03,1,TY,X0,B01,B02,B03,
     &            IHG0,IHC0,ISY0,LI0,NC0,INDPG0,IB0,NTVEC0,
     &            V0,F00,R0,TVEC0,ORIGIN0,RX,ISC,DELTA)
C     ==--------------------------------------------------------------==
C     It is assumed that the same 'type' of symmetry operations
C     (cubic/hexagonal) will apply to the crystal as well as the Bravais
C     lattice.
C     ==--------------------------------------------------------------==
      WRITE(IOUT,'(/,1X,19("*"),A,25("*"))')
     &   ' GENERATION OF SPECIAL POINTS '
C     Parameter Q of Monkhorst and Pack, generalized for 3 axes B1,2,3
      WRITE(IOUT,'(A,/,1X,3I5)')
     &     ' MONKHORST-PACK PARAMETERS (GENERALIZED) IQ1,IQ2,IQ3:',
     &     IQ1,IQ2,IQ3
C     WVK0 is the shift of the whole mesh (see Macdonald)
      WRITE(IOUT,'(A,/,1X,3F10.5)')
     &     ' CONSTANT VECTOR SHIFT (MACDONALD) OF THIS MESH:',WVK0
      IF(IABS(IQ1) + IABS(IQ2) + IABS(IQ3) .EQ. 0) GOTO 710
      IF(ABS(ISTRIZ) .NE. 1) THEN
        WRITE(IOUT,'(" INVALID SWITCH FOR SYMMETRIZATION",I10)') ISTRIZ
        WRITE(6,'(" INVALID SWITCH FOR SYMMETRIZATION",I10)') ISTRIZ
        CALL STOPGM('K290','ISTRIZ WRONG ARGUMENT')
      ENDIF
      WRITE(IOUT,'(" SYMMETRIZATION SWITCH: ",I3,$)') ISTRIZ
      IF(ISTRIZ.EQ.1) THEN
        WRITE(IOUT,'(" (SYMMETRIZATION OF MONKHORST-PACK MESH)")')
      ELSE
        WRITE(IOUT,'(" (NO SYMMETRIZATION OF MONKHORST-PACK MESH)")')
      ENDIF
C     Set to 0.
      DO I=1, NKPOINT
        LWGHT(I) = 0
      ENDDO
C     ==--------------------------------------------------------------==
C     == Generation of the points (they are not multiplied            ==
c     == by 2*Pi because B1,2,3  were not,either)                     ==
C     ==--------------------------------------------------------------==
      IF(NC.GT.NC0) THEN
C       Due to non-use of primitive cell, the crystal has more
C       rotations than Bravais lattice.
C       We use only the rotations for Bravais lattices
        IF(NTVEC.EQ.1) THEN
          WRITE(6,*) 'NUMBER OF ROTATIONS FOR BRAVAIS LATTICE',NC0
          WRITE(6,*) 'NUMBER OF ROTATIONS FOR CRYSTAL LATTICE',NC
          WRITE(6,*) 'NO DUPLICATION FOUND'
          CALL STOPGM('ERROR',
     &         'SOMETHING IS WRONG IN GROUP DETERMINATION')
        ENDIF
        NC=NC0
        DO I=1,NC0
          IB(I)=IB0(I)
        ENDDO
        WRITE(IOUT,'(/,1X,20("!"),"WARNING",20("!"))')
        WRITE(IOUT,'(A)')
     &       ' THE CRYSTAL HAS MORE SYMMETRY THAN THE BRAVAIS LATTICE'
        WRITE(IOUT,'(A)')
     &       ' BECAUSE THIS IS NOT A PRIMITIVE CELL'
        WRITE(IOUT,'(A)')
     &       ' USE ONLY SYMMETRY FROM BRAVAIS LATTICE'
        WRITE(IOUT,'(1X,20("!"),"WARNING",20("!"),/)')
      ENDIF
      CALL SPPT2 (IOUT,IQ1,IQ2,IQ3,WVK0,NKPOINT,
     &     A01,A02,A03,B01,B02,B03,
     &     INVADD,NC,IB,R,NTOT,WVKL,LWGHT,LROT,NC0,IB0,ISTRIZ,
     &     NHASH,INCLUD,LIST,RLIST,DELTA)
C     ==--------------------------------------------------------------==
C     == Check on error signals                                       ==
C     ==--------------------------------------------------------------==
      WRITE(IOUT,'(/,1X,I5," SPECIAL POINTS GENERATED")') NTOT
      IF(NTOT .EQ. 0) THEN
        GOTO 710
      ELSE IF(NTOT .LT. 0) THEN
        WRITE(IOUT,'(A,I5,/,A,/,A)') ' DIMENSION NKPOINT =',NKPOINT,
     &       ' INSUFFICIENT FOR ACCOMMODATING ALL THE SPECIAL POINTS',
     &       ' WHAT FOLLOWS IS AN INCOMPLETE LIST'
        NTOT = IABS(NTOT)
      ENDIF
C     Before using the list WVKL as wave vectors, they have to be
C     multiplied by 2*Pi
C     The list of weights LWGHT is not normalized
      ISWGHT = 0
      DO I=1,NTOT
        ISWGHT = ISWGHT + LWGHT(I)
      ENDDO
      WRITE(IOUT,'(8X,A,T33,A,4X,A)')
     &     'WAVEVECTOR K','WEIGHT','UNFOLDING ROTATIONS'
C     Set near-zeroes equal to zero:
      DO L=1,NTOT
        DO I = 1,3
          IF(DABS(WVKL(I,L)) .LT. DELTA) WVKL(I,L) = 0.D0
        ENDDO
        IF(ISTRIN .NE. 0) THEN
C         Express special points in (unstrained) basis.
          PROJ1 = 0.D0
          PROJ2 = 0.D0
          PROJ3 = 0.D0
          DO I = 1,3
            PROJ1 = PROJ1 + WVKL(I,L)*A01(I)
            PROJ2 = PROJ2 + WVKL(I,L)*A02(I)
            PROJ3 = PROJ3 + WVKL(I,L)*A03(I)
          ENDDO
          DO I = 1,3
            WVKL(I,L) = PROJ1*B1(I) + PROJ2*B2(I) + PROJ3*B3(I)
          ENDDO
        ENDIF
        LMAX=LWGHT(L)
        WRITE(IOUT,FMT='(1X,I5,3F8.4,I8,T42,12I3)')
     &       L,(WVKL(I,L),I=1,3),LWGHT(L),(LROT(I,L),I=1,MIN(LMAX,12))
        DO J=13,LMAX,12
          WRITE(IOUT,FMT='(T42,12I3)')
     &       (LROT(I,L),I=J,MIN(LMAX,J-1+12))
        ENDDO
      ENDDO
      WRITE(IOUT,'(24X,"TOTAL:",I8)') ISWGHT
C     ==--------------------------------------------------------------==
 710  CONTINUE
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
