C     ==================================================================
      SUBROUTINE GROUP1(IOUT,A1,A2,A3,NAT,TY,X,B1,B2,B3,
     &                  IHG,IHC,ISY,LI,NC,INDPG,IB,NTVEC,
     &                  V,F0,R,TVEC,ORIGIN,RX,ISC,DELTA)
C     ==--------------------------------------------------------------==
C     == WRITTEN ON SEPTEMBER 10TH - FROM THE ACMI COMPLEX            ==
C     == (WORLTON AND WARREN, COMPUT.PHYS.COMMUN. 8,71-84 (1974))     ==
C     == (AND 3,88-117 (1972))                                        ==
C     == BASIC CRYSTALLOGRAPHIC INFORMATION                           ==
C     == ABOUT A GIVEN CRYSTAL STRUCTURE.                             ==
C     == SUBROUTINES NEEDED: PGL1,ATFTM1,ROT1,RLV3                    ==
C     ==--------------------------------------------------------------==
C     == INPUT DATA:                                                  ==
C     == IOUT ... NUMBER OF THE OUTPUT UNIT FOR ON-LINE PRINTING      ==
C     ==          OF VARIOUS MESSAGES                                 ==
C     ==          IF IOUT.LE.0 NO MESSAGE                             ==
C     == A1,A2,A3 .. ELEMENTARY TRANSLATIONS OF THE LATTICE, IN SOME  ==
C     ==          UNIT OF LENGTH                                      ==
C     == NAT .... NUMBER OF ATOMS IN THE UNIT CELL                    ==
C     ==          ALL THE DIMENSIONS ARE SET FOR NAT .LE. 20          ==
C     == TY ..... INTEGERS DISTINGUISHING BETWEEN THE ATOMS OF        ==
C     ==          DIFFERENT TYPE. TY(I) IS THE TYPE OF THE I-TH ATOM  ==
C     ==          OF THE BASIS                                        ==
C     == X ...... CARTESIAN COORDINATES OF THE NAT ATOMS OF THE BASIS ==
C     == DELTA... REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)           ==
C     ==--------------------------------------------------------------==
C     == OUTPUT DATA:                                                 ==
C     == B1,B2,B3 .. RECIPROCAL LATTICE VECTORS, NOT MULTIPLIED BY    ==
C     ==          ANY 2PI, IN UNITS RECIPROCAL TO THOSE OF A1,A2,A3   ==
C     == IHG .... POINT GROUP OF THE PRIMITIVE LATTICE, HOLOHEDRAL    ==
C     ==          GROUP NUMBER:                                       ==
C     ==          IHG=1 STANDS FOR TRICLINIC    SYSTEM                ==
C     ==          IHG=2 STANDS FOR MONOCLINIC   SYSTEM                ==
C     ==          IHG=3 STANDS FOR ORTHORHOMBIC SYSTEM                ==
C     ==          IHG=4 STANDS FOR TETRAGONAL   SYSTEM                ==
C     ==          IHG=5 STANDS FOR CUBIC        SYSTEM                ==
C     ==          IHG=6 STANDS FOR TRIGONAL     SYSTEM                ==
C     ==          IHG=7 STANDS FOR HEXAGONAL    SYSTEM                ==
C     == IHC .... CODE DISTINGUISHING BETWEEN HEXAGONAL AND CUBIC     ==
C     ==          GROUPS                                              ==
C     ==          IHC=0 STANDS FOR HEXAGONAL GROUPS                   ==
C     ==          IHC=1 STANDS FOR CUBIC GROUPS                       ==
C     == ISY .... CODE INDICATING WHETHER THE SPACE GROUP IS          ==
C     ==          SYMMORPHIC OR NONSYMMORPHIC                         ==
C     ==          ISY= 0 NONSYMMORPHIC GROUP                          ==
C     ==          ISY= 1 SYMMORPHIC GROUP                             ==
C     ==          ISY=-1 SYMMORPHIC GROUP WITH NON-STANDARD ORIGIN    ==
C     ==          ISY=-2 UNDETERMINED (NORMALLY NEVER)                ==
C     ==          THE GROUP IS CONSIDERED SYMMORPHIC IF FOR EACH      ==
C     ==          OPERATION OF THE POINT GROUP THE SUM OF THE 3       ==
C     ==          COMPONENTS OF ABS(V(N)) (NONPRIMITIVE TRANSLATION,  ==
C     ==          SEE BELOW) IS LT. 0.0001                            ==
C     == ORIGIN   STANDARD ORIGIN IF SYMMORPHIC (CRYSTAL COORDINATES) ==
C     == LI ..... CODE INDICATING WHETHER THE POINT GROUP             ==
C     ==          OF THE CRYSTAL CONTAINS INVERSION OR NOT            ==
C     ==          (OPERATIONS 13 OR 25 IN RESPECTIVELY HEXAGONAL      ==
C     ==          OR CUBIC GROUPS).                                   ==
C     ==          LI=0 MEANS: DOES NOT CONTAIN INVERSION              ==
C     ==          LI.GT.0 MEANS: THERE IS INVERSION IN THE POINT      ==
C     ==                         GROUP OF THE CRYSTAL                 ==
C     == NC ..... TOTAL NUMBER OF ELEMENTS IN THE POINT GROUP OF THE  ==
C     ==          CRYSTAL                                             ==
C     == INDPG .. POINT GROUP INDEX (DETERMINED IF SYMMORPHIC GROUP)  ==
C     == IB ..... LIST OF THE ROTATIONS CONSTITUTING THE POINT GROUP  ==
C     ==          OF THE CRYSTAL. THE NUMBERING IS THAT DEFINED IN    ==
C     ==          WORLTON AND WARREN, I.E. THE ONE MATERIALIZED IN THE==
C     ==          ARRAY R (SEE BELOW)                                 ==
C     ==          ONLY THE FIRST NC ELEMENTS OF THE ARRAY IB ARE      ==
C     ==          MEANINGFUL                                          ==
C     == NTVEC .. NUMBER OF TRANSLATIONAL VECTORS                     ==
C     ==          ASSOCIATED WITH IDENTITY OPERATOR I.E.              ==
C     ==          GIVES THE NUMBER OF IDENTICAL PRIMITIVE CELLS       ==
C     == V ...... NONPRIMITIVE TRANSLATIONS (IN THE CASE OF NONSYMMOR-==
C     ==          PHIC GROUPS). V(I,N) IS THE I-TH COMPONENT          ==
C     ==          OF THE TRANSLATION CONNECTED WITH THE N-TH ELEMENT  ==
C     ==          OF THE POINT GROUP (I.E. WITH THE ROTATION          ==
C     ==          NUMBER IB(N) ).                                     ==
C     ==          ATTENTION: V(I) ARE NOT CARTESIAN COMPONENTS,       ==
C     ==          THEY REFER TO THE SYSTEM A1,A2,A3.                  ==
C     == F0 ..... THE FUNCTION DEFINED IN MARADUDIN, IPATOVA BY       ==
C     ==          EQ. (3.2.12): ATOM TRANSFORMATION TABLE.            ==
C     ==          THE ELEMENT F0(N,KAPA) MEANS THAT THE N-TH          ==
C     ==          OPERATION OF THE SPACE GROUP (I.E. OPERATION NUMBER ==
C     ==          IB(N), TOGETHER WITH AN EVENTUAL NONPRIMITIVE       ==
C     ==          TRANSLATION  V(N)) TRANSFERS THE ATOM KAPA INTO THE ==
C     ==          ATOM F0(N,KAPA).                                    ==
C     ==          THE 49TH LINE GIVES EQUIVALENT ATOMS FOR            ==
C     ==          FRACTIONAl TRANSLATIONS ASSOCIATED WITH IDENTITY    ==
C     == R ...... LIST OF THE 3 X 3 ROTATION MATRICES                 ==
C     ==          (XYZ REPRESENTATION OF THE O(H) OR D(6)H GROUPS)    ==
C     ==          ALL 48 OR 24 MATRICES ARE LISTED.                   ==
C     ==          FOLLOW NOTATION OF WORLTON-WARREN(1972)             ==
C     == TVEC  .. LIST OF NTVEC TRANSLATIONAL VECTORS                 ==
C     ==          ASSOCIATED WITH IDENTITY OPERATOR                   ==
C     ==          TVEC(1:3,1) = \(0,0,0\)                             ==
C     ==          (CRYSTAL COORDINATES)                               ==
C     == RX ..... SCRATCH ARRAY                                       ==
C     == ISC .... SCRATCH ARRAY                                       ==
C     ==--------------------------------------------------------------==
C     == PRINTED OUTPUT:                                              ==
C     == PROGRAM PRINTS THE TYPE OF THE LATTICE (IHG, IN WORDS),      ==
C     == LISTS THE OPERATIONS OF THE  POINT GROUP OF THE              ==
C     == CRYSTAL, INDICATES WHETHER THE SPACE GROUP IS SYMMORPHIC OR  ==
C     == NONSYMMORPHIC AND WHETHER THE POINT GROUP OF THE CRYSTAL     ==
C     == CONTAINS INVERSION.                                          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER IOUT,NAT,IHG,IHC,ISY,LI,NC,INDPG,NTVEC,
     &        TY(NAT),IB(48),F0(49,NAT),ISC(NAT)
      REAL*8  A1(3),A2(3),A3(3),X(3,NAT),RX(3,NAT),TVEC(3,NAT),
     &        B1(3),B2(3),B3(3),ORIGIN(3),DELTA,
     &        V(3,48),R(3,3,48)
C     Variables
      REAL*8    A(3,3), AI(3,3),AP(3,3),API(3,3)
      INTEGER   I,NCPRIM
C     ==--------------------------------------------------------------==
      DO I = 1,3
        A(I,1) = A1(I)
        A(I,2) = A2(I)
        A(I,3) = A3(I)
      ENDDO
C     ==--------------------------------------------------------------==
C     == A(I,J) IS THE I-TH CARTESIAN COMPONENT OF THE J-TH PRIMITIVE ==
C     == TRANSLATION VECTOR OF THE DIRECT LATTICE                     ==
C     == TY(I) IS AN INTEGER DISTINGUISHING ATOMS OF DIFFERENT TYPE,  ==
C     == I.E., DIFFERENT ATOMIC SPECIES                               ==
C     == X(J,I) IS THE J-TH CARTESIAN COMPONENT OF THE POSITION       ==
C     == VECTOR FOR THE I-TH ATOM IN THE UNIT CELL.                   ==
C     ==--------------------------------------------------------------==
C     ==DETERMINE PRIMITIVE LATTICE VECTORS FOR THE RECIPROCAL LATTICE==
C     ==--------------------------------------------------------------==
      CALL CALBREC(A,AI)
      DO I = 1,3
        B1(I)  = AI(1,I)
        B2(I)  = AI(2,I)
        B3(I)  = AI(3,I)
      ENDDO
C     ==--------------------------------------------------------------==
C     Determination of the translation vectors associated with
C     the Identity matrix i.e. if the cell is duplicated
C     Give also the ``primitive lattice''
      CALL PRIMLATT(A,AI,AP,API,NAT,TY,X,NTVEC,TVEC,F0,ISC,DELTA)
C     ==--------------------------------------------------------------==
C     Determination of the holohedral group (and crystal system)
      CALL PGL1(AP,API,IHC,NC,IB,IHG,R,DELTA)
      IF(NTVEC.GT.1) THEN
C       All rotations found by PGL1 have axes in x, y or z cart. axis
C       So we have too check if we do not loose symmetry
        NCPRIM=NC
C       The hexagonal system is found if the z axis is the sixfold axis
        CALL PGL1(A,AI,IHC,NC,IB,IHG,R,DELTA)
        IF(NCPRIM.GT.NC) THEN
C         More symmetry with 
          CALL PGL1(AP,API,IHC,NC,IB,IHG,R,DELTA)
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
C     Determination of the space group
      CALL ATFTM1(IOUT,R,V,X,F0,ORIGIN,IB,TY,NAT,IHG,IHC,RX,
     &            NC,INDPG,NTVEC,A,AI,LI,ISY,ISC,DELTA)
C     ==--------------------------------------------------------------==
      IF (IOUT.GT.0) THEN
        IF(LI .GT. 0) THEN
          WRITE(IOUT,'(1X,A)')
     &         'THE POINT GROUP OF THE CRYSTAL CONTAINS THE INVERSION'
        ENDIF
        WRITE(IOUT,*)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CALBREC(A,AI)
C     ==--------------------------------------------------------------==
C     == CALCULATE RECIPROCAL VECTOR BASIS (AI(1:3,1:3))              ==
C     == INPUT:                                                       ==
C     ==   A(3,3) A(I,J) IS THE I-TH CARTESIAN COMPONENT              ==
C     ==          OF THE J-TH PRIMITIVE TRANSLATION VECTOR OF         ==
C     ==          THE DIRECT LATTICE                                  ==
C     == OUTPUT:                                                      ==
C     ==   AI(3,3) RECIPROCAL VECTOR BASIS                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8 A(3,3),AI(3,3)
C     Variables
      REAL*8  DET
      INTEGER I,IL,IU,J,JL,JU
C     ==--------------------------------------------------------------==
      DET = A(1,1)*A(2,2)*A(3,3) + A(2,1)*A(1,3)*A(3,2) +
     +      A(3,1)*A(1,2)*A(2,3) - A(1,1)*A(2,3)*A(3,2) -
     +      A(2,1)*A(1,2)*A(3,3) - A(3,1)*A(1,3)*A(2,2)
      DET = 1.D0/DET
      DO I = 1,3
        IL = 1
        IU = 3
        IF(I .EQ. 1) IL = 2
        IF(I .EQ. 3) IU = 2
        DO J = 1,3
          JL = 1
          JU = 3
          IF(J .EQ. 1) JL = 2
          IF(J .EQ. 3) JU = 2
          AI(J,I) = (-1.D0)**(I+J) * DET *
     +    ( A(IL,JL) * A(IU,JU) - A(IL,JU) * A(IU,JL) )
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PRIMLATT(A,AI,AP,API,NAT,TY,X,NTVEC,TVEC,F0,ISC,DELTA)
C     ==--------------------------------------------------------------==
C     == DETERMINATION OF THE TRANSLATION VECTORS ASSOCIATED WITH     ==
C     == THE IDENTITY SYMMETRY I.E. IF THE CELL IS DUPLICATED         ==
C     == GIVE ALSO THE PRIMITIVE DIRECT AND RECIPROCAL LATTICE VECTOR ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   A(3,3)   A(I,J) IS THE I-TH CARTESIAN COMPONENT            ==
C     ==            OF THE J-TH TRANSLATION VECTOR OF                 ==
C     ==            THE DIRECT LATTICE                                ==
C     ==   AI(3,3)  RECIPROCAL VECTOR BASIS (CARTESIAN)               ==
C     ==   NAT      NUMBER OF ATOMS                                   ==
C     ==   TY(NAT)  TYPE OF ATOMS                                     ==
C     ==   X(3,NAT) ATOMIC COORDINATES IN CARTESIAN COORDINATES       ==
C     == DELTA      REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)         ==
C     == OUTPUT:                                                      ==
C     ==   AP(3,3)  COMPONENTS OF THE PRIMITIVE TRANSLATION VECTORS   ==
C     ==   API(3,3) PRIMITIVE RECIPROCAL BASIS VECTORS                ==
C     ==            BOTH BAISI ARE IN CARTESIAN COORDINATES           ==
C     ==   NTVEC    NUMBER OF TRANSLATION VECTORS (FRACTIONNAL)       ==
C     ==   TVEC(3,NTVEC) COMPONENTS OF TRANSLATIONAL VECTORS          ==
C     ==                 (CRYSTAL COORDINATES)                        ==
C     ==   F0(49,NAT)    GIVES INEQUIVALENT ATOM FOR EACH ATOM        ==
C     ==                 THE 49-TH LINE                               ==
C     ==   ISC(NAT)      SCRATCH ARRAY                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER NAT,NTVEC,TY(NAT),F0(49,NAT),ISC(NAT)
      REAL*8 A(3,3),AI(3,3),AP(3,3),API(3,3),
     &       X(3,NAT),TVEC(3,NAT),DELTA
C     Variables
      INTEGER I,J,K2,IL,IV
      REAL*8  XB(3),VR(3)
      LOGICAL OKSYM
C     ==--------------------------------------------------------------==
C     First we check if there exist fractional translational vectors
C     associated with Identity operation i.e.
C     if the cell is duplicated or not.
      NTVEC=1
      TVEC(1,1)=0.D0
      TVEC(2,1)=0.D0
      TVEC(3,1)=0.D0
      DO I=1,NAT
        F0(49,I)=I
      ENDDO
      DO K2=2,NAT
        IF(TY(1) .NE. TY(K2)) GO TO 100
        DO I=1,3
          XB(I)=X(I,K2)-X(I,1)
        ENDDO
C       A fractional translation vector VR is defined.
        CALL RLV3(AI,XB,VR,IL,DELTA)
        CALL CHECKRLV3(1,NAT,TY,X,X,VR,F0,AI,ISC,.TRUE.,OKSYM,DELTA)
        IF(OKSYM) THEN
C         A fractional translational vector is found
          NTVEC=NTVEC+1
C         F0(49,1:NAT) gives number of equivalent atoms
C         and has atom indexes of inequivalent atoms (for translation)
          DO I=1,NAT
            IF(F0(49,I).GT.F0(1,I)) F0(49,I)=F0(1,I)
          ENDDO
          DO I=1,3
            TVEC(I,NTVEC)=VR(I)
          ENDDO
        ENDIF
 100    CONTINUE
      ENDDO
C     ==-------------------------------------------------------------==
      DO I=1,3
        AP(1,I)=A(1,I)
        AP(2,I)=A(2,I)
        AP(3,I)=A(3,I)
        API(1,I)=AI(1,I)
        API(2,I)=AI(2,I)
        API(3,I)=AI(3,I)
      ENDDO
      IF(NTVEC.EQ.1) THEN
C       The current cell is definitely a primitive one
C       Copy A and AI to AP and API
      ELSE
C       We are looking for the primitive lattice vector basis set
C       AP is our current lattice vector basis
        DO IV=2,NTVEC
C         TVEC in cartesian coordinates
          DO I=1,3
            XB(I)=TVEC(1,IV)*A(I,1)
     .           +TVEC(2,IV)*A(I,2)
     .           +TVEC(3,IV)*A(I,3)
          ENDDO
C         We calculare TVEC in AP basis
          CALL RLV3(API,XB,VR,IL,DELTA)
          DO I=1,3
            IF(DABS(VR(I)).GT.DELTA) THEN
              IL=NINT(1.D0/DABS(VR(I)))
              IF(IL.GT.1) THEN
C               We replace AP(1:3,I) by TVEC(1:3,IV)
                DO J=1,3
                  AP(J,I)=XB(J)
                ENDDO
C               Calculate new API
                CALL CALBREC(AP,API)
                GOTO 200
              ENDIF
            ENDIF
          ENDDO
 200      CONTINUE
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PGL1 (A,AI,IHC,NC,IB,IHG,R,DELTA)
C     ==--------------------------------------------------------------==
C     == WRITTEN ON SEPTEMBER 11TH, 1979 - FROM ACMI COMPLEX          ==
C     == AUXILIARY SUBROUTINE TO GROUP1                               ==
C     == SUBROUTINE PGL DETERMINES THE POINT GROUP OF THE LATTICE     ==
C     == AND THE CRYSTAL SYSTEM.                                      ==
C     == SUBROUTINES NEEDED: ROT1, RLV3                               ==
C     ==--------------------------------------------------------------==
C     == WARNING: FOR THE HEXAGONAL SYSTEM, THE 3RD AXIS SUPPOSE      ==
C     ==          TO BE THE SIX-FOLD AXIS                             ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     == A ..... DIRECT LATTICE VECTORS                               ==
C     == AI .... RECIPROCAL LATTICE VECTORS                           ==
C     == DELTA.. REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)            ==
C     ==--------------------------------------------------------------==
C     == OUTPUT:                                                      ==
C     == IHC .... CODE DISTINGUISHING BETWEEN HEXAGONAL AND CUBIC     ==
C     ==          GROUPS                                              ==
C     ==          IHC=0 STANDS FOR HEXAGONAL GROUPS                   ==
C     ==          IHC=1 STANDS FOR CUBIC GROUPS                       ==
C     == NC .... NUMBER OF ROTATIONS IN THE POINT GROUP               ==
C     == IB .... SET OF ROTATION                                      ==
C     == IHG .... POINT GROUP OF THE PRIMITIVE LATTICE, HOLOHEDRAL    ==
C     ==          GROUP NUMBER:                                       ==
C     ==          IHG=1 STANDS FOR TRICLINIC    SYSTEM                ==
C     ==          IHG=2 STANDS FOR MONOCLINIC   SYSTEM                ==
C     ==          IHG=3 STANDS FOR ORTHORHOMBIC SYSTEM                ==
C     ==          IHG=4 STANDS FOR TETRAGONAL   SYSTEM                ==
C     ==          IHG=5 STANDS FOR CUBIC        SYSTEM                ==
C     ==          IHG=6 STANDS FOR TRIGONAL     SYSTEM                ==
C     ==          IHG=7 STANDS FOR HEXAGONAL    SYSTEM                ==
C     == R ...... LIST OF THE 3 X 3 ROTATION MATRICES                 ==
C     ==          (XYZ REPRESENTATION OF THE O(H) OR D(6)H GROUPS)    ==
C     ==          ALL 48 OR 24 MATRICES ARE LISTED.                   ==
C     ==          FOLLOW NOTATION OF WORLTON-WARREN(1972)             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER IHC,NC,IHG,IB(48)
      REAL*8  A(3,3), AI(3,3),R(3,3,48),DELTA
C     ==--------------------------------------------------------------==
C     Variables
      REAL*8    VR(3),XA(3),TR
      INTEGER   NR,N,K,I,J,LX
C     ==--------------------------------------------------------------==
      DO IHC=0,1
C       IHC is 0 for hexagonal groups and 1 for cubic groups.
        IF(IHC.EQ.0) THEN
          NR = 24
        ELSE
          NR = 48
        ENDIF
        NC = 0
C       Constructs rotation operations.
        CALL ROT1(IHC,R)
        DO N = 1,NR
          IB(N) = 0
C         Rotate the A1,2,3 vectors by rotation No. N
          DO K = 1,3
            DO I = 1,3
              XA(I) = 0.D0
              DO J = 1,3
                XA(I) = XA(I) + R(I,J,N) * A(J,K)
              ENDDO
            ENDDO
            CALL RLV3(AI,XA,VR,LX,DELTA)
            TR = 0.D0
            DO I = 1,3
              TR = TR + DABS(VR(I))
            ENDDO
C           If VR.ne.0, then XA cannot be a multiple of a lattice vector
            IF(TR .GT. DELTA) GOTO 140
          ENDDO
          NC = NC + 1
          IB(NC) = N
 140      CONTINUE
        ENDDO
C       ==------------------------------------------------------------==
C       IHG stands for holohedral group number.
        IF(IHC .EQ. 0)  THEN
C         Hexagonal group:
          IF(NC  .EQ. 12) IHG = 6
          IF(NC  .GT. 12) IHG = 7
          IF(NC  .GE. 12) RETURN
C         Too few operations, try cubic group: (IHC=1,NR=48)
        ELSE
C         Cubic group:
          IF(NC  .LT. 4) IHG = 1
          IF(NC  .EQ. 4)  IHG = 2
          IF(NC  .GT. 4)  IHG = 3
          IF(NC  .EQ. 16) IHG = 4
          IF(NC  .GT. 16) IHG = 5
          RETURN
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE RLV3(AI,XB,VR,IL,DELTA)
C     ==--------------------------------------------------------------==
C     == WRITTEN ON SEPTEMBER 11TH, 1979 - FROM ACMI COMPLEX          ==
C     == AUXILIARY SUBROUTINE TO GROUP1                               ==
C     == SUBROUTINE RLV REMOVES A DIRECT LATTICE VECTOR               ==
C     == FROM XB LEAVING THE REMAINDER IN VR.                         ==
C     == IF A NONZERO LATTICE VECTOR WAS REMOVED, IL IS MADE NONZERO. ==
C     == VR STANDS FOR V-REFERENCE.                                   ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==    AI(I,J) ARE THE RECIPROCAL LATTICE VECTORS,               ==
C     ==            B(I) = AI(I,J),J=1,2,3                            ==
C     ==    XB(1:3) VECTOR IN CARTESIAN COORDINATES                   ==
C     == DELTA      REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)         ==
C     == OUTPUT:                                                      ==
C     ==    VR IS NOT GIVEN IN CARTESIAN COORDINATES BUT              ==
C     ==       IN THE SYSTEM A1,A2,A3 (CRYSTAL COORDINATES)           ==
C     ==       AND BETWEEN -1/2 AND 1/2                               ==
C     ==    IL ABS OF VR                                              ==
C     == K.K., 23.10.1979                                             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   IL
      REAL*8    AI(3,3),XB(3),VR(3),DELTA
C     Variables
      REAL*8    TS
      INTEGER   I,J
C     ==--------------------------------------------------------------==
      IL = 0
      DO I = 1,3
        VR(I) = 0.D0
      ENDDO
      TS = DABS(XB(1))+DABS(XB(2))+DABS(XB(3))
      IF(TS .LE. DELTA) RETURN
      DO I = 1,3
        VR(I) = VR(I) + AI(I,1)*XB(1)+AI(I,2)*XB(2)+AI(I,3)*XB(3)
        IL = IL + NINT(DABS(VR(I)))
C       Change in order to have correct determination of origin and
C       symmorphic group (T.D 30/03/98) 
C        VR(I) = - MOD(DBLE(VR(I)),1.D0)
        VR(I) = NINT(VR(I)) - VR(I)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE ATFTM1(IOUT,R,V,X,F0,ORIGIN,IB,TY,NAT,IHG,IHC,
     &                  RX,NC,INDPG,NTVEC,A,AI,LI,ISY,ISC,DELTA)
C     ==--------------------------------------------------------------==
C     == WRITTEN ON SEPTEMBER 11TH, 1979 - FROM ACMI COMPLEX          ==
C     == AUXILIARY SUBROUTINE TO GROUP1                               ==
C     == SUBROUTINE ATFTMT DETERMINES                                 ==
C     ==            THE POINT GROUP OF THE CRYSTAL,                   ==
C     ==            THE ATOM TRANSFORMATION TABLE,F0,                 ==
C     ==            THE FRACTIONAL TRANSLATIONS,V,                    ==
C     ==            ASSOCIATED WITH EACH ROTATION.                    ==
C     == SUBROUTINES NEEDED: RLV3 CHECKRLV3 SYMMORPHIC STOPGM XSTRING ==
C     == MAY 14TH,1998: A LOT OF CHANGES (ARGUMENTS)                  ==
C     ==                BETTER DETERMINATION OF V                     ==
C     == SEP 15TH,1998: DETERMINATION OF FRACTIONAL TRANSLATIONAL VEC.==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   IOUT Logical file number (output)                          ==
C     ==        If IOUT.LE.0 no message                               ==
C     ==   IHG  Holohedral group number (determined by PGL1)          ==
C     ==   IHC  Code distinguishing between hexagonal and cubic groups==
C     ==          IHC=0 stands for hexagonal groups                   ==
C     ==          IHC=1 stands for cubic groups                       ==
C     ==   NC   Number of rotation operations                         ==
C     ==   NAT  Number of atoms (used in the routine)                 ==
C     ==   X    Coordinates of atoms (cartesian)                      ==
C     ==   TY   Type of atoms                                         ==
C     ==   R    Sets of transformation operations (cartesian)         ==
C     ==   IB   Index giving NC operations in R                       ==
C     ==   AI   Reciprocal lattice vectors                            ==
C     ==   NTVEC Number of translational vectors                      ==
C     ==        associated with Identity                              ==
C     ==        if primitive cell NTVEC=1, TVEC=(0,0,0)               ==
C     == DELTA  REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)             ==
C     == OUTPUT:                                                      ==
C     ==   RX(3,NAT) Scratch array                                    ==
C     ==   ISC(NAT)  Scratch array                                    ==
C     ==   NC        is modified (number of symmetry operations)      ==
C     ==   INDPG     Point group index                                ==
C     ==   V(3,48)   The fractional translations associated           ==
C     ==             with each rotation (crystal coordinates)         ==
C     ==   F0(1:48,NAT)                                               ==
C     ==        The atom transformation table for rotation (48,NAT)   ==
C     ==   ORIGIN Standard origin if symmorphic (crystal coordinates) ==
C     ==   ISY  = 1 Isommorphic group                                 ==
C     ==        =-1 Isommorphic group with non-standard origin        ==
C     ==        = 0 Non-Isommorphic group                             ==
C     ==        =-2 Undetermined (normally never)                     ==
C     == LI ..... Code indicating whether the point group             ==
C     ==          of the crystal contains inversion or not            ==
C     ==          (operations 13 or 25 in respectively hexagonal      ==
C     ==          or cubic groups).                                   ==
C     ==          LI=0    : does not contain inversion                ==
C     ==          LI.GT.0 : there is inversion in the point           ==
C     ==                    group of the crystal                      ==
C     ==--------------------------------------------------------------==
C     INDPG group   indpg   group    indpg  group     indpg   group   ==
C     == 1    1 (c1)    9    3m (c3v)   17  4/mmm(d4h)  25   222(d2)  ==
C     == 2   <1>(ci)   10   <3>m(d3d)   18     6 (c6)   26   mm2(c2v) ==
C     == 3    2 (c2)   11     4 (c4)    19    <6>(c3h)  27   mmm(d2h) ==
C     == 4    m (c1h)  12    <4>(s4)    20    6/m(c6h)  28   23 (t)   ==
C     == 5   2/m(c2h)  13    4/m(c4h)   21    622(d6)   29   m3 (th)  ==
C     == 6    3 (c3)   14    422(d4)    22    6mm(c6v)  30   432(o)   ==
C     == 7   <3>(c3i)  15    4mm(c4v)   23  <6>m2(d3h)  31 <4>3m(td)  ==
C     == 8   32 (d3)   16  <4>2m(d2d)   24  6/mmm(d6h)  32   m3m(oh)  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER IOUT,NAT,IHG,IHC,NC,INDPG,NTVEC,ISY,
     &        IB(48),F0(49,NAT),TY(NAT),ISC(NAT)
      REAL*8  R(3,3,48),V(3,48),X(3,NAT),RX(3,NAT),
     &        A(3,3),AI(3,3),DELTA
C     Variables
      INTEGER   IIS(48),
     &          NCA,N,L,K,I,J,K2,Il,NI,LI,INFO
      REAL*8    VR(3),ORIGIN(3),XB(3),VC(3,48),
     &          VS
      LOGICAL   OKSYM,NODUPLI
      CHARACTER ICST(7)*12,PGRP(32)*5,PGRD(32)*3,
     &          RNAME_CUBIC(48)*10, RNAME_HEXA(24)*11
C     ==--------------------------------------------------------------==
      DATA      ICST
     &         /'TRICLINIC','MONOCLINIC','ORTHORHOMBIC',
     &          'TETRAGONAL','CUBIC','TRIGONAL','HEXAGONAL'/
C     ==--------------------------------------------------------------==
      DATA PGRP /'    1','  <1>','    2','    m',
     *           '  2/m','    3','  <3>','   32',
     *           '   3m',' <3>m','    4','  <4>',
     *           '  4/m','  422','  4mm','<4>2m',
     *           '4/mmm','    6','  <6>','  6/m',
     *           '  622','  6mm','<6>m2','6/mmm',
     *           '  222','  mm2','  mmm','   23',
     *           '   m3','  432','<4>3m','  m3m'/
C     ==--------------------------------------------------------------==
      DATA PGRD /'c1 ','ci ','c2 ','c1h',
     *           'c2h','c3 ','c3i','d3 ',
     *           'c3v','d3 ','c4 ','s4 ',
     *           'c4h','d4 ','c4v','d2d',
     *           'd4h','c6 ','c3h','c6h',
     *           'd6 ','c6v','d3h','d6h',
     *           'd2 ','c2v','d2h','t  ',
     *           'th ','o  ','td ','oh '/
C     ==--------------------------------------------------------------==
C     Name of 48 rotations (convention Warren-Worlton)
      DATA      RNAME_CUBIC
     &    /' 1        ',' 2[ 1 0 0]',' 2[ 0 1 0]',' 2[ 0 0 1]',
     &     ' 3[-1-1-1]',' 3[ 1 1-1]',' 3[-1 1 1]',' 3[ 1-1 1]',
     &     ' 3[ 1 1 1]',' 3[-1 1-1]',' 3[-1-1 1]',' 3[ 1-1-1]',
     &     ' 2[-1 1 0]',' 4[ 0 0 1]',' 4[ 0 0-1]',' 2[ 1 1 0]',
     &     ' 2[ 0-1 1]',' 2[ 0 1 1]',' 4[ 1 0 0]',' 4[-1 0 0]',
     &     ' 2[-1 0 1]',' 4[ 0-1 0]',' 2[ 1 0 1]',' 4[ 0 1 0]',
     &     '-1        ','-2[ 1 0 0]','-2[ 0 1 0]','-2[ 0 0 1]',
     &     '-3[-1-1-1]','-3[ 1 1-1]','-3[-1 1 1]','-3[ 1-1 1]',
     &     '-3[ 1 1 1]','-3[-1 1-1]','-3[-1-1 1]','-3[ 1-1-1]',
     &     '-2[-1 1 0]','-4[ 0 0 1]','-4[ 0 0-1]','-2[ 1 1 0]',
     &     '-2[ 0-1 1]','-2[ 0 1 1]','-4[ 1 0 0]','-4[-1 0 0]',
     &     '-2[-1 0 1]','-4[ 0-1 0]','-2[ 1 0 1]','-4[ 0 1 0]'/
      DATA      RNAME_HEXA
     &    /' 1         ',' 6[ 0 0  1]',' 3[ 0 0  1]',' 2[ 0 0  1]',
     &     ' 3[ 0 0 -1]',' 6[ 0 0 -1]',' 2[ 0 1  0]',' 2[-1 1  0]',
     &     ' 2[ 1 0  0]',' 2[ 2 1  0]',' 2[ 1 1  0]',' 2[ 1 2  0]',
     &     '-1         ','-6[ 0 0  1]','-3[ 0 0  1]','-2[ 0 0  1]',
     &     '-3[ 0 0 -1]','-6[ 0 0 -1]','-2[ 0 1  0]','-2[-1 1  0]',
     &     '-2[ 1 0  0]','-2[ 2 1  0]','-2[ 1 1  0]','-2[ 1 2  0]'/
C     ==--------------------------------------------------------------==
      NODUPLI=NTVEC.EQ.1
      NCA = 0
      DO N = 1,48
        IIS(N) = 0
      ENDDO
C     Calculate translational vector for each operation
C     and atom transformation table.
      DO N = 1,NC
        L = IB(N)
        IIS(L) = 1
        DO K = 1,NAT
          DO I = 1,3
            RX(I,K)=R(I,1,L)*X(1,K)+R(I,2,L)*X(2,K)+R(I,3,L)*X(3,K)
          ENDDO
        ENDDO
        DO K=1,3
          VR(K)=0.D0
        ENDDO
C       First we determine for VR=(/0,0,0/)
C       IMPORTANT IF NOT UNIQUE ATOMS FOR DETERMINATION OF SYMMORPHIC
        CALL CHECKRLV3(N,NAT,TY,RX,X,VR,F0,AI,ISC,NODUPLI,OKSYM,DELTA)
        IF(OKSYM) THEN
          GOTO 190
        ENDIF
C       Now we try other possible VR
C       F0(49,1:NAT) has only inequivalent atom indexes for translation
        DO K2 = 1,NAT
          IF(F0(49,K2).LT.K2) GOTO 185
          IF(TY(1) .NE. TY(K2)) GO TO 185
          DO I=1,3
            XB(I)=RX(I,1)-X(I,K2)
          ENDDO
C         A translation vector VR is defined.
          CALL RLV3(AI,XB,VR,IL,DELTA)
C         ==----------------------------------------------------------==
C         == SUBROUTINE RLV3 REMOVES A DIRECT LATTICE VECTOR FROM XB  ==
C         == LEAVING THE REMAINDER IN VR. IF A NONZERO LATTICE        ==
C         == VECTOR WAS REMOVED, IL IS MADE NONZERO.                  ==
C         == VR STANDS FOR V-REFERENCE.                               ==
C         == VR IS NOT GIVEN IN CARTESIAN COORDINATES BUT             ==
C         == IN THE SYSTEM A1,A2,A3.     K.K., 23.10.1979             ==
C         ==----------------------------------------------------------==
          CALL CHECKRLV3(N,NAT,TY,RX,X,VR,F0,AI,ISC,NODUPLI,OKSYM,DELTA)
          IF(OKSYM) THEN
            GOTO 190
          ENDIF
 185      CONTINUE
        ENDDO
        IIS(L) = 0
        GO TO 210
 190    CONTINUE
        NCA = NCA + 1
        DO I = 1,3
          V(I,NCA)=VR(I)
        ENDDO
C       ==------------------------------------------------------------==
C       == V(I,N) IS THE I-TH COMPONENT OF THE FRACTIONAL             ==
C       == TRANSLATION ASSOCIATED WITH THE ROTATION N.                ==
C       == ATTENTION: V(I) ARE NOT CARTESIAN COMPONENTS, THEY ARE     ==
C       == GIVEN IN THE SYSTEM A1,A2,A3.                              ==
C       == K.K., 23. 10. 1979                                         ==
C       ==------------------------------------------------------------==
 210    CONTINUE
      ENDDO
C     Remove unused operations
      I  = 0
      NI = 13
      IF(IHG .LT. 6) NI = 25
      LI = 0
      DO N = 1,NC
        L = IB(N)
        IF(IIS(L) .EQ. 0) GO TO 230
        I = I + 1
        IB(I) = IB(N)
        IF(IB(I) .EQ. NI) LI = I
        DO K = 1,NAT
          F0(I,K) = F0(N,K)
        ENDDO
 230    CONTINUE
      ENDDO
C     ==--------------------------------------------------------------==
      NC = I
      VS = 0.D0
      DO N = 1,NC
        VS = VS + DABS(V(1,N))+DABS(V(2,N))+DABS(V(3,N)) 
      ENDDO
C     THE ORIGINAL VALUE DELTA=0.0001 WAS MODIFIED
C     BY K.K. , SEPTEMBER 1979 TO 0.0005
C     AND RETURNED TO 0.0001 BY RJN OCT 1987
      IF(VS .GT. DELTA) THEN
        ISY = 0
      ELSE 
        ISY = 1
      ENDIF 
C     ==--------------------------------------------------------------==
C     Determination of the point group
C     (Thierry Deutsch - 1998 [Maybe not complete!!])
      IF(IHG.LT.6) THEN
        IF(NC.EQ.0) THEN
          WRITE(6,'(" ATFTM1! IHG=",A," NC=",I2)') ICST(IHG),NC
          CALL STOPGM('ATFTM1','NUMBER OF ROTATION NULL')
C       Triclinic system
        ELSEIF(NC.EQ.1) THEN
C         IB=1
          INDPG=1               !1 (c1)
        ELSEIF(NC.EQ.2.AND.IB(2).EQ.25) THEN
C         IB=1 25
          INDPG=2               !<1>(ci)
        ELSEIF(NC.EQ.2.AND.(
     &         IB(2).EQ.4.OR.   !2[001]
     &         IB(2).EQ.2.OR.   !2[100]
     &         IB(2).EQ.3) ) THEN !2[010]
C         Monoclinic system
C         IB=1 4 (z-axis) OR
C         IB=1 2 (x-axis) OR
C         IB=1 3 (y-axis)
          INDPG=3               !2 (c2)
        ELSEIF(NC.EQ.2.AND.(
     &         IB(2).EQ.28.OR.
     &         IB(2).EQ.26.OR.
     &         IB(2).EQ.27) ) THEN
C         IB=1 28 (z-axis) OR
C         IB=1 26 (x-axis) OR
C         IB=1 27 (y-axis)
          INDPG=4               !m (c1h)
        ELSEIF(NC.EQ.4.AND.(
     &         IB(4).EQ.28.OR.  !2[001]
     &         IB(4).EQ.27.OR.  !2[010]
     &         IB(4).EQ.26.OR.  !2[100]
     &         IB(4).EQ.37.OR.  !-2[-110]
     &         IB(4).EQ.40) ) THEN !2[110]
C         IB=1  4 25 28 (z-axis)  OR
C         IB=1  2 25 26 (x-axis)  OR
C         IB=1  3 25 27 (y-axis)  OR
C         IB=1 13 25 37 (-xy-axis)OR
C         IB=1 16 25 40 (xy-axis)
          INDPG=5               !2/m(c2h)
        ELSEIF(NC.EQ.4.AND.(
     &         IB(4).EQ.15.OR.
     &         IB(4).EQ.20.OR.
     &         IB(4).EQ.24) ) THEN
C         Tetragonal system
C         IB=1 4 14 15 (z-axis) OR
C         IB=1 2 19 20 (x-axis) OR
C         IB=1 3 22 24 (y-axis)
          INDPG=11              !4 (c4)
        ELSEIF(NC.EQ.4.AND.(
     &         IB(4).EQ.39.OR.
     &         IB(4).EQ.44.OR.
     &         IB(4).EQ.48) ) THEN
C         IB=1 4 38 39 (z-axis) OR
C         IB=1 2 43 44 (x-axis) OR
C         IB=1 3 46 48 (y-axis)
          INDPG=12              !<4>(s4)
        ELSEIF(NC.EQ.8.AND.(
     &         (IB(3).EQ.14.AND.IB(8).EQ.39).OR.
     &         (IB(3).EQ.19.AND.IB(8).EQ.44).OR.
     &         (IB(3).EQ.22.AND.IB(8).EQ.48)) ) THEN
C         IB=1 4 14 15 28 25 38 39 (z-axis) OR
C         IB=1 2 19 20 26 25 43 44 (x-axis) OR
C         IB=1 3 22 24 27 25 46 48 (y-axis)
          INDPG=13              !422(d4)
        ELSEIF(NC.EQ.8.AND.IB(4).EQ.4.AND.(
     &         IB(8).EQ.16.OR.
     &         IB(8).EQ.20.OR.
     &         IB(8).EQ.24) ) THEN
C         IB=1 2 3 4 13 14 15 16 (z-axis) OR
C         IB=1 2 3 4 17 19 20 18 (x-axis) OR
C         IB=1 2 3 4 21 22 24 23 (y-axis)
          INDPG=14              !4/m(c4h)
        ELSEIF(NC.EQ.8.AND.(
     &         IB(8).EQ.40.OR.
     &         IB(8).EQ.42.OR.
     &         IB(8).EQ.47) ) THEN
C         IB=1 4 14 15 26 27 37 40 (z-axis) OR
C         IB=1 2 19 20 28 27 41 42 (x-axis) OR
C         IB=1 3 22 24 26 28 45 47 (y-axis)
          INDPG=15              !4mm(c4v)
        ELSEIF(NC.EQ.8.AND.(
     &         (IB(3).EQ.13.AND.IB(8).EQ.39).OR.
     &         (IB(3).EQ.17.AND.IB(8).EQ.44).OR.
     &         (IB(3).EQ.21.AND.IB(8).EQ.48)) ) THEN
C         IB=1 4 13 16 26 27 38 39 (z-axis) OR
C         IB=1 2 17 18 28 27 43 44 (x-axis) OR
C         IB=1 3 21 23 26 28 46 48 (y-axis)
          INDPG=16              !<4>2m(d2d)
        ELSEIF(NC.EQ.16.AND.(
     &         IB(16).EQ.40.OR.
     &         IB(16).EQ.44.OR.
     &         IB(16).EQ.48) )THEN
C         IB=1 2 3 4 13 14 15 16 25 26 27 28 37 38 39 40 (z-axis) OR
C         IB=1 2 3 4 17 19 20 18 25 26 27 28 41 43 44 42 (x-axis) OR
c         IB=1 2 3 4 21 22 24 23 25 26 27 28 45 46 48 47 (y-axis)
          INDPG=17              !4/mmm(d4h)
        ELSEIF(NC.EQ.4.AND.(IB(4).EQ.4) )THEN
C         Orthorhombic system
C         IB=1 2  3  4
          INDPG=25              !222(d2)
        ELSEIF(NC.EQ.4.AND.(
     &         IB(4).EQ.27.OR.
     &         IB(4).EQ.28) ) THEN
C         IB=1 3 26 27 (z-axis) OR
C         IB=1 2 27 28 (x-axis) OR
C         IB=1 4 26 28 (y-axis) OR
          INDPG=26              !mm2(c2v)
        ELSEIF(NC.EQ.8) THEN
C         IB=1 2 3 4 25 26 27 28
          INDPG=27              !mmm(d2h)
        ELSEIF(NC.EQ.12.AND.(
     &         IB(12).EQ.12.OR.
     &         IB(12).EQ.47.OR.
     &         IB(12).EQ.45) ) THEN
C         Cubic system
C         IB=1 2  3  4  5  6  7  8  9 10 11 12 OR
C         IB=1 5 11 13 18 23 25 30 35 37 42 47 OR
C         IB=1 8 10 16 18 21 25 32 34 40 42 45
          INDPG=28              !23 (t)
        ELSEIF(NC.EQ.24.AND.IB(24).EQ.36) THEN
c         IB= 1  2  3  4  5  6  7  8  9 10 11 12
C            25 26 27 28 29 30 31 32 33 34 35 36
          INDPG=29              !m3 (th)
        ELSEIF(NC.EQ.24.AND.IB(24).EQ.24) THEN
C         IB=1 2 3 4 5 6 7 8 9 10 11 12
C           13 14 15 16 17 18 19 20 21 22 23 24
          INDPG=30              !432 (o)
        ELSEIF(NC.EQ.24.AND.IB(24).EQ.48) THEN
C         IB=1 2 3 4 5 6 7 8 9 10 11 12
C           37 38 39 40 41 42 43 45 46 47 48
          INDPG=31              !<4>3m(td)
        ELSEIF(NC.EQ.48) THEN
C         IB=1..48
          INDPG=32              !m3m(oh)
        ELSE
C          WRITE(6,'(" ATFTM1! IHG=",A," NC=",I2)') ICST(IHG),NC
C          WRITE(6,'(" ATFTM1!",19I3)') (IB(I),I=1,NC)
C          WRITE(6,'(" ATFTM1! THIS CASE IS UNKNOWN IN THE DATABASE")')
C         Probably a sub-group of 32
          INDPG=-32
        ENDIF
      ELSEIF(IHG.GE.6) THEN
        IF(NC.EQ.0) THEN
          WRITE(6,'(" ATFTM1! IHG=",A," NC=",I2)') ICST(IHG),NC
          CALL STOPGM('ATFTM1','NUMBER OF ROTATION NULL')
C       Triclinic system
        ELSEIF(NC.EQ.1) THEN
C         IB=1
          INDPG=1               !1 (c1)
        ELSEIF(NC.EQ.2.AND.IB(2).EQ.13) THEN
C         IB=1 13
          INDPG=2               !<1>(ci)
        ELSEIF(NC.EQ.2.AND.(
     &         IB(2).EQ.4) ) THEN !2[001]
C         Monoclinic system
C         IB=1 4
          INDPG=3               !2 (c2)
        ELSEIF(NC.EQ.2.AND.(
     &         IB(2).EQ.16) ) THEN
C         IB=1 16
          INDPG=4               !m (c1h)
        ELSEIF(NC.EQ.4.AND.(
     &         IB(4).EQ.24.OR.
     &         IB(4).EQ.20) ) THEN 
C         IB=1 12 13 24 OR
C         IB=1  8 13 20
          INDPG=5               !2/m(c2h)
        ELSEIF(NC.EQ.3.AND.IB(3).EQ.5) THEN
C         Trigonal system
C         IB=1 3 5
          INDPG=6               !3 (c3)
        ELSEIF(NC.EQ.6.AND.IB(6).EQ.17) THEN
C         IB=1 13 15 17 35
          INDPG=7               !<3>(c3i)
        ELSEIF(NC.EQ.6.AND.IB(6).EQ.11) THEN
C         IB=1 7 9 11 35
          INDPG=8               !32 (d3)
        ELSEIF(NC.EQ.6.AND.IB(6).EQ.23) THEN
C         IB=1 3 5 19 21 23
          INDPG=9               !3m (c3v)
        ELSEIF(NC.EQ.12.AND.IB(12).EQ.23) THEN
C         IB=1 3 5 7 9 11 13 15 17 19 21 23
          INDPG=10              !<3>m(d3d)
        ELSEIF(NC.EQ.6.AND.IB(6).EQ.6) THEN
C         Hexagonal system
C         IB=1 2 3 4 5 6
          INDPG=18              !6 (c6)
        ELSEIF(NC.EQ.6.AND.IB(6).EQ.18) THEN
C         IB=1 3 5 14 16 18
          INDPG=19              !<6>(c3h)
        ELSEIF(NC.EQ.12.AND.IB(12).EQ.18) THEN
C         IB=1 2 3 4 5 6 13 14 15 16 17 18
          INDPG=20              !6/m(c6h)
        ELSEIF(NC.EQ.12.AND.IB(12).EQ.12) THEN
C         IB=1 2 3 4 5 6 7 8 9 10 11 12
          INDPG=21              !622(d6)
        ELSEIF(NC.EQ.12.AND.IB(2).EQ.2.AND.IB(12).EQ.24) THEN
C         IB=1 2 3 4 5 6 19 20 21 22 23 24
          INDPG=22              !6mm(c6v)
        ELSEIF(NC.EQ.12.AND.IB(2).EQ.3.AND.IB(12).EQ.24) THEN
C         IB=1 3 5 7 9 11 14 16 18 20 22 24
          INDPG=23              !<6>m2(d3h)
        ELSEIF(NC.EQ.24) THEN
C         IB=1..24
          INDPG=24              !6/mmm(d6h)
        ELSE
C         Probably a sub-group of 24
C          WRITE(6,'(" ATFTM1! IHG=",A," NC=",I2)') ICST(IHG),NC
C          WRITE(6,'(" ATFTM1!",48I3)') (IB(I),I=1,NC)
C          WRITE(6,'(" ATFTM1! THIS CASE IS UNKNOWN IN THE DATABASE")')
          INDPG=-24
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
C     == Determination if the space group is symmorphic or not        ==
C     ==--------------------------------------------------------------==
      IF(ISY.NE.1) THEN
C       Transform V in cartesian coordinates
        DO N=1,NC
          VC(1,N)=A(1,1)*V(1,N)+A(1,2)*V(2,N)+A(1,3)*V(3,N)
          VC(2,N)=A(2,1)*V(1,N)+A(2,2)*V(2,N)+A(2,3)*V(3,N)
          VC(3,N)=A(3,1)*V(1,N)+A(3,2)*V(2,N)+A(3,3)*V(3,N)
        ENDDO
        CALL SYMMORPHIC(NC,IB,R,VC,AI,INFO,ORIGIN,DELTA)
        IF(INFO.EQ.1) THEN
          CALL RLV3(AI,ORIGIN,XB,IL,DELTA)
C         !!!RLV3 determines -XB in crystal coordinates
C         !!We want between 0.0 and 1.0
          DO I=1,3
            IF(-XB(I).GE.0.D0) THEN
              ORIGIN(I)=-XB(I)
            ELSE
              ORIGIN(I)=1.D0-XB(I)
            ENDIF
          ENDDO
          DO I=1,3
            XB(I)=A(I,1)*ORIGIN(1)+A(I,2)*ORIGIN(2)+A(I,3)*ORIGIN(3)
          ENDDO
          ISY=-1
        ELSEIF(INFO.EQ.0) THEN
          ISY=0
        ELSE
          ISY=-2
        ENDIF
      ELSE
        DO I=1,3
          ORIGIN(I)=0.D0
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     == Output                                                       ==
C     ==--------------------------------------------------------------==
      IF(IOUT.GT.0) THEN
        WRITE(IOUT,*)
        CALL XSTRING(ICST(IHG),I,J)
        IF( (IHG .EQ. 7 .AND. NC .EQ. 24).OR.
     &       (IHG .EQ. 5 .AND. NC .EQ. 48) ) THEN
          WRITE(IOUT,'(A,A,A)')
     &         ' THE POINT GROUP OF THE CRYSTAL IS THE FULL ',
     &         ICST(IHG)(I:J),
     &         ' GROUP'
        ELSE
          WRITE(IOUT,'(A,A,A,I2,A)')
     &         ' THE CRYSTAL SYSTEM IS ',
     &         ICST(IHG)(I:J),
     &         ' WITH ',NC,' OPERATIONS:'
          IF(IHC.EQ.0) THEN
            WRITE(IOUT,'( 5(5(A13),/))') (RNAME_HEXA(IB(I)),I=1,NC)
          ELSE
            WRITE(IOUT,'(10(5(A13),/))') (RNAME_CUBIC(IB(I)),I=1,NC)
          ENDIF
        ENDIF
C       ==------------------------------------------------------------==
        IF(ISY.EQ.1) THEN
          WRITE(IOUT,'(A)')
     &         ' THE SPACE GROUP OF THE CRYSTAL IS SYMMORPHIC'
        ELSEIF(ISY.EQ.-1) THEN
          WRITE(IOUT,'(A)')
     &         ' THE SPACE GROUP OF THE CRYSTAL IS SYMMORPHIC'
          WRITE(IOUT,'(A,A,/,T3,3F10.6,3X,3F10.6)')
     &         ' THE STANDARD ORIGIN OF COORDINATES IS:   ',
     &         '[CARTESIAN]   [CRYSTAL]',XB,ORIGIN
        ELSEIF(ISY.EQ.0) THEN
          WRITE(IOUT,'(A,/,3X,A,F15.6,A)')
     &         ' THE SPACE GROUP IS NON-SYMMORPHIC,',
     &         ' (SUM OF TRANSLATION VECTORS=',VS,')'
        ELSEIF(ISY.EQ.-2) THEN
          WRITE(IOUT,'(A,A)')
     &         ' ATFTM1| CANNOT DETERMINE IF THE SPACE GROUP IS',
     &         ' SYMMORPHIC OR NOT'
          WRITE(IOUT,'(A,/,A,A,/,3X,A,F15.6,A)')
     &         ' THE SPACE GROUP IS NON-SYMMORPHIC,',
     &         ' OR ELSE A NON STANDARD ORIGIN OF COORDINATES WAS',
     &         ' USED.',
     &         ' (SUM OF TRANSLATION VECTORS=',VS,')'
        ENDIF
        IF(INDPG.GT.0) THEN
          CALL XSTRING(PGRP(INDPG),I,J)
          CALL XSTRING(PGRD(INDPG),K,L)
          WRITE(IOUT,'(A,A,"(",A,")",T56,"[INDEX=",I2,"]")')
     &         ' THE POINT GROUP OF THE CRYSTAL IS  ',PGRP(INDPG)(I:J),
     &         PGRD(INDPG)(K:L),INDPG
        ELSE
          CALL XSTRING(PGRP(-INDPG),I,J)
          CALL XSTRING(PGRD(-INDPG),K,L)
          WRITE(IOUT,'(A,I2,A,A,"(",A,")",T56,"[INDEX=",I2,"]")')
     &         ' POINT GROUP: GROUP ORDER=',NC,
     &         '    SUBGROUP OF ',PGRP(-INDPG)(I:J),
     &         PGRD(-INDPG)(K:L),-INDPG
        ENDIF
        IF(NTVEC.EQ.1) THEN
          WRITE(IOUT,'(A,T60,I6)')
     &         ' NUMBER OF PRIMITIVE CELL:',NTVEC
        ELSE
          WRITE(IOUT,'(A,T60,I6)')
     &         ' NUMBER OF PRIMITIVE CELLS:',NTVEC
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CHECKRLV3(N,NAT,TY,RX,X,VR,F0,AI,ISC,
     &                     NODUPLI,OKSYM,DELTA)
C     ==--------------------------------------------------------------==
C     == WRITTEN IN MAY 14TH, 1998 (T.D.)                             ==
C     == CHECK IF RX+VR GIVES THE SAME LATTICE AS X                   ==
C     == BUILD THE ATOM TRANSFORMATION TABLE                          ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   N   ROTATION NUMBER (INDEX USED IN F0 BETWEEN 1 AND 48)    ==
C     ==   NAT           NUMBER OF ATOMS                              ==
C     ==   TY(1:NAT)     TYPE OF ATOMS                                ==
C     ==   RX(1:3,1:NAT) ATOMIC COORDINATES FROM Nth ROTATION (CART.) ==
C     ==   X(1:3,1:NAT)  ATOMIC COORDINATES (CARTESIAN)               ==
C     ==   VR(1:3)       TRANSLATION VECTOR (CRYSTAL COOR.)           ==
C     ==   AI(1:3,1:3)   LATTICE RECIPROCAL VECTORS                   ==
C     ==   NODUPLI       .TRUE., THE CELL IS A PRIMITIVE ONE          ==
C     ==                 WE CAN SPEED UP                              ==
C     == DELTA           REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)    ==
C     == OUTPUT:                                                      ==
C     ==   F0(1:49,1:NAT) ATOM TRANSFORMATION TABLE                   ==
C     ==      F0 IS THE FUNCTION DEFINED IN MARADUDIN AND VOSK0       ==
C     ==      BY EQ.(2.35).                                           ==
C     ==      IT DEFINES THE ATOM TRANSFORMATION TABLE                ==
C     ==   OKSYM          TRUE IF RX+VR = X                           ==
C     ==   ISC(1:NAT)     SCRATCH ARRAY                               ==
C     ==                  USED TO SPEED UP THE ROUTINE                ==
C     ==                  EACH ATOM IS ONLY ONCE AN IMAGE             ==
C     ==                  IF NO DUPLICATION OF THE CELL               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   N,NAT,TY(NAT),F0(49,NAT),ISC(NAT)
      REAL*8    RX(3,NAT),X(3,NAT),VR(3),AI(3,3),DELTA
      LOGICAL   OKSYM,NODUPLI
C     Variables
      REAL*8    XB(3),VT(3)
      INTEGER   IA,IB,IL
C     ==--------------------------------------------------------------==
      DO IA=1,NAT
        ISC(IA)=0
      ENDDO
C     Now we check if ROT(N)+VR gives a correct symmetry.
      DO IA = 1,NAT
        DO IB = 1,NAT
          IF(TY(IA).EQ.TY(IB).AND.ISC(IB).EQ.0) THEN
            XB(1)=RX(1,IA) - X(1,IB)
            XB(2)=RX(2,IA) - X(2,IB)
            XB(3)=RX(3,IA) - X(3,IB)
            CALL RLV3(AI,XB,VT,IL,DELTA)
C           VT STANDS FOR V-TEST
            OKSYM=(DABS((VR(1)-VT(1))-NINT(VR(1)-VT(1))).LT.DELTA).AND.
     &            (DABS((VR(2)-VT(2))-NINT(VR(2)-VT(2))).LT.DELTA).AND.
     &            (DABS((VR(3)-VT(3))-NINT(VR(3)-VT(3))).LT.DELTA)
            IF(OKSYM) THEN
              IF(NODUPLI) ISC(IB)=1
              F0(N,IA)=IB
C             IR+VR is the good one: another symmetry operation
C             Next atom
              GOTO 100
            ENDIF
          ENDIF
        ENDDO
C       VR is not the correct translation vector
        RETURN
 100    CONTINUE
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SYMMORPHIC(NC,IB,R,V,AI,INFO,ORIGIN,DELTA)
C     ==--------------------------------------------------------------==
C     == Check if the group is symmorphic with a non-standard origin  ==
C     == WARNING: If there are equivalent atoms, this routine could   ==
C     == not determine if the space group is symmorphic               ==
C     == So you have to check if the solution V=0 works (see ATFTM1)  ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   NC Number of operations                                    ==
C     ==   IB(NC) Index of operation in R                             ==
C     ==   R(3,3,48) Rotations                                        ==
C     ==   V(3,NC) Fractional translations related to R(3,3,IB(NC))   ==
C     ==           R AND V ARE IN CARTESIAN COORDINATES               ==
C     ==   AI(I,J) ARE THE RECIPROCAL LATTICE VECTORS,                ==
C     ==           B(I) = AI(I,J),J=1,2,3                             ==
C     == DELTA     REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)          ==
C     ==                                                              ==
C     == OUTPUT:                                                      ==
C     ==   ORIGIN(1:3) Give standard origin (cartesian coordinates)   ==
C     ==            Give the standard origin with smallest coordinates==
C     ==            if NTVEC /= 1                                     ==
C     ==   INFO = 1 The group is symmorphic                           ==
C     ==   INFO = 0 The group is not symmorphic                       ==
C     ==   INFO =-1 The routine cannot determine                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER NC,IB(NC),INFO
      REAL*8  R(3,3,48),V(3,NC),AI(3,3),ORIGIN(3),DELTA
C     Variables
      INTEGER IOK(3),IGOOD(3),
     &        IR,I,J,I1,J1,IL,IERROR,IMISSING3,IMISSING2,IONLY
      REAL*8  DIF,R3(3,3),R2(2,2),WORK(3,3),VR(3),XB(3),DIAG
C     ==--------------------------------------------------------------==
C     Find a point A / V_R = (1-R).OA
      DO I=1,3
        IOK(I)=0
      ENDDO
      DO I=1,3
        ORIGIN(I)=0.D0
      ENDDO
      DO IR=1,NC
        DIF=V(1,IR)*V(1,IR)+V(2,IR)*V(2,IR)+V(3,IR)*V(3,IR)
        IF(DIF.GT.DELTA*DELTA) THEN
          DO I=1,3
            IGOOD(I) = 1
          ENDDO
C         V is non-zero. Construct matrix 1-R
          DO I=1,3
            DO J=1,3
              R3(I,J) = -R(I,J,IB(IR))
            ENDDO
            R3(I,I) = 1 + R3(I,I)
          ENDDO
          CALL INVMAT(3,R3,WORK,IERROR)
          IF(IERROR.EQ.0) THEN
C           The matrix 3x3 has an inverse.
            DO I=1,3
              VR(I)= R3(I,1)*V(1,IR)
     &             + R3(I,2)*V(2,IR)
     &             + R3(I,3)*V(3,IR)
            ENDDO
          ELSE
C           IERROR gives the column which causes some trouble
C           Construct matrix 1-R with 2x2
            IGOOD(IERROR)=0
            IMISSING3=IERROR
            I1=0
            DO I=1,3
              IF(I.NE.IERROR) THEN
                I1=I1+1
                J1=0
                DO J=1,3
                  IF(J.NE.IERROR) THEN
                    J1=J1+1
                    R2(I1,J1) = -R(I,J,IB(IR))
                  ENDIF
                ENDDO
                R2(I1,I1) = 1 + R2(I1,I1)
              ENDIF
            ENDDO
            CALL INVMAT(2,R2,WORK,IERROR)
            IF(IERROR.EQ.0) THEN
C             The matrix 2X2 has an inverse.
C             Solve Vxy = (1-R).OAxy + OAz R3z (z is IMISSING3)
              I1=0
              DO I=1,3
                IF(IGOOD(I).EQ.1) THEN
                  I1=I1+1
                  VR(I)=0.D0
                  J1=0
                  DO J=1,3
                    IF(IGOOD(J).EQ.1) THEN
                      J1=J1+1
                      VR(I)=VR(I)+R2(I1,J1)*(V(J,IR)+
     &                     ORIGIN(IMISSING3)*R(J,IMISSING3,IB(IR)))
                    ENDIF
                  ENDDO
                ELSE
                  VR(I)=ORIGIN(I)
                ENDIF
              ENDDO
            ELSE
C             Construct matrix 1-R with 1x1
              I1=0
              DO I=1,3
                IF(I.NE.IMISSING3) THEN
                  I1=I1+1
                  IF(I1.EQ.IERROR) THEN
                    IGOOD(I)=0
                    IMISSING2=I
                  ELSE
                    IONLY=I
                  ENDIF
                ENDIF
              ENDDO
              DIAG=(1-R(IONLY,IONLY,IB(IR)))
              IF(DABS(DIAG).GT.DELTA) THEN
                VR(IONLY) = 1.D0/DIAG*(V(IONLY,IR) +
     &             ORIGIN(IMISSING3)*R(IONLY,IMISSING3,IB(IR)) +
     &             ORIGIN(IMISSING2)*R(IONLY,IMISSING2,IB(IR)) )
              ELSE
                VR(IONLY) = ORIGIN(IONLY)
                IGOOD(IONLY)=0
              ENDIF
              VR(IMISSING3) = ORIGIN(IMISSING3)
              VR(IMISSING2) = ORIGIN(IMISSING2)
            ENDIF
          ENDIF
C         ==----------------------------------------------------------==
C         Compare VR with ORIGIN
          DIF=0.D0
C         If NTVEC /=1 there are NTVEC possible standard origins
          DO I=1,3
            IF(IOK(I).EQ.1) THEN
              DIF = DIF + DABS(ORIGIN(I)-VR(I))
            ENDIF
          ENDDO
          IF(DIF.GT.DELTA) THEN
C           Non-symmorphic
            INFO=0
            RETURN
          ELSE
            DO I=1,3
              IF(IOK(I).NE.1.AND.IGOOD(I).EQ.1) THEN
                IOK(I)=1
                ORIGIN(I)=VR(I)
              ENDIF
            ENDDO
          ENDIF
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
      IF(IOK(1).EQ.0.AND.IOK(2).EQ.0.AND.IOK(3).EQ.0) THEN
C       Cannot not determine
        INFO=-1
        RETURN
      ENDIF
C     The group is symmorphic
      INFO=1
C     Check
      DO IR=1,NC
        DO I=1,3
          VR(I)= R(I,1,IB(IR))*ORIGIN(1)
     &         + R(I,2,IB(IR))*ORIGIN(2)
     &         + R(I,3,IB(IR))*ORIGIN(3)
          VR(I) = (ORIGIN(I) - VR(I)) - V(I,IR)
        ENDDO
        CALL RLV3(AI,VR,XB,IL,DELTA)
        DIF=DABS(XB(1))+DABS(XB(2))+DABS(XB(3))
        IF(DIF.GT.DELTA) THEN
C         Non-symmorphic
          INFO=0
          RETURN
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE ROT1(IHC,R)
C     ==--------------------------------------------------------------==
C     == WRITTEN ON FEBRUARY 17TH, 1976                               ==
C     == GENERATION OF THE X,Y,Z-TRANSFORMATION MATRICES 3X3          ==
C     == FOR HEXAGONAL AND CUBIC GROUPS                               ==
C     == SUBROUTINES NEEDED -- NONE                                   ==
C     ==--------------------------------------------------------------==
C     == THIS IS IDENTICAL WITH THE SUBROUTINE ROT OF WORLTON-WARREN  ==
C     == (IN THE AC-COMPLEX), ONLY THE WAY OF TRANSFERRING THE DATA   ==
C     == WAS CHANGED                                                  ==
C     ==--------------------------------------------------------------==
C     == INPUT DATA:                                                  ==
C     == IHC SWITCH DETERMINING IF WE DESIRE                          ==
C     ==     THE HEXAGONAL GROUP(IHC=0) OR THE CUBIC GROUP (IHC=1)    ==
C     == OUTPUT DATA:                                                 ==
C     == R...THE 3X3 MATRICES OF THE DESIRED COORDINATE REPRESENTATION==
C     ==     THEIR NUMBERING CORRESPONDS TO THE SYMMETRY ELEMENTS AS  ==
C     ==     LISTE IN WORLTON-WARREN                                  ==
C     ==              (COMPUT. PHYS. COMM. 3(1972) 88--117)           ==
C     == FOR IHC=0 THE FIRST 24 MATRICES OF THE ARRAY R REPRESENT     ==
C     ==           THE FULL HEXAGONAL GROUP D(6H)                     ==
C     == FOR IHC=1 THE FIRST 48 MATRICES OF THE ARRAY R REPRESENT     ==
C     ==           THE FULL CUBIC GROUP O(H)                          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER IHC
      REAL*8  R(3,3,48)
C     Variables
      INTEGER I,J,K,N,NV
      REAL*8  C,S
C     ==--------------------------------------------------------------==
      DO J=1,3
        DO I=1,3
          DO N=1,48
            R(I,J,N)=0.D0
          ENDDO
        ENDDO
      ENDDO
      IF(IHC.EQ.0) THEN
C       ==------------------------------------------------------------==
C       DEFINE THE GENERATORS FOR THE ROTATION MATRICES--HEXAGONAL GROUP
C       ==------------------------------------------------------------==
        C = 0.5D0
        S = 0.5D0*DSQRT(3.0D0)
        R(1,1, 2) =  C
        R(1,2, 2) = -S
        R(2,1, 2) =  S
        R(2,2, 2) =  C
        R(1,1, 7) = -C
        R(1,2, 7) = -S
        R(2,1, 7) = -S
        R(2,2, 7) =  C
        DO N = 1,6
          R(3,3,N   ) =  1.D0
          R(3,3,N+18) =  1.D0
          R(3,3,N+6 ) = -1.D0
          R(3,3,N+12) = -1.D0
        ENDDO
C       ==------------------------------------------------------------==
C       == GENERATE THE REST OF THE ROTATION MATRICES                 ==
C       ==------------------------------------------------------------==
        DO I = 1,2
          R(I,I,1) = 1.D0
          DO J = 1,2
            R(I,J,6) = R(J,I,2)
            DO K = 1,2
              R(I,J, 3) = R(I,J, 3) + R(I,K,2) * R(K,J,2)
              R(I,J, 8) = R(I,J, 8) + R(I,K,2) * R(K,J,7)
              R(I,J,12) = R(I,J,12) + R(I,K,7) * R(K,J,2)
            ENDDO
          ENDDO
        ENDDO
        DO I = 1,2
          DO J = 1,2
            R(I,J,5) = R(J,I,3)
            DO K = 1,2
              R(I,J, 4) = R(I,J, 4) + R(I,K, 2) * R(K,J,3)
              R(I,J, 9) = R(I,J, 9) + R(I,K, 2) * R(K,J,8)
              R(I,J,10) = R(I,J,10) + R(I,K,12) * R(K,J,3)
              R(I,J,11) = R(I,J,11) + R(I,K,12) * R(K,J,2)
            ENDDO
          ENDDO
        ENDDO
        DO N = 1,12
          NV = N + 12
          DO I = 1,2
            DO J = 1,2
              R(I,J,NV) = - R(I,J,N)
            ENDDO
          ENDDO
        ENDDO
      ELSE
C       ==------------------------------------------------------------==
C       == DEFINE THE GENERATORS FOR THE ROTATION MATRICES-CUBIC GROUP==
C       ==------------------------------------------------------------==
        R(1,3, 9) = 1.D0
        R(2,1, 9) = 1.D0
        R(3,2, 9) = 1.D0
        R(1,1,19) =  1.D0
        R(2,3,19) = -1.D0
        R(3,2,19) =  1.D0
        DO I = 1,3
          R(I,I,1) = 1.D0
          DO J = 1,3
            R(I,J,20) = R(J,I,19)
            R(I,J, 5) = R(J,I, 9)
            DO K  = 1,3
              R(I,J, 2) = R(I,J, 2) + R(I,K,19) * R(K,J,19)
              R(I,J,16) = R(I,J,16) + R(I,K, 9) * R(K,J,19)
              R(I,J,23) = R(I,J,23) + R(I,K,19) * R(K,J, 9)
            ENDDO
          ENDDO
        ENDDO
        DO I = 1,3
          DO J = 1,3
            DO K = 1,3
              R(I,J, 6) = R(I,J, 6) + R(I,K, 2) * R(K,J, 5)
              R(I,J, 7) = R(I,J, 7) + R(I,K,16) * R(K,J,23)
              R(I,J, 8) = R(I,J, 8) + R(I,K, 5) * R(K,J, 2)
              R(I,J,10) = R(I,J,10) + R(I,K, 2) * R(K,J, 9)
              R(I,J,11) = R(I,J,11) + R(I,K, 9) * R(K,J, 2)
              R(I,J,12) = R(I,J,12) + R(I,K,23) * R(K,J,16)
              R(I,J,14) = R(I,J,14) + R(I,K,16) * R(K,J, 2)
              R(I,J,15) = R(I,J,15) + R(I,K, 2) * R(K,J,16)
              R(I,J,22) = R(I,J,22) + R(I,K,23) * R(K,J, 2)
              R(I,J,24) = R(I,J,24) + R(I,K, 2) * R(K,J,23)
            ENDDO
          ENDDO
        ENDDO
        DO I=1,3
          DO J=1,3
            DO K=1,3
              R(I,J, 3) = R(I,J, 3) + R(I,K, 5) * R(K,J,12)
              R(I,J, 4) = R(I,J, 4) + R(I,K, 5) * R(K,J,10)
              R(I,J,13) = R(I,J,13) + R(I,K,23) * R(K,J,11)
              R(I,J,17) = R(I,J,17) + R(I,K,16) * R(K,J,12)
              R(I,J,18) = R(I,J,18) + R(I,K,16) * R(K,J,10)
              R(I,J,21) = R(I,J,21) + R(I,K,12) * R(K,J,15)
            ENDDO
          ENDDO
        ENDDO
        DO N = 1,24
          NV = N + 24
          R(1,1,NV) = -R(1,1,N)
          R(1,2,NV) = -R(1,2,N)
          R(1,3,NV) = -R(1,3,N)
          R(2,1,NV) = -R(2,1,N)
          R(2,2,NV) = -R(2,2,N)
          R(2,3,NV) = -R(2,3,N)
          R(3,1,NV) = -R(3,1,N)
          R(3,2,NV) = -R(3,2,N)
          R(3,3,NV) = -R(3,3,N)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SPPT2(IOUT,IQ1,IQ2,IQ3,WVK0,NKPOINT,
     &                 A1,A2,A3,B1,B2,B3,
     &                 INV,NC,IB,R,NTOT,WVKL,LWGHT,LROT,
     &                 NCBRAV,IBRAV,ISTRIZ,
     &                 NHASH,INCLUD,LIST,RLIST,DELTA)
C     ==--------------------------------------------------------------==
C     == WRITTEN ON SEPTEMBER 12-20TH, 1979 BY K.K.                   ==
C     == MODIFIED 26-MAY-82 BY OLE HOLM NIELSEN                       ==
C     == GENERATION OF SPECIAL POINTS FOR AN ARBITRARY LATTICE,       ==
C     == FOLLOWING THE METHOD MONKHORST,PACK,                         ==
C     == PHYS. REV. B13 (1976) 5188                                   ==
C     == MODIFIED BY MACDONALD, PHYS. REV. B18 (1978) 5897            ==
C     == THE SUBROUTINE IS WRITTEN ASSUMING THAT THE POINTS ARE       ==
C     == GENERATED IN THE RECIPROCAL SPACE.                           ==
C     == IF, HOWEVER, THE B1,B2,B3 ARE REPLACED BY A1,A2,A3, THEN     ==
C     == SPECIAL POINTS IN THE DIRECT SPACE CAN BE PRODUCED, AS WELL. ==
C     == (NO MULTIPLICATION BY 2PI IS THEN NECESSARY.)                ==
C     == IN THE CASE OF NONSYMMORPHIC GROUPS, THE APPLICATION IN THE  ==
C     == DIRECT SPACE WOULD PROBABLY REQUIRE A CERTAIN CAUTION.       ==
C     == SUBROUTINES NEEDED: BZDEFI,BZRDUC,INBZ,MESH                  ==
C     == IN THE CASES WHERE THE POINT GROUP OF THE CRYSTAL DOES NOT   ==
C     == CONTAIN INVERSION. THE LATTER MAY BE ADDED IF WE WISH        ==
C     == (SEE COMMENT TO THE SWITCH INV).                             ==
C     == REDUCTION TO THE 1ST BRILLOUIN ZONE IS DONE                  ==
C     == BY ADDING G-VECTORS TO FIND THE SHORTEST WAVE-VECTOR.        ==
C     == THE ROTATIONS OF THE BRAVAIS LATTICE ARE APPLIED TO THE      ==
C     == MONKHORST/PACK MESH IN ORDER TO FIND ALL K-POINTS            ==
C     == THAT ARE RELATED BY SYMMETRY. (OLE HOLM NIELSEN)             ==
C     ==--------------------------------------------------------------==
C     == INPUT DATA:                                                  ==
C     == IOUT:    LOGICAL UNIT FOR OUTPUT                             ==
C     ==          IF (IOUT.LE.0) NO MESSAGE                           ==
C     == IQ1,IQ2,IQ3 .. PARAMETER Q OF MONKHORST AND PACK,            ==
C     ==          GENERALIZED AND DIFFERENT FOR THE 3 DIRECTIONS B1,  ==
C     ==          B2 AND B3                                           ==
C     == WVK0 ... THE 'ARBITRARY' SHIFT OF THE WHOLE MESH, DENOTED K0 ==
C     ==          IN MACDONALD. WVK0 = 0 CORRESPONDS TO THE ORIGINAL  ==
C     ==          SCHEME OF MONKHORST AND PACK.                       ==
C     ==          UNITS: 2PI/(UNITS OF LENGTH  USED IN A1, A2, A3),   ==
C     ==          I.E. THE SAME  UNITS AS THE GENERATED SPECIAL POINTS==
C     == NKPOINT .. VARIABLE DIMENSION OF THE (OUTPUT) ARRAYS WVKL,   ==
C     ==          LWGHT,LROT, I.E. SPACE RESERVED FOR THE SPECIAL     ==
C     ==          POINTS AND ACCESSORIES.                             ==
C     ==          NKPOINT HAS TO BE .GE. NTOT (TOTAL NUMBER OF SPECIAL==
C     ==          POINTS. THIS IS CHECKED BY THE SUBROUTINE.          ==
C     == ISTRIZ . INDICATES WHETHER ADDITIONAL MESH POINTS SHOULD BE  ==
C     ==          GENERATED BY APPLYING GROUP OPERATIONS TO THE MESH. ==
C     ==          ISTRIZ=+1 MEANS SYMMETRIZE                          ==
C     ==          ISTRIZ=-1 MEANS DO NOT SYMMETRIZE                   ==
C     == THE FOLLOWING INPUT DATA MAY BE OBTAINED FROM THE SBRT.      ==
C     == B1,B2,B3 .. RECIPROCAL LATTICE VECTORS, NOT MULTIPLIED BY    ==
C     ==          GROUP1: ANY 2PI (IN UNITS RECIPROCAL TO THOSE       ==
C     ==                  OF A1,A2,A3)                                ==
C     == INV .... CODE INDICATING WHETHER WE WISH TO ADD THE INVERSION==
C     ==          TO THE POINT GROUP OF THE CRYSTAL OR NOT (IN THE    ==
C     ==          CASE THAT THE POINT GROUP DOES NOT CONTAIN ANY).    ==
C     ==          INV=0 MEANS: DO NOT ADD INVERSION                   ==
C     ==          INV.NE.0 MEANS: ADD THE INVERSION                   ==
C     ==          INV.NE.0 SHOULD BE THE STANDARD CHOICE WHEN SPPT2   ==
C     ==          IS USED IN RECIPROCAL SPACE - IN ORDER TO MAKE      ==
C     ==          USE OF THE HERMITICITY OF HAMILTONIAN.              ==
C     ==          WHEN USED IN DIRECT SPACE, THE RIGHT CHOICE OF INV  ==
C     ==          WILL DEPEND ON THE NATURE OF THE PHYSICAL PROBLEM.  ==
C     ==          IN THE CASES WHERE THE INVERSION IS ADDED BY THE    ==
C     ==          SWITCH INV, THE LIST IB WILL NOT BE MODIFIED BUT IN ==
C     ==          THE OUTPUT LIST LROT SOME OF THE OPERATIONS WILL    ==
C     ==          APPEAR WITH NEGATIVE SIGN; THIS MEANS THAT THEY HAVE==
C     ==          TO BE APPLIED MULTIPLIED BY INVERSION.              ==
C     == NC ..... TOTAL NUMBER OF ELEMENTS IN THE POINT GROUP OF THE  ==
C     ==          CRYSTAL                                             ==
C     == IB ..... LIST OF THE ROTATIONS CONSTITUTING THE POINT GROUP  ==
C     ==          OF THE CRYSTAL. THE NUMBERING IS THAT DEFINED IN    ==
C     ==          WORLTON AND WARREN, I.E. THE ONE MATERIALIZED IN THE==
C     ==          ARRAY R (SEE BELOW)                                 ==
C     ==          ONLY THE FIRST NC ELEMENTS OF THE ARRAY IB ARE      ==
C     ==          MEANINGFUL                                          ==
C     == R ...... LIST OF THE 3 X 3 ROTATION MATRICES                 ==
C     ==          (XYZ REPRESENTATION OF THE O(H) OR D(6)H GROUPS)    ==
C     ==          ALL 48 OR 24 MATRICES ARE LISTED.                   ==
C     == NCBRAV . TOTAL NUMBER OF ELEMENTS IN RBRAV                   ==
C     == IBRAV .. LIST OF NCBRAV OPERATIONS OF THE BRAVAIS LATTICE    ==
C     == DELTA    REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)           ==
C     ==--------------------------------------------------------------==
C     == OUTPUT DATA:                                                 ==
C     == NTOT ... TOTAL NUMBER OF SPECIAL POINTS                      ==
C     ==          IF NTOT APPEARS NEGATIVE, THIS IS AN ERROR SIGNAL   ==
C     ==          WHICH MEANS THAT THE DIMENSION NKPOINT WAS CHOSEN   ==
C     ==          TOO SMALL SO THAT THE ARRAYS WVKL ETC. CANNOT       ==
C     ==          ACCOMODATE ALL THE GENERATED SPECIAL POINTS.        ==
C     ==          IN THIS CASE THE ARRAYS WILL BE FILLED UP TO NKPOINT==
C     ==          AND FURTHER GENERATION OF NEW POINTS WILL BE        ==
C     ==          INTERRUPTED.                                        ==
C     == WVKL ... LIST OF SPECIAL POINTS.                             ==
C     ==          CARTESIAN COORDINATES AND NOT MULTIPLIED BY 2*PI.   ==
C     ==          ONLY THE FIRST NTOT VECTORS ARE MEANINGFUL          ==
C     ==          ALTHOUGH NO 2 POINTS FROM THE LIST ARE EQUIVALENT   ==
C     ==          BY SYMMETRY, THIS SUBROUTINE STILL HAS A KIND OF    ==
C     ==          'BEAUTY DEFECT': THE POINTS FINALLY                 ==
C     ==          SELECTED ARE NOT NECESSARILY SITUATED IN A          ==
C     ==          'COMPACT' IRREDUCIBLE BRILL.ZONE; THEY MIGHT LIE IN ==
C     ==          DIFFERENT IRREDUCIBLE PARTS OF THE B.Z. - BUT THEY  ==
C     ==          DO REPRESENT AN IRREDUCIBLE SET FOR INTEGRATION     ==
C     ==          OVER THE ENTIRE B.Z.                                ==
C     == LWGHT ... THE LIST OF WEIGHTS OF THE CORRESPONDING POINTS.   ==
C     ==          THESE WEIGHTS ARE NOT NORMALIZED (JUST INTEGERS)    ==
C     == LROT ... FOR EACH SPECIAL POINT THE 'UNFOLDING ROTATIONS'    ==
C     ==          ARE LISTED. IF E.G. THE WEIGHT OF THE I-TH SPECIAL  ==
C     ==          POINT IS LWGHT(I), THEN THE ROTATIONS WITH NUMBERS  ==
C     ==          LROT(J,I), J=1,2,...,LWGHT(I) WILL 'SPREAD' THIS    ==
C     ==          SINGLE POINT FROM THE IRREDUCIBLE PART OF B.Z. INTO ==
C     ==          SEVERAL POINTS IN AN ELEMENTARY UNIT CELL           ==
C     ==          (PARALLELOPIPED) OF THE RECIPROCAL SPACE.           ==
C     ==          SOME OPERATION NUMBERS IN THE LIST LROT MAY APPEAR  ==
C     ==          NEGATIVE, THIS MEANS THAT THE CORRESPONDING ROTATION==
C     ==          HAS TO BE APPLIED WITH INVERSION (THE LATTER HAVING ==
C     ==          BEEN ARTIFICIALLY ADDED AS SYMMETRY OPERATION IN    ==
C     ==          CASE INV.NE.0).NO OTHER EFFORT WAS TAKEN,TO RENUMBER==
C     ==          THE ROTATIONS WITH MINUS SIGN OR TO EXTEND THE      ==
C     ==          LIST OF THE POINT-GROUP OPERATIONS IN THE LIST NB.  ==
C     == INCLUD ... INTEGER ARRAY USED BY SPPT2 INCLUD(NKPOINT)       ==
C     ==          THE FIRST BIT (0) IS USED BY THE ROUTINE.           ==
C     ==          THE OTHER BITS GIVE THE K-POINT INDEX IN            ==
C     ==          THE SPECIAL K-POINT TABLE.                          ==
C     ==--------------------------------------------------------------==
C     == NHASH    USED BY MESH ROUTINE                                ==
C     == LIST     INTEGER ARRAY USED BY MESH  LIST(NHASH+NKPOINT)     ==
C     == RLIST    REAL*8  ARRAY USED BY MESH  RLIST(3,NKPOINT)        ==
C     ==--------------------------------------------------------------==
C     == Use bit manipulations functions                              ==
C     ==  IBSET(I,POS) sets the bit POS to 1 in I integer             ==
C     ==  IBCLR(I,POS) clears the bit POS to 1 in I integer           ==
C     ==  BTEST(I,POS) .TRUE. if bit POS is 1 in I integer            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER IOUT,IQ1,IQ2,IQ3,NKPOINT,INV,NC,NTOT,NCBRAV,ISTRIZ,NHASH,
     &        IB(48),IBRAV(48),LWGHT(NKPOINT),LROT(48,NKPOINT),
     &        INCLUD(NKPOINT),LIST(NKPOINT+NHASH)
      REAL*8  WVK0(3),A1(3),A2(3),A3(3),B1(3),B2(3),B3(3),R(3,3,48),
     &        WVKL(3,NKPOINT),RLIST(3,NKPOINT),DELTA
C     ==--------------------------------------------------------------==
C     Variables
      INTEGER   YES,NO
      PARAMETER (YES = 1, NO = 0)
      INTEGER   NRSDIR
      PARAMETER (NRSDIR = 100)
      REAL*8    WVK(3),WVA(3),RSDIR(4,NRSDIR),PROJA(3),PROJB(3),
     &          UR1,UR2,UR3,DIFF
      INTEGER   I,J,NPLANE,IGARB0,IGARBG,IMESH,I1,I2,I3,IOP,IREMOV,K,
     &          JPLACE,IWVK,IGARBAGE,N,IBSIGN,
     &          INBZ
      INTEGER   IPLACE
      DATA      IPLACE /-2/
C     ==--------------------------------------------------------------==
      NTOT = 0
      DO I=1,NKPOINT
        LROT(1,I)=1
        DO J=2,48
          LROT(J,I)=0
        ENDDO
      ENDDO
      DO I = 1,NKPOINT
        INCLUD(I) = NO
      ENDDO
      DO I=1,3
        WVA(I)=0.D0
      ENDDO
C     ==--------------------------------------------------------------==
C     == DEFINE THE 1ST BRILLOUIN ZONE                                ==
C     ==--------------------------------------------------------------==
      CALL BZDEFI(IOUT,B1,B2,B3,RSDIR,NRSDIR,NPLANE,DELTA)
C     ==--------------------------------------------------------------==
C     == Generation of the mesh (they are not multiplied by 2*pi) by  ==
c     == the Monkhorst/Pack algorithm, supplemented by all rotations  ==
C     ==--------------------------------------------------------------==
C     Initialize the list of vectors
      IPLACE = - 2
      CALL MESH(IOUT,WVA,IPLACE,IGARB0,IGARBG,NKPOINT,NHASH,
     &          LIST,RLIST,DELTA)
      IMESH = 0
      DO I1=1,IQ1
        DO I2=1,IQ2
          DO I3=1,IQ3
            UR1=DBLE(1 + IQ1 - 2*I1)/DBLE(2*IQ1)
            UR2=DBLE(1 + IQ2 - 2*I2)/DBLE(2*IQ2)
            UR3=DBLE(1 + IQ3 - 2*I3)/DBLE(2*IQ3)
            DO I=1,3
              WVK(I) = UR1*B1(I) + UR2*B2(I) + UR3*B3(I) + WVK0(I)
            ENDDO
C           Reduce WVK to the 1st Brillouin zone
            CALL BZRDUC(WVK,A1,A2,A3,B1,B2,B3,RSDIR,
     &                  NRSDIR,NPLANE,DELTA)
            IF(ISTRIZ .EQ. 1) THEN
C             Symmetrization of the k-points mesh.
C             Apply all the Bravais lattice operations to WVK
              DO IOP = 1,NCBRAV
                DO I=1,3
                  WVA(I) = 0.D0
                  DO J = 1,3
                    WVA(I) = WVA(I) + R(I,J,IBRAV(IOP))*WVK(J)
                  ENDDO
                ENDDO
C               Check that WVA is inside the 1 Bz.
                IF(INBZ(WVA,RSDIR,NRSDIR,NPLANE,DELTA).EQ.NO) GOTO 450
C               Place WVA in list
                IPLACE = 0
                CALL MESH(IOUT,WVA,IPLACE,IGARB0,IGARBG,
     &                    NKPOINT,NHASH,LIST,RLIST,DELTA)
C               If WVA was new (and therefore inserted), 
C               IPLACE is the number.
                IF(IPLACE .GT. 0) IMESH = IPLACE
                IF(IPLACE .GT. NKPOINT) GOTO 470
              ENDDO
            ELSE
C             Place WVK in list
              IPLACE = 0
              CALL MESH(IOUT,WVK,IPLACE,IGARB0,IGARBG,
     &                  NKPOINT,NHASH,LIST,RLIST,DELTA)
              IMESH = IPLACE
              IF(IPLACE .GT. NKPOINT) GOTO 470
            ENDIF
          ENDDO
        ENDDO
      ENDDO
      IF(IOUT.GT.0) THEN
C       IMESH: Number of k points in the mesh.
        WRITE(IOUT,
     &     '(" THE WAVEVECTOR MESH CONTAINS ",I5," POINTS")') IMESH
       WRITE(IOUT,'(" THE POINTS ARE:")')
       DO I = 1,IMESH
         CALL MESH(IOUT,WVA,I,IGARB0,IGARBG,NKPOINT,NHASH,
     &             LIST,RLIST,DELTA)
         IF(MOD(I,2).EQ.1) THEN
           WRITE(IOUT,'(1X,I5,3F10.4,$)') I,WVA
         ELSE
           WRITE(IOUT,'(1X,I5,3F10.4)') I,WVA
         ENDIF
       ENDDO
       WRITE(IOUT,*)
      ENDIF
C     ==--------------------------------------------------------------==
      IF(ISTRIZ .EQ. 1) THEN
C       Now figure out if any special point difference (K - K'') is an
C       integral multiple of a reciprocal-space vector
        IREMOV = 0
        DO I = 1 , (IMESH - 1)
          IPLACE = I
          CALL MESH(IOUT,WVA,IPLACE,IGARB0,IGARBG,
     &              NKPOINT,NHASH,LIST,RLIST,DELTA)
C         Project WVA onto B1,2,3:
          PROJA(1) = 0.D0
          PROJA(2) = 0.D0
          PROJA(3) = 0.D0
          DO K = 1,3
            PROJA(1) = PROJA(1) + WVA(K)*A1(K)
            PROJA(2) = PROJA(2) + WVA(K)*A2(K)
            PROJA(3) = PROJA(3) + WVA(K)*A3(K)
          ENDDO
C         Now loop over all the rest of the mesh points
          DO J = (I + 1), IMESH
            JPLACE = J
            CALL MESH(IOUT,WVK,JPLACE,IGARB0,IGARBG,
     &                NKPOINT,NHASH,LIST,RLIST,DELTA)
C           Project WVK onto B1,2,3:
            PROJB(1) = 0.D0
            PROJB(2) = 0.D0
            PROJB(3) = 0.D0
            DO K = 1,3
              PROJB(1) = PROJB(1) + WVK(K)*A1(K)
              PROJB(2) = PROJB(2) + WVK(K)*A2(K)
              PROJB(3) = PROJB(3) + WVK(K)*A3(K)
            ENDDO
C           Check (PROJA - PROJB): Is it integral ?
            DO K = 1,3
              DIFF = PROJA(K) - PROJB(K)
              IF(DABS(DBLE(NINT(DIFF)) - DIFF ) .GT. DELTA) GOTO 280
            ENDDO
C           DIFF is integral: remove WVK from mesh:
            CALL REMOVE(WVK,JPLACE,IGARB0,IGARBG,
     &                  NKPOINT,NHASH,LIST,RLIST,DELTA)
C           If WVK actually removed, increment IREMOV
            IF(JPLACE .GT. 0) IREMOV = IREMOV + 1
 280        CONTINUE
          ENDDO
        ENDDO
        IF(IREMOV.GT.0 .AND. IOUT.GT.0)
     &       WRITE(IOUT,'(A,A,/,1X,I6,A,/)')
     &       ' SOME OF THESE MESH POINTS ARE RELATED BY LATTICE ',
     &       'TRANSLATION VECTORS',
     &       IREMOV,' OF THE MESH POINTS REMOVED.'
      ENDIF
C     ==--------------------------------------------------------------==
C     == IN THE MESH OF WAVEVECTORS, NOW SEARCH FOR EQUIVALENT POINTS:==
C     == THE INVERSION (TIME REVERSAL !) MAY BE USED.                 ==
C     ==--------------------------------------------------------------==
      DO IWVK = 1,IMESH
C        IF(INCLUD(IWVK) .EQ. YES) GOTO 350
        IF(BTEST(INCLUD(IWVK),0)) GOTO 350
C       IWVK has not been encountered previously: new special point,
C       (only if WVK is not a garbage vector, however.)
C        INCLUD(IWVK) = YES
        INCLUD(IWVK)=IBSET(INCLUD(IWVK),0)
        IPLACE = IWVK
        CALL MESH(IOUT,WVK,IPLACE,IGARB0,IGARBG,
     &            NKPOINT,NHASH,LIST,RLIST,DELTA)
C       Find out whether Wvk is in the garbage list
        CALL GARBAG(WVK,IGARBAGE,IGARB0,
     &              NKPOINT,NHASH,LIST,RLIST,DELTA)
        IF(IGARBAGE .GT. 0) GOTO 350
        NTOT = NTOT + 1
C       Give the index in the special k points table.
        INCLUD(IWVK)=INCLUD(IWVK)+NTOT*2
        DO I = 1,3
          WVKL(I,NTOT) = WVK(I)
        ENDDO
        LWGHT(NTOT) = 1
C       ==-----------------------------------------------------------==
C       Find all the equivalent points (symmetry given by atoms)
        DO N = 1,NC
C         Rotate:
          DO I = 1,3
            WVA(I) = 0.D0
            DO J = 1,3
              WVA(I) = WVA(I) + R(I,J,IB(N))*WVK(J)
            ENDDO
          ENDDO
          IBSIGN = + 1
 363      CONTINUE
C         Find WVA in the list
          IPLACE = -1
          CALL MESH(IOUT,WVA,IPLACE,IGARB0,IGARBG,
     &              NKPOINT,NHASH,LIST,RLIST,DELTA)
          IF(IPLACE.EQ.0) THEN
            IF(ISTRIZ.EQ.-1) THEN
C             No symmetrisation -> WVA not in the list
              GOTO 364
            ELSE
C             Find out whether WVA is in the garbage list
              CALL GARBAG(WVA,IGARBAGE,IGARB0,
     &             NKPOINT,NHASH,LIST,RLIST,DELTA)
              IF (IGARBAGE.EQ.0) THEN
C               I think this case is impossible (NC <= NCBRAV)
C               Error message
                GOTO 490
              ENDIF
            ENDIF
          ENDIF
C         Find out whether WVA is in the garbage list
          CALL GARBAG(WVA,IGARBAGE,IGARB0,
     &                NKPOINT,NHASH,LIST,RLIST,DELTA)
          IF(IGARBAGE .GT. 0) GOTO 370
C         Was WVA encountered before ?
C          IF(INCLUD(IPLACE) .EQ. YES) GOTO 364
          IF(BTEST(INCLUD(IPLACE),0)) GOTO 364
C         Increment weight.
          LWGHT(NTOT) = LWGHT(NTOT) + 1
          LROT(LWGHT(NTOT),NTOT) = IB(N)*IBSIGN
C          INCLUD(IPLACE) = YES
          INCLUD(IPLACE)=IBSET(INCLUD(IPLACE),0)
C         This k-point is an image of a special k-point.
C         Put the index of the special k-point.
          INCLUD(IPLACE)=INCLUD(IPLACE)+NTOT*2
 364      CONTINUE
          IF(IBSIGN .EQ. -1 .OR. INV .EQ. 0) GOTO 370
C         The case where we also apply the inversion to WVA
C         Repeat the search, but for -WVA
          IBSIGN = - 1
          DO I = 1,3
            WVA(I) = - WVA(I)
          ENDDO
          GOTO 363
 370      CONTINUE
        ENDDO
 350    CONTINUE
      ENDDO
C     ==--------------------------------------------------------------==
C     == TOTAL NUMBER OF SPECIAL POINTS: NTOT                         ==
C     == BEFORE USING THE LIST WVKL AS WAVE VECTORS, THEY HAVE TO BE  ==
C     == MULTIPLIED BY 2*PI                                           ==
C     == THE LIST OF WEIGHTS LWGHT IS NOT NORMALIZED                  ==
C     ==--------------------------------------------------------------==
      IF(NTOT .GT. NKPOINT) THEN
        WRITE(IOUT,*) 'IN SPPT2 NUMBER OF SPECIAL POINTS = ',NTOT
        WRITE(IOUT,*) 'BUT NKPOINT = ',NKPOINT
        NTOT = -1
      ENDIF
      IF(IOUT.GT.0) THEN
C       Write the index table relating k points in the mesh
C       with special k points
        WRITE(IOUT,'(/,4X,A)')
     &       'CROSS TABLE RELATING MESH POINTS WITH SPECIAL POINTS:'
        WRITE(IOUT,'(5(4X,"IK -> SK"))')
        DO I = 1,IMESH
          IPLACE = INCLUD(I)/2
          WRITE(IOUT,'(1X,I5,1X,I5,$)') I,IPLACE
          IF(MOD(I,5).EQ.0) WRITE(IOUT,*)
        ENDDO
        IF(MOD(J-1,5).NE.0) WRITE(IOUT, *)
      ENDIF
      RETURN
C     ==--------------------------------------------------------------==
C     == ERROR MESSAGES                                               ==
C     ==--------------------------------------------------------------==
 450  CONTINUE
      WRITE(IOUT,'(A,/)') ' SUBROUTINE SPPT2 *** FATAL ERROR ***'
      WRITE(IOUT,'(A,3F10.4,/,A,3F10.4,A,/,A,I3,A)')
     &     ' THE VECTOR     ',WVA,
     &     ' GENERATED FROM ',WVK,' IN THE BASIC MESH',
     &     ' BY ROTATION NO. ',IBRAV(IOP),' IS OUTSIDE THE 1BZ'
      CALL STOPGM('SPPT2','VECTOR OUTSIDE THE 1BZ')
C     ==--------------------------------------------------------------==
 470  CONTINUE
      WRITE(IOUT,'(A,/)') ' SUBROUTINE SPPT2 *** FATAL ERROR ***'
      WRITE(IOUT,*) 'MESH SIZE EXCEEDS NKPOINT=',NKPOINT
      CALL STOPGM('SPPT2','MESH SIZE EXCEEDED')
C     ==--------------------------------------------------------------==
 490  CONTINUE
      WRITE(IOUT,'(A,/)') ' SUBROUTINE SPPT2 *** FATAL ERROR ***'
      WRITE(IOUT,'(A,3F10.4,/,A,3F10.4,A,/,A,I3,A)')
     &     ' THE VECTOR     ',WVA,
     &     ' GENERATED FROM ',WVK,' IN THE BASIC MESH',
     &     ' BY ROTATION NO. ',IB(N),' IS NOT IN THE LIST'
      CALL STOPGM('SPPT2','VECTOR NOT IN THE LIST')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MESH (IOUT,WVK,IPLACE,IGARB0,IGARBG,
     &                 NMESH,NHASH,LIST,RLIST,DELTA)
C     ==--------------------------------------------------------------==
C     == MESH MAINTAINS A LIST OF VECTORS FOR PLACEMENT AND/OR LOOKUP ==
C     ==                                                              ==
C     == ADDITIONAL ENTRY POINTS: REMOVE .... REMOVE VECTOR FROM LIST ==
C     ==                          GARBAG .... WAS VECTOR REMOVED ?    ==
C     ==                                                              ==
C     == WVK ....... VECTOR                                           ==
C     == IPLACE .... ON INPUT: -2 MEANS: INITIALIZE  THE LIST         ==
C     ==                                 (AND RETURN)                 ==
C     ==                       -1 MEANS: FIND WVK IN THE LIST         ==
C     ==                        0 MEANS: ADD  WVK TO THE LIST         ==
C     ==                       >0 MEANS: RETURN WVK NO. IPLACE        ==
C     ==            ON OUTPUT: THE POSITION ASSIGNED TO WVK           ==
C     ==                       (=0 IF WVK IS NOT IN THE LIST)         ==
C     == DELTA      REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8    WVK(3),DELTA
      INTEGER   IOUT,IPLACE,IGARB0,IGARBG,
     &          NMESH,NHASH,LIST(NHASH+NMESH)
      REAL*8    RLIST(3,NMESH)
C     ==--------------------------------------------------------------==
C     Variables
      INTEGER   NIL
      PARAMETER (NIL = 0)
      INTEGER   IHASH,I,J,IPOINT
      REAL*8    RHASH,DELTA1
C     ==--------------------------------------------------------------==
C     Save
      INTEGER   ISTORE
      SAVE      ISTORE
C     ==--------------------------------------------------------------==
C     == Initialization                                               ==
C     ==--------------------------------------------------------------==
      DELTA1=10.D0*DELTA
      IF(IPLACE .LE. -2) THEN
        DO I = 1,NHASH+NMESH
          LIST(I) = NIL
        ENDDO
        ISTORE = 1
C       IGARB0 points to a linked list of removed WVKS (the garbage).
        IGARB0 = 0
        IGARBG = 0
        RETURN
C     ==--------------------------------------------------------------==
      ELSEIF( (IPLACE.GT.-2) .AND. (IPLACE.LE.0) ) THEN
C       The particular HASH function used in this case:
        RHASH = 0.7890D0*WVK(1)
     &        + 0.6810D0*WVK(2)
     &        + 0.5811D0*WVK(3) + DELTA
        IHASH = INT(DABS(RHASH) * DBLE(NHASH))
        IHASH = MOD(IHASH,NHASH) + NMESH + 1
C       Search for WVK in linked list
        IPOINT = LIST(IHASH)
        DO I = 1,100
C         List exhausted
          IF(IPOINT .EQ. NIL) GOTO 130
C         Compare WVK with this element
          DO J = 1,3
            IF(DABS(WVK(J) - RLIST(J,IPOINT)) .GT. DELTA1) GOTO 115
          ENDDO
C         WVK located
          GOTO 160
C         Next element of list
 115      CONTINUE
          IHASH = IPOINT
          IPOINT = LIST(IHASH)
        ENDDO
C       List too long
        WRITE(6,'(2A,/,A)')
     &       ' SUBROUTINE MESH *** FATAL ERROR *** LINKED LIST',
     &      ' TOO LONG ***', ' CHOOSE A BETTER HASH-FUNCTION'
        CALL STOPGM('MESH','WARNING')
C       WVK was not found
 130    CONTINUE
        IF(IPLACE.EQ.-1) THEN
C         IPLACE=-1 : search for WVK unsuccessful
          IPLACE = 0
          RETURN
        ELSE
C         IPLACE=0: add WVK to the list
          LIST(IHASH) = ISTORE
          IF(ISTORE .GT. NMESH) THEN
            WRITE(6,'(A)') 'SUBROUTINE MESH *** FATAL ERROR ***'
            WRITE(6,'(A,I10,A,/,A,3F10.5)')
     &           ' ISTORE=',ISTORE,' EXCEEDS DIMENSIONS',
     &           ' WVK = ',WVK
            CALL STOPGM('MESH','WARNING')
          ENDIF
          LIST(ISTORE) = NIL
          DO I = 1,3
            RLIST(I,ISTORE) = WVK(I)
          ENDDO
          ISTORE = ISTORE + 1
          IPLACE = ISTORE - 1
          RETURN
        ENDIF
C       WVK was found
 160    CONTINUE
        IF(IPLACE .EQ. 0) RETURN
C       IPLACE=-1
        IPLACE = IPOINT
        RETURN
      ELSE
C     ==--------------------------------------------------------------==
C     == Return a wavevector (IPLACE > 0)                             ==
C     ==--------------------------------------------------------------==
        IPOINT = IPLACE
        IF(IPOINT .GE. ISTORE) GOTO 190
        DO I = 1,3
          WVK(I) = RLIST(I,IPOINT)
        ENDDO
        RETURN
      ENDIF
C     ==--------------------------------------------------------------==
C     == Error - beyond list                                          ==
C     ==--------------------------------------------------------------==
 190  CONTINUE
      WRITE(IOUT,'(A,/,A,I5,A,/)')
     &     ' SUBROUTINE MESH *** WARNING ***',
     &     ' IPLACE = ',IPLACE,
     &     ' IS BEYOND THE LISTS - WVK SET TO 1.0E38'
      DO I = 1,3
        WVK(I) = 1.0D38
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE REMOVE (WVK,IPLACE,IGARB0,IGARBG,
     &                   NMESH,NHASH,LIST,RLIST,DELTA)
C     ==--------------------------------------------------------------==
C     == ENTRY POINT FOR REMOVING A WAVEVECTOR                        ==
C     ==                                                              ==
C     == INPUT:                                                       ==
C     ==   WVK(3)                                                     ==
C     ==   DELTA   REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)          ==
C     == OUTPUT:
C     ==   IPLACE ..... 1 IF WVK WAS REMOVED                          ==
C     ==                0 IF WVK WAS NOT REMOVED                      ==
C     ==                  (WVK NOT IN THE LINKED LISTS)               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8    WVK(3),DELTA
      INTEGER   IPLACE,IGARB0,IGARBG,
     &          NMESH,NHASH,LIST(NHASH+NMESH)
      REAL*8    RLIST(3,NMESH)
C     ==--------------------------------------------------------------==
C     Variables
      INTEGER   NIL
      PARAMETER (NIL = 0)
      INTEGER   IHASH,IPOINT,I,J
      REAL*8    RHASH,DELTA1
C     ==--------------------------------------------------------------==
      DELTA1=10.D0*DELTA
C     The particular hash function used in this case:
      RHASH = 0.7890D0*WVK(1)
     &      + 0.6810D0*WVK(2)
     &      + 0.5811D0*WVK(3) + DELTA
      IHASH = INT(DABS(RHASH) * DBLE(NHASH))
      IHASH = MOD(IHASH,NHASH) + NMESH + 1
C     Search for WVK in linked list
      IPOINT = LIST(IHASH)
      DO I = 1,100
C       List exhausted
        IF(IPOINT .EQ. NIL) THEN
C         WVK was not found in the mesh:
          IPLACE = 0
          RETURN
        ENDIF
C       Compare WVK with this element
        DO J = 1,3
          IF(DABS(WVK(J) - RLIST(J,IPOINT)) .GT. DELTA1) GOTO 215
        ENDDO
C       WVK located, now remove it from the list:
        LIST(IHASH) = LIST(IPOINT)
C       LIST(IHASH) now points to the next element in the list,
c       and the present WVK has become garbage.
C       Add WVK to the list of garbage:
        IF(IGARB0 .EQ. 0) THEN
C         Start up the garbage list:
          IGARB0 = IPOINT
        ELSE
          LIST(IGARBG) = IPOINT
        ENDIF
        IGARBG = IPOINT
        LIST(IGARBG) = NIL
        IPLACE = 1
        RETURN
C       Next element of list
 215    CONTINUE
        IHASH = IPOINT
        IPOINT = LIST(IHASH)
      ENDDO
C     List too long
      CALL STOPGM('MESH','LIST TOO LONG')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GARBAG (WVK,IPLACE,IGARB0,
     &                   NMESH,NHASH,LIST,RLIST,DELTA)
C     ==--------------------------------------------------------------==
C     == ENTRY POINT FOR CHECKING IF A WAVEVECTOR                     ==
C     ==             IS IN THE GARBAGE LIST                           ==
C     == INPUT:                                                       ==
C     ==   WVK(3)                                                     ==
C     ==   DELTA      REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)       ==
C     ==                                                              ==
C     == OUTPUT:                                                      ==
C     ==   IPLACE  ..... I > 0 IS THE PLACE IN THE GARBAGE LIST       ==
C     ==                     0 IF WVK NOT AMONG THE GARBAGE           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   IPLACE,IGARB0,
     &          NMESH,NHASH,LIST(NHASH+NMESH)
      REAL*8    WVK(3),DELTA,RLIST(3,NMESH)
C     ==--------------------------------------------------------------==
C     Variables
      INTEGER   NIL
      PARAMETER (NIL = 0)
      INTEGER   IPOINT,I,J,IHASH
      REAL*8    DELTA1
C     ==--------------------------------------------------------------==
      DELTA1=10.D0*DELTA
C     Search for WVK in linked list
C     Point to the garbage list
      IPOINT = IGARB0
      DO I = 1 , NMESH
C       LIST EXHAUSTED
        IF(IPOINT .EQ. NIL) THEN
C         WVK was not found in the mesh:
          IPLACE = 0
          RETURN
        ENDIF
C       Compare WVK with this element
        DO J = 1,3
          IF(DABS(WVK(J) - RLIST(J,IPOINT)) .GT. DELTA1) GOTO 315
        ENDDO
C       WVK was located in the garbage list
        IPLACE = I
        RETURN
C       Next element of list
 315    CONTINUE
        IHASH = IPOINT
        IPOINT = LIST(IHASH)
      ENDDO
C     List too long
      CALL STOPGM('GARBAG','LIST TOO LONG')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE BZRDUC(WVK,A1,A2,A3,B1,B2,B3,RSDIR,NRSDIR,NPLANE,DELTA)
C     ==--------------------------------------------------------------==
C     == REDUCE WVK TO LIE ENTIRELY WITHIN THE 1ST BRILLOUIN ZONE     ==
C     == BY ADDING B-VECTORS                                          ==
C     == DELTA      REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   NRSDIR,NPLANE
      REAL*8    WVK(3),A1(3),A2(3),A3(3),B1(3),B2(3),B3(3),
     &          RSDIR(4,NRSDIR),DELTA
C     ==--------------------------------------------------------------==
C     Variables
C     Look around +/- "NZONES" to locate vector
c     NZONES may need to be increased for very anisotropic zones
      INTEGER   NZONES,NNN,NN
      PARAMETER (NZONES=4, NNN=2*NZONES+1, NN=NZONES+1)
      INTEGER   YES
      PARAMETER (YES = 1)
      REAL*8    WVA(3),WB(3)
      INTEGER   NN1,NN2,NN3,N1,I1,N2,I2,N3,I3,I,
     &          INBZ
C     ==--------------------------------------------------------------==
C     WVK already inside 1Bz
      IF(INBZ(WVK,RSDIR,NRSDIR,NPLANE,DELTA).EQ.YES) RETURN
C     Express WVK in the basis of B1,2,3.
C     This permits an estimate of how far WVK is from the 1Bz.
      WB(1) = WVK(1)*A1(1) + WVK(2)*A1(2) + WVK(3)*A1(3)
      WB(2) = WVK(1)*A2(1) + WVK(2)*A2(2) + WVK(3)*A2(3)
      WB(3) = WVK(1)*A3(1) + WVK(2)*A3(2) + WVK(3)*A3(3)
      NN1 = NINT(WB(1))
      NN2 = NINT(WB(2))
      NN3 = NINT(WB(3))
C     Look around the estimated vector for the one truly inside the 1Bz
      DO N1 = 1,NNN
        I1 = NN - N1 - NN1
        DO N2 = 1,NNN
          I2 = NN - N2 - NN2
          DO N3 = 1,NNN
            I3 = NN - N3 - NN3
            DO I = 1,3
              WVA(I) = WVK(I) + DBLE(I1)*B1(I) + DBLE(I2)*B2(I) +
     +             DBLE(I3)*B3(I)
            ENDDO
            IF(INBZ(WVA,RSDIR,NRSDIR,NPLANE,DELTA).EQ.YES) GOTO 210
          ENDDO
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     Fatal error
      WRITE(6,'(A,/,A,3F10.4,A)')
     &     ' SUBROUTINE BZRDUC *** FATAL ERROR ***',
     &     ' WAVEVECTOR ',WVK,' COULD NOT BE REDUCED TO THE 1BZ'
      CALL STOPGM('BZRDUC','WARNING')
C     ==--------------------------------------------------------------==
C     The reduced vector
 210  CONTINUE
      DO I = 1,3
        WVK(I) = WVA(I)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION INBZ(WVK,RSDIR,NRSDIR,NPLANE,DELTA)
C     ==--------------------------------------------------------------==
C     == IS WVK IN THE 1ST BRILLOUIN ZONE ?                           ==
C     == CHECK WHETHER WVK LIES INSIDE ALL THE PLANES                 ==
C     == THAT DEFINE THE 1BZ.                                         ==
C     == DELTA      REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   NRSDIR,NPLANE
      REAL*8    WVK(3),RSDIR(4,NRSDIR),DELTA
      INTEGER   INBZ
C     Variables
      INTEGER   YES,NO
      PARAMETER (YES = 1, NO = 0)
      INTEGER   N
      REAL*8    PROJCT
C     ==--------------------------------------------------------------==
      INBZ = NO
      DO N = 1,NPLANE
        PROJCT = (RSDIR(1,N)*WVK(1) + RSDIR(2,N)*WVK(2) +
     +            RSDIR(3,N)*WVK(3) ) / RSDIR(4,N)
C       WVK is outside the Bz
        IF(DABS(PROJCT) .GT. 0.5D0 + DELTA) RETURN
      ENDDO
      INBZ = YES
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE BZDEFI(IOUT,B1,B2,B3,RSDIR,NRSDIR,NPLANE,DELTA)
C     ==--------------------------------------------------------------==
C     == FIND THE VECTORS WHOSE HALVES DEFINE THE 1ST BRILLOUIN ZONE  ==
C     == OUTPUT:                                                      ==
C     == NPLANE TELLS HOW MANY ELEMENTS OF RSDIR CONTAIN              ==
C     ==        NORMAL VECTORS DEFINING THE PLANES                    ==
C     == METHOD: STARTING WITH THE PARALLELOPIPED SPANNED BY B1,2,3   ==
C     == AROUND THE ORIGIN, VECTORS INSIDE A SUFFICIENTLY LARGE       ==
C     == SPHERE ARE TESTED TO SEE WHETHER THE PLANES AT 1/2*B WILL    ==
C     == FURTHER CONFINE THE 1BZ.                                     ==
C     == THE RESULTING VECTORS ARE NOT CLEANED TO AVOID REDUNDANT     ==
C     == PLANES.                                                      ==
C     == DELTA      REQUIRED ACCURACY (1.D-6 IS A GOOD VALUE)         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER IOUT,NRSDIR,NPLANE
      REAL*8  B1(3),B2(3),B3(3),RSDIR(4,NRSDIR),DELTA
C     Variables
      REAL*8    BVEC(3),
     &          B1LEN,B2LEN,B3LEN,BMAX,PROJCT
      INTEGER   NB1,NB2,NB3,I,J,NNB1,NNB2,NNB3,N1,I1,N2,I2,N3,I3,N,
     &          INITLZ
      DATA      INITLZ /0/
C     ==--------------------------------------------------------------==
      IF(INITLZ .NE. 0) RETURN
C     Once initialized, we do not repeat the calculation
      INITLZ = 1
      B1LEN = B1(1)**2 + B1(2)**2 + B1(3)**2
      B2LEN = B2(1)**2 + B2(2)**2 + B2(3)**2
      B3LEN = B3(1)**2 + B3(2)**2 + B3(3)**2
C     Lattice containing entirely the brillouin zone
      BMAX = B1LEN + B2LEN + B3LEN
      NB1 = INT(DSQRT(BMAX/B1LEN) + DELTA) + 1
      NB2 = INT(DSQRT(BMAX/B2LEN) + DELTA) + 1
      NB3 = INT(DSQRT(BMAX/B3LEN) + DELTA) + 1
C     PRINT *,'NB1,2,3 = ',NB1,NB2,NB3
      DO I = 1,NRSDIR
        RSDIR(1,I) = 0.D0
        RSDIR(2,I) = 0.D0
        RSDIR(3,I) = 0.D0
        RSDIR(4,I) = 0.D0
      ENDDO
C     1Bz is certainly confined inside the 1/2(B1,B2,B3) parallelopiped
      DO I = 1,3
        RSDIR(I,1) = B1(I)
        RSDIR(I,2) = B2(I)
        RSDIR(I,3) = B3(I)
      ENDDO
      RSDIR(4,1) = B1LEN
      RSDIR(4,2) = B2LEN
      RSDIR(4,3) = B3LEN
C     Starting confinement: 3 planes
      NPLANE = 3
      NNB1 = 2*NB1 + 1
      NNB2 = 2*NB2 + 1
      NNB3 = 2*NB3 + 1
      DO N1 = 1,NNB1
        I1 = NB1 + 1 - N1
        DO N2 = 1,NNB2
          I2 = NB2 + 1 - N2
          DO N3 = 1,NNB3
            I3 = NB3 + 1 - N3
            IF(I1.EQ.0 .AND. I2.EQ.0 .AND. I3.EQ.0) GOTO 150
            DO I = 1,3
              BVEC(I) = DBLE(I1)*B1(I) + DBLE(I2)*B2(I) +
     &             DBLE(I3)*B3(I)
            ENDDO
C           Does the plane of 1/2*BVEC narrow down the 1Bz ?
            DO N = 1,NPLANE
              PROJCT = 0.5D0*(RSDIR(1,N)*BVEC(1) + RSDIR(2,N)*BVEC(2)
     &               + RSDIR(3,N)*BVEC(3) ) / RSDIR(4,N)
C             1/2*BVEC is outside the Bz - skip this direction
C             The 1.D-6 takes care of single points touching the Bz,
C             and of the -(plane)
              IF(DABS(PROJCT) .GT. 0.5D0 - DELTA) GOTO 150
            ENDDO
C           1/2*BVEC further confines the 1Bz - include into RSDIR
            NPLANE = NPLANE + 1
C           PRINT *,NPLANE,' PLANE INCLUDED, I1,2,3 = ',I1,I2,I3
            IF(NPLANE .GT. NRSDIR) GOTO 470
            DO I = 1,3
              RSDIR(I,NPLANE) = BVEC(I)
            ENDDO
C           Length squared
            RSDIR(4,NPLANE) = BVEC(1)**2 + BVEC(2)**2 + BVEC(3)**2
 150        CONTINUE
          ENDDO
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     Print information
      WRITE(IOUT,'(A,I3,A,/,A,/,100(1X,3F10.4,/))')
     &     ' THE 1ST BRILLOUIN ZONE IS CONFINED BY (AT MOST)',
     &     NPLANE,' PLANES',
     &     ' AS DEFINED BY THE +/- HALVES OF THE VECTORS:',
     &     ((RSDIR(I,N),I=1,3),N=1,NPLANE)
      RETURN
C     ==--------------------------------------------------------------==
C     Error messages
 470  CONTINUE
      WRITE(6,'(A)') ' SUBROUTINE BZDEFI *** FATAL ERROR ***'
      WRITE(6,'(" TOO MANY PLANES, NRSDIR = ",I5)') NRSDIR
      CALL STOPGM('BZDEFI','WARNING')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
