C     ==================================================================
      SUBROUTINE KIN_ENERGY(C0,NSTATE,RSUM)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE KINETIC ENERGY EKIN. IT IS DONE IN RECIPROCAL SPACE     ==
C     ==  WHERE THE ASSOCIATED OPERATORS ARE DIAGONAL.                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ener.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'prcp.inc'
      INCLUDE 'spin.inc'
C     Arguments
      INTEGER    NSTATE
      COMPLEX*16 C0(NGW,NSTATE)
      REAL*8     RSUM
C     Variables
      DIMENSION  F(NSTATE)
      INTEGER    ISUB,I,IG,IS1
      REAL*8     XKIN,SK1,SK2,XSKIN,ARG,G2,RA,RB,IMA,IMB
      REAL*8     DELTAKIN
      PARAMETER  (DELTAKIN=1.D-10)
      REAL*8     DOTP
      EXTERNAL   DOTP
#ifdef __DERF
      REAL*8     DERF
      EXTERNAL   DERF
#endif
#if defined(__SR8000)
      REAL*8     G2W(*)
      POINTER    (IP_G2W,G2W)
      INTEGER    IFIRST
      SAVE       IFIRST,IP_G2W
      DATA       IFIRST /0/
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('KIN_ENERGY',ISUB)
C     ==--------------------------------------------------------------==
#if defined(__SR8000)
      IF(IFIRST.EQ.0) THEN
        CALL MEMORY(IP_G2W,NGW,'G2W')
        IFIRST=1
      ENDIF
#endif
C     ==--------------------------------------------------------------==
C     Accumulate the charge and kinetic energy
      RSUM=0.D0
      XKIN=0.D0
!$OMP parallel do private(I,SK1,XSKIN,IS1,ARG,G2,IG) 
!$OMP+ reduction(+:RSUM,XKIN)
!$OMP+ if(NSTATE.GT.100)
      DO I=1,NSTATE
        IF(F(I).NE.0.D0) THEN
          RSUM=RSUM+F(I)*DOTP(NGW,C0(1,I),C0(1,I))
          SK1=0.0D0
          IF(AKIN.GT.DELTAKIN) THEN
            XSKIN=1.D0/GSKIN
            IS1=1
            IF(GEQ0) THEN
              IS1=2
              ARG=-GCKIN*XSKIN
              G2=0.5D0*GAKIN*(1.D0+DERF(ARG))
              SK1=SK1+DREAL(G2*DCONJG(C0(1,I))*C0(1,I))
            ENDIF
#if defined(__SR8000)
*poption parallel
*poption tlocal(ARG)
            DO IG=IS1,NGW
              ARG=(HG(IG)-GCKIN)*XSKIN
              G2W(IG)=HG(IG)+GAKIN*(1.D0+DERF(ARG))
            ENDDO
#ifdef __SR8000
*poption parallel, tlocal(IG), psum(SK1)
#endif 
            DO IG=IS1,NGW
              SK1=SK1+DREAL(G2W(IG)*DCONJG(C0(IG,I))*C0(IG,I))
            ENDDO
#else
!$OMP parallel do private(IG,ARG,G2) shared(IS1,NGW,I)
!$OMP+ reduction(+:SK1)
!$OMP+ if(NSTATE.LE.100)
            DO IG=IS1,NGW
              ARG=(HG(IG)-GCKIN)*XSKIN
              G2=HG(IG)+GAKIN*(1.D0+DERF(ARG))
              SK1=SK1+DREAL(G2*DCONJG(C0(IG,I))*C0(IG,I))
            ENDDO
#endif
          ELSE
!$OMP parallel do private(IG) shared(NGW,I)
!$OMP+ reduction(+:SK1)
!$OMP+ if(NSTATE.LE.100)
#ifdef __SR8000
*poption parallel, tlocal(IG), psum(SK1)
#endif 
            DO IG=1,NGW
              SK1=SK1+DREAL(HG(IG)*DCONJG(C0(IG,I))*C0(IG,I))
            ENDDO
          ENDIF
          XKIN=XKIN+F(I)*SK1
        ENDIF
      ENDDO
      EKIN=XKIN*TPIBA2
C     Other kinetic energies for CAS22 method
      IF(TLSE .AND. (TCAS22.OR.TPENAL)) THEN
        IF(AKIN.GT.DELTAKIN) CALL STOPGM('RHOOFR',
     &                       'CAS22 and AKIN not implemented')
        SK1=0.D0
        DO IG=1,NGW
          RA=DREAL(C0(IG,IALPHA))
          RB=DREAL(C0(IG,IBETA))
          IMA=DIMAG(C0(IG,IALPHA))
          IMB=DIMAG(C0(IG,IBETA))
          SK1=SK1+HG(IG)*(RA*RB+IMA*IMB)
        ENDDO
        EKIN_AB = SK1 * TPIBA2
      ENDIF
      IF(TLSE .AND. TCAS22) THEN
        SK1=0.D0
        SK2=0.D0
        DO IG=1,NGW
          SK1=SK1+DREAL(HG(IG)*DCONJG(C0(IG,IALPHA))*C0(IG,IALPHA))
          SK2=SK2+DREAL(HG(IG)*DCONJG(C0(IG,IBETA))*C0(IG,IBETA))
        ENDDO
        EKIN_A = EKIN - TPIBA2 * ( SK2 - SK1 )
        EKIN_2 = EKIN + TPIBA2 * ( SK2 - SK1 )
      ENDIF
C
      CALL TIHALT('KIN_ENERGY',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
