C     ==================================================================
      SUBROUTINE FNL_FKP_P(C0,SCR,LSCR,H1NL,F,NSTATE,K_,FKPFIRST)
C     ==--------------------------------------------------------------==
C     == Computes H1NL (from FNL00), the perturbation needed for      ==
C     == the KPERT perturbation                                       ==
C     == the derivative of the nonlocal pseudopotential with respect  ==
C     == to the K_ component of the K vector, applied to c0.          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nlps.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sgpp.inc'
      INCLUDE 'kpnt.inc' 
      INCLUDE 'geq0.inc'
      INCLUDE 'response_p.inc'
C     Arguments
      INTEGER    LSCR,K_,NSTATE
      COMPLEX*16 H1NL(NGWK,NSTATE),C0(NGWk,NSTATE)
      REAL*8     F(NSTATE),SCR(LSCR)
      real*8     FNL(*),DFNL(*)       ! from sfac.inc
C     Variables
      DIMENSION  EIGR(NGW,NAT),TWNL(NGWK,NHXS,NSX,NKPNT),
     &     fnl00(IMAGP,NAT,NHXS,NSTATE),
     &     dtwnl_dk(2*NGW,NHXS,NSX,3),
     &     DFNL_DK(2,NAT,NHXS,NSTATE,3),
     &     DDFNL_DDK(2,NAT,NHXS,NSTATE,3,3),
     &     E2_NL_C0(3,3)
      COMPLEX*16 CTM,i_to_l

      REAL*8     DD0,DD1
      INTEGER    ISUB,istate,IV,IG,ISP_,IAT,IA,K2_,L,JV,KI,LI,L2,LJ,KJ
      LOGICAL    FKPFIRST
C     ==--------------------------------------------------------------==
      CALL TISET(' FNL_FKP_P',ISUB)

      CALL AZZERO(H1NL,2*NGWK*NSTATE)
      iat = 0
C     ==--------------------------------------------------------------==
      DO ISTATE = 1,NSTATE
         DO ISP_ = 1,NSP
C     ==--------------------------------------------------------------==
C     ==  VANDERBILT                                                  ==
C     ==--------------------------------------------------------------==
            IF(TVAN(ISP_)) THEN
               call STOPGM('K_PERT_PREP',
     &              'Vanderbilt PP not implemented')
            ELSEIF (TSGP(ISP_)) THEN
C     ==--------------------------------------------------------------==
C     ==  STEFAN GOEDECKER                                            ==
C     ==--------------------------------------------------------------==
               IF(PARENT .AND. ISTATE .EQ. 1) THEN
                  WRITE(6,'(A,I5)')
     &                 "STEFAN GOEDECKER PP for species",ISP_
               ENDIF
               IF (TFDIST) THEN
                  CALL STOPGM('K_PERT_PREP',
     &                 'distributed DFNLs not supported yet.')
               ENDIF
               DO IA=1,NA(ISP_)
                  IAT=IAT+1
                  DO IV=1,NGH(ISP_)
                     L=NGHTOL(IV,ISP_)+1
                     KI=LFVAL(IV,ISP_)
                     LI=LPVAL(IV,ISP_)
                     DD0=0.0D0
                     DD1 = 0.0D0
                     DO JV=1,NGH(ISP_)
                        L2=NGHTOL(JV,ISP_)+1
                        LJ=LPVAL(JV,ISP_)
                        IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                           KJ=LFVAL(JV,ISP_)
                           DD0=DD0+FNL00(1,IAT,JV,ISTATE)
     &                          *HLSG(KI,KJ,L,ISP_)
                           DD1=DD1+
     &                          DFNL_DK(2,IAT,JV,ISTATE,K_)
     &                          *HLSG(KI,KJ,L,ISP_)
                        ENDIF
                     ENDDO
                     CTM=-F(istate)*(0.0D0,-1.0D0)**NGHTOL(IV,ISP_)
                     DO IG=1,NGW
                        H1NL(IG,istate)=H1NL(IG,istate)+
     &                       CTM * EIGR(IG,IAT)*
     &                       (TWNL(IG,IV,ISP_,1) * DD1 +
     &                       (0.0d0,-1.0d0)*DTWNL_DK(IG,IV,ISP_,K_)*
     &                       DD0)
                     ENDDO

c     E2_NL_C0 quadratic terms depending only on the C0 wavefunctions
                     if(FKPFIRST) then
                        DO K2_ = 1,3
                           E2_NL_C0(K_,K2_)=E2_NL_C0(K_,K2_)+
     &                          F(istate)*
     &                          (DD0*DDFNL_DDK(1,IAT,IV,ISTATE,K_,K2_)+
     &                          DD1*DFNL_DK(2,IAT,IV,ISTATE,K2_))
                        ENDDO
                     endif      
                  ENDDO
               ENDDO
            ELSE

C     ==--------------------------------------------------------------==
C     ==  BACHELET HAMANN SCHLUTER                                    ==
C     ==--------------------------------------------------------------==
               DO ia = 1,na(ISP_)
                  iat = iat + 1
                  DO IV = 1,NGH(ISP_)
                     i_to_l = (0.0D0,-1.0D0)**NGHTOL(IV,ISP_)
                     CTM    = - i_to_l * F(istate)*WSG(ISP_,IV)
                     DO IG=1,NGW
                        H1NL(IG,istate)=H1NL(IG,istate)+
     &                       CTM * EIGR(IG,IAT)*
     &                       (TWNL(IG,IV,ISP_,1) * 
     &                       DFNL_DK(2,IAT,IV,ISTATE,K_)+
     &                       (0.0d0,-1.0d0)*DTWNL_DK(IG,IV,ISP_,K_)*
     &                       FNL00(1,IAT,IV,istate))
                     ENDDO

c     E2_NL_C0 quadratic terms depending only on the C0 wavefunctions

                     if(FKPFIRST) then
                        DO K2_ = 1,3
                           E2_NL_C0(K_,K2_)=E2_NL_C0(K_,K2_)+
     &                          F(istate)*wsg(isp_,IV)*
     &                          (FNL00(1,IAT,IV,istate)
     &                          *DDFNL_DDK(1,IAT,IV,ISTATE,K_,K2_)+
     &                          DFNL_DK(2,IAT,IV,ISTATE,K_)
     &                          *DFNL_DK(2,IAT,IV,ISTATE,K2_))
                        ENDDO
                     endif      
                  ENDDO
               ENDDO
            ENDIF
         ENDDO
         iat = 0
      ENDDO

      CALL TIHALT(' FNL_FKP_P',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
c     ==================================================================
      SUBROUTINE KIN_PERT(C0,SCR,LSCR,H1NL,F,NSTATE,K_)
C     ==--------------------------------------------------------------==
C     == Computes H1NL, the perturbation needed for                   ==
C     == the k-point perturbation.                                    ==
C     == the gradient is applied to C0                                ==
C     == it is done only for one direction k (=xyz)                   ==
C     == i * |HINL> = i * (-) (d |u0> / d rk)
C     == H1NL(IG,ISTATE) = (-) F(ISTATE) * (-i * Gk(IG)) * C0(IG,ISTATE)
C     == the (-) is due to the optimization algorithm 
C     ==--------------------------------------------------------------==

      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'response_p.inc'
C     Arguments
      INTEGER    LSCR,NSTATE,K_
      COMPLEX*16 H1NL(NGWK,NSTATE),C0(NGWK,NSTATE)
      REAL*8     F(NSTATE)
      REAL*8     SCR(LSCR)
C     Variables
      COMPLEX*16 G 
      INTEGER    istate,IG

C     ==--------------------------------------------------------------==
c      CALL TISET(' KIN_PERT',ISUB)
C     ==--------------------------------------------------------------==
C     ==  kinetic part of the perturbation                            ==
C     ==--------------------------------------------------------------==
c     forces are computed with the negative sign because of 
c     the optimization algorithm
      DO istate=1,NSTATE
         DO IG=1,NGWK
            G=DCMPLX(0.D0,-tpiba*GK(K_,IG))
            H1NL(IG,istate)=H1NL(IG,istate)-
     &           F(istate)*G*C0(IG,istate)     
         ENDDO
      ENDDO
c      CALL TIHALT(' KIN_PERT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================













