      SUBROUTINE  S12K_P(C1K,S12,A_TEMP,W,Z,AUX,NSTATE)
C     ==-------------------------------------------------------------==
C     ==                     COMPUTES:                               ==
C     ==   The inverse of the overlap matrix for the set of wfn      ==
C     ==   relative to the K-vector IKIND                            ==
C     ==-------------------------------------------------------------==

C     ==-------------------------------------------------------------==
C     ==     Compute the matrix S^(-1/2)[K]                          ==
C     ==-------------------------------------------------------------==

      implicit none
      include 'system.h'
      include 'sfac.inc'

C     ARGUMENTS
      INTEGER    NSTATE
      COMPLEX*16 C1K(NGW,NSTATE)
      REAL*8     S12(NSTATE,NSTATE),z(NSTATE,NSTATE),W(NSTATE),
     &     A_TEMP(NSTATE*NSTATE),AUX(3*2*NSTATE)

C     VARIABLES
      INTEGER    K,J,I,M,L,L0
      REAL*8     FAC
      REAL*8  FNL(*),DFNL(*)

      CALL CSMAT(S12,C1K,FNL,NSTATE,1)

      K = 0
      DO J = 1,NSTATE
         DO I = J,NSTATE
            K=K+1
            A_TEMP(K) = S12(I,J)
         ENDDO
      ENDDO

      CALL DSPEVY(1,A_TEMP,W,Z,NSTATE,NSTATE,AUX,3*NSTATE)

      CALL AZZERO(A_TEMP,NSTATE*NSTATE)
      DO M = 1,NSTATE
         W(M) = 1.D0/DSQRT(W(M))
         L = 1
         DO J = 1,NSTATE
            FAC = W(M)*Z(J,M)
            CALL DAXPY(NSTATE,FAC,Z(1,M),1,A_TEMP(L),1)
            L = L+NSTATE
         ENDDO
      ENDDO
      L = 1 
      DO J = 1,NSTATE
         DO I = 1,NSTATE
            S12(I,J) = A_TEMP(L)
            L = L+1
         ENDDO
      ENDDO

      L0=0
      DO J = 1,NSTATE
         DO I = J,NSTATE
            L=L0+I
            S12(I,J) = A_TEMP(L)
            S12(J,I) = S12(I,J)
         ENDDO
         L0=L0+NSTATE
      ENDDO

      RETURN
      END
C     ==---------------------------------------------------------------==


C     ==---------------------------------------------------------------==
      SUBROUTINE  HAMOFK_P(Z11,S12,NSTATE,IKK,KSQUARE,Z,SCR2,LSCR2)
C     ==---------------------------------------------------------------==
C     ==                       COMPUTES:                               ==
C     ==  The       FINAL HAMILTONIAN MATRIX OF K                      ==
C     ==---------------------------------------------------------------==

      implicit none
      include 'system.h'
      include 'kpts.inc'
      include 'kpnt.inc'
      include 'response_p.inc'

C     ARGUMRNTS
      INTEGER    NSTATE,IKK,LSCR2
      REAL*8     SCR2(LSCR2),Z(NSTATE,NSTATE)
      real*8     Z11(NSTATE,NSTATE),S12(NSTATE,NSTATE)
      REAL*8     KSQUARE

C     VARIABLES
      REAL*8     SumRe,SumIm
      INTEGER    ISTATE,JSTATE,IK1,IK2,I
      REAL*8     CFAC(3,3),FAC1(3,3)
      COMPLEX*16 CK2
      REAL*8     HH(2,NSTATE,NSTATE)
      pointer    (IP_HH,HH)
      REAL*8     B_TEMP(NSTATE,NSTATE),C_TEMP(NSTATE,NSTATE)
      pointer    (IP_B_TEMP,B_TEMP), (IP_C_TEMP,C_TEMP)
      DIMENSION wk(NKPTS),RK(3,NKPTS)
      DIMENSION HAMILK(2*NSTATE,2*NSTATE)
      DIMENSION H1_00(NSTATE,NSTATE,3),H2_00(NSTATE,NSTATE,3,3)
      DIMENSION H1_10(NSTATE,NSTATE,3,3),H0_11(NSTATE,NSTATE,3,3)



      CALL GET_ADDR(IP_HH,SCR2(1))
      CALL GET_ADDR(IP_B_TEMP,SCR2(1))
      CALL GET_ADDR(IP_C_TEMP,SCR2(1+NSTATE*NSTATE))

      CALL AZZERO(HH,2*NSTATE*NSTATE)


C     ==-------------------------------------------------------------==
C     ==     First block <c0|H|c0> = (z11 - (1/2)K^2) d_ij +         ==
C     ==           kx    H1_00(x) +Ky    H1_00(y) +Kz    H1_00(z)    ==
C     ==           Kx Kx H2_00(xx)+Kx Ky H2_00(xy)+Kx Kz H2_00(xz)   ==
C     ==           Ky Kx H2_00(yx)+Ky Ky H2_00(yy)+Ky Kz H2_00(yz)   ==
C     ==           Kz Kx H2_00(zx)+Kz Ky H2_00(zy)+Kz Kz H2_00(zz)   ==
C     ==-------------------------------------------------------------==

      DO ISTATE = 1,NSTATE
         HAMILK(ISTATE,ISTATE) = dcmplx(-z11(ISTATE,ISTATE)*0.5d0-
     &        0.5D0*TPIBA2*KSQUARE,0.0D0)
         DO JSTATE = ISTATE+1,NSTATE
            DO IK1 = 1,3
               HAMILK(JSTATE,ISTATE)= HAMILK(JSTATE,ISTATE)+
     &              TPIBA*RK(IK1,IKK)*H1_00(JSTATE,ISTATE,IK1)
               DO IK2 = 1,3
                  HAMILK(JSTATE,ISTATE)= 
     &                 HAMILK(JSTATE,ISTATE)+
     &                 TPIBA2*RK(IK1,IKK)*RK(IK2,IKK)*
     &                 H2_00(JSTATE,ISTATE,IK1,IK2)
               ENDDO
            ENDDO
            HAMILK(ISTATE,JSTATE)= 
     &           DCONJG(HAMILK(JSTATE,ISTATE))
         ENDDO
      ENDDO
      

C     ==-------------------------------------------------------------==
C     ==     Second block <c1|H|c0>{1j} = -i *( Sum_ab [             ==
C     ==           Ka Kb Sum_l [S^(-1/2){il} H1_10(ab){lj}]])        ==
C     ==            matrix product                                   ==
C     ==-------------------------------------------------------------==


      CALL AZZERO(HH,2*NSTATE*NSTATE)
      DO IK1 = 1,3
         DO IK2 = 1,3
C     K-dependent factor
c     CFAC(IK2,IK1) = -uimag*RK(IK1,IKK)*RK(IK2,IKK)*TPIBA2
            CFAC(IK2,IK1) = -RK(IK1,IKK)*RK(IK2,IKK)*TPIBA2
         ENDDO
      ENDDO

      DO ISTATE = 1,NSTATE
         DO JSTATE = 1,NSTATE
            DO IK1 = 1,3
               DO IK2 =  1,3
                  SumRe = DREAL(H1_10(JSTATE,ISTATE,IK2,IK1))
                  SumIm = DIMAG(H1_10(JSTATE,ISTATE,IK2,IK1))

                  HH(1,JSTATE,ISTATE)=HH(1,JSTATE,ISTATE)
     &                 -SumIm*CFAC(IK2,IK1)
                  HH(2,JSTATE,ISTATE)=HH(2,JSTATE,ISTATE)
     &                 +SumRe*CFAC(IK2,IK1)
c     HH(JSTATE,ISTATE)=HH(JSTATE,ISTATE)+
c     &                    CFAC(IK2,IK1)*H1_10(JSTATE,ISTATE,IK2,IK1)
               ENDDO
            ENDDO
         ENDDO
      ENDDO

      DO ISTATE = 1,NSTATE
         DO JSTATE = 1,NSTATE

            SumRe = 0.0d0
            SumIm = 0.0D0

            DO I = 1,NSTATE
               SumRe = SumRe + 
     &              S12(I,JSTATE)*
     &              HH(1,I,ISTATE)
               SumIm = SumIm + 
     &              S12(I,JSTATE)*
     &              HH(2,I,ISTATE) 
            ENDDO
            HAMILK(JSTATE+NSTATE,ISTATE) = 
     &           HAMILK(JSTATE+NSTATE,ISTATE)+
     &           DCMPLX(SumRe,SumIm)
            HAMILK(JSTATE,ISTATE+NSTATE) = 
     &           DCONJG(HAMILK(JSTATE+NSTATE,ISTATE))

         ENDDO
      ENDDO

C     

C     ==-------------------------------------------------------------==
C     ==     Third  block <c1|H|c1>{1j} = ( Sum_ab [                 ==
C     ==   Ka Kb Sum_lm [S^(-1/2){il} H1_10(ab){lm} S^(-1/2){mj}]])  ==
C     ==         double   matrix product                             ==
C     ==-------------------------------------------------------------==

      CK2 = DCMPLX(TPIBA2*KSQUARE*0.5D0,0.0D0)

      CALL AZZERO(Z,NSTATE*NSTATE)
      DO IK1 = 1,3
         DO IK2 = 1,3
            FAC1(IK2,IK1) = RK(IK1,IKK)*RK(IK2,IKK)*TPIBA2
         ENDDO
      ENDDO

      DO ISTATE = 1,NSTATE
         DO JSTATE = 1,NSTATE
            DO IK1 = 1,3
               DO IK2 =  1,3
                  Z(JSTATE,ISTATE)=Z(JSTATE,ISTATE)+
     &                 FAC1(IK2,IK1)*H0_11(JSTATE,ISTATE,IK2,IK1)
               ENDDO
            ENDDO
         ENDDO
      ENDDO



C     ==--------------------------------------------------------------==
C     ==         N^4 loop parted in 3 loops (sebast)                  ==


      CALL AZZERO(C_TEMP,NSTATE*NSTATE)
      CALL AZZERO(B_TEMP,NSTATE*NSTATE)

      CALL DGEMM('N','N',NSTATE,NSTATE,NSTATE,1.0D0,Z,NSTATE,
     &     S12,NSTATE,0.0D0,C_TEMP,NSTATE)

      CALL DGEMM('N','N',NSTATE,NSTATE,NSTATE,1.0D0,S12,NSTATE,
     &     C_TEMP,NSTATE,0.0D0,B_TEMP,NSTATE)


      DO ISTATE = 1,NSTATE
         HAMILK(ISTATE+NSTATE,ISTATE+NSTATE) =
     &        HAMILK(ISTATE+NSTATE,ISTATE+NSTATE) - CK2

         DO JSTATE = 1,NSTATE
            HAMILK(JSTATE+NSTATE,ISTATE+NSTATE) = 
     &           HAMILK(JSTATE+NSTATE,ISTATE+NSTATE)+
     &           DCMPLX(B_TEMP(JSTATE,ISTATE),0.0D0)
         ENDDO
      ENDDO

      RETURN
      END


