C     ==================================================================
      SUBROUTINE LATGEN(IBRAV,CELLDM,A1,A2,A3,OMEGA)
C     ==--------------------------------------------------------------==
C     ==  SETS UP THE CRYSTALLOGRAPHIC VECTORS A1,A2, AND A3.         ==
C     ==  AND THE VOLUME OF THE CELL                                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   IBRAV
      REAL*8    A1(3),A2(3),A3(3),CELLDM(6),OMEGA
C     Variables
      REAL*8    SR3
      PARAMETER (SR3=1.73205080756887729352D0)
      REAL*8    TERM,CBYA,TERM1,TERM2,SINVAL,SINGAM
      INTEGER   IR
C     ==--------------------------------------------------------------==
      DO IR=1,3
        A1(IR)=0.D0
        A2(IR)=0.D0
        A3(IR)=0.D0
      ENDDO
      IF(CELLDM(1).EQ.0)
     &     CALL STOPGM('LATGEN','THE LATTICE CONSTANT IS NULL!')
C     ==--------------------------------------------------------------==
C     ==  IBRAV =  1   Simple Cubic                                   ==
C     ==           2   Face Centered Cubic                            ==
C     ==           3   Body Centered Cubic                            ==
C     ==           4   Hexagonal                                      ==
C     ==           5   Rhombohedral or trigonal                       ==
C     ==           6   Tetragonal                                     ==
C     ==           7   Body Centred Tetragonal                        ==
C     ==           8   Orthorhombic                                   ==
C     ==          12   Monoclinic (1 angle different from 90)         ==
C     ==          14   Triclinic (3 angles different from 90)         ==
C     ==--------------------------------------------------------------==
      IF    (IBRAV.EQ.1) THEN
C       ==------------------------------------------------------------==
C       == Simple Cubic                                               ==
C       ==------------------------------------------------------------==
        A1(1)=CELLDM(1)
        A2(2)=CELLDM(1)
        A3(3)=CELLDM(1)
      ELSEIF(IBRAV.EQ.2) THEN
C       ==------------------------------------------------------------==
C       == Face Centered Cubic                                        ==
C       ==------------------------------------------------------------==
        TERM=CELLDM(1)/2.D0
        A1(1)=-TERM
        A1(3)= TERM
        A2(2)= TERM
        A2(3)= TERM
        A3(1)=-TERM
        A3(2)= TERM
      ELSEIF(IBRAV.EQ.3) THEN
C       ==------------------------------------------------------------==
C       == Body Centered Cubic                                        ==
C       ==------------------------------------------------------------==
        TERM=CELLDM(1)/2.D0
        DO IR=1,3
          A1(IR)=TERM
          A2(IR)=TERM
          A3(IR)=TERM
        ENDDO
        A2(1)=-TERM
        A3(1)=-TERM
        A3(2)=-TERM
      ELSEIF(IBRAV.EQ.4) THEN
C       ==------------------------------------------------------------==
C       == Hexagonal                                                  ==
C       ==------------------------------------------------------------==
        IF(CELLDM(3).EQ.0.D0)
     &       CALL STOPGM('LATGEN', 'C/A (THIRD ARGUMENT) IS NULL')
        CBYA=CELLDM(3)
        A1(1)=CELLDM(1)
        A2(1)=-CELLDM(1)/2.D0
        A2(2)=CELLDM(1)*SR3/2.D0
        A3(3)=CELLDM(1)*CBYA
      ELSEIF(IBRAV.EQ.5) THEN
C       ==------------------------------------------------------------==
C       == Trigonal or rhombohedral                                   ==
C       ==------------------------------------------------------------==
        IF(CELLDM(4).EQ.1.D0)
     &       CALL STOPGM('LATGEN','THE ANGLE IS NULL!')
        TERM1=DSQRT(1.D0+2.D0*CELLDM(4))
        TERM2=DSQRT(1.D0-CELLDM(4))
        A2(2)=DSQRT(2.D0)*CELLDM(1)*TERM2/SR3
        A2(3)=CELLDM(1)*TERM1/SR3
        A1(1)=CELLDM(1)*TERM2/DSQRT(2.D0)
        A1(2)=-A1(1)/SR3
        A1(3)=A2(3)
        A3(1)=-A1(1)
        A3(2)=A1(2)
        A3(3)=A2(3)
      ELSEIF(IBRAV.EQ.6) THEN
C       ==------------------------------------------------------------==
C       == Tetragonal                                                 ==
C       ==------------------------------------------------------------==
        IF(CELLDM(3).EQ.0.D0)
     &       CALL STOPGM('LATGEN', 'C/A (THIRD ARGUMENT) IS NULL')
        CBYA=CELLDM(3)
        A1(1)=CELLDM(1)
        A2(2)=CELLDM(1)
        A3(3)=CELLDM(1)*CBYA
      ELSEIF(IBRAV.EQ.7) THEN
C       ==------------------------------------------------------------==
C       == Body Centred Tetragonal                                    ==
C       ==------------------------------------------------------------==
        CBYA=CELLDM(3)
        A2(1)=CELLDM(1)/2.D0
        A2(2)=A2(1)
        A2(3)=CBYA*CELLDM(1)/2.D0
        A1(1)=A2(1)
        A1(2)=-A2(1)
        A1(3)=A2(3)
        A3(1)=-A2(1)
        A3(2)=-A2(1)
        A3(3)=A2(3)
      ELSEIF(IBRAV.EQ.8) THEN
C       ==------------------------------------------------------------==
C       == Orthorhombic                                               ==
C       ==------------------------------------------------------------==
        IF(CELLDM(2).EQ.0.D0)
     &       CALL STOPGM('LATGEN', 'THE SECOND ARGUMENT IS NULL')
        IF(CELLDM(3).EQ.0.D0)
     &       CALL STOPGM('LATGEN', 'THE THIRD ARGUMENT IS NULL')
        A1(1)=CELLDM(1)
        A2(2)=CELLDM(1)*CELLDM(2)
        A3(3)=CELLDM(1)*CELLDM(3)
      ELSEIF(IBRAV.EQ.12) THEN
C       ==------------------------------------------------------------==
C       == Monoclinic (1 angle different from 90)                     ==
C       ==------------------------------------------------------------==
        IF(CELLDM(2).EQ.0.D0)
     &       CALL STOPGM('LATGEN', 'THE SECOND ARGUMENT IS NULL')
        IF(CELLDM(3).EQ.0.D0)
     &       CALL STOPGM('LATGEN', 'THE THIRD ARGUMENT IS NULL')
        IF(CELLDM(4).EQ.1.D0)
     &       CALL STOPGM('LATGEN','THE ANGLE IS NULL!')
        SINVAL=DSQRT(1.D0-CELLDM(4)**2)
        A1(1)=CELLDM(1)
        A2(1)=CELLDM(1)*CELLDM(2)*CELLDM(4)
        A2(2)=CELLDM(1)*CELLDM(2)*SINVAL
        A3(3)=CELLDM(1)*CELLDM(3)
      ELSEIF(IBRAV.EQ.14) THEN
C       ==------------------------------------------------------------==
C       == Triclinic (3 angles different from 90)                     ==
C       ==------------------------------------------------------------==
        IF(CELLDM(2).EQ.0.D0)
     &       CALL STOPGM('LATGEN', 'THE SECOND ARGUMENT IS NULL')
        IF(CELLDM(3).EQ.0.D0)
     &       CALL STOPGM('LATGEN', 'THE THIRD ARGUMENT IS NULL')
        IF(CELLDM(4).EQ.1.D0)
     &       CALL STOPGM('LATGEN','THE FIRST ANGLE IS NULL!')
        IF(CELLDM(5).EQ.1.D0)
     &       CALL STOPGM('LATGEN','THE SECOND ANGLE IS NULL!')
        IF(CELLDM(6).EQ.1.D0)
     &       CALL STOPGM('LATGEN','THE THIRD ANGLE IS NULL!')
C       Compatiblity between angle
        IF( ACOS(CELLDM(4))+ACOS(CELLDM(5)).LT.ACOS(CELLDM(6)) )
     &       CALL STOPGM('LATGEN','ALPHA + BETA >= GAMMA')
        IF( ACOS(CELLDM(5))+ACOS(CELLDM(6)).LT.ACOS(CELLDM(4)) )
     &       CALL STOPGM('LATGEN','BETA + GAMMA >= ALPHA')
        IF( ACOS(CELLDM(6))+ACOS(CELLDM(4)).LT.ACOS(CELLDM(5)) )
     &       CALL STOPGM('LATGEN','GAMMA + ALPHA >= BETA')
        SINGAM=DSQRT(1.D0-CELLDM(6)**2)
        A1(1)=CELLDM(1)
        A2(1)=CELLDM(1)*CELLDM(2)*CELLDM(6)
        A2(2)=CELLDM(1)*CELLDM(2)*SINGAM
        A3(1)=CELLDM(1)*CELLDM(3)*CELLDM(5)
        A3(2)=CELLDM(1)*CELLDM(3)*(CELLDM(4)-CELLDM(5)*CELLDM(6))/SINGAM
        TERM=DSQRT((1.D0+2.D0*CELLDM(4)*CELLDM(5)*CELLDM(6)
     &       -CELLDM(4)**2-CELLDM(5)**2
     &       -CELLDM(6)**2)/(1.D0-CELLDM(6)**2))
        A3(3)=CELLDM(1)*CELLDM(3)*TERM
      ELSEIF(IBRAV.EQ.9 .OR.
     &       IBRAV.EQ.10.OR.
     &       IBRAV.EQ.11.OR.
     &       IBRAV.EQ.13) THEN
C       ==------------------------------------------------------------==
C       == Not defined IBRAV numbers (9, 10, 11, 13)                  ==
C       ==------------------------------------------------------------==
        WRITE(6,'(A,I3,A)') ' BRAVAIS LATTICE',IBRAV,
     &      ' NOT PROGRAMMED. STOPPING' 
        CALL STOPGM('LATGEN',' ')
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  Compute the volume of the Supercell (OMEGA)                 ==
C     ==--------------------------------------------------------------==
      CALL OMEGAGEN(A1,A2,A3,OMEGA)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GENLAT(A1,A2,A3,CELLDM,OMEGA)
C     ==--------------------------------------------------------------==
C     == COMPUTE CELLDM(1:6) Dx Dy Dz alpha(yz) beta(zx) gamma(xy)    ==
C     == FROM A1(3), A2(3), A3(3)                                     ==
C     ==--------------------------------------------------------------==
C     Arguments
      REAL*8    A1(3),A2(3),A3(3),CELLDM(6),OMEGA
C     ==--------------------------------------------------------------==
C     CELLDM(1) = |A1|
      CELLDM(1) = SQRT(A1(1)*A1(1)+A1(2)*A1(2)+A1(3)*A1(3))
      IF (CELLDM(1).EQ.0.D0) THEN
        WRITE(6,FMT=*) 'GENLAT! THE FIRST CELL VECTOR A1=',A1
        CALL STOPGM('GENLAT','|A1| == 0.0')
      ENDIF
C     CELLDM(2) = |A2|/|A1|
      CELLDM(2) = SQRT(A2(1)*A2(1)+A2(2)*A2(2)+A2(3)*A2(3))
      IF (CELLDM(2).EQ.0.D0) THEN
        WRITE(6,FMT=*) 'GENLAT! THE SECOND CELL VECTOR A2=',A2
        CALL STOPGM('GENLAT','|A2| == 0.0')
      ENDIF
C     CELLDM(3) = |A3|/|A1|
      CELLDM(3) = SQRT(A3(1)*A3(1)+A3(2)*A3(2)+A3(3)*A3(3))
      IF (CELLDM(3).EQ.0.D0) THEN
        WRITE(6,FMT=*) 'GENLAT! THE THIRD CELL VECTOR A3=',A3
        CALL STOPGM('GENLAT','|A3| == 0.0')
      ENDIF
C     CELLDM(4) = A2.A3/(|A2|.|A3|)
      CELLDM(4) =
     &     (A2(1)*A3(1)+A2(2)*A3(2)+A2(3)*A3(3))/(CELLDM(2)*CELLDM(3))
      IF (CELLDM(4).EQ.1.D0) THEN
        CALL STOPGM('GENLAT','A2 AND A3 ARE IDENTICAL!')
      ENDIF
C     CELLDM(5) = A3.A1/(|A3|.|A1|)
      CELLDM(5) =
     &     (A3(1)*A1(1)+A3(2)*A1(2)+A3(3)*A1(3))/(CELLDM(3)*CELLDM(1))
      IF (CELLDM(5).EQ.1.D0) THEN
        CALL STOPGM('GENLAT','A3 AND A1 ARE IDENTICAL!')
      ENDIF
C     CELLDM(6) = A1.A2/(|A1|.|A2|)
      CELLDM(6) =
     &     (A1(1)*A2(1)+A1(2)*A2(2)+A1(3)*A2(3))/(CELLDM(1)*CELLDM(2))
      IF (CELLDM(6).EQ.1.D0) THEN
        CALL STOPGM('GENLAT','A1 AND A2 ARE IDENTICAL!')
      ENDIF
      CELLDM(2) = CELLDM(2)/CELLDM(1)
      CELLDM(3) = CELLDM(3)/CELLDM(1)
C     ==--------------------------------------------------------------==
C     ==  Compute the volume of the Supercell (OMEGA)                 ==
C     ==--------------------------------------------------------------==
      CALL OMEGAGEN(A1,A2,A3,OMEGA)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE OMEGAGEN(A1,A2,A3,OMEGA)
C     ==--------------------------------------------------------------==
C     == COMPUTE THE VOLUME OF THE SUPERCELL (OMEGA)                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8 A1(3),A2(3),A3(3)
      REAL*8 OMEGA
C     Local variables
      REAL*8 S
      INTEGER I,J,K,L,IPERM,IVEC
C     ==--------------------------------------------------------------==
      OMEGA=0.D0
      DO IVEC = 1,2
        IF(IVEC.EQ.1) THEN
          S=1.D0
          I=1
          J=2
          K=3
        ELSE
          S=-1.D0
          I=2
          J=1
          K=3
        ENDIF
        DO IPERM=1,3
          OMEGA=OMEGA+S*A1(I)*A2(J)*A3(K)
          L=I
          I=J
          J=K
          K=L
        ENDDO
      ENDDO
      OMEGA=DABS(OMEGA)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

