#ifdef __SR8000
*option MP(P(0)), LANGLVL(SAVE(0))
#endif
C     ==================================================================
      SUBROUTINE LR_IN
C     ==--------------------------------------------------------------==
C     ==  THIS ROUTINE READS THE SECTION &LINRES &END ON UNIT IUNIT   ==
C     ==--------------------------------------------------------------==
C     ==  THE INPUT HAS TO BE IN THE FOLLOWING FORMAT                 ==
C     ==     &LINRES                                                  ==
C     ==        Options                                               ==
C     ==     &END                                                     ==
C     ==--------------------------------------------------------------==
C     ==  LIST OF OPTIONS                                             ==
C     ==                                                              ==
C     ==  XC_ANALYTIC       :use analytic second and third            ==
C     ==                     derivatives of the XC functional         ==
C     ==                     (only available for some LDA             ==
C     ==                     functionals)                             ==
C     ==  XC_DD_ANALYTIC    :use analytic second derivatives of some  ==
C     ==                     LDA and gradient corrected XC            ==
C     ==                     functionals (third derivatives either    ==
C     ==                     LDA or numeric (XC_ANALYTIC)             ==
C     ==  DIFF FORMULA      :Number of points used in finite diff     ==
C     ==    ndpoint                                                   ==
C     ==  XC_EPS            :finite difference parameter for XC       ==
C     ==    eps              derivative                               ==
C     ==  MAXSTEP           :Max. number of optimization steps        ==
C     ==    nmaxlr                                                    ==
C     ==  HTHRS             :Threshold for Hessian                    ==
C     ==    lr_hthrs                                                  ==
C     ==  OPTIMIZER [SD,DIIS,PCG,AUTO] :Optimizer for LR equations    ==
C     ==  THAUTO            :Threshold to switch from PCG to ODIIS    ==
C     ==    thauto(1) thauto(2)                                       ==
C     ==  ZDIIS             :size of ZDIIS buffer                     ==
C     ==   mldiis                                                     ==
C     ==  STEPLENGTH        :Step length for SD and PCG               ==
C     ==    dlrs                                                      ==
C     ==  CONVERGENCE       :Conv. criteria for LR optimisation       ==
C     ==    tol_lr                                                    ==
C     ==  QS_LIMIT          :Tollerance above which we use QS search  ==
C     ==    tol_qs                                                    ==
C     ==  GAUGE [PARA|GEN|ALL] :Gauge of response functions           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'linres.inc'
      INTEGER   IUNIT,IERR,INSCAN,IGAUGE
      CHARACTER LINE*80
      INTEGER   MSGLEN
C     ==--------------------------------------------------------------==
      IF(.NOT.PARENT) GOTO 9999
      IUNIT = 5
C     ==--------------------------------------------------------------==
C     ==  DEFAULTS                                                    ==
C     ==--------------------------------------------------------------==
      TXC_ANALYTIC=.FALSE.
      TXC_DD_ANA=.FALSE.
      IGAUGE=0
      XC_EPS=0.0005D0
      NMAXLR=1000
      LR_HTHRS=0.5D0
      LOPTI=0
      DLRS=0.1D0
      TOL_LR=1.D-5
      TOL_QS=1.D-4
      THAUTO(1)=1.D-1
      THAUTO(2)=1.D-3
      NDPOINT=2
      MLDIIS=4
C     ==--------------------------------------------------------------==
      IERR=INSCAN(IUNIT,'&LINRES')
      IF(IERR.NE.0) GOTO 100
C     ==--------------------------------------------------------------==
   10 CONTINUE
        READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
        IF(INDEX(LINE,'&END').NE.0) GOTO 100
        IF(INDEX(LINE,'XC_AN').NE.0) THEN
C..analytic dmu/dn
          TXC_ANALYTIC=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'XC_DD_AN').NE.0) THEN
C..analytic dmu/dn (gradient corrected)
          TXC_DD_ANA=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'XC_EPS').NE.0) THEN
C..finite difference parameter for dmu/dn
          READ(IUNIT,ERR=99,END=99,FMT=*) XC_EPS
          GOTO 10
        ELSE IF(INDEX(LINE,'MAXSTEP').NE.0) THEN
C..max. number of optimisation steps
          READ(IUNIT,ERR=99,END=99,FMT=*) NMAXLR
          GOTO 10
        ELSE IF(INDEX(LINE,'HTHRS').NE.0) THEN
C..Hessian threshold
          READ(IUNIT,ERR=99,END=99,FMT=*) LR_HTHRS
          GOTO 10
        ELSE IF(INDEX(LINE,'ZDIIS').NE.0) THEN
C..Size of ZDIIS buffer
          READ(IUNIT,ERR=99,END=99,FMT=*) MLDIIS
          GOTO 10
        ELSE IF(INDEX(LINE,'OPTIM').NE.0) THEN
C..Optimizer
          IF(INDEX(LINE,'SD').NE.0) LOPTI=1
          IF(INDEX(LINE,'PCG').NE.0) LOPTI=2
          IF(INDEX(LINE,'DIIS').NE.0) LOPTI=3
          IF(INDEX(LINE,'AUTO').NE.0) LOPTI=0
          GOTO 10
        ELSE IF(INDEX(LINE,'GAUGE').NE.0) THEN
C..Gauge of response functions
          IF(INDEX(LINE,'PARA').NE.0) IGAUGE=1
          IF(INDEX(LINE,'GEN').NE.0) IGAUGE=2
          IF(INDEX(LINE,'ALL').NE.0) IGAUGE=3
          GOTO 10
        ELSE IF(INDEX(LINE,'STEPL').NE.0) THEN
C..Step length      
          READ(IUNIT,ERR=99,END=99,FMT=*) DLRS
          GOTO 10
        ELSE IF(INDEX(LINE,'CONVE').NE.0) THEN
C..Convergence criteria
          READ(IUNIT,ERR=99,END=99,FMT=*) TOL_LR
          GOTO 10
        ELSE IF(INDEX(LINE,'QS_LI').NE.0) THEN
C..TOLERANCE FOR QS IN PCG
          READ(IUNIT,ERR=99,END=99,FMT=*) TOL_QS
          GOTO 10
        ELSE IF(INDEX(LINE,'THAUTO').NE.0) THEN
C..TOLERANCE FOR SWITCH FROM PCG TO ODIIS
          READ(IUNIT,ERR=99,END=99,FMT=*) THAUTO(1),THAUTO(2)
          GOTO 10
        ELSE IF(INDEX(LINE,'DIFF').NE.0 .AND.
     *          INDEX(LINE,'FORM').NE.0) THEN
C..Number of finite difference points
          READ(IUNIT,ERR=99,END=99,FMT=*) NDPOINT
          GOTO 10
        ELSE IF(INDEX(LINE,'TAMM').NE.0 .OR.
     *          INDEX(LINE,'DIAGO').NE.0 .OR.
     *          INDEX(LINE,'STATE').NE.0 ) THEN
          WRITE(6,*) " INPUT HAS CHANGED "
          WRITE(6,*) " IF YOU WANT TO PERFORM A TDDFT CALCULATION"
          WRITE(6,*) " USE THE &TDDFT &END SECTION"
          CALL STOPGM("LR_IN","SEE FILE LR_IN")
        ELSE 
C..Dummy line
          GOTO 10
        ENDIF
C     ==--------------------------------------------------------------==
   99 CONTINUE
      WRITE(6,*) ' LR_IN  : ERROR IN READING INPUT FILE'
      WRITE(6,*) LINE
      CALL STOPGM('LR_IN',' ')
C     ==--------------------------------------------------------------==
  100 CONTINUE
C     ==--------------------------------------------------------------==
C     ==  DETERMINE GAUGE                                             ==
C     ==--------------------------------------------------------------==
      TGAUGE_ALL=.FALSE.
      IF(IGAUGE.EQ.0) THEN      ! Default
        TPARA_GAUGE=(.NOT.TLSE)
      ELSE IF(IGAUGE.EQ.1) THEN ! Parallel transport
        TPARA_GAUGE=.TRUE.
        IF(TLSE) THEN
          WRITE(6,*) 'WARNING! Enforcing parallel-transport gauge ',
     $          'with ROKS'
          WRITE(6,*) 'Results may be wrong!'
        ENDIF
      ELSE IF(IGAUGE.EQ.2) THEN ! General between blocks
        TPARA_GAUGE=.FALSE.
        TGAUGE_ALL=(.NOT.TLSE)
        IF(.NOT.TLSE) THEN
          WRITE(6,*) 'WARNING! Enforcing non-parallel-transport gauge'
          WRITE(6,*) 'Convergence may be slower!'
        ENDIF
      ELSE IF(IGAUGE.EQ.3) THEN ! General for all orbitals
        TPARA_GAUGE=.FALSE.
        TGAUGE_ALL=.TRUE.
        WRITE(6,*) 'WARNING! Enforcing non-parallel-transport gauge ',
     $       'for all orbitals'
        WRITE(6,*) 'Convergence may be slower!'
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  WRITE INFO TO OUTPUT                                        ==
C     ==--------------------------------------------------------------==
      WRITE(6,'(/,A,A)')
     *     ' *********************** LINEAR RESPONSE',
     *     ' ************************'
      IF(TXC_DD_ANA) THEN
        IF(TXC_ANALYTIC) THEN
          WRITE(6,'(A)') 
     *        ' Analytic second derivatives of GC XC functionals'
          WRITE(6,'(A)') 
     *        ' Analytic third derivatives of LDA XC functionals'
        ELSE
          WRITE(6,'(A)') 
     *        ' Analytic second derivatives of GC XC functionals'
          IF(TDDFT.AND.(MD.OR.GEOPT.OR.TDEBFOR.OR.VIBRAT)) THEN
            WRITE(6,'(A)')
     *          ' Numeric third derivatives of GC XC functionals'
            WRITE(6,'(A,T56,E10.3)') 
     *          ' Step size for numeric dmu/dn :',XC_EPS
            WRITE(6,'(A,T56,I10)') 
     *          ' Number of calculations for dmu/dn :',NDPOINT
          ENDIF
        ENDIF
      ELSE
        IF(TXC_ANALYTIC) THEN
          WRITE(6,'(A)') 
     *          ' Analytic second derivatives of LDA XC functionals'
        ELSE
          WRITE(6,'(A,T56,E10.3)') 
     *          ' Step size for numeric dmu/dn :',XC_EPS
          WRITE(6,'(A,T56,I10)') 
     *          ' Number of calculations for dmu/dn :',NDPOINT
        ENDIF
      ENDIF
      WRITE(6,'(A,T58,I8)') 
     *              ' Maximum number of optimisation steps:',NMAXLR
      WRITE(6,'(A,T58,F8.4)') 
     *              ' Threshold for Hessian (Preconditioner)',LR_HTHRS
      IF(LOPTI.EQ.0) THEN
            WRITE(6,'(A,T56,A10)') 
     *      ' Optimizer for LR equations ',' AUTOMATIC'
            WRITE(6,'(T20,A,T60,I6)') ' Size of ODIIS buffer ',mdiis
            WRITE(6,'(T20,A,T60,I6)') ' Size of ZDIIS buffer ',mldiis
            WRITE(6,'(T20,A,T56,E10.4)') 
     *      ' Switch from PCG to ODIIS at ',THAUTO(1)
            WRITE(6,'(T20,A,T56,E10.4)') 
     *      ' Switch to full preconditioning at ',THAUTO(2)
      ENDIF
      IF(LOPTI.EQ.1) 
     *      WRITE(6,'(A,T56,A10)') 
     *      ' Optimizer for LR equations ','        SD'
      IF(LOPTI.EQ.2) 
     *      WRITE(6,'(A,T56,A10)') 
     *      ' Optimizer for LR equations ','       PCG'
      IF(LOPTI.EQ.2.AND..NOT.TXC_ANALYTIC) 
     *      WRITE(6,'(A,T56,E10.4)') 
     *      ' Tolerance for quadratic search ',tol_qs 
      IF(LOPTI.EQ.3) 
     *      WRITE(6,'(A,T56,A10)') 
     *      ' Optimizer for LR equations ','      DIIS'
      WRITE(6,'(A,T56,F10.4)') 
     *      ' Step length',DLRS
      WRITE(6,'(A,T56,E10.4)') ' Convergence criteria ',TOL_LR 
        IF(TPARA_GAUGE) THEN
          WRITE(6,*) 'Parallel-transport gauge for response functions'
        ELSE
          IF(TGAUGE_ALL) THEN
            WRITE(6,*) 'Generic gauge for all response functions'
          ELSE
            WRITE(6,*) 'Generic gauge for involved response functions'
          ENDIF
        ENDIF
      WRITE(6,'(A,A)')
     *     ' ***************************************',
     *     '*************************'
C     ==--------------------------------------------------------------==
 9999 CONTINUE
#ifdef PARALLEL
      REA1=0.D0
      INT1=0
      LOG1=.FALSE.
      MSGLEN = LR_REAL * 8
      CALL MY_BCAST(REA1,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = LR_INT * 8/IRAT
      CALL MY_BCAST(INT1,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = LR_LOG * 8/IRAT
      CALL MY_BCAST(LOG1,MSGLEN,SOURCE,ALLGRP)
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#ifdef __SR8000
*option MP(P(0)), LANGLVL(SAVE(0))
#endif
C     ==================================================================
      SUBROUTINE TDDFT_INPUT
C     ==--------------------------------------------------------------==
C     ==  THIS ROUTINE READS THE SECTION &TDDFS &END ON UNIT IUNIT    ==
C     ==--------------------------------------------------------------==
C     ==  THE INPUT HAS TO BE IN THE FOLLOWING FORMAT                 ==
C     ==     &TDDFT                                                   ==
C     ==        Options                                               ==
C     ==     &END                                                     ==
C     ==--------------------------------------------------------------==
C     ==  LIST OF OPTIONS                                             ==
C     ==                                                              ==
C     ==  TD_METHOD_A  [functional]                                   ==
C     ==  DIAGONALIZER [DAVIDSON,.......] :Diagonalizer for Spectra   ==
C     ==  DAVIDSON PARAMETER                                          ==
C     ==   ndavmax,epstdav,ndavspace                                  ==
C     ==  DAVIDSON RDIIS                                              ==
C     ==   nrdiismax,nrestdmax,rdiistin                               ==
C     ==  PCG PARAMETER                                               ==
C     ==   pcgiter,epstdav                                            ==
C     ==   pcgiter,epstdav,pcgstep                                    ==
C     ==  ROTATION PARAMETER                                          ==
C     ==   rotit,epsrot,rotstep                                       ==
C     ==  STATES [MIXED,SINGLET,TRIPLET]                              ==
C     ==    ns                                                        ==
C     ==  LOCALIZATION                                                ==
C     ==  MOLECULAR STATES                                            ==
C     ==  REORDER                                                     ==
C     ==   nreor                                                      ==
C     ==    i1 i2 ... in                                              ==
C     ==  REORDER LOCAL                                               ==
C     ==   zatom                                                      ==
C     ==    i1 i2 ... in                                              ==
C     ==  RANDOMIZE                                                   ==
C     ==   tdrandom                                                   ==
C     ==  LR-TDDFT                                                    ==
C     ==  TAMM-DANCOFF [SUBSPACE,OPTIMIZE]                            ==
C     ==   [msubta]                                                   ==
C     ==  FORCE STATE                                                 ==
C     ==   fstate                                                     ==
C     ==  OSCILLATOR STRENGTH options: compute oscillator strengths,  ==
C     ==     options:QP,BF,BFH,BERRY   if QP only for KS states       ==
C     ==                               (i.e. no TDDFT). BF uses       ==
C     ==                               Bessel function integration for==
C     ==                               the transition dipoles. BFH    ==
C     ==                               is the Hutter-Bessel option.   ==
C     ==                               lberry uses generalised Berry  ==
C     ==                               phase formula.                 ==
C     ==  PROPERTY  [STATE]                                           ==
C     ==   npstate                                                    ==
C     ==  OUTPUT {FULL,NORMAL,MINIMAL}                                ==
C     ==  LANDAU-ZENER [DIAG]                                         ==
C     ==  T-SHTDDFT                                                   ==
C     ==  EXTPOT                                                      ==
C     ==   aampl,adir,afreq,apara1                                    ==
C     ==  OUTPUT {FULL,NORMAL,MINIMAL}                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'tpot.inc'
      INCLUDE 'linres.inc'
      INTEGER   IUNIT,IERR,INSCAN,NSS,NN
      CHARACTER LINE*80
      INTEGER   MSGLEN,I
      DIMENSION IREOR(*)
C     ==--------------------------------------------------------------==
      IF(.NOT.PARENT) GOTO 9999
      IUNIT = 5
C     ==--------------------------------------------------------------==
C     ==  DEFAULTS                                                    ==
C     ==--------------------------------------------------------------==
      TDA=.TRUE.
      TDACANON=.FALSE.
      NS_MIX=0
      NS_SIN=0
      NS_TRI=0
      LDIAG=1
      NDAVMAX=100
      EPSTDAV=-1.D0
      NDAVSPACE=10
      TDRANDOM=-1.D0
      MSUBTA=0
      TDPCGMIN=.FALSE.
      TDPCGITER=100
      TDPCGSTEP=0.5D0
      EPSROT=1.D-6
      ROTIT=50
      ROTSTEP=0.5D0
      FSTATE=1
      TDLOCAL=.FALSE.
      TREORDER=.FALSE.
      MOLSTAT=.FALSE.
      TPROP=.FALSE.
      NPSTATE=0
      NREOR=0
      ZATOM=0
      NTDIISMAX=20
      NRESTDMAX=3
      RDIISTIN=1.D-3
      los    = .false.
      lqp    = .false.
      lbf    = .false.
      lbfh   = .false.
      lberry = .false.
      TD_METHOD=0
      KS_FUNCTIONAL=0
      TD_FUNCTIONAL=0
      KXC_FUNCTIONAL=0
      IOUTPUT=0
C..TSH[
      TDLZ=.FALSE.
      TDTULLY=.FALSE.
      SH_DIAG=.FALSE.
      TDEXTPOT=.FALSE.
C..TSH]
C     ==--------------------------------------------------------------==
      IERR=INSCAN(IUNIT,'&TDDFT')
      IF(IERR.NE.0) GOTO 100
C     ==--------------------------------------------------------------==
   10 CONTINUE
        READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
        IF(INDEX(LINE,'&END').NE.0) GOTO 100
        IF(INDEX(LINE,'DIAGO').NE.0) THEN
C..Diagonalizer
          IF(INDEX(LINE,'NONHERMIT').NE.0 .AND.
     *       INDEX(LINE,'DAVIDSON').NE.0) THEN
            LDIAG=2
          ELSEIF(INDEX(LINE,'DAVIDSON').NE.0) THEN
            LDIAG=1
          ELSEIF(INDEX(LINE,'PCG').NE.0) THEN
            IF(INDEX(LINE,'MINIM').NE.0) TDPCGMIN=.TRUE.
            LDIAG=3
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'DAVIDSON').NE.0 .AND.
     *          INDEX(LINE,'PARAM').NE.0) THEN
C..Davidson parameters
          READ(IUNIT,ERR=99,END=99,FMT=*) NDAVMAX,EPSTDAV,NDAVSPACE
          GOTO 10
        ELSE IF(INDEX(LINE,'DAVIDSON').NE.0 .AND.
     *          INDEX(LINE,'RDIIS').NE.0) THEN
C..Davidson (RDIIS) parameters
          READ(IUNIT,ERR=99,END=99,FMT=*) NTDIISMAX,NRESTDMAX,RDIISTIN
          GOTO 10
        ELSE IF(INDEX(LINE,'PCG').NE.0 .AND.
     *          INDEX(LINE,'PARAM').NE.0) THEN
C..PCG parameters
          IF(PCGMIN) THEN
            READ(IUNIT,ERR=99,END=99,FMT=*) TDPCGITER,EPSTDAV
          ELSE
            READ(IUNIT,ERR=99,END=99,FMT=*) TDPCGITER,EPSTDAV,TDPCGSTEP
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'MOLECULAR').NE.0 .AND.
     *          INDEX(LINE,'STATES').NE.0) THEN
C..Use molecular states for statistical potential mixing
          MOLSTAT=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'STATE').NE.0 .AND.
     *          INDEX(LINE,'FORCE').NE.0) THEN
C..State to calculate force from
          READ(IUNIT,ERR=99,END=99,FMT=*) FSTATE
          GOTO 10
        ELSE IF(INDEX(LINE,'STATE').NE.0) THEN
          READ(IUNIT,ERR=99,END=99,FMT=*) NSS
          IF(INDEX(LINE,'MIXED').NE.0) NS_MIX=NSS
          IF(INDEX(LINE,'SINGL').NE.0) NS_SIN=NSS
          IF(INDEX(LINE,'TRIPL').NE.0) NS_TRI=NSS
          GOTO 10
        ELSE IF(INDEX(LINE,'ROTATION').NE.0 .AND.
     *          INDEX(LINE,'PARAM').NE.0) THEN
C..Subspace rotation parameters
          READ(IUNIT,ERR=99,END=99,FMT=*) ROTIT,EPSROT,ROTSTEP
          GOTO 10
        ELSE IF(INDEX(LINE,'RANDOMIZE').NE.0) THEN
C..Randomize parameter
          READ(IUNIT,ERR=99,END=99,FMT=*) TDRANDOM
          GOTO 10
        ELSE IF(INDEX(LINE,'REORDER').NE.0) THEN
C..Reorder states
          TREORDER=.TRUE.
          IF(INDEX(LINE,'LOCAL').NE.0) THEN
            TDLOCAL=.TRUE.
            READ(IUNIT,ERR=99,END=99,FMT=*) ZATOM
            CALL MEMORY(IP_IREOR,ZATOM,"IREOR")
            READ(IUNIT,ERR=99,END=99,FMT=*) (IREOR(I),I=1,ZATOM)
          ELSE
            READ(IUNIT,ERR=99,END=99,FMT=*) NREOR
            CALL MEMORY(IP_IREOR,NREOR,"IREOR")
            READ(IUNIT,ERR=99,END=99,FMT=*) (IREOR(I),I=1,NREOR)
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'LOCAL').NE.0) THEN
C..localize states
          TDLOCAL=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'PROPERTY').NE.0) THEN
C..Property calculation
          TPROP=.TRUE.
          IF(INDEX(LINE,'STATE').NE.0) THEN
            READ(IUNIT,ERR=99,END=99,FMT=*) NPSTATE
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'TD_METHOD_A').NE.0) THEN
C..Advanced TD Methods
          TD_METHOD=1
          IF(INDEX(LINE,'LDA').NE.0) THEN
             TD_FUNCTIONAL=1
          ELSEIF(INDEX(LINE,'BP').NE.0) THEN
             TD_FUNCTIONAL=2
          ELSEIF(INDEX(LINE,'PBE0').NE.0) THEN
             TD_FUNCTIONAL=20
          ELSEIF(INDEX(LINE,'PBE').NE.0) THEN
             TD_FUNCTIONAL=3
          ELSEIF(INDEX(LINE,'BLYP').NE.0) THEN
             TD_FUNCTIONAL=4
          ELSEIF(INDEX(LINE,'OLYP').NE.0) THEN
             TD_FUNCTIONAL=5
          ELSEIF(INDEX(LINE,'HCTH').NE.0) THEN
             TD_FUNCTIONAL=6
          ELSEIF(INDEX(LINE,'SAOP').NE.0) THEN
             TD_FUNCTIONAL=10
          ELSEIF(INDEX(LINE,'LB94').NE.0) THEN
             TD_FUNCTIONAL=11
          ELSEIF(INDEX(LINE,'GLLB').NE.0) THEN
             TD_FUNCTIONAL=12
          ELSEIF(INDEX(LINE,'B3LYP').NE.0) THEN
             TD_FUNCTIONAL=21
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'LR-TDDFT').NE.0) THEN
C..Full LR-TDDFT
          TDA=.FALSE.
          GOTO 10
        ELSE IF(INDEX(LINE,'TAMM').NE.0 .AND.
     *          INDEX(LINE,'DANCOFF').NE.0) THEN
C..Tamm-Dancoff Approximation
          TDA=.TRUE.
          IF(INDEX(LINE,'OPTIM').NE.0) THEN
            TDACANON=.FALSE.
          ELSE
            TDACANON=.TRUE.
          ENDIF
          IF(INDEX(LINE,'SUBSPACE').NE.0) THEN
            READ(IUNIT,ERR=99,END=99,FMT=*) MSUBTA
          ELSE
            MSUBTA=-1
          ENDIF
          GOTO 10
        ELSE IF(INDEX(LINE,'OSCILLATOR').NE.0 .AND.
     *          INDEX(LINE,'STRENGTH').NE.0) THEN
C..Oscillator strengths
                los = .true.
                if ((index(line,'QP').ne.0)) lqp = .true.
                if ((index(line,'BF').ne.0)) lbf = .true.
                if ((index(line,'BFH').ne.0)) then
                  lbfh = .true.
                  lbf  = .false.
                end if
                if ((index(line,'BERRY').ne.0)) then
                  lberry = .true.
                  lbfh   = .false.
                  lbf    = .false.
                  lqp    = .false.
                end if
             GOTO 10
        ELSE IF(INDEX(LINE,'OUTPUT').NE.0) THEN
          IF(INDEX(LINE,'NORMAL').NE.0) IOUTPUT=0
          IF(INDEX(LINE,'FULL').NE.0) IOUTPUT=1
          IF(INDEX(LINE,'MINIMAL').NE.0) IOUTPUT=-1
          GOTO 10
        ELSE IF(INDEX(LINE,'LZ-SHTDDFT').NE.0) THEN
CLZ Landau-Zener surface hopping sheme
          TDLZ=.TRUE.
          IF(INDEX(LINE,'DIAG').NE.0) SH_DIAG=.TRUE.
          GOTO 10
        ELSE IF (INDEX(LINE,'T-SHTDDFT').NE.0 ) then
C..TSH[
           TDTULLY=.TRUE.
           SH_DIAG=.FALSE.
           GOTO 10
        ELSE IF (INDEX(LINE,'EXTPOT').NE.0 ) then
           TDEXTPOT=.TRUE.
           READ(IUNIT,ERR=99,END=99,FMT=*) 
     &     AAMPL,ADIR,AFREQ,APARA1
           GOTO 10
C..TSH]
        ELSE
C..Dummy line
          GOTO 10
        ENDIF
C     ==--------------------------------------------------------------==
   99 CONTINUE
      WRITE(6,*) ' TDDFT_INPUT  : ERROR IN READING INPUT FILE'
      WRITE(6,*) LINE
      CALL STOPGM('TDDFT_INPUT',' ')
C     ==--------------------------------------------------------------==
  100 CONTINUE
      IF(NS_MIX+NS_SIN+NS_TRI.EQ.0) THEN
        IF(TLSD) THEN
          NS_MIX=1
        ELSE
          NS_SIN=1
        ENDIF
      ENDIF
      IF(TLSD.AND.NS_MIX.EQ.0) THEN
        NS_MIX=1
        NS_SIN=0
        NS_TRI=0
      ENDIF
      IF(.NOT.TLSD.AND.NS_SIN+NS_TRI.EQ.0) THEN
        NS_MIX=0
        NS_SIN=1
        NS_TRI=0
      ENDIF
      IF(TDA .AND. MSUBTA.GT.0) THEN
        IF(.NOT.TDACANON) LDIAG=3
      ENDIF
      IF(.NOT.TDA .AND. LDIAG.EQ.1) LDIAG=2
      IF(EPSTDAV.LT.0.D0) THEN
        IF(LDIAG.EQ.1) EPSTDAV=1.D-8
        IF(LDIAG.EQ.2) EPSTDAV=1.D-6
        IF(LDIAG.EQ.3) EPSTDAV=1.D-7
      ENDIF
      IF(TD_METHOD.NE.0) THEN
        CALL TDM_FUN(KS_FUNCTIONAL,-1)
        CALL SWITCH_FUNCTIONALS
        CALL TDM_FUN(KXC_FUNCTIONAL,-1)
        CALL SWITCH_FUNCTIONALS
      ENDIF
C..TSH[
      NROOT_SH=MAX(NS_SIN,NS_TRI,NS_MIX,2)
C..TSH]
C     ==--------------------------------------------------------------==
C     ==  WRITE INFO TO OUTPUT                                        ==
C     ==--------------------------------------------------------------==
      WRITE(6,'(/,A,A)')
     *     ' ***************************  TDDFT ',
     *     ' ****************************'
      IF(TD_METHOD.EQ.1) THEN
        WRITE(6,'(A)') ' Use Time-Dependent DFT Perturbation Method A'
        IF(TD_FUNCTIONAL.EQ.1) THEN
          WRITE(6,'(T34,A,T58,A)') ' STATE FUNCTIONAL','     LDA'
        ELSEIF(TD_FUNCTIONAL.EQ.2) THEN
          WRITE(6,'(T34,A,T58,A)') ' STATE FUNCTIONAL','      BP'
        ELSEIF(TD_FUNCTIONAL.EQ.3) THEN
          WRITE(6,'(T34,A,T58,A)') ' STATE FUNCTIONAL','     PBE'
        ELSEIF(TD_FUNCTIONAL.EQ.4) THEN
          WRITE(6,'(T34,A,T58,A)') ' STATE FUNCTIONAL','    BLYP'
        ELSEIF(TD_FUNCTIONAL.EQ.5) THEN
          WRITE(6,'(T34,A,T58,A)') ' STATE FUNCTIONAL','    OLYP'
        ELSEIF(TD_FUNCTIONAL.EQ.6) THEN
          WRITE(6,'(T34,A,T58,A)') ' STATE FUNCTIONAL','    HCTH'
        ELSEIF(TD_FUNCTIONAL.EQ.10) THEN
          WRITE(6,'(T34,A,T58,A)') ' STATE FUNCTIONAL','    SAOP'
          IF(MOLSTAT) THEN
            WRITE(6,'(T45,A)') 'WITH MOLECULAR STATES'
          ENDIF
        ELSEIF(TD_FUNCTIONAL.EQ.11) THEN
          WRITE(6,'(T34,A,T58,A)') ' STATE FUNCTIONAL','    LB94'
        ELSEIF(TD_FUNCTIONAL.EQ.12) THEN
          WRITE(6,'(T34,A,T58,A)') ' STATE FUNCTIONAL','    GLLB'
        ELSEIF(TD_FUNCTIONAL.EQ.20) THEN
          WRITE(6,'(T34,A,T58,A)') ' STATE FUNCTIONAL','    PBE0'
        ELSEIF(TD_FUNCTIONAL.EQ.21) THEN
          WRITE(6,'(T34,A,T58,A)') ' STATE FUNCTIONAL','   B3LYP'
        ELSE
          CALL STOPGM("TD_METHOD","UNSUPPORTED STATE FUNCTIONAL")
        ENDIF
        IF(KXC_FUNCTIONAL.EQ.0) THEN
          WRITE(6,'(T34,A,T58,A)') ' KERNEL FUNCTIONAL','    NONE'
        ELSEIF(KXC_FUNCTIONAL.EQ.1) THEN
          WRITE(6,'(T34,A,T58,A)') ' KERNEL FUNCTIONAL','     LDA'
        ELSEIF(KXC_FUNCTIONAL.EQ.2) THEN
          WRITE(6,'(T34,A,T58,A)') ' KERNEL FUNCTIONAL','      BP'
        ELSEIF(KXC_FUNCTIONAL.EQ.3) THEN
          WRITE(6,'(T34,A,T58,A)') ' KERNEL FUNCTIONAL','     PBE'
        ELSEIF(KXC_FUNCTIONAL.EQ.4) THEN
          WRITE(6,'(T34,A,T58,A)') ' KERNEL FUNCTIONAL','    BLYP'
        ELSEIF(KXC_FUNCTIONAL.EQ.5) THEN
          WRITE(6,'(T34,A,T58,A)') ' KERNEL FUNCTIONAL','    OLYP'
        ELSEIF(KXC_FUNCTIONAL.EQ.6) THEN
          WRITE(6,'(T34,A,T58,A)') ' KERNEL FUNCTIONAL','    HCTH'
        ELSEIF(KXC_FUNCTIONAL.EQ.10) THEN
          WRITE(6,'(T34,A,T58,A)') ' KERNEL FUNCTIONAL','    SAOP'
        ELSEIF(KXC_FUNCTIONAL.EQ.20) THEN
          WRITE(6,'(T34,A,T58,A)') ' KERNEL FUNCTIONAL','    PBE0'
        ELSEIF(KXC_FUNCTIONAL.EQ.21) THEN
          WRITE(6,'(T34,A,T58,A)') ' KERNEL FUNCTIONAL','   B3LYP'
        ELSE
          CALL STOPGM("TD_METHOD","UNSUPPORTED KERNEL FUNCTIONAL")
        ENDIF
      ENDIF
      IF(TXC_DD_ANA) THEN
        WRITE(6,'(A)') 
     *        ' Analytic second derivatives of GC XC functionals'
      ELSEIF(TXC_ANALYTIC) THEN
        WRITE(6,'(A)') 
     *        ' Analytic second derivatives of LDA XC functionals'
      ELSE
        WRITE(6,'(A,T56,E10.3)') 
     *        ' Step size for numeric dmu/dn :',XC_EPS
        WRITE(6,'(A,T56,I10)') 
     *        ' Number of calculations for dmu/dn :',NDPOINT
      ENDIF
      IF(TDA) THEN
        IF(MSUBTA.GT.0) THEN
          WRITE(6,'(A,T56,I10)') 
     *      ' Tamm-Dancoff Approximation (Subspace) ',MSUBTA
          IF(TDACANON) THEN
            WRITE(6,'(T36,A)') ' Fixed Subspace Approximation'
          ELSE
            WRITE(6,'(T36,A)') ' Optimized Subspace Approximation'
          ENDIF
        ELSE
          WRITE(6,'(A)') ' Tamm-Dancoff Approximation '
        ENDIF
      ENDIF
      IF(LDIAG.EQ.1) THEN
        WRITE(6,'(A,T56,A10)') 
     *      ' Diagonalization Method ','  DAVIDSON'
        WRITE(6,'(T10,A,T56,I10)') 
     *           ' Max. number of iterations',NDAVMAX      
        WRITE(6,'(T10,A,T56,E10.3)') ' Convergence criteria',EPSTDAV      
        WRITE(6,'(T10,A,T56,I10)') ' Max. size of Davidson matrix',
     *     NDAVSPACE    
        WRITE(6,'(T10,A)') ' RDIIS Parameters'
        WRITE(6,'(T10,A,I4,T30,A,I4,T46,A,E10.3)') 
     *   ' NTDIISMAX=',NTDIISMAX,' NRESTDMAX=',NRESTDMAX,
     *   ' RDIISTIN=',RDIISTIN
      ELSEIF(LDIAG.EQ.2) THEN
        WRITE(6,'(A,T42,A24)') 
     *      ' Diagonalization Method ','  NON-HERMITIAN DAVIDSON'
        WRITE(6,'(T10,A,T56,I10)') 
     *           ' Max. number of iterations',NDAVMAX      
        WRITE(6,'(T10,A,T56,E10.3)') ' Convergence criteria',EPSTDAV      
        WRITE(6,'(T10,A,T56,I10)') ' Max. size of Davidson matrix',
     *     NDAVSPACE    
      ELSEIF(LDIAG.EQ.3) THEN
        WRITE(6,'(A,T42,A24)') 
     *      ' Diagonalization Method ','    CONJUGATED GRADIENTS'
        WRITE(6,'(T10,A,T56,I10)') 
     *      ' Max. number of iterations',TDPCGITER      
        IF(TDPCGMIN) THEN
          WRITE(6,'(T10,A)') ' Line minimization'
        ELSE
          WRITE(6,'(T10,A,T56,E10.3)') ' Fixed step size',TDPCGSTEP      
        ENDIF
        WRITE(6,'(T10,A,T56,E10.3)') ' Convergence criteria',EPSTDAV      
      ENDIF
      IF(TDA.AND.MSUBTA.GT.0.AND..NOT.TDACANON) THEN
        WRITE(6,'(A)') ' Subspace Rotation by Exponentials'      
        WRITE(6,'(T10,A,T56,I10)') ' Max. number of iterations',ROTIT
        WRITE(6,'(T10,A,T56,E10.3)') ' Convergence criteria',EPSROT
        WRITE(6,'(T10,A,T56,E10.3)') ' Fixed step size',ROTSTEP      
      ENDIF
      IF(NS_SIN.GT.0) 
     *   WRITE(6,'(A,T56,I10)') ' Number of Singlet States ',NS_SIN
      IF(NS_TRI.GT.0) 
     *   WRITE(6,'(A,T56,I10)') ' Number of Triplet States ',NS_TRI
      IF(NS_MIX.GT.0) 
     *   WRITE(6,'(A,T56,I10)') ' Number of Mixed States ',NS_MIX
      WRITE(6,'(A,T56,I10)') ' Forces calculated for state ',FSTATE
      IF(TDRANDOM.GT.0.D0) THEN
        WRITE(6,'(A,T51,E15.6)') 
     *      ' Randomize Initial Guess (Amplitude)',TDRANDOM
      ENDIF
      IF(TDLOCAL) THEN
        WRITE(6,'(A)') ' Localize ground state orbitals'
      ENDIF
      IF(TREORDER) THEN
        WRITE(6,'(A)') ' Reorder ground state orbitals'
        IF(ZATOM.NE.0) THEN
          WRITE(6,'(T40,A)') "w.r.t. distance to atoms"
          WRITE(6,'(T20,10I5)') (IREOR(I),I=1,ZATOM)
        ELSE
          WRITE(6,'(T20,10I5)') (IREOR(I),I=1,NREOR)
        ENDIF
      ENDIF
      IF(LOS) THEN
        IF(.NOT.LQP) THEN
          WRITE(6,'(A)') ' Compute TDDFT oscillator strengths'
          IF(LBERRY) WRITE(6,'(A)') '      Use Berry phase formula'
        ELSE
          WRITE(6,'(A)')
     >' Compute only Kohn-Sham oscillator strengths (no TDDFT)'
        END IF
      END IF
      IF(TPROP) THEN
        IF(NPSTATE.EQ.0) THEN
          WRITE(6,'(A)') 
     *           ' Compute excited state properties for all states'
        ELSE
          WRITE(6,'(A,T63,I3)') 
     *           ' Compute excited state properties for state',NPSTATE
        ENDIF
      ENDIF
      IF(TDLZ) THEN
        WRITE(6,'(A)')
     *           ' Landau-Zener surface hopping TDDFT dynamics'
      ENDIF
C..TSH[
      IF (TDTULLY) then
        WRITE(6,'(A)') ' TDDFT FEWEST SWITCHES: Tullys surface hopping'
      ENDIF
C..TSH]
      WRITE(6,'(A,A)')
     *     ' ***************************************',
     *     '*************************'
C     ==--------------------------------------------------------------==
 9999 CONTINUE
#ifdef PARALLEL
      REA2=0.d0
      INT2=0
      LOG2=.false.
      MSGLEN = TD_REAL * 8
      CALL MY_BCAST(REA2,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = TD_INT * 8/IRAT
      CALL MY_BCAST(INT2,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = TD_LOG * 8/IRAT
      CALL MY_BCAST(LOG2,MSGLEN,SOURCE,ALLGRP)
      IF(TREORDER) THEN
        IF(.NOT.PARENT) THEN
            NN=MAX(ZATOM,NREOR)
            CALL MEMORY(IP_IREOR,NN,"IREOR")
        ENDIF
        MSGLEN = NREOR * 8/IRAT
        CALL MY_BCAST(IREOR,MSGLEN,SOURCE,ALLGRP)
      ENDIF
      MSGLEN = 4 * 8/IRAT
      CALL MY_BCAST(TD_METHOD,MSGLEN,SOURCE,ALLGRP)
C..TSH[
      MSGLEN= 1 * 8/IRAT
      CALL MY_BCAST(TDTULLY,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(TDEXTPOT,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(NROOT_SH,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(ADIR,MSGLEN,SOURCE,ALLGRP)
C..TSH]
#endif
C     ==--------------------------------------------------------------==
      IF(MOLSTAT) CALL MS_IN
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#ifdef __SR8000
*option MP(P(0)), LANGLVL(SAVE(0))
#endif
C     ==================================================================
      SUBROUTINE MS_IN
C     ==--------------------------------------------------------------==
C     ==  THIS ROUTINE READS THE SECTION &MOLSTATES &END ON UNIT IUNIT==
C     ==--------------------------------------------------------------==
C     ==  THE INPUT HAS TO BE IN THE FOLLOWING FORMAT                 ==
C     ==     &MOLSTATES                                               ==
C     ==        Options                                               ==
C     ==     &END                                                     ==
C     ==--------------------------------------------------------------==
C     ==  LIST OF OPTIONS                                             ==
C     ==                                                              ==
C     ==  MOLECULES                                                   ==
C     ==  n1 n2 n3 ... (all atom)                                     ==
C     ==  STATES                                                      ==
C     ==  na nb nc ... (all molecules)                                ==
C     ==                                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'mols.inc'
      INCLUDE 'adat.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'coor.inc'
      INTEGER   IUNIT,IERR,INSCAN,LEN,I,IAT,IS,IA,K
      CHARACTER LINE*80
      INTEGER   MSGLEN
C     ==--------------------------------------------------------------==
      LEN=NAT/IRAT + 2
      CALL MEMORY(IP_MSNUM,LEN,"MSNUM")
      CALL MEMORY(IP_MSTAT,LEN,"MSTAT")
      CALL IAZZERO(MSNUM,NAT)
      CALL IAZZERO(MSTAT,NAT)
      NUMOL=0
C     ==--------------------------------------------------------------==
      IF(.NOT.PARENT) GOTO 9999
      IUNIT = 5
C     ==--------------------------------------------------------------==
C     ==  DEFAULTS                                                    ==
C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
      IERR=INSCAN(IUNIT,'&MOLSTATES')
      IF(IERR.NE.0) GOTO 100
C     ==--------------------------------------------------------------==
   10 CONTINUE
        READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
        IF(INDEX(LINE,'&END').NE.0) GOTO 100
        IF(INDEX(LINE,'MOLECULES').NE.0) THEN
          READ(IUNIT,ERR=99,END=99,FMT=*) (MSNUM(I),I=1,NAT)
          DO I=1,NAT
            IF(NUMOL.LT.MSNUM(I)) NUMOL=MSNUM(I)
          ENDDO
          GOTO 10
        ELSE IF(INDEX(LINE,'STATES').NE.0) THEN
          IF(NUMOL.LE.0) CALL STOPGM("MS_IN","MOLECULES NOT DEFINED")
          READ(IUNIT,ERR=99,END=99,FMT=*) (MSTAT(I),I=1,NUMOL)
          GOTO 10
        ELSE 
C..Dummy line
          GOTO 10
        ENDIF
C     ==--------------------------------------------------------------==
   99 CONTINUE
      WRITE(6,*) ' MS_IN  : ERROR IN READING INPUT FILE'
      WRITE(6,*) LINE
      CALL STOPGM('MS_IN',' ')
C     ==--------------------------------------------------------------==
  100 CONTINUE
C     ==--------------------------------------------------------------==
C     ==  WRITE INFO TO OUTPUT                                        ==
C     ==--------------------------------------------------------------==
      WRITE(6,'(/,A,A)')
     *     ' *********************** MOLECULAR STATES',
     *     ' ***********************'
      DO I=1,NUMOL
        WRITE(6,'(A,I5,T52,A,I5)') '  MOLECULE :',I,'STATES :',MSTAT(I)
        IAT=0
        DO IS=1,NSP
          DO IA=1,NA(IS)
            IAT=IAT+1
            IF(I.EQ.MSNUM(IAT)) THEN
              WRITE(6,'(T6,I4,4X,A2,4X,3(F15.6))')
     *             IAT,EL(IATYP(IS)),(TAU0(K,IA,IS),K=1,3)
            ENDIF
          ENDDO
        ENDDO
      ENDDO
      WRITE(6,'(A,A)')
     *     ' ***************************************',
     *     '*************************'
C     ==--------------------------------------------------------------==
 9999 CONTINUE
#ifdef PARALLEL
      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(NUMOL,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = NAT * 8/IRAT
      CALL MY_BCAST(MSNUM,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = NUMOL * 8/IRAT
      CALL MY_BCAST(MSTAT,MSGLEN,SOURCE,ALLGRP)
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
