C hello emacs, this is -*- fortran -*-
C     ==================================================================
C     == INPUT PARAMETERS FOR LINEAR SCALING OPTIMIZATION             ==
C     ==--------------------------------------------------------------==
C     == NREM     Number of remembered steps for L-BFGS               ==
C     ==          Default: 0 -> MIN(40,NDIM) at first step            ==
C     == NTRUST   Type of trust radius / line search                  ==
C     ==          Default: 0                                          ==
C     ==          0: HDLCopt trust radius algorithm                   ==
C     ==          1: Line search                                      ==
C     ==          2: CPMD optimizer trust radius algorithm            ==
C     == NRESTT   Periodic restart every NRESTT performed steps       ==
C     ==          Default: 0 (never)                                  ==
C     == NTRSTR   Number of performed TR steps before line search     ==
C     ==          Default: 0 (keep trust radius or line search)       ==
C     == TRUSTR   Maximum and initial trust radius                    ==
C     ==          Default: 0.5                                        ==
C     == See also BLOCK DATA LBFGS_INIT                               ==
C     ==--------------------------------------------------------------==
      INTEGER         NREM, NTRUST, NRESTT, NTRSTR
      COMMON /LSCALS/ NREM, NTRUST, NRESTT, NTRSTR
      REAL*8          STEP_BMB_L, STEP_INI_L, STEP_MAX_L, STEP_MIN_L,
     $                TRUSTR
      COMMON /LSCALO/ STEP_BMB_L, STEP_INI_L, STEP_MAX_L, STEP_MIN_L,
     $                TRUSTR
C
C     ==================================================================
C     == CONSTANTS FOR LINEAR SCALING OPTIMIZATION                    ==
C     ==--------------------------------------------------------------==
C     == BETAP_L, BETA_L: beta' and beta for Wolfe conditions         ==
C     ==--------------------------------------------------------------==
      REAL*8 BETAP_L, BETA_L
      PARAMETER (BETAP_L = 1.0D-4, BETA_L = 0.9D0)
C
C     ==================================================================
C     == SHARED INFORMATION FOR LINEAR SCALING OPTIMIZATION           ==
C     ==--------------------------------------------------------------==
      LOGICAL         LFEVMN, LINILS
      COMMON /LSCALL/ LFEVMN, LINILS
      INTEGER         IBND_L, IGPNT, INFO_L, IPNT_L, IPRNT, ISPNT,
     $                ITER_L, IWPNT, JUMP, LWLBFGS, NFEV_L,
     $                NFEVMN, NRES_L, NSTEP_L, NSTRIED_L, NPT_L
      COMMON /LSCALI/ IBND_L, IGPNT, INFO_L, IPNT_L, IPRNT, ISPNT,
     $                ITER_L, IWPNT, JUMP, LWLBFGS, NFEV_L,
     $                NFEVMN, NRES_L, NSTEP_L, NSTRIED_L, NPT_L
      REAL*8          DEACCP, ETOTMN, OLDENE, SCL1_L, SCL2_L, STEP_L,
     $                SZ_L, SZ_LMN
      COMMON /LSCALR/ DEACCP, ETOTMN, OLDENE, SCL1_L, SCL2_L, STEP_L,
     $                SZ_L, SZ_LMN
      REAL*8          WLBFGS(*)
      POINTER         (IP_WLBFGS,WLBFGS)
      COMMON /LSCALP/ IP_WLBFGS
C     ==================================================================
C
C     ==================================================================
C     == INPUT PARAMETERS FOR TRANSITION STATE SEARCH                 ==
C     ==--------------------------------------------------------------==
C     == MODE       Hessian eigenmode to be followed                  ==
C     == MODELK     0: Mode following, 1: mode locked                 ==
C     == NCORE      Number of atoms in the reaction core              ==
C     == NSMAXP     Maximum number of P-RFO steps                     ==
C     == NVAR       Number of variables in the core                   ==
C     == NSVIB      Vibrational analysis is done every NSVIB steps    ==
C     == ICORE      Atom sequence numbers in the reaction core        ==
C     == CUTEIG     Cutoff for Hessian eigenvalues                    ==
C     == DEMIN_P    Minimum energy change to nevertheless accept step ==
C     == OMIN       Minimum overlap between Hessian modes             ==
C     == RMAX_P     Maximum ratio actual / predicted energy change    ==
C     == RMIN_P     Minimum ratio actual / predicted energy change    ==
C     == STEP_INI_P Initial P-RFO trust radius                        ==
C     == STEP_MAX_P Maximum P-RFO trust radius                        ==
C     == STEP_MIN_P Minimum P-RFO trust radius                        ==
C     == TOLENV     Gradient tolerance for the environment            ==
C     == TRUSTP     User set P-RFO trust radius                       ==
C     == EPS_H      Step for finite-difference Hessian                ==
C     == M_HESS     Type of initial Hessian (0: INIHES, 1: HESSIN)    ==
C     ==--------------------------------------------------------------==
      INTEGER         MODE, MODELK, NCORE, NSMAXP, NVAR, NSVIB, M_HESS
      COMMON /PRFOPI/ MODE, MODELK, NCORE, NSMAXP, NVAR, NSVIB, M_HESS
      INTEGER         ICORE(*)
      POINTER         (IP_ICORE,ICORE)
      COMMON /PRFOPP/ IP_ICORE
      REAL*8          CUTEIG, DEMIN_P, OMIN, RMAX_P, RMIN_P, STEP_INI_P,
     $                STEP_MAX_P, STEP_MIN_P, TOLENV, TRUSTP, EPS_H
      COMMON /PRFOPR/ CUTEIG, DEMIN_P, OMIN, RMAX_P, RMIN_P, STEP_INI_P,
     $                STEP_MAX_P, STEP_MIN_P, TOLENV, TRUSTP, EPS_H
C
C     ==================================================================
C     == CONSTANTS FOR TRANSITION STATE SEARCH                        ==
C     == FSTEP_P: Step for determining the minimizing lambda in P-RFO ==
C     ==--------------------------------------------------------------==
      REAL*8    FSTEP_P
      PARAMETER (FSTEP_P = 5.0D-2)
C
C     ==================================================================
C     == SHARED INFORMATION FOR TRANSITION STATE SEARCH               ==
C     ==--------------------------------------------------------------==
      LOGICAL         LALLHS, LLOCK, LMICRO, LNOMAP, LPRHES, LRSTRT,
     $                LUNDOP, LVLHES
      COMMON /PRFOSL/ LALLHS, LLOCK, LMICRO, LNOMAP, LPRHES, LRSTRT,
     $                LUNDOP, LVLHES
      INTEGER         LMAP_P, LWPRFO, MODCUR, NSHESS, NSTEP_P, NSTRIED_P
      COMMON /PRFOSI/ LMAP_P, LWPRFO, MODCUR, NSHESS, NSTEP_P, NSTRIED_P
      REAL*8          DEPRED, OPRVEN, OSTEP_P, OSZ_P, PRVENE,
     $                STEP_P, SZ_P
      COMMON /PRFOSR/ DEPRED, OPRVEN, OSTEP_P, OSZ_P, PRVENE,
     $                STEP_P, SZ_P
      INTEGER         MAP_P(*)
      REAL*8          WPRFO(*) 
      POINTER         (IP_MAP_P,MAP_P),   (IP_WPRFO,WPRFO)
      REAL*8          HESSCR(NVAR,NVAR),  OLDHSS(NVAR,NVAR),
     $                VMODE(NVAR),        GX(NVAR),
     $                OLDGX(NVAR),        OLDG(NVAR),
     $                OOLDG(NVAR),        EIGVAL(NVAR),
     $                OLDEIG(NVAR),       EIGVEC(NVAR,NVAR),
     $                OLDEV(NVAR,NVAR),   STEP(NVAR)
      POINTER         (IP_HESSCR,HESSCR), (IP_OLDHSS,OLDHSS),
     $                (IP_VMODE,VMODE),   (IP_GX,GX),
     $                (IP_OLDGX,OLDGX),   (IP_OLDG,OLDG),
     $                (IP_OOLDG,OOLDG),   (IP_EIGVAL,EIGVAL),
     $                (IP_STEP,STEP),     (IP_OLDEIG,OLDEIG),
     $                (IP_EIGVEC,EIGVEC), (IP_OLDEV,OLDEV)
      COMMON /PRFOSP/ IP_MAP_P, IP_WPRFO,
     $                IP_HESSCR, IP_OLDHSS, IP_VMODE,  IP_GX,
     $                IP_OLDGX,  IP_OLDG,   IP_OOLDG,  IP_EIGVAL,
     $                IP_OLDEIG, IP_EIGVEC, IP_OLDEV,  IP_STEP
C
C     ==================================================================
C     == SHARED INFORMATION FOR STACKS                                ==
C     ==--------------------------------------------------------------==
      INTEGER         IELSTK, NSTACK,
     $                ST_NOP, ST_POP, ST_PUSH, ST_GET, ST_PUT
      PARAMETER       (IELSTK =  1, NSTACK  = 1,
     $                ST_NOP  =  0,
     $                ST_POP  = -2, ST_PUSH = 2,
     $                ST_GET  = -1, ST_PUT  = 1)
      INTEGER         IOPNXT(NSTACK), ISTLEN(NSTACK), ISZSTK(NSTACK),
     $                ITPSTK(NSTACK)
      COMMON /STKI/   IOPNXT, ISTLEN, ISZSTK, ITPSTK
      REAL*8          ELESTK(*)
      POINTER         (IP_ELESTK,ELESTK)
      COMMON /STKP/   IP_ELESTK
C
C     ==================================================================
C     == SHARED INFORMATION FOR ADAPTIVE TOLERANCE                    ==
C     ==--------------------------------------------------------------==
      LOGICAL         TNOFOR
      COMMON /ADTOLL/ TNOFOR
      INTEGER         NRLLST
      COMMON /ADTOLI/ NRLLST
      REAL*8          DELAST, DEMIN, GNMIN, TOLMIN, TOLREL
      COMMON /ADTOLR/ DELAST, DEMIN, GNMIN, TOLMIN, TOLREL
