C     ==================================================================
      SUBROUTINE LSD_ELF(C0,TAU0,RHOE,V,SCR,LSCR,NSTATE,NKPOINT)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     %% THE ELECTRON LOCALIZATION FUNCTION FOR THE UNRESTRICTED CASE %%
C     %% PLUS FOR ALPHA AND BETA SPIN INDIVIDUALLY                    %%
C     %% use keywords: LSD and ELF an the 3 fcts are calculated       %%
C     %% automatically                                                %%
C     %% formulas based on: M. Kohut and A.Savin, INT. J. Quant.      %%
C     %% Chem,60, 875-882 (1996), eq. (2) for alpha and beta spin,    %%
C     %% eq (9) for the spin unrestricted case                        %%
C     %% more information in the elf.F file                           %%
C     %% 17/03/00 last check: Kirchner                                %%
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'poin.inc'
      INCLUDE 'qspl.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'prden.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'pimd.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'bsym.inc'
      INCLUDE 'mw.inc'
C     Arguments
      INTEGER    LSCR,NSTATE,NKPOINT
      COMPLEX*16 C0(NGWK,NSTATE,NKPOINT),V(MAXFFT)
      REAL*8     TAU0(*),RHOE(NNR1,2),SCR(LSCR)
C     Variables
      DIMENSION  F(NSTATE,NKPTS)
      COMPLEX*16 VA(MAXFFT),VB(MAXFFT)
      COMPLEX*16 VTEMP(NHG),VTEMPA(NHG),VTEMPB(NHG)
      POINTER    (IP_VA,VA),(IP_VB,VB),(IP_VTEMP,VTEMP),
     &           (IP_VTEMPA,VTEMPA),(IP_VTEMPB,VTEMPB)
      REAL*8     ELFF(NNR1)
      REAL*8     ELFFA(NNR1),ELFFB(NNR1)
      POINTER    (IP_ELFF,ELFF),
     &           (IP_ELFFA,ELFFA),(IP_ELFFB,ELFFB)
      CHARACTER  CFLBOD*12,CFLBODA*12,CFLBODB*12
      CHARACTER  CIPNUM*12,CIPNUMA*12,CIPNUMB*12
      CHARACTER  FILEN*15,TAG*30
      CHARACTER  FILENA*15,FILENB*15 
      REAL*8     COEFE,RHOM,COEFS,AELF,AELFA,AELFB
      REAL*8     COEFE23,COEFE53,P53
      REAL*8     ELFMIN,ELFMINA,ELFMINB
      REAL*8     ELFMAX,ELFMAXA,ELFMAXB
      INTEGER    IKIND,LELF,IR,IG,I,K,N1,N2,I1,I2,SPACE
C     ==--------------------------------------------------------------==
      IF(PARENT) WRITE(6,'(/,A)') ' CALCULATE LSD_ELF FUNCTION '
      CALL GIVE_SCR_LSD_ELF(LELF,TAG,NSTATE)
      CALL TEST_SCR('LSD_ELF',TAG,LSCR,LELF)
C     ==--------------------------------------------------------------==
C     Allocations of arrays
      CALL MEMORY(IP_ELFF,NNR1,'ELFF')
      CALL MEMORY(IP_ELFFA,NNR1,'ELFFA')
      CALL MEMORY(IP_ELFFB,NNR1,'ELFFB')
      CALL MEMORY(IP_VA,2*MAXFFT,'VA')
      CALL MEMORY(IP_VB,2*MAXFFT,'VB')
      CALL MEMORY(IP_VTEMP, 2*NHG,'VTEMP')
      CALL MEMORY(IP_VTEMPA,2*NHG,'VTEMPA')
      CALL MEMORY(IP_VTEMPB,2*NHG,'VTEMPB')
C     ==--------------------------------------------------------------==
      CALL PHFAC(TAU0)
C
      DO IKIND=1,NKPOINT
        CALL RNLSM(C0(1,1,IKIND),NSTATE,V,SCR,LSCR,1,IKIND,.FALSE.)
      ENDDO
      IF(TKPNT) THEN
        CALL RHOOFR_C(C0,RHOE,V,SCR,LSCR,NSTATE)
      ELSE
        CALL RHOOFR(C0,RHOE,V,SCR,LSCR,NSTATE)
      ENDIF
C     alpha spin: rho_alpha(IR)=(RHOE(IR,1)-RHOE(IR,2)
      DO IR=1,NNR1
        VA(IR)=DCMPLX((RHOE(IR,1)-RHOE(IR,2)),0.D0)
      ENDDO
      CALL FWFFT(VA)
      DO IG=1,NHG
        VTEMPA(IG) = VA(NZH(IG))
      ENDDO
C     beta spin: rho_beta(IR)=RHOE(IR,2)
      DO IR=1,NNR1
        VB(IR)=DCMPLX(RHOE(IR,2),0.D0)
      ENDDO
      CALL FWFFT(VB)
      DO IG=1,NHG
        VTEMPB(IG) = VB(NZH(IG))
      ENDDO
C.....calculate |nabla*rhoa|^2 and |nabla*rhob|^2
      CALL ZAZZERO(VA,MAXFFT)
      CALL ZAZZERO(VB,MAXFFT)
C     |nabla*rhoa| and |nabla*rhob|
!CDIR NODEP
      DO IG=1,NHG
        VA(NZH(IG))=TPIBA*UIMAG*GK(1,IG)*VTEMPA(IG)
        VB(NZH(IG))=TPIBA*UIMAG*GK(1,IG)*VTEMPB(IG)
        VA(INDZ(IG))=-TPIBA*UIMAG*GK(1,IG)*DCONJG(VTEMPA(IG))
        VB(INDZ(IG))=-TPIBA*UIMAG*GK(1,IG)*DCONJG(VTEMPB(IG))
      ENDDO
      CALL INVFFT(VA)
      CALL INVFFT(VB)
C     |nabla*rhoa|^2 and |nabla*rhob|^2
      DO IR=1,NNR1
        ELFFA(IR)=DREAL(VA(IR))*DREAL(VA(IR))
        ELFFB(IR)=DREAL(VB(IR))*DREAL(VB(IR))
      ENDDO

      CALL ZAZZERO(VA,MAXFFT)
      CALL ZAZZERO(VB,MAXFFT)
C     double FFT
!CDIR NODEP
      DO IG=1,NHG
        VA(NZH(IG))=TPIBA*(UIMAG*GK(2,IG)-GK(3,IG))*VTEMPA(IG)
        VB(NZH(IG))=TPIBA*(UIMAG*GK(2,IG)-GK(3,IG))*VTEMPB(IG)
        VA(INDZ(IG))=TPIBA*(-UIMAG*GK(2,IG)+GK(3,IG))*
     *       DCONJG(VTEMPA(IG))
        VB(INDZ(IG))=TPIBA*(-UIMAG*GK(2,IG)+GK(3,IG))*
     *       DCONJG(VTEMPB(IG))
      ENDDO
      CALL INVFFT(VA)
      CALL INVFFT(VB)
      DO IR=1,NNR1
        ELFFA(IR)=ELFFA(IR)+DREAL(VA(IR)*DCONJG(VA(IR)))
        ELFFB(IR)=ELFFB(IR)+DREAL(VB(IR)*DCONJG(VB(IR)))
      ENDDO
C.....end calculate |nabla*rhoa|^2 and |nabla*rhob|^2
C
C     calculate C_F
      P53=5.D0/3.D0
      COEFE=PI**2
      COEFE=(3.D0*COEFE)**(2.D0/3.D0)
      COEFE=0.3D0*COEFE
      COEFE23 = COEFE*2**(2.D0/3.D0)
      COEFE53 = COEFE*2**P53
C     1/8 (|nabla*rhoa|^2/rhoa + |nabla*rhob|^2/rhob )/denominator
      DO IR=1,NNR1
        RHOM=DMAX1(RHOE(IR,1),0.D0) 
        IF (RHOM.GT.ELFCUT) THEN
          ELFF(IR)=-ELFFA(IR)/(8.D0*(RHOE(IR,1)-RHOE(IR,2)))
          ELFF(IR)=ELFF(IR)-ELFFB(IR)/(8.D0*RHOE(IR,2))
          ELFF(IR) = ELFF(IR)/(COEFE23*((RHOE(IR,1)
     $         -RHOE(IR,2))**P53+RHOE(IR,2)**P53))
        ELSE
          ELFF(IR)=0.D0
        ENDIF
C       alpha part: 1/4*|nabla*rhoa|^2/rhoa/denominator_a
        RHOM=DMAX1((RHOE(IR,1)-RHOE(IR,2)),0.D0) 
        IF (RHOM.GT.(ELFCUT)) THEN
          ELFFA(IR)=-ELFFA(IR)/(4.D0*(RHOE(IR,1)-RHOE(IR,2)))
          ELFFA(IR)=ELFFA(IR)/(COEFE53*
     $         ((RHOE(IR,1)-RHOE(IR,2))**P53))
        ELSE
          ELFFA(IR)=0.D0
        ENDIF
C       beta part: 1/4*|nabla*rhoa|^2/rhoa/denominator_b
        RHOM=DMAX1(RHOE(IR,2),0.D0) 
        IF (RHOM.GT.ELFCUT) THEN
          ELFFB(IR)=-ELFFB(IR)/(4.D0*RHOE(IR,2))
          ELFFB(IR)=ELFFB(IR)/(COEFE53*RHOE(IR,2)**P53) 
        ELSE
          ELFFB(IR)=0.D0
        ENDIF
      ENDDO
C
C     calculate 0.5*SUM_i{|nabla*phi_i|^2 / D_h }
      DO IKIND=1,NKPOINT
        DO I=1,NSTATE
          COEFS=F(I,IKIND)/OMEGA
          DO K=1,3
            CALL ZAZZERO(V,MAXFFT)
            IF(TKPNT) THEN
              call stopgm('elf','not programmed for k-points')
!CDIR NODEP
              DO IG=1,NGW
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
C here we have to add GK for the K-POINTS
C does this also need symmetrization?????
C!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                V(NZHS(IG))=TPIBA*UIMAG*GK(K,IG)*C0(IG,I,IKIND)
                V(INDZS(IG))=DCONJG(TPIBA*UIMAG*GK(K,IG)*C0(IG,I,IKIND))
              ENDDO
            ELSE
!CDIR NODEP
              DO IG=1,NGW
                V(NZHS(IG))=TPIBA*UIMAG*GK(K,IG)*C0(IG,I,IKIND)
                V(INDZS(IG))=DCONJG(TPIBA*UIMAG*GK(K,IG)*C0(IG,I,IKIND))
              ENDDO
            ENDIF
            CALL S_INVFFT(V)
C..here to add Vanderbilt part of nabla*psi
            IF(TIVAN) THEN
              CALL STOPGM('ELF','VANDERBILT NOT IMPLEMENTED')
            ENDIF
            DO IR=1,NNR1
              IF (I .LE. NSUP) THEN
                RHOM=DMAX1((RHOE(IR,1)-RHOE(IR,2)),0.D0) 
                IF (RHOM.GT.ELFCUT) THEN
                  ELFFA(IR)=ELFFA(IR)+COEFS*DREAL(V(IR))**2
     $                 /(COEFE53*(RHOE(IR,1)-RHOE(IR,2))**P53)
                ENDIF
              ELSEIF (I .GT. NSUP) THEN
                RHOM=DMAX1(RHOE(IR,2),0.D0) 
                IF (RHOM.GT.ELFCUT) THEN
                  ELFFB(IR)=ELFFB(IR)+COEFS*DREAL(V(IR))**2
     $                 /(COEFE53*RHOE(IR,2)**P53)
                ENDIF
              ENDIF
              RHOM=DMAX1(RHOE(IR,1),0.D0) 
              IF (RHOM.GT.ELFCUT) THEN
                ELFF(IR)=ELFF(IR)+0.5D0*COEFS*DREAL(V(IR))**2/
     $               (COEFE23*((RHOE(IR,1)-RHOE(IR,2))**P53
     $               +RHOE(IR,2)**P53))
              ENDIF
            ENDDO
          ENDDO
        ENDDO
      ENDDO
C..calculate 1/(1+(D/D_h)^2)
      DO IR=1,NNR1
        RHOM=DMAX1(RHOE(IR,1),0.D0) 
        IF (RHOM.GT.ELFCUT) THEN
          AELF=ELFF(IR)+ELFEPS/(COEFE23*((RHOE(IR,1)
     $         -RHOE(IR,2))**P53+RHOE(IR,2)**P53))
          ELFF(IR)=1.D0/(1.D0+AELF**2)
        ENDIF
C..alpha elf
        RHOM=DMAX1((RHOE(IR,1)-RHOE(IR,2)),0.D0) 
        IF (RHOM.GT.ELFCUT) THEN
          AELFA=ELFFA(IR)+ELFEPS/(COEFE53*
     $         (RHOE(IR,1)-RHOE(IR,2))**P53)
          ELFFA(IR)=1.D0/(1.D0+AELFA**2)
        ENDIF
C..beta elf
        RHOM=DMAX1(RHOE(IR,2),0.D0) 
        IF (RHOM.GT.ELFCUT) THEN
          AELFB=ELFFB(IR)+ELFEPS/(COEFE53*RHOE(IR,2)**P53)
          ELFFB(IR)=1.D0/(1.D0+AELFB**2)
        ENDIF
      ENDDO

      FILEN='LSD_ELF'
      FILENA='ELF_ALPHA'
      FILENB='ELF_BETA'
C..   VTEMP=ELF in G-space
      DO IR=1,NNR1
        V(IR)=DCMPLX(ELFF(IR),0.D0)
        VA(IR)=DCMPLX(ELFFA(IR),0.D0)
        VB(IR)=DCMPLX(ELFFB(IR),0.D0)
      ENDDO

      CALL FWFFT(V)
      CALL FWFFT(VA)
      CALL FWFFT(VB)

      DO IG=1,NHG
        VTEMP(IG) = V(NZH(IG))
        VTEMPA(IG) = VA(NZH(IG))
        VTEMPB(IG) = VB(NZH(IG))
      ENDDO

      IF(TPATH) THEN
        IF(IPCURR.EQ.0) THEN
          FILEN='LSD_ELF'
          FILENA='ELF_ALPHA'
          FILENB='ELF_BETA'
        ELSE
          CFLBOD='LSD_ELF_'
          CFLBODA='ELF_ALPHA_'
          CFLBODB='ELF_BETA_'
          WRITE(CIPNUM,'(I4)') IPCURR
          WRITE(CIPNUMA,'(I4)') IPCURR
          WRITE(CIPNUMB,'(I4)') IPCURR
          CALL XSTRING(CFLBOD,N1,N2)
          CALL XSTRING(CIPNUM,I1,I2)
          FILEN=CFLBOD(N1:N2)//CIPNUM(I1:I2)
          CALL XSTRING(CFLBODA,N1,N2)
          CALL XSTRING(CIPNUMA,I1,I2)
          FILENA=CFLBODA(N1:N2)//CIPNUMA(I1:I2)
          CALL XSTRING(CFLBODB,N1,N2)
          CALL XSTRING(CIPNUMB,I1,I2)
          FILENB=CFLBODB(N1:N2)//CIPNUMB(I1:I2)
        ENDIF
      ELSEIF (TMW) THEN
        CFLBOD='LSD_ELF_'
        CALL MW_FILENAME(CFLBOD,FILEN,WALKER_ID)
        CFLBODA='ELF_ALPHA_'
        CALL MW_FILENAME(CFLBODA,FILENA,WALKER_ID)
        CFLBODB='ELF_BETA_'
        CALL MW_FILENAME(CFLBODB,FILENB,WALKER_ID)
      ELSE
        FILEN='LSD_ELF'
        FILENA='ELF_ALPHA'
        FILENB='ELF_BETA'
      ENDIF

C CB: Filenames for the BS states
      if(BSYMM) then
        SPACE=index(FILEN,' ')
        if(BSCLCS.eq.1) then
          if(SPACE.GT.10) then
            FILEN='LSD_ELF.bs'
          else       
            FILEN(SPACE:SPACE+2)='.bs'
          endif  
        else
          if(SPACE.GT.10) then
            FILEN='LSD_ELF.hs'
          else      
            FILEN(SPACE:SPACE+2)='.hs'
          endif  
        endif  
        SPACE=index(FILEN,'LSD_ELF')
        FILENA=FILEN
        FILENA(SPACE:SPACE+6)='ELF_ALP'
        FILENB=FILEN
        FILENB(SPACE:SPACE+6)='ELF_BET'
      endif

      CALL DENSTO(VTEMP,TAU0,FILEN)
      CALL DENSTO(VTEMPA,TAU0,FILENA)
      CALL DENSTO(VTEMPB,TAU0,FILENB)
      ELFMIN=1.D30
      ELFMAX=-1.D30
      DO IR=1,NNR1
        ELFMAX=DMAX1(ELFMAX,ELFF(IR))
        ELFMIN=DMIN1(ELFMIN,ELFF(IR))
        ELFMAXA=DMAX1(ELFMAX,ELFFA(IR))
        ELFMINA=DMIN1(ELFMIN,ELFFA(IR))
        ELFMAXB=DMAX1(ELFMAX,ELFFB(IR))
        ELFMINB=DMIN1(ELFMIN,ELFFB(IR))
      ENDDO

      CALL GLOMAX(1,ELFMAX)
      CALL GLOMIN(1,ELFMIN)
      CALL GLOMAX(1,ELFMAXA)
      CALL GLOMIN(1,ELFMINA)
      CALL GLOMAX(1,ELFMAXB)
      CALL GLOMIN(1,ELFMINB)
      IF(PARENT) THEN
        WRITE(6,'(A,T50,F15.8)')
     $       'LSD_ELF: MINIMUM OF LSD_ELF  = ',ELFMIN
        WRITE(6,'(A,T52,F13.8)')
     $       'LSD_ELF: MAXIMUM OF LSD_ELF  = ',ELFMAX
        WRITE(6,'(A,T50,F15.8)')
     $       'ELFA: MINIMUM OF ELF_ALPHA  = ',ELFMINA
        WRITE(6,'(A,T52,F13.8)')
     $       'ELFA: MAXIMUM OF ELF_ALPHA  = ',ELFMAXA
        WRITE(6,'(A,T50,F15.8)')
     $       'ELFB: MINIMUM OF ELF_BETA  = ',ELFMINB
        WRITE(6,'(A,T52,F13.8)')
     $       'ELFB: MAXIMUM OF ELF_BETA  = ',ELFMAXB
      ENDIF
C     ==--------------------------------------------------------------==
C     Deallocations.
      CALL FREEM(IP_ELFF)
      CALL FREEM(IP_ELFFA)
      CALL FREEM(IP_ELFFB)
      CALL FREEM(IP_VA)
      CALL FREEM(IP_VB)
      CALL FREEM(IP_VTEMP)
      CALL FREEM(IP_VTEMPA)
      CALL FREEM(IP_VTEMPB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_LSD_ELF(LELF,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'        !NHG
C     Arguments
      INTEGER   LELF,NSTATE
      CHARACTER TAG*30
C     Variables
      INTEGER   LRNLSM,LRHOOFR
C     ==--------------------------------------------------------------==
      LELF=2*NHG
      CALL GIVE_SCR_RNLSM(LRNLSM,TAG,NSTATE,.FALSE.)
      CALL GIVE_SCR_RHOOFR(LRHOOFR,TAG)
      LELF=MAX(LELF,LRNLSM,LRHOOFR)
      TAG='MAX(LELF,LRNLSM,LRHOOFR)'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
