#ifdef __SR8000
*option MP(P(0)), LANGLVL(SAVE(0))
#endif
C     ==================================================================
      SUBROUTINE XC_LSD(RHO,ETA,EX,EC,VXA,VCA,VXB,VCB)
C     ==--------------------------------------------------------------==
C     ==  LSD EXCHANGE AND CORRELATION FUNCTIONALS                    ==
C     ==                                                              ==
C     ==  EXCHANGE  :  SLATER alpha                                   ==
C     ==               OPTX                                           ==
C     ==  CORRELATION : PERDEW & ZUNGER                               ==
C     ==                VOSKO, WILK & NUSSAIR                         ==
C     ==                LEE, YANG & PARR                              ==
C     ==                PERDEW & WANG                                 ==
C     ==                HCTH/120                                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'func.inc'
      INCLUDE 'cnst.inc'
      PARAMETER (SMALL=1.D-20)
      PARAMETER (PI34= 0.75D0 / PI
     +         ,THIRD=1.D0/3.D0)
C     ==--------------------------------------------------------------==
C..Exchange
      IF(MFXCX.EQ.1 .AND. RHO.GT.SMALL) THEN
        CALL LSD_SX(RHO,ETA,EX,VXA,VXB,SALPHA)
      ELSE
        EX=0.0D0
        VXA=0.0D0
        VXB=0.0D0
      ENDIF
C..Correlation
      IF(RHO.LE.SMALL) THEN
        EC  = 0.0D0
        VCA = 0.0D0
        VCB = 0.0D0
        EX  = 0.0D0
        VXA = 0.0D0
        VXB = 0.0D0
      ELSE IF(MFXCC.EQ.1) THEN
        RS=(PI34/RHO)**THIRD
        IFLG=2
        IF(RS.LT.1.0D0) IFLG=1
        CALL LSD_PZ(RS,ETA,EC,VCA,VCB,IFLG)
cmb   ELSEIF(MFXCC.EQ.2) THEN
cmb     STOP "LSD_VWN"
      ELSEIF(MFXCC.EQ.3) THEN
        CALL LSD_LYP(RHO,ETA,EC,VCA,VCB)
cmb   ELSEIF(MFXCC.EQ.4) THEN
cmb     STOP "LSD_PW"
      ELSEIF(MFXCC.EQ.9) THEN
        CALL LSD_PADE(RHO,ETA,EC,VCA,VCB)
cmb   ELSEIF(MFXCC.EQ.10) THEN
cmb     STOP 
      ELSE
        EC=0.0D0
        VCA=0.0D0
        VCB=0.0D0
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GC_LSD(RHOA,RHOB,GRHOAA,GRHOBB,GRHOAB,SX,SC,V1XA,V2XA,
     *                  V1XB,V2XB,V1CA,V2CA,V1CB,V2CB,V2XAB,V2CAB)
C     ==--------------------------------------------------------------==
C     ==  GRADIENT CORRECTIONS FOR EXCHANGE AND CORRELATION           ==
C     ==                                                              ==
C     ==  EXCHANGE  :  BECKE88                                        ==
C     ==               GGAX                                           ==
C     ==               PBEX                                           ==
C     ==               revPBEX                                        ==
C     ==               HCTH/120                                       ==
C     ==               OPTX                                           ==
C     ==  CORRELATION : PERDEW86                                      ==
C     ==                LEE, YANG & PARR                              ==
C     ==                GGAC                                          ==
C     ==                PBEC                                          ==
C     ==                HCTH/120                                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'func.inc'
      PARAMETER(SMALL=1.D-20)
C     ==--------------------------------------------------------------==
C..Exchange
      IF(MGCX.EQ.1) THEN
        CALL LSD_B88(BBETA,RHOA,RHOB,GRHOAA,GRHOBB,
     *               SX,V1XA,V2XA,V1XB,V2XB)
        V2XAB=0.0D0
      ELSEIF(MGCX.EQ.2) THEN
        CALL LSD_GGAX(RHOA,RHOB,GRHOAA,GRHOBB,SX,V1XA,V2XA,V1XB,V2XB)
        V2XAB=0.0D0
      ELSEIF(MGCX.EQ.3) THEN
        CALL LSD_PBEX(RHOA,RHOB,GRHOAA,GRHOBB,SX,V1XA,V2XA,V1XB,V2XB)
        V2XAB=0.0D0
      ELSEIF(MGCX.EQ.4) THEN
        CALL LSD_revPBEX(RHOA,RHOB,GRHOAA,GRHOBB,SX,V1XA,V2XA,V1XB,V2XB)
        V2XAB=0.0D0
      ELSEIF(MGCX.EQ.5.AND.MGCC.EQ.5) THEN
        CALL LSD_HCTH
     .  (RHOA,RHOB,GRHOAA,GRHOBB,SX,V1XA,V2XA,V1XB,V2XB)
        SC=0
        V1CA=0.0D0
        V2CA=0.0D0
        V1CB=0.0D0
        V2CB=0.0D0 
        V2XAB=0.0D0
        V2CAB=0.0D0
      ELSEIF(MGCX.EQ.6) THEN
        CALL LSD_OPTX
     .  (RHOA,RHOB,GRHOAA,GRHOBB,SX,V1XA,V2XA,V1XB,V2XB)
      ELSEIF(MGCX.EQ.7) THEN
        CALL LSD_B88(BBETA,RHOA,RHOB,GRHOAA,GRHOBB,
     *               SXA,V1XAA,V2XAA,V1XBA,V2XBA)
        CALL LSD_GGAX(RHOA,RHOB,GRHOAA,GRHOBB,
     *                SXB,V1XAB,V2XAB,V1XBB,V2XBB)
        SX=0.722D0*SXA+0.347D0*SXB
        V1XA=0.722D0*V1XAA+0.347D0*V1XAB
        V2XA=0.722D0*V2XAA+0.347D0*V2XAB
        V1XB=0.722D0*V1XBA+0.347D0*V1XBB
        V2XB=0.722D0*V2XBA+0.347D0*V2XBB
      ELSEIF(MGCX.EQ.8) THEN
        CALL LSD_B88(BBETA,RHOA,RHOB,GRHOAA,GRHOBB,
     *               SXA,V1XAA,V2XAA,V1XBA,V2XBA)
        CALL LSD_GGAX(RHOA,RHOB,GRHOAA,GRHOBB,
     *                SXB,V1XAB,V2XAB,V1XBB,V2XBB)
        SX=0.542D0*SXA+0.167D0*SXB
        V1XA=0.542D0*V1XAA+0.167D0*V1XAB
        V2XA=0.542D0*V2XAA+0.167D0*V2XAB
        V1XB=0.542D0*V1XBA+0.167D0*V1XBB
        V2XB=0.542D0*V2XBA+0.167D0*V2XBB
      ELSEIF(MGCX.EQ.9) THEN
        CALL LSD_PBESX(RHOA,RHOB,GRHOAA,GRHOBB,SX,V1XA,V2XA,V1XB,V2XB)
        V2XAB=0.0D0
      ELSE
        SX=0.0D0
        V1XA=0.0D0
        V2XA=0.0D0
        V1XB=0.0D0
        V2XB=0.0D0
        V2XAB=0.0D0
      ENDIF
C..Correlation
      SC=0.0D0
      V1CA=0.0D0
      V2CA=0.0D0
      V1CB=0.0D0
      V2CB=0.0D0
      V2CAB=0.0D0
      IF(MGCC.EQ.1) THEN
        RHO=RHOA+RHOB
        GRHO=GRHOAA+2.D0*GRHOAB+GRHOBB
        IF(RHO.GT.SMALL) 
     *      CALL LSD_P86(RHOA,RHOB,GRHO,SC,V1CA,V2CA,V1CB,V2CB,V2CAB)
      ELSEIF(MGCC.EQ.2) THEN
        RHO=RHOA+RHOB
        IF(RHO.GT.SMALL) 
     *    CALL LSD_GLYP(RHOA,RHOB,GRHOAA,GRHOAB,GRHOBB,SC,
     *                  V1CA,V2CA,V1CB,V2CB,V2CAB)
cmb   ELSEIF(MGCC.EQ.3) THEN
cmb     STOP "LSD_GGAC"
      ELSEIF(MGCC.EQ.4) THEN
        RHO=RHOA+RHOB
        IF(RHO.GT.SMALL) 
     *    CALL LSD_PBEC(RHOA,RHOB,GRHOAA,GRHOAB,GRHOBB,SC,
     *                  V1CA,V2CA,V1CB,V2CB,V2CAB)
      ELSEIF(MGCC.EQ.7) THEN
        RHO=RHOA+RHOB
        IF(RHO.GT.SMALL) 
     *    CALL LSD_PBESC(RHOA,RHOB,GRHOAA,GRHOAB,GRHOBB,SC,
     *                  V1CA,V2CA,V1CB,V2CB,V2CAB)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_SX(RHO,ETA,EX,VXA,VXB,ALPHA)
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (F = -1.39578858466194911D0)
      PARAMETER (THIRD=1.D0/3.D0,F43=4.D0/3.D0)
C     ==--------------------------------------------------------------==
      RHOA=0.5D0*RHO*(1.D0+ETA)
      RHOB=0.5D0*RHO*(1.D0-ETA)
      RSA = RHOA**THIRD
      RSB = RHOB**THIRD
      EX = F*ALPHA*(RHOA*RSA+RHOB*RSB)/RHO
      VXA = F43*F*ALPHA*RSA
      VXB = F43*F*ALPHA*RSB
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_PZ(RS,ETA,EPZ,VPZA,VPZB,IFLG)
C     ==--------------------------------------------------------------==
C     ==  J.P. PERDEW AND ALEX ZUNGER PRB 23, 5048 (1981)             ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (AU=0.0311D0,BU=-0.048D0,CU=0.0020D0,DU=-0.0116D0,
     *           GCU=-0.1423D0,B1U=1.0529D0,B2U=0.3334D0)
      PARAMETER (AP=0.01555D0,BP=-0.0269D0,CP=0.0007D0,DP=-0.0048D0,
     *           GCP=-0.0843D0,B1P=1.3981D0,B2P=0.2611D0)
      PARAMETER (F13=1.D0/3.D0,F43=4.D0/3.D0)
C     ==--------------------------------------------------------------==
      IF(IFLG.EQ.1) THEN
C..High density formula
        XLN=LOG(RS)
        EU=AU*XLN+BU+CU*RS*XLN+DU*RS
        EP=AP*XLN+BP+CP*RS*XLN+DP*RS
        VU=AU*XLN+(BU-AU/3.D0)+2.D0/3.D0*CU*RS*XLN+
     *              (2.D0*DU-CU)/3.D0*RS
        VP=AP*XLN+(BP-AP/3.D0)+2.D0/3.D0*CP*RS*XLN+
     *              (2.D0*DP-CP)/3.D0*RS
      ELSEIF(IFLG.EQ.2) THEN
C..Interpolation formula
        RS1=SQRT(RS)
        RS2=RS
        OXU=1.D0+B1U*RS1+B2U*RS2
        DOXU=1.D0+7.D0/6.D0*B1U*RS1+4.D0/3.D0*B2U*RS2
        EU=GCU/OXU
        VU=EU*DOXU/OXU
        OXP=1.D0+B1P*RS1+B2P*RS2
        DOXP=1.D0+7.D0/6.D0*B1P*RS1+4.D0/3.D0*B2P*RS2
        EP=GCP/OXP
        VP=EP*DOXP/OXP
      ENDIF
      FETA=((1.D0+ETA)**F43+(1.D0-ETA)**F43-2.D0)/(2.D0**F43-2.D0)
      DFETA=F43*((1.D0+ETA)**F13-(1.D0-ETA)**F13)/(2.D0**F43-2.D0)
      EPZ=EU+FETA*(EP-EU)
      VPZA=VU+FETA*(VP-VU)+(EP-EU)*(1.D0-ETA)*DFETA
      VPZB=VU+FETA*(VP-VU)+(EP-EU)*(-1.D0-ETA)*DFETA
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_LYP(RHO,ETA,ELYP,VALYP,VBLYP)
C     ==--------------------------------------------------------------==
C     ==  C. LEE, W. YANG, AND R.G. PARR, PRB 37, 785 (1988)          ==
C     ==  THIS IS ONLY THE LDA PART                                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (SMALL=1.D-24)
      PARAMETER (A=0.04918D0,B=0.132D0,C=0.2533D0,D=0.349D0)
      PARAMETER (CF=2.87123400018819108D0)
C     ==--------------------------------------------------------------==
      RA=RHO*0.5D0*(1.D0+ETA)
      RA=DMAX1(RA,SMALL)
      RB=RHO*0.5D0*(1.D0-ETA)
      RB=DMAX1(RB,SMALL)
      RM3=RHO**(-1.D0/3.D0)
      DR=(1.D0+D*RM3)
      E1=4.D0*A*RA*RB/RHO/DR
      OR=EXP(-C*RM3)/DR*RM3**11.D0
      DOR=-1.D0/3.D0*RM3**4*OR*(11.D0/RM3-C-D/DR)
      E2=2.D0**(11.D0/3.D0)*CF*A*B*OR*RA*RB*(RA**(8.d0/3.d0)+
     *      RB**(8.d0/3.d0))
      ELYP=(-E1-E2)/RHO
      DE1A=-E1*(1.D0/3.D0*D*RM3**4/DR+1.D0/RA-1.D0/RHO)
      DE1B=-E1*(1.D0/3.D0*D*RM3**4/DR+1.D0/RB-1.D0/RHO)
      DE2A=-2.D0**(11.D0/3.D0)*CF*A*B*(DOR*RA*RB*(RA**(8.d0/3.d0)+
     *      RB**(8.d0/3.d0))+OR*RB*(11.D0/3.D0*RA**(8.d0/3.d0)+
     *      RB**(8.d0/3.d0)))
      DE2B=-2.D0**(11.D0/3.D0)*CF*A*B*(DOR*RA*RB*(RA**(8.d0/3.d0)+
     *      RB**(8.d0/3.d0))+OR*RA*(11.D0/3.D0*RB**(8.d0/3.d0)+
     *      RA**(8.d0/3.d0)))
      VALYP=DE1A+DE2A
      VBLYP=DE1B+DE2B
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_PADE(RHO,ETA,EC,VCA,VCB)
C     ==--------------------------------------------------------------==
C     ==  PADE APPROXIMATION                                          ==
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A0=.4581652932831429d0,A1=2.217058676663745d0,
     &           A2=0.7405551735357053d0,A3=0.01968227878617998d0)
      PARAMETER (B1=1.0D0,B2=4.504130959426697d0,
     &           B3=1.110667363742916d0,B4=0.02359291751427506d0)
      PARAMETER (DA0=.119086804055547D0,DA1=.6157402568883345d0,
     &           DA2=.1574201515892867d0,DA3=.003532336663397157d0)
      PARAMETER (DB1=0.0d0,DB2=.2673612973836267d0,
     &           DB3=.2052004607777787d0,DB4=.004200005045691381d0)
      PARAMETER (RSFAC=.6203504908994000d0)
      PARAMETER (FSFAC=1.92366105093153617d0)
C     ==--------------------------------------------------------------==
      RS=RSFAC*RHO**(-1.d0/3.d0)
      FS=FSFAC*((1.d0+ETA)**(4.d0/3.d0)+(1.d0-ETA)**(4.d0/3.d0)-2.d0)
      DFS=FSFAC*4.d0/3.d0*
     &              ((1.d0+ETA)**(1.d0/3.d0)-(1.d0-ETA)**(1.d0/3.d0))
      DFSA=DFS*(1.d0-ETA)
      DFSB=DFS*(-1.d0-ETA)
      A0P=A0+FS*DA0
      A1P=A1+FS*DA1
      A2P=A2+FS*DA2
      A3P=A3+FS*DA3
      B1P=B1+FS*DB1
      B2P=B2+FS*DB2
      B3P=B3+FS*DB3
      B4P=B4+FS*DB4
      TOP=A0P+RS*(A1P+RS*(A2P+RS*A3P))
      DTOP=A1P+RS*(2.d0*A2P+RS*3.d0*A3P)
      TOPX=DA0+RS*(DA1+RS*(DA2+RS*DA3))
      BOT=RS*(B1P+RS*(B2P+RS*(B3P+RS*B4P)))
      DBOT=B1P+RS*(2.d0*B2P+RS*(3.d0*B3P+RS*4.d0*B4P))
      BOTX=RS*(DB1+RS*(DB2+RS*(DB3+RS*DB4)))
      EC=-TOP/BOT
      VC=EC+RS*(DTOP/BOT-TOP*DBOT/(BOT*BOT))/3.d0
      DX=-(TOPX/BOT-TOP*BOTX/(BOT*BOT))
      VCA=VC+DX*DFSA
      VCB=VC+DX*DFSB
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_B88(B1,RHOA,RHOB,GRHOA,GRHOB,
     *                   SX,V1XA,V2XA,V1XB,V2XB)
C     ==--------------------------------------------------------------==
C BECKE EXCHANGE: PRA 38, 3098 (1988)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(OB3=1.D0/3.D0,SMALL=1.D-20)
C     ==--------------------------------------------------------------==
      SX=0.0D0
      V1XA=0.0D0
      V2XA=0.0D0
      V1XB=0.0D0
      V2XB=0.0D0
      IF(ABS(RHOA).GT.SMALL) THEN
        AA    = GRHOA
        A     = SQRT(AA)
        A     = DMAX1(A,SMALL) 
        BR1   = RHOA**OB3
        BR2   = BR1*BR1
        BR4   = BR2*BR2
        XS    = A/BR4
        XS2   = XS*XS
        SA2B8 = SQRT(1.0D0+XS2)
        SHM1  = LOG(XS+SA2B8)
        DD    = 1.0D0 + 6.0D0*B1*XS*SHM1
        DD2   = DD*DD
        DDD   = 6.0D0*B1*(SHM1+XS/SA2B8)
        GF    = -B1*XS2/DD
        DGF   = (-2.0D0*B1*XS*DD + B1*XS2*DDD)/DD2
        SX    = GF*BR4
        V1XA  = 4.D0/3.D0*BR1*(GF-XS*DGF)
        V2XA  = DGF/A
      ENDIF
      IF(ABS(RHOB).GT.SMALL) THEN
        AA    = GRHOB
        A     = SQRT(AA)
        A     = DMAX1(A,SMALL) 
        BR1   = RHOB**OB3
        BR2   = BR1*BR1
        BR4   = BR2*BR2
        XS    = A/BR4
        XS2   = XS*XS
        SA2B8 = SQRT(1.0D0+XS2)
        SHM1  = LOG(XS+SA2B8)
        DD    = 1.0D0 + 6.0D0*B1*XS*SHM1
        DD2   = DD*DD
        DDD   = 6.0D0*B1*(SHM1+XS/SA2B8)
        GF    = -B1*XS2/DD
        DGF   = (-2.0D0*B1*XS*DD + B1*XS2*DDD)/DD2
        SX    = SX+GF*BR4
        V1XB  = 4.D0/3.D0*BR1*(GF-XS*DGF)
        V2XB  = DGF/A
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_GGAX(RHOA,RHOB,GRHOAA,GRHOBB,
     *                    SX,V1XA,V2XA,V1XB,V2XB)
C J.P.PERDEW ET AL. PRB 46 6671 (1992)
      IMPLICIT REAL*8 (A-H,O-Z)
C     ==--------------------------------------------------------------==
      RHO=2.D0*RHOA
      GRHO=4.D0*GRHOAA
      CALL GGAX(RHO,GRHO,SX1,V1XA,V2XA)
      RHO=2.D0*RHOB
      GRHO=4.D0*GRHOBB
      CALL GGAX(RHO,GRHO,SX2,V1XB,V2XB)
      SX = 0.5D0*SX1 + 0.5D0*SX2
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_PBESX(RHOA,RHOB,GRHOA,GRHOB,SX,V1XA,V2XA,V1XB,V2XB)
C J.P.PERDEW ET AL.  (2007)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(SMALL=1.D-20)
      PARAMETER(US=0.161620459673995492D0,AX=-0.738558766382022406D0,
     *          UM=0.1234567901234568D0,UK=0.8040D0,UL=UM/UK)
C     ==--------------------------------------------------------------==
      SXA=0.0D0
      SXB=0.0D0
      V1XA=0.0D0
      V2XA=0.0D0
      V1XB=0.0D0
      V2XB=0.0D0
      IF(ABS(RHOA).GT.SMALL) THEN
        RHO   = 2.D0*RHOA
        AA    = 4.D0*GRHOA
        RR    = RHO**(-4.D0/3.D0)
        EX    = AX/RR
        S2    = AA*RR*RR*US*US
        PO    = 1.D0/(1.D0 + UL*S2)
        FX    = UK-UK*PO
        SXA   = EX*FX
        DFX   = 2.D0*UK*UL*PO*PO
        V1XA  = 1.33333333333333D0*AX*RHO**0.333333333333D0*(FX-S2*DFX)
        V2XA  = EX*DFX*(US*RR)**2
      ENDIF
      IF(ABS(RHOB).GT.SMALL) THEN
        RHO   = 2.D0*RHOB
        AA    = 4.D0*GRHOB
        RR    = RHO**(-4.D0/3.D0)
        EX    = AX/RR
        S2    = AA*RR*RR*US*US
        PO    = 1.D0/(1.D0 + UL*S2)
        FX    = UK-UK*PO
        SXB   = EX*FX
        DFX   = 2.D0*UK*UL*PO*PO
        V1XB  = 1.33333333333333D0*AX*RHO**0.333333333333D0*(FX-S2*DFX)
        V2XB  = EX*DFX*(US*RR)**2
      ENDIF
      SX    = 0.5D0*(SXA+SXB)
      V2XA  = 2.D0*V2XA
      V2XB  = 2.D0*V2XB
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_PBEX(RHOA,RHOB,GRHOA,GRHOB,SX,V1XA,V2XA,V1XB,V2XB)
C J.P.PERDEW ET AL. PRL 77 3865 (1996)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(SMALL=1.D-20)
      PARAMETER(US=0.161620459673995492D0,AX=-0.738558766382022406D0,
     *          UM=0.2195149727645171D0,UK=0.8040D0,UL=UM/UK)
C     ==--------------------------------------------------------------==
      SXA=0.0D0
      SXB=0.0D0
      V1XA=0.0D0
      V2XA=0.0D0
      V1XB=0.0D0
      V2XB=0.0D0
      IF(ABS(RHOA).GT.SMALL) THEN
        RHO   = 2.D0*RHOA
        AA    = 4.D0*GRHOA
        RR    = RHO**(-4.D0/3.D0)
        EX    = AX/RR
        S2    = AA*RR*RR*US*US
        PO    = 1.D0/(1.D0 + UL*S2)
        FX    = UK-UK*PO
        SXA   = EX*FX
        DFX   = 2.D0*UK*UL*PO*PO
        V1XA  = 1.33333333333333D0*AX*RHO**0.333333333333D0*(FX-S2*DFX)
        V2XA  = EX*DFX*(US*RR)**2
      ENDIF
      IF(ABS(RHOB).GT.SMALL) THEN
        RHO   = 2.D0*RHOB
        AA    = 4.D0*GRHOB
        RR    = RHO**(-4.D0/3.D0)
        EX    = AX/RR
        S2    = AA*RR*RR*US*US
        PO    = 1.D0/(1.D0 + UL*S2)
        FX    = UK-UK*PO
        SXB   = EX*FX
        DFX   = 2.D0*UK*UL*PO*PO
        V1XB  = 1.33333333333333D0*AX*RHO**0.333333333333D0*(FX-S2*DFX)
        V2XB  = EX*DFX*(US*RR)**2
      ENDIF
      SX    = 0.5D0*(SXA+SXB)
      V2XA  = 2.D0*V2XA
      V2XB  = 2.D0*V2XB
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_revPBEX
     .           (RHOA,RHOB,GRHOA,GRHOB,SX,V1XA,V2XA,V1XB,V2XB)
C Y. ZHANG ET AL. PRL 80 890 (1998)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(SMALL=1.D-20)
      PARAMETER(US=0.161620459673995492D0,AX=-0.738558766382022406D0,
     *          UM=0.2195149727645171D0,UK=1.2450D0,UL=UM/UK)
C     ==--------------------------------------------------------------==
      SXA=0.0D0
      SXB=0.0D0
      V1XA=0.0D0
      V2XA=0.0D0
      V1XB=0.0D0
      V2XB=0.0D0
      IF(ABS(RHOA).GT.SMALL) THEN
        RHO   = 2.D0*RHOA
        AA    = 4.D0*GRHOA
        RR    = RHO**(-4.D0/3.D0)
        EX    = AX/RR
        S2    = AA*RR*RR*US*US
        PO    = 1.D0/(1.D0 + UL*S2)
        FX    = UK-UK*PO
        SXA   = EX*FX
        DFX   = 2.D0*UK*UL*PO*PO
        V1XA  = 1.33333333333333D0*AX*RHO**0.333333333333D0*(FX-S2*DFX)
        V2XA  = EX*DFX*(US*RR)**2
      ENDIF
      IF(ABS(RHOB).GT.SMALL) THEN
        RHO   = 2.D0*RHOB
        AA    = 4.D0*GRHOB
        RR    = RHO**(-4.D0/3.D0)
        EX    = AX/RR
        S2    = AA*RR*RR*US*US
        PO    = 1.D0/(1.D0 + UL*S2)
        FX    = UK-UK*PO
        SXB   = EX*FX
        DFX   = 2.D0*UK*UL*PO*PO
        V1XB  = 1.33333333333333D0*AX*RHO**0.333333333333D0*(FX-S2*DFX)
        V2XB  = EX*DFX*(US*RR)**2
      ENDIF
      SX    = 0.5D0*(SXA+SXB)
      V2XA  = 2.D0*V2XA
      V2XB  = 2.D0*V2XB
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_P86(RHOA,RHOB,GRHO,SC,V1CA,V2CA,V1CB,V2CB,V2CAB)
C     ==--------------------------------------------------------------==
C PERDEW CORRELATION: PRB 33, 8822 (1986)
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'cnst.inc'
      PARAMETER(P1=0.023266D0,P2=7.389D-6,P3=8.723D0,P4=0.472D0)
      PARAMETER(PC1=0.001667D0,PC2=0.002568D0,PCI=PC1+PC2)
      PARAMETER(OB3=1.D0/3.D0)
C     ==--------------------------------------------------------------==
      RHO=RHOA+RHOB
      AA    = GRHO
      A     = SQRT(AA)
      BR1   = RHO**OB3
      BR2   = BR1*BR1
      BR4   = BR2*BR2
      RS    = (3.D0/(FPI*RHO))**OB3
      RS2   = RS*RS
      RS3   = RS*RS2
      CNA   = PC2+P1*RS+P2*RS2
      CNB   = 1.D0+P3*RS+P4*RS2+1.D4*P2*RS3
      CN    = PC1 + CNA/CNB
      DRS   = -OB3*(3.D0/FPI)**OB3 / BR4
      DCNA  = (P1+2.D0*P2*RS)*DRS
      DCNB  = (P3+2.D0*P4*RS+3.D4*P2*RS2)*DRS
      DCN   = DCNA/CNB - CNA/(CNB*CNB)*DCNB
      S1    = SQRT(RHOA**(5.D0/3.D0)+RHOB**(5.D0/3.D0))
      S2    = 2.D0**OB3/RHO**(5.D0/6.D0)
      D     = S1*S2
      DDA   = S2*5.D0/6.D0*(RHOA**(2.D0/3.D0)/S1-S1/RHO)
      DDB   = S2*5.D0/6.D0*(RHOB**(2.D0/3.D0)/S1-S1/RHO)
      PHI   = 0.192D0*PCI/CN*A*RHO**(-7.D0/6.D0)
      EPHI  = EXP(-PHI)
      SC    = AA/BR4*CN*EPHI/D
      V1C   = SC*((1.D0+PHI)*DCN/CN -((4.D0/3.D0)-(7.D0/6.D0)*PHI)/RHO)
      V1CA  = -SC/D*DDA+V1C
      V1CB  = -SC/D*DDB+V1C
      V2C   = CN*EPHI/BR4*(2.D0-PHI)/D
      V2CA  = V2C
      V2CB  = V2C
      V2CAB = V2C
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_GLYP(RA,RB,GRHOAA,GRHOAB,GRHOBB,SC,
     *                                      V1CA,V2CA,V1CB,V2CB,V2CAB)
C     ==--------------------------------------------------------------==
C LEE, YANG PARR: GRADIENT CORRECTION PART
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(A=0.04918D0,B=0.132D0,C=0.2533D0,D=0.349D0)
C     ==--------------------------------------------------------------==
      RHO=RA+RB
      RM3=RHO**(-1.D0/3.D0)
      DR=(1.D0+D*RM3)
      OR=EXP(-C*RM3)/DR*RM3**11.D0
      DOR=-1.D0/3.D0*RM3**4*OR*(11.D0/RM3-C-D/DR)
      DER=C*RM3+D*RM3/DR
      DDER=1.D0/3.D0*(D*D*RM3**5/DR/DR-DER/RHO)
      DLAA=-A*B*OR*(RA*RB/9.D0*(1.D0-3*DER-(DER-11.D0)*RA/RHO)-RB*RB)
      DLAB=-A*B*OR*(RA*RB/9.D0*(47.D0-7.D0*DER)-4.D0/3.D0*RHO*RHO)
      DLBB=-A*B*OR*(RA*RB/9.D0*(1.D0-3*DER-(DER-11.D0)*RB/RHO)-RA*RA)
      DLAAA=DOR/OR*DLAA-A*B*OR*(RB/9.D0*(1.D0-3*DER-(DER-11.D0)*RA/RHO)-
     *      RA*RB/9.D0*((3.D0+RA/RHO)*DDER+(DER-11.D0)*RB/RHO/RHO))
      DLAAB=DOR/OR*DLAA-A*B*OR*(RA/9.D0*(1.D0-3.D0*DER
     &     -(DER-11.D0)*RA/RHO)-RA*RB/9.D0*((3.D0+RA/RHO)*DDER
     &     -(DER-11.D0)*RA/RHO/RHO)-2.D0*RB)
      DLABA=DOR/OR*DLAB-A*B*OR*(RB/9.D0*(47.D0-7.D0*DER)
     &     -7.D0/9.D0*RA*RB*DDER-8.D0/3.D0*RHO)
      DLABB=DOR/OR*DLAB-A*B*OR*(RA/9.D0*(47.D0-7.D0*DER)
     &     -7.D0/9.D0*RA*RB*DDER-8.D0/3.D0*RHO)
      DLBBA=DOR/OR*DLBB-A*B*OR*(RB/9.D0*(1.D0-3.D0*DER
     &     -(DER-11.D0)*RB/RHO)-RA*RB/9.D0*((3.D0+RB/RHO)*DDER
     &     -(DER-11.D0)*RB/RHO/RHO)-2.D0*RA)
      DLBBB=DOR/OR*DLBB-A*B*OR*(RA/9.D0*(1.D0-3.0D0*DER
     &     -(DER-11.D0)*RB/RHO)-RA*RB/9.D0*((3.D0+RB/RHO)*DDER
     &     +(DER-11.D0)*RA/RHO/RHO))
      SC=DLAA*GRHOAA+DLAB*GRHOAB+DLBB*GRHOBB
      V1CA=DLAAA*GRHOAA+DLABA*GRHOAB+DLBBA*GRHOBB
      V1CB=DLAAB*GRHOAA+DLABB*GRHOAB+DLBBB*GRHOBB
      V2CA=2.D0*DLAA
      V2CB=2.D0*DLBB
      V2CAB=DLAB
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_PBESC(RHOA,RHOB,GRHOAA,GRHOAB,GRHOBB,SC,
     *                    V1CA,V2CA,V1CB,V2CB,V2CAB)
C PBE Correlation functional
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER(SMALL=1.D-24)
      PARAMETER(BE=0.046D0,GA=0.031090690869654895D0)
      PARAMETER(OB3=1.D0/3.D0)
C     ==--------------------------------------------------------------==
      RHO=RHOA+RHOB
      ETA=(RHOA-RHOB)/RHO
      PHI=0.5D0*((1.d0+ETA)**(2.D0/3.D0)+(1.d0-ETA)**(2.D0/3.D0))
      PHI3=PHI*PHI*PHI
      GRHO=GRHOAA+2.D0*GRHOAB+GRHOBB
      CALL XC_LSD(RHO,ETA,EX,EC,VXA,VCA,VXB,VCB)
      IF (ABS(EC).LT.SMALL) THEN
        SC = 0.D0
        V1CA  = 0.D0
        V2CA  = 0.D0
        V1CB  = 0.D0
        V2CB  = 0.D0
        V2CAB = 0.D0
        RETURN
      ENDIF
      AA    = DMAX1(GRHO,SMALL)
      A     = SQRT(AA)
      RS    = (3.D0/(4.D0*PI*RHO))**OB3
      XKF   = (9.D0*PI/4.D0)**OB3/RS
      XKS   = SQRT(4.D0*XKF/PI)
      T     = A/(2.D0*XKS*RHO*PHI)
      EXPE  = EXP(-EC/(PHI3*GA))
      AF    = BE/GA * (1.D0/(EXPE-1.D0))
      Y     = AF*T*T
      XY    = (1.D0+Y)/(1.D0+Y+Y*Y)
      S1    = 1.D0+BE/GA*T*T*XY
      H0    = GA*PHI3 * LOG(S1)
      SC    = RHO*H0
c..
      IF(ETA.LT.0.999999999999D0 .AND. ETA.GT.-0.999999999999D0) THEN
        DTDPHI= -T/PHI
        DPHIDE= 1.D0/(3.D0*(1.d0+ETA)**OB3)-1.D0/(3.D0*(1.d0-ETA)**OB3)
        DEDRA = 2.D0*RHOB/(RHO*RHO)
        DEDRB = -2.D0*RHOA/(RHO*RHO)
        DPHIDA= DPHIDE*DEDRA
        DPHIDB= DPHIDE*DEDRB
        DTDRA = -T*(DPHIDA/PHI+7.D0/(6.D0*RHO))
        DTDRB = -T*(DPHIDB/PHI+7.D0/(6.D0*RHO))
        DADRA = AF*AF*EXPE/(-BE*PHI3)*(3.D0*EC/PHI*DPHIDA-(VCA-EC)/RHO)
        DADRB = AF*AF*EXPE/(-BE*PHI3)*(3.D0*EC/PHI*DPHIDB-(VCB-EC)/RHO)
        DSDA  = -BE/GA * AF * T**6 * (2.D0+Y) / (1.D0+Y+Y*Y)**2
        DSDT  = 2.D0*BE/GA * T * (1.D0+2.D0*Y) / (1.D0+Y+Y*Y)**2
        DSDRA = DSDA*DADRA + DSDT*DTDRA
        DSDRB = DSDA*DADRB + DSDT*DTDRB
        DHDT  = GA*PHI3/S1*DSDT
        DHDRA = 3.D0*H0/PHI*DPHIDA + GA*PHI3/S1*DSDRA
        DHDRB = 3.D0*H0/PHI*DPHIDB + GA*PHI3/S1*DSDRB
      ELSE
        DTDRA = -T*(7.D0/(6.D0*RHO))
        DTDRB = -T*(7.D0/(6.D0*RHO))
        DADRA = AF*AF*EXPE/(-BE*PHI3)*(-(VCA-EC)/RHO)
        DADRB = AF*AF*EXPE/(-BE*PHI3)*(-(VCB-EC)/RHO)
        DSDA  = -BE/GA * AF * T**6 * (2.D0+Y) / (1.D0+Y+Y*Y)**2
        DSDT  = 2.D0*BE/GA * T * (1.D0+2.D0*Y) / (1.D0+Y+Y*Y)**2
        DSDRA = DSDA*DADRA + DSDT*DTDRA
        DSDRB = DSDA*DADRB + DSDT*DTDRB
        DHDT  = GA*PHI3/S1*DSDT
        DHDRA = GA*PHI3/S1*DSDRA
        DHDRB = GA*PHI3/S1*DSDRB
      ENDIF
c..
      V1CA  = H0 + RHO*DHDRA
      V2CA  = RHO*DHDT*T/AA
      V1CB  = H0 + RHO*DHDRB
      V2CB  = RHO*DHDT*T/AA
      V2CAB = RHO*DHDT*T/AA
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_PBEC(RHOA,RHOB,GRHOAA,GRHOAB,GRHOBB,SC,
     *                    V1CA,V2CA,V1CB,V2CB,V2CAB)
C PBE Correlation functional
      IMPLICIT REAL*8 (A-H,O-Z)
      INCLUDE 'cnst.inc'
      PARAMETER(SMALL=1.D-24)
      PARAMETER(BE=0.06672455060314922D0,GA=0.031090690869654895D0)
      PARAMETER(OB3=1.D0/3.D0)
C     ==--------------------------------------------------------------==
      RHO=RHOA+RHOB
      ETA=(RHOA-RHOB)/RHO
      PHI=0.5D0*((1.d0+ETA)**(2.D0/3.D0)+(1.d0-ETA)**(2.D0/3.D0))
      PHI3=PHI*PHI*PHI
      GRHO=GRHOAA+2.D0*GRHOAB+GRHOBB
      CALL XC_LSD(RHO,ETA,EX,EC,VXA,VCA,VXB,VCB)
      IF (ABS(EC).LT.SMALL) THEN
        SC = 0.D0
        V1CA  = 0.D0
        V2CA  = 0.D0
        V1CB  = 0.D0
        V2CB  = 0.D0
        V2CAB = 0.D0
        RETURN
      ENDIF
      AA    = DMAX1(GRHO,SMALL)
      A     = SQRT(AA)
      RS    = (3.D0/(4.D0*PI*RHO))**OB3
      XKF   = (9.D0*PI/4.D0)**OB3/RS
      XKS   = SQRT(4.D0*XKF/PI)
      T     = A/(2.D0*XKS*RHO*PHI)
      EXPE  = EXP(-EC/(PHI3*GA))
      AF    = BE/GA * (1.D0/(EXPE-1.D0))
      Y     = AF*T*T
      XY    = (1.D0+Y)/(1.D0+Y+Y*Y)
      S1    = 1.D0+BE/GA*T*T*XY
      H0    = GA*PHI3 * LOG(S1)
      SC    = RHO*H0
c..
      IF(ETA.LT.0.999999999999D0 .AND. ETA.GT.-0.999999999999D0) THEN
        DTDPHI= -T/PHI
        DPHIDE= 1.D0/(3.D0*(1.d0+ETA)**OB3)-1.D0/(3.D0*(1.d0-ETA)**OB3)
        DEDRA = 2.D0*RHOB/(RHO*RHO)
        DEDRB = -2.D0*RHOA/(RHO*RHO)
        DPHIDA= DPHIDE*DEDRA
        DPHIDB= DPHIDE*DEDRB
        DTDRA = -T*(DPHIDA/PHI+7.D0/(6.D0*RHO))
        DTDRB = -T*(DPHIDB/PHI+7.D0/(6.D0*RHO))
        DADRA = AF*AF*EXPE/(-BE*PHI3)*(3.D0*EC/PHI*DPHIDA-(VCA-EC)/RHO)
        DADRB = AF*AF*EXPE/(-BE*PHI3)*(3.D0*EC/PHI*DPHIDB-(VCB-EC)/RHO)
        DSDA  = -BE/GA * AF * T**6 * (2.D0+Y) / (1.D0+Y+Y*Y)**2
        DSDT  = 2.D0*BE/GA * T * (1.D0+2.D0*Y) / (1.D0+Y+Y*Y)**2
        DSDRA = DSDA*DADRA + DSDT*DTDRA
        DSDRB = DSDA*DADRB + DSDT*DTDRB
        DHDT  = GA*PHI3/S1*DSDT
        DHDRA = 3.D0*H0/PHI*DPHIDA + GA*PHI3/S1*DSDRA
        DHDRB = 3.D0*H0/PHI*DPHIDB + GA*PHI3/S1*DSDRB
      ELSE
        DTDRA = -T*(7.D0/(6.D0*RHO))
        DTDRB = -T*(7.D0/(6.D0*RHO))
        DADRA = AF*AF*EXPE/(-BE*PHI3)*(-(VCA-EC)/RHO)
        DADRB = AF*AF*EXPE/(-BE*PHI3)*(-(VCB-EC)/RHO)
        DSDA  = -BE/GA * AF * T**6 * (2.D0+Y) / (1.D0+Y+Y*Y)**2
        DSDT  = 2.D0*BE/GA * T * (1.D0+2.D0*Y) / (1.D0+Y+Y*Y)**2
        DSDRA = DSDA*DADRA + DSDT*DTDRA
        DSDRB = DSDA*DADRB + DSDT*DTDRB
        DHDT  = GA*PHI3/S1*DSDT
        DHDRA = GA*PHI3/S1*DSDRA
        DHDRB = GA*PHI3/S1*DSDRB
      ENDIF
c..
      V1CA  = H0 + RHO*DHDRA
      V2CA  = RHO*DHDT*T/AA
      V1CB  = H0 + RHO*DHDRB
      V2CB  = RHO*DHDT*T/AA
      V2CAB = RHO*DHDT*T/AA
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSD_HCTH(rhoa,rhob,grhoa,grhob,sx,v1xa,v2xa,v1xb,v2xb)
C     HCTH, JCP 109, 6264 (1998)
C     Parameters set-up after N.L. Doltsisnis & M. Sprik (1999)
C     Present release: Tsukuba, 09/02/2005
c-----------------------------------------------------------------------
c     rho=rhoa+rhob
c     grhoa is the SQUARE of the gradient of rhoa --> gra=sqrt(grhoa)
c     grhob is the SQUARE of the gradient of rhob --> grb=sqrt(grhob)
c     sx  : total exchange correlation energy at point r 
c     v1xa: d(sx)/drhoa 
c     v1xb: d(sx)/drhob
c     v2xa: 1/gra*d(sx)/d(gra) 
c     v2xb: 1/grb*d(sx)/d(grb) 
c     Note: d(fab)/drhoa = d(fab)/drho * d(rho)/drhoa = d(fab)/drho
c           and analogously for rhob. We make use of this in the present
c           implementation whenever possible. V2CAB = 0
c--------------------------------------------------------------------------
      IMPLICIT REAL*8 (a-h,o-z)
      INCLUDE 'cnst.inc'
      PARAMETER(SMALL=1.D-15,SMALG=1.D-20)
      PARAMETER(o3=1.0d0/3.0d0,o43=4.0d0*o3,
     .          pi4=4.d0*pi,pi36=36.d0*pi,sixpi=6.d0/pi,
     .          pi34=3.d0/(4.D0*pi),yuu=81.0d0/(32.0d0*pi),
     .          f83=8.d0/3.d0,f43=4.d0/3.d0)
      parameter(pi34_o3=0.620350490899400017d0,
     .          pi36_o3=0.206783496966466672d0,
     .          two_o3=1.259921049894873165d0,
     .          yuu_o3=0.930525736349100025d0,
     .          sixpi_o3=1.240700981798800033d0) 
      DIMENSION cg0(6),cg1(6),caa(6),cab(6),cx(6),cal(6)
c
      two13=DEXP(o3*0.69314718055994531d0)
      fact=0.5d0/(two13-1.0d0)
      sx=0.0D+00
      v1xa=0.0D+00
      v2xa=0.0D+00
      v1xb=0.0D+00
      v2xb=0.0D+00
c
c.....coefficients for PW correlation...........................
      cg0(1)= 0.031091d0
      cg0(2)= 0.213700d0
      cg0(3)= 7.595700d0
      cg0(4)= 3.587600d0
      cg0(5)= 1.638200d0
      cg0(6)= 0.492940d0
      cg1(1)= 0.015545d0
      cg1(2)= 0.205480d0
      cg1(3)=14.118900d0
      cg1(4)= 6.197700d0
      cg1(5)= 3.366200d0
      cg1(6)= 0.625170d0
      cal(1)= 0.016887d0
      cal(2)= 0.111250d0
      cal(3)= 10.35700d0
      cal(4)= 3.623100d0
      cal(5)= 0.880260d0
      cal(6)= 0.496710d0
c.....coefficients for HCTH-19-4/HCTH-120 after Nikos............
      caa(1)=  0.489508D+00
      caa(2)= -0.260699D+00
      caa(3)=  0.432917D+00
      caa(4)= -0.199247D+01
      caa(5)=  0.248531D+01
      caa(6)=  0.200000D+00
      cab(1)=  0.514730D+00
      cab(2)=  0.692982D+01
      cab(3)= -0.247073D+02
      cab(4)=  0.231098D+02
      cab(5)= -0.113234D+02
      cab(6)=  0.006000D+00
      cx(1) =  0.109163D+01
      cx(2) = -0.747215D+00
      cx(3) =  0.507833D+01
      cx(4) = -0.410746D+01
      cx(5) =  0.117173D+01
      cx(6) =  0.004000D+00
c-> spin up + down treated simultaneously
cmb-> put a threshold to fix numerical instability
      rhoa=DMAX1(rhoa,SMALL) 
      rhob=DMAX1(rhob,SMALL)
      rho=rhoa+rhob
      gra=DSQRT(grhoa)
      grb=DSQRT(grhob)
cmb-> put a threshold to fix numerical instability
      gra=DMAX1(gra,SMALG)
      grb=DMAX1(grb,SMALG)
      rhoa_o3=rhoa**o3
      rhob_o3=rhob**o3
      rho_o3=rho**o3
      rhoa_o43=rhoa*rhoa_o3
      rhob_o43=rhob*rhob_o3
      rho_o43=rho*rho_o3
      xa=gra/rhoa_o43
      xb=grb/rhob_o43
      xa2=xa*xa
      xb2=xb*xb
      xab2=0.5d0*(xa2+xb2)
      ra=pi34_o3/rhoa_o3
      rb=pi34_o3/rhob_o3
      rab=pi34_o3/rho_o3
      dra_drhoa=-pi36_o3/rhoa_o43
      drb_drhob=-pi36_o3/rhob_o43
      drab_drho=-pi36_o3/rho_o43
      CALL pwcorr2(ra,cg1,g,dg)
      era1=g
      dera1_dra=dg
      CALL pwcorr2(rb,cg1,g,dg)
      erb1=g
      derb1_drb=dg
      CALL pwcorr2(rab,cg0,g,dg)
      erab0=g
      derab0_drab=dg
      CALL pwcorr2(rab,cg1,g,dg)
      erab1=g
      derab1_drab=dg
      CALL pwcorr2(rab,cal,g,dg)
      alpha=-g
      dalpha_drab=-dg
      zeta=(rhoa-rhob)/rho
c
      qrzp1=two_o3*rhoa_o3/rho_o3
      qr1mz=two_o3*rhob_o3/rho_o3
      ffun=((1d0+zeta)*qrzp1+(1d0-zeta)*qr1mz-2d0)*fact
      dffun=o43*(qrzp1-qr1mz)*fact
      d2ffun0=8d0/9d0*fact
      zeta3=zeta*zeta*zeta
      zeta4=zeta3*zeta
      erab=erab0+alpha*ffun*(1d0-zeta4)/d2ffun0
      erab=erab+(erab1-erab0)*ffun*zeta4
      derab_drab=derab0_drab+dalpha_drab*ffun*(1d0-zeta4)/d2ffun0
     .          +(derab1_drab-derab0_drab)*ffun*zeta4
      derab_dzeta=alpha/d2ffun0*(dffun*(1d0-zeta4)-4d0*zeta3*ffun)
     .           +(erab1-erab0)*(dffun*zeta4+4d0*zeta3*ffun)
c
      exa=-yuu_o3*rhoa_o43
      exb=-yuu_o3*rhob_o43
      dexa_drhoa=-sixpi_o3*rhoa_o3
      dexb_drhob=-sixpi_o3*rhob_o3
      dzeta_drhoa=2d0*rhob/(rho*rho)
      dzeta_drhob=-2d0*rhoa/(rho*rho)
c
      uaa=caa(6)*xa2
      uaa=uaa/(1.0d0+uaa)
      ubb=caa(6)*xb2
      ubb=ubb/(1.0d0+ubb)
      uab=cab(6)*xab2
      uab=uab/(1.0d0+uab)
      uxa=cx(6)*xa2
      uxa=uxa/(1.0d0+uxa)
      uxb=cx(6)*xb2
      uxb=uxb/(1.0d0+uxb)
      ffaa=rhoa*era1
      ffbb=rhob*erb1
      ffab=rho*erab0-ffaa-ffbb
      derab_drhoa=derab_drab*drab_drho+derab_dzeta*dzeta_drhoa
      derab_drhob=derab_drab*drab_drho+derab_dzeta*dzeta_drhob
      dffaa_drhoa=era1+rhoa*dera1_dra*dra_drhoa
      dffbb_drhob=erb1+rhob*derb1_drb*drb_drhob
      dffab_drhoa=erab+rho*derab_drhoa-dffaa_drhoa
      dffab_drhob=erab+rho*derab_drhob-dffbb_drhob
cmb-> i-loop removed
      f8ra=(f83/rhoa)/(1.0d0+caa(6)*xa2)
      f8rb=(f83/rhob)/(1.0d0+caa(6)*xb2)
      f4ra=(f43/rhoa)*(xa2/xab2)/(1.0d0+cab(6)*xab2)
      f4rb=(f43/rhob)*(xb2/xab2)/(1.0d0+cab(6)*xab2)
      f8xa=(f83/rhoa)/(1.0d0+cx(6)*xa2)
      f8xb=(f83/rhob)/(1.0d0+cx(6)*xb2)
      f2ga=2.0d0/gra/(1.0d0+caa(6)*xa2)
      f2gb=2.0d0/grb/(1.0d0+caa(6)*xb2)
      fgaa=(xa2/xab2)/gra/(1.0d0+cab(6)*xab2)
      fgbb=(xb2/xab2)/grb/(1.0d0+cab(6)*xab2)
      fxaa=2.0d0/gra/(1.0d0+cx(6)*xa2)
      fxbb=2.0d0/grb/(1.0d0+cx(6)*xb2)
c
      gaa=caa(1)+uaa*(caa(2)+uaa*(caa(3)+uaa*(caa(4)+uaa*caa(5))))
      gbb=caa(1)+ubb*(caa(2)+ubb*(caa(3)+ubb*(caa(4)+ubb*caa(5))))
      gab=cab(1)+uab*(cab(2)+uab*(cab(3)+uab*(cab(4)+uab*cab(5))))
      gxa=cx(1)+uxa*(cx(2)+uxa*(cx(3)+uxa*(cx(4)+uxa*cx(5))))
      gxb=cx(1)+uxb*(cx(2)+uxb*(cx(3)+uxb*(cx(4)+uxb*cx(5))))
      uaai=uaa*(caa(2)+uaa*(2.0d0*caa(3)+uaa*(3.0d0*caa(4)
     .     +uaa*4.0d0*caa(5))))
      ubbi=ubb*(caa(2)+ubb*(2.0d0*caa(3)+ubb*(3.0d0*caa(4)
     .     +ubb*4.0d0*caa(5))))
      uabi=uab*(cab(2)+uab*(2.0d0*cab(3)+uab*(3.0d0*cab(4)
     .     +uab*4.0d0*cab(5))))
      uxai=uxa*(cx(2)+uxa*(2.0d0*cx(3)+uxa*(3.0d0*cx(4)
     .     +uxa*4.0d0*cx(5))))
      uxbi=uxb*(cx(2)+uxb*(2.0d0*cx(3)+uxb*(3.0d0*cx(4)
     .     +uxb*4.0d0*cx(5))))
      dgaa_drhoa=-f8ra*uaai 
      dgbb_drhob=-f8rb*ubbi
      dgab_drhoa=-f4ra*uabi
      dgab_drhob=-f4rb*uabi
      dgxa_drhoa=-f8xa*uxai
      dgxb_drhob=-f8xb*uxbi
      dgaa_dgra=f2ga*uaai
      dgbb_dgrb=f2gb*ubbi
      dgab_dgra=fgaa*uabi
      dgab_dgrb=fgbb*uabi
      dgxa_dgra=fxaa*uxai
      dgxb_dgrb=fxbb*uxbi
c
      sx=exa*gxa+exb*gxb+ffaa*gaa+ffbb*gbb+ffab*gab
      v1xa=dexa_drhoa*gxa+exa*dgxa_drhoa
     .    +dffaa_drhoa*gaa+ffaa*dgaa_drhoa
     .    +dffab_drhoa*gab+ffab*dgab_drhoa
      v2xa=(exa*dgxa_dgra+ffaa*dgaa_dgra+ffab*dgab_dgra)/gra
      v1xb=dexb_drhob*gxb+exb*dgxb_drhob
     .    +dffbb_drhob*gbb+ffbb*dgbb_drhob
     .    +dffab_drhob*gab+ffab*dgab_drhob
      v2xb=(exb*dgxb_dgrb+ffbb*dgbb_dgrb+ffab*dgab_dgrb)/grb
c 
      RETURN
      END
C =-------------------------------------------------------------------=
      SUBROUTINE pwcorr2(r,c,g,dg)
      IMPLICIT real*8 (a-h,o-z)
      DIMENSION c(6)
      r12=DSQRT(r)
      r32=r*r12
      r2=r*r
      rb=c(3)*r12+c(4)*r+c(5)*r32+c(6)*r2
      sb=1.0d0+1.0d0/(2.0d0*c(1)*rb)
      g=-2.0d0*c(1)*(1.0d0+c(2)*r)*dlog(sb)
      drb=c(3)/(2.0d0*r12)+c(4)+1.5d0*c(5)*r12+2.0d0*c(6)*r
      dg=(1.0d0+c(2)*r)*drb/(rb*rb*sb)
     .   -2.0d0*c(1)*c(2)*dlog(sb)
      RETURN
      END 
C     ==================================================================
      SUBROUTINE LSD_OPTX(rhoa,rhob,grhoa,grhob
     .                   ,sx,v1xa,v2xa,v1xb,v2xb)
C     OPTX, Handy et al. JCP 116, p. 5411 (2002) and refs. therein
C     Present release: Tsukuba, 21/6/2002
c--------------------------------------------------------------------------
c     grho_a,b is the SQUARE of the gradient of rho_a,b ! 
c     sx  : total exchange correlation energy at point r
c     v1x_a,b : d(sx)/drho_a,b
c     v2x_a,b : 1/gr_a,b * d(sx)/d(gr_a,b)
c--------------------------------------------------------------------------
      IMPLICIT REAL*8 (a-h,o-z)
      INCLUDE 'cnst.inc'
      PARAMETER(SMALL=1.D-20,SMAL2=1.D-08)
C.......coefficients and exponents.......................
      PARAMETER(o43=4.0d0/3.0d0
     .         ,gam=0.006D0,a1cx=0.9784571170284421D0,a2=1.43169D0)
C.......OPTX in compact form.............................
C--> ALPHA
      IF(RHOA.LE.SMALL) THEN
        exa=0.0D0
        v1xa=0.0D0
        v2xa=0.0D0
      ELSE
        gra=DMAX1(grhoa,SMAL2)
        rhoa43=rhoa**o43
        xa=DSQRT(gra)/rhoa43
        gamxa2=gam*xa*xa
        udena=1.d0/(1.d0+gamxa2)
        uua=a2*gamxa2*gamxa2*udena*udena
        udena=4.00d0*rhoa43*uua*udena
        exa=-rhoa43*(a1cx+uua)
        v1xa=o43*(exa+udena)/rhoa
        v2xa=-udena/gra
      ENDIF
C--> BETA
      IF(RHOB.LE.SMALL) THEN
        exb=0.0D0
        v1xb=0.0D0
        v2xb=0.0D0
      ELSE
        grb=DMAX1(grhob,SMAL2)
        rhob43=rhob**o43
        xb=DSQRT(grb)/rhob43
        gamxb2=gam*xb*xb
        udenb=1.d0/(1.d0+gamxb2)
        uub=a2*gamxb2*gamxb2*udenb*udenb
        udenb=4.00d0*rhob43*uub*udenb
        exb=-rhob43*(a1cx+uub)
        v1xb=o43*(exb+udenb)/rhob
        v2xb=-udenb/grb
      ENDIF
      sx=exa+exb
C
      RETURN
      END
C =-------------------------------------------------------------------=
