C     ==================================================================
      SUBROUTINE MATRIX_P(C0,CU1,NSTATE,PSI,SCR,LSCR)
C     ==================================================================
C     ==                        COMPUTES                              ==
C     ==        The  matrixes NSTATExNSTATE not dependent on k vector ==
C     ==    The final Hamiltonian matrix is constitued by combinations==
C     ==    of these matrixes. The computed matrixes depend on the    ==
C     ==    x,y and z directions and on the overlapping wavefunctions ==
C     ==    At last they are:
C     ==    <c0|H1x|c0>,<c0|H1y|c0>,<c0|H1z|c0>
C     ==    <c0|H2xx|c0>,<c0|H2xy|c0>,<c0|H2xz|c0>
C     ==    <c0|H2yx|c0>,<c0|H2yy|c0>,<c0|H2yz|c0>
C     ==    <c0|H2zx|c0>,<c0|H2zy|c0>,<c0|H2zz|c0>
C     ==    <c1x|H1x|c0>,<c1x|H1y|c0>,<c1x|H1z|c0>
C     ==    <c1y|H1x|c0>,<c1y|H1y|c0>,<c1y|H1z|c0>
C     ==    <c1z|H1x|c0>,<c1z|H1y|c0>,<c1z|H1z|c0>
C     ==    <c1x|H0|c1x>,<c1x|H0|c1y>,<c1x|H0|c1z>
C     ==    <c1y|H0|c1x>,<c1y|H0|c1y>,<c1y|H0|c1z>
C     ==    <c1z|H0|c1x>,<c1z|H0|c1y>,<c1z|H0|c1z>
C     ==--------------------------------------------------------------==

C     F(istate) = 1
C     Matrix defined with reverse sign
C     Matrix complex and hermitian
C     Computed blocks: <c0|H|c0> <c1|H|c0> <c1|H|c1>

C     forces always put in vector c2
C     then called overlap routine to build  the matrix

      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc' 
      INCLUDE 'cnst.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'nlps.inc'        !NLM NGH
      INCLUDE 'ions.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'sgpp.inc'
      INCLUDE 'response_p.inc'

C     ARGUMENTS
      INTEGER    NSTATE, LSCR
      COMPLEX*16 C0(NGW,NSTATE),CU1(NGW,NSTATE,3),psi(*)
      COMPLEX*16 SCR(LSCR)

C     VARIABLES
      DIMENSION  TWNL(NGWK,NHXS,NSX,NKPNT),
     &           EIGR(NGW,NAT),
     &           fnl00(IMAGP,NAT,NHXS,NSTATE),
     &           dtwnl_dk(2*NGW,NHXS,NSX,3),
     &           ddtwnl_ddk(2*NGW,NHXS,NSX,3,3)
      DIMENSION  DFNL_DK(2,NAT,NHXS,NSTATE,3),
     &           DDFNL_DDK(2,NAT,NHXS,NSTATE,3,3)
      DIMENSION  H1_00(NSTATE,NSTATE,3)
      DIMENSION  H2_00(NSTATE,NSTATE,3,3)
      DIMENSION  H1_10(NSTATE,NSTATE,3,3)
      DIMENSION  H0_11(NSTATE,NSTATE,3,3)

      INTEGER    ISUB,IV,IA,IG,ISP_,IAT,
     &           LMATRIX_P, ISTATE
      INTEGER    K_,KK_,K2_,L,JV,KI,LI,L2,LJ,KJ
      INTEGER    IL_AUXC,IL_DDIA,LRSCPOT

      COMPLEX*16 C2(2*NGW,NSTATE),C00(2*NGW,NSTATE)
      pointer    (IP_C2,C2), (IP_C00,C00)      
      COMPLEX*16 CTM,i_to_l,G
      COMPLEX*16 ZZERO,ZONE
      PARAMETER  (ZZERO=(0.D0,0.D0),ZONE=(1.D0,0.D0)) 

      REAL*8     DD0,DD1(3),DD2(3)
      REAL*8     TIMEF,TIME1,TIME2,TCPU
      EXTERNAL   TIMEF
      REAL*8     FOC(NSTATE),DDIA(IMAGP*NAX),auxc(*)
      pointer    (ip_ddia,ddia),(ip_auxc,auxc)
      REAL*8     FNL(1,nat,nhxs,nstate,nkpnt),DFNL(*) ! from sfac.inc

      CHARACTER  TAG*30


C     ==--------------------------------------------------------------==
      CALL TISET('    MATRIX_P',ISUB)
C     SCR test.
      CALL GIVE_SCR_MATRIX_P(LMATRIX_P,TAG,NSTATE)
      CALL TEST_SCR('MATRIX_P',TAG,LSCR,LMATRIX_P)
C     ==--------------------------------------------------------------==
      TIME1 = TIMEF()
C     ALLOCATE MEMORY

      CALL MEMORY(IP_C2,2*2*NGW*NSTATE,'C2')
      CALL ZAZZERO(C2,2*NGW*NSTATE)      
      CALL MEMORY(IP_C00,2*2*NGW*NSTATE,'C00')
      CALL ZAZZERO(C00,2*NGW*NSTATE)
      CALL MEMORY(IP_H1_00,2*3*NSTATE*NSTATE,'H1_00')
      CALL ZAZZERO(H1_00,3*NSTATE*NSTATE)
      CALL MEMORY(IP_H2_00,2*3*3*NSTATE*NSTATE,'H2_00')
      CALL ZAZZERO(H2_00,3*3*NSTATE*NSTATE)
      CALL MEMORY(IP_H1_10,2*3*3*NSTATE*NSTATE,'H1_10')
      CALL ZAZZERO(H1_10,3*3*NSTATE*NSTATE)
      CALL MEMORY(IP_H0_11,3*3*NSTATE*NSTATE,'H0_11')
      CALL AZZERO(H0_11,3*3*NSTATE*NSTATE)

      call give_scr_rscpot(lrscpot,tag,.false.)
      call test_scr('matrix_P',tag,lscr,lrscpot)

      DO ISTATE = 1,NSTATE
         DO IG = 1,NGW
            c00(IG,ISTATE) = c0(IG,ISTATE)
            c00(IG+NGW,ISTATE) = dconjg(c0(IG,ISTATE))
         ENDDO
      ENDDO

      if(GEQ0)   CALL ZCLEAN_K(C00,NSTATE,NGW)

C     M_0H0 = <c0|H|c0>
C     devo calcolare:
C     H1_x_00 = <c0|-i dx|c0>
C     H1_y_00 = <c0|-i dy|c0>
C     H1_z_00 = <c0|-i dz|c0>

      DO K_ = 1,3
         CALL ZAZZERO(C2,2*NGW*NSTATE)
         DO ISTATE = 1,NSTATE
            DO IG = 1,NGW
               G=DCMPLX(tpiba*GK(K_,IG),0.0d0)
               C2(IG,ISTATE) =  G*C00(IG,ISTATE)
               C2(IG+NGW,ISTATE)= -G*C00(IG+NGW,ISTATE)
            ENDDO
         ENDDO

C     H1_x_00 =+ <c0|d_x P>w<P |c0>+<c0|P>w<d_x P|c0>
C     H1_y_00 =+ <c0|d_y P>w<P |c0>+<c0|P>w<d_y P|c0>
C     H1_z_00 =+ <c0|d_z P>w<P |c0>+<c0|P>w<d_z P|c0>

         iat = 0
         DO ISTATE = 1,NSTATE
            DO ISP_ = 1,NSP
C     ==--------------------------------------------------------------==
C     ==  VANDERBILT                                                  ==
C     ==--------------------------------------------------------------==
               IF(TVAN(ISP_)) THEN
                  call STOPGM('K_PERT_PREP',
     &                 'Vanderbilt PP not implemented')

               ELSEIF (TSGP(ISP_)) THEN
C     ==--------------------------------------------------------------==
C     ==  STEFAN GOEDECKER                                            ==
C     ==--------------------------------------------------------------==
                  DO IA=1,NA(ISP_)
                     IAT=IAT+1
                     DO IV=1,NGH(ISP_)
                        L=NGHTOL(IV,ISP_)+1
                        KI=LFVAL(IV,ISP_)
                        LI=LPVAL(IV,ISP_)
                        DD0=0.0D0
                        DO K2_ = 1,3
                           DD2(K2_) = 0.0D0
                           DD1(K2_) = 0.0D0
                        ENDDO
                        DO JV=1,NGH(ISP_)
                           L2=NGHTOL(JV,ISP_)+1
                           LJ=LPVAL(JV,ISP_)
                           IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                              KJ=LFVAL(JV,ISP_)
                              DD0=DD0+FNL00(1,IAT,JV,ISTATE)
     &                             *HLSG(KI,KJ,L,ISP_)
                              DD1(K_)=DD1(K_)+
     &                             DFNL_DK(2,IAT,JV,ISTATE,K_)
     &                             *HLSG(KI,KJ,L,ISP_)
                           ENDIF
                        ENDDO
                        CTM=- DCMPLX(0.0D0,-1.0D0)**NGHTOL(IV,ISP_)
                        DO IG=1,NGW
                           C2(IG,istate)=C2(IG,istate)+
     &                          CTM * EIGR(IG,IAT)*
     &                          (TWNL(IG,IV,ISP_,1) * 
     &                          DCMPLX(0.D0,1.D0)*DD1(K_)+
     &                          DTWNL_DK(IG,IV,ISP_,K_)*DD0)

                           C2(IG+NGW,istate)=C2(IG+NGW,istate)+
     &                          CTM * dconjg(EIGR(IG,IAT))*
     &                          ((-1)**(NGHTOL(IV,ISP_))*
     &                                  TWNL(IG,IV,ISP_,1) * 
     &                          DCMPLX(0.D0,1.D0)* DD1(K_)+
     &                          DTWNL_DK(IG+NGW,IV,ISP_,K_)*DD0)
                        ENDDO
                     ENDDO
                  ENDDO
               ELSE

                  DO ia = 1,na(ISP_)
                     iat = iat + 1
                     DO IV = 1,NGH(ISP_)

C     ==--------------------------------------------------------------==
C     ==  BACHELET HAMANN SCHLUTER                                    ==
C     ==--------------------------------------------------------------==
                        i_to_l = DCMPLX(0.0D0,-1.0D0)**NGHTOL(IV,ISP_)
                        CTM    = - i_to_l *WSG(ISP_,IV)
                        DO IG=1,NGW
                           C2(IG,istate)=C2(IG,istate)+
     &                          CTM * EIGR(IG,IAT)*
     &                          (TWNL(IG,IV,ISP_,1) * 
     &                          DCMPLX(0.D0,1.D0)*
     &                          DFNL_DK(2,IAT,IV,ISTATE,K_)+
     &                          DTWNL_DK(IG,IV,ISP_,K_)*
     &                          FNL00(1,IAT,IV,istate))

                           C2(IG+NGW,istate)=C2(IG+NGW,istate)+
     &                          CTM * dconjg(EIGR(IG,IAT))*
     &                          ((-1)**(NGHTOL(IV,ISP_))*
     &                             TWNL(IG,IV,ISP_,1) * 
     &                          DCMPLX(0.D0,1.D0)*
     &                          DFNL_DK(2,IAT,IV,ISTATE,K_)+
     &                          DTWNL_DK(IG+NGW,IV,ISP_,K_)*
     &                          FNL00(1,IAT,IV,istate))
                        ENDDO
                     ENDDO
                  ENDDO
               ENDIF
            ENDDO
            iat = 0
         ENDDO
         if(GEQ0)   CALL ZCLEAN_K(C2,NSTATE,NGW)

         CALL ZGEMM('C','N',NSTATE,NSTATE,2*NGW,ZONE,C00(1,1),
     &        2*NGW,C2(1,1),2*NGW,ZZERO,H1_00(1,1,K_),NSTATE) 

#ifdef PARALLEL
         CALL GLOSUM(2*NSTATE*NSTATE,H1_00(1,1,K_))
#endif
      ENDDO

C     
C     H2_ab_00 = <c0|d_a P>w<d_b P |c0>+
C     <c0|d_ab P>w<P |c0>+<c0| P>w<d_ab P|c0>
C     
      DO K_ = 1,3
         DO KK_ = 1,3
            CALL ZAZZERO(C2,2*NGW*NSTATE)
            iat = 0
C     ==--------------------------------------------------------------==
            DO ISTATE = 1,NSTATE
               iat = 0
               DO ISP_ = 1,NSP
                  IF (TSGP(ISP_)) THEN
C     ==--------------------------------------------------------------==
C     ==  STEFAN GOEDECKER                                            ==
C     ==--------------------------------------------------------------==
                     DO IA=1,NA(ISP_)
                        IAT=IAT+1
                        DO IV=1,NGH(ISP_)
                           L=NGHTOL(IV,ISP_)+1
                           KI=LFVAL(IV,ISP_)
                           LI=LPVAL(IV,ISP_)
                           DD0=0.0D0
                           DO K2_ = 1,3
                              DD2(K2_) = 0.0D0
                              DD1(K2_) = 0.0D0
                           ENDDO
                           DO JV=1,NGH(ISP_)
                              L2=NGHTOL(JV,ISP_)+1
                              LJ=LPVAL(JV,ISP_)
                              IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                                 KJ=LFVAL(JV,ISP_)
                                 DD0=DD0+FNL00(1,IAT,JV,ISTATE)
     &                                *HLSG(KI,KJ,L,ISP_)
                                 DD1(KK_)=DD1(KK_)+
     &                                DFNL_DK(2,IAT,JV,ISTATE,KK_)
     &                                *HLSG(KI,KJ,L,ISP_)
                                 DD2(KK_)=DD2(KK_)+
     &                                DDFNL_DDK(1,IAT,JV,ISTATE,K_,
     &                                KK_)
     &                                *HLSG(KI,KJ,L,ISP_)
                              ENDIF
                           ENDDO
                           CTM=- DCMPLX(0.0D0,-1.0D0)**NGHTOL(IV,ISP_)
                           DO IG=1,NGW
                              C2(IG,istate)=C2(IG,istate)+
     &                             CTM * EIGR(IG,IAT)*
     &                             (DTWNL_DK(IG,IV,ISP_,K_)*
     &                             DCMPLX(0.D0,1.D0)*DD1(KK_)+0.5D0* 
     &                             DDTWNL_DDK(IG,IV,ISP_,K_,KK_)*DD0+
     &                             TWNL(IG,IV,ISP_,1)*0.5D0*DD2(KK_))

                              C2(IG+NGW,istate)=C2(IG+NGW,istate)+
     &                             CTM * dconjg(EIGR(IG,IAT))*
     &                             (DTWNL_DK(IG+NGW,IV,ISP_,K_)*
     &                             DCMPLX(0.D0,1.D0)*DD1(KK_)+0.5D0*
     &                             DDTWNL_DDK(IG+NGW,IV,ISP_,K_,KK_)*
     &                             DD0+0.5D0*
     &                            (-1)**(NGHTOL(IV,ISP_))*
     &                               TWNL(IG,IV,ISP_,1)*DD2(KK_))
                           ENDDO
                        ENDDO
                     ENDDO
                  ELSE

                     DO ia = 1,na(ISP_)
                        iat = iat + 1

                        DO IV = 1,NGH(ISP_)
                           i_to_l = DCMPLX(0.0D0,-1.0D0)
     &                          **NGHTOL(IV,ISP_)
                           CTM    = - i_to_l *WSG(ISP_,IV)

                           DO IG=1,NGW
                              C2(IG,istate)=C2(IG,istate)+
     &                             CTM * EIGR(IG,IAT)*
     &                             (DTWNL_DK(IG,IV,ISP_,K_)*
     &                             DCMPLX(0.D0,1.D0)* 
     &                             DFNL_DK(2,IAT,IV,ISTATE,KK_)+0.5D0*
     &                             DDTWNL_DDK(IG,IV,ISP_,K_,KK_)*
     &                             FNL00(1,IAT,IV,istate)+0.5D0*
     &                             TWNL(IG,IV,ISP_,1)*
     &                             DDFNL_DDK(1,IAT,IV,ISTATE,K_,KK_))

                              C2(IG+NGW,istate)=C2(IG+NGW,istate)+
     &                             CTM * dconjg(EIGR(IG,IAT))*
     &                             (DTWNL_DK(IG+NGW,IV,ISP_,K_)*
     &                             DCMPLX(0.D0,1.D0)*
     &                             DFNL_DK(2,IAT,IV,ISTATE,KK_)+0.5D0*
     &                             DDTWNL_DDK(IG+NGW,IV,ISP_,K_,KK_)*
     &                             FNL00(1,IAT,IV,istate)+0.5D0*
     &                            (-1)**(NGHTOL(IV,ISP_))*
     &                             TWNL(IG,IV,ISP_,1)*
     &                             DDFNL_DDK(1,IAT,IV,ISTATE,K_,KK_))
                           ENDDO
                        ENDDO
                     ENDDO
                  ENDIF
               ENDDO
            ENDDO
            if(GEQ0)   CALL ZCLEAN_K(C2,NSTATE,NGW)

            CALL ZGEMM('C','N',NSTATE,NSTATE,2*NGW,ZONE,C00(1,1),
     &           2*NGW,C2(1,1),2*NGW,ZZERO,H2_00(1,1,K_,KK_),NSTATE) 

#ifdef PARALLEL
            CALL SUMHMAT(H2_00(1,1,K_,KK_),NSTATE,SCR,LSCR)
#endif
         ENDDO
      ENDDO

C     <c1|H|c0>

      DO K_ = 1,3
         CALL ZAZZERO(C00,2*NGW*NSTATE)
         DO ISTATE = 1,NSTATE
            DO IG = 1,NGW
               c00(IG,ISTATE) = CU1(IG,ISTATE,K_)
               c00(IG+NGW,ISTATE) = dconjg(CU1(IG,ISTATE,K_))
            ENDDO
         ENDDO

         if(GEQ0)   CALL ZCLEAN_K(C00,NSTATE,NGW)

C     H1_x_10 = <c1|-i dx|c0>
C     H1_y_10 = <c1|-i dy|c0>
C     H1_z_10 = <c1|-i dz|c0>

         DO KK_ = 1,3
            CALL ZAZZERO(C2,2*NGW*NSTATE)

            DO ISTATE = 1,NSTATE

               DO IG = 1,NGW
                  G=DCMPLX(tpiba*GK(KK_,IG),0.0D0)

                  C2(IG,ISTATE) = G*C0(IG,ISTATE)
                  C2(IG+NGW,ISTATE)=-G*dconjg(C0(IG,ISTATE))
               ENDDO
            ENDDO

C     H1_x_10 =+ <c0|d_x P>w<P |c0>+<c0|P>w<d_x P|c0>
C     H1_y_10 =+ <c0|d_y P>w<P |c0>+<c0|P>w<d_y P|c0>
C     H1_z_10 =+ <c0|d_z P>w<P |c0>+<c0|P>w<d_z P|c0>

            iat = 0
            DO ISTATE = 1,NSTATE
               DO ISP_ = 1,NSP
                  IF (TSGP(ISP_)) THEN
C     ==--------------------------------------------------------------==
C     ==  STEFAN GOEDECKER                                            ==
C     ==--------------------------------------------------------------==
                     DO IA=1,NA(ISP_)
                        IAT=IAT+1
                        DO IV=1,NGH(ISP_)
                           L=NGHTOL(IV,ISP_)+1
                           KI=LFVAL(IV,ISP_)
                           LI=LPVAL(IV,ISP_)
                           DD0=0.0D0
                           DO K2_ = 1,3
                              DD2(K2_) = 0.0D0
                              DD1(K2_) = 0.0D0
                           ENDDO
                           DO JV=1,NGH(ISP_)
                              L2=NGHTOL(JV,ISP_)+1
                              LJ=LPVAL(JV,ISP_)
                              IF(L.EQ.L2.AND.LI.EQ.LJ) THEN
                                 KJ=LFVAL(JV,ISP_)
                                 DD0=DD0+FNL00(1,IAT,JV,ISTATE)
     &                                *HLSG(KI,KJ,L,ISP_)
                                 DD1(KK_)=DD1(KK_)+
     &                                DFNL_DK(2,IAT,JV,ISTATE,KK_)
     &                                *HLSG(KI,KJ,L,ISP_)
                              ENDIF
                           ENDDO
                           CTM=- DCMPLX(0.0D0,-1.0D0)**NGHTOL(IV,ISP_)
                           DO IG=1,NGW
                              C2(IG,istate)=C2(IG,istate)+
     &                             CTM * EIGR(IG,IAT)*
     &                             (TWNL(IG,IV,ISP_,1) * 
     &                             DCMPLX(0.D0,1.D0)*DD1(KK_)+
     &                             DTWNL_DK(IG,IV,ISP_,KK_)*DD0)

                              C2(IG+NGW,istate)=C2(IG+NGW,istate)+
     &                             CTM * dconjg(EIGR(IG,IAT))*
     &                             ((-1)**(NGHTOL(IV,ISP_))*
     &                                   TWNL(IG,IV,ISP_,1)*
     &                             DCMPLX(0.D0,1.D0)*DD1(KK_)+
     &                             DTWNL_DK(IG+NGW,IV,ISP_,KK_)*DD0)
                           ENDDO
                        ENDDO
                     ENDDO
                  ELSE
                     DO ia = 1,na(ISP_)
                        iat = iat + 1
                        DO IV = 1,NGH(ISP_)
                           i_to_l = DCMPLX(0.0D0,-1.0D0)
     &                          **NGHTOL(IV,ISP_)
                           CTM    = - i_to_l *WSG(ISP_,IV)

                           DO IG=1,NGW
                              C2(IG,istate)=C2(IG,istate)+
     &                             CTM * EIGR(IG,IAT)*
     &                             (TWNL(IG,IV,ISP_,1) * 
     &                             DCMPLX(0.D0,1.D0)*
     &                             DFNL_DK(2,IAT,IV,ISTATE,KK_)+
     &                             DTWNL_DK(IG,IV,ISP_,KK_)*
     &                             FNL00(1,IAT,IV,istate))

                              C2(IG+NGW,istate)=C2(IG+NGW,istate)+
     &                             CTM * dconjg(EIGR(IG,IAT))*
     &                             ((-1)**(NGHTOL(IV,ISP_))*
     &                                 TWNL(IG,IV,ISP_,1)*
     &                             DCMPLX(0.D0,1.D0)
     &                             *DFNL_DK(2,IAT,IV,ISTATE,KK_)+
     &                             DTWNL_DK(IG+NGW,IV,ISP_,KK_)*
     &                             FNL00(1,IAT,IV,istate))
                           ENDDO
                        ENDDO
                     ENDDO
                  ENDIF
               ENDDO
               iat = 0
            ENDDO
            if(GEQ0)   CALL ZCLEAN_K(C2,NSTATE,NGW)

            CALL ZGEMM('C','N',NSTATE,NSTATE,2*NGW,ZONE,C00(1,1),
     &           2*NGW,C2(1,1),2*NGW,ZZERO,H1_10(1,1,K_,KK_),NSTATE) 

#ifdef PARALLEL
            CALL GLOSUM(2*NSTATE*NSTATE,H1_10(1,1,K_,KK_))
#endif
         ENDDO
      ENDDO

      CALL FREEM(IP_C2)
      CALL FREEM(IP_C00)

C     <c1|H|c1>
C     in this case  only H0 is applied
C     the same structure of h0psi1 is used, but 
C     H0_11 defined as real, since CU1 are real

      do istate = 1,nstate
         foc(istate)=1.0d0
      enddo

      CALL MEMORY(IP_C2,2*NGW*NSTATE,'C2')
      CALL ZAZZERO(C2,NGW*NSTATE)

      CALL GIVE_SCR_FNONLOC(IL_AUXC,IL_DDIA,NSTATE)
      CALL MEMORY(IP_AUXC,IL_AUXC,'AUXC')
      CALL AZZERO(AUXC,IL_AUXC)
      CALL MEMORY(IP_DDIA,IL_DDIA,'DDIA')
      CALL AZZERO(DDIA,IL_DDIA)

      DO K_ = 1,3
         CALL ZAZZERO(C2,NGW*NSTATE)
         CALL AZZERO(fnl,nat*nhxs*nstate*1)

         call rnlsm(CU1(1,1,K_),nstate,psi,scr,lscr,1,1,.false.)

c     c2(g)  =  <g|  v_Hxc(0)  |cu1>; vpsi also initializes c12
         call vpsi(CU1(1,1,K_),c2,foc,VofRho0,psi,scr,nstate,1,nlsd)

c     c2(g) +=  <g|  | fnl[cu1] >
         call fnonloc(c2,foc,nstate,auxc,ddia,1,nlsd)

         DO KK_ = 1,3
c     calculate the hamiltonian matrix <c1|H0|c1> = <c1|c2>:
            call ovlap(nstate,H0_11(1,1,KK_,K_),CU1(1,1,KK_),c2)

#ifdef PARALLEL
            call glosum(nstate*nstate,H0_11(1,1,KK_,K_))
#endif
         ENDDO
      ENDDO

      CALL FREEM(IP_C2)
      CALL FREEM(IP_AUXC)
      CALL FREEM(IP_DDIA)

      if(parent) then
         time2 = timef()
         tcpu = (time2 - time1)*0.001D0
         write(6,'(a,t50,f8.2,a8)')
     &        ' cpu time for matrix elements calculation:',
     &        tcpu,' seconds'
      endif

      call tihalt('    MATRIX_P',ISUB)
      RETURN
      END
C     ==============================================================
      SUBROUTINE GIVE_SCR_MATRIX_P(LMATRIX_P,TAG,NSTATE)
      implicit none
      include 'system.h'        
      include 'ropt.inc'
      character*(*) tag
      integer LMATRIX_P,NSTATE,l_rnlsm,l_auxc,l_ddia,lmat

      call give_scr_rnlsm(l_rnlsm,tag,nstate,.false.)
      call give_scr_fnonloc(l_auxc,l_ddia,nstate)
      lmat = 2*NSTATE*(NSTATE+1)

      LMATRIX_P = max(l_rnlsm,l_auxc+l_ddia,lmat)
      return
      end
c     ==================================================================
