C     ==================================================================
      SUBROUTINE META_CELL(VELP,SCR,LSCR,LQUENCH,LMETARES)
C     ==--------------------------------------------------------------==
C     ==  Calculates the contributions to the forces on the cell      ==
C     ==  due to the hills that have been accumulated in the space    ==
C     ==  of the collective variables.                                ==
C     ==  As collective variables we take functions of the cell       ==
C     ==  In the simplest case they are directly the 6 parameters     ==
C     ==  that determine the metric tensor.                           ==
C     ==  in an isotropic case they are the cell side lengths         == 
C     ==  This routine drives the metadynamics of the cell            ==
C     ==--------------------------------------------------------------==

C     NCOLVAR   : # of collective variables
C     LHILLS    : true when hills are used in the dynamics of the constraints
C     I_META    : # of steps in the meta-dynamics
C     I_META_MAX: max # of steps in the meta-dynamics
C     I_META_RES: if restart from previous meta dynamics, 
C                 # of steps already done
C     I_HTST    : # of steps since last HILL has been added
C     HT_IST    : actual values of collective variables, dim NCOLVAR
C     HLLH      : hills altitude 
C     HLLW      : hills amplitude 
C     CSCL_FAC  : scale factors required for an homogeneous dynamic in the
C                 MCNSTR directions in the space of phases
C                 they can be read from input and can be tuned on the 
C                 amplitudes of the collective variables fluctuationso
C     F_HILL     : force contribution on ions due to the accumulation 
C                  of hills  in the space of CV 
C                  dim (NODIM)
C     META_RESTART: restart for a previous meta-dynamics
C                   the previous history of the run is read from file 
C                   further output will be appended 
C     HT_PATH    :  CV history (dim NTOT_ITER*NCOLVAR)
C     SCL_PATH   :  CV scaling factors history (dim NTOT_ITER*NCOLVAR)
C     HLLW_VAL   :  hills amplitude (dim NTOT_ITER)
C     HLLH_VAL   :  hills altitude  (dim NTOT_ITER)
C     F_AVER     :  accumulator for the calculation of the average force
C                   acting on the CV, as a contribution of the 
C                   underlaying potential 
C     IF_AVER    :  counter for the calculation of the average
C     DISPLACEMENT: norm of the displacment in CV space
C     ==--------------------------------------------------------------==

      IMPLICIT NONE
      INCLUDE  'system.h'
      INCLUDE  'ions.inc'
      INCLUDE  'cotr.inc'
      INCLUDE  'rmas.inc'
      INCLUDE  'cnst_dyn.inc'
      INCLUDE  'cnst.inc'
      INCLUDE  'metr.inc'
      INCLUDE  'strs.inc'
      INCLUDE  'irat.inc'
      INCLUDE  'tpar.inc'
      INCLUDE  'ropt.inc'
      INCLUDE  'soft.inc'
      INCLUDE  'store.inc'
      INCLUDE  'fileopen.inc'
c      INCLUDE  'adat.inc'


C     Arguments
      INTEGER       LSCR
      REAL*8        VELP(3,NAX,NSX),SCR(LSCR)
      LOGICAL       LQUENCH,LMETARES
C     Variables
      INTEGER       NTOT_ITER
      REAL*8        F_HILL(6)
      REAL*8        HT_LAST(NCOLVAR),F_FREE(NCOLVAR),
     &              F1_HT(NCOLVAR),F_AVER(NCOLVAR),
     &              HH_TEST(NCOLVAR),HT_SCL(NCOLVAR),
     &              FVB_DER(NCOLVAR)

      pointer      (IP_HT_LAST,HT_LAST),
     &             (IP_F1_HT,F1_HT),(IP_F_FREE,F_FREE),
     &             (IP_F_AVER,F_AVER),(IP_HH_TEST,HH_TEST),
     &             (IP_HT_SCL,HT_SCL),(IP_FVB_DER,FVB_DER)
      DIMENSION     DOF_INCV(NODIM,NCOLVAR,18),
     &              DET_HT(9,NCOLVAR)
      DIMENSION     HT_PATH(I_META_MAX+I_META_RES,NCOLVAR),
     &              CSCL_VAL(I_META_RES+I_META_MAX,NCOLVAR),
     &              HLLW_VAL(I_META_RES+I_META_MAX),
     &              HLLH_VAL(I_META_RES+I_META_MAX)
      DIMENSION     CSCL_FAC(3,NCOLVAR)

      INTEGER       IFIRST,I_META,I_HTST,I_TEMP,IF_AVER
      DATA          IFIRST,I_META,I_HTST,I_TEMP,IF_AVER / 5*0 /
      SAVE          IFIRST,I_META,I_HTST,I_TEMP,IF_AVER

      CHARACTER*10  FILE1,FILE2
      DATA          FILE1,FILE2 /'colvar_mtd','parvar_mtd'/

      LOGICAL       HILL_ADD,LSKIP,FERROR
      CHARACTER*10  CH_TEMP
      SAVE          IP_HT_LAST,IP_F_AVER,IP_F_FREE,IP_F1_HT,IP_FVB_DER

      REAL*8        HSCR(3,3),TOLLM,FACT,FACT1,FACT2,
     &              HH_MIN,HH_MAX,HH,HHH,HLLH0,DISPLACEMENT
      INTEGER       ISUB,II,IHT,IJ,LENGTH,ISC2,IA,IS,
     &              LSCR2,IP,IDOF,msglen,KK
      INTEGER       L_F_HT,L_SCR_HIL,K,L,IINDEX,I,J,I1,I2,J1,J2
      REAL*8        MIN_DISP,DIFF,AV_DISP,DH_TMP(3,3),F_FF(6),F2
      REAL*8        HT_TMP(3,3), HT_FOR(3,3), HFORCE(3,3),hdet(9,6)
      REAL*8        HLLW2,DISP1,DISP2,W1,W2,EEPS,ht_1(6),ht_2(6)
      REAL*8        AA1(3),AA2(3),AA3(3),cosa,cosb,cosc
C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     Set time for the routine
      CALL TISET(' META_CELL',ISUB)

C     ==--------------------------------------------------------------==
      CALL AZZERO(SCR,LSCR)
      IF(.NOT.PARENT) GOTO 9999

C     ==--------------------------------------------------------------==
C     Temporary Array

      L_F_HT    = NCOLVAR
      L_SCR_HIL = NCOLVAR*NCOLVAR+30*NCOLVAR


      LENGTH  = L_F_HT+L_SCR_HIL

      CALL  GET_ADDR(IP_HH_TEST ,SCR(1))
      ISC2         = 1+L_F_HT
      LSCR2        = LSCR - (L_F_HT)

      NTOT_ITER = I_META_MAX+I_META_RES
      TOLLM     = TOLL_AVCV

      IF(IFIRST .EQ. 0) THEN
        I_META  = I_META_RES+1
        I_TEMP  = 0
        I_HTST  = 0
        IF_AVER = 0
C     Allocate Memory

        CALL MEMORY(IP_HT_LAST,NCOLVAR,'HT_LAST')
        CALL AZZERO(HT_LAST,NCOLVAR)
        CALL MEMORY(IP_F_AVER,NCOLVAR,'F_AVER')
        CALL AZZERO(F_AVER,NCOLVAR)
        CALL MEMORY(IP_F_FREE,NCOLVAR,'F_FREE')
        CALL AZZERO(F_FREE,NCOLVAR)
        CALL MEMORY(IP_F1_HT,NCOLVAR,'F1_HT')
        CALL AZZERO(F1_HT,NCOLVAR)
        CALL MEMORY(IP_FVB_DER,NCOLVAR,'FVB_DER')
        CALL AZZERO(FVB_DER,NCOLVAR)

        CALL MEMORY(IP_HT_PATH,NCOLVAR*NTOT_ITER,'HT_PATH')
        CALL AZZERO(HT_PATH,NCOLVAR*NTOT_ITER)
        CALL MEMORY(IP_CSCL_VAL,NCOLVAR*NTOT_ITER,'CSCL_VAL')
        CALL AZZERO(CSCL_VAL,NCOLVAR*NTOT_ITER)
        CALL MEMORY(IP_HLLW_VAL,NTOT_ITER,'HLLH_VAL')
        CALL AZZERO(HLLW_VAL,NTOT_ITER)
        CALL MEMORY(IP_HLLH_VAL,NTOT_ITER,'HLLH_VAL')
        CALL AZZERO(HLLH_VAL,NTOT_ITER)

C     If META_RESTART read data from proper input
        IF(META_RESTART) THEN
          FERROR=.FALSE.
          CALL FILEOPEN(51,FILE1,FO_OLD,FERROR)
          IF(FERROR) GOTO 20
          CALL FILEOPEN(52,FILE2,FO_OLD,FERROR)
          IF(FERROR) GOTO 20
          REWIND (51)
          REWIND (52)
          DO II = 1,I_META_RES
            READ(51,ERR=20,END=20,FMT=*)  IJ, 
     &                    (HT_PATH(II,IHT),IHT=1,NCOLVAR),
     &                    (CSCL_VAL(II,IHT),IHT=1,NCOLVAR)
C           from output forces and parameters
            READ(52,ERR=20,END=20,FMT=*) IJ, 
     &                    HLLW_VAL(II),HLLH_VAL(II)
          ENDDO
          CALL FILECLOSE(51)
          CALL FILECLOSE(52)
          GOTO 100
 20         CONTINUE
          WRITE(6,*) ' ERROR WHILE READING RESTART DATA '
          CALL STOPGM('META_CELL',' ')
 100        CONTINUE
        ENDIF   !META_RESTART

        WRITE(CH_TEMP,'(A5)') '     '
        DO I=1,6
          HT_NAME(I) = CH_TEMP
        END DO
        IF(LFULLC) THEN
          WRITE(CH_TEMP,'(A5)')'SIDE1'
          HT_NAME(1) = CH_TEMP
          WRITE(CH_TEMP,'(A5)')'SIDE2'
          HT_NAME(2) = CH_TEMP
          WRITE(CH_TEMP,'(A5)')'SIDE3'
          HT_NAME(3) = CH_TEMP
          WRITE(CH_TEMP,'(A4)')'cosA'
          HT_NAME(4) = CH_TEMP
          WRITE(CH_TEMP,'(A4)')'cosB'
          HT_NAME(5) = CH_TEMP
          WRITE(CH_TEMP,'(A4)')'cosC'
          HT_NAME(6) = CH_TEMP
        ELSEIF(LISOC) THEN
          WRITE(CH_TEMP,'(A5)')'SIDE1'
          HT_NAME(1) = CH_TEMP
          WRITE(CH_TEMP,'(A5)')'SIDE2'
          HT_NAME(2) = CH_TEMP
          WRITE(CH_TEMP,'(A5)')'SIDE3'
          HT_NAME(3) = CH_TEMP
        ENDIF
      ENDIF

C     ==--------------------------------------------------------------==
C     Calculate the Collective Variables as functions of the cell parameters 
      CALL AZZERO(DET_HT,9*NCOLVAR)
      IF(LFULLC) THEN
        CALL PARFULL(HT,HT_IST,DET_HT)
      ELSEIF(LISOC) THEN
        CALL PARISO(HT,HT_IST,DET_HT)
      ELSEIF(L2DC) THEN
        DO IHT = 1,2
        ENDDO
      ENDIF

C     ==--------------------------------------------------------------==
C     Store CV values and Check displacement
      
      IF(IFIRST .EQ. 0) THEN
          DO IHT = 1,NCOLVAR
            HT_LAST(IHT) = HT_IST(IHT)
          ENDDO
        IFIRST = 1
        GOTO 200
      ENDIF

      HILL_ADD = .FALSE.
      IF(TTRAJOVRR) NTRAJ=999999999

      I_HTST = I_HTST + 1

      IF(I_HTST .EQ.  INTER_HILL ) THEN
        MIN_DISP = 100.0D0
        II = INTER_HILL
        AV_DISP = 0.0D0
        DO IHT = 1,NCOLVAR
          DIFF = ((HT_IST(IHT)-HT_LAST(IHT))/CSCL_FAC(1,IHT))**2.0D0
          AV_DISP = AV_DISP + DIFF
          MIN_DISP = MIN(DIFF,MIN_DISP)
        ENDDO
        AV_DISP = DSQRT(AV_DISP)
        IF(AV_DISP .GT. TOLLM .OR. I_META .EQ. 1) HILL_ADD = .TRUE.

        write(6,'(/10x,A,f10.6,A,f10.6,A,f10.6/)') 
     &      'MIN DISP. = ',DSQRT(MIN_DISP),'  ||DISP.tot|| = ', AV_DISP,
     &      '  Tolerance ',TOLLM
      ELSEIF(I_HTST .GT.  INTER_HILL ) THEN
        I_TEMP = I_TEMP + 1
        II = INTER_HILL
        MIN_DISP = 100.0D0
        DO IHT = 1,NCOLVAR
          DIFF = ((HT_IST(IHT)-HT_LAST(IHT))/CSCL_FAC(1,IHT))**2
          SCR(IHT+ISC2) = DIFF
          MIN_DISP = MIN(DIFF,MIN_DISP)
        ENDDO
        IF(MOD(I_TEMP,ICHECK) .EQ. 0 ) THEN
          AV_DISP = 0.0D0 
          DO IHT = 1,NCOLVAR
            AV_DISP = AV_DISP+SCR(IHT+ISC2)
          ENDDO
          AV_DISP = DSQRT(AV_DISP)
          write(6,'(/10x,A,f10.6,A,f10.6,A,f10.6/)') 
     &      'MIN DISP. = ',DSQRT(MIN_DISP),'  ||DISP.tot|| = ',AV_DISP,
     &       '  Tolerance ',TOLLM
           IF(AV_DISP .GT. TOLLM .OR. I_HTST .EQ. INTER_HILL_MAX) THEN 
            HILL_ADD = .TRUE.
          ENDIF
        ENDIF
      ENDIF 

C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     Calculation of the global forces on the collective variables
C     in order to determine the optimal hills height that guarantees 
C     the maximum balance between the opposite contribution 

      CALL HT_FORCES(HH_TEST,VELP,SCR(ISC2),LSCR2)
!$OMP parallel do private(IHT)
      DO IHT = 1,NCOLVAR
        F_AVER(IHT) = F_AVER(IHT) + HH_TEST(IHT) 
     &                  + F1_HT(IHT)/CSCL_FAC(1,IHT)
        F_FREE(IHT) = F_FREE(IHT) + HH_TEST(IHT)
      ENDDO
      IF_AVER = IF_AVER + 1

C     ==--------------------------------------------------------------==
C     New Step of Meta-Dynamics in th Space of CV
      IF(HILL_ADD) THEN
        IF(MOD(I_META,TR_FREQ) .EQ. 0 .OR. I_META .EQ. 1 ) THEN 
          IF(TTRAJOVRR) NTRAJ=1
        ENDIF

C     ==--------------------------------------------------------------==
C     Squared Norm of the Displacement in the CV space

        DISPLACEMENT = 0.0D0
        DISP1=0.0D0
        DO IHT = 1,NCOLVAR
          DISPLACEMENT = DISPLACEMENT + 
     &    (HT_IST(IHT)-HT_LAST(IHT))*(HT_IST(IHT)-HT_LAST(IHT))/
     &    (CSCL_FAC(1,IHT)*CSCL_FAC(1,IHT))
        ENDDO
        DISP2 =  DISPLACEMENT
        DISPLACEMENT = DSQRT(DISPLACEMENT)
c        write(6,*) 'dis ' , DISPLACEMENT
C     Average forces on the collective variables 
!$OMP parallel do private(IHT)
        DO IHT = 1,NCOLVAR
          F_AVER(IHT)  = F_AVER(IHT)* CSCL_FAC(1,IHT)/DBLE(IF_AVER)
          F_FREE(IHT)  = F_FREE(IHT)* CSCL_FAC(1,IHT)/DBLE(IF_AVER)
        ENDDO
        IF_AVER = 0
C     ==--------------------------------------------------------------==
C     Tuning Hills parameters 
        IF(LTUNE_HH) THEN
          HH_MIN = 100.D0
          HH_MAX = 0.0D0
          LSKIP = .FALSE.

          DO IHT = 1,NCOLVAR
            HLLW2 = HLLW*HLLW
!            Fact = (HT_IST(IHT)-HT_LAST(IHT))*
!     &           (1.0D0/(HLLW*HLLW)+1.0D0/(DISPLACEMENT*DISPLACEMENT))

             FACT = CSCL_FAC(1,IHT)/(HT_IST(IHT)-HT_LAST(IHT))*
     &              dexp(0.5D0)*HLLW2*DISP2/
     &             (DISP2*dexp(-.5D0*DISP2/HLLW2)+
     &               HLLW2*(dexp(-.5D0*DISP2/HLLW2)-
     &               dexp(-.5D0*RSHIFT*RSHIFT)))

c            HHH     = 0.5D0*dabs(F_AVER(IHT)*DEXP(0.5D0)/FACT)
            HHH     = 0.1D0*dabs(F_AVER(IHT)*FACT)
            HH_MIN       = MIN(HH_MIN,HHH)
            HH_MAX       = MAX(HH_MAX,HHH)
          ENDDO

          HH       = (HH_MIN + HH_MAX)*0.5D0
          IF(HH_MIN .GT. HTOP .OR. HH_MAX .LT. HLOW) THEN
            LSKIP = .TRUE.
            WRITE(6,'(A,f12.6,A,f12.6,A)') 'Hill top > ', HTOP*AU_KCM,
     &                 ' Kcal, or Hill top < ',HLOW*AU_KCM,' Kcal',
     &                 'keep the prev. value'
          ENDIF
          IF(.NOT. LSKIP .AND. HH .LT. HLOW) THEN
            HLLH = (HLOW+HH_MAX)*0.5D0
          ELSEIF(.NOT. LSKIP .AND. HH .GT. HTOP) THEN
            HLLH = (HH_MIN+HTOP)*0.5D0
          ELSEIF(.NOT. LSKIP ) THEN
            HLLH = HH
          ELSE
            HLLH = HLLH
          ENDIF
          WRITE(6,'(/A,f12.6,3(3x,A,f12.6)/)')
     &              '||DISP|| = ',DISPLACEMENT, 
     &              'HH tuned min = ', HH_MIN,'HH tuned max = ', HH_MAX,
     &              ' HH used ',HLLH
        ENDIF

C     ==--------------------------------------------------------------==
C     Update accumulators
!$OMP parallel do private(IHT)
        DO IHT  = 1,NCOLVAR
          HT_PATH(I_META,IHT) =  HT_LAST(IHT)
          CSCL_VAL(I_META,IHT)=  CSCL_FAC(1,IHT)
          HLLW_VAL(I_META)    =  HLLW
          HLLH_VAL(I_META)    =  HLLH
        ENDDO
C     ==--------------------------------------------------------------==

C     Open output file
        IF(PARENT)
     &   CALL HT_WRITE_OUT(I_META,HT_LAST,F1_HT,F_AVER,F_FREE,FVB_DER,
     &                     DISPLACEMENT)
         CALL AZZERO(F_AVER,NCOLVAR)
         CALL AZZERO(F_FREE,NCOLVAR)
C     ==--------------------------------------------------------------==
!$OMP parallel do private(IHT)
        DO IHT = 1,NCOLVAR
          HT_LAST(IHT)   = HT_IST(IHT)
        ENDDO

        I_META = I_META + 1
        I_TEMP = 0
        I_HTST = 0
c        stop
      ENDIF        !HILL_ADD



C     ==--------------------------------------------------------------==
C     Calculation of the contributions to the forces on ions
C     due to HILLS in the space of CV

C     Calculate hills contribution to forces

 200  CONTINUE
      IF(LHILLS) THEN
        IINDEX = I_META
c        call DCOPY(9,ht,1,ht_tmp,1)
c        eeps = 0.00001D0
c        do i = 1,3
c          do j = 1,3
c            ht(i,j) = ht_tmp(i,j) + eeps
c            CALL PARFULL(HT,HT_IST,DET_HT)
c            CALL HILLS_SALS_SHIFT(HT_IST,NCOLVAR,IINDEX,NTOT_ITER,F1_HT,
c     &         HT_LAST,SCR(ISC2),LSCR2,I_HTST)
c            W1=GAUSSPOT
c            ht(i,j) = ht_tmp(i,j) - eeps
c            CALL PARFULL(HT,HT_IST,DET_HT)
c            CALL HILLS_SALS_SHIFT(HT_IST,NCOLVAR,IINDEX,NTOT_ITER,F1_HT,
c     &         HT_LAST,SCR(ISC2),LSCR2,I_HTST)
c            W2=GAUSSPOT
c            HT_FOR(i,j) = -(W1-W2)/eeps/2.0D0
c            ht(i,j) = ht_tmp(i,j)
cc            write(6,'(2I4,2f12.8)') i,j,W1,W2
c          enddo
c        enddo

c        CALL PARFULL(HT,HT_IST,DET_HT)

        CALL AZZERO(F1_HT,NCOLVAR)

        IF(HLORE) THEN
          CALL HILLS_LOR(HT_IST,NCOLVAR,IINDEX,NTOT_ITER,F1_HT,
     &                   SCR(ISC2),LSCR2,1,0)
        ELSEIF(HRATIO) THEN
          CALL HILLS_RATIO(HT_IST,NCOLVAR,IINDEX,NTOT_ITER,F1_HT,
     &         HT_LAST,SCR(ISC2),LSCR2,I_HTST,1,0)
        ELSEIF(HSHIFT) THEN
          CALL HILLS_SALS_SHIFT(HT_IST,NCOLVAR,IINDEX,NTOT_ITER,F1_HT,
     &         HT_LAST,SCR(ISC2),LSCR2,I_HTST,1,0)
        ELSE IF(SPHERE) THEN
          CALL HILLS(HT_IST,NCOLVAR,I_META,NTOT_ITER,F1_HT,
     &                   SCR(ISC2),LSCR2,1,0)
        ELSE
          CALL HILLS_SALS(HT_IST,NCOLVAR,IINDEX,NTOT_ITER,F1_HT,
     &         HT_LAST,SCR(ISC2),LSCR2,I_HTST,1,0)
        ENDIF


      ENDIF

      CALL AZZERO(FVB_DER,NCOLVAR)
      IF(TVOLBOUND) THEN
        AA1(1) = HT(1,1)
        AA1(2) = HT(1,2)
        AA1(3) = HT(1,3)
        AA2(1) = HT(2,1)
        AA2(2) = HT(2,2)
        AA2(3) = HT(2,3)
        AA3(1) = HT(3,1)
        AA3(2) = HT(3,2)
        AA3(3) = HT(3,3)
        CALL  OMEGAGEN(AA1,AA2,AA3,OMEGA)
        
        F2 = 0.0D0
        IF(OMEGA .LT. VOLMIN+VOLMIN/100.D0) THEN
          F2 = +4.0D0*VBARRIER*(VOLMIN*VOLMIN*VOLMIN*VOLMIN)/
     &         (OMEGA**(5.0D0))
          write(6,'(72A)') ('*',i = 1,72) 
          write(6,'(10x,A,f12.4,A,f12.4)') 
     &         'WARNING: Volume = ',OMEGA,' <= Volmin = ',VOLMIN
          write(6,'(72A)')  ('*',i = 1,72)
          write(6,'(10x,A,f14.6)') 'Force: ',F2
        ELSEIF(OMEGA .GT. VOLMAX-VOLMAX/100.D0) THEN
          F2 = -4.0D0*VBARRIER/(VOLMAX*VOLMAX*VOLMAX*VOLMAX)*
     &                       (OMEGA*OMEGA*OMEGA)
          write(6,'(72A)') ('*',i = 1,72) 
          write(6,'(10x,A,f12.4,A,f12.4)') 
     &         'WARNING: Volume = ',OMEGA,' >= Volmax = ',VOLMAX
          write(6,'(72A)')  ('*',i = 1,72)
          write(6,'(10x,A,2f14.6)') 'Force: ',F2,VBARRIER
        ENDIF
        IF(LFULLC) THEN
          FACT1 = 1.0D0+2.0D0*HT_IST(4)*HT_IST(5)*HT_IST(6)-
     &             HT_IST(4)*HT_IST(4)-HT_IST(5)*HT_IST(5)-
     &             HT_IST(6)*HT_IST(6)
          FACT1 = dsqrt(FACT1)
          
          FVB_DER(1) = F2 *HT_IST(2)* HT_IST(3)*FACT1
          FVB_DER(2) = F2 *HT_IST(1)* HT_IST(3)*FACT1
          FVB_DER(3) = F2 *HT_IST(1)* HT_IST(2)*FACT1

          FACT2 = (HT_IST(1) * HT_IST(2)* HT_IST(3))/FACT1

          FVB_DER(4) = F2 * FACT2*(HT_IST(5)*HT_IST(6)-HT_IST(4))
          FVB_DER(5) = F2 * FACT2*(HT_IST(4)*HT_IST(6)-HT_IST(5))
          FVB_DER(6) = F2 * FACT2*(HT_IST(4)*HT_IST(5)-HT_IST(6))

        ELSEIF(LISOC) THEN
          cosa = AA2(1)*AA3(1)+AA2(2)*AA3(2)+AA2(3)*AA3(3)  
          cosa = cosa  / (HT_IST(2)* HT_IST(3))
          cosb = AA1(1)*AA3(1)+AA1(2)*AA3(2)+AA1(3)*AA3(3)
          cosb = cosb  / (HT_IST(1)* HT_IST(3))
          cosc = AA1(1)*AA2(1)+AA1(2)*AA2(2)+AA1(3)*AA2(3)
          cosc = cosc  / (HT_IST(1)* HT_IST(2))

          FACT1 = 1.0D0+2.0D0*cosa*cosb*cosc-cosa*cosa-
     &                        cosb*cosb-cosc*cosc
          FACT1 = dsqrt(FACT1)

          FVB_DER(1) = F2 *HT_IST(2)* HT_IST(3)*FACT1
          FVB_DER(2) = F2 *HT_IST(1)* HT_IST(3)*FACT1
          FVB_DER(3) = F2 *HT_IST(1)* HT_IST(2)*FACT1

        ELSEIF(L2DC) THEN
          DO IHT = 1,2
          ENDDO
        ENDIF
        
      ENDIF

      CALL AZZERO(FH_CELL,9)
      DO IHT = 1,NCOLVAR
        DO I = 1,3
          DO J = 1,3
            FH_CELL(J,I) = FH_CELL(J,I) +
     &                    (F1_HT(IHT)/CSCL_FAC(1,IHT)+FVB_DER(IHT))*
     &                     DET_HT(J+(I-1)*3,IHT)
          ENDDO
        ENDDO
      ENDDO

               



C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     IF MAX_STEP reached or EXIT file exist

      IF(INFI.EQ.NOMORE .OR. I_META-1.EQ.NTOT_ITER) THEN
        EXSOFT = .TRUE.

        WRITE(6,'(A)') 
     &         ' <<<<<<<<  CELL COLLECTIVE VARIABLE INFO   >>>>>>>>>'
        WRITE(6,'(4x,A,6x,A)') 'TYPE','VALUE'

        DO IHT = 1,NCOLVAR
          WRITE(6,'(2X,A10,2X,f10.4)') HT_NAME(IHT),HT_IST(IHT)  
        ENDDO
 
C     Deallocate
        CALL FREEM(IP_HT_LAST)
        CALL FREEM(IP_HT_PATH)
        CALL FREEM(IP_CSCL_VAL)
        CALL FREEM(IP_HLLW_VAL)
        CALL FREEM(IP_HLLH_VAL)
      ENDIF
 9999 CONTINUE
      CALL MY_SYNC(ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(EXSOFT,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(I_META ,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(I_HTST ,MSGLEN,SOURCE,ALLGRP)

C     WRITE RESTART
      IF((MOD(I_META,ST_FREQ) .EQ. 0 .AND. I_HTST .EQ. 0)
     &                                   .OR. EXSOFT) THEN
        LMETARES = .TRUE.
      ENDIF

C     QUENCH BO
      IF(MOD(I_META,QW_FREQ) .EQ. 0 .AND. I_HTST .EQ. 0)
     &                               LQUENCH = .TRUE.

C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     Halt time for the routine
      CALL TIHALT(' META_CELL',ISUB)

      RETURN
      END
C     ==================================================================

      SUBROUTINE HT_WRITE_OUT(I_META,HT_LAST,F1_HT,F_AVER,F_FREE,
     &                        FVB_DER,DISPLACEMENT)

      IMPLICIT NONE
      INCLUDE  'system.h'
      INCLUDE  'cnst_dyn.inc'
      INCLUDE  'cnst.inc'
      INCLUDE  'ener.inc'
      INCLUDE  'metr.inc'
      INCLUDE  'ropt.inc'
      INCLUDE  'fileopen.inc'

      INTEGER       I_META
      REAL*8        HT_LAST(NCOLVAR),F1_HT(NCOLVAR),FVB_DER(NCOLVAR),
     &              F_AVER(NCOLVAR),F_FREE(NCOLVAR),DISPLACEMENT

      DIMENSION     CSCL_FAC(3,NCOLVAR)

      CHARACTER*10  FILE1,FILE2,FILE3,FILE4,FILE5
      DATA          FILE1,FILE2,FILE3,FILE4,FILE5
     *     /'colvar_mtd','parvar_mtd','forfac_mtd',
     &       'disvar_mtd', 'enevar_mtd' /

      CHARACTER     LINEFORM*100,CHNUM*10
      LOGICAL       FERROR

      INTEGER       IA,IE,IAA,IEE,ICV,IFIRST
      SAVE          IFIRST
      DATA          IFIRST /FO_VERB/
      REAL*8        EKINH,DDOT
      EXTERNAL      DDOT

C     ==--------------------------------------------------------------==
C     Open output files
      CALL FILEOPEN(51,FILE1,FO_APP+IFIRST,FERROR)
      CALL FILEOPEN(52,FILE2,FO_APP+IFIRST,FERROR)
      CALL FILEOPEN(53,FILE3,FO_APP+IFIRST,FERROR)
      CALL FILEOPEN(54,FILE4,FO_APP+IFIRST,FERROR)
      CALL FILEOPEN(55,FILE5,FO_APP+IFIRST,FERROR)
C     Verbose open only on the first run
      IFIRST=0
C     ==--------------------------------------------------------------==

C     Write output
      WRITE(CHNUM,'(I5)') NCOLVAR
      CALL XSTRING(CHNUM,IAA,IEE)
      LINEFORM = 
     &     '(1X,I7,'//CHNUM(IAA:IEE)//'E14.6,'//CHNUM(IAA:IEE)//'F10.6)'
      WRITE(51,LINEFORM) NFI,(HT_LAST(ICV),ICV=1,NCOLVAR),
     &                     (CSCL_FAC(1,ICV),ICV=1,NCOLVAR)

      LINEFORM =  '(1X,I7,3F14.6)'
      WRITE(52,LINEFORM) NFI,DISPLACEMENT,HLLW,HLLH
C     Close files
      CALL FILECLOSE(51)
      CALL FILECLOSE(52)

C     Print force factors
      LINEFORM =  '(1X,I7,'//CHNUM(IAA:IEE)//'E14.6,'
     &     //CHNUM(IAA:IEE)//'E14.6,'//CHNUM(IAA:IEE)//'E16.4)'
      WRITE(53,LINEFORM) NFI,(F_FREE(ICV),ICV=1,NCOLVAR),
     *                          (F1_HT(ICV),ICV=1,NCOLVAR),
     *                          (FVB_DER(ICV),ICV=1,NCOLVAR)
c     *                          (F_AVER(ICV),ICV=1,NCOLVAR)

C     Close files
      CALL FILECLOSE(53)

C     Print last displacements
      LINEFORM =  '(I6,'//CHNUM(IAA:IEE)//'f11.6)'
      WRITE(54,LINEFORM) NFI,
     &      ((HT_IST(ICV)-HT_LAST(ICV)),ICV=1,NCOLVAR)

C     Close files
      CALL FILECLOSE(54)

C     Print temperature of the cell and kinetic energy
c      EHAM_HILL=EKINP+ETOT+ENOSE+ENOSP+ECNSTR+EKINC+EKINH
      EKINH=0.5D0*CMASS*DDOT(9,HTVEL,1,HTVEL,1)
      LINEFORM = '(I6,5f16.8)'
      WRITE(55,LINEFORM) NFI,
     &                   EKINH,GAUSSPOT,
     &                   ETOT,EHAM_HILL,EHAM_HILL+GAUSSPOT
      CALL FILECLOSE(55)

C     ==--------------------------------------------------------------==

      RETURN
      END
C     ==--------------------------------------------------------------==

      SUBROUTINE HT_FORCES(CV_F_NEW,VELP,SCR,LSCR)

      IMPLICIT NONE

      INCLUDE 'system.h'
      INCLUDE 'cnst_dyn.inc'
      INCLUDE 'prcp.inc'
      INCLUDE 'strs.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'tpar.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'rmas.inc'

C     Arguments
      INTEGER       LSCR
      REAL*8        CV_F_NEW(NCOLVAR),VELP(3,NAX,NSX),SCR(LSCR)
 
C     Variables
      INTEGER       IFIRST
      DATA          IFIRST /0/
      SAVE          IFIRST

      DIMENSION     DET_HT(9,NCOLVAR)
      DIMENSION     CSCL_FAC(3,NCOLVAR)

      REAL*8        HT_NEW(3,3),HT_OLD(3,3),HT_NNEW(3,3),HT_TMP(3,3) 
      REAL*8        HT_FOR(3,3)
      REAL*8        HT_IST_O(NCOLVAR),DET_HT_O(9,NCOLVAR)
      POINTER       (IP_HT_IST_O,HT_IST_O),(IP_DET_HT_O,DET_HT_O)
      REAL*8        HT_IST_N(NCOLVAR),DET_HT_N(9,NCOLVAR)
      POINTER       (IP_HT_IST_N,HT_IST_N),(IP_DET_HT_N,DET_HT_N)
      REAL*8        HT_IST_NN(NCOLVAR),DET_HT_NN(9,NCOLVAR)
      POINTER       (IP_HT_IST_NN,HT_IST_NN),(IP_DET_HT_NN,DET_HT_NN)

      REAL*8        CV_F_OLD(NCOLVAR),HT_DIFF(NCOLVAR)
      POINTER       (IP_CV_F_OLD,CV_F_OLD),(IP_HT_DIFF,HT_DIFF)
 
      SAVE          IP_CV_F_OLD

      REAL*8        TOL_CV_F,TOL_CV_X
      PARAMETER     (TOL_CV_F=5.D-10,TOL_CV_X=5.D-8)
      INTEGER       MAXRAT,MRDIIS
      PARAMETER     (MAXRAT=200,MRDIIS=5)
      REAL*8        DX(9),ASL(NCOLVAR,NCOLVAR),XLO(NCOLVAR,*)
      POINTER       (IP_ASL,ASL),(IP_XLO,XLO)
      REAL*8        ERR(NCOLVAR,*),V(MRDIIS+1),DIISM(MRDIIS+1,MRDIIS+1)
      POINTER       (IP_ERR,ERR)
      REAL*8        FACT,ERRF,ERRX,DASUM,DDOT
      EXTERNAL      DASUM,DDOT
      INTEGER       IPVT(*)
      POINTER       (IP_IPVT,IPVT)

      REAL*8        FOMEGA,FO,HSCR(3,3),HV_OLD(3,3)
      REAL*8        HV_NEW(3,3),HT_FM(3,3)
      INTEGER       I,J,IHT,LENGTH,K,ITER,L_HT
      INTEGER       IDIIS,INFO,M,IA,JJ,II

C     ==--------------------------------------------------------------==
      
#ifdef _BGL 
      L_HT = NCOLVAR + MOD(NCOLVAR,2)
      LENGTH = (32+2*MRDIIS)*L_HT+L_HT*L_HT
#else
      LENGTH = (32+2*MRDIIS)*NCOLVAR+NCOLVAR*NCOLVAR
      L_HT = NCOLVAR 
#endif
      CALL GET_ADDR(IP_HT_IST_O,SCR(1))
      CALL GET_ADDR(IP_HT_IST_N,SCR(1+L_HT))
      CALL GET_ADDR(IP_HT_IST_NN,SCR(1+2*L_HT))
      CALL GET_ADDR(IP_DET_HT_O,SCR(1+3*L_HT))
      CALL GET_ADDR(IP_DET_HT_N,SCR(1+3*L_HT+9*L_HT))
      CALL GET_ADDR(IP_DET_HT_NN,SCR(1+3*L_HT+2*9*L_HT))

      CALL GET_ADDR(IP_HT_DIFF,SCR(1+3*L_HT+3*9*L_HT))

      CALL GET_ADDR(IP_ASL,SCR(1+31*L_HT))
      CALL GET_ADDR(IP_IPVT,SCR(1+31*L_HT+NCOLVAR*NCOLVAR))
      CALL GET_ADDR(IP_XLO,SCR(1+32*L_HT+NCOLVAR*NCOLVAR))
      CALL GET_ADDR(IP_ERR,SCR(1+32*L_HT+NCOLVAR*NCOLVAR+
     &                          NCOLVAR*MRDIIS))

      CALL TEST_SCR('HT_FORCES','NCOLVAR',LSCR,LENGTH)

C     Memory Allocation at IFIRST=0
      IF(IFIRST .EQ. 0) THEN
        CALL MEMORY(IP_CV_F_OLD,NCOLVAR,'CV_F_OLD')
        IFIRST = 1
        CALL AZZERO(CV_F_OLD,NCOLVAR)
      ENDIF
      CALL DCOPY(9,HTFOR(1,1),1,HT_TMP(1,1),1)
      CALL AZZERO(HSCR,9)
      DO I=1,NSP
        FACT=PMA(I)
        DO IA=1,NA(I)
          HSCR(1,1)=HSCR(1,1)+FACT*VELP(1,IA,I)*VELP(1,IA,I)
          HSCR(2,1)=HSCR(2,1)+FACT*VELP(2,IA,I)*VELP(1,IA,I)
          HSCR(3,1)=HSCR(3,1)+FACT*VELP(3,IA,I)*VELP(1,IA,I)
          HSCR(1,2)=HSCR(1,2)+FACT*VELP(1,IA,I)*VELP(2,IA,I)
          HSCR(2,2)=HSCR(2,2)+FACT*VELP(2,IA,I)*VELP(2,IA,I)
          HSCR(3,2)=HSCR(3,2)+FACT*VELP(3,IA,I)*VELP(2,IA,I)
          HSCR(1,3)=HSCR(1,3)+FACT*VELP(1,IA,I)*VELP(3,IA,I)
          HSCR(2,3)=HSCR(2,3)+FACT*VELP(2,IA,I)*VELP(3,IA,I)
          HSCR(3,3)=HSCR(3,3)+FACT*VELP(3,IA,I)*VELP(3,IA,I)
        ENDDO
      ENDDO

      CALL DGEMM('N','N',3,3,3,1.D0,HSCR(1,1),3,HTM1(1,1),3,1.D0,
     &           HT_TMP(1,1),3)

      CALL DGEMM('N','N',3,3,3,1.0D0,HT_FM(1,1),3,HTM1(1,1),3,0.0D0,
     &           HT_FOR(1,1),3)


      FACT=DT_IONS/(2.0D0*CMASS)
      IF(TZFLEX) THEN
C       z-direction flexible cell
        FOMEGA=0.0D0
        FO=OMEGA**(2.D0/3.D0) * OMEGA0**(1.D0/3.D0)
        FOMEGA=DDOT(9,HT_TMP,1,HT0,1)/FO
        HT_OLD(1,3)=HT(1,3)-DT_IONS*(FACT*FOMEGA*HT0(3,1)/FO)
        HT_OLD(2,3)=HT(2,3)-DT_IONS*(FACT*FOMEGA*HT0(3,2)/FO)
        HT_OLD(3,3)=HT(3,3)-DT_IONS*(FACT*FOMEGA*HT0(3,3)/FO)
      ELSEIF(TISOT) THEN
C       Isotropic cell
        FOMEGA=0.0D0
        FO=3.D0 * OMEGA**(2.D0/3.D0) * OMEGA0**(1.D0/3.D0)
        FOMEGA=DDOT(9,HT_TMP,1,HT0,1)/FO
        DO K=1,3
          HT_OLD(1,K)=HT(1,K)-DT_IONS*(FACT*FOMEGA*HT0(K,1)/FO)
          HT_OLD(2,K)=HT(2,K)-DT_IONS*(FACT*FOMEGA*HT0(K,2)/FO)
          HT_OLD(3,K)=HT(3,K)-DT_IONS*(FACT*FOMEGA*HT0(K,3)/FO)
        ENDDO
      ELSE
C       General
        DO K=1,3
          HV_OLD(1,K)=HTVEL(1,K) - FACT*(HT_TMP(1,K)-FH_CELL(1,K))
          HV_OLD(2,K)=HTVEL(2,K) - FACT*(HT_TMP(2,K)-FH_CELL(2,K))
          HV_OLD(3,K)=HTVEL(3,K) - FACT*(HT_TMP(3,K)-FH_CELL(3,K))
          HT_OLD(1,K)=HT(1,K)-DT_IONS*HV_OLD(1,K)
          HT_OLD(2,K)=HT(2,K)-DT_IONS*HV_OLD(2,K)
          HT_OLD(3,K)=HT(3,K)-DT_IONS*HV_OLD(3,K)
        ENDDO
      ENDIF
c
      IF(LFULLC) THEN
        CALL PARFULL(HT_OLD,HT_IST_O,DET_HT_O)
      ELSEIF(LISOC) THEN
        CALL PARISO(HT_OLD,HT_IST_O,DET_HT_O)
      ELSEIF(L2DC) THEN
        DO IHT = 1,2
        ENDDO
      ENDIF


      CALL DCOPY(9,HT,1,HT_NEW,1)
      CALL DCOPY(9,HTVEL,1,HV_NEW,1)

      IF(LFULLC) THEN
        CALL PARFULL(HT_NEW,HT_IST_N,DET_HT_N)
      ELSEIF(LISOC) THEN
        CALL PARISO(HT_NEW,HT_IST_N,DET_HT_N)
      ELSEIF(L2DC) THEN
        DO IHT = 1,2
        ENDDO
      ENDIF

C     First Guess for the forces
      CALL AZZERO(DX,9)
      DO IHT=1,NCOLVAR
        DO I=1,3
          DO J=1,3
            DX(J+(I-1)*3)=DX(J+(I-1)*3)+
     &                   CV_F_OLD(IHT)*DET_HT_N(J+(I-1)*3,IHT)
          ENDDO
        ENDDO
      ENDDO

      DO I = 1,3
        DO J = 1,3
          HT_FOR(J,I) = DX(J+(I-1)*3)
        ENDDO
      ENDDO

      FACT=DT_IONS/(2.0D0*CMASS)
      IF(TZFLEX) THEN
C       z-direction flexible cell
        FO=OMEGA**(2.D0/3.D0) * OMEGA0**(1.D0/3.D0)
        FOMEGA=DDOT(9,HT_FOR,1,HT0,1)/FO
        HT_NNEW(1,3)=HT_NEW(1,3)-FACT*FOMEGA*HT0(1,3)/FO
        HT_NNEW(2,3)=HT_NEW(2,3)-FACT*FOMEGA*HT0(2,3)/FO
        HT_NNEW(3,3)=HT_NEW(3,3)-FACT*FOMEGA*HT0(3,3)/FO
      ELSE IF(TISOT) THEN
C       Isotropic cell
        FO=3.D0 * OMEGA**(2.D0/3.D0) * OMEGA0**(1.D0/3.D0)
        FOMEGA=DDOT(9,HT_FOR,1,HT0,1)/FO
        DO K=1,3
          HT_NNEW(1,K)=HT_NEW(1,K)-FACT*FOMEGA*HT0(1,K)/FO
          HT_NNEW(2,K)=HT_NEW(2,K)-FACT*FOMEGA*HT0(2,K)/FO
          HT_NNEW(3,K)=HT_NEW(3,K)-FACT*FOMEGA*HT0(3,K)/FO
        ENDDO
      ELSE
C       General
        DO I=1,3
          HT_NNEW(I,1)=HT_NEW(I,1) - DT_IONS*HV_NEW(I,1)
     &                 + DT_IONS * FACT*HT_FOR(I,1) ! HV_NNEW(I,K)
          HT_NNEW(I,2)=HT_NEW(I,2) - DT_IONS*HV_NEW(I,2)
     &                 + DT_IONS * FACT*HT_FOR(I,2) ! HV_NNEW(I,K)
          HT_NNEW(I,3)=HT_NEW(I,3) - DT_IONS*HV_NEW(I,3)
     &                 + DT_IONS * FACT*HT_FOR(I,3) ! HV_NNEW(I,K)
        ENDDO
      ENDIF

      CALL DCOPY(NCOLVAR,CV_F_OLD,1,CV_F_NEW,1)

C     Iterativ calculation of lambda
      DO ITER=1,MAXRAT
C     Calculate Collective Variables values and differences
C     for the current value of lambda
        CALL AZZERO(DX,9)

        IF(LFULLC) THEN
          CALL PARFULL(HT_NNEW,HT_IST_NN,DET_HT_NN)
        ELSEIF(LISOC) THEN
          CALL PARISO(HT_NNEW,HT_IST_NN,DET_HT_NN)
        ELSEIF(L2DC) THEN
          DO IHT = 1,2
          ENDDO
        ENDIF

        DO I = 1,NCOLVAR
          HT_DIFF(I) =  -(HT_IST_NN(I)-HT_IST_O(I))
        ENDDO

        ERRF=DASUM(NCOLVAR,HT_DIFF(1),1)
c        write(6,'(i5,f12.8)') ITER,ERRF 
        IF(ERRF .LT. TOL_CV_F) GOTO 100

C       Derivatives of sigma wrt lambda
        FACT = DT_IONS*DT_IONS/(2.0D0*CMASS)
        DO I=1,NCOLVAR
          DO J=1,NCOLVAR
            ASL(I,J)=0.0D0
            DO II=1,3
              DO JJ = 1,3
                ASL(I,J)=ASL(I,J)-FACT*
     &              DET_HT_NN(JJ+(II-1)*3,I)*DET_HT_N(JJ+(II-1)*3,J)
              ENDDO
            ENDDO
          ENDDO
        ENDDO
C       Solve for asl*cg=fc
C       LAPACK matrix solver
        CALL DGESV(NCOLVAR,1,ASL,NCOLVAR,IPVT,HT_DIFF,NCOLVAR,INFO)

        IF (INFO.NE.0) CALL STOPGM(' HT_FORCES|','ERROR IN DGESV')
        ERRX=DASUM(NCOLVAR,HT_DIFF(1),1)
C       DIIS!
        IDIIS=MOD(ITER-1,MRDIIS)+1
!$OMP parallel do private(IHT)
        DO I = 1,NCOLVAR
          XLO(I,IDIIS) = CV_F_NEW(I)
        ENDDO

        CALL DCOPY(NCOLVAR,HT_DIFF(1),1,ERR(1,IDIIS),1)
        IF(ITER.GT.MRDIIS) THEN
          M=MRDIIS+1
          CALL AZZERO(DIISM,M*M)
          CALL AZZERO(V,M)
          DO I=1,MRDIIS
            DO J=1,MRDIIS
              DIISM(I,J)=DDOT(NCOLVAR,ERR(1,I),1,ERR(1,J),1)
            ENDDO
            DIISM(M,I)=1.0D0
            DIISM(I,M)=1.0D0
          ENDDO 

          V(M)=1.0D0
          CALL SOLVE(DIISM,M,M,V)

          CALL AZZERO(HT_DIFF,NCOLVAR)
          CALL AZZERO(CV_F_NEW,NCOLVAR)
          DO I=1,MRDIIS
            DO J=1,NCOLVAR
              HT_DIFF(J)  = HT_DIFF(J) + V(I)*ERR(J,I)
              CV_F_NEW(J) = CV_F_NEW(J)+ V(I)*XLO(J,I)
            ENDDO
          ENDDO
        ENDIF
!$OMP parallel do private(I)
#ifdef _vpp_
!OCL NOALIAS
#endif
#ifdef __NEC
!CDIR NODEP
#endif
        DO  I = 1,NCOLVAR
          CV_F_NEW(I)   = CV_F_NEW(I)+ HT_DIFF(I)
        ENDDO

        IF(ERRX .LT. TOL_CV_X) GOTO 100
C     Update forces
        CALL AZZERO(DX,6)
        DO IHT=1,NCOLVAR
          DO I=1,3
            DO J = 1,3
              DX(J+(I-1)*3)=DX(J+(I-1)*3)
     &                    +CV_F_NEW(IHT)*DET_HT_N(J+(I-1)*3,IHT)
            ENDDO
          ENDDO
        ENDDO

        DO I = 1,3
          DO J = 1,3
            HT_FOR(J,I) = DX(J+(I-1)*3)
          ENDDO
        ENDDO

        FACT=DT_IONS/(2.0D0*CMASS)
        IF(TZFLEX) THEN
C         z-direction flexible cell
          FO=OMEGA**(2.D0/3.D0) * OMEGA0**(1.D0/3.D0)
          FOMEGA=DDOT(9,HT_FOR,1,HT0,1)/FO
          HT_NNEW(1,3)=HT_NEW(1,3)-FACT*FOMEGA*HT0(3,1)/FO
          HT_NNEW(2,3)=HT_NEW(2,3)-FACT*FOMEGA*HT0(3,2)/FO
          HT_NNEW(3,3)=HT_NEW(3,3)-FACT*FOMEGA*HT0(3,3)/FO
        ELSE IF(TISOT) THEN
C         Isotropic cell
          FO=3.D0 * OMEGA**(2.D0/3.D0) * OMEGA0**(1.D0/3.D0)
          FOMEGA=DDOT(9,HT_FOR,1,HT0,1)/FO
          DO K=1,3
            HT_NNEW(1,K)=HT_NEW(1,K)-FACT*FOMEGA*HT0(K,1)/FO
            HT_NNEW(2,K)=HT_NEW(2,K)-FACT*FOMEGA*HT0(K,2)/FO
            HT_NNEW(3,K)=HT_NEW(3,K)-FACT*FOMEGA*HT0(K,3)/FO
          ENDDO
        ELSE
C         General
          DO I=1,3
            HT_NNEW(I,1)=HT_NEW(I,1) + DT_IONS*FACT*HT_FOR(I,1)
     *                 - DT_IONS*HV_NEW(I,1) 
            HT_NNEW(I,2)=HT_NEW(I,2) + DT_IONS*FACT*HT_FOR(I,2)
     *                 - DT_IONS*HV_NEW(I,2) 
            HT_NNEW(I,3)=HT_NEW(I,3) + DT_IONS*FACT*HT_FOR(I,3)
     *                 - DT_IONS*HV_NEW(I,3)
          ENDDO
        ENDIF
      ENDDO 

      WRITE(6,'(/,A,I7,A)') ' HT_FORCES|  did not converge '
      DO I=1,NCOLVAR
c        IF(DABS(HT_DIFF(I)) .GT. TOL_CV_X ) THEN 
          CV_F_NEW(I) = 0.0D0 ! CV_F_OLD(I)/2.0D0
c        ENDIF 
      ENDDO
  100 CONTINUE

      CALL DCOPY(NCOLVAR,CV_F_NEW,1,CV_F_OLD,1)

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
      SUBROUTINE META_CELL_INP(IUNIT,LINE)
C     ==--------------------------------------------------------------==
C     ==  Reads Input for Metadynamics of the cell                    ==
C     ==--------------------------------------------------------------==

      IMPLICIT  NONE

      INCLUDE 'system.h'
      INCLUDE 'prcp.inc'
      INCLUDE 'cnst_dyn.inc'
      INCLUDE  'cnst.inc'

C     Arguments
      INTEGER      IUNIT
      CHARACTER*80 LINE
C     Variables
      INTEGER      II,IJ,IA,IE,IC,IOuT,ICV,IHT,INUM
      REAL*8       TEMP,FMIN,FMAX
      DIMENSION    CSCL_FAC(3,*),TAD_SCF(*)
      LOGICAL      ERREAD

C     ==--------------------------------------------------------------==
      LMETA_CELL = .TRUE.
      NCOLVAR = 6


      IF(INDEX(LINE,'FULL').NE.0) THEN
        LFULLC = .TRUE.
        LISOC  = .FALSE.
        L2DC   = .FALSE.
        NCOLVAR = 6
      ELSEIF(INDEX(LINE,'ISOT').NE.0) THEN
        LISOC  = .TRUE. 
        LFULLC = .FALSE.
        L2DC   = .FALSE.
        NCOLVAR = 3
      ELSEIF(INDEX(LINE,'2D').NE.0) THEN
        L2DC   = .TRUE.
        LFULLC = .FALSE.
        LISOC  = .FALSE.
        NCOLVAR = 2
      ENDIF


C     Allocate Memory
      CALL MEMORY(IP_CSCL_FAC,3*NCOLVAR,'CSCL_FAC')
      CALL MEMORY(IP_TAD_SCF,NCOLVAR,'TAD_SCF')
      CALL MEMORY(IP_HT_IST,NCOLVAR,'HT_IST')
      CALL AZZERO(HT_IST,NCOLVAR)
      CALL MEMORY(IP_DET_HT,NCOLVAR*9,'DET_HT')
      CALL AZZERO(DET_HT,NCOLVAR*9)


C     Set Default Values
#ifdef __SR11000
*poption parallel, tlocal(IC)
*voption indep(CSCL_FAC,TAD_SCF)
#endif
        DO IC = 1,NCOLVAR
          CSCL_FAC(1,IC) = 1.0D0
          CSCL_FAC(2,IC) = 0.2D0
          CSCL_FAC(3,IC) = 2.D0
          TAD_SCF(IC)  = .FALSE.
        ENDDO

   10 CONTINUE

      READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE

      IF(INDEX(LINE,'END').NE.0 .AND. INDEX(LINE,'METADYN').NE.0)
     &     GOTO 30


      IF(INDEX(LINE,'METASTEPNUM').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) I_META_MAX
        GOTO 10
C     ==-------- RESTART META DYNAMICS FROM METASTEPI_META_RES -------==
      ELSEIF(INDEX(LINE,'META_RESTART').NE.0) THEN
        META_RESTART = .TRUE.
        READ(IUNIT,ERR=20,END=20,FMT=*) I_META_RES
        GOTO 10
C     ==------------------ CV PARAMETERS  ----------------------------==
      ELSEIF(INDEX(LINE,'DEF').NE.0.
     &             AND. INDEX(LINE,'VARIABLE').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) INUM
        DO ICV = 1,INUM   
          READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
          CALL READSI(LINE,1,IOUT,IHT,ERREAD)
          II=INDEX(LINE,'SCA')
          IJ=INDEX(LINE,'SCF')
          IF(II.NE.0) THEN
            IA = II+3
            TAD_SCF(IHT) = .TRUE.
            CALL READSR(LINE,IA,IE,CSCL_FAC(1,IHT),   ERREAD)
            IA = IE
            CALL READSR(LINE,IA,IE,CSCL_FAC(2,IHT),   ERREAD)
            IA = IE
            CALL READSR(LINE,IA,IE,CSCL_FAC(3,IHT),   ERREAD)
            IF(ERREAD) GOTO 22
          ELSEIF(IJ.NE.0) THEN
            IA = IJ+3
            TAD_SCF(IHT) = .FALSE.
            CALL READSR(LINE,IA,IE,CSCL_FAC(1,IHT),   ERREAD)
            IF(ERREAD) GOTO 22
          ENDIF
        ENDDO
        GOTO 10
C     ==------------- DIMENSIONS OF HILLS IN CV SPACE-----------------==
      ELSEIF(INDEX(LINE,'HILLS').NE.0) THEN
        IF(INDEX(LINE,'OFF').NE.0) THEN
          LHILLS=.FALSE.
        ELSE
          LHILLS = .TRUE.
          IF(INDEX(LINE,'LOREN').NE.0) THEN
            HLORE = .TRUE.
          ELSEIF(INDEX(LINE,'RATIO').NE.0) THEN
            HRATIO = .TRUE.
            HSHIFT = .FALSE.
            IJ=INDEX(LINE,'POW')
            IF(IJ.NE.0) THEN
              IA = IJ+3
              CALL READSR(LINE,IA,IE,EXPUP,   ERREAD)
              IF(ERREAD) GOTO 20
              IA = IE
              CALL READSR(LINE,IA,IE,EXPDOWN,   ERREAD)
              IF(ERREAD) GOTO 20
              IA = IE
              CALL READSR(LINE,IA,IE,FBOOST,   ERREAD)
              IF(ERREAD) GOTO 20
            ENDIF
          ELSEIF(INDEX(LINE,'SHIFT').NE.0) THEN
            HSHIFT = .TRUE.
            HRATIO = .FALSE.
            IJ=INDEX(LINE,'RCUT')
            IF(IJ.NE.0) THEN
              IA = IJ+4
              CALL READSR(LINE,IA,IE,RSHIFT,   ERREAD)
              IF(ERREAD) GOTO 20
              IA = IE
              CALL READSR(LINE,IA,IE,FBOOST,   ERREAD)
              IF(ERREAD) GOTO 20
            ENDIF

          ENDIF
          II=INDEX(LINE,'=')
          IF(II.NE.0) THEN
            IA = II+1
            CALL READSR(LINE,IA,IE,HLLW,   ERREAD)
            IF(ERREAD) GOTO 20
            IA = IE
            CALL READSR(LINE,IA,IE,HLLH,   ERREAD)
            IF(ERREAD) GOTO 20
          ENDIF
        ENDIF
        GOTO 10
C     ==---------------------- TUNING PARAMETERS ---------------------==
      ELSEIF(INDEX(LINE,'TUNING').NE.0) THEN
        IF(INDEX(LINE,'OFF').NE.0) THEN
          LTUNE_HH     = .FALSE.
          LTUNE_CSCL   = .FALSE.
        ENDIF     
        IF(INDEX(LINE,'HHEIGHT').NE.0) THEN
          LTUNE_HH     = .TRUE.
          II=INDEX(LINE,'=')
          IF(II.NE.0) THEN
            IA = II+1
            CALL READSR(LINE,IA,IE,HLOW,   ERREAD)
            IF(ERREAD) GOTO 20
            IA = IE
            CALL READSR(LINE,IA,IE,HTOP,   ERREAD)
            IF(ERREAD) GOTO 20
            IF(HTOP.LT.HLOW) THEN
              TEMP = HLOW
              HLOW = HTOP
              HTOP = TEMP
            ENDIF
          ENDIF
        ENDIF
        GOTO 10
C     ==---------MAX # OF DYN. STEPS BETWEEN 2 METASTEPS--------------==
      ELSEIF(INDEX(LINE,'MAXSTEPNUM').NE.0 .AND.
     &             INDEX(LINE,'INTERMETA') .NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) INTER_HILL_MAX
        GOTO 10
C     ==---------MIN # OF DYN. STEPS BETWEEN 2 METASTEPS--------------==
      ELSEIF(INDEX(LINE,'MINSTEPNUM').NE.0 .AND.
     &             INDEX(LINE,'INTERMETA') .NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) INTER_HILL
        GOTO 10
C     ==------------ INTERVAL BETWEEN DISPLACEMENT CHECKS-------------==
      ELSEIF(INDEX(LINE,'CHECK') .NE.0 .AND.
     &             INDEX(LINE,'DELAY').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) ICHECK
        GOTO 10
C     ==------------------------TOLL_FCNST----------------------------==
      ELSEIF(INDEX(LINE,'MOVEMENT') .NE.0 .AND.
     &             INDEX(LINE,'CHECK').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) TOLL_AVCV
        GOTO 10
C     ==----------- STORE RESTART AND TRAJECTORIES -------------------==
      ELSEIF(INDEX(LINE,'METASTORE') .NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) ST_FREQ,TR_FREQ,QW_FREQ
        GOTO 10
C     ==----- MAX ELECT. KIN. ENERGY (when above QUENCH BO) ----------==
      ELSEIF(INDEX(LINE,'MAXKINEN').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
        CALL READSR(LINE,1,IOUT,TOLKIN,ERREAD)
        GOTO 10
C     =------ RESTRAIN THE CELL VOLUME IN THE RANGE VOLMIN VOLMAX ----==
      ELSEIF(INDEX(LINE,'RESTR').NE.0.AND.INDEX(LINE,'VOLU').NE.0)THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) FMIN,FMAX,VBARRIER
        VOLMIN = OMEGA0*FMIN
        VOLMAX = OMEGA0*FMAX
        TVOLBOUND = .TRUE.
        GOTO 10
      ELSE
        GOTO 10
      ENDIF  !file biginning

   20 CONTINUE
        WRITE(6,*) ' ERROR WHILE READING METADYNAMICS VARIABLES '
        WRITE(6,*) '       last line read: '
        WRITE(6,*)  LINE
        CALL STOPGM('M_CELL_INP',' ')      
 22     CONTINUE
        WRITE(6,*) ' ERROR WHILE READING SCAL. FAC. or MASS or KH.'
        CALL STOPGM('M_CELL_INP',' ') 

   30 CONTINUE
C     ==--------------------------------------------------------------==
C     TESTS

      IF(.NOT. TPRCP) CALL STOPGM('M_CELL_INP',
     &                  'VARIABLE CELL MD IS REQUIRED')
      IF(NCOLVAR .EQ. 0) CALL STOPGM('M_CELL_INP',
     &                  '# OF COLLECTIVE VARIABLES IS ZERO')
      DO IC = 1,NCOLVAR
        IF(TAD_SCF(IC)) THEN
          LTUNE_CSCL = .TRUE.
          GOTO 24
        ENDIF
      ENDDO
 24   CONTINUE
      IF(.NOT. LHILLS) THEN
        LTUNE_HH     = .FALSE.
        LTUNE_CSCL   = .FALSE.
      ENDIF

C     ==--------------------------------------------------------------==
C     Print info 
      WRITE(6,*) 
      WRITE(6,'(72("*"))')
      WRITE(6,'(1x,A,A)') 
     &         '***          META DYNAMICS OF THE VARIABLE CELL (PR)',
     &            '          ***'
      WRITE(6,'(4x,A,I5)') '- Number of Collective Variables =',
     &                  NCOLVAR
     
      IF(L2DC) WRITE(6,'(4x,A)') '- ONLY 2 SIDE LENGTHS ' 
      IF(LISOC) WRITE(6,'(4x,A)') '- SIDE LENGTHS '
      IF(LFULLC) WRITE(6,'(4x,A)') '- FREE CELL '   
      WRITE(6,'(6x,A)')     
     &   'colvar_mtd   : CV values and scaling factors'
      WRITE(6,'(6x,A/6x,A/6x,A)') 
     &   'dcolvar_mtd  : CV diffusivities,',
     &   '              norm of the displacement along the traj.',
     &   '              transversal hill width (fixed),',
     &   '              and hills heights HLLH'
       WRITE(6,'(6x,A/6x,A/6x,A)') 
     &   'forfact_mtd  : av. forces from underlying pot. ,',
     &   '               forces from time dependent pot. at this step',
     &   '               total  forces averaged between 2 metastep '

        WRITE(6,'(6x,A)')
     &   'dispvar_mtd  : CV displacements'

        WRITE(6,'(6x,A/6x,A/9x,A)') 
     &   'envar_mtd  : Ekin of the cell, ',
     &   '             gaussians pot. (time dependent term), ETOT',
     &   '            EHAM, EHAM + Gaussians '


      WRITE(6,'(6x,A,F12.6)')
     &    '- Minimum. CV Displacement Required to Place New a Hill',
     &    TOLL_AVCV
      WRITE(6,'(6x,A,I5,A)') 
     &   '- Update of Gaussian Pot. every ', 
     &           INTER_HILL,' MD steps (when CV moved enough)'
      WRITE(6,'(6x,A,I10)') 
     &   '- Max. Number of MD Steps before next Update',INTER_HILL_MAX
      WRITE(6,'(6x,A,I10,A)')
     &    '- RESTART File Saved every ', ST_FREQ, ' Metasteps'
      WRITE(6,'(6x,A,I10,A)')
     &    '- TRAJECTORY File Appended every', TR_FREQ,' Metasteps'
      WRITE(6,'(6x,A,I10,A)')
     &    '- QUENCH BO performed every', QW_FREQ, ' Metasteps'
      IF(TC .AND. TOLKIN .LT. EKINW+TOLL) THEN 
        WRITE(6,'(6x,A,F12.6)')
     &    '- QUENCH BO performed when EL. Ekin is > ', TOLKIN
      ELSEIF(TOLKIN.NE.-1.0D0) THEN
        WRITE(6,'(6x,A,F12.6)')
     &    '- QUENCH BO performed when EL. Ekin is > ', TOLKIN
      ENDIF

      IF(META_RESTART) WRITE(6,'(6x,A,1X,I8)') 
     &       '- Restart from a previous Metadynamics at step',I_META_RES
      IF(LHILLS) THEN 
        WRITE(6,'(6x,A)')
     &           '- Time Dependent Potential Active: '
        WRITE(6,'(15x,A,F9.5,A,F9.5)') ' Transversal Width = ',
     &                   HLLW,'    Initial Height = ',HLLH
        IF(HSHIFT) THEN
          WRITE(6,'(6x,A/15x,A,F8.3,A,f8.3)') 
     &                '- Hills Shape : Truncated Gaussians',
     &                ' Rcut = ',RSHIFT,'     Width Factor = ',FBOOST
        ELSEIF(HRATIO) THEN
          WRITE(6,'(6x,A/15x,A,F8.3,F8.3,A,f8.3)') 
     &          '- Hills Shape : Rational Function',
     &          ' Exponents : ',EXPUP,EXPDOWN,
     &          '     Width Factor = ',FBOOST
        ENDIF
        IF(LTUNE_HH) 
     &    WRITE(6,'(6x,A/15x,A,F10.6,A,f10.6,A/)')
     &      '- Hills Height tuned on the Underlying potential',
     &      'min. Height ',HLOW*AU_KCM,' Kcal,    max Height ',
     &         HTOP*AU_KCM,' Kcal.'
        IF(LTUNE_CSCL)
     &    WRITE(6,'(6x,A)') 
     &      '- Some scaling factors might be adjusted along the run'
      ENDIF

      IF(TVOLBOUND) THEN
        WRITE(6,'(4x,A,f12.5,A,f12.5,A)') 
     &            '- Volume Restrained in range: ',
     &               VOLMIN, '-', VOLMAX, ' a.u.'
      ENDIF

      WRITE(6,'(6x,A)') '- CV Parameters: '
      DO IHT = 1,NCOLVAR
        WRITE(6,'(6x,A,I4,A,f8.4)') 
     &                  '          ', iht, ' SC ', CSCL_FAC(1,IHT)
      ENDDO
 
c      stop 'input'
      WRITE(6,'(80("*"))')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PARFULL(HT,HT_IST,DET)
      IMPLICIT NONE

c      include 'metr.inc'

      REAL*8  HT(3,3),HT_IST(6),DET(9,6)
      REAL*8  al,bl,cl,cosa,cosb,cosg
      REAL*8  HTEMP(3,3,6),num
      REAL*8  ual,ubl,ucl,ualbl,ualcl,ublcl

      al = HT(1,1)*HT(1,1) +
     &     HT(1,2)*HT(1,2) +
     &     HT(1,3)*HT(1,3)
      al = DSQRT(al)

      bl = HT(2,1)*HT(2,1) +
     &     HT(2,2)*HT(2,2) +
     &     HT(2,3)*HT(2,3)
      bl = dsqrt(bl)
      cosg = HT(1,1)*HT(2,1)+
     &       HT(1,2)*HT(2,2)+
     &       HT(1,3)*HT(2,3)
      cosg = cosg/(al*bl)

      cl = HT(3,1)*HT(3,1) +
     &     HT(3,2)*HT(3,2) +
     &     HT(3,3)*HT(3,3)
      cl = dsqrt(cl)

      ual = 1.0D0/al
      ubl = 1.0D0/bl
      ucl = 1.0D0/cl
      ualbl = ual*ubl
      ualcl = ual*ucl
      ublcl = ubl*ucl

      cosg = HT(1,1)*HT(2,1)+
     &       HT(1,2)*HT(2,2)+
     &       HT(1,3)*HT(2,3)
      cosg = cosg*ualbl

      cosb = HT(1,1)*HT(3,1)+
     &       HT(1,2)*HT(3,2)+
     &       HT(1,3)*HT(3,3)
      cosb = cosb*ualcl

      cosa = HT(2,1)*HT(3,1)+
     &       HT(2,2)*HT(3,2)+
     &       HT(2,3)*HT(3,3)
      cosa = cosa*ublcl

      CALL AZZERO(HTEMP,9*6)
       

      htemp(1,1,1) =ual*HT(1,1)
      htemp(1,2,1) =ual*HT(1,2)
      htemp(1,3,1) =ual*HT(1,3)

      htemp(2,1,2) =ubl*HT(2,1)
      htemp(2,2,2) =ubl*HT(2,2)
      htemp(2,3,2) =ubl*HT(2,3)

      htemp(3,1,3) =ucl*HT(3,1)
      htemp(3,2,3) =ucl*HT(3,2)
      htemp(3,3,3) =ucl*HT(3,3)

      num =  HT(2,1)*HT(3,1)+ HT(2,2)*HT(3,2)+ HT(2,3)*HT(3,3)
      htemp(2,1,4) =ublcl*(ht(3,1)-num*htemp(2,1,2)*ubl)
      htemp(3,1,4) =ublcl*(ht(2,1)-num*htemp(3,1,3)*ucl)
      htemp(2,2,4) =ublcl*(ht(3,2)-num*htemp(2,2,2)*ubl)
      htemp(3,2,4) =ublcl*(ht(2,2)-num*htemp(3,2,3)*ucl)
      htemp(2,3,4) =ublcl*(ht(3,3)-num*htemp(2,3,2)*ubl)
      htemp(3,3,4) =ublcl*(ht(2,3)-num*htemp(3,3,3)*ucl)


      num =  HT(1,1)*HT(3,1)+ HT(1,2)*HT(3,2)+ HT(1,3)*HT(3,3)
      htemp(1,1,5) =ualcl*(ht(3,1)-num*htemp(1,1,1)*ual)
      htemp(3,1,5) =ualcl*(ht(1,1)-num*htemp(3,1,3)*ucl)
      htemp(1,2,5) =ualcl*(ht(3,2)-num*htemp(1,2,1)*ual)
      htemp(3,2,5) =ualcl*(ht(1,2)-num*htemp(3,2,3)*ucl)
      htemp(1,3,5) =ualcl*(ht(3,3)-num*htemp(1,3,1)*ual)
      htemp(3,3,5) =ualcl*(ht(1,3)-num*htemp(3,3,3)*ucl)

      num =  HT(1,1)*HT(2,1)+ HT(1,2)*HT(2,2)+ HT(1,3)*HT(2,3)
      htemp(1,1,6) =ualbl*(ht(2,1)-num*htemp(1,1,1)*ual)
      htemp(2,1,6) =ualbl*(ht(1,1)-num*htemp(2,1,2)*ubl)
      htemp(1,2,6) =ualbl*(ht(2,2)-num*htemp(1,2,1)*ual)
      htemp(2,2,6) =ualbl*(ht(1,2)-num*htemp(2,2,2)*ubl)
      htemp(1,3,6) =ualbl*(ht(2,3)-num*htemp(1,3,1)*ual)
      htemp(2,3,6) =ualbl*(ht(1,3)-num*htemp(2,3,2)*ubl)


      CALL AZZERO(DET,6*6)

      CALL DCOPY(9,HTEMP(1,1,1),1,DET(1,1),1)
      CALL DCOPY(9,HTEMP(1,1,2),1,DET(1,2),1)
      CALL DCOPY(9,HTEMP(1,1,3),1,DET(1,3),1)
      CALL DCOPY(9,HTEMP(1,1,4),1,DET(1,4),1)
      CALL DCOPY(9,HTEMP(1,1,5),1,DET(1,5),1)
      CALL DCOPY(9,HTEMP(1,1,6),1,DET(1,6),1)

      HT_IST(1) = al
      HT_IST(2) = bl
      HT_IST(3) = cl
      HT_IST(4) = cosa
      HT_IST(5) = cosb
      HT_IST(6) = cosg

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PARISO(HT,HT_IST,DET)
      IMPLICIT NONE

c      include 'metr.inc'

      REAL*8  HT(3,3),HT_IST(3),DET(6,3)
      REAL*8  al,bl,cl,cosa,cosb,cosg
      REAL*8  HTEMP(3,3,3)

      al = HT(1,1)*HT(1,1) +
     &     HT(1,2)*HT(1,2) +
     &     HT(1,3)*HT(1,3)
      al = DSQRT(al)

      bl = HT(2,1)*HT(2,1) +
     &     HT(2,2)*HT(2,2) +
     &     HT(2,3)*HT(2,3)
      bl = dsqrt(bl)
      cosg = HT(1,1)*HT(2,1)+
     &       HT(1,2)*HT(2,2)+
     &       HT(1,3)*HT(2,3)
      cosg = cosg/(al*bl)

      cl = HT(3,1)*HT(3,1) +
     &     HT(3,2)*HT(3,2) +
     &     HT(3,3)*HT(3,3)
      cl = dsqrt(cl)
      cosb = HT(1,1)*HT(3,1)+
     &       HT(1,2)*HT(3,2)+
     &       HT(1,3)*HT(3,3)
      cosb = cosb/(al*cl)
      cosa = HT(2,1)*HT(3,1)+
     &       HT(2,2)*HT(3,2)+
     &       HT(2,3)*HT(3,3)
      cosa = cosa/(bl*cl)


      CALL AZZERO(HTEMP,3*3*3)

      htemp(1,1,1) =1.D0/al*HT(1,1)
      htemp(1,2,1) =1.D0/al*HT(1,2)
      htemp(1,3,1) =1.D0/al*HT(1,3)

      htemp(2,1,2) =1.D0/bl*HT(2,1)
      htemp(2,2,2) =1.D0/bl*HT(2,2)
      htemp(1,3,2) =1.D0/bl*HT(2,3)

      htemp(3,1,3) =1.D0/cl*HT(3,1)
      htemp(3,2,3) =1.D0/cl*HT(3,2)
      htemp(3,3,3) =1.D0/cl*HT(3,3)

      CALL AZZERO(DET,6*3)

      CALL DCOPY(9,HTEMP(1,1,1),1,DET(1,1),1)
      CALL DCOPY(9,HTEMP(1,1,2),1,DET(1,2),1)
      CALL DCOPY(9,HTEMP(1,1,3),1,DET(1,3),1)

      HT_IST(1) = al
      HT_IST(2) = bl
      HT_IST(3) = cl

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE DELROT_CELL(HA,HB)
      IMPLICIT NONE

C     Arguments 
      REAL*8  HA(3,3),HB(3,3)

C     Variables
      REAl*8  ROT(3,3),TRANSL(3)
      REAL*8  X,Y,Z,XT,YT,ZT
      REAL*8  VEC_A(3,4),VEC_B(3,4)
      INTEGER IND(4),IV
      REAL*8  D(3,4),D_R(3,3,3,4)

      VEC_A(1,1) = 0.0D0
      VEC_A(2,1) = 0.0D0
      VEC_A(3,1) = 0.0D0

      VEC_B(1,1) = 0.0D0
      VEC_B(2,1) = 0.0D0
      VEC_B(3,1) = 0.0D0
      do iv = 1,4
        ind(iv) = iv
      enddo
      do IV = 1,3
        VEC_A(1,IV+1) = HA(IV,1)
        VEC_A(2,IV+1) = HA(IV,2)
        VEC_A(3,IV+1) = HA(IV,3)
        VEC_B(1,IV+1) = HB(IV,1)
        VEC_B(2,IV+1) = HB(IV,2)
        VEC_B(3,IV+1) = HB(IV,3)
      enddo  

      CALL rmsd_rot(4,VEC_B,VEC_A,ind,transl,rot,4,d,d_r,.true.)

      DO IV = 1,3
        xt = VEC_B(1,IV+1)
        yt = VEC_B(2,IV+1)
        zt = VEC_B(3,IV+1)
        x =rot(1,1)*xt + rot(1,2)*yt + rot(1,3)*zt
        y =rot(2,1)*xt + rot(2,2)*yt + rot(2,3)*zt
        z =rot(3,1)*xt + rot(3,2)*yt + rot(3,3)*zt
        HB(IV,1) = x
        HB(IV,2) = y
        HB(IV,3) = z
      ENDDO

      RETURN
      END     
C     ==================================================================
