C     ==================================================================
      SUBROUTINE META_COLVAR(TAUP,VELP,FION,TSCR,SCR,LSCR,
     &                       LQUENCH,LMETARES,EKINC,EKINP)
C     ==--------------------------------------------------------------==
C     ==  Calculates the contributions to the forces on ions          ==
C     ==  due to the hills that have been accumulated in the space    ==
C     ==  of the defined collective variables the hills and to the    ==
C     ==  walls that in that same space limit the wandering of the CV ==
C     ==  F_HILL  and   F_WALL
C     ==  The variation of the thajectory due to these forces is also ==
C     ==  calculated and the ions coordinated updated                 ==
C     ==--------------------------------------------------------------==

C     NCOLVAR   : # of collective variables
C     LHILLS    : true when hills are used in the dynamics of the constraints
C     I_META    : # of steps in the meta-dynamics
C     I_META_MAX: max # of steps in the meta-dynamics
C     I_META_RES: if restart from previous meta dynamics, 
C                 # of steps already done
C     I_CVST    : # of steps since last HILL has been added
C     CV_IST    : actual values of collective variables, dim NCOLVAR
C     CVD_IST   : actual values of diffusion coefficient of CV
C     HLLH      : hills height 
C     HLLW      : hills amplitude 
C     CSCL_FAC  : scale factors required for an homogeneous dynamic in the
C                 NCOLVAR dimensions of the CV space 
C                 they can be read from input and can be tuned on the 
C                 amplitudes of the collective variables fluctuations
C     IBOUND    : 1 if a wall constraining the CV within a certain region of space
C                 otherwise 0 (integer, dimension NCOLVAR)
C     VBOUND    : parameters for the definition of constraining potential
C                 (real, dimension 4*NCOLVAR)
C                 if WALL+ VBOUND(2) is the barrier for CV_DYN > VBOUND(1)
C                 if WALL- VBOUND(4) is the barrier for CV_DYN < VBOUND(3)
C     F_HILL     : force contribution to the ions due to the accumulation 
C                  of hills  in the space of CV 
C                  dim (NODIM)
C     F_WALL     : force contribution due to the constraining VBOUND
C                  dim (NODIM)
C     META_RESTART: restart for a previous meta-dynamics
C                   acculators and the constraints path have to be read 
C                   from the right files
C                   the output is appended 
C     CV_PATH    :  CV history (dim NTOT_ITER*NCOLVAR)
C     CSCL_VAL   :  CV scaling factors history (dim NTOT_ITER*NCOLVAR)
C     HLLW_VAL   :  hills amplitude (dim NTOT_ITER)
C     HLLH_VAL   :  hills altitude  (dim NTOT_ITER)
C     F_AVER     :  accumulator for the calculation of the average force
C                   acting on the CV, as a contribution of the 
C                   underlaying potential 
C     IF_AVER    :  counter for the calculation of the average
C     DISPLACEMENT: norm of the displacment in CV space
C     ==--------------------------------------------------------------==

      IMPLICIT NONE
      INCLUDE  'system.h'
      INCLUDE  'ions.inc'
      INCLUDE  'cotr.inc'
      INCLUDE  'cnst_dyn.inc'
      INCLUDE  'irat.inc'
      INCLUDE  'tpar.inc'
      INCLUDE  'ropt.inc'
      INCLUDE  'soft.inc'
      INCLUDE  'store.inc'
      INCLUDE  'adat.inc'
      INCLUDE  'cnst.inc'
      INCLUDE  'strs.inc'
      INCLUDE  'fileopen.inc'

C     Arguments
      INTEGER       LSCR
      REAL*8        TAUP(3,NAX,NSX),VELP(3,NAX,NSX),
     &              FION(3,NAX,NSX),TSCR(3,NAX,NSX),SCR(LSCR),
     &              EKINC,EKINP
      LOGICAL       LQUENCH,LMETARES
C     Variables
      INTEGER       NTOT_ITER
      REAL*8        F_HILL(NODIM),F_WALL(NODIM)
      pointer       (IP_F_HILL,F_HILL),(IP_F_WALL,F_WALL)

      REAL*8        CVD_IST(NCOLVAR),CV_AVER(NCOLVAR),CV_LAST(NCOLVAR),
     &              F1_CV(NCOLVAR),F2_CV(NCOLVAR),F_AVER(NCOLVAR),
     &              HH_TEST(NCOLVAR),CV_SCL(NCOLVAR),CVD_SCL(NCOLVAR),
     &              HHH,HHHT

      REAL*8        CV_STORE(INTER_HILL,NCOLVAR)
      pointer      (IP_CVD_IST,CVD_IST),(IP_CV_AVER,CV_AVER),
     &             (IP_CV_LAST,CV_LAST),
     &             (IP_F1_CV,F1_CV),(IP_F2_CV,F2_CV),
     &             (IP_F_AVER,F_AVER),(IP_HH_TEST,HH_TEST),
     &             (IP_CV_SCL,CV_SCL),(IP_CVD_SCL,CVD_SCL),
     &             (IP_CV_STORE,CV_STORE)
      SAVE          IP_CVD_IST,IP_CV_SCL,IP_CVD_SCL 
      DIMENSION     DOF_INCV(NODIM,NCOLVAR,18),
     &              DET_COLVAR(NODIM,NCOLVAR),DET_CELVAR(3,3,NCOLVAR)
      DIMENSION     CV_PATH(I_META_MAX+I_META_RES,NCOLVAR),
     &              CSCL_VAL(I_META_RES+I_META_MAX,NCOLVAR),
     &              HLLW_VAL(I_META_RES+I_META_MAX),
     &              HLLH_VAL(I_META_RES+I_META_MAX)
      DIMENSION     VBOUND(4,NCOLVAR),CSCL_FAC(3,NCOLVAR),
     &              IBOUND(NCOLVAR)

      INTEGER       IFIRST,I_META,I_CVST,I_TEMP,IF_AVER
      DATA          IFIRST,I_META,I_CVST,I_TEMP,IF_AVER / 5*0 /
      SAVE          IFIRST,I_META,I_CVST,I_TEMP,IF_AVER

      CHARACTER*20  FILE1,FILE2
      DATA          FILE1,FILE2 /'colvar_mtd','parvar_mtd'/

      LOGICAL       HILL_ADD,POSINST,FINAL
      LOGICAL       TUNE1,TUNE2,LSKIP,FERROR
      CHARACTER     TAG

      SAVE          IP_CV_STORE,IP_CV_LAST,IP_F_AVER,IP_F1_CV,IP_F2_CV

      REAL*8        TOLLM,MAXF2,FACT,DIF_CV,DISP2,HLLW2,
     &              HH_MIN,HH_MAX,HH,HLLH0,DISPLACEMENT
      INTEGER       ISUB,II,ICV,IJ,LENGTH,ISC2,IA,IS,
     &              LSCR2,IP,IDOF,msglen
      INTEGER       L_F_HILL,L_F_WALL,L_CV_AVER,L_F_CV,
     &              L_CV_SCL,L_CVD_SCL,L_SCR_HIL
      SAVE          HLLH0
C     ==--------------------------------------------------------------==
C     Set time for the routine
      CALL TISET(' META_COLV',ISUB)

C     ==--------------------------------------------------------------==
      CALL AZZERO(SCR,LSCR)
      IF(.NOT.PARENT) GOTO 9999
      FINAL = .FALSE.

C     ==--------------------------------------------------------------==
C     Temporary Array

#ifdef _BGL 
      L_F_HILL  =  NODIM+MOD(NODIM,2)
      L_F_WALL  =  NODIM+MOD(NODIM,2)
      L_CV_AVER = NCOLVAR+MOD(NCOLVAR,2)
      L_F_CV    = NCOLVAR+MOD(NCOLVAR,2)
      L_CV_SCL  = NCOLVAR+MOD(NCOLVAR,2)
      L_CVD_SCL = NCOLVAR+MOD(NCOLVAR,2)
      L_SCR_HIL = MAX(4*NCOLVAR,2*NCOLVAR*NCOLVAR+NODIM+NODIM*NCOLVAR)


      LENGTH  = L_F_HILL+L_F_WALL+L_CV_AVER+L_F_CV+L_CV_SCL+
     &          L_CVD_SCL+L_SCR_HIL
#else
      L_F_HILL  =  NODIM
      L_F_WALL  =  NODIM
      L_CV_AVER = NCOLVAR
      L_F_CV    = NCOLVAR
      L_CV_SCL  = NCOLVAR
      L_CVD_SCL = NCOLVAR
      L_SCR_HIL = MAX(4*NCOLVAR,2*NCOLVAR*NCOLVAR+NODIM+NODIM*NCOLVAR)


      LENGTH  = L_F_HILL+L_F_WALL+L_CV_AVER+L_F_CV+L_CV_SCL+
     &          L_CVD_SCL+L_SCR_HIL
#endif

      CALL TEST_SCR('META_COLVAR',TAG,LSCR,LENGTH)

      CALL  GET_ADDR(IP_F_HILL ,SCR(1))
      CALL  GET_ADDR(IP_F_WALL ,SCR(1+L_F_HILL))
      CALL  GET_ADDR(IP_CV_AVER,SCR(1+L_F_HILL+L_F_WALL))
      CALL  GET_ADDR(IP_HH_TEST ,SCR(1+L_F_HILL+L_F_WALL+L_CV_AVER))
      CALL  GET_ADDR(IP_CV_SCL ,SCR(1+L_F_HILL+L_F_WALL+L_CV_AVER+
     &                            L_F_CV))
      CALL  GET_ADDR(IP_CVD_SCL,SCR(1+L_F_HILL+L_F_WALL+L_CV_AVER+
     &                            L_F_CV+L_CV_SCL))
      ISC2         = 1+L_F_HILL+L_F_WALL+L_CV_AVER+
     &                            L_F_CV+L_CV_SCL+L_CVD_SCL
      LSCR2        = LSCR - L_F_HILL+L_F_WALL+L_CV_AVER+
     &                            L_F_CV+L_CV_SCL+L_CVD_SCL

      NTOT_ITER = I_META_MAX+I_META_RES
      TOLLM     = TOLL_AVCV

      IF(IFIRST .EQ. 0) THEN
        I_META  = I_META_RES+1
        I_TEMP  = 0
        I_CVST  = 0
        IF_AVER = 0

C     Allocate Memory

        CALL MEMORY(IP_CVD_IST,NCOLVAR,'CVD_IST')
        CALL AZZERO(CVD_IST,NCOLVAR)
        CALL MEMORY(IP_CV_STORE,NCOLVAR*INTER_HILL,'CV_STORE')
        CALL AZZERO(CV_STORE,NCOLVAR*INTER_HILL)
        CALL MEMORY(IP_CV_LAST,NCOLVAR,'CV_LAST')
        CALL AZZERO(CV_LAST,NCOLVAR)
        CALL MEMORY(IP_F_AVER,NCOLVAR,'F_AVER')
        CALL AZZERO(F_AVER,NCOLVAR)
        CALL MEMORY(IP_F2_CV,NCOLVAR,'F2_CV')
        CALL AZZERO(F2_CV,NCOLVAR)
        CALL MEMORY(IP_F1_CV,NCOLVAR,'F1_CV')
        CALL AZZERO(F1_CV,NCOLVAR)

        CALL MEMORY(IP_CV_PATH,NCOLVAR*NTOT_ITER,'CV_PATH')
        CALL AZZERO(CV_PATH,NCOLVAR*NTOT_ITER)
        CALL MEMORY(IP_CSCL_VAL,NCOLVAR*NTOT_ITER,'CSCL_VAL')
        CALL AZZERO(CSCL_VAL,NCOLVAR*NTOT_ITER)
        CALL MEMORY(IP_HLLW_VAL,NTOT_ITER,'HLLH_VAL')
        CALL AZZERO(HLLW_VAL,NTOT_ITER)
        CALL MEMORY(IP_HLLH_VAL,NTOT_ITER,'HLLH_VAL')
        CALL AZZERO(HLLH_VAL,NTOT_ITER)


C     If META_RESTART read data from proper input
        IF(META_RESTART) THEN
          IF(TRESFILE) THEN
            CALL RMTDRES(50,CVD_IST,.TRUE.,NTOT_ITER)
          ELSE
            CALL FILEOPEN(51,FILE1,FO_OLD,FERROR)
            CALL FILEOPEN(52,FILE2,FO_OLD,FERROR)
            REWIND (51)
            REWIND (52)
            DO II = 1,I_META_RES
              READ(51,ERR=20,END=20,FMT=*)  IJ, 
     &                    (CV_PATH(II,ICV),ICV=1,NCOLVAR),
     &                    (CSCL_VAL(II,ICV),ICV=1,NCOLVAR)
C            from output forces and parameters
              READ(52,ERR=20,END=20,FMT=*) IJ, 
     &                    DISPLACEMENT,HLLW_VAL(II),HLLH_VAL(II)
            ENDDO
            CALL FILECLOSE(51)
            CALL FILECLOSE(52)
            GOTO 100
 20         CONTINUE
            WRITE(6,*) ' ERROR WHILE READING RESTART DATA '
            CALL STOPGM('META_COLVAR',' ')
 100        CONTINUE
          ENDIF  ! TRESFILE
        ENDIF   !META_RESTART
      ENDIF

C     ==--------------------------------------------------------------==
C     Calculate the Collective Variables and their Derivatives
      CALL COLVAROFR(TAUP,TSCR)

C     ==--------------------------------------------------------------==
C     Store CV values and Check averages
      
      HILL_ADD = .FALSE.
      ROUT = .FALSE.
      I_CVST = I_CVST + 1

      POSINST = .TRUE.
      TUNE1   = .FALSE.
      TUNE2   = .TRUE.

      IF(IFIRST .EQ. 0) THEN
!$OMP parallel do private(ICV)
        DO ICV = 1,NCOLVAR
          CV_LAST(ICV) = CV_IST(ICV)
        ENDDO
        IFIRST = 1
        HLLH0 = HLLH
      ENDIF

      IF(.NOT. POSINST) THEN
        IP = ISC2
        CALL CALC_AVER(TOLLM,I_CVST,I_TEMP,I_META,CV_STORE,CV_LAST,
     &                     CV_AVER,SCR(IP),LSCR2,HILL_ADD)
      ELSEIF(POSINST) THEN
        IP = ISC2
        CALL CALC_POS(TOLLM,I_CVST,I_TEMP,I_META,CV_LAST,
     &                     CV_AVER,SCR(IP),LSCR2,HILL_ADD)
      ENDIF
      IF(LMETARES) THEN
        FINAL = .TRUE.
        GOTO 9999
      ENDIF
C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     Calculation of the forces on the collective variables
C     This is used in order to determine the optimal hills parameters 
C     In F_aver the forces on CV are accumulated in order to calculate
C     their average value when the meta step is concluded

      IF(TUNE1 .AND. LTUNE_HH ) THEN
        CALL TUNE_HEIGHT(F_AVER,VELP,FION,TSCR,SCR(ISC2),LSCR2)
        IF_AVER = IF_AVER + 1
      ELSEIF(TUNE2 .AND. LTUNE_HH ) THEN
        CALL CV_FORCES3(TAUP,TSCR,VELP,HH_TEST,SCR(ISC2),LSCR2)
!$OMP parallel do private(ICV)
        DO ICV = 1,NCOLVAR
          F_AVER(ICV) = F_AVER(ICV) + HH_TEST(ICV)
        ENDDO
        IF_AVER = IF_AVER + 1
      ENDIF

C     ==--------------------------------------------------------------==
C     New Step of Meta-Dynamics in th Space of CV
      IF(HILL_ADD) THEN
        IF(MOD(I_META,TR_FREQ) .EQ. 0 .OR. I_META .EQ. 1 ) ROUT = .TRUE.

C     ==--------------------------------------------------------------==
C     Squared Norm of the Displacement in the CV space

        DISPLACEMENT = 0.0D0
        DO ICV = 1,NCOLVAR
          DIF_CV = CV_AVER(ICV)-CV_LAST(ICV)
          IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
            DIF_CV = DIF_CV - 2.0D0*PI
          ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
            DIF_CV = DIF_CV + 2.0D0*PI
          ENDIF
          FACT = DIF_CV/CSCL_FAC(1,ICV)
          DISPLACEMENT = DISPLACEMENT + FACT*FACT
        ENDDO
        DISP2 = DISPLACEMENT
        DISPLACEMENT = DSQRT(DISPLACEMENT)

C     ==--------------------------------------------------------------==
C     Tuning Hills parameters 

        IF(LTUNE_HH) THEN
          HH_MIN = 1000.D0
          HH_MAX = 0.0D0
          LSKIP = .FALSE.
          HLLW2 = HLLW*HLLW
          
          HHHT = 0.0D0
          DO ICV = 1,NCOLVAR
            F_AVER(ICV)  = F_AVER(ICV) / DBLE(IF_AVER)
            DIF_CV = CV_AVER(ICV)-CV_LAST(ICV)
            IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
              DIF_CV = DIF_CV - 2.0D0*PI
            ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
              DIF_CV = DIF_CV + 2.0D0*PI
            ENDIF

            FACT = CSCL_FAC(1,ICV)*CSCL_FAC(1,ICV)/DIF_CV *
     &              dexp(0.5D0)*HLLW2*DISP2/
     &             (DISP2*dexp(-.5D0*DISP2/HLLW2)+
     &               HLLW2*(dexp(-.5D0*DISP2/HLLW2)-
     &               dexp(-.5D0*RSHIFT*RSHIFT)))


            HHH     = 0.5D0*dabs(F_AVER(ICV)*FACT)
            HHHT = HHHT + HHH

            WRITE(6,'(2x,A,I4,3F14.6,2x,2F14.6)') 'FCV ',
     &           ICV,F_AVER(ICV),HHH,FACT

            HH_MIN       = MIN(HH_MIN,HHH)
            HH_MAX       = MAX(HH_MAX,HHH)
          ENDDO
          HH       =  HHHT / DBLE(NCOLVAR)
          IF(HH_MIN .GT. HTOP .OR. HH_MAX .LT. HLOW) THEN
            LSKIP = .TRUE.
            WRITE(6,'(A,f10.5,A,f10.5)') 'Hill top > ', HTOP*AU_KCM,
     &                 ' Kcal, or Hill top < ',HLOW*AU_KCM,' Kcal'
          ENDIF
          IF(.NOT. LSKIP .AND. HH .LT. HLOW) THEN
            HLLH = (HLOW+HH_MAX)*0.5D0
          ELSEIF(.NOT. LSKIP .AND. HH .GT. HTOP) THEN
            HLLH = (HH_MIN+HTOP)*0.5D0
          ELSEIF(.NOT. LSKIP ) THEN
            HLLH = HH
          ELSE
            HLLH = HLLH*0.2D0+HLLH0*0.8D0
          ENDIF

          WRITE(6,'(/A,f10.5,3(3x,A,f10.5)/)')
     &              '||DISP|| = ',DISPLACEMENT, 
     &              'HH tuned min = ', HH_MIN,'HH tuned max = ', HH_MAX,
     &              ' HH used ',HLLH
          IF_AVER = 0
          CALL AZZERO(F_AVER,NCOLVAR)

          IF(THILLVOL) THEN
            HLLW = (HVOL0/HLLH)**(1.0D0/DBLE(NCOLVAR))
          ENDIF
        ENDIF

C     ==--------------------------------------------------------------==
C     Diffusion calculation

        CALL CV_DIFFUSION(CV_LAST,CVD_IST,CVD_SCL,CV_SCL,I_META,
     &            SCR(ISC2),LSCR2,NTOT_ITER)

C     ==--------------------------------------------------------------==

C     Open output file
        IF(PARENT)
     &   CALL CV_WRITE_OUT(I_META,CV_LAST,CV_AVER,F1_CV,F2_CV,
     &           CVD_IST,CV_SCL,CVD_SCL,DISPLACEMENT,EKINC,EKINP)
C     ==--------------------------------------------------------------==
!$OMP parallel do private(ICV)
        DO ICV = 1,NCOLVAR
          CV_LAST(ICV)   = CV_AVER(ICV)
        ENDDO

        I_META = I_META + 1
        I_TEMP = 0
        I_CVST = 0
      ENDIF        !HILL_ADD

C     ==--------------------------------------------------------------==
C     Calculation of the contributions to the forces on ions 
C     due to HILLS and WALLS in the space of CV

C     Calculate hills contribution to forces
      IF(LHILLS) THEN
        CALL AZZERO(F_HILL,NODIM)
        CALL AZZERO(F1_CV,NCOLVAR)
        IF(HLORE) THEN
          CALL HILLS_LOR(CV_IST,NCOLVAR,I_META,NTOT_ITER,F1_CV,
     &                   SCR(ISC2),LSCR2,1,0)
        ELSEIF(HRATIO) THEN
          CALL HILLS_RATIO(CV_IST,NCOLVAR,I_META,NTOT_ITER,F1_CV,
     &         CV_LAST,SCR(ISC2),LSCR2,I_CVST,1,0)
        ELSEIF(HSHIFT) THEN
          CALL HILLS_SALS_SHIFT(CV_IST,NCOLVAR,I_META,NTOT_ITER,F1_CV,
     &         CV_LAST,SCR(ISC2),LSCR2,I_CVST,1,0)
      ELSE IF(SPHERE) THEN
          CALL HILLS(CV_IST,NCOLVAR,I_META,NTOT_ITER,F1_CV,
     &                   SCR(ISC2),LSCR2,1,0)
        ELSE
          CALL HILLS_SALS(CV_IST,NCOLVAR,I_META,NTOT_ITER,F1_CV,
     &         CV_LAST,SCR(ISC2),LSCR2,I_CVST,1,0)
        ENDIF

        DO ICV = 1,NCOLVAR
          DO IDOF = 1,NODIM
            F_HILL(IDOF) = F_HILL(IDOF) + 
     &                     F1_CV(ICV)*
     &                     DET_COLVAR(IDOF,ICV)/CSCL_FAC(1,ICV)
          ENDDO
        ENDDO
      ENDIF

C     Calculate walls contribution to forces
      CALL AZZERO(F_WALL,NODIM)
      CALL AZZERO(F2_CV,NCOLVAR)
      MAXF2 = 0.0D0
      DO ICV = 1,NCOLVAR
        IF(IBOUND(ICV) .EQ. 1) THEN
          CALL SETWALL(CV_IST(ICV),VBOUND(1,ICV),F2_CV(ICV))
        ELSEIF(IBOUND(ICV) .EQ. 2) THEN
          CALL SETWALL_OLD(CV_IST(ICV),TYCVAR(ICV),VBOUND(1,ICV),
     &                     F2_CV(ICV))
        ENDIF
        MAXF2 = MAX(MAXF2,DABS(F2_CV(ICV)))
        IF(F2_CV(ICV) .NE. 0.0D0) THEN
          DO IDOF = 1,NODIM
            F_WALL(IDOF) = F_WALL(IDOF) + 
     &                     F2_CV(ICV)*
     &                     DET_COLVAR(IDOF,ICV)/CSCL_FAC(1,ICV)
          ENDDO
        ENDIF
      ENDDO


C     ==--------------------------------------------------------------==
C     Update forces on ions

      CALL AZZERO(TSCR,3*NAX*NSX)
      CALL GETTAU(TSCR,F_HILL)
#ifdef __SR11000
*poption parallel, tlocal(IS,IA,FACT)
*voption indep(DTB2MI,FHILLS,TSCR)
#endif
      DO IS=1,NSP
        FACT= DT_IONS*DTB2MI(IS)
        DO IA=1,NA(IS)
           FHILLS(1,IA,IS) = TSCR(1,IA,IS)
           FHILLS(2,IA,IS) = TSCR(2,IA,IS)
           FHILLS(3,IA,IS) = TSCR(3,IA,IS)
        ENDDO
      ENDDO

      IF(MAXF2 .GT. 1.0D-12) THEN
        CALL AZZERO(TSCR,3*NAX*NSX)
        CALL GETTAU(TSCR,F_WALL)
#ifdef __SR11000
*poption parallel, tlocal(IS,IA,FACT)
*voption indep(DTB2MI,FHILLS,TSCR)
#endif
        DO IS=1,NSP
          FACT= DT_IONS*DTB2MI(IS)
          DO IA=1,NA(IS)
           FHILLS(1,IA,IS) = FHILLS(1,IA,IS)+TSCR(1,IA,IS)
           FHILLS(2,IA,IS) = FHILLS(2,IA,IS)+TSCR(2,IA,IS)
           FHILLS(3,IA,IS) = FHILLS(3,IA,IS)+TSCR(3,IA,IS)
          ENDDO
        ENDDO
      ENDIF

C     ==--------------------------------------------------------------==
C     Update stress tensor: contributions of CV as functions of HT
      IF(TCVCELL) THEN
        CALL AZZERO(DSTRCV,6)
        DO ICV = 1,NCOLVAR
          FACT = F2_CV(ICV)/CSCL_FAC(1,ICV)

          DO II = 1,6
            DSTRCV(II) = DSTRCV(II) + 
     &                   FACT*DET_CELVAR(ALPHA(II),BETA(II),ICV)
          ENDDO
        ENDDO
      ENDIF


C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     IF MAX_STEP reached or EXIT file exist

      IF(INFI.EQ.NOMORE .OR. I_META-1.EQ.NTOT_ITER) THEN
        EXSOFT = .TRUE.

        CALL  COLVARPR
      ENDIF
 9999 CONTINUE
      CALL MY_SYNC(ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(EXSOFT,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(I_META ,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(I_CVST ,MSGLEN,SOURCE,ALLGRP)

C     WRITE RESTART
      IF((MOD(I_META,ST_FREQ) .EQ. 0 .AND. I_CVST .EQ. 0)
     &                       .OR. EXSOFT .OR. LMETARES) THEN
        LMETARES = .TRUE.
        IF(I_META .GT. 1) THEN
           CALL WMTDRES(50,NTOT_ITER,I_META)
        ENDIF


      ENDIF

C     QUENCH BO
      IF(.NOT.TMDBO.AND.
     &    MOD(I_META,QW_FREQ) .EQ. 0 .AND. I_CVST .EQ. 0)
     &    LQUENCH = .TRUE.

      IF(FINAL .AND. PARENT) THEN
C     Deallocate
        CALL FREEM(IP_CV_STORE)
        CALL FREEM(IP_CV_LAST)
        CALL FREEM(IP_CV_PATH)
        CALL FREEM(IP_CSCL_VAL)
        CALL FREEM(IP_HLLW_VAL)
        CALL FREEM(IP_HLLH_VAL)
      ENDIF
C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     Halt time for the routine
      CALL TIHALT(' META_COLV',ISUB)

      RETURN
      END
C     ==================================================================

