#ifdef __SR11000
*option MP(P(0)), LANGLVL(SAVE(0))
#endif
C     ==================================================================
      SUBROUTINE META_COLVAR_INP(IUNIT)
C     ==--------------------------------------------------------------==
C     ==  Reads Collective Variables Input for Metadynamics           ==
C     ==--------------------------------------------------------------==
C
C     NCOLVAR   :  number of Collective Variables
C     TYCVAR    :  type of Collective Variables
C     ATCVAR    :  atoms which define the Collective Variables
C     CVPAR     :  other parameters that are required for the C.V.
C     I_META_MAX:  max # of steps in the MetaDynamics
C     I_META_RES: # of meta-steps performed in a previous meta-run
C                 used in case of RESTART from previous run 
C     INTER_HILL: # of ions dynamic steps between two meta steps
C     INTER_AV_CV: # of IONS configurations requied for the calculation 
C                 of the collective variables as average  
C     HLLH      : hills altitude 
C     HLLW      : hills amplitude 
C     CSCL_FAC  : scale factors required for an homogeneous dynamic in the
C                 NCOLVAR directions in the space of C.V.
C                 they can be read from input or determined from the
C                 second derivatives of the potential well
C                 a tuning of this factors along the dynamics is also
C                 to be considered
C     IBOUND    : 1 if a wall-potential constrains the C.V. 
C                                within a certain region of space 
C                 otherwise 0 (integer, dimension NCOLVAR)
C     VBOUND    : parameters for the wall-potential definition  see sub. SETWALL
C     LAGRANGIAN  The dynamics of the Collective Variable is driven by 
C                 an extended Lagrangean method
C                 With this option several parameter are to be defined
C           CV_TEMP0:  Initial Temperature of Collective Variables 
C                      Maxwell-Boltzman velocity distribution
C           LCVTC   :  Temperature Control flag
C                      parameters (Velocity Rescaling) CV_TEMP CV_DTEMP
C           CV_MASS :  Masses of collective Varibles
C           KHARM   :  Harmonic constants for the Harmonic potential Vharm 
C     ==--------------------------------------------------------------==
C     how C.V. definition is read
C
C     DEFINE VARIABLE
C     ==   nfix                                                       ==
C     ==  DIST      IA1  IA2             "SCA/SCF" sc min max  "KCV" k "MCV" m  ==
C     ==  STRETCH   IA1  IA2             "SCA/SCF" sc min max  "KCV" k "MCV" m  ==    
C     ==  BEND      IA1  IA2  IA3        "SCA/SCF" sc min max  "KCV" k "MCV" m  ==    
C     ==  TORSION   IA1  IA2  IA3  IA4   "SCA/SCF" sc min max  "KCV" k "MCV" m  ==    
C     ==  OUTP      IA1  IA2  IA3  IA4   "SCA/SCF" sc min max  "KCV" k "MCV" m  ==    
C     ==  COORD     IA1   K  RC          "SCA/SCF" sc min max  "KCV" k "MCV" m  ==    
C     ==  DIFFER    IA1  IA2  IA3        "SCA/SCF" sc min max  "KCV" k "MCV" m  ==    
C     ==  COORSP    IA1  ns1  K RC       "SCA/SCF" sc min max  "KCV" k "MCV" m  ==    
C     ==  COOR_RF   IA1  ns1  exp1  exp2  RC "SCA/SCF" sc min max "KCV" k "MCV" m  ==   
C     ==  COORGROUP NA   K               "SCA/SCF" sc min max  "KCV" k "MCV" m  ==
C     ==            in the following line:                                      ==
C     ==            IA1  NB1                                                    ==
C     ==                   IB1...IB_NB1  RC_IA1                                 ==
C     ==  COOR_RF  "INDAT" IA1  numatoms  exp1  exp2  RC                        ==
C                                        "SCA/SCF" sc min max  "KCV" k "MCV" m  == 
C                  in the following line write the list of atoms
C     ==  BNSWT     IA1  IA2  exp1  exp2  RC "SCA/SCF" sc min max "KCV" k "MCV" m ==
C     ==  TOT_COOR  ns1  ns2  exp1  exp2  RC "SCA/SCF" sc min max "KCV" k "MCV" m ==
C     ==  RMSD_AB   numsp ns1 . ns_numsp     "SCA/SCF" sc min max "KCV" k "MCV" m ==
C     ==  DISPL1    numspA numspB spA_1 ..spA_numspA spB_1 ..spB_numspB           ==
C                                            "SCA/SCF" sc min max "KCV" k "MCV" m ==
C     ==  DISPL1  "INDAT" numatA numatB spA_1  list atA   list atB                ==
C                                            "SCA/SCF" sc min max "KCV" k "MCV" m ==
C              if the lists are too long, write in line "LONGLIST" and
C              in following lines: indexes A (groups of 10 per line) (last line what remains)
C              in following lines: indexes B (groups of 10 per line)       ''
C     ==  PLNANG    IA1 IA2 IA3  IB1 IB2 IB3  "SCA/SCF" sc min max  "KCV" k "MCV" m  ==
C     ==  HBONDCH   NRES1  NATTR NRES2 NH  exp1_H exp2_H r0_h exp1_A esp2_A r0_A     ==
C                                              "SCA/SCF" sc min max "KCV" k  "MCV" m ==
C              in following line : indexes RES1
C              in following lines: indexes hbond attractors (groups of 10 per line)
C              in following line : indexes RES2
C              in following lines: indexes h attractors (groups of 10 per line)
C         HBONDPAR  "RCW" rcw "RCH" rch  "LENGTH"  nshort nlong "PAR" bbetha  const 'LUPD' lupd
C     ==  DIFCOOR   IA1 IA2 ns1 exp1  exp2  RC   "SCA/SCF" sc min max "KCV" k "MCV" m ==
C     ==  DIFCOOR "INDAT" IA1 IA2 numatoms  exp1  exp2  RC                            ==
C                                               "SCA/SCF" sc min max  "KCV" k "MCV" m ==
C     ==  COOR_CHAIN ISP1 ISP2 ISP3 n m rc1 rc2 "SCA/SCF" sc min max  "KCV" k "MCV" m ==
C     ==  HYDRONIUM ISP1 ISP2     "SCA/SCF" sc min max "KCV" k "MCV" m    "="         ==
C              in following line : "H:" n m rc "O:" n m rc "FCUT:" n m rc "LAMBDA" L
C     ==  HYD_CUT ISP1 ISP2 NUML  "SCA/SCF" sc min max "KCV" k "MCV" m    "="         ==
C  in following line : "H:" nH mH rcH "O:" nO mO rcO "M:" nM mM rcM1 rcM2 "FCUT:" nL mL rcL "LAMBDA" L
C              in following line : list of numL atomic indexes
C     ==  DIS_HYD ISP1 ISP2 IAT   "SCA/SCF" sc min max "KCV" k "MCV" m    "="         ==
C              in following line : "H:" n m rc "O:" n m rc "FCUT:" n m rc "LAMBDA" L
C     ==  SPIN  IAT                            "SCA/SCF" sc min max "KCV" k "MCV" m   ==
C     ==  VOLVAR                               "SCA/SCF" sc min max "KCV" k "MCV" m   ==
C     ==  CELLSIDE 1/2/3                       "SCA/SCF" sc min max "KCV" k "MCV" m   ==
C     ==  CELLANGLE 1/2/3                      "SCA/SCF" sc min max "KCV" k "MCV" m   ==
C     ==  ...  
C     END DEFINITION

      IMPLICIT  NONE

      INCLUDE  'system.h'
      INCLUDE  'cotr.inc'
      INCLUDE  'irat.inc'
      INCLUDE  'store.inc'
      INCLUDE  'ions.inc'   ! NAT
      INCLUDE  'prcp.inc'
      INCLUDE  'mw.inc'
      INCLUDE  'cnst_dyn.inc'
      INCLUDE  'cnst.inc'
      INCLUDE  'mm_dim.inc'
      INCLUDE  'mm_input.inc'

C     Arguments
      INTEGER    IUNIT
C     Variables
      INTEGER    IC,I,INUM,II,IJ,IA,IAA,IE,IEE,IND,K,ITYP,NNVAR
      REAL*8     VBAR,R_WALL,CPOS_0,TEMP,R0_SHIFT
      DIMENSION  IBOUND(*),VBOUND(4,*),CSCL_FAC(3,*),TAD_SCF(*),
     &           IATDLMN(*),IQATDLMN(*)
      CHARACTER  LINE*80,LINE2*80
      CHARACTER  STYP(36)*15,LINEFORM*100,CHNUM*10,CHNUM2*10
      DATA       STYP /
     &          '        STRETCH','           BEND','        TORSION',
     &          '       DISTANCE','           OUTP','        COORDIN',
     &          '      DIFFEREN.','         COORSP','        COOR_RF',
     &          '      BN.SWITCH','       TOT C_RF','        RMSD_AB',
     &          '         DISPL1','        COORDIS','         PLNANG',
     &          '        HBONDCH','        DIFCOOR','      COO_CHAIN',
     &          '      HYDRONIUM','        DIS_HYD','           SPIN',
     &          '         VOLVAR','       CELLSIDE','      CELLANGLE',
     &          '       COOR_SEQ','       RMSD_SEQ','        HYD_CUT',
     &          '      D_HYD_CUT','      COORGROUP','        DISAXIS',
     &          '      DIPOLERHO','      DIPOLETHA','      DIPOLEPHI',
     &          '         VCOORS','  DIFF TOT C_RF','|DIFF TOT C_RF|'/ 
      LOGICAL    ERREAD,STATUS
      INTEGER    I1,I2,IOUT,NUMSPEC,NUMSPEC2,NUMTOT,MY_NAT
      INTEGER    FIRSTDLMN,DLMN_NUM,NLOOP1,NLOOP2,IEND,IDUMMY, 
     c           SECONDTDLMN,
     c           IATMA,IATMB,FIRSTCORGRP,IPGRPA,IPGRPB,IPGRPA1,IPGRPB1,
     c           IPGRPB2,IPGRPB3,KK,IWALK1
      DATA       FIRSTDLMN,SECONDTDLMN,DLMN_NUM/3*0/
      SAVE       FIRSTDLMN,SECONDTDLMN,DLMN_NUM
      DATA       FIRSTCORGRP/0/
      SAVE       FIRSTCORGRP
      REAL*8     FMIN,FMAX
      REAL*8     dummy(*)
      POINTER    (ip_dummy,dummy)
C     ==--------------------------------------------------------------==
      LCOLVARDYN = .TRUE.

C     get total number of atoms and set indexing.
      if(qmmm) then
        call mm_dim(mm_go_mm,status)
        MY_NAT=NAT
      else
        MY_NAT=NATm
      endif
C     ==--------------------------------------------------------------==
   10 CONTINUE
      READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE

      IF(INDEX(LINE,'END').NE.0 .AND. INDEX(LINE,'METADYN').NE.0)
     &     GOTO 30
      IF(INDEX(LINE,'DEF').NE.0.AND. INDEX(LINE,'VARIABLE').NE.0)
     &    THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) NNVAR

C     Allocate Memory
        CALL MEMORY(IP_TYCVAR,NNVAR/IRAT+1,'TYCVAR')
        CALL IAZZERO(TYCVAR,NNVAR)
        CALL MEMORY(IP_ATCVAR,15*NNVAR/IRAT+1,'ATCVAR')
        CALL IAZZERO(ATCVAR,15*NNVAR)
        CALL MEMORY(IP_IANGCV,NNVAR/IRAT+1,'IANGCV')
        CALL IAZZERO(IANGCV,NNVAR)
        CALL MEMORY(IP_CVPAR,10*NNVAR,'CVPAR')
        CALL AZZERO(CVPAR,10*NNVAR)
        CALL MEMORY(IP_IBOUND,NNVAR/IRAT+1,'IBOUND')
        CALL IAZZERO(IBOUND,NNVAR)
        CALL MEMORY(IP_VBOUND,NNVAR*4,'VBOUND')
        CALL AZZERO(VBOUND,4*NNVAR)
        CALL MEMORY(IP_CSCL_FAC,3*NNVAR,'CSCL_FAC')
        CALL MEMORY(IP_TAD_SCF,NNVAR,'TAD_SCF')
        CALL MEMORY(IP_CV_MASS,NNVAR,'CV_MASS')
        CALL AZZERO(CV_MASS,NNVAR)
        CALL MEMORY(IP_KHARM,NNVAR,'KHARM')
        CALL AZZERO(KHARM,NNVAR)
        CALL MEMORY(IP_SPECINDEX,NNVAR/IRAT+1,'SPECINDEX')
        CALL IAZZERO(SPECINDEX,NNVAR)
        CALL MEMORY(IP_CV_DYN_0,NNVAR,'CV_DYN_0')
        CALL AZZERO(CV_DYN_0,NNVAR)
        CALL MEMORY(IP_INITIAL_VALUE,NNVAR/IRAT+1,'INITIAL_VALUE')
C$OMP parallel do private(I)
        DO I=1,NNVAR
          INITIAL_VALUE(I)=.FALSE.
        ENDDO

        IF(TMULTI) THEN
          CALL MEMORY(IP_HWM,NSUBSYS,'HWM')
          CALL MEMORY(IP_HHM,NSUBSYS,'HHM')
          CALL MEMORY(IP_HLHM,NSUBSYS,'HLHM')
          CALL MEMORY(IP_HTHM,NSUBSYS,'HTHM')
          CALL MEMORY(IP_HVM0,NSUBSYS,'HVM0')
          DO I = 1,NSUBSYS
            HWM(I) = HLLW
            HHM(I) = HLLH
            HLHM(I) = HLOW
            HTHM(I) = HTOP
          ENDDO
        ENDIF

C     Set Default Values
#ifdef __SR11000
*poption parallel, tlocal(IC)
*voption indep(CSCL_FAC,TAD_SCF)
#endif
        DO IC = 1,NNVAR
          CSCL_FAC(1,IC) = 1.0D0
          CSCL_FAC(2,IC) = 0.2D0
          CSCL_FAC(3,IC) = 2.D0
          TAD_SCF(IC)  = .FALSE.
        ENDDO
        GOTO 12
C     Read Type and Atoms for the Collective Variables to be set 
 11     READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
        IF(INDEX(LINE,'END').NE.0 .AND. INDEX(LINE,'DEF').NE.0)
     &     GOTO 10
 12     CONTINUE
        READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
C        write(6,*) LINE
        IF(INDEX(LINE,'END').NE.0 .AND. INDEX(LINE,'DEF').NE.0) THEN
C         Problem the number of coll. variables NCOLVAR .NE. given NFIX.
          WRITE(6,'(1X,A,I5)') 
     &            'M_COLVAR_INP! # OF GIVEN C.V. =',NNVAR
          WRITE(6,'(1X,A,I5)') 
     &            'M_COLVAR_INP! # OF FOUND C.V.=',NCOLVAR
          WRITE(6,'(1X,A)')    
     &            'M_COLVAR_INP! END C.V. IS REACHED.'
          CALL STOPGM('M_COLVAR_INP',
     &            'ERROR WHILE READING FIX STRUCTURES')
C       ==------------------- DISTANCE --------------------------------==
        ELSEIF(INDEX(LINE,'DIST').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 4
          I1=INDEX(LINE,'DIST')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
        ELSEIF(INDEX(LINE,'DISAXIS').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 30
          I1=INDEX(LINE,'DISAXIS')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(3,NCOLVAR)=IDUMMY
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==--------------- DIFFERENCE AMONG DISTANCES ------------------==
        ELSEIF(INDEX(LINE,'DIFFER').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 7
          I1=INDEX(LINE,'DIFFER')+6
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(3,NCOLVAR)=NAT_cpmd(IDUMMY)
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==------------------ STRETCH ----------------------------------==
        ELSEIF(INDEX(LINE,'STRETCH').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 1
          I1=INDEX(LINE,'STRETCH')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==------------------  BEND ------------------------------------==
        ELSEIF(INDEX(LINE,'BEND').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 2 
          I1=INDEX(LINE,'BEND')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(3,NCOLVAR)=NAT_cpmd(IDUMMY)
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==------------------ TORSION ANGLE  ----------------------------==
        ELSEIF(INDEX(LINE,'TORSION').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 3
          IANGCV(NCOLVAR) = 1
          I1=INDEX(LINE,'TORSION')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(3,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(4,NCOLVAR)=NAT_cpmd(IDUMMY)
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==------------------ OUT OF LPANE ANGLE ------------------------==
        ELSEIF(INDEX(LINE,'OUTP').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 5
          IANGCV(NCOLVAR) = 1
          I1=INDEX(LINE,'OUTP')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(3,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(4,NCOLVAR)=NAT_cpmd(IDUMMY)
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==------------------ COORDINATION NUMBER ------------------------==
        ELSEIF(INDEX(LINE,'COORD').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 6
          I1=INDEX(LINE,'COORD')+5
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==------------ SPECIES DEPENDENT COORDINATION NUMBER  -----------==
C       ==                 f = 1/(1+exp(k*(R-R_0)))                      ==
        ELSEIF(INDEX(LINE,'COORSP').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 8
          I1=INDEX(LINE,'COORSP')+6
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
Cmb For QMMM use the qmmm CV
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==--- SPECIES DEPENDENT COORDINATION NUMBER OF POINT IN SPACE ---=
C       ==                 f = 1/(1+exp(k*(R-R_0)))                      =
        ELSEIF(INDEX(LINE,'VCOORS').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 34
          I1=INDEX(LINE,'VCOORS')+6
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
          READ(IUNIT,*) CVPAR(3,NCOLVAR),CVPAR(4,NCOLVAR),
     &                   CVPAR(5,NCOLVAR)
     

          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)  

C       ==-------------- COORDINATION NUMBER FOR SPECIES IN GROUP-----------------==
C       ==                 f = sum_i^NA sum_j^NB(i) 1/(1+exp(k*(R_ij-R_0(i)) ) )  ==

        ELSEIF(INDEX(LINE,'COORGROUP').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 29
          I1=INDEX(LINE,'COORGROUP')+9
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD) !number of a atoms
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD) !parameter k
          I1=IOUT
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     c                    CV_DYN_0,INITIAL_VALUE,
     c                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
          IF(FIRSTCORGRP.EQ.0) THEN
            CALL MEMORY(IP_IATCNGA,
     c                 ATCVAR(1,NCOLVAR)*NNVAR/IRAT+1,'IATCNGA')!stores index of atom A
            CALL MEMORY(IP_IATCNGB,
     c       ATCVAR(1,NCOLVAR)*MAX_NATCNGB*NNVAR/IRAT+1,'IATCNGA')!stores index of atom B
            CALL MEMORY(IP_NATCNGB,
     c       ATCVAR(1,NCOLVAR)*NNVAR/IRAT+1,'NATCNGB')!stores number of atoms of B for each A
            CALL MEMORY(IP_RCCNGA,
     c       ATCVAR(1,NCOLVAR)*NNVAR,'RCCNGA')         !stores the cut-off distances for all A
            CALL IAZZERO(IATCNGA,ATCVAR(1,NCOLVAR)*NNVAR)
            CALL IAZZERO(IATCNGB,ATCVAR(1,NCOLVAR)*MAX_NATCNGB*NNVAR)
            CALL AZZERO (RCCNGA,ATCVAR(1,NCOLVAR)*NNVAR)
            FIRSTCORGRP = 1
            IPGRPA=0
            IPGRPB=0
            IPGRPB2=0
          ENDIF
          DO IATMA=1,ATCVAR(1,NCOLVAR)
            IPGRPA=IPGRPA+1
            READ(IUNIT,*) IATCNGA(IPGRPA), RCCNGA(IPGRPA),
     c                    NATCNGB(IPGRPA)
            IATCNGA(IPGRPA)=NAT_cpmd(IATCNGA(IPGRPA)) !A's in CPMD order
            IF(NATCNGB(IPGRPA).GT.MAX_NATCNGB)THEN
                WRITE(6,'(/,A,I8)')
     C          'Maximum number of type B atoms for COORGROUP =',
     c             MAX_NATCNGB
                WRITE(6,'(A,I8,A,I8)')
     c          'For coll. variable ',NCOLVAR,' number of B atoms =',
     c           NATCNGB(IPGRPA)
                CALL STOPGM('M_COLVAR_INP',
     c          'MAX_NATCNGB < NATCNGB(IPGRPA)')
            ENDIF
            READ(IUNIT,*)
     c           (IATCNGB(IPGRPB+IATMB),IATMB=1,NATCNGB(IPGRPA)) !read in atoms B for a give A atom
            DO IATMB=1,NATCNGB(IPGRPA)
              IATCNGB(IPGRPB+IATMB)=NAT_cpmd(IATCNGB(IPGRPB+IATMB)) !B's in CPMD order
            END DO
            IPGRPB=IPGRPB+NATCNGB(IPGRPA)
          ENDDO
          IPGRPB2=IPGRPB2+MAX_NATCNGB*ATCVAR(1,NCOLVAR)
          IPGRPB=IPGRPB2
C       ==------------ SPECIES DEPENDENT COORDINATION NUMBER -------------==
C       ==            f =sum_i (1-(R/R_0)^n)/(1-(R/R_0)^(n+m))
        ELSEIF(INDEX(LINE,'COOR_RF').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 9
          I1=INDEX(LINE,'COOR_RF')+7
          IF(INDEX(LINE,'INDAT') .NE. 0) THEN 
            SPECINDEX(NCOLVAR) = 0
            I1 = INDEX(LINE,'INDAT') +5
          ELSEIF(INDEX(LINE,'ELEMENT') .NE. 0 .AND. 
     &           INDEX(LINE,'SOLUTE').NE.0) THEN 
            SPECINDEX(NCOLVAR) = -1
            I1 = INDEX(LINE,'SOLUTE') +6
          ELSEIF(INDEX(LINE,'ELEMENT') .NE. 0 .AND. 
     &           INDEX(LINE,'SOLVENT').NE.0) THEN 
            SPECINDEX(NCOLVAR) = -2
            I1 = INDEX(LINE,'SOLVENT') +7
          ELSEIF(INDEX(LINE,'ELEMENT') .NE. 0 ) THEN
            SPECINDEX(NCOLVAR) = -3
            I1 = INDEX(LINE,'ELEMENT') +7
          ELSEIF(INDEX(LINE,'SEQUENCE') .NE. 0 .AND. 
     &           INDEX(LINE,'MM').NE.0) THEN 
            SPECINDEX(NCOLVAR) = -4
            I1 = INDEX(LINE,'MM') +2
          ELSEIF(INDEX(LINE,'SEQUENCE') .NE. 0) THEN 
            SPECINDEX(NCOLVAR) = -5
            I1 = INDEX(LINE,'SEQUENCE') +8
          ELSEIF(INDEX(LINE,'SEQ_ELE') .NE. 0 .AND. 
     &           INDEX(LINE,'MM').NE.0) THEN 
            SPECINDEX(NCOLVAR) = -6
            I1 = INDEX(LINE,'MM') +2
          ELSEIF(INDEX(LINE,'SEQ_ELE') .NE. 0) THEN 
            SPECINDEX(NCOLVAR) = -7
            I1 = INDEX(LINE,'SEQ_ELE') + 7
          ELSE
            SPECINDEX(NCOLVAR) = 1
          ENDIF

          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          IF(SPECINDEX(NCOLVAR) .EQ. 0) THEN
            IF(FIRSTDLMN .EQ. 0) THEN
              CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
              CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
              FIRSTDLMN = 1
            ENDIF
            IF(qmmm)  CALL STOPGM('M_COLVAR_INP',
     &                 'INDAT not implemented for QMMM')
            READ(IUNIT,*)  (IATDLMN(II+MY_NAT*(NCOLVAR-1)),
     &                           II=1,ATCVAR(2,NCOLVAR))

          ELSEIF(SPECINDEX(NCOLVAR) .LT. -3) THEN
             READ(IUNIT, *) ATCVAR(5,NCOLVAR), ATCVAR(6,NCOLVAR)
          ENDIF

          II=INDEX(LINE,'2SHELL')
          IF(II.NE.0) THEN
            I1=II+6   ! missing line   
            CALL READSR(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
            IA=ATCVAR(3,NCOLVAR)
            IE=ATCVAR(4,NCOLVAR)
            IF(MOD(IA,2).NE.0 .OR. MOD(IE,2).NE.0 ) THEN
              CALL STOPGM('M_COLVAR_INP',
     &        'for COOR_RF with 2nd shell use even exponents')
            ENDIF
          ENDIF
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)

C       ==--------------- BOND SWITCH WITH RATIONAL F -------------------==
C       ==            f = (1+(R/R_0)^n)/(1+(R/R_0)^(n+m))
        ELSEIF(INDEX(LINE,'BNSWT').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 10
          I1=INDEX(LINE,'BNSWT')+5
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==--------- TOTAL SPECIES DEPENDENT COORDINATION NUMBER ---------==
C       ==      f =sum_j[sum_i (1+(Rij/R_0)^n)/(1+(Rij/R_0)^(n+m))]/NA   ==
        ELSEIF(INDEX(LINE,'TOT_COOR').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 11
          I1=INDEX(LINE,'TOT_COOR') + 8
          IF(INDEX(LINE,'INDAT') .NE. 0) THEN 
            SPECINDEX(NCOLVAR) = 0
            I1 = INDEX(LINE,'INDAT') +5
          ELSEIF(INDEX(LINE,'ELEMENT') .NE. 0 .AND. 
     &           INDEX(LINE,'SOLUTE').NE.0) THEN 
            SPECINDEX(NCOLVAR) = -1
            I1 = INDEX(LINE,'SOLUTE') +6
          ELSEIF(INDEX(LINE,'ELEMENT') .NE. 0 .AND. 
     &           INDEX(LINE,'SOLVENT').NE.0) THEN 
            SPECINDEX(NCOLVAR) = -2
            I1 = INDEX(LINE,'SOLVENT') +7
          ELSEIF(INDEX(LINE,'ELEMENT') .NE. 0 ) THEN
            SPECINDEX(NCOLVAR) = -3
            I1 = INDEX(LINE,'ELEMENT') +7
          ELSEIF(INDEX(LINE,'SEQUENCE') .NE. 0 .AND. 
     &           INDEX(LINE,'MM').NE.0) THEN 
            SPECINDEX(NCOLVAR) = -4
            I1 = INDEX(LINE,'MM') +2
          ELSEIF(INDEX(LINE,'SEQUENCE') .NE. 0) THEN 
            SPECINDEX(NCOLVAR) = -5
            I1 = INDEX(LINE,'SEQUENCE') +8
          ELSEIF(INDEX(LINE,'SEQ_ELE') .NE. 0 .AND. 
     &           INDEX(LINE,'MM').NE.0) THEN 
            SPECINDEX(NCOLVAR) = -6
            I1 = INDEX(LINE,'MM') +2
          ELSEIF(INDEX(LINE,'SEQ_ELE') .NE. 0) THEN 
            SPECINDEX(NCOLVAR) = -7
            I1 = INDEX(LINE,'SEQ_ELE') + 7
  
          ELSE
            SPECINDEX(NCOLVAR) = 1
          ENDIF
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          IF(SPECINDEX(NCOLVAR) .EQ. 0) THEN
            IF(FIRSTDLMN .EQ. 0) THEN
              CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
              CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
              FIRSTDLMN = 1
            ENDIF

            IF(qmmm)  CALL STOPGM('M_COLVAR_INP',
     &                 'INDAT not implemented for QMMM')

            READ(IUNIT,*)  (IATDLMN(II+MY_NAT*(NCOLVAR-1)),
     &                           II=1,ATCVAR(2,NCOLVAR))

          ELSEIF(SPECINDEX(NCOLVAR) .LT. -3) THEN
             READ(IUNIT, *) ATCVAR(5,NCOLVAR), ATCVAR(6,NCOLVAR)
          ENDIF

          II=INDEX(LINE,'2SHELL')
          IF(II.NE.0) THEN
            I1=II+6   ! missing line   
            CALL READSR(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
            IA=ATCVAR(3,NCOLVAR)
            IE=ATCVAR(4,NCOLVAR)
            IF(MOD(IA,2).NE.0 .OR. MOD(IE,2).NE.0 ) THEN
              CALL STOPGM('M_COLVAR_INP',
     &        'for TOT_COOR with 2nd shell use even exponents')
            ENDIF
          ENDIF
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==--------- TOTAL SPECIES DEPENDENT COORDINATION NUMBER OF DIFFERENCE ---------==
C       ==   f =sum_j[sum_i (1+(Rij/R_0)^n)/(1+(Rij/R_0)^(n+m))]/NA-sum_j[sum_k[]]/NA ==
        ELSEIF(INDEX(LINE,'TOT_DIFCOOR').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          ! Tag the absolute value of the difference 
          IF (INDEX(LINE,'|TOT_DIFCOOR|').NE.0) THEN
             TYCVAR(NCOLVAR) = 36
             I1=INDEX(LINE,'|TOT_DIFCOOR|') + 13
          ELSE
             TYCVAR(NCOLVAR) = 35
             I1=INDEX(LINE,'TOT_DIFCOOR') + 11
          ENDIF
          SPECINDEX(NCOLVAR) = 1
          !three species
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          !n, m for rational function.
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(5,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)

          !scaling factor, k, m, low and upper bounds, turning on lagrange formulations     
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==--------- COORDINATION NUMBER BETWEEN SEQUENCES (QMMM) ---------==
C       ==      f =sum_j[sum_i (1+(Rij/R_0)^n)/(1+(Rij/R_0)^(n+m))]/NA   ==
        ELSEIF(INDEX(LINE,'COOR_SEQ').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 25
          I1=INDEX(LINE,'COOR_SEQ') + 8
          
          IF(INDEX(LINE,'SOLUTE_1') .NE. 0 .AND. 
     &           INDEX(LINE,'SOLVENT').NE.0) THEN 
            SPECINDEX(NCOLVAR) = 1
            I1 = INDEX(LINE,'SOLVENT') +7
          ELSEIF(INDEX(LINE,'SOLUTE_1') .NE. 0 .AND. 
     &           INDEX(LINE,'SOLUTE_2').NE.0) THEN 
            SPECINDEX(NCOLVAR) = 2
            I1 = INDEX(LINE,'SOLUTE_2') +8
          ELSEIF(INDEX(LINE,'SEQ_1') .NE. 0 .AND.
     &           INDEX(LINE,'SOLVENT') .NE. 0) THEN
            SPECINDEX(NCOLVAR) = 3
            I1 = INDEX(LINE,'SOLVENT') +7
          ELSEIF(INDEX(LINE,'SEQ_1') .NE. 0 .AND.
     &           INDEX(LINE,'SEQ_2') .NE. 0 ) THEN
            SPECINDEX(NCOLVAR) = 4
            I1 = INDEX(LINE,'SEQ_2') +5
          ELSEIF(INDEX(LINE,'SEQ_1') .NE. 0 .AND.
     &           INDEX(LINE,'SEQ_MM2') .NE. 0) THEN
            SPECINDEX(NCOLVAR) = 5
            I1 = INDEX(LINE,'SEQ_MM2') +7
          ELSEIF(INDEX(LINE,'SEQ_QM1') .NE. 0 .AND.
     &           INDEX(LINE,'SEQ_MM2') .NE. 0) THEN
            SPECINDEX(NCOLVAR) = 6
            I1 = INDEX(LINE,'SEQ_MM2') +7
          ELSEIF(INDEX(LINE,'SEQ_MM1') .NE. 0 .AND.
     &           INDEX(LINE,'SEQ_MM2') .NE. 0 ) THEN
            SPECINDEX(NCOLVAR) = 7
            I1 = INDEX(LINE,'SEQ_MM2') +7
          ELSEIF(INDEX(LINE,'SEQ_QM1') .NE. 0 .AND.
     &           INDEX(LINE,'SEQ_2') .NE. 0 ) THEN
            SPECINDEX(NCOLVAR) = 8
            I1 = INDEX(LINE,'SEQ_2') +5
          END IF

          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)

          IF(SPECINDEX(NCOLVAR) .EQ. 3) THEN
             READ(IUNIT, *) ATCVAR(5,NCOLVAR), ATCVAR(6,NCOLVAR)
          ELSEIF(SPECINDEX(NCOLVAR) .GT. 3) THEN
             READ(IUNIT, *) ATCVAR(5,NCOLVAR), ATCVAR(6,NCOLVAR),
     &            ATCVAR(7,NCOLVAR), ATCVAR(8,NCOLVAR)
          ENDIF

          II=INDEX(LINE,'2SHELL')
          IF(II.NE.0) THEN
            I1=II+6   ! missing line   
            CALL READSR(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
            IA=ATCVAR(3,NCOLVAR)
            IE=ATCVAR(4,NCOLVAR)
            IF(MOD(IA,2).NE.0 .OR. MOD(IE,2).NE.0 ) THEN
              CALL STOPGM('M_COLVAR_INP',
     &        'for COOR_SEQ with 2nd shell use even exponents')
            ENDIF
          ENDIF
          
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
       
C       ==------- RMSD wrt 2 configurations A and B (read from file)-----==
C       ==                f = (RMSD_A-RMSD_B)/(RMSD_A+RMSD_B)            ==
        ELSEIF(INDEX(LINE,'RMSD_AB').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 12
          IF(.NOT. TRMSD_AB) THEN
            IF(MAX_NNVAR.LT.NNVAR) THEN
              IF(PARENT) WRITE(6,*) "NNVAR:",NNVAR,
     &                             "  MAX_NNVAR:",MAX_NNVAR
              CALL STOPGM("FILE_STR_AB",
     &             "PARAMETER MAX_NNVAR too small")
            ENDIF

            CALL MEMORY(IP_ICV_RMSD_AB,NNVAR/IRAT+1,'ICV_RMSD_AB')
            CALL IAZZERO(ICV_RMSD_AB,NNVAR)
            TRMSD_AB=.TRUE.
            NRMSD_AB = 0
          ENDIF
          ICV_RMSD_AB(NCOLVAR)=NCOLVAR
          NRMSD_AB  = NRMSD_AB + 1

          I1=INDEX(LINE,'RMSD_AB') + 7
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          IF(ATCVAR(1,NCOLVAR) .GT. 13) CALL STOPGM('M_COLVAR_INP',
     &        'RMSD_AB! too many species (>13)')
          NUMSPEC = ATCVAR(1,NCOLVAR)
          DO I = 1,NUMSPEC
            CALL READSI(LINE,I1,IOUT,ATCVAR(I+1,NCOLVAR),ERREAD)
            I1=IOUT
          ENDDO
          II=INDEX(LINE,'FILEAB')
          IF(II.NE.0) THEN
            IA = II+6
            LINE2  = LINE(IA:IA+20)
            CALL XSTRING(LINE2,IA,IE)
            FILE_STR_AB(NCOLVAR) = LINE2(IA:IE)
          ELSE
            FILE_STR_AB(NCOLVAR) = 'STRUCTURE_AB'
          ENDIF
          ATCVAR(NUMSPEC+2,NCOLVAR) = NRMSD_AB

          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)

        ELSEIF(INDEX(LINE,'RMSD_SEQ').NE. 0) THEN
! RMSD for QMMM, only absolute value wrt initial state for the moment
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 26
          I1=INDEX(LINE,'RMSD_SEQ') + 8

          IF(INDEX(LINE,'SOLUTE') .NE. 0 ) THEN
            SPECINDEX(NCOLVAR) = 1
            I1 = INDEX(LINE,'SOLUTE') + 6
          ELSEIF(INDEX(LINE,'SOLVENT') .NE. 0) THEN
            SPECINDEX(NCOLVAR) = 2
            I1 = INDEX(LINE,'SOLVENT') + 7
          ELSEIF(INDEX(LINE,'SEQUENCE') .NE. 0) THEN
            SPECINDEX(NCOLVAR) = 3
            I1 = INDEX(LINE,'SEQUENCE') + 8
          ENDIF

          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          ! Determine max number of atoms to allocate the least space
          IF (ATCVAR(2,NCOLVAR) .GT. MAXRMSDAT) THEN
             MAXRMSDAT =    ATCVAR(2,NCOLVAR)
          END IF  

          IF(ATCVAR(1,NCOLVAR) .GT. 10) CALL STOPGM('M_COLVAR_INP',
     &        'RMSD_SEQ! too many species (>10)')
          NUMSPEC = ATCVAR(1,NCOLVAR)
          DO I = 1,NUMSPEC
            CALL READSI(LINE,I1,IOUT,ATCVAR(I+4,NCOLVAR),ERREAD)
            I1=IOUT
          ENDDO


          IF(SPECINDEX(NCOLVAR) .EQ. 3) THEN
             READ(IUNIT, *) ATCVAR(3,NCOLVAR), ATCVAR(4,NCOLVAR)
          ENDIF
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     c                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)

!
C       ==--- (lmn) Displacement of one species wrt configuration A -----==
C     FIXME: AK 2005/05/27 convert to use Gromos ordering of atom indices
        ELSEIF(INDEX(LINE,'DISPL1').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 13
          I1=INDEX(LINE,'DISPL1') + 6
          IF(INDEX(LINE,'INDAT') .NE. 0) THEN 
            SPECINDEX(NCOLVAR) = 0
            I1 = INDEX(LINE,'INDAT') + 5
          ELSE
            SPECINDEX(NCOLVAR) = 1
          ENDIF
          IF(SPECINDEX(NCOLVAR) .EQ. 1) THEN
            CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
            I1=IOUT
            NUMSPEC = ATCVAR(1,NCOLVAR)
            CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
            I1=IOUT
            NUMSPEC2 = ATCVAR(2,NCOLVAR)
            IF(NUMSPEC+NUMSPEC2 .GT. 10) CALL STOPGM('M_COLVAR_INP',
     &        'DISPL! too many species (>10)')
            DO I = 1,NUMSPEC
              CALL READSI(LINE,I1,IOUT,ATCVAR(I+2,NCOLVAR),ERREAD)
             I1=IOUT
            ENDDO
            IF(NUMSPEC2 .NE. 0) THEN
              I2=NUMSPEC+2
              DO I = 1,NUMSPEC2
                CALL READSI(LINE,I1,IOUT,ATCVAR(I+I2,NCOLVAR),ERREAD)
                I1=IOUT
              ENDDO
            ENDIF
            I2=NUMSPEC+NUMSPEC2+2
          ELSE
            CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
            I1=IOUT
            IF(ATCVAR(1,NCOLVAR) .GT. MY_NAT/2) 
     &        CALL STOPGM('M_COLVAR_INP',
     &        'DISPL! too many atoms  (>NAT/2)')
            NUMSPEC = ATCVAR(1,NCOLVAR)
            CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
            I1=IOUT
            NUMSPEC2 = ATCVAR(2,NCOLVAR)
            IF(NUMSPEC+NUMSPEC2 .GT. MY_NAT) THEN
              CALL STOPGM('M_COLVAR_INP','DISPL1: NAT1+NAT2 > NAT')
            ENDIF

            IF(FIRSTDLMN .EQ. 0) THEN
              CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
              CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
              FIRSTDLMN = 1
            ENDIF
            IF(INDEX(LINE,'LONGLIST') .NE. 0) THEN 
              I1 = INDEX(LINE,'LONGLIST') + 8
              NLOOP1 = NUMSPEC/10+1
              IF(MOD(NUMSPEC,10) .EQ. 0.0D0) NLOOP1 = NUMSPEC/10

              NLOOP2 = NUMSPEC2/10+1
              IF(MOD(NUMSPEC2,10) .EQ. 0.0D0) NLOOP2 = NUMSPEC2/10

              DO I = 1,NLOOP1
                READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE2
                IEND = 10
                IF(I .EQ. NLOOP1 .AND. 
     &              NUMSPEC .GT. INT(NUMSPEC/10)*10) THEN
                   IEND = NUMSPEC - INT(NUMSPEC/10)*10
                ENDIF

                I1=1
                DO II = 1,IEND
                  CALL READSI(LINE2,I1,IOUT,
     &               IATDLMN(II+(I-1)*10+MY_NAT*(NCOLVAR-1)),ERREAD)
                  I1=IOUT
                ENDDO
              ENDDO

              IF(NUMSPEC2 .NE. 0) THEN
                DO I = 1,NLOOP2
                  READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE2
                  IEND = 10
                  IF(I .EQ. NLOOP2 .AND. 
     &                NUMSPEC2 .GT. INT(NUMSPEC2/10)*10) THEN
                     IEND = NUMSPEC2 - INT(NUMSPEC2/10)*10
                  ENDIF

                  I1 = 1
                  DO II = 1,IEND
                    CALL READSI(LINE2,I1,IOUT,
     &                   IATDLMN(II+(I-1)*10+NUMSPEC+MY_NAT*(NCOLVAR-1))
     &                   ,ERREAD)
                    I1=IOUT
                  ENDDO
                ENDDO

              ENDIF

            ELSE
              DO I = 1,NUMSPEC
                CALL READSI(LINE,I1,IOUT,
     &               IATDLMN(I+MY_NAT*(NCOLVAR-1)),ERREAD)
               I1=IOUT
              ENDDO
              IF(NUMSPEC2 .NE. 0) THEN
                DO I = 1,NUMSPEC2
                  CALL READSI(LINE,I1,IOUT,
     &                 IATDLMN(I+NUMSPEC+MY_NAT*(NCOLVAR-1)),ERREAD)
                  I1=IOUT
                ENDDO
              ENDIF
              I2=2
            ENDIF
          ENDIF
          II=INDEX(LINE,'MIL')
          IF(II.NE.0) THEN
            I1 = II+3
              DO I = 1,3
                CALL READSI(LINE,I1,IOUT,ATCVAR(I+I2,NCOLVAR),ERREAD)
                I1=IOUT
              ENDDO
          ELSE
            WRITE(6,'(A,A)') 'WARNING: Miller indeces not found,',
     &                   '  default used 100'  
            ATCVAR(1+I2,NCOLVAR)=1
            ATCVAR(2+I2,NCOLVAR)=0
            ATCVAR(3+I2,NCOLVAR)=0
          ENDIF
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==---- Angle between 2 planes, each defined by 3 given pnts  -----==
        ELSEIF(INDEX(LINE,'PLNANG').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 15
          IANGCV(NCOLVAR) = 2
          I1=INDEX(LINE,'PLNANG')+6
          IANGCV(NCOLVAR) = 1
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(3,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(4,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(5,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(6,NCOLVAR)=NAT_cpmd(IDUMMY)
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==--------------------- Hydrogen Bonds Chain  --------------------==
        ELSEIF(INDEX(LINE,'HBONDCH').NE.0) THEN
C     FIXME: AK 2005/05/27 convert to use Gromos ordering of atom indices
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 16
          I1=INDEX(LINE,'HBONDCH')+7
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD) !NRES1
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD) !NATTR
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD) !NRES2
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD) !NH
          I1=IOUT

          CALL READSI(LINE,I1,IOUT,ATCVAR(5,NCOLVAR),ERREAD) !EXP1_H
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(6,NCOLVAR),ERREAD) !EXP2_H
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD) ! R0_H
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(7,NCOLVAR),ERREAD) !EXP1_ATTR 
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(8,NCOLVAR),ERREAD) !EXP2_ATTR 
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD) !R0_ATTR

          IF(ERREAD) CALL STOPGM('M_COLVAR_INP',
     &         'ERROR in reading HBONDCH line')

          IF(FIRSTDLMN .EQ. 0) THEN
            CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
            CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
            FIRSTDLMN = 1
          ENDIF
          IEND = 0
          READ(IUNIT,*)  (IATDLMN(II+IEND+MY_NAT*(NCOLVAR-1)),
     &                           II=1,ATCVAR(1,NCOLVAR))
c          write(6,*) (IATDLMN(II+IEND+MY_NAT*(NCOLVAR-1)),
c     &                           II=1,ATCVAR(1,NCOLVAR))
          IEND = IEND + ATCVAR(1,NCOLVAR)
          READ(IUNIT,*)  (IATDLMN(II+IEND+MY_NAT*(NCOLVAR-1)),
     &                           II=1,ATCVAR(2,NCOLVAR))
c          write(6,*) (IATDLMN(II+IEND+MY_NAT*(NCOLVAR-1)),
c     &                           II=1,ATCVAR(2,NCOLVAR))
          IEND = IEND + ATCVAR(2,NCOLVAR)

          READ(IUNIT,*)  (IATDLMN(II+IEND+MY_NAT*(NCOLVAR-1)),
     &                           II=1,ATCVAR(3,NCOLVAR))
c          write(6,*) (IATDLMN(II+IEND+MY_NAT*(NCOLVAR-1)),
c     &                           II=1,ATCVAR(3,NCOLVAR))

          IEND = IEND + ATCVAR(3,NCOLVAR)
          READ(IUNIT,*)  (IATDLMN(II+IEND+MY_NAT*(NCOLVAR-1)),
     &                           II=1,ATCVAR(4,NCOLVAR))

          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     c                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
 
         IF(INDEX(LINE,'PARA').NE.0) THEN
            READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE2
            II=INDEX(LINE2,'RCW')
            IF(II.NE.0) THEN
            IA = II+3
            CALL READSR(LINE2,IA,IE,RCW,   ERREAD)
            ENDIF
            II=INDEX(LINE2,'RCH')
            IF(II.NE.0) THEN
            IA = II+3
            CALL READSR(LINE2,IA,IE,RCH,   ERREAD)
            ENDIF
            II=INDEX(LINE2,'NSHORT')
            IF(II.NE.0) THEN
            IA = II+6
            CALL READSI(LINE2,IA,IE,NSHORT,   ERREAD)
            ENDIF
            II=INDEX(LINE2,'NLONG')
            IF(II.NE.0) THEN
            IA = II+5
            CALL READSI(LINE2,IA,IE,NLONG,   ERREAD)
            ENDIF
            II=INDEX(LINE2,'BBETA')
            IF(II.NE.0) THEN
            IA = II+6
            CALL READSR(LINE2,IA,IE,BBETA,   ERREAD)
            ENDIF
            II=INDEX(LINE2,'OPTDIST')
            IF(II.NE.0) THEN
            IA = II+7
            CALL READSR(LINE2,IA,IE,OPTDIST,   ERREAD)
            ENDIF
            II=INDEX(LINE2,'LUPD')
            IF(II.NE.0) THEN
            IA = II+4
            CALL READSI(LINE2,IA,IE,LUPD,   ERREAD)
            ENDIF
            IF(ERREAD) CALL STOPGM('M_COLVAR_INP',
     &         'ERROR in reading H-CHAIN paraneters')
          ENDIF
C       ==------------ DIFFERENCE OF COORDINATION NUMBERS -------------==
C       ==  f =sum_i (1-(Rbi/R_0)^n)/(1-(Rbi/R_0)^(n+m))-
C       ==     sum_i (1-(Rai/R_0)^n)/(1-(Rai/R_0)^(n+m))
        ELSEIF(INDEX(LINE,'DIFCOOR').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 17
          I1=INDEX(LINE,'DIFCOOR')+7
          IF(INDEX(LINE,'INDAT') .NE. 0) THEN 
            SPECINDEX(NCOLVAR) = 0
            I1 = INDEX(LINE,'INDAT') + 5
          ELSE
            SPECINDEX(NCOLVAR) = 1
          ENDIF

          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(5,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)

          IF(SPECINDEX(NCOLVAR) .EQ. 0) THEN
            IF(FIRSTDLMN .EQ. 0) THEN
              CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
              CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
              FIRSTDLMN = 1
            ENDIF

            READ(IUNIT,*)  (IATDLMN(II+MY_NAT*(NCOLVAR-1)),
     &                           II=1,ATCVAR(3,NCOLVAR))
          ENDIF
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==------------ COORDINATION OF SECOND NEIGHBORS -------------==
C       ==    f =sum_jik [(1+(Rij/R_0)^n)/(1+(Rij/R_0)^(n+m))*       ==
C       ==                1+(Rjk/R_0)^n)/(1+(Rjk/R_0)^(n+m))]/NA/NB  == 

        ELSEIF(INDEX(LINE,'COOR_CHAIN').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 18
          I1=INDEX(LINE,'COOR_CHAIN')+10 
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(5,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)

          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)

!       ==------------ PRESENCE OF THE HYDRONIUM COMPLEX -----------==
!       ==     F =(1/lambda)*log(sum_k exp{lambda*f(n_H)*n_O})      ==
!       ==     f(n_h) = 1-(1-(n_h/rc_3)**n_3)/(1-(n_h/rc_3)**m_3)   ==
!       ==     n_h =  sum_j(1-(r_kj/rc_1)**n_1)/(1-(r_kj/rc_1)**m_1)==
!       ==     n_o =  sum_i(1-(r_ki/rc_2)**n_2)/(1-(r_ki/rc_2)**m_2)==

        ELSEIF(INDEX(LINE,'HYDRONIUM').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 19
          I1=INDEX(LINE,'HYDRONIUM')+9 
C          Defaults
          ATCVAR(3,NCOLVAR) = 6
          ATCVAR(4,NCOLVAR) = 8
          CVPAR(1,NCOLVAR)  = 2.4D0
          ATCVAR(5,NCOLVAR) = 6
          ATCVAR(6,NCOLVAR) = 6
          CVPAR(2,NCOLVAR)  = 5.5D0
          ATCVAR(7,NCOLVAR) = 10
          ATCVAR(8,NCOLVAR) = 15
          CVPAR(3,NCOLVAR)  = 2.40D0
          CVPAR(4,NCOLVAR)  = 10.0D0

          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          
          II=INDEX(LINE,'=')
          IF(II.NE.0) THEN
            READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE2
            II=INDEX(LINE2,'H:')
            IF(II.NE.0) THEN
              I1 = II+2
              CALL READSI(LINE2,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
            ENDIF
            II=INDEX(LINE2,'O:')
            IF(II.NE.0) THEN
              I1 = II+2
              CALL READSI(LINE2,I1,IOUT,ATCVAR(5,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(6,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
              I1=IOUT
            ENDIF
            II=INDEX(LINE2,'FCUT:')
            IF(II.NE.0) THEN
              I1 = II+5
              CALL READSI(LINE2,I1,IOUT,ATCVAR(7,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(8,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(3,NCOLVAR),ERREAD)
            ENDIF
            II=INDEX(LINE2,'LAMBDA:')
            IF(II.NE.0) THEN
              I1 = II+7
              CALL READSR(LINE2,I1,IOUT,CVPAR(4,NCOLVAR),ERREAD)
            ENDIF
          ENDIF

          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)

!       ==------------ PRESENCE OF THE HYDRONIUM COMPLEX B -----------==
!       ==     F =fc(nL)(1/lambda)*log(sum_k exp{lambda*M(n_H)*M(n_O)})      ==
!       ==     M(n_h) = 1-(1-(n_h/rc_3)**n_M)/(1-(n_h/rc_3)**m_M)     ==
!       ==     M(n_o) = 1-(1-(n_o/rc_4)**n_M)/(1-(n_h/rc_4)**m_M)     ==
!       ==     n_h =  sum_j(1-(r_kj/rc_1)**n_1)/(1-(r_kj/rc_1)**m_1)  ==
!       ==     n_o =  sum_i(1-(r_ki/rc_2)**n_2)/(1-(r_ki/rc_2)**m_2)  ==
!       ==     nL  = sum_l(n_hl) 
!       ==     n_hl = sum_j(1-(r_lj/rc_1)**n_1)/(1-(r_lj/rc_1)**m_1)  ==
!       ==     fc(nL) = (1-(nL/rc_5)**n_L)/ (1-(nL/rc_5)**m_L)

        ELSEIF(INDEX(LINE,'HYD_CUT').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 27
          I1=INDEX(LINE,'HYD_CUT')+7 
!          Defaults

          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(11,NCOLVAR),ERREAD)
          
          READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE2
          II=INDEX(LINE2,'H:')
          IF(II.NE.0) THEN
              I1 = II+2
              CALL READSI(LINE2,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          ENDIF
          II=INDEX(LINE2,'O:')
          IF(II.NE.0) THEN
              I1 = II+2
              CALL READSI(LINE2,I1,IOUT,ATCVAR(5,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(6,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
              I1=IOUT
          ENDIF
          II=INDEX(LINE2,'M:')
          IF(II.NE.0) THEN
              I1 = II+2
              CALL READSI(LINE2,I1,IOUT,ATCVAR(7,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(8,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(3,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(4,NCOLVAR),ERREAD)
              I1=IOUT
          ENDIF
          II=INDEX(LINE2,'FCUT:')
          IF(II.NE.0) THEN
              I1 = II+5
              CALL READSI(LINE2,I1,IOUT,ATCVAR(9,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(10,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(5,NCOLVAR),ERREAD)
          ENDIF
          II=INDEX(LINE2,'LAMBDA:')
          IF(II.NE.0) THEN
              I1 = II+7
              CALL READSR(LINE2,I1,IOUT,CVPAR(6,NCOLVAR),ERREAD)
          ENDIF

          IF(FIRSTDLMN .EQ. 0) THEN
              CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
              CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
              FIRSTDLMN = 1
          ENDIF
          READ(IUNIT,*)  (IATDLMN(II+MY_NAT*(NCOLVAR-1)),
     &                           II=1,ATCVAR(11,NCOLVAR))


          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)


C       ==------------     DIPOLERHO          -------------==
C       ==Norm of d = [1/Q sum_iq_i(R_i-r_0)]

        ELSEIF(INDEX(LINE,'DIPOLERHO').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 31
          I1=INDEX(LINE,'DIPOLERHO')+9

          
         IF(FIRSTDLMN .EQ. 0) THEN
             CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
             CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
             FIRSTDLMN = 1                                        
         ENDIF                                                    

         IF(SECONDTDLMN .EQ. 0) THEN
             CALL MEMORY(IP_IQATDLMN,MY_NAT*NNVAR,'IQATDLMN')
             CALL IAZZERO(IQATDLMN,MY_NAT*NNVAR)
             SECONDTDLMN = 1
         ENDIF


          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)= NAT_cpmd(IDUMMY)
          I1=IOUT          
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)= IDUMMY  
 
          READ(IUNIT,*)  (IATDLMN(II+MY_NAT*(NCOLVAR-1)),
     &                    II=1,ATCVAR(2,NCOLVAR))
                   


         READ(IUNIT,*)  (IQATDLMN(II+MY_NAT*(NCOLVAR-1)), 
     &                             II=1,ATCVAR(2,NCOLVAR))



          
         CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)  

        
C       ==------------     DIPOLETHETA          -------------==
C       ==Theta of d = [1/Q sum_iq_i(R_i-r_0)]

        ELSEIF(INDEX(LINE,'DIPOLETHA').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 32
          IANGCV(NCOLVAR) = 1
          I1=INDEX(LINE,'DIPOLETHA')+9

          
         IF(FIRSTDLMN .EQ. 0) THEN
             CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
             CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
             FIRSTDLMN = 1                                        
         ENDIF                                                    

         IF(SECONDTDLMN .EQ. 0) THEN
             CALL MEMORY(IP_IQATDLMN,MY_NAT*NNVAR,'IQATDLMN')
             CALL IAZZERO(IQATDLMN,MY_NAT*NNVAR)
             SECONDTDLMN = 1
         ENDIF

          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)= NAT_cpmd(IDUMMY)
          I1=IOUT          
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)= IDUMMY  
   
          READ(IUNIT,*)  (IATDLMN(II+MY_NAT*(NCOLVAR-1)),          
     &                             II=1,ATCVAR(2,NCOLVAR))
          
          READ(IUNIT,*)  (IQATDLMN(II+MY_NAT*(NCOLVAR-1)),   
     &                             II=1,ATCVAR(2,NCOLVAR))
  
          
         CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &              NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)

               
C       ==------------     DIPOLEPHI          -------------==
C       ==Phi of d = [1/Q sum_iq_i(R_i-r_0)]

        ELSEIF(INDEX(LINE,'DIPOLEPHI').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 33
          IANGCV(NCOLVAR) = 2
          I1=INDEX(LINE,'DIPOLEPHI')+9

          
         IF(FIRSTDLMN .EQ. 0) THEN
             CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR,'IATDLMN')
             CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
             FIRSTDLMN = 1                                        
         ENDIF                                                    

         IF(SECONDTDLMN .EQ. 0) THEN
             CALL MEMORY(IP_IQATDLMN,MY_NAT*NNVAR,'IQATDLMN')
             CALL IAZZERO(IQATDLMN,MY_NAT*NNVAR)
             SECONDTDLMN = 1
         ENDIF

          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)= NAT_cpmd(IDUMMY)
          I1=IOUT          
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)= IDUMMY  
   
          READ(IUNIT,*)  (IATDLMN(II+MY_NAT*(NCOLVAR-1)),          
     &                             II=1,ATCVAR(2,NCOLVAR))
          
          READ(IUNIT,*)  (IQATDLMN(II+MY_NAT*(NCOLVAR-1)),   
     &                             II=1,ATCVAR(2,NCOLVAR))
  
          
         CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                  NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
                                                                       
!       ==------------ DISTANCE BETWEEN ION AND HYDRONIUM ----------==
!       ==     F =(1/lambda)*log(sum_k exp{lambda*f(n_H)*n_O})      ==
!       ==     f(n_h) = 1-(1-(n_h/rc_3)**n_3)/(1-(n_h/rc_3)**m_3)   ==
!       ==     n_h =  sum_j(1-(r_kj/rc_1)**n_1)/(1-(r_kj/rc_1)**m_1)==
!       ==     n_o =  sum_i(1-(r_ki/rc_2)**n_2)/(1-(r_ki/rc_2)**m_2)==

        ELSEIF(INDEX(LINE,'DIS_HYD').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 20 
          I1=INDEX(LINE,'DIS_HYD')+7 
C          Defaults
          ATCVAR(4,NCOLVAR) = 6
          ATCVAR(5,NCOLVAR) = 8
          CVPAR(1,NCOLVAR)  = 2.4D0
          ATCVAR(6,NCOLVAR) = 6
          ATCVAR(7,NCOLVAR) = 6
          CVPAR(2,NCOLVAR)  = 5.5D0
          ATCVAR(8,NCOLVAR) = 10
          ATCVAR(9,NCOLVAR) = 15
          CVPAR(3,NCOLVAR)  = 2.40D0
          CVPAR(4,NCOLVAR)  = 100.0D0

          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT 
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          
          II=INDEX(LINE,'=')
          IF(II.NE.0) THEN
            READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE2
            II=INDEX(LINE2,'H:')
            IF(II.NE.0) THEN
              I1 = II+2
              CALL READSI(LINE2,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(5,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
            ENDIF
            II=INDEX(LINE2,'O:')
            IF(II.NE.0) THEN
              I1 = II+2
              CALL READSI(LINE2,I1,IOUT,ATCVAR(6,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(7,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
              I1=IOUT
            ENDIF
            II=INDEX(LINE2,'FCUT:')
            IF(II.NE.0) THEN
              I1 = II+5
              CALL READSI(LINE2,I1,IOUT,ATCVAR(8,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(9,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(3,NCOLVAR),ERREAD)
            ENDIF
            II=INDEX(LINE2,'LAMBDA:')
            IF(II.NE.0) THEN
              I1 = II+7
              CALL READSR(LINE2,I1,IOUT,CVPAR(4,NCOLVAR),ERREAD)
            ENDIF
          ENDIF ! cmb - this endif must stay here as in version 3.11

          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)


!       ==------------ DISTANCE BETWEEN ION AND HYDRONIUM COMPLEX B -----------==
!       ==     F =fc(nL)[sum_k R_kI e^{Lambda*nHk*nOk}]/[sum_k e^{Lambda*nHk*nOk}]  ==
!       ==     nHk =  sum_j(1-(r_kj/rc_1)**n_1)/(1-(r_kj/rc_1)**m_1)==
!       ==     nOk =  sum_i(1-(r_ki/rc_2)**n_2)/(1-(r_ki/rc_2)**m_2)==
!       ==     nL  = sum_l(n_hl) 
!       ==     n_hl = sum_j(1-(r_lj/rc_1)**n_1)/(1-(r_lj/rc_1)**m_1)  ==
!       ==     fc(nL) = (1-(nL/rc_5)**n_L)/ (1-(nL/rc_5)**m_L)

        ELSEIF(INDEX(LINE,'HYD_D_CUT').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 28
          I1=INDEX(LINE,'HYD_D_CUT')+9
!          Defaults

          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(10,NCOLVAR),ERREAD)
          
          READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE2
          II=INDEX(LINE2,'H:')
          IF(II.NE.0) THEN
              I1 = II+2
              CALL READSI(LINE2,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(5,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          ENDIF
          II=INDEX(LINE2,'O:')
          IF(II.NE.0) THEN
              I1 = II+2
              CALL READSI(LINE2,I1,IOUT,ATCVAR(6,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(7,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
              I1=IOUT
          ENDIF
          II=INDEX(LINE2,'FCUT:')
          IF(II.NE.0) THEN
              I1 = II+5
              CALL READSI(LINE2,I1,IOUT,ATCVAR(8,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSI(LINE2,I1,IOUT,ATCVAR(9,NCOLVAR),ERREAD)
              I1=IOUT
              CALL READSR(LINE2,I1,IOUT,CVPAR(3,NCOLVAR),ERREAD)
          ENDIF
          II=INDEX(LINE2,'LAMBDA:')
          IF(II.NE.0) THEN
              I1 = II+7
              CALL READSR(LINE2,I1,IOUT,CVPAR(4,NCOLVAR),ERREAD)
          ENDIF

          IF(FIRSTDLMN .EQ. 0) THEN
              CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
              CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
              FIRSTDLMN = 1
          ENDIF
          READ(IUNIT,*)  (IATDLMN(II+MY_NAT*(NCOLVAR-1)),
     &                           II=1,ATCVAR(10,NCOLVAR))


          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C       ==------------ LOCALIZATION OF THE SPIN DENSITY  -----------==
        ELSEIF(INDEX(LINE,'SPIN').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 21
          IF(.not. TLSD)  CALL STOPGM('M_COLVAR_INP',
     &      ' ERROR: LSD is required for SPIN CV')
          IF(.NOT. TLOCALIZESPIN) THEN
            CALL MEMORY(IP_RCC0,3*NNVAR,'RCC0')
            CALL AZZERO(RCC0,3*NNVAR)
            CALL MEMORY(IP_ICV_SPIN,NNVAR/IRAT+1,'ICV_SPIN')
            CALL IAZZERO(ICV_SPIN,NNVAR)
            TLOCALIZESPIN=.TRUE.
          ENDIF
          ICV_SPIN(NCOLVAR)=NCOLVAR

          I1=INDEX(LINE,'SPIN')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          IF(ERREAD) GOTO 22
          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
        ELSEIF(INDEX(LINE,'VOLVAR').NE.0) THEN
          IF(.NOT. TPRCP) THEN
            CALL STOPGM('M_COLVAR_INP',
     &      ' ERROR: Parr.-Rahm. required for variable cell')
          ENDIF
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 22
          IF(.NOT. TCVCELL) THEN
            CALL MEMORY(IP_ICV_CELL,NNVAR/IRAT+1,'ICV_CELL')
            CALL IAZZERO(ICV_CELL,NNVAR)
            TCVCELL=.TRUE.
          ENDIF

          ICV_CELL(NCOLVAR)=NCOLVAR

          I1=INDEX(LINE,'VOLVAR')+6
C           CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
C           IF(ERREAD) GOTO 22

          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
        ELSEIF(INDEX(LINE,'CELLSIDE').NE.0) THEN
          IF(.NOT. TPRCP) THEN
            CALL STOPGM('M_COLVAR_INP',
     &      ' ERROR: Parrnello-Rahman required for variable cell')
          ENDIF
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 23
          IF(.NOT. TCVCELL) THEN
            CALL MEMORY(IP_ICV_CELL,NNVAR/IRAT+1,'ICV_CELL')
            CALL IAZZERO(ICV_CELL,NNVAR)
            TCVCELL=.TRUE.
          ENDIF

          ICV_CELL(NCOLVAR)=NCOLVAR

          I1=INDEX(LINE,'CELLSIDE')+8
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          IF(ERREAD) GOTO 22

          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
        ELSEIF(INDEX(LINE,'CELLANGLE').NE.0) THEN
          IF(.NOT. TPRCP) THEN
            CALL STOPGM('M_COLVAR_INP',
     &      ' ERROR: Parrinello-Rahman required for variable cell')
          ENDIF
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 24
          IF(.NOT. TCVCELL) THEN
            CALL MEMORY(IP_ICV_CELL,NNVAR/IRAT+1,'ICV_CELL')
            CALL IAZZERO(ICV_CELL,NNVAR)
            TCVCELL=.TRUE.
          ENDIF

          ICV_CELL(NCOLVAR)=NCOLVAR

          I1=INDEX(LINE,'CELLANGLE')+9
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          IF(ERREAD) GOTO 22

          CALL PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &                    CV_DYN_0,INITIAL_VALUE,
     &                    NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)

        ENDIF

        IF(ERREAD) GOTO 20

        IF(NCOLVAR.GT.NNVAR) THEN
C         Problem the number of coll.variables NCOLVAR .NE. given NFIX.
          WRITE(6,'(1X,A,I5)') 
     &         'META_COLVAR_INP! # OF GIVEN COL. VAR= ',NNVAR
          WRITE(6,'(1X,A,I5)') 
     &         'META_COLVAR_INP! # OF FOUND COL. VAR=',NCOLVAR
          WRITE(6,'(1X,A)')
     &         'THE NUMBER OF GIVEN COL. VAR. IS TOO SMALL.'
          CALL STOPGM('M_COLVAR_INP',
     &         'ERROR WHILE READING FIX STRUCTURES')
        ELSEIF(NCOLVAR.LT.NNVAR) THEn
          GOTO 12
        ELSE
          GOTO 11
        ENDIF
C     ==------------ MAX NUMBER OF METADYNAMICS STEPS ----------------==
      ELSEIF(INDEX(LINE,'METASTEPNUM').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) I_META_MAX
        GOTO 10
C     ==-------- RESTART META DYNAMICS FROM METASTEPI_META_RES -------==
      ELSEIF(INDEX(LINE,'META_RESTART').NE.0) THEN
        META_RESTART = .TRUE.
        IF(INDEX(LINE,'RFILE').NE.0) THEN
          TRESFILE = .TRUE.
          CALL MEMORY(ip_dummy,NCOLVAR,'dummy')
          IF(TMULTI) THEN
            CALL RMTDRESM(50,dummy,.FALSE.,1)
          ELSEIF(TMW)THEN
            DO IWALK1=1,NWALK
              CALL MW_FILENAME('MTD_RESTART_',FMTDRES,IWALK1)
              CALL RMTDRES_MW(50,dummy,.FALSE.,1,IWALK1)
            END DO
          ELSE
            CALL RMTDRES(50,dummy,.FALSE.,1)
          ENDIF
          CALL FREEM(ip_dummy)
        ELSE
          READ(IUNIT,ERR=20,END=20,FMT=*) I_META_RES
        ENDIF
        GOTO 10
C     ==---------------- MORE THAN ONE SET OF CV ---------------------==
      ELSEIF(INDEX(LINE,'MULTI').NE.0 .AND. 
     &             INDEX(LINE,'NUM') .NE.0) THEN
        IF(.NOT. TMULTI) CALL STOPGM('M_COLVAR_INP',
     &          'Multy Meta Dynamics not active. ')
        CALL MEMORY(IP_NCVSYS,NSUBSYS/IRAT + 1,'NCVSYS')
        READ(IUNIT,*) (NCVSYS(I),I = 1,NSUBSYS)
        GOTO 10
C     ==------------- DIMENSIONS OF HILLS IN CV SPACE-----------------==
      ELSEIF(INDEX(LINE,'HILLS').NE.0) THEN
        IF(INDEX(LINE,'OFF').NE.0) THEN
          LHILLS=.FALSE.
        ELSE
          LHILLS = .TRUE.
          IF(INDEX(LINE,'LOREN').NE.0) THEN
            HLORE = .TRUE.
          ELSE IF(INDEX(LINE,'SPHERE').NE.0) THEN
            SPHERE= .TRUE.
          ELSEIF(INDEX(LINE,'RATIO').NE.0) THEN
            HRATIO = .TRUE.
            HSHIFT = .FALSE.
            IJ=INDEX(LINE,'POW')
            IF(IJ.NE.0) THEN
              IA = IJ+3
              CALL READSR(LINE,IA,IE,EXPUP,   ERREAD)
              IF(ERREAD) GOTO 20
              IA = IE
              CALL READSR(LINE,IA,IE,EXPDOWN,   ERREAD)
              IF(ERREAD) GOTO 20
              IA = IE
              CALL READSR(LINE,IA,IE,FBOOST,   ERREAD)
              IF(ERREAD) GOTO 20
            ENDIF
          ELSEIF(INDEX(LINE,'SHIFT').NE.0) THEN
            HSHIFT = .TRUE.
            HRATIO = .FALSE.
            IJ=INDEX(LINE,'RCUT')
            IF(IJ.NE.0) THEN
              IA = IJ+4
              CALL READSR(LINE,IA,IE,R0_SHIFT,   ERREAD)
              IF(ERREAD) GOTO 20
              IA = IE
              CALL READSR(LINE,IA,IE,FBOOST,   ERREAD)
              IF(ERREAD) GOTO 20
            ENDIF

          ENDIF
          II=INDEX(LINE,'=')
          IF(II.NE.0) THEN
            IF(TMULTI) THEN
              DO I = 1, NSUBSYS
                READ(IUNIT,ERR=20,END=20,FMT=*)
     &           HWM(I),HHM(I),HLHM(I),HTHM(I)
                 HVM0(I) = HHM(I) * HLLW**DBLE(NCVSYS(I))
              ENDDO
            ELSE
              IA = II+1
              CALL READSR(LINE,IA,IE,HLLW,   ERREAD)
              IF(ERREAD) GOTO 20
              IA = IE
              CALL READSR(LINE,IA,IE,HLLH,   ERREAD)
              IF(ERREAD) GOTO 20
              HVOL0 = HLLH * (HLLW)**DBLE(NCOLVAR)
            ENDIF
          ENDIF
        ENDIF
        GOTO 10
C     ==---------------------- TUNING PARAMETERS ---------------------==
      ELSEIF(INDEX(LINE,'TUNING').NE.0) THEN
        IF(INDEX(LINE,'OFF').NE.0) THEN
          LTUNE_HH     = .FALSE.
          LTUNE_CSCL   = .FALSE.
        ENDIF     
        IF(INDEX(LINE,'HHEIGHT').NE.0) THEN
          LTUNE_HH     = .TRUE.
          II=INDEX(LINE,'=')
          IF(II.NE.0) THEN
            IA = II+1
            CALL READSR(LINE,IA,IE,HLOW,   ERREAD)
            IF(ERREAD) GOTO 20
            IA = IE
            CALL READSR(LINE,IA,IE,HTOP,   ERREAD)
            IF(ERREAD) GOTO 20
            IF(HTOP.LT.HLOW) THEN
              TEMP = HLOW
              HLOW = HTOP
              HTOP = TEMP
            ENDIF
          ENDIF
        ENDIF
        GOTO 10
C     ==---------MAX # OF DYN. STEPS BETWEEN 2 METASTEPS--------------==
      ELSEIF(INDEX(LINE,'MAXSTEPNUM').NE.0 .AND.
     &             INDEX(LINE,'INTERMETA') .NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) INTER_HILL_MAX
        GOTO 10
C     ==---------MIN # OF DYN. STEPS BETWEEN 2 METASTEPS--------------==
      ELSEIF(INDEX(LINE,'MINSTEPNUM').NE.0 .AND.
     &             INDEX(LINE,'INTERMETA') .NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) INTER_HILL
        GOTO 10
C     ==------------ INTERVAL BETWEEN DISPLACEMENT CHECKS-------------==
      ELSEIF(INDEX(LINE,'CHECK') .NE.0 .AND.
     &             INDEX(LINE,'DELAY').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) ICHECK
        GOTO 10
C     ==------------------------TOLL_FCNST----------------------------==
      ELSEIF(INDEX(LINE,'MOVEMENT') .NE.0 .AND.
     &             INDEX(LINE,'CHECK').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) TOLL_AVCV
        GOTO 10
C     ==----------- STORE RESTART AND TRAJECTORIES -------------------==
      ELSEIF(INDEX(LINE,'METASTORE') .NE.0) THEN
C     don t use TR_FREQ if NO TRAJECTORY is given.
        IF ((INDEX(LINE,'TRAJ').NE.0).AND.(INDEX(LINE,'NO').NE.0))THEN
          TTRAJOVRR=.FALSE.
        ENDIF
        READ(IUNIT,ERR=20,END=20,FMT=*) ST_FREQ,TR_FREQ,QW_FREQ
        GOTO 10
C     ==----------- DEF. OF BOUNDARIES IN CV. SPACE  ---------------==
      ELSEIF(INDEX(LINE,'CVSPACE').NE.0 .AND.
     &             INDEX(LINE,'BOUND') .NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) INUM
        IF(ERREAD) GOTO 20
        DO I = 1,INUM
          READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE2
          READ(LINE2,ERR=20,END=20,FMT=*) IND,vbar,r_wall,cpos_0
          II=INDEX(LINE2,'EPS')
          IF(II.NE.0) THEN
            IA = II+3
            CALL READSR(LINE2,IA,IE,VBOUND(4,IND),ERREAD)
          ELSE
            VBOUND(4,IND) = 0.01D0
          ENDIF
          IF(IBOUND(IND) .NE. 0 .OR. IND .GT. NCOLVAR)
     &          CALL STOPGM('M_COLVAR_INP',
     &          'ERROR IN  BOUNDARIES OF COLLECTIVE VAR. ')
          IBOUND(IND) = 2
          VBOUND(1,IND) = vbar
          VBOUND(2,IND) = r_wall
          VBOUND(3,IND) = cpos_0
        ENDDO
        GOTO 10
C     ==----------- COLLECTIVE VARIABLE OUTPUT  -------------------==
      ELSEIF(INDEX(LINE,'MONITOR') .NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) WCV_FREQ
        TCVMONITOR = .TRUE.
        GOTO 10
C     ==----- MAX ELECT. KIN. ENERGY (when above QUENCH BO) ----------==
      ELSEIF(INDEX(LINE,'MAXKINEN').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
        CALL READSR(LINE,1,IOUT,TOLKIN,ERREAD)
        GOTO 10
C     ==-------------- EXTENDED LAGRANGIAN INFO ------ ---------------==
      ELSEIF(INDEX(LINE,'LAGRANGE').NE.0) THEN
        LEXTLAGRANGE=.TRUE.
        IF(INDEX(LINE,'TEMPERATURE').NE.0) THEN
          READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
          CALL READSR(LINE,1,IOUT,CV_TEMP0,ERREAD)
          IF(ERREAD) CALL STOPGM('M_COLVAR_INP','ERR READ TEMP0 ') !GOTO 20
          GOTO 10
        ELSEIF(INDEX(LINE,'TEMPCONT').NE.0) THEN
          IF(INDEX(LINE,'GLOBAL').NE.0) THEN
            LTCGLOBAL = .TRUE.
          ELSE
            LCVTC = .TRUE.
            READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
            CALL READSR(LINE,1,IOUT,CV_TEMP,ERREAD)
            IF(ERREAD) 
     &        CALL STOPGM('M_COLVAR_INP','ERR READ TEMP ') !GOTO 20
            I1=IOUT+1
            CALL READSR(LINE,I1,IOUT,CV_DTEMP,ERREAD)
            IF(ERREAD) 
     &        CALL STOPGM('M_COLVAR_INP','ERR READ DTEMP ') !GOTO 20
          ENDIF
          GOTO 10
        ELSEIF(INDEX(LINE,'TSCALE').NE.0) THEN
          TCVSCALE = .TRUE.
          GOTO 10
        ELSEIF(INDEX(LINE,'LANGEVIN').NE.0) THEN
          TCVLANGEVIN = .TRUE.                   
          READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE 
            CALL READSR(LINE,1,IOUT,CV_LANGEVINTEMP,ERREAD) 
            IF(ERREAD)
     &     CALL STOPGM('M_COLVAR_INP','ERR READ CV_LANGEVINTEMP') !GOTO 20
            I1=IOUT+1
            CALL READSR(LINE,I1,IOUT,CV_LANGAMMA,ERREAD)
            IF(ERREAD)
     &        CALL STOPGM('M_COLVAR_INP','ERR READ CV_LANGAMMA') !GOTO 20
          GOTO 10
        ELSEIF(INDEX(LINE,'KADAPT').NE.0) THEN
          LKFIX = .FALSE.
          GOTO 10
        ELSE
          GOTO 10
        ENDIF
C     ==-------------------- CV  ANALYSIS -------------- ---------------==
      ELSEIF(INDEX(LINE,'ANALYS').NE.0) THEN
        TCVANALYSIS  = .TRUE.
        GOTO 10
C     ==-------------------- Volume of hills kept constant --------------==
      ELSEIF(INDEX(LINE,'HILLVOLUME').NE.0) THEN
        THILLVOL = .TRUE.
        GOTO 10
      ELSEIF(INDEX(LINE,'SKIPHILL').NE.0)THEN
        SKIPHILL_MW=.TRUE.
        GOTO 10
      ELSEIF(INDEX(LINE,'RANDWALK').NE.0)THEN
        RANDWALK=.TRUE.
        GOTO 10
C     =------ RESTRAIN THE CELL VOLUME IN THE RANGE VOLMIN VOLMAX ----==
      ELSEIF(INDEX(LINE,'RESTR').NE.0.AND.INDEX(LINE,'VOLU').NE.0)THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) FMIN,FMAX,VBARRIER
        VOLMIN = OMEGA0*FMIN
        VOLMAX = OMEGA0*FMAX
        TVOLBOUND = .TRUE.
        GOTO 10
      ELSE 
        GOTO 10
      ENDIF  !file beginning

   20 CONTINUE
        WRITE(6,*) ' ERROR WHILE READING METADYNAMICS VARIABLES '
        WRITE(6,*) '       last line read: '
        WRITE(6,*)  LINE
        CALL STOPGM('M_COLVAR_INP',' ')      
 22     CONTINUE
        WRITE(6,*) ' ERROR WHILE READING SCAL. FAC. or MASS or KH.'
        CALL STOPGM('M_COLVAR_INP',' ') 

   30 CONTINUE
C     ==--------------------------------------------------------------==
C     TESTS

      IF(NCOLVAR .EQ. 0) CALL STOPGM('M_COLVAR_INP',
     &                '# OF COLLECTIVE VARIABLES IS ZERO')
      DO IC = 1,NCOLVAR
        IF(TAD_SCF(IC)) THEN
          LTUNE_CSCL = .TRUE.
          GOTO 24
        ENDIF
      ENDDO
 24   CONTINUE
      IF(TCVANALYSIS) THEN
        LEXTLAGRANGE =.TRUE.
        LTUNE_HH     = .FALSE.
        LTUNE_CSCL   = .FALSE.
        LHILLS       = .FALSE.
        LCVTC        = .FALSE.
        CALL IAZZERO(IBOUND,NNVAR)
        CALL DCOPY(NNVAR,1.D0,0,CV_MASS,1)
        CALL AZZERO(KHARM,NNVAR)
        TOLL_AVCV  = 0.0D0
        CV_TEMP0 = 0.0D0
        INTER_HILL = 10
      ENDIF
      IF(HLLW .EQ. 0.0D0) THEN
        CALL STOPGM('M_COLVAR_INP',
     &               'Hill-width = 0 is too dangerous')
      ENDIF
      IF(QW_FREQ.LE.0) THEN
        CALL STOPGM('M_COLVAR_INP',
     &               'Illegal QUENCH BO frequency')
      ENDIF
      IF(THILLVOL .AND. .NOT.LTUNE_HH ) THEN
        WRITE(6,'(A,A)') ' WARNING: the hill tuning is off,',
     &                 ' height and wdth are constant'
        THILLVOL = .FALSE.
      ENDIF
      IF(TVOLBOUND .AND. .NOT. TPRCP) THEN
        WRITE(6,'(A,A)') ' WARNING: if not Parr.-Rahm. ',
     &                   ' the volume of the cell is constant'
        TVOLBOUND = .FALSE.
      ENDIF
C     ==--------------------------------------------------------------==
C     Print initialization

      WRITE(6,*) 
      WRITE(6,'(72("*"))')
      WRITE(6,'(1x,A,A)') 
     &         '***             META DYNAMICS OF COLLECTIVE VARIABLES',
     &         '             ***'
      WRITE(6,'(4x,A,I5)') '- Number of Collective Variables =',
     &                  NCOLVAR
      IF(TMULTI) THEN
        WRITE(6,'(4x,A,I5)') '- Multiple meta dynamics: # of CV sets =',
     &        NSUBSYS
        WRITE(6,'(12x,A,10(I4))') ' # of CV per set =',
     &                          (NCVSYS(I),I=1,NSUBSYS)
      ENDIF
      IF(NWALK.GT.1)
     &   WRITE(6,'(4x,A,I5)')
     &      '- Multiple walker metadnamics: # of walkers=',NWALK
      WRITE(6,'(4x,A)') '- Names of Output Files :'
      WRITE(6,'(6x,A)')     
     &   'colvar_mtd   : CV values and scaling factors'
      WRITE(6,'(6x,A/6x,A)') 
     &   'parvar_mtd  : norm of the displacement along the traj.',
     &   '              transversal hill width (fixed),',
     &   '              hills heights HLLH'
      IF(LEXTLAGRANGE) THEN
        WRITE(6,'(6x,A)') 
     &   'istvar_mtd: istantaneous CV and difference wrt dynamic CV'
        WRITE(6,'(6x,A/6x,A/6x,A)')
     &   'disvar_mtd  : CV displacements and ',
     &   '               diffusivities of the CV ',
     &   '               force constants for the coupling potential,'
      WRITE(6,'(6x,A/6x,A/6x,A)') 
     &   'forfac_mtd  : forces due to coupling potential,',
     &   '               forces due to accumulation of hills',
     &   '               forces due to boundary potential.'
        WRITE(6,'(6x,A)') 'velvar_mtd  : CV velocities.'
        WRITE(6,'(6x,A/10x,A/10x,A)') 
     &   'enevar_mtd  :  ions Temp., el. Ekin, CV Ekin (EKCV), ',
     &   '         coupling pot. (VH), gaussians pot. (VG),',
     &   '         i+el. pot. energy (EKS), H0+EKCV+VH,H0+EKCV+VH+VG'
      ELSE
        WRITE(6,'(6x,A)') 
     &   'scavar_mtd: scaled CV and scaled diffusivities'
        WRITE(6,'(6x,A)') 
     &   'disvar_mtd  :  displacements and diffusivities of the CV.'
        WRITE(6,'(6x,A/6x,A)') 
     &   'forfac_mtd  : forces due to  accumulation of hills,',
     &   '               forces due to boundary potential.'
        WRITE(6,'(6x,A/6x,A/9x,A)') 
     &   'enevar_mtd  :  ions Temp., el. Ekin, ',
     &   '               gaussians pot. (VG),',
     &   '            ions+el. pot. energy(EKS), H0+VG '

      ENDIF

      WRITE(6,'(6x,A,I10,A)')
     &    '- Collective variables written every ', WCV_FREQ, ' MD Steps'
      WRITE(6,'(6x,A,I10,A)')
     &    '- RESTART File Saved every ', ST_FREQ, ' Metasteps'
      IF(TTRAJOVRR) WRITE(6,'(6x,A,I10,A)')
     &    '- TRAJECTORY File Appended every', TR_FREQ,' Metasteps'
      WRITE(6,'(6x,A,I10,A)')
     &    '- QUENCH BO performed every', QW_FREQ, ' Metasteps'
      IF(TC .AND. TOLKIN .LT. EKINW+TOLL) THEN 
        WRITE(6,'(6x,A,F12.6)')
     &    '- QUENCH BO performed when EL. Ekin is > ', TOLKIN
      ELSEIF(TOLKIN.NE.-1.0D0) THEN
        WRITE(6,'(6x,A,F12.6)')
     &    '- QUENCH BO performed when EL. Ekin is > ', TOLKIN
      ENDIF
      IF(META_RESTART) WRITE(6,'(6x,A,1X,I8)') 
     &       '- Restart from a previous Metadynamics at step',I_META_RES
      IF(LHILLS) THEN 
        WRITE(6,'(6x,A)')
     &           '- Time Dependent Potential Active: '

        IF(TMULTI) THEN
          DO I = 1,NSUBSYS
            WRITE(6,'(12x,A,I4,A,F9.5,A,F9.5)') 
     &                ' set ', I, ' Transversal Width = ',
     &                 HWM(I),'    Initial Height = ',HHM(I)
          ENDDO
        ELSE
          WRITE(6,'(15x,A,F9.5,A,F9.5)') ' Transversal Width = ',
     &                   HLLW,'    Initial Height = ',HLLH
        ENDIF
        IF(HSHIFT) THEN
          WRITE(6,'(6x,A/15x,A,F8.3,A,f8.3)') 
     &                '- Hills Shape : Truncated Gaussians',
     &                ' Rcut = ',RSHIFT,'     Width Factor = ',FBOOST
        ELSEIF(HRATIO) THEN
          WRITE(6,'(6x,A/15x,A,F8.3,F8.3,A,f8.3)') 
     &          '- Hills Shape : Rational Function',
     &          ' Exponents : ',EXPUP,EXPDOWN,
     &          '     Width Factor = ',FBOOST
        ENDIF
        IF(LTUNE_HH) THEN
          WRITE(6,'(6x,A)')
     &      '- Hills Height tuned on the Underlying potential'
          IF(TMULTI) THEN
            DO I = 1,NSUBSYS
              WRITE(6,'(12x,A,I4,A,F10.6,A,f10.6,A)') ' set ',I,
     &        'min. Height ',HLHM(I)*AU_KCM,' Kcal,    max Height ',
     &         HTHM(I)*AU_KCM,' Kcal.'
            ENDDO
          ELSE
            WRITE(6,'(15x,A,F10.6,A,f10.6,A/)')
     &        'min. Height ',HLOW*AU_KCM,' Kcal,    max Height ',
     &         HTOP*AU_KCM,' Kcal.'
          ENDIF
          IF(THILLVOL) THEN
            WRITE(6,'(12x,A)') 
     &           ' Hill volume Height*(Width)^(NCOLVAR) = constant'
          ENDIF
        ENDIF
        IF(LTUNE_CSCL)
     &    WRITE(6,'(6x,A)') 
     &      '- Some scaling factors might be adjusted along the run'
      ENDIF

      IF(LHILLS) THEN
        WRITE(6,'(6x,A,F12.6)')
     &    '- Minimum. CV Displacement Required to Place New a Hill',
     &    TOLL_AVCV
        WRITE(6,'(6x,A,I5,A)') 
     &   '- Update of Gaussian Pot. every ', 
     &           INTER_HILL,' MD steps (when CV moved enough)'
        WRITE(6,'(6x,A,I10)') 
     &   '- Max. Number of MD Steps before next Update',INTER_HILL_MAX
      ENDIF
      IF(TCVANALYSIS) THEN
        WRITE(6,'(6x,A,A)') '- MD run for the analysis of CV behavior ',
     &         'without additional potentials'
        WRITE(6,'(6x,A,I5,A)') 
     &   '- Output written every ', 
     &           INTER_HILL,' MD steps'
      ENDIF
      IF(LEXTLAGRANGE) THEN
        WRITE(6,'(/6x,A,A)') 
     &         '***         CV DYNAMICS VIA EXTENDED LAGRANGIAN',
     &         '         ***'
        WRITE(6,'(10x,A,f9.3,A)')  
     &             '- Initial CV Temperature  ',CV_TEMP0,' K'
        IF(LCVTC) WRITE(6,'(10x,A/6x,A,f9.3,A,A,f9.3,A)') 
     &       '- CV Temperature controlled by velocity rescaling',
     &       '        mean T = ',CV_TEMP,' K,',
     &       '      tolerance= ',CV_DTEMP,' K'
        IF(TCVLANGEVIN) WRITE(6,'(10x,A/6x,A,f9.3,A,A,f9.6,A)')      
     &   '- CV Temperature controlled by Langevin Dynamics',
     &       '        mean T = ',CV_LANGEVINTEMP,' K,',
     &       '      Gamma = ',CV_LANGAMMA,' a. u.'                   
        IF(.NOT.LKFIX)  WRITE(6,'(10x,A,A/)')
     &     '- Force Constants for Coupling Pot.',
     &     ' Tuned on Underlying Pot. Surf.' 
      ENDIF

      IF(TVOLBOUND) THEN
        WRITE(6,'(6x,A/A,f12.6,A/A,f12.6,A)') 
     &             '- PR MD with control on Volume: ',
     &             '                   min Volume = ', VOLMIN,' a.u.^3',
     &             '                   max Volume = ', VOLMAX, 'a.u.^3'

      ENDIF

      WRITE(6,'(/15x,A)') '   >>>>>>> COLLECTIVE VARIABLE INFO <<<<<<<'
      WRITE(6,'(2x,A,5x,A,5x,A,20x,A,11x,A)')
     &                 'TYPE','ATOMS AND SPEC.',
     &                 'PARAMETERS',
     &                 'SCALING FAC.','BOX POT '

      IPGRPA1=0
      IPGRPB1=0
      DO I=1,NCOLVAR
        ITYP=TYCVAR(I)
C COORD
        IF(ITYP.EQ.6) THEN
          WRITE(6,'(A,4X,I5,15X,2f8.4,4x,3F8.4,5x,4F8.4)') STYP(ITYP),
     &         NAT_grm(ATCVAR(1,I)),CVPAR(1,I),CVPAR(2,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     &         (VBOUND(K,I),K=1,4)
        ELSEIF(ITYP.EQ.30) THEN
          WRITE(6,'(A,4X,3I5,15X,2f8.4,4x,3F8.4,5x,4F8.4)') STYP(ITYP),
     &         NAT_grm(ATCVAR(1,I)),NAT_grm(ATCVAR(2,I)),ATCVAR(2,I),
     &         CVPAR(1,I),CVPAR(2,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     &         (VBOUND(K,I),K=1,4)
C COORSP
        ELSEIF(ITYP.EQ.8) THEN
          WRITE(6,'(A,2X,2I5,15X,2f8.4,5x,3F6.2,5x,4F8.4)') STYP(ITYP),
     *         NAT_grm(ATCVAR(1,I)),ATCVAR(2,I),CVPAR(1,I),CVPAR(2,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     &         (VBOUND(K,I),K=1,4)
        ELSEIF(ITYP.EQ.29) THEN
          WRITE(6,'(A,I5,3F16.2,4F8.4)') STYP(ITYP),ATCVAR(1,I),
     c        CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     c        (VBOUND(K,I),K=1,4)
          IPGRPB3=0
          DO K=1,ATCVAR(1,I)
              IPGRPA1=IPGRPA1+1
              WRITE(6,'(I8,F16.5,I8)')IATCNGA(IPGRPA1),RCCNGA(IPGRPA1),
     c                                NATCNGB(IPGRPA1)
              DO KK=1,NATCNGB(IPGRPA1)
                 IPGRPB3=IPGRPB3+1
                 WRITE(6,'(I8)')IATCNGB(IPGRPB1+IPGRPB3)
              END DO
          END DO
          IPGRPB1=IPGRPB1+ATCVAR(1,I)*MAX_NATCNGB
C DIPOLERHO 
        ELSEIF(ITYP.EQ.31) THEN
          WRITE(6,'(A,2X,1I5,18X,1I5, 18x,3F6.2,5x,4F8.4)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     &         (VBOUND(K,I),K=1,4)                                     
C DIPOLETHETA
        ELSEIF(ITYP.EQ.32) THEN
          WRITE(6,'(A,2X,1I5,18X,1I5,18x,3F6.2,5x,4F8.4)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     &         (VBOUND(K,I),K=1,4)                                     
C DIPOLEPHI
        ELSEIF(ITYP.EQ.33) THEN
          WRITE(6,'(A,2X,1I5,18X,1I5,18x,3F6.2,5x,4F8.4)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     &         (VBOUND(K,I),K=1,4)                                     
C VCOORS 
        ELSEIF(ITYP.EQ.34) THEN
          WRITE(6,'(A,2X,1I5,20X,2f8.4,5x,3F6.2,5x,4F8.4)') STYP(ITYP),
     *         ATCVAR(1,I),CVPAR(1,I),CVPAR(2,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     &         (VBOUND(K,I),K=1,4)                                     
C RMSD_AB, DISPL1, COORDIS
        ELSEIF(ITYP .GE.9 .AND. ITYP.LE.11) THEN
          IF(CVPAR(2,I) .NE. 0.0D0) THEN
            WRITE(6,'(A,2I5,5x,A,2I4,2f7.3,2x,3F8.4,3x,4F7.3)') 
     &         STYP(ITYP),NAT_grm(ATCVAR(1,I)),ATCVAR(2,I),
     *         '2 SHELL',ATCVAR(3,I),ATCVAR(4,I),
     &         CVPAR(1,I),CVPAR(2,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     &         (VBOUND(K,I),K=1,4)
          ELSE
            WRITE(6,'(A,2I5,13x,2I4,f7.3,7x,3F8.4,5x,4F8.4)')
     *         STYP(ITYP),NAT_grm(ATCVAR(1,I)),ATCVAR(2,I),
     &         ATCVAR(3,I),ATCVAR(4,I),
     &         CVPAR(1,I),CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     &         (VBOUND(K,I),K=1,4)
          ENDIF
        ELSEIF(ITYP .EQ. 17) THEN
          WRITE(6,'(A,3I4,11x,2I3,f7.3,7x,3F8.4,5x,4F8.4)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &         ATCVAR(5,I),CVPAR(1,I),CSCL_FAC(1,I),CSCL_FAC(2,I),
     &         CSCL_FAC(3,I),(VBOUND(K,I),K=1,4)
        ELSEIF(ITYP .EQ. 18) THEN
          WRITE(6,'(A,3I4,8x,2I3,2f7.3,3x,3F8.4,5x,4F8.4)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &         ATCVAR(5,I),CVPAR(1,I),CSCL_FAC(1,I),CSCL_FAC(2,I),
     &         CSCL_FAC(3,I),(VBOUND(K,I),K=1,4)
        ELSEIF(ITYP .EQ. 12) THEN
          NUMSPEC = ATCVAR(1,I)
          WRITE(CHNUM,'(I5)') NUMSPEC
          CALL XSTRING(CHNUM,IA,IE)
          LINEFORM ='(A,2X,I4,A,'//CHNUM(IA:IE)//
     &               'I4,10x,3f8.3,5x,4F8.4)'
          WRITE(6,LINEFORM) STYP(ITYP),ATCVAR(1,I),
     &    ' Species ',(ATCVAR(k,I),K=1,NUMSPEC),CSCL_FAC(1,I),
     &      CSCL_FAC(2,I),CSCL_FAC(3,I),(VBOUND(K,I),K=1,4)
        ELSEIF(ITYP.EQ.13 .AND. SPECINDEX(I) .EQ. 1) THEN
          NUMSPEC = ATCVAR(1,I)
          NUMSPEC2 = ATCVAR(2,I)
          NUMTOT  = NUMSPEC+NUMSPEC2
          WRITE(CHNUM,'(I5)') NUMSPEC
          CALL XSTRING(CHNUM,IA,IE)
          IF(NUMSPEC2 .NE. 0) THEN
            WRITE(CHNUM2,'(I5)') NUMSPEC2
            CALL XSTRING(CHNUM2,IAA,IEE)
            LINEFORM ='(A,2X,I4,A9,'//CHNUM(IA:IE)//'I4,A4,I4,A9,'
     &              //CHNUM2(IA:IE)//'I4,A15,3I3,2X,3f8.4)'
            WRITE(6,LINEFORM) STYP(ITYP),ATCVAR(1,I),
     &      ' Species: ',(ATCVAR(K+2,I),K=1,NUMSPEC),' vs ',ATCVAR(2,I),
     &      ' Species: ',(ATCVAR(K+2+NUMSPEC,I),K=1,NUMSPEC2),
     &      ' Miller Index: ',(ATCVAR(K+2+NUMTOT,I),K=1,3),
     &        CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I)
          ELSE
            LINEFORM ='(A,2X,I4,A9,'//CHNUM(IA:IE)//
     &                'I4,A7,A15,3I3,2X,3f8.4)'
           WRITE(6,LINEFORM) STYP(ITYP),ATCVAR(1,I),
     &      ' Species: ',(ATCVAR(K+2,I),K=1,NUMSPEC),' vs all',
     &      ' Miller Index: ',(ATCVAR(K+2+NUMTOT,I),K=1,3),
     &        CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I)
          ENDIF
        ELSEIF(ITYP.EQ.13 .AND. SPECINDEX(I) .NE. 1) THEN
          WRITE(CHNUM,'(I5)') ATCVAR(1,I)
          CALL XSTRING(CHNUM,IA,IE)
          NUMSPEC2 = ATCVAR(2,I)
          IF(NUMSPEC2 .NE. 0) THEN
            WRITE(CHNUM2,'(I5)') ATCVAR(2,I)
            CALL XSTRING(CHNUM2,IAA,IEE)
            LINEFORM ='(A9,2X,A5,'//CHNUM(IA:IE)//'I4,A8,'
     &                //CHNUM2(IAA:IEE)//'I4,A10,3I3,2X,A4,3f8.4)'
            WRITE(6,LINEFORM) STYP(ITYP),' at.',
     &         (IATDLMN(K+MY_NAT*(I-1)),K=1,ATCVAR(1,I)),
     &         ' vs at.',(IATDLMN(ATCVAR(1,I)+K+MY_NAT*(I-1)),
     &         K=1,ATCVAR(2,I)),
     &      ' Mil. In.',(ATCVAR(K+2,I),K=1,3),' SCF',
     &        CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I)
          ELSE
            LINEFORM ='(A,2X,A5,'//CHNUM(IA:IE)//
     &                'I4,A8,A10,3I3,2X,A4,3f8.4)'
            WRITE(6,LINEFORM) STYP(ITYP),' at.',
     &         (IATDLMN(K+MY_NAT*(I-1)),K=1,ATCVAR(1,I)),
     &         ' vs all ',' Mil. In.',(ATCVAR(K+2,I),K=1,3),
     &       ' SCF', CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I)
          ENDIF
        
        ELSEIF(ITYP.EQ.15) THEN
          WRITE(6,'(A,6I5,13x,3F8.4,5x,4F8.4)') STYP(ITYP),
     *         (ATCVAR(K,I),K=1,6),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     &         (VBOUND(K,I),K=1,4)
        ELSEIF(ITYP.EQ.16) THEN
          WRITE(6,'(A,4I5,23x,3F8.4)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I)
          WRITE(6,'(15x,A,2I3,A,F8.4,A,2I3,A,F8.4)')
     *         'expH ',ATCVAR(5,I),ATCVAR(6,I),' R0_H ',CVPAR(1,I),
     *         '   expA ',ATCVAR(7,I),ATCVAR(8,I),' R0_A ',CVPAR(2,I)
          WRITE(6,'(15x,A,2I3,A,2F8.4,A,F8.4,A,I4)')
     *         'max & min L',NSHORT,NLONG,'neigh. dist.',
     *         RCW,RCH,'opt. H-A dist. ',OPTDIST,'UPD. NEIGH. ',LUPD
        ELSEIF(ITYP.EQ.19) THEN
          WRITE(6,'(A,3x,2I3,8x,A,f6.2,1x,A,f5.2,1x,A,f5.2,2x,A,4F6.2)')
     &         STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),
     *         'H:',CVPAR(1,I),
     &         'O:',CVPAR(2,I),
     &        'F_c: ',CVPAR(3,I),
     &        'L:',CVPAR(4,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I)
        ELSEIF(ITYP.EQ.27) THEN
          WRITE(6,'(A,3x,3I3,2x,6(A,f6.2,1x),2x,3F6.2)')
     &         STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),ATCVAR(11,I),
     *         'H:',CVPAR(1,I),
     &         'O:',CVPAR(2,I),
     &        'nH0: ',CVPAR(3,I),
     &        'nO0: ',CVPAR(4,I),
     &        'nL0: ',CVPAR(5,I),
     &        'L:',CVPAR(6,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I)
        ELSEIF(ITYP.EQ.20) THEN
          WRITE(6,'(A,3x,3I3,5x,A,f6.2,1x,A,f5.2,1x,A,f5.2,2x,A,4F6.2)')
     &         STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),
     *         'H:',CVPAR(1,I),
     &         'O:',CVPAR(2,I),
     &        'F_c: ',CVPAR(3,I),
     &        'L:',CVPAR(4,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I)
        ELSEIF(ITYP.EQ.28) THEN
          WRITE(6,'(A,3x,4I3,5x,A,f6.2,1x,A,f5.2,1x,A,f5.2,2x,A,4F6.2)')
     &         STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),
     &         ATCVAR(3,I),ATCVAR(10,I),
     *         'H:',CVPAR(1,I),
     &         'O:',CVPAR(2,I),
     &        'nL0: ',CVPAR(3,I),
     &        'L:',CVPAR(4,I),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I)

        ELSE
          WRITE(6,'(A,4I5,23x,3F8.4,5x,4F8.4)') STYP(ITYP),
     &         NAT_grm(ATCVAR(1,I)),NAT_grm(ATCVAR(2,I)),
     &         NAT_grm(ATCVAR(3,I)),NAT_grm(ATCVAR(4,I)),
     &         CSCL_FAC(1,I),CSCL_FAC(2,I),CSCL_FAC(3,I),
     &         (VBOUND(K,I),K=1,4)
        ENDIF
        IF(ITYP.EQ.2 .OR. ITYP.EQ.3 .OR. ITYP.EQ.5) THEN
          CALL RADDEG(VBOUND(1,I),-1)
          CALL RADDEG(VBOUND(3,I),-1)
        ENDIF
      ENDDO

C      stop 'input'
      WRITE(6,'(80("*"))')
C     ==--------------------------------------------------------------==
      if(qmmm) call mm_dim(mm_revert,status)
      RETURN
      END
C     ==================================================================
      SUBROUTINE COLVAR_STRUCTURE(TAU0,TSCR)
C     ==--------------------------------------------------------------==
C     == Determination of Collective Variable Structures              ==
C     ==--------------------------------------------------------------==
C
C     DOF_INCV   : assign 1 to each dof which partecipate to the CV
C                  dim (NODIM,NCOLVAR,12)
C                  because if the dof is used for a certain CV ICV then 
C                  it has an index among the ions used for the same CV
C     CV_IST     : at each ions dynamic step the CV value is calculated
C                  dim (NCOLVAR)
C     DET_COLVAR : at each dynmic step the derivatives of the CV 
C                  wrt to each DOF are calculated. 
C                  dim (NODIM,NCOLVAR)
C     DET_CELVAR : if TCVCELL some CV are functions of the CELL rather than R
C                  at each dynmic step the derivatives of the CV 
C                  wrt to the cell parameters are calculated
C                  dim (3,3,NCOLVAR)

C     CV_STORE   : storage of the last INTER_AV_CV istantaneous CV values
C                  in order to be able do calculate the average
C     CV_PATH    : storage of the average values which give the path in
C                  the CV space 
C
C     ==--------------------------------------------------------------==    

      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'mw.inc'
      INCLUDE 'cnst_dyn.inc'
      INCLUDE 'mm_dim.inc'
      INCLUDE 'mm_input.inc'

C     Arguments
      REAL*8    TAU0(3,NAX,NSX),TSCR(3*NAX*NSX)
C     Variables
      REAL*8    DXPAR(*)
      POINTER   (IP_DXPAR,DXPAR)
      DIMENSION DOF_INCV(NODIM,NCOLVAR,18),
     &          DET_COLVAR(NODIM,NCOLVAR),DET_CELVAR(3,3,NCOLVAR)
      INTEGER   ICV,IA,IB,IC,ID,IE,IG,ITYP,MY_NAT
      LOGICAL   STATUS

C     ==--------------------------------------------------------------==
      CALL DUM2(TAU0,TSCR)
      IF(NCOLVAR .EQ. 0) RETURN

C     get total number of atoms and set indexing.
      if(qmmm) then
        call mm_dim(mm_go_mm,status)
        MY_NAT=NAT
      else
        MY_NAT=NATm
      endif

      CALL MEMORY(IP_DOF_INCV,18*NCOLVAR*NODIM,'DOF_INCV')
      CALL AZZERO(DOF_INCV,18*NCOLVAR*NODIM)
      CALL MEMORY(IP_CV_IST,NCOLVAR*NWALK,'CV_IST')
      CALL AZZERO(CV_IST,NCOLVAR*NWALK)
      CALL MEMORY(IP_DET_COLVAR,NCOLVAR*NODIM,'DET_COLVAR')
      CALL AZZERO(DET_COLVAR,NCOLVAR*NODIM)

      IF(TCVCELL) THEN
        CALL MEMORY(IP_DET_CELVAR,NCOLVAR*9,'DET_CELVAR')
        CALL AZZERO(DET_CELVAR,NCOLVAR*9)
      ENDIF

C     Assign the DPF that take part to the defined CV
      DO ICV = 1,NCOLVAR
        ITYP = TYCVAR(ICV)
        IF(ITYP .NE. 6 .AND. ITYP .LT. 8 ) THEN
          IA = ATCVAR(1,ICV)
          IB = ATCVAR(2,ICV)
          IC = ATCVAR(3,ICV)
          ID = ATCVAR(4,ICV)

          CALL GET_DOF(IA,ICV,1,MY_NAT)
          CALL GET_DOF(IB,ICV,2,MY_NAT)
          CALL GET_DOF(IC,ICV,3,MY_NAT)
          CALL GET_DOF(ID,ICV,4,MY_NAT)
        ELSEIF(ITYP .EQ. 15) THEN
          IA = ATCVAR(1,ICV)
          IB = ATCVAR(2,ICV)
          IC = ATCVAR(3,ICV)
          ID = ATCVAR(4,ICV)
          IE = ATCVAR(5,ICV)
          IG = ATCVAR(6,ICV)

          CALL GET_DOF(IA,ICV,1,MY_NAT)
          CALL GET_DOF(IB,ICV,2,MY_NAT)
          CALL GET_DOF(IC,ICV,3,MY_NAT)
          CALL GET_DOF(ID,ICV,4,MY_NAT)
          CALL GET_DOF(IE,ICV,5,MY_NAT)
          CALL GET_DOF(IG,ICV,6,MY_NAT)

        ENDIF
      ENDDO


C     Calculate starting values of CV and derivatives wrt ions coordinates

      CALL COLVAROFR(TAU0,TSCR)
      CALL COLVARPR

      if(qmmm) call mm_dim(mm_revert,status)
      RETURN
      END

C     ==================================================================
      SUBROUTINE GET_DOF(IA,ICV,N,MY_NAT)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'cnst_dyn.inc'
C     Arguments
      INTEGER   IA,ICV,N,MY_NAT
C     Variables
      REAL*8    FF
      INTEGER   NN,JX,JY,JZ,NAA,ITYP,KX,JJ,J,IB
      DIMENSION DOF_INCV(NODIM,NCOLVAR,18)
C     ==--------------------------------------------------------------==
      IF(IA.EQ.0) RETURN
      NN=(N-1)*3
      IF(IA.LT.0) THEN
        WRITE(6,'(1X,A,I5,A,I5,A,I1)') 'GET_DOF! IA=',IA,
     *                         ' COL.VAR. # =',ICV,' N=',N
        CALL STOPGM('GET_DOF','IA IS LESS THAN ZERO')
      ELSEIF(IA.LE.MY_NAT) THEN
        JX=LSKPTR(1,IA)
        JY=LSKPTR(2,IA)
        JZ=LSKPTR(3,IA)
        IF(JX.NE.0) DOF_INCV(JX,ICV,NN+1)=1.0D0
        IF(JY.NE.0) DOF_INCV(JY,ICV,NN+2)=1.0D0
        IF(JZ.NE.0) DOF_INCV(JZ,ICV,NN+3)=1.0D0
      ELSEIF(IA.LE.MY_NAT+NDAT) THEN
C        WRITE(6,'(1X,A,I5,A)') 'GET_DOF! IA=',IA,
C     &                'COL. VAR. WITH DUMMY ATOMS NOT IMPLEMENTED YET'
        NAA=IA-MY_NAT
        ITYP=LISTDA(NAA,1)
        IF(ITYP.EQ.1) THEN
          RETURN
        ELSEIF(ITYP.EQ.2) THEN
          KX=LISTDA(NAA,2)
          JJ=LISTD2(1,KX)
          FF=1.0D0/DBLE(JJ)
          DO J=1,JJ
            IB=LISTD2(J+1,KX)
            JX=LSKPTR(1,IB)
            JY=LSKPTR(2,IB)
            JZ=LSKPTR(3,IB)
            IF(JX.NE.0) DOF_INCV(JX,ICV,NN+1)=FF
            IF(JY.NE.0) DOF_INCV(JY,ICV,NN+2)=FF
            IF(JZ.NE.0) DOF_INCV(JZ,ICV,NN+3)=FF
          ENDDO
        ENDIF
      ELSE
        WRITE(6,'(1X,A,I5,A,I5,A,I1)') 'GET_DOF! IA=',IA,
     &                         ' COL.VAR. # =',ICV,' N=',N
        CALL STOPGM('GET_DOF','INCORRECT VALUE FOR IA')
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE COLVAROFR(TAU0,TSCR)
C     ==--------------------------------------------------------------==
C     == Calculates istantaneous values of the collective variables   ==
C     == and the derivatives wrt the ions coordinates                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE

      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'cnst_dyn.inc'

C     Arguments
      REAL*8      TAU0(3,NAX,NSX),TSCR(3,NAX*NSX)

C     Variables
      INTEGER     ISUB,ICV,IAT(15),ITYP,KMAX,K,IDOF,IPATMB,IPATMA
      INTEGER     numO,numH,numL
      REAL*8      DIFF,X1(3),X2(3),X3(3),X4(3),X5(3),X6(3),DX(18),
     &            C_KM,C_RCM,SIGN,AA,BB,R0_SHIFT,C_RC1,C_RC2
      DIMENSION   DET_COLVAR(NODIM,NCOLVAR),DET_CELVAR(3,3,NCOLVAR),
     &            DOF_INCV(NODIM,NCOLVAR,18),IATDLMN(NAT,*),
     &            IQATDLMN(NAT,*)
      REAL*8      RC_HY(3),LAMBDA
      INTEGER     NEXP(3),MEXP(3)
C     ==--------------------------------------------------------------==
      CALL TISET(' COLVAROFR',ISUB)

      CALL DUM2(TAU0,TSCR)
      CALL AZZERO(DET_COLVAR,NCOLVAR*NODIM)
      IF(TCVCELL) THEN
        CALL AZZERO(DET_CELVAR,NCOLVAR*9)
      ENDIF

      IPATMB=0
      IPATMA=0
      DO ICV=1,NCOLVAR
        DIFF = 0.0D0
        ITYP=TYCVAR(ICV)
        IAT(1)=ATCVAR(1,ICV)
        IAT(2)=ATCVAR(2,ICV)
        IAT(3)=ATCVAR(3,ICV)
        IAT(4)=ATCVAR(4,ICV)
        IAT(5)=ATCVAR(5,ICV)
        IAT(6)=ATCVAR(6,ICV)
        IAT(7)=ATCVAR(7,ICV)
        IAT(8)=ATCVAR(8,ICV)
        IAT(9)=ATCVAR(9,ICV)
        IAT(10)=ATCVAR(10,ICV)
        IAT(11)=ATCVAR(11,ICV)
        IF(ITYP.EQ.1) THEN
C..stretch
          CALL FILLC(IAT(1),TSCR,X1)
          CALL FILLC(IAT(2),TSCR,X2)
          CALL FUNCR(DIFF,CV_IST(ICV),0.0D0,X1,X2)
          CALL DIFFR(DX,X1,X2)
          KMAX=6
C..angle
        ELSEIF(ITYP.EQ.2) THEN
          CALL FILLC(IAT(1),TSCR,X1)
          CALL FILLC(IAT(2),TSCR,X2)
          CALL FILLC(IAT(3),TSCR,X3)
          CALL FUNCT(DIFF,CV_IST(ICV),0.0D0,X1,X2,X3)
          CALL DIFFT(DX,X1,X2,X3)
          KMAX=9
        ELSEIF(ITYP.EQ.3) THEN
C..dihedral angle
          CALL FILLC(IAT(1),TSCR,X1)
          CALL FILLC(IAT(2),TSCR,X2)
          CALL FILLC(IAT(3),TSCR,X3)
          CALL FILLC(IAT(4),TSCR,X4)
          CALL FUNCO(DIFF,CV_IST(ICV),0.0D0,X1,X2,X3,X4,sign)
          CALL DIFFO(DX,X1,X2,X3,X4,sign)
          KMAX=12
        ELSEIF(ITYP.EQ.4) THEN
C..distance
          CALL FILLC(IAT(1),TSCR,X1)
          CALL FILLC(IAT(2),TSCR,X2)
          CALL FUNCD(DIFF,CV_IST(ICV),0.0D0,X1,X2)
          CALL DIFFD(DX,X1,X2)
          KMAX=6
!..displacement in a coordinate
        ELSEIF(ITYP.EQ.30) THEN
          CALL FILLC(IAT(1),TSCR,X1)
          CALL FILLC(IAT(2),TSCR,X2)
          DO K=1,3
            IF(K.NE.IAT(3))THEN
              X1(K)=0.0D0
              X2(K)=0.0D0
            END IF
          END DO
          CALL FUNCD(DIFF,CV_IST(ICV),0.0D0,X1,X2)
          CALL DIFFD(DX,X1,X2)
          KMAX=6
        ELSEIF(ITYP.EQ.5) THEN
C..out of plane
          CALL FILLC(IAT(1),TSCR,X1)
          CALL FILLC(IAT(2),TSCR,X2)
          CALL FILLC(IAT(3),TSCR,X3)
          CALL FILLC(IAT(4),TSCR,X4)
          CALL FUNCP_mia(DIFF,CV_IST(ICV),0.0D0,X1,X2,X3,X4,DX)
C          CALL DIFFP(DX,X1,X2,X3,X4)
          KMAX=12
        ELSEIF(ITYP.EQ.6) THEN
C..coordination number
          C_KM     = CVPAR(1,ICV) 
          C_RCM    = CVPAR(2,ICV)
          CALL COORNUM(IAT(1),C_KM,C_RCM,TSCR,NAT,DET_COLVAR(1,ICV),
     *                 LSKPTR,DIFF,CV_IST(ICV),0.0D0)
          KMAX=0
        ELSEIF(ITYP.EQ.7) THEN
C..difference between distances- 3 atoms
          CALL FILLC(IAT(1),TSCR,X1)
          CALL FILLC(IAT(2),TSCR,X2)
          CALL FILLC(IAT(3),TSCR,X3)
          CALL FUNCDD(DIFF,CV_IST(ICV),0.0D0,X1,X2,X3)
          CALL DIFFDD(DX,X1,X2,X3)
          KMAX=9
        ELSEIF(ITYP.EQ.8) THEN
C..species dependent coordination number with Fermi function
          C_KM     = CVPAR(1,ICV) 
          C_RCM    = CVPAR(2,ICV)
          CALL COORNUMSP(IAT(1),IAT(2),C_KM,C_RCM,TSCR,
     *             DET_COLVAR(1,ICV),LSKPTR,DIFF,CV_IST(ICV),0.0D0)
          KMAX=0
        ELSEIF(ITYP.EQ.29) THEN
C..  Total Coordination between a specified group
          C_RCM=CVPAR(1,ICV) !kappa
          CALL COORNUMGRP(ATCVAR(1,ICV),MAX_NATCNGB,
     &           NATCNGB(IPATMA+1),IATCNGA(IPATMA+1),
     &           IATCNGB(IPATMB+1),
     &           C_RCM,RCCNGA(IPATMA+1),TSCR,DET_COLVAR(1,ICV),LSKPTR,
     &           DIFF,CV_IST(ICV),0.0D0)
          IPATMA=IPATMA+ATCVAR(1,ICV)
          IPATMB=IPATMB+ATCVAR(1,ICV)*MAX_NATCNGB
          KMAX=0
        ELSEIF(ITYP.EQ.9) THEN
C..species dependent coordination number with rational function
          C_RCM    = CVPAR(1,ICV)
          R0_SHIFT   = CVPAR(2,ICV)
          IF(SPECINDEX(ICV).EQ.1) THEN
            CALL COORN_RF(IAT(1),IAT(2),IAT(3),IAT(4),C_RCM,R0_SHIFT,
     *      TSCR,DET_COLVAR(1,ICV),DIFF,CV_IST(ICV),
     &      0.0D0,1,IATDLMN)
          ELSE IF(SPECINDEX(ICV) .LT. 0) THEN
            CALL  COORN_RF_Q(IAT,C_RCM,R0_SHIFT,TSCR,
     &            DET_COLVAR(1,ICV),DIFF,CV_IST(ICV),
     &            0.0D0,SPECINDEX(ICV))
          ELSE
            CALL COORN_RF(IAT(1),IAT(2),IAT(3),IAT(4),C_RCM,R0_SHIFT,
     *                TSCR,DET_COLVAR(1,ICV),DIFF,CV_IST(ICV),
     *                   0.0D0,0,IATDLMN(1,ICV))
          ENDIF
          KMAX=0
        ELSEIF(ITYP.EQ.10) THEN
C..bond switch with rational function
          C_RCM    = CVPAR(1,ICV)
          CALL BNDSWITCH(IAT(1),IAT(2),IAT(3),IAT(4),C_RCM,TSCR,
     *                 DET_COLVAR(1,ICV),DIFF,CV_IST(ICV),0.0D0)
          KMAX=0
        ELSEIF(ITYP.EQ.11) THEN
C..total species dependent coordination number with rational function
          C_RCM    = CVPAR(1,ICV)
          R0_SHIFT   = CVPAR(2,ICV)
          IF(SPECINDEX(ICV).EQ.1) THEN
            CALL COORNTOT_RF(IAT(1),IAT(2),IAT(3),IAT(4),C_RCM,R0_SHIFT,
     *           TSCR,DET_COLVAR(1,ICV),DIFF,CV_IST(ICV),
     &           0.0D0,1,IATDLMN)
          ELSE IF(SPECINDEX(ICV) .LT. 0) THEN
            CALL  COORNTOT_RF_Q(IAT,C_RCM,R0_SHIFT,TSCR,
     &            DET_COLVAR(1,ICV),DIFF,CV_IST(ICV),
     &            0.0D0,SPECINDEX(ICV))
          ELSE
            CALL COORNTOT_RF(IAT(1),IAT(2),IAT(3),IAT(4),C_RCM,R0_SHIFT,
     *           TSCR,DET_COLVAR(1,ICV),DIFF,CV_IST(ICV),
     *           0.0D0,0,IATDLMN(1,ICV))
          ENDIF
          KMAX=0
       ELSEIF(ITYP.EQ.35) THEN
C..total species dependent difference of coordination numbers with rational function
          C_RCM    = CVPAR(1,ICV)
          R0_SHIFT   = CVPAR(2,ICV)
          CALL DIFCNTOT_RF(IAT(1),IAT(2),IAT(3),IAT(4),IAT(5),C_RCM,
     *           R0_SHIFT,TSCR,DET_COLVAR(1,ICV),DIFF,CV_IST(ICV),
     &           0.0D0,1,IATDLMN)
          KMAX=0
       ELSEIF(ITYP.EQ.36) THEN
C..total species dependent absolute value of difference of coordination numbers with rational function
          C_RCM    = CVPAR(1,ICV)
          R0_SHIFT   = CVPAR(2,ICV)
          CALL ABS_DIFCNTOT_RF(IAT(1),IAT(2),IAT(3),IAT(4),IAT(5),C_RCM,
     *           R0_SHIFT,TSCR,DET_COLVAR(1,ICV),DIFF,CV_IST(ICV),
     &           0.0D0,1,IATDLMN)
          KMAX=0
        ELSEIF(ITYP.EQ.25) THEN
C...coordination numbers calculated between sequences of atoms
          C_RCM    = CVPAR(1,ICV)
          R0_SHIFT   = CVPAR(2,ICV)
          CALL COORNSEQ_RF(IAT,C_RCM,R0_SHIFT,TSCR,
     &         DET_COLVAR(1,ICV),
     &         DIFF,CV_IST(ICV),0.0D0,SPECINDEX(ICV))
          KMAX = 0
        ELSEIF(ITYP.EQ.12) THEN
C..RMSD wrt 2 configurations A and B
          CALL RMSD_AB(IAT(1),ATCVAR(2,ICV),ATCVAR(IAT(1)+2,ICV),
     *                 TSCR,NAT,DET_COLVAR(1,ICV),
     *                 LSKPTR,DIFF,CV_IST(ICV),0.0D0)
          KMAX=0
         ELSEIF(ITYP.EQ.26) THEN
!..RMSD wrt initial configuration
          CALL RMSD_SEQ_A(IAT,MAXRMSDAT,TSCR,DET_COLVAR(1,ICV),
     *               DIFF,CV_IST(ICV),0.0D0,SPECINDEX(ICV))
          KMAX=0
        ELSEIF(ITYP.EQ.13) THEN
C..Displacement in the direction (lmn) of a species wrt configuration A 

          IF(SPECINDEX(ICV).EQ.1) THEN
            CALL DISP_lnm(IAT(1),IAT(2),ATCVAR(3,ICV),
     *                    ATCVAR(3+IAT(1)+IAT(2),ICV),
     *                    IATDLMN,TSCR,NAT,DET_COLVAR(1,ICV),LSKPTR,
     *                    CV_IST(ICV),SPECINDEX(ICV))
            KMAX=0
          ELSE

            CALL DISP_lnm(IAT(1),IAT(2),ATCVAR,ATCVAR(3,ICV),
     *                 IATDLMN(1,ICV),TSCR,NAT,DET_COLVAR(1,ICV),LSKPTR,
     *                 CV_IST(ICV),SPECINDEX(ICV))
            KMAX=0
          ENDIF
        ELSEIF(ITYP.EQ.15) THEN
C..Angle between 2 planes, each defined by 3 pnts  
          CALL FILLC(IAT(1),TSCR,X1)
          CALL FILLC(IAT(2),TSCR,X2)
          CALL FILLC(IAT(3),TSCR,X3)
          CALL FILLC(IAT(4),TSCR,X4)
          CALL FILLC(IAT(5),TSCR,X5)
          CALL FILLC(IAT(6),TSCR,X6)
          CALL APLANE(DIFF,CV_IST(ICV),0.0D0,X1,X2,X3,X4,X5,X6,DX)
          KMAX=18
        ELSEIF(ITYP.EQ.16) THEN
C..Hydrogen Bond Chain

          CALL CHAIN_DR(IAT(1),IAT(2),IAT(3),IAT(4),
     &                  IAT(5),IAT(6),CVPAR(1,ICV),
     &                  IAT(7),IAT(8),CVPAR(2,ICV), 
     &                  IATDLMN(1,ICV),TSCR,DET_COLVAR(1,ICV),
     *                  LSKPTR,CV_IST(ICV))
        ELSEIF(ITYP.EQ.17) THEN
C..Difference in Coordination numbers (rational function)
          C_RCM    = CVPAR(1,ICV)
          R0_SHIFT   = CVPAR(2,ICV)
          IF(SPECINDEX(ICV).EQ.1) THEN
            CALL COORN_RF(IAT(2),IAT(3),IAT(4),IAT(5),C_RCM,R0_SHIFT,
     *      TSCR,DET_COLVAR(1,ICV),DIFF,BB,0.0D0,1,IATDLMN)
C          write(6,*) DET_COLVAR
            CALL DSCAL(NODIM,-1.0D0,DET_COLVAR(1,ICV),1)
            CALL COORN_RF(IAT(1),IAT(3),IAT(4),IAT(5),C_RCM,R0_SHIFT,
     *      TSCR,DET_COLVAR(1,ICV),DIFF,AA,0.0D0,1,IATDLMN)
C          write(6,*) DET_COLVAR
            CV_IST(ICV) = AA - BB
          ELSE
            CALL COORN_RF(IAT(2),IAT(3),IAT(4),IAT(5),C_RCM,R0_SHIFT,
     *                   TSCR,DET_COLVAR(1,ICV),DIFF,BB,
     *                   0.0D0,0,IATDLMN(1,ICV))
            CALL DSCAL(NODIM,-1.0D0,DET_COLVAR(1,ICV),1)
            CALL COORN_RF(IAT(1),IAT(3),IAT(4),IAT(5),C_RCM,R0_SHIFT,
     *                   TSCR,DET_COLVAR(1,ICV),DIFF,AA,
     *                   0.0D0,0,IATDLMN(1,ICV))
            CV_IST(ICV) = AA - BB
          ENDIF
          KMAX=0
        ELSEIF(ITYP.EQ.18) THEN
C.. Coordination numbers of second neighbors (rational function)
          C_RC1    = CVPAR(1,ICV)
          C_RC2    = CVPAR(2,ICV)
          CALL CTOT_CHAIN(IAT(1),IAT(2),IAT(3),IAT(4),IAT(5),
     &                    C_RC1,C_RC2,TSCR,DET_COLVAR(1,ICV),LSKPTR,
     &                    DIFF,CV_IST(ICV),0.0D0)
          KMAX=0
         ELSEIF(ITYP.EQ.31) THEN
C...DIPOLERHO
            CALL CDIPOLERHO(IAT(1),TSCR,DIFF,CV_IST(ICV),
     &           DET_COLVAR(1,ICV),IAT(2),IATDLMN(1,ICV),
     &           IQATDLMN(1,ICV),0.0D0)
            KMAX = 0
         ELSEIF(ITYP.EQ.32) THEN                                
C...  DIPOLETHETA
            CALL CDIPOLETHETA(IAT(1),TSCR,DIFF,CV_IST(ICV),
     &           DET_COLVAR(1,ICV),IAT(2),IATDLMN(1,ICV),
     &           IQATDLMN(1,ICV),0.0D0)
            KMAX = 0
C_DIPOLEPHI
         ELSEIF(ITYP.EQ.33) THEN 
            CALL CDIPOLEPHI(IAT(1),TSCR,DIFF,CV_IST(ICV),                
     &           DET_COLVAR(1,ICV),IAT(2),IATDLMN(1,ICV),
     &           IQATDLMN(1,ICV),0.0D0)
            KMAX = 0
            
         ELSEIF(ITYP.EQ.34) THEN
C     _VCOORS
            C_KM  = CVPAR(1,ICV) 
            C_RCM = CVPAR(2,ICV)
            X1(1) = CVPAR(3,ICV) 
            X1(2) = CVPAR(4,ICV)
            X1(3) = CVPAR(5,ICV)
            CALL VCOORS(IAT(1),X1,C_KM,C_RCM,TSCR,
     &           DET_COLVAR(1,ICV),LSKPTR,DIFF,CV_IST(ICV),0.0D0)
            KMAX=0                                                   
        ELSEIF(ITYP.EQ.19) THEN
C.. Presence of Hydronium

          NEXP(1) = IAT(3)
          MEXP(1) = IAT(4)
          RC_HY(1)   = CVPAR(1,ICV)
          NEXP(2) = IAT(5)
          MEXP(2) = IAT(6)
          RC_HY(2)   = CVPAR(2,ICV)
          NEXP(3) = IAT(7)
          MEXP(3) = IAT(8)
          RC_HY(3)   = CVPAR(3,ICV)
          LAMBDA  = CVPAR(4,ICV)
          CALL HYD_PRESENCE(IAT(1),IAT(2),NEXP,MEXP,RC_HY,LAMBDA,
     &              TSCR,DET_COLVAR(1,ICV),LSKPTR,CV_IST(ICV))
          KMAX=0
        ELSEIF(ITYP.EQ.27) THEN
!.. Presence of Hydronium B (with cutoff)

          numO = NA(IAT(1))
          numH = NA(IAT(2))
          numL = IAT(11)
          CALL HYD_CUTOFF(IAT,CVPAR(1,ICV),numO,numH,numL,
     &         IATDLMN(1,ICV),
     &         TSCR,DET_COLVAR(1,ICV),LSKPTR,CV_IST(ICV))
          KMAX=0

        ELSEIF(ITYP.EQ.20) THEN
C.. Presence of Hydronium, distance between ion and hydronium
          NEXP(1) = IAT(4)
          MEXP(1) = IAT(5)
          RC_HY(1)   = CVPAR(1,ICV)
          NEXP(2) = IAT(6)
          MEXP(2) = IAT(7)
          RC_HY(2)   = CVPAR(2,ICV)
          NEXP(3) = IAT(8)
          MEXP(3) = IAT(9)
          RC_HY(3)   = CVPAR(3,ICV)
          LAMBDA  = CVPAR(4,ICV)
          CALL HYD_ION_DISTANCE(IAT(1),IAT(2),IAT(3),NEXP,MEXP,RC_HY,
     &             LAMBDA,TSCR,DET_COLVAR(1,ICV),LSKPTR,CV_IST(ICV))
          KMAX=0

        ELSEIF(ITYP.EQ.28) THEN
!.. Distance Hydronium - ion with cutoff

          numO = NA(IAT(1))
          numH = NA(IAT(2))
          numL = IAT(10)
          CALL CUT_HYD_ION_DISTANCE(IAT,CVPAR(1,ICV),numO,numH,numL,
     &         IATDLMN(1,ICV),
     &         TSCR,DET_COLVAR(1,ICV),LSKPTR,CV_IST(ICV))
          KMAX=0

        ELSEIF(ITYP.EQ.21) THEN
C.. Localization of the spin density 
C..  independent on the ionic positions

          CALL FILLC(IAT(1),TSCR,X1)
C     RCC0 is the position of the reference atom 
          RCC0(1,ICV)=X1(1)
          RCC0(2,ICV)=X1(2)
          RCC0(3,ICV)=X1(3)
C     RCC  is the position of the center of the polarized density
          X2(1)= RCC(1)
          X2(2)= RCC(2)
          X2(3)= RCC(3)

          CALL FUNCL(DIFF,CV_IST(ICV),0.0D0,X1,X2,DX)

          KMAX=6
        ELSEIF(ITYP.EQ.22) THEN
C..  Volume of the Cell (only if TPRCP)
          CALL VOLUME_CV(CV_IST(ICV),DET_CELVAR(1,1,ICV))
          KMAX=0

        ELSEIF(ITYP.EQ.23) THEN
C..  Side of the Cell (only if TPRCP)
          CALL SIDE_CV(CV_IST(ICV),IAT(1),DET_CELVAR(1,1,ICV))
          KMAX=0

        ELSEIF(ITYP.EQ.24) THEN
C..  ANGLE of the Cell (only if TPRCP)
          CALL ANGLE_CV(CV_IST(ICV),IAT(1),DET_CELVAR(1,1,ICV))
          KMAX=0

        ELSE
          WRITE(6,*) ' COLVAROFR! I=',ICV,' TYPE=',ITYP
          CALL STOPGM('COLVAROFR','UNKNOWN TYPE OF COLLECTIVE VARIABLE')
        END IF

        DO K=1,KMAX
          DO IDOF=1,NODIM
           DET_COLVAR(IDOF,ICV)=DET_COLVAR(IDOF,ICV)+
     &                          DX(K)*DOF_INCV(IDOF,ICV,K)
          ENDDO
        ENDDO

      ENDDO  ! NCOLVAR
      CALL TIHALT(' COLVAROFR',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE COLVARPR
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst_dyn.inc'
      INCLUDE 'ions.inc'

C     Variables
      CHARACTER STYP(36)*15,LINEFORM*100,CHNUM*10,CHNUM2*10
      DATA      STYP /
     &          '        STRETCH','           BEND','        TORSION',
     &          '       DISTANCE','           OUTP','        COORDIN',
     &          '      DIFFEREN.','         COORSP','        COOR_RF',
     &          '      BN.SWITCH','       TOT C_RF','        RMSD_AB',
     &          '         DISPL1','        COORDIS','         PLNANG',
     &          '        HBONDCH','        DIFCOOR','      COO_CHAIN',
     &          '      HYDRONIUM','        DIS_HYD','           SPIN',
     &          '         VOLVAR','       CELLSIDE','      CELLANGLE',
     &          '       COOR_SEQ','       RMSD_SEQ','        HYD_CUT',
     &          '      D_HYD_CUT','      COORGROUP','        DISAXIS',
     &          '      DIPOLERHO','      DIPOLETHA','      DIPOLEPHI',
     &          '         VCOORS','  DIFF TOT C_RF','|DIFF TOT C_RF|'/
      INTEGER   I,ITYP,NUMSPEC,NUMSPEC2,NUMTOT,K,IAA,IEE,IA,IE
      DIMENSION IATDLMN(*),IQATDLMN(*)
      REAL*8    CVAL
C     ==--------------------------------------------------------------==
C     ==  PRINT SOME INFO                                             ==
C     ==--------------------------------------------------------------==
 
      WRITE(6,'(A)') 
     &         ' <<<<<<<<<   COLLECTIVE VARIABLE INFO   >>>>>>>>>>>'
      WRITE(6,'(4x,A,6x,A,5x,A,20x,A)') 'TYPE','ATOMS AND SPECIES',
     *                 'PARAMETERS','VALUE'
      DO I=1,NCOLVAR
        ITYP=TYCVAR(I)
        CVAL = CV_IST(I)
        IF(ITYP.EQ.2.OR.ITYP.EQ.3.OR.ITYP.EQ.5) THEN
          CALL RADDEG(CVAL,1)
        ENDIF
        IF(ITYP.EQ.6) THEN
          WRITE(6,'(A,I5,20x,2F8.4,14x,F8.4)') STYP(ITYP),ATCVAR(1,I),
     *         CVPAR(1,I),CVPAR(2,I),
     &         CVAL
        ELSEIF(ITYP.EQ.30) THEN
          WRITE(6,'(A,3I5,20x,2F8.4,14x,F8.4)') STYP(ITYP),ATCVAR(1,I),
     *         ATCVAR(2,I), ATCVAR(3,I),CVPAR(1,I),CVPAR(2,I),CVAL
        ELSEIF(ITYP.EQ.8) THEN
          WRITE(6,'(A,2I5,15x,2F8.4,16x,F8.4)') STYP(ITYP),ATCVAR(1,I),
     *         ATCVAR(2,I),CVPAR(1,I),CVPAR(2,I),
     &         CVAL
        ELSEIF(ITYP.EQ.29) THEN
          WRITE(6,'(A,I5,2F8.4)') STYP(ITYP),ATCVAR(1,I),CVPAR(1,I),CVAL
        ELSEIF(ITYP.GE.9 .AND. ITYP.LE. 11) THEN
          IF(CVPAR(2,I) .NE. 0.0D0) THEN
            WRITE(6,'(A,4x,2I5,2x,A,2I4,2f6.3,17x,F8.4)') 
     &         STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),
     *         '2 SHELL',ATCVAR(3,I),ATCVAR(4,I),
     &         CVPAR(1,I),CVPAR(2,I),CVAL
          ELSE
            WRITE(6,'(A,4x,2I5,9x,2I4,f6.3,17x,F8.4)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &         CVPAR(1,I),CVAL
          ENDIF
        ELSEIF(ITYP.EQ.17) THEN
          WRITE(6,'(A,4x,3I4,8x,2I3,f6.3,17x,F8.4)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &         ATCVAR(5,I),CVPAR(1,I),CVAL
        ELSEIF(ITYP.EQ.18) THEN
          WRITE(6,'(A,4x,3I4,4x,2I3,2f6.3,17x,F8.4)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &         ATCVAR(5,I),CVPAR(1,I),CVAL
        ELSEIF(ITYP.EQ.12) THEN
          NUMSPEC  = ATCVAR(1,I)

          WRITE(CHNUM,'(I5)') NUMSPEC
          CALL XSTRING(CHNUM,IA,IE)

          LINEFORM ='(A,2X,I4,A9,'//CHNUM(IA:IE)//'I4,18x,F8.4)'
          WRITE(6,LINEFORM) STYP(ITYP),ATCVAR(1,I),' Species ',
     &    (ATCVAR(K+1,I),K=1,NUMSPEC),CVAL
        ELSEIF(ITYP.EQ.13 .AND. SPECINDEX(I) .EQ. 1) THEN
          NUMSPEC = ATCVAR(1,I)
          NUMSPEC2 = ATCVAR(2,I)
          NUMTOT  = NUMSPEC + NUMSPEC2
          WRITE(CHNUM,'(I5)') NUMSPEC
          CALL XSTRING(CHNUM,IA,IE)
          IF(NUMSPEC2 .NE. 0) THEN
            WRITE(CHNUM2,'(I5)') NUMSPEC2
            CALL XSTRING(CHNUM2,IAA,IEE)
            LINEFORM ='(A,1X,A11,'//CHNUM(IA:IE)//'I4,A11,'
     &      //CHNUM2(IAA:IEE)//'I4,A11,3I3,f12.6)'
            WRITE(6,LINEFORM) STYP(ITYP),
     &      ' Species 1 ',(ATCVAR(K+2,I),K=1,NUMSPEC),
     &      ' Species 2 ',(ATCVAR(K+2+NUMSPEC,I),K=1,NUMSPEC2),
     &      ' MillerInd:',(ATCVAR(K+2+NUMTOT,I),K=1,3),
     &      CVAL
          ELSE
            LINEFORM ='(A,1X,A,'//CHNUM(IA:IE)//'I4,A10,3I3,f12.6)'
            WRITE(6,LINEFORM) STYP(ITYP),
     &      ' Species 1 ',(ATCVAR(K+2,I),K=1,NUMSPEC),
     &      ' Mil.Ind.',(ATCVAR(K+2+NUMTOT,I),K=1,3),
     &      CVAL
          ENDIF
        ELSEIF(ITYP.EQ.13 .AND. SPECINDEX(I) .NE. 1) THEN
          WRITE(CHNUM,'(I5)') ATCVAR(1,I)
          CALL XSTRING(CHNUM,IA,IE)
          NUMSPEC2 = ATCVAR(2,I)
          IF(NUMSPEC2 .NE. 0) THEN
            WRITE(CHNUM2,'(I5)') ATCVAR(2,I)
            CALL XSTRING(CHNUM2,IAA,IEE)
            LINEFORM ='(A9,2X,A5,'//CHNUM(IA:IE)//'I4,A8,'
     &                //CHNUM2(IAA:IEE)//'I4,A10,3I3,16X,f12.6)'
            WRITE(6,LINEFORM) STYP(ITYP),' at.',
     &         (IATDLMN(K+NAT*(I-1)),K=1,ATCVAR(1,I)),
     &         ' vs at.',(IATDLMN(ATCVAR(1,I)+K+NAT*(I-1)),
     &         K=1,ATCVAR(2,I)),
     &      ' Mil. In.',(ATCVAR(K+2,I),K=1,3), CVAL
          ELSE
            LINEFORM ='(A,2X,A5,'//CHNUM(IA:IE)//
     &                'I4,A8,A10,3I3,16X,f12.6)'
            WRITE(6,LINEFORM) STYP(ITYP),' at.',
     &         (IATDLMN(K+NAT*(I-1)),K=1,ATCVAR(1,I)),
     &         ' vs all ',' Mil. In.',(ATCVAR(K+2,I),K=1,3),
     &       CVAL
          ENDIF
        
        ELSEIF(ITYP.EQ.15) THEN
          WRITE(6,'(A,6I5,20x,F10.5)') STYP(ITYP),
     *         (ATCVAR(K,I),K=1,6),
     &         CVAL


        ELSEIF(ITYP.EQ.16) THEN
          WRITE(6,'(A,4I5,30x,F8.4)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &         CVAL
        ELSEIF(ITYP.EQ.19) THEN
          WRITE(6,'(A,2I4,3x,2I3,f6.2,2I3,f6.2,2I3,2f6.2,F8.4)')
     &         STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),
     *         ATCVAR(3,I),ATCVAR(4,I),CVPAR(1,I),
     &         ATCVAR(5,I),ATCVAR(6,I),CVPAR(2,I),
     &         ATCVAR(7,I),ATCVAR(8,I),CVPAR(3,I),
     &         CVPAR(4,I),CVAL
        ELSEIF(ITYP.EQ.27) THEN
        WRITE(6,'(A,3I4,3x,2I3,f6.2,2I3,f6.2,2I3,2f6.2,'//
     &        '2I3,2f6.2,F8.4)')
     &         STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),ATCVAR(11,I),
     *         ATCVAR(3,I),ATCVAR(4,I),CVPAR(1,I),
     &         ATCVAR(5,I),ATCVAR(6,I),CVPAR(2,I),
     &         ATCVAR(7,I),ATCVAR(8,I),CVPAR(3,I),
     &         CVPAR(4,I),ATCVAR(9,I),ATCVAR(10,I),
     &         CVPAR(5,I),CVPAR(6,I),CVAL
        ELSEIF(ITYP.EQ.28) THEN
        WRITE(6,'(A,3I4,3x,2I3,f6.2,2I3,f6.2,2I3,f6.2,'//
     &        'I3,f6.2,F8.4)')
     &         STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),
     *         ATCVAR(4,I),ATCVAR(5,I),CVPAR(1,I),
     &         ATCVAR(6,I),ATCVAR(7,I),CVPAR(2,I),
     &         ATCVAR(8,I),ATCVAR(9,I),CVPAR(3,I),
     &         ATCVAR(10,I),CVPAR(4,I),CVAL
        ELSEIF(ITYP.EQ.20) THEN
          WRITE(6,'(A,3I4,1x,2I3,f6.2,2I3,f6.2,2I3,2f6.2,F8.4)')
     &         STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),
     &         ATCVAR(4,I),ATCVAR(5,I),CVPAR(1,I),
     &         ATCVAR(6,I),ATCVAR(7,I),CVPAR(2,I),
     &         ATCVAR(8,I),ATCVAR(9,I),CVPAR(3,I),
     &         CVPAR(4,I),CVAL
        ELSEIF(ITYP.EQ.28) THEN                                        
           WRITE(6,'(A,I5,25x,I4,25x,f6.4)')
     &          STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),CVAL
           
           
        ELSEIF(ITYP.EQ.29) THEN                             
           WRITE(6,'(A,I5,25x,I4,25x,f6.4)')
     &          STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),CVAL          
           
        ELSEIF(ITYP.EQ.30) THEN                           
           WRITE(6,'(A,I5,25x,I4,25x,f6.4)')
     &          STYP(ITYP),ATCVAR(1,I),ATCVAR(2,I),CVAL         
     
        ELSEIF(ITYP.EQ.31) THEN                           
           WRITE(6,'(A,I5,22x,f6.4,2x,f6.4,18x,f6.4)')                              
     &          STYP(ITYP),ATCVAR(1,I),CVPAR(1,I),CVPAR(2,I),
     &          CVAL
        ELSEIF(ITYP.EQ.35) THEN                           
           WRITE(6,'(A,5I6,25x,F10.5)') STYP(ITYP),ATCVAR(1,I),
     &          ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &          ATCVAR(5,I),CVAL
        ELSEIF(ITYP.EQ.36) THEN                           
           WRITE(6,'(A,5I6,25x,F10.5)') STYP(ITYP),ATCVAR(1,I),
     &          ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &          ATCVAR(5,I),CVAL
        ELSE
           WRITE(6,'(A,4I5,25x,F10.5)') STYP(ITYP),ATCVAR(1,I),
     &          ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &          CVAL
        ENDIF
      ENDDO
      WRITE(6,*)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PARAM_META(LINE,TAD_SCF,VBOUND,IBOUND,
     &           CV_DYN_0,INITIAL_VALUE,
     &           NCOLVAR,CSCL_FAC,KHARM,CV_MASS,LEXTLAGRANGE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      CHARACTER  LINE*80
      INTEGER NCOLVAR,IBOUND(*)
      REAL*8 CSCL_FAC(3,*),KHARM(*),CV_MASS(*),VBOUND(4,*),CV_DYN_0(*)
      LOGICAL LEXTLAGRANGE,TAD_SCF(*),INITIAL_VALUE(*)
C
      INTEGER II,IJ,IA,IE
      LOGICAL ERREAD
C
      II=INDEX(LINE,'SCA')
      IJ=INDEX(LINE,'SCF')
      IF(II.NE.0) THEN
        IA = II+3
        TAD_SCF(NCOLVAR) = .TRUE.
        CALL READSR(LINE,IA,IE,CSCL_FAC(1,NCOLVAR),   ERREAD)
        IA = IE
        CALL READSR(LINE,IA,IE,CSCL_FAC(2,NCOLVAR),   ERREAD)
        IA = IE
        CALL READSR(LINE,IA,IE,CSCL_FAC(3,NCOLVAR),   ERREAD)
        IA = IE
        IF(ERREAD) GOTO 22
      ELSEIF(IJ.NE.0) THEN
        IA = IJ+3
        TAD_SCF(NCOLVAR) = .FALSE.
        CALL READSR(LINE,IA,IE,CSCL_FAC(1,NCOLVAR),   ERREAD)
        IF(ERREAD) GOTO 22
      ENDIF
      II=INDEX(LINE,'KCV')
      IF(II .NE. 0) THEN
        LEXTLAGRANGE=.TRUE.
        IA = II+3
        CALL READSR(LINE,IA,IE,KHARM(NCOLVAR),   ERREAD)
        IF(ERREAD) GOTO 22
      ENDIF
      IJ=INDEX(LINE,'MCV')
      IF(IJ .NE. 0) THEN
        LEXTLAGRANGE=.TRUE.
        IA = IJ+3
        CALL READSR(LINE,IA,IE,CV_MASS(NCOLVAR),   ERREAD)
        IF(ERREAD) GOTO 22
      ENDIF
C     ==----------- DEF. OF BOUNDARIES IN CV. SPACE  ---------------==
      II=INDEX(LINE,'WALL+')
      IF(II .NE. 0) THEN
        IBOUND(NCOLVAR)=1
        IA = II+5
        ! position of the wall
        CALL READSR(LINE,IA,IE,VBOUND(1,NCOLVAR),   ERREAD)
        IA = IE
        ! height of the wall (hartrees)
        CALL READSR(LINE,IA,IE,VBOUND(2,NCOLVAR),   ERREAD)
      ENDIF
      II=INDEX(LINE,'WALL-')
      IF(II .NE. 0) THEN
        IBOUND(NCOLVAR)=1
        IA = II+5
        ! position of the wall
        CALL READSR(LINE,IA,IE,VBOUND(3,NCOLVAR),   ERREAD)
        IA = IE
        ! height of the wall (hartrees)
        CALL READSR(LINE,IA,IE,VBOUND(4,NCOLVAR),   ERREAD)
      ENDIF
cmb-ale - we want to be free to set the initial value !
      II=INDEX(LINE,'INITIAL_VALUE')
      IF(II .NE. 0) THEN
        INITIAL_VALUE(NCOLVAR)=.TRUE.
        IA=II+13
        CALL READSR(LINE,IA,IE,CV_DYN_0(NCOLVAR),ERREAD)
      ENDIF

      RETURN
22    CONTINUE
      WRITE(6,*) ' ERROR WHILE READING SCAL. FAC. or MASS or KH.'
      WRITE(6,'(A80)') LINE
      CALL STOPGM('M_COLVAR_INP',' ')
      END
C     ==================================================================
