C     ==================================================================
      SUBROUTINE MTIN
C     ==--------------------------------------------------------------==
C     == SCREENING FUNCTION FOR TUCKERMAN POISSON SOLVER              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'isos.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'fft.inc'
      REAL*8  EEPS
      PARAMETER (EEPS=1.D-20)
      COMPLEX VR(*)
      POINTER (IP_VR,VR)
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_VR,2*MAXFFT,'VR')
      CALL MEMORY(IP_SCG,2*NHG,'SCG')
      IF(TONED.OR.TTWOD) CALL STOPGM('MTIN','not implemented')
C     ==------------------------------------------------------------==
      CALL SCREENA(EEPS)
      CALL SCREENB(VR,EEPS)
      CALL FREEM(IP_VR)
      IF(PARENT) CALL PRMEM('   CLUSTER')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MOIN
C     ==--------------------------------------------------------------==
C     == SCREENING FUNCTION FOR MORTENSEN POISSON SOLVER              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cnst.inc'
      INTEGER IG,IG1,NZ
      REAL*8 G2,ALPHA,GX,GY,GZ,GXY,GYZ,X,Y,Z,BJ0,BJ1,BK0,BK1
      REAL*8 BESSJ0,BESSJ1,BESSK0,BESSK1
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_SCG,2*NHG,'SCG')
      IF(TONED) THEN
C..we assume periodicity in x, square box in y and z, no total charge
        Z=A3(3)*0.5D0
        IG1=1
        IF(GEQ0) IG1=2
        DO IG=IG1,NHG
          G2=TPIBA2*HG(IG)
          GX=GK(1,IG)*TPIBA
          GY=GK(2,IG)
          GZ=GK(3,IG)
          GYZ=TPIBA*DSQRT(GZ*GZ+GY*GY)
          BJ0=BESSJ0(Z*GYZ)
          BJ1=BESSJ1(Z*GYZ)
          BK0=BESSK0(Z*GX)
          BK1=BESSK1(Z*GX)
          SCG(IG)=FPI/G2 * (1.D0 + Z*(GYZ*BJ1*BK0-GX*BJ0*BK1))
        ENDDO
        IF(GEQ0) SCG(1)=0.D0
      ELSEIF(TTWOD) THEN
C..we assume orthorhobmic box with surface in specified plane, no total charge
        IF(SNORMAL.EQ.1) THEN
          Z=A1(1)
        ELSE IF(SNORMAL.EQ.2) THEN 
          Z=A2(2)
        ELSE IF(SNORMAL.EQ.3) THEN 
          Z=A3(3)
        ENDIF
        IG1=1
        IF(GEQ0) IG1=2
        DO IG=IG1,NHG
          G2=TPIBA2*HG(IG)
          GX=GK(1,IG)
          GY=GK(2,IG)
          GZ=GK(3,IG)
          IF(SNORMAL.EQ.1) THEN           ! YZ PLANE
            GXY=TPIBA*DSQRT(GY*GY+GZ*GZ)
            NZ=NINT(Z/ALAT*GX)
          ELSE IF(SNORMAL.EQ.2) THEN      ! ZX PLANE
            GXY=TPIBA*DSQRT(GZ*GZ+GX*GX)
            NZ=NINT(Z/ALAT*GY)
          ELSE IF(SNORMAL.EQ.3) THEN      ! XY PLANE
            GXY=TPIBA*DSQRT(GX*GX+GY*GY)
            NZ=NINT(Z/ALAT*GZ)
          ENDIF
          SCG(IG)=FPI/G2 * (1.D0 - (-1.D0)**NZ * exp(-0.5D0*Z*GXY))
C..following lines would be rigorous: uncomment in case of problems
C         GZ=TPIBA*GZ
C         SCG(IG)=FPI/G2 * (1.D0 - cos(0.5D0*Z*GZ)*exp(-0.5*Z*GXY))
        ENDDO
        IF(GEQ0) SCG(1)=0.D0
      ELSE
        ALPHA=ALAT*0.5D0
        IG1=1
        IF(GEQ0) IG1=2
        DO IG=IG1,NHG
          G2=TPIBA2*HG(IG)
          SCG(IG)=FPI/G2 * (1.D0 - COS(ALPHA*SQRT(G2)))
        ENDDO
        IF(GEQ0) SCG(1)=0.5D0*FPI*ALPHA*ALPHA
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SCREENA(EEPS)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'geq0.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'cppt.inc'
C     Arguments
      REAL*8  EEPS
C     Variables
      INTEGER IG,IG1
      REAL*8  ALPHA,BETA,RMIN,T0,T1,T2,T3,G2,G
      REAL*8  ERFI
      EXTERNAL ERFI
C     ==--------------------------------------------------------------==
      RMIN=MIN(A1(1),A2(2),A3(3))
      ALPHA=ALPHAL/RMIN
      BETA=SQRT(-LOG(EEPS))/RMIN
      BETA=MAX(0.2D0,BETA)
      T0=2.D0*PI*ALPHA/(BETA*BETA)/SQRT(ALPHA*ALPHA+BETA*BETA)
      T1=2.D0*PI**(1.5D0)/BETA
      T2=-0.25D0/(BETA*BETA)
      T3=SQRT(ALPHA*ALPHA/(ALPHA*ALPHA+BETA*BETA))*0.5D0/BETA
      IG1=1
      IF(GEQ0) IG1=2
      DO IG=IG1,NHG
        G2=TPIBA2*HG(IG)
        G=SQRT(G2)
        SCG(IG)=T1/G*ERFI(G*T3,T2*G2)
      ENDDO
      IF(GEQ0) SCG(1)=T0
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SCREENB(VR,EEPS)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'fft.inc'
C     Arguments
      COMPLEX*16 VR(KR1,KR2S,KR3S)
      REAL*8     EEPS
C     Variables
      INTEGER I,II,J,K,ISUB,IG,IG1
      REAL*8 DX,DY,DZ,R2,R,X,Y,Z,AX,AY,AZ,OP,OA
      REAL*8 ALPHA,BETA,RMIN,G2,FB
#ifdef __DERF
      REAL*8   DERF
      EXTERNAL DERF
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('   SCREENB',ISUB)
      CALL AZZERO(VR,2*MAXFFT)
      RMIN=MIN(A1(1),A2(2),A3(3))
      ALPHA=ALPHAL/RMIN
      BETA=SQRT(-LOG(EEPS))/RMIN
      BETA=MAX(0.2D0,BETA)
      DX=A1(1)/DBLE(NR1S)
      DY=A2(2)/DBLE(NR2S)
      DZ=A3(3)/DBLE(NR3S)
      AX=A1(1)*0.5D0
      AY=A2(2)*0.5D0
      AZ=A3(3)*0.5D0
      OP=OMEGA
      DO K=1,NR3S
        Z=DBLE(K-1)*DZ
        IF(Z.GT.AZ) Z=2.D0*AZ-Z
        DO J=1,NR2S
          Y=DBLE(J-1)*DY
          IF(Y.GT.AY) Y=2.D0*AY-Y
          DO II=1,NR1
            I=II+NRXPL(MEPOS,1)-1
            X=DBLE(I-1)*DX
            IF(X.GT.AX) X=2.D0*AX-X
            R2=X*X+Y*Y+Z*Z
            R=SQRT(R2)
            FB=1.D0-EXP(-BETA*BETA*R2)
            IF(R.GT.1.D-10) THEN
              VR(II,J,K)=OP*DERF(ALPHA*R)/R*FB
            ELSE
              VR(II,J,K)=2.D0*OP*ALPHA/SQRT(PI)*FB
            ENDIF
          ENDDO
        ENDDO
      ENDDO
      CALL FWFFT(VR)
      DO IG=1,NHG
        SCG(IG) = SCG(IG) + VR(NZH(IG),1,1)
      ENDDO
C
      OP=FPI
      OA=1.D0/(4.D0*ALPHA*ALPHA)
      IG1=1
      IF(GEQ0) IG1=2
      DO IG=IG1,NHG
        G2=TPIBA2*HG(IG)
        SCG(IG) = SCG(IG) + FPI/G2 - OP*EXP(-G2*OA)/G2
      ENDDO
      IF(GEQ0) SCG(1)=SCG(1) + OP*OA
      CALL TIHALT('   SCREENB',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION ERFI(X,Y)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8 ERFI,X,Y
C     Variables
      REAL*8 U,V
      LOGICAL FLAG
C     COMPLEX*16 A
C     ==--------------------------------------------------------------==
      CALL WOFZ(X,0.D0,U,V,FLAG)
C     A=DCMPLX(0.D0,1.D0)*(1.D0-EXP(X*X)*DCMPLX(U,V))
C     ERFI=DREAL(A)
      ERFI=V*EXP(X*X+Y)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C      ALGORITHM 680, COLLECTED ALGORITHMS FROM ACM.
C      THIS WORK PUBLISHED IN TRANSACTIONS ON MATHEMATICAL SOFTWARE,
C      VOL. 16, NO. 1, PP. 47.
      SUBROUTINE WOFZ (XI, YI, U, V, FLAG)
C
C  GIVEN A COMPLEX NUMBER Z = (XI,YI), THIS SUBROUTINE COMPUTES
C  THE VALUE OF THE FADDEEVA-FUNCTION W(Z) = EXP(-Z**2)*ERFC(-I*Z),
C  WHERE ERFC IS THE COMPLEX COMPLEMENTARY ERROR-FUNCTION AND I
C  MEANS SQRT(-1).
C  THE ACCURACY OF THE ALGORITHM FOR Z IN THE 1ST AND 2ND QUADRANT
C  IS 14 SIGNIFICANT DIGITS; IN THE 3RD AND 4TH IT IS 13 SIGNIFICANT
C  DIGITS OUTSIDE A CIRCULAR REGION WITH RADIUS 0.126 AROUND A ZERO
C  OF THE FUNCTION.
C  ALL REAL VARIABLES IN THE PROGRAM ARE DOUBLE PRECISION.
C
C
C  THE CODE CONTAINS A FEW COMPILER-DEPENDENT PARAMETERS :
C     RMAXREAL = THE MAXIMUM VALUE OF RMAXREAL EQUALS THE ROOT OF
C                RMAX = THE LARGEST NUMBER WHICH CAN STILL BE
C                IMPLEMENTED ON THE COMPUTER IN DOUBLE PRECISION
C                FLOATING-POINT ARITHMETIC
C     RMAXEXP  = LN(RMAX) - LN(2)
C     RMAXGONI = THE LARGEST POSSIBLE ARGUMENT OF A DOUBLE PRECISION
C                GONIOMETRIC FUNCTION (DCOS, DSIN, ...)
C  THE REASON WHY THESE PARAMETERS ARE NEEDED AS THEY ARE DEFINED WILL
C  BE EXPLAINED IN THE CODE BY MEANS OF COMMENTS
C
C
C  PARAMETER LIST
C     XI     = REAL      PART OF Z
C     YI     = IMAGINARY PART OF Z
C     U      = REAL      PART OF W(Z)
C     V      = IMAGINARY PART OF W(Z)
C     FLAG   = AN ERROR FLAG INDICATING WHETHER OVERFLOW WILL
C              OCCUR OR NOT; TYPE LOGICAL;
C              THE VALUES OF THIS VARIABLE HAVE THE FOLLOWING
C              MEANING :
C              FLAG=.FALSE. : NO ERROR CONDITION
C              FLAG=.TRUE.  : OVERFLOW WILL OCCUR, THE ROUTINE
C                             BECOMES INACTIVE
C  XI, YI      ARE THE INPUT-PARAMETERS
C  U, V, FLAG  ARE THE OUTPUT-PARAMETERS
C
C  FURTHERMORE THE PARAMETER FACTOR EQUALS 2/SQRT(PI)
C
C  THE ROUTINE IS NOT UNDERFLOW-PROTECTED BUT ANY VARIABLE CAN BE
C  PUT TO 0 UPON UNDERFLOW;
C
C  REFERENCE - GPM POPPE, CMJ WIJERS; MORE EFFICIENT COMPUTATION OF
C  THE COMPLEX ERROR-FUNCTION, ACM TRANS. MATH. SOFTWARE.
C
*
*
*
*
      IMPLICIT NONE
*
C     Arguments
      REAL*8  XI,YI,U,V
      LOGICAL FLAG
C     Variables
      LOGICAL A, B
      REAL*8    FACTOR,RMAXREAL,RMAXEXP,RMAXGONI
      PARAMETER (FACTOR   = 1.12837916709551257388D0,
     *           RMAXREAL = 0.5D+154,
     *           RMAXEXP  = 708.503061461606D0,
     *           RMAXGONI = 3.53711887601422D+15)
      REAL*8    XABS,YABS,X,Y,QRHO,XABSQ,XQUAD,YQUAD,XSUM,YSUM,
     &          XAUX,U1,V1,DAUX,U2,V2,H,H2,QLAMBDA,RX,RY,SX,SY,
     &          TX,TY,C,W1
      INTEGER   N,J,I,KAPN,NU,NP1
*
      FLAG = .FALSE.
*
      XABS = DABS(XI)
      YABS = DABS(YI)
      X    = XABS/6.3D0
      Y    = YABS/4.4D0
*
C
C     THE FOLLOWING IF-STATEMENT PROTECTS
C     QRHO = (X**2 + Y**2) AGAINST OVERFLOW
C
      IF ((XABS.GT.RMAXREAL).OR.(YABS.GT.RMAXREAL)) GOTO 100
*
      QRHO = X**2 + Y**2
*
      XABSQ = XABS**2
      XQUAD = XABSQ - YABS**2
      YQUAD = 2*XABS*YABS
*
      A     = QRHO.LT.0.085264D0
*
      IF (A) THEN
C
C  IF (QRHO.LT.0.085264D0) THEN THE FADDEEVA-FUNCTION IS EVALUATED
C  USING A POWER-SERIES (ABRAMOWITZ/STEGUN, EQUATION (7.1.5), P.297)
C  N IS THE MINIMUM NUMBER OF TERMS NEEDED TO OBTAIN THE REQUIRED
C  ACCURACY
C
        QRHO  = (1.D0-0.85D0*Y)*DSQRT(QRHO)
        N     = IDNINT(6.D0 + 72.D0*QRHO)
        J     = 2*N+1
        XSUM  = 1.0D0/DBLE(J)
        YSUM  = 0.0D0
        DO 10 I=N, 1, -1
          J    = J - 2
          XAUX = (XSUM*XQUAD - YSUM*YQUAD)/DBLE(I)
          YSUM = (XSUM*YQUAD + YSUM*XQUAD)/DBLE(I)
          XSUM = XAUX + 1.D0/DBLE(J)
 10     CONTINUE
        U1   = -FACTOR*(XSUM*YABS + YSUM*XABS) + 1.0D0
        V1   =  FACTOR*(XSUM*XABS - YSUM*YABS)
        DAUX =  DEXP(-XQUAD)
        U2   =  DAUX*DCOS(YQUAD)
        V2   = -DAUX*DSIN(YQUAD)
*
        U    = U1*U2 - V1*V2
        V    = U1*V2 + V1*U2
*
      ELSE
C
C  IF (QRHO.GT.1.O) THEN W(Z) IS EVALUATED USING THE LAPLACE
C  CONTINUED FRACTION
C  NU IS THE MINIMUM NUMBER OF TERMS NEEDED TO OBTAIN THE REQUIRED
C  ACCURACY
C
C  IF ((QRHO.GT.0.085264D0).AND.(QRHO.LT.1.0)) THEN W(Z) IS EVALUATED
C  BY A TRUNCATED TAYLOR EXPANSION, WHERE THE LAPLACE CONTINUED FRACTION
C  IS USED TO CALCULATE THE DERIVATIVES OF W(Z)
C  KAPN IS THE MINIMUM NUMBER OF TERMS IN THE TAYLOR EXPANSION NEEDED
C  TO OBTAIN THE REQUIRED ACCURACY
C  NU IS THE MINIMUM NUMBER OF TERMS OF THE CONTINUED FRACTION NEEDED
C  TO CALCULATE THE DERIVATIVES WITH THE REQUIRED ACCURACY
C
*
        IF (QRHO.GT.1.D0) THEN
          H    = 0.0D0
          KAPN = 0
          QRHO = DSQRT(QRHO)
          NU   = IDINT(3.D0 + (1442.D0/(26.D0*QRHO+77.D0)))
        ELSE
          QRHO = (1.D0-Y)*DSQRT(1.D0-QRHO)
          H    = 1.88D0*QRHO
          H2   = 2.0D0*H
          KAPN = IDNINT(7.D0  + 34.D0*QRHO)
          NU   = IDNINT(16.D0 + 26.D0*QRHO)
        ENDIF
*
        B = (H.GT.0.0D0)
*
        IF (B) QLAMBDA = H2**KAPN
*
        RX = 0.0D0
        RY = 0.0D0
        SX = 0.0D0
        SY = 0.0D0
*
        DO 11 N=NU, 0, -1
          NP1 = N + 1
          TX  = YABS + H + NP1*RX
          TY  = XABS - NP1*RY
          C   = 0.5D0/(TX**2 + TY**2)
          RX  = C*TX
          RY  = C*TY
          IF ((B).AND.(N.LE.KAPN)) THEN
            TX = QLAMBDA + SX
            SX = RX*TX - RY*SY
            SY = RY*TX + RX*SY
            QLAMBDA = QLAMBDA/H2
          ENDIF
 11     CONTINUE
*
        IF (H.EQ.0.D0) THEN
          U = FACTOR*RX
          V = FACTOR*RY
        ELSE
          U = FACTOR*SX
          V = FACTOR*SY
        END IF
*
        IF (YABS.EQ.0.D0) U = DEXP(-XABS**2)
*
      END IF
*
*
C
C  EVALUATION OF W(Z) IN THE OTHER QUADRANTS
C
*
      IF (YI.LT.0.D0) THEN
*
        IF (A) THEN
          U2    = 2.D0*U2
          V2    = 2.D0*V2
        ELSE
          XQUAD =  -XQUAD
*
C
C         THE FOLLOWING IF-STATEMENT PROTECTS 2*EXP(-Z**2)
C         AGAINST OVERFLOW
C
          IF ((YQUAD.GT.RMAXGONI).OR.
     *        (XQUAD.GT.RMAXEXP)) GOTO 100
*
          W1 =  2.D0*DEXP(XQUAD)
          U2  =  W1*DCOS(YQUAD)
          V2  = -W1*DSIN(YQUAD)
        END IF
*
        U = U2 - U
        V = V2 - V
        IF (XI.GT.0.D0) V = -V
      ELSE
        IF (XI.LT.0.D0) V = -V
      END IF
*
      RETURN
*
  100 FLAG = .TRUE.
      RETURN
*
      END
C     ==================================================================
      FUNCTION bessj0(x)
      REAL*8 bessj0,x
      REAL*8 ax,xx,z
      REAL*8 p1,p2,p3,p4,p5,q1,q2,q3,q4,q5,r1,r2,r3,r4,r5,r6,
     *s1,s2,s3,s4,s5,s6,y
      SAVE p1,p2,p3,p4,p5,q1,q2,q3,q4,q5,r1,r2,r3,r4,r5,r6,s1,s2,s3,s4,
     *s5,s6
      DATA p1,p2,p3,p4,p5/1.d0,-.1098628627d-2,.2734510407d-4,
     *-.2073370639d-5,.2093887211d-6/, q1,q2,q3,q4,q5/-.1562499995d-1,
     *.1430488765d-3,-.6911147651d-5,.7621095161d-6,-.934945152d-7/
      DATA r1,r2,r3,r4,r5,r6/57568490574.d0,-13362590354.d0,
     *651619640.7d0,-11214424.18d0,77392.33017d0,-184.9052456d0/,s1,s2,
     *s3,s4,s5,s6/57568490411.d0,1029532985.d0,9494680.718d0,
     *59272.64853d0,267.8532712d0,1.d0/
      if(abs(x).lt.8.D0)then
        y=x**2
        bessj0=(r1+y*(r2+y*(r3+y*(r4+y*(r5+y*r6)))))/(s1+y*(s2+y*(s3+y*
     *(s4+y*(s5+y*s6)))))
      else
        ax=abs(x)
        z=8.d0/ax
        y=z**2
        xx=ax-0.785398164d0
        bessj0=sqrt(0.636619772d0/ax)*(cos(xx)*(p1+y*(p2+y*(p3+y*(p4+y*
     *p5))))-z*sin(xx)*(q1+y*(q2+y*(q3+y*(q4+y*q5)))))
      endif
      return
      END
C     ==================================================================
      FUNCTION bessj1(x)
      REAL*8 bessj1,x
      REAL*8 ax,xx,z
      REAL*8 p1,p2,p3,p4,p5,q1,q2,q3,q4,q5,r1,r2,r3,r4,r5,r6,
     *s1,s2,s3,s4,s5,s6,y
      SAVE p1,p2,p3,p4,p5,q1,q2,q3,q4,q5,r1,r2,r3,r4,r5,r6,s1,s2,s3,s4,
     *s5,s6
      DATA r1,r2,r3,r4,r5,r6/72362614232.d0,-7895059235.d0,
     *242396853.1d0,-2972611.439d0,15704.48260d0,-30.16036606d0/,s1,s2,
     *s3,s4,s5,s6/144725228442.d0,2300535178.d0,18583304.74d0,
     *99447.43394d0,376.9991397d0,1.d0/
      DATA p1,p2,p3,p4,p5/1.d0,.183105d-2,-.3516396496d-4,
     *.2457520174d-5,-.240337019d-6/, q1,q2,q3,q4,q5/.04687499995d0,
     *-.2002690873d-3,.8449199096d-5,-.88228987d-6,.105787412d-6/
      if(abs(x).lt.8.D0)then
        y=x**2
        bessj1=x*(r1+y*(r2+y*(r3+y*(r4+y*(r5+y*r6)))))/(s1+y*(s2+y*(s3+
     *       y*(s4+y*(s5+y*s6)))))
      else
        ax=dabs(x)
        z=8.d0/ax
        y=z**2
        xx=ax-2.356194491d0
        bessj1=dsqrt(0.636619772d0/ax)*(dcos(xx)*(p1+y*(p2+y*(p3+y*(p4+y
     &  *p5))))-z*dsin(xx)*(q1+y*(q2+y*(q3+y*(q4+y*q5)))))*dsign(1.d0,x)
      endif
      return
      END
C     ==================================================================
      FUNCTION bessk0(x)
      REAL*8 bessk0,x
CU    USES bessi0
      REAL*8 bessi0
      REAL*8 p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,y
      SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7
      DATA p1,p2,p3,p4,p5,p6,p7/-0.57721566d0,0.42278420d0,0.23069756d0,
     *0.3488590d-1,0.262698d-2,0.10750d-3,0.74d-5/
      DATA q1,q2,q3,q4,q5,q6,q7/1.25331414d0,-0.7832358d-1,0.2189568d-1,
     *-0.1062446d-1,0.587872d-2,-0.251540d-2,0.53208d-3/
      if (x.le.2.0d0) then
        y=x*x/4.0d0
        bessk0=(-log(x/2.d0)*bessi0(x))+(p1+y*(p2+y*(p3+y*(p4+y*(p5+y*
     *       (p6+y*p7))))))
      else
        y=(2.0d0/x)
        bessk0=(dexp(-x)/dsqrt(x))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     *       q7))))))
      endif
      return
      END
C     ==================================================================
      FUNCTION bessk1(x)
      REAL*8 bessk1,x
CU    USES bessi1
      REAL*8 bessi1
      REAL*8 p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,y
      SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7
      DATA p1,p2,p3,p4,p5,p6,p7/1.0d0,0.15443144d0,-0.67278579d0,
     *-0.18156897d0,-0.1919402d-1,-0.110404d-2,-0.4686d-4/
      DATA q1,q2,q3,q4,q5,q6,q7/1.25331414d0,0.23498619d0,-0.3655620d-1,
     *0.1504268d-1,-0.780353d-2,0.325614d-2,-0.68245d-3/
      if (x.le.2.d0) then
        y=x*x/4.d0
        bessk1=(dlog(x/2.d0)*bessi1(x))+(1.d0/x)*(p1+y*(p2+y*(p3+y*(p4+y
     *       *(p5+y*(p6+y*p7))))))
      else
        y=2.d0/x
        bessk1=(dexp(-x)/sqrt(x))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     *       q7))))))
      endif
      return
      END
C     ==================================================================
      FUNCTION bessi0(x)
      REAL*8 bessi0,x
      REAL*8 ax
      REAL*8 p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9,y
      SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9
      DATA p1,p2,p3,p4,p5,p6,p7/1.0d0,3.5156229d0,3.0899424d0,
     *1.2067492d0,0.2659732d0,0.360768d-1,0.45813d-2/
      DATA q1,q2,q3,q4,q5,q6,q7,q8,q9/0.39894228d0,0.1328592d-1,
     *0.225319d-2,-0.157565d-2,0.916281d-2,-0.2057706d-1,0.2635537d-1,
     *-0.1647633d-1,0.392377d-2/
      if (dabs(x).lt.3.75d0) then
        y=(x/3.75d0)**2
        bessi0=p1+y*(p2+y*(p3+y*(p4+y*(p5+y*(p6+y*p7)))))
      else
        ax=dabs(x)
        y=3.75d0/ax
        bessi0=(dexp(ax)/dsqrt(ax))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     *       (q7+y*(q8+y*q9))))))))
      endif
      return
      END
C     ==================================================================
      FUNCTION bessi1(x)
      REAL*8 bessi1,x
      REAL*8 ax
      REAL*8 p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9,y
      SAVE p1,p2,p3,p4,p5,p6,p7,q1,q2,q3,q4,q5,q6,q7,q8,q9
      DATA p1,p2,p3,p4,p5,p6,p7/0.5d0,0.87890594d0,0.51498869d0,
     *0.15084934d0,0.2658733d-1,0.301532d-2,0.32411d-3/
      DATA q1,q2,q3,q4,q5,q6,q7,q8,q9/0.39894228d0,-0.3988024d-1,
     *-0.362018d-2,0.163801d-2,-0.1031555d-1,0.2282967d-1,-0.2895312d-1,
     *0.1787654d-1,-0.420059d-2/
      if (dabs(x).lt.3.75D0) then
        y=(x/3.75d0)**2
        bessi1=x*(p1+y*(p2+y*(p3+y*(p4+y*(p5+y*(p6+y*p7))))))
      else
        ax=dabs(x)
        y=3.75d0/ax
        bessi1=(dexp(ax)/dsqrt(ax))*(q1+y*(q2+y*(q3+y*(q4+y*(q5+y*(q6+y*
     *       (q7+y*(q8+y*q9))))))))
        if(x.lt.0.D0)bessi1=-bessi1
      endif
      return
      END






