C     ==================================================================
      SUBROUTINE MULTTB(INDPG,NROT,NTVEC,NAT,XTABLE,
     &                  INVE,MULTAB,IRT,PRT)
C     ==--------------------------------------------------------------==
C     ==  CONSTRUCT GOUP MULTIPLICATION TABLE                         ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   INDPG Group number (if 0 return otherwise not use)         ==
C     ==   NROT  Number of rotations                                  ==
C     ==   NTVEC Number of translation vectors associated with        ==
C     ==         Identity rotation                                    ==
C     ==         NTVEC /= 1 if the cell is not the primitive one      ==
C     ==   NAT   Number of atoms                                      ==
C     ==   XTABLE(3,3,NROT) Rotations                                 ==
C     ==   INVE(120) Number of inverse operation for each rotation    ==
C     ==   IRT(120,NAT)   Atom transformation table for rotations     ==
C     ==   PRT .TRUE. PRINT SOME INFORMATION                          ==
C     == OUTPUT:                                                      ==
C     ==   MULTAB(120,120) multiplication table                       ==
C     ==   INVE(NROT) inverse operation for each rotation             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   INDPG,NROT,NTVEC,NAT,
     &          INVE(120),MULTAB(120,120),IRT(120,NAT)
      REAL*8    XTABLE(3,3,NROT)
      LOGICAL   PRT
C     Variables
      LOGICAL   LMTEST
      CHARACTER FFORMAT*15
      REAL*8    XAB(3,3)
      INTEGER   IRTM(200),IDGEN(12),
     &          TCN(24),TCLASS(24,120),
     &          IRA,IRB,I,J,IRC,IDIFF,IA,IR,IX,ICLASS,
     &          IY,ID3MAX,I1,I2,I3
C     ==--------------------------------------------------------------==
      IF(INDPG.EQ.0) RETURN
      LMTEST=.TRUE.
C     ==--------------------------------------------------------------==
C     Group multiplication table
      DO IRA=1,NROT
        DO IRB=1,NROT
          DO J=1,3
            DO I=1,3
              XAB(I,J)=XTABLE(I,1,IRB)*XTABLE(1,J,IRA)+
     *                 XTABLE(I,2,IRB)*XTABLE(2,J,IRA)+
     *                 XTABLE(I,3,IRB)*XTABLE(3,J,IRA)
            ENDDO
          ENDDO
          DO IRC=1,NROT
            IDIFF=0
            DO J=1,3
              DO I=1,3
                IDIFF=IDIFF+NINT(ABS(XTABLE(I,J,IRC)-XAB(I,J)))
              ENDDO
            ENDDO
            IF(IDIFF.EQ.0) THEN
              MULTAB(IRA,IRB)=IRC
              GOTO 100
            ENDIF
          ENDDO
          WRITE(6,*)
     &         'MULTTB| CANNOT FIND PROPER MULTIPLICATION OF ',
     *         IRA,' AND ',IRB
          CALL STOPGM('MULTTB',' ')
 100      CONTINUE
C         ==---------------------------------------------------------==
C         Test if the cell is a primary cell (or no replication).
          IRC=MULTAB(IRA,IRB)
          DO IA=1,NAT
            IRTM(IA)=IRT(IRB,IRT(IRA,IA))
          ENDDO
          IDIFF=0
          DO I=1,NAT
            IDIFF=IDIFF+ABS(IRT(IRC,I)-IRTM(I))
          ENDDO
          IF(IDIFF.NE.0) THEN
            IF(LMTEST) THEN
              IF(PRT) WRITE(6,*)
     &             'SUPERCELL ATOMS FORM NO BASIS FOR ',
     *             'THIS POINT GROUP'
            ENDIF
            LMTEST=.FALSE.
          ENDIF
        ENDDO
      ENDDO
      IF(NTVEC.NE.1) THEN
        IF(LMTEST) THEN
          IF(PRT) WRITE(6,*)
     &         'MULTTB| SUPERCELL ATOMS FORM NO BASIS FOR ',
     *         'THIS POINT GROUP'
        ENDIF
        LMTEST=.FALSE.
      ENDIF
C     ==--------------------------------------------------------------==
C     Look for the inverse of a symmetry operation
      DO IR=1,NROT
        DO IX=1,NROT
          IF(MULTAB(IR,IX).EQ.1) THEN
            INVE(IR)=IX
            GOTO 200
          ENDIF
        ENDDO
        WRITE(6,*)
     &       'MULTTB| CANNOT FIND INVERSE OPERATION OF ',IR
        CALL STOPGM('MULTTB',' ')
 200    CONTINUE
      ENDDO
C     ==--------------------------------------------------------------==
C     Generate the class structure of the group
      ICLASS=0
      DO IR=1,NROT
        DO I=1,ICLASS
          DO J=1,TCN(I)
            IF(IR.EQ.TCLASS(I,J)) GOTO 300
          ENDDO
        ENDDO
        ICLASS=ICLASS+1
        TCN(ICLASS)=0
        DO IX=1,NROT
          IY=MULTAB(IX,IR)
          IY=MULTAB(IY,INVE(IX))
          DO I=1,TCN(ICLASS)
            IF(TCLASS(ICLASS,I).EQ.IY) GOTO 400
          ENDDO
          TCN(ICLASS)=TCN(ICLASS)+1
          TCLASS(ICLASS,TCN(ICLASS))=IY
 400      CONTINUE
        ENDDO
 300    CONTINUE
      ENDDO
      IF(PRT) WRITE(6,'(A,T63,I3)') 
     *     ' NUMBER OF IRREDUCIBLE REPRESENTATIONS:',ICLASS
C     ==--------------------------------------------------------------==
C     Calculation of degre of irreducible representation
C     Sum d_{IR}^2=NROT with d=1,2 or 3
C     Maximum number of IR of degree 3
      ID3MAX=NROT/3**2
      DO I3=ID3MAX,0,-1
C       ID2 and ID1 are automatically determined
        I=NROT-I3*3**2-(ICLASS-I3)
        IF(MOD(I,3).EQ.0) THEN
C         We got a solution
          I2=I/3
          I1=ICLASS-I3-I2
          GOTO 500
        ENDIF
      ENDDO
C     Error
      CALL STOPGM('MULTTB','DEGRES OF IR NO DETERMINED')
 500  CONTINUE
      DO IR=1,ICLASS
        IF(IR.LE.I1) THEN
          IDGEN(IR)=1
        ELSEIF(IR.LE.(I1+I2)) THEN
          IDGEN(IR)=2
        ELSE
          IDGEN(IR)=3
        ENDIF
      ENDDO
      IF(PRT) THEN
        WRITE(FFORMAT,'(A,I2,A,I2,A)')
     &       '(A,T',66-ICLASS*3,',',ICLASS,'I3)'
       WRITE(6,FFORMAT)
     &         ' DIMENSION OF IR:',(IDGEN(I),I=1,ICLASS)
      ENDIF
      IY=0
      DO I=1,ICLASS
        IY=IY+IDGEN(I)*IDGEN(I)
      ENDDO
      IF(IY.NE.NROT) CALL STOPGM('MULTTB',
     &     'INCORRECT CALUCLATION OF DIMENSIONS OF IR')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
