C     ==================================================================
C     == INTERFACE FOR MESSAGE PASSING CALLS:                         ==
C     ==================================================================
C CPMD (currently) only supports message passing via MPI.
#define __MPI   1
#define __OTHER 0

#ifdef PARALLEL
#define MP_LIBRARY __MPI
#endif

C     ==================================================================
      SUBROUTINE MY_START
C     ==--------------------------------------------------------------==
C     == Initialisation of message passing calls                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'pstat.inc'
C     Variables
      INTEGER I
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
      INTEGER IERR
C     ==--------------------------------------------------------------==
      CALL MPI_INIT(IERR)
      IF(IERR.NE.0) CALL STOPGM('MPI_INIT','IERR.NE.0')
#else
      CALL STOPGM('MPI_INIT','Unknown MP Interface')
#endif
      CALL AZZERO(CMLEN,CMPAR)
      DO I=1,CMPAR
        CMCAL(I)=1.D-20
        CMTIM(I)=1.D-20
      ENDDO
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_END
C     ==--------------------------------------------------------------==
C     == End of message passing calls. Display some information       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'pstat.inc'
C     Variables
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
      INTEGER IERR
#endif
      CHARACTER FFORMAT1*50,FFORMAT2*60
      PARAMETER (FFORMAT1=
     &     '(1X,"=",A,T27,F10.0," BYTES",8X,F12.0,2X,"=")')
      PARAMETER (FFORMAT2=
     &     '(1X,"=",A,T27,F10.3,"  MB/S",6X,F10.3," SEC",2X,"=")')
      INTEGER I
      REAL*8  XLN
C     ==--------------------------------------------------------------==
      IF(PARENT.AND.NPROC.GT.1) THEN
        DO I=1,CMPAR
          IF(CMCAL(I).LT.1) THEN
             CMLEN(I)=0
             CMCAL(I)=1
          ENDIF
          IF(CMTIM(I).LT.1.D-8) THEN
             CMLEN(I)=0
             CMTIM(I)=1
          ENDIF
        ENDDO
        XLN=LOG(DBLE(NPROC))/LOG(2.D0)
C       ==------------------------------------------------------------==
        WRITE(6,'(/,1X,64("="))')
        WRITE(6,'(1X,"=",A,2X,A,2X,A,2X,"=")') ' COMMUNICATION TASK',
     *       'AVERAGE MESSAGE LENGTH','NUMBER OF CALLS'
        WRITE(6,FFORMAT1) ' SEND/RECEIVE',
     &       CMLEN(IPAR_SEND)/CMCAL(IPAR_SEND),CMCAL(IPAR_SEND)
        WRITE(6,FFORMAT1) ' BROADCAST',        
     &       CMLEN(IPAR_CAST)/CMCAL(IPAR_CAST),CMCAL(IPAR_CAST)
        WRITE(6,FFORMAT1) ' GLOBAL SUMMATION', 
     &       CMLEN(IPAR_GSUM)/CMCAL(IPAR_GSUM),CMCAL(IPAR_GSUM)
        WRITE(6,FFORMAT1) ' GLOBAL MULTIPLICATION',
     &       CMLEN(IPAR_GMUL)/CMCAL(IPAR_GMUL),CMCAL(IPAR_GMUL)
        WRITE(6,FFORMAT1) ' ALL TO ALL COMM ', 
     &       CMLEN(IPAR_AALL)/CMCAL(IPAR_AALL),CMCAL(IPAR_AALL)
C       ==------------------------------------------------------------==
        WRITE(6,'(1X,"=",29X,A11,10X,A10,2X,"=")')
     *       'PERFORMANCE','TOTAL TIME'
        WRITE(6,FFORMAT2) ' SEND/RECEIVE',
     &       CMLEN(IPAR_SEND)/CMTIM(IPAR_SEND)*1.D-3,
     *       CMTIM(IPAR_SEND)*1.D-3
        WRITE(6,FFORMAT2) ' BROADCAST',
     &       CMLEN(IPAR_CAST)/CMTIM(IPAR_CAST)*1.D-3,
     *       CMTIM(IPAR_CAST)*1.D-3
        WRITE(6,FFORMAT2) ' GLOBAL SUMMATION',
     &       XLN*CMLEN(IPAR_GSUM)/CMTIM(IPAR_GSUM)*1.D-3,
     *       CMTIM(IPAR_GSUM)*1.D-3
        WRITE(6,FFORMAT2) ' GLOBAL MULTIPLICATION',
     &       XLN*CMLEN(IPAR_GMUL)/CMTIM(IPAR_GMUL)*1.D-3,
     *       CMTIM(IPAR_GMUL)*1.D-3
        WRITE(6,FFORMAT2) ' ALL TO ALL COMM ',
     &       CMLEN(IPAR_AALL)/CMTIM(IPAR_AALL)*1.D-3,
     *       CMTIM(IPAR_AALL)*1.D-3
        WRITE(6,'(1X,"=",A,17X,A,6X,F10.3,A,2X,"=")')
     *       ' SYNCHRONISATION ','      ',CMTIM(IPAR_SYNC)*1.D-3,' SEC'
        WRITE(6,'(1X,64("="))')
        CALL MY_FLUSH(6)
      ENDIF
#if defined(__ES)
      IF(PARENT) CLOSE(6)
#endif
#if MP_LIBRARY == __MPI
      CALL MPI_FINALIZE(IERR)
      IF(IERR.NE.0) CALL STOPGM('MPI_FINALIZE','IERR.NE.0')
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_GROUP(GSIZE,GLIST,LABEL,GID,PCOMM)
C     ==--------------------------------------------------------------==
C     == Forms groups of processors                                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER GSIZE,GLIST(*),LABEL,GID,PCOMM
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER WORLD,NEWGROUP,IERR
C     ==--------------------------------------------------------------==
      CALL MPI_COMM_GROUP(PCOMM,WORLD,IERR)
      CALL MPI_GROUP_INCL(WORLD,GSIZE,GLIST,NEWGROUP,IERR)
      CALL MPI_COMM_CREATE(PCOMM,NEWGROUP,GID,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_GROUP','IERR.NE.0')
#else
C     ==--------------------------------------------------------------==
      GID=0
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_STOPALL(CODE)
C     ==--------------------------------------------------------------==
C     == STOP ALL PROCESSORS                                          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER CODE
#if MP_LIBRARY == __MPI
C     Variables
      INCLUDE 'mpif.h'
      INTEGER IERR
C     ==--------------------------------------------------------------==
      CALL MPI_ABORT(MPI_COMM_WORLD,CODE,IERR)
      IF(IERR.NE.0) CALL STOPGM('MPI_ABORT','IERR.NE.0')
#else
      STOP 'MY_STOPALL'
#endif
C     ==--------------------------------------------------------------==
      END
C     ==================================================================
      SUBROUTINE MY_SYNC(GID)
C     ==--------------------------------------------------------------==
C     == SYNCHRONISATION OF ALL PROCESSORS                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'pstat.inc'
C     Arguments
      INTEGER GID
#if MP_LIBRARY == __MPI
C     Variables
      INTEGER IERR
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      TIM1=TIMEF()
      CALL MPI_BARRIER(GID,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_SYNC',' ')
#else
C     Variables
      REAL*8 TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      TIM1=TIMEF()
#endif
      TIM2=TIMEF()
      CMTIM(IPAR_SYNC)=CMTIM(IPAR_SYNC)+TIM2-TIM1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_ENVIRON(NUMTASK,TASKID)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER NUMTASK,TASKID
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER IERR
C     ==--------------------------------------------------------------==
      CALL MPI_COMM_RANK(MPI_COMM_WORLD,TASKID,IERR)
      CALL MPI_COMM_SIZE(MPI_COMM_WORLD,NUMTASK,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_ENVIRON','IERR.NE.0')
#else
C     ==--------------------------------------------------------------==
      NUMTASK = 1
      TASKID  = 0
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_TASK_QUERY(ALLGRP)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER ALLGRP
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     ==--------------------------------------------------------------==
      ALLGRP=MPI_COMM_WORLD
#else
C     ==--------------------------------------------------------------==
      ALLGRP=0
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_BCAST(MSG,MSGLEN,SOURCE,GID)
C     ==--------------------------------------------------------------==
C     == Broadcast message MSG(MSGLEN) from SOURCE to GID proc groups ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'pstat.inc'
C     Arguments
      INTEGER MSGLEN,MSG(MSGLEN),SOURCE,GID
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER IERR
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_CAST)=CMCAL(IPAR_CAST)+1.0D0
      CMLEN(IPAR_CAST)=CMLEN(IPAR_CAST)+MSGLEN
      TIM1=TIMEF()
      CALL MPI_BCAST(MSG,MSGLEN,MPI_BYTE,SOURCE,GID,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_BCAST','IERR.NE.0')
#else
C     Variables
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_CAST)=CMCAL(IPAR_CAST)+1.0D0
      CMLEN(IPAR_CAST)=CMLEN(IPAR_CAST)+MSGLEN
      TIM1=TIMEF()
#endif
      TIM2=TIMEF()
      CMTIM(IPAR_CAST)=CMTIM(IPAR_CAST)+TIM2-TIM1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE my_char_bcast(par,str,strlen,source,group)
C     ==--------------------------------------------------------------==
C     ==    quick and dirty fix to broadcasting character*(*) vars
C     ==    broadcast a character*(strlen) variable from source
C     ==    to group, by converting it to an array of integers
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'irat.inc'
C     Arguments
      LOGICAL par               ! true if routine called by sender
      INTEGER strlen            ! length of string to send
      INTEGER source, group     ! source to send from, group to send to
      CHARACTER str(*)          ! string to send
C     Variables
      INTEGER maxlen 
      PARAMETER( maxlen=128 )
      INTEGER ibuf(maxlen), j, msglen
C     ==--------------------------------------------------------------==
      IF( strlen.gt.maxlen )
     $     call stopgm('char_bcast','string too long')
      IF( par ) THEN
        DO j=1,strlen          ! copy string to array of ints on source
          ibuf(j)=ichar(str(j))
        END DO
      END IF
      msglen = strlen * 8/IRAT
      CALL my_bcast(ibuf,msglen,source,group)
      DO j=1,strlen            ! copy array of ints to string on group
        str(j)=char(ibuf(j))
      END DO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     ==================================================================
      SUBROUTINE MY_ISEND(OUTMSG,MSGLEN,DEST,ITYPE,GID)
C     ==--------------------------------------------------------------==
C     == Send message OUTMSG(MSGLEN) to DEST processor                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'pstat.inc'
C     Arguments 
      INTEGER MSGLEN,OUTMSG(MSGLEN),DEST,ITYPE,GID
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER IERR,ireq
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_SEND)=CMCAL(IPAR_SEND)+1.0D0
      CMLEN(IPAR_SEND)=CMLEN(IPAR_SEND)+MSGLEN
      TIM1=TIMEF()
      CALL MPI_SEND(OUTMSG,MSGLEN,MPI_BYTE,DEST,ITYPE,
     *              GID,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_IBSEND','IERR.NE.0')
#else 
C     Variables
      REAL*8 TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_SEND)=CMCAL(IPAR_SEND)+1.0D0
      CMLEN(IPAR_SEND)=CMLEN(IPAR_SEND)+MSGLEN
      TIM1=TIMEF()
#endif
      TIM2=TIMEF()
      CMTIM(IPAR_SEND)=CMTIM(IPAR_SEND)+TIM2-TIM1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     ==================================================================
      SUBROUTINE MY_BSEND(OUTMSG,MSGLEN,DEST,ITYPE)
C     ==--------------------------------------------------------------==
C     == Send message OUTMSG(MSGLEN) to DEST processor                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'pstat.inc'
C     Arguments
      INTEGER MSGLEN,OUTMSG(MSGLEN),DEST,ITYPE
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER IERR
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_SEND)=CMCAL(IPAR_SEND)+1.0D0
      CMLEN(IPAR_SEND)=CMLEN(IPAR_SEND)+MSGLEN
      TIM1=TIMEF()
      CALL MPI_SEND(OUTMSG,MSGLEN,MPI_BYTE,DEST,ITYPE,
     *              MPI_COMM_WORLD,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_BSEND','IERR.NE.0')
#else
C     Variables
      REAL*8 TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_SEND)=CMCAL(IPAR_SEND)+1.0D0
      CMLEN(IPAR_SEND)=CMLEN(IPAR_SEND)+MSGLEN
      TIM1=TIMEF()
#endif
      TIM2=TIMEF()
      CMTIM(IPAR_SEND)=CMTIM(IPAR_SEND)+TIM2-TIM1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_BRECV(INMSG,MSGLEN,SOURCE,ITYPE,NBYTES)
C     ==--------------------------------------------------------------==
C     == Receive message INMSG(MSGLEN) from SOURCE                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'pstat.inc'
C     Arguments
      INTEGER MSGLEN,INMSG(MSGLEN),SOURCE,ITYPE,NBYTES
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER STATUS(MPI_STATUS_SIZE),IERR
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_SEND)=CMCAL(IPAR_SEND)+1.0D0
      CMLEN(IPAR_SEND)=CMLEN(IPAR_SEND)+MSGLEN
      TIM1=TIMEF()
      CALL MPI_RECV(INMSG,MSGLEN,MPI_BYTE,SOURCE,ITYPE,MPI_COMM_WORLD,
     *              STATUS,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_BRECV','IERR.NE.0')
#else
C     Variables
      REAL*8 TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_SEND)=CMCAL(IPAR_SEND)+1.0D0
      CMLEN(IPAR_SEND)=CMLEN(IPAR_SEND)+MSGLEN
      TIM1=TIMEF()
#endif
      TIM2=TIMEF()
      CMTIM(IPAR_SEND)=CMTIM(IPAR_SEND)+TIM2-TIM1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_IRECV(INMSG,MSGLEN,SOURCE,ITYPE,NBYTES,GID)
C     ==--------------------------------------------------------------==
C     == Receive message INMSG(MSGLEN) from SOURCE                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE'pstat.inc'
C     Arguments
      INTEGER MSGLEN,INMSG(MSGLEN),SOURCE,ITYPE,NBYTES, GID
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER STATUS(MPI_STATUS_SIZE),IERR
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_SEND)=CMCAL(IPAR_SEND)+1.0D0
      CMLEN(IPAR_SEND)=CMLEN(IPAR_SEND)+MSGLEN
      TIM1=TIMEF()
      CALL MPI_RECV(INMSG,MSGLEN,MPI_BYTE,SOURCE,ITYPE,GID,
     *              STATUS,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_IRECV','IERR.NE.0')
#else
C     Variables
      REAL*8 TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_SEND)=CMCAL(IPAR_SEND)+1.0D0
      CMLEN(IPAR_SEND)=CMLEN(IPAR_SEND)+MSGLEN
      TIM1=TIMEF()
#endif
      TIM2=TIMEF()
      CMTIM(IPAR_SEND)=CMTIM(IPAR_SEND)+TIM2-TIM1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_CONCAT(OUTMSG,INMSG,BLKLEN,GID)
C     ==--------------------------------------------------------------==
C     == Concat all OUTMSG from GID group processor into INMSG        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER OUTMSG(*),INMSG(*),BLKLEN,GID
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER IERR
C     ==--------------------------------------------------------------==
      CALL MPI_ALLGATHER(OUTMSG,BLKLEN,MPI_BYTE,INMSG,BLKLEN,
     *                   MPI_BYTE,GID,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_WAIT','IERR.NE.0')
#else
      INCLUDE 'irat.inc'
C     Variables
      INTEGER LEN
C     ==--------------------------------------------------------------==
      LEN = BLKLEN/(8/IRAT)  
      CALL ICOPY(LEN,OUTMSG,1,INMSG,1)
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_CONCATV(OUTMSG,INMSG,BLKLEN,RECVCNT,RECVDISPL,GID)
C     ==--------------------------------------------------------------==
C     == Concat all OUTMSG of different lengths from GID group        ==
C     == processor into INMSG. NOTE DOUBLE PRECISION ARRAYS USED      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8   OUTMSG(*),INMSG(*)
      INTEGER BLKLEN,GID,RECVCNT(*),RECVDISPL(*)
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER IERR
C     ==--------------------------------------------------------------==
      CALL MPI_ALLGATHERV(OUTMSG,BLKLEN,MPI_DOUBLE_PRECISION,INMSG, 
     *                    RECVCNT,RECVDISPL,MPI_DOUBLE_PRECISION
     *                   ,GID,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_WAIT','IERR.NE.0')
#else
      INCLUDE 'irat.inc'
C     Variables
      INTEGER LEN
C     ==--------------------------------------------------------------==
      CALL STOPGM('MY_CONCATV','NOT AVAILABLE')
      LEN = BLKLEN/(8/IRAT)
      CALL ICOPY(LEN,OUTMSG,1,INMSG,1)
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     ==================================================================
      SUBROUTINE MY_SOURCE_CONCATV(OUTMSG,INMSG,BLKLEN,RECVCNT,
     &                             RECVDISPL,ROOT,GID)
C     ==--------------------------------------------------------------==
C     == Concat all OUTMSG of different lengths from GID group        ==
C     == processor into INMSG. NOTE DOUBLE PRECISION ARRAYS USED      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8   OUTMSG(*),INMSG(*)
      INTEGER BLKLEN,GID,RECVCNT(*),RECVDISPL(*),ROOT
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER IERR
C     ==--------------------------------------------------------------==
      CALL MPI_GATHERV(OUTMSG,BLKLEN,MPI_DOUBLE_PRECISION,INMSG,
     *                    RECVCNT,RECVDISPL,MPI_DOUBLE_PRECISION
     *                   ,ROOT,GID,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_WAIT','IERR.NE.0')
#else
      INCLUDE 'irat.inc'
C     Variables
      INTEGER LEN
C     ==--------------------------------------------------------------==
      CALL STOPGM('MY_SOURCE_CONCATV','NOT AVAILABLE')
      LEN = BLKLEN/(8/IRAT)
      CALL ICOPY(LEN,OUTMSG,1,INMSG,1)
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#ifdef _PTP
C     ==================================================================
      SUBROUTINE MY_TRANS(OUTMSG,INMSG,BLKLEN,GROUP)
C     ==--------------------------------------------------------------==
C     == Matrix Transpose: Group 1 : all processors                   ==
C     ==                   Group 2 : orbital split                    ==
C     ==                   Group 3 : plane wave split                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'benc.inc'
      INCLUDE 'pstat.inc'
      INCLUDE 'irat.inc'        !IRAT
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
#endif
C     Arguments
      INTEGER OUTMSG(*),INMSG(*),BLKLEN,GID,GROUP
C     Variables
      INTEGER NN
      REAL*8  TIM1,TIM2,TIMEF
#if MP_LIBRARY == __MPI
      INTEGER IERR,LEN,TAG,I,IO
#else
      INTEGER LEN
#endif
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_AALL)=CMCAL(IPAR_AALL)+1.0D0
      TIM1=TIMEF()
      TAG=201
      IF(GROUP.EQ.1) THEN
        GID=ALLGRP
        NN=NPROC
      ELSEIF(GROUP.EQ.2) THEN
        GID=MEOGRP
        NN=NOGRP
      ELSEIF(GROUP.EQ.3) THEN
        GID=MEPGRP
        NN=NPGRP
      ENDIF
#if MP_LIBRARY == __MPI
      CALL MPI_COMM_RANK(GID,IO,IERR)
      IF(IERR.NE.0) CALL STOPGM('MPI_COMM_RANK','IERR.NE.0')
      DO  I=0,NN-1
        IF(I.EQ.IO) THEN
         LEN= BLKLEN/(8/IRAT)
         CALL   ICOPY(LEN,OUTMSG(1+I*BLKLEN),1,INMSG(1+I*BLKLEN),1)
        ELSEIF(I.LT.IO) THEN
         CALL MPI_SEND(OUTMSG(1+I*BLKLEN),BLKLEN,MPI_BYTE,TAG,GID,IERR)
         IF(IERR.NE.0) CALL STOPGM('MPI_SEND','IERR.NE.0')
         CALL MPI_RECV(INMSG(1+I*BLKLEN),BLKLEN,
     .                 MPI_BYTE,TAG,GID,MPI_STATUS_IGNORE,IERR)
         IF(IERR.NE.0) CALL STOPGM('MPI_RECV','IERR.NE.0')
        ELSEIF(I.GT.IO) THEN
         CALL MPI_RECV(INMSG(1+I*BLKLEN),BLKLEN,
     .                 MPI_BYTE,TAG,GID,MPI_STATUS_IGNORE,IERR)
         IF(IERR.NE.0) CALL STOPGM('MPI_RECV','IERR.NE.0')
         CALL MPI_SEND(OUTMSG(1+I*BLKLEN),BLKLEN,MPI_BYTE,TAG,GID,IERR)
         IF(IERR.NE.0) CALL STOPGM('MPI_SEND','IERR.NE.0')
        ENDIF
      ENDDO
#else
      LEN = BLKLEN/(8/IRAT)
      CALL ICOPY(LEN,OUTMSG(1),1,INMSG(1),1)
#endif
      TIM2=TIMEF()
      CMTIM(IPAR_AALL)=CMTIM(IPAR_AALL)+TIM2-TIM1
      CMLEN(IPAR_AALL)=CMLEN(IPAR_AALL)+BLKLEN*NN
C     ==--------------------------------------------------------------==
      RETURN
      END
#else
      SUBROUTINE MY_TRANS(OUTMSG,INMSG,BLKLEN,GROUP)
C     ==--------------------------------------------------------------==
C     == Matrix Transpose: Group 1 : all processors                   ==
C     ==                   Group 2 : orbital split                    ==
C     ==                   Group 3 : plane wave split                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'benc.inc'
      INCLUDE 'pstat.inc'
      INCLUDE 'irat.inc'        !IRAT
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
#endif
C     Arguments
      INTEGER OUTMSG(*),INMSG(*),BLKLEN,GID,GROUP
C     Variables
      INTEGER NN
      REAL*8  TIM1,TIM2,TIMEF
#if MP_LIBRARY == __MPI
      INTEGER IERR
#else
      INTEGER LEN
#endif
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_AALL)=CMCAL(IPAR_AALL)+1.0D0
      TIM1=TIMEF()
      IF(GROUP.EQ.1) THEN
        GID=ALLGRP
        NN=NPROC
      ELSEIF(GROUP.EQ.2) THEN
        GID=MEOGRP
        NN=NOGRP
      ELSEIF(GROUP.EQ.3) THEN
        GID=MEPGRP
        NN=NPGRP
      ENDIF
#if MP_LIBRARY == __MPI
      CALL MPI_ALLTOALL(OUTMSG,BLKLEN,MPI_BYTE,INMSG,BLKLEN,
     *                    MPI_BYTE,GID,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_TRANS','IERR.NE.0')
#else
      LEN = BLKLEN/(8/IRAT)
      CALL ICOPY(LEN,OUTMSG(1),1,INMSG(1),1)
#endif
      TIM2=TIMEF()
      CMTIM(IPAR_AALL)=CMTIM(IPAR_AALL)+TIM2-TIM1
      CMLEN(IPAR_AALL)=CMLEN(IPAR_AALL)+BLKLEN*NN
C     ==--------------------------------------------------------------==
      RETURN
      END
#endif
C     ==================================================================
      SUBROUTINE MY_CART(NOGRP,NPGRP,NOLIST,NPLIST,MEOGRP,MEPGRP)
C     ==--------------------------------------------------------------==
C     == Forms cartesian groups of processors                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER NOGRP,NPGRP,NOLIST(*),NPLIST(*),MEOGRP,MEPGRP,OGR,PGR
      INTEGER WORLD
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER   CART_GRP,I,IO,NP(2),IERR,IOO
      LOGICAL   PERIOD(2),ORDER,S_COL(2),S_ROW(2)
C     ==--------------------------------------------------------------==
      ORDER=.TRUE.
      PERIOD(1)=.FALSE.
      PERIOD(2)=.FALSE.
      S_COL(2)=.TRUE.
      S_COL(1)=.FALSE.
      S_ROW(1)=.TRUE.
      S_ROW(2)=.FALSE.
      NP(1)=NPGRP
      NP(2)=NOGRP
      CALL MPI_CART_CREATE(MPI_COMM_WORLD,2,NP,PERIOD,ORDER,
     .                     CART_GRP,IERR)
      CALL MPI_CART_SUB(CART_GRP,S_ROW,MEPGRP,IERR)
      CALL MPI_CART_SUB(CART_GRP,S_COL,MEOGRP,IERR)
      CALL MPI_COMM_GROUP(MPI_COMM_WORLD,WORLD,IERR)
      CALL MPI_COMM_GROUP(MEPGRP,PGR,IERR)
      CALL MPI_COMM_GROUP(MEOGRP,OGR,IERR)
      DO I=1,NPGRP
        CALL MPI_CART_RANK(MEPGRP,I-1,IOO,IERR)
        CALL MPI_GROUP_TRANSLATE_RANKS(PGR,1,IOO,WORLD,IO,IERR)
        NPLIST(I)=IO
      ENDDO
      DO I=1,NOGRP
        CALL MPI_CART_RANK(MEOGRP,I-1,IOO,IERR)
        CALL MPI_GROUP_TRANSLATE_RANKS(OGR,1,IOO,WORLD,IO,IERR)
        NOLIST(I)=IO
      ENDDO
      IF(IERR.NE.0) CALL STOPGM('MY_CART','IERR.NE.0')
#else
C     ==--------------------------------------------------------------==
      MEOGRP=0
      MEPGRP=0
#endif
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_COMBINE(OUTMSG,INMSG,MSGLEN,IFUNC,GID)
C     ==--------------------------------------------------------------==
C     == Combine array INMSG into OUTMSG with an operation given by:  ==
C     ==    IFUNC=1  Global summation over double precision array     ==
C     ==    IFUNC=2  Global summation over integer                    ==
C     ==    IFUNC=3  Maximum over double precsion array               ==
C     ==    IFUNC=4  Absolute values over double precision array      ==
C     ==    IFUNC=5  Global multiplication over double precision array==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'pstat.inc'
C     Arguments
      INTEGER MSGLEN,OUTMSG(MSGLEN),INMSG(MSGLEN),IFUNC,GID
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
      INCLUDE 'irat.inc'
C     Variables
      INTEGER LEN,IERR
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      IF(IFUNC.EQ.5) THEN
        CMCAL(IPAR_GMUL)=CMCAL(IPAR_GMUL)+1.0D0
        CMLEN(IPAR_GMUL)=CMLEN(IPAR_GMUL)+MSGLEN
      ELSE
        CMCAL(IPAR_GSUM)=CMCAL(IPAR_GSUM)+1.0D0
        CMLEN(IPAR_GSUM)=CMLEN(IPAR_GSUM)+MSGLEN
      ENDIF
      TIM1=TIMEF()
      IF(IFUNC.EQ.1) THEN
        LEN=MSGLEN/8
#if defined(CRAY) && !defined(CRAY32)
        CALL MPI_ALLREDUCE(OUTMSG,INMSG,LEN,MPI_REAL,
     *             MPI_SUM,GID,IERR)
#elif __NEC
        CALL MPI_ALLREDUCE(OUTMSG,INMSG,LEN,MPI_REAL,
     *             MPI_SUM,GID,IERR)
#else
        CALL MPI_ALLREDUCE(OUTMSG,INMSG,LEN,MPI_DOUBLE_PRECISION,
     *             MPI_SUM,GID,IERR)
#endif
      ELSEIF(IFUNC.EQ.2) THEN
        LEN=MSGLEN/(8/IRAT)
        CALL MPI_ALLREDUCE(OUTMSG,INMSG,LEN,MPI_INTEGER,MPI_SUM,
     *                     GID,IERR)
      ELSEIF(IFUNC.EQ.3) THEN
        LEN=MSGLEN/8
        CALL MPIABS(OUTMSG,LEN)
        CALL MPI_ALLREDUCE(OUTMSG,INMSG,LEN,MPI_DOUBLE_PRECISION,
     *             MPI_MAX,GID,IERR)
      ELSEIF(IFUNC.EQ.4) THEN
        LEN=MSGLEN/8
        CALL MPIABS(OUTMSG,LEN)
#if defined(CRAY) && !defined(CRAY32)
        CALL MPI_ALLREDUCE(OUTMSG,INMSG,LEN,MPI_REAL,
     *             MPI_MIN,GID,IERR)
#elif __NEC
        CALL MPI_ALLREDUCE(OUTMSG,INMSG,LEN,MPI_REAL,
     *             MPI_MIN,GID,IERR)
#else
        CALL MPI_ALLREDUCE(OUTMSG,INMSG,LEN,MPI_DOUBLE_PRECISION,
     *             MPI_MIN,GID,IERR)
#endif
        ELSEIF(IFUNC.EQ.5) THEN
        LEN=MSGLEN/8
#if defined(CRAY) && !defined(CRAY32)
        CALL MPI_ALLREDUCE(OUTMSG,INMSG,LEN,MPI_REAL,
     *             MPI_PROD,GID,IERR)
#elif __NEC
        CALL MPI_ALLREDUCE(OUTMSG,INMSG,LEN,MPI_REAL,
     *             MPI_PROD,GID,IERR)
#else
        CALL MPI_ALLREDUCE(OUTMSG,INMSG,LEN,MPI_DOUBLE_PRECISION,
     *             MPI_PROD,GID,IERR)
#endif
      ELSE
        CALL STOPGM('MY_COMBINE','NOT AVAILABLE')
      ENDIF
      IF(IERR.NE.0) CALL STOPGM('MY_COMBINE','IERR.NE.0')
#else
      INCLUDE 'irat.inc'
C     Variables
      INTEGER LEN
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      IF(IFUNC.EQ.5) THEN
        CMCAL(IPAR_GMUL)=CMCAL(IPAR_GMUL)+1.0D0
        CMLEN(IPAR_GMUL)=CMLEN(IPAR_GMUL)+MSGLEN
      ELSE
        CMCAL(IPAR_GSUM)=CMCAL(IPAR_GSUM)+1.0D0
        CMLEN(IPAR_GSUM)=CMLEN(IPAR_GSUM)+MSGLEN
      ENDIF
      TIM1=TIMEF()
      LEN=MSGLEN/(8/IRAT)
      IF(IFUNC.EQ.1) THEN
        CALL ICOPY(LEN,OUTMSG,1,INMSG,1)
      ELSEIF(IFUNC.EQ.2) THEN
        CALL ICOPY(LEN,OUTMSG,1,INMSG,1)
      ELSEIF(IFUNC.EQ.3) THEN
        CALL ICOPY(LEN,OUTMSG,1,INMSG,1)
      ELSEIF(IFUNC.EQ.4) THEN
        CALL ICOPY(LEN,OUTMSG,1,INMSG,1)
      ELSEIF(IFUNC.EQ.5) THEN
        CALL ICOPY(LEN,OUTMSG,1,INMSG,1)
      ELSE
        CALL STOPGM('MY_COMBINE','NOT AVAILABLE')
      ENDIF
#endif
      TIM2=TIMEF()
      IF(IFUNC.EQ.5) THEN
        CMTIM(IPAR_GMUL)=CMTIM(IPAR_GMUL)+TIM2-TIM1
      ELSE
        CMTIM(IPAR_GSUM)=CMTIM(IPAR_GSUM)+TIM2-TIM1
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_COMBINE_SOURCE(OUTMSG,INMSG,MSGLEN,IFUNC,ROOT,GID)
C     ==--------------------------------------------------------------==
C     == Combine array INMSG into OUTMSG with an operation given by:  ==
C     ==    IFUNC=1  Global summation over double precision array     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'pstat.inc'
C     Arguments
      INTEGER MSGLEN,OUTMSG(MSGLEN),INMSG(MSGLEN),IFUNC,GID,ROOT
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
      INCLUDE 'irat.inc'
C     Variables
      INTEGER LEN,IERR
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_GSUM)=CMCAL(IPAR_GSUM)+1.0D0
      CMLEN(IPAR_GSUM)=CMLEN(IPAR_GSUM)+MSGLEN
      TIM1=TIMEF()
      IF(IFUNC.EQ.1) THEN
        LEN=MSGLEN/8
#if defined(CRAY) && !defined(CRAY32)
        CALL MPI_REDUCE(OUTMSG,INMSG,LEN,MPI_REAL,
     *             MPI_SUM,ROOT,GID,IERR)
#elif __NEC
        CALL MPI_REDUCE(OUTMSG,INMSG,LEN,MPI_REAL,
     *             MPI_SUM,ROOT,GID,IERR)
#else
        CALL MPI_REDUCE(OUTMSG,INMSG,LEN,MPI_DOUBLE_PRECISION,
     *             MPI_SUM,ROOT,GID,IERR)
#endif
      ELSE
        CALL STOPGM('MY_COMBINE','NOT AVAILABLE')
      ENDIF
      IF(IERR.NE.0) CALL STOPGM('MY_COMBINE','IERR.NE.0')
#else
      INCLUDE 'irat.inc'
C     Variables
      INTEGER LEN
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_GSUM)=CMCAL(IPAR_GSUM)+1.0D0
      CMLEN(IPAR_GSUM)=CMLEN(IPAR_GSUM)+MSGLEN
      TIM1=TIMEF()
      LEN=MSGLEN/(8/IRAT)
      IF(IFUNC.EQ.1) THEN
        CALL ICOPY(LEN,OUTMSG,1,INMSG,1)
      ELSE
        CALL STOPGM('MY_COMBINE','NOT AVAILABLE')
      ENDIF
#endif
      TIM2=TIMEF()
      CMTIM(IPAR_GSUM)=CMTIM(IPAR_GSUM)+TIM2-TIM1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#ifdef  _BGL  
C     ==================================================================
      SUBROUTINE MY_SHIFT(MSEND,MRECV,MSGLEN,MEP,IP,GID)
      IMPLICIT REAL*8(A-H,O-Z)
      INTEGER MSGLEN,MEP,IP,GID
      REAL*8 MSEND(*),MRECV(*)
      INCLUDE 'system.h'
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
      INTEGER STATUS(MPI_STATUS_SIZE)
       CALL TISET('  MY_SHIFT',ISUB)
      IF(GID.NE.ALLGRP) CALL STOPGM('MY_SHIFT','NOT IMPLEMENTED')
      IPME=PGROUP(MEPOS+1)
      IPP=MOD(IPME+IP+NPROC,NPROC)
      IPM=MOD(IPME-IP+NPROC,NPROC)
      IPSEND=PGROUP(IPP+1)
      IPRECV=PGROUP(IPM+1)
      ITYPE=1
      IREQUEST=1
      CALL MPI_SENDRECV(MSEND,MSGLEN,MPI_BYTE,IPSEND,ITYPE,
     *                  MRECV,MSGLEN,MPI_BYTE,IPRECV,ITYPE,
     *                  MPI_COMM_WORLD,STATUS,IERR)
      IF(IERR.NE.0) CALL STOPGM('MPI_SENDRECV','IERR.NE.0')
      CALL TIHALT('  MY_SHIFT',ISUB)
#else
      CALL STOPGM('MY_SHIFT','NOT INSTALLED')
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#else
      SUBROUTINE MY_SHIFT(MSEND,MRECV,MSGLEN,MEP,IP,GID)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER MSGLEN,MEP,IP,GID
      REAL*8  MSEND(*),MRECV(*)
#if MP_LIBRARY == __MPI
      INCLUDE 'system.h'
      INCLUDE 'pstat.inc'
      INCLUDE 'mpif.h'
C     Variables
      INTEGER STATUS(MPI_STATUS_SIZE),IPSEND,IPRECV,
     &        whoami,howmany,
     &        ITYPE,IREQUEST,IERR
C     ==--------------------------------------------------------------==
      call mpi_comm_rank(gid,whoami,ierr)
      if (ierr.ne.0) call stopgm('MY_SHIFT','(mpi/rank) wrong group id')
      call mpi_comm_size(gid,howmany,ierr)
      if (ierr.ne.0) call stopgm('MY_SHIFT','(mpi/size) wrong group id')
      ipsend=mod(whoami+ip+howmany,howmany)
      iprecv=mod(whoami-ip+howmany,howmany)

      CMCAL(IPAR_SEND)=CMCAL(IPAR_SEND)+1.0D0
      CMLEN(IPAR_SEND)=CMLEN(IPAR_SEND)+MSGLEN
      ITYPE=1
      IREQUEST=1
      CALL MPI_ISEND(MSEND,MSGLEN,MPI_BYTE,IPSEND,ITYPE,
     &              gid,IREQUEST,IERR)
      if (ierr.ne.0) call stopgm('MY_SHIFT','(mpi/send) error...')
      CALL MPI_RECV(MRECV,MSGLEN,MPI_BYTE,IPRECV,ITYPE,
     &              gid,STATUS,IERR)
      if (ierr.ne.0) call stopgm('MY_SHIFT','(mpi/recv) error...')
      CALL MPI_WAIT(IREQUEST,STATUS,IERR)
      if (ierr.ne.0) call stopgm('MY_SHIFT','(mpi/wait) error...')
#else
      CALL STOPGM('MY_SHIFT','NOT INSTALLED')
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#endif

      SUBROUTINE MPIABS(A,L)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER L
      REAL*8  A(L)
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
      DO I=1,L
        A(I)=ABS(A(I))
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MY_SENDRECV(SBUF,SCOUNT,DEST,RBUF,RCOUNT,SOURCE,GID)
C     ==--------------------------------------------------------------==
C     == Send/Receive message                                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'pstat.inc'
C     Arguments
      INTEGER SCOUNT,RCOUNT,DEST,SOURCE,GID
      INTEGER SBUF(*),RBUF(*)
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER IERR,STAG,RTAG,STATUS(MPI_STATUS_SIZE)
      REAL*8  TIM1,TIM2,TIMEF
      EXTERNAL TIMEF
C     ==--------------------------------------------------------------==
      CMCAL(IPAR_SEND)=CMCAL(IPAR_SEND)+1.0D0
      CMLEN(IPAR_SEND)=CMLEN(IPAR_SEND)+SCOUNT
      TIM1=TIMEF()
      STAG=1
      RTAG=1
      CALL MPI_SENDRECV(SBUF,SCOUNT,MPI_BYTE,DEST,STAG,
     *                  RBUF,RCOUNT,MPI_BYTE,SOURCE,RTAG,GID,
     *                  STATUS,IERR)
      IF(IERR.NE.0) CALL STOPGM('MY_SENDRECV','IERR.NE.0')
      TIM2=TIMEF()
      CMTIM(IPAR_SEND)=CMTIM(IPAR_SEND)+TIM2-TIM1
#else
      CALL STOPGM("MY_SENDRECV","Not implemented")
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE MY_SPLIT(OLDCOMM,COLOR,KEY,GID,NEWRANK)
C     ==--------------------------------------------------------------==
C     == Splits processors into groups                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER OLDCOMM,COLOR,KEY,LABEL,GID,NEWRANK
C LABEL is not used !
#if MP_LIBRARY == __MPI
      INCLUDE 'mpif.h'
C     Variables
      INTEGER IERR
      CHARACTER STROUT*60
C     ==--------------------------------------------------------------==
      CALL MPI_COMM_SPLIT(OLDCOMM,COLOR,KEY,GID,IERR)
      IF(IERR.NE.0) THEN
!        CALL STR4STOPGM('MPI_SPLIT: IERR.EQ.',IERR,STROUT)
        CALL STOPGM('MY_SPLIT',STROUT)
      END IF

      CALL MPI_COMM_RANK(GID,NEWRANK,IERR)
      IF(IERR.NE.0) THEN
!        CALL STR4STOPGM('MPI_RANK: IERR.EQ.',IERR,STROUT)
        CALL STOPGM('MY_SPLIT',STROUT)
      END IF
#else
C     ==--------------------------------------------------------------==
      GID=0
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
