#ifndef __VECTOR
C     ==================================================================
      SUBROUTINE LYPUU(NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION,EEXCU,FLOP)
C     ==--------------------------------------------------------------==
c adds unpolarized exc potential to uion and sums exc energy
c written by S. Goedecker, Stuttgart, march 1996
c modified by J. Hutter, Stuttgart, may/august 1996
c non-gradient part of Lee,Yang and Parr functional (non-spin polarised)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (F1 = -1.10783814957303361D0)
      PARAMETER (THIRD=1.D0/3.D0,F43=4.D0/3.D0)
      PARAMETER (A=0.04918D0,B=0.132D0,C=0.2533D0,D=0.349D0)
      PARAMETER (CF=2.87123400018819108D0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1,KM2,KM3),UION(2,KM1,KM2,KM3)
      INTEGER I1,I2,I3,II2,II3,IC,NCOUNT  
      REAL*8 FA1,FA2,BCF,EEXCU,X1,X2,X3,X4,RHOU1,RHOU2,RHOU3,RHOU4
     .      ,D1,D2,D3,D4,RSX1,RSX2,RSX3,RSX4,ECRS1,ECRS2,ECRS3,ECRS4
     .      ,OX1,OX2,OX3,OX4,ELYP1,ELYP2,ELYP3,ELYP4,EPSXCU1,EPSXCU2
     .      ,EPSXCU3,EPSXCU4,VXC1,VXC2,VXC3,VXC4
C     ==--------------------------------------------------------------==
      FA1=F1*2.D0/3.D0
      FA2=FA1*F43
      BCF=B*Cf
      EEXCU=0.D0
      X1=1.D0
      X2=1.D0
      X3=1.D0
      X4=1.D0
      IC=0
      II3=MOD(NR3,2)
      II2=MOD(NR2,2)
      IF(II3.NE.0.OR.II2.NE.0) CALL STOPGM('LYPUU','ODD DIMENSION')
C$OMP parallel do private(I1,I2,I3,RHOU1,RHOU2,RHOU3,RHOU4,D1,D2,D3,D4)
C$OMP+ firstprivate(X1,X2,X3,X4)
C$OMP+ private(NCOUNT,RSX1,RSX2,RSX3,RSX4,ECRS1,ECRS2,ECRS3,ECRS4)
C$OMP+ private(OX1,OX2,OX3,OX4,ELYP1,ELYP2,ELYP3,ELYP4)
C$OMP+ private(EPSXCU1,EPSXCU2,EPSXCU3,EPSXCU4)
C$OMP+ private(VXC1,VXC2,VXC3,VXC4)
C$OMP+ reduction(+:EEXCU,IC)
C..loops are four times unrolled
      DO I3=1,NR3,2
        DO I2=1,NR2,2
          DO I1=1,NR1
            RHOU1=DMAX1(RHO(I1,I2+0,I3+0),SMALL)
            RHOU2=DMAX1(RHO(I1,I2+1,I3+0),SMALL)
            RHOU3=DMAX1(RHO(I1,I2+0,I3+1),SMALL)
            RHOU4=DMAX1(RHO(I1,I2+1,I3+1),SMALL)
C..iterative calculation of rho**(1/3) (using Newton method)
            DO NCOUNT=1,100
              D1=X1-RHOU1/(X1*X1)
              D2=X2-RHOU2/(X2*X2)
              D3=X3-RHOU3/(X3*X3)
              D4=X4-RHOU4/(X4*X4)
              IC=IC+1
              X1=X1-.333333333333333D0*D1
              X2=X2-.333333333333333D0*D2
              X3=X3-.333333333333333D0*D3
              X4=X4-.333333333333333D0*D4
              IF((D1*D1+D2*D2+D3*D3+D4*D4).LT.EPS2) GOTO 10
            ENDDO
            X1=RHOU1**(1.D0/3.D0)
            X2=RHOU2**(1.D0/3.d0)
            X3=RHOU3**(1.d0/3.d0)
            X4=RHOU4**(1.d0/3.d0)
   10       CONTINUE
            RSX1=1.D0/X1
            RSX2=1.D0/X2
            RSX3=1.D0/X3
            RSX4=1.D0/X4
c
            ECRS1=BCF*DEXP(-C*RSX1)
            ECRS2=BCF*DEXP(-C*RSX2)
            ECRS3=BCF*DEXP(-C*RSX3)
            ECRS4=BCF*DEXP(-C*RSX4)
            OX1=1.D0/(1.D0+D*RSX1)
            OX2=1.D0/(1.D0+D*RSX2)
            OX3=1.D0/(1.D0+D*RSX3)
            OX4=1.D0/(1.D0+D*RSX4)
            ELYP1=-A*OX1*(1.D0+ECRS1)
            ELYP2=-A*OX2*(1.D0+ECRS2)
            ELYP3=-A*OX3*(1.D0+ECRS3)
            ELYP4=-A*OX4*(1.D0+ECRS4)
            EPSXCU1=FA1*X1+ELYP1
            EPSXCU2=FA1*X2+ELYP2
            EPSXCU3=FA1*X3+ELYP3
            EPSXCU4=FA1*X4+ELYP4
            VXC1=FA2*X1+ELYP1-0.33333333333333D0*RSX1*A*OX1*
     *                               (D*OX1+ECRS1*(D*OX1+C))
            VXC2=FA2*X2+ELYP2-0.33333333333333D0*RSX2*A*OX2*
     *                               (D*OX2+ECRS2*(D*OX2+C))
            VXC3=FA2*X3+ELYP3-0.33333333333333D0*RSX3*A*OX3*
     *                               (D*OX3+ECRS3*(D*OX3+C))
            VXC4=FA2*X4+ELYP4-0.33333333333333D0*RSX4*A*OX4*
     *                               (D*OX4+ECRS4*(D*OX4+C))
            EEXCU=EEXCU+EPSXCU1*RHOU1
            UION(1,I1,I2+0,I3+0)=UION(1,I1,I2+0,I3+0)+VXC1
            EEXCU=EEXCU+EPSXCU2*RHOU2
            UION(1,I1,I2+1,I3+0)=UION(1,I1,I2+1,I3+0)+VXC2
            EEXCU=EEXCU+EPSXCU3*RHOU3
            UION(1,I1,I2+0,I3+1)=UION(1,I1,I2+0,I3+1)+VXC3
            EEXCU=EEXCU+EPSXCU4*RHOU4
            UION(1,I1,I2+1,I3+1)=UION(1,I1,I2+1,I3+1)+VXC4
          ENDDO
        ENDDO
      ENDDO
C..counting floating point operations
      FADD   = IC * 11.D0 + (NR1*NR2*NR3) *  9.D0
      FMULT  = IC * 12.D0 + (NR1*NR2*NR3) * 15.D0
      FDIV   = IC *  4.D0 + (NR1*NR2*NR3) *  2.D0
      FSPEZ  =              (NR1*NR2*NR3) *  1.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE XALPU(SALPHA,NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION,
     *                 EEXCU,FLOP)
C     ==--------------------------------------------------------------==
C..X-Alpha potential               
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (F1 = -1.10783814957303361D0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1,KM2,KM3),UION(2,KM1,KM2,KM3)
      INTEGER I1,I2,I3,II2,II3,IC,NCOUNT
      REAL*8 FA1,FA2,EEXCU,X1,X2,X3,X4,RHOU1,RHOU2,RHOU3,RHOU4
     .      ,D1,D2,D3,D4,EPSXCU1,EPSXCU2,EPSXCU3,EPSXCU4
     .      ,VXC1,VXC2,VXC3,VXC4
C     ==--------------------------------------------------------------==
      FA1=F1*SALPHA
      FA2=FA1*4.D0/3.0D0
      EEXCU=0.D0
      X1=1.D0
      X2=1.D0
      X3=1.D0
      X4=1.D0
      IC=0
      II3=MOD(NR3,2)
      II2=MOD(NR2,2)
      IF(II3.NE.0.OR.II2.NE.0) CALL STOPGM('XALPU','ODD DIMENSION')
C$OMP parallel do private(I1,I2,I3,RHOU1,RHOU2,RHOU3,RHOU4,D1,D2,D3,D4)
C$OMP+ firstprivate(X1,X2,X3,X4)
C$OMP+ private(NCOUNT,EPSXCU1,EPSXCU2,EPSXCU3,EPSXCU4)
C$OMP+ private(VXC1,VXC2,VXC3,VXC4)
C$OMP+ reduction(+:EEXCU,IC)
C..loops are four times unrolled
      DO I3=1,NR3,2
        DO I2=1,NR2,2
          DO I1=1,NR1
            RHOU1=DMAX1(RHO(I1,I2+0,I3+0),SMALL)
            RHOU2=DMAX1(RHO(I1,I2+1,I3+0),SMALL)
            RHOU3=DMAX1(RHO(I1,I2+0,I3+1),SMALL)
            RHOU4=DMAX1(RHO(I1,I2+1,I3+1),SMALL)
C..iterative calculation of rho**(1/3) (using Newton method)
            DO NCOUNT=1,100
              D1=X1-RHOU1/(X1*X1)
              D2=X2-RHOU2/(X2*X2)
              D3=X3-RHOU3/(X3*X3)
              D4=X4-RHOU4/(X4*X4)
              IC=IC+1
              X1=X1-.333333333333333D0*D1
              X2=X2-.333333333333333D0*D2
              X3=X3-.333333333333333D0*D3
              X4=X4-.333333333333333D0*D4
              IF((D1*D1+D2*D2+D3*D3+D4*D4).LT.EPS2) GOTO 10
            ENDDO
            X1=RHOU1**(1.D0/3.D0)
            X2=RHOU2**(1.D0/3.d0)
            X3=RHOU3**(1.d0/3.d0)
            X4=RHOU4**(1.d0/3.d0)
   10       CONTINUE
            EPSXCU1=FA1*X1
            EPSXCU2=FA1*X2
            EPSXCU3=FA1*X3
            EPSXCU4=FA1*X4
            VXC1=FA2*X1
            VXC2=FA2*X2
            VXC3=FA2*X3
            VXC4=FA2*X4
            EEXCU=EEXCU+EPSXCU1*RHOU1
            UION(1,I1,I2+0,I3+0)=UION(1,I1,I2+0,I3+0)+VXC1
            EEXCU=EEXCU+EPSXCU2*RHOU2
            UION(1,I1,I2+1,I3+0)=UION(1,I1,I2+1,I3+0)+VXC2
            EEXCU=EEXCU+EPSXCU3*RHOU3
            UION(1,I1,I2+0,I3+1)=UION(1,I1,I2+0,I3+1)+VXC3
            EEXCU=EEXCU+EPSXCU4*RHOU4
            UION(1,I1,I2+1,I3+1)=UION(1,I1,I2+1,I3+1)+VXC4
          ENDDO
        ENDDO
      ENDDO
C..counting floating point operations
      FADD   = IC * 11.D0 + (NR1*NR2*NR3) *  2.D0
      FMULT  = IC * 12.D0 + (NR1*NR2*NR3) *  3.D0
      FDIV   = IC *  4.D0 + (NR1*NR2*NR3) *  0.D0
      FSPEZ  =              (NR1*NR2*NR3) *  0.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MIKEU(NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION,EEXCU,FLOP)
C     ==--------------------------------------------------------------==
C..Pade approximation to LDA functional
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A0U=.4581652932831429D0,
     *           A1U=2.217058676663745D0,
     *           A2U=0.7405551735357053D0,
     *           A3U=0.01968227878617998D0)
      PARAMETER (B1U=1.0D0,
     *           B2U=4.504130959426697D0,
     *           B3U=1.110667363742916D0,
     *           B4U=0.02359291751427506D0)
      PARAMETER
     *  (C1U=4.D0*A0U*B1U/3.D0,C2U=5.D0*A0U*B2U/3.D0+A1U*B1U,
     *   C3U=2.D0*A0U*B3U+4.D0*A1U*B2U/3.D0+2.D0*A2U*B1U/3.D0,
     *   C4U=7.D0*A0U*B4U/3.D0+5.D0*A1U*B3U/3.D0+A2U*B2U+A3U*B1U/3.D0,
     *   C5U=2.D0*A1U*B4U+4.D0*A2U*B3U/3.D0+2.D0*A3U*B2U/3.D0,
     *   C6U=5.D0*A2U*B4U/3.D0+A3U*B3U,C7U=4.D0*A3U*B4U/3.D0)
      PARAMETER (RSFAC=.6203504908994000D0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1,KM2,KM3),UION(2,KM1,KM2,KM3)
      INTEGER I1,I2,I3,II2,II3,IC,NCOUNT
      REAL*8 EEXCU,X1,X2,X3,X4,RHOU1,RHOU2,RHOU3,RHOU4,D1,D2,D3,D4
     .      ,RSU1,RSU2,RSU3,RSU4,TOPU1,TOPU2,TOPU3,TOPU4,DTOPU1,DTOPU2
     .      ,DTOPU3,DTOPU4,BOTU1,BOTU2,BOTU3,BOTU4,EPSXCU1,EPSXCU2
     .      ,EPSXCU3,EPSXCU4,VXC1,VXC2,VXC3,VXC4
C     ==--------------------------------------------------------------==
      EEXCU=0.D0
      X1=1.D0
      X2=1.D0
      X3=1.D0
      X4=1.D0
      IC=0
      II3=MOD(NR3,2)
      II2=MOD(NR2,2)
      IF(II3.NE.0.OR.II2.NE.0) CALL STOPGM('MIKEU','ODD DIMENSION')
C..loops are four times unrolled
C$OMP parallel do private(I1,I2,I3,RHOU1,RHOU2,RHOU3,RHOU4,D1,D2,D3,D4)
C$OMP+ firstprivate(X1,X2,X3,X4)
C$OMP+ private(NCOUNT,RSU1,RSU2,RSU3,RSU4,TOPU1,TOPU2,TOPU3,TOPU4)
C$OMP+ private(DTOPU1,DTOPU2,DTOPU3,DTOPU4,BOTU1,BOTU2,BOTU3,BOTU4)
C$OMP+ private(EPSXCU1,EPSXCU2,EPSXCU3,EPSXCU4,VXC1,VXC2,VXC3,VXC4)
C$OMP+ reduction(+:EEXCU,IC)
      DO I3=1,NR3,2
        DO I2=1,NR2,2
          DO I1=1,NR1
            RHOU1=DMAX1(RHO(I1,I2+0,I3+0),SMALL)
            RHOU2=DMAX1(RHO(I1,I2+1,I3+0),SMALL)
            RHOU3=DMAX1(RHO(I1,I2+0,I3+1),SMALL)
            RHOU4=DMAX1(RHO(I1,I2+1,I3+1),SMALL)
C..iterative calculation of rho**(1/3) (using Newton method)
            DO NCOUNT=1,100
              D1=X1-RHOU1/(X1*X1)
              D2=X2-RHOU2/(X2*X2)
              D3=X3-RHOU3/(X3*X3)
              D4=X4-RHOU4/(X4*X4)
              IC=IC+1
              X1=X1-.333333333333333D0*D1
              X2=X2-.333333333333333D0*D2
              X3=X3-.333333333333333D0*D3
              X4=X4-.333333333333333D0*D4
              IF((D1*D1+D2*D2+D3*D3+D4*D4).LT.EPS2) GOTO 10
            ENDDO
            X1=RHOU1**(1.D0/3.D0)
            X2=RHOU2**(1.D0/3.d0)
            X3=RHOU3**(1.d0/3.d0)
            X4=RHOU4**(1.d0/3.d0)
   10       CONTINUE
            RSU1=RSFAC/X1
            RSU2=RSFAC/X2
            RSU3=RSFAC/X3
            RSU4=RSFAC/X4
            TOPU1=A2U+RSU1*A3U
            TOPU2=A2U+RSU2*A3U
            TOPU3=A2U+RSU3*A3U
            TOPU4=A2U+RSU4*A3U
            TOPU1=A1U+RSU1*TOPU1
            TOPU2=A1U+RSU2*TOPU2
            TOPU3=A1U+RSU3*TOPU3
            TOPU4=A1U+RSU4*TOPU4
            TOPU1=A0U+RSU1*TOPU1
            TOPU2=A0U+RSU2*TOPU2
            TOPU3=A0U+RSU3*TOPU3
            TOPU4=A0U+RSU4*TOPU4
            DTOPU1=C6U+RSU1*C7U
            DTOPU2=C6U+RSU2*C7U
            DTOPU3=C6U+RSU3*C7U
            DTOPU4=C6U+RSU4*C7U
            DTOPU1=C5U+RSU1*DTOPU1
            DTOPU2=C5U+RSU2*DTOPU2
            DTOPU3=C5U+RSU3*DTOPU3
            DTOPU4=C5U+RSU4*DTOPU4
            DTOPU1=C4U+RSU1*DTOPU1
            DTOPU2=C4U+RSU2*DTOPU2
            DTOPU3=C4U+RSU3*DTOPU3
            DTOPU4=C4U+RSU4*DTOPU4
            DTOPU1=C3U+RSU1*DTOPU1
            DTOPU2=C3U+RSU2*DTOPU2
            DTOPU3=C3U+RSU3*DTOPU3
            DTOPU4=C3U+RSU4*DTOPU4
            DTOPU1=C2U+RSU1*DTOPU1
            DTOPU2=C2U+RSU2*DTOPU2
            DTOPU3=C2U+RSU3*DTOPU3
            DTOPU4=C2U+RSU4*DTOPU4
            DTOPU1=C1U+RSU1*DTOPU1
            DTOPU2=C1U+RSU2*DTOPU2
            DTOPU3=C1U+RSU3*DTOPU3
            DTOPU4=C1U+RSU4*DTOPU4
            DTOPU1=-RSU1*DTOPU1
            DTOPU2=-RSU2*DTOPU2
            DTOPU3=-RSU3*DTOPU3
            DTOPU4=-RSU4*DTOPU4
            BOTU1=B3U+RSU1*B4U
            BOTU2=B3U+RSU2*B4U
            BOTU3=B3U+RSU3*B4U
            BOTU4=B3U+RSU4*B4U
            BOTU1=B2U+RSU1*BOTU1
            BOTU2=B2U+RSU2*BOTU2
            BOTU3=B2U+RSU3*BOTU3
            BOTU4=B2U+RSU4*BOTU4
            BOTU1=B1U+RSU1*BOTU1
            BOTU2=B1U+RSU2*BOTU2
            BOTU3=B1U+RSU3*BOTU3
            BOTU4=B1U+RSU4*BOTU4
            BOTU1=RSU1*BOTU1
            BOTU2=RSU2*BOTU2
            BOTU3=RSU3*BOTU3
            BOTU4=RSU4*BOTU4
            EPSXCU1=TOPU1*(-1.D0/BOTU1)
            EPSXCU2=TOPU2*(-1.D0/BOTU2)
            EPSXCU3=TOPU3*(-1.D0/BOTU3)
            EPSXCU4=TOPU4*(-1.D0/BOTU4)
            EEXCU=EEXCU+EPSXCU1*RHOU1
            VXC1=DTOPU1*(-1.D0/BOTU1)*(-1.D0/BOTU1)
            UION(1,I1,I2+0,I3+0)=UION(1,I1,I2+0,I3+0)+VXC1
            EEXCU=EEXCU+EPSXCU2*RHOU2
            VXC2=DTOPU2*(-1.D0/BOTU2)*(-1.D0/BOTU2)
            UION(1,I1,I2+1,I3+0)=UION(1,I1,I2+1,I3+0)+VXC2
            EEXCU=EEXCU+EPSXCU3*RHOU3
            VXC3=DTOPU3*(-1.D0/BOTU3)*(-1.D0/BOTU3)
            UION(1,I1,I2+0,I3+1)=UION(1,I1,I2+0,I3+1)+VXC3
            EEXCU=EEXCU+EPSXCU4*RHOU4
            VXC4=DTOPU4*(-1.D0/BOTU4)*(-1.D0/BOTU4)
            UION(1,I1,I2+1,I3+1)=UION(1,I1,I2+1,I3+1)+VXC4
          ENDDO
        ENDDO
      ENDDO
C..counting floating point operations
      FADD   = IC * 11.D0 + (NR1*NR2*NR3) * 14.D0
      FMULT  = IC * 12.D0 + (NR1*NR2*NR3) * 18.D0
      FDIV   = IC *  4.D0 + (NR1*NR2*NR3) *  2.D0
      FSPEZ  =              (NR1*NR2*NR3) *  0.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MIKESP(NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION1,UION2,
     *                  EEXCU,FLOP)
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A0U=.4581652932831429D0,
     *           A1U=2.217058676663745D0,
     *           A2U=0.7405551735357053D0,
     *           A3U=0.01968227878617998d0)
      PARAMETER (B1U=1.0D0,
     *           B2U=4.504130959426697D0,
     *           B3U=1.110667363742916D0,
     *           B4U=0.02359291751427506D0)
      PARAMETER (DA0=.119086804055547D0,
     *           DA1=.6157402568883345D0,
     *           DA2=.1574201515892867D0,
     *           DA3=.003532336663397157d0)
      PARAMETER (DB1=0.0D0,
     *           DB2=.2673612973836267D0,
     *           DB3=.2052004607777787D0,
     *           DB4=.004200005045691381D0)
      PARAMETER (T1=.854960467080683D0,
     *           T2=.07916300621117439D0,
     *           T3=.02580127609845684D0,
     *           T4=.0121839359353824D0,
     *           T5=.006919272259599879D0,
     *           T6=.004391524649611372D0,
     *           T7=.003002751897170168D0,
     *           T8=.00216587382212552D0,
     *           T9=.01141189204579585D0)
      PARAMETER (RSFAC=.6203504908994000D0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1,KM2,KM3,2),UION1(2,KM1,KM2,KM3),
     *          UION2(2,KM1,KM2,KM3)
      INTEGER I1,I2,I3,II2,II3,IC,NCOUNT
      REAL*8 EEXCU,X1,X2,X3,X4,RHOU1,RHOU2,RHOU3,RHOU4,D1,D2,D3,D4
     .      ,RHOA1,RHOA2,RHOA3,RHOA4,RHOB1,RHOB2,RHOB3,RHOB4
     .      ,ETA1,ETA2,ETA3,ETA4,ET1,FXS1,DFXS1,ET2,FXS2
     .      ,DFXS2,ET3,FXS3,DFXS3,ET4,FXS4,DFXS4,A0P1,A1P1
     .      ,A2P1,A3P1,B2P1,B3P1,B4P1,A0P2,A1P2,A2P2,A3P2
     .      ,B2P2,B3P2,B4P2,A0P3,A1P3,A2P3,A3P3,B2P3,B3P3
     .      ,B4P3,A0P4,A1P4,A2P4,A3P4,B2P4,B3P4,B4P4,RSU1
     .      ,RSU2,RSU3,RSU4,TOP1,TOP2,TOP3,TOP4,DTOP1,DTOP2
     .      ,DTOP3,DTOP4,XTOP1,XTOP2,XTOP3,XTOP4,BOT1,BOT2
     .      ,BOT3,BOT4,DBOT1,DBOT2,DBOT3,DBOT4,XBOT1,XBOT2
     .      ,XBOT3,XBOT4,OBOT1,OBOT2,OBOT3,OBOT4,EPSXCU1
     .      ,EPSXCU2,EPSXCU3,EPSXCU4,DVXC1,DXA1,DXB1,DVXC2
     .      ,DXA2,DXB2,DVXC3,DXA3,DXB3,DVXC4,DXA4,DXB4
     .      ,VXC1,VXC2,VXC3,VXC4

C     ==--------------------------------------------------------------==
      EEXCU=0.D0
      X1=1.D0
      X2=1.D0
      X3=1.D0
      X4=1.D0
      IC=0
      II3=MOD(NR3,2)
      II2=MOD(NR2,2)
      IF(II3.NE.0.OR.II2.NE.0) CALL STOPGM('MIKESP','ODD DIMENSION')
C$OMP parallel do private(I1,I2,I3,RHOU1,RHOU2,RHOU3,RHOU4,D1,D2,D3,D4)
C$OMP+ firstprivate(X1,X2,X3,X4)
C$OMP+ private(NCOUNT,RHOA1,RHOA2,RHOA3,RHOA4,RHOB1,RHOB2,RHOB3,RHOB4) 
C$OMP+ private(ETA1,ETA2,ETA3,ETA4,ET1,FXS1,DFXS1,ET2,FXS2)
C$OMP+ private(DFXS2,ET3,FXS3,DFXS3,ET4,FXS4,DFXS4,A0P1,A1P1)
C$OMP+ private(A2P1,A3P1,B2P1,B3P1,B4P1,A0P2,A1P2,A2P2,A3P2)
C$OMP+ private(B2P2,B3P2,B4P2,A0P3,A1P3,A2P3,A3P3,B2P3,B3P3)
C$OMP+ private(B4P3,A0P4,A1P4,A2P4,A3P4,B2P4,B3P4,B4P4,RSU1)
C$OMP+ private(RSU2,RSU3,RSU4,TOP1,TOP2,TOP3,TOP4,DTOP1,DTOP2)
C$OMP+ private(DTOP3,DTOP4,XTOP1,XTOP2,XTOP3,XTOP4,BOT1,BOT2)
C$OMP+ private(BOT3,BOT4,DBOT1,DBOT2,DBOT3,DBOT4,XBOT1,XBOT2)
C$OMP+ private(XBOT3,XBOT4,OBOT1,OBOT2,OBOT3,OBOT4,EPSXCU1)
C$OMP+ private(EPSXCU2,EPSXCU3,EPSXCU4,DVXC1,DXA1,DXB1,DVXC2)
C$OMP+ private(DXA2,DXB2,DVXC3,DXA3,DXB3,DVXC4,DXA4,DXB4)
C$OMP+ private(VXC1,VXC2,VXC3,VXC4)
C$OMP+ private(B1P1,B1P2,B1P3,B1P4) ! bug fix
C$OMP+ reduction(+:EEXCU,IC)
      DO I3=1,NR3,2
        DO I2=1,NR2,2
          DO I1=1,NR1
            RHOA1=DMAX1(RHO(I1,I2+0,I3+0,1)-RHO(I1,I2+0,I3+0,2),SMALL)
            RHOA2=DMAX1(RHO(I1,I2+1,I3+0,1)-RHO(I1,I2+1,I3+0,2),SMALL)
            RHOA3=DMAX1(RHO(I1,I2+0,I3+1,1)-RHO(I1,I2+0,I3+1,2),SMALL)
            RHOA4=DMAX1(RHO(I1,I2+1,I3+1,1)-RHO(I1,I2+1,I3+1,2),SMALL)
            RHOB1=DMAX1(RHO(I1,I2+0,I3+0,2),SMALL)
            RHOB2=DMAX1(RHO(I1,I2+1,I3+0,2),SMALL)
            RHOB3=DMAX1(RHO(I1,I2+0,I3+1,2),SMALL)
            RHOB4=DMAX1(RHO(I1,I2+1,I3+1,2),SMALL)
            RHOU1=RHOA1+RHOB1
            RHOU2=RHOA2+RHOB2
            RHOU3=RHOA3+RHOB3
            RHOU4=RHOA4+RHOB4
            ETA1=(RHOA1-RHOB1)/RHOU1
            ETA2=(RHOA2-RHOB2)/RHOU2
            ETA3=(RHOA3-RHOB3)/RHOU3
            ETA4=(RHOA4-RHOB4)/RHOU4
            ET1=ETA1*ETA1
            FXS1=ET1*(T1+ET1*(T2+ET1*(T3+ET1*(T4+ET1*(T5+ET1*
     *           (T6+ET1*(T7+ET1*(T8+ET1*T9))))))))
            DFXS1=(2.D0*ETA1)*(T1+ET1*(2.D0*T2+ET1*(3.D0*T3+ET1*
     *           (4.D0*T4+ET1*(5.D0*T5+ET1*(6.D0*T6+ET1*(7.D0*T7+ET1*
     *           (8.D0*T8+ET1*9.D0*T9))))))))
            ET2=ETA2*ETA2
            FXS2=ET2*(T1+ET2*(T2+ET2*(T3+ET2*(T4+ET2*(T5+ET2*
     *           (T6+ET2*(T7+ET2*(T8+ET2*T9))))))))
            DFXS2=(2.D0*ETA2)*(T1+ET2*(2.D0*T2+ET2*(3.D0*T3+ET2*
     *           (4.D0*T4+ET2*(5.D0*T5+ET2*(6.D0*T6+ET2*(7.D0*T7+ET2*
     *           (8.D0*T8+ET2*9.D0*T9))))))))
            ET3=ETA3*ETA3
            FXS3=ET3*(T1+ET3*(T2+ET3*(T3+ET3*(T4+ET3*(T5+ET3*
     *           (T6+ET3*(T7+ET3*(T8+ET3*t9))))))))
            DFXS3=(2.D0*ETA3)*(T1+ET3*(2.D0*T2+ET3*(3.D0*T3+ET3*
     *           (4.D0*T4+ET3*(5.D0*T5+ET3*(6.D0*T6+ET3*(7.D0*T7+ET3*
     *           (8.D0*T8+ET3*9.D0*T9))))))))
            ET4=ETA4*ETA4
            FXS4=ET4*(T1+ET4*(T2+ET4*(T3+ET4*(T4+ET4*(T5+ET4*
     *           (T6+ET4*(T7+ET4*(T8+ET4*T9))))))))
            DFXS4=(2.D0*ETA4)*(T1+ET4*(2.D0*T2+ET4*(3.D0*T3+ET4*
     *           (4.D0*T4+ET4*(5.D0*T5+ET4*(6.D0*T6+ET4*(7.D0*T7+ET4*
     *           (8.D0*T8+ET4*9.D0*T9))))))))
C
            A0P1=A0U+FXS1*DA0
            A1P1=A1U+FXS1*DA1
            A2P1=A2U+FXS1*DA2
            A3P1=A3U+FXS1*DA3
            B1P1=B1U+FXS1*DB1
            B2P1=B2U+FXS1*DB2
            B3P1=B3U+FXS1*DB3
            B4P1=B4U+FXS1*DB4
C
            A0P2=A0U+FXS2*DA0
            A1P2=A1U+FXS2*DA1
            A2P2=A2U+FXS2*DA2
            A3P2=A3U+FXS2*DA3
            B1P2=B1U+FXS2*DB1
            B2P2=B2U+FXS2*DB2
            B3P2=B3U+FXS2*DB3
            B4P2=B4U+FXS2*DB4
C
            A0P3=A0U+FXS3*DA0
            A1P3=A1U+FXS3*DA1
            A2P3=A2U+FXS3*DA2
            A3P3=A3U+FXS3*DA3
            B1P3=B1U+FXS3*DB1
            B2P3=B2U+FXS3*DB2
            B3P3=B3U+FXS3*DB3
            B4P3=B4U+FXS3*DB4
C
            A0P4=A0U+FXS4*DA0
            A1P4=A1U+FXS4*DA1
            A2P4=A2U+FXS4*DA2
            A3P4=A3U+FXS4*DA3
            B1P4=B1U+FXS4*DB1
            B2P4=B2U+FXS4*DB2
            B3P4=B3U+FXS4*DB3
            B4P4=B4U+FXS4*DB4
C
C..iterative calculation of rho**(1/3) (using Newton method)
            DO NCOUNT=1,100
              D1=X1-RHOU1/(X1*X1)
              D2=X2-RHOU2/(X2*X2)
              D3=X3-RHOU3/(X3*X3)
              D4=X4-RHOU4/(X4*X4)
              IC=IC+1
              X1=X1-.333333333333333D0*D1
              X2=X2-.333333333333333D0*D2
              X3=X3-.333333333333333D0*D3
              X4=X4-.333333333333333D0*D4
              IF((D1*D1+D2*D2+D3*D3+D4*D4).LT.EPS2) GOTO 10
            ENDDO
            X1=RHOU1**(1.D0/3.D0)
            X2=RHOU2**(1.D0/3.d0)
            X3=RHOU3**(1.d0/3.d0)
            X4=RHOU4**(1.d0/3.d0)
   10       CONTINUE
            RSU1=RSFAC/X1
            RSU2=RSFAC/X2
            RSU3=RSFAC/X3
            RSU4=RSFAC/X4
            TOP1=A0P1+RSU1*(A1P1+RSU1*(A2P1+RSU1*A3P1))
            TOP2=A0P2+RSU2*(A1P2+RSU2*(A2P2+RSU2*A3P2))
            TOP3=A0P3+RSU3*(A1P3+RSU3*(A2P3+RSU3*A3P3))
            TOP4=A0P4+RSU4*(A1P4+RSU4*(A2P4+RSU4*A3P4))
            DTOP1= A1P1+RSU1*(2.D0*A2P1+RSU1*3.D0*A3P1)
            DTOP2= A1P2+RSU2*(2.D0*A2P2+RSU2*3.D0*A3P2)
            DTOP3= A1P3+RSU3*(2.D0*A2P3+RSU3*3.D0*A3P3)
            DTOP4= A1P4+RSU4*(2.D0*A2P4+RSU4*3.D0*A3P4)
            XTOP1=DA0+RSU1*(DA1+RSU1*(DA2+RSU1*DA3))
            XTOP2=DA0+RSU2*(DA1+RSU2*(DA2+RSU2*DA3))
            XTOP3=DA0+RSU3*(DA1+RSU3*(DA2+RSU3*DA3))
            XTOP4=DA0+RSU4*(DA1+RSU4*(DA2+RSU4*DA3))
            BOT1=RSU1*(B1P1+RSU1*(B2P1+RSU1*(B3P1+RSU1*B4P1)))
            BOT2=RSU2*(B1P2+RSU2*(B2P2+RSU2*(B3P2+RSU2*B4P2)))
            BOT3=RSU3*(B1P3+RSU3*(B2P3+RSU3*(B3P3+RSU3*B4P3)))
            BOT4=RSU4*(B1P4+RSU4*(B2P4+RSU4*(B3P4+RSU4*B4P4)))
            DBOT1=B1P1+RSU1*(2.D0*B2P1+RSU1*(3.D0*B3P1+RSU1*4.D0*B4P1))
            DBOT2=B1P2+RSU2*(2.D0*B2P2+RSU2*(3.D0*B3P2+RSU2*4.D0*B4P2))
            DBOT3=B1P3+RSU3*(2.D0*B2P3+RSU3*(3.D0*B3P3+RSU3*4.D0*B4P3))
            DBOT4=B1P4+RSU4*(2.D0*B2P4+RSU4*(3.D0*B3P4+RSU4*4.D0*B4P4))
            XBOT1=RSU1*(DB1+RSU1*(DB2+RSU1*(DB3+RSU1*DB4)))
            XBOT2=RSU2*(DB1+RSU2*(DB2+RSU2*(DB3+RSU2*DB4)))
            XBOT3=RSU3*(DB1+RSU3*(DB2+RSU3*(DB3+RSU3*DB4)))
            XBOT4=RSU4*(DB1+RSU4*(DB2+RSU4*(DB3+RSU4*DB4)))
            OBOT1=1.D0/BOT1
            OBOT2=1.D0/BOT2
            OBOT3=1.D0/BOT3
            OBOT4=1.D0/BOT4
            EPSXCU1=-TOP1*OBOT1
            EPSXCU2=-TOP2*OBOT2
            EPSXCU3=-TOP3*OBOT3
            EPSXCU4=-TOP4*OBOT4
            EEXCU=EEXCU+EPSXCU1*RHOU1
            VXC1=EPSXCU1+RSU1*(DTOP1*OBOT1-DBOT1*(TOP1*OBOT1*OBOT1))*
     *           0.333333333333333D0
            DVXC1=-(XTOP1*OBOT1-XBOT1*(TOP1*OBOT1*OBOT1))
            DXA1=VXC1+DVXC1*DFXS1*(1.D0-ETA1)
            DXB1=VXC1-DVXC1*DFXS1*(1.D0+ETA1)
            UION1(1,I1,I2+0,I3+0)=UION1(1,I1,I2+0,I3+0)+DXA1
            UION2(1,I1,I2+0,I3+0)=UION2(1,I1,I2+0,I3+0)+DXB1
            EEXCU=EEXCU+EPSXCU2*RHOU2
            VXC2=EPSXCU2+RSU2*(DTOP2*OBOT2-DBOT2*(TOP2*OBOT2*OBOT2))*
     *           0.333333333333333D0
            DVXC2=-(XTOP2*OBOT2-XBOT2*(TOP2*OBOT2*OBOT2))
            DXA2=VXC2+DVXC2*DFXS2*(1.D0-ETA2)
            DXB2=VXC2-DVXC2*DFXS2*(1.D0+ETA2)
            UION1(1,I1,I2+1,I3+0)=UION1(1,I1,I2+1,I3+0)+DXA2
            UION2(1,I1,I2+1,I3+0)=UION2(1,I1,I2+1,I3+0)+DXB2
            EEXCU=EEXCU+EPSXCU3*RHOU3
            VXC3=EPSXCU3+RSU3*(DTOP3*OBOT3-DBOT3*(TOP3*OBOT3*OBOT3))*
     *           0.333333333333333D0
            DVXC3=-(XTOP3*OBOT3-XBOT3*(TOP3*OBOT3*OBOT3))
            DXA3=VXC3+DVXC3*DFXS3*(1.D0-ETA3)
            DXB3=VXC3-DVXC3*DFXS3*(1.D0+ETA3)
            UION1(1,I1,I2+0,I3+1)=UION1(1,I1,I2+0,I3+1)+DXA3
            UION2(1,I1,I2+0,I3+1)=UION2(1,I1,I2+0,I3+1)+DXB3
            EEXCU=EEXCU+EPSXCU4*RHOU4
            VXC4=EPSXCU4+RSU4*(DTOP4*OBOT4-DBOT4*(TOP4*OBOT4*OBOT4))*
     *           0.333333333333333D0
            DVXC4=-(XTOP4*OBOT4-XBOT4*(TOP4*OBOT4*OBOT4))
            DXA4=VXC4+DVXC4*DFXS4*(1.D0-ETA4)
            DXB4=VXC4-DVXC4*DFXS4*(1.D0+ETA4)
            UION1(1,I1,I2+1,I3+1)=UION1(1,I1,I2+1,I3+1)+DXA4
            UION2(1,I1,I2+1,I3+1)=UION2(1,I1,I2+1,I3+1)+DXB4
          ENDDO
        ENDDO
      ENDDO
C..counting floating point operations
      FADD   = IC * 11.D0 + (NR1*NR2*NR3) * 53.D0
      FMULT  = IC * 12.D0 + (NR1*NR2*NR3) * 76.D0
      FDIV   = IC *  4.D0 + (NR1*NR2*NR3) *  3.D0
      FSPEZ  =              (NR1*NR2*NR3) *  0.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SLYPSP(NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION1,UION2,SALPHA,
     *                  EEXCU,FLOP)
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A=0.04918D0,B=0.132D0,C=0.2533D0,D=0.349D0,
     *           CF=2.87123400018819108D0)
      PARAMETER (F1 = -1.39578858466194911D0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1,KM2,KM3,2),UION1(2,KM1,KM2,KM3),
     *          UION2(2,KM1,KM2,KM3)
      INTEGER I1,I2,I3,II2,II3,IC,NCOUNT
      REAL*8 CFX,F1S,F1U,EEXCU,X1,X2,X3,X4,Y1,Y2,Y3,Y4
     .      ,RHOA1,RHOA2,RHOA3,RHOA4,RHOB1,RHOB2,RHOB3,RHOB4
     .      ,RHO1,RHO2,RHO3,RHO4,RSA1,RSA2,RSA3,RSA4,EX1,EX2
     .      ,EX3,EX4,VXA1,VXA2,VXA3,VXA4,VXB1,VXB2,VXB3,VXB4
     .      ,DD1,DD2,DD3,DD4,ECR1,ECR2,ECR3,ECR4,OR1,OR2,OR3,OR4
     .      ,DOR1,DOR2,DOR3,DOR4,RHOAT1,RHOAT2,RHOAT3,RHOAT4
     .      ,RHOBT1,RHOBT2,RHOBT3,RHOBT4,EC1,EC2,EC3,EC4
     .      ,VCA1,VCA2,VCA3,VCA4,VCB1,VCB2,VCB3,VCB4
C     ==--------------------------------------------------------------==
      CFX=CF*2.D0**(11.D0/3.D0)
      F1S=F1*SALPHA
      F1U=F1S*1.3333333333333333d0
      EEXCU=0.D0
      X1=1.D0
      X2=1.D0
      X3=1.D0
      X4=1.D0
      Y1=1.D0
      Y2=1.D0
      Y3=1.D0
      Y4=1.D0
      IC=0
      II3=MOD(NR3,2)
      II2=MOD(NR2,2)
      IF(II3.NE.0.OR.II2.NE.0) CALL STOPGM('SLYPSP','ODD DIMENSION')
C$OMP parallel do private(I1,I2,I3,RHOU1,RHOU2,RHOU3,RHOU4,D1,D2,D3,D4)
C$OMP+ firstprivate(X1,X2,X3,X4,Y1,Y2,Y3,Y4)
C$OMP+ private(RHOA1,RHOA2,RHOA3,RHOA4,RHOB1,RHOB2,RHOB3,RHOB4)
C$OMP+ private(NCOUNT,RHO1,RHO2,RHO3,RHO4,RSA1,RSA2,RSA3,RSA4)
C$OMP+ private(EX1,EX2,EX3,EX4,VXA1,VXA2,VXA3,VXA4,VXB1,VXB2,VXB3,VXB4)
C$OMP+ private(DD1,DD2,DD3,DD4,ECR1,ECR2,ECR3,ECR4,OR1,OR2,OR3,OR4)
C$OMP+ private(DOR1,DOR2,DOR3,DOR4,RHOAT1,RHOAT2,RHOAT3,RHOAT4)
C$OMP+ private(RHOBT1,RHOBT2,RHOBT3,RHOBT4,EC1,EC2,EC3,EC4)
C$OMP+ private(VCA1,VCA2,VCA3,VCA4,VCB1,VCB2,VCB3,VCB4)
C$OMP+ private(RSB1,RSB2,RSB3,RSB4,RS1,RS2,RS3,RS4) ! bug fix
C$OMP+ reduction(+:EEXCU,IC)
      DO I3=1,NR3,2
        DO I2=1,NR2,2
          DO I1=1,NR1
            RHOA1=RHO(I1,I2+0,I3+0,1)-RHO(I1,I2+0,I3+0,2)
            RHOA2=RHO(I1,I2+1,I3+0,1)-RHO(I1,I2+1,I3+0,2)
            RHOA3=RHO(I1,I2+0,I3+1,1)-RHO(I1,I2+0,I3+1,2)
            RHOA4=RHO(I1,I2+1,I3+1,1)-RHO(I1,I2+1,I3+1,2)
            RHOA1=DMAX1(RHOA1,SMALL)
            RHOA2=DMAX1(RHOA2,SMALL)
            RHOA3=DMAX1(RHOA3,SMALL)
            RHOA4=DMAX1(RHOA4,SMALL)
            RHOB1=DMAX1(RHO(I1,I2+0,I3+0,2),SMALL)
            RHOB2=DMAX1(RHO(I1,I2+1,I3+0,2),SMALL)
            RHOB3=DMAX1(RHO(I1,I2+0,I3+1,2),SMALL)
            RHOB4=DMAX1(RHO(I1,I2+1,I3+1,2),SMALL)
            RHO1=RHOA1+RHOB1
            RHO2=RHOA2+RHOB2
            RHO3=RHOA3+RHOB3
            RHO4=RHOA4+RHOB4
C
C..iterative calculation of rho**(1/3) (using Newton method)
            DO NCOUNT=1,100
              D1=X1-RHOA1/(X1*X1)
              D2=X2-RHOA2/(X2*X2)
              D3=X3-RHOA3/(X3*X3)
              D4=X4-RHOA4/(X4*X4)
              IC=IC+1
              X1=X1-.333333333333333D0*D1
              X2=X2-.333333333333333D0*D2
              X3=X3-.333333333333333D0*D3
              X4=X4-.333333333333333D0*D4
              IF((D1*D1+D2*D2+D3*D3+D4*D4).LT.EPS2) GOTO 10
            ENDDO
            X1=RHOA1**(1.D0/3.D0)
            X2=RHOA2**(1.D0/3.D0)
            X3=RHOA3**(1.D0/3.D0)
            X4=RHOA4**(1.D0/3.D0)
   10       CONTINUE
            RSA1=X1
            RSA2=X2
            RSA3=X3
            RSA4=X4
            DO NCOUNT=1,100
              D1=X1-RHOB1/(X1*X1)
              D2=X2-RHOB2/(X2*X2)
              D3=X3-RHOB3/(X3*X3)
              D4=X4-RHOB4/(X4*X4)
              IC=IC+1
              X1=X1-.333333333333333D0*D1
              X2=X2-.333333333333333D0*D2
              X3=X3-.333333333333333D0*D3
              X4=X4-.333333333333333D0*D4
              IF((D1*D1+D2*D2+D3*D3+D4*D4).LT.EPS2) GOTO 20
            ENDDO
            X1=RHOB1**(1.D0/3.D0)
            X2=RHOB2**(1.D0/3.D0)
            X3=RHOB3**(1.D0/3.D0)
            X4=RHOB4**(1.D0/3.D0)
   20       CONTINUE
            RSB1=X1
            RSB2=X2
            RSB3=X3
            RSB4=X4
            DO NCOUNT=1,100
              D1=Y1-RHO1/(Y1*Y1)
              D2=Y2-RHO2/(Y2*Y2)
              D3=Y3-RHO3/(Y3*Y3)
              D4=Y4-RHO4/(Y4*Y4)
              IC=IC+1
              Y1=Y1-.333333333333333D0*D1
              Y2=Y2-.333333333333333D0*D2
              Y3=Y3-.333333333333333D0*D3
              Y4=Y4-.333333333333333D0*D4
              IF((D1*D1+D2*D2+D3*D3+D4*D4).LT.EPS2) GOTO 30
            ENDDO
            Y1=RHO1**(1.D0/3.D0)
            Y2=RHO2**(1.D0/3.D0)
            Y3=RHO3**(1.D0/3.D0)
            Y4=RHO4**(1.D0/3.D0)
   30       CONTINUE
            RS1=Y1
            RS2=Y2
            RS3=Y3
            RS4=Y4
c..Slater exchange
            EX1=F1S*(RHOA1*RSA1+RHOB1*RSB1)
            EX2=F1S*(RHOA2*RSA2+RHOB2*RSB2)
            EX3=F1S*(RHOA3*RSA3+RHOB3*RSB3)
            EX4=F1S*(RHOA4*RSA4+RHOB4*RSB4)
            VXA1=F1U*RSA1
            VXA2=F1U*RSA2
            VXA3=F1U*RSA3
            VXA4=F1U*RSA4
            VXB1=F1U*RSB1
            VXB2=F1U*RSB2
            VXB3=F1U*RSB3
            VXB4=F1U*RSB4
c..Lee Yang Parr
            DD1=1.D0/(1.D0+D/RS1)
            DD2=1.D0/(1.D0+D/RS2)
            DD3=1.D0/(1.D0+D/RS3)
            DD4=1.D0/(1.D0+D/RS4)
            ECR1=DEXP(-C/RS1)
            ECR2=DEXP(-C/RS2)
            ECR3=DEXP(-C/RS3)
            ECR4=DEXP(-C/RS4)
            OR1=ECR1*DD1/(RHO1*RHO1*RHO1*RS1*RS1)
            OR2=ECR2*DD2/(RHO2*RHO2*RHO2*RS2*RS2)
            OR3=ECR3*DD3/(RHO3*RHO3*RHO3*RS3*RS3)
            OR4=ECR4*DD4/(RHO4*RHO4*RHO4*RS4*RS4)
            DOR1=-OR1*0.333333333333333D0/(RHO1*RS1)*(11.D0*RS1-C-D*DD1)
            DOR2=-OR2*0.333333333333333D0/(RHO2*RS2)*(11.D0*RS2-C-D*DD2)
            DOR3=-OR3*0.333333333333333D0/(RHO3*RS3)*(11.D0*RS3-C-D*DD3)
            DOR4=-OR4*0.333333333333333D0/(RHO4*RS4)*(11.D0*RS4-C-D*DD4)
            RHOAT1=RHOA1*RHOA1*RSA1*RSA1
            RHOBT1=RHOB1*RHOB1*RSB1*RSB1
            RHOAT2=RHOA2*RHOA2*RSA2*RSA2
            RHOBT2=RHOB2*RHOB2*RSB2*RSB2
            RHOAT3=RHOA3*RHOA3*RSA3*RSA3
            RHOBT3=RHOB3*RHOB3*RSB3*RSB3
            RHOAT4=RHOA4*RHOA4*RSA4*RSA4
            RHOBT4=RHOB4*RHOB4*RSB4*RSB4
            EC1=-4.D0*A*RHOA1*RHOB1/RHO1*DD1-CFX*A*B*OR1*RHOA1*RHOB1*
     *          (RHOAT1+RHOBT1)
            VCA1=-4.D0*A*DD1*RHOA1*RHOB1/RHO1*(D*0.33333333333333D0*DD1/
     *           (RHO1*RS1)+(1.D0/RHOA1)-(1.D0/RHO1))-CFX*A*B*(DOR1*
     *           RHOA1*RHOB1*(RHOAT1+RHOBT1)+OR1*RHOB1*
     *           (3.6666666666666667D0*RHOAT1+RHOBT1))
            VCB1=-4.D0*A*DD1*RHOA1*RHOB1/RHO1*(D*0.33333333333333D0*
     *           DD1/(RHO1*RS1)+(1.D0/RHOB1)-(1.D0/RHO1))-CFX*A*B*
     *           (DOR1*RHOA1*RHOB1*(RHOAT1+RHOBT1)+OR1*RHOA1*
     *           (3.6666666666666667D0*RHOBT1+RHOAT1))
            EC2=-4.D0*A*RHOA2*RHOB2/RHO2*DD2-CFX*A*B*OR2*RHOA2*RHOB2*
     *          (RHOAT2+RHOBT2)
            VCA2=-4.D0*A*DD2*RHOA2*RHOB2/RHO2*(D*0.33333333333333D0*DD2/
     *           (RHO2*RS2)+(1.D0/RHOA2)-(1.D0/RHO2))-CFX*A*B*(DOR2*
     *           RHOA2*RHOB2*(RHOAT2+RHOBT2)+OR2*RHOB2*
     *           (3.6666666666666667D0*RHOAT2+RHOBT2))
            VCB2=-4.D0*A*DD2*RHOA2*RHOB2/RHO2*(D*0.33333333333333D0*DD2/
     *           (RHO2*RS2)+(1.D0/RHOB2)-(1.D0/RHO2))-CFX*A*B*(DOR2*
     *           RHOA2*RHOB2*(RHOAT2+RHOBT2)+OR2*RHOA2*
     *           (3.6666666666666667D0*RHOBT2+RHOAT2))
            EC3=-4.D0*A*RHOA3*RHOB3/RHO3*DD3-CFX*A*B*OR3*RHOA3*RHOB3*
     *          (RHOAT3+RHOBT3)
            VCA3=-4.D0*A*DD3*RHOA3*RHOB3/RHO3*(D*0.33333333333333D0*DD3/
     *           (RHO3*RS3)+(1.D0/RHOA3)-(1.D0/RHO3))-CFX*A*B*(DOR3*
     *           RHOA3*RHOB3*(RHOAT3+RHOBT3)+OR3*RHOB3*
     *           (3.6666666666666667D0*RHOAT3+RHOBT3))
            VCB3=-4.D0*A*DD3*RHOA3*RHOB3/RHO3*(D*0.33333333333333D0*DD3/
     *           (RHO3*RS3)+(1.D0/RHOB3)-(1.D0/RHO3))-CFX*A*B*(DOR3*
     *           RHOA3*RHOB3*(RHOAT3+RHOBT3)+OR3*RHOA3*
     *           (3.6666666666666667D0*RHOBT3+RHOAT3))
            EC4=-4.D0*A*RHOA4*RHOB4/RHO4*DD4-CFX*A*B*OR4*RHOA4*RHOB4*
     *          (RHOAT4+RHOBT4)
            VCA4=-4.D0*A*DD4*RHOA4*RHOB4/RHO4*(D*0.33333333333333D0*DD4/
     *           (RHO4*RS4)+(1.D0/RHOA4)-(1.D0/RHO4))-CFX*A*B*(DOR4*
     *           RHOA4*RHOB4*(RHOAT4+RHOBT4)+OR4*RHOB4*
     *           (3.6666666666666667D0*RHOAT4+RHOBT4))
            VCB4=-4.D0*A*DD4*RHOA4*RHOB4/RHO4*(D*0.33333333333333D0*DD4/
     *           (RHO4*RS4)+(1.D0/RHOB4)-(1.D0/RHO4))-CFX*A*B*(DOR4*
     *           RHOA4*RHOB4*(RHOAT4+RHOBT4)+OR4*RHOA4*
     *           (3.6666666666666667D0*RHOBT4+RHOAT4))
            EEXCU=EEXCU+(EX1+EC1)
            EEXCU=EEXCU+(EX2+EC2)
            EEXCU=EEXCU+(EX3+EC3)
            EEXCU=EEXCU+(EX4+EC4)
            UION1(1,I1,I2+0,I3+0)=UION1(1,I1,I2+0,I3+0)+VXA1+VCA1
            UION1(1,I1,I2+1,I3+0)=UION1(1,I1,I2+1,I3+0)+VXA2+VCA2
            UION1(1,I1,I2+0,I3+1)=UION1(1,I1,I2+0,I3+1)+VXA3+VCA3
            UION1(1,I1,I2+1,I3+1)=UION1(1,I1,I2+1,I3+1)+VXA4+VCA4
            UION2(1,I1,I2+0,I3+0)=UION2(1,I1,I2+0,I3+0)+VXB1+VCB1
            UION2(1,I1,I2+1,I3+0)=UION2(1,I1,I2+1,I3+0)+VXB2+VCB2
            UION2(1,I1,I2+0,I3+1)=UION2(1,I1,I2+0,I3+1)+VXB3+VCB3
            UION2(1,I1,I2+1,I3+1)=UION2(1,I1,I2+1,I3+1)+VXB4+VCB4
          ENDDO
        ENDDO
      ENDDO
C..counting floating point operations
      FADD   = IC * 11.D0 + (NR1*NR2*NR3) * 23.D0
      FMULT  = IC * 12.D0 + (NR1*NR2*NR3) * 65.D0
      FDIV   = IC *  4.D0 + (NR1*NR2*NR3) * 13.D0
      FSPEZ  =              (NR1*NR2*NR3) *  1.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE XALPSP(SALPHA,NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION1,UION2,
     *                   EEXCU,FLOP)
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (F1 = -1.39578858466194911d0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1,KM2,KM3,2),UION1(2,KM1,KM2,KM3),
     *          UION2(2,KM1,KM2,KM3)
      INTEGER I1,I2,I3,II2,II3,IC,NCOUNT
      REAL*8 F1S,F1U,EEXCU,X1,X2,X3,X4,Y1,Y2,Y3,Y4,D1,D2,D3,D4
     .      ,RHOA1,RHOA2,RHOA3,RHOA4,RHOB1,RHOB2,RHOB3,RHOB4
     .      ,EX1,EX2,EX3,EX4,VXA1,VXA2,VXA3,VXA4
     .      ,VXB1,VXB2,VXB3,VXB4
C     ==--------------------------------------------------------------==
      F1S=F1*SALPHA
      F1U=F1S*1.3333333333333333d0
      EEXCU=0.D0
      X1=1.D0
      X2=1.D0
      X3=1.D0
      X4=1.D0
      Y1=1.D0
      Y2=1.D0
      Y3=1.D0
      Y4=1.D0
      IC=0
      II3=MOD(NR3,2)
      II2=MOD(NR2,2)
      IF(II3.NE.0.OR.II2.NE.0) CALL STOPGM('XALPSP','ODD DIMENSION')
C$OMP parallel do private(I1,I2,I3,D1,D2,D3,D4)
C$OMP+ firstprivate(X1,X2,X3,X4,Y1,Y2,Y3,Y4)
C$OMP+ private(NCOUNT,RHOA1,RHOA2,RHOA3,RHOA4,RHOB1,RHOB2,RHOB3,RHOB4)
C$OMP+ private(EX1,EX2,EX3,EX4,VXA1,VXA2,VXA3,VXA4)
C$OMP+ private(VXB1,VXB2,VXB3,VXB4)
C$OMP+ private(RHO1,RHO2,RHO3,RHO4)  ! bug fix
C$OMP+ private(RSA1,RSA2,RSA3,RSA4,RSB1,RSB2,RSB3,RSB4)  ! bug fix
C$OMP+ reduction(+:EEXCU,IC)
      DO I3=1,NR3,2
        DO I2=1,NR2,2
          DO I1=1,NR1
            RHOA1=RHO(I1,I2+0,I3+0,1)-RHO(I1,I2+0,I3+0,2)
            RHOA2=RHO(I1,I2+1,I3+0,1)-RHO(I1,I2+1,I3+0,2)
            RHOA3=RHO(I1,I2+0,I3+1,1)-RHO(I1,I2+0,I3+1,2)
            RHOA4=RHO(I1,I2+1,I3+1,1)-RHO(I1,I2+1,I3+1,2)
            RHOA1=DMAX1(RHOA1,SMALL)
            RHOA2=DMAX1(RHOA2,SMALL)
            RHOA3=DMAX1(RHOA3,SMALL)
            RHOA4=DMAX1(RHOA4,SMALL)
            RHOB1=DMAX1(RHO(I1,I2+0,I3+0,2),SMALL)
            RHOB2=DMAX1(RHO(I1,I2+1,I3+0,2),SMALL)
            RHOB3=DMAX1(RHO(I1,I2+0,I3+1,2),SMALL)
            RHOB4=DMAX1(RHO(I1,I2+1,I3+1,2),SMALL)
            RHO1=RHOA1+RHOB1
            RHO2=RHOA2+RHOB2
            RHO3=RHOA3+RHOB3
            RHO4=RHOA4+RHOB4
C
C..iterative calculation of rho**(1/3) (using Newton method)
            DO NCOUNT=1,100
              D1=X1-RHOA1/(X1*X1)
              D2=X2-RHOA2/(X2*X2)
              D3=X3-RHOA3/(X3*X3)
              D4=X4-RHOA4/(X4*X4)
              IC=IC+1
              X1=X1-.333333333333333D0*D1
              X2=X2-.333333333333333D0*D2
              X3=X3-.333333333333333D0*D3
              X4=X4-.333333333333333D0*D4
              IF((D1*D1+D2*D2+D3*D3+D4*D4).LT.EPS2) GOTO 10
            ENDDO
            X1=RHOA1**(1.D0/3.D0)
            X2=RHOA2**(1.D0/3.D0)
            X3=RHOA3**(1.D0/3.D0)
            X4=RHOA4**(1.D0/3.D0)
   10       CONTINUE
            RSA1=X1
            RSA2=X2
            RSA3=X3
            RSA4=X4
            DO NCOUNT=1,100
              D1=Y1-RHOB1/(Y1*Y1)
              D2=Y2-RHOB2/(Y2*Y2)
              D3=Y3-RHOB3/(Y3*Y3)
              D4=Y4-RHOB4/(Y4*Y4)
              IC=IC+1
              Y1=Y1-.333333333333333D0*D1
              Y2=Y2-.333333333333333D0*D2
              Y3=Y3-.333333333333333D0*D3
              Y4=Y4-.333333333333333D0*D4
              IF((D1*D1+D2*D2+D3*D3+D4*D4).LT.EPS2) GOTO 20
            ENDDO
            Y1=RHOB1**(1.D0/3.D0)
            Y2=RHOB2**(1.D0/3.D0)
            Y3=RHOB3**(1.D0/3.D0)
            Y4=RHOB4**(1.D0/3.D0)
   20       CONTINUE
            RSB1=Y1
            RSB2=Y2
            RSB3=Y3
            RSB4=Y4
c..Slater exchange
            EX1=F1S*(RHOA1*RSA1+RHOB1*RSB1)
            EX2=F1S*(RHOA2*RSA2+RHOB2*RSB2)
            EX3=F1S*(RHOA3*RSA3+RHOB3*RSB3)
            EX4=F1S*(RHOA4*RSA4+RHOB4*RSB4)
            VXA1=F1U*RSA1
            VXA2=F1U*RSA2
            VXA3=F1U*RSA3
            VXA4=F1U*RSA4
            VXB1=F1U*RSB1
            VXB2=F1U*RSB2
            VXB3=F1U*RSB3
            VXB4=F1U*RSB4
c
            EEXCU=EEXCU+EX1
            EEXCU=EEXCU+EX2
            EEXCU=EEXCU+EX3
            EEXCU=EEXCU+EX4
            UION1(1,I1,I2+0,I3+0)=UION1(1,I1,I2+0,I3+0)+VXA1
            UION1(1,I1,I2+1,I3+0)=UION1(1,I1,I2+1,I3+0)+VXA2
            UION1(1,I1,I2+0,I3+1)=UION1(1,I1,I2+0,I3+1)+VXA3
            UION1(1,I1,I2+1,I3+1)=UION1(1,I1,I2+1,I3+1)+VXA4
            UION2(1,I1,I2+0,I3+0)=UION2(1,I1,I2+0,I3+0)+VXB1
            UION2(1,I1,I2+1,I3+0)=UION2(1,I1,I2+1,I3+0)+VXB2
            UION2(1,I1,I2+0,I3+1)=UION2(1,I1,I2+0,I3+1)+VXB3
            UION2(1,I1,I2+1,I3+1)=UION2(1,I1,I2+1,I3+1)+VXB4
          ENDDO
        ENDDO
      ENDDO
C..counting floating point operations
      FADD   = IC * 11.D0 + (NR1*NR2*NR3) *  5.D0
      FMULT  = IC * 12.D0 + (NR1*NR2*NR3) *  5.D0
      FDIV   = IC *  4.D0 + (NR1*NR2*NR3) *  0.D0
      FSPEZ  =              (NR1*NR2*NR3) *  0.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#else
C     ==================================================================
      SUBROUTINE LYPUU(NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION,EEXCU,FLOP)
C     ==--------------------------------------------------------------==
c adds unpolarized exc potential to uion and sums exc energy
c written by S. Goedecker, Stuttgart, march 1996
c modified by J. Hutter, Stuttgart, may/august 1996
c non-gradient part of Lee,Yang and Parr functional (non-spin polarised)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (F1 = -1.10783814957303361D0)
      PARAMETER (THIRD=1.D0/3.D0,F43=4.D0/3.D0)
      PARAMETER (A=0.04918D0,B=0.132D0,C=0.2533D0,D=0.349D0)
      PARAMETER (CF=2.87123400018819108D0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1*KM2*KM3),UION(2,KM1*KM2*KM3)
      DIMENSION EECPU(128),ICPU(128)
      INTEGER I,IC,KKK
      REAL*8 FA1,FA2,BCF,EEXCU,RHOU,X,RSX,ECRS,OX,ELYP
     .      ,EPSXCU,VXC,EEXC1X
C     ==--------------------------------------------------------------==
      FA1=F1*0.66666666666666D0
      FA2=FA1*F43
      BCF=B*Cf
      EEXCU=0.D0
      IC=0
      KKK=KM1*KM2*KM3
C$OMP parallel do private(I,RHOU,X,RSX,ECRS,OX,ELYP,EPSXCU,VXC,EEXC1X)
C$OMP+ reduction(+:EEXCU,IC)
      DO I=1,KKK
        RHOU=DABS(RHO(I))
        IF(RHOU.GT.SMALL)THEN
          X=RHOU**(0.333333333333333d0)
          RSX=1.D0/X
          ECRS=BCF*DEXP(-C*RSX)
          OX=1.D0/(1.D0+D*RSX)
          ELYP=-A*OX*(1.D0+ECRS)
          EPSXCU=FA1*X+ELYP
          VXC=FA2*X+ELYP-0.33333333333333D0*RSX*A*OX*
     *          (D*OX+ECRS*(D*OX+C))
          UION(1,I)=UION(1,I)+VXC
          IC = IC + 1
          EEXCU = EEXCU + EPSXCU*RHOU
        ENDIF
      ENDDO
C..counting floating point operations
      FADD   = IC *  9.D0
      FMULT  = IC * 15.D0
      FDIV   = IC *  2.D0
      FSPEZ  = IC *  2.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE XALPU(SALPHA,NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION,
     *                 EEXCU,FLOP)
C     ==--------------------------------------------------------------==
C..X-Alpha potential
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (F1 = -1.10783814957303361D0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1*KM2*KM3),UION(2,KM1*KM2*KM3)
      DIMENSION EECPU(128),ICPU(128)
      INTEGER I,IC,KKK
      REAL*8 FA1,FA2,EEXCU,RHOU,X,EPSXCU,VXC,EEXC1X
C     ==--------------------------------------------------------------==
      FA1=F1*SALPHA
      FA2=FA1*1.33333333333333D0
      EEXCU=0.D0
      IC=0
      KKK=KM1*KM2*KM3
C$OMP parallel do private(I,RHOU,X,EPSXCU,VXC,EEXC1X)
C$OMP+ reduction(+:EEXCU,IC)
      DO I=1,KKK
        RHOU=DABS(RHO(I))
        IF(RHOU.GT.SMALL)THEN
          X=RHOU**(0.33333333333333D0)
          EPSXCU=FA1*X
          VXC=FA2*X
          UION(1,I)=UION(1,I)+VXC
          IC=IC+1
          EEXCU=EEXCU+EPSXCU*RHOU
        ENDIF
      ENDDO
C..counting floating point operations
      FADD   = IC *  2.D0
      FMULT  = IC *  3.D0
      FDIV   = IC *  0.D0
      FSPEZ  = IC *  1.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MIKEU(NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION,EEXCU,FLOP)
C     ==--------------------------------------------------------------==
C..Pade approximation to LDA functional
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A0U=.4581652932831429D0,
     *           A1U=2.217058676663745D0,
     *           A2U=0.7405551735357053D0,
     *           A3U=0.01968227878617998D0)
      PARAMETER (B1U=1.0D0,
     *           B2U=4.504130959426697D0,
     *           B3U=1.110667363742916D0,
     *           B4U=0.02359291751427506D0)
      PARAMETER
     *  (C1U=4.D0*A0U*B1U/3.D0,C2U=5.D0*A0U*B2U/3.D0+A1U*B1U,
     *   C3U=2.D0*A0U*B3U+4.D0*A1U*B2U/3.D0+2.D0*A2U*B1U/3.D0,
     *   C4U=7.D0*A0U*B4U/3.D0+5.D0*A1U*B3U/3.D0+A2U*B2U+A3U*B1U/3.D0,
     *   C5U=2.D0*A1U*B4U+4.D0*A2U*B3U/3.D0+2.D0*A3U*B2U/3.D0,
     *   C6U=5.D0*A2U*B4U/3.D0+A3U*B3U,C7U=4.D0*A3U*B4U/3.D0)
      PARAMETER (RSFAC=.6203504908994000D0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1*KM2*KM3),UION(2,KM1*KM2*KM3)
      DIMENSION EECPU(128),ICPU(128)
      INTEGER I,IC,KKK
      REAL*8 EEXCU,RHOU,RSU,TOPU,DTOPU,BOTU,EPSXCU,VXC,EEXC1X
C     ==--------------------------------------------------------------==
      EEXCU=0.D0
      IC=0
      KKK=KM1*KM2*KM3
C$OMP parallel do private(I,RHOU,RSU,TOPU,DTOPU,BOTU,EPSXCU,VXC,EEXC1X)
C$OMP+ reduction(+:EEXCU,IC)
      DO I=1,KKK
        RHOU=DABS(RHO(I))
        IF(RHOU.GT.SMALL)THEN
          RSU=RSFAC*RHOU**(-0.33333333333333D0)
          TOPU=A0U+RSU*(A1U+RSU*(A2U+RSU*A3U))
          DTOPU=-RSU*(C1U+RSU*(C2U+RSU*(C3U+RSU*
     *                 (C4U+RSU*(C5U+RSU*(C6U+RSU*C7U))))))
          BOTU=1.D0/(RSU*(B1U+RSU*(B2U+RSU*(B3U+RSU*B4U))))
          EPSXCU=-TOPU*BOTU
          VXC=DTOPU*BOTU*BOTU
          UION(1,I)=UION(1,I)+VXC
          IC=IC+1
          EEXCU=EEXCU+EPSXCU*RHOU
        ENDIF
      ENDDO
C..counting floating point operations
      FADD   = IC * 14.D0
      FMULT  = IC * 19.D0
      FDIV   = IC *  1.D0
      FSPEZ  = IC *  1.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MIKESP(NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION1,UION2,
     *                  EEXCU,FLOP)
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A0U=.4581652932831429D0,
     *           A1U=2.217058676663745D0,
     *           A2U=0.7405551735357053D0,
     *           A3U=0.01968227878617998d0)
      PARAMETER (B1U=1.0D0,
     *           B2U=4.504130959426697D0,
     *           B3U=1.110667363742916D0,
     *           B4U=0.02359291751427506D0)
      PARAMETER (DA0=.119086804055547D0,
     *           DA1=.6157402568883345D0,
     *           DA2=.1574201515892867D0,
     *           DA3=.003532336663397157d0)
      PARAMETER (DB1=0.0D0,
     *           DB2=.2673612973836267D0,
     *           DB3=.2052004607777787D0,
     *           DB4=.004200005045691381D0)
      PARAMETER (T1=.854960467080683D0,
     *           T2=.07916300621117439D0,
     *           T3=.02580127609845684D0,
     *           T4=.0121839359353824D0,
     *           T5=.006919272259599879D0,
     *           T6=.004391524649611372D0,
     *           T7=.003002751897170168D0,
     *           T8=.00216587382212552D0,
     *           T9=.01141189204579585D0)
      PARAMETER (RSFAC=.6203504908994000D0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1*KM2*KM3,2),UION1(2,KM1*KM2*KM3),
     *          UION2(2,KM1*KM2*KM3)
      INTEGER I,IC
      REAL*8 EEXCU,RHOA,RHOB,RHOU,ETA,ET,FXS,DFXS,A0P,A1P,A2P,A3P
     .      ,B1P,B2P,B3P,B4P,X,RSU,TOP,DTOP,XTOP,BOT,DBOT,XBOT,OBOT
     .      ,EPSXCU,VXC,DVXC,DXA,DXB,EEXC1X
C     ==--------------------------------------------------------------==
      EEXCU=0.D0
      IC=0
C$OMP parallel do private(I,RHOA,RHOB,RHOU,ETA,ET,FXS)
C$OMP+ private(DFXS,A0P,A1P,A2P,A3P,B1P,B2P,B3P,B4P)
C$OMP+ private(X,RSU,TOP,DTOP,XTOP,BOT,DBOT)
C$OMP+ private(XBOT,OBOT,EPSXCU,VXC,DVXC,DXA,DXB,EEXC1X)
C$OMP+ reduction(+:EEXCU,IC)
      DO I=1,KM1*KM2*KM3
        RHOA=DMAX1(RHO(I,1)-RHO(I,2),SMALL)
        RHOB=DMAX1(RHO(I,2),SMALL)
        RHOU=RHOA+RHOB
        IF(RHOU.GT.2.25D0*SMALL) THEN
          ETA=(RHOA-RHOB)/RHOU
          ET=ETA*ETA
          FXS=ET*(T1+ET*(T2+ET*(T3+ET*(T4+ET*(T5+ET*
     *         (T6+ET*(T7+ET*(T8+ET*T9))))))))
          DFXS=(2.D0*ETA)*(T1+ET*(2.D0*T2+ET*(3.D0*T3+ET*
     *         (4.D0*T4+ET*(5.D0*T5+ET*(6.D0*T6+ET*(7.D0*T7+ET*
     *         (8.D0*T8+ET*9.D0*T9))))))))
C
          A0P=A0U+FXS*DA0
          A1P=A1U+FXS*DA1
          A2P=A2U+FXS*DA2
          A3P=A3U+FXS*DA3
          B1P=B1U+FXS*DB1
          B2P=B2U+FXS*DB2
          B3P=B3U+FXS*DB3
          B4P=B4U+FXS*DB4
C
          X=RHOU**(0.333333333333D0)
          RSU=RSFAC/X
          TOP=A0P+RSU*(A1P+RSU*(A2P+RSU*A3P))
          DTOP= A1P+RSU*(2.D0*A2P+RSU*3.D0*A3P)
          XTOP=DA0+RSU*(DA1+RSU*(DA2+RSU*DA3))
          BOT=RSU*(B1P+RSU*(B2P+RSU*(B3P+RSU*B4P)))
          DBOT=B1P+RSU*(2.D0*B2P+RSU*(3.D0*B3P+RSU*4.D0*B4P))
          XBOT=RSU*(DB1+RSU*(DB2+RSU*(DB3+RSU*DB4)))
          OBOT=1.D0/BOT
          EPSXCU=-TOP*OBOT
          IC=IC+1
          EEXCU=EEXCU+EPSXCU*RHOU
          VXC=EPSXCU+RSU*(DTOP*OBOT-DBOT*(TOP*OBOT*OBOT))*
     *           0.333333333333333D0
          DVXC=-(XTOP*OBOT-XBOT*(TOP*OBOT*OBOT))
          DXA=VXC+DVXC*DFXS*(1.D0-ETA)
          DXB=VXC-DVXC*DFXS*(1.D0+ETA)
          UION1(1,I)=UION1(1,I)+DXA
          UION2(1,I)=UION2(1,I)+DXB
        ENDIF
      ENDDO
C..counting floating point operations
      FADD   = IC * 55.D0
      FMULT  = IC * 76.D0
      FDIV   = IC *  3.D0
      FSPEZ  = IC *  1.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SLYPSP(NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION1,UION2,SALPHA,
     *                  EEXCU,FLOP)
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (A=0.04918D0,B=0.132D0,C=0.2533D0,D=0.349D0,
     *           CF=2.87123400018819108D0)
      PARAMETER (F1 = -1.39578858466194911D0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1*KM2*KM3,2),UION1(2,KM1*KM2*KM3),
     *          UION2(2,KM1*KM2*KM3)
      DIMENSION EECPU(128),ICPU(128)
      INTEGER I,IC,KKK
      REAL*8 CFX,F1S,F1U,EEXCU,RHOA,RHOB,RHOT,RSA,RSB,RS,ORS,ORHO
     .      ,EX,VXA,VXB,DD,ECR,OR,DOR,RHOAT,RHOBT,EC
     .      ,VCA,VCB,EEXC1X
C     ==--------------------------------------------------------------==
      CFX=CF*2.D0**3.66666666666666D0
      F1S=F1*SALPHA
      F1U=F1S*1.3333333333333333d0
      EEXCU=0.D0
      IC=0
      KKK=KM1*KM2*KM3
C$OMP parallel do private(I,RHOA,RHOB,RHOT,RSA,RSB,RS,ORS,ORHO,EX)
C$OMP+ private(VXA,VXB,DD,ECR,OR,DOR,RHOAT,RHOBT,EC,VCA,VCB,EEXC1X)
C$OMP+ reduction(+:EEXCU,IC)
      DO I=1,KKK
        RHOA=RHO(I,1)-RHO(I,2)
        RHOA=DMAX1(RHOA,SMALL)
        RHOB=DMAX1(RHO(I,2),SMALL)
        RHOT=RHOA+RHOB
        IF(RHOT.GT.2.D0*SMALL)THEN
          RHOA=DMAX1(RHOA,SMALL)
          RHOB=DMAX1(RHOB,SMALL)
          RSA=RHOA**(0.3333333333333d0)
          RSB=RHOB**(0.3333333333333d0)
          RS=RHOT**(0.3333333333333d0)
          ORS=1.D0/RS
          ORHO=1.D0/RHOT
c..Slater exchange
          EX=F1S*(RHOA*RSA+RHOB*RSB)
          VXA=F1U*RSA
          VXB=F1U*RSB
c..Lee Yang Parr
          DD=1.D0/(1.D0+D*ORS)
          ECR=DEXP(-C*ORS)
          OR=ECR*DD*ORHO*ORHO*ORHO*ORS*ORS
          DOR=-OR*0.333333333333333D0*ORHO*ORS*(11.D0*RS-C-D*DD)
          RHOAT=RHOA*RHOA*RSA*RSA
          RHOBT=RHOB*RHOB*RSB*RSB
          EC=-4.D0*A*RHOA*RHOB*ORHO*DD-CFX*A*B*OR*RHOA*RHOB*
     *         (RHOAT+RHOBT)
          VCA=-4.D0*A*DD*RHOA*RHOB*ORHO*(D*0.333333333333333D0*DD*
     *          (ORHO*ORS)+(1.D0/RHOA)-ORHO)-CFX*A*B*(DOR*
     *          RHOA*RHOB*(RHOAT+RHOBT)+OR*RHOB*
     *          (3.6666666666666667D0*RHOAT+RHOBT))
          VCB=-4.D0*A*DD*RHOA*RHOB*ORHO*(D*0.333333333333333D0*DD*
     *          (ORHO*ORS)+(1.D0/RHOB)-ORHO)-CFX*A*B*(DOR*
     *          RHOA*RHOB*(RHOAT+RHOBT)+OR*RHOA*
     *          (3.6666666666666667D0*RHOBT+RHOAT))
          UION1(1,I)=UION1(1,I)+VXA+VCA
          UION2(1,I)=UION2(1,I)+VXB+VCB
          IC=IC+1
          EEXCU=EEXCU+EX+EC
        ENDIF
      ENDDO
C..counting floating point operations
      FADD   = IC * 24.D0
      FMULT  = IC * 74.D0
      FDIV   = IC *  5.D0
      FSPEZ  = IC *  4.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE XALPSP(SALPHA,NR1,NR2,NR3,KM1,KM2,KM3,RHO,UION1,UION2,
     *                   EEXCU,FLOP)
C     ==--------------------------------------------------------------==
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (F1 = -1.39578858466194911d0)
      PARAMETER (EPS2=1.D-20,SMALL=1.D-24)
      DIMENSION RHO(KM1*KM2*KM3,2),UION1(2,KM1*KM2*KM3),
     *          UION2(2,KM1*KM2*KM3)
      DIMENSION EECPU(128),ICPU(128)
      INTEGER I,IC,KKK
      REAL*8 F1S,F1U,EEXCU,RHOA,RHOB,RHOT,RSA,RSB,EX,VXA,VXB,EEXC1X
C     ==--------------------------------------------------------------==
      F1S=F1*SALPHA
      F1U=F1S*1.3333333333333333d0
      EEXCU=0.D0
      IC=0
      KKK=KM1*KM2*KM3
C$OMP parallel do private(I,RHOA,RHOB,RHOT,RSA,RSB,EX,VXA,VXB,EEXC1X)
C$OMP+ reduction(+:EEXCU,IC)
      DO I=1,KKK
        RHOA=RHO(I,1)-RHO(I,2)
        RHOA=DMAX1(RHOA,SMALL)
        RHOB=DMAX1(RHO(I,2),SMALL)
        RHOT=RHOA+RHOB
        IF(RHOT.GT.2.D0*SMALL) THEN
          RSA=RHOA**(0.33333333333333D0)
          RSB=RHOB**(0.33333333333333D0)
c..Slater exchange
          EX=F1S*(RHOA*RSA+RHOB*RSB)
          VXA=F1U*RSA
          VXB=F1U*RSB
          UION1(1,I)=UION1(1,I)+VXA
          UION2(1,I)=UION2(1,I)+VXB
          IC=IC+1  
          EEXCU=EEXCU+EX
        ENDIF
      ENDDO
C..counting floating point operations
      FADD   = IC *  5.D0
      FMULT  = IC *  5.D0
      FDIV   = IC *  0.D0
      FSPEZ  = IC *  2.D0
      FLOP=FADD+FMULT+FDIV+FSPEZ
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#endif
