C     ==================================================================
C     THIS FILE contains all routines that are related to the "full"
C     calculation of the NMR chemical shift tensor.
C     ==================================================================
      SUBROUTINE make_listofstates(neighborlist,dowfopt,nstate)
C     This routine looks at the lowerleft positions for all states (and
C     all B-field directions) and makes a neighborlist such that 
C     states that follow each other in this list have a very close 
C     lower-left-corner. Note that this proximity search is done 
C     _projecting_ the distance on the plane orthogonal to the B direction.

      IMPLICIT NONE
      include 'system.h'
      include 'response_p.inc'
      integer nstate,neighborlist(3,nstate)
      logical dowfopt(3,nstate)

      real*8  distance,min_distance,give_min_dist
      integer is,initial_state,next_initial_state,present_index,ir,iB
      logical already_done(nstate)
      pointer (ip_already_done,already_done)

      call memory(ip_already_done,nstate,'LISTOFSTATES')

      do iB=1,3
         initial_state=1
         present_index=2
         neighborlist(iB,1) = 1 ! INITIAL STATE.
         already_done(1) = .true.
         dowfopt(iB,1)     = .true.
         do is=2,nstate
            already_done(is) = .false.
            dowfopt(iB,is)     = .true.
         enddo

         do while (present_index .le. nstate)
            min_distance = alat ! dummy: any large value.

C     Search for states which are within a range of firstshelldistance:
            do is=1,nstate
               IF (.not. already_done(is)) THEN
                  distance = give_min_dist(is,initial_state,iB)
                  if (distance .le. firstshelldistance) then
                     neighborlist(iB,present_index) = is
                     present_index = present_index +1
                     already_done(is) = .true.
                     dowfopt(iB,is) = .false.

C     Now: search for the NEXT state:
C     Take that one whose distance to the present one
C     is closest to 2*firstshelldistance.
                  elseif (abs(distance-2*firstshelldistance)
     &                    .lt. min_distance) then
                     min_distance = distance
                     next_initial_state = is
                  endif         ! distance vs. firstshelldistance
               ENDIF            ! ...if this state not already done
            enddo               ! istate; find all states within 2.5A.
            if (present_index .le. nstate) then
                                ! ...then there are more states to do
               initial_state = next_initial_state
               neighborlist(iB,present_index) = initial_state
               present_index = present_index +1
               already_done(initial_state) = .true.
               dowfopt(iB,initial_state) = .true.
            endif
         enddo                  ! do while not finished with all states
      enddo                     ! iB


C     CHECK IT:
      do iB=1,3
         do is=1,nstate
            already_done(is) = .false.
         enddo
         do is=1,nstate
            already_done(neighborlist(iB,is)) = .true.
          if (parent) write(6,'(A,I1,A,I4,A,3I4,L1)') 'field in ',iB,
     &           ', state ',neighborlist(iB,is),'  --> ',
     &           (lower_left(ir,neighborlist(iB,is)),ir=1,3),
     &           dowfopt(iB,neighborlist(iB,is))
         enddo

         do is=1,nstate
            if (.not. already_done(is)) then
               print *,'ERROR ERRORIS: state ',is,' not in list!!!'
            endif
         enddo
      enddo
C     checking completed.

      call freem(ip_already_done)
      return
      end
C     ==================================================================

C     ==================================================================
#define mindistroutine calc_min_dist_alat_smooth
Cdefine mindistroutine calc_min_dist_alat
C     ==================================================================
      SUBROUTINE DO_FULL(i_state,iB,dowfopt,h1psi0,c0,c1,scr,psi,rhoe,
     &     eirop,eivps,z11,nstate,lscr)
      IMPLICIT NONE
      include 'system.h'
      include 'soft.inc'
      include 'response_p.inc'
C     Arguments:
      integer     i_state,iB,nstate,lscr
      logical     dowfopt
      complex*16  h1psi0(ngw,nstate),c0(ngw,nstate),c1(ngw,nstate,6)
      real*8      scr,psi,rhoe,eirop,eivps,z11,
     &    shift_sumup(3,3,nax,nsx),chi_sumup(3,3),
     &    shift_saved(3,3,nax,nsx),chi_saved(3,3)

C     Variables:
      real*8      dummy,dij(3),norm,ENERGY,dd
      integer     is,ii,iii,i_c1,isub,min_dist,ir,i_state_previous(3)
      save        i_state_previous
      data        i_state_previous /0,0,0/
      character   xyz(3)
      data        xyz /'x','y','z'/

 89   FORMAT ("*  FULL calculation: state ",I4,
     &     ", field in O",A1,19X,"*")
 98   FORMAT (52("*"),"NMR*RESPONSE*")

      i_c1 = iB+3

C     Initialization / very first call:
      if (i_state_previous(iB) .eq. 0) then
         call azzero(c1(1,1,i_c1),2*ngw*nstate)
         T_INITIALGUESS_C1 = .false.
         i_state_previous(iB) = i_state
      else
         T_INITIALGUESS_C1 = .true.
      endif


C     Preparation of B_iB . (d_k - d_fix) x p |psi_0_k>:
      if (parent) then
         write (6,98)
         write (6,89) i_state,xyz(iB)
         write (6,98)
      endif
      call make_123(iB,ii,iii)
      if (dowfopt) then
         call azzero(h1psi0,2*ngw*nstate)
         do is=1,nstate         ! apply dd_ii p_iii - dd_iii p_ii:
                                ! with dd = d_is - d_istate
            call mindistroutine(is,i_state,dij)
            call apply_op_p (c0(1,is), scr, ii, ngw)
            call DAXPY(2*ngw, dij(iii),scr,1,h1psi0(1,is),1)
            call apply_op_p (c0(1,is), scr, iii, ngw)
            call DAXPY(2*ngw,-dij(ii),scr,1,h1psi0(1,is),1)
         enddo
         timetag='preparation.'
         if (parent) call printtime
         if (parent) write (6,98)
         if (exsoft) return
      endif


C     Prepare a good guess for C1 from the previous optimization:
C     this should be:
C     B  .  {d_k - d_i} x |psi1^p>
C     where k is the new and i is the old correction state.
C     B = B * O_beta
C     For an unknown reason, the convergence is better WITHOUT this
C     adjustment. Bill Gates alone may have an idea why.
      if (i_state .ne. i_state_previous(iB)) then
         call mindistroutine
     &        (i_state,i_state_previous(iB),dij)
         call DAXPY(2*nstate*ngw, dij(ii),c1(1,1,iii),1,
     &        c1(1,1,i_c1),1)
         call DAXPY(2*nstate*ngw,-dij(iii),c1(1,1,ii),1,
     &        c1(1,1,i_c1),1)
         call ortho_p(nstate,c0,c1(1,1,i_c1))
      endif
      if (dowfopt) then
         call tiset('PSI1-WF-OPT',isub)
         call rwfopt_p(c0,c1(1,1,i_c1),psi,rhoe,dummy,scr,lscr,
     &        eirop,eivps,dummy,h1psi0,
     &        z11,nstate,dummy)
         call tihalt('PSI1-WF-OPT',isub)
         timetag='calculation of psi^1.'
         if (parent) call printtime
      endif

      if (TCURRENT)
     &     call DAXPY(2*ngw, 1.d0, c1(1,i_state,i_c1),1,
     &     c1(1,i_state,i_c1+3),1)

      if (parent) write (6,98)
      call calc_L_one(c0,c1(1,1,i_c1),nstate,
     &     iB,psi,scr,rhoe,i_state)
      timetag='shift contribution.'
      if (parent) call printtime
            
      call calc_chi_L_one(c0,c1(1,1,i_c1),nstate,
     &     iB,psi,scr,i_state)
      timetag='susceptibility contribution.'
      if (parent) call printtime
      CALL MEMORY_CHECK


      i_state_previous(iB) = i_state
      return
      end
c     ==================================================================
c     ==================================================================
c     ==================================================================
      SUBROUTINE optimize_llc(neighborlist,dowfopt,nstate)
      IMPLICIT NONE
      include 'system.h'
      include 'response_p.inc'
      integer nstate,neighborlist(3,nstate)
      logical dowfopt(3,nstate)

      integer presentllc(3),deviation,averagedeviation(3),count
      integer is,isi,nst,ib,ibb,ibbb,ir,iprev,newllc(3)

      if (parent) then
         do is=1,nstate
            write (6,'("State ",I3,": LLC before = ",3I5)')
     &           is,(lower_left(ir,is),ir=1,3)
         enddo
      endif
      do iB=1,3
         call make_123(ib,ibb,ibbb)
         do isi=1,nstate
            is = neighborlist(iB,isi)
            if (dowfopt(iB,is)) then
c     THEN... start a new averaging process.
               do ir=1,3
                  presentllc(ir) = lower_left(ir,is)
                  averagedeviation(ir) = 0
               enddo
               count = 1
            else
               count = count + 1
               do ir=1,3
                  deviation = lower_left(ir,is)-presentllc(ir)
                  deviation = mod(deviation + (NRiS(ir)*3)/2,NRiS(ir))
     &                 - NRiS(ir)/2
                  averagedeviation(ir) = averagedeviation(ir)
     &                 + deviation
                  newllc(ir) = presentllc(ir) 
     &                 + NINT(averagedeviation(ir)/DBLE(count))
                  newllc(ir) = mod( newllc(ir) + NRiS(ir),NRiS(ir))
               enddo
C     Set the averages for the previous states:
               do iprev = 1, count
                  is = neighborlist(iB,isi-iprev+1)
                  lower_left(ibb,is)  = newllc(ibb)
                  lower_left(ibbb,is) = newllc(ibbb)
               enddo
            endif
         enddo


         if (parent) then
            do isi=1,nstate
               is = neighborlist(iB,isi)
               write (6,
     &              '("State ",I3,": LLC after (iB=",I1,") : ",3I5)')
     &              is,iB,(lower_left(ir,is),ir=1,3)
            enddo
         endif
      enddo


      return
      end

c     ==================================================================
c     ==================================================================
      SUBROUTINE optimize_llc_indep(neighborlist,dowfopt,nstate)
      IMPLICIT NONE
      include 'system.h'
c      include 'cell.inc'
      include 'response_p.inc'
      integer nstate,neighborlist(3,nstate)
      logical dowfopt(3,nstate)

      integer old_llc(3,nstate)
      pointer (ip_old,old_llc)
      logical done(nstate),alldone
      pointer (ip_done,done)
c      integer presentllc(3),deviation,averagedeviation(3),count
      integer is,iss,ir,inow,inext,dist,maxdist(3),
     &     istot,count,distsum,disttonext,newllc
      real*8  aa(3)

      call memory(ip_old,3*nstate,'OLD_LLC')
      call memory(ip_done,nstate,'DONE')

      aa(1) = a1(1)*a1(1) + a1(2)*a1(2) + a1(3)*a1(3)
      aa(2) = a2(1)*a2(1) + a2(2)*a2(2) + a2(3)*a2(3)
      aa(3) = a3(1)*a3(1) + a3(2)*a3(2) + a3(3)*a3(3)

      do ir=1,3
         aa(ir) = dsqrt(aa(ir))
         maxdist(ir) =  NINT(firstshelldistance / aa(ir) * NRiS(ir))
         if (parent) print *,
     &        ' ***************  maxdist points in ',ir,': ',maxdist(ir)
         do is=1,nstate
            old_llc(ir,is) = lower_left(ir,is)
            neighborlist(ir,is) = 0
         enddo
      enddo



      do ir=1,3
         do is=1,nstate
            done(is) = .false.
         enddo
         inext = 1
         istot = 1
         do while (istot .le. nstate)
            inow = inext
            done(inow) = .true.
            neighborlist(ir,istot) = inow
            istot = istot + 1
            count = 1
            disttonext = nris(ir) ! any large dummy value
            distsum = 0
c            if (parent) print *,'*** NEW START: ir=',ir,'; is=',inow,
c     &           '; llc=',lower_left(ir,inow)
            do is=1,nstate
               if (.not. done(is)) then
                  dist = lower_left(ir,is) - lower_left(ir,inow) 
     &                 + 3*NRis(ir)/2
                  dist = mod(dist, NRiS(ir)) - NRis(ir)/2
C     dist is now the relative grid point displacement of state is
C     with respect to state inow.
                  if (abs(dist) .le. maxdist(ir)) then
                     neighborlist(ir,istot) = is
                     distsum = distsum + dist
                     count = count + 1
                     istot = istot + 1
                     done(is) = .true.
c                     if (parent)
c     &                    print *,'*** FOUND CLOSE: ir=',ir,'; is=',is,
c     &                    '; llc=',lower_left(ir,is)
                  elseif (abs(abs(dist) - 2*maxdist(ir)) 
     &                    .lt. disttonext) then
                     inext = is
                     disttonext = abs(abs(dist) - 2*maxdist(ir))
                  endif
               endif
            enddo
C     NOW the loop is finished. Bring together all those states that
C     have been found to be close enough:
            newllc = mod(lower_left(ir,inow) + NINT(distsum/DBLE(count))
     &            + 2*NRiS(ir)-1, NRiS(ir))+1
c            if (parent) print *,'*** NEW AVG: ir=',ir,
c     &           '; llc=',newllc
            do is=istot-count,istot-1
               iss = neighborlist(ir,is)
               lower_left(ir,iss) = newllc
            enddo
         enddo
      enddo


      if (parent) then
         do ir=1,3
            print *,'*** ir=',ir
            do is=1,nstate
               print *,' old/new (is=',is,
     &              '):  ',old_llc(ir,is), lower_left(ir,is)
            enddo
         enddo
      endif

      call freem(ip_old)

      return
      end

c     ==================================================================
