C     ==================================================================
C     THIS FILE contains all functions and routines for handling
C     problems related to the position operator.
C     ==================================================================



c     ==================================================================
      INTEGER FUNCTION rgrid(position,direction)
C     RETURNS the integer grid index ix/iy/iz of an arbitrary position
C     in space (also outside the box). Convention: 1 <= rgrid <= NR#S
C     where    # = (direction).
c     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      include 'gvec.inc'
      real*8 position(3),lattice(3),projector(3),projection
      integer direction,nris,iprojection
      pointer  (ip_projector,projector),(ip_lattice,lattice)
c     ==--------------------------------------------------------------==
      if (direction .eq. 1) then
         CALL GET_ADDR(IP_projector,b1)
         CALL GET_ADDR(IP_lattice,a1)
         nris=nr1s
      elseif (direction .eq. 2) then
         CALL GET_ADDR(IP_projector,b2)
         CALL GET_ADDR(IP_lattice,a2)
         nris=nr2s
      elseif (direction .eq. 3) then
         CALL GET_ADDR(IP_projector,b3)
         CALL GET_ADDR(IP_lattice,a3)
         nris=nr3s
      else
         call stopgm('RGRID','INVALID DIRECTION')
      endif

      projection = (position(1)*projector(1) +
     &     position(2)*projector(2) +
     &     position(3)*projector(3)) /alat

      iprojection = NINT( (projection - NINT(projection - 0.5d0))
     &     * DBLE(nris)) + 1
      if (iprojection .gt. nris) iprojection=1

      rgrid = iprojection

      return
      end
c     ==================================================================
      SUBROUTINE putongrid(position)
C     CHANGES the position such that it is placed on the nearest 
C     real space gridpoint. No unit-cell-wrapping is done.
c     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      include 'gvec.inc'
      real*8 position(3)
      integer i1,i2,i3
c     ==--------------------------------------------------------------==
      i1 = NINT((position(1)*b1(1)
     &     +position(2)*b1(2)+position(3)*b1(3))*nr1s/alat)
      i2 = NINT((position(1)*b2(1)
     &     +position(2)*b2(2)+position(3)*b2(3))*nr2s/alat)
      i3 = NINT((position(1)*b3(1)
     &     +position(2)*b3(2)+position(3)*b3(3))*nr3s/alat)

      position(1) = i1*a1(1)/DBLE(nr1s)
     &     + i2*a2(1)/DBLE(nr2s) + i3*a3(1)/DBLE(nr3s)
      position(2) = i1*a1(2)/DBLE(nr1s)
     &     + i2*a2(2)/DBLE(nr2s) + i3*a3(2)/DBLE(nr3s)
      position(3) = i1*a1(3)/DBLE(nr1s) 
     &     + i2*a2(3)/DBLE(nr2s) + i3*a3(3)/DBLE(nr3s)

      return
      end
c     ==================================================================
c     ==================================================================
      SUBROUTINE set_origin(istate)
      IMPLICIT NONE
      include 'system.h'
      include 'response_p.inc'
      INTEGER  istate,ik
c     =----------------------------------------------------------------=
      do ik=1,3
        llc_ind(ik) = lower_left(ik,istate)
        llc_val(ik) = lower_left_value(ik,istate)
      enddo
      return
      end
c     ==================================================================
      SUBROUTINE force_xyz(nstate)
c     =----------------------------------------------------------------=
      implicit none
      include    'system.h'
      include    'response_p.inc'
      integer    is,ik,bound,nstate
      real*8     avg

      do ik=1,3
        IF (TFORCE_XYZ(ik)) THEN
          avg = DBLE(lower_left(ik,1))
          do is=2,nstate
            avg = avg + DBLE(bound( 
     &          lower_left(ik,is)-lower_left(ik,1)+nris(ik)/2, 
     &          nris(ik)) - nris(ik)/2) / DBLE(nstate)
          enddo
          do is=1,nstate
            lower_left(ik,is) = bound(NINT(avg),nris(ik))
          enddo
        ENDIF
      enddo

      return
      end
c     ==================================================================
      SUBROUTINE calc_lower_left_optimize(c0,d0,nstate,psi,scr)
c     =----------------------------------------------------------------=
c     PARALLEL
c     EXPECTS as input the gnd-state wavefunction c0.
c     OUTPUT  is the integer array lower_left which will
c     \       be set to the indices of the lower left corner of the
c     \       box which is best for the wavefunction c0(*),
C     \       WHERE ITS OVERLAP WITH SURROUNDING ORBITALS IS
C     \       ALSO TAKEN INTO CONSIDERATION.
C     ***     NEW VERSION. Uses the G-space variant to calculate
C     ***     the penalty functional:
C     ***     Int[  |psi_i|^2   exp  -(x-x0)^2 / 2 sigma^2 ]
C     ***     Evalutation in G-space.
C     ***     Final formula for Penalty[x0]:
C     ***     Sum[g: g.B2=0 AND g.B3=0 AND |g| < sqrt(20)/sigma ]
C     ***         exp - 0.5 (2 pi sigma g.B1)^2 
C     ***         Re{ rho(g)  exp - 2 pi i  x0  g.B1 }
C     
C     ***     the sqrt(20) corresponds to an exp(-(..)^2) of 10^-10.
c     =----------------------------------------------------------------=
C     sigma[response_p.inc] is the spread of the damping 
C     Gaussian in units of the 
C     whole cell. Thus 0.1 means that the Gaussian spread is 0.1*alat.
c     =----------------------------------------------------------------=
      IMPLICIT   NONE
      include    'system.h'
      include    'cppt.inc'     ! GK...
      include    'cnst.inc'     ! PI...
      include    'fft.inc'      ! MAXFFT...
      include    'irat.inc'     ! IRAT...
      include    'geq0.inc'
      include    'response_p.inc'
C
C     Arguments
      integer       nstate
      complex*16    c0(ngw,nstate),d0(ngw,nstate)
      real*8        psi(*),scr(*)
C     Variables
      complex*16    rho(*)      ! to access the SCR as a complex array
      pointer       (ip_rho,rho)
      integer       max_jkl,ii,ij,ik,il,ggj,ggk,ggl
      real*8        DOT_GA,DOTP
      external   DOTP
      real*8        penalty(3,*),min_penalty,
     &    alpha
      pointer       (ip_penalty,penalty)
      integer       is,js,ks,ig,ig1,ir,msglen

      integer       ig_indexarray(3,100)
C     This array contains the numbers IG of the G-vectors G which
C     satisfy  G.B2 = 0  and  G.B3 = 0  (for ig_indexarray(1,*))
C     resp.    G.B1 = 0  and  G.B3 = 0  (for ig_indexarray(2,*)) etc.

c     OVERLAP quantities:
      real*8        density_overlap(nstate,nstate)
      logical       found_new_state,tag(nstate),done(nstate)
      pointer       (ip_density_overlap,density_overlap),
     &              (ip_tag,tag),(ip_done,done)
c     =----------------------------------------------------------------=
      call memory(ip_density_overlap,nstate*nstate,'densovl')
      call azzero(density_overlap,nstate*nstate)
      call memory(ip_tag,nstate,'nmr-tag')
      call azzero(tag,nstate)
      call memory(ip_done,nstate,'nmr-done')
      call azzero(done,nstate)
      call memory(ip_penalty, 3*max(nr1s,nr2s,nr3s),'penalty')
      call azzero(   penalty, 3*max(nr1s,nr2s,nr3s))

      max_jkl=NINT(2.d0/sigma)
      if (max_jkl .ge. 100)  call stopgm('NMR/LLCOPT',
     &    'INTERNAL ERROR, sigma too small (<0.02).')
      CALL GET_ADDR(IP_rho,scr)


C     The ig - Index array is now built up. A G-vector (ig) is "good"
C     if two of its reciprocal Miller indices, G.B1, G.B2 and G.B3 are zero.
C     Then, the corresponding IG-value is stored in the array
C     "ig_indexarray".
      ij=0
      il=0
      ik=0
      ig1=1
      if (geq0) ig1=2
      do ig=ig1,ngw
        ggj = NINT(DOT_GA(IG,1))
        ggk = NINT(DOT_GA(IG,2))
        ggl = NINT(DOT_GA(IG,3))
        if (ggk.eq.0 .and. ggl.eq.0 .and. ggj .le. 6.8d0/sigma) then
          ij = ij + 1
          ig_indexarray(1,ij) = ig
        elseif (ggj.eq.0 .and. ggl.eq.0 .and. ggk .le. 6.8d0/sigma) then
          ik = ik + 1
          ig_indexarray(2,ik) = ig
        elseif (ggj.eq.0 .and. ggk.eq.0 .and. ggl .le. 6.8d0/sigma) then
          il = il + 1
          ig_indexarray(3,il) = ig
        endif
        if (ij.gt.max_jkl .or. ik.gt.max_jkl .or. il.gt.max_jkl) then
          write (6,'(A,I3,A,I6,4(A,I3))') 'PROC ',MEPOS,': IG=',ig,
     &        '; jkl=(',ggj,',',ggk,',',ggl,'), max=',max_jkl
          call stopgm('NMR/LLC',
     &        'INTERNAL ERROR. Contact the STUPIDO PROGRAMMER')
        endif
      enddo
C     ij, ik, and il now contain the number of g-vectors found.
C     =----------------------------------------------------------------=
C     The orbital density needs to be computed. The orbital densities are
C     stored in G space in the D0 array and the overlap between
C     these densities is done:
      call azzero(scr,2*nnr1)
      call azzero(psi,2*nnr1)
      do is=1,nstate
        call fftTOr(c0(1,is),scr,psi,ngw,.false.)
        do ir=1,nnr1
          scr(ir) = scr(ir)*scr(ir)
        enddo
        call fftTOg(scr,d0(1,is),psi,ngw,.true.)
      enddo
C     create density overlap matrix, stored in density_overlap:
      do is=2,nstate
        do js=1,is-1
          density_overlap(is,js) = DOTP(ngw,d0(1,is),d0(1,js))
          density_overlap(js,is) = density_overlap(is,js)
        enddo
      enddo
#ifdef PARALLEL
      call glosum(nstate*nstate,density_overlap)
#endif
      do is=1,nstate
        do js=1,nstate
          if (density_overlap(is,js).lt.-1d0) then
            write(6,'(A,I3,A,2I5,A,F20.10)') 
     &          'PROC ',ME,': STATES ',is,js,
     &          ' ==> ',density_overlap(is,js)
            call stopgm('LLC/OPT','negative density overlap')
          endif
        enddo
      enddo
c     =----------------------------------------------------------------=
C     Now, all those densities are summed together where the overlap is
C     larger than the predefined value OVERLAP_THREASHOLD
      do is=1,nstate
        done(is) = .false.      ! initialization
      enddo
      do is=1,nstate
        if (.not. done(is)) then
          do js=1,nstate
            tag(js) = .false.   ! sub-initialization
          enddo
          tag(is) = .true.
          found_new_state = .true.
          do while (found_new_state)
            found_new_state = .false.
            do js=1,nstate
              if (tag(js)) then
                do ks=1,nstate
                  if (density_overlap(js,ks).gt.OVERLAP_THREASHOLD) 
     &                then
                    if (.not. tag(ks)) then
                      tag(ks) = .true.
                      found_new_state = .true.
                    endif
                  endif
                enddo
              endif
            enddo
          enddo                 ! "find new states" -- loop.
          do js=1,nstate
            if (tag(js)) done(js) = .true.
          enddo
C     Now, in "tag()", we have all those states which we want to consider
C     together. We have to add up their densities:
          call azzero(rho,2*ngw)
          do js=1,nstate
            if (tag(js)) then
              call DAXPY(2*ngw,1d0,d0(1,js),1,rho,1)
              if (parent) 
     &            write (6,'(A,I6,A,I6)') 'State group ',is,
     &            ': Joining state ',js
            endif
          enddo


c     =----------------------------------------------------------------=
C     Next, the penalty functional is calculated
C     for the accumulated orbital densities
C     \  foreach point on the three axis
C     \     using each one of the previously determined G-vectors.
c     =----------------------------------------------------------------=
C     Calculate the actual penalty values in the "first" direction.
C     Formula: sum_{g: k=0, l=0} exp(-0.5 (2pi j sigma)^2)
C     \                               exp(2pi i  ix/Nx j)
          call azzero(penalty,3*max(nr1s,nr2s,nr3s))
c     do ir=NRXPL(MEPOS,1),NRXPL(MEPOS,2)
          do ir=1,nr1s          ! NB: NOT from NRXPL(1) to (2)!!!
            alpha = DBLE(ir-1)/DBLE(NR1S)
            if (geq0) penalty(1,ir) = 0.5d0*DREAL(RHO(1))
            do ii=1,ij          ! sum over all the g vectors 
                                ! in the "first" direction
              ig=ig_indexarray(1,ii)
              penalty(1,ir) = penalty(1,ir)
     &            + dexp(-0.5d0*(sigma*DOT_GA(ig,1))**2) ! the damping
     &            * DREAL(
     &            exp(DCMPLX(0d0,+DOT_GA(ig,1)*alpha)) ! the phase
     &            * rho(ig))
            enddo               ! the g-vectors
          enddo                 ! the points ir=1..NR1S

C     The "second" direction: Same formula except that g.A2 != 0
C     and g.A1 = g.A3 = 0
          do ir=1,nr2s
            alpha = DBLE(ir-1)/DBLE(NR2S)
            if (geq0) penalty(2,ir) = 0.5d0*DREAL(RHO(1))
            do ii=1,ik          ! sum over all the g vectors 
                                ! in the "first" direction
              ig=ig_indexarray(2,ii)
              penalty(2,ir) = penalty(2,ir)
     &            + exp(-0.5d0*(sigma*DOT_GA(ig,2))**2) ! the damping
     &            * DREAL(
     &            exp(DCMPLX(0d0,+DOT_GA(ig,2)*alpha)) ! the phase
     &            * rho(ig))
            enddo               ! the g-vectors
          enddo                 ! the points ir=1..NR2S

C     The "third" direction: Same formula except that g.A3 != 0
C     and g.A1 = g.A2 = 0
          do ir=1,nr3s
            alpha = DBLE(ir-1)/DBLE(NR3S)
            if (geq0) penalty(3,ir) = 0.5d0*DREAL(RHO(1))
            do ii=1,il          ! sum over all the g vectors 
                                ! in the "first" direction
              ig=ig_indexarray(3,ii)
              penalty(3,ir) = penalty(3,ir)
     &            + exp(-0.5d0*(sigma*DOT_GA(ig,3))**2) ! the damping
     &            * DREAL(
     &            exp(DCMPLX(0d0,+DOT_GA(ig,3)*alpha)) ! the phase
     &            * rho(ig))
            enddo               ! the g-vectors
          enddo                 ! the points ir=1..NR3S


#ifdef PARALLEL
          call glosum(3*max(nr1s,nr2s,nr3s),penalty)
#endif


C     The minimum is searched, and its coordinates are stored
C     in the lower_left - array:
          min_penalty=1.d3
          do ir=1,NR1S
            if (penalty(1,ir) .lt. min_penalty) then
              min_penalty = penalty(1,ir)
              lower_left(1,is) = ir
            endif
          enddo
          min_penalty=1.d3
          do ir=1,NR2S
            if (penalty(2,ir) .lt. min_penalty) then
              min_penalty = penalty(2,ir)
              lower_left(2,is) = ir
            endif
          enddo
          min_penalty=1.d3
          do ir=1,NR3S
            if (penalty(3,ir) .lt. min_penalty) then
              min_penalty = penalty(3,ir)
              lower_left(3,is) = ir
            endif
          enddo


          do js=1,nstate
            if (tag(js)) then
              lower_left(1,js) = lower_left(1,is)
              lower_left(2,js) = lower_left(2,is)
              lower_left(3,js) = lower_left(3,is)

              lower_left_value(1,js) = - NR1S/2
              lower_left_value(2,js) = - NR2S/2
              lower_left_value(3,js) = - NR3S/2
            endif
          enddo
        endif                   ! if not done.
      enddo                     ! is

      call freem(ip_density_overlap)
      call freem(ip_tag)
      call freem(ip_done)
      call freem(ip_penalty)

      return
      end
c     ==================================================================
      SUBROUTINE calc_lower_left_new(c0,nstate,psi,scr)
c     =----------------------------------------------------------------=
c     PARALLEL
c     EXPECTS as input the gnd-state wavefunction c0.
c     OUTPUT  is the integer array lower_left which will
c     \       be set to the indices of the lower left corner of the
c     \       box which is best for the wavefunction c0(*).
C     ***     NEW VERSION. Uses the G-space variant to calculate
C     ***     the penalty functional:
C     ***     Int[  |psi_i|^2   exp  -(x-x0)^2 / 2 sigma^2 ]
C     ***     Evalutation in G-space.
C     ***     Final formula for Penalty[x0]:
C     ***     Sum[g: g.B2=0 AND g.B3=0 AND |g| < sqrt(20)/sigma ]
C     ***         exp - 0.5 (2 pi sigma g.B1)^2 
C     ***         Re{ rho(g)  exp - 2 pi i  x0  g.B1 }
C     
C     ***     the sqrt(20) corresponds to an exp(-(..)^2) of 10^-10.
c     =----------------------------------------------------------------=
C     sigma[response_p.inc] is the spread of the damping 
C     Gaussian in units of the 
C     whole cell. Thus 0.1 means that the Gaussian spread is 0.1*alat.
c     =----------------------------------------------------------------=
      IMPLICIT   NONE
      include    'system.h'
      include    'cppt.inc'     ! GK...
      include    'cnst.inc'     ! PI...
      include    'fft.inc'      ! MAXFFT...
      include    'irat.inc'     ! IRAT...
      include    'geq0.inc'
      include    'response_p.inc'
C
C     Arguments
      integer       nstate
      complex*16    c0(ngw,nstate)
      real*8        psi(*),scr(*)
C     Variables
      complex*16  rho(*)        ! to access the SCR as a complex array
      pointer       (ip_rho,rho)
      integer       max_jkl,ii,ij,ik,il,ggj,ggk,ggl
      real*8        DOT_GA
      real*8        penalty(3,*),min_penalty,
     &    alpha
      pointer       (ip_penalty,penalty)
      integer       is,ig,ig1,ir,msglen

      integer       ig_indexarray(3,100)
C     This array contains the numbers IG of the G-vectors G which
C     satisfy  G.B2 = 0  and  G.B3 = 0  (for ig_indexarray(1,*))
C     resp.    G.B1 = 0  and  G.B3 = 0  (for ig_indexarray(2,*)) etc.

      max_jkl=NINT(2.d0/sigma)
      if (max_jkl .ge. 100)  call stopgm('NMR/LLC',
     &    'INTERNAL ERROR, sigma too small (<0.02).')
      CALL GET_ADDR(IP_rho,scr)

      call memory(ip_penalty, 3*max(nr1s,nr2s,nr3s),'penalty')

c     calculate the damping exponentials
c      do ij=1,max_jkl
c        damping(ij) = exp( -  0.5 * (2*3.141592654d0*sigma*DBLE(ij))**2)
c      enddo


C     The ig - Index array is now built up. A G-vector (ig) is "good"
C     if two of its reciprocal Miller indices, G.B1, G.B2 and G.B3 are zero.
C     Then, the corresponding IG-value is stored in the array
C     "ig_indexarray".
      ij=0
      il=0
      ik=0
      ig1=1
      if (geq0) ig1=2
      do ig=ig1,ngw
        ggj = NINT(DOT_GA(IG,1))
        ggk = NINT(DOT_GA(IG,2))
        ggl = NINT(DOT_GA(IG,3))
        if (ggk.eq.0 .and. ggl.eq.0 .and. ggj .le. 6.8d0/sigma) then
          ij = ij + 1
          ig_indexarray(1,ij) = ig
        elseif (ggj.eq.0 .and. ggl.eq.0 .and. ggk .le. 6.8d0/sigma) then
          ik = ik + 1
          ig_indexarray(2,ik) = ig
        elseif (ggj.eq.0 .and. ggk.eq.0 .and. ggl .le. 6.8d0/sigma) then
          il = il + 1
          ig_indexarray(3,il) = ig
        endif
        if (ij.gt.max_jkl .or. ik.gt.max_jkl .or. il.gt.max_jkl) then
          write (6,'(A,I3,A,I6,4(A,I3))') 'PROC ',MEPOS,': IG=',ig,
     &        '; jkl=(',ggj,',',ggk,',',ggl,'), max=',max_jkl
          call stopgm('NMR/LLC',
     &        'INTERNAL ERROR. Contact the STRONZO PROGRAMMER')
        endif
      enddo
C     ij, ik, and il now contain the number of g-vectors found.



C     Here, the penalty functional is calculated
C     foreach state
C     \  foreach point on the three axis
C     \     using each one of the previously determined G-vectors.
      do is=1,nstate

C     The orbital density needs to be computed:
        call azzero(scr,2*nnr1)
        call azzero(psi,2*nnr1)
        call fftTOr(c0(1,is),scr,psi,ngw,.false.)
        do ir=1,nnr1
          scr(ir) = scr(ir)*scr(ir)
        enddo
        call fftTOg(scr,scr,psi,nhg,.true.)
                                ! NB: SCR == RHO, see variable definitions.


C     Calculate the actual penalty values in the "first" direction.
C     Formula: sum_{g: k=0, l=0} exp(-0.5 (2pi j sigma)^2)
C     \                               exp(2pi i  ix/Nx j)
        call azzero(penalty,3*max(nr1s,nr2s,nr3s))
c        do ir=NRXPL(MEPOS,1),NRXPL(MEPOS,2)
        do ir=1,nr1s            ! NB: NOT from NRXPL(1) to (2)!!!
          alpha = DBLE(ir-1)/DBLE(NR1S)
          if (geq0) penalty(1,ir) = 0.5d0*DREAL(RHO(1)) ! the G=0 contribution.
          do ii=1,ij            ! sum over all the g vectors
                                ! in the "first" direction
            ig=ig_indexarray(1,ii)
            penalty(1,ir) = penalty(1,ir)
     &          + exp(-0.5d0*(sigma*DOT_GA(ig,1))**2) ! the damping
     &          * DREAL(
     &          exp(DCMPLX(0d0,+DOT_GA(ig,1)*alpha)) ! the phase
     &          * rho(ig))
          enddo                 ! the g-vectors
        enddo                   ! the points ir=1..NR1S

C     The "second" direction: Same formula except that g.A2 != 0
C     and g.A1 = g.A3 = 0
        do ir=1,nr2s
          alpha = DBLE(ir-1)/DBLE(NR2S)
          if (geq0) penalty(2,ir) = 0.5d0*DREAL(RHO(1))
          do ii=1,ik
            ig=ig_indexarray(2,ii)
            penalty(2,ir) = penalty(2,ir)
     &          + exp(-0.5d0*(sigma*DOT_GA(ig,2))**2) ! the damping
     &          * DREAL(
     &          exp(DCMPLX(0d0,+DOT_GA(ig,2)*alpha)) ! the phase
     &          * rho(ig))
          enddo                 ! the g-vectors
        enddo                   ! the points ir=1..NR2S

C     The "third" direction: Same formula except that g.A3 != 0
C     and g.A1 = g.A2 = 0
        do ir=1,nr3s
          alpha = DBLE(ir-1)/DBLE(NR3S)
          if (geq0) penalty(3,ir) = 0.5d0*DREAL(RHO(1))
          do ii=1,il
            ig=ig_indexarray(3,ii)
            penalty(3,ir) = penalty(3,ir)
     &          + exp(-0.5d0*(sigma*DOT_GA(ig,3))**2) ! the damping
     &          * DREAL(
     &          exp(DCMPLX(0d0,+DOT_GA(ig,3)*alpha)) ! the phase
     &          * rho(ig))
          enddo                 ! the g-vectors
        enddo                   ! the points ir=1..NR3S


#ifdef PARALLEL
        call glosum(3*max(nr1s,nr2s,nr3s),penalty)
#endif


C     The minimum is searched, and its coordinates are stored
C     in the lower_left - array:
        if (parent) then
          min_penalty=1.d0
          do ir=1,NR1S
            if (penalty(1,ir) .lt. min_penalty) then
              min_penalty = penalty(1,ir)
              lower_left(1,is) = ir
            endif
          enddo
          min_penalty=1.d0
          do ir=1,NR2S
            if (penalty(2,ir) .lt. min_penalty) then
              min_penalty = penalty(2,ir)
              lower_left(2,is) = ir
            endif
          enddo
          min_penalty=1.d0
          do ir=1,NR3S
            if (penalty(3,ir) .lt. min_penalty) then
              min_penalty = penalty(3,ir)
              lower_left(3,is) = ir
            endif
          enddo
        endif                   ! parent.


        lower_left_value(1,is) = - NR1S/2
        lower_left_value(2,is) = - NR2S/2
        lower_left_value(3,is) = - NR3S/2

      enddo                     ! the states is=1,nstate

#ifdef PARALLEL
      msglen = 3*nstate*8/IRAT
      call MY_BCAST(lower_left,msglen,source,allgrp)
#endif

      call freem(ip_penalty)

      return
      end
c     ==================================================================
      subroutine apply_op_rx(inp,beta,outp,alpha,coord)
c     ==================================================================
c     PARALLEL
c     The most tricky routine in the whole NMR calculation.
c     Read with care. ;-)
c     =----------------------------------------------------------------=
c     CALCULATES:
c     outp = alpha * outp  +  beta * r_coord * inp
c     the two arrays may be identical (!)
c
c     applies an (1 - exp-(x^2)/l^2) where x is the distance to the
c     nearest LLC point. l is nr_s/20.
c     =----------------------------------------------------------------=
      implicit none
      include 'system.h'
      include 'coor.inc'
      include 'cppt.inc'
      include 'fft.inc'
      include 'response_p.inc'
c     =----------------------------------------------------------------=
c     Arguments:
      real*8   inp(kr1,kr2,kr3),outp(kr1,kr2,kr3),alpha,beta
      integer  coord
c     =----------------------------------------------------------------=
c     variables:
      real*8   Rop0,Rop,delta,x,in,out,fsmooth1,fsmooth2,fsmooth3,
     &    aN1,aN2,aN3           ! a_i(coord) / N_i
#define divisor 20
c     the number nr1s is divided by to obtain lambda
      integer  maxnris,i1,i2,i3,isub,bound,i_llc,ind_ll1_rel
      real*8   smoothing(3,0:*),lambda2(3),delta2,Ndelta2(3),coeff
      pointer  (ip_smoothing,smoothing)
      save     ip_smoothing,maxnris,aN1,aN2,aN3
      data     maxnris /0/
c     SMOOTHING: indices go from 0 to maxnris, 0 means that we are
c     at the LLC (smoothing-factor has value 0).
c     =----------------------------------------------------------------=
      call tiset('     r_psi',isub)

      if (coord .EQ. 0 .AND. maxnris .NE. 0) then
        call freem(ip_smoothing)
        maxnris = 0
        return
      endif

      if (maxnris .EQ. 0) then  ! smoothing not yet allocated...
        maxnris = max(nr1s,nr2s,nr3s)
        call memory(ip_smoothing,3*(maxnris+2),'SMOOTH')
        lambda2(1) = DBLE(nr1s*nr1s)/DBLE(divisor*divisor)
        lambda2(2) = DBLE(nr2s*nr2s)/DBLE(divisor*divisor)
        lambda2(3) = DBLE(nr3s*nr3s)/DBLE(divisor*divisor)
        if (TSMOOTHING) then
          do i1=0,maxnris+1
            delta2            = DBLE(i1*i1)
            Ndelta2(1)        = DBLE((i1-nr1s) * (i1-nr1s))
            Ndelta2(2)        = DBLE((i1-nr2s) * (i1-nr2s))
            Ndelta2(3)        = DBLE((i1-nr3s) * (i1-nr3s))
            smoothing(1,i1) = (1 - exp(- delta2   /lambda2(1)))
     &          *             (1 - exp(-Ndelta2(1)/lambda2(1)))
            smoothing(2,i1) = (1 - exp(- delta2   /lambda2(2)))
     &          *             (1 - exp(-Ndelta2(2)/lambda2(2)))
            smoothing(3,i1) = (1 - exp(- delta2   /lambda2(3)))
     &          *             (1 - exp(-Ndelta2(3)/lambda2(3)))
          enddo
        else                    ! no smoothing:
          do i1=0,maxnris+1
            smoothing(1,i1) = 1.d0
            smoothing(2,i1) = 1.d0
            smoothing(3,i1) = 1.d0
          enddo
        endif
      endif


      if (coord .lt. 1 .or. coord .gt. 3) then
        write (6,*) '*** apply_op_rx: WRONG COORDINATE. Nothing done.'
        return
      endif


      aN1   = a1(coord) / DBLE(NR1S)
      aN2   = a2(coord) / DBLE(NR2S)
      aN3   = a3(coord) / DBLE(NR3S)
      ind_ll1_rel    =  llc_ind(1) - (NRXPL(MEPOS,1)-1)
      Rop0  = (llc_val(3) +       1 + nr3s-llc_ind(3)) * aN3
     &    +   (llc_val(2) +       1 + nr2s-llc_ind(2)) * aN2
     &    +   (llc_val(1) + bound(2 + nr1s-ind_ll1_rel,nr1s) -1) * aN1


C     NB: After one X-loop in parallel, the Rop-variable does NOT
C     have the value it had before the loop: We add nr1 times a1/nr1S, but
C     we substract a1. The result is NOT zero. (!!)

      if (ind_ll1_rel .le. 1) then
        do i3          =  1,nr3
          fsmooth3     =  beta     * smoothing(3,abs(i3-llc_ind(3)))
          if (i3 .EQ. llc_ind(3))
     &        Rop0     =  Rop0 - a3(coord)
          do i2        =  1,llc_ind(2)-1
            fsmooth2   =  fsmooth3 * smoothing(2,llc_ind(2)-i2)
            Rop        =  Rop0
            do i1      =  1,nr1
              fsmooth1 =  fsmooth2 * smoothing(1,i1-ind_ll1_rel)
              out      =  alpha * outp(i1,i2,i3)
              in       =  fsmooth1 * inp(i1,i2,i3)
              outp(i1,i2,i3) = out + Rop * in
              Rop      =  Rop + aN1
            enddo               ! i1
            Rop0       =  Rop0 + aN2
          enddo                 ! i2, first loop
          Rop0         =  Rop0 - a2(coord)
          do i2        =  llc_ind(2),nr2
            fsmooth2   =  fsmooth3 * smoothing(2,i2-llc_ind(2))
            Rop        =  Rop0
            do i1      =  1,nr1
              fsmooth1 =  fsmooth2 * smoothing(1,i1-ind_ll1_rel)
              out      =  alpha * outp(i1,i2,i3)
              in       =  fsmooth1 * inp(i1,i2,i3)
              outp(i1,i2,i3) = out + Rop * in
              Rop      =  Rop + aN1
            enddo               ! i1
            Rop0       =  Rop0 + aN2
          enddo                 ! i2, second loop
          Rop0         =  Rop0 + aN3
        enddo
      elseif (ind_ll1_rel .gt. nr1) then
                                ! THEN the (relative) LLC is 
                                ! (right) outside of MY cell.
        do i3          =  1,nr3
          fsmooth3     =  beta     * smoothing(3,abs(i3-llc_ind(3)))
          if (i3 .EQ. llc_ind(3))
     &        Rop0     =  Rop0 - a3(coord)
          do i2        =  1,llc_ind(2)-1
            fsmooth2   =  fsmooth3 * smoothing(2,llc_ind(2)-i2)
            Rop        =  Rop0
            do i1      =  1,nr1
              fsmooth1 =  fsmooth2 * smoothing(1,ind_ll1_rel-i1)
              out      =  alpha * outp(i1,i2,i3)
              in       =  fsmooth1 * inp(i1,i2,i3)
              outp(i1,i2,i3) = out + Rop * in
              Rop      =  Rop + aN1
            enddo               ! i1
            Rop0       =  Rop0 + aN2
          enddo                 ! i2, first loop
          Rop0         =  Rop0 - a2(coord)
          do i2        =  llc_ind(2),nr2
            fsmooth2   =  fsmooth3 * smoothing(2,i2-llc_ind(2))
            Rop        =  Rop0
            do i1      =  1,nr1
              fsmooth1 =  fsmooth2 * smoothing(1,ind_ll1_rel-i1)
              out      =  alpha * outp(i1,i2,i3)
              in       =  fsmooth1 * inp(i1,i2,i3)
              outp(i1,i2,i3) = out + Rop * in
              Rop      =  Rop + aN1
            enddo               ! i1
            Rop0       =  Rop0 + aN2
          enddo                 ! i2, second loop
          Rop0         =  Rop0 + aN3
        enddo
      else                      ! THEN, 1 < ind_ll1_rel < nr1+1,
                                ! i.e. the llc is within MY cell
        do i3          =  1,nr3
          fsmooth3     =  beta     * smoothing(3,abs(i3-llc_ind(3)))
          if (i3 .EQ. llc_ind(3))
     &        Rop0     =  Rop0 - a3(coord)
          do i2        =  1,llc_ind(2)-1
            fsmooth2   =  fsmooth3 * smoothing(2,llc_ind(2)-i2)
            Rop        =  Rop0
            do i1      =  1,ind_ll1_rel-1
              fsmooth1 =  fsmooth2 * smoothing(1,ind_ll1_rel-i1)
              out      =  alpha * outp(i1,i2,i3)
              in       =  fsmooth1 * inp(i1,i2,i3)
              outp(i1,i2,i3) = out + Rop * in
              Rop      =  Rop + aN1
            enddo               ! i1, first loop
            Rop        =  Rop - a1(coord) ! here is THE wrap.
            do i1      =  ind_ll1_rel,nr1
              fsmooth1 =  fsmooth2 * smoothing(1,i1-ind_ll1_rel)
              out      =  alpha * outp(i1,i2,i3)
              in       =  fsmooth1 * inp(i1,i2,i3)
              outp(i1,i2,i3) = out + Rop * in
              Rop      =  Rop + aN1
            enddo               ! i1, second loop
            Rop0       =  Rop0 + aN2
          enddo                 ! i2, first loop
          Rop0         =  Rop0 - a2(coord)
          do i2        =  llc_ind(2),nr2
            fsmooth2   =  fsmooth3 * smoothing(2,i2-llc_ind(2))
            Rop        =  Rop0
            do i1      =  1,ind_ll1_rel-1
              fsmooth1 =  fsmooth2 * smoothing(1,ind_ll1_rel-i1)
              out      =  alpha * outp(i1,i2,i3)
              in       =  fsmooth1 * inp(i1,i2,i3)
              outp(i1,i2,i3) = out + Rop * in
              Rop      =  Rop + aN1
            enddo               ! i1, first loop
            Rop        =  Rop - a1(coord) ! here is THE wrap.
            do i1      =  ind_ll1_rel,nr1
              fsmooth1 =  fsmooth2 * smoothing(1,i1-ind_ll1_rel)
              out      =  alpha * outp(i1,i2,i3)
              in       =  fsmooth1 * inp(i1,i2,i3)
              outp(i1,i2,i3) = out + Rop * in
              Rop      =  Rop + aN1
            enddo               ! i1, second loop
            Rop0       =  Rop0 + aN2
          enddo                 ! i2, second loop
          Rop0         =  Rop0 + aN3
        enddo
      endif

      call tihalt('     r_psi',isub)
      return
      end
c     ==================================================================
      integer function bound(i,imax)
c     returns i mod imax
      implicit none
      integer i,imax
      if (i .lt. 1) then
        bound = i + imax*(1+INT(-i/imax))
      elseif (i .gt. imax) then
        bound = i - imax*INT((i-1)/imax)
      else
        bound = i
      endif
      return
      end
c     ==================================================================
      REAL*8 FUNCTION TRIVIAL_DIST(pos1,pos2)
      implicit none
      real*8 pos1(3),pos2(3),d
      d = (pos1(1)-pos2(1)) **2 
     &    + (pos1(2)-pos2(2)) **2 + (pos1(3)-pos2(3)) **2
      TRIVIAL_DIST = dsqrt(d)
      return
      end
c     ==================================================================
      REAL*8 FUNCTION PBC_DIST(pos1,pos2)
C     Calculates and returns the distance (in a.u.) of two given
C     positions in space adopting the minimum image convention.
C     The positions must be given as a 3D-position vector in a.u.
      implicit none
      include 'system.h'
      real*8 pos1(3),pos2(3),pos3(3),d,min_dist,TRIVIAL_DIST
      integer n1,n2,n3,ik

      min_dist = 100000d0
      do n1=-2,2
        do n2=-2,2
          do n3=-2,2
            do ik=1,3
              pos3(ik) = pos2(ik) + n1*a1(ik) + n2*a2(ik) + n3*a3(ik)
              d        = TRIVIAL_DIST(pos1,pos3)
              if ( d .lt. min_dist ) min_dist = d
            enddo
          enddo
        enddo
      enddo

      pbc_dist = min_dist
      return
      end
c     ==================================================================
      INTEGER FUNCTION MIN_DIST(pos1,pos2,N_PBC)
C     computes pos1 - pos2 in the minimum image convention, using
C     the periodicity defined through N_PBC.
C     The actual calculation is:
C     min_dist(r1,r2,N) = min [ r1 - r2,  r1 - r2 + N, r1 - r2 - N ]
C     The minimum procedure is understood as the minimum absolute value,
C     whereas the value returned from this function will contain the 
C     correct sign.
      implicit none
      integer pos1,pos2,N_PBC,r12(5),i

      do i=-2,2
         r12(i+3) = pos1-pos2 + i*N_PBC
c         d12(i+3) = abs(r12(i+3))
      enddo

      MIN_DIST = r12(1)
      do i=2,5
         if (abs(r12(i)) .lt. abs(min_dist)) then
C     identical to the previous routine would be ".le" instead of .lt.
            min_dist = r12(i)
         endif
      enddo


      return
      end
c     ==================================================================
c     ==================================================================
      SUBROUTINE calc_min_dist(is1,is2,dij)
C     Returns in dij the minimum image convention vector that links the 
C     two virtual cells of states is1 and is2: dij = r_is2 - r_is1,
C     i.e. the connection vector FROM is1 TO is2.
C     NB: This routine assumes that all lower_left_value being identical!!
C     VARIANT: COMPLETE DIRECT SPACE DISTANCE CALCULATION
      IMPLICIT NONE
      include 'system.h'
      include 'response_p.inc'
C     Arguments:
      integer     is1,is2
      real*8      dij(3)
C     Variables:
      integer     ir,n1,n2,n3
      real*8      dij0(3),dij_test(3),Ndij,Ndij0,Ndij_test

      Ndij = 0d0
      do ir=1,3
         dij0(ir) = 
     &        (lower_left(1,is2)-lower_left(1,is1)) *a1(ir)/NR1S +
     &        (lower_left(2,is2)-lower_left(2,is1)) *a2(ir)/NR2S +
     &        (lower_left(3,is2)-lower_left(3,is1)) *a3(ir)/NR3S
         Ndij0 = Ndij0 + dij0(ir)*dij0(ir)
         if (.NOT.
     &        (lower_left_value(ir,is1).EQ.lower_left_value(ir,is2)))
     &        then
            write (6,*) is1,is2,ir,
     &           lower_left_value(ir,is1),lower_left_value(ir,is2)
            call stopgm('CALC_MIN_DIST','LLC-values are not equal!!')
         endif
      enddo
      Ndij = Ndij0
      call DCOPY(3,dij0,1,dij,1)

      do n1=-1,1
         do n2=-1,1
            do n3=-1,1
               Ndij_test = 0d0
               do ir=1,3
                  dij_test(ir) = dij0(ir)
     &                 + n1*a1(ir) + n2*a2(ir) + n3*a3(ir)
                  Ndij_test = Ndij_test + dij_test(ir)*dij_test(ir)
               enddo
               if (Ndij_test .lt. Ndij) then
                  call DCOPY(3,dij_test,1,dij,1)
                  Ndij = Ndij_test
               endif
            enddo
         enddo
      enddo

      return
      end
c     ==================================================================
c     ==================================================================
      SUBROUTINE calc_min_dist_alat(is1,is2,dij)
C     Returns in dij the minimum image convention vector that links the 
C     two virtual cells of states is1 and is2: dij = r_is2 - r_is1,
C     i.e. the connection vector FROM is1 TO is2.
C     NB: This routine assumes that all lower_left_value being identical!!
C     VARIANT: DISTANCE CALCULATION in units of the lattice vectors.
C     only identical to the previous routine if the cell is 
C     cubic/orthorhombic!
      IMPLICIT NONE
      include 'system.h'
      include 'response_p.inc'
C     Arguments:
      integer     is1,is2
      real*8      dij(3)
C     Variables:
      integer     min_dist,ijk(3),i
      real*8      ibyN(3)


      do i=1,3
         ijk(i)  = min_dist(lower_left(i,is2),lower_left(i,is1),nris(i))
         ibyN(i) = ijk(i) / DBLE(nris(i))
      enddo

C     Direction A1:
      do i=1,3
         dij(i)  =  ibyN(1)*A1(i) + ibyN(2)*A2(i) + ibyN(3)*A3(i)
      enddo


      return
      end
c     ==================================================================
c     ==================================================================
      SUBROUTINE calc_min_dist_alat_smooth(is1,is2,dij)
C     Returns in dij the minimum image convention vector that links the 
C     two virtual cells of states is1 and is2: dij = r_is2 - r_is1,
C     i.e. the connection vector FROM is1 TO is2.
C     NB: This routine assumes that all lower_left_value being identical!!
C     VARIANT: DISTANCE CALCULATION in units of the lattice vectors.
C     only identical to the previous routine if the cell is 
C     cubic/orthorhombic!
C     ADDITIONAL SMOOTHING for the cases where the distance between the
C     states is roughly half a lattice vector, at least for one of 
C     the lattice directions.
      IMPLICIT NONE
      include 'system.h'
      include 'response_p.inc'
C     Arguments:
      integer     is1,is2
      real*8      dij(3)
C     Variables:
      integer     min_dist,ijk(3),i
      real*8      ibyN(3),smoothing(3),arg,spread
      parameter   (spread=0.1d0)

      do i=1,3
         ijk(i)  = min_dist(lower_left(i,is2),lower_left(i,is1),nris(i))
         ibyN(i) = DBLE(ijk(i)) / DBLE(nris(i))
C     min_dist returns integers (from -N/2 to +N/2)
C     the smoothing is then based on these integers.
         arg     =  dabs(ibyN(i)) - 0.5d0
         smoothing(i) = 1.d0-exp( -0.5d0* (arg/spread)**2 )
      enddo

C     Direction A1:
      do i=1,3
         dij(i)  =  smoothing(1)*ibyN(1)*A1(i) 
     &        + smoothing(2)*ibyN(2)*A2(i) 
     &        + smoothing(3)*ibyN(3)*A3(i)
      enddo


      return
      end
c     ==================================================================
      REAL*8 FUNCTION give_min_dist(is1,is2,iB)
C     Returns the minimum image convention DISTANCE (in a.u.) that links the 
C     two virtual cells of states is1 and is2: dij = r_is2 - r_is1,
C     i.e. the connection vector FROM is1 TO is2.
C     The states are FIRST PROJECTED onto the "iB" plane, i.e. the distance
C     is actually calculated as  || O(iB) x  ( r_is2 - r_is1 ) ||
C     NB: This routine assumes that all lower_left_value being identical!!
      IMPLICIT NONE
      include 'system.h'
      include 'response_p.inc'
C     Arguments:
      integer     is1,is2,iB
C     Variables:
      real*8      dij(3)
      integer     iBB, iBBB
      call calc_min_dist(is1,is2,dij)
      call make_123(iB,iBB,iBBB)
      give_min_dist = dsqrt(dij(iBB)*dij(iBB) + dij(iBBB)*dij(iBBB))
      return
      end
c     ==================================================================
      SUBROUTINE print_LLC(nstate)
      implicit none
      include 'system.h'
      include 'response_p.inc'
      include 'coor.inc'
      include 'ions.inc'
      integer nstate
      integer is,ik,bound,isp,iat,iatsp
      real*8  r_l(3),r_c(3),pbc_dist


      if (.not. parent) return

      write(6,89)
      write(6,90)
      do is=1,nstate
        do ik=1,3
          r_l(ik) = 0d0
          r_c(ik) = 0d0
          r_l(ik) = r_l(ik) + (lower_left(1,is)-1)*a1(ik)/DBLE(NRIS(1))
          r_c(ik) = r_c(ik) 
     &       + (bound(lower_left(1,is)-lower_left_value(1,is),NRIS(1))
     &          -1)
     &       * a1(ik)/DBLE(NRIS(1))
          r_l(ik) = r_l(ik) + (lower_left(2,is)-1)*a2(ik)/DBLE(NRIS(2))
          r_c(ik) = r_c(ik) 
     &       + (bound(lower_left(2,is)-lower_left_value(2,is),NRIS(2))
     &          -1)
     &       * a2(ik)/DBLE(NRIS(2))
          r_l(ik) = r_l(ik) + (lower_left(3,is)-1)*a3(ik)/DBLE(NRIS(3))
          r_c(ik) = r_c(ik) 
     &       + (bound(lower_left(3,is)-lower_left_value(3,is),NRIS(3))
     &          -1)
     &       * a3(ik)/DBLE(NRIS(3))
        enddo

        write(6,81) is,
     &      (r_l(ik),ik=1,3),
     &      (r_c(ik),ik=1,3)
      enddo
      write(6,91)

      return


 81   FORMAT ("* [",I4,"]: ",3F7.2,5X,3F7.2,7X,"*")
 89   FORMAT ("** VIRTUAL CELLS ",30("*"),"*NMR/EPR*RESPONSE*")
 90   FORMAT ("** STATE ** LLC_x  LLC_y  LLC_z  ",
     &    "***  CTR_x  CTR_y  CTR_z *******")
 91   FORMAT (47("*"),"*NMR/EPR*RESPONSE*")
      end
C     ==================================================================
      SUBROUTINE print_dist_R_wc(nstate)
      implicit none
      include 'system.h'
      include 'response_p.inc'
      include 'coor.inc'
      include 'ions.inc'
      integer nstate
      integer is,ik,bound,isp,iat,iatsp
      real*8  r_l(3),r_c(3),pbc_dist

 81   FORMAT ("* [",I4,"]: ",3F7.2,5X,3F7.2,7X,"*")
 70   FORMAT ("* DIST |   ATOM/TOTAL   STATE   DISTANCE [au]")
 71   FORMAT ("* DIST |   ",I4,"/",I4,4X,I5,3X,F12.6)

      if (.not. parent) return

      write (6,*) 'FIRST METHOD: LLC (FROM LOCALIZE)'
      do is=1,nstate
        write (6,70)
        iatsp = 0
        do isp=1,nsp
          do iat=1,na(isp)
            iatsp=iatsp+1
            do ik=1,3
              r_c(ik) = 0d0
              r_c(ik) = r_c(ik) 
     &            + (bound(lower_left(1,is)
     &            -lower_left_value(1,is),NRIS(1))
     &            -1)* a1(ik)/DBLE(NRIS(1))
              r_c(ik) = r_c(ik) 
     &            + (bound(lower_left(2,is)
     &            -lower_left_value(2,is),NRIS(2))
     &            -1)* a2(ik)/DBLE(NRIS(2))
              r_c(ik) = r_c(ik) 
     &            + (bound(lower_left(3,is)
     &            -lower_left_value(3,is),NRIS(3))
     &            -1)* a3(ik)/DBLE(NRIS(3))
            enddo
            write (6,71) iat,iatsp,is,pbc_dist(tau0(1,iat,isp),r_c)
          enddo
        enddo
      enddo

      write (6,*) 'SECOND METHOD: WANNIERCENTERS (FROM LOCALIZE)'
      do is=1,nstate
        write (6,70)
        iatsp = 0
        do isp=1,nsp
          do iat=1,na(isp)
            iatsp=iatsp+1
            write (6,71) iat,iatsp,is,
     &          pbc_dist(tau0(1,iat,isp),wanniercenters(1,is))
          enddo
        enddo
      enddo

      return
      end
