c     ==================================================================
      SUBROUTINE dia_shifts(c0,iB,psi,scr,nstate)
c     ==--------------------------------------------------------------==
C     Computes the diamagnetic ground state contribution to the shifts
C     in the IGLO method: The current for field B along direction iB is
C     j(r0) = e/2  (r0-R_i) x B  * |psi0_i(r0)|^2
C     With the help of j(r), we calculate the shift contributions. The
C     output is directly stored in the shift matrix.
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'fft.inc'
      include 'response_p.inc'
#ifdef __PGI
#define ACHAR CHAR
#endif
c     ==--------------------------------------------------------------==
C     Arguments
      integer    nstate,iB
      complex*16 c0(ngw,nstate),psi(*),scr(maxfft,4)
C     Local Variables
      integer    is,iBB,iBBB,ir,iBind
      complex*16 current(nhg,2),GvecJ(nhg)
      real*8     rhoi(nnr1),overall_factor
      pointer    (ip_current,current),(ip_rhoi,rhoi),(ip_GvecJ,GvecJ)
c     ==--------------------------------------------------------------==
      CALL GET_ADDR(IP_rhoi,scr(1,1))
      CALL GET_ADDR(IP_current,scr(1,2))
      CALL GET_ADDR(IP_GvecJ,scr(1,4))
      call azzero(current(1,1),2*maxfft)
      call azzero(current(1,2),2*maxfft)
      call azzero(rhoi,        2*maxfft)
      call azzero(GvecJ,       2*maxfft)

      call make_123(iB,iBB,iBBB)
C     j(*,1) holds j_iBB, j(*,2) holds j_iBBB

      do is=1,nstate
        call fftTOr(c0(1,is),rhoi,psi,ngw,.false.)
!$OMP parallel do private(ir)
        DO ir=1,nnr1
          rhoi(ir) = rhoi(ir)**2
        ENDDO

C     The current component j1 = j_iB+1 holds + (r-R)_iB+2 * rho_i,
C     and                   j2 = j_iB+2 holds - (r-R)_iB+1 * rho_i.
C     Compute it:
        call set_origin(is)
        call apply_op_rx(rhoi,+0.5d0,current(1,1),1d0,iBBB)
        call apply_op_rx(rhoi,-0.5d0,current(1,2),1d0,iBB)
      enddo                     ! istates

      call fft2TOg(current(1,1),current(1,1),
     &    current(1,2),current(1,2), psi,nhg,.true.)
C     At this point, we have applied a field B along iB. This field has
C     induced a current J = current(1) * e_iB+1 + current(2) * e_iB+2.
C     This current now induces a field at location R with the strength
C     exp iGR  G x J  / |G|^2. Compute its three (!) components:


      overall_factor = -2d0 * shift_factor * alat
      call DSCAL(2*nhg,overall_factor, current(1,1),1)
      call DSCAL(2*nhg,overall_factor, current(1,2),1)


C     First, treat the contribution...
C     from j_iB+1 = current(1)
C     to Bind_iB+2
C     using G_iB
      call apply_op_GbGG(current(1,1),iB,GvecJ)
      call compute_shift_dotproducts(iB,iBBB,GvecJ,+1d0)

C     from j_iB+1 = current(1)
C     to Bind_iB
C     using G_iBBB
      call apply_op_GbGG(current(1,1),iBBB,GvecJ)
      call compute_shift_dotproducts(iB,iB,GvecJ,-1d0)

C     from j_iB+2 = current(2)
C     to Bind_iB+1
C     using G_iB
      call apply_op_GbGG(current(1,2),iB,GvecJ)
      call compute_shift_dotproducts(iB,iBB,GvecJ,-1d0)

C     from j_iB+2 = current(2)
C     to Bind_iB
C     using G_iB+1
      call apply_op_GbGG(current(1,2),iBB,GvecJ)
      call compute_shift_dotproducts(iB,iB,GvecJ,+1d0)
c     ==--------------------------------------------------------------==
      return
      end
c     ==================================================================
      SUBROUTINE apply_op_GbGG(j,ik,GvecJ)
c     ==--------------------------------------------------------------==
c     computes
c     GvecJ = -i G_ik / G^2  j(G).
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'geq0.inc'
      include 'cppt.inc'
      integer    ik,ig,ig1
      complex*16 j(nhg),GvecJ(nhg)
c     ==--------------------------------------------------------------==
      if (geq0) then
        ig1=2
        GvecJ(1) = DCMPLX(0d0,0d0)
      else
        ig1=1
      endif
!$OMP parallel do private(ig)
      DO ig=ig1,nhg
        GvecJ(ig) = DCMPLX(0d0,-GK(ik,ig)/HG(ig)) * j(ig)
      ENDDO
      return
      end
c     ==================================================================
      SUBROUTINE compute_shift_dotproducts(i1,i2,GvecJ,prefactor)
c     takes as input the array
c     GvecJ  =  -iG_x / G^2  j_y(G)
c     and adds the result of the expression
c     Sum_G  exp - i G R_ion  GvecJ
c     to the shift matrix (i1,i2)
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'response_p.inc'
      include 'ions.inc'
      integer i1,i2,isa,isp,iat
      real*8  GvecJ(*),eigr_dot,prefactor
c     ==--------------------------------------------------------------==
      isa = 0
      do isp=1,nsp
        do iat=1,na(isp)
          isa = isa + 1
          shift_matrix(i1,i2,iat,isp) = 
     &        shift_matrix(i1,i2,iat,isp) 
     &        + prefactor*eigr_dot(isa,GvecJ)
        enddo
      enddo
      return
      end
c     ==================================================================
      SUBROUTINE calc_p(c0,c1,nstate,nu,psi,scr,lscr)
c     ==================================================================
c     PARALLEL
c     ***
c     OPTIMIZED MAXIMALLY FOR CPU TIME,
c     NEEDS scratch of size 12*nnr1.
c     ***
c     ==--------------------------------------------------------------==
c     EXPECTS input:     c0 gnd state wfns,
c     \                  c1 response wavefunctions onto the perturbation
c     \                     h1 = p_j
c     \                  j  the index of the component of p
c     \                     needed to put the result in the right place
c     CALCULATES OUTPUT: shift_alpha,beta  =
c     4 pi/G^2 exp(-iGR) * epsi(alpha,kappa,gamma) epsi(beta,mu,nu)
c     * G_gamma
c     * (psi0 r_mu grad_kappa psi1p_nu - psi1p_nu r_mu grad_kappa psi0)(r)
c
c     WHICH EVALUATES TO:
c
c     4 pi/G^2 exp (-iGR) * epsi(alpha,kappa,gamma) epsi(beta,mu,nu)
c     * (2 r_mu (grad_gamma psi1p_nu) (grad_kappa psi0)         [TERM 1]
c     >   +  delta(gamma,mu) psi1p_nu (grad_kappa psi0)         [TERM 2]
c     >   +  delta(kappa,mu) psi0     (grad_gamma psi1p_nu))(r) [TERM 3]
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'coor.inc'
      include 'ions.inc'
      include 'fft.inc'
      include 'cnst.inc'
      include 'adat.inc'
      include 'response_p.inc'
      include 'prop.inc'
      include 'geq0.inc'
      include 'elct.inc'
c     ==--------------------------------------------------------------==
      integer    nu,nstate,lscr
      complex*16 c0(ngw,*),c1(ngw,*),psi(*)
      real*8     scr(nnr1,*)
c     ==--------------------------------------------------------------==
      dimension  f(nstate)
      integer    is,isub,iat,isp,isa,nu1,nu2,alpha,beta,gamma,kappa,
     &     ibeta,ikappa,igamma,complete_123
      real*8     eigr_dot,epsi,contrib,eps,
     &     psi0(nnr1),psi1(nnr1),ppsi0(nnr1,2),ppsi1(nnr1,2),
     &     produkt(nnr1,3,2)
c     produkt(ir;alpha=1,2,3;beta=nu+1,nu+2)
c     ppsi#(ir,gamma1/gamma2 resp. kappa1/kappa2)
      pointer    (ip_psi0,psi0),(ip_psi1,psi1),
     &     (ip_ppsi0,ppsi0),(ip_ppsi1,ppsi1),(ip_produkt,produkt)
      character  tag*30
c     ==--------------------------------------------------------------==
      call test_scr('p-calc',tag,lscr,12*nnr1)
      call tiset('  shift(p)',isub)
      call make_123(nu,nu1,nu2)
      CALL GET_ADDR(IP_ppsi0,scr(1,1))
      CALL GET_ADDR(IP_ppsi1,scr(1,3))
      CALL GET_ADDR(IP_produkt,scr(1,5))
      CALL GET_ADDR(IP_psi0,scr(1,11))
      CALL GET_ADDR(IP_psi1,scr(1,12))
      call azzero(scr,12*nnr1)
c     ==--------------------------------------------------------------==
      do is=1,nstate
         call set_origin(is)
         call fft2TOr(c0(1,is),psi0,c1(1,is),psi1,psi,ngw,.false.)
c     ==--  ARRAYS:  grad_1,2 psi0  AND  grad_2,3 psi1:  -------------==
         call apply_op_p(c0(1,is),ppsi0(1,1),1,ngw)
         call apply_op_p(c0(1,is),ppsi0(1,2),2,ngw)
         call apply_op_p(c1(1,is),ppsi1(1,1),2,ngw)
         call apply_op_p(c1(1,is),ppsi1(1,2),3,ngw)
         call fft2TOr(ppsi0,ppsi0,ppsi0(1,2),ppsi0(1,2),psi,ngw,.false.)
         call fft2TOr(ppsi1,ppsi1,ppsi1(1,2),ppsi1(1,2),psi,ngw,.false.)
c     ==--------------------------------------------------------------==
c     TERM 1:
         call consolidate(ppsi0(1,1),ppsi1(1,1), psi) ! kappa=1 gamma=2 -> a=3, +
         call apply_op_rx(psi,+2d0*f(is),produkt(1,3,1),1d0,nu2) ! beta=nu+1, mu=nu+2, +
         call apply_op_rx(psi,-2d0*f(is),produkt(1,3,2),1d0,nu1) ! beta=nu+2, mu=nu+1, -

         call consolidate(ppsi0(1,1),ppsi1(1,2), psi) ! kappa=1 gamma=3 -> a=2, -
         call apply_op_rx(psi,-2d0*f(is),produkt(1,2,1),1d0,nu2) ! beta=nu+1, mu=nu+2, +
         call apply_op_rx(psi,+2d0*f(is),produkt(1,2,2),1d0,nu1) ! beta=nu+2, mu=nu+1, -

         call consolidate(ppsi0(1,2),ppsi1(1,2), psi) ! kappa=2 gamma=3 -> a=1, +
         call apply_op_rx(psi,+2d0*f(is),produkt(1,1,1),1d0,nu2) ! beta=nu+1, mu=nu+2, +
         call apply_op_rx(psi,-2d0*f(is),produkt(1,1,2),1d0,nu1) ! beta=nu+2, mu=nu+1, -
c     ==--------------------------------------------------------------==
C     TERM 2:
         do kappa=1,2
           ikappa = kappa
           call consolidate(psi1,ppsi0(1,ikappa),psi)
           if (kappa.NE.nu) then
             alpha  = nu
             beta   = kappa
             if (beta.EQ.nu1) ibeta = 1
             if (beta.EQ.nu2) ibeta = 2
             call DAXPY(nnr1,f(is),psi,1,produkt(1,alpha,ibeta),1)
           else
             do alpha=1,3
               beta = alpha
               if (beta.NE.nu) then
                 if (beta.EQ.nu1) ibeta = 1
                 if (beta.EQ.nu2) ibeta = 2
                 call DAXPY(nnr1,-f(is),psi,1,produkt(1,alpha,ibeta),1)
               endif
             enddo
           endif
         enddo
C     TERM 3:
         do gamma=2,3
           igamma = gamma-1
           call consolidate(psi0,ppsi1(1,igamma),psi)
           if (gamma.EQ.nu) then
             do alpha=1,3
               beta = alpha
               if (beta.NE.nu) then
                 if (beta.EQ.nu1) ibeta = 1
                 if (beta.EQ.nu2) ibeta = 2
                 call DAXPY(nnr1,f(is),psi,1,produkt(1,alpha,ibeta),1)
               endif
             enddo
           else
             alpha = nu
             beta  = gamma
             if (beta.EQ.nu1) ibeta = 1
             if (beta.EQ.nu2) ibeta = 2
             call DAXPY(nnr1,-f(is),psi,1,produkt(1,alpha,ibeta),1)
           endif
         enddo
c     ==--  NEW ARRAYS: grad_3 psi0  AND  grad_1 psi1  ---------------==
         call apply_op_p(c0(1,is),ppsi0(1,1),3,ngw)
         call apply_op_p(c1(1,is),ppsi1(1,2),1,ngw)
         call fft2TOr(ppsi0,ppsi0,ppsi1(1,2),ppsi1(1,2),psi,ngw,.false.)
c     ==--  TERM 1:  -------------------------------------------------==
         call consolidate(ppsi0(1,2),ppsi1(1,2), psi) ! kappa=2 gamma=1 -> a=3, -
         call apply_op_rx(psi,-2d0*f(is),produkt(1,3,1),1d0,nu2) ! beta=nu+1, mu=nu+2, +
         call apply_op_rx(psi,+2d0*f(is),produkt(1,3,2),1d0,nu1) ! beta=nu+2, mu=nu+1, -

         call consolidate(ppsi0(1,1),ppsi1(1,2), psi) ! kappa=3 gamma=1 -> a=2, +
         call apply_op_rx(psi,+2d0*f(is),produkt(1,2,1),1d0,nu2) ! beta=nu+1, mu=nu+2, +
         call apply_op_rx(psi,-2d0*f(is),produkt(1,2,2),1d0,nu1) ! beta=nu+2, mu=nu+1, -

         call consolidate(ppsi0(1,1),ppsi1(1,1), psi) ! kappa=3 gamma=2 -> a=1, -
         call apply_op_rx(psi,-2d0*f(is),produkt(1,1,1),1d0,nu2) ! beta=nu+1, mu=nu+2, +
         call apply_op_rx(psi,+2d0*f(is),produkt(1,1,2),1d0,nu1) ! beta=nu+2, mu=nu+1, -
c     ==--------------------------------------------------------------==
C     TERM 2:
         kappa=3
         ikappa = 1
         call consolidate(psi1,ppsi0(1,ikappa),psi)
         if (kappa.NE.nu) then
           alpha  = nu
           beta   = kappa
           if (beta.EQ.nu1) ibeta = 1
           if (beta.EQ.nu2) ibeta = 2
           call DAXPY(nnr1,f(is),psi,1,produkt(1,alpha,ibeta),1)
         else
           do alpha=1,3
             beta = alpha
             if (beta.NE.nu) then
               if (beta.EQ.nu1) ibeta = 1
               if (beta.EQ.nu2) ibeta = 2
               call DAXPY(nnr1,-f(is),psi,1,produkt(1,alpha,ibeta),1)
             endif
           enddo
         endif
C     TERM 3:
         gamma  = 1
         igamma = 2
         call consolidate(psi0,ppsi1(1,igamma),psi)
         if (gamma.EQ.nu) then
           do alpha=1,3
             beta = alpha
             if (beta.NE.nu) then
               if (beta.EQ.nu1) ibeta = 1
               if (beta.EQ.nu2) ibeta = 2
               call DAXPY(nnr1,f(is),psi,1,produkt(1,alpha,ibeta),1)
             endif
           enddo
         else
           alpha = nu
           beta  = gamma
           if (beta.EQ.nu1) ibeta = 1
           if (beta.EQ.nu2) ibeta = 2
           call DAXPY(nnr1,-f(is),psi,1,produkt(1,alpha,ibeta),1)
         endif
c     ==--------------------------------------------------------------==
      enddo                     ! is

      call fft2TOg(produkt(1,1,1),produkt(1,1,1),
     &     produkt(1,1,2),produkt(1,1,2), psi,nhg,.true.)
      call fft2TOg(produkt(1,2,1),produkt(1,2,1),
     &     produkt(1,2,2),produkt(1,2,2), psi,nhg,.true.)
      call fft2TOg(produkt(1,3,1),produkt(1,3,1),
     &     produkt(1,3,2),produkt(1,3,2), psi,nhg,.true.)
      call nmr_interaction3
     &     (produkt(1,1,1),produkt(1,2,1),produkt(1,3,1))
      call nmr_interaction3
     &     (produkt(1,1,2),produkt(1,2,2),produkt(1,3,2))

      isa = 0
      do isp=1,nsp
        do iat=1,na(isp)
          isa = isa + 1
          do alpha=1,3
            contrib=0.5d0*shift_factor
     &                *eigr_dot(isa,produkt(1,alpha,1))
            shift_matrix(nu1,alpha,iat,isp) = 
     &            shift_matrix(nu1,alpha,iat,isp) + contrib

            contrib=0.5d0*shift_factor
     &                *eigr_dot(isa,produkt(1,alpha,2))
            shift_matrix(nu2,alpha,iat,isp) = 
     &            shift_matrix(nu2,alpha,iat,isp) + contrib
C     NB: There is a nonvanishing (but <0.0001) probability that
C     nu1/2 and alpha indices are to be exchanged. But I think this way
C     it is right.
          enddo               ! alpha
        enddo                 ! atoms
      enddo                   ! atoms

      call tihalt('  shift(p)',isub)
      RETURN
      END
c     ==================================================================
      SUBROUTINE calc_L(c0,c1,nstate,a,psi,scr,ppsi)
c     ==================================================================
c     PARALLEL
c     ***
c     OPTIMIZED MAXIMALLY FOR CPU TIME,
c     NEEDS scratch of size 9*nnr1.
c     ***
c     NB: ALL arrays *_contrib stay separately on the processors,
c     only at the very end, they are GLOSUMmed. (explicitely in the
c     main program)
c     ==--------------------------------------------------------------==
c     EXPECTS input:     c0 gnd state wfns,
c     \                  c1 response wavefunctions onto the perturbation
c     \                     h1 = L_a
c     \                  a  the direction of the field (= the comp of L)
c     \                     needed to put the result in the right place
c     CALCULATES output: L_contrib(nat,nsp,l,k,a) [l,k=1..3] onto which 
c     \                  the following contribution is added:
c     \                  Sum_is  <phi0|  p_l  1/|r-R|  p_k  |phi1[L_a]>
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'coor.inc'
      include 'ions.inc'
      include 'fft.inc'
      include 'cnst.inc'
      include 'adat.inc'
      include 'response_p.inc'
      include 'prop.inc'
      include 'geq0.inc'
      include 'elct.inc'
c     ==--------------------------------------------------------------==
      integer    nstate,a
      complex*16 c0(ngw,nstate),c1(ngw,nstate),psi(*)
      real*8     scr(nnr1,*),ppsi(nnr1)
c     ==--------------------------------------------------------------==
      dimension  f(nstate)
      integer    is,ir,isub,iat,isp,isa,b,bb,bbb
      real*8     eigr_dot,epsi,contrib,
     &     ppsi0(nnr1,3),ppsi1(nnr1,3),produkt(nnr1,3)
      pointer    (ip_ppsi0,ppsi0),(ip_ppsi1,ppsi1),(ip_produkt,produkt)
c     ==--------------------------------------------------------------==
      call tiset('  shift(L)',isub)
      CALL GET_ADDR(IP_ppsi0,scr(1,1))
      CALL GET_ADDR(IP_ppsi1,scr(1,4))
      CALL GET_ADDR(IP_produkt,scr(1,7))

      call azzero(produkt,3*nnr1)
      do is=1,nstate
        do b=1,3
          call apply_op_p(c0(1,is), ppsi0(1,b),  b, ngw)
          call apply_op_p(c1(1,is), ppsi1(1,b),  b, ngw)
          call fft2TOr(ppsi0(1,b),ppsi0(1,b),
     &        ppsi1(1,b),ppsi1(1,b), psi,ngw,.false.)
        enddo

        do b=1,3
          call make_123(b,bb,bbb)
          DO ir=1,nnr1
            produkt(ir,b) = produkt(ir,b) 
     .        +f(is)*( ppsi1(ir,bb)*ppsi0(ir,bbb)
     .                -ppsi1(ir,bbb)*ppsi0(ir,bb) )
          ENDDO                 ! ir
        enddo                   ! b = 1,3
      enddo                     ! states

      call fft2TOg(produkt(1,1),produkt(1,1), 
     &     produkt(1,2),produkt(1,2), psi,nhg,.true.)
      call  fftTOg(produkt(1,3),produkt(1,3), psi,nhg,.true.)
      call nmr_interaction3(produkt(1,1),produkt(1,2),produkt(1,3))

      isa = 0
      DO isp=1,nsp
        DO iat=1,na(isp)
          isa = isa + 1
          shift_matrix(a,1,iat,isp) = shift_matrix(a,1,iat,isp)
     &              +   shift_factor*eigr_dot(isa,produkt(1,1))
          shift_matrix(a,2,iat,isp) = shift_matrix(a,2,iat,isp)
     &              +   shift_factor*eigr_dot(isa,produkt(1,2))
          shift_matrix(a,3,iat,isp) = shift_matrix(a,3,iat,isp)
     &              +   shift_factor*eigr_dot(isa,produkt(1,3))
        ENDDO                   ! atoms
      ENDDO                     ! species
      call tihalt('  shift(L)',isub)
      return
      end

c     ==================================================================
      SUBROUTINE calc_L_one(c0,c1,nstate,a,psi,scr,ppsi,istate)
c     ==================================================================
C     IDENTICAL to calc_L, except that the stuff is only done for istate
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'coor.inc'
      include 'ions.inc'
      include 'fft.inc'
      include 'cnst.inc'
      include 'adat.inc'
      include 'response_p.inc'
      include 'prop.inc'
      include 'geq0.inc'
      include 'elct.inc'
c     ==--------------------------------------------------------------==
      integer    nstate,a,istate
      complex*16 c0(ngw,nstate),c1(ngw,nstate),psi(*)
      real*8     scr(nnr1,*),ppsi(nnr1)
c     ==--------------------------------------------------------------==
      dimension  f(nstate)
      integer    ir,isub,iat,isp,isa,b,bb,bbb
      real*8     eigr_dot,epsi,contrib,
     &     ppsi0(nnr1,3),ppsi1(nnr1,3),produkt(nnr1,3)
      pointer    (ip_ppsi0,ppsi0),(ip_ppsi1,ppsi1),(ip_produkt,produkt)
c     ==--------------------------------------------------------------==
      call tiset('  shift(L)',isub)
      CALL GET_ADDR(IP_ppsi0,scr(1,1))
      CALL GET_ADDR(IP_ppsi1,scr(1,4))
      CALL GET_ADDR(IP_produkt,scr(1,7))

      call azzero(produkt,3*nnr1)
      do b=1,3
        call apply_op_p(c0(1,istate), ppsi0(1,b),  b, ngw)
        call apply_op_p(c1(1,istate), ppsi1(1,b),  b, ngw)
        call fft2TOr(ppsi0(1,b),ppsi0(1,b),
     &      ppsi1(1,b),ppsi1(1,b), psi,ngw,.false.)
      enddo

      do b=1,3
        call make_123(b,bb,bbb)
        DO ir=1,nnr1
          produkt(ir,b) = produkt(ir,b) 
     .                 +f(istate)*( ppsi1(ir,bb)*ppsi0(ir,bbb)
     .                             -ppsi1(ir,bbb)*ppsi0(ir,bb) )
        ENDDO                   ! ir
      enddo                     ! b = 1,3

      call fft2TOg(produkt(1,1),produkt(1,1), 
     &    produkt(1,2),produkt(1,2), psi,nhg,.true.)
      call  fftTOg(produkt(1,3),produkt(1,3), psi,nhg,.true.)
      call nmr_interaction3(produkt(1,1),produkt(1,2),produkt(1,3))

      isa = 0
      DO isp=1,nsp
        DO iat=1,na(isp)
          isa = isa + 1
          shift_matrix(a,1,iat,isp) = shift_matrix(a,1,iat,isp)
     &              +   shift_factor*eigr_dot(isa,produkt(1,1))
          shift_matrix(a,2,iat,isp) = shift_matrix(a,2,iat,isp)
     &              +   shift_factor*eigr_dot(isa,produkt(1,2))
          shift_matrix(a,3,iat,isp) = shift_matrix(a,3,iat,isp)
     &              +   shift_factor*eigr_dot(isa,produkt(1,3))
        ENDDO                   ! atoms
      ENDDO                     ! species
      call tihalt('  shift(L)',isub)
      return
      end
c     ==================================================================
      SUBROUTINE nmr_interaction3(func1,func2,func3)
c     ==--------------------------------------------------------------==
c     PARALLEL
      implicit none
      include 'system.h'
      include 'cnst.inc'
      include 'geq0.inc'
      include 'cppt.inc'
c     ==--------------------------------------------------------------==
      integer ig,ig1
      real*8  x,xx,func1(2,*),func2(2,*),func3(2,*)
      xx = fpi / tpiba2
      ig1=1
      if (geq0) ig1=2
!$OMP parallel do private(ig,x)
      DO ig=ig1,nhg
        x = xx / HG(ig)
        func1(1,ig) = func1(1,ig) * x
        func1(2,ig) = func1(2,ig) * x
        func2(1,ig) = func2(1,ig) * x
        func2(2,ig) = func2(2,ig) * x
        func3(1,ig) = func3(1,ig) * x
        func3(2,ig) = func3(2,ig) * x
      ENDDO
      return
      end
C     ==================================================================
      SUBROUTINE nmr_interaction(func1,func2)
c     ==--------------------------------------------------------------==
c     INPUT:   Two functions f(G), packed storage, dim=NHG
c     OUTPUT:  Each function is overwritten by  4 pi / G^2 * f(G)
c     \        The G=0 component is NOT CHANGED.
c     PARALLEL
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'cnst.inc'
      include 'geq0.inc'
      include 'cppt.inc'
c     ==--------------------------------------------------------------==
      integer ig,ig1
      real*8  x,xx,func1(2,*),func2(2,*)
      xx = fpi / tpiba2
      ig1=1
      if (geq0) ig1=2
!$OMP parallel do private(ig,x)
      DO ig=ig1,nhg
        x = xx / HG(ig)
        func1(1,ig) = func1(1,ig) * x
        func1(2,ig) = func1(2,ig) * x
        func2(1,ig) = func2(1,ig) * x
        func2(2,ig) = func2(2,ig) * x
      ENDDO
      return
      end
c     ==================================================================
      SUBROUTINE print_shift_matrix(nstate,TSUSCEPTIBILITY)
c     ==--------------------------------------------------------------==
C     PRINTS, but does NOT MODIFY the shift_matrix
      implicit none
      include 'system.h'
      include 'ions.inc'
      include 'adat.inc'
      include 'cppt.inc'
      include 'coor.inc'
      include 'response_p.inc'
      logical    TSUSCEPTIBILITY
      real*8     eigenvalues(3,nax,nsx)
      pointer    (ip_eigenvalues,eigenvalues)
      real*8     maxshift, minshift, isoshift,epsi,work(3*3),
     &  scr(3,3),scr2(3,3),delta
      integer    isp,iat,iatom,i,j,l,k,a,b,info,nstate,nst
#if defined(__SUN)
      CHARACTER*4 ACHAR
#endif
c     ==--------------------------------------------------------------==
      call memory(ip_eigenvalues,  nax*nsx*3,'shift-eigenv')
c     ==--------------------------------------------------------------==
c     each atom SEPARATELY:
      iatom = 0
      do isp = 1,nsx
        do iat = 1, na(isp)
          iatom = iatom + 1
          if (parent) write (6, '(9("=")," ATOM ",A2,I3,'//
     &        '",  NUC SHIELD MATRIX *ALL* STATES ",'//
     &        '6("="),"(PV)",2("="))') EL(IATYP(ISP)),iat
          call azzero(scr,3*3)
          call azzero(scr2,3*3)
          if (TSUSCEPTIBILITY) then
             call DAXPY(3*3,-chi_SI_to_shift_ppm,chi,1,scr,1)
             call DAXPY(3*3,-chi_SI_to_shift_ppm,chi,1,scr2,1)
          endif
          do i=1,3
            do j=1,3
              scr2(i,j) =  scr2(i,j) + shift_matrix(i,j,iat,isp)
              scr(i,j)  =  scr(i,j)+0.5d0*(shift_matrix(i,j,iat,isp)
     &            + shift_matrix(j,i,iat,isp))
            enddo               ! i
          enddo                 ! j
#ifdef PARALLEL
          call glosum(3*3,scr)
          call glosum(3*3,scr2)
#endif
          call DSYEV('N','U', 3, scr, 3, eigenvalues(1,iat,isp), 
     &        work, 9, info)
          if (info .ne. 0) write (6, *)'ERROR IN DSYEV, code ',info
          do a=1,3
            if (parent) 
     &          write (6, '(A2,I3," [all] S_",A1,"* = ",3F9.2,F18.2)')
     &          EL(IATYP(ISP)),iat,ACHAR(a+119),
     &          (scr2(a,b),b=1,3),
     &          eigenvalues(a,iat,isp)
          enddo                 ! a
        enddo                   ! iat
      enddo                     ! isp
      if (parent)
     &    write (6, '(9("=")," RAW SHIELD MATRICES END ",31("="))')
c     ==--------------------------------------------------------------==
c     FINAL LIST, this time INCLUDING THE SUSCEPTIBILITY CORRECTION:
      if (TSUSCEPTIBILITY) then
         delta = -chi_SI_iso*chi_SI_to_shift_ppm
         if (parent) write (6, '("===== NUC SHIELD'//
     &        ' ====(PV 1)===(PV 2)===(PV 3)====(iso)==(aniso)==")')
         iatom = 0
         do isp = 1,nsx
            do iat = 1, na(isp)
               iatom = iatom + 1
               isoshift = (eigenvalues(1,iat,isp)
     &              +eigenvalues(2,iat,isp)
     &              +eigenvalues(3,iat,isp)) / 3d0
               if (parent) write (6, '(A2,I3,A,I3,A,5F9.2)')
     &              EL(IATYP(ISP)),iat,'/',iatom,
     &              '   SHIELD :',
     &              (eigenvalues(i,iat,isp),i=1,3),
     &              isoshift,
     &              eigenvalues(3,iat,isp)-
     &              (eigenvalues(1,iat,isp)
     &              +eigenvalues(2,iat,isp))/2.d0
            enddo
         enddo
         if (parent) write (6, '(5("=")," NET SHIELD ",48("="))')
         if (parent) write (6, '(5("=")," NET SHIELD, '//
     &        'susceptibility correction = ",'//
     &        'F6.3,13("="))') delta
      endif
c     ==--------------------------------------------------------------==
      call freem(ip_eigenvalues)
      RETURN
      END
C     ==================================================================
      SUBROUTINE calc_shift_via_current(j1,j2,isa,iB,shift,psi)
c     ==--------------------------------------------------------------==
c     PARALLEL
c     EXPECTS: The currents j1, j2 in G-space.
c     OUTPUT:  The shift value for atom ISA.
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'response_p.inc'
      include 'fft.inc'
      include 'gvec.inc'
      include 'cnst.inc'
      complex*16  j1(nhg),j2(nhg),psi(*)
      real*8      shift,eigr_dot
      integer     ig,iB,iBB,iBBB,isub,isa
c     ==--------------------------------------------------------------==
      CALL TISET('  j/SHIFTS',ISUB)

      call make_123(iB,iBB,iBBB)

      do ig=1,nhg
        psi(ig) = -UIMAG * (j1(ig)*GK(iBBB,ig) - j2(ig)*GK(iBB,ig))
        if (HG(ig) .gt. 1d-4) psi(ig) = psi(ig) / HG(ig)
      enddo

      shift = shift - 2d0*alat*shift_factor * eigr_dot(isa,psi)

      CALL TIHALT('  j/SHIFTS',ISUB)
      return
      end
c     ==================================================================
