#define warning_level 5d-3
c     ==================================================================
      subroutine consolidate(src1, src2, dest)
c     PARALLEL
c     =----------------------------------------------------------------=
c     INPUT:  src1,2:   wavefunctions in R-space
c     OUTPUT: dest(r) = src1(r)  * src2(r)
c     src1,src2,dest MAY BE IDENTICAL.
c     =----------------------------------------------------------------=
      implicit none
      include 'system.h'
      include 'coor.inc'
      include 'cppt.inc'
      include 'gvec.inc'
      include 'fft.inc'
c     =----------------------------------------------------------------=
c     Arguments:
      real*8       src1(nnr1),src2(nnr1),dest(nnr1)
c     Local variables:
      integer  ir
      real*8   x1,x2
c     =----------------------------------------------------------------=
!$OMP parallel do private(ir,x1,x2)
      do ir=1,nnr1-1,2
         x1 =  src1(ir)   * src2(ir)
         x2 =  src1(ir+1) * src2(ir+1)
         dest(ir)   = x1
         dest(ir+1) = x2
      enddo
      return
      end
c     ==================================================================
      subroutine apply_op_p(src, dest, coordinate, dim)
c     PARALLEL
c     =----------------------------------------------------------------=
c     INPUT:  src:   wavefunctions in G-space
c     \       coord: whether p_x (coord=1) or p_y/z (coord=2/3) should
c     \              be applied.
c     OUTPUT: dest:  the resulting wavefunction in G-space
c     \              = d/dx or d/dy or d/dz   src
c     \              previous content of dest is overwritten.
c     \       src and dest MAY BE IDENTICAL.
c     =----------------------------------------------------------------=
      implicit none
      include 'system.h'
      include 'coor.inc'
      include 'cppt.inc'
      include 'gvec.inc'
      include 'fft.inc'
c     =----------------------------------------------------------------=
c     Arguments:
      real*8    src(2,ngw), dest(2,ngw)
      integer   coordinate
c     Local variables:
      integer  ig,isub,dim
      real*8   coeff,src_RE,src_IM
c     =----------------------------------------------------------------=
      call tiset('     p_psi',isub)
!$OMP parallel do private(ig,src_RE,src_IM,coeff)
      do ig=1,dim
        src_RE     =   src(1,ig)
        src_IM     =   src(2,ig)
        coeff      =   tpiba  * GK(coordinate,ig)
        dest(1,ig) = - coeff  * src_IM
        dest(2,ig) =   coeff  * src_RE
      enddo

      call tihalt('     p_psi',isub)
      return
      end
c     ==================================================================
      subroutine fftTOr(src,dest,psi,dim,do_phase)
c     dim(src) = ngw or nhg (cmplx), dim(dest) = nnr1 (real)
c     NB: src and dest may be the same!
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'coor.inc'
      include 'fft.inc'
      complex*16 psi(*),src(ngw)
      real*8     dest(*)
      integer    ig,ir,dim,isub
      logical    do_phase
c     =----------------------------------------------------------------=
      call tiset('  FFT -> R',isub)
      call zazzero(psi,maxfft)
      if (dim .eq. ngw) then
!CDIR NODEP
#ifdef _vpp_
!OCL NOVREC
#endif 
        do ig=1,dim
          psi(nzhs(ig))  = src(ig)
          psi(indzs(ig)) = DCONJG(src(ig))
        enddo
        call s_invfft(psi)
        if (do_phase) call phase(psi)
      elseif (dim .eq. nhg) then
!CDIR NODEP
#ifdef _vpp_
!OCL NOVREC
#endif 
        do ig=1,dim
          psi(nzh(ig))  = src(ig)
          psi(indz(ig)) = DCONJG(src(ig))
        enddo
        call invfft(psi)
      else
        call stopgm('fftTOr   ','wrong dimension.')
      endif
!$OMP parallel do private(ir)
      do ir=1,nnr1
        dest(ir)  = DREAL(PSI(ir))
      enddo
      call tihalt('  FFT -> R',isub)
      return
      end
c     ==================================================================
      subroutine fft2TOr(src1,dest1,src2,dest2,psi,dim,do_phase)
c     dim(src) = ngw (cmplx), dim(dest) = nnr1 (real)
c     NB: src and dest may be the same!
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'coor.inc'
      include 'fft.inc'
      include 'cnst.inc'
      real*8     psi(2,*),src1(2,*),src2(2,*),dest1(*),dest2(*),
     &    re1,re2,im1,im2
      integer    ig,ir,dim,isub
      logical    do_phase


      call tiset('FFT^2 -> R',isub)
      call azzero(psi,2*maxfft)
      if (dim .eq. ngw) then
!CDIR NODEP
#ifdef _vpp_
!OCL NOVREC
#endif 
        do ig=1,dim
          re1               =   src1(1,ig)
          re2               =   src2(1,ig)
          im1               =   src1(2,ig)
          im2               =   src2(2,ig)
          psi(1,nzhs(ig))   =   re1 - im2
          psi(2,nzhs(ig))   =   im1 + re2
          psi(1,indzs(ig))  =   re1 + im2
          psi(2,indzs(ig))  = - im1 + re2
        enddo
        call s_invfft(psi)
        if (do_phase) call phase(psi)
      elseif (dim .eq. nhg) then
!CDIR NODEP
#ifdef _vpp_
!OCL NOVREC
#endif 
        do ig=1,dim
          re1              =   src1(1,ig)
          re2              =   src2(1,ig)
          im1              =   src1(2,ig)
          im2              =   src2(2,ig)
          psi(1,nzh(ig))   =   re1 - im2
          psi(2,nzh(ig))   =   im1 + re2
          psi(1,indz(ig))  =   re1 + im2
          psi(2,indz(ig))  = - im1 + re2
        enddo
        call invfft(psi)
      else
        call stopgm('fft2TOr    ','wrong dimension.')
      endif

!$OMP parallel do private(ir)
      do ir=1,nnr1-1,2
        dest1(ir)  = PSI(1,ir)
        dest2(ir)  = PSI(2,ir)
        dest1(ir+1)  = PSI(1,ir+1)
        dest2(ir+1)  = PSI(2,ir+1)
      enddo

      call tihalt('FFT^2 -> R',isub)
      return
      end
c     ==================================================================
      subroutine fftTOg(src,dest,psi,dim,do_phase)
c     NB: src and dest may be the same!
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'coor.inc'
      include 'fft.inc'
      include 'response_p.inc'
c      include    'prop.inc'
      complex*16 psi(*),dest(ngw)
      real*8     src(*),norm1,norm2,DDOT,dotp
      external   DDOT,DOTP
      integer    ig,ir,dim,isub
      logical    do_phase

      call tiset('  FFT -> G',isub)
      if (.NOT. TFAST) then
        norm1 = DDOT(nnr1,src,1,src,1)/DBLE(nr1s*nr2s*nr3s)
#ifdef PARALLEL
        call glosum(1,norm1)
#endif
        norm1 = sqrt(norm1)
      endif

      call zazzero(psi,maxfft)
!$OMP parallel do private(ir)
      do ir=1,nnr1
        psi(ir)  = DCMPLX(src(ir),0d0)
      enddo
      if (dim .eq. ngw) then
        if (do_phase) call phase(psi)
        call s_fwfft(psi)
!$OMP parallel do private(ig)
        do ig=1,dim
          dest(ig)  = PSI(nzhs(ig))
        enddo
      else
        call fwfft(psi)
!$OMP parallel do private(ig)
        do ig=1,dim
          dest(ig)  = PSI(nzh(ig))
        enddo
      endif

      if (.NOT. TFAST) then
        norm2 = DOTP(dim, dest, dest)
#ifdef PARALLEL
        call glosum(1,norm2)
#endif
        norm2 = sqrt(norm2)
        if (abs(norm1-norm2)/norm2 .GT. warning_level) then
          if (parent.and.(.not.tdummyatom)) write (6, 80)
     &        DBLE(100d0)*(norm1-norm2)/norm2
        endif
      endif

      call tihalt('  FFT -> G',isub)
      return
 80   FORMAT ("*  FFT INFORMATION LOSS WARNING: ",F10.3,
     &    "% difference in norm *")
      end
c     ==================================================================
      subroutine fft2TOg(src1,dest1,src2,dest2,psi,dim,do_phase)
c     NB: src and dest may be the same!
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'coor.inc'
      include 'cnst.inc'
      include 'fft.inc'
      include 'response_p.inc'
c      include 'prop.inc'
      real*8     psi(2,*),dest1(2,ngw),dest2(2,ngw)
      real*8     src1(*),src2(*),norm1a,norm2a,norm1b,norm2b,DDOT,dotp,
     &    re1,re2,im1,im2
      external   DDOT,DOTP
      integer    ig,ir,dim,isub
      logical    do_phase
      call tiset('FFT^2 -> G',isub)
      if (.NOT. TFAST) then
        norm1a = DDOT(nnr1,src1,1,src1,1)/DBLE(nr1s*nr2s*nr3s)
        norm2a = DDOT(nnr1,src2,1,src2,1)/DBLE(nr1s*nr2s*nr3s)
#ifdef PARALLEL
        call glosum(1,norm1a)
        call glosum(1,norm2a)
#endif
        norm1a = sqrt(norm1a)
        norm2a = sqrt(norm2a)
      endif


      call azzero(psi,2*maxfft)
!$OMP parallel do private(ir)
      do ir=1,nnr1-1,2
        psi(1,ir) = src1(ir)
        psi(2,ir) = src2(ir)
        psi(1,ir+1) = src1(ir+1)
        psi(2,ir+1) = src2(ir+1)
      enddo
      if (dim .eq. ngw) then
        if (do_phase) call phase(psi)
        call s_fwfft(psi)
!$OMP parallel do private(ig,re1,re2,im1,im2)
        do ig=1,dim
          re1 = 0.5d0*PSI(1,nzhs(ig))
          re2 = 0.5d0*PSI(1,indzs(ig))
          im1 = 0.5d0*PSI(2,nzhs(ig))
          im2 = 0.5d0*PSI(2,indzs(ig))

          dest1(1,ig) =   re1 + re2
          dest1(2,ig) =   im1 - im2
          dest2(1,ig) =   im1 + im2
          dest2(2,ig) = - re1 + re2
        enddo
      else
        call fwfft(psi)
!$OMP parallel do private(ig,re1,re2,im1,im2)
        do ig=1,dim
          re1 = 0.5d0*PSI(1,nzh(ig))
          re2 = 0.5d0*PSI(1,indz(ig))
          im1 = 0.5d0*PSI(2,nzh(ig))
          im2 = 0.5d0*PSI(2,indz(ig))

          dest1(1,ig) =   re1 + re2
          dest1(2,ig) =   im1 - im2
          dest2(1,ig) =   im1 + im2
          dest2(2,ig) = - re1 + re2
        enddo
      endif

      if (.NOT. TFAST) then
        norm1b = DOTP(dim, dest1, dest1)
        norm2b = DOTP(dim, dest2, dest2)
#ifdef PARALLEL
        call glosum(1,norm1b)
        call glosum(1,norm2b)
#endif
        norm1b = sqrt(norm1b)
        norm2b = sqrt(norm2b)
        if (abs(norm1a-norm1b)/norm1b .GT. warning_level) then
          if (parent) write (6, 80)
     &        DBLE(100d0)*(norm1a-norm1b)/norm1b
        endif
        if (abs(norm2a-norm2b)/norm2b .GT. warning_level) then
          if (parent) write (6, 80)
     &        DBLE(100d0)*(norm2a-norm2b)/norm2b
        endif
      endif

      call tihalt('FFT^2 -> G',isub)
      return

 80   FORMAT ("*  FFT INFORMATION LOSS WARNING: ",F10.3,
     &    "% difference in norm *")
      end
c     ==================================================================
      subroutine print_wavefunctions(c0,nstate,psi)
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'coor.inc'
      include 'ions.inc'
      include 'cnst.inc'
      include 'fft.inc'
      integer    is,nstate,numstates,isp,iat,nnat
      complex*16 psi(*),c0(ngw,nstate)
      character*80  filename
      real*8     center(3),psiofr(*)
      pointer    (ip_psiofr,psiofr)

      call memory(ip_psiofr,nnr1+16,'PSI_OF_R')

      numstates = nstate
      if (nstate .gt. 50) then
        write (6,*) ' PRINT C0: TOO MANY STATES. PRINTING ONLY 1..50.'
        numstates = 50
      endif
      center(1) = 0d0
      center(2) = 0d0
      center(3) = 0d0
      do isp=1,nsp
         do iat=1,na(isp)
            center(1) = center(1) + tau0(1,iat,isp)
            center(2) = center(2) + tau0(2,iat,isp)
            center(3) = center(3) + tau0(3,iat,isp)
            nnat = nnat + 1
         enddo
      enddo
      center(1) =  center(1) / DBLE(nnat)
      center(2) =  center(2) / DBLE(nnat)
      center(3) =  center(3) / DBLE(nnat)

      do is=1,numstates
         call fftTOr(c0(1,is),psiofr,psi,ngw,.true.)
         write (filename,'(A,I2.2,A)') 'psi0-',is,'.cube '
         call cubefile(filename,psiofr,center,psi,.false.)
      enddo

      call freem(ip_psiofr)

      return
      end
c     ==================================================================
      subroutine print_orbital_densities(c0,nstate,psi,scr)
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'coor.inc'
      include 'ions.inc'
      include 'cnst.inc'
      include 'fft.inc'
      include 'response_p.inc'
      integer    is,ir,nstate,numstates,isp,iat,nnat
      complex*16 scr(*),c0(ngw,nstate)
      real*8     psi(*),center(3)
      character*80  filename
      real*8     rhotot(*)
      pointer    (ip_rhotot,rhotot)


      call memory(ip_rhotot,nnr1,'rho_tmp')
      call azzero(rhotot,nnr1)

      numstates = nstate
      if (nstate .gt. 50) then
        write (6,*) ' PRINT RHO0: TOO MANY STATES. PRINTING ONLY 1..50.'
        numstates = 50
      endif

      center(1) = 0d0
      center(2) = 0d0
      center(3) = 0d0
      do isp=1,nsp
         do iat=1,na(isp)
            center(1) = center(1) + tau0(1,iat,isp)
            center(2) = center(2) + tau0(2,iat,isp)
            center(3) = center(3) + tau0(3,iat,isp)
            nnat = nnat + 1
         enddo
      enddo
      center(1) =  center(1) / DBLE(nnat)
      center(2) =  center(2) / DBLE(nnat)
      center(3) =  center(3) / DBLE(nnat)



      do is=1,numstates
         call fftTOr(c0(1,is),psi,scr,ngw,.false.)
!$OMP parallel do private(ir)
         do ir=1,nnr1
            psi(ir)  =  psi(ir)*psi(ir)
         enddo
         call DAXPY(nnr1,1.d0,psi,1,rhotot,1)
         write (filename,'(A,I2.2,A)') 'rho0-',is,'.cube '
         call cubefile(filename,psi,center,scr,.true.)
      enddo
      write (filename,'(A)') 'rho0-all.cube '
      call cubefile(filename,rhotot,center,scr,.true.)
      write (filename,'(A)') 'vofrho0.cube '
      call cubefile(filename,vofrho0,center,scr,.true.)
      call freem(ip_rhotot)
      return
      end
c     ==================================================================
      real*8 function epsi(a,b,c)
c     returns 0,+-1; the totally antisymmetric tensor epsilon
      implicit none
      integer a,b,c
      epsi = 0d0
      if ((b.eq.a+1 .or. b.eq.a-2) .and. (c.eq.b+1 .or. c.eq.b-2))
     &  epsi = 1d0
      if ((b.eq.a-1 .or. b.eq.a+2) .and. (c.eq.b-1 .or. c.eq.b+2))
     &  epsi = -1d0
      return
      end
c     ==================================================================
      subroutine make_123(first,second,third)
c     calculates second = first + 1,
c     \          third  = first + 2, each one modulo 3.
      implicit none
      integer first, second, third
      second = first + 1
      third  = first + 2
      if (second .gt. 3) second = second - 3
      if (third  .gt. 3) third  = third  - 3
      return
      end
c     ==================================================================
      integer function complete_123(first,second)
c     calculates third: third <> first, third <> second.
      implicit none
      integer first, second, third, f2,f3
      if (first .EQ. second)
     &     call stopgm('complete_123','invalid operands')
      call make_123(first,f2,f3)
      third = 0
      if (f2 .EQ. second) third = f3
      if (f3 .EQ. second) third = f2
      if (third .EQ. 0) call stopgm('complete_123','internal error')
      complete_123 = third
      return
      end
c     ==================================================================
      subroutine printtime
      implicit  none
      include 'system.h'
      include 'response_p.inc'
      real*8    timef,lasttime,delta_t
      external  timef
      save      lasttime
      data      lasttime /0.d0/

      if (.not. parent) return

      if (lasttime .lt.1d-10) then
        lasttime = timef()
        return
      endif

      delta_t = (timef()-lasttime)/1000.d0
      lasttime = timef()
      write (6, '("*  TCPU = ",F7.1," sec: ",A40," *")') 
     &    delta_t,timetag

      return
      end
C     ==================================================================
      SUBROUTINE print_configuration
C     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'response_p.inc'
      character*32 text1,text2
      integer i
C     ==--------------------------------------------------------------==
      write (6,91)
      text1 = 'RESPONSE TYPE:'
      IF (TNMR_FULL) THEN
        text2 = 'NMR / FULL calculation '
        write (6,92) text1,text2
      ELSE
        text2 = 'NMR chemical shifts'
        write (6,92) text1,text2
      ENDIF
      text1 = ' '
      text2 = 'magnetic susceptibility tensors'
      write (6,92) text1,text2

      IF (T_RESTART_C1) THEN
         text1 = 'RESTART FULL CALCULATION'
         text2 = 'FROM PREVIOUS RUN'
         write (6,92) text1,text2
      ENDIF
C     ==-------------------------------------------==
      text1 = 'CALCULATION METHOD:'
      text2 = 'CSGT'
      if (INMR_METHOD .eq. INMR_IGLO) text2 = 'IGLO'
      write (6,92) text1,text2
C     ==-------------------------------------------==
      text1 = 'OUTPUT:'
      text2 = 'standard'
      if (TVERBOSE) text2 = 'VERBOSE'
      write (6,92) text1,text2
C     ==-------------------------------------------==
      text1 = 'ELECTRONIC CURRENT:'
      text2 = 'not computed'
      if (TCURRENT) text2 = 'COMPUTED -> jB_.dens)'
      write (6,92) text1,text2
C     ==-------------------------------------------==
      text1 = 'ORBITAL PLOTS:'
      text2 = 'not done'
      if (TPRINTWFNS) text2 = 'WAVEFUNCTIONS'
      if (TPRINTRHO)  text2 = 'DENSITIES'
      if (TPRINTRHO.and.TPRINTWFNS)
     &    text2 = 'WAVEFUNCTIONS and DENSITIES'
      write (6,92) text1,text2
C     ==-------------------------------------------==
      text1 = 'FFT[R->G] - LOSSES:'
      text2 = 'warnings issued'
      if (TFAST) text2 = 'IGNORED'
      write (6,92) text1,text2
C     ==-------------------------------------------==
      text1 = 'LOCALIZATION:'
      text2 = 'wannier functions'
      if (.not. TLOCALIZE) text2 = 'CANONICAL ORBITALS'
      write (6,92) text1,text2
C     ==-------------------------------------------==
      text1 = 'VIRTUAL CELL MODE:'
      text2 = 'borders at minimum density'
      if (INMR_VIRTUAL .eq. INMR_WC) text2 = 'USING WANNIERCENTERS'
      if (INMR_VIRTUAL .eq. INMR_NOVIRT) text2 = 'NO VIRTUAL CELLS'
      write (6,92) text1,text2
      if (INMR_VIRTUAL .eq. INMR_LLC) then
        text1 = 'GAUSSIAN SPREAD:'
        write (text2,'(F10.3)') sigma
        write (6,92) text1,text2
        text1 = ' '
        if (TNMR_OVERLAPS) then
          text2 = 'OVERLAP OPTIMIZATION'
          write (6,92) text1,text2
          text1 = 'OVL THREASHOLD:'
          write (text2,'(F10.3)') OVERLAP_THREASHOLD
        else
          text2 = 'each state individually'
        endif
        write (6,92) text1,text2
      endif
C     ==-------------------------------------------==
      text1 = 'SMOOTHING MODE:'
      text2 = 'at +-10% of virtual cells'
      if (.not. TSMOOTHING) text2 = 'NO SMOOTHING'
      write (6,92) text1,text2
C     ==-------------------------------------------==

      write (6,91)
      return
c     ==--------------------------------------------------------------==
 91   FORMAT (52("*"),"NMR*RESPONSE*")
 92   FORMAT ("* ",A24,X,A30,"       *")

      end
C     ==================================================================

