C     ==================================================================
      SUBROUTINE NUMPW
C     ==--------------------------------------------------------------==
C     == CALCULATES THE NUMBER OF UNIQUE PW COMPONENTS FOR THE DENSITY==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'gvec.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'sphe.inc'
C     Variables
      REAL*8    DELTA
      PARAMETER (DELTA=1.D-6)
      REAL*8    TABG2((NR1-1)*(NR2-1)*(NR3-1)/2)
      INTEGER   NTAB ((NR1-1)*(NR2-1)*(NR3-1)/2)
      POINTER   (IP_TABG2,TABG2), (IP_NTAB,NTAB)
      REAL*8    GLOW,GHIGH,ENERGY,G2,T,GKPWF
      INTEGER   NR1M1,NR2M1,NR3M1,NR1F,NR2F,NR3F,
     &          I,IG,ILOOP,J,JMIN,JMAX,K,KMIN,KMAX,NUMTAB,IR,ISUB 
C     ==--------------------------------------------------------------==
      CALL TISET('     NUMPW',ISUB)
C     Calculate the reciprocal lattice vectors B1,B2 and B3
      CALL RECIPS(ALAT,A1,A2,A3,B1,B2,B3)
C     Save NR1, NR2, and NR3 for MESH option.
      NR1F=NR1
      NR2F=NR2
      NR3F=NR3
C     Maximum K point amplitude (Kmax) in Brillouin zone.
      GCUTKA=GKPWF(B1,B2,B3,TKPNT.AND.TSPHERE)
      IF(GCUTKA.EQ.0.D0) THEN
        GCUTWMIN=GCUTW
        GCUTWMAX=GCUTW
      ELSE
C       For each k and each G, |k+G|^2 < GCUTW
C       so we define GCUTWMIN and GCUTWMAX as:
C       GCUTWMIN < |G|^2 < GCUTWMAX we have to apply a mask.
        GCUTWMIN = MAX(0.D0,SQRT(GCUTW)-SQRT(GCUTKA))**2
        GCUTWMAX = (SQRT(GCUTW)+SQRT(GCUTKA))**2
      ENDIF
C     GCUT has to be bigger than GCUTWMAX
      ENERGY=MAX(GCUT,GCUTWMAX)
C     Determination of NR1, NR2, and NR3
      CALL CHANGE_GCUT(ENERGY)
 30   CONTINUE
      NR1M1=NR1-1
      NR2M1=NR2-1
      NR3M1=NR3-1
C     To have the pw density number required
      IF(TNHGWISH) THEN
        NUMTAB=INT(NR1M1*NR2M1*NR3M1/2)
        CALL MEMORY(IP_TABG2,NUMTAB,'TABG2')
        CALL MEMORY(IP_NTAB ,NUMTAB,'NTAB')
      ENDIF
C     Construction of g-vectors
      IG=0
      GLOW=0.0D0
      GHIGH=GCUTWMAX
      DO ILOOP=1,2
        IF(ILOOP.EQ.1) THEN
          GLOW=0.0D0
          GHIGH=GCUTWMAX
        ELSEIF(ILOOP.EQ.2) THEN
          GLOW=GCUTWMAX
          GHIGH=GCUT
        ENDIF
        DO I=0,NR1-1
          JMIN=-NR2+1
          JMAX=NR2-1
          IF(I.EQ.0) JMIN=0
          DO J=JMIN,JMAX
            KMIN=-NR3+1
            KMAX=NR3-1
            IF(I.EQ.0.AND.J.EQ.0) KMIN=0
            DO K=KMIN,KMAX
              G2=0.D0
              DO IR=1,3
                T=DBLE(I)*B1(IR)+DBLE(J)*B2(IR)+
     *               DBLE(K)*B3(IR)
                G2=G2+T*T
              ENDDO
#ifdef __SR8000 
*soption nopredicate 
#endif 
              IF(G2.GE.GLOW.AND.G2.LT.GHIGH) THEN
                IG=IG+1
                IF(TNHGWISH) TABG2(IG)=G2
              ENDIF
            ENDDO
          ENDDO
        ENDDO
        IF(ILOOP.EQ.1) THEN
          NGW=IG
        ELSEIF(ILOOP.EQ.2) THEN
          NHG=IG
        ENDIF
      ENDDO
      IF(TNHGWISH) THEN
        CALL SORT2(TABG2,NHG,NTAB)
        IF(NHG.LT.NHGWISH) THEN
          WRITE(6,*)
     &         'THE CUTOFF ENERGY IS TOO SMALL TO HAVE NHG=',NHGWISH
          WRITE(6,*) 'NHG=',NHG
          CALL STOPGM('NUMPW','INCREASE ECUT')
        ENDIF
C       Change GCUT
        ENERGY=TABG2(NHGWISH)+DELTA
C       Use old value if MESH option
        NR1=NR1F
        NR2=NR2F
        NR3=NR3F
C       Change GCUT, NR1,NR2, and NR3
        CALL CHANGE_GCUT(ENERGY)
        TNHGWISH=.FALSE.
        CALL FREEM(IP_TABG2)
        CALL FREEM(IP_NTAB)
        GOTO 30
      ENDIF
      CALL TIHALT('     NUMPW',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CHANGE_GCUT(ENERGY)
C     ==--------------------------------------------------------------==
C     == CHANGE ECUT                                                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cell.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'sphe.inc'        !GCUTWMAX
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'        !BETAP BETAEL P
C     Arguments
      REAL*8  ENERGY
C     Variables
      REAL*8    BETAELMAX
      PARAMETER (BETAELMAX=1.D33)
      INTEGER   FFTCHK,MNR1,MNR2,MNR3
      REAL*8    AA1,AA2,AA3
      INTEGER   IBETAPF
      DATA      IBETAPF /0/
      SAVE      IBETAPF
C     ==--------------------------------------------------------------==
      GCUTW=ENERGY/CDUAL
C     GCUT has to be bigger than GCUTWMAX
      GCUT=MAX(ENERGY,GCUTWMAX)
      GCUTW=GCUT/CDUAL
      ECUT=GCUTW*TPIBA2
C     ==--------------------------------------------------------------==
C     ==  REAL SPACE MESH (from setsc.F)                              ==
C     ==--------------------------------------------------------------==
      IF(IBRAV.EQ.1) THEN
C       Simple cubic
        MNR1=NINT(ALAT/PI*SQRT(CDUAL*ECUT)+0.5D0)
        MNR2=MNR1
        MNR3=MNR1
      ELSEIF(IBRAV.EQ.2) THEN
C       Face centred cubic
        MNR1=NINT(0.75D0*ALAT/PI*SQRT(CDUAL*ECUT)+0.5D0)
        MNR2=MNR1
        MNR3=MNR1
      ELSEIF(IBRAV.EQ.3) THEN
C       Body centred cubic
C       0.85 too short (sometimes..Thierry Deutsch)
C       MNR1=NINT(0.85D0*ALAT/PI*SQRT(CDUAL*ECUT)+0.5D0)
        MNR1=NINT(0.875D0*ALAT/PI*SQRT(CDUAL*ECUT)+0.5D0)
        MNR2=MNR1
        MNR3=MNR1
      ELSE
        AA1=ALAT
        AA2=ALAT*CELLDM(2)
        AA3=ALAT*CELLDM(3)
        MNR1=NINT(AA1/PI*SQRT(CDUAL*ECUT)+0.5D0)
        MNR2=NINT(AA2/PI*SQRT(CDUAL*ECUT)+0.5D0)
        MNR3=NINT(AA3/PI*SQRT(CDUAL*ECUT)+0.5D0)
      ENDIF
C     For MESH option.
      MNR1=MAX(MNR1,NR1)
      MNR2=MAX(MNR2,NR2)
      MNR3=MAX(MNR3,NR3)
      NR1=FFTCHK(MNR1,2)
      NR2=FFTCHK(MNR2,2)
      NR3=FFTCHK(MNR3,2)
C     ==--------------------------------------------------------------==
C     TROTTER FACTORISATION: BETAP
      P=0.D0
      IF(TTROT) THEN
        IF(IBETAPF.EQ.0) THEN
          IF(BETAP.EQ.0D0) THEN
C           BETAP has to be set
            IBETAPF=1
          ELSE
C           BETAP sets by the user
            IBETAPF=-1
          ENDIF
        ENDIF
        IF(IBETAPF.EQ.1) BETAP = (ALAT/DBLE(NR1))**2/3.D0
        IF(BETAEL.LT.BETAELMAX) THEN
          P = BETAEL/BETAP
        ELSE
          P = 0
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION GKPWF(B1,B2,B3,TADD)
C     ==--------------------------------------------------------------==
C     == Calculate the maximum K point amplitude in Brillouin zone    ==
C     == in order to apply a spherical cutoff                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8  GKPWF
      REAL*8  B1(3),B2(3),B3(3)
      LOGICAL TADD
C     Variables
      REAL*8  B1L,B2L,B3L,B12L,B13L,B23L,B123L
      INTEGER I,J
C     ==--------------------------------------------------------------==
      IF(.NOT.TADD) THEN
        GKPWF=0.D0
        RETURN
      ENDIF
C     ==--------------------------------------------------------------==
      B1L=B1(1)**2+B1(2)**2+B1(3)**2
      B2L=B2(1)**2+B2(2)**2+B2(3)**2
      B3L=B3(1)**2+B3(2)**2+B3(3)**2
      GKPWF=MAX(B1L,B2L,B3L)
C     Diagonal 1,2
      DO I=-1,1,2
        B12L=(B1(1) + I*B2(1))**2 +
     &       (B1(2) + I*B2(2))**2 +
     &       (B1(3) + I*B2(3))**2
        IF(B12L.GT.GKPWF) GKPWF=B12L
      ENDDO
C     Diagonal 1,3
      DO I=-1,1,2
        B13L=(B1(1) + I*B3(1))**2 +
     &       (B1(2) + I*B3(2))**2 +
     &       (B1(3) + I*B3(3))**2
         IF(B13L.GT.GKPWF) GKPWF=B13L
      ENDDO
C     Diagonal 2,3
      DO I=-1,1,2
        B23L=(B2(1) + I*B3(1))**2 +
     &       (B2(2) + I*B3(2))**2 +
     &       (B2(3) + I*B3(3))**2
        IF(B23L.GT.GKPWF) GKPWF=B23L
      ENDDO
C     Diagonal 1,2,3
      DO J=-1,1,2
        DO I=-1,1,2
          B123L=(B1(1) + J*B2(1) + I*B3(1))**2
     &         +(B1(2) + J*B2(2) + I*B3(2))**2
     &         +(B1(3) + J*B2(3) + I*B3(3))**2
          IF(B123L.GT.GKPWF) GKPWF=B123L
        ENDDO
      ENDDO
      GKPWF=GKPWF/4.D0
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
