C     ==================================================================
      SUBROUTINE ORBHARD
C     ==--------------------------------------------------------------==
C     ==  ORBITAL HARDNESS MATRIX                                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'
      INCLUDE 'elct2.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'ddip.inc'
      INCLUDE 'wann.inc'
      INCLUDE 'linres.inc'
C     Variables
      COMPLEX*16 C0(*),C2(*),CM(*),SC0(*),CN(*)
      REAL*8     VPP(*)
      POINTER    (IP_C0,C0),(IP_C2,C2),(IP_CM,CM),(IP_SC0,SC0)
      POINTER    (IP_VPP,VPP),(IP_CN,CN)
      INTEGER    NSTATE,NC0,NCM,NC2,NSC0,NCN
#ifdef PARALLEL
      INTEGER    NXXFUN
      EXTERNAL   NXXFUN
#endif
C     ==--------------------------------------------------------------==
      IF ( TQMMM ) CALL STOPGM("ORBHARD","QMMM NOT IMPLEMENTED")
C     ==--------------------------------------------------------------==
      IF(TKPNT) CALL STOPGM('ORBHARD','K-Points not implemented')
      NSTATE=N
C..now the orbitals are stored in RESTART.1
      IF(TSDAN) THEN
        CALL OHIN
C..linear response code
        NC0=2*NGW*NSTATE+8
        NC2=2*NGW*NSTATE+8
        NCM=NC2
        NCN=1
        NSC0=1
        IF(LOCAL_ORB) THEN
          TWANN=.TRUE.
#ifdef PARALLEL
          LENBK=NXXFUN(NSTATE)
          NSC0=MAX(2*LENBK*NPROC,2*NGWK*NSTATE)
#else
          NSC0=2*NGWK*NSTATE
#endif
        ENDIF
        CALL MEMORY(IP_C0,NC0,'C0')
        CALL MEMORY(IP_CM,NCM,'CM')
        CALL MEMORY(IP_CN,NCN,'CN')
        CALL MEMORY(IP_C2,NC2,'C2')
        CALL MEMORY(IP_SC0,NSC0,'SC0')
        CALL MEMORY(IP_VPP,1,'VPP')
C..FNL and DFNL
        CALL FNLALLOC(NSTATE,.FALSE.,.FALSE.)
C
        CALL OHLR(C0,CM,C2,SC0)
      ELSE
C..finite difference code
        IF(TDIAG) THEN
          TFIXO=.TRUE.
          NC0=2*NGW*NSTATE+8
          NC2=2*NGWK*MAX(NUMAORMAX,NSTATE)+8
          IF(TDAVI) THEN
            NCM=2*NGWK*NDAVV
            NCN=2*NGWK*NDAVV
          ELSE
            IF(TFRSBLK) THEN
              NCM=2*NGWK*MAX(NSTATE,(NKRY_MAX+1)*NKRY_BLOCK)
            ELSE
              NCM=2*NGWK*MAX(NSTATE,NKRY_MAX*NKRY_BLOCK)
            ENDIF
            NCN=1
          ENDIF
          NSC0=1
        ELSE
          NC0=2*NGW*NSTATE+8
          NC2=2*NGW*NSTATE+8
          NSC0=1
          IF(TSDE) THEN
            NCM=1
            NCN=1
          ELSE IF(DIIS) THEN
            NCM=(NGWK*NSTATE+8)*MDIIS
            NCN=((NGWK*NSTATE+8)*MDIIS)/4
          ELSE IF(PCG) THEN
            NCM=2*NGWK*NSTATE
            NCN=1
          ENDIF
        ENDIF
        CALL MEMORY(IP_C0,NC0,'C0')
        CALL MEMORY(IP_CM,NCM,'CM')
        CALL MEMORY(IP_CN,NCN,'CN')
        CALL MEMORY(IP_C2,NC2,'C2')
        CALL MEMORY(IP_SC0,NSC0,'SC0')
        CALL MEMORY(IP_VPP,NGWK,'VPP')
C..FNL and DFNL
        CALL FNLALLOC(NSTATE,.FALSE.,.FALSE.)
C
        CALL OHFD(C0,CM,CN,C2,SC0,VPP)
      ENDIF
C
      IF(TMEMCHK) CALL PRINT_MEMORY
      CALL FREEM(IP_C0)
      CALL FREEM(IP_CM)
      CALL FREEM(IP_CN)
      CALL FREEM(IP_C2)
      CALL FREEM(IP_SC0)
      CALL FREEM(IP_VPP)
      CALL FNLDEALLOC(.FALSE.,.FALSE.)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE OHIN
C     ==--------------------------------------------------------------==
C     ==  THIS ROUTINE READS THE SECTION &HARNESS &END                ==
C     ==--------------------------------------------------------------==
C     ==  THE INPUT HAS TO BE IN THE FOLLOWING FORMAT                 ==
C     ==     &HARDNESS                                                ==
C     ==        Options                                               ==
C     ==     &END                                                     ==
C     ==--------------------------------------------------------------==
C     ==  LIST OF OPTIONS                                             ==
C     ==  LOCALIZE                                                    ==
C     ==  DIAGONAL [OFF]                                              ==
C     ==  ORBITALS                                                    ==
C     ==    numo                                                      ==
C     ==  REFATOM                                                     ==
C     ==    refat                                                     ==
C     ==                                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'linres.inc'
      INCLUDE 'irat.inc'
      INTEGER   IUNIT,IERR,INSCAN
      CHARACTER LINE*80
      INTEGER   MSGLEN
C     ==--------------------------------------------------------------==
C     ==  DEFAULTS                                                    ==
C     ==--------------------------------------------------------------==
      IF(.NOT.PARENT) GOTO 9999
      IUNIT = 5
      LOCAL_ORB=.FALSE.
      DIAGONAL=.TRUE.
      NUMO=-1
      REFAT=-1
C     ==--------------------------------------------------------------==
      IERR=INSCAN(IUNIT,'&HARDNESS')
      IF(IERR.NE.0) GOTO 100
C     ==--------------------------------------------------------------==
   10 CONTINUE
        READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
        IF(INDEX(LINE,'&END').NE.0) GOTO 100
        IF(INDEX(LINE,'LOCALIZE').NE.0) THEN
          LOCAL_ORB=.TRUE.
          GOTO 10
        ELSE IF(INDEX(LINE,'DIAGONAL').NE.0) THEN
          DIAGONAL=.TRUE.
          IF(INDEX(LINE,'OFF').NE.0) DIAGONAL=.FALSE.
          GOTO 10
        ELSE IF(INDEX(LINE,'ORBITAL').NE.0) THEN
          READ(IUNIT ,ERR=99,END=99,FMT=*) NUMO
          GOTO 10
        ELSE IF(INDEX(LINE,'REFATOM').NE.0) THEN
          READ(IUNIT ,ERR=99,END=99,FMT=*) REFAT
          GOTO 10
        ELSE 
C..Dummy line
          GOTO 10
        ENDIF
C     ==--------------------------------------------------------------==
   99 CONTINUE
      WRITE(6,*) ' OHIN  : ERROR IN READING INPUT FILE'
      CALL STOPGM('OHIN',' ')
C     ==--------------------------------------------------------------==
  100 CONTINUE
C     ==--------------------------------------------------------------==
C     ==  Test of options                                             ==
C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     ==  WRITE INFO TO OUTPUT                                        ==
C     ==--------------------------------------------------------------==
      WRITE(6,*)
      WRITE(6,'(A)') ' ORBITAL HARDNESS INPUT OPTIONS '
      IF(DIAGONAL) THEN
        WRITE(6,'(A)') ' CALCULATE DIAGONAL ORBITAL HARDNESS'
      ELSE
        WRITE(6,'(A)') ' CALCULATE GENERALIZE ORBITAL HARDNESS'
      ENDIF
      IF(LOCAL_ORB) WRITE(6,'(A)') ' USE WANNIER FUNCTION BASIS '
      IF(NUMO.LT.0) THEN
        WRITE(6,'(A)') ' INCLUDE ALL ORBITALS '
      ELSE
        WRITE(6,'(A,T60,I10)') ' NUMBER OF ORBITALS INCLUDED ',NUMO
      ENDIF
      IF(REFAT.GT.0) WRITE(6,'(A,T60,I10)') 
     * ' REFERENZ ATOM ',REFAT
      WRITE(6,*)
C     ==--------------------------------------------------------------==
 9999 CONTINUE
#ifdef PARALLEL
      MSGLEN = 2 * 8/IRAT
      CALL MY_BCAST(LOCAL_ORB,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 2 * 8/IRAT
      CALL MY_BCAST(NUMO,MSGLEN,SOURCE,ALLGRP)
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
