C     ==================================================================
      SUBROUTINE OVLAP(NSTATE,A,C1,C2)
C     ==--------------------------------------------------------------==
C     ==         COMPUTES THE OVERLAP MATRIX A = < C1 | C2 >          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'geq0.inc'
      INCLUDE 'spin.inc'
C     Arguments
      INTEGER    NSTATE
      COMPLEX*16 C1(NGW,*),C2(NGW,*)
      REAL*8     A(NSTATE,*)
C     Variables
      COMPLEX*16 CDUM1(*),CDUM2(*)
      POINTER    (IP_C1,CDUM1),(IP_C2,CDUM2)
      INTEGER    ISUB
      LOGICAL    SYMMETRIC
      LOGICAL    PTR_ASSOCIATED
      EXTERNAL   PTR_ASSOCIATED
C     ==--------------------------------------------------------------==
      IF(NGW.LE.0) THEN
        CALL AZZERO(A,NSTATE*NSTATE)
        RETURN
      ENDIF
      CALL TISET('     OVLAP',ISUB)
C..Check for symmetric overlap
      SYMMETRIC=.FALSE.
      CALL GET_ADDR(IP_C1,C1(1,1))
      CALL GET_ADDR(IP_C2,C2(1,1))
      SYMMETRIC=PTR_ASSOCIATED(IP_C1,IP_C2)
      IF(TLSD) THEN
        CALL AZZERO(A,NSTATE*NSTATE)
C..Alpha spin
        IF(SYMMETRIC) THEN
          CALL DSYRK('U','T',NSUP,2*NGW,2.D0,C1,2*NGW,0.D0,A,NSTATE)
          CALL DMATC('U',NSUP,A,NSTATE)
        ELSE
          CALL DGEMM('T','N',NSUP,NSUP,2*NGW,2.0D0,C1(1,1),2*NGW,
     *               C2(1,1),2*NGW,0.0D0,A(1,1),NSTATE)
        ENDIF
        IF(GEQ0)
     *    CALL DGER(NSUP,NSUP,-1.0D0,C1(1,1),2*NGW,
     *              C2(1,1),2*NGW,A(1,1),NSTATE)
C..Beta spin
        IF(SYMMETRIC) THEN
          CALL DSYRK('U','T',NSDOWN,2*NGW,2.D0,C1(1,NSUP+1),
     *               2*NGW,0.D0,A(NSUP+1,NSUP+1),NSTATE)
          CALL DMATC('U',NSDOWN,A(NSUP+1,NSUP+1),NSTATE)
        ELSE
          CALL DGEMM('T','N',NSDOWN,NSDOWN,2*NGW,2.0D0,C1(1,NSUP+1),
     *               2*NGW,C2(1,NSUP+1),2*NGW,0.0D0,
     *               A(NSUP+1,NSUP+1),NSTATE)
        ENDIF
        IF(GEQ0)
     *  CALL DGER(NSDOWN,NSDOWN,-1.0D0,C1(1,NSUP+1),2*NGW,
     *            C2(1,NSUP+1),2*NGW,A(NSUP+1,NSUP+1),NSTATE)
      ELSE
C..LDA
        IF(SYMMETRIC) THEN
          CALL DSYRK('U','T',NSTATE,2*NGW,2.D0,C1,2*NGW,0.D0,A,NSTATE)
          CALL DMATC('U',NSTATE,A,NSTATE)
        ELSE
          CALL DGEMM('T','N',NSTATE,NSTATE,2*NGW,2.0D0,C1(1,1),2*NGW,
     &               C2(1,1),2*NGW,0.0D0,A(1,1),NSTATE)
        ENDIF
        IF(GEQ0)
     *    CALL DGER(NSTATE,NSTATE,-1.0D0,C1(1,1),2*NGW,C2(1,1),2*NGW,
     &              A(1,1),NSTATE)
      ENDIF
      CALL TIHALT('     OVLAP',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE OVLAP2(NGW,N1,N2,A,C1,C2)
C     ==--------------------------------------------------------------==
C     ==         COMPUTES THE OVERLAP MATRIX A = < C1 | C2 >          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'geq0.inc'
      INTEGER NGW,N1,N2, ISUB
      COMPLEX*16 C1(NGW,*),C2(NGW,*)
      REAL*8 A(N1,*)
C     ==--------------------------------------------------------------==
      IF(NGW.LE.0) THEN
        CALL AZZERO(A,N1*N2)
        RETURN
      ENDIF
      CALL TISET('    OVLAP2',ISUB)
#if defined (__ES)
cmb   This is not necessary if your BLAS libraries are optimized for ES 
      CALL DGEMM2('T','N',N1,N2,2*NGW,2.0D0,C1(1,1),2*NGW,
     &           C2(1,1),2*NGW,0.0D0,A(1,1),N1)
#else
      CALL DGEMM('T','N',N1,N2,2*NGW,2.0D0,C1(1,1),2*NGW,
     &           C2(1,1),2*NGW,0.0D0,A(1,1),N1)
#endif
      IF(GEQ0)
     *     CALL DGER(N1,N2,-1.0D0,C1(1,1),2*NGW,C2(1,1),2*NGW,A(1,1),N1)
      CALL TIHALT('    OVLAP2',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE DMATC(UPLO,N,A,LDA)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      CHARACTER*1 UPLO
      INTEGER N,LDA, I,J
      REAL*8 A(LDA,N)
C     ==--------------------------------------------------------------==
      IF(UPLO.EQ.'U'.OR.UPLO.EQ.'u') THEN
!$OMP parallel do private(I,J) schedule(static,1)
        DO I=1,N
          DO J=I+1,N
            A(J,I)=A(I,J)
          ENDDO
        ENDDO
      ELSEIF(UPLO.EQ.'L'.OR.UPLO.EQ.'l') THEN
!$OMP parallel do private(I,J) schedule(static,1)
        DO I=1,N
          DO J=I+1,N
            A(I,J)=A(J,I)
          ENDDO
        ENDDO
      ELSE
        CALL STOPGM('DMATC',' ERROR ')
      ENDIF     
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE OVLAP_C(NSTATE,A,C1,C2)
C     ==--------------------------------------------------------------==
C     ==         COMPUTES THE OVERLAP MATRIX A = < C1 | C2 >          ==
C     ==         WITHOUT INVERSION SYMMETRY                           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      COMPLEX*16 ZONE,ZZERO
      PARAMETER  (ZZERO=(0.D0,0.D0),ZONE=(1.D0,0.D0))
      INCLUDE 'system.h'
      INCLUDE 'geq0.inc'
      INCLUDE 'spin.inc'
      INTEGER NSTATE, ISUB
      COMPLEX*16 C1(NGWK,*),C2(NGWK,*),A(NSTATE,*)
C     ==--------------------------------------------------------------==
      IF(NGWK.LE.0) THEN
#ifdef _BGL  
        CALL AZZERO(A,2*NSTATE*NSTATE)
#else
        CALL ZAZZERO(A,NSTATE*NSTATE)
#endif
        RETURN
      ENDIF
      CALL TISET ('   OVLAP_C',ISUB)
      IF(TLSD) THEN
#ifdef _BGL  
        CALL AZZERO(A,2*NSTATE*NSTATE)
#else
        CALL ZAZZERO(A,NSTATE*NSTATE)
#endif
C..Alpha spin
        CALL ZGEMM('C','N',NSUP,NSUP,NGWK,ZONE,C1(1,1),NGWK,C2(1,1),
     &     NGWK,ZZERO,A(1,1),NSTATE)
C..Beta spin
        CALL ZGEMM('C','N',NSDOWN,NSDOWN,NGWK,ZONE,C1(1,NSUP+1),
     &     NGWK,C2(1,NSUP+1),NGWK,ZZERO,A(NSUP+1,NSUP+1),NSTATE)
      ELSE
        CALL ZGEMM('C','N',NSTATE,NSTATE,NGWK,ZONE,C1(1,1),NGWK,
     &     C2(1,1),NGWK,ZZERO,A(1,1),NSTATE)
      ENDIF
      CALL TIHALT('   OVLAP_C',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE OVLAP_H(NSTATE,A,C1,C2)
C     ==--------------------------------------------------------------==
C     ==         COMPUTES THE OVERLAP MATRIX A = < C1 | C2 >          ==
C     ==         WITH INVERSION SYMMETRY (HERMITIAN)                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'geq0.inc'
      INTEGER NSTATE, I,J,ISUB
      COMPLEX*16 C1(NGWK,*),C2(NGWK,*),A(NSTATE,*),ZDOTC,TMPV
      EXTERNAL ZDOTC
C     ==--------------------------------------------------------------==
      IF(NGWK.LE.0) THEN
#ifdef _BGL  
        CALL AZZERO(A,2*NSTATE*NSTATE)
#else
        CALL ZAZZERO(A,NSTATE*NSTATE)
#endif
        RETURN
      ENDIF
      CALL TISET ('   OVLAP_H',ISUB)
C$OMP parallel do private(J,I,TMPV)
      DO J=1,NSTATE
        A(J,J)=ZDOTC(NGWK,C1(1,J),1,C2(1,J),1)
        DO I=J+1,NSTATE
          TMPV=ZDOTC(NGWK,C1(1,I),1,C2(1,J),1)
          A(I,J)=TMPV
          A(J,I)=DCONJG(TMPV)
        ENDDO
      ENDDO
      CALL TIHALT('   OVLAP_H',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE OVLAP2_C(NGWK,N1,N2,A,C1,C2)
C     ==--------------------------------------------------------------==
C     ==         COMPUTES THE OVERLAP MATRIX A = < C1 | C2 >          ==
C     ==         WITHOUT INVERSION SYMMETRY                           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      COMPLEX*16 ZONE,ZZERO
      PARAMETER(ZZERO=(0.D0,0.D0),ZONE=(1.D0,0.D0))
      INTEGER NGWK,N1,N2, ISUB
      COMPLEX*16 C1(NGWK,*),C2(NGWK,*),A(N1,*)
C     ==--------------------------------------------------------------==
      IF(NGWK.LE.0) THEN
#ifdef _BGL  
        CALL AZZERO(A,2*N1*N2)
#else
        CALL ZAZZERO(A,N1*N2)
#endif
        RETURN
      ENDIF
      CALL TISET ('  OVLAP2_C',ISUB)
      IF(N2.GT.1) THEN
        CALL ZGEMM('C','N',N1,N2,NGWK,ZONE,C1(1,1),NGWK,C2(1,1),
     &       NGWK,ZZERO,A(1,1),N1)
      ELSE
        CALL ZGEMV('C',NGWK,N1,ZONE,C1(1,1),NGWK,C2(1,1),1,
     &       ZZERO,A(1,1),1)
      ENDIF
      CALL TIHALT('  OVLAP2_C',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE OVLAP_ADD(NSTATE,A,C1,C2)
C     ==--------------------------------------------------------------==
C     ==         SUMS UP THE OVERLAP MATRIX A = A + < C1 | C2 >       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'geq0.inc'
      INCLUDE 'spin.inc'
      INTEGER NSTATE,ISUB
      COMPLEX*16 C1(NGW,*),C2(NGW,*)
      REAL*8 A(NSTATE,*)
C     ==--------------------------------------------------------------==
      IF(NGWK.LE.0) RETURN
      CALL TISET('     OVLAP',ISUB)
      IF(TLSD) THEN
C..Alpha spin
        CALL DGEMM('T','N',NSUP,NSUP,2*NGW,2.0D0,C1(1,1),2*NGW,
     &             C2(1,1),2*NGW,1.0D0,A(1,1),NSTATE)
        IF(GEQ0)
     &  CALL DGER(NSUP,NSUP,-1.0D0,C1(1,1),2*NGW,
     &             C2(1,1),2*NGW,A(1,1),NSTATE)
C..Beta spin
        CALL DGEMM('T','N',NSDOWN,NSDOWN,2*NGW,2.0D0,C1(1,NSUP+1),
     &             2*NGW,C2(1,NSUP+1),2*NGW,1.0D0,
     &             A(NSUP+1,NSUP+1),NSTATE)
        IF(GEQ0)
     &  CALL DGER(NSDOWN,NSDOWN,-1.0D0,C1(1,NSUP+1),2*NGW,
     &            C2(1,NSUP+1),2*NGW,A(NSUP+1,NSUP+1),NSTATE)

      ELSE
        CALL DGEMM('T','N',NSTATE,NSTATE,2*NGW,2.0D0,C1(1,1),2*NGW,
     &             C2(1,1),2*NGW,1.0D0,A(1,1),NSTATE)
        IF(GEQ0)
     &       CALL DGER(NSTATE,NSTATE,-1.0D0,C1(1,1),2*NGW,C2(1,1),
     &                 2*NGW,A(1,1),NSTATE)
      ENDIF
      CALL TIHALT('     OVLAP',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

