C     ==================================================================
      SUBROUTINE PBC(X1,Y1,Z1,X2,Y2,Z2,M,AP,IBRV)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      INCLUDE 'isos.inc'
      include 'mm_dim.inc'
C     Arguments
      REAL*8  X1,Y1,Z1,X2,Y2,Z2,AP(4)
      INTEGER M,IBRV
      REAL*8  A(4)
C     ==--------------------------------------------------------------==
C     AK: QM/MM bugfix. 2005/11/15
C     if we are in MM dimensions, we need to apply PBC for the MM box.
C     problem spotted by bernd ensing and marco de vivo.
      IF(mm_stat) THEN  ! true means QM dimensions. just copy
         A(1)=AP(1)
         A(2)=AP(2)
         A(3)=AP(3)
         A(4)=AP(4)
      ELSE
        IF(.NOT.TQMMM)CALL STOPGM('PBC','BUG:mm_stat=.false. w/o QM/MM')
        IF((IBRV.EQ.1).OR.(IBRV.EQ.8)) THEN  ! MM supports only orthorhombic boxes.
          A(1)=BOX_au(1)/2.0d0
          A(2)=BOX_au(2)/2.0d0
          A(3)=BOX_au(3)/2.0d0
        ELSE
          CALL STOPGM('PBC','QM/MM NEEDS ORTHORHOMBIC BOX')
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      IF(TCLUST.and.mm_stat) THEN
        IF(TONED) THEN
          CALL PBC1(X1,Y1,Z1,X2,Y2,Z2,M,A,IBRV)
        ELSEIF(TTWOD) THEN
          CALL PBC2(X1,Y1,Z1,X2,Y2,Z2,M,A,IBRV)
        ELSE
          X2=X1
          Y2=Y1
          Z2=Z1
        ENDIF
      ELSE
        CALL PBC3(X1,Y1,Z1,X2,Y2,Z2,M,A,IBRV)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PBC3(X1,Y1,Z1,X2,Y2,Z2,M,A,IBRAV)
C     ==--------------------------------------------------------------==
C     == CALCULATES THE PERIODIC BOUNDARY CONDITIONS FOR SOME         ==
C     == STRUCTURES:                                                  ==
C     ==             SIMPLE CUBIC          (IBRAV=1)                  ==
C     ==             FACE CENTERED CUBIC   (IBRAV=2)                  ==
C     ==             ORTHOROMBIC           (IBRAV=8)                  ==
C     ==             TRIGONAL              (IBRAV=12)                 ==
C     ==                                                              ==
C     == OTHER THAN THESE, WE USE A GENERAL ROUTINE                   ==
C     ==                                                              ==
C     == FOR FCC THE ROUTINE IS TAKEN FROM                            ==
C     == CCP5 INF. QUAT. #10,SEPT(1983)                               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'bc.inc'
      INCLUDE 'metr.inc'
C     Arguments
      REAL*8  X1,Y1,Z1,X2,Y2,Z2,A(4)
      INTEGER M,IBRAV
C     Variables
      REAL*8  B(4),C(4),DX,R(3),S(3),TA,AA,S2,X2T,Y2T,XT,X0,Y0,X3,Y3,XIC
      INTEGER I
C     ==--------------------------------------------------------------==
#ifdef __SGI
C..bug with the optimizer
      X2 = X1
      Y2 = Y1
      Z2 = Z1
#endif  
      TA=2.D0*A(1)*M
      AA=A(1)*M
      GO TO (5,10,99,25,99,99,99,15,99,99,99,20,99,25) IBRAV
C     ==--------------------------------------------------------------==
C     == Simple Cubic                                                 ==
C     ==--------------------------------------------------------------==
    5 CONTINUE
      X2=DMOD(X1,TA)
      Y2=DMOD(Y1,TA)
      Z2=DMOD(Z1,TA)
      X2=X2-TA*DINT(X2/AA)
      Y2=Y2-TA*DINT(Y2/AA)
      Z2=Z2-TA*DINT(Z2/AA)
      RETURN
C     ==--------------------------------------------------------------==
C     == Face Centered Cubic                                          ==
C     ==--------------------------------------------------------------==
   10 CONTINUE
      S2=DSQRT(2.0D0)
      X2T=( X1+Y1)/S2
      Y2T=(-X1+Y1)/S2
      X2=X2T
      Y2=Y2T
      X2=DMOD(X2,TA)
      Y2=DMOD(Y2,TA)
      Z2=DMOD(Z1,TA*S2)
      X2=X2-TA*DINT(X2/AA)
      Y2=Y2-TA*DINT(Y2/AA)
      Z2=Z2-TA*S2*DINT(Z2/(S2*AA))
      IF(DABS(X2)+DABS(Y2)+S2*DABS(Z2).LT.TA) GOTO 11
      X2=X2-DSIGN(AA,X2)
      Y2=Y2-DSIGN(AA,Y2)
      Z2=Z2-DSIGN(S2*AA,Z2)
   11 CONTINUE
      XT=(X2-Y2)/S2
      Y2=(X2+Y2)/S2
      X2=XT
      RETURN
C     ==--------------------------------------------------------------==
C     == Orthorhombic                                                 ==
C     ==--------------------------------------------------------------==
   15 CONTINUE
      DO I=1,3
        B(I)=2*A(I)*M
        C(I)=A(I)*M
      ENDDO
      X2=DMOD(X1,B(1))
      Y2=DMOD(Y1,B(2))
      Z2=DMOD(Z1,B(3))
      X2=X2-B(1)*DINT(X2/C(1))
      Y2=Y2-B(2)*DINT(Y2/C(2))
      Z2=Z2-B(3)*DINT(Z2/C(3))
      RETURN
C     ==--------------------------------------------------------------==
C     == Hexagonal and special case of triclinic: monocl., gamma<>90  ==
C     ==--------------------------------------------------------------==
 25   CONTINUE
      IF(A(2).EQ.0.0D0) GOTO 99
      DO I=1,3
        C(I)=A(I)*M
        B(I)=2*C(I)
      ENDDO
      Y0=Y1
      Y2=DMOD(Y1,B(2))
      Z2=DMOD(Z1,B(3))
      Y2=Y2-B(2)*DINT(Y2/C(2))
      Z2=Z2-B(3)*DINT(Z2/C(3))
      DX=(Y2-Y0)*A(4)           ! A(4)=SQRT(3)/3
      X2=DMOD(X1-DX,B(1))
      X2=X2-B(1)*DINT(X2/C(1))
      RETURN
C     ==--------------------------------------------------------------==
C     == Trigonal                                                     ==
C     ==--------------------------------------------------------------==
   20 CONTINUE
      DO 22 I=1,4
      B(I)=2*A(I)*M
      C(I)=A(I)*M
   22 CONTINUE
      Z2=DMOD(Z1,B(3))
      Z2=Z2-B(3)*DINT(Z2/C(3))
      CALL ROT(X1,Y1,X0,Y0,V1CRYS,1)
      X0=DMOD(X0,B(1))
      X0=X0-B(1)*DINT(X0/C(1))
      CALL ROT(X0,Y0,X3,Y3,V1CRYS,-1)
      CALL ROT(X3,Y3,X0,Y0,V2CRYS,1)
      X0=DMOD(X0,B(2))
      X0=X0-B(2)*DINT(X0/C(2))
      CALL ROT(X0,Y0,X3,Y3,V2CRYS,-1)
      CALL ROT(X3,Y3,X0,Y0,V3CRYS,1)
      X0=DMOD(X0,B(4))
      X0=X0-B(4)*DINT(X0/C(4))
      CALL ROT(X0,Y0,X3,Y3,V3CRYS,-1)
      CALL ROT(X3,Y3,X0,Y0,V1CRYS,1)
      X0=DMOD(X0,B(1))
      X0=X0-B(1)*DINT(X0/C(1))
      CALL ROT(X0,Y0,X2,Y2,V1CRYS,-1)
      RETURN
C     ==--------------------------------------------------------------==
C     == The rest : use a general procedure                           ==
C     ==--------------------------------------------------------------==
   99 CONTINUE
      R(1)=X1
      R(2)=Y1
      R(3)=Z1
      CALL DGEMV('T',3,3,1.0D0,HTM1,3,R,1,0.0D0,S,1)
      XIC=DBLE(M)
      S(1)=S(1)-NINT(S(1)/XIC)*XIC
      S(2)=S(2)-NINT(S(2)/XIC)*XIC
      S(3)=S(3)-NINT(S(3)/XIC)*XIC
      CALL DGEMV('T',3,3,1.0D0,HT,3,S,1,0.0D0,R,1)
      X2=R(1)
      Y2=R(2)
      Z2=R(3)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE ROT(X1,Y1,X0,Y0,VV,ISIGN)
C     ==--------------------------------------------------------------==
C     == THIS ROUTINE TRANSFORMS THE POINT (X1,Y1) INTO THE POINT     ==
C     == (X0,Y0) CORRESPONDING TO A ROTATION OF AN ANGLE WHICH COS    ==
C     == AND SIN ARE GIVEN BY VV(3) AND VV(4) RESPECTIVELY. THE       ==
C     == SENSE OF THE ROTATION IS GIVEN BY ISIGN (+ FOR CLOCKWISE,    ==
C     == AND - FOR ANTICLOCKWISE)                                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER ISIGN
      REAL*8  X1,Y1,X0,Y0,VV(4)
      X0=X1*VV(3)+Y1*VV(4)*DBLE(ISIGN)
      Y0=-X1*VV(4)*DBLE(ISIGN)+Y1*VV(3)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PBC1(X1,Y1,Z1,X2,Y2,Z2,M,A,IBRAV)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8  X1,Y1,Z1,X2,Y2,Z2,A(3)
      INTEGER M,IBRAV
C     Variables
      REAL*8  TA,AA
C     ==--------------------------------------------------------------==
#ifdef __SGI
      X2 = X1
      Y2 = Y1
      Z2 = Z1
#endif
      TA=2.D0*A(1)*M
      AA=A(1)*M
      X2=DMOD(X1,TA)
      X2=X2-TA*DINT(X2/AA)
      Y2=Y1
      Z2=Z1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PBC2(X1,Y1,Z1,X2,Y2,Z2,M,A,IBRAV)
      IMPLICIT NONE
      INCLUDE 'isos.inc'
C     Arguments
      REAL*8  X1,Y1,Z1,X2,Y2,Z2,A(4)
      INTEGER M,IBRAV
C     Variables
      REAL*8  B(3),C(3),TA,AA,DX,Y0
      INTEGER I
C     ==--------------------------------------------------------------==
#ifdef __SGI
      X2 = X1
      Y2 = Y1
      Z2 = Z1
#endif
      TA=2.D0*A(1)*M
      AA=A(1)*M
      IF(IBRAV.EQ.1) THEN
        IF(SNORMAL.EQ.1) THEN      ! YZ PLANE
          Y2=DMOD(Y1,TA)
          Z2=DMOD(Z1,TA)
          X2=X1
          Y2=Y2-TA*DINT(Y2/AA)
          Z2=Z2-TA*DINT(Z2/AA)
        ELSE IF(SNORMAL.EQ.2) THEN ! ZX PLANE
          X2=DMOD(X1,TA)
          Z2=DMOD(Z1,TA)
          X2=X2-TA*DINT(X2/AA)
          Y2=Y1
          Z2=Z2-TA*DINT(Z2/AA)
        ELSE IF(SNORMAL.EQ.3) THEN ! XY PLANE
          X2=DMOD(X1,TA)
          Y2=DMOD(Y1,TA)
          X2=X2-TA*DINT(X2/AA)
          Y2=Y2-TA*DINT(Y2/AA)
          Z2=Z1
        ENDIF
      ELSE IF (IBRAV.EQ.4) THEN
        IF(SNORMAL.EQ.1.OR.SNORMAL.EQ.2) 
     .    CALL STOPGM('PBC2','HEXAGONAL SURFACE MUST BE IN XY PLANE')
        DO I=1,2
          C(I)=A(I)*M
          B(I)=2*C(I)
        ENDDO
        Y0=Y1
        Y2=DMOD(Y1,B(2))
        Y2=Y2-B(2)*DINT(Y2/C(2))
        DX=(Y2-Y0)*A(4)         ! A(4)=SQRT(3)/3
        X2=DMOD(X1-DX,B(1))
        X2=X2-B(1)*DINT(X2/C(1))
        Z2=Z1
      ELSE
        DO I=1,3
          B(I)=2*A(I)*M
          C(I)=A(I)*M
        ENDDO
        IF(SNORMAL.EQ.1) THEN      ! YZ PLANE
          Y2=DMOD(Y1,B(2))
          Z2=DMOD(Z1,B(3))
          X2=X1
          Y2=Y2-B(2)*DINT(Y2/C(2))
          Z2=Z2-B(3)*DINT(Z2/C(3))
        ELSE IF(SNORMAL.EQ.2) THEN ! ZX PLANE
          X2=DMOD(X1,B(1))
          Z2=DMOD(Z1,B(3))
          X2=X2-B(1)*DINT(X2/C(1))
          Y2=Y1
          Z2=Z2-B(3)*DINT(Z2/C(3))
        ELSE IF(SNORMAL.EQ.3) THEN ! XY PLANE
          X2=DMOD(X1,B(1))
          Y2=DMOD(Y1,B(2))
          X2=X2-B(1)*DINT(X2/C(1))
          Y2=Y2-B(2)*DINT(Y2/C(2))
          Z2=Z1
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PBC_CLAS(X1,Y1,Z1,X2,Y2,Z2)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'clas.inc'
C     Arguments
      REAL*8  X1,Y1,Z1,X2,Y2,Z2
C     Variables
      REAL*8  R(3),S(3)
C     ==--------------------------------------------------------------==
      R(1)=X1
      R(2)=Y1
      R(3)=Z1
      CALL DGEMV('T',3,3,1.0D0,CLHTM1,3,R,1,0.0D0,S,1)
      S(1)=S(1)-NINT(S(1))
      S(2)=S(2)-NINT(S(2))
      S(3)=S(3)-NINT(S(3))
      CALL DGEMV('T',3,3,1.0D0,CLHT,3,S,1,0.0D0,R,1)
      X2=R(1)
      Y2=R(2)
      Z2=R(3)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

