C     ==================================================================
      SUBROUTINE PCGRAD(C0,C2,SC0,VPP,HNM1,
     &                  RHOE,PSI,SCR,LSCR,TAU0,NSTATE,DINIT)
C     ==--------------------------------------------------------------==
C     ==  PRECONDITIONED CONJUGATE GRADIENT OPTIMIZATION              ==
C     ==--------------------------------------------------------------==
C     == IMPROVED VERSIONS (October 2003)                             ==
C     ==--------------------------------------------------------------==
C     == In logscale, we should have a straight line if we plot CNORM ==
C     == GEMAX versus the number of iterations                        ==
C     == When CNORM is less than 1.e-5 (or 1.e-6), the error in the   ==
C     == determination of minimum from energy and from forces are not ==
C     == negligible.                                                  ==
C     == As we want decrease CNORM, instead of evaluation             ==
C     == twice the energy (E1, E2), we evaluate once the force and    ==
C     == energy of a point (F1,E1) and determnie the minimum along    ==
C     == the line only using forces (F0 and F1).                      ==
C     == So we decrease as we want CNORM.                             ==
C     == An evaluation of forces is three times more cpu-time more    ==
C     == consuming than an evaluation of energy, so we switch to this ==
C     == method only when CNORM is small.                             ==
C     ==--------------------------------------------------------------==
C     == Some tricks:                                                 ==
C     == The scalar product <C2.P> is evaluated only for the occupied ==
C     == states which is coherent with energy.                        ==
C     ==                                                              ==
C     == We add a FORGET parameter to forget a little bit.            ==
C     == This stabilizes the algorithm but it is not crucial.         ==
C     == At the beginning (ISTEP < 10) we want to forget a lot        ==
C     == because the steepest descent works fine.                     ==
C     ==                                                              ==
C     == We tested to refine the line minimization search if ALAM is  ==
C     == too big but there is no real improvement.                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'tpar.inc'
      INCLUDE 'pslo.inc'        !TIVAN
      INCLUDE 'nlps.inc'
      INCLUDE 'ropt.inc'        !MODENS
      INCLUDE 'norm.inc'        !GEMAX CNORM
      INCLUDE 'mm_input.inc'    !qmmm
      INCLUDE 'mm_dim.inc'      !QM/MM
C     Arguments
      INTEGER    LSCR,NSTATE
      COMPLEX*16 C0(NGW,NSTATE),C2(NGW,NSTATE),
     &           SC0(NGW,NSTATE),HNM1(NGW,NSTATE),PSI(NNR1)
      REAL*8     VPP(NGW),RHOE(NNR1),SCR(LSCR),TAU0(3*NAX*NSX)
      LOGICAL    DINIT
C     Variables
      INTEGER    MAXHIST
      PARAMETER  (MAXHIST=10)
      DIMENSION  F(NSTATE)
      REAL*8     TSCR(3*NAX*NSX)
      POINTER    (IP_TSCR,TSCR)
      CHARACTER  TAG*30
      LOGICAL    oldstatus,statusdummy
      REAL*8     GGNORM,ALAM,DE,A,DETOT,GAMMA,FORGET,
     &           DOTP,
     &           EHIST(MAXHIST),GHIST(MAXHIST),FHIST(MAXHIST)
      SAVE       EHIST,GHIST,FHIST
      EXTERNAL   DOTP
      INTEGER    ISUB,NOCC,I,IHIST,IG,LSCR2,
     &           LPCGRAD,
     &           ISTEP,ILSR
      DATA       ISTEP /0/, ILSR /0/
      SAVE       ISTEP,ILSR
C     ==--------------------------------------------------------------==
      CALL TISET('    PCGRAD',ISUB)
C     ==--------------------------------------------------------------==
C     we need to 'go mm' here so that the TSCR offset is correct.
      IF(qmmm) call mm_dim(mm_go_mm,oldstatus)
C     ==--------------------------------------------------------------==
C     Test SCR
      CALL GIVE_SCR_PCGRAD(LPCGRAD,TAG,NSTATE)
      CALL TEST_SCR('PCGRAD',TAG,LSCR,LPCGRAD)
#ifdef _BGL 
      LSCR2=LSCR-3*NAX*NSX-MOD(3*NAX*NSX,2)
#else
      LSCR2=LSCR-3*NAX*NSX
#endif
      CALL GET_ADDR(IP_TSCR,SCR(LSCR2+1))
C     ==--------------------------------------------------------------==
      IF(qmmm) call mm_dim(mm_go_qm,statusdummy)
C     ==--------------------------------------------------------------==
      NOCC=0
      DO I=1,NSTATE
        IF(F(I).GT.1.D-5) THEN
          NOCC=NOCC+1
        ENDIF
      ENDDO
      IF(DINIT.OR.ILSR.LT.0) THEN
C       We start a new direction.
        ISTEP=0
      ENDIF
      ISTEP=ISTEP+1
      IF(PREC.AND.ILSR.NE.-2) THEN
        CALL PRECON(C2,VPP,F,NGW,NSTATE,0)
      ENDIF
      GGNORM=0.0D0
#if !defined(__VECTOR)
C$OMP parallel do private(I) reduction(+:GGNORM) schedule(static)
#endif
      DO I=1,NOCC
        GGNORM=GGNORM+DOTP(NGW,C2(1,I),C2(1,I))
      ENDDO
#ifdef PARALLEL
      CALL GLOSUM(1,GGNORM)
#endif
      IF(ISTEP.LE.1) THEN
C     ==--------------------------------------------------------------==
C     ==  INITIAL CALL                                                ==
C     ==--------------------------------------------------------------==
        EHIST(1)=ETOT
        FHIST(1)=DT2BYE
        GHIST(1)=GGNORM
        IF(PCGMIN) THEN
          CALL DCOPY(2*NGW*NSTATE,C2(1,1),1,HNM1(1,1),1)
          ALAM=1.0D0
          IF(PREC.AND.ILSR.NE.-2) THEN
            ILSR=2
          ELSE
            ILSR=1
          ENDIF
          CALL LINESR(NSTATE,C0,HNM1,C2,SC0,VPP,TAU0,TSCR,
     &                RHOE,PSI,SCR,LSCR2,
     *                FHIST(1),DE,A,ALAM,ILSR)
          IF(PARENT) THEN
            WRITE(6,'(A,G8.3,A,T50,G20.13)') 
     *           ' LINE SEARCH : LAMBDA=',ALAM*FHIST(1),
     *           ' PREDICTED ENERGY =',DE
          ENDIF
        ELSE
          CALL DCOPY(2*NGW*NSTATE,C2(1,1),1,HNM1(1,1),1)
          CALL DAXPY(2*NGW*NSTATE,FHIST(1),C2(1,1),1,C0(1,1),1)
        ENDIF
      ELSE
C     ==--------------------------------------------------------------==
C     ==  CONJUGATE GRADIENT                                          ==
C     ==--------------------------------------------------------------==
        IF(ISTEP.LE.10) THEN
C         Almost steepest descent
          FORGET=0.60D0
        ELSEIF(ISTEP.LE.20) THEN
          FORGET=0.91D0
        ELSE
          FORGET=0.95D0
        ENDIF
        IF(ISTEP.GT.MAXHIST) THEN
          IHIST=MAXHIST
          DO I=1,MAXHIST-1
            EHIST(I)=EHIST(I+1)
            FHIST(I)=FHIST(I+1)
            GHIST(I)=GHIST(I+1)
          ENDDO
        ELSE
          IHIST=ISTEP
        ENDIF
        EHIST(IHIST)=ETOT
        GHIST(IHIST)=GGNORM
        DETOT=EHIST(IHIST)-EHIST(IHIST-1)
        FHIST(IHIST)=FHIST(IHIST-1)
        GAMMA=GHIST(IHIST)/GHIST(IHIST-1)

C       Multiply by a forget factor
        GAMMA=FORGET*GAMMA

        IF(PCGMIN) THEN
!$OMP parallel do private(I,IG)
          DO I=1,NSTATE
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(1000))
!IBM* INDEPENDENT
#endif
            DO IG=1,NGW
              HNM1(IG,I)=C2(IG,I)+HNM1(IG,I)*GAMMA
            ENDDO
          ENDDO
          IF(PREC) THEN
            ILSR=4
          ELSE
            ILSR=3
          ENDIF
          CALL LINESR(NSTATE,C0,HNM1,C2,SC0,VPP,TAU0,TSCR,
     &                RHOE,PSI,SCR,LSCR2,
     *                FHIST(IHIST),DE,A,ALAM,ILSR)
          IF(PARENT) THEN
            WRITE(6,'(A,G8.3,A,T50,G20.13)') 
     *           ' LINE SEARCH : LAMBDA=',ALAM*FHIST(IHIST),
     *           ' PREDICTED ENERGY =',DE
          ENDIF
        ELSE
          CALL DAXPY(2*NGW*NSTATE,GAMMA,HNM1(1,1),1,C2(1,1),1)
          CALL DCOPY(2*NGW*NSTATE,C2(1,1),1,HNM1(1,1),1)
          CALL DAXPY(2*NGW*NSTATE,FHIST(IHIST),HNM1(1,1),1,C0(1,1),1)
        ENDIF
      ENDIF
      IF(qmmm) call mm_dim(mm_revert,oldstatus)
      CALL TIHALT('    PCGRAD',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PRECON(C2,VPP,F,NGW,NSTATE,MODE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER    NGW,NSTATE,MODE
      COMPLEX*16 C2(NGW,NSTATE)
      REAL*8     VPP(NGW),F(NSTATE)
C     Variables
      REAL*8     FI(NSTATE)
      REAL*8     FF
      INTEGER    I,J
C     ==--------------------------------------------------------------==
#if defined(__VECTOR) || defined(_BGL)
      IF(MODE.EQ.0) THEN
!$OMP parallel do private(I,J,FF)
        DO I=1,NSTATE
          IF(F(I).GT.0.01D0) THEN
            FF=1.0D0/F(I)
          ELSE
            FF=1.0D0
          ENDIF
#ifdef _BGL 
          CALL ALIGNX(16,VPP(1))
!IBM* ASSERT (ITERCNT(1000))
!IBM* INDEPENDENT
#endif
          DO J=1,NGW
            C2(J,I)=FF*VPP(J)*C2(J,I)
          ENDDO
        ENDDO
      ELSE
!$OMP parallel do private(I,J,FF)
        DO I=1,NSTATE
          IF(F(I).GT.0.01D0) THEN
            FF=1.0D0/F(I)
          ELSE
            FF=1.0D0
          ENDIF
#ifdef _BGL 
          CALL ALIGNX(16,VPP(1))
!IBM* ASSERT (ITERCNT(1000))
!IBM* INDEPENDENT
#endif
          DO J=1,NGW
            C2(J,I)=C2(J,I)/(FF*VPP(J))
          ENDDO
        ENDDO
      ENDIF
#else
!$OMP parallel do private(I,J,FF) schedule(static)
      DO I=1,NSTATE
        IF(F(I).GT.0.01D0) THEN
          FI(I)=1.0D0/F(I)
        ELSE
          FI(I)=1.0D0
        ENDIF
      ENDDO

      IF(MODE.EQ.0) THEN
!$OMP parallel do private(I,J,FF) schedule(static)
#ifdef _BGL 
        CALL ALIGNX(16,VPP(1))
!IBM* ASSERT (ITERCNT(1000))
!IBM* INDEPENDENT
#endif
        DO J=1,NGW
          DO I=1,NSTATE
            C2(J,I)=FI(I)*VPP(J)*C2(J,I)
          ENDDO
        ENDDO
      ELSE
!$OMP parallel do private(I,J,FF) schedule(static)
#ifdef _BGL 
        CALL ALIGNX(16,VPP(1))
!IBM* ASSERT (ITERCNT(1000))
!IBM* INDEPENDENT
#endif
        DO J=1,NGW
          DO I=1,NSTATE
            C2(J,I)=C2(J,I)/(FF*VPP(J))
          ENDDO
        ENDDO
      ENDIF
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LINESR(NSTATE,X,P,C2,SC0,VPP,TAU0,TSCR,
     &                  RHOE,PSI,SCR,LSCR,FAC,DE,A,ALAM,ILSR)
C     ==--------------------------------------------------------------==
C     == ILSR (input/output)                                          ==
C     ==  1   P is only simple forces without preconditioning         ==
C     ==  2   P is the force with preconditioning                     ==
C     ==  3   P is the conjugate gradient scheme without precond.     ==
C     ==  4   P is the conjugate gradient scheme with preconditioning ==
C     ==--------------------------------------------------------------==
C     ==  0   Everything is correct                                   ==
C     == -1   Reset the direction                                     ==
C     == -2   Reset the direction without preconditioning             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ener.inc'
      INCLUDE 'elct.inc'        !F
      INCLUDE 'pslo.inc'        !TIVAN
      INCLUDE 'ions.inc'        !NAT
      INCLUDE 'nlps.inc'
      INCLUDE 'ropt.inc'        !MODENS
      INCLUDE 'norm.inc'        !CNORM
      INCLUDE 'mm_input.inc'    !qmmm
      INCLUDE 'mm_dim.inc'      !QM/MM
C     Arguments
      INTEGER    NSTATE,LSCR
      COMPLEX*16 X(NGW,*),P(NGW,*),C2(NGW,*),
     &           SC0(NGW,*),PSI(*)
      REAL*8     VPP(*),
     &           TAU0(*),TSCR(*),RHOE(*),SCR(*),
     &           FAC,DE,A,ALAM
      INTEGER    ILSR
C     Variables
      DIMENSION  F(NSTATE)
      CHARACTER  TAG*30
      REAL*8     E0,DD,E1,E2,DX,X0,EE,
     &           DOTP
      EXTERNAL   DOTP
      INTEGER    NFUNC,I,
     &           LLINESR
      REAL*8     B,C,FORCE0,FORCE1,EIGV,ALAM0
      LOGICAL    statusdummy,oldstatus
C     ==--------------------------------------------------------------==
C     Test SCR
      if(qmmm) call mm_dim(mm_go_mm,oldstatus)
      CALL GIVE_SCR_LINESR(LLINESR,TAG,NSTATE)
      CALL TEST_SCR('LINESR',TAG,LSCR,LLINESR)
C     ==--------------------------------------------------------------==
      if(qmmm) call mm_dim(mm_go_qm,statusdummy)
C     ==--------------------------------------------------------------==
      CALL EBACK(0)
      NFUNC=0
      E0=ETOT
      CALL DSCAL(2*NGW*NSTATE,FAC,P(1,1),1)
CAK  XETOT does not account for QM/MM, so we have to always do it expensively.
      IF(CNORM.LT.1.0D-5.OR.TINTER.or.qmmm) THEN
C       The norm is small.
C       To avoid the error between forces and energies
C       We use forces for each point to interpolate the new one.
        ALAM=1.0D0
        ALAM0=0.D0
C       Calculates the force in the direction HNM1
        FORCE0=0.0D0
        DO I=1,NSTATE
          IF(F(I).GT.1.D-5) THEN
            FORCE0=FORCE0+DOTP(NGW,P(1,I),C2(1,I))
          ENDIF
        ENDDO
        CALL GLOSUM(1,FORCE0)
C       First point
        CALL DAXPY(2*NGW*NSTATE,ALAM-ALAM0,P(1,1),1,X(1,1),1)
        ALAM0=ALAM

C       Calculate the new point
        IF(qmmm) THEN
          CALL mm_qmmm_FORCEDR(X,C2,SC0,RHOE,PSI,TAU0,TSCR,
     &                 EIGV,SCR,LSCR,NSTATE,1,.TRUE.,.FALSE.,.FALSE.)
        ELSE
          CALL FORCEDR(X,C2,SC0,RHOE,PSI,TAU0,TSCR,EIGV,SCR,LSCR,
     &         NSTATE,1,.TRUE.,(TINTER.EQV..TRUE.)) ! NOTE: we always have to calculate the forces for ego/gmx QM/MM
        ENDIF

        IF(PREC.AND.ILSR.NE.-2) THEN
          CALL PRECON(C2,VPP,F,NGW,NSTATE,0)
        ENDIF
C       Calculates the force in the direction P
        FORCE1=0.0D0
        DO I=1,NSTATE
C         Only for the occupied states
          IF(F(I).GT.1.D-5) THEN
            FORCE1=FORCE1+DOTP(NGW,P(1,I),C2(1,I))
          ENDIF
        ENDDO
        CALL GLOSUM(1,FORCE1)
        E1=ETOT
C       Determine X0 using E0, F0 and F1 (A.X0**2+B.X0+C)
        A = (FORCE1-FORCE0)/(2.D0*ALAM)
        B = FORCE0
        C = E0
        X0=-B/(2*A)
        DE = 0.5D0*(FORCE1-FORCE0)*X0*X0+FORCE0*X0+E0
        IF(X0.LT.0.D0) THEN
          IF(ILSR.EQ.1) THEN
C           The direction should be correct
C           Decrease the size step
            ALAM=0.05D0
          ELSE
C           Reset the direction
            IF(E1.LT.E0) THEN
              ALAM=1.D0
            ELSE
              ALAM=0.05D0
            ENDIF
          ENDIF
          ILSR=-2
C         New point
          CALL DAXPY(2*NGW*NSTATE,ALAM-ALAM0,P(1,1),1,X(1,1),1)
        ELSE
          ILSR=0
          ALAM=X0
          IF(ALAM.GT.3.D0) THEN
           ALAM=3.0D0
          ENDIF
C         New point
          CALL DAXPY(2*NGW*NSTATE,ALAM-ALAM0,P(1,1),1,X(1,1),1)
        ENDIF

      ELSE

        ALAM=1.0D0
C       Standard scheme (low time-consuming)
        DD=0.0D0
        DO I=1,NSTATE
          DD=DD+DOTP(NGW,P(1,I),P(1,I))
        ENDDO
        CALL GLOSUM(1,DD)
C       First point
        CALL DCOPY(2*NGW*NSTATE,X(1,1),1,C2(1,1),1)
        CALL DAXPY(2*NGW*NSTATE,ALAM,P(1,1),1,X(1,1),1)
        ALAM0=ALAM
C       Calculate the new point
        CALL XETOT(NSTATE,X,SC0,TAU0,TSCR,RHOE,PSI,SCR,LSCR)
        IF(ETOT.GT.E0) THEN
          E2=ETOT
          ALAM=0.5D0
        ELSE
          E1=ETOT
          ALAM=2.0D0
        ENDIF
C       Second point
        CALL DCOPY(2*NGW*NSTATE,C2(1,1),1,X(1,1),1)
        CALL DAXPY(2*NGW*NSTATE,ALAM,P(1,1),1,X(1,1),1)
        ALAM0=ALAM
C       Calculate the new point
        CALL XETOT(NSTATE,X,SC0,TAU0,TSCR,RHOE,PSI,SCR,LSCR)      
        IF(ALAM.GT.1.D0) THEN
          E2=ETOT
          DX=1.0D0
        ELSE
          E1=ETOT
          DX=0.5D0
        ENDIF
C       Extrapolation
        IF((2*E0-4*E1+2*E2).EQ.0.D0) THEN
C         Very rare (T.D.)
          X0=DX
        ELSE
          X0=DX*(4*E1-E2-3*E0)/(2*E0-4*E1+2*E2)
        ENDIF
        
        IF((2*X0+DX).EQ.0.D0) THEN
          A=0.D0
        ELSE
          A=(E1-E0)/DX/(2*X0+DX)
        ENDIF
        EE=E0-A*X0*X0
        ALAM=-X0
        IF(ALAM.GT.0.D0) THEN
C         Limit the move
          ILSR=0
          IF(ALAM.GT.3.D0) THEN
C           New minimisation
            ALAM=3.0D0
          ENDIF
        ELSE
C         ALAM is negative, take the lowest energy between E0, E1 and E2
          IF(E2.LT.E0) THEN
            ALAM=2*DX
            ILSR=0
          ELSEIF(E1.LT.E0) THEN
            ALAM=DX
            ILSR=0
          ELSE
C           E0 < E1 and E2
            ALAM=0.05D0
C           Reset the direction
            ILSR=-2
          ENDIF
        ENDIF
        DE=EE+A*(X0+ALAM)**2
        A=A/DD
C       New point
        CALL DCOPY(2*NGW*NSTATE,C2(1,1),1,X(1,1),1)
        CALL DAXPY(2*NGW*NSTATE,ALAM,P(1,1),1,X(1,1),1)
      ENDIF
      
      CALL DSCAL(2*NGW*NSTATE,1.D0/FAC,P(1,1),1)
      IF(ALAM.LT.0.1D0) THEN
        FAC=FAC*0.25D0
      ELSEIF(ALAM.GT.1.0D0) THEN
        FAC=FAC*1.50D0
      ELSEIF(ALAM.LT.0.2D0) THEN
        FAC=FAC*0.75D0
      ENDIF
      CALL EBACK(1)
C     ==--------------------------------------------------------------==
      if(qmmm) call mm_dim(mm_revert,oldstatus)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE XETOT(NSTATE,X,SC0,TAU0,TSCR,RHOE,PSI,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     == CALCULATE THE TOTAL ENERGY FOR A GIVEN WAVEFUNCTION X        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NGW, NNR1, NAX, NSX
      INCLUDE 'pslo.inc'        !TIVAN
C     Arguments
      INTEGER    NSTATE,LSCR
      COMPLEX*16 X(NGW,NSTATE),SC0(NGW,NSTATE),PSI(NNR1)
      REAL*8     TAU0(3*NAX*NSX),TSCR(3*NAX*NSX),RHOE(NNR1),SCR(LSCR)
C     Variables
      CHARACTER  TAG*30
      INTEGER    LXETOT
C     ==--------------------------------------------------------------==
C     Test SCR
      CALL GIVE_SCR_XETOT(LXETOT,TAG,NSTATE)
      CALL TEST_SCR('LINESR',TAG,LSCR,LXETOT)
C     ==--------------------------------------------------------------==
      IF(NONORT.AND.(.NOT.QUENCHB)) THEN
        CALL STOPGM('XETOT','NOT IMPLEMENTED')
      ELSE
          IF(TIVAN) THEN
            CALL RNLSM(X,NSTATE,PSI,SCR,LSCR,1,1,.FALSE.)
          ENDIF
          CALL PREORTHO(X,NSTATE,SCR,LSCR)
          CALL ORTHO(NSTATE,X,SC0,SCR,LSCR)
          CALL RNLSM(X,NSTATE,PSI,SCR,LSCR,1,1,.FALSE.)
          CALL RSCPOT(X,TAU0,TSCR,RHOE,PSI,SCR,LSCR,
     &                .FALSE.,.FALSE.,NSTATE,1)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE EBACK(MODE)
C     ==--------------------------------------------------------------==
C     == Save or restore values of energies                           ==
C     == Mode 0 -> SAVE                                               ==
C     ==      1 -> RESTORE                                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ener.inc'
      INCLUDE 'spin.inc'
C     Arguments
      INTEGER MODE
C     Variables
      REAL*8  BETOT,BEKIN,BEPSEU,BENL,BEHT,BEHEP,BEHEE,BEHII,
     &        BEXC,BVXC,BEGC,BESELF,BESR,
     &        BECNSTR,BERESTR,
     &        BCSUMG,BCSUMR,BCAS(21),BBC(4)
      SAVE    BETOT,BEKIN,BEPSEU,BENL,BEHT,BEHEP,BEHEE,BEHII,
     &        BEXC,BVXC,BEGC,BESELF,BESR,
     &        BECNSTR,BERESTR,
     &        BCSUMG,BCSUMR,BCAS,BBC
C     ==--------------------------------------------------------------==
      IF(MODE.EQ.0) THEN
        BETOT=ETOT
        BEKIN=EKIN
        BEPSEU=EPSEU
        BENL=ENL
        BEHT=EHT
        BEHEP=EHEP
        BEHEE=EHEE
        BEHII=EHII
        BEXC=EXC
        BVXC=VXC
        BESELF=ESELF
        BESR=ESR
        BEGC=EGC
        BECNSTR=ECNSTR
        BERESTR=ERESTR
        BCSUMG=CSUMG
        BCSUMR=CSUMR
        IF(TLSE .AND. TCAS22) THEN
          CALL DCOPY(21,ETOT_A,1,BCAS,1)
          CALL DCOPY(4,ETOT_B,1,BBC,1)
        ENDIF
      ELSE
        ETOT=BETOT
        EKIN=BEKIN
        EPSEU=BEPSEU
        ENL=BENL
        EHT=BEHT
        EHEP=BEHEP
        EHEE=BEHEE
        EHII=BEHII
        EXC=BEXC
        VXC=BVXC
        ESELF=BESELF
        ESR=BESR
        EGC=BEGC
        ECNSTR=BECNSTR
        ERESTR=BERESTR
        CSUMG=BCSUMG
        CSUMR=BCSUMR
        IF(TLSE .AND. TCAS22) THEN
          CALL DCOPY(21,BCAS,1,ETOT_A,1)
          CALL DCOPY(4,BBC,1,ETOT_B,1)
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_PCGRAD(LPCGRAD,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INCLUDE 'system.h'        !NAX NSX
      INCLUDE 'mm_input.inc'    !qmmm
      INCLUDE 'mm_dim.inc'
      INTEGER   LPCGRAD,NSTATE
      CHARACTER TAG*30
      LOGICAL OLDSTATUS
C     ==--------------------------------------------------------------==
      if(qmmm) call mm_dim(mm_go_mm,oldstatus)
      CALL GIVE_SCR_LINESR(LPCGRAD,TAG,NSTATE)
      LPCGRAD=LPCGRAD+3*NAX*NSX
      if(qmmm) call mm_dim(mm_revert,oldstatus)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_LINESR(LLINESR,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'mm_input.inc'    !qmmm
C     Arguments
      INTEGER   LFORCEDR,LLINESR,LRHOOFR,NSTATE
      CHARACTER TAG*30
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_XETOT(LLINESR,TAG,NSTATE)
      CALL GIVE_SCR_FORCEDR(LFORCEDR,TAG,NSTATE,.TRUE.,.FALSE.)
      CALL GIVE_SCR_RHOOFR(LRHOOFR,TAG)
      LLINESR=MAX(LLINESR,LFORCEDR,LRHOOFR)
      IF(qmmm)THEN
        LLINESR=MAX(LLINESR,KR1*KR2S*KR3S)
        LLINESR=MAX(LLINESR,NAX*NSX*3)
      ENDIF

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_XETOT(LXETOT,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NONORT
C     Arguments
      INTEGER   LXETOT,NSTATE
      CHARACTER TAG*30
C     Variables
      INTEGER   LORTHO,LRNLSM,LRSCPOT
C     ==--------------------------------------------------------------==
      IF(NONORT.AND.(.NOT.QUENCHB)) THEN
        LORTHO=0
        LRNLSM=0
        LRSCPOT=0
      ELSE
        CALL GIVE_SCR_ORTHO(LORTHO,TAG,NSTATE)
        CALL GIVE_SCR_RNLSM(LRNLSM,TAG,NSTATE,.FALSE.)
        CALL GIVE_SCR_RSCPOT(LRSCPOT,TAG,.FALSE.)
      ENDIF
      LXETOT=MAX(LORTHO,LRNLSM,LRSCPOT)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
