c     ==================================================================
      subroutine pcgrad_p(c0,c1,c12,vpp,
     &    psi,scr,lscr,nstate,reset_cg,z11)
c     ==--------------------------------------------------------------==
c     ==  preconditioned conjugate gradient optimization              ==
c     ==                                                              ==
c     ==  INPUT: C12:          the force  d E / d Psi_1               ==
c     ==         C0,vpp,z11:   the usual ingredients                  ==
c     ==                                                              ==
c     ==  NB: The gradient does NOT need to be orthogonalized wrt the ==
C     ==      ground state.                                           ==
C     ==      This routine also checks the CONVERGENCE CONDITION and  ==
C     ==      computes the gemax and cnorm values.                    ==
c     ==                                                              ==
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'response_p.inc'
      include 'norm.inc'
      include 'geq0.inc'

c     arguments
      integer    lscr,nstate
      complex*16 c0(*),c12(*),c1(*),scr(*),psi(*)
      real*8     vpp(*),z11(*)
      logical    reset_cg

c     variables & functions
C     conj grad variables:
C     dir or d means direction (numerical recipes: the "h" or "p")
C     g means gradient  (numerical recipes: the "g" or "r")
C     d_old is the (conjugate) direction of the _last_ step.
C     A is "the matrix", i.e. (H0 - eps_ij)
      real*8     g(*),dir(*),A__dir(*)
      pointer    (ip_dir,dir),(ip_A__dir,A__dir),
     &     (ip_g,g)
      save       ip_g,ip_dir,ip_A__dir

      real*8     alpha, beta, avg_alpha
      real*8     dir_grad,dir__A__dir, g__PREC__A__dir,
     &     g_PREC_g,g_PREC_g__old,g_PREC_g_old
      COMMON     /static_vars/ avg_alpha,beta,
     &     g_PREC_g,g_PREC_g_old,
     &     dir_grad,dir__A__dir
C     Attention, __old means that the _number_ is from the last
C     cycle, whereas _old refers to the _vector_ from last cycle.


C     technical variables:
      character  tag*30
      real*8     ENERGY
      integer    isub,is,ig,istep,ifirst
      save       istep,ifirst
      data       ifirst /0/
      DIMENSION S_STAR(NSTATE,NSTATE)

c     ==--  Initialization  ------------------------------------------==
      call tiset('  pcgrad_p',isub)
      call test_scr('pcgrad_p',tag,lscr,2*ngwk*nstate)
      if (reset_cg) then
         istep=1
         reset_cg = .false.
         avg_alpha = 0.d0
      endif
      if (ifirst.ne.2604) then
         ifirst = 2604
         istep=1
         call memory(ip_dir,2*ngw*nstate,'old_d')
         call azzero(   dir,2*ngw*nstate)
         call memory(ip_A__dir,2*ngw*nstate,'A__old_d')
         call azzero(   A__dir,2*ngw*nstate)
         if (tpolak_p) then
            call memory(ip_g,2*ngw*nstate,'grad_old')
            call azzero(   g,2*ngw*nstate)
         endif
      endif



c     ==-  Preconditioning ------------------------------------------==
C   DMB
      IF (TSIMPLE_MODEL) THEN
         call simple_ortho_p(nstate,c0,c12,S_star)
      ELSE
         call ortho_p(nstate,c0,c12)
      ENDIF
C   DMB
      if (prec_p) then
         call precon_p(c12,scr,vpp,nstate,z11,1)
C   DMB
         IF (TSIMPLE_MODEL) THEN
            call simple_ortho_p(nstate,c0,scr,S_star)
         ELSE
            call ortho_p(nstate,c0,scr)
      ENDIF
C   DMB
      else
         call DCOPY(2*ngw*nstate,c12,1, scr,1)
      endif

      call csize(c12,nstate,gemax,cnorm)

      if (geq0) then
         call zclean(scr,nstate,ngw)
         call zclean(c12,nstate,ngw)
      endif
C     C12 is the original gradient, SCR the preconditioned one.


C     Some scalar products, needed for the CG factors alpha and beta:
      g_PREC_g_old  = 0d0
      g_PREC_g__old = g_PREC_g
      g_PREC_g      = ENERGY(scr,c12,nstate)
      if (tpolak_p) then
         if (istep .gt. 1) then
            g_PREC_g_old = ENERGY(g,c12,nstate)
         else
            g_PREC_g_old = 0.d0
         endif
         call DCOPY(2*ngw*nstate,scr,1,g,1)
      endif
#ifdef PARALLEL
      call glosum(2,g_PREC_g)   ! g_PREC_g and g_PREC_g_old
#endif




C     calculation of the new conj direction:
C
C     dir_new = precon * gradient  +  beta * dir_old
C
C     with beta = - (gradient*precon*A*dir_old) / (dir_old*A*dir_old)
C     
C     or alternatively,
C     beta = (gradient*precon*gradient)_present / ()_last_cycle
C
C     or with the POLAK formula
C     beta = ( (gradient_present - gradiend_last_cycle) 
C     \                      * precon* gradient_present)
C     \        / (gradient*precon*gradient)_last_cycle
C
      if (istep .eq. 1) then
         call azzero(dir,2*ngw*nstate)
         beta = 0.d0
         call DCOPY(2*ngw*nstate, scr,1, dir,1)
         avg_alpha = 1.0d0 * CG_FACTOR
      else                      ! then istep .ge. 2
         if (tpolak_p) then
            if (g_PREC_g__old .gt. 1d-20) 
     &         beta = (g_PREC_g - g_PREC_g_old) / g_PREC_g__old
         else
            if (g_PREC_g__old .gt. 1d-20) 
     &         beta = g_PREC_g / g_PREC_g__old
         endif
         if (beta .gt. 0.5d0)  beta = 0.0d0
         call DSCAL(2*nstate*ngw,beta,dir,1)
         call DAXPY(2*ngw*nstate, 1.d0, scr,1, dir,1)
C     THIS (dir) is now the NEW CONJUGATE DIRECTION
      endif



      if (istep.gt.1 .and. istep .le. CG_ANALYTIC) then
C     Calculation of the step length (alpha):
C     alpha = - (dir * gradient) / (dir * A * dir)
         call A_p(dir,A__dir,c0,nstate,psi,scr,lscr,z11)
         dir__A__dir = ENERGY(dir,A__dir,nstate)
         dir_grad    = ENERGY(dir,c12,     nstate)
#ifdef PARALLEL
         call glosum(2,dir_grad) ! dir_grad and dir__A__dir
#endif
         alpha       = - dir_grad / dir__A__dir
         alpha = alpha * CG_FACTOR
C     alpha is now the COMPUTED STEP LENGTH.
         avg_alpha = (avg_alpha * (istep-1) + alpha)/(istep*1.d0)
      else
         if (abs(avg_alpha) .LT. 0.01d0) avg_alpha = 0.01d0
         alpha = avg_alpha
      endif

C     APPLY THE CONJUGATE GRADIENT STEP:
      call DAXPY(2*ngw*nstate, alpha, dir, 1, c1, 1)
      call ortho_p(nstate,c0,c1)

      istep = istep + 1
      call tihalt('  pcgrad_p',isub)
      return
      end
c     ==================================================================


c     ==================================================================
      subroutine precon_p(c2,c2dest,vpp,nstate,z11,direction)
c     APPLIES  (H0 - E_ks) ^ -1  to the argument c2.
c     vpp must contain    G^2 + Coulomb-terms(G)
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'response_p.inc'
c     arguments
      integer    nstate,direction
      complex*16 c2(ngw,*),c2dest(ngw,*)
      real*8     vpp(ngw),z11(nstate,nstate)
c     variables
      integer    ig,is
      real*8     z11_avg,arg,hthrs2
      complex*16 c2value

      real*8     vpp2(ngw,*),vpp2value
      pointer    (ip_vpp2,vpp2)
      integer    vpp2_allocated
      data       vpp2_allocated /0/
      save       vpp2_allocated,ip_vpp2
c     ==--------------------------------------------------------------==
C     New technique: The new array vpp2 will now contain the ready-to-use 
C     preconditioner. It is computed at the first time this routine is called.
C     For the simple preconditioners (one for all states), it has dimension
C     NGW, for the state-dependent preconditioners NGW*NSTATE.

C     The routine can now also apply the INVERSE preconditioner. To do
C     so, -1 has to be given as argument (direction). A normal call would
C     use +1.


C     Simple preconditioners (state-invariant)
      IF (direction .eq. 1) THEN
         IF      (PRECONDITIONER_P .EQ. 1
     &        .OR. PRECONDITIONER_P .EQ. 2) THEN
            if (vpp2_allocated .ne. 2604) then ! the it is not yet allocated.
               if (vpp2_allocated .ne. 0)
     &              call stopgm('pcgrad/precon',
     &              'INTERNAL ERROR: PRECONDITIONER TYPE CHANGED')
               vpp2_allocated = 2604
               call memory(ip_vpp2,ngw,'pcgrad/vpp2')
               call azzero(vpp2,ngw)
               
               z11_avg = 0d0
               do is=1,nstate
                  z11_avg = z11_avg - z11(is,is)
               enddo
               z11_avg = z11_avg / DBLE(nstate)
               hthrs2  = hthrs_p*hthrs_p
               
               IF (PRECONDITIONER_P .EQ. 1) THEN
!$OMP parallel do private(ig,arg)
                  do ig=1,ngw
                     arg = vpp(ig) + z11_avg
                     arg = 1.d0 / DSQRT( arg*arg + hthrs2 )
                     if (abs(arg) .le. 1d-10) arg = 1d-10
C     This is for safety: at a given point, we need the inverse
C     of vpp2!
                     vpp2(ig,1) = arg
                  enddo
               ELSE IF (PRECONDITIONER_P .EQ. 2) THEN
!$OMP parallel do private(ig,arg)
                  do ig=1,ngw
                     arg = vpp(ig) + z11_avg
                     arg = arg / ( arg*arg + hthrs2 )
                     if (abs(arg) .le. 1d-10) arg = 1d-10
                     vpp2(ig,1) = arg
                  enddo
               ELSE
                  call stopgm('pcgrad_p/precon',
     &                 'INTERNAL ERROR: preconditioner index')
               ENDIF
            endif               ! allocation+calculation of preconditioner

                                ! now, use the preconditioner:
!$OMP parallel do private(ig,is,vpp2value,c2value)
            do ig=1,ngw
               vpp2value     = vpp2(ig,1)
               do is=1,nstate
                  c2value       = c2(ig,is)
                  c2dest(ig,is) = c2value * vpp2value
               enddo
            enddo
            

C     STATE-DEPENDENT preconditioners:
         ELSEIF      (PRECONDITIONER_P .EQ. 3
     &           .OR. PRECONDITIONER_P .EQ. 4) THEN
            if (vpp2_allocated .ne. 2602) then ! the it is not yet allocated.
               if (vpp2_allocated .ne. 0)
     &              call stopgm('pcgrad/precon',
     &              'INTERNAL ERROR: PRECONDITIONER TYPE CHANGED')
               vpp2_allocated = 2602
               call memory(ip_vpp2,ngw*nstate,'pcgrad/vpp2')
               call azzero(vpp2,ngw*nstate)
               
               hthrs2  = hthrs_p*hthrs_p
               IF (PRECONDITIONER_P .EQ. 3) THEN
!$OMP parallel do private(is,ig,arg) 
                  do is=1,nstate
                     do ig=1,ngw
                        arg = vpp(ig)-z11(is,is)
                        arg = arg * arg + hthrs2
                        arg = 1.d0 / dsqrt(arg)
                        if (abs(arg) .le. 1d-10) arg = 1d-10
                        vpp2(ig,is) = arg
                     enddo
                  enddo
               ELSEIF (PRECONDITIONER_P .EQ. 4) THEN
!$OMP parallel do private(is,ig,arg) 
                  do is=1,nstate
                     do ig=1,ngw
                        arg = vpp(ig)-z11(is,is)
                        arg = arg / (arg * arg + hthrs2)
                        if (abs(arg) .le. 1d-10) arg = 1d-10
                        vpp2(ig,is) = arg
                     enddo
                  enddo
               ELSE
                  call stopgm('pcgrad_p/precon',
     &                 'INTERNAL ERROR: preconditioner index')
               ENDIF
            endif               ! allocation+calculation of preconditioner


                                ! now, use the preconditioner:
!$OMP parallel do private(is,ig,c2value) 
            do is=1,nstate
               do ig=1,ngw
                  c2value       = c2(ig,is)
                  c2dest(ig,is) = c2value * vpp2(ig,is)
               enddo
            enddo
         ELSE
            call stopgm('PCGRAD_P','UNKNOWN PRECONDITIONER TYPE')
         ENDIF


      ELSEIF (direction .eq. -1) THEN
         if (vpp2_allocated .eq. 2604
     &        .OR. vpp2_allocated .eq. 2602) then
            IF      (PRECONDITIONER_P .EQ. 1
     &           .OR. PRECONDITIONER_P .EQ. 2) THEN
!$OMP parallel do private(is,ig,c2value)
               do is=1,nstate
                  do ig=1,ngw
                     c2value       = c2(ig,is)
                     c2dest(ig,is) = c2value / vpp2(ig,1)
                  enddo
               enddo
            ELSEIF (PRECONDITIONER_P .EQ. 3
     &              .OR. PRECONDITIONER_P .EQ. 4) THEN
!$OMP parallel do private(is,ig,c2value)
               do is=1,nstate
                  do ig=1,ngw
                     c2value       = c2(ig,is)
                     c2dest(ig,is) = c2value / vpp2(ig,is)
                  enddo
               enddo
            ELSE
               call stopgm('PCGRAD_P','UNKNOWN PRECONDITIONER TYPE')
            ENDIF
         else
            call stopgm('pcgrad_p',
     &           'INTERNAL ERROR:inverse preconditioner '//
     &           'called too early')
         endif
      ELSE
         call stopgm('pcgrad_p','INTERNAL PROBLEM: old format')
      ENDIF


      return
      end
c     ==================================================================

c     ==================================================================
      SUBROUTINE A_p(arg,result,c0,nstate,psi,scr,lscr,z11)
c     ==--------------------------------------------------------------==
c     The energy functional is of the form
c     E(c1) = <c1| F0 |c1>  +  E_nonloc[c1]. Here we calculate the
c     the bilinear part of the functional ( F0 |c1> ):
c     (1-P_o)  ( h0  |arg_a>  -  eps_lagr_ba  |arg_b> )
c     where arg (=argument) is the vector (of wavefunctions)
c     on which the operator is applied. The resulting vector
c     is then stored in result.
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'ions.inc'
      include 'nlps.inc'
      include 'elct.inc'
      include 'response_p.inc'
      integer     nstate,lscr
      complex*16  arg(ngw,nstate),result(ngw,nstate),c0(ngw,nstate),
     &    psi(*)
      real*8      scr(*),auxc(*),ddia(nax)
      pointer     (ip_ddia,ddia),(ip_auxc,auxc)
      real*8      z11(nstate,nstate)
      integer     i,j,il_ddia,il_auxc,il_scr
      character   tag*30
c     ==--------------------------------------------------------------==
c     scratch business:
      CALL GIVE_SCR_RNLSM(il_scr,TAG,NSTATE,.false.)
      CALL GIVE_SCR_FNONLOC(IL_AUXC,IL_DDIA,NSTATE)
#ifdef _BGL 
      IL_AUXC=IL_AUXC+MOD(IL_AUXC,2)
#endif
      il_scr = max(il_scr,IL_AUXC+IL_DDIA,2*nnr1)
C     2*nnr1 is for the vpsi routine. ('PSIAB')
      call test_scr('pcgrad_p/Ap',tag,lscr,il_scr)

      call azzero(scr,il_scr)
      CALL GET_ADDR(IP_AUXC,SCR(1))
      CALL GET_ADDR(IP_DDIA,SCR(1+IL_AUXC))

c     ==--------------------------------------------------------------==
c     H0  |arg_a_1>:
      call rnlsm(arg,nstate,psi,scr,lscr,1,1,.false.)
      call vpsi(arg,result,f,vofrho0,psi,scr,nstate,1,1)
      call fnonloc(result,f,nstate,auxc,ddia,1,1)

c     - epsilon |arg>:
      if (((TNMR.OR.TEPR).and.TLOCALIZE).OR.TINTERACTION) then
        call DSYMM ( 'R','U', 2*ngw, nstate, 1.d0,
     &      z11, nstate, arg, 2*ngw, 1.d0, result, 2*ngw)
      else
        do i=1,nstate
          CALL DAXPY(2*ngw, z11(i,i), arg(1,i),1, result(1,i),1)
        enddo
      endif

      return
      end
c     ==================================================================
