c     ==================================================================
      subroutine ortho_p(nstate,c0,c1)
      implicit none
      include 'system.h'
      include 'elct.inc'
      include 'cppt.inc'
      include 'response_p.inc'
      complex*16  c0(ngwk,*),c1(ngwk,*)
      real*8      overlap(*)
      integer     isub,nstate,ifirst
      pointer     (ip_overlap,overlap)
      save        ifirst,ip_overlap
      data        ifirst /0/
      call tiset('ortho_p    ',isub)

      if (ifirst .EQ. 0) then
        ifirst = 1
        call memory(ip_overlap,nstate*nstate,'ovl/ortho_p')
        call azzero(   overlap,nstate*nstate)
      endif

      call ovlap(nstate,overlap,c0,c1)

#ifdef PARALLEL
      call glosum(nstate*nstate,overlap)
#endif
      call DGEMM('N','N',2*ngw,nstate,nstate,-1.d0,c0,
     &    2*ngw,overlap,nstate,+1.d0,c1,2*ngw)


      call tihalt('ortho_p    ',isub)
      return
      end
c     ==================================================================
      subroutine restrain_ngw_zero(cx,ngw_zero,ngw,nstate)
      implicit none
      integer ngw,ngw_zero,nstate,is,ig
      complex*16 cx(ngw,nstate)
      if (ngw_zero .lt. ngw) then
!$OMP parallel do private(is,ig)
        do is=1,nstate
          do ig=ngw_zero+1,ngw
            cx(ig,is) = DCMPLX(0d0,0d0)
          enddo
        enddo
      endif
      return
      end
c     ==================================================================
c     ==================================================================
      subroutine lag_mult(c0,c1,psi,scr,lscr,rhoe,z11,nstate)
c     ==--------------------------------------------------------------==
C     Calculates <psi_i| H^0 |psi_j> and diagonalizes it, such that
C     the |psi_i> array contains the eigenstates of H^0.
C     The diagonalization is NOT done if TNMR and TLOCALIZE are set.
C     C1 is overwritten.
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'ions.inc'
      include 'coor.inc'
      include 'spin.inc'
      include 'elct.inc'
      include 'pslo.inc'
      include 'geq0.inc'
      include 'response_p.inc'  ! TLOCALIZE
      integer    nstate,lscr
      complex*16 c0(ngw,nstate),c1(ngw,nstate)
      real*8     z11(nstate,nstate)
c     ==--------------------------------------------------------------==
c      dimension  f(*)
      complex*16 psi(*)
      real*8     eigv(*),scr(*),rhoe(*)
      pointer    (ip_eigv,eigv)
      integer    msglen,info,is
c     ==--------------------------------------------------------------==
      call memory(ip_eigv,nstate,'eigv')

c     calculate the potential and apply it to c0
      call forces(c0,c1,tau0,fion,rhoe,psi,scr,lscr,
     &                  nstate,1,.false.,.false.)

c     calculate the hamiltonian matrix <c0|H|c0> = <c0|c1>:
      call ovlap(nstate,z11,c0,c1)
#ifdef PARALLEL
      call glosum(nstate*nstate,z11)
#endif


c     diagonalize the hamiltonian matrix <c0|H|c0> = z11.
c     IF TNMR, then we need localized (instead of canonical) orbitals.
      if (((TNMR.OR.TEPR).AND.TLOCALIZE).or. TINTERACTION) then
        call DSCAL(nstate*nstate,-1d0,z11,1)
      else
        if(parent) then
          if(tlsd) then
            call DSYEV('V','U',nsup,z11(1,1),nstate,eigv(1),
     *        scr,lscr,info)
            call DSYEV('V','U',nsdown,z11(nsup+1,nsup+1),
     *        nstate,eigv(nsup+1),scr,lscr,info)
          else
            call DSYEV('V','U',nstate,z11,nstate,eigv,scr,lscr,info)
          endif
          if(info.ne.0) call stopgm('lag_mult','dsyev')
        endif
#ifdef PARALLEL
        msglen = nstate*nstate * 8
        call my_bcast(z11,msglen,source,allgrp)
        msglen = nstate * 8
        call my_bcast(eigv,msglen,source,allgrp)
#endif
c     rotate c0 in canonical form
        call rotate(1.d0,c0,0.d0,c1,z11,nstate,2*ngw,tlsd,nsup,nsdown)
        call DCOPY(2*nstate*ngw,c1,1,c0,1)
        call azzero(z11,nstate*nstate)
!$OMP parallel do private(is)
        do is=1,nstate
          z11(is,is)=-eigv(is)
        enddo
      endif

      call freem(ip_eigv)
c     ==--------------------------------------------------------------==
      return
      end
c     ==================================================================
      REAL*8 FUNCTION ENERGY(A,B,nstate)
C     INPUT:  A,B: Wavefunction arrays
C     OUTPUT:  = <A|B>
c     NB: GLOSUM must be done elsewhere!!
      implicit   none
      include   'system.h'
      include 'response_p.inc'

      integer   isub,is,nstate,ngw1
      real*8    DOTP,A(2*ngw,*),B(2*ngw,*),result
      EXTERNAL   DOTP
      call tiset('ENERGY       ',isub)
      ngw1=ngw
      if (cutoff_restr) ngw1=ngw_zero
      result = 0d0
      do is=1,nstate
        result = result + DOTP(ngw1,A(1,is),B(1,is))
      enddo
      call tihalt('ENERGY       ',isub)
      energy = result
      return
      end
c     ==================================================================
