C     ==================================================================
      SUBROUTINE PI_CNTL
C     ==--------------------------------------------------------------==
C     ==  THIS ROUTINE READS THE SECTION &PIMD &END ON INPUT UNIT     ==
C     ==--------------------------------------------------------------==
C     ==  THE INPUT HAS TO BE IN THE FOLLOWING FORMAT                 ==
C     ==     &PIMD                                                    ==
C     ==        Options                                               ==
C     ==     &END                                                     ==
C     ==--------------------------------------------------------------==
C     ==  LIST OF OPTIONS                                             ==
C     ==                                                              ==
C     ==    TROTTER DIMENSION                                         ==
C     ==      np_total                                                ==
C     ==    CENTROID DYNAMICS                                         ==
C     ==    CLASSICAL TEST                                            == 
C     ==    STAGING                                                   ==
C     ==      facstage                                                ==
C     ==    NORMAL MODES                                              ==
C     ==      facstage                                                ==
C     ==    DEBROGLIE [CENTROID]                                      ==
C     ==      tempb                                                   ==
C     ==    FACMASS                                                   ==
C     ==      wmass                                                   ==
C     ==    INITIALIZATION                                            ==
C     ==    READ REPLICAS                                             ==
C     ==       filename                                               ==
C     ==    GENERATE REPLICAS                                         ==
C     ==    PROCESSOR GROUPS                                          ==
C     ==       pc_groups                                              ==
C     ==    OUTPUT [ALL,GROUPS,PARENT]                                ==
C     ==    PRINT LEVEL                                               ==
C     ==       levprt                                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'pimd.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'ropt.inc'
      INCLUDE 'store.inc'
C     Variables
      CHARACTER LINE*80
      INTEGER   IUNIT,IERR,INSCAN,NFIX,IA,I1,I2,MSGLEN
C     ==--------------------------------------------------------------==
C     ==  DEFAULTS                                                    ==
C     ==--------------------------------------------------------------==
      IF(.NOT.PARENT) GOTO 9999
      IUNIT = 5
      RREP=.FALSE.
      REPREAD=.FALSE.
      TREAD_CENT = .TRUE.
      TSTAGE=.FALSE.
      TPINM=.FALSE.
      TINIT=.FALSE.
      TCENTRO=.FALSE.
      TESTPI=.FALSE.
      NP_TOTAL = 1
      TEMPB = 500.D0
      WMASS = 1.D0
      FACSTAGE=1.D0
      PC_GROUPS=0
      REPFNAME=' '
      LEVPRT=0
      LOUTFN=0
C     ==--------------------------------------------------------------==
      IERR=INSCAN(IUNIT,'&PIMD')
      IF(IERR.NE.0) GOTO 100
C     ==--------------------------------------------------------------==
   10 CONTINUE
        READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
        IF(INDEX(LINE,'&END').NE.0) GOTO 100
        IF(INDEX(LINE,'TROT').NE.0 .AND. INDEX(LINE,'DIMEN').NE.0) THEN
C..Treat the nuclear coordinates in path integral representation
C..with Trotter dimension NP_TOTAL
          READ(5,ERR=99,END=99,FMT=*) NP_TOTAL
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'CENTRO').NE.0.AND.INDEX(LINE,'DYNAM').NE.0) THEN
C..do centroid path integral MD and
CDM C..do centroid path integral MD (only together with normal modes) and
C..do not thermostat centroid mode, i.e., first Trotter slice
          TCENTRO=.TRUE.
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'CLASSI').NE.0.AND.INDEX(LINE,'TEST').NE.0) THEN
            TESTPI=.TRUE.
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'STAGING').NE.0) THEN
C..use the staging representation for the path integrals
          TSTAGE=.TRUE.
CDM1 delete following two lines:
C..the ficticious staging mass is (FACSTAGE*WMASS)-times heavier
C..than the corresponding real physical mass of the same species
C
C..the fictitious non-staging masses are FACSTAGE time lighter than
C..the staging mass (which has the physical mass as fictitious mass)
CDM2
          READ(5,ERR=99,END=99,FMT=*) FACSTAGE
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'NORMAL').NE.0) THEN
C..use the normal mode representation for the path integrals
          TPINM=.TRUE.
CDM1 delete following two lines:
C..the ficticious normal mode mass is (FACSTAGE*WMASS)-times heavier
C..than the corresponding real physical mass of the same species
C
C..the fictitious non-centroid masses are FACSTAGE time lighter than
C..the centroid mass (which has the physical mass as fictitious mass)
CDM2
          READ(5,ERR=99,END=99,FMT=*) FACSTAGE
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'DEBROG').NE.0) THEN
CDM1 C..use TEMPB as the reference temperature (in Kelvin) to generate the
CDM C..De Broglie displaced NP-1 other particles
C..create free particle paths using a Gaussian Levy walk
C..at temperature TEMPB (in Kelvin) according to the masses of the species
C..starting from the first bead from the classical configuration
C..that is obtained from input 
CDM2
          READ(5,ERR=99,END=99,FMT=*) TEMPB
CDM1
          TREAD_CENT = .FALSE. 
CDM2
CDM1 C..Classical coordinates are read in as centroid positions and
CDM C..normal modes are sampled.
C..create free particle paths using a Gaussian normal mode sampling
C..at temperature TEMPB (in Kelvin) according to the masses of the species
C..where the classical configuration yields the centroid positions
CDM2
          IF(INDEX(LINE,'CENTR').NE.0) TREAD_CENT = .TRUE.
          GOTO 10
        ELSE IF
     *     (INDEX(LINE,'FACMASS').NE.0) THEN
CDM1: delete the following 3 lines:
CMASS..ficticious ionic masses are M*P*(\beta *\hbar / WMASS)**2
CMASS..no specification of WMASS -> ficticious ionic masses are set to M
CMASS.. (give WMASS in atomic units)
CDM2
C..ficticious ionic masses are WMASS*M
C..no specification of WMASS -> WMASS=1.0 (choice of Berne et al.)
           READ(5,ERR=99,END=99,FMT=*) WMASS
           GOTO 10
        ELSE IF
     *    (INDEX(LINE,'INITIALIZA').NE.0) THEN
CDM1 C..initialization run 
C..initialization run: supply full quantum start configuration either by
C..(i)  generation -> with RREP=.TRUE.    or by
C..(ii) reading it -> with REPREAD=.TRUE.
C..is default if no restart of coordinates
CDM2
          TINIT=.TRUE.
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'GENERATE').NE.0) THEN
CDM     *    (INDEX(LINE,'REGENERATE').NE.0) THEN
CDM1 C..generate replica coordinates
C..generate replica coordinates as specified by 'DEBROGLIE' keyword
CDM2
          RREP=.TRUE.
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'READ').NE.0.AND.INDEX(LINE,'REPL').NE.0) THEN
CDM1 C..read replica coordinates from external file
C..read replica coordinates from external file to be named 'repfname'  
C..in the loop format "DO IP, DO IS, DO IA ... (K=1,3)" (see 'rread.F')
C..with an empty line before EVERY set of replicas (also the first!)
CDM2
          REPREAD=.TRUE.
          READ(IUNIT,ERR=99,END=99,FMT='(A80)') repfname
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'PROC').NE.0 .AND. INDEX(LINE,'GROUP').NE.0) THEN
C..number of processor goups 
          READ(5,ERR=99,END=99,FMT=*) PC_GROUPS
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'PRIN').NE.0 .AND. INDEX(LINE,'LEVEL').NE.0) THEN
CDM1 C..printing level 
C..printing level: minimal output for LEVPRT < 5 
CDM2
          READ(5,ERR=99,END=99,FMT=*) LEVPRT
          GOTO 10
        ELSE IF
     *    (INDEX(LINE,'OUTPUT').NE.0) THEN
C..output files for each processor, group or only grandparent
          IF(INDEX(LINE,'PARENT').NE.0) LOUTFN=0
          IF(INDEX(LINE,'GROUP').NE.0) LOUTFN=1
          IF(INDEX(LINE,'ALL').NE.0) LOUTFN=2
          GOTO 10
        ELSE 
C..Dummy line
          GOTO 10
        ENDIF
C     ==--------------------------------------------------------------==
   99 CONTINUE
      CALL STOPGM('PI_CNTL',' ERROR READING INPUT FILE')
C     ==--------------------------------------------------------------==
  100 CONTINUE
C     ==--------------------------------------------------------------==
C     ==  Test of options                                             ==
C     ==--------------------------------------------------------------==
CDM1
      IF(NP_TOTAL.GT.MAXNP) THEN
        WRITE(6,'(A,I4,A,A,I4)') 
     *  ' SELECTED TROTTER NUMBER ',NP_TOTAL,' LARGER THAN',
     *  ' CURRENT MAXIMUM VALUE MAXNP = ',MAXNP 
        CALL STOPGM('PI_CNTL',' TROTTER NUMBER LARGER THAN MAXNP ')
      ENDIF
      IF(NOGRP.GT.1) CALL STOPGM("PI_CNTL","NO TASK GROUPS ALLOWED")
      IF(TPRES) THEN
        WRITE(6,*) ' !! NO STRESS CALCULATION WITH PATH INTEGRALS !!'
        TPRES=.FALSE.
      ENDIF
CDM2
      IF(.NOT.TESTPI) THEN
        IF(NP_TOTAL.EQ.1) THEN
          IF(TSTAGE.OR.TPINM) THEN
            WRITE(6,'(A)') ' STAGING OR NORMAL OPTION NEED NP_TOTAL > 1'
            CALL STOPGM('PI_CNTL',' WRONG OPTION ')
          END IF
        END IF
      END IF
      IF(MD) THEN
        RREP=.FALSE.
        REPREAD=.FALSE.
        IF(TEMPW.LE.1.0d-5) CALL STOPGM('PI_CNTL',
     &    'YOU HAVE TO SET TEMPERATURE OR USE A THERMOSTAT')
        IF(TRANE) WRITE(6,*) 
     *    ' RANDOMIZE WAVEFUNCTIONS : OPTION NOT IMPLEMENTED'
        IF(QUENCHB) CALL STOPGM('PI_CNTL',
     *    ' QUENCH TO BO SURFACE : OPTION NOT IMPLEMENTED')
      ELSEIF(WFOPT) THEN
        PC_GROUPS = 1
        IF(.NOT.RESTART) THEN
          TINIT=.TRUE. 
        ENDIF
        IF(TINIT) THEN
          IF(.NOT.(RREP.OR.REPREAD)) RREP=.TRUE.
        ELSE
          RCO=.TRUE.
        ENDIF
      ELSE
        WRITE(6,*) ' ONLY MD OR WAVEFUNCTION OPTIMIZATION IMPLEMENTED'
        CALL STOPGM('PI_CNTL',' WRONG OPTION ')
      ENDIF
      NFIX=0
      DO IA=1,NAT
        IF(LSKCOR(1,IA).EQ.0) NFIX=NFIX+1 
        IF(LSKCOR(2,IA).EQ.0) NFIX=NFIX+1 
        IF(LSKCOR(3,IA).EQ.0) NFIX=NFIX+1 
      ENDDO
      NFIX=NFIX+MCNSTR
      IF(NFIX.GT.0.AND.(.NOT.(TPINM.OR.TSTAGE))) THEN
        WRITE(6,'(A,A)') ' GEOMETRY CONSTRAINTS ONLY TOGETHER WITH ', 
     *         ' STAGING OR NORMAL MODE REPRESENTATION '
        CALL STOPGM('PI_CNTL','CONSTRAINTS')
      ENDIF
CDM1
      IF(TCENTRO) THEN
        IF(.NOT.TPINM) 
     *    WRITE(6,'(/,A,A,/)')
     *  ' WARNING FROM PI_CNTL: ONLY NORMAL MODE PROPAGATOR YIELDS',
     *  ' CANONICAL DENSITY DISTRIBUTION: THINK ABOUT YOUR INPUT !!'
        IF((WMASS.GT.1.D0).OR.(WMASS.LT.1.D0)) THEN
          WRITE(6,'(/,A,/)')
     *  ' WARNING FROM PI_CNTL: WMASS HAS TO BE UNITY FOR CENTROID PIMD'
          WRITE(6,'(/,A,/)')
     *  '                       THIS IS NOW DONE !'
          WMASS=1.D0
        ENDIF
      ELSE
        IF((FACSTAGE.GT.1.D0).OR.(FACSTAGE.LT.1.D0)) THEN
          WRITE(6,'(/,A,/)')
     *  ' !! WARNING FROM PI_CNTL: WHY IS FACSTAGE NOT UNITY ?'
        ENDIF
      ENDIF
CDM2
C     ==--------------------------------------------------------------==
C     ==  WRITE INFO TO OUTPUT                                        ==
C     ==--------------------------------------------------------------==
      WRITE(6,*)
      WRITE(6,'(1X,19(">"),A,19("<"))') ' PATH-INTEGRAL PARAMETERS '
CDM1
      IF(TESTPI) THEN
        NP_TOTAL=1
        TREAD_CENT=.FALSE.
        TSTAGE=.FALSE.
        TPINM=.FALSE.
        WRITE(6,'(A,A)')
     *  ' !!!! THIS IS A TEST WITH NP=1 FOR COMPARISON WITH',
     *  ' CLASSICAL RUNS'  
        WRITE(6,'(A)')
     *  ' !!!! PRIMITIVE PROPAGATOR AND LEVY INITIAL GUESS USED'
      ENDIF
      IF(TCENTRO) THEN
        WRITE(6,'(A)')
     *  ' >>>> ADIABATIC QUASICLASSICAL CENTROID DYNAMICS' 
        IF(TPINM) WRITE(6,'(A)')
     *  ' >>>> YIELDING THE CANONICAL BOLTZMANN DISTRIBUTION'
        IF(TSTAGE) WRITE(6,'(A)')
     *  ' >>>> YIELDING THE WIGNER DISTRIBUTION'
      ENDIF
CDM2
      WRITE(6,'(A,I4)') ' >>>> TROTTER DIMENSION :',NP_TOTAL
      IF(TINIT) WRITE(6,'(A)') ' >>>> INITIALIZATION RUN        '
CDM1
C      IF(RREP) WRITE(6,'(A)') ' >>>> GENERATE REPLICA COORDINATES '
      IF(RREP) THEN
        WRITE(6,'(A)') 
     *  ' >>>> GENERATE QUANTUM FREE PARTICLE REPLICA COORDINATES '
        IF(TREAD_CENT) THEN
          WRITE(6,'(A)')  
     *    ' >>>> USING THE CLASSICAL POSITIONS AS CENTROIDS'
        ELSE
          WRITE(6,'(A)')  
     *    ' >>>> USING THE GAUSSIAN LEVY WALK IN REAL SPACE'
        ENDIF
        WRITE(6,'(A,F12.2,A)') 
     *  ' >>>> AT A TEMPERATURE OF : ',TEMPB,'K'
      ENDIF
CDM2
      CALL XSTRING(REPFNAME,I1,I2)
      IF(REPREAD) WRITE(6,'(A,A)') 
     * ' >>>> READ REPLICA COORDINATES FROM FILE ',REPFNAME(I1:I2)
CDM1
      IF(TSTAGE) THEN
        WRITE(6,'(A)') ' >>>> USE STAGING VARIABLES '
      ELSEIF(TPINM) THEN
        WRITE(6,'(A)') ' >>>> USE NORMAL MODE VARIABLES '
      ELSE
        WRITE(6,'(A)') ' >>>> USE PRIMITIVE VARIABLES '
      ENDIF
CDM2
      IF(TPINM.OR.TSTAGE) THEN
CDM        WRITE(6,'(A,G12.6)') ' >>>> FACMASS : ',WMASS
        WRITE(6,'(A,G12.6)') ' >>>> SCALING FACTOR FACMASS : ',WMASS
CDM        WRITE(6,'(A,G12.6)') ' >>>> FACSTAGE: ',FACSTAGE
        WRITE(6,'(A,G12.6)') ' >>>> MASS DISPARITY FACSTAGE: ',FACSTAGE
      ENDIF
      WRITE(6,'(A,I4)') ' >>>> PRINT LEVEL       :',LEVPRT
      WRITE(6,'(A)') ' >>>>'
C     ==--------------------------------------------------------------==
 9999 CONTINUE
C     ==--------------------------------------------------------------==
C     ==  BROADCAST INFO TO ALL OTHER PROCESSORS                      ==
C     ==--------------------------------------------------------------==
C..PIMD 
      MSGLEN = PI_LOG * 8/IRAT
      CALL MY_BCAST(PILOG,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = PI_REL * 8
      CALL MY_BCAST(PIREL,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = PI_INT * 8/IRAT
      CALL MY_BCAST(PIINT,MSGLEN,SOURCE,ALLGRP)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================


