C     ==================================================================
      SUBROUTINE PM_CNTL
C     ==--------------------------------------------------------------==
C     ==  THIS ROUTINE READS THE SECTION &PATH &END ON INPUT UNIT     ==
C     ==--------------------------------------------------------------==
C     ==  THE INPUT HAS TO BE IN THE FOLLOWING FORMAT                 ==
C     ==     &PATH                                                    ==
C     ==        Options                                               ==
C     ==     &END                                                     ==
C     ==--------------------------------------------------------------==
C     ==  LIST OF OPTIONS                                             ==
C     ==                                                              ==
C     ==    REPLICA NUMBER                                            ==
C     ==      np_total                                                ==
C     ==    CLASSICAL TEST                                            == 
C     ==    INITIALIZATION                                            ==
C     ==    READ REPLICAS                                             ==
C     ==       filename                                               ==
C     ==    GENERATE REPLICAS                                         ==
C     ==    PROCESSOR GROUPS                                          ==
C     ==       pc_groups                                              ==
C     ==    OUTPUT [ALL,GROUPS,PARENT]                                ==
C     ==    PRINT LEVEL                                               ==
C     ==       levprt                                                 ==
C     ==    NLOOP                                                     ==
C     ==      nloop                                                   ==
C     ==    FACTOR                                                    ==
C     ==      factor                                                  ==
C     ==    ALPHA                                                     ==
C     ==      alpha_pmin                                              ==
C     ==    NEQUI                                                     ==
C     ==      nequi                                                   ==
C     ==    NPREVIOUS [LATEST]                                        ==
C     ==       nloopold                                               ==
C     ==    PROJOUT                                                   ==
C     ==    CONVERGENCE                                               ==
C     ==       tolds                                                  ==
C     ==    COLLECTIVE VARIABLES                                      ==
C     ==       ncvsp                                                  ==
C     ==       irest(1) irest(2) ... irest(ncvsp)                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'pimd.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'ropt.inc'
      INCLUDE 'store.inc'
      INCLUDE 'mfep.inc'
C     Variables
      CHARACTER LINE*80
      INTEGER   IUNIT,IERR,INSCAN,NFIX,IA,I1,I2,MSGLEN,I,
     &          IRE,MINIR,MAXIR
      LOGICAL   TCVSP
C     ==--------------------------------------------------------------==
C     ==  DEFAULTS                                                    ==
C     ==--------------------------------------------------------------==
      IF(.NOT.PARENT) GOTO 9999
      IUNIT = 5
      RREP=.FALSE.
      REPREAD=.FALSE.
      TREAD_CENT = .TRUE.
      TSTAGE=.FALSE.
      TPINM=.FALSE.
      TINIT=.FALSE.
      TCENTRO=.FALSE.
      TESTPI=.FALSE.
      NP_TOTAL = 1
      TEMPB = 500.D0
      WMASS = 1.D0
      FACSTAGE=1.D0
      PC_GROUPS=0
      REPFNAME=' '
      LEVPRT=0
      LOUTFN=0
      NLOOP=1
      NLOOPOLD=0
      NEQUI=0
      FACTOR=1.0D0
      ALPHA_PMIN=0.2D0
      PROJOUT=.FALSE.
      RLATE_STRING=.FALSE.
      TOLDS=0.0D0
      TCVSP=.FALSE.
      NCVSP=MRESTR
C     ==--------------------------------------------------------------==
      IERR=INSCAN(IUNIT,'&PATH')
      IF(IERR.NE.0) GOTO 100
C     ==--------------------------------------------------------------==
 10   CONTINUE
      READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
      IF(INDEX(LINE,'&END').NE.0) THEN
        GOTO 100
      ENDIF
C     ==--------------------------------------------------------------==
      IF(INDEX(LINE,'REPL').NE.0 .AND. INDEX(LINE,'NUM').NE.0) THEN
C       Number of replica NP_TOTAL
        READ(5,ERR=99,END=99,FMT=*) NP_TOTAL
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'CLASS').NE.0.AND.INDEX(LINE,'TEST').NE.0) THEN
        TESTPI=.TRUE.
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'INITIALIZA').NE.0) THEN
C       Initialization run: supply full start configuration either by
C       (i)  generation -> with RREP=.TRUE.    or by
C       (ii) reading it -> with REPREAD=.TRUE.
C       is default if no restart of coordinates
        TINIT=.TRUE.
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'GENERATE').NE.0) THEN
C       Generate replica coordinates
        RREP=.TRUE.
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'READ').NE.0.AND.INDEX(LINE,'REPL').NE.0) THEN
C       Read replica coordinates from external file 
C       to be named 'repfname' in the loop format 
C       "DO IP, DO IS, DO IA ... (K=1,3)" (see 'rread.F')
C       with an empty line before EVERY set of replicas (also the first!)
        REPREAD=.TRUE.
        READ(IUNIT,ERR=99,END=99,FMT='(A80)') repfname
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'PROC').NE.0.AND.INDEX(LINE,'GROUP').NE.0) THEN
C       Number of processor goups 
        READ(5,ERR=99,END=99,FMT=*) PC_GROUPS
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'PRIN').NE.0.AND.INDEX(LINE,'LEVEL').NE.0) THEN
C       Printing level: minimal output for LEVPRT < 5 
        READ(5,ERR=99,END=99,FMT=*) LEVPRT
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'OUTPUT').NE.0) THEN
C       Output files for each processor, group or only grandparent
        IF(INDEX(LINE,'PARENT').NE.0) LOUTFN=0
        IF(INDEX(LINE,'GROUP').NE.0) LOUTFN=1
        IF(INDEX(LINE,'ALL').NE.0) LOUTFN=2
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'NLOOP').NE.0) THEN
C       Number of loop for minimum free energy path
        READ(5,ERR=99,END=99,FMT=*) NLOOP
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'FACTOR').NE.0) THEN
C       Factor for path propagation
        READ(5,ERR=99,END=99,FMT=*) FACTOR
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'ALPHA').NE.0) THEN
C       Factor for path smoothing
        READ(5,ERR=99,END=99,FMT=*) ALPHA_PMIN
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'NEQUI').NE.0) THEN
C       Number of equilibration steps to be skipped 
C       for minimum free energy path
        READ(5,ERR=99,END=99,FMT=*) NEQUI
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'NPREVIOUS').NE.0) THEN
C       Loop number of last run read from LATEST_STRING file
C       or specified below
        IF(INDEX(LINE,'LATEST').NE.0) THEN
          RLATE_STRING=.TRUE.
        ELSE
          READ(5,ERR=99,END=99,FMT=*) NLOOPOLD
        ENDIF
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'PROJOUT').NE.0) THEN
C       Forces projected out
        PROJOUT=.TRUE.
        GOTO 10
      ELSE IF
     *       (INDEX(LINE,'CONVERG').NE.0) THEN
C       Convergence criterion on string positions
        READ(5,ERR=99,END=99,FMT=*) TOLDS
        GOTO 10
      ELSE IF(INDEX(LINE,'COLLE').NE.0.AND.
     *        INDEX(LINE,'VAR').NE.0) THEN
C       Confined collective variables space
        READ(5,ERR=99,END=99,FMT=*) NCVSP
        CALL MEMORY(IP_IREST,NCVSP,'IREST')
        READ(5,ERR=99,END=99,FMT=*) (IREST(I),I=1,NCVSP)
        TCVSP=.TRUE.
        GOTO 10
      ELSE 
C       Dummy line
        GOTO 10
      ENDIF
C     ==--------------------------------------------------------------==
 99   CONTINUE
      WRITE(6,*) ' PM_CNTL: ERROR IN READING INPUT FILE'
      CALL STOPGM('PM_CNTL',' ')
C     ==--------------------------------------------------------------==
 100  CONTINUE
C     ==--------------------------------------------------------------==
C     ==  Test of options                                             ==
C     ==--------------------------------------------------------------==
      IF(NP_TOTAL.GT.MAXNP) THEN
        WRITE(6,'(A,I4,A,A,I4)') 
     *       ' SELECTED REPLICA NUMBER ',NP_TOTAL,' LARGER THAN',
     *       ' CURRENT MAXIMUM VALUE MAXNP = ',MAXNP 
        CALL STOPGM('PM_CNTL',' REPLICA NUMBER LARGER THAN MAXNP ')
      ENDIF
      IF(NOGRP.GT.1) CALL STOPGM("PM_CNTL","NO TASK GROUPS ALLOWED")
      IF(GEOPT.OR.MD) THEN
      ELSEIF(WFOPT) THEN
        PC_GROUPS = 1
        IF(.NOT.RESTART) THEN
          TINIT=.TRUE. 
        ENDIF
        IF(TINIT) THEN
          IF(.NOT.(RREP.OR.REPREAD)) RREP=.TRUE.
        ELSE
          RCO=.TRUE.
        ENDIF
      ELSE
        WRITE(6,*) ' ONLY MD, AND GEOMETRY OR WAVEFUNCTION ',
     &      'OPTIMIZATION IMPLEMENTED'
        CALL STOPGM('PM_CNTL',' WRONG OPTION ')
      ENDIF
      NFIX=0
      DO IA=1,NAT
        IF(LSKCOR(1,IA).EQ.0) THEN
          NFIX=NFIX+1 
        ENDIF
        IF(LSKCOR(2,IA).EQ.0) THEN
          NFIX=NFIX+1 
        ENDIF
        IF(LSKCOR(3,IA).EQ.0) THEN
          NFIX=NFIX+1 
        ENDIF
      ENDDO
      NFIX=NFIX+MCNSTR
      IF((NOMORE-NEQUI*NTRAJ).LE.0) THEN
        WRITE(6,'(A,I6,A)') ' REQUESTED MAXSTEP OF ',NOMORE,
     &                      ' IS TOO SMALL'
        CALL STOPGM('PM_CNTL',' WRONG OPTION ')
      ENDIF
      IF(TCVSP) THEN
        IF(NCVSP.LT.1.OR.NCVSP.GT.MRESTR)
     &    CALL STOPGM('PM_CNTL',' WRONG NUMBER OF NCVSP ')
      ELSE
        CALL MEMORY(IP_IREST,NCVSP,'IREST')
        DO I=1,NCVSP
          IREST(I)=I
        ENDDO
      ENDIF
      MINIR=1
      MAXIR=MRESTR
      DO I=1,NCVSP
        IRE=IREST(I)
        MINIR=MIN(IRE,MINIR)
        MAXIR=MAX(IRE,MAXIR)
      ENDDO
      IF(MINIR.LT.1.OR.MAXIR.GT.MRESTR)
     &  CALL STOPGM('PM_CNTL',' WRONG NUMBER OF IREST ')
C     ==--------------------------------------------------------------==
C     ==  WRITE INFO TO OUTPUT                                        ==
C     ==--------------------------------------------------------------==
      WRITE(6,*)
      WRITE(6,'(1X,19(">"),A,15("<"))') ' PATH MINIMISATION PARAMETERS '
      IF(TESTPI) THEN
        NP_TOTAL=1
        WRITE(6,'(1X,4("!"),A)')
     *  ' THIS IS A TEST WITH NP=1 FOR COMPARISON WITH CLASSICAL RUNS'  
        WRITE(6,'(1X,4("!"),A)')
     *  ' PRIMITIVE PROPAGATOR AND LEVY INITIAL GUESS USED'
      ENDIF
      WRITE(6,'(1X,4(">"),A,I4)')
     &     ' NUMBER OF REPLICA:',NP_TOTAL
      IF(TINIT) THEN
        WRITE(6,'(1X,4(">"),A)')
     &       ' INITIALIZATION RUN'
      ENDIF
      IF(RREP) THEN
        WRITE(6,'(1X,4(">"),A)')
     &       ' GENERATE REPLICA COORDINATES '
      ENDIF
      CALL XSTRING(REPFNAME,I1,I2)
      IF(REPREAD) THEN
        WRITE(6,'(1X,4(">"),A)') 
     * ' READ REPLICA COORDINATES FROM FILE:'
        WRITE(6,'(1X,4(">"),A,A,A)') '"',REPFNAME(I1:I2),'"'
      ENDIF
      WRITE(6,'(1X,4(">"),A,I4)')
     &     ' PRINT LEVEL:',LEVPRT
      IF(TOLDS.GT.0.0D0)
     &  WRITE(6,'(1X,4(">"),A,1PE12.6)')
     &       ' CONVERGENCE CRITERIA FOR STRING POSITIONS: ',TOLDS
      IF(NCVSP.LT.MRESTR) THEN
        WRITE(6,'(1X,4(">"),A,I3)')
     &       ' DIMENSION OF CONFINED CV SPACE: ',NCVSP
        WRITE(6,'(1X,4(">"),A)')
     &       '   RESTRAINT NUMBER: '
        WRITE(6,'(1X,4(">"),T10,17I3)') (IREST(I),I=1,NCVSP)
      ENDIF
      WRITE(6,'(1X,4(">"))')
C     ==--------------------------------------------------------------==
 9999 CONTINUE
C     ==--------------------------------------------------------------==
C     ==  BROADCAST INFO TO ALL OTHER PROCESSORS                      ==
C     ==--------------------------------------------------------------==
      MSGLEN = PI_LOG * 8/IRAT
      CALL MY_BCAST(PILOG,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = PI_REL * 8
      CALL MY_BCAST(PIREL,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = PI_INT * 8/IRAT
      CALL MY_BCAST(PIINT,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 4 *8/IRAT
      CALL MY_BCAST(NLOOP,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 2 *8/IRAT
      CALL MY_BCAST(PROJOUT,MSGLEN,SOURCE,ALLGRP)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
