C     ==================================================================
      SUBROUTINE PM_INIT
C     ==--------------------------------------------------------------==
C     ==    INITIALIZE PATH MINIMISATION RUNS                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'pimd.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'adat.inc'
      INCLUDE 'fileopen.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'mfep.inc'
      INCLUDE 'envj.inc'  !TJLIMIT

C     Variables
      REAL*8    RPROC
      INTEGER   MSGLEN,I,NPP,NLOW,NHIGH,NPLOCAL,J,IP,MLOW,MHIGH
#ifdef PARALLEL
      CHARACTER FILEOUT*12,FILEON*12
      INTEGER   IPP,I1,I2,LABEL
#endif
C     ==--------------------------------------------------------------==
      MSGLEN = 1 * 8
      CALL MY_BCAST(ECUT,MSGLEN,SOURCE,ALLGRP)
C     ALLGRP becomes SUPERGROUP
      GRANDPARENT=PARENT
      SUPERGROUP=ALLGRP
      SUPERSOURCE=SOURCE
      NPROC_TOT=NPROC
C     Number of processor groups
      IF(PARENT.AND.PC_GROUPS.EQ.0) THEN
        DO I=1,NPROC
          IF(MOD(NPROC,I).EQ.0) THEN
            NPP=NPROC/I
            IF(MOD(NP_TOTAL,NPP).EQ.0) THEN
              PC_GROUPS=NPP
              GOTO 100
            ENDIF
          ENDIF
        ENDDO
        PC_GROUPS=1
 100    CONTINUE
      ENDIF
      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(PC_GROUPS,MSGLEN,SOURCE,ALLGRP)
      PC_GRP(1)=0
      PCG_POS=1
#ifdef PARALLEL
C     Generate processor groups
      CALL IAZZERO(PGROUP,MAXCPU)
      CALL IAZZERO(NLINK,MAXCPU)
      RPROC=DBLE(NPROC)/PC_GROUPS
      DO I=1,PC_GROUPS
        NLOW=NINT((I-1)*RPROC)
        PC_GRP(I)=NLOW
        NHIGH=NINT(I*RPROC)-1
        IF(ME.GE.NLOW .AND. ME.LE.NHIGH) THEN
          NPROC=NHIGH-NLOW+1
          PARENT=NLOW.EQ.ME
          SOURCE=0
          MEPOS=ME-NLOW
          PCG_POS=I
          DO IP=1,NPROC
            PGROUP(IP)=NLOW+IP-1
            IPP=PGROUP(IP)
            NLINK(IPP)=IP-1
          ENDDO
          LABEL=I
        ENDIF
      ENDDO
      CALL MY_GROUP(NPROC,PGROUP(1),LABEL,ALLGRP,SUPERGROUP)
      CALL MY_SYNC(SUPERGROUP)
      LABEL=PC_GROUPS+1
      CALL MY_GROUP(PC_GROUPS,PC_GRP,LABEL,PARENTGROUP,SUPERGROUP)
      IF(LOUTFN.EQ.0) THEN
        IF(.NOT.GRANDPARENT) THEN
#if defined(__WINNT)
          WRITE(FILEON,'(I3)') PCG_POS
          CALL XSTRING(FILEON,I1,I2)
          FILEOUT='OUTPUT_'//FILEON(I1:I2)
          CLOSE(6)
          OPEN(UNIT=6,FILE=FILEOUT)
#else
          CLOSE(6)
          OPEN(UNIT=6,FILE='/dev/null')
#if defined(__Linux) || defined (ALPHALINUX)
C     AK: we need to do the same with the c-style stdout for printmemsize
          CALL SILENTSTDOUT
#endif
#endif
        ENDIF
      ELSEIF(LOUTFN.EQ.1) THEN
        IF(PARENT.AND..NOT.GRANDPARENT) THEN
          WRITE(FILEON,'(I3)') PCG_POS
          CALL XSTRING(FILEON,I1,I2)
          FILEOUT='OUTPUT_'//FILEON(I1:I2)
          CLOSE(6)
          OPEN(UNIT=6,FILE=FILEOUT)
#if defined(__Linux) || defined (ALPHALINUX)
C     avoid printmemsize messes
          CALL SILENTSTDOUT
#endif
        ELSEIF(.NOT.GRANDPARENT) THEN
#if defined(__WINNT)
          WRITE(FILEON,'(I3)') PCG_POS
          CALL XSTRING(FILEON,I1,I2)
          FILEOUT='OUTPUT_'//FILEON(I1:I2)
          CLOSE(6)
          OPEN(UNIT=6,FILE=FILEOUT)
#else
          CLOSE(6)
          OPEN(UNIT=6,FILE='/dev/null')
#if defined(__Linux) || defined (ALPHALINUX)
C     AK: we need to do the same with the c-style stdout for printmemsize
          CALL SILENTSTDOUT
#endif
#endif
        ENDIF
      ELSEIF(LOUTFN.EQ.2) THEN
        IF(.NOT.GRANDPARENT) THEN
          WRITE(FILEON,'(I3)') ME
          CALL XSTRING(FILEON,I1,I2)
          FILEOUT='OUTPUT_'//FILEON(I1:I2)
          CLOSE(6)
          OPEN(UNIT=6,FILE=FILEOUT)
#if defined(__Linux) || defined (ALPHALINUX)
C     avoid printmemsize messes
          CALL SILENTSTDOUT
#endif
        ENDIF
      ENDIF
#endif
C     Establish connection replica index with pc-group
      RPROC=DBLE(NP_TOTAL)/PC_GROUPS
      DO I=1,PC_GROUPS
        NLOW=NINT((I-1)*RPROC) + 1
        NHIGH=NINT(I*RPROC)
        NPLOCAL=NHIGH-NLOW+1
        DO J=NLOW,NHIGH
          NUM_NP(J)=I
        ENDDO
        IF(PCG_POS.EQ.I) THEN
          NP_LOW=NLOW
          NP_HIGH=NHIGH
          NP_LOCAL=NPLOCAL
        ENDIF
      ENDDO
C     Connectivity of replica
      DO IP=1,NP_TOTAL
        IPP1(IP)=IP+1
        IPM1(IP)=IP-1
      ENDDO
      IPP1(NP_TOTAL)=1
      IPM1(1)=NP_TOTAL
C     Print some info
      IF(GRANDPARENT) THEN
        WRITE(6,'(A,A)') ' >>>>       PC_GROUP',
     *              '         PROCESSORS           REPLICA INDICES'
        RPROC=DBLE(NP_TOTAL)/PC_GROUPS
        DO I=1,PC_GROUPS
          NLOW=NINT((I-1)*RPROC) + 1
          NHIGH=NINT(I*RPROC)
          MLOW=PC_GRP(I)
          IF(I.EQ.PC_GROUPS) THEN
            MHIGH=NPROC_TOT-1
          ELSE
            MHIGH=PC_GRP(I+1)-1
          ENDIF
          WRITE(6,'(A,5X,I4,10X,I4,A,I4,14X,I3,A,I3)') ' >>>>   ',I,
     *                      MLOW,' - ',MHIGH,NLOW,' - ',NHIGH
        ENDDO
        WRITE(6,'(1X,32(">"),32("<"))')
      ENDIF

C     Constraints
c      IF(PCG_POS.NE.1) THEN
c        MCNSTR=0
c      ENDIF

C     Broadcast constraint and/or restraint stuff if necessary
      IF(PARENT) THEN
        IF(MCNSTR.GT.0) THEN
          IF(.NOT.GRANDPARENT) THEN
            CALL MEMORY(IP_NTCNST,6*MCNSTR/IRAT,'NTCNST')
            CALL MEMORY(IP_CNSVAL,MCNSTR,'CNSVAL')
            CALL MEMORY(IP_CNPAR,2*MCNSTR,'CNPAR')
            CALL MEMORY(IP_GRATE,MCNSTR,'GRATE')
            CALL MEMORY(IP_CNSVAL_DEST,MCNSTR,'CNSVAL_DEST')
          ENDIF
          MSGLEN = 6*MCNSTR * 8/IRAT
          CALL MY_BCAST(NTCNST,MSGLEN,SUPERSOURCE,PARENTGROUP)
          MSGLEN = 1*MCNSTR * 8
          CALL MY_BCAST(CNSVAL,MSGLEN,SUPERSOURCE,PARENTGROUP)
          CALL MY_BCAST(GRATE,MSGLEN,SUPERSOURCE,PARENTGROUP)
          CALL MY_BCAST(CNSVAL_DEST,MSGLEN,SUPERSOURCE,PARENTGROUP)
          MSGLEN = 2*MCNSTR * 8
          CALL MY_BCAST(CNPAR,MSGLEN,SUPERSOURCE,PARENTGROUP)
        ENDIF
        IF(MRESTR.NE.0) THEN
          IF(.NOT.GRANDPARENT) THEN
            CALL MEMORY(IP_NTREST,6*MRESTR/IRAT+1,'NTREST')
            CALL MEMORY(IP_RESVAL,MRESTR,'RESVAL')
            CALL MEMORY(IP_RESFOR,MRESTR,'RESFOR')
            CALL MEMORY(IP_RESPAR,2*MRESTR,'RESPAR')
            CALL MEMORY(IP_GSRATE,MRESTR,'GSRATE')
            CALL MEMORY(IP_RESVAL_DEST,MRESTR,'RESVAL_DEST')
            CALL MEMORY(IP_RESPOS,3*MRESTR,'RESPOS') ! cmb-kk
          ENDIF
          MSGLEN=(6*MRESTR/IRAT+1)*8
          CALL MY_BCAST(NTREST,MSGLEN,SUPERSOURCE,PARENTGROUP)
          MSGLEN=MRESTR*8
          CALL MY_BCAST(RESVAL,MSGLEN,SUPERSOURCE,PARENTGROUP)
          CALL MY_BCAST(RESFOR,MSGLEN,SUPERSOURCE,PARENTGROUP)
          CALL MY_BCAST(GSRATE,MSGLEN,SUPERSOURCE,PARENTGROUP)
          CALL MY_BCAST(RESVAL_DEST,MSGLEN,SUPERSOURCE,PARENTGROUP)
          MSGLEN=2*MRESTR*8
          CALL MY_BCAST(RESPAR,MSGLEN,SUPERSOURCE,PARENTGROUP)
          MSGLEN=3*MRESTR*8
          CALL MY_BCAST(RESPOS,MSGLEN,SUPERSOURCE,PARENTGROUP)
        ENDIF
      ENDIF

C     Broadcast maximum time for the job
      IF(PARENT) CALL MY_BCAST(TJLIMIT,8,SUPERSOURCE,PARENTGROUP)

      MSGLEN=1*8/IRAT
      CALL MY_BCAST(NDAT,MSGLEN,SUPERSOURCE,SUPERGROUP)
      CALL MY_BCAST(NDAT1,MSGLEN,SUPERSOURCE,SUPERGROUP)
      CALL MY_BCAST(NDAT2,MSGLEN,SUPERSOURCE,SUPERGROUP)
      CALL MY_BCAST(NDAT3,MSGLEN,SUPERSOURCE,SUPERGROUP)

      IF (NDAT.GT.0) THEN
         MSGLEN=2*MAXDUM*8/IRAT
         CALL MY_BCAST(LISTDA,MSGLEN,SUPERSOURCE,SUPERGROUP)
         MSGLEN=MAXLI*MAXDUM*8/IRAT
         CALL MY_BCAST(LISTD2,MSGLEN,SUPERSOURCE,SUPERGROUP)
      ENDIF

C     Distribute information to all parents
C     FPATH
C     FIXME: FPATH is distributed at the end of control_bcast.F
C            check whether this is still needed at all. AK 2005/04/04
      IF(PARENT) THEN
        CALL SENDCHAR(FPATH,FO_FPATH_MAX,SUPERSOURCE,PARENTGROUP)
      ENDIF

C     List and flags for collective variables 
      IF(PARENT) THEN
        IF(NCVSP.GE.1.AND.NCVSP.LE.MRESTR) THEN
          IF(.NOT.GRANDPARENT) CALL MEMORY(IP_IREST,NCVSP,'IREST')
          MSGLEN=NCVSP*8/IRAT
          CALL MY_BCAST(IREST,MSGLEN,SUPERSOURCE,PARENTGROUP)
        ELSE
          CALL STOPGM('PM_INIT',' WRONG NUMBER OF NCVSP ')
        ENDIF 
        CALL MEMORY(IP_LCVSP,MRESTR,'LCVSP')
        DO I=1,MRESTR
          LCVSP(I)=.FALSE.
        ENDDO
        DO I=1,NCVSP
          LCVSP(IREST(I))=.TRUE.
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
