C     ==================================================================
      SUBROUTINE POSUPI(TAU0,TAUP,VELP)
C     ==--------------------------------------------------------------==
C     ==  UPDATE OF THE POSITIONS FOR VELOCITY VERLET                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'tpar.inc'
      INCLUDE 'ions.inc'
C     Arguments
      REAL*8  TAU0(3,NAX,*),TAUP(3,NAX,*),VELP(3,NAX,*)
C     Variables
      INTEGER IS,IA,IAT
C     ==--------------------------------------------------------------==
#if defined(__VECTOR)
!$OMP parallel do private(IA,IS,IAT)
#else
!$OMP parallel do private(IA,IS,IAT) schedule(static)
#endif
      DO IAT=1,NAT
        IA=IATPT(1,IAT)
        IS=IATPT(2,IAT)
        TAUP(1,IA,IS)=TAU0(1,IA,IS)+DT_IONS*VELP(1,IA,IS)
        TAUP(2,IA,IS)=TAU0(2,IA,IS)+DT_IONS*VELP(2,IA,IS)
        TAUP(3,IA,IS)=TAU0(3,IA,IS)+DT_IONS*VELP(3,IA,IS)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE POSUPIH_ISO(TAU0,TAUP,VELP)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'tpar.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'metr.inc'
C     Arguments
      REAL*8  TAU0(3,NAX,*),TAUP(3,NAX,*),VELP(3,NAX,*)      
C     Variables
      REAL*8  S1,ARG,SE,S2,SC,SINHXBX
      INTEGER IS,IA,I,J,IAT
C     ==--------------------------------------------------------------==
C     ==  UPDATE OF THE POSITIONS CONSTANT PRESSURE                   ==
C     ==--------------------------------------------------------------==
      S1 = DEXP(DT_IONS*VEPS)
      ARG = 0.5D0*DT_IONS*VEPS
      SE = DEXP(ARG)
      S2 = SE*SINHXBX(ARG)
#if defined (__VECTOR)
!$OMP parallel do private(IA,IS,IAT)
#else
!$OMP parallel do private(IA,IS,IAT) schedule(static)
#endif
      DO IAT=1,NAT
        IA=IATPT(1,IAT)
        IS=IATPT(2,IAT)
        TAUP(1,IA,IS)=TAU0(1,IA,IS)*S1+DT_IONS*VELP(1,IA,IS)*S2
        TAUP(2,IA,IS)=TAU0(2,IA,IS)*S1+DT_IONS*VELP(2,IA,IS)*S2
        TAUP(3,IA,IS)=TAU0(3,IA,IS)*S1+DT_IONS*VELP(3,IA,IS)*S2
      ENDDO
C     ==--------------------------------------------------------------==
C     ==  UPDATE OF THE BAROSTAT                                      ==
C     ==--------------------------------------------------------------==
      EPS = EPS + DT_IONS*VEPS
C     ==--------------------------------------------------------------==
C     ==  COMPUTE NEW CELL MATRIX                                     ==
C     ==--------------------------------------------------------------==
      DO J=1,3
        DO I=1,3
          HT(I,J) = HT(I,J)*S1
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE POSUPIHSHOCK(TAU0,TAUP,VELP)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'tpar.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'metr.inc'
C     Arguments
      REAL*8  TAU0(3,NAX,*),TAUP(3,NAX,*),VELP(3,NAX,*)      
C     Variables
      REAL*8  S1,ARG,SE,S2,SINHXBX
      INTEGER IS,IA
C     ==--------------------------------------------------------------==
C     ==  UPDATE OF THE POSITIONS CONSTANT PRESSURE                   ==
C     ==--------------------------------------------------------------==
      S1 = EXP(DT_IONS*VEPS)
      ARG = 0.5D0*DT_IONS*VEPS
      SE = EXP(ARG)
      S2 = SE*SINHXBX(ARG)
!$OMP   parallel do private(IA,IS) schedule(dynamic) 
      DO IS=1,NSP
        DO IA=1,NA(IS)
          TAUP(1,IA,IS)=TAU0(1,IA,IS)*S1+DT_IONS*VELP(1,IA,IS)*S2
          TAUP(2,IA,IS)=TAU0(2,IA,IS)+DT_IONS*VELP(2,IA,IS)
          TAUP(3,IA,IS)=TAU0(3,IA,IS)+DT_IONS*VELP(3,IA,IS)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     ==  COMPUTE NEW CELL MATRIX                                     ==
C     ==--------------------------------------------------------------==
      HT(1,1) = HT(1,1)*S1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE POSUPIH(TAU0,TAUP,VELP)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'tpar.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'metr.inc'
C     Arguments
      REAL*8  TAU0(3,NAX,*),TAUP(3,NAX,*),VELP(3,NAX,*)
C     Variables
      REAL*8  HVD(3),HVEV(3,3),VTMP1(3),RTMP1(3),
     &        HDSCR(3,3),HTMP1(3,3),
     &        DTH,ARG1,ARG2,ARG3,E1,E2,E3,S1,S2,S3,
     &        SINHXBX
      INTEGER I,IS,IA,IERR,J
C     ==--------------------------------------------------------------==
C     == DIAGONALIZE HTVEL MATRIX                                     ==
C     ==--------------------------------------------------------------==
      DO J=1,3
         DO I=1,3
            HDSCR(I,J) = HTVEL(I,J)
         ENDDO
      ENDDO
      CALL JACOBI(3,3,HDSCR,HVD,HVEV,IERR)
C     ==--------------------------------------------------------------==
C     ==  UPDATE OF THE POSITIONS CONSTANT PRESSURE                   ==
C     ==--------------------------------------------------------------==
      DTH = 0.5D0*DT_IONS
      ARG1 = HVD(1)*DTH
      ARG2 = HVD(2)*DTH
      ARG3 = HVD(3)*DTH
      E1 = DEXP(HVD(1)*DT_IONS)
      E2 = DEXP(HVD(2)*DT_IONS)
      E3 = DEXP(HVD(3)*DT_IONS)
      S1 = DEXP(ARG1)*SINHXBX(ARG1)
      S2 = DEXP(ARG2)*SINHXBX(ARG2)
      S3 = DEXP(ARG3)*SINHXBX(ARG3)
      DO IS=1,NSP
        DO IA=1,NA(IS)
          RTMP1(1) = HVEV(1,1)*TAU0(1,IA,IS) + 
     *               HVEV(2,1)*TAU0(2,IA,IS) +
     *               HVEV(3,1)*TAU0(3,IA,IS)
          RTMP1(2) = HVEV(1,2)*TAU0(1,IA,IS) + 
     *               HVEV(2,2)*TAU0(2,IA,IS) +
     *               HVEV(3,2)*TAU0(3,IA,IS)
          RTMP1(3) = HVEV(1,3)*TAU0(1,IA,IS) + 
     *               HVEV(2,3)*TAU0(2,IA,IS) +
     *               HVEV(3,3)*TAU0(3,IA,IS)
          VTMP1(1) = HVEV(1,1)*VELP(1,IA,IS) + 
     *               HVEV(2,1)*VELP(2,IA,IS) +
     *               HVEV(3,1)*VELP(3,IA,IS)
          VTMP1(2) = HVEV(1,2)*VELP(1,IA,IS) + 
     *               HVEV(2,2)*VELP(2,IA,IS) +
     *               HVEV(3,2)*VELP(3,IA,IS)
          VTMP1(3) = HVEV(1,3)*VELP(1,IA,IS) + 
     *               HVEV(2,3)*VELP(2,IA,IS) +
     *               HVEV(3,3)*VELP(3,IA,IS)
C
          RTMP1(1) = RTMP1(1)*E1 + DT_IONS*VTMP1(1)*S1
          RTMP1(2) = RTMP1(2)*E2 + DT_IONS*VTMP1(2)*S2
          RTMP1(3) = RTMP1(3)*E3 + DT_IONS*VTMP1(3)*S3
C
          TAUP(1,IA,IS) = HVEV(1,1)*RTMP1(1) +
     *                    HVEV(1,2)*RTMP1(2) +
     *                    HVEV(1,3)*RTMP1(3)
          TAUP(2,IA,IS) = HVEV(2,1)*RTMP1(1) +
     *                    HVEV(2,2)*RTMP1(2) +
     *                    HVEV(2,3)*RTMP1(3)
          TAUP(3,IA,IS) = HVEV(3,1)*RTMP1(1) +
     *                    HVEV(3,2)*RTMP1(2) +
     *                    HVEV(3,3)*RTMP1(3)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     ==  UPDATE OF THE CELL MATRIX                                   ==
C     ==--------------------------------------------------------------==
      DO J=1,3
        HTMP1(1,J) = 0.D0
        HTMP1(2,J) = 0.D0
        HTMP1(3,J) = 0.D0
      ENDDO
      DO J=1,3
         DO I=1,3
            HTMP1(I,J) = HTMP1(I,J)
     &           + HVEV(1,I)*HT(J,1)
     &           + HVEV(2,I)*HT(J,2)
     &           + HVEV(3,I)*HT(J,3)
        ENDDO
      ENDDO
      DO I=1,3
        HTMP1(1,I) = HTMP1(1,I)*E1
        HTMP1(2,I) = HTMP1(2,I)*E2
        HTMP1(3,I) = HTMP1(3,I)*E3
      ENDDO
      DO J=1,3
        HT(1,J) = 0.D0
        HT(2,J) = 0.D0
        HT(3,J) = 0.D0
      ENDDO
      DO J=1,3
         DO I=1,3
            HT(I,J) = HT(I,J)
     &           + HVEV(J,1)*HTMP1(1,I)
     &           + HVEV(J,2)*HTMP1(2,I)
     &           + HVEV(J,3)*HTMP1(3,I)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION SINHXBX(X)
C     ==--------------------------------------------------------------==
C     ==  COMPUTES SINH(X)/X BY A 10TH ORDER MACLAURIN SERIES USING   ==
C     ==  HORNER'S METHOD (HORNY'S METHOD?)                           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8 X
      REAL*8 SINHXBX
C     Variables
      REAL*8 A2,A4,A6,A8,A10,X2
C     ==--------------------------------------------------------------==
      A2  = 1.D0/6.D0
      A4 = 1.D0/120.D0
      A6  = 1.D0/5040.D0
      A8  = 1.D0/362880.D0
      A10 = 1.D0/39916800.D0
      X2 = X*X
      SINHXBX = ((((A10*X2 + A8)*X2 + A6)*X2 + A4)*X2 + A2)*X2 + 1.D0
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==--------------------------------------------------------------==
C     ==================================================================
      SUBROUTINE POSUPIF(TAU0,TAUP,VELP,ERREAD)
C     ==--------------------------------------------------------------==
C     ==  UPDATE OF THE POSITIONS FOR VELOCITY VERLET                 ==
C     ==  form a previous TRAJECTORY file named TRAJSAVED             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'tpar.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'store.inc'
      INCLUDE 'fileopen.inc'
C     Arguments
      REAL*8  TAU0(3,NAX,*),TAUP(3,NAX,*),VELP(3,NAX,*)
C     Variables
      INTEGER IS,IA,IFIRST,NF,I,IIN,IOUT,IN
      LOGICAL ERREAD
      DATA    IFIRST /1/
      SAVE    IFIRST
      CHARACTER*200 LINE
      CHARACTER*2   CATO
C     ==--------------------------------------------------------------==
      IF (.NOT.PARENT) RETURN
      ERREAD=.FALSE.
      IF (XTIN) THEN
        IF (IFIRST.EQ.1) THEN
          CALL FILEOPEN(64,'TRAJSAVED.xyz',FO_OLD,ERREAD)
          IF (ERREAD) CALL STOPGM('POSUPIF',' TRAJSAVED.xyz NOT FOUND')
          IFIRST=0
          DO IN=1,NSKIP+NSAMPLE-1
             READ(64,'(A200)',ERR=99,END=99) LINE
             READ(64,'(A200)',ERR=99,END=99) LINE
             DO IS=1,NSP
                DO IA=1,NA(IS)
                   READ(64,'(A200)',ERR=99,END=99) LINE
                   IF (INDEX(LINE,'DATA').NE.0) 
     &               READ(64,'(A80)',ERR=99,END=99) LINE
                ENDDO
             ENDDO
          ENDDO
        ELSE
          DO IN=1,NSAMPLE-1
             READ(64,'(A200)',ERR=99,END=99) LINE
             READ(64,'(A200)',ERR=99,END=99) LINE
             DO IS=1,NSP
                DO IA=1,NA(IS)
                   READ(64,'(A200)',ERR=99,END=99) LINE
                   IF (INDEX(LINE,'DATA').NE.0) 
     &               READ(64,'(A80)',ERR=99,END=99) LINE
                ENDDO
             ENDDO
          ENDDO
        ENDIF
        READ(64,'(A200)',ERR=99,END=99) LINE
        READ(64,'(A200)',ERR=99,END=99) LINE
        DO IS=1,NSP
          DO IA=1,NA(IS)
            READ(64,*,ERR=99,END=99) 
     &        CATO,TAUP(1,IA,IS),TAUP(2,IA,IS),TAUP(3,IA,IS)
          ENDDO
        ENDDO
        CALL DSCAL(3*NSP*NAX,FBOHR,TAUP,1)
        CALL AZZERO(VELP,3*NSP*NAX)
      ELSE
        IF (IFIRST.EQ.1) THEN
          CALL FILEOPEN(64,'TRAJSAVED',FO_OLD,ERREAD)
          IF (ERREAD) CALL STOPGM('POSUPIF',' TRAJSAVED NOT FOUND')
          IFIRST=0
          DO IN=1,NSKIP+NSAMPLE-1
             DO IS=1,NSP
                DO IA=1,NA(IS)
                   READ(64,'(A200)',ERR=99,END=99) LINE
                   IF (INDEX(LINE,'DATA').NE.0) 
     &               READ(64,'(A80)',ERR=99,END=99) LINE
                ENDDO
             ENDDO
          ENDDO
        ELSE
          DO IN=1,NSAMPLE-1
             DO IS=1,NSP
                DO IA=1,NA(IS)
                   READ(64,'(A200)',ERR=99,END=99) LINE
                   IF (INDEX(LINE,'DATA').NE.0) 
     &               READ(64,'(A80)',ERR=99,END=99) LINE
                ENDDO
             ENDDO
          ENDDO
        ENDIF
        DO IS=1,NSP
          DO IA=1,NA(IS)
            READ(64,'(A200)',ERR=99,END=99) LINE
            IF (INDEX(LINE,'DATA').NE.0) 
     &        READ(64,'(A80)',ERR=99,END=99) LINE
            IIN=1
            CALL READSI(LINE,IIN,IOUT,NF,ERREAD)
            IF(ERREAD) GOTO 99
            DO I=1,3
               IIN=IOUT+1
               CALL READSR(LINE,IIN,IOUT,TAUP(I,IA,IS),ERREAD)
            ENDDO
            IF(ERREAD) GOTO 99
            DO I=1,3
               IIN=IOUT+1
               CALL READSR(LINE,IIN,IOUT,VELP(I,IA,IS),ERREAD)
            ENDDO
            IF(ERREAD) GOTO 99
          ENDDO
        ENDDO
      ENDIF
      RETURN
   99 CONTINUE
      ERREAD=.TRUE.
      END
C     ==================================================================
