C     ==================================================================
      SUBROUTINE POTMED(RHOE,RCUT,TAU0,ACHRG,ICHRG)
C     ==--------------------------------------------------------------==
C     == COMPUTES ATOMIC AVERAGED ELECTROSTATIC POTENTIAL INSIDE RC   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'qspl.inc'
C     Arguments
      REAL*8    RHOE(NNR1),RCUT,TAU0(3,NAX,NSX),ACHRG(NAT),ICHRG(NAT)
C     Variables
      INTEGER   ISUB,IS,ISH,L,MMAX,IA,IAT,IGRID
      REAL*8    CHRG
C     ==--------------------------------------------------------------==
      CALL TISET('    POTMED',ISUB)
C     ==--------------------------------------------------------------==
C..initialization
C     ==--------------------------------------------------------------==
c..charge integration
      IAT=0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          IAT=IAT+1
          CALL GETPOT(RHOE,TAU0(1,IA,IS),RCUT,CHRG,IGRID)
          ACHRG(IAT)=CHRG
          ICHRG(IAT)=DBLE(IGRID)
        ENDDO
      ENDDO
      CALL GLOSUM(NAT,ACHRG)
      CALL GLOSUM(NAT,ICHRG)
      IAT=0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          IAT=IAT+1
          CHRG=ACHRG(IAT)/ICHRG(IAT)
          ACHRG(IAT)=CHRG
        ENDDO
      ENDDO
      CALL TIHALT('    POTMED',ISUB)
C     ==--------------------------------------------------------------==
C     Deallocation of local variables
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GETPOT(RHOE,TAU,RCUT,CHRG,IGRID)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'qspl.inc'
      INCLUDE 'metr.inc'
C     Arguments
      INTEGER   MMAX
      REAL*8    RHOE(KR1,KR2,*),TAU(3),RCUT,CHRG
C     Variables
      REAL*8    SC1(3),SC2(3),DA1(3,3),XPTAU(3),X1,Y1,Z1,X,Y,Z,
     &          R2,R
      INTEGER   NPTAU(3),MR(3),I,IORGX,IORGY,IORGZ,IGRID,IX,II,I1,J,JJ,
     &          J1,K,KK,K1
C     ==--------------------------------------------------------------==
C..Atom position
      CHRG=0
      X1=TAU(1)
      Y1=TAU(2)
      Z1=TAU(3)
      CALL PBC(X1,Y1,Z1,SC1(1),SC1(2),SC1(3),1,APBC,IBRAV)
      CALL DGEMV('T',3,3,1.D0,HTM1,3,SC1,1,0.D0,SC2,1)
      IF(SC2(1).LT.0.D0) SC2(1)=SC2(1)+1.0D0
      IF(SC2(2).LT.0.D0) SC2(2)=SC2(2)+1.0D0
      IF(SC2(3).LT.0.D0) SC2(3)=SC2(3)+1.0D0
      NPTAU(1)=NINT(SC2(1)*NR1S)+1
      NPTAU(2)=NINT(SC2(2)*NR2S)+1
      NPTAU(3)=NINT(SC2(3)*NR3S)+1
      SC1(1)=SC2(1)-DBLE(NPTAU(1)-1)/DBLE(NR1S)
      SC1(2)=SC2(2)-DBLE(NPTAU(2)-1)/DBLE(NR2S)
      SC1(3)=SC2(3)-DBLE(NPTAU(3)-1)/DBLE(NR3S)
      IF(NPTAU(1).GT.NR1S) NPTAU(1)=1
      IF(NPTAU(2).GT.NR2S) NPTAU(2)=1
      IF(NPTAU(3).GT.NR3S) NPTAU(3)=1
      CALL DGEMV('T',3,3,1.D0,HT,3,SC1,1,0.D0,XPTAU,1)
      DO I=1,3
        DA1(I,1)=A1(I)/DBLE(NR1S)
        DA1(I,2)=A2(I)/DBLE(NR2S)
        DA1(I,3)=A3(I)/DBLE(NR3S)
      ENDDO
C..Interaction region
      SC1(1)=RCUT
      SC1(2)=RCUT
      SC1(3)=RCUT
      CALL DGEMV('T',3,3,1.D0,HTM1,3,SC1,1,0.D0,SC2,1)
      MR(1)=NINT(SC2(1)*NR1S)+1
      MR(2)=NINT(SC2(2)*NR2S)+1
      MR(3)=NINT(SC2(3)*NR3S)+1
      MR(1)=MIN(MR(1),NR1S/2-1)
      MR(2)=MIN(MR(2),NR2S/2-1)
      MR(3)=MIN(MR(3),NR3S/2-1)
C..Origin of density grid
      IORGX=NPTAU(1)-MR(1)
      IORGY=NPTAU(2)-MR(2)
      IORGZ=NPTAU(3)-MR(3)
      IF(IORGX.LT.1) IORGX=IORGX+NR1S
      IF(IORGY.LT.1) IORGY=IORGY+NR2S
      IF(IORGZ.LT.1) IORGZ=IORGZ+NR3S
C..loop over all grid points
      CHRG=0.0D0
      IGRID=0
      DO I=0,2*MR(1)
        IX=IORGX+I
        IF(IX.GT.NR1S) IX=IX-NR1S
        IF(IX.GE.NRXPL(MEPOS,1).AND.IX.LE.NRXPL(MEPOS,2)) THEN
          II=IX-NRXPL(MEPOS,1)+1
          I1=MR(1)-I
          DO J=0,2*MR(2)
            JJ=IORGY+J
            IF(JJ.GT.NR2S) JJ=JJ-NR2S
            J1=MR(2)-J
            DO K=0,2*MR(3)
              KK=IORGZ+K
              IF(KK.GT.NR3S) KK=KK-NR3S
              K1=MR(3)-K
              CALL POSITX(X,Y,Z,I1,J1,K1,DA1,XPTAU(1))
              R2=X*X+Y*Y+Z*Z
              R=SQRT(R2)
              IF(R.LT.RCUT) THEN
                IGRID=IGRID+1
                CHRG=CHRG+RHOE(II,JJ,KK)
              ENDIF
            ENDDO
          ENDDO
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C     ==================================================================
      SUBROUTINE PRINTPOT(TAU0,ACHRG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'adat.inc'
C     Arguments
      REAL*8  TAU0(3,NAX,NSX),ACHRG(NAT)
C     Variables
      INTEGER IAT,IS,IA,K
C     ==--------------------------------------------------------------==
      WRITE(6,'(/,1X,64("*"))')
      WRITE(6,'(A,10X,A,/,15X,A)')
     * '    ATOM  ','     COORDINATES                   AVERAGED POT.'
     *,'     X         Y         Z           POT'
      IAT=0   
      DO IS=1,NSP
        DO IA=1,NA(IS)
          IAT=IAT+1
          WRITE(6,'(1X,I4,1X,A2,5X,3F10.4,2X,1F10.5)')
     *     IAT,EL(IATYP(IS)),(TAU0(K,IA,IS),K=1,3),ACHRG(IAT)
        ENDDO   
      ENDDO     
      WRITE(6,'(1X,64("*"),/)')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

