C     ==================================================================
      SUBROUTINE PPENER(EH,EI,EE,EPS,V,VTEMP,EIVPS,EIROP)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     == ELECTROSTATIC AND PSEUDOPOTENTIAL ENERGIES AND THE           ==
C     == POTENTIAL ENERGY CONTRIBUTIONS TO THE FORCES ON THE IONS     ==
C     ==--------------------------------------------------------------==
C     == Input:                                                       ==
C     ==   V     Electronic density in G-space (input)                ==
C     ==   EIVPS phase factor times local pseudopotential  (VPS)      ==
C     ==   EIROP phase factor times Gaussian charge distributions     ==
C     ==         which replaced ionic point charges (RHOPS)           ==
C     == Output:                                                      ==
C     ==   EH    Hartree term                                         ==
C     ==         (e-e, ion-ion and e-ion [part] interactions)         ==
C     ==         Used by Car-Parrinello scheme                        ==
C     ==   EI    Gaussian charge dist. (RHOPS) (Ion-Ion interaction)  ==
C     ==   EE    e-e interaction                                      ==
C     ==         Both used by diagonalisation schemes (no G=0 term)   ==
C     ==   EPS   Integration of electronic density*local pp           ==
C     ==         EPSEU=2.D0*DREAL(EPS)*OMEGA)                         ==
C     ==   VPLOC G=0 pp part (given from pp data) (Energy=NEL*VPLOC)  ==
C     ==   VTEMP Stored the potential in G-space                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'simul.inc'
C     Arguments
      COMPLEX*16 EH,EI,EE,EPS,
     &           V(MAXFFT),VTEMP(NHG),EIVPS(NHG),EIROP(NHG)
C     Variables
      COMPLEX*16 VP,RP,RHET,RHOG,RHETS,RHOGS,VCG
      INTEGER    IG1,IG
C     ==--------------------------------------------------------------==
      IF(GEQ0) THEN
        VP=EIVPS(1)
        VPLOC=DREAL(VP)
        EPS=0.5D0*VP*DCONJG(V(NZH(1)))
        IG1=2
        RP=EIROP(1)
        RHET=V(NZH(1))
        RHOG=RHET+RP
        EH=0.5D0*SCG(1)*DREAL(RHOG)*DREAL(RHOG)
        EI=0.5D0*SCG(1)*RP*RP
        EE=0.5D0*SCG(1)*RHET*RHET
        VTEMP(1)=SCG(1)*RHOG
      ELSE
        IG1=1
        EPS=(0.0D0,0.0D0)
        VPLOC=0.0D0
        EH=(0.0D0,0.0D0)
        EI=(0.0D0,0.0D0)
        EE=(0.0D0,0.0D0)
      ENDIF
C$OMP  parallel do private(IG,VP,RP,RHET,RHOG,RHETS,RHOGS,VCG)
C$OMP+ reduction(+:EH,EI,EE,EPS)
#ifdef __SR8000
*poption parallel
*poption tlocal(IG,VP,RP,RHET,RHOG,RHETS,RHOGS,VCG)
*poption psum(EH,EI,EE,EPS)
#endif 
      DO IG=IG1,NHG
C       ==------------------------------------------------------------==
C       == In VTEMP it is stored temporarily the potential in G-space.==
C       ==------------------------------------------------------------==
        VP=EIVPS(IG)
        RP=EIROP(IG)
        RHET=V(NZH(IG))
        RHOG=RHET+RP
        RHETS=DCONJG(RHET)
        RHOGS=DCONJG(RHOG)
        VCG=SCG(IG)*RHOG
        VTEMP(IG)=VCG+VP
C       Hartree term (e-e, ion-ion and e-ion [part] interactions)
        EH=EH+VCG*RHOGS
C       Gaussian charge dist. (RHOPS) Ion-Ion interaction
        EI=EI+SCG(IG)*RP*DCONJG(RP)
C       e-e interaction
        EE=EE+SCG(IG)*RHET*DCONJG(RHET)
        EPS=EPS+RHETS*VP
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
