C     ==================================================================
      SUBROUTINE PRCNOSMOVE_ISO(VELP,STEP,IP)
C     ==--------------------------------------------------------------==
C     ==  ALLOWS FOR NOSE-HOOVER CHAIN DYNAMICS                       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nose.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'metr.inc'
C     Arguments
      INTEGER IP
      REAL*8  VELP(3,NAX,NSX),STEP
C     Variables
      REAL*8  FETC(NCHX),AT3,EKINH,HKEWANT,AA,GEPS_P,GEPS_V
      INTEGER L,K,IA,IS
C     ==--------------------------------------------------------------==
      AT3 = 3.D0*DBLE(NAT)
C     ==--------------------------------------------------------------==
C     == CALCULATE ALL NOSE-HOOVER CHAIN FORCES                       ==
C     == INCLUDING CELL THERMOSTAT FORCES                             ==
C     ==--------------------------------------------------------------==
C CELL KINETIC ENERGY
      EKINH=0.5D0*CMASS*VEPS*VEPS
C CELL THERMOSTAT FORCES
      HKEWANT = 0.5D0*GCKT
      FETC(1) = 2.D0*(EKINH - HKEWANT) 
      DO L=2,NCHC
       EKINH = 0.5D0*QNOSCC(L-1)*ETCDOT(L-1,IP)*ETCDOT(L-1,IP)
       HKEWANT = 0.5D0*TEMPC/FACTEM
       FETC(L) = 2.D0*(EKINH - HKEWANT)
      ENDDO
C     ==--------------------------------------------------------------==
C     == MOVE LAST CHAIN ELEMENTS                                     ==
C     ==--------------------------------------------------------------==
        ETCDOT(NCHC,IP) = ETCDOT(NCHC,IP) + 
     *                 0.25D0*STEP*FETC(NCHC)*QNOSCC(NCHC)
C     ==--------------------------------------------------------------==
C     ==  LOOP OVER REST OF NOSE-HOOVER CHAIN                         ==
C     ==--------------------------------------------------------------==
        DO L=1,NCHC-1
          AA = DEXP(-0.125D0*STEP*ETCDOT(NCHC+1-L,IP))
          ETCDOT(NCHC-L,IP) = ETCDOT(NCHC-L,IP)*AA*AA +
     *                     0.25D0*STEP*FETC(NCHC-L)*AA/QNOSCC(NCHC-L)
        ENDDO
C     ==----------------------------------------------------------------==
C     ==  PROPAGATE BAROSTAT VELOCITY                                   ==
C     ==----------------------------------------------------------------==
       GEPS_P = HTFP(1,1) + HTFP(2,2) + HTFP(3,3)
       GEPS_V = 0.D0
       DO K=1,3
        DO IS=1,NSP
         DO IA=1,NA(IS)
            GEPS_V = GEPS_V + (1.D0 + 3.D0/AT3)*
     *                        PMA(IS)*VELP(K,IA,IS)*VELP(K,IA,IS)
         ENDDO
        ENDDO
       ENDDO  
       AA = DEXP(-0.125D0*STEP*ETCDOT(1,IP))
       VEPS = VEPS*AA*AA + 0.25D0*STEP*(GEPS_P + GEPS_V)*AA/CMASS       
C     ==----------------------------------------------------------------==
C     ==  SCALE VELOCITIES                                              ==
C     ==----------------------------------------------------------------==
      AA = DEXP(-0.5D0*STEP*(1.D0+3.D0/AT3)*VEPS)
      DO IS=1,NSP
       DO IA=1,NA(IS)
          VELP(1,IA,IS) = VELP(1,IA,IS)*AA
          VELP(2,IA,IS) = VELP(2,IA,IS)*AA
          VELP(3,IA,IS) = VELP(3,IA,IS)*AA
         ENDDO
        ENDDO
C
C     ==----------------------------------------------------------------==
C     ==  PROPAGATE THERMOSTAT POSITIONS                                ==
C     ==----------------------------------------------------------------==
        DO L=1,NCHC
          ETC(L,IP) = ETC(L,IP) + 0.5D0*STEP*ETCDOT(L,IP)
        ENDDO
C     ==----------------------------------------------------------------==
C     ==  PROPAGATE BAROSTAT VELOCITY                                   ==
C     ==----------------------------------------------------------------==
       GEPS_P = HTFP(1,1) + HTFP(2,2) + HTFP(3,3)
       GEPS_V = 0.D0
       DO K=1,3
        DO IS=1,NSP
         DO IA=1,NA(IS)
            GEPS_V = GEPS_V + (1.D0 + 3.D0/AT3)*
     *                        PMA(IS)*VELP(K,IA,IS)*VELP(K,IA,IS)
         ENDDO
        ENDDO
       ENDDO  
       AA = DEXP(-0.125D0*STEP*ETCDOT(1,IP))
       VEPS = VEPS*AA*AA + 0.25D0*STEP*(GEPS_P + GEPS_V)*AA/CMASS       
C     ==--------------------------------------------------------------==
C     == RECALCULATE ALL NOSE-HOOVER CHAIN FORCES                     ==
C     == INCLUDING CELL THERMOSTAT FORCES                             ==
C     ==--------------------------------------------------------------==
C CELL KINETIC ENERGY
      EKINH=0.5D0*CMASS*VEPS*VEPS
C CELL THERMOSTAT FORCES
      HKEWANT = 0.5D0*GCKT
      FETC(1) = 2.D0*(EKINH - HKEWANT) 
C     ==-------------------------------------------------------------==
C     ==  LOOP OVER CHAIN AGAIN                                      ==
C     ==-------------------------------------------------------------==
        DO L=1,NCHC-1
          AA = DEXP(-0.125D0*STEP*ETCDOT(L+1,IP))
          ETCDOT(L,IP) = ETCDOT(L,IP)*AA*AA +
     *             0.25D0*STEP*FETC(L)*AA/QNOSCC(L)
          EKINH = 0.5D0*QNOSCC(L)*ETCDOT(L,IP)*ETCDOT(L,IP)
          HKEWANT = 0.5D0*TEMPC/FACTEM
          FETC(L+1) = 2.0D0*(EKINH - HKEWANT)
        ENDDO
C     ==--------------------------------------------------------------==
C     == MOVE LAST CHAIN ELEMENT                                      ==
C     ==--------------------------------------------------------------==
        ETCDOT(NCHC,IP) = ETCDOT(NCHC,IP) + 
     *                     0.25D0*STEP*FETC(NCHC)/QNOSCC(NCHC)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
C
C
C     ==================================================================
      SUBROUTINE PRCNOSMOVE(VELP,STEP,IP)
C     ==--------------------------------------------------------------==
C     ==  ALLOWS FOR NOSE-HOOVER CHAIN DYNAMICS                       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nose.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'metr.inc'
C     Arguments
      INTEGER IP
      REAL*8  VELP(3,NAX,NSX),STEP
C     Variables
      REAL*8  FETC(NCHX),HTFV(3,3),SCMAT(3,3),SCMD(3),SCMEV(3,3),
     &        VTMP1(3),AT3,EKINH,HKEWANT,AA,TRHV,AA1,AA2,AA3,
     &        DDOT
      EXTERNAL  DDOT
      INTEGER L,IS,IA,K,IERR
C     ==--------------------------------------------------------------==
      AT3 = 3.D0*DBLE(NAT)
C     ==--------------------------------------------------------------==
C     == CALCULATE ALL NOSE-HOOVER CHAIN FORCES                       ==
C     == INCLUDING CELL THERMOSTAT FORCES                             ==
C     ==--------------------------------------------------------------==
C CELL KINETIC ENERGY
      EKINH=0.5D0*CMASS*DDOT(9,HTVEL,1,HTVEL,1)
C CELL THERMOSTAT FORCES
      HKEWANT = 0.5D0*GCKT
      FETC(1) = 2.D0*(EKINH - HKEWANT) 
      DO L=2,NCHC
       EKINH = 0.5D0*QNOSCC(L-1)*ETCDOT(L-1,IP)*ETCDOT(L-1,IP)
       HKEWANT = 0.5D0*TEMPC/FACTEM
       FETC(L) = 2.D0*(EKINH - HKEWANT)
      ENDDO
C     ==--------------------------------------------------------------==
C     == MOVE LAST CHAIN ELEMENTS                                     ==
C     ==--------------------------------------------------------------==
        ETCDOT(NCHC,IP) = ETCDOT(NCHC,IP) + 
     *                 0.25D0*STEP*FETC(NCHC)*QNOSCC(NCHC)
C     ==--------------------------------------------------------------==
C     ==  LOOP OVER REST OF NOSE-HOOVER CHAIN                         ==
C     ==--------------------------------------------------------------==
        DO L=1,NCHC-1
          AA = DEXP(-0.125D0*STEP*ETCDOT(NCHC+1-L,IP))
          ETCDOT(NCHC-L,IP) = ETCDOT(NCHC-L,IP)*AA*AA +
     *                     0.25D0*STEP*FETC(NCHC-L)*AA/QNOSCC(NCHC-L)
        ENDDO
C     ==--------------------------------------------------------------==
C     ==  GET VELOCITY-DEPENDENT PART OF CELL FORCE                   ==
C     ==--------------------------------------------------------------==
        CALL AZZERO(HTFV,9)
        DO IS=1,NSP
         DO IA=1,NA(IS)
          DO K=1,3
           DO L=1,3
            HTFV(K,L) = HTFV(K,L) + PMA(IS)*VELP(K,IA,IS)*VELP(L,IA,IS)
           ENDDO
           HTFV(K,K) = HTFV(K,K) + 
     *                 PMA(IS)*VELP(K,IA,IS)*VELP(K,IA,IS)/AT3
          ENDDO
         ENDDO
        ENDDO  
C     ==--------------------------------------------------------------==
C     ==  PROPAGATE BAROSTAT (CELL VELOCITY)                          ==
C     ==--------------------------------------------------------------==
       AA = DEXP(-0.125D0*STEP*ETCDOT(1,IP))
       DO K=1,3
        DO L=1,3
         HTVEL(K,L) = HTVEL(K,L)*AA*AA +
     *                0.25D0*STEP*(HTFV(K,L) + HTFP(K,L))*AA/CMASS
        ENDDO
       ENDDO
C     ==----------------------------------------------------------------==
C     ==  FOR VELOCITY SCALING, FORM MATRIX v_h + Tr(v_h)/N + v_zeta1   ==
C     ==----------------------------------------------------------------==
        TRHV = HTVEL(1,1) + HTVEL(2,2) + HTVEL(3,3)
        DO K=1,3
         DO L=1,3
          SCMAT(K,L) = HTVEL(K,L)
         ENDDO
         SCMAT(K,K) = SCMAT(K,K) + TRHV/AT3
        ENDDO
C     ==----------------------------------------------------------------==
C     ==  DIAGONALIZE THIS MATRIX AND SAVE EIGENVALUES AND EIGENVECTORS ==
C     ==----------------------------------------------------------------==
       CALL JACOBI(3,3,SCMAT,SCMD,SCMEV,IERR)
C     ==----------------------------------------------------------------==
C     ==  ROTATE, SCALE, AND ROTATE BACK FOR VELOCITIES                 ==
C     ==----------------------------------------------------------------==
      AA1 = DEXP(-0.5D0*STEP*SCMD(1))
      AA2 = DEXP(-0.5D0*STEP*SCMD(2))
      AA3 = DEXP(-0.5D0*STEP*SCMD(3))
      DO IS=1,NSP
       DO IA=1,NA(IS)
C
          VTMP1(1) = SCMEV(1,1)*VELP(1,IA,IS) + SCMEV(2,1)*VELP(2,IA,IS)
     *             + SCMEV(3,1)*VELP(3,IA,IS)
          VTMP1(2) = SCMEV(1,2)*VELP(1,IA,IS) + SCMEV(2,2)*VELP(2,IA,IS)
     *             + SCMEV(3,2)*VELP(3,IA,IS)
          VTMP1(3) = SCMEV(1,3)*VELP(1,IA,IS) + SCMEV(2,3)*VELP(2,IA,IS)
     *             + SCMEV(3,3)*VELP(3,IA,IS)
C
          VTMP1(1) = VTMP1(1)*AA1
          VTMP1(2) = VTMP1(2)*AA2
          VTMP1(3) = VTMP1(3)*AA3
C
          VELP(1,IA,IS) = SCMEV(1,1)*VTMP1(1) + SCMEV(1,2)*VTMP1(2)
     *                  + SCMEV(1,3)*VTMP1(3)
          VELP(2,IA,IS) = SCMEV(2,1)*VTMP1(1) + SCMEV(2,2)*VTMP1(2)
     *                  + SCMEV(2,3)*VTMP1(3)
          VELP(3,IA,IS) = SCMEV(3,1)*VTMP1(1) + SCMEV(3,2)*VTMP1(2)
     *                  + SCMEV(3,3)*VTMP1(3)
         ENDDO
        ENDDO
C
C     ==----------------------------------------------------------------==
C     ==  PROPAGATE THERMOSTAT POSITIONS                                ==
C     ==----------------------------------------------------------------==
        DO L=1,NCHC
          ETC(L,IP) = ETC(L,IP) + 0.5D0*STEP*ETCDOT(L,IP)
        ENDDO
C     ==--------------------------------------------------------------==
C     ==  GET VELOCITY-DEPENDENT PART OF CELL FORCE                   ==
C     ==--------------------------------------------------------------==
        CALL AZZERO(HTFV,9)
        DO IS=1,NSP
         DO IA=1,NA(IS)
          DO K=1,3
           DO L=1,3
            HTFV(K,L) = HTFV(K,L) + PMA(IS)*VELP(K,IA,IS)*VELP(L,IA,IS)
           ENDDO
           HTFV(K,K) = HTFV(K,K) + 
     *                 PMA(IS)*VELP(K,IA,IS)*VELP(K,IA,IS)/AT3
          ENDDO
         ENDDO
        ENDDO  
C     ==--------------------------------------------------------------==
C     ==  PROPAGATE BAROSTAT (CELL VELOCITY)                          ==
C     ==--------------------------------------------------------------==
       AA = DEXP(-0.125D0*STEP*ETCDOT(1,IP))
       DO K=1,3
        DO L=1,3
         HTVEL(K,L) = HTVEL(K,L)*AA*AA +
     *                0.25D0*STEP*(HTFV(K,L) + HTFP(K,L))*AA/CMASS
        ENDDO
       ENDDO
C     ==--------------------------------------------------------------==
C     == RECALCULATE ALL NOSE-HOOVER CHAIN FORCES                     ==
C     == INCLUDING CELL THERMOSTAT FORCES                             ==
C     ==--------------------------------------------------------------==
C CELL KINETIC ENERGY
      EKINH=0.5D0*CMASS*DDOT(9,HTVEL,1,HTVEL,1)
C CELL THERMOSTAT FORCES
      HKEWANT = 0.5D0*GCKT
      FETC(1) = 2.D0*(EKINH - HKEWANT) 
C     ==-------------------------------------------------------------==
C     ==  LOOP OVER CHAIN AGAIN                                      ==
C     ==-------------------------------------------------------------==
        DO L=1,NCHC-1
          AA = DEXP(-0.125D0*STEP*ETCDOT(L+1,IP))
          ETCDOT(L,IP) = ETCDOT(L,IP)*AA*AA +
     *             0.25D0*STEP*FETC(L)*AA/QNOSCC(L)
          EKINH = 0.5D0*QNOSCC(L)*ETCDOT(L,IP)*ETCDOT(L,IP)
          HKEWANT = 0.5D0*TEMPC/FACTEM
          FETC(L+1) = 2.0D0*(EKINH - HKEWANT)
        ENDDO
C     ==--------------------------------------------------------------==
C     == MOVE LAST CHAIN ELEMENT                                      ==
C     ==--------------------------------------------------------------==
        ETCDOT(NCHC,IP) = ETCDOT(NCHC,IP) + 
     *                     0.25D0*STEP*FETC(NCHC)/QNOSCC(NCHC)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================











