! a minimal implementation of L'Ecuyer MRG32k3a
! with wrappers to simplify calling from around CPMD
      
!    /* Does a matrix-vector multiply modulo m. used to skip forward */
      SUBROUTINE MVMULTM( A,s,v,m)
      IMPLICIT NONE
      REAL*8 A(3,3)
      REAL*8 s(3),v(3),m
      REAL*8 x(3)
      REAL*8 two17,two53
      PARAMETER( two17  =    131072.0d0                )
      PARAMETER( two53  =    9007199254740992.0d0      )
      
      REAL*8 nx,ad217,a2
      INTEGER i, j
      do i=1,3
        x(i)=0
        do j=1,3
          a2=a(i,j)
          nx=x(i)+a2*s(j)
          if ((nx .ge. two53) .or. (nx .le. -two53)) then
            ad217 = INT(a2 / two17)
            a2 = a2 - ad217 * two17
            nx = ad217 * s(j)
            nx = (nx - INT(nx / m) * m) * two17 + a2 * s(j) + x(i)
          endif
          nx = nx - INT(nx / m)*m
          if  (nx.lt.0.0d0) then
            x(i)=nx+m
          else  
            x(i)=nx
          end if
        end do
      end do
      v=x
      END SUBROUTINE

      ! extracts a uniform prn within (0,1]
      REAL*8 FUNCTION PRNGUNIF(sprng)
      IMPLICIT NONE
      REAL*8 sprng(2,3)
      LOGICAL anti
      PARAMETER( anti=.false. )

      REAL*8 a12,a13n,a21,a23n
      PARAMETER( a12    =    1403580.0d0               )
      PARAMETER( a13n   =     810728.0d0               )
      PARAMETER( a21    =     527612.0d0               )
      PARAMETER( a23n   =    1370589.0d0               )
      REAL*8 m1,m2
      PARAMETER( m1     =  4294967087.0d0              )
      PARAMETER( m2     =  4294944443.0d0              )

      REAL*8 p1,p2,u

      ! advances stream by one
      p1=a12*sprng(1,2)-a13n*sprng(1,1)
      p1=p1-INT(p1/m1)*m1
      if (p1.lt.0.0d0) p1=p1+m1

      sprng(1,1)=sprng(1,2)
      sprng(1,2)=sprng(1,3)
      sprng(1,3)=p1

      p2=a21*sprng(2,3)-a23n*sprng(2,1)
      p2=p2-INT(p2/m2)*m2
      if (p2.lt.0.0d0) p2=p2+m2

      sprng(2,1)=sprng(2,2)
      sprng(2,2)=sprng(2,3)
      sprng(2,3)=p2

      ! computes a PRN out of the status
      if (p1.gt.p2) then
        u=(p1-p2)*2.328306549295727688d-10
      else
        u=(p1-p2+m1)*2.328306549295727688d-10
      end if
      if (anti) then
        PRNGUNIF=1-u
      else
        PRNGUNIF=u
      end if

      END FUNCTION

      ! Initializes the status of the PRNG using a simple linear congruential generator
      SUBROUTINE PRNGSEED(seed,sprng)
      IMPLICIT NONE
      INTEGER seed
      REAL*8 m1,m2
      PARAMETER( m1     =  4294967087.0d0              )
      PARAMETER( m2     =  4294944443.0d0              )

      REAL*8 sprng(2,3)
      INTEGER i, last
      last = seed
      do i=1,3
        last= IEOR(last,ISHFT(LAST,-30))
        last=last*1812433253 + i
        sprng(1,i)=last
        if (sprng(1,i).lt.0) sprng(1,i)=-sprng(1,i)
        do while (sprng(1,i).gt.m1) 
          sprng(1,i)=sprng(1,i)-m1
        end do
      end do
      do i=1,3
        last= IEOR(last,ISHFT(LAST,-30))
        last=last*1812433253 + i
        sprng(2,i)=last
        if (sprng(2,i).lt.0) sprng(2,i)=-sprng(2,i)
        do while (sprng(2,i).gt.m2)
          sprng(2,i)=sprng(2,i)-m2
        end do
      end do
      END SUBROUTINE

      ! Fast forward across the stream, to get an 'independent' sequence
      SUBROUTINE PRNGSKIP(sprng,nsprng)
      IMPLICIT NONE
      REAL*8 sprng(2,3), nsprng(2,3), tmp(2,3)
      REAL*8 m1,m2
      PARAMETER( m1     =  4294967087.0d0              )
      PARAMETER( m2     =  4294944443.0d0              )
      REAL*8 SHIFT1(3,3)
      DATA SHIFT1  / 82758667.0d0, 3672831523.0d0, 3672091415.0d0, 
     &             1871391091.0d0,   69195019.0d0, 3528743235.0d0,
     &             4127413238.0d0, 1871391091.0d0,   69195019.0d0 /
  
      REAL*8 SHIFT2(3,3)  
      DATA SHIFT2 /1511326704.0d0, 4292754251.0d0, 3859662829.0d0, 
     &             3759209742.0d0, 1511326704.0d0, 4292754251.0d0,
     &             1610795712.0d0, 3889917532.0d0, 3708466080.0d0 /

      tmp=sprng
      CALL MVMULTM(SHIFT1,tmp(1,:),tmp(1,:),M1)
      CALL MVMULTM(SHIFT2,tmp(2,:),tmp(2,:),M2)
      nsprng=tmp
      END SUBROUTINE

! here we use a ratio-of-uniforms rather than box-muller, because 
! it is as fast (or faster) and we avoid storing buffers (which makes restart more compact)
      REAL*8 FUNCTION PRNGGAUSS(sprng)
      IMPLICIT NONE 
      REAL*8 PRNGUNIF
      REAL*8 sprng(2,3)
      
      REAL(8) u,v,x,y,q
      DO
        u = PRNGUNIF(sprng)
        v = PRNGUNIF(sprng)
        v = 1.7156d0 * (v-0.5d0)
           
        x = u - 0.449871d0
        y = ABS(v)+0.386595d0
        q = x*x + y*(0.19600d0*y-0.25472d0*x)
        IF (q .lt. 0.27597d0)  EXIT
        IF (q .gt. 0.27846d0) CYCLE
        IF (v*v.lt.-4.0d0*log(u)*u*u) EXIT
      ENDDO
      PRNGGAUSS=v/u
      END FUNCTION
   
      SUBROUTINE PRNGINIT()
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'prng.inc'
      INTEGER i
!     TEST OF THE IMPLEMENTATION : the sum must be ~ 5001090.95 (see L'Ecuyer, Op.Res. 47, 159 (1999))
!      REAL(8) ss
!      REPSEED=12345
!      ss=0.0d0
!      do i=1,10000000
!        ss=ss+REPPRNGU()
!      end do
!      write(6,*) "PRNG SUM IS ", ss

      CALL PRNGSEED(IPRNG, REPSEED)
      CALL PRNGSKIP(REPSEED,PARASEED)
      CALL PRNGPARASKIP(PARASEED,PARASEED)
      END SUBROUTINE

      SUBROUTINE PRNGPARASKIP(sprng,nsprng)
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'prng.inc'
      INTEGER i
      REAL*8 sprng(2,3), nsprng(2,3), tmp(2,3)
      tmp=sprng
      do i=0,me-1
        CALL PRNGSKIP(tmp,tmp)
      end do
      nsprng=tmp
      END SUBROUTINE

! this set of functions generate uniform (*U) and Gaussian (*G) 
! pseudo random numbers. 
! REP* versions are initialized so as to generate the same stream on all nodes
! PARA* versions generate independent streams on all nodes.
! both functions should be called always by all instances of the code: 
! in this way, restart should work properly.
! NB: RUNS WITH DIFFERENT NUMBER OF PROCESSORS WILL GIVE DIFFERENT 
!     (but statistically equivalent) RESULTS
      REAL*8 FUNCTION REPPRNGU()
      IMPLICIT NONE
      REAL*8 PRNGUNIF
      REAL *8 REPSEED(2,3), PARASEED(2,3)
      COMMON/PRNG/ REPSEED, PARASEED

      REPPRNGU=PRNGUNIF(REPSEED)
      END FUNCTION

      REAL*8 FUNCTION REPPRNGG()
      IMPLICIT NONE
      REAL*8 PRNGGAUSS
      REAL *8 REPSEED(2,3), PARASEED(2,3)
      COMMON/PRNG/ REPSEED, PARASEED

      REPPRNGG=PRNGGAUSS(REPSEED)
      END FUNCTION

      REAL*8 FUNCTION PARAPRNGU()
      IMPLICIT NONE
      REAL*8 PRNGUNIF
      REAL *8 REPSEED(2,3), PARASEED(2,3)
      COMMON/PRNG/ REPSEED, PARASEED

      PARAPRNGU=PRNGUNIF(PARASEED)
      END FUNCTION

      REAL*8 FUNCTION PARAPRNGG()
      IMPLICIT NONE
      REAL*8 PRNGGAUSS
      REAL *8 REPSEED(2,3), PARASEED(2,3)
      COMMON/PRNG/ REPSEED, PARASEED

      PARAPRNGG=PRNGGAUSS(PARASEED)
      END FUNCTION
