      SUBROUTINE PROYLM(C0,CENTER,NY,NST,RMAX,NMAX)
C     ==--------------------------------------------------------------==
C     == ----  PROJECT WAVEFUNCTIONS ON REAL SPHERICAL HARMONICS -----==      
C     ==--------------------------------------------------------------==
C     == -----           PROGRAM CHANGES COEFFICIENTS             ---===
      IMPLICIT NONE
      INCLUDE 'system.h'     
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'      
      INCLUDE 'spin.inc'
      INCLUDE 'adat.inc'      
      INCLUDE 'irat.inc'
      INCLUDE 'prop.inc'
      INCLUDE 'fileopen.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'gvec.inc'
C     ARGUMENTS
      INTEGER NY,NST,NMAX
      COMPLEX*16 C0(NGW,NST)
      REAL*8 CENTER(3),RMAX
C     VARIABLES
      REAL*8 XTMAT(NY,NST),RAD(0:NMAX,NST),
     &     W(NY,NST), RLM(0:NMAX), MESH(0:NMAX),
     &     GK2(3,NGW), HG2(NGW), 
     &     SUM(0:INT(SQRT(ABS(1.0D0*NY-1.0))),NST),
     &     DEVL(0:*), BSL(NGW,1:NMAX), SPREAD(NST)
      COMPLEX*16 CY(NGW,NY), CC(NGW,NY), 
     &     EGR(NGW), CYT(NY,NGW)
      POINTER  (IP_XTMAT,XTMAT), (IP_CY,CY),
     &     (IP_CC,CC), (IP_RAD,RAD), 
     &     (IP_EGR,EGR), (IP_W,W),  
     &     (IP_RLM, RLM),(IP_CYT,CYT),
     &     (IP_SUM,SUM), (IP_MESH,MESH),
     &     (IP_GK2,GK2), (IP_HG2,HG2),
     &     (IP_DEVL,DEVL), (IP_BSL,BSL),
     &     (IP_SPREAD,SPREAD)

      INTEGER ISUB, I, J, L, M, K, LMAX, J1, L1, M1,
     &     JMAX, S, EMP, MSGLEN, L2, M2, JMAX2
      REAL*8 BESSL, DOTP, DP, SQOFPI, DR, NORM, FPIDSOM,
     &     SFC, TWPI, VMAX, ZERO
      EXTERNAL DOTP, BESSL   
      LOGICAL FERROR, LOGM, EQM
      CHARACTER*40 DG

      PARAMETER(SQOFPI=0.2820947917738781434D0)      
C     ==---------------------------------------------------------------==
      CALL TISET('   PROWYLM',ISUB)      
      CALL MEMORY(IP_CY,2*NGW*NY,'CY')
      CALL MEMORY(IP_CYT,2*NGW*NY,'CYT')
C     TRANSLATION VECTOR
      IF(PARENT)THEN
	 WRITE(6,'(1X,A)') 'ALL INPUT PARAMETERS :'
         WRITE(6,'(1X,A,3(F10.5))') 'CENTER OF CLUSTER :', 
     &        CENTER(1), CENTER(2),CENTER(3)
      ENDIF
      TWPI=ATAN(1.0D0)*8.0D0
C     LOAD ALL SPHERICAL HARMONICS 
      CALL LOADYLM(CYT,NY,NGW)
C     TRANSPOSE CYT
      DO I=1,NY
         DO J=1,NGW
            CY(J,I)=CYT(I,J)
         ENDDO
      ENDDO
          
      CALL FREEM(IP_CYT)
C     OTHER ALLOCATION 
      CALL MEMORY(IP_EGR,2*NGW,'EGR')
      CALL MEMORY(IP_HG2,NGW,'HG2')
      CALL MEMORY(IP_GK2,3*NGW,'GK2')
C     INITIALIZE CARTESIAN COORDINATES OF RECIPROCAL LATTICE VECTORS
      DO I=1,NGW
        GK2(1,I)=(GK(1,I)*B1(1)+GK(2,I)*B2(1)+GK(3,I)*B3(1))*TWPI/ALAT
        GK2(2,I)=(GK(1,I)*B1(2)+GK(2,I)*B2(2)+GK(3,I)*B3(2))*TWPI/ALAT
        GK2(3,I)=(GK(1,I)*B1(3)+GK(2,I)*B2(3)+GK(3,I)*B3(3))*TWPI/ALAT
        HG2(I)=DSQRT(GK2(1,I)*GK2(1,I)+GK2(2,I)*GK2(2,I)+
     &       GK2(3,I)*GK2(3,I))        
      ENDDO
C     FIND EXP(I*<G,R>)  
      DO I=1,NGW
         DP=CENTER(1)*GK2(1,I)+CENTER(2)*GK2(2,I)+CENTER(3)*GK2(3,I)
         EGR(I)=DCMPLX(COS(DP),SIN(DP))
      ENDDO
      CALL FREEM(IP_GK2)
      CALL MEMORY(IP_XTMAT,NY*NST,'XTMAT')
      CALL MEMORY(IP_CC,2*NGW*NY,'CC')
C     FIND OVERLAP MATRIX
      CALL DCOPY(2*NGW*NY,CY,1,CC,1)      

      DO I=1,NGW
         DO J=1,NY
c            CC(I,J)=CC(I,J)*DCMPLX(DREAL(EGR(I)),-DIMAG(EGR(I)))
         ENDDO
      ENDDO

      DO I=1,NST
         DO J=1,NGW
            C0(J,I)=C0(J,I)*EGR(J)
         ENDDO
      ENDDO
      
      DO I=1,NY
         SFC=DOTP(NGW,CC(1,I),CC(1,I))         
#ifdef PARALLEL
         CALL GLOSUM(1,SFC)
#endif
         SFC=1.D0/DSQRT(SFC)
         CALL ZDSCAL(NGW,SFC,CC(1,I),1)        
      ENDDO
     
      CALL OVLAP2(NGW,NY,NST,XTMAT,CC,C0)
#ifdef PARALLEL
      CALL GLOSUM(NY*NST,XTMAT)
#endif
      
C     WRITE OVERLAP MATRIX
      IF(PARENT)THEN 
         CALL INTS40(DG,NY,S)
         CALL FILEOPEN(41,'YLMCOEF',FO_DEF,FERROR)      
         WRITE(41,'('//DG(41-S:40)//'(F18.14))') 
     &        ((XTMAT(L,K),L=1,NY),K=1,NST)
         CALL FILECLOSE(41) 
      ENDIF
      CALL FREEM(IP_CC)

      IF(RADIAL)THEN         
         CALL MEMORY(IP_SPREAD,NST,'SPREAD')
         IF(SPREAD_HAM)THEN 
C           READ WANNIER INPUT         
            IF(PARENT) THEN
               CALL FILEOPEN(60,'SPREAD',FO_DEF,FERROR)
               DO I=1,NST
                  READ(60,*) J, DR, NORM, ZERO, SPREAD(I)
               ENDDO
               CALL FILECLOSE(60)
               WRITE(6,"(1X,A,F10.5,A,F10.5)")  'PROJECT STATES ONLY 
     &              WITH SPREAD FROM ', SPR_MIN,' TO ',SPR_MAX
            ENDIF
C     SPREAD
#ifdef PARALLEL
            MSGLEN=8*NST
            CALL MY_BCAST(SPREAD,MSGLEN,SOURCE,ALLGRP)
#endif
         ELSE
           SPR_MIN=-1.0D0
           SPR_MAX=10.0D0
           DO I=1,NST
            SPREAD(I)=0.0D0
           ENDDO	
         ENDIF
         IF(PARENT)THEN
           WRITE(6,*) ' RECORD OF ALL RADIAL PARTS'
           CALL FILEOPEN(41,'RADIAL',FO_DEF+FO_UFO,FERROR)
         ENDIF 
C     FIND RADIAL PART OF L-TH COMPONENT  AND WEIGHTS.
         LMAX=INT(SQRT(ABS(1.0D0*NY-1.0)))
C	 NUMBER OF SEGMENTS SHOULD BE EVEN
         IF(MOD(NMAX,2).EQ.1) NMAX=NMAX+1
         CALL MEMORY(IP_RLM,(NMAX+1),'RLM')
         CALL MEMORY(IP_MESH,NMAX+1,'MESH')
         CALL MEMORY(IP_SUM,(LMAX+1)*NST,'SUM')         
         CALL MEMORY(IP_DEVL,LMAX+1,'DEVL')
         CALL MEMORY(IP_W,NY*NST,'W')
         CALL MEMORY(IP_BSL,NGW*NMAX,'BSL')
         CALL AZZERO(DEVL,LMAX+1)         
         FPIDSOM=FPI/DSQRT(OMEGA)   
         LOGM=.FALSE.
         EQM=.TRUE.
         MESH(0)=0.0D0
C        LOGARITHMIC 
         IF(LOGM)THEN
	    IF(PARENT)THEN
	       WRITE(6,*) 'LOGARITHMIC GRID IS USED'
	    ENDIF	
            MESH(1)=0.01
            SFC=LOG(RMAX/MESH(1))/DBLE(NMAX-1)
            SFC=DEXP(SFC)
            DO I=2,NMAX
               MESH(I)=MESH(I-1)*SFC
            ENDDO   
         ENDIF
C        EQUIDISTANT
         IF(EQM)THEN
	    IF(PARENT)THEN
              WRITE(6,*) 'EQUIDISTANT GRID IS USED'	
            ENDIF
            DR=RMAX/DBLE(NMAX)  
            DO I=1,NMAX
               MESH(I)=DR*I
            ENDDO 
         ENDIF      
   
         IF(PARENT)THEN
           WRITE(6,'(1X,A,I8)') 'NUMBER OF GRIDS =',NMAX
           WRITE(6,'(1X,A,F10.5)') 'RADIUS OF INTEGRATION =',RMAX
         ENDIF	 

         CALL AZZERO(SUM,(LMAX+1)*NST)
         IF(PARENT)THEN
c        to radial file all parameters
           WRITE(41) LOGM,EQM,NMAX,RMAX,
     &     SPREAD_HAM, SPR_MIN, SPR_MAX,
     &       LMAX,NST 
         ENDIF
         DO L=0,LMAX
            IF(PARENT)THEN
	      WRITE(6,*) 'CURRENT L =', L
	    ENDIF 
            CALL LOADBSL(NGW,NMAX,L,HG2,MESH,BSL)
            DO M=1,2*L+1
               DO I=1,NST
                  IF((SPREAD(I).GT.SPR_MIN).AND.
     &                 (SPREAD(I).LT.SPR_MAX)) THEN
                     J1=L*L+M
C     POINT FROM DR, RMAX
                     IF(L.EQ.0)THEN                     
                        DO J=1,NMAX
                           RLM(J)=0.0D0
                           DO K=1,NGW                  
                              IF(HG(K).LT.1D-6)THEN
                                 RLM(J)=RLM(J)+
     &                                BSL(K,J)*
     &                                DREAL(C0(K,I))*
     &                                SQOFPI
                              ELSE 
                                 RLM(J)=RLM(J)+
     &                                2*BSL(K,J)*
     &                                DREAL(C0(K,I))*
     &                                SQOFPI
                              ENDIF  
                           ENDDO                     
                           RLM(J)=RLM(J)*FPIDSOM
                        ENDDO            
                     ELSE 
                        IF(MOD(L,2).EQ.0)THEN
                           DO J=1,NMAX                  
                              RLM(J)=0.0D0
                              DO K=1,NGW                  
                                 IF(HG(K).GT.1D-6)THEN
                                    RLM(J)=RLM(J)+
     &                                   BSL(K,J)*
     &                                   DREAL(C0(K,I))*DREAL(CY(K,J1)) 
                                 ENDIF
                              ENDDO
                              RLM(J)=2*RLM(J)*FPIDSOM
                           ENDDO
                        ELSE
                           DO J=1,NMAX                  
                              RLM(J)=0.0D0                              
                              DO K=1,NGW
                                 IF(HG(K).GT.1D-6)THEN
                                    RLM(J)=RLM(J)+
     &                                   BSL(K,J)*
     &                                   DIMAG(C0(K,I))*
     &                                   DIMAG(CY(K,J1))
                                 ENDIF
                              ENDDO
                              RLM(J)=2*RLM(J)*FPIDSOM                  
                           ENDDO
                        ENDIF
                     ENDIF                              
#ifdef PARALLEL
                     CALL GLOSUM((NMAX+1),RLM(0))
#endif
                     IF(PARENT) THEN
                        WRITE(41)(RLM(J),J=0,NMAX) 
                     ENDIF
                     IF(EQM)THEN
                        CALL INTSIMP(NMAX,DR,RLM(0),W(J1,I))
                     ELSE	
                        CALL WINT(NMAX,MESH,RLM(0),W(J1,I))
                     ENDIF
                     SUM(L,I)=SUM(L,I)+W(J1,I)               
C     I=1,NST
                  ENDIF
               ENDDO
C     M=1,2*L+1
            ENDDO 
C     L=0,L
         ENDDO            

         IF(PARENT) CALL FILECLOSE(41)
C     WRITE ALL WEIGHTS
         IF(PARENT)THEN
            IF(NY.EQ.1)THEN
               WRITE(6,'(1X,A)') 'ORBITAL AND MAIN HARMONIC(L)'
            ELSE
               WRITE(6,'(1X,A)') 'ORBITAL AND 2 FIRST MAIN HARMONICS(L)'
            ENDIF            
            DO I=1,NST
               IF((SPREAD(I).GT.SPR_MIN).AND.
     &              (SPREAD(I).LT.SPR_MAX)) THEN               
                  CALL MAXARR(LMAX+1,JMAX,JMAX2,SUM(0,I))
                  IF(NY.EQ.1)THEN
                     WRITE(6,'(1X,I8,A)') I,'-TH ORBITAL'
                  ELSE 
                     WRITE(6,'(1X,I8,A,2(/,1X,T10,I6,F10.5))') 
     &                    I,'-TH ORBITAL', 
     &                    JMAX-1, SUM(JMAX-1,I), JMAX2-1, SUM(JMAX2-1,I)
                  ENDIF
                  DO L=0,LMAX
                     DO M=1,2*L+1
                        J1=L*L+M
                        WRITE(6,*) 'W(',J1,')=',W(J1,I)
                     ENDDO
                     WRITE(6,*) 'SUM(',L,')=',SUM(L,I)
                  ENDDO
               ENDIF  
            ENDDO
            
            DO I=1,NST
               IF((SPREAD(I).GT.SPR_MIN).AND.
     &              (SPREAD(I).LT.SPR_MAX)) THEN
                  CALL MAXARR(LMAX+1,JMAX,JMAX2,SUM(0,I))
                  IF(NY.EQ.1)THEN
                     WRITE(6,'(1X,I8,A)') I,'-TH ORBITAL'
                  ELSE 
                     WRITE(6,'(1X,I8,A,2(/,1X,T10,I6,F10.5))') 
     &                    I,'-TH ORBITAL', 
     &                    JMAX-1, SUM(JMAX-1,I), JMAX2-1, SUM(JMAX2-1,I)
                  ENDIF
               ENDIF
            ENDDO      
         ENDIF                  

         IF(PARENT)THEN
C     CALCULATE AND WRITE DEVIATIONS 
            DO L=0,LMAX
               DEVL(L)=0.0D0
            ENDDO          
            JMAX=0
            DO I=1,NST
               IF((SPREAD(I).GT.SPR_MIN).AND.
     &              (SPREAD(I).LT.SPR_MAX)) THEN               
                  JMAX=JMAX+1
                  DO L=0,LMAX
                     DO J=L,LMAX
                        DEVL(J)=DEVL(J)+SUM(L,I)
                     ENDDO
                  ENDDO             
               ENDIF
            ENDDO
               
            DO J=0,LMAX
               DEVL(J)=1.0D0-DEVL(J)/DBLE(JMAX)
            ENDDO

            DO J=0,LMAX
               WRITE(6,*)  'E(',J,') = ',DEVL(J)
            ENDDO

            CALL FILEOPEN(41,'WEIGHTS',FO_DEF+FO_UFO,FERROR)
            WRITE(41) ((W(K,I),I=1,NST),K=1,NY)
            CALL FILECLOSE(41) 

            CALL FILEOPEN(41,'SUMWEIGHTS',FO_DEF+FO_UFO,FERROR)           
            WRITE(41) ((SUM(K,I),I=1,NST),K=0,LMAX)
            CALL FILECLOSE(41)             
         ENDIF 
      ENDIF
      CALL TIHALT('   PROWYLM',ISUB)
      END SUBROUTINE PROYLM 

      SUBROUTINE LOADYLM(CYT,NY,NNGW)
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
C     ARGUMENTS
      INTEGER NY,NNGW
      COMPLEX*16 CYT(NY,NNGW)

C     VARIABLES
      INTEGER ISUB, I, LMAX
      REAL*8 X, Y, Z, NORM, SQOFPI
      PARAMETER(SQOFPI=0.2820947917738781434D0)
      CALL TISET('   LOADYLM',ISUB)   
      LMAX=INT(SQRT(ABS(1.0D0*NY-1.0)))
      DO I=1,NNGW
         IF(HG(I).LT.1D-6)THEN            
            CALL AZZERO(CYT(1,I),NY*2)
            CYT(1,I)=DCMPLX(SQOFPI,0.0D0)
         ELSE   
            NORM=DMAX1(DSQRT(HG(I)),1.0D-6)
            X=GK(1,I)/NORM
            Y=GK(2,I)/NORM
            Z=GK(3,I)/NORM
            CALL LOADALL(X,Y,Z,CYT(1,I),NY,LMAX)
         ENDIF
      ENDDO                    
      CALL TIHALT('   LOADYLM',ISUB)
      END SUBROUTINE LOADYLM

      SUBROUTINE LOADALL(X,Y,Z,YLM,N,LMAX)
      IMPLICIT NONE
C     ARGUMENTS
      INTEGER N, LMAX
      REAL*8  X,Y,Z
      COMPLEX*16 YLM(N)
C     VARIABLES
      INTEGER L, M, I, J, K, LMM
      REAL*8 COPH(0:*), SIPH(0:*), LAY1(0:*), LAY2(0:*), 
     &     LAY3(0:*)
      POINTER (IP_COPH,COPH), (IP_SIPH,SIPH),(IP_LAY1,LAY1),
     &     (IP_LAY2,LAY2),(IP_LAY3,LAY3)
      REAL*8 R, IKS, SQOFPI, SQ2, TWL1, SQ
      COMPLEX*16 C, IL
      PARAMETER(SQOFPI=0.2820947917738781434D0)
      PARAMETER(SQ2=1.41421356237309504880D0)

      CALL MEMORY(IP_COPH,LMAX+1,'COPH')
      CALL MEMORY(IP_SIPH,LMAX+1,'SIPH')
      CALL MEMORY(IP_LAY1,LMAX+1,'LAY1')
      CALL MEMORY(IP_LAY2,LMAX+1,'LAY2')
      CALL MEMORY(IP_LAY3,LMAX+1,'LAY3')

      COPH(0)=1.0D0
      SIPH(0)=0.0D0
      C=DCMPLX(1.0D0,0.0D0)
      DO I=1,LMAX
         C=C*DCMPLX(X,Y)
         COPH(I)=DREAL(C)
         SIPH(I)=DIMAG(C)
      ENDDO
      
      LAY1(0)=1.0D0
      LAY2(0)=Z
      LAY2(1)=1.0D0
      
      SQ=SQOFPI*SQ2
      DO L=0,LMAX
         J=L*L+L+1
         TWL1=DSQRT(2.0D0*DBLE(L)+1.0D0)
         YLM(J)=SQOFPI*TWL1
         DO M=1,L
           J=J+1
           YLM(J)=SQ*TWL1
         ENDDO
      ENDDO

C    LEGENDRE POLYNOMIALS
      YLM(1)=YLM(1)*LAY1(0)
      IF(LMAX.GT.0)THEN
         YLM(2)=YLM(2)*LAY2(1)
         YLM(3)=YLM(3)*LAY2(0)
         YLM(4)=YLM(4)*LAY2(1)
      ENDIF

      J=2*2
      DO L=2,LMAX
         TWL1=2.0D0*DBLE(L)-1.0D0
         DO I=0,L-2           
           LAY3(I)=(TWL1*Z*LAY2(I)-DBLE(L+I-1)*LAY1(I))/DBLE(L-I)
         ENDDO
         LAY3(L)=1.0D0
         LAY3(L-1)=Z*TWL1
        
         DO I=0,L
            LAY1(I)=LAY2(I)
            LAY2(I)=LAY3(I)
         ENDDO           
         
         J=J+L
         DO I=0,L
            J=J+1
            YLM(J)=LAY3(I)*YLM(J)
         ENDDO
      ENDDO
      
    
      IF(LMAX.GT.0)THEN
         YLM(2)=YLM(2)/SQ2
         YLM(4)=YLM(4)/SQ2
      ENDIF

      DO L=2,LMAX
         J=L*L+L
         DO M=0,L
            J=J+1
            R=1.0D0
            LMM=L-M
            DO I=1,M
              IKS=(2.0D0*DBLE(I)-1.0D0)
              R=R*IKS*IKS/((IKS+DBLE(LMM)+1.0D0)*(IKS+DBLE(LMM)))
            ENDDO
            YLM(J)=YLM(J)*DSQRT(R)
         ENDDO
      ENDDO     
    
      IL=DCMPLX(1.0D0,0.0D0)
      DO L=1,LMAX
         IL=IL*DCMPLX(0.0D0,1.0D0)
         J=L*L
         K=2*L+1+J
         DO M=L,1,-1
            J=J+1
            YLM(J)=YLM(K)*COPH(M)*IL
            K=K-1
         ENDDO
         J=J+1
         YLM(J)=YLM(J)*IL
         DO M=1,L
            J=J+1
            YLM(J)=YLM(J)*SIPH(M)*IL
         ENDDO
      ENDDO
      CALL FREEM(IP_COPH)
      CALL FREEM(IP_SIPH)
      CALL FREEM(IP_LAY1)
      CALL FREEM(IP_LAY2)
      CALL FREEM(IP_LAY3)
      END SUBROUTINE LOADALL 

      SUBROUTINE LOADBSL(NGW,NMAX,L,HG2,MESH,BSL)
      IMPLICIT NONE
C     ARGUMENTS
      INTEGER NGW, NMAX, L
      REAL*8  HG2(NGW), MESH(0:NMAX), BSL(NGW,1:NMAX)
C     VARIABLES
      INTEGER I, J, ISUB
      REAL*8 BESSL 
      EXTERNAL BESSL    
      CALL TISET('   LOADBSL',ISUB)
      
      DO I=1,NMAX
        DO J=1,NGW
          BSL(J,I)=BESSL(L,HG2(J)*MESH(I))
        ENDDO
      ENDDO
            
      CALL TIHALT('   LOADBSL',ISUB)
      END SUBROUTINE
      
      SUBROUTINE INTS40(DIGIT,N,S)
      IMPLICIT NONE
C     ARGUMENTS
      INTEGER N, S
      CHARACTER*40 DIGIT
C     VARIABLES
      INTEGER Y, R, I
      
      Y=N
      IF(Y.LE.0)THEN
         DIGIT(40:40)='0'
         S=1
      ELSE
         I=41
         DO WHILE(Y.GT.0)
           I=I-1
           R=MOD(Y,10)
           IF(R.EQ.0)THEN
              DIGIT(I:I)='0'
           ELSE IF(R.EQ.1)THEN
               DIGIT(I:I)='1'
           ELSE IF(R.EQ.2)THEN
               DIGIT(I:I)='2'
           ELSE IF(R.EQ.3)THEN
               DIGIT(I:I)='3'
           ELSE IF(R.EQ.4)THEN
               DIGIT(I:I)='4' 
           ELSE IF(R.EQ.5)THEN
                DIGIT(I:I)='5'
           ELSE IF(R.EQ.6)THEN
                DIGIT(I:I)='6'
           ELSE IF(R.EQ.7)THEN
                DIGIT(I:I)='7'
           ELSE IF(R.EQ.8)THEN
                DIGIT(I:I)='8'
           ELSE IF(R.EQ.9)THEN
                DIGIT(I:I)='9'
           ENDIF   
           Y=Y/10
         ENDDO
         S=41-I
      ENDIF
      END SUBROUTINE INTS40

      SUBROUTINE  WINT(N,MESH,RAD,W)
      IMPLICIT NONE
C     ARGUMENTS
      INTEGER N
      REAL*8 MESH(N), RAD(0:N), W
C     RESULT IN W
C     VARIABLES
      INTEGER  I
      REAL*8 RI, RI1, SUM               

      SUM=0.0D0
      RI=MESH(1)
      SUM=SUM+RI*RI*RAD(1)*RAD(1)*0.5
      !ZERO POINT
      DO I=1,N-1
         RI=MESH(I)
         RI1=MESH(I+1)
         SUM=SUM+(RAD(I)*RAD(I)*RI*RI+RAD(I+1)*RAD(I+1)*RI1*RI1)*0.5*
     &        (RI1-RI)
      ENDDO     
      W=SUM
      END SUBROUTINE  WINT

      SUBROUTINE INTSIMP(N,DR,RAD,W)
      IMPLICIT NONE
C     ARGUMENTS    
      INTEGER N
      REAL*8 RAD(0:N), DR, W      
C     RESULT IN W
C     VARIABLES     
      INTEGER I
      REAL*8 R1, R2, SUM, C43, C23    
      
      C43=4.0D0/3.0D0
      C23=2.0D0/3.0D0 
      R1=DR      
      R2=R1+DR
      SUM=0.0D0
      DO I=1,N-3,2
         SUM=SUM+C43*R1*R1*RAD(I)*RAD(I)+C23*R2*R2*RAD(I+1)*RAD(I+1)
         R1=R2+DR
         R2=R1+DR
      ENDDO     
      
      SUM=SUM+C43*R1*R1*RAD(N-1)*RAD(N-1)+
     &	      1.0D0/3.0D0*R2*R2*RAD(N)*RAD(N)
          
      SUM=SUM*DR     
      W=SUM 
      END SUBROUTINE INTSIMP

      SUBROUTINE MAXARR(NY,JMAX,JMAX2,W)
      IMPLICIT NONE
C     ARGUMENTS
      INTEGER NY,JMAX,JMAX2
      REAL*8 W(NY)
C     VARIABLES 
      INTEGER I
      REAL*8 MX, MX2
      
      IF(NY.NE.1)THEN
         IF(W(1)>W(2))THEN
            MX=W(1)
            JMAX=1    
            MX2=W(2)
            JMAX2=2            
         ELSE
            MX=W(2)
            JMAX=2  
            MX2=W(1)
            JMAX2=1  
         ENDIF
      ELSE
         MX=W(1)
         JMAX=1              
      ENDIF

      DO I=3,NY
         IF(W(I)>MX)THEN
            MX2=MX
            JMAX2=JMAX
            MX=W(I)            
            JMAX=I            
         ELSE IF(W(I)>MX2)THEN
            JMAX2=I
            MX2=W(I)
         ENDIF
      ENDDO
                  
      END  SUBROUTINE MAXARR
      





      
