C     ==================================================================
      SUBROUTINE PRPCMOVE_ISO(VELP,STEP)
C     ==--------------------------------------------------------------==
C     ==  ALLOWS FOR NOSE-HOOVER CHAIN DYNAMICS                       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nose.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'metr.inc'
C     Arguments
      REAL*8  VELP(3,NAX,NSX),STEP
C     Variables
      REAL*8  AT3,GEPS_P,GEPS_V,AA
      INTEGER K,IS,IA
C     ==--------------------------------------------------------------==
      AT3 = 3.D0*DBLE(NAT)
C     ==--------------------------------------------------------------==
C     ==  PROPAGATE BAROSTAT VELOCITY                                 ==
C     ==--------------------------------------------------------------==
      GEPS_P = HTFP(1,1) + HTFP(2,2) + HTFP(3,3)
      GEPS_V = 0.D0
      DO K=1,3
        DO IS=1,NSP
          DO IA=1,NA(IS)
            GEPS_V = GEPS_V + (1.D0 + 3.D0/AT3)*
     *           PMA(IS)*VELP(K,IA,IS)*VELP(K,IA,IS)
          ENDDO
        ENDDO
      ENDDO  
      VEPS = VEPS + 0.25D0*STEP*(GEPS_P + GEPS_V)/CMASS       
C     ==--------------------------------------------------------------==
C     ==  SCALE VELOCITIES                                            ==
C     ==--------------------------------------------------------------==
      AA = DEXP(-0.5D0*STEP*(1.D0+3.D0/AT3)*VEPS)
      DO IS=1,NSP
        DO IA=1,NA(IS)
          VELP(1,IA,IS) = VELP(1,IA,IS)*AA
          VELP(2,IA,IS) = VELP(2,IA,IS)*AA
          VELP(3,IA,IS) = VELP(3,IA,IS)*AA
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     ==  PROPAGATE BAROSTAT VELOCITY                                 ==
C     ==--------------------------------------------------------------==
      GEPS_P = HTFP(1,1) + HTFP(2,2) + HTFP(3,3)
      GEPS_V = 0.D0
      DO K=1,3
        DO IS=1,NSP
          DO IA=1,NA(IS)
            GEPS_V = GEPS_V + (1.D0 + 3.D0/AT3)*
     *           PMA(IS)*VELP(K,IA,IS)*VELP(K,IA,IS)
          ENDDO
        ENDDO
      ENDDO  
      VEPS = VEPS + 0.25D0*STEP*(GEPS_P + GEPS_V)/CMASS       
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PRPCMOVE(VELP,STEP)
C     ==--------------------------------------------------------------==
C     ==  ALLOWS FOR NOSE-HOOVER CHAIN DYNAMICS                       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nose.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'metr.inc'
C     Arguments
      REAL*8  VELP(3,NAX,NSX),STEP
C     Variables
      REAL*8  HTFV(3,3),SCMAT(3,3),SCMD(3),SCMEV(3,3),VTMP1(3),
     &        AT3,AA1,AA2,AA3,TRHV
      INTEGER IS,IA,K,L,IERR
C     ==--------------------------------------------------------------==
      AT3 = 3.D0*DBLE(NAT)
C     ==--------------------------------------------------------------==
C     ==  GET VELOCITY-DEPENDENT PART OF CELL FORCE                   ==
C     ==--------------------------------------------------------------==
      CALL AZZERO(HTFV,9)
      DO IS=1,NSP
        DO IA=1,NA(IS)
          DO K=1,3
            DO L=1,3
              HTFV(K,L) = HTFV(K,L) +
     &             PMA(IS)*VELP(K,IA,IS)*VELP(L,IA,IS)
            ENDDO
            HTFV(K,K) = HTFV(K,K) + 
     *           PMA(IS)*VELP(K,IA,IS)*VELP(K,IA,IS)/AT3
          ENDDO
        ENDDO
      ENDDO  
C     ==--------------------------------------------------------------==
C     ==  PROPAGATE BAROSTAT (CELL VELOCITY)                          ==
C     ==--------------------------------------------------------------==
      DO K=1,3
        DO L=1,3
          HTVEL(K,L) = HTVEL(K,L) +
     *         0.25D0*STEP*(HTFV(K,L) + HTFP(K,L))/CMASS
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     ==  FOR VELOCITY SCALING, FORM MATRIX v_h + Tr(v_h)/N + v_zeta1 ==
C     ==--------------------------------------------------------------==
      TRHV = HTVEL(1,1) + HTVEL(2,2) + HTVEL(3,3)
      DO K=1,3
        DO L=1,3
          SCMAT(K,L) = HTVEL(K,L)
        ENDDO
        SCMAT(K,K) = SCMAT(K,K) + TRHV/AT3
      ENDDO
C     ==--------------------------------------------------------------==
C     == DIAGONALIZE THIS MATRIX AND SAVE EIGENVALUES AND EIGENVECTORS==
C     ==--------------------------------------------------------------==
      CALL JACOBI(3,3,SCMAT,SCMD,SCMEV,IERR)
C     ==--------------------------------------------------------------==
C     ==  ROTATE, SCALE, AND ROTATE BACK FOR VELOCITIES               ==
C     ==--------------------------------------------------------------==
      AA1 = DEXP(-0.5D0*STEP*SCMD(1))
      AA2 = DEXP(-0.5D0*STEP*SCMD(2))
      AA3 = DEXP(-0.5D0*STEP*SCMD(3))
      DO IS=1,NSP
        DO IA=1,NA(IS)
C         
          VTMP1(1) = SCMEV(1,1)*VELP(1,IA,IS) + SCMEV(2,1)*VELP(2,IA,IS)
     *         + SCMEV(3,1)*VELP(3,IA,IS)
          VTMP1(2) = SCMEV(1,2)*VELP(1,IA,IS) + SCMEV(2,2)*VELP(2,IA,IS)
     *         + SCMEV(3,2)*VELP(3,IA,IS)
          VTMP1(3) = SCMEV(1,3)*VELP(1,IA,IS) + SCMEV(2,3)*VELP(2,IA,IS)
     *         + SCMEV(3,3)*VELP(3,IA,IS)
C         
          VTMP1(1) = VTMP1(1)*AA1
          VTMP1(2) = VTMP1(2)*AA2
          VTMP1(3) = VTMP1(3)*AA3
C         
          VELP(1,IA,IS) = SCMEV(1,1)*VTMP1(1) + SCMEV(1,2)*VTMP1(2)
     *         + SCMEV(1,3)*VTMP1(3)
          VELP(2,IA,IS) = SCMEV(2,1)*VTMP1(1) + SCMEV(2,2)*VTMP1(2)
     *         + SCMEV(2,3)*VTMP1(3)
          VELP(3,IA,IS) = SCMEV(3,1)*VTMP1(1) + SCMEV(3,2)*VTMP1(2)
     *         + SCMEV(3,3)*VTMP1(3)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     ==  GET VELOCITY-DEPENDENT PART OF CELL FORCE                   ==
C     ==--------------------------------------------------------------==
      CALL AZZERO(HTFV,9)
      DO IS=1,NSP
        DO IA=1,NA(IS)
          DO K=1,3
            DO L=1,3
              HTFV(K,L) = HTFV(K,L) +
     &             PMA(IS)*VELP(K,IA,IS)*VELP(L,IA,IS)
            ENDDO
            HTFV(K,K) = HTFV(K,K) + 
     *           PMA(IS)*VELP(K,IA,IS)*VELP(K,IA,IS)/AT3
          ENDDO
        ENDDO
      ENDDO  
C     ==--------------------------------------------------------------==
C     ==  PROPAGATE BAROSTAT (CELL VELOCITY)                          ==
C     ==--------------------------------------------------------------==
      DO K=1,3
        DO L=1,3
          HTVEL(K,L) = HTVEL(K,L) +
     *         0.25D0*STEP*(HTFV(K,L) + HTFP(K,L))/CMASS
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================



