C     ==================================================================
      SUBROUTINE RADIN(MESH,C,FUNC,ASUM)
C     ==--------------------------------------------------------------==
C     == SIMPSONS RULE INTEGRATION FOR HERMAN SKILLMAN MESH           ==
C     == MESH - # OF MESH POINTS                                      ==
C     == C    - 0.8853418/Z**(1/3.)                                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER  MESH
      REAL*8   FUNC(MESH),C,ASUM
C     Variables
      INTEGER  NBLOCK,I,J,K,I1
      REAL*8   A1,A2E,H,A2O,A2ES
C     ==--------------------------------------------------------------==
      A1=0.0D0
      A2E=0.0D0
      ASUM=0.0D0
      H=0.0025D0*C
      NBLOCK=MESH/40
      I=1
      FUNC(1)=0.0D0
      DO J=1,NBLOCK
        DO K=1,20
          I=I+2
          I1=I-1
          A2ES=A2E
          A2O=FUNC(I1)/12.D0
          A2E=FUNC(I)/12.D0
          A1=A1+5.D0*A2ES+8.D0*A2O-A2E
          FUNC(I1)=ASUM+A1*H
          A1=A1-A2ES+8.D0*A2O+5.D0*A2E
          FUNC(I)=ASUM+A1*H
        ENDDO
        ASUM=FUNC(I)
        A1=0.D0
        H=H+H
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE RADLG(MESH,FUNC,RAB,ASUM)
C     ==--------------------------------------------------------------==
c     == SIMPSONS RULE INTEGRATOR FOR FUNCTION STORED ON THE          ==
C     == RADIAL LOGARITHMIC MESH                                      ==
C     == RAB(MESH) : LOGARITHMIC RADIAL MESH INFORMATION              ==
C     == FUNC(MESH): FUNCTION TO BE INTEGRATED                        ==
C     ==--------------------------------------------------------------==
C     == ROUTINE ASSUMES THAT MESH IS AN ODD NUMBER SO RUN CHECK      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER MESH
      REAL*8  FUNC(MESH),RAB(MESH),ASUM
C     Variables
      INTEGER I
      REAL*8  R12,F3,F2,F1
C     ==--------------------------------------------------------------==
      IF ( MESH - ( MESH / 2 ) * 2 .NE. 1 ) THEN
        WRITE(6,*) '***ERROR IN SUBROUTINE RADLG'
        WRITE(6,*) 'ROUTINE ASSUMES MESH IS ODD BUT MESH =',MESH
        CALL STOPGM('RADLG',' ')
      ENDIF
C     ==--------------------------------------------------------------==
      ASUM = 0.0D0
      R12 = 1.0D0 / 12.0D0
      F3  = FUNC(1) * RAB(1) * R12
      FUNC(1) = 0.0D0
C     ==--------------------------------------------------------------==
      DO I = 2,MESH-1,2
        F1 = F3
        F2 = FUNC(I) * RAB(I) * R12
        F3 = FUNC(I+1) * RAB(I+1) * R12
        ASUM = ASUM + 5.0D0*F1 + 8.0D0*F2 - 1.0D0*F3
        FUNC(I) = ASUM
        ASUM = ASUM - 1.0D0*F1 + 8.0D0*F2 + 5.0D0*F3
        FUNC(I+1) = ASUM
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
