C     ==================================================================
      SUBROUTINE RATOM
C     ==--------------------------------------------------------------==
C     ==  Reads Section &ATOMS from Unit iunit                        ==
C     ==--------------------------------------------------------------==
C     ==  DESCRIPTION OF INPUT                                        ==
C     ==                                                              ==
C     ==  &ATOMS                                                      ==
C     ==                                                              ==
C     ==  *ECPNAME [OPTIONS]                                          ==
C     ==  {LMAX LOC SKIP , LMAX=L [LOC=L,SKIP=L]}                     ==
C     ==  NATOMS                                                      ==
C     ==    X     Y     Z                                             ==
C     ==    X     Y     Z                                             ==
C     ==    ....                                                      ==
C     ==                                                              ==
C     ==  ATOMIC CHARGES                                              ==
C     ==   c1 c2 ... cnsp                                             ==
C     ==                                                              ==
C     ==  MOVIE TYPE                                                  ==
C     ==   nt1 nt2 ...                                                ==
C     ==                                                              ==
C     ==  DUMMY ATOMS                                                 ==
C     ==   ndat                                                       ==
C     ==   TYPEx   x    y    z                                        ==
C     ==   TYPEy   n    n1 n2 n3 ...                                  ==
C     ==  ....                                                        ==
C     ==                                                              ==
C     ==  GENERATE COORDINATES                                        ==
C     ==   iun1  iun2  ...  iunn                                      ==
C     ==                                                              ==
C     ==  ISOTOPE                                                     ==
C     ==   mass1                                                      ==
C     ==   ....                                                       ==
C     ==                                                              ==
C     ==  CONFINEMENT POTENTIAL                                       ==
C     ==   a1 rc1                                                     ==
C     ==   ....                                                       ==
C     ==                                                              ==
C     ==  CHANGE BONDS                                                ==
C     ==    NBOAD                                                     ==
C     ==    I  J   +/- 1                                              ==
C     ==    ....                                                      ==
C     ==                                                              ==
C     ==  CONSTRAINTS                                                 ==
C     ==    ....                                                      ==
C     ==  END CONSTRAINTS                                             ==
C     ==                                                              ==
C     ==  META DYNAMICS {COLLECTIVE VARIABLE; FIX ; MULTI}            ==
C     ==                                                              ==
C     ==  SADDLE POINT                                                ==
C     ==                                                              ==
C     ==  VELOCITIES                                                  ==
C     ==    ....                                                      ==
C     ==  END VELOCITIES                                              ==
C     ==                                                              ==
C     ==  &END                                                        ==
C     ==   OPTIONS: GAUSS-HERMIT=n  |  KLEINMAN-BYLANDER              ==
C     ==            NLCC                                              ==
C     ==            RAGGIO=ragg                                       ==
C     ==            BINARY | FORMATTED                                ==
C     ==            UPF                                               ==
C     ==            NONE                                              ==
C     ==            CLASSIC                                           ==
C     ==            _EAM_                                             ==
C     ==            FRAC                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'coor.inc'
      INCLUDE 'dpot.inc'
      INCLUDE 'atom.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'movi.inc'
      INCLUDE 'nlcc.inc'
      INCLUDE 'ragg.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'symm.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'clas.inc'
      INCLUDE 'atwf.inc'
      INCLUDE 'mm_input.inc'
      INCLUDE 'mm_dim.inc'
      INCLUDE 'mw.inc'
      INCLUDE 'cnst_dyn.inc'
C     Variables
      DIMENSION ATCHG(*)
      INTEGER   I,J,INSCAN,IUNIT,IERR,IA,IE,I1,I2,I3,I4,I11,IOUT,
     &          IL,ILM,NGHL,NGHLN,IGH,K,IS,IAT,ILOC,ISKIP,LMAXG,IV
      REAL*8    RAGGNEW,NEL_EAM,NEL_SUM
      CHARACTER LINE*127,ECPNAM*40
      LOGICAL   ERREAD,TNONE,TUPF,TFRAC
      INTEGER   MSGLEN,NCLS,NCLA,NASP,NSX_q
C     ==--------------------------------------------------------------==
C     Initialization
      if(qmmm)then
         NSX_q=NSPq
         NSX=NSPm
      else
         NSX_q=NSX
         NSPm=NSX
      endif
C
      PCONF=.FALSE.
C
      CALL MEMORY(IP_LSKCOR,3*NAX*NSX,'LSKCOR')
      CALL MEMORY(IP_LSKPTR,3*NAX*NSX,'LSKPTR')
      CALL MEMORY(IP_ATCHG,NSX_q,'ATCHG')
      CALL AZZERO(ATCHG,NSX_q)
      CALL IAZZERO(LSKPTR,3*NAX*NSX)
      DO I=1,NAX*NSX
        DO J=1,3
          LSKCOR(J,I)=1
        ENDDO
      ENDDO
C     Velocities
cMcB
      IF ( NTX_vel.ne.1 ) THEN
         CALL AZZERO(VELP,3*NAX*NSX)
      ENDIF 
cMcB
C     Real space function and grids for Kleinman-Bylander + Potentials
      CALL AZZERO(GNL,MMAXX*NSX_q*LMAXX)
      CALL AZZERO(RPS,MMAXX*NSX_q*LMAXX)
      CALL AZZERO(VR,MMAXX*NSX_q*LMAXX)
      CALL AZZERO(RW,MMAXX*NSX_q)
      CALL AZZERO(RV,MMAXX*NSX_q)
C     ==--------------------------------------------------------------==
      IF(.NOT.PARENT) GOTO 9999
C     ==--------------------------------------------------------------==
      IUNIT = 5
      IERR=INSCAN(IUNIT,'&ATOMS')
      IF(IERR.NE.0) THEN
        WRITE(6,'(A,A,I3)')
     &       ' RATOM| COULD NOT FIND SECTION &ATOMS ON  UNIT',IUNIT
        CALL STOPGM('RATOM',' ')
      ENDIF
C     ==--------------------------------------------------------------==
      DO I=1,MAXSP
        IMTYP(I)=0
        TEAM(I)=.FALSE.
        TKB(I)=.FALSE.
      ENDDO
      LFCOM=.FALSE.
      TGENC=.FALSE.
      NSP=0
      NEL=0.D0
      NEL_EAM=0.0D0
      NEL_SUM=0.0D0
      NDAT  = 0
      NDAT1 = 0
      NDAT2 = 0
      NDAT3 = 0
      MCNSTR = 0
      MRESTR = 0
      NCLS = 0
      NCLA = 0
C     ==--------------------------------------------------------------==
   10 CONTINUE
      READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
      IF(INDEX(LINE,'&END').NE.0) GOTO 30
      IF(LINE(1:1).NE.'*') GOTO 10
C     NEW SPECIES found
      CALL XSTRING(LINE,IA,IE)
      ECPNAM=LINE(2:IE)
      IA=IE+1
      IE=LEN(LINE)
      IF(IE.LT.IA) IE=IA

      NCLS=NCLS+1
      IF(INDEX(LINE(IA:IE),'CLASSIC').NE.0) THEN
        I1=IA-1+INDEX(LINE(IA:IE),'LABEL')
        IF(I1.GE.IA) THEN
          I11=I1+INDEX(LINE(I1:127),'=')
          CLAB(NCLS)(1:4)=LINE(I11:I11+3)
        ELSE
          CALL STOPGM('RATOM','NO LABEL FOR CLASSICAL ATOM')
        ENDIF
        I1=IA-1+INDEX(LINE(IA:IE),'MASS')
        IF(I1.GE.IA) THEN
          I11=I1+INDEX(LINE(I1:127),'=')
          CALL READSR(LINE(I11:127),1,IOUT,CLMAS(NCLS),ERREAD)
        ELSE
          CALL STOPGM('RATOM','NO MASS FOR CLASSICAL ATOM')
        ENDIF
        READ(IUNIT,END=20,ERR=20,FMT=*) NASP
        NCRANG(1,NCLS)=NCLA+1
        NCLA=NCLA+NASP
        NCRANG(2,NCLS)=NCLA
        DO I=NCRANG(1,NCLS),NCRANG(2,NCLS)
          READ(IUNIT,END=20,ERR=20,FMT=*) (CLASC(K,I),K=1,3)
          CLTYP(I)=NCLS
        ENDDO
        IS_QM(NCLS)=0
      ELSEIF(INDEX(LINE(IA:IE),'_EAM_').NE.0) THEN
          NSP=NSP+1
          TEAM(NSP)=.TRUE.
          TNONE=.FALSE.
          TVAN(NSP)=.FALSE.
          TBIN(NSP)=.FALSE.
          TNLCC(NSP)=.FALSE.
          TKB(NSP)=.FALSE.
          NGHL=0
          LMAX(NSP)=0
          LLOC(NSP)=0
          LSKIP(NSP)=1
          NGH(NSP)=0
          CALL RECPEAM(NSP,ECPNAM)
          READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C     ATOMIC COORDINATES
          READ(IUNIT,END=20,ERR=20,FMT=*) NA(NSP)
          DO I=1,NA(NSP)
            READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
            IA=1
            DO K=1,3
              CALL READSR(LINE,IA,IE,TAU0(K,I,NSP),ERREAD)
              IF(ERREAD) GOTO 20
              IA=IE+1
            ENDDO
          ENDDO
      ELSE
          NSP=NSP+1
          RAGGNEW=-1.D0
          TNONE=.FALSE.
          TFRAC=.FALSE.
          IF(INDEX(LINE(IA:IE),'NONE').NE.0) TNONE=.TRUE.
          IF(INDEX(LINE(IA:IE),'FRAC').NE.0) TFRAC=.TRUE.
          TUPF=.FALSE.
          IF(INDEX(LINE(IA:IE),'UPF').NE.0) TUPF=.TRUE.
          TVAN(NSP)=.FALSE.
          TBIN(NSP)=.FALSE.
          IF(INDEX(LINE(IA:IE),'FORMAT').NE.0) TVAN(NSP)=.TRUE.
          IF(INDEX(LINE(IA:IE),'BINARY').NE.0) THEN
            IF (TVAN(NSP))
     &         CALL STOPGM('RATOM','BINARY or FORMATTED uspp file?')
            TVAN(NSP)=.TRUE.
            TBIN(NSP)=.TRUE.
          ENDIF
          TNLCC(NSP)=.FALSE.
          IF(INDEX(LINE(IA:IE),'NLCC').NE.0) TNLCC(NSP)=.TRUE.
          NGHL=20
          I1=IA-1+INDEX(LINE(IA:IE),'GAUSS')
          I2=IA-1+INDEX(LINE(IA:IE),'KLEINMAN')
          IF(I1.GE.IA) THEN
            TKB(NSP)=.FALSE.
            I11=I1+INDEX(LINE(I1:127),'=')
            IF(I11.GT.I1) 
     &         CALL READSI(LINE(I11:127),1,IOUT,NGHL,ERREAD)
          ENDIF
          IF(I2.GT.IA) TKB(NSP)=.TRUE.
          IF(I1.EQ.(IA-1) .AND. I2.EQ.(IA-1)) THEN
            TKB(NSP)=.FALSE.
          ENDIF
          I3=IA-1+INDEX(LINE(IA:IE),'RAGGIO')
          IF(I3.GE.IA) THEN
            I4=I3+INDEX(LINE(I3:127),'=')
            CALL READSR(LINE(I4:127),1,IOUT,RAGGNEW,ERREAD)
          ENDIF
          CALL LREAD(IUNIT,LMAX(NSP),ILOC,ISKIP)
          LLOC(NSP)=LMAX(NSP)+1
          LSKIP(NSP)=LMAX(NSP)+2
          NGH(NSP)=0
          IF(TKB(NSP)) THEN
            IF(ILOC.GE.0.AND.ILOC.LE.LMAX(NSP)) LLOC(NSP)=ILOC+1
            IF(ISKIP.GE.0.AND.ISKIP.LE.LMAX(NSP)) LSKIP(NSP)=ISKIP+1
          ELSEIF(I1.NE.0) THEN
            CALL GHERMIT(NGHL,RGH(1,NSP),WGH(1,NSP))
            NGHLN=0
            DO I=1,NGHL
              IF(WGH(I,NSP).GT.1.D-10) THEN
                NGHLN=NGHLN+1
                WGH(NGHLN,NSP)=WGH(I,NSP)
                RGH(NGHLN,NSP)=RGH(I,NSP)
              ENDIF
            ENDDO
            NGHL=NGHLN
            RMAXN(NSP)=DBLE(NGHL)
            IF(ILOC.GE.0.AND.ILOC.LE.LMAX(NSP)) LLOC(NSP)=ILOC+1
            IF(ISKIP.GE.0.AND.ISKIP.LE.LMAX(NSP)) LSKIP(NSP)=ISKIP+1
            LMAXG=LMAX(NSP)+1
            IF(LMAXG.EQ.LLOC(NSP)) LMAXG=LMAX(NSP)
            IV=0
            DO IL=1,LMAXG
              DO ILM=1,2*IL-1
                DO IGH=1,NGHL
                  IV=IV+1
                  NGHTOL(IV,NSP)=IL-1
                ENDDO
              ENDDO
            ENDDO
            NGH(NSP)=IV
          ENDIF
          LMAX(NSP)=LMAX(NSP)+1
          IF(TCLAS) THEN
            I1=IA-1+INDEX(LINE(IA:IE),'LABEL')
            IF(I1.GE.IA) THEN
              I11=I1+INDEX(LINE(I1:127),'=')
              CLAB(NCLS)(1:4)=LINE(I11:I11+3)
            ELSE
              CALL STOPGM('RATOM','NO LABEL FOR QUANTUM ATOM')
            ENDIF
          ENDIF
C     PSEUDOPOTENTIAL
          if(qmmm)NSX=NSPq
          IF(TNONE) THEN
            CALL ALLELEC(NSP,ECPNAM)
          ELSEIF(TUPF) THEN
            CALL RECPUPF(NSP,ECPNAM)
          ELSE
            CALL RECPNEW(NSP,ECPNAM)
          ENDIF
          if(qmmm)NSX=NSPm
          IF(RAGGNEW.GT.0.D0) RAGGIO(NSP)=RAGGNEW
C     ATOMIC COORDINATES
          READ(IUNIT,END=20,ERR=20,FMT=*) IAT
          IF(IAT.NE.NA(NSP)) CALL STOPGM('RATOM',
     &         'INCONSISTENT READ OF &ATOMS SECTION')
          IF(.NOT.TEAM(NSP)) THEN
C            FRACTIONAL CORE CHARGE?
             IF(TFRAC) THEN
                NEL_SUM=NEL_SUM+DBLE(NA(NSP))*ZV(NSP)
             ELSE
                NEL=NEL+DBLE(NA(NSP)*NINT(ZV(NSP)))
             ENDIF
          ELSE
             NEL_EAM=NEL_EAM+DBLE(NA(NSP))*ZV(NSP)
          ENDIF
          if(.not. qmmm)then
            DO I=1,NA(NSP)
              READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
              IA=1
              DO K=1,3
                CALL READSR(LINE,IA,IE,TAU0(K,I,NSP),ERREAD)
                IF(ERREAD) GOTO 20
                IA=IE+1
              ENDDO
            ENDDO
          endif
          IF(TCLAS) THEN
            NCRANG(1,NCLS)=NCLA+1
            NCLA=NCLA+NA(NSP)
            NCRANG(2,NCLS)=NCLA
            DO I=NCRANG(1,NCLS),NCRANG(2,NCLS)
              CLTYP(I)=NCLS
            ENDDO
            IS_QM(NCLS)=NSP
          ENDIF
        ENDIF
        GOTO 10
C     ==--------------------------------------------------------------==
 20   CONTINUE
      WRITE(6,*) ' RATOM| ERROR WHILE READING ON UNIT ',IUNIT
      CALL STOPGM('RATOM',' ')
C     ==--------------------------------------------------------------==
 30   CONTINUE
C     ==--------------------------------------------------------------==
C     HANDLE FRACTIONAL CORE CHARGES
      IF(DABS(DBLE(NINT(NEL_SUM))-NEL_SUM).GT.1.0D-10) THEN
         WRITE(6,'(/,A,/,A,F20.10,/,A,I9,/)')
     *      ' RATOM| WARNING! SUM OF CORE CHARGES IS NOT INTEGER',
     *      ' RATOM| SUM OF FRACTIONAL CORE CHARGES IS:', NEL_SUM,
     *      ' RATOM| NUMBER OF ELECTRONS ADDED IS:     ', NINT(NEL_SUM)
      ENDIF
      NEL=NEL+DBLE(NINT(NEL_SUM))
C     ==--------------------------------------------------------------==
      NBOAD=0
      IERR=INSCAN(IUNIT,'&ATOMS')
C     ==--------------------------------------------------------------==
!     ==--------------------------------------------------------------==
!     ==  Metadynamics Defaults
      CALL META_DYN_DEF
!     ==--------------------------------------------------------------==
 40   CONTINUE

      READ(IUNIT,END=50,ERR=50,FMT='(A)') LINE
      IF(INDEX(LINE,'&END').NE.0) GOTO 60
      IF(INDEX(LINE,'ISOTOPE').NE.0) THEN
C       READ ISOTOPIC MASSES
        DO I=1,NSP
          READ(IUNIT,END=50,ERR=50,FMT=*) PMA0(I)
        ENDDO
        GOTO 40
      ELSEIF(INDEX(LINE,'ATOMIC').NE.0 .AND. 
     *       INDEX(LINE,'CHARGE').NE.0) THEN
        READ(IUNIT,END=50,ERR=50,FMT=*) (ATCHG(IA),IA=1,NSX_q)
        GOTO 40
      ELSEIF(INDEX(LINE,'CONFIN').NE.0 .AND. 
     *       INDEX(LINE,'POTENT').NE.0) THEN
C       Confinement potentials
        PCONF=.TRUE.
        DO I=1,NSP
          READ(IUNIT,END=50,ERR=50,FMT=*) PALPHA(I),PRC(I)
        ENDDO
        GOTO 40
      ELSEIF(INDEX(LINE,'MOVIE').NE.0 .AND. 
     *       INDEX(LINE,'TYPE').NE.0) THEN
C       CHANGE DEFAULT MOVIE TYPE
        READ(IUNIT,END=50,ERR=50,FMT=*) (IMTYP(IS),IS=1,NSP)
        GOTO 40
      ELSEIF(INDEX(LINE,'GENER').NE.0 .AND. 
     *       INDEX(LINE,'COORD').NE.0) THEN
C       GENERATE ATOMIC COORDINATES FROM SYMMETRY UNIQUE ONES
        READ(IUNIT,END=50,ERR=50,FMT=*) (IUN(IS),IS=1,NSP)
        TGENC=.TRUE.
        GOTO 40
      ELSEIF(INDEX(LINE,'DUMMY').NE.0 .AND. 
     *       INDEX(LINE,'ATOM').NE.0) THEN
C       READ DUMMY ATOM DEFINITION
        READ(IUNIT,END=50,ERR=50,FMT=*) NDAT
        IF(NDAT.GT.MAXDUM) THEN
          WRITE(6,*) ' TOO MANY DUMMY ATOMS, CHANGE MAXDUM',
     *         ' IN INCLUDE FILE COTR.INC'
          CALL STOPGM('RATOM',' ')
        ENDIF
        DO IAT=1,NDAT
          READ(IUNIT,END=50,ERR=50,FMT='(A)') LINE
          CALL RDUMAT(LINE,IAT)
        ENDDO
        GOTO 40
      ELSEIF(INDEX(LINE,'CHANGE').NE.0 .AND. 
     *       INDEX(LINE,'BONDS').NE.0) THEN
C       Add or delete bonds for the empirical Hessian
        READ(IUNIT,END=50,ERR=50,FMT=*) NBOAD
        CALL MEMORY(IP_BOAD,3*NBOAD,'BOAD')
        DO I=1,NBOAD
          READ(IUNIT,END=50,ERR=50,FMT=*) BOAD(1,I),BOAD(2,I),
     *         BOAD(3,I)
        ENDDO
        GOTO 40
      ELSEIF(INDEX(LINE,'CONSTRAINT').NE.0) THEN
C       Input constraints on the geometry
        CALL CONINP(IUNIT)
        GOTO 40
      ELSEIF(INDEX(LINE,'META').NE.0 .AND.
     &       INDEX(LINE,'DYNAMICS').NE.0) THEN
        IF(INDEX(LINE,'COLLE').NE.0 .AND.
     &       INDEX(LINE,'VAR').NE.0) THEN
C      Input for the meta dynamics in the space of the Collective Variables
          CALL META_COLVAR_INP(IUNIT)
        ELSEIF(INDEX(LINE,'MULTI').NE.0) THEN
C         Input for the meta dynamics with multiple walkers
          IF(INDEX(LINE,'WALKERS').NE.0)THEN
             TMW=.TRUE. 
             IL=INDEX(LINE,'NW=')
             IF(IL.NE.0) THEN
               IA = IL+3
               CALL READSI(LINE,IA,IE,NWALK,ERREAD)
             ELSE 
               CALL STOPGM('RATOM','NUMBER OF WALKERS NOT SPECIFIED')
             ENDIF
             CALL META_COLVAR_INP(IUNIT)
          ELSE
C         Input for the meta dynamics in the space of the Collective Variables
C         more than one set of CV is activated
             TMULTI = .TRUE.
             IL=INDEX(LINE,'NS=')
             IF(IL.NE.0) THEN
               IA = IL+3
               CALL READSI(LINE,IA,IE,NSUBSYS,   ERREAD)
             ENDIF
             CALL META_COLVAR_INP(IUNIT)
          END IF
        ELSEIF(INDEX(LINE,'CELL').NE.0) THEN
C      Input for the metadynamics of the cell (only if TPRCP)
          CALL META_CELL_INP(IUNIT,LINE)
        ENDIF
        GOTO 40
      ELSEIF(INDEX(LINE,'SADDLE').NE.0 .AND. 
     &       INDEX(LINE,'POINT') .NE. 0) THEN
        CALL TST2MIN_INP(IUNIT)
        GOTO 40
      ELSEIF(INDEX(LINE,'VELOC').NE.0) THEN
C       Input velocities
        CALL VELOCITINP(IUNIT)
        GOTO 40
      ELSE
        GOTO 40
      ENDIF
 50   CONTINUE
      WRITE(6,*) ' RATOM| ERROR WHILE READING KEYWORDS'
      WRITE(6,*) LINE
      CALL STOPGM('RATOM',' ')
C     ==--------------------------------------------------------------==
 60   CONTINUE
C     ==--------------------------------------------------------------==
      IF(ABS(NEL_EAM).GT.1.D-10) THEN
         CALL STOPGM('RATOM','EAM charges do not add up to zero')
      ENDIF
9999  CONTINUE
C     ==--------------------------------------------------------------==
C     COTR
      MSGLEN = 6 * 8/IRAT
      CALL MY_BCAST(LFCOM,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 2 * 8/IRAT
      CALL MY_BCAST(MRESTR,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 3*NAX*NSPm * 8/IRAT
      CALL MY_BCAST(LSKCOR,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 3*NAX*NSPm * 8/IRAT
      CALL MY_BCAST(LSKPTR,MSGLEN,SOURCE,ALLGRP)
C     ATOM
      MSGLEN = MAXSP * 8
      CALL MY_BCAST(CLOGVP,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = MAXSP * 8
      CALL MY_BCAST(CLOGW,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = MAXSP * 8/IRAT
      CALL MY_BCAST(MESHVP,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = MAXSP * 8/IRAT
      CALL MY_BCAST(MESHW,MSGLEN,SOURCE,ALLGRP)
C     NLPS
      MSGLEN = MAXSP * 8
      CALL MY_BCAST(RMAXN,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = NSX_q*NHX * 8
      CALL MY_BCAST(RGH,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = NSX_q*NHX * 8
      CALL MY_BCAST(WGH,MSGLEN,SOURCE,ALLGRP)
C     NGH/NGHTOL
      MSGLEN = NSX_q*NHX * 8/IRAT
      CALL MY_BCAST(NGHTOL,MSGLEN,SOURCE,ALLGRP)
C     NGH/NGHCOM 
      MSGLEN = NSX_q*NHX * 8/IRAT
      CALL MY_BCAST(NGHCOM,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = MAXSP * 8/IRAT
      CALL MY_BCAST(NGH,MSGLEN,SOURCE,ALLGRP)
C     ATCHG
      MSGLEN = NSX_q * 8
      CALL MY_BCAST(ATCHG,MSGLEN,SOURCE,ALLGRP)
C     CLASSICAL ATOMS
      IF(TCLAS) THEN
        MSGLEN = MAXCLT * 8/IRAT
        CALL MY_BCAST(IS_QM,MSGLEN,SOURCE,ALLGRP)
        MSGLEN = NCLATOM * 8/IRAT
        CALL MY_BCAST(CLTYP,MSGLEN,SOURCE,ALLGRP)
        MSGLEN = 2*MAXCLT * 8/IRAT
        CALL MY_BCAST(NCRANG,MSGLEN,SOURCE,ALLGRP)
        MSGLEN = MAXCLT * 8
        CALL MY_BCAST(CLMAS,MSGLEN,SOURCE,ALLGRP)
        MSGLEN = 3*NCLATOM * 8
        CALL MY_BCAST(CLASC,MSGLEN,SOURCE,ALLGRP)
      ENDIF
C     CONFINEMENT POTENTIAL
      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(PCONF,MSGLEN,SOURCE,ALLGRP)
      IF(PCONF) THEN
        MSGLEN = 2*MAXSP * 8
        CALL MY_BCAST(PALPHA(1),MSGLEN,SOURCE,ALLGRP)
      ENDIF
C     META DYNAMICS OF ORDER PARAMETERS
      MSGLEN = MAXLMETA * 8/IRAT
      CALL MY_BCAST(LCNSTDYN,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = MAXIMETA * 8/IRAT
      CALL MY_BCAST(I_META_MAX,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = MAXRMETA * 8
      CALL MY_BCAST(HLLW,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = MAXLCV * 8/IRAT
      CALL MY_BCAST(NCOLVAR,MSGLEN,SOURCE,ALLGRP)
      IF(TLOCALIZESPIN) THEN
        IF(.NOT. PARENT) THEN
          CALL MEMORY(IP_ICV_SPIN,NCOLVAR/IRAT+1,'ICV_SPIN')
          CALL IAZZERO(ICV_SPIN,NCOLVAR)
          CALL MEMORY(IP_RCC0,3*NCOLVAR,'RCC0')
          CALL AZZERO(RCC0,3*NCOLVAR)
          CALL MEMORY(IP_KHARM,NCOLVAR,'KHARM')
          CALL AZZERO(KHARM,NCOLVAR)
        ENDIF
        
        MSGLEN = NCOLVAR * 8/IRAT
        CALL MY_BCAST(ICV_SPIN,MSGLEN,SOURCE,ALLGRP)
        MSGLEN = 3*NCOLVAR * 8
        CALL MY_BCAST(RCC0,MSGLEN,SOURCE,ALLGRP)
        MSGLEN = NCOLVAR * 8
        CALL MY_BCAST(KHARM,MSGLEN,SOURCE,ALLGRP)
      ENDIF
C     MULTIPLE WALKER METADYNAMICS
      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(TMW,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 2 * 8/IRAT
      CALL MY_BCAST(NWALK,MSGLEN,SOURCE,ALLGRP)
C     ==--------------------------------------------------------------==
      if(qmmm)then
         NSX=NSPq
      endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LREAD(IUNIT,LMAX,LOC,ISKIP)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   IUNIT,LMAX,ISKIP
C     Variables
      INTEGER   I1,I2,IOUT,IIN,LVAL,LOC
      CHARACTER LINE*80
      LOGICAL   ERREAD
C     ==--------------------------------------------------------------==
      READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
      IF(INDEX(LINE,'LMAX').NE.0) THEN
        I1=INDEX(LINE,'LMAX')
        I2=I1+INDEX(LINE(I1:80),'=')
        LMAX=LVAL(LINE(I2:I2))
        IF(INDEX(LINE,'LOC').NE.0) THEN
          I1=INDEX(LINE,'LOC')
          I2=I1+INDEX(LINE(I1:80),'=')
          LOC=LVAL(LINE(I2:I2))
        ELSE
          LOC=-1
        ENDIF
        IF(INDEX(LINE,'SKIP').NE.0) THEN
          I1=INDEX(LINE,'SKIP')
          I2=I1+INDEX(LINE(I1:80),'=')
          ISKIP=LVAL(LINE(I2:I2))
        ELSE
          ISKIP=-1
        ENDIF
      ELSE
        CALL READSI(LINE,1,IOUT,LMAX,ERREAD)
        IIN=IOUT
        CALL READSI(LINE,IIN,IOUT,LOC,ERREAD)
        IIN=IOUT
        CALL READSI(LINE,IIN,IOUT,ISKIP,ERREAD)
      ENDIF
      GOTO 30
   20 CONTINUE
        WRITE(6,*) ' RATOM: ERROR WHILE READING ON UNIT ',IUNIT
        CALL STOPGM('RATOM',' ')
   30 CONTINUE
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION LVAL(L)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      CHARACTER L*1
      INTEGER   LVAL
C     ==--------------------------------------------------------------==
      LVAL=-1
      IF(L.EQ.'S') LVAL=0
      IF(L.EQ.'P') LVAL=1
      IF(L.EQ.'D') LVAL=2
      IF(L.EQ.'F') LVAL=3
      IF(L.EQ.'G') LVAL=4
      IF(L.EQ.'H') LVAL=5
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE RDUMAT(LINE,IAT)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'adat.inc'
      INCLUDE 'cotr.inc'
C     Arguments
      CHARACTER LINE*80
      INTEGER   IAT,I1,K,IOUT
C     Variables
      LOGICAL   ERREAD
C     ==--------------------------------------------------------------==
      IF(INDEX(LINE,'TYPE1').NE.0) THEN
        I1=INDEX(LINE,'TYPE1')+5
        NDAT1=NDAT1+1
        LISTDA(IAT,1)=1
        LISTDA(IAT,2)=NDAT1
        DO K=1,3
          CALL READSR(LINE,I1,IOUT,DUMMY1(K,NDAT1),ERREAD)
          I1=IOUT
        ENDDO
      ELSEIF(INDEX(LINE,'TYPE2').NE.0) THEN
        I1=INDEX(LINE,'TYPE2')+5
        NDAT2=NDAT2+1
        LISTDA(IAT,1)=2
        LISTDA(IAT,2)=NDAT2
        CALL READSI(LINE,I1,IOUT,LISTD2(1,NDAT2),ERREAD)
        DO K=1,LISTD2(1,NDAT2)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,LISTD2(K+1,NDAT2),ERREAD)
        ENDDO
      ELSEIF(INDEX(LINE,'TYPE3').NE.0) THEN
        I1=INDEX(LINE,'TYPE3')+5
        NDAT3=NDAT3+1
        LISTDA(IAT,1)=3
        LISTDA(IAT,2)=NDAT3
        CALL READSI(LINE,I1,IOUT,LISTD3(1,NDAT3),ERREAD)
        DO K=1,LISTD3(1,NDAT3)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,LISTD3(K+1,NDAT3),ERREAD)
        ENDDO
      ELSE
        WRITE(6,*) ' ERROR IN DUMMY ATOM INPUT LINE ',IAT
        CALL STOPGM('RATOM',' ')
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
