C     ==================================================================
      SUBROUTINE RECPUPF(ISP,ECPNAM)
C     ==--------------------------------------------------------------==
C     ==  Reads Pseudopotential Input (UPF Format)                    ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     == ISP     Species index                                        ==
C     == ECPNAM  Filename of pseudopotential file                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'atom.inc'
      INCLUDE 'adat.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'ragg.inc'
      INCLUDE 'dpot.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'func.inc'
      INCLUDE 'nlcc.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'vdbt.inc'
      INCLUDE 'vdbp.inc'
      INCLUDE 'sgpp.inc'
C     Arguments
      INTEGER   ISP
      CHARACTER ECPNAM*(*)
C     Variables
      INTEGER   IUNIT
      PARAMETER (IUNIT=21)
      INTEGER   IA,IE,IERR,INSCAN,IOUT,IT,I,J,K,L,M,
     &          IV,JV,IR,LP,LENECP,MESHV,NPROJE,NWFN,IBC
      REAL*8    DIJ
      REAL*8,ALLOCATABLE,DIMENSION(:)::    DIJ_TMP
      CHARACTER LINE*80,ECPLIB*120,FNAMES*200,ENDC*1
      CHARACTER LONGLINE*5000
      LOGICAL   EXISTS,ERREAD,UPF2
C     ==--------------------------------------------------------------==
      TSGP(ISP)=.FALSE.
      TNUM(ISP)=.FALSE.
      TLOG(ISP)=.FALSE.
      CALL GET_PPLIB(ECPLIB,LENECP)
      CALL XSTRING(ECPNAM,IA,IE)
      FNAMES=ECPLIB(1:LENECP)//ECPNAM(IA:IE)
      INQUIRE(FILE=FNAMES,EXIST=EXISTS)
      IF(.NOT.EXISTS) THEN
          FNAMES=ECPNAM(IA:IE)
          INQUIRE(FILE=FNAMES,EXIST=EXISTS)
          IF(.NOT.EXISTS) THEN
             WRITE(6,*) ' RECPUPF| ECPFILE NOT FOUND ',FNAMES
             CALL STOPGM('RECPUPF',' ')
          ENDIF
      ENDIF
      ECPFILES(ISP)=FNAMES
      OPEN(UNIT=IUNIT,FILE=FNAMES,STATUS='OLD')
      REWIND(IUNIT)
C     ==--------------------------------------------------------------==
C..Determine which UPF version we are using
      UPF2=.TRUE.
      IERR=INSCAN(IUNIT,'<UPF ')
      IF(IERR.NE.0) THEN
        UPF2=.FALSE.
      END IF
C     ==--------------------------------------------------------------==
C..The info section: no difference between V1 and V2
      IERR=INSCAN(IUNIT,'<PP_INFO>')
      WRITE(VDBTI(1,ISP),'(66("+"))')
      VDBTI(2,ISP) = ECPNAM(IA:IE)
      WRITE(VDBTI(3,ISP),'(66("+"))')
      IT=3
      DO I=1,60
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
        IF(INDEX(LINE,'</PP_INFO>').NE.0) GOTO 10
        IT=IT+1
        VDBTI(IT,ISP)=LINE(1:66)
      ENDDO
   10 CONTINUE
      IT=IT+1
      WRITE(VDBTI(IT,ISP),'(66("+"))')
      ITMAX(ISP)=IT
C     ==--------------------------------------------------------------==
C..General info about PP and atom
      IF (UPF2) THEN
        CALL GET_SECTION_INFO(IUNIT,'<PP_HEADER',LONGLINE,
     *                        LEN(LONGLINE))
C       Element
        CALL GET_KEY(LONGLINE,"element",LINE)
        CALL XSTRING(LINE,IA,IE)
        CALL FIND_ELEMENT(LINE(IA:IE),IATYP(ISP))
C       Ultrasoft|Normconserving
        CALL GET_KEY(LONGLINE,"is_ultrasoft",LINE)
        IF(INDEX(LINE,"T").NE.0) TVAN(ISP)=.TRUE.
        IF(TVAN(ISP)) THEN
          CALL STOPGM('RECPUPF','ULTRASOFT UPF FILES NOT SUPPORTED')
        ENDIF
C       NLCC
        CALL GET_KEY(LONGLINE,"core_correction",LINE)
        CALL XSTRING(LINE,IA,IE)
        IF(LINE(IA:IA).EQ."T") TNLCC(ISP)=.TRUE.
C       Valence charge
        CALL GET_KEY(LONGLINE,"z_valence",LINE)
        CALL READSR(LINE,1,IOUT,ZV(ISP),ERREAD)
C       LMAX
        CALL GET_KEY(LONGLINE,"l_max",LINE)
        CALL READSI(LINE,1,IOUT,LMAX(ISP),ERREAD)
        LMAX(ISP)=LMAX(ISP)+1
        LLOC(ISP)=LMAX(ISP)
        LSKIP(ISP)=LMAX(ISP)+5
        TSGP(ISP)=.TRUE.
        TNUM(ISP)=.TRUE.
        TKB(ISP)=.FALSE.
        IGAU(ISP)=0
C       Mesh points
        CALL GET_KEY(LONGLINE,"mesh_size",LINE)
        CALL READSI(LINE,1,IOUT,MESHV,ERREAD)
        IF(MESHV.GT.MMAXX) THEN
          CALL XSTRING(ECPNAM,IA,IE)
          WRITE(6,*) ' RECPUPF! MESH FOR ',ECPNAM(IA:IE),' IS ',
     &                 MESHV
          WRITE(6,*) ' RECPUPF! MAX NUMBER OF SPLINE POINTS:',MMAXX
          WRITE(6,*) ' RECPUPF! INCREASE SPLINE POINTS NUMBER'
          CALL STOPGM('RECPUPF','MESH TOO BIG')
        ENDIF
        MESHVP(ISP)=MESHV
C       Number of wavefunctions, projectors
        CALL GET_KEY(LONGLINE,"number_of_wfc",LINE)
        CALL READSI(LINE,1,IOUT,NWFN,ERREAD)
        CALL GET_KEY(LONGLINE,"number_of_proj",LINE)
        CALL READSI(LINE,1,IOUT,NPROJE,ERREAD)
      ELSE
        IERR=INSCAN(IUNIT,'<PP_HEADER>')
        IF(IERR.NE.0) THEN
          WRITE(6,*) ' RECPUPF| PP_HEADER SECTION NOT FOUND '
          CALL STOPGM('RECPUPF',' ')
        ENDIF
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C       Version number (not used)
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C       Element
        CALL XSTRING(LINE,IA,IE)
        CALL FIND_ELEMENT(LINE(IA:IE),IATYP(ISP))
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C       Ultrasoft|Normconserving
        IF(INDEX(LINE,"US").NE.0) TVAN(ISP)=.TRUE.
        IF(TVAN(ISP)) THEN
          CALL STOPGM('RECPUPF','ULTRASOFT UPF FILES NOT SUPPORTED')
        ENDIF
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C       NLCC
        CALL XSTRING(LINE,IA,IE)
        IF(LINE(IA:IA).EQ."T") TNLCC(ISP)=.TRUE.
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C       DFT FUNCTIONAL (not used)
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C       Valence charge
        CALL READSR(LINE,1,IOUT,ZV(ISP),ERREAD)
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C       Total energy (not used)
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C       Suggested cutoffs (not used)
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C       LMAX
        CALL READSI(LINE,1,IOUT,LMAX(ISP),ERREAD)
        LMAX(ISP)=LMAX(ISP)+1
        LLOC(ISP)=LMAX(ISP)
        LSKIP(ISP)=LMAX(ISP)+5
        TSGP(ISP)=.TRUE.
        TNUM(ISP)=.TRUE.
        TKB(ISP)=.FALSE.
        IGAU(ISP)=0
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C       Mesh points
        CALL READSI(LINE,1,IOUT,MESHV,ERREAD)
        IF(MESHV.GT.MMAXX) THEN
          CALL XSTRING(ECPNAM,IA,IE)
          WRITE(6,*) ' RECPUPF! MESH FOR ',ECPNAM(IA:IE),' IS ',
     &                 MESHV
          WRITE(6,*) ' RECPUPF! MAX NUMBER OF SPLINE POINTS:',MMAXX
          WRITE(6,*) ' RECPUPF! INCREASE SPLINE POINTS NUMBER'
          CALL STOPGM('RECPUPF','MESH TOO BIG')
        ENDIF
        MESHVP(ISP)=MESHV
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
C       Number of wavefunctions, projectors
        CALL READSI(LINE,1,IOUT,NWFN,ERREAD)
        CALL READSI(LINE,IOUT+1,IOUT,NPROJE,ERREAD)
C       Further info (not used)
      END IF
C   
      PMA0(ISP)=ATWT(IATYP(ISP))
      RAGGIO(ISP)=DEFRAG(IATYP(ISP))
C     ==--------------------------------------------------------------==
C..Radial Mesh
      IF (UPF2) THEN
        ENDC=" "
      ELSE
        ENDC=">"
      END IF
      IERR=INSCAN(IUNIT,'<PP_MESH'//ENDC)
      IF(IERR.NE.0) THEN
        WRITE(6,*) ' RECPUPF| PP_MESH SECTION NOT FOUND '
        CALL STOPGM('RECPUPF',' ')
      ENDIF
      IERR=INSCAN(IUNIT,'<PP_R'//ENDC)
      IF(IERR.NE.0) THEN
        WRITE(6,*) ' RECPUPF| PP_R SECTION NOT FOUND '
        CALL STOPGM('RECPUPF',' ')
      ENDIF
      READ(IUNIT,END=20,ERR=20,FMT=*) (RV(IR,ISP),IR=1,MESHV)
      IERR=INSCAN(IUNIT,'<PP_RAB'//ENDC)
      IF(IERR.NE.0) THEN
        WRITE(6,*) ' RECPUPF| PP_R SECTION NOT FOUND '
        CALL STOPGM('RECPUPF',' ')
      ENDIF
      READ(IUNIT,END=20,ERR=20,FMT=*) (RW(IR,ISP),IR=1,MESHV)
C     ==--------------------------------------------------------------==
C..Local Potential section
      IF (UPF2) THEN
        ENDC=" "
      ELSE
        ENDC=">"
      END IF
      IERR=INSCAN(IUNIT,'<PP_LOCAL'//ENDC)
      IF(IERR.NE.0) THEN
        WRITE(6,*) ' RECPUPF| PP_LOCAL SECTION NOT FOUND '
        CALL STOPGM('RECPUPF',' ')
      ENDIF
      READ(IUNIT,END=20,ERR=20,FMT=*) (VR(IR,ISP,1),IR=1,MESHV)
      CALL DSCAL(MESHV,0.5D0,VR(1,ISP,1),1)
C     ==--------------------------------------------------------------==
C..Nonlocal Potential section
      IF(NPROJE.GT.0) THEN
        IERR=INSCAN(IUNIT,'<PP_NONLOCAL')
        IF(IERR.NE.0) THEN
          WRITE(6,*) ' RECPUPF| PP_NONLOCAL SECTION NOT FOUND '
          CALL STOPGM('RECPUPF',' ')
        ENDIF
        DO I=1,LMAXX
          NPRO(I,ISP)=0
        ENDDO
        IBC=0
  100   CONTINUE
        READ(IUNIT,END=20,ERR=20,FMT='(A)') LINE
        IF(INDEX(LINE,'</PP_NONLOCAL>').NE.0) THEN
          CALL STOPGM('RECPUPF','ERROR READING NL PROJECTORS')
        ENDIF
        IF(INDEX(LINE,'<PP_BETA').NE.0) THEN
          IBC=IBC+1
          IF (UPF2) THEN
            WRITE(ENDC,'(I0)')IBC
            CALL GET_SECTION_INFO(IUNIT,'<PP_BETA.'//ENDC,LONGLINE,
     *                            LEN(LONGLINE))
            CALL GET_KEY(LONGLINE,"index",LINE)
            CALL READSI(LINE,1,IOUT,I,ERREAD)
            CALL GET_KEY(LONGLINE,"angular_momentum",LINE)
            CALL READSI(LINE,1,IOUT,L,ERREAD)
            CALL GET_KEY(LONGLINE,"size",LINE)
            CALL READSI(LINE,1,IOUT,K,ERREAD)
          ELSE
            READ(IUNIT,END=20,ERR=20,FMT=*) I,L
            READ(IUNIT,END=20,ERR=20,FMT=*) K
          END IF
          PPLIST(I,1,ISP)=L
          NPRO(L+1,ISP)=NPRO(L+1,ISP)+1
          IF(NPRO(L+1,ISP).GT.MPRO) THEN
            CALL STOPGM('RECPUPF','TOO MANY PROJECTORS')
          ENDIF
          PPLIST(I,2,ISP)=NPRO(L+1,ISP)
          READ(IUNIT,END=20,ERR=20,FMT=*) (GNL(IR,ISP,IBC),IR=1,K)
          DO IR=K+1,MESHV
            GNL(IR,ISP,IBC)=0.D0
          ENDDO
        ELSE
          GOTO 100
        ENDIF
        IF(IBC.LT.NPROJE) GOTO 100
C     ==--------------------------------------------------------------==
C..set up 
        IV=0
        LP=0
        DO L=1,LMAX(ISP)
          DO M=1,2*L-1
            LP=LP+1
            DO K=1,NPRO(L,ISP)
              IV=IV+1
              NGHTOL(IV,ISP)=L-1
              LPVAL(IV,ISP)=LP
              LFVAL(IV,ISP)=K
            ENDDO
          ENDDO
        ENDDO
        NGH(ISP)=IV
C     ==--------------------------------------------------------------==
C..DIJ section
        IERR=INSCAN(IUNIT,'<PP_DIJ')
        IF(IERR.NE.0) THEN
          WRITE(6,*) ' RECPUPF| PP_DIJ SECTION NOT FOUND '
          CALL STOPGM('RECPUPF',' ')
        ENDIF
        DO L=1,LMAX(ISP)
          DO I=1,MPRO 
            DO J=1,MPRO 
              HLSG(I,J,L,ISP)=0.D0
            ENDDO
          ENDDO
        ENDDO
        IF (UPF2) THEN
          ALLOCATE(DIJ_TMP(NPROJE*NPROJE))
          READ(IUNIT,END=20,ERR=20,FMT=*) 
     *        (DIJ_TMP(IR),IR=1,NPROJE*NPROJE)
          IR=0
          DO I=1,NPROJE
            DO J=1,NPROJE
               IR=IR+1
               IF (I/=J) THEN
                 IF(DIJ_TMP(IR)/=0.D0) THEN
                   CALL STOPGM('RECPUPF','NON-ZERO DIJ IS INVALID!')
                 ENDIF
                 CYCLE
               END IF
               IF(PPLIST(I,1,ISP).NE.PPLIST(J,1,ISP)) THEN
                 CALL STOPGM('RECPUPF','INVALID DIJ')
               ENDIF
               L=PPLIST(I,1,ISP)+1
               IV=PPLIST(I,2,ISP)
               JV=PPLIST(J,2,ISP)
               HLSG(IV,JV,L,ISP)=0.5D0*DIJ_TMP(IR)
               HLSG(JV,IV,L,ISP)=0.5D0*DIJ_TMP(IR)
            END DO
          END DO
          DEALLOCATE(DIJ_TMP)
        ELSE
          READ(IUNIT,END=20,ERR=20,FMT=*) M
          DO K=1,M
            READ(IUNIT,END=20,ERR=20,FMT=*) I,J,DIJ
            IF(PPLIST(I,1,ISP).NE.PPLIST(J,1,ISP)) THEN
              CALL STOPGM('RECPUPF','INVALID DIJ')
            ENDIF
            L=PPLIST(I,1,ISP)+1
            IV=PPLIST(I,2,ISP)
            JV=PPLIST(J,2,ISP)
            HLSG(IV,JV,L,ISP)=0.5D0*DIJ
            HLSG(JV,IV,L,ISP)=0.5D0*DIJ
          ENDDO
        ENDIF
      ENDIF
      IF(TNLCC(ISP)) THEN
C     ==--------------------------------------------------------------==
C..NLCC section
        IF (UPF2) THEN
          ENDC=" "
        ELSE
          ENDC=">"
        END IF
        NLCCT(ISP)=3
        IERR=INSCAN(IUNIT,'<PP_NLCC'//ENDC)
        IF(IERR.NE.0) THEN
          WRITE(6,*) ' RECPUPF| PP_NLCC SECTION NOT FOUND '
          CALL STOPGM('RECPUPF',' ')
        ENDIF
        READ(IUNIT,END=20,ERR=20,FMT=*) (CORECG(IR,ISP),IR=1,MESHV)
        MESHCC(ISP)=MESHV
      ENDIF
      RETURN
C     ==--------------------------------------------------------------==
   20 CONTINUE
        WRITE(6,*) ' ERROR WHILE READING PP DEFINITIONS'
        CALL STOPGM('RECPUPF',' ')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GET_SECTION_INFO(IUNIT, HEADER, LINE, LLINE)
      IMPLICIT NONE
      INTEGER IUNIT, LLINE
      CHARACTER(LEN=LLINE) :: LINE
      CHARACTER(LEN=*)     :: HEADER
      INTEGER IERR, IND, INSCAN
C     ==--------------------------------------------------------------==
      IERR=INSCAN(IUNIT,TRIM(HEADER))
      IF(IERR.NE.0) THEN
        WRITE(6,*) ' RECPUPF| '//TRIM(HEADER)//' SECTION NOT FOUND '
        CALL STOPGM('RECPUPF (GET_SECTION_INFO)',' ')
      ENDIF
      BACKSPACE(IUNIT)

      IND=1
      READ(IUNIT,END=20,ERR=20,FMT='(A)')LINE(IND:)
      IND=LEN_TRIM(LINE)+1
      DO WHILE ((INDEX(LINE,'>').EQ.0).AND.(IND.LE.LLINE))
        READ(IUNIT,END=20,ERR=20,FMT='(A)')LINE(IND:)
        IND=LEN_TRIM(LINE)+1
      END DO
      IF (IND.GT.LLINE) THEN
        WRITE(6,*) ' SECTION TOO LONG! INCREASE INTERNAL BUFFER!'
        CALL STOPGM('RECPUPF (GET_SECTION_INFO)',' ')
      END IF
      RETURN
C     ==--------------------------------------------------------------==
   20 CONTINUE
      WRITE(6,*) ' ERROR WHILE READING: GET_SECTION_INFO'
      CALL STOPGM('RECPUPF',' ')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GET_KEY(LINE,FIELD,VALUE)
      CHARACTER(LEN=*) :: LINE, FIELD, VALUE
C     ==--------------------------------------------------------------==
      IND=INDEX(LINE,TRIM(FIELD)//'="')
      IF (IND==0) THEN
        WRITE(6,*) ' NO FIELD IN SECTION!'
        CALL STOPGM('RECPUPF (GET_KEY)',' ')
      END IF
      INDF=INDEX(LINE(IND+LEN_TRIM(FIELD)+2:),'"')
      VALUE=LINE(IND+LEN_TRIM(FIELD)+2:IND+LEN_TRIM(FIELD)+2+INDF-2)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE FIND_ELEMENT(SYMB,IA)
      INCLUDE 'adat.inc'
      CHARACTER SYMB*(*)
      INTEGER   IA
      INTEGER   I,I1,I2
C     ==--------------------------------------------------------------==
      IA=0
      DO I=1,99
        CALL XSTRING(EL(I),I1,I2)
        IF(EL(I)(I1:I2).EQ.SYMB) THEN
          IA=I
          GOTO 100
        ENDIF
      ENDDO
      WRITE(6,"(A,A)") 'FIND_ELEMENT| Could not find element ',SYMB
      CALL STOPGM('FIND_ELEMENT','INPUT ERROR')
  100 CONTINUE
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
