C     ==================================================================
      SUBROUTINE REPGEN(TAU0)
C     ==--------------------------------------------------------------==
C     ==   creates the NP-1 replica for the coordinates based on      ==
C     ==   the first replica (read in from standard input):           ==
C     ==   uses the Levy-Brown procedure from                         ==
C     ==   Fosdick, Jordan: PR 143 (1966) p.58                        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'adat.inc'
      INCLUDE 'pimd.inc'
CDM1
      INCLUDE 'movi.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'store.inc'
      INCLUDE 'fileopen.inc'
CDM2
C     Arguments
      REAL*8 TAU0(3,NAX,*)
C     Variables
      REAL*8    TAU(MAXNP+1),RMEAN(3),
     &          RNP,OMEGAP,OM2P,GAUSS,VARFP,VAR,DISMAX,DISMIN,
     &          DIS,SIGMA
      REAL*8    GASDEV
      EXTERNAL  GASDEV
      INTEGER   IP,IS,IA,K,IAT,IZ0,IT0,MSGLEN
      LOGICAL   FERROR
C     ==--------------------------------------------------------------==
C     Trotter time displacements
cdeb 
c      write(6,*) ' me np     ',me,np_total
cdeb    
      DO IP=1,NP_TOTAL+1
        TAU(IP)=(DBLE(IP)-1.D0)/DBLE(NP_TOTAL)
      ENDDO
C     ==--------------------------------------------------------------==
C     ==  if(tread_cent) then read in classical coordinates as        ==
C     ==  centroids and sample randomly the normal modes              ==
C     ==--------------------------------------------------------------==
      IF(GRANDPARENT) THEN
        RNP=DBLE(NP_TOTAL)
        IF(TREAD_CENT) THEN
         OMEGAP = DSQRT(RNP)*TEMPB/FACTEM
         OM2P = OMEGAP*OMEGAP
cdeb     
c      write(6,*) ' me tempb om2p  ',me,tempb,om2p
cdeb
         DO IS=1,NSP
          DO IA=1,NA(IS)
           DO K=1,3
            TREPNM(K,IA,IS,1)=TAU0(K,IA,IS)
            DO IP=2,NP_TOTAL
             GAUSS = GASDEV()
             SIGMA = DSQRT(TEMPB/(FACTEM*OM2P*FLNM(IP)*PMAR(IS)))
cdeb     
c      write(6,*) ' is, ia, k, ip ',is,ia,k,ip 
c      write(6,*) ' me gauss sigma flnm(ip) pmar(is)  ',
c     *       me, gauss, sigma, flnm(ip), pmar(is) 
cdeb
             TREPNM(K,IA,IS,IP) = GAUSS*SIGMA
            ENDDO
           ENDDO
          ENDDO
         ENDDO
C TRANSFORM FROM NORMAL MODES TO PRIMITIVE COORDINATES
         CALL PINMTRANS(TREP,TREPNM,1)
         CALL DCOPY(3*NAX*NSX,TREP,1,TAU0,1)
        ELSE
         DO IS=1,NSP
           DO IA=1,NA(IS)
             DO K=1,3
               TREP(K,IA,IS,1)=TAU0(K,IA,IS)
               DO IP=1,NP_TOTAL-1
                 GAUSS = GASDEV()
                TREP(K,IA,IS,IP+1)=
     *          ( TREP(K,IA,IS,IP)*(TAU(NP_TOTAL+1)-TAU(IP+1))
     *          + TREP(K,IA,IS,1)*(TAU(IP+1)-TAU(IP)) ) /
     *          (TAU(NP_TOTAL+1)-TAU(IP))
     *          + GAUSS*DSQRT(
     *          (FACTEM/(PMAR(IS)*TEMPB))*
     *          (TAU(IP+1)-TAU(IP))*(TAU(NP_TOTAL+1)-TAU(IP+1)) /
     *          (TAU(NP_TOTAL+1)-TAU(IP))
     *          )
              ENDDO
            ENDDO
           ENDDO
         ENDDO
        ENDIF
CDM1
C       final check if all positions are inside cluster
        IF(TCLUST) THEN
          DO IS=1,NSP
            DO IA=1,NA(IS)
              DO K=1,3
                DO IP=1,NP_TOTAL
                  IF( (TREP(K,IA,IS,IP).GE.ALAT) .OR.
     *                (TREP(K,IA,IS,IP).LE.0.d0) ) THEN
                    WRITE(6,*)
     *          ' !! STOP IN REPGEN: TAU(K,IA,IS,IP) OUTSIDE CLUSTER'
                WRITE(6,*) ALAT,SNGL(TREP(K,IA,IS,IP)),K,IA,IS,IP
                    CALL STOPGM('REPGEN','REPLICA OUTSIDE CLUSTER')
                  ENDIF
                ENDDO
              ENDDO
            ENDDO
          ENDDO
        ENDIF
CDM2
C
C     check the radius of gyration and compare to free particle
C     (Gillan in Computer Modelling of Fluids Polymers and Solids 1990) 
        WRITE(6,'(/,1X,63("*"))')
        WRITE(6,'(" *",61X,"*")')
        WRITE(6,'(" *",23X,A,23X,"*")') ' R OF GYRATION '
        WRITE(6,'(" *",61X,"*")')
        WRITE(6,'(1X,63("*"),/)')
        IF(TREAD_CENT) THEN
          WRITE(6,*) ' CLASSICAL CONFIGURATION READ IN AS CENTROIDS'
          WRITE(6,*) ' NORMAL MODES SAMPLED DIRECTLY'
          IF(.NOT.TPINM) THEN
           IF(TSTAGE) THEN
            WRITE(6,*) ' STAGING COORDINATES COMPUTED FROM NORMAL MODES'
           ELSE
            WRITE(6,*) 
     *           ' PRIMITIVE COORDINATES COMPUTED FROM NORMAL MODES'
           ENDIF
          ENDIF
        ELSE
          WRITE(6,*) '  REPLICA HAVE GAUSSIAN DISTRIBUTION AROUND IP=1'
        ENDIF
        WRITE(6,*) '  '
        WRITE(6,'(A,A,A,A,A)') '   ATOM   R OF GYRATION',
     *                 '  FREE PARTICLE',
     *                 '  MAXDIS',
     *                 '  MINDIS',
     *                 ' (ANGSTROM)'
CDM     *                 ' (ANGSTROEM)'
C
        IAT=0
        DO IS=1,NSP
          VARFP=(FACTEM/(4.0D0*PMAR(IS)*TEMPB))
          VARFP=DSQRT(VARFP)
          DO IA=1,NA(IS)
            DO K=1,3
              RMEAN(K)=0.0D0
              DO IP=1,NP_TOTAL
                RMEAN(K)=RMEAN(K)+TREP(K,IA,IS,IP)
              ENDDO
              RMEAN(K)=RMEAN(K)/RNP
            ENDDO
            VAR=0.0D0
            DO IP=1,NP_TOTAL
              DO K=1,3
                VAR=VAR+(TREP(K,IA,IS,IP)-RMEAN(K))**2
              ENDDO
            ENDDO
            VAR=DSQRT(VAR/RNP)
            DISMAX=-10.D0
            DISMIN=100000.0D0
            DO IP=1,NP_TOTAL
              IF(IP.EQ.NP_TOTAL) THEN
                DIS=(TREP(1,IA,IS,IP)-TREP(1,IA,IS,1))**2
     *              +(TREP(2,IA,IS,IP)-TREP(2,IA,IS,1))**2
     *              +(TREP(3,IA,IS,IP)-TREP(3,IA,IS,1))**2
              ELSE
                DIS=(TREP(1,IA,IS,IP)-TREP(1,IA,IS,IP+1))**2
     *              +(TREP(2,IA,IS,IP)-TREP(2,IA,IS,IP+1))**2
     *              +(TREP(3,IA,IS,IP)-TREP(3,IA,IS,IP+1))**2
              ENDIF
              DIS=DSQRT(DIS)
              IF(DIS.GT.DISMAX) DISMAX=DIS
              IF(DIS.LT.DISMIN) DISMIN=DIS 
            ENDDO
C
            IAT=IAT+1
            WRITE(6,'(I4,2X,A2,3X,F8.5,7X,F8.5,4X,F8.5,1X,F8.5)') 
     *        IAT,EL(IATYP(IS)),
     *        VAR/FBOHR,VARFP/FBOHR,DISMAX/FBOHR,DISMIN/FBOHR
          ENDDO
        ENDDO
cdeb
c         write(6,*) ' me fbohr ',me,fbohr 
cdeb
        WRITE(6,*) '  '
C 
C CHECK POSITIONS OF CENTROIDS
C
        DO IS=1,NSP
          DO IA=1,NA(IS)
            DO K=1,3
              RMEAN(K)=0.0D0
              DO IP=1,NP_TOTAL
                RMEAN(K)=RMEAN(K)+TREP(K,IA,IS,IP)
              ENDDO
              RMEAN(K)=RMEAN(K)/RNP
            ENDDO
          ENDDO
        ENDDO
      ENDIF
      MSGLEN = 3*NAX*NSX*NP_TOTAL * 8
      CALL MY_BCAST(TREP,MSGLEN,SUPERSOURCE,SUPERGROUP)
      IF(TREAD_CENT) THEN
        MSGLEN = 3*NAX*NSX*NP_TOTAL * 8
        CALL MY_BCAST(TREPNM,MSGLEN,SUPERSOURCE,SUPERGROUP)
      ENDIF
CDM1
C..WRITE MOVIE FILE
      IF(GRANDPARENT.AND.MOUT) THEN
        CALL FILEOPEN(11,'MOVIE',FO_DEF,FERROR)
        DO IP=1,NP_TOTAL
          DO IS=1,NSP
            IZ0=IMTYP(IS)
            DO IA=1,NA(IS)
              IF(IATYP(IS).EQ.4) THEN
C..Change Be -> Na ; problem with movie
                IT0=11
                WRITE(11,'(3(2X,F12.4),2I4)')
     *                   (TREP(K,IA,IS,IP)/FBOHR,K=1,3),IT0,IZ0
              ELSE
                WRITE(11,'(3(2X,F12.4),2I4)')
     *                   (TREP(K,IA,IS,IP)/FBOHR,K=1,3),IATYP(IS),IZ0
              ENDIF
            ENDDO
          ENDDO
        ENDDO
        CALL FILECLOSE(11)
      ENDIF 
CDM2
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION GASDEV()
C     ==--------------------------------------------------------------==
C     ==   produce gaussian random number (use XRANF)                 ==
C     ==   from NUMERICAL RECIPES, CH. 7.3 p. 203                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      REAL*8 GASDEV
C     Variables
      REAL*8  V1,V2,FAC,R,XRANF
      EXTERNAL XRANF
      INTEGER ISET
      DATA    ISET /0/
C 05.07.96 new:
      REAL*8 GSET
      DATA   GSET /0.D0/
      SAVE   ISET,GSET
C 05.07.69      SAVE ISET 
C     ==--------------------------------------------------------------==
      IF (ISET.EQ.0) THEN
1       V1=2.D0*XRANF()-1.D0
        V2=2.D0*XRANF()-1.D0
        R=V1**2+V2**2
        IF(R.GE.1.D0)GO TO 1
        FAC=DSQRT(-2.D0*DLOG(R)/R)
        GSET=V1*FAC
        GASDEV=V2*FAC
        ISET=1
      ELSE
        GASDEV=GSET
        ISET=0
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
