      SUBROUTINE RESPIN_P
C     ==--------------------------------------------------------------==
C     ==  THIS ROUTINE READS THE SECTION &RESP &END ON UNIT IUNIT     ==
C     ==--------------------------------------------------------------==
C     ==  THE INPUT HAS TO BE IN THE FOLLOWING FORMAT                 ==
C     ==     &RESP                                                    ==
C     ==        Options                                               ==
C     ==     &END                                                     ==
C     ==--------------------------------------------------------------==
C     ==  LIST OF OPTIONS                                             ==
C     ==                                                              ==
C     ==  KEEPREALSPACE                                               ==
C     ==    like the standard CPMD option, this keeps the C0 ground   ==
C     ==    state wavefunctions in the direct space representation    ==
C     ==    during the calculation. Can save a lot of time, but is    ==
C     ==    incredibly memory intensive.                             ==
C     ==                                                              ==
C     ==                                                              ==
C     ==  CONVERGENCE                                                 ==
C     ==    [0.00001]                                                 ==
C     ==    convergence criterium on the gradient dE/dpsi             ==
C     ==                                                              ==
C     ==  NOOPT                                                       ==
C     ==    Do not perform a ground state wfn optimization. Be sure   ==
C     ==    the restarted wfn is at the BO-surface.                   ==
C     ==                                                              ==
C     ==  HTHRS or HAMiltonian CUToff                                 ==
C     ==    [0.5]                                                     ==
C     ==    The value where the preconditioner (the approximate       ==
C     ==    Greens function (V_kin+V_coul-eps_KS)^-1) is truncated.   ==
C     ==                                                              ==
C     ==  POLAK                                                       ==
C     ==    Uses the Polak-Ribiere formula for the conjugate          ==
C     ==    gradient algorithm. Can be safer in the convergence.      ==
C     ==                                                              ==
C     ==  CG-ANALYTIC                                                 ==
C     ==    The number of steps for which the step length in the      ==
C     ==    conjugate gradient optimization is calculated assuming    ==
C     ==    a quadratic functional E(2) (quadratic in C1). Default    ==
C     ==    is 99, and 3 for NMR.                                     ==
C     ==                                                              ==
C     ==  CG-FACTOR                                                   ==
C     ==    The analytic length calculation yields in general         ==
C     ==    a result that is slightly too large. This factor          ==
C     ==    is used to correct for that deficiency. Default is        ==
C     ==    0.8.                                                      ==
C     ==                                                              ==
C     ==                                                              ==
C     ==  TIGHTPREC                                                   ==
C     ==    Uses a harder preconditioner. For experts: The            ==
C     ==    Hamiltonian is approximated by the kinetic energy,        ==
C     ==    the G-diagonal Coulomb potential and the KS-energies.     ==
C     ==    The number obtained this way must not be close to zero.   ==
C     ==    This is achieved by smoothing it with                     ==
C     ==      x -> f(x) = sqrt(x^2 + eps^2) [default]                 ==
C     ==    or                                                        ==
C     ==      x -> f(x) = (x^2 + eps^2)/x   [this option]             ==
C     ==    The HARD option conserves the sign of the approximate     ==
C     ==    Hamiltonian whereas the default formula does never        ==
C     ==                                                              ==
C     ==    diverge.                                                  ==
C     ==                                                              ==
C     ==                                                              ==
C     ==    RAMAN                                                     ==
C     ==                                                              ==
C     ==    HARDNESS                                                  ==
C     ==                                                              ==
C     ==    PHONON                                                    ==
C     ==      lambda                                                  ==
C     ==                                                              ==
C     ==    LANCZOS  [CONTINUE,DETAILS]                               ==
C     ==    lanczos_dim  iterations   conv_threshold                  ==
C     ==    lanczos_dim= dimension of the vibrational d.o.f.          ==
C     ==    iterations = no. of iterations desired for this run       ==
C     ==    conv_threshold = threshold for convergence on eigenvectors==
C     ==    CONTINUE = argument for continuig Lanczos diagonalization ==
C     ==               from a previous run                            ==
C     ==               (reads file LANCZOS_CONTINUE)                  ==
C     ==    DETAILS  = argument for verbosity. prints a lot of stuff  ==
C     ==                                                              ==
C     ==    EIGEN                                                     ==
C     ==      lanczos-dimemsion num_converged_states conv_threshold   ==
C     ==                                                              ==
C     ==      shift_constant                                          ==
C     ==                                                              ==
C     ==    FUKUI [N=NF] [COEFFICIENTS]                               ==
C     ==      n [weight]                                              ==
C     ==      ... (NF lines)                                          ==
C     ==                                                              ==
C     ==    NMR [options, see response_p.inc]                         ==
C     ==       CURRENT [TCURRENT]:    Calculate the current density   ==
C     ==         and the shift tensor using the current density       ==
C     ==       VERBOSE [TVERBOSE]:    Print additional (technical)    ==
C     ==         information                                          ==
C     ==       PSI0 [TPRINTWFNS]:     Write the Wannier states to     ==
C     ==         files named psi0-##.dens                             ==
C     ==       RHO0 [TPRINTRHO ]:     Write the orbital densities     ==
C     ==         to files named rho0-##.dens                          ==
C     ==       SPLIT [TSPLITSTATES]:  Calculate the contribution of 
C     ==         each electronic state (to the shift tensor) separately
C     ==       ISO [TISOONLY]:        Calculate only diagonal values 
C     ==         of the shifttensor (S_ii)
C     ==       LLC [!TCOMMONLLC]:     Special option. EXPERIMENTAL.
C     ==       NOVIRT [TNOVIRTUAL]:   Implies NOLOC, and uses the same 
C     ==         virtual cell (equal to the physical cell) for all the
C     ==         orbitals. (ONLY FOR MOLECULES! )
C     ==       NOLOC [!TLOCALIZE]:    Do not use Wannier functions but 
C     ==         canonical orbitals (ONLY FOR MOLECULES! Use of NOVIRT
C     ==         instead of NOLOC is recommended.)
C     ==       FAST [TFAST]:          Omit some time-consuming operations,
C     ==         e.g. the loss in information when doing FFTs R->G
C     ==       WANNIERCENTER [TWC]:   The center of the wavefunction-specific
C     ==         box is put where psi0^2 reaches its maximum. Default is
C     ==         that the LLC is placed where psi0^2 -> min.
C     ==       NOSMOOTH [!TSMOOTHING]: By default, an (exp - x^2) 
C     ==         is applied around the LLC. Switched off by this option.
C     ==       TRIPACK/SIXPACK:       Distribute the perturbation 
C     ==         calculations for p_i and L_i (the momentum and 
C     ==         angular momentum operators applied to Phi_0)
C     ==         on several proc groups.
C     ==           TRIPACK: nmr_threads = 3
C     ==           SIXPACK: nmr_threads = 6
C     ==       IGLO:                  Do an IGLO-type calculation.
C     ==                              (no longer supported)           ==
C     ==       FULL:                 Calculate the errors due to the  ==
C     ==         neglected current contribution Delta J (see paper)   ==
C     ==       RESTART                                                ==
C     ==         restart a previous (or interrupted) calculation      ==
C     ==       ISOLATED                                               ==
C     ==         equivalent to OVERLAP / 0.1                          ==
C     ==                                                              ==
C     ==    EPR                                                       ==
C     ==      ...with the same options as for the NMR keyword.        ==
C     ==      PLEASE NOTE that the EPR keyword has to be preceded     ==
C     ==      by at least one space, due to overlap with other        ==
C     ==      keywords (stateprec).                                   ==
C     ==                                                              ==
C     ==                                                              ==
C     ==                                                              ==
C     ==    SIGMA                                                     ==
C     ==      [0.1] Enter a value for the spread of the Gaussian      ==
C     ==      for the calculation of the virtual boxes (see also      ==
C     ==      nmr_util_p.F, routine calc_lower_left). The LLC         ==
C     ==      of the virtual cell is put where the integral of the    ==
C     ==      orbital density over the Gaussian exp -(x-x0)^2/2sigma  ==
C     ==      is smallest. Should be used with care (may affect the   ==
C     ==      results if chosen incorrectly).                         ==
C     ==                                                              ==
C     ==    DISCARD [OFF,PARTIAL,TOTAL,LINEAR]                        ==
C     ==      keyword for discarding trivial modes in vibrational     ==
C     ==      analysis (boht PHONONS and LANCZOS)                     ==
C     ==      OFF = argument for performing no projection             ==
C     ==      PARTIAL = argument for projecting out only tranlsations ==
C     ==                (this is the default)                         ==
C     ==      TOTAL = argument for projecting both rotations and      ==
C     ==              translations                                    ==
C     ==      LINEAR = argument for projecting rotations in a linear  ==
C     ==               molecule (not implemented yet)                 ==
C     ==                                                              ==
c     [marcella]
c     ==     
c     ==    KPERT[options, see response_p.inc]
C     ==        sampling of the BZ                                    ==
C     ==        MONKHORSTPACK [TMONKP]                                ==
C     ==        NK1  NK2  NK3                                         ==
C     ==        otherwise read K-POINT coordinates,                   ==
C     ==                  {SCALE} for scalingwrt basis vectors in BZ  ==
C     ==                  NKPTS                                       ==
C     ==                  RK(1,IK) RK(2,IK) RK(3,IK) WK(IK)           ==
C     ==                  ........                                    ==
C     ==       Type of calculation                                    ==
C     ==       OPTIMIZATION RESPONSE WFN:                             ==
C     ==       WRITE_C1 {RESTART_CU1} C1 are written in RESTA_P       == 
C     ==       BUILD_C00{TK_PREP_DIAG} a standard restart file is     ==
C     ==                prepared where 2*NSTATE*NKPTS wfn are stored  ==
C     ==                for each  k-point:
C     ==                1 : NSTATE are ground state wfn : C0          ==
C     ==                NSTATE+1 : 2*NSTATE are                       ==
C     ==                          i(kx c1x + ky c1y + kz c1z)         ==
C     ==                this RESTART should be used by standard CPMD  ==
C     ==                with K-POINT option (same k vectors)          ==
C     ==                for the calculation of the KS energies by a   ==
C     ==                single diagonalization of the H matrix        ==
C     ==                (option ONLYDIAG[TONLYDIAG] see sysin.F)      ==
C     ==                {READ_C1}[TKREAD_C1] C1 are not calculated    ==
C     ==                                    but read in RESTA_P       ==
C     ==       NORESTART only calculation of C1 and E2                ==
C     ==       HAMILTONIAN[TPERT_HAMILT]{READ_C1} C0 and C1 are used  ==
C     ==                  for H(k) construction with the second order ==
C     ==                  approximation, and KS energies are computed ==
C     ==                  {{C0},{i(kxC1x+kyc1y+kxc1z)}} 2*NSTATE      ==
c     [anatole]                                                       ==
C     ==                                                              ==
C     ==       DUMMY_OPT for the linking dummyatom optimization       ==
C     ==          DUMPBD parabolic descent optimizer, don't trust it  ==
C     ==          DUMSD  steepest descent, more trustworthy           ==
C     ==          DUMN   Newton optimizer, not trustworthy at all     ==
C     ==          DUMMY_STEP asks for step-sizes for the steepest     ==
C     ==                     descent for each of the 5 sigma of a     ==
C     ==                     1996 SG-PP of the 2nd row                ==
C     ==                                                              ==
C     ==       DUMMY_REF will print out a reference electron density  ==
C     ==                 and coordinates                              ==
C     ==                                                              ==
C     ==       DUMMY_TEC will just print out coordinates              ==
C     ==                                                   
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'irat.inc'
      INCLUDE 'store.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'response_p.inc' 
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
C     Variables
      CHARACTER LINE*80,NEWLINE*80
      INTEGER   IUNIT,I,II,J,IS,IOUT,ION,IA,IE,I1,IK,
     &          INSCAN,NSTATE,IRESPONSE
      LOGICAL   ERREAD,READ_MORE
      DIMENSION RK(3,*),WK(*)
#ifdef PARALLEL
      INTEGER MSGLEN
#endif
C     ==--------------------------------------------------------------==
C     ==  DEFAULTS                                                    ==
C     ==--------------------------------------------------------------==
      IUNIT=INSYS
      NSTATE=N
      RESPONSE_RUNNING = .FALSE. ! this is for WRENER only.
      TKEEPREALSPACEWFN = .false.
      T_RESTART_C1 = .false.
      T_INITIALGUESS_C1 = .false.

C     anatoles variables:
      TDUMMYATOM=.FALSE.
      TDUMMYATOM_REF=.FALSE.
      TDUMMYATOM_TEC=.FALSE.
      TOACP4F = .FALSE.
      TDUMMY_STEP=.FALSE.
      TDUMPBD=.FALSE.
      TDUMN=.FALSE.
      TDUMSD=.FALSE.

c     TYPE of the perturbation:
      TRAMAN=.FALSE.
      PHONON=.FALSE.
      TEIGENSYSTEM=.FALSE.
      THARDNESS=.FALSE.
      TNMR=.FALSE.
      TEPR=.FALSE.
      PR_ENERGY=.TRUE.
      TKPERT=.FALSE.      
      TFUKUI=.FALSE.

C     INTERACTION routines, initialization
      TINTERACTION = .FALSE.
      CUTOFF_TRICK = .FALSE.
      CUTOFF_RESTR = .FALSE.
      Tlinearscaling = .FALSE.
C     Default number of SCF iterations
      BPTSCFIter=1
C     initialising the Wannier rotation scalar variables (DMB)
      BLmax=4
      NMOL=1
C     initialisation flags
      IN_BTHETA=.FALSE.
      IN_BVEC=.FALSE.
      IN_BDXVEC=.FALSE.
C     By default no save, no load, full rotations, 
C     single reference file, no gaussian representation
      WANN_SAVE=.FALSE.
      WANN_LOAD=.FALSE.
      WANN_ALLROT=.TRUE.
      WANN_MULTI=.FALSE.
      WANN_GAUSS=.FALSE.
C     by default only one localisation cycle if WC prediction is on...
      WC_PRED_STEP=1000000

C     d2Exc/dn1 dn2 calculation using LDA (FALSE by default)
      FASTDEXC=.FALSE.

C     by default there is a non-zero force inducing a response
      TNORESPONSE=.FALSE.

C     by default the tolerance on the SCF-PT cycles is 1E-6
      SCF_TOL=1D-6

C     By default, W0 is not saved in a RESTART file
      SAVE_LOCALISED_W0=.FALSE.

C     By default use wannier functions for W0 and not atomic wf
      TATOMICWAVEFUNCTION=.FALSE.

C     By default use wannier density and not atomic one
      TATOMICRHO=.FALSE.

C     By default the Wannier orbitals are orthogonalised
      TORTHOG_WANNIER=.TRUE.

C     By default full PT calculation
      TSIMPLE_MODEL=.FALSE.
      TINTER_ORTHO = .FALSE.
      
C     BornOppenheimer dynamics.. 
C     By default no predictions of the WC position
      WCPREDICT=.FALSE.
      WC_sample=99

C     Maximum number of states saved at the end of PT-INTERACTION
      max_num_disp=10
      
C     use parallel version by default
      TMANNO = .FALSE.
C     end of initialisation (DMB)

c     wfn optimization method:
      TSDE_P=.FALSE.
      PREC_P=.TRUE.
      PCG_P=.TRUE.
      PCGMIN_P=.TRUE.
      TPOLAK_P=.FALSE.
      HTHRS_P=0.5d0
      TOLOG_P=0.00001D0
      CG_FACTOR=0.8d0
      PRECONDITIONER_P=1
      CG_ANALYTIC=99
      OPT1st = .FALSE.
      MDIIS_p = 10

C     Lanczos Phonon option:
      tlanphon=.false.
      tlanph_cont=.false.
      details=.false.

c ... discarding defaults
      projout=.true.
      rotout=.false.

      

      TEIGENS=.FALSE.


c     NMR special options:
      TCURRENT=.FALSE.
      TVERBOSE=.FALSE.
      TPRINTWFNS=.FALSE.
      TPRINTRHO=.FALSE.
      TLOCALIZE=.TRUE.
      TFAST=.FALSE.
      TSMOOTHING=.TRUE.
      sigma=0.1d0
      nmr_threads=1
      INMR_METHOD=INMR_DEFAULT
      INMR_VIRTUAL=INMR_DEFAULT
      TNMR_FULL = .FALSE.
      TNMR_ONLY_ERROR = .FALSE.
      TNMR_OVERLAPS= .FALSE.
      TFORCE_XYZ(1)= .FALSE.
      TFORCE_XYZ(2)= .FALSE.
      TFORCE_XYZ(3)= .FALSE.
      nmr_superparent   = parent
      nmr_supergroup    = allgrp
      nmr_total_nproc   = nproc
      nmr_supersource   = source

c     EPR special options:
      TEVERBOSE=.FALSE.
      TEPRINTWFNS=.FALSE.
      TEPRINTRHO=.FALSE.
      TELOCALIZE=.TRUE.
      TEFAST=.FALSE.
      TESMOOTHING=.TRUE.
      IEPR_METHOD=IEPR_DEFAULT
      IEPR_VIRTUAL=IEPR_DEFAULT
      TEPR_FULL = .FALSE.
      TEPR_ONLY_ERROR = .FALSE.
      TEPR_OVERLAPS= .FALSE.
      TEFORCE_XYZ(1)= .FALSE.
      TEFORCE_XYZ(2)= .FALSE.
      TEFORCE_XYZ(3)= .FALSE.
      TEPR_SMART = .FALSE. 
      TEPR_HYP = .FALSE.
      TEPR_OWNPOT = .FALSE.

c     KPERT special option
      RESTART_CU1 = .FALSE.
      TK_PREP_DIAG =.FALSE.
      TKREAD_C1 =.FALSE.
      TPERT_HAMILT =.FALSE.

C     Num Orbitals for Fukui
      numf=0
      ERREAD = .FALSE.

      IF(PARENT) THEN
C     ==------------------------------------------------------------==
        READ_MORE = (INSCAN(IUNIT,'&RESP') .EQ. 0)
c     read more [data from input file] if &RESP was found.

        DO WHILE (READ_MORE)

           READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
           READ_MORE = (INDEX(LINE,'&END').EQ.0)
c     read more if &END is not part of the current line.

           IF(INDEX(LINE,'RESTART').NE.0) THEN
C     A RESTART implies that C0 is not re-optimized!
              T_RESTART_C1 = .TRUE.
              WFOPT=.FALSE.
              RESTART=.TRUE.
              RWF=.TRUE.
           ENDIF

C      anatoles calls:
           IF(INDEX(LINE,'OACP DENSITY').NE.0) THEN
             TDUMMYATOM   = .TRUE.
             READ(IUNIT,ERR=99,END=99,FMT='(A80)') NEWLINE
             CALL XSTRING(NEWLINE,IA,IE)
             CALL READSI(NEWLINE,IA,IE,TDUMNUM,ERREAD)
             IF(ERREAD) GOTO 99
             IA=IE
             CALL READSR(NEWLINE,IA,IE,DUMCRIT,ERREAD)
             IF(ERREAD) GOTO 99

             READ(IUNIT,ERR=99,END=99,FMT='(A80)') NEWLINE
             CALL XSTRING(NEWLINE,IA,IE)
             CALL READSR(NEWLINE,IA,IE,DUMSTEP1,ERREAD)
             IF(ERREAD) GOTO 99
             IA=IE
             CALL READSR(NEWLINE,IA,IE,DUMSTEP2,ERREAD)
             IF(ERREAD) GOTO 99
             IA=IE
             CALL READSR(NEWLINE,IA,IE,DUMSTEP3,ERREAD)
             IF(ERREAD) GOTO 99
             IA=IE
             CALL READSR(NEWLINE,IA,IE,DUMSTEP4,ERREAD)
             IF(ERREAD) GOTO 99
             IA=IE
             CALL READSR(NEWLINE,IA,IE,DUMSTEP5,ERREAD)
             IF(ERREAD) GOTO 99

             READ(IUNIT,ERR=99,END=99,FMT='(A80)') NEWLINE
             CALL XSTRING(NEWLINE,IA,IE)
             CALL READSR(NEWLINE,IA,IE,RAD_DUM,ERREAD)
             IF(ERREAD) GOTO 99
             IA=IE
             CALL READSR(NEWLINE,IA,IE,RAD_NORM,ERREAD)
             IF(ERREAD) GOTO 99
           ENDIF
           IF(INDEX(LINE,'OACP REF_DENSITY').NE.0)
     &         TDUMMYATOM_REF = .TRUE.
           IF(INDEX(LINE,'OACP FORCE').NE.0) THEN
              TOACP4F = .TRUE.
              read(iunit,err=99,end=99,fmt=*)
     &        TDUMNUM,DUMCRIT,TFDIMEN
              read(iunit,err=99,end=99,fmt=*)
     &        ANG_MOM,DUMSTEP4,DUMSTEP5
           ENDIF

           IF(INDEX(LINE,'RAMAN').NE.0)       TRAMAN       = .TRUE.
           IF(INDEX(LINE,'PHONON').NE.0)      PHONON       = .TRUE.
           IF(INDEX(LINE,'EIGEN').NE.0) THEN
              Teigensystem = .TRUE.
              READ(IUNIT,ERR=99,END=99,FMT=*)
     &          lanczos_dim,num_lanczos_states,conv_threshold,
     &          eigenvalue_shift
           ENDIF 
           IF(INDEX(LINE,'HARDNESS').NE.0) THEN
              THARDNESS    = .TRUE.
              READ(IUNIT,ERR=99,END=99,FMT=*)
     &          lanczos_dim,num_lanczos_states,conv_threshold,
     &          eigenvalue_shift
           ENDIF

           IF(INDEX(LINE,'FUKUI').NE.0)  THEN
              TFUKUI    = .TRUE.
              TWEIGHT   = .TRUE.
              IF (INDEX(LINE,'COEF').NE.0) TWEIGHT=.FALSE.
              I1=INDEX(LINE,'=')
              IF (I1.NE.0) THEN
                 CALL READSI(LINE,I1+1,IOUT,NUMF,ERREAD)
              ELSE
                 NUMF=1
              ENDIF
              IF (ERREAD) GOTO 99
              CALL MEMORY(IP_NF,NUMF,'NF')
              CALL MEMORY(IP_WGHTF,NUMF+1,'WGHTF')
              DO I=1,NUMF
                 READ(IUNIT,ERR=99,END=99,FMT='(A80)') NEWLINE
                 CALL READSI(NEWLINE,1,IOUT,NF(I),ERREAD)
                 IF (ERREAD) GOTO 99
                 IF (NF(I).GE.0) 
     &                 call  STOPGM('RESPIN_P','NF must be negative')
                 CALL READSR(NEWLINE,IOUT,I1,WGHTF(I),ERREAD)
                 IF (ERREAD) WGHTF(I)=1.0D0
c                 READ(IUNIT,ERR=99,END=99,FMT=*) NF(I),WGHTF(I)
              ENDDO
              WGHTF(NUMF+1)=1.0D0
           ENDIF
           IF(INDEX(LINE,'INTERACT').NE.0)    TINTERACTION = .TRUE.
           IF(INDEX(LINE,'KPERT').NE.0)      then 
              write(6,'("************* KPERT RESPONSE ********")')
              WRITE(6,'(">>>>>>>>>>>>>>>   READ THE KPERT OPTIONS")')
              TKPERT       = .TRUE.
              if (INDEX(LINE,'MONKH') .NE. 0) THEN
                 TMONKP = .TRUE.
                 READ(IUNIT,ERR=99,END=99,FMT=*) nk1,nk2,nk3
                 write(6,
     &                '("MONKHORST_PACK MESH ",i4,"X",i4,"X",i4)')
     &                nk1,nk2,nk3 
              else
                 write(6,'("CHOSEN SET OF K VECTORS")')
                 if (INDEX(LINE,'SCALE') .NE. 0) THEN
                    TKSCALE =.true.
                 endif
                 READ(IUNIT,ERR=99,END=99,FMT=*) NKPTS
c     
                 write(6,'("NKPTS=",i5)') NKPTS
c     
                 IF(NKPTS.LE.0)
     &                CALL STOPGM
     &                ('  RESPIN','WRONG NUMBER OF K POINTS') 
                 CALL MEMORY(IP_RK,3*NKPTS,'RK')
                 CALL MEMORY(IP_WK,  NKPTS,'WK')
                 DO IK=1,NKPTS
                    READ(IUNIT,ERR=99,END=99,FMT='(A)') LINE
                    IA=1
                    DO I=1,3
                       CALL READSR(LINE,IA,IE,RK(I,IK),ERREAD)
                       IF(ERREAD) THEN
                          WRITE(6,
     &                         '(" KPOINTS: RK(",I1,",",I4,")=")')
     &                         I,IK
                          WRITE(6,*) 'ERROR READING KPOINTS INPUT'
                          GOTO 99
                       ENDIF
                       IA=IE
                    ENDDO
                    CALL READSR(LINE,IA,IE,WK(IK),ERREAD)
                    IF(ERREAD) THEN
                       WRITE(6,'(" KPOINTS: WEIGHT OF KPOINT ",I4)')
     &                      IK
                       WRITE(6,*) 'ERROR READING KPOINTS INPUT'
                       GOTO 99
                    ENDIF
                 ENDDO
              ENDIF
 75           CONTINUE
              READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
              IF(INDEX(LINE,'WRITE_C1') .NE. 0) THEN  
                 WRITE(6,
     &           '(">>>>>>>>>> AT THE END, WRITE THE 3 C1 SETS",'
     &                //'" ON 3 SEPARATED RESTART FILES")')
                 RESTART_CU1 = .TRUE.
                 GOTO 75
              ELSEIF(INDEX(LINE,'BUILD_C00') .NE. 0) THEN 
                 WRITE(6,
     &         '(">>>>>>>>>> USE THE C0 AND THE C1 TO BUILD THE C00")')
                 TK_PREP_DIAG =.TRUE.
                 IF(INDEX(LINE,'READ_C1').NE. 0) THEN
                 WRITE(6,'(">>>>>>>>>> THE C1 ARE READ FROM RESTA_P")')
                    TKREAD_C1 = .TRUE.
                 ENDIF
                 GOTO 75
              ELSEIF(INDEX(LINE,'HAMILTONIAN') .NE. 0) THEN 
                 WRITE(6,
     &           '(">>>>>>>>>> BUILD H BY USING THE PERT. THEORY")')
                 WRITE(6,
     &          '(">>>>>>>>>> H IS APPROXIMATED TO SECOND ORDER IN K")')
                 TPERT_HAMILT =.TRUE.
                 IF(INDEX(LINE,'READ_C1').NE. 0) THEN
                    WRITE(6,
     &           '(">>>>>>>>>> THE C1 ARE READ FROM RESTA_P")')
                    TKREAD_C1 = .TRUE.
                 ENDIF
                 GOTO 75
              ELSEIF(INDEX(LINE,'NORESTART') .NE. 0) THEN
                 RESTART_CU1 = .FALSE.
                 TK_PREP_DIAG =.FALSE.
                 WRITE(6,'(">>>>>>>>>> NO RESTART FILE IS WRITTEN")') 
                 GOTO 75
              ELSE
                GOTO 76
              ENDIF
 76           CONTINUE
                IF(TK_PREP_DIAG .AND. TPERT_HAMILT) CALL STOPGM
     &                ('  RESPIN',' 2 MANY METHODS')
           ENDIF
           IF(INDEX(LINE,'LANCZOS').NE.0)      THEN 
              tlanphon=.TRUE.
              if(index(line,'CONT').ne.0) tlanph_cont=.true.
              READ(IUNIT,ERR=99,END=99,FMT=*) lanczos_dim,
     &             nlan_st,conv_threshold
              lanczos_dim=lanczos_dim+1
           endif
c ... VERBOSITY IN LANCZOS ROUTINE
           if(index(line,'DETAILS').ne.0) details=.true.
C ... DISCARDING TRANSLATIONAL AND/OR ROTATIONAL DEGREES OF FREEDOM
           if(index(line,'DISCARD').ne.0) then
              if(index(line,'OFF').ne.0) then
                 projout=.false.
                 rotout=.false.
              elseif(index(line,'PARTIAL').ne.0) THEN
                 projout=.true.
                 rotout=.false.
              elseif(index(line,'TOTAL').ne.0) then
                 projout=.true.
                 rotout=.true.
              elseif(index(line,'LINEAR').ne.0) then
                 WRITE(6,'(A)') ' LINEAR MOLECULE NOT IMPLEMENTED YET '
                 CALL STOPGM('RESPIN_P',' KEYWORD LINEAR NOT'//
     &                '  IMPLEMENTED YET ')
              else
                 WRITE(6,'(A)') ' KEYWORD DISCARD NEEDS AN ARGUMENT '
                 CALL STOPGM('RESPIN_P',' KEYWORD DISCARD NEEDS AN '
     &                //'ARGUMENT ')
              endif
           endif


C     ==--  NMR  ---------------------------------------------------==
           IF(INDEX(LINE,'NMR ').NE.0)         THEN
              TNMR         = .TRUE.
C     IF(INDEX(LINE,' IGLO').NE.0)     INMR_METHOD  = INMR_IGLO
C     No longer supported.
              IF(INDEX(LINE,' CSGT').NE.0)     INMR_METHOD  = INMR_CSGT
              IF(INDEX(LINE,' FAST').NE.0)     TFAST        = .TRUE.
              IF(INDEX(LINE,' NOLOC').NE.0)    TLOCALIZE    = .FALSE.
              IF(INDEX(LINE,' NOVIRT').NE.0)   
     &             INMR_VIRTUAL = INMR_NOVIRT
              IF(INDEX(LINE,' WANNIER').NE.0)  INMR_VIRTUAL = INMR_WC
              IF(INDEX(LINE,' LLC').NE.0)      INMR_VIRTUAL = INMR_LLC
              IF(INDEX(LINE,' FORCEX').NE.0)   TFORCE_XYZ(1)= .TRUE.
              IF(INDEX(LINE,' FORCEY').NE.0)   TFORCE_XYZ(2)= .TRUE.
              IF(INDEX(LINE,' FORCEZ').NE.0)   TFORCE_XYZ(3)= .TRUE.
              IF(INDEX(LINE,' CURRENT').NE.0)  TCURRENT     = .TRUE.
              IF(INDEX(LINE,' VERBOSE').NE.0)  TVERBOSE     = .TRUE.
              IF(INDEX(LINE,' PSI0').NE.0)     TPRINTWFNS   = .TRUE.
              IF(INDEX(LINE,' RHO0').NE.0)     TPRINTRHO    = .TRUE.
              IF(INDEX(LINE,' NOSMOOTH').NE.0) TSMOOTHING   = .FALSE.
              IF(INDEX(LINE,' TRIPACK').NE.0)  NMR_THREADS  = 3
              IF(INDEX(LINE,' SIXPACK').NE.0)  NMR_THREADS  = 6
              CG_ANALYTIC=3
              IF (NMR_THREADS .NE. 1) THEN
                 WFOPT=.FALSE.
                 RESTART=.TRUE.
                 RWF=.TRUE.
              ENDIF
           ENDIF                ! NMR-line
           IF(INDEX(LINE,'ISOLAT').NE.0) THEN
              TNMR_OVERLAPS= .TRUE.
              OVERLAP_THREASHOLD=0.1D0
              TNMR_FULL = .FALSE.
           ENDIF 
           IF(INDEX(LINE,'OVERLAP').NE.0) THEN
              TNMR_OVERLAPS= .TRUE.
              READ(IUNIT,ERR=99,END=99,FMT='(A80)') NEWLINE
              CALL READSR(NEWLINE ,1,IOUT,OVERLAP_THREASHOLD,ERREAD)
              IF(ERREAD) 
     &             CALL STOPGM('RESPIN','ERROR reading ovl threashold')
           ENDIF 
           IF(INDEX(LINE,'FULL').NE.0)    THEN
              TNMR_FULL   = .TRUE.
           ENDIF
C     NMR END


C     ==--  EPR  ---------------------------------------------------==
           IF(INDEX(LINE,' EPR').NE.0)         THEN
              TEPR         = .TRUE.
              IF(INDEX(LINE,' CSGT').NE.0)     IEPR_METHOD   = IEPR_CSGT
              IF(INDEX(LINE,' FAST').NE.0)     TEFAST        = .TRUE.
              IF(INDEX(LINE,' NOLOC').NE.0)    TELOCALIZE    = .FALSE.
              IF(INDEX(LINE,' NOVIRT').NE.0)   
     &             IEPR_VIRTUAL = IEPR_NOVIRT
              IF(INDEX(LINE,' WANNIER').NE.0)  IEPR_VIRTUAL  = IEPR_WC
              IF(INDEX(LINE,' LLC').NE.0)      IEPR_VIRTUAL  = IEPR_LLC
              IF(INDEX(LINE,' FORCEX').NE.0)   TEFORCE_XYZ(1)= .TRUE.
              IF(INDEX(LINE,' FORCEY').NE.0)   TEFORCE_XYZ(2)= .TRUE.
              IF(INDEX(LINE,' FORCEZ').NE.0)   TEFORCE_XYZ(3)= .TRUE.
              IF(INDEX(LINE,' VERBOSE').NE.0)  TEVERBOSE     = .TRUE.
              IF(INDEX(LINE,' PSI0').NE.0)     TEPRINTWFNS   = .TRUE.
              IF(INDEX(LINE,' RHO0').NE.0)     TEPRINTRHO    = .TRUE.
              IF(INDEX(LINE,' NOSMOOTH').NE.0) TESMOOTHING   = .FALSE.
              IF(INDEX(LINE,' HYP').NE.0)      TEPR_HYP   = .TRUE.
              CG_ANALYTIC=99
C Synchronize NMR/EPR options
              INMR_METHOD=IEPR_METHOD
              TFAST=TEFAST
              TLOCALIZE=TELOCALIZE
              INMR_VIRTUAL=IEPR_VIRTUAL
              TFORCE_XYZ(1)=TEFORCE_XYZ(1)
              TFORCE_XYZ(2)=TEFORCE_XYZ(2)
              TFORCE_XYZ(3)=TEFORCE_XYZ(3)
              TVERBOSE=TEVERBOSE
              TPRINTWFNS=TEPRINTWFNS
              TPRINTRHO=TEPRINTRHO
              TSMOOTHING=TESMOOTHING
           ENDIF                ! EPR-line
           IF(INDEX(LINE,'OWNPOT').NE.0) THEN
              TEPR_OWNPOT = .TRUE.
              READ(IUNIT,ERR=99,END=99,FMT='(A80)') NEWLINE
              CALL READSR(NEWLINE ,1,IOUT,EPR_OWNPOT_VALUE,ERREAD)
              IF(ERREAD)
     &             CALL STOPGM('RESPIN','ERROR reading potential 
     &                         smoothing value')
              IF(EPR_OWNPOT_VALUE.LE.0D0)
     &             CALL STOPGM('RESPIN','ERROR reading potential 
     &                         smoothing value (must be > zero)')
           ENDIF
           IF(INDEX(LINE,'SMART').NE.0) THEN
              TEPR_SMART = .TRUE.
              READ(IUNIT,ERR=99,END=99,FMT='(A80)') NEWLINE
              CALL READSR(NEWLINE ,1,IOUT,EPR_SMART_VALUE,ERREAD)
              IF(ERREAD)
     &             CALL STOPGM('RESPIN','ERROR reading threshold
     &                         value in FULL calculation')
              IF((EPR_SMART_VALUE.LT.0D0).OR.(EPR_SMART_VALUE.GT.1D0)) 
     &             CALL STOPGM('RESPIN','ERROR reading threshold
     &              value in FULL calculation (must be >= 0 and <= 1)')
           ENDIF

C These are already read in the NMR section
           TEPR_OVERLAPS = TNMR_OVERLAPS
           TEPR_FULL = TNMR_FULL
C     EPR END


C     ==------------------------------------------------------------==
           if(index(line,'DETAILS').ne.0) details=.true.


C     ==--  INTERACTION  -------------------------------------------==
           if (TINTERACTION) THEN
              IF (INDEX(LINE,' LINEARSCALING').NE.0) THEN
C     We impose only that the response orbitals be orthogonal 
C     to their "own" ground state orbital, i.e. <psi (1)_i | psi(0)_i> = 0 
C     but not for other psi(0)_j.
                 Tlinearscaling = .TRUE.
              ENDIF

              IF (INDEX(LINE,' ORTHOGONAL').NE.0) THEN
C     Then, we want the perturbation orbitals to be orthogonal 
C     between themselves.
                 TINTER_ORTHO = .TRUE.
              ENDIF

              IF(INDEX(LINE,'LMAX').NE.0) THEN
C     reads in the maximum angular momentum used for rotations of Wannier functions
                 READ(IUNIT,ERR=99,END=99,FMT=*) BLmax
              ENDIF

              IF(INDEX(LINE,'NMOL').NE.0) THEN
C     reads in the number of molecules present in the system
                 READ(IUNIT,ERR=99,END=99,FMT=*) NMOL
              ENDIF

              IF(INDEX(LINE,'NSAVESTATE').NE.0) THEN
C     reads in the number of states to display and save at the end of the PT-INTERACTION calculation
                 READ(IUNIT,ERR=99,END=99,FMT=*) max_num_disp
              ENDIF
    
              IF(INDEX(LINE,'SELFCONSISTENT').NE.0) THEN
C     reads in the number of self-consistent iteration for the optimisation of W1
                 READ(IUNIT,ERR=99,END=99,FMT=*) BPTSCFIter
              ENDIF
              IF(INDEX(LINE,'SCFPTTOL').NE.0) THEN
C     reads in the convergence tolerance on the self-consistent iteration for th
                  READ(IUNIT,ERR=99,END=99,FMT=*) SCF_TOL
              ENDIF

              IF(INDEX(LINE,'THETA').NE.0) THEN
C     reads in the rotation angle used for rotations of Wannier functions
C     Allocating memory
                 CALL MEMORY(IP_BTHETA,NMOL,'BTHETA')
C     Reading in the different data
                 DO i=1,NMOL
                    READ(IUNIT,ERR=99,END=99,FMT=*) BTHETA(i)
                 ENDDO
C     Setting flag to initialised!
                 IN_BTHETA=.TRUE.
              ENDIF

              IF(INDEX(LINE,'VEC').NE.0) THEN
C     reads in the rotation axis used for rotations of Wannier functions
C     Allocating memory
                 CALL MEMORY(IP_BVECx,NMOL,'BVECx')
                 CALL MEMORY(IP_BVECy,NMOL,'BVECy')
                 CALL MEMORY(IP_BVECz,NMOL,'BVECz')
C     Reading in the different data
                 DO i=1,NMOL
                    READ(IUNIT,ERR=99,END=99,FMT=*) BVECx(i),BVECy(i),
     $                   BVECz(i)
                 ENDDO
C     Setting flag to initialised!
                 IN_BVEC=.TRUE.
              ENDIF
    
              IF(INDEX(LINE,'DX').NE.0) THEN
C     reads in the displacement vector for the wannier function 
C     Allocating memory
                 CALL MEMORY(IP_BDXVECx,NMOL,'BDXVECx')
                 CALL MEMORY(IP_BDXVECy,NMOL,'BDXVECy')
                 CALL MEMORY(IP_BDXVECz,NMOL,'BDXVECz')
C     Reading in the different data
                 DO i=1,NMOL
                    READ(IUNIT,ERR=99,END=99,FMT=*) BDXVECx(i),
     $                   BDXVECy(i),BDXVECz(i)
                 ENDDO
C     Setting flag to initialised!
                 IN_BDXVEC=.TRUE.
              ENDIF
    
              IF(INDEX(LINE,'WLOAD').NE.0) THEN
C     READING the wannier funtions FROM file
                 WANN_LOAD=.TRUE.
                 WANN_SAVE=.FALSE.
              ENDIF
              
              IF(INDEX(LINE,'WSAVE').NE.0) THEN
C     SAVING the wannier funtions TO a file
                 WANN_LOAD=.FALSE.
                 WANN_SAVE=.TRUE.
              ENDIF
              
              IF(INDEX(LINE,'NOROT').NE.0) THEN
C     rotation by zero degrees is made without projection
                 WANN_ALLROT=.FALSE.
              ENDIF
              
              IF(INDEX(LINE,'WGAUSSIAN').NE.0) THEN
C     Using a Gaussian representation for the input/output of Wannier functions 
                 WANN_GAUSS=.TRUE.
              ENDIF
              
              IF(INDEX(LINE,'MULTIREF').NE.0) THEN
C     Allowing a reference wannier file for each molecule
                 WANN_MULTI=.TRUE.
              endif
              
              IF(INDEX(LINE,'TRICK').NE.0) THEN
C     Cutoff trick (lower cutoff for PT wavefunction)    
                 CUTOFF_TRICK=.TRUE.
              endif
              
              IF(INDEX(LINE,'RESTRAINED').NE.0) THEN
C     Cutoff is lowered for the perturbation calculation 
                 CUTOFF_RESTR=.TRUE.
              endif
              
C     Disables the localisation of the optimised wf 
              IF(INDEX(LINE,' NOLOC').NE.0) TLOCALIZE = .FALSE.
              
C     Enables the Wannier centre prediction routine (for BOdyn ONLY...)
              IF (INDEX(LINE,' WCPREDICT').NE.0) THEN
                 WCPREDICT=.TRUE.
C     initialises the sample counter
                 WC_sample=1
C                 reads the number of MD steps between localisation cycles 
                  READ(IUNIT,ERR=99,END=99,FMT=*) WC_PRED_STEP
              endif

C     Uses LDA to calculate the d2Exc/dn1 dn2 in PT-GGA calculations
               IF(INDEX(LINE,' FASTDEXC').NE.0) FASTDEXC = .TRUE.

C     Saves the orthogonalised W0 in a RESTART file and quits
               IF(INDEX(LINE,' SAVELOCW0').NE.0) 
     $            SAVE_LOCALISED_W0 = .TRUE.

C     Use the atomic wavefunction for W0                     
               IF(INDEX(LINE,' ATOMICWAVEFUNCTION').NE.0) 
     $            TATOMICWAVEFUNCTION = .TRUE.
C     Use the atomic density with Wannier W0
               IF(INDEX(LINE,' ATOMICRHO').NE.0) 
     $            TATOMICRHO = .TRUE.
C     Use non orthogonal Wannier combination
               IF(INDEX(LINE,' NONORTHOGWANNIER').NE.0) 
     $            TORTHOG_WANNIER = .FALSE.
C     Use simplified interaction model
               IF(INDEX(LINE,' SIMPLEMODEL').NE.0) 
     $            TSIMPLE_MODEL = .TRUE.
C     Use OLD MANNO PT-INTER code (only works in serial)
               IF(INDEX(LINE,' MANNOPT').NE.0) 
     $            TMANNO = .TRUE.
           endif                ! INTERACTION-parameters
           
C     ==--  GENERAL  -----------------------------------------------==
C..   REALSPACE representation of the ground state orbitals:
           IF(INDEX(LINE,'REALSPACE').NE.0) THEN
              TKEEPREALSPACEWFN = .true.
           ENDIF
C..   Polak-Ribieres-Variant of the conjugate gradients:
           IF(INDEX(LINE,'POLAK').NE.0)  THEN
              TPOLAK_P=.TRUE.
           ENDIF
C..   Additional optimization control options:
           IF(INDEX(LINE,'NOOPT').NE.0)  THEN
              WFOPT=.FALSE.
              RESTART=.TRUE.
              RWF=.TRUE.
           ENDIF
C     Number of steps for which the CG-length is calculated analytically
           IF(INDEX(LINE,'CG-ANALYTIC').NE.0)  THEN
              READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
              CALL READSI(LINE ,1,IOUT,CG_ANALYTIC,ERREAD)
              IF(ERREAD) GOTO 99
           ENDIF
C     Factor the last analytic length is multiplied with,
C     for istep > CG-ANALYTIC
           IF(INDEX(LINE,'CG-FACTOR').NE.0)  THEN
              READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
              CALL READSR(LINE ,1,IOUT,CG_FACTOR,ERREAD)
              IF(ERREAD) GOTO 99
           ENDIF
C     Preconditioner type (divergence smoothing formula)
           IF(INDEX(LINE,'TIGHTPREC').NE.0) THEN
              PRECONDITIONER_P=2
C     State dependent preconditioner 
C     DS variant
           ELSEIF(INDEX(LINE,'STATEPREC').NE.0) THEN
              PRECONDITIONER_P=3
C     AP variant
           ELSEIF(INDEX(LINE,'TIGHTSTATEPREC').NE.0) THEN
              PRECONDITIONER_P=4
           ENDIF
           IF(INDEX(LINE,'NOMIN').NE.0)  PCGMIN_P=.FALSE.

C     Used by no-pcg, for a first step with pcg
           IF(INDEX(LINE,'PCG_FIRST').NE.0)  OPT1st=.TRUE.


C..   DIIS optimization (preconditioning active)
           IF( INDEX(LINE,'DIIS').NE.0) THEN
              PCG_P=.FALSE.
              TSDE_P=.FALSE.
              DIIS_P=.TRUE.
           ENDIF
           IF (INDEX(LINE,'MDIIS').NE.0)THEN
              PCG_P=.FALSE.
              TSDE_P=.FALSE.
              DIIS_P=.TRUE.
              READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
              CALL READSI(LINE,1,IE,MDIIS_p,ERREAD)
              IF(ERREAD) GOTO 99
           ENDIF
C..   Steepest descent:
           IF( INDEX(LINE,'STEE').NE.0 .AND. 
     &          INDEX(LINE,'DESC').NE.0) THEN
              PCG_P=.FALSE.
              TSDE_P=.TRUE.
              call stopgm('RESPIN_P',
     &             'WHO THE HELL is sill using steepest descent?')
           ENDIF
           IF(INDEX(LINE,'PCG_P').NE.0) THEN
C..   Precond. Conjugate Gradients
              TSDE_P=.FALSE.
              PCG_P=.TRUE.
              DIIS_P=.FALSE.
           ENDIF
C     Convergence threashold (max gradient)
           IF(INDEX(LINE,'CONVERGE').NE.0) THEN
              READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
              CALL READSR(LINE ,1,IOUT,TOLOG_P,ERREAD)
              IF(ERREAD) GOTO 99
           ENDIF
           IF  ((INDEX(LINE,'HTHRS').NE.0) .OR.
     &          (INDEX(LINE,'HAM')*INDEX(LINE,'CUT').NE. 0)) THEN
C..   Threshold for Hessian cutoff in preconditioning matrix
              READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
              CALL READSR(LINE ,1,IOUT,HTHRS_P,ERREAD)
              IF(ERREAD) GOTO 99
           ENDIF
           IF  (INDEX(LINE,'SIGMA').NE.0) THEN
C..   The spread of the Gaussian in the virtual cell calculation
              READ(IUNIT,ERR=99,END=99,FMT='(A80)') LINE
              CALL READSR(LINE ,1,IOUT,sigma,ERREAD)
              IF(ERREAD) GOTO 99
           ENDIF
C     ==------------------------------------------------------------==
        ENDDO                   ! while READ_MORE
C     ==------------------------------------------------------------==
C     ==------------------------------------------------------------==
C     If we are loading a saved wavefunction:
C     no optimisations and restart (use different file format..)
        if ((WANN_LOAD.OR.TATOMICWAVEFUNCTION) 
     &       .AND. TINTERACTION) THEN
           WFOPT = .false.
           RESTART = .false.
           RWF = .false.
        endif
        if(parent.AND.TMANNO.AND.(NPROC.GT.1)) then
           write(6,*) 'Old MANNOPT code is not parallel...'
           write(6,*) 'Switching to PARA-PT instead'
           TMANNO=.FALSE.
        endif
C     ==------------------------------------------------------------==
      ENDIF                     ! parent

#ifdef PARALLEL
      MSGLEN = LRESPONSE *8/IRAT
      CALL MY_BCAST(RESPONSE_RUNNING,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = RRESPONSE *8
      CALL MY_BCAST(HTHRS_P,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = L_EIGEN_INTEGERS * 8/IRAT
      CALL MY_BCAST(LANCZOS_DIM,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = L_EIGEN_REALS * 8
      CALL MY_BCAST(CONV_THRESHOLD,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = L_nmr_options * 8/IRAT
      CALL MY_BCAST(T_NMR_FIRST,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = L_epr_options * 8/IRAT
      CALL MY_BCAST(T_EPR_FIRST,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(NMR_THREADS,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 1 * 8
      CALL MY_BCAST(SIGMA,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(OVERLAP_THREASHOLD,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EPR_OWNPOT_VALUE,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(EPR_SMART_VALUE,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = LENKPERT *8/IRAT
      CALL MY_BCAST(RESTART_CU1,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = LINTERACTION *8/IRAT
      CALL MY_BCAST(INTERACTION_FIRST,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 1 *8
      CALL MY_BCAST(SCF_TOL,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = 1 *8/IRAT
      CALL MY_BCAST(wfopt,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(restart,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(rwf,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(DETAILS,MSGLEN,SOURCE,ALLGRP)
      call MY_BCAST(nlan_st,msglen,source,allgrp)
      msglen= 1 *8/irat
      CALL MY_BCAST(numf,msglen,source,allgrp)
      CALL MY_BCAST(tweight,msglen,source,allgrp)
      if (numf.ne.0) then
         if (.not.parent) CALL MEMORY(IP_NF,NUMF,'NF')
         if (.not.parent) CALL MEMORY(IP_WGHTF,NUMF+1,'WGHTF')
         msglen=numf*8
         call my_bcast(nf,msglen,source,allgrp)
         msglen=numf*8+8
         call my_bcast(wghtf,msglen,source,allgrp)
      endif
C     ==------------------------------------------------------------==
      IF (.NOT. PARENT) THEN
         IF (TINTERACTION) THEN
            IF (IN_BTHETA) THEN
               CALL MEMORY(IP_BTHETA,NMOL,'BTHETA')
               MSGLEN = NMOL*8
               CALL MY_BCAST(BTHETA(1),MSGLEN,SOURCE,ALLGRP)
            ENDIF
            IF (IN_BVEC) THEN
               CALL MEMORY(IP_BVECx,NMOL,'BVECx')
               CALL MEMORY(IP_BVECy,NMOL,'BVECy')
               CALL MEMORY(IP_BVECz,NMOL,'BVECz')
               MSGLEN =  NMOL * 8
               CALL MY_BCAST(BVECx,MSGLEN,SOURCE,ALLGRP)
               CALL MY_BCAST(BVECy,MSGLEN,SOURCE,ALLGRP)
               CALL MY_BCAST(BVECz,MSGLEN,SOURCE,ALLGRP)
            ENDIF
            IF (IN_BDXVEC) THEN
               CALL MEMORY(IP_BDXVECx,NMOL,'BDXVECx')
               CALL MEMORY(IP_BDXVECy,NMOL,'BDXVECy')
               CALL MEMORY(IP_BDXVECz,NMOL,'BDXVECz')
               MSGLEN =  NMOL * 8
               CALL MY_BCAST(BDXVECx,MSGLEN,SOURCE,ALLGRP)
               CALL MY_BCAST(BDXVECy,MSGLEN,SOURCE,ALLGRP)
               CALL MY_BCAST(BDXVECz,MSGLEN,SOURCE,ALLGRP)
            ENDIF
         ENDIF
      ENDIF
C     ==------------------------------------------------------------==
#endif
C     ==------------------------------------------------------------==
C     Are all options consistent? [Only one response at a time...!]
      IRESPONSE=0
      IF (TNMR) IRESPONSE=IRESPONSE+1
      IF (TEPR) IRESPONSE=IRESPONSE+1
      IF (TRAMAN) IRESPONSE=IRESPONSE+1
      IF (TEIGENSYSTEM) IRESPONSE=IRESPONSE+1
      IF (PHONON) IRESPONSE=IRESPONSE+1
      IF (THARDNESS) IRESPONSE=IRESPONSE+1
      IF (TFUKUI) IRESPONSE=IRESPONSE+1
      IF (TINTERACTION) IRESPONSE=IRESPONSE+1
      IF (TLANPHON) IRESPONSE=IRESPONSE+1
      IF (TKPERT) IRESPONSE=IRESPONSE+1
      IF (TDUMMYATOM) IRESPONSE=IRESPONSE+1

      IF ((.NOT.TINR).AND.(IRESPONSE.NE.1)) THEN
         IF (PARENT) THEN
            WRITE (6,*) 'PLEASE CHOOSE EXACTLY ONE OF THE FOLLOWING ',
     &          '&RESP OPTIONS:'
            WRITE (6,*) '       NMR'
            WRITE (6,*) '       EPR'
            WRITE (6,*) '       RAMAN'
            WRITE (6,*) '       PHONON'
            WRITE (6,*) '       EIGENSYSTEM'
            WRITE (6,*) '       HARDNESS'
            WRITE (6,*) '       FUKUI'
            WRITE (6,*) '       INTERACTION'
            WRITE (6,*) '       LANCZOS'
            WRITE (6,*) '       KPERT'
            WRITE (6,*) '       OAPC'
         ENDIF
         CALL STOPGM('RESPIN','REQUESTED FLAGS INCONSISTENT '
     &       // 'WITH LINEAR RESPONSE CODE')
      ENDIF

      IF(TKPERT)THEN
         TSYMKP=.FALSE.
         TKFULL=.FALSE.
         TKALL=.FALSE.
         TKBLOCK= .FALSE.
         NKPNT=1
         NBLKP=1
      ENDIF
      IF(TKPERT .AND. TKPNT) THEN
         CALL STOPGM
     &        ('  RESPIN',
     &        'STANDARD K-POINT AND KPERT ARE NOT CONSISTENT')
      ENDIF
      

      RETURN
C     ==--------------------------------------------------------------==
 99   CONTINUE
      CALL STOPGM('RESPIN',' ERROR IN READING INPUT FILE')
      END
C     ==--------------------------------------------------------------==
