C     ==================================================================
      SUBROUTINE RESTART_P(C1,tag,nstate,ireadwrite)
C     ==--------------------------------------------------------------==
C     == Read (1) or write (2) one wavefunction in Version 3.0 format ==
C     == into a file: RESTART.tag.                                    ==
C     ==                                                              ==
C     == IF THE FILE DOES NOT EXIST (for the read case), then the     ==
C     == wavefunction is just put to zero. No error is issued.        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h' 
      INCLUDE 'coor.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'response_p.inc'
      INCLUDE 'fileopen.inc'
C     Arguments
      INTEGER    NSTATE,IREADWRITE
      CHARACTER  tag*32
      COMPLEX*16 C1(NGW,*)
C     Variables
      INTEGER    fileunit
      CHARACTER  FILENAME*128
      INTEGER    IA,IE,ISUB,is,ir,info,controlbyte
      integer    nstate_,ngws_,llc(3),irecord
      real*8     a1_(3),a2_(3),a3_(3)
      INTEGER    IERROR
      LOGICAL    FEXIST,error
#ifdef PARALLEL
      INTEGER    MSGLEN
#endif
C     ==--------------------------------------------------------------==
      CALL TISET(' RESTART_P',ISUB)

C     Construct the filename
      fileunit = 2602
      call xstring(tag,ia,ie)
      filename = 'RESTART.'//tag(ia:ie)
      call xstring(filename,ia,ie)

C     ==--------------------------------------------------------------==
      if (ireadwrite .eq. RESPONSE_WRITE) then ! WRITE mode
         if (parent) then
            error=.false.
            CALL FILEOPEN(fileunit,filename(ia:ie),FO_DEF+FO_UFO,error)
            REWIND (fileunit,ERR=401)
            goto 402
 401        continue
            error=.true.
 402        continue
            if (error) call stopgm('RESTART_P',
     &           'ERROR OPENING RESTART FILE ('
     &           //filename(ia:ie)//') for writing.')

            controlbyte = 260273
            write (fileunit) controlbyte
            write (fileunit) a1,a2,a3
            write (fileunit) nstate,ngws
         endif                  ! parent
C     ==--------------------------------------------------------------==
         IERROR=0
         CALL WR30WFN(fileunit,IERROR,nstate,C1,tau0,'NIL')
         IF (IERROR.NE.0 .AND. parent) THEN
            write (6,'(A,I4,A)') 'RESTART_P: ERROR ',IERROR,
     &           ' WRITING RESTART_P file! File is corrupt!'
         ENDIF
C     ==--------------------------------------------------------------==
         if(parent) then
            call xstring(filename,IA,IE)
            write (6,'(A)') 
     &           'RESPONSE wavefunction written to RESTART file '
     &           //filename(IA:IE)//'.'
            CALL FILECLOSE(fileunit)
         endif
C     ==--------------------------------------------------------------==
      elseif (ireadwrite .eq. RESPONSE_READ) then ! READ mode
C     ==--------------------------------------------------------------==
         if (parent) then 
           error=.false.
           CALL FILEOPEN(fileunit,filename(ia:ie),FO_OLD+FO_UFO,error)
           IF(error) call stopgm('RESTART_P',
     &           'RESTART FILE NOT FOUND: '//filename(ia:ie)//'.')
C     if the file _does_ exist: read it
           rewind (fileunit,ERR=501)
           error=.false.
           goto 502
 501       continue
           error=.true.
 502       continue
           if (error) then
             call stopgm('RESTART_P',
     &            'ERROR OPENING RESTART FILE ('
     &            //filename(ia:ie)//') for reading.')
           endif
C     ==--------------------------------------------------------------==
C     GENERAL PART of restart file:
            read (fileunit) controlbyte
            read (fileunit) a1_,a2_,a3_
            read (fileunit) nstate_,ngws_
            error=.false.
            if (controlbyte .ne. 260273)
     &           error = .true.
            do ir=1,3
               if ((a1(ir) .NE. a1_(ir))
     &              .OR. (a2(ir) .NE. a2_(ir))
     &              .OR. (a3(ir) .NE. a3_(ir)))
     &              error = .true.
            enddo
            if ((nstate_.NE.nstate).OR.(ngws.NE.ngws_))
     &           error = .true.
            if (error) then
              call stopgm('RESTART_P',
     &             'ERROR READING RESTART FILE ('
     &             //filename(ia:ie)//'): wrong parameters.')
            endif
C     ==--------------------------------------------------------------==
            READ(fileunit) irecord ! dummy read.
          endif                 ! parent
          CALL RD30WFN(fileunit,C1,NSTATE,TAU0,INFO,
     &         .FALSE.,NSTATE,1,'NIL')
C     ==--------------------------------------------------------------==
         if(parent) then
            call xstring(filename,IA,IE)
            write (6,'(A)') 
     &           'RESPONSE wavefunction read from RESTART file '
     &           //filename(IA:IE)//'.'
            CALL FILECLOSE(fileunit)
         endif
         T_INITIALGUESS_C1 = .TRUE.
      else                      ! read-write mode
         call stopgm('RESTART_P','BAD RESTART MODE.')
      endif                     ! read - write
C     ==--------------------------------------------------------------==
      CALL TIHALT(' RESTART_P',ISUB)
      RETURN
      END
C     ==================================================================





c     ==================================================================
C     The SAVE/RESTORE combination needs a little explanation for 
C     use in parallel. When several processors keep only their parts of
C     the shift and chi matrices, a proper save/restore is not easy.
C     Instead, in this case, a global sum is done at each save.
C     
C     When restoring, only the parent reads the file, whereas the other
C     processors assume to have initially empty (zero) shift
C     and chi matrices.

c     ==================================================================
      SUBROUTINE restart_nmr(simple_done,full_done,
     &     shift_sumup,chi_sumup,
     &     shift_saved,chi_saved,nstate,ireadwrite)
      implicit none
      include 'system.h'
      include 'irat.inc'
      include 'response_p.inc'
      include 'fileopen.inc'

C     Arguments:
      integer     nstate,ireadwrite
      real*8 shift_sumup(3,3,nax,nsx),chi_sumup(3,3),
     &       shift_saved(3,3,nax,nsx),chi_saved(3,3)
      logical     simple_done(6),full_done(3,nstate)

C     Variables
      integer     msglen,fileunit
      character   filename*128
      real*8      scr(3*3*nax*nsx,2),scr2(3*3,2),a1_(3),a2_(3),a3_(3)
      pointer     (ip_scr,scr),(ip_scr2,scr2)
      integer     is,ir,controlbyte,cbyte,ia,ie,nstate_,ngws_,ll_(3)
      logical     error,errors

      fileunit=2604
      controlbyte=19732604
      filename='RESTART.NMR'
      call xstring(filename,ia,ie)
      call memory(ip_scr,2*3*3*nax*nsx,'SAVE-ERR/SCR')
      call memory(ip_scr2,2*3*3,'SAVE-ERR/SCR2')

      if (ireadwrite .eq. RESPONSE_WRITE) then ! WRITE MODE
C     ==--------------------------------------------------------------==
C     Preparation:
         call DCOPY(3*3*nax*nsx,shift_saved,1,scr(1,1),1)
         call DCOPY(3*3*nax*nsx,shift_sumup,1,scr(1,2),1)
         call DCOPY(3*3,chi_saved,1,scr2(1,1),1)
         call DCOPY(3*3,chi_sumup,1,scr2(1,2),1)
#ifdef PARALLEL
         call glosum(2*3*3,scr2)
         call glosum(2*3*3*nax*nsx,scr)
#endif

         if (parent) then
            error = .false.
            CALL FILEOPEN(fileunit,filename(ia:ie),FO_DEF+FO_UFO,error)
            rewind (fileunit)
            write (fileunit) controlbyte
            write (fileunit) a1,a2,a3
            write (fileunit) nstate,ngws
            do is=1,nstate
               write (fileunit) (lower_left(ir,is),ir=1,3)
            enddo
            write (fileunit) simple_done
            write (fileunit) full_done
            write (fileunit) scr
            write (fileunit) scr2
            CALL FILECLOSE(fileunit)
         endif                  ! Parent
C     ==--------------------------------------------------------------==
      elseif (ireadwrite .eq. RESPONSE_READ) then ! READ MODE
C     ==--------------------------------------------------------------==
C     OPEN the file
         if (parent) then
            error = .false.
            CALL FILEOPEN(fileunit,filename(ia:ie),FO_OLD+FO_UFO,error)
            if (error) then
              call stopgm('RESTART_P',
     &             'ERROR OPENING RESTART FILE ('
     &             //filename(ia:ie)//'.')
            endif
          endif
C     ==--------------------------------------------------------------==
C     READ the file and check control parameters
         if (parent) then
            rewind(fileunit)
            errors = .false.
            read (fileunit) cbyte
            read (fileunit) a1_,a2_,a3_
            read (fileunit) nstate_,ngws_
            error =  (cbyte .ne. controlbyte)
            errors = errors .or. error
            if (error) write (6,*) 'CONTROL BYTE CHANGED.'
            error =  (nstate_ .ne. nstate)
            errors = errors .or. error
            if (error) write (6,*) 'NSTATE CHANGED.'
            error =  (ngws_ .ne. ngws)
            errors = errors .or. error
            if (error) write (6,*) 'NGWS CHANGED.'
            do ir=1,3
               error = (abs(a1(ir)-a1_(ir)).GE.1d-9)
     &              .or. (abs(a2(ir)-a2_(ir)).GE.1d-9)
     &              .or. (abs(a3(ir)-a3_(ir)).GE.1d-9)
               errors = errors .or. error
               if (error) write (6,*) 'UNIT CELL CHANGED.'
            enddo
            do is=1,nstate
               read (fileunit) (ll_(ir),ir=1,3)
               do ir=1,3
                  error = (ll_(ir) .ne. lower_left(ir,is))
                  errors = errors .or. error
                  if (error) write (6,*) 'LLC CHANGED.'
               enddo
            enddo
         endif                  ! parent
#ifdef PARALLEL
         msglen=1*8/IRAT
         call my_bcast(errors,msglen,SOURCE,ALLGRP)
#endif
         if (errors) then
            call stopgm('RESTART_P',
     &           'ERROR READING RESTART FILE ('
     &           //filename(ia:ie)//': parameters changed!!')
         endif
C     ==--------------------------------------------------------------==
C     READ the values calculated so far
         if (parent) then
            read (fileunit) simple_done
            read (fileunit) full_done
            read (fileunit) scr
            read (fileunit) scr2
            CALL FILECLOSE(fileunit)
            call DCOPY(3*3*nax*nsx,scr(1,1),1,shift_saved,1)
            call DCOPY(3*3*nax*nsx,scr(1,2),1,shift_sumup,1)
            call DCOPY(3*3,scr2(1,1),1,chi_saved,1)
            call DCOPY(3*3,scr2(1,2),1,chi_sumup,1)
         else
            call azzero(shift_saved,3*3*nax*nsx)
            call azzero(shift_sumup,3*3*nax*nsx)
            call azzero(chi_saved,3*3)
            call azzero(chi_sumup,3*3)
         endif
#ifdef PARALLEL
         msglen = 6*8/IRAT
         call my_bcast(simple_done,msglen,source,allgrp)
         msglen = 3*nstate*8/IRAT
         call my_bcast(full_done,msglen,source,allgrp)
#endif
         if(parent) then
            write (6,'(A)') 
     &           'NMR RESTART DATA read from file '
     &           //filename(IA:IE)//'.'
         endif
      else                      ! READ/WRITE MODES.
         call stopgm('RESTART_NMR','BAD RESTART MODE.')
      endif                     ! read - write

      call freem(ip_scr)
      call freem(ip_scr2)

      RETURN
      END
c     ==================================================================


c     ==================================================================
      SUBROUTINE restart_epr(simple_done,
     &     nstate,ireadwrite)
      implicit none
      include 'system.h'
      include 'irat.inc'
      include 'response_p.inc'
C     Arguments:
      integer     nstate,ireadwrite
      logical     simple_done(9)

C     Variables
      integer     msglen,fileunit
      character   filename*128
      real*8      a1_(3),a2_(3),a3_(3)
      integer     is,ir,controlbyte,cbyte,ia,ie,nstate_,ngws_,ll_(3)
      logical     error,errors

      fileunit=2604
      controlbyte=19732604
      filename='RESTART.EPR'
      call xstring(filename,ia,ie)

      if (ireadwrite .eq. RESPONSE_WRITE) then ! WRITE MODE
C     ==--------------------------------------------------------------==
         if (parent) then
            OPEN(UNIT=fileunit,FILE=filename,
     &           STATUS='UNKNOWN',FORM='UNFORMATTED')
            rewind (fileunit)
            write (fileunit) controlbyte
            write (fileunit) a1,a2,a3
            write (fileunit) nstate,ngws
            do is=1,nstate
               write (fileunit) (lower_left(ir,is),ir=1,3)
            enddo
            write (fileunit) simple_done
            close(fileunit)
         endif                  ! Parent
C     ==--------------------------------------------------------------==
      elseif (ireadwrite .eq. RESPONSE_READ) then ! READ MODE
C     ==--------------------------------------------------------------==
C     OPEN the file
         if (parent) then
            error = .false.
            OPEN(UNIT=fileunit,FILE=filename,
     &           STATUS='OLD',FORM='UNFORMATTED')
            goto 602
 601        error=.true.
            rewind(fileunit)
 602        continue
         endif
#ifdef PARALLEL
         msglen=1*8/IRAT
         call my_bcast(error,msglen,SOURCE,ALLGRP)
#endif
         if (error) then
            call stopgm('RESTART_P',
     &           'ERROR OPENING RESTART FILE ('
     &           //filename(ia:ie)//'.')
         endif
C     ==--------------------------------------------------------------==
C     READ the file and check control parameters
         if (parent) then
            errors = .false.
            read (fileunit) cbyte
            read (fileunit) a1_,a2_,a3_
            read (fileunit) nstate_,ngws_
            error =  (cbyte .ne. controlbyte)
            errors = errors .or. error
            if (error) write (6,*) 'CONTROL BYTE CHANGED.'
            error =  (nstate_ .ne. nstate)
            errors = errors .or. error
            if (error) write (6,*) 'NSTATE CHANGED.'
            error =  (ngws_ .ne. ngws)
            errors = errors .or. error
            if (error) write (6,*) 'NGWS CHANGED.'
            do ir=1,3
               error = (abs(a1(ir)-a1_(ir)).GE.1d-9)
     &              .or. (abs(a2(ir)-a2_(ir)).GE.1d-9)
     &              .or. (abs(a3(ir)-a3_(ir)).GE.1d-9)
               errors = errors .or. error
               if (error) write (6,*) 'UNIT CELL CHANGED.'
            enddo
            do is=1,nstate
               read (fileunit) (ll_(ir),ir=1,3)
               do ir=1,3
                  error = (ll_(ir) .ne. lower_left(ir,is))
                  errors = errors .or. error
                  if (error) write (6,*) 'LLC CHANGED.'
               enddo
            enddo
         endif                  ! parent
#ifdef PARALLEL
         msglen=1*8/IRAT
         call my_bcast(errors,msglen,SOURCE,ALLGRP)
#endif
         if (errors) then
            call stopgm('RESTART_P',
     &           'ERROR READING RESTART FILE ('
     &           //filename(ia:ie)//': parameters changed!!')
         endif
C     ==--------------------------------------------------------------==
C     READ the values calculated so far
         if (parent) then
            read (fileunit) simple_done
            close(fileunit)
         endif
#ifdef PARALLEL
         msglen = 9*8/IRAT
         call my_bcast(simple_done,msglen,source,allgrp)
#endif
         if(parent) then
            call xstring(filename,IA,IE)
            write (6,'(A)') 
     &           'EPR RESTART DATA read from file '
     &           //filename(IA:IE)//'.'
            close(fileunit)
         endif
      else                      ! READ/WRITE MODES.
         call stopgm('RESTART_EPR','BAD RESTART MODE.')
      endif                     ! read - write

      RETURN
      END
c     ==================================================================



