C     ==================================================================
      SUBROUTINE RGGEN
C     ==--------------------------------------------------------------==
C     == GENERATES THE RECIPROCAL LATTICE VECTORS WITH LENGTH SQUARED ==
C     == LESS THAN GCUT, AND RETURNS THEM IN ORDER OF INCREASING      ==
C     == LENGTH.                                                      ==
C     == G=I*B1+J*B2+K*B3, WHERE B1,B2,B3 ARE THE VECTORS DEFINING    ==
C     == THE RECIPROCAL LATTICE, AND I,J,K EACH GO FROM -(NR-1) TO    ==
C     == +(NR-1).                                                     ==
C     == N1,N2, AND N3 ARE THE FAST-FOURIER TRANSFORM INDICES OF G.   ==
C     == IF I.GE.0, N1=I+1; IF I.LT.0, N1=I+1+NR, WITH SIMILAR        ==
C     == DEFINITIONS FOR N2 AND N3. THIS CONNECTS G S WITH NEGATIVE   ==
C     == COEFFICIENTS C TO THOSE WITH POSITIVE COEFFICIENTS BY        ==
C     == INTRODUCING A FACTOR EXP(M*2PI*I)=1 IN THE FOURIER TRANSFORM.==
C     == GCUT SHOULD BE .GE. GCUTW. NG IS THE TOTAL NUMBER OF VECTORS ==
C     == WITH LENGTH SQUARED LESS THAN GCUT. NGW IS THE TOTAL NUMBER  ==
C     == OF VECTORS WITH LENGTH SQUARED LESS THAN GCUTW.              ==
C     == THE G S ARE IN UNITS OF 2PI/A.                               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'gvec.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'cell.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'sphe.inc'        !GCUTWMAX
      INCLUDE 'andr.inc'
      INCLUDE 'broy.inc' 
      INCLUDE 'fileopen.inc'
C     Variables
      REAL*8    DELTA
      PARAMETER (DELTA=1.D-6)
      INTEGER   ISUB,NH1,NH2,NH3,IG,I,J,K,INDY1,INDY2,INDY3,NSHL,
     &          ISH,IG1,INFO,IGS
      REAL*8    G2,T1,T2,T3,HGOLD,DIFF,GLEN,GCUTBROY
      LOGICAL   DEBUG,FERROR
C     ==--------------------------------------------------------------==
      DEBUG=.FALSE.
      CALL TISET('     RGGEN',ISUB)
C     Leading dimensions
      CALL LEADIM(NR1,NR2,NR3,KR1,KR2,KR3)
      NNR1=KR1*KR2*KR3
      NH1=NR1/2+1
      NH2=NR2/2+1
      NH3=NR3/2+1
C     Global System Parameters
      CALL GLOPAR
C     Generate Taskgroups
      CALL GROUPS
#ifdef PARALLEL
C     Distribution of Work and Information to all Nodes
      CALL LOADPA
#else
      CALL LOADSE
#endif
      CALL ALIASING
#if defined(__VECTOR)
C$OMP parallel do private(IG,I,J,K,T1,T2,T3)
#else
C$OMP parallel do private(IG,I,J,K,T1,T2,T3) schedule(static)
#endif
      DO IG=1,NHG
        I=INYH(1,IG)-NH1
        J=INYH(2,IG)-NH2
        K=INYH(3,IG)-NH3
        T1=DBLE(I)*B1(1)+DBLE(J)*B2(1)+DBLE(K)*B3(1)
        T2=DBLE(I)*B1(2)+DBLE(J)*B2(2)+DBLE(K)*B3(2)
        T3=DBLE(I)*B1(3)+DBLE(J)*B2(3)+DBLE(K)*B3(3)
        HG(IG)=T1*T1+T2*T2+T3*T3
      ENDDO
C     Broyden mixing cutoff and number of plane waves 
      NGBROY=0
      IF(ECUTBROY.LE.0.D0) THEN
        ECUTBROY=GCUT*TPIBA2
        GCUTBROY=GCUT
      ELSE
        GCUTBROY=ECUTBROY/TPIBA2
      ENDIF
      IF(TGBROY) THEN
        DO IG=1,NHG
          G2=HG(IG)
          IF(G2.LT.GCUTBROY) THEN 
            NGBROY=NGBROY+1
          ENDIF
        ENDDO
      ENDIF
      IF(DEBUG) THEN
C       Test of reordering
        INFO=0
        DO IG=2,NHG
          IF(HG(IG)-HG(IG-1).LT.-EPSG) THEN
            WRITE(6,*) 'PROC=',ME,' IG=',IG,
     &           ' HG(IG)=',HG(IG),' HG(IG-1)=',HG(IG-1)
            INFO=1
          ENDIF
        ENDDO
        IF(INFO.EQ.1) CALL STOPGM('RGGEN','BAD REORDERING')
      ENDIF
      CALL MEMORY(IP_NZH,NHG/IRAT+1,'NZH')
      CALL MEMORY(IP_INDZ,NHG/IRAT+1,'INDZ')
      DO IG=1,NHG
        INDY1=INYH(1,IG)
        INDY2=INYH(2,IG)
        INDY3=INYH(3,IG)
        NZH(IG) = INDY1 + (INDY2-1)*KR1S + (INDY3-1)*KR1S*KR2S
        INDY1=-INDY1+NH1*2
        INDY2=-INDY2+NH2*2
        INDY3=-INDY3+NH3*2
        INDZ(IG) = INDY1 + (INDY2-1)*KR1S + (INDY3-1)*KR1S*KR2S
      ENDDO
C     Number of Shells
      NHGL=1
      HGOLD=HG(1)
      DO IG=2,NHG
        IF(ABS(HG(IG)-HGOLD).GT.DELTA) THEN
          NHGL=NHGL+1
          HGOLD=HG(IG)
          IF(HG(IG).LT.GCUTWMAX) NGWL=NHGL
        ENDIF
      ENDDO
      IF(NGWL.EQ.0) THEN
        WRITE(6,'(1X,A)')
     &       'INCREASE THE FOURIER MESH OR THE ENERGY CUTOFF'
        CALL STOPGM('RGGEN','MESH TOO SMALL')
      ENDIF
      CALL MEMORY(IP_GL,NHGL,'GL')
      NSHL=1
      GL(1)=HG(1)
      DO IG=2,NHG
        IF(ABS(HG(IG)-GL(NSHL)).GT.DELTA) THEN
          NSHL=NSHL+1
          IF(NSHL.GT.NHGL) CALL STOPGM('RGGEN', 'NSHL BIGGER THAN NHGL')
          GL(NSHL)=HG(IG)
        ENDIF
      ENDDO
      CALL MEMORY(IP_GK,3*NHG,'GK')
      CALL MEMORY(IP_IGL,NHG/IRAT+1,'IGL')
      CALL MEMORY(IP_ISPTR,(NHGL+4)/IRAT+1,'ISPTR')
C     Pointers PW --> Shell
      NSHL=1
      DO IG=1,NHG
  100   CONTINUE
        IF(ABS(HG(IG)-GL(NSHL)).GT.DELTA) THEN
          NSHL=NSHL+1
          GOTO 100
        ELSE
          IGL(IG)=NSHL
        ENDIF
      ENDDO
      IF (DEBUG) THEN
C       Test of shell reordering
        INFO=0
        DO ISH=2,NSHL
          IF(GL(ISH)-GL(ISH-1).LT.-EPSG) THEN
            WRITE(6,*) 'PROC=',ME,' ISH=',ISH,
     &           ' GL(ISH)=',GL(ISH),' GL(ISH-1)=',GL(ISH-1)
            INFO=1
          ENDIF
        ENDDO
        IF(INFO.EQ.1) CALL STOPGM('RGGEN','BAD SHELL ORDERING')
      ENDIF
C     Pointers to Shells
      DO ISH=1,NHGL
        GLEN=GL(ISH)
        IG1=1
        IF(ISH.GT.1) IG1=ISPTR(ISH-1)
        DO IG=IG1,NHG
          DIFF=HG(IG)-GLEN
          IF(ABS(DIFF).LE.DELTA .OR. DIFF.GT.DELTA) THEN
            ISPTR(ISH)=IG
            GOTO 200
          ENDIF
        ENDDO
        ISPTR(ISH)=NHG+1
  200   CONTINUE
      ENDDO
      ISPTR(NHGL+1)=NHG+1
C     AK: for the sake of consistency. CELLDM() (the CELL keyword)
C     AK: should _always_ be the actual cell, regardless of whether
C     AK: we specify a reference cell or not. ...and particularly
C     AK: it should not depend on whether we want to calculate
C     AK: the stress tensor or not.
C     AK: 2008/05/30.   
C      IF(TPRCP .OR. TPRES) THEN
      IF(.NOT. TCELLVECTORS) THEN
        CALL LATGEN(IBRAV,CELLDM,A1,A2,A3,OMEGA)
      ENDIF
      DO I=1,3
        HT(1,I) = A1(I)
        HT(2,I) = A2(I)
        HT(3,I) = A3(I)
      ENDDO
      CALL IHMAT(HT,HTM1,OMEGA)
C AK:      ENDIF
      CALL GVECTOR
      CALL TIHALT('     RGGEN',ISUB)
C     ==--------------------------------------------------------------==
      CALL NUMCPUS(NCPUS)
      IF(PARENT) THEN
        WRITE(6,'(/," ",10("OPENMP"),"OPEN")')
        WRITE(6,'(A,T60,I6)') " NUMBER OF CPUS PER TASK",NCPUS
        WRITE(6,'(" ",10("OPENMP"),"OPEN",/)')
      ENDIF
C     ==--------------------------------------------------------------==
      IF(PARENT) CALL PRMEM('     RGGEN')
C     ==--------------------------------------------------------------==
Cmb...write the file of the G-shell (to be used for S(q))
      IF(PARENT .AND. GSHELLOUT) THEN
        CALL FILEOPEN(71,"GSHELL",FO_DEF,FERROR)
        WRITE(71,'(1x,2f15.6,i7)') 0.0D0,0.0D0,0 ! dummy origin
        WRITE(71,'(1x,2f15.6,i7)')
     .        gl(1),DSQRT(gl(1))*TPIBA,1
        DO IGS=2,NHGL
           IF(GL(IGS).NE.GL(IGS-1)) THEN
           WRITE(71,'(1x,2f15.6,i7)')
     .           GL(IGS),DSQRT(GL(IGS))*TPIBA,IGS
           ENDIF
        ENDDO
        CALL FILECLOSE(71)
      ENDIF
      RETURN
      END
C     ==================================================================
      SUBROUTINE GVECTOR
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'gvec.inc'
      INCLUDE 'cppt.inc'
C     Variables
      INTEGER NH1,NH2,NH3,IG,IRI1,IRI2,IRI3,ISH
C     ==--------------------------------------------------------------==
      CALL RECIPS(ALAT,A1,A2,A3,B1,B2,B3)
      NH1=NR1S/2+1
      NH2=NR2S/2+1
      NH3=NR3S/2+1
      DO IG=1,NHG
        IRI1=INYH(1,IG)-NH1
        IRI2=INYH(2,IG)-NH2
        IRI3=INYH(3,IG)-NH3
        GK(1,IG)=IRI1*B1(1)+IRI2*B2(1)+IRI3*B3(1)
        GK(2,IG)=IRI1*B1(2)+IRI2*B2(2)+IRI3*B3(2)
        GK(3,IG)=IRI1*B1(3)+IRI2*B2(3)+IRI3*B3(3)
        HG(IG)=GK(1,IG)**2+GK(2,IG)**2+GK(3,IG)**2
      ENDDO
      DO ISH=1,NHGL
        IG=ISPTR(ISH)
        GL(ISH)=HG(IG)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GSORT(HG,INYH,NHG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'irat.inc'
C     Arguments
      INTEGER NHG,INYH(3,*)
      REAL*8  HG(NHG)
C     Variables
      INTEGER INDEX(NHG)
      POINTER (IP_INDEX,INDEX)
      INTEGER IG,ICURR,IT
C     ==--------------------------------------------------------------==
C     REORDER THE G S IN ORDER OF INCREASING MAGNITUDE
      CALL MEMORY(IP_INDEX,NHG/IRAT+1,'INDEX')
      CALL SORT2(HG,NHG,INDEX)
      DO 25 IG=1,NHG-1
        ICURR=IG
   30   CONTINUE
        IF(INDEX(ICURR).NE.IG) THEN
          IT=INYH(1,ICURR)
          INYH(1,ICURR)=INYH(1,INDEX(ICURR))
          INYH(1,INDEX(ICURR))=IT
          IT=INYH(2,ICURR)
          INYH(2,ICURR)=INYH(2,INDEX(ICURR))
          INYH(2,INDEX(ICURR))=IT
          IT=INYH(3,ICURR)
          INYH(3,ICURR)=INYH(3,INDEX(ICURR))
          INYH(3,INDEX(ICURR))=IT
          IT=ICURR
          ICURR=INDEX(ICURR)
          INDEX(IT)=IT
          IF(INDEX(ICURR).EQ.IG) THEN
            INDEX(ICURR)=ICURR
            GOTO 25
          ENDIF
          GOTO 30
        ENDIF
   25 CONTINUE
      CALL FREEM(IP_INDEX)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE ALIASING
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'gvec.inc'
      INCLUDE 'cppt.inc'
C     Variables
      INTEGER NH1,NH2,NH3,IF1,IF2,IF3,IG,IRI1,IRI2,IRI3,IFF
#ifdef PARALLEL
      REAL*8  XIF1,XIF2,XIF3
#endif
C     ==--------------------------------------------------------------==
      NH1=NR1S/2+1
      NH2=NR2S/2+1
      NH3=NR3S/2+1
      IF1=0
      IF2=0
      IF3=0
      DO IG=1,NHG
        IRI1=INYH(1,IG)
        IRI2=INYH(2,IG)
        IRI3=INYH(3,IG)
        IF(IRI1.LT.1) IF1=MAX(IF1,ABS(IRI1+1))
        IF(IRI2.LT.1) IF2=MAX(IF2,ABS(IRI2+1))
        IF(IRI3.LT.1) IF3=MAX(IF3,ABS(IRI3+1))
        IF(IRI1.GT.NR1S) IF1=MAX(IF1,IRI1-NR1S)
        IF(IRI2.GT.NR2S) IF2=MAX(IF2,IRI2-NR2S)
        IF(IRI3.GT.NR3S) IF3=MAX(IF3,IRI3-NR3S)
      ENDDO
#ifdef PARALLEL
      XIF1=IF1
      XIF2=IF2
      XIF3=IF3
      CALL GLOMAX(1,XIF1)
      CALL GLOMAX(1,XIF2)
      CALL GLOMAX(1,XIF3)
      IF1=XIF1
      IF2=XIF2
      IF3=XIF3
#endif
      IF(PARENT) THEN
        IF(IF1.GT.0) WRITE(6,'(A,I3,A,I3,A)')
     &       ' SET MESH PARAMETER NR1 TO ',NR1S+2*IF1,
     &       ' (CURRENT ',NR1S,')'
        IF(IF2.GT.0) WRITE(6,'(A,I3,A,I3,A)')
     &       ' SET MESH PARAMETER NR2 TO ',NR2S+2*IF2,
     &       ' (CURRENT ',NR2S,')'
        IF(IF3.GT.0) WRITE(6,'(A,I3,A,I3,A)')
     &       ' SET MESH PARAMETER NR3 TO ',NR3S+2*IF3,
     &       ' (CURRENT ',NR3S,')'
        IFF=IF1+IF2+IF3
        IF(IFF.GT.0) THEN
          WRITE(6,'(A,3I4)') ' CURRENT MESH: ',NR1S,NR2S,NR3S
          WRITE(6,'(A)')
     &         ' USE OPTION MESH IN SYSTEM SECTION'
          CALL STOPGM('ALIASING','MESH TOO SMALL')
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
