C     ==================================================================
      SUBROUTINE RHOOFR(C0,RHOE,PSI,SCR,LSCR,NSTATE)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE NORMALIZED ELECTRON DENSITY RHOE IN REAL SPACE          ==
C     ==  THE KINETIC ENERGY EKIN. IT IS DONE IN RECIPROCAL SPACE     ==
C     ==  WHERE THE ASSOCIATED OPERATORS ARE DIAGONAL.                ==
C     ==  RHOE IS OBTAINED FOURIER TRANSFORMING THE WFN TO REAL       ==
C     ==  SPACE (PSI).                                                ==
C     ==--------------------------------------------------------------==
C     == WARNING: ALL WAVEFUNCTIONS C0 HAVE TO BE ORTHOGONAL          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'rswf.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'prcp.inc'
      INCLUDE 'ropt.inc'
      INCLUDE 'symm.inc'
      INCLUDE 'dg.inc'

C     Arguments
      INTEGER    NSTATE,LSCR,IR
      COMPLEX*16 C0(NGW,NSTATE),PSI(*)
      REAL*8     RHOE(NNR1,NLSD),SCR(LSCR)
C     Variables
      DIMENSION  F(NSTATE)
      REAL*8     FNL(*),DFNL(*),RHOEG(*),PSIX(*)
      POINTER    (IP_RHOEG,RHOEG),(IP_PSIX,PSIX)
      INTEGER    ISUB,LEAD,LEADX,NNRX,IFFT,I,L,IS1,IS2,LRHOOFR,
     &           IG,ID,NSTA,IB,IBB,IWF,ISPIN1,ISPIN2,IAT,IS,IA,ITG
      REAL*8     R1,R2,TT,RSUM,RSUMV,RSUM1,RSUM1ABS
      REAL*8     COEF3,COEF4,DELTA,O3
      PARAMETER  (DELTA=1.D-6,O3=0.33333333333333333D0)
      REAL*8     RTO,RSP,RAL,RBE
      CHARACTER  TAG*30
      LOGICAL    TFCAL
#ifdef PARALLEL
      INTEGER    MSGLEN,NL2,IX1,IXP,IX
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('    RHOOFR',ISUB)
C     ==--------------------------------------------------------------==
      CALL KIN_ENERGY(C0,NSTATE,RSUM)
C     ==--------------------------------------------------------------==
C     Test on SCR dimension
      CALL GIVE_SCR_RHOOFR(LRHOOFR,TAG)
      CALL TEST_SCR('RHOOFR',TAG,LSCR,LRHOOFR)
C     ==--------------------------------------------------------------==
C     CASPUR 2/5/04
C     Initialize FFT datastructure
      IF (TDG) THEN
         IF(TLSE)CALL STOPGM(' RHOOFR',
     *               'TDG TOGETHER WITH LSE IS NOT YET IMPLEMENTED ')
         IF(NOGRP.GT.1)CALL STOPGM(' RHOOFR',
     *            'TDG TOGETHER WITH TASK GROUPS NOT YET IMPLEMENTED ')
         CALL SETFFTN(IPOOLDG)
      ELSE
         CALL SETFFTN(0)
      ENDIF
C     ==--------------------------------------------------------------==
#ifdef PARALLEL
      IF(NOGRP.EQ.1) THEN
        LEAD  = QR1S*MSRAYS
        LEADX = NNR1
        IFFT  = 1
        NNRX  = LLR1
      ELSE
        IF (TDG) CALL STOPGM('RHOOFR',
     &        'GROUP AND DOUBLE GRID NOT YET IMPLEMENTED')
        LEAD  = KR1S*NGRM
        LEADX = KRX*KR2S*KR3S
        IFFT  = 2
        NNRX  = KRX*KR2S*KR3S
      ENDIF
#else
      LEAD  = LLR1
      LEADX = NNR1
      NNRX  = LLR1
      IFFT  = 1
#endif
      CALL GET_ADDR(IP_PSIX,PSI(1))
      CALL GET_ADDR(IP_RHOEG,RHOE(1,1))
C     Initialize
      CALL AZZERO(RHOE,NLSD*LEADX)
C     Loop over the electronic states
      DO ID=1,NSTATE,2*NOGRP
        NSTA=MIN((NSTATE-ID+2)/2,NOGRP)
        TFCAL=RSACTIVE
        DO I=ID,MIN(ID+2*NSTA-1,NSTATE)
          TFCAL=TFCAL.OR.(F(I).NE.0.D0)
        ENDDO
        IF(TFCAL) THEN
#ifdef _BGL 
          CALL AZZERO(PSI,2*MAXFFTN*NOGRP)
#else
          CALL ZAZZERO(PSI,MAXFFTN*NOGRP)
#endif
!CDIR NODEP
          DO IB=1,NSTA
            I=ID+2*(IB-1)
            IBB=(IB-1)*LEAD
            IS1=I
            IS2=I+1
            IF(IS2.GT.NSTATE) THEN
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(1000))
!IBM$ INDEPENDENT
#endif
!$OMP parallel do private(IG)
!CDIR NODEP
#ifdef _vpp_
!OCL NOVREC
#endif
#ifdef CRAYX1
!dir$ concurrent
#endif
#ifdef __SR8000
*poption parallel, tlocal(IG)
#endif
              DO IG=1,JGW
                PSI(NZFS(IG)+IBB)=C0(IG,IS1)
                PSI(INZS(IG)+IBB)=DCONJG(C0(IG,IS1))
              ENDDO
            ELSE
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(1000))
!IBM$ INDEPENDENT
#endif
!$OMP parallel do private(IG)
!CDIR NODEP
#ifdef _vpp_
!OCL NOVREC
#endif
#ifdef CRAYX1
!dir$ concurrent
#endif
#ifdef __SR8000
*poption parallel, tlocal(IG)
#endif
              DO IG=1,JGW
                PSI(NZFS(IG)+IBB)=C0(IG,IS1)+UIMAG*C0(IG,IS2)
                PSI(INZS(IG)+IBB)=DCONJG(C0(IG,IS1))+
     *                       UIMAG*DCONJG(C0(IG,IS2))
              ENDDO
            ENDIF
            IF(GEQ0.AND.(IS2.GT.NSTATE)) THEN
              PSI(NZFS(1)+IBB)=C0(1,IS1)
            ELSEIF(GEQ0) THEN
              PSI(NZFS(1)+IBB)=C0(1,IS1)+UIMAG*C0(1,IS2)
            ENDIF
          ENDDO
C     ==--------------------------------------------------------------==
C     ==  Fourier transform the wave functions to real space.         ==
C     ==  In the array PSI was used also the fact that the wave       ==
C     ==  functions at Gamma are real, to form a complex array (PSI)  ==
C     ==  with the wave functions corresponding to two different      ==
C     ==  states (i and i+1) as the real and imaginary part. This     ==
C     ==  allows to call the FFT routine 1/2 of the times and save    ==
C     ==  time.                                                       ==
C     ==--------------------------------------------------------------==
          IF(IFFT.EQ.1) THEN
            CALL INVFFTN(PSI,.TRUE.)
          ELSE
            CALL SG_INVFFT(PSI)
          ENDIF
C         Store real space wavefunctions
          IF(RSACTIVE) THEN
            IF(NOGRP.EQ.1) THEN
              IF(ID.LE.MAXSTATES) THEN
                IWF=ID/2+1
                CALL DCOPY(2*LEADX,PSI(1),1,RSWF(1,IWF),1)
              ENDIF
            ELSE
              ITG=(ID/(2*NOGRP)+1)
              IF(ITG.LE.MAXSTATES) THEN
                IWF=(ID/(2*NOGRP))+1
                CALL DCOPY(2*LEADX,PSI(1),1,RSWF(1,IWF),1)
              ENDIF
            ENDIF
          ENDIF
C         Compute the charge density from the wave functions
C         in real space 
          IF(NOGRP.GT.1) THEN
            IS2=0
            DO IB=1,NSTA
              IF(NOLIST(IB).EQ.ME) THEN
                I=ID+2*(IB-1)
                IS1=I
                IS2=I+1
              ENDIF
            ENDDO
          ELSE
            IS1=ID
            IS2=ID+1
          ENDIF
          COEF3=F(IS1)/OMEGA
          IF(IS2.GT.NSTATE) THEN
            COEF4=0.0D0
          ELSE
            COEF4=F(IS2)/OMEGA
          ENDIF
          IF(IS2.EQ.0) THEN
            COEF3=0.0D0
            COEF4=0.0D0
          ENDIF
          IF(TLSD) THEN
            ISPIN1=0
            ISPIN2=0
            IF(IS1.GT.NSUP) ISPIN1=LEADX
            IF(IS2.GT.NSUP) ISPIN2=LEADX
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
!IBM$ INDEPENDENT, NEW(R1,R2)
#endif
!$OMP parallel do private(L,R1,R2)
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
            DO L=1,NNRX
              R1=DREAL(PSI(L))
              R2=DIMAG(PSI(L))
              RHOEG(L+ISPIN1)=RHOEG(L+ISPIN1)+COEF3*R1*R1
              RHOEG(L+ISPIN2)=RHOEG(L+ISPIN2)+COEF4*R2*R2
            ENDDO
          ELSEIF(TLSE) THEN
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
!IBM$ INDEPENDENT, NEW(R1,R2,TT)
#endif
!$OMP parallel do private(L,R1,R2,TT)
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
            DO L=1,NNRX
              R1=DREAL(PSI(L))
              R2=DIMAG(PSI(L))
              TT=COEF3*R1*R1+COEF4*R2*R2
              RHOEG(L)=RHOEG(L)+TT
            ENDDO
            IF(IS1.EQ.IALPHA) THEN
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
!IBM$ INDEPENDENT
#endif
!$OMP parallel do private(L) 
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
              DO L=1,NNRX
                RHOEG(L+2*LEADX)=RHOEG(L+2*LEADX)
     *                          +COEF3*DREAL(PSI(L))**2
              ENDDO
            ELSEIF(IS1.EQ.IBETA) THEN
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
!IBM$ INDEPENDENT
#endif
!$OMP parallel do private(L) 
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
              DO L=1,NNRX
                RHOEG(L+LEADX)=RHOEG(L+LEADX)
     *                        +COEF3*DREAL(PSI(L))**2
              ENDDO
            ENDIF
            IF(IS2.EQ.IALPHA) THEN
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
!IBM$ INDEPENDENT
#endif
!$OMP parallel do private(L) 
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
              DO L=1,NNRX
                RHOEG(L+2*LEADX)=RHOEG(L+2*LEADX)
     *                          +COEF4*DIMAG(PSI(L))**2
              ENDDO
            ELSEIF(IS2.EQ.IBETA) THEN
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
!IBM$ INDEPENDENT
#endif
!$OMP parallel do private(L) 
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
              DO L=1,NNRX
                RHOEG(L+LEADX)=RHOEG(L+LEADX)
     *                        +COEF4*DIMAG(PSI(L))**2
              ENDDO
            ENDIF
          ELSE
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
!IBM$ INDEPENDENT, NEW(R1,R2,TT)
#endif
!$OMP parallel do private(L,R1,R2,TT) 
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
            DO L=1,NNRX
              R1=DREAL(PSI(L))
              R2=DIMAG(PSI(L))
              TT=COEF3*R1*R1+COEF4*R2*R2
              RHOEG(L)=RHOEG(L)+TT
            ENDDO
          ENDIF
        ENDIF                   !endif TFCAL
      ENDDO                     !End loop over the electronic states
C     ==--------------------------------------------------------------==
#ifdef PARALLEL
      IF(NOGRP.GT.1) THEN
C       Summation of density within orbital split
        NL2=NLINK(NOLIST(NOGRP))
        IX1=NRXPL(MEPOS,1)-NRXPL(NL2,1)
        MSGLEN= 8 * NNRX
        IF(TLSD) THEN
          CALL MY_COMBINE(RHOEG(1),PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE(1,1),NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,1),KR1)
          ENDDO
          CALL MY_COMBINE(RHOEG(LEADX+1),PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE(1,2),NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,2),KR1)
          ENDDO
        ELSEIF(TLSE) THEN
          CALL MY_COMBINE(RHOEG(1),PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE(1,1),NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,1),KR1)
          ENDDO
          CALL MY_COMBINE(RHOEG(LEADX+1),PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE(1,2),NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,2),KR1)
          ENDDO
          CALL MY_COMBINE(RHOEG(2*LEADX+1),PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE(1,3),NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,3),KR1)
          ENDDO
        ELSE
          CALL MY_COMBINE(RHOEG,PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE,NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,1),KR1)
          ENDDO
        ENDIF
      ENDIF
#endif
C CASPUR 2/9/04
      IF (TDG) THEN
        IF (TLSE) THEN
          CALL STOPGM(' RHOOFR',
     *               'TDG TOGETHER WITH LSE IS NOT YET IMPLEMENTED ')
        ELSE IF (TLSD) THEN 
!$OMP parallel do private(IR)
          DO IR=1,LLR1
            PSI(IR)=DCMPLX(RHOE(IR,1), RHOE(IR,2))
          ENDDO
          CALL FWFFTN(PSI,.false.)
          CALL MOVEPSIH(PSI,SCR,LSCR,2)
          CALL SETFFTN(0)
          CALL INVFFT(PSI)
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
!IBM$ INDEPENDENT
#endif
!$OMP parallel do private(IR)
          DO IR=1,NNR1
            RHOE(IR,1)=DREAL(PSI(IR))
            RHOE(IR,2)=DIMAG(PSI(IR))
          ENDDO
        ELSE
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
!IBM$ INDEPENDENT
          CALL ALIGNX(16,PSI(1))
          CALL ALIGNX(16,RHOE(1,1))
#endif
!$OMP parallel do private(IR)
          DO IR=1,LLR1
            PSI(IR)=DCMPLX(RHOE(IR,1), 0.D0)
          ENDDO
          CALL FWFFTN(PSI,.false.) 
          CALL MOVEPSIH(PSI,SCR,LSCR,1)
          CALL SETFFTN(0)
          CALL INVFFT(PSI)
          CALL DCOPY(NNR1, PSI, 2, RHOE, 1)
        ENDIF 
      ENDIF
C     MOVE DENSITY ACCORDING TO MOVEMENT OF ATOMS
      IF(MODENS) CALL MOVERHO(RHOE,PSI,SCR,LSCR)
C     CONTRIBUTION OF THE VANDERBILT PP TO RHOE
      IF(TIVAN) THEN
        IF(TLSD) THEN
C         ALPHA SPIN
          CALL RHOV(NSTATE,1,NSUP,RSUMV,PSI,SCR,LSCR)
          RSUM=RSUM+OMEGA*RSUMV
!$OMP parallel do private(I)
          DO I=1,NNR1
            RHOE(I,1)=RHOE(I,1)+DREAL(PSI(I))
          ENDDO
C         BETA SPIN
          CALL RHOV(NSTATE,NSUP+1,NSTATE,RSUMV,PSI,SCR,LSCR)
          RSUM=RSUM+OMEGA*RSUMV
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
!IBM$ INDEPENDENT
          CALL ALIGNX(16,PSI(1))
          CALL ALIGNX(16,RHOE(1,2))
#endif
!$OMP parallel do private(I)
          DO I=1,NNR1
            RHOE(I,2)=RHOE(I,2)+DREAL(PSI(I))
          ENDDO
        ELSE
          CALL RHOV(NSTATE,1,NSTATE,RSUMV,PSI,SCR,LSCR)
          RSUM=RSUM+OMEGA*RSUMV
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
!IBM$ INDEPENDENT
          CALL ALIGNX(16,PSI(1))
          CALL ALIGNX(16,RHOE(1,1))
#endif
!$OMP parallel do private(I)
          DO I=1,NNR1
            RHOE(I,1)=RHOE(I,1)+DREAL(PSI(I))
          ENDDO
        ENDIF
C       Vanderbilt Charges
        IF(PARENT) THEN
          CALL AZZERO(SCR,NAT)
          CALL AUGCHG(FNL,F,SCR,NSTATE)
          IAT=0
          DO IS=1,NSP
            VDBCHG(IS)=0.D0
            DO IA=1,NA(IS)
              IAT=IAT+1
              VDBCHG(IS)=VDBCHG(IS)+SCR(IAT)
            ENDDO
            VDBCHG(IS)=VDBCHG(IS)/DBLE(NA(IS))
          ENDDO
        ENDIF
      ENDIF
C     ALPHA+BETA DENSITY IN RHOE(*,1), BETA DENSITY IN RHOE(*,2)
      CSUMS=0.D0
      CSUMSABS=0.D0
      IF(TLSD) THEN
        RSUM1=0.D0
        RSUM1ABS=0.D0
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
#endif
C$OMP parallel do private(I) shared(NNR1,RHOE)
C$OMP+ reduction(+:RSUM1,RSUM1ABS)
        DO I=1,NNR1
          RSUM1 = RSUM1 + (RHOE(I,1) - RHOE(I,2))
          RSUM1ABS = RSUM1ABS + DABS(RHOE(I,1) - RHOE(I,2))
          RHOE(I,1) = RHOE(I,1) + RHOE(I,2)
        ENDDO
        CSUMS=RSUM1*OMEGA/DBLE(LR1S*LR2S*LR3S)
        CSUMSABS=RSUM1ABS*OMEGA/DBLE(LR1S*LR2S*LR3S)
C     ALPHA+BETA DENSITY IN RHOE(*,1), BETA DENSITY IN RHOE(*,2) ; M STATE
C     ALPHA+BETA DENSITY IN RHOE(*,3), BETA DENSITY IN RHOE(*,4) ; T STATE
      ELSEIF(TLSE) THEN
        IF(TCAS22) THEN
C         Calculate "ground state" density and D-EX DENSITY
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT, NEW(RTO,RBE,RAL)
#endif
!$OMP parallel do private(I,RTO,RBE,RAL)
          DO I=1,NNR1
            RTO=RHOE(I,1)
            RBE=RHOE(I,2)
            RAL=RHOE(I,3)
            RHOE(I,6)=RTO-RBE+RAL
            RHOE(I,7)=RTO+RBE-RAL
          ENDDO
        ENDIF
        IF(TLSETS) THEN
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT, NEW(RTO,RBE,RAL)
#endif
!$OMP parallel do private(I,RTO,RBE,RAL,RSP)
          DO I=1,NNR1
            RTO=RHOE(I,1)
            RBE=RHOE(I,2)
            RAL=RHOE(I,3)
            RSP=0.5D0*(RTO-RAL-RBE)            
            RHOE(I,2)=RSP+RBE+O3*RAL
            RHOE(I,3)=RTO
            RHOE(I,4)=RSP+RBE+2.D0*O3*RAL
          ENDDO
        ELSE
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT, NEW(RTO,RBE,RAL)
#endif
!$OMP parallel do private(I,RTO,RBE,RAL,RSP)
          DO I=1,NNR1
            RTO=RHOE(I,1)
            RBE=RHOE(I,2)
            RAL=RHOE(I,3)
            RSP=0.5D0*(RTO-RAL-RBE)
            RHOE(I,2)=RSP+RBE
            RHOE(I,3)=RTO
            RHOE(I,4)=RSP+RAL+RBE
          ENDDO
        ENDIF
        IF(TROSS.OR.TCAS22.OR.TPENAL) THEN
C         WE ALSO NEED THE A*B DENSITY FOR THE EXCHANGE CONTRIBUTION
C         OR THE OFF DIAGONAL ELEMENTS IN THE CAS22 METHOD
          CALL AZZERO(RHOE(1,5),NNR1)
          CALL RHOABOFR(1,C0(1,IALPHA),C0(1,IBETA),RHOE(1,5),PSI)
        ENDIF
      ENDIF
C     HERE TO CHECK THE INTEGRAL OF THE CHARGE DENSITY
C     RSUM1=DASUM(NNR1,RHOE(1,1),1)
C     --> with VDB PP RHOE might be negative in some points
      RSUM1=0.D0
#if defined(__SR8000)
*poption parallel, tlocal(I), psum(RSUM1)
#elif defined(_BGL) 
!IBM* ASSERT (ITERCNT(10000))
!IBM* INDEPENDENT, REDUCTION(RSUM1)
!$OMP parallel do private(I) shared(NNR1,RHOE)
!$OMP+ reduction(+:RSUM1)
#else
!$OMP parallel do private(I) shared(NNR1,RHOE)
!$OMP+ reduction(+:RSUM1)
#endif
      DO I=1,NNR1
        RSUM1=RSUM1+RHOE(I,1)
      ENDDO
      RSUM1=RSUM1*OMEGA/DBLE(NR1S*NR2S*NR3S)
      CSUMG=RSUM
      CSUMR=RSUM1
      CALL GLOSUM(4,CSUMG)
      IF(PARENT.AND.ABS(CSUMR-CSUMG).GT.DELTA) THEN
        WRITE(6,'(A,T46,F20.12)') ' IN FOURIER SPACE:', CSUMG
        WRITE(6,'(A,T46,F20.12)') ' IN REAL SPACE:', CSUMR
        IF(INDPG.NE.0.AND.CDUAL.LT.4.D0)
     &       WRITE(6,*) 'YOUR DUAL NUMBER ',CDUAL,
     &       ' COULD BE TOO SMALL WITH DENSITY SYMMETRISATION'
        CALL STOPGM('RHOOFR','TOTAL DENSITY SUMS ARE NOT EQUAL')
      ENDIF
C     TAU FUNCTION
      IF(TTAU) CALL TAUOFR(C0,PSI,NSTATE)
C
      CALL TIHALT('    RHOOFR',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_RHOOFR(LRHOOFR,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !PARENT NNR1
      INCLUDE 'pslo.inc'        !TIVAN
      INCLUDE 'ions.inc'        !NAT
      INCLUDE 'ropt.inc'        !MODENS
      INCLUDE 'spin.inc'        !NLSD
      INCLUDE 'dg.inc'          !TDG
C     Arguments
      INTEGER   LRHOOFR
      CHARACTER TAG*30
C     Variables
      INTEGER LRHOV,LSYMRHO,LMOVERHO,LDG
C     ==--------------------------------------------------------------==
      LDG=0
      IF(TDG) LDG=2*NHG*NLSD+100
      LMOVERHO=0
      IF(TIVAN) THEN !RHOOFR and RHOOFR_C
        CALL GIVE_SCR_RHOV(LRHOV,TAG)
        LRHOOFR=MAX(NAT,LRHOV)  !AUGCHG
      ELSE
        LRHOOFR=0
      ENDIF
      CALL GIVE_SCR_SYMRHO(LSYMRHO,TAG)
      IF(MODENS) CALL GIVE_SCR_MOVERHO(LMOVERHO,TAG)
      LRHOOFR=MAX(LRHOOFR,LSYMRHO,LMOVERHO)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MOVEPSIH(A,SCR,LSCR,ITASK)
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
      INTEGER ITASK,LSCR
      COMPLEX*16 A(*),AQ1,AQ2, c1, c2
      REAL*8 SCR(*)
      INTEGER IG
      LOGICAL DEBUG
      COMPLEX*16 B1(*),B2(*)
      POINTER (IP_B1,B1),(IP_B2,B2)
C     ==--------------------------------------------------------------==
      DEBUG=.FALSE.
      CALL SCRSET(SCR,LSCR,DEBUG)
C
      IF(ITASK.EQ.1) THEN
        CALL SCRPTR(SCR,LSCR,IP_B1,2*NHG,'B1',DEBUG)
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(1000))
!IBM* INDEPENDENT
#endif
!$OMP parallel do private(IG)
        DO IG=1,JHG
          B1(IG)=A(NZFF(IG))
        ENDDO
#ifdef _BGL 
        CALL AZZERO(A,2*MAXFFT)
#else
        CALL ZAZZERO(A,MAXFFT)
#endif
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(1000))
!IBM* INDEPENDENT
#endif
        DO IG=1,JHG
          A(NZH(IG))=B1(IG)
          A(INDZ(IG))=DCONJG(B1(IG))
        ENDDO
        IF(GEQ0) A(NZH(1))=B1(1)
      ELSEIF(ITASK.EQ.2) THEN
        CALL SCRPTR(SCR,LSCR,IP_B1,2*NHG,'B1',DEBUG)
        CALL SCRPTR(SCR,LSCR,IP_B2,2*NHG,'B2',DEBUG)
        C1=DCMPLX(0.5D0, 0.d0)
        C2=DCMPLX(0.0D0,-0.5d0)
C THE TWO COMPONENT OF DENSITY (REAL RUNCTION IN R SPACE)
C TRANSFORMED AT ONCE
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(1000))
!IBM* INDEPENDENT,NEW(AQ1,AQ2)
#endif
!$OMP parallel do private(IG,AQ1,AQ2) shared(C1,C2)
        DO IG=1,JHG
cmb           AQ1=C1*(A(NZFF(IG))+DCONJG(A(INZF(IG))))
cmb           AQ2=C2*(A(NZFF(IG))-DCONJG(A(INZF(IG))))
           AQ1=A(NZFF(IG))         ! access only once
           AQ2=DCONJG(A(INZF(IG))) ! memory locations
           B1(IG)=C1*(AQ1+AQ2)
           B2(IG)=C2*(AQ1-AQ2)
        ENDDO
        IF(GEQ0) THEN
          B1(1) = DCMPLX(DREAL(A(NZFF(1))),0.d0)
          B2(1) = DCMPLX(DIMAG(A(NZFF(1))),0.d0)
        ENDIF
#ifdef _BGL 
        CALL AZZERO(A,2*MAXFFT)
#else
        CALL ZAZZERO(A,MAXFFT)
#endif
#ifdef _BGL 
!IBM* ASSERT (ITERCNT(1000))
!IBM* INDEPENDENT
#endif
        DO IG=1,JHG
          A(NZH(IG))=B1(IG)+UIMAG*B2(IG)
          A(INDZ(IG))=DCONJG(B1(IG))+UIMAG*DCONJG(B2(IG))
        ENDDO
        IF(GEQ0) A(NZH(1))=B1(1)+UIMAG*B2(1)
      ELSE
        CALL STOPGM('MOVEPSIH',' ITASK=? ')
      ENDIF
C
      CALL SCRREL(SCR,LSCR,DEBUG)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
