C     ==================================================================
      SUBROUTINE RHOOFR_KDP(C0,CKDP,RHOE,PSI,SCR,LSCR,NSTATE)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE NORMALIZED ELECTRON DENSITY RHOE IN REAL SPACE          ==
C     ==  FOR THE K.DOT.P (OPTIMAL BASIS SET) METHOD                  ==
C     ==  CO IS INPUT ORBITALS                                        ==
C     ==  CKDP IS INPUT TRANSFORMED ORBITALS                          ==
C     ==--------------------------------------------------------------==
C     == WARNING: ALL WAVEFUNCTIONS C0 HAVE TO BE ORTHOGONAL          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'prcp.inc'
      INCLUDE 'ropt.inc'
      INCLUDE 'symm.inc'
C     Arguments
      INTEGER    NSTATE,LSCR
      COMPLEX*16 C0(NGW,NSTATE),PSI(MAXFFT*NOGRP)
      COMPLEX*16 CKDP(NGW,NSTATE)
      REAL*8     RHOE(NNR1,NLSD),SCR(LSCR)
C     Variables
      DIMENSION  F(NSTATE)
      REAL*8     FNL(*),DFNL(*)
      REAL*8     RHOEG(*),PSIX(*)
      POINTER    (IP_RHOEG,RHOEG),(IP_PSIX,PSIX)
      INTEGER    ISUB,LEAD,LEADX,NNRX,IFFT,I,L,IS1,LRHOOFR,
     &           IG,ID,NSTA,IB,IBB,IWF,ISPIN1,IAT,IS,IA
      REAL*8     DOTP,COEF3,R1,R2,TT,RSUM,RSUMV,RSUM1,RSUM1ABS
      EXTERNAL   DOTP
      REAL*8     DELTA,DELTAKIN
      PARAMETER  (DELTA=1.D-6,DELTAKIN=1.D-10)
      COMPLEX*16 CZR1,CZR2
      REAL*8     RTO,RSP,RAL,RBE
      CHARACTER  TAG*30
      LOGICAL    TFCAL
#ifdef PARALLEL
      INTEGER    MSGLEN,NL2,IX1,IXP,IX
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('RHOOFR_KDP',ISUB)
C     ==--------------------------------------------------------------==
C     Test on SCR dimension
      CALL GIVE_SCR_RHOOFR_KDP(LRHOOFR,TAG)
      CALL TEST_SCR('RHOOFR_KDP',TAG,LSCR,LRHOOFR)
C     ==--------------------------------------------------------------==
#ifdef PARALLEL
      IF(NOGRP.EQ.1) THEN
        LEAD  = KR1S*NGRAYS
        LEADX = NNR1
        IFFT=1
        NNRX=NNR1
      ELSE
        LEAD  = KR1S*NGRM
        LEADX = KRX*KR2S*KR3S
        IFFT=2
        NNRX=KRX*KR2S*KR3S
      ENDIF
#else
      LEAD  = NNR1
      LEADX = NNR1
      NNRX=NNR1
      IFFT=1
#endif
      CALL GET_ADDR(IP_PSIX,PSI(1))
      CALL GET_ADDR(IP_RHOEG,RHOE(1,1))
C     Initialize
      CALL AZZERO(RHOEG,NLSD*LEADX)
c
C     Accumulate the charge and kinetic energy
      RSUM=0.D0
      DO I=1,NSTATE
        RSUM=RSUM+DOTP(NGW,C0(1,I),CKDP(1,I))
      ENDDO

C     Loop over the electronic states
      DO ID=1,NSTATE,NOGRP
        NSTA=MIN(NSTATE-ID+1,NOGRP)
        TFCAL=.TRUE.
        IF(TFCAL) THEN
          CALL ZAZZERO(PSI,MAXFFT*NOGRP)
#ifdef __SR8000
*poption parallel
#endif
          DO IB=1,NSTA
            I=ID+(IB-1)
            IBB=(IB-1)*LEAD
            IS1=I
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(1000))
#endif
#ifdef _vpp_
!OCL NOVREC(PSI)
#endif
#ifdef CRAYX1
!dir$ concurrent
#endif
!$OMP parallel do private(IG,CZR1,CZR2)
!CDIR NODEP
            DO IG=1,NGW
              CZR1=C0(IG,IS1)
              CZR2=CKDP(IG,IS1)
              PSI(NZHS(IG)+IBB)=CZR1+UIMAG*CZR2
              PSI(INDZS(IG)+IBB)=DCONJG(CZR1)+UIMAG*DCONJG(CZR2)
            ENDDO
            IF(GEQ0) THEN
              PSI(NZHS(1)+IBB)=C0(1,IS1)+UIMAG*CKDP(1,IS1)
            ENDIF
          ENDDO
C     ==--------------------------------------------------------------==
C     ==  Fourier transform the wave functions to real space.         ==
C     ==  In the array PSI was used also the fact that the wave       ==
C     ==  functions at Gamma are real, to form a complex array (PSI)  ==
C     ==  with the wave functions corresponding to two different      ==
C     ==  states (i and i+1) as the real and imaginary part. This     ==
C     ==  allows to call the FFT routine 1/2 of the times and save    ==
C     ==  time.                                                       ==
C     ==--------------------------------------------------------------==
          IF(IFFT.EQ.1) THEN
            CALL S_INVFFT(PSI)
          ELSE
            CALL SG_INVFFT(PSI)
          ENDIF
C         Compute the charge density from the wave functions
C         in real space 
          IF(NOGRP.GT.1) THEN
            DO IB=1,NSTA
              IF(NOLIST(IB).EQ.ME) THEN
                I=ID+(IB-1)
                IS1=I
              ENDIF
            ENDDO
          ELSE
            IS1=ID
          ENDIF
          COEF3=1.D0/OMEGA
          IF(TLSD) THEN
            IF(IS1.GT.NSUP) THEN
              ISPIN1=LEADX
            ELSE
              ISPIN1=0
            ENDIF
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
#endif
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
!$OMP parallel do private(L,R1,R2,TT)
            DO L=ISPIN1+1,ISPIN1+NNRX
              R1=DREAL(PSI(L))
              R2=DIMAG(PSI(L))
              TT=COEF3*R1*R2
              RHOEG(L)=RHOEG(L)+TT
            ENDDO
          ELSEIF(TLSE) THEN
            CALL STOPGM('RHOOFR_KDP',' NOT IMPLEMENTED TLSE')
          ELSE
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
#endif
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
!$OMP parallel do private(L,R1,R2,TT)
            DO L=1,NNRX
              R1=DREAL(PSI(L))
              R2=DIMAG(PSI(L))
              TT=COEF3*R1*R2
              RHOEG(L)=RHOEG(L)+TT
            ENDDO
          ENDIF
        ENDIF                   !endif TFCAL
      ENDDO                     !End loop over the electronic states
C     ==--------------------------------------------------------------==
#ifdef PARALLEL
      IF(NOGRP.GT.1) THEN
C       Summation of density within orbital split
        NL2=NLINK(NOLIST(NOGRP))
        IX1=NRXPL(MEPOS,1)-NRXPL(NL2,1)
        MSGLEN= 8 * NNRX
        IF(TLSD) THEN
          CALL MY_COMBINE(RHOEG(1),PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE(1,1),NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,1),KR1)
          ENDDO
          CALL MY_COMBINE(RHOEG(LEADX+1),PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE(1,2),NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,2),KR1)
          ENDDO
        ELSEIF(TLSE) THEN
          CALL STOPGM('RHOOFR_KDP',' NOT IMPLEMENTED TLSE')
        ELSE
          CALL MY_COMBINE(RHOEG,PSIX,MSGLEN,1,MEOGRP)
          CALL AZZERO(RHOE,NNR1)
          DO IXP=1,NR1
            IX=IX1+IXP
            CALL DCOPY(KR2S*KR3S,PSIX(IX),KRX,RHOE(IXP,1),KR1)
          ENDDO
        ENDIF
      ENDIF
#endif
C     MOVE DENSITY ACCORDING TO MOVEMENT OF ATOMS
      IF(MODENS) CALL MOVERHO(RHOE,PSI,SCR,LSCR)
C     CONTRIBUTION OF THE VANDERBILT PP TO RHOE
      IF(TIVAN) THEN
        IF(TLSD) THEN
C         ALPHA SPIN
          CALL RHOV(NSTATE,1,NSUP,RSUMV,PSI,SCR,LSCR)
          RSUM=RSUM+OMEGA*RSUMV
!$OMP parallel do private(I)
          DO I=1,NNR1
            RHOE(I,1)=RHOE(I,1)+DREAL(PSI(I))
          ENDDO
C         BETA SPIN
          CALL RHOV(NSTATE,NSUP+1,NSTATE,RSUMV,PSI,SCR,LSCR)
          RSUM=RSUM+OMEGA*RSUMV
!$OMP parallel do private(I)
          DO I=1,NNR1
            RHOE(I,2)=RHOE(I,2)+DREAL(PSI(I))
          ENDDO
        ELSE
          CALL RHOV(NSTATE,1,NSTATE,RSUMV,PSI,SCR,LSCR)
          RSUM=RSUM+OMEGA*RSUMV
!$OMP parallel do private(I)
          DO I=1,NNR1
            RHOE(I,1)=RHOE(I,1)+DREAL(PSI(I))
          ENDDO
        ENDIF
C       Vanderbilt Charges
        IF(PARENT) THEN
          CALL AZZERO(SCR,NAT)
          CALL AUGCHG(FNL,F,SCR,NAT)
          IAT=0
          DO IS=1,NSP
            VDBCHG(IS)=0.0D0
            DO IA=1,NA(IS)
              IAT=IAT+1
              VDBCHG(IS)=VDBCHG(IS)+SCR(IAT)
            ENDDO
            VDBCHG(IS)=VDBCHG(IS)/NA(IS)
          ENDDO
        ENDIF
      ENDIF
C     ALPHA+BETA DENSITY IN RHOE(*,1), BETA DENSITY IN RHOE(*,2)
      CSUMS=0.D0
      CSUMSABS=0.D0
      IF(TLSD) THEN
        RSUM1=0.D0
        RSUM1ABS=0.D0
        DO I=1,NNR1
          RSUM1 = RSUM1 + RHOE(I,1) - RHOE(I,2)
          RSUM1ABS = RSUM1ABS + ABS(RHOE(I,1) - RHOE(I,2))
          RHOE(I,1) = RHOE(I,1) + RHOE(I,2)
        ENDDO
        CSUMS=RSUM1*OMEGA/DBLE(NR1S*NR2S*NR3S)
        CSUMSABS=RSUM1ABS*OMEGA/DBLE(NR1S*NR2S*NR3S)
      ELSEIF(TLSE) THEN
        CALL STOPGM('RHOOFR_KDP',' NOT IMPLEMENTED TLSE')
      ENDIF
C     SYMMETRIZE DENSITY IF POINT GROUP SPECIFIED
C     (NEED FOR SPECIAL K-POINTS).
      IF(TSYMRHO) THEN
        IF(TLSD) THEN
          CALL SYMRHO(RHOE(1,1),PSI,SCR,LSCR)
          CALL SYMRHO(RHOE(1,2),PSI,SCR,LSCR)
        ELSE
          CALL SYMRHO(RHOE(1,1),PSI,SCR,LSCR)
        ENDIF
      ENDIF
C     HERE TO CHECK THE INTEGRAL OF THE CHARGE DENSITY
C     RSUM1=DASUM(NNR1,RHOE(1,1),1)
C     --> with VDB PP RHOE might be negative in some points
      RSUM1=0.D0
!$OMP parallel do private(I) reduction(+:RSUM1)
      DO I=1,NNR1
        RSUM1=RSUM1+RHOE(I,1)
      ENDDO
      RSUM1=RSUM1*OMEGA/DBLE(NR1S*NR2S*NR3S)
      CSUMG=RSUM
      CSUMR=RSUM1
      CALL GLOSUM(4,CSUMG)
      IF(PARENT.AND.DABS(CSUMR-CSUMG).GT.DELTA) THEN
        WRITE(6,'(A,T45,F20.12)') 'IN FOURIER SPACE:', CSUMG
        WRITE(6,'(A,T45,F20.12)') 'IN REAL SPACE:', CSUMR
        IF(INDPG.NE.0.AND.CDUAL.LT.4.D0)
     &       WRITE(6,*) 'YOUR DUAL NUMBER ',CDUAL,
     &       ' COULD BE TOO SMALL WITH DENSITY SYMMETRISATION'
        CALL STOPGM('RHOOFR','TOTAL DENSITY SUMS ARE NOT EQUAL')
      ENDIF
      CALL TIHALT('RHOOFR_KDP',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_RHOOFR_KDP(LRHOOFR,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !PARENT NNR1
      INCLUDE 'pslo.inc'        !TIVAN
      INCLUDE 'ions.inc'        !NAT
      INCLUDE 'ropt.inc'        !MODENS
C     Arguments
      INTEGER   LRHOOFR
      CHARACTER TAG*30
C     Variables
      INTEGER LRHOV,LSYMRHO,LMOVERHO
C     ==--------------------------------------------------------------==
      LMOVERHO=0
      IF(TIVAN.AND.PARENT) THEN !RHOOFR and RHOOFR_C
        CALL GIVE_SCR_RHOV(LRHOV,TAG)
        LRHOOFR=MAX(NAT,LRHOV)  !AUGCHG
      ELSE
        LRHOOFR=0
      ENDIF
      CALL GIVE_SCR_SYMRHO(LSYMRHO,TAG)
      IF(MODENS) CALL GIVE_SCR_MOVERHO(LMOVERHO,TAG)
      LRHOOFR=MAX(LRHOOFR,LSYMRHO,LMOVERHO)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
