c     ==================================================================
      subroutine rhoofr_p(c0,c1,drhoe,psi,scr,lscr,nstate)
c     ==--------------------------------------------------------------==
c     ==                        COMPUTES                              ==
c     ==  the first order electron density drhoe in real space.       ==
c     ==  drhoe =  <psi_i1|r><r|psi_i0> + cc                          ==
c     ==           - (<psi_i1|psi_j0>+<psi_i0|psi_j1>) *              ==
c     ==               * <psi_j0|r><r|psi_i0>                         ==
c     ==                                                              ==
c     ==  the kinetic energy ekin. It is done in reciprocal space     ==
c     ==  where the associated operators are diagonal.                ==
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'spin.inc'
      include 'elct.inc'
      include 'cnst.inc'
      include 'geq0.inc'
      include 'pslo.inc'
      include 'ener.inc'
      include 'nlps.inc'
      include 'cppt.inc'
      include 'sfac.inc'
      include 'fft.inc'
      include 'rswf.inc'
      include 'ions.inc'
      include 'prcp.inc'
      include 'response_p.inc'
      include 'ropt.inc'
c     arguments
      integer    nstate,lscr
      complex*16 c0(ngw,nstate),c1(ngw,nstate),psi(nnr1)
c      complex*16 psi1(nnr1)
      real*8     drhoe(nnr1,nlsd),scr(lscr)
c     variables
      dimension  F(nstate)
      real*8     FNL(*),DFNL(*)
      complex*16 t1,t2
      real*8     rhoeg(*),psix(*)
      pointer    (ip_rhoeg,rhoeg),(ip_psix,psix)
      integer    isub,lead,leadx,nnrx,ifft,i,l,is1,is2,lrhoofr,
     &           ig,id,nsta,ib,ibb,iwf,ispin1,ispin2,iat,is,ia
      real*8     dotp,xkin,sk1,xskin,arg,g2,coef3,coef4,
     &           r1,r2,tt,rsum,rsumv,rsum1,rsum1abs
      external   dotp
      character  tag*30
      logical    tfcal


#ifdef PARALLEL
      INTEGER    MSGLEN,NL2,IX1,IXP,IX
#endif
c     ==--------------------------------------------------------------==
c     When calculating the magnetic response, the perturbation operator
c     and thus the first order wavefunction are PURELY IMAGINARY. 
c     Therefore, the first order density is zero by construction.
      if (TNMR .OR. TEPR) return
      if (cutoff_restr) call restrain_ngw_zero(c1,ngw_zero,ngw,nstate)
c     ==--------------------------------------------------------------==
      call tiset('    rhoofr_p',isub)
c     ==--------------------------------------------------------------==
c     test on scr dimension
      call give_scr_rhoofr_p(lrhoofr,tag)
      call test_scr('rhoofr_p',tag,lscr,lrhoofr)
c     ==--------------------------------------------------------------==
#ifdef PARALLEL
      if(nogrp.eq.1) then
        lead  = kr1s*ngrays
        leadx = nnr1
        ifft=1
        nnrx=nnr1
      else
        lead  = kr1s*ngrm
        leadx = krx*kr2s*kr3s
        ifft=2
        nnrx=krx*kr2s*kr3s
      endif
#else
      lead  = nnr1
      leadx = nnr1
      nnrx=nnr1
      ifft=1
#endif
      CALL GET_ADDR(IP_psix,psi(1))
      CALL GET_ADDR(IP_rhoeg,drhoe(1,1))
      call azzero(rhoeg,nlsd*leadx)

c     accumulate the charge and kinetic energy
      rsum=0.d0
      do i=1,nstate
         if(f(i).ne.0.d0) then
            rsum=rsum+2.d0*f(i)*dotp(ngw,c0(1,i),c1(1,i))
         endif
      enddo

c     loop over the electronic states
      do id=1,nstate,nogrp
         nsta=min((nstate-id+2)/2,nogrp)
         tfcal=.false.
         do i=id,min(id+2*nsta-1,nstate)
            tfcal=tfcal.or.(f(i).ne.0.d0)
         enddo
         if(tfcal) then
            call zazzero(psi,maxfft*nogrp)
            do ib=1,nsta
               i=id+2*(ib-1)
               ibb=(ib-1)*lead
               is1=i
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(1000))
!IBM$ INDEPENDENT
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
#ifdef CRAYX1
!dir$ concurrent
#endif
!CDIR NODEP
!$OMP parallel do private(IG)
               do ig=1,ngw
                  psi(nzhs(ig)+ibb)=c0(ig,is1)+uimag*c1(ig,is1)
                  psi(indzs(ig)+ibb)=dconjg(c0(ig,is1))+
     *                 uimag*dconjg(c1(ig,is1))
               enddo
               if(geq0) then
                  psi(nzhs(1)+ibb)=c0(1,is1)+uimag*c1(1,is1)
               endif
            end do
C     ==--------------------------------------------------------------==
C     ==  Fourier transform the wave functions to real space.         ==
C     ==  ASSUMING THAT phi_0 AND phi_1 ARE REAL!!
C     ==--------------------------------------------------------------==
            if(ifft.eq.1) then
               call s_invfft(psi)
            else
               call sg_invfft(psi)
            endif
c     compute the charge density from the wave functions
c     in real space 
            if(nogrp.gt.1) then
               do ib=1,nsta
                  if(nolist(ib).eq.me) then
                     i=id+2*(ib-1)
                     is1=i
                  endif
               enddo
            else
               is1=id
            endif
            coef3=f(is1)/omega
            if(tlsd) then
               ispin1=0
               if(is1.gt.nsup) ispin1=leadx
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
!$OMP parallel do private(l,r1,r2)
               do  l=1,nnrx
                  r1=dreal(psi(l))
                  r2=dimag(psi(l))
                  rhoeg(l+ispin1)=rhoeg(l+ispin1)+coef3*r1*r2*2.d0
               enddo
            else
#ifdef _BGL 
!IBM$ ASSERT (ITERCNT(10000))
#endif
#ifdef _vpp_
!OCL NOVREC
#endif
!$OMP parallel do private(l,r1,r2,tt)
               do l=1,nnrx
                  r1=dreal(psi(l))
                  r2=dimag(psi(l))
                  tt=coef3*r1*r2*2.d0
                  rhoeg(l)=rhoeg(l)+tt
               enddo
            end if
         end if                 !tfcal
      enddo                     !end loop over the electronic states
c     ==--------------------------------------------------------------==
#ifdef PARALLEL
      if(nogrp.gt.1) then
c       summation of density within orbital split
        nl2=nlink(nolist(nogrp))
        ix1=nrxpl(mepos,1)-nrxpl(nl2,1)
        msglen= 8 * nnrx
        if(tlsd) then
          call my_combine(rhoeg(1),psix,msglen,1,meogrp)
          call azzero(drhoe(1,1),nnr1)
          do ixp=1,nr1
            ix=ix1+ixp
            call DCOPY(kr2s*kr3s,psix(ix),krx,drhoe(ixp,1),kr1)
          enddo
          call my_combine(rhoeg(leadx+1),psix,msglen,1,meogrp)
          call azzero(drhoe(1,2),nnr1)
          do ixp=1,nr1
            ix=ix1+ixp
            call DCOPY(kr2s*kr3s,psix(ix),krx,drhoe(ixp,2),kr1)
          enddo
        else
          call my_combine(rhoeg,psix,msglen,1,meogrp)
          call azzero(drhoe,nnr1)
          do ixp=1,nr1
            ix=ix1+ixp
            call DCOPY(kr2s*kr3s,psix(ix),krx,drhoe(ixp,1),kr1)
          enddo
        endif
      endif
#endif

c     move density according to movement of atoms
      if (modens)  then
         CALL stopgm('rhoofr_p','MOVE DENSITY not supported.')
         call moverho(drhoe,psi,scr,lscr)
      endif
c     contribution of the vanderbilt pp to rhoe
      if(tivan) then
         CALL stopgm('rhoofr_p','raman vanderbilt not implemented')
      endif
c     symmetrize density if point group specified 
c     (need for special k-points.
      if(tlsd) then
         CALL stopgm('rhoofr_p','raman: LSD not implemented')
      else
        call symrho(drhoe(1,1),psi,scr,lscr)
      endif
c     alpha+beta density in rhoe(*,1), beta density in rhoe(*,2)
      csums=0.d0
      csumsabs=0.d0
      if(tlsd) then
        rsum1=0.d0
        rsum1abs=0.d0
        do i=1,nnr1
          rsum1 = rsum1 + drhoe(i,1) - drhoe(i,2)
          rsum1abs = rsum1abs + DABS(drhoe(i,1)-drhoe(i,2))
          drhoe(i,1) = drhoe(i,1) + drhoe(i,2)
        enddo
        csums=rsum1*omega/DBLE(nr1s*nr2s*nr3s)
        csumsabs=rsum1abs*omega/DBLE(nr1s*nr2s*nr3s)
      endif
c     here to check the integral of the charge density
      rsum1=0.d0
!$OMP parallel do private(i) reduction(+:rsum1)
      do i=1,nnr1
         rsum1=rsum1+drhoe(i,1)
      end do
      rsum1=rsum1*omega/DBLE(nr1s*nr2s*nr3s)
      csumg=rsum
      csumr=rsum1
#ifdef PARALLEL
      call glosum(4,csumg)
#endif
      if(parent.and.abs(csumr-csumg).gt.1.d-6) then
        write(6,'(a,t45,f20.12)') 'in fourier space:', csumg
        write(6,'(a,t45,f20.12)') 'in real space:', csumr
        call stopgm('rhoofr_p','total density sums are not equal')
      endif
      if(parent.and.abs(csumr).gt.1.d-6) then
        write(6,'(a,t45,f20.12)') 'in real space:', csumr
c        call stopgm('rhoofr_p','density in real space not zero')
      endif
      if(parent.and.abs(csumg).gt.1.d-6) then
        write(6,'(a,t45,f20.12)') 'in fourier space:', csumg
c        call stopgm('rhoofr_p','density in g space not zero')
      endif

      call tihalt('    rhoofr_p',isub)
c     ==--------------------------------------------------------------==
      return
      end
c     ==================================================================
      subroutine give_scr_rhoofr_p(lrhoofr,tag)
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'        !parent nnr1
      include 'pslo.inc'        !tivan
      include 'ions.inc'        !nat
      include 'ropt.inc'        !modens
c     arguments
      integer   lrhoofr
      character tag*30
c     variables
      integer lrhov,lsymrho_f
c     ==--------------------------------------------------------------==
      if(tivan.and.parent) then !rhoofr and rhoofr_c
        call give_scr_rhov(lrhov,tag)
        lrhoofr=max(nat,lrhov)  !augchg
      else
        lrhoofr=0
      endif
      call give_scr_symrho(lsymrho_f,tag)
      lrhoofr=max(lrhoofr,lsymrho_f)
      if(modens) lrhoofr = max(lrhoofr,nnr1*2) !moverho
c     ==--------------------------------------------------------------==
      return
      end
c     ==================================================================
