C     ==================================================================
      SUBROUTINE RHOV(NSTATE,IS1,IS2,RSUMV,PSI,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     ==         THIS ROUTINE CALCULATES THE VANDERBILT DENSITY       ==
C     ==                                                              ==
C     == N_V(G) = SUM_I,IJ RHO_I,IJ Q_I,JI(G) E^-IG.R_I               ==
C     == RHO_I,IJ = SUM_N < BETA_I,I | PSI_N >< PSI_N | BETA_I,J >    ==
C     ==                                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'geq0.inc'
C     Arguments
      INTEGER    NSTATE,IS1,IS2,LSCR
      COMPLEX*16 PSI(*)
      REAL*8     RSUMV,SCR(*)
C     Variables
      DIMENSION  F(*),EIGRB(NHG,*)
      REAL*8     DDIA(*)
      COMPLEX*16 DELTAR(*),QG(*),CTMP(*)
      POINTER    (IP_DELTAR,DELTAR),(IP_QG,QG),(IP_CTMP,CTMP),
     &           (IP_DDIA,DDIA)
      REAL*8     FNL(NAT,NHXS,*),DFNL(*)
      REAL*8     FAC,SUM
      INTEGER    ISUB,IL_DELTAR,IL_QG,IL_DDIA,IL_CTMP,
     &           LENGTH,ISA0,IS,IV,I,IA,ISA,IG,JV
C     ==--------------------------------------------------------------==
      CALL TISET('      RHOV',ISUB)
      IF(TFDIST) CALL STOPGM('RHOV','TFDIST NOT IMPLEMENTED')
      IF(IMAGP.EQ.2)CALL STOPGM('RHOV','K-POINT NOT IMPLEMENTED')
C     IF(NOGRP.GT.1) CALL STOPGM('RHOV','NO GROUPS ALLOWED')
C     SCR partition
      IL_DELTAR = 2*NHG
      IL_QG     = 2*NHG
      IL_DDIA   = NAX
      IL_CTMP   = 2*NHG
      LENGTH    = IL_DELTAR+IL_QG+IL_DDIA+IL_CTMP
      CALL TEST_SCR('RHOV','6*NHG+NAX',LSCR,LENGTH)
      CALL GET_ADDR(IP_DELTAR,SCR(1))
      CALL GET_ADDR(IP_QG,SCR(1+IL_DELTAR))
      CALL GET_ADDR(IP_DDIA,SCR(1+IL_DELTAR+IL_QG))
      CALL GET_ADDR(IP_CTMP,SCR(1+IL_DELTAR+IL_QG+IL_DDIA))
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(DELTAR,NHG)
      ISA0=0
      DO IS=1,NSP
        IF(TVAN(IS)) THEN 
          DO IV=1,NGH(IS)
            DO JV=IV,NGH(IS)
              CALL QVAN2(IV,JV,IS,QG)
              IF(BIGMEM) THEN
                CALL AZZERO(DDIA,NA(IS))
                DO I=IS1,IS2
                  DO IA=1,NA(IS)
                    ISA=ISA0+IA
                    DDIA(IA)=DDIA(IA)+F(I)*FNL(ISA,IV,I)*FNL(ISA,JV,I)
                  ENDDO
                ENDDO
                FAC=1.0D0
                IF(IV.NE.JV) FAC=2.0D0
                CALL DGEMV('N',2*NHG,NA(IS),FAC,EIGRB(1,ISA0+1),2*NHG,
     *                     DDIA,1,0.0D0,CTMP,1)
!$OMP parallel do private(IG)
!CDIR NODEP
                DO IG=1,NHG
                  DELTAR(IG)=DELTAR(IG)+QG(IG)*CTMP(IG)
                ENDDO
              ELSE
                DO IA=1,NA(IS)
                  ISA=ISA0+IA
C$OMP parallel do private(IG)
#ifdef __SR8000
*poption parallel
#endif
                  DO IG=1,NHG
                    CTMP(IG)=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     *                        EI3(ISA,INYH(3,IG))
                  ENDDO
                  SUM=0.0D0
C$OMP parallel do private(I) reduction(+:SUM)
                  DO I=IS1,IS2
                    SUM = SUM + F(I)*FNL(ISA,IV,I)*FNL(ISA,JV,I)
                  ENDDO
                  IF(IV.NE.JV) SUM=2.D0*SUM 
!$OMP parallel do private(IG)
!CDIR NODEP
                  DO IG=1,NHG
                    DELTAR(IG)=DELTAR(IG)+QG(IG)*SUM*CTMP(IG)
                  ENDDO
                ENDDO
              ENDIF
            ENDDO
          ENDDO
        ENDIF
        ISA0=ISA0+NA(IS)
      ENDDO
      IF(GEQ0) THEN
        RSUMV=DREAL(DELTAR(1))
      ELSE
        RSUMV=0.0D0
      ENDIF
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(PSI,MAXFFT)
C$OMP parallel do private(IG) shared(PSI)
!CDIR NODEP
#ifdef __SR8000
*poption parallel
#endif
      DO IG=1,NHG
        PSI(NZH(IG))=DELTAR(IG)
        PSI(INDZ(IG))=DCONJG(DELTAR(IG))
      ENDDO
      CALL INVFFT(PSI)
      CALL TIHALT('      RHOV',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_RHOV(LRHOV,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NHG NAX
C     Arguments
      INTEGER   LRHOV
      CHARACTER TAG*30
C     Variables
C     ==--------------------------------------------------------------==
      LRHOV=6*NHG+NAX
      TAG='6*NHG+NAX'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
