C     ==================================================================
      SUBROUTINE RHOV1(RG,NSTATE,IST,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     ==         THIS ROUTINE CALCULATES THE VANDERBILT DENSITY       ==
C     ==                FOR A SINGLE STATE                            ==
C     == N_V(G) = SUM_I,IJ RHO_I,IJ Q_I,JI(G) E^-IG.R_I               ==
C     == RHO_I,IJ = SUM_N < BETA_I,I | PSI_N >< PSI_N | BETA_I,J >    ==
C     ==                                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'geq0.inc'
C     Arguments
      INTEGER    NSTATE,IST,LSCR
      COMPLEX*16 RG(NHG)
      REAL*8     SCR(LSCR)
C     Variables
      DIMENSION  EIGRB(NHG,NAT)
      COMPLEX*16 DELTAR(NHG),QG(NHG),CTMP(NHG)
      POINTER    (IP_DELTAR,DELTAR),(IP_QG,QG),(IP_CTMP,CTMP)
      REAL*8     FNL(NAT,NHXS,NSTATE),DFNL(*)
      REAL*8     SUM
      INTEGER    ISUB,IL_DELTAR,IL_QG,IL_CTMP,LENGTH,
     &           IG,ISA0,IV,JV,ISA,IA,IS
C     ==--------------------------------------------------------------==
      CALL TISET('     RHOV1',ISUB)
      IF(TFDIST) CALL STOPGM('RHOV1','TFDIST NOT IMPLEMENTED')
      IF(IMAGP.EQ.2)CALL STOPGM('RHOV1','K-POINT NOT IMPLEMENTED')
C     SCR partition
      IL_DELTAR = 2*NHG
      IL_QG     = 2*NHG
      IL_CTMP   = 2*NHG
      LENGTH    = IL_DELTAR+IL_QG+IL_CTMP
      CALL TEST_SCR('RHOV','6*NHG',LSCR,LENGTH)
      CALL GET_ADDR(IP_DELTAR,SCR(1))
      CALL GET_ADDR(IP_QG,SCR(1+IL_DELTAR))
      CALL GET_ADDR(IP_CTMP,SCR(1+IL_DELTAR+IL_QG))
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(DELTAR,NHG)
C FIXME: check out, whether this can be threaded with OpenMP. AK 2005/05/14
      ISA0=0
      DO IS=1,NSP
        IF(TVAN(IS)) THEN 
          DO IV=1,NGH(IS)
            DO JV=IV,NGH(IS)
              CALL QVAN2(IV,JV,IS,QG)
              IF(BIGMEM) THEN
                CALL ZAZZERO(CTMP,NHG)
                DO IA=1,NA(IS)
                  ISA=ISA0+IA
                  SUM = FNL(ISA,IV,IST)*FNL(ISA,JV,IST)
                  IF(IV.NE.JV) SUM=2.D0*SUM 
                  CALL DAXPY(2*NHG,SUM,EIGRB(1,ISA),1,CTMP(1),1)
                ENDDO
!$OMP parallel do private(IG)
!CDIR NODEP
                DO IG=1,NHG
                  DELTAR(IG)=DELTAR(IG)+QG(IG)*CTMP(IG)
                ENDDO
              ELSE
                DO IA=1,NA(IS)
                  ISA=ISA0+IA
!$OMP parallel do private(IG)
                  DO IG=1,NHG
                    CTMP(IG)=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     *                        EI3(ISA,INYH(3,IG))
                  ENDDO
                  SUM = FNL(ISA,IV,IST)*FNL(ISA,JV,IST)
                  IF(IV.NE.JV) SUM=2.D0*SUM 
!$OMP parallel do private(IG)
!CDIR NODEP
                  DO IG=1,NHG
                    DELTAR(IG)=DELTAR(IG)+QG(IG)*SUM*CTMP(IG)
                  ENDDO
                ENDDO
              ENDIF
            ENDDO
          ENDDO
        ENDIF
        ISA0=ISA0+NA(IS)
      ENDDO
C     ==--------------------------------------------------------------==
!$OMP parallel do private(IG)
!CDIR NODEP
      DO IG=1,NHG
        RG(IG)=RG(IG)+DELTAR(IG)
      ENDDO
      CALL TIHALT('     RHOV1',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_RHOV1(LRHOV1,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NHG
C     Arguments
      INTEGER   LRHOV1
      CHARACTER TAG*30
C     Variables
C     ==--------------------------------------------------------------==
      LRHOV1=6*NHG
      TAG='6*NHG'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
