C     ==================================================================
      SUBROUTINE RKPNT
C     ==================================================================
C     == ROUTINE TO SET UP K-POINT MESH AND                           ==
C     == THE ARRAYS HGKP AND HGKM WHISH HOLD                          ==
C     == THE QUANTITIES |k+g|^2 and |k-g|^2                           ==
C     ==================================================================
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'irat.inc'
      INCLUDE 'envj.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'sphe.inc'
C     Variables
      DIMENSION HGKP(NHG,*),HGKM(NHG,*),F(N,*),RK(3,*),WK(*),
     &          NGWKPS(*),MASKGW(*)
      CHARACTER FFORMAT*9
      INTEGER   ISUB,IK,IKPT,I1,I2,NH1,NH2,NH3,IKK,IG,I,J,K,IR
      REAL*8    G2P,G2M,T,TP,TM
      REAL*8    TEMP(3)
C     ==--------------------------------------------------------------==
      CALL TISET('     RKPNT',ISUB)
C     ==--------------------------------------------------------------==
C     == ALLOCATE MEMORY FOR THE PERMANENT ARRAYS                     ==
C     ==--------------------------------------------------------------==
C     Set NKPNT = number of kpoints in memory.
      IF(TKBLOCK) THEN
        IF(NKPNT.GE.NKPTS) THEN
          NKPNT=NKPTS
          TKBLOCK=.FALSE.
          TKALL=.FALSE.
          TKNOSWAP=.FALSE.
          TKBCALC=.FALSE.
          NBLKP=1
        ELSE
          NBLKP=NKPTS/NKPNT
          IF(MOD(NKPTS,NKPNT).NE.0) NBLKP=NBLKP+1
        ENDIF
      ELSE
        NKPNT=NKPTS
        NBLKP=1
      ENDIF
      CALL MEMORY(IP_NKPBL,NBLKP,'NKPBL')
      CALL MEMORY(IP_KPBEG,NBLKP,'KPBEG')
      IK=0
      DO IKPT=1,NBLKP
        KPBEG(IKPT)=IK
        NKPBL(IKPT)=MIN(NKPTS,NKPNT*IKPT)-NKPNT*(IKPT-1)
        IK=IK+NKPBL(IKPT)
      ENDDO
C     ==--------------------------------------------------------------==
C     ALLOCATE MEMORY FOR THE PERMANENT ARRAYS
      IF(TKBLOCK.AND.(TKALL.OR.TKBCALC)) THEN
        NKPTALL=NKPNT
      ELSE
        NKPTALL=NKPTS
      ENDIF
      CALL MEMORY(IP_HGKP,NHG*NKPTALL,'HGKP')
      CALL MEMORY(IP_HGKM,NHG*NKPTALL,'HGKM')
      CALL AZZERO(HGKP,NHG*NKPTALL)
      CALL AZZERO(HGKM,NHG*NKPTALL)
C     ==--------------------------------------------------------------==
C     IF USED BLOCK OPTION: MESSAGE
      IF(PARENT.AND.TKBLOCK) THEN
#ifdef PARALLEL
        IF(.NOT.TKBCALC.AND..NOT.TKNOSWAP) THEN
          CALL STOPGM('RKPNT',
     &         'BLOCK OPTION NOT AVAILABLE ON PARALLEL MACHINE')
        ENDIF
#endif
        WRITE(6,'(/,1x,64("*"))')
        WRITE(6,'(A,T60,I6)')
     &       ' TOTAL NUMBER OF K POINTS:',NKPTS
        WRITE(6,'(A,T60,I6)')
     &       ' CALCULATING K POINTS PER BLOCK OF:',NKPNT
        WRITE(6,'(A,T60,I6)')
     &       ' NUMBER OF BLOCKS:',NBLKP
        IF(.NOT.TKNOSWAP.AND..NOT.TKBCALC) THEN
          WRITE(6,'(A)')   ' USING SWAP FILES IN DIRECTORY: '
          CALL XSTRING(TMPDIR,I1,I2)
          WRITE(FFORMAT,'(A,I2,A)') '(T',MAX(2,65-(I2-I1)),',A)'
          WRITE(6,FFORMAT) TMPDIR(I1:I2)
        ENDIF
        IF(TKNOSWAP) THEN
          WRITE(6,'(A)')
     &           ' WAVEFUNCTIONS ARE NOT RELEVANT [NOT SAVED].'
        ELSE
          WRITE(6,'(A)')
     &           ' WAVEFUNCTIONS ARE STORED IN SWAP FILE.'
        ENDIF
        IF(TKALL) THEN
          WRITE(6,'(A,A)') ' ALL ARRAYS DEPENDING ON K POINTS ',
     &          'ARE STORED IN SWAP FILE.'
        ELSEIF(TKBCALC) THEN
          WRITE(6,'(A,A)')
     &         ' ALL ARRAYS DEPENDING ON K POINTS ',
     &         'ARE CALCULATED EACH TIME.'
        ENDIF
        WRITE(6,'(A)')
     &       ' PHASE FACTORS ARE CALCULATED EACH TIME.'
        WRITE(6,'(1x,64("*"),/)')
      ENDIF
C     ==--------------------------------------------------------------==
C     SCALED OPTION with K-points
      IF(TKSCALE) THEN
        DO IKK=1,NKPTS
          DO IR=1,3
            TEMP(IR)=RK(1,IKK)*B1(IR)+RK(2,IKK)*B2(IR)+RK(3,IKK)*B3(IR)
          ENDDO
          RK(1,IKK)=TEMP(1)
          RK(2,IKK)=TEMP(2)
          RK(3,IKK)=TEMP(3)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      DO IKPT=1,NBLKP
        CALL CALC_HGK(IKPT)
        IF(TKBLOCK) CALL WKPT_SWAP(HGKP,1,IKPT,'HGKP HGKM')
      ENDDO
C     ==--------------------------------------------------------------==
C     SET UP ARRAYS FOR SPHERICAL CUTOFF OPTION
      CALL MEMORY(IP_NGWKPS,NKPTS/IRAT+1,'NGWKPS')
      IF(TSPHERE) THEN
        DO IG=1,NGW
          IF(HG(IG).GT.GCUTWMIN) GOTO 1000
        ENDDO
 1000   CONTINUE
        MASKL=NGW-IG+1
        CALL MEMORY(IP_MASKGW,(2*NKPTALL*MASKL),'MASKGW')
        CALL SETMASK(GCUTW)
      ELSE
        DO IK=1,NKPTS
          NGWKPS(IK)=NGWKS
        ENDDO
      ENDIF
      CALL TIHALT('     RKPNT',ISUB)
      IF(PARENT) CALL PRMEM('     RKPNT')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE BZMESH(NKPOINT)
C     ==--------------------------------------------------------------==
C     == GENERATE A UNIFORM (MONKHORST-PACK) MESH IN THE KX>0 HALF OF ==
C     == THE BRILLOUIN ZONE.                                          ==
C     == REF: HJ MONKHORST AND JD PACK, PRB 13, 5188, (1976).         ==
C     == USE K290 ROUTINE TO GENERATE SPECIAL K-POINTS                ==
C     == USE INTERNAL FILES TO COMMUNICATE WITH K290 INSTEAD OF FORT.x==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'irat.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'coor.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'envj.inc'
      INCLUDE 'symm.inc'        !DELTASYM
      INCLUDE 'fileopen.inc'
C     Arguments
      INTEGER NKPOINT
C     Variables
      DIMENSION RK(3,*),WK(*)
C     ==--------------------------------------------------------------==
      INTEGER   IOUT
      CHARACTER DATX*26, FILENAME*80, FFORMAT*30
C     ==--------------------------------------------------------------==
C     LATTICE STRUCTURE
      REAL*8  STRAIN(6),XKAPA(3,NAT),RX(3,NAT),TVECT(3,NAT)
      POINTER (IP_XKAPA,XKAPA),(IP_RX,RX),(IP_TVECT,TVECT)
      INTEGER TY(NAT),ISC(NAT),F0(49,NAT)
      POINTER (IP_TY,TY),(IP_ISC,ISC),(IP_F0,F0)
C     SPECIAL POINTS
      REAL*8  WVKL(3,NKPOINT)
      POINTER (IP_WVKL,WVKL)
      INTEGER LWGHT(NKPOINT),LROT(48,NKPOINT)
      POINTER (IP_LWGHT,LWGHT),(IP_LROT,LROT)
C     Used by SPPT2 and MESH routines.
      INTEGER INCLUD(NKPOINT),LIST(*)
      POINTER (IP_INCLUD,INCLUD),(IP_LIST,LIST)
      REAL*8  RLIST(3,NKPOINT)
      POINTER (IP_RLIST,RLIST)
C     ==--------------------------------------------------------------==
      REAL*8    SUM,WEIGHT,XRANDOM(3),
     &          XRANF
      EXTERNAL  XRANF
      INTEGER   ISUB,IU1,IU2,IH1,IH2,I,IS,IA,NAT0,ISTRIZ,NTVECT,
     &          NHASH,ISUM,IKIND,L
      LOGICAL   FERROR
C     ==--------------------------------------------------------------==
      CALL TISET('    BZMESH',ISUB)
C     ==--------------------------------------------------------------==
C     == Standard output of K290                                      ==
C     ==--------------------------------------------------------------==
      IOUT=91
      CALL FILEOPEN(IOUT,'KPTS_GENERATION',FO_DEF,FERROR)
      WRITE(IOUT,'(75("*"),/," K290 - SPECIAL K POINTS GENERATION - ")')
      CALL DATUM(DATX)
      CALL XSTRING(HNAME,IH1,IH2)
      WRITE(IOUT,'(75("*"),/," COMPUTER: ",A," AT ",A26,/)')
     &     HNAME(IH1:IH2),DATX
      CALL XSTRING(USER,IU1,IU2)
      WRITE(IOUT,
     &     '(" CPMD JOB - (",A,") PID= ",I7," ON ",A,10(" "))')
     &     USER(IU1:IU2), MY_PID, HNAME(IH1:IH2)
      CALL MY_GETARG(1,FILENAME)
      CALL XSTRING(FILENAME,IU1,IU2)
      WRITE(FFORMAT,'(A,I2,A)') '(A,T',MAX(21,45-(IU2-IU1)),',A,/)'
      WRITE(IOUT,FFORMAT)   ' FROM THE INPUT FILE : ',FILENAME(IU1:IU2)
C     ==--------------------------------------------------------------==
C     == Allocations of arrays                                        ==
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_XKAPA, 3*NAT,                 'XKAPA')
      CALL MEMORY(IP_RX   , 3*NAT,                 'RX')
      CALL MEMORY(IP_TVECT, 3*NAT,                 'TVECT')
      CALL MEMORY(IP_TY,    NAT/IRAT+1,            'TY')
      CALL MEMORY(IP_ISC,   NAT/IRAT+1,            'ISC')
      CALL MEMORY(IP_F0,    (49*NAT)/IRAT+1,       'F0')
      CALL MEMORY(IP_WVKL,  3*NKPOINT,             'WVKL')
      CALL MEMORY(IP_LWGHT, NKPOINT/IRAT+1,        'LWGHT')
      CALL MEMORY(IP_LROT,  (48*NKPOINT)/IRAT+1,   'LROT')
      CALL MEMORY(IP_INCLUD,NKPOINT/IRAT+1,        'INCLUD')
      NHASH = MAX(2000,NKPOINT/10)
      CALL MEMORY(IP_LIST,  (NKPOINT+NHASH)/IRAT+1,'LIST')
      CALL MEMORY(IP_RLIST, 3*NKPOINT,             'RLIST')
C     ==--------------------------------------------------------------==
C     == INPUT DATA FOR K290                                          ==
C     ==--------------------------------------------------------------==
C     Symmetrization of Monkhorst-Pack mesh
      IF(TSYMKP) THEN
        ISTRIZ = 1
      ELSE
        ISTRIZ = -1
      ENDIF
C     Strain
      DO I = 1,6
        STRAIN(I) = 0.D0
      ENDDO
C     If option FULL (symmetry with disorder atoms).
      DO I=1,3
        XRANDOM(I)=0.D0
      ENDDO
      NAT0=0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          IF(TKFULL) THEN
            DO I=1,3
              XRANDOM(I)=0.1D0*XRANF()
            ENDDO
          ENDIF
          NAT0=NAT0+1
          TY(NAT0) = IS
          DO I=1,3
            XKAPA(I,NAT0) = TAU0(I,IA,IS)+XRANDOM(I)*ALAT
          ENDDO
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      WRITE(6,'(/,1X,19("*"),A,16("*"))')
     &     ' SPECIAL K-POINTS GENERATION '
      WRITE(6,'(" DIMENSIONS ARE:")')
      WRITE(6,'(3X," NUMBER OF ATOMS",T58,I8)') NAT
      WRITE(6,'(3X," K POINTS MONKHORST-PACK MESH",T42,3I8)')
     &     NK1,NK2,NK3
      WRITE(6,'(3X," MAXIMAL NUMBER OF K POINTS",T58,I8)') NKPOINT
      WRITE(6,'(3X," CONSTANT VECTOR SHIFT (MACDONALD)",T42,3F8.3)')
     &     WVK0
      IF(TSYMKP) THEN
        WRITE(6,'(3X," SYMMETRIC SPECIAL K POINTS")')
      ELSE
        WRITE(6,'(3X," NON SYMMETRIC SPECIAL K POINTS")')
      ENDIF
      IF(TKFULL) WRITE(6,'(3X,A,T41,A25)')
     &     ' FULL MONKHORST-PACK MESH','(ONLY INVERSION SYMMETRY)'
C     ==--------------------------------------------------------------==
C     == SUBROUTINE K290                                              ==
C     ==--------------------------------------------------------------==
      CALL K290(IOUT,NAT,NKPOINT,NSP,NK1,NK2,NK3,ISTRIZ,
     &     A1,A2,A3,ALAT,STRAIN,XKAPA,RX,TVECT,
     &     TY,ISC,F0,NTVECT,WVK0,WVKL,LWGHT,LROT,
     &     NHASH,INCLUD,LIST,RLIST,DELTASYM)
C     ==--------------------------------------------------------------==
      ISUM=0
      NKPTS=0
      DO IKIND=1,NKPOINT
        IF(LWGHT(IKIND).NE.0) THEN
          NKPTS=IKIND
          ISUM = ISUM + LWGHT(NKPTS)
        ELSE
          GOTO 1000
        ENDIF
      ENDDO
 1000 CONTINUE
      SUM=DBLE(ISUM)
      CALL MEMORY(IP_WK,NKPTS,'WK')
      CALL MEMORY(IP_RK,3*NKPTS,'RK')
      DO IKIND = 1 , NKPTS
        WEIGHT = DBLE(LWGHT(IKIND))/SUM                         
        WK(IKIND)=WEIGHT
C       wvkl are in cartesian coordinates.
        DO L = 1 , 3 
          RK(L,IKIND)=DBLE(WVKL(L,IKIND))
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      WRITE(6,'(A,T15,A51)') ' -->        ',
     &     '(SEE THE FILE KPTS_GENERATION FOR MORE INFORMATION)'
      WRITE(6,'(1X,A,T58,I8)')
     &    'NUMBER OF SPECIAL K POINTS (IN CARTESIAN COORDINATES):',
     &     NKPTS
      WRITE(6,'(T5,"NKP",T14,"KX",T29,"KY",T44,"KZ",T58,"WEIGHT")')
      DO IKIND = 1 , NKPTS
        WRITE(6,'(2X,I4,3F15.6,F14.6)')
     &       IKIND,(RK(L,IKIND),L=1,3),WK(IKIND)
      ENDDO
      WRITE(6,'(1X,64("*"),/)')
C     ==--------------------------------------------------------------==
      CALL FILECLOSE(IOUT)
      CALL FREEM(IP_XKAPA)
      CALL FREEM(IP_RX)
      CALL FREEM(IP_TVECT)
      CALL FREEM(IP_TY)
      CALL FREEM(IP_ISC)
      CALL FREEM(IP_F0)
      CALL FREEM(IP_WVKL)
      CALL FREEM(IP_LWGHT)
      CALL FREEM(IP_LROT)
      CALL FREEM(IP_INCLUD)
      CALL FREEM(IP_LIST)
      CALL FREEM(IP_RLIST)
      CALL TIHALT('    BZMESH',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SETMASK(GCUTW)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NKPNT NKPTS NGW GCUTW
      INCLUDE 'kpts.inc'        !TKBLOCK
      INCLUDE 'kpnt.inc'        !HGKP HGKM
      INCLUDE 'cppt.inc'        !HG
      INCLUDE 'sphe.inc'        !MASKGW NGWKPS
C     Arguments
      REAL*8 GCUTW
C     Variables
      DIMENSION MASKGW(2,MASKL,NKPNT),NGWKPS(NKPTS)
      DIMENSION HGKP(NHG,NKPNT),HGKM(NHG,NKPNT)
      INTEGER IKPT,IG
      REAL*8  XSUM
C     ==--------------------------------------------------------------==
C     MASKGW(1,*,*) mask for  g
C     MASKGW(2,*,*) mask for -g
      DO IKPT=1,NBLKP
        IF(TKBLOCK) CALL RKPT_SWAP(HGKP,1,IKPT,'HGKP HGKM')
        CALL CALC_MASKGW(IKPT)
        IF(TKBLOCK) CALL WKPT_SWAP(MASKGW,1,IKPT,'MASKGW')
      ENDDO
      XSUM=0.D0
      DO IG=1,NGW
        IF(HG(IG).LT.GCUTW) XSUM=XSUM+1.D0
      ENDDO
      CALL GLOSUM(1,XSUM)
      NGW_GAMMA=2*NINT(XSUM)-1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CALC_MASKGW(IKPT)
C     ==--------------------------------------------------------------==
C     == CALCULATION OF MASKGW FOR A SET OF NKPNT K POINTS            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NKPNT NKPTS NGW NKPBL
      INCLUDE 'kpnt.inc'        !HGKP HGKM 
      INCLUDE 'sphe.inc'        !MASKGW NGWKPS
      INCLUDE 'gvec.inc'        !GCUTW
C     Arguments
      INTEGER IKPT
C     Variables
      DIMENSION MASKGW(2,MASKL,NKPNT),NGWKPS(NKPTS)
      DIMENSION HGKP(NHG,NKPNT),HGKM(NHG,NKPNT)
      INTEGER IK,IKK,IG,IGG,NGLOW
      REAL*8  XSUM
C     ==--------------------------------------------------------------==
C     MASKGW(1,*,*) mask for  g
C     MASKGW(2,*,*) mask for -g
      NGLOW=NGW-MASKL+1
      CALL AZZERO(MASKGW,2*MASKL*NKPNT)
      DO IK=1,NKPBL(IKPT)
        IKK=KPBEG(IKPT)+IK
        DO IG=NGLOW,NGW
          IGG=IG-NGLOW+1
          IF(HGKP(IG,IK).LT.GCUTW) MASKGW(1,IGG,IK)=1.0D0
          IF(HGKM(IG,IK).LT.GCUTW) MASKGW(2,IGG,IK)=1.0D0
        ENDDO
        NGWKPS(IKK)=2*(NGW-MASKL)
        DO IGG=1,MASKL
          IF(MASKGW(1,IGG,IK).GT.0) NGWKPS(IKK)=NGWKPS(IKK)+1
          IF(MASKGW(2,IGG,IK).GT.0) NGWKPS(IKK)=NGWKPS(IKK)+1
        ENDDO
        XSUM=NGWKPS(IKK)
        CALL GLOSUM(1,XSUM)
        NGWKPS(IKK)=NINT(XSUM)-1
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CALC_HGK(IKPT)
C     ==--------------------------------------------------------------==
C     == CALCULATION OF HGKM AND HGKP FOR A SET OF NKPNT K POINTS     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NKPNT NKPTS NHG NR1S NR2S NR3S KPBEG
      INCLUDE 'elct.inc'        !N F(N,NKPTS)
      INCLUDE 'kpnt.inc'        !RK,WK,HGKP,HGKM
      INCLUDE 'gvec.inc'        !B1 B2 B3
      INCLUDE 'cppt.inc'        !INYH
C     Arguments
      INTEGER IKPT
C     Variables
      DIMENSION HGKP(NHG,NKPNT),HGKM(NHG,NKPNT),
     &          F(N,NKPTS),RK(3,NKPTS),WK(NKPTS)
      INTEGER IK,IKK,IG,I,J,K,IR,NH1,NH2,NH3
      REAL*8  G2P,G2M,T,TP,TM
C     ==--------------------------------------------------------------==
      NH1=NR1S/2+1
      NH2=NR2S/2+1
      NH3=NR3S/2+1
      DO IK=1,NKPBL(IKPT)
        IKK=KPBEG(IKPT)+IK
        DO IG=1,NHG
          I=INYH(1,IG)-NH1
          J=INYH(2,IG)-NH2
          K=INYH(3,IG)-NH3
          G2P=0.D0
          G2M=0.D0 
          DO IR=1,3
            T=DBLE(I)*B1(IR)+DBLE(J)*B2(IR)+DBLE(K)*B3(IR)
            TP=RK(IR,IKK)+T
            TM=RK(IR,IKK)-T
            G2P=G2P+TP*TP
            G2M=G2M+TM*TM
          ENDDO
          HGKP(IG,IK)=G2P
          HGKM(IG,IK)=G2M
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
