C     ==================================================================
      SUBROUTINE RLBFGS (XPAR, DXPAR, NDIM, ETOT, LRET)
C     ==--------------------------------------------------------------==
C     ==  LIMITED MEMORY BFGS QUASI-NEWTON OPTIMIZER                  ==
C     ==                                                              ==
C     ==  Reference:                                                  ==
C     ==  D. Liu and J. Nocedal, Math. Progr. B 45 (1989) 503-528     ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   XPAR(NDIM)  Coordinates                                    ==
C     ==   DXPAR(NDIM) Gradient                                       ==
C     ==   NDIM        Number of degrees of freedom                   ==
C     ==   ETOT        Total energy (for trust radius or line search) ==
C     ==   LRET        Accepted step: return (no new step) if .T.     ==
C     == OUTPUT:                                                      ==
C     ==   XPAR(NDIM)  New coordinates                                ==
C     ==   LRET        Return to get new energy / gradient if .T.     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'tpar.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'store.inc' ! print level
      INCLUDE 'lscal.inc' ! parameters and pointers to work space
C     Arguments
      LOGICAL LRET
      INTEGER NDIM
      REAL*8  XPAR(NDIM), DXPAR(NDIM), ETOT
C     Externals
      REAL*8 DDOT
      EXTERNAL DDOT
C     Variables
      LOGICAL LSTPLS
      INTEGER I, ICP, ICPND, IGCN, INMC, IPT, ISCN, ISUB,
     $        MAXFEV
      REAL*8  ALPHA, BETA, DIAG, ETOTSC, GRDNRM, GRDSQU, GRDSTP,
     $        RHO, SCL, SCLPRV, SQ, STPPRD, YR
C     ==--------------------------------------------------------------==
      CALL TISET('    RLBFGS',ISUB)
 10   CONTINUE
C     ==--------------------------------------------------------------==
C     ==  All scratch space needs to be preserved (Hessian etc.)      ==
C     ==    See lscal.inc for shared information, work arrays, etc.   ==
C     ==    See BLOCK DATA below for initial values and explanations  ==
C     ==--------------------------------------------------------------==
C     ==  Organization of L-BFGS work space                           ==
C     ==                                                              ==
C     ==  The subroutine implements the formula given by Nocedal in   ==
C     ==  Formula by J. Nocedal, Math. of Computation, 24, 1980, 773. ==
C     ==  Therefore, the organization of WLBFGS is similar to the     ==
C     ==  reference implementation by Nocedal.                        ==
C     ==                                                              ==
C     ==  Start    Size      Meaning                                  ==
C     ==  -----    ----      -------                                  ==
C     ==  1        NDIM      Work area / space for previous gradient  ==
C     ==  NDIM+1   NREM      Storage for RHO scalars                  ==
C     ==  2*NDIM+1 NREM      Storage for ALPHA scalars                ==
C     ==  ISPNT    NDIM*NREM NREM performed steps                     ==
C     ==  IGPNT    NDIM*NREM NREM gradient diffs res. from steps      ==
C     ==  IWPNT    NDIM      Work area for line search                ==
C     ==--------------------------------------------------------------==
C     ==  These tests are performed every time                        ==
C     ==  See also lscal.inc                                          ==
C     ==--------------------------------------------------------------==
      IPRNT = IPRINT(IPRINT_LSCAL)
      IF (NSTRIED_L.EQ.0 .AND. IPRNT.GE.1) THEN
         WRITE (6,*) 'ENTERING LIMITED MEMORY BFGS OPTIMIZER'
      END IF
      IF (NREM.EQ.0) THEN
         NREM = MIN (40, NDIM)
         IF (IPRNT.GE.1) WRITE (6,*)
     $        'SETTING THE DEFAULT NUMBER OF REMEMBERED L-BFGS STEPS'
      END IF
      IF (NSTRIED_L.EQ.0 .AND. TRUSTR.NE.0.0D0) THEN
         STEP_INI_L = TRUSTR
         STEP_MAX_L = TRUSTR
      END IF
      IF (NSTRIED_L.EQ.0 .AND. NTRUST.NE.0) THEN
         STEP_INI_L = STEP_INI_L / 5.0D0
      END IF
      IF (NSTRIED_L.EQ.0 .AND. IPRNT.GE.1) THEN
         WRITE (6,*) 'L-BFGS INPUT OPTIONS:'
         WRITE (6,'(3X,A,I5)') 'NREM: ', NREM
         WRITE (6,'(3X,A,I5)') 'NTRUST: ', NTRUST
         WRITE (6,'(3X,A,I5)') 'NRESTT: ', NRESTT
         WRITE (6,'(3X,A,I5)') 'NTRSTR: ', NTRSTR
         WRITE (6,*)
         IF (NTRUST.EQ.0) THEN
            IF (TRUSTR.EQ.0.0D0) THEN
               WRITE (6,*) 'L-BFGS COMPILED-IN OPTIONS:'
            ELSE
               WRITE (6,*) 'L-BFGS OPTIONS BASED ON INPUT:'
            END IF
            WRITE (6,'(3X,A,F12.8)') 'STEP_INI_L: ', STEP_INI_L
            WRITE (6,'(3X,A,F12.8)') 'STEP_MIN_L: ', STEP_MIN_L
            WRITE (6,'(3X,A,F12.8)') 'STEP_MAX_L: ', STEP_MAX_L
            WRITE (6,'(3X,A,F12.8)') 'STEP_BMB_L: ', STEP_BMB_L
         ELSE
            WRITE (6,'(3X,A,F12.8)')
     $           'INITIAL GUESS FOR LINE SEARCH STEP: ', STEP_INI_L
         END IF
         WRITE (6,*)
      END IF
      IF (NRESTT.GT.0 .AND. NSTEP_L.GT.0 .AND. JUMP.EQ.2) THEN
         IF (MOD(NSTEP_L,NRESTT).EQ.0) THEN
            IF (IPRNT.GE.1) WRITE (6,*) 'PERIODIC RESET OF L-BFGS'
            ITER_L = 0
            IBND_L = 0
            IPNT_L = 0
            NPT_L  = 0
            STEP_L = STEP_INI_L
            LINILS = .TRUE.
         END IF
      END IF
      IF (NTRSTR.GT.0 .AND. NSTEP_L.GT.NTRSTR) NTRUST = 1
      IF (NTRUST.EQ.1 .AND. NSTEP_L.EQ.0 .AND. TOLAD.GT.0.0D0) THEN
         NTRUST = 0
         NTRSTR = 2
         IF (IPRNT.GE.1) WRITE (6,'(1X,A,A)') 'ADAPTIVE TOLERANCE: ',
     $        'USING TRUST RADIUS FOR THE FIRST TWO STEPS'
      END IF
      IF ((NTRUST.EQ.1 .OR. NTRSTR.GT.0) .AND. TOLAD.GT.0.0D0 .AND.
     $     LMICRO) THEN
         NTRUST = 0
         NTRSTR = 0
         IF (IPRNT.GE.1) WRITE (6,'(1X,A,/,1X,A)')
     $        'MICROITERATIVE TS SEARCH AND ADAPTIVE TOLERANCE:',
     $        'USING TRUST RADIUS INSTEAD OF LINE SEARCH'
      END IF
C     ==--------------------------------------------------------------==
C     ==  Initialization  / Reset                                     ==
C     ==  JUMP = 0 / -1                                               ==
C     ==--------------------------------------------------------------==
      IF (JUMP.LE.0) THEN
         IF (IPRNT.GE.1) WRITE (6,*) '  JUMP=0, INITIALIZING L-BFGS'
         ISPNT  = NDIM+2*NREM          ! pointer to step area
         IGPNT  = ISPNT + NDIM*NREM    ! pointer to gradient area
         IWPNT  = IGPNT + NDIM*NREM    ! pointer to line search area
         IPNT_L = 0                    ! counter for circular storage
         IBND_L = 0                    ! number of stored steps
         ITER_L = 0                    ! number of performed steps
         NPT_L  = 0                    ! offset in the L-BFGS history
         CALL AZZERO (WLBFGS, NDIM+2*NREM+2*NDIM*NREM)
         CALL DCOPY (NDIM, DXPAR, 1, WLBFGS(ISPNT+1), 1)
         CALL DSCAL (NDIM, -1.0D0, WLBFGS(ISPNT+1), 1) ! DIAG = 1
         STEP_L = STEP_INI_L
         IF (.NOT.LRET) JUMP = 2
      END IF
C     ==--------------------------------------------------------------==
C     ==  Form an L-BFGS step                                         ==
C     ==  JUMP = 2                                                    ==
C     ==--------------------------------------------------------------==
      IF (JUMP.EQ.2) THEN
         IF (IPRNT.GE.1) WRITE(6,*) '  JUMP=2, FORMING L-BFGS STEP'
         ITER_L = ITER_L + 1
         IF (ITER_L.GT.1) THEN
            IBND_L = ITER_L - 1
            IF (ITER_L.GT.NREM) IBND_L = NREM
            GRDSTP = DDOT (NDIM, WLBFGS(IGPNT+NPT_L+1), 1,
     $                     WLBFGS(ISPNT+NPT_L+1), 1)
            GRDSQU = DDOT (NDIM, WLBFGS(IGPNT+NPT_L+1), 1,
     $                     WLBFGS(IGPNT+NPT_L+1), 1)
            DIAG = GRDSTP / GRDSQU  ! only unit matrix
C     ==--------------------------------------------------------------==
C     ==  Formula by J. Nocedal, Math. of Computation, 24, 1980, 773  ==
C     ==--------------------------------------------------------------==
            ICP = IPNT_L
            IF (IPNT_L.EQ.0) ICP = NREM
            RHO = 1.0D0 / GRDSTP
            WLBFGS(NDIM+ICP) = RHO
            CALL DCOPY (NDIM, DXPAR, 1, WLBFGS, 1)
            CALL DSCAL (NDIM, -1.0D0, WLBFGS, 1)
            ICP = IPNT_L
C
            DO I = 1,IBND_L
               ICP = ICP - 1
               IF (ICP.EQ.-1) ICP = NREM - 1
               ICPND = ICP * NDIM
               SQ = DDOT (NDIM, WLBFGS(ISPNT+ICPND+1), 1, WLBFGS, 1)
               INMC = NDIM + NREM + ICP + 1 ! for alpha
               IGCN = IGPNT + ICPND         ! grad. step I-1 steps ago
               ALPHA = SQ * WLBFGS(NDIM+ICP+1)
               WLBFGS(INMC) = ALPHA
               CALL DAXPY (NDIM, -ALPHA, WLBFGS(IGCN+1), 1, WLBFGS, 1)
            END DO              ! I = 1,IBND_L
C
            CALL DSCAL (NDIM, DIAG, WLBFGS, 1)
C
            DO I = 1,IBND_L
               ICPND = ICP * NDIM
               YR = DDOT (NDIM, WLBFGS(IGPNT+ICPND+1), 1, WLBFGS, 1)
               BETA = YR * WLBFGS(NDIM+ICP+1)
               INMC = NDIM + NREM + ICP + 1 ! for beta
               BETA = WLBFGS(INMC) - BETA
               ISCN = ISPNT + ICPND         ! step IBND_L-I steps ago
               CALL DAXPY (NDIM, BETA, WLBFGS(ISCN+1), 1, WLBFGS, 1)
               ICP = ICP + 1
               IF (ICP.EQ.NREM) ICP = 0
            END DO              ! I = 1,IBND_L
C
C     ==  Store new search direction to WLBFGS                        ==
C
            CALL DCOPY (NDIM, WLBFGS, 1, WLBFGS(ISPNT+IPNT_L*NDIM+1), 1)
         END IF                                 ! IF (ITER.NE.1)
         OLDENE = ETOT                          ! save energy
         CALL DCOPY (NDIM, DXPAR, 1, WLBFGS, 1) ! save gradient
         CALL STACK_SCHED (IELSTK, ST_PUT)      ! save electronic WF
         NPT_L  = IPNT_L * NDIM
C
         INFO_L = 0
         IF (NTRUST.EQ.1) THEN                  ! line search
            IF (LINILS) THEN
               GRDNRM = DSQRT (DDOT (NDIM, DXPAR, 1, DXPAR, 1))
               SZ_L = STEP_L / GRDNRM ! trust radius first trial step
               LINILS = .FALSE.
            ELSE
               SZ_L = 1.0D0
               STPPRD = DSQRT (DDOT (NDIM, WLBFGS(ISPNT+NPT_L+1), 1,
     $              WLBFGS(ISPNT+NPT_L+1), 1))
               IF (STPPRD.GT.STEP_MAX_L) SZ_L = STEP_MAX_L / STPPRD
            END IF
         END IF
C
         JUMP = 3 + 10*NTRUST
      END IF                                    ! IF (JUMP.EQ.2)
C     ==--------------------------------------------------------------==
C     ==  Perform an L-BFGS step - HDLCopt trust radius algorithm     ==
C     ==  JUMP = 3                                                    ==
C     ==--------------------------------------------------------------==
      IF (JUMP.EQ.3) THEN
         IF (IPRNT.GE.1) WRITE(6,*) '  JUMP=3, PERFORMING L-BFGS STEP'
         IPT = ISPNT + NPT_L    ! pointer to stored new seach direction
         SZ_L = DSQRT (DDOT (NDIM, WLBFGS(IPT+1), 1, WLBFGS(IPT+1), 1))
C
C     ==  Replace stored search direction by actual step in WLBFGS    ==
C
         IF (IPRNT.GE.1) THEN
            WRITE (6,'(5X,A,F10.5)') 'TRUST RADIUS STEP:   ', STEP_L
            WRITE (6,'(5X,A,F10.5)') 'PREDICTED STEP SIZE: ', SZ_L
            WRITE (6,'(5X,A,F10.5)') 'PERFORMED STEP SIZE: ',
     $             MIN (SZ_L, STEP_L)
         END IF
         IF (SZ_L.GT.STEP_L) THEN
            SCL = STEP_L / SZ_L
            CALL DSCAL (NDIM, SCL, WLBFGS(IPT+1), 1) ! scale step
         END IF
C
         SCL1_L = DDOT (NDIM, DXPAR, 1, WLBFGS(IPT+1), 1)
         IF (SCL1_L.GT.0.0D0) THEN ! invert step direction if upward
            IF (IPRNT.GE.1) WRITE (6,*) '  INVERTING L-BFGS DIRECTION'
            SCL1_L = -SCL1_L
            CALL DSCAL (NDIM, -1.0D0, WLBFGS(IPT+1), 1)
         END IF
C
         CALL DAXPY (NDIM, 1.0D0, WLBFGS(IPT+1), 1, XPAR, 1) ! do step
C
         NSTRIED_L = NSTRIED_L + 1
         LRET = .TRUE.
         JUMP = 4
         GOTO 999               ! return for new energy / gradient
      END IF                    ! IF (JUMP.EQ.3)
C     ==--------------------------------------------------------------==
C     ==  Test an L-BFGS step - HDLCopt trust radius algorithm        ==
C     ==  JUMP = 4                                                    ==
C     ==--------------------------------------------------------------==
      IF (JUMP.EQ.4) THEN
         IF (IPRNT.GE.1) WRITE(6,*) '  JUMP=4, TESTING L-BFGS STEP'
         IPT = ISPNT + NPT_L       ! pointer to stored performed step
C
C     ==  Failure: energy increased                                   ==
C
         IF (ETOT.GT.OLDENE) THEN
            IF (IPRNT.GE.1) WRITE (6,'(5X,A,F12.6,A,F12.6)')
     $           'ENERGY INCREASED FROM ', OLDENE, ' TO ', ETOT
            STEP_L = 0.5D0 * MIN (SZ_L, STEP_L)
            CALL DAXPY (NDIM, -1.0D0, WLBFGS(IPT+1), 1, XPAR, 1)
            CALL STACK_SCHED (IELSTK, ST_GET) ! restore electronic WF
C
            IF (STEP_L.LT.STEP_MIN_L .AND. NRES_L.EQ.0) THEN
               NRES_L = NRES_L + 1 ! increment number of resets
               IF (IPRNT.GE.1)
     $              WRITE (6,*) '    STEP TOO SMALL, RESET L-BFGS'
               JUMP = -1
               GOTO 10             ! go to beginning of routine
            ELSE IF (STEP_L.LE.STEP_BMB_L) THEN
C AK: 2005/03/28. we should not stop here. 
C               CALL STOPGM ('RLBFGS', 'L-BFGS STEP TOO SMALL')
                WRITE(6,*) ' RLBFGS| WARNING! L-BFGS STEP TOO SMALL!'
                WRITE(6,*) ' RLBFGS| STOP ME IF THIS HAPPENS TOO OFTEN'
                GOTO 999
            ELSE                   ! simply reject step and retry
               IF (IPRNT.GE.1)
     $              WRITE (6,*) '    TRYING SMALLER STEP'
               JUMP = 3 + 10*NTRUST
               GOTO 10             ! go to beginning of routine
            END IF
C
C     ==  Success: energy decreased                                   ==
C
         ELSE                      ! IF (ETOT.GT.OLDENE) ...
            IF (IPRNT.GE.1) WRITE (6,'(5X,A,F12.6,A,F12.6)')
     $           'ENERGY DECREASED FROM ', OLDENE, ' TO ', ETOT
            NRES_L = 0
            NSTEP_L = NSTEP_L + 1
C
C     ==  Store gradient step to WLBFGS                               ==
C
            CALL DCOPY (NDIM, DXPAR, 1, WLBFGS(IGPNT+NPT_L+1), 1)
            CALL DAXPY (NDIM, -1.0D0, WLBFGS, 1,
     $                  WLBFGS(IGPNT+NPT_L+1), 1)
            SCL2_L = DDOT (NDIM, DXPAR, 1, WLBFGS(IPT+1), 1)
C
            IPNT_L = IPNT_L + 1            ! new entry in WLBFGS
            IF (IPNT_L.EQ.NREM) IPNT_L = 0 ! cycle if history full
C
            IF (ETOT.LE.OLDENE+SCL1_L*BETAP_L .AND.
     $          DABS(SCL2_L).LE.DABS(SCL1_L)*BETA_L) THEN
               IF (IPRNT.GE.1)
     $              WRITE (6,*) '    STEP PASSED WOLFE CONDITIONS'
               STEP_L = 2.0D0 * STEP_L     ! Wolfe conditions satisfied
            ELSE
               IF (IPRNT.GE.1)
     $              WRITE (6,*) '    STEP FAILED WOLFE CONDITIONS'
               STEP_L = MIN (SZ_L, STEP_L) ! Wolfe cond. not satisfied
            END IF
C
            IF (ETOT.LE.OLDENE+SCL1_L*BETAP_L .AND. STEP_L.LE.SZ_L) THEN
               STEP_L = 1.25D0 * STEP_L    ! Small step but energy decr.
            END IF
C
            DEACCP = ETOT - OLDENE ! energy change of step
            STEP_L = MIN (STEP_L, STEP_MAX_L)
            JUMP = 2               ! next action: form an L-BFGS step
            IF (LRET) THEN
               LRET = .FALSE.      ! we do not need new energy / gradient
               GOTO 999            ! return to check convergence
            ELSE
               GOTO 10             ! go to beginning of optimizer
            END IF
         END IF                    ! IF (ETOT.GT.OLDENE) ... ELSE ...
      END IF                       ! IF (JUMP.EQ.4)
C     ==--------------------------------------------------------------==
C     ==  Perform an L-BFGS step - line search                        ==
C     ==  JUMP = 13                                                   ==
C     ==--------------------------------------------------------------==
      IF (JUMP.EQ.13) THEN
         IF (IPRNT.GE.1) WRITE(6,*) '  JUMP=13, PERFORMING LINE SEARCH'
         MAXFEV = 4
         NSTRIED_L = NSTRIED_L + 1
C
         IF (IPRNT.GE.1) THEN
            STPPRD = DSQRT (DDOT (NDIM, WLBFGS(ISPNT+NPT_L+1), 1,
     $                    WLBFGS(ISPNT+NPT_L+1), 1))
            WRITE (6,'(5X,A,F12.6)') 'PREDICTED STEP:      ', STPPRD
            IF (NFEV_L.GE.1) THEN
               IF (ETOT.GT.OLDENE) THEN
                  WRITE (6,'(5X,A,F12.6,A,F12.6)')
     $                 'ENERGY INCREASED FROM ', OLDENE, ' TO ', ETOT
               ELSE
                  WRITE (6,'(5X,A,F12.6,A,F12.6)')
     $                 'ENERGY DECREASED FROM ', OLDENE, ' TO ', ETOT
               END IF
            END IF
         END IF
C
         IF (INFO_L.EQ.0) THEN         ! remember the best cycle
            ETOTMN = ETOT              ! no step yet done
            NFEVMN = -1
            SZ_LMN = STEP_MIN_L/STPPRD ! to avoid singularities
         ELSE IF (ETOT.LT.ETOTMN) THEN
            ETOTMN = ETOT              ! lowest energy so far
            NFEVMN = NFEV_L
            SZ_LMN = SZ_L
         END IF
C
         SCLPRV = SZ_L
         ETOTSC = ETOT
         CALL LSRCH (NDIM, XPAR, ETOTSC, DXPAR, WLBFGS(ISPNT+NPT_L+1),
     $               SZ_L, MAXFEV, INFO_L, NFEV_L, WLBFGS(IWPNT))
C
         IF (IPRNT.GE.1) THEN
            WRITE (6,'(5X,A,I3)') 'LINE SEARCH CYCLE: ', NFEV_L
            WRITE (6,'(5X,A,F12.6)') 'LINE SEARCH SCALING: ', SZ_L
            WRITE (6,'(5X,A,F12.6)') 'ENERGY CHANGE:  ', ETOTSC-OLDENE
         END IF
C
         IF (LFEVMN) THEN           ! accept step of minimum in any case
            LFEVMN = .FALSE.
            INFO_L = 1
         END IF
C
         LSTPLS = .FALSE.
         IF (INFO_L.EQ.3) THEN       ! maximum number of cycles reached
            IF (IPRNT.GE.1) WRITE (6,'(5X,A,I3)')
     $           'MAXIMUM NUMBER OF LINE SEARCH CYCLES REACHED'
            LSTPLS = .TRUE.
         ELSE IF (INFO_L.EQ.-1) THEN ! calculate energy / gradient
            CALL STACK_SCHED (IELSTK, ST_GET) ! restore electronic WF
            IF (ABS(STPPRD*(SZ_L-SCLPRV)).LE.STEP_MIN_L .AND.
     $           NFEV_L.GT.1) THEN
               IF (IPRNT.GE.1) THEN
                  WRITE (6,'(5X,A,A,1PE10.4)') 'MINIMUM ALLOWED ',
     $                 'CHANGE OF A LINE SEARCH STEP: ', STEP_MIN_L
                  WRITE (6,'(5X,A,1PE10.4)') 'ACTUAL CHANGE: ',
     $                 STPPRD*(SZ_L-SCLPRV)
               END IF
               LSTPLS = .TRUE.
            ELSE IF (ABS(STPPRD*SZ_L).LE.STEP_MIN_L .AND.
     $           NFEV_L.GT.1) THEN
               IF (IPRNT.GE.1) THEN
                  WRITE (6,'(5X,A,A,1PE10.4)') 'MINIMUM ALLOWED ',
     $                 'LINE SEARCH STEP: ', STEP_MIN_L
                  WRITE (6,'(5X,A,1PE10.4)') 'ACTUAL STEP: ',
     $                 STPPRD*SZ_L
               END IF
               LSTPLS = .TRUE.
            ELSE
               JUMP = 13
               GOTO 999              ! return
            END IF
         ELSE IF (INFO_L.EQ.1) THEN  ! step ok
            NSTEP_L = NSTEP_L + 1
            JUMP = 14
         ELSE                        ! error condition
            WRITE (6,*) 'ERROR IN LINE SEARCH, CODE ', INFO_L
            CALL STOPGM ('RLBFGS','CANNOT RECOVER')
         END IF
C
         IF (LSTPLS) THEN            ! maximum number of cycles
            IF (NFEVMN.LT.NFEV_L-2 .AND. 
     $           ABS(STPPRD*(SZ_L-SZ_LMN)).GT.STEP_MIN_L) THEN
               IF (IPRNT.GE.1) WRITE (6,'(5X,A,I3)')
     $              'GOING BACK TO GUESS ', NFEVMN
               SCLPRV = SZ_L
               SZ_L = SZ_LMN
               CALL DAXPY (NDIM, SZ_L-SCLPRV, WLBFGS(ISPNT+NPT_L+1),
     $              1, XPAR, 1)
               LFEVMN = .TRUE.
               JUMP = 13
               GOTO 999         ! return
            ELSE
               IF (IPRNT.GE.1) WRITE (6,'(5X,A)')
     $              'ACCEPTING PREVIOUS GUESS'
               CALL DAXPY (NDIM, SCLPRV-SZ_L, WLBFGS(ISPNT+NPT_L+1),
     $              1, XPAR, 1)
               NSTEP_L = NSTEP_L + 1
               JUMP = 14
            END IF
         END IF
C
      END IF                         ! IF (JUMP.EQ.13)
C     ==--------------------------------------------------------------==
C     ==  Perform the L-BFGS step determined by the line search       ==
C     ==  JUMP = 14                                                   ==
C     ==--------------------------------------------------------------==
      IF (JUMP.EQ.14) THEN
         IF (IPRNT.GE.1) THEN
            WRITE(6,*) '  JUMP=14, ACCEPTING STEP'
            WRITE (6,'(5X,A,F10.7)') 'PERFORMED STEP:      ',STPPRD*SZ_L
         END IF
C
         CALL DSCAL (NDIM, SZ_L, WLBFGS(ISPNT+NPT_L+1), 1)
         CALL DCOPY (NDIM, DXPAR, 1, WLBFGS(IGPNT+NPT_L+1), 1)
         CALL DAXPY (NDIM, -1.0D0, WLBFGS, 1, WLBFGS(IGPNT+NPT_L+1), 1)
C
         DEACCP = ETOT - OLDENE                 ! energy change
         IPNT_L = IPNT_L + 1                    ! new entry in WLBFGS
         IF (IPNT_L.EQ.NREM) IPNT_L = 0         ! cycle if history full
C
         JUMP = 2               ! unconditional: next step
         IF (LRET) THEN
            LRET = .FALSE.      ! we do not need new energy / gradient
            GOTO 999            ! return to test convergence
         ELSE
            GOTO 10             ! go to beginning of optimizer
         END IF
      END IF                    ! IF (JUMP.EQ.14)
C     ==--------------------------------------------------------------==
 999  CONTINUE                  ! RETURN
      CALL TIHALT('    RLBFGS',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE STACK_INIT (ISTACK, LENENT, ISIZE, IOPER)
C     ==--------------------------------------------------------------==
C     ==  Stack routines for correct store / restore of distributed   ==
C     ==  data controlled from routines running on one node only      ==
C     ==                                                              ==
C     ==  The operations are scheduled using STACK_SCHED and executed ==
C     ==  later by STACK_DO_SCHED on all processors (which resets the ==
C     ==  schedule). The information and pointers for the stacks are  ==
C     ==  defined in lscal.inc                                        ==
C     ==                                                              ==
C     ==  Currently, overlow and underflow are allowed. This might    ==
C     ==  change later                                                ==
C     ==                                                              ==
C     ==  Supported operations:                                       ==
C     ==  ST_NOP:  no operation                                       ==
C     ==  ST_PUSH: add current vector to the top of the stack         ==
C     ==  ST_PUT:  top of the stack is replaced by the current vector ==
C     ==  ST_POP:  get and remove the vector on the top of the stack  ==
C     ==  ST_GET:  get the vector on the top of the stack             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'lscal.inc'
C     Arguments
      INTEGER ISTACK, LENENT, IOPER, ISIZE
C     ==--------------------------------------------------------------==
      IF (ITPSTK(ISTACK) .GT. -1) RETURN ! Stack already initialized
      IF (ISTACK.EQ.IELSTK) THEN
         ISZSTK(ISTACK) = ISIZE
         ISTLEN(ISTACK) = LENENT
         ITPSTK(ISTACK) = 0
         IOPNXT(ISTACK) = IOPER
         CALL MEMORY (IP_ELESTK, ISZSTK(ISTACK)*LENENT, 'ELESTK')
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE STACK_SCHED (ISTACK, IOPER)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'lscal.inc'
C     Arguments
      INTEGER ISTACK, IOPER
C     ==--------------------------------------------------------------==
      IOPNXT(ISTACK) = IOPER
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE STACK_DO_SCHED (ISTACK, VAR, LENENT)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'lscal.inc'
      INCLUDE 'irat.inc'
C     Arguments
      INTEGER ISTACK, LENENT
      REAL*8 VAR(LENENT)
C     Variables
      INTEGER IOPER, MSGLEN
C     ==--------------------------------------------------------------==
      IOPER = IOPNXT(ISTACK)
      MSGLEN = 8/IRAT
      CALL MY_BCAST (IOPER, MSGLEN, SOURCE, ALLGRP)
      CALL STACK_DO (ISTACK, IOPER, VAR, LENENT)
      IOPNXT(ISTACK) = ST_NOP
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE STACK_DO (ISTACK, IOPER, VAR, LENENT)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'lscal.inc'
C     ==--------------------------------------------------------------==
C     Arguments
      INTEGER ISTACK, IOPER, LENENT
      REAL*8  VAR(LENENT)
C     Variables
      INTEGER IB
C     ==--------------------------------------------------------------==
C
C     ==  CATCH ERROR CONDITIONS                                      ==
C
      IF (ITPSTK(ISTACK).LT.0) THEN
         IF (PARENT) WRITE (6,*) 'STACK ', ISTACK, ' UNINITIALIZED'
         CALL STOPGM ('STACK_DO_SCHED', 'CANNOT RECOVER')
      END IF
      IF (ITPSTK(ISTACK).EQ.0 .AND.
     $     (IOPER.EQ.ST_POP .OR. IOPER.EQ.ST_GET)) THEN
         IF (PARENT) WRITE (6,*) 'NO DATA ON STACK ', ISTACK
         CALL STOPGM ('STACK_DO_SCHED', 'CANNOT RECOVER')
      END IF
      IF (ISTLEN(ISTACK).NE.LENENT) THEN
         IF (PARENT) THEN
            WRITE (6,*) 'STACK ', ISTACK, ' ENTRY SIZE MISMATCH'
            WRITE (6,*) 'ALLOCATED:         ', ISTLEN(ISTACK)
            WRITE (6,*) 'DATA TO BE STORED: ', LENENT
         END IF
         CALL STOPGM ('STACK_DO_SCHED', 'CANNOT RECOVER')
      END IF
C
C     ==  DETERMINE PLACE WHERE TO PUT / GET DATA AND NEW TOP         ==
C
      IF (IOPER.EQ.ST_PUT) THEN
         IF (ITPSTK(ISTACK).EQ.0) ITPSTK(ISTACK) = 1
         IB = (ITPSTK(ISTACK)-1) * ISTLEN(ISTACK)
      ELSE IF (IOPER.EQ.ST_GET) THEN
         IB = (ITPSTK(ISTACK)-1) * ISTLEN(ISTACK)
      ELSE IF (IOPER.EQ.ST_PUSH) THEN
         IF (ITPSTK(ISTACK).LT.ISZSTK(ISTACK)) THEN
            ITPSTK(ISTACK) = ITPSTK(ISTACK) + 1
         ELSE
            ITPSTK(ISTACK) = 1
            IF (PARENT)
     $           WRITE (6,*) 'WARNING: STACK ', ISTACK, ' OVERFLOW'
         END IF
         IB = (ITPSTK(ISTACK)-1) * ISTLEN(ISTACK)
      ELSE IF (IOPER.EQ.ST_POP) THEN
         IB = (ITPSTK(ISTACK)-1) * ISTLEN(ISTACK)
         IF (ITPSTK(ISTACK).GT.1) THEN
            ITPSTK(ISTACK) = ITPSTK(ISTACK) - 1
         ELSE
            ITPSTK(ISTACK) = ISZSTK(ISTACK)
            IF (PARENT)
     $           WRITE (6,*) 'WARNING: STACK ', ISTACK, ' UNDERFLOW'
         END IF         
      END IF
C
C     ==  COPY THE DATA                                               ==
C
      IF (IOPER.EQ.ST_PUSH .OR. IOPER.EQ.ST_PUT) THEN
         IF (ISTACK.EQ.IELSTK) THEN
            CALL DCOPY (LENENT, VAR, 1, ELESTK(IB+1), 1)
         END IF
      ELSE IF (IOPER.EQ.ST_POP .OR. IOPER.EQ.ST_GET) THEN
         IF (ISTACK.EQ.IELSTK) THEN
            CALL DCOPY (LENENT, ELESTK(IB+1), 1, VAR, 1)
         END IF
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE STACK_DESTROY (ISTACK)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'lscal.inc'
C     Arguments
      INTEGER ISTACK
C     ==--------------------------------------------------------------==
      IF (ITPSTK(ISTACK).EQ.-1) RETURN
      IF (ISTACK.EQ.IELSTK) THEN
         CALL FREEM (IP_ELESTK)
      END IF
      IOPNXT(ISTACK) = ST_NOP
      ISZSTK(ISTACK) =  1
      ISTLEN(ISTACK) =  0
      ITPSTK(ISTACK) = -1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE STACK_DEF
C     ==--------------------------------------------------------------==
C     == Block data: init variables for the wavefunction stack        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'lscal.inc'
      INTEGER I
C     ==--------------------------------------------------------------==
      DO I = 1,NSTACK
         IOPNXT(I) = ST_NOP ! Do nothing by default
         ISZSTK(I) =  1     ! The stacks are one entry deep
         ISTLEN(I) =  0     ! No default entry size
         ITPSTK(I) = -1     ! Stacks not initialized yet
      END DO
C     ==--------------------------------------------------------------==
      END
C     ==================================================================
      SUBROUTINE GIVE_WORK_LBFGS (NDIM, LRLBFGS)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'lscal.inc'
C     Arguments
      INTEGER NDIM, LRLBFGS
C     ==--------------------------------------------------------------==
      IF (NREM.EQ.0) THEN
         LRLBFGS = 2*NDIM + 2*MAX(40,NDIM) + 2*NDIM*MAX(40,NDIM)
      ELSE
         LRLBFGS = 2*NDIM + 2*NREM + 2*NDIM*NREM
      END IF
      LWLBFGS = LRLBFGS ! Store for straightforward writing of RESTART
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_RLBFGS (LRLBFGS, TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !PARENT MGDIIS
      INCLUDE 'cotr.inc'        !NODIM
C     Arguments
      INTEGER LRLBFGS
      CHARACTER*30 TAG
C     ==--------------------------------------------------------------==
      LRLBFGS = 0
      TAG = '0'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSCAL_INIT
C     ==--------------------------------------------------------------==
C     == Block data: init variables for L-BFGS, P-RFO, MI, and stack  ==
C     ==--------------------------------------------------------------==
      CALL TOL_INIT_SHARED
      CALL LBFGS_INIT
      CALL PRFO_INIT
      CALL STACK_DEF
C     ==--------------------------------------------------------------==
      END
C     ==================================================================
      SUBROUTINE LBFGS_INIT
C     ==--------------------------------------------------------------==
C     == Block data: init variables for L-BFGS                        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'lscal.inc'
C     ==--------------------------------------------------------------==
      NREM       = 0         ! Number of remembered steps
      NTRUST     = 0         ! HDLCopt trust radius algorithm
      NRESTT     = 0         ! Periodic restart
      NTRSTR     = 0         ! Trust radius then line search
      JUMP       = 0         ! Optimizer status (next action)
      LINILS     = .TRUE.    ! Line search not yet initialized
      LWLBFGS    = 0         ! WLBFGS not yet allocated
      NRES_L     = 0         ! Number of resets
      NSTEP_L    = 0         ! Number of performed L-BFGS steps
      NSTRIED_L  = 0         ! Number of tried L-BFGS steps
      STEP_INI_L = 0.5D+0    ! Initial trust radius
      STEP_MIN_L = TOLNG     ! Minimum trust radius for reset
      STEP_BMB_L = 1.D-2*TOLNG    ! Minimum trust radius for error
      STEP_MAX_L = 0.5D+0    ! Maximum trust radius
      TRUSTR     = 0.0D0     ! For non-default trust radius (input)
      OLDENE     = 0.0D0     ! For adaptive tol. of other optim.
      LFEVMN     = .FALSE.   ! Line search status ok
C     ==--------------------------------------------------------------==
      END
C     ==================================================================
      SUBROUTINE WR30LBFGS (NW, IERROR, LSAVC0)
C     ==--------------------------------------------------------------==
C     ==  Writes L-BFGS optimizer status and history to the RESTART   ==
C     ==  file NW. Limits and other status variables come first to    ==
C     ==  enable reading program allocating the memory required to    ==
C     ==  hold the information.                                       ==
C     ==  Change this routine consistently with                       ==
C     ==  lscal.inc and RD30LBFGS                                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'lscal.inc'
C     Arguments
      INTEGER NW, IERROR
      LOGICAL LSAVC0
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
C     Input parameters (common blocks /LSCALS/ and /LSCALO/)
      WRITE (UNIT=NW,IOSTAT=IERROR)
     &     NREM, NTRUST, NRESTT, NTRSTR
      WRITE (UNIT=NW,IOSTAT=IERROR)
     &     STEP_BMB_L, STEP_INI_L, STEP_MAX_L, STEP_MIN_L, TRUSTR
C     Shared information (common blocks /LSCALL/, /LSCALI/ and /LSCALR/)
      WRITE (UNIT=NW,IOSTAT=IERROR) LFEVMN, LINILS
      WRITE (UNIT=NW,IOSTAT=IERROR)
     &     IBND_L, IGPNT, INFO_L, IPNT_L, IPRNT, ISPNT,
     $     ITER_L, IWPNT, JUMP, LWLBFGS, NFEV_L,
     $     NFEVMN, NRES_L, NSTEP_L, NSTRIED_L, NPT_L
      WRITE (UNIT=NW,IOSTAT=IERROR)
     &     DEACCP, ETOTMN, OLDENE, SCL1_L, SCL2_L, STEP_L, SZ_L, SZ_LMN
C     L-BFGS history
      WRITE (UNIT=NW,IOSTAT=IERROR) (WLBFGS(I), I=1,LWLBFGS)
C     Electronic state if requested
      IF (LSAVC0) THEN
         WRITE (UNIT=NW,IOSTAT=IERROR) NSTACK
      ELSE
         WRITE (UNIT=NW,IOSTAT=IERROR) 0
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE RD30LBFGS (NR, IRECORD)
C     ==--------------------------------------------------------------==
C     ==  Restart from checkpoint. See WR30LBFGS                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'lscal.inc'
C     Arguments
      INTEGER NR, IRECORD
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
C     Input parameters (common blocks /LSCALS/ and /LSCALO/)
      READ (NR) NREM, NTRUST, NRESTT, NTRSTR
      READ (NR) STEP_BMB_L, STEP_INI_L, STEP_MAX_L, STEP_MIN_L, TRUSTR
C     Shared information (common blocks /LSCALL/, /LSCALI/ and /LSCALR/)
      READ (NR) LFEVMN, LINILS
      READ (NR) IBND_L, IGPNT, INFO_L, IPNT_L, IPRNT, ISPNT,
     $           ITER_L, IWPNT, JUMP, LWLBFGS, NFEV_L,
     $           NFEVMN, NRES_L, NSTEP_L, NSTRIED_L, NPT_L
      READ (NR) DEACCP, ETOTMN, OLDENE, SCL1_L, SCL2_L, STEP_L,
     $           SZ_L, SZ_LMN
C     L-BFGS history
      CALL MEMORY (IP_WLBFGS, LWLBFGS, 'WLBFGS')
      READ (NR) (WLBFGS(I), I=1,LWLBFGS)
C     Electronic state if stored
      READ (NR) I
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE LSRCH (NDIM, POINT, ENRG, GRAD, DIRCTN,
     $                  STEP, MAXSTEP, INFO, NSTEP, WORK)
C     ==--------------------------------------------------------------==
C     ==  This is a common driver for line search routines.           ==
C     ==  Hard-coded parameter ILSTYP selects the routine doing the   ==
C     ==  work:                                                       ==
C     ==  1. CPMD routine MINLIN in sdion.F. This is a very simple    ==
C     ==     golden section bracketing routine.                       ==
C     ==     This option is not very optimized and most likely will   ==
C     ==     not improve the performance over the educated guess of   ==
C     ==     the trust radius scheme.                                 ==
C     ==  2. The routine used in Nocedal reference implementation     ==
C     ==     of L-BFGS based on a routine by More and Thuente at      ==
C     ==     Argonne. This routine finds a step which satisfies the   ==
C     ==     Wolfe criteria for sufficient decrease and curvature.    ==
C     ==     Please obtain the routine and a license from J. Nocedal  ==
C     ==     directly at http://www.ece.nwu.edu/~nocedal/             ==
C     ==                                                              ==
C     ==  Arguments:                                                  ==
C     ==                                                              ==
C     ==  Return code INFO:                                           ==
C     ==  -1    Calculate energy / gradient of the predicted step     ==
C     ==   0    Error                                                 ==
C     ==   1    Found an acceptable step                              ==
C     ==   3    Maximum number of line search cycles reached          ==
C     ==  other Error                                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT  NONE
C     Arguments
      INTEGER   NDIM, MAXSTEP, INFO, NSTEP
      REAL*8    POINT(NDIM), ENRG, GRAD(NDIM), DIRCTN(NDIM), STEP,
     $          WORK(NDIM)
C     Variables
      LOGICAL   LFIRST
      INTEGER   ILSTYP, INFOM, NSTEPM
      REAL*8    FTOL, OLDSTP, XTOL, SCALAR
      REAL*8    STEPS(MAXSTEP), ENRGS(MAXSTEP), DNRGS(MAXSTEP)
      POINTER   (IP_STEPS,STEPS), (IP_ENRGS,ENRGS), (IP_DNRGS,DNRGS)
      EXTERNAL  SCALAR
      SAVE      LFIRST, NSTEPM, IP_STEPS, IP_ENRGS, IP_DNRGS
C     Select routine
      PARAMETER (ILSTYP = 1)
      DATA      LFIRST / .TRUE. /
C     ==--------------------------------------------------------------==
      IF (ILSTYP .EQ. 1) THEN
         IF (LFIRST) THEN
            WRITE (6,*) 'L-BFGS WARNING: you are using the built-in ',
     $           'golden section line'
            WRITE (6,*) '  search, see rlbfgs.F for more information'
            CALL MEMORY (IP_STEPS, MAXSTEP, 'L_STEPS')
            CALL MEMORY (IP_ENRGS, MAXSTEP, 'L_ENRGS')
            CALL MEMORY (IP_DNRGS, MAXSTEP, 'L_DNRGS')
            NSTEPM = 0
            LFIRST = .FALSE.
         END IF
         IF (INFO .EQ. 0) THEN
            NSTEPM = 0          ! driver forced new search direction
         END IF
         IF (NSTEPM .EQ. 0) THEN
            NSTEP  = 0
            OLDSTP = 0.0D0
         ELSE
            NSTEP = NSTEP + 1
            OLDSTP = STEP
         END IF
         NSTEPM = NSTEPM + 1
         ENRGS(NSTEPM) = ENRG
         DNRGS(NSTEPM) = SCALAR (NDIM, GRAD, DIRCTN)
         IF (NSTEP.EQ.0 .AND. DNRGS(NSTEPM).GE.0.0D0) THEN
            WRITE (6,*) '    WARNING: INVERTING L-BFGS SEARCH DIRECTION'
            CALL DSCAL (NDIM, -1.0D0, DIRCTN, 1)
            CALL DSCAL (NSTEPM, -1.0D0, DNRGS, 1)
         END IF
         IF (NSTEPM .EQ. 1) THEN
            STEPS(NSTEPM) = 0.0D0
         ELSE
            STEPS(NSTEPM) = STEP
         END IF
         CALL MINLIN (NSTEPM, STEPS, ENRGS, DNRGS, STEP, INFOM)
         IF (INFOM.EQ.2) THEN
            INFO = -1
            STEP = 0.1D0*STEP   ! quadratic approximation invalid
            CALL DAXPY (NDIM, STEP-OLDSTP, DIRCTN, 1, POINT, 1)
            WRITE (6,*) 'LSRCH WARNING: INTERVAL TOO LARGE, DOWNSIZING',
     $           ' MAXIMUM STEP LENGTH'
         ELSE IF (INFOM.EQ.1 .AND. NSTEP.GT.1) THEN
            INFO = 1            ! accept the step
            NSTEPM = 0
         ELSE IF (NSTEP.GE.MAXSTEP) THEN
c           INFO = 3            ! too many line search cycles, would be
            INFO = 1            ! INFO=3 but golden section -> low anyway
            NSTEPM = 0
         ELSE IF (INFOM.EQ.0 .OR. NSTEP.LE.1) THEN
            INFO = -1           ! more line search cycles required
            CALL DAXPY (NDIM, STEP-OLDSTP, DIRCTN, 1, POINT, 1)
         ELSE
            INFO = 0            ! error
         END IF
      ELSE IF (ILSTYP .EQ. 2) THEN
         XTOL = 1.0D-16         ! machine precision
         FTOL = 1.0D-4
         CALL MCSRCH (NDIM, POINT, ENRG, GRAD, DIRCTN,
     $        STEP, FTOL, XTOL, MAXSTEP, INFO, NSTEP, WORK)
      ELSE
         CALL STOPGM ('WRONG LINE SEARCH ROUTINE', 'CHECK RLBFGS.F')
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MCSRCH (NDIM, POINT, ENRG, GRAD, DIRCTN,
     $           STEP, FTOL, XTOL, MAXSTEP, INFO, NSTEP, WORK)
C     ==--------------------------------------------------------------==
C     ==  Please rename or remove this stub if you have obtained the  ==
C     ==  license for routine MCSRCH from J. Nocedal                  ==
C     ==  (http://www.ece.nwu.edu/~nocedal/)                          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER NDIM, MAXSTEP, INFO, NSTEP
      REAL*8  POINT(NDIM), ENRG, GRAD(NDIM), DIRCTN(NDIM), STEP, FTOL,
     $        XTOL, WORK(NDIM)
C     ==--------------------------------------------------------------==
C     In the obtained MCSRCH, you can change these lines:
C     DGINIT = ZERO
C     DO 10 J = 1, N
C        DGINIT = DGINIT + G(J)*S(J)
C  10    CONTINUE
C     IF (DGINIT .GE. ZERO) THEN
C        WRITE(LP,15)
C  15    FORMAT(/'  THE SEARCH DIRECTION IS NOT A DESCENT DIRECTION')
C        RETURN
C        ENDIF
C
C     into the lines:
C     DGINIT = ZERO
C     DO 10 J = 1, N
C        DGINIT = DGINIT + G(J)*S(J)
C  10    CONTINUE
C     IF (DGINIT .GE. ZERO) THEN
C        WRITE(6,15)
C  15    FORMAT('     WARNING: INVERTING L-BFGS SEARCH DIRECTION')
C        DO 16 J = 1, N
C           S(J) = -S(J)
C 16     CONTINUE
C        DGINIT = -DGINIT
C     ENDIF
C     ==--------------------------------------------------------------==
      CALL STOPGM ('MCSRCH NOT INCLUDED', 'CHECK RLBFGS.F (LSRCH)')
      END      
C     ==================================================================
