C     ==================================================================
      SUBROUTINE RNLRH(ENL,NSTATE,NKPOINT)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==     THE NON-LOCAL POTENTIAL CONTRIBUTION TO THE TOTAL        ==
C     ==     ENERGY, I.E. ENL                                         ==
C     ==     K-POINTS IMPLEMENTED (FNL IS COMPLEX -> IMAGP=2)         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sgpp.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'spin.inc'
C     Arguments
      INTEGER   NSTATE,NKPOINT
      REAL*8    ENL
C     Variables
      DIMENSION F(NSTATE,NKPOINT),WK(NKPOINT),
     &          FNL2(IMAGP,NAT,NHXS,LDF2,NKPOINT)
      REAL*8    FNL(IMAGP,NAT,NHXS,NSTATE,NKPOINT),DFNL(*)
      INTEGER   ISUB,ISA0,IS,IV,JV,I,ISA,IA,L,LI,KI,L2,LJ,KJ,IK,II
      REAL*8    WEIGHT,SUM
C     ==--------------------------------------------------------------==
      ENL=0.D0
C     If no non-local components -> return.
      IF(NLM.EQ.0) RETURN
C     ==--------------------------------------------------------------==
      CALL TISET('     RNLRH',ISUB)
C     ==--------------------------------------------------------------==
C     == Compute the non-local contribution to the total energy (ENL) ==
C     ==--------------------------------------------------------------==
      DO IK=1,NKPOINT
        ISA0=0
        DO IS=1,NSP
          IF(TVAN(IS)) THEN
C           VANDERBILT PSEUDOPOTENTIAL
            IF(IMAGP.EQ.2)
     &           CALL STOPGM('RNLRH','K-POINTS NOT IMPLEMENTED')
            DO IV=1,NGH(IS)
              DO JV=1,NGH(IS)
                SUM=0.0D0
C               Remember: NST12(MEPOS,1)=1
C               ST12(MEPOS,2)=NSTATE for serial jobs
!$OMP parallel do private(I,IA,ISA,II) reduction(+:SUM)
#ifdef __SR8000
*poption parallel, tlocal(I,IA,ISA,II), psum(SUM)
#endif 
                DO I=NST12(MEPOS,1),NST12(MEPOS,2)
                  II=I-NST12(MEPOS,1)+1
                  IF(TFDIST) THEN
                    DO IA=1,NA(IS)
                      ISA=ISA0+IA
                      SUM=SUM+F(I,IK)*FNL2(1,ISA,IV,II,IK)*
     &                                FNL2(1,ISA,JV,II,IK)
                    ENDDO
                  ELSE
                    DO IA=1,NA(IS)
                      ISA=ISA0+IA
                      SUM=SUM+F(I,IK)*FNL2(1,ISA,IV,I,IK)*
     &                                FNL2(1,ISA,JV,I,IK)
                    ENDDO
                  ENDIF
                ENDDO
                ENL=ENL+DVAN(JV,IV,IS)*SUM
              ENDDO
            ENDDO
          ELSEIF(TSGP(IS)) THEN
C           Stefan Goedecker pp
            DO IV=1,NGH(IS)
              L=NGHTOL(IV,IS)+1
              KI=LFVAL(IV,IS)
              LI=LPVAL(IV,IS)
              DO JV=IV,NGH(IS)
                L2=NGHTOL(JV,IS)+1
                LJ=LPVAL(JV,IS)
                IF(L2.EQ.L.AND.LI.EQ.LJ) THEN
                  KJ=LFVAL(JV,IS)
                  SUM=0.0D0
!$OMP parallel do private(I,IA,ISA,WEIGHT,II) reduction(+:SUM)
#ifdef __SR8000
*poption parallel, tlocal(I,IA,ISA,WEIGHT,II), psum(SUM)
#endif 
                  DO I=NST12(MEPOS,1),NST12(MEPOS,2)
                    WEIGHT=WK(IK)*F(I,IK)
                    IF(WEIGHT.EQ.0.D0) GOTO 2000
                    II=I-NST12(MEPOS,1)+1
                    IF(IMAGP.EQ.1) THEN
                      IF(TFDIST) THEN
                        DO IA=1,NA(IS)
                          ISA=ISA0+IA
                          SUM=SUM+WEIGHT*FNL2(1,ISA,IV,II,IK)*
     *                                   FNL2(1,ISA,JV,II,IK)
                        ENDDO
                      ELSE
                        DO IA=1,NA(IS)
                          ISA=ISA0+IA
                          SUM=SUM+WEIGHT*FNL2(1,ISA,IV,I,IK)*
     *                                   FNL2(1,ISA,JV,I,IK)
                        ENDDO
                      ENDIF
                    ELSE
                      IF(TFDIST) THEN
                        DO IA=1,NA(IS)
                          ISA=ISA0+IA
                          SUM=SUM+WEIGHT*
     *                      (FNL2(1,ISA,IV,II,IK)*FNL2(1,ISA,JV,II,IK)
     *                      +FNL2(2,ISA,IV,II,IK)*FNL2(2,ISA,JV,II,IK))
                        ENDDO
                      ELSE
                        DO IA=1,NA(IS)
                          ISA=ISA0+IA
                          SUM=SUM+WEIGHT*
     *                      (FNL2(1,ISA,IV,I,IK)*FNL2(1,ISA,JV,I,IK)
     *                      +FNL2(2,ISA,IV,I,IK)*FNL2(2,ISA,JV,I,IK))
                        ENDDO
                      ENDIF
                    ENDIF
 2000               CONTINUE
                  ENDDO
                  IF(IV.NE.JV) SUM=2.D0*SUM
                  ENL=ENL+SUM*HLSG(KI,KJ,L,IS)
                ENDIF
              ENDDO
            ENDDO
          ELSE
C           BHS AND RELATED 
            DO IV=1,NGH(IS)
              SUM=0.0D0
!$OMP parallel do private(I,IA,II,ISA) reduction(+:SUM)
#ifdef __SR8000
*poption parallel, tlocal(I,IA,II,ISA), psum(SUM)
#endif 
              DO I=NST12(MEPOS,1),NST12(MEPOS,2)
                II=I-NST12(MEPOS,1)+1
                IF(IMAGP.EQ.1) THEN
                  IF(TFDIST) THEN
                    DO IA=1,NA(IS)
                      ISA=ISA0+IA
                      SUM=SUM+F(I,IK)*WK(IK)*
     *                      FNL2(1,ISA,IV,II,IK)*FNL2(1,ISA,IV,II,IK)
                    ENDDO
                  ELSE
                    DO IA=1,NA(IS)
                      ISA=ISA0+IA
                      SUM=SUM+F(I,IK)*WK(IK)*
     *                      FNL2(1,ISA,IV,I,IK)*FNL2(1,ISA,IV,I,IK)
                    ENDDO
                  ENDIF
                ELSE
                  IF(TFDIST) THEN
                    DO IA=1,NA(IS)
                      ISA=ISA0+IA
                      SUM=SUM+F(I,IK)*WK(IK)*
     *                     (FNL2(1,ISA,IV,II,IK)*FNL2(1,ISA,IV,II,IK)
     *                     +FNL2(2,ISA,IV,II,IK)*FNL2(2,ISA,IV,II,IK))
                    ENDDO
                  ELSE
                    DO IA=1,NA(IS)
                      ISA=ISA0+IA
                      SUM=SUM+F(I,IK)*WK(IK)*
     *                     (FNL2(1,ISA,IV,I,IK)*FNL2(1,ISA,IV,I,IK)
     *                     +FNL2(2,ISA,IV,I,IK)*FNL2(2,ISA,IV,I,IK))
                    ENDDO
                  ENDIF
                ENDIF
              ENDDO
              ENL=ENL+WSG(IS,IV)*SUM
            ENDDO
          ENDIF
          ISA0=ISA0+NA(IS)
        ENDDO
      ENDDO
      IF(TLSE .AND. TCAS22) CALL RNLCAS
      CALL TIHALT('     RNLRH',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE RNLCAS
C     ==--------------------------------------------------------------==
C     == Calculates NL PP contribution to CAS22 Energies              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sgpp.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'spin.inc'
C     ==--------------------------------------------------------------==
      DIMENSION F(*),FNL2(*)
      REAL*8    FNL(LDF1,NHXS,*),DFNL(*),SM
      INTEGER   IAT,IA,IS,IV,JV,IATL,L,LI,KI,L2,LJ,KJ
C     ==--------------------------------------------------------------==
      ENL_AB = 0.D0
      ENL_A  = ENL
      DO IAT=IPEPT(1,MEPOS),IPEPT(2,MEPOS)
        IA=IATPT(1,IAT)
        IS=IATPT(2,IAT)
        IF(TVAN(IS)) THEN
C         VANDERBILT PSEUDOPOTENTIAL
        ELSEIF(TSGP(IS)) THEN
C         Stefan Goedecker pp
          DO IV=1,NGH(IS)
            L=NGHTOL(IV,IS)+1
            LI=LPVAL(IV,IS)
            KI=LFVAL(IV,IS)
            DO JV=IV,NGH(IS)
              IF(IV.EQ.JV) THEN
                SM=1.D0
              ELSE
                SM=2.D0
              ENDIF
              L2=NGHTOL(JV,IS)+1
              LJ=LPVAL(JV,IS)
              IF(L2.EQ.L.AND.LI.EQ.LJ) THEN
                KJ=LFVAL(JV,IS)
                IF(TFDIST) THEN
                  IATL=IAT-IPEPT(1,MEPOS)+1
                ELSE
                  IATL=IAT
                ENDIF
                ENL_AB=ENL_AB+HLSG(KI,KJ,L,IS)*0.5D0*SM*
     &               (FNL(IATL,IV,IALPHA)*FNL(IATL,JV,IBETA)+
     &                FNL(IATL,JV,IALPHA)*FNL(IATL,IV,IBETA))
                ENL_A=ENL_A+HLSG(KI,KJ,L,IS)*SM*
     &               (FNL(IATL,IV,IALPHA)*FNL(IATL,JV,IALPHA)-
     &                FNL(IATL,IV,IBETA)*FNL(IATL,JV,IBETA))
              ENDIF
            ENDDO
          ENDDO
        ELSE
C         BHS AND RELATED
          DO IV=1,NGH(IS)
            IF(TFDIST) THEN
              IATL=IAT-IPEPT(1,MEPOS)+1
            ELSE
              IATL=IAT
            ENDIF
            ENL_AB=ENL_AB+WSG(IS,IV)*FNL(IATL,IV,IALPHA)*
     &                               FNL(IATL,IV,IBETA)
            ENL_A=ENL_A+WSG(IS,IV)*(FNL(IATL,IV,IALPHA)**2
     &                             -FNL(IATL,IV,IBETA)**2)
          ENDDO
        ENDIF
      ENDDO
      ENL_2  = ENL - (ENL_A - ENL)
C     ==--------------------------------------------------------------==
      RETURN
      END
CEHR[
C     ==================================================================
      SUBROUTINE RNLRHG(ENL,NSTATE,NKPOINT)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==     THE NON-LOCAL POTENTIAL CONTRIBUTION TO THE TOTAL        ==
C     ==     ENERGY, I.E. ENL                                         ==
C     ==     K-POINTS IMPLEMENTED (FNL IS COMPLEX -> IMAGP=2)         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sgpp.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'spin.inc'
C     Arguments
      INTEGER   NSTATE,NKPOINT
      REAL*8    ENL
C     Variables 
      DIMENSION F(NSTATE,NKPOINT),WK(NKPOINT),
     &          FNL2(IMAGP,NAT,NHXS,LDF2,NKPOINT),
     &          FNLGP(IMAGP,NAT,NHXS,LDF2,NKPOINT)
      REAL*8    FNL(IMAGP,NAT,NHXS,NSTATE,NKPOINT),DFNL(*)
      INTEGER   ISUB,ISA0,IS,IV,JV,I,ISA,IA,L,LI,KI,L2,LJ,KJ,IK,II
      REAL*8    WEIGHT,SUM
C     ==--------------------------------------------------------------==
      ENL=0.D0
C     If no non-local components -> return.
      IF(NLM.EQ.0) RETURN
C     ==--------------------------------------------------------------==
      CALL TISET('     RNLRH',ISUB)
C     ==--------------------------------------------------------------==
C     == Compute the non-local contribution to the total energy (ENL) ==
C     ==--------------------------------------------------------------==
      DO IK=1,NKPOINT
        ISA0=0
        DO IS=1,NSP
C           BHS AND RELATED
            DO IV=1,NGH(IS)
              SUM=0.0D0
              DO I=NST12(MEPOS,1),NST12(MEPOS,2)
                II=I-NST12(MEPOS,1)+1
                IF(IMAGP.EQ.1) THEN
                  IF(TFDIST) THEN
                    DO IA=1,NA(IS)
                      ISA=ISA0+IA
                      SUM=SUM+F(I,IK)*WK(IK)*
     *                      FNL2(1,ISA,IV,II,IK)*FNL2(1,ISA,IV,II,IK)
                    ENDDO
                  ELSE
                    DO IA=1,NA(IS)
                      ISA=ISA0+IA
                      SUM=SUM+F(I,IK)*WK(IK)*
     *                      FNL2(1,ISA,IV,I,IK)*FNL2(1,ISA,IV,I,IK)
                    ENDDO
                  ENDIF
                ELSE
                  IF(TFDIST) THEN
                    DO IA=1,NA(IS)
                      ISA=ISA0+IA
                      SUM=SUM+F(I,IK)*WK(IK)*
     *                     (FNL2(1,ISA,IV,II,IK)*FNL2(1,ISA,IV,II,IK)
     *                     +FNL2(2,ISA,IV,II,IK)*FNL2(2,ISA,IV,II,IK))
                    ENDDO
                  ELSE
                    DO IA=1,NA(IS)
                      ISA=ISA0+IA
                      SUM=SUM+F(I,IK)*WK(IK)* 2.D0 *
     *                   ( (FNL(1,ISA,IV,I,IK)*FNLGP(1,ISA,IV,I,IK)
     *                     +FNL(2,ISA,IV,I,IK)*FNLGP(2,ISA,IV,I,IK))
     *                   )
                    ENDDO
                  ENDIF
                ENDIF
              ENDDO
              ENL=ENL+WSG(IS,IV)*SUM
            ENDDO
          ISA0=ISA0+NA(IS)
        ENDDO
      ENDDO
      IF(TLSE .AND. TCAS22) CALL RNLCAS
      CALL TIHALT('     RNLRH',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
CEHR]

