C     ==================================================================
C     REMOVE ROTATIONAL VELOCITIES. 
C     Last Update: 2008/06/11 by AK
C     To integrate more cleanly with the logic of the velocity verlet
C     algorithm you have to call this subroutine twice. First with 
C     INOUT set to .TRUE., where the routine will store the angular
C     rotation velocity vector in LM(3) and position vectors of the atoms
C     relative to the center of mass in TSCR(3,NAT). This call will remove
C     only half of the velocity and has to be called before the first 
C     (half) velocity update step. For the second call, which has to come
C     after the second (half) velocity update step, with INOUT set to
C     .FALSE. those data is used to remove the remainder of the 
C     rotational motion.
C     ==================================================================
      SUBROUTINE ROTVEL(TAU0,VELP,LM,TSCR,INOUT)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'nose.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'cotr.inc'
C     Arguments
      REAL*8     TAU0(3,NAX,*),VELP(3,NAX,*),LM(3),TSCR(3,*)
      LOGICAL    INOUT  ! true  -> write LM(3)/TSCR(NAT)
                        ! false -> read  LM(3)/TSCR(NAT)
C     Variables
      REAL*8     VSCR(3,NAT),PSCR(NAT)
      REAL*8     CM(3),EVL(3),EVM(3,3)
      REAL*8     LTEN(3,3),WK(9),WX(3),SMALL
      REAL*8     R0,R1,R2,R3,EKINP1,TEMPP1,EKINP2,TEMPP2,PSCRT,
     &    TSCAL,VSCALE,VTMP1,VTMP2,VTMP3
      INTEGER    IS,IA,IAT,IERR
      PARAMETER (SMALL=1.0D-10)  ! small angular momentum where we stop considering it
C     ==--------------------------------------------------------------==
C     COMPUTE THE IONIC TEMPERATURE TEMPP1
      CALL EKINPP(EKINP1,VELP)
      TEMPP1=EKINP1*FACTEM*2.0D0/GLIB

      IF(INOUT) THEN
C     GET CENTER OF MASS AND COPY TO HELPER ARRAY
        CM(1)=0.0D0
        CM(2)=0.0D0
        CM(3)=0.0D0
        PSCRT=0.0D0
!$OMP parallel do private(IAT,IA,IS,R0,R1,R2,R3) reduction(+:PSCRT,CM) 
        DO IAT=1,NAT
          IA=IATPT(1,IAT)
          IS=IATPT(2,IAT)
          R0=PMA0(IS)
          PSCRT=PSCRT+R0
          PSCR(IAT)=R0
          R1=TAU0(1,IA,IS)
          TSCR(1,IAT)=R1
          CM(1)=CM(1)+R1*R0
          R2=TAU0(2,IA,IS)
          TSCR(2,IAT)=R2
          CM(2)=CM(2)+R2*R0
          R3=TAU0(3,IA,IS)
          TSCR(3,IAT)=R3
          CM(3)=CM(3)+R3*R0
          VSCR(1,IAT)=VELP(1,IA,IS)
          VSCR(2,IAT)=VELP(2,IA,IS)
          VSCR(3,IAT)=VELP(3,IA,IS)
        END DO
        CM(1)=CM(1)/PSCRT
        CM(2)=CM(2)/PSCRT
        CM(3)=CM(3)/PSCRT

C     translate C.O.M to origin.
!$OMP parallel do private(IAT)
        DO IAT=1,NAT
          TSCR(1,IAT)=TSCR(1,IAT)-CM(1)
          TSCR(2,IAT)=TSCR(2,IAT)-CM(2)
          TSCR(3,IAT)=TSCR(3,IAT)-CM(3)
        END DO

C     get total angular momentum
        LM(1)=0.0d0
        LM(2)=0.0d0
        LM(3)=0.0d0
!$OMP parallel do private(IAT,VTMP1,VTMP2,VTMP3,R1)
!$OMP+ reduction(+:LM)
        DO IAT=1,NAT
          VTMP1=TSCR(2,IAT)*VSCR(3,IAT)-TSCR(3,IAT)*VSCR(2,IAT)
          VTMP2=TSCR(3,IAT)*VSCR(1,IAT)-TSCR(1,IAT)*VSCR(3,IAT)
          VTMP3=TSCR(1,IAT)*VSCR(2,IAT)-TSCR(2,IAT)*VSCR(1,IAT)
          R1=PSCR(IAT)
          LM(1)=LM(1)+VTMP1*R1
          LM(2)=LM(2)+VTMP2*R1
          LM(3)=LM(3)+VTMP3*R1
        END DO

C     get moment of inertia
        CALL AZZERO(LTEN,9)
        DO IAT=1,NAT
          LTEN(1,1)=LTEN(1,1)+PSCR(IAT)*(TSCR(2,IAT)**2+TSCR(3,IAT)**2)
          LTEN(1,2)=LTEN(1,2)-PSCR(IAT)*(TSCR(1,IAT)*TSCR(2,IAT))
          LTEN(1,3)=LTEN(1,3)-PSCR(IAT)*(TSCR(1,IAT)*TSCR(3,IAT))
          
          LTEN(2,1)=LTEN(2,1)-PSCR(IAT)*(TSCR(1,IAT)*TSCR(2,IAT))
          LTEN(2,2)=LTEN(2,2)+PSCR(IAT)*(TSCR(1,IAT)**2+TSCR(3,IAT)**2)
          LTEN(2,3)=LTEN(2,3)-PSCR(IAT)*(TSCR(2,IAT)*TSCR(3,IAT))
          
          LTEN(3,1)=LTEN(3,1)-PSCR(IAT)*(TSCR(1,IAT)*TSCR(3,IAT))
          LTEN(3,2)=LTEN(3,2)-PSCR(IAT)*(TSCR(2,IAT)*TSCR(3,IAT))
          LTEN(3,3)=LTEN(3,3)+PSCR(IAT)*(TSCR(1,IAT)**2+TSCR(2,IAT)**2)
        END DO

        IERR=0
        CALL DCOPY(9,LTEN,1,EVM,1)
        CALL DSYEV('V','U',3,EVM,3,EVL,WK,9,IERR)

C     get angular velocity in body frame. ignore if moment of inertia is small.
        CALL DGEMV('T',3,3,1.0D0,EVM,3,LM,1,0.0D0,WX,1)
        IF(DABS(EVL(1)).GT.SMALL) THEN
          WX(1)=WX(1)/EVL(1)
        ELSE
          WX(1)=0.0d0
        ENDIF
        IF(DABS(EVL(2)).GT.SMALL) THEN
          WX(2)=WX(2)/EVL(2)
        ELSE
          WX(2)=0.0d0
        ENDIF
        IF(DABS(EVL(3)).GT.SMALL) THEN
          WX(3)=WX(3)/EVL(3)
        ELSE
          WX(3)=0.0d0
        ENDIF
C     DEBUG
C      write(6,'(A,3F12.8,3F12.8)') 'ANG VEL in body axis:', WX, EVL
C     transform back to lab frame and subtract
        CALL DGEMV('N',3,3,1.0D0,EVM,3,WX,1,0.0D0,LM,1)
      ENDIF

!$OMP parallel do private(IS,IA,IAT,VTMP1,VTMP2,VTMP3)
      DO IAT=1,NAT
        IA=IATPT(1,IAT)
        IS=IATPT(2,IAT)
        VTMP1=LM(2)*TSCR(3,IAT)-LM(3)*TSCR(2,IAT)
        VTMP2=LM(3)*TSCR(1,IAT)-LM(1)*TSCR(3,IAT)
        VTMP3=LM(1)*TSCR(2,IAT)-LM(2)*TSCR(1,IAT)
        VELP(1,IA,IS)=VELP(1,IA,IS)-0.5D0*VTMP1
        VELP(2,IA,IS)=VELP(2,IA,IS)-0.5D0*VTMP2
        VELP(3,IA,IS)=VELP(3,IA,IS)-0.5D0*VTMP3
      ENDDO

C     COMPUTE THE IONIC TEMPERATURE AFTER THE REMOVAL
      CALL EKINPP(EKINP2,VELP)
      TEMPP2=EKINP2*FACTEM*2.0D0/GLIB
      
C     rescale velocities to conserve the total energy
      IF(TEMPP2.GT.1.D-5) THEN
        TSCAL=TEMPP1/TEMPP2
        VSCALE=DSQRT(TSCAL)
!$OMP parallel do private(IS,IA,IAT) shared(VSCALE)
        DO IAT=1,NAT
          IA=IATPT(1,IAT)
          IS=IATPT(2,IAT)
          VELP(1,IA,IS)=VELP(1,IA,IS)*VSCALE
          VELP(2,IA,IS)=VELP(2,IA,IS)*VSCALE
          VELP(3,IA,IS)=VELP(3,IA,IS)*VSCALE
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
