C     ==================================================================
      SUBROUTINE RPRFO (XPAR, DXPAR, ETOT, SCR, LSCR,
     $                  LTOLAD, GENVMX, LCVMSK, LPEXIT)
C     ==--------------------------------------------------------------==
C     ==  PARTITIONED RATIONAL FUNCTION OPTIMIZER                     ==
C     ==                                                              ==
C     ==  Reference for classical P-RFO transition state search:      ==
C     ==  A. Banerjee, N. Adams, J. Simons, and R. Shepard,           ==
C     ==    J. Phys. Chem. 89, 1985, 52                               ==
C     ==                                                              ==
C     ==  Reference for microiterative P-RFO / L-BFGS TS search:      ==
C     ==  S.R. Billeter, A.J. Turner, W. Thiel,                       ==
C     ==    Phys. Chem. Chem. Phys.                                   ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   XPAR(NDIM)  Coordinates                                    ==
C     ==   DXPAR(NDIM) Gradient                                       ==
C     ==   NDIM        Number of degrees of freedom                   ==
C     ==   ETOT        Total energy (for trust radius or line search) ==
C     ==   SCR(LSCR)   Scratch space                                  ==
C     == OUTPUT:                                                      ==
C     ==   XPAR(NDIM)  New coordinates                                ==
C     ==   LTOLAD      .FALSE. if full electronic convergence is req. ==
C     ==   GENVMX      Maximum gradient component of the environment  ==
C     ==   LCVMSK      Force the system 'not converged' if .FALSE.    ==
C     ==   LPEXIT      Force the system 'converged' if .TRUE.         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'tpar.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'store.inc' ! print level
      INCLUDE 'lscal.inc' ! parameters and pointers to work space
C     Arguments
      LOGICAL LTOLAD, LCVMSK, LPEXIT
      INTEGER LSCR
      REAL*8  XPAR(NODIM), DXPAR(NODIM), ETOT, SCR(LSCR), GENVMX
C     Externals
      REAL*8 DDOT
      EXTERNAL DDOT
C     Variables
      CHARACTER*30 TAG
      LOGICAL LCONV, LFIRST, LREJ, LRET, LRHE
      INTEGER I, INFO, ISUB, J, LRPRFO, NEGEV, NENV, NZEREV, NVAR2
      REAL*8  DEACT, RATIO
      LOGICAL LFIX
      SAVE    LFIX        ! during energy / gradient calc
C     ==--------------------------------------------------------------==
C     ==  Organization of P-RFO work space                            ==
C     ==                                                              ==
C     ==  The subroutine closely follows the implementation of the    ==
C     ==  HDLC optimizer (reference see above).                       ==
C     ==                                                              ==
C     ==  All arrays are parts of the saved work array WPRFO.         ==
C     ==  The pointers are in /PRFOSP/ in lscal.inc.                  ==
C     ==                                                              ==
C     ==  Name   Size        Meaning                                  ==
C     ==  ----   -----       -------                                  ==
C     ==  HESSCR (NVAR,NVAR) Current Hessian of the core              ==
C     ==  OLDHSS (NVAR,NVAR) Previous Hessian matrix                  ==
C     ==  VMODE  (NVAR)      Mode to be followed                      ==
C     ==  GX     (NVAR)      Gradient in Hessian eigenspace           ==
C     ==  OLDGX  (NVAR)      Previous GX                              ==
C     ==  OLDG   (NVAR)      Previous gradient                        ==
C     ==  OOLDG  (NVAR)      Old previous gradient                    ==
C     ==  EIGVAL (NVAR)      Current Hessian eigenvalues              ==
C     ==  OLDEIG (NVAR)      Previous Hessian eigenvalues             ==
C     ==  EIGVEC (NVAR,NVAR) Current Hessian eigenvectors             ==
C     ==  OLDEV  (NVAR,NVAR) Previous Hessian eigenvectors            ==
C     ==  STEP   (NVAR)      Performed step                           ==
C     ==--------------------------------------------------------------==
      INTEGER IHESSC, IOLDHS, IVMODE, IGX,    IOLDGX, IOLDG,  IOOLDG,
     $        IEIGVL, IOLDEG, IEIGVC, IOLDEV, ISTEP
C     ==--------------------------------------------------------------==
C     ==  Organization of P-RFO scratch space (NENV=NODIM-NVAR)       ==
C     ==                                                              ==
C     ==  Name   Size        Meaning                                  ==
C     ==  ----   -----       -------                                  ==
C     ==  COORD  (NVAR)      Coordinates of the core                  ==
C     ==  GRAD   (NVAR)      Gradient of the core                     ==
C     ==  VARS   (NENV)      Coordinates of the environment           ==
C     ==  DVARS  (NENV)      Gradient of the environment              ==
C     ==  S      (NVAR)      Work array for Hessian update            ==
C     ==  T      (NVAR)      Work array for Hessian update            ==
C     ==  WDIAG  (3*NVAR)    Work array for diagonalization (overlap  ==
C     ==                     with S and T) - could be larger          ==
C     ==--------------------------------------------------------------==
      REAL*8  COORD(NVAR),      GRAD(NVAR)
      POINTER (IP_COORD,COORD), (IP_GRAD,GRAD)
      REAL*8  VARS(NODIM-NVAR), DVARS(NODIM-NVAR)
      POINTER (IP_VARS,VARS),   (IP_DVARS,DVARS)
      REAL*8  S(NVAR),  T(NVAR),  WDIAG(3*NVAR)
      POINTER (IP_S,S), (IP_T,T), (IP_WDIAG,WDIAG)
C     ==--------------------------------------------------------------==
      CALL TISET('     RPRFO',ISUB)
C     ==--------------------------------------------------------------==
C     ==  These tests are performed every time                        ==
C     ==  See also lscal.inc                                          ==
C     ==--------------------------------------------------------------==
      IPRNT = IPRINT(IPRINT_LSCAL)
      LFIRST = (NSTRIED_P.EQ.0 .AND. NSHESS.EQ.0 .AND. NSTRIED_L.EQ.0)
      LPEXIT = .FALSE.
      IF (LFIRST .AND. IPRNT.GE.1) THEN
         WRITE (6,*) 'INITIALIZING TRANSITION STATE SEARCH'
      END IF
      IF (NVAR.EQ.0) THEN
         NVAR = NODIM
         IF (IPRNT.GE.1) THEN
            WRITE (6,*) 'NO REACTION CORE DEFINED, CORE SPANS SYSTEM'
            WRITE (6,*) '  (ALL ', NVAR, ' DEGREES OF FREEDOM)'
         END IF
      END IF
      LMICRO = (NVAR.LT.NODIM)
      IF (LFIRST .AND. IPRNT.GE.1) THEN
         IF (LMICRO) THEN
            WRITE (6,*) '  MICROITERATIVE TRANSITION STATE SEARCH'
         ELSE
            WRITE (6,*) '  CONVENTIONAL TRANSITION STATE SEARCH'
         END IF
      END IF
      IF (LMICRO) THEN
         IF (TOLENV.EQ.0.0D0) THEN
            TOLENV = TOLNG / 3.0D0
            IF (IPRNT.GE.1) THEN
              WRITE (6,'(3X,A,F10.6)')
     $           'SETTING DEFAULT TOLERANCE FOR ENVIRONMENT: ', TOLENV
            ENDIF
         END IF
      END IF
      IF (TRUSTP.NE.0.0D0) THEN
         STEP_INI_P = TRUSTP * 0.2D0
         STEP_MAX_P = TRUSTP
      END IF
      IF (LFIRST .AND. IPRNT.GE.1) THEN
         WRITE (6,*) 'P-RFO INPUT OPTIONS:'
         WRITE (6,'(3X,A,I6)') 'NVAR:   ', NVAR
         WRITE (6,'(3X,A,I6)') 'MODE:   ', MODE
         WRITE (6,'(3X,A,I6)') 'NSMAXP: ', NSMAXP
         IF (MODELK.EQ.1) THEN
            WRITE (6,'(3X,A)') 'MODE IS LOCKED FOR TS SEARCH'
         ELSE
            WRITE (6,'(3X,A)') 'MODE FOLLOWING FOR TS SEARCH SELECTED'
         END IF
         IF (LPRHES) THEN
            WRITE (6,*) 'R/T MODES ARE PROJECTED OUT OF HESSIAN'
            IF (LMICRO) WRITE (6,'(A,A)') ' WARNING! THIS OPTION IS ',
     $           'IGNORED BECAUSE OF PARTITIONED SYSTEM'
         ELSE
            WRITE (6,*) 'R/T MODES ARE NOT PROJECTED OUT OF HESSIAN'
            IF (.NOT.LMICRO) WRITE (6,'(A,A)') ' WARNING! BE CAREFUL ',
     $           'SELECTING THE EIGENMODE'
         END IF
         IF (M_HESS.EQ.1)
     $        WRITE (6,'(3X,A)') 'HESSIAN FROM CPMD IS USED'
         WRITE (6,*)
         IF (TRUSTP.EQ.0.0D0) THEN
            WRITE (6,*) 'P-RFO COMPILED-IN TRUST RADIUS SETTINGS:'
         ELSE
            WRITE (6,*) 'P-RFO TRUST RADIUS SETTINGS BASED ON INPUT:'
         END IF
         WRITE (6,'(3X,A,F12.8)') 'STEP_INI_P: ', STEP_INI_P
         WRITE (6,'(3X,A,F12.8)') 'STEP_MIN_P: ', STEP_MIN_P
         WRITE (6,'(3X,A,F12.8)') 'STEP_MAX_P: ', STEP_MAX_P
         WRITE (6,*)
         WRITE (6,*) 'P-RFO STEP ACCEPTANCE CRITERIA:'
         WRITE (6,'(3X,A,F12.8)') 'MINIMUM OVERLAP BETWEEN MODES:   ',
     $        OMIN
         WRITE (6,'(3X,A,F12.8)') 'MAXIMUM ENERGY CHANGE OF A STEP: ',
     $        DEMIN_P
         WRITE (6,'(3X,A,F12.8)') 'ENERGY CHANGE, MINIMUM RATIO:    ',
     $        RMIN_P
         WRITE (6,'(3X,A,F12.8)') 'ENERGY CHANGE, MAXIMUM RATIO:    ',
     $        RMAX_P
         WRITE (6,*)
      END IF
C     ==--------------------------------------------------------------==
C     ==  Test and allocate scratch space for PRFO and UPDHES         ==
C     ==--------------------------------------------------------------==
      NVAR2 = NVAR*NVAR
      NENV  = NODIM-NVAR
      CALL GIVE_SCR_RPRFO (LRPRFO, TAG)
      CALL TEST_SCR ('RPRFO', TAG, LSCR, LRPRFO)
      CALL GET_ADDR(IP_COORD,SCR(1))
      CALL GET_ADDR(IP_GRAD,SCR(1+NVAR))
      I        = NVAR + NVAR
      CALL GET_ADDR(IP_VARS,SCR(1+I))
      CALL GET_ADDR(IP_DVARS,SCR(1+I+NENV))
      I        = I + NENV + NENV
      CALL GET_ADDR(IP_S,SCR(1+I))
      CALL GET_ADDR(IP_T,SCR(1+I+NVAR))
      CALL GET_ADDR(IP_WDIAG,S(1))
C     ==--------------------------------------------------------------==
C     ==  Splice the coordinates between core and environment         ==
C     ==  No splicing needs to be done when returning here            ==
C     ==--------------------------------------------------------------==
      IF (JUMP.GT.0) CALL CRDCO (XPAR, COORD, VARS, DXPAR, GRAD, DVARS)
 10   CONTINUE
C     ==--------------------------------------------------------------==
C     ==  Initialization  / Reset                                     ==
C     ==  JUMP = 0 / -1                                               ==
C     ==--------------------------------------------------------------==
      IF (JUMP.LE.0) THEN
         IF (IPRNT.GE.1) WRITE (6,*) '  JUMP=0, INITIALIZING P-RFO'
      ELSE IF (LRSTRT) THEN
         IF (IPRNT.GE.1) WRITE (6,'(3X,A,I1,A)') 'JUMP=', JUMP,
     $        ', RESUMING FROM RESTART FILE'
      END IF
C     ==  ALLOCATE WORK SPACE                                         ==
      IF (JUMP.LE.0 .OR. LRSTRT) THEN
         IF (LALLHS) THEN       ! Hessian not yet allocated
            IHESSC = 1
            IOLDHS = IHESSC + NVAR2
            CALL GET_ADDR(IP_HESSCR,WPRFO(IHESSC))
            LALLHS = .FALSE.
         ELSE                   ! Hessian from RESTART file
            IOLDHS = 1
         END IF
         IVMODE = IOLDHS + NVAR2
         IGX    = IVMODE + NVAR
         IOLDGX = IGX    + NVAR
         IOLDG  = IOLDGX + NVAR
         IOOLDG = IOLDG  + NVAR
         IEIGVL = IOOLDG + NVAR
         IOLDEG = IEIGVL + NVAR
         IEIGVC = IOLDEG + NVAR
         IOLDEV = IEIGVC + NVAR2
         ISTEP  = IOLDEV + NVAR2
         CALL GET_ADDR(IP_OLDHSS,WPRFO(IOLDHS))
         CALL GET_ADDR(IP_VMODE,WPRFO(IVMODE))
         CALL GET_ADDR(IP_GX,WPRFO(IGX))
         CALL GET_ADDR(IP_OLDGX,WPRFO(IOLDGX))
         CALL GET_ADDR(IP_OLDG,WPRFO(IOLDG))
         CALL GET_ADDR(IP_OOLDG,WPRFO(IOOLDG))
         CALL GET_ADDR(IP_EIGVAL,WPRFO(IEIGVL))
         CALL GET_ADDR(IP_OLDEIG,WPRFO(IOLDEG))
         CALL GET_ADDR(IP_EIGVEC,WPRFO(IEIGVC))
         CALL GET_ADDR(IP_OLDEV,WPRFO(IOLDEV))
         CALL GET_ADDR(IP_STEP,WPRFO(ISTEP))
      END IF                       ! IF (JUMP.LE.0 .OR. LRSTRT)
C     ==  INITIALIZE FLAGS RETURNED TO RGMOPT IF RESTART              ==
      IF (LRSTRT) THEN
         IF (JUMP.GE.5 .AND. JUMP.LE.7) THEN
            LTOLAD = .FALSE.
         ELSE
            LTOLAD = .TRUE.
         END IF
         LRSTRT = .FALSE.
      END IF                       ! IF (LRSTRT)
C     ==  DONE ALLOCATING WORK SPACE                                  ==
      IF (JUMP.LE.0) THEN
         LLOCK = (MODELK.EQ.1)     ! Mode locking if LLOCK
         STEP_P = STEP_INI_P
C     ==  PREPARE MICROITERATIVE OPTIMIZATION                         ==
         IF (LMICRO) THEN
            IF (.NOT.LNOMAP .AND. LMAP_P.EQ.0) THEN ! Mapping to core
               CALL MEMORY (IP_MAP_P, NODIM, 'MAP_P')
               LMAP_P = NODIM
               CALL CLCMAP (MAP_P, NODIM, NVAR, ICORE, NCORE)
               CALL FREEM (IP_ICORE)
            END IF
            JUMP = 0               ! Initialize L-BFGS for environment
            LCVMSK = .FALSE.       ! CNVENV tells if we are converged
            LRET = .TRUE.
            CALL RLBFGS (VARS, DVARS, NENV, ETOT, LRET)
            JUMP = 2               ! Check if already converged
C     ==  NO MICROITERATIVE OPTIMIZATION                              ==
         ELSE
            IF (LVLHES) THEN
               JUMP = 5            ! Hessian from RESTART file, go ahead
               LCVMSK = .TRUE.     ! Do not start if already converged
            ELSE
               JUMP = 7            ! Calculate Hessian
               LCVMSK = .FALSE.    ! We cannot be considered converged
            END IF
            LNOMAP = .TRUE.        ! All atoms map one-to-one
            LTOLAD = .FALSE.       ! No relaxed electronic convergence
         END IF
         CALL CRDCO (XPAR, COORD, VARS, DXPAR, GRAD, DVARS)
      END IF                       ! IF (JUMP.LE.0)
C     ==--------------------------------------------------------------==
C     ==  Check if environment converged (gradient only)              ==
C     ==  JUMP = 2 (form L-BFGS step)                                 ==
C     ==--------------------------------------------------------------==
      IF (JUMP.EQ.2) THEN
         CALL CNVENV (DVARS, GENVMX, LCONV)
         IF (IPRNT.GE.1) THEN
            WRITE (6,'(3X,A,F10.6)')
     $           'TOLERANCE FOR ENVIRONMENT:  ', TOLENV
            WRITE (6,'(3X,A,F10.6)')
     $           'MAXIMUM GRADIENT COMPONENT: ', GENVMX
         END IF
         IF (LCONV) THEN           ! TOLAD is in system.h
            IF (LTOLAD .AND. NSTRIED_P.NE.0 .AND. TOLAD.GT.0.0D0) THEN
               IF (IPRNT.GE.1) WRITE (6,*)
     $           'RELAXING ENVIRONMENT, STANDARD ELECTRONIC CONVERGENCE'
               LTOLAD = .FALSE.    ! No relaxed electronic convergence
               LCONV  = .FALSE.    ! Another cycle testing convergence
            ELSE IF (.NOT.LVLHES) THEN
               IF (IPRNT.GE.1) WRITE (6,*)
     $             'ENVIRONMENT CONVERGED, CALCULATING HESSIAN'
               JUMP = 7            ! Calculate Hessian
            ELSE
               IF (IPRNT.GE.1) WRITE (6,*)
     $             'ENVIRONMENT CONVERGED, OPTIMIZING CORE'
               LCVMSK = .TRUE.     ! We may be converged
               JUMP = 5            ! Form P-RFO step
            END IF
         ELSE
            LRET = .TRUE.
            CALL RLBFGS (VARS, DVARS, NENV, ETOT, LRET)
            IF (LRET) GOTO 999     ! End of routine
            IF (.NOT.LRET) GOTO 10 ! Beginning of routine
         END IF                    ! IF (LCONV) ... ELSE ...
      END IF                       ! IF (JUMP.EQ.2)
C     ==--------------------------------------------------------------==
C     ==  These states belong to environment L-BFGS                   ==
C     ==  JUMP <> 5, 6, 8                                             ==
C     ==--------------------------------------------------------------==
      IF (JUMP.NE.5.AND.JUMP.NE.6.AND.JUMP.NE.7.AND.JUMP.NE.8) THEN
         LRET = .TRUE.
         CALL RLBFGS (VARS, DVARS, NENV, ETOT, LRET)
         IF (LRET) GOTO 999     ! End of routine
         IF (.NOT.LRET) GOTO 10 ! Beginning of routine
      END IF
C     ==--------------------------------------------------------------==
C     ==  Form P-RFO step: Diagonalize Hessian and store old values   ==
C     ==  JUMP = 5                                                    ==
C     ==--------------------------------------------------------------==
      IF (JUMP.EQ.5) THEN
         IF (IPRNT.GE.1) WRITE (6,*) '  JUMP=5, DIAGONALIZING HESSIAN'
         IF (IPRNT.GE.2) THEN   ! should be 2
            DO I = 1,NVAR
               WRITE (6,'(5X,A,I3)') 'HESSIAN, ROW ', I
               WRITE (6,'(5X,3E12.5)') (HESSCR(I,J), J=1,NVAR) ! transp.
            END DO
         END IF
C
C     ==  STORE THINGS IN CASE OF A REJECTED STEP (SEE ALSO JUMP=6)   ==
C
         CALL DCOPY (NVAR,  GX,     1, OLDGX,  1)
         CALL DCOPY (NVAR,  OLDG,   1, OOLDG,  1)
         CALL DCOPY (NVAR,  EIGVAL, 1, OLDEIG, 1)
         CALL DCOPY (NVAR2, HESSCR, 1, OLDHSS, 1)
         CALL DCOPY (NVAR2, EIGVEC, 1, OLDEV,  1)
C
C     ==  UPDATE HESSIAN OR PROJECT OUT R/T MOTION OF HESSIAN         ==
C
         IF (NSTEP_P.GT.0) THEN
            CALL UPDHES (HESSCR, NVAR, GRAD, OLDG, STEP, STEP_P, S, T)
         ELSE IF (LPRHES) THEN
            CALL PRJHES (HESSCR, NVAR, COORD, GRAD, EIGVEC, IPRNT)
         END IF
C
C     ==  STORE MORE THINGS BEFORE THE ACTUAL STEP                    ==
C
         PRVENE = ETOT
         CALL DCOPY (NVAR, GRAD, 1, OLDG, 1)
C
C     ==  DIAGONALIZE HESSIAN                                         ==
C
         CALL DCOPY (NVAR2, HESSCR, 1, EIGVEC, 1)
         CALL DSYEV ('V', 'U', NVAR, EIGVEC, NVAR, EIGVAL, WDIAG,
     $               3*NVAR, INFO)
         IF (INFO.NE.0) THEN
            CALL STOPGM ('RPRFO', 'HESSIAN COULD NOT BE DIAGONALIZED')
         END IF
C
C     ==  GRADIENT IN HESSIAN EIGENSPACE                              ==
C
         CALL DGEMV ('T', NVAR, NVAR, 1.0D0, EIGVEC, NVAR, GRAD, 1,
     $               0.0D0, GX, 1)
         IF (IPRNT.GE.1) THEN
            DO I = 1,NVAR
               IF (I.LE.9 .OR. IPRNT.GE.2) THEN
                  WRITE (6,'(5X,A,I3,A,E12.5,A,E12.5)') 'EIGENVALUE ',
     $                 I, ': ', EIGVAL(I), ' GRADIENT: ', GX(I)
                  IF (IPRNT.GE.2)
     $                 WRITE (6,'(5X,3E12.5)') (EIGVEC(J,I), J=1,NVAR)
               END IF
            END DO
            IF (LPRHES) WRITE (6,'(5X,A,E8.2,A)') 'EIGENVALUES BELOW ',
     $           CUTEIG, ' ARE SET TO ZERO ALONG WITH GRADIENT'
         END IF
C
C     ==  APPLY EIGENVALUE CUTOFF AND COUNT NEGATIVE EIGENVALUES      ==
C
         NEGEV = 0
         NZEREV = 0
         DO I = 1,NVAR
            IF (ABS(EIGVAL(I)).LT.CUTEIG) THEN
               EIGVAL(I) = 0.0D0
               NZEREV = NZEREV + 1
               IF (LPRHES) GX(I) = 0.0D0
            END IF
            IF (EIGVAL(I).LT.0.0D0) NEGEV = NEGEV + 1
         END DO
         IF (IPRNT.GE.1) WRITE (6,*) '    P-RFO: HESSIAN HAS ', NEGEV,
     $        ' NEGATIVE EIGENVALUE(S)'
         IF (IPRNT.GE.1 .AND. NZEREV.GT.0) WRITE (6,*)
     $        '    P-RFO: HESSIAN HAS ', NZEREV, ' ZERO EIGENVALUES'
         CALL STACK_SCHED (IELSTK, ST_PUT)      ! save electronic WF
C
C     ==  SELECT LOWEST NON-ZERO EIGENVALUE IF PROJECTING HESSIAN     ==
C
         IF (LPRHES .AND. MODE.EQ.-1) THEN
            DO I = 1,NVAR
               IF (ABS(EIGVAL(I)).NE.0.0D0 .AND. MODE.EQ.-1)
     $              MODE = I
            END DO
            IF (IPRNT.GE.1) WRITE (6,*) '    P-RFO: SELECTED MODE ',
     $           MODE
         END IF
         JUMP = 8
      END IF                    ! IF (JUMP.EQ.5)
C     ==--------------------------------------------------------------==
C     ==  Form P-RFO step: do the work                                ==
C     ==  JUMP = 8 (return here if step is rejected in JUMP=6)        ==
C     ==--------------------------------------------------------------==
      IF (JUMP.EQ.8) THEN
         IF (IPRNT.GE.1) WRITE (6,*) '  JUMP=8, FORMING P-RFO STEP'
C
C     ==  CALCULATE HESSIAN MODE OVERLAP (MODE: PREVIOUS -> CURRENT)  ==
C
         CALL OVRLAP (EIGVEC, EIGVAL, VMODE, NVAR, MODE, OMIN,
     $                NSTEP_P, LLOCK, LREJ, LFIX, IPRNT)
C
         IF (LREJ) THEN
            IF (IPRNT.GE.1) WRITE (6,*)
     $           '  P-RFO: INSUFFICIENT OVERLAP, UNDOING PREVIOUS STEP'
            STEP_P = OSTEP_P
            SZ_P   = OSZ_P
            PRVENE = OPRVEN
C
            CALL DCOPY (NVAR,  OLDGX,  1, GX,     1)
            CALL DCOPY (NVAR,  OLDG,   1, GRAD,   1)
            CALL DCOPY (NVAR,  OOLDG,  1, OLDG,   1)
            CALL DCOPY (NVAR,  OLDEIG, 1, EIGVAL, 1)
            CALL DCOPY (NVAR2, OLDEV,  1, EIGVEC, 1)
            CALL DCOPY (NVAR2, OLDHSS, 1, HESSCR, 1)
C
            CALL DAXPY (NVAR, -1.0D0, STEP, 1, COORD, 1)
C
            STEP_P  = 0.5D0 * MIN (STEP_P, SZ_P)
            OSTEP_P = STEP_P
            OSZ_P   = SZ_P
            LREJ    = .FALSE.
         END IF
C
C     ==  CALCULATE THE STEP (STEP WILL BE THE ACTUAL GEOMETRY STEP)  ==
C
         CALL FRMSTP (EIGVAL, EIGVEC, GX, NVAR, STEP, STEP_P, SZ_P,
     $                MODE, FSTEP_P, DEPRED, IPRNT)
C
C     ==  DO THE STEP AND RETURN FOR NEW ENERGY / GRADIENT            ==
C
         CALL DAXPY (NVAR, 1.0D0, STEP, 1, COORD, 1)
C
         NSTRIED_P = NSTRIED_P + 1
         JUMP = 6
         GOTO 999
      END IF                    ! IF (JUMP.EQ.5)
C     ==--------------------------------------------------------------==
C     ==  TEST P-RFO step                                             ==
C     ==  JUMP = 6                                                    ==
C     ==--------------------------------------------------------------==
      IF (JUMP.EQ.6) THEN
         IF (IPRNT.GE.1) WRITE (6,*) '  JUMP=6, TESTING P-RFO STEP'
         DEACT = ETOT - PRVENE
         RATIO = DEACT / DEPRED
         IF (IPRNT.GE.1) THEN
            WRITE (6,*) '    P-RFO TRUST RADIUS TESTS'
            WRITE (6,'(5X,A,F14.7)') 'PREDICTED ENERGY CHANGE: ', DEPRED
            WRITE (6,'(5X,A,F14.7)') 'ACTUAL ENERGY CHANGE:    ', DEACT
         END IF
C
         IF ((RATIO.LT.RMIN_P .OR. RATIO.GT.RMAX_P) .AND.
     $       (DEPRED.GT.DEMIN_P .OR. DEACT.GT.DEMIN_P)) THEN
            IF (STEP_P.GE.STEP_MIN_P) THEN
               LREJ = .TRUE.
               IF (IPRNT.GE.1) THEN
                  WRITE (6,*) '    ENERGY RATIO OUTSIDE ALLOWED RANGE'
                  WRITE (6,'(3X,A,F10.6)')
     $                 'REJECTING STEP, REDUCING TRUST RADIUS TO ',
     $                 0.5D0 * MIN (STEP_P, SZ_P)
               END IF
            ELSE                ! IF (STEP_P.GE.STEP_MIN_P)
               LREJ = .FALSE.
               IF (IPRNT.GE.1) WRITE (6,*) '    ENERGY RATIO BAD ', 
     $              'BUT TRUST RADIUS ALREADY MINIMUM: ', STEP_P
            END IF
         ELSE                   ! IF (RATIO UNACCETPABLE) ...
            LREJ = .FALSE.
            IF (IPRNT.GE.1) WRITE (6,*) '    P-RFO STEP ACCEPTED'
         END IF                 ! IF (RATIO UNACCEPTABLE) ... ELSE ...
C
C     ==  UNDO REJECTED STEP AND RETRY                                ==
C
         IF (LREJ) THEN
            STEP_P = 0.5D0 * MIN (STEP_P, SZ_P)
c            write (6,*) 'Step to be undone: ' ! remove
c            write (6,'(3f10.5)') (step(i),i=1,nvar) ! remove
            CALL DAXPY (NVAR, -1.0D0, STEP, 1, COORD, 1)
            CALL STACK_SCHED (IELSTK, ST_GET) ! restore electronic WF
            JUMP = 8            ! RETRY STEP
            GOTO 10             ! FROM BEGINNING
         END IF                 ! IF (LREJ)
C
C     ==  STEP ACCEPTED - ADJUST TRUST RADIUS TO AVOID REJECTED STEPS ==
C
         NSTEP_P = NSTEP_P + 1
         OSTEP_P = STEP_P
         OSZ_P   = SZ_P
         OPRVEN  = PRVENE
         DEACCP  = ETOT - PRVENE ! energy change of accepted step
         IF (LFIX) THEN
            IF (STEP_P.GE.STEP_MIN_P) THEN
               STEP_P = 0.5D0 * STEP_P
               IF (IPRNT.GE.1)
     $              WRITE (6,*) '    REDUCING P-RFO TRUST RADIUS'
            ELSE
               IF (IPRNT.GE.1)
     $              WRITE (6,*) '    NOT REDUCING P-RFO TRUST RADIUS'
            END IF
         ELSE                   ! IF (LFIX) ...
            IF (RATIO.GE.0.75D0 .AND. RATIO.LE.4.0D0/3.0D0
     $          .AND. SZ_P.GE.STEP_P-1.0D-2) THEN
               STEP_P = SQRT(2.0D0) * STEP_P
               IF (RATIO.GE.0.9D0 .AND. RATIO.LE.1.1D0) THEN
                  STEP_P = SQRT(2.0D0) * STEP_P
               END IF
            END IF              ! IF (RATIO REASONABLE)
         END IF                 ! IF (LFIX) ... ELSE ...
         STEP_P = MAX (STEP_P, STEP_MIN_P)
         STEP_P = MIN (STEP_P, STEP_MAX_P)
         IF (IPRNT.GE.1) THEN
            WRITE (6,'(5X,A,F10.6)') 'NEW TRUST RADIUS:    ', STEP_P
         END IF
C
C     ==  DETERMINE NEXT ACTION AND GO TO BEGINNING                   ==
C
         LPEXIT = (NSTRIED_P.GE.NSMAXP)
         IF (LMICRO) THEN
            IF (IPRNT.GE.1) WRITE (6,*)
     $           '  ALLOWING ADAPTIVE ELECTRONIC CONVERGENCE CRITERIA'
            LTOLAD = .TRUE.     ! Allow relaxed electronic convergence
            LCVMSK = .FALSE.    ! CNVENV determines if we are converged
            JUMP = 2            ! Form L-BFGS step
         ELSE
            LCVMSK = .TRUE.     ! Driving code may tell we are converged
            JUMP = 5            ! Form P-RFO step
         END IF
         IF (NSVIB.GT.0) THEN
            IF (MOD(NSTEP_P,NSVIB).EQ.0) CALL PVIBANA
         END IF
         GOTO 10
      END IF                    ! IF (JUMP.EQ.6)
C     ==--------------------------------------------------------------==
C     ==  Calculate initial Hessian                                   ==
C     ==  JUMP = 7                                                    ==
C     ==--------------------------------------------------------------==
      IF (JUMP.EQ.7) THEN
         IF (IPRNT.GE.1) WRITE (6,*) '  JUMP=7, CALCULATING HESSIAN'
         IF (M_HESS.EQ.1) THEN  ! Hessian from CPMD
            IF (IPRNT.GE.1) WRITE (6,*) '  USING HESSIAN FROM CPMD'
            LRHE = .TRUE.
            CALL HESSIN (XPAR, LRHE)
            CALL PHESSCO
C           IF (LPRHES)
C    $           CALL PRJHES (HESSCR, NVAR, COORD, GRAD, EIGVEC, IPRNT)
            LVLHES = .TRUE.     ! Hessian valid now
            LCVMSK = .TRUE.     ! Driving code may determine converg.
            NSHESS = 0
            JUMP = 5
            GOTO 10
         END IF
         IF (M_HESS.EQ.0) THEN  ! Hessian from finite difference
            CALL INIHES (HESSCR, NVAR, COORD, GRAD, NSHESS, OLDG, IPRNT,
     $           EPS_H, LUNDOP, IELSTK, ST_PUT, ST_GET)
C           IF (LPRHES)
C    $           CALL PRJHES (HESSCR, NVAR, COORD, GRAD, EIGVEC, IPRNT)
            NSHESS = NSHESS + 1
            IF (NSHESS.GT.2*NVAR) LVLHES = .TRUE. ! Hessian valid now
         END IF
         IF (LVLHES) THEN
            M_HESS = 0             ! Do not read same Hessian again
            LCVMSK = .TRUE.        ! Driving code may determine converg.
            LPEXIT = (NSMAXP.EQ.0) ! Exit if Hessian only
            NSHESS = 0
            IF (NSVIB.NE.0) CALL PVIBANA
            JUMP = 5
         END IF
      END IF                    ! IF (JUMP.EQ.7)
C     ==--------------------------------------------------------------==
C     ==  Splice new coordinates from core and environment            ==
C     ==--------------------------------------------------------------==
 999  CONTINUE                  ! graceful return
      CALL CRDCI (XPAR, COORD, VARS)
C     ==--------------------------------------------------------------==
      CALL TIHALT('     RPRFO',ISUB)
      RETURN
      END
C
C     ==================================================================
      SUBROUTINE INIHES (HESSCR, NVAR, COORD, GRAD, NSHESS, OLDG, IPRNT,
     $                   EPS, LUNDOP, IELSTK, ST_PUT, ST_GET)
C     ==--------------------------------------------------------------==
C     ==  Calculates the finite difference Hessian                    ==
C     ==                                                              ==
C     ==  The Hessian is symmetrized on the fly to parallelize well.  ==
C     ==  Therefore, the old gradient needs to be stored in OLDG.     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      LOGICAL LUNDOP
      INTEGER NVAR, NSHESS, IPRNT, IELSTK, ST_PUT, ST_GET
      REAL*8  HESSCR(NVAR,NVAR), COORD(NVAR), GRAD(NVAR), OLDG(NVAR),
     $        EPS
C     Variables
      LOGICAL LWFSTK
      INTEGER IPLC, IPOS
      SAVE    LWFSTK
C     ==--------------------------------------------------------------==
      DATA    LWFSTK / .FALSE. / ! No wavefunction on stack yet
C     ==--------------------------------------------------------------==
      IF (.NOT.LWFSTK) THEN
         CALL STACK_SCHED (IELSTK, ST_PUT) ! save electronic WF
         LWFSTK = .TRUE.
      END IF
C
C     ==  STARTING FROM A PARTIALLY CALCULATED HESSIAN ON CHECKPOINT  ==
C
      IF (LUNDOP) THEN
         IPOS = NSHESS/2 + 1
         IF (IPRNT.GE.1) WRITE (6,*)
     $        '   HESSIAN: UNDOING DIMENSION ', IPOS
         COORD(IPOS) = COORD(IPOS) + 2*EPS
         NSHESS = NSHESS - 1
         LUNDOP = .FALSE.
         RETURN
      END IF
C
C     ==  INITIAL CALL: PREPARE                                       ==
C
      IF (NSHESS.EQ.0) THEN
         IF (IPRNT.GE.1) WRITE (6,*) '    HESSIAN: INITIAL CALL'
         COORD(1) = COORD(1) + EPS
         CALL AZZERO (HESSCR, NVAR*NVAR)
         CALL STACK_SCHED (IELSTK, ST_PUT) ! save electronic WF
         LWFSTK = .TRUE.
C
C     ==  OTHERWISE: CALCULATE COLUMN / ROW                           ==
C
      ELSE
         IPLC = MOD (NSHESS, 2)
         IPOS = NSHESS/2 + IPLC
         IF (IPRNT.GE.1) WRITE (6,*) '    HESSIAN: DIMENSION ', IPOS
C
C     ==  FINITE DIFFERENCE STEP IPOS BACKWARD (ODD CALLS)            ==
C
         IF (IPLC.EQ.1) THEN
            CALL DCOPY (NVAR, GRAD, 1, OLDG, 1)
            COORD(IPOS) = COORD(IPOS) - 2*EPS  ! back current step
C
C     ==  FINITE DIFFERENCE STEP IPOS DONE (FORWARD - EVEN CALLS)     ==
C
         ELSE
            CALL DAXPY (NVAR, -1.0D0, GRAD, 1, OLDG, 1)
            CALL DAXPY (NVAR, 0.25D0/EPS, OLDG, 1,
     $                  HESSCR(1,IPOS), 1)
            CALL DAXPY (NVAR, 0.25D0/EPS, OLDG, 1,
     $                  HESSCR(IPOS,1), NVAR)
            COORD(IPOS) = COORD(IPOS) + EPS    ! undo current step
            IPOS = IPOS + 1
            IF (IPOS.LE.NVAR) THEN
               COORD(IPOS) = COORD(IPOS) + EPS ! do next step
            END IF
         END IF
         CALL STACK_SCHED (IELSTK, ST_GET) ! restore electronic WF
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C
C     ==================================================================
      SUBROUTINE UPDHES (HESSCR, NVAR, GRAD, OLDG, STEP, STEPSZ, S, T)
C     ==--------------------------------------------------------------==
C     ==  Updates the Hessian HESSCR of the reaction core using the   ==
C     ==  Powell formula                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER NVAR
      REAL*8  HESSCR(NVAR,NVAR), GRAD(NVAR), OLDG(NVAR), STEP(NVAR),
     $        STEPSZ, S(NVAR),  T(NVAR)
C     Variables
      REAL*8  STEPTT, STSQNV
C     Externals
      REAL*8 DDOT
      EXTERNAL DDOT
C     ==--------------------------------------------------------------==
C
C     ==  T = HESS * STEP                                             ==
C
      CALL DSYMV ('U', NVAR, 1.0D0, HESSCR, NVAR, STEP, 1, 0.0D0, T, 1)
C
C     ==  T = GRAD-OLDG-T,   S = GRAD-OLDG                            ==
C
      CALL DCOPY (NVAR, GRAD, 1, S, 1)
      CALL DAXPY (NVAR, -1.0D0, OLDG, 1, S, 1)
      CALL DAXPY (NVAR, -1.0D0, S, 1, T, 1)
      CALL DSCAL (NVAR, -1.0D0, T, 1)
C
      STSQNV = 1.0D0 / (STEPSZ * STEPSZ)
      STEPTT = DDOT (NVAR, STEP, 1, T, 1) * STSQNV * STSQNV
C
C     ==  H(i,j) = H(i,j) + STSQNV * (ST(i)T(j) + ST(j)T(i))          ==
C     ==                  - STEPTT * ST(i)ST(j)                       ==
C
      CALL DSYR2 ('U', NVAR, STSQNV, STEP, 1, T, 1, HESSCR, NVAR)
      CALL DSYR ('U', NVAR, -STEPTT, STEP, 1, HESSCR, NVAR)
C
      RETURN
      END
C
C     ==================================================================
      SUBROUTINE PRJHES (HESSCR, NVAR, COORD, GRAD, SDER, IPRNT)
C     ==--------------------------------------------------------------==
C     ==  Project out translational / rotational components of the    ==
C     ==  Hessian                                                     ==
C     ==  Note: SDER is only work space                               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'coor.inc'
C     Arguments
      INTEGER NVAR, IPRNT
      REAL*8  HESSCR(NVAR,NVAR), COORD(NVAR), GRAD(NVAR),
     $        SDER(NVAR,NVAR)
C     ==--------------------------------------------------------------==
      IF (NVAR.NE.3*NAT) THEN
         IF (IPRNT.GE.0) THEN
            WRITE (6,'(A,I4,A,I4)') ' WARNING! 3*NAT:', 3*NAT,
     $           ' DOES NOT MATCH NVAR:', NVAR
            WRITE (6,*) 'HESSIAN CLEANING MUST BE SKIPPED!'
         END IF
         RETURN
      ELSE
         IF (IPRNT.GE.1) WRITE (6,*) '    CLEANING HESSIAN'
      END IF
C
      CALL DCOPY (NVAR*NVAR, HESSCR, 1, SDER, 1)
      CALL PURGEH (SDER, HESSCR, TAU0)
      CALL DCOPY (NVAR*NVAR, SDER, 1, HESSCR, 1)
C     ==--------------------------------------------------------------==
      RETURN
      END
C
C     ==================================================================
      SUBROUTINE OVRLAP (EIGVEC, EIGVAL, VMODE, NVAR, MODE, OMIN,
     $                   NSTEP_P, LLOCK, LREJ, LFIX, IPRNT)
C     ==--------------------------------------------------------------==
C     ==  Calculate the overlap between the mode to be followed and   ==
C     ==  the current eigenvectors of the Hessian                     ==
C     ==  Determine the mode MODCUR to be followed                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      LOGICAL LLOCK, LREJ, LFIX
      INTEGER NVAR, MODE, NSTEP_P, IPRNT
      REAL*8  EIGVEC(NVAR,NVAR), EIGVAL(NVAR), VMODE(NVAR), OMIN
C     Variables
      INTEGER I, NEWMOD
      REAL*8  OVLP, OVLPMX
C     Externals
      REAL*8   DDOT
      EXTERNAL DDOT
C     ==--------------------------------------------------------------==
      NEWMOD = MODE
      LREJ = .FALSE.
      LFIX = .FALSE.
C
C     ==  INITIAL CALL: STORE MODE IN VMODE                           ==
C
      IF (NSTEP_P.EQ.0) THEN
         IF (IPRNT.GE.1) THEN
            WRITE (6,*) '    INITIAL HESSIAN MODE: ', MODE
            WRITE (6,'(5X,A,F10.6)') 'HESSIAN EIGENVALUE: ',EIGVAL(MODE)
            IF (LLOCK) THEN
               WRITE (6,*) '    THIS MODE IS LOCKED'
               DO I = 1,MODE
                  WRITE (6,'(5X,A,I1,A,F10.6)') 'HESSIAN EV(', I, '): ',
     $                 EIGVAL(I)                  
               END DO
            ELSE
               WRITE (6,'(5X,A,A,F10.5)') 'MODE FOLLOWING IS SWITCHED ',
     $              'ON, MINIMUM OVERLAP: ', OMIN
            END IF
         END IF
         CALL DCOPY (NVAR, EIGVEC(1,NEWMOD), 1, VMODE, 1)
      ELSE
         IF (LLOCK) THEN
            OVLP = ABS (DDOT (NVAR, EIGVEC(1,NEWMOD), 1, VMODE, 1))
            IF (IPRNT.GE.1) WRITE (6,*) '    MODE LOCKED, OVERLAP: ',
     $           OVLP
         ELSE
C
C     ==  DETERMINE WHICH MODE HAS THE GREATEST OVERLAP               ==
C
            OVLPMX = 0.0D0
            DO I = 1,NVAR
               OVLP = ABS (DDOT (NVAR, EIGVEC(1,I), 1, VMODE, 1))
               IF (OVLP.GT.OVLPMX) THEN
                  NEWMOD = I
                  OVLPMX = OVLP
               END IF
            END DO
            IF (IPRNT.GE.1) THEN
               WRITE (6,'(5X,A,I3,A,F10.6)') 'OVERLAP OF CURRENT MODE ',
     $              NEWMOD, ' WITH PREVIOUS MODE IS ', OVLPMX
            END IF
C
C     ==  CHECK IF OVERLAP IS SUFFICIENT                              ==
C
            IF (OVLPMX.LT.OMIN) THEN
               LREJ = .TRUE.
               IF (IPRNT.GE.1) WRITE (6,*)
     $               '    THIS IS INSUFFICIENT, REJECTING STEP'
            ELSE IF (OVLPMX.LT.SQRT(OMIN)) THEN
               LFIX = .TRUE.
               IF (IPRNT.GE.1) WRITE (6,*)
     $               '    THIS IS TIGHT, DENYING STEP SIZE TO INCREASE'
            END IF
C
C     ==  STORE NEW EIGENMODE IN VMODE                                ==
C
            CALL DCOPY (NVAR, EIGVEC(1,NEWMOD), 1, VMODE, 1)
            IF (IPRNT.GE.1) THEN
               IF (NEWMOD.NE.MODE) THEN
                  WRITE (6,*) '  WARNING: SWITCHING MODE FROM ', MODE,
     $                 ' TO ', NEWMOD
               END IF
               DO I = 1,MAX(NEWMOD,MODE)
                  WRITE (6,'(5X,A,I1,A,F10.6)') 'HESSIAN EV(', I, '): ',
     $                 EIGVAL(I)                  
               END DO
            END IF
         END IF                 ! IF (NSTEP_P.GE.1) ... ELSE ...
      END IF
      IF (.NOT.LREJ) MODE = NEWMOD
C     ==--------------------------------------------------------------==
      RETURN
      END
C
C     ==================================================================
      SUBROUTINE FRMSTP (EIGVAL, EIGVEC, GX, NVAR, STEP, STEP_P, SZ_P,
     $                   MODE, FSTEP, DEPRED, IPRNT)
C     ==--------------------------------------------------------------==
C     ==  Form a P-RFO step using an already diagonalized Hessian     ==
C     ==  and predict energy change                                   ==
C     ==                                                              ==
C     ==  Arguments:                                                  ==
C     ==  EIGVAL: (in)  Eigenvalues of the Hessian                    ==
C     ==  EIGVEC: (in)  Eigenvectors of the Hessian                   ==
C     ==  GX:     (in)  Gradient in the Hessian eigenspace            ==
C     ==  STEP:   (out) Calculated step                               ==
C     ==  STEP_P: (in)  Trust radius for P-RFO                        ==
C     ==  SZ_P:   (out) Actual step size                              ==
C     ==  MODE:   (in)  Mode to be maximized                          ==
C     ==  FSTEP:  (par) Step size to for finding lambdas              ==
C     ==  DEPRED: (out) Predicted energy change                       ==
C     ==  IPRNT:  (in)  Print level                                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER NVAR, MODE, IPRNT
      REAL*8  EIGVAL(NVAR), EIGVEC(NVAR,NVAR), GX(NVAR), STEP(NVAR),
     $        STEP_P, SZ_P, FSTEP, DEPRED
C     Variables
      LOGICAL LDOSCL, LLOW, LUPP
      INTEGER I, MODMAX, MODMIN, NCNT
      REAL*8  BIG, BLOW, BUPP, BSTEP, EIGMAX, EIGMIN, EPS, EPS2, EPS6,
     $        FLAM, FLOW, FUPP, LMBMAX, LMBMIN, SCALE,
     $        SSZMIN, SSZMAX, SSZTOL, TEMP, XLMMAX, XLMMIN
C     Externals
      REAL*8   DDOT
      EXTERNAL DDOT
C     ==--------------------------------------------------------------==
      LDOSCL = .FALSE.
      MODMAX = MODE
      MODMIN = 1
      IF (MODMAX.EQ.1) MODMIN = 2
      EIGMAX = EIGVAL(MODMAX)
      EIGMIN = EIGVAL(MODMIN)
C
C     ==  SET BISECTION STEP SIZES AND TOLERANCES                     ==
C
      BIG    = 1.0D+3
      EPS    = 1.0D-12
      SSZMIN = MAX (ABS(EIGMIN)*EPS, 10.0D0*EPS)
      SSZMAX = MAX (BIG, ABS(EIGMIN)) * BIG
      SSZTOL = 1.0D-8
      EPS2   = 1.0D-2           ! for steps along eigenmodes
      EPS6   = 1.0D-6           ! for determining if scaling large steps
c      write (6,*) 'fstep, sszmin, sszmax: ', fstep, sszmin, sszmax
C
C     ==  FIND LMBMAX (MAXIMIZE MODMAX), LMBMIN (MINIMIZE ALL OTHERS) ==
C
      LMBMAX = 0.5D0 * (EIGMAX + SQRT (EIGMAX**2 + 4.0D0*GX(MODMAX)**2))
C
      LMBMIN = 0.0D0
      BSTEP  = FSTEP
      IF (EIGMIN.LE.0.0D0) LMBMIN = EIGMIN - BSTEP
      IF (EIGMIN.GT.0.0D0) BSTEP  = EIGMIN
      BLOW = LMBMIN - BSTEP
      BUPP = LMBMIN + 0.5D0*BSTEP
C
C     ==  BISECT THE INTERVALS BLOW / BUP (WHILE FLOW*FUPP >= 0)      ==
C
 50   FLOW = 0.0D0
      FUPP = 0.0D0
      DO I = 1,NVAR
         FLOW = FLOW + GX(I)**2/(BLOW-EIGVAL(I))
         FUPP = FUPP + GX(I)**2/(BUPP-EIGVAL(I))
      END DO
      FLOW = FLOW - GX(MODMAX)**2/(BLOW-EIGVAL(MODMAX))
      FUPP = FUPP - GX(MODMAX)**2/(BUPP-EIGVAL(MODMAX))
      FLOW = FLOW - BLOW
      FUPP = FUPP - BUPP
      IF (FLOW*FUPP.GE.0.0D0) THEN
         BLOW = BLOW - (EIGMIN-BLOW)
         BUPP = BUPP + 0.5D0 * (EIGMIN-BUPP)
         LLOW = .FALSE.
         IF (BLOW.LE.-SSZMAX) THEN
            BLOW = -SSZMAX
            LLOW = .TRUE.
         END IF
         LUPP = .FALSE.
         IF (ABS(EIGMIN-BUPP).LE.SSZMIN) THEN
            BUPP = EIGMIN - SSZMIN
            LUPP = .TRUE.
         END IF
         IF (LLOW.AND.LUPP) THEN
            WRITE (6,*)'  P-RFO: FIXED STEP SIZE NOT IMPLEMENTED'
            CALL STOPGM ('RPRFO', 'PROBLEMS BRACKETING LAMBDA')
         END IF
         GOTO 50
      END IF                    ! WHILE (FLOW*FUPP.GE.0.0D0)
C
      NCNT   = 0
      XLMMIN = 0.0D0
C
C     ==  BISECT LMBMIN / BLOW / BUPP (WHILE LMBMIN NOT CONVERGED)    ==
C
 80   CONTINUE                  ! WHILE (ABS(XLMMIN-LMBMIN).GT.SSZTOL)
         FLOW = 0.0D0
         FUPP = 0.0D0
         FLAM = 0.0D0
         LMBMIN = 0.5D0 * (BLOW+BUPP)
         DO I = 1,NVAR
            FLOW = FLOW + GX(I)**2/(BLOW-EIGVAL(I))
            FUPP = FUPP + GX(I)**2/(BUPP-EIGVAL(I))
            FLAM = FLAM + GX(I)**2/(LMBMIN-EIGVAL(I))
         END DO
         FLOW = FLOW - GX(MODMAX)**2/(BLOW-EIGVAL(MODMAX))
         FUPP = FUPP - GX(MODMAX)**2/(BUPP-EIGVAL(MODMAX))
         FLAM = FLAM - GX(MODMAX)**2/(LMBMIN-EIGVAL(MODMAX))
         FLOW = FLOW - BLOW
         FUPP = FUPP - BUPP
         FLAM = FLAM - LMBMIN
         IF (ABS(XLMMIN-LMBMIN).GE.SSZTOL) THEN
            NCNT = NCNT + 1
            IF (NCNT.GT.1000) THEN
               WRITE (6,*) '  P-RFO: TOO MANY ITERATIONS FOR LAMBDA'
               CALL STOPGM ('RPFRO', 'PROBLEM BISECTING LAMBDA')
            END IF
            XLMMIN = LMBMIN
            IF (FLAM*FUPP.LT.0.0D0) BLOW = LMBMIN
            IF (FLAM*FLOW.LT.0.0D0) BUPP = LMBMIN
            GOTO 80
         END IF
      CONTINUE                  ! WHILE (ABS(XLMMIN-LMBMIN).GT.SSZTOL)
C
C     ==  NOW WE HAVE LMBMAX, LMBMIN: PRINT IF REQUESTED              ==
C
 100  XLMMAX = LMBMAX
      XLMMIN = LMBMIN
      IF (IPRNT.GE.1) THEN
         WRITE (6,'(5X,A,F14.5)')
     $        'P-RFO: LAMBDA MAXIMIZING TS MODE:         ', LMBMAX
         WRITE (6,'(5X,A,F14.5)')
     $        'P-RFO: LAMBDA MINIMIZING ALL OTHER MODES: ', LMBMIN
      END IF
C
C     ==  CALCULATE THE P-RFO STEP NOW                                ==
C
      CALL AZZERO (STEP, NVAR)
      DO I = 1,NVAR
         IF (I.EQ.MODMAX) THEN
            TEMP = GX(I) / (LMBMAX-EIGVAL(I))
         ELSE IF (LMBMIN.EQ.0.0D0 .AND. ABS(EIGVAL(I)).LT.EPS2) THEN
            TEMP = 0.0D0
         ELSE
            TEMP = GX(I) / (LMBMIN-EIGVAL(I))
         END IF
         CALL DAXPY (NVAR, TEMP, EIGVEC(1,I), 1, STEP, 1)
      END DO
C
C     ==  SCALE THE STEP                                              ==
C
      SZ_P = SQRT (DDOT (NVAR, STEP, 1, STEP, 1))
      IF (SZ_P .LT. STEP_P+EPS6) THEN
         IF (IPRNT.GE.1) THEN
            WRITE (6,'(5X,A,F10.7)') 'TRUST RADIUS STEP:   ', STEP_P
            WRITE (6,'(5X,A,F10.7)') 'PERFORMED STEP SIZE: ', SZ_P
         END IF
         SCALE = 1.0D0
      ELSE
         IF (LDOSCL) THEN
            IF (IPRNT.GE.1) THEN
               WRITE (6,'(5X,A,F10.7)') 'TRUST RADIUS STEP:   ', STEP_P
               WRITE (6,'(5X,A,F10.7)') 'PREDICTED STEP SIZE: ', SZ_P
               WRITE (6,'(5X,A,F10.7)') 'PERFORMED STEP SIZE: ', STEP_P
            END IF
            SCALE = STEP_P / SZ_P
            CALL DSCAL (NVAR, SCALE, STEP, 1)
         ELSE
            IF (IPRNT.GE.1) WRITE (6,'(5X,A,F12.7,/,5X,A)')
     $           'CANNOT SCALE STEP OF SIZE ', SZ_P,
     $           'FINDING LAMBDAS FOR TRUST RADIUS'
            LDOSCL = .TRUE.
            GOTO 140
         END IF
      END IF
C
C     ==  PREDICT THE ENERGY CHANGE                                   ==
C
      DEPRED = 0.0D0
      DO I = 1,NVAR
         IF (I.EQ.MODMAX) THEN
            TEMP = SCALE * GX(I) / (LMBMAX-EIGVAL(I))
         ELSE IF (LMBMIN.EQ.0.0D0 .AND. ABS(EIGVAL(I)).LT.EPS2) THEN
            TEMP = 0.0D0
         ELSE
            TEMP = SCALE * GX(I) / (LMBMIN-EIGVAL(I))
         END IF
         DEPRED = DEPRED + TEMP*GX(I) + 0.5D0*TEMP*TEMP*EIGVAL(I)
      END DO
      RETURN
C
C     ==  FIND LAMBDAS FOR STEP WITH TRUST RADIUS                     ==
C
 140  CONTINUE
      LMBMIN = 0.0D0
      BSTEP  = FSTEP
      IF (EIGMIN.LE.0.0D0) LMBMIN = EIGMIN - BSTEP
      IF (EIGMIN.GT.0.0D0) BSTEP  = EIGMIN
      BLOW = LMBMIN - BSTEP
      BUPP = LMBMIN + 0.5D0*BSTEP
C
C     ==  BISECTION (TRUST RADIUS STEP SIZE) - SEE LOOP 50            ==
C
 150  FLOW = 0.0D0
      FUPP = 0.0D0
      DO I = 1,NVAR
         FLOW = FLOW + (GX(I)/(BLOW-EIGVAL(I)))**2
         FUPP = FUPP + (GX(I)/(BUPP-EIGVAL(I)))**2
      END DO
      FLOW = FLOW - (GX(MODMAX)/(BLOW-EIGVAL(MODMAX)))**2
     $            + (GX(MODMAX)/(BLOW+EIGVAL(MODMAX)))**2
      FUPP = FUPP - (GX(MODMAX)/(BUPP-EIGVAL(MODMAX)))**2
     $            + (GX(MODMAX)/(BUPP-EIGVAL(MODMAX)))**2
      FLOW = FLOW - STEP_P**2
      FUPP = FUPP - STEP_P**2
c      write (6,*)'before 50 loop, flow, fupp, llow, lupp, blow, bupp: ',
c     $        flow, fupp, llow, lupp, blow, bupp ! remove
      IF (FLOW*FUPP.GE.0.0D0) THEN
         BLOW = BLOW - (EIGMIN-BLOW)
         BUPP = BUPP + 0.5D0 * (EIGMIN-BUPP)
         LLOW = .FALSE.
         IF (BLOW.LE.-SSZMAX) THEN
            BLOW = -SSZMAX
            LLOW = .TRUE.
         END IF
         LUPP = .FALSE.
         IF (ABS(EIGMIN-BUPP).LE.SSZMIN) THEN
            BUPP = EIGMIN - SSZMIN
            LUPP = .TRUE.
         END IF
         IF (LLOW.AND.LUPP) THEN
            WRITE (6,*)'  FIXED STEP SIZE NOT IMPLEMENTED, STOPPING NOW'
            CALL STOPGM ('RPRFO', 'PROBLEMS BRACKETING LAMBDA')
         END IF
c         write (6,*)'end 50 loop, flow, fupp, llow, lupp, blow, bupp: ',
c     $        flow, fupp, llow, lupp, blow, bupp ! remove
         GOTO 150
      END IF                    ! WHILE (FLOW*FUPP.GE.0.0D0)
C
      NCNT   = 0
      XLMMIN = 0.0D0
C
C     ==  BISECTION PART 2 (TRUST RADIUS STEP SIZE) - SEE LOOP 80     ==
C
 180  CONTINUE                  ! WHILE (ABS(XLMMIN-LMBMIN).GT.SSZTOL)
         FLOW = 0.0D0
         FUPP = 0.0D0
         FLAM = 0.0D0
         LMBMIN = 0.5D0 * (BLOW+BUPP)
         DO I = 1,NVAR
            FLOW = FLOW + (GX(I)/(BLOW-EIGVAL(I)))**2
            FUPP = FUPP + (GX(I)/(BUPP-EIGVAL(I)))**2
            FLAM = FLAM + (GX(I)/(LMBMIN-EIGVAL(I)))**2
         END DO
         FLOW = FLOW - (GX(MODMAX)/(BLOW-EIGVAL(MODMAX)))**2
     $               + (GX(MODMAX)/(BLOW+EIGVAL(MODMAX)))**2
         FUPP = FUPP - (GX(MODMAX)/(BUPP-EIGVAL(MODMAX)))**2
     $               + (GX(MODMAX)/(BUPP+EIGVAL(MODMAX)))**2
         FLAM = FLAM - (GX(MODMAX)/(LMBMIN-EIGVAL(MODMAX)))**2
     $               + (GX(MODMAX)/(LMBMIN+EIGVAL(MODMAX)))**2
         FLOW = FLOW - STEP_P**2
         FUPP = FUPP - STEP_P**2
         FLAM = FLAM - STEP_P**2
c         write(6,*)'within 80 loop, flow, flam, fupp, lmbmin, xlmmin: ',
c     $     flow, flam, fupp, lmbmin, xlmmin ! remove
         IF (ABS(XLMMIN-LMBMIN).GE.SSZTOL) THEN
            NCNT = NCNT + 1
            IF (NCNT.GT.1000) THEN
               WRITE (6,*) '  P-RFO: TOO MANY ITERATIONS FOR LAMBDA'
               CALL STOPGM ('RPFRO', 'PROBLEM BISECTING LAMBDA')
            END IF
            XLMMIN = LMBMIN
            IF (FLAM*FUPP.LT.0.0D0) BLOW = LMBMIN
            IF (FLAM*FLOW.LT.0.0D0) BUPP = LMBMIN
            GOTO 180
         END IF
      CONTINUE                  ! WHILE (ABS(XLMMIN-LMBMIN).GT.SSZTOL)
C
      LMBMAX = -LMBMIN
C
C     ==  FOUND LAMBDAS, DOING THE STEP NOW                           ==
C
      GOTO 100
C     ==--------------------------------------------------------------==
      END
C     ==================================================================
      SUBROUTINE CRDCO (XPAR, COORD, VARS, DXPAR, GRAD, DVARS)
C     ==--------------------------------------------------------------==
C     ==  Splices the coordinates and gradient into a core and an     ==
C     ==  environment part (check out)                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'tpar.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'lscal.inc'
C     Arguments
      REAL*8  XPAR(NODIM), COORD(NVAR), VARS(NODIM-NVAR),
     $        DXPAR(NODIM), GRAD(NVAR), DVARS(NODIM-NVAR)
C     Variables
      INTEGER I, J
C     ==--------------------------------------------------------------==
      IF (NVAR .EQ. NODIM) THEN
         CALL DCOPY (NVAR, XPAR,  1, COORD, 1)
         CALL DCOPY (NVAR, DXPAR, 1, GRAD,  1)
      ELSE IF (LNOMAP) THEN
         CALL DCOPY (NVAR, XPAR(1),  1, COORD, 1)
         CALL DCOPY (NVAR, DXPAR(1), 1, GRAD, 1)
         CALL DCOPY (NODIM-NVAR, XPAR(NVAR+1),  1, VARS,  1)
         CALL DCOPY (NODIM-NVAR, DXPAR(NVAR+1), 1, DVARS, 1)
      ELSE
         DO I = 1,NVAR
            COORD(I)  = XPAR(MAP_P(I))
            GRAD(I)   = DXPAR(MAP_P(I))
         END DO
         J = NVAR + 1
         DO I = 1,NODIM-NVAR
            VARS(I)  = XPAR(MAP_P(J))
            DVARS(I) = DXPAR(MAP_P(J))
            J = J + 1
         END DO
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C
C     ==================================================================
      SUBROUTINE CRDCI (XPAR, COORD, VARS)
C     ==--------------------------------------------------------------==
C     ==  Splices the coordinates from the core and the environement  ==
C     ==  into the new common coordinates (check in)                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'tpar.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'lscal.inc'
C     Arguments
      REAL*8  XPAR(NODIM), COORD(NVAR), VARS(NODIM-NVAR)
C     Variables
      INTEGER I, J
C     ==--------------------------------------------------------------==
      IF (NVAR .EQ. NODIM) THEN
         CALL DCOPY (NVAR, COORD, 1, XPAR, 1)
      ELSE IF (LNOMAP) THEN
         CALL DCOPY (NVAR, COORD, 1, XPAR(1), 1)
         CALL DCOPY (NODIM-NVAR, VARS, 1, XPAR(NVAR+1), 1)
      ELSE
         DO I = 1,NVAR
            XPAR(MAP_P(I)) = COORD(I)
         END DO
         J = NVAR + 1
         DO I = 1,NODIM-NVAR
            XPAR(MAP_P(J)) = VARS(I)
            J = J + 1
         END DO
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C
C     ==================================================================
      SUBROUTINE CNVENV (DVARS, GRMAX, LCONV)
C     ==--------------------------------------------------------------==
C     ==  Tests if the environment is converged                       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'tpar.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'lscal.inc' ! parameters and pointers to work space
C     Arguments
      LOGICAL LCONV
      REAL*8  DVARS(NODIM-NVAR), GRMAX
C     Variables
      INTEGER I
      REAL*8  GRCUR
C     ==--------------------------------------------------------------==
      GRMAX = 0.0D0
      DO I = 1,NODIM-NVAR
         IF (ABS(DVARS(I)) .GT. GRMAX) THEN
            GRMAX = ABS(DVARS(I))
         END IF
      END DO
      LCONV = (GRMAX.LT.TOLENV)
C     ==--------------------------------------------------------------==
      RETURN
      END
C
C     ==================================================================
      SUBROUTINE CLCMAP (MAP_P, NDIM, NVAR, ICORE, NCORE)
C     ==--------------------------------------------------------------==
C     ==  Calculate the mapping MAP_P between the degrees of freedom  ==
C     ==  to be optimized (NODIM) and the ones of the reaction core   ==
C     ==  (NVAR) and the environment (NODIM-NVAR) according to the    ==
C     ==  atom list ICORE of the reaction core.                       ==
C     ==                                                              ==
C     ==  MAP_P(i_in_coord) = i_in_xpar (from RGMOPT)                 ==
C     ==  i_in_coord      = 1 .. NVAR:       index in COORD (core)    ==
C     ==  i_in_coord-NVAR = 1 .. NODIM-NVAR: index in VARS  (env.)    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cotr.inc'
C     Arguments
      INTEGER NDIM, NVAR, NCORE
      INTEGER MAP_P(NDIM), ICORE(NVAR/3)
C     Variables
      LOGICAL LENV
      INTEGER I, IA, IAT, IC, IE, IS, K, L
C     ==--------------------------------------------------------------==
      IAT = 0
      IC  = 0
      IE  = NVAR
      L   = 0
      DO IS = 1,NSP
         DO IA = 1,NA(IS)
            IAT = IAT + 1
C
C     ==  CHECK IF ATOM IAT IS CONTAINED IN ICORE(1..NCORE)           ==
C
            I = 1
            LENV = .TRUE.
 10         IF (LENV.AND.I.LE.NCORE) THEN
               LENV = (IAT.NE.ICORE(I))
               I = I + 1
               GO TO 10
            END IF
C
C     ==  REGISTER DEGREE OF FREEDOM EITHER FOR CORE OR ENVIRONMENT   ==
C
            DO K = 1,3
               IF (LSKCOR(K,IAT).NE.0) THEN
                  L = L + 1         ! L is index in XPAR
                  IF (LENV) THEN
                     IE = IE + 1
                     MAP_P(IE) = L
                  ELSE
                     IC = IC + 1
                     MAP_P(IC) = L
                  END IF
               ENDIF
            ENDDO
         ENDDO
      ENDDO
C
C     ==  FLAG INCONSISTENT INPUT                                     ==
C
      IF (IC.NE.NVAR .OR. IE.NE.NODIM) THEN
         WRITE (6,*) '  INCONSISTENT INPUT: ', NCORE, ' ATOMS IN CORE'
         WRITE (6,*) '  NUMBER OF DEGREES OF FREEDOM SPECIFIED: ', NVAR
         WRITE (6,*) '  NUMBER OF DEGREES OF FREEDOM FOUND:     ', IC
         CALL STOPGM ('CLCMAP', 'INCONSISTENT INPUT')
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C
C     ==================================================================
      SUBROUTINE GIVE_WORK_PRFO (NDIM, LRLBFGS, LRPRFO)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'lscal.inc'
C     Arguments
      INTEGER NDIM, LRLBFGS, LRPRFO
C     ==--------------------------------------------------------------==
      IF (NVAR.EQ.0) THEN         ! set NVAR to NDIM later at init
         LRPRFO  = 4*NDIM*NDIM + 8*NDIM
         LRLBFGS = 0
      ELSE IF (NVAR.EQ.NDIM) THEN ! Reaction core spans entire system
         LRPRFO  = 4*NVAR*NVAR + 8*NVAR
         LRLBFGS = 0
      ELSE IF (NVAR.LT.NDIM) THEN ! Reaction core spans a subsystem
         LRPRFO  = 4*NVAR*NVAR + 8*NVAR
         CALL GIVE_WORK_LBFGS (NDIM-NVAR, LRLBFGS)
      ELSE                        ! More variables than DOF
         WRITE (6,*) 'INVALID INPUT FOR TS SEARCH IN REACTION CORE'
         WRITE (6,*) 'NUMBER OF DEGREES OF FREEDOM OF SYSTEM: ', NDIM
         WRITE (6,*) 'NUMBER OF VARIABLES FOR TS SEARCH:      ', NVAR
         CALL STOPGM ('RPRFO', 'NUMBER OF DOF MISMATCH')
      END IF
      IF (.NOT.LALLHS) LRPRFO = LRPRFO - NVAR*NVAR ! Allocated in RV30
      LWPRFO = LRPRFO ! Store for straightforward writing of RESTART
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_RPRFO (LRPRFO, TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !PARENT MGDIIS
      INCLUDE 'cotr.inc'        !NODIM
      INCLUDE 'lscal.inc'       !NVAR,LWPRFO
C     Arguments
      INTEGER LRPRFO
      CHARACTER*30 TAG
C     ==--------------------------------------------------------------==
      IF (NVAR.EQ.0) THEN
         LRPRFO = 3*NODIM + 2*NODIM
      ELSE
         LRPRFO = 3*NVAR + 2*NODIM
      END IF
      TAG = '2*NVAR+2*NODIM'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PRFO_INIT
C     ==--------------------------------------------------------------==
C     == Block data: init variables for P-RFO and MI TS search        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'lscal.inc'
C     ==--------------------------------------------------------------==
C     Parameters
      MODE       = 1         ! Hessian eigenmode to be followed
      MODELK     = 0         ! Mode following by default
      NSMAXP     = -1        ! Maximum number of P-RFO steps
      NVAR       = 0         ! Number of degrees of freedom in core
      NSVIB      = 0         ! Never do vibrational analysis
      M_HESS     = 0         ! Finite-difference Hessian (INIHES)
      CUTEIG     = 1.0D-9    ! Cutoff for Hessian eigenvalues
      DEMIN_P    = 2.0D-5    ! Maximum energy change of a step
      OMIN       = 0.5D0     ! Minimum overlap of Hessian mode
      RMAX_P     = 1.0D+1    ! Maximum ratio for predicted energy
      RMIN_P     = 0.0D0     ! Minimum ratio for predicted energy
      STEP_INI_P = 0.4D-1    ! Initial trust radius
      STEP_MAX_P = 0.2D0     ! Maximum trust radius
      STEP_MIN_P = 1.0D-5    ! Minimum trust radius
      TOLENV     = 0.0D0     ! Tolerance for the environment
      TRUSTP     = 0.0D0     ! Input trust radius for P-RFO
      EPS_H      = 2.0D-2    ! Finite-difference displacement (Hessian)
C     Shared information
      LALLHS     = .TRUE.    ! Allocate Hessian in RGMOPT
      LMICRO     = .FALSE.   ! Conventional TS search
      LNOMAP     = .TRUE.    ! Default map for core / environment
      LVLHES     = .FALSE.   ! No valid Hessian yet
      LPRHES     = .FALSE.   ! Do not project R/T of Hessian
      LRSTRT     = .FALSE.   ! Did not start from RESTART
      LUNDOP     = .FALSE.   ! Do not undo the last Hessian step
      LWPRFO     = 0         ! No work space allocated yet
      LMAP_P     = 0         ! No core atoms map allocated yet
      NSTEP_P    = 0         ! Number of performed P-RFO steps
      NSTRIED_P  = 0         ! Number of tried P-RFO steps
      NSHESS     = 0         ! Current step for Hessian calculation
C     ==--------------------------------------------------------------==
      END
C     ==================================================================
      SUBROUTINE PVIBANA
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'        ! NAT
      INCLUDE 'cotr.inc'        ! HESS
      INCLUDE 'lscal.inc'
C     Variables
      INTEGER ND
      REAL*8  SDER(3*NAT,3*NAT)
      POINTER (IP_SDER,SDER)
C     ==--------------------------------------------------------------==
      ND = 3*NAT
      CALL MEMORY (IP_HESS, ND*ND, 'HESS')
      CALL MEMORY (IP_SDER, ND*ND, 'SDER')
      IF (NPARA.LT.2) NPARA = 2
      CALL PHESSCI (HESS, ND)
      CALL DCOPY (ND*ND, HESS, 1, SDER, 1)
      CALL VIBANA (SDER)
      CALL FREEM (IP_SDER)
      CALL FREEM (IP_HESS)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PHESSCI (HESS, NODIM)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'        ! NAT
      INCLUDE 'coor.inc'        ! TAU0
      INCLUDE 'lscal.inc'
C     Arguments
      INTEGER NODIM
      REAL*8  HESS(NODIM,NODIM)
C     Variables
      INTEGER I, IM, J, JM
      REAL*8  HSCR(2*3*NAT*3*NAT)
      POINTER (IP_HSCR,HSCR)
C     ==--------------------------------------------------------------==
      IF (.NOT.PARENT) RETURN
      IF (NPARA.GE.3) CALL MEMORY (IP_HSCR, 2*3*NAT*3*NAT, 'HSCR')
C     Create map core <-> active if not yet there
      IF (.NOT.LNOMAP .AND. LMAP_P.EQ.0) THEN
         CALL MEMORY (IP_MAP_P, NODIM, 'MAP_P')
         LMAP_P = NODIM
         CALL CLCMAP (MAP_P, NODIM, NVAR, ICORE, NCORE)
         CALL FREEM (IP_ICORE)
      END IF
C     Init the full Hessian with unit matrix
      IF (NPARA.EQ.2) THEN
         CALL UNITMX(HESS,NODIM)
C     Init the full Hessian with empirical Hessian
      ELSE IF (NPARA.GE.3) THEN
         NPARA = NPARA - 3
         CALL EMPFOR(TAU0,HSCR)
         CALL GETHES(HSCR)
         NPARA = NPARA + 3
      END IF
C     Partial Hessian is usually stored as upper triangular matrix
      DO I = 1,NVAR-1
         CALL DCOPY (NVAR-I, HESSCR(I,I+1), NVAR, HESSCR(I+1,I), 1)
      END DO
C     Check in partial Hessian
      IF (NVAR.EQ.NODIM) THEN
         CALL DCOPY (NVAR*NVAR, HESSCR, 1, HESS, 1)
      ELSE IF (LNOMAP) THEN
         DO I = 1,NVAR
            DO J = 1,NODIM
               HESS(I,J) = 0.0D0
               HESS(J,I) = 0.0D0
            END DO
         END DO
         DO I = 1,NVAR
            CALL DCOPY (NVAR, HESSCR(1,I), 1, HESS(1,I), 1)
         END DO
      ELSE
         DO I = 1,NVAR
            IM = MAP_P(I)
            DO J = 1,NODIM
               HESS(IM,J) = 0.0D0
               HESS(J,IM) = 0.0D0
            END DO
         END DO
         DO I = 1,NVAR
            IM = MAP_P(I)
            DO J = 1,NVAR
               JM = MAP_P(J)
               HESS(JM,IM) = HESSCR(J,I)
            END DO
         END DO
      END IF
      IF (NPARA.GE.3) CALL FREEM (IP_HSCR)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PHESSCO
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'        ! NAT
      INCLUDE 'cotr.inc'        ! HESS
      INCLUDE 'lscal.inc'
C     Variables
      INTEGER I, IM, J, JM
C     ==--------------------------------------------------------------==
      IF (.NOT.PARENT) RETURN
      IF (LNOMAP) THEN
         DO I = 1,NVAR
            CALL DCOPY (NVAR, HESS(1,I), 1, HESSCR(1,I), 1)
         END DO
      ELSE
         DO I = 1,NVAR
            IM = MAP_P(I)
            DO J = 1,NVAR
               JM = MAP_P(J)
               HESSCR(J,I) = HESS(JM,IM)
            END DO
         END DO
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE WR30PRFO (NW, IERROR, IRECL)
C     ==--------------------------------------------------------------==
C     ==  Writes P-RFO optimizer status to the RESTART file NW. The   ==
C     ==  partial Hessian has its own record. See also WR30LBFGS      ==
C     ==  Change this routine consistently with                       ==
C     ==  lscal.inc and RD30PRFO                                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'lscal.inc'
      INCLUDE 'cotr.inc'
C     Arguments
      INTEGER NW, IERROR, IRECL
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
C     NODIM is not yet determined when reading the RESTART file
      WRITE (UNIT=NW,IOSTAT=IERROR)
     &     NODIM
C     Input parameters (common blocks /PRFOPI/ and /PRFOPR/)
      WRITE (UNIT=NW,IOSTAT=IERROR)
     &     MODE, MODELK, NCORE, NSMAXP, NVAR
      WRITE (UNIT=NW,IOSTAT=IERROR)
     &     CUTEIG, DEMIN_P, OMIN, RMAX_P, RMIN_P, STEP_INI_P,
     $     STEP_MAX_P, STEP_MIN_P, TOLENV, TRUSTP
C     Shared information (common blocks /PRFOSL/, /PRFOSI/ and /PRFOSR/)
      WRITE (UNIT=NW,IOSTAT=IERROR)
     &     LALLHS, LLOCK, LMICRO, LNOMAP, LPRHES, LRSTRT, LUNDOP,
     $     LVLHES
      WRITE (UNIT=NW,IOSTAT=IERROR)
     &     LWPRFO, MODCUR, NSHESS, NSTEP_P, NSTRIED_P
      WRITE (UNIT=NW,IOSTAT=IERROR)
     &     DEPRED, OPRVEN, OSTEP_P, OSZ_P, PRVENE, STEP_P, SZ_P
C     Work space (common block /PRFOSP/)
      WRITE (UNIT=NW,IOSTAT=IERROR) (WPRFO(I), I=1,LWPRFO)
      IF (LMAP_P.GT.0) THEN
         WRITE (UNIT=NW,IOSTAT=IERROR) (MAP_P(I), I=1,LMAP_P)
      ELSE
         WRITE (UNIT=NW,IOSTAT=IERROR) 0
      END IF
C     Determine if L-BFGS history needs to be saved as well
      IF (NVAR.LT.NODIM) THEN
         IRECL = 1
      ELSE
         IRECL = 0
      END IF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE RD30PRFO (NR, IRECORD, IRECL)
C     ==--------------------------------------------------------------==
C     ==  Restart from checkpoint. See WR30PRFO                       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'lscal.inc'
C     Arguments
      INTEGER NR, IRECORD, IRECL
C     Variables
      LOGICAL LVLHSLOC
      INTEGER I, NDIMSV, NVARNP
C     ==--------------------------------------------------------------==
      NVARNP = NVAR
C     NODIM is not yet determined at this point
      READ (NR) NDIMSV
C     Input parameters (common blocks /PRFOPI/ and /PRFOPR/)
      READ (NR) MODE, MODELK, NCORE, NSMAXP, NVAR
      READ (NR) CUTEIG, DEMIN_P, OMIN, RMAX_P, RMIN_P, STEP_INI_P,
     $          STEP_MAX_P, STEP_MIN_P, TOLENV, TRUSTP
C     Check if RESTART file is consistent with current parameters
      IF (NVARNP.NE.NVAR) THEN
         WRITE (6,'(1X,A,A,I3)') 'RD30PRFO| WARNING! ',
     $        'NVAR ON CHECKPOINT: ', NVAR
         WRITE (6,'(1X,A,I3)') 'NVAR IN INPUT: ', NVARNP
         WRITE (6,'(1X,A)') 'NOT USING RESTART FILE FOR OPTIMIZATION'
         NVAR = NVARNP
         CALL FSKIP (NR, IRECORD-2)
C     Read L-BFGS status and history if microiterative optimization
         IRECL = 0
         RETURN
      ELSE IF (NVAR.LT.NDIMSV) THEN
         IRECL = 1
      ELSE
         IRECL = 0
      END IF
C     Shared information (common blocks /PRFOSL/, /PRFOSI/ and /PRFOSR/)
      READ (NR) LALLHS, LLOCK, LMICRO, LNOMAP, LPRHES, LRSTRT, LUNDOP,
     $          LVLHSLOC
      READ (NR) LWPRFO, MODCUR, NSHESS, NSTEP_P, NSTRIED_P
      READ (NR) DEPRED, OPRVEN, OSTEP_P, OSZ_P, PRVENE, STEP_P, SZ_P
C     Work space (common block /PRFOSP/)
      CALL MEMORY (IP_WPRFO, LWPRFO, 'WPRFO')
      READ (NR) (WPRFO(I), I=1,LWPRFO)
      IF (LMAP_P.GT.0) THEN
         CALL MEMORY (IP_MAP_P, LMAP_P, 'MAP_P')
         READ (NR) (MAP_P(I), I=1,LMAP_P)
      ELSE
         READ (NR) I
      END IF
C     Warn if no Hessian was read
      IF (LVLHSLOC.AND..NOT.LVLHES) THEN
         WRITE (6,'(1X,A,A)') 'RD30PRFO| WARNING! NO HESSIAN READ ',
     $        'FROM RESTART FILE BUT P-RFO STATUS REQUIRES ONE'
         WRITE (6,'(1X,A)') 'WILL RECALCULATE HESSIAN WHEN REQUIRED'
      END IF
C     Set pointers within work space and return codes to RGMOPT
      LRSTRT = .TRUE.
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
