C     ==================================================================
      SUBROUTINE RREADF
C     ==--------------------------------------------------------------==
C     ==   Read replica coordinates from file                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'pimd.inc'
CDM1
      INCLUDE 'adat.inc'
      INCLUDE 'fileopen.inc'
CDM2
C     Local variables
      INTEGER   IUNIT
      PARAMETER (IUNIT=34)
      REAL*8    TAUS(3,NAX,NSP)
      POINTER   (IP_TAUS,TAUS)
      CHARACTER*80 LINE
      INTEGER   IP,IA,IB,IE,IS,I,IAT,K,MSGLEN
      REAL*8    RNP,VAR,DISMAX,DISMIN,DIS
      LOGICAL   FERROR
CDM1
      REAL*8    RMEAN(3)
CDM2
C     ==--------------------------------------------------------------==
      IF(GRANDPARENT) THEN
        CALL XSTRING(REPFNAME,IB,IE)
        RNP=DBLE(NP_TOTAL)
        FERROR=.FALSE.
        CALL FILEOPEN(IUNIT,REPFNAME(IB:IE),FO_OLD,FERROR)
        IF(FERROR) CALL STOPGM('RREADF','CANNOT OPEN REPLICA FILE')
        REWIND(IUNIT)
CDM1
C       NOTE EMPTY LINE IN FRONT OF EVERY REPLICA BLOCK
CDM2
C       We use these empty lines to specify some keywords:
C       ANGSTROM BOHR SCALE
        DO IP=1,NP_TOTAL
          READ(IUNIT,'(A)') LINE
          DO IS=1,NSP
            DO IA=1,NA(IS)
              READ(IUNIT,*) (TREP(I,IA,IS,IP),I=1,3)
            ENDDO
          ENDDO
          IF(INDEX(LINE,'SCALE').NE.0) THEN
C           Scale options: atomic coordinates in direct lattice basis
            CALL MEMORY(IP_TAUS,3*NAX*NSP,'TAUS')
            CALL DCOPY(3*NAX*NSX,TREP(1,1,1,IP),1,TAUS,1)
            CALL S_TO_C(TAUS,TREP(1,1,1,IP))
            CALL FREEM(IP_TAUS)
          ELSEIF(INDEX(LINE,'ANGST').NE.0) THEN
C           Conversion into bohr
            DO IS=1,NSP
              DO IA=1,NA(IS)
                DO I=1,3
                  TREP(I,IA,IS,IP)=TREP(I,IA,IS,IP)*FBOHR
                ENDDO
              ENDDO
            ENDDO
          ENDIF
        ENDDO
        CALL FILECLOSE(IUNIT)
        WRITE(6,'(A,A,A,A)') ' REPLICA GEOMETRIES READ FROM FILE:',
     &       '"',REPFNAME(IB:IE),'"'
CDM1
C       Only for path integral calculations
        IF(TPIMD) THEN
C..check the radius of gyration and compare to free particle
C..(Gillan in Computer Modelling of Fluids Polymers and Solids 1990)
          WRITE(6,'(/,1X,63("*"))')
          WRITE(6,'(" *",61X,"*")')
          WRITE(6,'(" *",23X,A,23X,"*")') ' R OF GYRATION '
          WRITE(6,'(" *",61X,"*")')
          WRITE(6,'(1X,63("*"),/)')
          WRITE(6,'(A,A,A,A)') '   ATOM   R OF GYRATION',
     *         '  MAXDIS',
     *         '   MINDIS',
     *         '   (ANGSTROM)'
C         
          IAT=0
          DO IS=1,NSP
CDM         VARFP=(315777.0D0/(4.0D0*PMAR(IS)*TEMPB))
CDM         VARFP=DSQRT(VARFP)
            DO IA=1,NA(IS)
              DO K=1,3
                RMEAN(K)=0.0D0
                DO IP=1,NP_TOTAL
                  RMEAN(K)=RMEAN(K)+TREP(K,IA,IS,IP)
                ENDDO
                RMEAN(K)=RMEAN(K)/RNP
              ENDDO
              VAR=0.0D0
              DO IP=1,NP_TOTAL
                DO K=1,3
                  VAR=VAR+(TREP(K,IA,IS,IP)-RMEAN(K))**2
                ENDDO
              ENDDO
              VAR=DSQRT(VAR/RNP)
              DISMAX=-10.D0
              DISMIN=100000.0D0
              DO IP=1,NP_TOTAL
                IF(IP.EQ.NP_TOTAL) THEN
                  DIS=(TREP(1,IA,IS,IP)-TREP(1,IA,IS,1))**2
     *                 +(TREP(2,IA,IS,IP)-TREP(2,IA,IS,1))**2
     *                 +(TREP(3,IA,IS,IP)-TREP(3,IA,IS,1))**2
                ELSE
                  DIS=(TREP(1,IA,IS,IP)-TREP(1,IA,IS,IP+1))**2
     *                 +(TREP(2,IA,IS,IP)-TREP(2,IA,IS,IP+1))**2
     *                 +(TREP(3,IA,IS,IP)-TREP(3,IA,IS,IP+1))**2
                ENDIF
                DIS=DSQRT(DIS)
                IF(DIS.GT.DISMAX) DISMAX=DIS
                IF(DIS.LT.DISMIN) DISMIN=DIS
              ENDDO
C             
              IAT=IAT+1
              WRITE(6,'(I4,2X,A2,3X,F8.5,4X,F8.5,1X,F8.5)')
CDM           WRITE(6,'(I4,2X,A2,3X,F8.5,7X,F8.5,4X,F8.5,1X,F8.5)')
     *             IAT,EL(IATYP(IS)),
     *             VAR/FBOHR,DISMAX/FBOHR,DISMIN/FBOHR
CDM  *        VAR/FBOHR,VARFP0/FBOHR,DISMAX/FBOHR,DISMIN/FBOHR
            ENDDO
          ENDDO
          WRITE(6,*) '  '
        ENDIF                   !IF(TPIMD)
CDM2
      ENDIF                     !IF(GRANDPARENT)
      MSGLEN = 3*NAX*NSX*NP_TOTAL * 8
      CALL MY_BCAST(TREP,MSGLEN,SUPERSOURCE,SUPERGROUP)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
