C     ==================================================================
      SUBROUTINE RSCPOT(C0,TAU0,FION,RHOE,PSI,SCR,LSCR,
     &                  TFOR,TSTRESS,NSTATE,NKPOINT)
C     ==--------------------------------------------------------------==
C     == CALCULATE POT |PSI> AND THE ATOMIC FORCES (POT PART)         ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   C0(NGW,NSTATE) ELECTRONIC WAVEFUNCTIONS                    ==
C     ==   TAU0           ATOMIC COORDINATES                          ==
C     ==   LSCR           LENGTH OF SCRATCH ARRAY                     ==
C     ==   TFOR           .TRUE. CALCULATES THE ATOMIC FORCES         ==
C     ==   TSTRESS        .TRUE. CALCULATES THE STRESS TENSOR         ==
C     ==   NSTATE         NUMBER OF STATES                            ==
C     ==   NKPOINT        NUMBER OF K POINTS                          ==
C     == OUTPUT:                                                      ==
C     ==   FION      ATOMIC FORCES                                    ==
C     ==   RHOE      ELECTRONIC POTENTIAL                             ==
C     ==   PSI       USED FOR FFT                                     ==
C     ==   SCR(LSCR) SCRATCH ARRAY                                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'ropt.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'prop.inc'
      INCLUDE 'mm_input.inc'
      INCLUDE 'vdwc.inc'
      INCLUDE 'cnst_dyn.inc'

C     Arguments
      INTEGER    LSCR,NSTATE,NKPOINT
CEHR
      COMPLEX*16 C0(NGWK,*),PSI(NNR1)
CEHR
      REAL*8     TAU0(3*NAX*NSX),FION(3*NAX*NSX),RHOE(NNR1,NLSD),
     &           SCR(LSCR)
      LOGICAL    TFOR,TSTRESS
C     Variables
      REAL*8     E_MM
      DIMENSION  F(NSTATE,NKPOINT),WK(NKPOINT)
      CHARACTER  TAG*30
      logical    dorho
      INTEGER    NFTO,LRSCPOT,
     &           IPSTO,I
      DATA       IPSTO /0/
      SAVE       IPSTO
C     ==--------------------------------------------------------------==
C     SCR test
      CALL GIVE_SCR_RSCPOT(LRSCPOT,TAG,TSTRESS)
      CALL TEST_SCR('RSCPOT',TAG,LSCR,LRSCPOT)
C     ==--------------------------------------------------------------==
C     == Calculate NON-LOCAL PARTS of the energy and forces, which    ==
C     == depend on the wavefunctions themselves, and not on the       ==
C     == electronic density.                                          ==
C     ==--------------------------------------------------------------==
      CALL RNLRH(ENL,NSTATE,NKPOINT)
C     ==--------------------------------------------------------------==
C     == Compute the residual part of the ion-ion interaction (ESR)   ==
C     == due to the overlap of the smeared ionic charge densities     ==
C     == corresponding to different atoms.                            ==
C     ==--------------------------------------------------------------==
      NFTO=3*NAX*NSX
      CALL AZZERO(FION,NFTO)
      CALL RPIIINT(ESR,TAU0,FION,IESR,TFOR)
C     ==--------------------------------------------------------------==
C     == Compute the electronic density and kinetic energy            ==
C     ==--------------------------------------------------------------==
      IF(TKPNT) THEN
        CALL RHOOFR_C(C0,RHOE,PSI,SCR,LSCR,NSTATE)
      ELSE
        dorho=.not.qmmm .or. (qmmm.and.BSYMM)
     &        .or.(qmmm .and. coupl_model.eq.0)
        if(dorho) then
          CALL RHOOFR(C0,RHOE,PSI,SCR,LSCR,NSTATE)
        endif
      ENDIF
      if((teprefg.and.tlsd).or.tefg) then
        call save_rho(rhoe)
      endif

C     ==--------------------------------------------------------------==
C     == Calculate the Stress Tensor                                  ==
C     ==--------------------------------------------------------------==
      IF(TSTRESS) THEN
        CALL STRESS(C0,TAU0,F,PSI,SCR,LSCR,NSTATE)
      ENDIF
C     ==--------------------------------------------------------------==
C     == Compute the LOCAL PARTS of the energy and the forces, which  ==
C     == depend exclusively on the electronic density.                ==
C     ==--------------------------------------------------------------==
      IF(CDFT.AND.TFOR) CALL CDFT_FORCES(FION,TAU0,RHOE)
      CALL VOFRHO(TAU0,FION,RHOE,PSI,SCR,LSCR,TFOR,TSTRESS)
C     ==--------------------------------------------------------------==
C     == Force on the ions from nonlocal pseudopotential              ==
C     ==--------------------------------------------------------------==
      IF(TFOR) THEN
        CALL RNLFOR(FION,F,WK,NSTATE,NKPOINT)
      ENDIF
C     ==--------------------------------------------------------------==
C     == Energy and force on the ions for empirical van der Waals     ==
C     ==--------------------------------------------------------------==
      EVDW=0.D0
      IF(VDWC) THEN
        CALL VDW(TAU0,NVDW,IDVDW,IVDW,JVDW,VDWST,VDWRM,VDWBE,
     &           VDWEPS,S6GRIM,NXVDW,NYVDW,NZVDW,EVDW,FION,DEVDW)
      ENDIF
C     ==--------------------------------------------------------------==
C     == QM/MM of Roethlisberger group                                ==
C     ==--------------------------------------------------------------==
      E_MM=0.D0
#if defined (__QMECHCOUPL)
      call mm_cpmd_MM_energy(E_MM)
      IF(TFOR) THEN
         CALL mm_cpmd_add_MM_forces_f77(FION)
      END IF
#endif
C     ==--------------------------------------------------------------==
C     == TOTAL ENERGY                                                 ==
C     ==--------------------------------------------------------------==
      EEIG = 0.D0
      ETOT = EKIN + EHT + EPSEU + ENL + EXC + E_MM
     .     + EVDW   ! Empirical van der Waals correction
#ifdef PARALLEL
C     Sum ETOT,EKIN,EPSEU,ENL,EHT,EHEP,EHEE,EHII,EXC,VXC,EGC
      CALL GLOSUM(11,ETOT)
#endif
C     == Metadynamics : spin den localization                         ==
      IF(TLOCALIZESPIN) THEN
        ETOT = ETOT + EN_HARM_LOCS
      ENDIF
      IF(TLSE .AND. TCAS22) THEN
        IF(DABS(E_MM).GT.1.D-10) THEN
          IF(PARENT) THEN
              WRITE(6,*) " ******************************************"
              WRITE(6,*) " WARNING: MM ENERGY CONTRIBUTION NOT TESTED"
              WRITE(6,*) " ******************************************"
          ENDIF
        ENDIF
        CALL GLOSUM(21,ETOT_A)
        CALL GLOSUM(3,ETOT_B)
        ETOT_A = EKIN_A+EPSEU_A+ENL_A+EHT_A+ESR-ESELF+EXC_A+
     &           (E_MM-EEXT+EEXT_A)
        ETOT_2 = EKIN_2+EPSEU_2+ENL_2+EHT_2+ESR-ESELF+EXC_2+
     &           (E_MM-EEXT+EEXT_2)
        ETOT_B = ETOT
        ETOT_AB = EKIN_AB+EPSEU_AB+ENL_AB+EHT_AB+EXC_AB+EEXT_AB
        IF(ETOT_B .LT. ETOT_A) THEN
          IF(PARENT) THEN
            WRITE(6,*) " ******************************************"
            WRITE(6,*) " WRONG ORDER OF STATES: NOW SWITCH A AND B"
            WRITE(6,*) " ******************************************"
            CALL DSWAP(2*NGW,C0(1,IALPHA),1,C0(1,IBETA),1)
          ENDIF
        ENDIF
        CASANG = 0.5D0*ATAN(2.D0*ETOT_AB/(ETOT_A-ETOT_B))
        ECAS   = DSIN(CASANG)**2 * ETOT_A + DCOS(CASANG)**2 * ETOT_B -
     &           2.D0*DSIN(CASANG)*DCOS(CASANG) * ETOT_AB
        ECAS   = ECAS - ETOT_B
        ETOT   = ETOT_B + ECAS
        ETOT_T = ETOT_B - EXC + ETOT_T
C       Now get the final potentials and forces
        CALL VCOMP(RHOE(1,1),RHOE(1,4))
        IF(TFOR) THEN
          CALL CTFOR(FION)
        ENDIF
      ELSEIF(TLSE .AND. TPENAL) THEN
        CALL GLOSUM(1,EKIN_AB)
        CALL GLOSUM(1,EHT_AB)
        ETOT_AB = EKIN_AB + EHT_AB
        ETOT = ETOT + ETOT_AB*ETOT_AB
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_RSCPOT(LRSCPOT,TAG,TSTRESS)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   LRSCPOT
      CHARACTER TAG*30
      LOGICAL   TSTRESS
C     Arguments
      INTEGER   LRHOOFR,LSTRESS,LVOFRHO
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_RHOOFR(LRHOOFR,TAG)
      IF(TSTRESS) THEN
        CALL GIVE_SCR_STRESS(LSTRESS,TAG)
      ELSE
        LSTRESS=0
      ENDIF
      CALL GIVE_SCR_VOFRHO(LVOFRHO,TAG)
      LRSCPOT=MAX(LRHOOFR,LSTRESS,LVOFRHO)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
