C     ==================================================================
C     == ROUTINES TO MANAGE SWAP FILES                                ==
C     == USED FOR K POINTS (BLOCK OPTIONS)                            ==
C     ==--------------------------------------------------------------==
C     == IF you need to add an array for k points, you have to change ==
C     == three routines:                                              ==
C     ==  - WKPT_SWAP                                                 ==
C     ==  - RKPT_SWAP                                                 ==
C     ==  - KPT_NOSWAP                                                ==
C     == and the include file swap.inc.                               ==
C     ==--------------------------------------------------------------==
      SUBROUTINE WKPT_SWAP(C0,NSTATE,IKPT,TAG)
C     ==--------------------------------------------------------------==
C     == PERFORMS SWAP FOR K-POINTS CALCULATION (WRITE)               ==
C     == IFILE=1  C0  DIRECT ACCESS                                   ==
C     == IFILE=2  HGKP HGKM TWNL EMK2 MASKGW SEQUENTIAL ACCESS        ==
C     ==          INITIALIZED ONCE                                    ==
C     == IFILE=3  ALM                        SEQUENTIAL ACCESS        ==
C     ==          REINITIALIZED FOR EACH MOVE OF IONS COORDINATES     ==
C     ==--------------------------------------------------------------==
C     == TSWCALC  Calculate array depending on k points               ==
C     ==          Use IFILE=1 for C0                                  ==
C     ==          ISWACCE(:,:,ISW_CALC) is used to count the number   ==
C     ==                 of calculation                               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !for NKPNT,NGWK
      INCLUDE 'elct.inc'        !for N
      INCLUDE 'cppt.inc'        !for TWNL
      INCLUDE 'ions.inc'        !for NAT
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'        !for ALM and EMK2
      INCLUDE 'swap.inc'        !for initialization
      INCLUDE 'kpts.inc'        !for NKPBL
      INCLUDE 'kpnt.inc'        !for EIGKR,HGKP,HGKM
      INCLUDE 'sphe.inc'        !for MASKGW,MASKL
C     Arguments
      INTEGER    NSTATE,IKPT
      COMPLEX*16 C0(NGWK,NSTATE,NKPNT)
      CHARACTER  TAG*(*)
C     Variables
      CHARACTER LINE*(80)
      DIMENSION HGKP(NHG,NKPNT),HGKM(NHG,NKPNT),
     &          TWNL(NGWK,NHXS,NSX,NKPNT),EMK2(NGWK,NKPNT),
     &          EIGKR(NGWK,NAT,NKPNT),MASKGW(2,MASKL,NKPNT)
      INTEGER   I1,NKPOINT
C     ==--------------------------------------------------------------==
      I1=LEN(TAG)
      LINE=TAG(1:I1)
      NKPOINT=NKPBL(IKPT)
C     ==--------------------------------------------------------------==
      IF(ISWFIRST.EQ.0) THEN
        CALL INI_SWAP(0)
        IF(TSWAPC0) THEN
          CALL DIRECT_SWAP(1,2*NGWK*N*NKPNT)
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      IF(TKALL) THEN
        IF(INDEX(LINE,'HGKP').NE.0)
     &       CALL WRI_SWAP(2,HGKP,NHG*NKPOINT,IKPT,'HGKP')
        IF(INDEX(LINE,'HGKM').NE.0)
     &       CALL WRI_SWAP(2,HGKM,NHG*NKPOINT,IKPT,'HGKM')
        IF(TSPHERE.AND.INDEX(LINE,'MASKGW').NE.0)
     &       CALL WRI_SWAP(2,MASKGW,2*MASKL*NKPOINT,IKPT,'MASKGW')
        IF(INDEX(LINE,'TWNL').NE.0)
     &       CALL WRI_SWAP(2,TWNL,NGWK*NHXS*NSX*NKPOINT,IKPT,'TWNL')
        IF(TTROT.AND.INDEX(LINE,'EMK2').NE.0)
     &       CALL WRI_SWAP(2,EMK2,NGWK*NKPOINT,IKPT,'EMK2')
        IF(INDEX(LINE,'EIGKR').NE.0)
     &       CALL KPT_NOSWAP(IKPT,1,'EIGKR')
        IF(TTROT.AND.INDEX(LINE,'ALM').NE.0)
     &       CALL WRI_SWAP(3,ALM,IMAGP*NLM*NLM*NKPOINT,IKPT,'ALM')
      ELSEIF(TSWCALC) THEN
C       Calculate each time. Writing is doing nothing.
      ELSE
        CALL KPT_NOSWAP(IKPT,1,LINE)
      ENDIF
C     ==--------------------------------------------------------------==
      IF(INDEX(LINE,'C0').NE.0) THEN
        IF(TSWAPC0) THEN
          CALL WRI_SWAP(1,C0,2*NGWK*NSTATE*NKPOINT,IKPT,'C0')
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE RKPT_SWAP(C0,NSTATE,IKPT,TAG)
C     ==--------------------------------------------------------------==
C     == PERFORMS SWAP FOR K-POINTS CALCULATION (READ)                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'        !TWNL
      INCLUDE 'ions.inc'        !NAT
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'        !ALM
      INCLUDE 'kpts.inc'        !NKPBL
      INCLUDE 'kpnt.inc'        !EIGKR HGKP HGKM
      INCLUDE 'sphe.inc'        !MASKGW
      INCLUDE 'swap.inc'        !ISWACCE
C     Arguments
      INTEGER    NSTATE,IKPT
      COMPLEX*16 C0(NGWK,NSTATE,NKPNT)
      CHARACTER  TAG*(*)
C     Variables
      DIMENSION  ISWMEM(ISWMAX,ISWFILEMAX),
     &           ISWACCE(2,NSWBLOCK*ISWMAX,ISWFILEMAX)
      DIMENSION  HGKP(NHG,NKPNT),HGKM(NHG,NKPNT),EIGKR(NGWK,NAT,NKPNT),
     &           TWNL(NGWK,NHXS,NSX,NKPNT),MASKGW(2,MASKL,NKPNT)
      CHARACTER  LINE*(80)
      INTEGER    I1,NKPOINT
C     ==--------------------------------------------------------------==
      I1=LEN(TAG)
      LINE=TAG(1:I1)
      NKPOINT=NKPBL(IKPT)
C     ==--------------------------------------------------------------==
      IF(TKALL) THEN
        IF(INDEX(LINE,'HGKP').NE.0)
     &       CALL REA_SWAP(2,HGKP,NHG*NKPOINT,IKPT,'HGKP')
        IF(INDEX(LINE,'HGKM').NE.0)
     &       CALL REA_SWAP(2,HGKM,NHG*NKPOINT,IKPT,'HGKM')
        IF(TSPHERE.AND.INDEX(LINE,'MASKGW').NE.0)
     &       CALL REA_SWAP(2,MASKGW,2*MASKL*NKPOINT,IKPT,'MASKGW')
        IF(INDEX(LINE,'TWNL').NE.0)   
     &       CALL REA_SWAP(2,TWNL,NGWK*NHXS*NSX*NKPOINT,IKPT,'TWNL')
        IF(TTROT.AND.INDEX(LINE,'EMK2').NE.0)
     &       CALL REA_SWAP(2,EMK2,NGWK*NKPOINT,IKPT,'EMK2')
        IF(INDEX(LINE,'EIGKR').NE.0)
     &       CALL CALC_EIGKR(IKPT)
        IF(TTROT.AND.INDEX(LINE,'ALM').NE.0)
     &       CALL REA_SWAP(3,ALM,IMAGP*NLM*NLM*NKPOINT,IKPT,'ALM')
C       ==------------------------------------------------------------==
      ELSEIF(TSWCALC) THEN
 1000   CONTINUE
C       Calculate each time we need.
        IF(INDEX(LINE,'HGKP').NE.0.OR.INDEX(LINE,'HGKM').NE.0) THEN
          IF(ISWMEM(ISW_HGKP,ISW_CALC).NE.IKPT) THEN
            ISWACCE(2,ISW_HGKP,ISW_CALC)=ISWACCE(2,ISW_HGKP,ISW_CALC)+1
            ISWACCE(2,ISW_HGKM,ISW_CALC)=ISWACCE(2,ISW_HGKM,ISW_CALC)+1
            CALL CALC_HGK(IKPT)
            ISWMEM(ISW_HGKP,ISW_CALC)=IKPT
            ISWMEM(ISW_HGKM,ISW_CALC)=IKPT
          ENDIF
        ENDIF
C       Need HGKM and HGKP before.
        IF(TSPHERE.AND.INDEX(LINE,'MASKGW').NE.0) THEN
          IF(ISWMEM(ISW_MASKGW,ISW_CALC).NE.IKPT) THEN
            IF(ISWMEM(ISW_HGKP,ISW_CALC).NE.IKPT) THEN
C             Add HGKP calculation.
              WRITE(LINE,'(A,A)') LINE(1:I1),' HGKP'
              I1=I1+5
              GOTO 1000
            ENDIF
            ISWACCE(2,ISW_MASKGW,ISW_CALC)=
     &           ISWACCE(2,ISW_MASKGW,ISW_CALC)+1
            CALL CALC_MASKGW(IKPT)
            ISWMEM(ISW_MASKGW,ISW_CALC)=IKPT
          ENDIF
        ENDIF
C       Need HGKM and HGKP and MASKGW before.
        IF(INDEX(LINE,'TWNL').NE.0) THEN
          IF(ISWMEM(ISW_TWNL,ISW_CALC).NE.IKPT) THEN
            IF(ISWMEM(ISW_HGKP,ISW_CALC).NE.IKPT) THEN
C             Add HGKP calculation.
              WRITE(LINE,'(A,A)') LINE(1:I1),' HGKP'
              I1=I1+5
              GOTO 1000
            ENDIF
            IF(ISWMEM(ISW_MASKGW,ISW_CALC).NE.IKPT) THEN
C             Add MASKGW calculation.
              WRITE(LINE,'(A,A)') LINE(1:I1),' MASKGW'
              I1=I1+8
              GOTO 1000
            ENDIF
            ISWACCE(2,ISW_TWNL,ISW_CALC)=ISWACCE(2,ISW_TWNL,ISW_CALC)+1
            CALL CALC_TWNL(IKPT)
            ISWMEM(ISW_TWNL,ISW_CALC)=IKPT
          ENDIF
        ENDIF
C       Need HGKM and HGKP before.
        IF(TTROT.AND.INDEX(LINE,'EMK2').NE.0) THEN
          IF(ISWMEM(ISW_EMK2,ISW_CALC).NE.IKPT) THEN
            IF(ISWMEM(ISW_HGKP,ISW_CALC).NE.IKPT) THEN
C             Add HGKP calculation.
              WRITE(LINE,'(A,A)') LINE(1:I1),' HGKP'
              I1=I1+5
              GOTO 1000
            ENDIF
            ISWACCE(2,ISW_EMK2,ISW_CALC)=ISWACCE(2,ISW_EMK2,ISW_CALC)+1
            CALL CALC_EMK2(IKPT)
            ISWMEM(ISW_EMK2,ISW_CALC)=IKPT
          ENDIF
        ENDIF
        IF(INDEX(LINE,'EIGKR').NE.0) THEN
          IF(ISWMEM(ISW_EIGKR,ISW_CALC).NE.IKPT) THEN
            ISWACCE(2,ISW_EIGKR,ISW_CALC)=
     &           ISWACCE(2,ISW_EIGKR,ISW_CALC)+1
            CALL CALC_EIGKR(IKPT)
            ISWMEM(ISW_EIGKR,ISW_CALC)=IKPT
          ENDIF
        ENDIF
C       Need TWNL and EIGKR before.
        IF(TTROT.AND.INDEX(LINE,'ALM').NE.0) THEN
          IF(ISWMEM(ISW_ALM,ISW_CALC).NE.IKPT) THEN
            IF(ISWMEM(ISW_HGKP,ISW_CALC).NE.IKPT) THEN
C             Add HGKP calculation.
              WRITE(LINE,'(A,A)') LINE(1:I1),' HGKP'
              I1=I1+5
              GOTO 1000
            ENDIF
            IF(ISWMEM(ISW_EIGKR,ISW_CALC).NE.IKPT) THEN
C             Add EIGKR calculation.
              WRITE(LINE,'(A,A)') LINE(1:I1),' EIGKR'
              I1=I1+6
              GOTO 1000
            ENDIF
            ISWACCE(2,ISW_ALM,ISW_CALC)=ISWACCE(2,ISW_ALM,ISW_CALC)+1
            CALL CALC_K_ALM(IKPT)
            ISWMEM(ISW_ALM,ISW_CALC)=IKPT
          ENDIF
        ENDIF
C       ==------------------------------------------------------------==
      ELSE
        CALL KPT_NOSWAP(IKPT,0,LINE)
      ENDIF
C     ==--------------------------------------------------------------==
      IF(INDEX(LINE,'C0').NE.0) THEN
        IF(TSWAPC0) THEN
          CALL REA_SWAP(1,C0,2*NGWK*NSTATE*NKPOINT,IKPT,'C0')
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE KPT_NOSWAP(IKPT,IFLAG,TAG)
C     ==--------------------------------------------------------------==
C     == DO NOT USE SWAP FILE, CHANGE ADDRESSES OF POINTER            ==
C     == IFLAG=1 WRITE                                                ==
C     ==       0 READ                                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'        !for TWNL and YLMB
      INCLUDE 'ions.inc'        !for NAT
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'        !for ALM and EMK2
      INCLUDE 'sfac.inc'        !for EIGR
      INCLUDE 'kpts.inc'        !for NKPBL
      INCLUDE 'kpnt.inc'        !for EIGKR,HGKP,HGKM,EIKR
      INCLUDE 'sphe.inc'        !for MASKGW
C     Arguments
      INTEGER   IKPT,IFLAG
      CHARACTER TAG*(*)
C     Variables
      DIMENSION HGKP(NHG,NKPNT),HGKM(NHG,NKPNT),EIGR(NGW,NAT),
     &          TWNL(NGWK,NHXS,NSX,NKPNT),EMK2(NGWK,NKPNT),
     &          EIGKR(NGWK,NAT,NKPNT),EIKR(NKPTS,NAT),
     &          MASKGW(2,MASKL,NKPNT)
      REAL*8    FNL(*),DFNL(*)
      INTEGER   LEIGKR(0:NBLKP),LALM(0:NBLKP),
     &          LHGKP(0:NBLKP),LHGKM(0:NBLKP),
     &          LTWNL(0:NBLKP),LEMK2(0:NBLKP),
     &          LMASKGW(0:NBLKP)
      POINTER   (IP_LEIGKR,LEIGKR),(IP_LALM,LALM),
     &          (IP_LHGKP,LHGKP),(IP_LHGKM,LHGKM),
     &          (IP_LTWNL,LTWNL),(IP_LEMK2,LEMK2),
     &          (IP_LMASKGW,LMASKGW)
      CHARACTER LINE*(80)
      INTEGER    I1,NBLKP1
      INTEGER   I_FIRST
      DATA      I_FIRST /0/
      SAVE      I_FIRST,
     &          IP_LEIGKR,IP_LALM,IP_LHGKP,IP_LHGKM,IP_LTWNL,IP_LEMK2
C     ==--------------------------------------------------------------==
      I1=LEN(TAG)
      LINE=TAG(1:I1)
      IF(I_FIRST.EQ.0) THEN
        NBLKP1=NBLKP+1
        IF(.NOT.TKALL) THEN
          IF(TTROT) THEN
            CALL MEMORY(IP_LALM,NBLKP1,'LALM')
            CALL IAZZERO  (LALM,NBLKP1)
            LALM(0)=1
          ENDIF
          CALL MEMORY(IP_LHGKP,NBLKP1,'LHGKP')
          CALL IAZZERO  (LHGKP,NBLKP1)
          LHGKP(0)=1
          CALL MEMORY(IP_LHGKM,NBLKP1,'LHGKM')
          CALL IAZZERO  (LHGKM,NBLKP1)
          LHGKM(0)=1
          CALL MEMORY(IP_LTWNL,NBLKP1,'LTWNL')
          CALL IAZZERO  (LTWNL,NBLKP1)
          LTWNL(0)=1
          CALL MEMORY(IP_LEMK2,NBLKP1,'LEMK2')
          CALL IAZZERO  (LEMK2,NBLKP1)
          LEMK2(0)=1
          IF(TSPHERE) THEN
            CALL MEMORY(IP_LMASKGW,NBLKP1,'LMASKGW')
            CALL IAZZERO  (LMASKGW,NBLKP1)
            LMASKGW(0)=1
          ENDIF
        ENDIF
        CALL MEMORY(IP_LEIGKR,NBLKP1,'LEIGKR')
        CALL IAZZERO  (LEIGKR,NBLKP1)
        LEIGKR(0)=1
        I_FIRST=1
      ENDIF
      IF(.NOT.TKALL) THEN
        IF(INDEX(LINE,'HGKP').NE.0)
     &     CALL PT_CHANGE(IP_HGKP,LHGKP,IKPT,NHG*NKPNT,IFLAG)
        IF(INDEX(LINE,'HGKM').NE.0)
     &     CALL PT_CHANGE(IP_HGKM,LHGKM,IKPT,NHG*NKPNT,IFLAG)
        IF(INDEX(LINE,'TWNL').NE.0)
     &     CALL PT_CHANGE(IP_TWNL,LTWNL,IKPT,NGWK*NHXS*NSX*NKPNT,IFLAG)
        IF(TTROT) THEN
          IF(INDEX(LINE,'EMK2').NE.0)
     &     CALL PT_CHANGE(IP_EMK2,LEMK2,IKPT,NGWK*NKPNT,IFLAG)
          IF(INDEX(LINE,'ALM').NE.0)
     &     CALL PT_CHANGE(IP_ALM,LALM,IKPT,IMAGP*NLM*NLM*NKPNT,IFLAG)
        ENDIF
        IF(TSPHERE.AND.INDEX(LINE,'MASKGW').NE.0) THEN
          CALL PT_CHANGE(IP_MASKGW,LMASKGW,IKPT,
     &         2*MASKL*NKPNT,IFLAG)
        ENDIF
      ENDIF
      IF(INDEX(LINE,'EIGKR').NE.0) THEN
        IF(IFLAG.EQ.0) THEN
          IF(LEIGKR(0).NE.IKPT) THEN
            CALL CALC_EIGKR(IKPT)
            LEIGKR(0)=IKPT
          ENDIF
        ELSE
          LEIGKR(0)=IKPT
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE INI_SWAP(IFILE)
C     ==================================================================
C     == INITIALIZES SWAP FILES                                       ==
C     ==================================================================
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'envj.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'kpts.inc'        !TKBLOCK TKNOSWAP TKBCALC
      INCLUDE 'swap.inc'
C     Arguments
      INTEGER   IFILE
C     Variables
      DIMENSION IUNITSW(*),ISWCUR(*),ISWEND(*),ISWREC(*),
     &          ISIDISW(*),ISWRES(*)
      DIMENSION ISWMEM(*),ISWPOS(*),ISWSIZE(*),ISWACCE(*)
      INTEGER   ISUB,LENTMP,IIISW,NSW,IISW,I1,I2,I,IRECORDSIZE
      CHARACTER ACFILE*1
C     ==--------------------------------------------------------------==
      CALL TISET('  INI_SWAP',ISUB)
C     ==--------------------------------------------------------------==
      IF(IFILE.EQ.0) THEN
        IF(ISWFIRST.NE.0) THEN
          CALL STOPGM(' INI_SWAP',' SWAP ALREADY INITIALIZED')
        ENDIF
        ISWFIRST=1
        IF(TKBLOCK) THEN
          NSWBLOCK=NBLKP
        ELSE
          NSWBLOCK=1
        ENDIF
C       Initialization of list of data in swap files.
        IIISW=ISWFILEMAX
        NSW  =ISWFILEMAX*ISWMAX
        IISW =ISWFILEMAX*ISWMAX*NSWBLOCK
        CALL MEMORY(IP_IUNITSW,IIISW,              'IUNITSW')
        CALL MEMORY(IP_ISIDISW,IIISW/IRAT+1,       'ISIDISW')
        CALL MEMORY(IP_ISWRES,IIISW/IRAT+1,        'ISWRES')
        CALL MEMORY(IP_ISWMEM,NSW/IRAT+1,          'ISWMEM')
        CALL MEMORY(IP_ISWPOS,IISW/IRAT+1,         'ISWPOS')
        CALL MEMORY(IP_ISWSIZE,IISW/IRAT+1,        'ISWSIZE')
        CALL MEMORY(IP_ISWACCE,2*IISW/IRAT+1,      'ISWACCE')
        CALL MEMORY(IP_ISWCUR,IIISW/IRAT+1,        'ISWCUR')
        CALL MEMORY(IP_ISWEND,IIISW/IRAT+1,        'ISWEND')
        CALL MEMORY(IP_ISWREC,IIISW/IRAT+1,        'ISWREC')
        CALL IAZZERO(ISWMEM,NSW)
        CALL IAZZERO(ISWPOS,IISW)
        CALL IAZZERO(ISWSIZE,IISW)
        CALL IAZZERO(ISWACCE,2*IISW)
        LENTMP=INDEX(TMPDIR, ' ') - 1
        WRITE(SMYPID,'(I6)') MY_PID
        CALL XSTRING(SMYPID,I1,I2)
        SMYPID=SMYPID(I1:I2)
        DO I=1,ISWFILEMAX
          WRITE(SWFILE(I),'(A,A,I1,A,A)')
     &         TMPDIR(1:LENTMP),'/SWAP_',I,'.',SMYPID
          ISWCUR(I)=1
          ISWEND(I)=0
          ISWREC(I)=0
          IUNITSW(I)=0
          ACCESSW(I)='S'
          ISIDISW(I)=0
          ISWRES(I)=0
        ENDDO
        TSWCALC=TKBCALC
        TSWAPC0=.NOT.TKNOSWAP
        CALL TIHALT('  INI_SWAP',ISUB)
        RETURN
      ENDIF
C     ==--------------------------------------------------------------==
C     Special case if we want to calculate k point per k point(TKNOSWAP)
      IF(IFILE.EQ.1.AND.(.NOT.TSWAPC0)) THEN
C       Do nothing (no swap file for C0).
        CALL TIHALT('  INI_SWAP',ISUB)
        RETURN
      ENDIF
C     ==--------------------------------------------------------------==
C     Open swap files
      ACFILE=ACCESSW(IFILE)
      IF(ACFILE.EQ.'S') THEN
        IUNITSW(IFILE)=50+IFILE
        OPEN(UNIT=IUNITSW(IFILE),FILE=SWFILE(IFILE),ACCESS='SEQUENTIAL',
     &       FORM='UNFORMATTED',STATUS='NEW')
      ELSEIF(ACFILE.EQ.'D') THEN
        IF(ISIDISW(IFILE).LE.0)
     &       CALL STOPGM('INI_SWAP','SIZE OF RECORD NOT SPECIFIED')
        IUNITSW(IFILE)=50+IFILE
C       WARNING: THE RECORDSIZE IS MACHINE/COMPILER-DEPENDENT!!
#if defined(__SUN) || defined(__PGI) || defined(LINUX_IFC) || defined(__IBM) || defined(__HP) || defined(__OSX) || defined(__OSX_IFC) || defined(__WINNT)
C       these machines specify the length in number of characters.
        IRECORDSIZE=8*ISIDISW(IFILE)
#elif defined(__SGI) || defined(__alpha)
C       these machines specify the length in number of real words.
        IRECORDSIZE=2*ISIDISW(IFILE)
#else
        WRITE(6,'(/,2A,/,2A)') ' WARNING| RECORDSIZE MULTIPLIER FOR',
     &   ' DIRECT ACCESS SWAP FILES NOT KNOWN.',
     &   ' WARNING| USING DEFAULT OF 2.',
     &   ' CHANGE IN FILE rwswap.F IF NEEDED.'
        IRECORDSIZE=2*ISIDISW(IFILE)
#endif        
        OPEN(UNIT=IUNITSW(IFILE),FILE=SWFILE(IFILE),
     &       ACCESS='DIRECT',RECL=IRECORDSIZE,
     &       FORM='UNFORMATTED',STATUS='NEW')
      ELSE
        CALL STOPGM('INI_SWAP','ACCESS SEQUENTIAL OR DIRECT')
      ENDIF
      CALL TIHALT('  INI_SWAP',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE DIRECT_SWAP(IFILE,ISIZE)
C     ==--------------------------------------------------------------==
C     == SPECIFIED FOR A FILE THE DIRECT ACCESS AND THE SIZE OF RECORD==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'swap.inc'
C     Arguments
      INTEGER   IFILE,ISIZE
C     Variables
      DIMENSION IUNITSW(ISWFILEMAX),ISIDISW(ISWFILEMAX)
      INTEGER   IUNIT
C     ==--------------------------------------------------------------==
      IF((ISWFIRST.EQ.0).OR.(IFILE.GT.ISWFILEMAX)) THEN
        WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
        CALL STOPGM(' DIRECT_SWAP','SWAP FILE DOESN''T EXIST!!')
      ENDIF
      IUNIT=IUNITSW(IFILE)
      IF(IUNIT.EQ.0) THEN
        ACCESSW(IFILE)='D'
        IF(ISIZE.LE.0) THEN
          WRITE(6,*) ' SIZE OF RECORD=',ISIZE
          CALL STOPGM(' DIRECT_SWAP',
     &         'SIZE OF RECORD HAS TO BE POSITIVE')
        ELSE
          ISIDISW(IFILE)=ISIZE
        ENDIF
      ELSE
        WRITE(6,*) ' SWAP FILE NUMBER:',IFILE,' IS ALREADY INITIALIZED'
        CALL STOPGM(' DIRECT_SWAP','DIRECT ACCESS IMPOSSIBLE')
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE WRI_SWAP(IFILE,A,NA,IKPT,NAME)
C     ==--------------------------------------------------------------==
C     == WRITE IN SWAP FILES                                          ==
C     == IFILE=1 SWAP PERMANENT FILE                                  ==
C     ==       2 SWAP TEMPORARY FILE                                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'swap.inc'
      INTEGER IFILE,NA,IKPT
      CHARACTER*(*) NAME
      REAL*8  A(NA)
      DIMENSION ISWMEM(ISWMAX,ISWFILEMAX),
     &     ISWPOS(NSWBLOCK,ISWMAX,ISWFILEMAX),
     &     ISWSIZE(NSWBLOCK*ISWMAX,ISWFILEMAX),
     &     ISWACCE(2,NSWBLOCK*ISWMAX,ISWFILEMAX),IUNITSW(ISWFILEMAX),
     &     ISWCUR(ISWFILEMAX),ISWEND(ISWFILEMAX),ISWREC(ISWFILEMAX),
     &     ISIDISW(ISWFILEMAX)
C     Variables
      CHARACTER*8 SNAME, ACFILE*1
      INTEGER IN1,IN2,IS1,IS2,ISUB,IUNIT,INDI,IPOSITION
C     ==--------------------------------------------------------------==
      CALL TISET('  WRI_SWAP',ISUB)
C     ==--------------------------------------------------------------==
      IN2=LEN(NAME)
      CALL XSTRING(NAME,IN1,IN2)
      IF((ISWFIRST.EQ.0).OR.(IFILE.GT.ISWFILEMAX)) THEN
        WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
        CALL STOPGM(' WRI_SWAP','SWAP FILE DOESN''T EXIST!!')
      ENDIF
      IUNIT=IUNITSW(IFILE)
      ACFILE=ACCESSW(IFILE)
      IF(IUNIT.EQ.0) THEN
        CALL INI_SWAP(IFILE)
        IUNIT=IUNITSW(IFILE)
      ENDIF
      IF(IKPT.GT.NSWBLOCK) THEN
        CALL STOPGM(' WRI_SWAP','INDEX IN K POINTS BLOCK WRONG')
      ENDIF
      DO INDI=1,ISWEND(IFILE)
        SNAME=SWNAME(INDI,IFILE)
        CALL XSTRING(SNAME,IS1,IS2)
        IF(SNAME(IS1:IS2).EQ.NAME(IN1:IN2)) GOTO 30
      ENDDO
 30   CONTINUE
      IF( (ISWEND(IFILE).EQ.0).OR.(INDI.GT.ISWEND(IFILE)) ) THEN
        ISWEND(IFILE)=ISWEND(IFILE)+1
        IF(ISWEND(IFILE).GT.ISWMAX) THEN
          CALL STOPGM('WRI_SWAP','INCREASE ISWMAX')
        ENDIF
        SWNAME(ISWEND(IFILE),IFILE)=NAME(IN1:IN2)//' '
        INDI=ISWEND(IFILE)
      ENDIF
      IPOSITION=ISWPOS(IKPT,INDI,IFILE)
      IF(IPOSITION.NE.0) THEN
C       Possible to write if the size is the same.
        IF(ACFILE.EQ.'S') THEN
          WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
          WRITE(6,*) ' TYPE OF ACCESS: SEQUENTIAL'
          WRITE(6,*) ' REWRITE FOR ',NAME,' (IKPT=',IKPT,')'
          CALL STOPGM(' WRI_SWAP', 'REWRITE IS FORBIDDEN!')
        ELSE
          IF(ISWSIZE(IPOSITION,IFILE).NE.NA) THEN
            WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
            WRITE(6,*) ' TYPE OF ACCESS: DIRECT'
            WRITE(6,*) ' REWRITE FOR ',NAME,' (IKPT=',IKPT,')'
            WRITE(6,*) ' OLD SIZE=',ISWSIZE(IPOSITION,IFILE)
            WRITE(6,*) ' NEW SIZE=',NA
            CALL STOPGM(' WRI_SWAP','WRONG SIZE OF BLOCK')
          ENDIF
          CALL GO_SWAP(IFILE,IPOSITION)
        ENDIF
      ELSE
        IF( (ACFILE.EQ.'D').AND.(NA.GT.ISIDISW(IFILE)) ) THEN
          WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
          WRITE(6,*) ' TYPE OF ACCESS: DIRECT'
          WRITE(6,*) ' REWRITE FOR ',NAME,' (IKPT=',IKPT,')'
          WRITE(6,*) ' RECORD SIZE=',ISIDISW(IFILE)
          WRITE(6,*) ' WANTED SIZE=',NA
          CALL STOPGM('WRI_SWAP','INCREASE RECORD SIE')
        ENDIF
        CALL GO_SWAP(IFILE,ISWREC(IFILE)+1)
        ISWREC(IFILE)=ISWREC(IFILE)+1
        IPOSITION=ISWREC(IFILE)
      ENDIF
      IF(ACFILE.EQ.'S') THEN
        WRITE(UNIT=IUNIT,ERR=998) A
      ELSE
C       WARNING: THE RECORDSIZE IS PROCESSOR-DEPENDENT!!
C       (SEE INI_SWAP)
        WRITE(UNIT=IUNIT,ERR=999,REC=IPOSITION) A
      ENDIF
      IPOSITION=ISWCUR(IFILE)
      ISWPOS(IKPT,INDI,IFILE)=IPOSITION
      ISWSIZE(IPOSITION,IFILE)=NA
      ISWACCE(1,IPOSITION,IFILE)=ISWACCE(1,IPOSITION,IFILE)+1
      ISWCUR(IFILE)=ISWCUR(IFILE)+1
      ISWMEM(INDI,IFILE)=IKPT
      CALL TIHALT('  WRI_SWAP',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
 998  CONTINUE
      CALL STOPGM(' WRI_SWAP', 'CAN''T WRITE IN SEQUENTIAL SWAP FILE')
 999  CONTINUE
      CALL STOPGM(' WRI_SWAP', 'CAN''T WRITE IN DIRECT SWAP FILE')
      END
C     ==================================================================
      SUBROUTINE REA_SWAP(IFILE,A,NA,IKPT,NAME)
C     ==--------------------------------------------------------------==
C     == READ IN SWAP FILES                                           ==
C     == IFILE=0 SWAP PERMANENT FILE                                  ==
C     ==       1 SWAP TEMPORARY FILE                                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INCLUDE 'swap.inc'
      INTEGER   IFILE,NA,IKPT
      CHARACTER NAME*(*)
      REAL*8    A(NA)
C     Variables
      DIMENSION ISWMEM(ISWMAX,ISWFILEMAX),
     &          ISWPOS(NSWBLOCK,ISWMAX,ISWFILEMAX),
     &          ISWSIZE(NSWBLOCK*ISWMAX,ISWFILEMAX),
     &          ISWACCE(2,NSWBLOCK*ISWMAX,ISWFILEMAX),
     &          IUNITSW(ISWFILEMAX),ISWCUR(ISWFILEMAX),
     &          ISWEND(ISWFILEMAX),ISWREC(ISWFILEMAX)
      INTEGER   IS1,IS2,IN1,IN2,ISUB,IUNIT,INDI,IPOSITION
      CHARACTER SNAME*8
C     ==--------------------------------------------------------------==
      CALL TISET('  REA_SWAP',ISUB)
C     ==--------------------------------------------------------------==
      IN2=LEN(NAME)
      CALL XSTRING(NAME,IN1,IN2)
      IF(ISWFIRST.EQ.0.OR.IFILE.GT.ISWFILEMAX) THEN
        WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
        CALL STOPGM(' REA_SWAP','SWAP FILE DOESN''T EXIST!!')
      ENDIF
      IUNIT=IUNITSW(IFILE)
      IF(IUNIT.EQ.0) THEN
        WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
        CALL STOPGM(' REA_SWAP','SWAP FILE DOESN''T EXIST!!')
      ENDIF
      IF(IKPT.GT.NSWBLOCK) THEN
        CALL STOPGM(' REA_SWAP','INDEX BLOCK WRONG')
      ENDIF
      DO INDI=1,ISWEND(IFILE)
        SNAME=SWNAME(INDI,IFILE)
        CALL XSTRING(SNAME,IS1,IS2)
        IF(SNAME(IS1:IS2).EQ.NAME(IN1:IN2)) GOTO 30
      ENDDO
      IF( (ISWEND(IFILE).EQ.0).OR.(INDI.GT.ISWEND(IFILE)) ) THEN
        WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
        WRITE(6,*) ' ',NAME(IN1:IN2), ' BLOCK=',IKPT
        CALL STOPGM(' REA_SWAP', 'READ ERROR')
      ENDIF
 30   CONTINUE
      IF(ISWMEM(INDI,IFILE).EQ.IKPT) THEN
        CALL TIHALT('  REA_SWAP',ISUB)
        RETURN
      ENDIF
      IPOSITION=ISWPOS(IKPT,INDI,IFILE)
      IF(IPOSITION.EQ.0) THEN
        WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
        WRITE(6,*) ' ',NAME(IN1:IN2), ' BLOCK=',IKPT
        CALL STOPGM(' REA_SWAP', 'RECORD DOESN''T EXIST!!')
      ENDIF
      IF(ISWSIZE(IPOSITION,IFILE).NE.NA) THEN
        WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
        WRITE(6,*) ' ',NAME(IN1:IN2), ' BLOCK=',IKPT
        CALL STOPGM(' REA_SWAP', 'ARRAY SIZE IS WRONG')
      ENDIF
      IF(ACCESSW(IFILE).EQ.'S') THEN
        CALL GO_SWAP(IFILE,IPOSITION)
        READ(UNIT=IUNIT,ERR=999,END=998) A
        ISWCUR(IFILE)=ISWCUR(IFILE)+1
      ELSE
        READ(UNIT=IUNIT,ERR=999,REC=IPOSITION) A
        ISWCUR(IFILE)=IPOSITION+1
      ENDIF
      ISWACCE(2,IPOSITION,IFILE)=ISWACCE(2,IPOSITION,IFILE)+1
      ISWMEM(INDI,IFILE)=IKPT
      CALL TIHALT('  REA_SWAP',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
 998  CONTINUE
      CALL STOPGM(' REA_SWAP', 'END OF SWAP FILE')
 999  CONTINUE
      CALL STOPGM(' REA_SWAP', 'CAN''T READ IN SWAP FILE')
      END
C     ==================================================================
      SUBROUTINE GO_SWAP(IFILE,IPOSITION)
C     ==--------------------------------------------------------------==
C     == GO TO IPOSITION IN THE IFILE IF SEQUENTIAL FILE              ==
C     == OTHERWISE SET ISWCUR(IFILE) TO IPOSITION                     == 
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'swap.inc'
C     Arguments
      INTEGER   IFILE,IPOSITION
C     Variables
      DIMENSION IUNITSW(ISWFILEMAX),ISWCUR(ISWFILEMAX),
     &          ISWREC(ISWFILEMAX)
      INTEGER   IUNIT
C     ==--------------------------------------------------------------==
      IF(ISWFIRST.EQ.0.OR.IFILE.GT.ISWFILEMAX)
     &     CALL STOPGM(' GO_SWAP', 'BAD SWAP FILE NUMBER')
      IUNIT=IUNITSW(IFILE)
      IF(ACCESSW(IFILE).EQ.'S') THEN
        IF(IUNIT.EQ.0) THEN
          WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
          CALL STOPGM(' GO_SWAP','SWAP FILE DOESN''T EXIST!!')
        ENDIF
        IF(IPOSITION.GT.ISWREC(IFILE)+1) THEN
          CALL STOPGM(' GO_SWAP', 'BAD SPECIFIED POSITION')
        ENDIF
        REWIND(IUNIT)
        CALL FSKIP(IUNIT,IPOSITION-1)
      ENDIF
      ISWCUR(IFILE)=IPOSITION
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE END_SWAP
C     ==--------------------------------------------------------------==
C     == DEALLOCATE ALL VARIABLES FOR SWAP AND DELETE FILES           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'swap.inc'
C     Variables
      DIMENSION ISWMEM(ISWMAX,ISWFILEMAX),
     &          ISWPOS(NSWBLOCK,ISWMAX,ISWFILEMAX),
     &          ISWSIZE(NSWBLOCK*ISWMAX,ISWFILEMAX),
     &          ISWACCE(2,NSWBLOCK*ISWMAX,ISWFILEMAX),
     &          IUNITSW(ISWFILEMAX),ISWCUR(ISWFILEMAX),
     &          ISWEND(ISWFILEMAX),ISWREC(ISWFILEMAX),
     &          ISIDISW(ISWFILEMAX)
      INTEGER   IFILE,IUNIT, IFIRST
C     ==--------------------------------------------------------------==
      IF(ISWFIRST.EQ.0) RETURN
      IFIRST=0
      DO IFILE=1,ISWFILEMAX
        IUNIT=IUNITSW(IFILE)
        IF(IUNIT.NE.0) THEN
          IF(IFIRST.EQ.0) THEN
            WRITE(6,'(//,1x,64("="))')
            WRITE(6,'(T22,A)') 'SWAP FILES INFORMATION'
            IFIRST=1
          ENDIF
          CALL STATE_SWAP(IFILE)
          CALL DEL_SWAP(IFILE,I_STOPGM)
        ELSE
          IF(IFILE.EQ.ISW_CALC.AND.TSWCALC) THEN
            CALL CALC_SWAP
          ENDIF
        ENDIF
      ENDDO
      CALL FREEM(IP_IUNITSW)
      CALL FREEM(IP_ISWMEM)
      CALL FREEM(IP_ISWPOS)
      CALL FREEM(IP_ISWSIZE)
      CALL FREEM(IP_ISWACCE)
      CALL FREEM(IP_ISWCUR)
      CALL FREEM(IP_ISWEND)
      CALL FREEM(IP_ISWREC)
      CALL FREEM(IP_ISIDISW)
      CALL FREEM(IP_ISWRES)
      NSWBLOCK=0
      ISWFIRST=0
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE DEL_SWAP(IFILE,IFLAG)
C     ==--------------------------------------------------------------==
C     == DELETE SWAP FILE IFILE                                       ==
C     == IFLAG IS TO AVOID RECURSIVE CALL OF STOPGM                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'swap.inc'
C     Arguments
      INTEGER   IFILE,IFLAG
C     Variables
      DIMENSION IUNITSW(ISWFILEMAX),ISIDISW(ISWFILEMAX)
      INTEGER   IUNIT
      CHARACTER ORDER*280
C     ==--------------------------------------------------------------==
      IF(IFLAG.EQ.I_NOSTOPGM) THEN
        IF(ISWFIRST.EQ.0.OR.IFILE.GT.ISWFILEMAX)
     &       CALL STOPGM(' DEL_SWAP', 'BAD SWAP FILE NUMBER')
      ENDIF
      IUNIT=IUNITSW(IFILE)
      IF(IFLAG.EQ.I_NOSTOPGM) THEN
        IF(IUNIT.EQ.0) THEN
          WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
          CALL STOPGM(' DEL_SWAP','SWAP FILE DOESN''T EXIST!!')
        ENDIF
      ENDIF
      CLOSE(UNIT=IUNIT)
#if defined(__SUN)
      ORDER = '\\rm -f '//SWFILE(IFILE)
#else
      ORDER = 'rm -f '//SWFILE(IFILE)
#endif
#if defined(MYRINET) || defined(CRAY_XT3) || defined(__WINNT)
      CALL RMFILE(SWFILE(IFILE))
#else
      CALL SYSTEM(ORDER)
#endif
      CALL RES_SWAP(IFILE,IFLAG)
      IUNITSW(IFILE)=0
      ACCESSW(IFILE)='S'
      ISIDISW(IFILE)=0
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE RES_SWAP(IFILE,IFLAG)
C     ==--------------------------------------------------------------==
C     == RESET COUNTERS OF SWAP FILE IFILE                            ==
C     == NOT CLOSE THE FILE                                           ==
C     == IFLAG IS TO AVOID RECURSIVE CALL OF STOPGM                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE  'swap.inc'
C     Arguments
      INTEGER   IFILE,IFLAG
C     Variables
      DIMENSION ISWMEM(ISWMAX,ISWFILEMAX),
     &          ISWPOS(NSWBLOCK,ISWMAX,ISWFILEMAX),
     &          ISWSIZE(NSWBLOCK*ISWMAX,ISWFILEMAX),
     &          ISWACCE(2,NSWBLOCK*ISWMAX,ISWFILEMAX),
     &          IUNITSW(ISWFILEMAX),ISWRES(ISWFILEMAX),
     &          ISWCUR(ISWFILEMAX),ISWEND(ISWFILEMAX),ISWREC(ISWFILEMAX)
      INTEGER   IUNIT
C     ==--------------------------------------------------------------==
      IF(IFLAG.EQ.I_NOSTOPGM) THEN
        IF(ISWFIRST.EQ.0.OR.IFILE.GT.ISWFILEMAX)
     &       CALL STOPGM(' RES_SWAP', 'BAD SWAP FILE NUMBER')
      ENDIF
      IUNIT=IUNITSW(IFILE)
      IF(IFLAG.EQ.I_NOSTOPGM) THEN
        IF(IUNIT.EQ.0) THEN
          WRITE(6,*) ' SWAP FILE NUMBER:',IFILE
          CALL STOPGM(' RES_SWAP','SWAP FILE DOESN''T EXIST!!')
        ENDIF
      ENDIF
      CALL IAZZERO(ISWMEM(1,IFILE)   ,ISWMAX)
      CALL IAZZERO(ISWPOS(1,1,IFILE) ,  NSWBLOCK*ISWMAX)
      CALL IAZZERO(ISWSIZE(1,IFILE)  ,  NSWBLOCK*ISWMAX)
      CALL IAZZERO(ISWACCE(1,1,IFILE),2*NSWBLOCK*ISWMAX)
      ISWCUR(IFILE)=1
      ISWEND(IFILE)=0
      ISWREC(IFILE)=0
      ISWRES(IFILE)=ISWRES(IFILE)+1
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE STATE_SWAP(IFILE)
C     ==--------------------------------------------------------------==
C     == STATE OF SWAP FILE IFILE                                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'swap.inc'
      INTEGER IFILE
      DIMENSION ISWMEM(ISWMAX,ISWFILEMAX),
     &     ISWPOS(NSWBLOCK,ISWMAX,ISWFILEMAX),
     &     ISWSIZE(NSWBLOCK*ISWMAX,ISWFILEMAX),
     &     ISWACCE(2,NSWBLOCK*ISWMAX,ISWFILEMAX),
     &     IUNITSW(ISWFILEMAX),ISWCUR(ISWFILEMAX),ISWEND(ISWFILEMAX),
     &     ISWREC(ISWFILEMAX),ISIDISW(ISWFILEMAX),ISWRES(ISWFILEMAX)
C     Variables
      CHARACTER FFORMAT*15
      INTEGER   IUNIT,I1,I2,IN,IB,IPOSITION
C     ==--------------------------------------------------------------==
      IF(ISWFIRST.EQ.0) THEN
        WRITE(6,*) ' NO SWAP FILE'
        RETURN
      ENDIF
      IF(IFILE.GT.ISWFILEMAX.OR.IFILE.LE.0) THEN
        WRITE(6,'(A,I6,T51,A15)')
     &       ' SWAP FILE NUMBER:',IFILE,'DOESN''T EXIST!!'
        RETURN
      ENDIF
      IUNIT=IUNITSW(IFILE)
      IF(IUNIT.EQ.0) THEN
        WRITE(6,'(A,I6,T51,A15)')
     &       ' SWAP FILE NUMBER:',IFILE,'DOESN''T EXIST!!'
        RETURN
      ENDIF
C     ==--------------------------------------------------------------==
      WRITE(6,'(1x,64("="))')
      CALL XSTRING(SWFILE(IFILE),I1,I2)
      IF(65-(I2-I1).LE.18) THEN
        WRITE(FFORMAT,'(A,I2,A)') '(A,/,T',MAX(1,65-(I2-I1)),',A)'
      ELSE
        WRITE(FFORMAT,'(A,I2,A)') '(A,T',MAX(18,65-(I2-I1)),',A)'
      ENDIF
      WRITE(6,FFORMAT)    ' SWAP FILE NAME: ',SWFILE(IFILE)(I1:I2)
      WRITE(6,'(A,T60,I6)') ' SWAP FILE NUMBER:',IFILE
      WRITE(6,'(A,T60,I6)') ' SWAP FILE UNIT:',IUNIT
      IF(ACCESSW(IFILE).EQ.'S') THEN
        WRITE(6,'(" SWAP FILE ACCESS:",T56,"SEQUENTIAL")')
      ELSE
        WRITE(6,'(" SWAP FILE ACCESS:",T56,"    DIRECT")')
        WRITE(6,'(A,T46,I20)') ' RECORD SIZE:',ISIDISW(IFILE)
      ENDIF
      IF(ISWRES(IFILE).NE.0)
     &     WRITE(6,'(A,T60,I6)') ' NUMBER OF RESET:',ISWRES(IFILE)
      WRITE(6,'(A,T60,I6)') ' NUMBER OF RECORDS:',ISWREC(IFILE)
      WRITE(6,'(A,T60,I6)') ' CURRENT RECORD:', ISWCUR(IFILE)
      WRITE(6,'(A,T60,I6)') ' NUMBER OF VARIABLES:',ISWEND(IFILE)
      WRITE(6,'(A,T60,I6)') ' NUMBER OF BLOCK PER VARIABLE:', NSWBLOCK
      DO IN=1,ISWEND(IFILE)
        CALL XSTRING(SWNAME(IN,IFILE),I1,I2)
        WRITE(6,'(I6,1X,A,T44,A18,I3,A1)')
     &       IN,SWNAME(IN,IFILE)(I1:I2),
     &       '(IN MEMORY BLOCK=',ISWMEM(IN,IFILE),')'
        DO IB=1,NSWBLOCK
          IF(ISWPOS(IB,IN,IFILE).NE.0) THEN
            IPOSITION=ISWPOS(IB,IN,IFILE)
            WRITE(6,'(5X,A,I5,T18,A,I5,T29,A,I8,T43,A,I5,T56,A,I5)')
     &           'BLOCK=' ,IB,
     &           'REC='   ,IPOSITION,
     &           'SIZE='  ,ISWSIZE(IPOSITION,IFILE),
     &           'WRITE=' ,ISWACCE(1,IPOSITION,IFILE),
     &           'READ='  ,ISWACCE(2,IPOSITION,IFILE)
          ENDIF
        ENDDO
      ENDDO
      WRITE(6,'(1x,64("="))')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CALC_SWAP
C     ==--------------------------------------------------------------==
C     == GIVE THE NUMBER OF CALCULATION IF TSWCALC                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'swap.inc'
      DIMENSION ISWACCE(2,NSWBLOCK*ISWMAX,ISWFILEMAX)
C     ==--------------------------------------------------------------==
      IF(.NOT.TSWCALC) RETURN
      WRITE(6,'(1x,64("="))')
      WRITE(6,'(1X,A)')
     &     'NUMBER OF CALCULATIONS FOR ARRAY DEPENDING ON K POINTS:'
      IF(ISWACCE(2,ISW_HGKP,ISW_CALC).NE.0)   WRITE(6,'(1X,A,T60,I6)')
     &     'HGKP  ',ISWACCE(2,ISW_HGKP,ISW_CALC)
      IF(ISWACCE(2,ISW_HGKM,  ISW_CALC).NE.0) WRITE(6,'(1X,A,T60,I6)')
     &     'HGKM  ',ISWACCE(2,ISW_HGKM,ISW_CALC)
      IF(ISWACCE(2,ISW_MASKGW,ISW_CALC).NE.0) WRITE(6,'(1X,A,T60,I6)')
     &     'MASKGW',ISWACCE(2,ISW_TWNL,ISW_CALC)
      IF(ISWACCE(2,ISW_TWNL,  ISW_CALC).NE.0) WRITE(6,'(1X,A,T60,I6)')
     &     'TWNL  ',ISWACCE(2,ISW_TWNL,ISW_CALC)
      IF(ISWACCE(2,ISW_EMK2,  ISW_CALC).NE.0) WRITE(6,'(1X,A,T60,I6)')
     &     'EMK2  ',ISWACCE(2,ISW_EMK2,ISW_CALC)
      IF(ISWACCE(2,ISW_EIGKR, ISW_CALC).NE.0) WRITE(6,'(1X,A,T60,I6)')
     &     'EIGKR ',ISWACCE(2,ISW_EIGKR,ISW_CALC)
      IF(ISWACCE(2,ISW_ALM,   ISW_CALC).NE.0) WRITE(6,'(1X,A,T60,I6)')
     &     'ALM   ',ISWACCE(2,ISW_ALM,ISW_CALC)
      WRITE(6,'(1x,64("="))')
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE INQ_SWAP(KBEG,KEND,KINC)
C     ==--------------------------------------------------------------==
C     == GIVE THE ORDER TO PERFORM LOOP ON K POINTS BLOCK             ==
C     == IN ORDER TO REDUCE READ NUMBER                               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'kpts.inc'        !for NKPBL
      INCLUDE 'swap.inc'        !for initialization
C     Arguments
      INTEGER   KBEG,KEND,KINC
C     Variables
      DIMENSION ISWMEM(ISWMAX,ISWFILEMAX)
C     ==--------------------------------------------------------------==
      IF(.NOT.TKBLOCK .OR. ISWFIRST.EQ.0) THEN
        KBEG=1
        KEND=NBLKP
        KINC=1
        RETURN
      ENDIF
      IF(ISWMEM(1,1).EQ.NBLKP) THEN
        KBEG=NBLKP
        KEND=1
        KINC=-1
      ELSE
        KBEG=1
        KEND=NBLKP
        KINC=1
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE RESKPT_SWAP
C     ==--------------------------------------------------------------==
C     == RESET SWAP FILE IF TKALL                                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'swap.inc'
      INCLUDE 'kpts.inc'
C     ==--------------------------------------------------------------==
      IF(TKALL) THEN
        CALL RES_SWAP(3,I_NOSTOPGM)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PT_CHANGE(IP_POINT,LLIST,IKPT,ISIZE,IFLAG)
C     ==--------------------------------------------------------------==
C     == IFLAG=1 WRITE                                                ==
C     ==       0 READ                                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      REAL*8  POINT
      POINTER (IP_POINT,POINT)
      INTEGER LLIST(0:NBLKP),IKPT,ISIZE,IFLAG, I,ICURSOR
C     ==--------------------------------------------------------------==
      IF(IFLAG.EQ.0) THEN
        IF(LLIST(IKPT).EQ.0) THEN
          CALL STOPGM('PT_CHANGE','THE BLOCK IS NOT WRITTEN')
        ENDIF
        ICURSOR=LLIST(IKPT)-LLIST(0)
        LLIST(0)=LLIST(IKPT)
      ELSE
        LLIST(IKPT)=LLIST(0)
        DO I=1,NBLKP
          IF(LLIST(I).EQ.0) THEN
            ICURSOR=I-LLIST(0)
            LLIST(0)=I
            GOTO 10
          ENDIF
        ENDDO
        ICURSOR=LLIST(1)-LLIST(0)
        LLIST(0)=LLIST(1)
 10     CONTINUE
      ENDIF
      IP_POINT=IP_POINT+8*ISIZE*ICURSOR
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      BLOCK DATA INISWAP
      IMPLICIT NONE
      INCLUDE 'swap.inc'
      DATA ISWFIRST /0/
      END
C     ==================================================================
