C     ==================================================================
      SUBROUTINE SDION(XPAR,DXPAR,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     ==  UPDATE IONIC POSITIONS WITH STEEPEST DESCENT OR             ==
C     ==         WITH CONJUGATE GRADIENT                              ==
C     ==  ADD MINIMIZATION ALONG THE LINE                             ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   XPAR(NODIM)                                                ==
C     ==  DXPAR(NODIM)                                                ==
C     == OUTPUT:                                                      ==
C     ==   XPAR(NODIM)                                                ==
C     ==  (NODIM Number of Freedom degres)                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'tpar.inc'        !DT2BYM
      INCLUDE 'isos.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'ener.inc'        !ETOT
C     Arguments
      INTEGER LSCR
      REAL*8  XPAR(NODIM),DXPAR(NODIM),SCR(LSCR)
C     Variables
      INTEGER   NSTEPMAX
      PARAMETER (NSTEPMAX=20)
      REAL*8    HLEN,PF1,DDOLD,DD,DR,DSTEP,
     &          EE(NSTEPMAX),DX(NSTEPMAX),DE(NSTEPMAX),
     &          F2,F2OLD,FPOLD,FPOLDB,P2,P2OLD,FAC1,BETA,
     &          DDOT,SCALAR
      EXTERNAL  DDOT
      INTEGER   INFO,I,
     &          IFIRST,NSTEP
      SAVE      IFIRST,NSTEP,
     &          DDOLD,DSTEP,DD,EE,DX,DE,F2OLD,FPOLD,P2OLD
      DATA      IFIRST /0/
C     ==--------------------------------------------------------------==
      CALL TEST_SCR('SDION','NODIM',LSCR,NODIM)
      IF(IFIRST.EQ.0) THEN
C       Determines the first step
        DSTEP=0.D0
        DO I=1,NSP
          DSTEP=DSTEP+DT2BYM(I)
        ENDDO
        DSTEP=DSTEP/DBLE(NSP)
        DD=DSTEP
        CALL MEMORY(IP_PXPAR,NODIM,'PXPAR')
        CALL AZZERO(PXPAR,NODIM)
        IF(TCGP) THEN
          FAC1=0.91D0
          F2OLD = 1.D0
          P2OLD = 0.D0
        ELSE
          FAC1=0.D0
        ENDIF
        NSTEP=0
        IFIRST=1
      ENDIF
      IF(TSDPLINE.OR.TCGP) THEN
C       ==------------------------------------------------------------==
C       == MINIMIZATION ALONG THE LINE BY STEEPEST DESCENT            ==
C       ==------------------------------------------------------------==
        NSTEP=NSTEP+1
        EE(NSTEP)=ETOT
        DE(NSTEP)=SCALAR(NODIM,PXPAR,DXPAR)
        IF(NSTEP.EQ.1) THEN
          DX(NSTEP)=0.D0
          IF(DSTEP.EQ.0.D0) THEN
            DSTEP=DD
          ELSE
            DD=DSTEP
          ENDIF
        ELSE
          DX(NSTEP)=DSTEP
        ENDIF
        CALL MINLIN(NSTEP,DX,EE,DE,DSTEP,INFO)
        IF(INFO.NE.0) THEN
C         First step
          DDOLD=0.D0
          DX(NSTEP)=0.D0
          IF(TCGP) THEN
C           Conjugate gradient
            F2=SCALAR(NODIM,DXPAR,DXPAR)
            IF(INFO.EQ.2) THEN
C             We reset the conjugate gradient direction
              DO I=1,NODIM
                PXPAR(I)=0.D0
              ENDDO
              P2OLD=0.D0
            ENDIF
            FPOLD=0.D0
            DO I=1,NODIM
              FPOLD=DXPAR(I)*PXPAR(I)
            ENDDO
            IF(F2OLD.EQ.0.D0) THEN
              BETA=0.D0
            ELSE
              BETA = F2 / F2OLD * FAC1
            ENDIF
            FPOLDB = FPOLD * BETA
            P2 = F2 + P2OLD*BETA*BETA + 2.D0*FPOLDB
C           If cos( (P,F) ) < 0.1 , we reinitialize P = F .
            IF(F2.NE.0.D0.AND.P2.NE.0.D0) THEN
              IF( (F2 + FPOLDB)/SQRT(F2*P2).LT.0.1D0) THEN
                BETA=0.D0
                P2=F2
              ENDIF
            ENDIF
            F2OLD = F2
            P2OLD = P2
C           First step
            DO I=1,NODIM
              PXPAR(I)=DXPAR(I)+BETA*PXPAR(I)
            ENDDO
            DE(1)=SCALAR(NODIM,PXPAR,DXPAR)
          ELSE
C           Steepest descent
            BETA=0.D0
C           First step
            DO I=1,NODIM
              PXPAR(I)=DXPAR(I)
            ENDDO
            DE(1)=SCALAR(NODIM,DXPAR,DXPAR)
          ENDIF
        ENDIF
        DR=DSTEP-DDOLD
C       Compute the ionic steps
        DO I=1,NODIM
          SCR(I)=-DR*PXPAR(I)
        ENDDO
        DDOLD=DSTEP
      ELSE
C       ==------------------------------------------------------------==
C       == OLD VERSION OF SDION                                       ==
C       ==------------------------------------------------------------==
C       Compute the ionic steps
        DO I=1,NODIM
          SCR(I)=-DTM(I)*DXPAR(I)
        ENDDO
C       Check if displacements are not too big
        HLEN=SQRT(DDOT(NODIM,SCR(1),1,SCR(1),1))
        IF(HLEN.GT.DMAX) THEN
          PF1=DMAX/HLEN
          CALL DSCAL(NODIM,PF1,SCR(1),1)
        ENDIF
      ENDIF
C     Fix the center of mass (if LFCOM=.TRUE.)
      CALL FIXCOM(SCR)
      CALL DAXPY(NODIM,1.0D0,SCR(1),1,XPAR(1),1)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE MINLIN(NSTEP,DX,EE,DE,DSTEP,INFO)
C     ==--------------------------------------------------------------==
C     == MINIMISATION ALONG LINE                                      ==
C     == The NSTEPth point is the last calculated one                 ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   NSTEP   Number of steps                                    ==
C     ==   DX(NSTEP) Abscissae of points                              ==
C     ==   EE(NSTEP) Energy of points                                 ==
C     ==   DE(NSTEP  First derivative of points                       ==
C     == OUTPUT:                                                      ==
C     ==   DSTEP   Estimated abscissae for root                       ==
C     ==   INFO    0 O.K. we continue                                 ==
C     ==           1 We need to have initialization                   ==
C     ==             (new direction)                                  ==
C     ==   NSTEP   Number of points stored for the next call          ==
C     ==   DX,EE, and EE are sorted                                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   NSTEP,INFO
      REAL*8    DX(NSTEP),EE(NSTEP),DE(NSTEP),DSTEP
C     Variables
      REAL*8    GOLDEN
      PARAMETER (GOLDEN=1.618034D0)
      INTEGER   NSTEPMAX
      PARAMETER (NSTEPMAX=20)
      INTEGER   INDEX(NSTEPMAX),I
      REAL*8    XNEW(NSTEPMAX),DNEW,DMIN,DMAX
C     ==--------------------------------------------------------------==
C     One point
      IF(NSTEP.LE.1) THEN
        INFO=1
        RETURN
      ENDIF
C     Determine min et max
      DMIN=DX(1)
      DMAX=DX(1)
      DO I=1,NSTEP
        IF(DMIN.GT.DX(I)) DMIN=DX(I)
        IF(DMAX.LT.DX(I)) DMAX=DX(I)
      ENDDO
C     We sort the points
      CALL SORT2(EE,NSTEP,INDEX)
      CALL SORT(NSTEP,DX,XNEW,INDEX)
      CALL SORT(NSTEP,DE,XNEW,INDEX)
      CALL XLAG2(NSTEP,EE,DX,DE,DMIN,DMAX,DNEW,INFO)
      IF(INFO.EQ.0) THEN
C       Good
        DSTEP=DNEW
        IF(NSTEP.EQ.2) THEN
C         We check if the approximation is O.K.
          INFO=0
        ELSE
C         Check is the last point has the lowest EE
          IF(INDEX(1).EQ.NSTEP) THEN
C           We have finished line minimisation
            INFO=1
            NSTEP=1
C           DSTEP a little bit bigger
            DSTEP=DSTEP*1.1D0
          ELSE
C           Use the lowest
            DSTEP=DX(1)
            NSTEP=0
            INFO=0
          ENDIF
        ENDIF
        RETURN
      ELSEIF(INFO.EQ.-1) THEN
C       DNEW smaller than DMIN
C       Error
        INFO=2
        NSTEP=1
      ELSEIF(INFO.EQ.1) THEN
C       DNEW bigger than DMAX
C       Check if DNEW not too big
        IF(DNEW.GT.DMAX*GOLDEN*GOLDEN*GOLDEN*GOLDEN) THEN
          DSTEP=DMAX*GOLDEN*GOLDEN*GOLDEN*GOLDEN
        ELSE
          DSTEP=DNEW
        ENDIF
        INFO=0
        NSTEP=1
      ELSEIF(INFO.EQ.2) THEN
C       No interpolation
        INFO=1
        NSTEP=1
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SORT(N,X,XNEW,INDEX)
C     ==--------------------------------------------------------------==
C     == SORT X ARRAY WITH INDEX information                          ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==  N Dimension of X(1:N), X(1:N) and INDEX(1:N)                ==
C     == OUTPUT:   X(1:N)                                             ==
C     ==  XNEW(1:N) is a scratch array                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER N,INDEX(N)
      REAL*8  X(N),XNEW(N)
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
      DO I = 1 , N
        XNEW(I)=X(INDEX(I))
      ENDDO
      CALL DCOPY(N,XNEW,1,X,1)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE XLAG2(NSTEP,E,D,DE,DMIN,DMAX,DSTEP,INFO)
C     ==--------------------------------------------------------------==
C     == GIVE ROOT OF THE SECOND POLYNOMIAL GIVEN BY TWO POINTS       ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==   E(1:NSTEP)    Energy for each point                        ==
C     ==   D(1:NSTEP)    Coordinate                                   ==
C     ==   DE(1:NSTEP)   First derivative of energy along the line    ==
C     ==   DMIN          Minimal abscissae (bounds)                   ==
C     ==   DMAX          maximal abscissae                            ==
C     == OUTPUT:                                                      ==
C     ==   DSTEP the abscissae of the new point                       ==
C     ==   INFO  0 DSTEP is between DMIN and DMAX                     ==
C     ==        -1 DSTEP < DMIN                                       ==
C     ==         1 DSTEP > DMAX                                       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   NSTEP,INFO
      REAL*8    E(NSTEP),D(NSTEP),DE(NSTEP),DMIN,DMAX,DSTEP
C     Variables
      INTEGER   IOTHER,I
C     ==--------------------------------------------------------------==
      IF(NSTEP.LE.1) RETURN
C     ==--------------------------------------------------------------==
C     The first one has the lowest values
      IOTHER=0
      DO I=2,NSTEP
        IF(DE(1)*DE(I).LT.0.D0) THEN
          IOTHER=I
          GOTO 100
        ENDIF
      ENDDO
 100  CONTINUE
      IF(IOTHER.NE.0) THEN
C       Good , we have bounds
        INFO=0
        IF(DE(IOTHER)-DE(1).EQ.0) THEN
C         No interpolation (normally impossible)
          INFO=2
        ELSE
          DSTEP=D(1)-DE(1)*(D(IOTHER)-D(1))/(DE(IOTHER)-DE(1))
        ENDIF
      ELSE
C       Use the two lowest values
        IF(DE(2)-DE(1).EQ.0) THEN
C         No interpolation (normally impossible)
          INFO=2
        ELSE          
          DSTEP=D(1)-DE(1)*(D(2)-D(1))/(DE(2)-DE(1))
        ENDIF
C       Check if the root is inside the bounds
        IF(DSTEP.LT.DMIN) THEN
          INFO=-1
        ELSEIF(DSTEP.GT.DMAX) THEN
          INFO=1
        ELSE
          INFO=0
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION SCALAR(N,A,B)
C     ==--------------------------------------------------------------==
C     == SCALAR PRODUCT                                               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER N
      REAL*8  A(N),B(N)
      REAL*8  SCALAR
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
      SCALAR=0.D0
      DO I=1,N
        SCALAR=SCALAR+A(I)*B(I)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_SDION(LSDION,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !PARENT
      INCLUDE 'cotr.inc'        !NODIM
C     Arguments
      INTEGER LSDION
      CHARACTER*30 TAG
C     ==--------------------------------------------------------------==
      IF(PARENT) THEN
        LSDION=NODIM
        TAG ='NODIM'
      ELSE
        LSDION=0
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
