C     ==================================================================
      SUBROUTINE SECDPT
C     ==--------------------------------------------------------------==
C     ==  VIBRATIONAL ANALYSIS                                        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'atwf.inc'        !NUMAORMAX
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'linres.inc'
      INCLUDE 'cotr.inc'        !NODIM
      INCLUDE 'prop.inc'        !LDIP,DBERRY
      INCLUDE 'bsym.inc'
C     Variables
      COMPLEX*16 C0(*),C1(*),C2(*),PME(*),GDE(*),SC0(*),CG0(*)
      REAL*8  VPP(*),EIGV(*)
      POINTER (IP_C0,C0),(IP_C2,C2),(IP_PME,PME),(IP_VPP,VPP),
     &        (IP_EIGV,EIGV),(IP_GDE,GDE),(IP_C1,C1),
     &        (IP_SC0,SC0),(IP_CG0,CG0)
      INTEGER NC,NSTATE,NC0,NC1,NC2,NCE,NSC0,NXX,LENBK
      DIMENSION F(*)
      REAL*8  SDER(NODIM,NODIM)
      POINTER (IP_SDER,SDER)
      INTEGER NVPP,NGDE,NPME,NEIG
      INTEGER I,MS,MM,NUS,NACT
#ifdef PARALLEL
      INTEGER   NXXFUN
#endif
C     ==--------------------------------------------------------------==
C     ==  MEMORY ALLOCATION                                           ==
C     ==--------------------------------------------------------------==
C     ==                      WAVEFUNCTIONS                           ==
C     ==  CG0   : WAVEFUNCTIONS AT EQUILIBRIUM GEOMETRY               ==
C     ==  C0    : WAVEFUNCTIONS                                       ==
C     ==  C2    : GRADIENTS                                           ==
C     ==  SC0   : S**(-1/2)*C0 (nonorthogonal orbitals)               ==
C     ==  EIGV  : ORBITAL ENERGIES                                    ==
C     ==--------------------------------------------------------------==
      IF ( TQMMM ) CALL STOPGM("SECDPT","QMMM NOT IMPLEMENTED")
C     ==--------------------------------------------------------------==
      IF(TDDFT) THEN
        CALL LR_IN
        CALL TDDFT_INPUT
      ENDIF
      NC=2*NGW*N+8
      NSTATE=N
      NSC0=1
      IF(TDIAG) THEN
        IF(TDAVI) CALL STOPGM('SECDPT',' TDAVI NOT IMPLEMENTED')
        NC0=(2*NGWK*NSTATE*NKPNT+8)*BSFAC
        NC2=(2*NGWK*MAX(NUMAORMAX,NSTATE)+8)*BSFAC
        NEIG=NSTATE*NKPTS*BSFAC
      ELSE
        NC0=(2*NGWK*NSTATE*NKPNT+8)*BSFAC
        NC2=(2*NGWK*MAX(NUMAORMAX,NSTATE*NKPNT)+8)*BSFAC
        NSC0=max(2*NGWK*N*NKPNT*BSFAC,nc2)
        NEIG=NSTATE*NKPTS*BSFAC
      ENDIF
      IF(DBERRY) THEN
#ifdef PARALLEL
        LENBK=NXXFUN(NSTATE)
        NXX=MAX(2*LENBK*NPROC,NC2)
#else
        NXX=NC2
#endif
        NSC0=MAX(NXX,NSC0)
        NC2=MAX(NXX,NC2)
      ENDIF
      IF(TDDFT) THEN
        DO I=1,NSTATE
          IF(F(I).LT.1.D-6) CALL STOPGM("TDDFT",
     *                            "ALL STATES HAVE TO BE OCCUPIED")
        ENDDO
        MS=MAX(NS_MIX,NS_SIN,NS_TRI)
        IF(TDA.AND.LDIAG.EQ.1) THEN
          MM=1
        ELSE
          MM=2
        ENDIF
        IF(TDA.AND.MSUBTA.GT.0) THEN
          NACT=MSUBTA
        ELSE
          NACT=NSTATE
        ENDIF
        NOLR=NACT
        NC1=2*NGW*NACT*MS*MM
        NUA=NSTATE
        NUB=NACT
        NLINW=MS
        NLINR=0
        NUS=2*NUA*NUB
      ELSE
        NC1=1
        NUA=1
        NUB=1
        NLINW=0
        NLINR=0
        NUS=1
      ENDIF
      CALL MEMORY(IP_CG0,NC0,'CG0')
      CALL MEMORY(IP_C0,NC0,'C0')
      CALL MEMORY(IP_C1,NC1,'C1')
      CALL MEMORY(IP_C2,NC2,'C2')
      CALL MEMORY(IP_SC0,NSC0,'SC0')
      CALL MEMORY(IP_EIGV,NEIG,'EIGV')
      CALL MEMORY(IP_UROT,NUS,'UROT')
      CALL AZZERO(UROT,NUS)
      CALL GET_ADDR(IP_CLRWF,C1(1))
C     ==--------------------------------------------------------------==
C     ==  PME   : DIIS WF             HNM1 (for CG)                   ==
C     ==  GDE   : DIIS GRADIENTS                                      ==
C     ==  VPP   : WF HESSIAN                                          ==
C     ==--------------------------------------------------------------==
      IF(.NOT.TSDAN) THEN
        IF(TSDE) THEN
          NPME = 1
          NGDE = 1
          NVPP = 1
        ELSE IF(DIIS) THEN
          NPME = (NGWK*NSTATE+8)*MDIIS*NKPNT
          NGDE = ((NGWK*NSTATE+8)*MDIIS*NKPNT)/4
          NVPP = NGWK
        ELSE IF(PCG) THEN
          NPME = 2*NGWK*NSTATE*NKPNT
          NGDE = 1
          NVPP = NGWK
        ELSE IF(TDIAG) THEN
          IF(TFRSBLK) THEN
            NPME=2*NGWK*MAX(NSTATE,(NKRY_MAX+1)*NKRY_BLOCK)
          ELSE
            NPME=2*NGWK*MAX(NSTATE,NKRY_MAX*NKRY_BLOCK)
          ENDIF
          NGDE = 1
          NVPP = NGWK
        ELSE
          IF(PARENT)
     *    WRITE(6,*) ' WRONG OPTION FOR WAVEFUNCTION OPTIMIZATION'
          CALL STOPGM('SECDPT',' ')
        ENDIF
        CALL MEMORY(IP_PME,NPME,'PME')
        CALL MEMORY(IP_GDE,NGDE,'GDE')
        CALL MEMORY(IP_VPP,NVPP,'VPP')
      ENDIF
C     ==--------------------------------------------------------------==
C     FNL..DFNL
      CALL FNLALLOC(N,.TRUE.,.FALSE.)
      IF(PARENT) CALL PRMEM('    SECDPT')
C     ==--------------------------------------------------------------==
C..from a stored Hessian
      IF(TSDIN) THEN
        IF(BSYMM)CALL STOPGM('SECDPT',
     &           'BSYMM AND STORED_HESSIAN NOT IMPLEMENTED')
        IF(PARENT) CALL MEMORY(IP_SDER,9*NAT*NAT,'SDER')
        CALL HESFILE(SDER,EIGV,C0,C2,SC0)
        CALL VIBANA(SDER)
        IF(PARENT) CALL FREEM(IP_SDER)
      ELSE IF(TSDAN) THEN
C..linear response
        IF(TDDFT) CALL STOPGM("SECDPT","TDDFT NOT IMPLEMENTED")
        IF(BSYMM)CALL STOPGM('SECDPT',
     &           'BSYMM AND LINEAR RESPONSE NOT IMPLEMENTED')
        CALL SDLINRES(C0,C2,SC0,CG0,EIGV)
      ELSE
C..finite differences
        IF(PARENT) WRITE(6,'(A)') ' START FINITE DIFFERENCES'
        CALL SECDER(C0,C1,C2,SC0,CG0,PME,GDE,VPP,EIGV)
      ENDIF
C     ==--------------------------------------------------------------==
      IF(TMEMCHK) CALL PRINT_MEMORY
      CALL FREEM(IP_CG0)
      CALL FREEM(IP_C0)
      CALL FREEM(IP_C1)
      CALL FREEM(IP_C2)
      CALL FREEM(IP_EIGV)
      CALL FREEM(IP_SC0)
      CALL FREEM(IP_UROT)
      CALL FNLDEALLOC(.TRUE.,.FALSE.)
      IF(.NOT.TSDAN) THEN
        CALL FREEM(IP_PME)
        CALL FREEM(IP_GDE)
        CALL FREEM(IP_VPP)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
