C     ==================================================================
      SUBROUTINE READ_IREC(IREC)
C     ==--------------------------------------------------------------==
C     ==                     GENERAL FILE FORMAT (FOR IREC ALSO)      ==
C     ==--------------------------------------------------------------==
C     ==  Section  1: Header                                          ==
C     ==  Section  2: Symmetry and Cell info                          ==
C     ==  Section  3: Number of atomic species and atoms per species  ==
C     ==  Section  4: Atomic coordinates                              ==
C     ==  Section  5: Atomic velocities                               ==
C     ==  Section  6: Initial atomic coordinates                      ==
C     ==  Section  7: Cutoff, # of electrons, grid                    ==
C     ==  Section  8: States, dual, plane waves                       ==
C     ==  Section  9: PW Coefficients                                 ==
C     ==  Section 10: PW Velocities                                   ==
C     ==  Section 11: Accumulators                                    ==
C     ==  Section 12: Nose thermostats general info                   ==
C     ==  Section 13: Nose thermostats electrons                      ==
C     ==  Section 14: Nose thermostats ions                           ==
C     ==  Section 15: Nose thermostats ions (ULTRA)                   ==
C     ==  Section 16: Nose thermostats ions (MASSIVE)                 ==
C     ==  Section 17: Nose thermostats cell                           ==
C     ==  Section 18: Potential                                       ==
C     ==  Section 19: PW single states                                ==
C     ==  Section 20: H Matrices (cell parameters)                    ==
C     ==  Section 21: K-Points                                        ==
C     ==  Section 22: Electron density                                ==
C     ==  Section 23: Occupation numbers                              ==
C     ==  Section 24: Fermi energy and eigenvalues                    ==
C     ==  Section 25: Classical Particles (coordinates and velocities)==
C     ==  Section 26: LinRes PW Coefficients                          ==
C     ==  Section 27: LinRes PW Velocities                            ==
C     ==  Section 28: Partial Hessian (for microiterative TS search)  ==
C     ==  Section 29: P-RFO status and work arrays                    ==
C     ==  Section 30: L-BFGS history and status                       ==
C     ==  Section 31: Adaptive tolerance status                       ==
C     ==  Section 32: Constraints values                              ==
C     ==  Section 33: Cell translation vector in QM/MM runs           ==
C     ==  Section 34: Local temperature parameters                    ==
C     ==  Section 35: Wavefunction history for extrapolation          ==
C     ==  Section 36: Surface Hopping I                               ==
C     ==  Section 37: Surface Hopping II                              ==
C     ==  Section 38: Pseudo random number generator                  ==
C     ==  Section 39: (generalized) Langevin equation                 ==
C     ==  Section 40 - 99 : empty                                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'store.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'clas.inc'
      INCLUDE 'nose.inc'
      INCLUDE 'gle.inc'
      INCLUDE 'shop_rest.inc'
      INTEGER IREC(100), ISETONE
C     ==--------------------------------------------------------------==
      CALL IAZZERO(IREC,100)
      IREC(IREC_INFO) = 1
      IREC(IREC_CELL) = 1
      IREC(IREC_NAS)  = 1
      IREC(IREC_CO)   = ISETONE(RCO)
      IREC(IREC_VEL)  = ISETONE(RVEL)
      IREC(IREC_ICO)  = 1
      IREC(IREC_CUT)  = 1
      IREC(IREC_SDP)  = 1
      IREC(IREC_WF)   = ISETONE(RWF)
      IREC(IREC_PWV)  = ISETONE(RWF.AND. .NOT.(GEOPT.OR.TDIAG.OR.WFOPT))
      IREC(IREC_AC)   = ISETONE(RAC)
      IREC(IREC_NOSE) = ISETONE((RNOE.OR.RNOP.OR.RNOC).AND.MD)
      IREC(IREC_NOE)  = ISETONE(RNOE.AND.MD)
      IREC(IREC_NOP1) = ISETONE(RNOP.AND.MD.AND..NOT.
     &                          (TULTRA.OR.TMNOSE.OR.TLOCT))
      IREC(IREC_NOP2) = ISETONE(RNOP.AND.MD.AND.TULTRA)
      IREC(IREC_NOP3) = ISETONE(RNOP.AND.MD.AND.TMNOSE)
      IREC(IREC_NOP4) = ISETONE(RNOP.AND.MD.AND.TLOCT)
      IREC(IREC_NOC)  = ISETONE(RNOC.AND.MD)
      IREC(IREC_POT)  = ISETONE(RPOT.AND.SPOT)
      IREC(IREC_HE)   = ISETONE(RCELL.AND.TPRCP)
      IREC(IREC_KPT)  = ISETONE(RKPT)
      IREC(IREC_RHO)  = ISETONE(RRHO.AND.TDIAG)
      IREC(IREC_OCC)  = ISETONE(ROCC)
      IREC(IREC_EIGV) = ISETONE(REIGV)
      IREC(IREC_CLAS) = ISETONE(TCLAS.AND.RCO)
      IREC(IREC_LRWF) = ISETONE((TDDFT.OR.TSPEC).AND.RLR)
      IREC(IREC_LRWV) = 0
      IREC(IREC_PHES) = ISETONE(RPHES)
      IREC(IREC_LBFGS)= ISETONE(RLSCL.AND.LBFGS)
      IREC(IREC_PRFO) = ISETONE(RLSCL.AND.PRFO)
      IREC(IREC_RDTL) = ISETONE(RDTL)
      IREC(IREC_CONS) = ISETONE(RCON)
      IREC(IREC_CTRANS) = ISETONE(TQMMM)
      IREC(IREC_XTRP) = ISETONE(TEXTRAP.AND.RXTP)
      IREC(IREC_SHOP) = ISETONE(TSHOPRES.AND.(.NOT.TFUSI))
      IREC(IREC_SHOPBO) = ISETONE(TSHOPRES.AND.TMDBO.AND.(.NOT.TFUSI))
      IREC(IREC_PRNG) = ISETONE(RPRNG)
      IREC(IREC_GLE) = ISETONE(RGLE.AND.MD.AND.(GLE_MODE.NE.0))
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE WRITE_IREC(IREC)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'irat.inc'
      INCLUDE 'store.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'clas.inc'
      INCLUDE 'nose.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'gle.inc'
      INTEGER IREC(100), ISETONE, MSGLEN
C     ==--------------------------------------------------------------==
      CALL IAZZERO(IREC,100)
      IREC(IREC_INFO) = 1
      IREC(IREC_CELL) = 1
      IREC(IREC_NAS)  = 1
      IREC(IREC_CO)   = 1
      IREC(IREC_VEL)  = ISETONE(.NOT.KSENER)
      IREC(IREC_CUT)  = 1
      IREC(IREC_SDP)  = 1
      IREC(IREC_ICO)  = 1
      IREC(IREC_CUT)  = 1
      IREC(IREC_SDP)  = 1
      IREC(IREC_WF)   = ISETONE(SWF)
      IF(TONLYDIAG) IREC(IREC_WF)   = 0
C     By default, PW velocities are not stored for diagonalization.
      IREC(IREC_PWV)  = ISETONE(.NOT.(TDIAG.OR.GEOPT.OR.WFOPT))
C     By default, accumulators are stored for MD and GEOMETRY.
      IREC(IREC_AC)   = ISETONE((GEOPT.OR.MD).AND..NOT.TDEBACC)
C     Nose information
      IREC(IREC_NOSE) = ISETONE(TNOSEE.OR.TNOSEP.OR.TNOSEC)
      IREC(IREC_NOE)  = ISETONE(TNOSEE)
      IREC(IREC_NOP1) = ISETONE(TNOSEP.AND.
     *                         .NOT.(TULTRA.OR.TMNOSE.OR.TLOCT))
      IREC(IREC_NOP2) = ISETONE(TNOSEP.AND.TULTRA)
      IREC(IREC_NOP3) = ISETONE(TNOSEP.AND.TMNOSE)
      IREC(IREC_NOP4) = ISETONE(TNOSEP.AND.TLOCT)
      IREC(IREC_NOC)  = ISETONE(TNOSEC)
C     By default, the potential are stored for Kohn-Sham calculation
      IREC(IREC_POT)  = ISETONE(SPOT)
C     By default, the cell information are stored
      IREC(IREC_HE)   = 1
C     K-points
      IREC(IREC_KPT)  = ISETONE(TKPNT)
C     By default, density is stored for diagonalization
      IREC(IREC_RHO)  = ISETONE(SRHO)
C     By default, the occupation numbers are stored for diagonalization
      IREC(IREC_OCC)  = ISETONE(TDIAG)
C     By default, the fermi energy and eigenvalues are stored for diag.
      IREC(IREC_EIGV) = ISETONE(TDIAG)
      IREC(IREC_CLAS) = ISETONE(TCLAS)
C     TDDFT
      IREC(IREC_LRWF) = ISETONE(TDDFT.OR.TSPEC)
      IREC(IREC_LRWV) = 0
C     The partial Hessian is written whenever it is valid
      IREC(IREC_PHES) = ISETONE(PRFO)
C     The linear scaling optimizers should always dump their status
      IREC(IREC_LBFGS)= ISETONE(LBFGS)
      IREC(IREC_PRFO) = ISETONE(PRFO)
      IREC(IREC_RDTL) = ISETONE(TOLENE.NE.0.0D0.OR.TOLAD.NE.0.0D0)
      IREC(IREC_CONS) = ISETONE(MCNSTR.NE.0)
      IREC(IREC_CTRANS) = ISETONE(TQMMM)
      IREC(IREC_XTRP) = ISETONE(TEXTRAP.AND.TSTRXTP)
      IREC(IREC_SHOP) = ISETONE(TSHOP.AND.(.NOT.TFUSI))
      IREC(IREC_SHOPBO) = ISETONE(TSHOP.AND.TMDBO.AND.(.NOT.TFUSI))
      IREC(IREC_PRNG) = ISETONE(IPRNG.NE.0)
      IREC(IREC_GLE) = ISETONE(GLE_MODE.NE.0)

      MSGLEN=100* 8/IRAT
      CALL MY_BCAST(IREC,MSGLEN,SOURCE,ALLGRP)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION ISETONE(ZLOG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      LOGICAL ZLOG
      INTEGER ISETONE
C     ==--------------------------------------------------------------==
      IF (ZLOG) THEN
        ISETONE = 1
      ELSE
        ISETONE = 0
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
