C     ==================================================================
      SUBROUTINE SETSC
C     ==--------------------------------------------------------------==
C     == INITIALIZATION                                               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'cell.inc'
      INCLUDE 'prcp.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'bc.inc'
      INCLUDE 'clas.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'symm.inc'        !XTABLE
C     Variables
      REAL*8    DELTAKIN
      PARAMETER (DELTAKIN=1.D-10)
      INTEGER INDX(4),KK,I,I1,I2
      REAL*8  XL(4),SINN,BBBB,CCCC,EEDIF,FACT
#ifdef PARALLEL
      INTEGER MSGLEN
#endif
C     ==--------------------------------------------------------------==
C     Go to message passing
      IF(.NOT.PARENT) GOTO 9999
C     ==--------------------------------------------------------------==
C     ==  SETUP DIMENSIONS OF THE SUPERCELL FOR APPLYING PBC          ==
C     ==--------------------------------------------------------------==
      IF(IBRAV.EQ.-2 .OR. TCELLVECTORS) THEN
C       LATTICE VECTORS GIVEN
        IF(.NOT.BOHR) THEN
          DO I=1,3
            A1(I) = A1(I)*FBOHR
            A2(I) = A2(I)*FBOHR
            A3(I) = A3(I)*FBOHR
          ENDDO
        ENDIF
        CALL GENLAT(A1,A2,A3,CELLDM,OMEGA)
      ELSE
        IF(.NOT.BOHR) CELLDM(1)=CELLDM(1)*FBOHR
        IF(.NOT.BOHR) CELLRF(1)=CELLRF(1)*FBOHR
        IF(.NOT.BOHR) RCERF=RCERF*FBOHR
      ENDIF

C     ==--------------------------------------------------------------==
      IF(TCELLREFVEC) THEN
C       LATTICE VECTORS GIVEN
        IF(.NOT.BOHR) THEN
          DO I=1,3
            AR1(I) = AR1(I)*FBOHR
            AR2(I) = AR2(I)*FBOHR
            AR3(I) = AR3(I)*FBOHR
          ENDDO
        ENDIF
        CALL GENLAT(AR1,AR2,AR3,CELLRF,OMEGA)
      ELSEIF(CELLRF(1).LE.0.0D0) THEN
        CALL DCOPY(6,CELLDM(1),1,CELLRF(1),1)
        IF(TCELLVECTORS) THEN
          TCELLREFVEC = .TRUE.
          DO I=1,3
            AR1(I) = A1(I) 
            AR2(I) = A2(I)
            AR3(I) = A3(I)
          ENDDO
        ENDIF
      ENDIF
      IF(TPRCP.AND.(.NOT.TISOT))THEN
C       Triclinic cell
        ALAT=CELLRF(1)
      ELSEIF(IBRAV.EQ.-2 .OR. TCELLVECTORS) THEN
C       Triclinic cell
        ALAT=CELLRF(1)
      ELSE
        SINN=DSQRT(1.D0-CELLDM(4)**2)
        ALAT=CELLRF(1)
        IF(IBRAV.EQ.1) THEN
C         Simple cubic
          APBC(1)=ALAT/2.D0
        ELSEIF(IBRAV.EQ.2) THEN
C         Face centred cubic
          APBC(1)=ALAT*DSQRT(2.D0)/4.D0
        ELSEIF(IBRAV.EQ.4) THEN
C         Hexagonal
          APBC(1)=ALAT/2.D0
          APBC(2)=ALAT*SQRT(3.D0)/4.D0
          APBC(3)=ALAT/2.D0*CELLRF(3)
          APBC(4)=SQRT(3.D0)/3.D0
        ELSEIF(IBRAV.EQ.8) THEN
C         Orthorhombic
          APBC(1)=ALAT/2.D0
          APBC(2)=ALAT/2.D0*CELLRF(2)
          APBC(3)=ALAT/2.D0*CELLRF(3)
        ELSEIF(IBRAV.EQ.12) THEN
CJOK      NEW GEOMETRY PROGRAMMED (JOK)
          BBBB=ALAT*CELLRF(2)*CELLRF(4)
          CCCC=ALAT*CELLRF(2)*SINN
C         LOOK FOR THE CLOSEST IMAGES TO BUILD THE WIGNER-SEITZ CELL.
          DO KK=1,4
            XL(KK)=(ALAT-DBLE(KK-1)*BBBB)**2+(DBLE(KK-1)*CCCC)**2
          ENDDO
C         ORDER THE IMAGES WITH INCREASING DISTANCES
          CALL SORT2(XL,4,INDX)
C         HERE WE DEFINE THE DIMENSIONS OF THE W-S CELL. APBC(1),APBC(2)
C         AND APBC(4) ARE IN THE (X,Y) PLANE, WHERE THE CELL IS A NON-
C         REGULAR HEXAGON; WHILE APBC(3) CORRESPONDS TO THE Z-DIRECTION.
C         THE HEXAGON IS DEFINED BY THE 3 CLOSEST IMAGES IN THE (X,Y)
C         PLANE. TWO OF THEM WERE CALCULATED BEFORE, AND THE 3RD (THE
C         SMALLEST ONE) IS THAT IN THE DIRECTION OF THE LATTICE VECTOR
C         NUMBER 2 (CELLDM(2)).
          APBC(1)=DSQRT(XL(1))/2.D0
          APBC(2)=ALAT/2.D0*CELLRF(2)
          APBC(3)=ALAT/2.D0*CELLRF(3)
          APBC(4)=DSQRT(XL(2))/2.D0
        ELSEIF(IBRAV.EQ.14) THEN
C         Special case of triclinic: monoclinic with gamma<>90 degrees
          IF(ABS(CELLRF(4)).LT.1.D-10.AND.ABS(CELLRF(5)).LT.1.D-10) THEN
            APBC(1)=ALAT/2.D0
            APBC(2)=ALAT/2.D0*CELLRF(2)*SQRT(1.D0-CELLRF(6)**2)
            APBC(3)=ALAT/2.D0*CELLRF(3)
            APBC(4)=-CELLRF(6)/SQRT(1.D0-CELLRF(6)**2)
          ELSE
            APBC(2)=0.0D0
          ENDIF
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  CONSTANTS RELATED TO THE DIMENSIONS OF THE RECIPROCAL CELL  ==
C     ==--------------------------------------------------------------==
      TPIBA=2.D0*PI/ALAT
      TPIBA2=TPIBA*TPIBA
      GCUTW=ECUT/TPIBA2
      IF(AKIN.GT.DELTAKIN) THEN
        GCKIN=ECKIN/TPIBA2
        GAKIN=AKIN/TPIBA2
        GSKIN=SKIN/TPIBA2
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  DUAL APPROXIMATION  ***  BE CAREFUL ***                     ==
C     ==--------------------------------------------------------------==
      IF(DUAL) THEN
        IF(CDUAL.LT.1.D0) CDUAL=1.D0
      ELSE
        CDUAL=4.0D0
      ENDIF
      GCUT=GCUTW*CDUAL
C     ==--------------------------------------------------------------==
C     ==  METRIC TENSOR OF SUPERCELL                                  ==
C     ==--------------------------------------------------------------==
      IF(TCELLVECTORS) THEN
        DO I=1,3
          HT(1,I) = A1(I)
          HT(2,I) = A2(I)
          HT(3,I) = A3(I)
        ENDDO
      ELSEIF(IBRAV.GE.0) THEN
        IF(TPRCP) THEN
          CALL LATGEN(IBRAV,CELLDM,A1,A2,A3,OMEGA)
        ELSE
          CALL LATGEN(IBRAV,CELLDM,A1,A2,A3,OMEGA)
          CALL CRY(IBRAV,A1,A2,A3,INDX)
        ENDIF
        DO I=1,3
          HT(1,I) = A1(I)
          HT(2,I) = A2(I)
          HT(3,I) = A3(I)
        ENDDO
      ENDIF 
      CALL IHMAT(HT,HTM1,OMEGA)
      CALL DCOPY(9,  HT(1,1),1,  HT0(1,1),1)
      CALL DCOPY(9,HTM1(1,1),1,HTM10(1,1),1)

      DO I=1,3
        FACT=1.0D0/DSQRT(HT(I,1)**2+HT(I,2)**2+HT(I,3)**2)
        HUNIT(I,1)=FACT*HT(I,1)
        HUNIT(I,2)=FACT*HT(I,2)
        HUNIT(I,3)=FACT*HT(I,3)
      ENDDO

      OMEGA0=OMEGA
C     ==--------------------------------------------------------------==
C     ==  GENERATE LATTICE VECTORS                                    ==
C     ==--------------------------------------------------------------==
      IF(TCELLREFVEC) THEN
        DO I = 1,3
          A1(I)=AR1(I)
          A2(I)=AR2(I)
          A3(I)=AR3(I)
        ENDDO
        CALL OMEGAGEN(A1,A2,A3,OMEGA0)
      ELSEIF(IBRAV.GE.0) THEN
        IF(TPRCP) THEN
          CALL LATGEN(IBRAV,CELLRF,A1,A2,A3,OMEGA0)
        ELSE
          CALL LATGEN(IBRAV,CELLRF,A1,A2,A3,OMEGA0)
          CALL CRY(IBRAV,A1,A2,A3,INDX)
        ENDIF
      ENDIF 
      IF(IBRAV.EQ.-2) THEN
C       LATTICE VECTORS ARE GIVEN
C       WE USE TRICLINIC SYSTEM
        IBRAV=14
      ENDIF

C     ==--------------------------------------------------------------==
C     ==  SET POINT GROUP TRANSFORMATION MATRICES                     ==
C     ==--------------------------------------------------------------==
      CALL IAZZERO(STABLE,9*48)
      CALL AZZERO(XTABLE,9*120)
      IF(INDPG.NE.0.AND..NOT.TMSYM) THEN
        IF    (IBRAV.EQ.1) THEN
          CALL PGSYM(INDPG,STABLE,NROT)
        ELSEIF(IBRAV.EQ.2) THEN
          CALL FCCSYM(INDPG,STABLE,NROT)
        ELSEIF(IBRAV.EQ.3) THEN
          CALL BCCSYM(INDPG,STABLE,NROT)
        ELSEIF(IBRAV.EQ.4) THEN
          CALL PGSYM(INDPG,STABLE,NROT)
        ELSEIF(IBRAV.EQ.5) THEN
          CALL TRGSYM(INDPG,STABLE,NROT)
        ELSEIF(IBRAV.EQ.6) THEN
          CALL PGSYM(INDPG,STABLE,NROT)
        ELSEIF(IBRAV.EQ.7) THEN
          CALL BCTSYM(INDPG,STABLE,NROT)
        ELSEIF(IBRAV.EQ.8) THEN
          CALL PGSYM(INDPG,STABLE,NROT)
        ELSEIF(IBRAV.EQ.10) THEN
          CALL ALFASYM(INDPG,STABLE,NROT)
        ELSEIF(IBRAV.EQ.12) THEN
          CALL PGSYM(INDPG,STABLE,NROT)
        ELSEIF(IBRAV.EQ.14) THEN
          CALL PGSYM(INDPG,STABLE,NROT)
        ENDIF
        DO I=1,NROT
          DO I1=1,3
            DO I2=1,3
C             (T. Deutsch 29/1/98 change to transpose)
C             XTABLE(I1,I2,I)=DBLE(STABLE(I1,I2,I))
              XTABLE(I1,I2,I)=DBLE(STABLE(I2,I1,I))
            ENDDO
          ENDDO
        ENDDO
      ELSEIF(INDPG.NE.0.AND.TMSYM) THEN
        CALL MOLSYM(STAG,NAXIS,XTABLE,NROT)
      ENDIF
C     ==--------------------------------------------------------------==
C     ==  CLASSICAL BOX                                               ==
C     ==--------------------------------------------------------------==
      IF(CELLCL(1).EQ.0.0D0) THEN
        CALL DCOPY(6,CELLDM(1),1,CELLCL(1),1)
      ENDIF
      EEDIF=0.D0
      DO I=1,6
        EEDIF=EEDIF+(CELLDM(I)-CELLCL(I))**2
      ENDDO
      IF(EEDIF.GT.1.D-12) THEN
        IF(.NOT.TCLUST) THEN
          WRITE(6,*) ' ISOLATED SYSTEM OPTION REQUIRED FOR '
          WRITE(6,*) ' NONEQUAL CLASSICAL AND QUANTUM CELLS'
          CALL STOPGM('SETSC','SPECIFY IBRAV=0')
        ELSEIF(CELLCL(1).LT.CELLDM(1)) THEN
          WRITE(6,*) ' CLASSICAL CELL < QUANTUM CELL '
          CALL STOPGM('SETSC','CL CELL < QM CELL')
        ENDIF
      ENDIF
      CALL LATGEN(IBRAV,CELLCL,CLA1,CLA2,CLA3,CLOMEGA)
      DO I=1,3
        CLHT(1,I) = CLA1(I)
        CLHT(2,I) = CLA2(I)
        CLHT(3,I) = CLA3(I)
      ENDDO
      CALL IHMAT(CLHT,CLHTM1,CLOMEGA)
C     ==--------------------------------------------------------------==
 9999 CONTINUE
#ifdef PARALLEL
C..PARM
      MSGLEN = 17 * 8
      CALL MY_BCAST(ALAT,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 4 * 8/IRAT
      CALL MY_BCAST(IBRAV,MSGLEN,SOURCE,ALLGRP)
C..PRCP
      MSGLEN = MPRCP * 8
      CALL MY_BCAST(AKIN,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(TISOT,MSGLEN,SOURCE,ALLGRP)
      CALL MY_BCAST(TZFLEX,MSGLEN,SOURCE,ALLGRP)
C..METR
      MSGLEN = 6*9 * 8
      CALL MY_BCAST(HT(1,1),MSGLEN,SOURCE,ALLGRP)
C..BC
      MSGLEN = 12 * 8
      CALL MY_BCAST(V1CRYS,MSGLEN,SOURCE,ALLGRP)
C..GVEC
      MSGLEN = 11 * 8
      CALL MY_BCAST(B1,MSGLEN,SOURCE,ALLGRP)
C..CELLDM
      MSGLEN = 15 * 8
      CALL MY_BCAST(CELLDM,MSGLEN,SOURCE,ALLGRP)
C..TCELLVECTORS
      MSGLEN = 2 * 8/IRAT
      CALL MY_BCAST(TCELLVECTORS,MSGLEN,SOURCE,ALLGRP)
C..CELLCL
      MSGLEN = 34 * 8
      CALL MY_BCAST(CELLCL,MSGLEN,SOURCE,ALLGRP)
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE IHMAT(HT,HTM1,OMEGA)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8  HT(3,3),HTM1(3,3),OMEGA
C     Variables
      REAL*8  AUX(1000),S
      INTEGER I,J,K,IPERM,L,INFO
C     ==--------------------------------------------------------------==
      CALL DCOPY(9,HT(1,1),1,HTM1(1,1),1)
      CALL INVMAT(3,HTM1,AUX,INFO)
C     Compute the volume of the Supercell (OMEGA) 
      OMEGA=0.D0
      S=1.D0
      I=1
      J=2
      K=3
 101  CONTINUE
      DO IPERM=1,3
        OMEGA=OMEGA+S*HT(1,I)*HT(2,J)*HT(3,K)
        L=I
        I=J
        J=K
        K=L
      ENDDO
      I=2
      J=1
      K=3
      S=-S
      IF(S.LT.0.D0) GO TO 101
      OMEGA=DABS(OMEGA)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
