c     ==================================================================
      SUBROUTINE propagate_c(nstate,neltra,shprob)
c     ==--------------------------------------------------------------==
      implicit none
      INCLUDE 'system.h'
      INCLUDE 'linres.inc'
c     Arguments
      integer    nstate,neltra
      real*8     shprob(neltra,*)
c     Variables
      integer    it,iv,is,k,j
      real*8     rdaux(neltra,neltra),vaux(neltra),t1,t2,timec
      complex*16 caux(neltra),cnew(neltra),cdot(neltra)
      pointer    (ip_cdot,cdot),(ip_cnew,cnew)
      pointer    (ip_rdaux,rdaux),(ip_vaux,vaux),(ip_caux,caux)
      dimension  v_sh(2,neltra),c_sh(2,neltra),shsigma(2,neltra,neltra)

      call memory(ip_rdaux,neltra*neltra,'rdaux')
      call memory(ip_vaux,neltra,'vaux')
      call memory(ip_caux,2*neltra,'caux')
      call memory(ip_cnew,2*neltra,'cnew')
      call memory(ip_cdot,2*neltra,'cdot')

      call azzero(rdaux,neltra*neltra)
      call azzero(vaux,neltra)
      call zazzero(caux,neltra)
      call zazzero(cnew,neltra)
      call zazzero(cdot,neltra)
c     propagate coefficients
      if(parent) write(6,'(/," PROPAGATION OF COEFFICIENTS")')
      do it=1,neltra
        caux(it)=c_sh(2,it)
        vaux(it)=v_sh(2,it)
        do iv=1,neltra
          rdaux(it,iv)=shsigma(2,it,iv)
        enddo
      enddo
      call deriv_sh(caux,cdot,rdaux,vaux,neltra)
      t1=timec()
      call rk4(caux,neltra,cdot,cnew,shprob)
      t2=timec()
      if(parent)then
       WRITE(6,'(/,A,F9.3)')
     &    " TIME FOR RUNGE-KUTTA INTEGRATION [s]",(T2-T1)*0.001D0
      endif
      do it=1,neltra
        c_sh(1,it)=cnew(it)
      enddo
c
      call freem(ip_rdaux)
      call freem(ip_vaux)
      call freem(ip_caux)
      call freem(ip_cdot)
      call freem(ip_cnew)
c     ==--------------------------------------------------------------==
      return
      end
c     ==================================================================
      subroutine deriv_sh(caux,cdot,rdaux,vaux,neltra)
c     ==--------------------------------------------------------------==
      implicit none
      INCLUDE 'system.h'
c
c     Arguments
      integer    neltra
      real*8     rdaux,vaux
      complex*16 caux,cdot
      
c     Variables
      integer    it,iv
      complex*16 rdauxcom,vauxcom
      dimension  caux(neltra),cdot(neltra),rdaux(neltra,neltra)
      dimension  vaux(neltra)

      call zazzero(cdot,neltra)
c
      do it=1,neltra
        cdot(it)=(0.0D0,-1.0D0)*vaux(it)*caux(it)
      enddo
      do it=1,neltra 
       do iv=1,neltra
         cdot(it)=cdot(it)-caux(iv)*rdaux(it,iv)
       enddo
      enddo
c
      return   
      end
c     ==================================================================
      subroutine rk4(cold,neltra,dydx,cnew,shprob)
c     ==--------------------------------------------------------------==
      implicit none
      INCLUDE 'system.h'
      INCLUDE 'tpar.inc'
      INCLUDE 'linres.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'nose.inc'
      INCLUDE 'spin.inc'

c     Arguments
      integer    neltra
      real*8     shprob(neltra,*)
      complex*16 cold(neltra),dydx(neltra),cnew(neltra)
      
c     Variables
      integer    ix,it,iv,ia,nspline,msglen
      complex*16 dyt(*),dym(*),a_matrix(neltra,neltra)
      pointer    (ip_dyt,dyt),(ip_dym,dym),(ip_vaux,vaux)
      pointer    (ip_a_matrix,a_matrix)
      real*8     hh,h6,vaux(neltra),rdaux(neltra,neltra)
      real*8     xranf,proba_max,proba_min
      real*8     delta_rd(neltra,neltra),delta_v(neltra),test
      dimension  v_sh(2,neltra),shsigma(2,neltra,neltra)
c     interpolation parameters----------------
      real*8     fdata(nint(1.0/step_diff)*2+1,neltra,neltra)
      real*8     vdata(nint(1.0/step_diff)*2+1,neltra)
      pointer    (ip_fdata,fdata),(ip_vdata,vdata)
c
      if(parent)then
        write(6,*) 'FOURTH ORDER RUNGE-KUTTA INTEGRATION'
        write(6,'(A,E13.5)') ' integration step:', step_diff
      endif
      nspline=nint(1.0/step_diff)*2
      hh=step_diff*delt_ions/2.D0
      h6=step_diff*delt_ions/6.D0
c
      call memory(ip_dyt,2*neltra,'dyt')
      call memory(ip_dym,2*neltra,'dym')
      call memory(ip_vaux,neltra,'vaux')
      call memory(ip_fdata,(nspline+1)*neltra*neltra,'fdata')
      call memory(ip_vdata,(nspline+1)*neltra,'vdata')
      call zazzero(dyt,neltra)
      call zazzero(dym,neltra)
      call azzero(vaux,neltra)
      call azzero(vdata,(nspline+1)*neltra)
      call azzero(fdata,(nspline+1)*neltra*neltra)
      call memory(ip_a_matrix,2*neltra*neltra,'a_matrix')
      call zazzero(a_matrix,neltra*neltra)
      call azzero(shprob,neltra*neltra)
c
c     begin linear interpolation of shsigma and v_sh
      do it=1,neltra
        do iv=1,neltra
           delta_rd(it,iv)=0.5*step_diff*(shsigma(1,it,iv)-
     &                                    shsigma(2,it,iv))
        enddo
        delta_v(it)=0.5*step_diff*(v_sh(1,it)-v_sh(2,it))
      enddo
      do ia = 1,nspline+1
         do it=1,neltra
            do iv=1,neltra
              fdata(ia,it,iv)=shsigma(2,it,iv)+(ia-1)*delta_rd(it,iv)
            enddo
            vdata(ia,it)=v_sh(2,it)+(ia-1)*delta_v(it)
         enddo
      enddo
c     end interpolation
c
c     Comment on the interpolation:
c     the time evolution occurs between odd interpolation points:
c     1 to 3 to 5 to ... each of length h.
c     the values of "shsigma" and "v_sh" at the even interpolation points
c     2, 4, 6, ... are used to compute the derivatives (deriv_sh) at 
c     the mid-step (time hh)
c
      do ix=1,nspline-1,2
c
c       The RK4 STEPS -------------------------------------------------
c       update coefficients and parameters to the mid-step 
c       (even spline index)
c       propagation time hh
        do it=1,neltra
           cnew(it)=cold(it)+hh*dydx(it)
           vaux(it)=vdata(ix+1,it)
           do iv=1,neltra
              rdaux(it,iv)=fdata(ix+1,it,iv)
           enddo
        enddo
c       ... and compute the new derivative
        call deriv_sh(cnew,dyt,rdaux,vaux,neltra)
c
c       Re-update the coefficients with the new derivative
c       propagation time hh
        do it=1,neltra
           cnew(it)=cold(it)+hh*dyt(it)
        enddo
c       ... and compute the new derivative
        call deriv_sh(cnew,dym,rdaux,vaux,neltra)
c
c       Update coefficients and parameters to the full-step: h
        do it=1,neltra
           cnew(it)=cold(it)+2.D0*hh*dym(it)
           vaux(it)=vdata(ix+2,it)
           do iv=1,neltra
              rdaux(it,iv)=fdata(ix+2,it,iv)
           enddo
c          compute the sum of the derivatives
           dym(it)=dyt(it)+dym(it)
        enddo
c       Compute the new derivatives at the full step
        call deriv_sh(cnew,dyt,rdaux,vaux,neltra)
c       The funal coefficients are
        do it=1,neltra
           cnew(it)=cold(it)+h6*(dydx(it)+dyt(it)+2.D0*dym(it))
        enddo
c       The RK4 STEPS -------------------------------------------------
c
c       Prepare for the next itaration 
        call deriv_sh(cnew,dydx,rdaux,vaux,neltra)
        do it=1,neltra
          cold(it)=cnew(it)
        enddo
c
c       collect quantities for the transition probability
        do it=1,neltra
         a_matrix(it,fstate+1)=cnew(it)*dconjg(cnew(fstate+1))
        enddo
        do it=1,neltra
             shprob(it,fstate+1)=shprob(it,fstate+1) +
     &         (dreal(a_matrix(it,fstate+1))*rdaux(it,fstate+1))
     &         /dreal(a_matrix(fstate+1,fstate+1))
        enddo
      enddo
c
c     compute the transition probability
      do it=1,neltra
        shprob(it,fstate+1)=
     &        -2.D0 * shprob(it,fstate+1) * (delt_ions * step_diff)
c
      enddo
c
      call freem(ip_a_matrix)
      call freem(ip_dyt)
      call freem(ip_dym)
      call freem(ip_vaux)
      call freem(ip_fdata)
      call freem(ip_vdata)
c     ==--------------------------------------------------------------==     
      return
      end
c     ==================================================================
      SUBROUTINE DO_TULLYTEST(neltra,shprob)
c     ==--------------------------------------------------------------==
      implicit none
      INCLUDE 'system.h'
c     INCLUDE 'tpar.inc'
      INCLUDE 'linres.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'nose.inc'
      INCLUDE 'fileopen.inc'
      INCLUDE 'mm_dim.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'mm_input.inc'

c     Arguments
      integer    neltra
      real*8     shprob(neltra,*)

c     Variables
      logical    debug,tfg
      integer    it,msglen,cqmd,IS,IA
      real*8     xranf,test,proba_max,proba_min,qmdoff,dene
      dimension  v_sh(2,neltra)
c
      debug=.true.
      proba_max=0.D0
      proba_min=0.D0
      test=xranf()

      if(parent)then
        do it=1,neltra
         if(it.ne.fstate+1)then
           if(.not.qmmm)then
            dt_sh=(v_sh(1,fstate+1)-v_sh(1,it))*factem*2.D0/glib
           else
            cqmd=0
            DO IS=1,NSPq
              DO IA=1,NAq(IS)
                  cqmd=cqmd+1
              ENDDO
            ENDDO
            qmdoff = 3*dble(cqmd)
            dt_sh=(v_sh(1,fstate+1)-v_sh(1,it))*factem*2.D0/qmdoff
           endif
           proba_min=proba_max
           if(shprob(it,fstate+1).lt.0)then
C            write(6,*) 
C    &        'neg. probability',it,shprob(it,fstate+1),'set to 0'
             shprob(it,fstate+1)=0.D0
           endif
           test=xranf()
           proba_max=proba_max+shprob(it,fstate+1)
           if (debug) then
            write(6,'(/,A,I3,A,I3,A)') 
     &            ' PROBABILTY OF A SURFACE HOP FROM STATE',
     &              fstate,' TO',it-1,':'
            write(6,'(A,F10.6,A,F10.6,A)') 
     &            '   Interval      : [',proba_min,',',proba_max,']'
            write(6,'(A,F10.6)') 
     &            '   Diff          :', proba_max-proba_min
            write(6,'(A,F10.6,/)') '   Random number :',test
           endif
c          if random number in the right interval
           if (tdextpot) then
            if(test.lt.proba_max.and.test.gt.proba_min)then
c             the external field must contain a quanta of the correct frequency
              dene=v_sh(1,it)-v_sh(1,fstate+1)
c             call check_fermi_golden_rule(dene,tfg)
              call sh_photon_ex(dene)
c             if (tfg) then
                write(6,'(/,A,I4,A,I4,/)')
     &         ' SH TRANSITION FROM STATE ',fstate,'TO STATE ',it-1
               fstate=it-1
               tully_sh=.true.
               if(it.ne.1)then
                s0_sh=.false.
               else
                s0_sh=.true.
               endif
C              no rescaling of the velocities in this case
               dt_sh=0.D0
               goto 100
c             endif
            endif 
           else
            if(test.lt.proba_max.and.test.gt.proba_min)then
              if (dt_sh.gt.tempp_sh) then
               write(6,'(/,A,I4,A,I4,A,/)') 
     &         ' SURFACE HOP FROM ',fstate,' TO ',it-1,' IS FRUSTRATED'
              elseif ((it-1).gt.fstate) then
               write(6,'(/,A,I4,A,I4,A,/)')
     &         ' SURFACE HOP FROM ',fstate,' TO ',it-1,' IS FRUSTRATED'
C              NACVs are needed for the computation of the frustrated hops.
C              (S. Hammes-Schiffer, J.C. Tully, J.Chem.Phys. 101, 4657 (1994))
C              NACVs not yet implemented in the TSH dynamics.
              else
               write(6,'(/,A,I4,A,I4,/)') 
     &         ' SH TRANSITION FROM STATE ',fstate,'TO STATE ',it-1
               fstate=it-1
               tully_sh=.true.
               if(it.ne.1)then
                s0_sh=.false.
               else
                s0_sh=.true.
               endif
               goto 100
              endif
            endif
           endif
         endif
        enddo
      endif
100   continue
#ifdef PARALLEL
      msglen = 1 * 8/irat
      call my_bcast(fstate,msglen,source,allgrp)
      call my_bcast(tully_sh,msglen,source,allgrp)
      call my_bcast(s0_sh,msglen,source,allgrp)
#endif
      if (parent) write(6,'(/,A,I5,/)') "RUN ON SURFACE ", fstate
c     ==--------------------------------------------------------------==
      return
      end
c     ==================================================================
      subroutine check_fermi_golden_rule(dene,tfg)
      implicit none
      INCLUDE 'system.h'
c
c     Variables
      real*8  dene
      logical tfg
c
      tfg=.true.
c     ==--------------------------------------------------------------==
      return
      end
c     ==================================================================
      subroutine sh_photon_ex(dene)
c     ==--------------------------------------------------------------==
      implicit none
      include 'linres.inc' 
      real*8   dene
      if ((dene.gt.(afreq-0.1*afreq)).and.
     &    (dene.le.(afreq+0.1*afreq))) then
	sh_phex=.true.
      else
	sh_phex=.false.
      endif
C
      WRITE(6,*) 'WARNING'
      WRITE(6,*) 'sh_phex SETTED to TRUE in sh_util.F line 394'
      sh_phex=.true.
c     ==--------------------------------------------------------------==      
      return
      end
c     ==================================================================


c     ==================================================================

