c     ==================================================================
      SUBROUTINE SIMPLE_MODEL_p(C0,c1,psi,rhoe,scr,lscr,nstate,eirop,
     $                          eivps)
c     ==================================================================
C     This subroutine calculates the interaction energy between molecules
C     using non-orthogonal orbitals and a simplified model for perturbation
C     theory. So far, this requires that each molecule in the system was
C     previously subjected to a gas phase calculation where both its ground
C     state wave function (W0) and its density-dependent energy terms 
C     (EHT+EPSEU+EXC) are saved (ref-wannier-xx.dat and Erhomol files).
C                                                                    DMB.

C     PARALLEL
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'response_p.inc'
      INCLUDE 'prop.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'ddip.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'wann.inc'
      INCLUDE 'coor.inc'
      INCLUDE 'store.inc'
      INCLUDE 'ropt.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'elct.inc'        !F(NSTATE,NKPNT)
      INCLUDE 'ener.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'atwf.inc'        !CATOM ZXMAT XXMAT
      INCLUDE 'nlcc.inc'        !TINLC
      INCLUDE 'kpnt.inc'        !WK
      INCLUDE 'tpar.inc'        !dt2bye
      INCLUDE 'spin.inc'        !NSUP,NSDOWN
      INCLUDE 'linres.inc'      !TXC_ANALYTIC
      INCLUDE 'geq0.inc'        !geq0
      INCLUDE 'soft.inc'        !softexit
      INCLUDE 'fileopen.inc'
c     ==--------------------------------------------------------------==
c     Arguments & common variables
      integer    nstate,lscr
      complex*16 c0(ngw,nstate),c1(ngw,nstate),psi(MAXFFT*NOGRP)
      real*8     scr(2*maxfft,*),rhoe(*)
C     for Daniel s PT (pseudo FNL proj)
      complex*16 eirop(nhg),eivps(nhg)
      DIMENSION S_STAR(NSTATE,NSTATE)
c     ==--------------------------------------------------------------==
C     variables
      integer    is
#ifdef PARALLEL
      INTEGER NXXFUN,MSGLEN
      EXTERNAL NXXFUN
#endif
      DIMENSION  WK(*)
      CHARACTER  TAG*30
      INTEGER    NFTO,LRSCPOT,KD,I,J
      INTEGER NC,irec(100  )
      REAL*8  E_rho_mol(NSTATE)
      POINTER (ip_E_rho_mol,E_rho_mol)
      REAL*8  E_SUM_rho_mol,E_SUM_h0_diag
      REAL*8  E_SUM_h0_off_diag,E_rho_diff
      REAL*8  E_SUM_h0,E0tot,E1tot,Erho1(2)
      REAL*8  E2tot,my_etot_2,Efunct
      COMPLEX*16 h0_c0(NGW,NSTATE)
      POINTER (ip_h0_c0,h0_c0)
      COMPLEX*16 h0_Eij_c0(NGW,NSTATE)
      POINTER (ip_h0_Eij_c0,h0_Eij_c0)
      COMPLEX*16 h0_c1(NGW,NSTATE)
      POINTER (ip_h0_c1,h0_c1)
      COMPLEX*16 v1_c0(NGW,NSTATE)
      POINTER (ip_v1_c0,v1_c0)
      COMPLEX*16 total_force(NGW,NSTATE)
      POINTER (ip_total_force,total_force)
      REAL*8  S_matrix(NSTATE,NSTATE)
      POINTER (ip_S_matrix,S_matrix)
      REAL*8  H_matrix(NSTATE,NSTATE)
      POINTER (ip_H_matrix,H_matrix)
      REAL*8  one_m_S_matrix(NSTATE,NSTATE)
      POINTER (ip_one_m_S_matrix,one_m_S_matrix)
      REAL*8  one_m_S2_matrix(NSTATE,NSTATE)
      POINTER (ip_one_m_S2_matrix,one_m_S2_matrix)
      REAL*8  my_unit_matrix(NSTATE,NSTATE)
      POINTER (ip_my_unit_matrix,my_unit_matrix)
      REAL*8  rho_mol(nnr1)
      POINTER (ip_rho_mol,rho_mol)
      REAL*8  rho1(nnr1)
      POINTER (ip_rho1,rho1)
      REAL*8  vrho0(nnr1)
      POINTER (ip_vrho0,vrho0)
C     occupation number array
      DIMENSION  F(NSTATE,*)
C     scratch array 
      complex*16 cs(NGW,NSTATE)
      POINTER (IP_CS,CS)
C     daniels pt
      real*8     dummy,fnl,dfnl
      LOGICAL FERROR 
c     ==--------------------------------------------------------------==
      IF(TTAU) CALL STOPGM("simple_model_p","No Tau functionals")
C FIRST INITIALISATIONS
C     SCR test
      CALL GIVE_SCR_RSCPOT(LRSCPOT,TAG,.false.)
      CALL TEST_SCR('RSCPOT',TAG,LSCR,LRSCPOT)

C     Zeroing all energy terms to test the calculation
      ETOT=0
      EKIN=0
      EHT=0
      EPSEU=0
      ENL=0
      EXC=0

C     initialising the arrays
      CALL MEMORY(IP_E_rho_mol,NMOL,'E_rho_mol')
      CALL MEMORY(IP_S_matrix,NSTATE*NSTATE,'S_matrix')
      CALL MEMORY(IP_h0_c0,2*NGW*NSTATE,'h0_c0')
      CALL MEMORY(IP_h0_Eij_c0,2*NGW*NSTATE,'h0_Eij_c0')
      CALL MEMORY(IP_h0_c1,2*NGW*NSTATE,'h0_c1')
      CALL MEMORY(IP_v1_c0,2*NGW*NSTATE,'v1_c0')
      CALL MEMORY(IP_total_force,2*NGW*NSTATE,'total_force')
      CALL MEMORY(IP_H_matrix,NSTATE*NSTATE,'H_matrix')
      CALL MEMORY(IP_one_m_S_matrix,NSTATE*NSTATE,'one_m_S_matrix')
      CALL MEMORY(IP_one_m_S2_matrix,NSTATE*NSTATE,'one_m_S2_matrix')
      CALL MEMORY(IP_my_unit_matrix,NSTATE*NSTATE,'my_unit_matrix')
      CALL MEMORY(IP_rho_mol,nnr1,'rho_mol')
      CALL MEMORY(IP_rho1,nnr1,'rho1')
      CALL MEMORY(IP_vrho0,nnr1,'vrho0')
      CALL MEMORY(IP_CS,NC,'CS/tmp   ')
C     S_star is defined in response_p.inc...
      CALL MEMORY(IP_S_star,NSTATE*NSTATE,'S_star')

C NOW WE CAN CALCULATE....
      if (parent) then
         write(6,*)
         write(6,*) '+-------------------------------+'
         write(6,*) '|   SIMPLE INTERACTION MODEL    |'
         write(6,*) '| USING NON-ORTHOGONAL ORBITALS |'
         write(6,*) '+-------------------------------+'
         write(6,*)

         write(6,*) '***  CALCULATING E0  ***'
      endif
C trying the manual way of calculating the energy...(taken from noforce)
C FIRST DENSITY DEPENDENT TERMS ONLY (using non-orthogonal density..)
C     ion-ion interaction
      NFTO=3*NAX*NSX
      CALL AZZERO(FION,NFTO)
      CALL RPIIINT(ESR,TAU0,FION,IESR,.false.)

C     calculate the density and the wrong KE (not orthogonal..)
C     building the density state by state
      CALL AZZERO(rho0,nnr1)

      E_SUM_rho_mol=0.D0

C     opening erhomol file
      CALL FILEOPEN(500,'Erhomol',FO_OLD,FERROR)
C     Did the file open OK?
      IF (FERROR) THEN
         IF (PARENT) THEN
            write(6,*) 'Error opening the file Erhomol...'
            CALL STOPGM('simple_model_p','file open error')
         ENDIF
      ENDIF

C     constructing total density from molecular fragments
      IF (PARENT) THEN
         write(6,*)
         write(6,*) 'Isolated molecular densities'
      ENDIF

      do j=1,NMOL
C        first zeroing the molecular density
         CALL AZZERO(rho_mol,nnr1)
C        loops over states of one molecule
         do i=STATEmin(j),STATEmax(j)
C           calculating density for molecule j state by state
            CALL RHOSOFR(C0(1,i),RHOE,PSI)
C           adding state density rhoe times occupation number to molecular density rho_mol
            call DAXPY(nnr1,F(i,1),RHOE,1,rho_mol,1)
C           adding state density rhoe times occupation number to total density rho0 
            call DAXPY(nnr1,F(i,1),RHOE,1,rho0,1)
         enddo

C        calculating molecular E[rho_mol] terms

C        for now, we just read them from a file...
C        Reading energy of molecule j
         read(500,*) E_rho_mol(j)

C        calculate EHT, EPSEU, EXC
c        CALL VOFRHO(TAU0,FION,rho_mol,PSI,SCR,LSCR,.false.,.false.)
C        storing the corresponding energy terms..
c        E_rho_mol(j)=EHT+EPSEU+EXC
c        print *,'EHT   (TEST   W0)=        ',EHT,' A.U.'
c        print *,'EPSEU (TEST   W0)=        ',EPSEU,' A.U.'
c        print *,'EXC   (TEST   W0)=        ',EXC,' A.U.'
         if(parent) write(6,*) 'molecule',j,'E[rho_mol] = ',E_rho_mol(j)

C        then adding the energies together 
         E_SUM_rho_mol=E_SUM_rho_mol+E_rho_mol(j)
      ENDDO

      if (parent) then
         write(6,*) 
         write(6,*) 'Sum of E[rho_mol] =',E_SUM_rho_mol
         CALL FILECLOSE(500)
      endif
C  NOW USING TOTAL DENSITY
C     first copy rho0 zero to preserve it for later...
      CALL DCOPY(nnr1,rho0,1,vrho0,1)

C     calculate the density dependent parts.. (EHT,EPSEU,EXC)
      CALL VOFRHO(TAU0,FION,vrho0,PSI,SCR,LSCR,.false.,.false.)

      if (parent) then
         write(6,*)
         write(6,*) 'TOTAL DENSITY DEPENDENT TERMS'
         write(6,*) '***** ENERGY DECOMPOSITION *****'
         write(6,*) 'EHT   (       W0)= ',EHT,' A.U.'
         write(6,*) 'EPSEU (       W0)= ',EPSEU,' A.U.'
         write(6,*) 'EXC   (       W0)= ',EXC,' A.U.'
         write(6,*)
         write(6,*) 'E[rho0] =',EHT+EPSEU+EXC
      endif
C END  OF DENSITY ONLY TERMS

C new non-orthogonal correction thing...
      if (parent) then
         write(6,*)
         write(6,*) 'WAVEFUNCTION DEPENDENT TERMS'
      endif
 
C     Calculating H[rho0]|c0i>  CS is a tmp array
      call HPSI(c0,h0_c0,CS,vrho0,PSI,SCR,LSCR,NSTATE,1,1)

C     multiplying h0_c0 by the occupation number
      DO I=1,NSTATE
        CALL DSCAL(2*NGW,F(I,1),h0_c0(1,I),1)
      ENDDO

C     then H_matrix=<c0j|h0_c0>
      call ovlap(nstate,H_matrix,C0,h0_c0)

C     changing sign of H matrix (force->hamiltonian)
      call DSCAL(NSTATE*NSTATE,-1.d0,H_matrix,1)

C     now printing...
      call print_matrix(H_matrix,NSTATE,NSTATE,' H matrix ')

C     Calculating <c0i|c0j> for the non-orthogonal states
      call ovlap(nstate,S_matrix,C0,C0)

C     calculating (1-S) matrix
C          creating a unit matrix
      call UNITMX(my_unit_matrix,NSTATE)
C          copying unit matrix to one_m_S_matrix
      CALL DCOPY(NSTATE*NSTATE,my_unit_matrix,1,one_m_S_matrix,1)
C          then adding -S to unit matrix
      call DAXPY(NSTATE*NSTATE,-1.d0,S_matrix,1,one_m_S_matrix,1)
c     printing
      CALL print_matrix(one_m_S_matrix,NSTATE,NSTATE,'1-S matrix')

C     calculating (1-S)(1-S) matrix
      CALL DGEMM('N','N',NSTATE,NSTATE,NSTATE,1.D0,one_m_S_matrix,
     $        NSTATE,one_m_S_matrix,NSTATE,0.D0,one_m_S2_matrix,NSTATE)
c     printing
      CALL print_matrix(one_m_S2_matrix,NSTATE,NSTATE,'(1-S)2 mat')

C     Building S_star matrix which has (1-S)**2 on its diagonal
C     and (1-S) on the off-diagonal elements
C     copying one_m_S_matrix to S_star
      CALL DCOPY(NSTATE*NSTATE,one_m_S_matrix(1,1),1,S_star(1,1),1)

C     now affecting the (1-S)**2 elements
      do i=1,NSTATE
         S_star(i,i)=one_m_S2_matrix(i,i)
      enddo
c     printing
      CALL print_matrix(S_star,NSTATE,NSTATE,'S_star    ')

C     Now summing the energy terms
      E_SUM_h0_diag=0.d0
      E_SUM_h0_off_diag=0.d0
      DO i=1,NSTATE
         DO j=1,NSTATE
C           off diagonal term (1-S)ij <psi0j|H[rho(0)]|psi0i>
            IF (i .NE. J) THEN
               E_SUM_h0_off_diag=E_SUM_h0_off_diag+
     $                           one_m_S_matrix(i,j)*H_matrix(j,i)
            ENDIF
         ENDDO
C        diagonal term [(1-S)(1-S)]ii <psi0i|H[rho(0)]|psi0i>
         E_SUM_h0_diag=E_SUM_h0_diag+
     $                          one_m_S2_matrix(i,i)*H_matrix(i,i)
      ENDDO 

C     wavefunction terms
      if (parent) then
         write(6,*) 'E[H0]ii (diagonal)=',E_SUM_h0_diag
         write(6,*) 'E[H0]ij (off diagonal)=',E_SUM_h0_off_diag
         write(6,*) 
      endif

C     sum of wavefunction correction terms
      E_SUM_h0=E_SUM_h0_diag+E_SUM_h0_off_diag
      if (parent) write(6,*) 'E[H0] (total)=',E_SUM_h0

C     Density energy difference
      E_rho_diff=EHT+EPSEU+EXC-E_SUM_rho_mol
      if (parent) write(6,*) 'E[rho0]-Sum(E[rho_mol])=',E_rho_diff
    
C     TOTAL interaction energy E0
      E0tot=E_SUM_h0+E_rho_diff
      if (parent) THEN
         write(6,*)
         write(6,*) 'E0tot = E[H0]+E[rho0]-Sum(E[rho_mol]) =',E0tot
C        the same in mH
         write(6,*) 'Interaction energy (mH)=',1000*E0tot
      endif

C     Done with the first part...
      if (parent) then
         write(6,*) 
         write(6,*) '*** FINISHED WITH E0 ***'
      endif
 
C     OPTIMISATION FOR C1
C     it s easier to let the usual PT to finish the job....
      if (parent) then
         write(6,*) 
         write(6,*) '*** CALCULATING DFPT CORRECTIONS ***'
      endif

C     Use Daniel s PT to optimise C1
      call rwfopt_p(c0,c1,psi,rho0,rho1,scr,lscr,eirop,eivps,dummy,
     &              h0_c0,H_matrix,nstate,dummy)

      if (parent) then
         write(6,*) 
         write(6,*) '*** FINISHED WITH DFPT CORRECTIONS ***'
      endif
C     ETOT now contains E1+E2
C     END OF OPTIM FOR C1
      
C     TOTAL interaction energy E0+(E1+E2)
      Efunct=E0tot+ETOT
      if (parent) THEN
         write(6,*)
         write(6,*) 'Efunct =',Efunct
C        the same in mH
         write(6,*) 'EFI energy (mH)=',1000*Efunct
      endif


      if (parent) then
         write(6,*)
         write(6,*) '+-------------------------------+'
         write(6,*) '|   EXITING...                  |'
         write(6,*) '|   SIMPLE INTERACTION MODEL    |'
         write(6,*) '+-------------------------------+'
      endif 
C     simple model finishes here...
C     freeing the memory
      CALL FREEM(IP_E_rho_mol)
      CALL FREEM(IP_S_matrix)
      CALL FREEM(IP_h0_c0)
      CALL FREEM(IP_h0_Eij_c0)
      CALL FREEM(IP_h0_c1)
      CALL FREEM(IP_v1_c0)
      CALL FREEM(IP_H_matrix)
      CALL FREEM(IP_one_m_S_matrix)
      CALL FREEM(IP_one_m_S2_matrix)
      CALL FREEM(IP_S_star)
      CALL FREEM(IP_my_unit_matrix)
      CALL FREEM(IP_rho_mol)
      CALL FREEM(IP_rho1)
      CALL FREEM(IP_vrho0)
      CALL FREEM(IP_CS)
C     ==--------------------------------------------------------------==
      return
      END
c     ==================================================================

C     ==================================================================
      SUBROUTINE optim_DIIS_PRINT(step,grad_max,c_norm,ETOT,DETOT,TCPU)
C     ==--------------------------------------------------------------==
C     prints out step-by-step details of the convergence of DIIS minimisation
      IMPLICIT NONE
      INTEGER step
      REAL*8  grad_max,c_norm
      REAL*8  ETOT,DETOT,TCPU
C     ==--------------------------------------------------------------==
      IF(step.EQ.1) WRITE(6,'(A,A)')
     *               '   NFI      GEMAX     CNORM',
     *               '         ETOT(2)        DETOT     TCPU'
      WRITE(6,'(I6,F11.6,F10.6,F16.8,G13.4,F9.3)') 
     *    step,grad_max,c_norm,ETOT,DETOT,TCPU
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
 
c     ==================================================================
      subroutine simple_ortho_p(nstate,c0,c1,S_star)
c     ==================================================================
C     omS_matrix is (1-S) matrix
C     omS2_matrix is (1-S)**2 matrix
      implicit none
      include 'system.h'
      include 'elct.inc'
      include 'cppt.inc'
      integer     isub,k,j,i,nstate,ifirst,is
      complex*16  c0(ngwk,*),c1(ngwk,*)
      real*8      ovl01,overlap(NSTATE,NSTATE)
      real*8      S_star(NSTATE,NSTATE)
      real*8      star_ovl(NSTATE,NSTATE)
      pointer     (ip_overlap,overlap)
      pointer     (ip_star_ovl,star_ovl)
      save        ifirst,ip_overlap,ip_star_ovl
      data        ifirst /0/
      call tiset('simp_orth_p',isub)

      if (ifirst .EQ. 0) then
        ifirst = 1
        call memory(ip_overlap,nstate*nstate,'ovl/ortho_p')
        call memory(ip_star_ovl,nstate*nstate,'stovl_sim_p')
        call azzero(   overlap,nstate*nstate)
        call azzero(star_ovl,nstate*nstate)
      endif

C     first calculate <c1|c0>
      call ovlap(nstate,overlap,c0,c1)
#ifdef PARALLEL
      call glosum(nstate*nstate,overlap)
#endif

C     then state by state orthog... 
C     zero order term
      call DGEMM('N','N',2*ngw,nstate,nstate,-1.d0,c0,
     &    2*ngw,overlap,nstate,+1.d0,c1,2*ngw)

C     first+second order correction in S matrix
C     calculating (S_star)(overlap) matrix result in star_ovl
      CALL DGEMM('N','N',NSTATE,NSTATE,NSTATE,1.D0,S_star,
     $        NSTATE,overlap,NSTATE,0.D0,star_ovl,NSTATE)
C    Now applying star_ovl to C0 to get the final projector
C    and subtract that to C1
      call DGEMM('N','N',2*ngw,nstate,nstate,-1.d0,c0,
     &    2*ngw,star_ovl,nstate,+1.d0,c1,2*ngw)

      call tihalt('simp_orth_p',isub)
      return
      end
C     ==================================================================
