C     ==================================================================
      SUBROUTINE SSIC(EHSIC,RHOE,V)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     == SELF INTERACTION CORRECTION (SIC) TO THE HARTREE ENERGY AND  ==
C     == RELATED POTENTIAL CONTRIBUTIONS TO THE FORCES                ==
C     == default: asic=0.2d0 bsic=0.0 after PCCP 7, 1363-1367 (2005)  ==
C     == First version: Tsukuba, 01 September 2006                    ==
C     == Revision     : Tsukuba, 13 October   2006                    ==
C     ==--------------------------------------------------------------==
C     == Input:                                                       ==
C     ==   RHOE  Density in R-space (input)                           ==
C     ==   V     Potential in G-space (input)                         ==
C     ==   REMINDER - SPINDEN(i,j,k)=RHOE(i,j,k,1)-2*RHOE(i,j,k,2)    ==
C     ==              RHOE(i,j,k,1) = rho_alpha + rho_beta            ==
C     ==              RHOE(i,j,k,2) = rho_beta                        ==
C     == Output:                                                      ==
C     ==   EHSIC SIC correction to the e-e Hartree term               ==
C     ==   V     V+VSIC potential in R-space (spin-up)                ==
C     ==         V-VSIC potential in R-space (spin-down)              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'irat.inc'
C     Arguments
      COMPLEX*16 V(MAXFFT,NLSD)
      REAL*8     EHSIC,RHOE(NNR1,NLSD)
C     Variables
      INTEGER    IR,IG1,IG
cmb      REAL*8     asic,bsic
#ifdef PARALLEL
      REAL*8     EHSICC(2)
#endif
      COMPLEX*16 RHET,RHETS,EHSICG,SPING(MAXFFT),VSICT(NHG),VSIC(MAXFFT)
C     ==--------------------------------------------------------------==
C     == Transform SPINDEN(IR) into SPINDEN(IG)                       ==
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        SPING(IR)=DCMPLX(RHOE(IR,1)-2.0d0*RHOE(IR,2),0.D0)
      ENDDO
      CALL FWFFT(SPING)
C     ==--------------------------------------------------------------==
      IF(GEQ0) THEN
        IG1=2
        RHET=SPING(NZH(1))
        RHETS=SCG(1)*RHET
        EHSICG=0.5D0*RHETS*RHET
        VSICT(1)=RHETS
      ELSE
        IG1=1
        EHSICG=(0.0D0,0.0D0)
      ENDIF
!$OMP  parallel do private(IG,RHET,RHETS)
!$OMP+ reduction(+:EHSICG)
#ifdef __SR11000
*poption parallel
*poption tlocal(IG,RHET,RHETS)
*poption psum(EHSICG)
#endif 
      DO IG=IG1,NHG
C     ==------------------------------------------------------------==
C     == In VSICT we store temporarily the potential in G-space     ==
C     ==------------------------------------------------------------==
        RHET=SPING(NZH(IG))
        RHETS=SCG(IG)*RHET
C       Hartree e-e SIC potential
        VSICT(IG)=RHETS
C       Hartree e-e SIC energy
        EHSICG=EHSICG+RHETS*DCONJG(RHET)
      ENDDO
#ifdef PARALLEL
      EHSICC(1)=DREAL(EHSICG)
      EHSICC(2)=DIMAG(EHSICG)
      CALL GLOSUM(2,EHSICC)
      EHSIC=-asic*EHSICC(1)*OMEGA
#else
      EHSIC=-asic*DREAL(EHSICG)*OMEGA
#endif
C     ==------------------------------------------------------------==
C     == Go back in R-space and add/subtract VSIC to V(i,j,k,spin)  ==
C     ==------------------------------------------------------------==
      CALL ZAZZERO(VSIC,MAXFFT)
!CDIR NODEP
      DO IG=1,NHG
        VSIC(INDZ(IG))=DCONJG(VSICT(IG))
        VSIC(NZH(IG)) =VSICT(IG)
      ENDDO
      IF(GEQ0) THEN
        VSIC(NZH(1))=VSICT(1)
      ENDIF
      CALL INVFFT(VSIC)
cmb      IF(bsic.eq.0.d0) THEN
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        V(IR,1)=V(IR,1)-asic*VSIC(IR) ! alpha spin
        V(IR,2)=V(IR,2)+asic*VSIC(IR) ! beta  spin
      ENDDO
cmb      ELSE
cmb   put here the +/-(asic*VSIC(IR)+bsic*VXC(IR)) stuff
cmb      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
