C     ==================================================================
      subroutine pgsym(indpg,s,ntot)
C     ==--------------------------------------------------------------==
C     sets up the point-group rotation matrices for all 32 point groups
C     for the simple (p) bravais lattices.
C     These matrices all have integer elements, and express 
C     the relations between the primitive basis vectors 
C     due to each point-group rotation.  
C     The special axis is always the z-axis, one basal-plane
C     vector is always along x, and the other basal-plane vector is at
C     angle beta for monoclinic (beta is not actually used), at 120
C     degrees for trigonal and hexagonal(p) groups, and at 90 degrees
C     for remaining groups.
C     subroutine trgsym takes care of the trigonal(r) groups, subroutine
C     fccsym takes care of the cubic (f) groups, subroutine bccsym takes
C     care of the cubic (i) groups, and subroutine bctsym takes care of
C     the tetragonal (i) groups.  for these latter
C     groups, the crystallographic vectors are chosen
C     differently.  for the triclinic groups, the matrices are
C     simply identity and inversion.
C     ==--------------------------------------------------------------==
C     indpg is the point-group index, defined as follows:
C     ==--------------------------------------------------------------==
C     indpg   group    indpg   group     indpg   group     indpg   group
C     1     1 (c1)      9     3m (c3v)    17   4/mmm(d4h)   25   222(d2)
C     2    <1>(ci)     10     <3>m(d3d)   18   6 (c6)       26  mm2(c2v)
C     3     2 (c2)     11     4 (c4)      19   <6>(c3h)     27  mmm(d2h)
C     4     m (c1h)    12     <4>(s4)     20   6/m(c6h)     28  23 (t)
C     5     2/m(c2h)   13     4/m(c4h)    21   622(d6)      29  m3 (th)
C     6     3 (c3)     14     422(d4)     22   6mm(c6v)     30  432 (o)
C     7     <3>(c3i)   15     4mm(c4v)    23   <6>m2(d3h)   31 <4>3m(td)
C     8     32 (d3)    16    <4>2m(d2d)   24    6/mmm(d6h)  32  m3m(oh)
C     ==--------------------------------------------------------------==
      implicit none
C     Arguments
      integer indpg,s(3,3,48),ntot
C     Variables
      integer sum,i,j,k,kind,jind
C     ==--------------------------------------------------------------==
      if (indpg.lt.1) go to 100
      do k=1,48
        do j=1,3
          do i=1,3
            s(i,j,k)=0
          enddo
        enddo
      enddo
      do i=1,3
        s(i,i,1)=1
      enddo
C     ==--------------------------------------------------------------==
C     == Triclinic groups.                                            ==
C     ==--------------------------------------------------------------==
      if(indpg.eq.1) then
c       point group 1. [1]
        ntot=1
        return
      elseif(indpg.eq.2) then
c       add inversion for point group <1>. [2]
        ntot=2
        do i=1,3
          s(i,i,2)=-1
        enddo
        return
      endif
C     ==--------------------------------------------------------------==
C     == Monoclinic groups.                                           ==
C     ==--------------------------------------------------------------==
      if(indpg.le.5) then
        ntot=2
        do i=1,3
          s(i,i,2)=1
          s(i,i,3)=1
          s(i,i,4)=1
        enddo
        if (indpg.eq.3) then
c         point group 2. [3]
          s(1,1,2)=-1
          s(2,2,2)=-1
          return
        elseif(indpg.eq.4) then
c         point group m. [4]
          s(3,3,2)=-1
          return
        elseif(indpg.eq.5) then  
          ntot=4
c         point group 2/m. [5]
          s(1,1,2)=-1
          s(2,2,2)=-1
          s(3,3,3)=-1
          do i=1,3
            s(i,i,4)=-1
          enddo
          return
        endif
      endif
C     ==--------------------------------------------------------------==
C     == Trigonal groups.                                             ==
C     ==--------------------------------------------------------------==
      if(indpg.le.10) then
c       point group 3. [6]
        ntot=3
        s(1,2,2)=1
        s(2,1,2)=-1
        s(2,2,2)=-1
        s(3,3,2)=1
        s(1,1,3)=-1
        s(1,2,3)=-1
        s(2,1,3)=1
        s(3,3,3)=1
        if(indpg.eq.6) then
          return
        elseif(indpg.eq.8.or.indpg.eq.9) then
          ntot=6
c         set up c2x. 
          s(1,1,4)=1
          s(3,3,4)=-1
          s(2,1,4)=-1
          s(2,2,4)=-1
          if (indpg.eq.9) then
c           set up mx.
            do j=1,3
              do i=1,3
                s(i,j,4)=-s(i,j,4)
              enddo
            enddo
c           Unnecessary (Thierry Deutsch 13/1/98)
            s(3,3,4)=1
          endif
c         point group 32 =3(x)c2x [8] or 3m = 3(x)mx [9].
          do kind=2,3
            do i=1,3
              do j=1,3
                sum=0
                do k=1,3
                  sum=sum+s(i,k,4)*s(k,j,kind)
                enddo
                s(i,j,kind+3)=sum
              enddo
            enddo
          enddo
          return
        elseif(indpg.eq.7.or.indpg.eq.10) then
          ntot=6
c         point group <3> = 3(x)i [7]
          do kind=1,3
            do j=1,3
              do i=1,3
                s(i,j,kind+3)=-s(i,j,kind)
              enddo
            enddo
          enddo
          if (indpg.eq.7) then
            return
          else
c           point group <3>m = <3>(x)mx [10]
            ntot=12
            s(1,1,7)=-1
            s(2,1,7)=1
            s(2,2,7)=1
            s(3,3,7)=1
            do kind=2,6
              do j=1,3
                do i=1,3
                  sum=0
                  do k=1,3
                    sum=sum+s(i,k,7)*s(k,j,kind)
                  enddo
                  s(i,j,kind+6)=sum
                enddo
              enddo
            enddo
          endif
        endif
        return
      endif
C     ==--------------------------------------------------------------==
C     == Tetragonal groups.                                           ==
C     ==--------------------------------------------------------------==
      if(indpg.le.17) then
C       point group 4. [11]
        ntot=4
        s(1,2,2)=1
        s(2,1,2)=-1
        s(3,3,2)=1
        s(1,1,3)=-1
        s(2,2,3)=-1
        s(3,3,3)=1
        s(1,2,4)=-1
        s(2,1,4)=1
        s(3,3,4)=1
        if(indpg.eq.11) then
          return
        elseif(indpg.eq.12.or.indpg.eq.16) then
c         point group <4>. [12]
          do kind=2,4,2
            do j=1,3
              do i=1,3
                s(i,j,kind)=-s(i,j,kind)
              enddo
            enddo
          enddo
          if(indpg.eq.12) then
            return
          elseif(indpg.eq.16) then
c           point group <4>2m = <4>(x)c2x. [16]
            ntot=8
c           the following two cards are wrong (original janak s program)
c           s(1,1,5)= 1
c           s(2,2,5)=-1
c           the following two are the correct cards 
c           (stefano de gironcoli dixit ad 1989, 26 may)
            s(1,2,5)= 1
            s(2,1,5)= 1
            s(3,3,5)=-1
            do kind=2,4
              do j=1,3
                do i=1,3
                  sum=0
                  do k=1,3
                    sum=sum+s(i,k,5)*s(k,j,kind)
                  enddo
                  s(i,j,kind+4)=sum
                enddo
              enddo
            enddo
            return
          endif
        elseif(indpg.eq.13.or.indpg.eq.14.or.
     &         indpg.eq.15.or.indpg.eq.17) then
          ntot=8
          if (indpg.eq.13) then
C           point group 4/m = mz(x)4 [13]
            s(1,1,5)=1
            s(2,2,5)=1
            s(3,3,5)=-1
          elseif (indpg.eq.15) then
C           point group 4mm = mx(x)4 [15]
            s(1,1,5)=-1
            s(2,2,5)=1
            s(3,3,5)=1
          elseif (indpg.eq.14.or.indpg.eq.17) then
C           point groups 422 = c2x(x)4 or 4/mmm = i(x)422. [14]
            s(1,1,5)=1
            s(2,2,5)=-1
            s(3,3,5)=-1
          endif
C         Add 3 rotations products of 2, 3, 4 with 5.
          do kind=2,4
            do j=1,3
              do i=1,3
                sum=0
                do k=1,3
                  sum=sum+s(i,k,5)*s(k,j,kind)
                enddo
                s(i,j,kind+4)=sum
              enddo
            enddo
          enddo
          if (indpg.eq.14) then
            return
          elseif (indpg.eq.17) then
C           Add inversion and products. [17]
            ntot=16
            do kind=1,8
              do j=1,3
                do i=1,3
                  s(i,j,kind+8)=-s(i,j,kind)
                enddo
              enddo
            enddo
          endif
        endif
        return
      endif
C     ==--------------------------------------------------------------==
C     == Hexagonal point groups.                                      ==
C     ==--------------------------------------------------------------==
      if(indpg.le.24) then
        ntot=6
c       point group 6. [18]
        s(1,1,2)=1
        s(1,2,2)=1
        s(2,1,2)=-1
        s(1,2,3)=1
        s(2,1,3)=-1
        s(2,2,3)=-1
        s(1,1,4)=-1
        s(2,2,4)=-1
        s(1,1,5)=-1
        s(1,2,5)=-1
        s(2,1,5)=1
        s(1,2,6)=-1
        s(2,1,6)=1
        s(2,2,6)=1
        do i=2,6
          s(3,3,i)=1
        enddo
        if(indpg.eq.18) then
          return
        elseif (indpg.eq.19.or.indpg.eq.23) then
c         point group <6>. [19]
          do kind=2,6,2
            do j=1,3
              do i=1,3
                s(i,j,kind)=-s(i,j,kind)
              enddo
            enddo
          enddo
          if (indpg.eq.19) return
c         point group <6>m2 = c2y(x)<6>. [23]
          ntot=12
          s(1,1,7)=-1
c         There was a missing card (Thierry Deutsch 13/1/98):
          s(1,2,7)=-1
          s(2,2,7)=1
          s(3,3,7)=-1
          do kind=2,6
            do j=1,3
              do i=1,3
                sum=0
                do k=1,3
                  sum=sum+s(i,k,7)*s(k,j,kind)
                enddo
                s(i,j,kind+6)=sum
              enddo
            enddo
          enddo
          return
        endif
        ntot=12
        if(indpg.eq.20) then
c         point group 6/m = i(x)6 [20]
          do kind=1,6
            do j=1,3
              do i=1,3
                s(i,j,kind+6)=-s(i,j,kind)
              enddo
            enddo
          enddo
          return
        elseif(indpg.eq.22) then
c         point group 6mm = mx(x)6 [22]
c         the following two cards are wrong (original janak s program)
c         s(1,1,7)=-1
c         s(2,2,7)=1
c         the following two are the correct cards
c         (lausanne, 10 june 1988)
          s(1,2,7)=1
          s(2,1,7)=1
          s(3,3,7)=1
        else
c         point groups 622 = c2x(x)6 [21] or 6/mmm = i(x)622. [24]  
c         the following two cards are wrong (original janak s program)
c         48 s(1,1,7)=1
c         s(2,2,7)=-1
c         the following two are the correct cards as sgamed by s. massidda
          s(1,2,7)=1
          s(2,1,7)=1
          s(3,3,7)=-1
        endif
C       Add products 2, 3, 4, 5, 6 with 7
        do kind=2,6
          do j=1,3
            do i=1,3
              sum=0
              do k=1,3
                sum=sum+s(i,k,7)*s(k,j,kind)
              enddo
              s(i,j,kind+6)=sum
            enddo
          enddo
        enddo
        if(indpg.eq.24) then
          return
        else                    ! [21]
C         Add inversion and products
          ntot=24
          do kind=1,12
            do j=1,3
              do i=1,3
                s(i,j,kind+12)=-s(i,j,kind)
              enddo
            enddo
          enddo
        endif
        return
      endif
C     ==--------------------------------------------------------------==
C     == Orthorhombic point groups.                                   ==
C     ==--------------------------------------------------------------==
      if(indpg.le.27) then
        ntot=4
        if(indpg.eq.26) then
c         point group mm2. [26]
          do kind=2,4
            do i=1,3
              s(i,i,kind)=1
            enddo
          enddo
          s(1,1,2)=-1
          s(2,2,3)=-1
          s(1,1,4)=-1
          s(2,2,4)=-1
          return
        endif
c       point group 222. [25]
        do kind=2,4
          do i=1,3
            s(i,i,kind)=-1
          enddo
        enddo
        s(1,1,2)=1
        s(2,2,3)=1
        s(3,3,4)=1
        if(indpg.eq.25) then
          return
        elseif(indpg.eq.27) then
c         point group mmm = i(x)222 [27]
          ntot=8
          do kind=1,4
            do j=1,3
              do i=1,3
                s(i,j,kind+4)=-s(i,j,kind)
              enddo
            enddo
          enddo
        endif
        return
      endif
C     ==--------------------------------------------------------------==
C     == Cubic point groups.                                          ==
C     ==--------------------------------------------------------------==
      if(indpg.le.32) then
        ntot=12
c       point group 23. [28]
        do kind=2,4
          do i=1,3
            s(i,i,kind)=-1
          enddo
        enddo
        s(1,1,2)=1
        s(2,2,3)=1
        s(3,3,4)=1
        s(1,2,5)=1
        s(2,3,5)=1
        s(3,1,5)=1
        s(1,3,6)=1
        s(2,1,6)=1
        s(3,2,6)=1
        do kind=5,6
          do jind=2,4
            do j=1,3
              do i=1,3
                sum=0
                do k=1,3
                  sum=sum+s(i,k,jind)*s(k,j,kind)
                enddo
                s(i,j,kind+2*jind-2)=sum
              enddo
            enddo
          enddo
        enddo
        if(indpg.eq.28) return
        ntot=24
        if(indpg.eq.29) then
c         point group m3 = i(x)23 [29]
          do kind=1,12
            do j=1,3
              do i=1,3
                s(i,j,kind+12)=-s(i,j,kind)
              enddo
            enddo
          enddo
          return
        endif
        if(indpg.eq.30.or.indpg.eq.32) then
c         point group 432 = c4x(x)23 [30] [32]
          s(1,1,13)=1
          s(2,3,13)=1
          s(3,2,13)=-1
          do kind=2,12
            do j=1,3
              do i=1,3
                sum=0
                do k=1,3
                  sum=sum+s(i,k,13)*s(k,j,kind)
                enddo
                s(i,j,kind+12)=sum
              enddo
            enddo
          enddo
          if (indpg.eq.30) return
c         point group m3m = i(x)432 [32]
          ntot=48
          do kind=1,24
            do j=1,3
              do i=1,3
                s(i,j,kind+24)=-s(i,j,kind)
              enddo
            enddo
          enddo
          return
        endif
c       point <4>3m = <c4x>(x)23 [31]
        s(1,1,13)=-1
        s(2,3,13)=-1
        s(3,2,13)=1
        do kind=2,12
          do j=1,3
            do i=1,3
              sum=0
              do k=1,3
                sum=sum+s(i,k,13)*s(k,j,kind)
              enddo
              s(i,j,kind+12)=sum
            enddo
          enddo
        enddo
        return
      endif
C     ==--------------------------------------------------------------==
C     == If indpg exceeds 32 or is less than 1.                       ==
C     ==--------------------------------------------------------------==
 100  continue
      write(6,110) indpg
 110  format(' point-group index',i4,' is not between 1 and 32.',
     +     ' stopping')
      call stopgm('pgsym',' ')
      end
C     ==================================================================
      subroutine fccsym(indpg,s,ntot)
C     ==--------------------------------------------------------------==
c     point group rotation matrices for the fcc bravais lattice.  it is
c     assumed that a1=(a/2)(-1,0,1), a2=(a/2)(0,1,1), a3=(a/2)(-1,1,0).
c     the rotation matrices express how these basis vectors are trans-
c     formed into one another by point-group rotations.
C     ==--------------------------------------------------------------==
      implicit none
C     Arguments
      integer indpg,s(3,3,48),ntot
C     Variables
      integer sum,i,j,k,kind,jind
C     ==--------------------------------------------------------------==
      if (indpg.lt.28.or.indpg.gt.32) go to 100
      do k=1,48
        do j=1,3
          do i=1,3
            s(i,j,k)=0
          enddo
        enddo
      enddo
      do i=1,3
        s(i,i,1)=1
      enddo
      ntot=12
c     point group 23.
      s(1,2,2)=-1
      s(1,3,2)=1
      s(2,2,2)=-1
      s(3,1,2)=1
      s(3,2,2)=-1
      s(1,1,3)=-1
      s(2,1,3)=-1
      s(2,3,3)=1
      s(3,1,3)=-1
      s(3,2,3)=1
      s(1,2,4)=1
      s(1,3,4)=-1
      s(2,1,4)=1
      s(2,3,4)=-1
      s(3,3,4)=-1
      s(1,3,5)=-1
      s(2,2,5)=1
      s(2,3,5)=-1
      s(3,1,5)=1
      s(3,3,5)=-1
      s(1,1,6)=-1
      s(1,3,6)=1
      s(2,1,6)=-1
      s(2,2,6)=1
      s(3,1,6)=-1
      do kind=5,6
        do jind=2,4
          do j=1,3
            do i=1,3
              sum=0
              do k=1,3
                sum=sum+s(i,k,jind)*s(k,j,kind)
              enddo
              s(i,j,kind+2*jind-2)=sum
            enddo
          enddo
        enddo
      enddo
      if (indpg.eq.28) return
      ntot=24
      if (indpg.ne.29) go to 76
c     point group m3 = i(x)23
      do kind=1,12
        do j=1,3
          do i=1,3
            s(i,j,kind+12)=-s(i,j,kind)
          enddo
        enddo
      enddo
      return
 76   continue
      if (indpg.eq.31) go to 80
c     point group 432 = c4x(x)23
      s(1,3,13)=1
      s(2,1,13)=-1
      s(2,3,13)=1
      s(3,2,13)=-1
      s(3,3,13)=1
      do kind=2,12
        do j=1,3
          do i=1,3
            sum=0
            do k=1,3
              sum=sum+s(i,k,13)*s(k,j,kind)
            enddo
            s(i,j,kind+12)=sum
          enddo
        enddo
      enddo
      if (indpg.eq.30) return
c   point group m3m = i(x)432
      ntot=48
      do kind=1,24
        do j=1,3
          do i=1,3
            s(i,j,kind+24)=-s(i,j,kind)
          enddo
        enddo
      enddo
      return
c     point <4>3m = <c4x>(x)23
   80 continue
      s(1,3,13)=-1
      s(2,1,13)=1
      s(2,3,13)=-1
      s(3,2,13)=1
      s(3,3,13)=-1
      do kind=2,12
        do j=1,3
          do i=1,3
            sum=0
            do k=1,3
              sum=sum+s(i,k,13)*s(k,j,kind)
            enddo
            s(i,j,kind+12)=sum
          enddo
        enddo
      enddo
      return
c     here if indpg exceeds 32 or is less than 28.
  100 continue
      write(6,110) indpg
  110 format(' point-group index',i4,
     * ' is not between 28 and 32 for a cubic group. stopping')
      call stopgm('fccsym',' ')
      end
C     ==================================================================
      subroutine bccsym(indpg,s,ntot)
C     ==--------------------------------------------------------------==
c     point group rotation matrices for cubic (bcc) bravais lattice.
c     it is assumed that a1=(a/2)(1,1,1), a2=(a/2)(-1,1,1),
c     a3=(a/2)(-1,-1,1).  the rotation matrices express how these basis
c     vectors are transformed into one another by point-group rotations.
C     ==--------------------------------------------------------------==
      implicit none
C     Arguments
      integer indpg,s(3,3,48),ntot
C     Variables
      integer sum,i,j,k,kind,jind
C     ==--------------------------------------------------------------==
      do k=1,48
        do j=1,3
          do i=1,3
            s(i,j,k)=0
          enddo
        enddo
      enddo
      do i=1,3
        s(i,i,1)=1
      enddo
      if (indpg.lt.28.or.indpg.gt.32) go to 100
      ntot=12
c     point group 23.
      s(1,2,2)=-1
      s(2,1,2)=-1
      s(3,1,2)=-1
      s(3,2,2)=1
      s(3,3,2)=-1
      s(1,1,3)=-1
      s(1,2,3)=1
      s(1,3,3)=-1
      s(2,3,3)=-1
      s(3,2,3)=-1
      s(1,3,4)=1
      s(2,1,4)=1
      s(2,2,4)=-1
      s(2,3,4)=1
      s(3,1,4)=1
      s(1,1,5)=1
      s(2,1,5)=1
      s(2,2,5)=-1
      s(2,3,5)=1
      s(3,2,5)=-1
      s(1,1,6)=1
      s(2,3,6)=-1
      s(3,1,6)=-1
      s(3,2,6)=1
      s(3,3,6)=-1
      do kind=5,6
        do jind=2,4
          do j=1,3
            do i=1,3
              sum=0
              do k=1,3
                sum=sum+s(i,k,jind)*s(k,j,kind)
              enddo
              s(i,j,kind+2*jind-2)=sum
            enddo
          enddo
        enddo
      enddo
      if (indpg.eq.28) return
      ntot=24
      if (indpg.ne.29) go to 76
c     point group m3 = i(x)23
      do kind=1,12
        do j=1,3
          do i=1,3
            s(i,j,kind+12)=-s(i,j,kind)
          enddo
        enddo
      enddo
      return
 76   continue
      if (indpg.eq.31) go to 80
c     point group 432 = c4x(x)23
      s(1,3,13)=-1
      s(2,1,13)=-1
      s(2,2,13)=1
      s(2,3,13)=-1
      s(3,2,13)=1
      do kind=2,12
        do j=1,3
          do i=1,3
            sum=0
            do k=1,3
              sum=sum+s(i,k,13)*s(k,j,kind)
            enddo
            s(i,j,kind+12)=sum
          enddo
        enddo
      enddo
      if (indpg.eq.30) return
c     point group m3m = i(x)432
      ntot=48
      do kind=1,24
        do j=1,3
          do i=1,3
            s(i,j,kind+24)=-s(i,j,kind)
          enddo
        enddo
      enddo
      return
c     point <4>3m = <c4x>(x)23
 80   continue
      s(1,3,13)=1
      s(2,1,13)=1
      s(2,2,13)=-1
      s(2,3,13)=1
      s(3,2,13)=-1
      do kind=2,12
        do j=1,3
          do i=1,3
            sum=0
            do k=1,3
              sum=sum+s(i,k,13)*s(k,j,kind)
            enddo
            s(i,j,kind+12)=sum
          enddo
        enddo
      enddo
      return
c     here if indpg exceeds 32 or is less than 28.
 100  continue
      write(6,110) indpg
 110  format(' point-group index',i4,
     *     ' is not between 28 and 32 for a cubic group. stopping')
      call stopgm('bccsym',' ')
      end
C     ==================================================================
      subroutine trgsym(indpg,s,ntot)
C     ==--------------------------------------------------------------==
c     sets up the rotation matrices for trigonal(r) groups.  for these
c     groups, the z-axis is chosen as the 3-fold axis, but the
c     crystallographic vectors form a three-fold star around the z-axis,
c     and the primitive cell is a simple rhombohedron.
c     if c is the cosine of the angle between any pair of crystallo-
c     graphic vectors, and if tx=sqrt((1-c)/2), ty=sqrt((1-c)/6),
c     tz=sqrt((1+2c)/3), the crystallographic vectors are
c     a1=a(0,2ty,tz),  a2=a(tx,-ty,tz),  a3=a(-tx,-ty,tz).
C     ==--------------------------------------------------------------==
      implicit none
C     Arguments
      integer indpg,s(3,3,48),ntot
C     Variables
      integer sum,i,j,k,kind
C     ==--------------------------------------------------------------==
      if (indpg.lt.6.or.indpg.gt.10) go to 100
      do k=1,48
        do j=1,3
          do i=1,3
            s(i,j,k)=0
          enddo
        enddo
      enddo
      do i=1,3
        s(i,i,1)=1
      enddo
c     point group 3.
      ntot=3
      s(1,3,2)=1
      s(2,1,2)=1
      s(3,2,2)=1
      s(1,2,3)=1
      s(2,3,3)=1
      s(3,1,3)=1
      if (indpg.eq.6) return
      if (indpg.eq.7.or.indpg.eq.10) go to 15
      ntot=6
c     set up c2x.
      s(1,1,4)=-1
      s(3,2,4)=-1
      s(2,3,4)=-1
      if (indpg.eq.8) go to 12
c     set up mx.
      do j=1,3
        do i=1,3
          s(i,j,4)=-s(i,j,4)
        enddo
      enddo
c     point group 32 =3(x)c2x or 3m = 3(x)mx.
 12   continue
      do kind=2,3
        do i=1,3
          do j=1,3
            sum=0
            do k=1,3
              sum=sum+s(i,k,4)*s(k,j,kind)
            enddo
            s(i,j,kind+3)=sum
          enddo
        enddo
      enddo
      return
 15   continue
      ntot=6
c     point group <3> = 3(x)i
      do kind=1,3
        do j=1,3
          do i=1,3
            s(i,j,kind+3)=-s(i,j,kind)
          enddo
        enddo
      enddo
      if (indpg.eq.7) return
c     point group <3>m = <3>(x)mx
      ntot=12
      s(1,1,7)=1
      s(2,3,7)=1
      s(3,2,7)=1
      do kind=2,6
        do j=1,3
          do i=1,3
            sum=0
            do k=1,3
              sum=sum+s(i,k,7)*s(k,j,kind)
            enddo
            s(i,j,kind+6)=sum
          enddo
        enddo
      enddo
      return
  100 continue
      write(6,110) indpg
  110 format(' point group',i3,' is not trigonal. stopping')
      call stopgm('trgsym',' ')
      end
C     ==================================================================
      subroutine bctsym(indpg,s,ntot)
C     ==--------------------------------------------------------------==
c     point group rotation matrices for tetragonal (i) bravais lattices.
c     it is assumed that a1=(a/2,a/2,c/2), a2=(a/2,-a/2,c/2), and
c     a3=(-a/2,-a/2,c/2).  the rotation matrices express how these
c     basis vectors are transformed into one another by point group
c     rotations.
C     ==--------------------------------------------------------------==
      implicit none
C     Arguments
      integer indpg,s(3,3,48),ntot
C     Variables
      integer sum,i,j,k,kind
C     ==--------------------------------------------------------------==
      do k=1,48
        do j=1,3
          do i=1,3
            s(i,j,k)=0
          enddo
        enddo
      enddo
      do i=1,3
        s(i,i,1)=1
      enddo
      if (indpg.lt.11.or.indpg.gt.17) go to 40
c     point group 4.
      ntot=4
      s(1,2,2)=1
      s(2,3,2)=1
      s(3,1,2)=1
      s(3,2,2)=-1
      s(3,3,2)=1
      s(1,3,3)=1
      s(2,1,3)=1
      s(2,2,3)=-1
      s(2,3,3)=1
      s(3,1,3)=1
      s(1,1,4)=1
      s(1,2,4)=-1
      s(1,3,4)=1
      s(2,1,4)=1
      s(3,2,4)=1
      if (indpg.eq.11) return
      if (indpg.ne.12.and.indpg.ne.16) go to 25
c     point group <4>.
      do kind=2,4,2
        do j=1,3
          do i=1,3
            s(i,j,kind)=-s(i,j,kind)
          enddo
        enddo
      enddo
      if (indpg.eq.12) return
c     point group <4>2m = <4>(x)c2x.
      ntot=8
      s(1,1,5)=-1
      s(1,2,5)=1
      s(1,3,5)=-1
      s(2,3,5)=-1
      s(3,2,5)=-1
      do kind=2,4
        do j=1,3
          do i=1,3
            sum=0
            do k=1,3
              sum=sum+s(i,k,5)*s(k,j,kind)
            enddo
            s(i,j,kind+4)=sum
          enddo
        enddo
      enddo
      return
   25 continue
      ntot=8
      if (indpg.ne.13) go to 26
c     point group 4/m = mz(x)4
      s(1,3,5)=-1
      s(2,1,5)=-1
      s(2,2,5)=1
      s(2,3,5)=-1
      s(3,1,5)=-1
      go to 30
 26   continue
      if (indpg.ne.15) go to 27
c   point group 4mm = mx(x)4.
      s(1,1,5)=1
      s(1,2,5)=-1
      s(1,3,5)=1
      s(2,3,5)=1
      s(3,2,5)=1
      go to 30
c   point groups 422 = c2x(x)4 and 4/mmm = i(x)422.
 27   continue
      s(1,1,5)=-1
      s(1,2,5)=1
      s(1,3,5)=-1
      s(2,3,5)=-1
      s(3,2,5)=-1
 30   continue
      do kind=2,4
        do j=1,3
          do i=1,3
            sum=0
            do k=1,3
              sum=sum+s(i,k,5)*s(k,j,kind)
            enddo
            s(i,j,kind+4)=sum
          enddo
        enddo
      enddo
      if (indpg.ne.17) return
      ntot=16
      do kind=1,8
        do j=1,3
          do i=1,3
            s(i,j,kind+8)=-s(i,j,kind)
          enddo
        enddo
      enddo
      return
   40 continue
      write(6,45) indpg
   45 format(' point-group index',i3,
     *' is not between 11 and 17 for a tetragonal group. stopping')
      call stopgm('bctsym',' ')
      end
C     ==================================================================
      SUBROUTINE ALFASYM(INDPG,S,NTOT)
C     ==--------------------------------------------------------------==
C      SUBROUTINE PER LE SIMMETRIE DI ALFA-GA.
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER INDPG,S(3,3,48),NTOT
C     Variables
      INTEGER I,J,K
C     ==--------------------------------------------------------------==
      IF((INDPG.NE.1).AND.(INDPG.NE.25)) GO TO 100
      DO K=1,8
        DO I=1,3
          DO J=1,3
            S(I,J,K)=0
          ENDDO
        ENDDO
      ENDDO
      DO I=1,3
        S(I,I,1)=1
      ENDDO
      IF(INDPG.EQ.25) GO TO 20
      NTOT=1
      RETURN
 20   CONTINUE
      NTOT=4
C     R2
      S(1,1,2)=-1
      S(2,3,2)= 1
      S(3,2,2)= 1
C     R3
      S(1,1,3)=-1
      S(2,3,3)=-1
      S(3,2,3)=-1
C     R4
      S(1,1,4)=1
      S(2,2,4)=-1
      S(3,3,4)=-1
      RETURN
 100  CONTINUE
      WRITE(6,110)INDPG
 110  FORMAT(' POINT-GROUP INDEX ',I4,
     *     ' DOES NOT CORRESPOND TO ALFA-GA, STOPPING')
      CALL STOPGM('ALFASYM',' ')
C     ==--------------------------------------------------------------==
      END
C     ==================================================================
