C     ==================================================================
      SUBROUTINE SYMVEC(FORCE,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     == Symmetrization of ionic forces                               ==
C     == Do not depend on fractional translation                      ==
C     == Depend on translation vectors (if not primitive cell)        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'symm.inc'
      INCLUDE 'metr.inc'
C     Arguments
      INTEGER   LSCR
      REAL*8    FORCE(3,NAX,NSX),SCR(LSCR)
C     Variables
      REAL*8  EPSILON
      PARAMETER (EPSILON=1.D-15)
      DIMENSION IRTVEC(NAT,*)
      REAL*8    XAU(3,*),YAU(3,*)
      POINTER   (IP_XAU,XAU),(IP_YAU,YAU)
      REAL*8    GG
      CHARACTER TAG*30
      INTEGER   IAT,IS,IA,IR,IV,IB,L,LSYMVEC
C     ==--------------------------------------------------------------==
      IF((INDPG.EQ.0).OR.(NROT.EQ.1.AND.NTVEC.LE.1)) RETURN
      CALL GIVE_SCR_SYMVEC(LSYMVEC,TAG)
      CALL TEST_SCR('SYMVEC',TAG,LSCR,LSYMVEC)
#ifdef _BGL  
      CALL GET_ADDR(IP_XAU,SCR(1))
      CALL GET_ADDR(IP_YAU,SCR(3*NAT+MOD(3*NAT,2)+1))
#else
      CALL GET_ADDR(IP_XAU,SCR(1))
      CALL GET_ADDR(IP_YAU,SCR(3*NAT+1))
#endif
C     Transform to crystal axis basis
      IAT=0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          IAT=IAT+1
          CALL DGEMV('T',3,3,ALAT,HTM1(1,1),3,FORCE(1,IA,IS),1,0.0D0,
     *               XAU(1,IAT),1)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      CALL AZZERO(YAU,3*NAT)
C     Symmetry from rotations
      IF(NTVEC.EQ.0) THEN
        DO IA=1,NAT
          DO IR=1,NROT
            IB=IRT(IR,IA)
            DO L=1,3
              YAU(L,IA)=YAU(L,IA)+XTABLE(L,1,IR)*XAU(1,IB)+
     *                            XTABLE(L,2,IR)*XAU(2,IB)+
     *                            XTABLE(L,3,IR)*XAU(3,IB)
            ENDDO
          ENDDO
        ENDDO
        CALL DSCAL(3*NAT,1.D0/DBLE(NROT),YAU(1,1),1)
      ELSE
        DO IA=1,NAT
          DO IR=1,NROT
            DO IV=1,NTVEC
              IB=IRTVEC(IRT(IR,IA),IV)
              DO L=1,3
                YAU(L,IB)=YAU(L,IB)+XTABLE(L,1,IR)*XAU(1,IA)+
     *               XTABLE(L,2,IR)*XAU(2,IA)+
     *               XTABLE(L,3,IR)*XAU(3,IA)
              ENDDO
            ENDDO
          ENDDO
        ENDDO
        CALL DSCAL(3*NAT,1.D0/DBLE(NROT*NTVEC),YAU(1,1),1)
      ENDIF
C     ==--------------------------------------------------------------==
C     Transform back to cartesian coordinates
      IAT=0
      GG=1.D0/ALAT
      DO IS=1,NSP
        DO IA=1,NA(IS)
          IAT=IAT+1
          CALL DGEMV('T',3,3,GG,HT(1,1),3,YAU(1,IAT),1,0.D0,
     &               FORCE(1,IA,IS),1)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
C     Set to 0 if < EPSILON
      DO IS=1,NSP
        DO IA=1,NA(IS)
          DO L=1,3
            IF(ABS(FORCE(L,IA,IS)).LT.EPSILON) FORCE(L,IA,IS)=0.D0
          ENDDO
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SYMMAT(TNS,SCR,LSCR,ISU)
C     ==--------------------------------------------------------------==
C     == Symmetrization of matrix A(3,3) from each couple of atoms    ==
C     == Do not depend on fractional translations                     ==
C     == Depend on translation vectors (if not primitive cell)        ==
C     == TNS(3,NAT,3,NAT) sets of matrices                            ==
C     == ISU Give factor                                              ==
C     ==     ISU=0 FAC=1                                              ==
C     ==     ISU=1 FAC=ISYMU(IA) (n. of equiv. atoms for the line IA) ==
C     ==     ISU=2 FAC=ISYMU(IB) (n. of equiv. atoms for the row IB)  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'symm.inc'
      INCLUDE 'metr.inc'
C     Arguments
      INTEGER   LSCR,ISU
      REAL*8    TNS(3*NAT,3*NAT), SCR(LSCR)
C     Variables
      DIMENSION IRTVEC(NAT,NTVEC)
      REAL*8    XAU(3*NAT,3*NAT),YAU(3*NAT,3*NAT),XMAT(3,3)
      POINTER   (IP_XAU,XAU),(IP_YAU,YAU)
      CHARACTER TAG*30
      REAL*8    FAC,GG
      INTEGER   IA,IA1,IB,IB1,IR,IAN,IBN,IV,I,J,K,L,
     &          LSYMMAT
C     ==--------------------------------------------------------------==
      IF( (INDPG.EQ.0).OR.(NROT.EQ.1.AND.NTVEC.LE.1) ) RETURN
      CALL GIVE_SCR_SYMMAT(LSYMMAT,TAG)
      CALL TEST_SCR('SYMMAT','MM',LSCR,LSYMMAT)
      CALL GET_ADDR(IP_XAU,SCR(1))
#ifdef _BGL 
      CALL GET_ADDR(IP_YAU,SCR(3*NAT*3*NAT+MOD(3*NAT*3*NAT,2)+1))
#else
      CALL GET_ADDR(IP_YAU,SCR(3*NAT*3*NAT+1))
#endif
C     Transform to crystal axis basis
      DO IA=1,NAT
        IA1=3*(IA-1)+1
        DO IB=1,NAT
          IB1=3*(IB-1)+1
          CALL DGEMM('T','N',3,3,3,ALAT,HTM1(1,1),3,TNS(IA1,IB1),3*NAT,
     *                0.D0,XMAT(1,1),3)
          CALL DGEMM('N','N',3,3,3,ALAT,XMAT(1,1),3,HTM1(1,1),3,0.D0,
     &                XAU(IA1,IB1),3*NAT)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      CALL AZZERO(YAU,3*NAT*3*NAT)
C     Symmetry from rotations
      DO IA=1,NAT
        IA1=3*(IA-1)
        DO IB=1,NAT
          IB1=3*(IB-1)
          IF(ISU.EQ.0) THEN
            FAC=1.D0
          ELSEIF(ISU.EQ.1) THEN
            FAC=DBLE(ISYMU(IA))
          ELSEIF(ISU.EQ.2) THEN
            FAC=DBLE(ISYMU(IB))
          ELSE
            CALL STOPGM('SYMMAT',' ISU ')
          ENDIF
          IF(FAC.NE.0.D0) THEN
            DO IR=1,NROT
              DO IV=1,NTVEC
                IAN=3*( IRTVEC(IRT(IR,IA),IV) - 1 )
                IBN=3*( IRTVEC(IRT(IR,IB),IV) - 1 )
                CALL AZZERO(XMAT,9)
C               Matrix multiplication FAC XTABLE.XTABLE.XAU
                DO I=1,3
                  DO J=1,3
                    DO K=1,3
                      DO L=1,3
                        XMAT(I,J)=XMAT(I,J)+
     &                       XTABLE(I,K,IR)*XTABLE(J,L,IR)*
     *                       XAU(IA1+K,IB1+L)*FAC
                      ENDDO     !L
                    ENDDO       !K
                  ENDDO         !J
                ENDDO           !I
C               Put in YAU
                DO I=1,3
                  DO J=1,3
                    YAU(IAN+I,IBN+J)=YAU(IAN+I,IBN+J)+XMAT(I,J)
                  ENDDO         !J
                ENDDO           !I
              ENDDO             !IV
            ENDDO               !IR
          ENDIF                 !IF(FAC.NE.0.D0)
        ENDDO                   !IB
      ENDDO                  !IA
      CALL DSCAL(3*NAT*3*NAT,1.D0/DBLE(NROT*NTVEC),YAU(1,1),1)
C     ==--------------------------------------------------------------==
C     Transform back to cartesian coordinates
      GG=1.D0/ALAT
      DO IA=1,NAT
        IA1=3*(IA-1)+1
        DO IB=1,NAT
          IB1=3*(IB-1)+1
          CALL DGEMM('T','N',3,3,3,GG,HT(1,1),3,
     &                YAU(IA1,IB1),3*NAT,0.D0,XMAT(1,1),3)
          CALL DGEMM('N','N',3,3,3,GG,XMAT(1,1),3,
     &                HT(1,1),3,0.D0,TNS(IA1,IB1),3*NAT)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SYMSTRESS(STRESS)
C     ==--------------------------------------------------------------==
C     == Symmetrization of stress tensor                              ==
C     == Do not depend on fractional translations                     ==
C     == Do not depend on translation vectors                         ==
C     ==--------------------------------------------------------------==
C     == STRESS(1:6) is given with Voigt notation                     ==
C     == ALPHA(1:6)  first index for Voigt notation                   ==
C     == BETA(1:6)   second index                                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'symm.inc'
      INCLUDE 'metr.inc'
C     Arguments
      REAL*8  STRESS(3,3)
C     Variables
      REAL*8  EPSILON
      PARAMETER (EPSILON=1.D-15)
      REAL*8  SUM,ASTR(3,3),BSTR(3,3)
      INTEGER IR,I,J
C     ==--------------------------------------------------------------==
      IF((INDPG.EQ.0).OR.(NROT.EQ.1)) RETURN
C     Transform to crystal axis basis
      CALL DGEMM('T','N',3,3,3,1.D0,HTM1,3,STRESS,3,0.D0,ASTR,3)
      CALL DGEMM('N','T',3,3,3,1.D0,ASTR,3,HT,3,0.D0,STRESS,3)
      CALL AZZERO(BSTR,3*3)
      DO IR=1,NROT
        CALL DGEMM('N','N',3,3,3,1.D0,XTABLE(1,1,IR),3,STRESS,3,
     &       0.D0,ASTR,3)
        CALL DGEMM('N','N',3,3,3,1.D0,ASTR,3,XTABLE(1,1,INVE(IR)),3,
     &       1.D0,BSTR,3)
      ENDDO
      CALL DSCAL(3*3,1.D0/DBLE(NROT),BSTR,1)
C     Transform back to cartesian coordinates      
      CALL DGEMM('T','N',3,3,3,1.D0,HT,3,BSTR,3,0.D0,ASTR,3)
      CALL DGEMM('N','T',3,3,3,1.D0,ASTR,3,HTM1,3,0.D0,STRESS,3)
C     Symmetric matrix and set to 0 if < EPSILON
      DO I=1,3
        DO J=I,3
          IF(ABS(STRESS(J,I)).LT.EPSILON) THEN
            STRESS(J,I)=0.D0
            STRESS(I,J)=0.D0
          ELSE
            SUM=0.5D0*(STRESS(J,I)+STRESS(I,J))
            STRESS(J,I)=SUM
            STRESS(I,J)=SUM
          ENDIF
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SYMRHO(RHO,PSI,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     == SYMMETRIZE THE CHARGE DENSITY IN FOURIER SPACE               ==
C     == ONLY THE NHG COMPONENTS GIVEN BY NZH AND INDZ                ==
C     ==--------------------------------------------------------------==
C     == XTABLE is in crystal basis and FTABLE is in reciprocal basis ==
C     == For each operation IR, R^-1 gives the symmetry.              ==
C     == As R^-1 is in the group, we use XTABLE(IR) and FTAU(INVE(IR))==
C     ==--------------------------------------------------------------==
C     == More accurate than symmetrization in real space              ==
C     == Faster (20-60 times faster)                                  ==
C     ==--------------------------------------------------------------==
C     == IF DUAL IS LESS THAN 3, CALL SYMRHO_ALL WHICH SYMMETRIZES    ==
C     == ALL THE DENSITY ARRAY (MORE EXPENSIVE)                       ==
C     == ONLY IN SERIAL VERSION                                       ==
C     ==--------------------------------------------------------------==
C     ==  BTEST(I,POS) .TRUE. if bit POS is 1 in I integer            ==
C     ==  Chech if BTEST works with INTEGER*8                         ==
C     ==  otherwise use -D__NOINT8                                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'symm.inc'
      INCLUDE 'symm4.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'cnst.inc'        !PI
      INCLUDE 'fft.inc'
C     Arguments
      INTEGER    LSCR
      REAL*8     RHO(NNR1),SCR(LSCR)
      COMPLEX*16 PSI(MAXFFT)
C     Variables
      COMPLEX*16 PSIP(LSCR/2),ARRAY(NROT,(LSCR/(3*NROT)))
      POINTER    (IP_PSIP,PSIP),(IP_ARRAY,ARRAY)
      INTEGER    IROTP(NROT,(LSCR/(3*NROT)))
      POINTER    (IP_IROTP,IROTP)
      COMPLEX*16 CRHOP,CRHOM
#if defined(__NOINT8)
      LOGICAL    BTEST4
      INTEGER    NINSHEL(2,NHG),ISISHEL(2,NHG)
      INTEGER    IMASK(2),ISIGN(2),IR
#else
      INTEGER*8  NINSHEL(NHG),ISISHEL(NHG)
      INTEGER*8  IMASK,ISIGN,IR
#endif
      INTEGER    ISUB,IG,ISH,INDIG,IR2,NH1,NH2,NH3,
     &           IRFTAU,NPACK,IPACK,JNPACK,NZH0,INDZ0,
     &           ISHL,ISHIG,ISHEND,LSYMRHO
      REAL*8     VNROT,ARG,TMPI,TPI,RGL1,RGL2,RGL3
      LOGICAL    TCONJIG
      CHARACTER  TAG*30
C     ==--------------------------------------------------------------==
      IF ((INDPG.EQ.0.OR.INDPG.EQ.100) .OR.
     &    (NROT.EQ.1.AND.NTVEC.LE.1)) RETURN
C     ==--------------------------------------------------------------==
#ifndef PARALLEL
C     Call SYMRHO_ALL IF NEEDED (SMALL DUAL VALUE)
C      IF(  (CDUAL.LT.3.D0).OR.
C     &     (.NOT.TSYMMORPHIC.AND.INVERSION.NE.0)) THEN
C        CALL SYMRHO_ALL(RHO,PSI,SCR,LSCR)
C        RETURN
C      ENDIF
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('    SYMRHO',ISUB)
C     We need to initialize some arrays.
      IF(NUMSHEL.EQ.0) THEN
        CALL BUILDSHELL(SCR,LSCR)
      ENDIF
      CALL GIVE_SCR_SYMRHO(LSYMRHO,TAG)
      CALL TEST_SCR('SYMRHO',TAG,LSCR,LSYMRHO)
C     ==--------------------------------------------------------------==
      NH1=NR1S/2+1
      NH2=NR2S/2+1
      NH3=NR3S/2+1
      TPI=2.D0*PI
      TMPI=-2.D0*PI
      IF(TSYMMORPHIC) THEN
        IF(INVERSION.EQ.0) THEN
          CALL GET_ADDR(IP_PSIP,SCR(1))
        ENDIF
      ELSE
        IPACK=LSCR/(3*NROT)
#ifdef PARALLEL
        ARG=DBLE(IPACK)
        CALL GLOMIN(1,ARG)
        IPACK=INT(ARG)
#endif
        CALL GET_ADDR(IP_IROTP,SCR(1))
#ifdef _BGL 
        CALL GET_ADDR(IP_ARRAY,SCR(1+(NROT*IPACK)+MOD(NROT*IPACK,2)))
#else
        CALL GET_ADDR(IP_ARRAY,SCR(1+NROT*IPACK))
#endif
      ENDIF
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(PSI,MAXFFT)
      CALL DCOPY(NNR1,RHO(1),1,PSI(1),2)
C     Transform to reciprocal space
      CALL FWFFT(PSI)
C     ==--------------------------------------------------------------==
C     Symmetry from translation vectors -> extinction
      IF(IGEXTINC.NE.0) THEN
C       There is IGEXTINC extinction (RHO(IG)=0)
!CDIR NODEP
        DO IG=1,NHG
          IF(INDSHEL(IG).EQ.0) THEN
            PSI(NZH (IG))=DCMPLX(0.D0,0.D0)
            PSI(INDZ(IG))=DCMPLX(0.D0,0.D0)
          ENDIF
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     NROT = 1 (no rotation symmetry except the identical one).
      IF((NROT.GT.1).AND.TSYMMORPHIC) THEN
C       Translate density in order to have 0 as  centre of symmetry
        IF(.NOT.TORIGIN) THEN
!CDIR NODEP
          DO IG=1,NHG
            IF(INDSHEL(IG).NE.0) THEN
              ARG=TPI*(
     &             DBLE(INYH(1,IG)-NH1)*ORIGIN(1)+
     &             DBLE(INYH(2,IG)-NH2)*ORIGIN(2)+
     &             DBLE(INYH(3,IG)-NH3)*ORIGIN(3) )
              CRHOP=PSI(NZH(IG))*DCMPLX(COS(ARG),SIN(ARG))
              PSI(NZH (IG))=CRHOP
              PSI(INDZ(IG))=DCONJG(CRHOP)
            ENDIF
          ENDDO
        ENDIF
        IF(INVERSION.EQ.0) THEN
          VNROT=1.D0/DBLE(NROT)
          CALL ZAZZERO(PSIP,NUMSHEL)
C         Loop over G-vectors. For each equivalent shell,
C         the electronic density is the same.
          DO IG=1,NHG
            INDIG=INDSHEL(IG)
            IF(INDIG.NE.0) THEN
              PSIP(INDIG)=PSIP(INDIG)
     &             + ISYSHEL(1,IG)*PSI(NZH (IG))
     &             + ISYSHEL(2,IG)*PSI(INDZ(IG))
            ENDIF
          ENDDO
#ifdef PARALLEL
          CALL GLOSUM(2*NUMSHEL,PSIP)
#endif
!CDIR NODEP
          DO IG=1,NHG
            INDIG=INDSHEL(IG)
            IF(INDIG.NE.0) THEN
              IF(ISYSHEL(1,IG).GT.0) THEN
                IF(ISYSHEL(2,IG).EQ.0) THEN
                  CRHOP=VNROT*PSIP(INDIG)
                  PSI(NZH (IG))=CRHOP
                  PSI(INDZ(IG))=DCONJG(CRHOP)
                ELSE
                  ARG=VNROT*DREAL(PSIP(INDIG))
                  PSI(NZH (IG))=DCMPLX(ARG,0.D0)
                  PSI(INDZ(IG))=DCMPLX(ARG,0.D0)
                ENDIF
              ELSE
                CRHOP=VNROT*PSIP(INDIG)
                PSI(NZH (IG))=DCONJG(CRHOP)
                PSI(INDZ(IG))=CRHOP
              ENDIF
            ENDIF
          ENDDO
        ELSE
C         Special case for inversion
          VNROT=1.D0/DBLE(NROT/2)
          CALL AZZERO(SCR,NUMSHEL)
C         Loop over G-vectors. For each equivalent shell,
C         the electronic density is the same.
          DO IG=1,NHG
            INDIG=INDSHEL(IG)
            IF(INDIG.NE.0) THEN
              SCR(INDIG)=SCR(INDIG)+ISYSHEL(1,IG)*DREAL(PSI(NZH(IG)))
            ENDIF
          ENDDO
#ifdef PARALLEL
          CALL GLOSUM(NUMSHEL,SCR)
#endif
!CDIR NODEP
          DO IG=1,NHG
            INDIG=INDSHEL(IG)
            IF(INDIG.NE.0) THEN
              ARG=VNROT*SCR(INDIG)
              PSI(NZH (IG))=DCMPLX(ARG,0.D0)
              PSI(INDZ(IG))=DCMPLX(ARG,0.D0)
            ENDIF
          ENDDO
C         All PSI array is real
          CALL IMAGSET0(PSI,MAXFFT)
        ENDIF
C       Translate density to -ORIGIN
        IF(.NOT.TORIGIN) THEN
!CDIR NODEP
          DO IG=1,NHG
            IF(INDSHEL(IG).NE.0) THEN
              ARG=TMPI*(
     &             DBLE(INYH(1,IG)-NH1)*ORIGIN(1)+
     &             DBLE(INYH(2,IG)-NH2)*ORIGIN(2)+
     &             DBLE(INYH(3,IG)-NH3)*ORIGIN(3) )
              CRHOP=PSI(NZH(IG))*DCMPLX(COS(ARG),SIN(ARG))
              PSI(NZH (IG))=CRHOP
              PSI(INDZ(IG))=DCONJG(CRHOP)
            ENDIF
          ENDDO
        ENDIF
      ELSEIF(NROT.GT.1) THEN
C       ==------------------------------------------------------------==
C       == Non-symmorphic case.                                       ==
C       == Maybe you could have some trouble with the DUAL option     ==
C       == the only solution is to increase the DUAL parameter
C       ==------------------------------------------------------------==
        VNROT=1.D0/DBLE(NROT)
C       Loop over equivalent shell.
C       For each rotation, we collect the corresponding components.
C       Then we can calculate for each G-vector.
        DO ISHL=1,NUMSHEL,IPACK
          NPACK=MIN((NUMSHEL-ISHL+1),IPACK)
          CALL IAZZERO(IROTP,NROT*NPACK)
          CALL AZZERO(ARRAY,2*NROT*NPACK)
          ISHEND=ISHL+(NPACK-1)
          DO IG=1,NHG
            ISHIG=INDSHEL(IG)
            IF(  (ISHIG.GE.ISHL).AND.(ISHIG.LE.ISHEND) ) THEN
              CRHOP=PSI(NZH (IG))
              CRHOM=PSI(INDZ(IG))
#if defined(__NOINT8)
              IMASK(1)=NINSHEL(1,IG)
              IMASK(2)=NINSHEL(2,IG)
              ISIGN(1)=ISISHEL(1,IG)
              ISIGN(2)=ISISHEL(2,IG)
              JNPACK=(ISHIG-ISHL)+1
              IF(JNPACK.LT.1.OR.JNPACK.GT.NPACK) WRITE(6,*) JNPACK
              DO IR=1,NROT
                IF(BTEST4(IMASK(1),IR)) THEN
                  IF(BTEST4(ISIGN(1),IR)) THEN
                    ARRAY(IR,JNPACK)=CRHOP
                  ELSE
                    ARRAY(IR,JNPACK)=CRHOM
                  ENDIF
                  IROTP(IR,JNPACK)=IG
                ENDIF
              ENDDO
#else
              IMASK=NINSHEL(IG)
              ISIGN=ISISHEL(IG)
              JNPACK=(ISHIG-ISHL)+1
              IF(JNPACK.LT.1.OR.JNPACK.GT.NPACK) WRITE(6,*) JNPACK
              DO IR=1,NROT
                IF(BTEST(IMASK,IR)) THEN
                  IF(BTEST(ISIGN,IR)) THEN
                    ARRAY(IR,JNPACK)=CRHOP
                  ELSE
                    ARRAY(IR,JNPACK)=CRHOM
                  ENDIF
                  IROTP(IR,JNPACK)=IG
                ENDIF
              ENDDO
#endif
            ENDIF
          ENDDO
 100      CONTINUE
#ifdef    PARALLEL
C         Glosum for the NROT components for NPACK equivalent shells.
          CALL GLOSUM(2*NROT*NPACK,ARRAY)
#endif
          DO ISH=1,NPACK
            DO IR=1,NROT
              IF(IROTP(IR,ISH).NE.0) THEN
                IG=IROTP(IR,ISH)
                CRHOP=ARRAY(IR,ISH)
#if defined(__NOINT8)
                TCONJIG=BTEST4(ISISHEL(1,IG),IR)
#else
                TCONJIG=BTEST(ISISHEL(IG),IR)
#endif
                IF(TCONJIG) THEN
                  RGL1=INYH(1,IG)-NH1
                  RGL2=INYH(2,IG)-NH2
                  RGL3=INYH(3,IG)-NH3
                ELSE
                  RGL1=NH1-INYH(1,IG)
                  RGL2=NH2-INYH(2,IG)
                  RGL3=NH3-INYH(3,IG)
                ENDIF
C               Give the number of the inverse of the rotation
C               which gives RGL1,RGL2,RGL3 from IG
                DO IR2=1,NROT
                  IF(IR2.NE.IR) THEN
                    IF(IR.EQ.1) THEN
                      IRFTAU=INVE(IR2)
                    ELSE
                      IRFTAU=MULTAB(INVE(IR2),IR)
                    ENDIF
                    IF(TSYMMOR(IRFTAU)) THEN
                      CRHOP=CRHOP+ARRAY(IR2,ISH)
                    ELSE
                      ARG = TMPI *
     &                     ( RGL1*FTAU(1,IRFTAU)
     &                     + RGL2*FTAU(2,IRFTAU)
     &                     + RGL3*FTAU(3,IRFTAU) )
                      CRHOP=CRHOP+ARRAY(IR2,ISH)
     &                     *DCMPLX(COS(ARG),SIN(ARG))
                    ENDIF
                    IF(IG.EQ.IROTP(IR2,ISH)) IROTP(IR2,ISH)=0
                  ENDIF
                ENDDO
                NZH0 =NZH (IG)
                INDZ0=INDZ(IG)
                CRHOP=VNROT*CRHOP
                IF(TCONJIG) THEN
                  PSI(NZH0 )=CRHOP
                  PSI(INDZ0)=DCONJG(CRHOP)
                ELSE         
                  PSI(INDZ0)=CRHOP
                  PSI(NZH0 )=DCONJG(CRHOP)
                ENDIF
              ENDIF
            ENDDO
          ENDDO
        ENDDO                   !DO ISHL=1,NUMSHEL,IPACK
C       Special case for inversion. We do work in ALL ARRAY.
        IF(INVERSION.NE.0) THEN
          IF(TSYMMOR(INVERSION)) CALL IMAGSET0(PSI,MAXFFT)
        ENDIF
      ENDIF                     !IF(TSYMMORPHIC)
C     ==--------------------------------------------------------------==
C     Transform back to real space
      CALL INVFFT(PSI)
      CALL DCOPY(NNR1,PSI(1),2,RHO(1),1)
      CALL TIHALT('    SYMRHO',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE BUILDSHELL(SCR,LSCR)
C     ==--------------------------------------------------------------==
C     == Use bit manipulations functions                              ==
C     ==  IBSET(I,POS) sets the bit POS to 1 in I integer             ==
C     ==  BTEST(I,POS) .TRUE. if bit POS is 1 in I integer            ==
C     ==  Chech if BTEST and IBTEST work with INTEGER*8               ==
C     ==  otherwise use -D__NOINT8                                    ==
C     ==--------------------------------------------------------------==
C     == Symetrisation due to translation vectors (some extinction)   ==
C     == RHO(R+TVEC) =RHO(R)                                          ==
C     == i.e. RHO(G) = RHO(G) Exp(i G.TVEC)                           ==
C     == if Exp(i G.TVEC) = 1 INDSHEL gives equivalent shell          ==
C     == if Exp(i G.TVEC)\= 1 -> RHO(G) = 0                           ==
C     == By symMetry true for all G-vectors of the equivalent shell   ==
C     ==                                                              ==
C     == There are also other extinction associated with rotations    ==
C     == and glides.                                                  ==
C     == If there is some extinction IGEXTINC is set to their number  ==
C     == and INDSHEL(IG)=0                                            ==
C     ==--------------------------------------------------------------==
C     == The symetry are determined with a precision of DELTASYM      ==
C     == in the direct lattice vector basis                           ==
C     == Furthermore the ordering has a precision of EPSG*NHGS        ==
C     == So for translation COS(X) = 1.D0 with precision of DELTASYM  ==
C     == and for the equivalent shell determination                   ==
C     == DELTAG=MAX(DELTASYM*SQRT(HG(IG)),EPSG*NHGS)                  ==
C     ==--------------------------------------------------------------==
C     == IFIRST=0 first call to BUILDSHELL                            ==
C     == IFIRST=1 not the first call and before TSYMMORPHIC=.TRUE.    ==
C     == IFIRST=2 not the first call and before TSYMMORPHIC=.FALSE.   ==
C     ==--------------------------------------------------------------==
C     == TO DO:                                                       ==
C     ==   Parallel version. Instead of broadcasting one G vector per ==
C     ==   one, broadcast many G vectors to all processors.           ==
C     ==   In this case BUILDSHELL is more parallelizable.            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     ==--------------------------------------------------------------==
      INCLUDE 'system.h'        !symm.inc
      INCLUDE 'ions.inc'        !NAT (symm.inc)
      INCLUDE 'cnst.inc'        !PI
      INCLUDE 'symm.inc'
      INCLUDE 'symm4.inc'
      INCLUDE 'irat.inc'        !IRAT
      INCLUDE 'cppt.inc'        !INYH
      INCLUDE 'gvec.inc'        !EPSG
      INCLUDE 'geq0.inc'        !GEQ0
C     Arguments
      INTEGER   LSCR
      REAL*8    SCR(LSCR)
C     Variables
      INTEGER   INDNOSHEL
      PARAMETER (INDNOSHEL=2**(8*4-2))
      REAL*8    R(4),DELTAG,ARG,TPI
      CHARACTER TAG*30
C     NINSHEL is INTEGER*8
#if defined(__NOINT8)
      LOGICAL   BTEST4
      INTEGER   NINSHEL(2,NHG),ISISHEL(2,NHG)
      INTEGER   ISUM(2),IR
#else
      INTEGER*8 NINSHEL(NHG),ISISHEL(NHG)
      INTEGER*8 ISUM,IR
#endif
      INTEGER   NH1,NH2,NH3,MSGLEN,IP,IGG,IG,IFAC,IGBEG,IGEND,INVIR,
     &          IGL1,IGL2,IGL3,IG1,IG2,IG3,ISUB,ISH,NSHL,KSHELL,
     &          LBUILDSHELL,
     &          IFIRST
      DATA      IFIRST /0/
      SAVE      IFIRST
C     ==--------------------------------------------------------------==
      IF( (INDPG.EQ.0).OR.(NROT.EQ.1.AND.NTVEC.LE.1) ) RETURN
C     63 because we use integer*8
      IF(NROT.GT.63) CALL STOPGM('BUILDSHELL',
     &     'THE NUMBER OF ROTATIONS IS TOO BIG')
C     ==--------------------------------------------------------------==
      TPI=2.D0*PI
      CALL TISET('BUILDSHELL',ISUB)
      IF(IFIRST.EQ.0) THEN
C       First call.
        CALL MEMORY(IP_INDSHEL,NHG/IRAT+1,'INDSHEL')
        IF(TSYMMORPHIC) THEN
          IF(NROT.GT.1) THEN
            CALL MEMORY(IP_ISYSHEL,(NHG*2)/IRAT+1,'ISYSHEL')
          ENDIF
          IFIRST=1
        ELSE
          IF(NROT.GT.1) THEN
C           NINSHEL and ISISHEL are INTEGER*8
            CALL MEMORY(IP_NINSHEL,(NHG*2)/IRAT+1,'NINSHEL')
            CALL MEMORY(IP_ISISHEL,(NHG*2)/IRAT+1,'ISISHEL')
          ENDIF
          IFIRST=2
        ENDIF
      ELSEIF(IFIRST.EQ.1.AND.(.NOT.TSYMMORPHIC)) THEN
C       Before TSYMMORPHIC, now .NOT.TSYMMORPHIC
        IF(NROT.GT.1) THEN
C         Deallocation of ISYSHEL
          CALL FREEM(IP_ISYSHEL)
C         Allocations of NINSHEL and ISISHEL
          CALL MEMORY(IP_NINSHEL,(NHG*2)/IRAT+1,'NINSHEL')
          CALL MEMORY(IP_ISISHEL,(NHG*2)/IRAT+1,'ISISHEL')
        ENDIF
        IFIRST=2
      ELSEIF(IFIRST.EQ.2.AND.TSYMMORPHIC) THEN
C       Before .NOT.TSYMMORPHIC, now TSYMMORPHIC
        IF(NROT.GT.1) THEN
C         Deallocations of NINSHEL and ISISHEL
          CALL FREEM(IP_NINSHEL)
          CALL FREEM(IP_ISISHEL)
C         Allocation of ISYSHEL
          CALL MEMORY(IP_ISYSHEL,(NHG*2)/IRAT+1,'ISYSHEL')
        ENDIF
        IFIRST=1
      ENDIF
C     ==--------------------------------------------------------------==
      NH1=NR1S/2+1
      NH2=NR2S/2+1
      NH3=NR3S/2+1
      IGEXTINC=0
C     Special case for extinction only (no rotations)
      IF(NROT.EQ.1) THEN
C       NTVEC/=0 -> only extinction
        DO IG=1,NHG
          R(1)=DBLE(INYH(1,IG)-NH1)
          R(2)=DBLE(INYH(2,IG)-NH2)
          R(3)=DBLE(INYH(3,IG)-NH3)
C         Check if there is extinction or not
          DO IR=1,NTVEC
            ARG=COS( TPI*(R(1)*TVEC(1,IR)+
     &           R(2)*TVEC(2,IR)+R(3)*TVEC(3,IR)))
            IF(ABS(ARG-1.D0).GT.DELTASYM) THEN
C             Extinction
              IGEXTINC=IGEXTINC+1
              INDSHEL(IG)=0
              GOTO 50
            ENDIF
          ENDDO
C         No extinction
          INDSHEL(IG)=1
 50       CONTINUE
        ENDDO
C       Work is done
        NUMSHEL=1
        RETURN
      ENDIF
C     ==--------------------------------------------------------------==
      DO IG=1,NHG
        INDSHEL(IG)=INDNOSHEL
      ENDDO
      IF(TSYMMORPHIC) THEN
        CALL IAZZERO(ISYSHEL,2*NHG)
      ELSE
#if defined(__NOINT8)
        CALL IAZZERO(NINSHEL,NHG*2)
        CALL IAZZERO(ISISHEL,NHG*2)
#else
        CALL I8AZZERO(NINSHEL,NHG)
        CALL I8AZZERO(ISISHEL,NHG)
#endif
      ENDIF
      NUMSHEL=0
      MSGLEN= 4 * 8
C     Each processor sends
      DO IP=1,NPROC
        IF(PGROUP(IP).EQ.ME) THEN
          DO IG=1,NHG
            IF(INDSHEL(IG).EQ.INDNOSHEL) THEN
C             Another equivalent shell
              IG1=INYH(1,IG)-NH1
              IG2=INYH(2,IG)-NH2
              IG3=INYH(3,IG)-NH3
              R(1)=DBLE(IG1)
              R(2)=DBLE(IG2)
              R(3)=DBLE(IG3)
              R(4)=HG(IG)
              IF(NTVEC.NE.0) THEN
C               Check if there is extinction or not
                DO IR=1,NTVEC
                  ARG=COS( TPI*(R(1)*TVEC(1,IR)+
     &                 R(2)*TVEC(2,IR)+R(3)*TVEC(3,IR)))
                  IF(ABS(ARG-1.D0).GT.DELTASYM) THEN
C                   Extinction
                    IGEXTINC=IGEXTINC+1
                    INDSHEL(IG)=0
                    GOTO 200
                  ENDIF
                ENDDO
              ENDIF
              IF(.NOT.TSYMMORPHIC) THEN
C               For not symmorphic group, there are
C               rotation associated with glide:
C               Some extinction could be exist
              ENDIF
#ifdef PARALLEL
              CALL MY_BCAST(R,MSGLEN,PGROUP(IP),ALLGRP)
#endif
              NUMSHEL=NUMSHEL+1
              INDSHEL(IG)=NUMSHEL
              IF(TSYMMORPHIC) THEN
                ISYSHEL(1,IG)=ISYSHEL(1,IG)+1
              ELSE
                IR=1
#if defined(__NOINT8)
                CALL IBSET4(NINSHEL(1,IG),IR)
                CALL IBSET4(ISISHEL(1,IG),IR)
#else
                NINSHEL(IG)=IBSET(NINSHEL(IG),IR)
                ISISHEL(IG)=IBSET(ISISHEL(IG),IR)
#endif
              ENDIF
              IGBEG=IG
              NSHL=IGL(IG)
C             direct DELTASYM*ALAT
C             reciprocal 2PI/ALAT*HI(IG)*DELTASYM
              DELTAG=MAX(R(4)*DELTASYM,EPSG*DBLE(NHGS))
              DO ISH=NSHL,NHGL
                IF(ABS(GL(ISH)-R(4)).LE.DELTAG) THEN
                  IGEND=ISPTR(ISH+1)-1
                ELSE
                  IGEND=ISPTR(ISH+1)-1
                  GOTO 500
                ENDIF
              ENDDO
 500          CONTINUE
C             This processor finds equivalent G-vectors.
              DO IR=2,NROT
C               Rotations conserves norms
                IGL1 = NINT(
     &                 FTABLE(1,1,IR) * R(1)
     &               + FTABLE(1,2,IR) * R(2)
     &               + FTABLE(1,3,IR) * R(3) )
                IGL2 = NINT(
     &                 FTABLE(2,1,IR) * R(1)
     &               + FTABLE(2,2,IR) * R(2)
     &               + FTABLE(2,3,IR) * R(3) )    
                IGL3 = NINT(
     &                 FTABLE(3,1,IR) * R(1)
     &               + FTABLE(3,2,IR) * R(2)
     &               + FTABLE(3,3,IR) * R(3) )
                IF(IGL1.LT.0.OR.
     &               (IGL1.EQ.0.AND.IGL2.LT.0).OR.
     &               (IGL1.EQ.0.AND.IGL2.EQ.0.AND.IGL3.LT.0)) THEN
                  IFAC=2
                  IGL1=-IGL1+NH1
                  IGL2=-IGL2+NH2
                  IGL3=-IGL3+NH3
                ELSE
                  IFAC=1
                  IGL1=IGL1+NH1
                  IGL2=IGL2+NH2
                  IGL3=IGL3+NH3
                ENDIF
                IF(IGL1.EQ.IG1.AND.IGL2.EQ.IG2.AND.IGL3.EQ.IG3) THEN
C                 FTABLE.G=G vector
                  IF(TSYMMORPHIC) THEN
                    ISYSHEL(IFAC,IG)=ISYSHEL(IFAC,IG)+1
                  ELSE
#if defined(__NOINT8)
                    CALL IBSET4(NINSHEL(1,IG),IR)
                    IF(IFAC.EQ.1) THEN
                      CALL IBSET4(ISISHEL(1,IG),IR)
                    ENDIF
#else
                    NINSHEL(IG)=IBSET(NINSHEL(IG),IR)
                    IF(IFAC.EQ.1) THEN
                      ISISHEL(IG)=IBSET(ISISHEL(IG),IR)
                    ENDIF
#endif
                  ENDIF
                ELSE
C                 Looking for the image of IG.
                  DO IGG=IGBEG,IGEND
                    IF(  IGL1.EQ.INYH(1,IGG).AND.
     &                   IGL2.EQ.INYH(2,IGG).AND.
     &                   IGL3.EQ.INYH(3,IGG)) THEN
C                     Find an equivalent G-vector, continue
                      INDSHEL(IGG)=NUMSHEL
                      IF(TSYMMORPHIC) THEN
                        ISYSHEL(IFAC,IGG)=ISYSHEL(IFAC,IGG)+1
                      ELSE
#if defined(__NOINT8)   
                        CALL IBSET4(NINSHEL(1,IGG),IR)
                        IF(IFAC.EQ.1) THEN
                          CALL IBSET4(ISISHEL(1,IGG),IR)
                        ENDIF
#else                   
                        NINSHEL(IGG)=IBSET(NINSHEL(IGG),IR)
                        IF(IFAC.EQ.1)
     &                       ISISHEL(IGG)=IBSET(ISISHEL(IGG),IR)
#endif                  
                      ENDIF
                      GOTO 1000
                    ENDIF
                  ENDDO
                  IF(NPROC.EQ.1) THEN
                    WRITE(6,*) 'IG=',IG,' IGL=',IGL1,IGL2,IGL3
                    CALL STOPGM('BUILDSHELL','G-VECTOR NOT FOUND')
                  ENDIF
C                 Another rotation.
                ENDIF
 1000           CONTINUE
              ENDDO
C             No more equivalent G-vectors, now the next.
            ENDIF
 200        CONTINUE
          ENDDO
C         It is over for this processor, information for the other ones
          R(1)=-1.D0
          CALL MY_BCAST(R,MSGLEN,PGROUP(IP),ALLGRP)
        ELSE
C         The other processors listen to IP processor
          R(1)=1.D0
          NSHL=1
          DO WHILE(R(1).GE.0.D0)
            CALL MY_BCAST(R,MSGLEN,PGROUP(IP),ALLGRP)
            IF(R(1).LT.0.D0) GOTO 3000
C           Another equivalent shell
            NUMSHEL=NUMSHEL+1
C           Find first IG possible.
            KSHELL=0
C           Due to reordering we cannot trust the shells (see rggen).
C           direct DELTASYM*ALAT
C           reciprocal 2PI/ALAT*HG(IG)*DELTASYM
            DELTAG=MAX(R(4)*DELTASYM,EPSG*DBLE(NHGS))
            DO ISH=1,NHGL
              IF(ABS(GL(ISH)-R(4)).LE.DELTAG) THEN
                IF(KSHELL.EQ.0) THEN
                  NSHL=ISH
                  IGBEG=ISPTR(ISH)
                  IGEND=ISPTR(ISH+1)-1
                  KSHELL=1
                ELSE
                  IGEND=ISPTR(ISH+1)-1
                ENDIF
              ELSEIF(KSHELL.EQ.1) THEN
                IGEND=ISPTR(ISH+1)-1
              ELSEIF(GL(ISH).GT.(R(4)+DELTAG)) THEN
                GOTO 1500
              ENDIF
            ENDDO
 1500       CONTINUE
C           No equivalent shell for this processor.
            IF(KSHELL.EQ.0) GOTO 3000
            IG1=NINT(R(1))
            IG2=NINT(R(2))
            IG3=NINT(R(3))
            DO IR=2,NROT
C             Rotations conserves norms
              IGL1 = NINT(
     &               FTABLE(1,1,IR) * R(1)
     &             + FTABLE(1,2,IR) * R(2)
     &             + FTABLE(1,3,IR) * R(3) )    
              IGL2 = NINT(
     &               FTABLE(2,1,IR) * R(1)
     &             + FTABLE(2,2,IR) * R(2)
     &             + FTABLE(2,3,IR) * R(3) )    
              IGL3 = NINT(
     &               FTABLE(3,1,IR) * R(1)
     &             + FTABLE(3,2,IR) * R(2)
     &             + FTABLE(3,3,IR) * R(3) )
              IF(IGL1.LT.0.OR.
     &             (IGL1.EQ.0.AND.IGL2.LT.0).OR.
     &             (IGL1.EQ.0.AND.IGL2.EQ.0.AND.IGL3.LT.0)) THEN
                IFAC=2
                IGL1=-IGL1+NH1
                IGL2=-IGL2+NH2
                IGL3=-IGL3+NH3
              ELSE
                IFAC=1
                IGL1=IGL1+NH1
                IGL2=IGL2+NH2
                IGL3=IGL3+NH3
              ENDIF
              IF(IGL1.EQ.IG1.AND.IGL2.EQ.IG2.AND.IGL3.EQ.IG3) THEN
C               IGG=IG
              ELSE
                DO IGG=IGBEG,IGEND
                  IF(  IGL1.EQ.INYH(1,IGG).AND.
     &                 IGL2.EQ.INYH(2,IGG).AND.
     &                 IGL3.EQ.INYH(3,IGG)) THEN
C                   Find one.
                    INDSHEL(IGG)=NUMSHEL
                    IF(TSYMMORPHIC) THEN
                      ISYSHEL(IFAC,IGG)=ISYSHEL(IFAC,IGG)+1
                    ELSE
#if defined(__NOINT8)
                      CALL IBSET4(NINSHEL(1,IGG),IR)
                      IF(IFAC.EQ.1) THEN
                        CALL IBSET4(ISISHEL(1,IGG),IR)
                      ENDIF
#else
                      NINSHEL(IGG)=IBSET(NINSHEL(IGG),IR)
                      IF(IFAC.EQ.1)
     &                     ISISHEL(IGG)=IBSET(ISISHEL(IGG),IR)
#endif
                    ENDIF
                    GOTO 2000
                  ENDIF
                ENDDO
              ENDIF
 2000         CONTINUE
            ENDDO               !NROT
 3000       CONTINUE
          ENDDO                 !end of while
        ENDIF
      ENDDO
      CALL GIVE_SCR_BUILDSHELL(LBUILDSHELL,TAG)
      CALL TEST_SCR('BUILDSHELL',TAG,LSCR,LBUILDSHELL)
C     ==--------------------------------------------------------------==
C     Check if all G-vector has a shell
      DO IG=1,NHG
        IF(INDSHEL(IG).EQ.INDNOSHEL) THEN
          IF(NPROC.NE.1) WRITE(6,'("[PROC=",I4,"]",$)') ME
          WRITE(6,*) 'G-VECTOR=',IG,' HAS NO SHELL NUMBER'
          CALL STOPGM('BUILDSHELL','NO SHELL NUMBER')
        ENDIF
      ENDDO
      CALL AZZERO(SCR,NUMSHEL)
      IF(TSYMMORPHIC) THEN
        IF(INVERSION.EQ.0) THEN
          DO IG=1,NHG
            ISH=INDSHEL(IG)
            IF(ISH.NE.0) THEN
              SCR(ISH)=SCR(ISH)+DBLE(ISYSHEL(1,IG)+ISYSHEL(2,IG))
            ENDIF
          ENDDO
        ELSE
          IF(GEQ0) ISYSHEL(1,1)=ISYSHEL(1,1)/2
          DO IG=1,NHG
            ISH=INDSHEL(IG)
            IF(ISH.NE.0) THEN
              SCR(ISH)=SCR(ISH)+DBLE(2*ISYSHEL(1,IG))
            ENDIF
          ENDDO
        ENDIF
      ELSE
#if defined(__NOINT8)
        DO IG=1,NHG
          ISH=INDSHEL(IG)
          IF(ISH.NE.0) THEN
            ISUM(1)=NINSHEL(1,IG)
            ISUM(2)=NINSHEL(2,IG)
            IFAC=0
            DO IR=1,NROT
              IF(BTEST4(ISUM(1),IR)) IFAC=IFAC+1
            ENDDO
            SCR(ISH)=SCR(ISH)+DBLE(IFAC)
          ENDIF
        ENDDO
#else
        DO IG=1,NHG
          ISH=INDSHEL(IG)
          IF(ISH.NE.0) THEN
            ISUM=NINSHEL(IG)
            IFAC=0
            DO IR=1,NROT
              IF(BTEST(ISUM,IR)) IFAC=IFAC+1
            ENDDO
            SCR(ISH)=SCR(ISH)+DBLE(IFAC)
          ENDIF
        ENDDO
#endif
      ENDIF
#ifdef PARALLEL
      CALL GLOSUM(NUMSHEL,SCR)
#endif
      IF(PARENT) THEN
        DO ISH=1,NUMSHEL
          IF(ABS(SCR(ISH)-NROT).GT.0.0001D0) THEN
            WRITE(6,*) ISH,SCR(ISH),NROT
            CALL STOPGM('BUILDSHELL','BAD NUMBER OF ROTATIONS')
          ENDIF
        ENDDO
      ENDIF
      CALL TIHALT('BUILDSHELL',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
#ifndef PARALLEL
C     ==================================================================
      SUBROUTINE SYMRHO_ALL(RHO,PSI,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     == SYMMETRIZE THE CHARGE DENSITY IN ALL FOURIER SPACE           ==
C     == WARNING: !!!!!!!! NO PARALLEL VERSION !!!!!!!!               ==
C     ==--------------------------------------------------------------==
C     == XTABLE is in crystal basis and FTABLE is in reciprocal basis ==
C     == For each operation IR, R^-1 gives the symmetry.              ==
C     == As R^-1 is in the group, we use XTABLE(IR) and FTAU(INVE(IR))==
C     ==--------------------------------------------------------------==
C     == Symmetrize all density array, more ROBUST than SYMRHO if     ==
C     == CDUAL is less than 4.0                                       ==
C     == More time-consuming :                                        ==
C     == symmorphic case : 12 times roughly                           ==
C     == non-symmorphic with inversion : case roughly the same        ==
C     == Do not use translation vectors associated with Identity      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'gvec.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'symm.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'cnst.inc'        !PI
      INCLUDE 'fft.inc'
C     Arguments
      INTEGER    LSCR
      REAL*8     RHO(NNR1),SCR(LSCR)
      COMPLEX*16 PSI(NNR1)
C     Variables
      COMPLEX*16 PSIP(NROT)
      POINTER    (IP_PSIP,PSIP)
      INTEGER    IROTP(NROT),IROTM(NROT),IGIR(3,NROT),IB(NROT)
      POINTER    (IP_IROTP,IROTP),(IP_IROTM,IROTM),
     &           (IP_IGIR,IGIR),(IP_IB,IB)
      COMPLEX*16 CRHOP,CRHOM
      INTEGER    ISUB,IR,IR2,NH1,NH2,NH3,IG1,IG2,IG3,IND1,IND2,IND3,
     &           IRFTAU,IGL1,IGL2,IGL3,INDY1,INDY2,INDY3,IL,NROTATION,
     &           LSYMRHO_ALL,IL_NR
      REAL*8     VNROT,ARG,TPI,TMPI
      CHARACTER  TAG*30
C     ==--------------------------------------------------------------==
      IF((INDPG.EQ.0).OR.(NROT.EQ.1)) RETURN
#ifdef PARALLEL
      CALL STOPGM('SYMRHO_ALL','NO PARALLEL VERSION')
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('SYMRHO_ALL',ISUB)
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_SYMRHO_ALL(LSYMRHO_ALL,TAG)
      CALL TEST_SCR('SYMRHO_ALL',TAG,LSCR,LSYMRHO_ALL)
#ifdef _BGL 
      IL_NR=NROT+MOD(NROT,2)
      CALL GET_ADDR(IP_PSIP,SCR(1))
      CALL GET_ADDR(IP_IROTP,SCR(1+2*IL_NR))
      CALL GET_ADDR(IP_IROTM,SCR(1+3*IL_NR))
      CALL GET_ADDR(IP_IB,SCR(1+4*IL_NR))
      CALL GET_ADDR(IP_IGIR,SCR(1+5*IL_NR))
#else
      CALL GET_ADDR(IP_PSIP,SCR(1))
      CALL GET_ADDR(IP_IROTP,SCR(1+2*NROT))
      CALL GET_ADDR(IP_IROTM,SCR(1+3*NROT))
      CALL GET_ADDR(IP_IB,SCR(1+4*NROT))
      CALL GET_ADDR(IP_IGIR,SCR(1+5*NROT))
#endif
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(PSI,MAXFFT)
      CALL DCOPY(NNR1,RHO(1),1,PSI(1),2)
C     RHO is used as flag array
      CALL AZZERO(RHO,NNR1)
C     Transform to reciprocal space
      CALL FWFFT(PSI)
C     ==--------------------------------------------------------------==
C     == LOOP OVER G-VECTORS                                          ==
C     ==--------------------------------------------------------------==
      NH1=NR1/2+1
      NH2=NR2/2+1
      NH3=NR3/2+1
      TMPI=-2.D0*PI
      TPI=2.D0*PI
C     NZH(1) is unchanged.
C     If symmorphic but non-standard origin, we translate the density
C     of -ORIGIN
      IF(TSYMMORPHIC.AND..NOT.TORIGIN) THEN
        DO IND3=1,NR3
          IG3=IND3-NH3
          DO IND2=1,NR2
            IG2=IND2-NH2
            DO IND1=1,NR1
              IG1=IND1-NH1
              IL = IND1 + KR1S * ( IND2 - 1 + KR2S * (IND3 - 1) )
              ARG=TPI*(ORIGIN(1)*IG1+ORIGIN(2)*IG2+ORIGIN(3)*IG3)
              PSI(IL)=PSI(IL)*DCMPLX(COS(ARG),SIN(ARG))
            ENDDO
          ENDDO
        ENDDO
      ENDIF
      DO IND3=1,NR3
        IG3=IND3-NH3
        DO IND2=1,NR2
          IG2=IND2-NH2
          DO IND1=1,NR1
            IL = IND1 + KR1S * ( IND2 - 1 + KR2S * (IND3 - 1) )
            IF(RHO(IL).EQ.0.D0) THEN
              NROTATION=1
              IB(1)=1
              IROTP(1)=IL
              IG1=IND1-NH1
              IROTM(1)=(NH1-IG1)
     &             + KR1S * ( (NH2-IG2)-1 + KR2S * ((NH3-IG3)-1) )
              IGIR(1,1)=IG1
              IGIR(2,1)=IG2
              IGIR(3,1)=IG3
              DO IR=2,NROT
C               Rotations conserves norms
                IGL1 = NINT(
     &                 FTABLE(1,1,IR) * IG1
     &               + FTABLE(1,2,IR) * IG2
     &               + FTABLE(1,3,IR) * IG3 )    
                IGL2 = NINT(
     &               FTABLE(2,1,IR) * IG1
     &               + FTABLE(2,2,IR) * IG2
     &               + FTABLE(2,3,IR) * IG3 )    
                IGL3 = NINT(
     &               FTABLE(3,1,IR) * IG1
     &               + FTABLE(3,2,IR) * IG2
     &               + FTABLE(3,3,IR) * IG3 )
                INDY1=NH1+IGL1
                INDY2=NH2+IGL2
                INDY3=NH3+IGL3
                IF(  (INDY1.GE.1.AND.INDY1.LE.NR1).AND.
     &               (INDY2.GE.1.AND.INDY2.LE.NR2).AND.
     &               (INDY3.GE.1.AND.INDY3.LE.NR3) ) THEN
                  NROTATION=NROTATION+1
                  IB(NROTATION)=IR
                  IGIR(1,NROTATION)=IGL1
                  IGIR(2,NROTATION)=IGL2
                  IGIR(3,NROTATION)=IGL3
                  IROTP(NROTATION) = (NH1+IGL1)
     &                 + KR1S * ( (NH2+IGL2)-1 + KR2S * ((NH3+IGL3)-1) )
                  IROTM(NROTATION) = (NH1-IGL1)
     &                 + KR1S * ( (NH2-IGL2)-1 + KR2S * ((NH3-IGL3)-1) )
                ENDIF
              ENDDO
              VNROT=1.D0/DBLE(NROTATION)
C             Calculate for all points in the star.
              IF(TSYMMORPHIC) THEN
C               All FTAU=0.D0, this is easiest (symmorphic group).
                CRHOP=0.D0
                DO IR=1,NROTATION
                  CRHOP=CRHOP+PSI(IROTP(IR))
                ENDDO
                CRHOP=VNROT*CRHOP
                CRHOM=DCONJG(CRHOP)
                DO IR=1,NROTATION
                  PSI(IROTP(IR))=CRHOP
                ENDDO
                DO IR=1,NROTATION
                  PSI(IROTM(IR))=CRHOM
                ENDDO
                DO IR=1,NROTATION
                  RHO(IROTP(IR))=1.D0
                  RHO(IROTM(IR))=1.D0
                ENDDO
              ELSE
C               Some FTAU/=0.D0, so more tricky
                DO IR=1,NROTATION
                  PSIP(IR)=PSI(IROTP(IR))
                ENDDO
C               There is a difference between the first point and the
C               other ones for FTAU.
                DO IR=1,NROTATION
C                 To avoid the calculation twice if inversion symmetry.
                  IF(RHO(IROTP(IR)).EQ.0.D0) THEN
                    CRHOP=PSIP(IR)
                    DO IR2=1,NROTATION
                      IF(IR2.NE.IR) THEN
                        IF(IR.EQ.1) THEN
                          IRFTAU=INVE(IB(IR2))
                        ELSE
                          IRFTAU=MULTAB(INVE(IB(IR2)),IB(IR))
                        ENDIF
                        IF(TSYMMOR(IRFTAU)) THEN
                          CRHOP=CRHOP+PSIP(IR2)
                        ELSE
                          ARG = TMPI *
     &                         ( IGIR(1,IR)*FTAU(1,IRFTAU)
     &                         + IGIR(2,IR)*FTAU(2,IRFTAU)
     &                         + IGIR(3,IR)*FTAU(3,IRFTAU) )
                          CRHOP= CRHOP
     &                         + PSIP(IR2)*DCMPLX(COS(ARG),SIN(ARG))
                        ENDIF
                      ENDIF
                    ENDDO
C                   We can update PSI,
C                   because all points will be unchanged.
                    PSI(IROTP(IR))=VNROT*CRHOP
C                   The density is real.
                    PSI(IROTM(IR))=VNROT*DCONJG(CRHOP)
                    RHO(IROTP(IR))=1.D0
                    RHO(IROTM(IR))=1.D0
                  ENDIF         !IF(RHO(IROTP(IR))
                ENDDO           !DO IR=1,NROTATION
              ENDIF             !IF(TSYMMORPHIC)
            ENDIF               !IF(RHO(IL)
          ENDDO                 !IND1
        ENDDO                   !IND2
      ENDDO                     !IND3
C     If symmorphic but non-standard origin, we translate the density
C     of ORIGIN
      IF(TSYMMORPHIC.AND..NOT.TORIGIN) THEN
        DO IND3=1,NR3
          IG3=IND3-NH3
          DO IND2=1,NR2
            IG2=IND2-NH2
            DO IND1=1,NR1
              IG1=IND1-NH1
              IL = IND1 + KR1S * ( IND2 - 1 + KR2S * (IND3 - 1) )
              ARG=TMPI*(ORIGIN(1)*IG1+ORIGIN(2)*IG2+ORIGIN(3)*IG3)
              PSI(IL)=PSI(IL)*DCMPLX(COS(ARG),SIN(ARG))
            ENDDO
          ENDDO
        ENDDO
      ENDIF
C     Transform back to real space
      CALL INVFFT(PSI) 
      CALL DCOPY(NNR1,PSI(1),2,RHO(1),1)
      CALL TIHALT('SYMRHO_ALL',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
#endif
C     ==================================================================
      SUBROUTINE GIVE_SCR_SYMVEC(LSYMVEC,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !ions.inc
      INCLUDE 'ions.inc'        !NAT
      INCLUDE 'symm.inc'        !INDPG NROT
C     Arguments
      INTEGER   LSYMVEC
      CHARACTER TAG*30
C     ==--------------------------------------------------------------==
      IF( (INDPG.EQ.0).OR.(NROT.EQ.1) ) THEN
        LSYMVEC=0
      ELSE
        LSYMVEC=6*NAT+10
        TAG='6*NAT'
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_SYMMAT(LSYMMAT,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !ions.inc
      INCLUDE 'ions.inc'        !NAT
      INCLUDE 'symm.inc'        !INDP NROT
C     Arguments
      INTEGER   LSYMMAT
      CHARACTER TAG*30
C     ==--------------------------------------------------------------==
      IF( (INDPG.EQ.0).OR.(NROT.EQ.1) ) THEN
        LSYMMAT=0
      ELSE
        LSYMMAT=2*9*NAT*NAT+10
        TAG='2*9*NAT*NAT'
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_BUILDSHELL(LBUILDSHELL,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !symm.inc ions.inc
      INCLUDE 'ions.inc'        !symm.inc (NAT)
      INCLUDE 'symm.inc'        !INDP NROT
C     Arguments
      INTEGER   LBUILDSHELL
      CHARACTER TAG*30
C     ==--------------------------------------------------------------==
      IF( (INDPG.EQ.0).OR.(NROT.EQ.1) ) THEN
        LBUILDSHELL=0
      ELSE
        IF(NUMSHEL.EQ.0) THEN
          LBUILDSHELL=3*NHGS/NROT+10
          TAG='3*NHGS/NROT'
        ELSE
          LBUILDSHELL=NUMSHEL+10
          TAG='NUMSHEL'
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE IMAGSET0(PSI,N)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER N
      REAL*8  PSI(2,N)
C     Variables
      INTEGER I
C     ==--------------------------------------------------------------==
      DO I=1,N
        PSI(2,I)=0.D0
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_SYMRHO(LSYMRHO,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !symm.inc ions.inc
      INCLUDE 'ions.inc'        !symm.inc (NAT)
      INCLUDE 'symm.inc'        !INDP NROT TSYMMORPHIC INVERSION
C     Arguments
      INTEGER   LSYMRHO
      CHARACTER TAG*30
C     Variables
      INTEGER   MAXPACK
#ifdef PARALLEL
      PARAMETER (MAXPACK=20000)
#else
      PARAMETER (MAXPACK=10000)
#endif
      INTEGER   LPACK,LBUILDSHELL
C     ==--------------------------------------------------------------==
      IF( (INDPG.EQ.0).OR.(NROT.EQ.1) ) THEN
        LSYMRHO=0
#ifndef PARALLEL
      ELSEIF( (CDUAL.LT.3.D0).OR.
     &      (.NOT.TSYMMORPHIC.AND.INVERSION.NE.0) ) THEN
        CALL GIVE_SCR_SYMRHO_ALL(LSYMRHO,TAG)
#endif
      ELSE
        IF(TSYMMORPHIC) THEN
          IF(NUMSHEL.EQ.0) THEN
            LSYMRHO=NHGS/MAX(1,MIN(NROT/2,NROT/NPROC))
          ELSE
            LSYMRHO=NUMSHEL
          ENDIF
          IF(INVERSION.EQ.0) LSYMRHO=2*LSYMRHO !Use complex
        ELSE                    !PSIP, IROTP (if not symmorphic)
          IF(NUMSHEL.EQ.0) THEN
            LPACK=MAXPACK
          ELSE
            LPACK=MIN(MAXPACK,NUMSHEL*(NROT*2+NROT))
          ENDIF
          LSYMRHO=MAX(LPACK,NROT*2+NROT) 
        ENDIF
        CALL GIVE_SCR_BUILDSHELL(LBUILDSHELL,TAG)
        LSYMRHO=MAX(LBUILDSHELL,LSYMRHO)+10
        TAG='2*NUMSHEL'
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
#ifndef PARALLEL
C     ==================================================================
      SUBROUTINE GIVE_SCR_SYMRHO_ALL(LSYMRHO_ALL,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !symm.inc ions.inc
      INCLUDE 'ions.inc'        !symm.inc (NAT)
      INCLUDE 'symm.inc'        !INDP NROT
C     Arguments
      INTEGER   LSYMRHO_ALL
      CHARACTER TAG*30
C     ==--------------------------------------------------------------==
      IF( NPROC.EQ.1.AND.
     &     ((INDPG.EQ.0).OR.(NROT.EQ.1)) ) THEN
        LSYMRHO_ALL=0
      ELSE
        LSYMRHO_ALL=8*NROT+10
        TAG='8*NROT'
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#endif
#if defined(__NOINT8)
      FUNCTION BTEST4(I,POS)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      LOGICAL BTEST4
      INTEGER I(2),POS
C     ==--------------------------------------------------------------==
      IF(POS.GT.31) THEN
        BTEST4=BTEST(I(2),POS-32)
      ELSE
        BTEST4=BTEST(I(1),POS)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE IBSET4(I,POS)
      IMPLICIT NONE
      INTEGER I(2),POS
C     ==--------------------------------------------------------------==
      IF(POS.GT.31) THEN
        I(2)=IBSET(I(2),POS-32)
      ELSE
        I(1)=IBSET(I(1),POS)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
#endif
