C     ==================================================================
C     Main source file of the wavefunction combination method
C     also contains the KS Hamiltonian matrix output functions (FODFT)
C     H. Oberhofer (ho246@cam.ac.uk) 2009
C     ==================================================================

C     ==================================================================
      SUBROUTINE SYSCOMB(C0,CM,N_S0,N_S1,N_S0UP,N_S1UP)
C     ==--------------------------------------------------------------==
C     == Combine two reference systems from RESTART.R* files          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'shop.inc'
      INCLUDE 'filn.inc'
      INCLUDE 'coor.inc'
      INCLUDE 'store.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
C     Arguments
      COMPLEX*16 C0(NGW,*),CM(NGW,*)
      COMPLEX*16 CBU(NGW,*),CBD(NGW,*)
      POINTER   (IP_CBU,CBU), (IP_CBD,CBD)
      INTEGER    N_S0,N_S1,N_S0UP,N_S1UP,I
C     Variables
      REAL*8  EIGV(N),TAUI(3,NAX,NSX),COM(3)
      REAL*8  SCR(*)
      POINTER (IP_SCR,SCR)
      INTEGER NSTATE,NFI,IREC(100)
      INTEGER LORTHO
      CHARACTER*20 FILN_BACK
      CHARACTER TAG*30

      FILN_BACK=FILN
      IF(PARENT)WRITE(6,*) 'Attempting to combine Systems'
      IF(.NOT.(RCO.AND.RWF))THEN
         IF(PARENT)WRITE(6,*) ' RESTART OPTION NOT ACTIVATED'
         CALL STOPGM('SYSCOMB','        ')
      ENDIF
      IREC(IREC_WF)=1
      FILN = "RESTART.R1"
      N=N_S0
      IF(TLSD)THEN
        NSUP=N_S0UP
        NSDOWN=N-NSUP
      ENDIF
      CALL ZHRWF(1,IREC,C0,CM,N_S0,EIGV,TAU0,VELP,TAUI,NFI)
      FILN = "RESTART.R2"
      N=N_S1
      IF(TLSD)THEN
        NSUP=N_S1UP
        NSDOWN=N-NSUP
      ENDIF
      CALL ZHRWF(1,IREC,C0(1,N_S0+1),CM(1,N_S0+1),
     &           N_S1,EIGV,TAU0,VELP,TAUI,NFI)
      IF(RGEO.AND.PARENT) CALL GEOFILE(TAU0,VELP,'READ')
      CALL WRITE_IREC(IREC)
      NSTATE = N_S0 + N_S1
      N=NSTATE
      IF(TLSD)THEN
        NSUP=N_S0UP+N_S1UP
        NSDOWN=N-NSUP
      ENDIF

      IF(PARENT)WRITE(6,*)"Found ",NSTATE,"States"
      IF(TLSD)THEN
        IF(PARENT)THEN
          WRITE(6,*)"Sorting up and down spins."
          WRITE(6,*)N_S1UP,N_S0UP,N_S1,N_S0
        ENDIF
        CALL MEMORY(IP_CBU,2*NGW*N_S1UP,"CBU")
        CALL DCOPY(2*NGW*N_S1UP,C0(1,N_S0+1),1,CBU,1)
        CALL MEMORY(IP_CBD,2*NGW*(N_S0-N_S0UP),"CBD")
        CALL DCOPY(2*NGW*(N_S0-N_S0UP),C0(1,N_S0UP+1),1,CBD,1)
        CALL DCOPY(2*NGW*N_S1UP,CBU,1,C0(1,N_S0UP+1),1)
        CALL DCOPY(2*NGW*(N_S0-N_S0UP),CBD,1,C0(1,N_S0UP+N_S1UP+1),1)
      ENDIF

      IF(TSCOMBM)THEN
        IF(PARENT)WRITE(6,*)"Mirroring WF around Centre of Mass"
        CALL GETCOM(COM)
        DO I=1,NSTATE
          CALL MIRROR(C0(1,I),COM)
        ENDDO
      ENDIF
      
      IF(TSCORTHO)THEN
        IF(PARENT)WRITE(6,*)"Orthogonalising Combined WF"
        CALL GIVE_SCR_ORTHO(LORTHO,TAG,N)
        CALL MEMORY(IP_SCR,LORTHO,'SCR')
        CALL ORTHO(N,C0,CM,SCR,LORTHO)
      ENDIF

      IF(PARENT)THEN
      WRITE(6,*)"Combined Wavefunctions from RESTART.R1 and RESTART.R2"
        WRITE(6,*)"to RESTART.1."
        WRITE(6,*)"EXITING NOW"
      ENDIF

   
      CALL ZHWWF(2,IREC,C0,CM,NSTATE,EIGV,TAU0,TAU0,TAUI,NFI)
      FILN = FILN_BACK
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE WRITE_KSHAM(C0,CM,SC0,RHOE,PSI,EIGV)
C     ==--------------------------------------------------------------==
C     == Calculate and write out the KS Hamiltonian Matrix            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'shop.inc'
      INCLUDE 'filn.inc'
      INCLUDE 'coor.inc'
      INCLUDE 'store.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'prden.inc'
C     Arguments
      COMPLEX*16 C0(NGW,*),CM(NGW,*),SC0(*)
      COMPLEX*16 PSI(NNR1,*)
      REAL*8  EIGV(*)
C     Variables
      REAL*8  SCR(*)
      REAL*8 HMAT(N,N)
      REAL*8  RHOE(NNR1,*)
      POINTER (IP_SCR,SCR)
      INTEGER LFORCES,I,J,K,L
      LOGICAL FERR
      CHARACTER TAG*30

      CALL GIVE_SCR_FORCEDR(LFORCES,TAG,N,.FALSE.,.TRUE.)
      CALL MEMORY(IP_SCR,LFORCES,'SCR')

      IF(PARENT)WRITE(6,*)"Calculating KS Hamiltonian"
      CALL RHOOFR(C0,RHOE,PSI,SCR,LFORCES,N)
      IF(TLSD)THEN
        L=NSUP
      ELSE
        L=N
      ENDIF
      K=L/2
      IF(RHOOUT) THEN
        MWFN(1)=-K
        MWFN(2)=-L
        CALL RHOPRI(C0,TAU0,RHOE,PSI,SCR,LFORCES,N,1)
      ENDIF

      CALL FORCEDR(C0,CM,SC0,RHOE,PSI,TAUP,FION,EIGV,SCR,LFORCES,
     *               N,1,.FALSE.,.TRUE.)
      CALL OVLAP(N,HMAT,C0,CM)
      CALL GLOSUM(N*N,HMAT)

      IF(PARENT)THEN
        OPEN(41,FILE="KS_HAM",STATUS='UNKNOWN')
        REWIND(UNIT=41)
        DO I=1,N
          WRITE(41,*)(-HMAT(I,J),J=1,N)
        ENDDO
        CLOSE(41)
        IF(TLSD)THEN
          WRITE(6,'(1X,"HOMO matrix element (",I3,",",I3,") :"
     &        ,F14.8)') K,L,HMAT(K,L)
        ELSE
          WRITE(6,'(1X,"HOMO matrix element (",I3,",",I3,") :"
     &        ,F14.8)') K,L,HMAT(K,L)/2.0D0
        ENDIF

        WRITE(6,*)"EXITING NOW"
      ENDIF

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE GETCOM(COM)
C     ==--------------------------------------------------------------==
C     == Calculate the position of the Centre of Mass                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'coor.inc' !<-- coordinates
      INCLUDE 'rmas.inc'

      REAL*8     COM(3),CO(3),PMA00
      INTEGER    IS,IA
      DO IS=1,NSP
        DO IA=1,NA(IS)
          CO(1)=CO(1)+TAU0(1,IA,IS)*PMA0(IS)
          CO(2)=CO(2)+TAU0(2,IA,IS)*PMA0(IS)
          CO(3)=CO(3)+TAU0(3,IA,IS)*PMA0(IS)
          PMA00=PMA00+PMA0(IS)
        ENDDO
      ENDDO
      COM(1)=CO(1)/PMA00
      COM(2)=CO(2)/PMA00
      COM(3)=CO(3)/PMA00
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE MIRROR(WAVE,COM)
C     ==--------------------------------------------------------------==
C     == Mirror wavefunctions                                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'
      INCLUDE 'cnst.inc'

      COMPLEX*16 WAVE(NGW)
      COMPLEX*16 F
      REAL*8     COM(3),TEMP
      INTEGER    IG

      DO IG=1,NGW
!         TEMP=GK(1,IG)*COM(1)+GK(2,IG)*COM(2)+GK(3,IG)*COM(3)
!         F=EXP( DCMPLX(0.0D0,4.0D0*PI*TEMP))
!         WAVE(IG)=F*DCONJG(WAVE(IG))
        WAVE(IG)=DCONJG(WAVE(IG))
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE QM_CENTRE()
C     ==--------------------------------------------------------------==
C     == reposition Centre of Mass                                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'coor.inc' !<-- coordinates
      INCLUDE 'rmas.inc'
      INCLUDE 'cell.inc'

      INTEGER    IS,IA
      REAL*8     CX,CY,CZ,DCX,DCY,DCZ

      CX=0.0D0
      CY=0.0D0
      CZ=0.0D0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          CX=CX+TAU0(1,IA,IS)*PMA0(IS)/PMAT0
          CY=CY+TAU0(2,IA,IS)*PMA0(IS)/PMAT0
          CZ=CZ+TAU0(3,IA,IS)*PMA0(IS)/PMAT0
        ENDDO
      ENDDO
      DCX=0.5D0*CELLDM(1)-CX
      DCY=0.5D0*CELLDM(2)*CELLDM(1)-CY
      DCZ=0.5D0*CELLDM(3)*CELLDM(1)-CZ
      DO IS=1,NSP
        DO IA=1,NA(IS)
          TAU0(1,IA,IS)=TAU0(1,IA,IS)+DCX
          TAU0(2,IA,IS)=TAU0(2,IA,IS)+DCY
          TAU0(3,IA,IS)=TAU0(3,IA,IS)+DCZ
        ENDDO
      ENDDO
      IF(PARENT)WRITE(6,'(/,A,A,/)')
     *        ' >>>>>>>> CENTER OF MASS HAS BEEN MOVED',
     *                       ' TO CENTER OF BOX <<<<<<<<'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
