C     ==================================================================
C     == MAXSP  : MAXIMUM NUMBER OF ATOMIC SPECIES                    ==
C     == NHX    : MAXIMUM NUMBER OF GAUSS-HERMIT POINTS               ==
C     ==        : MAXIMUM NUMBER OF BETA FUNCTIONS (VANDERBILT)       ==
C     ==        : MAXIMUM NUMBER OF PROJECTORS FOR NON-LOCAL PP       ==
C     == NACX   : SIZE OF THE ACCUMULATOR ARRAY                       ==
C     == MAXDIS : MAX. NUMBER OF VECTORS IN DIIS                      ==
C     == MXGDIS : MAX. NUMBER OF VECTORS IN GDIIS                     ==
C     == NBRX   : NUMBER OF DISTINCT RADIAL BETA FUNCTIONS            ==
C     == LX     : 2*LMAX-1 WHERE LMAX=3 IF S,P AND D FUNCTIONS        ==
C     == NLX    : COMBINED ANGULAR MOMENTUM FOR LLi (VANDERBILT)      ==
C     == MX     : 2*LX-1                            (VANDERBILT)      ==
C     == LIX    : MAX. LLi                          (VANDERBILT)      ==
C     == MIX    : 2*LIX-1                           (VANDERBILT)      ==
C     == LMAXX  : MAXIMUM NUMBER OF ANGULAR MOMENTUM                  ==
C     == LM1X   : LMAXX-1                                             ==
C     ==================================================================
      INTEGER   MAXSP,NHX,NACX,MAXDIS,MXGDIS,NBRX,LX,NLX,MX,LIX,MIX,
     &          LMAXX,LM1X
      PARAMETER (MAXSP=140)
      PARAMETER (NHX=120)
      PARAMETER (NACX=50)
      PARAMETER (MAXDIS=20)
      PARAMETER (MXGDIS=20)
      PARAMETER (NBRX=6)
      PARAMETER (LX=5)
      PARAMETER (NLX=9,MX=2*LX-1,LIX=3,MIX=LIX*2-1)
      PARAMETER (LMAXX=4,LM1X=LMAXX-1)
C     ==================================================================
C     == MAXSYS : Maximum values for system                           ==
C     ==          NSX   Number of atomic species                      ==
C     ==          NAX   Maximum number of atoms per species           ==
C     ==          NCORX (NSX*(NSX+1))/2 (detsp.F)                     ==
C     ==          NHXS  Maximum number for all species as NHX         ==
C     ==          LPMAX Maximum value of angular momentum             ==
C     ==          MMAXX Maximum number of spline points               ==
C     ==--------------------------------------------------------------==
      INTEGER        NSX,NAX,NSAX,NCORX,NHXS,LPMAX,MMAXX
      COMMON/MAXSYS/ NSX,NAX,NSAX,NCORX,NHXS,LPMAX,MMAXX
C     ==================================================================
C     == SPAR   : Global array dimensions (parallel work)             ==
C     ==          NHGS = sum of NHG for all processors                ==
C     ==          NHGLS= sum of NHGL     --                           ==
C     ==          NHGKS= sum of NHGK     --                           ==
C     ==          NGWS = sum of NGW      --                           ==
C     ==          NGWKS= sum of NGWKS    --                           ==
C     ==          NGWLS= sum of NGWL     --                           ==
C     ==          NR1S,NR2S,NR3S total mesh dimension                 ==
C     ==================================================================
      INTEGER        NHGS,NHGLS,NHGKS,NGWS,NGWKS,NGWLS,
     &               NLDXS,NR1S,NR2S,NR3S
      COMMON/SPAR/   NHGS,NHGLS,NHGKS,NGWS,NGWKS,NGWLS,
     &               NLDXS,NR1S,NR2S,NR3S
C     ==--------------------------------------------------------------==
C     == NCPW   : Number of plane waves                               ==
C     ==          NHG  number of G-components for electronic density  ==
C     ==          NHGL number of G-shells for electronic density      ==
C     ==          NGW  number of G-components for wavefunctions       ==
C     ==          NGWL number of G-shells for wavefunctions           ==
C     ==--------------------------------------------------------------==
      INTEGER        NHG,NHGL,NGW,NGWL
      COMMON/NCPW/   NHG,NHGL,NGW,NGWL
C     ==--------------------------------------------------------------==
C     == NKPT   : Used with K-points options                          ==
C     ==          NHGK = 2 * NHG (no inversion symmetry)              ==
C     ==          NGWK = 2 * NGW (no inversion symmetry)              ==
C     ==          NKPNT - number of k points  (in memory)             ==
C     ==          NKPTS - number of total k points >= NKPNT           ==
C     ==          NBLKP - number of k points block                    ==
C     ==          NKPBL(NBLKP) - number of k points for each block    ==
C     ==          KPBEG(NBLKP) - number - 1 of first k-pt in block    ==
C     ==--------------------------------------------------------------==
      INTEGER        NHGK,NGWK,NKPNT,NKPTS,NBLKP
      COMMON/NKPT/   NHGK,NGWK,NKPNT,NKPTS,NBLKP
      INTEGER        NKPBL(*),KPBEG(*)
      POINTER        (IP_NKPBL,NKPBL),(IP_KPBEG,KPBEG)
      COMMON/NKIP/   IP_NKPBL,IP_KPBEG
C     ==--------------------------------------------------------------==
C     == PARM   : Information about supercell and mesh                ==
C     ==          ALAT lattice parameter                              ==
C     ==          A1(3), A2(3), A3(3) crystal vector basis            ==
C     ==          OMEGA Supercell volume                              ==
C     ==          TPIBA=2.D0*PI/ALAT                                  ==
C     ==          TPIBA2=TPIBA*TPIBA                                  ==
C     ==          APBC(4) used by PBC routine                         ==
C     ==                  for periodic boundary condition             ==
C     ==          IBRAV Determines shape and symmetry of the supercell==
C     ==              =0 Isolated system (cubic cell)                 ==
C     ==               1 Simple cubic                                 ==
C     ==               2 Face centred cubic                           ==
C     ==               3 Body centred cubic                           ==
C     ==               4 Hexagonal cell                               ==
C     ==               5 Trigonal or rhombohedral cell                ==
C     ==               6 Tetragonal                                   ==
C     ==               7 Body tetragonal                              ==
C     ==               8 Orthorhombic                                 ==
C     ==              12 Monoclinic                                   ==
C     ==              14 Triclinic                                    ==
C     ==          NR1, NR2, NR3 mesh dimension for each processor     ==
C     ==--------------------------------------------------------------==
      REAL*8         ALAT,A1(3),A2(3),A3(3),OMEGA,TPIBA,TPIBA2,APBC(4)
      INTEGER        IBRAV,NR1,NR2,NR3
      COMMON/PARM/   ALAT,A1,A2,A3,OMEGA,TPIBA,TPIBA2,APBC,
     *               IBRAV,NR1,NR2,NR3
C     ==--------------------------------------------------------------==
C     == FPAR   : Leading dimensions for some arrays, especially FFT  ==
C     ==          KR1, KR2, KR3 FFT mesh dimension for each proc.     ==
C     ==          KR1S, KR2S, KR3S FFT mesh for all proc.             ==
C     ==          KRX Used in FFTPRP routine with groups              ==
C     ==          KRY, KRZ UNUSED                                     ==
C     ==          NNR1 Number of real components for density          ==
C     ==               for each processor                             ==
C     ==--------------------------------------------------------------==
      INTEGER        KR1,KR2,KR3,KR1S,KR2S,KR3S,KRX,KRY,KRZ,NNR1
      COMMON/FPAR/   KR1,KR2,KR3,KR1S,KR2S,KR3S,KRX,KRY,KRZ,NNR1
C     ==--------------------------------------------------------------==
C     == ACCU   : Accumulator quantities                              ==
C     ==          NACC number of quantities calculated per self-c iter==
C     ==          ACC(1:NACC) values of calculated quantities         ==
C     ==--------------------------------------------------------------==
      INTEGER        NACC
      REAL*8         ACC
      COMMON/ACCU/   ACC(NACX),NACC
C     ==================================================================
C     == ALL THE INFO FOR PARALLEL                                    ==
C     ==================================================================
C     ==--------------------------------------------------------------==
C     == MAXCPU  : Maximum number of CPU
C     ==--------------------------------------------------------------==
      INTEGER       MAXCPU
      PARAMETER     (MAXCPU=8192)
C     ==--------------------------------------------------------------==
C     == PARENT  : .TRUE if IP = SOURCE                               ==
C     ==--------------------------------------------------------------==
      LOGICAL       PARENT,MMPARENT,QMMMPARENT,QMNODE,MMNODE
      COMMON/PARAL/ PARENT,MMPARENT,QMMMPARENT,QMNODE,MMNODE
C     ==--------------------------------------------------------------==
C     == NCPUS   : value of NCPUS environment variable                ==
C     == NPROC   : TOTAL NUMBER OF PROCESSORS                         ==
C     == ME      : CPU index                                          ==
C     == MEPOS   : equal to ME                                        ==
C     == SOURCE  : MASTER CPU index                                   ==
C     == IGEQ0   : PROCESSOR INDEX OF G=0 COMPONENT                   ==
C     == ALLGRP  : GIVEN VALUE FOR BROADCAST COMMUNICATION            ==
C     == NHRAYS  : number of rays for the processor for the density   ==
C     == NGRAYS  : number of rays for the processor for the wavefunc. ==
C     ==--------------------------------------------------------------==
      INTEGER       NCPUS,NPROC,ME,MEPOS,SOURCE,IGEQ0,ALLGRP,
     &              NHRAYS,NGRAYS,QMMMNPROC,MMNPROC,QMMMGRP,MMGRP,
     &              QMMMSOURCE,MMSOURCE
      COMMON/PARAI/ NCPUS,NPROC,ME,MEPOS,SOURCE,IGEQ0,ALLGRP,
     &              NHRAYS,NGRAYS,QMMMNPROC,MMNPROC,QMMMGRP,MMGRP,
     &              QMMMSOURCE,MMSOURCE
C     ==--------------------------------------------------------------==
C     == NRXPL   : NR1 for each processor                             ==
C     == NRZPL   : Number of z-planes for each processor              ==
C     == SPARM   : Information about dimension of distributed data    ==
C     == NST12   : index of the first and last distributed states     ==
C     ==         : per proc                                           ==
C     == PGROUP  : Pointer position --> PE number                     ==
C     == NLINK   : Pointer PE number --> position - 1                 ==
C     == NWA12   : index of distributed states (dynamical)            ==
C     ==--------------------------------------------------------------==
      INTEGER       NRXPL(0:MAXCPU,2),NRZPL(0:MAXCPU,2),
     &              SPARM(9,0:MAXCPU),NST12(0:MAXCPU,2),
     &              PGROUP(0:MAXCPU),NLINK(0:MAXCPU)
      COMMON/PARAP/ NRXPL,NRZPL,SPARM,NST12,PGROUP,NLINK
      INTEGER       NWA12(0:MAXCPU,2)
      COMMON/PARAW/ NWA12
C     ==================================================================
C     == MAPPING G VECTORS-->PROCESSES                                ==
C     ==================================================================
C     == SUBDIVIDE THE PROCESSORS IN GROUPS (see GROUPS routine)      ==
C     == MAPGP(1:NHG) : Use for distribution of wavefunction          ==
C     == MAXGRP=MAXCPU : Maximum taskgroups (for distributed FFT)     ==
C     == NOGRP  : Number of groups                                    ==
C     == NPGRP  : Number of processors per groups                     ==
C     == MEOGRP : Index of the group for the processor                ==
C     == MEPGRP : Index of the processor in the group                 ==
C     == MPEN   : Dimensions used for groups (see FFTPRP routine)     ==
C     == MPENM  : ''                  ''          ''                  ==
C     == NOLIST(1:MAXGRP) : List of processors in the orbital group   ==
C     == NPLIST(1:MAXGRP) : List of processors in the plane wave group==
C     ==--------------------------------------------------------------==
      INTEGER       MAPGP(*)
      POINTER       (IP_MAPGP,MAPGP)
      COMMON/MAPP/  IP_MAPGP
      INTEGER       MAXGRP
      PARAMETER     (MAXGRP=MAXCPU)
      INTEGER       NOGRP,NPGRP,MEOGRP,MEPGRP,MPEN,MPENM,
     &              NOLIST(MAXGRP),NPLIST(MAXGRP)
      COMMON/GROUP/ NOGRP,NPGRP,MEOGRP,MEPGRP,MPEN,MPENM,NOLIST,NPLIST
C     ==================================================================
C     == MAPPING ATOMS-->PROCESSES                                    ==
C     ==================================================================
C     == NATPE           =IPEPT(2,MEPOS)-IPEPT(1,MEPOS)+1             ==
C     ==                 Number of atoms per processes                ==
C     == NORBPE          =NST12(MEPOS,2)-NST12(MEPOS,1)+1             ==
C     ==                 Number of orbitals per processes             ==
C     == IPEPT(2,0:MAXCPU) Indexes of atoms per processes             ==
C     == IATPT(2,NAT)                                                 ==
C     == IATPE(NAT)                                                   ==
C     ==--------------------------------------------------------------==
      INTEGER NATPE,NORBPE
      INTEGER IPEPT(2,0:MAXCPU)
      INTEGER IATPT,IATPE
      DIMENSION IATPT(2,*),IATPE(*)
      POINTER(IP_IATPT,IATPT),(IP_IATPE,IATPE)
      COMMON/ATPT/NATPE,NORBPE,IPEPT,IP_IATPT,IP_IATPE
C     ==================================================================
C     == THE CONTROL PARAMETERS                                       ==
C     ==================================================================
C     == MLOGO : NUMBER OF LOGICAL VARIABLES IN CNTL COMMON           ==
C     == MINTE : NUMBER OF INTEGER VARIABLES IN CNTI COMMON           ==
C     == MREAL : NUMBER OF REAL*8  VARIABLES IN CNTR COMMON           ==
C     ==--------------------------------------------------------------==
      INTEGER   MLOGO,MINTE,MREAL
      PARAMETER (MLOGO=154)
      PARAMETER (MINTE=54)
      PARAMETER (MREAL=56)
C     ==================================================================
C     == A LOT OF ITEMS IN THE COMMON BLOCKS IS NEEDED FOR PARALLEL   ==
C     ==================================================================
C     == CNTL1  : Use for the broadcast (dummy variable)              ==
C     == MD     : Molecular dynamics option                           ==
C     == GEOPT  : Geometry optimization                               ==
C     == WFOPT  : Wavefunction optimization                           ==
C     == TLOWD  : Lowdin orthogonalization instead of Gram-Schmidt    ==
C     == TRANE  : Randomize wavefunctions                             ==
C     == TRANP  : Randomize ionic coordinates                         ==
C     == TC     : Temperature control for electrons                   ==
C     == TCP    : Temperature control for ions                        ==
C     == TBERP  : Berendsen for ions                                  == 
C     == TBERE  : Berendsen for electrons                             == 
C     == TBERC  : Berendsen for cell                                  == 
C     == ANNEI  : Simulated annealing for ions                        ==
C     == ANNEE  : Simulated annealing for electrons                   ==
C     == ANNEC  : Simulated annealing for cell                        ==
C     == DAMPI  : Damped dynamics for ions                            ==
C     == DAMPE  : Damped dynamics for electron                        ==
C     == DAMPC  : Damped dynamics for cell                            ==
C     == QUENCHP: Quench ionic velocities                             ==
C     == QUENCHE: Quench electronic velocities (Car-Parrinello)       ==
C     == QUENCHB: Quench system to the Born-Oppenheimer surface       ==
C     == CNSTMD : Iterative orthogonalization (Car-Parrinello)        ==
C     == TSDE   : Steepest descent (electronic coordinates)           ==
C     == TSDE_P : Steepest descent in DFTP
C     == TSDP   : Steepest descent (ionic coordinates)                ==
C     == DIIS   : Wavefunction optimization by DIIS                   ==
C     == PREC   : Preconditioning for Steepest descent (electronic)   ==
C     == PREC_P : Preconditioning for DFTP
C     == TSCALE : Use Scaled ionic coordinates                        ==
C     == BOHR   : Use atomic units for ionic coordinates (bohr)       ==
C     == PCG    : Wavefunction optimization by Precond. Conj. Gradient==
C     == PCG_P  : Wavefun. optimization by Precond.Conj.Gradient DFTP ==
C     == GDIIS  : Geometry optimization by GDIIS                      ==
C     == RFO    : Geometry optimization by Rational Function Approx.  ==
C     == BFGS   : Geometry optimization by quasi-Newton update        ==
C     == LBFGS  : Geometry optimization low-memory BFGS implicit upd. ==
C     == PRFO   : Use partitioned rational function optimizer (P-RFO) ==
C     == TSDIN  : Second derivatives from input                       ==
C     == TGC    : Gradient correction                                 ==
C     == TGCX   : Gradient correction for exchange part               ==
C     == TGCC   : Gradient correction for correlation part            ==
C     == TTAU   : Tau dependent functional                            ==
C     == THYBRID: Hybrid functionals                                  ==
C     == TSMOOTH: Smoothing of the density                            ==
C     == CALDIP : Calculate Dipole dynamics                           ==
C     == TNOSEE : Nose-Hoover thermostats for electrons dynamics      ==
C     == TNOSEP : Nose-Hoover thermostats for ions dynamics           ==
C     == TNOSES : One thermostat chain per species                    ==
C     == KSENER : Calculate Kohn-Sham energies for the final potential==
C     == WCOMP  : Wavefunction in compressed form                     ==
C     == NONORT : Use nonorthogonal orbital                           ==
C     == BIGMEM : Big memory option                                   ==
C     == THARM  : Harmonic reference system integration               ==
C     == TMASS  : Scaled electron masses                              ==
C     == KRWFN  : Store real space representation of wavefunctions    ==
C     == PCGMIN : Perform quadratic line search                       ==
C     == PCGMIN_P : Perform quadratic line search  in DFPT            ==
C     == PROPER : Properties calculation                              ==
C     == VIBRAT : Vibrational analysis                                ==
C     == TSDAN  : Linear Response for second derivatives              ==
C     == TLSD   : Local Spin Density Approximation                    ==
C     == TPRES  : Stress tensor calculation                           ==
C     == SIMUL  : Combined geometry/wavefunction scheme               ==
C     == TPRCP  : Parrinello-Rahman-Car-Parrinello                    ==
C     == TNOSEC : Nose-Hoover thermostats for cell dynamics           ==
C     == QUENCHC: Quench electronic velocities (Parrinello-Rahman)    ==
C     == TSDC   : Cell dynamics by Steepest descent                   ==
C     == TRANC  : Randomize cell                                      ==
C     == TMEMCHK: Memory checking                                     ==
C     == TPENN  : NPT dynamics                                        ==
C     == TFINT  : Free Energy Functional (Alavi''s method)            ==
C     == TDIAG  : Diagonalisation scheme (Lanczos or Davidson)        ==
C     == TDIAGOPT : Optimize then diagonalize                         ==
C     == TDAVI  : Davidson diagonalisation                            ==
C     == TLANC  : Lanczos diagonalisation                             ==
C     == TFRHO_UPW: Update Wavefunction with Rho fixed (DIIS + mixing)==
C     == TFRSBLK: New Block Lanczos diagonalisation                   ==
C     == TDIPD  : Dipole Dynamics                                     ==
C     == TINTER : Interface to classical MD program                   ==
C     == TCC    : Temperature Control for the cell                    ==
C     == TEPOT  : Electrostatic Potential                             ==
C     == TEXPOT : External Potential                                  ==
C     == TEXADD : Add energy and forces of external potential         ==
C     == TPATH  : Path Integrals (PIMD or PATH for saddle point)      ==
C     == TPIMD  : Path Integral Molecular Dynamics                    ==
C     == TPMIN  : Path Minimisation (Saddle point determination)      ==
C     == TRESCALE : Re-adjust ionic velocities only after restart to  ==
C     ==          desired temperature TEMPW                           ==
C     == TSYMRHO: SYMMTRIZE DENSITY                                   ==
C     == THARD  : Calculate orbital hardness matrix                   ==
C     == TSPEC  : Calculate spectra with TDDFT                        ==
C     == TSHOP  : Surface hopping dynamics S0/S1                      ==
C     == TFUSI  : Fuse two Restart files for S0/S1                    ==
C     == TMERGE : Merge two RESTART files and shift the wavefunction  ==
C     == TSEP   : Separate one Restart into two files for S0/S1       ==
C     == TREVERS: Invert ionic and electronic velocities              ==
C     == TQMMM  : Use the QM/MM code (full Bio code)                  ==
C     == TQMMECH: Use the QM/MM code (simple coupling only)           ==
C     == TDEBFOR: Run option for force debugging                      ==
C     == TDDFT  : Use TDDFT                                           ==
C     == TNOGEOCHECK : Dont check geometry for close atoms            ==
C     == TSSEL  : Print structure only for selected atoms             ==
C     == TWSERIAL : Calculate Wannier centers with serial code        ==
C     == TFIELD : Apply an external static electric field             ==
C     == TPOTENTIAL : Indicates a potential only functional           ==
C     == TR4A2A : Use single precision in ALLTOALL communication      ==
C     == TEXTRAP: Use wavefunction extrapolation in BOMD              ==
C     == TSTRXTP: Store wavefunction history in restart file          ==
C     == TASPC  : Use the ASPC interpolator for extrapolation         ==
C     == TDMAL  : Use distributed linear algebra routines             ==
C     == TPCGFI : Use PCG MINIMIZE for first Wfopt (EGO/GMX interface)==
C     == TPCGFIC: Use PCG MINIMIZE for first Wfopt (CDFT)             ==
C     == TCART  : Use cartesian taskgroups                            ==
C     == GSHELLOUT : Write file with Gshell information               ==
C     == TSHOCK : Shock wave simulation                               ==
C     == TSIC   : SS-SIC self-intercation correction                  ==
C     == TFFTWSDM: Use fft "wisdom" in FFTW interface                 ==
C     == TNACVS : TDDFT based non adiabatic coupling vectors          ==
C     == CDFT   : Use charge constraints                              ==
C     == CDFT_NEWT   : Use Newton Optimiser for CDFT                  ==
C     == CDFT_DEKK   : Use Dekker Optimiser for CDFT                  ==
C     == CDFT_WEIGHT : Write out a slice of the weight                ==
C     == CDFT_WF     : Write out the full weight                      ==
C     == TSYSCOMB    : Combine two reference systems                  ==
C     == TSCOMBM     : Mirror the resulting WF around COM             ==
C     == TSCORTHO    : Orthogonalise the resulting WF                 ==
C     == TKSHAM      : Write out Kohn Sham Hamiltonian                ==
C     == TIPAO       : Initialise with the primitive AO method        ==
C     == TLPOT  : Store local potential                               ==
C     == FMATCH : Use to run force matching in QM/MM                  ==
C     ==--------------------------------------------------------------==
      LOGICAL      CNTL1,MD,TMDBO,TMDFILE,GEOPT,WFOPT,TLOWD,
     *             TRANE,TRANP,THEAD,TC,TCP,ANNEI,ANNEE,ANNEC,
     *             DAMPI,DAMPE,DAMPC,QUENCHP,QUENCHE,QUENCHB,CNSTMD,
     *             STOPOT,TCHNGW,TSDE,TSDP,DIIS,PREC,TSCALE,
     *             BOHR,TCHNGR,PCG,GDIIS,RFO,BFGS,TGC, 
     *             TTAU,TGCX,TGCC,TSMOOTH,TBERP,TBERE,TBERC,
     *             CALDIP,TNOSEE,TNOSEP,TNOSES,TIMING,
     *             KSENER,ORBROT,RCOMP,WCOMP,NONORT,BIGMEM,GAUDYN,
     *             THARM,TMASS,FCNSTR,KRWFN,PCGMIN,POSVER,
     *             PROPER,VIBRAT,INITCM,FINALCM,TLSD,TPRES,SIMUL,
     *             TPRCP,TNOSEC,QUENCHC,TSDC,TRANC,
     *             TMEMCHK,TPENN,TFINT,TDIAG,TDIAGOPT,TDAVI,TLANC,
     *             TFRSBLK,TDIPD,TINTER,TCC,TEPOT,TEXPOT,TEXADD,
     *             TPATH,TPIMD,TPMIN,  
     *             TRESCALE,TSAMPL,TPREC,TFDIST,TSYMRHO,TRESPONSE,TSDAN,
     *             THARD,TSPEC,TSHOP,TFUSI,TMERGE,
     *             TREVERS,TQMMM,TQMMECH,TINR,
     *             THYBRID,TDEBFOR,TDDFT,LBFGS,PRFO,TSDIN,TNOGEOCHECK, 
     *             TSSEL,TFRHO_UPW,TWSERIAL,BSYMM,TFIELD,TPOTENTIAL,
     *             TR4A2A,TEXTRAP,TDMAL,TSEP,TPCGFI,TPCGFIC,TCART,
     *             GSHELLOUT,TSHOCK,TSIC,TSTRXTP,TASPC,TFFTWSDM,TNACVS,
     *             CDFT,CDFT_NEWT,CDFT_DEKK,CDFT_WEIGHT,CDFT_WF,
     *             TSYSCOMB,TSCOMBM,TSCORTHO,TIPAO,TKSHAM,FMATCH,
     *             CMPLX_WF,START_REAL,TMDEH,CHEBY,CAYLEY,RUKU,
     *             TGAUGEP,TGAUGEF,TPDIST,TPSPEC
      COMMON/CNTL/ CNTL1,MD,TMDBO,TMDFILE,GEOPT,WFOPT,TLOWD,
     *             TRANE,TRANP,THEAD,TC,TCP,ANNEI,ANNEE,ANNEC,
     *             DAMPI,DAMPE,DAMPC,QUENCHP,QUENCHE,QUENCHB,CNSTMD,
     *             STOPOT,TCHNGW,TSDE,TSDP,DIIS,PREC,TSCALE,
     *             BOHR,TCHNGR,PCG,GDIIS,RFO,BFGS,TGC,
     *             TTAU,TGCX,TGCC,TSMOOTH,TBERP,TBERE,TBERC,
     *             CALDIP,TNOSEE,TNOSEP,TNOSES,TIMING,
     *             KSENER,ORBROT,RCOMP,WCOMP,NONORT,BIGMEM,GAUDYN,
     *             THARM,TMASS,FCNSTR,KRWFN,PCGMIN,POSVER,
     *             PROPER,VIBRAT,INITCM,FINALCM,TLSD,TPRES,SIMUL,
     *             TPRCP,TNOSEC,QUENCHC,TSDC,TRANC,
     *             TMEMCHK,TPENN,TFINT,TDIAG,TDIAGOPT,TDAVI,TLANC,
     *             TFRSBLK,TDIPD,TINTER,TCC,TEPOT,TEXPOT,TEXADD,
     *             TPATH,TPIMD,TPMIN,
     *             TRESCALE,TSAMPL,TPREC,TFDIST,TSYMRHO,TRESPONSE,TSDAN,
     *             THARD,TSPEC,TSHOP,TFUSI,TMERGE,
     *             TREVERS,TQMMM,TQMMECH,TINR,
     *             THYBRID,TDEBFOR,TDDFT,LBFGS,PRFO,TSDIN,TNOGEOCHECK,
     *             TSSEL,TFRHO_UPW,TWSERIAL,BSYMM,TFIELD,TPOTENTIAL,
     *             TR4A2A,TEXTRAP,TDMAL,TSEP,TPCGFI,TPCGFIC,TCART,
     *             GSHELLOUT,TSHOCK,TSIC,TSTRXTP,TASPC,TFFTWSDM,TNACVS,
     *             CDFT,CDFT_NEWT,CDFT_DEKK,CDFT_WEIGHT,CDFT_WF,
     *             TSYSCOMB,TSCOMBM,TSCORTHO,TIPAO,TKSHAM,FMATCH,
     *             CMPLX_WF,START_REAL,TMDEH,CHEBY,CAYLEY,RUKU,
     *             TGAUGEP,TGAUGEF,TPDIST,TPSPEC
C     ==================================================================
C     == CNTI1  : Use for the broadcast (dummy variable)              ==
C     == NOMORE : Maximum number of steps                             ==
C     == NOMORE_ITER: Maximum number of iteration for SC              ==
C     ==              Used only with TDIAG or TMDBO                   ==
C     == MAXIT  : Parameters for the RATTLE step                      ==
C     == MDIIS  : Maximum number of vectors retained for DIIS (Wave.) ==
C     == INSYS  : Logic unit for properties (by default=5)            ==
C     == MGDIIS : Size of GDIIS matrix (geometry optimization)        ==
C     == NPARA  : Initial Hessian (Unit, Disco or Schlegel)           ==
C     == NCHP   : Number of Nose thermostats (ions)                   ==
C     == NCHB   : Number of Nose thermostats (cell)                   ==
C     == NCHS   : Number of Nose thermostats (electrons)              ==
C     == NCALLS0: Number of Yoshida-Suzuki steps (Nose)               ==
C     == NIT0   : Number of integration cycles (Nose)                 ==
C     == NKSSTA : Number of Kohn-Sham eigenvalues                     ==
C     == IMOVIE : Write movie file every imovie steps                 ==
C     == IPROJ  : Electronic gradient projection to be used           ==
C     == INWFUN : Wavefunction initialization (1=random, 2=atomic)    ==
C     == NRESTF : Number of different restart files                   ==
C     == NTRANS : Use in Lanczos scheme                               ==
C     == NTRAJ  : Trajectories are saved on file every NTRAJ          ==
C     == NPRES  : Stress tensor calculation each NPRES                ==
C     == NSPLP  : Number of points for spline                         ==
C     == NSORDER: Saddle point order for geo. opt. by rational func.  ==
C     == NVIB   : Write VIB[1,2].log file for vibrational analysis    ==
C     == NGXYZ  : Write xyz file every NGXYZ steps for geo. opt.      ==
C     == RCOMPB : Give index of read wavefunction compression scheme  ==
C     == WCOMPB : Give index of written wave. compression scheme      ==
C     == NDAVV  : Davidson parameter (n. of iteratation for one diag. ==
C     == N_FRIES: Lanczos para. (n. of iteratation for one diag.)     ==
C     == NKRY_MAX: Lanczos para.(Krylov subspace dimension)           ==
C     == NKRY_BLOCK: Lanczos para.(Block dimension)                   ==
C     == NPDIP:   Store dipole moments each NPDIP steps               ==
C     == IFTYPE : Interface to classical MD program (EGO)             ==
C     == ICMET  : Interface to classical MD program                   ==
C     == NRESET : WF steps between ODIIS resets on poor progress      ==
C     == NSTCNV : WF steps until WF convergence criteria are relaxed  ==
C     == NPERID : WF steps until WF is diagonalized                   ==
C     == NRPERI : DIIS resets until periodic WF diagonalization       ==
C     == MDIIS_FR: Maximum # of vectors retained for DIIS (rho fix)   ==
C     == MAXLDIIS: Maximum # of cycles at fixed rho                   ==
C     == MINLDIIS: Minimum # of cycles at fixed rho                   ==
C     == NSKIP: # of configurations to skip from TRAJSAVED            ==
C     == NSAMPLE: # of conf. to sample TRAJSAVED                      ==
C     == MEXTRA : Extrapolation order for BOMD                        ==
C     == NASPC  : Number of corrector steps when using ASPC           ==
C     == NSTBLK : CPU block size for dist. linalg                     ==
C     == IPRNG   : seed for pseudo-random number generator            ==
C     ==--------------------------------------------------------------==
      INTEGER      CNTI1,NOMORE,NOMORE_ITER,MAXIT,NPS,MDIIS,INSYS,
     *             MGDIIS,NPARA,NCHP,NCHB,NCHS,NCALLS0,NIT0,
     *             KSSTA,NKSSTA,IMOVIE,IPROJ,IVDBRD,IVDBWR,
     *             NTGAUS,INWFUN,NRESTF,NTRANS,NTRAJ,NPRES,NSPLP,
     *             NSORDER,NVIB,NGXYZ,RCOMPB,WCOMPB,NDAVV,N_FRIES,
     *             NKRY_MAX,NKRY_BLOCK,NPDIP,IFTYPE,ICMET,NPREC,
     *             NRESET,NSTCNV,MDIIS_FR,MAXLDIIS,MINLDIIS,
     *             NPERID,NRPERI,NSKIP,NSAMPLE,MEXTRA,NASPC,NSTBLK,
     *             IPRNG,LFIT
      COMMON/CNTI/ CNTI1,NOMORE,NOMORE_ITER,MAXIT,NPS,MDIIS,INSYS,
     *             MGDIIS,NPARA,NCHP,NCHB,NCHS,NCALLS0,NIT0,
     *             KSSTA,NKSSTA,IMOVIE,IPROJ,IVDBRD,IVDBWR,
     *             NTGAUS,INWFUN,NRESTF,NTRANS,NTRAJ,NPRES,NSPLP,
     *             NSORDER,NVIB,NGXYZ,RCOMPB,WCOMPB,NDAVV,N_FRIES,
     *             NKRY_MAX,NKRY_BLOCK,NPDIP,IFTYPE,ICMET,NPREC,
     *             NRESET,NSTCNV,MDIIS_FR,MAXLDIIS,MINLDIIS,
     *             NPERID,NRPERI,NSKIP,NSAMPLE,MEXTRA,NASPC,NSTBLK,
     *             IPRNG,LFIT
C     ==================================================================
C     == CNTR1  : Use for the broadcast (dummy variable)              ==
C     == DELT_ELEC : Time step for electrons                          ==
C     == DELT_IONS : Time step for ions                               ==
C     == EMASS  : Electronic mass (Car-Parrinello)                    ==
C     == EPSOG  : Parameters for the RATTLE step                      ==
C     == EKINW  : Electron dynamics with rescaling of velocities      ==
C     ==          Average kinetic energy                              ==
C     == TOLL   : Tolerance (electron dynamics)                       ==
C     == AMPRE  : Randomize wavefunction parameter                    ==
C     == TEMPW  : Ion dynamics with rescaling of velocities           ==
C     ==          desired temperature                                 ==
C     == TOLP   : Tolerance (ion dynamics)                            ==
C     == TAUBP  : Berendsen time constant for ions                    ==
C     == TAUBE  : Berendsen time constant for electrons               ==
C     == TAUBC  : Berendsen time constant for cell                    ==
C     == TRAMPT : Temperature ramping target value                    ==
C     == TRAMPR : Temperature ramping rate (in kelvin / a.u.)         ==
C     == ANNERI : Simulated annealing parameter (ions)                ==
C     == ANNERE : Simulated annealing parameter (electrons)           ==
C     == ANNERC : Simulated annealing parameter (cell)                ==
C     == DAMPGI : Damped dynamics friction parameter (ions)           ==
C     == DAMPGE : Damped dynamics friction parameter (electrons)      ==
C     == DAMPGC : Damped dyanmics friction parameter (cell)           ==
C     == HTHRS  : Hamiltonian cutoff                                  ==
C     == HTHRS  : Hamiltonian cutoff in DFTP                          ==
C     == TOLOG  : Convergence orbital tolerance                       ==
C     == TOLNG  : Convergence geometry tolerance                      ==
C     == TOLAD  : Adaptive convergence geometry tolerance: ratio      ==
C     == TOLENE : Adaptive convergence energy step tolerance: ratio   ==
C     == TOLFOR : Relaxed convergence criteria for force calculation  ==
C     == TOLINI : Initial convergence criterion for orbitals          ==
C     == TOLDETOT : Convergence orbital criteria based on total energy==
C     == ECUT   : Cutoff energy                                       ==
C     == SDELTA : Smooth density parameter                            ==
C     == SMF    : ''      ''      ''                                  ==
C     == GCEPS  : Gradient correction cutoff                          ==
C     == WNOSE0 : Characteristic frequency (electrons -- Nose)        ==
C     == WNOSP0 : Characteristic frequency (ions -- Nose)             ==
C     == EPSDAV : Davidson parameter                                  ==
C     == AMPRP  : Randomize ionic positions parameter                 ==
C     == FDIFF  : Finite difference step                              ==
C     == CMASS  : Fictitious MD cell mass                             ==
C     == AMPRC  : Randomize initial cell parameters, amplitude        ==
C     == TEMPC  : Target cell temperature(Kelvin -- Nose)             ==
C     == WNOSC0 : Characteristic frequency (cell -- Nose)             ==
C     == B2LIMIT: Lanczos parameter                                   ==
C     == EKINHR : Cell dynamics with rescaling of velocities          ==
C     ==          Average kinetic energy                              ==
C     == TOLC   : Tolerance (cell dynamics)                           ==
C     == TOLCG  : Convergence cell tolerance                          ==
C     == NEDOF0 : Scaling for elec. DOF (Nose)                        ==
C     == DSHIFT : Shift of potential in Davidson diagonalisation      ==
C     == TOLRHOFIX: Convergence tolerance when diis with fix rho      ==
C     == MEMSIZE: Memory reserved for real space wavefunction         ==
C     == ASIC   : alpha parameter for Hartree SIC correction          ==
C     == BSIC   : beta parameter for XC SIC correction                ==
C     == NOSPT0 : Temperature at which Nose velocities are initialized==
C     == GPOT,GENL,GFREQ: gauge potential in ehrenfest dynamics       ==
C     ==--------------------------------------------------------------==
      REAL*8       CNTR1,DELT_ELEC,DELT_IONS,EMASS,TRAMPT,TRAMPR,
     *             EPSOG,EKINW,TOLL,AMPRE,TEMPW,TOLP,TAUBP,TAUBE,TAUBC,
     *             ANNERI,HTHRS,TOLOG,TOLNG,ECUT,SDELTA,SMF,GCEPS,
     *             WNOSE0,WNOSP0,EPSDAV,AMPRP,FDIFF,CMASS,AMPRC,TEMPC,
     *             WNOSC0,B2LIMIT,EKINHR,TOLC,ANNERC,TOLCG,ANNERE,
     *             NEDOF0,DSHIFT,TOLAD,TOLENE,TOLFOR,TOLRHOFIX,TOLINI,
     *             TOLDETOT,MEMSIZE,ASIC,BSIC,NOSPT0,DAMPGI,DAMPGE,
     *             DAMPGC,GPOT,GENL,GFREQ
      COMMON/CNTR/ CNTR1,DELT_ELEC,DELT_IONS,EMASS,TRAMPT,TRAMPR,
     *             EPSOG,EKINW,TOLL,AMPRE,TEMPW,TOLP,TAUBP,TAUBE,TAUBC,
     *             ANNERI,HTHRS,TOLOG,TOLNG,ECUT,SDELTA,SMF,GCEPS,
     *             WNOSE0,WNOSP0,EPSDAV,AMPRP,FDIFF,CMASS,AMPRC,TEMPC,
     *             WNOSC0,B2LIMIT,EKINHR,TOLC,ANNERC,TOLCG,ANNERE,
     *             NEDOF0,DSHIFT,TOLAD,TOLENE,TOLFOR,TOLRHOFIX,TOLINI,
     *             TOLDETOT,MEMSIZE,ASIC,BSIC,NOSPT0,DAMPGI,DAMPGE,
     *             DAMPGC,GPOT,GENL,GFREQ
C     ==================================================================
C     == FILEPATH  OPTION (OTHERWISE="./")                            ==
C     == DEFINITION AND COMMON BLOCK MOVED TO FILE fileopen.inc       ==
C     ==================================================================
C     == DUAL OPTION: SIZE OF THE DENSITY MESH VERSUS                 ==
C     ==              THE WAVEFUNCTION CUTOFF (DEFAULT=4)             ==
C     ==      DUAL=.TRUE. IF USE THIS OPTION                          ==
C     ==--------------------------------------------------------------==
      REAL*8        CDUAL
      LOGICAL       DUAL
      COMMON/DUAL00/CDUAL,DUAL
C     ==================================================================
      INTEGER MAXRF,NSTEPWR,NFNOW,NRCOPY
      PARAMETER (MAXRF=10)
      COMMON/RESTF/NSTEPWR(MAXRF),NFNOW,NRCOPY
C     ==================================================================
      INTEGER       NSSEL
      POINTER       (IP_NASSEL,NASSEL)
      INTEGER       NASSEL(*)
      COMMON/SSEL1/ NSSEL
      COMMON/SSEL2/ IP_NASSEL
C     ==================================================================
C     == Control to dump local potential
C     ==================================================================
      LOGICAL      TLPOT
      COMMON/LOCPOT2/ TLPOT
C     ==================================================================
