C     ==================================================================
      SUBROUTINE TAUOFR(C0,PSI,NSTATE)
C     ==--------------------------------------------------------------==
C     ==          COMPUTES   TAU = SUM_i |nabla PSI_i|^2              ==
C     ==--------------------------------------------------------------==
C     == WARNING: ALL WAVEFUNCTIONS C0 HAVE TO BE ORTHOGONAL          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'tauf.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'pslo.inc'
C     Arguments
      INTEGER    NSTATE
      COMPLEX*16 C0(NGW,NSTATE),PSI(MAXFFT)
C     Variables
      REAL*8     TAU(NNR1,NLSD),VTAU(NNR1,NLSD)
      INTEGER    IS,IS1,IS2,IG,ISUB
      INTEGER    IFIRST
      DATA       IFIRST/0/
C     ==--------------------------------------------------------------==
      CALL TISET('    TAUOFR',ISUB)
C     ==--------------------------------------------------------------==
      IF(TIVAN) CALL STOPGM("TAUOFR","USPP NOT IMPLEMENTED")
      IF(TLSE) CALL STOPGM("TAUOFR","LSE NOT IMPLEMENTED")
C     ==--------------------------------------------------------------==
      IF(IFIRST.EQ.0) THEN
        IFIRST=1
        CALL MEMORY(IP_TAU,NLSD*NNR1,"TAU")
        CALL MEMORY(IP_VTAU,NLSD*NNR1,"VTAU")
      ENDIF
      ITAUR=1
C     ==--------------------------------------------------------------==
C     Initialize
      CALL AZZERO(TAU,NLSD*NNR1)
C     Loop over the electronic states
      DO IS=1,NSTATE,2
        IS1=IS
        IS2=IS+1
C x direction
        CALL ZAZZERO(PSI,MAXFFT)
        CALL DPSISC(C0,PSI,1,IS1,IS2,NSTATE)
        CALL S_INVFFT(PSI)
        CALL TAUADD(PSI,TAU,IS1,IS2,NSTATE)
C y direction
        CALL ZAZZERO(PSI,MAXFFT)
        CALL DPSISC(C0,PSI,2,IS1,IS2,NSTATE)
        CALL S_INVFFT(PSI)
        CALL TAUADD(PSI,TAU,IS1,IS2,NSTATE)
C z direction
        CALL ZAZZERO(PSI,MAXFFT)
        CALL DPSISC(C0,PSI,3,IS1,IS2,NSTATE)
        CALL S_INVFFT(PSI)
        CALL TAUADD(PSI,TAU,IS1,IS2,NSTATE)
      ENDDO
C     ==--------------------------------------------------------------==
      CALL TIHALT('    TAUOFR',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE DPSISC(C0,PSI,K,IS1,IS2,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'fft.inc'
C     Arguments
      INTEGER    NSTATE,K,IS1,IS2
      COMPLEX*16 C0(NGW,NSTATE),PSI(MAXFFT)
C     Variables
      INTEGER    IG
C     ==--------------------------------------------------------------==
      IF(IS2.GT.NSTATE) THEN
C$OMP parallel do private(IG) shared(k,is1)
        DO IG=1,NGW
          PSI(NZHS(IG))=GK(K,IG)*C0(IG,IS1)
          PSI(INDZS(IG))=-GK(K,IG)*DCONJG(C0(IG,IS1))
        ENDDO
      ELSE
C$OMP parallel do private(IG) shared(k,is1,is2)
        DO IG=1,NGW
          PSI(NZHS(IG))=GK(K,IG)*(C0(IG,IS1)+UIMAG*C0(IG,IS2))
          PSI(INDZS(IG))=-GK(K,IG)*(DCONJG(C0(IG,IS1))+
     *                        UIMAG*DCONJG(C0(IG,IS2)))
        ENDDO
      ENDIF
      IF(GEQ0) PSI(NZHS(1))=DCMPLX(0.D0,0.D0)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE TAUADD(PSI,TAU,IS1,IS2,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'fft.inc'
C     Arguments
      INTEGER    NSTATE,IS1,IS2
      COMPLEX*16 PSI(MAXFFT)
      REAL*8     TAU(NNR1,NLSD)
C     Variables
      DIMENSION  F(*)
      INTEGER    IR,ISPIN1,ISPIN2
      REAL*8     COEF1,COEF2,R1,R2
C     ==--------------------------------------------------------------==
      COEF1=0.5D0*TPIBA2*F(IS1)/OMEGA
      IF(IS2.GT.NSTATE) THEN
        COEF2=0.D0
      ELSE
        COEF2=0.5D0*TPIBA2*F(IS2)/OMEGA
      ENDIF
      IF(TLSD) THEN
         ISPIN1=1
         ISPIN2=1
         IF(IS1.GT.NSUP) ISPIN1=2
         IF(IS2.GT.NSUP) ISPIN2=2
C$OMP parallel do private(IR,R1,R2)
         DO IR=1,NNR1
           R1=DIMAG(PSI(IR))
           R2=DREAL(PSI(IR))
           TAU(IR,ISPIN1)=TAU(IR,ISPIN1)+COEF1*R1*R1
           TAU(IR,ISPIN2)=TAU(IR,ISPIN2)+COEF2*R2*R2
         ENDDO
      ELSE
C$OMP parallel do private(IR,R1,R2)
         DO IR=1,NNR1
           R1=DIMAG(PSI(IR))
           R2=DREAL(PSI(IR))
           TAU(IR,1)=TAU(IR,1)+COEF1*R1*R1+COEF2*R2*R2
         ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
