C     ==================================================================
C     Time Dependent Propagation using Cayley interpolation
C     author: I. Tavernelli
C
C     ==================================================================
      subroutine td_cayley(c0,c2,rhoe,psi,sc0,scr,lscr,
     &           effpot,nstate,eigv,first,do_dipole)
C     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'   ! ngw,nhg,nnr1,ai,nri,kri,omega,tpiba2
      include 'ions.inc'   ! nsp,na,zv
      include 'cppt.inc'   ! nzh,indz,hg,twnl
      include 'coor.inc'   ! tau0,taup,fion
      include 'dipo.inc'   ! dmom
      include 'fft.inc'    ! fourier transforms
      include 'cnst.inc'   ! pi
      include 'elct.inc'   ! f
      include 'sfac.inc'   ! eigr
      include 'sgpp.inc'   ! hlsg
      include 'nlps.inc'   ! ngh
      include 'geq0.inc'   ! geq0
      include 'kpts.inc'   ! tkpnt
      include 'ener.inc'   ! csumg
      include 'td_input.inc'
      include 'spin.inc'
      include 'soft.inc'
c     include 'fint.inc'
C
C     -- Arguments
C
      complex*16 c0(ngwk,nstate),c2(ngwk,nstate),psi(*),sc0(*)
      real*8     rhoe(nnr1,*),eigv(nstate),effpot(nnr1,*),scr(*)
      integer    nstate,lscr
      logical    first,do_dipole
C
C     -- Variables
C
      complex*16 phi(ngwk,nstate),hphi(ngwk,nstate)
      complex*16 v(*),eirop(nhg)
      complex*16 cor(nstate,nstate),rhog(*),respo(*)
      dimension  f(nstate),eigr(ngw,nat),twnl(ngwk,nhxs,nsx)
      real*8     fnl(*),dfnl(*),dist,neigv(nstate),pos(nnr1)
      real*8     rhoe0(nnr1,nlsd),rhoemax,time(10)
      real*8     dmom0(3),norms(nstate),sfkt_num,sfkt,sfkt_rec,chbcff
      real*8     tinc,tmax,pot,dotp,ddot,dipox,work_time,elfield
      real*8     hs(nat),prev_dist,maxr,norm,rhoer(*)
      integer    lst,ir,ig,is,irec(100),ispin,msglen,ciclo
      integer    lscr1,lscr2,lscr3
      logical    debug,fexist,second,statusdummy
      character  tag*30
      character*30 f_dipole,f_gauge_i,f_gauge_l,f_el,f_time
      pointer    (ip_phi,phi),(ip_v,v),(ip_hs,hs),(ip_eirop,eirop)
      pointer    (ip_cor,cor),(ip_pos,pos),(ip_rhoe0,rhoe0)
      pointer    (ip_norms,norms),(ip_hphi,hphi)
      pointer    (ip_neigv,neigv),(ip_rhog,rhog)
      pointer    (ip_respo,respo),(ip_rhoer,rhoer)
C
      integer    icall
      data       icall/0/
      save       icall,ip_rhoer
      real*8     last_time
      real*8     gpot_l0,gpot_l1,fgpot_l0,fgpot_l1
      real*8     gpot_i0,gpot_i1,fgpot_i0,fgpot_i1
      data       gpot_l0/0.D0/
      data       gpot_l1/0.D0/
      data       gpot_i0/0.D0/
      data       gpot_i1/0.D0/
      data       last_time/ 0.D0/
      save       last_time,gpot_l0,gpot_l1,fgpot_l0,fgpot_l1
C
C     Meaning of the time variables:
C     work_time if icall.eq.0 then 
C                  gets its value from the file
C               else
C                  gets its value from last_time
C               endif
C               In case of ruku it can also assume the intermidiate value
C               work_time+tt2 after half of the step (first=.true.)
C     last_time if icall.eq.0 then
C                 gets its value from work_time
C               else
C                 gets its value from the one save after the previous execution
C               endif
C               In case of ruku it only get updated if second=.true.
C               it never takes the intermidiat value as work_time
C
C     ttime     is a global time variable linked to the file abstime in ehrenfest.F
C               it gets update in td_cayley
C               At present only used with td_extpot
C
C     tintrvll  is a time intervall (not an absolute time)
C               if (ruku and first) tintrvll = tt2
C               if (ruku and second) tintrvll = tt
C
      second=.not.first
      f_dipole="dipole.dat"
      f_gauge_i="gaugefield_ind.dat"
      f_gauge_l="gaugefield_laser.dat"
      f_el="electricfield.dat"
      f_time="time.dat"
C     
      call give_scr_vofrho(lscr1,tag)
      call give_scr_hpsi(lscr2,tag,nstate)
      call give_scr_rnlsm(lscr3,tag,nstate,.false.)
      if (lscr.lt.(max(lscr1,lscr2,lscr3))) then
        if (parent) print*, 'SCR ARRAY TOO SMALL in td_cayley.F'
      endif
      ispin=nlsd
C
      call memory(ip_phi,2*ngwk*nstate,'phi')
      call memory(ip_hphi,2*ngwk*nstate,'hphi')
      call memory(ip_hs,nat,'hs')
      call memory(ip_v,4*maxfft*nlsd,'v')
      call memory(ip_eirop,2*nhg,'eirop')
      call memory(ip_rhoe0,nnr1*nlsd,'rhoe0')
      call memory(ip_pos,nnr1,'position')
      call memory(ip_cor,2*nstate*nstate,'cor')
      call memory(ip_norms,nstate,'norms')
      call memory(ip_neigv,nstate,'neigv')
      if (tpointch) then
       call memory(ip_respo,2*nhg,'respo')
       call memory(ip_rhog,2*nhg,'rhog')
       if (icall.eq.0) call memory(ip_rhoer,nnr1,'rhoer')
      endif
C
      debug=.false.
C     ==--------------------------------------------------------------==
C     ==-  open perturbation field file                              -==
C     ==--------------------------------------------------------------==
C
      if (parent) then
       write(6,'(1X,A,1X,F15.10)') 'time intervall    :',tintrvll
       write(6,'(1X,A,1X,F15.10)') 'accuracy          :',epsil
       write(6,'(1X,A,1X,F15.10)') 'pert. amplitude   :',ampl
       write(6,'(1X,A,1X,I15)')    '# cycles          :',n_cycles
       if (tpspec) then
         if (pert_type.eq.1) then
           write(6,'(/,1X,A)') 
     &      'EXTERNAL PULSE (DIRAC) AT t=0'
         elseif (pert_type.eq.2)  then
           write(6,'(/,1X,A)') 
     &     'EXTERNAL CONSTANT (HEAVISIDE) FIELD STARTING FROM t=0'
         endif
         if (pertdir.eq.1) then
          write(6,'(1X,A)') 'Direction perturbing field: x'
         elseif(pertdir.eq.2) then
          write(6,'(1X,A)') 'Direction perturbing field: y'
         elseif(pertdir.eq.3) then
          write(6,'(1X,A)') 'Direction perturbing field: z'
         else
          write(6,'(1X,A)') 'Isotropic perturbation'
         endif
       endif
      endif
C
C     ==--------------------------------------------------------------==
C     ==- Initialize TD run                                          -==
C     ==--------------------------------------------------------------==
C  
C     Initialize input orbitals PHI
C     -----------------------------
C
      do lst=1,nstate
        call dcopy(2*ngwk,c0(1,lst),1,phi(1,lst),1)
        if (tpspec) call dcopy(2*ngwk,c0(1,lst),1,hphi(1,lst),1)
      enddo
C
      call pos_oper(pos,pertdir)
      call rhoofr_c(phi,rhoe,psi,scr,lscr,nstate)
      norm=csumg
      if (parent.and.debug) 
     &   write(6,'(1X,A,1X,F10.4)') 'norm of the density ',norm
C
      call dcopy(nnr1*nlsd,rhoe,1,rhoe0,1)
C
C     ==--------------------------------------------------------------==
C     ==- Initialize variables and files                             -==
C     ==--------------------------------------------------------------==
      call eh_initialize_data
     &   (rhoe,rhoer,rhog,v,psi,eirop,elfield,scr,
     &    lscr,nstate,norms,do_dipole,dmom0,work_time,last_time,
     &    gpot_i0,gpot_i1,gpot_l0,gpot_l1,ispin,icall,
     &    f_dipole,f_time,f_gauge_i,f_gauge_l,f_el)
C
      if (parent) then
        if (do_dipole) open(unit=50,file=f_dipole,status='unknown',
     &                      position='append')
        if (tpointch)  open(unit=53,file=f_time,status='unknown',
     &                      position='append')
        if (tgaugep)   open(unit=72,file=f_gauge_i,status='unknown',
     &                      position='append')
        if (tgaugef)   open(unit=73,file=f_gauge_l,status='unknown',
     &                      position='append')
        if (td_extpot)  open(unit=74,file=f_el,status='unknown',
     &                      position='append')
      endif
C
C     ==--------------------------------------------------------------==
C     ==- Perturbation of the KS orbitals                            -==
C     ==--------------------------------------------------------------==
C
      if ((tpspec.and.stextpot.and.(icall.eq.0)).or.
     &    (tmdeh.and.(stextpot.and.(pert_type.eq.1)).and.(icall.eq.0)) 
     &   ) then
C       The reference value of the dipole at time 0 is needed
C       as reference for the calculation of the delta dipoles.
C       We need to store this value to be used in case of restart!
        if (parent) then
           write(50,'(4D25.15)') 0.0D0,dmom(1),dmom(2),dmom(3)
        endif
        if (pert_type.eq.1) then
         if (parent) print*, 'Applying perturbation ... '
         call azzero(phi,2*ngwk*nstate)
         call kick(hphi,nstate,norms,pos,phi,psi)
         call rhoofr_c(phi,rhoe,psi,scr,lscr,nstate)
         norm=csumg
         call azzero(v,2*maxfft*nlsd)
         if (parent.and.debug)
     &      print*,  'Norm of the pert. density ',norm
!$OMP parallel do private(ir)
         do ir=1,nnr1
          v(ir)=dcmplx(rhoe(ir,1),0.D0)
         enddo
         call fwfft(v)
         call dipo(tau0,eirop,v)
         if (parent) then
          write(6,'(/,1x,a,3f12.6)') 'Dipole after perturbation',
     &       dmom(1),dmom(2),dmom(3)
         endif
        elseif (pert_type.eq.2) then
        endif
      endif 
C
      icall=icall+1
C
C     ==--------------------------------------------------------------==
C     ==- Initialize cycle                                           -==
C     ==--------------------------------------------------------------==
C
      if (ruku) then
       if (first) then
        if (parent) 
     &    write(6,'(1x,A)') 'Computes effective potential...'
        call rhoofr_c(phi,rhoe,psi,scr,lscr,nstate)
        norm=csumg
        if (td_extpot) then
          if (pi_pulse) then
            call dext_pipot(ttime,elfield)
          else
            call dext_pot(ttime,elfield,0)
          endif
        endif
        if (tpointch) call dext_ch(work_time)
        call vofrho(tau0,fion,rhoe,v,scr,lscr,.false.,.false.)
        do is=1,ispin
         call dcopy(nnr1,rhoe(1,is),1,effpot(1,is),1)
        enddo
        if (tgaugep) then
         fgpot_i0=gpot_i0
         fgpot_i1=gpot_i1
        endif       
       else
        if (parent) 
     &    write(6,'(1X,A)') 'Uses previous effective potential...'
       endif
      endif
C
      work_time=last_time
C     ----------------------------------------------------------------==
      do ciclo=1,n_cycles
C     ----------------------------------------------------------------==
C      
       if (.not.ruku) then
         call rhoofr_c(phi,rhoe,psi,scr,lscr,nstate)
         norm=csumg
         call vofrho(tau0,fion,rhoe,v,scr,lscr,.false.,.false.)
       endif
C
       if (tpspec.and.(pert_type.eq.2)) 
     &   call daxpy(nnr1,ampl,pos,1,rhoe,1)
C
       if (tgaugep) then
         do lst=1,nstate
            call dcopy(2*ngwk,phi(1,lst),1,c0(1,lst),1)
         enddo
       endif
C
       if (parent) write(6,'(/,A,/)')
     & ' Propagation using cayley algorithm'
        if (ruku) then
          call cayley_step(
     &        phi,hphi,psi,nstate,effpot,sc0,neigv,scr,lscr,first)
        else
           call cayley_step(
     &        phi,hphi,psi,nstate,rhoe,sc0,neigv,scr,lscr,first)
        endif
C
C     ==--------------------------------------------------------------==
C     ==-  abdate the absolute time variables                        -==
C     ==--------------------------------------------------------------==
C
       if (ruku) then
        work_time=work_time+tt2
C       if (first) do not update last_time
        if (second) last_time=work_time
       else
        work_time=work_time+tintrvll
        last_time=work_time
       endif
C
C     ==--------------------------------------------------------------==
C     ==-  calculation of the dipol                                  -==
C     ==--------------------------------------------------------------==
C
       if (do_dipole.and.second) then
         if (parent) 
     &    write(6,'(/,A)') ' Calculation of the dipole'
C
C        -- calculate the density 
          call azzero(rhoe,nnr1*nlsd)
          call rhoofr_c(phi,rhoe,psi,scr,lscr,nstate)
          norm=csumg
          call dipolox(pos,rhoe,dipox)
          if (parent) write(49,'(2D25.15)') work_time,dipox 
          call azzero(v,2*maxfft*nlsd)
          call azzero(eirop,2*nhg)
          call eicalc(scr,eirop)
          rhoemax=0.D0
!$OMP parallel do private(ir)
          do ir=1,nnr1
             v(ir)=dcmplx(rhoe(ir,1),0.D0)
             if (dabs(rhoe(ir,1)-rhoe0(ir,1)).gt.rhoemax) 
     &           rhoemax=dabs(rhoe(ir,1)-rhoe0(ir,1))
          enddo
          call glomax(1,rhoemax)
          call fwfft(v)
          call dipo(tau0,eirop,v)
          if (parent) then 
           write(50,'(4D25.15)') work_time,
     &         (dmom(1)-dmom0(1)),
     &         (dmom(2)-dmom0(2)),
     &         (dmom(3)-dmom0(3))
          endif
C          
          if (parent.and.debug) write(6,'(4D14.6)') 
     &        work_time,dmom(1),dmom(2),dmom(3)
       endif 
C
C     ==--------------------------------------------------------------==
C     ==-  update external potentials
C     ==--------------------------------------------------------------==
C
       if (.not.ruku) then

         ttime=ttime+tintrvll
         if (td_extpot) then
           if (pi_pulse) then
             call dext_pipot(ttime,elfield)
           else
             call dext_pot(ttime,elfield,1)
           endif
         endif
         if (tpointch) call dext_ch(work_time)
         if (tpointch) then
          call dyn_analysis(first,phi,rhoe,rhoer,rhog,psi,scr,
     &     lscr,nstate,respo,work_time)
         endif
         if (tcurr) 
     &      call currentoper(phi,rhoe,nstate,tau0,scr,lscr,psi,1)
         if (tgaugep) then
           do lst=1,nstate
            call dcopy(2*ngwk,phi(1,lst),1,c2(1,lst),1)
           enddo
           time(1)=tintrvll
c          call gaugepot_ind(gpot_i0,gpot_i1,1,time,c0,c2,scr,lscr)
         endif
         if (tgaugef) then
c          call gaugepot_laser(gpot_l0,gpot_l1,work_time)
         endif
         if (parent.and.tgaugep)
     &     write(72,'(3d20.12)') work_time,gpot_i0,gpot_i1
         if (parent.and.tgaugef) 
     &     write(73,'(3d20.12)') work_time,gpot_l0,gpot_l1
         if (parent.and.td_extpot) 
     &     write(74,'(2d20.12)') ttime,elfield
         if (parent.and.tpointch) 
     &     write(53,'(3d20.12)') work_time

       else

         if (tgaugep) then
           do lst=1,nstate
            call dcopy(2*ngwk,phi(1,lst),1,c2(1,lst),1)
           enddo
           time(1)=tintrvll
           if (second) then
             gpot_i0=fgpot_i0
             gpot_i1=fgpot_i1
           endif
c          call gaugepot_ind(gpot_i0,gpot_i1,1,time,c0,c2,scr,lscr)
         endif
         if (tgaugef) then
c          call gaugepot_laser(gpot_l0,gpot_l1,work_time)
         endif
         if (second) then
          if (parent.and.tgaugep) 
     &     write(72,'(3d20.12)') work_time,gpot_i0,gpot_i1
          if (parent.and.tgaugef)
     &     write(73,'(3d20.12)') work_time,gpot_l0,gpot_l1
          if (parent.and.tpointch)
     &     write(53,'(3d20.12)') work_time
         endif

       endif

C      if (tgaugep) gpot=gpot+gpot_i0
       if (tgaugep) gpot=gpot_i0
       if (tgaugef) gpot=gpot+gpot_l0
C
       if (.not.exsoft) call testex(exsoft)
       if (exsoft) goto 111

C     ----------------------------------------------------------------==
      enddo   ! n_cycles
C     ----------------------------------------------------------------==
 111  continue
C     ==--------------------------------------------------------------==
C     ==-  'THAT s ALL FOLKS                                         -==
C     ==--------------------------------------------------------------==
C
       do lst=1,nstate
        call dcopy(2*ngwk,phi(1,lst),1,c0(1,lst),1)
       enddo
C
       if (ruku.and.first) then
         call rhoofr_c(c0,rhoe,psi,scr,lscr,nstate)
         norm=csumg
         ttime=ttime+tt2
         if (td_extpot) then
           if (pi_pulse) then
             call dext_pipot(ttime,elfield)
           else
             call dext_pot(ttime,elfield,1)
           endif
         endif
         if (tpointch) call dext_ch(work_time)
         ttime=ttime+tt2
         call vofrho(tau0,fion,rhoe,v,scr,lscr,.false.,.false.)
         do is=1,ispin
          call dcopy(nnr1,rhoe(1,is),1,effpot(1,is),1)
         enddo
       elseif (ruku.and.second) then
         if (tpointch) then
          call dyn_analysis(first,c0,rhoe,rhoer,rhog,psi,scr,
     &     lscr,nstate,respo,work_time)
         endif
         if (tcurr) 
     &     call currentoper(c0,rhoe,nstate,tau0,scr,lscr,psi,1)
         if (parent.and.(td_extpot)) then
           write(74,'(2d20.12)') ttime,elfield
          endif
       endif
C
C      CALL TCORREL(C0,PHI,NSTATE,COR)
C      PRINT*,DREAL(COR(1,1)),DIMAG(COR(1,2))
C
C     ==--------------------------------------------------------------==
C     ==-  write phi to RESTART.CWF.1 file                           -==
C     ==--------------------------------------------------------------== 
      irec(9)=1
C     CALL ZHWWF(2,IREC,C0, CM, NSTATE,EIGV,TAU0,VELP,TAUI,NFI)
      tag="wavefunctions"
      call tmpwr_prop(c0,nstate,1,tag) 
      call my_sync(allgrp)
      
      if (parent) then
       if (do_dipole) close(50)
       if (tpointch)  close(53)
       if (tgaugep)   close(72)
       if (tgaugef)   close(73)
       if (td_extpot)  close(74)
      endif
C     ==--------------------------------------------------------------== 
      if (tpointch) then
         call freem(ip_rhog)
         call freem(ip_respo)
      endif
      call freem(ip_neigv)
      call freem(ip_norms)
      call freem(ip_cor)
      call freem(ip_pos)
      call freem(ip_rhoe0)
      call freem(ip_eirop)
      call freem(ip_v)
      call freem(ip_hs)
      call freem(ip_hphi)
      call freem(ip_phi)
C     ==--------------------------------------------------------------==
      return
      end
C     ==================================================================
      subroutine cayley_step(
     &        phi,hphi,psi,nstate,rhoe,sc0,norms,scr,lscr,first)
C     ==--------------------------------------------------------------== 
C
      implicit none
      include 'system.h'
      include 'cnst.inc'
      include 'coor.inc'
      include 'ions.inc'
      include 'fft.inc'
      include 'sfac.inc'
      include 'geq0.inc'
      include 'td_input.inc'
      include 'spin.inc'
C
C     Arguments:
      integer    nstate,lscr
      logical    first
      real*8     scr(*),rhoe(*),norms(*),fnl(*),dfnl(*)
      complex*16 phi(ngwk,*),hphi(ngwk,nstate),psi(*)
      complex*16 sc0(*)
C     Variables:
      integer    lst,ig,iter,itermax
      real*8     delta,deltamax,deltaold,tinc2,shift
      complex*16 phiold,zdotc
      complex*16 phi0(ngwk,*),hphi0(ngwk,*)
      pointer    (ip_phi0,phi0),(ip_hphi0,hphi0)
c
      call memory(ip_phi0,2*ngwk*nstate,'phi0')
      call memory(ip_hphi0,2*ngwk*nstate,'hphi0')
C
      itermax=1
C
      tinc2=tintrvll/2.0D0
!$OMP parallel do private(lst,ig)
      do lst=1,nstate
        do ig=1,ngwk
           phi0(ig,lst)=phi(ig,lst)
        enddo
      enddo
C
      call hpsi(phi0,hphi0,sc0,rhoe,psi,scr,lscr,nstate,1,nlsd)
      call dscal(2*ngwk*nstate,-1.0D0,hphi0(1,1),1)
C
      iter=0
      deltamax=1.D0
      niterations=0
C     ---------------------------------------------------------------
      if (ruku.and.first) then
        if(parent) write(6,'(2X,A50)')
     & '#iter           dt/4       max.grad      precision'
      else
        if(parent) write(6,'(2X,A50)')
     & '#iter           dt/2       max.grad      precision'
      endif
 700  continue
      niterations=niterations+1
      deltaold=deltamax
      call hpsi(phi,hphi,sc0,rhoe,psi,scr,lscr,nstate,1,nlsd)
      call dscal(2*ngwk*nstate,-1.0D0,hphi(1,1),1)

      deltamax=0.D0
      do lst=1,nstate
        do ig=1,ngwk
          phiold=phi(ig,lst)

          phi(ig,lst)=phi0(ig,lst)
     &            - uimag * tinc2 * hphi0(ig,lst)/emass
     &            - uimag * tinc2 * hphi(ig,lst)/emass

           delta=abs(phiold-phi(ig,lst))
           if (delta.gt.deltamax) deltamax=delta
        enddo
      enddo
      call glomax(1,deltamax)
      iter=iter+1
      if(parent) 
     & write(6,'(2X,I5,3(E15.6))') iter,tinc2,deltamax,epsil
C     if ((deltamax.gt.epsil).and.(deltamax.le.deltaold)) goto 700
      if (deltamax.gt.epsil) goto 700
C     ---------------------------------------------------------------

      if (geq0) call zclean_k(phi,nstate,ngw)
      do lst=1,nstate
         norms(lst)=dreal(zdotc(ngwk,phi(1,lst),1,phi(1,lst),1))
#ifdef PARALLEL
         call glosum(1,norms(lst))
#endif
      enddo
C
      do lst=1,nstate
        call dscal(2*ngwk,1.D0/sqrt(norms(lst)),phi(1,lst),1)
      enddo

      if (iter.gt.itermax) itermax=iter
C
      if (parent)
     &   write(6,*) '  -- cayley max iteration steps ',itermax
      call freem(ip_hphi0)
      call freem(ip_phi0)
C
C     ==--------------------------------------------------------------==
      return
      end
C     ==================================================================
