C     ==================================================================
      SUBROUTINE TD_DAV(C0,C1,C2,SC0,DDXC,PSI,EIGV,DRHOE,
     &                  TEIG,SCR,LSCR,NSTATE,NROOT,ORBITAL,KPRINT)
C     ==--------------------------------------------------------------==
C     ==            Davidson diagonalization of TDDFT matrix          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'
      INCLUDE 'tpar.inc'
      INCLUDE 'soft.inc'
      INCLUDE 'norm.inc'
      INCLUDE 'ropt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'linres.inc'
C     Arguments
      INTEGER    LSCR,NSTATE,NROOT,KPRINT
      COMPLEX*16 C0(NGW,*),C1(NGW,NSTATE,*),C2(NGW,*),SC0(*)
      COMPLEX*16 PSI(*)
      REAL*8     EIGV(*),SCR(LSCR),DRHOE(*),DDXC(*),TEIG(*)
      CHARACTER*(*) ORBITAL
C     Variables
      LOGICAL    TRDIIS,TZERO
      INTEGER    MROOT,ND
      PARAMETER  (MROOT=20,ND=5*MROOT)
      DIMENSION  F(*)
      POINTER    (IP_CS,CS),(IP_CB,CB)
      COMPLEX*16 CS(NGW,NSTATE,*),CB(NGW,NSTATE,*)
      POINTER    (IP_AMAT,AMAT),(IP_DEIG,DEIG),(IP_VPP,VPP)
      POINTER    (IP_SOBACK,SOBACK)
      REAL*8     VPP(NGW),AMAT(ND,*),DEIG(*),SOBACK(*)
      CHARACTER  TAG*30
      REAL*8  E2(5),SO,DOTPS,SIG,SOI(MROOT),TIMEF
      EXTERNAL DOTPS,TIMEF
      INTEGER LTD_DAV,INFO,NDMAX,NCONV,INDEXR(ND),NZERO
      INTEGER I,J,K,INEW,ISTEP,IS,IB,NDIM,IG,NDD,ID,NSIGMA,NDOLD,NAMAT
      REAL*8  FEE,VP,SOMAX,TCPU,T1,T2,VPPMIN,DEIJ
#ifdef PARALLEL
      INTEGER MSGLEN
#endif
C     ==--------------------------------------------------------------==
      IF(.NOT.TDA) CALL STOPGM("TD_DAV",
     *                       "ONLY TAMM-DANCOFF APPROXIMATION ALLOWED")
      CALL GIVE_SCR_TD_DAV(LTD_DAV,TAG)
      CALL TEST_SCR('TD_DAV',TAG,LSCR,LTD_DAV)
C     ==--------------------------------------------------------------==
      IF(NROOT.GT.MROOT) CALL STOPGM("TD_DAV",
     *                               "MAX NUMBER OF ROOTS EXCEEDED")
      NDMAX = MAX(NROOT+1,NDAVSPACE)
      IF(NDMAX.GT.ND) CALL STOPGM("TD_DAV",
     *                               "MAX SUB SPACE SIZE EXCEEDED")
      CALL MEMORY(IP_CS,2*NGW*NSTATE*NDMAX,'CS')
      CALL MEMORY(IP_CB,2*NGW*NSTATE*NDMAX,'CB')
      CALL MEMORY(IP_AMAT,ND*NDMAX,'AMAT')
      CALL MEMORY(IP_DEIG,NDMAX,'DEIG')
      CALL AZZERO(AMAT,ND*NDMAX)
C     ==--------------------------------------------------------------==
      CALL MEMORY(IP_VPP,NGW,'VPP')
      CALL MEMORY(IP_SOBACK,NDAVMAX,'SOBACK')
C     ==--------------------------------------------------------------==
      IF ( PARENT ) THEN
        IF ( KPRINT.GE.0 ) THEN
          WRITE(6,'(1X,64("="))')
          WRITE(6,'(1X,"==",T13,A,T64,"==")')
     &       '   DAVIDSON DIAGONALISATION OF TDDFT MATRIX '
          WRITE(6,'(1X,64("="))')
        ENDIF
        IF ( KPRINT.EQ.0 ) THEN
          WRITE(6,'(A,T43,A)') 
     &   '  ITER      STATES      SUBSPACE','   RESIDUAL        TCPU'
        ENDIF
      ENDIF
C
      TRDIIS=.FALSE.
      PREC=.TRUE.
      CALL SETVPP(VPP,NGW,LR_HTHRS,VPPMIN)
C..initilize vectors
      CALL DCOPY(2*NGW*NSTATE*NROOT,C1,1,CB,1)
      DO I = 1, NROOT
        CALL LR_ORTHO(NSTATE,C0,CB(1,1,I),SCR,LSCR)
        DO J=1,I-1
          SO=DOTPS(NGW,NSTATE,CB(1,1,I),CB(1,1,J))
          CALL GLOSUM(1,SO)
          CALL DAXPY(2*NGW*NSTATE,-SO,CB(1,1,J),1,CB(1,1,I),1)
        ENDDO
        SO=DOTPS(NGW,NSTATE,CB(1,1,I),CB(1,1,I))
        CALL GLOSUM(1,SO)
        SO=1.D0/DSQRT(SO)
        CALL DSCAL(2*NGW*NSTATE,SO,CB(1,1,I),1)
      ENDDO
      NZERO=0
      NDIM=NROOT
      NSIGMA=0
      NCONV=0
      DO ISTEP=1,NDAVMAX
        T1=TIMEF()
1100    CONTINUE
C..initialize sigma vectors
        DO I = NSIGMA+1,NDIM
          CALL LR_FORCE(C0,CB(1,1,I),CS(1,1,I),SC0,E2,
     &                DDXC,PSI,DRHOE,EIGV,
     &                SCR,LSCR,NSTATE,"TDA",ORBITAL)
          CALL DSCAL(2*NGW*NSTATE,-1.D0,CS(1,1,I),1)
          CALL LR_ORTHO(NSTATE,C0,CS(1,1,I),SCR,LSCR)
        ENDDO
C..Davidson matrix
        NAMAT=NDIM
        CALL AZZERO(AMAT,NDMAX*ND)
        DO I=1,NDIM
          DO J=I,NDIM
            AMAT(I,J)=DOTPS(NGW,NSTATE,CB(1,1,I),CS(1,1,J))
            AMAT(J,I)=AMAT(I,J)
          ENDDO
        ENDDO
        CALL GLOSUM(ND*NDIM,AMAT)
        IF(PARENT) THEN
          CALL DSYEV('V','L',NDIM,AMAT,ND,DEIG,SCR,LSCR,INFO)
          IF(INFO.NE.0) CALL STOPGM('TD_DAV','INFO != 0 AFTER DSYEV')
        ENDIF
#ifdef PARALLEL
        MSGLEN = ND*NDIM * 8
        CALL MY_BCAST(AMAT,MSGLEN,SOURCE,ALLGRP)
        MSGLEN = NDIM * 8
        CALL MY_BCAST(DEIG,MSGLEN,SOURCE,ALLGRP)
#endif
C..Check for zero eigenvalues
        TZERO=.FALSE.
C..TSH[
        IF (.NOT.TDTULLY) THEN
         DO I=1,NROOT
           IF(DABS(DEIG(I))*2.D0*RY .LT. 0.01D0) THEN
             NZERO=NZERO+1
             TZERO=.TRUE.
           ENDIF
         ENDDO
        ENDIF
C..TSH]
        IF(TZERO) THEN
C..Remove zero eigenvectors
          IF(PARENT) WRITE(6,'(A)')
     &     ' WARNING| Remove zero eigenvalues, reinitialize states'
          CALL DAVRED(C0,C1,CB,CS,AMAT,NDIM,NROOT,NSTATE,ND)
          DO I=1,NROOT
            IF(DABS(DEIG(I))*2.D0*RY .LT. 0.01D0) THEN
              CALL RANDTOWF(C1(1,1,I),NSTATE,0,0)
              CALL LR_ORTHO(NSTATE,C0,C1(1,1,I),SCR,LSCR)
              DEIG(I)=I*1.D10
            ENDIF
          ENDDO
C         sort states, new states last
          CALL SORT2(DEIG,NROOT,INDEXR)
          DO I = 1, NROOT
            J=INDEXR(I)
            CALL DCOPY(2*NGW*NSTATE,C1(1,1,I),1,CB(1,1,J),1)
          ENDDO
C         orthogonalize
          DO I = 1, NROOT
            DO J=1,I-1
              SO=DOTPS(NGW,NSTATE,CB(1,1,I),CB(1,1,J))
              CALL GLOSUM(1,SO)
              CALL DAXPY(2*NGW*NSTATE,-SO,CB(1,1,J),1,CB(1,1,I),1)
            ENDDO
            SO=DOTPS(NGW,NSTATE,CB(1,1,I),CB(1,1,I))
            CALL GLOSUM(1,SO)
            SO=1.D0/DSQRT(SO)
            CALL DSCAL(2*NGW*NSTATE,SO,CB(1,1,I),1)
          ENDDO
          NDIM=NROOT
          NSIGMA=0
          NCONV=0
          GOTO 1100
        ELSEIF(NDIM+1.GT.NDMAX) THEN
C..Reduce vector subspace
          CALL DAVRED(C0,C1,CB,CS,AMAT,NDIM,NROOT,NSTATE,ND)
        ENDIF
C..Residuals
        NSIGMA=NDIM
        SOMAX=0.D0
        NDOLD=NDIM
        DO I = NCONV+1, NROOT
          CALL ZAZZERO(C2,NGW*NSTATE)
          DO J=1,NDIM
            CALL DAXPY(2*NGW*NSTATE,AMAT(J,I),CS(1,1,J),1,C2,1)
            FEE=-DEIG(I)*AMAT(J,I)
            CALL DAXPY(2*NGW*NSTATE,FEE,CB(1,1,J),1,C2,1)
          ENDDO
          SO=DOTPS(NGW,NSTATE,C2,C2)
          CALL GLOSUM(1,SO)
          SOI(I)=SO
C..Check for convergence
          IF(SO.LT.EPSTDAV.AND.I.EQ.NCONV+1) NCONV=NCONV+1
          IF(SO.GT.SOMAX) SOMAX=SO
          IF(SO.GT.EPSTDAV.AND.NDIM.LT.NDMAX) THEN
            NDIM=NDIM+1
            DO IS=1,NSTATE
              IF(INDEX(ORBITAL,"CANON").NE.0) THEN
                IB=IS
                SIG=-1.D0
              ELSE
                IB=(IS-1)*NSTATE+IS
                SIG=1.D0
              ENDIF
              DEIJ=SIG*EIGV(IB)-DEIG(I)
              DO IG=1,NGW
                CB(IG,IS,NDIM)=C2(IG,IS)/(VPP(IG)+DEIJ)
              ENDDO
            ENDDO
          ENDIF
        ENDDO
        T2=TIMEF()
        TCPU=(T2-T1)*0.001D0
        IF(PARENT) THEN
          IF(KPRINT.EQ.0) THEN
            WRITE(6,'(I6,7X,I5,9X,I5,T34,G20.8,T58,F8.2)')
     *             ISTEP,NCONV,NAMAT,SOMAX,TCPU
          ELSEIF(KPRINT.EQ.1) THEN
            WRITE(6,'(1X,64("-"))')
            WRITE(6,'(A,T20,I6,T34,A,T60,I6)') ' STEP  ',ISTEP,
     *        'NUMBER OF CONVERGED STATES   ',NCONV
            WRITE(6,'(A,T20,I6,T34,A,T54,G12.4)') 
     *        ' DAVIDSON MATRIX ',NAMAT,'MAXIMUM RESIDIUM  ',SOMAX
            WRITE(6,'(T34,A,T58,F8.1)') 'WALL TIME  ',TCPU
            DO I = 1, NCONV
              WRITE(6,'(A,T20,I6,T34,F14.6," eV",T54,G12.4)') 
     *           ' STATE ',I,DEIG(I)*RY*2.D0,SOI(I)
            ENDDO
            DO I = NCONV+1, NROOT
              WRITE(6,'(A,T20,I6," NC",T34,F14.6," eV",T54,G12.4)') 
     *           ' STATE ',I,DEIG(I)*RY*2.D0,SOI(I)
            ENDDO
          ENDIF
        ENDIF
        IF(.NOT.EXSOFT) CALL TESTEX(EXSOFT)
        IF(NCONV.EQ.NROOT .OR. ISTEP.EQ.NDAVMAX .OR. EXSOFT) GOTO 999
        SOBACK(ISTEP)=SOMAX
        IF(ISTEP.GT.4) THEN
          IF(SOMAX.LT.RDIISTIN) THEN
            IF(SOMAX.GT.0.9D0*SOBACK(ISTEP-4)) GOTO 999
          ENDIF
        ENDIF
C..orthogonalize
        DO I=NDOLD+1,NDIM
          CALL LR_ORTHO(NSTATE,C0,CB(1,1,I),SCR,LSCR)
          DO J=1,I-1
            SO=DOTPS(NGW,NSTATE,CB(1,1,I),CB(1,1,J))
            CALL GLOSUM(1,SO)
            CALL DAXPY(2*NGW*NSTATE,-SO,CB(1,1,J),1,CB(1,1,I),1)
          ENDDO
          SO=DOTPS(NGW,NSTATE,CB(1,1,I),CB(1,1,I))
          CALL GLOSUM(1,SO)
          CALL DSCAL(2*NGW*NSTATE,1.D0/DSQRT(SO),CB(1,1,I),1)
        ENDDO
      ENDDO
  999 CONTINUE
      IF(NCONV.LT.NROOT) TRDIIS=.TRUE.
C     ==--------------------------------------------------------------==
C..Davidson matrix
      CALL AZZERO(AMAT,NDMAX*ND)
      NDIM=MIN(NDIM,NSIGMA)
      DO I=1,NDIM
        DO J=I,NDIM
          AMAT(I,J)=DOTPS(NGW,NSTATE,CB(1,1,I),CS(1,1,J))
          AMAT(J,I)=AMAT(I,J)
        ENDDO
      ENDDO
      CALL GLOSUM(ND*NDIM,AMAT)
      IF(PARENT) THEN
        CALL DSYEV('V','L',NDIM,AMAT,ND,DEIG,SCR,LSCR,INFO)
        IF(INFO.NE.0) CALL STOPGM('TD_DAV','INFO != 0 AFTER DSYEV')
      ENDIF
#ifdef PARALLEL
      MSGLEN = ND*NDIM * 8
      CALL MY_BCAST(AMAT,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = NDIM * 8
      CALL MY_BCAST(DEIG,MSGLEN,SOURCE,ALLGRP)
#endif
      CALL AZZERO(C1,2*NGW*NSTATE*NROOT)
      DO I=1,NROOT
        DO J=1,NDIM
          CALL DAXPY(2*NGW*NSTATE,AMAT(J,I),CB(1,1,J),1,C1(1,1,I),1)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      IF(EXSOFT) TRDIIS=.FALSE.
      IF(TRDIIS) THEN
        J=NCONV+1
        DO I=J,NROOT
          DO K=1,I-1
            SO=DOTPS(NGW,NSTATE,C1(1,1,I),C1(1,1,K))
            CALL GLOSUM(1,SO)
            CALL DAXPY(2*NGW*NSTATE,-SO,C1(1,1,K),1,C1(1,1,I),1)
          ENDDO
          SO=DOTPS(NGW,NSTATE,C1(1,1,I),C1(1,1,I))
          CALL GLOSUM(1,SO)
          SO=1.D0/DSQRT(SO)
          CALL DSCAL(2*NGW*NSTATE,SO,C1(1,1,I),1)
          CALL TD_RDIIS(C0,C1,C2,SC0,CB,CS,I,NDMAX,NCONV,
     &                  VPP,DDXC,PSI,EIGV,DRHOE,DEIG(I),
     &                  SCR,LSCR,NSTATE,ORBITAL,KPRINT)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      CALL DCOPY (NROOT,DEIG,1,TEIG,1)
C     ==--------------------------------------------------------------==
      IF(PARENT.AND.NCONV.LT.NROOT) THEN
        WRITE(6,'(1X,64("!"))')
        WRITE(6,'(" !!",A,T64,"!!")')
     &       ' DAVIDSON| NOT ALL ROOTS ARE CONVERGED'
        WRITE(6,'(1X,64("!"))')
      ENDIF
C     ==--------------------------------------------------------------==
      CALL FREEM(IP_SOBACK)
      CALL FREEM(IP_VPP)
      CALL FREEM(IP_CS)
      CALL FREEM(IP_CB)
      CALL FREEM(IP_AMAT)
      CALL FREEM(IP_DEIG)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_TD_DAV(LTD_DAV,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'
      INCLUDE 'spin.inc'
C     Arguments
      INTEGER      LTD_DAV
      CHARACTER*30 TAG
C     Variables
      INTEGER LLR_FORCE,LLR_ORTHO
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_LR_FORCE(LLR_FORCE,"TDA",TAG)
      CALL GIVE_SCR_LR_ORTHO(LLR_ORTHO,N,TAG)
      LTD_DAV=MAX(LLR_FORCE,LLR_ORTHO)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION DOTPS(NGW,NSTATE,CA,CB)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      REAL*8     DOTPS
      INTEGER    NGW,NSTATE
      COMPLEX*16 CA(NGW,NSTATE),CB(NGW,NSTATE)
C     Variables
      INTEGER I
      REAL*8  DOTP
      EXTERNAL DOTP
C     ==--------------------------------------------------------------==
      DOTPS=0.D0
      DO I=1,NSTATE
        DOTPS=DOTPS+DOTP(NGW,CA(1,I),CB(1,I))
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE TD_SUB(C0,C1,C2,SC0,DDXC,PSI,EIGV,DRHOE,
     &                  TEIG,SCR,LSCR,MSUB,NSTATE,NROOT,ORBITAL,KPRINT)
C     ==--------------------------------------------------------------==
C     ==               updates the wavefunctions                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'
      INCLUDE 'tpar.inc'
      INCLUDE 'soft.inc'
      INCLUDE 'norm.inc'
      INCLUDE 'ropt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'linres.inc'
C     Arguments
      INTEGER    LSCR,NSTATE,NROOT,MSUB,KPRINT
      COMPLEX*16 C0(NGW,*),C1(NGW,MSUB,*),C2(NGW,*),SC0(*)
      COMPLEX*16 PSI(*)
      REAL*8     EIGV(*),SCR(LSCR),DRHOE(*),DDXC(*),TEIG(*)
      CHARACTER*(*) ORBITAL
C     Variables
      INTEGER    MROOT,ND
      PARAMETER  (MROOT=20,ND=5*MROOT)
      DIMENSION  F(*)
      POINTER    (IP_CS,CS),(IP_CB,CB)
      COMPLEX*16 CS(NGW,MSUB,*),CB(NGW,MSUB,*)
      POINTER    (IP_AMAT,AMAT),(IP_DEIG,DEIG),(IP_VPP,VPP)
      REAL*8     VPP(NGW),AMAT(ND,*),DEIG(*)
      CHARACTER  TAG*30
      REAL*8  E2(5),SO,DOTPS
      INTEGER LTD_DAV,INFO,NDMAX,NCONV,IB
      INTEGER I,J,INEW,ISTEP,IS,NDIM,IG,NDD,ID,NSIGMA,NDOLD,NAMAT
      REAL*8  FEE,VP,SOMAX,TCPU,T1,T2,TIMEF,SIG
      EXTERNAL TIMEF,DOTPS
#ifdef PARALLEL
      INTEGER MSGLEN
#endif
C     ==--------------------------------------------------------------==
      IF(.NOT.TDA) CALL STOPGM("TD_SUB",
     *                       "ONLY TAMM-DANCOFF APPROXIMATION ALLOWED")
      CALL GIVE_SCR_TD_DAV(LTD_DAV,TAG)
      CALL TEST_SCR('TD_DAV',TAG,LSCR,LTD_DAV)
C     ==--------------------------------------------------------------==
      IF(NROOT.GT.MROOT) CALL STOPGM("TD_SUB",
     *                               "MAX NUMBER OF ROOTS EXCEEDED")
      NDMAX = MAX(NROOT+1,NDAVSPACE)
      IF(NDMAX.GT.ND) CALL STOPGM("TD_SUB",
     *                               "MAX SUB SPACE SIZE EXCEEDED")
      CALL MEMORY(IP_VPP,NGW,'VPP')
      CALL MEMORY(IP_CS,2*NGW*MSUB*NDMAX,'CS')
      CALL MEMORY(IP_CB,2*NGW*MSUB*NDMAX,'CB')
      CALL MEMORY(IP_AMAT,ND*NDMAX,'AMAT')
      CALL MEMORY(IP_DEIG,NDMAX,'DEIG')
      CALL AZZERO(AMAT,ND*NDMAX)
C     ==--------------------------------------------------------------==
      PREC=.TRUE.
      CALL KSDIAG(VPP)
!$OMP parallel do private(IG,VP)
#ifdef __SR11000
*poption parallel, tlocal(IG,VP)
#endif
      DO IG=1,NGW
        VP=VPP(IG)
        VPP(IG)=MAX(VP,LR_HTHRS)
      ENDDO
C     ==--------------------------------------------------------------==
      IF ( PARENT .AND. KPRINT.GE.0 ) THEN
        WRITE(6,'(1X,64("="))')
        WRITE(6,'(1X,"==",T13,A,T64,"==")')
     &       '   DAVIDSON DIAGONALISATION OF TDDFT MATRIX '
        WRITE(6,'(1X,"==",T6,A,T64,"==")')
     &   ' TAMM-DANCOFF SUBSPACE APPROXIMATION (CANONICAL ORBITALS) '
        WRITE(6,'(1X,64("="))')
        WRITE(6,'(A,T43,A)') 
     &   '  ITER      STATES      SUBSPACE','   RESIDUAL        TCPU'
      ENDIF
C..initilize vectors
      CALL DCOPY(2*NGW*MSUB*NROOT,C1,1,CB,1)
      DO I = 1, NROOT
        CALL LR_ORTHOS(NSTATE,MSUB,C0,CB(1,1,I),SCR,LSCR)
        SO=DOTPS(NGW,MSUB,CB(1,1,I),CB(1,1,I))
        CALL GLOSUM(1,SO)
        CALL DSCAL(2*NGW*MSUB,1.D0/DSQRT(SO),CB(1,1,I),1)
      ENDDO
      NDIM=NROOT
      NSIGMA=0
      NCONV=0
      DO ISTEP=1,NDAVMAX
        T1=TIMEF()
C..initialize sigma vectors
        IF(INDEX(ORBITAL,"CANON").NE.0) THEN
          ID=NSTATE-MSUB+1
          IB=NSTATE-MSUB+1
        ELSE
          ID=NSTATE-MSUB+1
          IB=1
        ENDIF
        DO I = NSIGMA+1,NDIM
          CALL LR_FORCE_SUB(C0(1,ID),CB(1,1,I),CS(1,1,I),SC0,E2,
     &                DDXC,PSI,DRHOE,EIGV(IB),
     &                SCR,LSCR,NSTATE,MSUB,"TDA",ORBITAL)
          CALL DSCAL(2*NGW*MSUB,-1.D0,CS(1,1,I),1)
          CALL LR_ORTHOS(NSTATE,MSUB,C0,CS(1,1,I),SCR,LSCR)
        ENDDO
C..Davidson matrix
        NAMAT=NDIM
        CALL AZZERO(AMAT,NDMAX*ND)
        DO I=1,NDIM
          DO J=I,NDIM
            AMAT(I,J)=DOTPS(NGW,MSUB,CB(1,1,I),CS(1,1,J))
            AMAT(J,I)=AMAT(I,J)
          ENDDO
        ENDDO
        CALL GLOSUM(ND*NDIM,AMAT)
        IF(PARENT) THEN
          CALL DSYEV('V','L',NDIM,AMAT,ND,DEIG,SCR,LSCR,INFO)
          IF(INFO.NE.0) CALL STOPGM('TD_DAV','INFO != 0 AFTER DSYEV')
        ENDIF
#ifdef PARALLEL
        MSGLEN = ND*NDIM * 8
        CALL MY_BCAST(AMAT,MSGLEN,SOURCE,ALLGRP)
        MSGLEN = NDIM * 8
        CALL MY_BCAST(DEIG,MSGLEN,SOURCE,ALLGRP)
#endif
        IF(NDIM+1.GT.NDMAX) THEN
C..reduce trial vectors
          CALL AZZERO(C1,2*NGW*MSUB*NROOT)
          DO I=1,NROOT
            DO J=1,NDIM
              CALL DAXPY(2*NGW*MSUB,AMAT(J,I),CB(1,1,J),1,C1(1,1,I),1)
            ENDDO
          ENDDO
          CALL AZZERO(CB,2*NGW*MSUB*NROOT)
          DO I=1,NROOT
            DO J=1,NDIM
              CALL DAXPY(2*NGW*MSUB,AMAT(J,I),CS(1,1,J),1,CB(1,1,I),1)
            ENDDO
          ENDDO
          CALL DCOPY(2*MSUB*NGW*NROOT,CB,1,CS,1)
          CALL DCOPY(2*MSUB*NGW*NROOT,C1,1,CB,1)
          NDIM=NROOT
          CALL AZZERO(AMAT,NDMAX*ND)
          DO I=1,NROOT
            AMAT(I,I)=1.D0
          ENDDO
        ENDIF
C..Residuals
        NSIGMA=NDIM
        SOMAX=0.D0
        NDOLD=NDIM
        DO I = NCONV+1, NROOT
          CALL ZAZZERO(C2,NGW*MSUB)
          DO J=1,NDIM
            CALL DAXPY(2*NGW*MSUB,AMAT(J,I),CS(1,1,J),1,C2,1)
            FEE=-DEIG(I)*AMAT(J,I)
            CALL DAXPY(2*NGW*MSUB,FEE,CB(1,1,J),1,C2,1)
          ENDDO
          SO=DOTPS(NGW,MSUB,C2,C2)
          CALL GLOSUM(1,SO)
C..Check for convergence
          IF(SO.LT.EPSTDAV) NCONV=NCONV+1
          IF(SO.GT.SOMAX) SOMAX=SO
          IF(SO.GT.EPSTDAV.AND.NDIM.LT.NDMAX) THEN
            NDIM=NDIM+1
            DO IS=1,MSUB
              IF(INDEX(ORBITAL,"CANON").NE.0) THEN
                ID=NSTATE-MSUB+IS
                SIG=-1.D0
              ELSE
                ID=(IS-1)*MSUB+IS
                SIG=1.D0
              ENDIF
              DO IG=1,NGW
                CB(IG,IS,NDIM)=C2(IG,IS)/(VPP(IG)+SIG*EIGV(ID)-DEIG(I))
              ENDDO
            ENDDO
          ENDIF
        ENDDO
        T2=TIMEF()
        TCPU=(T2-T1)*0.0001D0
        IF(PARENT.AND.KPRINT.GE.0) 
     *      WRITE(6,'(I6,7X,I5,9X,I5,T34,1PG20.8,T58,F8.2)')
     *             ISTEP,NCONV,NAMAT,SOMAX,TCPU
        IF(.NOT.EXSOFT) CALL TESTEX(EXSOFT)
        IF(NCONV.EQ.NROOT .OR. ISTEP.EQ.NDAVMAX .OR. EXSOFT) GOTO 999
C..orthogonalize
        DO I=NDOLD+1,NDIM
          CALL LR_ORTHOS(NSTATE,MSUB,C0,CB(1,1,I),SCR,LSCR)
          DO J=1,I-1
            SO=DOTPS(NGW,MSUB,CB(1,1,I),CB(1,1,J))
            CALL GLOSUM(1,SO)
            CALL DAXPY(2*NGW*MSUB,-SO,CB(1,1,J),1,CB(1,1,I),1)
          ENDDO
          SO=DOTPS(NGW,MSUB,CB(1,1,I),CB(1,1,I))
          CALL GLOSUM(1,SO)
          CALL DSCAL(2*NGW*MSUB,1.D0/DSQRT(SO),CB(1,1,I),1)
        ENDDO
      ENDDO
  999 CONTINUE
C     ==--------------------------------------------------------------==
C..Davidson matrix
      CALL AZZERO(AMAT,NDMAX*ND)
      NDIM=MIN(NDIM,NSIGMA)
      DO I=1,NDIM
        DO J=I,NDIM
          AMAT(I,J)=DOTPS(NGW,MSUB,CB(1,1,I),CS(1,1,J))
          AMAT(J,I)=AMAT(I,J)
        ENDDO
      ENDDO
      CALL GLOSUM(ND*NDIM,AMAT)
      IF(PARENT) THEN
        CALL DSYEV('V','L',NDIM,AMAT,ND,DEIG,SCR,LSCR,INFO)
        IF(INFO.NE.0) CALL STOPGM('TD_DAV','INFO != 0 AFTER DSYEV')
      ENDIF
#ifdef PARALLEL
      MSGLEN = ND*NDIM * 8
      CALL MY_BCAST(AMAT,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = NDIM * 8
      CALL MY_BCAST(DEIG,MSGLEN,SOURCE,ALLGRP)
#endif
      CALL AZZERO(C1,2*NGW*MSUB*NROOT)
      DO I=1,NROOT
        DO J=1,NDIM
          CALL DAXPY(2*NGW*MSUB,AMAT(J,I),CB(1,1,J),1,C1(1,1,I),1)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      CALL DCOPY (NROOT,DEIG,1,TEIG,1)
C     ==--------------------------------------------------------------==
      IF(PARENT.AND.NCONV.LT.NROOT) THEN
        WRITE(6,'(1X,64("!"))')
        WRITE(6,'(" !!",A,T64,"!!")')
     &       ' DAVIDSON| NOT ALL ROOTS ARE CONVERGED'
        WRITE(6,'(1X,64("!"))')
      ENDIF
C     ==--------------------------------------------------------------==
      CALL FREEM(IP_VPP)
      CALL FREEM(IP_CS)
      CALL FREEM(IP_CB)
      CALL FREEM(IP_AMAT)
      CALL FREEM(IP_DEIG)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE SETVPP(VPP,NGW,THRES,VPPMIN)
      IMPLICIT NONE
      INTEGER NGW
      REAL*8 VPP(NGW)
      REAL*8 VPPMIN,THRES
      INTEGER IG
      REAL*8 VP
C     ==--------------------------------------------------------------==
      CALL KSDIAG(VPP)
      VPPMIN=1.D9
      DO IG=1,NGW
        VP=VPP(IG)
        VPP(IG)=MAX(VP,THRES)
        IF(VPP(IG).LT.VPPMIN) VPPMIN=VPP(IG)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE DAVRED(C0,C1,CB,CS,AMAT,NDIM,NROOT,NSTATE,ND)
C     ==--------------------------------------------------------------==
C     ==            Reduce Davidson vectors to nroots                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INTEGER    NROOT,NSTATE,NDIM,ND
      COMPLEX*16 C0(NGW,*),C1(NGW,NSTATE,*)
      COMPLEX*16 CS(NGW,NSTATE,*),CB(NGW,NSTATE,*)
      REAL*8     AMAT(ND,*)
C
      INTEGER    I,J
C     ==--------------------------------------------------------------==
C..reduce trial vectors
      CALL AZZERO(C1,2*NGW*NSTATE*NROOT)
      DO I=1,NROOT
        DO J=1,NDIM
          CALL DAXPY(2*NGW*NSTATE,AMAT(J,I),CB(1,1,J),1,C1(1,1,I),1)
        ENDDO
      ENDDO
      CALL AZZERO(CB,2*NGW*NSTATE*NROOT)
      DO I=1,NROOT
        DO J=1,NDIM
          CALL DAXPY(2*NGW*NSTATE,AMAT(J,I),CS(1,1,J),1,CB(1,1,I),1)
        ENDDO
      ENDDO
      CALL DCOPY(2*NSTATE*NGW*NROOT,CB,1,CS,1)
      CALL DCOPY(2*NSTATE*NGW*NROOT,C1,1,CB,1)
      NDIM=NROOT
      CALL AZZERO(AMAT,NROOT*ND)
!$OMP parallel do private(I)
#ifdef __SR11000
*poption parallel, tlocal(I)
#endif
      DO I=1,NROOT
        AMAT(I,I)=1.D0
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
