C     ==================================================================
      SUBROUTINE TD_NHDAV(C0,C1,C2,SC0,DDXC,PSI,EIGV,DRHOE,
     &                  TEIG,SCR,LSCR,NSTATE,NROOT,ORBITAL,KPRINT)
C     ==--------------------------------------------------------------==
C     ==  Non-Hermitian Davidson diagonalisation                      ==
C     ==  Stratman et al. JCP 109 8218 (1998)                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'
      INCLUDE 'tpar.inc'
      INCLUDE 'soft.inc'
      INCLUDE 'norm.inc'
      INCLUDE 'ropt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'linres.inc'
C     Arguments
      INTEGER    LSCR,NSTATE,NROOT,KPRINT
      COMPLEX*16 C0(NGW,*),C1(NGW,NSTATE,NROOT,2),C2(NGW,*),SC0(*)
      COMPLEX*16 PSI(*)
      REAL*8     EIGV(*),SCR(LSCR),DRHOE(*),DDXC(*),TEIG(*)
      CHARACTER*(*) ORBITAL
C     Variables
      INTEGER    MROOT,ND
      PARAMETER  (MROOT=20,ND=5*MROOT)
      DIMENSION  F(*)
      POINTER    (IP_CSP,CSP),(IP_CSM,CSM),(IP_CB,CB), (IP_CR,CR)
      COMPLEX*16 CSP(NGW,NSTATE,*),CSM(NGW,NSTATE,*),CB(NGW,NSTATE,*),
     &           CR(NGW,NSTATE,*)
      POINTER    (IP_APMAT,APMAT),(IP_AMMAT,AMMAT),(IP_BMMAT,BMMAT),
     &           (IP_DEIG,DEIG),(IP_VPP,VPP),(IP_IND,IND)
      REAL*8     VPP(NGW),APMAT(ND,*),AMMAT(ND,*),BMMAT(ND,*),DEIG(*)
      CHARACTER  TAG*30,HAMB*5,HAPB*5
      REAL*8  E2(5),SO,DOTPS,TX
      INTEGER LTD_NHDAV,INFO,NDMAX,NCONV,IND(*),NTOGO
      INTEGER I,J,INEW,ISTEP,IS,NDIM,IG,NDD,ID,NSIGMA,NDOLD,NAMAT
      REAL*8  FEE,VP,SOMAX,TCPU,T1,T2,TIMEF,DASUM
      EXTERNAL DOTPS,DASUM,TIMEF
#ifdef PARALLEL
      INTEGER MSGLEN
#endif
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_TD_NHDAV(LTD_NHDAV,TAG)
      CALL TEST_SCR('TD_NHDAV',TAG,LSCR,LTD_NHDAV)
C     ==--------------------------------------------------------------==
      NDMAX = MAX(2*NROOT+2,NDAVSPACE)
      NDMAX = 2*((NDMAX+1)/2)
      IF(NROOT.GT.MROOT) CALL STOPGM("TD_NHDAV",
     *                               "MAX NUMBER OF ROOTS EXCEEDED")
      IF(NDMAX.GT.ND) CALL STOPGM("TD_NHDAV",
     *                               "MAX SUBSPACE SIZE EXCEEDED")
      IF(INDEX(ORBITAL,"CANON").EQ.0) CALL STOPGM("TD_DAV",
     *                               "ONLY CANONICAL ORBITALS ALLOWED")
      CALL MEMORY(IP_VPP,NGW,'VPP')
      CALL MEMORY(IP_CSP,2*NGW*NSTATE*NDMAX,'CSP')
      CALL MEMORY(IP_CSM,2*NGW*NSTATE*NDMAX,'CSM')
      CALL MEMORY(IP_CB,2*NGW*NSTATE*NDMAX,'CB')
      CALL MEMORY(IP_CR,2*NGW*NSTATE*2,'CR')
      CALL MEMORY(IP_APMAT,ND*NDMAX,'APMAT')
      CALL MEMORY(IP_AMMAT,ND*NDMAX,'AMMAT')
      CALL MEMORY(IP_BMMAT,ND*NDMAX,'BMMAT')
      CALL MEMORY(IP_DEIG,NDMAX,'DEIG')
      CALL MEMORY(IP_IND,NDMAX,'IND')
      CALL AZZERO(APMAT,ND*NDMAX)
      CALL AZZERO(AMMAT,ND*NDMAX)
      CALL AZZERO(BMMAT,ND*NDMAX)
C     ==--------------------------------------------------------------==
      IF(TDA) THEN
        HAMB="TDA"
        HAPB="TDA"
      ELSE
        HAMB="TDAMB"
        HAPB="TDAPB"
      ENDIF
      NTOGO=NDAVMAX
C     ==--------------------------------------------------------------==
      PREC=.TRUE.
      CALL KSDIAG(VPP)
      DO IG=1,NGW
        VP=VPP(IG)
        VPP(IG)=MAX(VP,LR_HTHRS)
      ENDDO
C     ==--------------------------------------------------------------==
      IF ( PARENT .AND. KPRINT.GE.0 ) THEN
        WRITE(6,'(1X,64("="))')
        WRITE(6,'(1X,"==",T6,A,T64,"==")')
     & ' NON-HERMITIAN DAVIDSON DIAGONALISATION OF TDDFT MATRIX '
        WRITE(6,'(1X,64("="))')
      ENDIF
C..initilize vectors
1000  CONTINUE
      T1=TIMEF()
      CALL DCOPY(2*NGW*NSTATE*NROOT,C1,1,CB,1)
      NDIM=NROOT
      IF(.NOT.TDA) THEN
        DO I = 1, NROOT
          NDIM=NDIM+1
          IF(PARENT.AND.NDIM.GT.NDMAX) THEN
            CALL STOPGM("TD_NHDAV","SUBSPACE TOO SMALL")
          ENDIF
          VP=DASUM(2*NGW*NSTATE,C1(1,1,I,2),1)
          CALL GLOSUM(1,VP)
          IF(VP.GT.1.D-10) THEN
C           Left eigenvector is available
            CALL DCOPY(2*NGW*NSTATE,C1(1,1,I,2),1,CB(1,1,NDIM),1)
          ELSE
C           Generate left eigenvector
            J=NDIM-NROOT
            CALL LR_ORTHO(NSTATE,C0,CB(1,1,J),SCR,LSCR)
            CALL LR_FORCE(C0,CB(1,1,J),CB(1,1,NDIM),SC0,E2,
     &                  DDXC,PSI,DRHOE,EIGV,
     &                  SCR,LSCR,NSTATE,HAPB,ORBITAL)
          ENDIF
        ENDDO
      ENDIF
      DO I = 1, NDIM
        CALL LR_ORTHO(NSTATE,C0,CB(1,1,I),SCR,LSCR)
        DO J=1,I-1
          SO=DOTPS(NGW,NSTATE,CB(1,1,I),CB(1,1,J))
          CALL GLOSUM(1,SO)
          CALL DAXPY(2*NGW*NSTATE,-SO,CB(1,1,J),1,CB(1,1,I),1)
        ENDDO
        SO=DOTPS(NGW,NSTATE,CB(1,1,I),CB(1,1,I))
        CALL GLOSUM(1,SO)
        CALL DSCAL(2*NGW*NSTATE,1.D0/DSQRT(SO),CB(1,1,I),1)
      ENDDO
      T2=TIMEF()
      IF ( PARENT  .AND. KPRINT.GE.0) THEN
        WRITE(6,'(A,T56,I10)') ' Number of states initialized ',NDIM 
        TCPU=(T2-T1)*0.001D0
        WRITE(6,'(A,T56,F10.2,/)') ' Time for initialization ',TCPU 
        WRITE(6,'(A,T43,A)') 
     &   '  ITER      STATES      SUBSPACE','   RESIDUAL        TCPU'
      ENDIF
      NSIGMA=0
      NCONV=0
      DO ISTEP=1,NTOGO
        T1=TIMEF()
C..initialize sigma vectors
        DO I = NSIGMA+1,NDIM
          CALL LR_FORCE(C0,CB(1,1,I),CSP(1,1,I),SC0,E2,
     *                DDXC,PSI,DRHOE,EIGV,
     *                SCR,LSCR,NSTATE,HAPB,ORBITAL)
          CALL DSCAL(2*NGW*NSTATE,-1.D0,CSP(1,1,I),1)
          CALL LR_ORTHO(NSTATE,C0,CSP(1,1,I),SCR,LSCR)
          CALL LR_FORCE(C0,CB(1,1,I),CSM(1,1,I),SC0,E2,
     *                DDXC,PSI,DRHOE,EIGV,
     *                SCR,LSCR,NSTATE,HAMB,ORBITAL)
          CALL DSCAL(2*NGW*NSTATE,-1.D0,CSM(1,1,I),1)
          CALL LR_ORTHO(NSTATE,C0,CSM(1,1,I),SCR,LSCR)
        ENDDO
C..Davidson matrices
        NAMAT=NDIM
        CALL AZZERO(APMAT,NDMAX*ND)
        CALL AZZERO(AMMAT,NDMAX*ND)
        DO I=1,NDIM
          DO J=I,NDIM
            APMAT(I,J)=DOTPS(NGW,NSTATE,CB(1,1,I),CSP(1,1,J))
            APMAT(J,I)=APMAT(I,J)
            AMMAT(I,J)=DOTPS(NGW,NSTATE,CB(1,1,I),CSM(1,1,J))
            AMMAT(J,I)=AMMAT(I,J)
          ENDDO
        ENDDO
        CALL GLOSUM(ND*NDIM,APMAT)
        CALL GLOSUM(ND*NDIM,AMMAT)
C..Calculate BMMAT = AMMAT * APMAT
        IF(PARENT) THEN
          CALL DGEMM('N','N',NDIM,NDIM,NDIM,1.D0,AMMAT,ND,APMAT,ND,
     *               0.D0,BMMAT,ND)
C..diagonalise
          CALL DGEEV('V','V',NDIM,BMMAT,ND,DEIG,SCR,AMMAT,ND,APMAT,
     *                ND,SCR(ND+1),LSCR,INFO)
          IF(INFO.NE.0) CALL STOPGM('TD_NHDAV','INFO DGEEV')
          TX=0.D0
          DO I=1,NDIM
            TX=TX+ABS(SCR(I))
          ENDDO
          IF(TX.GT.1.D-10) CALL STOPGM('TD_NHDAV',
     *                                 'EIGNEVALUES NOT REAL')
          CALL SORT2(DEIG,NDIM,IND)
          CALL DCOPY(ND*NDIM,AMMAT,1,BMMAT,1)
          DO I=1,NDIM
            CALL DCOPY(NDIM,BMMAT(1,IND(I)),1,AMMAT(1,I),1)
          ENDDO
          CALL DCOPY(ND*NDIM,APMAT,1,BMMAT,1)
          DO I=1,NDIM
            CALL DCOPY(NDIM,BMMAT(1,IND(I)),1,APMAT(1,I),1)
          ENDDO
          DO I=1,NDIM
            DEIG(I)=DSQRT(DEIG(I))
          ENDDO
        ENDIF
#ifdef PARALLEL
        MSGLEN = ND*NDIM * 8
        CALL MY_BCAST(APMAT,MSGLEN,SOURCE,ALLGRP)
        MSGLEN = ND*NDIM * 8
        CALL MY_BCAST(AMMAT,MSGLEN,SOURCE,ALLGRP)
        MSGLEN = NDIM * 8
        CALL MY_BCAST(DEIG,MSGLEN,SOURCE,ALLGRP)
#endif
        IF(NDIM+1.GT.NDMAX) THEN
C..reduce trial vectors
          CALL AZZERO(C1(1,1,1,1),2*NGW*NSTATE*NROOT)
          CALL AZZERO(C1(1,1,1,2),2*NGW*NSTATE*NROOT)
          DO I=1,NROOT
            DO J=1,NDIM
              CALL DAXPY(2*NGW*NSTATE,APMAT(J,I),CB(1,1,J),1,
     *                   C1(1,1,I,1),1)
              CALL DAXPY(2*NGW*NSTATE,AMMAT(J,I),CB(1,1,J),1,
     *                   C1(1,1,I,2),1)
            ENDDO
          ENDDO
          NTOGO=NTOGO-NDIM/2
          GOTO 1000
        ENDIF
C..Residuals
        NSIGMA=NDIM
        SOMAX=0.D0
        NDOLD=NDIM
        DO I = NCONV+1, NROOT
          CALL AZZERO(CR,2*2*NGW*NSTATE)
          DO J=1,NDIM
            CALL DAXPY(2*NGW*NSTATE,AMMAT(J,I),CSM(1,1,J),1,CR(1,1,1),1)
            FEE=-DEIG(I)*APMAT(J,I)
            CALL DAXPY(2*NGW*NSTATE,FEE,CB(1,1,J),1,CR(1,1,1),1)
            CALL DAXPY(2*NGW*NSTATE,APMAT(J,I),CSP(1,1,J),1,CR(1,1,2),1)
            FEE=-DEIG(I)*AMMAT(J,I)
            CALL DAXPY(2*NGW*NSTATE,FEE,CB(1,1,J),1,CR(1,1,2),1)
          ENDDO
          SO=DOTPS(NGW,NSTATE,CR(1,1,1),CR(1,1,1)) +
     *       DOTPS(NGW,NSTATE,CR(1,1,2),CR(1,1,2))
          CALL GLOSUM(1,SO)
C..Check for convergence
          IF(SO.LT.EPSTDAV) NCONV=NCONV+1
          IF(SO.GT.SOMAX) SOMAX=SO
          IF(TDA) THEN
            IF(SO.GT.EPSTDAV.AND.NDIM.LT.NDMAX) THEN
              NDIM=NDIM+1
              DO IS=1,NSTATE
                DO IG=1,NGW
                  TX=1.D0/(VPP(IG)-EIGV(IS)-DEIG(I))
                  CB(IG,IS,NDIM)=TX*CR(IG,IS,1)
                ENDDO
              ENDDO
            ENDIF
          ELSE
            IF(SO.GT.EPSTDAV.AND.NDIM.LT.NDMAX-1) THEN
              NDIM=NDIM+2
              DO IS=1,NSTATE
                DO IG=1,NGW
                  TX=1.D0/(VPP(IG)-EIGV(IS)-DEIG(I))
                  CB(IG,IS,NDIM-1)=TX*CR(IG,IS,1)
                  CB(IG,IS,NDIM)=TX*CR(IG,IS,2)
                ENDDO
              ENDDO
            ENDIF
          ENDIF
        ENDDO
        T2=TIMEF()
        TCPU=(T2-T1)*0.001D0
        IF(PARENT .AND. KPRINT.GE.0) 
     *      WRITE(6,'(I6,7X,I5,9X,I5,T34,G20.8,T58,F8.2)')
     *             ISTEP,NCONV,NAMAT,SOMAX,TCPU
        IF(.NOT.EXSOFT) CALL TESTEX(EXSOFT)
        IF(NCONV.EQ.NROOT .OR. ISTEP.EQ.NDAVMAX .OR. EXSOFT) GOTO 999
C..orthogonalize
        DO I=NDOLD+1,NDIM
          CALL LR_ORTHO(NSTATE,C0,CB(1,1,I),SCR,LSCR)
          DO J=1,I-1
            SO=DOTPS(NGW,NSTATE,CB(1,1,I),CB(1,1,J))
            CALL GLOSUM(1,SO)
            CALL DAXPY(2*NGW*NSTATE,-SO,CB(1,1,J),1,CB(1,1,I),1)
          ENDDO
          SO=DOTPS(NGW,NSTATE,CB(1,1,I),CB(1,1,I))
          CALL GLOSUM(1,SO)
          CALL DSCAL(2*NGW*NSTATE,1.D0/DSQRT(SO),CB(1,1,I),1)
        ENDDO
      ENDDO
  999 CONTINUE
C     ==--------------------------------------------------------------==
C..Davidson matrices
      CALL AZZERO(APMAT,NDMAX*ND)
      CALL AZZERO(AMMAT,NDMAX*ND)
      NDIM=MIN(NDIM,NSIGMA)
      DO I=1,NDIM
        DO J=I,NDIM
          APMAT(I,J)=DOTPS(NGW,NSTATE,CB(1,1,I),CSP(1,1,J))
          APMAT(J,I)=APMAT(I,J)
          AMMAT(I,J)=DOTPS(NGW,NSTATE,CB(1,1,I),CSM(1,1,J))
          AMMAT(J,I)=AMMAT(I,J)
        ENDDO
      ENDDO
      CALL GLOSUM(ND*NDIM,APMAT)
      CALL GLOSUM(ND*NDIM,AMMAT)
C..Calculate BMMAT = AMMAT * APMAT
      IF(PARENT) THEN
        CALL DGEMM('N','N',NDIM,NDIM,NDIM,1.D0,AMMAT,ND,APMAT,ND,
     *               0.D0,BMMAT,ND)
C..diagonalise
        CALL DGEEV('V','V',NDIM,BMMAT,ND,DEIG,SCR,AMMAT,ND,APMAT,
     *                ND,SCR(ND+1),LSCR,INFO)
        IF(INFO.NE.0) CALL STOPGM('TD_NHDAV','INFO DGEEV')
        TX=0.D0
        DO I=1,NDIM
          TX=TX+ABS(SCR(I))
        ENDDO
        IF(TX.GT.1.D-10) CALL STOPGM('TD_NHDAV',
     *                               'EIGNEVALUES NOT REAL')
        CALL SORT2(DEIG,NDIM,IND)
        CALL DCOPY(ND*NDIM,AMMAT,1,BMMAT,1)
        DO I=1,NDIM
          CALL DCOPY(NDIM,BMMAT(1,IND(I)),1,AMMAT(1,I),1)
        ENDDO
        CALL DCOPY(ND*NDIM,APMAT,1,BMMAT,1)
        DO I=1,NDIM
          CALL DCOPY(NDIM,BMMAT(1,IND(I)),1,APMAT(1,I),1)
        ENDDO
        DO I=1,NDIM
          DEIG(I)=DSQRT(DEIG(I))
        ENDDO
      ENDIF
#ifdef PARALLEL
      MSGLEN = ND*NDIM * 8
      CALL MY_BCAST(APMAT,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = ND*NDIM * 8
      CALL MY_BCAST(AMMAT,MSGLEN,SOURCE,ALLGRP)
      MSGLEN = NDIM * 8
      CALL MY_BCAST(DEIG,MSGLEN,SOURCE,ALLGRP)
#endif
      CALL AZZERO(C1(1,1,1,1),2*NGW*NSTATE*NROOT)
      CALL AZZERO(C1(1,1,1,2),2*NGW*NSTATE*NROOT)
      DO I=1,NROOT
        DO J=1,NDIM
          CALL DAXPY(2*NGW*NSTATE,APMAT(J,I),CB(1,1,J),1,
     *               C1(1,1,I,1),1)
          CALL DAXPY(2*NGW*NSTATE,AMMAT(J,I),CB(1,1,J),1,
     *               C1(1,1,I,2),1)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      CALL DCOPY (NROOT,DEIG,1,TEIG,1)
C     ==--------------------------------------------------------------==
      IF(PARENT.AND.NCONV.LT.NROOT) THEN
        WRITE(6,'(1X,64("!"))')
        WRITE(6,'(" !!",A,T64,"!!")')
     &       ' DAVIDSON| NOT ALL ROOTS ARE CONVERGED'
        WRITE(6,'(1X,64("!"))')
      ENDIF
C     ==--------------------------------------------------------------==
      CALL FREEM(IP_VPP)
      CALL FREEM(IP_CSP)
      CALL FREEM(IP_CSM)
      CALL FREEM(IP_CB)
      CALL FREEM(IP_CR)
      CALL FREEM(IP_APMAT)
      CALL FREEM(IP_AMMAT)
      CALL FREEM(IP_BMMAT)
      CALL FREEM(IP_DEIG)
      CALL FREEM(IP_IND)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_TD_NHDAV(LTD_NHDAV,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'elct.inc'
      INCLUDE 'spin.inc'
C     Arguments
      INTEGER      LTD_NHDAV
      CHARACTER*30 TAG
C     Variables
      INTEGER    MROOT,ND
      PARAMETER  (MROOT=20,ND=5*MROOT)
      INTEGER LLR_FORCE,LLR_ORTHO,LOCAL
C     ==--------------------------------------------------------------==
      LOCAL = ND*ND + ND
      CALL GIVE_SCR_LR_FORCE(LLR_FORCE,"TDA",TAG)
      CALL GIVE_SCR_LR_ORTHO(LLR_ORTHO,N,TAG)
      LTD_NHDAV=MAX(LOCAL,LLR_FORCE,LLR_ORTHO)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
