c     ======================================================================
      subroutine td_os_berry(c0,c1,scr,lscr,eigenvalue,nstate)
c     ======================================================================
      implicit none

      include 'system.h'
      include 'irat.inc'
      include 'rmas.inc'
      include 'cnst.inc'
      include 'gvec.inc'  
      include 'geq0.inc'
      include 'prop.inc'
      include 'metr.inc'
      include 'ddip.inc'

c     Input:
      integer    nstate                            !Number of KS/lr orbitals
      complex*16 c0(ngw,nstate)             !Ground state Kohn-Sham orbitals
      complex*16 c1(ngw,nstate)                    !Linear response orbitals
      integer    lscr                                  !Scratch space length
      real*8     scr(*)                                             !Scratch
      real*8     eigenvalue                         !TDDFT transition energy

c     Local:
      complex*16 cwork(ngw,nstate,2)                             !Work array
      complex*16 cw(ngw,nstate)                                  !Work array
      complex*16 work(*)                                         !Work array
      complex*16 sc0(*)                                             !Scratch
      complex*16 ddmat_p(nstate,nstate)     !Opeigr matrix (lately inverted)
      complex*16 ddmat_1(nstate,nstate)      !Opeigr matrix (never inverted)
      complex*16 f_raman(ngw,nstate,3)             !Generalised perturbation
      complex*16 dd
      integer    ipiv(*)                              !For matrix operations
      integer    isub,il_framan,il_cw,il_cw1
      integer    i,j,k,k2,ifirst,ig
#ifdef PARALLEL
      integer nxxfun
      external nxxfun
#endif
      integer    nxx, n1, nop1, nop2, nop3, is1, info, nmcol
      complex*16 cone,czero
      real*8 ox, d, f3, f4, a, b, c, spinfac

c     Output:
      real*8 osx, osy, osz                              !"Transition dipole"

      pointer (ip_ipiv,ipiv),(ip_work,work)
      pointer (ip_ddmat_p,ddmat_p)
      pointer (ip_ddmat_1,ddmat_1)
      pointer (ip_cwork,cwork)
      integer mapful(*), mapcol(*)
      pointer (ip_mapful,mapful),(ip_mapcol,mapcol)
      pointer (ip_cw,cw),(ip_f_raman,f_raman)
      pointer (ip_sc0,sc0)

      save ip_cwork,ip_mapful,ip_mapcol,ifirst
      character  tag*30
      data       ifirst/0/
c     ----------------------------------------------------------------------

      call tiset('    td_os_raman_p',isub)

      cone=dcmplx(1.d0,0.d0)
      czero=dcmplx(0.d0,0.d0)
c     ----------------------------------------------------------------------
c     Memory allocations
c     ----------------------------------------------------------------------
      il_framan=6*ngw*nstate+12
      il_cw=2*ngw*nstate+8
      il_cw1=4*ngw*nstate
      call memory(ip_f_raman,il_framan,'f_raman')
      call zazzero(f_raman,3*ngw*nstate)
      call memory(ip_ddmat_p,2*nstate*nstate,'ddmat_p')
      call memory(ip_ddmat_1,2*nstate*nstate,'ddmat_1')
      call memory(ip_ipiv,50*(nstate/(irat)+1),'ipiv')
      call memory(ip_work,2*nstate*50,'work')
#ifdef PARALLEL
      LENBK=NXXFUN(NSTATE)
      NXX=MAX(2*LENBK*NPROC,il_cw)
#else 
      NXX=il_cw
#endif
      il_cw=nxx
      call memory(ip_sc0,nxx,'sc0')
      call memory(ip_cw,il_cw,'cw')
      call zazzero(cw,il_cw/2)
      if(ifirst.eq.0) then
         n1=0
         do i=0,nproc-1
            n1=max(n1,sparm(3,i))
         enddo
         ngwmax=n1
         call memory(ip_mapful,2*(ngws/irat+1),'mapful')
#ifdef PARALLEL
         nmcol=(nproc*ngwmax)/irat+1
#else 
         nmcol=1
#endif
         call memory(ip_mapcol,nmcol,'mapcol')
         call memory(ip_cwork,4*ngw*nstate,'cwork')
         call setdip(mapful,mapcol,scr,lscr)
         ifirst=1
      endif
c     ----------------------------------------------------------------------
c     End of memory allocations
c     ----------------------------------------------------------------------

c.....Spin degeneracy factor
      if(tlsd) then
         spinfac=1.d0
      else
         spinfac=2.d0
      endif

c     ----------------------------------------------------------------------
c     Set up phase operator matrices (k=1,3) and f_raman.
c     ----------------------------------------------------------------------
      osx = 0.d0
      osy = 0.d0
      osz = 0.d0
      do k=1,3

         call zazzero(cw,il_cw/2)         
         call zazzero(ddmat_p,nstate*nstate)
         call zazzero(ddmat_1,nstate*nstate)
         call zazzero(cwork,2*ngw*nstate)
         call azzero(sc0,nxx)
         call zazzero(f_raman,il_framan/2)

c.......Compute cwork = opeigr*c(g) and Q(i,j) = <phi_i | opeigr | phi_j>.
         if(ibrav.eq.1 .or. ibrav.eq.8) then
            nop1=k
            nop2=0
            nop3=0
         else if(ibrav.eq.2) then
            if(k.eq.1) then
               nop1=1
               nop2=2
               nop3=0
            else if(k.eq.2) then
               nop1=1
               nop2=3
               nop3=0
            else
               nop1=2
               nop2=3
               nop3=0
            end if
         else if(ibrav.eq.3) then
            if(k.eq.1) then
               nop1=1
               nop2=-2
               nop3=-3
            else if(k.eq.2) then
               nop1=1
               nop2=2
               nop3=-3
            else
               nop1=1
               nop2=2
               nop3=3
            end if
         else
            write (6,*) 
     &       'TD_OS_BERRY: ONLY SUPERCELLS OF TYPE 1,2,3,8 allowed.'
         end if
#ifdef PARALLEL
         call my_sync(allgrp)
#endif
         call opeigr_p(c0,cw,sc0,nstate,mapful,mapcol,ddmat_p,
     &        scr,lscr,nop1,nop2,nop3,dd,cwork)
#ifdef PARALLEL
         call my_sync(allgrp)
#endif

c........Make a copy of ddmat_p (ddmat_p need be inverted).
         call ZCOPY(nstate*nstate,ddmat_p,1,ddmat_1,1)

c     ----------------------------------------------------------------------
c     Compute inverse of ddmat_p (Q^-1) for mu=k in Putrino et al.
c     Note ddmat_p is overwrittwen.
c     ----------------------------------------------------------------------
         call ZGETRF(nstate,nstate,ddmat_p,nstate,ipiv,info)
         if(info.eq.0) then
            call ZGETRI(nstate,ddmat_p,nstate,ipiv,work,nstate,info)
         else
            call stopgm('td_os_berry','error in matrix inversion')
         endif
c     ----------------------------------------------------------------------
c     End.
c     ----------------------------------------------------------------------


c     ----------------------------------------------------------------------
c     Build f_raman.
c     ----------------------------------------------------------------------
c........cwork(1,*,*) = [opeigr*c(g)] * Q^-1;
c........cwork(2,*,*) = [opeigr*c(-g)] * Q^-1.
         do i=1,2
            call azzero(cw,il_cw)
            call ZGEMM('N','N',ngw,nstate,nstate,cone,cwork(1,1,i)
     &           ,ngw,ddmat_p(1,1),nstate,czero,cw(1,1),ngw)
            call ZCOPY(ngw*nstate,cw(1,1),1,cwork(1,1,i),1)
         end do
         call my_sync(allgrp)

         if(geq0) then
            is1=2
            do i=1,nstate
               b=dimag(cwork(1,i,1))
               f_raman(1,i,k)=dcmplx(b,0.d0)
            end do
         else
            is1=1
         endif

c        This is to have a sum running on ngw below.
         do i=1,nstate 
            do ig=is1,ngw         
               a=dreal(cwork(ig,i,1))
               b=dimag(cwork(ig,i,1))
               c=dreal(cwork(ig,i,2))
               d=dimag(cwork(ig,i,2))
               f3=a-c
               f4=b+d
               f_raman(ig,i,k)=0.5d0*dcmplx(f4,-f3)
            end do
         end do
c     ----------------------------------------------------------------------
c     End.
c     ----------------------------------------------------------------------


c     ----------------------------------------------------------------------
c     Compute oscillator strengths.
c     ----------------------------------------------------------------------
c.......Compute os = sum_k sum_i <chi_i| opeigr(k) | phi_i> / Q + c.c.
         if(geq0) then
           is1=2
           do i=1,nstate
             ox      =
     >               dreal( dconjg(c1(1,i))*f_raman(1,i,k) )
     >               +dreal( dconjg(f_raman(1,i,k))*c1(1,i) )
             if(k.eq.1) osx = osx + ox
             if(k.eq.2) osy = osy + ox
             if(k.eq.3) osz = osz + ox
           end do
         else
           is1=1
         end if
 
         do i=1,nstate
           do ig=is1,ngw
             ox      =
     >                2.d0*dreal( dconjg(c1(ig,i))*f_raman(ig,i,k)  )
     >               +2.d0*dreal( dconjg(f_raman(ig,i,k))*c1(ig,i)  )
             if(k.eq.1) osx = osx + ox
             if(k.eq.2) osy = osy + ox
             if(k.eq.3) osz = osz + ox
           end do
         end do
c     ----------------------------------------------------------------------
c     End.
c     ----------------------------------------------------------------------
      end do                                        !On cartesian components
#ifdef PARALLEL
      call my_sync(allgrp)
      call summat(osx,1,scr,lscr)
      call summat(osy,1,scr,lscr)
      call summat(osz,1,scr,lscr)
#endif

      if (parent)
     >   write(6,'(A,T25,A,F10.5," eV",T54,A,F10.5)')
     >   ' TD_OS_BERRY|',' dE=',
     >   eigenvalue*RY*2.D0,'f=',
     >   2.d0 *
     >   (2.d0/3.d0)* eigenvalue * ( (spinfac*osx)**2
     >   +   (spinfac*osy)**2
     >   +   (spinfac*osz)**2 )
      if (parent)
     >   write(6,'(A,T26,"x:",F10.5,"  y:",F10.5,"  z:",F10.5)')
     >   ' TD_OS_BERRY|',2.d0*osx, 2.d0*osy, 2.d0*osz

      call freem(ip_work)
      call freem(ip_f_raman)
      call freem(ip_ipiv)
      call freem(ip_ddmat_p)
      call freem(ip_ddmat_1)
      call freem(ip_cw)
      call freem(ip_sc0)
      call tihalt('    td_os_raman_p',isub)
c     ==--------------------------------------------------------------==
      return
      end

