C     ==================================================================
      SUBROUTINE TESTEX(SEXIT)
C     ==--------------------------------------------------------------==
C     ==  TEST FOR FILE EXIT                                          ==
C     ==  TEST FOR A SOFT EXIT CALL OVER N PROCESSORS                 ==
C     ==  TEST IF THE JOB TIME LIMIT IS ENOUGH                        ==
C     ==  CHECK MEMORY                                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'irat.inc'
      INCLUDE 'pimd.inc'
      INCLUDE 'mw.inc'
C     Arguments
      LOGICAL SEXIT
C     Variables
      LOGICAL TEST1,TEST2
      SAVE TEST2
      DATA TEST2 /.FALSE./
CPC   LOGICAL MPP_TERMINATE
#ifdef PARALLEL
      INTEGER IN,OUT, MSGLEN
#endif
      INTEGER MPIGROUP
      LOGICAL T_IO_NODE
#ifdef __ES 
      CHARACTER*145 STRING
      CHARACTER*130 EXIT_FPATH ! PATH TO EXIT FILE ON ES
      COMMON/FPATH/ EXIT_FPATH
#endif 
C     ==--------------------------------------------------------------==
C AK: 2007-07-02 
C with processor groups we cannot sync across the SUPERGROUP 
C as this will create a deadlock when using BO-PIMD or MW MTD.
      IF (TPATH) THEN
        IF(PC_GROUPS.GT.1) THEN
          T_IO_NODE = PARENT
          MPIGROUP = ALLGRP
        ELSE
          T_IO_NODE = GRANDPARENT
          MPIGROUP = SUPERGROUP
        ENDIF
      ELSE ! also IF(TMW)
        T_IO_NODE = PARENT
        MPIGROUP = ALLGRP
      ENDIF
      IF(T_IO_NODE) THEN
#ifdef __ES 
        INQUIRE(FILE=TRIM(EXIT_FPATH)//'EXIT',EXIST=TEST1)
#else 
        INQUIRE(FILE='EXIT',EXIST=TEST1)
#endif 
        IF(TEST1.OR.TEST2) THEN
          SEXIT=.TRUE.
          CALL SOFTEX
        ENDIF
C AK: 2007-07-02
C do not remove the EXIT file here for PIMD and MW MTD, so that _all_
C processor groups can see it before the first one deletes it.
C we set PC_GROUPS to 1 at the end of PI_MDPT and enter here again.
C for MW MTD the EXIT file is removed in TESTEX_MW instead. 
        IF((.NOT.(TPATH.OR.TMW)).OR.(TPATH.AND.(PC_GROUPS.EQ.1))) THEN
#if defined(MYRINET) || defined(CRAY_XT3) || defined(__WINNT)
          IF(TEST1) THEN
            CALL RMFILE('EXIT')
          ENDIF
#elif defined(_BGL)
          OPEN(7,FILE='EXIT',STATUS='UNKNOWN')
          CLOSE(7,STATUS='DELETE')
#elif defined(__ES)
          WRITE(STRING,'(A)') '/bin/rm -f '//TRIM(EXIT_FPATH)//'EXIT'
          IF(TEST1) THEN
            CALL SYSTEM(STRING)
          ENDIF
#else
          IF(TEST1) THEN
            CALL SYSTEM('/bin/rm -f EXIT')
          ENDIF
#endif
        ENDIF
C       Check job limit time
        CALL TILIMIT(TEST1)
        IF(TEST1) THEN
          SEXIT=.TRUE.
          CALL TILIMEX
        ENDIF
      ENDIF
#ifdef PARALLEL
      IN=0
      OUT=0
      IF(SEXIT) THEN
        IN=1
      ENDIF
      MSGLEN = 1 * 8/IRAT
      CALL MY_COMBINE(IN,OUT,MSGLEN,2,MPIGROUP)
      SEXIT = .FALSE.
      IF(OUT.NE.0) THEN
        SEXIT=.TRUE.
      ENDIF
#endif
      IF(SEXIT) THEN
        TEST2=.TRUE.
      ENDIF
C     Check memory
      CALL MEMORY_CHECK
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE TESTEX_MW(SEXIT)
C     ==--------------------------------------------------------------==
C     == Reset SEXIT=.TRUE. if any distributed SEXIT is set to .TRUE. == 
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'irat.inc'
      INCLUDE 'pimd.inc'
      INCLUDE 'mw.inc'
C     Arguments
      LOGICAL SEXIT
C     Variables
      LOGICAL TEST1
      INTEGER IN,OUT,MSGLEN
#ifdef __ES
      CHARACTER*145 STRING
      CHARACTER*130 EXIT_FPATH ! PATH TO EXIT FILE ON ES
      COMMON/FPATH/ EXIT_FPATH
#endif
C     ==--------------------------------------------------------------==
C     The first arriving processor deletes the EXIT file if exists
      IF(PARENT) THEN
#ifdef __ES
        INQUIRE(FILE=TRIM(EXIT_FPATH)//'EXIT',EXIST=TEST1)
#else
        INQUIRE(FILE='EXIT',EXIST=TEST1)
#endif
#if defined(MYRINET) || defined(CRAY_XT3) || defined(__WINNT)
        IF(TEST1) CALL RMFILE('EXIT')
#elif defined(_BGL)
        OPEN(7,FILE='EXIT',STATUS='UNKNOWN')
        CLOSE(7,STATUS='DELETE')
#elif defined(__ES)
        WRITE(STRING,'(A)') '/bin/rm -f '//TRIM(EXIT_FPATH)//'EXIT'
        IF(TEST1) CALL SYSTEM(STRING)
#else
        IF(TEST1) CALL SYSTEM('/bin/rm -f EXIT')
#endif
      ENDIF
      IN=0
      OUT=0
      IF(SEXIT) IN=1
      MSGLEN = 1 * 8/IRAT
      CALL MY_COMBINE(IN,OUT,MSGLEN,2,SUPERGROUP)
      SEXIT = .FALSE.
      IF(OUT.NE.0) SEXIT=.TRUE.
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
