C     ==================================================================
      SUBROUTINE TOTSTR
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'strs.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'prcp.inc'
      INCLUDE 'vdwc.inc'
      INCLUDE 'shock.inc'
      INCLUDE 'cnst_dyn.inc'
C     Variables
      REAL*8  TEMP(3,3)
      INTEGER KK,I,J
      LOGICAL DEBUG
C     ==--------------------------------------------------------------==
C     ==  TOTAL STRESS                                                ==
C     ==--------------------------------------------------------------==
      DEBUG=.FALSE.
C     ==--------------------------------------------------------------==

      DO KK=1,6
        PAIU(ALPHA(KK),BETA(KK)) = -( DEKIN(KK) + DEHT(KK) + DEXC(KK)
     *      + DESR(KK) + DEPS(KK) + DENL(KK) + DEVDW(KK) )
     *      + DEGC(KK) + DLAM(KK) + DECC(KK) + DSTRMETA(KK)
     *      + FVBOUND(KK) + DSTRCV(KK)
        PAIU(BETA(KK),ALPHA(KK)) = PAIU(ALPHA(KK),BETA(KK))
      ENDDO

      CALL GLOSUM(9,PAIU)

C     ==--------------------------------------------------------------==
C     Symmetrisation for DEKIN and DENL (other parts symmetric)
      CALL SYMSTRESS(PAIU)
C     ==--------------------------------------------------------------==
      CALL DCOPY(9,PAIU(1,1),1,HTFP(1,1),1)
      IF(TZFLEX) THEN
        HTFP(3,3)=HTFP(3,3)-DRUCK*OMEGA
      ELSE IF(TISOT.OR.TSHOCK) THEN
        DO I=1,3
          HTFP(I,I)=HTFP(I,I)-DRUCK*OMEGA
        ENDDO
      ELSE
        CALL DAXPY(9,-OMEGA,STENS(1,1),1,HTFP(1,1),1)
      ENDIF
      
C     ==--------------------------------------------------------------==
C     Transform to crystal axis basis (PAIU)
      CALL DGEMM('T','N',3,3,3,1.D0,HTM1,3,PAIU,3,0.D0,TEMP,3)
      CALL DGEMM('N','T',3,3,3,1.D0,TEMP,3,HT,3,0.D0,PAIL,3)
C     HTFP
      CALL DGEMM('N','N',3,3,3,1.0D0,HTFP(1,1),3,HTM1(1,1),3,0.0D0,
     &           HTFOR(1,1),3)

      IF(LMETA_CELL) THEN

        CALL MY_BCAST(FH_CELL,9,SOURCE,ALLGRP)
        DO I=1,3
          DO J = 1,3
            HTFOR(J,I) = HTFOR(J,I) + FH_CELL(J,I)
          ENDDO
        ENDDO
      ENDIF

C     ==--------------------------------------------------------------==
      IF(DEBUG) THEN
        CALL GLOSUM(6,DEKIN)
        CALL GLOSUM(6,DEHT)
        CALL GLOSUM(6,DEXC)
        CALL GLOSUM(6,DESR)
        CALL GLOSUM(6,DEPS)
        CALL GLOSUM(6,DENL)
        CALL GLOSUM(6,DEGC)
        CALL GLOSUM(6,DLAM)
        CALL GLOSUM(6,DECC)
        IF(PARENT) THEN
          WRITE(6,9999)
 9999     FORMAT(1X,4X,"DEKIN",5X,"DEHT",5X,"DEXC",5X,"DESR",
     *         5X,"DEPS",5X,"DENL",5X,"DEGC",5X,"DLAM",5X,"DECC")
          DO KK=1,6
            WRITE(6,'(1X,9F9.5)') -DEKIN(KK)/OMEGA,-DEHT(KK)/OMEGA,
     *                -DEXC(KK)/OMEGA,
     *                -DESR(KK)/OMEGA,-DEPS(KK)/OMEGA,-DENL(KK)/OMEGA,
     *                 DEGC(KK)/OMEGA, DLAM(KK)/OMEGA,DECC(KK)/OMEGA
          ENDDO
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE DSTRE(HTFOR,DSTRESS,DSTRESSMAX)
C     ==--------------------------------------------------------------==
C     == GIVE SUM ABS(HTFOR) and MAX COMPONENTS                       ==
C     ==--------------------------------------------------------------==
C     == INPUT: HTFOR(3,3) stress tensor - required stress tensor     ==
C     == OUTPUT: DSTRESS = Sum Abs(HTFOR)                             ==
C     ==         DSTRESSMAX = Max(HTFOR)                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !OMEGA
      INCLUDE 'cnst.inc'        !AU_KB
      INCLUDE 'prcp.inc'        !TISOT
C     Arguments
      REAL*8 HTFOR(3,3),DSTRESS,DSTRESSMAX
C     Variables
      INTEGER I,J
      REAL*8  ARG
C     ==--------------------------------------------------------------==
      DSTRESS=0.D0
      DSTRESSMAX=0.D0
      IF(TZFLEX) THEN
C     Z-DIRECTION FLEXIBLE CELL
        DSTRESS=DABS(HTFOR(3,3))
        DSTRESSMAX=DSTRESS
      ELSE IF(TISOT) THEN
C     ISOTROPIC CELL
        DSTRESS=DABS(HTFOR(1,1)+HTFOR(2,2)+HTFOR(3,3))
      ELSE
        DO I=1,3
          DO J=1,3
            ARG=DABS(HTFOR(J,I))
            DSTRESS=DSTRESS+ARG
            IF(DSTRESSMAX.LT.ARG) THEN
              DSTRESSMAX=ARG
            ENDIF
          ENDDO
        ENDDO
      ENDIF
      DSTRESS=DSTRESS*AU_KB/OMEGA
      RETURN
      END
C     ==================================================================
