C     ==================================================================
C     Source file of the CDFT transition matrix element calculation code
C     H. Oberhofer (ho246@cam.ac.uk) 2009
C     ==================================================================

C     ==================================================================
      SUBROUTINE OVERLAP(PA,PB,C,SAB)
C     ==--------------------------------------------------------------==
C     == Computes the overlap matrix S of PA and PB                   ==
C     == and its determinant to get SAB and the transposed of its     ==
C     == Cofactor Matrix                                              ==
C     == IN: PA,PB coefficients of the Orbitals of States A and B     ==
C     == OUT: SAB Overlap Matrix Element, C transposed cofactor       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cdft.inc'
      COMPLEX*16 PA(NGW,*),PB(NGW,*)
      INTEGER    IPIV(N),IL,JL,GL,I2,STATUS,OBLOCK
      INTEGER    ILAENV
      LOGICAL    NEG
      REAL*8     C(N,*),COUT(N,N),WORK(N,N),SAB,RES

      SAB=1.0D0
      CALL AZZERO(C,N*N)

      IF(TLSD)THEN
C UP
        DO IL=1,NSUP
          DO JL=1,NSUP
            RES=0.0D0
C$OMP parallel do private(GL) reduction(+:RES)
            DO GL=2,NGW
              RES=RES+2.0D0*(DREAL(PA(GL,IL))*
     *             DREAL(PB(GL,JL))+DIMAG(PA(GL,IL))*DIMAG(PB(GL,JL)))
            ENDDO
            IF(GEQ0)THEN
              C(IL,JL)=RES+DREAL(PA(1,IL))*DREAL(PB(1,JL))
            ELSE
              C(IL,JL)=RES+2.0D0*(DREAL(PA(1,IL))*DREAL(PB(1,JL))+
     *                        DIMAG(PA(1,IL))*DIMAG(PB(1,JL)))
            ENDIF
          ENDDO
        ENDDO
C Down
        DO IL=NSUP+1,N
          DO JL=NSUP+1,N
            RES=0.0D0
C$OMP parallel do private(GL) reduction(+:RES)
            DO GL=2,NGW
              RES=RES+2.0D0*(DREAL(PA(GL,IL))*
     *             DREAL(PB(GL,JL))+DIMAG(PA(GL,IL))*DIMAG(PB(GL,JL)))
            ENDDO
            IF(GEQ0)THEN
              C(IL,JL)=RES+DREAL(PA(1,IL))*DREAL(PB(1,JL))
            ELSE
              C(IL,JL)=RES+2.0D0*(DREAL(PA(1,IL))*DREAL(PB(1,JL))+
     *                        DIMAG(PA(1,IL))*DIMAG(PB(1,JL)))
            ENDIF
          ENDDO
        ENDDO
      ELSE
        DO IL=1,N
          DO JL=1,N
            RES=0.0D0
C$OMP parallel do private(GL) reduction(+:RES)
            DO GL=2,NGW
              RES=RES+2.0D0*(DREAL(PA(GL,IL))*
     *              DREAL(PB(GL,JL))+DIMAG(PA(GL,IL))*DIMAG(PB(GL,JL)))
            ENDDO
            IF(GEQ0)THEN
              C(IL,JL)=RES+DREAL(PA(1,IL))*DREAL(PB(1,JL))
            ELSE
              C(IL,JL)=RES+2.0D0*(DREAL(PA(1,IL))*DREAL(PB(1,JL))+
     *                        DIMAG(PA(1,IL))*DIMAG(PB(1,JL)))
            ENDIF
          ENDDO
        ENDDO
      ENDIF
      CALL GLOSUM(N*N,C)
      IF(PARENT.AND.TPHIO)THEN
        DO IL=1,N
          DO JL=1,N
            WRITE(71,'( F13.8,"  ",$)')C(IL,JL)
          ENDDO
          WRITE(71,*) " "
        ENDDO
        WRITE(71,*) " "
      ENDIF

C Decompose Matrix and calculate Determinant
      CALL IAZZERO(IPIV,N)
      CALL DGETRF(N,N,C,N,IPIV,STATUS)

      IF(STATUS.NE.0) THEN
        IF(PARENT)WRITE(6,*)"Output of Decomposition ",STATUS
        CALL STOPGM('OVERLAP','DECOMPOSITION FAILED')
      ENDIF
      NEG=.FALSE.
      DO IL=1,N
        SAB=SAB*C(IL,IL)
        IF(IPIV(IL).NE.IL) NEG=(.NOT.NEG)
      ENDDO
      IF(NEG)SAB=SAB*(-1.0D0)
      IF(.NOT.TLSD)SAB=SAB*SAB

C Calculate the inverse
      OBLOCK = ILAENV(1,"DGETRI"," ",N,N,-1,-1)
      CALL AZZERO(WORK,N*N)
      CALL DGETRI(N,C,N,IPIV,WORK,N*OBLOCK,STATUS)
      IF(STATUS.NE.0) THEN
        IF(PARENT)WRITE(6,*)"Output of Inversion ",STATUS
        CALL STOPGM('OVERLAP','INVERSION FAILED')
      ENDIF

      CALL AZZERO(WORK,N*N)
      DO IL=1,N
        DO JL=1,N
          WORK(IL,JL)=SAB*C(JL,IL)
        ENDDO
      ENDDO
      CALL DCOPY(N*N,WORK,1,C,1)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE MKFULLP(PA)
C     ==--------------------------------------------------------------==
C     == Gathers the full wavefunction                                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'transme.inc'

      COMPLEX*16 PA(*)
      INTEGER    IL,MSGL

      CALL AZZERO(W,2*MXHG)
      DO IL=1,NGW
        W(IL)=PA(IL)
      ENDDO

      MSGL=2*MXHG*8
      CALL MY_CONCAT(W,FULLP,MSGL,ALLGRP)

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================


C     ==================================================================
      SUBROUTINE TRANSME(PA,PB,IPAR,IPBR,FA,FB,VA,VB,NAI,NBI)
C     ==--------------------------------------------------------------==
C     == Computes the transition matrix element                       ==
C     == IN: PA,PB coefficients of the Orbitals of States A and B     ==
C     == IN: FA,FB uncorrected energies for States A and B            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'rswf.inc'
      INCLUDE 'cdft.inc'
      INCLUDE 'transme.inc'


      COMPLEX*16 PA(NGW,*),PB(NGW,*),IPAR(NNR1,*),IPBR(NNR1,*)
      REAL*8     PAR(NNR1,*),PBR(NNR1,*)
      REAL*8     MATEL,MATEL_REAL,VB(2),VA(2),temp
      REAL*8     FA,FB,EA,EB,SAA,SBB
      REAL*8     NAI(2),NBI(2)
      REAL*8     CAB(N,N),CBA(N,N),CBB(N,N),CAA(N,N)
      REAL*8     HAB,HBA,NA,NB
      REAL*8     HAB2,HBA2,NA2,NB2
      REAL*8     HABS,HBAS,NAS,NBS
      REAL*8     ZVDIFF,DIFFA,DIFFB
      POINTER    (IP_CAB,CAB)
      POINTER    (IP_CBA,CBA)
      POINTER    (IP_CBB,CBB)
      POINTER    (IP_CAA,CAA)
      POINTER    (IP_PAR,PAR)
      POINTER    (IP_PBR,PBR)
      REAL*8     H(2,2),T(2,2),WAB(2,2),S(2),V(2,2),L(2)
      REAL*8     X(2,2),HP(2,2),VI(2,2),E(2),HREST,CSM
      REAL*8     UP(2,2),HPROJ(2,2)! for projection
      INTEGER    IL,JL,ISUB,MSGL,LOWI,LOWI1,LOWI2,KL
!       DIMENSION  F(*)

      IF(PARENT)CALL TISET('   TRANSME',ISUB)
      HREST=0.0D0
      LOWI=1
      LOWI1=1
      LOWI2=NSUP+1
      
      IF(PARENT)WRITE(6,*)"Calculating Overlap Matrix"
      CALL MEMORY(IP_CAB,N*N,'CAB')
      CALL MEMORY(IP_CBA,N*N,'CBA')
      CALL MEMORY(IP_CAA,N*N,'CAA')
      CALL MEMORY(IP_CBB,N*N,'CBB')
      CALL MEMORY(IP_PAR,N*NNR1,'PAR')
      CALL MEMORY(IP_PBR,N*NNR1,'PBR')
      IF(PARENT.AND.TPHIO)THEN
        OPEN(71,FILE="PHI_MAT",STATUS='UNKNOWN')
        REWIND(UNIT=71)
      ENDIF
      CALL OVERLAP(PB,PB,CBB,SBB)
      CALL OVERLAP(PA,PA,CAA,SAA)
      CALL OVERLAP(PA,PB,CAB,S(2))
      CALL OVERLAP(PB,PA,CBA,S(1))
      IF(PARENT.AND.TPHIO)CLOSE(71)

C     ==--------------------------------------------------------------==

      HAB=HREST
      HBA=HAB
      CALL AZZERO(PAR(1,1),NNR1*N)
      CALL AZZERO(PBR(1,1),NNR1*N)
      IF(PARENT)WRITE(6,*)"Constructing real space wavefunctions"
      KL=1
      DO IL=1,N,2
C$OMP parallel do private(JL)
        DO JL=1,NNR1
          PAR(JL,IL)=DREAL(IPAR(JL,KL))
          IF(IL.LT.N) PAR(JL,IL+1)=DIMAG(IPAR(JL,KL))
        ENDDO
        KL=KL+1
      ENDDO
      KL=1
      DO IL=1,N,2
C$OMP parallel do private(JL)
        DO JL=1,NNR1
          PBR(JL,IL)=DREAL(IPBR(JL,KL))
          IF(IL.LT.N) PBR(JL,IL+1)=DIMAG(IPBR(JL,KL))
        ENDDO
        KL=KL+1
      ENDDO

      NA=0.0D0
      NB=NA
      HAB=0.0D0
      HBA=HAB
      NA2=0.0D0
      NB2=NA2
      HAB2=0.0D0
      HBA2=HAB2
      CSM=1.0D0
      IF(TSPINC)CSM=-1.0D0

C     ==--------------------------------------------------------------==
      IF(PARENT)WRITE(6,*)"Starting Summation"
      IF(TLSD)THEN
C  Up
        DO IL=LOWI1,NSUP
          DO JL=1,NSUP
            HAB=HAB+MATEL_REAL(PAR(1,IL),PBR(1,JL),WD)*CAB(IL,JL)
            NB=NB+MATEL_REAL(PBR(1,IL),PBR(1,JL),WD)*CBB(IL,JL)
            HBA=HBA+MATEL_REAL(PBR(1,IL),PAR(1,JL),WDIFF)*CBA(IL,JL)
            NA=NA+MATEL_REAL(PAR(1,IL),PAR(1,JL),WDIFF)*CAA(IL,JL)
          ENDDO
        ENDDO
C  Down
        DO IL=LOWI2,N
          DO JL=NSUP+1,N
            HAB2=HAB2+MATEL_REAL(PAR(1,IL),PBR(1,JL),WD)*CAB(IL,JL)
            NB2=NB2+MATEL_REAL(PBR(1,IL),PBR(1,JL),WD)*CBB(IL,JL)
            HBA2=HBA2+MATEL_REAL(PBR(1,IL),PAR(1,JL),WDIFF)*CBA(IL,JL)
            NA2=NA2+MATEL_REAL(PAR(1,IL),PAR(1,JL),WDIFF)*CAA(IL,JL)
          ENDDO
        ENDDO
        HAB=HAB+CSM*HAB2
        HBA=HBA+CSM*HBA2
        NA=NA+CSM*NA2
        NB=NB+CSM*NB2
      ELSE
        DO IL=LOWI,N
          DO JL=1,N
            HAB=HAB+MATEL_REAL(PAR(1,IL),PBR(1,JL),WD)*CAB(IL,JL)
            NB=NB+MATEL_REAL(PBR(1,IL),PBR(1,JL),WD)*CBB(IL,JL)
            HBA=HAB+MATEL_REAL(PBR(1,IL),PAR(1,JL),WDIFF)*CBA(IL,JL)
            NA=NA+MATEL_REAL(PAR(1,IL),PAR(1,JL),WDIFF)*CAA(IL,JL)
          ENDDO
        ENDDO
        HAB=HAB*2.0D0
        NB=NB*2.0D0
        HBA=HBA*2.0D0
        NA=NA*2.0D0
      ENDIF
      IF(PARENT)THEN
        DIFFA=NA+ZVDIFF()
        DIFFB=NB+ZVDIFF()
        IF(ABS(DIFFA-NAI(1)).GT.TRANSME_TOL
     *    .OR.ABS(DIFFB-NBI(1)).GT.TRANSME_TOL)THEN
          WRITE(6,'(1X,64("="),/)')
          WRITE(6,*)"WARNING!"
          WRITE(6,*)"INTEGRATED N VALUES DO NOT COINCIDE WITH N_C"
          WRITE(6,'(/,2X,"Integrated Values")')
          WRITE(6,'(3X,F13.8,F13.8,/)')DIFFA,DIFFB
          WRITE(6,'(/,2X,"N_C")')
          WRITE(6,'(3X,F13.8,F13.8,/)')NAI(1),NBI(1)
          WRITE(6,'(1X,64("="),/)')
        ENDIF
        WAB(1,2)=HBA
        WAB(2,1)=HAB
        WAB(1,1)=NB
        WAB(2,2)=NA

        H(1,1)=FB
        H(2,2)=FA
        H(2,1)=(FB+VB(1)*NB)*S(2)-VB(1)*WAB(1,2)
        H(1,2)=(FA+VA(1)*NA)*S(1)-VA(1)*WAB(2,1)

        CALL GENEIGV(S,WAB,V,L)
        CALL VINV(V,VI)
        CALL HPRIME(H,HP,S)
        CALL HTRANS2(T,V,VI,HP,H)
        CALL ODIAG(HP,X,E)
      ENDIF

      IF(THDAPROJ)THEN
        CALL DCOPY(4,X,1,UP,1)
        HPROJ(1,1)=E(1)
        HPROJ(2,2)=E(2)
        PROJNOW=.TRUE.
        CALL HDA_PROJECT(PA,PB,UP,HPROJ)
      ENDIF

      IF(PARENT)THEN
        WRITE(6,'(1X,64("-"),/)')
        WRITE(6,'(2X,"Matrix Element Calculation",/)')
        WRITE(6,'(2X,"Overlap Matrix (S)")')
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')SBB,S(1),
     *                                              S(2),SAA

        WRITE(6,'(2X,"Weight Matrix (W)")')
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')WAB(1,1),
     *                                  WAB(1,2),WAB(2,1),WAB(2,2)

        WRITE(6,'(2X,A)')"Non-orthogonal diabatic H Matrix (H'')"
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')T(1,1),T(1,2),
     *                                              T(2,1),T(2,2)

        WRITE(6,'(2X,A)')"Full non-orthogonal Hamiltonian (H')"
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')HP(1,1),HP(1,2),
     *                                              HP(2,1),HP(2,2)

        WRITE(6,'(2X,"Generalised Eigenvalues")')
        WRITE(6,'(3X,F14.8,F14.8,/)')L(1),L(2)
        WRITE(6,'(2X,"Generalised Eigenvectors (V)")')
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')V(1,1),V(1,2),
     *                                              V(2,1),V(2,2)
        WRITE(6,'(2X,"Orthogonal diabatic Hamiltonian (H)")')
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')H(1,1),H(1,2),
     *                                              H(2,1),H(2,2)
        WRITE(6,'(2X,"Adiabatic Hamiltonian")')
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')E(1),0.0D0,
     *          0.0D0,E(2)
        WRITE(6,'(2X,"Mixing Coefficients")')
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')X(1,1),X(1,2),
     *                                              X(2,1),X(2,2)
        IF(THDAPROJ)THEN
         WRITE(6,*)
         WRITE(6,'(2X,"Diabatic Hamiltonian in orthogonalised",/
     *               " dressed basis")')
         WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')HPROJ(1,1),
     *                            HPROJ(1,2),HPROJ(2,1),HPROJ(2,2)
        WRITE(6,'(2X,"Projection Matrix")')
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')UP(1,1),UP(1,2),
     *                                              UP(2,1),UP(2,2)
        ENDIF
        WRITE(6,'(1X,64("-"),/)')

        CALL TIHALT('   TRANSME',ISUB)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE TRANSMEEXT(PA,PB,IPAR,IPBR,FA,FB)
C     ==--------------------------------------------------------------==
C     == Computes the transition matrix element for a given external  ==
C     ==  potential, potentials for both states are stored in WA/WD   ==
C     == IN: PA,PB coefficients of the Orbitals of States A and B     ==
C     == IN: FA,FB uncorrected energies for States A and B            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'rswf.inc'
      INCLUDE 'cdft.inc'
      INCLUDE 'transme.inc'


      COMPLEX*16 PA(NGW,*),PB(NGW,*),IPAR(NNR1,*),IPBR(NNR1,*)
      REAL*8     PAR(NNR1,*),PBR(NNR1,*)
      REAL*8     MATEL_REAL,temp
      REAL*8     FA,FB,EA,EB,NA,NB,SAA,SBB
      REAL*8     CAB(N,N),CBA(N,N),CBB(N,N),CAA(N,N),HAB,HBA!,TEST
!       REAL*8     ZVDIFF,DIFFA,DIFFB
      POINTER    (IP_CAB,CAB)
      POINTER    (IP_CBA,CBA)
      POINTER    (IP_CBB,CBB)
      POINTER    (IP_CAA,CAA)
      POINTER    (IP_PAR,PAR)
      POINTER    (IP_PBR,PBR)
      REAL*8     H(2,2),T(2,2),WAB(2,2),S(2),V(2,2),L(2)
      REAL*8     X(2,2),HP(2,2),VI(2,2),E(2),HREST
      INTEGER    IL,JL,ISUB,MSGL,LOWI,LOWI1,LOWI2,KL
!       DIMENSION  F(*)

      IF(PARENT)CALL TISET('   TRANSME',ISUB)
      HREST=0.0D0
      LOWI=1
      LOWI1=1
      LOWI2=NSUP+1

      IF(PARENT)WRITE(6,*)"Calculating Overlap Matrix"
      CALL MEMORY(IP_CAB,N*N,'CAB')
      CALL MEMORY(IP_CBA,N*N,'CBA')
      CALL MEMORY(IP_CAA,N*N,'CAA')
      CALL MEMORY(IP_CBB,N*N,'CBB')
      CALL MEMORY(IP_PAR,N*NNR1,'PAR')
      CALL MEMORY(IP_PBR,N*NNR1,'PBR')
      IF(PARENT.AND.TPHIO)THEN
        OPEN(71,FILE="PHI_MAT",STATUS='UNKNOWN')
        REWIND(UNIT=71)
      ENDIF
      CALL OVERLAP(PB,PB,CBB,SBB)
      CALL OVERLAP(PA,PA,CAA,SAA)
      CALL OVERLAP(PA,PB,CAB,S(2))
      CALL OVERLAP(PB,PA,CBA,S(1))
      IF(PARENT.AND.TPHIO)CLOSE(71)

C     ==--------------------------------------------------------------==

      HAB=HREST
      HBA=HAB
      CALL AZZERO(PAR(1,1),NNR1*N)
      CALL AZZERO(PBR(1,1),NNR1*N)
      IF(PARENT)WRITE(6,*)"Constructing real space wavefunctions"
      KL=1
      DO IL=1,N,2
C$OMP parallel do private(JL)
        DO JL=1,NNR1
          PAR(JL,IL)=DREAL(IPAR(JL,KL))
          IF(IL.LT.N) PAR(JL,IL+1)=DIMAG(IPAR(JL,KL))
        ENDDO
        KL=KL+1
      ENDDO
      KL=1
      DO IL=1,N,2
C$OMP parallel do private(JL)
        DO JL=1,NNR1
          PBR(JL,IL)=DREAL(IPBR(JL,KL))
          IF(IL.LT.N) PBR(JL,IL+1)=DIMAG(IPBR(JL,KL))
        ENDDO
        KL=KL+1
      ENDDO
      NA=0.0D0
      NB=NA
C     ==--------------------------------------------------------------==
      IF(PARENT)WRITE(6,*)"Starting Summation"
      IF(TLSD)THEN
C  Up
        DO IL=LOWI1,NSUP
          DO JL=1,NSUP
            HAB=HAB+MATEL_REAL(PAR(1,IL),PBR(1,JL),WDIFF)*CAB(IL,JL)
            NB=NB+MATEL_REAL(PBR(1,IL),PBR(1,JL),WDIFF)*CBB(IL,JL)
            HBA=HBA+MATEL_REAL(PBR(1,IL),PAR(1,JL),WDIFF)*CBA(IL,JL)
            NA=NA+MATEL_REAL(PAR(1,IL),PAR(1,JL),WDIFF)*CAA(IL,JL)
          ENDDO
        ENDDO
C  Down
        DO IL=LOWI2,N
          DO JL=NSUP+1,N
            HAB=HAB+MATEL_REAL(PAR(1,IL),PBR(1,JL),WDIFF)*CAB(IL,JL)
            NB=NB+MATEL_REAL(PBR(1,IL),PBR(1,JL),WDIFF)*CBB(IL,JL)
            HBA=HBA+MATEL_REAL(PBR(1,IL),PAR(1,JL),WDIFF)*CBA(IL,JL)
            NA=NA+MATEL_REAL(PAR(1,IL),PAR(1,JL),WDIFF)*CAA(IL,JL)
          ENDDO
        ENDDO
      ELSE
        DO IL=LOWI,N
          DO JL=1,N
            HAB=HAB+MATEL_REAL(PAR(1,IL),PBR(1,JL),WDIFF)*CAB(IL,JL)
            NB=NB+MATEL_REAL(PBR(1,IL),PBR(1,JL),WDIFF)*CBB(IL,JL)
            HBA=HAB+MATEL_REAL(PBR(1,IL),PAR(1,JL),WDIFF)*CBA(IL,JL)
            NA=NA+MATEL_REAL(PAR(1,IL),PAR(1,JL),WDIFF)*CAA(IL,JL)
          ENDDO
        ENDDO
        HAB=HAB*2.0D0
        NB=NB*2.0D0
        HBA=HBA*2.0D0
        NA=NA*2.0D0
      ENDIF
      IF(PARENT)THEN
        WAB(1,2)=HBA
        WAB(2,1)=HAB
        WAB(1,1)=NB
        WAB(2,2)=NA

        H(1,1)=FB-NB
        H(2,2)=FA-NA
        H(2,1)=FB*S(2)-WAB(1,2)
        H(1,2)=FA*S(1)-WAB(2,1)
        H(2,1)=(H(1,2)+H(2,1))/2.0D0
        H(1,2)=H(2,1)

        CALL GENEIGV(S,WAB,V,L)
        CALL VINV(V,VI)
        CALL HPRIME(H,HP,S)
        CALL HTRANS2(T,V,VI,HP,H)
      ENDIF

      IF(PARENT)THEN
        WRITE(6,'(1X,64("-"),/)')
        WRITE(6,'(2X,"Matrix Element Calculation",/)')
        WRITE(6,'(2X,"Overlap Matrix (S)")')
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')SBB,S(1),
     *                                              S(2),SAA

        WRITE(6,'(2X,"Weight Matrix (W)")')
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')WAB(1,1),
     *                                  WAB(1,2),WAB(2,1),WAB(2,2)

        WRITE(6,'(2X,A)')"Non-orthogonal diabatic H Matrix (H'')"
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')T(1,1),T(1,2),
     *                                              T(2,1),T(2,2)

        WRITE(6,'(2X,A)')"Full non-orthogonal Hamiltonian (H')"
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')HP(1,1),HP(1,2),
     *                                              HP(2,1),HP(2,2)

        WRITE(6,'(2X,"Generalised Eigenvalues")')
        WRITE(6,'(3X,F14.8,F14.8,/)')L(1),L(2)
        WRITE(6,'(2X,"Generalised Eigenvectors (V)")')
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')V(1,1),V(1,2),
     *                                              V(2,1),V(2,2)
        WRITE(6,'(2X,"Orthogonal diabatic H Matrix (H)")')
        WRITE(6,'(3X,F14.8,F14.8,/,3X,F14.8,F14.8,/)')H(1,1),H(1,2),
     *                                              H(2,1),H(2,2)
        WRITE(6,'(1X,64("-"),/)')

        CALL TIHALT('   TRANSME',ISUB)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================


C     ==================================================================
      FUNCTION MATEL(PA)
C     ==--------------------------------------------------------------==
C     ==Calculate the W Matrix Element in G space (currently not used)==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'transme.inc'

      REAL*8     MATEL
      COMPLEX*16 PA(*),RES,BUF,PA0CON,PB0
      INTEGER    G,GP,GSTART,IP,GS
      INTEGER    GX(3),GPX(3),NH(3),GMINUS(3),GPLUS(3) ! GX : G vector; GPX : GP vector; GMINUS :  GPX-GX ; GPLUS : GPX+GX
      INTEGER    GTP,GTM!GP+G and GP-G
      LOGICAL    CON,CONP

C     ==--------------------------------------------------------------==
      NH(1)=NR1S/2+1
      NH(2)=NR2S/2+1
      NH(3)=NR3S/2+1

      PA0CON=DCONJG(PA(1))
      PB0=FULLP(1+IGEQ0*MXHG)

      IF(GEQ0)THEN
        RES=PA0CON*PB0*FULLW(1)
        GSTART=2
      ELSE
        RES=DCMPLX(0.0D0,0.0D0)
        GSTART=1
      ENDIF


      DO G=GSTART,NGW
        RES=RES+2.0D0*PB0*DREAL(PA(G)*FULLW(G+MEPOS*MXHG))

        GX(1)=INYH(1,G)-NH(1)
        GX(2)=INYH(2,G)-NH(2)
        GX(3)=INYH(3,G)-NH(3)
        DO IP=0,NPROC-1
        IF(IP.EQ.IGEQ0)THEN
          GS=2
        ELSE
          GS=1
        ENDIF
        DO GP=GS+IP*MXHG,SPARM(3,IP)+IP*MXHG
          GPX(1)=MINYH(1,GP)-NH(1)
          GPX(2)=MINYH(2,GP)-NH(2)
          GPX(3)=MINYH(3,GP)-NH(3)
          GMINUS(1)=GPX(1)-GX(1)
          IF(GMINUS(1).LT.0)THEN
            GMINUS(1)=NH(1)-GMINUS(1)
            CON=.TRUE.
          ELSE
            GMINUS(1)=GMINUS(1)+NH(1)
            CON=.FALSE.
          ENDIF
          GMINUS(2)=GPX(2)-GX(2)+NH(2)
          GMINUS(3)=GPX(3)-GX(3)+NH(3)
          GPLUS(1)=GPX(1)+GX(1)+NH(1)
          GPLUS(2)=GPX(2)+GX(2)+NH(2)
          GPLUS(3)=GPX(3)+GX(3)+NH(3)

          GTP=GINDEX(GPLUS(1),GPLUS(2),GPLUS(3))
          GTM=GINDEX(GMINUS(1),GMINUS(2),GMINUS(3))

          BUF=DCMPLX(0.0D0,0.0D0)
          IF(GTP.GT.0)THEN
            BUF=BUF+DREAL(PA(G)*FULLP(GP)*FULLW(GTP))
          ENDIF
          IF(GTM.GT.0)THEN
            IF(CON)THEN
              BUF=BUF+DREAL(PA(G)*DCONJG(FULLP(GP))*FULLW(GTM))
            ELSE
              BUF=BUF+DREAL(PA(G)*DCONJG(FULLP(GP)*FULLW(GTM)))
            ENDIF
          ENDIF
          RES=RES+2.0D0*BUF
        ENDDO
        ENDDO
      ENDDO
      IF(GEQ0)THEN
        DO IP=0,NPROC-1
        IF(IP.EQ.IGEQ0)THEN
          GS=2
        ELSE
          GS=1
        ENDIF
        DO GP=GS+IP*MXHG,SPARM(3,IP)+IP*MXHG
            RES=RES+2.0D0*PA0CON*
     *          DREAL(FULLP(GP)*FULLW(GP))
        ENDDO
        ENDDO
      ENDIF

      MATEL=DREAL(RES)
      CALL GLOSUM(1,MATEL)
      RETURN
      END
C     ==================================================================

C     ==================================================================
      FUNCTION MATEL_REAL(PA,PB,W)
C     ==--------------------------------------------------------------==
C     == Calculate the W Matrix Element in Real Space (more efficient)==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'rswf.inc'

      REAL*8     MATEL_REAL
      REAL*8     PA(*),PB(*),W(*),RES
      INTEGER    IL

      RES=0.0D0
C$OMP parallel do private(IL) reduction(+:RES)
      DO IL=1,NNR1
        RES=RES+PA(IL)*PB(IL)*W(IL)
      ENDDO

      RES=RES/(NR1S*NR2S*NR3S)

      CALL GLOSUM(1,RES)
      MATEL_REAL=RES
      RETURN
      END
C     ==================================================================


C     ==================================================================
      SUBROUTINE MKGINDEX()
C     ==--------------------------------------------------------------==
C     == Construct the G vector index, belongs to G space MATEL       ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'transme.inc'

      INTEGER   NIBUF(3,*),MSGL,HGP,IP
      POINTER   (IP_NIBUF,NIBUF)
      INTEGER   GI,MAXL,I,J,K,IR,I2,J2,K2,CHECK
      REAL*8    T,G2
      CALL MEMORY(IP_GINDEX,KR1S*KR2S*KR3S,'GINDEX')

C     ==--------------------------------------------------------------==
      MXHG=SPARM(1,0)
      DO IP=1,NPROC-1
        IF(SPARM(1,IP).GT.MXHG)MXHG=SPARM(1,IP)
      ENDDO
      MXHG=MXHG+1
      BIGA=MXHG*NPROC
      CALL MEMORY(IP_MINYH,(3*BIGA)/IRAT,'MINYH')
      CALL MEMORY(IP_NIBUF,(3*MXHG)/IRAT,'NIBUF')
C Get G Vector Mapping from nodes
      CALL AZZERO(NIBUF,3*MXHG/IRAT)
      DO GI=1,NHG
        NIBUF(1,GI)=INYH(1,GI)
        NIBUF(2,GI)=INYH(2,GI)
        NIBUF(3,GI)=INYH(3,GI)
      ENDDO
      MSGL=3*MXHG*8/IRAT
      CALL MY_CONCAT(NIBUF,MINYH,MSGL,ALLGRP)
C     ==--------------------------------------------------------------==
      MAXL=KR1S*KR2S*KR3S
      CALL AZZERO(GINDEX,MAXL)
      CHECK=0
      DO GI=1,BIGA
        I=MINYH(1,GI)
        J=MINYH(2,GI)
        K=MINYH(3,GI)
        I2=I-NR1S/2-1
        J2=J-NR2S/2-1
        K2=K-NR3S/2-1
        G2=0.0D0
        DO IR=1,3
          T=DBLE(I2)*B1(IR)+DBLE(J2)*B2(IR)+DBLE(K2)*B3(IR)
          G2=G2+T*T
        ENDDO
        IF(I.GT.0.AND.J.GT.0.AND.K.GT.0.AND.G2.LT.GCUT)THEN
          GINDEX(I,J,K)=GI
          CHECK=CHECK+1
        ENDIF
      ENDDO
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE GENEIGV(S,W,V,L)
C     ==--------------------------------------------------------------==
C     == Solve the generalised eigenvalue problem                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE

      REAL*8   S(2),W(2,2),V(2,2),L(2)! (SBA,SAB), ((NB,WBA)(WAB,NA)), ((Eigenvector1)(Eigenvector2))
      REAL*8   A,B,C,D,NORM

      A=1-S(1)*S(2)
      B=S(1)*W(2,1)+S(2)*W(1,2)-W(1,1)-W(2,2)
      C=W(1,1)*W(2,2)-W(1,2)*W(2,1)
      D=B*B-4.0D0*A*C

      L(1)=(-B-SQRT(D))/(2.0D0*A)
      L(2)=(-B+SQRT(D))/(2.0D0*A)

C      V(1,1)=L(1)*S(1)-W(2,1)
      V(1,1)=L(1)*S(1)-W(1,2)
      V(2,1)=W(1,1)-L(1)
      NORM=SQRT(V(1,1)*(V(1,1)+S(2)*V(2,1))+
     *     V(2,1)*(S(1)*V(1,1)+V(2,1)))
      V(1,1)=V(1,1)/NORM!normalise
      V(2,1)=V(2,1)/NORM!normalise
      V(1,2)=L(2)*S(1)-W(1,2)
C      V(1,2)=L(2)*S(1)-W(2,1)
      V(2,2)=W(1,1)-L(2)
      NORM=SQRT(V(1,2)*(V(1,2)+S(2)*V(2,2))+
     *     V(2,2)*(S(1)*V(1,2)+V(2,2)))
      V(1,2)=V(1,2)/NORM!normalise
      V(2,2)=V(2,2)/NORM!normalise

      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE HTRANS(T,V,H)
C     ==--------------------------------------------------------------==
C     == Transform the Hamiltonian                                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      REAL*8   T(2,2),H(2,2),V(2,2)

      T(1,1)=H(1,1)
      T(2,1)=H(2,1)!normal
      T(1,2)=H(1,2)!normal
      T(2,2)=H(2,2)

      H(1,1)=V(1,1)*V(1,1)*T(1,1)+V(1,1)*V(2,1)*T(1,2)+
     *       V(1,1)*V(2,1)*T(2,1)+V(2,1)*V(2,1)*T(2,2)
      H(2,1)=V(1,1)*V(1,2)*T(1,1)+V(2,1)*V(1,2)*T(1,2)+
     *       V(1,1)*V(2,2)*T(2,1)+V(2,1)*V(2,2)*T(2,2)
      H(1,2)=V(1,1)*V(1,2)*T(1,1)+V(1,1)*V(2,2)*T(1,2)+
     *       V(2,1)*V(1,2)*T(2,1)+V(2,1)*V(2,2)*T(2,2)
      H(2,2)=V(1,2)*V(1,2)*T(1,1)+V(1,2)*V(2,2)*T(1,2)+
     *       V(1,2)*V(2,2)*T(2,1)+V(2,2)*V(2,2)*T(2,2)

      H(1,2)=ABS(H(1,2))
      H(2,1)=ABS(H(2,1))

      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE HTRANS2(T,V,VI,HP,H)
C     ==--------------------------------------------------------------==
C     == Transform the Hamiltonian                                    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      REAL*8   T(2,2),H(2,2),HP(2,2),V(2,2),VI(2,2)

      T(1,1)=H(1,1)
      T(2,1)=H(2,1)
      T(1,2)=H(1,2)
      T(2,2)=H(2,2)

      H(1,1)=V(1,1)*VI(1,1)*HP(1,1)+V(2,1)*VI(1,1)*HP(1,2)+
     *       V(1,1)*VI(1,2)*HP(2,1)+V(2,1)*VI(1,2)*HP(2,2)
      H(2,1)=V(1,1)*VI(2,1)*HP(1,1)+V(2,1)*VI(2,1)*HP(1,2)+
     *       V(1,1)*VI(2,2)*HP(2,1)+V(2,1)*VI(2,2)*HP(2,2)
      H(1,2)=V(1,2)*VI(1,1)*HP(1,1)+V(2,2)*VI(1,1)*HP(1,2)+
     *       V(1,2)*VI(1,2)*HP(2,1)+V(2,2)*VI(1,2)*HP(2,2)
      H(2,2)=V(1,2)*VI(2,1)*HP(1,1)+V(2,2)*VI(2,1)*HP(1,2)+
     *       V(1,2)*VI(2,2)*HP(2,1)+V(2,2)*VI(2,2)*HP(2,2)

      H(1,2)=ABS(H(1,2))
      H(2,1)=ABS(H(2,1))

      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE HTRANS3(H,V)
C     ==--------------------------------------------------------------==
C     == Transform the Hamiltonian  (V.H.V^T)                         ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      REAL*8   T(2,2),H(2,2),V(2,2)

      T(1,1)=H(1,1)
      T(2,1)=H(2,1)
      T(1,2)=H(1,2)
      T(2,2)=H(2,2)

      H(1,1)=V(1,1)*V(1,1)*T(1,1)+V(1,1)*V(1,2)*T(1,2)+
     *       V(1,1)*V(1,2)*T(2,1)+V(1,2)*V(1,2)*T(2,2)
      H(2,1)=V(1,1)*V(2,1)*T(1,1)+V(1,2)*V(2,1)*T(1,2)+
     *       V(1,1)*V(2,2)*T(2,1)+V(1,2)*V(2,2)*T(2,2)
      H(1,2)=V(1,1)*V(2,1)*T(1,1)+V(1,1)*V(2,2)*T(1,2)+
     *       V(1,2)*V(2,1)*T(1,2)+V(1,2)*V(2,2)*T(2,2)
      H(2,2)=V(2,1)*V(2,1)*T(1,1)+V(2,1)*V(2,2)*T(1,2)+
     *       V(2,1)*V(2,2)*T(2,1)+V(2,2)*V(2,2)*T(2,2)

      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE ODIAG(H,V,L)
C     ==--------------------------------------------------------------==
C     == Solve the eigenvalue problem                                 ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      REAL*8   H(2,2),V(2,2),L(2),T

      T=SQRT(H(1,1)**2+H(2,2)**2-2.0D0*H(1,1)*H(2,2)+
     *       4.0D0*H(1,2)*H(2,1))

      L(1)=(H(1,1)+H(2,2)-T)/2.0D0
      L(2)=(H(1,1)+H(2,2)+T)/2.0D0

      V(1,1)=-(H(2,2)-H(1,1)+T)/(2.0D0*H(2,1))
      V(1,2)=-(H(2,2)-H(1,1)-T)/(2.0D0*H(2,1))
      V(2,1)=1.0D0
      V(2,2)=1.0D0
      T=SQRT(V(1,1)*V(1,1)+V(2,1)*V(2,1))
      V(1,1)=V(1,1)/T
      V(2,1)=V(2,1)/T
      T=SQRT(V(1,2)*V(1,2)+V(2,2)*V(2,2))
      V(1,2)=V(1,2)/T
      V(2,2)=V(2,2)/T

      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE HPRIME(H,HP,S)
C     ==--------------------------------------------------------------==
C     == Construct the full Hamiltonian                               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      REAL*8   H(2,2),HP(2,2),S(2),T

      T=1.0D0-S(1)**2
      H(2,1)=(H(2,1)+H(1,2))/2.0D0!symmetrised
      H(1,2)=H(2,1)!symmetrised

      HP(1,1)=(H(1,1)-S(1)*H(2,1))/T
      HP(1,2)=(H(1,2)-S(1)*H(2,2))/T
      HP(2,1)=(H(2,1)-S(2)*H(1,1))/T
      HP(2,2)=(H(2,2)-S(2)*H(1,2))/T
     
      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE VINV(V,VI)
C     ==--------------------------------------------------------------==
C     == Calculate the inverse transformation matrix                  ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      REAL*8   V(2,2),VI(2,2),T

      T=V(1,1)*V(2,2)-V(1,2)*V(2,1)
      VI(1,1)=V(2,2)/T
      VI(1,2)=-V(1,2)/T
      VI(2,1)=-V(2,1)/T
      VI(2,2)=V(1,1)/T

      RETURN
      END
C     ==================================================================

C     ==================================================================
      FUNCTION ZVDIFF()
C     ==--------------------------------------------------------------==
C     == Calculate the zv difference of Donor and Acceptor group      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cdft.inc'
      INTEGER IT
      REAL*8  RES,ZVDIFF

      RES=0.0D0
      DO IT=1,NACCR
        RES=RES-ZV(SPA(IT))
      ENDDO
      DO IT=1,NDON
        RES=RES+ZV(SPD(IT))
      ENDDO
      ZVDIFF=RES
      RETURN
      END

C     ==================================================================
      SUBROUTINE HDA_PROJECT(CC1,CC2,U,H)
C     ==--------------------------------------------------------------==
C     == Project constrained states on reference states which are     ==
C     == read from RESTART.REF1 and RESTART.REF2                      ==
C     == Parameters: CC1,CC2 ... constrained states                   ==
C     ==             U       ... 2D transform to adiabtic Ham  (in)   ==
C     ==             U       ... 2D transform to diabtic Ham   (out)  ==
C     ==             H       ... adiabatic Hamiltonian  (in)          ==
C     ==             H       ... diabatic Hamiltonian  (out)          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'shop.inc'
      INCLUDE 'filn.inc'
      INCLUDE 'coor.inc'
      INCLUDE 'store.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'cdft.inc'
      COMPLEX*16 CC1(NGW,*),CC2(NGW,*)
      COMPLEX*16 C01(NGW,*),C02(NGW,*)! reference states
      COMPLEX*16 CM(NGW,*)
      REAL*8     CAB(N,N)
      POINTER  (IP_C01,C01),(IP_C02,C02)
      POINTER  (IP_CM,CM)
      POINTER  (IP_CAB,CAB)
      REAL*8     U(2,2),H(2,2),S(2,2)
      INTEGER   IREC(100),NFI
      REAL*8  EIGV(N),TAUI(3,NAX,NSX),N1,N2
      CHARACTER*20 FILN_BACK
      REAL*8    SA,SB,KA1,KB1,KA2,KB2,GA1,GB1,GA2,GB2,SAB
      LOGICAL   RL_SAVE

      CALL MEMORY(IP_C01,2*NGW*N+8,'C01')
      CALL MEMORY(IP_C02,2*NGW*N+8,'C02')
      CALL MEMORY(IP_CAB,N*N,'CAB')
      RL_SAVE=RLATE
      FILN_BACK=FILN
      RLATE=.FALSE.

      CALL READ_IREC(IREC)
      IF(PARENT)WRITE(6,*)"Reading reference states"
      FILN = "RESTART.REF1"
      CALL ZHRWF(1,IREC,C01,CM,N,EIGV,TAU0,VELP,TAUI,NFI)
      FILN = "RESTART.REF2"
      CALL ZHRWF(1,IREC,C02,CM,N,EIGV,TAU0,VELP,TAUI,NFI)

      RLATE=RL_SAVE
      FILN=FILN_BACK

      IF(PARENT)WRITE(6,*)"Projecting constrained wavefunctions to",
     *          " reference states"

!     Get normalisation for reference states
      CALL OVERLAP(C01,C01,CAB,N1)
      CALL OVERLAP(C02,C02,CAB,N2)
      IF(PARENT)THEN 
        WRITE(6,'(2X,"Normalisation of reference states")')
        WRITE(6,'(3X,F14.8,F14.8,/)')N1,N2
      ENDIF


      CALL OVERLAP(CC2,C01,CAB,SB)
      CALL OVERLAP(CC1,C01,CAB,SA)
      IF(PARENT)THEN 
        WRITE(6,'(2X,"Overlap with reference state 1")')
        WRITE(6,'(3X,F14.8,F14.8,/)')SA,SB
      ENDIF
      KA1=U(1,1)*SB/N1+U(1,2)*SA/N1
      KA2=U(2,1)*SB/N1+U(2,2)*SA/N1
      CALL OVERLAP(CC2,C02,CAB,SB)
      CALL OVERLAP(CC1,C02,CAB,SA)
      IF(PARENT)THEN 
        WRITE(6,'(2X,"Overlap with reference state 2")')
        WRITE(6,'(3X,F14.8,F14.8,/)')SA,SB
      ENDIF
      KB1=U(1,1)*SB/N2+U(1,2)*SA/N2
      KB2=U(2,1)*SB/N2+U(2,2)*SA/N2
      CALL OVERLAP(CC1,CC2,CAB,SAB)

      N1=SQRT(U(1,1)*U(1,1)+2.0D0*U(1,1)*U(1,2)*SAB+U(1,2)*U(1,2))
      N2=SQRT(U(2,1)*U(2,1)+2.0D0*U(2,1)*U(2,2)*SAB+U(2,2)*U(2,2))

      GA1=KA1*U(1,1)/N1+KA2*U(2,1)/N2
      GA2=KA1*U(1,2)/N1+KA2*U(2,2)/N2
      GB1=KB1*U(1,1)/N1+KB2*U(2,1)/N2
      GB2=KB1*U(1,2)/N1+KB2*U(2,2)/N2


!     Construct S matrix
      S(1,1)=GB1*GB1+GB2*GB2+2.0D0*GB1*GB2*SAB
      S(2,2)=GA1*GA1+GA2*GA2+2.0D0*GA1*GA2*SAB
      S(1,2)=GA1*GB1+GA2*GB2+(GA1*GB2+GA2*GB1)*SAB
      S(2,1)=S(1,2)

      CALL MAT_ISQRT(S)

      U(1,1)=S(1,1)*KA1+S(1,2)*KB1
      U(1,2)=S(1,1)*KA2+S(1,2)*KB2
      U(2,1)=S(2,1)*KA1+S(2,2)*KB1
      U(2,2)=S(2,1)*KA2+S(2,2)*KB2

!     Transform the adiabatic Hamiltonian
      CALL HTRANS3(H,U)

      RETURN
      END
C     ==================================================================

C     ==================================================================
      SUBROUTINE MAT_ISQRT(U)
C     ==--------------------------------------------------------------==
C     == Calculates the square root of the inverse of a symmetric     ==
C     ==  2x2 Matrix                                                  ==
C     ==--------------------------------------------------------------==
      REAL*8  U(2,2),V(2,2),L(2)

      CALL ODIAG(U,V,L)

      L(1)=1.0D0/SQRT(L(1))
      L(2)=1.0D0/SQRT(L(2))

      U(1,1)=V(1,1)*V(1,1)*L(1)+V(1,2)*V(1,2)*L(2)
      U(2,1)=V(1,1)*V(2,1)*L(1)+V(1,2)*V(2,2)*L(2)
      U(1,2)=U(2,1)
      U(2,2)=V(2,1)*V(2,1)*L(1)+V(2,2)*V(2,2)*L(2)

      RETURN
      END
C     ==================================================================
