C     ==================================================================
      SUBROUTINE TST2MIN_INP(IUNIT)
C     ==--------------------------------------------------------------==
C     ==  Reads Collective Variables Input for Minima Finding         ==
C     ==--------------------------------------------------------------==
C
C     NCOLVAR   :  number of Collective Variables
C     TYCVAR    :  type of Collective Variables
C     ATCVAR    :  atoms which define the Collective Variables
C     CVPAR     :  other parameters that are required for the C.V.
C
C     ==--------------------------------------------------------------==
C     how C.V. definition is read
C
C     DEFINE VARIABLE
C     ==   nfix                                                       ==
C     ==  DIST      n1  n2                                            ==
C     ==  STRETCH   n1  n2                                            ==
C     ==  BEND      n1  n2  n3                                        ==
C     ==  TORSION   n1  n2  n3  n4                                    ==
C     ==  OUTP      n1  n2  n3  n4                                    ==
C     ==  COORD     n1   K  RC                                        ==
C     ==  DIFFER    n1  n2  n3                                        ==
C     ==  COORSP
C     ==  COOR_RF  {2SHELL}
C     ==  BNSWT
C     ==  TOT_COOR  {2SHELL}
C     ==  PLNANG
C     ==  DIFCOOR
C     ==  ...  
C     END DEFINITION
C     ==--------------------------------------------------------------==
C     NCVMIN    :  number of known possible minima
C     CV_MIN2   :  set of CV values for the all the known possible    ==
C                  minuma that could be found
C                  dimension  NCOLVAR*NCVMIN
C     CV_MIN_TOL:  tolerance for accepting the ions configuration as  ==
C                  a minimum configuration (%)
C
C     ==--------------------------------------------------------------==

      IMPLICIT  NONE

      INCLUDE  'system.h'
      INCLUDE  'cotr.inc'
      INCLUDE  'cnst_dyn.inc'
      INCLUDE  'irat.inc'
      INCLUDE  'store.inc'
      INCLUDE  'ions.inc' ! NAT
      INCLUDE  'mm_dim.inc'
      INCLUDE  'mm_input.inc'

C     Arguments
      INTEGER    IUNIT

C     Variable
      INTEGER    IC,I,INUM,II,IA,IE,IND,K,ITYP,NNVAR,NUMSPEC
      CHARACTER  LINE*80,LINE2*120
      CHARACTER  STYP(17)*10
      DATA       STYP /'  STRETCH','    BEND',' TORSION',
     *          'DISTANCE ','    OUTP',  'COORDIN.',
     *          'DIFFEREN.','  COORSP',' COOR_RF',
     &          'BN.SWITCH','TOT C_RF','  RMSD_AB',
     &          '   DISPL1','  COORDIS','  PLNANG',
     &          '  HBONDCH','  DIFCOOR'/
      REAL*8     TOL_TYP(13),AA,BB
      LOGICAL    ERREAD,STATUS

      INTEGER    I1,IOUT,ICV,IMIN,FIRSTDLMN,MY_NAT,IDUMMY
      DATA       FIRSTDLMN/0/
      SAVE       FIRSTDLMN
      DIMENSION  CV_MIN2(MAX_PIPPO,NCVMIN),
     &           CV_TOL_EXT(MAX_PIPPO,NCVMIN),IATDLMN(*)
      CHARACTER  LINEFORM*100,CHNUM*10
      INTEGER    IAA,IEE
C     ==--------------------------------------------------------------==

      LSADPNT  = .TRUE.

C     Default tolerance
      TOL_TYP(1) =  0.01D0  !strech
      TOL_TYP(2) =  1.00D0  !bend
      TOL_TYP(3) =  1.00D0  !torsion
      TOL_TYP(4) =  0.01D0  !distance
      TOL_TYP(5) =  1.00D0  !outp
      TOL_TYP(6) =  0.20D0  !coord
      TOL_TYP(7) =  0.01D0  !differ
      TOL_TYP(8) =  0.03D0  !coord species dependent
      TOL_TYP(9) =  0.03D0  !coord sp. dep. with rational function
      TOL_TYP(10)=  0.03D0  !bond switch  with rat. func.
      TOL_TYP(11)=  0.03D0  !sp. dep. total coord  with rat. func.
      TOL_TYP(12)=  1.00D0
      TOL_TYP(13)=  0.03D0

C     Default
c      CV_MIN_TOL = 0.1D0
      IMINCHECK  = 10
      MAX_MINCHK = 1
      MAX_SEARCH = 1000 
      NCOLVAR    = 0
      ITOL_TYPE = 1

C     ==--------------------------------------------------------------==
C     get total number of atoms and set indexing.
      if(qmmm) then
        call mm_dim(mm_go_mm,status)
        MY_NAT=NAT
      else
        MY_NAT=NATm
      endif

C     ==--------------------------------------------------------------==
   10 CONTINUE
      READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE

      IF(INDEX(LINE,'END').NE.0 .AND. INDEX(LINE,'SADDLE').NE.0)
     &     GOTO 30
      IF(INDEX(LINE,'DEF').NE.0.AND. 
     &             INDEX(LINE,'VARIABLE').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) NNVAR

C     Allocate Memory
        CALL MEMORY(IP_TYCVAR,NNVAR/IRAT+1,'TYCVAR')
        CALL IAZZERO(TYCVAR,NNVAR)
        CALL MEMORY(IP_ATCVAR,15*NNVAR/IRAT+1,'ATCVAR')
        CALL IAZZERO(ATCVAR,15*NNVAR)
        CALL MEMORY(IP_CVPAR,4*NNVAR,'CVPAR')
        CALL AZZERO(CVPAR,4*NNVAR)
        CALL MEMORY(IP_CV_MIN_TOL,NNVAR,'CV_MIN_TOL')
        CALL MEMORY(IP_SPECINDEX,NNVAR/IRAT+1,'SPECINDEX')
        CALL IAZZERO(SPECINDEX,NNVAR)


        GOTO 12
C     Read Type and Atoms for the Collective Variables to be set 
 11     READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
        IF(INDEX(LINE,'END').NE.0 .AND. INDEX(LINE,'DEF').NE.0)
     &     GOTO 10
 12     CONTINUE
        READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
        IF(INDEX(LINE,'END').NE.0 .AND. INDEX(LINE,'DEF').NE.0) THEN
C         Problem the number of coll. variables NCOLVAR .NE. given NFIX.
          WRITE(6,'(1X,A,I5)') 
     &            '! # OF GIVEN C.V. =',NNVAR
          WRITE(6,'(1X,A,I5)') 
     &            '! # OF FOUND C.V.=',NCOLVAR
          WRITE(6,'(1X,A)')    
     &            '! END C.V. IS REACHED.'
          CALL STOPGM('META_COLCAR_INP',
     &            'ERROR WHILE READING FIX STRUCTURES')
C       ==------------------- DISTANCE --------------------------------==
        ELSEIF(INDEX(LINE,'DIST').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 4
          I1=INDEX(LINE,'DIST')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          CV_MIN_TOL(NCOLVAR) = TOL_TYP(4)
C       ==--------------- DIFFERENCE AMONG DISTANCES ------------------==
        ELSEIF(INDEX(LINE,'DIFFER').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 7
          I1=INDEX(LINE,'DIFFER')+6
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(3,NCOLVAR)=NAT_cpmd(IDUMMY)
          CV_MIN_TOL(NCOLVAR) = TOL_TYP(7)
C        ==------------------ STRETCH ----------------------------------==
        ELSEIF(INDEX(LINE,'STRETCH').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 1
          I1=INDEX(LINE,'STRETCH')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          CV_MIN_TOL(NCOLVAR) = TOL_TYP(1)
C        ==------------------  BEND ------------------------------------==
        ELSEIF(INDEX(LINE,'BEND').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 2 
          I1=INDEX(LINE,'BEND')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(3,NCOLVAR)=NAT_cpmd(IDUMMY)
          CV_MIN_TOL(NCOLVAR) = TOL_TYP(2)
C        ==------------------ TORSION ANGLE  ----------------------------==
        ELSEIF(INDEX(LINE,'TORSION').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 3
          I1=INDEX(LINE,'TORSION')+7
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(3,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(4,NCOLVAR)=NAT_cpmd(IDUMMY)
          CV_MIN_TOL(NCOLVAR) = TOL_TYP(3)
C        ==------------------ OUT OF LPANE ANGLE ------------------------==
        ELSEIF(INDEX(LINE,'OUTP').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 5
          I1=INDEX(LINE,'OUTP')+4
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(3,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(4,NCOLVAR)=NAT_cpmd(IDUMMY)
          CV_MIN_TOL(NCOLVAR) = TOL_TYP(5)
C       ==------------------ COORDINATION NUMBER ------------------------==
        ELSEIF(INDEX(LINE,'COORD').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 6
          I1=INDEX(LINE,'COORD')+5
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
          CV_MIN_TOL(NCOLVAR) = TOL_TYP(6)
C       ==------------ SPECIES DEPENDENT COORDINATION NUMBER  -----------==
C       ==                 f = 1/(1+exp(k*(R-R_0)))                      ==
        ELSEIF(INDEX(LINE,'COORSP').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 8
          I1=INDEX(LINE,'COORSP')+6
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
C       ==------------ SPECIES DEPENDENT COORDINATION NUMBER -------------==
C       ==            f =sum_i (1+(R/R_0)^n)/(1+(R/R_0)^(n+m))
        ELSEIF(INDEX(LINE,'COOR_RF').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 9
          I1=INDEX(LINE,'COOR_RF')+7
          IF(INDEX(LINE,'INDAT') .NE. 0) THEN 
            SPECINDEX(NCOLVAR) = 0
            I1 = INDEX(LINE,'INDAT') + 5
          ELSE
            SPECINDEX(NCOLVAR) = 1
          ENDIF
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          IF(SPECINDEX(NCOLVAR) .EQ. 0) THEN
            IF(FIRSTDLMN .EQ. 0) THEN
              CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
              CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
              FIRSTDLMN = 1
            ENDIF

            READ(IUNIT,*)  (IATDLMN(II+MY_NAT*(NCOLVAR-1)),
     &                           II=1,ATCVAR(2,NCOLVAR))
 
          ENDIF
          II=INDEX(LINE,'2SHELL')
          IF(II.NE.0) THEN
            CALL READSI(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
            IA=ATCVAR(3,NCOLVAR)
            IE=ATCVAR(4,NCOLVAR)
            IF(MOD(IA,2).NE.0 .OR. MOD(IE,2).NE.0 ) THEN
              CALL STOPGM('TST2MIN_INP',
     &        'for COOR_RF with 2nd shell use even exponents')
            ENDIF
          ENDIF
C       ==--------------- BOND SWITCH WITH RATIONAL F -------------------==
C       ==            f = (1+(R/R_0)^n)/(1+(R/R_0)^(n+m))
        ELSEIF(INDEX(LINE,'BNSWT').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 10
          I1=INDEX(LINE,'BNSWT')+5
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
C       ==--------- TOTAL SPECIES DEPENDENT COORDINATION NUMBER ---------==
C       ==            f =SUM_j[sum_i (1+(R/R_0)^n)/(1+(R/R_0)^(n+m))]    ==
        ELSEIF(INDEX(LINE,'TOT_COOR').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 11
          I1=INDEX(LINE,'TOT_COOR') + 8
          IF(INDEX(LINE,'INDAT') .NE. 0) THEN 
            SPECINDEX(NCOLVAR) = 0
            I1 = INDEX(LINE,'INDAT') + 5
          ELSE
            SPECINDEX(NCOLVAR) = 1
          ENDIF
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          IF(SPECINDEX(NCOLVAR) .EQ. 0) THEN
            IF(FIRSTDLMN .EQ. 0) THEN
              CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
              CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
              FIRSTDLMN = 1
            ENDIF

            READ(IUNIT,*)  (IATDLMN(II+MY_NAT*(NCOLVAR-1)),
     &                           II=1,ATCVAR(2,NCOLVAR))
          ENDIF
          II=INDEX(LINE,'2SHELL')
          IF(II.NE.0) THEN
            CALL READSI(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
            IA=ATCVAR(3,NCOLVAR)
            IE=ATCVAR(4,NCOLVAR)
            IF(MOD(IA,2).NE.0 .OR. MOD(IE,2).NE.0 ) THEN
              CALL STOPGM('M_COLVAR_INP',
     &        'for TOT_COOR with 2nd shell use even exponents')
            ENDIF
          ENDIF
C       ==------- RMSD wrt 2 configurations A and B (read from file)-----==
C       ==                f = (RMSD_A-RMSD_B)/(RMSD_A+RMSD_B)            ==
        ELSEIF(INDEX(LINE,'RMSD_AB').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 12
          IF(.NOT. TRMSD_AB) THEN
            IF(MAX_NNVAR.LT.NNVAR) THEN
              IF(PARENT) WRITE(6,*) "NNVAR:",NNVAR,
     &                             "  MAX_NNVAR:",MAX_NNVAR
              CALL STOPGM("FILE_STR_AB",
     &                    "PARAMETER MAX_NNVAR too small")
            ENDIF

            CALL MEMORY(IP_ICV_RMSD_AB,NNVAR/IRAT+1,'ICV_RMSD_AB')
            CALL IAZZERO(ICV_RMSD_AB,NNVAR)
            TRMSD_AB=.TRUE.
            NRMSD_AB = 0
          ENDIF
          ICV_RMSD_AB(NCOLVAR)=NCOLVAR
          NRMSD_AB  = NRMSD_AB + 1

          I1=INDEX(LINE,'RMSD_AB') + 7
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          IF(ATCVAR(1,NCOLVAR) .GT. 13) CALL STOPGM('M_COLVAR_INP',
     &        'RMSD_AB! too many species (>13)')
          NUMSPEC = ATCVAR(1,NCOLVAR)
          DO I = 1,NUMSPEC
            CALL READSI(LINE,I1,IOUT,ATCVAR(I+1,NCOLVAR),ERREAD)
            I1=IOUT
          ENDDO
          II=INDEX(LINE,'FILEAB')
          IF(II.NE.0) THEN
            IA = II+6
            LINE2  = LINE(IA:IA+20)
            CALL XSTRING(LINE2,IA,IE)
            FILE_STR_AB(NCOLVAR) = LINE2(IA:IE)
          ELSE
            FILE_STR_AB(NCOLVAR) = 'STRUCTURE_AB'
          ENDIF
          ATCVAR(NUMSPEC+2,NCOLVAR) = NRMSD_AB
C       ==---- Angle between 2 planes, each defined by 3 given pnts  -----==
        ELSEIF(INDEX(LINE,'PLNANG').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 15
c          IANGCV(NCOLVAR) = 2
          I1=INDEX(LINE,'PLNANG')+6
          IANGCV(NCOLVAR) = 1
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(1,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(2,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(3,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(4,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(5,NCOLVAR)=NAT_cpmd(IDUMMY)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,IDUMMY,ERREAD)
          ATCVAR(6,NCOLVAR)=NAT_cpmd(IDUMMY)
C       ==------------ DIFFERENCE OF COORDINATION NUMBERS -------------==
C       ==  f =sum_i (1-(Rbi/R_0)^n)/(1-(Rbi/R_0)^(n+m))-
C       ==     sum_i (1-(Rai/R_0)^n)/(1-(Rai/R_0)^(n+m))
        ELSEIF(INDEX(LINE,'DIFCOOR').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 17
          I1=INDEX(LINE,'DIFCOOR')+7
          IF(INDEX(LINE,'INDAT') .NE. 0) THEN 
            SPECINDEX(NCOLVAR) = 0
            I1 = INDEX(LINE,'INDAT') + 5
          ELSE
            SPECINDEX(NCOLVAR) = 1
          ENDIF
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(5,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          IF(SPECINDEX(NCOLVAR) .EQ. 0) THEN
            IF(FIRSTDLMN .EQ. 0) THEN
              CALL MEMORY(IP_IATDLMN,MY_NAT*NNVAR/IRAT+1,'IATDLMN')
              CALL IAZZERO(IATDLMN,MY_NAT*NNVAR)
              FIRSTDLMN = 1
            ENDIF
            READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE2
            I1=1
            DO II = 1,ATCVAR(3,NCOLVAR)
              CALL READSI(LINE2,I1,IOUT,
     &               IATDLMN(II+MY_NAT*(NCOLVAR-1)),ERREAD)
                  I1=IOUT
            ENDDO
          ENDIF
C       ==------------ COORDINATION OF SECOND NEIGHBORS -------------==
C       ==    f =sum_jik [(1+(Rij/R_0)^n)/(1+(Rij/R_0)^(n+m))*       ==
C       ==                1+(Rjk/R_0)^n)/(1+(Rjk/R_0)^(n+m))]/NA/NB  == 

        ELSEIF(INDEX(LINE,'COOR_CHAIN').NE.0) THEN
          NCOLVAR = NCOLVAR + 1
          TYCVAR(NCOLVAR) = 18
          I1=INDEX(LINE,'COOR_CHAIN')+10 
          CALL READSI(LINE,I1,IOUT,ATCVAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(2,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(3,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(4,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSI(LINE,I1,IOUT,ATCVAR(5,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(1,NCOLVAR),ERREAD)
          I1=IOUT
          CALL READSR(LINE,I1,IOUT,CVPAR(2,NCOLVAR),ERREAD)
        ENDIF

        IF(NCOLVAR.GT.NNVAR) THEN
C         Problem the number of coll.variables NCOLVAR .NE. given NFIX.
          WRITE(6,'(1X,A,I5)') 
     &         'TST2MIN_INP! # OF GIVEN COL. VAR= ',NNVAR
          WRITE(6,'(1X,A,I5)') 
     &         'TST2MIN_INP! # OF FOUND COL. VAR=',NCOLVAR
          WRITE(6,'(1X,A)')
     &         'THE NUMBER OF GIVEN COL. VAR. IS TOO SMALL.'
          CALL STOPGM('TST2MIN_INP',
     &         'ERROR WHILE READING FIX STRUCTURES')
        ELSEIF(NCOLVAR.GT. MAX_PIPPO) THEN
          CALL STOPGM('TST2MIN_INP','NCOLVAR.GT. MAX_PIPPO')
        ELSEIF(NCOLVAR.LT.NNVAR) THEN
          GOTO 12
        ELSE
          GOTO 11
        ENDIF
      ELSEIF(INDEX(LINE,'KNOWN_MINIMA').NE.0) THEN
        ITOL_TYPE = 1
        IF(INDEX(LINE,'EXTENDED').NE.0) ITOL_TYPE = 2

        READ(IUNIT,ERR=20,END=20,FMT=*) NCVMIN

C     Allocate Memory
        CALL MEMORY(IP_CV_MIN2,NCVMIN*MAX_PIPPO,'CV_MIN')
        CALL AZZERO(CV_MIN2,NCVMIN*MAX_PIPPO)

        IF(NCOLVAR.EQ.0) CALL STOPGM('TST2MIN_INP',
     &      'SET of KNOWN MIN.: # of CV=0, CV to be defined before ')

        write(6,*) 'ITOL_TYPE ',ITOL_TYPE
        IF(ITOL_TYPE .EQ. 1) THEN
          DO IMIN = 1,NCVMIN
            READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
            I1 = 1
c           read minima
            DO ICV = 1,NCOLVAR
              CALL READSR(LINE,I1,IOUT,CV_MIN2(ICV,IMIN),ERREAD)
              I1 = IOUT
            ENDDO

          ENDDO
        ELSEIF(ITOL_TYPE .EQ. 2) THEN
          CALL MEMORY(IP_CV_TOL_EXT,NCVMIN*MAX_PIPPO,'CV_TOL_EXT')
          CALL AZZERO(CV_TOL_EXT,NCVMIN*MAX_PIPPO)

          DO IMIN = 1,NCVMIN
            READ(IUNIT,ERR=20,END=20,FMT='(A)') LINE
            I1 = 1
c           read minima
            DO ICV = 1,NCOLVAR
              CALL READSR(LINE,I1,IOUT,CV_MIN2(ICV,IMIN),ERREAD)
              I1 = IOUT
            ENDDO
c           read tolerance
            DO ICV = 1,NCOLVAR
              CALL READSR(LINE,I1,IOUT,CV_TOL_EXT(ICV,IMIN),ERREAD)
             I1 = IOUT
            ENDDO
c            write(6,*) imin, (CV_MIN2(ICV,IMIN),ICV = 1,NCOLVAR),
c     &                       (CV_TOL_EXT(ICV,IMIN),ICV = 1,NCOLVAR)
          ENDDO

        ENDIF
        GOTO 10
      ELSEIF(INDEX(LINE,'SADDLE').NE.0 .AND.
     &             INDEX(LINE,'TOLERANCE') .NE.0) THEN

        IF(NCOLVAR.EQ.0) CALL STOPGM('TST2MIN_INP',
     &      'SET of TOLERANCE.: # of CV=0, CV to be defined before ')

          READ(IUNIT,ERR=20,END=20,FMT=*) 
     &            (CV_MIN_TOL(ICV),ICV = 1,NCOLVAR)
c          write(6,*) 'tol ',(CV_MIN_TOL(ICV),ICV = 1,NCOLVAR)
        GOTO 10
      ELSEIF(INDEX(LINE,'STEPCHECK').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) LINE
        CALL READSI(LINE,1,IE,IMINCHECK,ERREAD)
        IF(ERREAD) GOTO 20
        GOTO 10
      ELSEIF(INDEX(LINE,'MAXCHECK').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) LINE
        CALL READSI(LINE,1,IE,MAX_MINCHK,ERREAD)
        IF(ERREAD) GOTO 20
        GOTO 10
      ELSEIF(INDEX(LINE,'MAXSEARCH').NE.0) THEN
        READ(IUNIT,ERR=20,END=20,FMT=*) LINE
        CALL READSI(LINE,1,IE,MAX_SEARCH,ERREAD)
        IF(ERREAD) GOTO 20
        GOTO 10
      ELSE
        GOTO 10
      ENDIF  !file biginning

   20 CONTINUE
        WRITE(6,*) ' ERROR WHILE READING METADYNAMICS VARIABLES '
        CALL STOPGM('TST2MIN_INP',' ')      
   30 CONTINUE
C     ==--------------------------------------------------------------==
C     TEST consistencies

      LMDREINIT   = .FALSE.
      IF(NCVMIN .EQ. 0) CALL STOPGM('TST2MIN_INP',' NO MINIMA DEFINED')
C     ==--------------------------------------------------------------==
C     Print initialization

      WRITE(6,*) 

      WRITE(6,'(1x,A,A)') 
     &         '***           FROM SADDLE TO MINIMA',
     &         '               ***'

      WRITE(6,'(5x,A,I5)')   '- Number of known minima = ',NCVMIN
      WRITE(6,'(5x,A,I5,A)') '- Check performed each ',
     &                            IMINCHECK,' MD step'
      WRITE(6,'(5x,A,I8)') '- Max. # of checks per search ', MAX_MINCHK
      WRITE(6,'(5x,A,I8)') '- Number of searching runs ',MAX_SEARCH

      WRITE(6,*)
      WRITE(6,'(5X,A,10X,A)') 'TYPE','PARAMETERS'
      DO I=1,NCOLVAR
        ITYP=TYCVAR(I)
        IF(ITYP .EQ. 6) THEN
          WRITE(6,'(A,1X,I5,1X,2f8.4)') STYP(ITYP),
     *         ATCVAR(1,I),CVPAR(1,I),CVPAR(2,I)
        ELSEIF(ITYP.EQ.8) THEN
          WRITE(6,'(A,1X,2I5,10X,2f8.4)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),CVPAR(1,I),CVPAR(2,I)
        ELSEIF(ITYP .GE.9 .AND. ITYP.LE.11) THEN
          WRITE(6,'(A,1X,2I4,6X,2I4,f8.5)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &         CVPAR(1,I)
        ELSEIF(ITYP.EQ.15) THEN
          WRITE(6,'(A,1X,6I5)') STYP(ITYP),
     *         (ATCVAR(K,I),K=1,6)
        ELSEIF(ITYP .EQ. 17) THEN
          WRITE(6,'(A,1X,3I4,2x,2I3,f7.3)') STYP(ITYP),
     *         ATCVAR(1,I),ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I),
     &         ATCVAR(5,I),CVPAR(1,I)
        ELSE
          WRITE(6,'(A,3X,4I6)') STYP(ITYP),ATCVAR(1,I),
     *         ATCVAR(2,I),ATCVAR(3,I),ATCVAR(4,I)
        ENDIF
      ENDDO
      WRITE(6,*)
      WRITE(CHNUM,'(I5)') NCOLVAR
      CALL XSTRING(CHNUM,IAA,IEE)
      LINEFORM = '(1X,I5,'//CHNUM(IAA:IEE)//'f16.8)'
      WRITE(6,'(A)') '       KNOWN MINIMA     '      
      DO IMIN = 1,NCVMIN
        write(6,LINEFORM) imin, (CV_MIN2(ICV,IMIN),ICV = 1,NCOLVAR)
      ENDDO
      WRITE(6,*)
      IF(ITOL_TYPE .EQ. 2) THEN
        WRITE(6,'(A)') '       TOLERANCE PER MINIMUM       '
        DO IMIN = 1,NCVMIN
          write(6,LINEFORM) imin, (CV_TOL_EXT(ICV,IMIN),ICV = 1,NCOLVAR)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      if(qmmm) call mm_dim(mm_revert,status)
      RETURN
      END
C     ==================================================================
