C     ==================================================================
      SUBROUTINE U_UPD_EXP_IDE(U_MAT,Z_MAT,XYZMAT,U_GRAD,SCR,
     &                  LSCR,NSTATE)
C     ==--------------------------------------------------------------==
C     == CALCULATES THE UNITARY TRANSFORMATION 
C     ==            WHICH OPTIMISE LOCALIZATION IN G SPACE            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      include 'spin.inc'
      include 'g_loc.inc'

c     Arguments
      INTEGER       NSTATE,LSCR
      COMPLEX*16    U_MAT(NSTATE,NSTATE),Z_MAT(NSTATE,NSTATE,3),
     &              U_GRAD(NSTATE,NSTATE),XYZMAT(*)
      REAL*8        SCR(LSCR)

c     Variables
      COMPLEX*16    A(NSTATE,NSTATE),B1(NSTATE,NSTATE),B2(NSTATE,NSTATE)
      pointer       (IP_A,A),(IP_B1,B1),(IP_B2,B2)

      DIMENSION     OMEGA_N(*)

      INTEGER       L_A,L_B,L_GAM,LSCR2,IZAMAX,ISUB
      REAL*8        DELTA,DIFGAM
      COMPLEX*16    U_TEMP(NSTATE,NSTATE),GAM(NSTATE*NSTATE)
      pointer       (IP_U_TEMP,U_TEMP),(IP_GAM,GAM)
      COMPLEX*16    ZONE,ZZERO,ZGIVE,CARG
      EXTERNAL      ZGIVE
      INTEGER       I,J,IX1,IX2,IXL,IMAX,ITER,LENGTH
      CHARACTER*30  TAG
      REAL*8        GRMAX,FAC,COMP,DIFF

      INTEGER       IGRMAX,NEXP,ICONT
      DATA          ICONT / 0 /
      SAVE          ICONT
      REAL*8        STEP_FAC,OFUN00
      DATA          OFUN00 / 0.0D0 /
      DATA          STEP_FAC / 1.0D0 /
      SAVE          STEP_FAC,OFUN00

C     ==-------------------------------------------------------------==
      CALL TISET(' UPD_EXP_I',ISUB)
      CALL GIVE_SCR_UPD_EXP_IDE(LENGTH,TAG,NSTATE)
      CALL TEST_SCR('U_UPD_EXP_IDE','LENGTH',LSCR,LENGTH)

      IF(TLSD)  CALL STOPGM('TRANUPD','LSD NOT IMPLEMENTED YET')

      L_A        = 2*NSTATE*NSTATE
      L_B        = 2*NSTATE*NSTATE
      L_GAM      = 2*NSTATE*NSTATE

      CALL GET_ADDR(IP_A     ,SCR(1))
      CALL GET_ADDR(IP_B1    ,SCR(1+L_A))
      CALL GET_ADDR(IP_B2    ,SCR(1+L_A+L_B))
      CALL GET_ADDR(IP_GAM   ,SCR(1+L_A+2*L_B))

      IX1 = 1+L_A+2*L_B+L_GAM
      LSCR2 = LSCR - (L_A+2*L_B+L_GAM)

      CALL GET_ADDR(IP_U_TEMP,U_GRAD)

      ZONE  = DCMPLX(1.0D0,0.0D0)
      ZZERO = DCMPLX(0.0D0,0.0D0)

c     time step
      DELTA = GLOC_STEP * STEP_FAC

c     this is U_MAT

c     Up_MAT is zero if first step
      CALL AZZERO(SCR,LSCR)
c      CALL AZZERO(A,L_A)
c      CALL AZZERO(B1,L_B)
c      CALL AZZERO(B2,L_B)

      NSTEP = NSTEP + 1

      IGRMAX=IZAMAX(NSTATE*NSTATE,U_GRAD,1)
      GRMAX = CDABS(ZGIVE(U_GRAD,IGRMAX))


      IF(GLOC_CONST .EQ. 3) THEN

C     UPDATE U_MAT WITH APPROXIMATION U = 1 + A (A antihermitian)
         CALL APPROX_U(A,U_GRAD,NSTATE)
         CALL DCOPY(2*NSTATE*NSTATE,A,1,U_TEMP,1)

      ELSEIF(GLOC_CONST .EQ. 2) THEN

        IF(TG_LINESEARCH) THEN
          IX1 = 1+L_A+2*L_B
          LSCR2 = LSCR - (L_A+2*L_B)
          CALL LINE_SEARCH_EXE(A,U_GRAD,Z_MAT,XYZMAT,B1,B2,
     &                      SCR(IX1),LSCR2,NSTATE,DELTA,STEP_FAC)
        ELSE
          IX1 = 1+L_A+2*L_B
          LSCR2 = LSCR - (L_A+2*L_B)
          CALL U_BY_ORTHO(A,U_GRAD,B1,B2,SCR(IX1),LSCR2,NSTATE,DELTA)
        ENDIF
        CALL DCOPY(2*NSTATE*NSTATE,A,1,U_TEMP,1)

      ELSEIF(GLOC_CONST .EQ. 1) THEN

c     update U_MAT_new =  U_MAT_old + dt U_GRAD

        DO I=1,NSTATE
          DO J=1,NSTATE
            U_TEMP(J,I)=U_MAT(J,I)+DELTA*U_GRAD(J,I)
          ENDDO
        ENDDO

        CALL  LAGRMULT(U_MAT,U_TEMP,A,B1,B2,GAM,SCR(IX1),LSCR2,NSTATE)
      
      ENDIF  !APPROX

      GMAX = 0.0D0
      GGNORM = 0.0D0
      DO I = 1,NSTATE
         DO J = 1,NSTATE
            CARG = U_TEMP(J,I)-U_MAT(J,I)
            DIFGAM=DREAL(CARG)*DREAL(CARG)+DIMAG(CARG)*DIMAG(CARG)
            GGNORM = GGNORM + DIFGAM
            GMAX = MAX(GMAX,DIFGAM)
         ENDDO
      ENDDO
      GMAX = dsqrt(GMAX)
      GGNORM = DSQRT(GGNORM)/DBLE(NSTATE)


      CALL DCOPY(2*NSTATE*NSTATE,U_TEMP(1,1),1,U_MAT(1,1),1)

c     calculate U_GRAD_new with the rotated U_MAT_new
      CALL AZZERO(SCR,LSCR)
      CALL Z_UPDATE(U_MAT,Z_MAT,SCR(1),NSTATE,NSTATE,3)
      CALL GZFUNC(Z_MAT,NSTATE,NSTATE)

c      IF(.NOT. TG_LINESEARCH) THEN
c        DIFF = OFUN - OFUN0
c        write(6,*) DIFF,OFUN,OFUN0
c        stop
c        IF(DABS(DIFF) .GT. GNORM) THEN 
c           STEP_FAC = STEP_FAC * (1.0D0 - GNORM/DIFF)
c        ELSE
c           write(6,'(/,A,I10,A,1PE12.6,/)')
c     &     '   U_UPD: WARNING ... at step ',NSTEP,
c     &     '  DIFF_OFUN = ', DIFF
c        ENDIF
c      ENDIF

      DIF_FUN = DABS(OFUN - OFUN00)
      OFUN00 = OFUN        
      IF(TG_ANTISYMM .AND. ICONT .LE. 100  )  THEN 
        ICONT = ICONT+1
        write(10,'(I10,3(2x,1PE12.6),1PE12.6,2x,1PE12.6,1x,f10.4)') 
     &     NSTEP,GMAX,OFUN,GRMAX,DIF_FUN,OMEGA_TOT,STEP_FAC
        write(12,'(I10,16f10.4)') NSTEP,(OMEGA_N(I),I=1,16)
      ELSE
        write(10,'(I10,3(2x,1PE12.6),1PE12.6,2x,1PE12.6,f10.5)') 
     &          NSTEP,GMAX,OFUN,GRMAX,DIF_FUN,GGNORM,STEP_FAC/10.0D0

      ENDIF

C     ==--------------------------------------------------------------==
      
      CALL TIHALT(' UPD_EXP_I',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE  U_GRAD_EXP_IDE(U_MAT,U_GRAD,Z_MAT,NSTATE)
C     ==--------------------------------------------------------------==
C     ==  CALCULATES THE GRADIENT OF THE LOCALIZATION FUNCTIONAL      ==
C     ==  WITH RESPECT TO THE UNITARY MATRIX COEFFICIENTS             ==
C     ==  U_GRAD_ij = -d OFUNC / d U_ij                               ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      include 'g_loc.inc'
      include 'spin.inc'
C input
      INTEGER    LSCR,NSTATE
      COMPLEX*16 U_MAT(NSTATE,NSTATE),U_GRAD(NSTATE,NSTATE),
     &           Z_MAT(NSTATE,NSTATE,3)
      REAL*8     FAC

C local
      INTEGER    I,J,K,L

C     ==--------------------------------------------------------------==


      FAC = 1.0D0/6.0D0
      DO K = 1,3
        DO J = 1,NSTATE
          DO I = 1,NSTATE
            U_GRAD(I,J) = U_GRAD(I,J)
     &                  +Z_MAT(I,J,K)*DCONJG(Z_MAT(J,J,K)) 
     &                  +Z_MAT(J,J,K)*DCONJG(Z_MAT(J,I,K))
          ENDDO
        ENDDO
      ENDDO

      DO I = 1,NSTATE
        U_GRAD(I,I) = U_GRAD(I,I) - 6.0D0
      ENDDO
c     FAC = 10.D0
c      DO I = 1,NSTATE
c        DO J = 1,NSTATE
c        IF(J .NE. I) THEN
c        U_GRAD(J,I) = U_GRAD(J,I)*FAC
c        ENDIF
c        ENDDO
c      ENDDO

C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
      SUBROUTINE GIVE_SCR_UPD_EXP_IDE(LTRANUPD,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'g_loc.inc'

C     Arguments
      INTEGER   LGRAD,L_ORTHO,L_MULTI,LZNUM,NSTATE,LTRANUPD,L_SEARCH,
     &          L_PRINT
      CHARACTER TAG*30
C     Variables

      CALL GIVE_SCR_ZNUM_MAT(LZNUM,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      LGRAD     = 2*NSTATE*NSTATE
      L_ORTHO   = LGRAD+3*2*NSTATE*NSTATE + 11*NSTATE
      L_MULTI   = LGRAD+6*2*NSTATE*NSTATE
      IF(TG_LINESEARCH) THEN 
        L_SEARCH   = LGRAD + 9*2*NSTATE*NSTATE
      ELSE
        L_SEARCH   = 1
      ENDIF
      IF(TGLOCREALP) THEN 
        L_PRINT = KR1*KR2S*KR3S + KR2S*KR3S
      ELSE
        L_PRINT =1
      ENDIF
C     ==--------------------------------------------------------------==
      LTRANUPD  = MAX(L_ORTHO,L_MULTI,LZNUM,L_SEARCH,L_PRINT)
      TAG='MAX(L_ORTHO,L_MULTI,LZNUM,...)'
      RETURN
      END
C     ==--------------------------------------------------------------==
C     ==================================================================
C     ==================================================================
