C     ==================================================================
      SUBROUTINE UP3_P(C0,CU1,TAU0,
     &     PSI,SCR,LSCR,
     &     NSTATE,TFOR,TSTRESS)
C     ==--------------------------------------------------------------==
C     ==          Computes:                                           ==
C     ==          New Set of basis wafeunctions, depending on k       ==
C     ==          linear combinations of the C0 and the C1            ==
C     ==          where the latter are obtained by the perturbation   ==
C     ==          due to the k point contribution to                  ==
C     ==             the standard Hamiltonian, considered in Gamma.   ==
C     ==          the perturbation contribution are imaginary in the  ==
C     ==          real space and depend on k: i * k * C1              ==
C     ==          even if in the perturbation algorithm the C1 are    ==
C     ==          considered as coefficient of real functions, and    ==
C     ==          they depend on the direction x, y, or z             ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'andp.inc'
      INCLUDE 'norm.inc'
      INCLUDE 'ropt.inc'
      INCLUDE 'soft.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'ener.inc'        !ETOT,EHT,EEIG,EXC,VXC
      INCLUDE 'elct.inc'        !NEL
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'store.inc'
      INCLUDE 'elct2.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'sphe.inc'
C     INCLUDE 'coor.inc'
      INCLUDE 'response_p.inc'
C     Arguments
      INTEGER    NSTATE,NFR,NHPSI,LSCR
      COMPLEX*16 C0(NGW,NSTATE),CU1(ngw,nstate,3),
     &     PSI(NNR1)
      REAL*8     TAU0(3,NAX,NSX),
     &     SCR(LSCR)
      LOGICAL    TFOR,TSTRESS
C     Variables
      DIMENSION  WK(NKPTS),RK(3,nkpts),F(NSTATE),
     &     RIN0(NNR1),ROUT0(NNR1),
     &     EMK2(*)
      COMPLEX*16    C00(2*NGW,2*NSTATE,NKPTS)
      COMPLEX*16    C11(2*NGW,2*NSTATE)
      pointer      (ip_C00,C00), (IP_C11,C11)
      REAL*8       EV(2*NSTATE,NKPTS)
      pointer     (ip_ev,ev)

      REAL*8     FNL(*),DFNL(*)
      REAL*8     EEIG1,SUM,RSUM,DDOT
      EXTERNAL   DDOT

      CHARACTER  TAG*30
      LOGICAL    TREFINE,TADJMU
      INTEGER    IDAMAX,NNX,NNXS,IKIND,IERR,
     &     I,IMAX,NCONV,LUPDRHO_P,IFIRST
C     
      integer    noccup,istat,jstat,isp,iv,isa,ia,j,IG
      INTEGER    IREC(100)

      COMPLEX*16 CTM,DDK,ZZERO,ZONE
      COMPLEX*16 ADAV(2*NSTATE,2*NSTATE)
      pointer    (IP_ADAV,ADAV)
      PARAMETER  (ZZERO=(0.D0,0.D0),ZONE=(1.D0,0.D0))
      real*8    sc0(1)
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_UP3_P(LUPDRHO_P,TAG,NSTATE)
      CALL TEST_SCR('UP3_P',TAG,LSCR,LUPDRHO_P)
C     ==--------------------------------------------------------------==
C     New Hilbert space, spanned by the 2*NSTATE wavefunction
C     This set is constructed separately per each K point
C     
      CALL MEMORY(ip_C00,4*NGW*2*NSTATE*NKPTS,'C00')
      CALL ZAZZERO(C00,2*NGW*2*NSTATE*NKPTS)
      CALL MEMORY(ip_C11,4*NGW*2*NSTATE,'C11')
      CALL ZAZZERO(C11,2*NGW*2*NSTATE)
      CALL MEMORY(IP_EV,2*NSTATE*NKPTS,'EV')
      CALL AZZERO(EV,2*NSTATE*NKPTS)
      CALL MEMORY(ip_ADAV,2*2*NSTATE*2*NSTATE,'ADAV')
      CALL ZAZZERO(ADAV,4*NSTATE*NSTATE)
C     ==--------------------------------------------------------------==
C     == ALLOCATE ARRAYS  for the new wavefunction                    ==
C     ==--------------------------------------------------------------==

      NNX=NNR1
      TKPNT = .true.
      TKSCALE = .false.
      TSPHERE = .false.

      N = 2*NSTATE
      if(parent) write(6,'("NGWKS 1 = ",i10)') NGWKS
      NGWK = 2*NGW 
      NGWKS = 2*NGWS
      if(parent) write(6,'("NGWKS = ",i10)') NGWKS
      if(parent) write(6,'("NHGS = ",i10)') NHGS
      if(parent) write(6,'("NHGLS = ",i10)') NHGLS


C     ==--------------------------------------------------------------==
C     == Calculate the Stress Tensor                                  ==
C     ==--------------------------------------------------------------==

      noccup = NSTATE

C     ==-------------------------------------------------------------==
C     Preparation of the projector for the non local potential
C     which depend on K point

      CALL RKPNT


      DO IKIND = 1, NKPTS

         DO I =1,NSTATE
            DO IG = 1,NGW
               C00(IG,I,IKIND) = C0(IG,I)
               C00(IG+NGW,I,IKIND)=DCONJG(C0(IG,I))
            ENDDO
         ENDDO

C     For each rk(1:3,1:nkpts) a new HAMILK is created
C     It is a block matrix, along the diagonal 

         CALL ZAZZERO(C00(1,NSTATE+1,IKIND),2*NGW*NSTATE)
         DO ISTAT = 1, NSTATE
            DO IG = 1, NGW
               C00(IG,ISTAT+NSTATE,IKIND) = 
     &              uimag*( CU1(IG,ISTAT,1)*RK(1,IKIND) +
     &              CU1(IG,ISTAT,2)*RK(2,IKIND)  +
     &              CU1(IG,ISTAT,3)*RK(3,IKIND) )*TPIBA
               C00(IG+NGW,ISTAT+NSTATE,IKIND) =
     &              -DCONJG(C00(IG,ISTAT+NSTATE,IKIND)) 
            ENDDO
         ENDDO 

      IF(GEQ0) CALL ZCLEAN_K(C00(1,1,IKIND),2*NSTATE,NGW)


c         DO ISTAT = 1,2*NSTATE
c            sum =0.0d0
c            DO IG = 1,2*NGW
c               sum = sum + 
c     &              dreal(c00(IG,ISTAT,IKIND))*
c     &              dreal(c00(IG,ISTAT,IKIND))+
c     &              dimag(c00(IG,ISTAT,IKIND))*
c     &              dimag(c00(IG,ISTAT,IKIND))
c            ENDDO
c            write(6,'("mod stat ",i5," = ",f14.8)') ISTAT, sum
c         ENDDO



         
c         write(6,'("costruiti c00 per k =  ",i4)') ikind 


         CALL GS_ORTHO_C(C00(1,1,IKIND),NSTATE,
     &        C00(1,NSTATE+1,IKIND),NSTATE,ADAV,SCR) 



c         DO ISTAT = 1,2*NSTATE
c            sum =0.0d0
c            DO IG = 1,2*NGW
c               sum = sum + 
cc     &              dreal(c00(IG,ISTAT,IKIND))*
c     &              dreal(c00(IG,ISTAT,IKIND))+
c     &              dimag(c00(IG,ISTAT,IKIND))*
c     &              dimag(c00(IG,ISTAT,IKIND))
c            ENDDO
c            write(6,'("mod stat ",i5," = ",f14.8)') ISTAT, sum
c         ENDDO


      ENDDO                     !  LOOP OVER IKIND


C     ==--------------------------------------------------------------==
C     == End loop over kpoints                                        ==
C     ==--------------------------------------------------------------==
      RSUM =0.D0
      DO IKIND = 1,NKPTS
         IF(PARENT) THEN
            WRITE(6,'(a,i4,a,f14.6)') 
     &           "WK(",IKIND,")= ",WK(IKIND)
         ENDIF
         DO I=1,NSTATE
            IF(F(I).NE.0.D0) THEN
               RSUM=RSUM+WK(IKIND)*F(I)*
     &              DDOT(NGWK*2,C00(1,I,IKIND),1,C00(1,I,IKIND),1)
            ENDIF
         ENDDO
      ENDDO      

      CALL GLOSUM(1,RSUM) 

      IF(PARENT) THEN
         WRITE(6,'(A,F13.6)')
     &        "TOTAL INTEGRATED DENSITY IN FOURIER SPACE  ", RSUM
      ENDIF

C     ==--------------------------------------------------------------==
C     == Write Restart File, with new wavefunctions                   ==
C     ==--------------------------------------------------------------==

      CALL WRITE_IREC(IREC)


      IREC(10) = 0

c      write(6,'("IREC : ")')
c      DO i =1,30
c         write(6,'("IREC ",i4," = ",i4)') i,IREC(i)
c      ENDDO

c     CALL ZHWWF(2,IREC,C0,C11,NSTATE,EV,TAU0,TAU0,TAU0,1)
      CALL ZHWWF(2,IREC,C00,C11,2*NSTATE,EV,TAU0,TAU0,TAU0,1) 

      CALL FREEM(IP_C00)
      CALL FREEM(IP_C11)
c     
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_UP3_P(LUPDRHO,TAG,NSTATE)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !TLANC NDAVV
      INCLUDE 'response_p.inc'
C     Arguments
      INTEGER   LUPDRHO,NSTATE

      CHARACTER TAG*30
C     Variables

      INTEGER   LWORK,LHPSI,LORTHO,LRNLSM,LVOFRHO

C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_RNLSM(LRNLSM,TAG,2*NSTATE,.FALSE.)
      CALL GIVE_SCR_ORTHO(LORTHO,TAG,2*NSTATE) 
      LHPSI =2*NSTATE+
     &       2*NAX+                   !DDIA
     &     MAX(2*2*NGW,LRNLSM)+       !AUXC and SCR
     &     100

      LUPDRHO= max(LHPSI,LORTHO) 
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
