#define warning_level 5d-3
C
C     WRITE A FORMATTED 'DENSITY-STYLE' CUBEFILE VERY SIMILAR
C     TO THOSE CREATED BY THE GAUSSIAN PROGRAM OR THE CUBEGEN UTILITY.
C     THE FORMAT IS AS FOLLOWS (LAST CHECKED AGAINST GAUSSIAN 98):
C
C     LINE   FORMAT      CONTENTS
C     ===============================================================
C      1     A           TITLE
C      2     A           DESCRIPTION OF PROPERTY STORED IN CUBEFILE
C      3     I5,3F12.6   #ATOMS, X-,Y-,Z-COORDINATES OF ORIGIN
C      4-6   I5,3F12.6   #GRIDPOINTS, INCREMENT VECTOR
C      #ATOMS LINES OF ATOM COORDINATES:
C      ...   I5,4F12.6   ATOM NUMBER, CHARGE, X-,Y-,Z-COORDINATE
C      REST: 6E13.5      CUBE DATA
C
C     ALL COORDINATES ARE GIVEN IN ATOMIC UNITS.
C
C     ==================================================================
      subroutine cubefile(filename,array,center,psi,thalfmesh)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cell.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'coor.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'response_p.inc'
      INCLUDE 'fileopen.inc'
C     ==--------------------------------------------------------------==
C     Arguments:
C     array: the R-space array to plot
C     filename: obvious
C     tau0: the ionic position vectors
C     center: the position (in the CPMD system) where I should place
C     \       the center of the plot
      character   filename*(*)
      logical     thalfmesh,ferror
      real*8      center(3),array(kr1,kr2s,kr3s),psi(kr2s,kr3s)

      integer     fileunit,iat,isp,ir,i1,i2,i3,ii1,ii2,ii3,ii3p,ia,ie,
     &     rgrid,ilowerleft(3),msglen,nbytes,msgid,source_pe,step
      real*8      lowerleft(3),s(3)
      data        fileunit /99/
      save        fileunit
C     ==--------------------------------------------------------------==
#ifdef PARALLEL
      msglen = 3*8
      call MY_BCAST(center,msglen,source,allgrp)
#endif
      step = 1
      if (thalfmesh) step=2
      nris(1)=nr1s
      nris(2)=nr2s
      nris(3)=nr3s

      do ir=1,3
         lowerleft(ir) = center(ir) 
     &        - 0.5d0*(a1(ir)+a2(ir)+a3(ir))
      enddo
      call DGEMV('T',3,3,1d0,htm1,3,lowerleft,1,0d0,s,1)
      do i1=1,3
         ilowerleft(i1) = NINT(s(i1)*nris(i1))+1
                                ! generates 1..NR1S + n*NR1S
         lowerleft(i1) = 0d0
         do i2=1,3
            lowerleft(i1)  = lowerleft(i1) +
     &           ht(i1,i2)*(ilowerleft(i2)-1)/DBLE(NRIS(i2))
         enddo
         ilowerleft(i1) = MOD((ilowerleft(i1)-1)+ 
     &        1000*NRIS(i1),NRIS(i1))+1
      enddo

 90      CONTINUE
      if (parent) then
         ia=1
         call xstring(filename,ia,ie)
         call fileopen(fileunit,filename(ia:ie),FO_DEF,ferror)
         if(ferror) goto 100

C     make the file identifiable by its name in the title string.
C     the second line tells other programs, that this is a
C     'density-style' cubefile (in case they actually check it).
         write (fileunit,'(A)') ' CPMD CUBE FILE: '//filename(ia:ie)
         write (fileunit,'(A)') ' Total SCF Density'

         write (fileunit,200) nat, (lowerleft(ir),ir=1,3)
C     note that we print the first mesh point in each direction
C     twice to have smooth transitions for periodic visualizations.
         write (fileunit,200) (nr1s/step)+1,
     &                    (dble(step)*a1(ir)/DBLE(nr1s),ir=1,3)
         write (fileunit,200) (nr2s/step)+1,
     &                    (dble(step)*a2(ir)/DBLE(nr2s),ir=1,3)
         write (fileunit,200) (nr3s/step)+1,
     &                    (dble(step)*a3(ir)/DBLE(nr3s),ir=1,3)

C     write atom coordinates
         do isp=1,nsp
            do iat=1,na(isp)
               write (fileunit,201) iatyp(isp),dble(iatyp(isp)),
     &              (tau0(ir,iat,isp),ir=1,3)
            enddo
         enddo
      endif

C     NOTE: the '+step' is needed to print the first meshpoint 
C     again at the end of the loop.
      do i1=ilowerleft(1),nr1s+ilowerleft(1)-1+step,step
         ii1=mod(i1-1,nr1s)+1
C     Now: Copy array(ii1,*,*) into psi. NB: This is one YZ-plane,
C     so it is NOT consecutively stored in memory...
#ifdef PARALLEL
         call my_sync(allgrp)   ! to avoid long transfer queues
C     find the PE who has the plane i1:
         source_pe = 0
         do while (.NOT. 
     &        (ii1.ge.NRXPL(source_pe,1) .AND.
     &        ii1.le.NRXPL(source_pe,2)))
            source_pe = source_pe + 1
         enddo
         source_pe = PGROUP(source_pe+1)

         if (MEPOS .EQ. source_pe) then
            call DCOPY(kr2s*kr3s, 
     &           array(ii1-NRXPL(MEPOS,1)+1,1,1),KR1,
     &           psi,1)
            if (.NOT. parent) then
               msglen = 8*kr2s*kr3s ! one X-plane.
               msgid = 2        ! MPI message tag.
               call MY_BSEND(psi,msglen,pgroup(1),msgid)
            endif
         elseif (parent) then
            msglen = 8*kr2s*kr3s ! one X-plane.
            msgid = 2           ! MPI message tag.
            call MY_BRECV(psi,msglen,source_pe,msgid,nbytes)
         endif
#else
         call DCOPY(kr2s*kr3s, array(ii1,1,1),KR1, psi,1)
#endif

C     Now, the parent should have the plane in PSI: write to cubefile.
C     again, add step to get one more meshpoint in y- and z-direction 
         if (parent) then
            do i2=ilowerleft(2),nr2s+ilowerleft(2)-1+step,step
               ii2=mod(i2-1,nr2s)+1
               write (fileunit,'(6E13.5)') (psi(ii2,mod(i3-1,nr3s)+1),
     $              i3=ilowerleft(3),nr3s+ilowerleft(3)-1+step,step)
            enddo
         endif
      enddo                     ! i1

      if (parent) call fileclose(fileunit)
      return

 120  CONTINUE
      print *,'ERROR FINALLY CLOSING FILE.'
      return

 200  FORMAT (I5,3F12.6)
 201  FORMAT (I5,4F12.6)

 100  CONTINUE
      print *,'ERROR OPENING FILE:',filename
      call fileclose(fileunit)
      print *,'FILE SUCCESSFULLY RE-CLOSED:',filename
 110  CONTINUE
C      goto 90
      return
      end
c     ==================================================================
      SUBROUTINE CLEAN_RSPACE(array)
      IMPLICIT NONE
      include 'system.h'
      real*8     array(kr1,kr2s,kr3s)
      integer    i1,i2,i3
      i1 = kr1
!$OMP parallel do private(i3,i2)
      do i3=1,nr3s
         do i2=1,nr2s
            array(i1,i2,i3) = 0d0
         enddo
      enddo
      i2 = kr2s
!$OMP parallel do private(i3,i1)
      do i3=1,nr3s
         do i1=1,nr1
            array(i1,i2,i3) = 0d0
         enddo
      enddo
      i3 = kr3s
!$OMP parallel do private(i2,i1)
      do i2=1,nr2s
         do i1=1,nr1
            array(i1,i2,i3) = 0d0
         enddo
      enddo
      return
      end
c     ==================================================================
      REAL*8 FUNCTION DOT_GA(ig,which_b)
C     computes and returns the dotproduct
C     G . B_i
C     where G is the reciprocal space vector defined by the index ig,
C     and B_i is the reciprocal lattice vector of the system, 
C     indexed by which_b. which_b=1, 2 or 3 corresponding to the system
C     vectors B1, B2, B3.
      include 'system.h'
      include 'cppt.inc'
      include 'gvec.inc'
c     =----------------------------------------------------------------=
      integer ig,which_b
      if (ig .gt. nhg) 
     &    call stopgm('DOT_GB','INTERNAL ERROR: wrong G vector index')

      if      (which_b .eq. 1) then
        DOT_GA = GK(1,ig)*A1(1) + GK(2,ig)*A1(2) + GK(3,ig)*A1(3)
      elseif (which_b .eq. 2) then
        DOT_GA = GK(1,ig)*A2(1) + GK(2,ig)*A2(2) + GK(3,ig)*A2(3)
      elseif (which_b .eq. 3) then
        DOT_GA = GK(1,ig)*A3(1) + GK(2,ig)*A3(2) + GK(3,ig)*A3(3)
      else
        call stopgm('DOT_GB','INTERNAL ERROR: wrong direction index')
      endif
      DOT_GA = TPIBA * DOT_GA
      return
      end
c     ==================================================================
      REAL*8 FUNCTION eigr_dot(isa,func)
c     ==--------------------------------------------------------------==
C     INPUT:   One function f(G), packed storage, dim=NHG
c     \        Species/Atom index ISA.
c     OUTPUT:  function value returns
c     \        Sum_G  exp(i G R_ion[isa])  f(G)
c     \        i.e. the Fourier transform of f(G) at the position of
c     \        the nucleus ISA.
c     \        ATTENTION: The G=0 component of func is set to zero!!
c     NOT PARALLEL -- corresponds to a DOTP-call
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'sfac.inc'
      include 'ions.inc'
      include 'geq0.inc'
c     ==--------------------------------------------------------------==
      integer     isa,ig,isub,ig1
      complex*16  expigr
      real*8      func(2,*),dotp,fnl,dfnl,dot,result
      external    dotp
      DIMENSION   EIGRB(NHG,NAT)
c     ==--------------------------------------------------------------==
      call tiset('EIGR_DOT       ',isub)
      ig1=1
      if (geq0) then
        ig1=2
        func(1,1) = 0d0
        func(2,1) = 0d0
      endif
      if (bigmem) then
        result = dotp(nhg,eigrb(1,isa),func)
      else
         result = 0d0
!$OMP parallel do private(ig,expigr,dot) reduction(+:result)
         do ig=ig1,nhg
            expigr = EI1(isa,INYH(1,ig))
     &           * EI2(isa,INYH(2,ig))
     &           * EI3(isa,INYH(3,ig))
            dot    = DREAL(expigr)*func(1,ig)
     &           + DIMAG(expigr)*func(2,ig)
            result = result + dot
         enddo
         result = 2.d0 * result
      endif
      call tihalt('EIGR_DOT       ',isub)
      eigr_dot=result
      return
      end
c     ==================================================================
c     ==================================================================
      REAL*8 FUNCTION eigr_dot2(isa,func)
c     ==--------------------------------------------------------------==
C     DIFFERENCE TO PREVIOUS ROUTINE (eigr_dot):
C     THIS ONE DOES NOT SET THE IG=0 COMPONENT TO ZERO!!
C     INPUT:   One function f(G), packed storage, dim=NHG
c     \        Species/Atom index ISA.
c     OUTPUT:  function value returns
c     \        Sum_G  exp(i G R_ion[isa])  f(G)
c     \        i.e. the Fourier transform of f(G) at the position of
c     \        the nucleus ISA.
c     \        ATTENTION: The G=0 component of func is set to zero!!
c     NOT PARALLEL -- corresponds to a DOTP-call
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'cppt.inc'
      include 'sfac.inc'
      include 'ions.inc'
      include 'geq0.inc'
c     ==--------------------------------------------------------------==
      integer     isa,ig,isub,ig1
      complex*16  expigr
      real*8      func(2,*),dotp,fnl,dfnl,dot,result
      external    dotp
      DIMENSION   EIGRB(NHG,NAT)
c     ==--------------------------------------------------------------==
      call tiset('EIGR_DOT2      ',isub)
      ig1=1
      if (geq0) then
        ig1=2
        expigr = EI1(isa,INYH(1,1))
     &       * EI2(isa,INYH(2,1))
     &       * EI3(isa,INYH(3,1))
        dot    = DREAL(expigr)*func(1,1)
     &       + DIMAG(expigr)*func(2,1)
        result = 0.5d0 * dot
      endif
      if (bigmem) then
        result = dotp(nhg,eigrb(1,isa),func)
      else
!$OMP parallel do private(ig,expigr,dot) reduction(+:result)
         do ig=ig1,nhg
            expigr = EI1(isa,INYH(1,ig))
     &           * EI2(isa,INYH(2,ig))
     &           * EI3(isa,INYH(3,ig))
            dot    = DREAL(expigr)*func(1,ig)
     &           + DIMAG(expigr)*func(2,ig)
            result = result + dot
         enddo
         result = 2.d0 * result
      endif
      eigr_dot2 = result
      call tihalt('EIGR_DOT2       ',isub)
      return
      end
c     ==================================================================

