C     ==================================================================
      SUBROUTINE V1OFRHO1(E2,DRHOE,DDXC,PSI,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     calculate the implicit part of the first order potential, i.e.
C     the part induced by the response of the density wrt the bare
C     perturbation.
C     input:  first order density in
C             drhoe(r) = <phi1|r><r|phi0> + cc
C     output: first order potential (implicit part) in
C             drhoe(r)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'spin.inc'        !nlsd
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'poin.inc'
      INCLUDE 'linres.inc'
      INCLUDE 'simul.inc'
C
      INTEGER    LSCR
      COMPLEX*16 PSI(*)
      REAL*8     E2(2),SCR(LSCR),DRHOE(NNR1,NLSD)
      REAL*8     DDXC(NNR1,*)
C
      DIMENSION  RHOO(NNR1,*)
      INTEGER    IR,IG,ISUB,IL_GRAD,IL_VTEMP,IL_VTMP,IG1,NLSDS
      POINTER    (IP_VTMP,VTMP),(IP_DXC,DXC),(IP_V1,V1)
      POINTER    (IP_VTEMP,VTEMP),(IP_GRAD,GRAD)
      POINTER    (IP_RHOX,RHOX),(IP_SCR2,SCR2)
      COMPLEX*16 VTMP(*),VTEMP(*),V1(*)
      REAL*8     DXC(NNR1,*),GRAD(*),RHOX(*),EXC2,EXC2M,EXC2T,EHT2,
     $           DRA,DRB,SCR2(*),DVMA,DVMB,DVTA,DVTB,TT
      LOGICAL    DEBUG
      INTEGER    LV1OFRHO1,LSCR2
      CHARACTER*30 TAG
C     ==--------------------------------------------------------------==
      CALL TISET('   V1OFRHO1',ISUB)
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_V1OFRHO1(LV1OFRHO1,TAG)
      CALL TEST_SCR('V1OFRHO1',TAG,LSCR,LV1OFRHO1)
C
      DEBUG=.FALSE.
      IF(TLSE) THEN
        NLSDS=2           ! Number of spin densities treated together
      ELSE
        NLSDS=NLSD
      ENDIF
      CALL SCRSET(SCR,LSCR,DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_V1,2*NHG,'V1',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_DXC,NNR1*NLSD,'DXC',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_RHOX,NNR1*NLSDS,'RHOX',DEBUG)
C
      IF(TGC) THEN
        IL_GRAD=NNR1*NLSDS*4
        IL_VTEMP=2*NHG*NLSDS
        IL_VTMP=MAX(2*NHG,NNR1*(2*NLSDS-1))
      ELSE
        IL_GRAD=1
        IL_VTEMP=1
        IL_VTMP=1
      ENDIF
      CALL SCRPTR(SCR,LSCR,IP_VTEMP,IL_VTEMP,'VTEMP',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_VTMP,IL_VTMP,'VTMP',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_GRAD,IL_GRAD,'GRAD',DEBUG)
C
      IF(TXC_DD_ANA) CALL SCRGET(SCR,LSCR,IP_SCR2,LSCR2,'SCR2',DEBUG)
C
      IF(TCLUST.AND.PS_TYPE.EQ.1) 
     *   CALL STOPGM('V1OFRHO1','HOCKNEY NOT IMPLEMENTED')
C..transform n1 to g-space
      CALL ZAZZERO(PSI,MAXFFT)
      DO IR=1,NNR1
        PSI(IR)=DCMPLX(DRHOE(IR,1),0.D0)
      ENDDO
      CALL FWFFT(PSI)
C     ==--------------------------------------------------------------==
C     == This is the Hartree Potential                                ==
C     ==--------------------------------------------------------------==
      EHT2=0.D0
      DO IG=1,NHG
        V1(IG) = PSI(NZH(IG)) * SCG(IG)
        EHT2 = EHT2 + DREAL(DCONJG(V1(IG))*PSI(NZH(IG)))
      ENDDO
      EHT2 = EHT2*OMEGA
C     ==--------------------------------------------------------------==
C     == XC contributions                                             ==
C     ==--------------------------------------------------------------==
      IF(TLSE) THEN
C       Switch to LSD for mixed and triplet states
        TLSE=.FALSE.
        TLSD=.TRUE.
        NLSD=2
        IF(TXC_ANALYTIC) THEN
          WRITE(6,*) 'V1OFRHO1: use XC_DD_ANALYTIC or numeric with LSE'
          CALL STOPGM('V1OFRHO1','XC_ANALYTIC not implemented')
        ENDIF
C       Mixed state
        CALL DD_XC(RHOO(1,1),DRHOE(1,1),RHOX,PSI,DXC(1,1),
     $             VTMP,VTEMP,GRAD,SCR2,LSCR2,.FALSE.)
        EXC2M=0.D0
        DO IR=1,NNR1
          DRB=DRHOE(IR,2)
          DRA=DRHOE(IR,1)-DRHOE(IR,2)
          EXC2M=EXC2M+DXC(IR,1)*DRA+DXC(IR,2)*DRB
        ENDDO
        EXC2M=0.5D0*EXC2M*OMEGA/DBLE(NR1S*NR2S*NR3S)
C       Triplet state
        CALL DD_XC(RHOO(1,3),DRHOE(1,3),RHOX,PSI,DXC(1,3),
     $             VTMP,VTEMP,GRAD,SCR2,LSCR2,.FALSE.)
        EXC2T=0.D0
        DO IR=1,NNR1
          DRB=DRHOE(IR,4)
          DRA=DRHOE(IR,3)-DRHOE(IR,4)
          EXC2T=EXC2T+DXC(IR,3)*DRA+DXC(IR,4)*DRB
        ENDDO
        EXC2T=0.5D0*EXC2T*OMEGA/DBLE(NR1S*NR2S*NR3S)
C       Join and switch back to LSE
        EXC2 = LSEA*EXC2M + LSEB*EXC2T
        IF(TLSETS) THEN
          TT=1.0D0/1.5D0
          DO IR=1,NNR1
            DVMA=DXC(IR,1)
            DVMB=DXC(IR,2)
            DVTA=DXC(IR,3)
            DVTB=DXC(IR,4)
            DXC(IR,1)=0.5D0*(LSEA*(DVMA+DVMB)+LSEB*(DVTA+DVTB))
            DXC(IR,2)=TT*(LSEA*(DVMA+0.5D0*DVMB)+LSEB*(0.5D0*DVTA+DVTB))
            DXC(IR,3)=LSEA*DVMB+LSEB*DVTB
          ENDDO
        ELSE
          DO IR=1,NNR1
            DVMA=DXC(IR,1)
            DVMB=DXC(IR,2)
            DVTA=DXC(IR,3)
            DVTB=DXC(IR,4)
            DXC(IR,1)=0.5D0*(LSEA*(DVMA+DVMB)+LSEB*(DVTA+DVTB))
            DXC(IR,2)=LSEA*DVMA+LSEB*DVTB
            DXC(IR,3)=LSEA*DVMB+LSEB*DVTB
          ENDDO
        ENDIF
        TLSE=.TRUE.
        TLSD=.FALSE.
        NLSD=4
      ELSEIF(TLSD) THEN
        IF(TXC_ANALYTIC) THEN
          DO IR=1,NNR1
            DRB=DRHOE(IR,2)
            DRA=DRHOE(IR,1)-DRHOE(IR,2)
            DXC(IR,1)=DDXC(IR,1)*DRA+DDXC(IR,3)*DRB
            DXC(IR,2)=DDXC(IR,2)*DRB+DDXC(IR,3)*DRA
          ENDDO
        ELSE
          CALL DD_XC(RHOO,DRHOE,RHOX,PSI,DXC,VTMP,VTEMP,GRAD,
     $          SCR2,LSCR2,.FALSE.)
        ENDIF
        EXC2=0.D0
        DO IR=1,NNR1
          DRB=DRHOE(IR,2)
          DRA=DRHOE(IR,1)-DRHOE(IR,2)
          EXC2=EXC2+DXC(IR,1)*DRA+DXC(IR,2)*DRB
        ENDDO
        EXC2=0.5D0*EXC2*OMEGA/DBLE(NR1S*NR2S*NR3S)
      ELSE
        IF(TXC_ANALYTIC) THEN
          DO IR=1,NNR1
            DXC(IR,1)=DDXC(IR,1)*DRHOE(IR,1)
          ENDDO
        ELSE
          CALL DD_XC(RHOO,DRHOE,RHOX,PSI,DXC,VTMP,VTEMP,GRAD,
     $          SCR2,LSCR2,.FALSE.)
        ENDIF
        EXC2=0.D0
        DO IR=1,NNR1
          EXC2=EXC2+DXC(IR,1)*DRHOE(IR,1)
        ENDDO
        EXC2=0.5D0*EXC2*OMEGA/DBLE(NR1S*NR2S*NR3S)
      ENDIF
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(PSI,MAXFFT)
!CDIR NODEP
      DO IG=1,NHG
         PSI(INDZ(IG)) = DCONJG(V1(IG))
         PSI(NZH(IG))  = V1(IG)
      ENDDO
      IF(GEQ0)   PSI(NZH(1)) = V1(1)
      CALL INVFFT(PSI)
C     ==--------------------------------------------------------------==
C     == Total potential in drhoe                                     ==
C     ==--------------------------------------------------------------==
      IF(TLSE) THEN
        DO IR=1,NNR1
          DRHOE(IR,1)=DREAL(PSI(IR))+DXC(IR,1)
          DRHOE(IR,2)=DREAL(PSI(IR))+DXC(IR,2)
          DRHOE(IR,3)=DREAL(PSI(IR))+DXC(IR,3)
        ENDDO
      ELSEIF(TLSD) THEN
        DO IR=1,NNR1
          DRHOE(IR,1)=DREAL(PSI(IR))+DXC(IR,1)
          DRHOE(IR,2)=DREAL(PSI(IR))+DXC(IR,2)
        ENDDO
      ELSE
        DO IR=1,NNR1
          DRHOE(IR,1)=DREAL(PSI(IR))+DXC(IR,1)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      E2(1) = EHT2
      E2(2) = EXC2
      CALL SCRREL(SCR,LSCR,DEBUG)
C     ==--------------------------------------------------------------==
      CALL TIHALT('   V1OFRHO1',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_V1OFRHO1(LV1OFRHO1,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'linres.inc'      ! TXC_DD_ANA
      INTEGER LV1OFRHO1
      INTEGER IL_GRAD,IL_VTEMP,IL_VTMP,IL_DXC,IL_V1,IL_RHOX,ILDD_XC_ANA,
     $        NLSDS
      CHARACTER*30 TAG
C     ==--------------------------------------------------------------==
      IF(TLSE) THEN
        NLSDS=2
      ELSE
        NLSDS=NLSD
      ENDIF
      IF(TGC) THEN
        IL_GRAD=NNR1*NLSDS*4
        IL_VTEMP=2*NHG*NLSDS
        IL_VTMP=MAX(2*NHG,NNR1*(2*NLSDS-1))
      ELSE
        IL_GRAD=1
        IL_VTEMP=1
        IL_VTMP=1
      ENDIF
      IF(TXC_DD_ANA) THEN
        CALL GIVE_SCR_DD_XC_ANA(ILDD_XC_ANA,TAG)
      ELSE
        ILDD_XC_ANA=1
      ENDIF
      IL_DXC=NNR1*NLSD
      IL_RHOX=NNR1*NLSDS
      IL_V1=2*NHG
      LV1OFRHO1=IL_GRAD+IL_VTEMP+IL_VTMP+IL_DXC+IL_V1+IL_RHOX+
     $          ILDD_XC_ANA
      LV1OFRHO1=LV1OFRHO1+200
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE VTDOFRHO1(E2,DRHOE,DDXC,PSI,SCR,LSCR,SWFUN)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'spin.inc'        !nlsd
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'poin.inc'
      INCLUDE 'linres.inc'
      INCLUDE 'simul.inc'
C
      INTEGER    LSCR
      COMPLEX*16 PSI(*)
      REAL*8     SCR(LSCR),DRHOE(NNR1,NLSD),E2(*)
      REAL*8     DDXC(NNR1,*)
      LOGICAL    SWFUN
C
      DIMENSION  RHOO(*)
      INTEGER    IR,IG,ISUB,IL_GRAD,IL_VTEMP,IL_VTMP,IG1
      POINTER    (IP_VTMP,VTMP),(IP_DXC,DXC),(IP_V1,V1)
      POINTER    (IP_VTEMP,VTEMP),(IP_GRAD,GRAD)
      POINTER    (IP_RHOX,RHOX),(IP_SCR2,SCR2)
      COMPLEX*16 VTMP(*),VTEMP(*),V1(*)
      REAL*8     DXC(NNR1,*),GRAD(*),RHOX(*),DRA,DRB,EHT2,EXC2,SCR2(*)
      LOGICAL    DEBUG
      INTEGER    LVTDOFRHO1,LV1,LDXC,LRHOX,NMU,LSCR2
      CHARACTER*30 TAG
C     ==--------------------------------------------------------------==
      CALL TISET(' VTDOFRHO1',ISUB)
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_VTDOFRHO1(LVTDOFRHO1,TAG)
      CALL TEST_SCR('VTDOFRHO1',TAG,LSCR,LVTDOFRHO1)
      EHT2=0.D0
      EXC2=0.D0
C
      DEBUG=.FALSE.
      CALL SCRSET(SCR,LSCR,DEBUG)
      IF(TLSD.OR.LRSYM.EQ.3) THEN
        NMU=2
      ELSE
        NMU=1
      ENDIF
      LV1=2*NHG
      LDXC=NMU*NNR1
      LRHOX=NMU*NNR1
      CALL SCRPTR(SCR,LSCR,IP_V1,LV1,'V1',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_DXC,LDXC,'DXC',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_RHOX,LRHOX,'RHOX',DEBUG)
      IF((TGC.OR.TD_TGC).AND..NOT.TXC_ANALYTIC) THEN
        IL_GRAD=NNR1*NMU*4
        IL_VTEMP=2*NHG*NMU
        IL_VTMP=MAX(2*NHG,NNR1*(2*NMU-1))
      ELSE
        IL_GRAD=1
        IL_VTEMP=1
        IL_VTMP=1
      ENDIF
      CALL SCRPTR(SCR,LSCR,IP_VTEMP,IL_VTEMP,'VTEMP',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_VTMP,IL_VTMP,'VTMP',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_GRAD,IL_GRAD,'GRAD',DEBUG)
C
      IF(TXC_DD_ANA) CALL SCRGET(SCR,LSCR,IP_SCR2,LSCR2,'SCR2',DEBUG)
C
      IF(TCLUST.AND.PS_TYPE.EQ.1) 
     *   CALL STOPGM('VTDOFRHO1','HOCKNEY NOT IMPLEMENTED')
C..first: Hartree (only if not triplet)
      IF(LRSYM.NE.3) THEN
C..transform n1 to g-space
        CALL ZAZZERO(PSI,MAXFFT)
        DO IR=1,NNR1
          PSI(IR)=DCMPLX(DRHOE(IR,1),0.D0)
        ENDDO
        CALL FWFFT(PSI)
C     ==--------------------------------------------------------------==
C     == This is the Hartree Potential                                ==
C     ==--------------------------------------------------------------==
        DO IG=1,NHG
          V1(IG) = PSI(NZH(IG)) * SCG(IG)
          EHT2 = EHT2 + DREAL(DCONJG(V1(IG))*PSI(NZH(IG)))
        ENDDO
      ENDIF
      EHT2 = EHT2*OMEGA
C     ==--------------------------------------------------------------==
C     == XC contributions                                             ==
C     ==--------------------------------------------------------------==
      IF(TLSD) THEN
        IF(TXC_ANALYTIC) THEN
          DO IR=1,NNR1
            DRB=DRHOE(IR,2)
            DRA=DRHOE(IR,1)-DRHOE(IR,2)
            DXC(IR,1)=DDXC(IR,1)*DRA+DDXC(IR,3)*DRB
            DXC(IR,2)=DDXC(IR,2)*DRB+DDXC(IR,3)*DRA
          ENDDO
        ELSE
          CALL DD_XC(RHOO,DRHOE,RHOX,PSI,DXC,VTMP,VTEMP,GRAD,
     $          SCR2,LSCR2,SWFUN)
        ENDIF
        DO IR=1,NNR1
          DRB=DRHOE(IR,2)
          DRA=DRHOE(IR,1)-DRHOE(IR,2)
          EXC2=EXC2+DXC(IR,1)*DRA+DXC(IR,2)*DRB
        ENDDO
        EXC2=0.5D0*EXC2*OMEGA/DBLE(NR1S*NR2S*NR3S)
      ELSEIF(LRSYM.EQ.1) THEN
C..SINGLET
        IF(TXC_ANALYTIC) THEN
          DO IR=1,NNR1
            DXC(IR,1)=DDXC(IR,1)*DRHOE(IR,1)
          ENDDO
        ELSE
          CALL DD_XC(RHOO,DRHOE,RHOX,PSI,DXC,VTMP,VTEMP,GRAD,
     $          SCR2,LSCR2,SWFUN)
        ENDIF
        DO IR=1,NNR1
          EXC2=EXC2+DXC(IR,1)*DRHOE(IR,1)
        ENDDO
        EXC2=0.5D0*EXC2*OMEGA/DBLE(NR1S*NR2S*NR3S)
      ELSEIF(LRSYM.EQ.3) THEN
C..TRIPLET
        IF(TXC_ANALYTIC) THEN
          DO IR=1,NNR1
            DXC(IR,1)=DDXC(IR,1)*DRHOE(IR,1)
          ENDDO
        ELSE
          CALL DD_XCT(RHOO,DRHOE,RHOX,PSI,DXC,VTMP,VTEMP,GRAD,
     $          SCR2,LSCR2,SWFUN)
        ENDIF
        DO IR=1,NNR1
          EXC2=EXC2+DXC(IR,1)*DRHOE(IR,1)
        ENDDO
        EXC2=0.5D0*EXC2*OMEGA/DBLE(NR1S*NR2S*NR3S)
      ELSE
        CALL STOPGM('VTDOFRHO1','WRONG LRSYM')
      ENDIF
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(PSI,MAXFFT)
      IF(LRSYM.NE.3) THEN
!CDIR NODEP
        DO IG=1,NHG
           PSI(INDZ(IG)) = DCONJG(V1(IG))
           PSI(NZH(IG))  = V1(IG)
        ENDDO
        IF(GEQ0)   PSI(NZH(1)) = V1(1)
        CALL INVFFT(PSI)
      ENDIF
      E2(1) = EHT2
      E2(2) = EXC2
C     ==--------------------------------------------------------------==
C     == Total potential in drhoe                                     ==
C     ==--------------------------------------------------------------==
      IF(TLSD) THEN
        DO IR=1,NNR1
          DRHOE(IR,1)=0.5D0*(DREAL(PSI(IR))+DXC(IR,1))
          DRHOE(IR,2)=0.5D0*(DREAL(PSI(IR))+DXC(IR,2))
        ENDDO
        IF(.NOT.TDA) CALL DSCAL(NNR1,2.D0,DRHOE(1,1),1)
        IF(.NOT.TDA) CALL DSCAL(NNR1,2.D0,DRHOE(1,2),1)
      ELSE
        DO IR=1,NNR1
          DRHOE(IR,1)=DREAL(PSI(IR))+DXC(IR,1)
        ENDDO
        IF(.NOT.TDA) CALL DSCAL(NNR1,2.D0,DRHOE(1,1),1)
      ENDIF
      CALL SCRREL(SCR,LSCR,DEBUG)
C     ==--------------------------------------------------------------==
      CALL TIHALT(' VTDOFRHO1',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_VTDOFRHO1(LVTDOFRHO1,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'linres.inc'
      LOGICAL TLSDSV
      INTEGER LVTDOFRHO1
      INTEGER IL_GRAD,IL_VTEMP,IL_VTMP,IL_DXC,IL_V1,IL_RHOX,NMU,
     $        LDD_XC_ANA
      CHARACTER*30 TAG
C     ==--------------------------------------------------------------==
      IF(TLSD.OR.NS_TRI.GT.0) THEN
        NMU=2
      ELSE
        NMU=1
      ENDIF
      IL_DXC=NNR1*NMU
      IL_RHOX=NNR1*NMU
      IL_V1=2*NHG
      IF((TGC.OR.TD_TGC).AND..NOT.TXC_ANALYTIC) THEN
        IL_GRAD=NNR1*NMU*4
        IL_VTEMP=2*NHG*NMU
        IL_VTMP=MAX(2*NHG,NNR1*(2*NMU-1))
      ELSE
        IL_GRAD=1
        IL_VTEMP=1
        IL_VTMP=1
      ENDIF
      IF(TXC_DD_ANA) THEN
        IF(NS_TRI.GT.0) THEN
          TLSDSV=TLSD
          TLSD=.TRUE.
          CALL GIVE_SCR_DD_XC_ANA(LDD_XC_ANA,TAG)
          TLSD=TLSDSV
        ELSE
          CALL GIVE_SCR_DD_XC_ANA(LDD_XC_ANA,TAG)
        ENDIF
      ELSE
        LDD_XC_ANA=1
      ENDIF
      LVTDOFRHO1=IL_GRAD+IL_VTEMP+IL_VTMP+IL_DXC+IL_V1+IL_RHOX+
     $           LDD_XC_ANA
      LVTDOFRHO1=LVTDOFRHO1+200
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================

