c     ==================================================================
      subroutine v1ofrho_p(v1_loc,eirop1,scr,lscr,drhoe,v,psi,first)
c     ==--------------------------------------------------------------==
c     calculate the implicit part of the first order potential, i.e.
c     the part induced by the response of the density wrt the bare
c     perturbation.
c      input: first order density in
c             drhoe(r) = <phi1|r><r|phi0> + cc
c     \       local potential in v_loc
c     output: first order potential in v(r):
c     \       v(r0)  =  drhoe(r) / |r0-r|  +  d v_xc / d rho  * drhoe
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'cnst.inc'
      include 'spin.inc'        !nlsd
      include 'cppt.inc'
      include 'geq0.inc'
      include 'fft.inc'
      include 'response_p.inc'
      include 'simul.inc'
C     Arguments
      integer    lscr
      real*8     drhoe(nnr1,nlsd),v(nnr1,nlsd)
      complex*16 v1_loc(nhg),eirop1(nhg),scr(*),psi(*)

C     Local Variables
      complex*16 eirop1gg,vg(nhg)
      pointer    (ip_vg,vg)     ! VG is a dummy array to access V
      real*8     GG,DDOT
      character  tag*30
      integer    lvofrho,i,il_grad,ig,ig1,ir,isub
      logical    first
c     ==--------------------------------------------------------------==
C     ==         VARIABLE USAGE:
c     == v:      electronic density in g-space (input)                ==
c     == eivps : phase factor times local pseudopotential  (vps)      ==
c     == eirop : phase factor times gaussian charge distributions     ==
c     ==         which replaced ionic point charges (rhops)           ==
c     ==--------------------------------------------------------------==
      call tiset('    v1ofrho',isub)
      call give_scr_v1ofrho(lvofrho,tag)
      call test_scr('v1ofrho',tag,lscr,lvofrho)
      CALL GET_ADDR(IP_vg,v)
      call zazzero(vg,nhg)
      ig1 = 1
      if (geq0 .and. (ibrav .ne. 0)) then
        ig1 = 2
      endif
c     ==--------------------------------------------------------------==
      eht1  = 0.d0
      if (.not. first) then     ! rho1 - dependent stuff
C     Hartree potential.
         call ffttog(drhoe,psi,scr,nhg,.true.)
C     The Coulomb potential is done from the total density, up+down
         if (geq0 .and. (ibrav .ne. 0)) then
            psi(1) = DCMPLX(0d0,0d0) ! This should be satisfied anyway.
            vg(1)  = DCMPLX(0d0,0d0)
         endif

C     Coulomb potential:
         do ig=ig1,nhg
            vg(ig) = psi(ig) * scg(ig)
            eht1   = eht1 + dreal(dconjg(vg(ig))*psi(ig))
         enddo
      else                      ! first=.true.
         call zazzero(psi,maxfft)
      endif


      if (phonon .or. tlanphon) then
         do ig=ig1,nhg
            eirop1gg = eirop1(ig) * scg(ig)
            vg(ig)   = vg(ig) + eirop1gg
            eht1     = eht1 + 2d0*DREAL(DCONJG(eirop1gg) * psi(ig))
         enddo
      endif


C     Local external potential.
C     TO BE ELIMINATED. Use   V1_NONLOC * PSI0   instead.
      eloc1 = 0.0d0
      if (.not. (TINTERACTION.or.TDUMMYATOM)) then
         do ig=ig1,nhg
            vg(ig) = vg(ig) + v1_loc(ig)
            eloc1  = eloc1 + 
     &              2d0 * DREAL(DCONJG(psi(ig))*v1_loc(ig))
         enddo
      endif

      eht1   =  eht1  * omega
      eloc1  =  eloc1 * omega

c     ==--  v(G) -> v(R)  --------------------------------------------==
      call ffttor(vg,v,psi,nhg,.true.)
C     NB: The arrays V and VG are actually pointing to identical 
C     memory addresses.

c     ==  XC potential  ----------------------------------------------==
      call v1xc_p(drhoe,v,psi,scr,lscr)
C     Adds the xc - contribution to V (creates spin split if 
C     required, V(up), V(down)) and calculates also the XC-energy EXC1

c     ==--------------------------------------------------------------==
c     == v contains the total potential in r-space                    ==

      call tihalt('    v1ofrho',isub)
      return
      end
c     ==================================================================
c     ==================================================================
      subroutine give_scr_v1ofrho(lvofrho,tag)
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'        !nhg
      include 'spin.inc'        !nlsd nlsx
      include 'pslo.inc'        !tivan
      include 'cppt.inc'
      include 'nlcc.inc'
      include 'isos.inc'
      include 'fft.inc'

c     arguments
      integer   lvofrho
      character tag*30
c     variables
      integer   l1,l2,l3,l4

      if(tclust) then
#ifdef PARALLEL
        l1       = 2*nhg*2+2*(nr1h+1)*nr2h*nr3pl
#else
        l1       = 2*nhg*2+2*(nr1h+1)*nr2h
#endif
      else
        l1       = 2*nhg*2
      endif
      call give_scr_v1xc_p(l2,tag)
      l4   = max(nhg*2*nlsx,2*nnr1,2*nhg*nlsd)


      lvofrho  =  max (l1,l2,l4) + 8
      tag      =  'v1 of rho'
c     ==--------------------------------------------------------------==
      return
      end
c     ==================================================================
