C     ==================================================================
      SUBROUTINE VDW(TAU0,NVDW,IDVDW,IVDW,JVDW,VDWST,VDWRM,VDWBE,
     +           VDWEPS,S6GRIM,NXVDW,NYVDW,NZVDW,EVDW,FION,DEVDW)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     == THE EMPIRICAL VAN DER WAALS CORRECTION TO THE TOTAL ENERGY,  == 
C     == IONIC FORCE, AND STRESS TENSOR ACCORDING TO THE FORMUALTION  ==
C     == OF R. LE SAR, J. PHYS. CHEM. 88, p. 4272 (1984)              ==
C     ==               J. CHEM. PHYS. 86, p. 1485 (1987)              ==
C     == Modified as in:                                              == 
C     ==   M. Elstner et al. J. Chem. Phys. 114, p. 5149 (2001)       ==
C     ==   or Grimme, J. Comput. Chem. 27, 1787, 2006 (IDVDW=3)       ==
C     == Present release: Munich/Philadelphia/Tsukuba, 11 May 2010    ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'strs.inc'
      INCLUDE 'metr.inc'
C     ARGUMENTS
      INTEGER NVDW,NXVDW,NYVDW,NZVDW
      REAL*8  TAU0(3,NAX,*)
      INTEGER IDVDW(*),IVDW(*),JVDW(*)
      REAL*8  VDWST(*),VDWRM(*),VDWBE(*)
      REAL*8  VDWEPS,EVDW,S6GRIM ! new argument functional-dependent !
      REAL*8  FION(3,NAX,*),DEVDW(*)
C     LOCAL VARIABLES
      REAL*8  RLP(3)
      INTEGER N,IV,JV,L_UPPER,M_UPPER,L,M,NX,NY,NZ
      REAL*8  REPS,XLM,YLM,ZLM,RLM1,RLM2,RPHI,FFAC,RLM6,RPOW,
     &        REXP,AEXP,RFAC
      DATA    REPS/1.0D-2/
C     ==--------------------------------------------------------------==
      EVDW=0.0D0
      CALL AZZERO(DEVDW,6)
      IF(PARENT) THEN 
        DO N=1,NVDW 
          IV=IVDW(N) 
          JV=JVDW(N) 
          L_UPPER=NA(IV) 
          M_UPPER=NA(JV) 
          DO L=1,L_UPPER 
            IF(IV.EQ.JV) THEN
              M_UPPER=L
            ENDIF
            DO M=1,M_UPPER 
              XLM=TAU0(1,L,IV)-TAU0(1,M,JV)
              YLM=TAU0(2,L,IV)-TAU0(2,M,JV)
              ZLM=TAU0(3,L,IV)-TAU0(3,M,JV)
              CALL PBC(XLM,YLM,ZLM,XLM,YLM,ZLM,1,APBC,IBRAV)
              DO NX=-NXVDW,NXVDW
                DO NY=-NYVDW,NYVDW 
                  DO NZ=-NZVDW,NZVDW
                    RLP(1)=XLM+NX*HT(1,1)+NY*HT(2,1)+NZ*HT(3,1) 
                    RLP(2)=YLM+NX*HT(1,2)+NY*HT(2,2)+NZ*HT(3,2)
                    RLP(3)=ZLM+NX*HT(1,3)+NY*HT(2,3)+NZ*HT(3,3)
                    RLM2=RLP(1)*RLP(1)+RLP(2)*RLP(2)+RLP(3)*RLP(3)
                    RLM1=DSQRT(RLM2)
                    IF(RLM1.LT.REPS) THEN 
                      RPHI=0.0D0 
                      FFAC=0.0D0 
                    ELSE IF(IDVDW(N).EQ.1) THEN 
                      RLM6=RLM2*RLM2*RLM2
                      RPOW=VDWBE(N)*(RLM1/VDWRM(N))**7
                      IF(DABS(RPOW).LT.1.0D0/VDWEPS) THEN 
                        REXP=DEXP(-RPOW)
                      ELSE 
                        REXP=0.0D0
                      ENDIF 
                      AEXP=1.0D0-REXP
                      RPHI=VDWST(N)/RLM6*AEXP**4
                      RFAC=-6.0D0/RLM1+28.0D0*RPOW*REXP/(RLM1*AEXP)
                      FFAC=RPHI*RFAC/RLM1
                    ELSE IF(IDVDW(N).EQ.2) THEN 
                      RPOW=VDWBE(N)*(RLM1-VDWRM(N))
                      AEXP=VDWST(N)*0.5D0 
                      IF(DABS(RPOW).LT.1.0D0/VDWEPS) THEN 
                        RPHI=AEXP*(1.0D0-TANH(RPOW)) 
                        RFAC=-VDWBE(N)/COSH(RPOW)**2 
                        FFAC=AEXP*RFAC/RLM1
                      ELSE 
                        RPHI=0.0D0 
                        FFAC=0.0D0 
                      ENDIF 
C-----------------------------------------------------------------------
C     DISPERSIONENERGY AND FORCES ACCORDING TO GRIMME
                    ELSE IF(IDVDW(N).EQ.3) THEN
                      RLM6=RLM2*RLM2*RLM2
                      RPOW=VDWBE(N)*(RLM1/VDWRM(N)-1.0D0)
                      IF(DABS(RPOW).LT.1.0D0/VDWEPS) THEN
                        REXP=1.0D0+DEXP(-RPOW)
                        RPHI=-S6GRIM*VDWST(N)/(RLM6*REXP)
                        FFAC=S6GRIM*VDWST(N)/(RLM6*REXP*RLM1)
     *                  *(6.0D0/RLM1-VDWBE(N)*(REXP-1.0D0)
     *                    /(VDWRM(N)*REXP))
                      ELSE
                        RPHI=0.0D0
                        FFAC=0.0D0
                      ENDIF
C------------------------------------------------------------------------
                    ENDIF
C..Energy
                    EVDW=EVDW+RPHI
C..Ionic forces 
                    FION(1,L,IV)=FION(1,L,IV)-FFAC*RLP(1) 
                    FION(2,L,IV)=FION(2,L,IV)-FFAC*RLP(2) 
                    FION(3,L,IV)=FION(3,L,IV)-FFAC*RLP(3) 
                    FION(1,M,JV)=FION(1,M,JV)+FFAC*RLP(1)
                    FION(2,M,JV)=FION(2,M,JV)+FFAC*RLP(2)
                    FION(3,M,JV)=FION(3,M,JV)+FFAC*RLP(3)
C..Stress tensor 
                    DEVDW(1)=DEVDW(1)
     .                      +FFAC*RLP(ALPHA(1))*RLP(BETA(1))
                    DEVDW(2)=DEVDW(2)
     .                      +FFAC*RLP(ALPHA(2))*RLP(BETA(2))
                    DEVDW(3)=DEVDW(3)
     .                      +FFAC*RLP(ALPHA(3))*RLP(BETA(3))
                    DEVDW(4)=DEVDW(4)
     .                      +FFAC*RLP(ALPHA(4))*RLP(BETA(4))
                    DEVDW(5)=DEVDW(5)
     .                      +FFAC*RLP(ALPHA(5))*RLP(BETA(5))
                    DEVDW(6)=DEVDW(6)
     .                      +FFAC*RLP(ALPHA(6))*RLP(BETA(6))
                  ENDDO 
                ENDDO 
              ENDDO 
            ENDDO 
          ENDDO 
        ENDDO
      ENDIF 
C     ==================================================================
      RETURN
      END
C     ==================================================================
