C     ==================================================================
      SUBROUTINE VHK(RHOE,FHK,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE ONE-PARTICLE POTENTIAL V IN REAL SPACE                  ==
C     ==  FOR THE THOMAS-FERMI FUNCTIONAL OF THE KINETIC ENERGY       ==
C     ==  AND THE EXCHANGE CORRELATION FUNCTIONAL                     ==
C     ==--------------------------------------------------------------==
C     == RHOE:  in electronic density in real space                   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'nlcc.inc'
C     Arguments
      INTEGER    LSCR
      REAL*8     RHOE(NNR1,NLSD),SCR(LSCR),FHK(NNR1,NLSD)
C     Variables
      DIMENSION  F(N)
      COMPLEX*16 VTMP(NHG,NLSX)
      COMPLEX*16 V(MAXFFT,NLSD),VTEMP(*)
      REAL*8     GRAD(NNR1,NLSD*4)
      POINTER    (IP_V,V),(IP_VTEMP,VTEMP)
      POINTER    (IP_VTMP,VTMP),(IP_GRAD,GRAD)
      INTEGER    ISUB,I,IG,IR,NNRS,KK,IL_VTMP,IL_GRAD,LENGTH
      REAL*8     SGCX,SGCC,SXC,VXC,ROE
      LOGICAL    DEBUG
C     ==--------------------------------------------------------------==
      CALL TISET('       VHK',ISUB)
C     ==--------------------------------------------------------------==
      IF(TTAU) THEN
        CALL STOPGM('VHK','META FUNCTIONALS NOT IMPLENTED')
      ENDIF
      SXC=0.D0
      SGCX=0.D0
      SGCC=0.D0
      VXC=0.D0
C     SCR partition.
      DEBUG=.FALSE.
      CALL SCRSET(SCR,LSCR,DEBUG)
      IL_VTMP = MAX(NNR1,NHG*2)*NLSX
      IF(TGC) THEN
        IL_GRAD=NNR1*NLSD*4
      ELSE
        IL_GRAD=0
      ENDIF
      CALL SCRPTR(SCR,LSCR,IP_V,MAXFFT*NLSD*2,'V',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_VTEMP,2*NHG*NLSD,'VTEMP',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_VTMP,IL_VTMP,'VTMP',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_GRAD,IL_GRAD,'GRAD',DEBUG)
C     ==--------------------------------------------------------------==
      IF(TINLC) CALL STOPGM("VHK","NLCC NOT SUPPORTED")
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(V,MAXFFT*NLSD)
      CALL XCENER(SXC,VXC,RHOE,RHOE,V)
C     ==--------------------------------------------------------------==
      IF(.NOT.TLSD) THEN
        DO IR=1,NNR1
          ROE=DMAX1(RHOE(IR,1),0.0D0)
          VXC = 4.785D0*ROE**(0.666666666667D0)
          V(IR,1)=V(IR,1)+DCMPLX(VXC,0.0D0)
        ENDDO
      ELSE
        DO IR=1,NNR1
          ROE=DMAX1(RHOE(IR,1)-RHOE(IR,2),0.0D0)
          VXC = 7.596D0*ROE**(0.666666666667D0)
          V(IR,1)=V(IR,1)+DCMPLX(VXC,0.0D0)
          ROE=DMAX1(RHOE(IR,2),0.0D0)
          VXC = 7.596D0*ROE**(0.666666666667D0)
          V(IR,2)=V(IR,2)+DCMPLX(VXC,0.0D0)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      IF(TGC) THEN
        IF(TLSD) THEN
!$OMP parallel do private(IR)
          DO IR=1,NNR1
            RHOE(IR,1)=RHOE(IR,1)-RHOE(IR,2)
          ENDDO
          CALL GRADEN(RHOE(1,1),V,GRAD(1,1),VTMP)
          CALL GRADEN(RHOE(1,2),V,GRAD(1,5),VTMP)
        ELSE
          CALL GRADEN(RHOE,V,GRAD,VTMP)
        ENDIF
        IF(TLSD) THEN
          CALL GCLSD(SGCX,SGCC,RHOE,V,VTEMP,VTMP,GRAD,.FALSE.)
        ELSE
          CALL GCENER(SGCX,SGCC,RHOE,V,VTEMP,VTMP,GRAD,.FALSE.)
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
!$OMP parallel do private(I,IR)
      DO I=1,NLSD
        DO IR=1,NNR1
          FHK(IR,I) = DREAL(V(IR,I))
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      CALL SCRREL(SCR,LSCR,DEBUG)
C     ==--------------------------------------------------------------==
      CALL TIHALT('       VHK',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_VHK(LVHK,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NHG TGC
      INCLUDE 'spin.inc'        !NLSD NLSX
      INCLUDE 'fft.inc'
C     Arguments
      INTEGER   LVHK
      CHARACTER TAG*30
C     Variables
      INTEGER LTGC,LTINLC
C     ==--------------------------------------------------------------==
      LTINLC=0
      IF(TGC) THEN
        LTGC=NNR1*NLSD*4        !GRADEN
      ELSE
        LTGC=0
      ENDIF
      LVHK = MAX(NNR1,NHG*2)*NLSX+LTGC+LTINLC !GCENER
      LVHK = LVHK + 2*MAXFFT*NLSD + 2*NHG
      LVHK = LVHK + 100  !For boundary checks in SCRPTR
      TAG='MAX(NNR1,NHG*2)*NLSX+NNR1*...'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
