C     ==================================================================
      SUBROUTINE VIBANA(SDER)
C     ==--------------------------------------------------------------==
C     ==  VIBRATIONAL ANALYSIS                                        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'coor.inc'
      INCLUDE 'cotr.inc'
      INCLUDE 'rmas.inc'
      INCLUDE 'symm.inc'
      INCLUDE 'adat.inc'  ! EL
      INCLUDE 'lscal.inc' ! MODE
C     ==--------------------------------------------------------------==
C     Arguments
      REAL*8    SDER(3*NAT,3*NAT)
C     Variables
      CHARACTER TAG*30
      INTEGER   I,IA,INFO,IS,IX,J,K,LSCR,NAUX
      REAL*8    XMASS
      REAL*8    SCR(*),VIBE(3*NAT),XMA(3*NAX*NSX)
      POINTER   (IP_SCR,SCR),(IP_VIBE,VIBE),(IP_XMA,XMA)
C     ==--------------------------------------------------------------==
      IF(.NOT.PARENT) RETURN
      CALL GIVE_SCR_VIBANA(LSCR,TAG)
      CALL MEMORY(IP_SCR,LSCR,'SCR')
      CALL MEMORY(IP_VIBE,3*NAT,'VIBE')
      CALL MEMORY(IP_XMA,3*NAX*NSX,'XMA')
C     Symmetrize Hessian
      IF(INDPG.NE.0.AND.(NROT.NE.1.OR.NTVEC.GT.1)) THEN
        CALL SYMMAT(SDER,SCR,LSCR,2)
      ENDIF
      CALL SYMMA(SDER,3*NAT)
C     Write Hessian to file HESSIAN
      IF(NODIM.EQ.3*NAT) THEN
        CALL DCOPY(NODIM*NODIM,SDER(1,1),1,HESS(1,1),1)
        CALL HESSOUT
      ELSE
        CALL DCOPY(9*NAT*NAT,SDER(1,1),1,HESS(1,1),1)
      ENDIF
C     Write output file for MOLVIB program
      CALL MOLVIB(TAU0,SDER)
C     Mass weighted force constants
      IX=0
      DO IS=1,NSP
        XMASS=SQRT(1.D0/PMA0(IS))
        DO IA=1,NA(IS)
          DO K=1,3
            IX=IX+1
            XMA(IX)=XMASS
          ENDDO
        ENDDO
      ENDDO
!$OMP parallel do private(I,J)
      DO I=1,3*NAT
        DO J=1,3*NAT
          SDER(I,J)=SDER(I,J)*XMA(I)*XMA(J)
        ENDDO
      ENDDO
C     Diagonalization
      NAUX=9*NAT*NAT
      INFO=0
      CALL DSYEV('V','U',3*NAT,SDER,3*NAT,VIBE,SCR,NAUX,INFO)
      IF(INFO.NE.0) CALL STOPGM('SECDER','DSYEV INFO')
!$OMP parallel do private(I)
      DO I=1,3*NAT
         VIBE(I)=SIGN(5140.487D0*DSQRT(DABS(VIBE(I))),VIBE(I))
      ENDDO
      WRITE(6,'(/," ",64("*"),/,A,/)')
     &     ' HARMONIC FREQUENCIES [cm**-1]:'
      WRITE(6,'(4(F16.4))') (VIBE(I),I=1,3*NAT)
C     Write output file for Eigenvectors   
      CALL VIBEIG(VIBE,SDER,3*NAT,.TRUE.)
C     Write a selected mode to standard output
      IF(MODE.NE.0) THEN
         WRITE(6,'(/," ",64("*"),/,A,I3,A,/)')
     &        ' POSITIONS AND HESSIAN EIGENMODE ', MODE,
     &        ' [atomic units]:'
        IX=0
        DO IS=1,NSP
          DO IA=1,NA(IS)
            IX=IX+1
            WRITE(6,'(1X,I3,1X,A2,3F8.4,1X,3(1PE11.3))')
     &           IX,EL(IATYP(IS)),
     &           (TAU0(K,IA,IS),K=1,3),
     &           (SDER(K+(IX-1)*3,MODE),K=1,3)
          ENDDO
        ENDDO
      ENDIF
C     Purification
      WRITE(6,'(/,A)') ' PURIFICATION OF DYNAMICAL MATRIX'
      CALL DCOPY(9*NAT*NAT,HESS,1,SDER,1)
ctu   purification only AFTER mass weighting of the Hessian
ctu   CALL PURGED(SDER,HESS,TAU0)
!$OMP parallel do private(I,J)
      DO I=1,3*NAT
        DO J=1,3*NAT
          SDER(I,J)=SDER(I,J)*XMA(I)*XMA(J)
        ENDDO
      ENDDO
      CALL PURGED(SDER,HESS,TAU0)
C     Diagonalization
      NAUX=9*NAT*NAT
      INFO=0
      CALL DSYEV('V','U',3*NAT,SDER,3*NAT,VIBE,SCR,NAUX,INFO)
      IF(INFO.NE.0) CALL STOPGM('SECDER','DSYEV INFO')
!$OMP parallel do private(I)
      DO I=1,3*NAT
        VIBE(I)=SIGN(5140.487D0*DSQRT(DABS(VIBE(I))),VIBE(I))
      ENDDO
      WRITE(6,'(/," ",64("*"),/,A,/)')
     &     ' HARMONIC FREQUENCIES [cm**-1]:'
      WRITE(6,'(4(F16.4))') (VIBE(I),I=1,3*NAT)
C     Write output file for Eigenvectors   
      CALL VIBEIG(VIBE,SDER,3*NAT,.FALSE.)
C     ==--------------------------------------------------------------==
      CALL FREEM(IP_XMA)
      CALL FREEM(IP_VIBE)
      CALL FREEM(IP_SCR)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE HESFILE(SDER,EIGV,C0,C2,SC0)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'        !NAT
      INCLUDE 'cotr.inc'        !NODIM
      INCLUDE 'coor.inc'        !TAU0
      INCLUDE 'spin.inc'        !NLSD
      INCLUDE 'store.inc'       !RHE
C     Arguments
      REAL*8     SDER(NODIM,NODIM),EIGV(*)
      COMPLEX*16 C0(*),C2(*),SC0(*)
C     Variables
      CHARACTER  TAG*30
      INTEGER    IL_RHOE,IL_PSI,LSCR
      REAL*8     RHOE(NNR1,NLSD),SCR(*),TSCR(3,NAX,NSX)
      COMPLEX*16 PSI(NNR1,NLSD)
      POINTER    (IP_PSI,PSI),(IP_RHOE,RHOE),(IP_SCR,SCR),
     $           (IP_TSCR,TSCR)
      INTEGER    IREC(100)
      DATA       IREC /1,1,1,1,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,1,80*0/
C     ==--------------------------------------------------------------==
      CALL RHOE_PSI_SIZE(IL_RHOE,IL_PSI)
      CALL MEMORY(IP_RHOE,IL_RHOE,'RHOE')
      CALL MEMORY(IP_PSI,IL_PSI,'PSI')
      CALL GIVE_SCR_HESFILE(LSCR,TAG)
      CALL MEMORY(IP_SCR,LSCR,'SCR')
      IF(PARENT) THEN
        CALL MEMORY(IP_TSCR,3*NAX*NSX,'TSCR')
        CALL MEMORY(IP_TAUP,3*NAX*NSX,'TAUP')
        CALL AZZERO(TSCR,3*NAX*NSX)
      ENDIF
C
      CALL INITRUN(IREC,C0,C2,SC0,RHOE,PSI,EIGV,SCR,LSCR)
      IF(PARENT) THEN
        CALL DETDOF(TAU0,TSCR)
        CALL HESSIN(TAU0,RHE)
        CALL DCOPY(9*NAT*NAT,HESS,1,SDER,1)
      ENDIF
C
      IF(PARENT) THEN
        CALL FREEM(IP_TAUP)
        CALL FREEM(IP_TSCR)
      ENDIF
      CALL FREEm(IP_RHOE)
      CALL FREEM(IP_PSI)
      CALL FREEM(IP_SCR)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_VIBANA(LVIBANA,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'        !NAT
C     Arguments
      INTEGER LVIBANA,LSYMMAT
      CHARACTER*30 TAG
C     ==--------------------------------------------------------------==
      LSYMMAT=0
      CALL GIVE_SCR_SYMMAT(LSYMMAT,TAG)
      LVIBANA=MAX(9*NAT*NAT+9*NAT,LSYMMAT)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_HESFILE(LHESFILE,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'        !NAT
C     Arguments
      INTEGER LHESFILE
      CHARACTER*30 TAG
C     Variables
      INTEGER LINITRUN
C     ==--------------------------------------------------------------==
      CALL GIVE_SCR_INITRUN(LINITRUN,TAG)
      LHESFILE=LINITRUN
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
