C     ==================================================================
      SUBROUTINE VLOCST(EPSEU,V,EIVPS)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES DEPS(1:6)                    ==
C     == HARTREE ENERGY CONTRIBUTION TO THE STRESS TENSOR             ==
C     == Nielsen and Martin, PRB 32,p 3793 (1985)                     ==
C     ==--------------------------------------------------------------==
C     == V(1:MAXFFT) [in] electronic density in reciprocal space      ==
C     == EIVPS(1:NHG): [in] phase factor times local pseudopotential  ==
C     ==             used for Vanderbilt pp                           ==
C     == EPSEU: Contribution of local pseudopotential part            ==
C     == Use:                                                         ==
C     ==      GAGK(1:NHG,1:6) GK_alpha * GK_beta                      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'strs.inc'
      INCLUDE 'str2.inc'
      INCLUDE 'ragg.inc'
      INCLUDE 'fft.inc'         !MAXFFT
C     Arguments
      REAL*8     EPSEU
      COMPLEX*16 V(MAXFFT),EIVPS(NHG)
C     Variables
      DIMENSION  DQG(NHG)
      INTEGER    ISUB,KK,IG,IG1
#ifdef __VECTOR 
      COMPLEX*16 TEMP,T1,T2,T3,T4,T5,T6
#endif 
C     ==--------------------------------------------------------------==
      CALL TISET('    VLOCST',ISUB)
      CALL ZAZZERO(DQG,NHG)
C     Calculate derivative local pp part*structure factor
      CALL FFSUM(DQG)
C     Sum excluding G=0
      IF(GEQ0) DQG(1)=(0.0D0,0.0D0)
#if defined(__VECTOR)
      T1=(0.0D0,0.0D0)
      T2=(0.0D0,0.0D0)
      T3=(0.0D0,0.0D0)
      T4=(0.0D0,0.0D0)
      T5=(0.0D0,0.0D0)
      T6=(0.0D0,0.0D0)
#ifdef __SR8000
*poption parallel, tlocal(IG) 
*poption psum(T1,T2,T3,T4,T5,T6)
#endif 
!$OMP parallel do private(IG) 
!$OMP+ reduction(+:T1,T2,T3,T4,T5,T6)
      DO IG=1,NHG
        T1=T1+DCONJG(V(NZH(IG)))*DQG(IG)*GAGK(IG,1)
        T2=T2+DCONJG(V(NZH(IG)))*DQG(IG)*GAGK(IG,2)
        T3=T3+DCONJG(V(NZH(IG)))*DQG(IG)*GAGK(IG,3)
        T4=T4+DCONJG(V(NZH(IG)))*DQG(IG)*GAGK(IG,4)
        T5=T5+DCONJG(V(NZH(IG)))*DQG(IG)*GAGK(IG,5)
        T6=T6+DCONJG(V(NZH(IG)))*DQG(IG)*GAGK(IG,6)
      ENDDO
      DEPST(1)=DEPST(1)+T1
      DEPST(2)=DEPST(2)+T2
      DEPST(3)=DEPST(3)+T3
      DEPST(4)=DEPST(4)+T4
      DEPST(5)=DEPST(5)+T5
      DEPST(6)=DEPST(6)+T6
#else 
!$OMP parallel do private(KK,IG)
      DO KK=1,6
        DO IG=1,NHG
          DEPST(KK)=DEPST(KK)+DCONJG(V(NZH(IG)))*DQG(IG)*GAGK(IG,KK)
        ENDDO
      ENDDO
#endif 
      IF(TIVAN) THEN
        DO KK=1,6
          IG1=1
          IF(GEQ0) IG1=2
          IF(DELTA(ALPHA(KK),BETA(KK)).GT.0.5D0) THEN
#ifdef __VECTOR 
            TEMP=(0.0D0,0.0D0)
#ifdef __SR8000
*poption parallel, tlocal(IG), psum(TEMP)
#endif 
!$OMP parallel do private(IG) reduction(+:TEMP)
            DO IG=IG1,NHG
              TEMP=TEMP+DCONJG(DRHOVG(IG,KK))*EIVPS(IG)
            ENDDO
            DEPST(KK)=DEPST(KK)+TEMP
#else 
            DO IG=IG1,NHG
              DEPST(KK)=DEPST(KK)+DCONJG(DRHOVG(IG,KK))*EIVPS(IG)
            ENDDO
#endif 
          ENDIF
        ENDDO
      ENDIF
C     Factor 2.D0 with GAGK
#ifdef __VECTOR 
      DEPST(1)=2.0D0*DEPST(1)
      DEPST(2)=2.0D0*DEPST(2)
      DEPST(3)=2.0D0*DEPST(3)
      DEPST(4)=2.0D0*DEPST(4)
      DEPST(5)=2.0D0*DEPST(5)
      DEPST(6)=2.0D0*DEPST(6)
#else 
      DO KK=1,6
        DEPST(KK)=2.0D0*DEPST(KK)
      ENDDO
#endif 
C     Add contribution of local part (average non-Coulomb part) 
#ifdef __VECTOR 
      DEPS(1)=-EPSEU*DELTA(ALPHA(1),BETA(1))+OMEGA*DREAL(DEPST(1))
      DEPS(2)=-EPSEU*DELTA(ALPHA(2),BETA(2))+OMEGA*DREAL(DEPST(2))
      DEPS(3)=-EPSEU*DELTA(ALPHA(3),BETA(3))+OMEGA*DREAL(DEPST(3))
      DEPS(4)=-EPSEU*DELTA(ALPHA(4),BETA(4))+OMEGA*DREAL(DEPST(4))
      DEPS(5)=-EPSEU*DELTA(ALPHA(5),BETA(5))+OMEGA*DREAL(DEPST(5))
      DEPS(6)=-EPSEU*DELTA(ALPHA(6),BETA(6))+OMEGA*DREAL(DEPST(6))
#else 
      DO KK=1,6
        DEPS(KK) = -EPSEU*DELTA(ALPHA(KK),BETA(KK)) + 
     *                                       OMEGA*DREAL(DEPST(KK))
      ENDDO
#endif 
      CALL TIHALT('    VLOCST',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
