C     ==================================================================
      SUBROUTINE VOFRHOA(TAU0,FION,RHOE,V,VTEMP,SCR,LSCR,
     *                   TFOR,TSTRESS)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE ONE-PARTICLE POTENTIAL V IN REAL SPACE                  ==
C     ==  THE TOTAL ENERGY ETOT                                       ==
C     ==  THE FORCES FION ACTING ON THE IONS                          ==
C     ==--------------------------------------------------------------==
C     ==  RHOE: in  electronic density in real space                  ==
C     ==  V:    out electronic density in G space                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'simul.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'efld.inc'
      INCLUDE 'elct.inc'        !NEL
CEHR[
      INCLUDE 'td_input.inc'
CEHR]
C     Arguments
      INTEGER    LSCR
      REAL*8     TAU0(3,NAX,NSX),FION(3,NAX,NSX),RHOE(NNR1),SCR(LSCR)
      COMPLEX*16 V(MAXFFT),VTEMP(NHG)
      LOGICAL    TFOR,TSTRESS
C     Variables
      COMPLEX*16 EIVPS(NHG),EIROP(NHG)
      POINTER    (IP_EIVPS,EIVPS),(IP_EIROP,EIROP)
      COMPLEX*16 EH,EI,EE,EPS
      REAL*8     FNL(*),DFNL(*),EXTF(*),EHS
      INTEGER    ISUB,NNRS,LENGTH,IR
      LOGICAL    DEBUG
C     ==--------------------------------------------------------------==
      CALL TISET('   VOFRHOA',ISUB)
C     SCR partition
      DEBUG=.FALSE.
      LENGTH   = 4*NHG + 200
      CALL TEST_SCR('VOFRHOA','LENGTH',LSCR,LENGTH)
C     ==--------------------------------------------------------------==
C     Arrays inside SCR.
      CALL SCRSET(SCR,LSCR,DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_EIVPS,2*NHG,'EIVPS',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_EIROP,2*NHG,'EIROP',DEBUG)
      NNRS  = NR1S*NR2S*NR3S
      CALL EICALC(EIVPS,EIROP)
C..External Field or forces of external potential
      EEXT=0.D0
CEHR[
      IF((TEXTFLD.OR.TEXADD).AND.(.NOT.TD_EXTPOT)) THEN
CEHR]      
        CALL EEXTERN(RHOE,V,EIROP,TAU0,FION,EEXT,TFOR)
#ifdef PARALLEL
        CALL GLOSUM(1,EEXT)
#endif
      ENDIF
C     TRANSFORM THE DENSITY TO G SPACE
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        V(IR)=DCMPLX(RHOE(IR),0.D0)
      ENDDO
      CALL FWFFT(V)
C     ==--------------------------------------------------------------==
C     ==                       PERIODIC SYSTEMS                       ==
C     ==--------------------------------------------------------------==
C     COMPUTE POTENTIAL ENERGY CONTRIBUTION TO FORCES ON IONS
      IF(TFOR) CALL POTFOR(FION,V,EIROP)
C     COMPUTE ELECTROSTATIC AND PSEUDOPOTENTIAL ENERGIES.
      CALL PPENER(EH,EI,EE,EPS,V,VTEMP,EIVPS,EIROP)
C     Hartree term for all charges
      EHEP = DREAL(EH)*OMEGA
C     Ion-Ion interaction (T.D. add VPLOC)
C     NEL*VPLOC is the G=0 part of the Hartree energy, for compatibility
C     reasons we add it to the band energy.
C     (Thierry Deutsch) to have independent term with Raggio
C     add NEL*VPLOC with Hartree energy(IGEQ0 processor has VPLOC!=0)
      EHII = DREAL(EI)*OMEGA + NEL*VPLOC
C     Electron-Electron interaction
      EHEE = DREAL(EE)*OMEGA
      IF(TDIAG) THEN
C       Ion-Ion interaction - e-e (double counting)
        EHT =  EHII-EHEE
     &       + ESR
     &       - ESELF/DBLE(NPROC)
      ELSE
        EHT =  EHEP
     &       + ESR
     &       - ESELF/DBLE(NPROC)
      ENDIF 
C     Local pseudopotential energies
      EPSEU=2.D0*DREAL(EPS)*OMEGA
      IF(TSTRESS) THEN
C       Hartree contribution to stress
        EHS=DREAL(EH)*OMEGA
        CALL HTRSTR(EHS,V,EIROP)
C       Local pseudopotential contribution to stress
        CALL VLOCST(EPSEU,V,EIVPS)
      ENDIF
C     Arrays inside SCR.
      CALL SCRREL(SCR,LSCR,DEBUG)
      CALL TIHALT('   VOFRHOA',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_VOFRHOA(LVOFRHOA,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NHG
      INCLUDE 'isos.inc'        !TCLUST
      INCLUDE 'cppt.inc'        !NR1H NR2H NR3PL
C     Arguments
      INTEGER   LVOFRHOA
      CHARACTER TAG*30
C     Variables
C     ==--------------------------------------------------------------==
      LVOFRHOA = 2*NHG*2 + 200
      TAG      ='2*NHG*2'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
