C     ==================================================================
      SUBROUTINE VOFRHOB(FION,RHOE,V,VTEMP,SCR,LSCR,TFOR,TSTRESS)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE ONE-PARTICLE POTENTIAL V IN REAL SPACE                  ==
C     ==  THE TOTAL ENERGY ETOT                                       ==
C     ==  THE FORCES FION ACTING ON THE IONS                          ==
C     ==--------------------------------------------------------------==
C     == RHOE:  in electronic density in real space                   ==
C     ==       out total electronic potential in real space           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'simul.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'nlcc.inc'
      INCLUDE 'strs.inc'
      INCLUDE 'str2.inc'
      INCLUDE 'efld.inc'
      INCLUDE 'tauf.inc'
      INCLUDE 'extpot.inc'
      INCLUDE 'isos.inc'
C     Arguments
      INTEGER    LSCR
      REAL*8     FION(3,NAX,NSX),
     &           RHOE(NNR1,NLSD),SCR(*)
      COMPLEX*16 V(MAXFFT,*),VTEMP(NHG,*)
      LOGICAL    TFOR,TSTRESS
C     Variables
      DIMENSION  F(N),DQG(*)
      COMPLEX*16 VTMP(NHG,*)
      REAL*8     RHOVAL(NNR1,*),EXTPOT(*),EXTF(*),GRAD(NNR1,*)
      REAL*8     VPT1(*),VPT2(*)
      REAL*8     TAU(NNR1,*),VTAU(NNR1,*)
      POINTER    (IP_VTMP,VTMP),(IP_GRAD,GRAD),(IP_RHOVAL,RHOVAL)
      POINTER    (IP_VPT1,VPT1),(IP_VPT2,VPT2)
      REAL*8     FNL(IMAGP,LDF1,NHXS,*),DFNL(*),SXC,SGCX,SGCC,VGC
      INTEGER    ISUB,IG,IR,NNRS,KK,
     &           IL_VTMP,IL_GRAD,IL_RHOVAL,IL_VPT1,IL_VPT2
      LOGICAL    DEBUG
C     ==--------------------------------------------------------------==
      CALL TISET('   VOFRHOB',ISUB)
C     SCR partition.
      DEBUG=.FALSE.
      CALL SCRSET(SCR,LSCR,DEBUG)
      IL_VTMP = MAX(NNR1,NHG*2)*NLSX      !GCENER and COREC
      IF(TGC) THEN
        IL_GRAD=NNR1*NLSD*4     !GRADEN
      ELSE
        IL_GRAD=0
      ENDIF
      IF(TINLC) THEN
        IL_RHOVAL=NNR1*NLSD     !XCENER
      ELSE
        IL_RHOVAL=0
      ENDIF
      IF(TPOTENTIAL) THEN
        IL_VPT1=NNR1*NLSD
        IL_VPT2=NNR1*NLSD
      ELSE
        IL_VPT1=0
        IL_VPT2=0
      ENDIF
      CALL SCRPTR(SCR,LSCR,IP_VTMP,IL_VTMP,'VTMP',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_GRAD,IL_GRAD,'GRAD',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_RHOVAL,IL_RHOVAL,'RHOVAL',DEBUG)
C     ==--------------------------------------------------------------==
C     == ADD CORE CHARGE TO RHOE                                      ==
C     ==--------------------------------------------------------------==
      IF(TINLC) THEN
        CALL DCOPY(NNR1*NLSD,RHOE(1,1),1,RHOVAL(1,1),1)
        CALL COREC(RHOE,VTMP,V)
        CALL DCOPY(2*NHG,VTEMP(1,1),1,VNLT(1),1)
      ENDIF
C     ==-------------------------------------------------------------==
C     == VTEMP (Potential in G-Space) -FFT-> V(R)                     ==
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(V,MAXFFT)
C$OMP parallel do private(IG)
#ifdef __NEC
!CDIR NODEP
#endif
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC(V)
#endif
      DO IG=1,NHG
        V(INDZ(IG),1) = DCONJG(VTEMP(IG,1))
        V(NZH(IG),1)  = VTEMP(IG,1)
      ENDDO
      IF(GEQ0) V(NZH(1),1) = VTEMP(1,1)
      CALL INVFFT(V)
C     ==--------------------------------------------------------------==
C     == ADD EXTERNAL POTENTIAL TO V                                  ==
C     ==--------------------------------------------------------------==
      IF(TEXTFLD) THEN
C..potential from classical interface
C$OMP parallel do private(IR) shared(V)
        DO IR=1,NNR1
          V(IR,1)=V(IR,1)+DCMPLX(EXTF(IR),0.D0)
        ENDDO
      ENDIF
      IF(TEXPOT) THEN
C..static external potential
C$OMP parallel do private(IR) shared(V)
        DO IR=1,NNR1
          V(IR,1)=V(IR,1)+DCMPLX(EXTPOT(IR),0.D0)
        ENDDO
      ENDIF
      IF((TEXPOT.OR.TEXTFLD).AND.TINLC) THEN
        CALL FWFFT(V)
C$OMP parallel do private(IG) shared(V)
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOALIAS
#endif
        DO IG=1,NHG
          VNLT(IG) = V(NZH(IG),1)
        ENDDO
        CALL INVFFT(V)
      ENDIF
C     == ADD TO THE POTENTIAL THE CONTRIBUTION FROM SPIN LOCALIZATION ==
      CALL LOCALIZESPIN(RHOE,V)
c     == ADD TO THE POTENTIAL THE SIC CONTRIBUTION ==
      IF(TSIC) THEN
        IF(PS_TYPE.EQ.1) THEN
          CALL STOPGM('SSIC','HOCKNEY POISSON SOLVER NOT IMPLEMENTED')
        ELSE
          CALL SSIC(EHSIC,RHOE,V) ! cmb_ssic
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
C     == COMPUTE EXCHANGE AND CORRELATION ENERGY (EXC)                ==
C     ==--------------------------------------------------------------==
      IF(TINLC) THEN
        CALL XCENER(SXC,VXC,RHOVAL,RHOE,V)
      ELSE
        CALL XCENER(SXC,VXC,RHOE,RHOE,V)
      ENDIF
      SGCX = 0.0D0
      SGCC = 0.0D0
      VGC  = 0.0D0
      IF(TGC) THEN
C       ==------------------------------------------------------------==
C       == CALCULATE THE GRADIENT OF THE DENSITY                      ==
C       ==------------------------------------------------------------==
        IF(TLSD) THEN
          IF(TSTRESS.OR.TDIAG) THEN
            CALL DCOPY(2*NNR1,V(1,1),1,DQG(1),1)
            CALL DCOPY(2*NNR1,V(1,2),1,DQG(NNR1+1),1)
          ENDIF
          CALL FWFFT(V(1,1))
          CALL ZGTHR(NHG,V(1,1),VTEMP(1,1),NZH)
          CALL FWFFT(V(1,2))
          CALL ZGTHR(NHG,V(1,2),VTEMP(1,2),NZH)
C$OMP parallel do private(IR) shared(RHOE)
          DO IR=1,NNR1
            RHOE(IR,1)=RHOE(IR,1)-RHOE(IR,2)
          ENDDO
          CALL GRADEN(RHOE(1,1),V,GRAD(1,1),VTMP)
          CALL GRADEN(RHOE(1,2),V,GRAD(1,5),VTMP)
        ELSE
          IF(TSTRESS.OR.TDIAG) CALL DCOPY(2*NNR1,V(1,1),1,DQG(1),1)
          CALL FWFFT(V)
          CALL ZGTHR(NHG,V,VTEMP,NZH)
          CALL GRADEN(RHOE,V,GRAD,VTMP)
        ENDIF
C     ==--------------------------------------------------------------==
C     == GRADIENT CORRECTION TO THE EXCHANGE ENERGY (EGCX)            ==
C     ==--------------------------------------------------------------==
        IF(TSTRESS .AND. TTAU) 
     *     CALL STOPGM('VOFRHOB','NO STRESS WITH TAU FUNCTIONAL')
        IF(TPOTENTIAL) THEN
          IF(TSTRESS) CALL STOPGM('VOFRHOB','NO STRESS WITH TPOTENTIAL')
          IF(TIVAN) CALL STOPGM('VOFRHOB','NO VDB PP WITH TPOTENTIAL')
          CALL SCRPTR(SCR,LSCR,IP_VPT1,IL_VPT1,'VPT1',DEBUG)
          CALL SCRPTR(SCR,LSCR,IP_VPT2,IL_VPT2,'VPT2',DEBUG)
          CALL VPOTENTIAL(RHOE,GRAD,V,VTEMP,VTMP,VPT1,VPT2)
        ELSEIF(TLSD) THEN
c         IF(TSTRESS) CALL STOPGM('VOFRHOB','NO STRESS WITH LSD AND GC')
          CALL GCLSD(SGCX,SGCC,RHOE,V,VTEMP,VTMP,GRAD,TSTRESS)
          IF(TSTRESS.OR.TDIAG) THEN
            CALL DAXPY(2*NNR1,-1.0D0,V(1,1),1,DQG(1),1)
            CALL DAXPY(2*NNR1,-1.0D0,V(1,2),1,DQG(NNR1+1),1)
            CALL DSCAL(4*NNR1,-1.0D0,DQG(1),1)
            VGC=0.0D0
            IF(TINLC) THEN
#if defined(__VECTOR)
C$OMP parallel do private(IR) reduction(+:VGC)
#else
C$OMP parallel do private(IR) reduction(+:VGC) schedule(static)
#endif
              DO IR=1,NNR1
                VGC=VGC+(RHOVAL(IR,1)-RHOVAL(IR,2))*DREAL(DQG(IR))
                VGC=VGC+RHOVAL(IR,2)*DREAL(DQG(NNR1+IR))
              ENDDO
            ELSE
#if defined(__VECTOR)
C$OMP parallel do private(IR) reduction(+:VGC)
#else
C$OMP parallel do private(IR) reduction(+:VGC) schedule(static)
#endif
              DO IR=1,NNR1
                VGC=VGC+RHOE(IR,1)*DREAL(DQG(IR))
                VGC=VGC+RHOE(IR,2)*DREAL(DQG(NNR1+IR))
              ENDDO
            ENDIF
            IF(TTAU) THEN
#if defined(__VECTOR)
C$OMP parallel do private(IR) reduction(+:VGC)
#else
C$OMP parallel do private(IR) reduction(+:VGC) schedule(static)
#endif
              DO IR=1,NNR1
                VGC=VGC+(TAU(IR,1)*VTAU(IR,1)+TAU(IR,2)*VTAU(IR,2))
              ENDDO
            ENDIF
          ENDIF
        ELSE
          CALL GCENER(SGCX,SGCC,RHOE,V,VTEMP,VTMP,GRAD,TSTRESS)
          IF(TSTRESS.OR.TDIAG) THEN
            CALL DAXPY(2*NNR1,-1.0D0,V(1,1),1,DQG(1),1)
            CALL DSCAL(2*NNR1,-1.0D0,DQG(1),1)
            VGC=0.0D0
            IF(TINLC) THEN
#if defined(__VECTOR)
C$OMP parallel do private(IR) reduction(+:VGC)
#else
C$OMP parallel do private(IR) reduction(+:VGC) schedule(static)
#endif
#ifdef __SR8000
*poption parallel, tlocal(IR), psum(VGC)
#endif 
              DO IR=1,NNR1
                VGC=VGC+RHOVAL(IR,1)*DREAL(DQG(IR))
              ENDDO
            ELSE
#if defined(__VECTOR)
C$OMP parallel do private(IR) reduction(+:VGC)
#else
C$OMP parallel do private(IR) reduction(+:VGC) schedule(static)
#endif
#ifdef __SR8000
*poption parallel, tlocal(IR), psum(VGC)
#endif 
              DO IR=1,NNR1
                VGC=VGC+RHOE(IR,1)*DREAL(DQG(IR))
              ENDDO
            ENDIF
            IF(TTAU) THEN
#if defined(__VECTOR)
C$OMP parallel do private(IR) reduction(+:VGC)
#else
C$OMP parallel do private(IR) reduction(+:VGC) schedule(static)
#endif
#ifdef __SR8000
*poption parallel, tlocal(IR), psum(VGC)
#endif
              DO IR=1,NNR1
                VGC=VGC+TAU(IR,1)*VTAU(IR,1)
              ENDDO
            ENDIF
          ENDIF
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
C     == XC CONTRIBUTION TO STRESS (VANDERBILT CHARGE)                ==
C     ==--------------------------------------------------------------==
      IF(TIVAN.AND.TSTRESS) 
     *     CALL XCSTR(DRHOVG,RHOE,DQG,DQG(MAXFFT+1)) 
C     ==--------------------------------------------------------------==
C     == NLCC CONTRIBUTION TO STRESS                                  ==
C     ==--------------------------------------------------------------==
      IF(TINLC.AND.TSTRESS) CALL NLCCSTR(DQG,DQG(NLSD*MAXFFT+1))
C     ==--------------------------------------------------------------==
C     == RELEASE TEMPORARY ARRAYS FROM SCRATCH SPACE                  ==
C     ==--------------------------------------------------------------==
      CALL SCRREL(SCR,LSCR,DEBUG)
C     ==--------------------------------------------------------------==
C     == V CONTAINS THE TOTAL POTENTIAL IN R-SPACE                    ==
C     == MOVE IT TO RHOE                                              ==
C     ==--------------------------------------------------------------==
#if defined (__VECTOR)
C$OMP parallel do private(IR)
#else
C$OMP parallel do private(IR) schedule(static)
#endif
      DO IR=1,NNR1
        RHOE(IR,1)=DREAL(V(IR,1))
      ENDDO
      IF(TLSD) THEN
#if defined (__VECTOR)
C$OMP parallel do private(IR)
#else
C$OMP parallel do private(IR) schedule(static)
#endif
        DO IR=1,NNR1
          RHOE(IR,2)=DREAL(V(IR,2))
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     == CORRECT ENERGY FOR CORE CHARGE                               ==
C     ==--------------------------------------------------------------==
      IF(TINLC) THEN
        SXC=SXC-EXCCO
        SGCX=SGCX-EGCXCO
        SGCC=SGCC-EGCCCO
      ENDIF
      IF(TIVAN.OR.(TINLC.AND.TFOR)) THEN
C     ==--------------------------------------------------------------==
C     == TRANSF. TOTAL POTENTIAL IN G-SPACE                           ==
C     == PUT THE TOTAL POTENTIAL IN G-SPACE INTO VTEMP                ==
C     ==--------------------------------------------------------------==
        CALL FWFFT(V)
        CALL ZGTHR(NHG,V,VTEMP,NZH)
        IF(TLSD) THEN
          CALL FWFFT(V(1,2))
          CALL ZGTHR(NHG,V(1,2),VTEMP(1,2),NZH)
        ENDIF
      ENDIF
      IF(TIVAN) THEN
        IF(IMAGP.EQ.2)CALL STOPGM('VOFRHOB','K-POINT NOT IMPLEMENTED')
C     ==--------------------------------------------------------------==
C     == CALCULATE DEEQ FOR VANDERBILT PP                             ==
C     == FORCE ON IONS DUE TO THE "VANDERBILT CHARGE"                 ==
C     ==--------------------------------------------------------------==
        IF(TLSD) THEN
          CALL NEWD(FNL(1,1,1,1),DEEQ(1,1,1,1),F(1),
     *              VTEMP(1,1),FION,SCR,LSCR,NSUP,TFOR)
          CALL NEWD(FNL(1,1,1,NSUP+1),DEEQ(1,1,1,2),F(NSUP+1),
     &              VTEMP(1,2),FION,SCR,LSCR,NSDOWN,TFOR)
        ELSE
          CALL NEWD(FNL,DEEQ,F,VTEMP,FION,SCR,LSCR,N,TFOR)
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
C     == FORCE ON IONS DUE TO NLCC                                    ==
C     ==--------------------------------------------------------------==
      IF(TINLC.AND.TFOR) CALL COFOR(FION,VTEMP)
C     ==--------------------------------------------------------------==
C     == SINGLE PIECES OF THE ENERGY:                                 ==
C     ==--------------------------------------------------------------==
      NNRS = NR1S*NR2S*NR3S
      EGC=(SGCC+SGCX)*OMEGA/DBLE(NNRS)
      EXC=SXC*OMEGA/DBLE(NNRS) + EGC
      VXC=(VXC + VGC)*OMEGA/DBLE(NNRS) 
      IF(TSTRESS) THEN
        DO KK=1,6
          DEXC(KK) = DEXC(KK) + (EXC-VXC)*DELTA(ALPHA(KK),BETA(KK))
        ENDDO
      ENDIF
      CALL TIHALT('   VOFRHOB',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_VOFRHOB(LVOFRHOB,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NHG TGC
      INCLUDE 'spin.inc'        !NLSD NLSX
      INCLUDE 'pslo.inc'        !TIVAN
      INCLUDE 'nlcc.inc'        !TINLC
C     Arguments
      INTEGER   LVOFRHOB
      CHARACTER TAG*30
C     Variables
      INTEGER LTGC,LTINLC,LVTMP,LNEWD
C     ==--------------------------------------------------------------==
      IF(TGC) THEN
        LTGC=NNR1*NLSD*4        !GRADEN
      ELSE
        LTGC=0
      ENDIF
      IF(TINLC) THEN            !XCENER
        LTINLC=NNR1*NLSD
      ELSE
        LTINLC=0
      ENDIF
      LVTMP=MAX(NNR1,NHG*2)*NLSX
      IF(TPOTENTIAL) THEN
        LVTMP=LVTMP+2*NNR1*NLSD
      ENDIF
      LVOFRHOB = LVTMP+LTGC+LTINLC !GCENER and COREC
      IF(TIVAN) THEN
        CALL GIVE_SCR_NEWD(LNEWD,TAG)
        LVOFRHOB=MAX(LVOFRHOB,LNEWD) 
      ENDIF
      LVOFRHOB = LVOFRHOB + 100  !For boundary checks in SCRPTR
      TAG='MAX(NNR1,NHG*2)*NLSX+.....'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE VPOTENTIAL(RHOE,GRAD,PSI,VTEMP,V1,V2,V3)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'tpot.inc'
      INCLUDE 'linres.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'fft.inc'
      REAL*8  RHOE(*),GRAD(NNR1,*),V1(NNR1,*),V2(*),V3(*)
      COMPLEX*16 PSI(MAXFFT,*),VTEMP(NHG,*)
      INTEGER IR,IG
      REAL*8  EXC
C     ==--------------------------------------------------------------==
      IF(TLSD) THEN
C$OMP parallel do private(IR)
        DO IR=1,NNR1
          V2(IR) = GRAD(IR,2)*GRAD(IR,6)+GRAD(IR,3)*GRAD(IR,7)+
     *             GRAD(IR,4)*GRAD(IR,8)
        ENDDO
        CALL DCOPY(NNR1,GRAD(1,5),1,GRAD(1,3),1)
        CALL DCOPY(NNR1,V2,1,GRAD(1,2),1)
      ENDIF
      CALL AZZERO(V1,NNR1*NLSD)
      IF(TD_FUNCTIONAL.EQ.10) THEN
        CALL SAOP(C0Y,EIGVAL,EIGREF,FOCCP,MSTATE,RHOE,GRAD,EXC,V1,V2,V3,
     *            GRAD(1,4),PSI)
      ELSEIF(TD_FUNCTIONAL.EQ.11) THEN
        CALL LB94M(RHOE,GRAD,EXC,V1)
      ELSEIF(TD_FUNCTIONAL.EQ.12) THEN
        CALL GLLB(C0Y,EIGVAL,EIGREF,FOCCP,MSTATE,RHOE,GRAD,EXC,
     *            V1,V2,PSI)
      ELSE
        CALL STOPGM("VPOTENTIAL","ILLEGAL TD_FUNCTIONAL")
      ENDIF
C
      IF(TLSD) THEN
        CALL ZAZZERO(PSI,MAXFFT)
C$OMP parallel do private(IG) shared(PSI)
        DO IG=1,NHG
          PSI(NZH(IG),1) = VTEMP(IG,1)
          PSI(INDZ(IG),1) = DCONJG(VTEMP(IG,1))
        ENDDO
        CALL INVFFT(PSI)
C$OMP parallel do private(IR) shared(PSI)
        DO IR=1,NNR1
          PSI(IR,1)=PSI(IR,1)+V1(IR,1)
        ENDDO
        CALL ZAZZERO(PSI(1,2),MAXFFT)
C$OMP parallel do private(IG) shared(PSI)
        DO IG=1,NHG
          PSI(NZH(IG),2) = VTEMP(IG,2)
          PSI(INDZ(IG),2) = DCONJG(VTEMP(IG,2))
        ENDDO
        CALL INVFFT(PSI(1,2))
C$OMP parallel do private(IR) shared(PSI)
        DO IR=1,NNR1
          PSI(IR,2)=PSI(IR,2)+V1(IR,2)
        ENDDO
      ELSE
        CALL ZAZZERO(PSI,MAXFFT)
C$OMP parallel do private(IG) shared(PSI)
        DO IG=1,NHG
          PSI(NZH(IG),1) = VTEMP(IG,1)
          PSI(INDZ(IG),1) = DCONJG(VTEMP(IG,1))
        ENDDO
        CALL INVFFT(PSI)
C$OMP parallel do private(IR) shared(PSI)
        DO IR=1,NNR1
          PSI(IR,1)=PSI(IR,1)+V1(IR,1)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
