C     ==================================================================
      SUBROUTINE VOFRHOC(TAU0,RHOE,V,VTEMP,SCR,LSCR,TFOR,TSTRESS)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE HARTREE AND XC ENERGY AND POTENTIALS (+FORCES) FOR      ==
C     ==  THE GROUND STATE AND OFF-DIAGONAL ELEMENT IN THE CAS22 METH.==
C     ==--------------------------------------------------------------==
C     ==  RHOE(.,1) Density of all 2 occ states                       ==
C     ==  RHOE(.,2) RHO(ab)                                           ==
C     ==  RHOE(.,3) "Ground state" density                            ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'simul.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'nlcc.inc'
      INCLUDE 'fcas.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'efld.inc'
      INCLUDE 'ropt.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'extpot.inc'
C     Arguments
      INTEGER    LSCR
      REAL*8     TAU0(3,NAX,NSX),RHOE(NNR1,4),SCR(LSCR)
      COMPLEX*16 V(MAXFFT,NLSD),VTEMP(NHG,*)
      LOGICAL    TFOR,TSTRESS
C     Variables
      REAL*8     FNL(*),DFNL(*)
      COMPLEX*16 EIVPS(NHG),EIROP(NHG),GRAD(*),VTMP(*)
      POINTER    (IP_EIVPS,EIVPS),(IP_EIROP,EIROP),(IP_GRAD,GRAD),
     &           (IP_VTMP,VTMP),(IP_FOCC,FOCC)
      REAL*8     EXTF(*),EXTPOT(*),SGCX,SGCC,SXC,
     &           EGCA,SXCAB,RINT(4),FIDE(1),WK,ESX,FOCC(*)
      INTEGER    ISUB,IR,IL_VTMP,NNRS,IG
      LOGICAL DEBUG
      COMPLEX*16 EH,EI,EE,EPS,EHAB,EPSAB
C     ==--------------------------------------------------------------==
      CALL TISET('   VOFRHOC',ISUB)
      IF(TSTRESS) CALL STOPGM("VOFRHOC","STRESS NOT IMPLEMENTED")
      IF(TTAU) THEN
        CALL STOPGM('VOFRHOC','META FUNCTIONALS NOT IMPLENTED')
      ENDIF
      DEBUG=.FALSE.
C     ==--------------------------------------------------------------==
      IF(TFOR) THEN
        IF(INIT_FLAG.EQ.0) THEN
          CALL MEMORY(IP_FION_A,3*NAX*NSX,"FION_A")
          CALL MEMORY(IP_FION_AB,3*NAX*NSX,"FION_AB")
          INIT_FLAG=1
        ENDIF
        CALL AZZERO(FION_A,3*NAX*NSX)
        CALL RPIIINT(ESX,TAU0,FION_A,IESR,TFOR)
        CALL AZZERO(FION_AB,3*NAX*NSX)
      ENDIF
C     ==--------------------------------------------------------------==
C     Test integral of densities
      CALL AZZERO(RINT,4)
      DO IR=1,NNR1
        RINT(1)=RINT(1)+RHOE(IR,1)
        RINT(2)=RINT(2)+RHOE(IR,2)
        RINT(3)=RINT(3)+RHOE(IR,3)
        RINT(4)=RINT(4)+RHOE(IR,4)
      ENDDO
      CALL GLOSUM(4,RINT)
      DO IR=1,4
        RINT(IR)=RINT(IR)*OMEGA/DBLE(NR1S*NR2S*NR3S)
      ENDDO
      IF(PARENT) THEN
        IF(ABS(RINT(1)-(NEL-2.D0)).GT.1.D-5) 
     &     WRITE(6,*) " WARNING : DENSITY RHO(1..N) ",RINT(1)
        IF(ABS(RINT(2)).GT.1.D-5) 
     &     WRITE(6,*) " WARNING : DENSITY RHO(A..B) ",RINT(2)
        IF(ABS(RINT(3)-NEL).GT.1.D-5) 
     &     WRITE(6,*) " WARNING : DENSITY RHO(1..A) ",RINT(3)
        IF(ABS(RINT(4)-NEL).GT.1.D-5) 
     &     WRITE(6,*) " WARNING : DENSITY RHO(1..B) ",RINT(4)
      ENDIF
C     SCR partition
      CALL SCRSET(SCR,LSCR,DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_EIVPS,2*NHG,'EIVPS',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_EIROP,2*NHG,'EIROP',DEBUG)
C     ==--------------------------------------------------------------==
      CALL EICALC(EIVPS,EIROP)
C     External Field
      EEXT_A=0.D0
      EEXT_AB=0.D0
      IF(TEXTFLD) THEN
        CALL EEXTERN(RHOE(1,2),V,EIROP,TAU0,FION_AB,EEXT_AB,TFOR)
        CALL EEXTERN(RHOE(1,3),V,EIROP,TAU0,FION_A,EEXT_A,TFOR)
        CALL EEXTERN(RHOE(1,4),V,EIROP,TAU0,FIDE,EEXT_2,.FALSE.)
      ENDIF
C     ==----------------HARTREE AND PSEUOPOTENTIAL--------------------==
C     TRANSFORM THE DENSITY TO G SPACE
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        V(IR,1)=DCMPLX(RHOE(IR,1),0.D0)
        V(IR,2)=DCMPLX(RHOE(IR,2),0.D0)
        V(IR,3)=DCMPLX(RHOE(IR,3),0.D0)
        V(IR,4)=DCMPLX(RHOE(IR,4),0.D0)
      ENDDO
      CALL FWFFT(V(1,1))
      CALL FWFFT(V(1,2))
      CALL FWFFT(V(1,3))
      CALL FWFFT(V(1,4))
C     COMPUTE CONTRIBUTION TO FORCES ON IONS FROM GROUND STATE
      IF(TFOR) THEN 
        CALL POTFOR(FION_A,V(1,3),EIROP)
        CALL PCASFOR(FION_AB,V(1,2),EIROP)
        WK=1.D0
        CALL SCRPTR(SCR,LSCR,IP_FOCC,N,'FOCC',DEBUG)
!$OMP parallel do private(IR)
        DO IR=1,N-1
          FOCC(IR)=2.D0
        ENDDO
        FOCC(IR)=0.D0
        CALL RNLFOR(FION_A,FOCC,WK,N,1)
        CALL SCRPTF(SCR,LSCR,IP_FOCC,N,'FOCC',DEBUG)
      ENDIF
C     COMPUTE ELECTROSTATIC AND PSEUDOPOTENTIAL ENERGIES.
      CALL PPENER(EH,EI,EE,EPS,V(1,3),VTEMP(1,3),EIVPS,EIROP)
      CALL HVL_AB(EHAB,EPSAB,V(1,1),V(1,2),VTEMP(1,1),VTEMP(1,2),
     &            EIVPS,EIROP)
      EHT_A = DREAL(EH)*OMEGA
      EPSEU_A = 2.D0*DREAL(EPS)*OMEGA
      EHT_AB = DREAL(EHAB)*OMEGA
      EPSEU_AB = 2.D0*DREAL(EPSAB)*OMEGA
      CALL PPENER(EH,EI,EE,EPS,V(1,4),VTEMP(1,4),EIVPS,EIROP)
      EHT_2 = DREAL(EH)*OMEGA
      EPSEU_2 = 2.D0*DREAL(EPS)*OMEGA
      CALL SCRPTF(SCR,LSCR,IP_EIVPS,2*NHG,'EIVPS',DEBUG)
      CALL SCRPTF(SCR,LSCR,IP_EIROP,2*NHG,'EIROP',DEBUG)
C     ADD CORE CHARGE TO RHOE
      IF(TINLC) THEN
        CALL STOPGM("VOFRHOC","NLCC not implemented")
        CALL COREC(RHOE(1,3),VTMP,V(1,3))
        CALL DCOPY(2*NHG,VTEMP(1,3),1,VNLT(1),1)
      ENDIF
C     VTEMP (Potential in G-Space) -FFT-> V(R)
      CALL ZAZZERO(V(1,1),MAXFFT)
      DO IG=1,NHG
        V(INDZ(IG),1) = DCONJG(VTEMP(IG,1))
        V(NZH(IG),1)  = VTEMP(IG,1)
      ENDDO
      IF(GEQ0) THEN
        V(NZH(1),1) = VTEMP(1,1)
      ENDIF
      CALL INVFFT(V(1,1))
C
      CALL ZAZZERO(V(1,2),MAXFFT)
      DO IG=1,NHG
        V(INDZ(IG),2) = DCONJG(VTEMP(IG,2))
        V(NZH(IG),2)  = VTEMP(IG,2)
      ENDDO
      IF(GEQ0) THEN
        V(NZH(1),2) = VTEMP(1,2)
      ENDIF
      CALL INVFFT(V(1,2))
C
      CALL ZAZZERO(V(1,3),MAXFFT)
      DO IG=1,NHG
        V(INDZ(IG),3) = DCONJG(VTEMP(IG,3))
        V(NZH(IG),3)  = VTEMP(IG,3)
      ENDDO
      IF(GEQ0) THEN
        V(NZH(1),3) = VTEMP(1,3)
      ENDIF
      CALL INVFFT(V(1,3))
C     ADD EXTERNAL POTENTIAL TO V
      IF(TEXTFLD) THEN
C     potential from classical interface
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          V(IR,1)=V(IR,1)+DCMPLX(EXTF(IR),0.D0)
          V(IR,2)=V(IR,2)+DCMPLX(EXTF(IR),0.D0)
          V(IR,3)=V(IR,3)+DCMPLX(EXTF(IR),0.D0)
        ENDDO
      ENDIF
      IF(TEXPOT) THEN
C     static external potential
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          V(IR,1)=V(IR,1)+DCMPLX(EXTPOT(IR),0.D0)
          V(IR,2)=V(IR,2)+DCMPLX(EXTPOT(IR),0.D0)
          V(IR,3)=V(IR,3)+DCMPLX(EXTPOT(IR),0.D0)
        ENDDO
      ENDIF
C     XC ENERGY FOR INTERACTION TERM
      CALL INTERXC(SXCAB,RHOE(1,1),RHOE(1,2),V(1,1),V(1,2))
C     COMPUTE EXCHANGE AND CORRELATION ENERGY (EXC)
      CALL XCENER(SXC,VXC,RHOE(1,3),RHOE(1,3),V(1,3))
      SGCX = 0.0D0
      SGCC = 0.0D0
      IF(TGC) THEN
C       CALCULATE THE GRADIENT OF THE DENSITY
        CALL SCRPTR(SCR,LSCR,IP_GRAD,4*NNR1,'GRAD',DEBUG)
        IL_VTMP = MAX(NNR1,NHG*2)
        CALL SCRPTR(SCR,LSCR,IP_VTMP,IL_VTMP,'VTMP',DEBUG)
        CALL FWFFT(V(1,3))
        CALL ZGTHR(NHG,V(1,3),VTEMP(1,3),NZH)
        CALL GRADEN(RHOE(1,3),V(1,3),GRAD,VTMP)
C       GRADIENT CORRECTION TO THE EXCHANGE ENERGY (EGCX)
        CALL GCENER(SGCX,SGCC,RHOE(1,3),V(1,3),VTEMP(1,3),
     &              VTMP,GRAD,TSTRESS)
        CALL SCRPTF(SCR,LSCR,IP_GRAD,4*NNR1,'GRAD',DEBUG)
        CALL SCRPTF(SCR,LSCR,IP_VTMP,IL_VTMP,'VTMP',DEBUG)
      ENDIF
C     V CONTAINS THE TOTAL POTENTIAL IN R-SPACE, MOVE IT TO RHOE
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        RHOE(IR,1)=DREAL(V(IR,1))
        RHOE(IR,2)=DREAL(V(IR,2))
        RHOE(IR,3)=DREAL(V(IR,3))
      ENDDO
C     CORRECT ENERGY FOR CORE CHARGE
      IF(TINLC) THEN
        SXC=SXC-EXCCO
        SGCX=SGCX-EGCXCO
        SGCC=SGCC-EGCCCO
      ENDIF
C     SINGLE PIECES OF THE ENERGY:
      NNRS = NR1S*NR2S*NR3S
      EGCA=(SGCC+SGCX)*OMEGA/DBLE(NNRS)
      EXC_A=SXC*OMEGA/DBLE(NNRS) + EGCA
      EXC_AB=SXCAB*OMEGA/DBLE(NNRS)
C     COMPUTE EXCHANGE AND CORRELATION ENERGY (EXC)
      CALL XCENER(SXC,VXC,RHOE(1,4),RHOE(1,4),V(1,4))
      SGCX = 0.0D0
      SGCC = 0.0D0
      IF(TGC) THEN
C       CALCULATE THE GRADIENT OF THE DENSITY
        CALL SCRPTR(SCR,LSCR,IP_GRAD,4*NNR1,'GRAD',DEBUG)
        IL_VTMP = MAX(NNR1,NHG*2)
        CALL SCRPTR(SCR,LSCR,IP_VTMP,IL_VTMP,'VTMP',DEBUG)
        CALL FWFFT(V(1,4))
        CALL ZGTHR(NHG,V(1,4),VTEMP(1,4),NZH)
        CALL GRADEN(RHOE(1,4),V(1,4),GRAD,VTMP)
C       GRADIENT CORRECTION TO THE EXCHANGE ENERGY (EGCX)
        CALL GCENER(SGCX,SGCC,RHOE(1,4),V(1,4),VTEMP(1,4),
     &              VTMP,GRAD,TSTRESS)
        CALL SCRPTF(SCR,LSCR,IP_GRAD,4*NNR1,'GRAD',DEBUG)
        CALL SCRPTF(SCR,LSCR,IP_VTMP,IL_VTMP,'VTMP',DEBUG)
      ENDIF
C     CORRECT ENERGY FOR CORE CHARGE
      IF(TINLC) THEN
        SXC=SXC-EXCCO
        SGCX=SGCX-EGCXCO
        SGCC=SGCC-EGCCCO
      ENDIF
C     SINGLE PIECES OF THE ENERGY:
      NNRS = NR1S*NR2S*NR3S
      EGCA=(SGCC+SGCX)*OMEGA/DBLE(NNRS)
      EXC_2=SXC*OMEGA/DBLE(NNRS) + EGCA
      IF(TINLC.AND.TFOR) THEN
C       TRANSF. TOTAL POTENTIAL IN G-SPACE
C       PUT THE TOTAL POTENTIAL IN G-SPACE INTO VTEMP
        CALL FWFFT(V(1,3))
        CALL ZGTHR(NHG,V(1,3),VTEMP(1,3),NZH)
      ENDIF
C     FORCE ON IONS DUE TO NLCC
      IF(TINLC.AND.TFOR) CALL COFOR(FION_A,VTEMP(1,3))
C
      CALL SCRREL(SCR,LSCR,DEBUG)
      CALL TIHALT('   VOFRHOC',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_VOFRHOC(LVOFRHOC,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NHG
      INCLUDE 'elct.inc'        !NHG
C     Arguments
      INTEGER   LVOFRHOC
      CHARACTER TAG*30
C     Variables
      INTEGER   LHT,LXC
C     ==--------------------------------------------------------------==
      LHT      = 2*NHG*2 + N
      IF(TGC) THEN
        LXC    = 4*NNR1 + MAX(NNR1,2*NHG)
      ELSE
        LXC    = 0
      ENDIF
      LVOFRHOC = MAX(LHT,LXC)
      TAG      ='MAX(LHT,LXC)'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE HVL_AB(EH,EPS,CR0,CRAB,V0,VAB,EIVPS,EIROP)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     == ELECTROSTATIC AND PSEUDOPOTENTIAL ENERGIES FOR INTERACTION   ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
C     Arguments
      COMPLEX*16 EH,EPS,
     &           CR0(*),CRAB(*),V0(NHG),VAB(NHG),
     &           EIVPS(NHG),EIROP(NHG)
C     Variables
      COMPLEX*16 VP,RABS,VCG
      INTEGER    IG1,IG
C     ==--------------------------------------------------------------==
      IF(GEQ0) THEN
        IG1=2
        V0(1)=DCMPLX(0.D0,0.D0)
        VAB(1)=SCG(1)*(EIROP(1)+CR0(NZH(1)))+EIVPS(1)
      ELSE
        IG1=1
      ENDIF
      EH=DCMPLX(0.0D0,0.0D0)
      EPS=DCMPLX(0.0D0,0.0D0)
      DO IG=IG1,NHG
        RABS=DCONJG(CRAB(NZH(IG)))
        V0(IG)=SCG(IG)*CRAB(NZH(IG))
        VCG=SCG(IG)*(EIROP(IG)+CR0(NZH(IG)))
        VP=EIVPS(IG)
        VAB(IG)=VCG+VP
C       Hartree term
        EH=EH+2.D0*VCG*RABS
C       Pseudopotential energy
        EPS=EPS+RABS*VP
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE INTERXC(SXCAB,R0,RAB,V0,VAB)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      REAL*8     SXCAB,R0(*),RAB(*)
      COMPLEX*16 V0(*),VAB(*)
      INTEGER    IR
      REAL*8     SMALL,CF,THIRD,R3
      PARAMETER  (SMALL=1.D-12,THIRD=0.33333333333333D0)
C     ==--------------------------------------------------------------==
      CF=4.D0*(3.D0/PI)**THIRD
      SXCAB=0.D0
      DO IR=1,NNR1
        IF(ABS(RAB(IR)).GT.SMALL.AND.R0(IR).GT.SMALL) THEN
          R3=R0(IR)**THIRD
          SXCAB=SXCAB-CF*RAB(IR)*R3
          VAB(IR)=VAB(IR)-CF*R3
          V0(IR)=V0(IR)-CF*THIRD*RAB(IR)/(R3*R3)
        ENDIF
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE VCOMP(VCAS,VAB)
C     ==--------------------------------------------------------------==
C     ==  COMPILES THE TOTAL POTENTIALS FOR THE CAS22 METHOD          ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ener.inc'
      REAL*8   VCAS(NNR1,3),VAB(NNR1,3)
      INTEGER  IR
      REAL*8   CA,SA,X,Y,Z
C     ==--------------------------------------------------------------==
      CA=DCOS(CASANG)
      SA=DSIN(CASANG)
      X=SA*SA
      Y=1.D0-X
      Z=-2.D0*CA*SA
!$OMP parallel do private(IR) shared(X,Y,Z)
      DO IR=1,NNR1
        VCAS(IR,1)=Y*VCAS(IR,1)+X*VAB(IR,3)+Z*VAB(IR,1)
        VCAS(IR,2)=Y*VCAS(IR,2)+2.D0*X*VAB(IR,3)
        VCAS(IR,3)=Y*VCAS(IR,3)
      ENDDO
!$OMP parallel do private(IR) shared(Z)
      DO IR=1,NNR1
        VAB(IR,1)=0.5D0*Z*VAB(IR,2)
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CTFOR(FION)
C     ==--------------------------------------------------------------==
C     ==  COMPILES THE TOTAL FORCES FOR THE CAS22 METHOD              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ener.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'fcas.inc'
      REAL*8   FION(3,NAX,*)
      INTEGER  IS,IA
      REAL*8   CA,SA,X,Y,Z
C     ==--------------------------------------------------------------==
      CA=DCOS(CASANG)
      SA=DSIN(CASANG)
      X=SA*SA
      Y=1.D0-X
      Z=-2.D0*CA*SA
      DO IS=1,NSP
        DO IA=1,NA(IS)
          FION(1,IA,IS)=Y*FION(1,IA,IS)+X*FION_A(1,IA,IS)
     &                                 +Z*FION_AB(1,IA,IS)
          FION(2,IA,IS)=Y*FION(2,IA,IS)+X*FION_A(2,IA,IS)
     &                                 +Z*FION_AB(2,IA,IS)
          FION(3,IA,IS)=Y*FION(3,IA,IS)+X*FION_A(3,IA,IS)
     &                                 +Z*FION_AB(3,IA,IS)
        ENDDO
      ENDDO
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE PCASFOR(FION,V,EIROP)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     == POTENTIAL ENERGY CONTRIBUTIONS TO THE FORCES ON THE IONS     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'sfac.inc'
C     Arguments
      REAL*8     FION(3,NAX,NSX)
      COMPLEX*16 V(NNR1),EIROP(NHG)
C     Variables
      DIMENSION  EIGRB(NHG,NAT)
      COMPLEX*16 RHETS,GX,GY,GZ,VCGS,TXX,TYY,TZZ,EI123,VTOT
      REAL*8     FNL(*),DFNL(*)
      REAL*8     OMTP
      INTEGER    IG1,IG,ISA,IS,IA
C     ==--------------------------------------------------------------==
      OMTP=2.D0*OMEGA*TPIBA
      IG1=1
      IF(GEQ0) IG1=2
#if defined (__VECTOR)
      ISA=0
      DO IS=1,NSP
        DO IA=1,NA(IS)
          ISA=ISA+1
          IF(BIGMEM) THEN
            DO IG=IG1,NHG
              EI123=EIGRB(IG,ISA)
              RHETS=DCONJG(V(NZH(IG)))
              GX=DCMPLX(0.D0,GK(1,IG))
              GY=DCMPLX(0.D0,GK(2,IG))
              GZ=DCMPLX(0.D0,GK(3,IG))
              VTOT=(SCG(IG)*DCONJG(EIROP(IG))+VPS(IS,IG))*RHETS
              TXX=VTOT*GX
              TYY=VTOT*GY
              TZZ=VTOT*GZ
              FION(1,IA,IS)=FION(1,IA,IS)+DREAL(EI123*TXX)*OMTP
              FION(2,IA,IS)=FION(2,IA,IS)+DREAL(EI123*TYY)*OMTP
              FION(3,IA,IS)=FION(3,IA,IS)+DREAL(EI123*TZZ)*OMTP
            ENDDO
          ELSE
            DO IG=IG1,NHG
              EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     *                    EI3(ISA,INYH(3,IG))
              RHETS=DCONJG(V(NZH(IG)))
              GX=DCMPLX(0.D0,GK(1,IG))
              GY=DCMPLX(0.D0,GK(2,IG))
              GZ=DCMPLX(0.D0,GK(3,IG))
              VTOT=(SCG(IG)*DCONJG(EIROP(IG))+VPS(IS,IG))*RHETS
              TXX=VTOT*GX
              TYY=VTOT*GY
              TZZ=VTOT*GZ
              FION(1,IA,IS)=FION(1,IA,IS)+DREAL(EI123*TXX)*OMTP
              FION(2,IA,IS)=FION(2,IA,IS)+DREAL(EI123*TYY)*OMTP
              FION(3,IA,IS)=FION(3,IA,IS)+DREAL(EI123*TZZ)*OMTP
            ENDDO
          ENDIF
        ENDDO
      ENDDO
#else
      DO IG=IG1,NHG
        RHETS=DCONJG(V(NZH(IG)))
        VCGS=SCG(IG)*DCONJG(EIROP(IG))
        GX=DCMPLX(0.D0,GK(1,IG))
        GY=DCMPLX(0.D0,GK(2,IG))
        GZ=DCMPLX(0.D0,GK(3,IG))
        ISA=0
        DO IS=1,NSP
          VTOT=(VCGS+VPS(IS,IG))*RHETS
          TXX=VTOT*GX
          TYY=VTOT*GY
          TZZ=VTOT*GZ
          DO IA=1,NA(IS)
            ISA=ISA+1
            IF(BIGMEM) THEN
              EI123=EIGRB(IG,ISA)
            ELSE
              EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     *                  EI3(ISA,INYH(3,IG))
            ENDIF
            FION(1,IA,IS)=FION(1,IA,IS)+DREAL(EI123*TXX)*OMTP
            FION(2,IA,IS)=FION(2,IA,IS)+DREAL(EI123*TYY)*OMTP
            FION(3,IA,IS)=FION(3,IA,IS)+DREAL(EI123*TZZ)*OMTP
          ENDDO
        ENDDO
      ENDDO
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE POTAB(EPOTAB,VAB,SCR,LSCR)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==        POTENTIAL CONTRIBUTION TO THE PENALTY FUNCTION        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'spin.inc'
C     Arguments
      INTEGER    LSCR
      COMPLEX*16 EPOTAB,VAB(NHG)
      REAL*8     SCR(LSCR)
C     Variables
      LOGICAL    DEBUG
      COMPLEX*16 EIVPS(*),EIROP(*)
      INTEGER    IG
      POINTER    (IP_EIVPS,EIVPS),(IP_EIROP,EIROP)
C     ==--------------------------------------------------------------==
      DEBUG=.FALSE.
      CALL SCRSET(SCR,LSCR,DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_EIVPS,2*NHG,'EIVPS',DEBUG)
      CALL SCRPTR(SCR,LSCR,IP_EIROP,2*NHG,'EIROP',DEBUG)
      CALL EICALC(EIVPS,EIROP)
C
      EPOTAB=DCMPLX(0.0D0,0.0D0)
      DO IG=1,NHG
        EPOTAB=EPOTAB+2.D0*APENAL*DCONJG(VAB(IG))*EIVPS(IG)
        VAB(IG)=0.5D0*APENAL*EIVPS(IG)
      ENDDO
      CALL SCRREL(SCR,LSCR,DEBUG)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
