C     ==================================================================
      SUBROUTINE VOFRHOH(TAU0,FION,RHOE,V,VTEMP,SCR,LSCR,TFOR)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE ONE-PARTICLE POTENTIAL V IN REAL SPACE                  ==
C     ==  THE TOTAL ENERGY ETOT                                       ==
C     ==  THE FORCES FION ACTING ON THE IONS                          ==
C     ==--------------------------------------------------------------==
C     ==  RHOE: in  electronic density in real space                  ==
C     ==  V:    out electronic density in G space                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'simul.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'efld.inc'
      INCLUDE 'elct.inc'
CEHR[
      INCLUDE 'td_input.inc'
CEHR]
C     Arguments
      INTEGER    LSCR
      REAL*8     TAU0(3,NAX,NSX),FION(3,NAX,NSX),RHOE(NNR1),SCR(LSCR)
      COMPLEX*16 V(MAXFFT),VTEMP(NHG)
      LOGICAL    TFOR
C     Variables
      DIMENSION  EIGRB(NHG,*)
      COMPLEX*16 EIVPS(NHG),EIROP(NHG)
#ifdef PARALLEL
      COMPLEX*16 QPHI((NR1H+1)*NR2H*NR3PL)
#else
      COMPLEX*16 QPHI((NR1H+1)*NR2H)
#endif
      POINTER    (IP_EIVPS,EIVPS),(IP_EIROP,EIROP),(IP_QPHI,QPHI)
      CHARACTER  TAG*30
      COMPLEX*16 EPS,VP,RP,RHOG,VZCP,EI123,GX,GY,GZ,
     *           TXX,TYY,TZZ,RHET,RHETS,RHOT
      REAL*8     FNL(*),DFNL(*),EXTF(*),
     &           EZE,OMTP,EHIP1,EHIP2
      INTEGER    ISUB,NNRS,IL_EIVPS,IL_EIROP,LVOFRHOH,
     &           IR,IG1,IG,IS,IA,ISA
C     ==--------------------------------------------------------------==
      CALL TISET('   VOFRHOH',ISUB)
C     SCR partition
      CALL GIVE_SCR_VOFRHOH(LVOFRHOH,TAG)
      CALL TEST_SCR('VOFRHOH',TAG,LSCR,LVOFRHOH)
      CALL GET_ADDR(IP_EIVPS,SCR(1))
      IL_EIVPS = 2*NHG
      CALL GET_ADDR(IP_EIROP,SCR(1+IL_EIVPS))
      IL_EIROP = 2*NHG
      IF(TCLUST) CALL GET_ADDR(IP_QPHI,SCR(1+IL_EIVPS+IL_EIROP))
C     ==--------------------------------------------------------------==
C     Arrays inside SCR.
      NNRS  = NR1S*NR2S*NR3S
      CALL EICALC(EIVPS,EIROP)
C..External Field or forces of external potential
      EEXT=0.D0
CEHR[
      IF((TEXTFLD.OR.TEXADD).AND.(.NOT.TD_EXTPOT)) THEN
CEHR]
        CALL EEXTERN(RHOE,V,EIROP,TAU0,FION,EEXT,TFOR)
#ifdef PARALLEL
        CALL GLOSUM(1,EEXT)
#endif
      ENDIF
C     TRANSFORM THE DENSITY TO G SPACE
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        V(IR)=DCMPLX(RHOE(IR),0.D0)
      ENDDO
      CALL FWFFT(V)
C     ==--------------------------------------------------------------==
C     ==                       ISOLATED SYSTEMS                       ==
C     ==          CALCULATE THE HARTREE POTENTIAL AND ENERGY          ==
C     ==--------------------------------------------------------------==
C     ZERO CORRECTION AND PSEUDO-POTENTIAL
      IF(GEQ0) THEN
        VP=EIVPS(1)
        RP=EIROP(1)
        VPLOC=DREAL(VP)
        RHOG=V(NZH(1))
        RHOT=RHOG+RP
        VTEMP(1)=HIPZ(1)*RHOT+VP
        EZE=0.5D0*HIPZ(1)*DREAL(RHOT)*DREAL(RHOT)
        EPS=0.5D0*DREAL(VP*RHOG)
        IG1=2
      ELSE
        EZE=0.0D0
        EPS=(0.0D0,0.0D0)
        IG1=1
      ENDIF
!$OMP parallel do private(IG,VP,RP,RHOG,RHOT,VZCP)
!$OMP+ reduction(+:EZE,EPS)
      DO IG=IG1,NHG
        VP=EIVPS(IG)
        RP=EIROP(IG)
        RHOG=V(NZH(IG))
        RHOT=RHOG+RP
        VZCP=HIPZ(IG)*RHOT
        VTEMP(IG)=VP+VZCP
        EZE=EZE+(DREAL(RHOT)*DREAL(VZCP)+DIMAG(RHOT)*DIMAG(VZCP))
        EPS=EPS+(DREAL(VP)*DREAL(RHOG)+DIMAG(VP)*DIMAG(RHOG))
      ENDDO
      EPSEU=2.D0*DREAL(EPS)*OMEGA
C     ==------------------------------------------------------------==
C     FORCES ON ATOMS DUE TO ZERO CORRECTION AND PSEUDO-POTENTIAL
      IF(TFOR) THEN
        OMTP=2.D0*OMEGA*TPIBA
        IF(GEQ0) THEN
          IG1=2
        ELSE
          IG1=1
        ENDIF
#if defined (__VECTOR)
        ISA=0
        DO IS=1,NSP
          DO IA=1,NA(IS)
            ISA=ISA+1
            IF(BIGMEM) THEN
              DO IG=IG1,NHG
                EI123=EIGRB(IG,ISA)
                RP=EIROP(IG)
                RHET=V(NZH(IG))
                RHOG=RHET+RP
                RHETS=DCONJG(RHET)
                GX=DCMPLX(0.D0,GK(1,IG))
                GY=DCMPLX(0.D0,GK(2,IG))
                GZ=DCMPLX(0.D0,GK(3,IG))
                VZCP=DCONJG(HIPZ(IG)*RHOG)
                TXX=(RHOPS(IS,IG)*VZCP+VPS(IS,IG)*RHETS)*GX
                TYY=(RHOPS(IS,IG)*VZCP+VPS(IS,IG)*RHETS)*GY
                TZZ=(RHOPS(IS,IG)*VZCP+VPS(IS,IG)*RHETS)*GZ
                FION(1,IA,IS)=FION(1,IA,IS)+DREAL(EI123*TXX)*OMTP
                FION(2,IA,IS)=FION(2,IA,IS)+DREAL(EI123*TYY)*OMTP
                FION(3,IA,IS)=FION(3,IA,IS)+DREAL(EI123*TZZ)*OMTP
              ENDDO
            ELSE
              DO IG=IG1,NHG
                EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     *                      EI3(ISA,INYH(3,IG))
                RP=EIROP(IG)
                RHET=V(NZH(IG))
                RHOG=RHET+RP
                RHETS=DCONJG(RHET)
                GX=DCMPLX(0.D0,GK(1,IG))
                GY=DCMPLX(0.D0,GK(2,IG))
                GZ=DCMPLX(0.D0,GK(3,IG))
                VZCP=DCONJG(HIPZ(IG)*RHOG)
                TXX=(RHOPS(IS,IG)*VZCP+VPS(IS,IG)*RHETS)*GX
                TYY=(RHOPS(IS,IG)*VZCP+VPS(IS,IG)*RHETS)*GY
                TZZ=(RHOPS(IS,IG)*VZCP+VPS(IS,IG)*RHETS)*GZ
                FION(1,IA,IS)=FION(1,IA,IS)+DREAL(EI123*TXX)*OMTP
                FION(2,IA,IS)=FION(2,IA,IS)+DREAL(EI123*TYY)*OMTP
                FION(3,IA,IS)=FION(3,IA,IS)+DREAL(EI123*TZZ)*OMTP
              ENDDO
            ENDIF
          ENDDO
        ENDDO
#else
        DO IG=IG1,NHG
          RP=EIROP(IG)
          RHET=V(NZH(IG))
          RHOG=RHET+RP
          RHETS=DCONJG(RHET)
          GX=DCMPLX(0.D0,GK(1,IG))
          GY=DCMPLX(0.D0,GK(2,IG))
          GZ=DCMPLX(0.D0,GK(3,IG))
          VZCP=DCONJG(HIPZ(IG)*RHOG)
          ISA=0
          DO IS=1,NSP
            TXX=(RHOPS(IS,IG)*VZCP+VPS(IS,IG)*RHETS)*GX
            TYY=(RHOPS(IS,IG)*VZCP+VPS(IS,IG)*RHETS)*GY
            TZZ=(RHOPS(IS,IG)*VZCP+VPS(IS,IG)*RHETS)*GZ
            DO IA=1,NA(IS)
              ISA=ISA+1
              IF(BIGMEM) THEN
                EI123=EIGRB(IG,ISA)
              ELSE
                EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     *                      EI3(ISA,INYH(3,IG))
              ENDIF
              FION(1,IA,IS)=FION(1,IA,IS)+DREAL(EI123*TXX)*OMTP
              FION(2,IA,IS)=FION(2,IA,IS)+DREAL(EI123*TYY)*OMTP
              FION(3,IA,IS)=FION(3,IA,IS)+DREAL(EI123*TZZ)*OMTP
            ENDDO
          ENDDO
        ENDDO
#endif
      ENDIF
C     ==------------------------------------------------------------==
C     HARTREE POTENTIAL
      IG1=1
      IF(GEQ0) IG1=2
!CDIR NODEP
!$OMP parallel do private(IG)
      DO IG=IG1,NHG
        V(NZH(IG))=V(NZH(IG))+EIROP(IG)
        V(INDZ(IG))=V(INDZ(IG))+DCONJG(EIROP(IG))
      ENDDO
      IF(GEQ0) V(NZH(1))=V(NZH(1))+EIROP(1)
      CALL INVFFT(V)
      CALL HIP(V,QPHI)
      EHIP1=0.0D0
#if defined(__VECTOR)
!$OMP parallel do private(IR) reduction(+:EHIP1)
#else
!$OMP parallel do private(IR) reduction(+:EHIP1) schedule(static)
#endif
      DO IR=1,NNR1
        EHIP1=EHIP1+DREAL(V(IR))*RHOE(IR)
      ENDDO
      CALL FWFFT(V)
      EHIP2=0.0D0
!$OMP parallel do private(IG,VZCP) reduction(+:EHIP2)
      DO IG=1,NHG
        VZCP=V(NZH(IG))
        EHIP2=EHIP2+(DREAL(VZCP)*DREAL(EIROP(IG))+
     *                     DIMAG(VZCP)*DIMAG(EIROP(IG)))
        VTEMP(IG)=VTEMP(IG)+VZCP
      ENDDO
      IF(GEQ0) EHIP2=EHIP2-0.5D0*DREAL(V(NZH(1)))*DREAL(EIROP(1))
      EHEP = 0.5D0*EHIP1*OMEGA/DBLE(NNRS)
     &       + EHIP2*OMEGA
     &       + EZE*OMEGA
      EHEE = 0.D0
      EHII = 0.D0
      EHT = 0.5D0*EHIP1*OMEGA/DBLE(NNRS)
     &       + EHIP2*OMEGA
     &       + EZE*OMEGA
     &       + ESR
     &       - ESELF/DBLE(NPROC)
C     FORCES ON ATOMS DUE TO HARTREE TERM
      IF(TFOR) THEN
        OMTP=2.D0*OMEGA*TPIBA
        IG1=1
        IF(GEQ0) IG1=2
#if defined (__VECTOR)
        ISA=0
        DO IS=1,NSP
          DO IA=1,NA(IS)
            ISA=ISA+1
            IF(BIGMEM) THEN
              DO IG=IG1,NHG
                EI123=EIGRB(IG,ISA)
                GX=DCMPLX(0.D0,GK(1,IG))
                GY=DCMPLX(0.D0,GK(2,IG))
                GZ=DCMPLX(0.D0,GK(3,IG))
                VZCP=DCONJG(V(NZH(IG)))
                TXX=RHOPS(IS,IG)*VZCP*GX
                TYY=RHOPS(IS,IG)*VZCP*GY
                TZZ=RHOPS(IS,IG)*VZCP*GZ
                FION(1,IA,IS)=FION(1,IA,IS)+DREAL(EI123*TXX)*OMTP
                FION(2,IA,IS)=FION(2,IA,IS)+DREAL(EI123*TYY)*OMTP
                FION(3,IA,IS)=FION(3,IA,IS)+DREAL(EI123*TZZ)*OMTP
              ENDDO
            ELSE
              DO IG=IG1,NHG
                GX=DCMPLX(0.D0,GK(1,IG))
                GY=DCMPLX(0.D0,GK(2,IG))
                GZ=DCMPLX(0.D0,GK(3,IG))
                EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     *                    EI3(ISA,INYH(3,IG))
                GX=DCMPLX(0.D0,GK(1,IG))
                GY=DCMPLX(0.D0,GK(2,IG))
                GZ=DCMPLX(0.D0,GK(3,IG))
                VZCP=DCONJG(V(NZH(IG)))
                TXX=RHOPS(IS,IG)*VZCP*GX
                TYY=RHOPS(IS,IG)*VZCP*GY
                TZZ=RHOPS(IS,IG)*VZCP*GZ
                FION(1,IA,IS)=FION(1,IA,IS)+DREAL(EI123*TXX)*OMTP
                FION(2,IA,IS)=FION(2,IA,IS)+DREAL(EI123*TYY)*OMTP
                FION(3,IA,IS)=FION(3,IA,IS)+DREAL(EI123*TZZ)*OMTP
              ENDDO
            ENDIF
          ENDDO
        ENDDO
#else
        DO IG=IG1,NHG
          GX=DCMPLX(0.D0,GK(1,IG))
          GY=DCMPLX(0.D0,GK(2,IG))
          GZ=DCMPLX(0.D0,GK(3,IG))
          VZCP=DCONJG(V(NZH(IG)))
          ISA=0
          DO IS=1,NSP
            TXX=RHOPS(IS,IG)*VZCP*GX
            TYY=RHOPS(IS,IG)*VZCP*GY
            TZZ=RHOPS(IS,IG)*VZCP*GZ
            DO IA=1,NA(IS)
              ISA=ISA+1
              IF(BIGMEM) THEN
                EI123=EIGRB(IG,ISA)
              ELSE
                EI123=EI1(ISA,INYH(1,IG))*EI2(ISA,INYH(2,IG))*
     *                    EI3(ISA,INYH(3,IG))
              ENDIF
              FION(1,IA,IS)=FION(1,IA,IS)+DREAL(EI123*TXX)*OMTP
              FION(2,IA,IS)=FION(2,IA,IS)+DREAL(EI123*TYY)*OMTP
              FION(3,IA,IS)=FION(3,IA,IS)+DREAL(EI123*TZZ)*OMTP
            ENDDO
          ENDDO
        ENDDO
#endif
      ENDIF
C..calculate energy for diagonalization schemes
      IF(TDIAG) THEN
C     TRANSFORM THE DENSITY TO G SPACE
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          V(IR)=DCMPLX(RHOE(IR),0.D0)
        ENDDO
        CALL FWFFT(V)
        IF(GEQ0) THEN
          RP=EIROP(1)
          RHOG=V(NZH(1))
          EZE=-0.5D0*HIPZ(1)*DREAL(RHOG)*DREAL(RHOG)
          EZE=EZE+0.5D0*HIPZ(1)*DREAL(RP)*DREAL(RP)
          IG1=2
        ELSE
          EZE=0.0D0
          EPS=(0.0D0,0.0D0)
          IG1=1
        ENDIF
!$OMP parallel do private(IG,RHOG,VZCP,RP) reduction(+:EZE)
        DO IG=IG1,NHG
          RHOG=V(NZH(IG))
          VZCP=HIPZ(IG)*RHOG
          EZE=EZE+(-DREAL(RHOG)*DREAL(VZCP)-DIMAG(RHOG)*DIMAG(VZCP))
          RP=EIROP(IG)
          VZCP=HIPZ(IG)*RP
          EZE=EZE+(DREAL(RP)*DREAL(VZCP)+DIMAG(RP)*DIMAG(VZCP))
        ENDDO
        EZE=EZE*OMEGA
        EHIP1 = 0.5D0*EHIP1*OMEGA/DBLE(NNRS)
        EHIP2 = EHIP2*OMEGA
        EHEP = EHIP2-EHIP1 + EZE
        EHEE = 0.D0
        EHII = 0.D0
        EHT =  EHIP2-EHIP1
     &         + EZE
     &         + ESR
     &         - ESELF/DBLE(NPROC)
      ENDIF
      CALL TIHALT('   VOFRHOH',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_VOFRHOH(LVOFRHOH,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NHG
C     Arguments
      INTEGER   LVOFRHOH
      CHARACTER TAG*30
C     Variables
      INTEGER   IL_QPHI
C     ==--------------------------------------------------------------==
      CALL GIVE_QPHI(IL_QPHI)
C     EIVPS(2*NHG) EIROP(2*NHG)
      LVOFRHOH=4*NHG+IL_QPHI
#ifdef PARALLEL
      TAG      ='2*NHG*2+2*(NR1H+1)*NR2H*NR3PL'
#else 
      TAG      ='2*NHG*2+2*(NR1H+1)*NR2H'
#endif
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
