C     ==================================================================
      SUBROUTINE VOFRHOS(FION,RHOE,V,VTEMP,SCR,LSCR,TFOR,TSTRESS)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE ONE-PARTICLE POTENTIAL V IN REAL SPACE                  ==
C     ==  THE TOTAL ENERGY ETOT                                       ==
C     ==  THE FORCES FION ACTING ON THE IONS                          ==
C     ==  SUBROUTINE FOR LOW SPIN EXCITATION                          ==
C     ==--------------------------------------------------------------==
C     == RHOE:  in electronic density in real space                   ==
C     ==       out total electronic potential in real space           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'simul.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'nlcc.inc'
      INCLUDE 'strs.inc'
      INCLUDE 'str2.inc'
      INCLUDE 'efld.inc'
      INCLUDE 'extpot.inc'
C     Arguments
      INTEGER    LSCR
      REAL*8     FION(3,NAX,NSX),
     &           RHOE(NNR1,NLSD),SCR(LSCR)
      COMPLEX*16 V(MAXFFT,NLSD),VTEMP(NHG,NLSD)
      LOGICAL    TFOR,TSTRESS
C     Variables
      DIMENSION  F(N),DQG(NNR1,*)
      COMPLEX*16 VTMP(NHG,NLSX),EPOTAB
      REAL*8     RHOVAL(NNR1,NLSD),EXTPOT(*),EXTF(*),GRAD(NNR1,8)
      POINTER    (IP_VTMP,VTMP),(IP_GRAD,GRAD),(IP_RHOVAL,RHOVAL)
      REAL*8     FNL(IMAGP,LDF1,NHXS,N),DFNL(*),VMA,VMB,VTA,VTB,
     &           SXC,SGCX,SGCC,VGC,OD
      REAL*8     SXC1,SXC2,VXC1,VXC2,VGC1,VGC2,VCC,
     &           SGCX1,SGCX2,SGCC1,SGCC2
      INTEGER    ISUB,IG,IR,NNRS,IL_VTMP,IL_GRAD,IL_RHOVAL,LENGTH
C     ==--------------------------------------------------------------==
      CALL TISET('   VOFRHOS',ISUB)
      IF(TSTRESS) THEN
        CALL STOPGM('VOFRHOS','STRESS NOT IMPLEMENTED')
      ENDIF
      IF(TTAU) THEN
        CALL STOPGM('VOFRHOS','META FUNCTIONALS NOT IMPLENTED')
      ENDIF
C     SCR partition.
#ifdef _BGL 
      IL_VTMP = MAX(NNR1,NHG*2)*NLSX +MOD(MAX(NNR1,NHG*2)*NLSX,2)
#else
      IL_VTMP = MAX(NNR1,NHG*2)*NLSX      !GCENER and COREC
#endif
      IF(TGC) THEN
        IL_GRAD=NNR1*8     !GRADEN
      ELSE
        IL_GRAD=0
      ENDIF
      IF(TINLC) THEN
        IL_RHOVAL=NNR1*NLSD     !XCENER
      ELSE
        IL_RHOVAL=0
      ENDIF
      LENGTH = IL_VTMP+MAX(IL_GRAD,IL_RHOVAL)
      IF(TIVAN) LENGTH=MAX(LENGTH,NHG*4+NAX) !NEWD
      CALL TEST_SCR('VOFRHOS','LENGTH',LSCR,LENGTH)
      CALL GET_ADDR(IP_VTMP,SCR(1))
      IF(TGC) CALL GET_ADDR(IP_GRAD,SCR(1+IL_VTMP))
      IF(TINLC) THEN
        CALL GET_ADDR(IP_RHOVAL,SCR(1+IL_VTMP))
      ELSE
        CALL GET_ADDR(IP_RHOVAL,RHOE(1,1))
      ENDIF
C     ==--------------------------------------------------------------==
C     == ADD CORE CHARGE TO RHOE                                      ==
C     ==--------------------------------------------------------------==
      IF(TINLC) THEN
        CALL DCOPY(NNR1*NLSD,RHOE(1,1),1,RHOVAL(1,1),1)
        CALL COREC(RHOE,VTMP,V)
        CALL DCOPY(2*NHG,VTEMP(1,1),1,VNLT(1),1)
      ENDIF
C     ==--------------------------------------------------------------==
C     == VTEMP (Potential in G-Space) -FFT-> V(R)                     ==
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(V,MAXFFT*NLSD)
!CDIR NODEP
      DO IG=1,NHG
        V(INDZ(IG),1) = DCONJG(VTEMP(IG,1))
        V(NZH(IG),1)  = VTEMP(IG,1)
      ENDDO
      IF(GEQ0) THEN
        V(NZH(1),1) = VTEMP(1,1)
      ENDIF
      CALL INVFFT(V)
C     ==--------------------------------------------------------------==
C     == ADD EXTERNAL POTENTIAL TO V                                  ==
C     ==--------------------------------------------------------------==
      IF(TEXTFLD) THEN
C..potential from classical interface
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          V(IR,1)=V(IR,1)+DCMPLX(EXTF(IR),0.D0)
        ENDDO
      ENDIF
      IF(TEXPOT) THEN
C..static external potential
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          V(IR,1)=V(IR,1)+DCMPLX(EXTPOT(IR),0.D0)
        ENDDO
      ENDIF
C     SAVE THE GROUND STATE DENSITY
      IF(TCAS22) THEN
C       Warning DQG is complex (address (1,2))
        CALL DCOPY(NNR1,RHOE(1,3),1,DQG(1,2),1)
        CALL DAXPY(NNR1,-1.D0,RHOE(1,4),1,DQG(1,2),1)
        CALL DSCAL(NNR1,2.D0,DQG(1,2),1)
      ENDIF
C     ==--------------------------------------------------------------==
C     == MIXED STATE                                                  ==
C     ==--------------------------------------------------------------==
C     == COMPUTE EXCHANGE AND CORRELATION ENERGY (EXC)                ==
C     ==--------------------------------------------------------------==
      CALL DCOPY(2*NNR1,V(1,1),1,DQG(1,1),1)
      CALL ZAZZERO(V,NNR1)
      SXC1  = 0.0D0
      VXC1  = 0.0D0
      SGCX1 = 0.0D0
      SGCC1 = 0.0D0
      VGC1  = 0.0D0
      IF(ABS(LSEA) .GT. 1.D-6) THEN
        TLSD=.TRUE.
        CALL XCENER(SXC1,VXC1,RHOVAL,RHOE(1,1),V)
        TLSD=.FALSE.
        IF(TGC) THEN
C         CALCULATE THE GRADIENT OF THE DENSITY
          CALL FWFFT(V(1,1))
          CALL ZGTHR(NHG,V(1,1),VTEMP(1,1),NZH)
          CALL FWFFT(V(1,2))
          CALL ZGTHR(NHG,V(1,2),VTEMP(1,2),NZH)
          DO IR=1,NNR1
            RHOE(IR,1)=RHOE(IR,1)-RHOE(IR,2)
          ENDDO
          CALL GRADEN(RHOE(1,1),V,GRAD(1,1),VTMP)
          CALL GRADEN(RHOE(1,2),V,GRAD(1,5),VTMP)
C         GRADIENT CORRECTION TO THE EXCHANGE ENERGY (EGCX)
          CALL GCLSD(SGCX1,SGCC1,RHOE,V,VTEMP,VTMP,GRAD,.FALSE.)
        ENDIF
      ENDIF
C     V CONTAINS THE MIXED STATE VXC IN R-SPACE, MOVE IT TO RHOE
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        RHOE(IR,1)=DREAL(V(IR,1))
        RHOE(IR,2)=DREAL(V(IR,2))
      ENDDO
C     ==--------------------------------------------------------------==
C     == TRIPLET STATE                                                ==
C     ==--------------------------------------------------------------==
C     == COMPUTE EXCHANGE AND CORRELATION ENERGY (EXC)                ==
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(V,NNR1)
      SXC2  = 0.0D0
      VXC2  = 0.0D0
      SGCX2 = 0.0D0
      SGCC2 = 0.0D0
      VGC2  = 0.0D0
      IF(ABS(LSEB) .GT. 1.D-6) THEN
        TLSD=.TRUE.
        CALL XCENER(SXC2,VXC2,RHOVAL,RHOE(1,3),V)
        TLSD=.FALSE.
        IF(TGC) THEN
C         CALCULATE THE GRADIENT OF THE DENSITY
          CALL FWFFT(V(1,1))
          CALL ZGTHR(NHG,V(1,1),VTEMP(1,1),NZH)
          CALL FWFFT(V(1,2))
          CALL ZGTHR(NHG,V(1,2),VTEMP(1,2),NZH)
!$OMP parallel do private(IR)
          DO IR=1,NNR1
            RHOE(IR,3)=RHOE(IR,3)-RHOE(IR,4)
          ENDDO
          CALL GRADEN(RHOE(1,3),V,GRAD(1,1),VTMP)
          CALL GRADEN(RHOE(1,4),V,GRAD(1,5),VTMP)
C         GRADIENT CORRECTION TO THE EXCHANGE ENERGY (EGCX)
          CALL GCLSD(SGCX2,SGCC2,RHOE(1,3),V,VTEMP,VTMP,GRAD,.FALSE.)
        ENDIF
      ENDIF
C     V CONTAINS THE TRIPLET STATE VXC IN R-SPACE, MOVE IT TO RHOE
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        RHOE(IR,3)=DREAL(V(IR,1))
        RHOE(IR,4)=DREAL(V(IR,2))
      ENDDO
      IF(TROOT) THEN
!$OMP parallel do private(IR,VMA,VMB,VTA,VTB,VCC) shared(LSEA,LSEB)
        DO IR=1,NNR1
          VMA=RHOE(IR,1)
          VMB=RHOE(IR,2)
          VTA=RHOE(IR,3)
          VTB=RHOE(IR,4)
          VCC=DREAL(DQG(IR,1))
          RHOE(IR,1)=VCC+0.5D0*(LSEA*VMA+LSEA*VMB+LSEB*VTA+LSEB*VTB)
          RHOE(IR,2)=0.5D0*(-LSEA*VMA+LSEA*VMB+LSEB*VTA-LSEB*VTB)
          RHOE(IR,3)=0.5D0*(LSEA*VMA-LSEA*VMB+LSEB*VTA-LSEB*VTB)
        ENDDO
      ELSE IF(TLSETS) THEN
!$OMP parallel do private(IR,VMA,VMB,VTA,VTB,VCC) shared(LSEA,LSEB)
        DO IR=1,NNR1
          VMA=RHOE(IR,1)
          VMB=RHOE(IR,2)
          VTA=RHOE(IR,3)
          VTB=RHOE(IR,4)
          VCC=DREAL(DQG(IR,1))*(LSEA+LSEB)
          RHOE(IR,1)=VCC+0.5D0*(LSEA*(VMA+VMB)+LSEB*(VTA+VTB))
          RHOE(IR,2)=VCC+
     $         (LSEA*(VMA+0.5D0*VMB)+LSEB*(0.5D0*VTA+VTB))/1.5D0
          RHOE(IR,3)=VCC+LSEA*VMB+LSEB*VTB
        ENDDO
      ELSE 
!$OMP parallel do private(IR,VMA,VMB,VTA,VTB,VCC) shared(LSEA,LSEB)
        DO IR=1,NNR1
          VMA=RHOE(IR,1)
          VMB=RHOE(IR,2)
          VTA=RHOE(IR,3)
          VTB=RHOE(IR,4)
          VCC=DREAL(DQG(IR,1))*(LSEA+LSEB)
          RHOE(IR,1)=VCC+0.5D0*(LSEA*(VMA+VMB)+LSEB*(VTA+VTB))
          RHOE(IR,2)=VCC+LSEA*VMA+LSEB*VTB
          RHOE(IR,3)=VCC+LSEA*VMB+LSEB*VTB
        ENDDO
      ENDIF
      IF(TCAS22) THEN
        CALL DCOPY(NNR1,DQG(1,2),1,RHOE(1,4),1)
      ENDIF
C     ==--------------------------------------------------------------==
C     == ADD THE PIECES                                               ==
C     ==--------------------------------------------------------------==
      SXC=LSEA*SXC1+LSEB*SXC2
      VXC=LSEA*VXC1+LSEB*VXC2
      VGC=LSEA*VGC1+LSEB*VGC2
      SGCX=LSEA*SGCX1+LSEB*SGCX2
      SGCC=LSEA*SGCC1+LSEB*SGCC2
C     ==--------------------------------------------------------------==
C     == CORRECT ENERGY FOR CORE CHARGE                               ==
C     ==--------------------------------------------------------------==
      IF(TINLC) THEN
        SXC=SXC-EXCCO
        SGCX=SGCX-EGCXCO
        SGCC=SGCC-EGCCCO
      ENDIF
      IF(TIVAN.OR.(TINLC.AND.TFOR)) THEN
C     ==--------------------------------------------------------------==
C     == TRANSF. TOTAL POTENTIAL IN G-SPACE                           ==
C     == PUT THE TOTAL POTENTIAL IN G-SPACE INTO VTEMP                ==
C     ==--------------------------------------------------------------==
        CALL FWFFT(V)
        CALL ZGTHR(NHG,V,VTEMP,NZH)
      ENDIF
C     ==--------------------------------------------------------------==
C     == FORCE ON IONS DUE TO NLCC                                    ==
C     ==--------------------------------------------------------------==
      IF(TINLC.AND.TFOR) CALL COFOR(FION,VTEMP)
C     ==--------------------------------------------------------------==
      IF(TPENAL) THEN
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          V(IR,1)=DCMPLX(RHOE(IR,5),0.D0)
        ENDDO
        CALL FWFFT(V)
        CALL ZGTHR(NHG,V,VTEMP,NZH)
        IF(TFOR) CALL POTABFOR(FION,VTEMP)
        CALL POTAB(EPOTAB,VTEMP,SCR,LSCR)
        EHT_AB=DREAL(EPOTAB)*OMEGA
        CALL ZAZZERO(V,MAXFFT)
        DO IG=1,NHG
          V(INDZ(IG),1) = DCONJG(VTEMP(IG,1))
          V(NZH(IG),1)  = VTEMP(IG,1)
        ENDDO
        IF(GEQ0) THEN
          V(NZH(1),1) = VTEMP(1,1)
        ENDIF
        CALL INVFFT(V)
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          RHOE(IR,5)=DREAL(V(IR,1))
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     == SINGLE PIECES OF THE ENERGY:                                 ==
C     ==--------------------------------------------------------------==
      NNRS = NR1S*NR2S*NR3S
      OD = OMEGA/DBLE(NNRS)
      EGC=(SGCC+SGCX)*OD
      EXC=SXC*OD + EGC
      VXC=(VXC + VGC)*OD
C     Triplet energy
      IF(TINLC) THEN
        ETOT_T = (SXC2-EXCCO+SGCX2-EGCXCO+SGCC2-EGCCCO)*OD
      ELSE
        ETOT_T = (SXC2+SGCX2+SGCC2)*OD
      ENDIF
C
      CALL TIHALT('   VOFRHOS',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_VOFRHOS(LVOFRHOS,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NHG TGC
      INCLUDE 'spin.inc'        !NLSD NLSX
      INCLUDE 'pslo.inc'        !TIVAN
      INCLUDE 'nlcc.inc'        !TINLC
C     Arguments
      INTEGER   LVOFRHOS
      CHARACTER TAG*30
C     Variables
      INTEGER LTGC,LTINLC
C     ==--------------------------------------------------------------==
      IF(TGC) THEN
        LTGC=NNR1*NLSD*4        !GRADEN
      ELSE
        LTGC=0
      ENDIF
      IF(TINLC) THEN            !XCENER
        LTINLC=NNR1*NLSD
      ELSE
        LTINLC=0
      ENDIF
      LVOFRHOS = MAX(NNR1,NHG*2)*NLSX+MAX(LTGC,LTINLC) !GCENER and COREC
      IF(TIVAN) LVOFRHOS=MAX(LVOFRHOS,NHG*4+NAX) !NEWD
      TAG='MAX(NNR1,NHG*2)*NLSX+.........'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE VOFRHOR(FION,RHOE,V,VTEMP,SCR,LSCR,TFOR,TSTRESS)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE ONE-PARTICLE POTENTIAL V IN REAL SPACE                  ==
C     ==  THE TOTAL ENERGY ETOT                                       ==
C     ==  THE FORCES FION ACTING ON THE IONS                          ==
C     ==  SUBROUTINE FOR LOW SPIN EXCITATION (ROSS TYPE)              ==
C     ==--------------------------------------------------------------==
C     == RHOE:  in electronic density in real space                   ==
C     ==       out total electronic potential in real space           ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'elct.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'pslo.inc'
      INCLUDE 'simul.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cvan.inc'
      INCLUDE 'nlps.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'nlcc.inc'
      INCLUDE 'strs.inc'
      INCLUDE 'str2.inc'
      INCLUDE 'efld.inc'
      INCLUDE 'func.inc'
      INCLUDE 'isos.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'extpot.inc'
C     Arguments
      INTEGER    LSCR
      REAL*8     FION(3,NAX,NSX),
     &           RHOE(NNR1,NLSD),SCR(LSCR)
      COMPLEX*16 V(MAXFFT,NLSD),VTEMP(NHG,NLSD)
      LOGICAL    TFOR,TSTRESS
C     Variables
      DIMENSION  F(N),DQG(NNR1*4)
      COMPLEX*16 VTMP(NHG,NLSX),SCGY,EPOTAB
      REAL*8     RHOVAL(NNR1,NLSD),EXTPOT(*),EXTF(*),GRAD(NNR1,8)
      POINTER    (IP_VTMP,VTMP),(IP_GRAD,GRAD),(IP_RHOVAL,RHOVAL)
      REAL*8     FNL(IMAGP,LDF1,NHXS,N),DFNL(*)
      REAL*8     SXC,SGCX,SGCC,VGC,EK,G2
      REAL*8     SXC1,SXC2,VXC1,VXC2,VGC1,VGC2,
     &           SGCX1,SGCX2,SGCC1,SGCC2,VP1,VP2,VP3
      INTEGER    ISUB,IG,IR,NNRS,IG1,IL_VTMP,IL_GRAD,IL_RHOVAL,LENGTH
      INTEGER    MFXCX_1,MFXCC_1,MGCX_1,MGCC_1
C     ==--------------------------------------------------------------==
      CALL TISET('   VOFRHOR',ISUB)
      IF(TSTRESS) THEN
        CALL STOPGM('VOFRHOR','STRESS NOT IMPLEMENTED')
      ENDIF
      IF(TTAU) THEN
        CALL STOPGM('VOFRHOR','META FUNCTIONALS NOT IMPLENTED')
      ENDIF
      IF(TIVAN) THEN
        CALL STOPGM('VOFRHOR','VDB PP NOT IMPLEMENTED')
      ENDIF
C     SCR partition.
#ifdef _BGL 
      IL_VTMP = MAX(NNR1,NHG*2)*NLSX +MOD(MAX(NNR1,NHG*2)*NLSX,2)
#else
      IL_VTMP = MAX(NNR1,NHG*2)*NLSX      !GCENER and COREC
#endif
      IF(TGC) THEN
        IL_GRAD=NNR1*8     !GRADEN
      ELSE
        IL_GRAD=0
      ENDIF
      IF(TINLC) THEN
        IL_RHOVAL=NNR1*NLSD     !XCENER
      ELSE
        IL_RHOVAL=0
      ENDIF
      LENGTH = IL_VTMP+MAX(IL_GRAD,IL_RHOVAL)
      CALL TEST_SCR('VOFRHOR','LENGTH',LSCR,LENGTH)
      CALL GET_ADDR(IP_VTMP,SCR(1))
      IF(TGC) THEN
        CALL GET_ADDR(IP_GRAD,SCR(1+IL_VTMP))
      ENDIF
      IF(TINLC) THEN
        CALL GET_ADDR(IP_RHOVAL,SCR(1+IL_VTMP))
      ELSE
        CALL GET_ADDR(IP_RHOVAL,RHOE(1,1))
      ENDIF
C     ==--------------------------------------------------------------==
      MFXCX_1 = MFXCX
      MFXCC_1 = MFXCC
      MGCX_1  = MGCX
      MGCC_1  = MGCC
      IF(MFXCC.EQ.9) THEN
        CALL STOPGM("ROSS METHOD"," PADE FUNCTIONAL NOT POSSIBLE")
      ENDIF
C     ==--------------------------------------------------------------==
C     == ADD CORE CHARGE TO RHOE                                      ==
C     ==--------------------------------------------------------------==
      IF(TINLC) THEN
        CALL DCOPY(NNR1*NLSD,RHOE(1,1),1,RHOVAL(1,1),1)
        CALL COREC(RHOE,VTMP,V)
        CALL DCOPY(2*NHG,VTEMP(1,1),1,VNLT(1),1)
      ENDIF
C     ==--------------------------------------------------------------==
C     == VTEMP (Potential in G-Space) -FFT-> V(R)                     ==
C     ==--------------------------------------------------------------==
      CALL ZAZZERO(V,MAXFFT*NLSD)
      DO IG=1,NHG
        V(INDZ(IG),1) = DCONJG(VTEMP(IG,1))
        V(NZH(IG),1)  = VTEMP(IG,1)
      ENDDO
      IF(GEQ0) THEN
        V(NZH(1),1) = VTEMP(1,1)
      ENDIF
      CALL INVFFT(V)
C     ==--------------------------------------------------------------==
C     == ADD EXTERNAL POTENTIAL TO V                                  ==
C     ==--------------------------------------------------------------==
      IF(TEXTFLD) THEN
C..potential from classical interface
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          V(IR,1)=V(IR,1)+DCMPLX(EXTF(IR),0.D0)
        ENDDO
      ENDIF
      IF(TEXPOT) THEN
C..static external potential
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          V(IR,1)=V(IR,1)+DCMPLX(EXTPOT(IR),0.D0)
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     == MIXED STATE                                                  ==
C     ==--------------------------------------------------------------==
      MFXCX = 0
      MFXCC = MFXCC_1
      MGCX  = 0
      MGCC  = MGCC_1
      TGCX=.FALSE.
      IF(MGCC.NE.0) THEN
        TGCC=.TRUE.
      ELSE
        TGCC=.FALSE.
      ENDIF
      TGC=TGCC.OR.TGCX
C     ==--------------------------------------------------------------==
C     == COMPUTE CORRELATION ENERGY (EC)                              ==
C     ==--------------------------------------------------------------==
      CALL DCOPY(2*NNR1,V(1,1),1,DQG(1),1)
      TLSD=.TRUE.
      CALL XCENER(SXC1,VXC1,RHOVAL,RHOE(1,1),V)
      TLSD=.FALSE.
      SGCX1 = 0.0D0
      SGCC1 = 0.0D0
      VGC1  = 0.0D0
      IF(TGC) THEN
C       CALCULATE THE GRADIENT OF THE DENSITY
        CALL FWFFT(V(1,1))
        CALL ZGTHR(NHG,V(1,1),VTEMP(1,1),NZH)
        CALL FWFFT(V(1,2))
        CALL ZGTHR(NHG,V(1,2),VTEMP(1,2),NZH)
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          RHOE(IR,1)=RHOE(IR,1)-RHOE(IR,2)
        ENDDO
        CALL GRADEN(RHOE(1,1),V,GRAD(1,1),VTMP)
        CALL GRADEN(RHOE(1,2),V,GRAD(1,5),VTMP)
C       GRADIENT CORRECTION TO THE EXCHANGE ENERGY (EGCX)
        CALL GCLSD(SGCX1,SGCC1,RHOE,V,VTEMP,VTMP,GRAD,.FALSE.)
      ENDIF
C     V CONTAINS THE TOTAL POTENTIAL IN R-SPACE, MOVE IT TO RHOE
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        RHOE(IR,1)=DREAL(V(IR,1))
        RHOE(IR,2)=DREAL(V(IR,2))
      ENDDO
C     ==--------------------------------------------------------------==
C     == TRIPLET STATE                                                ==
C     ==--------------------------------------------------------------==
      MFXCX = MFXCX_1
      MFXCC = 0
      MGCX  = MGCX_1
      MGCC  = 0
      TGCC=.FALSE.
      IF(MGCX.NE.0) THEN
        TGCX=.TRUE.
      ELSE
        TGCX=.FALSE.
      ENDIF
      TGC=TGCC.OR.TGCX
C     ==--------------------------------------------------------------==
C     == COMPUTE EXCHANGE ENERGY (EX)                                 ==
C     ==--------------------------------------------------------------==
      CALL DCOPY(2*NNR1,DQG(1),1,V(1,1),1)
      TLSD=.TRUE.
      CALL XCENER(SXC2,VXC2,RHOVAL,RHOE(1,3),V)
      TLSD=.FALSE.
      SGCX2 = 0.0D0
      SGCC2 = 0.0D0
      VGC2  = 0.0D0
      IF(TGC) THEN
C       CALCULATE THE GRADIENT OF THE DENSITY
        CALL FWFFT(V(1,1))
        CALL ZGTHR(NHG,V(1,1),VTEMP(1,1),NZH)
        CALL FWFFT(V(1,2))
        CALL ZGTHR(NHG,V(1,2),VTEMP(1,2),NZH)
        DO IR=1,NNR1
          RHOE(IR,3)=RHOE(IR,3)-RHOE(IR,4)
        ENDDO
        CALL GRADEN(RHOE(1,3),V,GRAD(1,1),VTMP)
        CALL GRADEN(RHOE(1,4),V,GRAD(1,5),VTMP)
C       GRADIENT CORRECTION TO THE EXCHANGE ENERGY (EGCX)
        CALL GCLSD(SGCX2,SGCC2,RHOE(1,3),V,VTEMP,VTMP,GRAD,.FALSE.)
      ENDIF
C     V CONTAINS THE TOTAL POTENTIAL IN R-SPACE, MOVE IT TO RHOE
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        RHOE(IR,3)=DREAL(V(IR,1))
        RHOE(IR,4)=DREAL(V(IR,2))
      ENDDO
C     ==--------------------------------------------------------------==
      MFXCX = MFXCX_1
      MFXCC = MFXCC_1
      MGCX  = MGCX_1
      MGCC  = MGCC_1
      IF(MGCX.NE.0) THEN
        TGCX=.TRUE.
      ELSE
        TGCX=.FALSE.
      ENDIF
      IF(MGCC.NE.0) THEN
        TGCC=.TRUE.
      ELSE
        TGCC=.FALSE.
      ENDIF
      TGC=TGCC.OR.TGCX
C     ==--------------------------------------------------------------==
C     == ADD THE PIECES                                               ==
C     ==--------------------------------------------------------------==
      SXC=SXC1+SXC2
      VXC=VXC1+VXC2
      VGC=VGC1+VGC2
      SGCX=SGCX1+SGCX2
      SGCC=SGCC1+SGCC2
C     ==--------------------------------------------------------------==
C     == CORRECT ENERGY FOR CORE CHARGE                               ==
C     ==--------------------------------------------------------------==
      IF(TINLC) THEN
        SXC=SXC-EXCCO
        SGCX=SGCX-EGCXCO
        SGCC=SGCC-EGCCCO
      ENDIF
      IF(TINLC.AND.TFOR) THEN
C     ==--------------------------------------------------------------==
C     == FORCE ON IONS DUE TO NLCC                                    ==
C     ==--------------------------------------------------------------==
        CALL STOPGM("VOFRHOR","DOES THIS WORK?")
        CALL FWFFT(V)
        CALL ZGTHR(NHG,V,VTEMP,NZH)
        CALL COFOR(FION,VTEMP)
      ENDIF
C     ==--------------------------------------------------------------==
C     There are always two potentials that contribute to the total
C     potential for each state 
C     occ = 0.5 * ( V1 + V2 + V3 + V4 )
C     alpha = V1 + V4
C     beta  = V2 + V4
C     -> add 0.5 * local potential to the XC parts, this sums always to 
C        one total contribution (1-0.5)
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        RHOE(IR,1)=RHOE(IR,1)-0.5D0*DREAL(DQG(IR))
        RHOE(IR,2)=RHOE(IR,2)-0.5D0*DREAL(DQG(IR))
        RHOE(IR,3)=RHOE(IR,3)-0.5D0*DREAL(DQG(IR))
        RHOE(IR,4)=RHOE(IR,4)-0.5D0*DREAL(DQG(IR))
      ENDDO
C     Rearange the potentials
!$OMP parallel do private(IR,VP1,VP2,VP3)
      DO IR=1,NNR1
        VP1 = 0.5D0*(RHOE(IR,1)+RHOE(IR,2)+RHOE(IR,3)+RHOE(IR,4))
        VP2 = RHOE(IR,1)+RHOE(IR,4)
        VP3 = RHOE(IR,2)+RHOE(IR,4)
        RHOE(IR,1) = VP1
        RHOE(IR,2) = VP2
        RHOE(IR,3) = VP3
      ENDDO
C     ==--------------------------------------------------------------==
C     == EXCHANGE POTENTIAL FROM A-B STATES                           ==
C     ==--------------------------------------------------------------==
C     TRANSFORM THE DENSITY TO G SPACE
      CALL ZAZZERO(V,MAXFFT)
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        V(IR,1)=DCMPLX(RHOE(IR,5),0.D0)
      ENDDO
      CALL FWFFT(V)
      EK=0.D0
      IF(TCLUST) THEN
        IF(PS_TYPE.EQ.1) THEN
          CALL STOPGM('VOFRHOR','HOCKNEY PS not impl.')
        ELSE
          DO IG=1,NHG
            VTEMP(IG,1) = 2.D0*V(NZH(IG),1) * SCG(IG)
            EK = EK + 2.D0 * DREAL(DCONJG(VTEMP(IG,1))*V(NZH(IG),1))
          ENDDO
        ENDIF
      ELSE
        IG1=1
        IF(GEQ0) IG1=2
        DO IG=IG1,NHG
          G2=TPIBA2*HG(IG)
          SCGY=DCMPLX(FPI/G2,0.D0)
          VTEMP(IG,1) = 2.D0*V(NZH(IG),1) * SCGY
          EK = EK + 2.D0 * DREAL(DCONJG(VTEMP(IG,1))*V(NZH(IG),1))
        ENDDO
        IF(GEQ0) VTEMP(1,1)=0.D0
      ENDIF
      CALL ZAZZERO(V,MAXFFT)
      DO IG=1,NHG
        V(INDZ(IG),1) = DCONJG(VTEMP(IG,1))
        V(NZH(IG),1)  = VTEMP(IG,1)
      ENDDO
      IF(GEQ0) THEN
        V(NZH(1),1) = VTEMP(1,1)
      ENDIF
      CALL INVFFT(V)
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        RHOE(IR,4)=DREAL(V(IR,1))
      ENDDO
C     ==--------------------------------------------------------------==
      IF(TPENAL) THEN
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          V(IR,1)=DCMPLX(RHOE(IR,5),0.D0)
        ENDDO
        CALL FWFFT(V)
        CALL ZGTHR(NHG,V,VTEMP,NZH)
        IF(TFOR) THEN
          CALL POTABFOR(FION,VTEMP)
        ENDIF
        CALL POTAB(EPOTAB,VTEMP,SCR,LSCR)
        EHT_AB=DREAL(EPOTAB)*OMEGA
        CALL ZAZZERO(V,MAXFFT)
        DO IG=1,NHG
          V(INDZ(IG),1) = DCONJG(VTEMP(IG,1))
          V(NZH(IG),1)  = VTEMP(IG,1)
        ENDDO
        IF(GEQ0) THEN
          V(NZH(1),1) = VTEMP(1,1)
        ENDIF
        CALL INVFFT(V)
!$OMP parallel do private(IR)
        DO IR=1,NNR1
          RHOE(IR,5)=DREAL(V(IR,1))
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
C     == SINGLE PIECES OF THE ENERGY:                                 ==
C     ==--------------------------------------------------------------==
      NNRS = NR1S*NR2S*NR3S
      EGC=(SGCC+SGCX)*OMEGA/DBLE(NNRS)
      EXC=SXC*OMEGA/DBLE(NNRS) + EGC + EK*OMEGA
C
      CALL TIHALT('   VOFRHOR',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_VOFRHOR(LVOFRHOR,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NHG TGC
      INCLUDE 'spin.inc'        !NLSD NLSX
      INCLUDE 'pslo.inc'        !TIVAN
      INCLUDE 'nlcc.inc'        !TINLC
C     Arguments
      INTEGER   LVOFRHOR
      CHARACTER TAG*30
C     Variables
      INTEGER LTGC,LTINLC
C     ==--------------------------------------------------------------==
      IF(TGC) THEN
        LTGC=NNR1*NLSD*4        !GRADEN
      ELSE
        LTGC=0
      ENDIF
      IF(TINLC) THEN            !XCENER
        LTINLC=NNR1*NLSD
      ELSE
        LTINLC=0
      ENDIF
      LVOFRHOR = MAX(NNR1,NHG*2)*NLSX+MAX(LTGC,LTINLC) !GCENER and COREC
      IF(TIVAN) LVOFRHOR=MAX(LVOFRHOR,NHG*4+NAX) !NEWD
      TAG='MAX(NNR1,NHG*2)*NLSX+.........'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
