C     ==================================================================
      SUBROUTINE VOFRHOT(TAU0,FION,RHOE,V,VTEMP,SCR,LSCR,TFOR)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==  THE ONE-PARTICLE POTENTIAL V IN REAL SPACE                  ==
C     ==  THE TOTAL ENERGY ETOT                                       ==
C     ==  THE FORCES FION ACTING ON THE IONS                          ==
C     ==--------------------------------------------------------------==
C     ==  RHOE: in  electronic density in real space                  ==
C     ==  V:    out electronic density in G space                     ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'cnst.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'simul.inc'
      INCLUDE 'sfac.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'efld.inc'
      INCLUDE 'elct.inc'
CHER[
      INCLUDE 'td_input.inc'
CEHR]
C     Arguments
      INTEGER    LSCR
      REAL*8     TAU0(3,NAX,NSX),FION(3,NAX,NSX),RHOE(NNR1),SCR(LSCR)
      COMPLEX*16 V(MAXFFT),VTEMP(NHG)
      LOGICAL    TFOR
C     Variables
      COMPLEX*16 EIVPS(NHG),EIROP(NHG)
      POINTER    (IP_EIVPS,EIVPS),(IP_EIROP,EIROP)
      COMPLEX*16 EH,EI,EE,EPS
      REAL*8     FNL(*),DFNL(*),EXTF(*)
      INTEGER    ISUB,NNRS,IL_EIVPS,IL_EIROP,LENGTH,IR
C     ==--------------------------------------------------------------==
      CALL TISET('   VOFRHOT',ISUB)
C     SCR partition
      IL_EIVPS = 2*NHG
      IL_EIROP = 2*NHG
      LENGTH   = IL_EIVPS+IL_EIROP
      CALL TEST_SCR('VOFRHOT','LENGTH',LSCR,LENGTH)
C     ==--------------------------------------------------------------==
C     Arrays inside SCR.
      CALL GET_ADDR(IP_EIVPS,SCR(1))
      CALL GET_ADDR(IP_EIROP,SCR(1+IL_EIVPS))
      NNRS  = NR1S*NR2S*NR3S
      CALL EICALC(EIVPS,EIROP)
C..External Field or forces of external potential
      EEXT=0.D0
CEHR[
      IF((TEXTFLD.OR.TEXADD).AND.(.NOT.TD_EXTPOT)) THEN
CEHR]
        CALL EEXTERN(RHOE,V,EIROP,TAU0,FION,EEXT,TFOR)
#ifdef PARALLEL
        CALL GLOSUM(1,EEXT)
#endif
      ENDIF
C     TRANSFORM THE DENSITY TO G SPACE
!$OMP parallel do private(IR)
      DO IR=1,NNR1
        V(IR)=DCMPLX(RHOE(IR),0.D0)
      ENDDO
      CALL FWFFT(V)
C     ==--------------------------------------------------------------==
C     ==        MARTYNA & TUCKERMAN CLUSTER METHOD                    ==
C     ==--------------------------------------------------------------==
C     COMPUTE POTENTIAL ENERGY CONTRIBUTION TO FORCES ON IONS
      IF(TFOR) CALL POTFOR(FION,V,EIROP)
C     COMPUTE ELECTROSTATIC AND PSEUDOPOTENTIAL ENERGIES.
      CALL PPENER(EH,EI,EE,EPS,V,VTEMP,EIVPS,EIROP)
C     Hartree term for all charges
      EHEP = DREAL(EH)*OMEGA
      EHII = DREAL(EI)*OMEGA + NEL*VPLOC
      EHEE = DREAL(EE)*OMEGA
      IF(TDIAG) THEN
C       Ion-Ion interaction - e-e (double counting)
        EHT =  EHII-EHEE
     &       + ESR
     &       - ESELF/DBLE(NPROC)
      ELSE
        EHT =  EHEP
     &       + ESR
     &       - ESELF/DBLE(NPROC)
      ENDIF
      EPSEU=2.D0*DREAL(EPS)*OMEGA
      CALL TIHALT('   VOFRHOT',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE GIVE_SCR_VOFRHOT(LVOFRHOT,TAG)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !NHG
      INCLUDE 'isos.inc'        !TCLUST
      INCLUDE 'cppt.inc'        !NR1H NR2H NR3PL
C     Arguments
      INTEGER   LVOFRHOT
      CHARACTER TAG*30
C     Variables
C     ==--------------------------------------------------------------==
      LVOFRHOT = 2*NHG*2
      TAG      ='2*NHG*2'
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
