C     ==================================================================
      SUBROUTINE V0PSI1_P(C0,C1,C12,VPOT,PSI,drhoe,scr,lscr,NSTATE)
C     ==================================================================
C     == applies the ground state potential (V0) to the perturbation  ==
C     == orbitals. TUNED VERSION, under construction!                 ==
C     == ************************************************************ ==
C     == VPOT:   IN INPUT POTENTIAL                                   ==
C     == C0,C1:  IN ORBITALS                                          ==
C     == DRHOE:  OUT FIRST ORDER DENSITY in REAL SPACE                ==
C     ==         (real*8 storage)                                     ==
C     == C12:    V[ n0 ] applied to C1                                ==
C     ==--  d.s. 3/2001  ---------------------------------------------==
C     ==--  d.s.11/2002  (lsd) ---------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'prcp.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'response_p.inc'
      INCLUDE 'elct.inc'        ! F(occ)
C     Arguments
      INTEGER    NSTATE,lscr
      COMPLEX*16 C0(NGW,NSTATE),C1(NGW,NSTATE),C12(NGW,NSTATE),
     &     PSI(MAXFFT),scr(*)
      REAL*8     VPOT(NNR1,nlsd),drhoe(nnr1,nlsd)
      DIMENSION  F(NSTATE)
C     Variables
      COMPLEX*16  FM,FP,psii,psin
      REAL*8     PSIRE(2,*),SCRRE(2,*),
     &     XSKIN,G2,rho1,rho2,density_factor,ff1,ff2
      POINTER    (IP_PSIRE,PSIRE),(IP_SCRRE,SCRRE)
      INTEGER    ISUB,IWF,lv0psi1,msglen,
     &     IG,IS,IS1,IS2,ir,iss,
     &     ilsd1,ilsd2
      character*30 tag
#ifdef __DERF
      REAL*8 DERF
      EXTERNAL DERF
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('    V0PSI1',ISUB)
      tag = 'V0PSI1:2*maxfft'
      lv0psi1 = 2*maxfft
      call test_scr('v0psi1_p',tag,lscr,lv0psi1)
      IF(NOGRP.NE.1) THEN
        call stopgm('V0psi1','ORBITAL-GROUPS not implemented.')
      ENDIF
      CALL GET_ADDR(IP_PSIRE,PSI)
      CALL GET_ADDR(IP_SCRRE,SCR)
      if (trho1) call azzero(drhoe,nnr1*nlsd)
      IF(TKPNT) THEN 
        call stopgm('V0psi1','K-POINTS not implemented.')
      ENDIF  

      DO IS=1,NSTATE,2
        IS1 = IS
        IS2 = IS+1
        iss = is / 2 + 1
        ilsd1=1
        ilsd2=1
        if (TLSD) then
           if (is1 .gt. nsup) ilsd1 = 2
           if (is2 .gt. nsup) ilsd2 = 2
        endif
        ff1 = F(IS1)
        ff2 = 0.d0
        if (IS2 .LE. NSTATE) ff2=F(IS2)
C     ==------------------------------------------------------------==
C     == FIRST, the GROUND STATE ORBITALS are FFTed to real space   ==
C     ==------------------------------------------------------------==
        if (trho1) then
          IF(TKEEPREALSPACEWFN) THEN
            CALL DCOPY(2*nnr1,c0real(1,iss),1,scr,1)
          ELSE
            CALL ZAZZERO(scr,MAXFFT)
            IF(IS2.GT.NSTATE) THEN
!CDIR NODEP
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC
#endif 
              DO IG=1,NGW
                scr(NZHS(IG))      = C0(IG,IS1)
                scr(INDZS(IG))     = DCONJG(C0(IG,IS1))
              ENDDO
              IF(GEQ0) scr(NZHS(1))=C0(1,IS1)
            ELSE
!CDIR NODEP
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOALIAS 
!OCL NOVREC  
#endif 
              DO IG=1,NGW
                scrre(1,nzhs(ig))  = DREAL(C0(IG,IS1))-DIMAG(C0(IG,IS2))
                scrre(2,nzhs(ig))  = DIMAG(C0(IG,IS1))+DREAL(C0(IG,IS2))
                scrre(1,indzs(ig)) = DREAL(C0(IG,IS1))+DIMAG(C0(IG,IS2))
                scrre(2,indzs(ig)) =-DIMAG(C0(IG,IS1))+DREAL(C0(IG,IS2))
              ENDDO
              IF(GEQ0) scr(NZHS(1))= C0(1,IS1) + UIMAG*C0(1,IS2)
            ENDIF               ! odd number of states and IS=last state...
            CALL S_INVFFT(scr)  ! DO IT.
          ENDIF                 ! keep real space wfns (C0)
        ENDIF                   ! if (trho1)
C     SCR now contains the real-space ground state wavefunctions.        



C     ==------------------------------------------------------------==
C     == SECOND, the PERTURBATION ORBITALS are FFTed to real space  ==
C     ==------------------------------------------------------------==
        CALL ZAZZERO(PSI,MAXFFT)
        IF(IS2.GT.NSTATE) THEN
!CDIR NODEP
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOVREC
#endif 
          DO IG=1,NGW
            PSI(NZHS(IG))=C1(IG,IS1)
            PSI(INDZS(IG))=DCONJG(C1(IG,IS1))
          ENDDO
          IF(GEQ0) PSI(NZHS(1))=C1(1,IS1)
        ELSE
!CDIR NODEP
#ifdef __SR8000
*poption parallel
#endif
#ifdef _vpp_
!OCL NOALIAS 
!OCL NOVREC  
#endif 
          DO IG=1,NGW
            psire(1,nzhs(ig))  = DREAL(C1(IG,IS1)) - DIMAG(C1(IG,IS2))
            psire(2,nzhs(ig))  = DIMAG(C1(IG,IS1)) + DREAL(C1(IG,IS2))
            psire(1,indzs(ig)) = DREAL(C1(IG,IS1)) + DIMAG(C1(IG,IS2))
            psire(2,indzs(ig)) =-DIMAG(C1(IG,IS1)) + DREAL(C1(IG,IS2))
          ENDDO
          IF(GEQ0) PSI(NZHS(1))=C1(1,IS1)+UIMAG*C1(1,IS2)
        ENDIF                   ! odd number of states and IS=last state...
        CALL S_INVFFT(PSI)      ! DO IT.
C     NB: If IS2 .GT. NSTATE, then this FFT will result in an array
C     PSI which is purely real. Therefore all the calculations below
C     will be done with zero numbers and thus not contribute.


C     ==------------------------------------------------------------==
C     == Compute the PERTURBATION DENSITY n^1 (r), and              ==
C     == Apply the potential (V) to the unperturbed orbitals        ==
C     ==------------------------------------------------------------==
        if (trho1) then
!$OMP parallel do private(ir,rho1,rho2)
           do ir=1,nnr1
              rho1 = scrre(1,ir)*psire(1,ir)
              rho2 = scrre(2,ir)*psire(2,ir)
              drhoe(ir,ilsd1)   = drhoe(ir,ilsd1)   + ff1*rho1
              drhoe(ir,ilsd2)   = drhoe(ir,ilsd2)   + ff2*rho2
              PSIRE(1,ir)   = VPOT(ir,ilsd1)  *PSIRE(1,ir)
              PSIRE(2,ir)   = VPOT(ir,ilsd2)  *PSIRE(2,ir)
           enddo
        else
!$OMP parallel do private(ir)
           do ir=1,nnr1
              PSIRE(1,ir)   = VPOT(ir,ilsd1)  *PSIRE(1,ir)
              PSIRE(2,ir)   = VPOT(ir,ilsd2)  *PSIRE(2,ir)
           enddo
        endif
C     ==------------------------------------------------------------==
C     == Back transform to reciprocal space the product V.PSI       ==
C     ==------------------------------------------------------------==
        CALL S_FWFFT(PSIRE)
C     ==------------------------------------------------------------==
C     == Decode the combination of wavefunctions and add the        ==
C     == kinetic energy term.                                       ==
C     ==------------------------------------------------------------==
        IF(AKIN.GT.1.d-10) THEN
          call stopgm('V0psi1','CONSTANT CUTOFF not implemented.')
        ENDIF
!$OMP parallel do private(IG,G2,psin,psii,FP,FM)
        DO IG=1,NGW
           G2 = - 0.5d0*TPIBA2*HG(IG)
           psin=PSI(NZHS(IG))  ! access only once
           psii=PSI(INDZS(IG)) ! these mem locations
           FP=-0.5d0*(psin+psii)
           FM=-0.5d0*(psin-psii)
cmb           FP = - 0.5d0*(PSI(NZHS(IG))+PSI(INDZS(IG)))
cmb           FM = - 0.5d0*(PSI(NZHS(IG))-PSI(INDZS(IG)))
           C12(IG,IS1)= ff1 * (G2*C1(IG,IS1)
     *          + DCMPLX(DREAL(FP),DIMAG(FM)))
           IF(IS2.LE.NSTATE)
     *          C12(IG,IS2)= ff2 * (G2*C1(IG,IS2)
     *          +  DCMPLX(DIMAG(FP),-DREAL(FM)))
        ENDDO
      ENDDO                     ! Loop over IS

      if (trho1) then
         if (TLSD) call DAXPY(nnr1,1.d0,drhoe(1,2),1,drhoe(1,1),1)
         density_factor = 2d0 / omega
         call DSCAL(nlsd*nnr1,density_factor,drhoe,1)
      endif

      CALL TIHALT('    V0PSI1',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE V1PSI0_P(c0,c12,vpot,psi,nstate)
C     ==================================================================
C     == VPOT:   IN INPUT POTENTIAL V_Hxc [n1]                        ==
C     == C12:    IN partial gradient  dE/dC1                         ==
C     == C12:    OUT C12 + V_Hxc [n1] * C0                            ==
C     ==--  d.s. 3/2001  ---------------------------------------------==
C     ==--  d.s.11/2002  (lsd)  --------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'spin.inc'
      INCLUDE 'cppt.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'prcp.inc'
      INCLUDE 'kpts.inc'
      INCLUDE 'kpnt.inc'
      INCLUDE 'response_p.inc'
      INCLUDE 'elct.inc'        ! F(occ)
C     Arguments
      INTEGER    NSTATE
      COMPLEX*16 C0(NGW,NSTATE),C12(NGW,NSTATE),
     &    PSI(NNR1)
      REAL*8     VPOT(NNR1,nlsd)
      DIMENSION  F(NSTATE)
C     Variables
      COMPLEX*16 FM,FP,psii,psin
      REAL*8     PSIRE(2,*),ff1,ff2
      POINTER    (IP_PSIRE,PSIRE)
      INTEGER    ISUB,IWF,IG,IS,IS1,IS2,ir,iss,
     &     ilsd1,ilsd2
      character*30 tag
#ifdef PARALLEL
      INTEGER MSGLEN
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('    V0PSI1',ISUB)

      IF(NOGRP.NE.1) THEN
        call stopgm('V0psi1','ORBITAL-GROUPS not implemented.')
      ENDIF
      IF(TKPNT) THEN 
        call stopgm('V0psi1','K-POINTS not implemented.')
      ENDIF  

      CALL GET_ADDR(IP_PSIRE,PSI)

      DO IS=1,NSTATE,2
        IS1 = IS
        IS2 = IS+1
        iss = is / 2 + 1
        ilsd1=1
        ilsd2=1
        if (TLSD) then
           if (is1 .gt. nsup) ilsd1 = 2
           if (is2 .gt. nsup) ilsd2 = 2
        endif
        ff1 = F(IS1)
        ff2 = 0.d0
        if (IS2 .LE. NSTATE) ff2=F(IS2)
C     ==------------------------------------------------------------==
C     == FIRST, the GROUND STATE ORBITALS are FFTed to real space   ==
C     ==------------------------------------------------------------==
        IF(TKEEPREALSPACEWFN) THEN
          CALL DCOPY(2*nnr1,c0real(1,iss),1,psi,1)
        ELSE
          CALL ZAZZERO(PSI,MAXFFT)
          IF(IS2.GT.NSTATE) THEN
!CDIR NODEP
#ifdef __SR8000
*poption parallel
#endif
            DO IG=1,NGW
              PSI(NZHS(IG))=C0(IG,IS1)
              PSI(INDZS(IG))=DCONJG(C0(IG,IS1))
            ENDDO
            IF(GEQ0) PSI(NZHS(1))=C0(1,IS1)
          ELSE
!CDIR NODEP
#ifdef __SR8000
*poption parallel
#endif
            DO IG=1,NGW
              psire(1,nzhs(ig))  = DREAL(C0(IG,IS1)) - DIMAG(C0(IG,IS2))
              psire(2,nzhs(ig))  = DIMAG(C0(IG,IS1)) + DREAL(C0(IG,IS2))
              psire(1,indzs(ig)) = DREAL(C0(IG,IS1)) + DIMAG(C0(IG,IS2))
              psire(2,indzs(ig)) =-DIMAG(C0(IG,IS1)) + DREAL(C0(IG,IS2))
            ENDDO
            IF(GEQ0) PSI(NZHS(1))=C0(1,IS1)+UIMAG*C0(1,IS2)
          ENDIF                 ! odd number of states and IS=last state...
          CALL S_INVFFT(PSI)    ! DO IT.
        ENDIF                   ! REAL SPACE c0
C     ==------------------------------------------------------------==
C     == Apply the potential (VPOT) to the orbitals                 ==
C     ==------------------------------------------------------------==
        do ir=1,nnr1
          PSIRE(1,ir)   = VPOT(ir,ilsd1)  *PSIRE(1,ir)
          PSIRE(2,ir)   = VPOT(ir,ilsd2)  *PSIRE(2,ir)
        enddo

C     ==------------------------------------------------------------==
C     == Back transform to reciprocal space the product V.PSI       ==
C     ==------------------------------------------------------------==
        CALL S_FWFFT(PSIRE)
C     ==------------------------------------------------------------==
C     == Decode the combination of wavefunctions                    ==
C     ==------------------------------------------------------------==
        IF(AKIN.GT.1.d-10) THEN
          call stopgm('V0psi1','CONSTANT CUTOFF not implemented.')
        ELSE
!CDIR NODEP
          DO IG=1,NGW
            psii=PSI(NZHS(IG))  ! access only once
            psin=PSI(INDZS(IG)) ! these mem locations
            FP=0.5d0*(psii+psin)
            FM=0.5d0*(psii-psin)
cmb            FP = 0.5d0*(PSI(NZHS(IG))+PSI(INDZS(IG)))
cmb            FM = 0.5d0*(PSI(NZHS(IG))-PSI(INDZS(IG)))
            C12(IG,IS1)=C12(IG,IS1)-ff1*DCMPLX(DREAL(FP),DIMAG(FM))
            IF(IS2.LE.NSTATE)
     *          C12(IG,IS2)=C12(IG,IS2)-ff2*DCMPLX(DIMAG(FP),-DREAL(FM))
          ENDDO
        ENDIF
      ENDDO                     ! Loop over IS

      CALL TIHALT('    V0PSI1',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================



C     ==================================================================
      SUBROUTINE allocate_c0real_p(c0,psi,nstate)
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'response_p.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'cppt.inc'
      integer nstate
      complex*16   c0(ngw,nstate)
      real*8  psi(2,*)
      integer il_c0real,is,is1,is2,iss,ig
C     ==--------------------------------------------------------------==

      il_c0real = INT((nstate)/2+1)*2 * nnr1
      call memory(ip_c0real, il_c0real, 'C0-REAL')
      call zazzero(c0real, nnr1*nstate/2)

      do is=1,nstate,2
        CALL ZAZZERO(PSI,MAXFFT)
        IS1 = IS
        IS2 = IS+1
        IF(IS2.GT.NSTATE) THEN
!CDIR NODEP
#ifdef __SR8000
*poption parallel
#endif
          DO IG=1,NGW
            PSI(1,NZHS(IG))   = DREAL(C0(IG,IS1))
            PSI(2,NZHS(IG))   = DIMAG(C0(IG,IS1))
            PSI(1,INDZS(IG))  = DREAL(C0(IG,IS1))
            PSI(2,INDZS(IG))  =-DIMAG(C0(IG,IS1))
          ENDDO
          IF(GEQ0) then
            PSI(1,NZHS(1))=DREAL(C0(1,IS1))
            PSI(2,NZHS(1))=0d0
          endif
        ELSE
!CDIR NODEP
#ifdef __SR8000
*poption parallel
#endif
          DO IG=1,NGW
            psi(1,nzhs(ig))  = DREAL(C0(IG,IS1)) - DIMAG(C0(IG,IS2))
            psi(2,nzhs(ig))  = DIMAG(C0(IG,IS1)) + DREAL(C0(IG,IS2))
            psi(1,indzs(ig)) = DREAL(C0(IG,IS1)) + DIMAG(C0(IG,IS2))
            psi(2,indzs(ig)) =-DIMAG(C0(IG,IS1)) + DREAL(C0(IG,IS2))
          ENDDO
          IF(GEQ0) then
            PSI(1,NZHS(1)) = DREAL(C0(1,IS1))
            PSI(2,NZHS(1)) = DREAL(C0(1,IS2))
          endif
        ENDIF                   ! odd number of states and IS=last state...
        CALL S_INVFFT(PSI)      ! DO IT.
        iss = is / 2 + 1
        call DCOPY(2*nnr1,psi,1,c0real(1,iss),1)
      enddo                     ! IS
      return
      end
C     ==================================================================

C     ==================================================================
      SUBROUTINE release_c0real_p
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'response_p.inc'
C     ==--------------------------------------------------------------==

      call freem(ip_c0real)

      return
      end
C     ==================================================================
