C     ==================================================================
      SUBROUTINE XCENER(SXC,VXC,RHOVAL,RHOE,V)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     == EXC:     EXCHANGE AND CORRELATION ENERGY                     ==
C     == VXC:     ENERGY FROM EXC POTENTIAL (Int{n exc(n)})           ==
C     == RHOVAL:  VALENCE CHARGE DENSITY                              ==
C     == RHOE:    TOTAL CHARGE DENSITY (IF TINLC CORE+VALENCE)        ==
C     == V:        IN -> POTENTIAL IN REAL SPACE                      ==
C     ==          OUT -> POTENTIAL + EXC POTENTIAL IN REAL SPACE      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'tbxc.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'fft.inc'
C     Arguments
      REAL*8     SXC,VXC,RHOVAL(NNR1,NLSD),RHOE(NNR1,NLSD)
      COMPLEX*16 V(MAXFFT,NLSD)
C     ==--------------------------------------------------------------==
      IF(TOLDCODE) THEN
        CALL XCENER_OLD(SXC,VXC,RHOVAL,RHOE,V)
      ELSE
        CALL XCENER_NEW(SXC,VXC,RHOVAL,RHOE,V)
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE XCENER_OLD(SXC,VXC,RHOVAL,RHOE,V)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     == EXC:     EXCHANGE AND CORRELATION ENERGY                     ==
C     == VXC:     ENERGY FROM EXC POTENTIAL (Int{n exc(n)})           ==
C     == RHOVAL:  VALENCE CHARGE DENSITY                              ==
C     == RHOE:    TOTAL CHARGE DENSITY                                ==
C     == V:        IN -> POTENTIAL IN REAL SPACE                      ==
C     ==          OUT -> POTENTIAL + EXC POTENTIAL IN REAL SPACE      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'tbxc.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'func.inc'
      INCLUDE 'fft.inc'
C     Arguments
      REAL*8     SXC,VXC,RHOVAL(NNR1,NLSD),RHOE(NNR1,NLSD)
      COMPLEX*16 V(MAXFFT,NLSD)
C     Variables
      REAL*8     ROE,VXC1,EXC1,RATIO,EX,EC,VX,VC,DD,EE,ETA,VXA,VCA,
     &           VXB,VCB,VXCA,VXCB,EXC
      INTEGER    ISUB,I,I1,IR
C     ==--------------------------------------------------------------==
      CALL TISET('    XCENER',ISUB)
      SXC=0.D0
      VXC=0.D0
      IF(TLSD) CALL DCOPY(2*NNR1,V(1,1),1,V(1,2),1)
      IF(MFXCC.NE.0.OR.MFXCX.NE.0) THEN
        IF(NARRAY.LT.2.AND..NOT.TLSD) THEN
#ifdef __SR8000
*poption parallel
*poption tlocal(EX,EC,VX,VC,VXC1,EXC1)
#endif
!$OMP parallel do private(I,ROE,EX,EC,VX,VC,VXC1,EXC1)
!$OMP+ reduction(+:SXC,VXC)
          DO I=1,NNR1
            ROE=DMAX1(RHOE(I,1),0.0D0)
            CALL XC(ROE,EX,EC,VX,VC)
            IF(THYBRID) THEN
              EX=EX*PXLDA
              VX=VX*PXLDA
              EC=EC*PCLDA
              VC=VC*PCLDA
            ENDIF
            VXC1=VX+VC
            EXC1=EX+EC
            V(I,1)=V(I,1)+DCMPLX(VXC1,0.0D0)
            SXC=SXC+EXC1*ROE
            VXC=VXC+VXC1*DMAX1(RHOVAL(I,1),0.0D0)
          ENDDO
        ELSE
          IF(.NOT.TLSD) THEN
#ifdef __SR8000
*poption parallel
*poption tlocal(EX,EC,VX,VC,VXC1,EXC1)
*poption tlocal(ROE)
*voption pvfunc(3)
#endif
!$OMP parallel do private(I,ROE,RATIO,I1,DD,EE)
!$OMP+ private(VXC1,EXC1,EX,VX,EC,VC)
!$OMP+ reduction(+:SXC,VXC)
            DO I=1,NNR1 
              ROE=DMAX1(RHOE(I,1),0.0D0)
              RATIO=(1.D0/DDRO)*ROE
              I1=RATIO
              IF(I1.LT.NARRAY) THEN
                DD=RATIO-DBLE(I1)
                EE=1.D0-DD
                VXC1=VVXC(I1)*EE+VVXC(I1+1)*DD
                EXC1=EEXC(I1)*EE+EEXC(I1+1)*DD
              ELSE
                CALL XC(ROE,EX,EC,VX,VC)
                IF(THYBRID) THEN
                  EX=EX*PXLDA
                  VX=VX*PXLDA
                  EC=EC*PCLDA
                  VC=VC*PCLDA
                ENDIF
                VXC1=VX+VC
                EXC1=EX+EC
              ENDIF
              V(I,1)=V(I,1)+DCMPLX(VXC1,0.0D0)
              SXC=SXC+EXC1*ROE
              VXC=VXC+VXC1*DMAX1(RHOVAL(I,1),0.0D0)
            ENDDO
          ELSE
C         ==----------------------------------------------------------==
C         ==  INPUT : RHOE(1..NNR1)  ALPHA + BETA DENSITY             ==
C         ==          RHOE(NNR1+1 ..2*NNR1)  BETA DENSITY             ==
C         ==          V(1..NNR1)   LOCAL POTENTIAL                    ==
C         ==  OUTPUT  V(1..NNR1)   ALPHA POTENTIAL                    ==
C         ==          V(NNR1+1..2*NNR1)   BETA POTENTIAL              ==
C         ==----------------------------------------------------------==
#ifdef __SR8000
*poption parallel
*poption tlocal(EX,EC,VXA,VCA,VXB,VCB,VXCA,VXCB,EXC)
#endif
!$OMP parallel do private(IR,ROE,ETA,EX,EC,VXA,VCA,VXCA)
!$OMP+ private(VXB,VCB,VXCB,EXC)
!$OMP+ reduction(+:SXC,VXC)
            DO IR=1,NNR1
              ROE = DMAX1(RHOE(IR,1),1.0D-15)
              ETA = (RHOE(IR,1)-2.D0*RHOE(IR,2))/ROE
              IF(DABS(ETA).GT.1.D0) ETA=SIGN(1.0D0,ETA)
              CALL XC_LSD(ROE,ETA,EX,EC,VXA,VCA,VXB,VCB)
              IF(THYBRID) THEN
                EX=EX*PXLDA
                VXA=VXA*PXLDA
                VXB=VXB*PXLDA
                EC=EC*PCLDA
                VCA=VCA*PCLDA
                VCB=VCB*PCLDA
              ENDIF
              VXCA=VXA+VCA
              VXCB=VXB+VCB
              EXC=EX+EC
              V(IR,1)=V(IR,1)+DCMPLX(VXCA,0.0D0)
              V(IR,2)=V(IR,2)+DCMPLX(VXCB,0.0D0)
              SXC=SXC+EXC*ROE
C.aa
              VXC=VXC+(VXCA*(RHOVAL(IR,1)-RHOVAL(IR,2))+
     *                VXCB*RHOVAL(IR,2))
C.aa
            ENDDO
          ENDIF
        ENDIF
      ENDIF
      CALL TIHALT('    XCENER',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE XCENER_NEW(SXC,VXC,RHOVAL,RHOE,V)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     == EXC:     EXCHANGE AND CORRELATION ENERGY                     ==
C     == VXC:     ENERGY FROM EXC POTENTIAL (Int{n exc(n)})           ==
C     == RHOVAL:  VALENCE CHARGE DENSITY                              ==
C     == RHOE:    TOTAL CHARGE DENSITY                                ==
C     == V:        IN -> POTENTIAL IN REAL SPACE                      ==
C     ==          OUT -> POTENTIAL + EXC POTENTIAL IN REAL SPACE      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'tbxc.inc'
      INCLUDE 'func.inc'
      INCLUDE 'spin.inc'
      INCLUDE 'fft.inc'
C     Arguments
      REAL*8     SXC,VXC,RHOVAL(NNR1,NLSD),RHOE(NNR1,NLSD)
      COMPLEX*16 V(MAXFFT,NLSD)
C     Variables
      REAL*8     FLOPS
      INTEGER    ISUB,NTAB,IR
C     ==--------------------------------------------------------------==
      CALL TISET('    XCENER',ISUB)
      SXC=0.D0
      VXC=0.D0
      NTAB=0
      IF(.NOT.TLSD) THEN
        IF(MFXCC.EQ.0.AND.MFXCX.EQ.1) THEN
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+(-DABS(RHOVAL(IR,1))*DREAL(V(IR,1)))
          ENDDO
          CALL XALPU(SALPHA,NR1,NR2S,NR3S,KR1,KR2S,KR3S,
     *               RHOE,V,SXC,FLOPS)
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+DABS(RHOVAL(IR,1))*DREAL(V(IR,1))
          ENDDO
        ELSEIF(MFXCC.EQ.3.AND.MFXCX.EQ.1) THEN
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+(-DABS(RHOVAL(IR,1))*DREAL(V(IR,1)))
          ENDDO
          CALL LYPUU(NR1,NR2S,NR3S,KR1,KR2S,KR3S,RHOE,V,SXC,FLOPS)
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+DABS(RHOVAL(IR,1))*DREAL(V(IR,1))
          ENDDO
        ELSEIF(MFXCC.EQ.9) THEN
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+(-DABS(RHOVAL(IR,1))*DREAL(V(IR,1)))
          ENDDO
          CALL MIKEU(NR1,NR2S,NR3S,KR1,KR2S,KR3S,RHOE,V,SXC,FLOPS)
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+DABS(RHOVAL(IR,1))*DREAL(V(IR,1))
          ENDDO
        ELSE
          CALL XCENER_OLD(SXC,VXC,RHOVAL,RHOE,V)
        ENDIF
      ELSE
C       ==------------------------------------------------------------==
C       ==  INPUT : RHOE(1..NNR1)  ALPHA + BETA DENSITY               ==
C       ==          RHOE(NNR1+1 ..2*NNR1)  BETA DENSITY               ==
C       ==          V(1..NNR1)   LOCAL POTENTIAL                      ==
C       ==  OUTPUT  V(1..NNR1,1)  ALPHA POTENTIAL                     ==
C       ==          V(1..NNR1,2)   BETA POTENTIAL                     ==
C       ==------------------------------------------------------------==
        CALL DCOPY(2*NNR1,V(1,1),1,V(1,2),1)
        IF(MFXCC.EQ.0.AND.MFXCX.EQ.1) THEN
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+(-DABS(RHOVAL(IR,1))*DREAL(V(IR,1)))
          ENDDO
          CALL XALPSP(SALPHA,NR1,NR2S,NR3S,KR1,KR2S,KR3S,
     *                RHOE,V(1,1),V(1,2),SXC,FLOPS)
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+(DABS(RHOVAL(IR,1)-RHOVAL(IR,2))*DREAL(V(IR,1))
     *             +DABS(RHOVAL(IR,2))*DREAL(V(IR,2)))
          ENDDO
        ELSEIF(MFXCC.EQ.3.AND.MFXCX.EQ.1) THEN
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+(-DABS(RHOVAL(IR,1))*DREAL(V(IR,1)))
          ENDDO
          CALL SLYPSP(NR1,NR2S,NR3S,KR1,KR2S,KR3S,RHOE,V(1,1),V(1,2),
     *                SALPHA,SXC,FLOPS)
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+(DABS(RHOVAL(IR,1)-RHOVAL(IR,2))*DREAL(V(IR,1))
     *             +DABS(RHOVAL(IR,2))*DREAL(V(IR,2)))
          ENDDO
        ELSEIF(MFXCC.EQ.9) THEN
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+(-DABS(RHOVAL(IR,1))*DREAL(V(IR,1)))
          ENDDO
          CALL MIKESP(NR1,NR2S,NR3S,KR1,KR2S,KR3S,RHOE,V(1,1),V(1,2),
     *                SXC,FLOPS)
!$OMP parallel do private(IR) reduction(+:VXC)
          DO IR=1,NNR1
            VXC=VXC+(DABS(RHOVAL(IR,1)-RHOVAL(IR,2))*DREAL(V(IR,1))
     *             +DABS(RHOVAL(IR,2))*DREAL(V(IR,2)))
          ENDDO
        ELSE
          CALL XCENER_OLD(SXC,VXC,RHOVAL,RHOE,V)
        ENDIF
      ENDIF
      CALL TIHALT('    XCENER',ISUB)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
