#ifdef __SR8000
*option MP(P(0)), LANGLVL(SAVE(0))
#endif
C     ==================================================================
      FUNCTION YLMR(L,IG,GK)
C     ==----------------------------------------------------------------==
C     == REAL SPHERICAL HARMONICS,  L IS COMBINED INDEX                 ==
C     == FOR lm  (L=1,2...9) ORDER:                                     ==
C     == s, p_x, p_z, p_y, d_xy, d_xz, d_z^2, d_yz, d_x^2-y^2           ==
C     ==----------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'cnst.inc'
C     Arguments
      INTEGER L,IG
      REAL*8  GK(3,*)
      REAL*8  YLMR
C     Variables
      REAL*8  X,Y,Z,R,HG,YLM,
     &        DMAX1
C     ==----------------------------------------------------------------==
      X = GK(1,IG)
      Y = GK(2,IG)
      Z = GK(3,IG)
      R = DMAX1(SQRT(X*X+Y*Y+Z*Z),1.0D-6)
      X = X/R
      Y = Y/R
      Z = Z/R
      IF(L.EQ.1) THEN
        YLMR = SQRT(1.0D0/FPI)
      ELSE IF(L.EQ.2) THEN
        YLMR = SQRT(3.0D0/FPI)*X
      ELSE IF(L.EQ.3) THEN
        YLMR = SQRT(3.0D0/FPI)*Z
      ELSE IF(L.EQ.4) THEN
        YLMR = SQRT(3.0D0/FPI)*Y
      ELSE IF(L.EQ.5) THEN
        YLMR = SQRT(15.0D0/FPI)*X*Y
      ELSE IF(L.EQ.6) THEN
        YLMR = SQRT(15.0D0/FPI)*X*Z
      ELSE IF(L.EQ.7) THEN
        YLMR = SQRT(5.0D0/FPI/4.D0)*(2.D0*Z*Z-X*X-Y*Y)
      ELSE IF(L.EQ.8) THEN
        YLMR = SQRT(15.0D0/FPI)*Y*Z
      ELSE IF(L.EQ.9) THEN
        YLMR = SQRT(15.0D0/FPI/4.D0)*(X*X-Y*Y)
      ELSE IF(L.GE.10) THEN
        HG=R*R
        CALL YLMR2(L,1,HG,GK(1,IG),YLM)
        YLMR=YLM
      ENDIF
C     ==----------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE BESS(XG,L,MMAX,R,JL)
C     ==----------------------------------------------------------------==
C     == CALCULATES SPHERICAL BESSEL FUNCTIONS                          ==
C     ==----------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   L,MMAX
      REAL*8    XG,JL(MMAX),R(MMAX)
C     Variables
      REAL*8    EPS
      PARAMETER (EPS=1.D-8)
      REAL*8    XRG,BESSL
      EXTERNAL  BESSL
      INTEGER   IR
C     ==----------------------------------------------------------------==
!$OMP parallel do private(IR,XRG) 
      DO IR=1,MMAX
        XRG=R(IR)*XG
        JL(IR)=BESSL(L-1,XRG)
      ENDDO
C     ==----------------------------------------------------------------==
      RETURN
      END
C     ====================================================================
      FUNCTION YLMKR(L,IG,GK,RK,IPLUS)
C     ==----------------------------------------------------------------==
C     == REAL SPHERICAL HARMONICS,  L IS COMBINED INDEX FOR lm          ==
C     == (L=1,2...9) ORDER:                                             ==
C     == s, p_x, p_z, p_y, d_xy, d_xz, d_z^2, d_yz, d_x^2-y^2           ==
C     ==----------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'cnst.inc'
C     Arguments
      INTEGER L,IG,IPLUS
      REAL*8  GK(3,*),RK(3)
      REAL*8  YLMKR
C     Variables
      REAL*8  X,Y,Z,HG,YLM,R
C     ==----------------------------------------------------------------==
      IF(IPLUS.EQ.1) THEN 
        X = RK(1)+GK(1,IG)
        Y = RK(2)+GK(2,IG)
        Z = RK(3)+GK(3,IG)
      ELSEIF(IPLUS.EQ.-1) THEN 
        X = RK(1)-GK(1,IG)
        Y = RK(2)-GK(2,IG)
        Z = RK(3)-GK(3,IG)
      ELSE
        CALL STOPGM(' YLMRK','IPLUS WRONG')
      ENDIF
      R = DMAX1(SQRT(X*X+Y*Y+Z*Z),1.0D-6)
      X = X/R
      Y = Y/R
      Z = Z/R
      IF(L.EQ.1) THEN
        YLMKR = SQRT(1.D0/FPI)
      ELSE IF(L.EQ.2) THEN
        YLMKR = SQRT(3.D0/FPI)*X
      ELSE IF(L.EQ.3) THEN
        YLMKR = SQRT(3.D0/FPI)*Z
      ELSE IF(L.EQ.4) THEN
        YLMKR = SQRT(3.D0/FPI)*Y
      ELSE IF(L.EQ.5) THEN
        YLMKR = SQRT(15.D0/FPI)*X*Y
      ELSE IF(L.EQ.6) THEN
        YLMKR = SQRT(15.D0/FPI)*X*Z
      ELSE IF(L.EQ.7) THEN
        YLMKR = SQRT(5.D0/FPI/4.D0)*(2.D0*Z*Z-X*X-Y*Y)
      ELSE IF(L.EQ.8) THEN
        YLMKR = SQRT(15.D0/FPI)*Y*Z
      ELSE IF(L.EQ.9) THEN
        YLMKR = SQRT(15.D0/FPI/4.D0)*(X*X-Y*Y)
      ELSE IF(L.GE.10) THEN
        CALL STOPGM(' YLMKR',' NOT IMPLEMENTED') 
        HG=R*R
        CALL YLMR2(L,1,HG,GK(1,IG),YLM)
        YLMKR=YLM
      ENDIF
C     ==----------------------------------------------------------------==
      RETURN
      END
C     ====================================================================
