C     ==================================================================
      SUBROUTINE YLMR2(L,NHG,HG,GK,YLM)
C     ==--------------------------------------------------------------==
C     REAL SPHERICAL HARMONICS,  L IS COMBINED INDEX FOR LM (L=1,2...25)
C     ORDER:  S, P_X, P_Z, P_Y, D_XY, D_XZ, D_Z^2, D_YZ, D_X^2-Y^2  ....
C     THE REAL SPHERICAL HARMONICS USED HERE FORM BASES FOR THE
C     IRREDUCIBLE REPRESENTATIONS OF THE GROUP O
C
C     SEE WIESSBLUTH 'ATOMS AND MOLECULES' PAGES 128-130
C     ERRORS IN WEISSBLUTH HAVE BEEN CORRECTED:
C        1.) ELIMINATION OF THE 7 FROM L=20
C        2.) ADDITION OF THE FACTOR 1./SQRT(12.) TO L=25
C
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'cnst.inc'
      INCLUDE 'geq0.inc'
C     Arguments
      INTEGER L,NHG
      REAL*8  HG(*),GK(3,*),YLM(*)
C     Variables
      INTEGER IG
      REAL*8  C
      LOGICAL ZERO
C     ==--------------------------------------------------------------==
C     NOTE :   Y_LM (G=0) = SQRT(FPI)  WHEN L=0  AND  = 0  WHEN L>0
C     ==--------------------------------------------------------------==
      ZERO=.FALSE.
      IF(GEQ0.AND.L.GT.1) THEN
        IF(ABS(HG(1)).LT.1.D-6) THEN
          ZERO=.TRUE.
          HG(1)=1.D0
        ENDIF
      ENDIF
      IF(L.EQ.1) THEN
        C=SQRT(1.D0/FPI)
        DO IG=1,NHG
          YLM(IG) = C
        ENDDO
      ELSE IF(L.EQ.2) THEN
        C=SQRT(3.D0/FPI)
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG
          YLM(IG) = C*GK(1,IG)/SQRT(HG(IG))   !   X
        ENDDO
      ELSE IF(L.EQ.3) THEN
        C=SQRT(3.D0/FPI)
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG
          YLM(IG) = C*GK(3,IG)/SQRT(HG(IG))   !   Z
        ENDDO
      ELSE IF(L.EQ.4) THEN
        C=SQRT(3.D0/FPI)
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG
          YLM(IG) = C*GK(2,IG)/SQRT(HG(IG))   !   Y
        ENDDO
      ELSE IF(L.EQ.5) THEN
        C=SQRT(15.D0/FPI)
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG
          YLM(IG) = C*GK(1,IG)*GK(2,IG)/HG(IG)   !  X*Y
        ENDDO
      ELSE IF(L.EQ.6) THEN
        C=SQRT(15.D0/FPI)
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG
          YLM(IG) = C*GK(1,IG)*GK(3,IG)/HG(IG)   !  X*Z
        ENDDO
      ELSE IF(L.EQ.7) THEN
        C=SQRT(5.D0/FPI/4.D0)
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG
          YLM(IG) = C*(3.D0*GK(3,IG)**2/HG(IG)-1.D0) ! (3.*Z*Z-1.0)
        ENDDO
      ELSE IF(L.EQ.8) THEN
        C=SQRT(15.D0/FPI)
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG
          YLM(IG) = C*GK(2,IG)*GK(3,IG)/HG(IG)   !  Y*Z
        ENDDO
      ELSE IF(L.EQ.9) THEN
        C=SQRT(15.D0/FPI/4.D0)
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG
          YLM(IG) = C*(GK(1,IG)**2-GK(2,IG)**2)/HG(IG)  ! X*X-Y*Y
        ENDDO
      ELSE IF(L.EQ.10) THEN
        C=SQRT(7.D0/FPI)*5.D0/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG
          YLM(IG) = C*GK(1,IG)*(GK(1,IG)**2-0.6D0*HG(IG))/
     +              (HG(IG)*SQRT(HG(IG)))                ! X(X^2-3R^2/5)
        ENDDO
      ELSE IF(L.EQ.11) THEN
        C=SQRT(7.D0/FPI)*5.D0/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! Y(Y^2-3R^2/5)
          YLM(IG) = C*GK(2,IG)*(GK(2,IG)**2-0.6D0*HG(IG))/
     +              (HG(IG)*SQRT(HG(IG)))
        ENDDO
      ELSE IF(L.EQ.12) THEN
        C=SQRT(7.D0*15.D0/FPI)
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                !  XYZ
          YLM(IG) = C*GK(1,IG)*GK(2,IG)*GK(3,IG)/(HG(IG)*SQRT(HG(IG)))
        ENDDO
      ELSE IF(L.EQ.13) THEN
        C=SQRT(7.D0/FPI)*5.D0/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! Z(Z^2-.6R^2)
          YLM(IG) = C*GK(3,IG)*(GK(3,IG)**2-0.6D0*HG(IG))/
     +              (HG(IG)*SQRT(HG(IG)))
        ENDDO
      ELSE IF(L.EQ.14) THEN
        C=SQRT(7.D0*15.D0/FPI)/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! Z(X^2-Y^2)
          YLM(IG) = C*GK(3,IG)*(GK(1,IG)**2-GK(2,IG)**2)/
     +              (HG(IG)*SQRT(HG(IG)))
        ENDDO
      ELSE IF(L.EQ.15) THEN
        C=SQRT(7.D0*15.D0/FPI)/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! Y(Z^2-X^2)
          YLM(IG) = C*GK(2,IG)*(GK(3,IG)**2-GK(1,IG)**2)/
     +              (HG(IG)*SQRT(HG(IG)))
        ENDDO
      ELSE IF(L.EQ.16) THEN
        C=SQRT(7.D0*15.D0/FPI)/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! X(Y^2-Z^2)
          YLM(IG) = C*GK(1,IG)*(GK(2,IG)**2-GK(3,IG)**2)/
     +              (HG(IG)*SQRT(HG(IG)))
        ENDDO
      ELSE IF(L.EQ.17) THEN
        C=SQRT(3.D0*7.D0/FPI)*5.D0/4.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! A1
          YLM(IG) = C*((GK(1,IG)**4+GK(2,IG)**4+GK(3,IG)**4)/
     +              (HG(IG)*HG(IG))-0.6D0)
        ENDDO
      ELSE IF(L.EQ.18) THEN
        C=SQRT(9.D0*35.D0/FPI)/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! YZ(Y^2-Z^2)
          YLM(IG) = C*GK(2,IG)*GK(3,IG)*(GK(2,IG)**2-GK(3,IG)**2)/
     +              (HG(IG)*HG(IG))
        ENDDO
      ELSE IF(L.EQ.19) THEN
        C=SQRT(9.D0*35.D0/FPI)/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! ZX(Z^2-X^2)
          YLM(IG) = C*GK(1,IG)*GK(3,IG)*(GK(3,IG)**2-GK(1,IG)**2)/
     +              (HG(IG)*HG(IG))
        ENDDO
      ELSE IF(L.EQ.20) THEN
        C=SQRT(9.D0*5.D0/FPI)/4.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! E\EPSILON
          YLM(IG) = C*((GK(1,IG)**4-GK(2,IG)**4)-
     +      6.D0*GK(3,IG)**2*(GK(1,IG)**2-GK(2,IG)**2))/(HG(IG)*HG(IG))
        ENDDO
      ELSE IF(L.EQ.21) THEN
        C=SQRT(9.D0*35.D0/FPI)/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! XY(X^2-Y^2)
          YLM(IG) = C*GK(1,IG)*GK(2,IG)*(GK(1,IG)**2-GK(2,IG)**2)/
     +              (HG(IG)*HG(IG))
        ENDDO
      ELSE IF(L.EQ.22) THEN
        C=SQRT(9.D0*5.D0/FPI)*7.D0/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! XY(Z^2-1/7*R^2)
          YLM(IG) = C*GK(1,IG)*GK(2,IG)*(GK(3,IG)**2-HG(IG)/7.D0)/
     +              (HG(IG)*HG(IG))
        ENDDO
      ELSE IF(L.EQ.23) THEN
        C=SQRT(9.D0*5.D0/FPI)*7.D0/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! ZX(Y^2-1/7*R^2)
          YLM(IG) = C*GK(1,IG)*GK(3,IG)*(GK(2,IG)**2-HG(IG)/7.D0)/
     +              (HG(IG)*HG(IG))
        ENDDO
      ELSE IF(L.EQ.24) THEN
        C=SQRT(9.D0*5.D0/FPI)*7.D0/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! YZ(X^2-1/7*R^2)
          YLM(IG) = C*GK(2,IG)*GK(3,IG)*(GK(1,IG)**2-HG(IG)/7.D0)/
     +              (HG(IG)*HG(IG))
        ENDDO
      ELSE IF(L.EQ.25) THEN
        C=SQRT(9.D0*5.D0/FPI/3.D0)*7.D0/2.D0
C$OMP parallel do private(IG) shared(YLM)
        DO IG=1,NHG                                ! E\THETA
          YLM(IG) = C*( GK(3,IG)**4-0.5D0*(GK(1,IG)**4+GK(2,IG)**4)-
     +         6.D0/7.D0*HG(IG)
     &         *(GK(3,IG)**2-0.5D0*(GK(1,IG)**2+GK(2,IG)**2) ))
     +         /( HG(IG)*HG(IG) )
        ENDDO
      ELSE IF(L.GE.26) THEN
        WRITE(6,*) 'GIVEN L=',L,' [MAX L = 26]'
        CALL STOPGM(' YLMR2',' HIGHER L NOT PROGRAMMED ')
      ENDIF
      IF(ZERO) THEN
        YLM(1)=0.0D0
        HG(1)=0.0D0
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
