C        1         2         3         4         5         6         7    
C23456789012345678901234567890123456789012345678901234567890123456789012
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                          C
C    Subroutine providing the entry + necessary initialisation calls for   C
C    the rotation of wannier functions expressed as a LC of momentum       C
C    vectors (g vectors)                                                   C
C                                                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      subroutine RotateMyWannier(NSTATE,C0,lscr,scr,psi)
      IMPLICIT NONE
C
C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)
      include 'response_p.inc'
      include 'ener.inc'
      include 'irat.inc'
      include 'symm.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'metr.inc'

      INTEGER NSTATE
      COMPLEX*16 C0(NGW,NSTATE)
C     scratch variables
      INTEGER lscr
      complex*16 psi(*)
      REAL*8  scr(*)
c

      INTEGER imol

      INTEGER I,J,M1,M2,indx,istate,jstate
      INTEGER Lindx,combi,maxcombi
C     TIMING variable
      INTEGER ISUB

      REAL*8  Qnorm,Vnorm,DXVnorm
      REAL*8  q(4),qmat(3,3),Rsph,RSPHGEN
      REAL*8  Ebefore,Eafter,Ediff
      EXTERNAL Rsph,RSPHGEN
      COMPLEX*16 BCrot(NGW,NSTATE)
      POINTER (IP_BCrot,BCrot)
      REAL*8  sum,BOVL2(NSTATE,NSTATE)
      REAL*8  YLMover(13,13)
      REAL*8  BYLM(NGW,(BLmax+1)**2)
      POINTER (IP_BYLM,BYLM)
      REAL*8  RLMAT((BLmax+1)**2,(BLmax+1)**2)
      POINTER (IP_RLMAT,RLMAT)
      REAL*8  SCALED_COORD(3),CART_COORD(3)

C     Initialising the timing for the routine...
      CALL TISET('  main_rot',ISUB)
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

C     Calculating the maximum number of combined LM angular momentum elements
      maxcombi=(BLmax+1)**2
     
C     Allocating the memory for the arrays...
      write(6,*) 'Allocating memory'
c     write(6,*) 'Alloc BCrot'
      CALL MEMORY(IP_BCrot,2*NGW*NSTATE,'BCrot')
c     write(6,*) 'OK'
c     write(6,*) 'Alloc BYLM'
      CALL MEMORY(IP_BYLM,NGW*maxcombi,'BYLM')
c     write(6,*) 'OK'
c     write(6,*) 'Alloc RLMAT'
      CALL MEMORY(IP_RLMAT,maxcombi*maxcombi,'RLMAT')
c     write(6,*) 'OK'
c     write(6,*) 'Alloc STATEmin'
      CALL MEMORY(IP_STATEmin,NMOL,'STATEmin')
c     write(6,*) 'OK'
c     write(6,*) 'Alloc STATEmax'
      CALL MEMORY(IP_STATEmax,NMOL,'STATEmax')
      write(6,*) 'OK'

c     write(6,*) 'TPIBA = ',tpiba
c     write(6,*) 'OMEGA = ',OMEGA
      write(6,*) 'Lmax = ',BLmax
      write(6,*) 'maxcombi = ',maxcombi
      write(6,*) 'NMOL = ',NMOL

      write(6,*)
C     Checking initialisations

C     THETA?
      IF (IN_BTHETA.EQV..FALSE.) THEN
         write(6,*) 'No value for theta... assuming 0.D0'
C        allocating memory
         CALL MEMORY(IP_BTHETA,NMOL,'BTHETA')
C        initialising to zero
         DO imol=1,NMOL
            BTHETA(imol)=0.D0
         ENDDO
      ENDIF

C     VEC?
      IF (IN_BVEC.EQV..FALSE.) THEN
         write(6,*) 'No rotation vector... assuming 0.D0'
C        allocating memory
         CALL MEMORY(IP_BVECx,NMOL,'BVECx')
         CALL MEMORY(IP_BVECy,NMOL,'BVECy')
         CALL MEMORY(IP_BVECz,NMOL,'BVECz')
C        initialising to zero
         DO imol=1,NMOL
            BVECx(imol)=0.D0
            BVECy(imol)=0.D0
            BVECz(imol)=0.D0
         ENDDO
      ENDIF

C     DXVEC?
      IF (IN_BDXVEC.EQV..FALSE.) THEN
         write(6,*) 'No displacement vector... assuming 0.D0'
C        allocating memory
         CALL MEMORY(IP_BDXVECx,NMOL,'BDXVECx')
         CALL MEMORY(IP_BDXVECy,NMOL,'BDXVECy')
         CALL MEMORY(IP_BDXVECz,NMOL,'BDXVECz')
C        initialising to zero
         DO imol=1,NMOL
            BDXVECx(imol)=0.D0
            BDXVECy(imol)=0.D0
            BDXVECz(imol)=0.D0
         ENDDO
      ENDIF

C     Scaling the displacement vectors... -------------- TEST
      IF (TSCALE) THEN
         write(6,*) 'Scaling DX vectors...'
         CALL DSCAL(NMOL,1.D0/SXSCALE,BDXVECx,1)
         CALL DSCAL(NMOL,1.D0/SYSCALE,BDXVECy,1)
         CALL DSCAL(NMOL,1.D0/SZSCALE,BDXVECz,1)
C        now transform the scaled coordinates to cartesian coords 
C        using the HT matrix (taken from S_to_C ...)
         DO imol=1,NMOL
            SCALED_COORD(1)=BDXVECx(imol)
            SCALED_COORD(2)=BDXVECy(imol)
            SCALED_COORD(3)=BDXVECz(imol)

            CALL DGEMV('T',3,3,1.0D0,HT,3,SCALED_COORD,1,
     *                  0.0D0,CART_COORD,1)
 
            BDXVECx(imol)=CART_COORD(1)
            BDXVECy(imol)=CART_COORD(2)
            BDXVECz(imol)=CART_COORD(3)
         ENDDO
         write(6,*) 'Scaling done.'
      ENDIF
C     -------------------------------------------------- TEST

C     Are we reading a saved original Wannier function?
      IF (WANN_LOAD) THEN
C        do we read a distinct reference wannier function for each molecule?
         IF (WANN_MULTI) THEN
C           YES...
            CALL Read_Multi_Wannier(NSTATE,C0,NMOL,STATEmin,STATEmax)
         ELSE
C           NO, one reference for all molecules
C           GAUSSIAN representation?
            IF (WANN_GAUSS) THEN
C              YES...
               CALL Gauss_gen_read(NSTATE,C0,NMOL,STATEmin,STATEmax)
            ELSE
C              No, usual C0 reading...
               CALL Read_My_Wannier(NSTATE,C0,NMOL,STATEmin,STATEmax)
            ENDIF
         ENDIF
      ELSE
         STATEmin(1)=1
         STATEmax(1)=NSTATE
      ENDIF

C     Are we saving the original Wannier functions?
      IF (WANN_SAVE) THEN
C        Calculating the energy of the starting Wannier function
         write(6,*) 'Energy of the starting wannier function'
         CALL Calc_Wannier_Energy(NSTATE,C0)

C        GAUSSIAN representation?
         IF (WANN_GAUSS) THEN
C           YES...
            CALL Save_My_Wannier_Gaussian(NSTATE,C0,ETOT,lscr,scr,psi)
         ELSE
C           No, usual G-space saving
            CALL Save_My_Wannier(NSTATE,C0,ETOT)
         ENDIF
      ENDIF

C     Storing the value of the total energy before rotation
      Ebefore=ETOT
      
C     Building the YLM matrix
      write(6,*) 'Building the YLM matrix up to Lmax=',BLmax
c     CALL Build_BYLM(BLmax,BYLM)
      CALL Build_BYLM_SPHGEN(BLmax,BYLM)
      write(6,*) 'done'

C     Printing out the real sperical harmonics value for the first 25 G vectors
c     write(6,*) 'BYLM (for L=2 for 25 1st G vectors):'
c     Lindx=2
c     DO M1=1,25 
c        write(6,FMT='(26(1X,F8.5))')
c    $       (BYLM(M1,Lindx**2+Lindx+M2+1),M2=-Lindx,Lindx)
c     ENDDO

C     doing the same with the RSPHGEN function
c     write(6,*) 'Legendre'
C     Building the YLM matrix
c     write(6,*) 'Building the YLM matrix up to Lmax=',BLmax
c     CALL Build_BYLM_SPHGEN(BLmax,BYLM)
c     write(6,*) 'done'

C     Printing out the real sperical harmonics value for the first 25 G vectors
c     write(6,*) 'BYLM (for L=2 for 25 1st G vectors):'
c     Lindx=2
c     DO M1=1,25 
c        write(6,FMT='(26(1X,F8.5))')
c    $       (BYLM(M1,Lindx**2+Lindx+M2+1),M2=-Lindx,Lindx)
c     ENDDO

C     calculating the overlap between the spherical harmonics
c     write(6,*) 'OVERLAP S(lm)[YLM(a)*YLM(b)] up to L = ',BLMAX
c     DO I=1,13
c        DO J=1,13
c           sum=0.D0
c           DO Lindx=1,maxcombi
c              sum=sum+BYLM(I,Lindx)*BYLM(J,Lindx)
C              write(6,*) Lindx,I,BYLM(I,Lindx),J,BYLM(J,Lindx),sum
c           ENDDO
c           YLMover(I,J)=sum
c        ENDDO
c        write(6,FMT='(13(1X,F8.5))') (YLMover(I,M1),M1=1,13)
c     ENDDO

C     Main loop over the molecules
      DO imol=1,NMOL
         write(6,*) '#########################'
         write(6,'(1x,a,i6,a)') '# DOING MOLECULE ',imol,' #'
         write(6,*) '#########################'
         write(6,*)
C        Constructing the rotation quaternion
         Vnorm=DSQRT(BVECx(imol)**2+BVECy(imol)**2+BVECz(imol)**2)
         if (Vnorm.gt.0) then
            q(1)=cos(BTHETA(imol)/2)
            q(2)=sin(BTHETA(imol)/2)*BVECx(imol)/Vnorm
            q(3)=sin(BTHETA(imol)/2)*BVECy(imol)/Vnorm
            q(4)=sin(BTHETA(imol)/2)*BVECz(imol)/Vnorm
C        if direction vector is no good then no rotation
         else
            q(1)=1
            q(2)=0
            q(3)=0
            q(4)=0
         endif

C        Printing out rotation parameters
         write(6,*) 'Rotation parameters:'
         write(6,FMT='(1X,A,F8.5,A,3(F8.5,A))') 'Theta= ',
     $   BTHETA(imol),' VEC: (',BVECx(imol),',',BVECy(imol),
     $   ',',BVECz(imol),' )'

         write(6,FMT='(1X,A,4(F8.5,A))') 'q = (',q(1),', ',q(2),
     $   ', ',q(3),', ',q(4),' )'
         Qnorm=DSQRT(q(1)**2+q(2)**2+q(3)**2+q(4)**2)
         write(6,*) 'Qnorm = ',Qnorm
         write(6,*)

C        Displacement vector
         write(6,*) 'Displacement parameters:'
         write(6,FMT='(1X,A,3(F8.5,A))') 'DXVEC: (',BDXVECx(imol),
     $   ',',BDXVECy(imol),',',BDXVECz(imol),' )'

C        Calculating norm of displacement vector (if ==0, no shift..)
         DXVnorm=DSQRT(BDXVECx(imol)**2+BDXVECy(imol)**2
     $            +BDXVECz(imol)**2)
         write(6,*) 'DXVnorm = ',DXVnorm
         write(6,*)

C        If all the rotations are performed (even rot by 0 deg)
         IF (WANN_ALLROT.OR.BTHETA(imol).NE.0.) THEN
            write(6,*) 'Performing rotation for molecule',imol
C           just testing the rotation matrices...
c           CALL buildqmat(qmat,q)
            CALL buildqmatSGEN(qmat,q)

            write(6,*) 'Matrix representation of the quaternion'
            DO I=1,3
               write(6,FMT='(3(1X,F8.5))') qmat(I,1),qmat(I,2),qmat(I,3)
            enddo

C           initialising the rotation matrx           
            CALL Build_RMAT(BLmax,RLMAT,q)

C           printing out all the rotation matrices by increasing L
            write(6,*) 'RLMAT:'
            Do Lindx=0,BLmax
               write(6,*) 'L = ',Lindx
               DO M1=-Lindx,Lindx
                  combi=Lindx**2+Lindx+M1+1
                  write(6,FMT='(13(1X,F8.5))') (RLMAT(combi,
     $                  Lindx**2+Lindx+M2+1),M2=-Lindx,Lindx)
               enddo
            enddo

C           Now performing the rotation of the wannier functions
            write(6,*) 'Performing the rotation'
            CALL RotateCoeff(STATEmin(imol),STATEmax(imol),NSTATE,BLmax,
     $                       RLMAT,BYLM,C0,BCrot)
            write(6,*) 'FINISHED'
         ELSE
C           replacing the rotation by zero by a simple copy of C0 to BCrot
            write(6,*) 'No rotations for molecule',imol,
     $                 ', copying the coefficients..'
            DO istate=STATEmin(imol),STATEmax(imol)
               DO indx=1,NGW
                  BCrot(indx,istate)=C0(indx,istate)
               ENDDO
            ENDDO
         ENDIF

C        Now we can shift the Wannier functions by the DX vector
C        if its norm is greater than zero
         if (DXVnorm.gt.0.) THEN
            write(6,*) 'Shifting the Wannier function'
            CALL Shift_My_Wannier(STATEmin(imol),STATEmax(imol),
     $      NSTATE,BCrot,BDXVECx(imol),BDXVECy(imol),BDXVECz(imol))
            write(6,*) 'FINISHED'
         ENDIF
      ENDDO

C     NOW: all the wannier functions are rotated and moved according to the 
C          data of the input file...

C     if requested orthogonalise the ROTATED states+renormalising...
      IF (TORTHOG_WANNIER.AND..NOT.TSIMPLE_MODEL) THEN
c     IF (TORTHOG_WANNIER.AND.TSIMPLE_MODEL) THEN
C        if we use simple_model the the orthogonalisation is done later..
         CALL Ortho_My_Wannier(NSTATE,BCrot)
      ENDIF

C     Overlap matrix for the rotated states
      write(6,*) 'Building overlap matrix for the ROTATED states' 
      IF (TORTHOG_WANNIER) write(6,*) 'after orthogonalisation'
      call ovlap(nstate,BOVL2,BCrot,BCrot)

      write(6,*) 'Checking overlap matrix...'
      DO istate=1,NSTATE
         sum=0.D0
         DO jstate=1,NSTATE
C           checking matrix symmetry...
            IF(BOVL2(istate,jstate).NE.BOVL2(jstate,istate)) THEN
            write(6,*) 'Error in matrix symm. for states :',istate,
     $                 'and ',jstate
            ENDIF
C           checking sum of off-diagonal elements
            sum=sum+BOVL2(istate,jstate)
         ENDDO

         IF (ABS(BOVL2(istate,istate)- 1.D0).gt.1.D-10) THEN
            write(6,*) 'Error of normalisation for state ',istate
         ENDIF

         IF (ABS(sum-BOVL2(istate,istate)).gt.1.D-10) THEN
            write(6,*) 'Detected overlap for state ',istate
            write(6,*) 'sum of off-diag elements = ',sum
         ENDIF
      enddo
      write(6,*) 'Finished'

      IF(NSTATE.lt.10) THEN
      write(6,FMT='(A7,11(1X,I12))') 'ISTATE',(jstate,jstate=1,NSTATE)
      DO istate=1,NSTATE
         write(6,FMT='(I7,11(1X,F12.6))') istate,(BOVL2(istate,jstate)
     $                                    ,jstate=1,NSTATE)
      enddo
      ENDIF

      write(6,*)
C     Copying the rotated Wannier functions to the original C0
      write(6,*) 'Now replacing original by rotated C0'
      call DCOPY(2*ngw*nstate,BCrot,1,C0,1)
     
C     Now calculating the energy of the rotated wannier funct.
c     write(6,*) 'Energy of the ROTATED Wannier system'
c     CALL Calc_Wannier_Energy(NSTATE,C0)

C     Storing ETOT after rotation
c     Eafter=ETOT
c     Ediff=Ebefore-Eafter
c     write(6,*) 'Energy difference =',Ediff

C     Stop the clock!!
      CALL TIHALT('  main_rot',ISUB)
      
C     Freeing the memory used by the routine
      CALL FREEM(IP_BCrot)
      CALL FREEM(IP_BYLM)
      CALL FREEM(IP_RLMAT)
c     CALL FREEM(IP_STATEmin)
c     CALL FREEM(IP_STATEmax)
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                 C
C     This subroutine saves the wannier functions in the file     C
C     "ref-wannier.dat". Upon completion the file will contain:   C
C     ETOT                                                        C
C     NSTATE                                                      C
C     NGW                                                         C
C     C0(1..NGW,1..NSTATE)                                        C
C                                                                 C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE Save_My_Wannier(NSTATE,C0,ETOT)
      IMPLICIT NONE

C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
      include 'fileopen.inc'

C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INTEGER NSTATE
      COMPLEX*16 C0(NGW,NSTATE)
      REAL*8  ETOT

      INTEGER istate,Gindx
      LOGICAL FERROR
#ifdef PARALLEL
      call stopgm('SAVE_MY_WANNIER','not parallel.')
#endif

C     opening the file
      CALL FILEOPEN(500,'ref-wannier.dat',FO_DEF,FERROR)
C     Did the file open OK?
      IF (FERROR) THEN
         write(6,*) 'Error creating the file ref-wannier.dat...'
         call stopgm('SAVE_MY_WANNIER','file open error')
      ENDIF
 
      write(6,*) 'Saving ',NSTATE,' states...'
      write(6,*) 'Total energy for this set Etot = ',ETOT

C     Saving data to file..
      write(500,*) ETOT
      write(500,*) NSTATE
      write(500,*) NGW   

C     Main loop on the states
      DO istate=1,NSTATE
C        Loop on the G vectors
         DO Gindx=1,NGW
           write(500,*) C0(Gindx,istate)
         ENDDO
      ENDDO

C     Closing the file...
      CALL FILECLOSE(500)

      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                 C
C     This subroutine reads the wannier functions saved in        C
C     "ref-wannier.dat", and build up the corresponding states    C
C     for each molecules...                                       C
C                                                                 C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE Read_My_Wannier(NSTATE,C0,NMOL,STATEmin,STATEmax)
      IMPLICIT NONE

C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
      include 'fileopen.inc'

C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INTEGER NSTATE
      COMPLEX*16 C0(NGW,NSTATE)
      INTEGER NMOL,STATEmin(NMOL),STATEmax(NMOL)

      INTEGER istate,Gindx,KD,mol,kstate
      INTEGER R_NSTATE,R_NGW,STATES_TO_READ
      REAL*8  ETOT
      COMPLEX*16 Ct(NGW,*)
      POINTER (IP_Ct,Ct)
      LOGICAL FERROR
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

C     opening the file
      FERROR=.FALSE.
      CALL FILEOPEN(500,'ref-wannier.dat',FO_OLD,FERROR)
C     Did the file open OK?
      IF (FERROR) THEN
         write(6,*) 'Error:the file ref-wannier.dat is not there!'
         CALL STOPGM('READ_MY_WANNIER','FILE NOT FOUND')
      ENDIF
 
      write(6,*) 'Reading data from file...'

C     Reading index data from file..
      read(500,*) ETOT
      read(500,*) R_NSTATE
      read(500,*) R_NGW   

      write(6,*) 'The file contains ',R_NSTATE,' states'
      write(6,*) 'Each state has ',R_NGW,' G vectors'
      write(6,*) 'Total energy of the saved state Etot = ',ETOT
C     Checking the number of G vectors...
      IF (R_NGW.NE.NGW) THEN
         write(6,*) 'The number of G vectors found in the file (',
     $              R_NGW,') do not match NGW in the simulation (',
     $              NGW,')'
         write(6,*) 'At present this is not allowed...aborting load'
         CALL STOPGM('READ_MY_WANNIER','INCONSISTENT READ')
      ENDIF

C     Calculating the number of states to read
      STATES_TO_READ=NSTATE/NMOL

      write(6,*) 'Number of states per molecule ',STATES_TO_READ

C     checking the number of states
      IF (STATES_TO_READ.NE.R_NSTATE) THEN
         write(6,*) 'Need more states for each molecules...exiting'
         CALL STOPGM('READ_MY_WANNIER','INCONSISTENT NUMBER OF STATES')
      ENDIF

C     Everything is OK...allocating the Ct array (2*dims for CPLX*16)
      
      CALL MEMORY(IP_Ct,2*R_NGW*R_NSTATE,'Ct')

C     reading the wannier functions
      write(6,*) 'Proceeding... loading the data for ',NMOL,
     $           ' molecule(s)'

C     Main loop on the read states
      DO istate=1,R_NSTATE
C        Loop on the read G vectors
         DO Gindx=1,R_NGW
           read(500,*) Ct(Gindx,istate)
         ENDDO
      ENDDO

C     Closing the file...
      CALL FILECLOSE(500)

      write(6,*) 'Building',NSTATE,' Wannier states'
C     Now filling up the C0 array with the required ammount of data
C     initialising the global state counter
      kstate=1
C     Loop on number of molecules
      DO mol=1,NMOL
         STATEmin(mol)=kstate
C        Loop on number of read states
         DO istate=1,R_NSTATE
C           Loop on Gvecs
            DO Gindx=1,R_NGW
              C0(Gindx,kstate)=Ct(Gindx,istate)
            ENDDO
C           updating the global state counter
            kstate=kstate+1
         ENDDO
         STATEmax(mol)=kstate-1
      ENDDO

      DO mol=1,NMOL
         write(6,*) 'molecule: ',mol,' STATEmin = ',
     $   STATEmin(mol),' STATEmax = ',STATEmax(mol)
      ENDDO

      write(6,*) 'Done!'
C     Freeing memory
      CALL FREEM(IP_Ct)

      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                 C
C     This subroutine reads a wannier functions for each molecule C
C     saved in "ref-wannier-1.dat", "ref-wannier-2.dat", etc..    C
C                                                                 C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE Read_MULTI_Wannier(NSTATE,C0,NMOL,STATEmin,STATEmax)
      IMPLICIT NONE

C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
      include 'fileopen.inc'

C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INTEGER NSTATE
      COMPLEX*16 C0(NGW,NSTATE)
      INTEGER NMOL,STATEmin(NMOL),STATEmax(NMOL)

      INTEGER istate,Gindx,KD,mol,kstate,IA,IE
      INTEGER R_NSTATE,R_NGW
      REAL*8  ETOT
      CHARACTER NAME*20,number*8
      LOGICAL FERROR
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

      write(6,*) 'Multi wannier reference file reading subroutine'
      write(6,*) 'this will read ',NMOL,' .dat files'
      
C     initialising the global state counter
      kstate=1
      FERROR=.FALSE.
C     main loop on the molecules...
      DO mol=1,NMOL

C        First creating the file name for the molecule mol
         write(number,'(I8)')mol 
C        removing the spaces before and after the number (CPMD routine)
         CALL XSTRING(number,IA,IE)

         name='ref-wannier-'//number(IA:IE)//'.dat'
         write(6,*) 'molecule ',mol,'opening file ',name

C        opening the file
         CALL FILEOPEN(500,NAME,FO_OLD,FERROR)
C        Did the file open OK?
         IF (FERROR) THEN
            write(6,*) 'Error:the file',NAME,'is not there!'
            CALL STOPGM('READ_MULTI_WANNIER','FILE NOT FOUND')
         ENDIF
 
         write(6,*) 'Reading data from file...'
   
C        Reading index data from file..
         read(500,*) ETOT
         read(500,*) R_NSTATE
         read(500,*) R_NGW   

         write(6,*) 'The file contains ',R_NSTATE,' states'
         write(6,*) 'Each state has ',R_NGW,' G vectors'
         write(6,*) 'Total energy of the saved state Etot = ',ETOT

C        Checking the number of G vectors...
         IF (R_NGW.NE.NGW) THEN
            write(6,*) 'The number of G vectors found in the file (',
     $                 R_NGW,') do not match NGW in the simulation (',
     $                 NGW,')'
            write(6,*) 'At present this is not allowed...aborting load'
            CALL STOPGM('READ_MY_WANNIER','INCONSISTENT READ')
         ENDIF

C        first state occupied by molecule mol
         STATEmin(mol)=kstate
C        Main loop on the read states
         DO istate=1,R_NSTATE
C           Loop on the read G vectors
            DO Gindx=1,R_NGW
               read(500,*) C0(Gindx,STATEmin(mol)+istate-1)
            ENDDO
C           updating the global state counter
            kstate=kstate+1
         ENDDO

C        final state occupied by molecule mol
         STATEmax(mol)=kstate-1
C        Closing the file...
         CALL FILECLOSE(500)
      ENDDO

      DO mol=1,NMOL
         write(6,*) 'molecule: ',mol,' STATEmin = ',
     $   STATEmin(mol),' STATEmax = ',STATEmax(mol)
      ENDDO

      write(6,*) 'Done!'
      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                 C
C     This subroutine shifts the wannier function by a vector     C
C     defined by Vx,Vy,Vz.                                        C
C                                                                 C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE Shift_My_Wannier(ST_STATE,FI_STATE,NSTATE,C0,
     $                            Vx,Vy,Vz)
      IMPLICIT NONE

C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
      INCLUDE 'cnst.inc'

C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INTEGER ST_STATE,FI_STATE,NSTATE
      COMPLEX*16 C0(NGW,NSTATE)
      REAL*8  Vx,Vy,Vz

      INTEGER istate,Gindx
      COMPLEX*16 EXPiGR
      REAL*8  GbyR
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

      write(6,*) 'DOING STATE ',ST_STATE,' TO ',FI_STATE

C     Main loop on the states
      DO istate=ST_STATE,FI_STATE
C        Loop on the G vectors
         DO Gindx=1,NGW
C          Calculating the product G.r
           GbyR=GK(1,Gindx)*Vx+GK(2,Gindx)*Vy+GK(3,Gindx)*Vz
C          write(6,*) 'GbyR = ',GbyR

C          Adjusting the units
           GbyR=-GbyR*TPIBA
C           write(6,*) 'GbyR = ',GbyR
           
C          Calculating E^i(G.r) using cos(x)+i*sin(x)
           EXPiGR=dcmplx(DCOS(GbyR),DSIN(GbyR))
C           write(6,*) 'EXPiGR = ',EXPiGR

C          multiplying the Wannier coeffs
           C0(Gindx,istate)=C0(Gindx,istate)*EXPiGR
         ENDDO
      ENDDO
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                 C
C     This subroutine orthogonalises the Wannier functions using  C
C     the lowdin method (PSI)i=sum(j)[Sij^-(1/2) * (PSI)j]        C
C     also called symmetric orthogonalisation method (Szabo p142) C
C                                                                 C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE Ortho_My_Wannier(NSTATE,Ct)
      IMPLICIT NONE

C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
      INCLUDE 'cnst.inc'

C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INTEGER NSTATE
      COMPLEX*16 Ct(NGW,NSTATE)

      INTEGER istate,jstate,Gindx,k
      COMPLEX*16 tmp,tmpA(NSTATE)
      REAL*8  S(NSTATE,NSTATE)
      REAL*8  Z(NSTATE*(NSTATE+1)/2)
      REAL*8  eigenval(NSTATE)
      REAL*8  eigenvecs(NSTATE,NSTATE)
      REAL*8  EVMatrix(NSTATE,NSTATE)
      REAL*8  EVMtmp(NSTATE,NSTATE)
      REAL*8  work(3*NSTATE)
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

      write(6,*) 'ORTHOGONALISATION, NSTATE = ',NSTATE

C     initialising counter for z matrix
      k=1
C     building up the S matrix(overlap)
      DO istate=1,NSTATE
         DO jstate=istate,NSTATE
            tmp=(0.D0,0.D0)
C           Calculating the overlap of the positive g vect coeffs
            DO Gindx=2,NGW
               tmp=tmp+DCONJG(Ct(Gindx,istate))*Ct(Gindx,jstate)
            ENDDO
C           Special case for G=0 and adding the other G vecs
            S(istate,jstate)=
     $            DBLE(DCONJG(Ct(1,istate)*Ct(1,jstate)+2.D0*tmp))
C           Symmetrising the matrix
            S(jstate,istate)=S(istate,jstate)
C           Filling the special column Z matrix for dspevy
            Z(k)=S(istate,jstate)
            k=k+1
         ENDDO
      ENDDO

C     if the number of states is no too big
      IF (NSTATE.lt.10) THEN
C        printing overlap matrix
         write(6,*) 'orthog: Starting S matrix'
         write(6,FMT='(A7,11(1X,I12))') 'ISTATE',
     $                                       (istate,istate=1,NSTATE)
         DO istate=1,NSTATE
            write(6,FMT='(I7,11(1X,F12.6))') istate,(S(istate,jstate)
     $         ,jstate=1,NSTATE)
         enddo
      ENDIF

C     Diagonalising the S matrix through the z column matrix
      CALL DSPEVY(1,Z,eigenval,eigenvecs,NSTATE,NSTATE,work,3*NSTATE)

C     if the number of states is no too big
      IF (NSTATE.lt.10) THEN
C        printing eigenval and eigenvecs
         write(6,*) 'orthog: eigenvalues + eigenvecs'
         write(6,FMT='(A7,A10,10(1X,I12))') 'ISTATE',' EIGENVAL',
     $       (jstate,jstate=1,NSTATE)
         DO istate=1,NSTATE
            write(6,FMT='(I7,F10.6,10(1X,F12.6))') istate,
     $      eigenval(istate),
     $      (eigenvecs(istate,jstate),jstate=1,NSTATE)
         enddo
      ENDIF

C     Preparing eigenvalue matrix for S^-1/2 
      DO istate=1,NSTATE
C        checking if all the eigenvalues are greater than 0
         IF(eigenval(istate).le. 0.D0) THEN
         write(6,*) 'linear dependency between orbitals detected at ',
     $              istate
         ENDIF
         DO jstate=1,NSTATE
            IF(istate.eq.jstate) THEN
              EVMatrix(istate,istate)=1.D0/DSQRT(eigenval(istate))
            ELSE
              EVMatrix(istate,jstate)=0.D0
            ENDIF
         ENDDO
      ENDDO

C     building the S^-1/2 matrix (evec.1/sqrt(eval).Tevec)
C     First EVMtmp=1/sqrt(eval).Tevec
      CALL DGEMM('N','T',NSTATE,NSTATE,NSTATE,1.D0,EVMatrix,
     $           NSTATE,eigenvecs,NSTATE,0.D0,EVMtmp,NSTATE)

C     Now replacing S by evec.EVMtmp
      CALL DGEMM('N','N',NSTATE,NSTATE,NSTATE,1.D0,eigenvecs,
     $           NSTATE,EVMtmp,NSTATE,0.D0,S,NSTATE)
     
C     if the number of states is no too big
      IF (NSTATE.lt.10) THEN
C        printing final transformation matrix
         write(6,*) 'orthog: FINAL S^-1/2 matrix'
         write(6,FMT='(A7,11(1X,I12))') 'ISTATE',
     $                                       (istate,istate=1,NSTATE)
         DO istate=1,NSTATE
            write(6,FMT='(I7,11(1X,F12.6))') istate,(S(istate,jstate)
     $        ,jstate=1,NSTATE)
         enddo
      ENDIF
C     Finally, performing the orthogonalisation transformation:
C     (PSI)i=sum(j)[Sij^-(1/2) * (PSI)j]        
     
      DO Gindx=1,NGW
         DO istate=1,NSTATE
            tmpA(istate)=(0.D0,0.D0)
            DO jstate=1,NSTATE
               tmpA(istate)=tmpA(istate)+
     $                (S(istate,jstate)*Ct(Gindx,jstate))
            ENDDO
         ENDDO
C        copying the orthogonalised coefficients
         DO istate=1,NSTATE
            Ct(Gindx,istate)=tmpA(istate)
         ENDDO
      ENDDO

      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                               C
C   This subroutine calculates the BYLM matrix using RSPHGEN!                   C
C   BYLM is a (NGW)x(Lmax+1)^2 matrix containing the value of the real sperical C
C   harmonics for each G vector.                                                C
C                                                                               C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      SUBROUTINE Build_BYLM_SPHGEN(Lmax,BYLM)
      IMPLICIT NONE

C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)
      include 'cnst.inc'

      INTEGER Lmax
      REAL*8  BYLM(NGW,(Lmax+1)**2)

      INTEGER Lindx,Mindx,combi,gwindx,Lbase
C     TIMING Variable
      INTEGER ISUB

      REAL*8  RSPHGEN
      EXTERNAL RSPHGEN
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

C     Initialising the timing for the routine...
      CALL TISET('B_SGENBYLM',ISUB)

C     Starting the loading loops
      DO Lindx=0,Lmax
C        Lbase is a time saving device to calculate the LM combined index
C        given by LM=L^2+L+M+1
         Lbase=Lindx**2+Lindx+1
         DO Mindx=-Lindx,Lindx
            combi=Lbase+Mindx
C           Loading up the values of the real sperical harmonics for a
C           particular COMBINED index for each G vector
            DO gwindx=1,NGW
                BYLM(gwindx,combi)=RSPHGEN(Lindx,Mindx,
     $          GK(1,gwindx),GK(2,gwindx),GK(3,gwindx))

               BYLM(gwindx,combi)=
     $         BYLM(gwindx,combi)/DSQRT(DBLE(Lmax+1))
            enddo
         enddo 
      enddo

C     Stop the clock!
      CALL TIHALT('B_SGENBYLM',ISUB)

      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                               C
C   This subroutine calculates the BYLM matrix                                  C
C   BYLM is a (NGW)x(Lmax+1)^2 matrix containing the value of the real sperical C
C   harmonics for each G vector.                                                C
C                                                                               C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      SUBROUTINE Build_BYLM(Lmax,BYLM)
      IMPLICIT NONE

C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INTEGER Lmax
      REAL*8  BYLM(NGW,(Lmax+1)**2)

      INTEGER Lindx,Mindx,combi,gwindx,Lbase
C     TIMING Variable
      INTEGER ISUB

      REAL*8  tempylm(NGW)
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

C     Initialising the timing for the routine...
      CALL TISET('Build_BYLM',ISUB)

C     Starting the loading loops
      DO Lindx=0,Lmax
C        Lbase is a time saving device to calculate the LM combined index
C        given by LM=L^2+L+M+1
         Lbase=Lindx**2+Lindx+1
         DO Mindx=-Lindx,Lindx
            combi=Lbase+Mindx

C     Loading up the values of the real sperical harmonics for a
C     particular COMBINED index for each G vector
 
C     Even newer routine (mathematica) up to L=19
c            CALL BYLMR119(combi,NGW,HG,GK,tempylm,Lmax)

C     Now storing these values in the BYLM array
             DO gwindx=1,NGW
                BYLM(gwindx,combi)=tempylm(gwindx)
             enddo
         enddo 
      enddo

C     Stop the clock!
      CALL TIHALT('Build_BYLM',ISUB)

      END


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                               C
C   This subroutine calculates the RLMAT matrix                                 C
C   RLMAT is a (Lmax+1)^2x(Lmax+1)^2 matrix for the rotation of real spherical  C
C   harmonics by a given quaternion q.                                          C
C                                                                               C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      SUBROUTINE Build_RMAT(Lmax,RLMAT,q)
      IMPLICIT NONE
      INTEGER Lmax
      REAL*8  q(4),RLmat((Lmax+1)**2,(Lmax+1)**2)

      INTEGER Lindx,Mindx,Mtindx,combi,combiT,Lbase
C     TIMING Variable
      INTEGER ISUB

      REAL*8  QMAT(3,3),Rsph
      EXTERNAL Rsph
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

C     Initialising the timing for the routine...
      CALL TISET('Build_RMAT',ISUB)

C     building the initial 3x3 rotation matrix from the quaternion q
c     CALL buildqmat(QMAT,q)
      CALL buildqmatSGEN(qmat,q)

C     Starting the loading loops
      DO Lindx=0,Lmax
C        Lbase is a time saving device to calculate the LM combined index
C        given by LM=L^2+L+M+1
         Lbase=Lindx**2+Lindx+1
         DO Mindx=-Lindx,Lindx
            combi=Lbase+Mindx
C           Building the rotation matrix for each COMBINED index
            DO Mtindx=-Lindx,Lindx
               combiT=Lbase+Mtindx
               RLMAT(combiT,combi)=
     $                  Rsph(Lindx,Mtindx,Mindx,QMAT,RLmat,Lmax)
            enddo
         enddo 
      enddo

C     Stop the clock!
      CALL TIHALT('Build_RMAT',ISUB)

      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                            C
C      This subroutine rotates a given set of coefficients (C0) of G         C
C      vectors using a pre-defined rotation matrix (RLMAT) and pre-defined   C
C      G vector sperical harmonic values matrix (BYLM), the resulting        C
C      coefficients are found in BCrot                                       C
C                                                                            C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      SUBROUTINE RotateCoeff(ST_STATE,FI_STATE,NSTATE,Lmax,RLMAT,BYLM,
     $                       C0,BCrot)
      IMPLICIT NONE

C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
      INCLUDE 'cnst.inc'

C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INTEGER ST_STATE,FI_STATE,NSTATE,Lmax
      REAL*8  RLmat((Lmax+1)**2,(Lmax+1)**2)
      REAL*8  BYLM(NGW,(Lmax+1)**2)
      COMPLEX*16 C0(NGW,NSTATE),BCrot(NGW,NSTATE)

      INTEGER Lindx,Mtindx,Mindx,combi,combiT,Lbase
      INTEGER istate,GFindx,GKindx,GFmin,GFmax
C     TIMING variable
      INTEGER ISUB

      REAL*8  Uvar,Upos,Uneg,sumlk
      COMPLEX*16 coeff
C     REAL*8  prefac
C     PARAMETER(prefac=8.0D0*3.141592653589793D+0**3)
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

C     Initialising the timing for the routine...
      CALL TISET(' rot_the_C0',ISUB)

C     write(6,*) 'prefac =',prefac

C     Main loop
      DO istate=ST_STATE,FI_STATE
C        Initialising GFmin and GFmax...
         GFmin=0
         GFmax=0
         write(6,*) 'doing state ',istate
C        Special case for G(1)
         BCrot(1,istate)=C0(1,istate)
C        Then proceed with G from 2 to NGW
         DO GFindx=2,NGW
            coeff=(0.D0,0.D0)
C           GFmax has to be set to the last G vec having a norm equal to GFmin
            if(GFindx.gt.GFmax) then
               GFmin=GFindx
               GFmax=GFindx
C      do while((HG(GFmax+1).le.(HG(GFmin)+1)).and.(GFmax.lt.NGW))
               do while((HG(GFmax+1).eq.HG(GFmin)).and.(GFmax.lt.NGW))
                  GFmax=GFmax+1
               enddo
            endif
C           Now GFmin is the index of the first Gvector and GFmax 
C           is the index of the last G vector having the same norm
            DO GKindx=GFmin,GFmax
C           Initialising the sums over Uvar for positive and negative G vectors
               Upos=0.D0
               Uneg=0.D0
               DO Lindx=0,Lmax
                  Uvar=0.D0
C                 Lbase is a time saving device to calculate the LM combined index
C                 given by LM=L^2+L+M+1
                  Lbase=Lindx**2+Lindx+1
                  DO Mtindx=-Lindx,Lindx
                     combiT=Lbase+Mtindx
                     sumlk=0.D0
                     DO Mindx=-Lindx,Lindx
                        combi=Lbase+Mindx
C                       Calculating the sum over Mindx of the spherical 
C                       harmonic for Gk by rotation matrix element
                        sumlk=sumlk+BYLM(GKindx,combi)
     $                        *RLMAT(combiT,combi)
                     enddo
C                    Now multiplying the result by the spherical
C                    harmonic of Gf
                     Uvar=Uvar+sumlk*BYLM(GFindx,combiT)
                  enddo
C                 Adding the positive and negative contributions
                  Upos=Upos+Uvar
                  Uneg=Uneg+(-1)**Lindx*Uvar
               enddo
C              Summing over the positive and negative Gvector coefficients
               coeff=coeff+C0(GKindx,istate)*Upos
     $              +DCONJG(C0(GKindx,istate))*Uneg
            enddo
C           Finally multiplicating the result by required pre-factors
            BCrot(GFindx,istate)=coeff
C           BCrot(GFindx,istate)=(prefac)*coeff
C           BCrot(GFindx,istate)=(prefac/HG(GFindx))*coeff
         enddo
      enddo

C     Stop the clock...
      CALL TIHALT(' rot_the_C0',ISUB)

      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                        C
C   Integer Dirac delta function with integer argument   C
C                                                        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER function IDelta(A)
      INTEGER A

      IDelta=1
      IF(A .ne. 0) then
         IDelta=0
      end if
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                       C
C   Subroutine builds the modified rotation matrix corresponding to a   C
C   given unit quaternion q                                             C
C                                                                       C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      subroutine buildqmat(qmat,q)
      REAL*8  q(4),qmat(3,3)

      qmat(1,1)=q(1)**2+q(2)**2-q(3)**2-q(4)**2
      qmat(2,1)=-2.D0 *(q(1)*q(3)+q(2)*q(4))
      qmat(3,1)=-2.D0 *(q(2)*q(3)-q(1)*q(4))
      qmat(1,2)=2.D0 *(q(1)*q(3)-q(2)*q(4))
      qmat(2,2)=q(1)**2-q(2)**2-q(3)**2+q(4)**2
      qmat(3,2)=2.D0 *(q(1)*q(2)+q(3)*q(4))
      qmat(1,3)=- 2.D0 *(q(2)*q(3)+q(1)*q(4))
      qmat(2,3)=-2.D0 *(q(1)*q(2)-q(3)*q(4))
      qmat(3,3)=q(1)**2-q(2)**2+q(3)**2-q(4)**2
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                       C
C   Subroutine builds the modified rotation matrix corresponding to a   C
C   given unit quaternion q to use with RSPHGEN spherical harmonics!    C
C                                                                       C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      subroutine buildqmatSGEN(qmat,q)
      REAL*8  q(4),qmat(3,3)

      qmat(1,1)=q(1)**2-q(2)**2+q(3)**2-q(4)**2
      qmat(2,1)=2.D0*(-(q(1)*q(2))-q(3)*q(4))
      qmat(3,1)=2.D0*(-(q(2)*q(3))+q(1)*q(4))
      qmat(1,2)=2.D0*(q(1)*q(2)-q(3)*q(4))
      qmat(2,2)=q(1)**2-q(2)**2-q(3)**2+q(4)**2
      qmat(3,2)=2.D0*(q(1)*q(3)+q(2)*q(4))
      qmat(1,3)=2.D0*(-(q(2)*q(3))-q(1)*q(4))
      qmat(2,3)=2.D0*(-(q(1)*q(3))+q(2)*q(4))
      qmat(3,3)=q(1)**2+q(2)**2-q(3)**2-q(4)**2

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                            C
C   General rotation matrix for real spherical harmonics JPC100(1996)6342    C
C   and JPCA102(1998)9099                                                    C
C   24/07/00 dmb: removed the recursion relationship to speed-up the code    C
C                                                                            C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      REAL*8  function Rsph(L,M1,M2,qmat,RLmat,Lmax)
      INTEGER L,M1,M2,Lmax
      REAL*8  qmat(3,3)
      REAL*8  RLmat((Lmax+1)**2,(Lmax+1)**2)
      REAL*8  smallu,Pfunc,smallv,bigV,smallw,bigW
      EXTERNAL smallu,Pfunc,smallv,bigV,smallw,bigW
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

      If((L.gt.1) .and. (abs(M1).le.L) .and. (abs(M2).le.L)) then
         Rsph=smallu(L,M1,M2)*Pfunc(0,L,M1,M2,qmat,RLmat,Lmax) +
     $         smallv(L,M1,M2)*bigV(L,M1,M2,qmat,RLmat,Lmax) +
     $         smallw(L,M1,M2)*bigW(L,M1,M2,qmat,RLmat,Lmax)
C     note that the bigU function is equivalent to the P function
      else
         Rsph=0
      end if
      if(L.eq.1) then
         if(abs(M1).lt.2 .and. abs(M2).lt.2) then
            Rsph=qmat(M1+2,M2+2)
         else
            Rsph=0
         end if
      end if
      if(L.eq.0) then
         Rsph=1
      end if
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                                C
C   small u function (note the conversion of integer to DBLE for L, M1 and M2)   C
C                                                                                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      REAL*8  function smallu(L,M1,M2)
      INTEGER L,M1,M2

C     write(6,*) 'received L =',L,' M1 =',M1,' M2 =',M2
      If(abs(M2).lt.L) then
         smallu=DSQRT(DBLE((L+M1)*(L-M1))/DBLE((L+M2)*(L-M2)))
      end if
      If(abs(M2).eq.L) then
         smallu=DSQRT(DBLE((L+M1)*(L-M1))/DBLE(2*L*(2*L-1)))
      end if
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                                C
C   small v function (note the conversion of integer to DBLE for L, M1 and M2)   C
C                                                                                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      REAL*8  function smallv(L,M1,M2)
      INTEGER L,M1,M2

C     write(6,*) 'received L =',L,' M1 =',M1,' M2 =',M2
      If(abs(M2).lt.L) then
         smallv=0.5D0*DSQRT(DBLE((1+IDelta(M1))*(L+abs(M1)-1)*
     $         (L+abs(M1)))/DBLE((L+M2)*(L-M2)))
     $         *DBLE(1-2*IDelta(M1))
      end if
      If(abs(M2).eq.L) then
         smallv=0.5D0*DSQRT(DBLE((1+IDelta(M1))*(L+abs(M1)-1)*
     $         (L+abs(M1)))/DBLE(2*L*(2*L-1)))
     $         *DBLE(1-2*IDelta(M1))
      end if
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                                C
C   small w function (note the conversion of integer to DBLE for L, M1 and M2)   C
C                                                                                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      REAL*8  function smallw(L,M1,M2)
      INTEGER L,M1,M2

C     write(6,*) 'received L =',L,' M1 =',M1,' M2 =',M2
      If(abs(M2).lt.L) then
         smallw=-0.5D0*Sqrt(DBLE((L-abs(M1)-1)*(L-abs(M1)))/
     $         DBLE((L+M2)*(L-M2)))*DBLE(1-IDelta(M1))
      end if
      If(abs(M2).eq.L) then
         smallw=-0.5D0*Sqrt(DBLE((L-abs(M1)-1)*(L-abs(M1)))/
     $         DBLE(2*L*(2*L-1)))*DBLE(1-IDelta(M1))
      end if
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                 C
C   P function (no conversions)   C
C                                 C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      REAL*8  function Pfunc(k,L,Mu,M2,qmat,RLmat,Lmax)
      INTEGER k,L,Mu,M2,Lmax
      INTEGER combiLMu,combi2,combi3
      REAL*8  qmat(3,3)
      REAL*8  RLmat((Lmax+1)**2,(Lmax+1)**2)

C     Calculating the common combined index for the L-1,Mu element of RLmat
      combiLMu=(L-1)**2+(L-1)+Mu+1

C     write(6,*) 'Pfunc received k =',k,' L =',L,' Mu =',Mu,' M2 =',M2
      If(abs(M2).lt.L) then
C        calculating the combined indices for RLmat        
         combi2=(L-1)**2+(L-1)+M2+1
C        now the value of the Pfunction
         Pfunc=qmat(k+2,2)*RLmat(combiLMu,combi2)
      end if

      If(M2.eq.L) then
C        calculating the combined indices for RLmat        
         combi2=(L-1)**2+(L-1)+(L-1)+1
         combi3=(L-1)**2+(L-1)+(1-L)+1
C        now the value of the Pfunction
         Pfunc=qmat(k+2,3)*RLmat(combiLMu,combi2)-
     $         qmat(k+2,1)*RLmat(combiLMu,combi3)
      end if

      If(M2.eq.-L) then
C        calculating the combined indices for RLmat        
         combi2=(L-1)**2+(L-1)+(1-L)+1
         combi3=(L-1)**2+(L-1)+(L-1)+1
C        now the value of the Pfunction
         Pfunc=qmat(k+2,3)*RLmat(combiLMu,combi2)+
     $         qmat(k+2,1)*RLmat(combiLMu,combi3)
      end if
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                             C
C   bigV function (note the conversion of integer to DBLE for L, M1 and M2)   C
C                                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      REAL*8  function bigV(L,M1,M2,qmat,RLmat,Lmax)
      INTEGER L,M1,M2,Lmax
      REAL*8  qmat(3,3)
      REAL*8  RLmat((Lmax+1)**2,(Lmax+1)**2)
      REAL*8  Pfunc
      EXTERNAL Pfunc
      INTEGER IDelta
      EXTERNAL IDelta

      If(M1.eq.0) then
         bigV=Pfunc(1,L,1,M2,qmat,RLmat,Lmax)+
     $        Pfunc(-1,L,-1,M2,qmat,RLmat,Lmax)
      end if
      If(M1.gt.0) then
         bigV=Pfunc(1,L,M1-1,M2,qmat,RLmat,Lmax)*
     $         DSQRT(1.D0+DBLE(IDelta(M1-1)))-
     $        Pfunc(-1,L,1-M1,M2,qmat,RLmat,Lmax)*
     $         (1.D0-DBLE(IDelta(M1-1)))
      end if
      if(M1.lt.0) then
          bigV=Pfunc(1,L,M1+1,M2,qmat,RLmat,Lmax)*
     $          (1.D0-DBLE(IDelta(M1+1))) +
     $         Pfunc(-1,L,-1-M1,M2,qmat,RLmat,Lmax)*
     $          DSQRT(1.D0+DBLE(IDelta(M1+1)))
      end if
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                             C
C   bigW function (note the conversion of integer to DBLE for L, M1 and M2)   C
C                                                                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      REAL*8  function bigW(L,M1,M2,qmat,RLmat,Lmax)
      INTEGER L,M1,M2,Lmax
      REAL*8  qmat(3,3)
      REAL*8  RLmat((Lmax+1)**2,(Lmax+1)**2)
      REAL*8  Pfunc
      EXTERNAL Pfunc

      If(M1.eq.0) then
         bigW=0
      end if
      If(M1.gt.0) then
         bigW=Pfunc(1,L,M1+1,M2,qmat,RLmat,Lmax)+
     $        Pfunc(-1,L,-1-M1,M2,qmat,RLmat,Lmax)
      end if
      if(M1.lt.0) then
         bigW=Pfunc(1,L,M1-1,M2,qmat,RLmat,Lmax)-
     $        Pfunc(-1,L,1-M1,M2,qmat,RLmat,Lmax)
      end if
      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                          C
C      This subroutine calculates the energy of a system   C
C      based on the value of the coefficients (C0) of      C
C      the G vectors. (Thanks to Aldo)                     C
C                                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      
      SUBROUTINE Calc_Wannier_Energy(NSTATE,C0)
      IMPLICIT NONE

      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'
      INCLUDE 'response_p.inc'
C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INCLUDE 'coor.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'wann.inc'
      INCLUDE 'sfac.inc'

      INTEGER NSTATE
      COMPLEX*16 C0(NGW,NSTATE)

C Assumed defined in includes files or above:
C ip_fion, fion, tau0, NNR1, LFORCES

C NECESSARY DEFs
      REAL*8     FNL(*),DFNL(*)
      INTEGER    LFORCES
C end of NECESSARY DEFs
      
      COMPLEX*16 CS(NGW,*)
      POINTER    (IP_CS,CS)

      CHARACTER  TAG*30
      INTEGER    IL_RHOE,IL_PSI
     
      REAL*8     SCR(*)
      POINTER    (IP_SCR,SCR)

      COMPLEX*16 dens(NNR1),PSI(NNR1)
      pointer    (ip_dens,dens),(ip_psi,psi)
#ifdef PARALLEL
      call stopgm('ROTATE_..._P','not parallel.')
#endif

C     Allocating the array Cs (scratch array?)
      CALL MEMORY(IP_CS,2*NGW*NSTATE,'CS')

C     Allocating scratch space !
      CALL GIVE_SCR_FORCES(LFORCES,TAG,NSTATE,.FALSE.,.FALSE.)
      write(6,*) 'Scratch dimensions = ',LFORCES
      CALL MEMORY(IP_SCR,LFORCES,'SCR')
      
C Calculation of the energy of the wannier function (C) Aldo
      write(6,*) 'Entering Energy calculation'

      call fnlalloc(NSTATE,.false.,.false.)

C     Calculating the size of the wf and density arrays
      call rhoe_psi_size(il_rhoe,il_psi)

      write(6,*) 'Rho size (',il_rhoe,
     $   ') psi size (',il_psi,')'

      call memory(ip_psi,il_psi,'psi')
      call memory(ip_dens,il_rhoe,'dens')
      call memory(ip_fion,3*nax*nsx,'fion')

C     Calculate energy!
C     Prefactor
      call phfac(tau0)
C     The forces routine calculates the energy
      call forces(c0,cs,tau0,fion,dens,PSI,SCR,LFORCES,
     $      NSTATE,1,.false.,.false.)

      write(6,*) 'WANNIER ENERGY'
      write(6,*) 'ETOTAL = ',ETOT,' A. U.'
      write(6,*) 'EKIN = ',EKIN,' A. U.'
      write(6,*) 'EHT = ',EHT,' A. U.'

      write(6,*) 'E[rho] = ',EHT+EPSEU+EXC,' A. U.'
      write(6,*)
C     Freeing the used memory
      CALL FREEM(IP_CS)
      CALL FREEM(IP_SCR)
      CALL FREEM(ip_psi)
      CALL FREEM(ip_dens)
      CALL FREEM(ip_fion)

      RETURN
      END
