C        1         2         3         4         5         6         7    
C23456789012345678901234567890123456789012345678901234567890123456789012
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                          C
C    Subroutine providing the entry + necessary initialisation calls for   C
C    the interaction of localised orbitals (PT interaction)                C
C                                                                          C
C    ------------------------ PARALLEL VERSION -------------------------   C
C                                                                          C
C    WARNING: this version DOES NOT perform any wavefunction rotation      C
C                                                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      subroutine RotateMyWannier_para(NSTATE,C0,lscr,scr,psi)
      IMPLICIT NONE
C
C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)
      include 'response_p.inc'
      include 'ener.inc'
      include 'irat.inc'
      include 'symm.inc'
      INCLUDE 'ions.inc'
      INCLUDE 'metr.inc'
      INCLUDE 'geq0.inc'

      INTEGER NSTATE
      COMPLEX*16 C0(NGW,NSTATE)
C     scratch variables
      INTEGER lscr
      complex*16 psi(*)
      REAL*8  scr(*)
c

      INTEGER imol

      INTEGER I,J,M1,M2,indx,istate,jstate
C     TIMING variable
      INTEGER ISUB

      REAL*8  Qnorm,Vnorm,DXVnorm
      REAL*8  Ebefore,Eafter,Ediff
      EXTERNAL Rsph,RSPHGEN
      COMPLEX*16 BCrot(NGW,NSTATE)
      POINTER (IP_BCrot,BCrot)
      COMPLEX*16 CSCR(NGW,NSTATE)
      POINTER (IP_CSCR,CSCR)
      REAL*8  sum,BOVL2(NSTATE,NSTATE)
      REAL*8  SCALED_COORD(3),CART_COORD(3)

C     Initialising the timing for the routine...
      CALL TISET('  main_rot',ISUB)

C     Allocating the memory for the arrays...
      if(parent) write(6,*) 'Allocating memory'
      CALL MEMORY(IP_BCrot,2*NGW*NSTATE,'BCrot')
      CALL MEMORY(IP_CSCR,2*NGW*NSTATE,'CSCR')
      CALL MEMORY(IP_STATEmin,NMOL,'STATEmin')
      CALL MEMORY(IP_STATEmax,NMOL,'STATEmax')
      if(parent) write(6,*) 'Done with memory allocation'
      if(parent) write(6,*)
C     Checking initialisations

C     DXVEC?
      IF (IN_BDXVEC.EQV..FALSE.) THEN
C        allocating memory
         CALL MEMORY(IP_BDXVECx,NMOL,'BDXVECx')
         CALL MEMORY(IP_BDXVECy,NMOL,'BDXVECy')
         CALL MEMORY(IP_BDXVECz,NMOL,'BDXVECz')
C        initialising to zero
         DO imol=1,NMOL
            BDXVECx(imol)=0.D0
            BDXVECy(imol)=0.D0
            BDXVECz(imol)=0.D0
         ENDDO
      ENDIF

C     Scaling the displacement vectors... -------------- TEST
      IF (TSCALE) THEN
         if(parent) write(6,*) 'Scaling DX vectors...'
         CALL DSCAL(NMOL,1.D0/SXSCALE,BDXVECx,1)
         CALL DSCAL(NMOL,1.D0/SYSCALE,BDXVECy,1)
         CALL DSCAL(NMOL,1.D0/SZSCALE,BDXVECz,1)
C        now transform the scaled coordinates to cartesian coords 
C        using the HT matrix (taken from S_to_C ...)
         DO imol=1,NMOL
            SCALED_COORD(1)=BDXVECx(imol)
            SCALED_COORD(2)=BDXVECy(imol)
            SCALED_COORD(3)=BDXVECz(imol)

            CALL DGEMV('T',3,3,1.0D0,HT,3,SCALED_COORD,1,
     *                  0.0D0,CART_COORD,1)
 
            BDXVECx(imol)=CART_COORD(1)
            BDXVECy(imol)=CART_COORD(2)
            BDXVECz(imol)=CART_COORD(3)
         ENDDO
         if(parent) write(6,*) 'Scaling done.'
      ENDIF
C     -------------------------------------------------- TEST

C     Are we reading a saved original Wannier function?
      IF (WANN_LOAD) THEN
C        do we read a distinct reference wannier function for each molecule?
         IF (WANN_MULTI) THEN
C           YES...
            CALL Read_MULTI_Wann_restart_P(NSTATE,C0)
         ELSE
C           NO, one reference for all molecules
C           GAUSSIAN representation?
            IF (WANN_GAUSS) THEN
C              YES...
               CALL Gauss_gen_read(NSTATE,C0,NMOL,STATEmin,STATEmax)
            ELSE
C              No, usual C0 reading...
               CALL Read_Wann_restart_P(NSTATE,C0)
            ENDIF
         ENDIF
      ELSE
         STATEmin(1)=1
         STATEmax(1)=NSTATE
      ENDIF

C     Are we saving the original Wannier functions?
      IF (WANN_SAVE) THEN
C        Calculating the energy of the starting Wannier function
         if(parent) write(6,*) 'Energy of the starting wannier function'
         CALL Calc_Wannier_Energy(NSTATE,C0)

C        GAUSSIAN representation?
         IF (WANN_GAUSS) THEN
C           YES...
            CALL Save_My_Wannier_Gaussian(NSTATE,C0,ETOT,lscr,scr,psi)
         ELSE
C           No, usual G-space saving
            CALL Save_Wann_restart_P(NSTATE,C0,ETOT)
         ENDIF
      ENDIF

C     Storing the value of the total energy before rotation
      Ebefore=ETOT
      
C     There are no rotations in this implementation so we copy C0 to BCrot
      call DCOPY(2*ngw*nstate,C0,1,BCrot,1)      
      
C     Main loop over the molecules
      DO imol=1,NMOL
         if (parent) then
           write(6,*) '#########################'
           write(6,'(1x,a,i6,a)') '# DISPL MOLECULE ',imol,' #'
           write(6,*) '#########################'
         endif

C        Displacement vector
        if (parent) then
         write(6,*) '# Displacement parameters:'
         write(6,FMT='(1X,A,3(F8.5,A))') '# DXVEC: (',BDXVECx(imol),
     $           ',',BDXVECy(imol),',',BDXVECz(imol),' )'
        endif

C        Calculating norm of displacement vector (if ==0, no shift..)
         DXVnorm=DSQRT(BDXVECx(imol)**2+BDXVECy(imol)**2
     $            +BDXVECz(imol)**2)
         if(parent) write(6,*) '# DXVnorm = ',DXVnorm
         if(parent) write(6,*) '# '

C        Now we can shift the Wannier functions by the DX vector
C        if its norm is greater than zero
         if (DXVnorm.gt.0.D0) THEN
            if (parent) write(6,*) '# Shifting the Wannier function'
            CALL Shift_My_Wannier(STATEmin(imol),STATEmax(imol),
     $      NSTATE,BCrot,BDXVECx(imol),BDXVECy(imol),BDXVECz(imol))
            if (parent) write(6,*) '# FINISHED'
         ELSE
            if(parent) write(6,*) '# NO TRANSLATION for molecule',imol
         ENDIF
         if(parent) write(6,*) '#########################'
         if(parent) write(6,*) 
      ENDDO

C     NOW: all the wannier functions are moved according to the 
C          data of the input file...

C     if requested orthogonalise the ROTATED states+renormalising... (CSCR is a scratch)
      IF (TORTHOG_WANNIER.AND..NOT.TSIMPLE_MODEL) THEN
          CALL LOWDIN(BCrot,CSCR,SCR,LSCR,NSTATE)
C         bit of cleaning just to be nice...
          IF(GEQ0) CALL ZCLEAN(C0,NSTATE,NGW)
      ENDIF

C     Overlap matrix for the rotated states
      if(parent) 
     &     write(6,*) 'Building overlap matrix for the WANNIER states'
      
      IF ((TORTHOG_WANNIER).AND.(parent)) 
     &   write(6,*) 'Wave function fragments are now orthogonalised'
C this is already parallel...
      call ovlap(nstate,BOVL2,BCrot,BCrot)
#ifdef PARALLEL
      call glosum(nstate*nstate,BOVL2)
#endif

      if (parent) write(6,*) 'Checking wave function overlap matrix...'
      DO istate=1,NSTATE
         sum=0.D0
         DO jstate=1,NSTATE
C           checking matrix symmetry...
            IF(BOVL2(istate,jstate).NE.BOVL2(jstate,istate)) THEN
            if (parent) write(6,*) 'Error in matrix symm. for states :',
     $                             istate,'and ',jstate
            ENDIF
C           checking sum of off-diagonal elements
            sum=sum+BOVL2(istate,jstate)
         ENDDO
         IF (ABS(BOVL2(istate,istate)- 1.D0).gt.1.D-10) THEN
            if (parent) write(6,*) 'Error of normalisation for state ',
     $                              istate
         ENDIF

         IF (ABS(sum-BOVL2(istate,istate)).gt.1.D-10) THEN
            if (parent) then
                write(6,*) 'Detected overlap for state ',istate
                write(6,*) 'sum of off-diag elements = ',sum
            endif
         ENDIF
      enddo
      if (parent) write(6,*) 'Finished'

      if (parent) write(6,*)
C     Copying the rotated Wannier functions to the original C0
      call DCOPY(2*ngw*nstate,BCrot,1,C0,1)
     
C     Stop the clock!!
      CALL TIHALT('  main_rot',ISUB)
      
C     Freeing the memory used by the routine
      CALL FREEM(IP_BCrot)
      CALL FREEM(IP_CSCR)
      end

C     ==================================================================
      SUBROUTINE Save_Wann_restart_P(nstate,C0,ETOT)
C     ==--------------------------------------------------------------==
C     ==                                                              ==
C     == Saves the Wannier functions in a restart file named:         ==
C     == RESTART.ref                                                  ==
C     ==                                                              ==
C     == NOTE: This works in parallel! (taken from RESTART_P)         ==
C     ==                                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h' 
      INCLUDE 'coor.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'response_p.inc'
      INCLUDE 'fileopen.inc'

C     Arguments
      INTEGER NSTATE
      COMPLEX*16 C0(NGW,*)
      REAL*8  ETOT
C     Variables
      INTEGER    fileunit
      CHARACTER  FILENAME*128
      INTEGER    IA,IE,ISUB,is,info,controlbyte
      LOGICAL    FEXIST,ERROR,FERROR
      INTEGER    IERROR
#ifdef PARALLEL
      INTEGER    MSGLEN
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('SAVE_WAN_P',ISUB)

C     setup filename
      fileunit = 2606
      filename = 'RESTART.ref'
      call xstring(filename,ia,ie)
C     ==--------------------------------------------------------------==
      if (parent) then
            CALL FILEOPEN(fileunit,filename,FO_DEF+FO_UFO,FERROR)
            IF(FERROR) GOTO 401
            REWIND (fileunit,ERR=401)
            error=.false.
            goto 402
 401        error=.true.
 402        continue

            if (error) then
              call stopgm('SAVE_WAN_P',
     &             'ERROR OPENING RESTART FILE ('
     &             //filename(ia:ie)//') for writing.')
            endif
C     ==--------------------------------------------------------------==
            write(6,*) 'Saving ',NSTATE,' states...'
            write(6,*) 'Total energy for this system:'
            write(6,*) 'Etot = ',ETOT,'A. U.'
            controlbyte = 730909
            write (fileunit) controlbyte
            write (fileunit) a1,a2,a3
            write (fileunit) nstate,ngw
         endif                  ! parent
C     ==--------------------------------------------------------------==
         CALL WR30WFN(fileunit,IERROR,nstate,C0,tau0,'NIL')
C     ==--------------------------------------------------------------==
         if(parent) then
            write (fileunit) etot
            write (6,'(A)') 
     &           ' WANNIER wavefunction written to RESTART file '
     &           //filename(IA:IE)//'.'
            CALL FILECLOSE(fileunit)
         endif
C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
      CALL TIHALT('SAVE_WAN_P',ISUB)
      RETURN
      END
C     ==================================================================


C     ==================================================================
      SUBROUTINE Read_Wann_restart_P(NSTATE,C0)
C     ==--------------------------------------------------------------==
C     ==                                                              ==
C     == Reads the Wannier functions from the restart file named      ==
C     == RESTART.ref                                                  ==
C     ==                                                              ==
C     == NOTE: This works in parallel! (taken from RESTART_P)         ==
C     ==                                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h' 
      INCLUDE 'coor.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'response_p.inc'
      INCLUDE 'fileopen.inc'

C     Arguments
      INTEGER NSTATE
      COMPLEX*16 C0(NGW,*)
C     Variables
      INTEGER    fileunit
      CHARACTER  FILENAME*128
      INTEGER    IA,IE,ISUB,is,ir,info,controlbyte
      integer    nstate_,ngw_,llc(3),irecord
      real*8     a1_(3),a2_(3),a3_(3)
      LOGICAL    FEXIST,ERROR,FERROR
      INTEGER istate,mol,kstate
      INTEGER STATES_TO_READ
      REAL*8  ETOT
      COMPLEX*16 Ct(NGW,*)
      POINTER (IP_Ct,Ct)

#ifdef PARALLEL
      INTEGER    MSGLEN
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('READ_WAN_P',ISUB)

C     setup filename
      fileunit = 2606
      filename = 'RESTART.ref'
      call xstring(filename,ia,ie)
C     Calculating the number of states to read
      STATES_TO_READ=NSTATE/NMOL

      if(parent) THEN 
        write(6,*) 'WLOAD needs',STATES_TO_READ,
     &       'states for each molecule'

        CALL FILEOPEN(fileunit,filename,FO_OLD+FO_UFO,FERROR)
        IF(FERROR) GOTO 501

        rewind (fileunit,ERR=501)
        error=.false.
        goto 502
 501    error=.true.
 502    continue

        if (error) then
          call stopgm('READ_WAN_P',
     &         'ERROR OPENING RESTART FILE ('
     &         //filename(ia:ie)//') for reading.')
        endif
C     ==--------------------------------------------------------------==
C     GENERAL PART of restart file:
            read (fileunit) controlbyte
            read (fileunit) a1_,a2_,a3_
            read (fileunit) nstate_,ngw_
            error=.false.
C           TEST our controlbyte... is it a wannier restart
            if (controlbyte .ne. 730909)
     &           call stopgm('READ_WAN_P',
     &           'ERROR READING RESTART FILE ('
     &           //filename(ia:ie)//'): wrong parameters.')
            
            write(6,*) 'The file contains ',nstate_,' states'
            write(6,*) 'Each state has ',ngw_,' G vectors'
C           TEST number of plane waves: has to be the same
c            if (NGW.NE.ngw_) then
c                         error = .true.
c                    call stopgm('READ_WAN_P',
c     $           'The number of G vectors found in the file '
c     $              //'do not match NGW in the simulation! '//
c     $              'At present this is not allowed...')
c                 endif
C           TEST number of states: enough for each molecule
            if (STATES_TO_READ.NE.nstate_) then
              call stopgm('READ_WAN_P',
     $             'Need more states for each molecules...exiting')
            endif
         endif                  ! parent stuff.
#ifdef PARALLEL
         msglen=1*8/IRAT
         call my_bcast(nstate_,msglen,SOURCE,ALLGRP)
#endif
C     ==--------------------------------------------------------------==         
C     Everything is OK...allocating the Ct array (2*dims for CPLX*16)
      CALL MEMORY(IP_Ct,2*NGW*NSTATE_,'Ct')

C     reading the wannier functions
      if(parent) write(6,*) 'Proceeding... loading the data for ',
     $           NMOL,' molecule(s)'
C     ==--------------------------------------------------------------==
         IF(PARENT) READ(fileunit) irecord ! dummy read.
         CALL RD30WFN(fileunit,Ct,NSTATE_,TAU0,INFO,
     &        .FALSE.,NSTATE_,1,'NIL')
C     ==--------------------------------------------------------------==
         if(parent) then
            read (fileunit) etot
            write(6,*) 'Total energy of the saved state'
            write(6,*) 'Etot = ',ETOT,'A. U.'
            call xstring(filename,IA,IE)
            write (6,'(A)') 
     &           ' WANNIER wavefunction read from RESTART file '
     &           //filename(IA:IE)//'.'
            CALL FILECLOSE(fileunit)
         endif
C     ==--------------------------------------------------------------==
      if(parent) write(6,*) 'Building',NSTATE,' Wannier states'
C     Now filling up the C0 array with the required ammount of data
C     initialising the global state counter
      kstate=1
C     Loop on number of molecules
      DO mol=1,NMOL
C        Storing the energy of the monomers on parent
         if(parent) EMONO(mol)=etot
C        Storing starting state for each monomer
         STATEmin(mol)=kstate
C        Loop on number of read states
         DO istate=1,NSTATE_
            call DCOPY(2*ngw,Ct(1,istate),1,C0(1,kstate),1)
C           updating the global state counter
            kstate=kstate+1
         ENDDO
         STATEmax(mol)=kstate-1
      ENDDO

      if(parent) then
         DO mol=1,NMOL
             write(6,*) 'molecule: ',mol,' STATEmin = ',
     $       STATEmin(mol),' STATEmax = ',STATEmax(mol)
         ENDDO

         write(6,*) 'Done!'
      endif
C     ==--------------------------------------------------------------==
      CALL TIHALT('READ_WAN_P',ISUB)
C     Freeing memory
      CALL FREEM(IP_Ct)
      RETURN
      END
C     ==================================================================


C     ==================================================================
      SUBROUTINE Read_MULTI_Wann_restart_P(NSTATE,C0)
C     ==--------------------------------------------------------------==
C     ==                                                              ==
C     == Reads a Wannier function for each molecule from a series     ==
C     == of restart file named:                                       ==
C     == RESTART.ref1                                                 ==
C     == RESTART.ref2                                                 ==
C     == RESTART.ref3                                                 ==
C     ==   ...                                                        ==
C     ==                                                              ==
C     == NOTE: This works in parallel! (taken from RESTART_P)         ==
C     ==                                                              ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h' 
      INCLUDE 'coor.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'response_p.inc'
      INCLUDE 'fileopen.inc'
C     Arguments
      INTEGER NSTATE
      COMPLEX*16 C0(NGW,*)
C     Variables
      INTEGER    fileunit
      CHARACTER  FILENAME*128,number*8
      INTEGER    IA,IE,ISUB,is,ir,info,controlbyte
      integer    nstate_,ngw_,llc(3),irecord
      real*8     a1_(3),a2_(3),a3_(3)
      LOGICAL    FEXIST,ERROR
      INTEGER istate,mol,kstate
      REAL*8  ETOT
      COMPLEX*16 Ct(NGW,*)
      POINTER (IP_Ct,Ct)

#ifdef PARALLEL
      INTEGER    MSGLEN
#endif
C     ==--------------------------------------------------------------==
      CALL TISET('READMULW_P',ISUB)
      if (parent) then
        write(6,*) '+-------------------------------------------------+'
        write(6,*) '|   PARALLEL PARALLEL PARALLEL PARALLEL PARALLEL  |'
        write(6,*) '| Multi wannier reference file reading subroutine |'
        write(6,*) '+-------------------------------------------------+'
        write(6,*) '(This  routine will read ',NMOL,' reference files)'
      endif
      
C     initialising the global state counter
      kstate=1
C     main loop on the molecules...
      DO mol=1,NMOL

C        setup filename
         fileunit = 2606
         
C        First creating the file name for the molecule mol
         write(number,'(I8)')mol 
C        removing the spaces before and after the number (CPMD routine)
         CALL XSTRING(number,IA,IE)

         filename = 'RESTART.ref'//number(IA:IE)
         call xstring(filename,ia,ie)
         
         if(parent) then
           write(6,*)
           write(6,*) '======== Reading data for molecule ',mol,
     $                ' ========'
           write(6,*) 'Opening file '//filename(ia:ie)
         endif
C     ==--------------------------------------------------------------==
         if (parent) then
           error=.false.
           CALL FILEOPEN(fileunit,filename,FO_OLD+FO_UFO,error)
           if (error) call stopgm('READMULW_P',
     &          'RESTART FILE NOT FOUND: '//filename(ia:ie)//'.')
            rewind (fileunit,ERR=501)
            error=.false.
            goto 502
 501        error=.true.
 502        continue
            if (error) then
              call stopgm('READMULW_P',
     &             'ERROR OPENING RESTART FILE ('
     &             //filename(ia:ie)//') for reading.')
            endif
C     ==--------------------------------------------------------------==
C     GENERAL PART of restart file:
            read (fileunit) controlbyte
            read (fileunit) a1_,a2_,a3_
            read (fileunit) nstate_,ngw_
            error=.false.
C           TEST our controlbyte... is it a wannier restart
            if (controlbyte .ne. 730909) error = .true.
            write(6,*) 'The file contains ',nstate_,' states'
            write(6,*) 'Each state has ',ngw_,' G vectors'
C           TEST number of plane waves: has to be the same
c            if (NGW.NE.ngw_) then
c                error = .true.
c                call stopgm('READMULW_P',
c     $           'The number of G vectors found in the file '
c     $              //'do not match NGW in the simulation! '//
c     $              'At present this is not allowed...')
c                 endif
            if (error) then
              call stopgm('READMULW_P',
     &             'ERROR READING RESTART FILE ('
     &             //filename(ia:ie)//'): wrong parameters.')
            endif
          endif                 ! parent stuff.
#ifdef PARALLEL
         msglen=1*8/IRAT
         call my_bcast(nstate_,msglen,SOURCE,ALLGRP)
#endif
C     ==--------------------------------------------------------------==         
C        Everything is OK...allocating the Ct array (2*dims for CPLX*16)
         CALL MEMORY(IP_Ct,2*NGW*NSTATE_,'Ct')
C        Clear memory just in case...
         CALL AZZERO(Ct,2*NGW*NSTATE_)
C     ==--------------------------------------------------------------==
C        reading the wannier functions
         IF(PARENT) READ(fileunit) irecord ! dummy read.
         CALL RD30WFN(fileunit,Ct,NSTATE_,TAU0,INFO,
     &        .FALSE.,NSTATE_,1,'NIL')
C     ==--------------------------------------------------------------==
         if(parent) then
            read (fileunit) etot
C           Storing monomer energies on the parent
            EMONO(mol)=etot
            write(6,*) 'Total energy of the saved state'
            write(6,*) 'Etot = ',ETOT,'A. U.'
            call xstring(filename,IA,IE)
            write (6,'(A)') 
     &           ' WANNIER wavefunction read from RESTART file '
     &           //filename(IA:IE)//'.'
            CALL FILECLOSE(fileunit)
         endif
C     ==--------------------------------------------------------------==
C        Now filling up the C0 array with the data from this file
         if(parent) then
            write(6,*) 'Building',nstate_,' Wannier states for ',
     $                 'molecule',mol
            write(6,*) 'Assigning states',kstate,'to',
     $                 kstate+nstate_-1
         endif
C        Assigning first and last state number for this molecule      
         STATEmin(mol)=kstate
         STATEmax(mol)=STATEmin(mol)+nstate_-1

C        copy each read state to C0 state by state
         DO istate=1,nstate_
            call DCOPY(2*ngw,Ct(1,istate),1,C0(1,kstate),1)
C           updating the global state counter
            kstate=kstate+1
         ENDDO
C     Freeing memory
         CALL FREEM(IP_Ct)
      ENDDO

      if(parent) then
         write(6,*)
         write(6,*) '======== Summary of assigned states  ========'
         DO mol=1,NMOL
           write(6,*) 'molecule: ',mol,' STATEmin = ',
     $     STATEmin(mol),' STATEmax = ',STATEmax(mol)
         ENDDO
         write(6,*)
        write(6,*) '+-------------------------------------------------+'
        write(6,*) '|   PARALLEL PARALLEL PARALLEL PARALLEL PARALLEL  |'
        write(6,*) '| Multi wannier subroutine FINISHED (no errors)   |'
        write(6,*) '+-------------------------------------------------+'
        write(6,*)
      endif
C     ==--------------------------------------------------------------==
      CALL TIHALT('READMULW_P',ISUB)
      RETURN
      END
C     ==================================================================


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                 C
C     This subroutine shifts the wannier function by a vector     C
C     defined by Vx,Vy,Vz.                                        C
C                                                                 C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE Shift_My_Wannier(ST_STATE,FI_STATE,NSTATE,C0,
     $                            Vx,Vy,Vz)
      IMPLICIT NONE

C     Including the "essential" variables...
      include 'system.h'
      include 'cppt.inc'
      INCLUDE 'cnst.inc'

C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INTEGER ST_STATE,FI_STATE,NSTATE
      COMPLEX*16 C0(NGW,NSTATE)
      REAL*8  Vx,Vy,Vz

      INTEGER istate,Gindx
      COMPLEX*16 EXPiGR
      REAL*8  GbyR

      if(parent) write(6,*) 'DOING STATE ',ST_STATE,' TO ',FI_STATE

C     Main loop on the states
      DO istate=ST_STATE,FI_STATE
C        Loop on the G vectors
         DO Gindx=1,NGW
C          Calculating the product G.r
               GbyR=GK(1,Gindx)*Vx+GK(2,Gindx)*Vy+GK(3,Gindx)*Vz

C          Adjusting the units
               GbyR=-GbyR*TPIBA

C          Calculating E^i(G.r) using cos(x)+i*sin(x)
           EXPiGR=dcmplx(DCOS(GbyR),DSIN(GbyR))

C          multiplying the Wannier coeffs
           C0(Gindx,istate)=C0(Gindx,istate)*EXPiGR
         ENDDO
      ENDDO
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                          C
C      This subroutine calculates the energy of a system   C
C      based on the value of the coefficients (C0) of      C
C      the G vectors. (Thanks to Aldo)                     C
C                                                          C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      
      SUBROUTINE Calc_Wannier_Energy(NSTATE,C0)
      IMPLICIT NONE

      INCLUDE 'system.h'
      INCLUDE 'cppt.inc'
      INCLUDE 'response_p.inc'
C     so from now on NGW is the number of plane waves, GK(3,NGW) is the array
C     of G vectors, HG(NGW) is the array containing the square of the norm of
C     each G vector. (according to Daniel...)

      INCLUDE 'coor.inc'
      INCLUDE 'cnst.inc'
      INCLUDE 'gvec.inc'
      INCLUDE 'ener.inc'
      INCLUDE 'geq0.inc'
      INCLUDE 'irat.inc'
      INCLUDE 'fft.inc'
      INCLUDE 'wann.inc'
      INCLUDE 'sfac.inc'

      INTEGER NSTATE
      COMPLEX*16 C0(NGW,NSTATE)

C Assumed defined in includes files or above:
C ip_fion, fion, tau0, NNR1, LFORCES

C NECESSARY DEFs
      REAL*8     FNL(*),DFNL(*)
      INTEGER    LFORCES
C end of NECESSARY DEFs
      
      COMPLEX*16 CS(NGW,*)
      POINTER    (IP_CS,CS)

      CHARACTER  TAG*30
      INTEGER    IL_RHOE,IL_PSI
     
      REAL*8     SCR(*)
      POINTER    (IP_SCR,SCR)

      COMPLEX*16 dens(NNR1),PSI(NNR1)
      pointer    (ip_dens,dens),(ip_psi,psi)

C     Allocating the array Cs (scratch array?)
      CALL MEMORY(IP_CS,2*NGW*NSTATE,'CS')

C     Allocating scratch space !
      CALL GIVE_SCR_FORCES(LFORCES,TAG,NSTATE,.FALSE.,.FALSE.)
      if(parent) write(6,*) 'Scratch dimensions = ',LFORCES
      CALL MEMORY(IP_SCR,LFORCES,'SCR')
      
C Calculation of the energy of the wannier function (C) Aldo
      if(parent) write(6,*) 'Entering Energy calculation'

      call fnlalloc(NSTATE,.false.,.false.)

C     Calculating the size of the wf and density arrays
      call rhoe_psi_size(il_rhoe,il_psi)

      if(parent) write(6,*) 'Rho size (',il_rhoe,
     $   ') psi size (',il_psi,')'

      call memory(ip_psi,il_psi,'psi')
      call memory(ip_dens,il_rhoe,'dens')
      call memory(ip_fion,3*nax*nsx,'fion')

C     Calculate energy!
C     Prefactor
      call phfac(tau0)
C     The forces routine calculates the energy
      call forces(c0,cs,tau0,fion,dens,PSI,SCR,LFORCES,
     $      NSTATE,1,.false.,.false.)

      if(parent) then
        write(6,*) 'WANNIER ENERGY'
        write(6,*) 'ETOTAL = ',ETOT,' A. U.'
        write(6,*) 'EKIN = ',EKIN,' A. U.'
        write(6,*) 'EHT = ',EHT,' A. U.'

        write(6,*) 'E[rho] = ',EHT+EPSEU+EXC,' A. U.'
        write(6,*)
      endif
      
C     Freeing the used memory
      CALL FREEM(IP_CS)
      CALL FREEM(IP_SCR)
      CALL FREEM(ip_psi)
      CALL FREEM(ip_dens)
      CALL FREEM(ip_fion)

      RETURN
      END
      
